--
--  Copyright (C) 2014-2022, AdaCore
--  SPDX-License-Identifier: Apache-2.0
--

with Ada.Containers;                  use Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Vectors;
with Ada.Directories;
with Ada.Exceptions;
with Ada.Finalization;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Ada.Strings.Wide_Wide_Unbounded; use Ada.Strings.Wide_Wide_Unbounded;
pragma Warnings (Off, "internal");
with Ada.Strings.Wide_Wide_Unbounded.Aux;
use Ada.Strings.Wide_Wide_Unbounded.Aux;
pragma Warnings (On, "internal");
with Ada.Text_IO;                     use Ada.Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with System;
with GNAT.Task_Lock;
with GNAT.Traceback.Symbolic;
with GNATCOLL.Traces;
with Langkit_Support.Adalog.Debug;
with Langkit_Support.Generic_API.Analysis;
with Langkit_Support.Generic_API.Introspection;
with Langkit_Support.Hashes; use Langkit_Support.Hashes;
with Langkit_Support.Images; use Langkit_Support.Images;
with Langkit_Support.Names;  use Langkit_Support.Names;
with Langkit_Support.Relative_Get;
with Libadalang.Private_Converters;
use Libadalang.Private_Converters;
pragma Warnings (Off, "referenced");
          with Ada.Containers.Hashed_Sets;
          with Libadalang.Env_Hooks;
            use Libadalang.Env_Hooks;
          with Libadalang.Implementation.Extensions;
          with Libadalang.Implementation.Extensions;
            use Libadalang.Implementation.Extensions;
          with Libadalang.Internal_Default_Provider;
          with Libadalang.Sources;
pragma Warnings (On, "referenced");
package body Libadalang.Implementation is
   use Precomputed_Symbols;
   pragma Warnings (Off, "has no effect");
   use Solver;
   pragma Warnings (On, "has no effect");
   package Context_Vectors is new Ada.Containers.Vectors
     (Index_Type   => Positive,
      Element_Type => Internal_Context);
   type Contexts_Destructor is limited
      new Ada.Finalization.Limited_Controlled with null record;
   overriding procedure Finalize (CD : in out Contexts_Destructor);
   --  Helper to destroy all contexts when terminating the process
   package Context_Pool is
      procedure Acquire (Context : out Internal_Context)
         with Post => Context /= null;
      --  If a context is free for reuse, increment its serial number and
      --  return it. Otherwise, allocate a new one. In any case, this does not
      --  initialize it, except for the Serial_Number field.
      procedure Release (Context : in out Internal_Context)
         with Pre  => Context /= null,
              Post => Context = null;
      --  Tag Context as free for reuse and set it to null
      procedure Free;
      --  Free all contexts in this pool. Intended to be called only when the
      --  process is terminating, to avoid reported memory leaks.
   private
      Available : Context_Vectors.Vector;
      --  List of allocated contexts that can be re-used right now
      CD : Contexts_Destructor with Unreferenced;
      --  Singleton whose only purpose is to free all contexts in Available
      --  when finalized.
   end Context_Pool;
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure);
   --  Common underlying implementation for Register_Destroyable_Gen
   pragma Warnings (Off, "referenced");
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access;
   pragma Warnings (On, "referenced");
   procedure Reset_Envs_Caches (Unit : Internal_Unit);
   --  Reset the env caches of all lexical environments created for ``Unit``
   procedure Destroy (Env : in out Lexical_Env_Access);
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean;
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean;
   --  Those maps are used to give unique ids to lexical envs while pretty
   --  printing them.
   package Address_To_Id_Maps is new Ada.Containers.Hashed_Maps
     (Lexical_Env, Integer, Hash, "=");
   type Dump_Lexical_Env_State is record
      Env_Ids : Address_To_Id_Maps.Map;
      --  Mapping: Lexical_Env -> Integer, used to remember which unique Ids we
      --  assigned to the lexical environments we found.
      Next_Id : Positive := 1;
      --  Id to assign to the next unknown lexical environment
      Root_Env : Lexical_Env;
      --  Lexical environment we consider a root (this is the Root_Scope from
      --  the current analysis context), or null if unknown.
   end record;
   --  Holder for the state of lexical environment dumpers
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String;
   --  If E is known, return its unique Id from State. Otherwise, assign it a
   --  new unique Id and return it.
   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "");
   --  Helper for the public overload, but working on the generic API node type
   ------------------------
   -- Precomputed_Symbol --
   ------------------------
   pragma Warnings (Off, "referenced");
   function Precomputed_Symbol
     (Index : Precomputed_Symbol_Index) return Text_Type is
   pragma Warnings (On, "referenced");
   begin
         declare
            Raw_Text : constant Text_Type := (case Index is
               when Precomputed_Sym => """&""",
               when Precomputed_Sym_1 => """&""""+""",
               when Precomputed_Sym_10 => """=""",
               when Precomputed_Sym_11 => """>""",
               when Precomputed_Sym_12 => """>=""",
               when Precomputed_Sym_2 => """*""",
               when Precomputed_Sym_20 => "<<>>",
               when Precomputed_Sym_3 => """**""",
               when Precomputed_Sym_4 => """+""",
               when Precomputed_Sym_5 => """-""",
               when Precomputed_Sym_6 => """/""",
               when Precomputed_Sym_7 => """/=""",
               when Precomputed_Sym_8 => """<""",
               when Precomputed_Sym_9 => """<=""",
               when Precomputed_Sym_Abort => "abort",
               when Precomputed_Sym_Abort_Signal => "Abort_Signal",
               when Precomputed_Sym_Abort_Signal_315 => "abort_signal_",
               when Precomputed_Sym_Abs => """abs""",
               when Precomputed_Sym_Abs_316 => "abs",
               when Precomputed_Sym_Abstract => "abstract",
               when Precomputed_Sym_Abstract_State => "Abstract_State",
               when Precomputed_Sym_Accept => "accept",
               when Precomputed_Sym_Access => "Access",
               when Precomputed_Sym_Access_319 => "access",
               when Precomputed_Sym_Ada => "Ada",
               when Precomputed_Sym_Ada_320 => "ada",
               when Precomputed_Sym_Add_Named => "Add_Named",
               when Precomputed_Sym_Add_Unnamed => "Add_Unnamed",
               when Precomputed_Sym_Address => "Address",
               when Precomputed_Sym_Address_Size => "Address_Size",
               when Precomputed_Sym_Adjacent => "Adjacent",
               when Precomputed_Sym_Adjacent_321 => "adjacent",
               when Precomputed_Sym_Aft => "Aft",
               when Precomputed_Sym_Aggregate => "Aggregate",
               when Precomputed_Sym_Aliased => "aliased",
               when Precomputed_Sym_Alignment => "Alignment",
               when Precomputed_Sym_All => "all",
               when Precomputed_Sym_And => """and""",
               when Precomputed_Sym_And_324 => "and",
               when Precomputed_Sym_Annotate => "Annotate",
               when Precomputed_Sym_Array => "array",
               when Precomputed_Sym_Asm_Input => "Asm_Input",
               when Precomputed_Sym_Asm_Input_326 => "asm_input",
               when Precomputed_Sym_Asm_Input_Operand => "Asm_Input_Operand",
               when Precomputed_Sym_Asm_Output => "Asm_Output",
               when Precomputed_Sym_Asm_Output_327 => "asm_output",
               when Precomputed_Sym_Asm_Output_Operand => "Asm_Output_Operand",
               when Precomputed_Sym_Assert => "Assert",
               when Precomputed_Sym_Assert_And_Cut => "Assert_And_Cut",
               when Precomputed_Sym_Assume => "Assume",
               when Precomputed_Sym_Asynchronous => "Asynchronous",
               when Precomputed_Sym_At => "at",
               when Precomputed_Sym_Atomic => "Atomic",
               when Precomputed_Sym_Atomic_Components => "Atomic_Components",
               when Precomputed_Sym_Attach_Handler => "Attach_Handler",
               when Precomputed_Sym_Aux_Dec => "Aux_DEC",
               when Precomputed_Sym_Base => "Base",
               when Precomputed_Sym_Begin => "begin",
               when Precomputed_Sym_Big_Integer => "Big_Integer",
               when Precomputed_Sym_Big_Integers => "Big_Integers",
               when Precomputed_Sym_Big_Numbers => "Big_Numbers",
               when Precomputed_Sym_Bit_Order => "Bit_Order",
               when Precomputed_Sym_Body => "body",
               when Precomputed_Sym_Boolean => "Boolean",
               when Precomputed_Sym_Buffer => "Buffer",
               when Precomputed_Sym_By_Entry => "By_Entry",
               when Precomputed_Sym_Callable => "Callable",
               when Precomputed_Sym_Caller => "Caller",
               when Precomputed_Sym_Case => "case",
               when Precomputed_Sym_Ceiling => "Ceiling",
               when Precomputed_Sym_Ceiling_332 => "ceiling",
               when Precomputed_Sym_Character => "Character",
               when Precomputed_Sym_Class => "Class",
               when Precomputed_Sym_Code_Address => "Code_Address",
               when Precomputed_Sym_Compile_Time_Error => "Compile_Time_Error",
               when Precomputed_Sym_Compile_Time_Warning => "Compile_Time_Warning",
               when Precomputed_Sym_Component_Size => "Component_Size",
               when Precomputed_Sym_Compose => "Compose",
               when Precomputed_Sym_Compose_333 => "compose",
               when Precomputed_Sym_Constant => "constant",
               when Precomputed_Sym_Constant_Indexing => "Constant_Indexing",
               when Precomputed_Sym_Constrained => "Constrained",
               when Precomputed_Sym_Contract_Cases => "Contract_Cases",
               when Precomputed_Sym_Convention => "Convention",
               when Precomputed_Sym_Copy_Sign => "Copy_Sign",
               when Precomputed_Sym_Copy_Sign_335 => "copy_sign",
               when Precomputed_Sym_Count => "Count",
               when Precomputed_Sym_Cursor => "Cursor",
               when Precomputed_Sym_Debug => "Debug",
               when Precomputed_Sym_Declare => "declare",
               when Precomputed_Sym_Decreases => "Decreases",
               when Precomputed_Sym_Default_Bit_Order => "Default_Bit_Order",
               when Precomputed_Sym_Default_Initial_Condition => "Default_Initial_Condition",
               when Precomputed_Sym_Default_Iterator => "Default_Iterator",
               when Precomputed_Sym_Default_Scalar_Storage_Order => "Default_Scalar_Storage_Order",
               when Precomputed_Sym_Definite => "Definite",
               when Precomputed_Sym_Delay => "delay",
               when Precomputed_Sym_Delta => "Delta",
               when Precomputed_Sym_Delta_338 => "delta",
               when Precomputed_Sym_Denorm => "Denorm",
               when Precomputed_Sym_Depends => "Depends",
               when Precomputed_Sym_Deref => "Deref",
               when Precomputed_Sym_Descriptor_Size => "Descriptor_Size",
               when Precomputed_Sym_Digits => "Digits",
               when Precomputed_Sym_Digits_339 => "digits",
               when Precomputed_Sym_Discard_Names => "Discard_Names",
               when Precomputed_Sym_Do => "do",
               when Precomputed_Sym_Dummy => "__dummy",
               when Precomputed_Sym_Duration => "Duration",
               when Precomputed_Sym_Dynamic_Predicate => "Dynamic_Predicate",
               when Precomputed_Sym_Elaborate_Body => "Elaborate_Body",
               when Precomputed_Sym_Element => "Element",
               when Precomputed_Sym_Else => "else",
               when Precomputed_Sym_Elsif => "elsif",
               when Precomputed_Sym_Enabled => "Enabled",
               when Precomputed_Sym_End => "end",
               when Precomputed_Sym_Ensures => "Ensures",
               when Precomputed_Sym_Entity => "Entity",
               when Precomputed_Sym_Entry => "entry",
               when Precomputed_Sym_Enum_Rep => "Enum_Rep",
               when Precomputed_Sym_Enum_Rep_345 => "enum_rep",
               when Precomputed_Sym_Enum_Val => "Enum_Val",
               when Precomputed_Sym_Enum_Val_346 => "enum_val",
               when Precomputed_Sym_Epsilon => "Epsilon",
               when Precomputed_Sym_Exception => "exception",
               when Precomputed_Sym_Exception_Id => "Exception_Id",
               when Precomputed_Sym_Exception_Occurrence => "Exception_Occurrence",
               when Precomputed_Sym_Exceptions => "Exceptions",
               when Precomputed_Sym_Exit => "exit",
               when Precomputed_Sym_Exponent => "Exponent",
               when Precomputed_Sym_Exponent_349 => "exponent",
               when Precomputed_Sym_Export => "Export",
               when Precomputed_Sym_External_Tag => "External_Tag",
               when Precomputed_Sym_Fast_Math => "Fast_Math",
               when Precomputed_Sym_Finalization_Size => "Finalization_Size",
               when Precomputed_Sym_First => "First",
               when Precomputed_Sym_First_Bit => "First_Bit",
               when Precomputed_Sym_First_Valid => "First_Valid",
               when Precomputed_Sym_Fixed_Value => "Fixed_Value",
               when Precomputed_Sym_Fixed_Value_350 => "fixed_value",
               when Precomputed_Sym_Floor => "Floor",
               when Precomputed_Sym_Floor_351 => "floor",
               when Precomputed_Sym_For => "for",
               when Precomputed_Sym_Fore => "Fore",
               when Precomputed_Sym_Fraction => "Fraction",
               when Precomputed_Sym_Fraction_353 => "fraction",
               when Precomputed_Sym_Function => "function",
               when Precomputed_Sym_Generic => "generic",
               when Precomputed_Sym_Ghost => "Ghost",
               when Precomputed_Sym_Global => "Global",
               when Precomputed_Sym_Gnat => "GNAT",
               when Precomputed_Sym_Gnatprove => "GNATprove",
               when Precomputed_Sym_Goto => "goto",
               when Precomputed_Sym_Has_Access_Values => "Has_Access_Values",
               when Precomputed_Sym_Has_Discriminants => "Has_Discriminants",
               when Precomputed_Sym_Has_Same_Storage => "Has_Same_Storage",
               when Precomputed_Sym_Has_Tagged_Values => "Has_Tagged_Values",
               when Precomputed_Sym_Identity => "Identity",
               when Precomputed_Sym_If => "if",
               when Precomputed_Sym_Image => "Image",
               when Precomputed_Sym_Image_358 => "image",
               when Precomputed_Sym_Img => "Img",
               when Precomputed_Sym_Implicit_Dereference => "Implicit_Dereference",
               when Precomputed_Sym_Import => "Import",
               when Precomputed_Sym_Import_Function => "Import_Function",
               when Precomputed_Sym_Import_Procedure => "Import_Procedure",
               when Precomputed_Sym_In => "in",
               when Precomputed_Sym_Increases => "Increases",
               when Precomputed_Sym_Independent => "Independent",
               when Precomputed_Sym_Independent_Components => "Independent_Components",
               when Precomputed_Sym_Index => "Index",
               when Precomputed_Sym_Initial_Condition => "Initial_Condition",
               when Precomputed_Sym_Initialized => "Initialized",
               when Precomputed_Sym_Inline => "Inline",
               when Precomputed_Sym_Input => "Input",
               when Precomputed_Sym_Input_360 => "input",
               when Precomputed_Sym_Integer => "Integer",
               when Precomputed_Sym_Integer_Literal => "Integer_Literal",
               when Precomputed_Sym_Integer_Value => "Integer_Value",
               when Precomputed_Sym_Integer_Value_361 => "integer_value",
               when Precomputed_Sym_Interface => "Interface",
               when Precomputed_Sym_Interface_362 => "interface",
               when Precomputed_Sym_Interrupt_Handler => "Interrupt_Handler",
               when Precomputed_Sym_Invalid_Value => "Invalid_Value",
               when Precomputed_Sym_Invariant => "Invariant",
               when Precomputed_Sym_Invariant_Class => "Invariant'Class",
               when Precomputed_Sym_Is => "is",
               when Precomputed_Sym_Iterable => "Iterable",
               when Precomputed_Sym_Iterator_Element => "Iterator_Element",
               when Precomputed_Sym_Iterator_Interfaces => "Iterator_Interfaces",
               when Precomputed_Sym_Large => "Large",
               when Precomputed_Sym_Last => "Last",
               when Precomputed_Sym_Last_Bit => "Last_Bit",
               when Precomputed_Sym_Last_Valid => "Last_Valid",
               when Precomputed_Sym_Leading_Part => "Leading_Part",
               when Precomputed_Sym_Leading_Part_364 => "leading_part",
               when Precomputed_Sym_Left => "left",
               when Precomputed_Sym_Length => "Length",
               when Precomputed_Sym_Library_Level => "Library_Level",
               when Precomputed_Sym_Limited => "limited",
               when Precomputed_Sym_Loop => "loop",
               when Precomputed_Sym_Loop_Entry => "Loop_Entry",
               when Precomputed_Sym_Loop_Invariant => "Loop_Invariant",
               when Precomputed_Sym_Machine => "Machine",
               when Precomputed_Sym_Machine_368 => "machine",
               when Precomputed_Sym_Machine_Code => "Machine_Code",
               when Precomputed_Sym_Machine_Emax => "Machine_Emax",
               when Precomputed_Sym_Machine_Emin => "Machine_Emin",
               when Precomputed_Sym_Machine_Mantissa => "Machine_Mantissa",
               when Precomputed_Sym_Machine_Overflows => "Machine_Overflows",
               when Precomputed_Sym_Machine_Radix => "Machine_Radix",
               when Precomputed_Sym_Machine_Rounding => "Machine_Rounding",
               when Precomputed_Sym_Machine_Rounding_369 => "machine_rounding",
               when Precomputed_Sym_Machine_Rounds => "Machine_Rounds",
               when Precomputed_Sym_Mantissa => "Mantissa",
               when Precomputed_Sym_Max => "Max",
               when Precomputed_Sym_Max_370 => "max",
               when Precomputed_Sym_Max_Alignment_For_Allocation => "Max_Alignment_For_Allocation",
               when Precomputed_Sym_Max_Integer_Size => "Max_Integer_Size",
               when Precomputed_Sym_Max_Size_In_Storage_Elements => "Max_Size_In_Storage_Elements",
               when Precomputed_Sym_Maximum_Alignment => "Maximum_Alignment",
               when Precomputed_Sym_Mechanism_Code => "Mechanism_Code",
               when Precomputed_Sym_Min => "Min",
               when Precomputed_Sym_Min_371 => "min",
               when Precomputed_Sym_Mod => """mod""",
               when Precomputed_Sym_Mod_170 => "Mod",
               when Precomputed_Sym_Mod_372 => "mod",
               when Precomputed_Sym_Model => "Model",
               when Precomputed_Sym_Model_373 => "model",
               when Precomputed_Sym_Model_Emin => "Model_Emin",
               when Precomputed_Sym_Model_Epsilon => "Model_Epsilon",
               when Precomputed_Sym_Model_Mantissa => "Model_Mantissa",
               when Precomputed_Sym_Model_Of => "Model_Of",
               when Precomputed_Sym_Model_Small => "Model_Small",
               when Precomputed_Sym_Modulus => "Modulus",
               when Precomputed_Sym_New => "new",
               when Precomputed_Sym_Nextpart => "__nextpart",
               when Precomputed_Sym_No_Elaboration_Code => "No_Elaboration_Code",
               when Precomputed_Sym_No_Elaboration_Code_All => "No_Elaboration_Code_All",
               when Precomputed_Sym_No_Return => "No_Return",
               when Precomputed_Sym_Not => """not""",
               when Precomputed_Sym_Not_375 => "not",
               when Precomputed_Sym_Null => "null",
               when Precomputed_Sym_Numerics => "Numerics",
               when Precomputed_Sym_Object_Size => "Object_Size",
               when Precomputed_Sym_Obsolescent => "Obsolescent",
               when Precomputed_Sym_Of => "of",
               when Precomputed_Sym_Off => "Off",
               when Precomputed_Sym_Old => "Old",
               when Precomputed_Sym_On => "On",
               when Precomputed_Sym_Or => """or""",
               when Precomputed_Sym_Or_378 => "or",
               when Precomputed_Sym_Others => "others",
               when Precomputed_Sym_Out => "out",
               when Precomputed_Sym_Output => "Output",
               when Precomputed_Sym_Output_381 => "output",
               when Precomputed_Sym_Overlaps_Storage => "Overlaps_Storage",
               when Precomputed_Sym_Overriding => "overriding",
               when Precomputed_Sym_Pack => "Pack",
               when Precomputed_Sym_Package => "package",
               when Precomputed_Sym_Passed_By_Reference => "Passed_By_Reference",
               when Precomputed_Sym_Pos => "Pos",
               when Precomputed_Sym_Pos_384 => "pos",
               when Precomputed_Sym_Position => "Position",
               when Precomputed_Sym_Post => "Post",
               when Precomputed_Sym_Post_Class => "Post'Class",
               when Precomputed_Sym_Postcondition => "Postcondition",
               when Precomputed_Sym_Postcondition_Class => "Postcondition'Class",
               when Precomputed_Sym_Pragma => "pragma",
               when Precomputed_Sym_Pre => "Pre",
               when Precomputed_Sym_Pre_Class => "Pre'Class",
               when Precomputed_Sym_Precondition => "Precondition",
               when Precomputed_Sym_Precondition_Class => "Precondition'Class",
               when Precomputed_Sym_Pred => "Pred",
               when Precomputed_Sym_Pred_386 => "pred",
               when Precomputed_Sym_Predicate => "Predicate",
               when Precomputed_Sym_Predicate_Failure => "Predicate_Failure",
               when Precomputed_Sym_Preelaborable_Initialization => "Preelaborable_Initialization",
               when Precomputed_Sym_Preelaborate => "Preelaborate",
               when Precomputed_Sym_Private => "private",
               when Precomputed_Sym_Privatepart => "__privatepart",
               when Precomputed_Sym_Procedure => "procedure",
               when Precomputed_Sym_Protected => "protected",
               when Precomputed_Sym_Pure => "Pure",
               when Precomputed_Sym_Put_Image => "Put_Image",
               when Precomputed_Sym_Put_Image_390 => "put_image",
               when Precomputed_Sym_Raise => "raise",
               when Precomputed_Sym_Range => "Range",
               when Precomputed_Sym_Range_392 => "range",
               when Precomputed_Sym_Range_Length => "Range_Length",
               when Precomputed_Sym_Read => "Read",
               when Precomputed_Sym_Read_393 => "read",
               when Precomputed_Sym_Real_Literal => "Real_Literal",
               when Precomputed_Sym_Record => "record",
               when Precomputed_Sym_Reduce => "Reduce",
               when Precomputed_Sym_Refined_Depends => "Refined_Depends",
               when Precomputed_Sym_Refined_Global => "Refined_Global",
               when Precomputed_Sym_Refined_Post => "Refined_Post",
               when Precomputed_Sym_Refined_State => "Refined_State",
               when Precomputed_Sym_Rem => """rem""",
               when Precomputed_Sym_Rem_395 => "rem",
               when Precomputed_Sym_Remainder => "Remainder",
               when Precomputed_Sym_Remainder_396 => "remainder",
               when Precomputed_Sym_Remote_Call_Interface => "Remote_Call_Interface",
               when Precomputed_Sym_Remote_Types => "Remote_Types",
               when Precomputed_Sym_Renames => "renames",
               when Precomputed_Sym_Requeue => "requeue",
               when Precomputed_Sym_Requires => "Requires",
               when Precomputed_Sym_Restrictions => "Restrictions",
               when Precomputed_Sym_Result => "Result",
               when Precomputed_Sym_Return => "return",
               when Precomputed_Sym_Reverse => "reverse",
               when Precomputed_Sym_Right => "right",
               when Precomputed_Sym_Root_Buffer_Type => "Root_Buffer_Type",
               when Precomputed_Sym_Root_Integer => "root_integer",
               when Precomputed_Sym_Root_Real => "root_real",
               when Precomputed_Sym_Root_Storage_Pool => "Root_Storage_Pool",
               when Precomputed_Sym_Root_Stream_Type => "Root_Stream_Type",
               when Precomputed_Sym_Root_Types => "root_types_",
               when Precomputed_Sym_Round => "Round",
               when Precomputed_Sym_Round_405 => "round",
               when Precomputed_Sym_Rounding => "Rounding",
               when Precomputed_Sym_Rounding_406 => "rounding",
               when Precomputed_Sym_S => "S",
               when Precomputed_Sym_Safe_First => "Safe_First",
               when Precomputed_Sym_Safe_Large => "Safe_Large",
               when Precomputed_Sym_Safe_Last => "Safe_Last",
               when Precomputed_Sym_Safe_Small => "Safe_Small",
               when Precomputed_Sym_Scalar_Storage_Order => "Scalar_Storage_Order",
               when Precomputed_Sym_Scale => "Scale",
               when Precomputed_Sym_Scaling => "Scaling",
               when Precomputed_Sym_Scaling_407 => "scaling",
               when Precomputed_Sym_Select => "select",
               when Precomputed_Sym_Separate => "separate",
               when Precomputed_Sym_Shared_Passive => "Shared_Passive",
               when Precomputed_Sym_Signed_Zeros => "Signed_Zeros",
               when Precomputed_Sym_Size => "Size",
               when Precomputed_Sym_Skip_Flow_And_Proof => "Skip_Flow_And_Proof",
               when Precomputed_Sym_Skip_Proof => "Skip_Proof",
               when Precomputed_Sym_Small => "Small",
               when Precomputed_Sym_Small_Denominator => "Small_Denominator",
               when Precomputed_Sym_Small_Numerator => "Small_Numerator",
               when Precomputed_Sym_Some => "some",
               when Precomputed_Sym_Spark_Mode => "SPARK_Mode",
               when Precomputed_Sym_Stable_Properties => "Stable_Properties",
               when Precomputed_Sym_Standard => "Standard",
               when Precomputed_Sym_Standard_411 => "standard",
               when Precomputed_Sym_Static_Predicate => "Static_Predicate",
               when Precomputed_Sym_Storage_Elements => "Storage_Elements",
               when Precomputed_Sym_Storage_Pool => "Storage_Pool",
               when Precomputed_Sym_Storage_Pools => "Storage_Pools",
               when Precomputed_Sym_Storage_Size => "Storage_Size",
               when Precomputed_Sym_Storage_Unit => "Storage_Unit",
               when Precomputed_Sym_Stream_Size => "Stream_Size",
               when Precomputed_Sym_Streams => "Streams",
               when Precomputed_Sym_String => "String",
               when Precomputed_Sym_String_Literal => "String_Literal",
               when Precomputed_Sym_Strings => "Strings",
               when Precomputed_Sym_Structural => "Structural",
               when Precomputed_Sym_Style_Checks => "Style_Checks",
               when Precomputed_Sym_Subprogram_Variant => "Subprogram_Variant",
               when Precomputed_Sym_Subtype => "subtype",
               when Precomputed_Sym_Succ => "Succ",
               when Precomputed_Sym_Succ_413 => "succ",
               when Precomputed_Sym_Synchronization => "Synchronization",
               when Precomputed_Sym_Synchronized => "synchronized",
               when Precomputed_Sym_System => "System",
               when Precomputed_Sym_System_Allocator_Alignment => "System_Allocator_Alignment",
               when Precomputed_Sym_Tag => "Tag",
               when Precomputed_Sym_Tagged => "tagged",
               when Precomputed_Sym_Tags => "Tags",
               when Precomputed_Sym_Target_Name => "Target_Name",
               when Precomputed_Sym_Task => "task",
               when Precomputed_Sym_Task_Id => "Task_Id",
               when Precomputed_Sym_Task_Identification => "Task_Identification",
               when Precomputed_Sym_Terminate => "terminate",
               when Precomputed_Sym_Terminated => "Terminated",
               when Precomputed_Sym_Test_Case => "Test_Case",
               when Precomputed_Sym_Text_Buffers => "Text_Buffers",
               when Precomputed_Sym_Text_Io => "text_io",
               when Precomputed_Sym_Then => "then",
               when Precomputed_Sym_To_Address => "To_Address",
               when Precomputed_Sym_Truncation => "Truncation",
               when Precomputed_Sym_Truncation_420 => "truncation",
               when Precomputed_Sym_Type => "type",
               when Precomputed_Sym_Type_Class => "Type_Class",
               when Precomputed_Sym_Type_Invariant => "Type_Invariant",
               when Precomputed_Sym_Type_Invariant_Class => "Type_Invariant'Class",
               when Precomputed_Sym_Type_Key => "Type_Key",
               when Precomputed_Sym_Unbiased_Rounding => "Unbiased_Rounding",
               when Precomputed_Sym_Unbiased_Rounding_422 => "unbiased_rounding",
               when Precomputed_Sym_Unchecked_Access => "Unchecked_Access",
               when Precomputed_Sym_Unchecked_Union => "Unchecked_Union",
               when Precomputed_Sym_Unconstrained_Array => "Unconstrained_Array",
               when Precomputed_Sym_Universal_Fixed_Type => "Universal_Fixed_Type_",
               when Precomputed_Sym_Universal_Int_Type => "Universal_Int_Type_",
               when Precomputed_Sym_Universal_Real_Type => "Universal_Real_Type_",
               when Precomputed_Sym_Unreferenced => "Unreferenced",
               when Precomputed_Sym_Unrestricted_Access => "Unrestricted_Access",
               when Precomputed_Sym_Until => "until",
               when Precomputed_Sym_Update => "Update",
               when Precomputed_Sym_Use => "use",
               when Precomputed_Sym_Vads_Size => "VADS_Size",
               when Precomputed_Sym_Val => "Val",
               when Precomputed_Sym_Val_425 => "val",
               when Precomputed_Sym_Valid => "Valid",
               when Precomputed_Sym_Valid_Scalars => "Valid_Scalars",
               when Precomputed_Sym_Value => "Value",
               when Precomputed_Sym_Value_426 => "value",
               when Precomputed_Sym_Value_Size => "Value_Size",
               when Precomputed_Sym_Variable_Indexing => "Variable_Indexing",
               when Precomputed_Sym_Volatile => "Volatile",
               when Precomputed_Sym_Volatile_Components => "Volatile_Components",
               when Precomputed_Sym_Warnings => "Warnings",
               when Precomputed_Sym_Wchar_T_Size => "Wchar_T_Size",
               when Precomputed_Sym_When => "when",
               when Precomputed_Sym_While => "while",
               when Precomputed_Sym_Wide_Character => "Wide_Character",
               when Precomputed_Sym_Wide_Image => "Wide_Image",
               when Precomputed_Sym_Wide_Image_429 => "wide_image",
               when Precomputed_Sym_Wide_String => "Wide_String",
               when Precomputed_Sym_Wide_Text_Io => "wide_text_io",
               when Precomputed_Sym_Wide_Value => "Wide_Value",
               when Precomputed_Sym_Wide_Value_431 => "wide_value",
               when Precomputed_Sym_Wide_Wide_Character => "Wide_Wide_Character",
               when Precomputed_Sym_Wide_Wide_Image => "Wide_Wide_Image",
               when Precomputed_Sym_Wide_Wide_Image_432 => "wide_wide_image",
               when Precomputed_Sym_Wide_Wide_String => "Wide_Wide_String",
               when Precomputed_Sym_Wide_Wide_Text_Io => "wide_wide_text_io",
               when Precomputed_Sym_Wide_Wide_Value => "Wide_Wide_Value",
               when Precomputed_Sym_Wide_Wide_Value_434 => "wide_wide_value",
               when Precomputed_Sym_Width => "Width",
               when Precomputed_Sym_With => "with",
               when Precomputed_Sym_Word_Size => "Word_Size",
               when Precomputed_Sym_Write => "Write",
               when Precomputed_Sym_Write_436 => "write",
               when Precomputed_Sym_Xor => """xor""",
               when Precomputed_Sym_Xor_437 => "xor"
            );
            Symbol : constant Symbolization_Result :=
                  Libadalang.Sources.Canonicalize (Raw_Text)
            ;
         begin
            if Symbol.Success then
               return Symbol.Symbol;
            else
               raise Program_Error with
                 "Cannot canonicalize symbol literal: " & Image (Raw_Text);
            end if;
         end;
   end Precomputed_Symbol;
   ----------------------------
   -- Construct_Entity_Array --
   ----------------------------
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access
   is
      Ret : Internal_Entity_Array_Access :=
        Create_Internal_Entity_Array (V.Length);
   begin
      for J in V.First_Index .. V.Last_Index loop
         Ret.Items (J) := V.Get (J);
      end loop;
      declare
         Tmp : AST_Envs.Entity_Vectors.Vector := V;
      begin
         Tmp.Destroy;
      end;
      return Ret;
   end Construct_Entity_Array;
   -----------
   -- Image --
   -----------
   function Image (Self : Symbol_Type) return String_Type is
   begin
      return Create_String (Image (Self));
   end Image;
   ------------------
   -- Context_Pool --
   ------------------
   package body Context_Pool is
      -------------
      -- Acquire --
      -------------
      procedure Acquire (Context : out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;
         if Available.Is_Empty then
            Context := new Analysis_Context_Type;
            Context.Serial_Number := 1;
         else
            Context := Available.Last_Element;
            Available.Delete_Last;
         end if;
         GNAT.Task_Lock.Unlock;
         Context.Initialized := False;
         Context.Ref_Count := 1;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Acquire;
      -------------
      -- Release --
      -------------
      procedure Release (Context : in out Internal_Context) is
      begin
         GNAT.Task_Lock.Lock;
         Available.Append (Context);
         Context.Serial_Number := Context.Serial_Number + 1;
         Context := null;
         GNAT.Task_Lock.Unlock;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Release;
      ----------
      -- Free --
      ----------
      procedure Free is
      begin
         GNAT.Task_Lock.Lock;
         for C of Available loop
            Free (C);
         end loop;
         GNAT.Task_Lock.Unlock;
      exception
         when others =>
            GNAT.Task_Lock.Unlock;
            raise;
      end Free;
   end Context_Pool;
   --------------
   -- Finalize --
   --------------
   overriding procedure Finalize (CD : in out Contexts_Destructor) is
      pragma Unreferenced (CD);
   begin
      Context_Pool.Free;
   end Finalize;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (File_Reader : in out Internal_File_Reader_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_File_Reader'Class, Internal_File_Reader_Access);
   begin
      if File_Reader /= null and then File_Reader.all.Dec_Ref then
         Destroy (File_Reader);
      end if;
   end Dec_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Self : in out Internal_Event_Handler_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Event_Handler'Class, Internal_Event_Handler_Access);
   begin
      if Self /= null and then Self.all.Dec_Ref then
         Destroy (Self);
      end if;
   end Dec_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Provider : in out Internal_Unit_Provider_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Unit_Provider'Class, Internal_Unit_Provider_Access);
   begin
      if Provider /= null and then Provider.all.Dec_Ref then
         Destroy (Provider);
      end if;
   end Dec_Ref;
   ----------------
   -- Get_Env_Id --
   ----------------
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String
   is
      C        : Address_To_Id_Maps.Cursor;
      Inserted : Boolean;
   begin
      if E = Null_Lexical_Env then
         return "$null";
      elsif E = State.Root_Env then
         --  Insert root env with a special Id so that we only print it once
         State.Env_Ids.Insert (E, -1, C, Inserted);
         return "$root";
      end if;
      State.Env_Ids.Insert (E, State.Next_Id, C, Inserted);
      if Inserted then
         State.Next_Id := State.Next_Id + 1;
      end if;
      return '@' & Stripped_Image (Address_To_Id_Maps.Element (C));
   end Get_Env_Id;
   pragma Warnings (Off, "referenced");
   function To_Lookup_Kind_Type (K : Lookup_Kind) return Lookup_Kind_Type
   is
     (Lookup_Kind_Type'Val (Lookup_Kind'Pos (K)));
   pragma Warnings (On, "referenced");
   ----------------------
   -- Allocate_Context --
   ----------------------
   function Allocate_Context return Internal_Context is
   begin
      return Context : Internal_Context do
         Context_Pool.Acquire (Context);
      end return;
   end Allocate_Context;
   ------------------------
   -- Initialize_Context --
   ------------------------
   procedure Initialize_Context
     (Context        : Internal_Context;
      Charset        : String;
      File_Reader    : Internal_File_Reader_Access;
      Unit_Provider  : Internal_Unit_Provider_Access;
      Event_Handler  : Internal_Event_Handler_Access;
      With_Trivia    : Boolean;
      Tab_Stop       : Positive)
   is
      Actual_Charset : constant String :=
        (if Charset = "" then Default_Charset else Charset);
      Symbols        : constant Precomputed_Symbol_Table
        := Create_Symbol_Table;
   begin
      Context.Initialized := True;
      Context.Symbols := Symbol_Table (Symbols);
      Context.Charset := To_Unbounded_String (Actual_Charset);
      Context.Tab_Stop := Tab_Stop;
      Context.With_Trivia := With_Trivia;
      Context.Root_Scope := Create_Static_Lexical_Env
        (Parent    => Null_Lexical_Env,
         Node      => null,
         Sym_Table => Context.Symbols);
      --  Create a new ownership share for Event_Handler so that it lives at
      --  least as long as this analysis context.
      Context.Event_Handler := Event_Handler;
      if Context.Event_Handler /= null then
         Context.Event_Handler.Inc_Ref;
      end if;
      --  Create a new ownership share for File_Reader so that it lives at
      --  least as long as this analysis context.
      Context.File_Reader := File_Reader;
      if Context.File_Reader /= null then
         Context.File_Reader.Inc_Ref;
      end if;
      --  Create a new ownership share for Unit_Provider so that it lives at
      --  least as long as this analysis context.
      Context.Unit_Provider := Unit_Provider;
      if Context.Unit_Provider /= null then
         Context.Unit_Provider.Inc_Ref;
      end if;
         if Context.Unit_Provider = null then
            Context.Unit_Provider := Libadalang.Internal_Default_Provider.Create;
         end if;
      Initialize (Context.Parser);
      Context.Discard_Errors_In_Populate_Lexical_Env := True;
      Context.Logic_Resolution_Timeout :=
        Langkit_Support.Adalog.Default_Timeout_Ticks_Number;
      Context.In_Populate_Lexical_Env := False;
      Context.Cache_Version := 0;
      Context.Reparse_Cache_Version := 0;
      Context.Rewriting_Handle := No_Rewriting_Handle_Pointer;
      Context.Templates_Unit := No_Analysis_Unit;
      Context.Available_Rebindings := Env_Rebindings_Vectors.Empty_Vector;
Fetch_Standard (Context);
   end Initialize_Context;
   -----------------
   -- Create_Unit --
   -----------------
   function Create_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Unit : Internal_Unit;
   begin
      Unit := Create_Special_Unit
        (Context, Normalized_Filename, Charset, Rule);
      Context.Units.Insert (Normalized_Filename, Unit);
      return Unit;
   end Create_Unit;
   --------------
   -- Get_Unit --
   --------------
   function Get_Unit
     (Context           : Internal_Context;
      Filename, Charset : String;
      Reparse           : Boolean;
      Input             : Internal_Lexer_Input;
      Rule              : Grammar_Rule;
      Is_Internal       : Boolean := False) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant GNATCOLL.VFS.Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur     : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
      Created : constant Boolean := Cur = No_Element;
      Unit    : Internal_Unit;
      Actual_Charset : Unbounded_String;
      Refined_Input  : Internal_Lexer_Input := Input;
   begin
      --  Determine which encoding to use. Use the Charset parameter (if
      --  provided), otherwise use the context-wide default.
      Actual_Charset := (if Charset'Length /= 0
                         then To_Unbounded_String (Charset)
                         else Context.Charset);
      if Refined_Input.Kind = File then
         Refined_Input.Filename := Normalized_Filename;
      end if;
      if Refined_Input.Kind in File | Bytes_Buffer then
         Refined_Input.Charset := Actual_Charset;
         --  Unless the caller requested a specific charset for this unit,
         --  allow the lexer to automatically discover the source file encoding
         --  before defaulting to the context-specific one. We do this trying
         --  to match a byte order mark.
         Refined_Input.Read_BOM := Charset'Length = 0;
      end if;
      --  Create the Internal_Unit if needed
      Unit :=
        (if Created
         then Create_Unit (Context, Normalized_Filename,
                           To_String (Actual_Charset), Rule)
         else Element (Cur));
      --  If an internal unit is requested, set the corresponding flag.
      --  Otherwise, make sure that the unit we return isn't internal.
      if Is_Internal then
         Unit.Is_Internal := True;
      end if;
      --  (Re)parse it if needed
      if Created or else Reparse then
         --  It is illegal to reparse an internal unit for public API users.
         --  Since public APIs do not allow to pass True to Is_Internal, we can
         --  check here that only the implementation can ask to reparse an
         --  internal unit.
         if Unit.Is_Internal and then not Is_Internal then
            raise Precondition_Failure with "cannot reparse an internal unit";
         end if;
         declare
            Reparsed : Reparsed_Unit;
         begin
            Do_Parsing (Unit, Refined_Input, Reparsed);
            Update_After_Reparse (Unit, Reparsed);
         end;
         --  Now that we have removed reparsed the unit, update its current
         --  charset.
         Unit.Charset := Actual_Charset;
      end if;
      if Context.Event_Handler /= null then
         Context.Event_Handler.Unit_Parsed_Callback
           (Context  => Context,
            Unit     => Unit,
            Reparsed => not Created and then Reparse);
      end if;
      return Unit;
   end Get_Unit;
   --------------
   -- Has_Unit --
   --------------
   function Has_Unit
     (Context : Internal_Context; Unit_Filename : String) return Boolean is
   begin
      return Context.Units.Contains
        (Normalized_Unit_Filename (Context, Unit_Filename));
   end Has_Unit;
   -------------------
   -- Get_From_File --
   -------------------
   function Get_From_File
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Reparse  : Boolean;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind     => File,
         Charset  => <>,
         Read_BOM => False,
         Filename => <>);
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      return Get_Unit (Context, Filename, Charset, Reparse, Input, Rule);
   end Get_From_File;
   ---------------------
   -- Get_From_Buffer --
   ---------------------
   function Get_From_Buffer
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Buffer   : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind        => Bytes_Buffer,
         Charset     => <>,
         Read_BOM    => False,
         Bytes       => Buffer'Address,
         Bytes_Count => Buffer'Length);
   begin
      if Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot parse from buffer during tree rewriting";
      elsif Context.File_Reader /= null then
         raise Precondition_Failure with
            "cannot parse from buffer with a file reader";
      end if;
      return Get_Unit (Context, Filename, Charset, True, Input, Rule);
   end Get_From_Buffer;
   --------------------
   -- Get_With_Error --
   --------------------
   function Get_With_Error
     (Context  : Internal_Context;
      Filename : String;
      Error    : Text_Type;
      Charset  : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur                 : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
   begin
      if Cur = No_Element then
         declare
            Unit : constant Internal_Unit := Create_Unit
              (Context, Normalized_Filename, Charset, Rule);
         begin
            Append (Unit.Diagnostics, No_Source_Location_Range, Error);
            return Unit;
         end;
      else
         return Element (Cur);
      end if;
   end Get_With_Error;
   -----------------------
   -- Get_From_Provider --
   -----------------------
   function Get_From_Provider
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Charset : String;
      Reparse : Boolean) return Internal_Unit is
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      declare
         Result      : Internal_Unit;
         Dummy_Index : Positive;
      begin
         Context.Unit_Provider.Get_Unit_And_PLE_Root
           (Context, Name, Kind, Charset, Reparse, Result, Dummy_Index);
         return Result;
      exception
         when Precondition_Failure | Property_Error =>
            raise Invalid_Unit_Name_Error with
               "Invalid unit name: " & Image (Name, With_Quotes => True)
               & " (" & Analysis_Unit_Kind'Image (Kind) & ")";
      end;
   end Get_From_Provider;
   -------------------
   -- Unit_Provider --
   -------------------
   function Unit_Provider
     (Context : Internal_Context) return Internal_Unit_Provider_Access
   is (Context.Unit_Provider);
   ------------------
   -- Resolve_Unit --
   ------------------
   procedure Resolve_Unit
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Unit    : out Resolved_Unit)
   is
      --  Look for the cache entry corresponding to Unit; create one if needed
      Dummy    : Resolved_Unit_Array;
      Key      : constant Symbol_Type := Find (Context.Symbols, Name);
      Pos      : Unit_Provider_Cache_Maps.Cursor;
      Inserted : Boolean;
   begin
      Context.Unit_Provider_Cache.Insert (Key, Dummy, Pos, Inserted);
      declare
         Units : Resolved_Unit_Array renames
           Context.Unit_Provider_Cache.Reference (Pos);
         U     : Resolved_Unit renames Units (Kind);
      begin
         --  If the cache entry is not populated for the requested kind, run
         --  the query and save the result for later requests.
         if U.Filename = null then
            declare
               Provider : Internal_Unit_Provider'Class renames
                 Context.Unit_Provider.all;
               Filename : Unbounded_String;
            begin
               Provider.Get_Unit_Location
                 (Name           => Name,
                  Kind           => Kind,
                  Filename       => Filename,
                  PLE_Root_Index => U.PLE_Root_Index);
               Provider.Get_Unit_And_PLE_Root
                 (Context        => Context,
                  Name           => Name,
                  Kind           => Kind,
                  Unit           => U.Unit,
                  PLE_Root_Index => U.PLE_Root_Index);
               U.Filename := new String'(To_String (Filename));
            end;
         end if;
         Unit := U;
      end;
   end Resolve_Unit;
   -----------------------
   -- Get_Unit_Location --
   -----------------------
   procedure Get_Unit_Location
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Filename       : out String_Access;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Filename := U.Filename;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_Location;
   ---------------------------
   -- Get_Unit_And_PLE_Root --
   ---------------------------
   procedure Get_Unit_And_PLE_Root
     (Context        : Internal_Context;
      Name           : Text_Type;
      Kind           : Analysis_Unit_Kind;
      Unit           : out Internal_Unit;
      PLE_Root_Index : out Positive)
   is
      U : Resolved_Unit;
   begin
      Resolve_Unit (Context, Name, Kind, U);
      Unit := U.Unit;
      PLE_Root_Index := U.PLE_Root_Index;
   end Get_Unit_And_PLE_Root;
   ----------
   -- Hash --
   ----------
   function Hash (Context : Internal_Context) return Hash_Type is
      function H is new Hash_Access (Analysis_Context_Type, Internal_Context);
   begin
      return H (Context);
   end Hash;
   ---------------------
   -- Has_With_Trivia --
   ---------------------
   function Has_With_Trivia (Context : Internal_Context) return Boolean is
   begin
      return Context.With_Trivia;
   end Has_With_Trivia;
   --------------------------------------------
   -- Discard_Errors_In_Populate_Lexical_Env --
   --------------------------------------------
   procedure Discard_Errors_In_Populate_Lexical_Env
     (Context : Internal_Context; Discard : Boolean) is
   begin
      Context.Discard_Errors_In_Populate_Lexical_Env := Discard;
   end Discard_Errors_In_Populate_Lexical_Env;
   ----------------------------------
   -- Set_Logic_Resolution_Timeout --
   ----------------------------------
   procedure Set_Logic_Resolution_Timeout
     (Context : Internal_Context; Timeout : Natural) is
   begin
      Context.Logic_Resolution_Timeout := Timeout;
   end Set_Logic_Resolution_Timeout;
   --------------------------
   -- Has_Rewriting_Handle --
   --------------------------
   function Has_Rewriting_Handle (Context : Internal_Context) return Boolean is
   begin
      return Context.Rewriting_Handle /= No_Rewriting_Handle_Pointer;
   end Has_Rewriting_Handle;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Context : Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Context : in out Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count - 1;
         if Context.Ref_Count = 0 then
            --  If this context was not completely initialized, just release
            --  the allocated object. Do the full destruction otherwise.
            if Context.Initialized then
               Destroy (Context);
            end if;
            Context_Pool.Release (Context);
         end if;
      end if;
   end Dec_Ref;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Context : Internal_Context) is
   begin
      --  Destroy all named environment data structures
      for Desc of Context.Named_Envs loop
         for V of Desc.Foreign_Nodes loop
            V.Destroy;
         end loop;
         Destroy (Desc);
      end loop;
      Context.Named_Envs.Clear;
      --  If we are asked to free this context, it means that no one else have
      --  references to its analysis units, so it's safe to destroy these.
      for Unit of Context.Units loop
         Destroy (Unit);
      end loop;
      Context.Units := Units_Maps.Empty_Map;
      Context.Filenames := Virtual_File_Maps.Empty_Map;
      declare
         procedure Destroy is new Ada.Unchecked_Deallocation
           (Env_Rebindings_Type, Env_Rebindings);
         AR : Env_Rebindings_Vectors.Vector renames
            Context.Available_Rebindings;
         R  : Env_Rebindings;
      begin
         for I in AR.First_Index .. AR.Last_Index loop
            R := AR.Get (I);
            Destroy (R);
         end loop;
         AR.Destroy;
      end;
      for Pos in Context.Unit_Provider_Cache.Iterate loop
         declare
            Units : Resolved_Unit_Array renames
              Context.Unit_Provider_Cache.Reference (Pos);
         begin
            for U of Units loop
               Free (U.Filename);
            end loop;
         end;
      end loop;
      Context.Unit_Provider_Cache.Clear;
      Destroy (Context.Templates_Unit);
      AST_Envs.Destroy (Context.Root_Scope);
      Destroy (Context.Symbols);
      Destroy (Context.Parser);
      Dec_Ref (Context.File_Reader);
      Dec_Ref (Context.Unit_Provider);
      Dec_Ref (Context.Event_Handler);
   end Destroy;
   -------------
   -- Context --
   -------------
   function Context (Unit : Internal_Unit) return Internal_Context is
   begin
      return Unit.Context;
   end Context;
   ----------
   -- Hash --
   ----------
   function Hash (Unit : Internal_Unit) return Hash_Type is
      function H is new Hash_Access (Analysis_Unit_Type, Internal_Unit);
   begin
      return H (Unit);
   end Hash;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String) is
      Dummy : constant Internal_Unit := Get_From_File
        (Unit.Context, +Unit.Filename.Full_Name, Charset,
         Reparse => True,
         Rule    => Unit.Rule);
   begin
      null;
   end Reparse;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String; Buffer : String)
   is
      Dummy : constant Internal_Unit := Get_From_Buffer
        (Unit.Context, +Unit.Filename.Full_Name, Charset, Buffer, Unit.Rule);
   begin
      null;
   end Reparse;
   -----------------------
   -- Reset_Envs_Caches --
   -----------------------
   procedure Reset_Envs_Caches (Unit : Internal_Unit) is
      procedure Internal (Node : Bare_Ada_Node);
      --  Reset env caches in ``Node`` and then in its children recursively
      Generic_Unit : constant Generic_Unit_Ptr := Convert_Unit (Unit);
      --------------
      -- Internal --
      --------------
      procedure Internal (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         --  Make sure to only reset caches of envs belonging to this unit
         if Node.Self_Env.Owner = Generic_Unit then
            Reset_Caches (Node.Self_Env);
         end if;
         for I in 1 .. Children_Count (Node) loop
            Internal (Child (Node, I));
         end loop;
      end Internal;
   begin
      Internal (Unit.Ast_Root);
   end Reset_Envs_Caches;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   procedure Populate_Lexical_Env
     (Unit           : Internal_Unit;
      PLE_Root_Index : Positive
   ) is
      Context : constant Internal_Context := Unit.Context;
      Saved_In_Populate_Lexical_Env : constant Boolean :=
        Context.In_Populate_Lexical_Env;
      Has_Errors : Boolean := False;
      --  Whether at least one Property_Error occurred during this PLE pass
   begin
      --  TODO??? Handle env invalidation when reparsing a unit and when a
      --  previous call raised a Property_Error.
      --  If we have already run PLE on this root, there is nothing to do.
      --  Otherwise, keep track of the fact that PLE was requested for it,
      --  possibly extending the vector if needed.
      if Unit.Env_Populated_Roots.Last_Index >= PLE_Root_Index
         and then Unit.Env_Populated_Roots.Get (PLE_Root_Index)
      then
         return;
      end if;
      for Dummy in Unit.Env_Populated_Roots.Last_Index + 1 .. PLE_Root_Index
      loop
         Unit.Env_Populated_Roots.Append (False);
      end loop;
      Unit.Env_Populated_Roots.Set (PLE_Root_Index, True);
      --  Create context for the PLE run: all exit points must call the Cleanup
      --  procedure above first to clean this context.
      Context.In_Populate_Lexical_Env := True;
      if Main_Trace.Active then
         Main_Trace.Trace
           ("Populating lexical envs for"
            & " root" & PLE_Root_Index'Image & " of"
            & " unit: " & Basename (Unit));
         Main_Trace.Increase_Indent;
      end if;
      --  Fetch the node on which to run PLE: it's the unit root node, or one
      --  of its children if PLE roots are enabled and the unit has a list of
      --  PLE roots. Then run PLE itself.
      declare
         PLE_Root : Bare_Ada_Node := Unit.Ast_Root;
      begin
            if Unit.Ast_Root /= null
               and then Unit.Ast_Root.Kind
                        = Ada_Compilation_Unit_List
            then
               PLE_Root := Child (Unit.Ast_Root, PLE_Root_Index);
            end if;
         if PLE_Root /= null then
            Has_Errors := Populate_Lexical_Env (PLE_Root);
         end if;
      end;
      --  Restore the context for PLE run (undo what was done above)
      Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
      if Main_Trace.Active then
         Main_Trace.Decrease_Indent;
         Main_Trace.Trace
           ("Finished populating lexical envs for"
            & " root" & PLE_Root_Index'Image & " of"
            & " unit: " & Basename (Unit));
      end if;
      Reset_Envs_Caches (Unit);
      if Has_Errors and then not Context.Discard_Errors_In_Populate_Lexical_Env
      then
         raise Property_Error with
            "errors occurred in Populate_Lexical_Env";
      end if;
   end Populate_Lexical_Env;
   -----------------------------------
   -- Populate_Lexical_Env_For_Unit --
   -----------------------------------
   procedure Populate_Lexical_Env_For_Unit (Node : Bare_Ada_Node) is
      Root  : Bare_Ada_Node;
      Index : Natural;
   begin
      Lookup_PLE_Root (Node, Root, Index);
      if Index = 0 then
         Index := 1;
      end if;
      Populate_Lexical_Env (Node.Unit, Index);
   end Populate_Lexical_Env_For_Unit;
   ------------------
   -- Get_Filename --
   ------------------
   function Get_Filename (Unit : Internal_Unit) return String is
     (+Unit.Filename.Full_Name);
   -----------------
   -- Get_Charset --
   -----------------
   function Get_Charset (Unit : Internal_Unit) return String is
   begin
      return To_String (Unit.Charset);
   end Get_Charset;
   ---------------------
   -- Has_Diagnostics --
   ---------------------
   function Has_Diagnostics (Unit : Internal_Unit) return Boolean is
   begin
      return not Unit.Diagnostics.Is_Empty;
   end Has_Diagnostics;
   -----------------
   -- Diagnostics --
   -----------------
   function Diagnostics (Unit : Internal_Unit) return Diagnostics_Array is
      Result : Diagnostics_Array (1 .. Natural (Unit.Diagnostics.Length));
      I      : Natural := 1;
   begin
      for D of Unit.Diagnostics loop
         Result (I) := D;
         I := I + 1;
      end loop;
      return Result;
   end Diagnostics;
   ---------------------------
   -- Format_GNU_Diagnostic --
   ---------------------------
   function Format_GNU_Diagnostic
     (Unit : Internal_Unit; D : Diagnostic) return String
   is
      Filename : constant String := Basename (Unit);
      Sloc     : constant Source_Location := Start_Sloc (D.Sloc_Range);
      Msg      : constant String :=
         Image
           (Ada.Strings.Wide_Wide_Unbounded.To_Wide_Wide_String (D.Message));
   begin
      return (Filename
              & (if Sloc = No_Source_Location then "" else ":" & Image (Sloc))
              & ": " & Msg);
   end Format_GNU_Diagnostic;
   ----------
   -- Root --
   ----------
   function Root (Unit : Internal_Unit) return Bare_Ada_Node is
     (Unit.Ast_Root);
   -----------------
   -- First_Token --
   -----------------
   function First_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            First_Token_Or_Trivia (Unit.TDH)));
   ----------------
   -- Last_Token --
   ----------------
   function Last_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.Context,
                            Unit.TDH'Access,
                            Last_Token_Or_Trivia (Unit.TDH)));
   -----------------
   -- Token_Count --
   -----------------
   function Token_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Tokens.Length);
   ------------------
   -- Trivia_Count --
   ------------------
   function Trivia_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Trivias.Length);
   ----------
   -- Text --
   ----------
   function Text (Unit : Internal_Unit) return Text_Type is
   begin
      return Text (First_Token (Unit), Last_Token (Unit));
   end Text;
   ------------------
   -- Lookup_Token --
   ------------------
   function Lookup_Token
     (Unit : Internal_Unit; Sloc : Source_Location) return Token_Reference
   is
      Result : constant Token_Or_Trivia_Index := Lookup_Token (Unit.TDH, Sloc);
   begin
      return Wrap_Token_Reference (Unit.Context, Unit.TDH'Access, Result);
   end Lookup_Token;
   ---------------------
   -- Lookup_PLE_Root --
   ---------------------
   procedure Lookup_PLE_Root
     (Node  : Bare_Ada_Node;
      Root  : out Bare_Ada_Node;
      Index : out Natural)
   is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  If this unit does not contain a list of PLE roots, just return the
      --  unit root node.
      if Unit.PLE_Roots_Starting_Token.Is_Empty then
         Root := Unit.Ast_Root;
         Index := 0;
         return;
      end if;
      --  Otherwise, look for the last PLE root whose first token (in
      --  Unit.PLE_Roots_Starting_Token) appears before Node's (T). This vector
      --  is sorted by construction, so we can perform a binary search.
      declare
         T      : constant Token_Index := Node.Token_Start_Index;
         Tokens : Token_Index_Vectors.Vector renames
           Unit.PLE_Roots_Starting_Token;
         First : Positive := Tokens.First_Index;
         Last  : Positive := Tokens.Last_Index;
         I     : Positive;
      begin
         while First < Last loop
            --  Because we look for the "floor" (last element that is <= T), we
            --  need to look at the value in Last when there are only two
            --  elements left to look at. If we did not do that, then we would
            --  go into an infinite loop when Tokens[First] < T.
            I := (if First + 1 = Last
                  then Last
                  else (First + Last) / 2);
            declare
               I_T : constant Token_Index := Tokens.Get (I);
            begin
               if I_T <= T then
                  First := I;
               else
                  Last := I - 1;
               end if;
            end;
         end loop;
         Root := Child (Unit.Ast_Root, First);
         Index := First;
      end;
   end Lookup_PLE_Root;
   --------------
   -- Ple_Root --
   --------------
   function Ple_Root
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is
      Root        : Bare_Ada_Node;
      Dummy_Index : Natural;
   begin
      if Node = null then
         raise Property_Error with "null node dereference";
      end if;
      Lookup_PLE_Root (Node, Root, Dummy_Index);
      return Root;
   end Ple_Root;
   ----------------------
   -- Dump_Lexical_Env --
   ----------------------
   procedure Dump_Lexical_Env (Unit : Internal_Unit) is
      Node     : constant Bare_Ada_Node := Unit.Ast_Root;
      Root_Env : constant Lexical_Env := Unit.Context.Root_Scope;
      State    : Dump_Lexical_Env_State := (Root_Env => Root_Env, others => <>);
      function Get_Parent (Env : Lexical_Env) return Lexical_Env
      is (Unwrap (Env).Parent);
      --------------------------
      -- Explore_Parent_Chain --
      --------------------------
      procedure Explore_Parent_Chain (Env : Lexical_Env) is
         P : Lexical_Env;
      begin
         if Env /= Null_Lexical_Env then
            P := Get_Parent (Env);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (P, State));
            Explore_Parent_Chain (P);
         end if;
      end Explore_Parent_Chain;
      --------------
      -- Internal --
      --------------
      procedure Internal (Current : Bare_Ada_Node) is
         Explore_Parent : Boolean := False;
         Env, Parent    : Lexical_Env;
      begin
         if Current = null then
            return;
         end if;
         --  We only dump environments that we haven't dumped before. This way
         --  we'll only dump environments at the site of their creation, and
         --  not in any subsequent link. We use the Env_Ids map to check which
         --  envs we have already seen or not.
         if not State.Env_Ids.Contains (Current.Self_Env) then
            Env := Current.Self_Env;
            Parent := Get_Parent (Env);
            Explore_Parent := not State.Env_Ids.Contains (Parent);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (Parent, State));
            if Explore_Parent then
               Explore_Parent_Chain (Parent);
            end if;
         end if;
         for Child of Internal_Bare_Ada_Node_Array'(Children (Current))
         loop
            Internal (Child);
         end loop;
      end Internal;
      --  This procedure implements the main recursive logic of dumping the
      --  environments.
   begin
      Internal (Bare_Ada_Node (Node));
   end Dump_Lexical_Env;
   --------------
   -- Get_Line --
   --------------
   function Get_Line
     (Unit : Internal_Unit; Line_Number : Positive) return Text_Type
   is
   begin
      return Get_Line (Unit.TDH, Line_Number);
   end Get_Line;
   -----------
   -- Print --
   -----------
   procedure Print (Unit : Internal_Unit; Show_Slocs : Boolean) is
   begin
      if Unit.Ast_Root = null then
         Put_Line ("<empty analysis unit>");
      else
         Print (Unit.Ast_Root, Show_Slocs);
      end if;
   end Print;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia (Unit : Internal_Unit) is
      procedure Process (Trivia : Token_Index) is
         Data : constant Stored_Token_Data :=
            Unit.TDH.Trivias.Get (Natural (Trivia)).T;
      begin
         Put_Line (Image (Text (Unit.TDH, Data)));
      end Process;
      Last_Token : constant Token_Index :=
         Token_Index (Token_Vectors.Last_Index (Unit.TDH.Tokens) - 1);
      --  Index for the last token in Unit excluding the Termination token
      --  (hence the -1).
   begin
      for Tok of Get_Leading_Trivias (Unit.TDH) loop
         Process (Tok);
      end loop;
      PP_Trivia (Unit.Ast_Root);
      for Tok of Get_Trivias (Unit.TDH, Last_Token) loop
         Process (Tok);
      end loop;
   end PP_Trivia;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Unit : in out Internal_Unit) is
   begin
      if Unit = No_Analysis_Unit then
         return;
      end if;
      --  Clear the env caches while the unit is still fully alive to make sure
      --  that what is accessed in ``Lexical_Env_Cache_Updated`` is still
      --  valid, as it will be called back by lexical envs that are being
      --  destroyed.
      Reset_Envs_Caches (Unit);
      Unit.PLE_Roots_Starting_Token.Destroy;
      Unit.Env_Populated_Roots.Destroy;
      Unit.Exiled_Entries.Destroy;
      Unit.Foreign_Nodes.Destroy;
      Unit.Exiled_Entries_In_NED.Destroy;
      Unit.Exiled_Envs.Destroy;
      Unit.Named_Envs.Destroy;
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);
         Destroy (Unit.Memoization_Map);
      Destroy_Rebindings (Unit.Rebindings'Access);
      Unit.Rebindings.Destroy;
      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;
      Free (Unit.TDH);
      Free (Unit.Ast_Mem_Pool);
      Destroy_Unit_Destroyables (Unit);
      Destroyable_Vectors.Destroy (Unit.Destroyables);
for Cur in Unit.Nodes_Nameres.Iterate loop
   declare
      V : Resolution_Val renames Unit.Nodes_Nameres.Reference (Cur);
   begin
      Free_Memoized_Error (V.Exc_Id, V.Exc_Msg);
      Dec_Ref (V.Return_Value);
   end;
end loop;
      Free (Unit);
   end Destroy;
   -------------------
   -- Is_Token_Node --
   -------------------
   function Is_Token_Node (Node : Bare_Ada_Node) return Boolean is
   begin
      return Is_Token_Node (Node.Kind);
   end Is_Token_Node;
   ------------------
   -- Is_Synthetic --
   ------------------
   function Is_Synthetic (Node : Bare_Ada_Node) return Boolean is
   begin
      return Node.Kind in Synthetic_Nodes;
   end Is_Synthetic;
   ------------------------------
   -- Raise_Property_Exception --
   ------------------------------
   procedure Raise_Property_Exception
     (Node    : Bare_Ada_Node;
      Exc     : Ada.Exceptions.Exception_Id;
      Message : String)
   is
      Sloc_Prefix : constant String :=
        (if Node = null
         then ""
         else Ada.Directories.Simple_Name (Get_Filename (Unit (Node)))
              & ":" & Image (Sloc_Range (Node)) & ": ");
   begin
      Ada.Exceptions.Raise_Exception (Exc, Sloc_Prefix & Message);
   end Raise_Property_Exception;
   ------------------------------
   -- Register_Destroyable_Gen --
   ------------------------------
   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access)
   is
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Destroy_Procedure);
      procedure Destroy_Procedure (Object : in out T_Access) renames Destroy;
   begin
      Register_Destroyable_Helper
        (Unit,
         Object.all'Address,
         Convert (Destroy_Procedure'Address));
   end Register_Destroyable_Gen;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Ada_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Absolute_Get
        (T : Bare_Ada_Node_Array_Access; Index : Integer)
         return Bare_Ada_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Ada_Node_Array_Access) return Bare_Ada_Node_Array_Access is
      Ret : Bare_Ada_Node_Array_Access := Create_Bare_Ada_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Ada_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Ada_Node_Array (Items_Count : Natural) return Bare_Ada_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Ada_Node_Array_Type
       else new Bare_Ada_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Ada_Node_Array
     (Items : Internal_Bare_Ada_Node_Array) return Bare_Ada_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Ada_Node_Array_Type;
      end if;
      return new Bare_Ada_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Ada_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Anonymous_Expr_Decl_Vectors is new Langkit_Support.Vectors (Bare_Anonymous_Expr_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Anonymous_Expr_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Anonymous_Expr_Decl
   is
      function Absolute_Get
        (T : Bare_Anonymous_Expr_Decl_Array_Access; Index : Integer)
         return Bare_Anonymous_Expr_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Anonymous_Expr_Decl,
         Sequence_Type => Bare_Anonymous_Expr_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Anonymous_Expr_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Bare_Anonymous_Expr_Decl_Array_Access is
      Ret : Bare_Anonymous_Expr_Decl_Array_Access := Create_Bare_Anonymous_Expr_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Anonymous_Expr_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Anonymous_Expr_Decl_Array (Items_Count : Natural) return Bare_Anonymous_Expr_Decl_Array_Access
   is (if Items_Count = 0
       then No_Bare_Anonymous_Expr_Decl_Array_Type
       else new Bare_Anonymous_Expr_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Anonymous_Expr_Decl_Array
     (Items : Internal_Bare_Anonymous_Expr_Decl_Array) return Bare_Anonymous_Expr_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Anonymous_Expr_Decl_Array_Type;
      end if;
      return new Bare_Anonymous_Expr_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Anonymous_Expr_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Bare_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Compilation_Unit
   is
      function Absolute_Get
        (T : Bare_Compilation_Unit_Array_Access; Index : Integer)
         return Bare_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Compilation_Unit,
         Sequence_Type => Bare_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Compilation_Unit_Array_Access) return Bare_Compilation_Unit_Array_Access is
      Ret : Bare_Compilation_Unit_Array_Access := Create_Bare_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Compilation_Unit_Array (Items_Count : Natural) return Bare_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Bare_Compilation_Unit_Array_Type
       else new Bare_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Compilation_Unit_Array
     (Items : Internal_Bare_Compilation_Unit_Array) return Bare_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Compilation_Unit_Array_Type;
      end if;
      return new Bare_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Compilation_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Name_Vectors is new Langkit_Support.Vectors (Bare_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Name
   is
      function Absolute_Get
        (T : Bare_Name_Array_Access; Index : Integer)
         return Bare_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Name,
         Sequence_Type => Bare_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Name_Array_Access) return Bare_Name_Array_Access is
      Ret : Bare_Name_Array_Access := Create_Bare_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Name_Array (Items_Count : Natural) return Bare_Name_Array_Access
   is (if Items_Count = 0
       then No_Bare_Name_Array_Type
       else new Bare_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Name_Array
     (Items : Internal_Bare_Name_Array) return Bare_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Name_Array_Type;
      end if;
      return new Bare_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Pragma_Node_Vectors is new Langkit_Support.Vectors (Bare_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Pragma_Node
   is
      function Absolute_Get
        (T : Bare_Pragma_Node_Array_Access; Index : Integer)
         return Bare_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Pragma_Node,
         Sequence_Type => Bare_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Pragma_Node_Array_Access) return Bare_Pragma_Node_Array_Access is
      Ret : Bare_Pragma_Node_Array_Access := Create_Bare_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Pragma_Node_Array (Items_Count : Natural) return Bare_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Pragma_Node_Array_Type
       else new Bare_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Pragma_Node_Array
     (Items : Internal_Bare_Pragma_Node_Array) return Bare_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Pragma_Node_Array_Type;
      end if;
      return new Bare_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Pragma_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Single_Tok_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Single_Tok_Node
   is
      function Absolute_Get
        (T : Bare_Single_Tok_Node_Array_Access; Index : Integer)
         return Bare_Single_Tok_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Single_Tok_Node,
         Sequence_Type => Bare_Single_Tok_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Single_Tok_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Single_Tok_Node_Array_Access) return Bare_Single_Tok_Node_Array_Access is
      Ret : Bare_Single_Tok_Node_Array_Access := Create_Bare_Single_Tok_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Single_Tok_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Single_Tok_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Single_Tok_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Single_Tok_Node_Array (Items_Count : Natural) return Bare_Single_Tok_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Single_Tok_Node_Array_Type
       else new Bare_Single_Tok_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Single_Tok_Node_Array
     (Items : Internal_Bare_Single_Tok_Node_Array) return Bare_Single_Tok_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Single_Tok_Node_Array_Type;
      end if;
      return new Bare_Single_Tok_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Single_Tok_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Single_Tok_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Bare_Use_Clause_Vectors is new Langkit_Support.Vectors (Bare_Use_Clause);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Bare_Use_Clause_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Use_Clause
   is
      function Absolute_Get
        (T : Bare_Use_Clause_Array_Access; Index : Integer)
         return Bare_Use_Clause
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Use_Clause,
         Sequence_Type => Bare_Use_Clause_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Use_Clause;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Use_Clause_Array_Access) return Bare_Use_Clause_Array_Access is
      Ret : Bare_Use_Clause_Array_Access := Create_Bare_Use_Clause_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Use_Clause_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Use_Clause_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Use_Clause_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Use_Clause_Array (Items_Count : Natural) return Bare_Use_Clause_Array_Access
   is (if Items_Count = 0
       then No_Bare_Use_Clause_Array_Type
       else new Bare_Use_Clause_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Use_Clause_Array
     (Items : Internal_Bare_Use_Clause_Array) return Bare_Use_Clause_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Use_Clause_Array_Type;
      end if;
      return new Bare_Use_Clause_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Use_Clause_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Use_Clause_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Aspect_Vectors is new Langkit_Support.Vectors (Internal_Aspect);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Aspect_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Aspect
   is
      function Absolute_Get
        (T : Internal_Aspect_Array_Access; Index : Integer)
         return Internal_Aspect
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Aspect,
         Sequence_Type => Internal_Aspect_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Aspect;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Aspect;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Aspect_Array_Access) return Internal_Aspect_Array_Access is
      Ret : Internal_Aspect_Array_Access := Create_Internal_Aspect_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Aspect_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Aspect_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Aspect_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Aspect_Array (Items_Count : Natural) return Internal_Aspect_Array_Access
   is (if Items_Count = 0
       then No_Internal_Aspect_Array_Type
       else new Internal_Aspect_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Aspect_Array
     (Items : Internal_Internal_Aspect_Array) return Internal_Aspect_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Aspect_Array_Type;
      end if;
      return new Internal_Aspect_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Aspect_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Aspect_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Aspect_Array_Access) return Internal_Aspect_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Aspect,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Aspect renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Aspect_Array_Access :=
               Create_Internal_Aspect_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Aspect renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Completion_Item_Vectors is new Langkit_Support.Vectors (Internal_Completion_Item);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Completion_Item_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Completion_Item
   is
      function Absolute_Get
        (T : Internal_Completion_Item_Array_Access; Index : Integer)
         return Internal_Completion_Item
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Completion_Item,
         Sequence_Type => Internal_Completion_Item_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Completion_Item;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Completion_Item;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Completion_Item_Array_Access) return Internal_Completion_Item_Array_Access is
      Ret : Internal_Completion_Item_Array_Access := Create_Internal_Completion_Item_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Completion_Item_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Completion_Item_Array (Items_Count : Natural) return Internal_Completion_Item_Array_Access
   is (if Items_Count = 0
       then No_Internal_Completion_Item_Array_Type
       else new Internal_Completion_Item_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Completion_Item_Array
     (Items : Internal_Internal_Completion_Item_Array) return Internal_Completion_Item_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Completion_Item_Array_Type;
      end if;
      return new Internal_Completion_Item_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Completion_Item_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- To_Iterator --
      -----------------
      function To_Iterator
        (Self    : Internal_Completion_Item_Array_Access;
         Context : Internal_Context) return Internal_Completion_Item_Iterator_Access
      is
      begin
         Inc_Ref (Self);
         return new Internal_Internal_Completion_Item_Iterator'
           (Ref_Count  => 1,
            Safety_Net => Create_Safety_Net (Context),
            Elements   => Self,
            Index      => 1);
      end To_Iterator;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Discriminant_Values_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Discriminant_Values
   is
      function Absolute_Get
        (T : Internal_Discriminant_Values_Array_Access; Index : Integer)
         return Internal_Discriminant_Values
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Discriminant_Values,
         Sequence_Type => Internal_Discriminant_Values_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Discriminant_Values;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Discriminant_Values;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Discriminant_Values_Array_Access) return Internal_Discriminant_Values_Array_Access is
      Ret : Internal_Discriminant_Values_Array_Access := Create_Internal_Discriminant_Values_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Discriminant_Values_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Discriminant_Values_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Discriminant_Values_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Discriminant_Values_Array (Items_Count : Natural) return Internal_Discriminant_Values_Array_Access
   is (if Items_Count = 0
       then No_Internal_Discriminant_Values_Array_Type
       else new Internal_Discriminant_Values_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Discriminant_Values_Array
     (Items : Internal_Internal_Discriminant_Values_Array) return Internal_Discriminant_Values_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Discriminant_Values_Array_Type;
      end if;
      return new Internal_Discriminant_Values_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Discriminant_Values_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Discriminant_Values_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Doc_Annotation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Doc_Annotation
   is
      function Absolute_Get
        (T : Internal_Doc_Annotation_Array_Access; Index : Integer)
         return Internal_Doc_Annotation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Doc_Annotation,
         Sequence_Type => Internal_Doc_Annotation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Doc_Annotation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Doc_Annotation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Doc_Annotation_Array_Access) return Internal_Doc_Annotation_Array_Access is
      Ret : Internal_Doc_Annotation_Array_Access := Create_Internal_Doc_Annotation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Doc_Annotation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Doc_Annotation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Doc_Annotation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Doc_Annotation_Array (Items_Count : Natural) return Internal_Doc_Annotation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Doc_Annotation_Array_Type
       else new Internal_Doc_Annotation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Doc_Annotation_Array
     (Items : Internal_Internal_Doc_Annotation_Array) return Internal_Doc_Annotation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Doc_Annotation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Doc_Annotation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Doc_Annotation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Doc_Annotation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Accept_Stmt_Vectors is new Langkit_Support.Vectors (Internal_Entity_Accept_Stmt);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Accept_Stmt_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Accept_Stmt
   is
      function Absolute_Get
        (T : Internal_Entity_Accept_Stmt_Array_Access; Index : Integer)
         return Internal_Entity_Accept_Stmt
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Accept_Stmt,
         Sequence_Type => Internal_Entity_Accept_Stmt_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Accept_Stmt;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Accept_Stmt;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Accept_Stmt_Array_Access) return Internal_Entity_Accept_Stmt_Array_Access is
      Ret : Internal_Entity_Accept_Stmt_Array_Access := Create_Internal_Entity_Accept_Stmt_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Accept_Stmt_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Accept_Stmt_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Accept_Stmt_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Accept_Stmt_Array (Items_Count : Natural) return Internal_Entity_Accept_Stmt_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Accept_Stmt_Array_Type
       else new Internal_Entity_Accept_Stmt_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Accept_Stmt_Array
     (Items : Internal_Internal_Entity_Accept_Stmt_Array) return Internal_Entity_Accept_Stmt_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Accept_Stmt_Array_Type;
      end if;
      return new Internal_Entity_Accept_Stmt_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Accept_Stmt_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Accept_Stmt_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Ada_Node_List_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Ada_Node_List
   is
      function Absolute_Get
        (T : Internal_Entity_Ada_Node_List_Array_Access; Index : Integer)
         return Internal_Entity_Ada_Node_List
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Ada_Node_List,
         Sequence_Type => Internal_Entity_Ada_Node_List_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Ada_Node_List;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Ada_Node_List;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Ada_Node_List_Array_Access) return Internal_Entity_Ada_Node_List_Array_Access is
      Ret : Internal_Entity_Ada_Node_List_Array_Access := Create_Internal_Entity_Ada_Node_List_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Ada_Node_List_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Ada_Node_List_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Ada_Node_List_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Ada_Node_List_Array (Items_Count : Natural) return Internal_Entity_Ada_Node_List_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Ada_Node_List_Array_Type
       else new Internal_Entity_Ada_Node_List_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Ada_Node_List_Array
     (Items : Internal_Internal_Entity_Ada_Node_List_Array) return Internal_Entity_Ada_Node_List_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Ada_Node_List_Array_Type;
      end if;
      return new Internal_Entity_Ada_Node_List_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Ada_Node_List_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Ada_Node_List_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Vectors is new Langkit_Support.Vectors (Internal_Entity);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity
   is
      function Absolute_Get
        (T : Internal_Entity_Array_Access; Index : Integer)
         return Internal_Entity
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity,
         Sequence_Type => Internal_Entity_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Array_Access) return Internal_Entity_Array_Access is
      Ret : Internal_Entity_Array_Access := Create_Internal_Entity_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Array (Items_Count : Natural) return Internal_Entity_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Array
     (Items : AST_Envs.Entity_Array) return Internal_Entity_Array_Access
   is (if Items'Length = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'
         (N         => Items'Length,
          Items     => Implementation.Internal_Internal_Entity_Array (Items),
          Ref_Count => 1));
   function Create_Internal_Entity_Array
     (Items : Internal_Internal_Entity_Array) return Internal_Entity_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Array_Type;
      end if;
      return new Internal_Entity_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Base_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Assoc,
         Sequence_Type => Internal_Entity_Base_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Assoc_Array_Access) return Internal_Entity_Base_Assoc_Array_Access is
      Ret : Internal_Entity_Base_Assoc_Array_Access := Create_Internal_Entity_Base_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Assoc_Array (Items_Count : Natural) return Internal_Entity_Base_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Assoc_Array_Type
       else new Internal_Entity_Base_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Assoc_Array
     (Items : Internal_Internal_Entity_Base_Assoc_Array) return Internal_Entity_Base_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Base_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Formal_Param_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Formal_Param_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Formal_Param_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Formal_Param_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Formal_Param_Decl,
         Sequence_Type => Internal_Entity_Base_Formal_Param_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Formal_Param_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Formal_Param_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
      Ret : Internal_Entity_Base_Formal_Param_Decl_Array_Access := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Formal_Param_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Formal_Param_Decl_Array_Type
       else new Internal_Entity_Base_Formal_Param_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array
     (Items : Internal_Internal_Entity_Base_Formal_Param_Decl_Array) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Formal_Param_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Base_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Type_Decl,
         Sequence_Type => Internal_Entity_Base_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
      Ret : Internal_Entity_Base_Type_Decl_Array_Access := Create_Internal_Entity_Base_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Type_Decl_Array_Type
       else new Internal_Entity_Base_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Type_Decl_Array
     (Items : Internal_Internal_Entity_Base_Type_Decl_Array) return Internal_Entity_Base_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Base_Type_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Base_Type_Decl_Array_Access :=
               Create_Internal_Entity_Base_Type_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      ----------
      -- Hash --
      ----------
      function Hash (R : Internal_Entity_Base_Type_Decl_Array_Access) return Hash_Type is
         Result : Hash_Type := Initial_Hash;
      begin
         for I in R.Items'Range loop
            Result := Combine (Result, Hash (R.Items (I)));
         end loop;
         return Result;
      end Hash;
      package Internal_Entity_Basic_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Basic_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Decl,
         Sequence_Type => Internal_Entity_Basic_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
      Ret : Internal_Entity_Basic_Decl_Array_Access := Create_Internal_Entity_Basic_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Decl_Array (Items_Count : Natural) return Internal_Entity_Basic_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Decl_Array_Type
       else new Internal_Entity_Basic_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Decl_Array
     (Items : Internal_Internal_Entity_Basic_Decl_Array) return Internal_Entity_Basic_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Decl_Array_Type;
      end if;
      return new Internal_Entity_Basic_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Basic_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Basic_Decl_Array_Access :=
               Create_Internal_Entity_Basic_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Internal_Entity_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Compilation_Unit
   is
      function Absolute_Get
        (T : Internal_Entity_Compilation_Unit_Array_Access; Index : Integer)
         return Internal_Entity_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Compilation_Unit,
         Sequence_Type => Internal_Entity_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Compilation_Unit;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
      Ret : Internal_Entity_Compilation_Unit_Array_Access := Create_Internal_Entity_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Compilation_Unit_Array (Items_Count : Natural) return Internal_Entity_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Compilation_Unit_Array_Type
       else new Internal_Entity_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Compilation_Unit_Array
     (Items : Internal_Internal_Entity_Compilation_Unit_Array) return Internal_Entity_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Compilation_Unit_Array_Type;
      end if;
      return new Internal_Entity_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Compilation_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Compilation_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Compilation_Unit_Array_Access :=
               Create_Internal_Entity_Compilation_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Declarative_Part_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Declarative_Part
   is
      function Absolute_Get
        (T : Internal_Entity_Declarative_Part_Array_Access; Index : Integer)
         return Internal_Entity_Declarative_Part
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Declarative_Part,
         Sequence_Type => Internal_Entity_Declarative_Part_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Declarative_Part;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Declarative_Part;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Declarative_Part_Array_Access) return Internal_Entity_Declarative_Part_Array_Access is
      Ret : Internal_Entity_Declarative_Part_Array_Access := Create_Internal_Entity_Declarative_Part_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Declarative_Part_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Declarative_Part_Array (Items_Count : Natural) return Internal_Entity_Declarative_Part_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Declarative_Part_Array_Type
       else new Internal_Entity_Declarative_Part_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Declarative_Part_Array
     (Items : Internal_Internal_Entity_Declarative_Part_Array) return Internal_Entity_Declarative_Part_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Declarative_Part_Array_Type;
      end if;
      return new Internal_Entity_Declarative_Part_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Declarative_Part_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Declarative_Part_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Defining_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Defining_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Defining_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Defining_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Defining_Name_Array_Access; Index : Integer)
         return Internal_Entity_Defining_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Defining_Name,
         Sequence_Type => Internal_Entity_Defining_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Defining_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Defining_Name;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Defining_Name_Array_Access) return Internal_Entity_Defining_Name_Array_Access is
      Ret : Internal_Entity_Defining_Name_Array_Access := Create_Internal_Entity_Defining_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Defining_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Defining_Name_Array (Items_Count : Natural) return Internal_Entity_Defining_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Defining_Name_Array_Type
       else new Internal_Entity_Defining_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Defining_Name_Array
     (Items : Internal_Internal_Entity_Defining_Name_Array) return Internal_Entity_Defining_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Defining_Name_Array_Type;
      end if;
      return new Internal_Entity_Defining_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Defining_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Defining_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Expr,
         Sequence_Type => Internal_Entity_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Expr_Array_Access) return Internal_Entity_Expr_Array_Access is
      Ret : Internal_Entity_Expr_Array_Access := Create_Internal_Entity_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Expr_Array (Items_Count : Natural) return Internal_Entity_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Expr_Array_Type
       else new Internal_Entity_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Expr_Array
     (Items : Internal_Internal_Entity_Expr_Array) return Internal_Entity_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Expr_Array_Type;
      end if;
      return new Internal_Entity_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Generic_Instantiation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Generic_Instantiation
   is
      function Absolute_Get
        (T : Internal_Entity_Generic_Instantiation_Array_Access; Index : Integer)
         return Internal_Entity_Generic_Instantiation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Generic_Instantiation,
         Sequence_Type => Internal_Entity_Generic_Instantiation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Generic_Instantiation;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Generic_Instantiation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Internal_Entity_Generic_Instantiation_Array_Access is
      Ret : Internal_Entity_Generic_Instantiation_Array_Access := Create_Internal_Entity_Generic_Instantiation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Generic_Instantiation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Generic_Instantiation_Array (Items_Count : Natural) return Internal_Entity_Generic_Instantiation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Generic_Instantiation_Array_Type
       else new Internal_Entity_Generic_Instantiation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Generic_Instantiation_Array
     (Items : Internal_Internal_Entity_Generic_Instantiation_Array) return Internal_Entity_Generic_Instantiation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Generic_Instantiation_Array_Type;
      end if;
      return new Internal_Entity_Generic_Instantiation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Generic_Instantiation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Identifier_Vectors is new Langkit_Support.Vectors (Internal_Entity_Identifier);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Identifier_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Identifier
   is
      function Absolute_Get
        (T : Internal_Entity_Identifier_Array_Access; Index : Integer)
         return Internal_Entity_Identifier
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Identifier,
         Sequence_Type => Internal_Entity_Identifier_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Identifier;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Identifier;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Identifier_Array_Access) return Internal_Entity_Identifier_Array_Access is
      Ret : Internal_Entity_Identifier_Array_Access := Create_Internal_Entity_Identifier_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Identifier_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Identifier_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Identifier_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Identifier_Array (Items_Count : Natural) return Internal_Entity_Identifier_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Identifier_Array_Type
       else new Internal_Entity_Identifier_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Identifier_Array
     (Items : Internal_Internal_Entity_Identifier_Array) return Internal_Entity_Identifier_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Identifier_Array_Type;
      end if;
      return new Internal_Entity_Identifier_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Identifier_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Identifier_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Mode_Vectors is new Langkit_Support.Vectors (Internal_Entity_Mode);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Mode_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Mode
   is
      function Absolute_Get
        (T : Internal_Entity_Mode_Array_Access; Index : Integer)
         return Internal_Entity_Mode
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Mode,
         Sequence_Type => Internal_Entity_Mode_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Mode;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Mode;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Mode_Array_Access) return Internal_Entity_Mode_Array_Access is
      Ret : Internal_Entity_Mode_Array_Access := Create_Internal_Entity_Mode_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Mode_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Mode_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Mode_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Mode_Array (Items_Count : Natural) return Internal_Entity_Mode_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Mode_Array_Type
       else new Internal_Entity_Mode_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Mode_Array
     (Items : Internal_Internal_Entity_Mode_Array) return Internal_Entity_Mode_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Mode_Array_Type;
      end if;
      return new Internal_Entity_Mode_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Mode_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Mode_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Name);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Name_Array_Access; Index : Integer)
         return Internal_Entity_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Name,
         Sequence_Type => Internal_Entity_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Name;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Name_Array_Access) return Internal_Entity_Name_Array_Access is
      Ret : Internal_Entity_Name_Array_Access := Create_Internal_Entity_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Name_Array (Items_Count : Natural) return Internal_Entity_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Name_Array_Type
       else new Internal_Entity_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Name_Array
     (Items : Internal_Internal_Entity_Name_Array) return Internal_Entity_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Name_Array_Type;
      end if;
      return new Internal_Entity_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Name_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Param_Spec_Vectors is new Langkit_Support.Vectors (Internal_Entity_Param_Spec);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Param_Spec_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Param_Spec
   is
      function Absolute_Get
        (T : Internal_Entity_Param_Spec_Array_Access; Index : Integer)
         return Internal_Entity_Param_Spec
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Param_Spec,
         Sequence_Type => Internal_Entity_Param_Spec_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Param_Spec;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Param_Spec;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Param_Spec_Array_Access) return Internal_Entity_Param_Spec_Array_Access is
      Ret : Internal_Entity_Param_Spec_Array_Access := Create_Internal_Entity_Param_Spec_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Param_Spec_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Param_Spec_Array (Items_Count : Natural) return Internal_Entity_Param_Spec_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Param_Spec_Array_Type
       else new Internal_Entity_Param_Spec_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Param_Spec_Array
     (Items : Internal_Internal_Entity_Param_Spec_Array) return Internal_Entity_Param_Spec_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Param_Spec_Array_Type;
      end if;
      return new Internal_Entity_Param_Spec_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Param_Spec_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Param_Spec_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Pragma_Node_Vectors is new Langkit_Support.Vectors (Internal_Entity_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Pragma_Node
   is
      function Absolute_Get
        (T : Internal_Entity_Pragma_Node_Array_Access; Index : Integer)
         return Internal_Entity_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Pragma_Node,
         Sequence_Type => Internal_Entity_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Pragma_Node;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Pragma_Node_Array_Access) return Internal_Entity_Pragma_Node_Array_Access is
      Ret : Internal_Entity_Pragma_Node_Array_Access := Create_Internal_Entity_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Pragma_Node_Array (Items_Count : Natural) return Internal_Entity_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Pragma_Node_Array_Type
       else new Internal_Entity_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Pragma_Node_Array
     (Items : Internal_Internal_Entity_Pragma_Node_Array) return Internal_Entity_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Pragma_Node_Array_Type;
      end if;
      return new Internal_Entity_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Pragma_Node_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Decl,
         Sequence_Type => Internal_Entity_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Decl;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Decl_Array_Access) return Internal_Entity_Type_Decl_Array_Access is
      Ret : Internal_Entity_Type_Decl_Array_Access := Create_Internal_Entity_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Decl_Array_Type
       else new Internal_Entity_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Decl_Array
     (Items : Internal_Internal_Entity_Type_Decl_Array) return Internal_Entity_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Decl_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Entity_Type_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Type_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Expr,
         Sequence_Type => Internal_Entity_Type_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Expr_Array_Access) return Internal_Entity_Type_Expr_Array_Access is
      Ret : Internal_Entity_Type_Expr_Array_Access := Create_Internal_Entity_Type_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Expr_Array (Items_Count : Natural) return Internal_Entity_Type_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Expr_Array_Type
       else new Internal_Entity_Type_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Expr_Array
     (Items : Internal_Internal_Entity_Type_Expr_Array) return Internal_Entity_Type_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Expr_Array_Type;
      end if;
      return new Internal_Entity_Type_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Assoc,
         Sequence_Type => Internal_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Env_Assoc_Array_Access) return Internal_Env_Assoc_Array_Access is
      Ret : Internal_Env_Assoc_Array_Access := Create_Internal_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Env_Assoc_Array (Items_Count : Natural) return Internal_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Assoc_Array_Type
       else new Internal_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Env_Assoc_Array
     (Items : Internal_Internal_Env_Assoc_Array) return Internal_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Assoc_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Expected_Type_For_Expr_Vectors is new Langkit_Support.Vectors (Internal_Expected_Type_For_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Expected_Type_For_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Expected_Type_For_Expr
   is
      function Absolute_Get
        (T : Internal_Expected_Type_For_Expr_Array_Access; Index : Integer)
         return Internal_Expected_Type_For_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Expected_Type_For_Expr,
         Sequence_Type => Internal_Expected_Type_For_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Expected_Type_For_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Expected_Type_For_Expr;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Internal_Expected_Type_For_Expr_Array_Access is
      Ret : Internal_Expected_Type_For_Expr_Array_Access := Create_Internal_Expected_Type_For_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Expected_Type_For_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Expected_Type_For_Expr_Array (Items_Count : Natural) return Internal_Expected_Type_For_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Expected_Type_For_Expr_Array_Type
       else new Internal_Expected_Type_For_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Expected_Type_For_Expr_Array
     (Items : Internal_Internal_Expected_Type_For_Expr_Array) return Internal_Expected_Type_For_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Expected_Type_For_Expr_Array_Type;
      end if;
      return new Internal_Expected_Type_For_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Expected_Type_For_Expr_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Inner_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Inner_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Inner_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Inner_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Inner_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Inner_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Inner_Env_Assoc,
         Sequence_Type => Internal_Inner_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Inner_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Inner_Env_Assoc;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Inner_Env_Assoc_Array_Access) return Internal_Inner_Env_Assoc_Array_Access is
      Ret : Internal_Inner_Env_Assoc_Array_Access := Create_Internal_Inner_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Inner_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Inner_Env_Assoc_Array (Items_Count : Natural) return Internal_Inner_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Inner_Env_Assoc_Array_Type
       else new Internal_Inner_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Inner_Env_Assoc_Array
     (Items : Internal_Internal_Inner_Env_Assoc_Array) return Internal_Inner_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Inner_Env_Assoc_Array_Type;
      end if;
      return new Internal_Inner_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Inner_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Logic_Context_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Logic_Context
   is
      function Absolute_Get
        (T : Internal_Logic_Context_Array_Access; Index : Integer)
         return Internal_Logic_Context
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Logic_Context,
         Sequence_Type => Internal_Logic_Context_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Logic_Context;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Logic_Context;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Logic_Context_Array_Access) return Internal_Logic_Context_Array_Access is
      Ret : Internal_Logic_Context_Array_Access := Create_Internal_Logic_Context_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Logic_Context_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Logic_Context_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Logic_Context_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Logic_Context_Array (Items_Count : Natural) return Internal_Logic_Context_Array_Access
   is (if Items_Count = 0
       then No_Internal_Logic_Context_Array_Type
       else new Internal_Logic_Context_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Logic_Context_Array
     (Items : Internal_Internal_Logic_Context_Array) return Internal_Logic_Context_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Logic_Context_Array_Type;
      end if;
      return new Internal_Logic_Context_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Logic_Context_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Logic_Context_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Param_Actual_Vectors is new Langkit_Support.Vectors (Internal_Param_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Param_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Actual
   is
      function Absolute_Get
        (T : Internal_Param_Actual_Array_Access; Index : Integer)
         return Internal_Param_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Actual,
         Sequence_Type => Internal_Param_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Actual;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Actual_Array_Access) return Internal_Param_Actual_Array_Access is
      Ret : Internal_Param_Actual_Array_Access := Create_Internal_Param_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Actual_Array (Items_Count : Natural) return Internal_Param_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Actual_Array_Type
       else new Internal_Param_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Actual_Array
     (Items : Internal_Internal_Param_Actual_Array) return Internal_Param_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Actual_Array_Type;
      end if;
      return new Internal_Param_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Actual_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Param_Match_Vectors is new Langkit_Support.Vectors (Internal_Param_Match);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Param_Match_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Match
   is
      function Absolute_Get
        (T : Internal_Param_Match_Array_Access; Index : Integer)
         return Internal_Param_Match
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Match,
         Sequence_Type => Internal_Param_Match_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Match;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Match;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Match_Array_Access) return Internal_Param_Match_Array_Access is
      Ret : Internal_Param_Match_Array_Access := Create_Internal_Param_Match_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Match_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Match_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Match_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Match_Array (Items_Count : Natural) return Internal_Param_Match_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Match_Array_Type
       else new Internal_Param_Match_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Match_Array
     (Items : Internal_Internal_Param_Match_Array) return Internal_Param_Match_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Match_Array_Type;
      end if;
      return new Internal_Param_Match_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Match_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Match_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Ref_Result_Vectors is new Langkit_Support.Vectors (Internal_Ref_Result);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Ref_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Ref_Result
   is
      function Absolute_Get
        (T : Internal_Ref_Result_Array_Access; Index : Integer)
         return Internal_Ref_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Ref_Result,
         Sequence_Type => Internal_Ref_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Ref_Result;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Ref_Result;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Ref_Result_Array_Access) return Internal_Ref_Result_Array_Access is
      Ret : Internal_Ref_Result_Array_Access := Create_Internal_Ref_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Ref_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Ref_Result_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Ref_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Ref_Result_Array (Items_Count : Natural) return Internal_Ref_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Ref_Result_Array_Type
       else new Internal_Ref_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Ref_Result_Array
     (Items : Internal_Internal_Ref_Result_Array) return Internal_Ref_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Ref_Result_Array_Type;
      end if;
      return new Internal_Ref_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Ref_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Ref_Result_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Shape_Vectors is new Langkit_Support.Vectors (Internal_Shape);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Shape_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Shape
   is
      function Absolute_Get
        (T : Internal_Shape_Array_Access; Index : Integer)
         return Internal_Shape
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Shape,
         Sequence_Type => Internal_Shape_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Shape;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Shape;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Shape_Array_Access) return Internal_Shape_Array_Access is
      Ret : Internal_Shape_Array_Access := Create_Internal_Shape_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Shape_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Shape_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Shape_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Shape_Array (Items_Count : Natural) return Internal_Shape_Array_Access
   is (if Items_Count = 0
       then No_Internal_Shape_Array_Type
       else new Internal_Shape_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Shape_Array
     (Items : Internal_Internal_Shape_Array) return Internal_Shape_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Shape_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Shape_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Shape_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Shape_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Single_Actual_Vectors is new Langkit_Support.Vectors (Internal_Single_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Single_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Actual
   is
      function Absolute_Get
        (T : Internal_Single_Actual_Array_Access; Index : Integer)
         return Internal_Single_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Actual,
         Sequence_Type => Internal_Single_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Actual;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Actual_Array_Access) return Internal_Single_Actual_Array_Access is
      Ret : Internal_Single_Actual_Array_Access := Create_Internal_Single_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Actual_Array (Items_Count : Natural) return Internal_Single_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Actual_Array_Type
       else new Internal_Single_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Actual_Array
     (Items : Internal_Internal_Single_Actual_Array) return Internal_Single_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Actual_Array_Type;
      end if;
      return new Internal_Single_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Actual_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Solver_Diagnostic_Vectors is new Langkit_Support.Vectors (Internal_Solver_Diagnostic);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Solver_Diagnostic_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Solver_Diagnostic
   is
      function Absolute_Get
        (T : Internal_Solver_Diagnostic_Array_Access; Index : Integer)
         return Internal_Solver_Diagnostic
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Solver_Diagnostic,
         Sequence_Type => Internal_Solver_Diagnostic_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Solver_Diagnostic;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Solver_Diagnostic;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Solver_Diagnostic_Array_Access) return Internal_Solver_Diagnostic_Array_Access is
      Ret : Internal_Solver_Diagnostic_Array_Access := Create_Internal_Solver_Diagnostic_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Solver_Diagnostic_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Solver_Diagnostic_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Solver_Diagnostic_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Solver_Diagnostic_Array (Items_Count : Natural) return Internal_Solver_Diagnostic_Array_Access
   is (if Items_Count = 0
       then No_Internal_Solver_Diagnostic_Array_Type
       else new Internal_Solver_Diagnostic_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Solver_Diagnostic_Array
     (Items : Internal_Internal_Solver_Diagnostic_Array) return Internal_Solver_Diagnostic_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Solver_Diagnostic_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Solver_Diagnostic_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Solver_Diagnostic_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Solver_Diagnostic_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Substitution_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Substitution
   is
      function Absolute_Get
        (T : Internal_Substitution_Array_Access; Index : Integer)
         return Internal_Substitution
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Substitution,
         Sequence_Type => Internal_Substitution_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Substitution;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Substitution;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Substitution_Array_Access) return Internal_Substitution_Array_Access is
      Ret : Internal_Substitution_Array_Access := Create_Internal_Substitution_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Substitution_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Substitution_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Substitution_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Substitution_Array (Items_Count : Natural) return Internal_Substitution_Array_Access
   is (if Items_Count = 0
       then No_Internal_Substitution_Array_Type
       else new Internal_Substitution_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Substitution_Array
     (Items : Internal_Internal_Substitution_Array) return Internal_Substitution_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Substitution_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Substitution_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Substitution_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Substitution_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Internal_Unit_Vectors is new Langkit_Support.Vectors (Internal_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Unit
   is
      function Absolute_Get
        (T : Internal_Unit_Array_Access; Index : Integer)
         return Internal_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Unit,
         Sequence_Type => Internal_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
      Ret : Internal_Unit_Array_Access := Create_Internal_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Unit_Array (Items_Count : Natural) return Internal_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Unit_Array_Type
       else new Internal_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Unit_Array
     (Items : Internal_Internal_Unit_Array) return Internal_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Unit_Array_Type;
      end if;
      return new Internal_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Unit_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Unit_Array_Access :=
               Create_Internal_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Internal_User_Defined_Function_Subp_Spec_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_User_Defined_Function_Subp_Spec
   is
      function Absolute_Get
        (T : Internal_User_Defined_Function_Subp_Spec_Array_Access; Index : Integer)
         return Internal_User_Defined_Function_Subp_Spec
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_User_Defined_Function_Subp_Spec,
         Sequence_Type => Internal_User_Defined_Function_Subp_Spec_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_User_Defined_Function_Subp_Spec;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_User_Defined_Function_Subp_Spec;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_User_Defined_Function_Subp_Spec_Array_Access) return Internal_User_Defined_Function_Subp_Spec_Array_Access is
      Ret : Internal_User_Defined_Function_Subp_Spec_Array_Access := Create_Internal_User_Defined_Function_Subp_Spec_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_User_Defined_Function_Subp_Spec_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_User_Defined_Function_Subp_Spec_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_User_Defined_Function_Subp_Spec_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_User_Defined_Function_Subp_Spec_Array (Items_Count : Natural) return Internal_User_Defined_Function_Subp_Spec_Array_Access
   is (if Items_Count = 0
       then No_Internal_User_Defined_Function_Subp_Spec_Array_Type
       else new Internal_User_Defined_Function_Subp_Spec_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_User_Defined_Function_Subp_Spec_Array
     (Items : Internal_Internal_User_Defined_Function_Subp_Spec_Array) return Internal_User_Defined_Function_Subp_Spec_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_User_Defined_Function_Subp_Spec_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_User_Defined_Function_Subp_Spec_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_User_Defined_Function_Subp_Spec_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_User_Defined_Function_Subp_Spec_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Lexical_Env_Vectors is new Langkit_Support.Vectors (Lexical_Env);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Lexical_Env_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Lexical_Env
   is
      function Absolute_Get
        (T : Lexical_Env_Array_Access; Index : Integer)
         return Lexical_Env
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Lexical_Env,
         Sequence_Type => Lexical_Env_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Lexical_Env;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_Env;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Lexical_Env_Array_Access) return Lexical_Env_Array_Access is
      Ret : Lexical_Env_Array_Access := Create_Lexical_Env_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Lexical_Env_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Lexical_Env_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Lexical_Env_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Lexical_Env_Array (Items_Count : Natural) return Lexical_Env_Array_Access
   is (if Items_Count = 0
       then No_Lexical_Env_Array_Type
       else new Lexical_Env_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Lexical_Env_Array
     (Items : Internal_Lexical_Env_Array) return Lexical_Env_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Lexical_Env_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Lexical_Env_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Lexical_Env_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Lexical_Env_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Logic_Equation_Vectors is new Langkit_Support.Vectors (Logic_Equation);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Logic_Equation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Logic_Equation
   is
      function Absolute_Get
        (T : Logic_Equation_Array_Access; Index : Integer)
         return Logic_Equation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Logic_Equation,
         Sequence_Type => Logic_Equation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Logic_Equation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Null_Logic_Equation;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Logic_Equation_Array_Access) return Logic_Equation_Array_Access is
      Ret : Logic_Equation_Array_Access := Create_Logic_Equation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Logic_Equation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Logic_Equation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Logic_Equation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Logic_Equation_Array (Items_Count : Natural) return Logic_Equation_Array_Access
   is (if Items_Count = 0
       then No_Logic_Equation_Array_Type
       else new Logic_Equation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Logic_Equation_Array
     (Items : Internal_Logic_Equation_Array) return Logic_Equation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Logic_Equation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Logic_Equation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Logic_Equation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Logic_Equation_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package String_Type_Vectors is new Langkit_Support.Vectors (String_Type);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : String_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return String_Type
   is
      function Absolute_Get
        (T : String_Type_Array_Access; Index : Integer)
         return String_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => String_Type,
         Sequence_Type => String_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : String_Type;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_String;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : String_Type_Array_Access) return String_Type_Array_Access is
      Ret : String_Type_Array_Access := Create_String_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
      ------------------
      -- Join_Strings --
      ------------------
      function Join_Strings
        (Separator : String_Type;
         Strings   : String_Type_Array_Access) return String_Type
      is
         Separator_Length : constant Natural := Separator.Length;
         Length           : Natural := 0;
         First            : Boolean;
      begin
         --  First, compute the length of the result: the sum of all string
         --  lengths in Strings.
         First := True;
         for S of Strings.Items loop
            if First then
               First := False;
            else
               Length := Length + Separator_Length;
            end if;
            Length := Length + S.Length;
         end loop;
         --  Create the result string with the correct length. Do not use our
         --  constructor and initialize the content in-place, to avoid extra
         --  copies.
         return Result : constant String_Type :=
            new String_Record (Length)
         do
            Result.Ref_Count := 1;
            --  Now copy the content of all strings into the result
            declare
               Last : Natural := 0;
            begin
               First := True;
               for S of Strings.Items loop
                  if First then
                     First := False;
                  else
                     Result.Content (Last + 1 .. Last + Separator_Length) :=
                        Separator.Content;
                     Last := Last + Separator_Length;
                  end if;
                  Result.Content (Last + 1 .. Last + S.Length) := S.Content;
                  Last := Last + S.Length;
               end loop;
            end;
         end return;
      end Join_Strings;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : String_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : String_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out String_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_String_Type_Array (Items_Count : Natural) return String_Type_Array_Access
   is (if Items_Count = 0
       then No_String_Type_Array_Type
       else new String_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_String_Type_Array
     (Items : Internal_String_Type_Array) return String_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_String_Type_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new String_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : String_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : String_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      package Symbol_Type_Vectors is new Langkit_Support.Vectors (Symbol_Type);
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_Node;
      T       : Symbol_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access; Index : Integer)
         return Symbol_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type,
         Sequence_Type => Symbol_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Symbol_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Symbol;
      else
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "out-of-bounds array access");
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
      Ret : Symbol_Type_Array_Access := Create_Symbol_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Symbol_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Symbol_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Symbol_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Symbol_Type_Array (Items_Count : Natural) return Symbol_Type_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Type
       else new Symbol_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Symbol_Type_Array
     (Items : Internal_Symbol_Type_Array) return Symbol_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Type;
      end if;
      return new Symbol_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Symbol_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Symbol_Type_Array_Access) return String is
         Result : Unbounded_String;
      begin
         Append (Result, "[");
         for I in A.Items'Range loop
            if I > A.Items'First then
               Append (Result, ", ");
            end if;
            Append (Result, Trace_Image (A.Items (I)));
         end loop;
         Append (Result, "]");
         return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Symbol_Type,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Symbol_Type renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Symbol_Type_Array_Access :=
               Create_Symbol_Type_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Symbol_Type renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Bare_Ada_Node_Iterator_Access;
      Element : out Bare_Ada_Node) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Completion_Item_Iterator_Access;
      Element : out Internal_Completion_Item) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Iterator_Access) return String is
      begin
         return "<Iterator of CompletionItem, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Entity_Iterator_Access;
      Element : out Internal_Entity) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode.entity, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Inner_Env_Assoc_Iterator_Access;
      Element : out Internal_Inner_Env_Assoc) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Iterator_Access) return String is
      begin
         return "<Iterator of InnerEnvAssoc, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Internal_Unit) return Boolean is
   begin
      return Left.Filename < Right.Filename;
   end "<";
function Hash (Key : Mmz_Key_Item) return Hash_Type;
function Equivalent (L, R : Mmz_Key_Item) return Boolean;
procedure Destroy (Key : in out Mmz_Key_Array_Access);
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key_Item) return Boolean is
begin
   if L.Kind /= R.Kind then
      return False;
   end if;
   case L.Kind is
         when Mmz_Bare_Access_Def =>
               return L.As_Bare_Access_Def = R.As_Bare_Access_Def;
         when Mmz_Bare_Ada_Node =>
               return L.As_Bare_Ada_Node = R.As_Bare_Ada_Node;
         when Mmz_Bare_Anonymous_Expr_Decl =>
               return L.As_Bare_Anonymous_Expr_Decl = R.As_Bare_Anonymous_Expr_Decl;
         when Mmz_Bare_Array_Subcomponent_Choice_Name =>
               return L.As_Bare_Array_Subcomponent_Choice_Name = R.As_Bare_Array_Subcomponent_Choice_Name;
         when Mmz_Bare_Array_Type_Def =>
               return L.As_Bare_Array_Type_Def = R.As_Bare_Array_Type_Def;
         when Mmz_Bare_Assoc_List =>
               return L.As_Bare_Assoc_List = R.As_Bare_Assoc_List;
         when Mmz_Bare_Base_Aggregate =>
               return L.As_Bare_Base_Aggregate = R.As_Bare_Base_Aggregate;
         when Mmz_Bare_Base_Id =>
               return L.As_Bare_Base_Id = R.As_Bare_Base_Id;
         when Mmz_Bare_Base_Subp_Spec =>
               return L.As_Bare_Base_Subp_Spec = R.As_Bare_Base_Subp_Spec;
         when Mmz_Bare_Base_Type_Decl =>
               return L.As_Bare_Base_Type_Decl = R.As_Bare_Base_Type_Decl;
         when Mmz_Bare_Basic_Decl =>
               return L.As_Bare_Basic_Decl = R.As_Bare_Basic_Decl;
         when Mmz_Bare_Body_Node =>
               return L.As_Bare_Body_Node = R.As_Bare_Body_Node;
         when Mmz_Bare_Compilation_Unit =>
               return L.As_Bare_Compilation_Unit = R.As_Bare_Compilation_Unit;
         when Mmz_Bare_Decimal_Fixed_Point_Def =>
               return L.As_Bare_Decimal_Fixed_Point_Def = R.As_Bare_Decimal_Fixed_Point_Def;
         when Mmz_Bare_Defining_Name =>
               return L.As_Bare_Defining_Name = R.As_Bare_Defining_Name;
         when Mmz_Bare_End_Name =>
               return L.As_Bare_End_Name = R.As_Bare_End_Name;
         when Mmz_Bare_Enum_Literal_Decl =>
               return L.As_Bare_Enum_Literal_Decl = R.As_Bare_Enum_Literal_Decl;
         when Mmz_Bare_Enum_Type_Def =>
               return L.As_Bare_Enum_Type_Def = R.As_Bare_Enum_Type_Def;
         when Mmz_Bare_Expr =>
               return L.As_Bare_Expr = R.As_Bare_Expr;
         when Mmz_Bare_For_Loop_Var_Decl =>
               return L.As_Bare_For_Loop_Var_Decl = R.As_Bare_For_Loop_Var_Decl;
         when Mmz_Bare_Formal_Discrete_Type_Def =>
               return L.As_Bare_Formal_Discrete_Type_Def = R.As_Bare_Formal_Discrete_Type_Def;
         when Mmz_Bare_Formal_Subp_Decl =>
               return L.As_Bare_Formal_Subp_Decl = R.As_Bare_Formal_Subp_Decl;
         when Mmz_Bare_Generic_Instantiation =>
               return L.As_Bare_Generic_Instantiation = R.As_Bare_Generic_Instantiation;
         when Mmz_Bare_Mod_Int_Type_Def =>
               return L.As_Bare_Mod_Int_Type_Def = R.As_Bare_Mod_Int_Type_Def;
         when Mmz_Bare_Name =>
               return L.As_Bare_Name = R.As_Bare_Name;
         when Mmz_Bare_Ordinary_Fixed_Point_Def =>
               return L.As_Bare_Ordinary_Fixed_Point_Def = R.As_Bare_Ordinary_Fixed_Point_Def;
         when Mmz_Bare_Private_Type_Def =>
               return L.As_Bare_Private_Type_Def = R.As_Bare_Private_Type_Def;
         when Mmz_Bare_Real_Type_Def =>
               return L.As_Bare_Real_Type_Def = R.As_Bare_Real_Type_Def;
         when Mmz_Bare_Record_Type_Def =>
               return L.As_Bare_Record_Type_Def = R.As_Bare_Record_Type_Def;
         when Mmz_Bare_Signed_Int_Type_Def =>
               return L.As_Bare_Signed_Int_Type_Def = R.As_Bare_Signed_Int_Type_Def;
         when Mmz_Bare_Subtype_Decl =>
               return L.As_Bare_Subtype_Decl = R.As_Bare_Subtype_Decl;
         when Mmz_Bare_Subunit =>
               return L.As_Bare_Subunit = R.As_Bare_Subunit;
         when Mmz_Bare_Task_Type_Decl =>
               return L.As_Bare_Task_Type_Decl = R.As_Bare_Task_Type_Decl;
         when Mmz_Bare_Type_Decl =>
               return L.As_Bare_Type_Decl = R.As_Bare_Type_Decl;
         when Mmz_Bare_Type_Expr =>
               return L.As_Bare_Type_Expr = R.As_Bare_Type_Expr;
         when Mmz_Boolean =>
               return L.As_Boolean = R.As_Boolean;
         when Mmz_Env_Rebindings =>
               return L.As_Env_Rebindings = R.As_Env_Rebindings;
         when Mmz_Integer =>
               return L.As_Integer = R.As_Integer;
         when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
               return Equivalent (L.As_Internal_Entity_Base_Type_Decl_Array_Access, R.As_Internal_Entity_Base_Type_Decl_Array_Access);
         when Mmz_Internal_Entity_Expr =>
               return L.As_Internal_Entity_Expr = R.As_Internal_Entity_Expr;
         when Mmz_Internal_Entity_Info =>
               return L.As_Internal_Entity_Info = R.As_Internal_Entity_Info;
         when Mmz_Internal_Metadata =>
               return L.As_Internal_Metadata = R.As_Internal_Metadata;
         when Mmz_Lexical_Env =>
               return Equivalent (L.As_Lexical_Env, R.As_Lexical_Env);
         when Mmz_Symbol_Type =>
               return L.As_Symbol_Type = R.As_Symbol_Type;
   end case;
end Equivalent;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key_Item) return Hash_Type is
begin
   case Key.Kind is
         when Mmz_Bare_Access_Def =>
            return Hash (Key.As_Bare_Access_Def);
         when Mmz_Bare_Ada_Node =>
            return Hash (Key.As_Bare_Ada_Node);
         when Mmz_Bare_Anonymous_Expr_Decl =>
            return Hash (Key.As_Bare_Anonymous_Expr_Decl);
         when Mmz_Bare_Array_Subcomponent_Choice_Name =>
            return Hash (Key.As_Bare_Array_Subcomponent_Choice_Name);
         when Mmz_Bare_Array_Type_Def =>
            return Hash (Key.As_Bare_Array_Type_Def);
         when Mmz_Bare_Assoc_List =>
            return Hash (Key.As_Bare_Assoc_List);
         when Mmz_Bare_Base_Aggregate =>
            return Hash (Key.As_Bare_Base_Aggregate);
         when Mmz_Bare_Base_Id =>
            return Hash (Key.As_Bare_Base_Id);
         when Mmz_Bare_Base_Subp_Spec =>
            return Hash (Key.As_Bare_Base_Subp_Spec);
         when Mmz_Bare_Base_Type_Decl =>
            return Hash (Key.As_Bare_Base_Type_Decl);
         when Mmz_Bare_Basic_Decl =>
            return Hash (Key.As_Bare_Basic_Decl);
         when Mmz_Bare_Body_Node =>
            return Hash (Key.As_Bare_Body_Node);
         when Mmz_Bare_Compilation_Unit =>
            return Hash (Key.As_Bare_Compilation_Unit);
         when Mmz_Bare_Decimal_Fixed_Point_Def =>
            return Hash (Key.As_Bare_Decimal_Fixed_Point_Def);
         when Mmz_Bare_Defining_Name =>
            return Hash (Key.As_Bare_Defining_Name);
         when Mmz_Bare_End_Name =>
            return Hash (Key.As_Bare_End_Name);
         when Mmz_Bare_Enum_Literal_Decl =>
            return Hash (Key.As_Bare_Enum_Literal_Decl);
         when Mmz_Bare_Enum_Type_Def =>
            return Hash (Key.As_Bare_Enum_Type_Def);
         when Mmz_Bare_Expr =>
            return Hash (Key.As_Bare_Expr);
         when Mmz_Bare_For_Loop_Var_Decl =>
            return Hash (Key.As_Bare_For_Loop_Var_Decl);
         when Mmz_Bare_Formal_Discrete_Type_Def =>
            return Hash (Key.As_Bare_Formal_Discrete_Type_Def);
         when Mmz_Bare_Formal_Subp_Decl =>
            return Hash (Key.As_Bare_Formal_Subp_Decl);
         when Mmz_Bare_Generic_Instantiation =>
            return Hash (Key.As_Bare_Generic_Instantiation);
         when Mmz_Bare_Mod_Int_Type_Def =>
            return Hash (Key.As_Bare_Mod_Int_Type_Def);
         when Mmz_Bare_Name =>
            return Hash (Key.As_Bare_Name);
         when Mmz_Bare_Ordinary_Fixed_Point_Def =>
            return Hash (Key.As_Bare_Ordinary_Fixed_Point_Def);
         when Mmz_Bare_Private_Type_Def =>
            return Hash (Key.As_Bare_Private_Type_Def);
         when Mmz_Bare_Real_Type_Def =>
            return Hash (Key.As_Bare_Real_Type_Def);
         when Mmz_Bare_Record_Type_Def =>
            return Hash (Key.As_Bare_Record_Type_Def);
         when Mmz_Bare_Signed_Int_Type_Def =>
            return Hash (Key.As_Bare_Signed_Int_Type_Def);
         when Mmz_Bare_Subtype_Decl =>
            return Hash (Key.As_Bare_Subtype_Decl);
         when Mmz_Bare_Subunit =>
            return Hash (Key.As_Bare_Subunit);
         when Mmz_Bare_Task_Type_Decl =>
            return Hash (Key.As_Bare_Task_Type_Decl);
         when Mmz_Bare_Type_Decl =>
            return Hash (Key.As_Bare_Type_Decl);
         when Mmz_Bare_Type_Expr =>
            return Hash (Key.As_Bare_Type_Expr);
         when Mmz_Boolean =>
            return Hash (Key.As_Boolean);
         when Mmz_Env_Rebindings =>
            return Hash (Key.As_Env_Rebindings);
         when Mmz_Integer =>
            return Hash (Key.As_Integer);
         when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
            return Hash (Key.As_Internal_Entity_Base_Type_Decl_Array_Access);
         when Mmz_Internal_Entity_Expr =>
            return Hash (Key.As_Internal_Entity_Expr);
         when Mmz_Internal_Entity_Info =>
            return Hash (Key.As_Internal_Entity_Info);
         when Mmz_Internal_Metadata =>
            return Hash (Key.As_Internal_Metadata);
         when Mmz_Lexical_Env =>
            return Hash (Key.As_Lexical_Env);
         when Mmz_Symbol_Type =>
            return Hash (Key.As_Symbol_Type);
   end case;
end Hash;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key) return Hash_Type is
   Result : Hash_Type := Mmz_Property'Pos (Key.Property);
begin
   for K of Key.Items.all loop
      Result := Combine (Result, Hash (K));
   end loop;
   return Result;
end Hash;
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key) return Boolean is
   L_Items : Mmz_Key_Array renames L.Items.all;
   R_Items : Mmz_Key_Array renames R.Items.all;
begin
   if L.Property /= R.Property or else L_Items'Length /= R_Items'Length then
      return False;
   end if;
   for I in L_Items'Range loop
      if not Equivalent (L_Items (I), R_Items (I)) then
         return False;
      end if;
   end loop;
   return True;
end Equivalent;
-------------
-- Destroy --
-------------
procedure Destroy (Map : in out Memoization_Maps.Map) is
   use Memoization_Maps;
   --  We need keys and values to be valid when clearing the memoization map,
   --  but on the other hand we need to free keys and values as well. To
   --  achieve both goals, we first copy key and values into arrays, then we
   --  clear the map, and then we free keys/values in the arrays. Allocate both
   --  arrays on the heap to avoid stack overflow, as they can be quite big.
   Length : constant Natural := Natural (Map.Length);
   type Key_Array is array (1 .. Length) of Mmz_Key_Array_Access;
   type Key_Array_Access is access Key_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Key_Array, Key_Array_Access);
   type Value_Array is array (1 .. Length) of Mmz_Value;
   type Value_Array_Access is access Value_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Value_Array, Value_Array_Access);
   Keys   : Key_Array_Access := new Key_Array;
   Values : Value_Array_Access := new Value_Array;
   I      : Positive := 1;
begin
   for Cur in Map.Iterate loop
      Keys (I) := Key (Cur).Items;
      Values (I) := Element (Cur);
      I := I + 1;
   end loop;
   Map.Clear;
   for K_Array of Keys.all loop
      Destroy (K_Array);
   end loop;
   for V of Values.all loop
      case V.Kind is
         when Mmz_Error =>
            Free_Memoized_Error (V.Exc_Id, V.Exc_Msg);
            when Mmz_Internal_Entity_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Array_Access);
            when Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access);
            when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Base_Type_Decl_Array_Access);
            when Mmz_Internal_Entity_Basic_Decl_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Basic_Decl_Array_Access);
            when Mmz_Internal_Entity_Compilation_Unit_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Compilation_Unit_Array_Access);
            when Mmz_Internal_Entity_Expr_Array_Access =>
               Dec_Ref (V.As_Internal_Entity_Expr_Array_Access);
            when Mmz_Internal_Env_Assoc =>
               Dec_Ref (V.As_Internal_Env_Assoc);
            when Mmz_Internal_Env_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Env_Assoc_Array_Access);
            when Mmz_Internal_Inner_Env_Assoc_Array_Access =>
               Dec_Ref (V.As_Internal_Inner_Env_Assoc_Array_Access);
            when Mmz_Internal_Param_Match_Array_Access =>
               Dec_Ref (V.As_Internal_Param_Match_Array_Access);
            when Mmz_Internal_Single_Actual_Array_Access =>
               Dec_Ref (V.As_Internal_Single_Actual_Array_Access);
            when Mmz_Internal_Unit_Array_Access =>
               Dec_Ref (V.As_Internal_Unit_Array_Access);
            when Mmz_Lexical_Env =>
               Dec_Ref (V.As_Lexical_Env);
         when others => null;
      end case;
   end loop;
   Free (Keys);
   Free (Values);
end Destroy;
-------------
-- Destroy --
-------------
procedure Destroy (Key : in out Mmz_Key_Array_Access) is
   procedure Free is new Ada.Unchecked_Deallocation
     (Mmz_Key_Array, Mmz_Key_Array_Access);
begin
      for K of Key.all loop
         case K.Kind is
               when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
                  Dec_Ref (K.As_Internal_Entity_Base_Type_Decl_Array_Access);
               when Mmz_Lexical_Env =>
                  Dec_Ref (K.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Key);
end Destroy;
-------------------------
-- Find_Memoized_Value --
-------------------------
function Find_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : out Memoization_Handle;
   Value      : out Mmz_Value;
   Create_Key : access function return Mmz_Key) return Boolean
is
   Inserted : Boolean;
begin
   --  Make sure that we don't lookup stale caches
   Reset_Caches (Unit);
   --  Initialize handle: create the key and create a cursor pointing to an
   --  existing entry.
   Handle.Key := Create_Key.all;
   Handle.Cache_Version := Unit.Cache_Version;
   Value := (Kind => Mmz_Evaluating);
   Unit.Memoization_Map.Insert (Handle.Key, Value, Handle.Cur, Inserted);
   --  No existing entry yet? The above just created one. Otherwise, destroy
   --  our key and reuse the existing entry's.
   if not Inserted then
      Destroy (Handle.Key.Items);
      Handle.Key := Memoization_Maps.Key (Handle.Cur);
      Value := Memoization_Maps.Element (Handle.Cur);
   end if;
   return not Inserted;
end Find_Memoized_Value;
------------------------
-- Add_Memoized_Value --
------------------------
procedure Add_Memoized_Value
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Value  : Mmz_Value;
   Stored : out Boolean) is
begin
   --  If Handle was created using a memoization map that has been since then
   --  reset, do nothing: the result can be partly stale due to the event that
   --  triggered the memoization tables reset.
   Stored := Unit.Cache_Version <= Handle.Cache_Version;
   if Stored then
      Unit.Memoization_Map.Replace_Element (Handle.Cur, Value);
   end if;
end Add_Memoized_Value;
------------------------
-- Add_Memoized_Error --
------------------------
procedure Add_Memoized_Error
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Exc    : Ada.Exceptions.Exception_Occurrence;
   Stored : out Boolean)
is
   Value : Mmz_Value (Kind => Mmz_Error);
begin
   Store_Memoized_Error (Exc, Value.Exc_Id, Value.Exc_Msg);
   Add_Memoized_Value (Unit, Handle, Value, Stored);
   if not Stored then
      Free_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
   end if;
end Add_Memoized_Error;
----------------------------
-- Reraise_Memoized_Error --
----------------------------
procedure Reraise_Memoized_Error (Value : Mmz_Value) is
begin
   Reraise_Memoized_Error (Value.Exc_Id, Value.Exc_Msg);
end Reraise_Memoized_Error;
--------------------------
-- Store_Memoized_Error --
--------------------------
procedure Store_Memoized_Error
  (Exc     : Ada.Exceptions.Exception_Occurrence;
   Exc_Id  : out Ada.Exceptions.Exception_Id;
   Exc_Msg : out String_Access) is
begin
   Exc_Id := Ada.Exceptions.Exception_Identity (Exc);
   Exc_Msg := new String'(Ada.Exceptions.Exception_Message (Exc));
end Store_Memoized_Error;
-------------------------
-- Free_Memoized_Error --
-------------------------
procedure Free_Memoized_Error
  (Exc_Id  : in out Ada.Exceptions.Exception_Id;
   Exc_Msg : in out String_Access)
is
   pragma Unreferenced (Exc_Id);
begin
   Free (Exc_Msg);
end Free_Memoized_Error;
----------------------------
-- Reraise_Memoized_Error --
----------------------------
procedure Reraise_Memoized_Error
  (Exc_Id  : Ada.Exceptions.Exception_Id;
   Exc_Msg : String_Access) is
begin
   Ada.Exceptions.Raise_Exception (Exc_Id, Exc_Msg.all & " (memoized)");
end Reraise_Memoized_Error;
   package Solver_Diagnostic_Vectors is new Langkit_Support.Vectors
     (Internal_Solver_Diagnostic);
   ----------------------------
   -- Allocate_Logic_Context --
   ----------------------------
   function Allocate_Logic_Context
     (Ctx : Internal_Logic_Context) return Internal_Logic_Context_Access
   is (new Internal_Logic_Context'(Ctx));
   -------------------------
   -- Trace_Logic_Context --
   -------------------------
   function Trace_Logic_Context
     (Ctx : Internal_Logic_Context_Access) return String
   is (Trace_Image (Ctx.all));
   -----------------
   -- Deep_Equals --
   -----------------
   function Deep_Equals
     (X, Y : Internal_Logic_Context_Access) return Boolean
   is (X.all = Y.all);
   ------------------------
   -- Free_Logic_Context --
   ------------------------
   procedure Free_Logic_Context
     (Ctx : in out Internal_Logic_Context_Access)
   is
      procedure Free is new Ada.Unchecked_Deallocation
        (Internal_Logic_Context, Internal_Logic_Context_Access);
   begin
      Free (Ctx);
   end Free_Logic_Context;
   -------------------
   -- Solve_Wrapper --
   -------------------
   function Solve_Wrapper
     (R            : Solver.Relation;
      Context_Node : Bare_Ada_Node) return Boolean is
   begin
      if Context_Node /= null and then Langkit_Support.Adalog.Debug.Debug then
         Assign_Names_To_Logic_Vars (Context_Node);
      end if;
      begin
         return Solver.Solve_First
           (R, Timeout => Context_Node.Unit.Context.Logic_Resolution_Timeout);
      exception
         when Langkit_Support.Adalog.Early_Binding_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "invalid equation for logic resolution");
         when Langkit_Support.Adalog.Timeout_Error =>
            Raise_Property_Exception
              (Context_Node,
               Property_Error'Identity,
               "logic resolution timed out");
      end;
   end Solve_Wrapper;
   ----------------------------
   -- Solve_With_Diagnostics --
   ----------------------------
   function Solve_With_Diagnostics
     (R            : Solver.Relation;
      Context_Node : Bare_Ada_Node) return Internal_Solver_Result
   is
      Ret : Internal_Solver_Result :=
        (True, No_Internal_Solver_Diagnostic_Array_Type);
      Acc : Solver_Diagnostic_Vectors.Vector;
      --  Vectors that will accumulate diagnostic emitted during resolution
      procedure Emit_Diagnostic (Diag : Internal_Solver_Diagnostic) is
      begin
         Acc.Append (Diag);
      end Emit_Diagnostic;
   begin
      Ret.Success := Solve_Wrapper (R, Context_Node);
      if not Ret.Success then
         Ret.Success := Solver.Solve_First
           (R,
            Solve_Options => (Report_Errors => True),
            Diag_Emitter => Emit_Diagnostic'Unrestricted_Access,
            Timeout => Context_Node.Unit.Context.Logic_Resolution_Timeout);
         Ret.Diagnostics := Create_Internal_Solver_Diagnostic_Array
           (Acc.Length);
         for I in 1 .. Acc.Length loop
            Ret.Diagnostics.Items (I) := Acc.Get (I);
         end loop;
         Acc.Destroy;
      end if;
      return Ret;
   end Solve_With_Diagnostics;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Env : in out Lexical_Env_Access) is
      Mutable_Env : Lexical_Env :=
        (Wrap (Env), 0, Env.Kind, No_Generic_Unit, 0);
   begin
      Destroy (Mutable_Env);
      Env := null;
   end Destroy;
   ----------------
   -- Initialize --
   ----------------
   procedure Initialize
     (Self              : Bare_Ada_Node;
      Kind              : Ada_Node_Kind_Type;
      Unit              : Internal_Unit;
      Token_Start_Index : Token_Index;
      Token_End_Index   : Token_Index;
      Parent            : Bare_Ada_Node := null;
      Self_Env          : Lexical_Env := AST_Envs.Empty_Env) is
   begin
      pragma Unreferenced (Kind);
      Self.Parent := Parent;
      Self.Unit := Unit;
      Self.Token_Start_Index := Token_Start_Index;
      Self.Token_End_Index := Token_End_Index;
      Self.Self_Env := Self_Env;
      Self.Last_Attempted_Child := -1;
   end Initialize;
   --------------------
   -- Use_Direct_Env --
   --------------------
   procedure Use_Direct_Env (State : in out PLE_Node_State; Env : Lexical_Env)
   is
   begin
      State.Current_Env := Env;
      State.Current_NED := null;
   end Use_Direct_Env;
   -------------------
   -- Use_Named_Env --
   -------------------
   procedure Use_Named_Env
     (State   : in out PLE_Node_State;
      Context : Internal_Context;
      Name    : Symbol_Type) is
   begin
      State.Current_NED := Get_Named_Env_Descriptor (Context, Name);
      State.Current_Env := State.Current_NED.Env_With_Precedence;
   end Use_Named_Env;
   ---------------------
   -- Set_Initial_Env --
   ---------------------
   procedure Set_Initial_Env
     (Self         : Bare_Ada_Node;
      State        : in out PLE_Node_State;
      Env          : Internal_Designated_Env;
      DSL_Location : String) is
   begin
      case Env.Kind is
         when None =>
            Use_Direct_Env (State, Empty_Env);
         when Current_Env =>
            null;
         when Named_Env =>
            Use_Named_Env (State, Self.Unit.Context, Env.Env_Name);
         when Direct_Env =>
            --  Sanitize this environment value: make sure it's a non-foreign
            --  and primary environment.
            if Env.Direct_Env.Kind /= Static_Primary then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "Cannot set an env that is not static-primary as the"
                  & " initial env");
            elsif Is_Foreign_Strict (Env.Direct_Env, Self) then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "unsound foreign environment in SetInitialEnv ("
                  & DSL_Location & ")");
            end if;
            Use_Direct_Env (State, Env.Direct_Env);
      end case;
   end Set_Initial_Env;
   ----------------
   -- Add_To_Env --
   ----------------
   procedure Add_To_Env
     (Self         : Bare_Ada_Node;
      State        : PLE_Node_State;
      Key          : Symbol_Type;
      Value        : Bare_Ada_Node;
      Md           : Internal_Metadata;
      Resolver     : Entity_Resolver;
      Dest_Env     : Internal_Designated_Env;
      DSL_Location : String)
   is
      Context    : constant Internal_Context := Self.Unit.Context;
      Root_Scope : Lexical_Env renames Context.Root_Scope;
      --  Shortcuts
      Actual_Dest_Env : Lexical_Env;
      Dest_NED        : Named_Env_Descriptor_Access;
      --  Description for the destination environment
   begin
      --  Skip the env addition if explicitly requested
      if Key = No_Symbol
         or else Value = null
         or else (case Dest_Env.Kind is
                  when None        => True,
                  when Current_Env => False,
                  when Named_Env   => Dest_Env.Env_Name = No_Symbol,
                  when Direct_Env  => Dest_Env.Direct_Env = Empty_Env)
      then
         return;
      end if;
      if Value.Unit /= Self.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add_to_env an AST node that comes from another analysis"
            & " unit");
      end if;
         --  Make sure metadata does not contain any foreign node
         if (Md.Primitive /= null and then Md.Primitive.Unit /= Self.Unit) or else (Md.Primitive_Real_Type /= null and then Md.Primitive_Real_Type.Unit /= Self.Unit)
         then
            Raise_Property_Exception
              (Self,
               Property_Error'Identity,
               "Cannot add metadata that contains foreign nodes");
         end if;
      --  Then determine the destination environment
      case Dest_Env.Kind is
         when None =>
            raise Program_Error with "unreachable code";
         when Current_Env =>
            --  Just use the current environment
            Dest_NED := State.Current_NED;
            Actual_Dest_Env := State.Current_Env;
         when Named_Env =>
            --  There is an environment name: just lookup the corresponding
            --  NED/env.
            Dest_NED := Get_Named_Env_Descriptor (Context, Dest_Env.Env_Name);
            Actual_Dest_Env := Dest_NED.Env_With_Precedence;
         when Direct_Env =>
            --  There is an explicit destination environment
            Dest_NED := null;
            Actual_Dest_Env := Dest_Env.Direct_Env;
      end case;
      --  Sanitize it
      if Actual_Dest_Env.Kind /= Static_Primary then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add elements to a lexical env that is not static-primary");
      elsif
         --  Since lexical envs need to sort the foreign nodes they contain,
         --  and that the total order on nodes is not defined for synthetic
         --  nodes, it is not possible to add a synthetic node to a foreign
         --  lexical environment.
         --
         --  This reasoning applies to environments that belong to foreign
         --  units, but also to the root environment.
         Is_Foreign (Actual_Dest_Env, Self) and then Is_Synthetic (Value)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "Cannot add a synthetic node to a lexical env from another"
            & " analysis unit");
      elsif
         --  Reject direct references to foreign destination environments.
         --
         --  This is an attempt at identifying uses of the unsound relocation
         --  mechanism (as opposed to named environments), so this applies to
         --  all foreign environments (root scope included).
         DSL_Location'Length > 0
         and then Dest_Env.Kind = Direct_Env
         and then Is_Foreign_Strict (Actual_Dest_Env, Self)
      then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "unsound foreign environment in AddToEnv (" & DSL_Location & ")");
      end if;
      --  Now that everything is sanitized, we can proceed with the actual
      --  key/value pair addition. Note that this does nothing if
      --  Actual_Dest_Env ended up empty.
      Add (Actual_Dest_Env, Thin (Key), Value, Md, Resolver);
      --  If we're adding the element to an environment by env name, we must
      --  register this association in two places: in the target named env
      --  entry, and in Value's unit.
      if Dest_NED /= null then
         declare
            use NED_Assoc_Maps;
            FN    : Map renames Dest_NED.Foreign_Nodes;
            Dummy : Boolean;
            Cur   : Cursor;
         begin
            FN.Insert (Key      => Key,
                       New_Item => Internal_Map_Node_Vectors.Empty_Vector,
                       Position => Cur,
                       Inserted => Dummy);
            declare
               V : Internal_Map_Node_Vectors.Vector renames
                  FN.Reference (Cur);
            begin
               V.Append ((Value, null, Md, Resolver));
               --  Note that the rebindings field is unused by the relocation
               --  mechanism (since we don't even allow adding env entries
               --  with custom rebindings), hence we simply leave it to null.
            end;
         end;
         Value.Unit.Exiled_Entries_In_NED.Append ((Dest_NED, Key, Value));
      --  Otherwise, if we're adding the element to an environment that belongs
      --  to a different unit, or to the root scope, then...
      elsif Is_Foreign_Not_Empty (Actual_Dest_Env, Self) then
         --  Add the Key/Value association to the list of entries contained in
         --  other units, so we can remove them when reparsing Value's unit.
         Value.Unit.Exiled_Entries.Append ((Actual_Dest_Env, Key, Value));
         if Actual_Dest_Env /= Root_Scope then
            --  Add Val to the list of foreign nodes that Actual_Dest_Env's
            --  unit contains, so that when that unit is reparsed, we can call
            --  Add_To_Env again on those nodes.
            Convert_Unit (Actual_Dest_Env.Owner).Foreign_Nodes.Append
              ((Value, Self.Unit));
         end if;
      end if;
   end Add_To_Env;
   -------------
   -- Ref_Env --
   -------------
   procedure Ref_Env
     (Self                : Bare_Ada_Node;
      Dest_Env            : Lexical_Env;
      Ref_Env_Nodes       : in out Bare_Ada_Node_Array_Access;
      Resolver            : Lexical_Env_Resolver;
      Kind                : Ref_Kind;
      Cats                : Ref_Categories;
      Shed_Rebindings     : Boolean) is
   begin
      for N of Ref_Env_Nodes.Items loop
         if N /= null then
            if N.Unit /= Self.Unit then
               Raise_Property_Exception
                 (Self,
                  Property_Error'Identity,
                  "attempt to add a referenced environment to a foreign unit");
            end if;
            Reference (Dest_Env, N, Resolver, Kind, Cats, Shed_Rebindings);
         end if;
      end loop;
      Dec_Ref (Ref_Env_Nodes);
   end Ref_Env;
   -------------
   -- Add_Env --
   -------------
   procedure Add_Env
     (Self              : Bare_Ada_Node;
      State             : in out PLE_Node_State;
      No_Parent         : Boolean;
      Transitive_Parent : Boolean;
      Names             : in out Symbol_Type_Array_Access)
   is
      Parent_From_Name : constant Boolean := State.Current_NED /= null;
      --  Does the parent environment comes from a named environment lookup?
      --  Determine the parent of this new environment:
      --
      --  (1) no parent if requested;
      --  (2) the current environment as the static parent if it comes from a
      --      named env lookup or if it is not foreign (or is the empty/root
      --      environment).
      Parent : constant Lexical_Env :=
        (if No_Parent
         then Null_Lexical_Env
         else State.Current_Env);
   begin
      --  Create the environment itself
      Self.Self_Env := Create_Static_Lexical_Env
        (Parent            => Parent,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Sym_Table         => Self.Unit.Context.Symbols);
      --  If the parent of this new environment comes from a named environment
      --  lookup, register this new environment so that its parent is updated
      --  when the precence for this named environment changes.
      if Parent_From_Name then
         declare
            NED : constant Named_Env_Descriptor_Access := State.Current_NED;
         begin
            Self.Unit.Exiled_Envs.Append ((NED, Self.Self_Env));
            NED.Foreign_Envs.Insert (Self, Self.Self_Env);
         end;
      end if;
      --  From now on, the current environment is Self.Self_Env, with a direct
      --  access to it. It does not go through the env naming scheme, since
      --  only this node and its children (i.e. non-foreign nodes) will access
      --  it as a "current" environment during PLE.
      Use_Direct_Env (State, Self.Self_Env);
      --  Register the environment we just created on all the requested names
      if Names /= null then
         declare
            Context   : constant Internal_Context := Self.Unit.Context;
            Env       : constant Lexical_Env := Self.Self_Env;
            NENU      : NED_Maps.Map renames
               State.Unit_State.Named_Envs_Needing_Update;
         begin
            for N of Names.Items loop
               Register_Named_Env (Context, N, Env, NENU);
            end loop;
            Dec_Ref (Names);
         end;
      end if;
   end Add_Env;
   ---------------------
   -- Pre_Env_Actions --
   ---------------------
   procedure Pre_Env_Actions
     (Self            : Bare_Ada_Node;
      State           : in out PLE_Node_State;
      Add_To_Env_Only : Boolean := False) is
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            With_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Access_To_Subp_Def =>
            Access_To_Subp_Def_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Named_Stmt =>
            Named_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Extended_Return_Stmt =>
            Extended_Return_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Begin_Block .. Ada_Decl_Block =>
            Block_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Stmt =>
            For_Loop_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            Accept_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Pragma_Node =>
            Pragma_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Quantified_Expr =>
            Quantified_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Decl_Expr =>
            Decl_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Private_Part =>
            Private_Part_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Decl =>
            Single_Task_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Protected_Decl =>
            Single_Protected_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Renaming_Decl =>
            Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
            Object_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Number_Decl =>
            Number_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Label_Decl =>
            Label_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Renaming_Decl =>
            Generic_Subp_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Renaming_Decl =>
            Generic_Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
            Generic_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Decl =>
            Generic_Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Var_Decl =>
            For_Loop_Var_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Handler =>
            Exception_Handler_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Decl =>
            Exception_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Index_Spec =>
            Entry_Index_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body =>
            Task_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body =>
            Protected_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body =>
            Package_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Body =>
            Entry_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body_Stub =>
            Task_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Subp_Body_Stub =>
            Subp_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body_Stub =>
            Protected_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body_Stub =>
            Package_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            Base_Subp_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Internal =>
            Generic_Subp_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
            Enum_Literal_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Decl =>
            Entry_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Synthetic_Subp_Decl =>
            Basic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
            Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Type_Decl =>
            Single_Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Type_Decl =>
            Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Type_Decl =>
            Protected_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            Incomplete_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Subtype_Decl =>
            Subtype_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Discrete_Base_Subtype_Decl =>
            Base_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Decl =>
            Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Internal =>
            Generic_Package_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Param_Spec =>
            Param_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Discriminant_Spec =>
            Discriminant_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Component_Decl =>
            Component_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_State_Decl =>
            Abstract_State_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      when others =>  null; 
   end case;
   end Pre_Env_Actions;
   ----------------------
   -- Post_Env_Actions --
   ----------------------
   procedure Post_Env_Actions
     (Self : Bare_Ada_Node; State : in out PLE_Node_State) is
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            null;
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Post_Env_Actions (Self, State);
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Post_Env_Actions (Self, State);
            when Ada_Access_To_Subp_Def =>
            null;
            when Ada_Named_Stmt =>
            null;
            when Ada_Extended_Return_Stmt =>
            null;
            when Ada_Begin_Block .. Ada_Decl_Block =>
            null;
            when Ada_For_Loop_Stmt =>
            null;
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            null;
            when Ada_Pragma_Node =>
            null;
            when Ada_Quantified_Expr =>
            null;
            when Ada_Decl_Expr =>
            null;
            when Ada_Private_Part =>
            null;
            when Ada_Single_Task_Decl =>
            null;
            when Ada_Single_Protected_Decl =>
            null;
            when Ada_Package_Renaming_Decl =>
            null;
            when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
            null;
            when Ada_Number_Decl =>
            null;
            when Ada_Label_Decl =>
            null;
            when Ada_Generic_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
            null;
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Post_Env_Actions (Self, State);
            when Ada_Generic_Package_Decl =>
            null;
            when Ada_For_Loop_Var_Decl =>
            null;
            when Ada_Exception_Handler =>
            null;
            when Ada_Exception_Decl =>
            null;
            when Ada_Entry_Index_Spec =>
            null;
            when Ada_Task_Body =>
            null;
            when Ada_Protected_Body =>
            null;
            when Ada_Package_Body =>
            null;
            when Ada_Entry_Body =>
            null;
            when Ada_Task_Body_Stub =>
            null;
            when Ada_Subp_Body_Stub =>
            null;
            when Ada_Protected_Body_Stub =>
            null;
            when Ada_Package_Body_Stub =>
            null;
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Subp_Internal =>
            null;
            when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
            null;
            when Ada_Entry_Decl =>
            null;
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Synthetic_Subp_Decl =>
            Basic_Subp_Decl_Post_Env_Actions (Self, State);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
            Type_Decl_Post_Env_Actions (Self, State);
            when Ada_Single_Task_Type_Decl =>
            null;
            when Ada_Task_Type_Decl =>
            null;
            when Ada_Protected_Type_Decl =>
            null;
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            null;
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Subtype_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl =>
            null;
            when Ada_Package_Decl =>
            null;
            when Ada_Generic_Package_Internal =>
            null;
            when Ada_Param_Spec =>
            null;
            when Ada_Discriminant_Spec =>
            null;
            when Ada_Component_Decl =>
            null;
            when Ada_Abstract_State_Decl =>
            null;
      when others =>  null; 
   end case;
   end Post_Env_Actions;
   ----------------
   -- Get_Symbol --
   ----------------
   function Get_Symbol
     (Node : Bare_Ada_Node) return Symbol_Type is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the symbol of a null node");
      end if;
      return Get_Symbol (Token (Node, Node.Token_Start_Index));
   end Get_Symbol;
   ----------
   -- Text --
   ----------
   function Text
     (Node : Bare_Ada_Node) return Text_Type
   is
   begin
      if Node = null then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "cannot get the text of a null node");
      end if;
      declare
         Start_T : constant Token_Reference :=
            Token (Node, Node.Token_Start_Index);
         End_T   : constant Token_Reference :=
            Token (Node, Node.Token_End_Index);
      begin
         --  No text is associated to synthetic and ghost nodes
         if Is_Synthetic (Node) then
            return "";
         end if;
         if Is_Ghost (Node) then
            return "";
         end if;
         return Text (Start_T, End_T);
      end;
   end Text;
   ---------------------
   -- Is_Visible_From --
   ---------------------
   function Is_Visible_From
     (Self                     : Bare_Ada_Node;
      Referenced_Env, Base_Env : Lexical_Env) return Boolean
   is
      Referenced_Unit : constant Internal_Unit :=
         Convert_Unit (Referenced_Env.Owner);
      Base_Unit       : constant Internal_Unit :=
         Convert_Unit (Base_Env.Owner);
   begin
      if Referenced_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "referenced environment does not belong to any analysis unit");
      elsif Base_Unit = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "base environment does not belong to any analysis unit");
      end if;
      return Is_Referenced_From (Referenced_Unit, Base_Unit);
   end Is_Visible_From;
   ----------
   -- Unit --
   ----------
   function Unit (Node : Bare_Ada_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Unit;
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node;
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node);
   --  Implementation helpers for the looking up process
   -----------------
   -- Set_Parents --
   -----------------
   procedure Set_Parents
     (Node, Parent : Bare_Ada_Node)
   is
   begin
      if Node = null then
         return;
      end if;
      Node.Parent := Bare_Ada_Node (Parent);
      for I in 1 .. Children_Count (Node) loop
         Set_Parents (Child (Node, I), Node);
      end loop;
   end Set_Parents;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Node : Bare_Ada_Node) is
   begin
      if Node = null then
         return;
      end if;
      Free_User_Fields (Node);
      for I in 1 .. Children_Count (Node) loop
         Destroy (Child (Node, I));
      end loop;
   end Destroy;
   -----------
   -- Child --
   -----------
   function Child (Node  : Bare_Ada_Node;
                   Index : Positive) return Bare_Ada_Node
   is
      Result          : Bare_Ada_Node;
      Index_In_Bounds : Boolean;
   begin
      Get_Child (Node, Index, Index_In_Bounds, Result);
      return Result;
   end Child;
   --------------
   -- Traverse --
   --------------
   function Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
              return Visit_Status)
     return Visit_Status
   is
      Status : Visit_Status := Into;
   begin
      if Node /= null then
         Status := Visit (Node);
         --  Skip processing the child nodes if the returned status is Over
         --  or Stop. In the former case the previous call to Visit has taken
         --  care of processing the needed childs, and in the latter case we
         --  must immediately stop processing the tree.
         if Status = Into then
            for I in 1 .. Children_Count (Node) loop
               declare
                  Cur_Child : constant Bare_Ada_Node :=
                     Child (Node, I);
               begin
                  if Cur_Child /= null then
                     Status := Traverse (Cur_Child, Visit);
                     exit when Status /= Into;
                  end if;
               end;
            end loop;
         end if;
      end if;
      if Status = Stop then
         return Stop;
      --  At this stage the Over status has no sense and we just continue
      --  processing the tree.
      else
         return Into;
      end if;
   end Traverse;
   --------------
   -- Traverse --
   --------------
   procedure Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
                               return Visit_Status)
   is
      Result_Status : Visit_Status;
      pragma Unreferenced (Result_Status);
   begin
      Result_Status := Traverse (Node, Visit);
   end Traverse;
   ------------------------
   -- Traverse_With_Data --
   ------------------------
   function Traverse_With_Data
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node;
                               Data : in out Data_Type)
                               return Visit_Status;
      Data  : in out Data_Type)
      return Visit_Status
   is
      function Helper (Node : Bare_Ada_Node) return Visit_Status;
      ------------
      -- Helper --
      ------------
      function Helper (Node : Bare_Ada_Node) return Visit_Status is
      begin
         return Visit (Node, Data);
      end Helper;
      Saved_Data : Data_Type;
      Result     : Visit_Status;
   begin
      if Reset_After_Traversal then
         Saved_Data := Data;
      end if;
      Result := Traverse (Node, Helper'Access);
      if Reset_After_Traversal then
         Data := Saved_Data;
      end if;
      return Result;
   end Traverse_With_Data;
   ----------------
   -- Sloc_Range --
   ----------------
   function Sloc_Range
     (Node : Bare_Ada_Node) return Source_Location_Range
   is
      type Token_Anchor is (T_Start, T_End);
      type Token_Pos is record
         Pos    : Token_Index;
         Anchor : Token_Anchor;
      end record;
      TDH                    : Token_Data_Handler renames Node.Unit.TDH;
      Token_Start, Token_End : Token_Pos;
      function Get (Index : Token_Index) return Stored_Token_Data is
        (Get_Token (TDH, Index));
      function Sloc (T : Token_Pos) return Source_Location is
        (if T.Anchor = T_Start
         then Sloc_Start (TDH, Get (T.Pos))
         else Sloc_End (TDH, Get (T.Pos)));
   begin
      if Is_Synthetic (Node) then
         return (if Node.Parent = null
                 then No_Source_Location_Range
                 else Sloc_Range (Node.Parent));
      end if;
      if Is_Ghost (Node) then
         Token_Start := (if Node.Token_Start_Index = 1
                         then (1, T_Start)
                         else (Node.Token_Start_Index - 1, T_End));
         Token_End := Token_Start;
      else
         Token_Start := (Node.Token_Start_Index, T_Start);
         Token_End := (Node.Token_End_Index, T_End);
      end if;
      if Snaps_At_Start (Node)
         and then not Is_Ghost (Node)
         and then Token_Start.Pos /= 1
      then
         Token_Start := (Token_Start.Pos - 1, T_End);
      end if;
      if Snaps_At_End (Node) and then Token_End.Pos /= Last_Token (TDH) then
         Token_End := (Token_End.Pos + 1, T_Start);
      end if;
      return Make_Range (Sloc (Token_Start), Sloc (Token_End));
   end Sloc_Range;
   ------------
   -- Lookup --
   ------------
   function Lookup
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      Position : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      if Sloc = No_Source_Location then
         return null;
      end if;
      Lookup_Relative
        (Bare_Ada_Node (Node), Sloc, Position, Result);
      return Result;
   end Lookup;
   ---------------------
   -- Lookup_Internal --
   ---------------------
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      --  For this implementation helper (i.e. internal primitive), we can
      --  assume that all lookups fall into this node's sloc range.
      pragma Assert (Compare (Sloc_Range (Node), Sloc) = Inside);
      Children : constant Internal_Bare_Ada_Node_Array :=
         Implementation.Children (Node);
      Pos      : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      --  Look for a child node that contains Sloc (i.e. return the most
      --  precise result).
      for Child of Children loop
         --  Note that we assume here that child nodes are ordered so that the
         --  first one has a sloc range that is before the sloc range of the
         --  second child node, etc.
         if Child /= null then
            Lookup_Relative (Child, Sloc, Pos, Result);
            case Pos is
               when Before =>
                   --  If this is the first node, Sloc is before it, so we can
                   --  stop here.  Otherwise, Sloc is between the previous
                   --  child node and the next one...  so we can stop here,
                   --  too.
                   return Node;
               when Inside =>
                   return Result;
               when After =>
                   --  Sloc is after the current child node, so see with the
                   --  next one.
                   null;
            end case;
         end if;
      end loop;
      --  If we reach this point, we found no children that covers Sloc, but
      --  Node still covers it (see the assertion).
      return Node;
   end Lookup_Internal;
   -------------
   -- Compare --
   -------------
   function Compare
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Relative_Position is
   begin
      return Compare (Sloc_Range (Node), Sloc);
   end Compare;
   ---------------------
   -- Lookup_Relative --
   ---------------------
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node)
   is
      Result : constant Relative_Position :=
        Compare (Node, Sloc);
   begin
      Position := Result;
      Node_Found := (if Result = Inside
                     then Lookup_Internal (Node, Sloc)
                     else null);
   end Lookup_Relative;
   -------------
   -- Compare --
   -------------
   function Compare
     (Self, Left, Right : Bare_Ada_Node;
      Relation          : Comparison_Relation) return Boolean
   is
      LS, RS : Source_Location;
   begin
      if Left = null or else Right = null or else Left.Unit /= Right.Unit then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "invalid node comparison");
      end if;
      LS := Start_Sloc (Sloc_Range (Left));
      RS := Start_Sloc (Sloc_Range (Right));
      return (case Relation is
              when Langkit_Support.Types.Less_Than        => LS < RS,
              when Langkit_Support.Types.Less_Or_Equal    => LS <= RS,
              when Langkit_Support.Types.Greater_Than     => LS > RS,
              when Langkit_Support.Types.Greater_Or_Equal => LS >= RS);
   end Compare;
   --------------
   -- Children --
   --------------
   function Children
     (Node : Bare_Ada_Node) return Internal_Bare_Ada_Node_Array
   is
      First : constant Integer := Bare_Ada_Node_Vectors.Index_Type'First;
      Last  : constant Integer := First + Children_Count (Node) - 1;
   begin
      return A : Internal_Bare_Ada_Node_Array (First .. Last)
      do
         for I in First .. Last loop
            A (I) := Child (Node, I);
         end loop;
      end return;
   end Children;
   function Children
     (Node : Bare_Ada_Node) return Bare_Ada_Node_Array_Access
   is
      C : Internal_Bare_Ada_Node_Array := Children (Node);
   begin
      return Ret : Bare_Ada_Node_Array_Access :=
         Create_Bare_Ada_Node_Array (C'Length)
      do
         Ret.Items := C;
      end return;
   end Children;
   ---------
   -- Get --
   ---------
   function Get
     (Self    : Bare_Ada_Node;
      Node    : Bare_Ada_List;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Length (Node : Bare_Ada_List) return Natural
      is (Node.Count);
      --  Wrapper around the Length primitive to get the compiler happy for the
      --  the package instantiation below.
      function Absolute_Get
        (L     : Bare_Ada_List;
         Index : Integer) return Bare_Ada_Node
      is (L.Nodes.all (Index + 1));
      --  L.Nodes is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_List,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Node = null and then Or_Null then
         return null;
      elsif Relative_Get (Node, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds AST list access");
      end if;
   end Get;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia
     (Node        : Bare_Ada_Node;
      Line_Prefix : String := "")
   is
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      if Node = null then
         Put_Line (Line_Prefix & "None");
         return;
      end if;
      Put_Line (Line_Prefix & Kind_Name (Node));
      for C of Children_And_Trivia (Node) loop
         case C.Kind is
            when Trivia =>
               Put_Line (Children_Prefix & Image (Text (C.Trivia)));
            when Child =>
               PP_Trivia (C.Node, Children_Prefix);
         end case;
      end loop;
   end PP_Trivia;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   function Populate_Lexical_Env (Node : Bare_Ada_Node) return Boolean is
      Context    : constant Internal_Context := Node.Unit.Context;
      Unit_State : aliased PLE_Unit_State := (Named_Envs_Needing_Update => <>);
      Root_State : constant PLE_Node_State :=
        (Unit_State  => Unit_State'Unchecked_Access,
         Current_Env => Context.Root_Scope,
         Current_NED => null);
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean;
      --  Do the lexical env population on Node and recurse on its children
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State);
      --  Given a node and its PLE state, register Node.Self_Env as being
      --  initialized through the named environment mechanism, if that's indeed
      --  the case. Do nothing otherwise.
      -----------------------
      -- Populate_Internal --
      -----------------------
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean
      is
         Result : Boolean := False;
         State  : PLE_Node_State := Parent_State;
      begin
         if Node = null then
            return Result;
         end if;
         --  By default (i.e. unless env actions add a new env), the
         --  environment we store in Node is the current one.
         Node.Self_Env := State.Current_Env;
         --  Run pre/post actions, and run PLE on children in between. Make
         --  sure we register the potential foreign Node.Self_Env environment
         --  at the end, even when an exception interrupts PLE to keep the
         --  state consistent.
         begin
            Pre_Env_Actions (Node, State);
            if State.Current_Env /= Null_Lexical_Env then
               Node.Self_Env := State.Current_Env;
               Register_Foreign_Env (Node, State);
            end if;
            --  Call recursively on children
            for I in First_Child_Index (Node) .. Last_Child_Index (Node) loop
               Result := Populate_Internal
                 (Child (Node, I), State) or else Result;
            end loop;
            Post_Env_Actions (Node, State);
         exception
            when Exc : Precondition_Failure | Property_Error =>
               if PLE_Errors_Trace.Is_Active then
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      "Exception raised during PLE "
                      & Ada.Exceptions.Exception_Name (Exc) & " : "
                      & Ada.Exceptions.Exception_Message (Exc));
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      GNAT.Traceback.Symbolic.Symbolic_Traceback (Exc));
               end if;
               Register_Foreign_Env (Node, State);
               return True;
         end;
         return Result;
      end Populate_Internal;
      --------------------------
      -- Register_Foreign_Env --
      --------------------------
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State) is
      begin
         if State.Current_NED /= null then
            State.Current_NED.Nodes_With_Foreign_Env.Insert (Node);
            Node.Unit.Nodes_With_Foreign_Env.Insert (Node, State.Current_NED);
         end if;
      end Register_Foreign_Env;
   begin
      --  This is intended to be called on the root node only (when there is no
      --  PLE root) or on a PLE root (child of the root node with a specific
      --  kind).
      if
         Node.Parent /= null
            and then Node.Kind /= Ada_Compilation_Unit
      then
         raise Program_Error;
      end if;
      return Result : constant Boolean :=
         Populate_Internal (Node, Root_State)
      do
         Update_Named_Envs (Context, Unit_State.Named_Envs_Needing_Update);
      end return;
   end Populate_Lexical_Env;
   ------------------------------
   -- AST_Envs_Node_Text_Image --
   ------------------------------
   function AST_Envs_Node_Text_Image
     (Node  : Bare_Ada_Node;
      Short : Boolean := True) return Text_Type is
   begin
      if Short then
         if Node = null then
            return "null";
         end if;
         return To_Text (Basename (Node.Unit))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node))));
      else
         return Short_Text_Image (Node);
      end if;
   end AST_Envs_Node_Text_Image;
   -------------------
   -- Is_Rebindable --
   -------------------
   function Is_Rebindable (Node : Bare_Ada_Node) return Boolean is
   begin
         return Node.Kind in Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl;
   end Is_Rebindable;
   -----------------------
   -- Acquire_Rebinding --
   -----------------------
   function Acquire_Rebinding
     (Node             : Bare_Ada_Node;
      Parent           : Env_Rebindings;
      Old_Env, New_Env : Lexical_Env) return Env_Rebindings
   is
      Result    : Env_Rebindings;
      Available : Env_Rebindings_Vectors.Vector renames
         Node.Unit.Context.Available_Rebindings;
   begin
      --  Use an existing and available Env_Rebindings_Type record for Node's
      --  Context, otherwise allocate a new rebinding.
      Result := (if Available.Is_Empty
                 then new Env_Rebindings_Type'(Version => 0, others => <>)
                 else Available.Pop);
      Result.Parent := Parent;
      Result.Old_Env := Old_Env;
      Result.New_Env := New_Env;
      Result.Children := Env_Rebindings_Vectors.Empty_Vector;
      return Result;
   end Acquire_Rebinding;
   -----------------------
   -- Release_Rebinding --
   -----------------------
   procedure Release_Rebinding (Self : in out Env_Rebindings) is
      Available : Env_Rebindings_Vectors.Vector renames
         Unwrap (Self.Old_Env).Node.Unit.Context.Available_Rebindings;
   begin
      --  Bumping the version number, to invalidate existing references to
      --  Self.
      Self.Version := Self.Version + 1;
      Self.Children.Destroy;
      Available.Append (Self);
      Self := null;
   end Release_Rebinding;
   ------------------------
   -- Register_Rebinding --
   ------------------------
   procedure Register_Rebinding
     (Node : Bare_Ada_Node; Rebinding : Env_Rebindings) is
   begin
      Node.Unit.Rebindings.Append (Rebinding);
   end Register_Rebinding;
      -------------------------------
      -- Lexical_Env_Cache_Updated --
      -------------------------------
      procedure Lexical_Env_Cache_Updated
        (Node         : Bare_Ada_Node;
         Delta_Amount : Long_Long_Integer)
      is
         Ctx : constant Internal_Context := Node.Unit.Context;
         Ctx_Stats : Context_Env_Caches_Stats renames Ctx.Env_Caches_Stats;
         All_Env_Caches_Entry_Count : constant Long_Long_Integer :=
           Ctx_Stats.Entry_Count;
         --  Snapshot of the total number of entries before any collection is
         --  attempted. This is needed because if a collection happens,
         --  ``Ctx.Env_Caches_Stats.Entry_Count`` will be updated on the go.
      begin
         Ctx_Stats.Entry_Count :=
           All_Env_Caches_Entry_Count + Delta_Amount;
         Node.Unit.Env_Caches_Stats.Entry_Count :=
           Node.Unit.Env_Caches_Stats.Entry_Count + Delta_Amount;
         --  If the number of entries exceeds the threshold we had set, attempt
         --  to invalidate caches. Don't do anything if this notification was
         --  for removed entries (i.e. if ``Delta_Amount`` is negative) as it
         --  could mean that we are already in the middle of a collection.
         if Delta_Amount > 0 and then
            Ctx_Stats.Entry_Count > Ctx.Env_Caches_Collection_Threshold
         then
            if Cache_Invalidation_Trace.Is_Active then
               Cache_Invalidation_Trace.Trace
                 ("Attempting cache collection because number of entries"
                  & " reached" & All_Env_Caches_Entry_Count'Image);
               Cache_Invalidation_Trace.Increase_Indent;
            end if;
            for Unit of Ctx.Units loop
               if Libadalang.Implementation.Extensions.Should_Collect_Env_Caches
                 (Ctx, Unit, All_Env_Caches_Entry_Count)
               then
                  --  Clear all caches and set counters that are meant to
                  --  track events since the unit's last collection.
                  Reset_Envs_Caches (Unit);
                  Unit.Env_Caches_Stats.Lookup_Count := 0;
                  Unit.Env_Caches_Stats.Hit_Count := 0;
                  Unit.Env_Caches_Stats.Last_Overall_Lookup_Count :=
                    Ctx_Stats.Lookup_Count;
               end if;
               Unit.Env_Caches_Stats.Previous_Lookup_Count :=
                 Unit.Env_Caches_Stats.Lookup_Count;
            end loop;
            Ctx_Stats.Previous_Lookup_Count := Ctx_Stats.Lookup_Count;
            --  Setup the threshold so that the next collection is triggered
            --  when we reach the current number of entries (after this
            --  collection) plus the increment that is defined for this
            --  language.
            Ctx.Env_Caches_Collection_Threshold :=
              Ctx_Stats.Entry_Count
              + 100000;
            if Cache_Invalidation_Trace.Is_Active then
               Cache_Invalidation_Trace.Trace
                 ("New collection threshold :"
                  & Ctx.Env_Caches_Collection_Threshold'Image);
               Cache_Invalidation_Trace.Decrease_Indent;
            end if;
         end if;
      end Lexical_Env_Cache_Updated;
      ---------------------------------
      -- Lexical_Env_Cache_Looked_Up --
      ---------------------------------
      procedure Lexical_Env_Cache_Looked_Up (Node : Bare_Ada_Node) is
         Unit : constant Internal_Unit := Node.Unit;
         Ctx  : constant Internal_Context := Unit.Context;
      begin
         Unit.Env_Caches_Stats.Lookup_Count :=
            Unit.Env_Caches_Stats.Lookup_Count + 1;
         Ctx.Env_Caches_Stats.Lookup_Count :=
            Ctx.Env_Caches_Stats.Lookup_Count + 1;
      end Lexical_Env_Cache_Looked_Up;
      ---------------------------
      -- Lexical_Env_Cache_Hit --
      ---------------------------
      procedure Lexical_Env_Cache_Hit (Node : Bare_Ada_Node) is
         Unit : constant Internal_Unit := Node.Unit;
      begin
         Unit.Env_Caches_Stats.Hit_Count :=
            Unit.Env_Caches_Stats.Hit_Count + 1;
      end Lexical_Env_Cache_Hit;
   --------------------
   -- Element_Parent --
   --------------------
   function Element_Parent
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is (Node.Parent);
   ---------------
   -- Node_Unit --
   ---------------
   function Node_Unit (Node : Bare_Ada_Node) return Generic_Unit_Ptr is
   begin
      return Convert_Unit (Node.Unit);
   end Node_Unit;
   ----------
   -- Hash --
   ----------
   function Hash (Node : Bare_Ada_Node) return Hash_Type
   is
      function H is new Hash_Access
        (Root_Node_Record, Bare_Ada_Node);
   begin
      return H (Node);
   end Hash;
      function Hash (B : Boolean) return Hash_Type is (Boolean'Pos (B));
      function Hash (I : Integer) return Hash_Type is (Hash_Type'Mod (I));
   ------------------------
   -- Named environments --
   ------------------------
   ---------
   -- Add --
   ---------
   procedure Add
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : AST_Envs.Internal_Map_Node)
   is
      use NED_Assoc_Maps;
      Pos   : Cursor;
      Dummy : Boolean;
   begin
      --  Make sure there is a vector entry for Key
      Self.Insert (Key, Internal_Map_Node_Vectors.Empty_Vector, Pos, Dummy);
      --  Append Node to that vector
      declare
         V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Pos);
      begin
         V.Append (Node);
      end;
   end Add;
   ------------
   -- Remove --
   ------------
   procedure Remove
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : Bare_Ada_Node)
   is
      use NED_Assoc_Maps;
      V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Key);
   begin
      --  Remove the (assumed unique) entry in V whose node is Node. The order
      --  of items in V is not significant, so we can use Pop for efficient
      --  removal. Do the traversal in reverse order for correctness.
      for I in reverse 1 .. V.Length loop
         if V.Get_Access (I).Node = Node then
            V.Pop (I);
            exit;
         end if;
      end loop;
   end Remove;
   ------------------------------
   -- Get_Named_Env_Descriptor --
   ------------------------------
   function Get_Named_Env_Descriptor
     (Context : Internal_Context;
      Name    : Symbol_Type) return Named_Env_Descriptor_Access
   is
      use NED_Maps;
      --  Look for an existing entry for Name
      Pos : constant Cursor := Context.Named_Envs.Find (Name);
   begin
      if Has_Element (Pos) then
         return Element (Pos);
      end if;
      --  There is no such entry: create one
      return Result : constant Named_Env_Descriptor_Access :=
         new Named_Env_Descriptor'
           (Name                   => Name,
            Envs                   => <>,
            Env_With_Precedence    => Empty_Env,
            Foreign_Nodes          => <>,
            Foreign_Envs           => <>,
            Nodes_With_Foreign_Env => <>)
      do
         Context.Named_Envs.Insert (Name, Result);
      end return;
   end Get_Named_Env_Descriptor;
   ------------------------
   -- Register_Named_Env --
   ------------------------
   procedure Register_Named_Env
     (Context                   : Internal_Context;
      Name                      : Symbol_Type;
      Env                       : Lexical_Env;
      Named_Envs_Needing_Update : in out NED_Maps.Map)
   is
      NED_Access : constant Named_Env_Descriptor_Access :=
         Get_Named_Env_Descriptor (Context, Name);
      NED        : Named_Env_Descriptor renames NED_Access.all;
      Node       : constant Bare_Ada_Node := Env_Node (Env);
   begin
      NED.Envs.Insert (Node, Env);
      Node.Unit.Named_Envs.Append ((Name, Env));
      --  If that insertion must change the env that has precedence, signal
      --  that NED requires an update.
      if NED.Envs.First_Element /= NED.Env_With_Precedence then
         Named_Envs_Needing_Update.Include (Name, NED_Access);
      end if;
   end Register_Named_Env;
   ----------------------
   -- Update_Named_Env --
   ----------------------
   procedure Update_Named_Envs
     (Context : Internal_Context; Named_Envs : NED_Maps.Map)
   is
      Require_Cache_Reset : Boolean := False;
   begin
      for Cur in Named_Envs.Iterate loop
         declare
            NE      : Named_Env_Descriptor renames NED_Maps.Element (Cur).all;
            New_Env : constant Lexical_Env :=
              (if NE.Envs.Is_Empty
               then Empty_Env
               else NE.Envs.First_Element);
         begin
            --  If there was an environment with precedence, remove its foreign
            --  nodes.
            if NE.Env_With_Precedence /= Empty_Env then
               for Cur in NE.Foreign_Nodes.Iterate loop
                  declare
                     Key   : constant Symbol_Type :=
                        NED_Assoc_Maps.Key (Cur);
                     Nodes : Internal_Map_Node_Vectors.Vector renames
                        NE.Foreign_Nodes.Reference (Cur);
                  begin
                     for N of Nodes loop
                        Remove (NE.Env_With_Precedence, Thin (Key), N.Node);
                     end loop;
                  end;
               end loop;
            end if;
            --  Now, set the new environment that has precedence
            NE.Env_With_Precedence := New_Env;
            --  Add the foreign nodes to the new environment with precedence,
            --  if any.
            for Cur in NE.Foreign_Nodes.Iterate loop
               declare
                  Key   : constant Symbol_Type :=
                     NED_Assoc_Maps.Key (Cur);
                  Nodes : Internal_Map_Node_Vectors.Vector renames
                     NE.Foreign_Nodes.Reference (Cur);
               begin
                  for N of Nodes loop
                     Add (New_Env, Thin (Key), N.Node, N.Md, N.Resolver);
                  end loop;
               end;
            end loop;
            --  Set the parent environment of all foreign environments
            for Cur in NE.Foreign_Envs.Iterate loop
               declare
                  Env : Lexical_Env_Record renames
                     Unwrap (Sorted_Env_Maps.Element (Cur)).all;
               begin
                  Env.Parent := New_Env;
                  --  We have updated the lexical env hierarchy (e.g. an env
                  --  which had no parent may have one now), so the cached
                  --  entries for queries that traveresed the old env hierarchy
                  --  need to be invalidated.
                  Require_Cache_Reset := True;
               end;
            end loop;
            --  Update nodes whose environment was the old env with precedence
            for N of NE.Nodes_With_Foreign_Env loop
               N.Self_Env := New_Env;
            end loop;
         end;
      end loop;
      if Require_Cache_Reset then
         Invalidate_Caches (Context, Invalidate_Envs => True);
      end if;
   end Update_Named_Envs;
   --------------------------
   -- Big integers wrapper --
   --------------------------
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Image : String; Base : Integer := 10) return Big_Integer_Type
   is
      use GNATCOLL.GMP;
      use GNATCOLL.GMP.Integers;
   begin
      return new Big_Integer_Record'(Value     => Make (Image, Int (Base)),
                                     Ref_Count => 1);
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Big_Int : GNATCOLL.GMP.Integers.Big_Integer) return Big_Integer_Type
   is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (Big_Int);
      return Result;
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer (Int : Integer) return Big_Integer_Type is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (GNATCOLL.GMP.Long (Int));
      return Result;
   end Create_Big_Integer;
   -------------------------------
   -- Create_Public_Big_Integer --
   -------------------------------
   function Create_Public_Big_Integer
     (Big_Int : Big_Integer_Type) return GNATCOLL.GMP.Integers.Big_Integer is
   begin
      return Result : GNATCOLL.GMP.Integers.Big_Integer do
         Result.Set (Big_Int.Value);
      end return;
   end Create_Public_Big_Integer;
   -----------------
   -- Trace_Image --
   -----------------
   function Trace_Image (I : Big_Integer_Type) return String is
   begin
      return GNATCOLL.GMP.Integers.Image (I.Value);
   end Trace_Image;
   ----------------
   -- To_Integer --
   ----------------
   function To_Integer
     (Self    : Bare_Ada_Node;
      Big_Int : Big_Integer_Type) return Integer
   is
      Image : constant String := Big_Int.Value.Image;
   begin
      return Integer'Value (Image);
   exception
      when Constraint_Error =>
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out of range big integer");
   end To_Integer;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Big_Int : Big_Integer_Type) is
   begin
      if Big_Int.Ref_Count /= -1 then
         Big_Int.Ref_Count := Big_Int.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Big_Int : in out Big_Integer_Type) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Big_Integer_Record, Big_Integer_Type);
   begin
      if Big_Int = null or else Big_Int.Ref_Count = -1 then
         return;
      end if;
      Big_Int.Ref_Count := Big_Int.Ref_Count - 1;
      if Big_Int.Ref_Count = 0 then
         Destroy (Big_Int);
      end if;
   end Dec_Ref;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value = Right.Value;
   end Equivalent;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value < Right.Value;
   end "<";
   ----------
   -- "<=" --
   ----------
   function "<=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value <= Right.Value;
   end "<=";
   ---------
   -- ">" --
   ---------
   function ">" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value > Right.Value;
   end ">";
   ----------
   -- ">=" --
   ----------
   function ">=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value >= Right.Value;
   end ">=";
   ---------
   -- "+" --
   ---------
   function "+" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value + Right.Value);
   end "+";
   ---------
   -- "-" --
   ---------
   function "-" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value - Right.Value);
   end "-";
   ---------
   -- "-" --
   ---------
   function "-" (Value : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (-Value.Value);
   end "-";
   ------------------
   -- Unit_Version --
   ------------------
   function Unit_Version (Unit : Generic_Unit_Ptr) return Version_Number is
   begin
      return Convert_Unit (Unit).Unit_Version;
   end Unit_Version;
   -------------------------
   -- Get_Context_Version --
   -------------------------
   function Get_Context_Version
     (Node : Bare_Ada_Node) return Version_Number is
   begin
      return Node.Unit.Context.Cache_Version;
   end Get_Context_Version;
   ---------------
   --  Self_Env --
   ---------------
   function Self_Env (Node : Bare_Ada_Node) return Lexical_Env is
   begin
      return Node.Self_Env;
   end Self_Env;
   --------------------------
   -- Properties_May_Raise --
   --------------------------
   function Properties_May_Raise
     (Exc : Ada.Exceptions.Exception_Occurrence) return Boolean is
   begin
      return Ada.Exceptions.Exception_Identity (Exc) in
            Precondition_Failure'Identity
               |
            Property_Error'Identity
      ;
   end Properties_May_Raise;
   ----------------------
   -- Short_Text_Image --
   ----------------------
   function Short_Text_Image (Self : Bare_Ada_Node) return Text_Type
   is
   begin
      if Self = null then
         return "None";
      end if;
   case Self.Kind is
            when Ada_Char_Literal .. Ada_String_Literal =>
         return Base_Id_Short_Image (Self);
            when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
         return Defining_Name_Short_Image (Self);
            when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
         return Basic_Decl_Short_Image (Self);
      when others => 
         return "<" & To_Text (Kind_Name (Self))
                & " "
                & To_Text
                  (Ada.Directories.Simple_Name
                     (Get_Filename (Unit (Self))))
                & ":" & To_Text (Image (Sloc_Range (Self))) & ">";
   end case;
   end Short_Text_Image;
   --------------------
   -- Snaps_At_Start --
   --------------------
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return False;
   end case;
   end Snaps_At_Start;
   ------------------
   -- Snaps_At_End --
   ------------------
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return Is_Incomplete (Self);
   end case;
   end Snaps_At_End;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True)
      return Bare_Ada_Node_Array_Access
   is
      Count : Natural := 0;
      Start : Bare_Ada_Node :=
        (if With_Self then Node else Node.Parent);
      Cur   : Bare_Ada_Node := Start;
   begin
      while Cur /= null loop
         Count := Count + 1;
         Cur := Cur.Parent;
      end loop;
      declare
         Result : constant Bare_Ada_Node_Array_Access :=
            Create_Bare_Ada_Node_Array (Count);
      begin
         Cur := Start;
         for I in Result.Items'Range loop
            Result.Items (I) := Cur;
            Cur := Cur.Parent;
         end loop;
         return Result;
      end;
   end Parents;
   -----------------------
   -- First_Child_Index --
   -----------------------
   function First_Child_Index (Node : Bare_Ada_Node) return Natural
   is (1);
   ----------------------
   -- Last_Child_Index --
   ----------------------
   function Last_Child_Index (Node : Bare_Ada_Node) return Natural
   is (Children_Count (Node));
   ---------------
   -- Get_Child --
   ---------------
   procedure Get_Child
     (Node            : Bare_Ada_Node;
      Index           : Positive;
      Index_In_Bounds : out Boolean;
      Result          : out Bare_Ada_Node)
   is
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      Index_In_Bounds := True;
      Result := null;
      case Ada_Ada_Node (K) is
when Ada_Ada_List =>
declare
N_Bare_Ada_List : constant Bare_Ada_List := Node;
begin
                    if Index > N_Bare_Ada_List.Count then
                        Index_In_Bounds := False;
                    else
                        Result := N_Bare_Ada_List.Nodes (Index);
                    end if;
                    return;
end;
when Ada_Constrained_Array_Indices_Range =>
declare
N_Bare_Constrained_Array_Indices : constant Bare_Constrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Constrained_Array_Indices.Constrained_Array_Indices_F_List;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Indices_Range =>
declare
N_Bare_Unconstrained_Array_Indices : constant Bare_Unconstrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Indices.Unconstrained_Array_Indices_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Assoc_Range =>
declare
N_Bare_Aspect_Assoc : constant Bare_Aspect_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_At_Clause_Range =>
declare
N_Bare_At_Clause : constant Bare_At_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Def_Clause_Range =>
declare
N_Bare_Attribute_Def_Clause : constant Bare_Attribute_Def_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Attribute_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Rep_Clause_Range =>
declare
N_Bare_Enum_Rep_Clause : constant Bare_Enum_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Type_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Aggregate;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Rep_Clause_Range =>
declare
N_Bare_Record_Rep_Clause : constant Bare_Record_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_At_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Spec_Range =>
declare
N_Bare_Aspect_Spec : constant Bare_Aspect_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Spec.Aspect_Spec_F_Aspect_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Case_Assoc_Range =>
declare
N_Bare_Contract_Case_Assoc : constant Bare_Contract_Case_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Guard;
                            return;
                        when 2 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Consequence;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Argument_Assoc_Range =>
declare
N_Bare_Pragma_Argument_Assoc : constant Bare_Pragma_Argument_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Spec_Range =>
declare
N_Bare_Entry_Spec : constant Bare_Entry_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Family_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Spec_Range =>
declare
N_Bare_Subp_Spec : constant Bare_Subp_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Returns;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Binary_Spec_Range =>
declare
N_Bare_Synthetic_Binary_Spec : constant Bare_Synthetic_Binary_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Left_Param;
                            return;
                        when 2 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Right_Param;
                            return;
                        when 3 =>
                            Result := N_Bare_Synthetic_Binary_Spec.Synthetic_Binary_Spec_F_Return_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Unary_Spec_Range =>
declare
N_Bare_Synthetic_Unary_Spec : constant Bare_Synthetic_Unary_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Unary_Spec.Synthetic_Unary_Spec_F_Right_Param;
                            return;
                        when 2 =>
                            Result := N_Bare_Synthetic_Unary_Spec.Synthetic_Unary_Spec_F_Return_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_List_Range =>
declare
N_Bare_Component_List : constant Bare_Component_List := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_List.Component_List_F_Components;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_List.Component_List_F_Variant_Part;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Known_Discriminant_Part_Range =>
declare
N_Bare_Known_Discriminant_Part : constant Bare_Known_Discriminant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Known_Discriminant_Part.Known_Discriminant_Part_F_Discr_Specs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Completion_Formal_Params_Range =>
declare
N_Bare_Entry_Completion_Formal_Params : constant Bare_Entry_Completion_Formal_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Completion_Formal_Params.Entry_Completion_Formal_Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal_Part_Range =>
declare
N_Bare_Generic_Formal_Part : constant Bare_Generic_Formal_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal_Part.Generic_Formal_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Record_Def =>
declare
N_Bare_Base_Record_Def : constant Bare_Base_Record_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Record_Def.Base_Record_Def_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aggregate_Assoc_Range =>
declare
N_Bare_Aggregate_Assoc : constant Bare_Aggregate_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_Designators;
                            return;
                        when 2 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Composite_Constraint_Assoc_Range =>
declare
N_Bare_Composite_Constraint_Assoc : constant Bare_Composite_Constraint_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Composite_Constraint_Assoc.Composite_Constraint_Assoc_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Composite_Constraint_Assoc.Composite_Constraint_Assoc_F_Constraint_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Iterated_Assoc_Range =>
declare
N_Bare_Iterated_Assoc : constant Bare_Iterated_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_Key_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Assoc_Range =>
declare
N_Bare_Param_Assoc : constant Bare_Param_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_Designator;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Range =>
declare
N_Bare_Abstract_State_Decl : constant Bare_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Expr_Decl_Range =>
declare
N_Bare_Anonymous_Expr_Decl : constant Bare_Anonymous_Expr_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Expr_Decl.Anonymous_Expr_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Decl_Range =>
declare
N_Bare_Component_Decl : constant Bare_Component_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Component_Def;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Spec_Range =>
declare
N_Bare_Discriminant_Spec : constant Bare_Discriminant_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Type_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal =>
declare
N_Bare_Generic_Formal : constant Bare_Generic_Formal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal.Generic_Formal_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Spec_Range =>
declare
N_Bare_Param_Spec : constant Bare_Param_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Mode;
                            return;
                        when 4 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Type_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Default_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Formal_Param_Decl_Range =>
declare
N_Bare_Synthetic_Formal_Param_Decl : constant Bare_Synthetic_Formal_Param_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Formal_Param_Decl.Synthetic_Formal_Param_Decl_F_Param_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Package_Decl =>
declare
N_Bare_Base_Package_Decl : constant Bare_Base_Package_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Public_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Private_Part;
                            return;
                        when 5 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Type_Decl.Base_Type_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Type_Decl (K) is
when Ada_Subtype_Decl_Range =>
declare
N_Bare_Subtype_Decl : constant Bare_Subtype_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Type_Decl_Range =>
declare
N_Bare_Incomplete_Type_Decl : constant Bare_Incomplete_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Incomplete_Type_Decl.Incomplete_Type_Decl_F_Discriminants;
                            return;
                        when others => null;
                    end case;
case Ada_Incomplete_Type_Decl_Range (K) is
when Ada_Incomplete_Formal_Type_Decl_Range =>
declare
N_Bare_Incomplete_Formal_Type_Decl : constant Bare_Incomplete_Formal_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Formal_Type_Decl.Incomplete_Formal_Type_Decl_F_Is_Tagged;
                            return;
                        when 4 =>
                            Result := N_Bare_Incomplete_Formal_Type_Decl.Incomplete_Formal_Type_Decl_F_Default_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Tagged_Type_Decl_Range =>
declare
N_Bare_Incomplete_Tagged_Type_Decl : constant Bare_Incomplete_Tagged_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Tagged_Type_Decl.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Protected_Type_Decl_Range =>
declare
N_Bare_Protected_Type_Decl : constant Bare_Protected_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Interfaces;
                            return;
                        when 5 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Type_Decl_Range =>
declare
N_Bare_Task_Type_Decl : constant Bare_Task_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Decl =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Type_Def;
                            return;
                        when others => null;
                    end case;
case Ada_Type_Decl (K) is
when Ada_Concrete_Type_Decl_Range =>
declare
N_Bare_Concrete_Type_Decl : constant Bare_Concrete_Type_Decl := N_Bare_Type_Decl;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Concrete_Type_Decl.Concrete_Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Type_Decl_Range =>
declare
N_Bare_Formal_Type_Decl : constant Bare_Formal_Type_Decl := N_Bare_Type_Decl;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Formal_Type_Decl.Formal_Type_Decl_F_Default_Type;
                            return;
                        when 5 =>
                            Result := N_Bare_Formal_Type_Decl.Formal_Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when others => null;
end case;
end;
when Ada_Classic_Subp_Decl =>
declare
N_Bare_Classic_Subp_Decl : constant Bare_Classic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Classic_Subp_Decl (K) is
when Ada_Abstract_Subp_Decl_Range =>
declare
N_Bare_Abstract_Subp_Decl : constant Bare_Abstract_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Abstract_Subp_Decl.Abstract_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Subp_Decl =>
declare
N_Bare_Formal_Subp_Decl : constant Bare_Formal_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Decl_Range =>
declare
N_Bare_Subp_Decl : constant Bare_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Decl.Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Entry_Decl_Range =>
declare
N_Bare_Entry_Decl : constant Bare_Entry_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Literal_Decl_Range =>
declare
N_Bare_Enum_Literal_Decl : constant Bare_Enum_Literal_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Literal_Decl.Enum_Literal_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Internal_Range =>
declare
N_Bare_Generic_Subp_Internal : constant Bare_Generic_Subp_Internal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Subp_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Subp_Decl_Range =>
declare
N_Bare_Synthetic_Subp_Decl : constant Bare_Synthetic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Subp_Decl.Synthetic_Subp_Decl_F_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Body_Range =>
declare
N_Bare_Accept_Stmt_Body : constant Bare_Accept_Stmt_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt_Body.Accept_Stmt_Body_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Subp_Body =>
declare
N_Bare_Base_Subp_Body : constant Bare_Base_Subp_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Subp_Body (K) is
when Ada_Expr_Function_Range =>
declare
N_Bare_Expr_Function : constant Bare_Expr_Function := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Null_Subp_Decl_Range =>
declare
N_Bare_Null_Subp_Decl : constant Bare_Null_Subp_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Null_Subp_Decl.Null_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Range =>
declare
N_Bare_Subp_Body : constant Bare_Subp_Body := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Decls;
                            return;
                        when 5 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Stmts;
                            return;
                        when 6 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Renaming_Decl_Range =>
declare
N_Bare_Subp_Renaming_Decl : constant Bare_Subp_Renaming_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Package_Body_Stub_Range =>
declare
N_Bare_Package_Body_Stub : constant Bare_Package_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Stub_Range =>
declare
N_Bare_Protected_Body_Stub : constant Bare_Protected_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Stub_Range =>
declare
N_Bare_Subp_Body_Stub : constant Bare_Subp_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Subp_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Stub_Range =>
declare
N_Bare_Task_Body_Stub : constant Bare_Task_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Body_Range =>
declare
N_Bare_Entry_Body : constant Bare_Entry_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Index_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Aspects;
                            return;
                        when 5 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Barrier;
                            return;
                        when 6 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Decls;
                            return;
                        when 7 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Stmts;
                            return;
                        when 8 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Body_Range =>
declare
N_Bare_Package_Body : constant Bare_Package_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Package_Body.Package_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Range =>
declare
N_Bare_Protected_Body : constant Bare_Protected_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Range =>
declare
N_Bare_Task_Body : constant Bare_Task_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Task_Body.Task_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Index_Spec_Range =>
declare
N_Bare_Entry_Index_Spec : constant Bare_Entry_Index_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Decl_Range =>
declare
N_Bare_Exception_Decl : constant Bare_Exception_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Handler_Range =>
declare
N_Bare_Exception_Handler : constant Bare_Exception_Handler := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Handled_Exceptions;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Var_Decl_Range =>
declare
N_Bare_For_Loop_Var_Decl : constant Bare_For_Loop_Var_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Decl =>
declare
N_Bare_Generic_Decl : constant Bare_Generic_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Formal_Part;
                            return;
                        when others => null;
                    end case;
case Ada_Generic_Decl (K) is
when Ada_Generic_Package_Decl_Range =>
declare
N_Bare_Generic_Package_Decl : constant Bare_Generic_Package_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Package_Decl.Generic_Package_Decl_F_Package_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Decl_Range =>
declare
N_Bare_Generic_Subp_Decl : constant Bare_Generic_Subp_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Decl.Generic_Subp_Decl_F_Subp_Decl;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Generic_Package_Instantiation_Range =>
declare
N_Bare_Generic_Package_Instantiation : constant Bare_Generic_Package_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Generic_Pkg_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Instantiation_Range =>
declare
N_Bare_Generic_Subp_Instantiation : constant Bare_Generic_Subp_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Kind;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Subp_Name;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Generic_Subp_Name;
                            return;
                        when 5 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Params;
                            return;
                        when 6 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Renaming_Decl_Range =>
declare
N_Bare_Generic_Package_Renaming_Decl : constant Bare_Generic_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Renaming_Decl_Range =>
declare
N_Bare_Generic_Subp_Renaming_Decl : constant Bare_Generic_Subp_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Decl_Range =>
declare
N_Bare_Label_Decl : constant Bare_Label_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label_Decl.Label_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Decl_Range =>
declare
N_Bare_Named_Stmt_Decl : constant Bare_Named_Stmt_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt_Decl.Named_Stmt_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Number_Decl_Range =>
declare
N_Bare_Number_Decl : constant Bare_Number_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Object_Decl_Range =>
declare
N_Bare_Object_Decl : constant Bare_Object_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Mode;
                            return;
                        when 5 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Type_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Default_Expr;
                            return;
                        when 7 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Renaming_Clause;
                            return;
                        when 8 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Renaming_Decl_Range =>
declare
N_Bare_Package_Renaming_Decl : constant Bare_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Protected_Decl_Range =>
declare
N_Bare_Single_Protected_Decl : constant Bare_Single_Protected_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Interfaces;
                            return;
                        when 4 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Task_Decl_Range =>
declare
N_Bare_Single_Task_Decl : constant Bare_Single_Task_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Task_Decl.Single_Task_Decl_F_Task_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Alternative_Range =>
declare
N_Bare_Case_Stmt_Alternative : constant Bare_Case_Stmt_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Prelude;
                            return;
                        when 2 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Body;
                            return;
                        when 3 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Pragmas;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Clause_Range =>
declare
N_Bare_Component_Clause : constant Bare_Component_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Position;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Def_Range =>
declare
N_Bare_Component_Def : constant Bare_Component_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Aliased;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Constant;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Composite_Constraint_Range =>
declare
N_Bare_Composite_Constraint : constant Bare_Composite_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Composite_Constraint.Composite_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delta_Constraint_Range =>
declare
N_Bare_Delta_Constraint : constant Bare_Delta_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Digits_Constraint_Range =>
declare
N_Bare_Digits_Constraint : constant Bare_Digits_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Constraint_Range =>
declare
N_Bare_Range_Constraint : constant Bare_Range_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Constraint.Range_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Declarative_Part_Range =>
declare
N_Bare_Declarative_Part : constant Bare_Declarative_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Declarative_Part.Declarative_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Expr_Part_Range =>
declare
N_Bare_Elsif_Expr_Part : constant Bare_Elsif_Expr_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Then_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Stmt_Part_Range =>
declare
N_Bare_Elsif_Stmt_Part : constant Bare_Elsif_Stmt_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Expr_Range =>
declare
N_Bare_Abstract_State_Decl_Expr : constant Bare_Abstract_State_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl_Expr.Abstract_State_Decl_Expr_F_State_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Allocator_Range =>
declare
N_Bare_Allocator : constant Bare_Allocator := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Allocator.Allocator_F_Subpool;
                            return;
                        when 2 =>
                            Result := N_Bare_Allocator.Allocator_F_Type_Or_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Aggregate =>
declare
N_Bare_Base_Aggregate : constant Bare_Base_Aggregate := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Ancestor_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Bin_Op_Range =>
declare
N_Bare_Bin_Op : constant Bare_Bin_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Left;
                            return;
                        when 2 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Right;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Alternative_Range =>
declare
N_Bare_Case_Expr_Alternative : constant Bare_Case_Expr_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Concat_Op_Range =>
declare
N_Bare_Concat_Op : constant Bare_Concat_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Concat_Op.Concat_Op_F_First_Operand;
                            return;
                        when 2 =>
                            Result := N_Bare_Concat_Op.Concat_Op_F_Other_Operands;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Concat_Operand_Range =>
declare
N_Bare_Concat_Operand : constant Bare_Concat_Operand := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Concat_Operand.Concat_Operand_F_Operator;
                            return;
                        when 2 =>
                            Result := N_Bare_Concat_Operand.Concat_Operand_F_Operand;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Range =>
declare
N_Bare_Case_Expr : constant Bare_Case_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Expr_Range =>
declare
N_Bare_If_Expr : constant Bare_If_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Then_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Else_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Cases_Range =>
declare
N_Bare_Contract_Cases : constant Bare_Contract_Cases := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Cases.Contract_Cases_F_Contract_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Expr_Range =>
declare
N_Bare_Decl_Expr : constant Bare_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Format_String_Literal_Range =>
declare
N_Bare_Format_String_Literal : constant Bare_Format_String_Literal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Format_String_Literal.Format_String_Literal_F_Opening_Chunk;
                            return;
                        when 2 =>
                            Result := N_Bare_Format_String_Literal.Format_String_Literal_F_Mid_Exprs;
                            return;
                        when 3 =>
                            Result := N_Bare_Format_String_Literal.Format_String_Literal_F_Trailing_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Membership_Expr_Range =>
declare
N_Bare_Membership_Expr : constant Bare_Membership_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Membership_Exprs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Array_Subcomponent_Choice_Name_Range =>
declare
N_Bare_Array_Subcomponent_Choice_Name : constant Bare_Array_Subcomponent_Choice_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Array_Subcomponent_Choice_Name.Array_Subcomponent_Choice_Name_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Array_Subcomponent_Choice_Name.Array_Subcomponent_Choice_Name_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Defining_Name_Range =>
declare
N_Bare_Defining_Name : constant Bare_Defining_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Defining_Name.Defining_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discrete_Subtype_Name_Range =>
declare
N_Bare_Discrete_Subtype_Name : constant Bare_Discrete_Subtype_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discrete_Subtype_Name.Discrete_Subtype_Name_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Dotted_Name_Range =>
declare
N_Bare_Dotted_Name : constant Bare_Dotted_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_End_Name_Range =>
declare
N_Bare_End_Name : constant Bare_End_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_End_Name.End_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Explicit_Deref.Explicit_Deref_F_Prefix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Qual_Expr_Range =>
declare
N_Bare_Qual_Expr : constant Bare_Qual_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_F_Values;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Expr_Range =>
declare
N_Bare_Paren_Expr : constant Bare_Paren_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Expr.Paren_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Quantified_Expr_Range =>
declare
N_Bare_Quantified_Expr : constant Bare_Quantified_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Quantifier;
                            return;
                        when 2 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Loop_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Expr_Range =>
declare
N_Bare_Raise_Expr : constant Bare_Raise_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Un_Op_Range =>
declare
N_Bare_Un_Op : constant Bare_Un_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Op;
                            return;
                        when 2 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Iter_Filter_Range =>
declare
N_Bare_For_Loop_Iter_Filter : constant Bare_For_Loop_Iter_Filter := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Iter_Filter.For_Loop_Iter_Filter_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Format_String_Chunk_Range =>
declare
N_Bare_Format_String_Chunk : constant Bare_Format_String_Chunk := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Format_String_Chunk.Format_String_Chunk_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Format_String_Chunk.Format_String_Chunk_F_String_Tok;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Handled_Stmts_Range =>
declare
N_Bare_Handled_Stmts : constant Bare_Handled_Stmts := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Exceptions;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Library_Item_Range =>
declare
N_Bare_Library_Item : constant Bare_Library_Item := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Has_Private;
                            return;
                        when 2 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Item;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Spec_Range =>
declare
N_Bare_For_Loop_Spec : constant Bare_For_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Var_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Loop_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Has_Reverse;
                            return;
                        when 4 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Filter;
                            return;
                        when others => null;
                    end case;
end;
when Ada_While_Loop_Spec_Range =>
declare
N_Bare_While_Loop_Spec : constant Bare_While_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_While_Loop_Spec.While_Loop_Spec_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Multi_Abstract_State_Decl_Range =>
declare
N_Bare_Multi_Abstract_State_Decl : constant Bare_Multi_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Multi_Abstract_State_Decl.Multi_Abstract_State_Decl_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Params_Range =>
declare
N_Bare_Params : constant Bare_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Params.Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Abstract_State_Decl_Range =>
declare
N_Bare_Paren_Abstract_State_Decl : constant Bare_Paren_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Abstract_State_Decl.Paren_Abstract_State_Decl_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pp_Elsif_Directive_Range =>
declare
N_Bare_Pp_Elsif_Directive : constant Bare_Pp_Elsif_Directive := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pp_Elsif_Directive.Pp_Elsif_Directive_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Pp_Elsif_Directive.Pp_Elsif_Directive_F_Then_Kw;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pp_If_Directive_Range =>
declare
N_Bare_Pp_If_Directive : constant Bare_Pp_If_Directive := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pp_If_Directive.Pp_If_Directive_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Pp_If_Directive.Pp_If_Directive_F_Then_Kw;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Node_Range =>
declare
N_Bare_Pragma_Node : constant Bare_Pragma_Node := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Def_Range =>
declare
N_Bare_Protected_Def : constant Bare_Protected_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Public_Part;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Private_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Spec_Range =>
declare
N_Bare_Range_Spec : constant Bare_Range_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Spec.Range_Spec_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Renaming_Clause_Range =>
declare
N_Bare_Renaming_Clause : constant Bare_Renaming_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Renaming_Clause.Renaming_Clause_F_Renamed_Object;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_When_Part_Range =>
declare
N_Bare_Select_When_Part : constant Bare_Select_When_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Range =>
declare
N_Bare_Accept_Stmt : constant Bare_Accept_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Body_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Entry_Index_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Params;
                            return;
                        when others => null;
                    end case;
case Ada_Accept_Stmt_Range (K) is
when Ada_Accept_Stmt_With_Stmts_Range =>
declare
N_Bare_Accept_Stmt_With_Stmts : constant Bare_Accept_Stmt_With_Stmts := N_Bare_Accept_Stmt;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Base_Loop_Stmt =>
declare
N_Bare_Base_Loop_Stmt : constant Bare_Base_Loop_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Begin_Block_Range =>
declare
N_Bare_Begin_Block : constant Bare_Begin_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Block_Range =>
declare
N_Bare_Decl_Block : constant Bare_Decl_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Range =>
declare
N_Bare_Case_Stmt : constant Bare_Case_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Pragmas;
                            return;
                        when 3 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Alternatives;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Extended_Return_Stmt_Range =>
declare
N_Bare_Extended_Return_Stmt : constant Bare_Extended_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Stmt_Range =>
declare
N_Bare_If_Stmt : constant Bare_If_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Then_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Else_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Range =>
declare
N_Bare_Named_Stmt : constant Bare_Named_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Stmt;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_Stmt_Range =>
declare
N_Bare_Select_Stmt : constant Bare_Select_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Guards;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Else_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Abort_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abort_Stmt_Range =>
declare
N_Bare_Abort_Stmt : constant Bare_Abort_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abort_Stmt.Abort_Stmt_F_Names;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Assign_Stmt_Range =>
declare
N_Bare_Assign_Stmt : constant Bare_Assign_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Dest;
                            return;
                        when 2 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Stmt_Range =>
declare
N_Bare_Call_Stmt : constant Bare_Call_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Stmt.Call_Stmt_F_Call;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delay_Stmt_Range =>
declare
N_Bare_Delay_Stmt : constant Bare_Delay_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Has_Until;
                            return;
                        when 2 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exit_Stmt_Range =>
declare
N_Bare_Exit_Stmt : constant Bare_Exit_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Loop_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Cond_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Goto_Stmt_Range =>
declare
N_Bare_Goto_Stmt : constant Bare_Goto_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Goto_Stmt.Goto_Stmt_F_Label_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Range =>
declare
N_Bare_Label : constant Bare_Label := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label.Label_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Stmt_Range =>
declare
N_Bare_Raise_Stmt : constant Bare_Raise_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Requeue_Stmt_Range =>
declare
N_Bare_Requeue_Stmt : constant Bare_Requeue_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Call_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Has_Abort;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Return_Stmt_Range =>
declare
N_Bare_Return_Stmt : constant Bare_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Return_Stmt.Return_Stmt_F_Return_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Simple_Decl_Stmt_Range =>
declare
N_Bare_Simple_Decl_Stmt : constant Bare_Simple_Decl_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Simple_Decl_Stmt.Simple_Decl_Stmt_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subunit_Range =>
declare
N_Bare_Subunit : constant Bare_Subunit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subunit.Subunit_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Subunit.Subunit_F_Body;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Def_Range =>
declare
N_Bare_Task_Def : constant Bare_Task_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Interfaces;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Public_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Private_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Def.Task_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Access_Def =>
declare
N_Bare_Access_Def : constant Bare_Access_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Access_Def.Access_Def_F_Has_Not_Null;
                            return;
                        when others => null;
                    end case;
case Ada_Access_Def (K) is
when Ada_Access_To_Subp_Def_Range =>
declare
N_Bare_Access_To_Subp_Def : constant Bare_Access_To_Subp_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Has_Protected;
                            return;
                        when 3 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Access_Def_Range =>
declare
N_Bare_Anonymous_Type_Access_Def : constant Bare_Anonymous_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Anonymous_Type_Access_Def.Anonymous_Type_Access_Def_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Access_Def_Range =>
declare
N_Bare_Type_Access_Def : constant Bare_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_All;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Array_Type_Def_Range =>
declare
N_Bare_Array_Type_Def : constant Bare_Array_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Indices;
                            return;
                        when 2 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Component_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Derived_Type_Def_Range =>
declare
N_Bare_Derived_Type_Def : constant Bare_Derived_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Limited;
                            return;
                        when 3 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Synchronized;
                            return;
                        when 4 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Subtype_Indication;
                            return;
                        when 5 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Interfaces;
                            return;
                        when 6 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Record_Extension;
                            return;
                        when 7 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_With_Private;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Type_Def_Range =>
declare
N_Bare_Enum_Type_Def : constant Bare_Enum_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Type_Def.Enum_Type_Def_F_Enum_Literals;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Interface_Type_Def_Range =>
declare
N_Bare_Interface_Type_Def : constant Bare_Interface_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interface_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interfaces;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Mod_Int_Type_Def_Range =>
declare
N_Bare_Mod_Int_Type_Def : constant Bare_Mod_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Mod_Int_Type_Def.Mod_Int_Type_Def_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Private_Type_Def_Range =>
declare
N_Bare_Private_Type_Def : constant Bare_Private_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Limited;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decimal_Fixed_Point_Def_Range =>
declare
N_Bare_Decimal_Fixed_Point_Def : constant Bare_Decimal_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Digits;
                            return;
                        when 3 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Floating_Point_Def_Range =>
declare
N_Bare_Floating_Point_Def : constant Bare_Floating_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Num_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Ordinary_Fixed_Point_Def_Range =>
declare
N_Bare_Ordinary_Fixed_Point_Def : constant Bare_Ordinary_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Type_Def_Range =>
declare
N_Bare_Record_Type_Def : constant Bare_Record_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Limited;
                            return;
                        when 4 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Record_Def;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Signed_Int_Type_Def_Range =>
declare
N_Bare_Signed_Int_Type_Def : constant Bare_Signed_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Signed_Int_Type_Def.Signed_Int_Type_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Range =>
declare
N_Bare_Anonymous_Type : constant Bare_Anonymous_Type := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Type.Anonymous_Type_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subtype_Indication_Range =>
declare
N_Bare_Subtype_Indication : constant Bare_Subtype_Indication := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Has_Not_Null;
                            return;
                        when 2 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Constraint;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Synthetic_Type_Expr_Range =>
declare
N_Bare_Synthetic_Type_Expr : constant Bare_Synthetic_Type_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Synthetic_Type_Expr.Synthetic_Type_Expr_F_Target_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Index_Range =>
declare
N_Bare_Unconstrained_Array_Index : constant Bare_Unconstrained_Array_Index := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Index.Unconstrained_Array_Index_F_Subtype_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Unconstrained_Array_Index.Unconstrained_Array_Index_F_Lower_Bound;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Package_Clause_Range =>
declare
N_Bare_Use_Package_Clause : constant Bare_Use_Package_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Package_Clause.Use_Package_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Type_Clause_Range =>
declare
N_Bare_Use_Type_Clause : constant Bare_Use_Type_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Has_All;
                            return;
                        when 2 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Value_Sequence_Range =>
declare
N_Bare_Value_Sequence : constant Bare_Value_Sequence := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Value_Sequence.Value_Sequence_F_Iter_Assoc;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Range =>
declare
N_Bare_Variant : constant Bare_Variant := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant.Variant_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant.Variant_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Part_Range =>
declare
N_Bare_Variant_Part : constant Bare_Variant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Discr_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Variant;
                            return;
                        when others => null;
                    end case;
end;
when Ada_With_Clause_Range =>
declare
N_Bare_With_Clause : constant Bare_With_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Limited;
                            return;
                        when 2 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Private;
                            return;
                        when 3 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
      --  Execution should reach this point iff nothing matched this index, so
      --  we must be out of bounds.
      Index_In_Bounds := False;
   end Get_Child;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Langkit_Support.Generic_API.Analysis.Lk_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      use Langkit_Support.Generic_API.Analysis;
      use Langkit_Support.Generic_API.Introspection;
      T : Type_Ref;
   begin
      if Node.Is_Null then
         Put_Line ("None");
         return;
      end if;
      T := Type_Of (Node);
      Put (Line_Prefix & Image (Node_Type_Repr_Name (T)));
      if Show_Slocs then
         Put ("[" & Image (Node.Sloc_Range) & "]");
      end if;
      if Node.Is_Incomplete then
         Put (" <<INCOMPLETE>>");
      end if;
      if Node.Is_Token_Node then
         Put_Line (": " & Image (Node.Text));
      elsif Is_List_Node (Node) then
         --  List nodes are displayed in a special way (they have no field)
         declare
            Count : constant Natural := Node.Children_Count;
            Child : Lk_Node;
         begin
            if Count = 0 then
               Put_Line (": <empty list>");
               return;
            end if;
            New_Line;
            for I in 1 .. Count loop
               Child := Node.Child (I);
               if not Child.Is_Null then
                  Print (Child, Show_Slocs, Line_Prefix & "|  ");
               end if;
            end loop;
         end;
      else
         --  This is for regular nodes: display each syntax field (i.e.
         --  non-property member).
         declare
            Attr_Prefix     : constant String := Line_Prefix & "|";
            Children_Prefix : constant String := Line_Prefix & "|  ";
            M_List          : constant Struct_Member_Ref_Array := Members (T);
            Child           : Lk_Node;
         begin
            New_Line;
            for M of M_List loop
               if not Is_Property (M) and then not Is_Null_For (M, T) then
                  Child := As_Node (Eval_Node_Member (Node, M));
                  Put (Attr_Prefix
                       & Image (Format_Name (Member_Name (M), Lower)) & ":");
                  if Child.Is_Null then
                     Put_Line (" <null>");
                  else
                     New_Line;
                     Print (Child, Show_Slocs, Children_Prefix);
                  end if;
               end if;
            end loop;
         end;
      end if;
   end Print;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Bare_Ada_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      Entity : constant Internal_Entity := (Node, No_Entity_Info);
   begin
      Print (To_Generic_Node (Entity), Show_Slocs, Line_Prefix);
   end Print;
   ------------
   -- Parent --
   ------------
   function Parent (Node : Bare_Ada_Node) return Bare_Ada_Node is
   begin
      return Node.Parent;
   end Parent;
   ------------------
   -- Stored_Token --
   ------------------
   function Stored_Token
     (Node  : Bare_Ada_Node;
      Token : Token_Reference) return Token_Index
   is
      Index : constant Token_Or_Trivia_Index := Get_Token_Index (Token);
   begin
      if Node.Unit.TDH'Access /= Get_Token_TDH (Token) then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "Cannot associate a token and a node from different analysis"
            & " units");
      elsif Index.Trivia /= No_Token_Index then
         Raise_Property_Exception
           (Node,
            Property_Error'Identity,
            "A node cannot hold trivia");
      end if;
      return Index.Token;
   end Stored_Token;
   -------------------------
   -- Children_And_Trivia --
   -------------------------
   function Children_And_Trivia
     (Node : Bare_Ada_Node) return Bare_Children_Vector
   is
      Ret_Vec : Bare_Children_Vector;
      Ctx     : Internal_Context renames Node.Unit.Context;
      TDH     : Token_Data_Handler renames Node.Unit.TDH;
      procedure Append_Trivias (First, Last : Token_Index);
      --  Append all the trivias of tokens between indices First and Last to
      --  the returned vector.
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array;
      --  Return an array for all children in Parent that are not null
      --------------------
      -- Append_Trivias --
      --------------------
      procedure Append_Trivias (First, Last : Token_Index) is
      begin
         for I in First .. Last loop
            for D of Get_Trivias (TDH, I) loop
               Ret_Vec.Append
                 (Bare_Child_Record'
                    (Kind   => Trivia,
                     Trivia => Wrap_Token_Reference
                                 (Ctx, TDH'Access, (I, D))));
            end loop;
         end loop;
      end Append_Trivias;
      ---------------------
      -- Filter_Children --
      ---------------------
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array
      is
         Children : constant Internal_Bare_Ada_Node_Array :=
            Implementation.Children (Parent);
         Result   : Internal_Bare_Ada_Node_Array (Children'Range);
         Next     : Integer := Result'First;
      begin
         for I in Children'Range loop
            if Children (I) /= null then
               Result (Next) := Children (I);
               Next := Next + 1;
            end if;
         end loop;
         return Result (Result'First .. Next - 1);
      end Filter_Children;
      First_Child : constant Positive := 1;
      N_Children  : constant Internal_Bare_Ada_Node_Array :=
         Filter_Children (Node);
   begin
      if N_Children'Length > 0
        and then (Node.Token_Start_Index
                    /= N_Children (First_Child).Token_Start_Index)
      then
         Append_Trivias (Node.Token_Start_Index,
                         N_Children (First_Child).Token_Start_Index - 1);
      end if;
      --  Append each node to Ret_Vec, and append trivia that follow after each
      --  non-ghost nodes.
      for I in N_Children'Range loop
         Ret_Vec.Append (Bare_Child_Record'(Child, N_Children (I)));
         if not Is_Ghost (N_Children (I)) then
            Append_Trivias (N_Children (I).Token_End_Index,
                            (if I = N_Children'Last
                             then Node.Token_End_Index - 1
                             else N_Children (I + 1).Token_Start_Index - 1));
         end if;
      end loop;
      return Ret_Vec;
   end Children_And_Trivia;
   --------------
   -- Is_Ghost --
   --------------
   function Is_Ghost (Node : Bare_Ada_Node) return Boolean
   is (Node.Token_End_Index = No_Token_Index);
   -------------------
   -- Is_Incomplete --
   -------------------
   function Is_Incomplete (Node : Bare_Ada_Node) return Boolean
   is
      LGC : Bare_Ada_Node;
   begin
     if Is_List_Node (Node.Kind) then
        LGC := (if Last_Child_Index (Node) /= 0
                then Child (Node, Last_Child_Index (Node))
                else null);
        return LGC /= null and then Is_Incomplete (LGC);
      else
         return Node.Last_Attempted_Child > -1;
      end if;
   end;
   -----------------
   -- Token_Start --
   -----------------
   function Token_Start (Node : Bare_Ada_Node) return Token_Reference
   is (Token (Node, Node.Token_Start_Index));
   ---------------
   -- Token_End --
   ---------------
   function Token_End (Node : Bare_Ada_Node) return Token_Reference
   is
     (if Node.Token_End_Index = No_Token_Index
      then Token_Start (Node)
      else Token (Node, Node.Token_End_Index));
   -----------
   -- Token --
   -----------
   function Token
     (Node  : Bare_Ada_Node;
      Index : Token_Index) return Token_Reference
   is
      Unit    : constant Internal_Unit := Node.Unit;
      Context : constant Internal_Context := Unit.Context;
   begin
      return Wrap_Token_Reference
        (Context, Token_Data (Unit), (Index, No_Token_Index));
   end Token;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Bare_Ada_Node) return Boolean is
   begin
      --  Reject invalid inputs
      if Left /= null and Is_Synthetic (Left) then
         raise Property_Error with "left node is synthetic";
      elsif Right /= null and Is_Synthetic (Right) then
         raise Property_Error with "right node is synthetic";
      end if;
      --  Null nodes come first
      if Left = null then
         return Right /= null;
      elsif Right = null then
         return False;
      end if;
      --  So we have two non-null nodes. Sort by unit filename
      if Left.Unit < Right.Unit then
         return True;
      elsif Left.Unit /= Right.Unit then
         return False;
      end if;
      --  Both nodes come from the same unit: compare their token indexes
      if Left.Token_Start_Index < Right.Token_Start_Index then
         return True;
      elsif Left.Token_Start_Index > Right.Token_Start_Index then
         return False;
      else
         return Left.Token_End_Index < Right.Token_End_Index;
      end if;
   end "<";
   -------------
   -- Is_Null --
   -------------
   function Is_Null (Node : Bare_Ada_Node) return Boolean
   is (Node = null);
   ----------
   -- Kind --
   ----------
   function Kind (Node : Bare_Ada_Node) return Ada_Node_Kind_Type
   is (Node.Kind);
   -----------------
   -- Child_Index --
   -----------------
   function Child_Index (Node : Bare_Ada_Node) return Integer
   is
      N : Bare_Ada_Node := null;
   begin
      if Node.Parent = null then
         raise Property_Error with
            "Trying to get the child index of a root node";
      end if;
      for I in First_Child_Index (Node.Parent)
            .. Last_Child_Index (Node.Parent)
      loop
         N := Child (Node.Parent, I);
         if N = Node then
            return I - 1;
         end if;
      end loop;
      --  If we reach this point, then Node isn't a Child of Node.Parent. This
      --  is not supposed to happen.
      raise Program_Error;
   end Child_Index;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      Offset : Integer) return Bare_Ada_Node is
   begin
      --  Root nodes have no sibling: handle them now to avoid invalid requests
      --  in the code below.
      if Node.Parent = null then
         return null;
      end if;
      declare
         Node_Index : constant Positive := Child_Index (Node) + 1;
         --  Child_Index is 0-based, but the Child primitive expects a 1-based
         --  index.
         Sibling_Index : constant Integer := Node_Index + Offset;
      begin
         --  Child returns null for out-of-bound indexes
         return (if Sibling_Index >= 1
                 then Child (Node.Parent, Sibling_Index)
                 else null);
      end;
   end Fetch_Sibling;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info;
      Offset : Integer) return Internal_Entity
   is
      Sibling : constant Bare_Ada_Node := Fetch_Sibling (Node, Offset);
   begin
      --  Don't forget to clear entity info if the result is nul
      return (if Sibling = null
              then No_Entity
              else (Sibling, E_Info));
   end Fetch_Sibling;
   ----------------------
   -- Previous_Sibling --
   ----------------------
   function Previous_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, -1);
   end Previous_Sibling;
   ------------------
   -- Next_Sibling --
   ------------------
   function Next_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, 1);
   end Next_Sibling;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Self : Node_Builder_Type) is
   begin
      if Self.Ref_Count > 0 then
         Self.Ref_Count := Self.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Self : in out Node_Builder_Type) is
   begin
      if Self = null or else Self.Ref_Count < 0 then
         return;
      elsif Self.Ref_Count = 1 then
         Self.Release;
         Free (Self);
      else
         Self.Ref_Count := Self.Ref_Count - 1;
      end if;
   end Dec_Ref;
   ------------------------------
   -- Create_Copy_Node_Builder --
   ------------------------------
   function Create_Copy_Node_Builder
     (Value : Bare_Ada_Node) return Node_Builder_Type is
   begin
      --  No need to allocate a new builder if in practice it cannot be
      --  distinguished from the "null" builder.
      if Value = null then
         return Null_Node_Builder;
      else
         return new Copy_Node_Builder_Record'(Ref_Count => 1, Value => Value);
      end if;
   end Create_Copy_Node_Builder;
         type Bare_Synthetic_Subp_Decl_Node_Builder_Record is new Node_Builder_Record with
            record
               Synthetic_Subp_Decl_F_Spec : Bare_Base_Subp_Spec_Node_Builder;
            end record;
         type Bare_Synthetic_Subp_Decl_Node_Builder_Access is access all Bare_Synthetic_Subp_Decl_Node_Builder_Record;
         overriding function Build
           (Self              : Bare_Synthetic_Subp_Decl_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node;
            overriding procedure Release (Self : in out Bare_Synthetic_Subp_Decl_Node_Builder_Record);
         -----------
         -- Build --
         -----------
         overriding function Build
           (Self              : Bare_Synthetic_Subp_Decl_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node
         is
            Result : Bare_Ada_Node;
            Unit   : constant Internal_Unit := Self_Node.Unit;
            Env    : constant Lexical_Env :=
              (if Parent = null
               then Empty_Env
               else Parent.Self_Env);
         begin
            if Parent /= null and then Parent.Unit /= Unit then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic node parents must belong to the same unit as the"
                  & " nodes that trigger node synthetization");
            end if;
            if Is_Foreign_Strict (Env, Parent) then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic nodes cannot have foreign lexical envs");
            end if;
            Result := new Root_Node_Record
              (Ada_Synthetic_Subp_Decl);
            Initialize
              (Self => Result,
               Kind => Ada_Synthetic_Subp_Decl,
               Unit => Unit,
               Token_Start_Index => No_Token_Index,
               Token_End_Index   => No_Token_Index,
               Parent   => Parent,
               Self_Env => Env);
            Register_Destroyable (Unit, Result);
               declare
                  Children : array (1 .. 1)
                               of Bare_Ada_Node;
               begin
                     Children (1) :=
                       Self.Synthetic_Subp_Decl_F_Spec.Build (Result, Self_Node);
                        if Children (1) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticSubpDecl.spec cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                  Initialize_Fields_For_Synthetic_Subp_Decl
                    (Self => Result,
                     Synthetic_Subp_Decl_F_Spec => Children (1));
               end;
            return Result;
         end Build;
            -------------
            -- Release --
            -------------
            overriding procedure Release (Self : in out Bare_Synthetic_Subp_Decl_Node_Builder_Record) is
            begin
                     Dec_Ref (Self.Synthetic_Subp_Decl_F_Spec);
            end Release;
         function Create_Bare_Synthetic_Subp_Decl_Node_Builder
             (Synthetic_Subp_Decl_F_Spec : Bare_Base_Subp_Spec_Node_Builder)
           return Bare_Synthetic_Subp_Decl_Node_Builder
         is
            Builder : constant Bare_Synthetic_Subp_Decl_Node_Builder_Access := new Bare_Synthetic_Subp_Decl_Node_Builder_Record;
         begin
            Builder.Ref_Count := 1;
               Builder.Synthetic_Subp_Decl_F_Spec := Synthetic_Subp_Decl_F_Spec;
                  Inc_Ref (Builder.Synthetic_Subp_Decl_F_Spec);
            return Node_Builder_Type (Builder);
         end Create_Bare_Synthetic_Subp_Decl_Node_Builder;
         type Bare_Synthetic_Unary_Spec_Node_Builder_Record is new Node_Builder_Record with
            record
               Synthetic_Unary_Spec_Subp_Symbol : Symbol_Type;
               Synthetic_Unary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Unary_Spec_F_Return_Type_Expr : Bare_Synthetic_Type_Expr_Node_Builder;
            end record;
         type Bare_Synthetic_Unary_Spec_Node_Builder_Access is access all Bare_Synthetic_Unary_Spec_Node_Builder_Record;
         overriding function Build
           (Self              : Bare_Synthetic_Unary_Spec_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node;
            overriding procedure Release (Self : in out Bare_Synthetic_Unary_Spec_Node_Builder_Record);
         -----------
         -- Build --
         -----------
         overriding function Build
           (Self              : Bare_Synthetic_Unary_Spec_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node
         is
            Result : Bare_Ada_Node;
            Unit   : constant Internal_Unit := Self_Node.Unit;
            Env    : constant Lexical_Env :=
              (if Parent = null
               then Empty_Env
               else Parent.Self_Env);
         begin
            if Parent /= null and then Parent.Unit /= Unit then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic node parents must belong to the same unit as the"
                  & " nodes that trigger node synthetization");
            end if;
            if Is_Foreign_Strict (Env, Parent) then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic nodes cannot have foreign lexical envs");
            end if;
            Result := new Root_Node_Record
              (Ada_Synthetic_Unary_Spec);
            Initialize
              (Self => Result,
               Kind => Ada_Synthetic_Unary_Spec,
               Unit => Unit,
               Token_Start_Index => No_Token_Index,
               Token_End_Index   => No_Token_Index,
               Parent   => Parent,
               Self_Env => Env);
            Register_Destroyable (Unit, Result);
               declare
                  Children : array (1 .. 2)
                               of Bare_Ada_Node;
               begin
                     Children (1) :=
                       Self.Synthetic_Unary_Spec_F_Right_Param.Build (Result, Self_Node);
                        if Children (1) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticUnarySpec.right_param cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                     Children (2) :=
                       Self.Synthetic_Unary_Spec_F_Return_Type_Expr.Build (Result, Self_Node);
                        if Children (2) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticUnarySpec.return_type_expr cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                  Initialize_Fields_For_Synthetic_Unary_Spec
                    (Self => Result,
                     Synthetic_Unary_Spec_F_Right_Param => Children (1),
                     Synthetic_Unary_Spec_F_Return_Type_Expr => Children (2));
               end;
               Result.Synthetic_Unary_Spec_Subp_Symbol :=
                  Self.Synthetic_Unary_Spec_Subp_Symbol;
            return Result;
         end Build;
            -------------
            -- Release --
            -------------
            overriding procedure Release (Self : in out Bare_Synthetic_Unary_Spec_Node_Builder_Record) is
            begin
                     Dec_Ref (Self.Synthetic_Unary_Spec_F_Right_Param);
                     Dec_Ref (Self.Synthetic_Unary_Spec_F_Return_Type_Expr);
            end Release;
         function Create_Bare_Synthetic_Unary_Spec_Node_Builder
             (Synthetic_Unary_Spec_Subp_Symbol : Symbol_Type;
               Synthetic_Unary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Unary_Spec_F_Return_Type_Expr : Bare_Synthetic_Type_Expr_Node_Builder)
           return Bare_Synthetic_Unary_Spec_Node_Builder
         is
            Builder : constant Bare_Synthetic_Unary_Spec_Node_Builder_Access := new Bare_Synthetic_Unary_Spec_Node_Builder_Record;
         begin
            Builder.Ref_Count := 1;
               Builder.Synthetic_Unary_Spec_Subp_Symbol := Synthetic_Unary_Spec_Subp_Symbol;
               Builder.Synthetic_Unary_Spec_F_Right_Param := Synthetic_Unary_Spec_F_Right_Param;
                  Inc_Ref (Builder.Synthetic_Unary_Spec_F_Right_Param);
               Builder.Synthetic_Unary_Spec_F_Return_Type_Expr := Synthetic_Unary_Spec_F_Return_Type_Expr;
                  Inc_Ref (Builder.Synthetic_Unary_Spec_F_Return_Type_Expr);
            return Node_Builder_Type (Builder);
         end Create_Bare_Synthetic_Unary_Spec_Node_Builder;
         type Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record is new Node_Builder_Record with
            record
               Synthetic_Formal_Param_Decl_Param_Name : Symbol_Type;
               Synthetic_Formal_Param_Decl_F_Param_Type : Bare_Type_Expr_Node_Builder;
            end record;
         type Bare_Synthetic_Formal_Param_Decl_Node_Builder_Access is access all Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record;
         overriding function Build
           (Self              : Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node;
            overriding procedure Release (Self : in out Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record);
         -----------
         -- Build --
         -----------
         overriding function Build
           (Self              : Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node
         is
            Result : Bare_Ada_Node;
            Unit   : constant Internal_Unit := Self_Node.Unit;
            Env    : constant Lexical_Env :=
              (if Parent = null
               then Empty_Env
               else Parent.Self_Env);
         begin
            if Parent /= null and then Parent.Unit /= Unit then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic node parents must belong to the same unit as the"
                  & " nodes that trigger node synthetization");
            end if;
            if Is_Foreign_Strict (Env, Parent) then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic nodes cannot have foreign lexical envs");
            end if;
            Result := new Root_Node_Record
              (Ada_Synthetic_Formal_Param_Decl);
            Initialize
              (Self => Result,
               Kind => Ada_Synthetic_Formal_Param_Decl,
               Unit => Unit,
               Token_Start_Index => No_Token_Index,
               Token_End_Index   => No_Token_Index,
               Parent   => Parent,
               Self_Env => Env);
            Register_Destroyable (Unit, Result);
               declare
                  Children : array (1 .. 1)
                               of Bare_Ada_Node;
               begin
                     Children (1) :=
                       Self.Synthetic_Formal_Param_Decl_F_Param_Type.Build (Result, Self_Node);
                        if Children (1) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticFormalParamDecl.param_type cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                  Initialize_Fields_For_Synthetic_Formal_Param_Decl
                    (Self => Result,
                     Synthetic_Formal_Param_Decl_F_Param_Type => Children (1));
               end;
               Result.Synthetic_Formal_Param_Decl_Param_Name :=
                  Self.Synthetic_Formal_Param_Decl_Param_Name;
            return Result;
         end Build;
            -------------
            -- Release --
            -------------
            overriding procedure Release (Self : in out Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record) is
            begin
                     Dec_Ref (Self.Synthetic_Formal_Param_Decl_F_Param_Type);
            end Release;
         function Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder
             (Synthetic_Formal_Param_Decl_Param_Name : Symbol_Type;
               Synthetic_Formal_Param_Decl_F_Param_Type : Bare_Type_Expr_Node_Builder)
           return Bare_Synthetic_Formal_Param_Decl_Node_Builder
         is
            Builder : constant Bare_Synthetic_Formal_Param_Decl_Node_Builder_Access := new Bare_Synthetic_Formal_Param_Decl_Node_Builder_Record;
         begin
            Builder.Ref_Count := 1;
               Builder.Synthetic_Formal_Param_Decl_Param_Name := Synthetic_Formal_Param_Decl_Param_Name;
               Builder.Synthetic_Formal_Param_Decl_F_Param_Type := Synthetic_Formal_Param_Decl_F_Param_Type;
                  Inc_Ref (Builder.Synthetic_Formal_Param_Decl_F_Param_Type);
            return Node_Builder_Type (Builder);
         end Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder;
         type Bare_Synthetic_Type_Expr_Node_Builder_Record is new Node_Builder_Record with
            record
               Synthetic_Type_Expr_F_Target_Type : Bare_Base_Type_Decl_Node_Builder;
            end record;
         type Bare_Synthetic_Type_Expr_Node_Builder_Access is access all Bare_Synthetic_Type_Expr_Node_Builder_Record;
         overriding function Build
           (Self              : Bare_Synthetic_Type_Expr_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node;
            overriding procedure Release (Self : in out Bare_Synthetic_Type_Expr_Node_Builder_Record);
         -----------
         -- Build --
         -----------
         overriding function Build
           (Self              : Bare_Synthetic_Type_Expr_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node
         is
            Result : Bare_Ada_Node;
            Unit   : constant Internal_Unit := Self_Node.Unit;
            Env    : constant Lexical_Env :=
              (if Parent = null
               then Empty_Env
               else Parent.Self_Env);
         begin
            if Parent /= null and then Parent.Unit /= Unit then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic node parents must belong to the same unit as the"
                  & " nodes that trigger node synthetization");
            end if;
            if Is_Foreign_Strict (Env, Parent) then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic nodes cannot have foreign lexical envs");
            end if;
            Result := new Root_Node_Record
              (Ada_Synthetic_Type_Expr);
            Initialize
              (Self => Result,
               Kind => Ada_Synthetic_Type_Expr,
               Unit => Unit,
               Token_Start_Index => No_Token_Index,
               Token_End_Index   => No_Token_Index,
               Parent   => Parent,
               Self_Env => Env);
            Register_Destroyable (Unit, Result);
               declare
                  Children : array (1 .. 1)
                               of Bare_Ada_Node;
               begin
                     Children (1) :=
                       Self.Synthetic_Type_Expr_F_Target_Type.Build (Result, Self_Node);
                        if Children (1) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticTypeExpr.target_type cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                  Initialize_Fields_For_Synthetic_Type_Expr
                    (Self => Result,
                     Synthetic_Type_Expr_F_Target_Type => Children (1));
               end;
            return Result;
         end Build;
            -------------
            -- Release --
            -------------
            overriding procedure Release (Self : in out Bare_Synthetic_Type_Expr_Node_Builder_Record) is
            begin
                     Dec_Ref (Self.Synthetic_Type_Expr_F_Target_Type);
            end Release;
         function Create_Bare_Synthetic_Type_Expr_Node_Builder
             (Synthetic_Type_Expr_F_Target_Type : Bare_Base_Type_Decl_Node_Builder)
           return Bare_Synthetic_Type_Expr_Node_Builder
         is
            Builder : constant Bare_Synthetic_Type_Expr_Node_Builder_Access := new Bare_Synthetic_Type_Expr_Node_Builder_Record;
         begin
            Builder.Ref_Count := 1;
               Builder.Synthetic_Type_Expr_F_Target_Type := Synthetic_Type_Expr_F_Target_Type;
                  Inc_Ref (Builder.Synthetic_Type_Expr_F_Target_Type);
            return Node_Builder_Type (Builder);
         end Create_Bare_Synthetic_Type_Expr_Node_Builder;
         type Bare_Synthetic_Binary_Spec_Node_Builder_Record is new Node_Builder_Record with
            record
               Synthetic_Binary_Spec_Subp_Symbol : Symbol_Type;
               Synthetic_Binary_Spec_F_Left_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Binary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Binary_Spec_F_Return_Type_Expr : Bare_Type_Expr_Node_Builder;
            end record;
         type Bare_Synthetic_Binary_Spec_Node_Builder_Access is access all Bare_Synthetic_Binary_Spec_Node_Builder_Record;
         overriding function Build
           (Self              : Bare_Synthetic_Binary_Spec_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node;
            overriding procedure Release (Self : in out Bare_Synthetic_Binary_Spec_Node_Builder_Record);
         -----------
         -- Build --
         -----------
         overriding function Build
           (Self              : Bare_Synthetic_Binary_Spec_Node_Builder_Record;
            Parent, Self_Node : Bare_Ada_Node)
            return Bare_Ada_Node
         is
            Result : Bare_Ada_Node;
            Unit   : constant Internal_Unit := Self_Node.Unit;
            Env    : constant Lexical_Env :=
              (if Parent = null
               then Empty_Env
               else Parent.Self_Env);
         begin
            if Parent /= null and then Parent.Unit /= Unit then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic node parents must belong to the same unit as the"
                  & " nodes that trigger node synthetization");
            end if;
            if Is_Foreign_Strict (Env, Parent) then
               Raise_Property_Exception
                 (Self_Node,
                  Property_Error'Identity,
                  "synthetic nodes cannot have foreign lexical envs");
            end if;
            Result := new Root_Node_Record
              (Ada_Synthetic_Binary_Spec);
            Initialize
              (Self => Result,
               Kind => Ada_Synthetic_Binary_Spec,
               Unit => Unit,
               Token_Start_Index => No_Token_Index,
               Token_End_Index   => No_Token_Index,
               Parent   => Parent,
               Self_Env => Env);
            Register_Destroyable (Unit, Result);
               declare
                  Children : array (1 .. 3)
                               of Bare_Ada_Node;
               begin
                     Children (1) :=
                       Self.Synthetic_Binary_Spec_F_Left_Param.Build (Result, Self_Node);
                        if Children (1) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticBinarySpec.left_param cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                     Children (2) :=
                       Self.Synthetic_Binary_Spec_F_Right_Param.Build (Result, Self_Node);
                        if Children (2) = null then
                           Raise_Property_Exception
                             (Self_Node,
                              Property_Error'Identity,
                              "SyntheticBinarySpec.right_param cannot be null in synthetic"
                              & " nodes; add a nullable annotation to this"
                              & " field to allow it");
                        end if;
                     Children (3) :=
                       Self.Synthetic_Binary_Spec_F_Return_Type_Expr.Build (Result, Self_Node);
                  Initialize_Fields_For_Synthetic_Binary_Spec
                    (Self => Result,
                     Synthetic_Binary_Spec_F_Left_Param => Children (1),
                     Synthetic_Binary_Spec_F_Right_Param => Children (2),
                     Synthetic_Binary_Spec_F_Return_Type_Expr => Children (3));
               end;
               Result.Synthetic_Binary_Spec_Subp_Symbol :=
                  Self.Synthetic_Binary_Spec_Subp_Symbol;
            return Result;
         end Build;
            -------------
            -- Release --
            -------------
            overriding procedure Release (Self : in out Bare_Synthetic_Binary_Spec_Node_Builder_Record) is
            begin
                     Dec_Ref (Self.Synthetic_Binary_Spec_F_Left_Param);
                     Dec_Ref (Self.Synthetic_Binary_Spec_F_Right_Param);
                     Dec_Ref (Self.Synthetic_Binary_Spec_F_Return_Type_Expr);
            end Release;
         function Create_Bare_Synthetic_Binary_Spec_Node_Builder
             (Synthetic_Binary_Spec_Subp_Symbol : Symbol_Type;
               Synthetic_Binary_Spec_F_Left_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Binary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
               Synthetic_Binary_Spec_F_Return_Type_Expr : Bare_Type_Expr_Node_Builder)
           return Bare_Synthetic_Binary_Spec_Node_Builder
         is
            Builder : constant Bare_Synthetic_Binary_Spec_Node_Builder_Access := new Bare_Synthetic_Binary_Spec_Node_Builder_Record;
         begin
            Builder.Ref_Count := 1;
               Builder.Synthetic_Binary_Spec_Subp_Symbol := Synthetic_Binary_Spec_Subp_Symbol;
               Builder.Synthetic_Binary_Spec_F_Left_Param := Synthetic_Binary_Spec_F_Left_Param;
                  Inc_Ref (Builder.Synthetic_Binary_Spec_F_Left_Param);
               Builder.Synthetic_Binary_Spec_F_Right_Param := Synthetic_Binary_Spec_F_Right_Param;
                  Inc_Ref (Builder.Synthetic_Binary_Spec_F_Right_Param);
               Builder.Synthetic_Binary_Spec_F_Return_Type_Expr := Synthetic_Binary_Spec_F_Return_Type_Expr;
                  Inc_Ref (Builder.Synthetic_Binary_Spec_F_Return_Type_Expr);
            return Node_Builder_Type (Builder);
         end Create_Bare_Synthetic_Binary_Spec_Node_Builder;
   ----------------------
   -- Compare_Metadata --
   ----------------------
   --  Deactivate "not referenced" warnings because if the metadata struct has
   --  no fields, formals and temporaries won't be referenced in the two
   --  following functions.
   pragma Warnings (Off, "referenced");
   function Compare_Metadata (L, R : Internal_Metadata) return Boolean is
   begin
            if L.Primitive /= R.Primitive then
               return False;
            end if;
            if L.Primitive_Real_Type /= R.Primitive_Real_Type then
               return False;
            end if;
      return True;
   end Compare_Metadata;
   ----------
   -- Hash --
   ----------
   function Hash (Self : Internal_Metadata) return Hash_Type is
      Ret : Hash_Type := Langkit_Support.Hashes.Initial_Hash;
   begin
               Ret := Combine (Ret, Hash (Self.Primitive));
               Ret := Combine (Ret, Hash (Self.Primitive_Real_Type));
      return Ret;
   end Hash;
   pragma Warnings (On, "referenced");
   -------------
   -- Combine --
   -------------
   function Combine
     (L, R : Internal_Metadata) return Internal_Metadata
   is
      Ret : Internal_Metadata := No_Metadata;
   begin
         Ret.Dottable_Subp := L.Dottable_Subp or R.Dottable_Subp;
         if L.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := R.Primitive;
         elsif R.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := L.Primitive;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
         if L.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := R.Primitive_Real_Type;
         elsif R.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := L.Primitive_Real_Type;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
      return Ret;
   end Combine;
   -------------------------------
   -- Create_Static_Lexical_Env --
   -------------------------------
   function Create_Static_Lexical_Env
     (Parent            : Lexical_Env;
      Node              : Bare_Ada_Node;
      Sym_Table         : Symbol_Table;
      Transitive_Parent : Boolean := False) return Lexical_Env
   is
      Unit : constant Internal_Unit :=
        (if Node = null then null else Node.Unit);
   begin
      return Result : Lexical_Env := Create_Lexical_Env
        (Parent, Node, Transitive_Parent, Sym_Table, Convert_Unit (Unit))
      do
         if Unit /= null then
            Register_Destroyable (Unit, Unwrap (Result.Env));
         end if;
      end return;
   end Create_Static_Lexical_Env;
   ---------
   -- Get --
   ---------
   function Get
     (Self  : Bare_Ada_Node;
      A     : AST_Envs.Entity_Array;
      Index : Integer) return Internal_Entity
   is
      function Length (A : AST_Envs.Entity_Array) return Natural
      is (A'Length);
      function Get
        (A     : AST_Envs.Entity_Array;
         Index : Integer) return Internal_Entity
      is (A (Index + 1)); --  A is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Entity,
         Sequence_Type => AST_Envs.Entity_Array,
         Length        => Length,
         Get           => Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (A, Index, Result) then
         return Result;
      else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "out-of-bounds array access");
      end if;
   end Get;
   -----------
   -- Group --
   -----------
   function Group
     (Envs   : Lexical_Env_Array_Access;
      Env_Md : Internal_Metadata := No_Metadata) return Lexical_Env
   is (Group (Lexical_Env_Array (Envs.Items), Env_Md));
      --
      --  Untyped wrappers for Bare_Ada_Node
      --
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Base_Type_Decl
      --
function Base_Type_Decl_F_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
      --
      --  Untyped wrappers for Bare_Type_Decl
      --
function Type_Decl_P_Refined_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Body_Node
      --
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Package_Body
      --
function Package_Body_P_Package_Decl_Use_Clauses_Envs
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Task_Body
      --
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Generic_Instantiation
      --
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
function Generic_Instantiation_P_Parent_Instantiation_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Name
      --
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env;
   ------------------
   -- Children_Env --
   ------------------
   function Children_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is (Rebind_Env (Node.Self_Env, E_Info));
   --------------
   -- Node_Env --
   --------------
   function Node_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is
      function Get_Base_Env return Lexical_Env;
      --  Return the environment that we need to rebind before returning
      ------------------
      -- Get_Base_Env --
      ------------------
      function Get_Base_Env return Lexical_Env is
         pragma Warnings (Off, "referenced");
         function Get_Parent_Env return Lexical_Env;
         pragma Warnings (On, "referenced");
         --------------------
         -- Get_Parent_Env --
         --------------------
         function Get_Parent_Env return Lexical_Env is
            Parent : constant Lexical_Env := AST_Envs.Parent (Node.Self_Env);
         begin
            --  If Node is the root scope or the empty environment, Parent can
            --  be a wrapper around the null node. Turn this into the
            --  Empty_Env, as null envs are erroneous values in properties.
            return (if Unwrap (Parent) = null
                    then Empty_Env
                    else Parent);
         end Get_Parent_Env;
      begin
         return
           (if Node.Kind in Ada_Generic_Package_Internal | Ada_Package_Decl | Ada_Subtype_Decl | Ada_Incomplete_Type_Decl | Ada_Incomplete_Formal_Type_Decl | Ada_Incomplete_Tagged_Type_Decl | Ada_Protected_Type_Decl | Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl | Ada_Formal_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl | Ada_Subp_Decl | Ada_Entry_Decl | Ada_Enum_Literal_Decl | Ada_Synthetic_Char_Enum_Lit | Ada_Generic_Subp_Internal | Ada_Synthetic_Subp_Decl | Ada_Expr_Function | Ada_Null_Subp_Decl | Ada_Subp_Body | Ada_Subp_Renaming_Decl | Ada_Package_Body_Stub | Ada_Protected_Body_Stub | Ada_Subp_Body_Stub | Ada_Task_Body_Stub | Ada_Entry_Body | Ada_Package_Body | Ada_Protected_Body | Ada_Task_Body | Ada_Exception_Handler | Ada_Generic_Package_Decl | Ada_Generic_Subp_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Subp_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Generic_Subp_Renaming_Decl | Ada_Package_Renaming_Decl | Ada_Single_Protected_Decl | Ada_Single_Task_Decl | Ada_Private_Part | Ada_Decl_Expr | Ada_Quantified_Expr | Ada_Accept_Stmt | Ada_Accept_Stmt_With_Stmts | Ada_For_Loop_Stmt | Ada_Begin_Block | Ada_Decl_Block | Ada_Extended_Return_Stmt | Ada_Named_Stmt | Ada_Access_To_Subp_Def
            then Get_Parent_Env
            else Node.Self_Env);
      end Get_Base_Env;
      Base_Env : Lexical_Env := Get_Base_Env;
      Result   : constant Lexical_Env := Rebind_Env (Base_Env, E_Info);
   begin
      Dec_Ref (Base_Env);
      return Result;
   end Node_Env;
   ------------
   -- Parent --
   ------------
   function Parent
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      --  TODO: shed entity information as appropriate
      return (Node.Parent, E_Info);
   end Parent;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True;
      E_Info    : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Parents : Bare_Ada_Node_Array_Access := Parents (Node, With_Self);
      Result       : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Parents.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Parents.Items'Range loop
         Result.Items (I) := (Bare_Parents.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Parents);
      return Result;
   end Parents;
   --------------
   -- Children --
   --------------
   function Children
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Children : Bare_Ada_Node_Array_Access := Children (Node);
      Result        : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Children.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Children.Items'Range loop
         Result.Items (I) := (Bare_Children.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Children);
      return Result;
   end Children;
   ---------------------
   -- New_Unit_String --
   ---------------------
   function New_Unit_String
     (Unit : Internal_Unit; Str : String) return String_Access
   is
      procedure Register_Destroyable_String is new Register_Destroyable_Gen
        (String, String_Access, Free);
   begin
      return Ret : String_Access := new String'(Str) do
         Register_Destroyable_String (Unit, Ret);
      end return;
   end New_Unit_String;
   --------------------------------
   -- Assign_Names_To_Logic_Vars --
   --------------------------------
   procedure Assign_Names_To_Logic_Vars (Node : Bare_Ada_Node) is
      pragma Warnings (Off, "referenced");
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String);
      --  Assign a name to the LV logic variable. Node must be the node that
      --  owns LV, and Field must be the name of the field in Node that holds
      --  LV.
      ------------
      -- Assign --
      ------------
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String) is
      begin
         LV.Dbg_Name :=
           new String'(Image (Short_Text_Image (Node)) & "." & Field);
      end Assign;
      K : constant Ada_Node_Kind_Type := Node.Kind;
      pragma Warnings (On, "referenced");
   begin
      case Ada_Ada_Node (K) is
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Assign (N_Bare_Attribute_Ref,        N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Assign (N_Bare_Call_Expr,        N_Bare_Call_Expr.Call_Expr_R_Called_Spec,        "r_called_spec");
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Assign (N_Bare_Explicit_Deref,        N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec,        "r_called_spec");
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
Assign (N_Bare_Reduce_Attribute_Ref,        N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Assign (N_Bare_Target_Name,        N_Bare_Target_Name.Target_Name_R_Ref_Var,        "r_ref_var");
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
Assign (N_Bare_Update_Attribute_Ref,        N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_R_Ref_Var,        "r_ref_var");
end;
when others => null;
end case;
      for Child of Internal_Bare_Ada_Node_Array'(Children (Node)) loop
         if Child /= null then
            Assign_Names_To_Logic_Vars (Child);
         end if;
      end loop;
   end Assign_Names_To_Logic_Vars;
   --------------------------
   -- Initialization_Error --
   --------------------------
   function Initialization_Error
     (Exc : Ada.Exceptions.Exception_Occurrence)
      return Error_Initialization_State
   is
      use Ada.Exceptions;
   begin
      if
            Exception_Identity (Exc) = Precondition_Failure'Identity
         then
            return Raised_Precondition_Failure;
         elsif
            Exception_Identity (Exc) = Property_Error'Identity
         then
            return Raised_Property_Error;
      else
         raise Program_Error;
      end if;
   end Initialization_Error;
   ----------------------------------
   -- Reraise_Initialization_Error --
   ----------------------------------
   procedure Reraise_Initialization_Error
     (Node    : Bare_Ada_Node;
      State   : Error_Initialization_State;
      Message : String)
   is
      Exc : Ada.Exceptions.Exception_Id;
   begin
      case State is
            when Raised_Precondition_Failure =>
               Exc := Precondition_Failure'Identity;
            when Raised_Property_Error =>
               Exc := Property_Error'Identity;
      end case;
      Raise_Property_Exception (Node, Exc, Message);
   end Reraise_Initialization_Error;
   ----------------
   -- Text_Image --
   ----------------
   function Text_Image (Ent : Internal_Entity) return Text_Type is
   begin
      if Ent.Node /= null then
         declare
            Node_Image : constant Text_Type := Short_Text_Image (Ent.Node);
         begin
            return
            (if Ent.Info.Rebindings /= null
             then "<| "
             & Node_Image (Node_Image'First + 1 .. Node_Image'Last - 1) & " "
             & AST_Envs.Text_Image (Ent.Info.Rebindings) & " |>"
             else Node_Image);
         end;
      else
         return "None";
      end if;
   end Text_Image;
   ---------------------
   -- Full_Sloc_Image --
   ---------------------
   function Full_Sloc_Image (Node : Bare_Ada_Node) return String_Type
   is
      Res      : constant Text_Type :=
        To_Text
          (Ada.Directories.Simple_Name
             (Get_Filename (Unit (Node))))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node)))) & ": ";
   begin
      return Create_String (Res);
   end Full_Sloc_Image;
   -----------
   -- Image --
   -----------
   function Image (Ent : Internal_Entity) return String is
      Result : constant Text_Type := Text_Image (Ent);
   begin
      return Image (Result);
   end Image;
   ---------------
   -- Can_Reach --
   ---------------
   function Can_Reach (El, From : Bare_Ada_Node) return Boolean is
   begin
      return Ada_Node_P_Can_Reach (El, From);
   end Can_Reach;
   -----------------
   -- Hash_Entity --
   -----------------
   function Hash_Entity (Self : Internal_Entity) return Hash_Type is
   begin
      return Combine
        ((Hash (Self.Node), Hash (Self.Info.Rebindings), Hash (Self.Info.Md)));
   end Hash_Entity;
   --------------------
   -- Compare_Entity --
   --------------------
   function Compare_Entity (Left, Right : Internal_Entity) return Boolean
   is
   begin
      return Left.Node = Right.Node
             and then Left.Info.Rebindings = Right.Info.Rebindings
             and then Compare_Metadata (Left.Info.Md, Right.Info.Md);
   end Compare_Entity;
   --------------------------------
   -- Create_Dynamic_Lexical_Env --
   --------------------------------
   function Create_Dynamic_Lexical_Env
     (Self              : Bare_Ada_Node;
      Assocs_Getter     : Inner_Env_Assocs_Resolver;
      Assoc_Resolver    : Entity_Resolver;
      Transitive_Parent : Boolean;
      Sym_Table         : Symbol_Table) return Lexical_Env
   is
      Unit : constant Internal_Unit := Self.Unit;
   begin
      --  This restriction is necessary to avoid relocation issues when
      --  Self.Self_Env is terminated.
      if Is_Foreign_Strict (Self.Self_Env, Self) then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "cannot create a dynamic lexical env when Self.Self_Env is"
            & " foreign");
      end if;
      return Result : constant Lexical_Env := Create_Dynamic_Lexical_Env
        (Parent            => Null_Lexical_Env,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Owner             => Convert_Unit (Unit),
         Assocs_Getter     => Assocs_Getter,
         Assoc_Resolver    => Assoc_Resolver,
         Sym_Table         => Sym_Table)
      do
         --  Since dynamic lexical environments can only be created in lazy
         --  field initializers, it is fine to tie Result's lifetime to the
         --  its owning unit's lifetime.
         Register_Destroyable (Unit, Unwrap (Result));
      end return;
   end Create_Dynamic_Lexical_Env;
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node);
   --  Helper for the Register_Destroyable above
   ------------
   -- Length --
   ------------
   function Length (Node : Bare_Ada_List) return Natural
   is (if Node = null then 0 else Children_Count (Node));
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (B : Boolean) return String is
      begin
         return (if B then "True" else "False");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (I : Integer) return String is
      begin
         return Integer'Image (I);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : Symbol_Type) return String is
      begin
         return (if S = No_Symbol
                 then "None"
                 else Image (S, With_Quotes => True));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (C : Character_Type) return String is
         C_Str : constant Text_Type := (1 => C);
      begin
         return "'" & Image (C_Str) & "'";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : String_Type) return String is
      begin
         return Image (S.Content, With_Quotes => True);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Env : Lexical_Env) return String is
      begin
         case Env.Kind is
         when Static_Primary =>
            return "<LexicalEnv static-primary for "
                   & Trace_Image (Env_Node (Env)) & ">";
         when others =>
            return "<LexicalEnv synthetic>";
         end case;
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Env_Rebindings) return String is
      begin
         return Image (Text_Image (R));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Unit : Internal_Unit) return String is
      begin
         return "Internal_Unit (""" & Basename (Unit) & """)";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Eq : Logic_Equation) return String is
         pragma Unreferenced (Eq);
      begin
         return "<LogicEquation>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Var : Logic_Var) return String is
         pragma Unreferenced (Var);
      begin
         return "<LogicVariable>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (K : Analysis_Unit_Kind) return String is
      begin
         return Analysis_Unit_Kind'Image (K);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Self : Ref_Categories) return String is
         Result : Unbounded_String;
         First  : Boolean := True;
      begin
         Append (Result, "RefCategories(");
         for C in Ref_Category loop
            if Self (C) then
               if First then
                  First := False;
               else
                  Append (Result, ", ");
               end if;
               Append (Result, C'Image);
            end if;
         end loop;
         Append (Result, ")");
         return To_String (Result);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Metadata) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Dottable_Subp => " & Trace_Image (R.Dottable_Subp)
                        & ", "
                     & "Primitive => " & Trace_Image (R.Primitive)
                        & ", "
                     & "Primitive_Real_Type => " & Trace_Image (R.Primitive_Real_Type)
               & ")");
      end Trace_Image;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Info) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine ((Hash (R.Md), Hash (R.Rebindings), Hash (R.From_Rebound)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Md => " & Trace_Image (R.Md)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "From_Rebound => " & Trace_Image (R.From_Rebound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity
        (Node : Bare_Ada_Node; Info : Internal_Entity_Info)
         return Internal_Entity is
      begin
         if Node = null then
            return No_Entity;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr
        (Node : Bare_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Expr is
      begin
         if Node = null then
            return No_Entity_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Expr) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Aspect) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine ((Hash (R.Exists), Hash (R.Node), Hash (R.Value), Hash (R.Inherited)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Aspect) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Exists => " & Trace_Image (R.Exists)
                        & ", "
                     & "Node => " & Trace_Image (R.Node)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Inherited => " & Trace_Image (R.Inherited)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl
        (Node : Bare_Basic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Basic_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Completion_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Is_Dot_Call => " & Trace_Image (R.Is_Dot_Call)
                        & ", "
                     & "Is_Visible => " & Trace_Image (R.Is_Visible)
                        & ", "
                     & "Weight => " & Trace_Image (R.Weight)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Designated_Env) is
      begin
               Inc_Ref (R.Direct_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Designated_Env) is
      begin
               Dec_Ref (R.Direct_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Designated_Env) return Boolean is
      begin
         return L.Kind = R.Kind and then L.Env_Name = R.Env_Name and then Equivalent (L.Direct_Env, R.Direct_Env);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Designated_Env) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Kind => " & Trace_Image (R.Kind)
                        & ", "
                     & "Env_Name => " & Trace_Image (R.Env_Name)
                        & ", "
                     & "Direct_Env => " & Trace_Image (R.Direct_Env)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Ada_List
        (Node : Bare_Ada_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_List is
      begin
         if Node = null then
            return No_Entity_Ada_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ada_Node_List
        (Node : Bare_Ada_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_Node_List is
      begin
         if Node = null then
            return No_Entity_Ada_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Alternatives_List
        (Node : Bare_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name
        (Node : Bare_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Name is
      begin
         if Node = null then
            return No_Entity_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Tok_Node
        (Node : Bare_Single_Tok_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Tok_Node is
      begin
         if Node = null then
            return No_Entity_Single_Tok_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Tok_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Id
        (Node : Bare_Base_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Id is
      begin
         if Node = null then
            return No_Entity_Base_Id;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier
        (Node : Bare_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier is
      begin
         if Node = null then
            return No_Entity_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discriminant_Values) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Discriminant => " & Trace_Image (R.Discriminant)
                        & ", "
                     & "Values => " & Trace_Image (R.Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Doc_Annotation) is
      begin
               Inc_Ref (R.Key);
               Inc_Ref (R.Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Doc_Annotation) is
      begin
               Dec_Ref (R.Key);
               Dec_Ref (R.Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Doc_Annotation) return Boolean is
      begin
         return Equivalent (L.Key, R.Key) and then Equivalent (L.Value, R.Value);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Doc_Annotation) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Abort_Node
        (Node : Bare_Abort_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Node is
      begin
         if Node = null then
            return No_Entity_Abort_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Absent
        (Node : Bare_Abort_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Absent is
      begin
         if Node = null then
            return No_Entity_Abort_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Present
        (Node : Bare_Abort_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Present is
      begin
         if Node = null then
            return No_Entity_Abort_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt
        (Node : Bare_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt is
      begin
         if Node = null then
            return No_Entity_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Simple_Stmt
        (Node : Bare_Simple_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Stmt is
      begin
         if Node = null then
            return No_Entity_Simple_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Simple_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Stmt
        (Node : Bare_Abort_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Stmt is
      begin
         if Node = null then
            return No_Entity_Abort_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Node
        (Node : Bare_Abstract_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Node is
      begin
         if Node = null then
            return No_Entity_Abstract_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Absent
        (Node : Bare_Abstract_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Absent is
      begin
         if Node = null then
            return No_Entity_Abstract_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Subp_Decl
        (Node : Bare_Basic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classic_Subp_Decl
        (Node : Bare_Classic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Classic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Subp_Decl
        (Node : Bare_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Formal_Subp_Decl
        (Node : Bare_Abstract_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Present
        (Node : Bare_Abstract_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Present is
      begin
         if Node = null then
            return No_Entity_Abstract_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl
        (Node : Bare_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_Expr
        (Node : Bare_Abstract_State_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_List
        (Node : Bare_Abstract_State_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_List is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Subp_Decl
        (Node : Bare_Abstract_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Stmt
        (Node : Bare_Composite_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Stmt is
      begin
         if Node = null then
            return No_Entity_Composite_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt
        (Node : Bare_Accept_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Node
        (Node : Bare_Body_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Node is
      begin
         if Node = null then
            return No_Entity_Body_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_Body
        (Node : Bare_Accept_Stmt_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_Body is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node : Bare_Accept_Stmt_With_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_With_Stmts is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_With_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt_With_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Def
        (Node : Bare_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Def is
      begin
         if Node = null then
            return No_Entity_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_Def
        (Node : Bare_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_Def is
      begin
         if Node = null then
            return No_Entity_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_To_Subp_Def
        (Node : Bare_Access_To_Subp_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_To_Subp_Def is
      begin
         if Node = null then
            return No_Entity_Access_To_Subp_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_To_Subp_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Aggregate
        (Node : Bare_Base_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Aggregate is
      begin
         if Node = null then
            return No_Entity_Base_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate
        (Node : Bare_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate is
      begin
         if Node = null then
            return No_Entity_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc
        (Node : Bare_Basic_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate_Assoc
        (Node : Bare_Aggregate_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate_Assoc is
      begin
         if Node = null then
            return No_Entity_Aggregate_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Node
        (Node : Bare_Aliased_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Node is
      begin
         if Node = null then
            return No_Entity_Aliased_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Absent
        (Node : Bare_Aliased_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Absent is
      begin
         if Node = null then
            return No_Entity_Aliased_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Present
        (Node : Bare_Aliased_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Present is
      begin
         if Node = null then
            return No_Entity_Aliased_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Node
        (Node : Bare_All_Node; Info : Internal_Entity_Info)
         return Internal_Entity_All_Node is
      begin
         if Node = null then
            return No_Entity_All_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Absent
        (Node : Bare_All_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_All_Absent is
      begin
         if Node = null then
            return No_Entity_All_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Present
        (Node : Bare_All_Present; Info : Internal_Entity_Info)
         return Internal_Entity_All_Present is
      begin
         if Node = null then
            return No_Entity_All_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Allocator
        (Node : Bare_Allocator; Info : Internal_Entity_Info)
         return Internal_Entity_Allocator is
      begin
         if Node = null then
            return No_Entity_Allocator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Allocator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Expr_Decl
        (Node : Bare_Anonymous_Expr_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Expr_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Expr_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Expr_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Expr
        (Node : Bare_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type
        (Node : Bare_Anonymous_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Access_Def
        (Node : Bare_Base_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Base_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Access_Def
        (Node : Bare_Anonymous_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Decl
        (Node : Bare_Base_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Base_Type_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Decl
        (Node : Bare_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Decl
        (Node : Bare_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Indices
        (Node : Bare_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Subcomponent_Choice_Name
        (Node : Bare_Array_Subcomponent_Choice_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Subcomponent_Choice_Name is
      begin
         if Node = null then
            return No_Entity_Array_Subcomponent_Choice_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Subcomponent_Choice_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Type_Def
        (Node : Bare_Array_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Type_Def is
      begin
         if Node = null then
            return No_Entity_Array_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc
        (Node : Bare_Aspect_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc_List
        (Node : Bare_Aspect_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Clause
        (Node : Bare_Aspect_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Clause is
      begin
         if Node = null then
            return No_Entity_Aspect_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Spec
        (Node : Bare_Aspect_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Spec is
      begin
         if Node = null then
            return No_Entity_Aspect_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assign_Stmt
        (Node : Bare_Assign_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Assign_Stmt is
      begin
         if Node = null then
            return No_Entity_Assign_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assign_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc_List
        (Node : Bare_Basic_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assoc_List
        (Node : Bare_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_At_Clause
        (Node : Bare_At_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_At_Clause is
      begin
         if Node = null then
            return No_Entity_At_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_At_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Def_Clause
        (Node : Bare_Attribute_Def_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Def_Clause is
      begin
         if Node = null then
            return No_Entity_Attribute_Def_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Def_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Ref
        (Node : Bare_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc
        (Node : Bare_Base_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc is
      begin
         if Node = null then
            return No_Entity_Base_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc_List
        (Node : Bare_Base_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Base_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Decl
        (Node : Bare_Base_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Holder
        (Node : Bare_Base_Formal_Param_Holder; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Holder is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Holder;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Holder) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Loop_Stmt
        (Node : Bare_Base_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Base_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Package_Decl
        (Node : Bare_Base_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Record_Def
        (Node : Bare_Base_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Record_Def is
      begin
         if Node = null then
            return No_Entity_Base_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Body
        (Node : Bare_Base_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Spec
        (Node : Bare_Base_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subtype_Decl
        (Node : Bare_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Block_Stmt
        (Node : Bare_Block_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Stmt is
      begin
         if Node = null then
            return No_Entity_Block_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Begin_Block
        (Node : Bare_Begin_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Begin_Block is
      begin
         if Node = null then
            return No_Entity_Begin_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Begin_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bin_Op
        (Node : Bare_Bin_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Bin_Op is
      begin
         if Node = null then
            return No_Entity_Bin_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bin_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Stub
        (Node : Bare_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Box_Expr
        (Node : Bare_Box_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Box_Expr is
      begin
         if Node = null then
            return No_Entity_Box_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Box_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Aggregate
        (Node : Bare_Bracket_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Aggregate
        (Node : Bare_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Delta_Aggregate
        (Node : Bare_Bracket_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Expr
        (Node : Bare_Call_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr is
      begin
         if Node = null then
            return No_Entity_Call_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Stmt
        (Node : Bare_Call_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Stmt is
      begin
         if Node = null then
            return No_Entity_Call_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Cond_Expr
        (Node : Bare_Cond_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Cond_Expr is
      begin
         if Node = null then
            return No_Entity_Cond_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Cond_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr
        (Node : Bare_Case_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr is
      begin
         if Node = null then
            return No_Entity_Case_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative
        (Node : Bare_Case_Expr_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative_List
        (Node : Bare_Case_Expr_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt
        (Node : Bare_Case_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt is
      begin
         if Node = null then
            return No_Entity_Case_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative
        (Node : Bare_Case_Stmt_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative_List
        (Node : Bare_Case_Stmt_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Char_Literal
        (Node : Bare_Char_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Char_Literal is
      begin
         if Node = null then
            return No_Entity_Char_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Char_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classwide_Type_Decl
        (Node : Bare_Classwide_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classwide_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Classwide_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classwide_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit
        (Node : Bare_Compilation_Unit; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Compilation_Unit) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit_List
        (Node : Bare_Compilation_Unit_List; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit_List is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Clause
        (Node : Bare_Component_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Clause is
      begin
         if Node = null then
            return No_Entity_Component_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Decl
        (Node : Bare_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Def
        (Node : Bare_Component_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Def is
      begin
         if Node = null then
            return No_Entity_Component_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_List
        (Node : Bare_Component_List; Info : Internal_Entity_Info)
         return Internal_Entity_Component_List is
      begin
         if Node = null then
            return No_Entity_Component_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint
        (Node : Bare_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint is
      begin
         if Node = null then
            return No_Entity_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Constraint
        (Node : Bare_Composite_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Constraint is
      begin
         if Node = null then
            return No_Entity_Composite_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Constraint_Assoc
        (Node : Bare_Composite_Constraint_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Constraint_Assoc is
      begin
         if Node = null then
            return No_Entity_Composite_Constraint_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Constraint_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Op
        (Node : Bare_Concat_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Op is
      begin
         if Node = null then
            return No_Entity_Concat_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Operand
        (Node : Bare_Concat_Operand; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Operand is
      begin
         if Node = null then
            return No_Entity_Concat_Operand;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Operand) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concat_Operand_List
        (Node : Bare_Concat_Operand_List; Info : Internal_Entity_Info)
         return Internal_Entity_Concat_Operand_List is
      begin
         if Node = null then
            return No_Entity_Concat_Operand_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concat_Operand_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Formal_Subp_Decl
        (Node : Bare_Concrete_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concrete_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Type_Decl
        (Node : Bare_Concrete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concrete_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Node
        (Node : Bare_Constant_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Node is
      begin
         if Node = null then
            return No_Entity_Constant_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Absent
        (Node : Bare_Constant_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Absent is
      begin
         if Node = null then
            return No_Entity_Constant_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Present
        (Node : Bare_Constant_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Present is
      begin
         if Node = null then
            return No_Entity_Constant_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Array_Indices
        (Node : Bare_Constrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Constrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Indication
        (Node : Bare_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Subtype_Indication
        (Node : Bare_Constrained_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Constrained_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint_List
        (Node : Bare_Constraint_List; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint_List is
      begin
         if Node = null then
            return No_Entity_Constraint_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc
        (Node : Bare_Contract_Case_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc_List
        (Node : Bare_Contract_Case_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Cases
        (Node : Bare_Contract_Cases; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Cases is
      begin
         if Node = null then
            return No_Entity_Contract_Cases;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Cases) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Type_Def
        (Node : Bare_Real_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Type_Def is
      begin
         if Node = null then
            return No_Entity_Real_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decimal_Fixed_Point_Def
        (Node : Bare_Decimal_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Decimal_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Decimal_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decimal_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Block
        (Node : Bare_Decl_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Block is
      begin
         if Node = null then
            return No_Entity_Decl_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Expr
        (Node : Bare_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_List
        (Node : Bare_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_List is
      begin
         if Node = null then
            return No_Entity_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Declarative_Part
        (Node : Bare_Declarative_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Declarative_Part is
      begin
         if Node = null then
            return No_Entity_Declarative_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Declarative_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name
        (Node : Bare_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name_List
        (Node : Bare_Defining_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name_List is
      begin
         if Node = null then
            return No_Entity_Defining_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delay_Stmt
        (Node : Bare_Delay_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Delay_Stmt is
      begin
         if Node = null then
            return No_Entity_Delay_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delay_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Constraint
        (Node : Bare_Delta_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Constraint is
      begin
         if Node = null then
            return No_Entity_Delta_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Derived_Type_Def
        (Node : Bare_Derived_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Derived_Type_Def is
      begin
         if Node = null then
            return No_Entity_Derived_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Derived_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Digits_Constraint
        (Node : Bare_Digits_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Digits_Constraint is
      begin
         if Node = null then
            return No_Entity_Digits_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Digits_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node : Bare_Discrete_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Discrete_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Indication
        (Node : Bare_Discrete_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Name
        (Node : Bare_Discrete_Subtype_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Name is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier_List
        (Node : Bare_Identifier_List; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier_List is
      begin
         if Node = null then
            return No_Entity_Identifier_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Choice_List
        (Node : Bare_Discriminant_Choice_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Choice_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Choice_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Choice_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Part
        (Node : Bare_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec
        (Node : Bare_Discriminant_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec_List
        (Node : Bare_Discriminant_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Dotted_Name
        (Node : Bare_Dotted_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Dotted_Name is
      begin
         if Node = null then
            return No_Entity_Dotted_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Dotted_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part
        (Node : Bare_Elsif_Expr_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part_List
        (Node : Bare_Elsif_Expr_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part
        (Node : Bare_Elsif_Stmt_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part_List
        (Node : Bare_Elsif_Stmt_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_End_Name
        (Node : Bare_End_Name; Info : Internal_Entity_Info)
         return Internal_Entity_End_Name is
      begin
         if Node = null then
            return No_Entity_End_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_End_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Body
        (Node : Bare_Entry_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Body is
      begin
         if Node = null then
            return No_Entity_Entry_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Completion_Formal_Params
        (Node : Bare_Entry_Completion_Formal_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Completion_Formal_Params is
      begin
         if Node = null then
            return No_Entity_Entry_Completion_Formal_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Completion_Formal_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Decl
        (Node : Bare_Entry_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Decl is
      begin
         if Node = null then
            return No_Entity_Entry_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Index_Spec
        (Node : Bare_Entry_Index_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Index_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Index_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Index_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Spec
        (Node : Bare_Entry_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Lit_Synth_Type_Expr
        (Node : Bare_Enum_Lit_Synth_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Synth_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Synth_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Lit_Synth_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl
        (Node : Bare_Enum_Literal_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl_List
        (Node : Bare_Enum_Literal_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Rep_Clause
        (Node : Bare_Enum_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Enum_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Subp_Spec
        (Node : Bare_Enum_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Enum_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Type_Def
        (Node : Bare_Enum_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Type_Def is
      begin
         if Node = null then
            return No_Entity_Enum_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Decl
        (Node : Bare_Error_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Decl is
      begin
         if Node = null then
            return No_Entity_Error_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Stmt
        (Node : Bare_Error_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Stmt is
      begin
         if Node = null then
            return No_Entity_Error_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Decl
        (Node : Bare_Exception_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Decl is
      begin
         if Node = null then
            return No_Entity_Exception_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Handler
        (Node : Bare_Exception_Handler; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Handler is
      begin
         if Node = null then
            return No_Entity_Exception_Handler;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Handler) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exit_Stmt
        (Node : Bare_Exit_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Exit_Stmt is
      begin
         if Node = null then
            return No_Entity_Exit_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exit_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Explicit_Deref
        (Node : Bare_Explicit_Deref; Info : Internal_Entity_Info)
         return Internal_Entity_Explicit_Deref is
      begin
         if Node = null then
            return No_Entity_Explicit_Deref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Explicit_Deref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_List
        (Node : Bare_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_List is
      begin
         if Node = null then
            return No_Entity_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Alternatives_List
        (Node : Bare_Expr_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Expr_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Function
        (Node : Bare_Expr_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Function is
      begin
         if Node = null then
            return No_Entity_Expr_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt
        (Node : Bare_Extended_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Object_Decl
        (Node : Bare_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt_Object_Decl
        (Node : Bare_Extended_Return_Stmt_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Floating_Point_Def
        (Node : Bare_Floating_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Floating_Point_Def is
      begin
         if Node = null then
            return No_Entity_Floating_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Floating_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Iter_Filter
        (Node : Bare_For_Loop_Iter_Filter; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Iter_Filter is
      begin
         if Node = null then
            return No_Entity_For_Loop_Iter_Filter;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Iter_Filter) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Spec
        (Node : Bare_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Spec
        (Node : Bare_For_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_For_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Stmt
        (Node : Bare_For_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_For_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Var_Decl
        (Node : Bare_For_Loop_Var_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Var_Decl is
      begin
         if Node = null then
            return No_Entity_For_Loop_Var_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Var_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Discrete_Type_Def
        (Node : Bare_Formal_Discrete_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Discrete_Type_Def is
      begin
         if Node = null then
            return No_Entity_Formal_Discrete_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Discrete_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Type_Decl
        (Node : Bare_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Chunk
        (Node : Bare_Format_String_Chunk; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Chunk is
      begin
         if Node = null then
            return No_Entity_Format_String_Chunk;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Chunk) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Chunk_List
        (Node : Bare_Format_String_Chunk_List; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Chunk_List is
      begin
         if Node = null then
            return No_Entity_Format_String_Chunk_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Chunk_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Literal
        (Node : Bare_Format_String_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Literal is
      begin
         if Node = null then
            return No_Entity_Format_String_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Tok_Node
        (Node : Bare_Format_String_Tok_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Tok_Node is
      begin
         if Node = null then
            return No_Entity_Format_String_Tok_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Tok_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Tok_End
        (Node : Bare_Format_String_Tok_End; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Tok_End is
      begin
         if Node = null then
            return No_Entity_Format_String_Tok_End;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Tok_End) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Tok_Mid
        (Node : Bare_Format_String_Tok_Mid; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Tok_Mid is
      begin
         if Node = null then
            return No_Entity_Format_String_Tok_Mid;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Tok_Mid) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Tok_Start
        (Node : Bare_Format_String_Tok_Start; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Tok_Start is
      begin
         if Node = null then
            return No_Entity_Format_String_Tok_Start;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Tok_Start) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Format_String_Tok_String
        (Node : Bare_Format_String_Tok_String; Info : Internal_Entity_Info)
         return Internal_Entity_Format_String_Tok_String is
      begin
         if Node = null then
            return No_Entity_Format_String_Tok_String;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Format_String_Tok_String) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Decl
        (Node : Bare_Generic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal
        (Node : Bare_Generic_Formal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal is
      begin
         if Node = null then
            return No_Entity_Generic_Formal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node : Bare_Generic_Formal_Obj_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Obj_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Obj_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Obj_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Package
        (Node : Bare_Generic_Formal_Package; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Package is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Package;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Package) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Part
        (Node : Bare_Generic_Formal_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Part is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node : Bare_Generic_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node : Bare_Generic_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Instantiation
        (Node : Bare_Generic_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Decl
        (Node : Bare_Generic_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Instantiation
        (Node : Bare_Generic_Package_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Internal
        (Node : Bare_Generic_Package_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Renaming_Decl
        (Node : Bare_Generic_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Renaming_Decl
        (Node : Bare_Generic_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Decl
        (Node : Bare_Generic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Instantiation
        (Node : Bare_Generic_Subp_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Internal
        (Node : Bare_Generic_Subp_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node : Bare_Generic_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Goto_Stmt
        (Node : Bare_Goto_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Goto_Stmt is
      begin
         if Node = null then
            return No_Entity_Goto_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Goto_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Handled_Stmts
        (Node : Bare_Handled_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Handled_Stmts is
      begin
         if Node = null then
            return No_Entity_Handled_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Handled_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Expr
        (Node : Bare_If_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_If_Expr is
      begin
         if Node = null then
            return No_Entity_If_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Stmt
        (Node : Bare_If_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_If_Stmt is
      begin
         if Node = null then
            return No_Entity_If_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Type_Decl
        (Node : Bare_Incomplete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Formal_Type_Decl
        (Node : Bare_Incomplete_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Tagged_Type_Decl
        (Node : Bare_Incomplete_Tagged_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Tagged_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Tagged_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Tagged_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Num_Literal
        (Node : Bare_Num_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Num_Literal is
      begin
         if Node = null then
            return No_Entity_Num_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Num_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Int_Literal
        (Node : Bare_Int_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Int_Literal is
      begin
         if Node = null then
            return No_Entity_Int_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Int_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind
        (Node : Bare_Interface_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind is
      begin
         if Node = null then
            return No_Entity_Interface_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Limited
        (Node : Bare_Interface_Kind_Limited; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Limited is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Limited;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Limited) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Protected
        (Node : Bare_Interface_Kind_Protected; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Protected is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Protected;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Protected) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Synchronized
        (Node : Bare_Interface_Kind_Synchronized; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Synchronized is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Synchronized;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Synchronized) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Task
        (Node : Bare_Interface_Kind_Task; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Task is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Task;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Task) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Type_Def
        (Node : Bare_Interface_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Type_Def is
      begin
         if Node = null then
            return No_Entity_Interface_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type
        (Node : Bare_Iter_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type is
      begin
         if Node = null then
            return No_Entity_Iter_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_In
        (Node : Bare_Iter_Type_In; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_In is
      begin
         if Node = null then
            return No_Entity_Iter_Type_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_Of
        (Node : Bare_Iter_Type_Of; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_Of is
      begin
         if Node = null then
            return No_Entity_Iter_Type_Of;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_Of) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iterated_Assoc
        (Node : Bare_Iterated_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Iterated_Assoc is
      begin
         if Node = null then
            return No_Entity_Iterated_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iterated_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Known_Discriminant_Part
        (Node : Bare_Known_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Known_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Known_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Known_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label
        (Node : Bare_Label; Info : Internal_Entity_Info)
         return Internal_Entity_Label is
      begin
         if Node = null then
            return No_Entity_Label;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label_Decl
        (Node : Bare_Label_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Label_Decl is
      begin
         if Node = null then
            return No_Entity_Label_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Library_Item
        (Node : Bare_Library_Item; Info : Internal_Entity_Info)
         return Internal_Entity_Library_Item is
      begin
         if Node = null then
            return No_Entity_Library_Item;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Library_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Node
        (Node : Bare_Limited_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Node is
      begin
         if Node = null then
            return No_Entity_Limited_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Absent
        (Node : Bare_Limited_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Absent is
      begin
         if Node = null then
            return No_Entity_Limited_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Present
        (Node : Bare_Limited_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Present is
      begin
         if Node = null then
            return No_Entity_Limited_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Stmt
        (Node : Bare_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Membership_Expr
        (Node : Bare_Membership_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Membership_Expr is
      begin
         if Node = null then
            return No_Entity_Membership_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Membership_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mod_Int_Type_Def
        (Node : Bare_Mod_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Mod_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Mod_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mod_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode
        (Node : Bare_Mode; Info : Internal_Entity_Info)
         return Internal_Entity_Mode is
      begin
         if Node = null then
            return No_Entity_Mode;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Default
        (Node : Bare_Mode_Default; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Default is
      begin
         if Node = null then
            return No_Entity_Mode_Default;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Default) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In
        (Node : Bare_Mode_In; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In is
      begin
         if Node = null then
            return No_Entity_Mode_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In_Out
        (Node : Bare_Mode_In_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In_Out is
      begin
         if Node = null then
            return No_Entity_Mode_In_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Out
        (Node : Bare_Mode_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Out is
      begin
         if Node = null then
            return No_Entity_Mode_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Abstract_State_Decl
        (Node : Bare_Multi_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Multi_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Dim_Array_Assoc
        (Node : Bare_Multi_Dim_Array_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Dim_Array_Assoc is
      begin
         if Node = null then
            return No_Entity_Multi_Dim_Array_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Dim_Array_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name_List
        (Node : Bare_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Name_List is
      begin
         if Node = null then
            return No_Entity_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt
        (Node : Bare_Named_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt is
      begin
         if Node = null then
            return No_Entity_Named_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt_Decl
        (Node : Bare_Named_Stmt_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt_Decl is
      begin
         if Node = null then
            return No_Entity_Named_Stmt_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_No_Type_Object_Renaming_Decl
        (Node : Bare_No_Type_Object_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_No_Type_Object_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_No_Type_Object_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_No_Type_Object_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null
        (Node : Bare_Not_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null is
      begin
         if Node = null then
            return No_Entity_Not_Null;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Absent
        (Node : Bare_Not_Null_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Absent is
      begin
         if Node = null then
            return No_Entity_Not_Null_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Present
        (Node : Bare_Not_Null_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Present is
      begin
         if Node = null then
            return No_Entity_Not_Null_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Component_Decl
        (Node : Bare_Null_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Literal
        (Node : Bare_Null_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Literal is
      begin
         if Node = null then
            return No_Entity_Null_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Aggregate
        (Node : Bare_Null_Record_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Aggregate is
      begin
         if Node = null then
            return No_Entity_Null_Record_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Def
        (Node : Bare_Null_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Def is
      begin
         if Node = null then
            return No_Entity_Null_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Stmt
        (Node : Bare_Null_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Stmt is
      begin
         if Node = null then
            return No_Entity_Null_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Subp_Decl
        (Node : Bare_Null_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Number_Decl
        (Node : Bare_Number_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Number_Decl is
      begin
         if Node = null then
            return No_Entity_Number_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Number_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op
        (Node : Bare_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Op is
      begin
         if Node = null then
            return No_Entity_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Abs
        (Node : Bare_Op_Abs; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Abs is
      begin
         if Node = null then
            return No_Entity_Op_Abs;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Abs) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And
        (Node : Bare_Op_And; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And is
      begin
         if Node = null then
            return No_Entity_Op_And;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And_Then
        (Node : Bare_Op_And_Then; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And_Then is
      begin
         if Node = null then
            return No_Entity_Op_And_Then;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And_Then) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Concat
        (Node : Bare_Op_Concat; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Concat is
      begin
         if Node = null then
            return No_Entity_Op_Concat;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Concat) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Div
        (Node : Bare_Op_Div; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Div is
      begin
         if Node = null then
            return No_Entity_Op_Div;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Div) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Double_Dot
        (Node : Bare_Op_Double_Dot; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Double_Dot is
      begin
         if Node = null then
            return No_Entity_Op_Double_Dot;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Double_Dot) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Eq
        (Node : Bare_Op_Eq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Eq is
      begin
         if Node = null then
            return No_Entity_Op_Eq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Eq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gt
        (Node : Bare_Op_Gt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gt is
      begin
         if Node = null then
            return No_Entity_Op_Gt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gte
        (Node : Bare_Op_Gte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gte is
      begin
         if Node = null then
            return No_Entity_Op_Gte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_In
        (Node : Bare_Op_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_In is
      begin
         if Node = null then
            return No_Entity_Op_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lt
        (Node : Bare_Op_Lt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lt is
      begin
         if Node = null then
            return No_Entity_Op_Lt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lte
        (Node : Bare_Op_Lte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lte is
      begin
         if Node = null then
            return No_Entity_Op_Lte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Minus
        (Node : Bare_Op_Minus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Minus is
      begin
         if Node = null then
            return No_Entity_Op_Minus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Minus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mod
        (Node : Bare_Op_Mod; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mod is
      begin
         if Node = null then
            return No_Entity_Op_Mod;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mod) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mult
        (Node : Bare_Op_Mult; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mult is
      begin
         if Node = null then
            return No_Entity_Op_Mult;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mult) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Neq
        (Node : Bare_Op_Neq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Neq is
      begin
         if Node = null then
            return No_Entity_Op_Neq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Neq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not
        (Node : Bare_Op_Not; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not is
      begin
         if Node = null then
            return No_Entity_Op_Not;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not_In
        (Node : Bare_Op_Not_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not_In is
      begin
         if Node = null then
            return No_Entity_Op_Not_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or
        (Node : Bare_Op_Or; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or is
      begin
         if Node = null then
            return No_Entity_Op_Or;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or_Else
        (Node : Bare_Op_Or_Else; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or_Else is
      begin
         if Node = null then
            return No_Entity_Op_Or_Else;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or_Else) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Plus
        (Node : Bare_Op_Plus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Plus is
      begin
         if Node = null then
            return No_Entity_Op_Plus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Plus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Pow
        (Node : Bare_Op_Pow; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Pow is
      begin
         if Node = null then
            return No_Entity_Op_Pow;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Pow) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Rem
        (Node : Bare_Op_Rem; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Rem is
      begin
         if Node = null then
            return No_Entity_Op_Rem;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Rem) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Xor
        (Node : Bare_Op_Xor; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Xor is
      begin
         if Node = null then
            return No_Entity_Op_Xor;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Xor) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ordinary_Fixed_Point_Def
        (Node : Bare_Ordinary_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Ordinary_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Ordinary_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ordinary_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Others_Designator
        (Node : Bare_Others_Designator; Info : Internal_Entity_Info)
         return Internal_Entity_Others_Designator is
      begin
         if Node = null then
            return No_Entity_Others_Designator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Others_Designator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Node
        (Node : Bare_Overriding_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Node is
      begin
         if Node = null then
            return No_Entity_Overriding_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Not_Overriding
        (Node : Bare_Overriding_Not_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Not_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Not_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Not_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Overriding
        (Node : Bare_Overriding_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Unspecified
        (Node : Bare_Overriding_Unspecified; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Unspecified is
      begin
         if Node = null then
            return No_Entity_Overriding_Unspecified;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Unspecified) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body
        (Node : Bare_Package_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body is
      begin
         if Node = null then
            return No_Entity_Package_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body_Stub
        (Node : Bare_Package_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Package_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Decl
        (Node : Bare_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Renaming_Decl
        (Node : Bare_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Assoc
        (Node : Bare_Param_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Assoc is
      begin
         if Node = null then
            return No_Entity_Param_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec
        (Node : Bare_Param_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec is
      begin
         if Node = null then
            return No_Entity_Param_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec_List
        (Node : Bare_Param_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec_List is
      begin
         if Node = null then
            return No_Entity_Param_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Params
        (Node : Bare_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Params is
      begin
         if Node = null then
            return No_Entity_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Abstract_State_Decl
        (Node : Bare_Paren_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Paren_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Expr
        (Node : Bare_Paren_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Expr is
      begin
         if Node = null then
            return No_Entity_Paren_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Parent_List
        (Node : Bare_Parent_List; Info : Internal_Entity_Info)
         return Internal_Entity_Parent_List is
      begin
         if Node = null then
            return No_Entity_Parent_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Parent_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Directive
        (Node : Bare_Pp_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Else_Directive
        (Node : Bare_Pp_Else_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Else_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Else_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Else_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Elsif_Directive
        (Node : Bare_Pp_Elsif_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Elsif_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_Elsif_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Elsif_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_End_If_Directive
        (Node : Bare_Pp_End_If_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_End_If_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_End_If_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_End_If_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_If_Directive
        (Node : Bare_Pp_If_Directive; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_If_Directive is
      begin
         if Node = null then
            return No_Entity_Pp_If_Directive;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_If_Directive) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pp_Then_Kw
        (Node : Bare_Pp_Then_Kw; Info : Internal_Entity_Info)
         return Internal_Entity_Pp_Then_Kw is
      begin
         if Node = null then
            return No_Entity_Pp_Then_Kw;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pp_Then_Kw) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Argument_Assoc
        (Node : Bare_Pragma_Argument_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Argument_Assoc is
      begin
         if Node = null then
            return No_Entity_Pragma_Argument_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Argument_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node
        (Node : Bare_Pragma_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node is
      begin
         if Node = null then
            return No_Entity_Pragma_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node_List
        (Node : Bare_Pragma_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node_List is
      begin
         if Node = null then
            return No_Entity_Pragma_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Node
        (Node : Bare_Private_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Node is
      begin
         if Node = null then
            return No_Entity_Private_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Absent
        (Node : Bare_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Absent is
      begin
         if Node = null then
            return No_Entity_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Part
        (Node : Bare_Private_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Part is
      begin
         if Node = null then
            return No_Entity_Private_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Present
        (Node : Bare_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Present is
      begin
         if Node = null then
            return No_Entity_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Type_Def
        (Node : Bare_Private_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Type_Def is
      begin
         if Node = null then
            return No_Entity_Private_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Node
        (Node : Bare_Protected_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Node is
      begin
         if Node = null then
            return No_Entity_Protected_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Absent
        (Node : Bare_Protected_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Absent is
      begin
         if Node = null then
            return No_Entity_Protected_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body
        (Node : Bare_Protected_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body is
      begin
         if Node = null then
            return No_Entity_Protected_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body_Stub
        (Node : Bare_Protected_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Protected_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Def
        (Node : Bare_Protected_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Def is
      begin
         if Node = null then
            return No_Entity_Protected_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Present
        (Node : Bare_Protected_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Present is
      begin
         if Node = null then
            return No_Entity_Protected_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Type_Decl
        (Node : Bare_Protected_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Protected_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Public_Part
        (Node : Bare_Public_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Public_Part is
      begin
         if Node = null then
            return No_Entity_Public_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Public_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Qual_Expr
        (Node : Bare_Qual_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Qual_Expr is
      begin
         if Node = null then
            return No_Entity_Qual_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Qual_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantified_Expr
        (Node : Bare_Quantified_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Quantified_Expr is
      begin
         if Node = null then
            return No_Entity_Quantified_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantified_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier
        (Node : Bare_Quantifier; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier is
      begin
         if Node = null then
            return No_Entity_Quantifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_All
        (Node : Bare_Quantifier_All; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_All is
      begin
         if Node = null then
            return No_Entity_Quantifier_All;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_All) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_Some
        (Node : Bare_Quantifier_Some; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_Some is
      begin
         if Node = null then
            return No_Entity_Quantifier_Some;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_Some) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Expr
        (Node : Bare_Raise_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Expr is
      begin
         if Node = null then
            return No_Entity_Raise_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Stmt
        (Node : Bare_Raise_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Stmt is
      begin
         if Node = null then
            return No_Entity_Raise_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Constraint
        (Node : Bare_Range_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Constraint is
      begin
         if Node = null then
            return No_Entity_Range_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Spec
        (Node : Bare_Range_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Spec is
      begin
         if Node = null then
            return No_Entity_Range_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Literal
        (Node : Bare_Real_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Literal is
      begin
         if Node = null then
            return No_Entity_Real_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Def
        (Node : Bare_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Def is
      begin
         if Node = null then
            return No_Entity_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Rep_Clause
        (Node : Bare_Record_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Record_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Type_Def
        (Node : Bare_Record_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Type_Def is
      begin
         if Node = null then
            return No_Entity_Record_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reduce_Attribute_Ref
        (Node : Bare_Reduce_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Reduce_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Reduce_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reduce_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Relation_Op
        (Node : Bare_Relation_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Relation_Op is
      begin
         if Node = null then
            return No_Entity_Relation_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Relation_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Renaming_Clause
        (Node : Bare_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Requeue_Stmt
        (Node : Bare_Requeue_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Requeue_Stmt is
      begin
         if Node = null then
            return No_Entity_Requeue_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Requeue_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Return_Stmt
        (Node : Bare_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Node
        (Node : Bare_Reverse_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Node is
      begin
         if Node = null then
            return No_Entity_Reverse_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Absent
        (Node : Bare_Reverse_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Absent is
      begin
         if Node = null then
            return No_Entity_Reverse_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Present
        (Node : Bare_Reverse_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Present is
      begin
         if Node = null then
            return No_Entity_Reverse_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_Stmt
        (Node : Bare_Select_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Select_Stmt is
      begin
         if Node = null then
            return No_Entity_Select_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part
        (Node : Bare_Select_When_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part is
      begin
         if Node = null then
            return No_Entity_Select_When_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part_List
        (Node : Bare_Select_When_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part_List is
      begin
         if Node = null then
            return No_Entity_Select_When_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Signed_Int_Type_Def
        (Node : Bare_Signed_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Signed_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Signed_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Signed_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Simple_Decl_Stmt
        (Node : Bare_Simple_Decl_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Decl_Stmt is
      begin
         if Node = null then
            return No_Entity_Simple_Decl_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Simple_Decl_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Protected_Decl
        (Node : Bare_Single_Protected_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Protected_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Protected_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Protected_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Decl
        (Node : Bare_Single_Task_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Type_Decl
        (Node : Bare_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Type_Decl
        (Node : Bare_Single_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt_List
        (Node : Bare_Stmt_List; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt_List is
      begin
         if Node = null then
            return No_Entity_Stmt_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_String_Literal
        (Node : Bare_String_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_String_Literal is
      begin
         if Node = null then
            return No_Entity_String_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_String_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body
        (Node : Bare_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body_Stub
        (Node : Bare_Subp_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Subp_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Decl
        (Node : Bare_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind
        (Node : Bare_Subp_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind is
      begin
         if Node = null then
            return No_Entity_Subp_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Function
        (Node : Bare_Subp_Kind_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Function is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Procedure
        (Node : Bare_Subp_Kind_Procedure; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Procedure is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Procedure;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Procedure) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Renaming_Decl
        (Node : Bare_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Spec
        (Node : Bare_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Decl
        (Node : Bare_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subunit
        (Node : Bare_Subunit; Info : Internal_Entity_Info)
         return Internal_Entity_Subunit is
      begin
         if Node = null then
            return No_Entity_Subunit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subunit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Node
        (Node : Bare_Synchronized_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Node is
      begin
         if Node = null then
            return No_Entity_Synchronized_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Absent
        (Node : Bare_Synchronized_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Absent is
      begin
         if Node = null then
            return No_Entity_Synchronized_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Present
        (Node : Bare_Synchronized_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Present is
      begin
         if Node = null then
            return No_Entity_Synchronized_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synth_Anonymous_Type_Decl
        (Node : Bare_Synth_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synth_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Synth_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synth_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Binary_Spec
        (Node : Bare_Synthetic_Binary_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Binary_Spec is
      begin
         if Node = null then
            return No_Entity_Synthetic_Binary_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Binary_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Char_Enum_Lit
        (Node : Bare_Synthetic_Char_Enum_Lit; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Char_Enum_Lit is
      begin
         if Node = null then
            return No_Entity_Synthetic_Char_Enum_Lit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Char_Enum_Lit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Defining_Name
        (Node : Bare_Synthetic_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Synthetic_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Defining_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Formal_Param_Decl
        (Node : Bare_Synthetic_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Formal_Param_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Identifier
        (Node : Bare_Synthetic_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Identifier is
      begin
         if Node = null then
            return No_Entity_Synthetic_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Identifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Object_Decl
        (Node : Bare_Synthetic_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Renaming_Clause
        (Node : Bare_Synthetic_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Synthetic_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Subp_Decl
        (Node : Bare_Synthetic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Synthetic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Type_Expr
        (Node : Bare_Synthetic_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Synthetic_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Unary_Spec
        (Node : Bare_Synthetic_Unary_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Unary_Spec is
      begin
         if Node = null then
            return No_Entity_Synthetic_Unary_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Unary_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Node
        (Node : Bare_Tagged_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Node is
      begin
         if Node = null then
            return No_Entity_Tagged_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Absent
        (Node : Bare_Tagged_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Absent is
      begin
         if Node = null then
            return No_Entity_Tagged_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Present
        (Node : Bare_Tagged_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Present is
      begin
         if Node = null then
            return No_Entity_Tagged_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Target_Name
        (Node : Bare_Target_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Target_Name is
      begin
         if Node = null then
            return No_Entity_Target_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Target_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body
        (Node : Bare_Task_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body is
      begin
         if Node = null then
            return No_Entity_Task_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body_Stub
        (Node : Bare_Task_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Task_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Def
        (Node : Bare_Task_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Def is
      begin
         if Node = null then
            return No_Entity_Task_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Terminate_Alternative
        (Node : Bare_Terminate_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Terminate_Alternative is
      begin
         if Node = null then
            return No_Entity_Terminate_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Terminate_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Access_Def
        (Node : Bare_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Attributes_Repository
        (Node : Bare_Type_Attributes_Repository; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Attributes_Repository is
      begin
         if Node = null then
            return No_Entity_Type_Attributes_Repository;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Attributes_Repository) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Un_Op
        (Node : Bare_Un_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Un_Op is
      begin
         if Node = null then
            return No_Entity_Un_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Un_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index
        (Node : Bare_Unconstrained_Array_Index; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index_List
        (Node : Bare_Unconstrained_Array_Index_List; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index_List is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Indices
        (Node : Bare_Unconstrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unknown_Discriminant_Part
        (Node : Bare_Unknown_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Unknown_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Unknown_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unknown_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Node
        (Node : Bare_Until_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Node is
      begin
         if Node = null then
            return No_Entity_Until_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Absent
        (Node : Bare_Until_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Absent is
      begin
         if Node = null then
            return No_Entity_Until_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Present
        (Node : Bare_Until_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Present is
      begin
         if Node = null then
            return No_Entity_Until_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Update_Attribute_Ref
        (Node : Bare_Update_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Update_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Update_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Update_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Clause
        (Node : Bare_Use_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Package_Clause
        (Node : Bare_Use_Package_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Package_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Package_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Package_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Type_Clause
        (Node : Bare_Use_Type_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Type_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Type_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Type_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Value_Sequence
        (Node : Bare_Value_Sequence; Info : Internal_Entity_Info)
         return Internal_Entity_Value_Sequence is
      begin
         if Node = null then
            return No_Entity_Value_Sequence;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Value_Sequence) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant
        (Node : Bare_Variant; Info : Internal_Entity_Info)
         return Internal_Entity_Variant is
      begin
         if Node = null then
            return No_Entity_Variant;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_List
        (Node : Bare_Variant_List; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_List is
      begin
         if Node = null then
            return No_Entity_Variant_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_Part
        (Node : Bare_Variant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_Part is
      begin
         if Node = null then
            return No_Entity_Variant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Spec
        (Node : Bare_While_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_While_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Stmt
        (Node : Bare_While_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_While_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Clause
        (Node : Bare_With_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_With_Clause is
      begin
         if Node = null then
            return No_Entity_With_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private
        (Node : Bare_With_Private; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private is
      begin
         if Node = null then
            return No_Entity_With_Private;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Absent
        (Node : Bare_With_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Absent is
      begin
         if Node = null then
            return No_Entity_With_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Present
        (Node : Bare_With_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Present is
      begin
         if Node = null then
            return No_Entity_With_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Env_Assoc) is
      begin
               Inc_Ref (R.Dest_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Env_Assoc) is
      begin
               Dec_Ref (R.Dest_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Env_Assoc) return Boolean is
      begin
         return L.Key = R.Key and then L.Value = R.Value and then Equivalent (L.Dest_Env, R.Dest_Env) and then L.Metadata = R.Metadata;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Dest_Env => " & Trace_Image (R.Dest_Env)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Eval_Discrete_Range) is
      begin
               Inc_Ref (R.Low_Bound);
               Inc_Ref (R.High_Bound);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Eval_Discrete_Range) is
      begin
               Dec_Ref (R.Low_Bound);
               Dec_Ref (R.High_Bound);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Eval_Discrete_Range) return Boolean is
      begin
         return Equivalent (L.Low_Bound, R.Low_Bound) and then Equivalent (L.High_Bound, R.High_Bound);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Eval_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Expected_Type_For_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Expected_Type => " & Trace_Image (R.Expected_Type)
                        & ", "
                     & "Expr => " & Trace_Image (R.Expr)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Inner_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Logic_Context) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Ref_Node => " & Trace_Image (R.Ref_Node)
                        & ", "
                     & "Decl_Node => " & Trace_Image (R.Decl_Node)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Logic_Val_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Multidim_Aggregate_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Agg => " & Trace_Image (R.Agg)
                        & ", "
                     & "Typ => " & Trace_Image (R.Typ)
                        & ", "
                     & "Rank => " & Trace_Image (R.Rank)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Param => " & Trace_Image (R.Param)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Single_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Assoc => " & Trace_Image (R.Assoc)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Match) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Has_Matched => " & Trace_Image (R.Has_Matched)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
                        & ", "
                     & "Formal => " & Trace_Image (R.Formal)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Ref_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Ref => " & Trace_Image (R.Ref)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Def_Name => " & Trace_Image (R.Def_Name)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Shape) is
      begin
               Inc_Ref (R.Components);
               Inc_Ref (R.Discriminants_Values);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Shape) is
      begin
               Dec_Ref (R.Components);
               Dec_Ref (R.Discriminants_Values);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Shape) return Boolean is
      begin
         return Equivalent (L.Components, R.Components) and then Equivalent (L.Discriminants_Values, R.Discriminants_Values);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Shape) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Components => " & Trace_Image (R.Components)
                        & ", "
                     & "Discriminants_Values => " & Trace_Image (R.Discriminants_Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Solver_Diagnostic) is
      begin
               Inc_Ref (R.Message_Template);
               Inc_Ref (R.Args);
               Inc_Ref (R.Contexts);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Solver_Diagnostic) is
      begin
               Dec_Ref (R.Message_Template);
               Dec_Ref (R.Args);
               Dec_Ref (R.Contexts);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Solver_Diagnostic) return Boolean is
      begin
         return Equivalent (L.Message_Template, R.Message_Template) and then Equivalent (L.Args, R.Args) and then L.Location = R.Location and then Equivalent (L.Contexts, R.Contexts) and then L.Round = R.Round;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Solver_Diagnostic) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Message_Template => " & Trace_Image (R.Message_Template)
                        & ", "
                     & "Args => " & Trace_Image (R.Args)
                        & ", "
                     & "Location => " & Trace_Image (R.Location)
                        & ", "
                     & "Contexts => " & Trace_Image (R.Contexts)
                        & ", "
                     & "Round => " & Trace_Image (R.Round)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Solver_Result) is
      begin
               Inc_Ref (R.Diagnostics);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Solver_Result) is
      begin
               Dec_Ref (R.Diagnostics);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Solver_Result) return Boolean is
      begin
         return L.Success = R.Success and then Equivalent (L.Diagnostics, R.Diagnostics);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Solver_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Diagnostics => " & Trace_Image (R.Diagnostics)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Substitution) is
      begin
               Inc_Ref (R.To_Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Substitution) is
      begin
               Dec_Ref (R.To_Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Substitution) return Boolean is
      begin
         return L.From_Decl = R.From_Decl and then Equivalent (L.To_Value, R.To_Value) and then L.Value_Type = R.Value_Type;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Substitution) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "From_Decl => " & Trace_Image (R.From_Decl)
                        & ", "
                     & "To_Value => " & Trace_Image (R.To_Value)
                        & ", "
                     & "Value_Type => " & Trace_Image (R.Value_Type)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_User_Defined_Function_Subp_Spec) is
      begin
               Inc_Ref (R.Subp_Params_Types);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_User_Defined_Function_Subp_Spec) is
      begin
               Dec_Ref (R.Subp_Params_Types);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_User_Defined_Function_Subp_Spec) return Boolean is
      begin
         return Equivalent (L.Subp_Params_Types, R.Subp_Params_Types) and then L.Subp_Return_Type = R.Subp_Return_Type;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_User_Defined_Function_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Subp_Params_Types => " & Trace_Image (R.Subp_Params_Types)
                        & ", "
                     & "Subp_Return_Type => " & Trace_Image (R.Subp_Return_Type)
               & ")");
      end Trace_Image;
   type Ada_Node_P_Is_Not_Null_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Ada_Node_P_Is_Not_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate) return String;
   function Create_Ada_Node_P_Is_Not_Null_0_Predicate
      return Ada_Node_P_Is_Not_Null_0_Predicate is
   begin
      return Ada_Node_P_Is_Not_Null_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Ada_Node_P_Is_Not_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Ada_Node;
   begin
      Node := Entity.Node;
      return Ada_Node_P_Is_Not_Null (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate) return String is
   begin
      return "AdaNode.is_not_null";
   end Image;
   type Ada_Node_P_Missing_Entity_Error_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate) return String;
   function Create_Ada_Node_P_Missing_Entity_Error_0_Predicate
   (
         Error_Location : Bare_Ada_Node
   )
      return Ada_Node_P_Missing_Entity_Error_0_Predicate is
   begin
      return Ada_Node_P_Missing_Entity_Error_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Ada_Node;
   begin
      Node := Entity.Node;
      return Ada_Node_P_Missing_Entity_Error (Node);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (0);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("no such entity"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Ada_Node_P_Missing_Entity_Error_0_Predicate) return String is
   begin
      return "AdaNode.missing_entity_error";
   end Image;
   type Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate) return String;
   function Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate
   (
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Subp_Decl_Match_Signature (Node, Other => Self.Field_0, Origin => Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate) return String is
   begin
      return "BasicDecl.subp_decl_match_signature";
   end Image;
   type Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate) return String;
   function Create_Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate
   (
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate is
   begin
      return Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Access_To_Subp_Decl_Match_Signature (Node, Other => Self.Field_0, Origin => Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate) return String is
   begin
      return "BasicDecl.access_to_subp_decl_match_signature";
   end Image;
   type Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate) return String;
   function Create_Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate
   (
         Field_0 : Internal_Entity_Basic_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate is
   begin
      return Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Subp_Renaming_Decl_Match_Signature (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Other => Self.Field_0, Origin => Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate) return String is
   begin
      return "BasicDecl.subp_renaming_decl_match_signature";
   end Image;
   type Basic_Decl_P_Is_Subprogram_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Subprogram_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate) return String;
   function Create_Basic_Decl_P_Is_Subprogram_0_Predicate
      return Basic_Decl_P_Is_Subprogram_0_Predicate is
   begin
      return Basic_Decl_P_Is_Subprogram_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Subprogram_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Is_Subprogram (Node);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate) return String is
   begin
      return "BasicDecl.is_subprogram";
   end Image;
   type Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate) return String;
   function Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean
   )
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type (Node, Typ => Self.Field_0, Return_Obj => Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate) return String is
   begin
      return "BasicDecl.is_stream_subprogram_for_type";
   end Image;
   type Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
   end record;
      overriding function Call
        (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate) return String;
   function Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl
   )
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate is
   begin
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Basic_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Basic_Decl_P_Is_Put_Image_Subprogram_For_Type (Node, Typ => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate) return String is
   begin
      return "BasicDecl.is_put_image_subprogram_for_type";
   end Image;
   type Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Array_Or_Rec (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_or_rec";
   end Image;
   type Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Non_Null_Char_Type (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_non_null_char_type";
   end Image;
   type Base_Type_Decl_P_Is_Universal_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Universal_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_universal_type";
   end Image;
   type Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Not_Universal_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_not_universal_type";
   end Image;
   type Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Access_Type_Predicate (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_access_type_predicate";
   end Image;
   type Base_Type_Decl_P_Is_Iterator_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Iterator_Type (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_iterator_type";
   end Image;
   type Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate
      return Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Not_Root_Int_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_not_root_int_type";
   end Image;
   type Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate
   (
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Not_Any_Type (Node);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (0);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("No inferable type for expression"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_not_any_type";
   end Image;
   type Base_Type_Decl_P_Allows_String_Literal_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Allows_String_Literal_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Allows_String_Literal_0_Predicate is
   begin
      return Base_Type_Decl_P_Allows_String_Literal_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Allows_String_Literal (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (1);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("{} does not allow string literals"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entity;
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Allows_String_Literal_0_Predicate) return String is
   begin
      return "BaseTypeDecl.allows_string_literal";
   end Image;
   type Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Basic_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Subp_Access_Of (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_subp_access_of";
   end Image;
   type Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Tagged_Type_With_Deref (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (1);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("expected tagged type, got {}"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entity;
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_tagged_type_with_deref";
   end Image;
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Array_Def_With_Deref (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_def_with_deref";
   end Image;
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_array_def_with_deref_or_null";
   end Image;
   type Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Prefix_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_prefix_type";
   end Image;
   type Base_Type_Decl_P_Matching_Formal_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Formal_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Formal_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Formal_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Formal_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (2);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("expected {}, got {}"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entities (2);
      Args.Items (2) := Entities (1);
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Formal_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_formal_type";
   end Image;
   type Base_Type_Decl_P_Matching_Membership_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Membership_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Membership_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Membership_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Membership_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Membership_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Membership_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Membership_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Membership_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_membership_type";
   end Image;
   type Base_Type_Decl_P_Matching_Assign_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Assign_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Assign_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Assign_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Assign_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (2);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("expected {}, got {}"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entities (2);
      Args.Items (2) := Entities (1);
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Assign_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_assign_type";
   end Image;
   type Base_Type_Decl_P_Matching_Type_0_Predicate is
   new Solver_Ifc.N_Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Type_0_Predicate'(N => 2, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array
        ) return Boolean
   is
         Entity : Internal_Entity := Entities (1);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
               if Entities (2).Node /= null
                  and then Entities (2).Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Type (Node, (Node => Entities (2).Node, Info => Entities (2).Info), Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Matching_Type_0_Predicate;
            Entities : Entity_Vars.Value_Array;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (2);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("expected {}, got {}"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entities (2);
      Args.Items (2) := Entities (1);
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_type";
   end Image;
   type Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate
   (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Field_1 => Field_1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Matching_Allocator_Type (Node, Allocated_Type => Self.Field_0, Origin => Self.Field_1, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.matching_allocator_type";
   end Image;
   type Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   ;
   overriding function Image (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node;
         Error_Location : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0, Error_Location => Error_Location);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Base_Type_Decl_P_Derives_From_Std_Bool_Type (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   ------------
   -- Failed --
   ------------
      overriding procedure Failed
        (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate;
            Entity : Internal_Entity;
         Ctxs    : Solver_Ifc.Logic_Context_Array;
         Round   : Natural;
         Emitter : Solver_Ifc.Diagnostic_Emitter
        )
   is
      Args : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (1);
      Contexts : Internal_Logic_Context_Array_Access :=
         Create_Internal_Logic_Context_Array (Ctxs'Length);
      Diag : constant Internal_Solver_Diagnostic :=
        (Message_Template => Create_String ("expected boolean type, got {}"),
         Args             => Args,
         Contexts         => Contexts,
         Location         => Self.Error_Location,
         Round            => Round);
   begin
      Args.Items (1) := Entity;
      for I in Ctxs'Range loop
         Contexts.Items (I) := Ctxs (I).all;
      end loop;
      Emitter (Diag);
   end Failed;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.derives_from_std_bool_type";
   end Image;
   type Base_Type_Decl_P_Is_Task_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         null;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
         pragma Unreferenced (Self);
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_task_type";
   end Image;
   type Base_Type_Decl_P_Is_Discrete_Type_0_Predicate is
   new Solver_Ifc.Predicate_Type
   with record
         Field_0 : Bare_Ada_Node;
   end record;
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   ;
   overriding function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate) return String;
   function Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate
   (
         Field_0 : Bare_Ada_Node
   )
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate is
   begin
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate'(Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Ref_Count => 1, Field_0 => Field_0);
   end;
   ----------
   -- Call --
   ----------
      overriding function Call
        (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate;
            Entity : Internal_Entity
        ) return Boolean
   is
      Node : Bare_Base_Type_Decl;
   begin
               if Entity.Node /= null
                  and then Entity.Node.Kind
                           not in Ada_Base_Type_Decl
               then
                  Raise_Property_Exception
                    (Node, Property_Error'Identity, "mismatching node type");
               end if;
      Node := Entity.Node;
      return Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node, Origin => Self.Field_0, E_Info => Entity.Info);
   end Call;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate) return String is
   begin
      return "BaseTypeDecl.is_discrete_type";
   end Image;
   type Logic_Functor_885 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_885; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_885) return String;
   overriding function Convert (Self : Logic_Functor_885; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Accessed_Type_No_Call (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_885) return String is
   begin
      return ("BaseTypeDecl.accessed_type_no_call");
   end Image;
   type Logic_Functor_843 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_843; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_843) return String;
   overriding function Convert (Self : Logic_Functor_843; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Anonymous_Access_Type_Or_Null (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_843) return String is
   begin
      return ("BaseTypeDecl.anonymous_access_type_or_null");
   end Image;
   type Logic_Functor_867 is new Solver_Ifc.Combiner_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Combine (Self : Logic_Functor_867; Vals : Entity_Vars.Value_Array) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_867) return String;
   overriding function Combine (Self : Logic_Functor_867; Vals : Entity_Vars.Value_Array) return Internal_Entity is
         From : constant Internal_Entity := Vals (1);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
            if Vals (2).Node /= null
               and then Vals (2).Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
      Ret := Base_Type_Decl_P_Array_Concat_Result_Type (Node => From.Node, Other => (Node => Vals (2).Node,  Info => Vals (2).Info), Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Combine;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_867) return String is
   begin
      return ("BaseTypeDecl.array_concat_result_type");
   end Image;
   type Logic_Functor_841 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_841; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_841) return String;
   overriding function Convert (Self : Logic_Functor_841; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Base_Subtype (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_841) return String is
   begin
      return ("BaseTypeDecl.base_subtype");
   end Image;
   type Logic_Functor_895 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_895; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_895) return String;
   overriding function Convert (Self : Logic_Functor_895; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Comp_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_895) return String is
   begin
      return ("BaseTypeDecl.comp_type");
   end Image;
   type Logic_Functor_919 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_919; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_919) return String;
   overriding function Convert (Self : Logic_Functor_919; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Basic_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Corresponding_Char_Literal (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_919) return String is
   begin
      return ("BaseTypeDecl.corresponding_char_literal");
   end Image;
   type Logic_Functor_878 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_878; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_878) return String;
   overriding function Convert (Self : Logic_Functor_878; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Cursor_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_878) return String is
   begin
      return ("BaseTypeDecl.cursor_type");
   end Image;
   type Logic_Functor_866 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_866; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_866) return String;
   overriding function Convert (Self : Logic_Functor_866; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Derefed_Base_Subtype (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_866) return String is
   begin
      return ("BaseTypeDecl.derefed_base_subtype");
   end Image;
   type Logic_Functor_865 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_865; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_865) return String;
   overriding function Convert (Self : Logic_Functor_865; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Derefed_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_865) return String is
   begin
      return ("BaseTypeDecl.derefed_type");
   end Image;
   type Logic_Functor_868 is new Solver_Ifc.Combiner_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Combine (Self : Logic_Functor_868; Vals : Entity_Vars.Value_Array) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_868) return String;
   overriding function Combine (Self : Logic_Functor_868; Vals : Entity_Vars.Value_Array) return Internal_Entity is
         From : constant Internal_Entity := Vals (1);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
            if Vals (2).Node /= null
               and then Vals (2).Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for comb_prop");
            end if;
      Ret := Base_Type_Decl_P_Expected_Array_Concat_Operand_Type (Node => From.Node, Operand_Type => (Node => Vals (2).Node,  Info => Vals (2).Info), Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Combine;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_868) return String is
   begin
      return ("BaseTypeDecl.expected_array_concat_operand_type");
   end Image;
   type Logic_Functor_900 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_900; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_900) return String;
   overriding function Convert (Self : Logic_Functor_900; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Iterable_Comp_Type_Or_Null (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_900) return String is
   begin
      return ("BaseTypeDecl.iterable_comp_type_or_null");
   end Image;
   type Logic_Functor_870 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_870; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_870) return String;
   overriding function Convert (Self : Logic_Functor_870; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Discrete_Base_Subtype_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Base_Type_Decl_P_Scalar_Base_Type (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_870) return String is
   begin
      return ("BaseTypeDecl.scalar_base_type");
   end Image;
   type Logic_Functor_681 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_681; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_681) return String;
   overriding function Convert (Self : Logic_Functor_681; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Basic_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Basic_Decl_P_Identity_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_681) return String is
   begin
      return ("BasicDecl.identity_type");
   end Image;
   type Logic_Functor_945 is new Solver_Ifc.Converter_Type with
      null record;
   overriding function Convert (Self : Logic_Functor_945; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_945) return String;
   overriding function Convert (Self : Logic_Functor_945; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Classwide_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Base_Type_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => From.Node, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_945) return String is
   begin
      return ("BaseTypeDecl.classwide_type");
   end Image;
   type Logic_Functor_720 is new Solver_Ifc.Converter_Type with
      record
            Origin : Bare_Ada_Node;
      end record;
   overriding function Convert (Self : Logic_Functor_720; From : Internal_Entity) return Internal_Entity with Inline;
   overriding function Image (Self : Logic_Functor_720) return String;
   overriding function Convert (Self : Logic_Functor_720; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
            if From.Node /= null
               and then From.Node.Kind not in
                 Ada_Basic_Decl
            then
               Raise_Property_Exception
                 (From.Node,
                  Property_Error'Identity,
                  "mismatching node type for conv_prop");
            end if;
      Ret := Dispatcher_Basic_Decl_P_Expr_Type (Node => From.Node, Origin => Self.Origin, E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   -----------
   -- Image --
   -----------
   overriding function Image (Self : Logic_Functor_720) return String is
   begin
      return ("BasicDecl.expr_type");
   end Image;
   --
   --  Primitives for Bare_Ada_Node
   --
--# property-start AdaNode.declarative_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:47'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Declarative_Scope
  (Node : Bare_Ada_Node
  )
   return Bare_Declarative_Part
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_5 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_5 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_5;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4 '<Cast to DeclarativePart at nodes.lkt:48:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:48'
--# expr-start 3 '<Find at nodes.lkt:48:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:48'
   --# expr-start 1 '<FieldAccess for parents at nodes.lkt:48:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:48'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 1
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 2 '<IsA DeclarativePart at nodes.lkt:48:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:48'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Declarative_Part_Range; 
--# expr-done 2
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 3
   if Find_Result = null
      or else Find_Result.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_5;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_5;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Declarative_Scope;
--# end
--# property-start AdaNode.owning_unit_kind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:51'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Owning_Unit_Kind
  (Node : Bare_Ada_Node
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Analysis_Unit_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8 '<FieldAccess for unit_kind at nodes.lkt:52:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:52'
--# expr-start 7 '<Cast to CompilationUnit at nodes.lkt:52:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:52'
--# expr-start 6 '<FieldAccess for root at nodes.lkt:52:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:52'
--# expr-start 5 '<FieldAccess for unit at nodes.lkt:52:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:52'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 6
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_kind
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Unit_Kind (Node => Cast_Result);
--# end
--# expr-done 8
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Owning_Unit_Kind;
--# end
--# property-start AdaNode.withed_unit_helper 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:60'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Withed_Unit_Helper
  (Node : Bare_Ada_Node
      ; Unit_Name : Bare_Name
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit_name Unit_Name
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_7 with Inline_Always;
      Local_Unit_Name_Array : Symbol_Type_Array_Access;
Local_Spec : Bare_Compilation_Unit;
Local_Body : Bare_Compilation_Unit;
Ignored : Bare_Compilation_Unit;
Var_Expr_0 : Bare_Compilation_Unit;
Var_Expr : Bare_Compilation_Unit;
Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Compilation_Unit;
Is_Null : Boolean;
If_Result : Bare_Compilation_Unit;
Fld_3 : Bare_Compilation_Unit;
Fld_4 : Bare_Basic_Decl;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Is_Null_1 : Boolean;
If_Result_1 : Boolean;
Is_Null_2 : Boolean;
Is_Null_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Result_Var_1 : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
Let_Result_1 : Bare_Compilation_Unit;
Let_Result_2 : Bare_Compilation_Unit;
Let_Result_3 : Bare_Compilation_Unit;
            procedure Finalizer_Scope_7 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Unit_Name_Array);
            end Finalizer_Scope_7;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 24 '<Let at nodes.lkt:63:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:63'
--# scope-start
--# expr-start 9 '<FieldAccess for as_symbol_array at nodes.lkt:63:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:63'
      if Unit_Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Unit_Name);
--# end
--# expr-done 9
Local_Unit_Name_Array := Fld; Inc_Ref (Local_Unit_Name_Array);
--# bind unit_name_array Local_Unit_Name_Array
--# expr-start 23 '<Let at nodes.lkt:64:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:64'
--# scope-start
--# expr-start 10 '<FieldAccess for designated_compilation_unit at nodes.lkt:64:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:64'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Local_Unit_Name_Array, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 10
Local_Spec := Fld_1; 
--# bind spec Local_Spec
--# expr-start 22 '<Let at nodes.lkt:67:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:67'
--# scope-start
--# expr-start 13 '<If at nodes.lkt:67:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:67'
--# expr-start 11 '<IsNull at nodes.lkt:67:23>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:67'
Is_Null := Local_Spec = null; 
--# expr-done 11
if Is_Null then
   --# expr-start 12 '<FieldAccess for designated_compilation_unit at nodes.lkt:67:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:67'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Local_Unit_Name_Array, Kind => Unit_Body, Not_Found_Is_Error => False);
--# end
--# expr-done 12
   If_Result := Fld_2;
else
   If_Result := No_Bare_Ada_Node;
end if;
--# expr-done 13
Local_Body := If_Result; 
--# bind body Local_Body
--# expr-start 21 '<Let at nodes.lkt:73:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:73'
--# scope-start
--# expr-start 20 '<If at nodes.lkt:73:17>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:73'
Var_Expr_0 := Local_Body;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 15 '<IsA PackageBody at nodes.lkt:74:29>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:74'
--# expr-start 14 '<FieldAccess for decl at nodes.lkt:74:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:74'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_0);
--# end
--# expr-done 14
Is_A := Fld_4 /= null 
and then Fld_4.Kind in Ada_Package_Body_Range; 
--# expr-done 15
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 16 '<IsNull at nodes.lkt:74:59>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:74'
Is_Null_1 := Local_Spec = null; 
--# expr-done 16
   If_Result_1 := Is_Null_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 17 '<IsNull at nodes.lkt:75:15>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:75'
Is_Null_2 := Local_Spec = null; 
--# expr-done 17
if Is_Null_2 then
   --# expr-start 18 '<IsNull at nodes.lkt:75:32>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:75'
Is_Null_3 := Local_Body = null; 
--# expr-done 18
   If_Result_2 := Is_Null_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   --# expr-start 19 '<FieldAccess for designated_compilation_unit at nodes.lkt:75:51>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:75'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Local_Unit_Name_Array, Kind => Unit_Specification, Not_Found_Is_Error => True);
--# end
--# expr-done 19
   If_Result_4 := Fld_3;
else
   If_Result_4 := No_Bare_Ada_Node;
end if;
--# expr-done 20
Ignored := If_Result_4; 
--# bind _ Ignored
Var_Expr := Local_Spec;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Local_Body;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 21
Let_Result_1 := Let_Result; 
--# end
--# expr-done 22
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 23
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 24
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_7;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_7;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Withed_Unit_Helper;
--# end
--# property-start AdaNode.is_contract_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:83'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Contract_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Is_Equal_21 : Boolean;
If_Result_20 : Boolean;
Is_Equal_22 : Boolean;
If_Result_21 : Boolean;
Is_Equal_23 : Boolean;
If_Result_22 : Boolean;
Is_Equal_24 : Boolean;
If_Result_23 : Boolean;
Is_Equal_25 : Boolean;
If_Result_24 : Boolean;
Is_Equal_26 : Boolean;
If_Result_25 : Boolean;
Is_Equal_27 : Boolean;
If_Result_26 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 25 '<Symbol Pre at nodes.lkt:84:17>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 25
Is_Equal_27 := Name = Sym; 
if Is_Equal_27 then
   If_Result_26 := True;
else
--# expr-start 26 '<Symbol Pre'"'"'Class at nodes.lkt:84:26>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 26
Is_Equal_26 := Name = Sym_1; 
if Is_Equal_26 then
   If_Result_25 := True;
else
--# expr-start 27 '<Symbol Post at nodes.lkt:84:41>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 27
Is_Equal_25 := Name = Sym_2; 
if Is_Equal_25 then
   If_Result_24 := True;
else
--# expr-start 28 '<Symbol Post'"'"'Class at nodes.lkt:84:51>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 28
Is_Equal_24 := Name = Sym_3; 
if Is_Equal_24 then
   If_Result_23 := True;
else
--# expr-start 29 '<Symbol Refined_Post at nodes.lkt:84:67>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Post); 
--# expr-done 29
Is_Equal_23 := Name = Sym_4; 
if Is_Equal_23 then
   If_Result_22 := True;
else
--# expr-start 30 '<Symbol Precondition at nodes.lkt:84:85>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 30
Is_Equal_22 := Name = Sym_5; 
if Is_Equal_22 then
   If_Result_21 := True;
else
--# expr-start 31 '<Symbol Postcondition at nodes.lkt:84:103>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 31
Is_Equal_21 := Name = Sym_6; 
if Is_Equal_21 then
   If_Result_20 := True;
else
--# expr-start 32 '<Symbol Precondition'"'"'Class at nodes.lkt:84:122>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 32
Is_Equal_20 := Name = Sym_7; 
if Is_Equal_20 then
   If_Result_19 := True;
else
--# expr-start 33 '<Symbol Postcondition'"'"'Class at nodes.lkt:84:146>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 33
Is_Equal_19 := Name = Sym_8; 
if Is_Equal_19 then
   If_Result_18 := True;
else
--# expr-start 34 '<Symbol Invariant at nodes.lkt:84:171>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant); 
--# expr-done 34
Is_Equal_18 := Name = Sym_9; 
if Is_Equal_18 then
   If_Result_17 := True;
else
--# expr-start 35 '<Symbol Invariant'"'"'Class at nodes.lkt:84:186>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant_Class); 
--# expr-done 35
Is_Equal_17 := Name = Sym_10; 
if Is_Equal_17 then
   If_Result_16 := True;
else
--# expr-start 36 '<Symbol Type_Invariant at nodes.lkt:84:207>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant); 
--# expr-done 36
Is_Equal_16 := Name = Sym_11; 
if Is_Equal_16 then
   If_Result_15 := True;
else
--# expr-start 37 '<Symbol Type_Invariant'"'"'Class at nodes.lkt:84:227>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant_Class); 
--# expr-done 37
Is_Equal_15 := Name = Sym_12; 
if Is_Equal_15 then
   If_Result_14 := True;
else
--# expr-start 38 '<Symbol Predicate at nodes.lkt:84:253>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 38
Is_Equal_14 := Name = Sym_13; 
if Is_Equal_14 then
   If_Result_13 := True;
else
--# expr-start 39 '<Symbol Static_Predicate at nodes.lkt:84:268>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 39
Is_Equal_13 := Name = Sym_14; 
if Is_Equal_13 then
   If_Result_12 := True;
else
--# expr-start 40 '<Symbol Dynamic_Predicate at nodes.lkt:84:290>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dynamic_Predicate); 
--# expr-done 40
Is_Equal_12 := Name = Sym_15; 
if Is_Equal_12 then
   If_Result_11 := True;
else
--# expr-start 41 '<Symbol Default_Initial_Condition at nodes.lkt:84:313>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Initial_Condition); 
--# expr-done 41
Is_Equal_11 := Name = Sym_16; 
if Is_Equal_11 then
   If_Result_10 := True;
else
--# expr-start 42 '<Symbol Initial_Condition at nodes.lkt:84:344>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 42
Is_Equal_10 := Name = Sym_17; 
if Is_Equal_10 then
   If_Result_9 := True;
else
--# expr-start 43 '<Symbol Contract_Cases at nodes.lkt:84:367>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 43
Is_Equal_9 := Name = Sym_18; 
if Is_Equal_9 then
   If_Result_8 := True;
else
--# expr-start 44 '<Symbol Test_Case at nodes.lkt:84:387>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 44
Is_Equal_8 := Name = Sym_19; 
if Is_Equal_8 then
   If_Result_7 := True;
else
--# expr-start 45 '<Symbol Global at nodes.lkt:84:402>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 45
Is_Equal_7 := Name = Sym_20; 
if Is_Equal_7 then
   If_Result_6 := True;
else
--# expr-start 46 '<Symbol Refined_Global at nodes.lkt:84:414>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 46
Is_Equal_6 := Name = Sym_21; 
if Is_Equal_6 then
   If_Result_5 := True;
else
--# expr-start 47 '<Symbol Refined_State at nodes.lkt:84:434>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 47
Is_Equal_5 := Name = Sym_22; 
if Is_Equal_5 then
   If_Result_4 := True;
else
--# expr-start 48 '<Symbol Stable_Properties at nodes.lkt:84:453>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stable_Properties); 
--# expr-done 48
Is_Equal_4 := Name = Sym_23; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 49 '<Symbol Depends at nodes.lkt:84:476>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 49
Is_Equal_3 := Name = Sym_24; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 50 '<Symbol Refined_Depends at nodes.lkt:84:489>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 50
Is_Equal_2 := Name = Sym_25; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 51 '<Symbol Predicate_Failure at nodes.lkt:84:510>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate_Failure); 
--# expr-done 51
Is_Equal_1 := Name = Sym_26; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 52 '<Symbol SPARK_Mode at nodes.lkt:84:533>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:84'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 52
Is_Equal := Name = Sym_27; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
end if;
   If_Result_12 := If_Result_11;
end if;
   If_Result_13 := If_Result_12;
end if;
   If_Result_14 := If_Result_13;
end if;
   If_Result_15 := If_Result_14;
end if;
   If_Result_16 := If_Result_15;
end if;
   If_Result_17 := If_Result_16;
end if;
   If_Result_18 := If_Result_17;
end if;
   If_Result_19 := If_Result_18;
end if;
   If_Result_20 := If_Result_19;
end if;
   If_Result_21 := If_Result_20;
end if;
   If_Result_22 := If_Result_21;
end if;
   If_Result_23 := If_Result_22;
end if;
   If_Result_24 := If_Result_23;
end if;
   If_Result_25 := If_Result_24;
end if;
   If_Result_26 := If_Result_25;
end if;
         Property_Result := If_Result_26;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Contract_Aspect;
--# end
--# property-start AdaNode.aspect_has_forward_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:89'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Aspect_Has_Forward_Visibility
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
      Fld : Boolean;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 53 '<FieldAccess for is_contract_aspect at nodes.lkt:90:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:90'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_contract_aspect
Fld := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Name);
--# end
--# expr-done 53
if Fld then
   If_Result_1 := True;
else
--# expr-start 54 '<Symbol Iterator_Element at nodes.lkt:90:50>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:90'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 54
Is_Equal_1 := Name = Sym; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 55 '<Symbol Default_Iterator at nodes.lkt:90:72>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:90'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Iterator); 
--# expr-done 55
Is_Equal := Name = Sym_1; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Aspect_Has_Forward_Visibility;
--# end
--# property-start AdaNode.in_aspect_with_forward_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:92'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Aspect_With_Forward_Visibility
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_10 with Inline_Always;
      Var_Expr_1 : Bare_Aspect_Assoc;
Var_Expr_2 : Bare_Pragma_Node;
Var_Expr : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Pragma_Node;
Fld_3 : Bare_Identifier;
As_Entity_1 : Internal_Entity_Identifier;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Identifier;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_1 : Boolean;
Result_Var_2 : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null_2 : Boolean;
Not_Val : Boolean;
            procedure Finalizer_Scope_10 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_10;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 69 '<Not at nodes.lkt:92:53>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:92'
--# expr-start 68 '<IsNull at nodes.lkt:92:57>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:92'
--# expr-start 67 '<Find at nodes.lkt:92:57>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:92'
   --# expr-start 56 '<FieldAccess for parents at nodes.lkt:92:57>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:92'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 56
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
--# expr-start 57 '<Cast to AspectAssoc at nodes.lkt:93:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:93'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 57
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 61 '<FieldAccess for aspect_has_forward_visibility at nodes.lkt:94:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:94'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 60 '<FieldAccess for name_symbol at nodes.lkt:94:55>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:94'
--# expr-start 59 '<as_bare_entity at nodes.lkt:94:55>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:94'
--# expr-start 58 '<FieldAccess for id at nodes.lkt:94:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:94'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_1.Aspect_Assoc_F_Id;
--# expr-done 58
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 59
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result.Node);
--# end
--# expr-done 60
--# property-call-start AdaNode.aspect_has_forward_visibility
Fld_2 := Libadalang.Implementation.Ada_Node_P_Aspect_Has_Forward_Visibility (Node => Self, Name => Fld_1);
--# end
--# expr-done 61
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Var_Expr := Result_Var;
if Var_Expr /= False then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 62 '<Cast to Pragma at nodes.lkt:95:15>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:95'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 62
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 66 '<FieldAccess for aspect_has_forward_visibility at nodes.lkt:96:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:96'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 65 '<FieldAccess for name_symbol at nodes.lkt:96:55>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:96'
--# expr-start 64 '<as_bare_entity at nodes.lkt:96:55>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:96'
--# expr-start 63 '<FieldAccess for id at nodes.lkt:96:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:96'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_2.Pragma_Node_F_Id;
--# expr-done 63
Is_Null_1 := Fld_3 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Identifier;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 64
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result_1.Node);
--# end
--# expr-done 65
--# property-call-start AdaNode.aspect_has_forward_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Aspect_Has_Forward_Visibility (Node => Self, Name => Fld_4);
--# end
--# expr-done 66
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := False;
end if;
   Result_Var_2 := Result_Var_1;
end if;
      if Result_Var_2 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var_2;
         end loop;
      end;
--# expr-done 67
Is_Null_2 := Find_Result = null; 
--# expr-done 68
Not_Val := not (Is_Null_2); 
--# expr-done 69
         Property_Result := Not_Val;
   --# end
      Finalizer_Scope_10;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_10;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_In_Aspect_With_Forward_Visibility;
--# end
--# property-start AdaNode.in_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:101'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_11 with Inline_Always;
      Var_Expr_1 : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
            procedure Finalizer_Scope_11 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_11;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 74 '<AnyQuantifier at nodes.lkt:101:41>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:101'
   --# expr-start 70 '<FieldAccess for parents at nodes.lkt:101:41>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:101'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 70
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
--# expr-start 71 '<Cast to AspectAssoc at nodes.lkt:102:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:102'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 71
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 73 '<FieldAccess for name_is at nodes.lkt:102:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:102'
--# expr-start 72 '<FieldAccess for id at nodes.lkt:102:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:102'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_1.Aspect_Assoc_F_Id;
--# expr-done 72
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Name);
--# end
--# expr-done 73
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 74
         Property_Result := Quantifier_Result;
   --# end
      Finalizer_Scope_11;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_11;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_In_Aspect;
--# end
--# property-start AdaNode.empty_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:105'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Empty_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_12 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_12 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld);
            end Finalizer_Scope_12;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 79 '<FieldAccess for get_empty_env at nodes.lkt:106:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:106'
--# expr-start 78 '<Cast to CompilationUnit at nodes.lkt:106:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:106'
--# expr-start 77 '<Find at nodes.lkt:106:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:106'
   --# expr-start 75 '<FieldAccess for parents at nodes.lkt:106:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:106'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 75
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 76 '<IsA CompilationUnit at nodes.lkt:106:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:106'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 76
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 77
   if Find_Result = null
      or else Find_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 78
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Get_Empty_Env (Node => Cast_Result);
--# expr-done 79
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_12;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_12;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Empty_Env;
--# end
--# property-start AdaNode.is_not_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:109'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Not_Null
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 81 '<Not at nodes.lkt:114:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:114'
--# expr-start 80 '<IsNull at nodes.lkt:114:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:114'
Is_Null := Ent.Node = null; 
--# expr-done 80
Not_Val := not (Is_Null); 
--# expr-done 81
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Not_Null;
--# end
--# property-start AdaNode.eval_discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:117'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Eval_Discrete_Range
  (Node : Bare_Ada_Node
      ; Dr : Internal_Discrete_Range
  )
   return Internal_Eval_Discrete_Range
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind dr Dr
   Property_Result : Internal_Eval_Discrete_Range;
            procedure Finalizer_Scope_14 with Inline_Always;
            procedure Finalizer_Scope_1908 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Scope_Result : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Result_Var : Big_Integer_Type;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
New_Struct : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Is_Equal : Boolean;
If_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
            procedure Finalizer_Scope_14 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Big_Int);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_14;
            procedure Finalizer_Scope_1908 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1908;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 89 '<If at nodes.lkt:118:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:118'
--# expr-start 82 '<Eq at nodes.lkt:118:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:118'
Is_Equal := Dr = No_Discrete_Range; 
--# expr-done 82
if Is_Equal then
   If_Result := raise Precondition_Failure with "Attempting to evaluate a null discrete range";
else
   --# expr-start 88 '<New EvalDiscreteRange at nodes.lkt:118:141>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:118'
--# expr-start 84 '<FieldAccess for eval_as_int at nodes.lkt:119:93>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:119'
--# expr-start 83 '<FieldAccess for high_bound at nodes.lkt:119:93>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:119'
Fld_2 := Dr.High_Bound;
--# expr-done 83
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 84
--# expr-start 85 '<FieldAccess for low_bound at nodes.lkt:119:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:119'
Fld := Dr.Low_Bound;
--# expr-done 85
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind lb Var_Expr_0
--# expr-start 86 '<FieldAccess for eval_as_int at nodes.lkt:119:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:119'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 86
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1908;
   Result_Var := Scope_Result;
else
   --# expr-start 87 '<BigInteger 0 at nodes.lkt:119:77>' Big_Int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:119'
Big_Int := Create_Big_Integer ("0"); 
--# expr-done 87
   Result_Var := Big_Int;
end if;
      Inc_Ref (Result_Var);
Inc_Ref (Fld_3);
Inc_Ref (Result_Var);
New_Struct := (High_Bound => Fld_3, Low_Bound => Result_Var); 
--# expr-done 88
   If_Result := New_Struct;
end if;
      Inc_Ref (If_Result);
--# expr-done 89
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_14;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_14;
                     Finalizer_Scope_1908;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Eval_Discrete_Range;
--# end
--# property-start AdaNode.sym_join 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:123'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sym_Join
  (Node : Bare_Ada_Node
      ; Syms : Symbol_Type_Array_Access
      ; Sep : String_Type
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind syms Syms
   --# bind sep Sep
   Property_Result : String_Type;
            procedure Finalizer_Scope_15 with Inline_Always;
            procedure Finalizer_Scope_1909 with Inline_Always;
      Fld : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
            procedure Finalizer_Scope_15 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_15;
            procedure Finalizer_Scope_1909 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1909;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 92 '<Join at nodes.lkt:124:9>' Join_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:124'
--# expr-start 91 '<Map at nodes.lkt:124:18>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:124'
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Symbol_Type_Array_Access := Syms;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_0
      --# expr-start 90 '<FieldAccess for image at nodes.lkt:124:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:124'
Fld := Libadalang.Implementation.Image (Self => Item_0);
--# expr-done 90
         declare
            Item_To_Append : constant String_Type := Fld;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1909;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 91
Join_Result := Join_Strings (Sep, Map_Result); 
--# expr-done 92
         Property_Result := Join_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_15;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_15;
                     Finalizer_Scope_1909;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Sym_Join;
--# end
--# property-start AdaNode.enclosing_compilation_unit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:132'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Enclosing_Compilation_Unit
  (Node : Bare_Ada_Node
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_16 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_16 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_16;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 96 '<Cast to CompilationUnit at nodes.lkt:133:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:133'
--# expr-start 95 '<Find at nodes.lkt:133:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:133'
   --# expr-start 93 '<FieldAccess for parents at nodes.lkt:133:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 93
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 94 '<IsA CompilationUnit at nodes.lkt:133:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:133'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 94
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 95
   if Find_Result = null
      or else Find_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Find_Result;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 96
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_16;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_16;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Enclosing_Compilation_Unit;
--# end
--# property-start AdaNode.is_children_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:137'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Children_Env
  (Node : Bare_Ada_Node
      ; Parent : Lexical_Env
      ; Current_Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind parent Parent
   --# bind current_env Current_Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_17 with Inline_Always;
      Env_Parent : Lexical_Env := Empty_Env;
Fld : Boolean;
Is_Equal : Boolean;
If_Result : Boolean;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
            procedure Finalizer_Scope_17 is
            begin
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_17;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 104 '<If at nodes.lkt:137:78>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:137'
--# expr-start 97 '<Eq at nodes.lkt:137:81>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:137'
Is_Equal_2 := Equivalent (Parent, Empty_Env); 
--# expr-done 97
if Is_Equal_2 then
   If_Result_2 := False;
else
   --# expr-start 103 '<If at nodes.lkt:137:78>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:137'
--# expr-start 98 '<Eq at nodes.lkt:138:10>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:138'
Is_Equal_1 := Equivalent (Current_Env, Parent); 
--# expr-done 98
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 102 '<If at nodes.lkt:137:78>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:137'
--# expr-start 99 '<IsNull at nodes.lkt:139:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:139'
Is_Equal := Equivalent (Current_Env, Empty_Env); 
--# expr-done 99
if Is_Equal then
   If_Result := False;
else
   --# expr-start 101 '<FieldAccess for is_children_env at nodes.lkt:140:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:140'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 100 '<env_parent at nodes.lkt:140:39>' Env_Parent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:140'
Env_Parent := AST_Envs.Parent (Current_Env); 
--# expr-done 100
--# property-call-start AdaNode.is_children_env
Fld := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Parent, Current_Env => Env_Parent);
--# end
--# expr-done 101
   If_Result := Fld;
end if;
--# expr-done 102
   If_Result_1 := If_Result;
end if;
--# expr-done 103
   If_Result_2 := If_Result_1;
end if;
--# expr-done 104
         Property_Result := If_Result_2;
   --# end
      Finalizer_Scope_17;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_17;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Children_Env;
--# end
--# property-start AdaNode.without_md 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:143'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Without_Md
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 111 '<New AdaNode.entity at nodes.lkt:143:41>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:143'
--# expr-start 109 '<New EntityInfo at nodes.lkt:144:30>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:144'
--# expr-start 106 '<FieldAccess for from_rebound at nodes.lkt:145:78>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:145'
--# expr-start 105 '<FieldAccess for info at nodes.lkt:145:78>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:145'
Fld_3 := Ent.Info;
--# expr-done 105
Fld_4 := Fld_3.From_Rebound;
--# expr-done 106
--# expr-start 108 '<FieldAccess for rebindings at nodes.lkt:145:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:145'
--# expr-start 107 '<FieldAccess for info at nodes.lkt:145:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:145'
Fld_1 := Ent.Info;
--# expr-done 107
Fld_2 := Fld_1.Rebindings;
--# expr-done 108
New_Struct := (From_Rebound => Fld_4, Md => No_Metadata, Rebindings => Fld_2); 
--# expr-done 109
--# expr-start 110 '<FieldAccess for node at nodes.lkt:144:14>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:144'
Fld := Ent.Node;
--# expr-done 110
New_Struct_1 := (Info => New_Struct, Node => Fld); 
--# expr-done 111
         Property_Result := New_Struct_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Without_Md;
--# end
--# property-start AdaNode.get_uninstantiated_node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:153'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Uninstantiated_Node
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 112 '<as_bare_entity at nodes.lkt:153:54>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:153'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 112
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Get_Uninstantiated_Node;
--# end
--# property-start AdaNode.complete 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:157'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Iterator_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_20 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Completion_Item_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Completion_Item_Array_Access;
Iter : Internal_Completion_Item_Iterator_Access;
Dyn_Var_Bind_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_20 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Iter);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_20;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 120 '<bind at nodes.lkt:158:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:158'
--# expr-start 113 '<FieldAccess for origin_node at nodes.lkt:158:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:158'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 113
Bound_Origin := Fld; 
--# expr-start 119 '<FieldAccess for to_iterator at nodes.lkt:160:9>' Iter 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:160'
--# expr-start 118 '<Map at nodes.lkt:160:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:160'
   --# expr-start 114 '<FieldAccess for complete_items at nodes.lkt:160:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:160'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.complete_items'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Items (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 114
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Completion_Item_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
         --# expr-start 117 '<Not at nodes.lkt:167:20>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:167'
--# expr-start 116 '<IsA SyntheticSubpDecl, SyntheticObjectDecl at nodes.lkt:167:24>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:167'
--# expr-start 115 '<FieldAccess for decl at nodes.lkt:167:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:167'
Fld_2 := Item_0.Decl;
--# expr-done 115
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Synthetic_Subp_Decl_Range | Ada_Synthetic_Object_Decl_Range; 
--# expr-done 116
Not_Val := not (Is_A); 
--# expr-done 117
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Completion_Item := Item_0;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 118
Iter := To_Iterator (Map_Result, Self.Unit.Context); 
--# expr-done 119
Dyn_Var_Bind_Result := Iter; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 120
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_20;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_20;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete;
--# end
--# property-start AdaNode.complete_item_weight_matching_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:176'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Item_Weight_Matching_Type
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Name : Internal_Entity_Name
      ; Origin : Bare_Ada_Node
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind name Name
   --# bind origin Origin
   Property_Result : Integer;
      Local_Te_Not_Null : Boolean;
Local_Td : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Type_Expr;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Type_Expr;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_1 : Integer;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Type_Expr;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Integer;
Scope_Result_5 : Integer;
Result_Var_5 : Integer;
Let_Result : Integer;
Let_Result_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 139 '<Let at nodes.lkt:177:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:177'
--# scope-start
--# expr-start 123 '<Not at nodes.lkt:177:27>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:177'
--# expr-start 122 '<IsNull at nodes.lkt:177:31>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:177'
--# expr-start 121 '<FieldAccess for type_expression at nodes.lkt:177:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:177'
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Item.Node, E_Info => Item.Info);
--# end
--# expr-done 121
Is_Null := Fld.Node = null; 
--# expr-done 122
Not_Val := not (Is_Null); 
--# expr-done 123
Local_Te_Not_Null := Not_Val; 
--# bind te_not_null Local_Te_Not_Null
--# expr-start 138 '<Let at nodes.lkt:178:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:178'
--# scope-start
--# expr-start 124 '<FieldAccess for type_expression at nodes.lkt:178:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:178'
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Item.Node, E_Info => Item.Info);
--# end
--# expr-done 124
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 125 '<FieldAccess for designated_type_decl at nodes.lkt:178:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:178'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_2 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 125
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Local_Td := Result_Var; 
--# bind td Local_Td
--# expr-start 126 '<FieldAccess for expr_type at nodes.lkt:181:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:181'
      if Item.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Item.Node, Origin => Origin, E_Info => Item.Info);
--# end
--# expr-done 126
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 137 '<If at nodes.lkt:184:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
--# expr-start 127 '<FieldAccess for referenced_decl at nodes.lkt:184:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => False, E_Info => Name.Info);
--# end
--# expr-done 127
Var_Expr_2 := Fld_8;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 128 '<FieldAccess for type_expression at nodes.lkt:184:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 128
Var_Expr_3 := Fld_9;
if Var_Expr_3 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 130 '<FieldAccess for matching_assign_type at nodes.lkt:184:17>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
--# expr-start 129 '<FieldAccess for designated_type_decl at nodes.lkt:184:17>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_10 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 129
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_10.Node, Expected_Type => Local_Td, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 130
Scope_Result_3 := Fld_11;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   If_Result_2 := Local_Te_Not_Null;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   If_Result_3 := 100;
else
   --# expr-start 136 '<If at nodes.lkt:184:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:184'
--# expr-start 135 '<bind at nodes.lkt:187:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:187'
Bound_Imprecise_Fallback := True; 
--# expr-start 131 '<FieldAccess for referenced_decl at nodes.lkt:189:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:189'
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_4 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Name.Info);
--# end
--# expr-done 131
Var_Expr_4 := Fld_4;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 132 '<FieldAccess for type_expression at nodes.lkt:189:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:189'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 132
Var_Expr_5 := Fld_5;
if Var_Expr_5 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 134 '<FieldAccess for matching_assign_type at nodes.lkt:189:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:189'
--# expr-start 133 '<FieldAccess for designated_type_decl at nodes.lkt:189:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:189'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_6 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 133
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_6.Node, Expected_Type => Local_Td, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 134
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result := Local_Te_Not_Null;
else
   If_Result := False;
end if;
Dyn_Var_Bind_Result := If_Result; 
--# expr-done 135
if Dyn_Var_Bind_Result then
   If_Result_1 := 70;
else
   If_Result_1 := 50;
end if;
--# expr-done 136
   If_Result_3 := If_Result_1;
end if;
--# expr-done 137
Scope_Result_5 := If_Result_3;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := 0;
end if;
Let_Result := Result_Var_5; 
--# end
--# expr-done 138
Let_Result_1 := Let_Result; 
--# end
--# expr-done 139
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Item_Weight_Matching_Type;
--# end
--# property-start AdaNode.valid_keywords 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:202'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Valid_Keywords
  (Node : Bare_Ada_Node
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_22 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Sym_68 : Symbol_Type;
Sym_69 : Symbol_Type;
Sym_70 : Symbol_Type;
Sym_71 : Symbol_Type;
Sym_72 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_22 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_22;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 213 '<ArrayLiteral at nodes.lkt:203:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
--# expr-start 140 '<Symbol abort at nodes.lkt:203:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort); 
--# expr-done 140
--# expr-start 141 '<Symbol abs at nodes.lkt:203:20>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs_316); 
--# expr-done 141
--# expr-start 142 '<Symbol abstract at nodes.lkt:203:28>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abstract); 
--# expr-done 142
--# expr-start 143 '<Symbol accept at nodes.lkt:203:41>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Accept); 
--# expr-done 143
--# expr-start 144 '<Symbol access at nodes.lkt:203:52>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access_319); 
--# expr-done 144
--# expr-start 145 '<Symbol aliased at nodes.lkt:203:63>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aliased); 
--# expr-done 145
--# expr-start 146 '<Symbol all at nodes.lkt:203:75>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_All); 
--# expr-done 146
--# expr-start 147 '<Symbol and at nodes.lkt:203:83>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And_324); 
--# expr-done 147
--# expr-start 148 '<Symbol array at nodes.lkt:203:91>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Array); 
--# expr-done 148
--# expr-start 149 '<Symbol at at nodes.lkt:203:101>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_At); 
--# expr-done 149
--# expr-start 150 '<Symbol begin at nodes.lkt:203:108>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Begin); 
--# expr-done 150
--# expr-start 151 '<Symbol body at nodes.lkt:203:118>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Body); 
--# expr-done 151
--# expr-start 152 '<Symbol case at nodes.lkt:203:127>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Case); 
--# expr-done 152
--# expr-start 153 '<Symbol constant at nodes.lkt:203:136>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant); 
--# expr-done 153
--# expr-start 154 '<Symbol declare at nodes.lkt:203:149>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Declare); 
--# expr-done 154
--# expr-start 155 '<Symbol delay at nodes.lkt:203:161>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delay); 
--# expr-done 155
--# expr-start 156 '<Symbol delta at nodes.lkt:203:171>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delta_338); 
--# expr-done 156
--# expr-start 157 '<Symbol digits at nodes.lkt:203:181>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Digits_339); 
--# expr-done 157
--# expr-start 158 '<Symbol do at nodes.lkt:203:192>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Do); 
--# expr-done 158
--# expr-start 159 '<Symbol else at nodes.lkt:203:199>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Else); 
--# expr-done 159
--# expr-start 160 '<Symbol elsif at nodes.lkt:203:208>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elsif); 
--# expr-done 160
--# expr-start 161 '<Symbol end at nodes.lkt:203:218>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_End); 
--# expr-done 161
--# expr-start 162 '<Symbol entry at nodes.lkt:203:226>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Entry); 
--# expr-done 162
--# expr-start 163 '<Symbol exception at nodes.lkt:203:236>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception); 
--# expr-done 163
--# expr-start 164 '<Symbol exit at nodes.lkt:203:250>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exit); 
--# expr-done 164
--# expr-start 165 '<Symbol for at nodes.lkt:203:259>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_For); 
--# expr-done 165
--# expr-start 166 '<Symbol function at nodes.lkt:203:267>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Function); 
--# expr-done 166
--# expr-start 167 '<Symbol generic at nodes.lkt:203:280>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Generic); 
--# expr-done 167
--# expr-start 168 '<Symbol goto at nodes.lkt:203:292>' Sym_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Goto); 
--# expr-done 168
--# expr-start 169 '<Symbol if at nodes.lkt:203:301>' Sym_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_If); 
--# expr-done 169
--# expr-start 170 '<Symbol in at nodes.lkt:203:308>' Sym_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_In); 
--# expr-done 170
--# expr-start 171 '<Symbol interface at nodes.lkt:203:315>' Sym_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface_362); 
--# expr-done 171
--# expr-start 172 '<Symbol is at nodes.lkt:203:329>' Sym_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Is); 
--# expr-done 172
--# expr-start 173 '<Symbol limited at nodes.lkt:203:336>' Sym_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Limited); 
--# expr-done 173
--# expr-start 174 '<Symbol loop at nodes.lkt:203:348>' Sym_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop); 
--# expr-done 174
--# expr-start 175 '<Symbol mod at nodes.lkt:203:357>' Sym_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_372); 
--# expr-done 175
--# expr-start 176 '<Symbol new at nodes.lkt:203:365>' Sym_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_New); 
--# expr-done 176
--# expr-start 177 '<Symbol not at nodes.lkt:203:373>' Sym_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not_375); 
--# expr-done 177
--# expr-start 178 '<Symbol null at nodes.lkt:203:381>' Sym_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Null); 
--# expr-done 178
--# expr-start 179 '<Symbol others at nodes.lkt:203:390>' Sym_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Others); 
--# expr-done 179
--# expr-start 180 '<Symbol out at nodes.lkt:203:401>' Sym_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Out); 
--# expr-done 180
--# expr-start 181 '<Symbol of at nodes.lkt:203:409>' Sym_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Of); 
--# expr-done 181
--# expr-start 182 '<Symbol or at nodes.lkt:203:416>' Sym_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or_378); 
--# expr-done 182
--# expr-start 183 '<Symbol overriding at nodes.lkt:203:423>' Sym_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overriding); 
--# expr-done 183
--# expr-start 184 '<Symbol package at nodes.lkt:203:438>' Sym_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Package); 
--# expr-done 184
--# expr-start 185 '<Symbol pragma at nodes.lkt:203:450>' Sym_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pragma); 
--# expr-done 185
--# expr-start 186 '<Symbol private at nodes.lkt:203:461>' Sym_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Private); 
--# expr-done 186
--# expr-start 187 '<Symbol procedure at nodes.lkt:203:473>' Sym_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Procedure); 
--# expr-done 187
--# expr-start 188 '<Symbol protected at nodes.lkt:203:487>' Sym_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Protected); 
--# expr-done 188
--# expr-start 189 '<Symbol raise at nodes.lkt:203:501>' Sym_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Raise); 
--# expr-done 189
--# expr-start 190 '<Symbol range at nodes.lkt:203:511>' Sym_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range_392); 
--# expr-done 190
--# expr-start 191 '<Symbol record at nodes.lkt:203:521>' Sym_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Record); 
--# expr-done 191
--# expr-start 192 '<Symbol rem at nodes.lkt:203:532>' Sym_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem_395); 
--# expr-done 192
--# expr-start 193 '<Symbol renames at nodes.lkt:203:540>' Sym_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Renames); 
--# expr-done 193
--# expr-start 194 '<Symbol requeue at nodes.lkt:203:552>' Sym_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requeue); 
--# expr-done 194
--# expr-start 195 '<Symbol return at nodes.lkt:203:564>' Sym_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Return); 
--# expr-done 195
--# expr-start 196 '<Symbol reverse at nodes.lkt:203:575>' Sym_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Reverse); 
--# expr-done 196
--# expr-start 197 '<Symbol select at nodes.lkt:203:587>' Sym_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Select); 
--# expr-done 197
--# expr-start 198 '<Symbol separate at nodes.lkt:203:598>' Sym_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Separate); 
--# expr-done 198
--# expr-start 199 '<Symbol some at nodes.lkt:203:611>' Sym_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Some); 
--# expr-done 199
--# expr-start 200 '<Symbol subtype at nodes.lkt:203:620>' Sym_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Subtype); 
--# expr-done 200
--# expr-start 201 '<Symbol synchronized at nodes.lkt:203:632>' Sym_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Synchronized); 
--# expr-done 201
--# expr-start 202 '<Symbol tagged at nodes.lkt:203:649>' Sym_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tagged); 
--# expr-done 202
--# expr-start 203 '<Symbol task at nodes.lkt:203:660>' Sym_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task); 
--# expr-done 203
--# expr-start 204 '<Symbol terminate at nodes.lkt:203:669>' Sym_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Terminate); 
--# expr-done 204
--# expr-start 205 '<Symbol then at nodes.lkt:203:683>' Sym_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Then); 
--# expr-done 205
--# expr-start 206 '<Symbol type at nodes.lkt:203:692>' Sym_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type); 
--# expr-done 206
--# expr-start 207 '<Symbol until at nodes.lkt:203:701>' Sym_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Until); 
--# expr-done 207
--# expr-start 208 '<Symbol use at nodes.lkt:203:711>' Sym_68 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Use); 
--# expr-done 208
--# expr-start 209 '<Symbol when at nodes.lkt:203:719>' Sym_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_When); 
--# expr-done 209
--# expr-start 210 '<Symbol while at nodes.lkt:203:728>' Sym_70 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_While); 
--# expr-done 210
--# expr-start 211 '<Symbol with at nodes.lkt:203:738>' Sym_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_With); 
--# expr-done 211
--# expr-start 212 '<Symbol xor at nodes.lkt:203:747>' Sym_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:203'
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor_437); 
--# expr-done 212
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2, 4 => Sym_3, 5 => Sym_4, 6 => Sym_5, 7 => Sym_6, 8 => Sym_7, 9 => Sym_8, 10 => Sym_9, 11 => Sym_10, 12 => Sym_11, 13 => Sym_12, 14 => Sym_13, 15 => Sym_14, 16 => Sym_15, 17 => Sym_16, 18 => Sym_17, 19 => Sym_18, 20 => Sym_19, 21 => Sym_20, 22 => Sym_21, 23 => Sym_22, 24 => Sym_23, 25 => Sym_24, 26 => Sym_25, 27 => Sym_26, 28 => Sym_27, 29 => Sym_28, 30 => Sym_29, 31 => Sym_30, 32 => Sym_31, 33 => Sym_32, 34 => Sym_33, 35 => Sym_34, 36 => Sym_35, 37 => Sym_36, 38 => Sym_37, 39 => Sym_38, 40 => Sym_39, 41 => Sym_40, 42 => Sym_41, 43 => Sym_42, 44 => Sym_43, 45 => Sym_44, 46 => Sym_45, 47 => Sym_46, 48 => Sym_47, 49 => Sym_48, 50 => Sym_49, 51 => Sym_50, 52 => Sym_51, 53 => Sym_52, 54 => Sym_53, 55 => Sym_54, 56 => Sym_55, 57 => Sym_56, 58 => Sym_57, 59 => Sym_58, 60 => Sym_59, 61 => Sym_60, 62 => Sym_61, 63 => Sym_62, 64 => Sym_63, 65 => Sym_64, 66 => Sym_65, 67 => Sym_66, 68 => Sym_67, 69 => Sym_68, 70 => Sym_69, 71 => Sym_70, 72 => Sym_71, 73 => Sym_72)); 
--# expr-done 213
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_22;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_22;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Valid_Keywords;
--# end
--# property-start AdaNode.call_context 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:212'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Call_Context
  (Node : Bare_Ada_Node
      ; Ctx : Internal_Logic_Context
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind ctx Ctx
   Property_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_1921 with Inline_Always;
      Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Call_Expr;
Var_Expr_3 : Internal_Param_Match;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Var_Expr_7 : Internal_Entity_Name;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Call_Expr;
Fld_4 : Internal_Entity_Type_Expr;
Fld_5 : Bare_Type_Expr;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Param_Match_Array_Access;
Fld_12 : Internal_Single_Actual;
Fld_13 : Internal_Entity_Basic_Assoc;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Bare_Expr;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Find_Result : Internal_Param_Match;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Fld_18 : Internal_Entity_Type_Expr;
Fld_19 : Bare_Type_Expr;
Scope_Result : Bare_Type_Expr;
Fld_20 : Internal_Entity_Defining_Name;
Fld_21 : Bare_Defining_Name;
Cast_Result_3 : Bare_Ada_Node;
Cast_Result_4 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_5 : Bare_Ada_Node;
Fld_22 : Bare_Call_Expr;
Cast_Result_6 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Fld_23 : Internal_Entity_Defining_Name;
Fld_24 : Bare_Defining_Name;
Cast_Result_7 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Scope_Result_2 : Bare_Ada_Node;
Result_Var_2 : Bare_Ada_Node;
Scope_Result_3 : Bare_Ada_Node;
Fld_25 : Internal_Entity;
Fld_26 : Bare_Ada_Node;
Result_Var_3 : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Result_Var_4 : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_1921 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_1921;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 247 '<Let at nodes.lkt:213:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:213'
--# scope-start
--# expr-start 215 '<Cast to BasicDecl at nodes.lkt:213:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:213'
--# expr-start 214 '<FieldAccess for decl_node at nodes.lkt:213:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:213'
Fld := Ctx.Decl_Node;
--# expr-done 214
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 215
Local_Bd := Cast_Result; 
--# bind bd Local_Bd
Var_Expr_6 := Local_Bd;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 216 '<FieldAccess for subp_spec_or_null at nodes.lkt:215:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:215'
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 216
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 218 '<Cast to Name at nodes.lkt:216:23>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:216'
--# expr-start 217 '<FieldAccess for ref_node at nodes.lkt:216:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:216'
Fld_2 := Ctx.Ref_Node;
--# expr-done 217
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 218
Var_Expr_7 := Cast_Result_1;
if Var_Expr_7 /= No_Entity_Name then
   --# scope-start
--# expr-start 219 '<FieldAccess for parent_callexpr at nodes.lkt:216:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:216'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_callexpr
Fld_3 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Var_Expr_7.Node, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 219
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_1
--# expr-start 242 '<If at nodes.lkt:217:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
--# expr-start 221 '<Eq at nodes.lkt:217:28>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
--# expr-start 220 '<FieldAccess for node at nodes.lkt:217:36>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
Fld_22 := Var_Expr_1.Node;
--# expr-done 220
      Cast_Result_6 := Fld_22;
Is_Equal_1 := Self = Cast_Result_6; 
--# expr-done 221
if Is_Equal_1 then
--# expr-start 223 '<FieldAccess for node at nodes.lkt:217:49>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
--# expr-start 222 '<FieldAccess for returns at nodes.lkt:217:49>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 222
Fld_5 := Fld_4.Node;
--# expr-done 223
      Cast_Result_5 := Fld_5;
   If_Result := Cast_Result_5;
else
--# expr-start 235 '<Find at nodes.lkt:217:74>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
   --# expr-start 229 '<FieldAccess for match_formals at nodes.lkt:217:74>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:217'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 224 '<FieldAccess for abstract_formal_params at nodes.lkt:218:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:218'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 224
--# expr-start 225 '<FieldAccess for params at nodes.lkt:218:52>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:218'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 225
--# expr-start 228 '<FieldAccess for dottable_subp at nodes.lkt:218:65>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:218'
--# expr-start 227 '<FieldAccess for md at nodes.lkt:218:65>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:218'
--# expr-start 226 '<FieldAccess for info at nodes.lkt:218:65>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:218'
Fld_8 := Local_Bd.Info;
--# expr-done 226
Fld_9 := Fld_8.Md;
--# expr-done 227
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 228
--# property-call-start AdaNode.match_formals
Fld_11 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_6, Params => Fld_7, Is_Dottable_Subp => Fld_10);
--# end
--# expr-done 229
      Find_Result := No_Param_Match;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_11;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_2
      --# expr-start 234 '<Eq at nodes.lkt:220:29>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:220'
--# expr-start 233 '<FieldAccess for node at nodes.lkt:220:29>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:220'
--# expr-start 232 '<FieldAccess for expr at nodes.lkt:220:29>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:220'
--# expr-start 231 '<FieldAccess for assoc at nodes.lkt:220:29>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:220'
--# expr-start 230 '<FieldAccess for actual at nodes.lkt:220:29>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:220'
Fld_12 := Item_2.Actual;
--# expr-done 230
Fld_13 := Fld_12.Assoc;
--# expr-done 231
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 232
Fld_15 := Fld_14.Node;
--# expr-done 233
      Cast_Result_2 := Fld_15;
Is_Equal := Cast_Result_2 = Self; 
--# expr-done 234
      if Is_Equal then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Is_Equal;
         end loop;
      end;
--# expr-done 235
Var_Expr_3 := Find_Result;
if Var_Expr_3 /= No_Param_Match then
   --# scope-start
   --# bind pm Var_Expr_3
--# expr-start 239 '<FieldAccess for node at nodes.lkt:222:29>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
--# expr-start 238 '<FieldAccess for type_expression at nodes.lkt:222:29>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
--# expr-start 237 '<FieldAccess for formal_decl at nodes.lkt:222:29>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
--# expr-start 236 '<FieldAccess for formal at nodes.lkt:222:29>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
Fld_16 := Var_Expr_3.Formal;
--# expr-done 236
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 237
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_17.Node, E_Info => Fld_17.Info);
--# end
--# expr-done 238
Fld_19 := Fld_18.Node;
--# expr-done 239
Scope_Result := Fld_19;
   --# end
      Cast_Result_3 := Scope_Result;
   Result_Var := Cast_Result_3;
else
--# expr-start 241 '<FieldAccess for node at nodes.lkt:222:89>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
--# expr-start 240 '<FieldAccess for defining_name at nodes.lkt:222:89>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:222'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_20 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 240
Fld_21 := Fld_20.Node;
--# expr-done 241
      Cast_Result_4 := Fld_21;
   Result_Var := Cast_Result_4;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 242
Scope_Result_1 := If_Result;
   --# end
      Finalizer_Scope_1921;
   Result_Var_1 := Scope_Result_1;
else
--# expr-start 244 '<FieldAccess for node at nodes.lkt:223:32>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:223'
--# expr-start 243 '<FieldAccess for defining_name at nodes.lkt:223:32>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:223'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_23 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 243
Fld_24 := Fld_23.Node;
--# expr-done 244
      Cast_Result_7 := Fld_24;
   Result_Var_1 := Cast_Result_7;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 246 '<FieldAccess for node at nodes.lkt:224:28>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:224'
--# expr-start 245 '<FieldAccess for decl_node at nodes.lkt:224:28>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:224'
Fld_25 := Ctx.Decl_Node;
--# expr-done 245
Fld_26 := Fld_25.Node;
--# expr-done 246
   Result_Var_3 := Fld_26;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var_4; 
--# end
--# expr-done 247
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1921;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Call_Context;
--# end
--# property-start AdaNode.generic_instantiations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:232'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_24 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_24 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_24;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 250 '<FieldAccess for generic_instantiations_internal at nodes.lkt:233:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:233'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 249 '<FieldAccess for rebindings at nodes.lkt:233:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:233'
--# expr-start 248 '<FieldAccess for info at nodes.lkt:233:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:233'
Fld := Ent.Info;
--# expr-done 248
Fld_1 := Fld.Rebindings;
--# expr-done 249
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 250
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_24;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_24;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Generic_Instantiations;
--# end
--# property-start AdaNode.generic_instantiations_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:235'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations_Internal
  (Node : Bare_Ada_Node
      ; R : Env_Rebindings
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind r R
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_25 with Inline_Always;
      Local_Head : Internal_Entity_Generic_Instantiation;
Local_Tail : Internal_Entity_Generic_Instantiation_Array_Access;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Array_Lit : Internal_Entity_Generic_Instantiation_Array_Access;
Concat_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_25 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Local_Tail);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_25;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 261 '<If at nodes.lkt:236:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:236'
--# expr-start 251 '<Eq at nodes.lkt:236:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:236'
Is_Equal := R = null; 
--# expr-done 251
if Is_Equal then
   If_Result_1 := No_Internal_Entity_Generic_Instantiation_Array_Type;
else
   --# expr-start 260 '<Let at nodes.lkt:237:13>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:237'
--# scope-start
--# expr-start 255 '<as_bare_entity at nodes.lkt:237:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:237'
--# expr-start 254 '<Cast to GenericInstantiation at nodes.lkt:237:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:237'
--# expr-start 253 '<env_node at nodes.lkt:237:25>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:237'
--# expr-start 252 '<FieldAccess for new_env at nodes.lkt:237:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:237'
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := R.New_Env;
Inc_Ref (Fld);
--# expr-done 252
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 253
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 254
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 255
Local_Head := If_Result; 
--# bind head Local_Head
--# expr-start 259 '<Let at nodes.lkt:238:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:238'
--# scope-start
--# expr-start 257 '<FieldAccess for generic_instantiations_internal at nodes.lkt:238:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:238'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 256 '<FieldAccess for get_parent at nodes.lkt:238:61>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:238'
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := R.Parent;
--# expr-done 256
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 257
Local_Tail := Fld_2; Inc_Ref (Local_Tail);
--# bind tail Local_Tail
--# expr-start 258 '<ArrayLiteral at nodes.lkt:240:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:240'
Array_Lit := Create_Internal_Entity_Generic_Instantiation_Array (Internal_Internal_Entity_Generic_Instantiation_Array'(1 => Local_Head)); 
--# expr-done 258
Concat_Result := Concat (Array_Lit, Local_Tail); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 259
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 260
   If_Result_1 := Let_Result_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 261
         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_25;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_25;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Generic_Instantiations_Internal;
--# end
--# property-start AdaNode.remove_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:245'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Remove_Rebindings
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; Suffix : Env_Rebindings
  )
   return Env_Rebindings
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind suffix Suffix
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_26 with Inline_Always;
      Fld : Env_Rebindings;
Fld_1 : Env_Rebindings;
Fld_2 : Env_Rebindings;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Env_Rebindings;
Is_Equal_2 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Env_Rebindings;
            procedure Finalizer_Scope_26 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_26;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 274 '<If at nodes.lkt:245:88>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:245'
--# expr-start 262 '<IsNull at nodes.lkt:245:91>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:245'
Is_Equal_2 := Base = null; 
--# expr-done 262
if Is_Equal_2 then
   If_Result_2 := True;
else
   --# expr-start 263 '<IsNull at nodes.lkt:245:107>' Is_Equal_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:245'
Is_Equal_3 := Suffix = null; 
--# expr-done 263
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := Base;
else
   --# expr-start 273 '<If at nodes.lkt:245:88>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:245'
--# expr-start 266 '<Eq at nodes.lkt:246:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
--# expr-start 264 '<FieldAccess for old_env at nodes.lkt:246:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Base.Old_Env;
Inc_Ref (Fld_3);
--# expr-done 264
--# expr-start 265 '<FieldAccess for old_env at nodes.lkt:246:26>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Suffix.Old_Env;
Inc_Ref (Fld_4);
--# expr-done 265
Is_Equal := Equivalent (Fld_3, Fld_4); 
--# expr-done 266
if Is_Equal then
   --# expr-start 269 '<Eq at nodes.lkt:246:45>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
--# expr-start 267 '<FieldAccess for new_env at nodes.lkt:246:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Base.New_Env;
Inc_Ref (Fld_5);
--# expr-done 267
--# expr-start 268 '<FieldAccess for new_env at nodes.lkt:246:61>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Suffix.New_Env;
Inc_Ref (Fld_6);
--# expr-done 268
Is_Equal_1 := Equivalent (Fld_5, Fld_6); 
--# expr-done 269
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 272 '<FieldAccess for remove_rebindings at nodes.lkt:246:81>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 270 '<FieldAccess for get_parent at nodes.lkt:246:104>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Base.Parent;
--# expr-done 270
--# expr-start 271 '<FieldAccess for get_parent at nodes.lkt:246:121>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:246'
      if Suffix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Suffix.Parent;
--# expr-done 271
--# property-call-start AdaNode.remove_rebindings
Fld_2 := Libadalang.Implementation.Ada_Node_P_Remove_Rebindings (Node => Self, Base => Fld, Suffix => Fld_1);
--# end
--# expr-done 272
   If_Result_1 := Fld_2;
else
   If_Result_1 := Base;
end if;
--# expr-done 273
   If_Result_3 := If_Result_1;
end if;
--# expr-done 274
         Property_Result := If_Result_3;
   --# end
      Finalizer_Scope_26;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_26;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Remove_Rebindings;
--# end
--# property-start AdaNode.add_rebinding 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:276'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Add_Rebinding
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; Old_Env : Lexical_Env
      ; New_Env : Lexical_Env
  )
   return Env_Rebindings
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind old_env Old_Env
   --# bind new_env New_Env
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_27 with Inline_Always;
            procedure Finalizer_Scope_1924 with Inline_Always;
      Local_Parent_Env : Lexical_Env := Empty_Env;
Var_Expr_0 : Internal_Entity_Generic_Package_Instantiation;
Local_Gen_Env : Lexical_Env := Empty_Env;
Local_Info : Internal_Entity_Info;
Env_Node : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Rebinding : Env_Rebindings;
Fld_1 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Instantiation;
Fld_2 : Bare_Defining_Name;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Bare_Basic_Decl;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Env_Rebindings;
Fld_8 : Lexical_Env := Empty_Env;
Fld_9 : Env_Rebindings;
New_Struct : Internal_Entity_Info;
Shed_Entity_Info : Internal_Entity_Info;
Fld_10 : Env_Rebindings;
Let_Result : Env_Rebindings;
Let_Result_1 : Env_Rebindings;
Scope_Result : Env_Rebindings;
Rebinding_1 : Env_Rebindings;
Result_Var : Env_Rebindings;
Fld_11 : Env_Rebindings;
Fld_12 : Env_Rebindings;
Fld_13 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
If_Result : Env_Rebindings;
Is_Equal_1 : Boolean;
Env_Node_2 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_14 : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Env_Rebindings;
Let_Result_2 : Env_Rebindings;
            procedure Finalizer_Scope_27 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Parent_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_27;
            procedure Finalizer_Scope_1924 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Local_Gen_Env);
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1924;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 309 '<Let at nodes.lkt:279:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:279'
--# scope-start
--# expr-start 276 '<FieldAccess for node_env at nodes.lkt:279:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:279'
--# expr-start 275 '<env_node at nodes.lkt:279:26>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:279'
Env_Node := AST_Envs.Env_Node (New_Env); 
--# expr-done 275
      if Env_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Env_Node);
--# expr-done 276
Local_Parent_Env := Fld; Inc_Ref (Local_Parent_Env);
--# bind parent_env Local_Parent_Env
--# expr-start 308 '<If at nodes.lkt:281:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:281'
--# expr-start 277 '<IsNull at nodes.lkt:282:13>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:282'
Is_Equal_1 := Base = null; 
--# expr-done 277
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 280 '<Not at nodes.lkt:283:16>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:283'
--# expr-start 279 '<IsA GenericDecl at nodes.lkt:283:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:283'
--# expr-start 278 '<env_node at nodes.lkt:283:20>' Env_Node_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:283'
Env_Node_2 := AST_Envs.Env_Node (Local_Parent_Env); 
--# expr-done 278
Is_A := Env_Node_2 /= null 
and then Env_Node_2.Kind in Ada_Generic_Decl; 
--# expr-done 279
Not_Val := not (Is_A); 
--# expr-done 280
   If_Result_1 := Not_Val;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 282 '<Not at nodes.lkt:284:16>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:284'
--# expr-start 281 '<FieldAccess for is_rebound at nodes.lkt:284:20>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:284'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_rebound
Fld_14 := Libadalang.Implementation.Ada_Node_P_Is_Rebound (Node => Self, Base => Base, Old_Env => Local_Parent_Env);
--# end
--# expr-done 281
Not_Val_1 := not (Fld_14); 
--# expr-done 282
   If_Result_2 := Not_Val_1;
end if;
if If_Result_2 then
   --# expr-start 283 '<append_rebinding at nodes.lkt:286:13>' Rebinding 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:286'
Rebinding := AST_Envs.Append_Rebinding (Base, Old_Env, New_Env); 
--# expr-done 283
   If_Result_3 := Rebinding;
else
   --# expr-start 307 '<If at nodes.lkt:281:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:281'
--# expr-start 285 '<Eq at nodes.lkt:287:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:287'
--# expr-start 284 '<FieldAccess for old_env at nodes.lkt:287:14>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:287'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Base.Old_Env;
Inc_Ref (Fld_13);
--# expr-done 284
Is_Equal := Equivalent (Fld_13, Local_Parent_Env); 
--# expr-done 285
if Is_Equal then
--# expr-start 292 '<Cast to GenericPackageInstantiation at nodes.lkt:288:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:288'
--# expr-start 291 '<EnvGet at nodes.lkt:288:13>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:288'
--# expr-start 286 '<FieldAccess for new_env at nodes.lkt:288:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:288'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Base.New_Env;
Inc_Ref (Fld_1);
--# expr-done 286
--# expr-start 290 '<FieldAccess for name_symbol at nodes.lkt:289:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:289'
--# expr-start 289 '<FieldAccess for name at nodes.lkt:289:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:289'
--# expr-start 288 '<Cast to GenericPackageInstantiation at nodes.lkt:289:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:289'
--# expr-start 287 '<env_node at nodes.lkt:289:17>' Env_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:289'
Env_Node_1 := AST_Envs.Env_Node (New_Env); 
--# expr-done 287
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Env_Node_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 288
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Generic_Package_Instantiation_F_Name;
--# expr-done 289
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 290
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Fld_3), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 291
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 292
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
   --# bind gpi Var_Expr_0
--# expr-start 303 '<Let at nodes.lkt:293:21>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:293'
--# scope-start
--# expr-start 295 '<FieldAccess for children_env at nodes.lkt:293:35>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:293'
--# expr-start 294 '<FieldAccess for node at nodes.lkt:293:35>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:293'
--# expr-start 293 '<FieldAccess for nonbound_generic_decl_from_self at nodes.lkt:293:35>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:293'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_self
Fld_4 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self (Node => Var_Expr_0.Node);
--# end
--# expr-done 293
Fld_5 := Fld_4.Node;
--# expr-done 294
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Fld_5);
--# expr-done 295
Local_Gen_Env := Fld_6; Inc_Ref (Local_Gen_Env);
--# bind gen_env Local_Gen_Env
--# expr-start 302 '<Let at nodes.lkt:294:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:294'
--# scope-start
--# expr-start 299 '<New EntityInfo at nodes.lkt:294:32>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:294'
--# expr-start 298 '<FieldAccess for add_rebinding at nodes.lkt:296:36>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:296'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 296 '<FieldAccess for get_parent at nodes.lkt:297:29>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:297'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Base.Parent;
--# expr-done 296
--# expr-start 297 '<FieldAccess for instantiation_env at nodes.lkt:299:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:299'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_8 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Var_Expr_0.Node);
--# end
--# expr-done 297
--# property-call-start AdaNode.add_rebinding
Fld_9 := Libadalang.Implementation.Ada_Node_P_Add_Rebinding (Node => Self, Base => Fld_7, Old_Env => Local_Gen_Env, New_Env => Fld_8);
--# end
--# expr-done 298
New_Struct := (From_Rebound => False, Md => No_Metadata, Rebindings => Fld_9); 
--# expr-done 299
Local_Info := New_Struct; 
--# bind info Local_Info
--# expr-start 301 '<FieldAccess for rebindings at nodes.lkt:305:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:305'
--# expr-start 300 '<shed_rebindings at nodes.lkt:305:21>' Shed_Entity_Info 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:305'
Shed_Entity_Info := AST_Envs.Shed_Rebindings (Local_Info, Local_Gen_Env); 
--# expr-done 300
Fld_10 := Shed_Entity_Info.Rebindings;
--# expr-done 301
Let_Result := Fld_10; 
--# end
--# expr-done 302
Let_Result_1 := Let_Result; 
--# end
--# expr-done 303
Scope_Result := Let_Result_1;
   --# end
      Finalizer_Scope_1924;
   Result_Var := Scope_Result;
else
   --# expr-start 304 '<append_rebinding at nodes.lkt:307:29>' Rebinding_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:307'
Rebinding_1 := AST_Envs.Append_Rebinding (Base, Old_Env, New_Env); 
--# expr-done 304
   Result_Var := Rebinding_1;
end if;
   If_Result := Result_Var;
else
   --# expr-start 306 '<FieldAccess for add_rebinding at nodes.lkt:310:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:310'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 305 '<FieldAccess for get_parent at nodes.lkt:310:32>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:310'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Base.Parent;
--# expr-done 305
--# property-call-start AdaNode.add_rebinding
Fld_12 := Libadalang.Implementation.Ada_Node_P_Add_Rebinding (Node => Self, Base => Fld_11, Old_Env => Old_Env, New_Env => New_Env);
--# end
--# expr-done 306
   If_Result := Fld_12;
end if;
--# expr-done 307
   If_Result_3 := If_Result;
end if;
--# expr-done 308
Let_Result_2 := If_Result_3; 
--# end
--# expr-done 309
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_27;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_27;
                     Finalizer_Scope_1924;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Add_Rebinding;
--# end
--# property-start AdaNode.is_rebound 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:315'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Rebound
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; Old_Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind old_env Old_Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_28 with Inline_Always;
      Is_Equal : Boolean;
Not_Val : Boolean;
Fld : Lexical_Env := Empty_Env;
Is_Equal_1 : Boolean;
Fld_1 : Env_Rebindings;
Fld_2 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_28 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_28;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 311 '<Not at nodes.lkt:315:70>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:315'
--# expr-start 310 '<IsNull at nodes.lkt:315:74>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:315'
Is_Equal := Base = null; 
--# expr-done 310
Not_Val := not (Is_Equal); 
--# expr-done 311
if Not_Val then
--# expr-start 313 '<Eq at nodes.lkt:316:9>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:316'
--# expr-start 312 '<FieldAccess for old_env at nodes.lkt:316:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:316'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Base.Old_Env;
Inc_Ref (Fld);
--# expr-done 312
Is_Equal_1 := Equivalent (Fld, Old_Env); 
--# expr-done 313
if Is_Equal_1 then
   If_Result := True;
else
   --# expr-start 315 '<FieldAccess for is_rebound at nodes.lkt:316:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:316'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 314 '<FieldAccess for get_parent at nodes.lkt:316:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:316'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Base.Parent;
--# expr-done 314
--# property-call-start AdaNode.is_rebound
Fld_2 := Libadalang.Implementation.Ada_Node_P_Is_Rebound (Node => Self, Base => Fld_1, Old_Env => Old_Env);
--# end
--# expr-done 315
   If_Result := Fld_2;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_28;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_28;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Rebound;
--# end
--# property-start AdaNode.insert_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:322'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Insert_Rebindings
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; To_Insert : Env_Rebindings
  )
   return Env_Rebindings
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind to_insert To_Insert
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_29 with Inline_Always;
      Fld : Env_Rebindings;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Env_Rebindings;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Boolean;
If_Result : Env_Rebindings;
Is_Equal : Boolean;
If_Result_1 : Env_Rebindings;
Is_Equal_1 : Boolean;
If_Result_2 : Env_Rebindings;
            procedure Finalizer_Scope_29 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_29;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 327 '<If at nodes.lkt:322:91>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:322'
--# expr-start 316 '<IsNull at nodes.lkt:322:94>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:322'
Is_Equal_1 := To_Insert = null; 
--# expr-done 316
if Is_Equal_1 then
   If_Result_2 := Base;
else
   --# expr-start 326 '<If at nodes.lkt:322:91>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:322'
--# expr-start 317 '<IsNull at nodes.lkt:323:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:323'
Is_Equal := Base = null; 
--# expr-done 317
if Is_Equal then
   If_Result_1 := To_Insert;
else
   --# expr-start 325 '<If at nodes.lkt:322:91>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:322'
--# expr-start 319 '<FieldAccess for is_rebound at nodes.lkt:324:10>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:324'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 318 '<FieldAccess for old_env at nodes.lkt:324:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:324'
      if To_Insert = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := To_Insert.Old_Env;
Inc_Ref (Fld_5);
--# expr-done 318
--# property-call-start AdaNode.is_rebound
Fld_6 := Libadalang.Implementation.Ada_Node_P_Is_Rebound (Node => Self, Base => Base, Old_Env => Fld_5);
--# end
--# expr-done 319
if Fld_6 then
   If_Result := Base;
else
   --# expr-start 324 '<FieldAccess for add_rebinding at nodes.lkt:325:10>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:325'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 321 '<FieldAccess for insert_rebindings at nodes.lkt:326:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:326'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 320 '<FieldAccess for get_parent at nodes.lkt:326:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:326'
      if To_Insert = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := To_Insert.Parent;
--# expr-done 320
--# property-call-start AdaNode.insert_rebindings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Insert_Rebindings (Node => Self, Base => Base, To_Insert => Fld);
--# end
--# expr-done 321
--# expr-start 322 '<FieldAccess for old_env at nodes.lkt:326:61>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:326'
      if To_Insert = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := To_Insert.Old_Env;
Inc_Ref (Fld_2);
--# expr-done 322
--# expr-start 323 '<FieldAccess for new_env at nodes.lkt:326:80>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:326'
      if To_Insert = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := To_Insert.New_Env;
Inc_Ref (Fld_3);
--# expr-done 323
--# property-call-start AdaNode.add_rebinding
Fld_4 := Libadalang.Implementation.Ada_Node_P_Add_Rebinding (Node => Self, Base => Fld_1, Old_Env => Fld_2, New_Env => Fld_3);
--# end
--# expr-done 324
   If_Result := Fld_4;
end if;
--# expr-done 325
   If_Result_1 := If_Result;
end if;
--# expr-done 326
   If_Result_2 := If_Result_1;
end if;
--# expr-done 327
         Property_Result := If_Result_2;
   --# end
      Finalizer_Scope_29;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_29;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Insert_Rebindings;
--# end
--# property-start AdaNode.has_parent_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:333'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Parent_Rebindings
  (Node : Bare_Ada_Node
      ; Base : Env_Rebindings
      ; Parent : Env_Rebindings
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind base Base
   --# bind parent Parent
   Property_Result : Boolean;
      Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
Not_Val : Boolean;
Fld : Env_Rebindings;
Fld_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 328 '<Eq at nodes.lkt:333:83>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:333'
Is_Equal := Base = Parent; 
--# expr-done 328
if Is_Equal then
   If_Result := True;
else
   --# expr-start 329 '<IsNull at nodes.lkt:333:101>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:333'
Is_Equal_1 := Parent = null; 
--# expr-done 329
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 331 '<Not at nodes.lkt:334:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:334'
--# expr-start 330 '<IsNull at nodes.lkt:334:13>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:334'
Is_Equal_2 := Base = null; 
--# expr-done 330
Not_Val := not (Is_Equal_2); 
--# expr-done 331
if Not_Val then
   --# expr-start 333 '<FieldAccess for has_parent_rebindings at nodes.lkt:334:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:334'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 332 '<FieldAccess for get_parent at nodes.lkt:334:57>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:334'
      if Base = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Base.Parent;
--# expr-done 332
--# property-call-start AdaNode.has_parent_rebindings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Parent_Rebindings (Node => Self, Base => Fld, Parent => Parent);
--# end
--# expr-done 333
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Parent_Rebindings;
--# end
--# property-start AdaNode.logic_val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:342'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Logic_Val
  (Node : Bare_Ada_Node
      ; From_Node : Internal_Entity
      ; Lvar : Logic_Var
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind from_node From_Node
   --# bind lvar Lvar
   Property_Result : Internal_Logic_Val_Result;
      Local_Success : Boolean;
Fld : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
If_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 338 '<Let at nodes.lkt:343:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:343'
--# scope-start
--# expr-start 334 '<FieldAccess for resolve_names_from_closest_entry_point at nodes.lkt:343:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:343'
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => From_Node.Node, E_Info => From_Node.Info);
--# end
--# expr-done 334
Local_Success := Fld; 
--# bind success Local_Success
--# expr-start 337 '<New LogicValResult at nodes.lkt:345:9>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:345'
--# expr-start 336 '<If at nodes.lkt:346:36>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:346'
if Local_Success then
   --# expr-start 335 '<get_value at nodes.lkt:346:52>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:346'
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Lvar); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Lvar); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 335
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 336
New_Struct := (Success => Local_Success, Value => If_Result_1); 
--# expr-done 337
Let_Result := New_Struct; 
--# end
--# expr-done 338
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Logic_Val;
--# end
--# property-start AdaNode.semantic_parent_helper 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:350'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent_Helper
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1925 with Inline_Always;
      Var_Expr_0 : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity;
Env_Node : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
            procedure Finalizer_Scope_1925 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_1925;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Var_Expr_0 := Env;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr_0
--# expr-start 340 '<as_entity at nodes.lkt:351:18>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:351'
--# expr-start 339 '<env_node at nodes.lkt:351:18>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:351'
Env_Node := AST_Envs.Env_Node (Var_Expr_0); 
--# expr-done 339
Is_Null := Env_Node = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Env_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 340
Var_Expr := If_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 342 '<FieldAccess for semantic_parent_helper at nodes.lkt:351:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:351'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 341 '<env_parent at nodes.lkt:351:73>' Env_Parent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:351'
Env_Parent := AST_Envs.Parent (Var_Expr_0); 
--# expr-done 341
--# property-call-start AdaNode.semantic_parent_helper
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Env_Parent, E_Info => Ent.Info);
--# end
--# expr-done 342
   Result_Var := Fld;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1925;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1925;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Semantic_Parent_Helper;
--# end
--# property-start AdaNode.semantic_parent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:361'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_33 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity;
            procedure Finalizer_Scope_33 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_33;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 344 '<FieldAccess for semantic_parent_helper at nodes.lkt:362:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:362'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 343 '<FieldAccess for node_env at nodes.lkt:362:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:362'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 343
--# property-call-start AdaNode.semantic_parent_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Fld, E_Info => Ent.Info);
--# end
--# expr-done 344
         Property_Result := Fld_1;
   --# end
      Finalizer_Scope_33;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_33;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Semantic_Parent;
--# end
--# property-start AdaNode.semantic_parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:366'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parents
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_34 with Inline_Always;
            procedure Finalizer_Scope_1927 with Inline_Always;
      Var_Expr_0 : Internal_Entity;
Fld : Internal_Entity;
Array_Lit : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_34 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_34;
            procedure Finalizer_Scope_1927 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1927;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 345 '<FieldAccess for semantic_parent at nodes.lkt:367:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:367'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 345
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind sp Var_Expr_0
--# expr-start 346 '<ArrayLiteral at nodes.lkt:367:43>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:367'
Array_Lit := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => Var_Expr_0)); 
--# expr-done 346
--# expr-start 347 '<FieldAccess for semantic_parents at nodes.lkt:367:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:367'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parents
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parents (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 347
Concat_Result := Concat (Array_Lit, Fld_1); 
Scope_Result := Concat_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1927;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_34;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_34;
                     Finalizer_Scope_1927;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Semantic_Parents;
--# end
--# property-start AdaNode.parent_basic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:383'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Basic_Decl
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Gen_Decl : Internal_Entity_Generic_Decl;
Local_Gen_Body : Internal_Entity_Generic_Decl;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Var_Expr_2 : Internal_Entity;
Var_Expr_8 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_3 : Internal_Entity_Generic_Instantiation;
Var_Expr_4 : Internal_Entity_Generic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Fld_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Result_Var_1 : Internal_Entity_Generic_Decl;
Scope_Result_2 : Internal_Entity_Generic_Decl;
Result_Var_2 : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Result_Var_3 : Internal_Entity_Generic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Generic_Instantiation;
Scope_Result_4 : Internal_Entity_Generic_Instantiation;
Result_Var_4 : Internal_Entity_Generic_Instantiation;
Scope_Result_5 : Internal_Entity_Generic_Instantiation;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity_Basic_Decl;
Cast_Expr_3 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_6 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Result_Var_5 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result_7 : Internal_Entity_Basic_Decl;
Result_Var_6 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Result_Var_7 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 367 '<If at nodes.lkt:388:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:388'
--# expr-start 348 '<IsA ClasswideTypeDecl, DiscreteBaseSubtypeDecl, SynthAnonymousTypeDecl at nodes.lkt:388:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:388'
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Classwide_Type_Decl_Range | Ada_Discrete_Base_Subtype_Decl_Range | Ada_Synth_Anonymous_Type_Decl_Range; 
--# expr-done 348
if Is_A_1 then
   --# expr-start 350 '<FieldAccess for parent_basic_decl at nodes.lkt:391:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:391'
--# expr-start 349 '<FieldAccess for semantic_parent at nodes.lkt:391:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:391'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 349
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 350
   If_Result_1 := Fld_1;
else
   --# expr-start 366 '<Let at nodes.lkt:393:13>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:393'
--# scope-start
--# expr-start 351 '<Cast to GenericDecl at nodes.lkt:393:28>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:393'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;
--# expr-done 351
Local_Gen_Decl := Cast_Result; 
--# bind gen_decl Local_Gen_Decl
--# expr-start 365 '<Let at nodes.lkt:394:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:394'
--# scope-start
--# expr-start 352 '<Cast to Body at nodes.lkt:394:28>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:394'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 352
Var_Expr_8 := Cast_Result_1;
if Var_Expr_8 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 353 '<FieldAccess for decl_part at nodes.lkt:394:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:394'
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_8.Node, Imprecise_Fallback => False, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 353
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr_0
--# expr-start 354 '<Cast to GenericDecl at nodes.lkt:395:25>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:395'
Cast_Expr_2 := Var_Expr_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Decl;
   end if;
--# expr-done 354
Var_Expr := Cast_Result_2;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 356 '<Cast to GenericDecl at nodes.lkt:395:48>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:395'
--# expr-start 355 '<parent at nodes.lkt:395:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:395'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 355
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_3 := No_Entity_Generic_Decl;
   end if;
--# expr-done 356
   Result_Var := Cast_Result_3;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Generic_Decl;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Generic_Decl;
end if;
Local_Gen_Body := Result_Var_2; 
--# bind gen_body Local_Gen_Body
Var_Expr_4 := Local_Gen_Decl;
if Var_Expr_4 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_3 := Var_Expr_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Local_Gen_Body;
end if;
Var_Expr_1 := Result_Var_3;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 358 '<FieldAccess for get_instantiation at nodes.lkt:398:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:398'
--# expr-start 357 '<FieldAccess for decl at nodes.lkt:398:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:398'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_4 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 357
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_instantiation
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Get_Instantiation (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 358
Scope_Result_4 := Fld_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Generic_Instantiation;
end if;
Var_Expr_3 := Result_Var_4;
if Var_Expr_3 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_5 := Var_Expr_3;
   --# end
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result_5.Node,
         Info => Scope_Result_5.Info);
   Result_Var_7 := Cast_Result_5;
else
--# expr-start 359 '<FieldAccess for semantic_parent at nodes.lkt:399:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:399'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_6 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 359
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind sp Var_Expr_2
--# expr-start 364 '<If at nodes.lkt:400:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:400'
--# expr-start 360 '<IsA GenericSubpInternal, GenericPackageInternal at nodes.lkt:400:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:400'
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 360
if Is_A then
   --# expr-start 361 '<FieldAccess for parent_basic_decl at nodes.lkt:401:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:401'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_7 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 361
   If_Result := Fld_7;
else
--# expr-start 362 '<Cast to BasicDecl at nodes.lkt:403:21>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:403'
Cast_Expr_3 := Var_Expr_2; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 362
Var_Expr_5 := Cast_Result_4;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_6 := Var_Expr_5;
   --# end
   Result_Var_5 := Scope_Result_6;
else
   --# expr-start 363 '<FieldAccess for parent_basic_decl at nodes.lkt:403:42>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:403'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_8 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 363
   Result_Var_5 := Fld_8;
end if;
   If_Result := Result_Var_5;
end if;
--# expr-done 364
Scope_Result_7 := If_Result;
   --# end
   Result_Var_6 := Scope_Result_7;
else
   Result_Var_6 := No_Entity_Basic_Decl;
end if;
   Result_Var_7 := Result_Var_6;
end if;
Let_Result := Result_Var_7; 
--# end
--# expr-done 365
Let_Result_1 := Let_Result; 
--# end
--# expr-done 366
   If_Result_1 := Let_Result_1;
end if;
--# expr-done 367
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Parent_Basic_Decl;
--# end
--# property-start AdaNode.is_spark_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:416'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Spark_Impl
  (Node : Bare_Ada_Node
      ; Include_Skip_Proof_Annotations : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_skip_proof_annotations Include_Skip_Proof_Annotations
   Property_Result : Boolean;
            procedure Finalizer_Scope_36 with Inline_Always;
            procedure Finalizer_Scope_1939 with Inline_Always;
      Local_Spark_Mode : Internal_Aspect;
Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Body_Node;
Var_Expr_3 : Internal_Entity_Body_Node;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_3 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Body_Node;
Scope_Result_1 : Internal_Entity_Body_Node;
Fld_4 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Body_Node;
Result_Var_1 : Internal_Entity_Body_Node;
Fld_5 : Internal_Entity_Expr_Array_Access;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_6 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
Find_Result_1 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Result_Var_2 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_36 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_36;
            procedure Finalizer_Scope_1939 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1939;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 390 '<Let at nodes.lkt:417:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:417'
--# scope-start
--# expr-start 368 '<FieldAccess for spark_mode_aspect at nodes.lkt:417:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:417'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 368
Local_Spark_Mode := Fld; 
--# bind spark_mode Local_Spark_Mode
--# expr-start 389 '<If at nodes.lkt:421:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:421'
if Include_Skip_Proof_Annotations then
   --# expr-start 381 '<Not at nodes.lkt:421:48>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:421'
--# expr-start 380 '<IsNull at nodes.lkt:421:52>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:421'
--# expr-start 369 '<Cast to Body at nodes.lkt:421:52>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:421'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 369
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Body_Node then
   --# scope-start
   --# bind b Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 373 '<Cast to Body at nodes.lkt:422:35>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:422'
--# expr-start 372 '<Find at nodes.lkt:422:35>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:422'
   --# expr-start 370 '<FieldAccess for semantic_parents at nodes.lkt:422:35>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:422'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parents
Fld_4 := Libadalang.Implementation.Ada_Node_P_Semantic_Parents (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 370
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_4;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 371 '<IsA Body at nodes.lkt:422:71>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:422'
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Body_Node; 
--# expr-done 371
      if Is_A then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 372
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 373
   Result_Var_1 := Cast_Result_2;
end if;
Var_Expr_3 := Result_Var_1;
if Var_Expr_3 /= No_Entity_Body_Node then
   --# scope-start
   --# bind b Var_Expr_3
--# expr-start 379 '<Find at nodes.lkt:424:20>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:424'
   --# expr-start 374 '<FieldAccess for gnatprove_annotations at nodes.lkt:424:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:424'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.gnatprove_annotations
Fld_5 := Libadalang.Implementation.Body_Node_P_Gnatprove_Annotations (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 374
      Find_Result_1 := No_Entity_Expr;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_5;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_4
      --# expr-start 376 '<FieldAccess for name_symbol at nodes.lkt:425:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:425'
--# expr-start 375 '<Cast to Name at nodes.lkt:425:24>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:425'
Cast_Expr_2 := Item_4; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 375
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_3.Node);
--# end
--# expr-done 376
--# expr-start 377 '<Symbol Skip_Proof at nodes.lkt:425:52>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:425'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Skip_Proof); 
--# expr-done 377
Is_Equal_1 := Fld_6 = Sym_1; 
if Is_Equal_1 then
   If_Result_1 := True;
else
--# expr-start 378 '<Symbol Skip_Flow_And_Proof at nodes.lkt:425:68>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:425'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Skip_Flow_And_Proof); 
--# expr-done 378
Is_Equal := Fld_6 = Sym_2; 
   If_Result_1 := Is_Equal;
end if;
      if If_Result_1 then
         Find_Result_1 := Item_4; 
      end if;
   --# end
               exit when If_Result_1;
         end loop;
      end;
--# expr-done 379
Scope_Result_2 := Find_Result_1;
   --# end
      Finalizer_Scope_1939;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Expr;
end if;
Is_Null := Result_Var_2.Node = null; 
--# expr-done 380
Not_Val_1 := not (Is_Null); 
--# expr-done 381
   If_Result_2 := Not_Val_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   If_Result_3 := False;
else
   --# expr-start 388 '<If at nodes.lkt:421:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:421'
--# expr-start 383 '<Not at nodes.lkt:428:14>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:428'
--# expr-start 382 '<FieldAccess for exists at nodes.lkt:428:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:428'
Fld_3 := Local_Spark_Mode.Exists;
--# expr-done 382
Not_Val := not (Fld_3); 
--# expr-done 383
if Not_Val then
   If_Result := False;
else
--# expr-start 384 '<FieldAccess for value at nodes.lkt:429:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:429'
Fld_1 := Local_Spark_Mode.Value;
--# expr-done 384
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind mode Var_Expr_0
--# expr-start 387 '<FieldAccess for name_is at nodes.lkt:430:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:430'
--# expr-start 385 '<Cast to Name at nodes.lkt:430:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:430'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 385
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 386 '<Symbol On at nodes.lkt:430:45>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:430'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_On); 
--# expr-done 386
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result.Node, Sym => Sym);
--# end
--# expr-done 387
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 388
   If_Result_3 := If_Result;
end if;
--# expr-done 389
Let_Result := If_Result_3; 
--# end
--# expr-done 390
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_36;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_36;
                     Finalizer_Scope_1939;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Spark_Impl;
--# end
--# property-start AdaNode.has_spark_mode_on 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:442'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Spark_Mode_On
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 391 '<FieldAccess for is_spark_impl at nodes.lkt:442:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:442'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_spark_impl
Fld := Libadalang.Implementation.Ada_Node_P_Is_Spark_Impl (Node => Ent.Node, Include_Skip_Proof_Annotations => False, E_Info => Ent.Info);
--# end
--# expr-done 391
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Spark_Mode_On;
--# end
--# property-start AdaNode.is_subject_to_proof 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:447'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Subject_To_Proof
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 392 '<FieldAccess for is_spark_impl at nodes.lkt:447:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:447'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_spark_impl
Fld := Libadalang.Implementation.Ada_Node_P_Is_Spark_Impl (Node => Ent.Node, Include_Skip_Proof_Annotations => True, E_Info => Ent.Info);
--# end
--# expr-done 392
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Subject_To_Proof;
--# end
--# property-start AdaNode.designated_compilation_unit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:474'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Designated_Compilation_Unit
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
      ; Not_Found_Is_Error : Boolean
         := True
      ; Process_Parents : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   --# bind not_found_is_error Not_Found_Is_Error
   --# bind process_parents Process_Parents
   Property_Result : Bare_Compilation_Unit;
      Local_Designated_Analysis_Unit : Internal_Unit;
Fld : Internal_Unit;
Fld_1 : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 395 '<Let at nodes.lkt:475:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:475'
--# scope-start
--# expr-start 393 '<FieldAccess for get_unit at nodes.lkt:475:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:475'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed, Not_Found_Is_Error => Not_Found_Is_Error, Process_Parents => Process_Parents);
--# expr-done 393
Local_Designated_Analysis_Unit := Fld; 
--# bind designated_analysis_unit Local_Designated_Analysis_Unit
--# expr-start 394 '<FieldAccess for compilation_unit_with_name at nodes.lkt:479:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:479'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.compilation_unit_with_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Compilation_Unit_With_Name (Node => Self, Unit => Local_Designated_Analysis_Unit, Name => Name);
--# end
--# expr-done 394
Let_Result := Fld_1; 
--# end
--# expr-done 395
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Designated_Compilation_Unit;
--# end
--# property-start AdaNode.compilation_unit_with_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:485'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Compilation_Unit_With_Name
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
      ; Name : Symbol_Type_Array_Access
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit Unit
   --# bind name Name
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1944 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Match_0 : Bare_Compilation_Unit;
Match_1 : Bare_Compilation_Unit_List;
Match_2 : Bare_Pragma_Node_List;
Match_3 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Item_1 : Bare_Compilation_Unit;
Fld_1 : Symbol_Type_Array_Access;
Is_Equal : Boolean;
Find_Result : Bare_Compilation_Unit;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit_List;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Cast_Result_2 : Bare_Pragma_Node_List;
Let_Result_2 : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Scope_Result_4 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1944 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1944;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 396 '<FieldAccess for root at nodes.lkt:485:96>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:485'
      if Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Unit.Ast_Root;
--# expr-done 396
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 400 '<Match at nodes.lkt:486:17>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:486'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Compilation_Unit =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind single Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Compilation_Unit_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind multi Match_1
--# expr-start 399 '<Find at nodes.lkt:493:53>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:493'
      Find_Result := No_Bare_Ada_Node;
      if Match_1 = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Compilation_Unit_List := Match_1;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_1 := Untyped_Item_1; 
   --# scope-start
               --# bind c Item_1
      --# expr-start 398 '<Eq at nodes.lkt:494:24>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:494'
--# expr-start 397 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:494:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:494'
      if Item_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Item_1);
--# end
--# expr-done 397
Is_Equal := Equivalent (Fld_1, Name); 
--# expr-done 398
      if Is_Equal then
         Find_Result := Item_1; 
      end if;
   --# end
      Finalizer_Scope_1944;
               exit when Is_Equal;
         end loop;
      end;
      end if;
--# expr-done 399
Let_Result_1 := Find_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Pragma_Node_List =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Case_Stmt_Alternative_List | Ada_Concat_Operand_List .. Ada_Param_Spec_List | Ada_Select_When_Part_List .. Ada_Case_Stmt_Alternative | Ada_Component_Clause .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Unexpected analysis unit root"; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 400
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Bare_Ada_Node;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1944;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Compilation_Unit_With_Name;
--# end
--# property-start AdaNode.get_unit_root_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:507'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Unit_Root_Decl
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
      ; Not_Found_Is_Error : Boolean
         := True
      ; Process_Parents : Boolean
         := True
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   --# bind not_found_is_error Not_Found_Is_Error
   --# bind process_parents Process_Parents
   Property_Result : Bare_Basic_Decl;
      Local_Cu : Bare_Compilation_Unit;
Var_Expr_9 : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 403 '<Let at nodes.lkt:508:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:508'
--# scope-start
--# expr-start 401 '<FieldAccess for designated_compilation_unit at nodes.lkt:508:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:508'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed, Not_Found_Is_Error => Not_Found_Is_Error, Process_Parents => Process_Parents);
--# end
--# expr-done 401
Local_Cu := Fld; 
--# bind cu Local_Cu
Var_Expr_9 := Local_Cu;
if Var_Expr_9 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 402 '<FieldAccess for decl at nodes.lkt:512:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:512'
      if Var_Expr_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_9);
--# end
--# expr-done 402
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 403
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Get_Unit_Root_Decl;
--# end
--# property-start AdaNode.xref_initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:526'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Initial_Env
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_44 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_44 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_44;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 404 '<FieldAccess for children_env at nodes.lkt:526:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:526'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 404
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_44;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_44;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Initial_Env;
--# end
--# property-start AdaNode.bind_to_non_local 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:541'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bind_To_Non_Local
  (Node : Bare_Ada_Node
      ; Dest : Logic_Var
      ; Outer_Node : Bare_Ada_Node
      ; Node_Var : Logic_Var
      ; Entry_Point : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind dest Dest
   --# bind outer_node Outer_Node
   --# bind node_var Node_Var
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_45 with Inline_Always;
      Bind_Result : Logic_Equation;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
If_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_45 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_45;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 411 '<If at nodes.lkt:542:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
--# expr-start 407 '<Contains at nodes.lkt:542:12>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
   --# expr-start 405 '<FieldAccess for parents at nodes.lkt:542:12>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
      if Outer_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Outer_Node, True); 
--# expr-done 405
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_0
      --# expr-start 406 '<Eq at ???>' Is_Equal None
Cast_Expr := Item_0; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Entry_Point; 
--# expr-done 406
      Quantifier_Result := Is_Equal;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 407
if Quantifier_Result then
   --# expr-start 408 '<Bind at nodes.lkt:542:60>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
Dest.Value := No_Entity;
Entity_Vars.Reset (Dest);
Node_Var.Value := No_Entity;
Entity_Vars.Reset (Node_Var);
Bind_Result := Solver.Create_Unify (Dest, Node_Var, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:542:60") else null)); 
--# expr-done 408
   If_Result_1 := Bind_Result;
else
   --# expr-start 410 '<Bind at nodes.lkt:542:85>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
Dest.Value := No_Entity;
Entity_Vars.Reset (Dest);
--# expr-start 409 '<get_value at nodes.lkt:542:95>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:542'
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Node_Var); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Node_Var); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 409
Bind_Result_1 := Solver.Create_Assign (Dest, If_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:542:85") else null)); 
--# expr-done 410
   If_Result_1 := Bind_Result_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 411
         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_45;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_45;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Bind_To_Non_Local;
--# end
--# property-start AdaNode.sub_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:550'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sub_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_46 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_46 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_46;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 415 '<If at nodes.lkt:551:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:551'
--# expr-start 412 '<FieldAccess for xref_stop_resolution at nodes.lkt:551:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:551'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 412
if Fld_2 then
   --# expr-start 413 '<FieldAccess for stop_resolution_equation at nodes.lkt:551:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:551'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.stop_resolution_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 413
   If_Result := Fld;
else
   --# expr-start 414 '<FieldAccess for xref_equation at nodes.lkt:551:82>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:551'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 414
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 415
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_46;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_46;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Sub_Equation;
--# end
--# property-start AdaNode.resolve_children_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:563'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Children_Names
  (Node : Bare_Ada_Node
      ; Generate_Diagnostics : Boolean
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind generate_diagnostics Generate_Diagnostics
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_48 with Inline_Always;
            procedure Finalizer_Scope_1950 with Inline_Always;
      Var_Expr_1 : Bare_Ada_Node;
Node_Children : Bare_Ada_Node_Array_Access;
Bound_Entry_Point : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_2 : Boolean;
If_Result_2 : Boolean;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity;
Fld_3 : Boolean;
If_Result_4 : Boolean;
Fld_4 : Boolean;
If_Result_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
            procedure Finalizer_Scope_48 is
            begin
                     Dec_Ref (Node_Children);
            end Finalizer_Scope_48;
            procedure Finalizer_Scope_1950 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1950;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 429 '<AllQuantifier at nodes.lkt:563:68>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:563'
   --# expr-start 416 '<children at nodes.lkt:563:68>' Node_Children 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Children := Children (Self); 
--# expr-done 416
      Quantifier_Result := True;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Children;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_0
Var_Expr_1 := Item_0;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind c Var_Expr_1
--# expr-start 428 '<If at nodes.lkt:568:13>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:568'
--# expr-start 417 '<FieldAccess for xref_entry_point at nodes.lkt:568:16>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:568'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Var_Expr_1);
--# end
--# expr-done 417
if Fld_4 then
   If_Result_5 := True;
else
--# expr-start 425 '<If at nodes.lkt:569:17>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:569'
--# expr-start 419 '<FieldAccess for xref_stop_resolution at nodes.lkt:569:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:569'
--# expr-start 418 '<as_entity at nodes.lkt:569:20>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:569'
Is_Null_1 := Var_Expr_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => E_Info, Node => Var_Expr_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 418
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => If_Result_1.Node, Env => Env, Origin => Origin, E_Info => If_Result_1.Info);
--# end
--# expr-done 419
if Fld_2 then
   --# expr-start 424 '<bind at nodes.lkt:570:21>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:570'
Bound_Entry_Point := Var_Expr_1; 
--# expr-start 423 '<bind at nodes.lkt:571:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:571'
--# expr-start 420 '<FieldAccess for xref_initial_env at nodes.lkt:571:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:571'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 420
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 422 '<FieldAccess for resolve_own_names at nodes.lkt:573:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:573'
--# expr-start 421 '<as_entity at nodes.lkt:573:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:573'
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 421
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => If_Result.Node, Generate_Diagnostics => Generate_Diagnostics, Env => Bound_Env, Origin => Origin, Entry_Point => Bound_Entry_Point, E_Info => If_Result.Info);
--# expr-done 422
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 423
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 424
   If_Result_2 := Dyn_Var_Bind_Result_1;
else
   If_Result_2 := True;
end if;
--# expr-done 425
if If_Result_2 then
   --# expr-start 427 '<FieldAccess for resolve_children_names at nodes.lkt:575:19>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:575'
--# expr-start 426 '<as_entity at nodes.lkt:575:19>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:575'
Is_Null_2 := Var_Expr_1 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Var_Expr_1); 
   If_Result_3 := As_Entity_2;
end if;
--# expr-done 426
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => If_Result_3.Node, Generate_Diagnostics => Generate_Diagnostics, Env => Env, Origin => Origin, E_Info => If_Result_3.Info);
--# end
--# expr-done 427
   If_Result_4 := Fld_3;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 428
Scope_Result := If_Result_5;
   --# end
      Finalizer_Scope_1950;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 429
         Property_Result := Quantifier_Result;
   --# end
      Finalizer_Scope_48;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_48;
                     Finalizer_Scope_1950;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Children_Names;
--# end
--# property-start AdaNode.resolve_names_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:581'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal
  (Node : Bare_Ada_Node
      ; Generate_Diagnostics : Boolean
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind generate_diagnostics Generate_Diagnostics
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Bound_Entry_Point : Bare_Ada_Node;
Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 432 '<bind at nodes.lkt:582:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:582'
Bound_Entry_Point := Self; 
--# expr-start 430 '<FieldAccess for resolve_own_names at nodes.lkt:584:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:584'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Generate_Diagnostics => Generate_Diagnostics, Env => Env, Origin => Origin, Entry_Point => Bound_Entry_Point, E_Info => Ent.Info);
--# expr-done 430
if Fld then
   --# expr-start 431 '<FieldAccess for resolve_children_names at nodes.lkt:584:58>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:584'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Generate_Diagnostics => Generate_Diagnostics, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 431
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Dyn_Var_Bind_Result := If_Result; 
--# expr-done 432
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_Internal;
--# end
--# property-start AdaNode.resolve_names_internal_with_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:590'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal_With_Eq
  (Node : Bare_Ada_Node
      ; Additional_Equation : Logic_Equation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind additional_equation Additional_Equation
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_50 with Inline_Always;
      Local_Eq : Logic_Equation;
Bound_Entry_Point : Bare_Ada_Node;
Fld : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Solve_Success : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_50 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Local_Eq);
            end Finalizer_Scope_50;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 438 '<Let at nodes.lkt:591:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:591'
--# scope-start
--# expr-start 435 '<And at nodes.lkt:591:18>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:591'
--# expr-start 434 '<bind at nodes.lkt:592:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:592'
Bound_Entry_Point := Self; 
--# expr-start 433 '<FieldAccess for xref_equation at nodes.lkt:594:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:594'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Bound_Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 433
Dyn_Var_Bind_Result := Fld; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 434
And_Pred := Create_And (Dyn_Var_Bind_Result, Additional_Equation, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:591:18") else null)); 
--# expr-done 435
Local_Eq := And_Pred; Inc_Ref (Local_Eq);
--# bind eq Local_Eq
--# expr-start 436 '<solve at nodes.lkt:597:9>' Solve_Success 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:597'
Solve_Success := Solve_Wrapper (Local_Eq, Self); 
--# expr-done 436
if Solve_Success then
   --# expr-start 437 '<FieldAccess for resolve_children_names at nodes.lkt:597:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:597'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 437
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 438
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_50;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_50;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_Internal_With_Eq;
--# end
--# property-start AdaNode.resolve_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:606'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_51 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
            procedure Finalizer_Scope_51 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_51;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Resolve_Names,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 443 '<bind at nodes.lkt:607:9>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:607'
--# expr-start 439 '<FieldAccess for xref_initial_env at nodes.lkt:607:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:607'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 439
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 442 '<bind at nodes.lkt:608:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:608'
--# expr-start 440 '<FieldAccess for origin_node at nodes.lkt:608:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:608'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 440
Bound_Origin := Fld_1; 
--# expr-start 441 '<FieldAccess for resolve_names_internal at nodes.lkt:610:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:610'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Ent.Node, Generate_Diagnostics => False, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 441
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 442
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 443
         Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_51;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_51;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Resolve_Names;
--# end
--# property-start AdaNode.resolve_names_from_closest_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:632'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_From_Closest_Entry_Point
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_52 with Inline_Always;
            procedure Finalizer_Scope_1952 with Inline_Always;
      Var_Expr_0 : Boolean;
Var_Expr_10 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Bound_Entry_Point : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env_1 : Lexical_Env := Empty_Env;
Fld_6 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Bound_Entry_Point_1 : Bare_Ada_Node;
Fld_7 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
Fld_8 : Boolean;
If_Result : Boolean;
Dyn_Var_Bind_Result_4 : Boolean;
Dyn_Var_Bind_Result_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_9 : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_52 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_52;
            procedure Finalizer_Scope_1952 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env_1);
            end Finalizer_Scope_1952;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 461 '<If at nodes.lkt:635:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:635'
--# expr-start 444 '<FieldAccess for xref_entry_point at nodes.lkt:635:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:635'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_9 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Ent.Node);
--# end
--# expr-done 444
if Fld_9 then
   --# expr-start 450 '<bind at nodes.lkt:636:13>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:636'
--# expr-start 445 '<FieldAccess for xref_initial_env at nodes.lkt:636:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 445
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 449 '<bind at nodes.lkt:637:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:637'
--# expr-start 446 '<FieldAccess for origin_node at nodes.lkt:637:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:637'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 446
Bound_Origin := Fld_1; 
--# expr-start 448 '<bind at nodes.lkt:638:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:638'
Bound_Entry_Point := Self; 
--# expr-start 447 '<FieldAccess for resolve_own_names at nodes.lkt:640:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:640'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Generate_Diagnostics => False, Env => Bound_Env, Origin => Bound_Origin, Entry_Point => Bound_Entry_Point, E_Info => Ent.Info);
--# expr-done 447
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 448
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 449
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; 
--# expr-done 450
   If_Result_1 := Dyn_Var_Bind_Result_2;
else
--# expr-start 451 '<parent at nodes.lkt:643:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:643'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 451
Var_Expr_10 := Fld_3;
if Var_Expr_10 /= No_Entity then
   --# scope-start
--# expr-start 452 '<FieldAccess for resolve_names_from_closest_entry_point at nodes.lkt:643:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:643'
      if Var_Expr_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld_4 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 452
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= False then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 460 '<bind at nodes.lkt:645:21>' Dyn_Var_Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:645'
--# expr-start 453 '<FieldAccess for xref_initial_env at nodes.lkt:645:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:645'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_initial_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 453
Bound_Env_1 := Fld_5; Inc_Ref (Bound_Env_1);
--# expr-start 459 '<bind at nodes.lkt:646:21>' Dyn_Var_Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:646'
--# expr-start 454 '<FieldAccess for origin_node at nodes.lkt:646:35>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:646'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 454
Bound_Origin_1 := Fld_6; 
--# expr-start 458 '<If at nodes.lkt:650:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:650'
--# expr-start 455 '<FieldAccess for xref_stop_resolution at nodes.lkt:650:24>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:650'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, Env => Bound_Env_1, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 455
if Fld_8 then
   --# expr-start 457 '<bind at nodes.lkt:651:25>' Dyn_Var_Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:651'
Bound_Entry_Point_1 := Self; 
--# expr-start 456 '<FieldAccess for resolve_own_names at nodes.lkt:653:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:653'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Generate_Diagnostics => False, Env => Bound_Env_1, Origin => Bound_Origin_1, Entry_Point => Bound_Entry_Point_1, E_Info => Ent.Info);
--# expr-done 456
Dyn_Var_Bind_Result_3 := Fld_7; 
--# expr-done 457
   If_Result := Dyn_Var_Bind_Result_3;
else
   If_Result := True;
end if;
--# expr-done 458
Dyn_Var_Bind_Result_4 := If_Result; 
--# expr-done 459
Dyn_Var_Bind_Result_5 := Dyn_Var_Bind_Result_4; 
--# expr-done 460
Scope_Result := Dyn_Var_Bind_Result_5;
   --# end
      Finalizer_Scope_1952;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
--# expr-done 461
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_52;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_52;
                     Finalizer_Scope_1952;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Names_From_Closest_Entry_Point;
--# end
--# property-start AdaNode.children_nameres_diagnostics 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:676'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Children_Nameres_Diagnostics
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Solver_Diagnostic_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Solver_Diagnostic_Array_Access;
            procedure Finalizer_Scope_54 with Inline_Always;
            procedure Finalizer_Scope_1953 with Inline_Always;
            procedure Finalizer_Scope_1954 with Inline_Always;
      Var_Expr_1 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Fld_1 : Internal_Solver_Diagnostic_Array_Access;
Fld_2 : Internal_Solver_Diagnostic_Array_Access;
Concat_Result : Internal_Solver_Diagnostic_Array_Access;
Is_Null : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Solver_Diagnostic_Array_Access;
Scope_Result : Internal_Solver_Diagnostic_Array_Access;
Result_Var : Internal_Solver_Diagnostic_Array_Access;
Map_Result : Internal_Solver_Diagnostic_Array_Access;
            procedure Finalizer_Scope_54 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_54;
            procedure Finalizer_Scope_1953 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1953;
            procedure Finalizer_Scope_1954 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_1954;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 468 '<Mapcat at nodes.lkt:676:67>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:676'
   --# expr-start 462 '<children at nodes.lkt:676:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:676'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 462
      declare
         Map_Result_Vec : Internal_Solver_Diagnostic_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_0
Var_Expr_1 := Item_0;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind c Var_Expr_1
--# expr-start 467 '<If at nodes.lkt:678:20>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:678'
--# expr-start 463 '<IsNull at nodes.lkt:678:23>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:678'
Is_Null := Var_Expr_1.Node = null; 
--# expr-done 463
if Is_Null then
   If_Result := True;
else
   --# expr-start 464 '<FieldAccess for xref_entry_point at nodes.lkt:678:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:678'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Var_Expr_1.Node);
--# end
--# expr-done 464
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := No_Internal_Solver_Diagnostic_Array_Type;
else
   --# expr-start 465 '<FieldAccess for own_nameres_diagnostics at nodes.lkt:678:97>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:678'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Own_Nameres_Diagnostics (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 465
--# expr-start 466 '<FieldAccess for children_nameres_diagnostics at nodes.lkt:678:127>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:678'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.children_nameres_diagnostics
Fld_2 := Libadalang.Implementation.Ada_Node_P_Children_Nameres_Diagnostics (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 466
Concat_Result := Concat (Fld_1, Fld_2); 
   If_Result_1 := Concat_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 467
Scope_Result := If_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1954;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Solver_Diagnostic_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Solver_Diagnostic_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1953;
         end loop;
      end;
         Map_Result := Create_Internal_Solver_Diagnostic_Array
           (Items_Count => Natural (Internal_Solver_Diagnostic_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Solver_Diagnostic_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Solver_Diagnostic_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Solver_Diagnostic_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 468
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_54;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_54;
                     Finalizer_Scope_1953;
                     Finalizer_Scope_1954;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Children_Nameres_Diagnostics;
--# end
--# property-start AdaNode.nameres_diagnostics 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:685'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Nameres_Diagnostics
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Solver_Diagnostic_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Solver_Diagnostic_Array_Access;
            procedure Finalizer_Scope_55 with Inline_Always;
      Ignored : Boolean;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Fld_3 : Internal_Solver_Diagnostic_Array_Access;
Fld_4 : Internal_Solver_Diagnostic_Array_Access;
Concat_Result : Internal_Solver_Diagnostic_Array_Access;
Let_Result : Internal_Solver_Diagnostic_Array_Access;
Dyn_Var_Bind_Result : Internal_Solver_Diagnostic_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Solver_Diagnostic_Array_Access;
            procedure Finalizer_Scope_55 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_55;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 476 '<bind at nodes.lkt:686:9>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:686'
--# expr-start 469 '<FieldAccess for xref_initial_env at nodes.lkt:686:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:686'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Xref_Initial_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 469
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 475 '<bind at nodes.lkt:687:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:687'
--# expr-start 470 '<FieldAccess for origin_node at nodes.lkt:687:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:687'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 470
Bound_Origin := Fld_1; 
--# expr-start 474 '<Let at nodes.lkt:689:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:689'
--# scope-start
--# expr-start 471 '<FieldAccess for resolve_names_internal at nodes.lkt:689:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:689'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Ent.Node, Generate_Diagnostics => True, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 471
Ignored := Fld_2; 
--# bind _ Ignored
--# expr-start 472 '<FieldAccess for own_nameres_diagnostics at nodes.lkt:691:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:691'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Ada_Node_P_Own_Nameres_Diagnostics (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 472
--# expr-start 473 '<FieldAccess for children_nameres_diagnostics at nodes.lkt:691:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:691'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.children_nameres_diagnostics
Fld_4 := Libadalang.Implementation.Ada_Node_P_Children_Nameres_Diagnostics (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 473
Concat_Result := Concat (Fld_3, Fld_4); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 474
Dyn_Var_Bind_Result := Let_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 475
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 476
         Property_Result := Dyn_Var_Bind_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_55;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_55;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Nameres_Diagnostics;
--# end
--# property-start AdaNode.missing_entity_error 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:697'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Missing_Entity_Error
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 478 '<Not at nodes.lkt:697:40>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:697'
--# expr-start 477 '<IsNull at nodes.lkt:697:44>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:697'
Is_Null := Self = null; 
--# expr-done 477
Not_Val := not (Is_Null); 
--# expr-done 478
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Missing_Entity_Error;
--# end
--# property-start AdaNode.std 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:714'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 485 '<as_bare_entity at nodes.lkt:715:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 484 '<FieldAccess for item at nodes.lkt:715:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 483 '<Cast to LibraryItem at nodes.lkt:715:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 482 '<FieldAccess for body at nodes.lkt:715:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 481 '<Cast to CompilationUnit at nodes.lkt:715:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 480 '<FieldAccess for root at nodes.lkt:715:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
--# expr-start 479 '<FieldAccess for standard_unit at nodes.lkt:715:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:715'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Standard_Unit (Node => Self);
--# expr-done 479
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 480
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 481
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Body;
--# expr-done 482
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 483
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 484
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 485
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std;
--# end
--# property-start AdaNode.std_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:718'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_60 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_60 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_60;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 487 '<FieldAccess for children_env at nodes.lkt:718:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:718'
--# expr-start 486 '<FieldAccess for std at nodes.lkt:718:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:718'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std
Fld := Libadalang.Implementation.Ada_Node_P_Std (Node => Self);
--# end
--# expr-done 486
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 487
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_60;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_60;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Env;
--# end
--# property-start AdaNode.std_entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:722'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 490 '<FieldAccess for std_entity_implem at nodes.lkt:723:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:723'
--# expr-start 489 '<FieldAccess for root at nodes.lkt:723:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:723'
--# expr-start 488 '<FieldAccess for unit at nodes.lkt:723:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:723'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 488
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 489
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_entity_implem
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity_Implem (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 490
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Entity;
--# end
--# property-start AdaNode.std_entity_implem 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:726'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity_Implem
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_62 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
            procedure Finalizer_Scope_62 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_62;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Std_Entity_Implem,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 492 '<EnvGet at nodes.lkt:726:59>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:726'
--# expr-start 491 '<FieldAccess for std_env at nodes.lkt:726:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:726'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 491
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 492
         Property_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_62;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_62;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity,
                        As_Internal_Entity => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Std_Entity_Implem;
--# end
--# property-start AdaNode.bool_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:732'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 495 '<Cast to BaseTypeDecl at nodes.lkt:733:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:733'
--# expr-start 494 '<FieldAccess for std_entity at nodes.lkt:733:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:733'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 493 '<Symbol Boolean at nodes.lkt:733:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:733'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Boolean); 
--# expr-done 493
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 494
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 495
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Bool_Type;
--# end
--# property-start AdaNode.int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:737'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 498 '<Cast to BaseTypeDecl at nodes.lkt:738:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:738'
--# expr-start 497 '<FieldAccess for std_entity at nodes.lkt:738:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:738'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 496 '<Symbol Integer at nodes.lkt:738:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:738'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer); 
--# expr-done 496
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 497
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 498
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Int_Type;
--# end
--# property-start AdaNode.universal_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:742'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 501 '<Cast to BaseTypeDecl at nodes.lkt:743:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:743'
--# expr-start 500 '<FieldAccess for std_entity at nodes.lkt:743:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:743'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 499 '<Symbol Universal_Int_Type_ at nodes.lkt:743:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:743'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Universal_Int_Type); 
--# expr-done 499
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 500
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 501
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Int_Type;
--# end
--# property-start AdaNode.universal_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:747'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 504 '<Cast to BaseTypeDecl at nodes.lkt:748:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:748'
--# expr-start 503 '<FieldAccess for std_entity at nodes.lkt:748:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 502 '<Symbol Universal_Real_Type_ at nodes.lkt:748:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:748'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Universal_Real_Type); 
--# expr-done 502
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 503
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 504
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Real_Type;
--# end
--# property-start AdaNode.universal_fixed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:751'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Fixed_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 507 '<Cast to BaseTypeDecl at nodes.lkt:752:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:752'
--# expr-start 506 '<FieldAccess for std_entity at nodes.lkt:752:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:752'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 505 '<Symbol Universal_Fixed_Type_ at nodes.lkt:752:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:752'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Universal_Fixed_Type); 
--# expr-done 505
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 506
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 507
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Fixed_Type;
--# end
--# property-start AdaNode.std_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:756'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 510 '<Cast to BaseTypeDecl at nodes.lkt:757:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:757'
--# expr-start 509 '<FieldAccess for std_entity at nodes.lkt:757:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:757'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 508 '<Symbol Character at nodes.lkt:757:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:757'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Character); 
--# expr-done 508
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 509
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 510
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Char_Type;
--# end
--# property-start AdaNode.std_wide_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:761'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 513 '<Cast to BaseTypeDecl at nodes.lkt:762:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:762'
--# expr-start 512 '<FieldAccess for std_entity at nodes.lkt:762:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:762'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 511 '<Symbol Wide_Character at nodes.lkt:762:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:762'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Character); 
--# expr-done 511
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 512
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 513
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_Char_Type;
--# end
--# property-start AdaNode.std_wide_wide_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:766'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_Wide_Char_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 516 '<Cast to BaseTypeDecl at nodes.lkt:767:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:767'
--# expr-start 515 '<FieldAccess for std_entity at nodes.lkt:767:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:767'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 514 '<Symbol Wide_Wide_Character at nodes.lkt:767:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:767'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Character); 
--# expr-done 514
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 515
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 516
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_Wide_Char_Type;
--# end
--# property-start AdaNode.std_string_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:771'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_String_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 519 '<Cast to BaseTypeDecl at nodes.lkt:772:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:772'
--# expr-start 518 '<FieldAccess for std_entity at nodes.lkt:772:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:772'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 517 '<Symbol String at nodes.lkt:772:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:772'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 517
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 518
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 519
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_String_Type;
--# end
--# property-start AdaNode.std_wide_string_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:776'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_String_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 522 '<Cast to BaseTypeDecl at nodes.lkt:777:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:777'
--# expr-start 521 '<FieldAccess for std_entity at nodes.lkt:777:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:777'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 520 '<Symbol Wide_String at nodes.lkt:777:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:777'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 520
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 521
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 522
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_String_Type;
--# end
--# property-start AdaNode.std_wide_wide_string_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:781'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Wide_Wide_String_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 525 '<Cast to BaseTypeDecl at nodes.lkt:782:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:782'
--# expr-start 524 '<FieldAccess for std_entity at nodes.lkt:782:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:782'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 523 '<Symbol Wide_Wide_String at nodes.lkt:782:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:782'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 523
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 524
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 525
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Wide_Wide_String_Type;
--# end
--# property-start AdaNode.std_root_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:786'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Root_Types
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_74 with Inline_Always;
            procedure Finalizer_Scope_1955 with Inline_Always;
      Var_Expr_11 : Internal_Entity_Package_Decl;
Sym : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_74 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_74;
            procedure Finalizer_Scope_1955 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1955;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 528 '<Cast to PackageDecl at nodes.lkt:787:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:787'
--# expr-start 527 '<FieldAccess for std_entity at nodes.lkt:787:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:787'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 526 '<Symbol root_types_ at nodes.lkt:787:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:787'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Types); 
--# expr-done 526
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 527
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 528
Var_Expr_11 := Cast_Result;
if Var_Expr_11 /= No_Entity_Package_Decl then
   --# scope-start
--# expr-start 529 '<FieldAccess for children_env at nodes.lkt:787:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:787'
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# expr-done 529
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1955;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_74;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_74;
                     Finalizer_Scope_1955;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Std_Root_Types;
--# end
--# property-start AdaNode.root_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:790'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_75 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_75 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_75;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 533 '<Cast to BaseTypeDecl at nodes.lkt:790:49>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:790'
--# expr-start 532 '<EnvGet at nodes.lkt:790:49>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:790'
--# expr-start 530 '<FieldAccess for std_root_types at nodes.lkt:790:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:790'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 530
--# expr-start 531 '<Symbol root_integer at nodes.lkt:791:9>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:791'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Integer); 
--# expr-done 531
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 532
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 533
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_75;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_75;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Int_Type;
--# end
--# property-start AdaNode.root_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:795'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_76 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_76 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_76;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 537 '<Cast to BaseTypeDecl at nodes.lkt:795:50>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:795'
--# expr-start 536 '<EnvGet at nodes.lkt:795:50>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:795'
--# expr-start 534 '<FieldAccess for std_root_types at nodes.lkt:795:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:795'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 534
--# expr-start 535 '<Symbol root_real at nodes.lkt:796:9>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:796'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Real); 
--# expr-done 535
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 536
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 537
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_76;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_76;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Real_Type;
--# end
--# property-start AdaNode.system_address_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:800'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_System_Address_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_77 with Inline_Always;
            procedure Finalizer_Scope_1956 with Inline_Always;
      Var_Expr_12 : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_77 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_77;
            procedure Finalizer_Scope_1956 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1956;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 544 '<Cast to BaseTypeDecl at nodes.lkt:800:55>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:800'
--# expr-start 540 '<FieldAccess for get_unit_root_decl at nodes.lkt:800:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:800'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 539 '<ArrayLiteral at nodes.lkt:801:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:801'
--# expr-start 538 '<Symbol System at nodes.lkt:801:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:801'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 538
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 539
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 540
Var_Expr_12 := Fld;
if Var_Expr_12 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 543 '<EnvGet at nodes.lkt:800:55>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:800'
--# expr-start 541 '<FieldAccess for children_env at nodes.lkt:800:55>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:800'
      if Var_Expr_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_12);
--# expr-done 541
--# expr-start 542 '<Symbol Address at nodes.lkt:802:33>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:802'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 542
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_1), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 543
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1956;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 544
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_77;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_77;
                     Finalizer_Scope_1956;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_System_Address_Type;
--# end
--# property-start AdaNode.root_type_ops_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:806'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Type_Ops_Impl
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_78 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_78 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_78;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Root_Type_Ops_Impl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 550 '<Map at nodes.lkt:806:69>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:806'
   --# expr-start 546 '<EnvGet at nodes.lkt:806:69>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:806'
--# expr-start 545 '<FieldAccess for std_root_types at nodes.lkt:806:69>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:806'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_root_types
Fld := Libadalang.Implementation.Ada_Node_P_Std_Root_Types (Node => Self);
--# end
--# expr-done 545
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 546
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
         --# expr-start 548 '<FieldAccess for is_subprogram at nodes.lkt:809:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:809'
--# expr-start 547 '<Cast to BasicDecl at nodes.lkt:809:40>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:809'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 547
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 548
         if Fld_1 then
      --# expr-start 549 '<Cast to BasicDecl at nodes.lkt:809:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:809'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 549
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 550
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_78;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_78;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                        As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Root_Type_Ops_Impl;
--# end
--# property-start AdaNode.root_type_ops 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:814'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Type_Ops
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_79 with Inline_Always;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_79 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_79;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 553 '<FieldAccess for root_type_ops_impl at nodes.lkt:816:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:816'
--# expr-start 552 '<FieldAccess for root at nodes.lkt:816:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:816'
--# expr-start 551 '<FieldAccess for unit at nodes.lkt:816:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:816'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 551
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 552
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_type_ops_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops_Impl (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 553
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_79;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_79;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Type_Ops;
--# end
--# property-start AdaNode.exc_id_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:819'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Exc_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_80 with Inline_Always;
            procedure Finalizer_Scope_1958 with Inline_Always;
      Var_Expr_13 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_80 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_80;
            procedure Finalizer_Scope_1958 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1958;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 561 '<Cast to BaseTypeDecl at nodes.lkt:819:47>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:819'
--# expr-start 557 '<FieldAccess for get_unit_root_decl at nodes.lkt:819:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:819'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 556 '<ArrayLiteral at nodes.lkt:820:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:820'
--# expr-start 554 '<Symbol Ada at nodes.lkt:820:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:820'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 554
--# expr-start 555 '<Symbol Exceptions at nodes.lkt:820:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:820'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exceptions); 
--# expr-done 555
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 556
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 557
Var_Expr_13 := Fld;
if Var_Expr_13 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 560 '<EnvGet at nodes.lkt:819:47>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:819'
--# expr-start 558 '<FieldAccess for children_env at nodes.lkt:819:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:819'
      if Var_Expr_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_13);
--# expr-done 558
--# expr-start 559 '<Symbol Exception_Id at nodes.lkt:821:33>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:821'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception_Id); 
--# expr-done 559
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 560
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1958;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 561
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_80;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_80;
                     Finalizer_Scope_1958;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Exc_Id_Type;
--# end
--# property-start AdaNode.task_id_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:824'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Task_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_81 with Inline_Always;
            procedure Finalizer_Scope_1959 with Inline_Always;
      Var_Expr_14 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_81 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_81;
            procedure Finalizer_Scope_1959 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1959;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 569 '<Cast to BaseTypeDecl at nodes.lkt:824:48>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:824'
--# expr-start 565 '<FieldAccess for get_unit_root_decl at nodes.lkt:824:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:824'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 564 '<ArrayLiteral at nodes.lkt:825:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:825'
--# expr-start 562 '<Symbol Ada at nodes.lkt:825:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:825'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 562
--# expr-start 563 '<Symbol Task_Identification at nodes.lkt:825:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:825'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task_Identification); 
--# expr-done 563
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 564
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 565
Var_Expr_14 := Fld;
if Var_Expr_14 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 568 '<EnvGet at nodes.lkt:824:48>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:824'
--# expr-start 566 '<FieldAccess for children_env at nodes.lkt:824:48>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:824'
      if Var_Expr_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_14);
--# expr-done 566
--# expr-start 567 '<Symbol Task_Id at nodes.lkt:826:33>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:826'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Task_Id); 
--# expr-done 567
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 568
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1959;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 569
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_81;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_81;
                     Finalizer_Scope_1959;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Task_Id_Type;
--# end
--# property-start AdaNode.root_buffer_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:829'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Buffer_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_82 with Inline_Always;
            procedure Finalizer_Scope_1960 with Inline_Always;
      Var_Expr_15 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_3 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_82 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_82;
            procedure Finalizer_Scope_1960 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1960;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 578 '<Cast to BaseTypeDecl at nodes.lkt:829:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:829'
--# expr-start 574 '<FieldAccess for get_unit_root_decl at nodes.lkt:829:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:829'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 573 '<ArrayLiteral at nodes.lkt:830:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:830'
--# expr-start 570 '<Symbol Ada at nodes.lkt:830:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:830'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 570
--# expr-start 571 '<Symbol Strings at nodes.lkt:830:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:830'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Strings); 
--# expr-done 571
--# expr-start 572 '<Symbol Text_Buffers at nodes.lkt:830:30>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:830'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Text_Buffers); 
--# expr-done 572
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2)); 
--# expr-done 573
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 574
Var_Expr_15 := Fld;
if Var_Expr_15 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 577 '<EnvGet at nodes.lkt:829:52>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:829'
--# expr-start 575 '<FieldAccess for children_env at nodes.lkt:829:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:829'
      if Var_Expr_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_15);
--# expr-done 575
--# expr-start 576 '<Symbol Root_Buffer_Type at nodes.lkt:832:9>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:832'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Buffer_Type); 
--# expr-done 576
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_3), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 577
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1960;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 578
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_82;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_82;
                     Finalizer_Scope_1960;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Buffer_Type;
--# end
--# property-start AdaNode.root_stream_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Root_Stream_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_83 with Inline_Always;
            procedure Finalizer_Scope_1961 with Inline_Always;
      Var_Expr_16 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_83 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_83;
            procedure Finalizer_Scope_1961 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1961;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 588 '<Cast to BaseTypeDecl at nodes.lkt:836:52>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
--# expr-start 587 '<FieldAccess for classwide_type at nodes.lkt:836:52>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
--# expr-start 586 '<Cast to BaseTypeDecl at nodes.lkt:836:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
--# expr-start 582 '<FieldAccess for get_unit_root_decl at nodes.lkt:836:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 581 '<ArrayLiteral at nodes.lkt:837:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:837'
--# expr-start 579 '<Symbol Ada at nodes.lkt:837:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:837'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 579
--# expr-start 580 '<Symbol Streams at nodes.lkt:837:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:837'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Streams); 
--# expr-done 580
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 581
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 582
Var_Expr_16 := Fld;
if Var_Expr_16 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 585 '<EnvGet at nodes.lkt:836:52>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
--# expr-start 583 '<FieldAccess for children_env at nodes.lkt:836:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:836'
      if Var_Expr_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_16);
--# expr-done 583
--# expr-start 584 '<Symbol Root_Stream_Type at nodes.lkt:839:9>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:839'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Stream_Type); 
--# expr-done 584
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 585
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1961;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 586
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 587
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 588
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_83;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_83;
                     Finalizer_Scope_1961;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Root_Stream_Type;
--# end
--# property-start AdaNode.big_integer_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:843'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Big_Integer_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_84 with Inline_Always;
            procedure Finalizer_Scope_1962 with Inline_Always;
      Var_Expr_17 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_4 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_84 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_84;
            procedure Finalizer_Scope_1962 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1962;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 598 '<Cast to BaseTypeDecl at nodes.lkt:843:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:843'
--# expr-start 594 '<FieldAccess for get_unit_root_decl at nodes.lkt:843:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:843'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 593 '<ArrayLiteral at nodes.lkt:844:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:844'
--# expr-start 589 '<Symbol Ada at nodes.lkt:844:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:844'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 589
--# expr-start 590 '<Symbol Numerics at nodes.lkt:844:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:844'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Numerics); 
--# expr-done 590
--# expr-start 591 '<Symbol Big_Numbers at nodes.lkt:844:31>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:844'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Big_Numbers); 
--# expr-done 591
--# expr-start 592 '<Symbol Big_Integers at nodes.lkt:844:47>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:844'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Big_Integers); 
--# expr-done 592
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2, 4 => Sym_3)); 
--# expr-done 593
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 594
Var_Expr_17 := Fld;
if Var_Expr_17 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 597 '<EnvGet at nodes.lkt:843:52>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:843'
--# expr-start 595 '<FieldAccess for children_env at nodes.lkt:843:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:843'
      if Var_Expr_17 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_17);
--# expr-done 595
--# expr-start 596 '<Symbol Big_Integer at nodes.lkt:847:9>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:847'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Big_Integer); 
--# expr-done 596
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_4), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 597
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_1962;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 598
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_84;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_84;
                     Finalizer_Scope_1962;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Big_Integer_Type;
--# end
--# property-start AdaNode.parent_has_with_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:856'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Has_With_Visibility
  (Node : Bare_Ada_Node
      ; Refd_Unit : Internal_Unit
      ; Self_Cu : Bare_Compilation_Unit
      ; Has_Private_View : Boolean
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind refd_unit Refd_Unit
   --# bind self_cu Self_Cu
   --# bind has_private_view Has_Private_View
   Property_Result : Boolean;
      Local_Should_Have_Private_View : Boolean;
Var_Expr_0 : Internal_Entity;
Fld : Boolean;
If_Result : Boolean;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Boolean;
Fld_5 : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 609 '<Let at nodes.lkt:857:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:857'
--# scope-start
if Has_Private_View then
   If_Result := True;
else
   --# expr-start 599 '<FieldAccess for has_private_view at nodes.lkt:857:60>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:857'
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.has_private_view
Fld := Libadalang.Implementation.Compilation_Unit_P_Has_Private_View (Node => Self_Cu, Origin => Self);
--# end
--# expr-done 599
   If_Result := Fld;
end if;
Local_Should_Have_Private_View := If_Result; 
--# bind should_have_private_view Local_Should_Have_Private_View
--# expr-start 602 '<FieldAccess for semantic_parent at nodes.lkt:859:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:859'
--# expr-start 601 '<as_bare_entity at nodes.lkt:859:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:859'
--# expr-start 600 '<FieldAccess for decl at nodes.lkt:859:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:859'
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self_Cu);
--# end
--# expr-done 600
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 601
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 602
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind parent Var_Expr_0
--# expr-start 608 '<If at nodes.lkt:865:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
--# expr-start 603 '<IsA PrivatePart at nodes.lkt:865:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Private_Part_Range; 
--# expr-done 603
if Is_A then
   --# expr-start 604 '<Not at nodes.lkt:865:42>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
Not_Val := not (Local_Should_Have_Private_View); 
--# expr-done 604
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 606 '<FieldAccess for has_with_visibility at nodes.lkt:865:76>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
--# expr-start 605 '<parent at nodes.lkt:865:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 605
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_4 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Fld_3.Node, Refd_Unit => Refd_Unit, Omit_Privacy_Check => False);
--# end
--# expr-done 606
   If_Result_3 := Fld_4;
else
   --# expr-start 607 '<FieldAccess for has_with_visibility at nodes.lkt:865:152>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:865'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr_0.Node, Refd_Unit => Refd_Unit, Omit_Privacy_Check => Local_Should_Have_Private_View);
--# end
--# expr-done 607
   If_Result_3 := Fld_5;
end if;
--# expr-done 608
Scope_Result := If_Result_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 609
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Parent_Has_With_Visibility;
--# end
--# property-start AdaNode.has_private_part_parent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:878'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Private_Part_Parent
  (Node : Bare_Ada_Node
      ; Barrier : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind barrier Barrier
   Property_Result : Boolean;
            procedure Finalizer_Scope_86 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Is_A : Boolean;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_86 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_86;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 610 '<IsA PrivatePart at nodes.lkt:878:59>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:878'
Is_A := Self /= null 
and then Self.Kind in Ada_Private_Part_Range; 
--# expr-done 610
if Is_A then
   If_Result_1 := True;
else
--# expr-start 612 '<Not at nodes.lkt:879:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:879'
--# expr-start 611 '<Eq at nodes.lkt:879:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:879'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Barrier; 
--# expr-done 611
Not_Val := not (Is_Equal); 
--# expr-done 612
if Not_Val then
--# expr-start 614 '<env_node at nodes.lkt:879:29>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:879'
--# expr-start 613 '<FieldAccess for node_env at nodes.lkt:879:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:879'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 613
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 614
Var_Expr_0 := Env_Node;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind parent Var_Expr_0
--# expr-start 615 '<FieldAccess for has_private_part_parent at nodes.lkt:880:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:880'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_private_part_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Private_Part_Parent (Node => Var_Expr_0, Barrier => Barrier);
--# end
--# expr-done 615
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_86;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_86;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Private_Part_Parent;
--# end
--# property-start AdaNode.has_private_with_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:888'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Private_With_Visibility
  (Node : Bare_Ada_Node
      ; Self_Cu : Bare_Compilation_Unit
      ; Refd_Unit : Internal_Unit
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind self_cu Self_Cu
   --# bind refd_unit Refd_Unit
   Property_Result : Boolean;
            procedure Finalizer_Scope_87 with Inline_Always;
      Fld : Internal_Unit;
Is_Equal : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Unit_Array_Access;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_87 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_87;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 617 '<Eq at nodes.lkt:890:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:890'
--# expr-start 616 '<FieldAccess for unit at nodes.lkt:890:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:890'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 616
Is_Equal := Fld = Refd_Unit; 
--# expr-done 617
if Is_Equal then
   If_Result := True;
else
   --# expr-start 618 '<FieldAccess for has_private_view at nodes.lkt:893:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:893'
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.has_private_view
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Has_Private_View (Node => Self_Cu, Origin => Self);
--# end
--# expr-done 618
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 622 '<Not at nodes.lkt:897:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:897'
--# expr-start 621 '<Contains at nodes.lkt:897:13>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:897'
   --# expr-start 619 '<FieldAccess for privately_imported_units at nodes.lkt:897:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:897'
      if Self_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.privately_imported_units
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Privately_Imported_Units (Node => Self_Cu);
--# end
--# expr-done 619
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_2;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_1
      --# expr-start 620 '<Eq at ???>' Is_Equal_1 None
Is_Equal_1 := Item_1 = Refd_Unit; 
--# expr-done 620
      Quantifier_Result := Is_Equal_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 621
Not_Val := not (Quantifier_Result); 
--# expr-done 622
   If_Result_1 := Not_Val;
end if;
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_87;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_87;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Private_With_Visibility;
--# end
--# property-start AdaNode.has_with_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:904'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_With_Visibility
  (Node : Bare_Ada_Node
      ; Refd_Unit : Internal_Unit
      ; Omit_Privacy_Check : Boolean
         := False
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind refd_unit Refd_Unit
   --# bind omit_privacy_check Omit_Privacy_Check
   Property_Result : Boolean;
      Local_Cu : Bare_Compilation_Unit;
Var_Expr_0 : Internal_Entity_Base_Subp_Body;
Var_Expr_1 : Bare_Compilation_Unit;
Var_Expr_18 : Bare_Compilation_Unit;
Var_Expr_19 : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Internal_Unit;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_4 : Boolean;
If_Result_2 : Boolean;
Fld_5 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Bare_Compilation_Unit;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_9 : Boolean;
If_Result_5 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_6 : Boolean;
Fld_10 : Bare_Ada_Node;
Cast_Result_2 : Bare_Compilation_Unit;
Fld_11 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 641 '<Let at nodes.lkt:905:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:905'
--# scope-start
--# expr-start 623 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:905:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:905'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 623
Local_Cu := Fld; 
--# bind cu Local_Cu
--# expr-start 625 '<FieldAccess for is_referenced_from at nodes.lkt:909:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:909'
      if Refd_Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 624 '<FieldAccess for unit at nodes.lkt:909:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:909'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 624
Fld_2 := Libadalang.Implementation.Is_Referenced_From (Self => Refd_Unit, Unit => Fld_1);
--# expr-done 625
if Fld_2 then
if Omit_Privacy_Check then
   If_Result := True;
else
   --# expr-start 626 '<FieldAccess for has_private_with_visibility at nodes.lkt:910:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:910'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_private_with_visibility
Fld_3 := Libadalang.Implementation.Ada_Node_P_Has_Private_With_Visibility (Node => Self, Self_Cu => Local_Cu, Refd_Unit => Refd_Unit);
--# end
--# expr-done 626
   If_Result := Fld_3;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 627 '<FieldAccess for parent_has_with_visibility at nodes.lkt:914:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:914'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_has_with_visibility
Fld_4 := Libadalang.Implementation.Ada_Node_P_Parent_Has_With_Visibility (Node => Self, Refd_Unit => Refd_Unit, Self_Cu => Local_Cu, Has_Private_View => Omit_Privacy_Check);
--# end
--# expr-done 627
   If_Result_2 := Fld_4;
end if;
if If_Result_2 then
   If_Result_6 := True;
else
--# expr-start 630 '<Cast to BaseSubpBody at nodes.lkt:922:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:922'
--# expr-start 629 '<as_bare_entity at nodes.lkt:922:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:922'
--# expr-start 628 '<FieldAccess for decl at nodes.lkt:922:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:922'
      if Local_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Local_Cu);
--# end
--# expr-done 628
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result_3 := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_5); 
   If_Result_3 := As_Entity;
end if;
--# expr-done 629
   if If_Result_3.Node = null
      or else If_Result_3.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => If_Result_3.Node,
         Info => If_Result_3.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 630
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Base_Subp_Body then
   --# scope-start
   --# bind b Var_Expr_0
--# expr-start 637 '<If at nodes.lkt:923:24>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:923'
--# expr-start 631 '<FieldAccess for is_library_item at nodes.lkt:923:27>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:923'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Var_Expr_0.Node);
--# end
--# expr-done 631
if Fld_9 then
--# expr-start 633 '<FieldAccess for referenced_unit at nodes.lkt:923:52>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:923'
--# expr-start 632 '<FieldAccess for defining_name at nodes.lkt:923:52>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:923'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 632
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_unit
Fld_7 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Fld_6.Node, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 633
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind u Var_Expr_1
--# expr-start 636 '<If at nodes.lkt:931:28>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:931'
--# expr-start 634 '<Eq at nodes.lkt:931:31>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:931'
Cast_Expr := Local_Cu; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Var_Expr_1; 
--# expr-done 634
if Is_Equal then
   If_Result_4 := False;
else
Var_Expr_18 := Var_Expr_1;
if Var_Expr_18 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 635 '<FieldAccess for has_with_visibility at nodes.lkt:931:55>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:931'
      if Var_Expr_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_8 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr_18, Refd_Unit => Refd_Unit, Omit_Privacy_Check => True);
--# end
--# expr-done 635
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_4 := Result_Var;
end if;
--# expr-done 636
Scope_Result_1 := If_Result_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_5 := Result_Var_1;
else
   If_Result_5 := False;
end if;
--# expr-done 637
Scope_Result_2 := If_Result_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_6 := Result_Var_2;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
--# expr-start 639 '<Cast to CompilationUnit at nodes.lkt:942:13>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:942'
--# expr-start 638 '<FieldAccess for root at nodes.lkt:942:13>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:942'
      if Refd_Unit = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Refd_Unit.Ast_Root;
--# expr-done 638
   if Fld_10 = null
      or else Fld_10.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Fld_10;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 639
Var_Expr_19 := Cast_Result_2;
if Var_Expr_19 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 640 '<FieldAccess for is_text_io_child at nodes.lkt:942:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:942'
      if Var_Expr_19 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_text_io_child
Fld_11 := Libadalang.Implementation.Compilation_Unit_P_Is_Text_Io_Child (Node => Var_Expr_19);
--# end
--# expr-done 640
Scope_Result_3 := Fld_11;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_7 := Result_Var_3;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 641
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_With_Visibility;
--# end
--# property-start AdaNode.has_visibility 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:946'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Visibility
  (Node : Bare_Ada_Node
      ; Other_Entity : Internal_Entity
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind other_entity Other_Entity
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Synthetic_Object_Decl;
Var_Expr_20 : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_21 : Internal_Entity_Package_Renaming_Decl;
Var_Expr_22 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Synthetic_Object_Decl;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_1 : Internal_Entity_Info;
Fld_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Package_Renaming_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_6 : Bare_Ada_Node;
Fld_7 : Internal_Unit;
Fld_8 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 642 '<Cast to SyntheticObjectDecl at nodes.lkt:949:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:949'
Cast_Expr := Other_Entity; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Synthetic_Object_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Synthetic_Object_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Synthetic_Object_Decl;
   end if;
--# expr-done 642
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Synthetic_Object_Decl then
   --# scope-start
   --# bind sod Var_Expr_0
--# expr-start 643 '<FieldAccess for is_referred_by at nodes.lkt:950:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:950'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SyntheticObjectDecl.is_referred_by
Fld := Libadalang.Implementation.Synthetic_Object_Decl_P_Is_Referred_By (Node => Var_Expr_0.Node, Origin => Self);
--# end
--# expr-done 643
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
if Result_Var then
--# expr-start 644 '<Cast to GenericPackageInstantiation at nodes.lkt:955:17>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:955'
Cast_Expr_1 := Other_Entity; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 644
Var_Expr_20 := Cast_Result_1;
if Var_Expr_20 /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 646 '<FieldAccess for from_rebound at nodes.lkt:955:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:955'
--# expr-start 645 '<FieldAccess for info at nodes.lkt:955:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:955'
Fld_1 := Var_Expr_20.Info;
--# expr-done 645
Fld_2 := Fld_1.From_Rebound;
--# expr-done 646
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result := True;
else
--# expr-start 647 '<Cast to PackageRenamingDecl at nodes.lkt:956:18>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:956'
Cast_Expr_2 := Other_Entity; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 647
Var_Expr_21 := Cast_Result_2;
if Var_Expr_21 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
--# expr-start 649 '<FieldAccess for from_rebound at nodes.lkt:956:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:956'
--# expr-start 648 '<FieldAccess for info at nodes.lkt:956:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:956'
Fld_3 := Var_Expr_21.Info;
--# expr-done 648
Fld_4 := Fld_3.From_Rebound;
--# expr-done 649
Scope_Result_2 := Fld_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result := Result_Var_2;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 652 '<Not at nodes.lkt:958:17>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:958'
--# expr-start 650 '<Cast to BasicDecl at nodes.lkt:958:21>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:958'
Cast_Expr_3 := Other_Entity; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 650
Var_Expr_22 := Cast_Result_3;
if Var_Expr_22 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 651 '<FieldAccess for is_compilation_unit_root at nodes.lkt:958:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:958'
      if Var_Expr_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Var_Expr_22.Node);
--# end
--# expr-done 651
Scope_Result_3 := Fld_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
Not_Val := not (Result_Var_3); 
--# expr-done 652
   If_Result_1 := Not_Val;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 655 '<FieldAccess for has_with_visibility at nodes.lkt:961:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:961'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 654 '<FieldAccess for unit at nodes.lkt:961:42>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:961'
--# expr-start 653 '<FieldAccess for node at nodes.lkt:961:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:961'
Fld_6 := Other_Entity.Node;
--# expr-done 653
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Unit (Node => Fld_6);
--# expr-done 654
--# property-call-start AdaNode.has_with_visibility
Fld_8 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_7);
--# end
--# expr-done 655
   If_Result_2 := Fld_8;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Has_Visibility;
--# end
--# property-start AdaNode.resolve_generic_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:966'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Generic_Actual
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1975 with Inline_Always;
      Match_0 : Internal_Entity_Anonymous_Expr_Decl;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_23 : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Result_Var_2 : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Anonymous_Expr_Decl;
Cast_Expr_1 : Internal_Entity_Anonymous_Expr_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_1 : Internal_Entity;
Scope_Result_4 : Internal_Entity;
Cast_Result_6 : Internal_Entity;
Let_Result_2 : Internal_Entity;
Scope_Result_5 : Internal_Entity;
Match_Result : Internal_Entity;
            procedure Finalizer_Scope_1975 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1975;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 662 '<Match at nodes.lkt:966:53>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:966'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Expr_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Anonymous_Expr_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind aod Match_0
Cast_Expr_1 := Match_0; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind n Match_1
--# expr-start 657 '<Cast to AdaNode at nodes.lkt:975:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:975'
--# expr-start 656 '<FieldAccess for name_designated_type at nodes.lkt:975:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:975'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 656
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 657
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 658 '<Cast to AttributeRef at nodes.lkt:975:54>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:975'
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Attribute_Ref;
   end if;
--# expr-done 658
Var_Expr_23 := Cast_Result_1;
if Var_Expr_23 /= No_Entity_Attribute_Ref then
   --# scope-start
--# expr-start 659 '<FieldAccess for attribute_subprogram at nodes.lkt:975:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:975'
      if Var_Expr_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_1 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Var_Expr_23.Node, E_Info => Var_Expr_23.Info);
--# end
--# expr-done 659
Scope_Result_1 := Fld_1;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
      Cast_Result_2 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   Result_Var_1 := Cast_Result_2;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_2 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 661 '<at at nodes.lkt:976:15>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:976'
--# expr-start 660 '<FieldAccess for all_env_elements at nodes.lkt:976:15>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:976'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 660
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 661
   Result_Var_2 := Get_Result;
end if;
Let_Result_1 := Result_Var_2; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
      Finalizer_Scope_1975;
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
Let_Result_2 := No_Entity; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 662
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1975;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Resolve_Generic_Actual;
--# end
--# property-start AdaNode.top_level_use_package_clauses 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:987'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_91 with Inline_Always;
            procedure Finalizer_Scope_1981 with Inline_Always;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Package_Clause;
Fld_3 : Bare_Name_List;
Item_2 : Bare_Name;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_91 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_91;
            procedure Finalizer_Scope_1981 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1981;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 673 '<Mapcat at nodes.lkt:988:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
   --# expr-start 668 '<Map at nodes.lkt:988:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
   --# expr-start 666 '<FieldAccess for prelude at nodes.lkt:988:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
--# expr-start 665 '<Cast to CompilationUnit at nodes.lkt:988:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
--# expr-start 664 '<parent at nodes.lkt:988:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
--# expr-start 663 '<parent at nodes.lkt:988:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 663
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 664
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 665
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 666
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind p Item_0
         --# expr-start 667 '<IsA UsePackageClause at nodes.lkt:988:71>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:988'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Use_Package_Clause_Range; 
--# expr-done 667
         if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_0;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 668
      declare
         Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 672 '<Map at nodes.lkt:989:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:989'
   --# expr-start 670 '<FieldAccess for packages at nodes.lkt:989:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:989'
--# expr-start 669 '<Cast to UsePackageClause at nodes.lkt:989:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:989'
Cast_Expr := Item_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 669
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Use_Package_Clause_F_Packages;
--# expr-done 670
      declare
         Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_2 := Untyped_Item_2; 
   --# scope-start
               --# bind n Item_2
      --# expr-start 671 '<Cast to AdaNode at nodes.lkt:989:64>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:989'
Cast_Expr_1 := Item_2; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 671
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 672
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1981;
         end loop;
      end;
         Map_Result_2 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 673
         Property_Result := Map_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_91;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_91;
                     Finalizer_Scope_1981;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Package_Clauses;
--# end
--# property-start AdaNode.top_level_use_type_clauses 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:995'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Type_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_92 with Inline_Always;
            procedure Finalizer_Scope_1984 with Inline_Always;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Type_Clause;
Fld_3 : Bare_Name_List;
Item_2 : Bare_Name;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_92 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_92;
            procedure Finalizer_Scope_1984 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1984;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 684 '<Mapcat at nodes.lkt:996:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
   --# expr-start 679 '<Map at nodes.lkt:996:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
   --# expr-start 677 '<FieldAccess for prelude at nodes.lkt:996:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
--# expr-start 676 '<Cast to CompilationUnit at nodes.lkt:996:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
--# expr-start 675 '<parent at nodes.lkt:996:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
--# expr-start 674 '<parent at nodes.lkt:996:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 674
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 675
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 676
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 677
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind p Item_0
         --# expr-start 678 '<IsA UseTypeClause at nodes.lkt:996:71>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:996'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Use_Type_Clause_Range; 
--# expr-done 678
         if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_0;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 679
      declare
         Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 683 '<Map at nodes.lkt:997:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:997'
   --# expr-start 681 '<FieldAccess for types at nodes.lkt:997:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:997'
--# expr-start 680 '<Cast to UseTypeClause at nodes.lkt:997:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:997'
Cast_Expr := Item_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Type_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 680
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Use_Type_Clause_F_Types;
--# expr-done 681
      declare
         Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_2 := Untyped_Item_2; 
   --# scope-start
               --# bind n Item_2
      --# expr-start 682 '<Cast to AdaNode at nodes.lkt:997:58>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:997'
Cast_Expr_1 := Item_2; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 682
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 683
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1984;
         end loop;
      end;
         Map_Result_2 := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 684
         Property_Result := Map_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_92;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_92;
                     Finalizer_Scope_1984;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Type_Clauses;
--# end
--# property-start AdaNode.top_level_use_clauses 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1002'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Use_Clause_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_93 with Inline_Always;
      Local_Cu : Bare_Compilation_Unit;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Clause;
Is_A : Boolean;
Map_Result : Bare_Use_Clause_Array_Access;
Let_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_93 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_93;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 692 '<Let at nodes.lkt:1003:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1003'
--# scope-start
--# expr-start 687 '<Cast to CompilationUnit at nodes.lkt:1003:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1003'
--# expr-start 686 '<parent at nodes.lkt:1003:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1003'
--# expr-start 685 '<parent at nodes.lkt:1003:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1003'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 685
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 686
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 687
Local_Cu := Cast_Result; 
--# bind cu Local_Cu
--# expr-start 691 '<Map at nodes.lkt:1005:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1005'
   --# expr-start 688 '<FieldAccess for prelude at nodes.lkt:1005:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1005'
      if Local_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Local_Cu.Compilation_Unit_F_Prelude;
--# expr-done 688
      declare
         Map_Result_Vec : Bare_Use_Clause_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Use_Clause_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind p Item_0
         --# expr-start 689 '<IsA UseClause at nodes.lkt:1006:44>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1006'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Use_Clause; 
--# expr-done 689
         if Is_A then
      --# expr-start 690 '<Cast to UseClause at nodes.lkt:1006:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1006'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Clause
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 690
         declare
            Item_To_Append : constant Bare_Use_Clause := Cast_Result_1;
         begin
            Bare_Use_Clause_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Use_Clause_Array
           (Items_Count => Natural (Bare_Use_Clause_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Use_Clause_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Use_Clause_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Use_Clause_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 691
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 692
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_93;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_93;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Use_Clauses;
--# end
--# property-start AdaNode.top_level_with_package_clauses 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1013'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_With_Package_Clauses
  (Node : Bare_Ada_Node
      ; Include_Privates : Boolean
         := True
  )
   return Bare_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_94 with Inline_Always;
            procedure Finalizer_Scope_1987 with Inline_Always;
            procedure Finalizer_Scope_1988 with Inline_Always;
      Var_Expr_1 : Bare_With_Clause;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_With_Clause;
Fld_2 : Bare_Name_List;
Item_18 : Bare_Name;
Map_Result : Bare_Name_Array_Access;
Fld_3 : Bare_Private_Node;
Fld_4 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Bare_Name_Array_Access;
Scope_Result : Bare_Name_Array_Access;
Result_Var : Bare_Name_Array_Access;
Map_Result_1 : Bare_Name_Array_Access;
            procedure Finalizer_Scope_94 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_94;
            procedure Finalizer_Scope_1987 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1987;
            procedure Finalizer_Scope_1988 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_1988;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 702 '<Mapcat at nodes.lkt:1014:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1014'
   --# expr-start 694 '<FieldAccess for prelude at nodes.lkt:1014:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1014'
--# expr-start 693 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:1014:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1014'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 693
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Compilation_Unit_F_Prelude;
--# expr-done 694
      declare
         Map_Result_1_Vec : Bare_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result_1 := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind clause Item_0
--# expr-start 695 '<Cast to WithClause at nodes.lkt:1015:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1015'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_With_Clause_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 695
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind with_clause Var_Expr_1
--# expr-start 701 '<If at nodes.lkt:1016:34>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1016'
--# expr-start 697 '<FieldAccess for as_bool at nodes.lkt:1016:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1016'
--# expr-start 696 '<FieldAccess for has_private at nodes.lkt:1016:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1016'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_1.With_Clause_F_Has_Private;
--# expr-done 696
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_4 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_3);
--# end
--# expr-done 697
if Fld_4 then
   --# expr-start 698 '<Not at nodes.lkt:1016:75>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1016'
Not_Val := not (Include_Privates); 
--# expr-done 698
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := No_Bare_Name_Array_Type;
else
   --# expr-start 700 '<TakeWhile at ???>' Map_Result None
   --# expr-start 699 '<FieldAccess for packages at nodes.lkt:1016:124>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1016'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_1.With_Clause_F_Packages;
--# expr-done 699
      declare
         Map_Result_Vec : Bare_Name_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_2;
      begin
         for Untyped_Item_18 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_18 := Untyped_Item_18; 
   --# scope-start
         declare
            Item_To_Append : constant Bare_Name := Item_18;
         begin
            Bare_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Name_Array
           (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Name_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 700
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 701
Scope_Result := If_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1988;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Bare_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1987;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Name_Array
           (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 702
         Property_Result := Map_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_94;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_94;
                     Finalizer_Scope_1987;
                     Finalizer_Scope_1988;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_With_Package_Clauses;
--# end
--# property-start AdaNode.use_clauses_in_spec_of_subp_body 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1022'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_95 with Inline_Always;
            procedure Finalizer_Scope_1990 with Inline_Always;
            procedure Finalizer_Scope_1991 with Inline_Always;
            procedure Finalizer_Scope_1992 with Inline_Always;
      Local_Fqn : Symbol_Type_Array_Access;
Local_Spec : Bare_Compilation_Unit;
Var_Expr_24 : Bare_Compilation_Unit;
Var_Expr_25 : Bare_Basic_Decl;
Fld : Bare_Compilation_Unit;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
Fld_3 : Bare_Basic_Decl;
Fld_4 : Bare_Use_Clause_Array_Access;
As_Entity : Internal_Entity_Use_Clause;
Is_Null : Boolean;
If_Result : Internal_Entity_Use_Clause;
Fld_5 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_95 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Fqn);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_95;
            procedure Finalizer_Scope_1990 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1990;
            procedure Finalizer_Scope_1991 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1991;
            procedure Finalizer_Scope_1992 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1992;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 713 '<Let at nodes.lkt:1023:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1023'
--# scope-start
--# expr-start 704 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:1023:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1023'
--# expr-start 703 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:1023:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1023'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 703
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Fld);
--# end
--# expr-done 704
Local_Fqn := Fld_1; Inc_Ref (Local_Fqn);
--# bind fqn Local_Fqn
--# expr-start 712 '<Let at nodes.lkt:1024:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1024'
--# scope-start
--# expr-start 705 '<FieldAccess for designated_compilation_unit at nodes.lkt:1024:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1024'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Local_Fqn, Kind => Unit_Specification, Not_Found_Is_Error => False);
--# end
--# expr-done 705
Local_Spec := Fld_2; 
--# bind spec Local_Spec
Var_Expr_24 := Local_Spec;
if Var_Expr_24 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 706 '<FieldAccess for decl at nodes.lkt:1028:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1028'
      if Var_Expr_24 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_24);
--# end
--# expr-done 706
Var_Expr_25 := Fld_3;
if Var_Expr_25 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 711 '<env_group at nodes.lkt:1028:9>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1028'
--# expr-start 710 '<Map at nodes.lkt:1028:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1028'
   --# expr-start 707 '<FieldAccess for top_level_use_clauses at nodes.lkt:1028:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1028'
      if Var_Expr_25 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_clauses
Fld_4 := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Clauses (Node => Var_Expr_25);
--# end
--# expr-done 707
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Use_Clause_Array_Access := Fld_4;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind clause Item_0
      --# expr-start 709 '<FieldAccess for used_envs at nodes.lkt:1029:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1029'
--# expr-start 708 '<as_bare_entity at nodes.lkt:1029:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1029'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Use_Clause;
else
As_Entity := (Info => No_Entity_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 708
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_5 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 709
         declare
            Item_To_Append : constant Lexical_Env := Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1992;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 710
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 711
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1991;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1990;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 712
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 713
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_95;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_95;
                     Finalizer_Scope_1990;
                     Finalizer_Scope_1991;
                     Finalizer_Scope_1992;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body;
--# end
--# property-start AdaNode.use_clauses_in_generic_formal_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1038'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_96 with Inline_Always;
            procedure Finalizer_Scope_1994 with Inline_Always;
      Local_Gen_Decl : Internal_Entity_Generic_Decl;
Var_Expr_0 : Internal_Entity_Generic_Decl;
Var_Expr_26 : Internal_Entity_Body_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Generic_Formal_Part;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_96 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_96;
            procedure Finalizer_Scope_1994 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1994;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 719 '<Let at nodes.lkt:1039:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1039'
--# scope-start
--# expr-start 714 '<Cast to Body at nodes.lkt:1039:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1039'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 714
Var_Expr_26 := Cast_Result;
if Var_Expr_26 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 715 '<FieldAccess for safe_generic_decl_part at nodes.lkt:1039:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1039'
      if Var_Expr_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr_26.Node, E_Info => Var_Expr_26.Info);
--# end
--# expr-done 715
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Local_Gen_Decl := Result_Var; 
--# bind gen_decl Local_Gen_Decl
Var_Expr_0 := Local_Gen_Decl;
if Var_Expr_0 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_0
--# expr-start 717 '<FieldAccess for use_clauses_envs at nodes.lkt:1042:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1042'
--# expr-start 716 '<FieldAccess for formal_part at nodes.lkt:1042:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1042'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_0.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_0.Info);
--# expr-done 716
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericFormalPart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Generic_Formal_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 717
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1994;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 718 '<FieldAccess for empty_env at nodes.lkt:1042:68>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1042'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 718
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 719
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_96;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_96;
                     Finalizer_Scope_1994;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Use_Clauses_In_Generic_Formal_Part;
--# end
--# property-start AdaNode.nested_generic_formal_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1057'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Nested_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_97 with Inline_Always;
            procedure Finalizer_Scope_1996 with Inline_Always;
      Local_Gen_Decl : Internal_Entity_Generic_Decl;
Var_Expr_0 : Internal_Entity_Generic_Decl;
Var_Expr_27 : Internal_Entity_Body_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_97 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_97;
            procedure Finalizer_Scope_1996 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1996;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 726 '<Let at nodes.lkt:1058:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1058'
--# scope-start
--# expr-start 721 '<Cast to Body at nodes.lkt:1058:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1058'
--# expr-start 720 '<as_bare_entity at nodes.lkt:1058:24>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1058'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 720
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 721
Var_Expr_27 := Cast_Result;
if Var_Expr_27 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 722 '<FieldAccess for safe_generic_decl_part at nodes.lkt:1058:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1058'
      if Var_Expr_27.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr_27.Node, E_Info => Var_Expr_27.Info);
--# end
--# expr-done 722
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Local_Gen_Decl := Result_Var; 
--# bind gen_decl Local_Gen_Decl
Var_Expr_0 := Local_Gen_Decl;
if Var_Expr_0 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_0
--# expr-start 724 '<FieldAccess for children_env at nodes.lkt:1061:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1061'
--# expr-start 723 '<FieldAccess for node at nodes.lkt:1061:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1061'
Fld_1 := Var_Expr_0.Node;
--# expr-done 723
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 724
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1996;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 725 '<FieldAccess for empty_env at nodes.lkt:1061:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1061'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 725
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 726
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_97;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_97;
                     Finalizer_Scope_1996;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Nested_Generic_Formal_Part;
--# end
--# property-start AdaNode.is_package 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1066'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Package
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 727 '<IsA PackageDecl, PackageBody, GenericPackageInstantiation, PackageRenamingDecl, GenericPackageDecl at nodes.lkt:1067:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1067'
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Package_Body_Range | Ada_Generic_Package_Instantiation_Range | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range; 
--# expr-done 727
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Is_Package;
--# end
--# property-start AdaNode.default_initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1071'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Default_Initial_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_99 with Inline_Always;
            procedure Finalizer_Scope_1997 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_99 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_99;
            procedure Finalizer_Scope_1997 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1997;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 728 '<parent at nodes.lkt:1071:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1071'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 728
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 729 '<FieldAccess for children_env at nodes.lkt:1072:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1072'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_0);
--# expr-done 729
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1997;
   Result_Var := Scope_Result;
else
   --# expr-start 730 '<FieldAccess for children_env at nodes.lkt:1072:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1072'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 730
   Result_Var := Fld_2;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_99;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_99;
                     Finalizer_Scope_1997;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Default_Initial_Env;
--# end
--# property-start AdaNode.top_level_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1078'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Decl
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind unit Unit
   Property_Result : Bare_Basic_Decl;
      Var_Expr_28 : Internal_Unit;
Fld : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 733 '<FieldAccess for decl at nodes.lkt:1079:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1079'
--# expr-start 732 '<Cast to CompilationUnit at nodes.lkt:1079:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1079'
Var_Expr_28 := Unit;
if Var_Expr_28 /= null then
   --# scope-start
--# expr-start 731 '<FieldAccess for root at nodes.lkt:1079:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1079'
      if Var_Expr_28 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_28.Ast_Root;
--# expr-done 731
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Result_Var;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 732
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Cast_Result);
--# end
--# expr-done 733
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Top_Level_Decl;
--# end
--# property-start AdaNode.unpack_formals 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1082'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_101 with Inline_Always;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_101 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_101;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 736 '<FieldAccess for unpack_formals_impl at nodes.lkt:1083:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1083'
--# expr-start 735 '<FieldAccess for root at nodes.lkt:1083:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1083'
--# expr-start 734 '<FieldAccess for unit at nodes.lkt:1083:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1083'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 734
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Ast_Root;
--# expr-done 735
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals_Impl (Node => Fld_1, Formal_Params => Formal_Params);
--# end
--# expr-done 736
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_101;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_101;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Unpack_Formals;
--# end
--# property-start AdaNode.unpack_formals_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1085'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals_Impl
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_102 with Inline_Always;
            procedure Finalizer_Scope_1999 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_102 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_102;
            procedure Finalizer_Scope_1999 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1999;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 738 '<Mapcat at nodes.lkt:1086:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1086'
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Formal_Params;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind spec Item_0
      --# expr-start 737 '<FieldAccess for defining_names at nodes.lkt:1086:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1086'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 737
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1999;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 738
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_102;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_102;
                     Finalizer_Scope_1999;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Unpack_Formals_Impl;
--# end
--# property-start AdaNode.match_formals 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1091'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Match_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind formal_params Formal_Params
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_103 with Inline_Always;
            procedure Finalizer_Scope_2000 with Inline_Always;
      Local_Unpacked_Formals : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_0 : Internal_Entity_Assoc_List;
Var_Expr_3 : Bare_Base_Id;
Var_Expr_5 : Internal_Entity_Defining_Name;
Local_Idx : Integer;
Var_Expr_6 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Single_Actual_Array_Access;
Index_2 : Integer;
Arith_Result : Integer;
If_Result : Integer;
Get_Result : Internal_Entity_Defining_Name;
New_Struct : Internal_Param_Match;
Scope_Result : Internal_Param_Match;
Result_Var : Internal_Param_Match;
Let_Result : Internal_Param_Match;
Fld_2 : Bare_Base_Id;
Fld_3 : Internal_Entity_Name;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
New_Struct_1 : Internal_Param_Match;
Scope_Result_1 : Internal_Param_Match;
Result_Var_1 : Internal_Param_Match;
Scope_Result_2 : Internal_Param_Match;
Result_Var_2 : Internal_Param_Match;
Fld_5 : Bare_Base_Id;
Is_Null : Boolean;
If_Result_1 : Internal_Param_Match;
Map_Result : Internal_Param_Match_Array_Access;
Scope_Result_3 : Internal_Param_Match_Array_Access;
Result_Var_3 : Internal_Param_Match_Array_Access;
Let_Result_1 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_103 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Unpacked_Formals);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_103;
            procedure Finalizer_Scope_2000 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2000;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 755 '<Let at nodes.lkt:1092:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1092'
--# scope-start
--# expr-start 739 '<FieldAccess for unpack_formals at nodes.lkt:1092:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1092'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals
Fld := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Formal_Params);
--# end
--# expr-done 739
Local_Unpacked_Formals := Fld; Inc_Ref (Local_Unpacked_Formals);
--# bind unpacked_formals Local_Unpacked_Formals
Var_Expr_0 := Params;
if Var_Expr_0 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 754 '<Map at nodes.lkt:1095:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1095'
   --# expr-start 740 '<FieldAccess for unpacked_params at nodes.lkt:1095:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1095'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 740
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_1
               --# bind i Index_2
      --# expr-start 753 '<If at nodes.lkt:1096:27>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1096'
--# expr-start 742 '<IsNull at nodes.lkt:1096:30>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1096'
--# expr-start 741 '<FieldAccess for name at nodes.lkt:1096:30>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1096'
Fld_5 := Item_1.Name;
--# expr-done 741
Is_Null := Fld_5 = null; 
--# expr-done 742
if Is_Null then
   --# expr-start 747 '<Let at nodes.lkt:1097:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1097'
--# scope-start
--# expr-start 744 '<If at nodes.lkt:1097:31>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1097'
if Is_Dottable_Subp then
   --# expr-start 743 '<Arithmetic + at nodes.lkt:1097:56>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1097'
Arith_Result := (Index_2 + 1); 
--# expr-done 743
   If_Result := Arith_Result;
else
   If_Result := Index_2;
end if;
--# expr-done 744
Local_Idx := If_Result; 
--# bind idx Local_Idx
--# expr-start 745 '<at at nodes.lkt:1101:21>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1101'
Get_Result := Get (Self, Local_Unpacked_Formals, Local_Idx, True); 
--# expr-done 745
Var_Expr_6 := Get_Result;
if Var_Expr_6 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind sp Var_Expr_6
--# expr-start 746 '<New ParamMatch at nodes.lkt:1102:33>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1102'
New_Struct := (Actual => Item_1, Formal => Var_Expr_6, Has_Matched => True); 
--# expr-done 746
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Param_Match;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 747
   If_Result_1 := Let_Result;
else
--# expr-start 748 '<FieldAccess for name at nodes.lkt:1108:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1108'
Fld_2 := Item_1.Name;
--# expr-done 748
Var_Expr_3 := Fld_2;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind id Var_Expr_3
--# expr-start 751 '<Find at nodes.lkt:1109:33>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1109'
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Unpacked_Formals;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_4
      --# expr-start 750 '<FieldAccess for matches at nodes.lkt:1109:62>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1109'
--# expr-start 749 '<FieldAccess for name at nodes.lkt:1109:62>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1109'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Item_4.Node.Defining_Name_F_Name, Info => Item_4.Info);
--# expr-done 749
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Var_Expr_3; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_3.Node, N => Cast_Result);
--# end
--# expr-done 750
      if Fld_4 then
         Find_Result := Item_4; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 751
Var_Expr_5 := Find_Result;
if Var_Expr_5 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind sp Var_Expr_5
--# expr-start 752 '<New ParamMatch at nodes.lkt:1110:37>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1110'
New_Struct_1 := (Actual => Item_1, Formal => Var_Expr_5, Has_Matched => True); 
--# expr-done 752
Scope_Result_1 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Param_Match;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Param_Match;
end if;
   If_Result_1 := Result_Var_2;
end if;
--# expr-done 753
         declare
            Item_To_Append : constant Internal_Param_Match := If_Result_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 754
Scope_Result_3 := Map_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2000;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 755
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_103;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_103;
                     Finalizer_Scope_2000;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Match_Formals;
--# end
--# property-start AdaNode.choice_match 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1127'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Choice_Match
  (Node : Bare_Ada_Node
      ; Value : Big_Integer_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind value Value
   Property_Result : Boolean;
            procedure Finalizer_Scope_2006 with Inline_Always;
            procedure Finalizer_Scope_2007 with Inline_Always;
            procedure Finalizer_Scope_2009 with Inline_Always;
            procedure Finalizer_Scope_2011 with Inline_Always;
            procedure Finalizer_Scope_2012 with Inline_Always;
      Match_0 : Internal_Entity_Bin_Op;
Match_1 : Internal_Entity_Name;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Discrete_Range;
Local_Edr : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Match_2 : Internal_Entity_Subtype_Indication;
Var_Expr_2 : Internal_Discrete_Range;
Local_Edr_1 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Match_3 : Internal_Entity_Expr;
Match_4 : Internal_Entity_Others_Designator;
Match_5 : Internal_Entity;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Comp_Result : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
Comp_Result_1 : Boolean;
If_Result : Boolean;
Fld_4 : Big_Integer_Type;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Discrete_Range;
Fld_8 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_9 : Big_Integer_Type;
Comp_Result_2 : Boolean;
Fld_10 : Big_Integer_Type;
Comp_Result_3 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Bound_Origin : Bare_Ada_Node;
Bound_Imprecise_Fallback : Boolean;
Fld_11 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_1 : Boolean;
Fld_12 : Big_Integer_Type;
Is_Equal_1 : Boolean;
Result_Var_1 : Boolean;
Fld_13 : Internal_Discrete_Range;
Fld_14 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_15 : Big_Integer_Type;
Comp_Result_4 : Boolean;
Fld_16 : Big_Integer_Type;
Comp_Result_5 : Boolean;
If_Result_4 : Boolean;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Bound_Origin_1 : Bare_Ada_Node;
Bound_Imprecise_Fallback_1 : Boolean;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
If_Result_5 : Boolean;
Fld_19 : Big_Integer_Type;
Is_Equal_2 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Bin_Op;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_2 : Internal_Entity_Subtype_Indication;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_5 : Internal_Entity;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Match_Result : Boolean;
            procedure Finalizer_Scope_2006 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2006;
            procedure Finalizer_Scope_2007 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2007;
            procedure Finalizer_Scope_2009 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Local_Edr);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2009;
            procedure Finalizer_Scope_2011 is
            begin
                     Dec_Ref (Fld_14);
                     Dec_Ref (Local_Edr_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_2011;
            procedure Finalizer_Scope_2012 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_2012;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 787 '<Match at nodes.lkt:1127:45>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1127'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bo Match_0
--# expr-start 764 '<If at nodes.lkt:1132:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
--# expr-start 757 '<IsA Op.DoubleDot at nodes.lkt:1132:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
--# expr-start 756 '<FieldAccess for op at nodes.lkt:1132:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Op (Node => Match_0.Node.Bin_Op_F_Op, Info => Match_0.Info);
--# expr-done 756
Is_A := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 757
if Is_A then
--# expr-start 759 '<FieldAccess for eval_as_int at nodes.lkt:1132:48>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
--# expr-start 758 '<FieldAccess for left at nodes.lkt:1132:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Left, Info => Match_0.Info);
--# expr-done 758
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 759
Comp_Result := Value >= Fld_1; 
if Comp_Result then
--# expr-start 761 '<FieldAccess for eval_as_int at nodes.lkt:1132:83>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
--# expr-start 760 '<FieldAccess for right at nodes.lkt:1132:83>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Right, Info => Match_0.Info);
--# expr-done 760
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 761
Comp_Result_1 := Value <= Fld_3; 
   If_Result := Comp_Result_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
else
   --# expr-start 763 '<Eq at nodes.lkt:1132:111>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
--# expr-start 762 '<FieldAccess for eval_as_int at nodes.lkt:1132:120>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1132'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_4 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 762
Is_Equal := Equivalent (Value, Fld_4); 
--# expr-done 763
   If_Result_1 := Is_Equal;
end if;
--# expr-done 764
Let_Result_2 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
      Finalizer_Scope_2006;
            Match_Result := Scope_Result_3; 
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 765 '<FieldAccess for name_designated_type at nodes.lkt:1136:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1136'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 765
Var_Expr_0 := Fld_6;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr_0
--# expr-start 766 '<FieldAccess for discrete_range at nodes.lkt:1137:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1137'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 766
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_1
--# expr-start 770 '<Let at nodes.lkt:1139:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1139'
--# scope-start
--# expr-start 767 '<FieldAccess for eval_discrete_range at nodes.lkt:1139:31>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1139'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_8 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_1);
--# end
--# expr-done 767
Local_Edr := Fld_8; Inc_Ref (Local_Edr);
--# bind edr Local_Edr
--# expr-start 768 '<FieldAccess for low_bound at nodes.lkt:1141:30>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1141'
Fld_9 := Local_Edr.Low_Bound;
Inc_Ref (Fld_9);
--# expr-done 768
Comp_Result_2 := Value >= Fld_9; 
if Comp_Result_2 then
--# expr-start 769 '<FieldAccess for high_bound at nodes.lkt:1141:57>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1141'
Fld_10 := Local_Edr.High_Bound;
Inc_Ref (Fld_10);
--# expr-done 769
Comp_Result_3 := Value <= Fld_10; 
   If_Result_2 := Comp_Result_3;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 770
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_2009;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
if Result_Var then
   --# expr-start 773 '<bind at nodes.lkt:1144:17>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1144'
Bound_Origin := Self; 
--# expr-start 772 '<bind at nodes.lkt:1145:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1145'
Bound_Imprecise_Fallback := False; 
--# expr-start 771 '<FieldAccess for satisfies_type_predicates at nodes.lkt:1147:17>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1147'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Var_Expr_0.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback, Origin => Bound_Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 771
Dyn_Var_Bind_Result := Fld_11; 
--# expr-done 772
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 773
   If_Result_3 := Dyn_Var_Bind_Result_1;
else
   If_Result_3 := False;
end if;
Scope_Result_1 := If_Result_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 775 '<Eq at nodes.lkt:1148:28>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1148'
--# expr-start 774 '<FieldAccess for eval_as_int at nodes.lkt:1148:37>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1148'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_12 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 774
Is_Equal_1 := Equivalent (Value, Fld_12); 
--# expr-done 775
   Result_Var_1 := Is_Equal_1;
end if;
Let_Result_3 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
      Finalizer_Scope_2007;
            Match_Result := Scope_Result_4; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind st Match_2
--# expr-start 776 '<FieldAccess for discrete_range at nodes.lkt:1152:39>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1152'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_13 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 776
Var_Expr_2 := Fld_13;
if Var_Expr_2 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_2
--# expr-start 780 '<Let at nodes.lkt:1154:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1154'
--# scope-start
--# expr-start 777 '<FieldAccess for eval_discrete_range at nodes.lkt:1154:27>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1154'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_14 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_2);
--# end
--# expr-done 777
Local_Edr_1 := Fld_14; Inc_Ref (Local_Edr_1);
--# bind edr Local_Edr_1
--# expr-start 778 '<FieldAccess for low_bound at nodes.lkt:1156:26>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1156'
Fld_15 := Local_Edr_1.Low_Bound;
Inc_Ref (Fld_15);
--# expr-done 778
Comp_Result_4 := Value >= Fld_15; 
if Comp_Result_4 then
--# expr-start 779 '<FieldAccess for high_bound at nodes.lkt:1156:53>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1156'
Fld_16 := Local_Edr_1.High_Bound;
Inc_Ref (Fld_16);
--# expr-done 779
Comp_Result_5 := Value <= Fld_16; 
   If_Result_4 := Comp_Result_5;
else
   If_Result_4 := False;
end if;
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 780
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_2011;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
if Result_Var_2 then
   --# expr-start 784 '<bind at nodes.lkt:1159:13>' Dyn_Var_Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1159'
Bound_Origin_1 := Self; 
--# expr-start 783 '<bind at nodes.lkt:1160:13>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1160'
Bound_Imprecise_Fallback_1 := False; 
--# expr-start 782 '<FieldAccess for satisfies_type_predicates at nodes.lkt:1162:13>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1162'
--# expr-start 781 '<FieldAccess for designated_type at nodes.lkt:1162:13>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1162'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_2.Node, Origin => Bound_Origin_1, E_Info => Match_2.Info);
--# end
--# expr-done 781
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Fld_17.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback_1, Origin => Bound_Origin_1, E_Info => Fld_17.Info);
--# end
--# expr-done 782
Dyn_Var_Bind_Result_2 := Fld_18; 
--# expr-done 783
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 784
   If_Result_5 := Dyn_Var_Bind_Result_3;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind e Match_3
--# expr-start 786 '<Eq at nodes.lkt:1166:25>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1166'
--# expr-start 785 '<FieldAccess for eval_as_int at nodes.lkt:1166:34>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1166'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_19 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 785
Is_Equal_2 := Equivalent (Value, Fld_19); 
--# expr-done 786
Let_Result_5 := Is_Equal_2; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
      Finalizer_Scope_2012;
            Match_Result := Scope_Result_6; 
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_6 := True; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_7 := False; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 787
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2006;
                     Finalizer_Scope_2007;
                     Finalizer_Scope_2009;
                     Finalizer_Scope_2011;
                     Finalizer_Scope_2012;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Choice_Match;
--# end
--# property-start AdaNode.gnat_xref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1177'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Gnat_Xref
  (Node : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_105 with Inline_Always;
      Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Defining_Name;
Local_Dbd : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Boolean;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Local_Pd : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Internal_Entity_Subp_Body;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_7 : Internal_Entity_Defining_Name;
Var_Expr_29 : Internal_Entity_Name;
Var_Expr_9 : Internal_Entity_Defining_Name;
Var_Expr_10 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Generic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Defining_Name;
Cast_Expr_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Name;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Internal_Entity_Defining_Name;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Discriminant_Spec;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Expr_3 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_12 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Defining_Name;
Fld_14 : Boolean;
If_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
Cast_Expr_4 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Abstract_Subp_Decl;
Fld_15 : Internal_Entity_Base_Subp_Spec;
Fld_16 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Cast_Expr_5 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Basic_Subp_Decl;
Fld_18 : Internal_Entity_Base_Subp_Spec;
Fld_19 : Internal_Entity_Base_Type_Decl;
Fld_20 : Boolean;
Fld_21 : Internal_Entity_Base_Type_Decl;
Fld_22 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Scope_Result_3 : Internal_Entity_Defining_Name;
Fld_23 : Internal_Entity_Defining_Name;
Result_Var_3 : Internal_Entity_Defining_Name;
Scope_Result_4 : Internal_Entity_Defining_Name;
Result_Var_4 : Internal_Entity_Defining_Name;
Scope_Result_5 : Internal_Entity_Defining_Name;
Result_Var_5 : Internal_Entity_Defining_Name;
Cast_Expr_6 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Base_Subp_Body;
Fld_24 : Internal_Entity_Subp_Spec;
Fld_25 : Internal_Entity_Defining_Name;
Cast_Expr_7 : Internal_Entity;
Cast_Result_9 : Internal_Entity_Name;
Fld_26 : Internal_Entity_Defining_Name;
Fld_27 : Internal_Entity_Basic_Decl;
Cast_Expr_8 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Param_Spec;
Fld_28 : Internal_Entity_Defining_Name;
Cast_Expr_9 : Internal_Entity_Basic_Decl;
Cast_Result_11 : Internal_Entity_Object_Decl;
Cast_Expr_10 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Name;
Fld_29 : Symbol_Type;
Fld_30 : Internal_Entity_Basic_Decl;
Fld_31 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Scope_Result_7 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Cast_Expr_11 : Internal_Entity_Basic_Decl;
Cast_Result_13 : Internal_Entity_Body_Node;
Fld_32 : Internal_Entity_Basic_Decl;
Scope_Result_8 : Internal_Entity_Basic_Decl;
Result_Var_8 : Internal_Entity_Basic_Decl;
Fld_33 : Internal_Entity_Defining_Name;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
If_Result_2 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
If_Result_3 : Internal_Entity_Defining_Name;
Let_Result_1 : Internal_Entity_Defining_Name;
Scope_Result_9 : Internal_Entity_Defining_Name;
Result_Var_9 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Is_A_4 : Boolean;
If_Result_4 : Internal_Entity_Defining_Name;
Is_A_5 : Boolean;
If_Result_5 : Internal_Entity_Defining_Name;
Is_A_6 : Boolean;
If_Result_6 : Internal_Entity_Defining_Name;
Is_A_7 : Boolean;
If_Result_7 : Internal_Entity_Defining_Name;
Is_A_8 : Boolean;
Cast_Expr_12 : Internal_Entity_Basic_Decl;
Cast_Result_14 : Internal_Entity_Object_Decl;
Fld_34 : Internal_Entity_Constant_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_8 : Boolean;
Fld_35 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Internal_Entity_Defining_Name;
Fld_36 : Internal_Entity;
Is_A_9 : Boolean;
Scope_Result_11 : Boolean;
Result_Var_11 : Boolean;
If_Result_11 : Internal_Entity_Defining_Name;
Is_A_10 : Boolean;
Fld_37 : Internal_Entity;
Is_A_11 : Boolean;
Fld_38 : Internal_Entity;
Cast_Result_15 : Internal_Entity_Subp_Body;
Fld_39 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Scope_Result_12 : Boolean;
Result_Var_12 : Boolean;
If_Result_12 : Boolean;
If_Result_13 : Boolean;
If_Result_14 : Internal_Entity_Defining_Name;
Dyn_Var_Bind_Result : Internal_Entity_Defining_Name;
Let_Result_2 : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_105 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_105;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 874 '<Let at nodes.lkt:1178:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1178'
--# scope-start
--# expr-start 789 '<FieldAccess for enclosing_defining_name at nodes.lkt:1178:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1178'
--# expr-start 788 '<Cast to Name at nodes.lkt:1178:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1178'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 788
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 789
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 790 '<FieldAccess for basic_decl at nodes.lkt:1178:69>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1178'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 790
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Local_Bd := Result_Var; 
--# bind bd Local_Bd
--# expr-start 873 '<bind at nodes.lkt:1181:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1181'
Bound_Origin := Self; 
--# expr-start 872 '<If at nodes.lkt:1183:13>' If_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 791 '<IsA ParamSpec at nodes.lkt:1183:16>' Is_A_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
Is_A_10 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 791
if Is_A_10 then
--# expr-start 793 '<IsA BasicSubpDecl, ExprFunction, BaseTypeDecl, SubpBodyStub, NullSubpDecl at nodes.lkt:1184:17>' Is_A_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1184'
--# expr-start 792 '<FieldAccess for semantic_parent at nodes.lkt:1184:17>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1184'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_37 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 792
Is_A_11 := Fld_37.Node /= null 
and then Fld_37.Node.Kind in Ada_Basic_Subp_Decl | Ada_Expr_Function_Range | Ada_Base_Type_Decl | Ada_Subp_Body_Stub_Range | Ada_Null_Subp_Decl_Range; 
--# expr-done 793
if Is_A_11 then
   If_Result_12 := True;
else
--# expr-start 795 '<Cast to SubpBody at nodes.lkt:1184:118>' Cast_Result_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1184'
--# expr-start 794 '<FieldAccess for semantic_parent at nodes.lkt:1184:118>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1184'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_38 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 794
   if Fld_38.Node = null
      or else Fld_38.Node.Kind in Ada_Subp_Body_Range
   then
      Cast_Result_15 := Create_Internal_Entity_Subp_Body
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
   else
         Cast_Result_15 := No_Entity_Subp_Body;
   end if;
--# expr-done 795
Var_Expr_8 := Cast_Result_15;
if Var_Expr_8 /= No_Entity_Subp_Body then
   --# scope-start
   --# bind body Var_Expr_8
--# expr-start 797 '<IsNull at nodes.lkt:1185:31>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1185'
--# expr-start 796 '<FieldAccess for previous_part_for_decl at nodes.lkt:1185:31>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1185'
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_39 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Var_Expr_8.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 796
Is_Null_1 := Fld_39.Node = null; 
--# expr-done 797
Scope_Result_12 := Is_Null_1;
   --# end
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := False;
end if;
   If_Result_12 := Result_Var_12;
end if;
   If_Result_13 := If_Result_12;
else
   If_Result_13 := False;
end if;
if If_Result_13 then
   --# expr-start 800 '<FieldAccess for defining_name at nodes.lkt:1187:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1187'
--# expr-start 799 '<Cast to BasicDecl at nodes.lkt:1187:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1187'
--# expr-start 798 '<FieldAccess for semantic_parent at nodes.lkt:1187:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1187'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 798
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 799
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 800
   If_Result_14 := Fld_3;
else
   --# expr-start 871 '<If at nodes.lkt:1183:13>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
Var_Expr_6 := Local_Bd;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind v1 Var_Expr_6
--# expr-start 802 '<IsA GenericFormal at nodes.lkt:1188:32>' Is_A_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
--# expr-start 801 '<parent at nodes.lkt:1188:32>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Parent (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# expr-done 801
Is_A_9 := Fld_36.Node /= null 
and then Fld_36.Node.Kind in Ada_Generic_Formal; 
--# expr-done 802
Scope_Result_11 := Is_A_9;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := False;
end if;
if Result_Var_11 then
   --# expr-start 808 '<FieldAccess for defining_name at nodes.lkt:1188:65>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
--# expr-start 807 '<FieldAccess for decl at nodes.lkt:1188:65>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
--# expr-start 806 '<Cast to GenericDecl at nodes.lkt:1188:65>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
--# expr-start 805 '<Find at nodes.lkt:1188:65>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
   --# expr-start 803 '<FieldAccess for parents at nodes.lkt:1188:65>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parents (Node => Local_Bd.Node, With_Self => True, E_Info => Local_Bd.Info);
--# expr-done 803
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_4;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_7
      --# expr-start 804 '<IsA GenericDecl at nodes.lkt:1188:90>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1188'
Is_A := Item_7.Node /= null 
and then Item_7.Node.Kind in Ada_Generic_Decl; 
--# expr-done 804
      if Is_A then
         Find_Result := Item_7; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 805
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Decl;
   end if;
--# expr-done 806
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 807
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 808
   If_Result_11 := Fld_6;
else
   --# expr-start 870 '<If at nodes.lkt:1183:13>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 809 '<IsA ObjectDecl at nodes.lkt:1191:17>' Is_A_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1191'
Is_A_8 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 809
if Is_A_8 then
   --# expr-start 813 '<Not at nodes.lkt:1191:38>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1191'
--# expr-start 812 '<IsNull at nodes.lkt:1191:42>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1191'
--# expr-start 811 '<FieldAccess for has_constant at nodes.lkt:1191:42>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1191'
--# expr-start 810 '<Cast to ObjectDecl at nodes.lkt:1191:42>' Cast_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1191'
Cast_Expr_12 := Local_Bd; 
   if Cast_Expr_12.Node = null
      or else Cast_Expr_12.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_14 := Create_Internal_Entity_Object_Decl
        (Node => Cast_Expr_12.Node,
         Info => Cast_Expr_12.Info);
   else
         Cast_Result_14 := No_Entity_Object_Decl;
   end if;
--# expr-done 810
      if Cast_Result_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Constant_Node (Node => Cast_Result_14.Node.Object_Decl_F_Has_Constant, Info => Cast_Result_14.Info);
--# expr-done 811
Is_Null := Fld_34.Node = null; 
--# expr-done 812
Not_Val := not (Is_Null); 
--# expr-done 813
   If_Result_8 := Not_Val;
else
   If_Result_8 := False;
end if;
if If_Result_8 then
   --# expr-start 814 '<FieldAccess for is_in_private_part at nodes.lkt:1192:19>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1192'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_35 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 814
   If_Result_9 := Fld_35;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
   --# expr-start 817 '<FieldAccess for previous_part at nodes.lkt:1192:48>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1192'
--# expr-start 816 '<FieldAccess for enclosing_defining_name at nodes.lkt:1192:48>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1192'
--# expr-start 815 '<Cast to Name at nodes.lkt:1192:48>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1192'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 815
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_7 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 816
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.previous_part
Fld_8 := Libadalang.Implementation.Defining_Name_P_Previous_Part (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 817
   If_Result_10 := Fld_8;
else
   --# expr-start 869 '<If at nodes.lkt:1183:13>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 818 '<IsA DiscriminantSpec at nodes.lkt:1201:18>' Is_A_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1201'
Is_A_7 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Discriminant_Spec_Range; 
--# expr-done 818
if Is_A_7 then
   --# expr-start 828 '<Let at nodes.lkt:1202:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1202'
--# scope-start
--# expr-start 820 '<FieldAccess for parent_decl at nodes.lkt:1202:26>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1202'
--# expr-start 819 '<Cast to DiscriminantSpec at nodes.lkt:1202:26>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1202'
Cast_Expr_2 := Local_Bd; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Discriminant_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_4 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 819
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 820
Local_Pd := Fld_9; 
--# bind pd Local_Pd
--# expr-start 827 '<If at nodes.lkt:1204:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
--# expr-start 821 '<FieldAccess for is_in_private_part at nodes.lkt:1204:20>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
      if Local_Pd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Local_Pd.Node, E_Info => Local_Pd.Info);
--# end
--# expr-done 821
if Fld_14 then
--# expr-start 824 '<FieldAccess for previous_part at nodes.lkt:1204:49>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
--# expr-start 823 '<FieldAccess for enclosing_defining_name at nodes.lkt:1204:49>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
--# expr-start 822 '<Cast to Name at nodes.lkt:1204:49>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
Cast_Expr_3 := Ent; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Name
   then
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_5 := No_Entity_Name;
   end if;
--# expr-done 822
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_10 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 823
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.previous_part
Fld_11 := Libadalang.Implementation.Defining_Name_P_Previous_Part (Node => Fld_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_10.Info);
--# end
--# expr-done 824
Var_Expr := Fld_11;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 825 '<FieldAccess for defining_name at nodes.lkt:1204:109>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
      if Local_Pd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Pd.Node, E_Info => Local_Pd.Info);
--# end
--# expr-done 825
   Result_Var_1 := Fld_12;
end if;
   If_Result := Result_Var_1;
else
   --# expr-start 826 '<FieldAccess for defining_name at nodes.lkt:1204:133>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1204'
      if Local_Pd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Pd.Node, E_Info => Local_Pd.Info);
--# end
--# expr-done 826
   If_Result := Fld_13;
end if;
--# expr-done 827
Let_Result := If_Result; 
--# end
--# expr-done 828
   If_Result_7 := Let_Result;
else
   --# expr-start 868 '<If at nodes.lkt:1183:13>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 829 '<IsA AbstractSubpDecl at nodes.lkt:1206:18>' Is_A_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1206'
Is_A_6 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Abstract_Subp_Decl_Range; 
--# expr-done 829
if Is_A_6 then
   --# expr-start 833 '<FieldAccess for defining_name at nodes.lkt:1206:46>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1206'
--# expr-start 832 '<FieldAccess for primitive_subp_first_type at nodes.lkt:1206:46>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1206'
--# expr-start 831 '<FieldAccess for subp_decl_spec at nodes.lkt:1206:46>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1206'
--# expr-start 830 '<Cast to AbstractSubpDecl at nodes.lkt:1206:46>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1206'
Cast_Expr_4 := Local_Bd; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Abstract_Subp_Decl_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Abstract_Subp_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_6 := No_Entity_Abstract_Subp_Decl;
   end if;
--# expr-done 830
      if Cast_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_6.Node, E_Info => Cast_Result_6.Info);
--# end
--# expr-done 831
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_first_type
Fld_16 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_First_Type (Node => Fld_15.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_15.Info);
--# end
--# expr-done 832
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 833
   If_Result_6 := Fld_17;
else
   --# expr-start 867 '<If at nodes.lkt:1183:13>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 834 '<IsA BasicSubpDecl at nodes.lkt:1207:18>' Is_A_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1207'
Is_A_5 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 834
if Is_A_5 then
--# expr-start 837 '<FieldAccess for primitive_subp_first_type at nodes.lkt:1207:43>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1207'
--# expr-start 836 '<FieldAccess for subp_decl_spec at nodes.lkt:1207:43>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1207'
--# expr-start 835 '<Cast to BasicSubpDecl at nodes.lkt:1207:43>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1207'
Cast_Expr_5 := Local_Bd; 
   if Cast_Expr_5.Node = null
      or else Cast_Expr_5.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 835
      if Cast_Result_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_7.Node, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 836
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_first_type
Fld_19 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_First_Type (Node => Fld_18.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_18.Info);
--# end
--# expr-done 837
Var_Expr_3 := Fld_19;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind prim_typ Var_Expr_3
--# expr-start 838 '<FieldAccess for is_tagged_type at nodes.lkt:1208:31>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1208'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_20 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr_3.Node, Origin => Bound_Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 838
Var_Expr_4 := Fld_20;
if Var_Expr_4 /= False then
   --# scope-start
   --# bind _ Var_Expr_4
--# expr-start 839 '<FieldAccess for private_completion at nodes.lkt:1209:28>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1209'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_21 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 839
Var_Expr_5 := Fld_21;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pc Var_Expr_5
--# expr-start 840 '<FieldAccess for defining_name at nodes.lkt:1209:69>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1209'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 840
Scope_Result_2 := Fld_22;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Var_Expr_7 := Result_Var_2;
if Var_Expr_7 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_7
Scope_Result_3 := Var_Expr_7;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 841 '<FieldAccess for defining_name at nodes.lkt:1209:93>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1209'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_23 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 841
   Result_Var_3 := Fld_23;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Defining_Name;
end if;
Scope_Result_5 := Result_Var_4;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Defining_Name;
end if;
   If_Result_5 := Result_Var_5;
else
   --# expr-start 866 '<If at nodes.lkt:1183:13>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1183'
--# expr-start 842 '<IsA BaseSubpBody at nodes.lkt:1212:18>' Is_A_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1212'
Is_A_4 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 842
if Is_A_4 then
   --# expr-start 845 '<FieldAccess for subp_name at nodes.lkt:1212:42>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1212'
--# expr-start 844 '<FieldAccess for subp_spec at nodes.lkt:1212:42>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1212'
--# expr-start 843 '<Cast to BaseSubpBody at nodes.lkt:1212:42>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1212'
Cast_Expr_6 := Local_Bd; 
   if Cast_Expr_6.Node = null
      or else Cast_Expr_6.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_8 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   else
         Cast_Result_8 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 843
      if Cast_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Subp_Spec (Node => Cast_Result_8.Node.Base_Subp_Body_F_Subp_Spec, Info => Cast_Result_8.Info);
--# expr-done 844
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Defining_Name (Node => Fld_24.Node.Subp_Spec_F_Subp_Name, Info => Fld_24.Info);
--# expr-done 845
   If_Result_4 := Fld_25;
else
--# expr-start 846 '<Cast to Name at nodes.lkt:1213:18>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1213'
Cast_Expr_7 := Ent; 
   if Cast_Expr_7.Node = null
      or else Cast_Expr_7.Node.Kind in Ada_Name
   then
      Cast_Result_9 := Create_Internal_Entity_Name
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
   else
         Cast_Result_9 := No_Entity_Name;
   end if;
--# expr-done 846
Var_Expr_29 := Cast_Result_9;
if Var_Expr_29 /= No_Entity_Name then
   --# scope-start
--# expr-start 847 '<FieldAccess for gnat_xref_decl at nodes.lkt:1213:18>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1213'
      if Var_Expr_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.gnat_xref_decl
Fld_26 := Libadalang.Implementation.Name_P_Gnat_Xref_Decl (Node => Var_Expr_29.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_29.Info);
--# end
--# expr-done 847
Var_Expr_1 := Fld_26;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind ret Var_Expr_1
--# expr-start 865 '<Let at nodes.lkt:1215:21>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1215'
--# scope-start
--# expr-start 848 '<FieldAccess for basic_decl at nodes.lkt:1215:31>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1215'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_27 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 848
Local_Dbd := Fld_27; 
--# bind dbd Local_Dbd
--# expr-start 864 '<If at nodes.lkt:1217:21>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
--# expr-start 849 '<IsA ParamSpec at nodes.lkt:1217:24>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
Is_A_3 := Local_Dbd.Node /= null 
and then Local_Dbd.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 849
if Is_A_3 then
   --# expr-start 851 '<FieldAccess for decl_param at nodes.lkt:1217:46>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
--# expr-start 850 '<Cast to ParamSpec at nodes.lkt:1217:46>' Cast_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
Cast_Expr_8 := Local_Dbd; 
   if Cast_Expr_8.Node = null
      or else Cast_Expr_8.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_10 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_8.Node,
         Info => Cast_Expr_8.Info);
   else
         Cast_Result_10 := No_Entity_Param_Spec;
   end if;
--# expr-done 850
      if Cast_Result_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_28 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result_10.Node, Param => Var_Expr_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_10.Info);
--# end
--# expr-done 851
   If_Result_3 := Fld_28;
else
   --# expr-start 863 '<If at nodes.lkt:1217:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
--# expr-start 852 '<IsA ObjectDecl at nodes.lkt:1218:26>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1218'
Is_A_2 := Local_Dbd.Node /= null 
and then Local_Dbd.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 852
if Is_A_2 then
--# expr-start 856 '<FieldAccess for previous_part_for_name at nodes.lkt:1222:25>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1222'
--# expr-start 853 '<Cast to ObjectDecl at nodes.lkt:1222:25>' Cast_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1222'
Cast_Expr_9 := Local_Dbd; 
   if Cast_Expr_9.Node = null
      or else Cast_Expr_9.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_11 := Create_Internal_Entity_Object_Decl
        (Node => Cast_Expr_9.Node,
         Info => Cast_Expr_9.Info);
   else
         Cast_Result_11 := No_Entity_Object_Decl;
   end if;
--# expr-done 853
      if Cast_Result_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 855 '<FieldAccess for name_symbol at nodes.lkt:1222:67>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1222'
--# expr-start 854 '<Cast to Name at nodes.lkt:1222:67>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1222'
Cast_Expr_10 := Ent; 
   if Cast_Expr_10.Node = null
      or else Cast_Expr_10.Node.Kind in Ada_Name
   then
      Cast_Result_12 := Create_Internal_Entity_Name
        (Node => Cast_Expr_10.Node,
         Info => Cast_Expr_10.Info);
   else
         Cast_Result_12 := No_Entity_Name;
   end if;
--# expr-done 854
      if Cast_Result_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_12.Node);
--# end
--# expr-done 855
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Cast_Result_11.Node, Sym => Fld_29, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_11.Info);
--# end
--# expr-done 856
Var_Expr_2 := Fld_30;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ppn Var_Expr_2
--# expr-start 857 '<FieldAccess for defining_name at nodes.lkt:1222:108>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1222'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_31 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 857
Scope_Result_6 := Fld_31;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
Var_Expr_9 := Result_Var_6;
if Var_Expr_9 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_9
Scope_Result_7 := Var_Expr_9;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := Var_Expr_1;
end if;
   If_Result_2 := Result_Var_7;
else
   --# expr-start 862 '<If at nodes.lkt:1217:21>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1217'
--# expr-start 858 '<IsA Body at nodes.lkt:1224:26>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1224'
Is_A_1 := Local_Dbd.Node /= null 
and then Local_Dbd.Node.Kind in Ada_Body_Node; 
--# expr-done 858
if Is_A_1 then
   --# expr-start 861 '<FieldAccess for defining_name at nodes.lkt:1224:43>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1224'
--# expr-start 860 '<FieldAccess for decl_part at nodes.lkt:1224:44>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1224'
--# expr-start 859 '<Cast to Body at nodes.lkt:1224:44>' Cast_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1224'
Cast_Expr_11 := Local_Dbd; 
   if Cast_Expr_11.Node = null
      or else Cast_Expr_11.Node.Kind in Ada_Body_Node
   then
      Cast_Result_13 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_11.Node,
         Info => Cast_Expr_11.Info);
   else
         Cast_Result_13 := No_Entity_Body_Node;
   end if;
--# expr-done 859
      if Cast_Result_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_32 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_13.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_13.Info);
--# end
--# expr-done 860
Var_Expr_10 := Fld_32;
if Var_Expr_10 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_10
Scope_Result_8 := Var_Expr_10;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := Local_Dbd;
end if;
      if Result_Var_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_33 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Result_Var_8.Node, E_Info => Result_Var_8.Info);
--# end
--# expr-done 861
   If_Result_1 := Fld_33;
else
   If_Result_1 := Var_Expr_1;
end if;
--# expr-done 862
   If_Result_2 := If_Result_1;
end if;
--# expr-done 863
   If_Result_3 := If_Result_2;
end if;
--# expr-done 864
Let_Result_1 := If_Result_3; 
--# end
--# expr-done 865
Scope_Result_9 := Let_Result_1;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Defining_Name;
end if;
Scope_Result_10 := Result_Var_9;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
   If_Result_4 := Result_Var_10;
end if;
--# expr-done 866
   If_Result_5 := If_Result_4;
end if;
--# expr-done 867
   If_Result_6 := If_Result_5;
end if;
--# expr-done 868
   If_Result_7 := If_Result_6;
end if;
--# expr-done 869
   If_Result_10 := If_Result_7;
end if;
--# expr-done 870
   If_Result_11 := If_Result_10;
end if;
--# expr-done 871
   If_Result_14 := If_Result_11;
end if;
--# expr-done 872
Dyn_Var_Bind_Result := If_Result_14; 
--# expr-done 873
Let_Result_2 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 874
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_105;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_105;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Gnat_Xref;
--# end
--# property-start AdaNode.env_get_real_from_node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1235'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Real_From_Node
  (Node : Bare_Ada_Node
      ; From_Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind from_node From_Node
   Property_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_106 with Inline_Always;
      Local_C : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Defining_Name;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Subp_Spec;
As_Entity_1 : Internal_Entity_Base_Subp_Spec;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Bare_Defining_Name;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_2 : Bare_Renaming_Clause;
Fld_4 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_2 : Bare_Ada_Node;
Cast_Result_3 : Bare_Ada_Node;
Is_A_2 : Boolean;
If_Result_3 : Bare_Ada_Node;
Cast_Result_4 : Bare_Ada_Node;
Is_A_3 : Boolean;
If_Result_4 : Bare_Ada_Node;
Is_Null_2 : Boolean;
If_Result_5 : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
Is_Null_3 : Boolean;
If_Result_6 : Bare_Ada_Node;
            procedure Finalizer_Scope_106 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_106;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 898 '<If at nodes.lkt:1236:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1236'
--# expr-start 875 '<IsNull at nodes.lkt:1236:12>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1236'
Is_Null_3 := From_Node = null; 
--# expr-done 875
if Is_Null_3 then
   If_Result_6 := From_Node;
else
   --# expr-start 897 '<Let at nodes.lkt:1239:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1239'
--# scope-start
--# expr-start 878 '<Find at nodes.lkt:1239:21>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1239'
   --# expr-start 876 '<FieldAccess for parents at nodes.lkt:1239:21>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1239'
      if From_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (From_Node, True); 
--# expr-done 876
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 877 '<IsA GenericInstantiation, BaseSubpSpec, ComponentDecl, RenamingClause at nodes.lkt:1240:17>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1240'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Generic_Instantiation | Ada_Base_Subp_Spec | Ada_Component_Decl_Range | Ada_Renaming_Clause_Range; 
--# expr-done 877
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 878
Local_C := Find_Result; 
--# bind c Local_C
--# expr-start 896 '<If at nodes.lkt:1244:13>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1244'
--# expr-start 879 '<IsNull at nodes.lkt:1244:16>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1244'
Is_Null_2 := Local_C = null; 
--# expr-done 879
if Is_Null_2 then
   If_Result_5 := From_Node;
else
   --# expr-start 895 '<If at nodes.lkt:1244:13>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1244'
--# expr-start 880 '<IsA GenericInstantiation at nodes.lkt:1246:18>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1246'
Is_A_3 := Local_C /= null 
and then Local_C.Kind in Ada_Generic_Instantiation; 
--# expr-done 880
if Is_A_3 then
--# expr-start 884 '<FieldAccess for node at nodes.lkt:1251:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1251'
--# expr-start 883 '<FieldAccess for defining_name at nodes.lkt:1251:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1251'
--# expr-start 882 '<as_bare_entity at nodes.lkt:1251:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1251'
--# expr-start 881 '<Cast to GenericInstantiation at nodes.lkt:1251:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1251'
Cast_Expr := Local_C; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 881
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 882
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 883
Fld_1 := Fld.Node;
--# expr-done 884
      Cast_Result_4 := Fld_1;
   If_Result_4 := Cast_Result_4;
else
   --# expr-start 894 '<If at nodes.lkt:1244:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1244'
--# expr-start 885 '<IsA BaseSubpSpec at nodes.lkt:1252:18>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1252'
Is_A_2 := Local_C /= null 
and then Local_C.Kind in Ada_Base_Subp_Spec; 
--# expr-done 885
if Is_A_2 then
--# expr-start 889 '<FieldAccess for node at nodes.lkt:1253:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1253'
--# expr-start 888 '<FieldAccess for name at nodes.lkt:1253:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1253'
--# expr-start 887 '<as_bare_entity at nodes.lkt:1253:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1253'
--# expr-start 886 '<Cast to BaseSubpSpec at nodes.lkt:1253:17>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1253'
Cast_Expr_1 := Local_C; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 886
Is_Null_1 := Cast_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 887
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 888
Fld_3 := Fld_2.Node;
--# expr-done 889
      Cast_Result_3 := Fld_3;
   If_Result_3 := Cast_Result_3;
else
   --# expr-start 893 '<If at nodes.lkt:1244:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1244'
--# expr-start 890 '<IsA RenamingClause at nodes.lkt:1254:18>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1254'
Is_A_1 := Local_C /= null 
and then Local_C.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 890
if Is_A_1 then
   --# expr-start 892 '<parent at nodes.lkt:1258:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1258'
--# expr-start 891 '<Cast to RenamingClause at nodes.lkt:1258:17>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1258'
Cast_Expr_2 := Local_C; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Renaming_Clause_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 891
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Cast_Result_2.Parent; 
--# expr-done 892
   If_Result_2 := Fld_4;
else
   If_Result_2 := Local_C;
end if;
--# expr-done 893
   If_Result_3 := If_Result_2;
end if;
--# expr-done 894
   If_Result_4 := If_Result_3;
end if;
--# expr-done 895
   If_Result_5 := If_Result_4;
end if;
--# expr-done 896
Let_Result := If_Result_5; 
--# end
--# expr-done 897
   If_Result_6 := Let_Result;
end if;
--# expr-done 898
         Property_Result := If_Result_6;
   --# end
      Finalizer_Scope_106;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_106;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get_Real_From_Node;
--# end
--# property-start AdaNode.entity_no_md 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1265'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Entity_No_Md
  (Node : Bare_Ada_Node
      ; N : Bare_Ada_Node
      ; Rebindings : Env_Rebindings
      ; From_Rebound : Boolean
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind n N
   --# bind rebindings Rebindings
   --# bind from_rebound From_Rebound
   Property_Result : Internal_Entity;
      New_Struct : Internal_Entity_Info;
Is_Null : Boolean;
If_Result : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 902 '<New AdaNode.entity at nodes.lkt:1265:100>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1265'
--# expr-start 901 '<If at nodes.lkt:1266:22>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1266'
--# expr-start 899 '<IsNull at nodes.lkt:1266:25>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1266'
Is_Null := N = null; 
--# expr-done 899
if Is_Null then
   If_Result := No_Entity_Info;
else
   --# expr-start 900 '<New EntityInfo at nodes.lkt:1266:62>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1266'
New_Struct := (From_Rebound => From_Rebound, Md => No_Metadata, Rebindings => Rebindings); 
--# expr-done 900
   If_Result := New_Struct;
end if;
--# expr-done 901
New_Struct_1 := (Info => If_Result, Node => N); 
--# expr-done 902
         Property_Result := New_Struct_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Entity_No_Md;
--# end
--# property-start AdaNode.env_mappings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1273'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Mappings
  (Node : Bare_Ada_Node
      ; Defining_Names : Bare_Defining_Name_List
      ; Value : Bare_Ada_Node
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind defining_names Defining_Names
   --# bind value Value
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_108 with Inline_Always;
            procedure Finalizer_Scope_2030 with Inline_Always;
      Item_0 : Bare_Defining_Name;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_108 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_108;
            procedure Finalizer_Scope_2030 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2030;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 906 '<Map at nodes.lkt:1273:96>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1273'
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
      if Defining_Names = null then
      Map_Result := Create_Internal_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Defining_Names;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind n Item_0
      --# expr-start 905 '<New EnvAssoc at nodes.lkt:1274:16>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1274'
--# expr-start 903 '<New DesignatedEnv at nodes.lkt:1275:56>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1275'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 903
--# expr-start 904 '<FieldAccess for name_symbol at nodes.lkt:1275:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1275'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_0);
--# end
--# expr-done 904
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Value); 
--# expr-done 905
         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2030;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 906
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_108;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_108;
                     Finalizer_Scope_2030;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Mappings;
--# end
--# property-start AdaNode.comp_bind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1282'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Comp_Bind
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_109 with Inline_Always;
      Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_109 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_109;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 907 '<Bind at nodes.lkt:1283:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1283'
Left.Value := No_Entity;
Entity_Vars.Reset (Left);
Right.Value := No_Entity;
Entity_Vars.Reset (Right);
Bind_Result := Solver.Create_Propagate (Left, Right, Logic_Functor_895'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1283:9") else null)); 
--# expr-done 907
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_109;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_109;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Comp_Bind;
--# end
--# property-start AdaNode.universal_int_bind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1289'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_110 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_110 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_110;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 909 '<Bind at nodes.lkt:1290:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1290'
Type_Var.Value := No_Entity;
Entity_Vars.Reset (Type_Var);
--# expr-start 908 '<FieldAccess for universal_int_type at nodes.lkt:1290:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1290'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 908
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
Bind_Result := Solver.Create_Assign (Type_Var, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1290:9") else null)); 
--# expr-done 909
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_110;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_110;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Int_Bind;
--# end
--# property-start AdaNode.universal_real_bind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1296'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_111 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_111 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_111;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 911 '<Bind at nodes.lkt:1297:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1297'
Type_Var.Value := No_Entity;
Entity_Vars.Reset (Type_Var);
--# expr-start 910 '<FieldAccess for universal_real_type at nodes.lkt:1297:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 910
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
Bind_Result := Solver.Create_Assign (Type_Var, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1297:9") else null)); 
--# expr-done 911
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_111;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_111;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Universal_Real_Bind;
--# end
--# property-start AdaNode.origin_node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1301'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Origin_Node
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node;
      Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Expr_Function;
Fld : Bare_Expr;
Cast_Result_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Bare_Ada_Node;
Fld_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 917 '<If at nodes.lkt:1302:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1302'
--# expr-start 912 '<FieldAccess for in_aspect_with_forward_visibility at nodes.lkt:1302:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1302'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.in_aspect_with_forward_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_In_Aspect_With_Forward_Visibility (Node => Self);
--# end
--# expr-done 912
if Fld_1 then
   If_Result_1 := No_Bare_Ada_Node;
else
   --# expr-start 916 '<If at nodes.lkt:1302:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1302'
--# expr-start 913 '<IsA ExprFunction at nodes.lkt:1303:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1303'
Is_A := Self /= null 
and then Self.Kind in Ada_Expr_Function_Range; 
--# expr-done 913
if Is_A then
--# expr-start 915 '<FieldAccess for expr at nodes.lkt:1303:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1303'
--# expr-start 914 '<Cast to ExprFunction at nodes.lkt:1303:40>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1303'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Expr_Function_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 914
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Cast_Result.Expr_Function_F_Expr;
--# expr-done 915
      Cast_Result_1 := Fld;
   If_Result := Cast_Result_1;
else
   If_Result := Self;
end if;
--# expr-done 916
   If_Result_1 := If_Result;
end if;
--# expr-done 917
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Origin_Node;
--# end
--# property-start AdaNode.env_hook 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1310'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Hook
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Library_Item;
Match_0_1 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Match_2 : Bare_Ada_Node;
Match_1_1 : Bare_Subunit;
Match_2_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Body_Node;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Fld_3 : Boolean;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Library_Item;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Bare_Subunit;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Bare_Ada_Node;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Match_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 923 '<Match at nodes.lkt:1310:28>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1310'
--# expr-start 918 '<parent at nodes.lkt:1310:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1310'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 918
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_0 := Cast_Result_3; 
--# bind _ Match_0
--# expr-start 921 '<Match at nodes.lkt:1311:32>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1311'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind b Match_0_1
--# expr-start 919 '<FieldAccess for env_hook_body at nodes.lkt:1312:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1312'
      if Match_0_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.env_hook_body
Fld_1 := Libadalang.Implementation.Body_Node_P_Env_Hook_Body (Node => Match_0_1);
--# end
--# expr-done 919
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind bd Match_1
--# expr-start 920 '<FieldAccess for env_hook_basic_decl at nodes.lkt:1313:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1313'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Match_1);
--# end
--# expr-done 920
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 921
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_1_1 := Cast_Result_4; 
--# bind su Match_1_1
--# expr-start 922 '<FieldAccess for env_hook_subunit at nodes.lkt:1316:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1316'
      if Match_1_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.env_hook_subunit
Fld_3 := Libadalang.Implementation.Subunit_P_Env_Hook_Subunit (Node => Match_1_1);
--# end
--# expr-done 922
Let_Result_4 := Fld_3; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_2_1 := Cast_Result_5; 
--# bind _ Match_2_1
Let_Result_5 := False; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 923
         Property_Result := Match_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Hook;
--# end
--# property-start AdaNode.env_get 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1345'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_114 with Inline_Always;
            procedure Finalizer_Scope_2037 with Inline_Always;
            procedure Finalizer_Scope_2042 with Inline_Always;
      Local_Real_From_Node : Bare_Ada_Node;
Local_Results : Internal_Entity_Array_Access;
Var_Expr_0 : Bare_Ada_Node;
Match_0 : Bare_Basic_Decl;
Match_1 : Bare_Defining_Name;
Match_2 : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_30 : Internal_Entity_Defining_Name;
Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Defining_Name;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Boolean;
If_Result_3 : Bare_Basic_Decl;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Defining_Name;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_3 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
Fld_6 : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Scope_Result_4 : Internal_Entity_Array_Access;
Result_Var_1 : Internal_Entity_Array_Access;
Scope_Result_5 : Internal_Entity_Array_Access;
Result_Var_2 : Internal_Entity_Array_Access;
Let_Result_3 : Internal_Entity_Array_Access;
Let_Result_4 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_114 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Local_Results);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_114;
            procedure Finalizer_Scope_2037 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2037;
            procedure Finalizer_Scope_2042 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2042;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 941 '<Let at nodes.lkt:1346:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1346'
--# scope-start
--# expr-start 924 '<FieldAccess for env_get_real_from_node at nodes.lkt:1346:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1346'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 924
Local_Real_From_Node := Fld; 
--# bind real_from_node Local_Real_From_Node
--# expr-start 940 '<Let at nodes.lkt:1347:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1347'
--# scope-start
--# expr-start 925 '<EnvGet at nodes.lkt:1347:23>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1347'
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Thin (Symbol), Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Local_Real_From_Node)); 
--# expr-done 925
Local_Results := Env_Get_Result; Inc_Ref (Local_Results);
--# bind results Local_Results
Var_Expr_0 := Local_Real_From_Node;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind rfn Var_Expr_0
--# expr-start 935 '<Match at nodes.lkt:1355:22>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1355'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 929 '<If at nodes.lkt:1356:39>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1356'
--# expr-start 927 '<FieldAccess for defining_name at nodes.lkt:1356:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1356'
--# expr-start 926 '<as_bare_entity at nodes.lkt:1356:42>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1356'
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_0); 
   If_Result := As_Entity;
end if;
--# expr-done 926
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 927
Var_Expr_30 := Fld_1;
if Var_Expr_30 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 928 '<FieldAccess for name_is at nodes.lkt:1356:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1356'
      if Var_Expr_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_30.Node, Sym => Symbol);
--# end
--# expr-done 928
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := Match_0;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 929
Let_Result := If_Result_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind dn Match_1
--# expr-start 934 '<If at nodes.lkt:1357:42>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1357'
--# expr-start 930 '<FieldAccess for name_is at nodes.lkt:1357:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1357'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Match_1, Sym => Symbol);
--# end
--# expr-done 930
if Fld_5 then
   --# expr-start 933 '<FieldAccess for node at nodes.lkt:1357:69>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1357'
--# expr-start 932 '<FieldAccess for basic_decl at nodes.lkt:1357:69>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1357'
--# expr-start 931 '<as_bare_entity at nodes.lkt:1357:69>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1357'
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Defining_Name;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Match_1); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 931
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 932
Fld_4 := Fld_3.Node;
--# expr-done 933
   If_Result_3 := Fld_4;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 934
Let_Result_1 := If_Result_3; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 935
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind enclosing_bd Var_Expr_1
--# expr-start 939 '<Map at nodes.lkt:1365:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1365'
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Results;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_2
         --# expr-start 938 '<Not at nodes.lkt:1365:39>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1365'
--# expr-start 937 '<Eq at nodes.lkt:1365:39>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1365'
--# expr-start 936 '<FieldAccess for node at nodes.lkt:1365:39>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1365'
Fld_6 := Item_2.Node;
--# expr-done 936
Cast_Expr := Var_Expr_1; 
      Cast_Result_3 := Cast_Expr;
Is_Equal := Fld_6 = Cast_Result_3; 
--# expr-done 937
Not_Val := not (Is_Equal); 
--# expr-done 938
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_2;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 939
Scope_Result_4 := Map_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2042;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := Local_Results;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_5 := Result_Var_1;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2037;
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := Local_Results;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_3 := Result_Var_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 940
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 941
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_114;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_114;
                     Finalizer_Scope_2037;
                     Finalizer_Scope_2042;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get;
--# end
--# property-start AdaNode.env_get_public 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1373'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Public
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_115 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Package_Decl;
Fld : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_115 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_115;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 948 '<Map at nodes.lkt:1373:213>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1373'
   --# expr-start 942 '<FieldAccess for env_get at nodes.lkt:1373:213>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1373'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Symbol, Lookup => Lookup, From_Node => From_Node, Categories => Categories);
--# end
--# expr-done 942
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind x Item_0
--# expr-start 943 '<Cast to PackageDecl at nodes.lkt:1376:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1376'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 943
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Package_Decl then
   --# scope-start
   --# bind pkg Var_Expr_1
--# expr-start 947 '<Not at nodes.lkt:1377:22>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1377'
--# expr-start 946 '<Eq at nodes.lkt:1377:22>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1377'
--# expr-start 944 '<FieldAccess for name_symbol at nodes.lkt:1377:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1377'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_1.Node);
--# end
--# expr-done 944
--# expr-start 945 '<Symbol root_types_ at nodes.lkt:1377:43>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1377'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Types); 
--# expr-done 945
Is_Equal := Fld_1 = Sym; 
--# expr-done 946
Not_Val := not (Is_Equal); 
--# expr-done 947
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
         if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 948
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_115;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_115;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Env_Get_Public;
--# end
--# property-start AdaNode.synthesize_defining_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1384'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Synthesize_Defining_Name
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Bare_Defining_Name
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Bare_Defining_Name;
      New_Node : Bare_Synthetic_Identifier;
Cast_Result : Bare_Name;
New_Node_1 : Bare_Synthetic_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Synthesize_Defining_Name,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Defining_Name;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 950 '<New SyntheticDefiningName at nodes.lkt:1384:63>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1384'
--# expr-start 949 '<New SyntheticIdentifier at nodes.lkt:1385:40>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1385'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Identifier);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Identifier,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
      New_Node.Expr_Logic_Vars :=
         System.Null_Address;
      New_Node.Synthetic_Identifier_Sym :=
         Sym;
--# expr-done 949
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "DefiningName.name cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Defining_Name);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Defining_Name,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Defining_Name
     (Self => New_Node_1,
      Defining_Name_F_Name => Cast_Result);
      New_Node_1.Expr_Logic_Vars :=
         System.Null_Address;
--# expr-done 950
      Cast_Result_1 := New_Node_1;
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Bare_Defining_Name,
                        As_Bare_Defining_Name => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Synthesize_Defining_Name;
--# end
--# property-start AdaNode.create_unop_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1392'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Unop_Assoc
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_117 with Inline_Always;
      Sym : Symbol_Type;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Type_Expr;
New_Node_3 : Bare_Synthetic_Unary_Spec;
Cast_Result_1 : Bare_Base_Subp_Spec;
New_Node_4 : Bare_Synthetic_Subp_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result_2 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_117 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_117;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Unop_Assoc,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 958 '<New EnvAssoc at nodes.lkt:1392:89>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1392'
--# expr-start 951 '<New DesignatedEnv at nodes.lkt:1399:21>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1399'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 951
--# expr-start 957 '<New SyntheticSubpDecl at nodes.lkt:1393:23>' New_Node_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1393'
--# expr-start 956 '<New SyntheticUnarySpec at nodes.lkt:1394:18>' New_Node_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1394'
--# expr-start 952 '<New SyntheticTypeExpr at nodes.lkt:1397:37>' New_Node_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1397'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 952
--# expr-start 955 '<New SyntheticFormalParamDecl at nodes.lkt:1395:45>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1395'
--# expr-start 953 '<New SyntheticTypeExpr at nodes.lkt:1396:53>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1396'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 953
      Cast_Result := New_Node;
--# expr-start 954 '<Symbol right at nodes.lkt:1396:32>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1396'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Right); 
--# expr-done 954
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Cast_Result);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 955
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node_2 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.return_type_expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticUnarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Unary_Spec);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Unary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Unary_Spec
     (Self => New_Node_3,
      Synthetic_Unary_Spec_F_Return_Type_Expr => New_Node_2, Synthetic_Unary_Spec_F_Right_Param => New_Node_1);
      New_Node_3.Synthetic_Unary_Spec_Subp_Symbol :=
         Op;
--# expr-done 956
      Cast_Result_1 := New_Node_3;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_4 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_4,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_4);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_4,
      Synthetic_Subp_Decl_F_Spec => Cast_Result_1);
--# expr-done 957
      Cast_Result_2 := New_Node_4;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Op, Metadata => No_Metadata, Value => Cast_Result_2); 
--# expr-done 958
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_117;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_117;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Unop_Assoc;
--# end
--# property-start AdaNode.create_binop_assoc_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1407'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_Impl
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Type_Expr
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_118 with Inline_Always;
      Sym : Symbol_Type;
New_Node : Bare_Synthetic_Formal_Param_Decl;
Sym_1 : Symbol_Type;
New_Node_1 : Bare_Synthetic_Formal_Param_Decl;
New_Node_2 : Bare_Synthetic_Binary_Spec;
Cast_Result : Bare_Base_Subp_Spec;
New_Node_3 : Bare_Synthetic_Subp_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_118 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_118;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_Impl,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 966 '<New EnvAssoc at nodes.lkt:1407:102>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1407'
--# expr-start 959 '<New DesignatedEnv at nodes.lkt:1412:21>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1412'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 959
--# expr-start 965 '<New SyntheticSubpDecl at nodes.lkt:1408:23>' New_Node_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1408'
--# expr-start 964 '<New SyntheticBinarySpec at nodes.lkt:1409:18>' New_Node_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1409'
--# expr-start 961 '<New SyntheticFormalParamDecl at nodes.lkt:1410:44>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1410'
--# expr-start 960 '<Symbol left at nodes.lkt:1410:80>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1410'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Left); 
--# expr-done 960
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node,
      Synthetic_Formal_Param_Decl_F_Param_Type => Lhs);
      New_Node.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym;
--# expr-done 961
--# expr-start 963 '<New SyntheticFormalParamDecl at nodes.lkt:1410:118>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1410'
--# expr-start 962 '<Symbol right at nodes.lkt:1410:154>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1410'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Right); 
--# expr-done 962
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticFormalParamDecl.param_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Formal_Param_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Formal_Param_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Formal_Param_Decl
     (Self => New_Node_1,
      Synthetic_Formal_Param_Decl_F_Param_Type => Rhs);
      New_Node_1.Synthetic_Formal_Param_Decl_Param_Name :=
         Sym_1;
--# expr-done 963
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if New_Node = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.left_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
      if New_Node_1 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticBinarySpec.right_param cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Binary_Spec);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Binary_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Binary_Spec
     (Self => New_Node_2,
      Synthetic_Binary_Spec_F_Left_Param => New_Node, Synthetic_Binary_Spec_F_Return_Type_Expr => Ret, Synthetic_Binary_Spec_F_Right_Param => New_Node_1);
      New_Node_2.Synthetic_Binary_Spec_Subp_Symbol :=
         Op;
--# expr-done 964
      Cast_Result := New_Node_2;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticSubpDecl.spec cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_3 := new Root_Node_Record
  (Ada_Synthetic_Subp_Decl);
Initialize
  (Self => New_Node_3,
   Kind => Ada_Synthetic_Subp_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_3);
   Initialize_Fields_For_Synthetic_Subp_Decl
     (Self => New_Node_3,
      Synthetic_Subp_Decl_F_Spec => Cast_Result);
--# expr-done 965
      Cast_Result_1 := New_Node_3;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Op, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 966
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_118;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_118;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_Impl;
--# end
--# property-start AdaNode.create_binop_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1421'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Base_Type_Decl
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_119 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
New_Node_2 : Bare_Synthetic_Type_Expr;
Cast_Result_2 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_119 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_119;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 970 '<FieldAccess for create_binop_assoc_impl at nodes.lkt:1421:109>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1421'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 967 '<New SyntheticTypeExpr at nodes.lkt:1422:13>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1422'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Lhs);
--# expr-done 967
      Cast_Result := New_Node;
--# expr-start 968 '<New SyntheticTypeExpr at nodes.lkt:1422:49>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1422'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 968
      Cast_Result_1 := New_Node_1;
--# expr-start 969 '<New SyntheticTypeExpr at nodes.lkt:1422:85>' New_Node_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1422'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_2,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 969
      Cast_Result_2 := New_Node_2;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Cast_Result, Rhs => Cast_Result_1, Ret => Cast_Result_2);
--# end
--# expr-done 970
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_119;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_119;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc;
--# end
--# property-start AdaNode.create_binop_assoc_l_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1428'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_L_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Base_Type_Decl
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_120 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_120 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_120;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_L_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 973 '<FieldAccess for create_binop_assoc_impl at nodes.lkt:1428:112>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1428'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 971 '<New SyntheticTypeExpr at nodes.lkt:1429:18>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1429'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Rhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Rhs);
--# expr-done 971
      Cast_Result := New_Node;
--# expr-start 972 '<New SyntheticTypeExpr at nodes.lkt:1429:54>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1429'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 972
      Cast_Result_1 := New_Node_1;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Lhs, Rhs => Cast_Result, Ret => Cast_Result_1);
--# end
--# expr-done 973
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_120;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_120;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_L_Expr;
--# end
--# property-start AdaNode.create_binop_assoc_r_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1435'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_R_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Base_Type_Decl
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_121 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Type_Expr;
Cast_Result_1 : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_121 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_121;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_R_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 976 '<FieldAccess for create_binop_assoc_impl at nodes.lkt:1435:112>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1435'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 974 '<New SyntheticTypeExpr at nodes.lkt:1436:13>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1436'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Lhs = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Lhs);
--# expr-done 974
      Cast_Result := New_Node;
--# expr-start 975 '<New SyntheticTypeExpr at nodes.lkt:1436:54>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1436'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node_1,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 975
      Cast_Result_1 := New_Node_1;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Cast_Result, Rhs => Rhs, Ret => Cast_Result_1);
--# end
--# expr-done 976
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_121;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_121;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_R_Expr;
--# end
--# property-start AdaNode.create_binop_assoc_l_r_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1442'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Create_Binop_Assoc_L_R_Expr
  (Node : Bare_Ada_Node
      ; Op : Symbol_Type
      ; Lhs : Bare_Type_Expr
      ; Rhs : Bare_Type_Expr
      ; Ret : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind op Op
   --# bind lhs Lhs
   --# bind rhs Rhs
   --# bind ret Ret
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_122 with Inline_Always;
      New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_122 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_122;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Create_Binop_Assoc_L_R_Expr,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Op);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Lhs);
               Mmz_K.Items (4) := (Kind => Mmz_Bare_Type_Expr,
                                      As_Bare_Type_Expr => Rhs);
               Mmz_K.Items (5) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Ret);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 978 '<FieldAccess for create_binop_assoc_impl at nodes.lkt:1442:110>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1442'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 977 '<New SyntheticTypeExpr at nodes.lkt:1443:23>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1443'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Ret = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Ret);
--# expr-done 977
      Cast_Result := New_Node;
--# property-call-start AdaNode.create_binop_assoc_impl
Fld := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_Impl (Node => Self, Op => Op, Lhs => Lhs, Rhs => Rhs, Ret => Cast_Result);
--# end
--# expr-done 978
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_122;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_122;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc,
                        As_Internal_Env_Assoc => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ada_Node_P_Create_Binop_Assoc_L_R_Expr;
--# end
--# property-start '[dispatcher]AdaNode.custom_id_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : String_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Signed_Int_Type_Def | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.custom_id_text
                  Property_Result := Ada_Node_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.custom_id_text
                  Property_Result := Basic_Decl_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Synthetic_Type_Expr =>
                  --# property-call-start TypeExpr.custom_id_text
                  Property_Result := Type_Expr_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.custom_id_text
                  Property_Result := Anonymous_Type_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.custom_id_text
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Custom_Id_Text;
--# end
--# property-start '[dispatcher]AdaNode.complete_items' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete_Items
  (Node : Bare_Ada_Node
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Abstract_State_Decl_List | Ada_Constraint_List .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete_items
                  Property_Result := Ada_Node_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Alternatives_List =>
                  --# property-call-start AlternativesList.complete_items
                  Property_Result := Alternatives_List_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete_items
                  Property_Result := Dotted_Name_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start Identifier.complete_items
                  Property_Result := Identifier_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.complete_items
                  Property_Result := Subtype_Indication_P_Complete_Items
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Complete_Items;
--# end
--# property-start '[dispatcher]AdaNode.complete_item_weight' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete_Item_Weight
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Constant_Present | Ada_Delta_Constraint .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete_item_weight
                  Property_Result := Ada_Node_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint =>
                  --# property-call-start CompositeConstraint.complete_item_weight
                  Property_Result := Composite_Constraint_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete_item_weight
                  Property_Result := Dotted_Name_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.complete_item_weight
                  Property_Result := Assign_Stmt_P_Complete_Item_Weight
                    (Self
                        , Item
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Complete_Item_Weight;
--# end
--# property-start '[dispatcher]AdaNode.spark_mode_aspect' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Spark_Mode_Aspect
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Range_Constraint | Ada_Elsif_Expr_Part .. Ada_Format_String_Tok_String | Ada_Interface_Kind_Limited .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.spark_mode_aspect
                  Property_Result := Ada_Node_P_Spark_Mode_Aspect
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.spark_mode_aspect
                  Property_Result := Basic_Decl_P_Spark_Mode_Aspect
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Declarative_Part | Ada_Public_Part =>
                  --# property-call-start DeclarativePart.spark_mode_aspect
                  Property_Result := Declarative_Part_P_Spark_Mode_Aspect
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Private_Part =>
                  --# property-call-start PrivatePart.spark_mode_aspect
                  Property_Result := Private_Part_P_Spark_Mode_Aspect
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Handled_Stmts =>
                  --# property-call-start HandledStmts.spark_mode_aspect
                  Property_Result := Handled_Stmts_P_Spark_Mode_Aspect
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Spark_Mode_Aspect;
--# end
--# property-start '[dispatcher]AdaNode.immediate_declarative_region' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Immediate_Declarative_Region
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Declarative_Part | Ada_Public_Part .. Ada_Begin_Block | Ada_Case_Stmt .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.immediate_declarative_region
                  Property_Result := Ada_Node_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.immediate_declarative_region
                  Property_Result := Basic_Decl_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Private_Part =>
                  --# property-call-start PrivatePart.immediate_declarative_region
                  Property_Result := Private_Part_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Block =>
                  --# property-call-start DeclBlock.immediate_declarative_region
                  Property_Result := Decl_Block_P_Immediate_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Immediate_Declarative_Region;
--# end
--# property-start '[dispatcher]AdaNode.xref_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_All_Present | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Record_Def | Ada_Composite_Constraint_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Accept_Stmt_Body | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl .. Ada_Synthetic_Object_Decl | Ada_Compilation_Unit | Ada_Constant_Absent .. Ada_Constant_Present | Ada_Declarative_Part .. Ada_Elsif_Expr_Part | Ada_Case_Expr_Alternative | Ada_Concat_Operand | Ada_Contract_Cases | Ada_Discrete_Subtype_Name | Ada_Synthetic_Identifier | Ada_Format_String_Tok_End .. Ada_Limited_Present | Ada_Mode_Default .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_Reverse_Present | Ada_Error_Stmt | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Tagged_Present | Ada_Type_Attributes_Repository | Ada_Anonymous_Type_Access_Def | Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_equation
                  Property_Result := Ada_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.xref_equation
                  Property_Result := Constrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.xref_equation
                  Property_Result := Unconstrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_equation
                  Property_Result := Aspect_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_At_Clause =>
                  --# property-call-start AtClause.xref_equation
                  Property_Result := At_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Def_Clause =>
                  --# property-call-start AttributeDefClause.xref_equation
                  Property_Result := Attribute_Def_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Rep_Clause =>
                  --# property-call-start EnumRepClause.xref_equation
                  Property_Result := Enum_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Record_Rep_Clause =>
                  --# property-call-start RecordRepClause.xref_equation
                  Property_Result := Record_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.xref_equation
                  Property_Result := Base_Subp_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.xref_equation
                  Property_Result := Entry_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_equation
                  Property_Result := Aggregate_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_equation
                  Property_Result := Iterated_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_equation
                  Property_Result := Param_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_equation
                  Property_Result := Component_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_equation
                  Property_Result := Discriminant_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_equation
                  Property_Result := Param_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_equation
                  Property_Result := Subtype_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Formal_Type_Decl =>
                  --# property-call-start IncompleteFormalTypeDecl.xref_equation
                  Property_Result := Incomplete_Formal_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_equation
                  Property_Result := Protected_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_equation
                  Property_Result := Task_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Concrete_Type_Decl =>
                  --# property-call-start TypeDecl.xref_equation
                  Property_Result := Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Type_Decl =>
                  --# property-call-start FormalTypeDecl.xref_equation
                  Property_Result := Formal_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.xref_equation
                  Property_Result := Formal_Subp_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_equation
                  Property_Result := Expr_Function_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_equation
                  Property_Result := Subp_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_equation
                  Property_Result := Entry_Body_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_equation
                  Property_Result := Entry_Index_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_equation
                  Property_Result := Exception_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_equation
                  Property_Result := Exception_Handler_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.xref_equation
                  Property_Result := For_Loop_Var_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_equation
                  Property_Result := Generic_Instantiation_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_equation
                  Property_Result := Generic_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_equation
                  Property_Result := Number_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_equation
                  Property_Result := Object_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start NoTypeObjectRenamingDecl.xref_equation
                  Property_Result := No_Type_Object_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_equation
                  Property_Result := Package_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_equation
                  Property_Result := Single_Protected_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_equation
                  Property_Result := Case_Stmt_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_equation
                  Property_Result := Component_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Component_Def =>
                  --# property-call-start ComponentDef.xref_equation
                  Property_Result := Component_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint =>
                  --# property-call-start CompositeConstraint.xref_equation
                  Property_Result := Composite_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Delta_Constraint =>
                  --# property-call-start DeltaConstraint.xref_equation
                  Property_Result := Delta_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Digits_Constraint =>
                  --# property-call-start DigitsConstraint.xref_equation
                  Property_Result := Digits_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Range_Constraint =>
                  --# property-call-start RangeConstraint.xref_equation
                  Property_Result := Range_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_equation
                  Property_Result := Elsif_Stmt_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl_Expr =>
                  --# property-call-start AbstractStateDeclExpr.xref_equation
                  Property_Result := Abstract_State_Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.xref_equation
                  Property_Result := Allocator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.xref_equation
                  Property_Result := Base_Aggregate_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.xref_equation
                  Property_Result := Bin_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Box_Expr =>
                  --# property-call-start BoxExpr.xref_equation
                  Property_Result := Box_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Concat_Op =>
                  --# property-call-start ConcatOp.xref_equation
                  Property_Result := Concat_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.xref_equation
                  Property_Result := Case_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.xref_equation
                  Property_Result := If_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.xref_equation
                  Property_Result := Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Format_String_Literal =>
                  --# property-call-start FormatStringLiteral.xref_equation
                  Property_Result := Format_String_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Membership_Expr =>
                  --# property-call-start MembershipExpr.xref_equation
                  Property_Result := Membership_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name =>
                  --# property-call-start ArraySubcomponentChoiceName.xref_equation
                  Property_Result := Array_Subcomponent_Choice_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.xref_equation
                  Property_Result := Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_equation
                  Property_Result := Call_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.xref_equation
                  Property_Result := Defining_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.xref_equation
                  Property_Result := Dotted_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_equation
                  Property_Result := End_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.xref_equation
                  Property_Result := Explicit_Deref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.xref_equation
                  Property_Result := Qual_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Reduce_Attribute_Ref =>
                  --# property-call-start ReduceAttributeRef.xref_equation
                  Property_Result := Reduce_Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start BaseId.xref_equation
                  Property_Result := Base_Id_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.xref_equation
                  Property_Result := Char_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.xref_equation
                  Property_Result := Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.xref_equation
                  Property_Result := String_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.xref_equation
                  Property_Result := Null_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal =>
                  --# property-call-start IntLiteral.xref_equation
                  Property_Result := Int_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Real_Literal =>
                  --# property-call-start RealLiteral.xref_equation
                  Property_Result := Real_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.xref_equation
                  Property_Result := Target_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Update_Attribute_Ref =>
                  --# property-call-start UpdateAttributeRef.xref_equation
                  Property_Result := Update_Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.xref_equation
                  Property_Result := Paren_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Quantified_Expr =>
                  --# property-call-start QuantifiedExpr.xref_equation
                  Property_Result := Quantified_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.xref_equation
                  Property_Result := Raise_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.xref_equation
                  Property_Result := Un_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Iter_Filter =>
                  --# property-call-start ForLoopIterFilter.xref_equation
                  Property_Result := For_Loop_Iter_Filter_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Format_String_Chunk =>
                  --# property-call-start FormatStringChunk.xref_equation
                  Property_Result := Format_String_Chunk_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_equation
                  Property_Result := For_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_While_Loop_Spec =>
                  --# property-call-start WhileLoopSpec.xref_equation
                  Property_Result := While_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Others_Designator =>
                  --# property-call-start OthersDesignator.xref_equation
                  Property_Result := Others_Designator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_equation
                  Property_Result := Pragma_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_equation
                  Property_Result := Range_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Select_When_Part =>
                  --# property-call-start SelectWhenPart.xref_equation
                  Property_Result := Select_When_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
                  --# property-call-start AcceptStmt.xref_equation
                  Property_Result := Accept_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Stmt .. Ada_While_Loop_Stmt =>
                  --# property-call-start BaseLoopStmt.xref_equation
                  Property_Result := Base_Loop_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Begin_Block .. Ada_Decl_Block =>
                  --# property-call-start BlockStmt.xref_equation
                  Property_Result := Block_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt =>
                  --# property-call-start CaseStmt.xref_equation
                  Property_Result := Case_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Extended_Return_Stmt =>
                  --# property-call-start ExtendedReturnStmt.xref_equation
                  Property_Result := Extended_Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_If_Stmt =>
                  --# property-call-start IfStmt.xref_equation
                  Property_Result := If_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt =>
                  --# property-call-start NamedStmt.xref_equation
                  Property_Result := Named_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Select_Stmt =>
                  --# property-call-start SelectStmt.xref_equation
                  Property_Result := Select_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Abort_Stmt =>
                  --# property-call-start AbortStmt.xref_equation
                  Property_Result := Abort_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.xref_equation
                  Property_Result := Assign_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Call_Stmt =>
                  --# property-call-start CallStmt.xref_equation
                  Property_Result := Call_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Delay_Stmt =>
                  --# property-call-start DelayStmt.xref_equation
                  Property_Result := Delay_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Exit_Stmt =>
                  --# property-call-start ExitStmt.xref_equation
                  Property_Result := Exit_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Goto_Stmt =>
                  --# property-call-start GotoStmt.xref_equation
                  Property_Result := Goto_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Label =>
                  --# property-call-start Label.xref_equation
                  Property_Result := Label_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Null_Stmt =>
                  --# property-call-start NullStmt.xref_equation
                  Property_Result := Null_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Stmt =>
                  --# property-call-start RaiseStmt.xref_equation
                  Property_Result := Raise_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Requeue_Stmt =>
                  --# property-call-start RequeueStmt.xref_equation
                  Property_Result := Requeue_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Return_Stmt =>
                  --# property-call-start ReturnStmt.xref_equation
                  Property_Result := Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Simple_Decl_Stmt =>
                  --# property-call-start SimpleDeclStmt.xref_equation
                  Property_Result := Simple_Decl_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Terminate_Alternative =>
                  --# property-call-start TerminateAlternative.xref_equation
                  Property_Result := Terminate_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_equation
                  Property_Result := Subunit_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Task_Def =>
                  --# property-call-start TaskDef.xref_equation
                  Property_Result := Task_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.xref_equation
                  Property_Result := Access_To_Subp_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.xref_equation
                  Property_Result := Type_Access_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_equation
                  Property_Result := Array_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.xref_equation
                  Property_Result := Derived_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.xref_equation
                  Property_Result := Enum_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.xref_equation
                  Property_Result := Formal_Discrete_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.xref_equation
                  Property_Result := Interface_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.xref_equation
                  Property_Result := Mod_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.xref_equation
                  Property_Result := Private_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.xref_equation
                  Property_Result := Decimal_Fixed_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.xref_equation
                  Property_Result := Floating_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.xref_equation
                  Property_Result := Ordinary_Fixed_Point_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.xref_equation
                  Property_Result := Record_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.xref_equation
                  Property_Result := Signed_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.xref_equation
                  Property_Result := Anonymous_Type_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.xref_equation
                  Property_Result := Subtype_Indication_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Use_Package_Clause =>
                  --# property-call-start UsePackageClause.xref_equation
                  Property_Result := Use_Package_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Use_Type_Clause =>
                  --# property-call-start UseTypeClause.xref_equation
                  Property_Result := Use_Type_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Value_Sequence =>
                  --# property-call-start ValueSequence.xref_equation
                  Property_Result := Value_Sequence_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_equation
                  Property_Result := Variant_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_equation
                  Property_Result := With_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_stop_resolution' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Record_Def | Ada_Composite_Constraint_Assoc | Ada_Param_Assoc .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_stop_resolution
                  Property_Result := Ada_Node_P_Xref_Stop_Resolution
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_stop_resolution
                  Property_Result := Aggregate_Assoc_P_Xref_Stop_Resolution
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_stop_resolution
                  Property_Result := Iterated_Assoc_P_Xref_Stop_Resolution
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_stop_resolution
                  Property_Result := Call_Expr_P_Xref_Stop_Resolution
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_stop_resolution
                  Property_Result := Range_Spec_P_Xref_Stop_Resolution
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start '[dispatcher]AdaNode.stop_resolution_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.stop_resolution_equation
                  Property_Result := Ada_Node_P_Stop_Resolution_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.stop_resolution_equation
                  Property_Result := Call_Expr_P_Stop_Resolution_Equation
                    (Self
                        , Env
                        , Origin
                        , Entry_Point
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_entry_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Unconstrained_Array_Indices | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Accept_Stmt_Body | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl .. Ada_Synthetic_Object_Decl | Ada_Compilation_Unit | Ada_Component_Def .. Ada_Elsif_Expr_Part | Ada_Abstract_State_Decl_Expr .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_Dotted_Name | Ada_Explicit_Deref .. Ada_Un_Op | Ada_Format_String_Chunk .. Ada_Limited_Present | Ada_While_Loop_Spec .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Select_When_Part | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Type_Access_Def | Ada_Derived_Type_Def .. Ada_Until_Present | Ada_Value_Sequence .. Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_entry_point
                  Property_Result := Ada_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_entry_point
                  Property_Result := Aspect_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_At_Clause .. Ada_Record_Rep_Clause =>
                  --# property-call-start AspectClause.xref_entry_point
                  Property_Result := Aspect_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.xref_entry_point
                  Property_Result := Base_Subp_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_entry_point
                  Property_Result := Param_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_entry_point
                  Property_Result := Component_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_entry_point
                  Property_Result := Discriminant_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_entry_point
                  Property_Result := Param_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_entry_point
                  Property_Result := Subtype_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_entry_point
                  Property_Result := Protected_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_entry_point
                  Property_Result := Task_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.xref_entry_point
                  Property_Result := Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start AnonymousTypeDecl.xref_entry_point
                  Property_Result := Anonymous_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.xref_entry_point
                  Property_Result := Formal_Subp_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_entry_point
                  Property_Result := Expr_Function_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_entry_point
                  Property_Result := Subp_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_entry_point
                  Property_Result := Entry_Body_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_entry_point
                  Property_Result := Entry_Index_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_entry_point
                  Property_Result := Exception_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_entry_point
                  Property_Result := Exception_Handler_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_entry_point
                  Property_Result := Generic_Instantiation_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_entry_point
                  Property_Result := Generic_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_entry_point
                  Property_Result := Number_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.xref_entry_point
                  Property_Result := Object_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_entry_point
                  Property_Result := Package_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_entry_point
                  Property_Result := Single_Protected_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_entry_point
                  Property_Result := Case_Stmt_Alternative_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_entry_point
                  Property_Result := Component_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_entry_point
                  Property_Result := Elsif_Stmt_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.xref_entry_point
                  Property_Result := Defining_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_entry_point
                  Property_Result := End_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_For_Loop_Iter_Filter =>
                  --# property-call-start ForLoopIterFilter.xref_entry_point
                  Property_Result := For_Loop_Iter_Filter_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_entry_point
                  Property_Result := For_Loop_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_entry_point
                  Property_Result := Pragma_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Terminate_Alternative =>
                  --# property-call-start Stmt.xref_entry_point
                  Property_Result := Stmt_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_entry_point
                  Property_Result := Subunit_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_entry_point
                  Property_Result := Array_Type_Def_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
                  --# property-call-start UseClause.xref_entry_point
                  Property_Result := Use_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_entry_point
                  Property_Result := Variant_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_entry_point
                  Property_Result := With_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Ada_Node_P_Xref_Entry_Point;
--# end
--# property-start AdaNode.matching_with_use_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1536'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Matching_With_Use_Clause
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_2048 with Inline_Always;
      Var_Expr_0 : Bare_Use_Package_Clause;
Var_Expr_1 : Internal_Entity_With_Clause;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Use_Package_Clause;
Fld : Internal_Entity;
Cast_Result_1 : Internal_Entity_With_Clause;
Fld_1 : Bare_Name_List;
Len : Integer;
Fld_2 : Internal_Entity_Name_List;
Fld_3 : Bare_Name_List;
Len_1 : Integer;
Is_Equal : Boolean;
Fld_4 : Bare_Name_List;
Get_Result : Bare_Name;
Fld_5 : Symbol_Type_Array_Access;
Fld_6 : Internal_Entity_Name_List;
Fld_7 : Bare_Name_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_9 : Symbol_Type_Array_Access;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
            procedure Finalizer_Scope_2048 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2048;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 979 '<Cast to UsePackageClause at nodes.lkt:1537:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1537'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 979
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind uc Var_Expr_0
--# expr-start 981 '<Cast to WithClause at nodes.lkt:1538:21>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1538'
--# expr-start 980 '<FieldAccess for previous_sibling at nodes.lkt:1538:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1538'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Previous_Sibling (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 980
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_With_Clause_Range
   then
      Cast_Result_1 := Create_Internal_Entity_With_Clause
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result_1 := No_Entity_With_Clause;
   end if;
--# expr-done 981
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_With_Clause then
   --# scope-start
   --# bind wc Var_Expr_1
--# expr-start 986 '<Eq at nodes.lkt:1540:19>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1540'
--# expr-start 983 '<length at nodes.lkt:1540:19>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1540'
--# expr-start 982 '<FieldAccess for packages at nodes.lkt:1540:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1540'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_0.Use_Package_Clause_F_Packages;
--# expr-done 982
Len := Length (Fld_1); 
--# expr-done 983
--# expr-start 985 '<length at nodes.lkt:1540:43>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1540'
--# expr-start 984 '<FieldAccess for packages at nodes.lkt:1540:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1540'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name_List (Node => Var_Expr_1.Node.With_Clause_F_Packages, Info => Var_Expr_1.Info);
--# expr-done 984
Fld_3 := Fld_2.Node; 
Len_1 := Length (Fld_3); 
--# expr-done 985
Is_Equal := Len = Len_1; 
--# expr-done 986
if Is_Equal then
   --# expr-start 992 '<Eq at nodes.lkt:1541:23>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1541'
--# expr-start 989 '<FieldAccess for as_symbol_array at nodes.lkt:1541:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1541'
--# expr-start 988 '<at_or_raise at nodes.lkt:1541:23>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1541'
--# expr-start 987 '<FieldAccess for packages at nodes.lkt:1541:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1541'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr_0.Use_Package_Clause_F_Packages;
--# expr-done 987
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Get_Result := Get (Self, Fld_4, 0, False); 
--# expr-done 988
      if Get_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Get_Result);
--# end
--# expr-done 989
--# expr-start 991 '<FieldAccess for as_symbol_array at nodes.lkt:1542:26>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1542'
--# expr-start 990 '<FieldAccess for packages at nodes.lkt:1542:26>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1542'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name_List (Node => Var_Expr_1.Node.With_Clause_F_Packages, Info => Var_Expr_1.Info);
--# expr-done 990
Fld_7 := Fld_6.Node;
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Get_Result_1 := Get (Self, Fld_7, 0, False); 
Is_Null := Get_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Get_Result_1); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => If_Result.Node);
--# end
--# expr-done 991
Is_Equal_1 := Equivalent (Fld_5, Fld_9); 
--# expr-done 992
   If_Result_1 := Is_Equal_1;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_2048;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2048;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Matching_With_Use_Clause;
--# end
--# property-start AdaNode.custom_id_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1450'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_123 with Inline_Always;
      Str : String_Type;
            procedure Finalizer_Scope_123 is
            begin
                     Dec_Ref (Str);
            end Finalizer_Scope_123;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 993 '<String at nodes.lkt:1450:36>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1450'
Str := Create_String (""); 
--# expr-done 993
         Property_Result := Str;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_123;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_123;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.complete_items 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1456'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Items
  (Node : Bare_Ada_Node
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_124 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
Fld_5 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_6 : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_124 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_124;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1004 '<Map at nodes.lkt:1457:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1457'
   --# expr-start 995 '<FieldAccess for env_get_public at nodes.lkt:1457:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1457'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 994 '<FieldAccess for children_env at nodes.lkt:1457:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1457'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 994
--# property-call-start AdaNode.env_get_public
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Get_Public (Node => Self, Env => Fld, Symbol => No_Symbol);
--# end
--# expr-done 995
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 1003 '<New CompletionItem at nodes.lkt:1458:20>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1458'
--# expr-start 996 '<Cast to BasicDecl at nodes.lkt:1459:22>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1459'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 996
--# expr-start 999 '<FieldAccess for dottable_subp at nodes.lkt:1460:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1460'
--# expr-start 998 '<FieldAccess for md at nodes.lkt:1460:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1460'
--# expr-start 997 '<FieldAccess for info at nodes.lkt:1460:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1460'
Fld_2 := Item_0.Info;
--# expr-done 997
Fld_3 := Fld_2.Md;
--# expr-done 998
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 999
--# expr-start 1000 '<FieldAccess for has_visibility at nodes.lkt:1461:28>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1461'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 1000
--# expr-start 1002 '<FieldAccess for complete_item_weight at nodes.lkt:1462:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1462'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1001 '<Cast to BasicDecl at nodes.lkt:1462:50>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1462'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1001
--# property-call-start '[dispatcher]AdaNode.complete_item_weight'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Cast_Result_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1002
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_4, Is_Visible => Fld_5, Weight => Fld_6); 
--# expr-done 1003
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1004
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_124;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_124;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Items;
--# end
--# property-start AdaNode.complete_item_weight 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1474'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete_Item_Weight
  (Node : Bare_Ada_Node
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := 0;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Complete_Item_Weight;
--# end
--# property-start AdaNode.spark_mode_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1487'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Spark_Mode_Aspect
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
     Ent : Internal_Entity :=
       Internal_Entity'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
      Fld : Internal_Entity;
Fld_1 : Internal_Aspect;
Fld_2 : Internal_Entity;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1010 '<If at nodes.lkt:1488:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
--# expr-start 1007 '<Not at nodes.lkt:1488:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
--# expr-start 1006 '<IsNull at nodes.lkt:1488:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
--# expr-start 1005 '<parent at nodes.lkt:1488:16>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1005
Is_Null := Fld_2.Node = null; 
--# expr-done 1006
Not_Val := not (Is_Null); 
--# expr-done 1007
if Not_Val then
   --# expr-start 1009 '<FieldAccess for spark_mode_aspect at nodes.lkt:1488:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
--# expr-start 1008 '<parent at nodes.lkt:1488:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1488'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1008
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1009
   If_Result := Fld_1;
else
   If_Result := raise Precondition_Failure with "SPARK Mode does not apply here";
end if;
--# expr-done 1010
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Spark_Mode_Aspect;
--# end
--# property-start AdaNode.immediate_declarative_region 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1501'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Immediate_Declarative_Region
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Immediate_Declarative_Region;
--# end
--# property-start AdaNode.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1514'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property AdaNode.xref_equation not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Equation;
--# end
--# property-start AdaNode.xref_stop_resolution 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1517'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.stop_resolution_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1520'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_130 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_130 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_130;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1520:48") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_130;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_130;
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start AdaNode.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1531'
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node  := Bare_Ada_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ada_Node_P_Xref_Entry_Point;
--# end
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
   declare
      Result : constant Internal_Entity :=
         Ada_Node_P_Resolve_Generic_Actual (E.Node, E_Info);
   begin
         return (Node => Result.Node, Info => Result.Info);
   end;
end;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env
is
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body (E.Node);
   begin
         return Result;
   end;
end;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Use_Clauses_In_Generic_Formal_Part (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
begin
   declare
      Result : constant Lexical_Env :=
         Ada_Node_P_Nested_Generic_Formal_Part (E.Node);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Abort_Node
   --
--# property-start '[dispatcher]Abort.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abort_Node_P_As_Bool
  (Node : Bare_Abort_Node
  )
   return Boolean
is
   Self : Bare_Abort_Node  := Bare_Abort_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Abort_Node (Self.Kind) is
               when Ada_Abort_Absent =>
                  --# property-call-start Abort.Absent.as_bool
                  Property_Result := Abort_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abort_Present =>
                  --# property-call-start Abort.Present.as_bool
                  Property_Result := Abort_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Abort_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Absent
   --
--# property-start Abort.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1549'
pragma Warnings (Off, "is not referenced");
 function Abort_Absent_P_As_Bool
  (Node : Bare_Abort_Absent
  )
   return Boolean
is
   Self : Bare_Abort_Absent  := Bare_Abort_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abort_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Present
   --
--# property-start Abort.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1549'
pragma Warnings (Off, "is not referenced");
 function Abort_Present_P_As_Bool
  (Node : Bare_Abort_Present
  )
   return Boolean
is
   Self : Bare_Abort_Present  := Bare_Abort_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abort_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Node
   --
--# property-start '[dispatcher]Abstract.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abstract_Node_P_As_Bool
  (Node : Bare_Abstract_Node
  )
   return Boolean
is
   Self : Bare_Abstract_Node  := Bare_Abstract_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Abstract_Node (Self.Kind) is
               when Ada_Abstract_Absent =>
                  --# property-call-start Abstract.Absent.as_bool
                  Property_Result := Abstract_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abstract_Present =>
                  --# property-call-start Abstract.Present.as_bool
                  Property_Result := Abstract_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Abstract_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Absent
   --
--# property-start Abstract.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1554'
pragma Warnings (Off, "is not referenced");
 function Abstract_Absent_P_As_Bool
  (Node : Bare_Abstract_Absent
  )
   return Boolean
is
   Self : Bare_Abstract_Absent  := Bare_Abstract_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Present
   --
--# property-start Abstract.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1554'
pragma Warnings (Off, "is not referenced");
 function Abstract_Present_P_As_Bool
  (Node : Bare_Abstract_Present
  )
   return Boolean
is
   Self : Bare_Abstract_Present  := Bare_Abstract_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Ada_List
   --
   --
   --  Primitives for Bare_Ada_Node_List
   --
   --
   --  Primitives for Bare_Abstract_State_Decl_List
   --
   --
   --  Primitives for Bare_Alternatives_List
   --
--# property-start AlternativesList.enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1565'
pragma Warnings (Off, "is not referenced");
 function Alternatives_List_P_Enum_Type
  (Node : Bare_Alternatives_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Alternatives_List  := Bare_Alternatives_List (Node);
     Ent : Internal_Entity_Alternatives_List :=
       Internal_Entity_Alternatives_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Case_Stmt;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1014 '<Cast to CaseStmt at nodes.lkt:1566:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
--# expr-start 1013 '<parent at nodes.lkt:1566:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
--# expr-start 1012 '<parent at nodes.lkt:1566:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
--# expr-start 1011 '<parent at nodes.lkt:1566:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1011
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1012
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 1013
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Case_Stmt;
   end if;
--# expr-done 1014
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Case_Stmt then
   --# scope-start
   --# bind cs Var_Expr_0
--# expr-start 1016 '<FieldAccess for expression_type at nodes.lkt:1566:59>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
--# expr-start 1015 '<FieldAccess for expr at nodes.lkt:1566:59>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1566'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Var_Expr_0.Node.Case_Stmt_F_Expr, Info => Var_Expr_0.Info);
--# expr-done 1015
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_4 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 1016
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Alternatives_List_P_Enum_Type;
--# end
--# property-start AlternativesList.complete_items 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1570'
pragma Warnings (Off, "is not referenced");
 function Alternatives_List_P_Complete_Items
  (Node : Bare_Alternatives_List
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Alternatives_List  := Bare_Alternatives_List (Node);
     Ent : Internal_Entity_Alternatives_List :=
       Internal_Entity_Alternatives_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_156 with Inline_Always;
      Match_0 : Internal_Entity_Enum_Literal_Decl;
Match_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Integer;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Integer;
Scope_Result : Integer;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Integer;
Scope_Result_1 : Integer;
Match_Result : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_156 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_156;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1030 '<Map at nodes.lkt:1570:51>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1570'
   --# expr-start 1018 '<EnvGet at nodes.lkt:1570:51>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1570'
--# expr-start 1017 '<FieldAccess for children_env at nodes.lkt:1570:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1570'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 1017
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Thin (No_Symbol), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1018
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 1029 '<New CompletionItem at nodes.lkt:1571:16>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1571'
--# expr-start 1019 '<Cast to BasicDecl at nodes.lkt:1572:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1572'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 1019
--# expr-start 1022 '<FieldAccess for dottable_subp at nodes.lkt:1573:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1573'
--# expr-start 1021 '<FieldAccess for md at nodes.lkt:1573:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1573'
--# expr-start 1020 '<FieldAccess for info at nodes.lkt:1573:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1573'
Fld_1 := Item_0.Info;
--# expr-done 1020
Fld_2 := Fld_1.Md;
--# expr-done 1021
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 1022
--# expr-start 1023 '<FieldAccess for has_visibility at nodes.lkt:1574:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1574'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_4 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 1023
--# expr-start 1028 '<Match at nodes.lkt:1575:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1575'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind eld Match_0
--# expr-start 1027 '<If at nodes.lkt:1576:46>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1576'
--# expr-start 1026 '<Eq at nodes.lkt:1576:49>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1576'
--# expr-start 1024 '<FieldAccess for enum_type at nodes.lkt:1576:49>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1576'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AlternativesList.enum_type
Fld_5 := Libadalang.Implementation.Alternatives_List_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1024
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# expr-start 1025 '<FieldAccess for enum_type at nodes.lkt:1576:69>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1576'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 1025
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1026
if Is_Equiv then
   If_Result := 100;
else
   If_Result := 0;
end if;
--# expr-done 1027
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := 0; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 1028
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_4, Weight => Match_Result); 
--# expr-done 1029
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1030
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_156;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_156;
            raise;
      end;
   return Property_Result;
end Alternatives_List_P_Complete_Items;
--# end
   --
   --  Primitives for Bare_Constraint_List
   --
   --
   --  Primitives for Bare_Decl_List
   --
   --
   --  Primitives for Bare_Stmt_List
   --
   --
   --  Primitives for Bare_Aspect_Assoc_List
   --
   --
   --  Primitives for Bare_Base_Assoc_List
   --
   --
   --  Primitives for Bare_Basic_Assoc_List
   --
   --
   --  Primitives for Bare_Assoc_List
   --
--# property-start AssocList.actual_for_param_at 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1599'
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Actual_For_Param_At
  (Node : Bare_Assoc_List
      ; Param : Internal_Entity_Defining_Name
      ; Pos : Integer
      ; Default_Expr : Internal_Entity_Expr
         := No_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind pos Pos
   --# bind default_expr Default_Expr
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_160 with Inline_Always;
      Local_Up : Internal_Single_Actual_Array_Access;
Var_Expr_1 : Internal_Single_Actual;
Var_Expr_31 : Bare_Base_Id;
Fld : Internal_Single_Actual_Array_Access;
Fld_1 : Bare_Base_Id;
Fld_2 : Internal_Entity_Name;
Fld_3 : Bare_Name;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Single_Actual;
Fld_5 : Internal_Entity_Basic_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Get_Result : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Get_Result_1 : Internal_Single_Actual;
Is_Equal : Boolean;
Get_Result_2 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Bare_Ada_Node_Array_Access;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_160 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Up);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_160;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1050 '<Let at nodes.lkt:1600:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1600'
--# scope-start
--# expr-start 1031 '<FieldAccess for unpacked_params at nodes.lkt:1600:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1031
Local_Up := Fld; Inc_Ref (Local_Up);
--# bind up Local_Up
--# expr-start 1036 '<Find at nodes.lkt:1602:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1602'
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Local_Up;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
--# expr-start 1032 '<FieldAccess for name at nodes.lkt:1604:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1604'
Fld_1 := Item_0.Name;
--# expr-done 1032
Var_Expr_31 := Fld_1;
if Var_Expr_31 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1035 '<FieldAccess for matches at nodes.lkt:1604:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1604'
      if Var_Expr_31 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1034 '<FieldAccess for node at nodes.lkt:1604:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1604'
--# expr-start 1033 '<FieldAccess for name at nodes.lkt:1604:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1604'
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Param.Node.Defining_Name_F_Name, Info => Param.Info);
--# expr-done 1033
Fld_3 := Fld_2.Node;
--# expr-done 1034
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Var_Expr_31, N => Fld_3);
--# end
--# expr-done 1035
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 1036
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Single_Actual then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 1038 '<FieldAccess for expr at nodes.lkt:1606:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1606'
--# expr-start 1037 '<FieldAccess for assoc at nodes.lkt:1606:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1606'
Fld_5 := Var_Expr_1.Assoc;
--# expr-done 1037
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 1038
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 1049 '<If at nodes.lkt:1608:25>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1040 '<IsNull at nodes.lkt:1608:28>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1039 '<at at nodes.lkt:1608:28>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
Get_Result_1 := Get (Self, Local_Up, Pos, True); 
--# expr-done 1039
Is_Equal := Get_Result_1 = No_Single_Actual; 
--# expr-done 1040
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1045 '<Not at nodes.lkt:1608:48>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1044 '<IsNull at nodes.lkt:1608:52>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1043 '<FieldAccess for names at nodes.lkt:1608:52>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1042 '<FieldAccess for assoc at nodes.lkt:1608:52>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
--# expr-start 1041 '<at at nodes.lkt:1608:52>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1608'
Get_Result_2 := Get (Self, Local_Up, Pos, True); 
--# expr-done 1041
Fld_9 := Get_Result_2.Assoc;
--# expr-done 1042
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Fld_9.Node);
--# end
--# expr-done 1043
Is_Equal_1 := Equivalent (Fld_10, No_Bare_Ada_Node_Array_Type); 
--# expr-done 1044
Not_Val := not (Is_Equal_1); 
--# expr-done 1045
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := Default_Expr;
else
   --# expr-start 1048 '<FieldAccess for expr at nodes.lkt:1614:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1614'
--# expr-start 1047 '<FieldAccess for assoc at nodes.lkt:1614:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1614'
--# expr-start 1046 '<at at nodes.lkt:1614:17>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1614'
Get_Result := Get (Self, Local_Up, Pos, True); 
--# expr-done 1046
Fld_7 := Get_Result.Assoc;
--# expr-done 1047
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1048
   If_Result_1 := Fld_8;
end if;
--# expr-done 1049
   Result_Var_1 := If_Result_1;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1050
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_160;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_160;
            raise;
      end;
   return Property_Result;
end Assoc_List_P_Actual_For_Param_At;
--# end
--# property-start AssocList.unpacked_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1623'
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Unpacked_Params
  (Node : Bare_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Actual_Array_Access
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_161 with Inline_Always;
            procedure Finalizer_Scope_2056 with Inline_Always;
      Local_Names : Bare_Ada_Node_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld : Bare_Assoc_List;
Fld_1 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Fld_2 : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Base_Id;
New_Struct : Internal_Single_Actual;
Array_Lit : Internal_Single_Actual_Array_Access;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Id;
New_Struct_1 : Internal_Single_Actual;
Is_A : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Len : Integer;
Is_Equal : Boolean;
If_Result : Internal_Single_Actual_Array_Access;
Let_Result : Internal_Single_Actual_Array_Access;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Map_Result_1 : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_161 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_161;
            procedure Finalizer_Scope_2056 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Local_Names);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2056;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Assoc_List_Assoc_List_P_Unpacked_Params,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Assoc_List,
                                As_Bare_Assoc_List => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Single_Actual_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1062 '<Mapcat at nodes.lkt:1623:50>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1623'
Saved := Ent; 
Fld := Saved.Node;
      declare
         Map_Result_1_Vec : Internal_Single_Actual_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result_1 := Create_Internal_Single_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_1 := Saved.Info;
As_Entity := (Info => Fld_1, Node => Bare_Item_0); 
   If_Result_1 := As_Entity;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind pa Item_0
      --# expr-start 1061 '<Let at nodes.lkt:1625:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1625'
--# scope-start
--# expr-start 1051 '<FieldAccess for names at nodes.lkt:1625:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1625'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_0.Node);
--# end
--# expr-done 1051
Local_Names := Fld_2; Inc_Ref (Local_Names);
--# bind names Local_Names
--# expr-start 1060 '<If at nodes.lkt:1627:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
--# expr-start 1053 '<Eq at nodes.lkt:1627:16>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
--# expr-start 1052 '<length at nodes.lkt:1627:16>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
Len := Length (Local_Names); 
--# expr-done 1052
Is_Equal := Len = 0; 
--# expr-done 1053
if Is_Equal then
   --# expr-start 1055 '<ArrayLiteral at nodes.lkt:1627:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
--# expr-start 1054 '<New SingleActual at nodes.lkt:1627:42>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
Cast_Expr := No_Bare_Ada_Node; 
      Cast_Result := Cast_Expr;
New_Struct := (Assoc => Item_0, Name => Cast_Result); 
--# expr-done 1054
Array_Lit := Create_Internal_Single_Actual_Array (Internal_Internal_Single_Actual_Array'(1 => New_Struct)); 
--# expr-done 1055
   If_Result := Array_Lit;
else
   --# expr-start 1059 '<Map at nodes.lkt:1627:94>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1627'
      declare
         Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Local_Names;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_1
         --# expr-start 1056 '<IsA BaseId at nodes.lkt:1628:74>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1628'
Is_A := Item_1 /= null 
and then Item_1.Kind in Ada_Base_Id; 
--# expr-done 1056
         if Is_A then
      --# expr-start 1058 '<New SingleActual at nodes.lkt:1628:24>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1628'
--# expr-start 1057 '<Cast to BaseId at nodes.lkt:1628:42>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1628'
Cast_Expr_1 := Item_1; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1057
New_Struct_1 := (Assoc => Item_0, Name => Cast_Result_1); 
--# expr-done 1058
         declare
            Item_To_Append : constant Internal_Single_Actual := New_Struct_1;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Single_Actual_Array
           (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1059
   If_Result := Map_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 1060
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1061
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Single_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2056;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Single_Actual_Array
           (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Single_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Single_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1062
         Property_Result := Map_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_161;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_161;
                     Finalizer_Scope_2056;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Single_Actual_Array_Access,
                        As_Internal_Single_Actual_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Assoc_List_P_Unpacked_Params;
--# end
--# property-start AssocList.zip_with_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1638'
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Zip_With_Params
  (Node : Bare_Assoc_List
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Assoc_List  := Bare_Assoc_List (Node);
     Ent : Internal_Entity_Assoc_List :=
       Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_162 with Inline_Always;
            procedure Finalizer_Scope_2059 with Inline_Always;
            procedure Finalizer_Scope_2060 with Inline_Always;
            procedure Finalizer_Scope_2061 with Inline_Always;
            procedure Finalizer_Scope_2062 with Inline_Always;
            procedure Finalizer_Scope_2063 with Inline_Always;
            procedure Finalizer_Scope_2064 with Inline_Always;
            procedure Finalizer_Scope_2065 with Inline_Always;
            procedure Finalizer_Scope_2066 with Inline_Always;
            procedure Finalizer_Scope_2067 with Inline_Always;
            procedure Finalizer_Scope_2069 with Inline_Always;
            procedure Finalizer_Scope_2071 with Inline_Always;
            procedure Finalizer_Scope_2073 with Inline_Always;
            procedure Finalizer_Scope_2079 with Inline_Always;
      Local_Is_Dottable_Subp : Boolean;
Var_Expr_0 : Internal_Entity_Name;
Local_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Generic_Instantiation;
Match_2 : Internal_Entity_Composite_Constraint;
Match_3 : Internal_Entity_Base_Aggregate;
Match_4 : Internal_Entity;
Local_Others_Assoc : Internal_Entity_Basic_Assoc;
Local_Explicit_Matches : Internal_Param_Actual_Array_Access;
Var_Expr_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Default_Subp_Matches : Internal_Param_Actual_Array_Access;
Var_Expr_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Decl : Internal_Entity_Generic_Formal_Subp_Decl;
Local_Subp : Internal_Entity_Formal_Subp_Decl;
Var_Expr_8 : Internal_Entity_Generic_Formal_Subp_Decl;
Var_Expr_9 : Internal_Entity_Formal_Subp_Decl;
Local_Given_Matches : Internal_Param_Actual_Array_Access;
Local_Others_Matches : Internal_Param_Actual_Array_Access;
Var_Expr_11 : Internal_Entity_Basic_Assoc;
Var_Expr_32 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_33 : Internal_Entity_Generic_Decl;
Var_Expr_34 : Internal_Entity_Base_Type_Decl;
Var_Expr_35 : Internal_Entity_Base_Record_Def;
Var_Expr_36 : Internal_Entity_Basic_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Generic_Decl;
Fld_6 : Internal_Entity_Generic_Formal_Part;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Bare_Assoc_List;
Cast_Result_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_10 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_Base_Record_Def;
Fld_13 : Internal_Entity_Component_List;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_3 : Internal_Entity_Generic_Instantiation;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_4 : Internal_Entity_Composite_Constraint;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_5 : Internal_Entity_Base_Aggregate;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_6 : Internal_Entity;
Let_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Item_2 : Internal_Entity_Basic_Assoc;
Bare_Item_2 : Bare_Basic_Assoc;
Fld_18 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Find_Result : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Param_Match_Array_Access;
Fld_20 : Internal_Entity_Defining_Name;
Fld_21 : Internal_Single_Actual;
Fld_22 : Internal_Entity_Basic_Assoc;
Fld_23 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result : Internal_Param_Actual_Array_Access;
Scope_Result_11 : Internal_Param_Actual_Array_Access;
Result_Var_6 : Internal_Param_Actual_Array_Access;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_7 : Internal_Entity_Generic_Formal_Subp_Decl;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_8 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_24 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Formal_Subp_Decl;
Fld_25 : Internal_Entity_Defining_Name;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Generic_Instantiation;
Fld_28 : Internal_Entity_Basic_Decl;
Fld_29 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Cast_Result_11 : Internal_Entity_Expr;
Fld_30 : Internal_Entity_Expr;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Let_Result_5 : Internal_Param_Actual;
Let_Result_6 : Internal_Param_Actual;
Cast_Expr_3 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_12 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_31 : Internal_Entity_Basic_Decl;
Cast_Result_13 : Internal_Entity_Formal_Subp_Decl;
Fld_32 : Internal_Entity_Expr;
Is_A_2 : Boolean;
Fld_33 : Internal_Entity_Defining_Name;
Fld_34 : Internal_Entity_Defining_Name;
Cast_Result_14 : Internal_Entity;
Cast_Result_15 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Scope_Result_13 : Boolean;
Result_Var_8 : Boolean;
Scope_Result_14 : Boolean;
Result_Var_9 : Boolean;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Scope_Result_15 : Internal_Param_Actual_Array_Access;
Result_Var_10 : Internal_Param_Actual_Array_Access;
Concat_Result : Internal_Param_Actual_Array_Access;
Fld_35 : Internal_Entity_Defining_Name_Array_Access;
Fld_36 : Internal_Entity_Expr;
New_Struct_2 : Internal_Param_Actual;
Fld_37 : Internal_Entity_Defining_Name;
Cast_Result_16 : Internal_Entity;
Cast_Expr_4 : Internal_Entity_Defining_Name;
Cast_Result_17 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_2 : Boolean;
Not_Val_1 : Boolean;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_16 : Internal_Param_Actual_Array_Access;
Result_Var_11 : Internal_Param_Actual_Array_Access;
Concat_Result_1 : Internal_Param_Actual_Array_Access;
Let_Result_7 : Internal_Param_Actual_Array_Access;
Let_Result_8 : Internal_Param_Actual_Array_Access;
Let_Result_9 : Internal_Param_Actual_Array_Access;
Let_Result_10 : Internal_Param_Actual_Array_Access;
Let_Result_11 : Internal_Param_Actual_Array_Access;
Let_Result_12 : Internal_Param_Actual_Array_Access;
Let_Result_13 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_162 is
            begin
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Local_Params);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Local_Explicit_Matches);
                     Dec_Ref (Scope_Result_15);
                     Dec_Ref (Result_Var_10);
                     Dec_Ref (Local_Default_Subp_Matches);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Local_Given_Matches);
                     Dec_Ref (Scope_Result_16);
                     Dec_Ref (Result_Var_11);
                     Dec_Ref (Local_Others_Matches);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Let_Result_8);
                     Dec_Ref (Let_Result_9);
                     Dec_Ref (Let_Result_10);
                     Dec_Ref (Let_Result_11);
                     Dec_Ref (Let_Result_12);
                     Dec_Ref (Let_Result_13);
            end Finalizer_Scope_162;
            procedure Finalizer_Scope_2059 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2059;
            procedure Finalizer_Scope_2060 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2060;
            procedure Finalizer_Scope_2061 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2061;
            procedure Finalizer_Scope_2062 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2062;
            procedure Finalizer_Scope_2063 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2063;
            procedure Finalizer_Scope_2064 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_2064;
            procedure Finalizer_Scope_2065 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2065;
            procedure Finalizer_Scope_2066 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_2066;
            procedure Finalizer_Scope_2067 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2067;
            procedure Finalizer_Scope_2069 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_2069;
            procedure Finalizer_Scope_2071 is
            begin
                     Dec_Ref (Var_Expr_4);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2071;
            procedure Finalizer_Scope_2073 is
            begin
                     Dec_Ref (Var_Expr_6);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2073;
            procedure Finalizer_Scope_2079 is
            begin
                     Dec_Ref (Fld_35);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2079;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1136 '<Let at nodes.lkt:1641:9>' Let_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1641'
--# scope-start
--# expr-start 1066 '<bind at nodes.lkt:1642:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1642'
Bound_Imprecise_Fallback := False; 
--# expr-start 1064 '<Cast to Name at nodes.lkt:1644:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1644'
--# expr-start 1063 '<parent at nodes.lkt:1644:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1644'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1063
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 1064
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 1065 '<FieldAccess for is_dot_call at nodes.lkt:1644:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1644'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Var_Expr_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1065
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 1066
Local_Is_Dottable_Subp := Dyn_Var_Bind_Result; 
--# bind is_dottable_subp Local_Is_Dottable_Subp
--# expr-start 1135 '<Let at nodes.lkt:1646:9>' Let_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1646'
--# scope-start
--# expr-start 1067 '<parent at nodes.lkt:1646:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1646'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1067
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 1083 '<Match at nodes.lkt:1647:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1647'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 1068 '<FieldAccess for called_subp_spec at nodes.lkt:1648:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1648'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.called_subp_spec
Fld_3 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 1068
Var_Expr_32 := Fld_3;
if Var_Expr_32 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 1069 '<FieldAccess for abstract_formal_params at nodes.lkt:1648:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1648'
      if Var_Expr_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_32.Node, E_Info => Var_Expr_32.Info);
--# end
--# expr-done 1069
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2061;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2060;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Generic_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind i Match_1
--# expr-start 1070 '<FieldAccess for designated_generic_decl at nodes.lkt:1649:49>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1649'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 1070
Var_Expr_33 := Fld_5;
if Var_Expr_33 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 1072 '<FieldAccess for abstract_formal_params at nodes.lkt:1649:49>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1649'
--# expr-start 1071 '<FieldAccess for formal_part at nodes.lkt:1649:49>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1649'
      if Var_Expr_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_33.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_33.Info);
--# expr-done 1071
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1072
Scope_Result_2 := Fld_7;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2063;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2062;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Composite_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Composite_Constraint
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind c Match_2
--# expr-start 1073 '<FieldAccess for subtype at nodes.lkt:1650:48>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1650'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_8 := Libadalang.Implementation.Constraint_P_Subtype (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 1073
Var_Expr_34 := Fld_8;
if Var_Expr_34 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1074 '<FieldAccess for discriminants_list at nodes.lkt:1650:48>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1650'
      if Var_Expr_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_34.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_34.Info);
--# end
--# expr-done 1074
Scope_Result_3 := Fld_9;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2065;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2064;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Base_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind a Match_3
--# expr-start 1082 '<bind at nodes.lkt:1652:21>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1652'
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Bound_Origin := Cast_Result_1; 
--# expr-start 1081 '<bind at nodes.lkt:1653:21>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1653'
--# expr-start 1075 '<FieldAccess for node_env at nodes.lkt:1653:32>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1653'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 1075
Bound_Env := Fld_10; Inc_Ref (Bound_Env);
--# expr-start 1077 '<FieldAccess for record_def at nodes.lkt:1655:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1655'
--# expr-start 1076 '<FieldAccess for expression_type at nodes.lkt:1655:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1655'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_11 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 1076
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_11.Node, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 1077
Var_Expr_35 := Fld_12;
if Var_Expr_35 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 1080 '<FieldAccess for abstract_formal_params_for_assocs at nodes.lkt:1655:21>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1655'
--# expr-start 1078 '<FieldAccess for components at nodes.lkt:1655:21>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1655'
      if Var_Expr_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Component_List (Node => Var_Expr_35.Node.Base_Record_Def_F_Components, Info => Var_Expr_35.Info);
--# expr-done 1078
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1079 '<FieldAccess for ancestor_expr_type at nodes.lkt:1658:41>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1658'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_14 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 1079
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_15 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Fld_13.Node, Assocs => Ent, Stop_Recurse_At => Fld_14, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 1080
Scope_Result_4 := Fld_15;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2067;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Dyn_Var_Bind_Result_1 := Result_Var_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 1081
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 1082
Let_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Let_Result_3);
--# end
Scope_Result_8 := Let_Result_3;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2066;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Generic_Subp_Decl | Ada_Generic_Package_Renaming_Decl .. Ada_Constant_Present | Ada_Delta_Constraint .. Ada_Allocator | Ada_Bin_Op .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_6; 
--# bind _ Match_4
Let_Result_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result_4);
--# end
Scope_Result_9 := Let_Result_4;
   Inc_Ref (Scope_Result_9);
   --# end
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 1083
Scope_Result_10 := Match_Result;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_2059;
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Local_Params := Result_Var_5; Inc_Ref (Local_Params);
--# bind params Local_Params
--# expr-start 1134 '<Let at nodes.lkt:1664:9>' Let_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1664'
--# scope-start
--# expr-start 1087 '<Find at nodes.lkt:1664:28>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1664'
Saved := Ent; 
Fld_16 := Saved.Node;
      Find_Result := No_Entity_Basic_Assoc;
      if Fld_16 = null then
      Find_Result := No_Entity_Basic_Assoc;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_16;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null := Bare_Item_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_17 := Saved.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_2); 
   If_Result := As_Entity;
end if;
                  Item_2 := If_Result; 
   --# scope-start
               --# bind assoc Item_2
      --# expr-start 1086 '<AnyQuantifier at nodes.lkt:1665:24>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1665'
   --# expr-start 1084 '<FieldAccess for names at nodes.lkt:1665:24>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1665'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_2.Node);
--# end
--# expr-done 1084
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_18;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_3
      --# expr-start 1085 '<IsA OthersDesignator at nodes.lkt:1665:49>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1665'
Is_A := Item_3 /= null 
and then Item_3.Kind in Ada_Others_Designator_Range; 
--# expr-done 1085
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1086
      if Quantifier_Result then
         Find_Result := Item_2; 
      end if;
   --# end
      Finalizer_Scope_2069;
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 1087
Local_Others_Assoc := Find_Result; 
--# bind others_assoc Local_Others_Assoc
--# expr-start 1133 '<Let at nodes.lkt:1667:9>' Let_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1667'
--# scope-start
Var_Expr_4 := Local_Params;
      Inc_Ref (Var_Expr_4);
if Var_Expr_4 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_4
--# expr-start 1094 '<Map at nodes.lkt:1668:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1668'
   --# expr-start 1088 '<FieldAccess for match_formals at nodes.lkt:1668:20>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1668'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.match_formals
Fld_19 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Local_Params, Params => Ent, Is_Dottable_Subp => Local_Is_Dottable_Subp);
--# end
--# expr-done 1088
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_19;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_5
      --# expr-start 1093 '<New ParamActual at nodes.lkt:1669:24>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1669'
--# expr-start 1091 '<FieldAccess for expr at nodes.lkt:1670:44>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1670'
--# expr-start 1090 '<FieldAccess for assoc at nodes.lkt:1670:44>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1670'
--# expr-start 1089 '<FieldAccess for actual at nodes.lkt:1670:44>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1670'
Fld_21 := Item_5.Actual;
--# expr-done 1089
Fld_22 := Fld_21.Assoc;
--# expr-done 1090
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_23 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_22.Node, E_Info => Fld_22.Info);
--# end
--# expr-done 1091
--# expr-start 1092 '<FieldAccess for formal at nodes.lkt:1670:27>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1670'
Fld_20 := Item_5.Formal;
--# expr-done 1092
New_Struct := (Actual => Fld_23, Param => Fld_20); 
--# expr-done 1093
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1094
Scope_Result_11 := Map_Result;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_2071;
   Result_Var_6 := Scope_Result_11;
else
   Result_Var_6 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Local_Explicit_Matches := Result_Var_6; Inc_Ref (Local_Explicit_Matches);
--# bind explicit_matches Local_Explicit_Matches
--# expr-start 1132 '<Let at nodes.lkt:1674:9>' Let_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1674'
--# scope-start
Var_Expr_6 := Local_Params;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_6
--# expr-start 1121 '<Map at nodes.lkt:1675:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1675'
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Local_Params;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_7
--# expr-start 1095 '<Cast to GenericFormalSubpDecl at nodes.lkt:1687:27>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1687'
Cast_Expr_3 := Item_7; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_12 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_12 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 1095
Var_Expr_8 := Cast_Result_12;
if Var_Expr_8 /= No_Entity_Generic_Formal_Subp_Decl then
   --# scope-start
   --# bind fd Var_Expr_8
--# expr-start 1097 '<Cast to FormalSubpDecl at nodes.lkt:1688:29>' Cast_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1688'
--# expr-start 1096 '<FieldAccess for decl at nodes.lkt:1688:29>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1688'
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Basic_Decl (Node => Var_Expr_8.Node.Generic_Formal_F_Decl, Info => Var_Expr_8.Info);
--# expr-done 1096
   if Fld_31.Node = null
      or else Fld_31.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_13 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
   else
         Cast_Result_13 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 1097
Var_Expr_9 := Cast_Result_13;
if Var_Expr_9 /= No_Entity_Formal_Subp_Decl then
   --# scope-start
   --# bind subp Var_Expr_9
--# expr-start 1099 '<IsA BoxExpr, Name at nodes.lkt:1692:29>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1692'
--# expr-start 1098 '<FieldAccess for default_expr at nodes.lkt:1692:29>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1692'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Expr (Node => Var_Expr_9.Node.Formal_Subp_Decl_F_Default_Expr, Info => Var_Expr_9.Info);
--# expr-done 1098
Is_A_2 := Fld_32.Node /= null 
and then Fld_32.Node.Kind in Ada_Box_Expr_Range | Ada_Name; 
--# expr-done 1099
if Is_A_2 then
   --# expr-start 1104 '<Not at nodes.lkt:1695:29>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1695'
--# expr-start 1103 '<AnyQuantifier at nodes.lkt:1695:33>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1695'
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Local_Explicit_Matches;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_10
      --# expr-start 1102 '<Eq at nodes.lkt:1695:61>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1695'
--# expr-start 1100 '<FieldAccess for param at nodes.lkt:1695:61>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1695'
Fld_33 := Item_10.Param;
--# expr-done 1100
      Cast_Result_14 := Create_Internal_Entity
        (Node => Fld_33.Node,
         Info => Fld_33.Info);
--# expr-start 1101 '<FieldAccess for defining_name at nodes.lkt:1695:72>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1695'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_34 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 1101
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
Is_Equiv := Equivalent (Cast_Result_14, Cast_Result_15); 
--# expr-done 1102
      Quantifier_Result_1 := Is_Equiv;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 1103
Not_Val := not (Quantifier_Result_1); 
--# expr-done 1104
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
Scope_Result_13 := If_Result_2;
   --# end
   Result_Var_8 := Scope_Result_13;
else
   Result_Var_8 := False;
end if;
Scope_Result_14 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_14;
else
   Result_Var_9 := False;
end if;
         if Result_Var_9 then
      --# expr-start 1120 '<Let at nodes.lkt:1679:21>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1679'
--# scope-start
--# expr-start 1105 '<Cast to GenericFormalSubpDecl at nodes.lkt:1679:32>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1679'
Cast_Expr_1 := Item_7; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_7 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 1105
Local_Decl := Cast_Result_7; 
--# bind decl Local_Decl
--# expr-start 1119 '<Let at nodes.lkt:1680:21>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1680'
--# scope-start
--# expr-start 1108 '<Cast to FormalSubpDecl at nodes.lkt:1680:32>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1680'
--# expr-start 1107 '<FieldAccess for decl at nodes.lkt:1680:32>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1680'
--# expr-start 1106 '<Cast to GenericFormalSubpDecl at nodes.lkt:1680:32>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1680'
Cast_Expr_2 := Item_7; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_8 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 1106
      if Cast_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_8.Node.Generic_Formal_F_Decl, Info => Cast_Result_8.Info);
--# expr-done 1107
   if Fld_24.Node = null
      or else Fld_24.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_9 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
   else
         Cast_Result_9 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 1108
Local_Subp := Cast_Result_9; 
--# bind subp Local_Subp
--# expr-start 1118 '<New ParamActual at nodes.lkt:1682:21>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1682'
--# expr-start 1116 '<If at nodes.lkt:1683:60>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
--# expr-start 1110 '<IsA Name at nodes.lkt:1683:63>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
--# expr-start 1109 '<FieldAccess for default_expr at nodes.lkt:1683:63>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Expr (Node => Local_Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Local_Subp.Info);
--# expr-done 1109
Is_A_1 := Fld_30.Node /= null 
and then Fld_30.Node.Kind in Ada_Name; 
--# expr-done 1110
if Is_A_1 then
   --# expr-start 1111 '<FieldAccess for default_expr at nodes.lkt:1683:94>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Expr (Node => Local_Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Local_Subp.Info);
--# expr-done 1111
   If_Result_1 := Fld_26;
else
--# expr-start 1114 '<FieldAccess for designated_subprogram_from at nodes.lkt:1683:117>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1113 '<Cast to GenericInstantiation at nodes.lkt:1684:34>' Cast_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1684'
--# expr-start 1112 '<parent at nodes.lkt:1684:34>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1684'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1112
   if Fld_27.Node = null
      or else Fld_27.Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result_10 := Create_Internal_Entity_Generic_Instantiation
        (Node => Fld_27.Node,
         Info => Fld_27.Info);
   else
         Cast_Result_10 := No_Entity_Generic_Instantiation;
   end if;
--# expr-done 1113
--# property-call-start FormalSubpDecl.designated_subprogram_from
Fld_28 := Libadalang.Implementation.Formal_Subp_Decl_P_Designated_Subprogram_From (Node => Local_Subp.Node, Inst => Cast_Result_10, E_Info => Local_Subp.Info);
--# end
--# expr-done 1114
Var_Expr_36 := Fld_28;
if Var_Expr_36 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1115 '<FieldAccess for defining_name at nodes.lkt:1683:117>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
      if Var_Expr_36.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_29 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_36.Node, E_Info => Var_Expr_36.Info);
--# end
--# expr-done 1115
Scope_Result_12 := Fld_29;
   --# end
   Result_Var_7 := Scope_Result_12;
else
   Result_Var_7 := No_Entity_Defining_Name;
end if;
      Cast_Result_11 := Create_Internal_Entity_Expr
        (Node => Result_Var_7.Node,
         Info => Result_Var_7.Info);
   If_Result_1 := Cast_Result_11;
end if;
--# expr-done 1116
--# expr-start 1117 '<FieldAccess for defining_name at nodes.lkt:1683:31>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1683'
      if Local_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Decl.Node, E_Info => Local_Decl.Info);
--# end
--# expr-done 1117
New_Struct_1 := (Actual => If_Result_1, Param => Fld_25); 
--# expr-done 1118
Let_Result_5 := New_Struct_1; 
--# end
--# expr-done 1119
Let_Result_6 := Let_Result_5; 
--# end
--# expr-done 1120
         declare
            Item_To_Append : constant Internal_Param_Actual := Let_Result_6;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1121
Scope_Result_15 := Map_Result_1;
   Inc_Ref (Scope_Result_15);
   --# end
      Finalizer_Scope_2073;
   Result_Var_10 := Scope_Result_15;
else
   Result_Var_10 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_10);
Local_Default_Subp_Matches := Result_Var_10; Inc_Ref (Local_Default_Subp_Matches);
--# bind default_subp_matches Local_Default_Subp_Matches
--# expr-start 1131 '<Let at nodes.lkt:1701:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1701'
--# scope-start
Concat_Result := Concat (Local_Explicit_Matches, Local_Default_Subp_Matches); 
Local_Given_Matches := Concat_Result; Inc_Ref (Local_Given_Matches);
--# bind given_matches Local_Given_Matches
--# expr-start 1130 '<Let at nodes.lkt:1702:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1702'
--# scope-start
Var_Expr_11 := Local_Others_Assoc;
if Var_Expr_11 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind oa Var_Expr_11
--# expr-start 1129 '<Map at nodes.lkt:1703:21>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1703'
   --# expr-start 1122 '<FieldAccess for unpack_formals at nodes.lkt:1703:21>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1703'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.unpack_formals
Fld_35 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Local_Params);
--# end
--# expr-done 1122
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_35;
      begin
         for Item_12 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_12
         --# expr-start 1126 '<Not at nodes.lkt:1704:71>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
--# expr-start 1125 '<AnyQuantifier at nodes.lkt:1704:75>' Quantifier_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
      Quantifier_Result_2 := False;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Local_Given_Matches;
      begin
         for Item_13 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_13
      --# expr-start 1124 '<Eq at nodes.lkt:1704:100>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
--# expr-start 1123 '<FieldAccess for param at nodes.lkt:1704:100>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
Fld_37 := Item_13.Param;
--# expr-done 1123
      Cast_Result_16 := Create_Internal_Entity
        (Node => Fld_37.Node,
         Info => Fld_37.Info);
Cast_Expr_4 := Item_12; 
      Cast_Result_17 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_16, Cast_Result_17); 
--# expr-done 1124
      Quantifier_Result_2 := Is_Equiv_1;
   --# end
               exit when Quantifier_Result_2;
         end loop;
      end;
--# expr-done 1125
Not_Val_1 := not (Quantifier_Result_2); 
--# expr-done 1126
         if Not_Val_1 then
      --# expr-start 1128 '<New ParamActual at nodes.lkt:1704:24>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
--# expr-start 1127 '<FieldAccess for expr at nodes.lkt:1704:52>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1704'
      if Var_Expr_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 1127
New_Struct_2 := (Actual => Fld_36, Param => Item_12); 
--# expr-done 1128
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_2;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1129
Scope_Result_16 := Map_Result_2;
   Inc_Ref (Scope_Result_16);
   --# end
      Finalizer_Scope_2079;
   Result_Var_11 := Scope_Result_16;
else
   Result_Var_11 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_11);
Local_Others_Matches := Result_Var_11; Inc_Ref (Local_Others_Matches);
--# bind others_matches Local_Others_Matches
Concat_Result_1 := Concat (Local_Given_Matches, Local_Others_Matches); 
Let_Result_7 := Concat_Result_1; Inc_Ref (Let_Result_7);
--# end
--# expr-done 1130
Let_Result_8 := Let_Result_7; Inc_Ref (Let_Result_8);
--# end
--# expr-done 1131
Let_Result_9 := Let_Result_8; Inc_Ref (Let_Result_9);
--# end
--# expr-done 1132
Let_Result_10 := Let_Result_9; Inc_Ref (Let_Result_10);
--# end
--# expr-done 1133
Let_Result_11 := Let_Result_10; Inc_Ref (Let_Result_11);
--# end
--# expr-done 1134
Let_Result_12 := Let_Result_11; Inc_Ref (Let_Result_12);
--# end
--# expr-done 1135
Let_Result_13 := Let_Result_12; Inc_Ref (Let_Result_13);
--# end
--# expr-done 1136
         Property_Result := Let_Result_13;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_162;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_162;
                     Finalizer_Scope_2059;
                     Finalizer_Scope_2060;
                     Finalizer_Scope_2061;
                     Finalizer_Scope_2062;
                     Finalizer_Scope_2063;
                     Finalizer_Scope_2064;
                     Finalizer_Scope_2065;
                     Finalizer_Scope_2066;
                     Finalizer_Scope_2067;
                     Finalizer_Scope_2069;
                     Finalizer_Scope_2071;
                     Finalizer_Scope_2073;
                     Finalizer_Scope_2079;
            raise;
      end;
   return Property_Result;
end Assoc_List_P_Zip_With_Params;
--# end
   --
   --  Primitives for Bare_Case_Expr_Alternative_List
   --
   --
   --  Primitives for Bare_Case_Stmt_Alternative_List
   --
   --
   --  Primitives for Bare_Compilation_Unit_List
   --
   --
   --  Primitives for Bare_Concat_Operand_List
   --
   --
   --  Primitives for Bare_Contract_Case_Assoc_List
   --
   --
   --  Primitives for Bare_Defining_Name_List
   --
   --
   --  Primitives for Bare_Discriminant_Spec_List
   --
   --
   --  Primitives for Bare_Elsif_Expr_Part_List
   --
   --
   --  Primitives for Bare_Elsif_Stmt_Part_List
   --
   --
   --  Primitives for Bare_Enum_Literal_Decl_List
   --
   --
   --  Primitives for Bare_Expr_List
   --
   --
   --  Primitives for Bare_Expr_Alternatives_List
   --
   --
   --  Primitives for Bare_Format_String_Chunk_List
   --
   --
   --  Primitives for Bare_Identifier_List
   --
   --
   --  Primitives for Bare_Discriminant_Choice_List
   --
   --
   --  Primitives for Bare_Name_List
   --
   --
   --  Primitives for Bare_Parent_List
   --
   --
   --  Primitives for Bare_Param_Spec_List
   --
   --
   --  Primitives for Bare_Pragma_Node_List
   --
   --
   --  Primitives for Bare_Select_When_Part_List
   --
   --
   --  Primitives for Bare_Unconstrained_Array_Index_List
   --
   --
   --  Primitives for Bare_Variant_List
   --
   --
   --  Primitives for Bare_Aliased_Node
   --
--# property-start '[dispatcher]Aliased.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Aliased_Node_P_As_Bool
  (Node : Bare_Aliased_Node
  )
   return Boolean
is
   Self : Bare_Aliased_Node  := Bare_Aliased_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Aliased_Node (Self.Kind) is
               when Ada_Aliased_Absent =>
                  --# property-call-start Aliased.Absent.as_bool
                  Property_Result := Aliased_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Aliased_Present =>
                  --# property-call-start Aliased.Present.as_bool
                  Property_Result := Aliased_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Aliased_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Absent
   --
--# property-start Aliased.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1726'
pragma Warnings (Off, "is not referenced");
 function Aliased_Absent_P_As_Bool
  (Node : Bare_Aliased_Absent
  )
   return Boolean
is
   Self : Bare_Aliased_Absent  := Bare_Aliased_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aliased_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Present
   --
--# property-start Aliased.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1726'
pragma Warnings (Off, "is not referenced");
 function Aliased_Present_P_As_Bool
  (Node : Bare_Aliased_Present
  )
   return Boolean
is
   Self : Bare_Aliased_Present  := Bare_Aliased_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aliased_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Node
   --
--# property-start '[dispatcher]All.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_All_Node_P_As_Bool
  (Node : Bare_All_Node
  )
   return Boolean
is
   Self : Bare_All_Node  := Bare_All_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_All_Node (Self.Kind) is
               when Ada_All_Absent =>
                  --# property-call-start All.Absent.as_bool
                  Property_Result := All_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_All_Present =>
                  --# property-call-start All.Present.as_bool
                  Property_Result := All_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_All_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Absent
   --
--# property-start All.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1731'
pragma Warnings (Off, "is not referenced");
 function All_Absent_P_As_Bool
  (Node : Bare_All_Absent
  )
   return Boolean
is
   Self : Bare_All_Absent  := Bare_All_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end All_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Present
   --
--# property-start All.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1731'
pragma Warnings (Off, "is not referenced");
 function All_Present_P_As_Bool
  (Node : Bare_All_Present
  )
   return Boolean
is
   Self : Bare_All_Present  := Bare_All_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end All_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Array_Indices
   --
--# property-start '[dispatcher]ArrayIndices.ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Ndims
  (Node : Bare_Array_Indices
  )
   return Integer
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.ndims
                  Property_Result := Constrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.ndims
                  Property_Result := Unconstrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Ndims;
--# end
--# property-start '[dispatcher]ArrayIndices.constrain_index_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.constrain_index_expr
                  Property_Result := Constrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.constrain_index_expr
                  Property_Result := Unconstrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start '[dispatcher]ArrayIndices.index_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Index_Type
  (Node : Bare_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.index_type
                  Property_Result := Constrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.index_type
                  Property_Result := Unconstrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Index_Type;
--# end
--# property-start '[dispatcher]ArrayIndices.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Is_Static
  (Node : Bare_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Indices  := Bare_Array_Indices (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.is_static
                  Property_Result := Constrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.is_static
                  Property_Result := Unconstrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Constrained_Array_Indices
   --
      procedure Initialize_Fields_For_Constrained_Array_Indices
        (Self : Bare_Constrained_Array_Indices
         ; Constrained_Array_Indices_F_List : Bare_Constraint_List
        ) is
      begin
            Self.Constrained_Array_Indices_F_List := Constrained_Array_Indices_F_List;
      end Initialize_Fields_For_Constrained_Array_Indices;
   function Constrained_Array_Indices_F_List
     (Node : Bare_Constrained_Array_Indices) return Bare_Constraint_List
   is
   begin
         return Node.Constrained_Array_Indices_F_List;
   end;
--# property-start ConstrainedArrayIndices.ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1771'
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Ndims
  (Node : Bare_Constrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Constraint_List;
Len : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1138 '<length at nodes.lkt:1771:24>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1771'
--# expr-start 1137 '<FieldAccess for list at nodes.lkt:1771:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1771'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Constrained_Array_Indices_F_List;
--# expr-done 1137
Len := Length (Fld); 
--# expr-done 1138
         Property_Result := Len;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Ndims;
--# end
--# property-start ConstrainedArrayIndices.constrain_index_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1774'
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Constrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_288 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_288 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_288;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1143 '<And at nodes.lkt:1775:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1775'
--# expr-start 1141 '<Bind at nodes.lkt:1775:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1775'
--# expr-start 1139 '<FieldAccess for expected_type_var at nodes.lkt:1775:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1775'
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Index_Expr.Node);
--# expr-done 1139
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 1140 '<FieldAccess for index_type at nodes.lkt:1775:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1775'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1140
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1775:9") else null)); 
--# expr-done 1141
--# expr-start 1142 '<FieldAccess for matches_expected_type at nodes.lkt:1775:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1775'
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Index_Expr.Node, Origin => Origin);
--# end
--# expr-done 1142
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1775:9") else null)); 
--# expr-done 1143
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_288;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_288;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start ConstrainedArrayIndices.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1778'
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Constrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_289 with Inline_Always;
            procedure Finalizer_Scope_2083 with Inline_Always;
            procedure Finalizer_Scope_2084 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_3 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Pred_2 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_289 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_289;
            procedure Finalizer_Scope_2083 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2083;
            procedure Finalizer_Scope_2084 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred_2);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2084;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1157 '<LogicAll at nodes.lkt:1778:37>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1778'
--# expr-start 1156 '<Map at nodes.lkt:1778:37>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1778'
   --# expr-start 1144 '<FieldAccess for list at nodes.lkt:1778:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1144
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind index Item_0
      --# expr-start 1155 '<And at nodes.lkt:1779:20>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1779'
--# expr-start 1145 '<FieldAccess for sub_equation at nodes.lkt:1779:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1779'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 1145
--# expr-start 1146 '<Cast to Expr at nodes.lkt:1779:45>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1779'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 1146
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_1
--# expr-start 1154 '<And at nodes.lkt:1780:23>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1780'
--# expr-start 1151 '<And at nodes.lkt:1781:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1781'
--# expr-start 1148 '<Predicate on AdaNode.is_not_null at nodes.lkt:1781:17>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1781'
--# expr-start 1147 '<FieldAccess for type_var at nodes.lkt:1781:49>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1781'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 1147
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 1148
--# expr-start 1150 '<Predicate on BaseTypeDecl.is_discrete_type at nodes.lkt:1781:70>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1781'
--# expr-start 1149 '<FieldAccess for type_var at nodes.lkt:1781:112>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1781'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 1149
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred_1 := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 1150
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1781:17") else null)); 
--# expr-done 1151
--# expr-start 1153 '<Predicate on BaseTypeDecl.is_not_root_int_type at nodes.lkt:1782:19>' Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1782'
--# expr-start 1152 '<FieldAccess for type_var at nodes.lkt:1782:65>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1782'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 1152
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Pred_2 := Solver.Create_Predicate (Fld_6, Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate); 
--# expr-done 1153
And_Pred_1 := Create_And (And_Pred, Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1780:23") else null)); 
--# expr-done 1154
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2084;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1782:95") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1779:20") else null)); 
--# expr-done 1155
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2083;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1156
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1778:37") else null)); 
--# expr-done 1157
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_289;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_289;
                     Finalizer_Scope_2083;
                     Finalizer_Scope_2084;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start ConstrainedArrayIndices.index_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1787'
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Index_Type
  (Node : Bare_Constrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_290 with Inline_Always;
      Ignored : Boolean;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Node_Parents : Bare_Ada_Node_Array_Access;
Fld : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Constraint_List;
Fld_3 : Bare_Constraint_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_290 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_290;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1168 '<Let at nodes.lkt:1789:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
--# scope-start
--# expr-start 1162 '<FieldAccess for resolve_names at nodes.lkt:1789:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
--# expr-start 1161 '<as_entity at nodes.lkt:1789:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
--# expr-start 1160 '<Find at nodes.lkt:1789:17>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
   --# expr-start 1158 '<FieldAccess for parents at nodes.lkt:1789:17>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 1158
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 1159 '<FieldAccess for xref_entry_point at nodes.lkt:1789:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1789'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Item_0);
--# end
--# expr-done 1159
      if Fld then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld;
         end loop;
      end;
--# expr-done 1160
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1161
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1162
Ignored := Fld_1; 
--# bind _ Ignored
--# expr-start 1163 '<FieldAccess for list at nodes.lkt:1791:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1791'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1163
Fld_3 := Fld_2.Node;
Get_Result := Get (Self, Fld_3, Dim, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_4 := Fld_2.Info;
As_Entity_1 := (Info => Fld_4, Node => Get_Result); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_1 := If_Result_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 1167 '<Match at nodes.lkt:1792:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1792'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind st Match_0
--# expr-start 1164 '<FieldAccess for designated_type at nodes.lkt:1793:47>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1793'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1164
Let_Result := Fld_5; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 1166 '<Cast to BaseTypeDecl at nodes.lkt:1794:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1794'
--# expr-start 1165 '<FieldAccess for type_val at nodes.lkt:1794:33>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1794'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_6 := Libadalang.Implementation.Expr_P_Type_Val (Node => Match_1.Node);
--# end
--# expr-done 1165
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1166
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1167
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 1168
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_290;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_290;
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Index_Type;
--# end
--# property-start ConstrainedArrayIndices.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1801'
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Is_Static
  (Node : Bare_Constrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constrained_Array_Indices  := Bare_Constrained_Array_Indices (Node);
     Ent : Internal_Entity_Constrained_Array_Indices :=
       Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
If_Result : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1176 '<AllQuantifier at nodes.lkt:1801:29>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1801'
   --# expr-start 1169 '<FieldAccess for list at nodes.lkt:1801:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1801'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 1169
Fld_1 := Fld.Node;
      Quantifier_Result := True;
      if Fld_1 = null then
      null;
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result_1 := As_Entity;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind t Item_0
      --# expr-start 1175 '<Match at nodes.lkt:1802:16>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1802'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 1170 '<FieldAccess for is_static_subtype at nodes.lkt:1803:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1803'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_3 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 1170
Let_Result := Fld_3; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind e Match_1
--# expr-start 1172 '<FieldAccess for is_static_expr at nodes.lkt:1804:30>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1804'
--# expr-start 1171 '<FieldAccess for left at nodes.lkt:1804:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1804'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 1171
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_5 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 1172
if Fld_5 then
   --# expr-start 1174 '<FieldAccess for is_static_expr at nodes.lkt:1804:58>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1804'
--# expr-start 1173 '<FieldAccess for right at nodes.lkt:1804:58>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1804'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 1173
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 1174
   If_Result := Fld_7;
else
   If_Result := False;
end if;
Let_Result_1 := If_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1175
      Quantifier_Result := Match_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 1176
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Indices
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Indices
        (Self : Bare_Unconstrained_Array_Indices
         ; Unconstrained_Array_Indices_F_Types : Bare_Unconstrained_Array_Index_List
        ) is
      begin
            Self.Unconstrained_Array_Indices_F_Types := Unconstrained_Array_Indices_F_Types;
      end Initialize_Fields_For_Unconstrained_Array_Indices;
   function Unconstrained_Array_Indices_F_Types
     (Node : Bare_Unconstrained_Array_Indices) return Bare_Unconstrained_Array_Index_List
   is
   begin
         return Node.Unconstrained_Array_Indices_F_Types;
   end;
--# property-start UnconstrainedArrayIndices.ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1814'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Ndims
  (Node : Bare_Unconstrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
      --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Unconstrained_Array_Index_List;
Len : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1178 '<length at nodes.lkt:1814:24>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1814'
--# expr-start 1177 '<FieldAccess for types at nodes.lkt:1814:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1814'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Unconstrained_Array_Indices_F_Types;
--# expr-done 1177
Len := Length (Fld); 
--# expr-done 1178
         Property_Result := Len;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Ndims;
--# end
--# property-start UnconstrainedArrayIndices.constrain_index_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1817'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Unconstrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_293 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_293 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_293;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1183 '<And at nodes.lkt:1818:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1818'
--# expr-start 1181 '<Bind at nodes.lkt:1818:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1818'
--# expr-start 1179 '<FieldAccess for expected_type_var at nodes.lkt:1818:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1818'
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Index_Expr.Node);
--# expr-done 1179
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 1180 '<FieldAccess for index_type at nodes.lkt:1818:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1818'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1180
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1818:9") else null)); 
--# expr-done 1181
--# expr-start 1182 '<FieldAccess for matches_expected_type at nodes.lkt:1818:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1818'
      if Index_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Index_Expr.Node, Origin => Origin);
--# end
--# expr-done 1182
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1818:9") else null)); 
--# expr-done 1183
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_293;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_293;
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start UnconstrainedArrayIndices.index_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1821'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Index_Type
  (Node : Bare_Unconstrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_37 : Internal_Entity_Unconstrained_Array_Index;
Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Unconstrained_Array_Index;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1184 '<FieldAccess for types at nodes.lkt:1822:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1822'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1184
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, Dim, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_37 := If_Result;
if Var_Expr_37 /= No_Entity_Unconstrained_Array_Index then
   --# scope-start
--# expr-start 1185 '<FieldAccess for designated_type at nodes.lkt:1822:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1822'
      if Var_Expr_37.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnconstrainedArrayIndex.designated_type
Fld_3 := Libadalang.Implementation.Unconstrained_Array_Index_P_Designated_Type (Node => Var_Expr_37.Node, Origin => Origin, E_Info => Var_Expr_37.Info);
--# end
--# expr-done 1185
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Index_Type;
--# end
--# property-start UnconstrainedArrayIndices.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1825'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Unconstrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_295 with Inline_Always;
            procedure Finalizer_Scope_2095 with Inline_Always;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_0 : Bare_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Unconstrained_Array_Index;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_295 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_295;
            procedure Finalizer_Scope_2095 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2095;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1196 '<LogicAll at nodes.lkt:1825:37>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1825'
--# expr-start 1195 '<Map at nodes.lkt:1825:37>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1825'
   --# expr-start 1186 '<FieldAccess for types at nodes.lkt:1825:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1825'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1186
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result_1 := As_Entity;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind typ Item_0
      --# expr-start 1194 '<And at nodes.lkt:1826:18>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1826'
--# expr-start 1188 '<FieldAccess for xref_type_equation at nodes.lkt:1826:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1826'
--# expr-start 1187 '<FieldAccess for subtype_name at nodes.lkt:1826:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1826'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Item_0.Node.Unconstrained_Array_Index_F_Subtype_Name, Info => Item_0.Info);
--# expr-done 1187
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_4 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 1188
--# expr-start 1193 '<If at nodes.lkt:1827:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1827'
--# expr-start 1190 '<IsNull at nodes.lkt:1827:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1827'
--# expr-start 1189 '<FieldAccess for lower_bound at nodes.lkt:1827:16>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1827'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Item_0.Node.Unconstrained_Array_Index_F_Lower_Bound, Info => Item_0.Info);
--# expr-done 1189
Is_Null := Fld_7.Node = null; 
--# expr-done 1190
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1827:45") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 1192 '<FieldAccess for sub_equation at nodes.lkt:1827:56>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1827'
--# expr-start 1191 '<FieldAccess for lower_bound at nodes.lkt:1827:56>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1827'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Item_0.Node.Unconstrained_Array_Index_F_Lower_Bound, Info => Item_0.Info);
--# expr-done 1191
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_5.Info);
--# end
--# expr-done 1192
   If_Result := Fld_6;
end if;
      Inc_Ref (If_Result);
--# expr-done 1193
And_Pred := Create_And (Fld_4, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1826:18") else null)); 
--# expr-done 1194
         declare
            Item_To_Append : constant Logic_Equation := And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2095;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1195
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1825:37") else null)); 
--# expr-done 1196
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_295;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_295;
                     Finalizer_Scope_2095;
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start UnconstrainedArrayIndices.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1832'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Is_Static
  (Node : Bare_Unconstrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Unconstrained_Array_Indices  := Bare_Unconstrained_Array_Indices (Node);
     Ent : Internal_Entity_Unconstrained_Array_Indices :=
       Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_0 : Bare_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Name;
Fld_4 : Boolean;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1200 '<AllQuantifier at nodes.lkt:1832:29>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1832'
   --# expr-start 1197 '<FieldAccess for types at nodes.lkt:1832:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1832'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 1197
Fld_1 := Fld.Node;
      Quantifier_Result := True;
      if Fld_1 = null then
      null;
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind t Item_0
      --# expr-start 1199 '<FieldAccess for is_static_subtype at nodes.lkt:1833:16>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1833'
--# expr-start 1198 '<FieldAccess for subtype_name at nodes.lkt:1833:16>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1833'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Item_0.Node.Unconstrained_Array_Index_F_Subtype_Name, Info => Item_0.Info);
--# expr-done 1198
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_static_subtype
Fld_4 := Libadalang.Implementation.Name_P_Is_Static_Subtype (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 1199
      Quantifier_Result := Fld_4;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 1200
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Aspect_Assoc
   --
      procedure Initialize_Fields_For_Aspect_Assoc
        (Self : Bare_Aspect_Assoc
         ; Aspect_Assoc_F_Id : Bare_Name
         ; Aspect_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Aspect_Assoc_F_Id := Aspect_Assoc_F_Id;
            Self.Aspect_Assoc_F_Expr := Aspect_Assoc_F_Expr;
      end Initialize_Fields_For_Aspect_Assoc;
   function Aspect_Assoc_F_Id
     (Node : Bare_Aspect_Assoc) return Bare_Name
   is
   begin
         return Node.Aspect_Assoc_F_Id;
   end;
   function Aspect_Assoc_F_Expr
     (Node : Bare_Aspect_Assoc) return Bare_Expr
   is
   begin
         return Node.Aspect_Assoc_F_Expr;
   end;
--# property-start AspectAssoc.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1842'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Entry_Point
  (Node : Bare_Aspect_Assoc
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Xref_Entry_Point;
--# end
--# property-start AspectAssoc.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1845'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_298 with Inline_Always;
            procedure Finalizer_Scope_2097 with Inline_Always;
            procedure Finalizer_Scope_2099 with Inline_Always;
            procedure Finalizer_Scope_2100 with Inline_Always;
      Local_Target : Bare_Ada_Node;
Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_38 : Bare_Type_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Internal_Single_Actual_Array_Access;
Fld_6 : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Type_Decl;
Fld_9 : Bare_Type_Def;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Name;
Fld_13 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Type_Decl;
Fld_14 : Bare_Type_Def;
Scope_Result : Bare_Type_Def;
Result_Var : Bare_Type_Def;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_15 : Bare_Expr;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Equation;
Fld_24 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Fld_25 : Logic_Equation;
Fld_26 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Fld_27 : Logic_Var;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_7 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred_2 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
Fld_30 : Internal_Entity_Expr;
Is_A_1 : Boolean;
If_Result_4 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_31 : Logic_Equation;
Cast_Expr_3 : Bare_Ada_Node;
Cast_Result_8 : Bare_Type_Decl;
Fld_32 : Logic_Equation;
Fld_33 : Internal_Entity_Expr;
Cast_Result_9 : Internal_Entity_Identifier;
Fld_34 : Logic_Var;
Cast_Expr_4 : Bare_Ada_Node;
Cast_Result_10 : Bare_Type_Decl;
As_Entity_1 : Internal_Entity_Type_Decl;
Is_Null_1 : Boolean;
If_Result_5 : Internal_Entity_Type_Decl;
Fld_35 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Cast_Result_11 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_36 : Internal_Entity_Expr;
Cast_Result_12 : Internal_Entity_Identifier;
Fld_37 : Logic_Var;
Cast_Expr_5 : Bare_Ada_Node;
Cast_Result_13 : Bare_Type_Decl;
As_Entity_2 : Internal_Entity_Type_Decl;
Is_Null_2 : Boolean;
If_Result_6 : Internal_Entity_Type_Decl;
Fld_38 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_14 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_39 : Internal_Entity_Expr;
Cast_Result_15 : Internal_Entity_Base_Aggregate;
Fld_40 : Internal_Entity_Assoc_List;
Fld_41 : Internal_Single_Actual_Array_Access;
Index_2 : Integer;
True_Rel_2 : Logic_Equation;
Fld_42 : Internal_Entity_Basic_Assoc;
Fld_43 : Internal_Entity_Expr;
Fld_44 : Logic_Equation;
Comp_Result : Boolean;
If_Result_7 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Cast_Expr_6 : Bare_Ada_Node;
Cast_Result_16 : Bare_Type_Decl;
Fld_45 : Logic_Equation;
Fld_46 : Internal_Entity_Expr;
Cast_Result_17 : Internal_Entity_Name;
Fld_47 : Logic_Equation;
Fld_48 : Internal_Entity_Expr;
Cast_Result_18 : Internal_Entity_Name;
Fld_49 : Logic_Equation;
Fld_50 : Internal_Entity_Expr;
Cast_Result_19 : Internal_Entity_Name;
Fld_51 : Logic_Var;
Cast_Expr_7 : Bare_Ada_Node;
Cast_Result_20 : Bare_Basic_Decl;
As_Entity_3 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
If_Result_8 : Internal_Entity_Basic_Decl;
Pred_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Is_A_2 : Boolean;
If_Result_9 : Logic_Equation;
Fld_52 : Internal_Entity_Expr;
Fld_53 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Result_Var_1 : Logic_Equation;
True_Rel_4 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_54 : Internal_Entity_Name;
Sym_2 : Symbol_Type;
Fld_55 : Boolean;
If_Result_10 : Logic_Equation;
Fld_56 : Internal_Entity_Name;
Sym_3 : Symbol_Type;
Fld_57 : Boolean;
If_Result_11 : Logic_Equation;
Fld_58 : Internal_Entity_Name;
Sym_4 : Symbol_Type;
Fld_59 : Boolean;
If_Result_12 : Logic_Equation;
Fld_60 : Internal_Entity_Name;
Sym_5 : Symbol_Type;
Fld_61 : Boolean;
If_Result_13 : Logic_Equation;
Fld_62 : Internal_Entity_Name;
Sym_6 : Symbol_Type;
Fld_63 : Boolean;
If_Result_14 : Logic_Equation;
Fld_64 : Internal_Entity_Name;
Fld_65 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_2 : Boolean;
Is_Equal_3 : Boolean;
If_Result_15 : Boolean;
Is_Equal_4 : Boolean;
If_Result_16 : Boolean;
If_Result_17 : Logic_Equation;
Fld_66 : Internal_Entity_Name;
Sym_10 : Symbol_Type;
Fld_67 : Boolean;
If_Result_18 : Logic_Equation;
Fld_68 : Internal_Entity_Name;
Sym_11 : Symbol_Type;
Fld_69 : Boolean;
If_Result_19 : Logic_Equation;
Fld_70 : Internal_Entity_Name;
Sym_12 : Symbol_Type;
Fld_71 : Boolean;
Fld_72 : Internal_Entity_Name;
Sym_13 : Symbol_Type;
Fld_73 : Boolean;
If_Result_20 : Boolean;
If_Result_21 : Logic_Equation;
Fld_74 : Internal_Entity_Name;
Sym_14 : Symbol_Type;
Fld_75 : Boolean;
If_Result_22 : Logic_Equation;
Fld_76 : Internal_Entity_Name;
Fld_77 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_5 : Boolean;
Is_Equal_6 : Boolean;
If_Result_23 : Boolean;
If_Result_24 : Logic_Equation;
Fld_78 : Internal_Entity_Name;
Fld_79 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Is_Equal_7 : Boolean;
Is_Equal_8 : Boolean;
If_Result_25 : Boolean;
Is_Equal_9 : Boolean;
If_Result_26 : Boolean;
Is_Equal_10 : Boolean;
If_Result_27 : Boolean;
Is_Equal_11 : Boolean;
If_Result_28 : Boolean;
Is_Equal_12 : Boolean;
If_Result_29 : Boolean;
Is_Equal_13 : Boolean;
If_Result_30 : Boolean;
Is_Equal_14 : Boolean;
If_Result_31 : Boolean;
Is_Equal_15 : Boolean;
If_Result_32 : Boolean;
If_Result_33 : Logic_Equation;
Fld_80 : Internal_Entity_Name;
Fld_81 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Is_Equal_16 : Boolean;
Is_Equal_17 : Boolean;
If_Result_34 : Boolean;
If_Result_35 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_298 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_29);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_38);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Fld_45);
                     Dec_Ref (Fld_47);
                     Dec_Ref (Fld_49);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (True_Rel_4);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (If_Result_12);
                     Dec_Ref (If_Result_13);
                     Dec_Ref (If_Result_14);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (If_Result_18);
                     Dec_Ref (If_Result_19);
                     Dec_Ref (If_Result_21);
                     Dec_Ref (If_Result_22);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_33);
                     Dec_Ref (If_Result_35);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_298;
            procedure Finalizer_Scope_2097 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2097;
            procedure Finalizer_Scope_2099 is
            begin
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Fld_44);
                     Dec_Ref (If_Result_7);
            end Finalizer_Scope_2099;
            procedure Finalizer_Scope_2100 is
            begin
                     Dec_Ref (Fld_53);
            end Finalizer_Scope_2100;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1370 '<Let at nodes.lkt:1846:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1846'
--# scope-start
--# expr-start 1203 '<parent at nodes.lkt:1846:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1846'
--# expr-start 1202 '<parent at nodes.lkt:1846:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1846'
--# expr-start 1201 '<parent at nodes.lkt:1846:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1846'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1201
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1202
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 1203
Local_Target := Fld_2; 
--# bind target Local_Target
--# expr-start 1369 '<If at nodes.lkt:1849:9>' If_Result_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1205 '<FieldAccess for name_symbol at nodes.lkt:1849:12>' Fld_81 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1204 '<FieldAccess for id at nodes.lkt:1849:12>' Fld_80 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_80 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1204
      if Fld_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_81 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_80.Node);
--# end
--# expr-done 1205
--# expr-start 1206 '<Symbol Aggregate at nodes.lkt:1849:37>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 1206
Is_Equal_17 := Fld_81 = Sym_26; 
if Is_Equal_17 then
   If_Result_34 := True;
else
--# expr-start 1207 '<Symbol Iterable at nodes.lkt:1849:52>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 1207
Is_Equal_16 := Fld_81 = Sym_27; 
   If_Result_34 := Is_Equal_16;
end if;
if If_Result_34 then
   --# expr-start 1217 '<LogicAll at nodes.lkt:1850:13>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
--# expr-start 1216 '<Map at nodes.lkt:1850:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
   --# expr-start 1211 '<FieldAccess for unpacked_params at nodes.lkt:1850:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
--# expr-start 1210 '<FieldAccess for assocs at nodes.lkt:1850:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
--# expr-start 1209 '<Cast to Aggregate at nodes.lkt:1850:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
--# expr-start 1208 '<FieldAccess for expr at nodes.lkt:1850:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1850'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1208
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 1209
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 1210
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_5 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1211
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_5;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_3
      --# expr-start 1215 '<FieldAccess for xref_no_overloading at nodes.lkt:1851:25>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1851'
--# expr-start 1214 '<Cast to Name at nodes.lkt:1851:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1851'
--# expr-start 1213 '<FieldAccess for expr at nodes.lkt:1851:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1851'
--# expr-start 1212 '<FieldAccess for assoc at nodes.lkt:1851:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1851'
Fld_6 := Item_3.Assoc;
--# expr-done 1212
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1213
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1214
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Sequential => False, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1215
         declare
            Item_To_Append : constant Logic_Equation := Fld_8;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2097;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1216
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1850:13") else null)); 
--# expr-done 1217
   If_Result_35 := Logic_Boolean_Op;
else
   --# expr-start 1368 '<If at nodes.lkt:1849:9>' If_Result_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1219 '<FieldAccess for name_symbol at nodes.lkt:1856:14>' Fld_79 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1856'
--# expr-start 1218 '<FieldAccess for id at nodes.lkt:1856:14>' Fld_78 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1856'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_78 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1218
      if Fld_78.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_79 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_78.Node);
--# end
--# expr-done 1219
--# expr-start 1220 '<Symbol Pre at nodes.lkt:1857:14>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1857'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1220
Is_Equal_15 := Fld_79 = Sym_17; 
if Is_Equal_15 then
   If_Result_32 := True;
else
--# expr-start 1221 '<Symbol Post at nodes.lkt:1857:23>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1857'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1221
Is_Equal_14 := Fld_79 = Sym_18; 
if Is_Equal_14 then
   If_Result_31 := True;
else
--# expr-start 1222 '<Symbol Refined_Post at nodes.lkt:1857:33>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1857'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Post); 
--# expr-done 1222
Is_Equal_13 := Fld_79 = Sym_19; 
if Is_Equal_13 then
   If_Result_30 := True;
else
--# expr-start 1223 '<Symbol Type_Invariant at nodes.lkt:1857:51>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1857'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Invariant); 
--# expr-done 1223
Is_Equal_12 := Fld_79 = Sym_20; 
if Is_Equal_12 then
   If_Result_29 := True;
else
--# expr-start 1224 '<Symbol Invariant at nodes.lkt:1858:14>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1858'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invariant); 
--# expr-done 1224
Is_Equal_11 := Fld_79 = Sym_21; 
if Is_Equal_11 then
   If_Result_28 := True;
else
--# expr-start 1225 '<Symbol Predicate at nodes.lkt:1858:29>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1858'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 1225
Is_Equal_10 := Fld_79 = Sym_22; 
if Is_Equal_10 then
   If_Result_27 := True;
else
--# expr-start 1226 '<Symbol Static_Predicate at nodes.lkt:1858:44>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1858'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 1226
Is_Equal_9 := Fld_79 = Sym_23; 
if Is_Equal_9 then
   If_Result_26 := True;
else
--# expr-start 1227 '<Symbol Dynamic_Predicate at nodes.lkt:1859:14>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1859'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dynamic_Predicate); 
--# expr-done 1227
Is_Equal_8 := Fld_79 = Sym_24; 
if Is_Equal_8 then
   If_Result_25 := True;
else
--# expr-start 1228 '<Symbol Initial_Condition at nodes.lkt:1859:37>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1859'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 1228
Is_Equal_7 := Fld_79 = Sym_25; 
   If_Result_25 := Is_Equal_7;
end if;
   If_Result_26 := If_Result_25;
end if;
   If_Result_27 := If_Result_26;
end if;
   If_Result_28 := If_Result_27;
end if;
   If_Result_29 := If_Result_28;
end if;
   If_Result_30 := If_Result_29;
end if;
   If_Result_31 := If_Result_30;
end if;
   If_Result_32 := If_Result_31;
end if;
if If_Result_32 then
   --# expr-start 1251 '<bind at nodes.lkt:1864:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1864'
--# expr-start 1240 '<If at nodes.lkt:1865:17>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1865'
--# expr-start 1230 '<FieldAccess for name_symbol at nodes.lkt:1865:20>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1865'
--# expr-start 1229 '<FieldAccess for id at nodes.lkt:1865:20>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1865'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1229
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 1230
--# expr-start 1231 '<Symbol Pre at nodes.lkt:1865:45>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1865'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1231
Is_Equal_1 := Fld_13 = Sym; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 1232 '<Symbol Post at nodes.lkt:1865:54>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1865'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1232
Is_Equal := Fld_13 = Sym_1; 
   If_Result := Is_Equal;
end if;
if If_Result then
   --# expr-start 1235 '<IsA AccessToSubpDef at nodes.lkt:1866:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1866'
--# expr-start 1233 '<Cast to TypeDecl at nodes.lkt:1866:20>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1866'
Cast_Expr_1 := Local_Target; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Type_Decl
   then
      Cast_Result_3 := Cast_Expr_1;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 1233
Var_Expr_38 := Cast_Result_3;
if Var_Expr_38 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1234 '<FieldAccess for type_def at nodes.lkt:1866:20>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1866'
      if Var_Expr_38 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Var_Expr_38.Type_Decl_F_Type_Def;
--# expr-done 1234
Scope_Result := Fld_14;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_A := Result_Var /= null 
and then Result_Var.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 1235
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 1238 '<FieldAccess for children_env at nodes.lkt:1868:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1868'
--# expr-start 1237 '<FieldAccess for type_def at nodes.lkt:1868:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1868'
--# expr-start 1236 '<Cast to TypeDecl at nodes.lkt:1868:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1868'
Cast_Expr := Local_Target; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1236
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Cast_Result_2.Type_Decl_F_Type_Def;
--# expr-done 1237
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Fld_9);
--# expr-done 1238
   If_Result_2 := Fld_10;
else
   --# expr-start 1239 '<FieldAccess for node_env at nodes.lkt:1870:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1870'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1239
   If_Result_2 := Fld_11;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1240
Bound_Env := If_Result_2; Inc_Ref (Bound_Env);
--# expr-start 1250 '<And at nodes.lkt:1873:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
--# expr-start 1247 '<And at nodes.lkt:1873:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
--# expr-start 1244 '<Bind at nodes.lkt:1873:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
--# expr-start 1242 '<FieldAccess for expected_type_var at nodes.lkt:1873:17>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
--# expr-start 1241 '<FieldAccess for expr at nodes.lkt:1873:17>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Self.Aspect_Assoc_F_Expr;
--# expr-done 1241
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_15);
--# expr-done 1242
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
--# expr-start 1243 '<FieldAccess for bool_type at nodes.lkt:1873:48>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1873'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_17 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 1243
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
Bind_Result := Solver.Create_Assign (Fld_16, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1873:13") else null)); 
--# expr-done 1244
--# expr-start 1246 '<FieldAccess for sub_equation at nodes.lkt:1874:17>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1874'
--# expr-start 1245 '<FieldAccess for expr at nodes.lkt:1874:17>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1874'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1245
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_18.Info);
--# end
--# expr-done 1246
And_Pred := Create_And (Bind_Result, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1873:13") else null)); 
--# expr-done 1247
--# expr-start 1249 '<FieldAccess for matches_expected_formal_type at nodes.lkt:1875:17>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1875'
--# expr-start 1248 '<FieldAccess for expr at nodes.lkt:1875:17>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1875'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Self.Aspect_Assoc_F_Expr;
--# expr-done 1248
      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_21 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_20, Origin => Origin);
--# end
--# expr-done 1249
And_Pred_1 := Create_And (And_Pred, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1873:13") else null)); 
--# expr-done 1250
Dyn_Var_Bind_Result := And_Pred_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 1251
   If_Result_33 := Dyn_Var_Bind_Result;
else
   --# expr-start 1367 '<If at nodes.lkt:1849:9>' If_Result_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1253 '<FieldAccess for name_symbol at nodes.lkt:1877:14>' Fld_77 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1877'
--# expr-start 1252 '<FieldAccess for id at nodes.lkt:1877:14>' Fld_76 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1877'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_76 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1252
      if Fld_76.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_77 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_76.Node);
--# end
--# expr-done 1253
--# expr-start 1254 '<Symbol Contract_Cases at nodes.lkt:1878:14>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1878'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 1254
Is_Equal_6 := Fld_77 = Sym_15; 
if Is_Equal_6 then
   If_Result_23 := True;
else
--# expr-start 1255 '<Symbol Subprogram_Variant at nodes.lkt:1878:34>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1878'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Subprogram_Variant); 
--# expr-done 1255
Is_Equal_5 := Fld_77 = Sym_16; 
   If_Result_23 := Is_Equal_5;
end if;
if If_Result_23 then
   --# expr-start 1257 '<FieldAccess for sub_equation at nodes.lkt:1880:13>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1880'
--# expr-start 1256 '<FieldAccess for expr at nodes.lkt:1880:13>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1880'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1256
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_23 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_22.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_22.Info);
--# end
--# expr-done 1257
   If_Result_24 := Fld_23;
else
   --# expr-start 1366 '<If at nodes.lkt:1849:9>' If_Result_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1260 '<FieldAccess for name_is at nodes.lkt:1882:14>' Fld_75 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1882'
--# expr-start 1258 '<FieldAccess for id at nodes.lkt:1882:14>' Fld_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1882'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_74 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1258
      if Fld_74.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1259 '<Symbol Put_Image at nodes.lkt:1882:30>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1882'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 1259
--# property-call-start Name.name_is
Fld_75 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_74.Node, Sym => Sym_14);
--# end
--# expr-done 1260
if Fld_75 then
   --# expr-start 1270 '<And at nodes.lkt:1883:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1883'
--# expr-start 1263 '<FieldAccess for xref_no_overloading at nodes.lkt:1883:13>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1883'
--# expr-start 1262 '<Cast to Name at nodes.lkt:1883:13>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1883'
--# expr-start 1261 '<FieldAccess for expr at nodes.lkt:1883:13>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1883'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1261
   if Fld_24.Node = null
      or else Fld_24.Node.Kind in Ada_Name
   then
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1262
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_25 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_5.Node, Sequential => False, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 1263
--# expr-start 1269 '<Predicate on BasicDecl.is_put_image_subprogram_for_type at nodes.lkt:1885:19>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1885'
--# expr-start 1266 '<FieldAccess for ref_var at nodes.lkt:1887:17>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1887'
--# expr-start 1265 '<Cast to Name at nodes.lkt:1887:17>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1887'
--# expr-start 1264 '<FieldAccess for expr at nodes.lkt:1887:17>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1887'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1264
   if Fld_26.Node = null
      or else Fld_26.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 1265
      if Cast_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_6.Node);
--# end
--# expr-done 1266
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 1268 '<as_entity at nodes.lkt:1888:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1888'
--# expr-start 1267 '<Cast to BaseTypeDecl at nodes.lkt:1888:17>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1888'
Cast_Expr_2 := Local_Target; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_7 := Cast_Expr_2;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1267
Is_Null := Cast_Result_7 = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_7); 
   If_Result_3 := As_Entity;
end if;
--# expr-done 1268
Pred := Solver.Create_Predicate (Fld_27, Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate (If_Result_3)); 
--# expr-done 1269
And_Pred_2 := Create_And (Fld_25, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1883:13") else null)); 
--# expr-done 1270
   If_Result_22 := And_Pred_2;
else
   --# expr-start 1365 '<If at nodes.lkt:1849:9>' If_Result_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1273 '<FieldAccess for name_is at nodes.lkt:1892:14>' Fld_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1892'
--# expr-start 1271 '<FieldAccess for id at nodes.lkt:1892:14>' Fld_70 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1892'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_70 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1271
      if Fld_70.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1272 '<Symbol Global at nodes.lkt:1892:30>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1892'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 1272
--# property-call-start Name.name_is
Fld_71 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_70.Node, Sym => Sym_12);
--# end
--# expr-done 1273
if Fld_71 then
   If_Result_20 := True;
else
   --# expr-start 1276 '<FieldAccess for name_is at nodes.lkt:1893:17>' Fld_73 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1893'
--# expr-start 1274 '<FieldAccess for id at nodes.lkt:1893:17>' Fld_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1893'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_72 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1274
      if Fld_72.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1275 '<Symbol Refined_Global at nodes.lkt:1893:33>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1893'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 1275
--# property-call-start Name.name_is
Fld_73 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_72.Node, Sym => Sym_13);
--# end
--# expr-done 1276
   If_Result_20 := Fld_73;
end if;
if If_Result_20 then
   --# expr-start 1281 '<If at nodes.lkt:1895:13>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1895'
--# expr-start 1278 '<IsA NullLiteral at nodes.lkt:1895:16>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1895'
--# expr-start 1277 '<FieldAccess for expr at nodes.lkt:1895:16>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1895'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1277
Is_A_1 := Fld_30.Node /= null 
and then Fld_30.Node.Kind in Ada_Null_Literal_Range; 
--# expr-done 1278
if Is_A_1 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1895:46") else null)); 
   If_Result_4 := True_Rel;
else
   --# expr-start 1280 '<FieldAccess for sub_equation at nodes.lkt:1896:18>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1896'
--# expr-start 1279 '<FieldAccess for expr at nodes.lkt:1896:18>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1896'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1279
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_28.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_28.Info);
--# end
--# expr-done 1280
   If_Result_4 := Fld_29;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 1281
   If_Result_21 := If_Result_4;
else
   --# expr-start 1364 '<If at nodes.lkt:1849:9>' If_Result_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1284 '<FieldAccess for name_is at nodes.lkt:1900:14>' Fld_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1900'
--# expr-start 1282 '<FieldAccess for id at nodes.lkt:1900:14>' Fld_68 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1900'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_68 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1282
      if Fld_68.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1283 '<Symbol Convention at nodes.lkt:1900:30>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1900'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 1283
--# property-call-start Name.name_is
Fld_69 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_68.Node, Sym => Sym_11);
--# end
--# expr-done 1284
if Fld_69 then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1900:50") else null)); 
   If_Result_19 := True_Rel_1;
else
   --# expr-start 1363 '<If at nodes.lkt:1849:9>' If_Result_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1287 '<FieldAccess for name_is at nodes.lkt:1901:14>' Fld_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1901'
--# expr-start 1285 '<FieldAccess for id at nodes.lkt:1901:14>' Fld_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1901'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_66 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1285
      if Fld_66.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1286 '<Symbol Stable_Properties at nodes.lkt:1901:30>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1901'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stable_Properties); 
--# expr-done 1286
--# property-call-start Name.name_is
Fld_67 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_66.Node, Sym => Sym_10);
--# end
--# expr-done 1287
if Fld_67 then
   --# expr-start 1288 '<FieldAccess for stable_properties_assoc_equation at nodes.lkt:1902:13>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1902'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AspectAssoc.stable_properties_assoc_equation
Fld_31 := Libadalang.Implementation.Aspect_Assoc_P_Stable_Properties_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 1288
   If_Result_18 := Fld_31;
else
   --# expr-start 1362 '<If at nodes.lkt:1849:9>' If_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1290 '<FieldAccess for name_symbol at nodes.lkt:1903:14>' Fld_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1903'
--# expr-start 1289 '<FieldAccess for id at nodes.lkt:1903:14>' Fld_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1903'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_64 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1289
      if Fld_64.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_65 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_64.Node);
--# end
--# expr-done 1290
--# expr-start 1291 '<Symbol Integer_Literal at nodes.lkt:1903:39>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1903'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Literal); 
--# expr-done 1291
Is_Equal_4 := Fld_65 = Sym_7; 
if Is_Equal_4 then
   If_Result_16 := True;
else
--# expr-start 1292 '<Symbol Real_Literal at nodes.lkt:1904:39>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1904'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Real_Literal); 
--# expr-done 1292
Is_Equal_3 := Fld_65 = Sym_8; 
if Is_Equal_3 then
   If_Result_15 := True;
else
--# expr-start 1293 '<Symbol String_Literal at nodes.lkt:1905:39>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1905'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String_Literal); 
--# expr-done 1293
Is_Equal_2 := Fld_65 = Sym_9; 
   If_Result_15 := Is_Equal_2;
end if;
   If_Result_16 := If_Result_15;
end if;
if If_Result_16 then
   --# expr-start 1295 '<FieldAccess for user_defined_literals_equation at nodes.lkt:1906:13>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1906'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1294 '<Cast to TypeDecl at nodes.lkt:1906:49>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1906'
Cast_Expr_3 := Local_Target; 
   if Cast_Expr_3 = null
      or else Cast_Expr_3.Kind in Ada_Type_Decl
   then
      Cast_Result_8 := Cast_Expr_3;
   else
         Cast_Result_8 := No_Bare_Ada_Node;
   end if;
--# expr-done 1294
--# property-call-start AspectAssoc.user_defined_literals_equation
Fld_32 := Libadalang.Implementation.Aspect_Assoc_P_User_Defined_Literals_Equation (Node => Ent.Node, Target => Cast_Result_8, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 1295
   If_Result_17 := Fld_32;
else
   --# expr-start 1361 '<If at nodes.lkt:1849:9>' If_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1298 '<FieldAccess for name_is at nodes.lkt:1911:14>' Fld_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1911'
--# expr-start 1296 '<FieldAccess for id at nodes.lkt:1911:14>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1911'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_62 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1296
      if Fld_62.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1297 '<Symbol Constant_Indexing at nodes.lkt:1911:30>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1911'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 1297
--# property-call-start Name.name_is
Fld_63 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_62.Node, Sym => Sym_6);
--# end
--# expr-done 1298
if Fld_63 then
   --# expr-start 1306 '<Bind at nodes.lkt:1912:13>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1912'
--# expr-start 1301 '<FieldAccess for ref_var at nodes.lkt:1912:17>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1912'
--# expr-start 1300 '<Cast to Identifier at nodes.lkt:1912:17>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1912'
--# expr-start 1299 '<FieldAccess for expr at nodes.lkt:1912:17>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1912'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1299
   if Fld_33.Node = null
      or else Fld_33.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Identifier
        (Node => Fld_33.Node,
         Info => Fld_33.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1300
      if Cast_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_34 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_9.Node);
--# end
--# expr-done 1301
Fld_34.Value := No_Entity;
Entity_Vars.Reset (Fld_34);
--# expr-start 1305 '<at at nodes.lkt:1913:17>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1913'
--# expr-start 1304 '<FieldAccess for constant_indexing_fns at nodes.lkt:1913:17>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1913'
--# expr-start 1303 '<as_entity at nodes.lkt:1913:17>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1913'
--# expr-start 1302 '<Cast to TypeDecl at nodes.lkt:1913:17>' Cast_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1913'
Cast_Expr_4 := Local_Target; 
   if Cast_Expr_4 = null
      or else Cast_Expr_4.Kind in Ada_Type_Decl
   then
      Cast_Result_10 := Cast_Expr_4;
   else
         Cast_Result_10 := No_Bare_Ada_Node;
   end if;
--# expr-done 1302
Is_Null_1 := Cast_Result_10 = null; 
if Is_Null_1 then
   If_Result_5 := No_Entity_Type_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_10); 
   If_Result_5 := As_Entity_1;
end if;
--# expr-done 1303
      if If_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_35 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => If_Result_5.Node, E_Info => If_Result_5.Info);
--# end
--# expr-done 1304
Get_Result := Get (Self, Fld_35, 0, True); 
--# expr-done 1305
      Cast_Result_11 := Create_Internal_Entity
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_34, Cast_Result_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1912:13") else null)); 
--# expr-done 1306
   If_Result_14 := Bind_Result_1;
else
   --# expr-start 1360 '<If at nodes.lkt:1849:9>' If_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1309 '<FieldAccess for name_is at nodes.lkt:1914:14>' Fld_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1914'
--# expr-start 1307 '<FieldAccess for id at nodes.lkt:1914:14>' Fld_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1914'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1307
      if Fld_60.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1308 '<Symbol Variable_Indexing at nodes.lkt:1914:30>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1914'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 1308
--# property-call-start Name.name_is
Fld_61 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_60.Node, Sym => Sym_5);
--# end
--# expr-done 1309
if Fld_61 then
   --# expr-start 1317 '<Bind at nodes.lkt:1915:13>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1915'
--# expr-start 1312 '<FieldAccess for ref_var at nodes.lkt:1915:17>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1915'
--# expr-start 1311 '<Cast to Identifier at nodes.lkt:1915:17>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1915'
--# expr-start 1310 '<FieldAccess for expr at nodes.lkt:1915:17>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1915'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1310
   if Fld_36.Node = null
      or else Fld_36.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_12 := Create_Internal_Entity_Identifier
        (Node => Fld_36.Node,
         Info => Fld_36.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1311
      if Cast_Result_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_12.Node);
--# end
--# expr-done 1312
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
--# expr-start 1316 '<at at nodes.lkt:1916:17>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1916'
--# expr-start 1315 '<FieldAccess for variable_indexing_fns at nodes.lkt:1916:17>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1916'
--# expr-start 1314 '<as_entity at nodes.lkt:1916:17>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1916'
--# expr-start 1313 '<Cast to TypeDecl at nodes.lkt:1916:17>' Cast_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1916'
Cast_Expr_5 := Local_Target; 
   if Cast_Expr_5 = null
      or else Cast_Expr_5.Kind in Ada_Type_Decl
   then
      Cast_Result_13 := Cast_Expr_5;
   else
         Cast_Result_13 := No_Bare_Ada_Node;
   end if;
--# expr-done 1313
Is_Null_2 := Cast_Result_13 = null; 
if Is_Null_2 then
   If_Result_6 := No_Entity_Type_Decl;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_13); 
   If_Result_6 := As_Entity_2;
end if;
--# expr-done 1314
      if If_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_38 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => If_Result_6.Node, E_Info => If_Result_6.Info);
--# end
--# expr-done 1315
Get_Result_1 := Get (Self, Fld_38, 0, True); 
--# expr-done 1316
      Cast_Result_14 := Create_Internal_Entity
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_37, Cast_Result_14, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1915:13") else null)); 
--# expr-done 1317
   If_Result_13 := Bind_Result_2;
else
   --# expr-start 1359 '<If at nodes.lkt:1849:9>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1320 '<FieldAccess for name_is at nodes.lkt:1920:14>' Fld_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1920'
--# expr-start 1318 '<FieldAccess for id at nodes.lkt:1920:14>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1920'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1318
      if Fld_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1319 '<Symbol Annotate at nodes.lkt:1920:30>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1920'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 1319
--# property-call-start Name.name_is
Fld_59 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_58.Node, Sym => Sym_4);
--# end
--# expr-done 1320
if Fld_59 then
   --# expr-start 1330 '<LogicAll at nodes.lkt:1921:13>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
--# expr-start 1329 '<Map at nodes.lkt:1921:13>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
   --# expr-start 1324 '<FieldAccess for unpacked_params at nodes.lkt:1921:13>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
--# expr-start 1323 '<FieldAccess for assocs at nodes.lkt:1921:13>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
--# expr-start 1322 '<Cast to BaseAggregate at nodes.lkt:1921:13>' Cast_Result_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
--# expr-start 1321 '<FieldAccess for expr at nodes.lkt:1921:13>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1921'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1321
   if Fld_39.Node = null
      or else Fld_39.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result_15 := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_39.Node,
         Info => Fld_39.Info);
   else
         Cast_Result_15 := No_Entity_Base_Aggregate;
   end if;
--# expr-done 1322
      if Cast_Result_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_15.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_15.Info);
--# expr-done 1323
      if Fld_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_41 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_40.Node, E_Info => Fld_40.Info);
--# end
--# expr-done 1324
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_41;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_1
               --# bind i Index_2
      --# expr-start 1328 '<If at nodes.lkt:1922:28>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1922'
Comp_Result := Index_2 < 2; 
if Comp_Result then
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1922:42") else null)); 
   If_Result_7 := True_Rel_2;
else
   --# expr-start 1327 '<FieldAccess for annotate_argument_equation at nodes.lkt:1923:33>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1923'
--# expr-start 1326 '<FieldAccess for expr at nodes.lkt:1923:33>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1923'
--# expr-start 1325 '<FieldAccess for assoc at nodes.lkt:1923:33>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1923'
Fld_42 := Item_1.Assoc;
--# expr-done 1325
      if Fld_42.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_43 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_42.Node, E_Info => Fld_42.Info);
--# end
--# expr-done 1326
      if Fld_43.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.annotate_argument_equation
Fld_44 := Libadalang.Implementation.Expr_P_Annotate_Argument_Equation (Node => Fld_43.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_43.Info);
--# end
--# expr-done 1327
   If_Result_7 := Fld_44;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 1328
         declare
            Item_To_Append : constant Logic_Equation := If_Result_7;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2099;
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1329
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1921:13") else null)); 
--# expr-done 1330
   If_Result_12 := Logic_Boolean_Op_1;
else
   --# expr-start 1358 '<If at nodes.lkt:1849:9>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1333 '<FieldAccess for name_is at nodes.lkt:1925:14>' Fld_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1925'
--# expr-start 1331 '<FieldAccess for id at nodes.lkt:1925:14>' Fld_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1925'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1331
      if Fld_56.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1332 '<Symbol Implicit_Dereference at nodes.lkt:1925:30>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1925'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Implicit_Dereference); 
--# expr-done 1332
--# property-call-start Name.name_is
Fld_57 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_56.Node, Sym => Sym_3);
--# end
--# expr-done 1333
if Fld_57 then
   --# expr-start 1335 '<FieldAccess for implicit_dereference_equation at nodes.lkt:1926:13>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1926'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1334 '<Cast to TypeDecl at nodes.lkt:1926:48>' Cast_Result_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1926'
Cast_Expr_6 := Local_Target; 
   if Cast_Expr_6 = null
      or else Cast_Expr_6.Kind in Ada_Type_Decl
   then
      Cast_Result_16 := Cast_Expr_6;
   else
         Cast_Result_16 := No_Bare_Ada_Node;
   end if;
--# expr-done 1334
--# property-call-start AspectAssoc.implicit_dereference_equation
Fld_45 := Libadalang.Implementation.Aspect_Assoc_P_Implicit_Dereference_Equation (Node => Ent.Node, Target => Cast_Result_16, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 1335
   If_Result_11 := Fld_45;
else
   --# expr-start 1357 '<If at nodes.lkt:1849:9>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1849'
--# expr-start 1338 '<FieldAccess for name_is at nodes.lkt:1930:14>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1930'
--# expr-start 1336 '<FieldAccess for id at nodes.lkt:1930:14>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1930'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1336
      if Fld_54.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1337 '<Symbol Model_Of at nodes.lkt:1930:30>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1930'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Of); 
--# expr-done 1337
--# property-call-start Name.name_is
Fld_55 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_54.Node, Sym => Sym_2);
--# end
--# expr-done 1338
if Fld_55 then
   --# expr-start 1353 '<If at nodes.lkt:1931:13>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1931'
--# expr-start 1339 '<IsA BaseTypeDecl at nodes.lkt:1931:16>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1931'
Is_A_2 := Local_Target /= null 
and then Local_Target.Kind in Ada_Base_Type_Decl; 
--# expr-done 1339
if Is_A_2 then
   --# expr-start 1342 '<FieldAccess for xref_type_equation at nodes.lkt:1932:17>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1932'
--# expr-start 1341 '<Cast to Name at nodes.lkt:1932:17>' Cast_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1932'
--# expr-start 1340 '<FieldAccess for expr at nodes.lkt:1932:17>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1932'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1340
   if Fld_46.Node = null
      or else Fld_46.Node.Kind in Ada_Name
   then
      Cast_Result_17 := Create_Internal_Entity_Name
        (Node => Fld_46.Node,
         Info => Fld_46.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1341
      if Cast_Result_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_47 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result_17.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_17.Info);
--# end
--# expr-done 1342
   If_Result_9 := Fld_47;
else
   --# expr-start 1352 '<And at nodes.lkt:1934:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1934'
--# expr-start 1345 '<FieldAccess for xref_no_overloading at nodes.lkt:1934:17>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1934'
--# expr-start 1344 '<Cast to Name at nodes.lkt:1934:17>' Cast_Result_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1934'
--# expr-start 1343 '<FieldAccess for expr at nodes.lkt:1934:17>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1934'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1343
   if Fld_48.Node = null
      or else Fld_48.Node.Kind in Ada_Name
   then
      Cast_Result_18 := Create_Internal_Entity_Name
        (Node => Fld_48.Node,
         Info => Fld_48.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1344
      if Cast_Result_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_49 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_18.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_18.Info);
--# end
--# expr-done 1345
--# expr-start 1351 '<Predicate on BasicDecl.subp_decl_match_signature at nodes.lkt:1935:21>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1935'
--# expr-start 1348 '<FieldAccess for ref_var at nodes.lkt:1937:21>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1937'
--# expr-start 1347 '<Cast to Name at nodes.lkt:1937:21>' Cast_Result_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1937'
--# expr-start 1346 '<FieldAccess for expr at nodes.lkt:1937:21>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1937'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1346
   if Fld_50.Node = null
      or else Fld_50.Node.Kind in Ada_Name
   then
      Cast_Result_19 := Create_Internal_Entity_Name
        (Node => Fld_50.Node,
         Info => Fld_50.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1347
      if Cast_Result_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_51 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_19.Node);
--# end
--# expr-done 1348
Fld_51.Value := No_Entity;
Entity_Vars.Reset (Fld_51);
--# expr-start 1350 '<as_entity at nodes.lkt:1938:21>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1938'
--# expr-start 1349 '<Cast to BasicDecl at nodes.lkt:1938:21>' Cast_Result_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1938'
Cast_Expr_7 := Local_Target; 
   if Cast_Expr_7 = null
      or else Cast_Expr_7.Kind in Ada_Basic_Decl
   then
      Cast_Result_20 := Cast_Expr_7;
   else
         Cast_Result_20 := No_Bare_Ada_Node;
   end if;
--# expr-done 1349
Is_Null_3 := Cast_Result_20 = null; 
if Is_Null_3 then
   If_Result_8 := No_Entity_Basic_Decl;
else
As_Entity_3 := (Info => E_Info, Node => Cast_Result_20); 
   If_Result_8 := As_Entity_3;
end if;
--# expr-done 1350
Pred_1 := Solver.Create_Predicate (Fld_51, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (If_Result_8, Origin)); 
--# expr-done 1351
And_Pred_3 := Create_And (Fld_49, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1934:17") else null)); 
--# expr-done 1352
   If_Result_9 := And_Pred_3;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 1353
   If_Result_10 := If_Result_9;
else
   --# expr-start 1356 '<Or at nodes.lkt:1943:14>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1943'
--# expr-start 1354 '<FieldAccess for expr at nodes.lkt:1943:14>' Fld_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1943'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_52 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1354
Var_Expr_0 := Fld_52;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 1355 '<FieldAccess for sub_equation at nodes.lkt:1944:20>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1944'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_53 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1355
Scope_Result_1 := Fld_53;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2100;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1944:50") else null)); 
   Result_Var_1 := True_Rel_3;
end if;
      Inc_Ref (Result_Var_1);
True_Rel_4 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1945:14") else null)); 
Or_Pred := Create_Or (Result_Var_1, True_Rel_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1943:14") else null)); 
--# expr-done 1356
   If_Result_10 := Or_Pred;
end if;
      Inc_Ref (If_Result_10);
--# expr-done 1357
   If_Result_11 := If_Result_10;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 1358
   If_Result_12 := If_Result_11;
end if;
      Inc_Ref (If_Result_12);
--# expr-done 1359
   If_Result_13 := If_Result_12;
end if;
      Inc_Ref (If_Result_13);
--# expr-done 1360
   If_Result_14 := If_Result_13;
end if;
      Inc_Ref (If_Result_14);
--# expr-done 1361
   If_Result_17 := If_Result_14;
end if;
      Inc_Ref (If_Result_17);
--# expr-done 1362
   If_Result_18 := If_Result_17;
end if;
      Inc_Ref (If_Result_18);
--# expr-done 1363
   If_Result_19 := If_Result_18;
end if;
      Inc_Ref (If_Result_19);
--# expr-done 1364
   If_Result_21 := If_Result_19;
end if;
      Inc_Ref (If_Result_21);
--# expr-done 1365
   If_Result_22 := If_Result_21;
end if;
      Inc_Ref (If_Result_22);
--# expr-done 1366
   If_Result_24 := If_Result_22;
end if;
      Inc_Ref (If_Result_24);
--# expr-done 1367
   If_Result_33 := If_Result_24;
end if;
      Inc_Ref (If_Result_33);
--# expr-done 1368
   If_Result_35 := If_Result_33;
end if;
      Inc_Ref (If_Result_35);
--# expr-done 1369
Let_Result := If_Result_35; Inc_Ref (Let_Result);
--# end
--# expr-done 1370
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_298;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_298;
                     Finalizer_Scope_2097;
                     Finalizer_Scope_2099;
                     Finalizer_Scope_2100;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Xref_Equation;
--# end
--# property-start AspectAssoc.implicit_dereference_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1951'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Implicit_Dereference_Equation
  (Node : Bare_Aspect_Assoc
      ; Target : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_299 with Inline_Always;
            procedure Finalizer_Scope_2101 with Inline_Always;
      Local_Id : Internal_Entity_Identifier;
Local_Discr : Internal_Entity_Base_Formal_Param_Decl;
Fld : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Identifier;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Bare_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Quantifier_Result : Boolean;
Find_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_299 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_299;
            procedure Finalizer_Scope_2101 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2101;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1383 '<Let at nodes.lkt:1952:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1952'
--# scope-start
--# expr-start 1372 '<Cast to Identifier at nodes.lkt:1952:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1952'
--# expr-start 1371 '<FieldAccess for expr at nodes.lkt:1952:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1952'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1371
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result := Create_Internal_Entity_Identifier
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1372
Local_Id := Cast_Result; 
--# bind id Local_Id
--# expr-start 1382 '<Let at nodes.lkt:1953:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1953'
--# scope-start
--# expr-start 1379 '<Find at nodes.lkt:1953:21>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1953'
   --# expr-start 1374 '<FieldAccess for discriminants_list at nodes.lkt:1953:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1953'
--# expr-start 1373 '<as_entity at nodes.lkt:1953:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1953'
Is_Null := Target = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Target); 
   If_Result := As_Entity;
end if;
--# expr-done 1373
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 1374
      Find_Result := No_Entity_Base_Formal_Param_Decl;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind l Item_0
      --# expr-start 1378 '<AnyQuantifier at nodes.lkt:1954:20>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1954'
   --# expr-start 1375 '<FieldAccess for defining_names at nodes.lkt:1954:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1954'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1375
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
      --# expr-start 1377 '<FieldAccess for name_is at nodes.lkt:1954:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1954'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1376 '<GetSymbol at nodes.lkt:1954:60>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1954'
Fld_3 := Local_Id.Node; 
Sym := Get_Symbol (Fld_3); 
--# expr-done 1376
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_1.Node, Sym => Sym);
--# end
--# expr-done 1377
      Quantifier_Result := Fld_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1378
      if Quantifier_Result then
         Find_Result := Item_0; 
      end if;
   --# end
      Finalizer_Scope_2101;
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1379
Local_Discr := Find_Result; 
--# bind discr Local_Discr
--# expr-start 1381 '<Bind at nodes.lkt:1957:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1957'
--# expr-start 1380 '<FieldAccess for ref_var at nodes.lkt:1957:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1957'
      if Local_Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Id.Node);
--# end
--# expr-done 1380
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Local_Discr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1957:9") else null)); 
--# expr-done 1381
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1382
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1383
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_299;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_299;
                     Finalizer_Scope_2101;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Implicit_Dereference_Equation;
--# end
--# property-start AspectAssoc.stable_properties_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1963'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Stable_Properties_Assoc_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_300 with Inline_Always;
            procedure Finalizer_Scope_2103 with Inline_Always;
            procedure Finalizer_Scope_2105 with Inline_Always;
            procedure Finalizer_Scope_2106 with Inline_Always;
            procedure Finalizer_Scope_2111 with Inline_Always;
            procedure Finalizer_Scope_2113 with Inline_Always;
      Local_Identifiers : Internal_Entity_Identifier_Array_Access;
Match_0 : Internal_Entity_Aggregate;
Match_1 : Internal_Entity_Paren_Expr;
Match_2 : Internal_Entity_Expr;
Var_Expr_2 : Internal_Entity_Un_Op;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Map_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Array_Lit : Internal_Entity_Expr_Array_Access;
Array_Lit_1 : Internal_Entity_Expr_Array_Access;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Let_Result : Internal_Entity_Expr_Array_Access;
Scope_Result : Internal_Entity_Expr_Array_Access;
Cast_Result_1 : Internal_Entity_Paren_Expr;
Let_Result_1 : Internal_Entity_Expr_Array_Access;
Scope_Result_1 : Internal_Entity_Expr_Array_Access;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_2 : Internal_Entity_Expr_Array_Access;
Scope_Result_2 : Internal_Entity_Expr_Array_Access;
Match_Result : Internal_Entity_Expr_Array_Access;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Un_Op;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Map_Result_1 : Internal_Entity_Expr_Array_Access;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Identifier;
Map_Result_2 : Internal_Entity_Identifier_Array_Access;
Fld_8 : Symbol_Type;
Fld_9 : Bare_Ada_Node;
Fld_10 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Boolean;
If_Result_2 : Boolean;
Map_Result_3 : Internal_Entity_Array_Access;
Fld_13 : Logic_Var;
Bind_Result : Logic_Equation;
Map_Result_4 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Map_Result_5 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_300 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Local_Identifiers);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_300;
            procedure Finalizer_Scope_2103 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2103;
            procedure Finalizer_Scope_2105 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2105;
            procedure Finalizer_Scope_2106 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2106;
            procedure Finalizer_Scope_2111 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_2111;
            procedure Finalizer_Scope_2113 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2113;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1414 '<Let at nodes.lkt:1965:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1965'
--# scope-start
--# expr-start 1399 '<Map at nodes.lkt:1965:27>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1965'
   --# expr-start 1397 '<Map at nodes.lkt:1965:27>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1965'
   --# expr-start 1391 '<Match at nodes.lkt:1965:27>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1965'
--# expr-start 1384 '<FieldAccess for expr at nodes.lkt:1965:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1965'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1384
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Aggregate .. Ada_Bracket_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 1387 '<Map at nodes.lkt:1967:34>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1967'
   --# expr-start 1385 '<FieldAccess for assocs at nodes.lkt:1967:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1967'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Match_0.Node.Base_Aggregate_F_Assocs, Info => Match_0.Info);
--# expr-done 1385
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 1386 '<FieldAccess for expr at nodes.lkt:1967:54>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1967'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1386
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1387
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2103;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pe Match_1
--# expr-start 1389 '<ArrayLiteral at nodes.lkt:1970:35>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1970'
--# expr-start 1388 '<FieldAccess for expr at nodes.lkt:1970:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1970'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_1.Node.Paren_Expr_F_Expr, Info => Match_1.Info);
--# expr-done 1388
Array_Lit := Create_Internal_Entity_Expr_Array (Internal_Internal_Entity_Expr_Array'(1 => Fld_5)); 
--# expr-done 1389
Let_Result_1 := Array_Lit; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2105;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl_Expr .. Ada_Allocator | Ada_Delta_Aggregate .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 1390 '<ArrayLiteral at nodes.lkt:1971:23>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1971'
Array_Lit_1 := Create_Internal_Entity_Expr_Array (Internal_Internal_Entity_Expr_Array'(1 => No_Entity_Expr)); 
--# expr-done 1390
Let_Result_2 := Array_Lit_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2106;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 1391
      declare
         Map_Result_1_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Match_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
--# expr-start 1392 '<Cast to UnOp at nodes.lkt:1973:20>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1973'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Un_Op_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Un_Op
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Un_Op;
   end if;
--# expr-done 1392
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Un_Op then
   --# scope-start
   --# bind uo Var_Expr_2
--# expr-start 1396 '<If at nodes.lkt:1975:25>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1975'
--# expr-start 1394 '<IsA Op.Not at nodes.lkt:1975:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1975'
--# expr-start 1393 '<FieldAccess for op at nodes.lkt:1975:28>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1975'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Op (Node => Var_Expr_2.Node.Un_Op_F_Op, Info => Var_Expr_2.Info);
--# expr-done 1393
Is_A := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Op_Not_Range; 
--# expr-done 1394
if Is_A then
   --# expr-start 1395 '<FieldAccess for expr at nodes.lkt:1975:49>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1975'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Un_Op_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 1395
   If_Result_1 := Fld_6;
else
   If_Result_1 := Item_1;
end if;
--# expr-done 1396
Scope_Result_3 := If_Result_1;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := Item_1;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Expr := Result_Var;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1397
      declare
         Map_Result_2_Vec : Internal_Entity_Identifier_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Map_Result_1;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_3
      --# expr-start 1398 '<Cast to Identifier at nodes.lkt:1977:22>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1977'
Cast_Expr_1 := Item_3; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Identifier
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1398
         declare
            Item_To_Append : constant Internal_Entity_Identifier := Cast_Result_4;
         begin
            Internal_Entity_Identifier_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Identifier_Array
           (Items_Count => Natural (Internal_Entity_Identifier_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Identifier_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Identifier_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Identifier_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1399
Local_Identifiers := Map_Result_2; Inc_Ref (Local_Identifiers);
--# bind identifiers Local_Identifiers
--# expr-start 1413 '<LogicAll at nodes.lkt:1980:9>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1980'
--# expr-start 1412 '<Map at nodes.lkt:1980:9>' Map_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1980'
      declare
         Map_Result_5_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Identifier_Array_Access := Local_Identifiers;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_4
      --# expr-start 1411 '<LogicAny at nodes.lkt:1981:20>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1981'
--# expr-start 1410 '<Map at nodes.lkt:1981:20>' Map_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1981'
   --# expr-start 1407 '<Map at nodes.lkt:1981:20>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1981'
   --# expr-start 1402 '<FieldAccess for env_get at nodes.lkt:1981:20>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1981'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1400 '<FieldAccess for sym at nodes.lkt:1982:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1982'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_8 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Item_4.Node);
--# end
--# expr-done 1400
--# expr-start 1401 '<FieldAccess for origin_node at nodes.lkt:1982:70>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1982'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_9 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1401
--# property-call-start AdaNode.env_get
Fld_10 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_8, Lookup => Recursive, From_Node => Fld_9, Categories => (Inherited_Primitives => True, Nocat => True));
--# end
--# expr-done 1402
      declare
         Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_10;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_5
--# expr-start 1403 '<IsA SubpDecl, ExprFunction at nodes.lkt:1986:21>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1986'
Is_A_1 := Item_5.Node /= null 
and then Item_5.Node.Kind in Ada_Subp_Decl_Range | Ada_Expr_Function_Range; 
--# expr-done 1403
if Is_A_1 then
   --# expr-start 1406 '<FieldAccess for denotes_the_property_function at nodes.lkt:1987:23>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1987'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1405 '<FieldAccess for subp_spec_or_null at nodes.lkt:1987:55>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1987'
--# expr-start 1404 '<Cast to BasicDecl at nodes.lkt:1987:55>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1987'
Cast_Expr_2 := Item_5; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1404
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 1405
--# property-call-start BaseId.denotes_the_property_function
Fld_12 := Libadalang.Implementation.Base_Id_P_Denotes_The_Property_Function (Node => Item_4.Node, Subp_Spec => Fld_11, E_Info => Item_4.Info);
--# end
--# expr-done 1406
   If_Result_2 := Fld_12;
else
   If_Result_2 := False;
end if;
         if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity := Item_5;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 1407
      declare
         Map_Result_4_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_3;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_6
      --# expr-start 1409 '<Bind at nodes.lkt:1988:32>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1988'
--# expr-start 1408 '<FieldAccess for ref_var at nodes.lkt:1988:36>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1988'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Item_4.Node);
--# end
--# expr-done 1408
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Bind_Result := Solver.Create_Assign (Fld_13, Item_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1988:32") else null)); 
--# expr-done 1409
         declare
            Item_To_Append : constant Logic_Equation := Bind_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2113;
         end loop;
      end;
         Map_Result_4 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_4_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 1410
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result_4.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1981:20") else null)); 
--# expr-done 1411
         declare
            Item_To_Append : constant Logic_Equation := Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2111;
         end loop;
      end;
         Map_Result_5 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_5_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 1412
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_5.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1980:9") else null)); 
--# expr-done 1413
Let_Result_3 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 1414
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_300;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_300;
                     Finalizer_Scope_2103;
                     Finalizer_Scope_2105;
                     Finalizer_Scope_2106;
                     Finalizer_Scope_2111;
                     Finalizer_Scope_2113;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Stable_Properties_Assoc_Equation;
--# end
--# property-start AspectAssoc.user_defined_literals_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1995'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_User_Defined_Literals_Equation
  (Node : Bare_Aspect_Assoc
      ; Target : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_301 with Inline_Always;
      Fld : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Identifier;
Fld_1 : Logic_Var;
As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type;
Fld_4 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_301 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_301;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1423 '<Bind at nodes.lkt:1995:70>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1995'
--# expr-start 1417 '<FieldAccess for ref_var at nodes.lkt:1996:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1996'
--# expr-start 1416 '<Cast to Identifier at nodes.lkt:1996:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1996'
--# expr-start 1415 '<FieldAccess for expr at nodes.lkt:1996:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1996'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1415
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result := Create_Internal_Entity_Identifier
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1416
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result.Node);
--# end
--# expr-done 1417
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 1422 '<at at nodes.lkt:1997:9>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1997'
--# expr-start 1421 '<FieldAccess for user_defined_literal_fns at nodes.lkt:1997:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1997'
--# expr-start 1418 '<as_entity at nodes.lkt:1997:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1997'
Is_Null := Target = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Target); 
   If_Result := As_Entity;
end if;
--# expr-done 1418
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1420 '<FieldAccess for name_symbol at nodes.lkt:1998:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1998'
--# expr-start 1419 '<FieldAccess for id at nodes.lkt:1998:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:1998'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1419
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 1420
--# property-call-start TypeDecl.user_defined_literal_fns
Fld_4 := Libadalang.Implementation.Type_Decl_P_User_Defined_Literal_Fns (Node => If_Result.Node, Aspect => Fld_3, E_Info => If_Result.Info);
--# end
--# expr-done 1421
Get_Result := Get (Self, Fld_4, 0, True); 
--# expr-done 1422
      Cast_Result_1 := Create_Internal_Entity
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:1995:70") else null)); 
--# expr-done 1423
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_301;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_301;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_User_Defined_Literals_Equation;
--# end
--# property-start AspectAssoc.aspect_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2007'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Aspect_Name
  (Node : Bare_Aspect_Assoc
      ; N : Internal_Entity_Name
  )
   return String_Type
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
      --# bind self Self
   --# bind n N
   Property_Result : String_Type;
            procedure Finalizer_Scope_302 with Inline_Always;
            procedure Finalizer_Scope_2114 with Inline_Always;
            procedure Finalizer_Scope_2115 with Inline_Always;
      Match_0 : Internal_Entity_Base_Id;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Fld : Symbol_Type;
Fld_1 : String_Type;
Fld_2 : Internal_Entity_Name;
Fld_3 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Fld_6 : String_Type;
Concat_Result_1 : String_Type;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Base_Id;
Let_Result : String_Type;
Scope_Result : String_Type;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Let_Result_1 : String_Type;
Scope_Result_1 : String_Type;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_2 : String_Type;
Scope_Result_2 : String_Type;
Match_Result : String_Type;
            procedure Finalizer_Scope_302 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_302;
            procedure Finalizer_Scope_2114 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2114;
            procedure Finalizer_Scope_2115 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_2115;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1432 '<Match at nodes.lkt:2010:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2010'
      if N.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := N; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bid Match_0
--# expr-start 1425 '<FieldAccess for image at nodes.lkt:2011:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2011'
--# expr-start 1424 '<FieldAccess for sym at nodes.lkt:2011:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2011'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0.Node);
--# end
--# expr-done 1424
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 1425
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2114;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ar Match_1
--# expr-start 1427 '<FieldAccess for aspect_name at nodes.lkt:2012:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1426 '<FieldAccess for prefix at nodes.lkt:2012:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 1426
--# property-call-start AspectAssoc.aspect_name
Fld_3 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Self, N => Fld_2);
--# end
--# expr-done 1427
--# expr-start 1428 '<String at nodes.lkt:2012:68>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
Str := Create_String ("'"); 
--# expr-done 1428
Concat_Result := Concat_String (Fld_3, Str); 
--# expr-start 1431 '<FieldAccess for image at nodes.lkt:2012:74>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
--# expr-start 1430 '<FieldAccess for sym at nodes.lkt:2012:74>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
--# expr-start 1429 '<FieldAccess for attribute at nodes.lkt:2012:74>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2012'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Match_1.Node.Attribute_Ref_F_Attribute, Info => Match_1.Info);
--# expr-done 1429
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_4.Node);
--# end
--# expr-done 1430
Fld_6 := Libadalang.Implementation.Image (Self => Fld_5);
--# expr-done 1431
Concat_Result_1 := Concat_String (Concat_Result, Fld_6); 
Let_Result_1 := Concat_Result_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2115;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Precondition_Failure with "aspect_name called on an invalid aspect name"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 1432
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_302;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_302;
                     Finalizer_Scope_2114;
                     Finalizer_Scope_2115;
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Aspect_Name;
--# end
--# property-start AspectAssoc.is_ghost_code 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2018'
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Is_Ghost_Code
  (Node : Bare_Aspect_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc  := Bare_Aspect_Assoc (Node);
     Ent : Internal_Entity_Aspect_Assoc :=
       Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1434 '<FieldAccess for name_symbol at nodes.lkt:2019:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2019'
--# expr-start 1433 '<FieldAccess for id at nodes.lkt:2019:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2019'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 1433
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 1434
--# expr-start 1435 '<Symbol Pre at nodes.lkt:2019:34>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2019'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 1435
Is_Equal_2 := Fld_1 = Sym; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 1436 '<Symbol Post at nodes.lkt:2019:43>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2019'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 1436
Is_Equal_1 := Fld_1 = Sym_1; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 1437 '<Symbol Contract_Cases at nodes.lkt:2019:53>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2019'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 1437
Is_Equal := Fld_1 = Sym_2; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Assoc_P_Is_Ghost_Code;
--# end
   --
   --  Primitives for Bare_Aspect_Clause
   --
--# property-start AspectClause.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2025'
pragma Warnings (Off, "is not referenced");
 function Aspect_Clause_P_Xref_Entry_Point
  (Node : Bare_Aspect_Clause
  )
   return Boolean
is
   Self : Bare_Aspect_Clause  := Bare_Aspect_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aspect_Clause_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_At_Clause
   --
      procedure Initialize_Fields_For_At_Clause
        (Self : Bare_At_Clause
         ; At_Clause_F_Name : Bare_Base_Id
         ; At_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.At_Clause_F_Name := At_Clause_F_Name;
            Self.At_Clause_F_Expr := At_Clause_F_Expr;
      end Initialize_Fields_For_At_Clause;
   function At_Clause_F_Name
     (Node : Bare_At_Clause) return Bare_Base_Id
   is
   begin
         return Node.At_Clause_F_Name;
   end;
   function At_Clause_F_Expr
     (Node : Bare_At_Clause) return Bare_Expr
   is
   begin
         return Node.At_Clause_F_Expr;
   end;
--# property-start AtClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2034'
pragma Warnings (Off, "is not referenced");
 function At_Clause_P_Xref_Equation
  (Node : Bare_At_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_At_Clause  := Bare_At_Clause (Node);
     Ent : Internal_Entity_At_Clause :=
       Internal_Entity_At_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_305 with Inline_Always;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_305 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_305;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1442 '<And at nodes.lkt:2035:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2035'
--# expr-start 1439 '<FieldAccess for sub_equation at nodes.lkt:2035:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2035'
--# expr-start 1438 '<FieldAccess for name at nodes.lkt:2035:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2035'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.At_Clause_F_Name, Info => Ent.Info);
--# expr-done 1438
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 1439
--# expr-start 1441 '<FieldAccess for sub_equation at nodes.lkt:2035:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2035'
--# expr-start 1440 '<FieldAccess for expr at nodes.lkt:2035:38>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2035'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.At_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1440
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 1441
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2035:9") else null)); 
--# expr-done 1442
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_305;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_305;
            raise;
      end;
   return Property_Result;
end At_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Attribute_Def_Clause
   --
      procedure Initialize_Fields_For_Attribute_Def_Clause
        (Self : Bare_Attribute_Def_Clause
         ; Attribute_Def_Clause_F_Attribute_Expr : Bare_Name
         ; Attribute_Def_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.Attribute_Def_Clause_F_Attribute_Expr := Attribute_Def_Clause_F_Attribute_Expr;
            Self.Attribute_Def_Clause_F_Expr := Attribute_Def_Clause_F_Expr;
      end Initialize_Fields_For_Attribute_Def_Clause;
   function Attribute_Def_Clause_F_Attribute_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Name
   is
   begin
         return Node.Attribute_Def_Clause_F_Attribute_Expr;
   end;
   function Attribute_Def_Clause_F_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Expr
   is
   begin
         return Node.Attribute_Def_Clause_F_Expr;
   end;
--# property-start AttributeDefClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2045'
pragma Warnings (Off, "is not referenced");
 function Attribute_Def_Clause_P_Xref_Equation
  (Node : Bare_Attribute_Def_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Def_Clause  := Bare_Attribute_Def_Clause (Node);
     Ent : Internal_Entity_Attribute_Def_Clause :=
       Internal_Entity_Attribute_Def_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_306 with Inline_Always;
      Local_Attr : Internal_Entity_Attribute_Ref;
Local_Rel_Name : Symbol_Type;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Is_Equal : Boolean;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_12 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Name;
Fld_16 : Internal_Entity_Base_Type_Decl;
Pred_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Bare_Expr;
Fld_23 : Logic_Var;
Sym_1 : Symbol_Type;
Fld_24 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_25 : Bare_Expr;
Fld_26 : Logic_Var;
Fld_27 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Logic_Equation;
Sym_3 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Sym_4 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Logic_Equation;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_4 : Boolean;
Is_Equal_5 : Boolean;
If_Result_3 : Boolean;
Is_Equal_6 : Boolean;
If_Result_4 : Boolean;
Is_Equal_7 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_306 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_306;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1503 '<Let at nodes.lkt:2046:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2046'
--# scope-start
--# expr-start 1444 '<Cast to AttributeRef at nodes.lkt:2046:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2046'
--# expr-start 1443 '<FieldAccess for attribute_expr at nodes.lkt:2046:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2046'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Ent.Info);
--# expr-done 1443
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1444
Local_Attr := Cast_Result; 
--# bind attr Local_Attr
--# expr-start 1502 '<Let at nodes.lkt:2047:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2047'
--# scope-start
--# expr-start 1446 '<FieldAccess for name_symbol at nodes.lkt:2047:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2047'
--# expr-start 1445 '<FieldAccess for attribute at nodes.lkt:2047:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2047'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Local_Attr.Node.Attribute_Ref_F_Attribute, Info => Local_Attr.Info);
--# expr-done 1445
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 1446
Local_Rel_Name := Fld_2; 
--# bind rel_name Local_Rel_Name
--# expr-start 1501 '<If at nodes.lkt:2049:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
--# expr-start 1447 '<Symbol Read at nodes.lkt:2049:24>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 1447
Is_Equal_7 := Local_Rel_Name = Sym_5; 
if Is_Equal_7 then
   If_Result_5 := True;
else
--# expr-start 1448 '<Symbol Write at nodes.lkt:2049:34>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 1448
Is_Equal_6 := Local_Rel_Name = Sym_6; 
if Is_Equal_6 then
   If_Result_4 := True;
else
--# expr-start 1449 '<Symbol Input at nodes.lkt:2049:45>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 1449
Is_Equal_5 := Local_Rel_Name = Sym_7; 
if Is_Equal_5 then
   If_Result_3 := True;
else
--# expr-start 1450 '<Symbol Output at nodes.lkt:2049:56>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 1450
Is_Equal_4 := Local_Rel_Name = Sym_8; 
   If_Result_3 := Is_Equal_4;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   --# expr-start 1465 '<And at nodes.lkt:2049:71>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
--# expr-start 1462 '<And at nodes.lkt:2050:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1453 '<FieldAccess for xref_no_overloading at nodes.lkt:2050:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1452 '<Cast to Name at nodes.lkt:2050:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1451 '<FieldAccess for expr at nodes.lkt:2050:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1451
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1452
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_4 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 1453
--# expr-start 1461 '<Predicate on BasicDecl.is_stream_subprogram_for_type at nodes.lkt:2050:71>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1456 '<FieldAccess for ref_var at nodes.lkt:2050:123>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1455 '<Cast to Name at nodes.lkt:2050:123>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1454 '<FieldAccess for expr at nodes.lkt:2050:123>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1454
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 1455
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2.Node);
--# end
--# expr-done 1456
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 1458 '<FieldAccess for name_designated_type at nodes.lkt:2050:153>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1457 '<FieldAccess for prefix at nodes.lkt:2050:153>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Local_Attr.Node.Attribute_Ref_F_Prefix, Info => Local_Attr.Info);
--# expr-done 1457
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1458
--# expr-start 1460 '<Eq at nodes.lkt:2050:189>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
--# expr-start 1459 '<Symbol Input at nodes.lkt:2050:201>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2050'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 1459
Is_Equal := Local_Rel_Name = Sym; 
--# expr-done 1460
Pred := Solver.Create_Predicate (Fld_6, Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (Fld_8, Is_Equal)); 
--# expr-done 1461
And_Pred := Create_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2050:13") else null)); 
--# expr-done 1462
--# expr-start 1464 '<FieldAccess for sub_equation at nodes.lkt:2051:15>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2051'
--# expr-start 1463 '<FieldAccess for prefix at nodes.lkt:2051:15>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2051'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Local_Attr.Node.Attribute_Ref_F_Prefix, Info => Local_Attr.Info);
--# expr-done 1463
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_9.Info);
--# end
--# expr-done 1464
And_Pred_1 := Create_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2049:71") else null)); 
--# expr-done 1465
   If_Result_6 := And_Pred_1;
else
   --# expr-start 1500 '<If at nodes.lkt:2049:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2049'
--# expr-start 1466 '<Symbol Put_Image at nodes.lkt:2052:26>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2052'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 1466
Is_Equal_3 := Local_Rel_Name = Sym_4; 
if Is_Equal_3 then
   --# expr-start 1479 '<And at nodes.lkt:2052:44>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2052'
--# expr-start 1476 '<And at nodes.lkt:2053:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1469 '<FieldAccess for xref_no_overloading at nodes.lkt:2053:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1468 '<Cast to Name at nodes.lkt:2053:13>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1467 '<FieldAccess for expr at nodes.lkt:2053:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1467
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1468
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_12 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_3.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 1469
--# expr-start 1475 '<Predicate on BasicDecl.is_put_image_subprogram_for_type at nodes.lkt:2053:71>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1472 '<FieldAccess for ref_var at nodes.lkt:2053:126>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1471 '<Cast to Name at nodes.lkt:2053:126>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1470 '<FieldAccess for expr at nodes.lkt:2053:126>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1470
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 1471
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_4.Node);
--# end
--# expr-done 1472
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 1474 '<FieldAccess for name_designated_type at nodes.lkt:2053:156>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
--# expr-start 1473 '<FieldAccess for prefix at nodes.lkt:2053:156>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2053'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Local_Attr.Node.Attribute_Ref_F_Prefix, Info => Local_Attr.Info);
--# expr-done 1473
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_16 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# end
--# expr-done 1474
Pred_1 := Solver.Create_Predicate (Fld_14, Create_Basic_Decl_P_Is_Put_Image_Subprogram_For_Type_0_Predicate (Fld_16)); 
--# expr-done 1475
And_Pred_2 := Create_And (Fld_12, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2053:13") else null)); 
--# expr-done 1476
--# expr-start 1478 '<FieldAccess for sub_equation at nodes.lkt:2054:15>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2054'
--# expr-start 1477 '<FieldAccess for prefix at nodes.lkt:2054:15>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2054'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Local_Attr.Node.Attribute_Ref_F_Prefix, Info => Local_Attr.Info);
--# expr-done 1477
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_17.Info);
--# end
--# expr-done 1478
And_Pred_3 := Create_And (And_Pred_2, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2052:44") else null)); 
--# expr-done 1479
   If_Result_2 := And_Pred_3;
else
   --# expr-start 1499 '<And at nodes.lkt:2055:14>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2055'
--# expr-start 1483 '<And at nodes.lkt:2056:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2056'
--# expr-start 1481 '<FieldAccess for sub_equation at nodes.lkt:2056:13>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2056'
--# expr-start 1480 '<FieldAccess for expr at nodes.lkt:2056:13>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2056'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 1480
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_19.Info);
--# end
--# expr-done 1481
--# expr-start 1482 '<FieldAccess for sub_equation at nodes.lkt:2056:42>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2056'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Attr.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Attr.Info);
--# end
--# expr-done 1482
And_Pred_4 := Create_And (Fld_20, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2056:13") else null)); 
--# expr-done 1483
--# expr-start 1498 '<If at nodes.lkt:2058:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
--# expr-start 1485 '<Eq at nodes.lkt:2058:16>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
--# expr-start 1484 '<Symbol External_Tag at nodes.lkt:2058:28>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_External_Tag); 
--# expr-done 1484
Is_Equal_2 := Local_Rel_Name = Sym_3; 
--# expr-done 1485
if Is_Equal_2 then
   --# expr-start 1490 '<Bind at nodes.lkt:2058:49>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
--# expr-start 1487 '<FieldAccess for expected_type_var at nodes.lkt:2058:53>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
--# expr-start 1486 '<FieldAccess for expr at nodes.lkt:2058:53>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Self.Attribute_Def_Clause_F_Expr;
--# expr-done 1486
      if Fld_22 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_22);
--# expr-done 1487
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 1489 '<FieldAccess for std_entity at nodes.lkt:2058:84>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1488 '<Symbol String at nodes.lkt:2058:100>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 1488
--# property-call-start AdaNode.std_entity
Fld_24 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 1489
Bind_Result := Solver.Create_Assign (Fld_23, Fld_24, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2058:49") else null)); 
--# expr-done 1490
   If_Result_1 := Bind_Result;
else
   --# expr-start 1497 '<If at nodes.lkt:2058:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2058'
--# expr-start 1492 '<Eq at nodes.lkt:2059:18>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
--# expr-start 1491 '<Symbol Address at nodes.lkt:2059:30>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 1491
Is_Equal_1 := Local_Rel_Name = Sym_2; 
--# expr-done 1492
if Is_Equal_1 then
   --# expr-start 1496 '<Bind at nodes.lkt:2059:46>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
--# expr-start 1494 '<FieldAccess for expected_type_var at nodes.lkt:2059:50>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
--# expr-start 1493 '<FieldAccess for expr at nodes.lkt:2059:50>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Self.Attribute_Def_Clause_F_Expr;
--# expr-done 1493
      if Fld_25 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_25);
--# expr-done 1494
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
--# expr-start 1495 '<FieldAccess for system_address_type at nodes.lkt:2059:81>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2059'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.system_address_type
Fld_27 := Libadalang.Implementation.Ada_Node_P_System_Address_Type (Node => Ent.Node);
--# end
--# expr-done 1495
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_27.Node,
         Info => Fld_27.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_26, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2059:46") else null)); 
--# expr-done 1496
   If_Result := Bind_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2060:18") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 1497
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1498
And_Pred_5 := Create_And (And_Pred_4, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2055:14") else null)); 
--# expr-done 1499
   If_Result_2 := And_Pred_5;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1500
   If_Result_6 := If_Result_2;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 1501
Let_Result := If_Result_6; Inc_Ref (Let_Result);
--# end
--# expr-done 1502
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1503
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_306;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_306;
            raise;
      end;
   return Property_Result;
end Attribute_Def_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Rep_Clause
   --
      procedure Initialize_Fields_For_Enum_Rep_Clause
        (Self : Bare_Enum_Rep_Clause
         ; Enum_Rep_Clause_F_Type_Name : Bare_Name
         ; Enum_Rep_Clause_F_Aggregate : Bare_Base_Aggregate
        ) is
      begin
            Self.Enum_Rep_Clause_F_Type_Name := Enum_Rep_Clause_F_Type_Name;
            Self.Enum_Rep_Clause_F_Aggregate := Enum_Rep_Clause_F_Aggregate;
      end Initialize_Fields_For_Enum_Rep_Clause;
   function Enum_Rep_Clause_F_Type_Name
     (Node : Bare_Enum_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Enum_Rep_Clause_F_Type_Name;
   end;
   function Enum_Rep_Clause_F_Aggregate
     (Node : Bare_Enum_Rep_Clause) return Bare_Base_Aggregate
   is
   begin
         return Node.Enum_Rep_Clause_F_Aggregate;
   end;
--# property-start EnumRepClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2071'
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Xref_Equation
  (Node : Bare_Enum_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Rep_Clause  := Bare_Enum_Rep_Clause (Node);
     Ent : Internal_Entity_Enum_Rep_Clause :=
       Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_307 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_307 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_307;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1505 '<FieldAccess for xref_type_equation at nodes.lkt:2072:5>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2072'
--# expr-start 1504 '<FieldAccess for type_name at nodes.lkt:2072:5>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2072'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 1504
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_1 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 1505
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_307;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_307;
            raise;
      end;
   return Property_Result;
end Enum_Rep_Clause_P_Xref_Equation;
--# end
--# property-start EnumRepClause.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2077'
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Params
  (Node : Bare_Enum_Rep_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Enum_Rep_Clause  := Bare_Enum_Rep_Clause (Node);
     Ent : Internal_Entity_Enum_Rep_Clause :=
       Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_308 with Inline_Always;
      Local_El : Internal_Entity_Enum_Literal_Decl_List;
Local_Ra : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Cast_Result_2 : Internal_Entity_Enum_Type_Def;
Fld_4 : Internal_Entity_Enum_Literal_Decl_List;
Fld_5 : Internal_Entity_Base_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Enum_Literal_Decl_List;
Fld_7 : Bare_Enum_Literal_Decl_List;
Fld_8 : Internal_Entity_Info;
Item_0 : Internal_Entity_Enum_Literal_Decl;
Bare_Item_0 : Bare_Enum_Literal_Decl;
Index_1 : Integer;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
Map_Result : Internal_Param_Actual_Array_Access;
Let_Result : Internal_Param_Actual_Array_Access;
Let_Result_1 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_308 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_308;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1522 '<Let at nodes.lkt:2079:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# scope-start
--# expr-start 1513 '<FieldAccess for enum_literals at nodes.lkt:2079:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1512 '<Cast to EnumTypeDef at nodes.lkt:2079:18>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1511 '<FieldAccess for type_def at nodes.lkt:2079:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1510 '<Cast to TypeDecl at nodes.lkt:2079:18>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1509 '<FieldAccess for root_type at nodes.lkt:2079:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1508 '<Cast to BaseTypeDecl at nodes.lkt:2079:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1507 '<FieldAccess for referenced_decl at nodes.lkt:2079:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
--# expr-start 1506 '<FieldAccess for type_name at nodes.lkt:2079:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2079'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 1506
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld.Node, Imprecise_Fallback => False, E_Info => Fld.Info);
--# end
--# expr-done 1507
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1508
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Cast_Result.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1509
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 1510
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Cast_Result_1.Node.Type_Decl_F_Type_Def, Info => Cast_Result_1.Info);
--# expr-done 1511
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Enum_Type_Def
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_2 := No_Entity_Enum_Type_Def;
   end if;
--# expr-done 1512
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Enum_Literal_Decl_List (Node => Cast_Result_2.Node.Enum_Type_Def_F_Enum_Literals, Info => Cast_Result_2.Info);
--# expr-done 1513
Local_El := Fld_4; 
--# bind el Local_El
--# expr-start 1521 '<Let at nodes.lkt:2081:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2081'
--# scope-start
--# expr-start 1515 '<FieldAccess for assocs at nodes.lkt:2081:18>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2081'
--# expr-start 1514 '<FieldAccess for aggregate at nodes.lkt:2081:18>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2081'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Base_Aggregate (Node => Ent.Node.Enum_Rep_Clause_F_Aggregate, Info => Ent.Info);
--# expr-done 1514
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Fld_5.Node.Base_Aggregate_F_Assocs, Info => Fld_5.Info);
--# expr-done 1515
Local_Ra := Fld_6; 
--# bind ra Local_Ra
--# expr-start 1520 '<Map at nodes.lkt:2083:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2083'
Saved := Local_El; 
Fld_7 := Saved.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
         Index_1 := 0;
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_7;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
Fld_8 := Saved.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind l Item_0
               --# bind i Index_1
      --# expr-start 1519 '<New ParamActual at nodes.lkt:2084:23>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2084'
--# expr-start 1517 '<FieldAccess for actual_for_param_at at nodes.lkt:2085:38>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2085'
      if Local_Ra.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1516 '<FieldAccess for name at nodes.lkt:2085:61>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2085'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Defining_Name (Node => Item_0.Node.Enum_Literal_Decl_F_Name, Info => Item_0.Info);
--# expr-done 1516
--# property-call-start AssocList.actual_for_param_at
Fld_11 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Local_Ra.Node, Param => Fld_10, Pos => Index_1, E_Info => Local_Ra.Info);
--# end
--# expr-done 1517
--# expr-start 1518 '<FieldAccess for name at nodes.lkt:2085:23>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2085'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Defining_Name (Node => Item_0.Node.Enum_Literal_Decl_F_Name, Info => Item_0.Info);
--# expr-done 1518
New_Struct := (Actual => Fld_11, Param => Fld_9); 
--# expr-done 1519
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
               Index_1 := Index_1 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1520
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1521
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1522
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_308;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_308;
            raise;
      end;
   return Property_Result;
end Enum_Rep_Clause_P_Params;
--# end
   --
   --  Primitives for Bare_Record_Rep_Clause
   --
      procedure Initialize_Fields_For_Record_Rep_Clause
        (Self : Bare_Record_Rep_Clause
         ; Record_Rep_Clause_F_Name : Bare_Name
         ; Record_Rep_Clause_F_At_Expr : Bare_Expr
         ; Record_Rep_Clause_F_Components : Bare_Ada_Node_List
        ) is
      begin
            Self.Record_Rep_Clause_F_Name := Record_Rep_Clause_F_Name;
            Self.Record_Rep_Clause_F_At_Expr := Record_Rep_Clause_F_At_Expr;
            Self.Record_Rep_Clause_F_Components := Record_Rep_Clause_F_Components;
      end Initialize_Fields_For_Record_Rep_Clause;
   function Record_Rep_Clause_F_Name
     (Node : Bare_Record_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Record_Rep_Clause_F_Name;
   end;
   function Record_Rep_Clause_F_At_Expr
     (Node : Bare_Record_Rep_Clause) return Bare_Expr
   is
   begin
         return Node.Record_Rep_Clause_F_At_Expr;
   end;
   function Record_Rep_Clause_F_Components
     (Node : Bare_Record_Rep_Clause) return Bare_Ada_Node_List
   is
   begin
         return Node.Record_Rep_Clause_F_Components;
   end;
--# property-start RecordRepClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2098'
pragma Warnings (Off, "is not referenced");
 function Record_Rep_Clause_P_Xref_Equation
  (Node : Bare_Record_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Rep_Clause  := Bare_Record_Rep_Clause (Node);
     Ent : Internal_Entity_Record_Rep_Clause :=
       Internal_Entity_Record_Rep_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_309 with Inline_Always;
            procedure Finalizer_Scope_2118 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_309 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_309;
            procedure Finalizer_Scope_2118 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2118;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1527 '<And at nodes.lkt:2099:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2099'
--# expr-start 1524 '<FieldAccess for xref_type_equation at nodes.lkt:2099:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2099'
--# expr-start 1523 '<FieldAccess for name at nodes.lkt:2099:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2099'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Record_Rep_Clause_F_Name, Info => Ent.Info);
--# expr-done 1523
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_1 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 1524
--# expr-start 1525 '<FieldAccess for at_expr at nodes.lkt:2099:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2099'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Record_Rep_Clause_F_At_Expr, Info => Ent.Info);
--# expr-done 1525
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 1526 '<FieldAccess for sub_equation at nodes.lkt:2100:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2100'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1526
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2118;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2100:50") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2099:9") else null)); 
--# expr-done 1527
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_309;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_309;
                     Finalizer_Scope_2118;
            raise;
      end;
   return Property_Result;
end Record_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Aspect_Spec
   --
      procedure Initialize_Fields_For_Aspect_Spec
        (Self : Bare_Aspect_Spec
         ; Aspect_Spec_F_Aspect_Assocs : Bare_Aspect_Assoc_List
        ) is
      begin
            Self.Aspect_Spec_F_Aspect_Assocs := Aspect_Spec_F_Aspect_Assocs;
      end Initialize_Fields_For_Aspect_Spec;
   function Aspect_Spec_F_Aspect_Assocs
     (Node : Bare_Aspect_Spec) return Bare_Aspect_Assoc_List
   is
   begin
         return Node.Aspect_Spec_F_Aspect_Assocs;
   end;
   --
   --  Primitives for Bare_Base_Assoc
   --
--# property-start '[dispatcher]BaseAssoc.assoc_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Assoc_P_Assoc_Expr
  (Node : Bare_Base_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Assoc  := Bare_Base_Assoc (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Assoc (Self.Kind) is
               when Ada_Contract_Case_Assoc =>
                  --# property-call-start ContractCaseAssoc.assoc_expr
                  Property_Result := Contract_Case_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Argument_Assoc =>
                  --# property-call-start PragmaArgumentAssoc.assoc_expr
                  Property_Result := Pragma_Argument_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Contract_Case_Assoc
   --
      procedure Initialize_Fields_For_Contract_Case_Assoc
        (Self : Bare_Contract_Case_Assoc
         ; Contract_Case_Assoc_F_Guard : Bare_Ada_Node
         ; Contract_Case_Assoc_F_Consequence : Bare_Expr
        ) is
      begin
            Self.Contract_Case_Assoc_F_Guard := Contract_Case_Assoc_F_Guard;
            Self.Contract_Case_Assoc_F_Consequence := Contract_Case_Assoc_F_Consequence;
      end Initialize_Fields_For_Contract_Case_Assoc;
   function Contract_Case_Assoc_F_Guard
     (Node : Bare_Contract_Case_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Contract_Case_Assoc_F_Guard;
   end;
   function Contract_Case_Assoc_F_Consequence
     (Node : Bare_Contract_Case_Assoc) return Bare_Expr
   is
   begin
         return Node.Contract_Case_Assoc_F_Consequence;
   end;
--# property-start ContractCaseAssoc.assoc_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2124'
pragma Warnings (Off, "is not referenced");
 function Contract_Case_Assoc_P_Assoc_Expr
  (Node : Bare_Contract_Case_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Contract_Case_Assoc  := Bare_Contract_Case_Assoc (Node);
     Ent : Internal_Entity_Contract_Case_Assoc :=
       Internal_Entity_Contract_Case_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1528 '<FieldAccess for consequence at nodes.lkt:2124:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2124'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Contract_Case_Assoc_F_Consequence, Info => Ent.Info);
--# expr-done 1528
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Contract_Case_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Pragma_Argument_Assoc
   --
      procedure Initialize_Fields_For_Pragma_Argument_Assoc
        (Self : Bare_Pragma_Argument_Assoc
         ; Pragma_Argument_Assoc_F_Name : Bare_Name
         ; Pragma_Argument_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Pragma_Argument_Assoc_F_Name := Pragma_Argument_Assoc_F_Name;
            Self.Pragma_Argument_Assoc_F_Expr := Pragma_Argument_Assoc_F_Expr;
      end Initialize_Fields_For_Pragma_Argument_Assoc;
   function Pragma_Argument_Assoc_F_Name
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Name
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Name;
   end;
   function Pragma_Argument_Assoc_F_Expr
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Expr
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Expr;
   end;
--# property-start PragmaArgumentAssoc.assoc_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2132'
pragma Warnings (Off, "is not referenced");
 function Pragma_Argument_Assoc_P_Assoc_Expr
  (Node : Bare_Pragma_Argument_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Argument_Assoc  := Bare_Pragma_Argument_Assoc (Node);
     Ent : Internal_Entity_Pragma_Argument_Assoc :=
       Internal_Entity_Pragma_Argument_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1529 '<FieldAccess for expr at nodes.lkt:2132:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2132'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Pragma_Argument_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 1529
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Argument_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Holder
   --
--# property-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.abstract_formal_params
                  Property_Result := Base_Subp_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.abstract_formal_params
                  Property_Result := Synthetic_Binary_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.abstract_formal_params
                  Property_Result := Synthetic_Unary_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_List =>
                  --# property-call-start ComponentList.abstract_formal_params
                  Property_Result := Component_List_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Unknown_Discriminant_Part =>
                  --# property-call-start DiscriminantPart.abstract_formal_params
                  Property_Result := Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Known_Discriminant_Part =>
                  --# property-call-start KnownDiscriminantPart.abstract_formal_params
                  Property_Result := Known_Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Completion_Formal_Params =>
                  --# property-call-start EntryCompletionFormalParams.abstract_formal_params
                  Property_Result := Entry_Completion_Formal_Params_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Part =>
                  --# property-call-start GenericFormalPart.abstract_formal_params
                  Property_Result := Generic_Formal_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.unpacked_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2155'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Unpacked_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_314 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_314 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_314;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1531 '<FieldAccess for unpack_formals at nodes.lkt:2156:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2156'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1530 '<FieldAccess for abstract_formal_params at nodes.lkt:2156:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2156'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1530
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 1531
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_314;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_314;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Unpacked_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2162'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_315 with Inline_Always;
            procedure Finalizer_Scope_2119 with Inline_Always;
            procedure Finalizer_Scope_2121 with Inline_Always;
      Match_0 : Internal_Entity_Component_List;
Match_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_2 : Internal_Entity;
Bare_Item_2 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Match_Prefix : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_1 : Internal_Entity_Component_List;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Match_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_315 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_315;
            procedure Finalizer_Scope_2119 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2119;
            procedure Finalizer_Scope_2121 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2121;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1538 '<Match at nodes.lkt:2162:56>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2162'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Formal_Param_Holder (Match_Prefix.Node.Kind) is
         when Ada_Component_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Component_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind r Match_0
--# expr-start 1536 '<FieldAccess for unpack_formals at nodes.lkt:2163:34>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2163'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1535 '<FilterMap at nodes.lkt:2163:54>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2163'
   --# expr-start 1532 '<FieldAccess for components at nodes.lkt:2163:54>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2163'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Match_0.Node.Component_List_F_Components, Info => Match_0.Info);
--# expr-done 1532
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null := Bare_Item_2 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_2); 
   If_Result := As_Entity;
end if;
                  Item_2 := If_Result; 
   --# scope-start
               --# bind e Item_2
         --# expr-start 1533 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1533
         if Is_A then
      --# expr-start 1534 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1534
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1535
--# property-call-start AdaNode.unpack_formals
Fld_3 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Map_Result);
--# end
--# expr-done 1536
Let_Result := Fld_3; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2119;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec | Ada_Known_Discriminant_Part .. Ada_Generic_Formal_Part =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 1537 '<FieldAccess for unpacked_formal_params at nodes.lkt:2164:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2164'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1537
Let_Result_1 := Fld_4; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2121;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 1538
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_315;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_315;
                     Finalizer_Scope_2119;
                     Finalizer_Scope_2121;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.match_param_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2167'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_316 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_316 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_316;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1540 '<FieldAccess for match_formals at nodes.lkt:2167:98>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2167'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1539 '<FieldAccess for abstract_formal_params at nodes.lkt:2168:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2168'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1539
--# property-call-start AdaNode.match_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp);
--# end
--# expr-done 1540
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_316;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_316;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Param_List;
--# end
--# property-start BaseFormalParamHolder.nb_min_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2174'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Min_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_317 with Inline_Always;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_317 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_317;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1546 '<length at nodes.lkt:2175:9>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
--# expr-start 1545 '<Map at nodes.lkt:2175:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
   --# expr-start 1542 '<FieldAccess for unpacked_formal_params at nodes.lkt:2175:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
--# expr-start 1541 '<as_bare_entity at nodes.lkt:2175:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1541
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1542
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
         --# expr-start 1544 '<FieldAccess for is_mandatory at nodes.lkt:2175:68>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
--# expr-start 1543 '<FieldAccess for formal_decl at nodes.lkt:2175:68>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2175'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1543
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_1.Node);
--# end
--# expr-done 1544
         if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_0;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1545
Len := Length (Map_Result); 
--# expr-done 1546
         Property_Result := Len;
   --# end
      Finalizer_Scope_317;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_317;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Nb_Min_Params;
--# end
--# property-start BaseFormalParamHolder.nb_max_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2180'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Max_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_318 with Inline_Always;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_318 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_318;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1549 '<length at nodes.lkt:2181:9>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2181'
--# expr-start 1548 '<FieldAccess for unpacked_formal_params at nodes.lkt:2181:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2181'
--# expr-start 1547 '<as_bare_entity at nodes.lkt:2181:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2181'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1547
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1548
Len := Length (Fld); 
--# expr-done 1549
         Property_Result := Len;
   --# end
      Finalizer_Scope_318;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_318;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Nb_Max_Params;
--# end
--# property-start BaseFormalParamHolder.paramless 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2186'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Paramless
  (Node : Bare_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Can_Be : Boolean
         := True
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind dottable_subp Dottable_Subp
   --# bind can_be Can_Be
   Property_Result : Boolean;
      Local_Nb_Params : Integer;
Fld : Integer;
Fld_1 : Integer;
If_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : Boolean;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1555 '<Let at nodes.lkt:2187:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2187'
--# scope-start
--# expr-start 1552 '<If at nodes.lkt:2187:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2187'
if Can_Be then
   --# expr-start 1550 '<FieldAccess for nb_min_params at nodes.lkt:2187:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2187'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Self);
--# end
--# expr-done 1550
   If_Result := Fld;
else
   --# expr-start 1551 '<FieldAccess for nb_max_params at nodes.lkt:2187:66>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2187'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Self);
--# end
--# expr-done 1551
   If_Result := Fld_1;
end if;
--# expr-done 1552
Local_Nb_Params := If_Result; 
--# bind nb_params Local_Nb_Params
if Dottable_Subp then
   --# expr-start 1553 '<Eq at nodes.lkt:2189:28>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2189'
Is_Equal := Local_Nb_Params = 1; 
--# expr-done 1553
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1554 '<Eq at nodes.lkt:2189:47>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2189'
Is_Equal_1 := Local_Nb_Params = 0; 
--# expr-done 1554
   If_Result_2 := Is_Equal_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1555
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Paramless;
--# end
--# property-start BaseFormalParamHolder.is_matching_param_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2195'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Is_Matching_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_320 with Inline_Always;
      Local_Bare : Internal_Entity_Base_Formal_Param_Holder;
Local_Match_List : Internal_Param_Match_Array_Access;
Local_Nb_Max_Params : Integer;
Local_Nb_Min_Params : Integer;
As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Param_Match_Array_Access;
Fld_1 : Integer;
Arith_Result : Integer;
Fld_2 : Integer;
If_Result_1 : Integer;
Fld_3 : Integer;
Arith_Result_1 : Integer;
Fld_4 : Integer;
As_Entity_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Integer;
Fld_8 : Bare_Assoc_List;
Len : Integer;
Comp_Result : Boolean;
Fld_9 : Boolean;
Quantifier_Result : Boolean;
If_Result_5 : Boolean;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl;
Fld_12 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Len_1 : Integer;
Is_Equal : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
Let_Result_2 : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_320 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Match_List);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_320;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1583 '<Let at nodes.lkt:2196:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2196'
--# scope-start
--# expr-start 1556 '<as_bare_entity at nodes.lkt:2196:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2196'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1556
Local_Bare := If_Result; 
--# bind bare Local_Bare
--# expr-start 1582 '<Let at nodes.lkt:2197:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2197'
--# scope-start
--# expr-start 1557 '<FieldAccess for match_param_list at nodes.lkt:2197:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2197'
      if Local_Bare.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.match_param_list
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Local_Bare.Node, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp, E_Info => Local_Bare.Info);
--# end
--# expr-done 1557
Local_Match_List := Fld; Inc_Ref (Local_Match_List);
--# bind match_list Local_Match_List
--# expr-start 1581 '<Let at nodes.lkt:2205:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2205'
--# scope-start
--# expr-start 1561 '<If at nodes.lkt:2205:29>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2205'
if Is_Dottable_Subp then
   --# expr-start 1559 '<Arithmetic - at nodes.lkt:2205:54>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2205'
--# expr-start 1558 '<FieldAccess for nb_max_params at nodes.lkt:2205:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2205'
      if Local_Bare.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Local_Bare.Node);
--# end
--# expr-done 1558
Arith_Result := (Fld_1 - 1); 
--# expr-done 1559
   If_Result_1 := Arith_Result;
else
   --# expr-start 1560 '<FieldAccess for nb_max_params at nodes.lkt:2205:84>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2205'
      if Local_Bare.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Local_Bare.Node);
--# end
--# expr-done 1560
   If_Result_1 := Fld_2;
end if;
--# expr-done 1561
Local_Nb_Max_Params := If_Result_1; 
--# bind nb_max_params Local_Nb_Max_Params
--# expr-start 1580 '<Let at nodes.lkt:2206:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2206'
--# scope-start
--# expr-start 1570 '<If at nodes.lkt:2206:29>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2206'
if Is_Dottable_Subp then
   --# expr-start 1566 '<FieldAccess for is_mandatory at nodes.lkt:2211:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2211'
--# expr-start 1565 '<FieldAccess for formal_decl at nodes.lkt:2211:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2211'
--# expr-start 1564 '<at at nodes.lkt:2211:13>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2211'
--# expr-start 1563 '<FieldAccess for unpacked_formal_params at nodes.lkt:2211:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2211'
--# expr-start 1562 '<as_bare_entity at nodes.lkt:2211:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2211'
Is_Null_1 := Self = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Base_Formal_Param_Holder;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Self); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 1562
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 1563
Get_Result := Get (Self, Fld_5, 0, True); 
--# expr-done 1564
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 1565
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_6.Node);
--# end
--# expr-done 1566
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1568 '<Arithmetic - at nodes.lkt:2212:16>' Arith_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2212'
--# expr-start 1567 '<FieldAccess for nb_min_params at nodes.lkt:2212:16>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2212'
      if Local_Bare.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Local_Bare.Node);
--# end
--# expr-done 1567
Arith_Result_1 := (Fld_3 - 1); 
--# expr-done 1568
   If_Result_4 := Arith_Result_1;
else
   --# expr-start 1569 '<FieldAccess for nb_min_params at nodes.lkt:2212:46>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2212'
      if Local_Bare.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Local_Bare.Node);
--# end
--# expr-done 1569
   If_Result_4 := Fld_4;
end if;
--# expr-done 1570
Local_Nb_Min_Params := If_Result_4; 
--# bind nb_min_params Local_Nb_Min_Params
--# expr-start 1571 '<length at nodes.lkt:2214:9>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
Fld_8 := Params.Node; 
Len := Length (Fld_8); 
--# expr-done 1571
Comp_Result := Len <= Local_Nb_Max_Params; 
if Comp_Result then
   --# expr-start 1573 '<AllQuantifier at nodes.lkt:2214:46>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Local_Match_List;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_0
      --# expr-start 1572 '<FieldAccess for has_matched at nodes.lkt:2214:68>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
Fld_9 := Item_0.Has_Matched;
--# expr-done 1572
      Quantifier_Result := Fld_9;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 1573
   If_Result_5 := Quantifier_Result;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 1579 '<Eq at nodes.lkt:2214:87>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
--# expr-start 1578 '<length at nodes.lkt:2214:87>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
--# expr-start 1577 '<Map at nodes.lkt:2214:87>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2214'
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Local_Match_List;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_1
         --# expr-start 1576 '<FieldAccess for is_mandatory at nodes.lkt:2215:20>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2215'
--# expr-start 1575 '<FieldAccess for formal_decl at nodes.lkt:2215:20>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2215'
--# expr-start 1574 '<FieldAccess for formal at nodes.lkt:2215:20>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2215'
Fld_10 := Item_1.Formal;
--# expr-done 1574
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_11 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 1575
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_11.Node);
--# end
--# expr-done 1576
         if Fld_12 then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1577
Len_1 := Length (Map_Result); 
--# expr-done 1578
Is_Equal := Len_1 = Local_Nb_Min_Params; 
--# expr-done 1579
   If_Result_6 := Is_Equal;
else
   If_Result_6 := False;
end if;
Let_Result := If_Result_6; 
--# end
--# expr-done 1580
Let_Result_1 := Let_Result; 
--# end
--# expr-done 1581
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 1582
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 1583
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_320;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_320;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Is_Matching_Param_List;
--# end
--# property-start BaseFormalParamHolder.param_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2222'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Types
  (Node : Bare_Base_Formal_Param_Holder
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_321 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_321 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_321;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 1588 '<Map at nodes.lkt:2222:54>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2222'
   --# expr-start 1584 '<FieldAccess for unpacked_formal_params at nodes.lkt:2222:54>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2222'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1584
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_0
      --# expr-start 1587 '<FieldAccess for real_designated_type at nodes.lkt:2223:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2223'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1586 '<FieldAccess for type_expression at nodes.lkt:2223:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2223'
--# expr-start 1585 '<FieldAccess for formal_decl at nodes.lkt:2223:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2223'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1585
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1586
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld_2, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1587
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1588
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_321;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_321;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Param_Types;
--# end
--# property-start BaseFormalParamHolder.param_modes 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2227'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Modes
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Mode_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Mode_Array_Access;
            procedure Finalizer_Scope_322 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_2 : Internal_Entity_Mode;
Map_Result : Internal_Entity_Mode_Array_Access;
            procedure Finalizer_Scope_322 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_322;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1593 '<Map at nodes.lkt:2227:46>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2227'
   --# expr-start 1589 '<FieldAccess for unpacked_formal_params at nodes.lkt:2227:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2227'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1589
      declare
         Map_Result_Vec : Internal_Entity_Mode_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_0
      --# expr-start 1592 '<FieldAccess for mode at nodes.lkt:2228:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2228'
--# expr-start 1591 '<Cast to ParamSpec at nodes.lkt:2228:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2228'
--# expr-start 1590 '<FieldAccess for formal_decl at nodes.lkt:2228:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2228'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1590
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 1591
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Mode (Node => Cast_Result.Node.Param_Spec_F_Mode, Info => Cast_Result.Info);
--# expr-done 1592
         declare
            Item_To_Append : constant Internal_Entity_Mode := Fld_2;
         begin
            Internal_Entity_Mode_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Mode_Array
           (Items_Count => Natural (Internal_Entity_Mode_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Mode_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Mode_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Mode_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1593
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_322;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_322;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Param_Modes;
--# end
--# property-start BaseFormalParamHolder.shed_subp_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2233'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Shed_Subp_Rebindings
  (Node : Bare_Base_Formal_Param_Holder
      ; R : Env_Rebindings
  )
   return Env_Rebindings
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
      --# bind self Self
   --# bind r R
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_2127 with Inline_Always;
            procedure Finalizer_Scope_2128 with Inline_Always;
      Var_Expr_0 : Lexical_Env := Empty_Env;
Var_Expr_39 : Env_Rebindings;
Fld : Env_Rebindings;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Env_Rebindings;
            procedure Finalizer_Scope_2127 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2127;
            procedure Finalizer_Scope_2128 is
            begin
                     Dec_Ref (Var_Expr_0);
            end Finalizer_Scope_2128;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1599 '<If at nodes.lkt:2233:65>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2233'
Var_Expr_39 := R;
if Var_Expr_39 /= null then
   --# scope-start
--# expr-start 1594 '<FieldAccess for new_env at nodes.lkt:2233:68>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2233'
      if Var_Expr_39 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_39.New_Env;
Inc_Ref (Fld_2);
--# expr-done 1594
Var_Expr_0 := Fld_2;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_Env then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 1596 '<IsA GenericSubpInstantiation at nodes.lkt:2234:17>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2234'
--# expr-start 1595 '<env_node at nodes.lkt:2234:17>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2234'
Env_Node := AST_Envs.Env_Node (Var_Expr_0); 
--# expr-done 1595
Is_A := Env_Node /= null 
and then Env_Node.Kind in Ada_Generic_Subp_Instantiation_Range; 
--# expr-done 1596
Scope_Result := Is_A;
   --# end
      Finalizer_Scope_2128;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_2127;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 1598 '<FieldAccess for shed_subp_rebindings at nodes.lkt:2235:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2235'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1597 '<FieldAccess for get_parent at nodes.lkt:2235:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2235'
      if R = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := R.Parent;
--# expr-done 1597
--# property-call-start BaseFormalParamHolder.shed_subp_rebindings
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Shed_Subp_Rebindings (Node => Self, R => Fld);
--# end
--# expr-done 1598
   If_Result := Fld_1;
else
   If_Result := R;
end if;
--# expr-done 1599
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2127;
                     Finalizer_Scope_2128;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Shed_Subp_Rebindings;
--# end
--# property-start BaseFormalParamHolder.corresponding_actual_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2240'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Corresponding_Actual_Param
  (Node : Bare_Base_Formal_Param_Holder
      ; Name : Internal_Entity_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_324 with Inline_Always;
            procedure Finalizer_Scope_2129 with Inline_Always;
      Local_Actual_Spec : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Base_Formal_Param_Holder;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Index_1 : Integer;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Bare_Defining_Name;
Fld_4 : Bare_Defining_Name;
Cast_Result : Bare_Defining_Name;
Is_Equal : Boolean;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result_1 : Internal_Entity_Defining_Name;
Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_324 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_324;
            procedure Finalizer_Scope_2129 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2129;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1613 '<Let at nodes.lkt:2241:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2241'
--# scope-start
--# expr-start 1600 '<FieldAccess for corresponding_actual at nodes.lkt:2241:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2241'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.corresponding_actual'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Corresponding_Actual (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1600
Local_Actual_Spec := Fld; 
--# bind actual_spec Local_Actual_Spec
--# expr-start 1612 '<If at nodes.lkt:2243:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2243'
--# expr-start 1601 '<Eq at nodes.lkt:2243:12>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2243'
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Local_Actual_Spec; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1601
if Is_Equiv then
   If_Result_1 := Name;
else
   --# expr-start 1611 '<at at nodes.lkt:2243:47>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2243'
--# expr-start 1610 '<Mapcat at nodes.lkt:2243:47>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2243'
   --# expr-start 1602 '<FieldAccess for unpacked_formal_params at nodes.lkt:2243:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2243'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1602
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
         Index_1 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind param Item_0
               --# bind i Index_1
      --# expr-start 1609 '<If at nodes.lkt:2244:27>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
--# expr-start 1605 '<Eq at nodes.lkt:2244:30>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
--# expr-start 1603 '<FieldAccess for node at nodes.lkt:2244:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
Fld_3 := Item_0.Node;
--# expr-done 1603
      Cast_Result := Fld_3;
--# expr-start 1604 '<FieldAccess for node at nodes.lkt:2244:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
Fld_4 := Name.Node;
--# expr-done 1604
Is_Equal := Cast_Result = Fld_4; 
--# expr-done 1605
if Is_Equal then
   --# expr-start 1608 '<ArrayLiteral at nodes.lkt:2244:59>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
--# expr-start 1607 '<at at nodes.lkt:2244:60>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
--# expr-start 1606 '<FieldAccess for unpacked_formal_params at nodes.lkt:2244:60>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2244'
      if Local_Actual_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Local_Actual_Spec.Node, E_Info => Local_Actual_Spec.Info);
--# end
--# expr-done 1606
Get_Result := Get (Self, Fld_2, Index_1, True); 
--# expr-done 1607
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Get_Result)); 
--# expr-done 1608
   If_Result := Array_Lit;
else
   If_Result := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 1609
         for Item_To_Append of
               If_Result.Items
         loop
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2129;
               Index_1 := Index_1 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1610
Get_Result_1 := Get (Self, Map_Result, 0, True); 
--# expr-done 1611
   If_Result_1 := Get_Result_1;
end if;
--# expr-done 1612
Let_Result := If_Result_1; 
--# end
--# expr-done 1613
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_324;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_324;
                     Finalizer_Scope_2129;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Corresponding_Actual_Param;
--# end
--# property-start BaseFormalParamHolder.real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2252'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Local_Prim_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Anonymous_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Formal_Type_Decl;
Match_1 : Internal_Entity;
Var_Expr_40 : Internal_Entity_Base_Type_Decl;
Var_Expr_41 : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Ada_Node;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
Fld_7 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Fld_11 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Fld_12 : Internal_Entity_Info;
Fld_13 : Env_Rebindings;
Fld_14 : Env_Rebindings;
Fld_15 : Internal_Entity_Info;
Fld_16 : Boolean;
Fld_17 : Internal_Entity;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Formal_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Fld_19 : Internal_Entity_Base_Type_Decl;
Fld_20 : Internal_Entity_Base_Type_Decl;
Fld_21 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity_Anonymous_Type_Decl;
Fld_22 : Internal_Entity_Type_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result_5 : Boolean;
Result_Var_3 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_23 : Internal_Entity_Base_Type_Decl;
Fld_24 : Bare_Base_Type_Decl;
Scope_Result_6 : Bare_Base_Type_Decl;
Result_Var_4 : Bare_Base_Type_Decl;
Fld_25 : Internal_Entity_Base_Type_Decl;
Fld_26 : Bare_Base_Type_Decl;
Scope_Result_7 : Bare_Base_Type_Decl;
Result_Var_5 : Bare_Base_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_27 : Bare_Base_Type_Decl;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1655 '<Let at nodes.lkt:2254:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2254'
--# scope-start
--# expr-start 1622 '<Cast to BaseTypeDecl at nodes.lkt:2254:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2254'
--# expr-start 1621 '<FieldAccess for entity_no_md at nodes.lkt:2254:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2254'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1616 '<FieldAccess for primitive at nodes.lkt:2255:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
--# expr-start 1615 '<FieldAccess for md at nodes.lkt:2255:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
--# expr-start 1614 '<FieldAccess for info at nodes.lkt:2255:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
Fld := Ent.Info;
--# expr-done 1614
Fld_1 := Fld.Md;
--# expr-done 1615
Fld_2 := Fld_1.Primitive;
--# expr-done 1616
--# expr-start 1618 '<FieldAccess for rebindings at nodes.lkt:2255:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
--# expr-start 1617 '<FieldAccess for info at nodes.lkt:2255:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
Fld_3 := Ent.Info;
--# expr-done 1617
Fld_4 := Fld_3.Rebindings;
--# expr-done 1618
--# expr-start 1620 '<FieldAccess for from_rebound at nodes.lkt:2255:59>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
--# expr-start 1619 '<FieldAccess for info at nodes.lkt:2255:59>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2255'
Fld_5 := Ent.Info;
--# expr-done 1619
Fld_6 := Fld_5.From_Rebound;
--# expr-done 1620
--# property-call-start AdaNode.entity_no_md
Fld_7 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Fld_2, Rebindings => Fld_4, From_Rebound => Fld_6);
--# end
--# expr-done 1621
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1622
Local_Prim_Type := Cast_Result; 
--# bind prim_type Local_Prim_Type
--# expr-start 1654 '<If at nodes.lkt:2258:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2258'
--# expr-start 1624 '<IsNull at nodes.lkt:2258:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2258'
--# expr-start 1623 '<FieldAccess for node at nodes.lkt:2258:12>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2258'
Fld_27 := Local_Prim_Type.Node;
--# expr-done 1623
Is_Null := Fld_27 = null; 
--# expr-done 1624
if Is_Null then
   If_Result_3 := Typ;
else
   --# expr-start 1653 '<If at nodes.lkt:2258:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2258'
--# expr-start 1629 '<Eq at nodes.lkt:2259:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
Var_Expr_40 := Local_Prim_Type;
if Var_Expr_40 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1626 '<FieldAccess for node at nodes.lkt:2259:14>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
--# expr-start 1625 '<FieldAccess for canonical_type at nodes.lkt:2259:14>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
      if Var_Expr_40.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_23 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_40.Node, Origin => Origin, E_Info => Var_Expr_40.Info);
--# end
--# expr-done 1625
Fld_24 := Fld_23.Node;
--# expr-done 1626
Scope_Result_6 := Fld_24;
   --# end
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
      Cast_Result_8 := Result_Var_4;
Var_Expr_41 := Typ;
if Var_Expr_41 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1628 '<FieldAccess for node at nodes.lkt:2259:50>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
--# expr-start 1627 '<FieldAccess for canonical_type at nodes.lkt:2259:50>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
      if Var_Expr_41.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_25 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_41.Node, Origin => Origin, E_Info => Var_Expr_41.Info);
--# end
--# expr-done 1627
Fld_26 := Fld_25.Node;
--# expr-done 1628
Scope_Result_7 := Fld_26;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := No_Bare_Ada_Node;
end if;
Is_Equal := Cast_Result_8 = Result_Var_5; 
--# expr-done 1629
if Is_Equal then
   --# expr-start 1642 '<Match at nodes.lkt:2259:82>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
--# expr-start 1639 '<FieldAccess for entity_no_md at nodes.lkt:2259:88>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2259'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1632 '<FieldAccess for primitive_real_type at nodes.lkt:2260:13>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2260'
--# expr-start 1631 '<FieldAccess for md at nodes.lkt:2260:13>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2260'
--# expr-start 1630 '<FieldAccess for info at nodes.lkt:2260:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2260'
Fld_8 := Ent.Info;
--# expr-done 1630
Fld_9 := Fld_8.Md;
--# expr-done 1631
Fld_10 := Fld_9.Primitive_Real_Type;
--# expr-done 1632
Var_Expr := Fld_10;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 1633 '<FieldAccess for node at nodes.lkt:2260:50>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2260'
Fld_11 := Typ.Node;
--# expr-done 1633
      Cast_Result_1 := Fld_11;
   Result_Var := Cast_Result_1;
end if;
--# expr-start 1636 '<FieldAccess for shed_subp_rebindings at nodes.lkt:2266:13>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2266'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1635 '<FieldAccess for rebindings at nodes.lkt:2266:39>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2266'
--# expr-start 1634 '<FieldAccess for info at nodes.lkt:2266:39>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2266'
Fld_12 := Ent.Info;
--# expr-done 1634
Fld_13 := Fld_12.Rebindings;
--# expr-done 1635
--# property-call-start BaseFormalParamHolder.shed_subp_rebindings
Fld_14 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Shed_Subp_Rebindings (Node => Self, R => Fld_13);
--# end
--# expr-done 1636
--# expr-start 1638 '<FieldAccess for from_rebound at nodes.lkt:2266:62>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2266'
--# expr-start 1637 '<FieldAccess for info at nodes.lkt:2266:62>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2266'
Fld_15 := Ent.Info;
--# expr-done 1637
Fld_16 := Fld_15.From_Rebound;
--# expr-done 1638
--# property-call-start AdaNode.entity_no_md
Fld_17 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Result_Var, Rebindings => Fld_14, From_Rebound => Fld_16);
--# end
--# expr-done 1639
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_17; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind ft Match_0
--# expr-start 1640 '<FieldAccess for get_actual at nodes.lkt:2272:48>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2272'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start FormalTypeDecl.get_actual
Fld_18 := Libadalang.Implementation.Formal_Type_Decl_P_Get_Actual (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 1640
Let_Result := Fld_18; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Concrete_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind other Match_1
--# expr-start 1641 '<Cast to BaseTypeDecl at nodes.lkt:2273:27>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2273'
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1641
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1642
   If_Result_2 := Match_Result;
else
   --# expr-start 1652 '<If at nodes.lkt:2258:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2258'
--# expr-start 1643 '<Cast to AnonymousTypeDecl at nodes.lkt:2278:14>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2278'
Cast_Expr_3 := Typ; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_7 := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 1643
Var_Expr_0 := Cast_Result_7;
if Var_Expr_0 /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
   --# bind td Var_Expr_0
--# expr-start 1646 '<Not at nodes.lkt:2279:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2279'
--# expr-start 1645 '<IsA AccessToSubpDef at nodes.lkt:2279:25>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2279'
--# expr-start 1644 '<FieldAccess for type_def at nodes.lkt:2279:25>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2279'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Type_Def (Node => Var_Expr_0.Node.Type_Decl_F_Type_Def, Info => Var_Expr_0.Info);
--# expr-done 1644
Is_A := Fld_22.Node /= null 
and then Fld_22.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 1645
Not_Val := not (Is_A); 
--# expr-done 1646
Scope_Result_5 := Not_Val;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 1647 '<FieldAccess for accessed_type at nodes.lkt:2280:16>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2280'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1647
Var_Expr_1 := Fld_19;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_1
--# expr-start 1648 '<FieldAccess for real_type at nodes.lkt:2281:21>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2281'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_20 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1648
Var_Expr_2 := Fld_20;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind rat Var_Expr_2
--# expr-start 1651 '<If at nodes.lkt:2282:26>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2282'
--# expr-start 1649 '<Eq at nodes.lkt:2282:29>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2282'
Cast_Expr_1 := Var_Expr_1; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr_2; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_5, Cast_Result_6); 
--# expr-done 1649
if Is_Equiv then
   If_Result := Typ;
else
   --# expr-start 1650 '<FieldAccess for anonymous_access_type at nodes.lkt:2282:53>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2282'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_21 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1650
   If_Result := Fld_21;
end if;
--# expr-done 1651
Scope_Result_3 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_4 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_4;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var_2;
else
   If_Result_1 := Typ;
end if;
--# expr-done 1652
   If_Result_2 := If_Result_1;
end if;
--# expr-done 1653
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1654
Let_Result_2 := If_Result_3; 
--# end
--# expr-done 1655
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Real_Type;
--# end
--# property-start BaseFormalParamHolder.real_designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2347'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Designated_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Local_Md : Internal_Metadata;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Local_Rt : Internal_Entity_Base_Type_Decl;
Local_Base_Rebindings : Env_Rebindings;
Var_Expr_42 : Internal_Entity_Base_Type_Decl;
Var_Expr_43 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_7 : Env_Rebindings;
Scope_Result_1 : Env_Rebindings;
Result_Var_1 : Env_Rebindings;
Fld_8 : Bare_Type_Expr;
Fld_9 : Internal_Entity_Info;
Fld_10 : Env_Rebindings;
Fld_11 : Env_Rebindings;
Fld_12 : Internal_Entity_Info;
Fld_13 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Type_Expr;
Fld_14 : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1685 '<Let at nodes.lkt:2348:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2348'
--# scope-start
--# expr-start 1657 '<FieldAccess for md at nodes.lkt:2348:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2348'
--# expr-start 1656 '<FieldAccess for info at nodes.lkt:2348:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2348'
Fld := Ent.Info;
--# expr-done 1656
Fld_1 := Fld.Md;
--# expr-done 1657
Local_Md := Fld_1; 
--# bind md Local_Md
--# expr-start 1660 '<Cast to BaseTypeDecl at nodes.lkt:2350:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2350'
--# expr-start 1658 '<FieldAccess for designated_type at nodes.lkt:2350:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2350'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1658
Var_Expr_42 := Fld_2;
if Var_Expr_42 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1659 '<FieldAccess for without_md at nodes.lkt:2350:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2350'
      if Var_Expr_42.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_3 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr_42.Node, E_Info => Var_Expr_42.Info);
--# end
--# expr-done 1659
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1660
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 1684 '<If at nodes.lkt:2360:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2360'
--# expr-start 1661 '<FieldAccess for dottable_subp at nodes.lkt:2360:16>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2360'
Fld_15 := Local_Md.Dottable_Subp;
--# expr-done 1661
if Fld_15 then
   If_Result_3 := Var_Expr_0;
else
   --# expr-start 1683 '<Let at nodes.lkt:2361:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2361'
--# scope-start
--# expr-start 1662 '<FieldAccess for real_type at nodes.lkt:2361:26>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2361'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr_0, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1662
Local_Rt := Fld_4; 
--# bind rt Local_Rt
--# expr-start 1682 '<Let at nodes.lkt:2362:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
--# scope-start
--# expr-start 1665 '<as_bare_entity at nodes.lkt:2362:39>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
--# expr-start 1664 '<Cast to BaseTypeDecl at nodes.lkt:2362:39>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
--# expr-start 1663 '<FieldAccess for primitive_real_type at nodes.lkt:2362:39>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
Fld_5 := Local_Md.Primitive_Real_Type;
--# expr-done 1663
   if Fld_5 = null
      or else Fld_5.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_5;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1664
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1665
Var_Expr_43 := If_Result;
if Var_Expr_43 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1668 '<FieldAccess for find_base_type_rebindings at nodes.lkt:2362:39>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
      if Var_Expr_43.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1667 '<Cast to BaseTypeDecl at nodes.lkt:2362:121>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
--# expr-start 1666 '<FieldAccess for primitive at nodes.lkt:2362:121>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2362'
Fld_6 := Local_Md.Primitive;
--# expr-done 1666
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Fld_6;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1667
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Var_Expr_43.Node, Target => Cast_Result_2, Origin => Origin, E_Info => Var_Expr_43.Info);
--# end
--# expr-done 1668
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Local_Base_Rebindings := Result_Var_1; 
--# bind base_rebindings Local_Base_Rebindings
--# expr-start 1681 '<If at nodes.lkt:2364:17>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
--# expr-start 1669 '<Eq at nodes.lkt:2364:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
Cast_Expr := Var_Expr_0; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Local_Rt; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 1669
if Is_Equiv then
   --# expr-start 1680 '<If at nodes.lkt:2364:33>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
--# expr-start 1670 '<IsNull at nodes.lkt:2364:36>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
Is_Equal := Local_Base_Rebindings = null; 
--# expr-done 1670
if Is_Equal then
   If_Result_1 := Var_Expr_0;
else
   --# expr-start 1679 '<FieldAccess for designated_type at nodes.lkt:2364:72>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
--# expr-start 1678 '<New TypeExpr.entity at nodes.lkt:2364:72>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2364'
--# expr-start 1676 '<New EntityInfo at nodes.lkt:2365:41>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2365'
--# expr-start 1672 '<FieldAccess for from_rebound at nodes.lkt:2366:130>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2366'
--# expr-start 1671 '<FieldAccess for info at nodes.lkt:2366:130>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2366'
Fld_12 := Typ.Info;
--# expr-done 1671
Fld_13 := Fld_12.From_Rebound;
--# expr-done 1672
--# expr-start 1675 '<FieldAccess for insert_rebindings at nodes.lkt:2366:55>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2366'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1674 '<FieldAccess for rebindings at nodes.lkt:2366:78>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2366'
--# expr-start 1673 '<FieldAccess for info at nodes.lkt:2366:78>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2366'
Fld_9 := Typ.Info;
--# expr-done 1673
Fld_10 := Fld_9.Rebindings;
--# expr-done 1674
--# property-call-start AdaNode.insert_rebindings
Fld_11 := Libadalang.Implementation.Ada_Node_P_Insert_Rebindings (Node => Self, Base => Fld_10, To_Insert => Local_Base_Rebindings);
--# end
--# expr-done 1675
New_Struct := (From_Rebound => Fld_13, Md => No_Metadata, Rebindings => Fld_11); 
--# expr-done 1676
--# expr-start 1677 '<FieldAccess for node at nodes.lkt:2365:26>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2365'
Fld_8 := Typ.Node;
--# expr-done 1677
New_Struct_1 := (Info => New_Struct, Node => Fld_8); 
--# expr-done 1678
      if New_Struct_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => New_Struct_1.Node, Origin => Origin, E_Info => New_Struct_1.Info);
--# end
--# expr-done 1679
   If_Result_1 := Fld_14;
end if;
--# expr-done 1680
   If_Result_2 := If_Result_1;
else
   If_Result_2 := Local_Rt;
end if;
--# expr-done 1681
Let_Result := If_Result_2; 
--# end
--# expr-done 1682
Let_Result_1 := Let_Result; 
--# end
--# expr-done 1683
   If_Result_3 := Let_Result_1;
end if;
--# expr-done 1684
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result_2 := Result_Var_2; 
--# end
--# expr-done 1685
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Real_Designated_Type;
--# end
--# property-start BaseFormalParamHolder.call_argument_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2378'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Call_Argument_Equation
  (Node : Bare_Base_Formal_Param_Holder
      ; Param : Internal_Entity_Base_Formal_Param_Decl
      ; Arg : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind arg Arg
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_327 with Inline_Always;
      Local_Param_Type : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_327 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_327;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1692 '<Let at nodes.lkt:2379:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2379'
--# scope-start
--# expr-start 1687 '<FieldAccess for real_designated_type at nodes.lkt:2379:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2379'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1686 '<FieldAccess for type_expression at nodes.lkt:2379:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2379'
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Param.Node, E_Info => Param.Info);
--# end
--# expr-done 1686
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1687
Local_Param_Type := Fld_1; 
--# bind param_type Local_Param_Type
--# expr-start 1691 '<And at nodes.lkt:2381:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2381'
--# expr-start 1689 '<Bind at nodes.lkt:2381:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2381'
--# expr-start 1688 '<FieldAccess for expected_type_var at nodes.lkt:2381:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2381'
      if Arg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Arg.Node);
--# expr-done 1688
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Cast_Expr := Local_Param_Type; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2381:9") else null)); 
--# expr-done 1689
--# expr-start 1690 '<FieldAccess for matches_expected_formal_type at nodes.lkt:2381:79>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2381'
      if Arg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_3 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Arg.Node, Origin => Origin);
--# end
--# expr-done 1690
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2381:9") else null)); 
--# expr-done 1691
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1692
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_327;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_327;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Call_Argument_Equation;
--# end
--# property-start BaseFormalParamHolder.match_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2386'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Ignore_First_Param : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind ignore_first_param Ignore_First_Param
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_328 with Inline_Always;
            procedure Finalizer_Scope_2141 with Inline_Always;
            procedure Finalizer_Scope_2143 with Inline_Always;
      Local_Self_Params : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_0 : Internal_Entity_Defining_Name_Array_Access;
Local_Other_Params : Internal_Entity_Defining_Name_Array_Access;
Local_Self_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Var_Expr_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Local_Other_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Var_Expr_44 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Index_2 : Integer;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Index_5 : Integer;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Len : Integer;
Len_1 : Integer;
Is_Equal_2 : Boolean;
Len_2 : Integer;
Len_3 : Integer;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Index_7 : Integer;
Not_Val_2 : Boolean;
Fld_4 : Internal_Entity_Name;
Get_Result : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Bare_Name;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Get_Result_1 : Internal_Entity_Base_Type_Decl;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Quantifier_Result : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
Let_Result_2 : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_328 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Self_Params);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Other_Params);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Self_Types);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Local_Other_Types);
            end Finalizer_Scope_328;
            procedure Finalizer_Scope_2141 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2141;
            procedure Finalizer_Scope_2143 is
            begin
                     Dec_Ref (Var_Expr_3);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2143;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1724 '<Let at nodes.lkt:2389:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2389'
--# scope-start
--# expr-start 1693 '<FieldAccess for unpacked_formal_params at nodes.lkt:2389:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2389'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1693
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind params Var_Expr_0
--# expr-start 1697 '<If at nodes.lkt:2390:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2390'
if Ignore_First_Param then
   --# expr-start 1696 '<Map at nodes.lkt:2390:52>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2390'
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
               --# bind i Index_2
         --# expr-start 1695 '<Not at nodes.lkt:2390:77>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2390'
--# expr-start 1694 '<Eq at nodes.lkt:2390:77>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2390'
Is_Equal := Index_2 = 0; 
--# expr-done 1694
Not_Val := not (Is_Equal); 
--# expr-done 1695
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_1;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1696
   If_Result := Map_Result;
else
   If_Result := Var_Expr_0;
end if;
      Inc_Ref (If_Result);
--# expr-done 1697
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2141;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Self_Params := Result_Var; Inc_Ref (Local_Self_Params);
--# bind self_params Local_Self_Params
--# expr-start 1723 '<Let at nodes.lkt:2392:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2392'
--# scope-start
--# expr-start 1698 '<FieldAccess for unpacked_formal_params at nodes.lkt:2392:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2392'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1698
Local_Other_Params := Fld_1; Inc_Ref (Local_Other_Params);
--# bind other_params Local_Other_Params
--# expr-start 1722 '<Let at nodes.lkt:2393:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2393'
--# scope-start
--# expr-start 1699 '<FieldAccess for param_types at nodes.lkt:2393:26>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2393'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1699
Var_Expr_3 := Fld_2;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Internal_Entity_Base_Type_Decl_Array_Type then
   --# scope-start
   --# bind types Var_Expr_3
--# expr-start 1703 '<If at nodes.lkt:2394:24>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2394'
if Ignore_First_Param then
   --# expr-start 1702 '<Map at nodes.lkt:2394:51>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2394'
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
         Index_5 := 0;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Var_Expr_3;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_4
               --# bind i Index_5
         --# expr-start 1701 '<Not at nodes.lkt:2394:75>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2394'
--# expr-start 1700 '<Eq at nodes.lkt:2394:75>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2394'
Is_Equal_1 := Index_5 = 0; 
--# expr-done 1700
Not_Val_1 := not (Is_Equal_1); 
--# expr-done 1701
         if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
               Index_5 := Index_5 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1702
   If_Result_1 := Map_Result_1;
else
   If_Result_1 := Var_Expr_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1703
Scope_Result_1 := If_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2143;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Self_Types := Result_Var_1; Inc_Ref (Local_Self_Types);
--# bind self_types Local_Self_Types
--# expr-start 1721 '<Let at nodes.lkt:2396:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2396'
--# scope-start
--# expr-start 1704 '<FieldAccess for param_types at nodes.lkt:2396:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2396'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 1704
Local_Other_Types := Fld_3; Inc_Ref (Local_Other_Types);
--# bind other_types Local_Other_Types
--# expr-start 1707 '<Eq at nodes.lkt:2398:9>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
--# expr-start 1705 '<length at nodes.lkt:2398:9>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
Len := Length (Local_Self_Params); 
--# expr-done 1705
--# expr-start 1706 '<length at nodes.lkt:2398:33>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
Len_1 := Length (Local_Other_Params); 
--# expr-done 1706
Is_Equal_2 := Len = Len_1; 
--# expr-done 1707
if Is_Equal_2 then
   --# expr-start 1710 '<Eq at nodes.lkt:2398:59>' Is_Equal_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
--# expr-start 1708 '<length at nodes.lkt:2398:59>' Len_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
Len_2 := Length (Local_Self_Types); 
--# expr-done 1708
--# expr-start 1709 '<length at nodes.lkt:2398:82>' Len_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
Len_3 := Length (Local_Other_Types); 
--# expr-done 1709
Is_Equal_3 := Len_2 = Len_3; 
--# expr-done 1710
   If_Result_2 := Is_Equal_3;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 1720 '<AllQuantifier at nodes.lkt:2398:107>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2398'
      Quantifier_Result := True;
         Index_7 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Self_Params;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_6
               --# bind i Index_7
--# expr-start 1711 '<Not at nodes.lkt:2400:17>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
Not_Val_2 := not (Match_Names); 
--# expr-done 1711
if Not_Val_2 then
   If_Result_3 := True;
else
   --# expr-start 1716 '<FieldAccess for matches at nodes.lkt:2400:36>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
--# expr-start 1712 '<FieldAccess for name at nodes.lkt:2400:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_6.Node.Defining_Name_F_Name, Info => Item_6.Info);
--# expr-done 1712
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1715 '<FieldAccess for node at nodes.lkt:2400:51>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
--# expr-start 1714 '<FieldAccess for name at nodes.lkt:2400:51>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
--# expr-start 1713 '<at at nodes.lkt:2400:51>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2400'
Get_Result := Get (Self, Local_Other_Params, Index_7, True); 
--# expr-done 1713
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Get_Result.Node.Defining_Name_F_Name, Info => Get_Result.Info);
--# expr-done 1714
Fld_6 := Fld_5.Node;
--# expr-done 1715
--# property-call-start Name.matches
Fld_7 := Libadalang.Implementation.Name_P_Matches (Node => Fld_4.Node, N => Fld_6);
--# end
--# expr-done 1716
   If_Result_3 := Fld_7;
end if;
if If_Result_3 then
--# expr-start 1717 '<at at nodes.lkt:2401:19>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2401'
Get_Result_1 := Get (Self, Local_Self_Types, Index_7, True); 
--# expr-done 1717
Var_Expr_44 := Get_Result_1;
if Var_Expr_44 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1719 '<FieldAccess for matching_type at nodes.lkt:2401:19>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2401'
      if Var_Expr_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1718 '<at at nodes.lkt:2401:49>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2401'
Get_Result_2 := Get (Self, Local_Other_Types, Index_7, True); 
--# expr-done 1718
--# property-call-start BaseTypeDecl.matching_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr_44.Node, Expected_Type => Get_Result_2, Origin => Origin, E_Info => Var_Expr_44.Info);
--# end
--# expr-done 1719
Scope_Result_2 := Fld_8;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
else
   If_Result_4 := False;
end if;
      Quantifier_Result := If_Result_4;
   --# end
               exit when not Quantifier_Result;
               Index_7 := Index_7 + 1;
         end loop;
      end;
--# expr-done 1720
   If_Result_5 := Quantifier_Result;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 1721
Let_Result_1 := Let_Result; 
--# end
--# expr-done 1722
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 1723
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 1724
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_328;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_328;
                     Finalizer_Scope_2141;
                     Finalizer_Scope_2143;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Formal_Params;
--# end
--# property-start '[dispatcher]BaseFormalParamHolder.corresponding_actual' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Corresponding_Actual
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Entry_Spec .. Ada_Subp_Spec | Ada_Component_List .. Ada_Generic_Formal_Part =>
                  --# property-call-start BaseFormalParamHolder.corresponding_actual
                  Property_Result := Base_Formal_Param_Holder_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.corresponding_actual
                  Property_Result := Synthetic_Binary_Spec_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.corresponding_actual
                  Property_Result := Synthetic_Unary_Spec_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Holder_P_Corresponding_Actual;
--# end
--# property-start '[dispatcher]BaseFormalParamHolder.match_other' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Component_List .. Ada_Generic_Formal_Part =>
                  --# property-call-start BaseFormalParamHolder.match_other
                  Property_Result := Base_Formal_Param_Holder_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.match_other
                  Property_Result := Base_Subp_Spec_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Holder_P_Match_Other;
--# end
--# property-start BaseFormalParamHolder.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2409'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Corresponding_Actual
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_45 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Result_Var : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1726 '<Cast to BasicDecl at nodes.lkt:2409:65>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2409'
--# expr-start 1725 '<parent at nodes.lkt:2409:65>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2409'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1725
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 1726
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 1727 '<FieldAccess for corresponding_actual at nodes.lkt:2410:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2410'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.corresponding_actual'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Corresponding_Actual (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1727
Var_Expr_45 := Fld_1;
if Var_Expr_45 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1728 '<FieldAccess for formal_param_holder_or_null at nodes.lkt:2410:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2410'
      if Var_Expr_45.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr_45.Node, E_Info => Var_Expr_45.Info);
--# end
--# expr-done 1728
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Formal_Param_Holder;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Ent;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Corresponding_Actual;
--# end
--# property-start BaseFormalParamHolder.match_other 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2418'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder  := Bare_Base_Formal_Param_Holder (Node);
     Ent : Internal_Entity_Base_Formal_Param_Holder :=
       Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1729 '<FieldAccess for match_formal_params at nodes.lkt:2419:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2419'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Ent.Node, Other => Other, Match_Names => Match_Names, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1729
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Holder_P_Match_Other;
--# end
   --
   --  Primitives for Bare_Base_Subp_Spec
   --
--# property-start '[dispatcher]BaseSubpSpec.name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Name
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.name
                  Property_Result := Entry_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.name
                  Property_Result := Enum_Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.name
                  Property_Result := Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.name
                  Property_Result := Synthetic_Binary_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.name
                  Property_Result := Synthetic_Unary_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Name;
--# end
--# property-start '[dispatcher]BaseSubpSpec.returns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Returns
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.returns
                  Property_Result := Entry_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.returns
                  Property_Result := Enum_Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.returns
                  Property_Result := Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Binary_Spec =>
                  --# property-call-start SyntheticBinarySpec.returns
                  Property_Result := Synthetic_Binary_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Unary_Spec =>
                  --# property-call-start SyntheticUnarySpec.returns
                  Property_Result := Synthetic_Unary_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Returns;
--# end
--# property-start BaseSubpSpec.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2437'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Abstract_Formal_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_333 with Inline_Always;
      Fld : Internal_Entity_Param_Spec_Array_Access;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_333 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_333;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1732 '<Map at nodes.lkt:2438:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2438'
   --# expr-start 1730 '<FieldAccess for params at nodes.lkt:2438:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2438'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1730
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Param_Spec_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 1731 '<Cast to BaseFormalParamDecl at nodes.lkt:2438:34>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2438'
Cast_Expr := Item_0; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1731
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1732
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_333;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_333;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Abstract_Formal_Params;
--# end
--# property-start BaseSubpSpec.match_return_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2441'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Self_Ret : Internal_Entity_Base_Type_Decl;
Local_Other_Ret : Internal_Entity_Base_Type_Decl;
Var_Expr_46 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Fld_3 : Internal_Entity_Type_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Type_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Type_Expr;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1749 '<Let at nodes.lkt:2447:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2447'
--# scope-start
--# expr-start 1733 '<FieldAccess for return_type at nodes.lkt:2447:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2447'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1733
Local_Self_Ret := Fld; 
--# bind self_ret Local_Self_Ret
--# expr-start 1748 '<Let at nodes.lkt:2448:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2448'
--# scope-start
--# expr-start 1734 '<FieldAccess for return_type at nodes.lkt:2448:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2448'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 1734
Local_Other_Ret := Fld_1; 
--# bind other_ret Local_Other_Ret
--# expr-start 1736 '<IsNull at nodes.lkt:2451:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2451'
--# expr-start 1735 '<FieldAccess for returns at nodes.lkt:2451:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2451'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1735
Is_Null := Fld_2.Node = null; 
--# expr-done 1736
if Is_Null then
   --# expr-start 1738 '<IsNull at nodes.lkt:2451:40>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2451'
--# expr-start 1737 '<FieldAccess for returns at nodes.lkt:2451:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2451'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1737
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 1738
   If_Result := Is_Null_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_3 := True;
else
--# expr-start 1741 '<Not at nodes.lkt:2453:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
--# expr-start 1740 '<IsNull at nodes.lkt:2453:17>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
--# expr-start 1739 '<FieldAccess for returns at nodes.lkt:2453:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1739
Is_Null_2 := Fld_4.Node = null; 
--# expr-done 1740
Not_Val := not (Is_Null_2); 
--# expr-done 1741
if Not_Val then
   --# expr-start 1744 '<Not at nodes.lkt:2453:44>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
--# expr-start 1743 '<IsNull at nodes.lkt:2453:48>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
--# expr-start 1742 '<FieldAccess for returns at nodes.lkt:2453:48>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2453'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1742
Is_Null_3 := Fld_5.Node = null; 
--# expr-done 1743
Not_Val_1 := not (Is_Null_3); 
--# expr-done 1744
   If_Result_1 := Not_Val_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 1747 '<bind at nodes.lkt:2454:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2454'
--# expr-start 1745 '<FieldAccess for origin_node at nodes.lkt:2454:31>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2454'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1745
Bound_Origin := Fld_6; 
Var_Expr_46 := Local_Self_Ret;
if Var_Expr_46 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1746 '<FieldAccess for matching_type at nodes.lkt:2456:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2456'
      if Var_Expr_46.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr_46.Node, Expected_Type => Local_Other_Ret, Origin => Bound_Origin, E_Info => Var_Expr_46.Info);
--# end
--# expr-done 1746
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 1747
   If_Result_2 := Dyn_Var_Bind_Result;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 1748
Let_Result_1 := Let_Result; 
--# end
--# expr-done 1749
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Return_Type;
--# end
--# property-start BaseSubpSpec.match_signature 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2480'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Signature
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Match_Name : Boolean
      ; Use_Entity_Info : Boolean
         := True
      ; Ignore_First_Param : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_name Match_Name
   --# bind use_entity_info Use_Entity_Info
   --# bind ignore_first_param Ignore_First_Param
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Ent : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Defining_Name;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
If_Result_2 : Boolean;
Fld_5 : Boolean;
If_Result_3 : Boolean;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1760 '<Let at nodes.lkt:2481:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2481'
--# scope-start
--# expr-start 1751 '<If at nodes.lkt:2481:19>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2481'
if Use_Entity_Info then
   If_Result_1 := Ent;
else
   --# expr-start 1750 '<as_bare_entity at nodes.lkt:2481:53>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2481'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1750
   If_Result_1 := If_Result;
end if;
--# expr-done 1751
Local_Ent := If_Result_1; 
--# bind ent Local_Ent
--# expr-start 1752 '<Not at nodes.lkt:2485:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
Not_Val := not (Match_Name); 
--# expr-done 1752
if Not_Val then
   If_Result_2 := True;
else
   --# expr-start 1757 '<FieldAccess for matches at nodes.lkt:2485:31>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
--# expr-start 1754 '<FieldAccess for node at nodes.lkt:2485:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
--# expr-start 1753 '<FieldAccess for name at nodes.lkt:2485:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
      if Local_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Local_Ent.Node, E_Info => Local_Ent.Info);
--# end
--# expr-done 1753
Fld_1 := Fld.Node;
--# expr-done 1754
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1756 '<FieldAccess for node at nodes.lkt:2485:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
--# expr-start 1755 '<FieldAccess for name at nodes.lkt:2485:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2485'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1755
Fld_3 := Fld_2.Node;
--# expr-done 1756
      Cast_Result := Fld_3;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_1, N => Cast_Result);
--# end
--# expr-done 1757
   If_Result_2 := Fld_4;
end if;
if If_Result_2 then
   --# expr-start 1758 '<FieldAccess for match_return_type at nodes.lkt:2486:15>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2486'
      if Local_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Return_Type (Node => Local_Ent.Node, Other => Other, Origin => Origin, E_Info => Local_Ent.Info);
--# end
--# expr-done 1758
   If_Result_3 := Fld_5;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1759 '<FieldAccess for match_formal_params at nodes.lkt:2486:48>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2486'
      if Local_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Other; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Local_Ent.Node, Other => Cast_Result_1, Match_Names => Match_Name, Ignore_First_Param => Ignore_First_Param, Origin => Origin, E_Info => Local_Ent.Info);
--# end
--# expr-done 1759
   If_Result_4 := Fld_6;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 1760
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Signature;
--# end
--# property-start BaseSubpSpec.match_other 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2490'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Other
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   --# bind origin Origin
   Property_Result : Boolean;
      Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1762 '<FieldAccess for match_signature at nodes.lkt:2491:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2491'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1761 '<Cast to BaseSubpSpec at nodes.lkt:2491:30>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2491'
Cast_Expr := Other; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 1761
--# property-call-start BaseSubpSpec.match_signature
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Ent.Node, Other => Cast_Result, Match_Name => Match_Names, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1762
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Other;
--# end
--# property-start BaseSubpSpec.match_expected_user_defined_function 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2494'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Expected_User_Defined_Function
  (Node : Bare_Base_Subp_Spec
      ; Fn : Internal_User_Defined_Function_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind fn Fn
   Property_Result : Boolean;
            procedure Finalizer_Scope_337 with Inline_Always;
            procedure Finalizer_Scope_2151 with Inline_Always;
            procedure Finalizer_Scope_2152 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Len_1 : Integer;
Is_Equal : Boolean;
Index_2 : Integer;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_337 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_337;
            procedure Finalizer_Scope_2151 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2151;
            procedure Finalizer_Scope_2152 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2152;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1765 '<FieldAccess for matching_type at nodes.lkt:2495:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2495'
--# expr-start 1763 '<FieldAccess for return_type at nodes.lkt:2495:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2495'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 1763
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1764 '<FieldAccess for subp_return_type at nodes.lkt:2495:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2495'
Fld_1 := Fn.Subp_Return_Type;
--# expr-done 1764
--# property-call-start BaseTypeDecl.matching_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld.Node, Expected_Type => Fld_1, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 1765
if Fld_2 then
--# expr-start 1766 '<FieldAccess for unpacked_formal_params at nodes.lkt:2495:67>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2495'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1766
Var_Expr_0 := Fld_3;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind params Var_Expr_0
--# expr-start 1770 '<Eq at nodes.lkt:2496:25>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
--# expr-start 1767 '<length at nodes.lkt:2496:25>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
Len := Length (Var_Expr_0); 
--# expr-done 1767
--# expr-start 1769 '<length at nodes.lkt:2496:44>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
--# expr-start 1768 '<FieldAccess for subp_params_types at nodes.lkt:2496:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
Fld_4 := Fn.Subp_Params_Types;
Inc_Ref (Fld_4);
--# expr-done 1768
Len_1 := Length (Fld_4); 
--# expr-done 1769
Is_Equal := Len = Len_1; 
--# expr-done 1770
if Is_Equal then
   --# expr-start 1778 '<IsNull at nodes.lkt:2496:78>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
--# expr-start 1777 '<Map at nodes.lkt:2496:78>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2496'
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
               --# bind i Index_2
         --# expr-start 1776 '<Not at nodes.lkt:2497:27>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
--# expr-start 1775 '<FieldAccess for matching_type at nodes.lkt:2497:31>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
--# expr-start 1772 '<FieldAccess for formal_type at nodes.lkt:2497:31>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
--# expr-start 1771 '<FieldAccess for formal_decl at nodes.lkt:2497:31>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 1771
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_5.Info);
--# end
--# expr-done 1772
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1774 '<at at nodes.lkt:2497:75>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
--# expr-start 1773 '<FieldAccess for subp_params_types at nodes.lkt:2497:75>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2497'
Fld_7 := Fn.Subp_Params_Types;
Inc_Ref (Fld_7);
--# expr-done 1773
Get_Result := Get (Self, Fld_7, Index_2, True); 
--# expr-done 1774
--# property-call-start BaseTypeDecl.matching_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_6.Node, Expected_Type => Get_Result, Origin => No_Bare_Ada_Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1775
Not_Val := not (Fld_8); 
--# expr-done 1776
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_1;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_2152;
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1777
Is_Equal_1 := Equivalent (Map_Result, No_Internal_Entity_Defining_Name_Array_Type); 
--# expr-done 1778
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
Scope_Result := If_Result;
   --# end
      Finalizer_Scope_2151;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_337;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_337;
                     Finalizer_Scope_2151;
                     Finalizer_Scope_2152;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Match_Expected_User_Defined_Function;
--# end
--# property-start BaseSubpSpec.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2503'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Defining_Env
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_338 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_338 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_338;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1783 '<If at nodes.lkt:2504:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2504'
--# expr-start 1780 '<IsNull at nodes.lkt:2504:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2504'
--# expr-start 1779 '<FieldAccess for returns at nodes.lkt:2504:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2504'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1779
Is_Null := Fld_2.Node = null; 
--# expr-done 1780
if Is_Null then
   If_Result := Empty_Env;
else
   --# expr-start 1782 '<FieldAccess for defining_env at nodes.lkt:2504:62>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2504'
--# expr-start 1781 '<FieldAccess for return_type at nodes.lkt:2504:62>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2504'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1781
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 1782
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1783
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_338;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_338;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Defining_Env;
--# end
--# property-start BaseSubpSpec.potential_dottable_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2509'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Potential_Dottable_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_339 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Formal_Param_Decl;
Var_Expr_47 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_339 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_339;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1785 '<at at nodes.lkt:2509:59>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2509'
--# expr-start 1784 '<FieldAccess for abstract_formal_params at nodes.lkt:2509:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2509'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1784
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 1785
Var_Expr_0 := Get_Result;
if Var_Expr_0 /= No_Entity_Base_Formal_Param_Decl then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 1786 '<FieldAccess for type_expression at nodes.lkt:2510:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2510'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1786
Var_Expr_47 := Fld_1;
if Var_Expr_47 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1787 '<FieldAccess for element_type at nodes.lkt:2510:16>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2510'
      if Var_Expr_47.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.element_type
Fld_2 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Var_Expr_47.Node, Origin => Origin, E_Info => Var_Expr_47.Info);
--# end
--# expr-done 1787
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_339;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_339;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Potential_Dottable_Type;
--# end
--# property-start BaseSubpSpec.get_candidate_type_for_primitive 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2519'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive
  (Node : Bare_Base_Subp_Spec
      ; Type_Expr : Internal_Entity_Type_Expr
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind type_expr Type_Expr
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_340 with Inline_Always;
      Local_Decl_Scope : Bare_Declarative_Part;
Local_Typ : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Boolean;
Match_1 : Internal_Entity_Type_Expr;
Local_Canon_Type : Internal_Entity_Base_Type_Decl;
Local_Final_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Incomplete_Type_Decl;
Local_Type_Scope : Bare_Declarative_Part;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Bare_Declarative_Part;
Var_Expr_5 : Bare_Declarative_Part;
Var_Expr_6 : Bare_Declarative_Part;
Var_Expr_48 : Internal_Entity_Generic_Instantiation;
Var_Expr_49 : Internal_Entity;
Var_Expr_50 : Internal_Entity_Base_Type_Decl;
Var_Expr_51 : Internal_Entity_Type_Decl;
Var_Expr_52 : Internal_Entity_Base_Type_Decl;
Var_Expr_53 : Bare_Declarative_Part;
Var_Expr_54 : Bare_Base_Package_Decl;
Fld : Internal_Entity_Generic_Instantiation_Array_Access;
Get_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Declarative_Part;
Fld_7 : Bare_Declarative_Part;
Fld_8 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Not_Val : Boolean;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Expr;
Cast_Result_1 : Internal_Entity_Anonymous_Type;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_12 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_13 : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Result_Var_5 : Internal_Entity_Base_Type_Decl;
Fld_15 : Bare_Base_Type_Decl;
Fld_16 : Bare_Ada_Node;
Fld_17 : Bare_Ada_Node;
Cast_Result_4 : Bare_Declarative_Part;
Scope_Result_8 : Bare_Declarative_Part;
Result_Var_6 : Bare_Declarative_Part;
Fld_18 : Bare_Ada_Node;
Is_A_1 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_7 : Boolean;
Fld_19 : Bare_Ada_Node;
Is_A_2 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_8 : Boolean;
If_Result_3 : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Type_Decl;
Fld_20 : Boolean;
Scope_Result_11 : Boolean;
Result_Var_9 : Boolean;
If_Result_4 : Boolean;
Fld_21 : Boolean;
Scope_Result_12 : Boolean;
Result_Var_10 : Boolean;
Not_Val_1 : Boolean;
If_Result_5 : Boolean;
Cast_Expr_2 : Bare_Declarative_Part;
Cast_Result_6 : Bare_Ada_Node;
Fld_22 : Bare_Ada_Node;
Scope_Result_13 : Bare_Ada_Node;
Result_Var_11 : Bare_Ada_Node;
Cast_Result_7 : Bare_Base_Package_Decl;
Fld_23 : Bare_Public_Part;
Scope_Result_14 : Bare_Public_Part;
Result_Var_12 : Bare_Public_Part;
Cast_Result_8 : Bare_Ada_Node;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_6 : Boolean;
Scope_Result_15 : Boolean;
Result_Var_13 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_5 : Internal_Entity_Base_Type_Decl;
Let_Result_6 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_340 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_340;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1834 '<Let at nodes.lkt:2520:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2520'
--# scope-start
--# expr-start 1800 '<FieldAccess for node at nodes.lkt:2520:26>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2520'
--# expr-start 1799 '<Cast to DeclarativePart at nodes.lkt:2520:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2520'
--# expr-start 1798 '<If at nodes.lkt:2521:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2521'
--# expr-start 1789 '<IsA GenericSubpInternal at nodes.lkt:2521:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2521'
--# expr-start 1788 '<parent at nodes.lkt:2521:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2521'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Parent; 
--# expr-done 1788
Is_A := Fld_6 /= null 
and then Fld_6.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1789
if Is_A then
--# expr-start 1791 '<at at nodes.lkt:2524:17>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2524'
--# expr-start 1790 '<FieldAccess for generic_instantiations at nodes.lkt:2524:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2524'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1790
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 1791
Var_Expr_48 := Get_Result;
if Var_Expr_48 /= No_Entity_Generic_Instantiation then
   --# scope-start
--# expr-start 1792 '<parent at nodes.lkt:2524:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2524'
      if Var_Expr_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr_48.Node, E_Info => Var_Expr_48.Info);
--# expr-done 1792
Var_Expr_49 := Fld_1;
if Var_Expr_49 /= No_Entity then
   --# scope-start
--# expr-start 1793 '<parent at nodes.lkt:2524:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2524'
      if Var_Expr_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Var_Expr_49.Node, E_Info => Var_Expr_49.Info);
--# expr-done 1793
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   If_Result_1 := Result_Var_1;
else
   --# expr-start 1797 '<as_entity at nodes.lkt:2525:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2525'
--# expr-start 1796 '<parent at nodes.lkt:2525:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2525'
--# expr-start 1795 '<parent at nodes.lkt:2525:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2525'
--# expr-start 1794 '<parent at nodes.lkt:2525:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2525'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 1794
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 1795
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Fld_4.Parent; 
--# expr-done 1796
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_5); 
   If_Result := As_Entity;
end if;
--# expr-done 1797
   If_Result_1 := If_Result;
end if;
--# expr-done 1798
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result := No_Entity_Declarative_Part;
   end if;
--# expr-done 1799
Fld_7 := Cast_Result.Node;
--# expr-done 1800
Local_Decl_Scope := Fld_7; 
--# bind decl_scope Local_Decl_Scope
--# expr-start 1833 '<Let at nodes.lkt:2527:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2527'
--# scope-start
--# expr-start 1807 '<bind at nodes.lkt:2528:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2528'
--# expr-start 1801 '<FieldAccess for origin_node at nodes.lkt:2528:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2528'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_8 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1801
Bound_Origin := Fld_8; 
--# expr-start 1806 '<Match at nodes.lkt:2530:13>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2530'
      if Type_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Type_Expr; 
case Ada_Type_Expr (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind at Match_0
--# expr-start 1802 '<FieldAccess for element_type at nodes.lkt:2531:43>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2531'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.element_type
Fld_9 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1802
Var_Expr_0 := Fld_9;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind et Var_Expr_0
--# expr-start 1804 '<Not at nodes.lkt:2532:30>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2532'
--# expr-start 1803 '<FieldAccess for is_classwide at nodes.lkt:2532:34>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2532'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1803
Not_Val := not (Fld_10); 
--# expr-done 1804
Var_Expr_1 := Not_Val;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result_2 := Var_Expr_0;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_3; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Enum_Lit_Synth_Type_Expr .. Ada_Synthetic_Type_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind other Match_1
--# expr-start 1805 '<FieldAccess for designated_type at nodes.lkt:2537:31>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2537'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 1805
Let_Result_1 := Fld_11; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1806
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 1807
Local_Typ := Dyn_Var_Bind_Result; 
--# bind typ Local_Typ
--# expr-start 1832 '<Let at nodes.lkt:2541:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2541'
--# scope-start
--# expr-start 1811 '<If at nodes.lkt:2541:26>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2541'
if Canonicalize then
   --# expr-start 1810 '<bind at nodes.lkt:2542:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2542'
--# expr-start 1808 '<FieldAccess for origin_node at nodes.lkt:2542:27>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2542'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_12 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1808
Bound_Origin_1 := Fld_12; 
Var_Expr_50 := Local_Typ;
if Var_Expr_50 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1809 '<FieldAccess for canonical_type at nodes.lkt:2544:13>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2544'
      if Var_Expr_50.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_50.Node, Origin => Bound_Origin_1, E_Info => Var_Expr_50.Info);
--# end
--# expr-done 1809
Scope_Result_6 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_4; 
--# expr-done 1810
   If_Result_2 := Dyn_Var_Bind_Result_1;
else
   If_Result_2 := Local_Typ;
end if;
--# expr-done 1811
Local_Canon_Type := If_Result_2; 
--# bind canon_type Local_Canon_Type
--# expr-start 1831 '<Let at nodes.lkt:2546:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2546'
--# scope-start
--# expr-start 1812 '<Cast to IncompleteTypeDecl at nodes.lkt:2546:26>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2546'
Cast_Expr := Local_Canon_Type; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1812
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_2
--# expr-start 1813 '<FieldAccess for next_part at nodes.lkt:2547:20>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2547'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1813
Scope_Result_7 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := Local_Canon_Type;
end if;
Local_Final_Type := Result_Var_5; 
--# bind final_type Local_Final_Type
--# expr-start 1830 '<Let at nodes.lkt:2549:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2549'
--# scope-start
Var_Expr_3 := Local_Final_Type;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_3
--# expr-start 1817 '<Cast to DeclarativePart at nodes.lkt:2550:22>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2550'
--# expr-start 1816 '<parent at nodes.lkt:2550:22>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2550'
--# expr-start 1815 '<parent at nodes.lkt:2550:22>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2550'
--# expr-start 1814 '<FieldAccess for node at nodes.lkt:2550:22>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2550'
Fld_15 := Var_Expr_3.Node;
--# expr-done 1814
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Fld_15.Parent; 
--# expr-done 1815
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Fld_16.Parent; 
--# expr-done 1816
   if Fld_17 = null
      or else Fld_17.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result_4 := Fld_17;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 1817
Scope_Result_8 := Cast_Result_4;
   --# end
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Bare_Ada_Node;
end if;
Local_Type_Scope := Result_Var_6; 
--# bind type_scope Local_Type_Scope
--# expr-start 1829 '<If at nodes.lkt:2553:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2553'
Var_Expr_4 := Local_Type_Scope;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_4
--# expr-start 1819 '<IsA BasePackageDecl at nodes.lkt:2557:39>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2557'
--# expr-start 1818 '<parent at nodes.lkt:2557:39>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2557'
      if Var_Expr_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Var_Expr_4.Parent; 
--# expr-done 1818
Is_A_1 := Fld_18 /= null 
and then Fld_18.Kind in Ada_Base_Package_Decl; 
--# expr-done 1819
Scope_Result_9 := Is_A_1;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
Var_Expr_5 := Local_Decl_Scope;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_5
--# expr-start 1821 '<IsA BasePackageDecl at nodes.lkt:2557:95>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2557'
--# expr-start 1820 '<parent at nodes.lkt:2557:95>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2557'
      if Var_Expr_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Var_Expr_5.Parent; 
--# expr-done 1820
Is_A_2 := Fld_19 /= null 
and then Fld_19.Kind in Ada_Base_Package_Decl; 
--# expr-done 1821
Scope_Result_10 := Is_A_2;
   --# end
   Result_Var_8 := Scope_Result_10;
else
   Result_Var_8 := False;
end if;
   If_Result_3 := Result_Var_8;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
--# expr-start 1822 '<Cast to TypeDecl at nodes.lkt:2571:17>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2571'
Cast_Expr_1 := Local_Final_Type; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_5 := No_Entity_Type_Decl;
   end if;
--# expr-done 1822
Var_Expr_51 := Cast_Result_5;
if Var_Expr_51 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1823 '<FieldAccess for is_derived_tagged_type at nodes.lkt:2571:17>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2571'
      if Var_Expr_51.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.is_derived_tagged_type
Fld_20 := Libadalang.Implementation.Type_Decl_P_Is_Derived_Tagged_Type (Node => Var_Expr_51.Node, E_Info => Var_Expr_51.Info);
--# end
--# expr-done 1823
Scope_Result_11 := Fld_20;
   --# end
   Result_Var_9 := Scope_Result_11;
else
   Result_Var_9 := False;
end if;
   If_Result_4 := Result_Var_9;
end if;
if If_Result_4 then
   --# expr-start 1825 '<Not at nodes.lkt:2575:13>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2575'
Var_Expr_52 := Local_Final_Type;
if Var_Expr_52 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1824 '<FieldAccess for is_classwide at nodes.lkt:2575:17>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2575'
      if Var_Expr_52.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_21 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_52.Node, E_Info => Var_Expr_52.Info);
--# end
--# expr-done 1824
Scope_Result_12 := Fld_21;
   --# end
   Result_Var_10 := Scope_Result_12;
else
   Result_Var_10 := False;
end if;
Not_Val_1 := not (Result_Var_10); 
--# expr-done 1825
   If_Result_5 := Not_Val_1;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
Var_Expr_6 := Local_Type_Scope;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ds Var_Expr_6
Cast_Expr_2 := Local_Decl_Scope; 
      Cast_Result_6 := Cast_Expr_2;
Is_Equal_1 := Var_Expr_6 = Cast_Result_6; 
if Is_Equal_1 then
   If_Result_6 := True;
else
--# expr-start 1827 '<Cast to BasePackageDecl at nodes.lkt:2581:44>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2581'
Var_Expr_53 := Local_Decl_Scope;
if Var_Expr_53 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1826 '<parent at nodes.lkt:2581:44>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2581'
      if Var_Expr_53 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Var_Expr_53.Parent; 
--# expr-done 1826
Scope_Result_13 := Fld_22;
   --# end
   Result_Var_11 := Scope_Result_13;
else
   Result_Var_11 := No_Bare_Ada_Node;
end if;
   if Result_Var_11 = null
      or else Result_Var_11.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_7 := Result_Var_11;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 1827
Var_Expr_54 := Cast_Result_7;
if Var_Expr_54 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1828 '<FieldAccess for public_part at nodes.lkt:2581:44>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2581'
      if Var_Expr_54 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Var_Expr_54.Base_Package_Decl_F_Public_Part;
--# expr-done 1828
Scope_Result_14 := Fld_23;
   --# end
   Result_Var_12 := Scope_Result_14;
else
   Result_Var_12 := No_Bare_Ada_Node;
end if;
      Cast_Result_8 := Result_Var_12;
Is_Equal := Var_Expr_6 = Cast_Result_8; 
   If_Result_6 := Is_Equal;
end if;
Scope_Result_15 := If_Result_6;
   --# end
   Result_Var_13 := Scope_Result_15;
else
   Result_Var_13 := False;
end if;
   If_Result_7 := Result_Var_13;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
   If_Result_8 := Local_Final_Type;
else
   If_Result_8 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1829
Let_Result_2 := If_Result_8; 
--# end
--# expr-done 1830
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 1831
Let_Result_4 := Let_Result_3; 
--# end
--# expr-done 1832
Let_Result_5 := Let_Result_4; 
--# end
--# expr-done 1833
Let_Result_6 := Let_Result_5; 
--# end
--# expr-done 1834
         Property_Result := Let_Result_6;
   --# end
      Finalizer_Scope_340;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_340;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2590'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_341 with Inline_Always;
            procedure Finalizer_Scope_2173 with Inline_Always;
      Local_Params : Internal_Entity_Defining_Name_Array_Access;
Local_Types : Internal_Entity_Type_Expr_Array_Access;
Var_Expr_1 : Internal_Entity_Type_Expr;
Var_Expr_5 : Internal_Entity_Incomplete_Type_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Map_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_3 : Internal_Entity_Type_Expr;
Array_Lit : Internal_Entity_Type_Expr_Array_Access;
Scope_Result : Internal_Entity_Type_Expr_Array_Access;
Result_Var : Internal_Entity_Type_Expr_Array_Access;
Concat_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Incomplete_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Map_Result_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_341 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Params);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Local_Types);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_341;
            procedure Finalizer_Scope_2173 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2173;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Candidate_Primitive_Subp_Types,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1851 '<Let at nodes.lkt:2593:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2593'
--# scope-start
--# expr-start 1835 '<FieldAccess for unpacked_formal_params at nodes.lkt:2593:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2593'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1835
Local_Params := Fld; Inc_Ref (Local_Params);
--# bind params Local_Params
--# expr-start 1850 '<Let at nodes.lkt:2594:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
--# scope-start
--# expr-start 1838 '<Map at nodes.lkt:2594:21>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
      declare
         Map_Result_Vec : Internal_Entity_Type_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Params;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 1837 '<FieldAccess for type_expression at nodes.lkt:2594:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
--# expr-start 1836 '<FieldAccess for formal_decl at nodes.lkt:2594:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1836
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1837
         declare
            Item_To_Append : constant Internal_Entity_Type_Expr := Fld_2;
         begin
            Internal_Entity_Type_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Type_Expr_Array
           (Items_Count => Natural (Internal_Entity_Type_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1838
--# expr-start 1839 '<FieldAccess for returns at nodes.lkt:2594:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1839
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 1840 '<ArrayLiteral at nodes.lkt:2594:102>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2594'
Array_Lit := Create_Internal_Entity_Type_Expr_Array (Internal_Internal_Entity_Type_Expr_Array'(1 => Var_Expr_1)); 
--# expr-done 1840
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2173;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
Local_Types := Concat_Result; Inc_Ref (Local_Types);
--# bind types Local_Types
--# expr-start 1849 '<unique at nodes.lkt:2596:9>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2596'
--# expr-start 1848 '<Map at nodes.lkt:2596:9>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2596'
   --# expr-start 1845 '<Map at nodes.lkt:2596:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2596'
   --# expr-start 1842 '<Map at nodes.lkt:2596:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2596'
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Type_Expr_Array_Access := Local_Types;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_2
      --# expr-start 1841 '<FieldAccess for get_candidate_type_for_primitive at nodes.lkt:2597:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2597'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.get_candidate_type_for_primitive
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive (Node => Ent.Node, Type_Expr => Item_2, Canonicalize => Canonicalize, E_Info => Ent.Info);
--# end
--# expr-done 1841
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1842
      declare
         Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_3
         --# expr-start 1844 '<Not at nodes.lkt:2598:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2598'
--# expr-start 1843 '<IsNull at nodes.lkt:2598:29>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2598'
Is_Null := Item_3.Node = null; 
--# expr-done 1843
Not_Val := not (Is_Null); 
--# expr-done 1844
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 1845
      declare
         Map_Result_3_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_4
--# expr-start 1846 '<Cast to IncompleteTypeDecl at nodes.lkt:2599:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2599'
Cast_Expr := Item_4; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1846
Var_Expr_5 := Cast_Result;
if Var_Expr_5 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_5
--# expr-start 1847 '<FieldAccess for next_part at nodes.lkt:2599:55>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2599'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 1847
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Item_4;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Result_Var_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 1848
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 1849
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 1850
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1851
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_341;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_341;
                     Finalizer_Scope_2173;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                        As_Internal_Entity_Base_Type_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_first_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2605'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_342 with Inline_Always;
            procedure Finalizer_Scope_2178 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_342 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_342;
            procedure Finalizer_Scope_2178 is
            begin
                     Dec_Ref (Var_Expr_0);
            end Finalizer_Scope_2178;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1852 '<FieldAccess for candidate_primitive_subp_types at nodes.lkt:2606:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2606'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1852
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Base_Type_Decl_Array_Type then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 1853 '<at at nodes.lkt:2606:57>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2606'
Get_Result := Get (Self, Var_Expr_0, 0, True); 
--# expr-done 1853
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2178;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_342;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_342;
                     Finalizer_Scope_2178;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.candidate_primitive_subp_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2612'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_343 with Inline_Always;
      Cast_Expr : Bare_Base_Subp_Spec;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Find_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_343 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_343;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1858 '<bind at nodes.lkt:2613:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2613'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1857 '<Find at nodes.lkt:2615:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2615'
   --# expr-start 1854 '<FieldAccess for candidate_primitive_subp_types at nodes.lkt:2615:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2615'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Ent.Node, Canonicalize => Canonicalize, E_Info => Ent.Info);
--# end
--# expr-done 1854
      Find_Result := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 1856 '<FieldAccess for is_tagged_type at nodes.lkt:2615:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2615'
--# expr-start 1855 '<FieldAccess for full_view at nodes.lkt:2615:71>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2615'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 1855
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1856
      if Fld_2 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 1857
Dyn_Var_Bind_Result := Find_Result; 
--# expr-done 1858
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_343;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_343;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2626'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Follow_Generic : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_generic Follow_Generic
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_55 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Result_Var_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Result_Var_2 : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1866 '<Let at nodes.lkt:2629:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2629'
--# scope-start
--# expr-start 1859 '<FieldAccess for name at nodes.lkt:2629:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2629'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1859
Var_Expr_55 := Fld;
if Var_Expr_55 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1860 '<FieldAccess for basic_decl at nodes.lkt:2629:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2629'
      if Var_Expr_55.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_55.Node, E_Info => Var_Expr_55.Info);
--# end
--# expr-done 1860
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Local_Bd := Result_Var; 
--# bind bd Local_Bd
Var_Expr_0 := Local_Bd;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 1861 '<FieldAccess for canonical_part at nodes.lkt:2632:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2632'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1861
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr_1
--# expr-start 1865 '<If at nodes.lkt:2633:25>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2633'
--# expr-start 1862 '<Not at nodes.lkt:2633:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2633'
Not_Val := not (Follow_Generic); 
--# expr-done 1862
if Not_Val then
   --# expr-start 1863 '<IsA GenericSubpInternal at nodes.lkt:2633:51>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2633'
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1863
   If_Result := Is_A;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
   --# expr-start 1864 '<FieldAccess for subp_spec_or_null at nodes.lkt:2633:114>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2633'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, Follow_Generic => Follow_Generic, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1864
   If_Result_1 := Fld_3;
end if;
--# expr-done 1865
Scope_Result_1 := If_Result_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Ent;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Ent;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1866
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Decl_Spec;
--# end
--# property-start BaseSubpSpec.primitive_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2642'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1867 '<FieldAccess for decl_spec at nodes.lkt:2643:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2643'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Follow_Generic => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1867
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Decl_Spec;
--# end
--# property-start BaseSubpSpec.as_primitive_subp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2650'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_As_Primitive_Subp_Type
  (Node : Bare_Base_Subp_Spec
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2184 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Var_Expr_2 : Boolean;
Var_Expr_56 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_3 : Bare_Ada_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2184 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2184;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1868 '<FieldAccess for name at nodes.lkt:2652:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2652'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1868
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind name Var_Expr_0
Var_Expr_56 := Typ;
if Var_Expr_56 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1876 '<AnyQuantifier at nodes.lkt:2653:23>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
   --# expr-start 1871 '<EnvGet at nodes.lkt:2653:23>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
--# expr-start 1869 '<FieldAccess for primitives_env at nodes.lkt:2653:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
      if Var_Expr_56.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr_56.Node, E_Info => Var_Expr_56.Info);
--# end
--# expr-done 1869
--# expr-start 1870 '<FieldAccess for name_symbol at nodes.lkt:2653:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_0.Node);
--# end
--# expr-done 1870
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Thin (Fld_2), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1871
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind b Item_1
      --# expr-start 1875 '<Eq at nodes.lkt:2653:80>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
--# expr-start 1872 '<FieldAccess for node at nodes.lkt:2653:80>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
Fld_3 := Item_1.Node;
--# expr-done 1872
--# expr-start 1874 '<FieldAccess for node at nodes.lkt:2653:90>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
--# expr-start 1873 '<FieldAccess for basic_decl at nodes.lkt:2653:90>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2653'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1873
Fld_5 := Fld_4.Node;
--# expr-done 1874
      Cast_Result := Fld_5;
Is_Equal := Fld_3 = Cast_Result; 
--# expr-done 1875
      Quantifier_Result := Is_Equal;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1876
Var_Expr_2 := Quantifier_Result;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 1877 '<FieldAccess for real_type at nodes.lkt:2659:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2659'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Typ, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 1877
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_2184;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2184;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_As_Primitive_Subp_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2666'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_347 with Inline_Always;
            procedure Finalizer_Scope_2187 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_347 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_347;
            procedure Finalizer_Scope_2187 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2187;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 1878 '<FieldAccess for primitive_decl_spec at nodes.lkt:2666:63>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2666'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1878
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 1883 '<Map at nodes.lkt:2667:19>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2667'
   --# expr-start 1879 '<FieldAccess for candidate_primitive_subp_types at nodes.lkt:2667:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2667'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1879
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_1
         --# expr-start 1882 '<Not at nodes.lkt:2668:20>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2668'
--# expr-start 1881 '<IsNull at nodes.lkt:2668:24>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2668'
--# expr-start 1880 '<FieldAccess for as_primitive_subp_type at nodes.lkt:2668:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2668'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr_0.Node, Typ => Item_1, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1880
Is_Null := Fld_2.Node = null; 
--# expr-done 1881
Not_Val := not (Is_Null); 
--# expr-done 1882
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1883
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2187;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_347;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_347;
                     Finalizer_Scope_2187;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.primitive_subp_first_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2675'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 1884 '<FieldAccess for primitive_decl_spec at nodes.lkt:2675:61>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2675'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1884
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 1886 '<FieldAccess for as_primitive_subp_type at nodes.lkt:2676:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2676'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1885 '<FieldAccess for candidate_primitive_subp_first_type at nodes.lkt:2677:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2677'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_first_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_First_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1885
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr_0.Node, Typ => Fld_1, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1886
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2685'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 1887 '<FieldAccess for primitive_decl_spec at nodes.lkt:2685:62>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2685'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1887
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 1889 '<FieldAccess for as_primitive_subp_type at nodes.lkt:2686:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2686'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1888 '<FieldAccess for candidate_primitive_subp_tagged_type at nodes.lkt:2687:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2687'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1888
--# property-call-start BaseSubpSpec.as_primitive_subp_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_As_Primitive_Subp_Type (Node => Var_Expr_0.Node, Typ => Fld_1, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1889
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.has_controlling_result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2695'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Has_Controlling_Result
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1895 '<Let at nodes.lkt:2696:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2696'
--# scope-start
--# expr-start 1890 '<FieldAccess for candidate_primitive_subp_tagged_type at nodes.lkt:2696:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2696'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1890
Local_Typ := Fld; 
--# bind typ Local_Typ
--# expr-start 1892 '<Not at nodes.lkt:2698:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2698'
--# expr-start 1891 '<IsNull at nodes.lkt:2698:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2698'
Is_Null := Local_Typ.Node = null; 
--# expr-done 1891
Not_Val := not (Is_Null); 
--# expr-done 1892
if Not_Val then
   --# expr-start 1894 '<Eq at nodes.lkt:2698:29>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2698'
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1893 '<FieldAccess for return_type at nodes.lkt:2698:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2698'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 1893
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1894
   If_Result := Is_Equiv;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 1895
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Has_Controlling_Result;
--# end
--# property-start BaseSubpSpec.dottable_subp_of 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2707'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Dottable_Subp_Of
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_57 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Dottable_Subp_Of,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 1901 '<bind at nodes.lkt:2710:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2710'
--# expr-start 1896 '<FieldAccess for origin_node at nodes.lkt:2710:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2710'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1896
Bound_Origin := Fld; 
--# expr-start 1900 '<If at nodes.lkt:2712:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2712'
--# expr-start 1897 '<FieldAccess for nb_max_params at nodes.lkt:2712:16>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2712'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Ent.Node);
--# end
--# expr-done 1897
Comp_Result := Fld_3 > 0; 
if Comp_Result then
--# expr-start 1898 '<FieldAccess for potential_dottable_type at nodes.lkt:2712:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2712'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.potential_dottable_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Potential_Dottable_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1898
Var_Expr_57 := Fld_1;
if Var_Expr_57 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1899 '<FieldAccess for specific_type at nodes.lkt:2712:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2712'
      if Var_Expr_57.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_57.Node, E_Info => Var_Expr_57.Info);
--# end
--# expr-done 1899
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result := Result_Var;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1900
Dyn_Var_Bind_Result := If_Result; 
--# expr-done 1901
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Subp_Spec_P_Dottable_Subp_Of;
--# end
--# property-start BaseSubpSpec.return_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2719'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 1902 '<FieldAccess for returns at nodes.lkt:2720:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2720'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1902
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind rt Var_Expr_0
--# expr-start 1903 '<FieldAccess for real_designated_type at nodes.lkt:2720:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2720'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Var_Expr_0, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1903
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Return_Type;
--# end
--# property-start BaseSubpSpec.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2722'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Entry_Point
  (Node : Bare_Base_Subp_Spec
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Xref_Entry_Point;
--# end
--# property-start BaseSubpSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2725'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Equation
  (Node : Bare_Base_Subp_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
     Ent : Internal_Entity_Base_Subp_Spec :=
       Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_354 with Inline_Always;
            procedure Finalizer_Scope_2193 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_354 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_354;
            procedure Finalizer_Scope_2193 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2193;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1904 '<FieldAccess for returns at nodes.lkt:2725:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2725'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1904
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 1905 '<FieldAccess for sub_equation at nodes.lkt:2726:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2726'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1905
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2193;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2726:46") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_354;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_354;
                     Finalizer_Scope_2193;
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Xref_Equation;
--# end
--# property-start '[dispatcher]BaseSubpSpec.params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Synthetic_Binary_Spec .. Ada_Synthetic_Unary_Spec =>
                  --# property-call-start BaseSubpSpec.params
                  Property_Result := Base_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.params
                  Property_Result := Entry_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.params
                  Property_Result := Enum_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.params
                  Property_Result := Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subp_Spec_P_Params;
--# end
--# property-start BaseSubpSpec.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2731'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Base_Subp_Spec  := Bare_Base_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property BaseSubpSpec.params not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Spec_P_Params;
--# end
   --
   --  Primitives for Bare_Entry_Spec
   --
      procedure Initialize_Fields_For_Entry_Spec
        (Self : Bare_Entry_Spec
         ; Entry_Spec_F_Entry_Name : Bare_Defining_Name
         ; Entry_Spec_F_Family_Type : Bare_Ada_Node
         ; Entry_Spec_F_Entry_Params : Bare_Params
        ) is
      begin
            Self.Entry_Spec_F_Entry_Name := Entry_Spec_F_Entry_Name;
            Self.Entry_Spec_F_Family_Type := Entry_Spec_F_Family_Type;
            Self.Entry_Spec_F_Entry_Params := Entry_Spec_F_Entry_Params;
      end Initialize_Fields_For_Entry_Spec;
   function Entry_Spec_F_Entry_Name
     (Node : Bare_Entry_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Spec_F_Entry_Name;
   end;
   function Entry_Spec_F_Family_Type
     (Node : Bare_Entry_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Spec_F_Family_Type;
   end;
   function Entry_Spec_F_Entry_Params
     (Node : Bare_Entry_Spec) return Bare_Params
   is
   begin
         return Node.Entry_Spec_F_Entry_Params;
   end;
--# property-start EntrySpec.name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2745'
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Name
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1906 '<FieldAccess for entry_name at nodes.lkt:2745:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2745'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Spec_F_Entry_Name, Info => Ent.Info);
--# expr-done 1906
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Name;
--# end
--# property-start EntrySpec.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2747'
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Params
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_357 with Inline_Always;
            procedure Finalizer_Scope_2194 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
Item_1 : Internal_Entity_Param_Spec;
Bare_Item_1 : Bare_Param_Spec;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_357 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_357;
            procedure Finalizer_Scope_2194 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2194;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 1907 '<FieldAccess for entry_params at nodes.lkt:2747:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2747'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Spec_F_Entry_Params, Info => Ent.Info);
--# expr-done 1907
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Params then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 1909 '<Map at nodes.lkt:2748:16>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2748'
   --# expr-start 1908 '<FieldAccess for params at nodes.lkt:2748:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2748'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr_0.Node.Params_F_Params, Info => Var_Expr_0.Info);
--# expr-done 1908
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind p Item_1
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec := Item_1;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Param_Spec_Array
           (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1909
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2194;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_357;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_357;
                     Finalizer_Scope_2194;
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Params;
--# end
--# property-start EntrySpec.returns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2751'
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Returns
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Type_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Returns;
--# end
--# property-start EntrySpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2754'
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Xref_Equation
  (Node : Bare_Entry_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Spec  := Bare_Entry_Spec (Node);
     Ent : Internal_Entity_Entry_Spec :=
       Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_359 with Inline_Always;
            procedure Finalizer_Scope_2196 with Inline_Always;
      Var_Expr_0 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_359 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_359;
            procedure Finalizer_Scope_2196 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2196;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1910 '<FieldAccess for family_type at nodes.lkt:2754:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2754'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Spec_F_Family_Type, Info => Ent.Info);
--# expr-done 1910
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 1911 '<FieldAccess for sub_equation at nodes.lkt:2755:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2755'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1911
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2196;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:2755:46") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_359;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_359;
                     Finalizer_Scope_2196;
            raise;
      end;
   return Property_Result;
end Entry_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Subp_Spec
   --
--# property-start EnumSubpSpec.enum_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2765'
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Enum_Decl
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Literal_Decl
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Enum_Literal_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Enum_Literal_Decl;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1914 '<as_entity at nodes.lkt:2766:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2766'
--# expr-start 1913 '<Cast to EnumLiteralDecl at nodes.lkt:2766:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2766'
--# expr-start 1912 '<parent at nodes.lkt:2766:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2766'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1912
   if Fld = null
      or else Fld.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1913
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1914
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Enum_Decl;
--# end
--# property-start EnumSubpSpec.name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2768'
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Name
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
     Ent : Internal_Entity_Enum_Subp_Spec :=
       Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1916 '<FieldAccess for enum_decl_name at nodes.lkt:2768:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2768'
--# expr-start 1915 '<FieldAccess for enum_decl at nodes.lkt:2768:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2768'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1915
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_decl_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1916
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Name;
--# end
--# property-start EnumSubpSpec.returns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2770'
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Returns
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
     Ent : Internal_Entity_Enum_Subp_Spec :=
       Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 1918 '<FieldAccess for synth_type_expr at nodes.lkt:2770:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2770'
--# expr-start 1917 '<FieldAccess for enum_decl at nodes.lkt:2770:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2770'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1917
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumLiteralDecl.synth_type_expr
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Synth_Type_Expr (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1918
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Returns;
--# end
--# property-start EnumSubpSpec.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2772'
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Params
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Enum_Subp_Spec  := Bare_Enum_Subp_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Param_Spec_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Subp_Spec_P_Params;
--# end
   --
   --  Primitives for Bare_Subp_Spec
   --
      procedure Initialize_Fields_For_Subp_Spec
        (Self : Bare_Subp_Spec
         ; Subp_Spec_F_Subp_Kind : Bare_Subp_Kind
         ; Subp_Spec_F_Subp_Name : Bare_Defining_Name
         ; Subp_Spec_F_Subp_Params : Bare_Params
         ; Subp_Spec_F_Subp_Returns : Bare_Type_Expr
        ) is
      begin
            Self.Subp_Spec_F_Subp_Kind := Subp_Spec_F_Subp_Kind;
            Self.Subp_Spec_F_Subp_Name := Subp_Spec_F_Subp_Name;
            Self.Subp_Spec_F_Subp_Params := Subp_Spec_F_Subp_Params;
            Self.Subp_Spec_F_Subp_Returns := Subp_Spec_F_Subp_Returns;
      end Initialize_Fields_For_Subp_Spec;
   function Subp_Spec_F_Subp_Kind
     (Node : Bare_Subp_Spec) return Bare_Subp_Kind
   is
   begin
         return Node.Subp_Spec_F_Subp_Kind;
   end;
   function Subp_Spec_F_Subp_Name
     (Node : Bare_Subp_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Subp_Spec_F_Subp_Name;
   end;
   function Subp_Spec_F_Subp_Params
     (Node : Bare_Subp_Spec) return Bare_Params
   is
   begin
         return Node.Subp_Spec_F_Subp_Params;
   end;
   function Subp_Spec_F_Subp_Returns
     (Node : Bare_Subp_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Subp_Spec_F_Subp_Returns;
   end;
--# property-start SubpSpec.name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2782'
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Name
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1919 '<FieldAccess for subp_name at nodes.lkt:2782:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2782'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Subp_Spec_F_Subp_Name, Info => Ent.Info);
--# expr-done 1919
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Name;
--# end
--# property-start SubpSpec.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2784'
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Params
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_365 with Inline_Always;
            procedure Finalizer_Scope_2197 with Inline_Always;
      Var_Expr_58 : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Param_Spec;
Bare_Item_0 : Bare_Param_Spec;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_365 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_365;
            procedure Finalizer_Scope_2197 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2197;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 1920 '<FieldAccess for subp_params at nodes.lkt:2785:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2785'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Subp_Spec_F_Subp_Params, Info => Ent.Info);
--# expr-done 1920
Var_Expr_58 := Fld;
if Var_Expr_58 /= No_Entity_Params then
   --# scope-start
--# expr-start 1922 '<Map at nodes.lkt:2785:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2785'
   --# expr-start 1921 '<FieldAccess for params at nodes.lkt:2785:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2785'
      if Var_Expr_58.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr_58.Node.Params_F_Params, Info => Var_Expr_58.Info);
--# expr-done 1921
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind p Item_0
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec := Item_0;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Param_Spec_Array
           (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1922
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2197;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_365;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_365;
                     Finalizer_Scope_2197;
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Params;
--# end
--# property-start SubpSpec.returns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2787'
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Returns
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Spec  := Bare_Subp_Spec (Node);
     Ent : Internal_Entity_Subp_Spec :=
       Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1923 '<FieldAccess for subp_returns at nodes.lkt:2787:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2787'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Subp_Spec_F_Subp_Returns, Info => Ent.Info);
--# expr-done 1923
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Synthetic_Binary_Spec
   --
      procedure Initialize_Fields_For_Synthetic_Binary_Spec
        (Self : Bare_Synthetic_Binary_Spec
         ; Synthetic_Binary_Spec_F_Left_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Binary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Binary_Spec_F_Return_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Synthetic_Binary_Spec_F_Left_Param := Synthetic_Binary_Spec_F_Left_Param;
            Self.Synthetic_Binary_Spec_F_Right_Param := Synthetic_Binary_Spec_F_Right_Param;
            Self.Synthetic_Binary_Spec_F_Return_Type_Expr := Synthetic_Binary_Spec_F_Return_Type_Expr;
      Self.Synthetic_Binary_Spec_Subp_Symbol := No_Symbol;
      end Initialize_Fields_For_Synthetic_Binary_Spec;
   function Synthetic_Binary_Spec_F_Left_Param
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Left_Param;
   end;
   function Synthetic_Binary_Spec_F_Right_Param
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Right_Param;
   end;
   function Synthetic_Binary_Spec_F_Return_Type_Expr
     (Node : Bare_Synthetic_Binary_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Synthetic_Binary_Spec_F_Return_Type_Expr;
   end;
--# property-start SyntheticBinarySpec.name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2798'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Name
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1926 '<as_entity at nodes.lkt:2799:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2799'
--# expr-start 1925 '<FieldAccess for synthesize_defining_name at nodes.lkt:2799:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2799'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1924 '<FieldAccess for subp_symbol at nodes.lkt:2799:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2799'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Binary_Spec_Subp_Symbol;
--# expr-done 1924
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 1925
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1926
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Name;
--# end
--# property-start SyntheticBinarySpec.returns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2801'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Returns
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Binary_Spec :=
       Internal_Entity_Synthetic_Binary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1927 '<FieldAccess for return_type_expr at nodes.lkt:2801:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2801'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Synthetic_Binary_Spec_F_Return_Type_Expr, Info => Ent.Info);
--# expr-done 1927
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Returns;
--# end
--# property-start SyntheticBinarySpec.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2803'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Corresponding_Actual
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Binary_Spec :=
       Internal_Entity_Synthetic_Binary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Cast_Expr : Internal_Entity_Synthetic_Binary_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Corresponding_Actual;
--# end
--# property-start SyntheticBinarySpec.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2805'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Binary_Spec_P_Abstract_Formal_Params
  (Node : Bare_Synthetic_Binary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Synthetic_Binary_Spec  := Bare_Synthetic_Binary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Binary_Spec :=
       Internal_Entity_Synthetic_Binary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_370 with Inline_Always;
      Fld : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Array_Lit : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_370 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_370;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1932 '<ArrayLiteral at nodes.lkt:2806:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2806'
--# expr-start 1929 '<Cast to BaseFormalParamDecl at nodes.lkt:2806:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2806'
--# expr-start 1928 '<FieldAccess for left_param at nodes.lkt:2806:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2806'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Binary_Spec_F_Left_Param, Info => Ent.Info);
--# expr-done 1928
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 1929
--# expr-start 1931 '<Cast to BaseFormalParamDecl at nodes.lkt:2806:51>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2806'
--# expr-start 1930 '<FieldAccess for right_param at nodes.lkt:2806:51>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2806'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Binary_Spec_F_Right_Param, Info => Ent.Info);
--# expr-done 1930
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 1931
Array_Lit := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Internal_Internal_Entity_Base_Formal_Param_Decl_Array'(1 => Cast_Result, 2 => Cast_Result_1)); 
--# expr-done 1932
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_370;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_370;
            raise;
      end;
   return Property_Result;
end Synthetic_Binary_Spec_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Synthetic_Unary_Spec
   --
      procedure Initialize_Fields_For_Synthetic_Unary_Spec
        (Self : Bare_Synthetic_Unary_Spec
         ; Synthetic_Unary_Spec_F_Right_Param : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Unary_Spec_F_Return_Type_Expr : Bare_Synthetic_Type_Expr
        ) is
      begin
            Self.Synthetic_Unary_Spec_F_Right_Param := Synthetic_Unary_Spec_F_Right_Param;
            Self.Synthetic_Unary_Spec_F_Return_Type_Expr := Synthetic_Unary_Spec_F_Return_Type_Expr;
      Self.Synthetic_Unary_Spec_Subp_Symbol := No_Symbol;
      end Initialize_Fields_For_Synthetic_Unary_Spec;
   function Synthetic_Unary_Spec_F_Right_Param
     (Node : Bare_Synthetic_Unary_Spec) return Bare_Synthetic_Formal_Param_Decl
   is
   begin
         return Node.Synthetic_Unary_Spec_F_Right_Param;
   end;
   function Synthetic_Unary_Spec_F_Return_Type_Expr
     (Node : Bare_Synthetic_Unary_Spec) return Bare_Synthetic_Type_Expr
   is
   begin
         return Node.Synthetic_Unary_Spec_F_Return_Type_Expr;
   end;
--# property-start SyntheticUnarySpec.name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2816'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Name
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1935 '<as_entity at nodes.lkt:2817:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2817'
--# expr-start 1934 '<FieldAccess for synthesize_defining_name at nodes.lkt:2817:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2817'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1933 '<FieldAccess for subp_symbol at nodes.lkt:2817:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2817'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Unary_Spec_Subp_Symbol;
--# expr-done 1933
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 1934
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 1935
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Name;
--# end
--# property-start SyntheticUnarySpec.returns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2819'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Returns
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Unary_Spec :=
       Internal_Entity_Synthetic_Unary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Synthetic_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 1936 '<FieldAccess for return_type_expr at nodes.lkt:2819:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2819'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Type_Expr (Node => Ent.Node.Synthetic_Unary_Spec_F_Return_Type_Expr, Info => Ent.Info);
--# expr-done 1936
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Returns;
--# end
--# property-start SyntheticUnarySpec.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2821'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Corresponding_Actual
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Unary_Spec :=
       Internal_Entity_Synthetic_Unary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Cast_Expr : Internal_Entity_Synthetic_Unary_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Corresponding_Actual;
--# end
--# property-start SyntheticUnarySpec.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2823'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Unary_Spec_P_Abstract_Formal_Params
  (Node : Bare_Synthetic_Unary_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Synthetic_Unary_Spec  := Bare_Synthetic_Unary_Spec (Node);
     Ent : Internal_Entity_Synthetic_Unary_Spec :=
       Internal_Entity_Synthetic_Unary_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_374 with Inline_Always;
      Fld : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Array_Lit : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_374 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_374;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1939 '<ArrayLiteral at nodes.lkt:2824:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2824'
--# expr-start 1938 '<Cast to BaseFormalParamDecl at nodes.lkt:2824:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2824'
--# expr-start 1937 '<FieldAccess for right_param at nodes.lkt:2824:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2824'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Synthetic_Formal_Param_Decl (Node => Ent.Node.Synthetic_Unary_Spec_F_Right_Param, Info => Ent.Info);
--# expr-done 1937
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 1938
Array_Lit := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Internal_Internal_Entity_Base_Formal_Param_Decl_Array'(1 => Cast_Result)); 
--# expr-done 1939
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_374;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_374;
            raise;
      end;
   return Property_Result;
end Synthetic_Unary_Spec_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Component_List
   --
      procedure Initialize_Fields_For_Component_List
        (Self : Bare_Component_List
         ; Component_List_F_Components : Bare_Ada_Node_List
         ; Component_List_F_Variant_Part : Bare_Variant_Part
        ) is
      begin
            Self.Component_List_F_Components := Component_List_F_Components;
            Self.Component_List_F_Variant_Part := Component_List_F_Variant_Part;
      end Initialize_Fields_For_Component_List;
   function Component_List_F_Components
     (Node : Bare_Component_List) return Bare_Ada_Node_List
   is
   begin
         return Node.Component_List_F_Components;
   end;
   function Component_List_F_Variant_Part
     (Node : Bare_Component_List) return Bare_Variant_Part
   is
   begin
         return Node.Component_List_F_Variant_Part;
   end;
--# property-start ComponentList.type_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2832'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Def
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Def
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Def;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Type_Def;
As_Entity : Internal_Entity_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1943 '<as_entity at nodes.lkt:2832:39>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2832'
--# expr-start 1942 '<Cast to TypeDef at nodes.lkt:2832:39>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2832'
--# expr-start 1941 '<parent at nodes.lkt:2832:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2832'
--# expr-start 1940 '<parent at nodes.lkt:2832:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2832'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 1940
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1941
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Type_Def
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1942
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Type_Def;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1943
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Type_Def;
--# end
--# property-start ComponentList.type_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2834'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Decl
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 1946 '<Cast to TypeDecl at nodes.lkt:2834:41>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2834'
--# expr-start 1945 '<parent at nodes.lkt:2834:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2834'
--# expr-start 1944 '<FieldAccess for type_def at nodes.lkt:2834:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2834'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1944
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1945
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 1946
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Type_Decl;
--# end
--# property-start ComponentList.parent_component_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2836'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Parent_Component_List
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Var_Expr_59 : Internal_Entity_Derived_Type_Def;
Var_Expr_60 : Internal_Entity_Base_Record_Def;
Cast_Expr : Bare_Component_List;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Scope_Result : Internal_Entity_Component_List;
Result_Var : Internal_Entity_Component_List;
Scope_Result_1 : Internal_Entity_Component_List;
Result_Var_1 : Internal_Entity_Component_List;
Dyn_Var_Bind_Result : Internal_Entity_Component_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1952 '<bind at nodes.lkt:2837:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2837'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1948 '<Cast to DerivedTypeDef at nodes.lkt:2839:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2839'
--# expr-start 1947 '<FieldAccess for type_def at nodes.lkt:2839:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2839'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1947
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1948
Var_Expr_59 := Cast_Result_1;
if Var_Expr_59 /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 1950 '<FieldAccess for record_def at nodes.lkt:2839:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2839'
--# expr-start 1949 '<FieldAccess for base_type at nodes.lkt:2839:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2839'
      if Var_Expr_59.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr_59.Node, Origin => Bound_Origin, E_Info => Var_Expr_59.Info);
--# end
--# expr-done 1949
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1950
Var_Expr_60 := Fld_2;
if Var_Expr_60 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 1951 '<FieldAccess for comps at nodes.lkt:2839:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2839'
      if Var_Expr_60.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Var_Expr_60.Node, E_Info => Var_Expr_60.Info);
--# end
--# expr-done 1951
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Component_List;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Component_List;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 1952
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Parent_Component_List;
--# end
--# property-start ComponentList.abstract_formal_params_for_assocs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2843'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Assocs
  (Node : Bare_Component_List
      ; Assocs : Internal_Entity_Assoc_List
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind assocs Assocs
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_378 with Inline_Always;
      Local_Td : Internal_Entity_Type_Decl;
Local_Discriminants : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Discriminants_Matches : Internal_Param_Match_Array_Access;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Entity_Base_Record_Def;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_378 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Discriminants);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Discriminants_Matches);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_378;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1969 '<Let at nodes.lkt:2844:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2844'
--# scope-start
--# expr-start 1953 '<FieldAccess for type_decl at nodes.lkt:2844:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2844'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1953
Local_Td := Fld; 
--# bind td Local_Td
--# expr-start 1968 '<Let at nodes.lkt:2845:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2845'
--# scope-start
--# expr-start 1954 '<FieldAccess for discriminants_list at nodes.lkt:2845:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2845'
      if Local_Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Local_Td.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Local_Td.Info);
--# end
--# expr-done 1954
Local_Discriminants := Fld_1; Inc_Ref (Local_Discriminants);
--# bind discriminants Local_Discriminants
--# expr-start 1967 '<Let at nodes.lkt:2847:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2847'
--# scope-start
--# expr-start 1963 '<Map at nodes.lkt:2847:37>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2847'
   --# expr-start 1955 '<FieldAccess for match_formals at nodes.lkt:2847:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2847'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Local_Discriminants, Params => Assocs, Is_Dottable_Subp => False);
--# end
--# expr-done 1955
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
--# expr-start 1958 '<Not at nodes.lkt:2848:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
--# expr-start 1957 '<IsNull at nodes.lkt:2848:25>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
--# expr-start 1956 '<FieldAccess for formal at nodes.lkt:2848:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
Fld_3 := Item_0.Formal;
--# expr-done 1956
Is_Null := Fld_3.Node = null; 
--# expr-done 1957
Not_Val := not (Is_Null); 
--# expr-done 1958
if Not_Val then
   --# expr-start 1962 '<Contains at nodes.lkt:2848:47>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Local_Discriminants;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_3
      --# expr-start 1961 '<Eq at ???>' Is_Equiv None
Cast_Expr := Item_3; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1960 '<FieldAccess for formal_decl at nodes.lkt:2848:70>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
--# expr-start 1959 '<FieldAccess for formal at nodes.lkt:2848:70>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2848'
Fld_4 := Item_0.Formal;
--# expr-done 1959
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1960
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1961
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 1962
   If_Result := Quantifier_Result;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_0;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1963
Local_Discriminants_Matches := Map_Result; Inc_Ref (Local_Discriminants_Matches);
--# bind discriminants_matches Local_Discriminants_Matches
--# expr-start 1966 '<FieldAccess for abstract_formal_params_impl at nodes.lkt:2855:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2855'
--# expr-start 1965 '<FieldAccess for comps at nodes.lkt:2855:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2855'
--# expr-start 1964 '<FieldAccess for record_def at nodes.lkt:2855:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2855'
      if Local_Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Local_Td.Node, Origin => Origin, E_Info => Local_Td.Info);
--# end
--# expr-done 1964
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_7 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1965
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_8 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_7.Node, Discriminants => Local_Discriminants_Matches, Stop_Recurse_At => Stop_Recurse_At, Assocs => Assocs, E_Info => Fld_7.Info);
--# end
--# expr-done 1966
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
--# expr-done 1967
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1968
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1969
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_378;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_378;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_For_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_for_delta_assocs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2864'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Delta_Assocs
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_379 with Inline_Always;
            procedure Finalizer_Scope_2203 with Inline_Always;
            procedure Finalizer_Scope_2204 with Inline_Always;
            procedure Finalizer_Scope_2206 with Inline_Always;
      Local_Variant_Part_Components : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Self_Components : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Parent_Components : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_61 : Internal_Entity_Variant_Part;
Var_Expr_62 : Internal_Entity_Component_List;
Fld : Internal_Entity_Variant_Part;
Fld_1 : Internal_Entity_Variant_List;
Fld_2 : Bare_Variant_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Variant;
Bare_Item_0 : Bare_Variant;
Fld_4 : Internal_Entity_Component_List;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_6 : Internal_Entity_Ada_Node_List;
Fld_7 : Bare_Ada_Node_List;
Fld_8 : Internal_Entity_Info;
Item_4 : Internal_Entity;
Bare_Item_4 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Is_A : Boolean;
Map_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Entity_Component_List;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_379 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Variant_Part_Components);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Self_Components);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Parent_Components);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_379;
            procedure Finalizer_Scope_2203 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2203;
            procedure Finalizer_Scope_2204 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2204;
            procedure Finalizer_Scope_2206 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2206;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 1983 '<Let at nodes.lkt:2865:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2865'
--# scope-start
--# expr-start 1970 '<FieldAccess for variant_part at nodes.lkt:2865:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2865'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1970
Var_Expr_61 := Fld;
if Var_Expr_61 /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1974 '<Mapcat at nodes.lkt:2865:39>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2865'
   --# expr-start 1971 '<FieldAccess for variant at nodes.lkt:2865:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2865'
      if Var_Expr_61.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Variant_List (Node => Var_Expr_61.Node.Variant_Part_F_Variant, Info => Var_Expr_61.Info);
--# expr-done 1971
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind v Item_0
      --# expr-start 1973 '<FieldAccess for abstract_formal_params_for_delta_assocs at nodes.lkt:2866:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2866'
--# expr-start 1972 '<FieldAccess for components at nodes.lkt:2866:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2866'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Component_List (Node => Item_0.Node.Variant_F_Components, Info => Item_0.Info);
--# expr-done 1972
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_5 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1973
         for Item_To_Append of
               Fld_5.Items
         loop
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2204;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1974
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2203;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Variant_Part_Components := Result_Var; Inc_Ref (Local_Variant_Part_Components);
--# bind variant_part_components Local_Variant_Part_Components
--# expr-start 1982 '<Let at nodes.lkt:2868:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2868'
--# scope-start
--# expr-start 1978 '<FilterMap at nodes.lkt:2868:31>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2868'
   --# expr-start 1975 '<FieldAccess for components at nodes.lkt:2868:31>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2868'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1975
Fld_7 := Fld_6.Node;
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_7;
      begin
         for Untyped_Item_4 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_4 := Untyped_Item_4; 
Is_Null_1 := Bare_Item_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Bare_Item_4); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_4 := If_Result_1; 
   --# scope-start
               --# bind e Item_4
         --# expr-start 1976 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_4.Node /= null 
and then Item_4.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1976
         if Is_A then
      --# expr-start 1977 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_4; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1977
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 1978
Local_Self_Components := Map_Result_1; Inc_Ref (Local_Self_Components);
--# bind self_components Local_Self_Components
--# expr-start 1981 '<Let at nodes.lkt:2869:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2869'
--# scope-start
--# expr-start 1979 '<FieldAccess for parent_component_list at nodes.lkt:2869:33>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2869'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_9 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1979
Var_Expr_62 := Fld_9;
if Var_Expr_62 /= No_Entity_Component_List then
   --# scope-start
--# expr-start 1980 '<FieldAccess for abstract_formal_params_for_delta_assocs at nodes.lkt:2869:33>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2869'
      if Var_Expr_62.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_10 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Var_Expr_62.Node, E_Info => Var_Expr_62.Info);
--# end
--# expr-done 1980
Scope_Result_1 := Fld_10;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2206;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Parent_Components := Result_Var_1; Inc_Ref (Local_Parent_Components);
--# bind parent_components Local_Parent_Components
Concat_Result := Concat (Local_Variant_Part_Components, Local_Self_Components); 
Concat_Result_1 := Concat (Concat_Result, Local_Parent_Components); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1981
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1982
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1983
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_379;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_379;
                     Finalizer_Scope_2203;
                     Finalizer_Scope_2204;
                     Finalizer_Scope_2206;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_For_Delta_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2891'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_Impl
  (Node : Bare_Component_List
      ; Discriminants : Internal_Param_Match_Array_Access
      ; Include_Discriminants : Boolean
         := True
      ; Recurse : Boolean
         := True
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Assocs : Internal_Entity_Assoc_List
         := No_Entity_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind discriminants Discriminants
   --# bind include_discriminants Include_Discriminants
   --# bind recurse Recurse
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind assocs Assocs
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_380 with Inline_Always;
            procedure Finalizer_Scope_2208 with Inline_Always;
            procedure Finalizer_Scope_2209 with Inline_Always;
            procedure Finalizer_Scope_2211 with Inline_Always;
      Local_Self_Comps : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Ret : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_0 : Internal_Entity_Component_List;
Var_Expr_1 : Internal_Entity_Composite_Constraint;
Var_Expr_2 : Internal_Entity_Type_Decl;
Var_Expr_63 : Internal_Entity_Variant_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_5 : Internal_Entity;
Bare_Item_5 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Component_List;
Fld_6 : Internal_Entity_Type_Decl;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_9 : Internal_Entity_Subtype_Indication;
Fld_10 : Internal_Entity_Constraint;
Cast_Result_2 : Internal_Entity_Composite_Constraint;
Fld_11 : Internal_Entity_Assoc_List;
Scope_Result_1 : Internal_Entity_Assoc_List;
Result_Var_1 : Internal_Entity_Assoc_List;
Fld_12 : Internal_Param_Match_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Entity_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_16 : Internal_Entity_Type_Decl;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_380 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Local_Self_Comps);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Local_Ret);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_380;
            procedure Finalizer_Scope_2208 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2208;
            procedure Finalizer_Scope_2209 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2209;
            procedure Finalizer_Scope_2211 is
            begin
                     Dec_Ref (Fld_17);
            end Finalizer_Scope_2211;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2009 '<Let at nodes.lkt:2894:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2894'
--# scope-start
--# expr-start 1987 '<FilterMap at nodes.lkt:2894:26>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2894'
   --# expr-start 1984 '<FieldAccess for components at nodes.lkt:2894:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2894'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1984
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_5 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_5 := Untyped_Item_5; 
Is_Null := Bare_Item_5 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_5); 
   If_Result := As_Entity;
end if;
                  Item_5 := If_Result; 
   --# scope-start
               --# bind e Item_5
         --# expr-start 1985 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_5.Node /= null 
and then Item_5.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1985
         if Is_A then
      --# expr-start 1986 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_5; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1986
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 1987
--# expr-start 1988 '<FieldAccess for variant_part at nodes.lkt:2894:70>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2894'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1988
Var_Expr_63 := Fld_3;
if Var_Expr_63 /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1989 '<FieldAccess for get_components at nodes.lkt:2894:70>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2894'
      if Var_Expr_63.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start VariantPart.get_components
Fld_4 := Libadalang.Implementation.Variant_Part_P_Get_Components (Node => Var_Expr_63.Node, Discriminants => Discriminants, E_Info => Var_Expr_63.Info);
--# end
--# expr-done 1989
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2208;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
Local_Self_Comps := Concat_Result; Inc_Ref (Local_Self_Comps);
--# bind self_comps Local_Self_Comps
--# expr-start 2008 '<Let at nodes.lkt:2899:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2899'
--# scope-start
--# expr-start 2004 '<If at nodes.lkt:2899:19>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2899'
if Recurse then
--# expr-start 1990 '<FieldAccess for parent_component_list at nodes.lkt:2899:35>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2899'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_5 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1990
Var_Expr_0 := Fld_5;
if Var_Expr_0 /= No_Entity_Component_List then
   --# scope-start
   --# bind pcl Var_Expr_0
--# expr-start 2003 '<If at nodes.lkt:2900:22>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2900'
--# expr-start 1992 '<FieldAccess for matching_type at nodes.lkt:2900:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2900'
--# expr-start 1991 '<FieldAccess for type_decl at nodes.lkt:2900:25>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2900'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_14 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1991
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_14.Node, Expected_Type => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Fld_14.Info);
--# end
--# expr-done 1992
if Fld_15 then
   If_Result_1 := Local_Self_Comps;
else
   --# expr-start 2002 '<FieldAccess for abstract_formal_params_impl at nodes.lkt:2900:93>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2900'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2001 '<FieldAccess for match_formals at nodes.lkt:2901:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2901'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 1994 '<FieldAccess for discriminants_list at nodes.lkt:2902:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
--# expr-start 1993 '<FieldAccess for type_decl at nodes.lkt:2902:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_6 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 1993
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_6.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1994
--# expr-start 1999 '<Cast to CompositeConstraint at nodes.lkt:2902:74>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
--# expr-start 1998 '<FieldAccess for constraint at nodes.lkt:2902:74>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
--# expr-start 1997 '<FieldAccess for subtype_indication at nodes.lkt:2902:74>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
--# expr-start 1996 '<Cast to DerivedTypeDef at nodes.lkt:2902:74>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
--# expr-start 1995 '<FieldAccess for type_def at nodes.lkt:2902:74>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2902'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld_8 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1995
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1996
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Subtype_Indication (Node => Cast_Result_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Cast_Result_1.Info);
--# expr-done 1997
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Constraint (Node => Fld_9.Node.Subtype_Indication_F_Constraint, Info => Fld_9.Info);
--# expr-done 1998
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Composite_Constraint_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Composite_Constraint
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_2 := No_Entity_Composite_Constraint;
   end if;
--# expr-done 1999
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Composite_Constraint then
   --# scope-start
   --# bind cc Var_Expr_1
--# expr-start 2000 '<FieldAccess for constraints at nodes.lkt:2903:33>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2903'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Assoc_List (Node => Var_Expr_1.Node.Composite_Constraint_F_Constraints, Info => Var_Expr_1.Info);
--# expr-done 2000
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Assocs;
end if;
--# property-call-start AdaNode.match_formals
Fld_12 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Var_Expr_0.Node, Formal_Params => Fld_7, Params => Result_Var_1, Is_Dottable_Subp => False);
--# end
--# expr-done 2001
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_13 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Var_Expr_0.Node, Discriminants => Fld_12, Include_Discriminants => False, Stop_Recurse_At => Stop_Recurse_At, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2002
Concat_Result_1 := Concat (Fld_13, Local_Self_Comps); 
   If_Result_1 := Concat_Result_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2003
Scope_Result_2 := If_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2209;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Local_Self_Comps;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := Local_Self_Comps;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2004
Local_Ret := If_Result_2; Inc_Ref (Local_Ret);
--# bind ret Local_Ret
--# expr-start 2007 '<If at nodes.lkt:2913:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2913'
if Include_Discriminants then
--# expr-start 2005 '<FieldAccess for type_decl at nodes.lkt:2913:39>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2913'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_16 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2005
Var_Expr_2 := Fld_16;
if Var_Expr_2 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind decl Var_Expr_2
--# expr-start 2006 '<FieldAccess for discriminants_list at nodes.lkt:2914:23>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2914'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_2.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2006
Scope_Result_3 := Fld_17;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2211;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_2 := Concat (Result_Var_3, Local_Ret); 
   If_Result_3 := Concat_Result_2;
else
   If_Result_3 := Local_Ret;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2007
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 2008
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2009
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_380;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_380;
                     Finalizer_Scope_2208;
                     Finalizer_Scope_2209;
                     Finalizer_Scope_2211;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params_Impl;
--# end
--# property-start ComponentList.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2918'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_381 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_381 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_381;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2010 '<FieldAccess for abstract_formal_params_impl at nodes.lkt:2919:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2919'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Ent.Node, Discriminants => No_Internal_Param_Match_Array_Type, E_Info => Ent.Info);
--# end
--# expr-done 2010
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_381;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_381;
            raise;
      end;
   return Property_Result;
end Component_List_P_Abstract_Formal_Params;
--# end
--# property-start ComponentList.shapes 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2922'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Shapes
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_382 with Inline_Always;
            procedure Finalizer_Scope_2213 with Inline_Always;
            procedure Finalizer_Scope_2214 with Inline_Always;
            procedure Finalizer_Scope_2215 with Inline_Always;
      Local_Self_Comps : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_0 : Internal_Entity_Variant_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_6 : Internal_Entity;
Bare_Item_6 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
Item_1 : Internal_Entity_Variant;
Bare_Item_1 : Bare_Variant;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Shape_Array_Access;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Internal_Entity_Alternatives_List;
New_Struct : Internal_Discriminant_Values;
Array_Lit : Internal_Discriminant_Values_Array_Access;
Fld_12 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_1 : Internal_Shape_Array_Access;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Variant;
Map_Result_2 : Internal_Shape_Array_Access;
Scope_Result : Internal_Shape_Array_Access;
New_Struct_2 : Internal_Shape := No_Shape;
Array_Lit_1 : Internal_Shape_Array_Access;
Result_Var : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_382 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Self_Comps);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_382;
            procedure Finalizer_Scope_2213 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2213;
            procedure Finalizer_Scope_2214 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2214;
            procedure Finalizer_Scope_2215 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2215;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2030 '<Let at nodes.lkt:2923:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2923'
--# scope-start
--# expr-start 2014 '<FilterMap at nodes.lkt:2923:26>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2923'
   --# expr-start 2011 '<FieldAccess for components at nodes.lkt:2923:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2923'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 2011
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_6 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_6 := Untyped_Item_6; 
Is_Null := Bare_Item_6 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_6); 
   If_Result := As_Entity;
end if;
                  Item_6 := If_Result; 
   --# scope-start
               --# bind e Item_6
         --# expr-start 2012 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_6.Node /= null 
and then Item_6.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 2012
         if Is_A then
      --# expr-start 2013 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_6; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 2013
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2014
Local_Self_Comps := Map_Result; Inc_Ref (Local_Self_Comps);
--# bind self_comps Local_Self_Comps
--# expr-start 2015 '<FieldAccess for variant_part at nodes.lkt:2925:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2925'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 2015
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Variant_Part then
   --# scope-start
   --# bind vpart Var_Expr_0
--# expr-start 2027 '<Mapcat at nodes.lkt:2926:24>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2926'
   --# expr-start 2016 '<FieldAccess for variant at nodes.lkt:2926:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2926'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Var_Expr_0.Node.Variant_Part_F_Variant, Info => Var_Expr_0.Info);
--# expr-done 2016
Fld_5 := Fld_4.Node;
      declare
         Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result_2 := Create_Internal_Shape_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_1); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_1 := If_Result_1; 
   --# scope-start
               --# bind v Item_1
      --# expr-start 2026 '<Map at nodes.lkt:2927:24>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2927'
   --# expr-start 2018 '<FieldAccess for shapes at nodes.lkt:2927:24>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2927'
--# expr-start 2017 '<FieldAccess for components at nodes.lkt:2927:24>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2927'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Component_List (Node => Item_1.Node.Variant_F_Components, Info => Item_1.Info);
--# expr-done 2017
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.shapes
Fld_8 := Libadalang.Implementation.Component_List_P_Shapes (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 2018
      declare
         Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_8;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_2
      --# expr-start 2025 '<New Shape at nodes.lkt:2928:28>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2928'
--# expr-start 2019 '<FieldAccess for components at nodes.lkt:2929:49>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2929'
Fld_9 := Item_2.Components;
Inc_Ref (Fld_9);
--# expr-done 2019
Concat_Result := Concat (Local_Self_Comps, Fld_9); 
--# expr-start 2023 '<ArrayLiteral at nodes.lkt:2929:84>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2929'
--# expr-start 2022 '<New DiscriminantValues at nodes.lkt:2929:85>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2929'
--# expr-start 2020 '<FieldAccess for discr_name at nodes.lkt:2930:42>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2930'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Var_Expr_0.Node.Variant_Part_F_Discr_Name, Info => Var_Expr_0.Info);
--# expr-done 2020
--# expr-start 2021 '<FieldAccess for choices at nodes.lkt:2930:67>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2930'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Alternatives_List (Node => Item_1.Node.Variant_F_Choices, Info => Item_1.Info);
--# expr-done 2021
New_Struct := (Discriminant => Fld_10, Values => Fld_11); 
--# expr-done 2022
Array_Lit := Create_Internal_Discriminant_Values_Array (Internal_Internal_Discriminant_Values_Array'(1 => New_Struct)); 
--# expr-done 2023
--# expr-start 2024 '<FieldAccess for discriminants_values at nodes.lkt:2931:30>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2931'
Fld_12 := Item_2.Discriminants_Values;
Inc_Ref (Fld_12);
--# expr-done 2024
Concat_Result_1 := Concat (Array_Lit, Fld_12); 
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct_1 := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 2025
         declare
            Item_To_Append : constant Internal_Shape := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2215;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2026
         for Item_To_Append of
               Map_Result_1.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2214;
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 2027
Scope_Result := Map_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2213;
   Result_Var := Scope_Result;
else
   --# expr-start 2029 '<ArrayLiteral at nodes.lkt:2934:28>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2934'
--# expr-start 2028 '<New Shape at nodes.lkt:2934:29>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2934'
Inc_Ref (Local_Self_Comps);
Inc_Ref (No_Internal_Discriminant_Values_Array_Type);
New_Struct_2 := (Components => Local_Self_Comps, Discriminants_Values => No_Internal_Discriminant_Values_Array_Type); 
--# expr-done 2028
Array_Lit_1 := Create_Internal_Shape_Array (Internal_Internal_Shape_Array'(1 => New_Struct_2)); 
--# expr-done 2029
   Result_Var := Array_Lit_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2030
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_382;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_382;
                     Finalizer_Scope_2213;
                     Finalizer_Scope_2214;
                     Finalizer_Scope_2215;
            raise;
      end;
   return Property_Result;
end Component_List_P_Shapes;
--# end
--# property-start ComponentList.has_limited_component 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2942'
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Has_Limited_Component
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Component_List  := Bare_Component_List (Node);
     Ent : Internal_Entity_Component_List :=
       Internal_Entity_Component_List'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_64 : Internal_Entity_Base_Formal_Param_Decl;
Var_Expr_65 : Internal_Entity_Base_Type_Decl;
Var_Expr_66 : Internal_Entity_Variant_Part;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Quantifier_Result : Boolean;
Fld_5 : Internal_Entity_Variant_Part;
Fld_6 : Internal_Entity_Variant_List;
Fld_7 : Bare_Variant_List;
Fld_8 : Internal_Entity_Info;
Item_1 : Internal_Entity_Variant;
Bare_Item_1 : Bare_Variant;
Fld_9 : Internal_Entity_Component_List;
Fld_10 : Boolean;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Variant;
Quantifier_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2041 '<bind at nodes.lkt:2943:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2943'
Bound_Origin := No_Bare_Ada_Node; 
--# expr-start 2035 '<AnyQuantifier at nodes.lkt:2945:9>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2945'
   --# expr-start 2031 '<FieldAccess for components at nodes.lkt:2945:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2945'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 2031
Fld_1 := Fld.Node;
      Quantifier_Result := False;
      if Fld_1 = null then
      null;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind c Item_0
--# expr-start 2032 '<Cast to BaseFormalParamDecl at nodes.lkt:2946:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2946'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 2032
Var_Expr_64 := Cast_Result;
if Var_Expr_64 /= No_Entity_Base_Formal_Param_Decl then
   --# scope-start
--# expr-start 2033 '<FieldAccess for formal_type at nodes.lkt:2946:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2946'
      if Var_Expr_64.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Var_Expr_64.Node, Origin => Bound_Origin, E_Info => Var_Expr_64.Info);
--# end
--# expr-done 2033
Var_Expr_65 := Fld_3;
if Var_Expr_65 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2034 '<FieldAccess for is_limited_type at nodes.lkt:2946:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2946'
      if Var_Expr_65.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Var_Expr_65.Node, E_Info => Var_Expr_65.Info);
--# end
--# expr-done 2034
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      Quantifier_Result := Result_Var_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 2035
if Quantifier_Result then
   If_Result_2 := True;
else
--# expr-start 2036 '<FieldAccess for variant_part at nodes.lkt:2947:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2947'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 2036
Var_Expr_66 := Fld_5;
if Var_Expr_66 /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 2040 '<AnyQuantifier at nodes.lkt:2947:14>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2947'
   --# expr-start 2037 '<FieldAccess for variant at nodes.lkt:2947:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2947'
      if Var_Expr_66.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Variant_List (Node => Var_Expr_66.Node.Variant_Part_F_Variant, Info => Var_Expr_66.Info);
--# expr-done 2037
Fld_7 := Fld_6.Node;
      Quantifier_Result_1 := False;
      if Fld_7 = null then
      null;
      else
      declare
         Collection : constant Bare_Variant_List := Fld_7;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Variant;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Bare_Item_1); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_1 := If_Result_1; 
   --# scope-start
               --# bind v Item_1
      --# expr-start 2039 '<FieldAccess for has_limited_component at nodes.lkt:2948:20>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2948'
--# expr-start 2038 '<FieldAccess for components at nodes.lkt:2948:20>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2948'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Component_List (Node => Item_1.Node.Variant_F_Components, Info => Item_1.Info);
--# expr-done 2038
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.has_limited_component
Fld_10 := Libadalang.Implementation.Component_List_P_Has_Limited_Component (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 2039
      Quantifier_Result_1 := Fld_10;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 2040
Scope_Result_2 := Quantifier_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 2041
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_List_P_Has_Limited_Component;
--# end
   --
   --  Primitives for Bare_Discriminant_Part
   --
--# property-start DiscriminantPart.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2956'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Discriminant_Part  := Bare_Discriminant_Part (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Known_Discriminant_Part
   --
      procedure Initialize_Fields_For_Known_Discriminant_Part
        (Self : Bare_Known_Discriminant_Part
         ; Known_Discriminant_Part_F_Discr_Specs : Bare_Discriminant_Spec_List
        ) is
      begin
            Self.Known_Discriminant_Part_F_Discr_Specs := Known_Discriminant_Part_F_Discr_Specs;
      end Initialize_Fields_For_Known_Discriminant_Part;
   function Known_Discriminant_Part_F_Discr_Specs
     (Node : Bare_Known_Discriminant_Part) return Bare_Discriminant_Spec_List
   is
   begin
         return Node.Known_Discriminant_Part_F_Discr_Specs;
   end;
--# property-start KnownDiscriminantPart.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2964'
pragma Warnings (Off, "is not referenced");
 function Known_Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Known_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Known_Discriminant_Part  := Bare_Known_Discriminant_Part (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_385 with Inline_Always;
      Fld : Bare_Discriminant_Spec_List;
Item_0 : Bare_Discriminant_Spec;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Base_Formal_Param_Decl;
As_Entity : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_385 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_385;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2045 '<Map at nodes.lkt:2964:72>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2964'
   --# expr-start 2042 '<FieldAccess for discr_specs at nodes.lkt:2964:72>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2964'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Known_Discriminant_Part_F_Discr_Specs;
--# expr-done 2042
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Spec_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind e Item_0
      --# expr-start 2044 '<as_entity at nodes.lkt:2965:16>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2965'
--# expr-start 2043 '<Cast to BaseFormalParamDecl at nodes.lkt:2965:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2965'
Cast_Expr := Item_0; 
      Cast_Result := Cast_Expr;
--# expr-done 2043
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2044
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := If_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2045
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_385;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_385;
            raise;
      end;
   return Property_Result;
end Known_Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Unknown_Discriminant_Part
   --
   --
   --  Primitives for Bare_Entry_Completion_Formal_Params
   --
      procedure Initialize_Fields_For_Entry_Completion_Formal_Params
        (Self : Bare_Entry_Completion_Formal_Params
         ; Entry_Completion_Formal_Params_F_Params : Bare_Params
        ) is
      begin
            Self.Entry_Completion_Formal_Params_F_Params := Entry_Completion_Formal_Params_F_Params;
      end Initialize_Fields_For_Entry_Completion_Formal_Params;
   function Entry_Completion_Formal_Params_F_Params
     (Node : Bare_Entry_Completion_Formal_Params) return Bare_Params
   is
   begin
         return Node.Entry_Completion_Formal_Params_F_Params;
   end;
--# property-start EntryCompletionFormalParams.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2978'
pragma Warnings (Off, "is not referenced");
 function Entry_Completion_Formal_Params_P_Abstract_Formal_Params
  (Node : Bare_Entry_Completion_Formal_Params
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Entry_Completion_Formal_Params  := Bare_Entry_Completion_Formal_Params (Node);
     Ent : Internal_Entity_Entry_Completion_Formal_Params :=
       Internal_Entity_Entry_Completion_Formal_Params'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_386 with Inline_Always;
            procedure Finalizer_Scope_2222 with Inline_Always;
      Var_Expr_67 : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Param_Spec;
Bare_Item_0 : Bare_Param_Spec;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_386 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_386;
            procedure Finalizer_Scope_2222 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2222;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 2046 '<FieldAccess for params at nodes.lkt:2979:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2979'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Completion_Formal_Params_F_Params, Info => Ent.Info);
--# expr-done 2046
Var_Expr_67 := Fld;
if Var_Expr_67 /= No_Entity_Params then
   --# scope-start
--# expr-start 2049 '<Map at nodes.lkt:2979:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2979'
   --# expr-start 2047 '<FieldAccess for params at nodes.lkt:2979:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2979'
      if Var_Expr_67.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr_67.Node.Params_F_Params, Info => Var_Expr_67.Info);
--# expr-done 2047
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind p Item_0
      --# expr-start 2048 '<Cast to BaseFormalParamDecl at nodes.lkt:2979:40>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2979'
Cast_Expr := Item_0; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 2048
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2049
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2222;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_386;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_386;
                     Finalizer_Scope_2222;
            raise;
      end;
   return Property_Result;
end Entry_Completion_Formal_Params_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Part
   --
      procedure Initialize_Fields_For_Generic_Formal_Part
        (Self : Bare_Generic_Formal_Part
         ; Generic_Formal_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Generic_Formal_Part_F_Decls := Generic_Formal_Part_F_Decls;
      end Initialize_Fields_For_Generic_Formal_Part;
   function Generic_Formal_Part_F_Decls
     (Node : Bare_Generic_Formal_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Generic_Formal_Part_F_Decls;
   end;
--# property-start GenericFormalPart.abstract_formal_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2986'
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Abstract_Formal_Params
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Generic_Formal_Part  := Bare_Generic_Formal_Part (Node);
     Ent : Internal_Entity_Generic_Formal_Part :=
       Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_387 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_7 : Internal_Entity;
Bare_Item_7 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_387 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_387;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2053 '<FilterMap at nodes.lkt:2987:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2987'
   --# expr-start 2050 '<FieldAccess for decls at nodes.lkt:2987:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 2050
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_7 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_7 := Untyped_Item_7; 
Is_Null := Bare_Item_7 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_7); 
   If_Result := As_Entity;
end if;
                  Item_7 := If_Result; 
   --# scope-start
               --# bind e Item_7
         --# expr-start 2051 '<IsA BaseFormalParamDecl at ???>' Is_A None
Is_A := Item_7.Node /= null 
and then Item_7.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 2051
         if Is_A then
      --# expr-start 2052 '<Cast to BaseFormalParamDecl at ???>' Cast_Result None
Cast_Expr := Item_7; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 2052
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl := Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2053
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_387;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_387;
            raise;
      end;
   return Property_Result;
end Generic_Formal_Part_P_Abstract_Formal_Params;
--# end
--# property-start GenericFormalPart.use_clauses_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2991'
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Use_Clauses_Envs
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Formal_Part  := Bare_Generic_Formal_Part (Node);
     Ent : Internal_Entity_Generic_Formal_Part :=
       Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_388 with Inline_Always;
            procedure Finalizer_Scope_2225 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_3 : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_388 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_388;
            procedure Finalizer_Scope_2225 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2225;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2059 '<env_group at nodes.lkt:2991:42>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2991'
--# expr-start 2058 '<Map at nodes.lkt:2991:42>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2991'
   --# expr-start 2054 '<FieldAccess for decls at nodes.lkt:2991:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2991'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 2054
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind u Item_0
         --# expr-start 2055 '<IsA UseClause at nodes.lkt:2992:52>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2992'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Use_Clause; 
--# expr-done 2055
         if Is_A then
      --# expr-start 2057 '<FieldAccess for used_envs at nodes.lkt:2992:16>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2992'
--# expr-start 2056 '<Cast to UseClause at nodes.lkt:2992:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:2992'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 2056
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_3 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2057
         declare
            Item_To_Append : constant Lexical_Env := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_2225;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2058
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 2059
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_388;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_388;
                     Finalizer_Scope_2225;
            raise;
      end;
   return Property_Result;
end Generic_Formal_Part_P_Use_Clauses_Envs;
--# end
   --
   --  Primitives for Bare_Base_Record_Def
   --
      procedure Initialize_Fields_For_Base_Record_Def
        (Self : Bare_Base_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Self.Base_Record_Def_F_Components := Base_Record_Def_F_Components;
      end Initialize_Fields_For_Base_Record_Def;
   function Base_Record_Def_F_Components
     (Node : Bare_Base_Record_Def) return Bare_Component_List
   is
   begin
         return Node.Base_Record_Def_F_Components;
   end;
--# property-start BaseRecordDef.comps 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3002'
pragma Warnings (Off, "is not referenced");
 function Base_Record_Def_P_Comps
  (Node : Bare_Base_Record_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Base_Record_Def  := Bare_Base_Record_Def (Node);
     Ent : Internal_Entity_Base_Record_Def :=
       Internal_Entity_Base_Record_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Fld : Internal_Entity_Component_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2060 '<FieldAccess for components at nodes.lkt:3002:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_List (Node => Ent.Node.Base_Record_Def_F_Components, Info => Ent.Info);
--# expr-done 2060
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Record_Def_P_Comps;
--# end
   --
   --  Primitives for Bare_Null_Record_Def
   --
      procedure Initialize_Fields_For_Null_Record_Def
        (Self : Bare_Null_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Null_Record_Def;
   --
   --  Primitives for Bare_Record_Def
   --
      procedure Initialize_Fields_For_Record_Def
        (Self : Bare_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Record_Def;
   --
   --  Primitives for Bare_Basic_Assoc
   --
--# property-start '[dispatcher]BasicAssoc.expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Expr
  (Node : Bare_Basic_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.expr
                  Property_Result := Aggregate_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Composite_Constraint_Assoc =>
                  --# property-call-start CompositeConstraintAssoc.expr
                  Property_Result := Composite_Constraint_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.expr
                  Property_Result := Iterated_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.expr
                  Property_Result := Param_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Assoc_P_Expr;
--# end
--# property-start '[dispatcher]BasicAssoc.names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Names
  (Node : Bare_Basic_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.names
                  Property_Result := Aggregate_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Composite_Constraint_Assoc =>
                  --# property-call-start CompositeConstraintAssoc.names
                  Property_Result := Composite_Constraint_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.names
                  Property_Result := Iterated_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.names
                  Property_Result := Param_Assoc_P_Names
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Assoc_P_Names;
--# end
--# property-start BasicAssoc.get_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3026'
pragma Warnings (Off, "is not referenced");
 function Basic_Assoc_P_Get_Params
  (Node : Bare_Basic_Assoc
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Assoc  := Bare_Basic_Assoc (Node);
     Ent : Internal_Entity_Basic_Assoc :=
       Internal_Entity_Basic_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_159 with Inline_Always;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_159 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_159;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2068 '<Map at nodes.lkt:3027:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3027'
   --# expr-start 2063 '<FieldAccess for zip_with_params at nodes.lkt:3027:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3027'
--# expr-start 2062 '<Cast to AssocList at nodes.lkt:3027:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3027'
--# expr-start 2061 '<parent at nodes.lkt:3027:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3027'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2061
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2062
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Cast_Result.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 2063
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_0
         --# expr-start 2066 '<Eq at nodes.lkt:3028:36>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3028'
--# expr-start 2064 '<FieldAccess for actual at nodes.lkt:3028:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3028'
Fld_3 := Item_0.Actual;
--# expr-done 2064
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 2065 '<FieldAccess for expr at nodes.lkt:3028:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3028'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2065
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2066
         if Is_Equiv then
      --# expr-start 2067 '<FieldAccess for param at nodes.lkt:3028:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3028'
Fld_2 := Item_0.Param;
--# expr-done 2067
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Fld_2;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2068
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_159;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_159;
            raise;
      end;
   return Property_Result;
end Basic_Assoc_P_Get_Params;
--# end
   --
   --  Primitives for Bare_Aggregate_Assoc
   --
      procedure Initialize_Fields_For_Aggregate_Assoc
        (Self : Bare_Aggregate_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Aggregate_Assoc_F_Designators := Aggregate_Assoc_F_Designators;
            Self.Aggregate_Assoc_F_R_Expr := Aggregate_Assoc_F_R_Expr;
      end Initialize_Fields_For_Aggregate_Assoc;
   function Aggregate_Assoc_F_Designators
     (Node : Bare_Aggregate_Assoc) return Bare_Alternatives_List
   is
   begin
         return Node.Aggregate_Assoc_F_Designators;
   end;
   function Aggregate_Assoc_F_R_Expr
     (Node : Bare_Aggregate_Assoc) return Bare_Expr
   is
   begin
         return Node.Aggregate_Assoc_F_R_Expr;
   end;
--# property-start AggregateAssoc.expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3037'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Expr
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2069 '<FieldAccess for r_expr at nodes.lkt:3037:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3037'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aggregate_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 2069
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Expr;
--# end
--# property-start AggregateAssoc.names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3039'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Names
  (Node : Bare_Aggregate_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_391 with Inline_Always;
      Fld : Bare_Alternatives_List;
Item_0 : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_391 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_391;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2071 '<Map at nodes.lkt:3039:35>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3039'
   --# expr-start 2070 '<FieldAccess for designators at nodes.lkt:3039:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3039'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Aggregate_Assoc_F_Designators;
--# expr-done 2070
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind d Item_0
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_0;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2071
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_391;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_391;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Names;
--# end
--# property-start AggregateAssoc.xref_stop_resolution 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3042'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start AggregateAssoc.base_aggregate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3044'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Base_Aggregate
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2074 '<Cast to BaseAggregate at nodes.lkt:3045:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3045'
--# expr-start 2073 '<parent at nodes.lkt:3045:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3045'
--# expr-start 2072 '<parent at nodes.lkt:3045:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3045'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2072
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2073
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2074
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Base_Aggregate;
--# end
--# property-start AggregateAssoc.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3048'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_394 with Inline_Always;
      Local_Agg : Internal_Entity_Base_Aggregate;
Local_Mra : Internal_Multidim_Aggregate_Info;
Local_Td : Internal_Entity_Base_Type_Decl;
Local_Atd : Internal_Entity_Array_Type_Def;
Var_Expr_68 : Internal_Entity_Base_Type_Decl;
Var_Expr_69 : Internal_Entity_Aspect_Assoc;
Var_Expr_70 : Internal_Entity_Aspect_Assoc;
Var_Expr_71 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Equation;
Fld_10 : Logic_Equation;
Fld_11 : Logic_Equation;
Fld_12 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Is_Null : Boolean;
If_Result_1 : Logic_Equation;
Is_A : Boolean;
If_Result_2 : Logic_Equation;
Fld_16 : Internal_Entity;
Is_A_1 : Boolean;
If_Result_3 : Logic_Equation;
Is_A_2 : Boolean;
Sym : Symbol_Type;
Fld_17 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
Fld_18 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Aspect_Assoc;
Fld_19 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_20 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_6 : Logic_Equation;
Fld_21 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Aspect_Assoc;
Fld_22 : Internal_Entity_Name;
Sym_2 : Symbol_Type;
Fld_23 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_7 : Logic_Equation;
Sym_3 : Symbol_Type;
Fld_24 : Boolean;
If_Result_8 : Logic_Equation;
Sym_4 : Symbol_Type;
Fld_25 : Boolean;
If_Result_9 : Logic_Equation;
Sym_5 : Symbol_Type;
Fld_26 : Boolean;
Sym_6 : Symbol_Type;
Fld_27 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Logic_Equation;
Sym_7 : Symbol_Type;
Fld_28 : Boolean;
Sym_8 : Symbol_Type;
Fld_29 : Boolean;
If_Result_12 : Boolean;
If_Result_13 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_394 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (If_Result_13);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_394;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2137 '<Let at nodes.lkt:3049:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3049'
--# scope-start
--# expr-start 2075 '<FieldAccess for base_aggregate at nodes.lkt:3049:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3049'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2075
Local_Agg := Fld; 
--# bind agg Local_Agg
--# expr-start 2136 '<Let at nodes.lkt:3050:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3050'
--# scope-start
--# expr-start 2076 '<FieldAccess for multidim_root_aggregate at nodes.lkt:3050:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3050'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Local_Agg.Node, Origin => Origin, E_Info => Local_Agg.Info);
--# end
--# expr-done 2076
Local_Mra := Fld_1; 
--# bind mra Local_Mra
--# expr-start 2135 '<Let at nodes.lkt:3054:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3054'
--# scope-start
--# expr-start 2083 '<If at nodes.lkt:3054:18>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3054'
--# expr-start 2078 '<Not at nodes.lkt:3054:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3054'
--# expr-start 2077 '<IsNull at nodes.lkt:3054:25>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3054'
Is_Equal := Local_Mra = No_Multidim_Aggregate_Info; 
--# expr-done 2077
Not_Val := not (Is_Equal); 
--# expr-done 2078
if Not_Val then
   --# expr-start 2079 '<FieldAccess for typ at nodes.lkt:3055:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3055'
Fld_2 := Local_Mra.Typ;
--# expr-done 2079
   If_Result := Fld_2;
else
   --# expr-start 2082 '<Cast to BaseTypeDecl at nodes.lkt:3057:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3057'
--# expr-start 2081 '<FieldAccess for type_val at nodes.lkt:3057:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3057'
--# expr-start 2080 '<FieldAccess for base_aggregate at nodes.lkt:3057:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3057'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld_3 := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2080
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_4 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_3.Node);
--# end
--# expr-done 2081
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2082
   If_Result := Cast_Result;
end if;
--# expr-done 2083
Local_Td := If_Result; 
--# bind td Local_Td
--# expr-start 2134 '<Let at nodes.lkt:3059:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3059'
--# scope-start
Var_Expr_68 := Local_Td;
if Var_Expr_68 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2084 '<FieldAccess for array_def at nodes.lkt:3059:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3059'
      if Var_Expr_68.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr_68.Node, Origin => Origin, E_Info => Var_Expr_68.Info);
--# end
--# expr-done 2084
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Local_Atd := Result_Var; 
--# bind atd Local_Atd
--# expr-start 2133 '<If at nodes.lkt:3061:9>' If_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2086 '<FieldAccess for in_aspect at nodes.lkt:3061:12>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2085 '<Symbol Global at nodes.lkt:3061:26>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 2085
--# property-call-start AdaNode.in_aspect
Fld_28 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_7);
--# end
--# expr-done 2086
if Fld_28 then
   If_Result_12 := True;
else
   --# expr-start 2088 '<FieldAccess for in_aspect at nodes.lkt:3061:40>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2087 '<Symbol Refined_Global at nodes.lkt:3061:54>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 2087
--# property-call-start AdaNode.in_aspect
Fld_29 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_8);
--# end
--# expr-done 2088
   If_Result_12 := Fld_29;
end if;
if If_Result_12 then
   --# expr-start 2089 '<FieldAccess for globals_assoc_equation at nodes.lkt:3062:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3062'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.globals_assoc_equation
Fld_6 := Libadalang.Implementation.Aggregate_Assoc_P_Globals_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2089
   If_Result_13 := Fld_6;
else
   --# expr-start 2132 '<If at nodes.lkt:3061:9>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2091 '<FieldAccess for in_aspect at nodes.lkt:3063:14>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3063'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2090 '<Symbol Depends at nodes.lkt:3063:28>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3063'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 2090
--# property-call-start AdaNode.in_aspect
Fld_26 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_5);
--# end
--# expr-done 2091
if Fld_26 then
   If_Result_10 := True;
else
   --# expr-start 2093 '<FieldAccess for in_aspect at nodes.lkt:3063:43>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3063'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2092 '<Symbol Refined_Depends at nodes.lkt:3063:57>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3063'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 2092
--# property-call-start AdaNode.in_aspect
Fld_27 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_6);
--# end
--# expr-done 2093
   If_Result_10 := Fld_27;
end if;
if If_Result_10 then
   --# expr-start 2094 '<FieldAccess for depends_assoc_equation at nodes.lkt:3064:18>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3064'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.depends_assoc_equation
Fld_7 := Libadalang.Implementation.Aggregate_Assoc_P_Depends_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2094
   If_Result_11 := Fld_7;
else
   --# expr-start 2131 '<If at nodes.lkt:3061:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2096 '<FieldAccess for in_aspect at nodes.lkt:3065:14>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3065'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2095 '<Symbol Test_Case at nodes.lkt:3065:28>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3065'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 2095
--# property-call-start AdaNode.in_aspect
Fld_25 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_4);
--# end
--# expr-done 2096
if Fld_25 then
   --# expr-start 2097 '<FieldAccess for test_case_assoc_equation at nodes.lkt:3066:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3066'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.test_case_assoc_equation
Fld_8 := Libadalang.Implementation.Aggregate_Assoc_P_Test_Case_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2097
   If_Result_9 := Fld_8;
else
   --# expr-start 2130 '<If at nodes.lkt:3061:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2099 '<FieldAccess for in_aspect at nodes.lkt:3067:14>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3067'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2098 '<Symbol Refined_State at nodes.lkt:3067:28>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3067'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 2098
--# property-call-start AdaNode.in_aspect
Fld_24 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Local_Agg.Node, Name => Sym_3);
--# end
--# expr-done 2099
if Fld_24 then
   --# expr-start 2100 '<FieldAccess for exprs_assoc_equation at nodes.lkt:3070:13>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3070'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.exprs_assoc_equation
Fld_9 := Libadalang.Implementation.Aggregate_Assoc_P_Exprs_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2100
   If_Result_8 := Fld_9;
else
   --# expr-start 2129 '<If at nodes.lkt:3061:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2102 '<Cast to AspectAssoc at nodes.lkt:3076:14>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3076'
--# expr-start 2101 '<parent at nodes.lkt:3076:14>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3076'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Parent (Node => Local_Agg.Node, E_Info => Local_Agg.Info);
--# expr-done 2101
   if Fld_21.Node = null
      or else Fld_21.Node.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Aspect_Assoc
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
   else
         Cast_Result_2 := No_Entity_Aspect_Assoc;
   end if;
--# expr-done 2102
Var_Expr_69 := Cast_Result_2;
if Var_Expr_69 /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 2105 '<FieldAccess for name_is at nodes.lkt:3076:14>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3076'
--# expr-start 2103 '<FieldAccess for id at nodes.lkt:3076:14>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3076'
      if Var_Expr_69.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Name (Node => Var_Expr_69.Node.Aspect_Assoc_F_Id, Info => Var_Expr_69.Info);
--# expr-done 2103
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2104 '<Symbol Contract_Cases at nodes.lkt:3076:53>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3076'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 2104
--# property-call-start Name.name_is
Fld_23 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_22.Node, Sym => Sym_2);
--# end
--# expr-done 2105
Scope_Result_3 := Fld_23;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 2106 '<FieldAccess for contract_cases_assoc_equation at nodes.lkt:3077:13>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3077'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.contract_cases_assoc_equation
Fld_10 := Libadalang.Implementation.Aggregate_Assoc_P_Contract_Cases_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2106
   If_Result_7 := Fld_10;
else
   --# expr-start 2128 '<If at nodes.lkt:3061:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2108 '<Cast to AspectAssoc at nodes.lkt:3078:14>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3078'
--# expr-start 2107 '<parent at nodes.lkt:3078:14>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3078'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Parent (Node => Local_Agg.Node, E_Info => Local_Agg.Info);
--# expr-done 2107
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aspect_Assoc
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_1 := No_Entity_Aspect_Assoc;
   end if;
--# expr-done 2108
Var_Expr_70 := Cast_Result_1;
if Var_Expr_70 /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 2111 '<FieldAccess for name_is at nodes.lkt:3078:14>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3078'
--# expr-start 2109 '<FieldAccess for id at nodes.lkt:3078:14>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3078'
      if Var_Expr_70.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Var_Expr_70.Node.Aspect_Assoc_F_Id, Info => Var_Expr_70.Info);
--# expr-done 2109
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2110 '<Symbol Subprogram_Variant at nodes.lkt:3078:53>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3078'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Subprogram_Variant); 
--# expr-done 2110
--# property-call-start Name.name_is
Fld_20 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_19.Node, Sym => Sym_1);
--# end
--# expr-done 2111
Scope_Result_2 := Fld_20;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 2112 '<FieldAccess for subprogram_variant_assoc_equation at nodes.lkt:3079:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3079'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.subprogram_variant_assoc_equation
Fld_11 := Libadalang.Implementation.Aggregate_Assoc_P_Subprogram_Variant_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2112
   If_Result_6 := Fld_11;
else
   --# expr-start 2127 '<If at nodes.lkt:3061:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2113 '<IsA BracketAggregate at nodes.lkt:3080:14>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3080'
Is_A_2 := Local_Agg.Node /= null 
and then Local_Agg.Node.Kind in Ada_Bracket_Aggregate_Range; 
--# expr-done 2113
if Is_A_2 then
Var_Expr_71 := Local_Td;
if Var_Expr_71 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2115 '<FieldAccess for has_aspect at nodes.lkt:3080:42>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3080'
      if Var_Expr_71.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2114 '<Symbol Aggregate at nodes.lkt:3080:57>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3080'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 2114
--# property-call-start BasicDecl.has_aspect
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Var_Expr_71.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Var_Expr_71.Info);
--# end
--# expr-done 2115
Scope_Result_1 := Fld_17;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_4 := Result_Var_1;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 2116 '<FieldAccess for container_aggregate_equation at nodes.lkt:3081:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3081'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.container_aggregate_equation
Fld_12 := Libadalang.Implementation.Aggregate_Assoc_P_Container_Aggregate_Equation (Node => Ent.Node, Td => Local_Td, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2116
   If_Result_5 := Fld_12;
else
   --# expr-start 2126 '<If at nodes.lkt:3061:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2118 '<IsA AspectClause, AspectAssoc, PragmaArgumentAssoc at nodes.lkt:3082:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3082'
--# expr-start 2117 '<parent at nodes.lkt:3082:14>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3082'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Parent (Node => Local_Agg.Node, E_Info => Local_Agg.Info);
--# expr-done 2117
Is_A_1 := Fld_16.Node /= null 
and then Fld_16.Node.Kind in Ada_Aspect_Clause | Ada_Aspect_Assoc_Range | Ada_Pragma_Argument_Assoc_Range; 
--# expr-done 2118
if Is_A_1 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3083:18") else null)); 
   If_Result_3 := True_Rel;
else
   --# expr-start 2125 '<If at nodes.lkt:3061:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2119 '<IsA DeltaAggregate at nodes.lkt:3084:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3084'
Is_A := Local_Agg.Node /= null 
and then Local_Agg.Node.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 2119
if Is_A then
   --# expr-start 2120 '<FieldAccess for delta_aggregate_assoc_equation at nodes.lkt:3085:13>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3085'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.delta_aggregate_assoc_equation
Fld_13 := Libadalang.Implementation.Aggregate_Assoc_P_Delta_Aggregate_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2120
   If_Result_2 := Fld_13;
else
   --# expr-start 2124 '<If at nodes.lkt:3061:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3061'
--# expr-start 2121 '<IsNull at nodes.lkt:3086:14>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3086'
Is_Null := Local_Atd.Node = null; 
--# expr-done 2121
if Is_Null then
   --# expr-start 2122 '<FieldAccess for record_assoc_equation at nodes.lkt:3087:13>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3087'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.record_assoc_equation
Fld_14 := Libadalang.Implementation.Aggregate_Assoc_P_Record_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2122
   If_Result_1 := Fld_14;
else
   --# expr-start 2123 '<FieldAccess for array_assoc_equation at nodes.lkt:3089:13>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3089'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.array_assoc_equation
Fld_15 := Libadalang.Implementation.Aggregate_Assoc_P_Array_Assoc_Equation (Node => Ent.Node, Atd => Local_Atd, Mra => Local_Mra, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 2123
   If_Result_1 := Fld_15;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2124
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2125
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2126
   If_Result_5 := If_Result_3;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 2127
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 2128
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 2129
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 2130
   If_Result_9 := If_Result_8;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 2131
   If_Result_11 := If_Result_9;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 2132
   If_Result_13 := If_Result_11;
end if;
      Inc_Ref (If_Result_13);
--# expr-done 2133
Let_Result := If_Result_13; Inc_Ref (Let_Result);
--# end
--# expr-done 2134
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2135
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2136
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2137
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_394;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_394;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Xref_Equation;
--# end
--# property-start AggregateAssoc.container_aggregate_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3095'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Container_Aggregate_Equation
  (Node : Bare_Aggregate_Assoc
      ; Td : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind td Td
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_395 with Inline_Always;
      Local_Aggregate_Aspect : Internal_Entity_Aggregate;
Local_Entity_Name : Bare_Name;
Local_Element_Type : Internal_Entity_Base_Type_Decl;
Local_Key_Type : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_1 : Bare_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Is_Null_1 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_395 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_395;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2169 '<Let at nodes.lkt:3096:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3096'
--# scope-start
--# expr-start 2141 '<Cast to Aggregate at nodes.lkt:3096:32>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3096'
--# expr-start 2140 '<FieldAccess for value at nodes.lkt:3096:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3096'
--# expr-start 2139 '<FieldAccess for get_aspect at nodes.lkt:3096:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3096'
      if Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2138 '<Symbol Aggregate at nodes.lkt:3096:46>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3096'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 2138
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Td.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Td.Info);
--# end
--# expr-done 2139
Fld_1 := Fld.Value;
--# expr-done 2140
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 2141
Local_Aggregate_Aspect := Cast_Result; 
--# bind aggregate_aspect Local_Aggregate_Aspect
--# expr-start 2168 '<Let at nodes.lkt:3097:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3097'
--# scope-start
--# expr-start 2144 '<Cast to Name at nodes.lkt:3097:27>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3097'
--# expr-start 2143 '<at at nodes.lkt:3097:27>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3097'
--# expr-start 2142 '<FieldAccess for names at nodes.lkt:3097:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3097'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2142
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2143
   if Get_Result = null
      or else Get_Result.Kind in Ada_Name
   then
      Cast_Result_1 := Get_Result;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2144
Local_Entity_Name := Cast_Result_1; 
--# bind entity_name Local_Entity_Name
--# expr-start 2167 '<Let at nodes.lkt:3098:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3098'
--# scope-start
--# expr-start 2145 '<FieldAccess for element_type at nodes.lkt:3098:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3098'
      if Local_Aggregate_Aspect.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.element_type
Fld_3 := Libadalang.Implementation.Aggregate_P_Element_Type (Node => Local_Aggregate_Aspect.Node, E_Info => Local_Aggregate_Aspect.Info);
--# end
--# expr-done 2145
Local_Element_Type := Fld_3; 
--# bind element_type Local_Element_Type
--# expr-start 2166 '<Let at nodes.lkt:3099:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3099'
--# scope-start
--# expr-start 2146 '<FieldAccess for key_type at nodes.lkt:3099:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3099'
      if Local_Aggregate_Aspect.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.key_type
Fld_4 := Libadalang.Implementation.Aggregate_P_Key_Type (Node => Local_Aggregate_Aspect.Node, E_Info => Local_Aggregate_Aspect.Info);
--# end
--# expr-done 2146
Local_Key_Type := Fld_4; 
--# bind key_type Local_Key_Type
--# expr-start 2165 '<And at nodes.lkt:3101:9>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2155 '<And at nodes.lkt:3101:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2152 '<And at nodes.lkt:3101:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2149 '<Bind at nodes.lkt:3101:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2148 '<FieldAccess for expected_type_var at nodes.lkt:3101:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2147 '<FieldAccess for expr at nodes.lkt:3101:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2147
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5.Node);
--# expr-done 2148
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Cast_Expr := Local_Element_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3101:9") else null)); 
--# expr-done 2149
--# expr-start 2151 '<FieldAccess for sub_equation at nodes.lkt:3101:64>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2150 '<FieldAccess for expr at nodes.lkt:3101:64>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2150
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 2151
And_Pred := Create_And (Bind_Result, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3101:9") else null)); 
--# expr-done 2152
--# expr-start 2154 '<FieldAccess for matches_expected_type at nodes.lkt:3101:95>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
--# expr-start 2153 '<FieldAccess for expr at nodes.lkt:3101:95>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3101'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2153
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_9.Node, Origin => Origin);
--# end
--# expr-done 2154
And_Pred_1 := Create_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3101:9") else null)); 
--# expr-done 2155
--# expr-start 2164 '<If at nodes.lkt:3102:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
--# expr-start 2156 '<IsNull at nodes.lkt:3102:16>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
Is_Null_1 := Local_Entity_Name = null; 
--# expr-done 2156
if Is_Null_1 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3102:41") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 2163 '<And at nodes.lkt:3102:52>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
--# expr-start 2161 '<And at nodes.lkt:3102:52>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
--# expr-start 2158 '<Bind at nodes.lkt:3102:52>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
--# expr-start 2157 '<FieldAccess for expected_type_var at nodes.lkt:3102:56>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
      if Local_Entity_Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Local_Entity_Name);
--# expr-done 2157
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Cast_Expr_1 := Local_Key_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_11, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3102:52") else null)); 
--# expr-done 2158
--# expr-start 2160 '<FieldAccess for sub_equation at nodes.lkt:3102:103>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
--# expr-start 2159 '<as_entity at nodes.lkt:3102:103>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
Is_Null := Local_Entity_Name = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Local_Entity_Name); 
   If_Result := As_Entity;
end if;
--# expr-done 2159
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result.Info);
--# end
--# expr-done 2160
And_Pred_2 := Create_And (Bind_Result_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3102:52") else null)); 
--# expr-done 2161
--# expr-start 2162 '<FieldAccess for matches_expected_type at nodes.lkt:3102:144>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3102'
      if Local_Entity_Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Local_Entity_Name, Origin => Origin);
--# end
--# expr-done 2162
And_Pred_3 := Create_And (And_Pred_2, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3102:52") else null)); 
--# expr-done 2163
   If_Result_1 := And_Pred_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2164
And_Pred_4 := Create_And (And_Pred_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3101:9") else null)); 
--# expr-done 2165
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2166
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2167
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2168
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2169
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_395;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_395;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Container_Aggregate_Equation;
--# end
--# property-start AggregateAssoc.delta_aggregate_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3111'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Delta_Aggregate_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_396 with Inline_Always;
            procedure Finalizer_Scope_2234 with Inline_Always;
            procedure Finalizer_Scope_2238 with Inline_Always;
            procedure Finalizer_Scope_2239 with Inline_Always;
      Local_Agg : Internal_Entity_Base_Aggregate;
Local_Agg_Type : Internal_Entity_Base_Type_Decl;
Local_Positional_Assoc : Boolean;
Local_Agg_Components : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_1 : Boolean;
Local_Matches : Internal_Entity_Expr_Array_Access;
Var_Expr_3 : Bare_Name;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_72 : Bare_Name;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Name;
Fld_4 : Bare_Name;
Is_Null : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_5 : Boolean;
Not_Val_1 : Boolean;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_8 : Bare_Ada_Node_Array_Access;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_9 : Bare_Name;
Fld_10 : Internal_Entity_Name;
Fld_11 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Node_For_Entity : Bare_Expr;
As_Entity : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_2 : Internal_Entity_Expr;
Map_Result : Internal_Entity_Expr_Array_Access;
Fld_12 : Bare_Ada_Node_Array_Access;
Index_6 : Integer;
Get_Result : Internal_Entity_Expr;
Fld_13 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_4 : Bare_Expr;
As_Entity_1 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Cast_Expr_3 : Bare_Ada_Node;
Cast_Result_5 : Bare_Expr;
Fld_15 : Logic_Var;
Fld_16 : Bare_Ada_Node_Array_Access;
Get_Result_1 : Bare_Ada_Node;
Cast_Result_6 : Bare_Expr;
Fld_17 : Logic_Var;
Pred : Logic_Equation;
True_Rel : Logic_Equation;
Comp_Result : Boolean;
If_Result_4 : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr_4 : Bare_Ada_Node;
Cast_Result_7 : Bare_Expr;
Fld_18 : Logic_Var;
Fld_19 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Bind_Result : Logic_Equation;
Cast_Expr_5 : Bare_Ada_Node;
Cast_Result_9 : Bare_Expr;
Fld_20 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
If_Result_5 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Var;
Fld_23 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Var;
Fld_26 : Bare_Ada_Node_Array_Access;
Get_Result_2 : Bare_Ada_Node;
Cast_Result_11 : Bare_Expr;
Fld_27 : Logic_Var;
Bind_Result_2 : Logic_Equation;
If_Result_6 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Logic_Equation;
And_Pred_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_396 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Agg_Components);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Matches);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_29);
                     Dec_Ref (Fld_31);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_396;
            procedure Finalizer_Scope_2234 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2234;
            procedure Finalizer_Scope_2238 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2238;
            procedure Finalizer_Scope_2239 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2239;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2245 '<Let at nodes.lkt:3112:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3112'
--# scope-start
--# expr-start 2170 '<FieldAccess for base_aggregate at nodes.lkt:3112:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3112'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2170
Local_Agg := Fld; 
--# bind agg Local_Agg
--# expr-start 2244 '<Let at nodes.lkt:3113:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3113'
--# scope-start
--# expr-start 2172 '<Cast to BaseTypeDecl at nodes.lkt:3113:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3113'
--# expr-start 2171 '<FieldAccess for type_val at nodes.lkt:3113:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3113'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_1 := Libadalang.Implementation.Expr_P_Type_Val (Node => Local_Agg.Node);
--# end
--# expr-done 2171
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2172
Local_Agg_Type := Cast_Result; 
--# bind agg_type Local_Agg_Type
--# expr-start 2243 '<Let at nodes.lkt:3118:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3118'
--# scope-start
--# expr-start 2173 '<FieldAccess for is_array at nodes.lkt:3118:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3118'
      if Local_Agg_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Local_Agg_Type.Node, Origin => Origin, E_Info => Local_Agg_Type.Info);
--# end
--# expr-done 2173
if Fld_2 then
   --# expr-start 2179 '<Not at nodes.lkt:3118:56>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3118'
--# expr-start 2178 '<AnyQuantifier at nodes.lkt:3118:60>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3118'
   --# expr-start 2174 '<FieldAccess for names at nodes.lkt:3118:60>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3118'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2174
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
--# expr-start 2175 '<Cast to Name at nodes.lkt:3121:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3121'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Name
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2175
Var_Expr_72 := Cast_Result_1;
if Var_Expr_72 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2177 '<IsNull at nodes.lkt:3121:20>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3121'
--# expr-start 2176 '<FieldAccess for leftmost_name at nodes.lkt:3121:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3121'
      if Var_Expr_72 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_4 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Var_Expr_72);
--# end
--# expr-done 2176
Is_Null := Fld_4 = null; 
--# expr-done 2177
Scope_Result := Is_Null;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2178
Not_Val := not (Quantifier_Result); 
--# expr-done 2179
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Local_Positional_Assoc := If_Result; 
--# bind positional_assoc Local_Positional_Assoc
--# expr-start 2242 '<Let at nodes.lkt:3124:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3124'
--# scope-start
--# expr-start 2181 '<Not at nodes.lkt:3124:31>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3124'
--# expr-start 2180 '<FieldAccess for is_array at nodes.lkt:3124:35>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3124'
      if Local_Agg_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Local_Agg_Type.Node, Origin => Origin, E_Info => Local_Agg_Type.Info);
--# end
--# expr-done 2180
Not_Val_1 := not (Fld_5); 
--# expr-done 2181
Var_Expr_1 := Not_Val_1;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 2183 '<FieldAccess for unpack_formals at nodes.lkt:3125:20>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3125'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2182 '<FieldAccess for all_components at nodes.lkt:3125:40>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3125'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_components
Fld_6 := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Local_Agg.Node, Origin => Origin, Env => Env, E_Info => Local_Agg.Info);
--# end
--# expr-done 2182
--# property-call-start AdaNode.unpack_formals
Fld_7 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld_6);
--# end
--# expr-done 2183
Scope_Result_1 := Fld_7;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2234;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Agg_Components := Result_Var_1; Inc_Ref (Local_Agg_Components);
--# bind agg_components Local_Agg_Components
--# expr-start 2241 '<Let at nodes.lkt:3131:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3131'
--# scope-start
--# expr-start 2194 '<Map at nodes.lkt:3131:23>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3131'
   --# expr-start 2184 '<FieldAccess for names at nodes.lkt:3131:23>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3131'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2184
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_8;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 2193 '<If at nodes.lkt:3132:20>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3132'
--# expr-start 2185 '<IsA Name at nodes.lkt:3132:23>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3132'
Is_A := Item_2 /= null 
and then Item_2.Kind in Ada_Name; 
--# expr-done 2185
if Is_A then
--# expr-start 2187 '<FieldAccess for leftmost_name at nodes.lkt:3134:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3134'
--# expr-start 2186 '<Cast to Name at nodes.lkt:3134:17>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3134'
Cast_Expr_1 := Item_2; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Name
   then
      Cast_Result_2 := Cast_Expr_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2186
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_9 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Cast_Result_2);
--# end
--# expr-done 2187
Var_Expr_3 := Fld_9;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind id Var_Expr_3
--# expr-start 2190 '<Find at nodes.lkt:3135:29>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3135'
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Agg_Components;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_4
      --# expr-start 2189 '<FieldAccess for matches at nodes.lkt:3136:32>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3136'
--# expr-start 2188 '<FieldAccess for name at nodes.lkt:3136:32>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3136'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Item_4.Node.Defining_Name_F_Name, Info => Item_4.Info);
--# expr-done 2188
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => Fld_10.Node, N => Var_Expr_3);
--# end
--# expr-done 2189
      if Fld_11 then
         Find_Result := Item_4; 
      end if;
   --# end
               exit when Fld_11;
         end loop;
      end;
--# expr-done 2190
Scope_Result_2 := Find_Result;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
   If_Result_2 := Cast_Result_3;
else
   --# expr-start 2192 '<as_entity at nodes.lkt:3141:18>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3141'
--# expr-start 2191 '<No Expr at nodes.lkt:3141:18>' Node_For_Entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3141'
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 2191
Is_Null_1 := Node_For_Entity = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Expr;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 2192
   If_Result_2 := If_Result_1;
end if;
--# expr-done 2193
         declare
            Item_To_Append : constant Internal_Entity_Expr := If_Result_2;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2194
Local_Matches := Map_Result; Inc_Ref (Local_Matches);
--# bind matches Local_Matches
--# expr-start 2240 '<And at nodes.lkt:3144:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3144'
--# expr-start 2234 '<And at nodes.lkt:3144:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3144'
--# expr-start 2221 '<LogicAll at nodes.lkt:3144:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3144'
--# expr-start 2220 '<Map at nodes.lkt:3144:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3144'
   --# expr-start 2195 '<FieldAccess for names at nodes.lkt:3144:9>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3144'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2195
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
         Index_6 := 0;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_12;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_5
               --# bind i Index_6
      --# expr-start 2219 '<And at nodes.lkt:3145:23>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3145'
--# expr-start 2210 '<And at nodes.lkt:3145:23>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3145'
--# expr-start 2201 '<bind at nodes.lkt:3147:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3147'
--# expr-start 2196 '<at_or_raise at nodes.lkt:3147:28>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3147'
Get_Result := Get (Self, Local_Matches, Index_6, False); 
--# expr-done 2196
Var_Expr_7 := Get_Result;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind l Var_Expr_7
--# expr-start 2197 '<FieldAccess for children_env at nodes.lkt:3148:28>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3148'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Var_Expr_7.Node, E_Info => Var_Expr_7.Info);
--# expr-done 2197
Scope_Result_3 := Fld_13;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2239;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Env;
end if;
      Inc_Ref (Result_Var_3);
Bound_Env := Result_Var_3; Inc_Ref (Bound_Env);
--# expr-start 2200 '<FieldAccess for sub_equation at nodes.lkt:3152:17>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3152'
--# expr-start 2199 '<as_entity at nodes.lkt:3152:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3152'
--# expr-start 2198 '<Cast to Expr at nodes.lkt:3152:17>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3152'
Cast_Expr_2 := Item_5; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Expr
   then
      Cast_Result_4 := Cast_Expr_2;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 2198
Is_Null_2 := Cast_Result_4 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Expr;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_4); 
   If_Result_3 := As_Entity_1;
end if;
--# expr-done 2199
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result_3.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result_3.Info);
--# end
--# expr-done 2200
Dyn_Var_Bind_Result := Fld_14; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2201
--# expr-start 2209 '<If at nodes.lkt:3155:17>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3155'
Comp_Result := Index_6 > 0; 
if Comp_Result then
   --# expr-start 2208 '<Predicate on BaseTypeDecl.matching_type at nodes.lkt:3156:21>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3156'
--# expr-start 2203 '<FieldAccess for type_var at nodes.lkt:3157:32>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3157'
--# expr-start 2202 '<Cast to Expr at nodes.lkt:3157:32>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3157'
Cast_Expr_3 := Item_5; 
   if Cast_Expr_3 = null
      or else Cast_Expr_3.Kind in Ada_Expr
   then
      Cast_Result_5 := Cast_Expr_3;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 2202
      if Cast_Result_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result_5);
--# expr-done 2203
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 2207 '<FieldAccess for type_var at nodes.lkt:3158:32>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3158'
--# expr-start 2206 '<Cast to Expr at nodes.lkt:3158:32>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3158'
--# expr-start 2205 '<at_or_raise at nodes.lkt:3158:32>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3158'
--# expr-start 2204 '<FieldAccess for names at nodes.lkt:3158:32>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3158'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2204
Get_Result_1 := Get (Self, Fld_16, 0, False); 
--# expr-done 2205
   if Get_Result_1 = null
      or else Get_Result_1.Kind in Ada_Expr
   then
      Cast_Result_6 := Get_Result_1;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 2206
      if Cast_Result_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result_6);
--# expr-done 2207
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Pred := Solver.Create_N_Predicate ((Fld_15, Fld_17), Create_Base_Type_Decl_P_Matching_Type_0_Predicate (Origin, Self)); 
--# expr-done 2208
   If_Result_4 := Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3160:22") else null)); 
   If_Result_4 := True_Rel;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 2209
And_Pred := Create_And (Dyn_Var_Bind_Result, If_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3145:23") else null)); 
--# expr-done 2210
--# expr-start 2218 '<If at nodes.lkt:3169:17>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3169'
if Local_Positional_Assoc then
   --# expr-start 2217 '<And at nodes.lkt:3170:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3170'
--# expr-start 2214 '<Bind at nodes.lkt:3170:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3170'
--# expr-start 2212 '<FieldAccess for expected_type_var at nodes.lkt:3170:25>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3170'
--# expr-start 2211 '<Cast to Expr at nodes.lkt:3170:25>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3170'
Cast_Expr_4 := Item_5; 
   if Cast_Expr_4 = null
      or else Cast_Expr_4.Kind in Ada_Expr
   then
      Cast_Result_7 := Cast_Expr_4;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 2211
      if Cast_Result_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_7);
--# expr-done 2212
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
--# expr-start 2213 '<FieldAccess for index_type at nodes.lkt:3171:25>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3171'
      if Local_Agg_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_19 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Local_Agg_Type.Node, Dim => 0, Origin => Origin, E_Info => Local_Agg_Type.Info);
--# end
--# expr-done 2213
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_19.Node,
         Info => Fld_19.Info);
Bind_Result := Solver.Create_Assign (Fld_18, Cast_Result_8, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3170:21") else null)); 
--# expr-done 2214
--# expr-start 2216 '<FieldAccess for matches_expected_type at nodes.lkt:3172:25>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3172'
--# expr-start 2215 '<Cast to Expr at nodes.lkt:3172:25>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3172'
Cast_Expr_5 := Item_5; 
   if Cast_Expr_5 = null
      or else Cast_Expr_5.Kind in Ada_Expr
   then
      Cast_Result_9 := Cast_Expr_5;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 2215
      if Cast_Result_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_20 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Cast_Result_9, Origin => Origin);
--# end
--# expr-done 2216
And_Pred_1 := Create_And (Bind_Result, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3170:21") else null)); 
--# expr-done 2217
   If_Result_5 := And_Pred_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3173:22") else null)); 
   If_Result_5 := True_Rel_1;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 2218
And_Pred_2 := Create_And (And_Pred, If_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3145:23") else null)); 
--# expr-done 2219
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2238;
               Index_6 := Index_6 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2220
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3144:9") else null)); 
--# expr-done 2221
--# expr-start 2233 '<If at nodes.lkt:3178:13>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3178'
if Local_Positional_Assoc then
   --# expr-start 2225 '<Bind at nodes.lkt:3179:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3179'
--# expr-start 2223 '<FieldAccess for expected_type_var at nodes.lkt:3179:21>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3179'
--# expr-start 2222 '<FieldAccess for expr at nodes.lkt:3179:21>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3179'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2222
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_21.Node);
--# expr-done 2223
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 2224 '<FieldAccess for comp_type at nodes.lkt:3179:54>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3179'
      if Local_Agg_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_23 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Local_Agg_Type.Node, Origin => Origin, E_Info => Local_Agg_Type.Info);
--# end
--# expr-done 2224
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_23.Node,
         Info => Fld_23.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_22, Cast_Result_10, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3179:17") else null)); 
--# expr-done 2225
   If_Result_6 := Bind_Result_1;
else
   --# expr-start 2232 '<Bind at nodes.lkt:3181:17>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3181'
--# expr-start 2227 '<FieldAccess for expected_type_var at nodes.lkt:3181:21>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3181'
--# expr-start 2226 '<FieldAccess for expr at nodes.lkt:3181:21>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3181'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2226
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_24.Node);
--# expr-done 2227
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 2231 '<FieldAccess for type_var at nodes.lkt:3182:21>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3182'
--# expr-start 2230 '<Cast to Expr at nodes.lkt:3182:21>' Cast_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3182'
--# expr-start 2229 '<at_or_raise at nodes.lkt:3182:21>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3182'
--# expr-start 2228 '<FieldAccess for names at nodes.lkt:3182:21>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_26 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2228
Get_Result_2 := Get (Self, Fld_26, 0, False); 
--# expr-done 2229
   if Get_Result_2 = null
      or else Get_Result_2.Kind in Ada_Expr
   then
      Cast_Result_11 := Get_Result_2;
   else
         Cast_Result_11 := No_Bare_Ada_Node;
   end if;
--# expr-done 2230
      if Cast_Result_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result_11);
--# expr-done 2231
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
Bind_Result_2 := Solver.Create_Unify (Fld_25, Fld_27, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3181:17") else null)); 
--# expr-done 2232
   If_Result_6 := Bind_Result_2;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 2233
And_Pred_3 := Create_And (Logic_Boolean_Op, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3144:9") else null)); 
--# expr-done 2234
--# expr-start 2239 '<And at nodes.lkt:3185:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3185'
--# expr-start 2236 '<FieldAccess for sub_equation at nodes.lkt:3185:13>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3185'
--# expr-start 2235 '<FieldAccess for expr at nodes.lkt:3185:13>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3185'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2235
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_28.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_28.Info);
--# end
--# expr-done 2236
--# expr-start 2238 '<FieldAccess for matches_expected_type at nodes.lkt:3185:44>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3185'
--# expr-start 2237 '<FieldAccess for expr at nodes.lkt:3185:44>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3185'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2237
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_31 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_30.Node, Origin => Origin);
--# end
--# expr-done 2238
And_Pred_4 := Create_And (Fld_29, Fld_31, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3185:13") else null)); 
--# expr-done 2239
And_Pred_5 := Create_And (And_Pred_3, And_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3144:9") else null)); 
--# expr-done 2240
Let_Result := And_Pred_5; Inc_Ref (Let_Result);
--# end
--# expr-done 2241
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2242
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2243
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2244
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 2245
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_396;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_396;
                     Finalizer_Scope_2234;
                     Finalizer_Scope_2238;
                     Finalizer_Scope_2239;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Delta_Aggregate_Assoc_Equation;
--# end
--# property-start AggregateAssoc.record_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3193'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Record_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_397 with Inline_Always;
            procedure Finalizer_Scope_2243 with Inline_Always;
            procedure Finalizer_Scope_2244 with Inline_Always;
            procedure Finalizer_Scope_2245 with Inline_Always;
      Local_Agg : Internal_Entity_Base_Aggregate;
Local_Discr_Matches : Internal_Param_Match_Array_Access;
Local_Matches : Internal_Param_Match_Array_Access;
Local_Is_Others_Assoc : Boolean;
Local_Is_Box_Expr : Boolean;
Var_Expr_3 : Internal_Entity_Defining_Name;
Var_Expr_5 : Bare_Base_Id;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Param_Match_Array_Access;
Fld_2 : Internal_Single_Actual;
Fld_3 : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Aggregate_Assoc;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Aggregate_Assoc;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Param_Match_Array_Access;
Fld_7 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Fld_8 : Internal_Entity_Expr;
Is_A_1 : Boolean;
True_Rel : Logic_Equation;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Fld_13 : Internal_Entity_Defining_Name;
Fld_14 : Internal_Entity_Base_Formal_Param_Decl;
Fld_15 : Internal_Entity_Type_Expr;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_17 : Internal_Single_Actual;
Fld_18 : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Internal_Single_Actual;
Fld_22 : Internal_Entity_Basic_Assoc;
Fld_23 : Internal_Entity_Expr;
Fld_24 : Logic_Equation;
And_Pred_1 : Logic_Equation;
If_Result_1 : Logic_Equation;
Fld_25 : Internal_Single_Actual;
Fld_26 : Bare_Base_Id;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Defining_Name;
Fld_29 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
True_Rel_2 : Logic_Equation;
Fld_30 : Internal_Entity_Defining_Name;
Fld_31 : Internal_Entity_Expr;
Fld_32 : Logic_Var;
Fld_33 : Internal_Entity_Base_Formal_Param_Decl;
Fld_34 : Internal_Entity_Type_Expr;
Fld_35 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_36 : Internal_Entity_Expr;
Fld_37 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_38 : Internal_Entity_Expr;
Fld_39 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Result_Var_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
Not_Val_1 : Boolean;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_397 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Discr_Matches);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Matches);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_397;
            procedure Finalizer_Scope_2243 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_24);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2243;
            procedure Finalizer_Scope_2244 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2244;
            procedure Finalizer_Scope_2245 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_39);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2245;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2314 '<Let at nodes.lkt:3194:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3194'
--# scope-start
--# expr-start 2246 '<FieldAccess for base_aggregate at nodes.lkt:3194:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3194'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2246
Local_Agg := Fld; 
--# bind agg Local_Agg
--# expr-start 2313 '<Let at nodes.lkt:3196:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
--# scope-start
--# expr-start 2251 '<Map at nodes.lkt:3196:29>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
   --# expr-start 2247 '<FieldAccess for matched_discriminants at nodes.lkt:3196:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Local_Agg.Node, Origin => Origin, E_Info => Local_Agg.Info);
--# end
--# expr-done 2247
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
         --# expr-start 2250 '<Eq at nodes.lkt:3196:72>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
--# expr-start 2249 '<FieldAccess for assoc at nodes.lkt:3196:72>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
--# expr-start 2248 '<FieldAccess for actual at nodes.lkt:3196:72>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3196'
Fld_2 := Item_0.Actual;
--# expr-done 2248
Fld_3 := Fld_2.Assoc;
--# expr-done 2249
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2250
         if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_0;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2251
Local_Discr_Matches := Map_Result; Inc_Ref (Local_Discr_Matches);
--# bind discr_matches Local_Discr_Matches
--# expr-start 2312 '<Let at nodes.lkt:3206:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
--# scope-start
--# expr-start 2259 '<If at nodes.lkt:3206:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
--# expr-start 2253 '<Not at nodes.lkt:3206:26>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
--# expr-start 2252 '<IsNull at nodes.lkt:3206:30>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
Is_Equal := Equivalent (Local_Discr_Matches, No_Internal_Param_Match_Array_Type); 
--# expr-done 2252
Not_Val := not (Is_Equal); 
--# expr-done 2253
if Not_Val then
   If_Result := Local_Discr_Matches;
else
   --# expr-start 2258 '<Map at nodes.lkt:3206:76>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
   --# expr-start 2254 '<FieldAccess for matched_components at nodes.lkt:3206:76>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Local_Agg.Node, Origin => Origin, Env => Env, E_Info => Local_Agg.Info);
--# end
--# expr-done 2254
      declare
         Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_1
         --# expr-start 2257 '<Eq at nodes.lkt:3206:116>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
--# expr-start 2256 '<FieldAccess for assoc at nodes.lkt:3206:116>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
--# expr-start 2255 '<FieldAccess for actual at nodes.lkt:3206:116>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3206'
Fld_5 := Item_1.Actual;
--# expr-done 2255
Fld_6 := Fld_5.Assoc;
--# expr-done 2256
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 2257
         if Is_Equiv_1 then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2258
   If_Result := Map_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 2259
Local_Matches := If_Result; Inc_Ref (Local_Matches);
--# bind matches Local_Matches
--# expr-start 2311 '<Let at nodes.lkt:3208:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3208'
--# scope-start
--# expr-start 2262 '<AnyQuantifier at nodes.lkt:3208:31>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3208'
   --# expr-start 2260 '<FieldAccess for names at nodes.lkt:3208:31>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3208'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2260
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_7;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 2261 '<IsA OthersDesignator at nodes.lkt:3208:55>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3208'
Is_A := Item_2 /= null 
and then Item_2.Kind in Ada_Others_Designator_Range; 
--# expr-done 2261
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2262
Local_Is_Others_Assoc := Quantifier_Result; 
--# bind is_others_assoc Local_Is_Others_Assoc
--# expr-start 2310 '<Let at nodes.lkt:3212:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3212'
--# scope-start
--# expr-start 2264 '<IsA BoxExpr at nodes.lkt:3212:27>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3212'
--# expr-start 2263 '<FieldAccess for expr at nodes.lkt:3212:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3212'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2263
Is_A_1 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 2264
Local_Is_Box_Expr := Is_A_1; 
--# bind is_box_expr Local_Is_Box_Expr
--# expr-start 2309 '<If at nodes.lkt:3214:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3214'
--# expr-start 2265 '<Not at nodes.lkt:3214:12>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3214'
Not_Val_1 := not (Local_Is_Others_Assoc); 
--# expr-done 2265
if Not_Val_1 then
   --# expr-start 2294 '<LogicAll at nodes.lkt:3214:37>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3214'
--# expr-start 2293 '<Map at nodes.lkt:3214:37>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3214'
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Local_Matches;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind match_var Item_4
      --# expr-start 2292 '<And at nodes.lkt:3215:28>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3215'
--# expr-start 2285 '<If at nodes.lkt:3218:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3218'
if Local_Is_Box_Expr then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3218:37") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 2284 '<And at nodes.lkt:3218:48>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3218'
--# expr-start 2279 '<And at nodes.lkt:3219:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2274 '<Bind at nodes.lkt:3219:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2269 '<FieldAccess for expected_type_var at nodes.lkt:3219:25>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2268 '<FieldAccess for expr at nodes.lkt:3219:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2267 '<FieldAccess for assoc at nodes.lkt:3219:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2266 '<FieldAccess for actual at nodes.lkt:3219:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
Fld_9 := Item_4.Actual;
--# expr-done 2266
Fld_10 := Fld_9.Assoc;
--# expr-done 2267
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 2268
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_11.Node);
--# expr-done 2269
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 2273 '<FieldAccess for designated_type at nodes.lkt:3219:76>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2272 '<FieldAccess for type_expression at nodes.lkt:3219:76>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2271 '<FieldAccess for formal_decl at nodes.lkt:3219:76>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2270 '<FieldAccess for formal at nodes.lkt:3219:76>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
Fld_13 := Item_4.Formal;
--# expr-done 2270
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_14 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 2271
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 2272
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_15.Node, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 2273
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
Bind_Result := Solver.Create_Assign (Fld_12, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3219:21") else null)); 
--# expr-done 2274
--# expr-start 2278 '<FieldAccess for sub_equation at nodes.lkt:3219:148>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2277 '<FieldAccess for expr at nodes.lkt:3219:148>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2276 '<FieldAccess for assoc at nodes.lkt:3219:148>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
--# expr-start 2275 '<FieldAccess for actual at nodes.lkt:3219:148>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3219'
Fld_17 := Item_4.Actual;
--# expr-done 2275
Fld_18 := Fld_17.Assoc;
--# expr-done 2276
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_18.Node, E_Info => Fld_18.Info);
--# end
--# expr-done 2277
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_19.Info);
--# end
--# expr-done 2278
And_Pred := Create_And (Bind_Result, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3219:21") else null)); 
--# expr-done 2279
--# expr-start 2283 '<FieldAccess for matches_expected_assign_type at nodes.lkt:3220:23>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3220'
--# expr-start 2282 '<FieldAccess for expr at nodes.lkt:3220:23>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3220'
--# expr-start 2281 '<FieldAccess for assoc at nodes.lkt:3220:23>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3220'
--# expr-start 2280 '<FieldAccess for actual at nodes.lkt:3220:23>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3220'
Fld_21 := Item_4.Actual;
--# expr-done 2280
Fld_22 := Fld_21.Assoc;
--# expr-done 2281
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_23 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_22.Node, E_Info => Fld_22.Info);
--# end
--# expr-done 2282
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_24 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_23.Node, Origin => Origin);
--# end
--# expr-done 2283
And_Pred_1 := Create_And (And_Pred, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3218:48") else null)); 
--# expr-done 2284
   If_Result_1 := And_Pred_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2285
--# expr-start 2287 '<FieldAccess for name at nodes.lkt:3221:19>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3221'
--# expr-start 2286 '<FieldAccess for actual at nodes.lkt:3221:19>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3221'
Fld_25 := Item_4.Actual;
--# expr-done 2286
Fld_26 := Fld_25.Name;
--# expr-done 2287
Var_Expr_5 := Fld_26;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_5
--# expr-start 2291 '<Bind at nodes.lkt:3222:24>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3222'
--# expr-start 2288 '<FieldAccess for ref_var at nodes.lkt:3222:28>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3222'
      if Var_Expr_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_5);
--# end
--# expr-done 2288
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 2290 '<FieldAccess for formal_decl at nodes.lkt:3222:41>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3222'
--# expr-start 2289 '<FieldAccess for formal at nodes.lkt:3222:41>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3222'
Fld_28 := Item_4.Formal;
--# expr-done 2289
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_29 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_28.Node, E_Info => Fld_28.Info);
--# end
--# expr-done 2290
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_27, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3222:24") else null)); 
--# expr-done 2291
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2244;
   Result_Var := Scope_Result;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3222:86") else null)); 
   Result_Var := True_Rel_1;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (If_Result_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3215:28") else null)); 
--# expr-done 2292
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2243;
         end loop;
      end;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 2293
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3214:37") else null)); 
--# expr-done 2294
   If_Result_3 := Logic_Boolean_Op;
else
   --# expr-start 2308 '<If at nodes.lkt:3229:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3229'
if Local_Is_Box_Expr then
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3229:33") else null)); 
   If_Result_2 := True_Rel_2;
else
--# expr-start 2295 '<FieldAccess for first_unmatched_formal at nodes.lkt:3229:44>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3229'
      if Local_Agg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.first_unmatched_formal
Fld_30 := Libadalang.Implementation.Base_Aggregate_P_First_Unmatched_Formal (Node => Local_Agg.Node, Origin => Origin, Env => Env, E_Info => Local_Agg.Info);
--# end
--# expr-done 2295
Var_Expr_3 := Fld_30;
if Var_Expr_3 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind unmatched_formal Var_Expr_3
--# expr-start 2307 '<And at nodes.lkt:3230:39>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3230'
--# expr-start 2304 '<And at nodes.lkt:3231:21>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2301 '<Bind at nodes.lkt:3231:21>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2297 '<FieldAccess for expected_type_var at nodes.lkt:3231:25>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2296 '<FieldAccess for expr at nodes.lkt:3231:25>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_31 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2296
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_31.Node);
--# expr-done 2297
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
--# expr-start 2300 '<FieldAccess for designated_type at nodes.lkt:3231:58>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2299 '<FieldAccess for type_expression at nodes.lkt:3231:58>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2298 '<FieldAccess for formal_decl at nodes.lkt:3231:58>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_33 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 2298
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_33.Node, E_Info => Fld_33.Info);
--# end
--# expr-done 2299
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_35 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_34.Node, Origin => Origin, E_Info => Fld_34.Info);
--# end
--# expr-done 2300
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_35.Node,
         Info => Fld_35.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_32, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3231:21") else null)); 
--# expr-done 2301
--# expr-start 2303 '<FieldAccess for sub_equation at nodes.lkt:3231:130>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
--# expr-start 2302 '<FieldAccess for expr at nodes.lkt:3231:130>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3231'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2302
      if Fld_36.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_37 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_36.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_36.Info);
--# end
--# expr-done 2303
And_Pred_3 := Create_And (Bind_Result_2, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3231:21") else null)); 
--# expr-done 2304
--# expr-start 2306 '<FieldAccess for matches_expected_type at nodes.lkt:3232:23>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3232'
--# expr-start 2305 '<FieldAccess for expr at nodes.lkt:3232:23>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3232'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2305
      if Fld_38.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_39 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_38.Node, Origin => Origin);
--# end
--# expr-done 2306
And_Pred_4 := Create_And (And_Pred_3, Fld_39, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3230:39") else null)); 
--# expr-done 2307
Scope_Result_1 := And_Pred_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2245;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3232:72") else null)); 
   Result_Var_1 := True_Rel_3;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_2 := Result_Var_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2308
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2309
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 2310
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2311
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2312
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2313
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 2314
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_397;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_397;
                     Finalizer_Scope_2243;
                     Finalizer_Scope_2244;
                     Finalizer_Scope_2245;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Record_Assoc_Equation;
--# end
--# property-start AggregateAssoc.array_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3240'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Array_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Atd : Internal_Entity_Array_Type_Def
      ; Mra : Internal_Multidim_Aggregate_Info
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind atd Atd
   --# bind mra Mra
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_398 with Inline_Always;
            procedure Finalizer_Scope_2246 with Inline_Always;
            procedure Finalizer_Scope_2247 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Is_Equal : Boolean;
Fld_7 : Integer;
Fld_8 : Integer;
Arith_Result : Integer;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_9 : Internal_Entity_Alternatives_List;
Fld_10 : Bare_Alternatives_List;
Fld_11 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_12 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Name;
Fld_13 : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Fld_14 : Logic_Var;
Fld_15 : Integer;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var : Logic_Equation;
Is_A : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
And_Pred_3 : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_398 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_398;
            procedure Finalizer_Scope_2246 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2246;
            procedure Finalizer_Scope_2247 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2247;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2351 '<And at nodes.lkt:3240:97>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3240'
--# expr-start 2330 '<If at nodes.lkt:3243:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
--# expr-start 2315 '<IsNull at nodes.lkt:3243:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
Is_Equal := Mra = No_Multidim_Aggregate_Info; 
--# expr-done 2315
if Is_Equal then
   If_Result := True;
else
   --# expr-start 2319 '<Eq at nodes.lkt:3243:27>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
--# expr-start 2316 '<FieldAccess for rank at nodes.lkt:3243:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
Fld_7 := Mra.Rank;
--# expr-done 2316
--# expr-start 2318 '<Arithmetic - at nodes.lkt:3243:39>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
--# expr-start 2317 '<FieldAccess for array_ndims at nodes.lkt:3243:39>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3243'
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_8 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 2317
Arith_Result := (Fld_8 - 1); 
--# expr-done 2318
Is_Equal_1 := Fld_7 = Arith_Result; 
--# expr-done 2319
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 2329 '<And at nodes.lkt:3245:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3245'
--# expr-start 2326 '<And at nodes.lkt:3246:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
--# expr-start 2323 '<Bind at nodes.lkt:3246:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
--# expr-start 2321 '<FieldAccess for expected_type_var at nodes.lkt:3246:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
--# expr-start 2320 '<FieldAccess for expr at nodes.lkt:3246:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2320
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 2321
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 2322 '<FieldAccess for comp_type at nodes.lkt:3246:54>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 2322
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3246:17") else null)); 
--# expr-done 2323
--# expr-start 2325 '<FieldAccess for sub_equation at nodes.lkt:3246:75>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
--# expr-start 2324 '<FieldAccess for expr at nodes.lkt:3246:75>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3246'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2324
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 2325
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3246:17") else null)); 
--# expr-done 2326
--# expr-start 2328 '<FieldAccess for matches_expected_type at nodes.lkt:3247:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3247'
--# expr-start 2327 '<FieldAccess for expr at nodes.lkt:3247:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3247'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2327
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 2328
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3245:13") else null)); 
--# expr-done 2329
   If_Result_1 := And_Pred_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3251:13") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2330
--# expr-start 2350 '<LogicAll at nodes.lkt:3253:11>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3253'
--# expr-start 2349 '<Map at nodes.lkt:3253:11>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3253'
   --# expr-start 2331 '<FieldAccess for designators at nodes.lkt:3253:11>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3253'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2331
Fld_10 := Fld_9.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_10;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity;
else
Fld_11 := Fld_9.Info;
As_Entity := (Info => Fld_11, Node => Bare_Item_0); 
   If_Result_4 := As_Entity;
end if;
                  Item_0 := If_Result_4; 
   --# scope-start
               --# bind n Item_0
      --# expr-start 2348 '<And at nodes.lkt:3254:16>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3254'
--# expr-start 2332 '<FieldAccess for sub_equation at nodes.lkt:3254:16>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3254'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 2332
--# expr-start 2347 '<If at nodes.lkt:3255:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
--# expr-start 2333 '<IsA Name at nodes.lkt:3255:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Name; 
--# expr-done 2333
if Is_A then
   --# expr-start 2337 '<Not at nodes.lkt:3255:30>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
--# expr-start 2336 '<IsNull at nodes.lkt:3255:34>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
--# expr-start 2335 '<FieldAccess for name_designated_type at nodes.lkt:3255:34>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
--# expr-start 2334 '<Cast to Name at nodes.lkt:3255:34>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 2334
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_18 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 2335
Is_Null := Fld_18.Node = null; 
--# expr-done 2336
Not_Val := not (Is_Null); 
--# expr-done 2337
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2339 '<FieldAccess for xref_type_equation at nodes.lkt:3255:81>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
--# expr-start 2338 '<Cast to Name at nodes.lkt:3255:81>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 2338
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_13 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 2339
   If_Result_3 := Fld_13;
else
--# expr-start 2340 '<Cast to Expr at nodes.lkt:3255:118>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3255'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Expr
   then
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Expr;
   end if;
--# expr-done 2340
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 2346 '<And at nodes.lkt:3256:24>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
--# expr-start 2344 '<Bind at nodes.lkt:3256:24>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
--# expr-start 2341 '<FieldAccess for expected_type_var at nodes.lkt:3256:28>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 2341
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 2343 '<FieldAccess for index_type at nodes.lkt:3256:51>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
      if Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2342 '<FieldAccess for rank at nodes.lkt:3256:66>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
Fld_15 := Mra.Rank;
--# expr-done 2342
--# property-call-start ArrayTypeDef.index_type
Fld_16 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Atd.Node, Dim => Fld_15, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 2343
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_14, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3256:24") else null)); 
--# expr-done 2344
--# expr-start 2345 '<FieldAccess for matches_expected_type at nodes.lkt:3256:81>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3256'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_17 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_1.Node, Origin => Origin);
--# end
--# expr-done 2345
And_Pred_2 := Create_And (Bind_Result_1, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3256:24") else null)); 
--# expr-done 2346
Scope_Result := And_Pred_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2247;
   Result_Var := Scope_Result;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3256:120") else null)); 
   Result_Var := True_Rel_1;
end if;
      Inc_Ref (Result_Var);
   If_Result_3 := Result_Var;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2347
And_Pred_3 := Create_And (Fld_12, If_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3254:16") else null)); 
--# expr-done 2348
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2246;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2349
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3253:11") else null)); 
--# expr-done 2350
And_Pred_4 := Create_And (If_Result_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3240:97") else null)); 
--# expr-done 2351
         Property_Result := And_Pred_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_398;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_398;
                     Finalizer_Scope_2246;
                     Finalizer_Scope_2247;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Array_Assoc_Equation;
--# end
--# property-start AggregateAssoc.globals_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3264'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Globals_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_399 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_399 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_399;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2353 '<FieldAccess for sub_equation at nodes.lkt:3268:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3268'
--# expr-start 2352 '<FieldAccess for expr at nodes.lkt:3268:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3268'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2352
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 2353
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_399;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_399;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Globals_Assoc_Equation;
--# end
--# property-start AggregateAssoc.depends_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3273'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Depends_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_400 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Local_N : Bare_Ada_Node;
Var_Expr_73 : Internal_Entity_Un_Op;
True_Rel : Logic_Equation;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Is_A : Boolean;
Fld_3 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Un_Op;
Fld_4 : Internal_Entity_Expr;
Is_A_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_5 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
True_Rel_1 : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Fld_6 : Logic_Equation;
Is_Null_1 : Boolean;
Is_A_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Logic_Equation;
Let_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_400 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_400;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2371 '<And at nodes.lkt:3273:46>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3273'
--# expr-start 2362 '<If at nodes.lkt:3280:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
--# expr-start 2355 '<IsA NullLiteral at nodes.lkt:3280:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
--# expr-start 2354 '<FieldAccess for expr at nodes.lkt:3280:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2354
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Null_Literal_Range; 
--# expr-done 2355
if Is_A then
   If_Result := True;
else
--# expr-start 2357 '<Cast to UnOp at nodes.lkt:3280:42>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
--# expr-start 2356 '<FieldAccess for expr at nodes.lkt:3280:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2356
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Un_Op_Range
   then
      Cast_Result := Create_Internal_Entity_Un_Op
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Un_Op;
   end if;
--# expr-done 2357
Var_Expr_73 := Cast_Result;
if Var_Expr_73 /= No_Entity_Un_Op then
   --# scope-start
--# expr-start 2358 '<FieldAccess for expr at nodes.lkt:3280:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
      if Var_Expr_73.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Var_Expr_73.Node.Un_Op_F_Expr, Info => Var_Expr_73.Info);
--# expr-done 2358
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 2359 '<IsA NullLiteral at nodes.lkt:3280:80>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
Is_A_1 := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Null_Literal_Range; 
--# expr-done 2359
Scope_Result := Is_A_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
if If_Result then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3280:104") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 2361 '<FieldAccess for sub_equation at nodes.lkt:3280:115>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
--# expr-start 2360 '<FieldAccess for expr at nodes.lkt:3280:115>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3280'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2360
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 2361
   If_Result_1 := Fld_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2362
--# expr-start 2370 '<Let at nodes.lkt:3287:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3287'
--# scope-start
--# expr-start 2364 '<at at nodes.lkt:3287:17>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3287'
--# expr-start 2363 '<FieldAccess for names at nodes.lkt:3287:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3287'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2363
Get_Result := Get (Self, Fld_5, 0, True); 
--# expr-done 2364
Local_N := Get_Result; 
--# bind n Local_N
--# expr-start 2369 '<If at nodes.lkt:3289:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3289'
--# expr-start 2365 '<IsNull at nodes.lkt:3289:12>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3289'
Is_Null_1 := Local_N = null; 
--# expr-done 2365
if Is_Null_1 then
   If_Result_3 := True;
else
   --# expr-start 2366 '<IsA NullLiteral at nodes.lkt:3289:25>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3289'
Is_A_2 := Local_N /= null 
and then Local_N.Kind in Ada_Null_Literal_Range; 
--# expr-done 2366
   If_Result_3 := Is_A_2;
end if;
if If_Result_3 then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3289:47") else null)); 
   If_Result_4 := True_Rel_1;
else
   --# expr-start 2368 '<FieldAccess for sub_equation at nodes.lkt:3289:58>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3289'
--# expr-start 2367 '<as_entity at nodes.lkt:3289:58>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3289'
Is_Null := Local_N = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Local_N); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 2367
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result_2.Info);
--# end
--# expr-done 2368
   If_Result_4 := Fld_6;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 2369
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2370
And_Pred := Create_And (If_Result_1, Let_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3273:46") else null)); 
--# expr-done 2371
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_400;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_400;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Depends_Assoc_Equation;
--# end
--# property-start AggregateAssoc.test_case_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3295'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Test_Case_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_401 with Inline_Always;
      Var_Expr_0 : Bare_Base_Id;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Id;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_401 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_401;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2380 '<If at nodes.lkt:3298:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3298'
--# expr-start 2374 '<Cast to BaseId at nodes.lkt:3298:12>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3298'
--# expr-start 2373 '<at at nodes.lkt:3298:12>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3298'
--# expr-start 2372 '<FieldAccess for names at nodes.lkt:3298:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3298'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 2372
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2373
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Id
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2374
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 2375 '<FieldAccess for name_symbol at nodes.lkt:3299:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3299'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_0);
--# end
--# expr-done 2375
--# expr-start 2376 '<Symbol Requires at nodes.lkt:3299:45>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3299'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requires); 
--# expr-done 2376
Is_Equal_1 := Fld_3 = Sym; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 2377 '<Symbol Ensures at nodes.lkt:3299:59>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3299'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ensures); 
--# expr-done 2377
Is_Equal := Fld_3 = Sym_1; 
   If_Result := Is_Equal;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 2379 '<FieldAccess for sub_equation at nodes.lkt:3300:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3300'
--# expr-start 2378 '<FieldAccess for expr at nodes.lkt:3300:16>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3300'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2378
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 2379
   If_Result_1 := Fld_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3300:48") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2380
         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_401;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_401;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Test_Case_Assoc_Equation;
--# end
--# property-start AggregateAssoc.contract_cases_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Contract_Cases_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_402 with Inline_Always;
            procedure Finalizer_Scope_2251 with Inline_Always;
            procedure Finalizer_Scope_2252 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_402 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_402;
            procedure Finalizer_Scope_2251 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2251;
            procedure Finalizer_Scope_2252 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2252;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2402 '<And at nodes.lkt:3306:53>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
--# expr-start 2399 '<And at nodes.lkt:3306:53>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
--# expr-start 2396 '<And at nodes.lkt:3306:53>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
--# expr-start 2391 '<LogicAll at nodes.lkt:3306:53>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
--# expr-start 2390 '<Map at nodes.lkt:3306:53>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
   --# expr-start 2381 '<FieldAccess for designators at nodes.lkt:3306:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3306'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2381
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind d Item_0
--# expr-start 2382 '<Cast to Expr at nodes.lkt:3307:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3307'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 2382
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 2389 '<And at nodes.lkt:3308:20>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3308'
--# expr-start 2387 '<And at nodes.lkt:3309:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3309'
--# expr-start 2385 '<Bind at nodes.lkt:3309:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3309'
--# expr-start 2383 '<FieldAccess for expected_type_var at nodes.lkt:3309:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3309'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 2383
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 2384 '<FieldAccess for bool_type at nodes.lkt:3309:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3309'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 2384
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3309:17") else null)); 
--# expr-done 2385
--# expr-start 2386 '<FieldAccess for sub_equation at nodes.lkt:3309:66>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3309'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2386
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3309:17") else null)); 
--# expr-done 2387
--# expr-start 2388 '<FieldAccess for matches_expected_formal_type at nodes.lkt:3310:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3310'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Var_Expr_1.Node, Origin => Origin);
--# end
--# expr-done 2388
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3308:20") else null)); 
--# expr-done 2389
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2252;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3310:65") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2251;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2390
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3306:53") else null)); 
--# expr-done 2391
--# expr-start 2395 '<Bind at nodes.lkt:3314:9>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3314'
--# expr-start 2393 '<FieldAccess for expected_type_var at nodes.lkt:3314:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3314'
--# expr-start 2392 '<FieldAccess for expr at nodes.lkt:3314:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2392
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7.Node);
--# expr-done 2393
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 2394 '<FieldAccess for bool_type at nodes.lkt:3314:46>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3314'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_9 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 2394
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3314:9") else null)); 
--# expr-done 2395
And_Pred_2 := Create_And (Logic_Boolean_Op, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3306:53") else null)); 
--# expr-done 2396
--# expr-start 2398 '<FieldAccess for sub_equation at nodes.lkt:3315:11>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3315'
--# expr-start 2397 '<FieldAccess for expr at nodes.lkt:3315:11>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3315'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2397
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 2398
And_Pred_3 := Create_And (And_Pred_2, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3306:53") else null)); 
--# expr-done 2399
--# expr-start 2401 '<FieldAccess for matches_expected_formal_type at nodes.lkt:3315:42>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3315'
--# expr-start 2400 '<FieldAccess for expr at nodes.lkt:3315:42>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3315'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2400
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 2401
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3306:53") else null)); 
--# expr-done 2402
         Property_Result := And_Pred_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_402;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_402;
                     Finalizer_Scope_2251;
                     Finalizer_Scope_2252;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Contract_Cases_Assoc_Equation;
--# end
--# property-start AggregateAssoc.subprogram_variant_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3320'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Subprogram_Variant_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_403 with Inline_Always;
            procedure Finalizer_Scope_2253 with Inline_Always;
            procedure Finalizer_Scope_2254 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Base_Id;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_14 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Fld_15 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel_1 : Logic_Equation;
Result_Var : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_3 : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_403 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_403;
            procedure Finalizer_Scope_2253 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2253;
            procedure Finalizer_Scope_2254 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2254;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2429 '<LogicAll at nodes.lkt:3321:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3321'
--# expr-start 2428 '<Map at nodes.lkt:3321:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3321'
   --# expr-start 2403 '<FieldAccess for designators at nodes.lkt:3321:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3321'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2403
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result_3 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result_3 := As_Entity;
end if;
                  Item_0 := If_Result_3; 
   --# scope-start
               --# bind d Item_0
--# expr-start 2404 '<Cast to BaseId at nodes.lkt:3321:43>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3321'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Id
   then
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Id;
   end if;
--# expr-done 2404
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Id then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 2427 '<If at nodes.lkt:3322:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
--# expr-start 2405 '<FieldAccess for name_symbol at nodes.lkt:3322:16>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_1.Node);
--# end
--# expr-done 2405
--# expr-start 2406 '<Symbol Increases at nodes.lkt:3322:38>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Increases); 
--# expr-done 2406
Is_Equal_2 := Fld_15 = Sym_1; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 2407 '<Symbol Decreases at nodes.lkt:3322:53>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Decreases); 
--# expr-done 2407
Is_Equal_1 := Fld_15 = Sym_2; 
   If_Result_1 := Is_Equal_1;
end if;
if If_Result_1 then
   --# expr-start 2421 '<And at nodes.lkt:3322:71>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
--# expr-start 2418 '<Or at nodes.lkt:3325:17>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3325'
--# expr-start 2410 '<Predicate on BaseTypeDecl.is_discrete_type at nodes.lkt:3325:17>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3325'
--# expr-start 2409 '<FieldAccess for type_var at nodes.lkt:3327:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3327'
--# expr-start 2408 '<FieldAccess for expr at nodes.lkt:3327:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3327'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2408
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_3.Node);
--# expr-done 2409
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 2410
--# expr-start 2417 '<And at nodes.lkt:3329:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3329'
--# expr-start 2414 '<Bind at nodes.lkt:3329:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3329'
--# expr-start 2412 '<FieldAccess for expected_type_var at nodes.lkt:3329:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3329'
--# expr-start 2411 '<FieldAccess for expr at nodes.lkt:3329:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3329'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2411
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5.Node);
--# expr-done 2412
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 2413 '<FieldAccess for big_integer_type at nodes.lkt:3330:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3330'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.big_integer_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_Big_Integer_Type (Node => Ent.Node);
--# end
--# expr-done 2413
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3329:21") else null)); 
--# expr-done 2414
--# expr-start 2416 '<FieldAccess for matches_expected_type at nodes.lkt:3331:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3331'
--# expr-start 2415 '<FieldAccess for expr at nodes.lkt:3331:25>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3331'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2415
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 2416
And_Pred := Create_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3329:21") else null)); 
--# expr-done 2417
Or_Pred := Create_Or (Pred, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3325:17") else null)); 
--# expr-done 2418
--# expr-start 2420 '<FieldAccess for sub_equation at nodes.lkt:3333:19>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3333'
--# expr-start 2419 '<FieldAccess for expr at nodes.lkt:3333:19>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3333'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2419
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 2420
And_Pred_1 := Create_And (Or_Pred, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3322:71") else null)); 
--# expr-done 2421
   If_Result_2 := And_Pred_1;
else
   --# expr-start 2426 '<If at nodes.lkt:3322:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3322'
--# expr-start 2422 '<FieldAccess for name_symbol at nodes.lkt:3334:18>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3334'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_1.Node);
--# end
--# expr-done 2422
--# expr-start 2423 '<Symbol Structural at nodes.lkt:3334:40>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3334'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Structural); 
--# expr-done 2423
Is_Equal := Fld_14 = Sym; 
if Is_Equal then
   --# expr-start 2425 '<FieldAccess for sub_equation at nodes.lkt:3338:17>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3338'
--# expr-start 2424 '<FieldAccess for expr at nodes.lkt:3338:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3338'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2424
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_12.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_12.Info);
--# end
--# expr-done 2425
   If_Result := Fld_13;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3340:17") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2426
   If_Result_2 := If_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 2427
Scope_Result := If_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2254;
   Result_Var := Scope_Result;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3341:25") else null)); 
   Result_Var := False_Rel_1;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2253;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2428
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3321:9") else null)); 
--# expr-done 2429
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_403;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_403;
                     Finalizer_Scope_2253;
                     Finalizer_Scope_2254;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Subprogram_Variant_Assoc_Equation;
--# end
--# property-start AggregateAssoc.exprs_assoc_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3348'
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Exprs_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc  := Bare_Aggregate_Assoc (Node);
     Ent : Internal_Entity_Aggregate_Assoc :=
       Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_404 with Inline_Always;
            procedure Finalizer_Scope_2255 with Inline_Always;
            procedure Finalizer_Scope_2256 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_404 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_404;
            procedure Finalizer_Scope_2255 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2255;
            procedure Finalizer_Scope_2256 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2256;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2437 '<And at nodes.lkt:3349:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
--# expr-start 2433 '<LogicAll at nodes.lkt:3349:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
--# expr-start 2432 '<Map at nodes.lkt:3349:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
   --# expr-start 2430 '<FieldAccess for designators at nodes.lkt:3349:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 2430
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind d Item_0
      --# expr-start 2431 '<FieldAccess for sub_equation at nodes.lkt:3349:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 2431
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2255;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2432
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3349:9") else null)); 
--# expr-done 2433
--# expr-start 2435 '<Cast to Name at nodes.lkt:3349:65>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
--# expr-start 2434 '<FieldAccess for expr at nodes.lkt:3349:65>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3349'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2434
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 2435
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 2436 '<FieldAccess for sub_equation at nodes.lkt:3350:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3350'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2436
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2256;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3350:50") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Logic_Boolean_Op, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3349:9") else null)); 
--# expr-done 2437
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_404;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_404;
                     Finalizer_Scope_2255;
                     Finalizer_Scope_2256;
            raise;
      end;
   return Property_Result;
end Aggregate_Assoc_P_Exprs_Assoc_Equation;
--# end
   --
   --  Primitives for Bare_Multi_Dim_Array_Assoc
   --
      procedure Initialize_Fields_For_Multi_Dim_Array_Assoc
        (Self : Bare_Multi_Dim_Array_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Aggregate_Assoc
              (Self, Aggregate_Assoc_F_Designators, Aggregate_Assoc_F_R_Expr);
      end Initialize_Fields_For_Multi_Dim_Array_Assoc;
   --
   --  Primitives for Bare_Composite_Constraint_Assoc
   --
      procedure Initialize_Fields_For_Composite_Constraint_Assoc
        (Self : Bare_Composite_Constraint_Assoc
         ; Composite_Constraint_Assoc_F_Ids : Bare_Discriminant_Choice_List
         ; Composite_Constraint_Assoc_F_Constraint_Expr : Bare_Ada_Node
        ) is
      begin
            Self.Composite_Constraint_Assoc_F_Ids := Composite_Constraint_Assoc_F_Ids;
            Self.Composite_Constraint_Assoc_F_Constraint_Expr := Composite_Constraint_Assoc_F_Constraint_Expr;
      end Initialize_Fields_For_Composite_Constraint_Assoc;
   function Composite_Constraint_Assoc_F_Ids
     (Node : Bare_Composite_Constraint_Assoc) return Bare_Discriminant_Choice_List
   is
   begin
         return Node.Composite_Constraint_Assoc_F_Ids;
   end;
   function Composite_Constraint_Assoc_F_Constraint_Expr
     (Node : Bare_Composite_Constraint_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Composite_Constraint_Assoc_F_Constraint_Expr;
   end;
--# property-start CompositeConstraintAssoc.expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3363'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_Assoc_P_Expr
  (Node : Bare_Composite_Constraint_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Composite_Constraint_Assoc  := Bare_Composite_Constraint_Assoc (Node);
     Ent : Internal_Entity_Composite_Constraint_Assoc :=
       Internal_Entity_Composite_Constraint_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2439 '<Cast to Expr at nodes.lkt:3363:32>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3363'
--# expr-start 2438 '<FieldAccess for constraint_expr at nodes.lkt:3363:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3363'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Ent.Info);
--# expr-done 2438
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2439
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_Assoc_P_Expr;
--# end
--# property-start CompositeConstraintAssoc.names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3365'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_Assoc_P_Names
  (Node : Bare_Composite_Constraint_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Composite_Constraint_Assoc  := Bare_Composite_Constraint_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_406 with Inline_Always;
      Fld : Bare_Discriminant_Choice_List;
Item_0 : Bare_Identifier;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_406 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_406;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2442 '<Map at nodes.lkt:3365:35>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3365'
   --# expr-start 2440 '<FieldAccess for ids at nodes.lkt:3365:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3365'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Composite_Constraint_Assoc_F_Ids;
--# expr-done 2440
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Choice_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 2441 '<Cast to AdaNode at nodes.lkt:3365:55>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3365'
Cast_Expr := Item_0; 
      Cast_Result := Cast_Expr;
--# expr-done 2441
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2442
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_406;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_406;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_Assoc_P_Names;
--# end
   --
   --  Primitives for Bare_Iterated_Assoc
   --
      procedure Initialize_Fields_For_Iterated_Assoc
        (Self : Bare_Iterated_Assoc
         ; Iterated_Assoc_F_Spec : Bare_For_Loop_Spec
         ; Iterated_Assoc_F_Key_Expr : Bare_Expr
         ; Iterated_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Iterated_Assoc_F_Spec := Iterated_Assoc_F_Spec;
            Self.Iterated_Assoc_F_Key_Expr := Iterated_Assoc_F_Key_Expr;
            Self.Iterated_Assoc_F_R_Expr := Iterated_Assoc_F_R_Expr;
      end Initialize_Fields_For_Iterated_Assoc;
   function Iterated_Assoc_F_Spec
     (Node : Bare_Iterated_Assoc) return Bare_For_Loop_Spec
   is
   begin
         return Node.Iterated_Assoc_F_Spec;
   end;
   function Iterated_Assoc_F_Key_Expr
     (Node : Bare_Iterated_Assoc) return Bare_Expr
   is
   begin
         return Node.Iterated_Assoc_F_Key_Expr;
   end;
   function Iterated_Assoc_F_R_Expr
     (Node : Bare_Iterated_Assoc) return Bare_Expr
   is
   begin
         return Node.Iterated_Assoc_F_R_Expr;
   end;
--# property-start IteratedAssoc.expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3374'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Expr
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2443 '<FieldAccess for r_expr at nodes.lkt:3374:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3374'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Iterated_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 2443
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Expr;
--# end
--# property-start IteratedAssoc.names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3376'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Names
  (Node : Bare_Iterated_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Bare_Ada_Node_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Names;
--# end
--# property-start IteratedAssoc.base_aggregate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3378'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Base_Aggregate
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2446 '<Cast to BaseAggregate at nodes.lkt:3379:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3379'
--# expr-start 2445 '<parent at nodes.lkt:3379:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3379'
--# expr-start 2444 '<parent at nodes.lkt:3379:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3379'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2444
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2445
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2446
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Base_Aggregate;
--# end
--# property-start IteratedAssoc.xref_stop_resolution 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3382'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2449 '<IsA BaseAggregate at nodes.lkt:3382:40>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3382'
--# expr-start 2448 '<parent at nodes.lkt:3382:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3382'
--# expr-start 2447 '<parent at nodes.lkt:3382:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3382'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2447
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2448
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Base_Aggregate; 
--# expr-done 2449
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start IteratedAssoc.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3385'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_411 with Inline_Always;
            procedure Finalizer_Scope_2261 with Inline_Always;
      Local_Aggregate : Internal_Entity_Base_Aggregate;
Local_Root_Agg : Internal_Multidim_Aggregate_Info;
Local_Type_Decl : Internal_Entity_Base_Type_Decl;
Local_Array_Type_Def : Internal_Entity_Array_Type_Def;
Local_Container_Aggregate : Internal_Entity_Aggregate;
Local_Comp_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Array_Type_Def;
Local_Index_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Array_Type_Def;
Local_Spec_Success : Boolean;
Var_Expr_2 : Internal_Entity_Expr;
Var_Expr_74 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Sym : Symbol_Type;
Fld_5 : Internal_Aspect;
Fld_6 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_9 : Integer;
Fld_10 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_For_Loop_Spec;
Fld_13 : Internal_Entity_For_Loop_Spec;
Fld_14 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Fld_15 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Internal_Entity_For_Loop_Spec;
Fld_17 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_19 : Bare_For_Loop_Spec;
Fld_20 : Bare_Iter_Type;
Is_A : Boolean;
If_Result_1 : Logic_Equation;
Fld_21 : Boolean;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Equation;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
Fld_30 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_31 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_3 : Logic_Equation;
And_Pred_5 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Is_Equal_1 : Boolean;
Fld_32 : Integer;
Fld_33 : Integer;
Arith_Result : Integer;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result_4 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Let_Result_6 : Logic_Equation;
Let_Result_7 : Logic_Equation;
            procedure Finalizer_Scope_411 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_411;
            procedure Finalizer_Scope_2261 is
            begin
                     Dec_Ref (Fld_29);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_31);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2261;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2515 '<Let at nodes.lkt:3386:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3386'
--# scope-start
--# expr-start 2450 '<FieldAccess for base_aggregate at nodes.lkt:3386:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3386'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IteratedAssoc.base_aggregate
Fld := Libadalang.Implementation.Iterated_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2450
Local_Aggregate := Fld; 
--# bind aggregate Local_Aggregate
--# expr-start 2514 '<Let at nodes.lkt:3387:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3387'
--# scope-start
--# expr-start 2451 '<FieldAccess for multidim_root_aggregate at nodes.lkt:3387:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3387'
      if Local_Aggregate.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Local_Aggregate.Node, Origin => Origin, E_Info => Local_Aggregate.Info);
--# end
--# expr-done 2451
Local_Root_Agg := Fld_1; 
--# bind root_agg Local_Root_Agg
--# expr-start 2513 '<Let at nodes.lkt:3391:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
--# scope-start
--# expr-start 2457 '<If at nodes.lkt:3391:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
--# expr-start 2453 '<Not at nodes.lkt:3391:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
--# expr-start 2452 '<IsNull at nodes.lkt:3391:32>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
Is_Equal := Local_Root_Agg = No_Multidim_Aggregate_Info; 
--# expr-done 2452
Not_Val := not (Is_Equal); 
--# expr-done 2453
if Not_Val then
   --# expr-start 2454 '<FieldAccess for typ at nodes.lkt:3391:54>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
Fld_2 := Local_Root_Agg.Typ;
--# expr-done 2454
   If_Result := Fld_2;
else
   --# expr-start 2456 '<Cast to BaseTypeDecl at nodes.lkt:3391:72>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
--# expr-start 2455 '<FieldAccess for type_val at nodes.lkt:3391:72>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3391'
      if Local_Aggregate.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Local_Aggregate.Node);
--# end
--# expr-done 2455
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2456
   If_Result := Cast_Result;
end if;
--# expr-done 2457
Local_Type_Decl := If_Result; 
--# bind type_decl Local_Type_Decl
--# expr-start 2512 '<Let at nodes.lkt:3392:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3392'
--# scope-start
Var_Expr_74 := Local_Type_Decl;
if Var_Expr_74 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2458 '<FieldAccess for array_def at nodes.lkt:3392:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3392'
      if Var_Expr_74.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr_74.Node, Origin => Origin, E_Info => Var_Expr_74.Info);
--# end
--# expr-done 2458
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Local_Array_Type_Def := Result_Var; 
--# bind array_type_def Local_Array_Type_Def
--# expr-start 2511 '<Let at nodes.lkt:3394:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3394'
--# scope-start
--# expr-start 2462 '<Cast to Aggregate at nodes.lkt:3394:35>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3394'
--# expr-start 2461 '<FieldAccess for value at nodes.lkt:3394:35>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3394'
--# expr-start 2460 '<FieldAccess for get_aspect at nodes.lkt:3394:35>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3394'
      if Local_Type_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2459 '<Symbol Aggregate at nodes.lkt:3394:56>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3394'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 2459
--# property-call-start BasicDecl.get_aspect
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Local_Type_Decl.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Local_Type_Decl.Info);
--# end
--# expr-done 2460
Fld_6 := Fld_5.Value;
--# expr-done 2461
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 2462
Local_Container_Aggregate := Cast_Result_1; 
--# bind container_aggregate Local_Container_Aggregate
--# expr-start 2510 '<Let at nodes.lkt:3396:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3396'
--# scope-start
Var_Expr_0 := Local_Array_Type_Def;
if Var_Expr_0 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind atd Var_Expr_0
--# expr-start 2463 '<FieldAccess for comp_type at nodes.lkt:3397:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3397'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_7 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2463
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 2464 '<FieldAccess for element_type at nodes.lkt:3397:51>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3397'
      if Local_Container_Aggregate.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.element_type
Fld_8 := Libadalang.Implementation.Aggregate_P_Element_Type (Node => Local_Container_Aggregate.Node, E_Info => Local_Container_Aggregate.Info);
--# end
--# expr-done 2464
   Result_Var_1 := Fld_8;
end if;
Local_Comp_Type := Result_Var_1; 
--# bind comp_type Local_Comp_Type
--# expr-start 2509 '<Let at nodes.lkt:3400:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3400'
--# scope-start
Var_Expr_1 := Local_Array_Type_Def;
if Var_Expr_1 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind atd Var_Expr_1
--# expr-start 2466 '<FieldAccess for index_type at nodes.lkt:3401:22>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3401'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2465 '<FieldAccess for rank at nodes.lkt:3401:37>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3401'
Fld_9 := Local_Root_Agg.Rank;
--# expr-done 2465
--# property-call-start ArrayTypeDef.index_type
Fld_10 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Var_Expr_1.Node, Dim => Fld_9, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2466
Scope_Result_2 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 2467 '<FieldAccess for key_type at nodes.lkt:3401:65>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3401'
      if Local_Container_Aggregate.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.key_type
Fld_11 := Libadalang.Implementation.Aggregate_P_Key_Type (Node => Local_Container_Aggregate.Node, E_Info => Local_Container_Aggregate.Info);
--# end
--# expr-done 2467
   Result_Var_2 := Fld_11;
end if;
Local_Index_Type := Result_Var_2; 
--# bind index_type Local_Index_Type
--# expr-start 2508 '<Let at nodes.lkt:3409:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3409'
--# scope-start
--# expr-start 2483 '<FieldAccess for resolve_names_internal_with_eq at nodes.lkt:3409:28>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3409'
--# expr-start 2468 '<FieldAccess for spec at nodes.lkt:3409:28>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3409'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2468
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2482 '<If at nodes.lkt:3410:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2471 '<IsA IterType.In at nodes.lkt:3410:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2470 '<FieldAccess for loop_type at nodes.lkt:3410:16>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2469 '<FieldAccess for spec at nodes.lkt:3410:16>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Self.Iterated_Assoc_F_Spec;
--# expr-done 2469
      if Fld_19 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Fld_19.For_Loop_Spec_F_Loop_Type;
--# expr-done 2470
Is_A := Fld_20 /= null 
and then Fld_20.Kind in Ada_Iter_Type_In_Range; 
--# expr-done 2471
if Is_A then
   --# expr-start 2481 '<And at nodes.lkt:3410:56>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2476 '<Bind at nodes.lkt:3410:56>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2475 '<FieldAccess for expected_type_var at nodes.lkt:3410:60>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2474 '<Cast to Expr at nodes.lkt:3410:60>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2473 '<FieldAccess for iter_expr at nodes.lkt:3410:60>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2472 '<FieldAccess for spec at nodes.lkt:3410:60>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2472
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity (Node => Fld_13.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_13.Info);
--# expr-done 2473
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Expr
   then
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_2 := No_Entity_Expr;
   end if;
--# expr-done 2474
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_2.Node);
--# expr-done 2475
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
Cast_Expr := Local_Index_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_15, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3410:56") else null)); 
--# expr-done 2476
--# expr-start 2480 '<FieldAccess for matches_expected_type at nodes.lkt:3410:126>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2479 '<Cast to Expr at nodes.lkt:3410:126>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2478 '<FieldAccess for iter_expr at nodes.lkt:3410:126>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
--# expr-start 2477 '<FieldAccess for spec at nodes.lkt:3410:126>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3410'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2477
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity (Node => Fld_16.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_16.Info);
--# expr-done 2478
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Expr
   then
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_4 := No_Entity_Expr;
   end if;
--# expr-done 2479
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_18 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Cast_Result_4.Node, Origin => Origin);
--# end
--# expr-done 2480
And_Pred := Create_And (Bind_Result, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3410:56") else null)); 
--# expr-done 2481
   If_Result_1 := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3410:184") else null)); 
   If_Result_1 := True_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2482
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_21 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_12.Node, Additional_Equation => If_Result_1, Env => Env, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 2483
Local_Spec_Success := Fld_21; 
--# bind spec_success Local_Spec_Success
--# expr-start 2507 '<If at nodes.lkt:3413:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3413'
if Local_Spec_Success then
   --# expr-start 2506 '<If at nodes.lkt:3415:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
--# expr-start 2484 '<IsNull at nodes.lkt:3415:12>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
Is_Equal_1 := Local_Root_Agg = No_Multidim_Aggregate_Info; 
--# expr-done 2484
if Is_Equal_1 then
   If_Result_2 := True;
else
   --# expr-start 2488 '<Eq at nodes.lkt:3415:32>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
--# expr-start 2485 '<FieldAccess for rank at nodes.lkt:3415:32>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
Fld_32 := Local_Root_Agg.Rank;
--# expr-done 2485
--# expr-start 2487 '<Arithmetic - at nodes.lkt:3415:49>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
--# expr-start 2486 '<FieldAccess for array_ndims at nodes.lkt:3415:49>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3415'
      if Local_Array_Type_Def.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_33 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Local_Array_Type_Def.Node, Origin => Origin, E_Info => Local_Array_Type_Def.Info);
--# end
--# expr-done 2486
Arith_Result := (Fld_33 - 1); 
--# expr-done 2487
Is_Equal_2 := Fld_32 = Arith_Result; 
--# expr-done 2488
   If_Result_2 := Is_Equal_2;
end if;
if If_Result_2 then
   --# expr-start 2505 '<And at nodes.lkt:3417:13>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3417'
--# expr-start 2497 '<And at nodes.lkt:3418:17>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3418'
--# expr-start 2494 '<And at nodes.lkt:3419:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
--# expr-start 2490 '<FieldAccess for sub_equation at nodes.lkt:3419:21>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
--# expr-start 2489 '<FieldAccess for expr at nodes.lkt:3419:21>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2489
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_23 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_22.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_22.Info);
--# end
--# expr-done 2490
--# expr-start 2493 '<Bind at nodes.lkt:3419:52>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
--# expr-start 2492 '<FieldAccess for expected_type_var at nodes.lkt:3419:56>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
--# expr-start 2491 '<FieldAccess for expr at nodes.lkt:3419:56>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3419'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2491
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_24.Node);
--# expr-done 2492
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
Cast_Expr_1 := Local_Comp_Type; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_25, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3419:52") else null)); 
--# expr-done 2493
And_Pred_1 := Create_And (Fld_23, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3419:21") else null)); 
--# expr-done 2494
--# expr-start 2496 '<FieldAccess for matches_expected_type at nodes.lkt:3420:23>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3420'
--# expr-start 2495 '<FieldAccess for expr at nodes.lkt:3420:23>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3420'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_26 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2495
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_27 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_26.Node, Origin => Origin);
--# end
--# expr-done 2496
And_Pred_2 := Create_And (And_Pred_1, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3418:17") else null)); 
--# expr-done 2497
--# expr-start 2498 '<FieldAccess for key_expr at nodes.lkt:3423:17>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3423'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Ent.Node.Iterated_Assoc_F_Key_Expr, Info => Ent.Info);
--# expr-done 2498
Var_Expr_2 := Fld_28;
if Var_Expr_2 /= No_Entity_Expr then
   --# scope-start
   --# bind ke Var_Expr_2
--# expr-start 2504 '<And at nodes.lkt:3424:29>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3424'
--# expr-start 2502 '<And at nodes.lkt:3425:25>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3425'
--# expr-start 2499 '<FieldAccess for sub_equation at nodes.lkt:3425:25>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3425'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2499
--# expr-start 2501 '<Bind at nodes.lkt:3425:47>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3425'
--# expr-start 2500 '<FieldAccess for expected_type_var at nodes.lkt:3425:51>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3425'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_2.Node);
--# expr-done 2500
Fld_30.Value := No_Entity;
Entity_Vars.Reset (Fld_30);
Cast_Expr_2 := Local_Index_Type; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_30, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3425:47") else null)); 
--# expr-done 2501
And_Pred_3 := Create_And (Fld_29, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3425:25") else null)); 
--# expr-done 2502
--# expr-start 2503 '<FieldAccess for matches_expected_type at nodes.lkt:3426:27>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3426'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_31 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_2.Node, Origin => Origin);
--# end
--# expr-done 2503
And_Pred_4 := Create_And (And_Pred_3, Fld_31, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3424:29") else null)); 
--# expr-done 2504
Scope_Result_3 := And_Pred_4;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2261;
   Result_Var_3 := Scope_Result_3;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3426:67") else null)); 
   Result_Var_3 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_3);
And_Pred_5 := Create_And (And_Pred_2, Result_Var_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3417:13") else null)); 
--# expr-done 2505
   If_Result_3 := And_Pred_5;
else
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3432:13") else null)); 
   If_Result_3 := True_Rel_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2506
   If_Result_4 := If_Result_3;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3433:16") else null)); 
   If_Result_4 := False_Rel;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 2507
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2508
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2509
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2510
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2511
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 2512
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 2513
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 2514
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 2515
         Property_Result := Let_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_411;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_411;
                     Finalizer_Scope_2261;
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Equation;
--# end
--# property-start IteratedAssoc.xref_equation_for_reduce 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3439'
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation_For_Reduce
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc  := Bare_Iterated_Assoc (Node);
     Ent : Internal_Entity_Iterated_Assoc :=
       Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_412 with Inline_Always;
      Local_Expected_Typ : Logic_Var;
Local_Spec_Success : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Value_Sequence;
Fld_1 : Internal_Entity_Iterated_Assoc;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_For_Loop_Spec;
Fld_5 : Internal_Entity_For_Loop_Spec;
Fld_6 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_7 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_8 : Internal_Entity_For_Loop_Spec;
Fld_9 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_11 : Bare_For_Loop_Spec;
Fld_12 : Bare_Iter_Type;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_412 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_412;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2548 '<Let at nodes.lkt:3442:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
--# scope-start
--# expr-start 2520 '<FieldAccess for expected_type_var at nodes.lkt:3442:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
--# expr-start 2519 '<FieldAccess for expr at nodes.lkt:3442:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
--# expr-start 2518 '<FieldAccess for iter_assoc at nodes.lkt:3442:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
--# expr-start 2517 '<Cast to ValueSequence at nodes.lkt:3442:28>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
--# expr-start 2516 '<parent at nodes.lkt:3442:28>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3442'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2516
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Value_Sequence_Range
   then
      Cast_Result := Create_Internal_Entity_Value_Sequence
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Value_Sequence;
   end if;
--# expr-done 2517
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Iterated_Assoc (Node => Cast_Result.Node.Value_Sequence_F_Iter_Assoc, Info => Cast_Result.Info);
--# expr-done 2518
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2519
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2.Node);
--# expr-done 2520
Local_Expected_Typ := Fld_3; 
--# bind expected_typ Local_Expected_Typ
--# expr-start 2547 '<Let at nodes.lkt:3444:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3444'
--# scope-start
--# expr-start 2536 '<FieldAccess for resolve_names_internal_with_eq at nodes.lkt:3444:28>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3444'
--# expr-start 2521 '<FieldAccess for spec at nodes.lkt:3444:28>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3444'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2521
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2535 '<If at nodes.lkt:3445:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2524 '<IsA IterType.In at nodes.lkt:3445:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2523 '<FieldAccess for loop_type at nodes.lkt:3445:16>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2522 '<FieldAccess for spec at nodes.lkt:3445:16>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Iterated_Assoc_F_Spec;
--# expr-done 2522
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.For_Loop_Spec_F_Loop_Type;
--# expr-done 2523
Is_A := Fld_12 /= null 
and then Fld_12.Kind in Ada_Iter_Type_In_Range; 
--# expr-done 2524
if Is_A then
   --# expr-start 2534 '<And at nodes.lkt:3445:56>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2529 '<Bind at nodes.lkt:3445:56>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2528 '<FieldAccess for expected_type_var at nodes.lkt:3445:60>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2527 '<Cast to Expr at nodes.lkt:3445:60>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2526 '<FieldAccess for iter_expr at nodes.lkt:3445:60>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2525 '<FieldAccess for spec at nodes.lkt:3445:60>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2525
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity (Node => Fld_5.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_5.Info);
--# expr-done 2526
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Expr
   then
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Expr;
   end if;
--# expr-done 2527
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Cast_Result_1.Node);
--# expr-done 2528
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Local_Expected_Typ.Value := No_Entity;
Entity_Vars.Reset (Local_Expected_Typ);
Bind_Result := Solver.Create_Unify (Fld_7, Local_Expected_Typ, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3445:56") else null)); 
--# expr-done 2529
--# expr-start 2533 '<FieldAccess for matches_expected_type at nodes.lkt:3445:128>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2532 '<Cast to Expr at nodes.lkt:3445:128>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2531 '<FieldAccess for iter_expr at nodes.lkt:3445:128>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
--# expr-start 2530 '<FieldAccess for spec at nodes.lkt:3445:128>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3445'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 2530
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity (Node => Fld_8.Node.For_Loop_Spec_F_Iter_Expr, Info => Fld_8.Info);
--# expr-done 2531
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Expr
   then
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_2 := No_Entity_Expr;
   end if;
--# expr-done 2532
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Cast_Result_2.Node, Origin => Origin);
--# end
--# expr-done 2533
And_Pred := Create_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3445:56") else null)); 
--# expr-done 2534
   If_Result := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3445:186") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2535
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_13 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_4.Node, Additional_Equation => If_Result, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2536
Local_Spec_Success := Fld_13; 
--# bind spec_success Local_Spec_Success
--# expr-start 2546 '<If at nodes.lkt:3448:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3448'
if Local_Spec_Success then
   --# expr-start 2545 '<And at nodes.lkt:3448:30>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3448'
--# expr-start 2542 '<And at nodes.lkt:3449:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
--# expr-start 2538 '<FieldAccess for sub_equation at nodes.lkt:3449:13>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
--# expr-start 2537 '<FieldAccess for expr at nodes.lkt:3449:13>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2537
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_14.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_14.Info);
--# end
--# expr-done 2538
--# expr-start 2541 '<Bind at nodes.lkt:3449:44>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
--# expr-start 2540 '<FieldAccess for expected_type_var at nodes.lkt:3449:48>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
--# expr-start 2539 '<FieldAccess for expr at nodes.lkt:3449:48>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2539
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_16.Node);
--# expr-done 2540
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Local_Expected_Typ.Value := No_Entity;
Entity_Vars.Reset (Local_Expected_Typ);
Bind_Result_1 := Solver.Create_Unify (Fld_17, Local_Expected_Typ, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3449:44") else null)); 
--# expr-done 2541
And_Pred_1 := Create_And (Fld_15, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3449:13") else null)); 
--# expr-done 2542
--# expr-start 2544 '<FieldAccess for matches_expected_type at nodes.lkt:3450:15>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3450'
--# expr-start 2543 '<FieldAccess for expr at nodes.lkt:3450:15>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3450'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2543
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_19 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_18.Node, Origin => Origin);
--# end
--# expr-done 2544
And_Pred_2 := Create_And (And_Pred_1, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3448:30") else null)); 
--# expr-done 2545
   If_Result_1 := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3450:56") else null)); 
   If_Result_1 := False_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2546
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2547
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2548
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_412;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_412;
            raise;
      end;
   return Property_Result;
end Iterated_Assoc_P_Xref_Equation_For_Reduce;
--# end
   --
   --  Primitives for Bare_Param_Assoc
   --
      procedure Initialize_Fields_For_Param_Assoc
        (Self : Bare_Param_Assoc
         ; Param_Assoc_F_Designator : Bare_Ada_Node
         ; Param_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Assoc_F_Designator := Param_Assoc_F_Designator;
            Self.Param_Assoc_F_R_Expr := Param_Assoc_F_R_Expr;
      end Initialize_Fields_For_Param_Assoc;
   function Param_Assoc_F_Designator
     (Node : Bare_Param_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Param_Assoc_F_Designator;
   end;
   function Param_Assoc_F_R_Expr
     (Node : Bare_Param_Assoc) return Bare_Expr
   is
   begin
         return Node.Param_Assoc_F_R_Expr;
   end;
--# property-start ParamAssoc.expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3459'
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Expr
  (Node : Bare_Param_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
     Ent : Internal_Entity_Param_Assoc :=
       Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2549 '<FieldAccess for r_expr at nodes.lkt:3459:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3459'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 2549
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Expr;
--# end
--# property-start ParamAssoc.names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3461'
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Names
  (Node : Bare_Param_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_414 with Inline_Always;
      Fld : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_414 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_414;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2554 '<If at nodes.lkt:3462:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3462'
--# expr-start 2551 '<IsNull at nodes.lkt:3462:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3462'
--# expr-start 2550 '<FieldAccess for designator at nodes.lkt:3462:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3462'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Param_Assoc_F_Designator;
--# expr-done 2550
Is_Null := Fld_1 = null; 
--# expr-done 2551
if Is_Null then
   If_Result := No_Bare_Ada_Node_Array_Type;
else
   --# expr-start 2553 '<ArrayLiteral at nodes.lkt:3462:67>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3462'
--# expr-start 2552 '<FieldAccess for designator at nodes.lkt:3462:68>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3462'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Assoc_F_Designator;
--# expr-done 2552
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Fld)); 
--# expr-done 2553
   If_Result := Array_Lit;
end if;
      Inc_Ref (If_Result);
--# expr-done 2554
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_414;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_414;
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Names;
--# end
--# property-start ParamAssoc.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3464'
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Entry_Point
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2555 '<FieldAccess for is_static_attribute_assoc at nodes.lkt:3464:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3464'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ParamAssoc.is_static_attribute_assoc
Fld := Libadalang.Implementation.Param_Assoc_P_Is_Static_Attribute_Assoc (Node => Self);
--# end
--# expr-done 2555
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Xref_Entry_Point;
--# end
--# property-start ParamAssoc.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3467'
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Equation
  (Node : Bare_Param_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
     Ent : Internal_Entity_Param_Assoc :=
       Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_416 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_416 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_416;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2559 '<If at nodes.lkt:3468:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3468'
--# expr-start 2556 '<FieldAccess for xref_entry_point at nodes.lkt:3468:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3468'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Self);
--# end
--# expr-done 2556
if Fld_2 then
   --# expr-start 2558 '<FieldAccess for sub_equation at nodes.lkt:3468:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3468'
--# expr-start 2557 '<FieldAccess for expr at nodes.lkt:3468:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3468'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2557
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 2558
   If_Result := Fld_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3468:73") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2559
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_416;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_416;
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Xref_Equation;
--# end
--# property-start ParamAssoc.is_static_attribute_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3470'
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Is_Static_Attribute_Assoc
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc  := Bare_Param_Assoc (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_0 : Bare_Attribute_Ref;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Attribute_Ref;
Fld_2 : Bare_Identifier;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 2562 '<Cast to AttributeRef at nodes.lkt:3470:45>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3470'
--# expr-start 2561 '<parent at nodes.lkt:3470:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3470'
--# expr-start 2560 '<parent at nodes.lkt:3470:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3470'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2560
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2561
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2562
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ar Var_Expr_0
--# expr-start 2564 '<FieldAccess for name_symbol at nodes.lkt:3471:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
--# expr-start 2563 '<FieldAccess for attribute at nodes.lkt:3471:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_0.Attribute_Ref_F_Attribute;
--# expr-done 2563
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 2564
--# expr-start 2565 '<Symbol First at nodes.lkt:3471:47>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 2565
Is_Equal_5 := Fld_3 = Sym; 
if Is_Equal_5 then
   If_Result_4 := True;
else
--# expr-start 2566 '<Symbol Last at nodes.lkt:3471:58>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 2566
Is_Equal_4 := Fld_3 = Sym_1; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 2567 '<Symbol Range at nodes.lkt:3471:68>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 2567
Is_Equal_3 := Fld_3 = Sym_2; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 2568 '<Symbol Length at nodes.lkt:3471:79>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 2568
Is_Equal_2 := Fld_3 = Sym_3; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 2569 '<Symbol Has_Same_Storage at nodes.lkt:3471:91>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 2569
Is_Equal_1 := Fld_3 = Sym_4; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 2570 '<Symbol Overlaps_Storage at nodes.lkt:3471:113>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3471'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 2570
Is_Equal := Fld_3 = Sym_5; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
Scope_Result := If_Result_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Assoc_P_Is_Static_Attribute_Assoc;
--# end
   --
   --  Primitives for Bare_Basic_Decl
   --
   function Basic_Decl_F_Aspects
     (Node : Bare_Basic_Decl) return Bare_Aspect_Spec
   is
         Kind : constant Ada_Basic_Decl := Node.Kind;
   begin
         case Kind is
               when Ada_Abstract_State_Decl =>
         return Node.Abstract_State_Decl_F_Aspects;
               when Ada_Anonymous_Expr_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Component_Decl =>
         return Node.Component_Decl_F_Aspects;
               when Ada_Discriminant_Spec =>
         return Node.Discriminant_Spec_F_Aspects;
               when Ada_Generic_Formal_Obj_Decl | Ada_Generic_Formal_Package | Ada_Generic_Formal_Subp_Decl | Ada_Generic_Formal_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Param_Spec =>
         return Node.Param_Spec_F_Aspects;
               when Ada_Synthetic_Formal_Param_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Internal | Ada_Package_Decl =>
         return Node.Base_Package_Decl_F_Aspects;
               when Ada_Discrete_Base_Subtype_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Subtype_Decl =>
         return Node.Subtype_Decl_F_Aspects;
               when Ada_Classwide_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Formal_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Protected_Type_Decl =>
         return Node.Protected_Type_Decl_F_Aspects;
               when Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl =>
         return Node.Task_Type_Decl_F_Aspects;
               when Ada_Anonymous_Type_Decl | Ada_Synth_Anonymous_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Concrete_Type_Decl =>
         return Node.Concrete_Type_Decl_F_Aspects;
               when Ada_Formal_Type_Decl =>
         return Node.Formal_Type_Decl_F_Aspects;
               when Ada_Abstract_Subp_Decl =>
         return Node.Abstract_Subp_Decl_F_Aspects;
               when Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl =>
         return Node.Formal_Subp_Decl_F_Aspects;
               when Ada_Subp_Decl =>
         return Node.Subp_Decl_F_Aspects;
               when Ada_Entry_Decl =>
         return Node.Entry_Decl_F_Aspects;
               when Ada_Enum_Literal_Decl | Ada_Synthetic_Char_Enum_Lit =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Subp_Internal =>
         return Node.Generic_Subp_Internal_F_Aspects;
               when Ada_Synthetic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Accept_Stmt_Body =>
                     return No_Bare_Ada_Node;
               when Ada_Expr_Function =>
         return Node.Expr_Function_F_Aspects;
               when Ada_Null_Subp_Decl =>
         return Node.Null_Subp_Decl_F_Aspects;
               when Ada_Subp_Body =>
         return Node.Subp_Body_F_Aspects;
               when Ada_Subp_Renaming_Decl =>
         return Node.Subp_Renaming_Decl_F_Aspects;
               when Ada_Package_Body_Stub =>
         return Node.Package_Body_Stub_F_Aspects;
               when Ada_Protected_Body_Stub =>
         return Node.Protected_Body_Stub_F_Aspects;
               when Ada_Subp_Body_Stub =>
         return Node.Subp_Body_Stub_F_Aspects;
               when Ada_Task_Body_Stub =>
         return Node.Task_Body_Stub_F_Aspects;
               when Ada_Entry_Body =>
         return Node.Entry_Body_F_Aspects;
               when Ada_Package_Body =>
         return Node.Package_Body_F_Aspects;
               when Ada_Protected_Body =>
         return Node.Protected_Body_F_Aspects;
               when Ada_Task_Body =>
         return Node.Task_Body_F_Aspects;
               when Ada_Entry_Index_Spec =>
         return Node.Entry_Index_Spec_F_Aspects;
               when Ada_Error_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Exception_Decl =>
         return Node.Exception_Decl_F_Aspects;
               when Ada_Exception_Handler =>
                     return No_Bare_Ada_Node;
               when Ada_For_Loop_Var_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Instantiation =>
         return Node.Generic_Package_Instantiation_F_Aspects;
               when Ada_Generic_Subp_Instantiation =>
         return Node.Generic_Subp_Instantiation_F_Aspects;
               when Ada_Generic_Package_Renaming_Decl =>
         return Node.Generic_Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Subp_Renaming_Decl =>
         return Node.Generic_Subp_Renaming_Decl_F_Aspects;
               when Ada_Label_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Named_Stmt_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Number_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl | Ada_No_Type_Object_Renaming_Decl =>
         return Node.Object_Decl_F_Aspects;
               when Ada_Package_Renaming_Decl =>
         return Node.Package_Renaming_Decl_F_Aspects;
               when Ada_Single_Protected_Decl =>
         return Node.Single_Protected_Decl_F_Aspects;
               when Ada_Single_Task_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Synthetic_Object_Decl =>
                     return No_Bare_Ada_Node;
         end case;
   end;
--# property-start BasicDecl.spark_mode_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3484'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Spark_Mode_Aspect
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Aspect;
Var_Expr_2 : Internal_Entity_Pragma_Node;
Var_Expr_75 : Internal_Entity_Declarative_Part;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Declarative_Part;
Fld_4 : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_5 : Bare_Compilation_Unit;
Fld_6 : Internal_Entity_Pragma_Node;
Fld_7 : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Scope_Result_3 : Internal_Aspect;
Result_Var_3 : Internal_Aspect;
Sym_1 : Symbol_Type;
Fld_8 : Boolean;
If_Result_2 : Internal_Aspect;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Spark_Mode_Aspect,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Aspect;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2585 '<If at nodes.lkt:3485:5>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3485'
--# expr-start 2572 '<FieldAccess for has_aspect at nodes.lkt:3485:8>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3485'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2571 '<Symbol SPARK_Mode at nodes.lkt:3485:24>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3485'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 2571
--# property-call-start BasicDecl.has_aspect
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2572
if Fld_8 then
   --# expr-start 2574 '<FieldAccess for get_aspect at nodes.lkt:3485:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3485'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2573 '<Symbol SPARK_Mode at nodes.lkt:3485:60>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3485'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 2573
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2574
   If_Result_2 := Fld;
else
--# expr-start 2580 '<as_entity at nodes.lkt:3488:10>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3488'
--# expr-start 2579 '<FieldAccess for declarative_scope at nodes.lkt:3488:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3488'
--# expr-start 2578 '<If at nodes.lkt:3489:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3489'
--# expr-start 2575 '<FieldAccess for previous_part_for_decl at nodes.lkt:3489:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3489'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2575
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 2576 '<IsA BodyStub at nodes.lkt:3489:53>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3489'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Body_Stub; 
--# expr-done 2576
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 2577 '<FieldAccess for previous_part_for_decl at nodes.lkt:3489:74>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3489'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2577
   If_Result := Fld_1;
else
   If_Result := Ent;
end if;
--# expr-done 2578
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_3 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => If_Result.Node);
--# end
--# expr-done 2579
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Fld_3); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 2580
Var_Expr_75 := If_Result_1;
if Var_Expr_75 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 2581 '<FieldAccess for spark_mode_aspect at nodes.lkt:3488:10>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3488'
      if Var_Expr_75.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Var_Expr_75.Node, E_Info => Var_Expr_75.Info);
--# end
--# expr-done 2581
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Aspect then
   --# scope-start
   --# bind asp Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 2583 '<FieldAccess for spark_config_pragma at nodes.lkt:3493:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3493'
--# expr-start 2582 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:3493:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3493'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_5 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Ent.Node);
--# end
--# expr-done 2582
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.spark_config_pragma
Fld_6 := Libadalang.Implementation.Compilation_Unit_P_Spark_Config_Pragma (Node => Fld_5);
--# end
--# expr-done 2583
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 2584 '<FieldAccess for as_aspect at nodes.lkt:3494:20>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3494'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.as_aspect
Fld_7 := Libadalang.Implementation.Pragma_Node_P_As_Aspect (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2584
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Aspect;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Aspect;
end if;
   If_Result_2 := Result_Var_3;
end if;
--# expr-done 2585
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Aspect,
                        As_Internal_Aspect => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Spark_Mode_Aspect;
--# end
--# property-start BasicDecl.subp_constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3504'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_419 with Inline_Always;
      Var_Expr_76 : Internal_Entity_Defining_Name;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel : Logic_Equation;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_419 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_419;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2598 '<If at nodes.lkt:3508:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2588 '<FieldAccess for dottable_subp at nodes.lkt:3508:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2587 '<FieldAccess for md at nodes.lkt:3508:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2586 '<FieldAccess for info at nodes.lkt:3508:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
Fld_6 := Ent.Info;
--# expr-done 2586
Fld_7 := Fld_6.Md;
--# expr-done 2587
Fld_8 := Fld_7.Dottable_Subp;
--# expr-done 2588
if Fld_8 then
   --# expr-start 2597 '<And at nodes.lkt:3508:44>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2595 '<Bind at nodes.lkt:3508:44>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2589 '<FieldAccess for expected_type_var at nodes.lkt:3508:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Prefix);
--# expr-done 2589
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 2592 '<at at nodes.lkt:3508:76>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2591 '<FieldAccess for unpacked_formal_params at nodes.lkt:3508:76>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2590 '<FieldAccess for subp_spec_or_null at nodes.lkt:3508:76>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2590
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2591
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 2592
Var_Expr_76 := Get_Result;
if Var_Expr_76 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2594 '<FieldAccess for formal_type at nodes.lkt:3508:76>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
--# expr-start 2593 '<FieldAccess for formal_decl at nodes.lkt:3508:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
      if Var_Expr_76.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_76.Node, E_Info => Var_Expr_76.Info);
--# end
--# expr-done 2593
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 2594
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3508:44") else null)); 
--# expr-done 2595
--# expr-start 2596 '<FieldAccess for matches_expected_prefix_type at nodes.lkt:3508:164>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3508'
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_prefix_type
Fld_5 := Libadalang.Implementation.Expr_P_Matches_Expected_Prefix_Type (Node => Prefix, Origin => Origin);
--# end
--# expr-done 2596
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3508:44") else null)); 
--# expr-done 2597
   If_Result := And_Pred;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:3508:207") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 2598
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_419;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_419;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Constrain_Prefix;
--# end
--# property-start BasicDecl.env_hook_basic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3511'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Hook_Basic_Decl
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_2268 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Dotted_Name;
Var_Expr_1 : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_2268 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2268;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2609 '<If at nodes.lkt:3514:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
--# expr-start 2599 '<IsA PackageDecl, BasicSubpDecl, PackageRenamingDecl, GenericPackageDecl, GenericPackageInstantiation, GenericSubpInstantiation, GenericSubpDecl, SubpBody at nodes.lkt:3514:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Basic_Subp_Decl | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range | Ada_Generic_Package_Instantiation_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Decl_Range | Ada_Subp_Body_Range; 
--# expr-done 2599
if Is_A_1 then
--# expr-start 2603 '<Cast to DottedName at nodes.lkt:3514:182>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
--# expr-start 2602 '<FieldAccess for name at nodes.lkt:3514:182>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
--# expr-start 2601 '<FieldAccess for defining_name at nodes.lkt:3514:182>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
--# expr-start 2600 '<as_bare_entity at nodes.lkt:3514:182>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3514'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2600
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2601
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 2602
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 2603
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 2608 '<FieldAccess for get_unit at nodes.lkt:3515:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3515'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2605 '<FieldAccess for as_symbol_array at nodes.lkt:3516:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3516'
--# expr-start 2604 '<FieldAccess for prefix at nodes.lkt:3516:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3516'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 2604
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_2.Node);
--# end
--# expr-done 2605
--# expr-start 2607 '<Not at nodes.lkt:3516:123>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3516'
--# expr-start 2606 '<IsA SubpBody at nodes.lkt:3516:127>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3516'
Is_A := Self /= null 
and then Self.Kind in Ada_Subp_Body_Range; 
--# expr-done 2606
Not_Val := not (Is_A); 
--# expr-done 2607
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_3, Kind => Unit_Specification, Load_If_Needed => True, Not_Found_Is_Error => Not_Val);
--# expr-done 2608
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= null then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_2268;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
--# expr-done 2609
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2268;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Hook_Basic_Decl;
--# end
--# property-start BasicDecl.populate_body_unit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3523'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Body_Unit
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_421 with Inline_Always;
      Var_Expr_0 : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type_Array_Access;
Fld_3 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_421 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_421;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2616 '<If at nodes.lkt:3523:38>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3523'
--# expr-start 2610 '<FieldAccess for is_library_item at nodes.lkt:3523:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3523'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 2610
if Fld_4 then
--# expr-start 2615 '<FieldAccess for get_unit at nodes.lkt:3523:69>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3523'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2614 '<FieldAccess for as_symbol_array at nodes.lkt:3524:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3524'
--# expr-start 2613 '<FieldAccess for name at nodes.lkt:3524:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3524'
--# expr-start 2612 '<FieldAccess for defining_name at nodes.lkt:3524:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3524'
--# expr-start 2611 '<as_bare_entity at nodes.lkt:3524:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3524'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2611
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2612
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 2613
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_1.Node);
--# end
--# expr-done 2614
Fld_3 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_2, Kind => Unit_Body, Load_If_Needed => True, Not_Found_Is_Error => False);
--# expr-done 2615
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= null then
   --# scope-start
   --# bind _ Var_Expr_0
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
--# expr-done 2616
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_421;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_421;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Populate_Body_Unit;
--# end
--# property-start BasicDecl.has_top_level_env_name_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3529'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name_Impl
  (Node : Bare_Basic_Decl
      ; Allow_Bodies : Boolean
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind allow_bodies Allow_Bodies
   Property_Result : Boolean;
            procedure Finalizer_Scope_422 with Inline_Always;
      Local_Is_Decl : Boolean;
Local_Is_Body : Boolean;
Var_Expr_0 : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Is_A_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Is_A_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
Fld_2 : Boolean;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Is_A_4 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result_5 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_422 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_422;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2635 '<Let at nodes.lkt:3530:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3530'
--# scope-start
--# expr-start 2617 '<IsA BasePackageDecl, BasicSubpDecl, GenericDecl, TaskTypeDecl, ProtectedTypeDecl, SingleTaskDecl, SingleProtectedDecl at nodes.lkt:3530:23>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3530'
Is_A := Self /= null 
and then Self.Kind in Ada_Base_Package_Decl | Ada_Basic_Subp_Decl | Ada_Generic_Decl | Ada_Task_Type_Decl_Range | Ada_Protected_Type_Decl_Range | Ada_Single_Task_Decl_Range | Ada_Single_Protected_Decl_Range; 
--# expr-done 2617
Local_Is_Decl := Is_A; 
--# bind is_decl Local_Is_Decl
--# expr-start 2634 '<Let at nodes.lkt:3531:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3531'
--# scope-start
--# expr-start 2618 '<IsA Body at nodes.lkt:3531:23>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3531'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Body_Node; 
--# expr-done 2618
Local_Is_Body := Is_A_1; 
--# bind is_body Local_Is_Body
--# expr-start 2619 '<FieldAccess for is_compilation_unit_root at nodes.lkt:3533:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3533'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2619
if Fld then
   If_Result_7 := True;
else
if Local_Is_Decl then
   If_Result_1 := True;
else
if Allow_Bodies then
   If_Result := Local_Is_Body;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
--# expr-start 2621 '<env_node at nodes.lkt:3534:57>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3534'
--# expr-start 2620 '<FieldAccess for node_env at nodes.lkt:3534:57>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3534'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 2620
Env_Node := AST_Envs.Env_Node (Fld_1); 
--# expr-done 2621
Var_Expr_0 := Env_Node;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind env_node Var_Expr_0
--# expr-start 2622 '<Cast to BasicDecl at nodes.lkt:3535:31>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3535'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2622
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2629 '<If at nodes.lkt:3536:28>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3536'
--# expr-start 2623 '<Eq at nodes.lkt:3536:31>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3536'
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Cast_Result_1 = Var_Expr_1; 
--# expr-done 2623
if Is_Equal then
   If_Result_4 := True;
else
   --# expr-start 2628 '<FieldAccess for has_top_level_env_name_impl at nodes.lkt:3536:56>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3536'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
if Allow_Bodies then
   --# expr-start 2625 '<Not at nodes.lkt:3537:55>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3537'
--# expr-start 2624 '<IsA BaseSubpBody at nodes.lkt:3537:59>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3537'
Is_A_2 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 2624
Not_Val := not (Is_A_2); 
--# expr-done 2625
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2627 '<Not at nodes.lkt:3537:84>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3537'
--# expr-start 2626 '<IsA BaseSubpBody at nodes.lkt:3537:88>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3537'
Is_A_3 := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Subp_Body; 
--# expr-done 2626
Not_Val_1 := not (Is_A_3); 
--# expr-done 2627
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => If_Result_3);
--# end
--# expr-done 2628
   If_Result_4 := Fld_2;
end if;
--# expr-done 2629
Scope_Result := If_Result_4;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 2630 '<IsA PrivatePart at nodes.lkt:3538:36>' Is_A_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3538'
Is_A_4 := Var_Expr_0 /= null 
and then Var_Expr_0.Kind in Ada_Private_Part_Range; 
--# expr-done 2630
if Is_A_4 then
--# expr-start 2632 '<Cast to BasicDecl at nodes.lkt:3538:64>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3538'
--# expr-start 2631 '<parent at nodes.lkt:3538:64>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3538'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_0.Parent; 
--# expr-done 2631
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2632
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 2633 '<FieldAccess for has_top_level_env_name_impl at nodes.lkt:3539:33>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3539'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => Allow_Bodies);
--# end
--# expr-done 2633
Scope_Result_1 := Fld_4;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result_5 := Result_Var;
else
   If_Result_5 := False;
end if;
   Result_Var_1 := If_Result_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_6 := Result_Var_2;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 2634
Let_Result_1 := Let_Result; 
--# end
--# expr-done 2635
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_422;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_422;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Top_Level_Env_Name_Impl;
--# end
--# property-start BasicDecl.has_top_level_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3607'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_423 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Is_A : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
            procedure Finalizer_Scope_423 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_423;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2637 '<env_node at nodes.lkt:3612:9>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3612'
--# expr-start 2636 '<FieldAccess for children_env at nodes.lkt:3612:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3612'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2636
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 2637
Var_Expr_0 := Env_Node;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind env_node Var_Expr_0
--# expr-start 2638 '<Cast to BasicDecl at nodes.lkt:3613:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3613'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2638
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 2639 '<FieldAccess for has_top_level_env_name_impl at nodes.lkt:3614:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3614'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => True);
--# end
--# expr-done 2639
Scope_Result := Fld_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 2640 '<IsA PrivatePart at nodes.lkt:3614:88>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3614'
Is_A := Var_Expr_0 /= null 
and then Var_Expr_0.Kind in Ada_Private_Part_Range; 
--# expr-done 2640
if Is_A then
--# expr-start 2642 '<Cast to BasicDecl at nodes.lkt:3614:116>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3614'
--# expr-start 2641 '<parent at nodes.lkt:3614:116>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3614'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_0.Parent; 
--# expr-done 2641
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2642
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 2643 '<FieldAccess for has_top_level_env_name_impl at nodes.lkt:3615:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3615'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => True);
--# end
--# expr-done 2643
Scope_Result_1 := Fld_3;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   Result_Var_1 := If_Result;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
         Property_Result := Result_Var_2;
   --# end
      Finalizer_Scope_423;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_423;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Top_Level_Env_Name;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3621'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl
  (Node : Bare_Basic_Decl
      ; Self_Env : Lexical_Env
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind self_env Self_Env
   Property_Result : String_Type;
            procedure Finalizer_Scope_424 with Inline_Always;
            procedure Finalizer_Scope_2279 with Inline_Always;
      Var_Expr_0 : Bare_Basic_Decl;
Var_Expr : Bare_Basic_Decl;
Var_Expr_77 : Bare_Private_Part;
Fld : Bare_Compilation_Unit;
Fld_1 : Symbol_Type_Array_Access;
Str : String_Type;
Fld_2 : String_Type;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_4 : Bare_Ada_Node;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : String_Type;
Env_Node : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Private_Part;
Fld_7 : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_3 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Fld_8 : Lexical_Env := Empty_Env;
Fld_9 : String_Type;
Str_1 : String_Type;
Concat_Result : String_Type;
Fld_10 : Symbol_Type;
Fld_11 : String_Type;
Concat_Result_1 : String_Type;
Scope_Result_2 : String_Type;
Result_Var_2 : String_Type;
Is_A : Boolean;
If_Result : String_Type;
Fld_12 : Boolean;
If_Result_1 : String_Type;
            procedure Finalizer_Scope_424 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_424;
            procedure Finalizer_Scope_2279 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_2279;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2667 '<If at nodes.lkt:3624:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3624'
--# expr-start 2644 '<FieldAccess for is_compilation_unit_root at nodes.lkt:3624:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3624'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2644
if Fld_12 then
   --# expr-start 2648 '<FieldAccess for sym_join at nodes.lkt:3624:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3624'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2646 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:3625:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3625'
--# expr-start 2645 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:3625:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3625'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 2645
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Fld);
--# end
--# expr-done 2646
--# expr-start 2647 '<String at nodes.lkt:3625:81>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3625'
Str := Create_String ("."); 
--# expr-done 2647
--# property-call-start AdaNode.sym_join
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Self, Syms => Fld_1, Sep => Str);
--# end
--# expr-done 2648
   If_Result_1 := Fld_2;
else
   --# expr-start 2666 '<If at nodes.lkt:3624:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3624'
--# expr-start 2649 '<IsA GenericPackageInternal, GenericSubpInternal at nodes.lkt:3629:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range; 
--# expr-done 2649
if Is_A then
   --# expr-start 2654 '<FieldAccess for env_spec_fully_qualified_name_impl at nodes.lkt:3629:72>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
--# expr-start 2651 '<Cast to BasicDecl at nodes.lkt:3629:72>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
--# expr-start 2650 '<parent at nodes.lkt:3629:72>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 2650
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Basic_Decl
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2651
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2653 '<FieldAccess for node_env at nodes.lkt:3629:142>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
--# expr-start 2652 '<parent at nodes.lkt:3629:142>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3629'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 2652
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Fld_4);
--# expr-done 2653
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Cast_Result, Self_Env => Fld_5);
--# end
--# expr-done 2654
   If_Result := Fld_6;
else
--# expr-start 2656 '<Cast to BasicDecl at nodes.lkt:3632:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
--# expr-start 2655 '<env_node at nodes.lkt:3632:13>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
Env_Node := AST_Envs.Env_Node (Self_Env); 
--# expr-done 2655
   if Env_Node = null
      or else Env_Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Env_Node;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2656
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result;
else
   --# expr-start 2660 '<Cast to BasicDecl at nodes.lkt:3632:49>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
--# expr-start 2658 '<Cast to PrivatePart at nodes.lkt:3632:49>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
--# expr-start 2657 '<env_node at nodes.lkt:3632:49>' Env_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
Env_Node_1 := AST_Envs.Env_Node (Self_Env); 
--# expr-done 2657
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Private_Part_Range
   then
      Cast_Result_2 := Env_Node_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2658
Var_Expr_77 := Cast_Result_2;
if Var_Expr_77 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2659 '<parent at nodes.lkt:3632:49>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3632'
      if Var_Expr_77 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Var_Expr_77.Parent; 
--# expr-done 2659
Scope_Result_1 := Fld_7;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Result_Var;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 2660
   Result_Var_1 := Cast_Result_3;
end if;
Var_Expr_0 := Result_Var_1;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 2662 '<FieldAccess for env_spec_fully_qualified_name_impl at nodes.lkt:3635:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3635'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2661 '<FieldAccess for node_env at nodes.lkt:3635:68>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3635'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Node_Env (Node => Var_Expr_0);
--# expr-done 2661
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Var_Expr_0, Self_Env => Fld_8);
--# end
--# expr-done 2662
--# expr-start 2663 '<String at nodes.lkt:3635:85>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3635'
Str_1 := Create_String ("."); 
--# expr-done 2663
Concat_Result := Concat_String (Fld_9, Str_1); 
--# expr-start 2665 '<FieldAccess for image at nodes.lkt:3635:91>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3635'
--# expr-start 2664 '<FieldAccess for name_symbol at nodes.lkt:3635:91>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3635'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 2664
Fld_11 := Libadalang.Implementation.Image (Self => Fld_10);
--# expr-done 2665
Concat_Result_1 := Concat_String (Concat_Result, Fld_11); 
Scope_Result_2 := Concat_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2279;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_String;
end if;
      Inc_Ref (Result_Var_2);
   If_Result := Result_Var_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 2666
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2667
         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_424;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_424;
                     Finalizer_Scope_2279;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3642'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_425 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : String_Type;
            procedure Finalizer_Scope_425 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_425;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2669 '<FieldAccess for env_spec_fully_qualified_name_impl at nodes.lkt:3645:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3645'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2668 '<FieldAccess for children_env at nodes.lkt:3645:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3645'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2668
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Self, Self_Env => Fld);
--# end
--# expr-done 2669
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_425;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_425;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.top_level_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3650'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : String_Type;
            procedure Finalizer_Scope_426 with Inline_Always;
      Fld : String_Type;
Fld_1 : Boolean;
If_Result : String_Type;
            procedure Finalizer_Scope_426 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_426;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2672 '<If at nodes.lkt:3651:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3651'
--# expr-start 2670 '<FieldAccess for has_top_level_env_name at nodes.lkt:3651:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3651'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 2670
if Fld_1 then
   --# expr-start 2671 '<FieldAccess for env_spec_fully_qualified_name at nodes.lkt:3651:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3651'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_spec_fully_qualified_name
Fld := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 2671
   If_Result := Fld;
else
   If_Result := Empty_String;
end if;
      Inc_Ref (If_Result);
--# expr-done 2672
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_426;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_426;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Top_Level_Env_Name;
--# end
--# property-start BasicDecl.child_decl_initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3659'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Initial_Env
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Internal_Designated_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_427 with Inline_Always;
            procedure Finalizer_Scope_2280 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_427 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_427;
            procedure Finalizer_Scope_2280 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2280;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2673 '<FieldAccess for child_decl_initial_env_name at nodes.lkt:3660:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3660'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => Private_Part);
--# end
--# expr-done 2673
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 2674 '<New DesignatedEnv at nodes.lkt:3661:23>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3661'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 2674
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2280;
   Result_Var := Scope_Result;
else
   --# expr-start 2676 '<New DesignatedEnv at nodes.lkt:3663:28>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3663'
--# expr-start 2675 '<FieldAccess for default_initial_env at nodes.lkt:3664:86>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3664'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 2675
Inc_Ref (Fld_1);
New_Struct_1 := (Direct_Env => Fld_1, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 2676
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_427;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_427;
                     Finalizer_Scope_2280;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Initial_Env;
--# end
--# property-start BasicDecl.basic_decl_env_assocs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3674'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Basic_Decl_Env_Assocs
  (Node : Bare_Basic_Decl
      ; Dest_Env : Internal_Designated_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dest_env Dest_Env
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_428 with Inline_Always;
      Local_Name : Symbol_Type;
Local_Base_Assoc : Internal_Env_Assoc_Array_Access;
Local_Implicit_Neq_Assoc : Internal_Env_Assoc_Array_Access;
Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Sym : Symbol_Type;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit_1 : Internal_Env_Assoc_Array_Access;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_428 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Local_Base_Assoc);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Implicit_Neq_Assoc);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_428;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2689 '<Let at nodes.lkt:3675:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3675'
--# scope-start
--# expr-start 2677 '<FieldAccess for name_symbol at nodes.lkt:3675:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3675'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2677
Local_Name := Fld; 
--# bind name Local_Name
--# expr-start 2688 '<Let at nodes.lkt:3676:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3676'
--# scope-start
--# expr-start 2680 '<ArrayLiteral at nodes.lkt:3676:26>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3676'
--# expr-start 2679 '<New EnvAssoc at nodes.lkt:3676:27>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3676'
--# expr-start 2678 '<FieldAccess for name_symbol at nodes.lkt:3677:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3677'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2678
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Dest_Env);
New_Struct := (Dest_Env => Dest_Env, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 2679
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => New_Struct)); 
--# expr-done 2680
Local_Base_Assoc := Array_Lit; Inc_Ref (Local_Base_Assoc);
--# bind base_assoc Local_Base_Assoc
--# expr-start 2687 '<Let at nodes.lkt:3679:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
--# scope-start
--# expr-start 2686 '<If at nodes.lkt:3679:34>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
--# expr-start 2682 '<Eq at nodes.lkt:3679:37>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
--# expr-start 2681 '<Symbol "=" at nodes.lkt:3679:45>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 2681
Is_Equal := Local_Name = Sym_1; 
--# expr-done 2682
if Is_Equal then
   --# expr-start 2685 '<ArrayLiteral at nodes.lkt:3679:59>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
--# expr-start 2684 '<New EnvAssoc at nodes.lkt:3679:60>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3679'
--# expr-start 2683 '<Symbol "/=" at nodes.lkt:3680:17>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3680'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 2683
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Inc_Ref (Dest_Env);
New_Struct_1 := (Dest_Env => Dest_Env, Key => Sym, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 2684
Array_Lit_1 := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => New_Struct_1)); 
--# expr-done 2685
   If_Result := Array_Lit_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2686
Local_Implicit_Neq_Assoc := If_Result; Inc_Ref (Local_Implicit_Neq_Assoc);
--# bind implicit_neq_assoc Local_Implicit_Neq_Assoc
Concat_Result := Concat (Local_Base_Assoc, Local_Implicit_Neq_Assoc); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2687
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2688
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2689
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_428;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_428;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Basic_Decl_Env_Assocs;
--# end
--# property-start BasicDecl.child_decl_env_assocs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3693'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Env_Assocs
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_429 with Inline_Always;
            procedure Finalizer_Scope_2281 with Inline_Always;
      Local_Dest_Env : Internal_Designated_Env := No_Designated_Env;
Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_429 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Dest_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_429;
            procedure Finalizer_Scope_2281 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2281;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2694 '<Let at nodes.lkt:3694:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3694'
--# scope-start
--# expr-start 2690 '<FieldAccess for child_decl_initial_env_name at nodes.lkt:3694:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3694'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => False);
--# end
--# expr-done 2690
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind non_null_name Var_Expr_0
--# expr-start 2691 '<New DesignatedEnv at nodes.lkt:3695:32>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3695'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 2691
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2281;
   Result_Var := Scope_Result;
else
   --# expr-start 2692 '<New DesignatedEnv at nodes.lkt:3697:28>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3697'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 2692
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
Local_Dest_Env := Result_Var; Inc_Ref (Local_Dest_Env);
--# bind dest_env Local_Dest_Env
--# expr-start 2693 '<FieldAccess for basic_decl_env_assocs at nodes.lkt:3702:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3702'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.basic_decl_env_assocs
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Env_Assocs (Node => Ent.Node, Dest_Env => Local_Dest_Env, E_Info => Ent.Info);
--# end
--# expr-done 2693
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2694
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_429;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_429;
                     Finalizer_Scope_2281;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Env_Assocs;
--# end
--# property-start BasicDecl.is_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3707'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Formal
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2696 '<IsA GenericFormal at nodes.lkt:3707:29>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3707'
--# expr-start 2695 '<parent at nodes.lkt:3707:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3707'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2695
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal; 
--# expr-done 2696
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Formal;
--# end
--# property-start BasicDecl.canonical_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3738'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Canonical_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 2697 '<FieldAccess for previous_part_for_decl at nodes.lkt:3738:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3738'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2697
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 2698 '<FieldAccess for canonical_part at nodes.lkt:3739:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3739'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2698
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Canonical_Part;
--# end
--# property-start BasicDecl.all_previous_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3745'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Previous_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_434 with Inline_Always;
            procedure Finalizer_Scope_2283 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_434 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_434;
            procedure Finalizer_Scope_2283 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2283;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2699 '<FieldAccess for previous_part_for_decl at nodes.lkt:3745:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3745'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2699
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 2703 '<If at nodes.lkt:3746:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3746'
--# expr-start 2700 '<Eq at nodes.lkt:3746:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3746'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2700
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 2701 '<FieldAccess for all_previous_parts at nodes.lkt:3746:72>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3746'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2701
--# expr-start 2702 '<ArrayLiteral at nodes.lkt:3746:98>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3746'
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Var_Expr_0)); 
--# expr-done 2702
Concat_Result := Concat (Fld_1, Array_Lit); 
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 2703
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2283;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_434;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_434;
                     Finalizer_Scope_2283;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Previous_Parts;
--# end
--# property-start BasicDecl.all_next_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3752'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Next_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_435 with Inline_Always;
            procedure Finalizer_Scope_2284 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_435 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_435;
            procedure Finalizer_Scope_2284 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2284;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2704 '<FieldAccess for next_part_for_decl at nodes.lkt:3752:54>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3752'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2704
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr_0
--# expr-start 2708 '<If at nodes.lkt:3753:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3753'
--# expr-start 2705 '<Eq at nodes.lkt:3753:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3753'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2705
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 2706 '<ArrayLiteral at nodes.lkt:3753:72>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3753'
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Var_Expr_0)); 
--# expr-done 2706
--# expr-start 2707 '<FieldAccess for all_next_parts at nodes.lkt:3753:79>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3753'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2707
Concat_Result := Concat (Array_Lit, Fld_1); 
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 2708
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2284;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_435;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_435;
                     Finalizer_Scope_2284;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Next_Parts;
--# end
--# property-start BasicDecl.all_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3760'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_436 with Inline_Always;
      Local_Prevs : Internal_Entity_Basic_Decl_Array_Access;
Local_Nexts : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_436 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Prevs);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Nexts);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_436;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2713 '<Let at nodes.lkt:3761:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3761'
--# scope-start
--# expr-start 2709 '<FieldAccess for all_previous_parts at nodes.lkt:3761:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3761'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2709
Local_Prevs := Fld; Inc_Ref (Local_Prevs);
--# bind prevs Local_Prevs
--# expr-start 2712 '<Let at nodes.lkt:3762:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3762'
--# scope-start
--# expr-start 2710 '<FieldAccess for all_next_parts at nodes.lkt:3762:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3762'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2710
Local_Nexts := Fld_1; Inc_Ref (Local_Nexts);
--# bind nexts Local_Nexts
--# expr-start 2711 '<ArrayLiteral at nodes.lkt:3764:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3764'
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Ent)); 
--# expr-done 2711
Concat_Result := Concat (Local_Prevs, Array_Lit); 
Concat_Result_1 := Concat (Concat_Result, Local_Nexts); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2712
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2713
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_436;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_436;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_All_Parts;
--# end
--# property-start BasicDecl.unshed_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3771'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_437 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Is_Equal_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_437 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_437;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2722 '<If at nodes.lkt:3771:75>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3771'
--# expr-start 2714 '<Eq at nodes.lkt:3771:78>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3771'
Is_Equal_1 := Rebindings = null; 
--# expr-done 2714
if Is_Equal_1 then
   If_Result_1 := Ent;
else
   --# expr-start 2721 '<If at nodes.lkt:3771:75>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3771'
--# expr-start 2717 '<Eq at nodes.lkt:3772:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3772'
--# expr-start 2716 '<env_node at nodes.lkt:3772:10>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3772'
--# expr-start 2715 '<FieldAccess for old_env at nodes.lkt:3772:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3772'
      if Rebindings = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Rebindings.Old_Env;
Inc_Ref (Fld_3);
--# expr-done 2715
Env_Node := AST_Envs.Env_Node (Fld_3); 
--# expr-done 2716
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Env_Node = Cast_Result; 
--# expr-done 2717
if Is_Equal then
   --# expr-start 2718 '<FieldAccess for unshed_rebindings_helper at nodes.lkt:3772:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3772'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.unshed_rebindings_helper
Fld := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings_Helper (Node => Ent.Node, Rebindings => Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 2718
   If_Result := Fld;
else
   --# expr-start 2720 '<FieldAccess for unshed_rebindings at nodes.lkt:3773:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3773'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2719 '<FieldAccess for get_parent at nodes.lkt:3773:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3773'
      if Rebindings = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Rebindings.Parent;
--# expr-done 2719
--# property-call-start BasicDecl.unshed_rebindings
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Ent.Node, Rebindings => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 2720
   If_Result := Fld_2;
end if;
--# expr-done 2721
   If_Result_1 := If_Result;
end if;
--# expr-done 2722
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_437;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_437;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unshed_Rebindings;
--# end
--# property-start BasicDecl.unshed_rebindings_helper 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3778'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings_Helper
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Internal_Entity_Info;
Fld_3 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Info;
Fld_8 : Env_Rebindings;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2736 '<If at nodes.lkt:3781:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3781'
--# expr-start 2725 '<Eq at nodes.lkt:3781:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3781'
--# expr-start 2724 '<FieldAccess for rebindings at nodes.lkt:3781:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3781'
--# expr-start 2723 '<FieldAccess for info at nodes.lkt:3781:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3781'
Fld_7 := Ent.Info;
--# expr-done 2723
Fld_8 := Fld_7.Rebindings;
--# expr-done 2724
Is_Equal := Fld_8 = Rebindings; 
--# expr-done 2725
if Is_Equal then
   If_Result_1 := Ent;
else
   --# expr-start 2735 '<If at nodes.lkt:3781:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3781'
--# expr-start 2728 '<FieldAccess for has_parent_rebindings at nodes.lkt:3782:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3782'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2727 '<FieldAccess for rebindings at nodes.lkt:3782:53>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3782'
--# expr-start 2726 '<FieldAccess for info at nodes.lkt:3782:53>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3782'
Fld_4 := Ent.Info;
--# expr-done 2726
Fld_5 := Fld_4.Rebindings;
--# expr-done 2727
--# property-call-start AdaNode.has_parent_rebindings
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Parent_Rebindings (Node => Self, Base => Rebindings, Parent => Fld_5);
--# end
--# expr-done 2728
if Fld_6 then
   --# expr-start 2734 '<New BasicDecl.entity at nodes.lkt:3782:80>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3782'
--# expr-start 2733 '<New EntityInfo at nodes.lkt:3783:29>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3783'
--# expr-start 2730 '<FieldAccess for from_rebound at nodes.lkt:3784:70>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3784'
--# expr-start 2729 '<FieldAccess for info at nodes.lkt:3784:70>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3784'
Fld_2 := Ent.Info;
--# expr-done 2729
Fld_3 := Fld_2.From_Rebound;
--# expr-done 2730
--# expr-start 2732 '<FieldAccess for md at nodes.lkt:3784:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3784'
--# expr-start 2731 '<FieldAccess for info at nodes.lkt:3784:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3784'
Fld := Ent.Info;
--# expr-done 2731
Fld_1 := Fld.Md;
--# expr-done 2732
New_Struct := (From_Rebound => Fld_3, Md => Fld_1, Rebindings => Rebindings); 
--# expr-done 2733
New_Struct_1 := (Info => New_Struct, Node => Self); 
--# expr-done 2734
   If_Result := New_Struct_1;
else
   If_Result := raise Property_Error with "Incorrect rebindings";
end if;
--# expr-done 2735
   If_Result_1 := If_Result;
end if;
--# expr-done 2736
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unshed_Rebindings_Helper;
--# end
--# property-start BasicDecl.decl_private_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3789'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Decl_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Private_Part
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Private_Part;
      Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Single_Task_Decl;
Match_3 : Internal_Entity_Protected_Type_Decl;
Match_4 : Internal_Entity_Single_Protected_Decl;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Private_Part;
Fld_1 : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Internal_Entity_Single_Task_Type_Decl;
Fld_4 : Internal_Entity_Task_Def;
Fld_5 : Internal_Entity_Private_Part;
Fld_6 : Internal_Entity_Protected_Def;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Protected_Def;
Fld_9 : Internal_Entity_Private_Part;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Private_Part;
Scope_Result : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Cast_Result_2 : Internal_Entity_Single_Task_Decl;
Let_Result_2 : Internal_Entity_Private_Part;
Scope_Result_2 : Internal_Entity_Private_Part;
Cast_Result_3 : Internal_Entity_Protected_Type_Decl;
Let_Result_3 : Internal_Entity_Private_Part;
Scope_Result_3 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Single_Protected_Decl;
Let_Result_4 : Internal_Entity_Private_Part;
Scope_Result_4 : Internal_Entity_Private_Part;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Private_Part;
Scope_Result_5 : Internal_Entity_Private_Part;
Match_Result : Internal_Entity_Private_Part;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2747 '<Match at nodes.lkt:3789:52>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3789'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bpd Match_0
--# expr-start 2737 '<FieldAccess for private_part at nodes.lkt:3790:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3790'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 2737
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ttd Match_1
--# expr-start 2739 '<FieldAccess for private_part at nodes.lkt:3791:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3791'
--# expr-start 2738 '<FieldAccess for definition at nodes.lkt:3791:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3791'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Match_1.Node.Task_Type_Decl_F_Definition, Info => Match_1.Info);
--# expr-done 2738
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Fld_1.Node.Task_Def_F_Private_Part, Info => Fld_1.Info);
--# expr-done 2739
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind td Match_2
--# expr-start 2742 '<FieldAccess for private_part at nodes.lkt:3792:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3792'
--# expr-start 2741 '<FieldAccess for definition at nodes.lkt:3792:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3792'
--# expr-start 2740 '<FieldAccess for task_type at nodes.lkt:3792:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3792'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_2.Node.Single_Task_Decl_F_Task_Type, Info => Match_2.Info);
--# expr-done 2740
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Task_Def (Node => Fld_3.Node.Task_Type_Decl_F_Definition, Info => Fld_3.Info);
--# expr-done 2741
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Fld_4.Node.Task_Def_F_Private_Part, Info => Fld_4.Info);
--# expr-done 2742
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ptd Match_3
--# expr-start 2744 '<FieldAccess for private_part at nodes.lkt:3793:40>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3793'
--# expr-start 2743 '<FieldAccess for definition at nodes.lkt:3793:40>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3793'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Protected_Def (Node => Match_3.Node.Protected_Type_Decl_F_Definition, Info => Match_3.Info);
--# expr-done 2743
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Fld_6.Node.Protected_Def_F_Private_Part, Info => Fld_6.Info);
--# expr-done 2744
Let_Result_3 := Fld_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind spd Match_4
--# expr-start 2746 '<FieldAccess for private_part at nodes.lkt:3794:42>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3794'
--# expr-start 2745 '<FieldAccess for definition at nodes.lkt:3794:42>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3794'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Protected_Def (Node => Match_4.Node.Single_Protected_Decl_F_Definition, Info => Match_4.Info);
--# expr-done 2745
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Private_Part (Node => Fld_8.Node.Protected_Def_F_Private_Part, Info => Fld_8.Info);
--# expr-done 2746
Let_Result_4 := Fld_9; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Package_Renaming_Decl | Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Private_Part; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 2747
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Decl_Private_Part;
--# end
--# property-start BasicDecl.immediate_declarative_region 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3799'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Immediate_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_440 with Inline_Always;
            procedure Finalizer_Scope_2291 with Inline_Always;
            procedure Finalizer_Scope_2292 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Map_Result_1 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_440 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_440;
            procedure Finalizer_Scope_2291 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2291;
            procedure Finalizer_Scope_2292 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2292;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Immediate_Declarative_Region,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2754 '<env_group at nodes.lkt:3799:54>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3799'
--# expr-start 2753 '<Mapcat at nodes.lkt:3799:54>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3799'
   --# expr-start 2748 '<FieldAccess for all_previous_parts at nodes.lkt:3799:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3799'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 2748
--# expr-start 2749 '<ArrayLiteral at nodes.lkt:3799:83>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3799'
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Ent)); 
--# expr-done 2749
Concat_Result := Concat (Fld, Array_Lit); 
      declare
         Map_Result_1_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind part Item_0
      --# expr-start 2752 '<Map at nodes.lkt:3800:19>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3800'
   --# expr-start 2750 '<FieldAccess for declarative_parts at nodes.lkt:3800:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3800'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 2750
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Declarative_Part_Array_Access := Fld_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 2751 '<FieldAccess for children_env at nodes.lkt:3800:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3800'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Item_1.Node, E_Info => Item_1.Info);
--# expr-done 2751
         declare
            Item_To_Append : constant Lexical_Env := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2292;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2752
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2291;
         end loop;
      end;
         Map_Result_1 := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_1_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2753
Group_Env := Group (Map_Result_1, No_Metadata); 
--# expr-done 2754
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_440;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_440;
                     Finalizer_Scope_2291;
                     Finalizer_Scope_2292;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Immediate_Declarative_Region;
--# end
--# property-start BasicDecl.pragma_regions 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3805'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Pragma_Regions
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Ada_Node_List_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Ada_Node_List_Array_Access;
            procedure Finalizer_Scope_441 with Inline_Always;
            procedure Finalizer_Scope_2293 with Inline_Always;
            procedure Finalizer_Scope_2294 with Inline_Always;
            procedure Finalizer_Scope_2298 with Inline_Always;
            procedure Finalizer_Scope_2299 with Inline_Always;
      Local_Enclosing_Scope : Internal_Entity_Ada_Node_List_Array_Access;
Var_Expr_0 : Internal_Entity_Ada_Node_List;
Local_Private_Scope : Internal_Entity_Ada_Node_List_Array_Access;
Var_Expr_1 : Bare_Public_Part;
Match_0 : Bare_Base_Package_Decl;
Match_1 : Bare_Protected_Def;
Match_2 : Bare_Ada_Node;
Var_Expr_2 : Bare_Private_Part;
Local_Inner_Scope : Internal_Entity_Ada_Node_List_Array_Access;
Var_Expr_3 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Ada_Node_List;
Array_Lit : Internal_Entity_Ada_Node_List_Array_Access;
Scope_Result : Internal_Entity_Ada_Node_List_Array_Access;
Result_Var : Internal_Entity_Ada_Node_List_Array_Access;
Fld_1 : Bare_Declarative_Part;
Cast_Result_1 : Bare_Public_Part;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Private_Part;
Fld_4 : Bare_Private_Part;
Match_Prefix : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Package_Decl;
Let_Result : Bare_Private_Part;
Scope_Result_1 : Bare_Private_Part;
Cast_Result_3 : Bare_Protected_Def;
Let_Result_1 : Bare_Private_Part;
Scope_Result_2 : Bare_Private_Part;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_2 : Bare_Private_Part;
Scope_Result_3 : Bare_Private_Part;
Match_Result : Bare_Private_Part;
Fld_5 : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Array_Lit_1 : Internal_Entity_Ada_Node_List_Array_Access;
Scope_Result_4 : Internal_Entity_Ada_Node_List_Array_Access;
Result_Var_1 : Internal_Entity_Ada_Node_List_Array_Access;
Scope_Result_5 : Internal_Entity_Ada_Node_List_Array_Access;
Result_Var_2 : Internal_Entity_Ada_Node_List_Array_Access;
Fld_6 : Internal_Entity_Declarative_Part_Array_Access;
Get_Result : Internal_Entity_Declarative_Part;
Fld_7 : Internal_Entity_Ada_Node_List;
Array_Lit_2 : Internal_Entity_Ada_Node_List_Array_Access;
Scope_Result_6 : Internal_Entity_Ada_Node_List_Array_Access;
Result_Var_3 : Internal_Entity_Ada_Node_List_Array_Access;
Concat_Result : Internal_Entity_Ada_Node_List_Array_Access;
Concat_Result_1 : Internal_Entity_Ada_Node_List_Array_Access;
Let_Result_3 : Internal_Entity_Ada_Node_List_Array_Access;
Let_Result_4 : Internal_Entity_Ada_Node_List_Array_Access;
Let_Result_5 : Internal_Entity_Ada_Node_List_Array_Access;
            procedure Finalizer_Scope_441 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Enclosing_Scope);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Local_Private_Scope);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Local_Inner_Scope);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_441;
            procedure Finalizer_Scope_2293 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2293;
            procedure Finalizer_Scope_2294 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2294;
            procedure Finalizer_Scope_2298 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2298;
            procedure Finalizer_Scope_2299 is
            begin
                     Dec_Ref (Array_Lit_2);
            end Finalizer_Scope_2299;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2773 '<Let at nodes.lkt:3810:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3810'
--# scope-start
--# expr-start 2756 '<Cast to AdaNode.list at nodes.lkt:3810:31>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3810'
--# expr-start 2755 '<parent at nodes.lkt:3810:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3810'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2755
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Ada_Node_List_Range
   then
      Cast_Result := Create_Internal_Entity_Ada_Node_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Ada_Node_List;
   end if;
--# expr-done 2756
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Ada_Node_List then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 2757 '<ArrayLiteral at nodes.lkt:3810:75>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3810'
Array_Lit := Create_Internal_Entity_Ada_Node_List_Array (Internal_Internal_Entity_Ada_Node_List_Array'(1 => Var_Expr_0)); 
--# expr-done 2757
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2293;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Ada_Node_List_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Enclosing_Scope := Result_Var; Inc_Ref (Local_Enclosing_Scope);
--# bind enclosing_scope Local_Enclosing_Scope
--# expr-start 2772 '<Let at nodes.lkt:3813:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3813'
--# scope-start
--# expr-start 2759 '<Cast to PublicPart at nodes.lkt:3813:29>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3813'
--# expr-start 2758 '<FieldAccess for declarative_scope at nodes.lkt:3813:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3813'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_1 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2758
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Public_Part_Range
   then
      Cast_Result_1 := Fld_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2759
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 2763 '<Match at nodes.lkt:3814:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3814'
--# expr-start 2760 '<parent at nodes.lkt:3814:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3814'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_1.Parent; 
--# expr-done 2760
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind pkg Match_0
--# expr-start 2761 '<FieldAccess for private_part at nodes.lkt:3815:46>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3815'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Match_0.Base_Package_Decl_F_Private_Part;
--# expr-done 2761
Let_Result := Fld_3; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Protected_Def =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind ptd Match_1
--# expr-start 2762 '<FieldAccess for private_part at nodes.lkt:3816:43>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3816'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_1.Protected_Def_F_Private_Part;
--# expr-done 2762
Let_Result_1 := Fld_4; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Private_Present | Ada_Protected_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2763
Var_Expr_2 := Match_Result;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_2
--# expr-start 2766 '<ArrayLiteral at nodes.lkt:3818:26>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3818'
--# expr-start 2765 '<as_entity at nodes.lkt:3818:27>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3818'
--# expr-start 2764 '<FieldAccess for decls at nodes.lkt:3818:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3818'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Var_Expr_2.Declarative_Part_F_Decls;
--# expr-done 2764
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Fld_5); 
   If_Result := As_Entity;
end if;
--# expr-done 2765
Array_Lit_1 := Create_Internal_Entity_Ada_Node_List_Array (Internal_Internal_Entity_Ada_Node_List_Array'(1 => If_Result)); 
--# expr-done 2766
Scope_Result_4 := Array_Lit_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2298;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Internal_Entity_Ada_Node_List_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_5 := Result_Var_1;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2294;
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Internal_Entity_Ada_Node_List_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Local_Private_Scope := Result_Var_2; Inc_Ref (Local_Private_Scope);
--# bind private_scope Local_Private_Scope
--# expr-start 2771 '<Let at nodes.lkt:3821:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3821'
--# scope-start
--# expr-start 2768 '<at at nodes.lkt:3821:27>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3821'
--# expr-start 2767 '<FieldAccess for declarative_parts at nodes.lkt:3821:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3821'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2767
Get_Result := Get (Self, Fld_6, 0, True); 
--# expr-done 2768
Var_Expr_3 := Get_Result;
if Var_Expr_3 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind v3 Var_Expr_3
--# expr-start 2770 '<ArrayLiteral at nodes.lkt:3821:67>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3821'
--# expr-start 2769 '<FieldAccess for decls at nodes.lkt:3821:68>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3821'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_3.Node.Declarative_Part_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 2769
Array_Lit_2 := Create_Internal_Entity_Ada_Node_List_Array (Internal_Internal_Entity_Ada_Node_List_Array'(1 => Fld_7)); 
--# expr-done 2770
Scope_Result_6 := Array_Lit_2;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2299;
   Result_Var_3 := Scope_Result_6;
else
   Result_Var_3 := No_Internal_Entity_Ada_Node_List_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Local_Inner_Scope := Result_Var_3; Inc_Ref (Local_Inner_Scope);
--# bind inner_scope Local_Inner_Scope
Concat_Result := Concat (Local_Enclosing_Scope, Local_Private_Scope); 
Concat_Result_1 := Concat (Concat_Result, Local_Inner_Scope); 
Let_Result_3 := Concat_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2771
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 2772
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 2773
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_441;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_441;
                     Finalizer_Scope_2293;
                     Finalizer_Scope_2294;
                     Finalizer_Scope_2298;
                     Finalizer_Scope_2299;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Pragma_Regions;
--# end
--# property-start BasicDecl.get_aspect_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3828'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_2301 with Inline_Always;
      Var_Expr_78 : Internal_Entity_Aspect_Spec;
Fld : Internal_Entity_Aspect_Spec;
Fld_1 : Internal_Entity_Aspect_Assoc_List;
Fld_2 : Bare_Aspect_Assoc_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Aspect_Assoc;
Bare_Item_0 : Bare_Aspect_Assoc;
Fld_4 : Internal_Entity_Name;
Fld_5 : String_Type;
Fld_6 : String_Type;
Is_Equal : Boolean;
As_Entity : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Aspect_Assoc;
Find_Result : Internal_Entity_Aspect_Assoc;
Scope_Result : Internal_Entity_Aspect_Assoc;
Result_Var : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_2301 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2301;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2774 '<FieldAccess for get_aspect_spec at nodes.lkt:3828:63>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3828'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2774
Var_Expr_78 := Fld;
if Var_Expr_78 /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 2780 '<Find at nodes.lkt:3828:63>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3828'
   --# expr-start 2775 '<FieldAccess for aspect_assocs at nodes.lkt:3828:63>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3828'
      if Var_Expr_78.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr_78.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr_78.Info);
--# expr-done 2775
Fld_2 := Fld_1.Node;
      Find_Result := No_Entity_Aspect_Assoc;
      if Fld_2 = null then
      Find_Result := No_Entity_Aspect_Assoc;
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Aspect_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind asp Item_0
      --# expr-start 2779 '<Eq at nodes.lkt:3829:18>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3829'
--# expr-start 2777 '<FieldAccess for aspect_name at nodes.lkt:3829:18>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3829'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2776 '<FieldAccess for id at nodes.lkt:3829:34>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3829'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_0.Node.Aspect_Assoc_F_Id, Info => Item_0.Info);
--# expr-done 2776
--# property-call-start AspectAssoc.aspect_name
Fld_5 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Item_0.Node, N => Fld_4);
--# end
--# expr-done 2777
--# expr-start 2778 '<FieldAccess for image at nodes.lkt:3829:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3829'
Fld_6 := Libadalang.Implementation.Image (Self => Name);
--# expr-done 2778
Is_Equal := Equivalent (Fld_5, Fld_6); 
--# expr-done 2779
      if Is_Equal then
         Find_Result := Item_0; 
      end if;
   --# end
      Finalizer_Scope_2301;
               exit when Is_Equal;
         end loop;
      end;
      end if;
--# expr-done 2780
Scope_Result := Find_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Aspect_Assoc;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2301;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.get_aspect_spec_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3835'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Spec_Expr
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Expr;
      Var_Expr_79 : Internal_Entity_Aspect_Assoc;
Fld : Internal_Entity_Aspect_Assoc;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2781 '<FieldAccess for get_aspect_assoc at nodes.lkt:3836:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3836'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_aspect_assoc
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 2781
Var_Expr_79 := Fld;
if Var_Expr_79 /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 2782 '<FieldAccess for expr at nodes.lkt:3836:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3836'
      if Var_Expr_79.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr_79.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_79.Info);
--# expr-done 2782
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect_Spec_Expr;
--# end
--# property-start BasicDecl.library_item_pragmas 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3839'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Library_Item_Pragmas
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node_List
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node_List;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_2 : Internal_Entity_Pragma_Node_List;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_6 : Internal_Entity_Pragma_Node_List;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity;
Is_A : Boolean;
If_Result : Internal_Entity_Pragma_Node_List;
Fld_9 : Internal_Entity;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Pragma_Node_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 2798 '<If at nodes.lkt:3840:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2784 '<IsA LibraryItem at nodes.lkt:3840:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2783 '<parent at nodes.lkt:3840:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2783
Is_A_1 := Fld_9.Node /= null 
and then Fld_9.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 2784
if Is_A_1 then
   --# expr-start 2788 '<FieldAccess for pragmas at nodes.lkt:3840:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2787 '<Cast to CompilationUnit at nodes.lkt:3840:44>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2786 '<parent at nodes.lkt:3840:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2785 '<parent at nodes.lkt:3840:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2785
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2786
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 2787
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result.Info);
--# expr-done 2788
   If_Result_1 := Fld_2;
else
   --# expr-start 2797 '<If at nodes.lkt:3840:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3840'
--# expr-start 2791 '<IsA LibraryItem at nodes.lkt:3841:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2790 '<parent at nodes.lkt:3841:14>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2789 '<parent at nodes.lkt:3841:14>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2789
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# expr-done 2790
Is_A := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 2791
if Is_A then
   --# expr-start 2796 '<FieldAccess for pragmas at nodes.lkt:3841:53>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2795 '<Cast to CompilationUnit at nodes.lkt:3841:53>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2794 '<parent at nodes.lkt:3841:53>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2793 '<parent at nodes.lkt:3841:53>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
--# expr-start 2792 '<parent at nodes.lkt:3841:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2792
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 2793
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 2794
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Compilation_Unit;
   end if;
--# expr-done 2795
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result_1.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result_1.Info);
--# expr-done 2796
   If_Result := Fld_6;
else
   If_Result := No_Entity_Pragma_Node_List;
end if;
--# expr-done 2797
   If_Result_1 := If_Result;
end if;
--# expr-done 2798
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Library_Item_Pragmas;
--# end
--# property-start BasicDecl.get_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3852'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Previous_Parts_Only : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind previous_parts_only Previous_Parts_Only
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr_80 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Result_Var : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2799 '<FieldAccess for defining_name_or_raise at nodes.lkt:3853:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3853'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2799
Var_Expr_80 := Fld;
if Var_Expr_80 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2800 '<FieldAccess for get_aspect at nodes.lkt:3853:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3853'
      if Var_Expr_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Aspect (Node => Var_Expr_80.Node, Name => Name, Previous_Parts_Only => Previous_Parts_Only, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_80.Info);
--# end
--# expr-done 2800
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Aspect;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect;
--# end
--# property-start BasicDecl.has_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3864'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Previous_Parts_Only : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind previous_parts_only Previous_Parts_Only
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr_81 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2801 '<FieldAccess for defining_name_or_raise at nodes.lkt:3865:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3865'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2801
Var_Expr_81 := Fld;
if Var_Expr_81 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2802 '<FieldAccess for has_aspect at nodes.lkt:3865:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3865'
      if Var_Expr_81.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Var_Expr_81.Node, Name => Name, Previous_Parts_Only => Previous_Parts_Only, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_81.Info);
--# end
--# expr-done 2802
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Has_Aspect;
--# end
--# property-start BasicDecl.get_pragma 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3873'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Pragma
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
      Var_Expr_82 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Pragma_Node;
Scope_Result : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2803 '<FieldAccess for defining_name_or_raise at nodes.lkt:3874:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3874'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2803
Var_Expr_82 := Fld;
if Var_Expr_82 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2804 '<FieldAccess for get_pragma at nodes.lkt:3874:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3874'
      if Var_Expr_82.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_pragma
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Pragma (Node => Var_Expr_82.Node, Name => Name, E_Info => Var_Expr_82.Info);
--# end
--# expr-done 2804
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Pragma;
--# end
--# property-start BasicDecl.get_representation_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3880'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Representation_Clause
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
      Var_Expr_83 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Attribute_Def_Clause;
Scope_Result : Internal_Entity_Attribute_Def_Clause;
Result_Var : Internal_Entity_Attribute_Def_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2805 '<FieldAccess for defining_name_or_raise at nodes.lkt:3881:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3881'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2805
Var_Expr_83 := Fld;
if Var_Expr_83 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2806 '<FieldAccess for get_representation_clause at nodes.lkt:3881:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3881'
      if Var_Expr_83.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_representation_clause
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Representation_Clause (Node => Var_Expr_83.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_83.Info);
--# end
--# expr-done 2806
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Attribute_Def_Clause;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Representation_Clause;
--# end
--# property-start BasicDecl.get_at_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3886'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_At_Clause
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_At_Clause
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_At_Clause;
      Var_Expr_84 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_At_Clause;
Scope_Result : Internal_Entity_At_Clause;
Result_Var : Internal_Entity_At_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2807 '<FieldAccess for defining_name_or_raise at nodes.lkt:3887:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3887'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2807
Var_Expr_84 := Fld;
if Var_Expr_84 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2808 '<FieldAccess for get_at_clause at nodes.lkt:3887:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3887'
      if Var_Expr_84.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_at_clause
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_At_Clause (Node => Var_Expr_84.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_84.Info);
--# end
--# expr-done 2808
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_At_Clause;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_At_Clause;
--# end
--# property-start BasicDecl.get_annotations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3896'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Annotations
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_450 with Inline_Always;
            procedure Finalizer_Scope_2308 with Inline_Always;
      Var_Expr_85 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Aspect_Array_Access;
Scope_Result : Internal_Aspect_Array_Access;
Result_Var : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_450 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_450;
            procedure Finalizer_Scope_2308 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2308;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2809 '<FieldAccess for defining_name_or_raise at nodes.lkt:3897:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3897'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2809
Var_Expr_85 := Fld;
if Var_Expr_85 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2810 '<FieldAccess for get_annotations at nodes.lkt:3897:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3897'
      if Var_Expr_85.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_annotations
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Annotations (Node => Var_Expr_85.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_85.Info);
--# end
--# expr-done 2810
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2308;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Aspect_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_450;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_450;
                     Finalizer_Scope_2308;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Annotations;
--# end
--# property-start BasicDecl.is_ghost_code 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3901'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Ghost_Code
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_86 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 2811 '<FieldAccess for defining_name_or_raise at nodes.lkt:3901:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3901'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2811
Var_Expr_86 := Fld;
if Var_Expr_86 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2812 '<FieldAccess for is_ghost_code at nodes.lkt:3901:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3901'
      if Var_Expr_86.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr_86.Node, E_Info => Var_Expr_86.Info);
--# end
--# expr-done 2812
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Ghost_Code;
--# end
--# property-start BasicDecl.get_instantiation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3912'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Instantiation
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation;
            procedure Finalizer_Scope_2310 with Inline_Always;
      Local_Inst_Node : Bare_Generic_Instantiation;
Var_Expr_0 : Env_Rebindings;
Local_Designated_Decl : Internal_Entity_Generic_Decl;
Var_Expr_87 : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
Scope_Result : Bare_Generic_Instantiation;
Result_Var : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_3 : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Result_Var_1 : Internal_Entity_Generic_Decl;
New_Struct : Internal_Metadata;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Env_Rebindings;
Fld_9 : Internal_Entity_Info;
Fld_10 : Boolean;
New_Struct_1 : Internal_Entity_Info;
New_Struct_2 : Internal_Entity_Generic_Instantiation;
Fld_11 : Bare_Generic_Decl;
Fld_12 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation;
Let_Result : Internal_Entity_Generic_Instantiation;
Let_Result_1 : Internal_Entity_Generic_Instantiation;
            procedure Finalizer_Scope_2310 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2310;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2835 '<Let at nodes.lkt:3913:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3913'
--# scope-start
--# expr-start 2814 '<FieldAccess for rebindings at nodes.lkt:3913:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3913'
--# expr-start 2813 '<FieldAccess for info at nodes.lkt:3913:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3913'
Fld := Ent.Info;
--# expr-done 2813
Fld_1 := Fld.Rebindings;
--# expr-done 2814
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= null then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 2817 '<Cast to GenericInstantiation at nodes.lkt:3914:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3914'
--# expr-start 2816 '<env_node at nodes.lkt:3914:20>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3914'
--# expr-start 2815 '<FieldAccess for new_env at nodes.lkt:3914:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3914'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_0.New_Env;
Inc_Ref (Fld_2);
--# expr-done 2815
Env_Node := AST_Envs.Env_Node (Fld_2); 
--# expr-done 2816
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2817
Scope_Result := Cast_Result;
   --# end
      Finalizer_Scope_2310;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Local_Inst_Node := Result_Var; 
--# bind inst_node Local_Inst_Node
--# expr-start 2834 '<Let at nodes.lkt:3916:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3916'
--# scope-start
--# expr-start 2818 '<as_bare_entity at nodes.lkt:3916:31>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3916'
Is_Null := Local_Inst_Node = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Local_Inst_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 2818
Var_Expr_87 := If_Result;
if Var_Expr_87 /= No_Entity_Generic_Instantiation then
   --# scope-start
--# expr-start 2819 '<FieldAccess for designated_generic_decl at nodes.lkt:3916:31>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3916'
      if Var_Expr_87.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Var_Expr_87.Node, E_Info => Var_Expr_87.Info);
--# end
--# expr-done 2819
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Generic_Decl;
end if;
Local_Designated_Decl := Result_Var_1; 
--# bind designated_decl Local_Designated_Decl
--# expr-start 2833 '<If at nodes.lkt:3918:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3918'
--# expr-start 2822 '<Eq at nodes.lkt:3918:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3918'
--# expr-start 2820 '<FieldAccess for node at nodes.lkt:3918:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3918'
Fld_11 := Local_Designated_Decl.Node;
--# expr-done 2820
      Cast_Result_1 := Fld_11;
--# expr-start 2821 '<parent at nodes.lkt:3918:36>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3918'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Parent; 
--# expr-done 2821
Is_Equal := Cast_Result_1 = Fld_12; 
--# expr-done 2822
if Is_Equal then
   --# expr-start 2832 '<New GenericInstantiation.entity at nodes.lkt:3918:53>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3918'
--# expr-start 2831 '<New EntityInfo at nodes.lkt:3919:34>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3919'
--# expr-start 2824 '<FieldAccess for from_rebound at nodes.lkt:3925:33>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3925'
--# expr-start 2823 '<FieldAccess for info at nodes.lkt:3925:33>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3925'
Fld_9 := Ent.Info;
--# expr-done 2823
Fld_10 := Fld_9.From_Rebound;
--# expr-done 2824
--# expr-start 2825 '<New Metadata at nodes.lkt:3920:20>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3920'
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 2825
--# expr-start 2830 '<FieldAccess for remove_rebindings at nodes.lkt:3923:28>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3923'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2827 '<FieldAccess for rebindings at nodes.lkt:3924:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3924'
--# expr-start 2826 '<FieldAccess for info at nodes.lkt:3924:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3924'
Fld_4 := Ent.Info;
--# expr-done 2826
Fld_5 := Fld_4.Rebindings;
--# expr-done 2827
--# expr-start 2829 '<FieldAccess for rebindings at nodes.lkt:3924:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3924'
--# expr-start 2828 '<FieldAccess for info at nodes.lkt:3924:43>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3924'
Fld_6 := Local_Designated_Decl.Info;
--# expr-done 2828
Fld_7 := Fld_6.Rebindings;
--# expr-done 2829
--# property-call-start AdaNode.remove_rebindings
Fld_8 := Libadalang.Implementation.Ada_Node_P_Remove_Rebindings (Node => Self, Base => Fld_5, Suffix => Fld_7);
--# end
--# expr-done 2830
New_Struct_1 := (From_Rebound => Fld_10, Md => New_Struct, Rebindings => Fld_8); 
--# expr-done 2831
New_Struct_2 := (Info => New_Struct_1, Node => Local_Inst_Node); 
--# expr-done 2832
   If_Result_1 := New_Struct_2;
else
   If_Result_1 := No_Entity_Generic_Instantiation;
end if;
--# expr-done 2833
Let_Result := If_Result_1; 
--# end
--# expr-done 2834
Let_Result_1 := Let_Result; 
--# end
--# expr-done 2835
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2310;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Instantiation;
--# end
--# property-start BasicDecl.is_compilation_unit_root 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3932'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Compilation_Unit_Root
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_0 : Bare_Ada_Node;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Match_2 : Bare_Subunit;
Match_3 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Generic_Package_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subunit;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 2836 '<parent at nodes.lkt:3932:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3932'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 2836
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 2839 '<Match at nodes.lkt:3933:16>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3933'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind gen_pkg_decl Match_1
--# expr-start 2837 '<parent at nodes.lkt:3935:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3935'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Parent; 
--# expr-done 2837
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2838 '<IsA LibraryItem at nodes.lkt:3935:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3935'
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Library_Item_Range; 
--# expr-done 2838
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := False; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 2839
Scope_Result_5 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := False;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Compilation_Unit_Root;
--# end
--# property-start BasicDecl.populate_dependent_units 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3941'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Dependent_Units
  (Node : Bare_Basic_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_454 with Inline_Always;
      Fld : Bare_Name_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Map_Result : Bare_Compilation_Unit_Array_Access;
Fld_2 : Boolean;
If_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_454 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_454;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2844 '<If at nodes.lkt:3942:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3942'
--# expr-start 2840 '<FieldAccess for is_compilation_unit_root at nodes.lkt:3942:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3942'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2840
if Fld_2 then
   --# expr-start 2843 '<Map at nodes.lkt:3942:49>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3942'
   --# expr-start 2841 '<FieldAccess for top_level_with_package_clauses at nodes.lkt:3942:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3942'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 2841
      declare
         Map_Result_Vec : Bare_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind package_name Item_0
      --# expr-start 2842 '<FieldAccess for withed_unit_helper at nodes.lkt:3943:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3943'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.withed_unit_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Withed_Unit_Helper (Node => Self, Unit_Name => Item_0);
--# end
--# expr-done 2842
         declare
            Item_To_Append : constant Bare_Compilation_Unit := Fld_1;
         begin
            Bare_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Bare_Compilation_Unit_Array
           (Items_Count => Natural (Bare_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2843
   If_Result := Map_Result;
else
   If_Result := No_Bare_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2844
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_454;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_454;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Populate_Dependent_Units;
--# end
--# property-start BasicDecl.should_ref_generic_formals 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3948'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Should_Ref_Generic_Formals
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 2846 '<Not at nodes.lkt:3953:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3953'
--# expr-start 2845 '<FieldAccess for is_compilation_unit_root at nodes.lkt:3953:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3953'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 2845
Not_Val := not (Fld); 
--# expr-done 2846
if Not_Val then
   If_Result := True;
else
   --# expr-start 2848 '<IsA Subunit at nodes.lkt:3958:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3958'
--# expr-start 2847 '<parent at nodes.lkt:3958:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3958'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 2847
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Subunit_Range; 
--# expr-done 2848
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2849 '<IsA BaseSubpBody at nodes.lkt:3964:9>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3964'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 2849
   If_Result_1 := Is_A_1;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Should_Ref_Generic_Formals;
--# end
--# property-start BasicDecl.is_directly_reachable 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3969'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Directly_Reachable
  (Node : Bare_Basic_Decl
      ; Origin : Internal_Entity
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_456 with Inline_Always;
      Fld : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
            procedure Finalizer_Scope_456 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_456;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2856 '<Contains at nodes.lkt:3969:64>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3969'
   --# expr-start 2853 '<EnvGet at nodes.lkt:3969:64>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3969'
--# expr-start 2851 '<FieldAccess for node_env at nodes.lkt:3969:64>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3969'
--# expr-start 2850 '<FieldAccess for node at nodes.lkt:3969:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3969'
Fld := Origin.Node;
--# expr-done 2850
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Fld);
--# expr-done 2851
--# expr-start 2852 '<FieldAccess for name_symbol at nodes.lkt:3970:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3970'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 2852
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Thin (Fld_2), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 2853
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_8 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_8
      --# expr-start 2855 '<Eq at ???>' Is_Equiv None
--# expr-start 2854 '<as_bare_entity at nodes.lkt:3971:16>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3971'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2854
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Is_Equiv := Equivalent (Item_8, Cast_Result); 
--# expr-done 2855
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2856
         Property_Result := Quantifier_Result;
   --# end
      Finalizer_Scope_456;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_456;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Directly_Reachable;
--# end
--# property-start BasicDecl.is_visible 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3981'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Visible
  (Node : Bare_Basic_Decl
      ; From_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_node From_Node
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity;
Var_Expr_88 : Internal_Entity_Body_Node;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Fld_2 : Boolean;
Fld_3 : Bare_Compilation_Unit;
Fld_4 : Bare_Ada_Node;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
If_Result_1 : Boolean;
Fld_9 : Boolean;
Fld_10 : Internal_Entity;
Fld_11 : Internal_Entity;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Boolean;
Fld_12 : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_13 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Not_Val : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_14 : Boolean;
Fld_15 : Internal_Entity;
Is_A_1 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Fld_16 : Boolean;
If_Result_7 : Boolean;
Is_A_2 : Boolean;
If_Result_8 : Boolean;
Fld_17 : Internal_Entity_Info;
Fld_18 : Boolean;
If_Result_9 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2887 '<If at nodes.lkt:3984:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2858 '<FieldAccess for from_rebound at nodes.lkt:3984:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2857 '<FieldAccess for info at nodes.lkt:3984:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
Fld_17 := Ent.Info;
--# expr-done 2857
Fld_18 := Fld_17.From_Rebound;
--# expr-done 2858
if Fld_18 then
   If_Result_9 := True;
else
   --# expr-start 2886 '<If at nodes.lkt:3984:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2859 '<IsA ClasswideTypeDecl, DiscreteBaseSubtypeDecl at nodes.lkt:3987:14>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3987'
Is_A_2 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Classwide_Type_Decl_Range | Ada_Discrete_Base_Subtype_Decl_Range; 
--# expr-done 2859
if Is_A_2 then
   --# expr-start 2862 '<FieldAccess for is_visible at nodes.lkt:3987:71>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3987'
--# expr-start 2861 '<Cast to BaseTypeDecl at nodes.lkt:3987:71>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3987'
--# expr-start 2860 '<parent at nodes.lkt:3987:71>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2860
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2861
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_visible
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Cast_Result.Node, From_Node => From_Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2862
   If_Result_8 := Fld_1;
else
   --# expr-start 2885 '<If at nodes.lkt:3984:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2863 '<FieldAccess for is_in_private_part at nodes.lkt:3990:14>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3990'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2863
if Fld_16 then
--# expr-start 2864 '<FieldAccess for is_directly_reachable at nodes.lkt:3990:45>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3990'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_directly_reachable
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Directly_Reachable (Node => Self, Origin => From_Node);
--# end
--# expr-done 2864
if Fld_2 then
   --# expr-start 2867 '<FieldAccess for has_private_view at nodes.lkt:3997:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3997'
--# expr-start 2865 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:3997:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3997'
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => From_Node.Node);
--# end
--# expr-done 2865
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2866 '<FieldAccess for node at nodes.lkt:3997:69>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3997'
Fld_4 := From_Node.Node;
--# expr-done 2866
--# property-call-start CompilationUnit.has_private_view
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Has_Private_View (Node => Fld_3, Origin => Fld_4);
--# end
--# expr-done 2867
   If_Result := Fld_5;
else
   If_Result := False;
end if;
   If_Result_7 := If_Result;
else
   --# expr-start 2884 '<If at nodes.lkt:3984:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2868 '<FieldAccess for is_in_public_part at nodes.lkt:4002:14>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_public_part'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2868
if Fld_14 then
   If_Result_5 := True;
else
   --# expr-start 2870 '<IsA GenericFormalPackage at nodes.lkt:4002:42>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
--# expr-start 2869 '<parent at nodes.lkt:4002:42>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2869
Is_A_1 := Fld_15.Node /= null 
and then Fld_15.Node.Kind in Ada_Generic_Formal_Package_Range; 
--# expr-done 2870
   If_Result_5 := Is_A_1;
end if;
if If_Result_5 then
--# expr-start 2871 '<FieldAccess for is_directly_reachable at nodes.lkt:4002:83>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_directly_reachable
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Directly_Reachable (Node => Self, Origin => From_Node);
--# end
--# expr-done 2871
if Fld_6 then
   If_Result_1 := True;
else
   --# expr-start 2873 '<FieldAccess for is_visible at nodes.lkt:4002:124>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
--# expr-start 2872 '<FieldAccess for parent_basic_decl at nodes.lkt:4002:124>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_7 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2872
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_visible
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Fld_7.Node, From_Node => From_Node, E_Info => Fld_7.Info);
--# end
--# expr-done 2873
   If_Result_1 := Fld_8;
end if;
   If_Result_6 := If_Result_1;
else
   --# expr-start 2883 '<If at nodes.lkt:3984:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2874 '<FieldAccess for is_compilation_unit_root at nodes.lkt:4005:14>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4005'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 2874
if Fld_12 then
   --# expr-start 2877 '<Not at nodes.lkt:4005:50>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4005'
--# expr-start 2875 '<Cast to Body at nodes.lkt:4005:54>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4005'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 2875
Var_Expr_88 := Cast_Result_1;
if Var_Expr_88 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 2876 '<FieldAccess for is_subunit at nodes.lkt:4005:54>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4005'
      if Var_Expr_88.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_13 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Var_Expr_88.Node);
--# end
--# expr-done 2876
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Not_Val := not (Result_Var_1); 
--# expr-done 2877
   If_Result_3 := Not_Val;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 2882 '<If at nodes.lkt:3984:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:3984'
--# expr-start 2878 '<parent at nodes.lkt:4008:14>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4008'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2878
Var_Expr_0 := Fld_10;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 2880 '<IsA DeclarativePart at nodes.lkt:4008:37>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4008'
--# expr-start 2879 '<parent at nodes.lkt:4008:37>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4008'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 2879
Is_A := Fld_11.Node /= null 
and then Fld_11.Node.Kind in Ada_Declarative_Part_Range; 
--# expr-done 2880
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 2881 '<FieldAccess for is_directly_reachable at nodes.lkt:4008:72>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4008'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_directly_reachable
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Directly_Reachable (Node => Self, Origin => From_Node);
--# end
--# expr-done 2881
   If_Result_2 := Fld_9;
else
   If_Result_2 := raise Property_Error with "Only package-level declaration support visibility checks for now.";
end if;
--# expr-done 2882
   If_Result_4 := If_Result_2;
end if;
--# expr-done 2883
   If_Result_6 := If_Result_4;
end if;
--# expr-done 2884
   If_Result_7 := If_Result_6;
end if;
--# expr-done 2885
   If_Result_8 := If_Result_7;
end if;
--# expr-done 2886
   If_Result_9 := If_Result_8;
end if;
--# expr-done 2887
         Property_Result := If_Result_9;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Visible;
--# end
--# property-start BasicDecl.subp_decl_match_signature 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4013'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2888 '<FieldAccess for subp_spec_or_null at nodes.lkt:4013:69>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4013'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2888
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 2891 '<FieldAccess for match_signature at nodes.lkt:4014:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4014'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2890 '<Cast to SubpSpec at nodes.lkt:4015:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4015'
--# expr-start 2889 '<FieldAccess for subp_spec_or_null at nodes.lkt:4015:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4015'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 2889
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2890
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result.Node,
         Info => Cast_Result.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_0.Node, Other => Cast_Result_1, Match_Name => False, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2891
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Decl_Match_Signature;
--# end
--# property-start BasicDecl.access_to_subp_decl_match_signature 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4022'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Access_To_Subp_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Subp_Spec;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2895 '<FieldAccess for subp_spec at nodes.lkt:4023:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4023'
--# expr-start 2894 '<Cast to AccessToSubpDef at nodes.lkt:4023:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4023'
--# expr-start 2893 '<FieldAccess for access_def at nodes.lkt:4023:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4023'
--# expr-start 2892 '<FieldAccess for expr_type at nodes.lkt:4023:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4023'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2892
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2893
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 2894
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Cast_Result.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Cast_Result.Info);
--# expr-done 2895
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 2898 '<FieldAccess for match_signature at nodes.lkt:4024:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4024'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2897 '<Cast to SubpSpec at nodes.lkt:4025:17>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4025'
--# expr-start 2896 '<FieldAccess for subp_spec_or_null at nodes.lkt:4025:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4025'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 2896
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2897
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_0.Node, Other => Cast_Result_2, Match_Name => False, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2898
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Access_To_Subp_Decl_Match_Signature;
--# end
--# property-start BasicDecl.subp_renaming_decl_match_signature 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4050'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Renaming_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Prefix : Internal_Entity_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity_Subp_Spec;
Cast_Result_3 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 2899 '<FieldAccess for subp_decl_match_signature at nodes.lkt:4052:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4052'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_decl_match_signature
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Decl_Match_Signature (Node => Ent.Node, Other => Other, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2899
if Fld then
   If_Result_2 := True;
else
--# expr-start 2900 '<FieldAccess for subp_spec_or_null at nodes.lkt:4055:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4055'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2900
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 2902 '<Not at nodes.lkt:4056:23>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4056'
--# expr-start 2901 '<IsNull at nodes.lkt:4056:27>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4056'
Is_Null := Prefix.Node = null; 
--# expr-done 2901
Not_Val := not (Is_Null); 
--# expr-done 2902
if Not_Val then
   --# expr-start 2907 '<Eq at nodes.lkt:4059:17>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4059'
--# expr-start 2903 '<FieldAccess for expr_type at nodes.lkt:4060:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4060'
      if Prefix.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Prefix.Node, Origin => Origin, E_Info => Prefix.Info);
--# end
--# expr-done 2903
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
--# expr-start 2904 '<FieldAccess for accessed_type at nodes.lkt:4060:52>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4060'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2904
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 2905 '<FieldAccess for expr_type at nodes.lkt:4060:77>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4060'
      if Prefix.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Prefix.Node, Origin => Origin, E_Info => Prefix.Info);
--# end
--# expr-done 2905
   Result_Var_1 := Fld_4;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
--# expr-start 2906 '<FieldAccess for dottable_subp_of at nodes.lkt:4061:22>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4061'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2906
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2907
   If_Result := Is_Equiv;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 2910 '<FieldAccess for match_signature at nodes.lkt:4065:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4065'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 2909 '<Cast to SubpSpec at nodes.lkt:4066:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4066'
--# expr-start 2908 '<FieldAccess for subp_spec_or_null at nodes.lkt:4066:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4066'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 2908
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2909
      Cast_Result_3 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_2.Node,
         Info => Cast_Result_2.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_7 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_0.Node, Other => Cast_Result_3, Match_Name => False, Ignore_First_Param => True, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2910
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Renaming_Decl_Match_Signature;
--# end
--# property-start BasicDecl.base_subp_declarations_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4075'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations_Impl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_461 with Inline_Always;
            procedure Finalizer_Scope_2327 with Inline_Always;
            procedure Finalizer_Scope_2328 with Inline_Always;
      Local_Parent : Internal_Entity_Basic_Decl;
Local_Task_Or_Protected : Boolean;
Local_Spec : Internal_Entity_Base_Subp_Spec;
Local_Prims_Env : Lexical_Env := Empty_Env;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Subp_Spec;
Var_Expr_89 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Fld_11 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_12 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Subp_Spec;
Fld_14 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_15 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_461 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Prims_Env);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_461;
            procedure Finalizer_Scope_2327 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2327;
            procedure Finalizer_Scope_2328 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2328;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2943 '<Let at nodes.lkt:4076:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4076'
--# scope-start
--# expr-start 2912 '<FieldAccess for parent_basic_decl at nodes.lkt:4076:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4076'
--# expr-start 2911 '<FieldAccess for canonical_part at nodes.lkt:4076:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4076'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2911
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2912
Local_Parent := Fld_1; 
--# bind parent Local_Parent
--# expr-start 2942 '<Let at nodes.lkt:4077:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4077'
--# scope-start
--# expr-start 2913 '<IsA ProtectedTypeDecl, TaskTypeDecl at nodes.lkt:4077:33>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4077'
Is_A := Local_Parent.Node /= null 
and then Local_Parent.Node.Kind in Ada_Protected_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 2913
Local_Task_Or_Protected := Is_A; 
--# bind task_or_protected Local_Task_Or_Protected
--# expr-start 2941 '<Let at nodes.lkt:4081:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4081'
--# scope-start
--# expr-start 2916 '<FieldAccess for subp_spec_or_null at nodes.lkt:4081:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4081'
--# expr-start 2915 '<Cast to BasicDecl at nodes.lkt:4081:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4081'
--# expr-start 2914 '<FieldAccess for without_md at nodes.lkt:4081:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4081'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_2 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2914
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2915
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2916
Local_Spec := Fld_3; 
--# bind spec Local_Spec
--# expr-start 2940 '<Let at nodes.lkt:4084:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4084'
--# scope-start
--# expr-start 2928 '<If at nodes.lkt:4087:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4087'
if Local_Task_Or_Protected then
   --# expr-start 2924 '<env_group at nodes.lkt:4087:35>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4087'
--# expr-start 2921 '<Map at nodes.lkt:4088:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
   --# expr-start 2918 '<FieldAccess for base_types at nodes.lkt:4088:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
--# expr-start 2917 '<Cast to BaseTypeDecl at nodes.lkt:4088:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
Cast_Expr := Local_Parent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2917
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Cast_Result_1.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 2918
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_1
      --# expr-start 2920 '<FieldAccess for primitives_env at nodes.lkt:4088:62>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
--# expr-start 2919 '<FieldAccess for full_view at nodes.lkt:4088:62>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 2919
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 2920
         declare
            Item_To_Append : constant Lexical_Env := Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2327;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2921
--# expr-start 2923 '<ArrayLiteral at nodes.lkt:4088:97>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
--# expr-start 2922 '<FieldAccess for children_env at nodes.lkt:4088:98>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4088'
      if Local_Parent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Local_Parent.Node, E_Info => Local_Parent.Info);
--# expr-done 2922
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_7)); 
--# expr-done 2923
Concat_Result := Concat (Map_Result, Array_Lit); 
Group_Env := Group (Concat_Result, No_Metadata); 
--# expr-done 2924
   If_Result := Group_Env;
else
--# expr-start 2925 '<FieldAccess for candidate_primitive_subp_tagged_type at nodes.lkt:4091:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4091'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Local_Spec.Node, Canonicalize => False, E_Info => Local_Spec.Info);
--# end
--# expr-done 2925
Var_Expr_0 := Fld_8;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 2927 '<FieldAccess for primitives_env at nodes.lkt:4091:85>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4091'
--# expr-start 2926 '<FieldAccess for full_view at nodes.lkt:4091:85>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4091'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 2926
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 2927
Scope_Result := Fld_10;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2328;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
   If_Result := Result_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 2928
Local_Prims_Env := If_Result; Inc_Ref (Local_Prims_Env);
--# bind prims_env Local_Prims_Env
--# expr-start 2939 '<unique at nodes.lkt:4105:9>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4105'
--# expr-start 2938 '<Map at nodes.lkt:4105:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4105'
   --# expr-start 2930 '<EnvGet at nodes.lkt:4105:9>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4105'
--# expr-start 2929 '<FieldAccess for name_symbol at nodes.lkt:4106:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4106'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2929
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Local_Prims_Env, Key => Thin (Fld_11), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2930
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_2
--# expr-start 2931 '<Cast to BasicDecl at nodes.lkt:4108:64>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4108'
Cast_Expr_2 := Item_2; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2931
Var_Expr_89 := Cast_Result_3;
if Var_Expr_89 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2932 '<FieldAccess for subp_spec_or_null at nodes.lkt:4108:64>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4108'
      if Var_Expr_89.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_89.Node, E_Info => Var_Expr_89.Info);
--# end
--# expr-done 2932
Var_Expr_3 := Fld_13;
if Var_Expr_3 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr_3
--# expr-start 2935 '<bind at nodes.lkt:4110:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4110'
--# expr-start 2933 '<FieldAccess for origin_node at nodes.lkt:4110:35>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4110'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_14 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Var_Expr_3.Node);
--# end
--# expr-done 2933
Bound_Origin := Fld_14; 
--# expr-start 2934 '<FieldAccess for match_signature at nodes.lkt:4117:21>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4117'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_15 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_3.Node, Other => Local_Spec, Match_Name => False, Use_Entity_Info => True, Ignore_First_Param => Local_Task_Or_Protected, Origin => Bound_Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 2934
Dyn_Var_Bind_Result := Fld_15; 
--# expr-done 2935
Scope_Result_1 := Dyn_Var_Bind_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
      --# expr-start 2937 '<FieldAccess for canonical_part at nodes.lkt:4108:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4108'
--# expr-start 2936 '<Cast to BasicDecl at nodes.lkt:4108:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4108'
Cast_Expr_1 := Item_2; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2936
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Cast_Result_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 2937
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Fld_12;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2938
Unique_Array := Make_Unique (Map_Result_1); 
--# expr-done 2939
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 2940
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2941
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2942
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2943
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_461;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_461;
                     Finalizer_Scope_2327;
                     Finalizer_Scope_2328;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Base_Subp_Declarations_Impl;
--# end
--# property-start BasicDecl.base_subp_declarations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4134'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_462 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_462 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_462;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Base_Subp_Declarations,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 2946 '<If at nodes.lkt:4135:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4135'
--# expr-start 2944 '<FieldAccess for is_subprogram at nodes.lkt:4135:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4135'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 2944
if Fld_1 then
   --# expr-start 2945 '<FieldAccess for base_subp_declarations_impl at nodes.lkt:4135:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4135'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations_Impl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2945
   If_Result := Fld;
else
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2946
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_462;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_462;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                        As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Basic_Decl_P_Base_Subp_Declarations;
--# end
--# property-start BasicDecl.root_subp_declarations_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4140'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations_Impl
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_463 with Inline_Always;
      Local_Raw_Base_Decls : Internal_Entity_Basic_Decl_Array_Access;
Local_Parent : Internal_Entity_Basic_Decl;
Local_Task_Or_Protected : Boolean;
Local_Base_Decls : Internal_Entity_Basic_Decl_Array_Access;
Local_Base_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Basic_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Entity_Info;
Fld_5 : Internal_Metadata;
Fld_6 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Quantifier_Result : Boolean;
Not_Val_2 : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Bare_Ada_Node;
Fld_11 : Bare_Base_Type_Decl;
Cast_Result_4 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_463 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Raw_Base_Decls);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Local_Base_Decls);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Local_Base_Types);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_463;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 2981 '<Let at nodes.lkt:4145:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4145'
--# scope-start
--# expr-start 2947 '<FieldAccess for base_subp_declarations at nodes.lkt:4145:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4145'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2947
Local_Raw_Base_Decls := Fld; Inc_Ref (Local_Raw_Base_Decls);
--# bind raw_base_decls Local_Raw_Base_Decls
--# expr-start 2980 '<Let at nodes.lkt:4153:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4153'
--# scope-start
--# expr-start 2949 '<FieldAccess for parent_basic_decl at nodes.lkt:4153:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4153'
--# expr-start 2948 '<FieldAccess for canonical_part at nodes.lkt:4153:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4153'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2948
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2949
Local_Parent := Fld_2; 
--# bind parent Local_Parent
--# expr-start 2979 '<Let at nodes.lkt:4154:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4154'
--# scope-start
--# expr-start 2950 '<IsA ProtectedTypeDecl, TaskTypeDecl at nodes.lkt:4154:33>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4154'
Is_A := Local_Parent.Node /= null 
and then Local_Parent.Node.Kind in Ada_Protected_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 2950
Local_Task_Or_Protected := Is_A; 
--# bind task_or_protected Local_Task_Or_Protected
--# expr-start 2978 '<Let at nodes.lkt:4155:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
--# scope-start
--# expr-start 2956 '<If at nodes.lkt:4155:26>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
if Local_Task_Or_Protected then
   --# expr-start 2951 '<length at nodes.lkt:4155:51>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
Len := Length (Local_Raw_Base_Decls); 
--# expr-done 2951
Comp_Result := Len > 1; 
   If_Result := Comp_Result;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 2955 '<Map at nodes.lkt:4155:84>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Raw_Base_Decls;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_0
         --# expr-start 2954 '<Not at nodes.lkt:4155:114>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
--# expr-start 2953 '<Eq at nodes.lkt:4155:114>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
--# expr-start 2952 '<FieldAccess for node at nodes.lkt:4155:114>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4155'
Fld_3 := Item_0.Node;
--# expr-done 2952
      Cast_Result := Fld_3;
Is_Equal := Cast_Result = Self; 
--# expr-done 2953
Not_Val := not (Is_Equal); 
--# expr-done 2954
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_0;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2955
   If_Result_1 := Map_Result;
else
   If_Result_1 := Local_Raw_Base_Decls;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2956
Local_Base_Decls := If_Result_1; Inc_Ref (Local_Base_Decls);
--# bind base_decls Local_Base_Decls
--# expr-start 2977 '<Let at nodes.lkt:4157:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4157'
--# scope-start
--# expr-start 2963 '<unique at nodes.lkt:4157:26>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4157'
--# expr-start 2962 '<Map at nodes.lkt:4157:26>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4157'
      declare
         Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Base_Decls;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_1
      --# expr-start 2961 '<as_bare_entity at nodes.lkt:4158:20>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4158'
--# expr-start 2960 '<Cast to BaseTypeDecl at nodes.lkt:4158:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4158'
--# expr-start 2959 '<FieldAccess for primitive at nodes.lkt:4158:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4158'
--# expr-start 2958 '<FieldAccess for md at nodes.lkt:4158:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4158'
--# expr-start 2957 '<FieldAccess for info at nodes.lkt:4158:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4158'
Fld_4 := Item_1.Info;
--# expr-done 2957
Fld_5 := Fld_4.Md;
--# expr-done 2958
Fld_6 := Fld_5.Primitive;
--# expr-done 2959
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_6;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2960
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result_2 := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 2961
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := If_Result_2;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 2962
Unique_Array := Make_Unique (Map_Result_1); 
--# expr-done 2963
Local_Base_Types := Unique_Array; Inc_Ref (Local_Base_Types);
--# bind base_types Local_Base_Types
--# expr-start 2976 '<Map at nodes.lkt:4164:9>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4164'
   --# expr-start 2969 '<Map at nodes.lkt:4164:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4164'
      declare
         Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Local_Base_Types;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_2
         --# expr-start 2968 '<Not at nodes.lkt:4165:20>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4165'
--# expr-start 2967 '<AnyQuantifier at nodes.lkt:4165:24>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4165'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Local_Base_Types;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_3
--# expr-start 2965 '<Not at nodes.lkt:4165:46>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4165'
--# expr-start 2964 '<Eq at nodes.lkt:4165:46>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4165'
Cast_Expr := Item_2; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_3; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 2964
Not_Val_1 := not (Is_Equiv); 
--# expr-done 2965
if Not_Val_1 then
   --# expr-start 2966 '<FieldAccess for is_derived_type at nodes.lkt:4165:57>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4165'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_2.Node, Other_Type => Item_3, Origin => Origin, E_Info => Item_2.Info);
--# end
--# expr-done 2966
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
      Quantifier_Result := If_Result_3;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 2967
Not_Val_2 := not (Quantifier_Result); 
--# expr-done 2968
         if Not_Val_2 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_2;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 2969
      declare
         Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind root_type Item_4
      --# expr-start 2975 '<Find at nodes.lkt:4168:28>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4168'
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Base_Decls;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_5
      --# expr-start 2974 '<Eq at nodes.lkt:4169:24>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4169'
--# expr-start 2972 '<FieldAccess for primitive at nodes.lkt:4169:24>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4169'
--# expr-start 2971 '<FieldAccess for md at nodes.lkt:4169:24>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4169'
--# expr-start 2970 '<FieldAccess for info at nodes.lkt:4169:24>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4169'
Fld_8 := Item_5.Info;
--# expr-done 2970
Fld_9 := Fld_8.Md;
--# expr-done 2971
Fld_10 := Fld_9.Primitive;
--# expr-done 2972
--# expr-start 2973 '<FieldAccess for node at nodes.lkt:4169:47>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4169'
Fld_11 := Item_4.Node;
--# expr-done 2973
      Cast_Result_4 := Fld_11;
Is_Equal_1 := Fld_10 = Cast_Result_4; 
--# expr-done 2974
      if Is_Equal_1 then
         Find_Result := Item_5; 
      end if;
   --# end
               exit when Is_Equal_1;
         end loop;
      end;
--# expr-done 2975
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Find_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 2976
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 2977
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2978
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2979
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 2980
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 2981
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_463;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_463;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Root_Subp_Declarations_Impl;
--# end
--# property-start BasicDecl.root_subp_declarations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4187'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_464 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_464 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_464;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 2984 '<If at nodes.lkt:4188:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4188'
--# expr-start 2982 '<FieldAccess for is_subprogram at nodes.lkt:4188:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4188'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 2982
if Fld_1 then
   --# expr-start 2983 '<FieldAccess for root_subp_declarations_impl at nodes.lkt:4188:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4188'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.root_subp_declarations_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations_Impl (Node => Ent.Node, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2983
   If_Result := Fld;
else
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 2984
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_464;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_464;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Root_Subp_Declarations;
--# end
--# property-start BasicDecl.find_all_overrides 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4195'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Find_All_Overrides
  (Node : Bare_Basic_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_465 with Inline_Always;
            procedure Finalizer_Scope_2339 with Inline_Always;
            procedure Finalizer_Scope_2340 with Inline_Always;
      Local_Spec : Internal_Entity_Base_Subp_Spec;
Local_Prim_Type : Internal_Entity_Base_Type_Decl;
Local_Derivations : Internal_Entity_Type_Decl_Array_Access;
Local_Prims : Internal_Entity_Basic_Decl_Array_Access;
Local_Base_P : Internal_Entity_Basic_Decl;
Var_Expr_90 : Internal_Entity_Base_Subp_Spec;
Var_Expr_91 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
Fld_9 : Bare_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_10 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Fld_13 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_14 : Internal_Entity_Basic_Decl;
Fld_15 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_465 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Derivations);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_465;
            procedure Finalizer_Scope_2339 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2339;
            procedure Finalizer_Scope_2340 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Prims);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_2340;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3017 '<Let at nodes.lkt:4196:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4196'
--# scope-start
--# expr-start 2985 '<FieldAccess for subp_spec_or_null at nodes.lkt:4196:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4196'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2985
Local_Spec := Fld; 
--# bind spec Local_Spec
--# expr-start 3016 '<Let at nodes.lkt:4199:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4199'
--# scope-start
Var_Expr_90 := Local_Spec;
if Var_Expr_90 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 2986 '<FieldAccess for candidate_primitive_subp_tagged_type at nodes.lkt:4199:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4199'
      if Var_Expr_90.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Tagged_Type (Node => Var_Expr_90.Node, E_Info => Var_Expr_90.Info);
--# end
--# expr-done 2986
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Local_Prim_Type := Result_Var; 
--# bind prim_type Local_Prim_Type
--# expr-start 3015 '<Let at nodes.lkt:4200:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4200'
--# scope-start
Var_Expr_91 := Local_Prim_Type;
if Var_Expr_91 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2987 '<FieldAccess for find_all_derived_types at nodes.lkt:4200:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4200'
      if Var_Expr_91.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_all_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_All_Derived_Types (Node => Var_Expr_91.Node, Units => Units, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_91.Info);
--# end
--# expr-done 2987
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2339;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Derivations := Result_Var_1; Inc_Ref (Local_Derivations);
--# bind derivations Local_Derivations
--# expr-start 3014 '<unique at nodes.lkt:4202:9>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4202'
--# expr-start 3013 '<Mapcat at nodes.lkt:4202:9>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4202'
      declare
         Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Local_Derivations;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 3012 '<Let at nodes.lkt:4205:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
--# scope-start
--# expr-start 2992 '<Map at nodes.lkt:4205:29>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
   --# expr-start 2990 '<EnvGet at nodes.lkt:4205:29>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
--# expr-start 2988 '<FieldAccess for primitives_env at nodes.lkt:4205:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 2988
--# expr-start 2989 '<FieldAccess for name_symbol at nodes.lkt:4205:52>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2989
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Thin (Fld_4), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 2990
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 2991 '<Cast to BasicDecl at nodes.lkt:4205:83>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4205'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 2991
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 2992
Local_Prims := Map_Result; Inc_Ref (Local_Prims);
--# bind prims Local_Prims
--# expr-start 3011 '<Let at nodes.lkt:4210:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4210'
--# scope-start
--# expr-start 2995 '<Find at nodes.lkt:4210:34>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4210'
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Prims;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_2
      --# expr-start 2994 '<Eq at nodes.lkt:4210:52>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4210'
--# expr-start 2993 '<FieldAccess for node at nodes.lkt:4210:52>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4210'
Fld_5 := Item_2.Node;
--# expr-done 2993
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 2994
      if Is_Equal then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Is_Equal;
         end loop;
      end;
--# expr-done 2995
Local_Base_P := Find_Result; 
--# bind base_p Local_Base_P
--# expr-start 3010 '<Map at nodes.lkt:4212:21>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4212'
   --# expr-start 3006 '<Map at nodes.lkt:4212:21>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4212'
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Prims;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
--# expr-start 3000 '<Eq at nodes.lkt:4218:29>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4218'
--# expr-start 2998 '<FieldAccess for primitive at nodes.lkt:4218:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4218'
--# expr-start 2997 '<FieldAccess for md at nodes.lkt:4218:29>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4218'
--# expr-start 2996 '<FieldAccess for info at nodes.lkt:4218:29>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4218'
Fld_6 := Item_3.Info;
--# expr-done 2996
Fld_7 := Fld_6.Md;
--# expr-done 2997
Fld_8 := Fld_7.Primitive;
--# expr-done 2998
--# expr-start 2999 '<FieldAccess for node at nodes.lkt:4218:52>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4218'
Fld_9 := Item_0.Node;
--# expr-done 2999
      Cast_Result_2 := Fld_9;
Is_Equal_1 := Fld_8 = Cast_Result_2; 
--# expr-done 3000
if Is_Equal_1 then
   --# expr-start 3005 '<bind at nodes.lkt:4222:33>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4222'
--# expr-start 3001 '<FieldAccess for origin_node at nodes.lkt:4222:47>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4222'
      if Local_Base_P.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_10 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Local_Base_P.Node);
--# end
--# expr-done 3001
Bound_Origin := Fld_10; 
--# expr-start 3004 '<FieldAccess for match_signature at nodes.lkt:4224:33>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4224'
--# expr-start 3002 '<FieldAccess for subp_spec_or_null at nodes.lkt:4224:33>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4224'
      if Local_Base_P.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Local_Base_P.Node, E_Info => Local_Base_P.Info);
--# end
--# expr-done 3002
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3003 '<FieldAccess for subp_spec_or_null at nodes.lkt:4225:37>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4225'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_3.Node, E_Info => Item_3.Info);
--# end
--# expr-done 3003
--# property-call-start BaseSubpSpec.match_signature
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_11.Node, Other => Fld_12, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 3004
Dyn_Var_Bind_Result := Fld_13; 
--# expr-done 3005
   If_Result := Dyn_Var_Bind_Result;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3006
      declare
         Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result_1;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_4
      --# expr-start 3009 '<Cast to BasicDecl at nodes.lkt:4230:32>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4230'
--# expr-start 3008 '<FieldAccess for without_md at nodes.lkt:4230:32>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4230'
--# expr-start 3007 '<FieldAccess for canonical_part at nodes.lkt:4230:32>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4230'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Item_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_4.Info);
--# end
--# expr-done 3007
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_15 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 3008
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 3009
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 3010
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3011
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 3012
         for Item_To_Append of
               Let_Result_1.Items
         loop
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2340;
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 3013
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 3014
Let_Result_2 := Unique_Array; Inc_Ref (Let_Result_2);
--# end
--# expr-done 3015
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 3016
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 3017
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_465;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_465;
                     Finalizer_Scope_2339;
                     Finalizer_Scope_2340;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Find_All_Overrides;
--# end
--# property-start '[dispatcher]BasicDecl.defining_names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Names
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.defining_names
                  Property_Result := Abstract_State_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_names
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_names
                  Property_Result := Component_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_names
                  Property_Result := Discriminant_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.defining_names
                  Property_Result := Generic_Formal_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_names
                  Property_Result := Param_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.defining_names
                  Property_Result := Synthetic_Formal_Param_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_names
                  Property_Result := Base_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.defining_names
                  Property_Result := Base_Type_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl | Ada_Subp_Decl | Ada_Generic_Subp_Internal .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.defining_names
                  Property_Result := Basic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.defining_names
                  Property_Result := Formal_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.defining_names
                  Property_Result := Entry_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.defining_names
                  Property_Result := Enum_Literal_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.defining_names
                  Property_Result := Synthetic_Char_Enum_Lit_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body =>
                  --# property-call-start AcceptStmtBody.defining_names
                  Property_Result := Accept_Stmt_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_names
                  Property_Result := Base_Subp_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub =>
                  --# property-call-start PackageBodyStub.defining_names
                  Property_Result := Package_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body_Stub =>
                  --# property-call-start ProtectedBodyStub.defining_names
                  Property_Result := Protected_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.defining_names
                  Property_Result := Subp_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body_Stub =>
                  --# property-call-start TaskBodyStub.defining_names
                  Property_Result := Task_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.defining_names
                  Property_Result := Entry_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_names
                  Property_Result := Package_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.defining_names
                  Property_Result := Protected_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_names
                  Property_Result := Task_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_names
                  Property_Result := Entry_Index_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.defining_names
                  Property_Result := Error_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.defining_names
                  Property_Result := Exception_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.defining_names
                  Property_Result := Exception_Handler_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_names
                  Property_Result := For_Loop_Var_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_names
                  Property_Result := Generic_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.defining_names
                  Property_Result := Generic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_names
                  Property_Result := Generic_Package_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_names
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_names
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.defining_names
                  Property_Result := Generic_Subp_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Label_Decl =>
                  --# property-call-start LabelDecl.defining_names
                  Property_Result := Label_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_names
                  Property_Result := Named_Stmt_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.defining_names
                  Property_Result := Number_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.defining_names
                  Property_Result := Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_names
                  Property_Result := Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_names
                  Property_Result := Single_Protected_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_names
                  Property_Result := Single_Task_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Object_Decl =>
                  --# property-call-start SyntheticObjectDecl.defining_names
                  Property_Result := Synthetic_Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Defining_Names;
--# end
--# property-start BasicDecl.defining_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4245'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_467 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_467 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_467;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3019 '<at at nodes.lkt:4245:49>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4245'
--# expr-start 3018 '<FieldAccess for defining_names at nodes.lkt:4245:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4245'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3018
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 3019
         Property_Result := Get_Result;
   --# end
      Finalizer_Scope_467;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_467;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Name;
--# end
--# property-start BasicDecl.defining_name_or_raise 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4249'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name_Or_Raise
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_468 with Inline_Always;
      Local_Dns : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_468 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Dns);
            end Finalizer_Scope_468;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3024 '<Let at nodes.lkt:4250:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4250'
--# scope-start
--# expr-start 3020 '<FieldAccess for defining_names at nodes.lkt:4250:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4250'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3020
Local_Dns := Fld; Inc_Ref (Local_Dns);
--# bind dns Local_Dns
--# expr-start 3023 '<If at nodes.lkt:4252:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4252'
--# expr-start 3021 '<length at nodes.lkt:4252:12>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4252'
Len := Length (Local_Dns); 
--# expr-done 3021
Comp_Result := Len > 1; 
if Comp_Result then
   If_Result := raise Precondition_Failure with "BasicDecl with multiple defining names";
else
   --# expr-start 3022 '<at at nodes.lkt:4252:129>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4252'
Get_Result := Get (Self, Local_Dns, 0, True); 
--# expr-done 3022
   If_Result := Get_Result;
end if;
--# expr-done 3023
Let_Result := If_Result; 
--# end
--# expr-done 3024
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_468;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_468;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Name_Or_Raise;
--# end
--# property-start BasicDecl.identity_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4256'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Identity_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Exception_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Object_Decl;
Match_3 : Internal_Entity_For_Loop_Var_Decl;
Match_4 : Internal_Entity_Base_Formal_Param_Decl;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Exception_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Object_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_For_Loop_Var_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Formal_Param_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3030 '<Match at nodes.lkt:4256:49>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4256'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Exception_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Exception_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 3025 '<FieldAccess for exc_id_type at nodes.lkt:4257:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4257'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.exc_id_type
Fld := Libadalang.Implementation.Ada_Node_P_Exc_Id_Type (Node => Self);
--# end
--# expr-done 3025
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 3026 '<FieldAccess for task_id_type at nodes.lkt:4258:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4258'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 3026
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 3027 '<FieldAccess for task_id_type at nodes.lkt:4262:31>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4262'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 3027
Let_Result_2 := Fld_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_For_Loop_Var_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_For_Loop_Var_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 3028 '<FieldAccess for task_id_type at nodes.lkt:4266:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4266'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 3028
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
--# expr-start 3029 '<FieldAccess for task_id_type at nodes.lkt:4267:40>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4267'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 3029
Let_Result_4 := Fld_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Error_Decl | Ada_Exception_Handler | Ada_Generic_Package_Decl .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Protected_Decl | Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 3030
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Identity_Type;
--# end
--# property-start BasicDecl.is_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4272'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Array
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Integer;
Comp_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3031 '<FieldAccess for array_ndims at nodes.lkt:4272:28>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4272'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3031
Comp_Result := Fld > 0; 
         Property_Result := Comp_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Array;
--# end
--# property-start BasicDecl.subp_spec_or_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4279'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Spec_Or_Null
  (Node : Bare_Basic_Decl
      ; Follow_Generic : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_generic Follow_Generic
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Match_0 : Internal_Entity_Basic_Subp_Decl;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity_Subp_Body_Stub;
Match_3 : Internal_Entity_Generic_Subp_Decl;
Match_4 : Internal_Entity_Generic_Subp_Instantiation;
Match_5 : Internal_Entity_Generic_Subp_Renaming_Decl;
Match_6 : Internal_Entity_Basic_Decl;
Var_Expr_92 : Internal_Entity_Basic_Subp_Decl;
Var_Expr_93 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Generic_Subp_Internal;
Fld_4 : Internal_Entity_Subp_Spec;
If_Result : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Basic_Subp_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Cast_Expr : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Internal_Entity_Generic_Decl;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Result_Var_1 : Internal_Entity_Base_Subp_Spec;
Cast_Expr_1 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
If_Result_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Base_Subp_Body;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Let_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Subp_Body_Stub;
Cast_Result_6 : Internal_Entity_Base_Subp_Spec;
Let_Result_2 : Internal_Entity_Base_Subp_Spec;
Scope_Result_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_7 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_8 : Internal_Entity_Base_Subp_Spec;
Let_Result_3 : Internal_Entity_Base_Subp_Spec;
Scope_Result_5 : Internal_Entity_Base_Subp_Spec;
Cast_Result_9 : Internal_Entity_Generic_Subp_Instantiation;
Let_Result_4 : Internal_Entity_Base_Subp_Spec;
Scope_Result_6 : Internal_Entity_Base_Subp_Spec;
Cast_Result_10 : Internal_Entity_Generic_Subp_Renaming_Decl;
Let_Result_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result_7 : Internal_Entity_Base_Subp_Spec;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Subp_Spec;
Cast_Result_12 : Internal_Entity_Base_Subp_Spec;
Let_Result_6 : Internal_Entity_Base_Subp_Spec;
Scope_Result_8 : Internal_Entity_Base_Subp_Spec;
Match_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3044 '<Match at nodes.lkt:4279:80>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4279'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind subp Match_0
--# expr-start 3032 '<FieldAccess for subp_decl_spec at nodes.lkt:4280:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4280'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 3032
Let_Result := Fld; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind subp Match_1
--# expr-start 3033 '<FieldAccess for subp_spec at nodes.lkt:4281:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4281'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 3033
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind subp Match_2
--# expr-start 3034 '<FieldAccess for subp_spec at nodes.lkt:4282:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4282'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Match_2.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_2.Info);
--# expr-done 3034
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_6; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_7; 
--# bind gsp Match_3
--# expr-start 3037 '<If at nodes.lkt:4283:38>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4283'
if Follow_Generic then
   --# expr-start 3036 '<FieldAccess for subp_spec at nodes.lkt:4283:61>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4283'
--# expr-start 3035 '<FieldAccess for subp_decl at nodes.lkt:4283:61>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4283'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Generic_Subp_Internal (Node => Match_3.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Match_3.Info);
--# expr-done 3035
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 3036
   If_Result := Fld_4;
else
   If_Result := No_Entity_Subp_Spec;
end if;
--# expr-done 3037
      Cast_Result_8 := Create_Internal_Entity_Base_Subp_Spec
        (Node => If_Result.Node,
         Info => If_Result.Info);
Let_Result_3 := Cast_Result_8; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Generic_Subp_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_9; 
--# bind gsi Match_4
--# expr-start 3040 '<If at nodes.lkt:4284:47>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4284'
if Follow_Generic then
--# expr-start 3038 '<FieldAccess for designated_subp at nodes.lkt:4284:70>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4284'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld_5 := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 3038
Var_Expr_92 := Fld_5;
if Var_Expr_92 /= No_Entity_Basic_Subp_Decl then
   --# scope-start
--# expr-start 3039 '<FieldAccess for subp_spec_or_null at nodes.lkt:4284:70>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4284'
      if Var_Expr_92.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_92.Node, E_Info => Var_Expr_92.Info);
--# end
--# expr-done 3039
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
   If_Result_1 := Result_Var;
else
Cast_Expr := No_Entity_Subp_Spec; 
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result;
end if;
--# expr-done 3040
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_10; 
--# bind gsr Match_5
--# expr-start 3043 '<If at nodes.lkt:4285:46>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4285'
if Follow_Generic then
--# expr-start 3041 '<FieldAccess for resolve at nodes.lkt:4285:69>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4285'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_7 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_5.Node, E_Info => Match_5.Info);
--# end
--# expr-done 3041
Var_Expr_93 := Fld_7;
if Var_Expr_93 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 3042 '<FieldAccess for subp_spec_or_null at nodes.lkt:4285:69>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4285'
      if Var_Expr_93.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_93.Node, E_Info => Var_Expr_93.Info);
--# end
--# expr-done 3042
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Subp_Spec;
end if;
   If_Result_2 := Result_Var_1;
else
Cast_Expr_1 := No_Entity_Subp_Spec; 
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   If_Result_2 := Cast_Result_1;
end if;
--# expr-done 3043
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abstract_State_Decl .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Label_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_11; 
--# bind _ Match_6
Cast_Expr_2 := No_Entity_Subp_Spec; 
      Cast_Result_12 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Let_Result_6 := Cast_Result_12; 
--# end
Scope_Result_8 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 3044
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Subp_Spec_Or_Null;
--# end
--# property-start BasicDecl.formal_param_holder_or_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4289'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Formal_Param_Holder_Or_Null
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Match_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3048 '<Match at nodes.lkt:4289:72>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4289'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 3045 '<FieldAccess for discriminants at nodes.lkt:4290:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4290'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_0.Node.Type_Decl_F_Discriminants, Info => Match_0.Info);
--# expr-done 3045
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 3046 '<FieldAccess for params at nodes.lkt:4291:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4291'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 3046
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 3047 '<FieldAccess for subp_spec_or_null at nodes.lkt:4292:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4292'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3047
      Cast_Result_5 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3048
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Formal_Param_Holder_Or_Null;
--# end
--# property-start BasicDecl.is_subprogram 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4303'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Subprogram
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3049 '<IsA BasicSubpDecl, BaseSubpBody, SubpBodyStub, EntryDecl, GenericSubpDecl, GenericSubpInstantiation, GenericSubpRenamingDecl at nodes.lkt:4304:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4304'
Is_A := Self /= null 
and then Self.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range | Ada_Entry_Decl_Range | Ada_Generic_Subp_Decl_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Renaming_Decl_Range; 
--# expr-done 3049
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Subprogram;
--# end
--# property-start BasicDecl.is_valid_reducer_candidate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4309'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Valid_Reducer_Candidate
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_2364 with Inline_Always;
      Var_Expr_0 : Boolean;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Local_Param_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Local_Return_Type : Internal_Entity_Base_Type_Decl;
Local_Param_Modes : Internal_Entity_Mode_Array_Access;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Internal_Entity_Mode_Array_Access;
Get_Result : Internal_Entity_Mode;
Is_A : Boolean;
Get_Result_1 : Internal_Entity_Mode;
Is_A_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Is_Null : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result_1 : Boolean;
Let_Result_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
            procedure Finalizer_Scope_2364 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Local_Param_Types);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Local_Param_Modes);
            end Finalizer_Scope_2364;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3050 '<FieldAccess for is_subprogram at nodes.lkt:4311:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4311'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 3050
Var_Expr_0 := Fld;
if Var_Expr_0 /= False then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 3051 '<FieldAccess for subp_spec_or_null at nodes.lkt:4312:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4312'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3051
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 3067 '<Let at nodes.lkt:4314:21>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4314'
--# scope-start
--# expr-start 3052 '<FieldAccess for param_types at nodes.lkt:4314:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4314'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3052
Local_Param_Types := Fld_2; Inc_Ref (Local_Param_Types);
--# bind param_types Local_Param_Types
--# expr-start 3066 '<Let at nodes.lkt:4315:21>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4315'
--# scope-start
--# expr-start 3053 '<FieldAccess for return_type at nodes.lkt:4315:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4315'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3053
Local_Return_Type := Fld_3; 
--# bind return_type Local_Return_Type
--# expr-start 3055 '<Eq at nodes.lkt:4319:25>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4319'
--# expr-start 3054 '<length at nodes.lkt:4319:25>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4319'
Len := Length (Local_Param_Types); 
--# expr-done 3054
Is_Equal := Len = 2; 
--# expr-done 3055
if Is_Equal then
   --# expr-start 3065 '<If at nodes.lkt:4321:25>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4321'
--# expr-start 3056 '<IsNull at nodes.lkt:4321:28>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4321'
Is_Null := Local_Return_Type.Node = null; 
--# expr-done 3056
if Is_Null then
   --# expr-start 3062 '<Let at nodes.lkt:4324:29>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4324'
--# scope-start
--# expr-start 3057 '<FieldAccess for param_modes at nodes.lkt:4324:47>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4324'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_modes
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Modes (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3057
Local_Param_Modes := Fld_4; Inc_Ref (Local_Param_Modes);
--# bind param_modes Local_Param_Modes
--# expr-start 3059 '<IsA Mode.InOut at nodes.lkt:4326:29>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4326'
--# expr-start 3058 '<at at nodes.lkt:4326:29>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4326'
Get_Result := Get (Self, Local_Param_Modes, 0, True); 
--# expr-done 3058
Is_A := Get_Result.Node /= null 
and then Get_Result.Node.Kind in Ada_Mode_In_Out_Range; 
--# expr-done 3059
if Is_A then
   --# expr-start 3061 '<IsA Mode.In, Mode.Default at nodes.lkt:4326:63>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4326'
--# expr-start 3060 '<at at nodes.lkt:4326:63>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4326'
Get_Result_1 := Get (Self, Local_Param_Modes, 1, True); 
--# expr-done 3060
Is_A_1 := Get_Result_1.Node /= null 
and then Get_Result_1.Node.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 3061
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3062
   If_Result_1 := Let_Result;
else
   --# expr-start 3064 '<Eq at nodes.lkt:4330:29>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4330'
Cast_Expr := Local_Return_Type; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 3063 '<at at nodes.lkt:4330:44>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4330'
Get_Result_2 := Get (Self, Local_Param_Types, 0, True); 
--# expr-done 3063
      Cast_Result_1 := Create_Internal_Entity
        (Node => Get_Result_2.Node,
         Info => Get_Result_2.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3064
   If_Result_1 := Is_Equiv;
end if;
--# expr-done 3065
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result_1 := If_Result_2; 
--# end
--# expr-done 3066
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 3067
Scope_Result := Let_Result_2;
   --# end
      Finalizer_Scope_2364;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2364;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Valid_Reducer_Candidate;
--# end
--# property-start BasicDecl.is_stream_subprogram_for_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4337'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Stream_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Return_Obj : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind return_obj Return_Obj
   Property_Result : Boolean;
            procedure Finalizer_Scope_475 with Inline_Always;
            procedure Finalizer_Scope_2365 with Inline_Always;
            procedure Finalizer_Scope_2366 with Inline_Always;
      Local_Root_Stream_Type : Internal_Entity_Base_Type_Decl;
Local_Params : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_94 : Bare_Basic_Decl;
Var_Expr_95 : Internal_Entity_Base_Subp_Spec;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_6 : Boolean;
Get_Result : Internal_Entity_Defining_Name;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_475 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Params);
            end Finalizer_Scope_475;
            procedure Finalizer_Scope_2365 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2365;
            procedure Finalizer_Scope_2366 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2366;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3096 '<Let at nodes.lkt:4338:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
--# scope-start
--# expr-start 3077 '<Cast to BaseTypeDecl at nodes.lkt:4338:32>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
--# expr-start 3076 '<FieldAccess for classwide_type at nodes.lkt:4338:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
--# expr-start 3075 '<Cast to BaseTypeDecl at nodes.lkt:4338:32>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
--# expr-start 3071 '<FieldAccess for get_unit_root_decl at nodes.lkt:4338:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3070 '<ArrayLiteral at nodes.lkt:4339:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4339'
--# expr-start 3068 '<Symbol Ada at nodes.lkt:4339:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4339'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 3068
--# expr-start 3069 '<Symbol Streams at nodes.lkt:4339:22>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4339'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Streams); 
--# expr-done 3069
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 3070
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 3071
Var_Expr_94 := Fld;
if Var_Expr_94 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3074 '<EnvGet at nodes.lkt:4338:32>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
--# expr-start 3072 '<FieldAccess for children_env at nodes.lkt:4338:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4338'
      if Var_Expr_94 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_94);
--# expr-done 3072
--# expr-start 3073 '<Symbol Root_Stream_Type at nodes.lkt:4341:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4341'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Stream_Type); 
--# expr-done 3073
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 3074
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_2365;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3075
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 3076
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 3077
Local_Root_Stream_Type := Cast_Result_1; 
--# bind root_stream_type Local_Root_Stream_Type
--# expr-start 3095 '<Let at nodes.lkt:4343:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4343'
--# scope-start
--# expr-start 3078 '<FieldAccess for subp_spec_or_null at nodes.lkt:4343:22>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4343'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3078
Var_Expr_95 := Fld_3;
if Var_Expr_95 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3079 '<FieldAccess for unpacked_formal_params at nodes.lkt:4343:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4343'
      if Var_Expr_95.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_95.Node, E_Info => Var_Expr_95.Info);
--# end
--# expr-done 3079
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2366;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Params := Result_Var_1; Inc_Ref (Local_Params);
--# bind params Local_Params
--# expr-start 3094 '<bind at nodes.lkt:4345:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4345'
--# expr-start 3080 '<FieldAccess for origin_node at nodes.lkt:4345:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4345'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3080
Bound_Origin := Fld_5; 
--# expr-start 3081 '<FieldAccess for is_subprogram at nodes.lkt:4347:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4347'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 3081
if Fld_6 then
   --# expr-start 3085 '<FieldAccess for is_access_to at nodes.lkt:4348:13>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4348'
--# expr-start 3084 '<FieldAccess for formal_type at nodes.lkt:4348:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4348'
--# expr-start 3083 '<FieldAccess for formal_decl at nodes.lkt:4348:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4348'
--# expr-start 3082 '<at at nodes.lkt:4348:13>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4348'
Get_Result := Get (Self, Local_Params, 0, True); 
--# expr-done 3082
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 3083
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 3084
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_access_to
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Access_To (Node => Fld_8.Node, Typ => Local_Root_Stream_Type, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 3085
   If_Result := Fld_9;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3093 '<If at nodes.lkt:4350:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4350'
if Return_Obj then
   --# expr-start 3088 '<FieldAccess for matching_formal_type at nodes.lkt:4351:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4351'
--# expr-start 3087 '<FieldAccess for return_type at nodes.lkt:4351:17>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4351'
--# expr-start 3086 '<FieldAccess for subp_spec_or_null at nodes.lkt:4351:17>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4351'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3086
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_10.Node, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 3087
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_11.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 3088
   If_Result_1 := Fld_12;
else
   --# expr-start 3092 '<FieldAccess for matching_formal_type at nodes.lkt:4353:17>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4353'
--# expr-start 3091 '<FieldAccess for formal_type at nodes.lkt:4353:17>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4353'
--# expr-start 3090 '<FieldAccess for formal_decl at nodes.lkt:4353:17>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4353'
--# expr-start 3089 '<at at nodes.lkt:4353:17>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4353'
Get_Result_1 := Get (Self, Local_Params, 1, True); 
--# expr-done 3089
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_13 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 3090
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_14 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_13.Node, Origin => Bound_Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 3091
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_14.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 3092
   If_Result_1 := Fld_15;
end if;
--# expr-done 3093
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 3094
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3095
Let_Result_1 := Let_Result; 
--# end
--# expr-done 3096
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_475;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_475;
                     Finalizer_Scope_2365;
                     Finalizer_Scope_2366;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Stream_Subprogram_For_Type;
--# end
--# property-start BasicDecl.is_put_image_subprogram_for_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4359'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Put_Image_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   Property_Result : Boolean;
            procedure Finalizer_Scope_476 with Inline_Always;
            procedure Finalizer_Scope_2367 with Inline_Always;
      Local_Root_Buffer_Type : Internal_Entity_Base_Type_Decl;
Local_Params : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_96 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_5 : Boolean;
Get_Result : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result : Boolean;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_476 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Params);
            end Finalizer_Scope_476;
            procedure Finalizer_Scope_2367 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2367;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3114 '<Let at nodes.lkt:4360:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4360'
--# scope-start
--# expr-start 3099 '<Cast to BaseTypeDecl at nodes.lkt:4360:32>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4360'
--# expr-start 3098 '<FieldAccess for classwide_type at nodes.lkt:4360:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4360'
--# expr-start 3097 '<FieldAccess for root_buffer_type at nodes.lkt:4360:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4360'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_buffer_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Buffer_Type (Node => Self);
--# end
--# expr-done 3097
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3098
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 3099
Local_Root_Buffer_Type := Cast_Result; 
--# bind root_buffer_type Local_Root_Buffer_Type
--# expr-start 3113 '<Let at nodes.lkt:4361:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4361'
--# scope-start
--# expr-start 3100 '<FieldAccess for subp_spec_or_null at nodes.lkt:4361:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4361'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3100
Var_Expr_96 := Fld_2;
if Var_Expr_96 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3101 '<FieldAccess for unpacked_formal_params at nodes.lkt:4361:22>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4361'
      if Var_Expr_96.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_96.Node, E_Info => Var_Expr_96.Info);
--# end
--# expr-done 3101
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2367;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Params := Result_Var; Inc_Ref (Local_Params);
--# bind params Local_Params
--# expr-start 3112 '<bind at nodes.lkt:4364:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4364'
--# expr-start 3102 '<FieldAccess for origin_node at nodes.lkt:4364:27>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4364'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_4 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3102
Bound_Origin := Fld_4; 
--# expr-start 3103 '<FieldAccess for is_subprogram at nodes.lkt:4367:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4367'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 3103
if Fld_5 then
   --# expr-start 3107 '<FieldAccess for matching_formal_type at nodes.lkt:4367:42>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4367'
--# expr-start 3106 '<FieldAccess for formal_type at nodes.lkt:4367:42>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4367'
--# expr-start 3105 '<FieldAccess for formal_decl at nodes.lkt:4367:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4367'
--# expr-start 3104 '<at at nodes.lkt:4367:42>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4367'
Get_Result := Get (Self, Local_Params, 0, True); 
--# expr-done 3104
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 3105
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_6.Node, Origin => Bound_Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 3106
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_7.Node, Formal_Type => Local_Root_Buffer_Type, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 3107
   If_Result := Fld_8;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3111 '<FieldAccess for matching_formal_type at nodes.lkt:4368:19>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4368'
--# expr-start 3110 '<FieldAccess for formal_type at nodes.lkt:4368:19>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4368'
--# expr-start 3109 '<FieldAccess for formal_decl at nodes.lkt:4368:19>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4368'
--# expr-start 3108 '<at at nodes.lkt:4368:19>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4368'
Get_Result_1 := Get (Self, Local_Params, 1, True); 
--# expr-done 3108
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_9 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 3109
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_9.Node, Origin => Bound_Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 3110
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_10.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 3111
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 3112
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3113
Let_Result_1 := Let_Result; 
--# end
--# expr-done 3114
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_476;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_476;
                     Finalizer_Scope_2367;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Put_Image_Subprogram_For_Type;
--# end
--# property-start BasicDecl.can_be_paramless 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4374'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Can_Be_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3115 '<FieldAccess for subp_spec_or_null at nodes.lkt:4374:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4374'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3115
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_0
--# expr-start 3119 '<FieldAccess for paramless at nodes.lkt:4375:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4375'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3118 '<FieldAccess for dottable_subp at nodes.lkt:4375:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4375'
--# expr-start 3117 '<FieldAccess for md at nodes.lkt:4375:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4375'
--# expr-start 3116 '<FieldAccess for info at nodes.lkt:4375:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4375'
Fld_1 := Ent.Info;
--# expr-done 3116
Fld_2 := Fld_1.Md;
--# expr-done 3117
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 3118
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_0.Node, Dottable_Subp => Fld_3, Can_Be => True);
--# end
--# expr-done 3119
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Can_Be_Paramless;
--# end
--# property-start BasicDecl.is_paramless 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4380'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3120 '<FieldAccess for subp_spec_or_null at nodes.lkt:4380:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4380'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3120
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_0
--# expr-start 3124 '<FieldAccess for paramless at nodes.lkt:4381:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4381'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3123 '<FieldAccess for dottable_subp at nodes.lkt:4381:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4381'
--# expr-start 3122 '<FieldAccess for md at nodes.lkt:4381:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4381'
--# expr-start 3121 '<FieldAccess for info at nodes.lkt:4381:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4381'
Fld_1 := Ent.Info;
--# expr-done 3121
Fld_2 := Fld_1.Md;
--# expr-done 3122
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 3123
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_0.Node, Dottable_Subp => Fld_3, Can_Be => False);
--# end
--# expr-done 3124
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Paramless;
--# end
--# property-start BasicDecl.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4387'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr_97 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3125 '<FieldAccess for defining_name at nodes.lkt:4387:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4387'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3125
Var_Expr_97 := Fld;
if Var_Expr_97 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 3126 '<FieldAccess for relative_name at nodes.lkt:4387:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4387'
      if Var_Expr_97.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Var_Expr_97.Node, E_Info => Var_Expr_97.Info);
--# end
--# expr-done 3126
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Relative_Name;
--# end
--# property-start BasicDecl.relative_name_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4391'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Symbol_Type;
      Var_Expr_98 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3127 '<FieldAccess for relative_name at nodes.lkt:4391:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4391'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3127
Var_Expr_98 := Fld;
if Var_Expr_98 /= No_Entity_Name then
   --# scope-start
--# expr-start 3128 '<FieldAccess for name_symbol at nodes.lkt:4391:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4391'
      if Var_Expr_98.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_98.Node);
--# end
--# expr-done 3128
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Relative_Name_Text;
--# end
--# property-start BasicDecl.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4393'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Name_Symbol
  (Node : Bare_Basic_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3131 '<FieldAccess for name_symbol at nodes.lkt:4394:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4394'
--# expr-start 3130 '<FieldAccess for relative_name at nodes.lkt:4394:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4394'
--# expr-start 3129 '<as_bare_entity at nodes.lkt:4394:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4394'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3129
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3130
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 3131
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Name_Symbol;
--# end
--# property-start BasicDecl.body_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4404'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Body_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3135 '<Cast to Body at nodes.lkt:4404:46>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4404'
--# expr-start 3132 '<FieldAccess for next_part_for_decl at nodes.lkt:4404:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4404'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3132
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr_0
--# expr-start 3134 '<Match at nodes.lkt:4405:17>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4405'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 3133 '<FieldAccess for next_part_for_decl at nodes.lkt:4406:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4406'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3133
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3134
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 3135
         Property_Result := Cast_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Body_Part_For_Decl;
--# end
--# property-start BasicDecl.canonical_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4414'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3136 '<FieldAccess for previous_part_for_name at nodes.lkt:4414:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4414'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3136
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 3137 '<FieldAccess for canonical_part_for_name at nodes.lkt:4415:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4415'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part_for_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part_For_Name (Node => Var_Expr_0.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3137
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Canonical_Part_For_Name;
--# end
--# property-start BasicDecl.most_visible_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4423'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Most_Visible_Part
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3140 '<FieldAccess for most_visible_part_for_name at nodes.lkt:4423:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4423'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3139 '<FieldAccess for name_symbol at nodes.lkt:4424:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4424'
--# expr-start 3138 '<FieldAccess for defining_name_or_raise at nodes.lkt:4424:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4424'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3138
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 3139
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Ent.Node, Sym => Fld_1, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3140
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Most_Visible_Part;
--# end
--# property-start BasicDecl.most_visible_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4430'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Most_Visible_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Only_Backwards : Boolean
         := False
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind only_backwards Only_Backwards
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Self_Is_Visible : Boolean;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Fld : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_1 : Boolean;
If_Result_2 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
Not_Val_1 : Boolean;
If_Result_4 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3152 '<Let at nodes.lkt:4438:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4438'
--# scope-start
--# expr-start 3141 '<IsNull at nodes.lkt:4439:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4439'
Is_Null := Origin = null; 
--# expr-done 3141
if Is_Null then
   If_Result := True;
else
   --# expr-start 3143 '<Not at nodes.lkt:4439:31>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4439'
--# expr-start 3142 '<FieldAccess for is_in_private_part at nodes.lkt:4439:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4439'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3142
Not_Val := not (Fld); 
--# expr-done 3143
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_2 := True;
else
   --# expr-start 3145 '<FieldAccess for is_visible at nodes.lkt:4440:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4440'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3144 '<as_bare_entity at nodes.lkt:4440:30>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4440'
Is_Null_1 := Origin = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Origin); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 3144
--# property-call-start BasicDecl.is_visible
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Ent.Node, From_Node => If_Result_1, E_Info => Ent.Info);
--# end
--# expr-done 3145
   If_Result_2 := Fld_1;
end if;
Local_Self_Is_Visible := If_Result_2; 
--# bind self_is_visible Local_Self_Is_Visible
--# expr-start 3151 '<If at nodes.lkt:4444:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4444'
--# expr-start 3146 '<Not at nodes.lkt:4444:12>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4444'
Not_Val_1 := not (Local_Self_Is_Visible); 
--# expr-done 3146
if Not_Val_1 then
--# expr-start 3147 '<FieldAccess for previous_part_for_name at nodes.lkt:4444:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4444'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3147
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 3148 '<FieldAccess for most_visible_part_for_name at nodes.lkt:4445:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4445'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Var_Expr_0.Node, Sym => Sym, Only_Backwards => True, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3148
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
   If_Result_4 := Result_Var;
else
   --# expr-start 3150 '<If at nodes.lkt:4444:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4444'
if Only_Backwards then
   If_Result_3 := Ent;
else
   --# expr-start 3149 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:4451:14>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4451'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Ent.Node, Sym => Sym, Seq => True, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3149
   If_Result_3 := Fld_4;
end if;
--# expr-done 3150
   If_Result_4 := If_Result_3;
end if;
--# expr-done 3151
Let_Result := If_Result_4; 
--# end
--# expr-done 3152
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Most_Visible_Part_For_Name;
--# end
--# property-start BasicDecl.most_visible_forward_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4460'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Most_Visible_Forward_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Seq : Boolean
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind seq Seq
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_486 with Inline_Always;
      Local_Np : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Fld_5 : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Boolean;
If_Result_4 : Boolean;
Not_Val : Boolean;
If_Result_5 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Unit;
Fld_9 : Internal_Unit;
Is_Equal : Boolean;
If_Result_6 : Boolean;
Fld_10 : Bare_Basic_Decl;
Node_Comp : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
If_Result_9 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
If_Result_10 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_486 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_486;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3180 '<Let at nodes.lkt:4461:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4461'
--# scope-start
--# expr-start 3153 '<FieldAccess for next_part_for_name at nodes.lkt:4461:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4461'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3153
Local_Np := Fld; 
--# bind np Local_Np
--# expr-start 3179 '<If at nodes.lkt:4464:9>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
--# expr-start 3154 '<IsNull at nodes.lkt:4464:12>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
Is_Null_3 := Local_Np.Node = null; 
--# expr-done 3154
if Is_Null_3 then
   If_Result_10 := Ent;
else
   --# expr-start 3178 '<If at nodes.lkt:4464:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
--# expr-start 3155 '<IsNull at nodes.lkt:4466:14>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4466'
Is_Null_2 := Origin = null; 
--# expr-done 3155
if Is_Null_2 then
   --# expr-start 3156 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:4466:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4466'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Local_Np.Node, Sym => Sym, Seq => Seq, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Np.Info);
--# end
--# expr-done 3156
   If_Result_9 := Fld_1;
else
   --# expr-start 3177 '<If at nodes.lkt:4464:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
if Seq then
   --# expr-start 3159 '<Eq at nodes.lkt:4468:23>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4468'
--# expr-start 3157 '<FieldAccess for unit at nodes.lkt:4468:23>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4468'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Unit (Node => Origin);
--# expr-done 3157
--# expr-start 3158 '<FieldAccess for unit at nodes.lkt:4468:40>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4468'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Unit (Node => Local_Np.Node);
--# expr-done 3158
Is_Equal := Fld_8 = Fld_9; 
--# expr-done 3159
   If_Result_6 := Is_Equal;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 3161 '<OrderingTest '"'"'le'"'"' at nodes.lkt:4468:55>' Node_Comp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4468'
--# expr-start 3160 '<FieldAccess for node at nodes.lkt:4468:65>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4468'
Fld_10 := Local_Np.Node;
--# expr-done 3160
Node_Comp := Compare (Self, Origin, Fld_10, Less_Or_Equal); 
--# expr-done 3161
   If_Result_7 := Node_Comp;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
   If_Result_8 := Ent;
else
   --# expr-start 3176 '<If at nodes.lkt:4464:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
--# expr-start 3164 '<Not at nodes.lkt:4471:14>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4471'
--# expr-start 3162 '<FieldAccess for is_in_private_part at nodes.lkt:4472:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4472'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Local_Np.Node, E_Info => Local_Np.Info);
--# end
--# expr-done 3162
if Fld_6 then
   If_Result_4 := True;
else
   --# expr-start 3163 '<FieldAccess for is_in_public_part at nodes.lkt:4472:40>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4472'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_public_part'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Public_Part (Node => Local_Np.Node, E_Info => Local_Np.Info);
--# end
--# expr-done 3163
   If_Result_4 := Fld_7;
end if;
Not_Val := not (If_Result_4); 
--# expr-done 3164
if Not_Val then
   --# expr-start 3171 '<If at nodes.lkt:4473:16>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4473'
--# expr-start 3169 '<Contains at nodes.lkt:4473:19>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4473'
   --# expr-start 3166 '<EnvGet at nodes.lkt:4473:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4473'
--# expr-start 3165 '<FieldAccess for node_env at nodes.lkt:4473:19>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4473'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Node_Env (Node => Origin);
--# expr-done 3165
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3166
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_9 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_9
      --# expr-start 3168 '<Eq at ???>' Is_Equiv None
--# expr-start 3167 '<as_bare_entity at nodes.lkt:4475:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4475'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3167
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Is_Equiv := Equivalent (Item_9, Cast_Result); 
--# expr-done 3168
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3169
if Quantifier_Result then
   --# expr-start 3170 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:4475:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4475'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Local_Np.Node, Sym => Sym, Seq => Seq, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Np.Info);
--# end
--# expr-done 3170
   If_Result_1 := Fld_2;
else
   If_Result_1 := Ent;
end if;
--# expr-done 3171
   If_Result_5 := If_Result_1;
else
   --# expr-start 3175 '<If at nodes.lkt:4464:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4464'
--# expr-start 3173 '<FieldAccess for is_visible at nodes.lkt:4478:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4478'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3172 '<as_bare_entity at nodes.lkt:4478:28>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4478'
Is_Null_1 := Origin = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Origin); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 3172
--# property-call-start BasicDecl.is_visible
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Local_Np.Node, From_Node => If_Result_2, E_Info => Local_Np.Info);
--# end
--# expr-done 3173
if Fld_5 then
   --# expr-start 3174 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:4478:56>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4478'
      if Local_Np.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Local_Np.Node, Sym => Sym, Seq => Seq, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Np.Info);
--# end
--# expr-done 3174
   If_Result_3 := Fld_4;
else
   If_Result_3 := Ent;
end if;
--# expr-done 3175
   If_Result_5 := If_Result_3;
end if;
--# expr-done 3176
   If_Result_8 := If_Result_5;
end if;
--# expr-done 3177
   If_Result_9 := If_Result_8;
end if;
--# expr-done 3178
   If_Result_10 := If_Result_9;
end if;
--# expr-done 3179
Let_Result := If_Result_10; 
--# end
--# expr-done 3180
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_486;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_486;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Most_Visible_Forward_Part_For_Name;
--# end
--# property-start BasicDecl.fqn_extra_suffix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fqn_Extra_Suffix
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_487 with Inline_Always;
      Str : String_Type;
Str_1 : String_Type;
Str_2 : String_Type;
Str_3 : String_Type;
Is_A : Boolean;
If_Result : String_Type;
Is_A_1 : Boolean;
If_Result_1 : String_Type;
Is_A_2 : Boolean;
If_Result_2 : String_Type;
            procedure Finalizer_Scope_487 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (Str_2);
                     Dec_Ref (Str_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_487;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3190 '<If at nodes.lkt:4486:38>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
--# expr-start 3181 '<IsA ClasswideTypeDecl at nodes.lkt:4486:41>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
Is_A_2 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Classwide_Type_Decl_Range; 
--# expr-done 3181
if Is_A_2 then
   --# expr-start 3182 '<String at nodes.lkt:4486:72>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
Str := Create_String ("'Class"); 
--# expr-done 3182
   If_Result_2 := Str;
else
   --# expr-start 3189 '<If at nodes.lkt:4486:38>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
--# expr-start 3183 '<IsA DiscreteBaseSubtypeDecl at nodes.lkt:4487:10>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4487'
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Discrete_Base_Subtype_Decl_Range; 
--# expr-done 3183
if Is_A_1 then
   --# expr-start 3184 '<String at nodes.lkt:4487:47>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4487'
Str_1 := Create_String ("'Base"); 
--# expr-done 3184
   If_Result_1 := Str_1;
else
   --# expr-start 3188 '<If at nodes.lkt:4486:38>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4486'
--# expr-start 3185 '<IsA SynthAnonymousTypeDecl at nodes.lkt:4494:10>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4494'
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Synth_Anonymous_Type_Decl_Range; 
--# expr-done 3185
if Is_A then
   --# expr-start 3186 '<String at nodes.lkt:4494:46>' Str_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4494'
Str_2 := Create_String ("'Anonymous_Access"); 
--# expr-done 3186
   If_Result := Str_2;
else
   --# expr-start 3187 '<String at nodes.lkt:4495:10>' Str_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4495'
Str_3 := Create_String (""); 
--# expr-done 3187
   If_Result := Str_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 3188
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3189
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 3190
         Property_Result := If_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_487;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_487;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fqn_Extra_Suffix;
--# end
--# property-start BasicDecl.fully_qualified_name_string_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4499'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_String_Array
  (Node : Bare_Basic_Decl
      ; Include_Profile : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   Property_Result : String_Type_Array_Access;
            procedure Finalizer_Scope_488 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type_Array_Access;
            procedure Finalizer_Scope_488 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_488;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3193 '<FieldAccess for fully_qualified_name_impl at nodes.lkt:4500:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4500'
--# expr-start 3191 '<FieldAccess for defining_name_or_raise at nodes.lkt:4500:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4500'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3191
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3192 '<FieldAccess for fqn_extra_suffix at nodes.lkt:4501:53>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4501'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3192
--# property-call-start DefiningName.fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Fld.Node, Include_Profile => Include_Profile, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 3193
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_488;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_488;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name_String_Array;
--# end
--# property-start BasicDecl.fully_qualified_name_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4507'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Array
  (Node : Bare_Basic_Decl
      ; Include_Profile : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_489 with Inline_Always;
      Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_489 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_489;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3196 '<Map at nodes.lkt:4508:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4508'
   --# expr-start 3194 '<FieldAccess for fully_qualified_name_string_array at nodes.lkt:4508:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4508'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Ent.Node, Include_Profile => Include_Profile, E_Info => Ent.Info);
--# end
--# expr-done 3194
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 3195 '<to_symbol at nodes.lkt:4508:92>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4508'
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_0); 
--# expr-done 3195
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3196
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_489;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_489;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name_Array;
--# end
--# property-start BasicDecl.fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4512'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_490 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Join_Result : String_Type;
            procedure Finalizer_Scope_490 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_490;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3199 '<Join at nodes.lkt:4513:9>' Join_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4513'
--# expr-start 3197 '<String at nodes.lkt:4513:9>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4513'
Str := Create_String ("."); 
--# expr-done 3197
--# expr-start 3198 '<FieldAccess for fully_qualified_name_string_array at nodes.lkt:4513:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4513'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3198
Join_Result := Join_Strings (Str, Fld); 
--# expr-done 3199
         Property_Result := Join_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_490;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_490;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.canonical_fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4518'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_491 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type;
            procedure Finalizer_Scope_491 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_491;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3202 '<FieldAccess for canonical_fully_qualified_name_impl at nodes.lkt:4519:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4519'
--# expr-start 3200 '<FieldAccess for defining_name_or_raise at nodes.lkt:4519:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4519'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3200
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3201 '<FieldAccess for fqn_extra_suffix at nodes.lkt:4520:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4520'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3201
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Fld.Node, Include_Profile => False, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 3202
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_491;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_491;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.unique_identifying_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4532'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unique_Identifying_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_492 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : String_Type;
Fld_2 : String_Type;
            procedure Finalizer_Scope_492 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_492;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3205 '<FieldAccess for unique_identifying_name_impl at nodes.lkt:4533:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4533'
--# expr-start 3203 '<FieldAccess for defining_name_or_raise at nodes.lkt:4533:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4533'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3203
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3204 '<FieldAccess for fqn_extra_suffix at nodes.lkt:4533:75>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4533'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fqn_extra_suffix
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Fqn_Extra_Suffix (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3204
--# property-call-start DefiningName.unique_identifying_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Unique_Identifying_Name_Impl (Node => Fld.Node, Suffix => Fld_1, E_Info => Fld.Info);
--# end
--# expr-done 3205
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_492;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_492;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Unique_Identifying_Name;
--# end
--# property-start BasicDecl.custom_id_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4535'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Custom_Id_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_493 with Inline_Always;
            procedure Finalizer_Scope_2379 with Inline_Always;
            procedure Finalizer_Scope_2380 with Inline_Always;
            procedure Finalizer_Scope_2381 with Inline_Always;
            procedure Finalizer_Scope_2382 with Inline_Always;
            procedure Finalizer_Scope_2383 with Inline_Always;
            procedure Finalizer_Scope_2384 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Type_Expr;
Var_Expr_2 : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_4 : Internal_Entity_Type_Expr;
Var_Expr_5 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Subp_Spec;
Str : String_Type;
Fld_1 : Internal_Entity_Type_Expr;
Str_1 : String_Type;
Scope_Result : String_Type;
Result_Var : String_Type;
Concat_Result : String_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Str_2 : String_Type;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Type_Expr;
Fld_5 : String_Type;
Map_Result : String_Type_Array_Access;
Join_Result : String_Type;
Scope_Result_1 : String_Type;
Result_Var_1 : String_Type;
Concat_Result_1 : String_Type;
Fld_6 : Internal_Entity_Type_Expr;
Str_3 : String_Type;
Scope_Result_2 : String_Type;
Result_Var_2 : String_Type;
Concat_Result_2 : String_Type;
Fld_7 : Internal_Entity_Type_Expr;
Str_4 : String_Type;
Fld_8 : String_Type;
Concat_Result_3 : String_Type;
Scope_Result_3 : String_Type;
Result_Var_3 : String_Type;
Concat_Result_4 : String_Type;
Str_5 : String_Type;
Concat_Result_5 : String_Type;
Scope_Result_4 : String_Type;
Str_6 : String_Type;
Result_Var_4 : String_Type;
            procedure Finalizer_Scope_493 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Str_6);
                     Dec_Ref (Result_Var_4);
            end Finalizer_Scope_493;
            procedure Finalizer_Scope_2379 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_4);
                     Dec_Ref (Str_5);
                     Dec_Ref (Concat_Result_5);
            end Finalizer_Scope_2379;
            procedure Finalizer_Scope_2380 is
            begin
                     Dec_Ref (Str_1);
            end Finalizer_Scope_2380;
            procedure Finalizer_Scope_2381 is
            begin
                     Dec_Ref (Var_Expr_2);
                     Dec_Ref (Str_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_2381;
            procedure Finalizer_Scope_2382 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2382;
            procedure Finalizer_Scope_2383 is
            begin
                     Dec_Ref (Str_3);
            end Finalizer_Scope_2383;
            procedure Finalizer_Scope_2384 is
            begin
                     Dec_Ref (Str_4);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result_3);
            end Finalizer_Scope_2384;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3206 '<FieldAccess for subp_spec_or_null at nodes.lkt:4535:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4535'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3206
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_0
--# expr-start 3207 '<String at nodes.lkt:4538:17>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4538'
Str := Create_String ("("); 
--# expr-done 3207
--# expr-start 3208 '<FieldAccess for returns at nodes.lkt:4538:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4538'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3208
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 3209 '<String at nodes.lkt:4538:46>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4538'
Str_1 := Create_String ("("); 
--# expr-done 3209
Scope_Result := Str_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2380;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_String;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat_String (Str, Result_Var); 
--# expr-start 3210 '<FieldAccess for unpacked_formal_params at nodes.lkt:4538:53>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4538'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3210
Var_Expr_2 := Fld_2;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind ufp Var_Expr_2
--# expr-start 3216 '<Join at nodes.lkt:4539:22>' Join_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4539'
--# expr-start 3211 '<String at nodes.lkt:4539:22>' Str_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4539'
Str_2 := Create_String (", "); 
--# expr-done 3211
--# expr-start 3215 '<Map at nodes.lkt:4539:32>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4539'
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Var_Expr_2;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
      --# expr-start 3214 '<FieldAccess for custom_id_text at nodes.lkt:4540:24>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4540'
--# expr-start 3213 '<FieldAccess for type_expression at nodes.lkt:4540:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4540'
--# expr-start 3212 '<FieldAccess for formal_decl at nodes.lkt:4540:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4540'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Item_3.Node, E_Info => Item_3.Info);
--# end
--# expr-done 3212
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 3213
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 3214
         declare
            Item_To_Append : constant String_Type := Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2382;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3215
Join_Result := Join_Strings (Str_2, Map_Result); 
--# expr-done 3216
Scope_Result_1 := Join_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2381;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_String;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result_1 := Concat_String (Concat_Result, Result_Var_1); 
--# expr-start 3217 '<FieldAccess for returns at nodes.lkt:4542:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3217
Var_Expr_4 := Fld_6;
if Var_Expr_4 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind _ Var_Expr_4
--# expr-start 3218 '<String at nodes.lkt:4542:36>' Str_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
Str_3 := Create_String (")"); 
--# expr-done 3218
Scope_Result_2 := Str_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2383;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_String;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result_2 := Concat_String (Concat_Result_1, Result_Var_2); 
--# expr-start 3219 '<FieldAccess for returns at nodes.lkt:4542:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3219
Var_Expr_5 := Fld_7;
if Var_Expr_5 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr_5
--# expr-start 3220 '<String at nodes.lkt:4542:66>' Str_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
Str_4 := Create_String (" -> "); 
--# expr-done 3220
--# expr-start 3221 '<FieldAccess for custom_id_text at nodes.lkt:4542:75>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 3221
Concat_Result_3 := Concat_String (Str_4, Fld_8); 
Scope_Result_3 := Concat_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2384;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_String;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_4 := Concat_String (Concat_Result_2, Result_Var_3); 
--# expr-start 3222 '<String at nodes.lkt:4542:97>' Str_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
Str_5 := Create_String (")"); 
--# expr-done 3222
Concat_Result_5 := Concat_String (Concat_Result_4, Str_5); 
Scope_Result_4 := Concat_Result_5;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2379;
   Result_Var_4 := Scope_Result_4;
else
   --# expr-start 3223 '<String at nodes.lkt:4542:114>' Str_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4542'
Str_6 := Create_String (""); 
--# expr-done 3223
   Result_Var_4 := Str_6;
end if;
      Inc_Ref (Result_Var_4);
         Property_Result := Result_Var_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_493;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_493;
                     Finalizer_Scope_2379;
                     Finalizer_Scope_2380;
                     Finalizer_Scope_2381;
                     Finalizer_Scope_2382;
                     Finalizer_Scope_2383;
                     Finalizer_Scope_2384;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Custom_Id_Text;
--# end
--# property-start BasicDecl.does_aspects_make_preelaborable 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4551'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Does_Aspects_Make_Preelaborable
  (Node : Bare_Basic_Decl
      ; From_Body : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_body From_Body
   Property_Result : Boolean;
      Bound_Imprecise_Fallback : Boolean;
Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Not_Val : Boolean;
Sym_3 : Symbol_Type;
Fld_3 : Boolean;
Sym_4 : Symbol_Type;
Fld_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3235 '<bind at nodes.lkt:4552:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4552'
Bound_Imprecise_Fallback := False; 
--# expr-start 3225 '<FieldAccess for has_aspect at nodes.lkt:4556:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4556'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3224 '<Symbol Pure at nodes.lkt:4556:29>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4556'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 3224
--# property-call-start BasicDecl.has_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3225
if Fld then
   If_Result := True;
else
   --# expr-start 3227 '<FieldAccess for has_aspect at nodes.lkt:4557:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4557'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3226 '<Symbol Preelaborate at nodes.lkt:4557:30>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4557'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 3226
--# property-call-start BasicDecl.has_aspect
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3227
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3229 '<FieldAccess for has_aspect at nodes.lkt:4557:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4557'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3228 '<Symbol Shared_Passive at nodes.lkt:4557:66>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4557'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Shared_Passive); 
--# expr-done 3228
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_2, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3229
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 3230 '<Not at nodes.lkt:4558:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4558'
Not_Val := not (From_Body); 
--# expr-done 3230
if Not_Val then
--# expr-start 3232 '<FieldAccess for has_aspect at nodes.lkt:4561:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4561'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3231 '<Symbol Remote_Types at nodes.lkt:4561:37>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4561'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remote_Types); 
--# expr-done 3231
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_3, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3232
if Fld_3 then
   If_Result_2 := True;
else
   --# expr-start 3234 '<FieldAccess for has_aspect at nodes.lkt:4562:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4562'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3233 '<Symbol Remote_Call_Interface at nodes.lkt:4562:38>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4562'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remote_Call_Interface); 
--# expr-done 3233
--# property-call-start BasicDecl.has_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_4, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3234
   If_Result_2 := Fld_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
Dyn_Var_Bind_Result := If_Result_4; 
--# expr-done 3235
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Does_Aspects_Make_Preelaborable;
--# end
--# property-start BasicDecl.wrap_public_reference 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4586'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Wrap_Public_Reference
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Generic_Instantiation;
Scope_Result : Internal_Entity_Generic_Instantiation;
Cast_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3238 '<If at nodes.lkt:4587:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4587'
--# expr-start 3236 '<IsA GenericSubpInternal, GenericPackageInternal at nodes.lkt:4587:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4587'
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 3236
if Is_A then
--# expr-start 3237 '<FieldAccess for get_instantiation at nodes.lkt:4587:70>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4587'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_instantiation
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Instantiation (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3237
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
   Result_Var := Ent;
end if;
   If_Result := Result_Var;
else
   If_Result := Ent;
end if;
--# expr-done 3238
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Wrap_Public_Reference;
--# end
--# property-start '[dispatcher]BasicDecl.child_decl_initial_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Entry_Index_Spec | Ada_Exception_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.child_decl_initial_env_name
                  Property_Result := Basic_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.child_decl_initial_env_name
                  Property_Result := Error_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_decl
                  Property_Result := Basic_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.previous_part_for_decl
                  Property_Result := Object_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.is_static_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Package_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal | Ada_Accept_Stmt_Body .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_static_decl
                  Property_Result := Basic_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.is_static_decl
                  Property_Result := Anonymous_Expr_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.is_static_decl
                  Property_Result := Discrete_Base_Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.is_static_decl
                  Property_Result := Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_static_decl
                  Property_Result := Type_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.is_static_decl
                  Property_Result := Enum_Literal_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.is_static_decl
                  Property_Result := Synthetic_Subp_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_static_decl
                  Property_Result := Number_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.is_static_decl
                  Property_Result := Object_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.corresponding_actual' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Corresponding_Actual
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Param_Spec | Ada_Generic_Package_Internal .. Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.corresponding_actual
                  Property_Result := Basic_Decl_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.corresponding_actual
                  Property_Result := Synthetic_Formal_Param_Decl_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.corresponding_actual
                  Property_Result := Formal_Subp_Decl_P_Corresponding_Actual
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Corresponding_Actual;
--# end
--# property-start '[dispatcher]BasicDecl.is_library_item' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Package_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_library_item
                  Property_Result := Basic_Decl_P_Is_Library_Item
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Internal =>
                  --# property-call-start GenericPackageInternal.is_library_item
                  Property_Result := Generic_Package_Internal_P_Is_Library_Item
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Library_Item;
--# end
--# property-start '[dispatcher]BasicDecl.declarative_parts' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Declarative_Parts
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Task_Body_Stub | Ada_Entry_Index_Spec .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.declarative_parts
                  Property_Result := Basic_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.declarative_parts
                  Property_Result := Base_Package_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.declarative_parts
                  Property_Result := Protected_Type_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.declarative_parts
                  Property_Result := Task_Type_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body =>
                  --# property-call-start SubpBody.declarative_parts
                  Property_Result := Subp_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.declarative_parts
                  Property_Result := Entry_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.declarative_parts
                  Property_Result := Package_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.declarative_parts
                  Property_Result := Protected_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.declarative_parts
                  Property_Result := Task_Body_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.declarative_parts
                  Property_Result := Generic_Package_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.declarative_parts
                  Property_Result := Single_Protected_Decl_P_Declarative_Parts
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Declarative_Parts;
--# end
--# property-start '[dispatcher]BasicDecl.get_aspect_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Get_Aspect_Spec
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Spec
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Aspect_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.get_aspect_spec
                  Property_Result := Basic_Decl_P_Get_Aspect_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.get_aspect_spec
                  Property_Result := Classwide_Type_Decl_P_Get_Aspect_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericDecl.get_aspect_spec
                  Property_Result := Generic_Decl_P_Get_Aspect_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Get_Aspect_Spec;
--# end
--# property-start '[dispatcher]BasicDecl.is_imported' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_imported
                  Property_Result := Basic_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.is_imported
                  Property_Result := Generic_Subp_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Imported;
--# end
--# property-start '[dispatcher]BasicDecl.is_in_public_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_In_Public_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_in_public_part
                  Property_Result := Basic_Decl_P_Is_In_Public_Part
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_in_public_part
                  Property_Result := Classwide_Type_Decl_P_Is_In_Public_Part
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_In_Public_Part;
--# end
--# property-start '[dispatcher]BasicDecl.is_in_private_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_in_private_part
                  Property_Result := Basic_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_in_private_part
                  Property_Result := Classwide_Type_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start '[dispatcher]BasicDecl.defining_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl | Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body | Ada_Error_Decl .. Ada_Exception_Handler | Ada_Generic_Subp_Decl | Ada_Generic_Subp_Renaming_Decl .. Ada_Label_Decl | Ada_Number_Decl | Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.defining_env
                  Property_Result := Basic_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_env
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_env
                  Property_Result := Component_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_env
                  Property_Result := Discriminant_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_env
                  Property_Result := Param_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_env
                  Property_Result := Base_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.defining_env
                  Property_Result := Base_Subtype_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.defining_env
                  Property_Result := Classwide_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.defining_env
                  Property_Result := Incomplete_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.defining_env
                  Property_Result := Protected_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.defining_env
                  Property_Result := Task_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.defining_env
                  Property_Result := Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.defining_env
                  Property_Result := Basic_Subp_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body =>
                  --# property-call-start AcceptStmtBody.defining_env
                  Property_Result := Accept_Stmt_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_env
                  Property_Result := Base_Subp_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_env
                  Property_Result := Package_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_env
                  Property_Result := Task_Body_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_env
                  Property_Result := Entry_Index_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_env
                  Property_Result := For_Loop_Var_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_env
                  Property_Result := Generic_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_env
                  Property_Result := Generic_Package_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_env
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_env
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_env
                  Property_Result := Named_Stmt_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.defining_env
                  Property_Result := Object_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start NoTypeObjectRenamingDecl.defining_env
                  Property_Result := No_Type_Object_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_env
                  Property_Result := Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_env
                  Property_Result := Single_Protected_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_env
                  Property_Result := Single_Task_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , Include_Ud_Indexing
                        , Dottable_Type
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Defining_Env;
--# end
--# property-start '[dispatcher]BasicDecl.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.array_ndims
                  Property_Result := Basic_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.array_ndims
                  Property_Result := Component_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_ndims
                  Property_Result := Base_Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_ndims
                  Property_Result := Base_Subtype_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.array_ndims
                  Property_Result := Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Array_Ndims;
--# end
--# property-start '[dispatcher]BasicDecl.expr_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl .. Ada_Named_Stmt_Decl | Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Protected_Decl | Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.expr_type
                  Property_Result := Basic_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.expr_type
                  Property_Result := Base_Type_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.expr_type
                  Property_Result := Basic_Subp_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.expr_type
                  Property_Result := Base_Subp_Body_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.expr_type
                  Property_Result := Entry_Index_Spec_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.expr_type
                  Property_Result := Exception_Handler_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.expr_type
                  Property_Result := For_Loop_Var_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.expr_type
                  Property_Result := Generic_Subp_Instantiation_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.expr_type
                  Property_Result := Number_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start NoTypeObjectRenamingDecl.expr_type
                  Property_Result := No_Type_Object_Renaming_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.expr_type
                  Property_Result := Single_Task_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Expr_Type;
--# end
--# property-start '[dispatcher]BasicDecl.type_expression' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.type_expression
                  Property_Result := Basic_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.type_expression
                  Property_Result := Abstract_State_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.type_expression
                  Property_Result := Anonymous_Expr_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.type_expression
                  Property_Result := Component_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.type_expression
                  Property_Result := Discriminant_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.type_expression
                  Property_Result := Generic_Formal_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.type_expression
                  Property_Result := Param_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.type_expression
                  Property_Result := Synthetic_Formal_Param_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.type_expression
                  Property_Result := Basic_Subp_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.type_expression
                  Property_Result := Base_Subp_Body_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.type_expression
                  Property_Result := Subp_Body_Stub_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.type_expression
                  Property_Result := Object_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Object_Decl =>
                  --# property-call-start SyntheticObjectDecl.type_expression
                  Property_Result := Synthetic_Object_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Type_Expression;
--# end
--# property-start '[dispatcher]BasicDecl.constrain_prefix' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.constrain_prefix
                  Property_Result := Basic_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.constrain_prefix
                  Property_Result := Component_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.constrain_prefix
                  Property_Result := Param_Spec_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
                  --# property-call-start BasicSubpDecl.constrain_prefix
                  Property_Result := Basic_Subp_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.constrain_prefix
                  Property_Result := Base_Subp_Body_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Package_Decl | Ada_Entry_Index_Spec .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.next_part_for_decl
                  Property_Result := Basic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.next_part_for_decl
                  Property_Result := Base_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.next_part_for_decl
                  Property_Result := Basic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.next_part_for_decl
                  Property_Result := Entry_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.next_part_for_decl
                  Property_Result := Synthetic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt_Body .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Task_Body =>
                  --# property-call-start Body.next_part_for_decl
                  Property_Result := Body_Node_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.next_part_for_decl
                  Property_Result := Body_Stub_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.next_part_for_decl
                  Property_Result := Exception_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericDecl.next_part_for_decl
                  Property_Result := Generic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.next_part_for_decl
                  Property_Result := Generic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.next_part_for_decl
                  Property_Result := Object_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.next_part_for_name
                  Property_Result := Basic_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_name
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.next_part_for_name
                  Property_Result := Object_Decl_P_Next_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Next_Part_For_Name;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_name
                  Property_Result := Basic_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_name
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.previous_part_for_name
                  Property_Result := Object_Decl_P_Previous_Part_For_Name
                    (Self
                        , Sym
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Name;
--# end
--# property-start '[dispatcher]BasicDecl.is_constant_object' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Constant_Object
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Synthetic_Formal_Param_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
                  --# property-call-start BasicDecl.is_constant_object
                  Property_Result := Basic_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.is_constant_object
                  Property_Result := Component_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.is_constant_object
                  Property_Result := Discriminant_Spec_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_constant_object
                  Property_Result := Param_Spec_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Decl =>
                  --# property-call-start SubpDecl.is_constant_object
                  Property_Result := Subp_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.is_constant_object
                  Property_Result := Enum_Literal_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.is_constant_object
                  Property_Result := Exception_Handler_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.is_constant_object
                  Property_Result := For_Loop_Var_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_constant_object
                  Property_Result := Number_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
                  --# property-call-start ObjectDecl.is_constant_object
                  Property_Result := Object_Decl_P_Is_Constant_Object
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Decl_P_Is_Constant_Object;
--# end
--# property-start BasicDecl.child_decl_initial_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4594'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_496 with Inline_Always;
            procedure Finalizer_Scope_2386 with Inline_Always;
      Local_Defining_Name : Internal_Entity_Defining_Name;
Local_Child_Name : Internal_Entity_Dotted_Name;
Var_Expr_0 : Internal_Entity_Dotted_Name;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_4 : Internal_Entity_Name;
Fld_5 : String_Type;
Sym_1 : Symbol_Type;
If_Result_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Sym_2 : Symbol_Type;
Result_Var : Symbol_Type;
Fld_6 : Boolean;
If_Result_2 : Symbol_Type;
Fld_7 : String_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal : Boolean;
If_Result_3 : Symbol_Type;
Let_Result : Symbol_Type;
Let_Result_1 : Symbol_Type;
            procedure Finalizer_Scope_496 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_496;
            procedure Finalizer_Scope_2386 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2386;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3260 '<Let at nodes.lkt:4595:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4595'
--# scope-start
--# expr-start 3240 '<FieldAccess for defining_name at nodes.lkt:4595:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4595'
--# expr-start 3239 '<as_bare_entity at nodes.lkt:4595:29>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4595'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3239
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3240
Local_Defining_Name := Fld; 
--# bind defining_name Local_Defining_Name
--# expr-start 3259 '<Let at nodes.lkt:4596:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4596'
--# scope-start
--# expr-start 3242 '<Cast to DottedName at nodes.lkt:4596:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4596'
--# expr-start 3241 '<FieldAccess for name at nodes.lkt:4596:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4596'
      if Local_Defining_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Local_Defining_Name.Node.Defining_Name_F_Name, Info => Local_Defining_Name.Info);
--# expr-done 3241
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 3242
Local_Child_Name := Cast_Result; 
--# bind child_name Local_Child_Name
--# expr-start 3258 '<If at nodes.lkt:4600:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
--# expr-start 3246 '<Eq at nodes.lkt:4600:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
--# expr-start 3244 '<to_symbol at nodes.lkt:4600:12>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
--# expr-start 3243 '<FieldAccess for text at nodes.lkt:4600:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
      if Local_Defining_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Text (Node => Local_Defining_Name.Node);
--# expr-done 3243
Sym_3 := String_To_Symbol (Self, Self.Unit.Context, Fld_7); 
--# expr-done 3244
--# expr-start 3245 '<Symbol standard at nodes.lkt:4600:46>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard_411); 
--# expr-done 3245
Is_Equal := Sym_3 = Sym_4; 
--# expr-done 3246
if Is_Equal then
   If_Result_3 := No_Symbol;
else
   --# expr-start 3257 '<If at nodes.lkt:4600:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4600'
--# expr-start 3247 '<FieldAccess for is_library_item at nodes.lkt:4601:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4601'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 3247
if Fld_6 then
Var_Expr_0 := Local_Child_Name;
if Var_Expr_0 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 3255 '<If at nodes.lkt:4604:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
if Private_Part then
   --# expr-start 3251 '<to_symbol at nodes.lkt:4604:34>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
--# expr-start 3249 '<FieldAccess for text at nodes.lkt:4604:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
--# expr-start 3248 '<FieldAccess for prefix at nodes.lkt:4604:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 3248
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Text (Node => Fld_2.Node);
--# expr-done 3249
--# expr-start 3250 '<String at nodes.lkt:4604:53>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
Str := Create_String (".__privatepart"); 
--# expr-done 3250
Concat_Result := Concat_String (Fld_3, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 3251
   If_Result_1 := Sym;
else
   --# expr-start 3254 '<to_symbol at nodes.lkt:4604:86>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
--# expr-start 3253 '<FieldAccess for text at nodes.lkt:4604:86>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
--# expr-start 3252 '<FieldAccess for prefix at nodes.lkt:4604:86>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 3252
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Text (Node => Fld_4.Node);
--# expr-done 3253
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld_5); 
--# expr-done 3254
   If_Result_1 := Sym_1;
end if;
--# expr-done 3255
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_2386;
   Result_Var := Scope_Result;
else
   --# expr-start 3256 '<Symbol standard at nodes.lkt:4604:125>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4604'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard_411); 
--# expr-done 3256
   Result_Var := Sym_2;
end if;
   If_Result_2 := Result_Var;
else
   If_Result_2 := No_Symbol;
end if;
--# expr-done 3257
   If_Result_3 := If_Result_2;
end if;
--# expr-done 3258
Let_Result := If_Result_3; 
--# end
--# expr-done 3259
Let_Result_1 := Let_Result; 
--# end
--# expr-done 3260
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_496;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_496;
                     Finalizer_Scope_2386;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start BasicDecl.previous_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4619'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Body_Node;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3264 '<Match at nodes.lkt:4619:55>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4619'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind btd Match_0
--# expr-start 3262 '<Cast to BasicDecl at nodes.lkt:4620:35>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4620'
--# expr-start 3261 '<FieldAccess for previous_part at nodes.lkt:4620:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4620'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Match_0.Node, Go_To_Incomplete => True, E_Info => Match_0.Info);
--# end
--# expr-done 3261
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 3262
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind bd Match_1
--# expr-start 3263 '<FieldAccess for previous_part_internal at nodes.lkt:4621:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4621'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 3263
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3264
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4628'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4632'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Corresponding_Actual
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Corresponding_Actual;
--# end
--# property-start BasicDecl.is_library_item 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4635'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3266 '<IsA LibraryItem at nodes.lkt:4635:35>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4635'
--# expr-start 3265 '<parent at nodes.lkt:4635:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4635'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3265
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 3266
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Library_Item;
--# end
--# property-start BasicDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4639'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Declarative_Parts
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Declarative_Part_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Declarative_Parts;
--# end
--# property-start BasicDecl.get_aspect_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4645'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Spec
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Spec
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Aspect_Spec;
      Fld : Internal_Entity_Aspect_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3267 '<FieldAccess for aspects at nodes.lkt:4645:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4645'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 3267
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Get_Aspect_Spec;
--# end
--# property-start BasicDecl.is_imported 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4649'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_99 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3268 '<FieldAccess for defining_name_or_raise at nodes.lkt:4649:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4649'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3268
Var_Expr_99 := Fld;
if Var_Expr_99 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 3269 '<FieldAccess for is_imported at nodes.lkt:4649:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4649'
      if Var_Expr_99.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_imported
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Imported (Node => Var_Expr_99.Node, E_Info => Var_Expr_99.Info);
--# end
--# expr-done 3269
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.is_in_public_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4651'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Public_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3272 '<IsA PublicPart at nodes.lkt:4651:37>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4651'
--# expr-start 3271 '<parent at nodes.lkt:4651:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4651'
--# expr-start 3270 '<parent at nodes.lkt:4651:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4651'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3270
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 3271
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Public_Part_Range; 
--# expr-done 3272
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_In_Public_Part;
--# end
--# property-start BasicDecl.is_in_private_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4653'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3275 '<IsA PrivatePart at nodes.lkt:4653:38>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4653'
--# expr-start 3274 '<parent at nodes.lkt:4653:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4653'
--# expr-start 3273 '<parent at nodes.lkt:4653:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4653'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3273
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 3274
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Private_Part_Range; 
--# expr-done 3275
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4658'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4661'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3277 '<FieldAccess for array_ndims at nodes.lkt:4661:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4661'
--# expr-start 3276 '<FieldAccess for expr_type at nodes.lkt:4661:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4661'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3276
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3277
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4680'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3278 '<FieldAccess for type_expression at nodes.lkt:4681:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4681'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3278
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind te Var_Expr_0
--# expr-start 3279 '<FieldAccess for designated_type at nodes.lkt:4681:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4681'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3279
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Expr_Type;
--# end
--# property-start BasicDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4686'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Node_For_Entity : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3281 '<as_entity at nodes.lkt:4686:47>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4686'
--# expr-start 3280 '<No TypeExpr at nodes.lkt:4686:47>' Node_For_Entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4686'
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 3280
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
--# expr-done 3281
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4707'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_510 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_510 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_510;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4710:9") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_510;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_510;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4720'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_511 with Inline_Always;
      Ignored : Bare_Compilation_Unit;
Match_0 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_100 : Internal_Entity_Basic_Decl;
Var_Expr_101 : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Aspect_Assoc;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Is_A : Boolean;
Is_A_1 : Boolean;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_3 : Internal_Entity_Aspect_Assoc;
Scope_Result : Internal_Entity_Aspect_Assoc;
Sym_1 : Symbol_Type;
Fld_4 : Internal_Entity_Pragma_Node;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Is_Null_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Not_Val_1 : Boolean;
Fld_5 : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Result_Var_2 : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Result_Var_3 : Bare_Compilation_Unit;
Match_Prefix : Bare_Basic_Decl;
Cast_Result_2 : Bare_Body_Node;
Let_Result : Bare_Compilation_Unit;
Scope_Result_4 : Bare_Compilation_Unit;
Cast_Result_3 : Bare_Basic_Decl;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_5 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Fld_6 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_511 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_511;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3302 '<Let at nodes.lkt:4723:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4723'
--# scope-start
--# expr-start 3297 '<Match at nodes.lkt:4723:17>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4723'
--# expr-start 3283 '<FieldAccess for decl at nodes.lkt:4723:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4723'
--# expr-start 3282 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:4723:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4723'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 3282
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld);
--# end
--# expr-done 3283
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind _ Match_0
Let_Result := No_Bare_Ada_Node; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind b Match_1
--# expr-start 3284 '<as_bare_entity at nodes.lkt:4725:34>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4725'
Is_Null := Match_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3284
Var_Expr_100 := If_Result;
if Var_Expr_100 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3285 '<FieldAccess for defining_name at nodes.lkt:4725:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4725'
      if Var_Expr_100.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_100.Node, E_Info => Var_Expr_100.Info);
--# end
--# expr-done 3285
Var_Expr_101 := Fld_2;
if Var_Expr_101 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 3296 '<FieldAccess for referenced_unit at nodes.lkt:4725:34>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4725'
      if Var_Expr_101.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3295 '<Not at nodes.lkt:4726:64>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4726'
--# expr-start 3286 '<IsA BasePackageDecl, GenericPackageDecl, GenericInstantiation, PackageRenamingDecl, SubpRenamingDecl, GenericRenamingDecl at nodes.lkt:4731:25>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4731'
Is_A := Match_1 /= null 
and then Match_1.Kind in Ada_Base_Package_Decl | Ada_Generic_Package_Decl_Range | Ada_Generic_Instantiation | Ada_Package_Renaming_Decl_Range | Ada_Subp_Renaming_Decl_Range | Ada_Generic_Renaming_Decl; 
--# expr-done 3286
if Is_A then
   If_Result_3 := True;
else
--# expr-start 3287 '<IsA BasicSubpDecl, GenericSubpDecl at nodes.lkt:4735:25>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4735'
Is_A_1 := Match_1 /= null 
and then Match_1.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Subp_Decl_Range; 
--# expr-done 3287
if Is_A_1 then
   --# expr-start 3294 '<Not at nodes.lkt:4739:29>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4739'
--# expr-start 3293 '<IsNull at nodes.lkt:4739:33>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4739'
--# expr-start 3288 '<as_entity at nodes.lkt:4739:33>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4739'
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3288
Var_Expr_0 := If_Result_1;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 3290 '<FieldAccess for get_aspect_assoc at nodes.lkt:4740:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4740'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3289 '<Symbol Import at nodes.lkt:4740:59>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4740'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 3289
--# property-call-start BasicDecl.get_aspect_assoc
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Assoc (Node => Var_Expr_0.Node, Name => Sym, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3290
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
      Cast_Result := Create_Internal_Entity
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
--# expr-start 3292 '<FieldAccess for get_pragma at nodes.lkt:4740:74>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4740'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3291 '<Symbol Import at nodes.lkt:4740:87>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4740'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 3291
--# property-call-start BasicDecl.get_pragma
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Var_Expr_0.Node, Name => Sym_1, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3292
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   Result_Var := Cast_Result_1;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Is_Null_2 := Result_Var_1.Node = null; 
--# expr-done 3293
Not_Val := not (Is_Null_2); 
--# expr-done 3294
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Not_Val_1 := not (If_Result_3); 
--# expr-done 3295
--# property-call-start Name.referenced_unit
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Var_Expr_101.Node, Kind => Unit_Body, Not_Found_Is_Error => Not_Val_1);
--# end
--# expr-done 3296
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Bare_Ada_Node;
end if;
Let_Result_1 := Result_Var_3; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 3297
Ignored := Match_Result; 
--# bind _ Ignored
--# expr-start 3301 '<Cast to BasicDecl at nodes.lkt:4748:9>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4748'
--# expr-start 3300 '<EnvGet at nodes.lkt:4748:9>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4748'
--# expr-start 3298 '<FieldAccess for children_env at nodes.lkt:4748:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3298
--# expr-start 3299 '<Symbol __nextpart at nodes.lkt:4749:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4749'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 3299
Env_Get_Result := AST_Envs.Get_First (Self => Fld_6, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 3300
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3301
Let_Result_2 := Cast_Result_4; 
--# end
--# expr-done 3302
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_511;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_511;
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicDecl.next_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4770'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3303 '<FieldAccess for next_part_for_decl at nodes.lkt:4771:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4771'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3303
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Next_Part_For_Name;
--# end
--# property-start BasicDecl.previous_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4790'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Name
  (Node : Bare_Basic_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
     Ent : Internal_Entity_Basic_Decl :=
       Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3304 '<FieldAccess for previous_part_for_decl at nodes.lkt:4791:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4791'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3304
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Previous_Part_For_Name;
--# end
--# property-start BasicDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4795'
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Constant_Object
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl  := Bare_Basic_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property BasicDecl.is_constant_object not implemented";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Decl_P_Is_Constant_Object;
--# end
   --
   --  Primitives for Bare_Abstract_State_Decl
   --
      procedure Abstract_State_Decl_Pre_Env_Actions
        (Self            : Bare_Abstract_State_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_0 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:4815:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Abstract_State_Decl
        (Self : Bare_Abstract_State_Decl
         ; Abstract_State_Decl_F_Name : Bare_Defining_Name
         ; Abstract_State_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Abstract_State_Decl_F_Name := Abstract_State_Decl_F_Name;
            Self.Abstract_State_Decl_F_Aspects := Abstract_State_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_State_Decl;
   function Abstract_State_Decl_F_Name
     (Node : Bare_Abstract_State_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Abstract_State_Decl_F_Name;
   end;
--# property-start AbstractStateDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4810'
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Defining_Names
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
     Ent : Internal_Entity_Abstract_State_Decl :=
       Internal_Entity_Abstract_State_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_515 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_515 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_515;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3306 '<ArrayLiteral at nodes.lkt:4810:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4810'
--# expr-start 3305 '<FieldAccess for name at nodes.lkt:4810:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4810'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Abstract_State_Decl_F_Name, Info => Ent.Info);
--# expr-done 3305
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 3306
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_515;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_515;
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_P_Defining_Names;
--# end
--# property-start AbstractStateDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4812'
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Type_Expression
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Type_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_P_Type_Expression;
--# end
--# property-start AbstractStateDecl._env_mappings_0 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4815'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_0
  (Node : Bare_Abstract_State_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Abstract_State_Decl  := Bare_Abstract_State_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1664 with Inline_Always;
      Fld : Bare_Defining_Name;
Fld_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Abstract_State_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1664 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1664;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3310 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 3307 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 3307
--# expr-start 3309 '<FieldAccess for name_symbol at nodes.lkt:4815:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4815'
--# expr-start 3308 '<FieldAccess for name at nodes.lkt:4815:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4815'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Abstract_State_Decl_F_Name;
--# expr-done 3308
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 3309
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 3310
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1664;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1664;
            raise;
      end;
   return Property_Result;
end Env_Mappings_0;
--# end
   --
   --  Primitives for Bare_Anonymous_Expr_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Expr_Decl
        (Self : Bare_Anonymous_Expr_Decl
         ; Anonymous_Expr_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Anonymous_Expr_Decl_F_Expr := Anonymous_Expr_Decl_F_Expr;
      end Initialize_Fields_For_Anonymous_Expr_Decl;
   function Anonymous_Expr_Decl_F_Expr
     (Node : Bare_Anonymous_Expr_Decl) return Bare_Expr
   is
   begin
         return Node.Anonymous_Expr_Decl_F_Expr;
   end;
--# property-start AnonymousExprDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4835'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Names
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Defining_Names;
--# end
--# property-start AnonymousExprDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4839'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Env
  (Node : Bare_Anonymous_Expr_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_518 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_518 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_518;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3312 '<FieldAccess for defining_env at nodes.lkt:4840:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4840'
--# expr-start 3311 '<FieldAccess for type_expression at nodes.lkt:4840:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4840'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3311
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3312
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_518;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_518;
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Defining_Env;
--# end
--# property-start AnonymousExprDecl.get_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4847'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Get_Formal
  (Node : Bare_Anonymous_Expr_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_519 with Inline_Always;
      Local_Assoc : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_519 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_519;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Anonymous_Expr_Decl_Anonymous_Expr_Decl_P_Get_Formal,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Anonymous_Expr_Decl,
                                As_Bare_Anonymous_Expr_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Defining_Name;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3318 '<Let at nodes.lkt:4848:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4848'
--# scope-start
--# expr-start 3315 '<Cast to BasicAssoc at nodes.lkt:4848:21>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4848'
--# expr-start 3314 '<parent at nodes.lkt:4848:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4848'
--# expr-start 3313 '<FieldAccess for expr at nodes.lkt:4848:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4848'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 3313
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 3314
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Assoc
   then
      Cast_Result := Create_Internal_Entity_Basic_Assoc
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Assoc;
   end if;
--# expr-done 3315
Local_Assoc := Cast_Result; 
--# bind assoc Local_Assoc
--# expr-start 3317 '<at at nodes.lkt:4850:9>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4850'
--# expr-start 3316 '<FieldAccess for get_params at nodes.lkt:4850:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4850'
      if Local_Assoc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicAssoc.get_params
Fld_2 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Local_Assoc.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Assoc.Info);
--# end
--# expr-done 3316
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 3317
Let_Result := Get_Result; 
--# end
--# expr-done 3318
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_519;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_519;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Defining_Name,
                        As_Internal_Entity_Defining_Name => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Anonymous_Expr_Decl_P_Get_Formal;
--# end
--# property-start AnonymousExprDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4856'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Type_Expression
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3321 '<FieldAccess for type_expression at nodes.lkt:4857:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4857'
--# expr-start 3320 '<FieldAccess for basic_decl at nodes.lkt:4857:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4857'
--# expr-start 3319 '<FieldAccess for get_formal at nodes.lkt:4857:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4857'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousExprDecl.get_formal
Fld := Libadalang.Implementation.Anonymous_Expr_Decl_P_Get_Formal (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 3319
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3320
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3321
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Type_Expression;
--# end
--# property-start AnonymousExprDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4860'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Is_Static_Decl
  (Node : Bare_Anonymous_Expr_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Expr_Decl  := Bare_Anonymous_Expr_Decl (Node);
     Ent : Internal_Entity_Anonymous_Expr_Decl :=
       Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3323 '<FieldAccess for is_static_expr at nodes.lkt:4860:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4860'
--# expr-start 3322 '<FieldAccess for expr at nodes.lkt:4860:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4860'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 3322
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 3323
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Expr_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Decl
   --
--# property-start BaseFormalParamDecl.formal_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4872'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Formal_Type
  (Node : Bare_Base_Formal_Param_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_102 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3324 '<FieldAccess for type_expression at nodes.lkt:4873:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4873'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3324
Var_Expr_102 := Fld;
if Var_Expr_102 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 3325 '<FieldAccess for designated_type at nodes.lkt:4873:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4873'
      if Var_Expr_102.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_102.Node, Origin => Origin, E_Info => Var_Expr_102.Info);
--# end
--# expr-done 3325
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Formal_Type;
--# end
--# property-start BaseFormalParamDecl.parent_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4875'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Parent_Decl
  (Node : Bare_Base_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3327 '<Cast to BasicDecl at nodes.lkt:4876:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4876'
--# expr-start 3326 '<FieldAccess for semantic_parent at nodes.lkt:4876:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4876'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3326
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3327
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Parent_Decl;
--# end
--# property-start BaseFormalParamDecl.get_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4878'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Get_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Part : Internal_Entity_Basic_Decl
      ; Param : Symbol_Type
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   --# bind part Part
   --# bind param Param
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2400 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_103 : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Base_Formal_Param_Holder;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2400 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2400;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_0 := Part;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr_0
--# expr-start 3328 '<FieldAccess for formal_param_holder_or_null at nodes.lkt:4879:16>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4879'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3328
Var_Expr_103 := Fld;
if Var_Expr_103 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 3331 '<Find at nodes.lkt:4879:16>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4879'
   --# expr-start 3329 '<FieldAccess for unpacked_formal_params at nodes.lkt:4879:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4879'
      if Var_Expr_103.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_103.Node, E_Info => Var_Expr_103.Info);
--# end
--# expr-done 3329
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind sf Item_1
      --# expr-start 3330 '<FieldAccess for name_is at nodes.lkt:4879:87>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4879'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_1.Node, Sym => Param);
--# end
--# expr-done 3330
      if Fld_2 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 3331
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_2400;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Defining_Name;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2400;
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Get_Param;
--# end
--# property-start BaseFormalParamDecl.decl_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4885'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Decl_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Param : Internal_Entity_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind param Param
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Match_0 : Internal_Entity_Base_Subp_Body;
Match_1 : Internal_Entity_Accept_Stmt;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Entry_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Accept_Stmt;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_3 : Symbol_Type;
Fld_4 : Internal_Entity_Defining_Name;
Scope_Result_3 : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3337 '<FieldAccess for get_param at nodes.lkt:4885:73>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4885'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3335 '<Match at nodes.lkt:4886:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4886'
--# expr-start 3332 '<FieldAccess for semantic_parent at nodes.lkt:4886:15>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4886'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3332
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind body Match_0
--# expr-start 3333 '<FieldAccess for decl_part at nodes.lkt:4887:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4887'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3333
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Accept_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind accept Match_1
--# expr-start 3334 '<FieldAccess for corresponding_entry at nodes.lkt:4888:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4888'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.corresponding_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Corresponding_Entry (Node => Match_1.Node, Origin => No_Bare_Ada_Node, E_Info => Match_1.Info);
--# end
--# expr-done 3334
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Select_When_Part | Ada_For_Loop_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3335
--# expr-start 3336 '<FieldAccess for name_symbol at nodes.lkt:4890:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4890'
      if Param.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Param.Node);
--# end
--# expr-done 3336
--# property-call-start BaseFormalParamDecl.get_param
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Match_Result, Param => Fld_3);
--# end
--# expr-done 3337
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_3 := Var_Expr;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := Param;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Decl_Param;
--# end
--# property-start BaseFormalParamDecl.next_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4894'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Name
  (Node : Bare_Base_Formal_Param_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_104 : Internal_Entity_Basic_Decl;
Var_Expr_105 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3340 '<FieldAccess for get_param at nodes.lkt:4894:62>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4894'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3338 '<FieldAccess for parent_decl at nodes.lkt:4895:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4895'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3338
Var_Expr_104 := Fld;
if Var_Expr_104 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3339 '<FieldAccess for next_part_for_decl at nodes.lkt:4895:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4895'
      if Var_Expr_104.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr_104.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_104.Info);
--# end
--# expr-done 3339
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Sym);
--# end
--# expr-done 3340
Var_Expr_105 := Fld_2;
if Var_Expr_105 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 3341 '<FieldAccess for basic_decl at nodes.lkt:4894:62>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4894'
      if Var_Expr_105.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_105.Node, E_Info => Var_Expr_105.Info);
--# end
--# expr-done 3341
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Next_Part_For_Name;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4899'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Name
  (Node : Bare_Base_Formal_Param_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_106 : Internal_Entity_Basic_Decl;
Var_Expr_107 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3344 '<FieldAccess for get_param at nodes.lkt:4899:66>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4899'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3342 '<FieldAccess for parent_decl at nodes.lkt:4900:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4900'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3342
Var_Expr_106 := Fld;
if Var_Expr_106 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3343 '<FieldAccess for previous_part_for_decl at nodes.lkt:4900:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4900'
      if Var_Expr_106.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Var_Expr_106.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_106.Info);
--# end
--# expr-done 3343
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Sym);
--# end
--# expr-done 3344
Var_Expr_107 := Fld_2;
if Var_Expr_107 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 3345 '<FieldAccess for basic_decl at nodes.lkt:4899:66>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4899'
      if Var_Expr_107.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_107.Node, E_Info => Var_Expr_107.Info);
--# end
--# expr-done 3345
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Previous_Part_For_Name;
--# end
--# property-start BaseFormalParamDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4904'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3347 '<FieldAccess for next_part_for_name at nodes.lkt:4905:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4905'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3346 '<FieldAccess for name_symbol at nodes.lkt:4905:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4905'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3346
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Fld, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3347
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4908'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Base_Formal_Param_Decl :=
       Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3349 '<FieldAccess for previous_part_for_name at nodes.lkt:4909:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4909'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3348 '<FieldAccess for name_symbol at nodes.lkt:4909:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4909'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3348
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Fld, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3349
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start '[dispatcher]BaseFormalParamDecl.is_mandatory' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Formal_Param_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start BaseFormalParamDecl.is_mandatory
                  Property_Result := Base_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_mandatory
                  Property_Result := Param_Spec_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Formal_Param_Decl =>
                  --# property-call-start SyntheticFormalParamDecl.is_mandatory
                  Property_Result := Synthetic_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start BaseFormalParamDecl.is_mandatory 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4911'
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl  := Bare_Base_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
   --
   --  Primitives for Bare_Component_Decl
   --
      procedure Component_Decl_Pre_Env_Actions
        (Self            : Bare_Component_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_1 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:4966:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Component_Decl
        (Self : Bare_Component_Decl
         ; Component_Decl_F_Ids : Bare_Defining_Name_List
         ; Component_Decl_F_Component_Def : Bare_Component_Def
         ; Component_Decl_F_Default_Expr : Bare_Expr
         ; Component_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Component_Decl_F_Ids := Component_Decl_F_Ids;
            Self.Component_Decl_F_Component_Def := Component_Decl_F_Component_Def;
            Self.Component_Decl_F_Default_Expr := Component_Decl_F_Default_Expr;
            Self.Component_Decl_F_Aspects := Component_Decl_F_Aspects;
      end Initialize_Fields_For_Component_Decl;
   function Component_Decl_F_Ids
     (Node : Bare_Component_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Component_Decl_F_Ids;
   end;
   function Component_Decl_F_Component_Def
     (Node : Bare_Component_Decl) return Bare_Component_Def
   is
   begin
         return Node.Component_Decl_F_Component_Def;
   end;
   function Component_Decl_F_Default_Expr
     (Node : Bare_Component_Decl) return Bare_Expr
   is
   begin
         return Node.Component_Decl_F_Default_Expr;
   end;
--# property-start ComponentDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4923'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Env
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_531 with Inline_Always;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_531 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_531;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3352 '<FieldAccess for defining_env at nodes.lkt:4924:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4924'
--# expr-start 3351 '<FieldAccess for type_expr at nodes.lkt:4924:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4924'
--# expr-start 3350 '<FieldAccess for component_def at nodes.lkt:4924:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4924'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 3350
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 3351
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_2 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_1.Info);
--# end
--# expr-done 3352
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_531;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_531;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Defining_Env;
--# end
--# property-start ComponentDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4926'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Names
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_532 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_532 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_532;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3355 '<Map at nodes.lkt:4927:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4927'
   --# expr-start 3353 '<FieldAccess for ids at nodes.lkt:4927:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4927'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 3353
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 3354 '<as_entity at nodes.lkt:4927:29>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4927'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 3354
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3355
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_532;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_532;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Defining_Names;
--# end
--# property-start ComponentDecl.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4930'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Array_Ndims
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3358 '<FieldAccess for array_ndims at nodes.lkt:4930:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4930'
--# expr-start 3357 '<FieldAccess for type_expr at nodes.lkt:4930:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4930'
--# expr-start 3356 '<FieldAccess for component_def at nodes.lkt:4930:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4930'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 3356
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 3357
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.array_ndims
Fld_2 := Libadalang.Implementation.Type_Expr_P_Array_Ndims (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3358
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Array_Ndims;
--# end
--# property-start ComponentDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4932'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Type_Expression
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Bare_Component_Def;
Fld_1 : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3361 '<as_entity at nodes.lkt:4933:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4933'
--# expr-start 3360 '<FieldAccess for type_expr at nodes.lkt:4933:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4933'
--# expr-start 3359 '<FieldAccess for component_def at nodes.lkt:4933:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4933'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Component_Def;
--# expr-done 3359
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Component_Def_F_Type_Expr;
--# expr-done 3360
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3361
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Type_Expression;
--# end
--# property-start ComponentDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4935'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Is_Constant_Object
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Is_Constant_Object;
--# end
--# property-start ComponentDecl.constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4938'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Constrain_Prefix
  (Node : Bare_Component_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_536 with Inline_Always;
      True_Rel : Logic_Equation;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_536 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_536;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3370 '<If at nodes.lkt:4942:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4942'
--# expr-start 3364 '<AnyQuantifier at nodes.lkt:4942:12>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4942'
   --# expr-start 3362 '<FieldAccess for parents at nodes.lkt:4942:12>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4942'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 3362
      Quantifier_Result := False;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 3363 '<IsA ProtectedDef at nodes.lkt:4942:38>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4942'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Protected_Def_Range; 
--# expr-done 3363
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3364
if Quantifier_Result then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4942:62") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 3369 '<And at nodes.lkt:4945:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4945'
--# expr-start 3367 '<Bind at nodes.lkt:4945:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4945'
--# expr-start 3365 '<FieldAccess for expected_type_var at nodes.lkt:4945:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4945'
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Prefix);
--# expr-done 3365
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 3366 '<FieldAccess for container_type at nodes.lkt:4945:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4945'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentDecl.container_type
Fld_1 := Libadalang.Implementation.Component_Decl_P_Container_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3366
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4945:13") else null)); 
--# expr-done 3367
--# expr-start 3368 '<FieldAccess for matches_expected_prefix_type at nodes.lkt:4945:72>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4945'
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_prefix_type
Fld_2 := Libadalang.Implementation.Expr_P_Matches_Expected_Prefix_Type (Node => Prefix, Origin => Origin);
--# end
--# expr-done 3368
And_Pred := Create_And (Bind_Result, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4945:13") else null)); 
--# expr-done 3369
   If_Result := And_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 3370
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_536;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_536;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Constrain_Prefix;
--# end
--# property-start ComponentDecl.container_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4949'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Container_Type
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_537 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_537 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_537;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3375 '<as_entity at nodes.lkt:4950:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4950'
--# expr-start 3374 '<Cast to BaseTypeDecl at nodes.lkt:4950:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4950'
--# expr-start 3373 '<Find at nodes.lkt:4950:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4950'
   --# expr-start 3371 '<FieldAccess for parents at nodes.lkt:4950:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4950'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 3371
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 3372 '<IsA BaseTypeDecl at nodes.lkt:4950:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4950'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Base_Type_Decl; 
--# expr-done 3372
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 3373
   if Find_Result = null
      or else Find_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3374
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 3375
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_537;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_537;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Container_Type;
--# end
--# property-start ComponentDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4953'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Equation
  (Node : Bare_Component_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
     Ent : Internal_Entity_Component_Decl :=
       Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_538 with Inline_Always;
            procedure Finalizer_Scope_2413 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Component_Def;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_538 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_538;
            procedure Finalizer_Scope_2413 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2413;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3388 '<Let at nodes.lkt:4954:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4954'
--# scope-start
--# expr-start 3376 '<FieldAccess for expr_type at nodes.lkt:4954:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4954'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3376
Local_Typ := Fld; 
--# bind typ Local_Typ
--# expr-start 3387 '<And at nodes.lkt:4956:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4956'
--# expr-start 3379 '<FieldAccess for sub_equation at nodes.lkt:4956:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4956'
--# expr-start 3378 '<FieldAccess for type_expr at nodes.lkt:4956:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4956'
--# expr-start 3377 '<FieldAccess for component_def at nodes.lkt:4956:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4956'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 3377
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Fld_1.Node.Component_Def_F_Type_Expr, Info => Fld_1.Info);
--# expr-done 3378
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 3379
--# expr-start 3380 '<FieldAccess for default_expr at nodes.lkt:4956:57>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4956'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 3380
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr_0
--# expr-start 3386 '<And at nodes.lkt:4957:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4957'
--# expr-start 3384 '<And at nodes.lkt:4958:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4958'
--# expr-start 3382 '<Bind at nodes.lkt:4958:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4958'
--# expr-start 3381 '<FieldAccess for expected_type_var at nodes.lkt:4958:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4958'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 3381
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4958:17") else null)); 
--# expr-done 3382
--# expr-start 3383 '<FieldAccess for sub_equation at nodes.lkt:4958:54>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4958'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3383
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4958:17") else null)); 
--# expr-done 3384
--# expr-start 3385 '<FieldAccess for matches_expected_assign_type at nodes.lkt:4959:19>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4959'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 3385
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4957:21") else null)); 
--# expr-done 3386
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2413;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4959:66") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4956:9") else null)); 
--# expr-done 3387
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3388
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_538;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_538;
                     Finalizer_Scope_2413;
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Xref_Equation;
--# end
--# property-start ComponentDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4963'
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Entry_Point
  (Node : Bare_Component_Decl
  )
   return Boolean
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Decl_P_Xref_Entry_Point;
--# end
--# property-start ComponentDecl._env_mappings_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4966'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_1
  (Node : Bare_Component_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Component_Decl  := Bare_Component_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1665 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Component_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1665 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1665;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3390 '<FieldAccess for env_mappings at nodes.lkt:4966:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4966'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3389 '<FieldAccess for ids at nodes.lkt:4966:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4966'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 3389
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3390
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1665;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1665;
            raise;
      end;
   return Property_Result;
end Env_Mappings_1;
--# end
   --
   --  Primitives for Bare_Discriminant_Spec
   --
      procedure Discriminant_Spec_Pre_Env_Actions
        (Self            : Bare_Discriminant_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_2 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:4998:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Discriminant_Spec
        (Self : Bare_Discriminant_Spec
         ; Discriminant_Spec_F_Ids : Bare_Defining_Name_List
         ; Discriminant_Spec_F_Type_Expr : Bare_Type_Expr
         ; Discriminant_Spec_F_Default_Expr : Bare_Expr
         ; Discriminant_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Discriminant_Spec_F_Ids := Discriminant_Spec_F_Ids;
            Self.Discriminant_Spec_F_Type_Expr := Discriminant_Spec_F_Type_Expr;
            Self.Discriminant_Spec_F_Default_Expr := Discriminant_Spec_F_Default_Expr;
            Self.Discriminant_Spec_F_Aspects := Discriminant_Spec_F_Aspects;
      end Initialize_Fields_For_Discriminant_Spec;
   function Discriminant_Spec_F_Ids
     (Node : Bare_Discriminant_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Discriminant_Spec_F_Ids;
   end;
   function Discriminant_Spec_F_Type_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Type_Expr;
   end;
   function Discriminant_Spec_F_Default_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Default_Expr;
   end;
--# property-start DiscriminantSpec.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4977'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Names
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_540 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_540 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_540;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3393 '<Map at nodes.lkt:4978:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4978'
   --# expr-start 3391 '<FieldAccess for ids at nodes.lkt:4978:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4978'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 3391
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind id Item_0
      --# expr-start 3392 '<as_entity at nodes.lkt:4978:30>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4978'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 3392
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3393
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_540;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_540;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Defining_Names;
--# end
--# property-start DiscriminantSpec.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4981'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Env
  (Node : Bare_Discriminant_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_541 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_541 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_541;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3395 '<FieldAccess for defining_env at nodes.lkt:4981:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4981'
--# expr-start 3394 '<FieldAccess for type_expr at nodes.lkt:4981:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4981'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3394
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3395
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_541;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_541;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Defining_Env;
--# end
--# property-start DiscriminantSpec.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4983'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Type_Expression
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3396 '<FieldAccess for type_expr at nodes.lkt:4983:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4983'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3396
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Type_Expression;
--# end
--# property-start DiscriminantSpec.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4985'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Entry_Point
  (Node : Bare_Discriminant_Spec
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Xref_Entry_Point;
--# end
--# property-start DiscriminantSpec.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4987'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Is_Constant_Object
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Is_Constant_Object;
--# end
--# property-start DiscriminantSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4990'
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Equation
  (Node : Bare_Discriminant_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
     Ent : Internal_Entity_Discriminant_Spec :=
       Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_545 with Inline_Always;
            procedure Finalizer_Scope_2415 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_545 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_545;
            procedure Finalizer_Scope_2415 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2415;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3407 '<And at nodes.lkt:4991:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4991'
--# expr-start 3398 '<FieldAccess for sub_equation at nodes.lkt:4991:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4991'
--# expr-start 3397 '<FieldAccess for type_expr at nodes.lkt:4991:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4991'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3397
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 3398
--# expr-start 3399 '<FieldAccess for default_expr at nodes.lkt:4991:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4991'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 3399
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr_0
--# expr-start 3406 '<And at nodes.lkt:4992:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4992'
--# expr-start 3404 '<And at nodes.lkt:4993:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4993'
--# expr-start 3402 '<Bind at nodes.lkt:4993:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4993'
--# expr-start 3400 '<FieldAccess for expected_type_var at nodes.lkt:4993:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4993'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 3400
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 3401 '<FieldAccess for expr_type at nodes.lkt:4993:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4993'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3401
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4993:17") else null)); 
--# expr-done 3402
--# expr-start 3403 '<FieldAccess for sub_equation at nodes.lkt:4993:67>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4993'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3403
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4993:17") else null)); 
--# expr-done 3404
--# expr-start 3405 '<FieldAccess for matches_expected_assign_type at nodes.lkt:4994:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4994'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 3405
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4992:21") else null)); 
--# expr-done 3406
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2415;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4994:66") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:4991:9") else null)); 
--# expr-done 3407
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_545;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_545;
                     Finalizer_Scope_2415;
            raise;
      end;
   return Property_Result;
end Discriminant_Spec_P_Xref_Equation;
--# end
--# property-start DiscriminantSpec._env_mappings_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4998'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_2
  (Node : Bare_Discriminant_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Discriminant_Spec  := Bare_Discriminant_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1666 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1666 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1666;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3409 '<FieldAccess for env_mappings at nodes.lkt:4998:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4998'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3408 '<FieldAccess for ids at nodes.lkt:4998:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:4998'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 3408
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3409
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1666;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1666;
            raise;
      end;
   return Property_Result;
end Env_Mappings_2;
--# end
   --
   --  Primitives for Bare_Generic_Formal
   --
      procedure Initialize_Fields_For_Generic_Formal
        (Self : Bare_Generic_Formal
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Self.Generic_Formal_F_Decl := Generic_Formal_F_Decl;
      end Initialize_Fields_For_Generic_Formal;
   function Generic_Formal_F_Decl
     (Node : Bare_Generic_Formal) return Bare_Basic_Decl
   is
   begin
         return Node.Generic_Formal_F_Decl;
   end;
--# property-start GenericFormal.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5009'
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Defining_Names
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
     Ent : Internal_Entity_Generic_Formal :=
       Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_546 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_546 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_546;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3411 '<FieldAccess for defining_names at nodes.lkt:5010:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5010'
--# expr-start 3410 '<FieldAccess for decl at nodes.lkt:5010:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5010'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3410
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3411
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_546;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_546;
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Defining_Names;
--# end
--# property-start GenericFormal.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5012'
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Type_Expression
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Generic_Formal  := Bare_Generic_Formal (Node);
     Ent : Internal_Entity_Generic_Formal :=
       Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3413 '<FieldAccess for type_expression at nodes.lkt:5012:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5012'
--# expr-start 3412 '<FieldAccess for decl at nodes.lkt:5012:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5012'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3412
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3413
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Formal_P_Type_Expression;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Obj_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Obj_Decl
        (Self : Bare_Generic_Formal_Obj_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Obj_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Package
   --
      procedure Initialize_Fields_For_Generic_Formal_Package
        (Self : Bare_Generic_Formal_Package
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Package;
   --
   --  Primitives for Bare_Generic_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Subp_Decl
        (Self : Bare_Generic_Formal_Subp_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Type_Decl
        (Self : Bare_Generic_Formal_Type_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Type_Decl;
--# property-start GenericFormalTypeDecl.default_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5029'
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Type_Decl_P_Default_Type
  (Node : Bare_Generic_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Generic_Formal_Type_Decl  := Bare_Generic_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Generic_Formal_Type_Decl :=
       Internal_Entity_Generic_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Formal_Type_Decl;
Match_1 : Internal_Entity_Incomplete_Formal_Type_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Var_Expr_108 : Internal_Entity_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Formal_Type_Decl;
Let_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Incomplete_Formal_Type_Decl;
Let_Result_1 : Internal_Entity_Name;
Scope_Result_1 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Name;
Scope_Result_2 : Internal_Entity_Name;
Match_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3417 '<Match at nodes.lkt:5029:48>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5029'
--# expr-start 3414 '<FieldAccess for decl at nodes.lkt:5029:54>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5029'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 3414
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ft Match_0
--# expr-start 3415 '<FieldAccess for default_type at nodes.lkt:5030:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5030'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Formal_Type_Decl_F_Default_Type, Info => Match_0.Info);
--# expr-done 3415
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Incomplete_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Incomplete_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ift Match_1
--# expr-start 3416 '<FieldAccess for default_type at nodes.lkt:5031:47>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5031'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Incomplete_Formal_Type_Decl_F_Default_Type, Info => Match_1.Info);
--# expr-done 3416
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl .. Ada_Concrete_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Name; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3417
Var_Expr_108 := Match_Result;
if Var_Expr_108 /= No_Entity_Name then
   --# scope-start
--# expr-start 3418 '<FieldAccess for name_designated_type at nodes.lkt:5029:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5029'
      if Var_Expr_108.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_108.Node, E_Info => Var_Expr_108.Info);
--# end
--# expr-done 3418
Scope_Result_3 := Fld_3;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Formal_Type_Decl_P_Default_Type;
--# end
   --
   --  Primitives for Bare_Param_Spec
   --
      procedure Param_Spec_Pre_Env_Actions
        (Self            : Bare_Param_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_3 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:5082:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Param_Spec
        (Self : Bare_Param_Spec
         ; Param_Spec_F_Ids : Bare_Defining_Name_List
         ; Param_Spec_F_Has_Aliased : Bare_Aliased_Node
         ; Param_Spec_F_Mode : Bare_Mode
         ; Param_Spec_F_Type_Expr : Bare_Type_Expr
         ; Param_Spec_F_Default_Expr : Bare_Expr
         ; Param_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Param_Spec_F_Ids := Param_Spec_F_Ids;
            Self.Param_Spec_F_Has_Aliased := Param_Spec_F_Has_Aliased;
            Self.Param_Spec_F_Mode := Param_Spec_F_Mode;
            Self.Param_Spec_F_Type_Expr := Param_Spec_F_Type_Expr;
            Self.Param_Spec_F_Default_Expr := Param_Spec_F_Default_Expr;
            Self.Param_Spec_F_Aspects := Param_Spec_F_Aspects;
      end Initialize_Fields_For_Param_Spec;
   function Param_Spec_F_Ids
     (Node : Bare_Param_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Param_Spec_F_Ids;
   end;
   function Param_Spec_F_Has_Aliased
     (Node : Bare_Param_Spec) return Bare_Aliased_Node
   is
   begin
         return Node.Param_Spec_F_Has_Aliased;
   end;
   function Param_Spec_F_Mode
     (Node : Bare_Param_Spec) return Bare_Mode
   is
   begin
         return Node.Param_Spec_F_Mode;
   end;
   function Param_Spec_F_Type_Expr
     (Node : Bare_Param_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Param_Spec_F_Type_Expr;
   end;
   function Param_Spec_F_Default_Expr
     (Node : Bare_Param_Spec) return Bare_Expr
   is
   begin
         return Node.Param_Spec_F_Default_Expr;
   end;
--# property-start ParamSpec.is_mandatory 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5045'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Mandatory
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Is_Null : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3420 '<IsNull at nodes.lkt:5045:32>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5045'
--# expr-start 3419 '<FieldAccess for default_expr at nodes.lkt:5045:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5045'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Default_Expr;
--# expr-done 3419
Is_Null := Fld = null; 
--# expr-done 3420
         Property_Result := Is_Null;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Is_Mandatory;
--# end
--# property-start ParamSpec.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5047'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Names
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_550 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_550 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_550;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3423 '<Map at nodes.lkt:5048:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5048'
   --# expr-start 3421 '<FieldAccess for ids at nodes.lkt:5048:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5048'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 3421
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind id Item_0
      --# expr-start 3422 '<as_entity at nodes.lkt:5048:30>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5048'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 3422
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3423
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_550;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_550;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Defining_Names;
--# end
--# property-start ParamSpec.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5050'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Type_Expression
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3424 '<FieldAccess for type_expr at nodes.lkt:5050:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5050'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3424
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Type_Expression;
--# end
--# property-start ParamSpec.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5052'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Constant_Object
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Mode;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3426 '<IsA Mode.In, Mode.Default at nodes.lkt:5052:38>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5052'
--# expr-start 3425 '<FieldAccess for mode at nodes.lkt:5052:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5052'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Mode;
--# expr-done 3425
Is_A := Fld /= null 
and then Fld.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 3426
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Is_Constant_Object;
--# end
--# property-start ParamSpec.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5055'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Env
  (Node : Bare_Param_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_553 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_553 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_553;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3428 '<FieldAccess for defining_env at nodes.lkt:5055:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5055'
--# expr-start 3427 '<FieldAccess for type_expr at nodes.lkt:5055:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5055'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3427
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 3428
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_553;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_553;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Defining_Env;
--# end
--# property-start ParamSpec.constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5058'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Constrain_Prefix
  (Node : Bare_Param_Spec
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_554 with Inline_Always;
            procedure Finalizer_Scope_2421 with Inline_Always;
      Var_Expr_0 : Bare_Name;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Cast_Expr_1 : Bare_Expr;
Cast_Result_1 : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_554 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_554;
            procedure Finalizer_Scope_2421 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2421;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3434 '<And at nodes.lkt:5064:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5064'
--# expr-start 3430 '<Bind at nodes.lkt:5064:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5064'
--# expr-start 3429 '<FieldAccess for type_var at nodes.lkt:5064:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5064'
      if Prefix = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 3429
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5064:9") else null)); 
--# expr-done 3430
--# expr-start 3431 '<Cast to Name at nodes.lkt:5064:64>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5064'
Cast_Expr_1 := Prefix; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Name
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3431
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 3433 '<Bind at nodes.lkt:5065:23>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5065'
--# expr-start 3432 '<FieldAccess for subp_spec_var at nodes.lkt:5065:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5065'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Var_Expr_0);
--# end
--# expr-done 3432
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Cast_Expr_2 := No_Entity_Base_Formal_Param_Holder; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_1, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5065:23") else null)); 
--# expr-done 3433
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2421;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5065:99") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5064:9") else null)); 
--# expr-done 3434
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_554;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_554;
                     Finalizer_Scope_2421;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Constrain_Prefix;
--# end
--# property-start ParamSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5069'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Equation
  (Node : Bare_Param_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
     Ent : Internal_Entity_Param_Spec :=
       Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_555 with Inline_Always;
            procedure Finalizer_Scope_2422 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_555 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_555;
            procedure Finalizer_Scope_2422 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2422;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3446 '<Let at nodes.lkt:5070:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5070'
--# scope-start
--# expr-start 3435 '<FieldAccess for expr_type at nodes.lkt:5070:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5070'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3435
Local_Typ := Fld; 
--# bind typ Local_Typ
--# expr-start 3445 '<And at nodes.lkt:5072:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5072'
--# expr-start 3437 '<FieldAccess for sub_equation at nodes.lkt:5072:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5072'
--# expr-start 3436 '<FieldAccess for type_expr at nodes.lkt:5072:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5072'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 3436
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 3437
--# expr-start 3438 '<FieldAccess for default_expr at nodes.lkt:5072:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5072'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 3438
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr_0
--# expr-start 3444 '<And at nodes.lkt:5073:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5073'
--# expr-start 3442 '<And at nodes.lkt:5074:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5074'
--# expr-start 3440 '<Bind at nodes.lkt:5074:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5074'
--# expr-start 3439 '<FieldAccess for expected_type_var at nodes.lkt:5074:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5074'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 3439
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5074:17") else null)); 
--# expr-done 3440
--# expr-start 3441 '<FieldAccess for sub_equation at nodes.lkt:5074:54>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5074'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3441
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5074:17") else null)); 
--# expr-done 3442
--# expr-start 3443 '<FieldAccess for matches_expected_assign_type at nodes.lkt:5075:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5075'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 3443
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5073:21") else null)); 
--# expr-done 3444
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2422;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5075:66") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:5072:9") else null)); 
--# expr-done 3445
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3446
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_555;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_555;
                     Finalizer_Scope_2422;
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Xref_Equation;
--# end
--# property-start ParamSpec.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5079'
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Entry_Point
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Param_Spec_P_Xref_Entry_Point;
--# end
--# property-start ParamSpec._env_mappings_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5082'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_3
  (Node : Bare_Param_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Param_Spec  := Bare_Param_Spec (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1667 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Param_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1667 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1667;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3448 '<FieldAccess for env_mappings at nodes.lkt:5082:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5082'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3447 '<FieldAccess for ids at nodes.lkt:5082:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5082'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 3447
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3448
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1667;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1667;
            raise;
      end;
   return Property_Result;
end Env_Mappings_3;
--# end
   --
   --  Primitives for Bare_Synthetic_Formal_Param_Decl
   --
      procedure Initialize_Fields_For_Synthetic_Formal_Param_Decl
        (Self : Bare_Synthetic_Formal_Param_Decl
         ; Synthetic_Formal_Param_Decl_F_Param_Type : Bare_Type_Expr
        ) is
      begin
            Self.Synthetic_Formal_Param_Decl_F_Param_Type := Synthetic_Formal_Param_Decl_F_Param_Type;
      Self.Synthetic_Formal_Param_Decl_Param_Name := No_Symbol;
      end Initialize_Fields_For_Synthetic_Formal_Param_Decl;
   function Synthetic_Formal_Param_Decl_F_Param_Type
     (Node : Bare_Synthetic_Formal_Param_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Synthetic_Formal_Param_Decl_F_Param_Type;
   end;
--# property-start SyntheticFormalParamDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5093'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Defining_Names
  (Node : Bare_Synthetic_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_557 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_557 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_557;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3452 '<ArrayLiteral at nodes.lkt:5094:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5094'
--# expr-start 3451 '<as_entity at nodes.lkt:5094:10>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5094'
--# expr-start 3450 '<FieldAccess for synthesize_defining_name at nodes.lkt:5094:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3449 '<FieldAccess for param_name at nodes.lkt:5094:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Formal_Param_Decl_Param_Name;
--# expr-done 3449
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 3450
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3451
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 3452
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_557;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_557;
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Defining_Names;
--# end
--# property-start SyntheticFormalParamDecl.is_mandatory 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5096'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Synthetic_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start SyntheticFormalParamDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5098'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Type_Expression
  (Node : Bare_Synthetic_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Synthetic_Formal_Param_Decl :=
       Internal_Entity_Synthetic_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3453 '<FieldAccess for param_type at nodes.lkt:5098:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5098'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Synthetic_Formal_Param_Decl_F_Param_Type, Info => Ent.Info);
--# expr-done 3453
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Type_Expression;
--# end
--# property-start SyntheticFormalParamDecl.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5100'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Formal_Param_Decl_P_Corresponding_Actual
  (Node : Bare_Synthetic_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Synthetic_Formal_Param_Decl  := Bare_Synthetic_Formal_Param_Decl (Node);
     Ent : Internal_Entity_Synthetic_Formal_Param_Decl :=
       Internal_Entity_Synthetic_Formal_Param_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Cast_Expr : Internal_Entity_Synthetic_Formal_Param_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Formal_Param_Decl_P_Corresponding_Actual;
--# end
   --
   --  Primitives for Bare_Base_Package_Decl
   --
      procedure Initialize_Fields_For_Base_Package_Decl
        (Self : Bare_Base_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Package_Decl_F_Package_Name := Base_Package_Decl_F_Package_Name;
            Self.Base_Package_Decl_F_Aspects := Base_Package_Decl_F_Aspects;
            Self.Base_Package_Decl_F_Public_Part := Base_Package_Decl_F_Public_Part;
            Self.Base_Package_Decl_F_Private_Part := Base_Package_Decl_F_Private_Part;
            Self.Base_Package_Decl_F_End_Name := Base_Package_Decl_F_End_Name;
      end Initialize_Fields_For_Base_Package_Decl;
   function Base_Package_Decl_F_Package_Name
     (Node : Bare_Base_Package_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Package_Decl_F_Package_Name;
   end;
   function Base_Package_Decl_F_Public_Part
     (Node : Bare_Base_Package_Decl) return Bare_Public_Part
   is
   begin
         return Node.Base_Package_Decl_F_Public_Part;
   end;
   function Base_Package_Decl_F_Private_Part
     (Node : Bare_Base_Package_Decl) return Bare_Private_Part
   is
   begin
         return Node.Base_Package_Decl_F_Private_Part;
   end;
   function Base_Package_Decl_F_End_Name
     (Node : Bare_Base_Package_Decl) return Bare_End_Name
   is
   begin
         return Node.Base_Package_Decl_F_End_Name;
   end;
--# property-start BasePackageDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5114'
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Names
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_561 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_561 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_561;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3455 '<ArrayLiteral at nodes.lkt:5114:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5114'
--# expr-start 3454 '<FieldAccess for package_name at nodes.lkt:5114:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5114'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Package_Decl_F_Package_Name, Info => Ent.Info);
--# expr-done 3454
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 3455
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_561;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_561;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Defining_Names;
--# end
--# property-start BasePackageDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5117'
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Env
  (Node : Bare_Base_Package_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_562 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_562 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_562;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3456 '<FieldAccess for children_env at nodes.lkt:5117:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5117'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3456
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_562;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_562;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Defining_Env;
--# end
--# property-start BasePackageDecl.body_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5121'
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Body_Part
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Package_Body;
Dyn_Var_Bind_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3459 '<bind at nodes.lkt:5122:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5122'
Bound_Imprecise_Fallback := False; 
--# expr-start 3458 '<Cast to PackageBody at nodes.lkt:5124:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5124'
--# expr-start 3457 '<FieldAccess for body_part_for_decl at nodes.lkt:5124:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5124'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3457
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Body;
   end if;
--# expr-done 3458
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 3459
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Body_Part;
--# end
--# property-start BasePackageDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5127'
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Declarative_Parts
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
     Ent : Internal_Entity_Base_Package_Decl :=
       Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_564 with Inline_Always;
            procedure Finalizer_Scope_2423 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_1 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Array_Lit_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_564 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_564;
            procedure Finalizer_Scope_2423 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2423;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3462 '<ArrayLiteral at nodes.lkt:5128:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
--# expr-start 3461 '<Cast to DeclarativePart at nodes.lkt:5128:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
--# expr-start 3460 '<FieldAccess for public_part at nodes.lkt:5128:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Public_Part (Node => Ent.Node.Base_Package_Decl_F_Public_Part, Info => Ent.Info);
--# expr-done 3460
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 3461
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result)); 
--# expr-done 3462
--# expr-start 3464 '<Cast to DeclarativePart at nodes.lkt:5128:50>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
--# expr-start 3463 '<FieldAccess for private_part at nodes.lkt:5128:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Ent.Node.Base_Package_Decl_F_Private_Part, Info => Ent.Info);
--# expr-done 3463
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 3464
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 3465 '<ArrayLiteral at nodes.lkt:5128:99>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5128'
Array_Lit_1 := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Var_Expr_0)); 
--# expr-done 3465
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2423;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Array_Lit, Result_Var); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_564;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_564;
                     Finalizer_Scope_2423;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Declarative_Parts;
--# end
--# property-start BasePackageDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5134'
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Env_Names
  (Node : Bare_Base_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Base_Package_Decl  := Bare_Base_Package_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_565 with Inline_Always;
            procedure Finalizer_Scope_2424 with Inline_Always;
      Local_Fqn : String_Type;
Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Sym_1 : Symbol_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym_2 : Symbol_Type;
Array_Lit_1 : Symbol_Type_Array_Access;
Fld_1 : Bare_Private_Part;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_565 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Fqn);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_565;
            procedure Finalizer_Scope_2424 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2424;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3477 '<Let at nodes.lkt:5135:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5135'
--# scope-start
--# expr-start 3466 '<FieldAccess for top_level_env_name at nodes.lkt:5135:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5135'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3466
Local_Fqn := Fld; Inc_Ref (Local_Fqn);
--# bind fqn Local_Fqn
Var_Expr_0 := Local_Fqn;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 3476 '<If at nodes.lkt:5138:22>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3469 '<Not at nodes.lkt:5138:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3468 '<IsNull at nodes.lkt:5138:29>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3467 '<FieldAccess for private_part at nodes.lkt:5138:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Base_Package_Decl_F_Private_Part;
--# expr-done 3467
Is_Null := Fld_1 = null; 
--# expr-done 3468
Not_Val := not (Is_Null); 
--# expr-done 3469
if Not_Val then
   --# expr-start 3471 '<ArrayLiteral at nodes.lkt:5138:60>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3470 '<to_symbol at nodes.lkt:5138:61>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 3470
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 3471
   If_Result := Array_Lit;
else
   --# expr-start 3475 '<ArrayLiteral at nodes.lkt:5138:81>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3472 '<to_symbol at nodes.lkt:5138:82>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 3472
--# expr-start 3474 '<to_symbol at nodes.lkt:5138:97>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
--# expr-start 3473 '<String at nodes.lkt:5138:104>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5138'
Str := Create_String (".__privatepart"); 
--# expr-done 3473
Concat_Result := Concat_String (Var_Expr_0, Str); 
Sym_2 := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 3474
Array_Lit_1 := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym_1, 2 => Sym_2)); 
--# expr-done 3475
   If_Result := Array_Lit_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 3476
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2424;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3477
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_565;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_565;
                     Finalizer_Scope_2424;
            raise;
      end;
   return Property_Result;
end Base_Package_Decl_P_Env_Names;
--# end
   --
   --  Primitives for Bare_Generic_Package_Internal
   --
      procedure Generic_Package_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_4 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_5 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Package_Internal
        (Self : Bare_Generic_Package_Internal
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Generic_Package_Internal;
--# property-start GenericPackageInternal.is_library_item 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5148'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Internal_P_Is_Library_Item
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3480 '<IsA LibraryItem at nodes.lkt:5148:35>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5148'
--# expr-start 3479 '<parent at nodes.lkt:5148:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5148'
--# expr-start 3478 '<parent at nodes.lkt:5148:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5148'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3478
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 3479
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Library_Item_Range; 
--# expr-done 3480
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Internal_P_Is_Library_Item;
--# end
--# property-start GenericPackageInternal._env_trans_parent_4 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_4
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_4;
--# end
--# property-start GenericPackageInternal._env_names_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5151'
pragma Warnings (Off, "is not referenced");
 function Env_Names_5
  (Node : Bare_Generic_Package_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Package_Internal  := Bare_Generic_Package_Internal (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1669 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1669 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1669;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3481 '<FieldAccess for env_names at nodes.lkt:5151:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5151'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3481
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1669;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1669;
            raise;
      end;
   return Property_Result;
end Env_Names_5;
--# end
   --
   --  Primitives for Bare_Package_Decl
   --
      procedure Package_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_6 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_7 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:5159:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_8 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:5165:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_9 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_10 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_11 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_13 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_12 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_15 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_14 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Decl
        (Self : Bare_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Package_Decl;
--# property-start PackageDecl._env_do_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5158'
pragma Warnings (Off, "is not referenced");
 function Env_Do_6
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3482 '<FieldAccess for env_hook at nodes.lkt:5158:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5158'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3482
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_6;
--# end
--# property-start PackageDecl._env_init_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5163'
pragma Warnings (Off, "is not referenced");
 function Env_Init_7
  (Node : Bare_Package_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1671 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1671 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1671;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3483 '<FieldAccess for child_decl_initial_env at nodes.lkt:5163:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5163'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 3483
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1671;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1671;
            raise;
      end;
   return Property_Result;
end Env_Init_7;
--# end
--# property-start PackageDecl._env_mappings_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5165'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_8
  (Node : Bare_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
     Ent : Internal_Entity_Package_Decl :=
       Internal_Entity_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1672 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1672 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1672;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3484 '<FieldAccess for child_decl_env_assocs at nodes.lkt:5165:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5165'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3484
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1672;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1672;
            raise;
      end;
   return Property_Result;
end Env_Mappings_8;
--# end
--# property-start PackageDecl._env_trans_parent_9 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_9
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_9;
--# end
--# property-start PackageDecl._env_names_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5166'
pragma Warnings (Off, "is not referenced");
 function Env_Names_10
  (Node : Bare_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1674 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1674 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1674;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3485 '<FieldAccess for env_names at nodes.lkt:5166:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5166'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3485
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1674;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1674;
            raise;
      end;
   return Property_Result;
end Env_Names_10;
--# end
--# property-start PackageDecl._env_do_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5167'
pragma Warnings (Off, "is not referenced");
 function Env_Do_11
  (Node : Bare_Package_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1675 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1675 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1675;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3486 '<FieldAccess for populate_dependent_units at nodes.lkt:5167:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5167'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3486
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1675;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1675;
            raise;
      end;
   return Property_Result;
end Env_Do_11;
--# end
--# property-start PackageDecl._ref_env_nodes_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5169'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_12
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1676 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1676 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1676;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3487 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:5169:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5169'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3487
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1676;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1676;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_12;
--# end
--# property-start PackageDecl._ref_cond_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5169'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_13
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3489 '<IsA LibraryItem, Subunit at nodes.lkt:5169:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5169'
--# expr-start 3488 '<parent at nodes.lkt:5169:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5169'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3488
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3489
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_13;
--# end
--# property-start PackageDecl._ref_env_nodes_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5172'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_14
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1678 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1678 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1678;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3490 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:5172:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5172'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3490
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1678;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1678;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_14;
--# end
--# property-start PackageDecl._ref_cond_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5172'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_15
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl  := Bare_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3492 '<IsA LibraryItem, Subunit at nodes.lkt:5172:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5172'
--# expr-start 3491 '<parent at nodes.lkt:5172:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5172'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 3491
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3492
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_15;
--# end
   --
   --  Primitives for Bare_Base_Type_Decl
   --
      procedure Base_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Base_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_16 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:6624:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Base_Type_Decl
        (Self : Bare_Base_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Base_Type_Decl_F_Name := Base_Type_Decl_F_Name;
      Self.Bare_Base_Type_Decl_Lf_State_Attributes_Repo := Uninitialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Attributes_Repo := No_Bare_Ada_Node;
      Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitive_Subps := Uninitialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitive_Subps := No_Internal_Inner_Env_Assoc_Array_Type;
      Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitives_Env := Uninitialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitives_Env := Empty_Env;
      Self.Bare_Base_Type_Decl_Lf_State_Synthetic_Object_Decl_Env_Assoc := Uninitialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Synthetic_Object_Decl_Env_Assoc := No_Env_Assoc;
      Self.Bare_Base_Type_Decl_Lf_State_Dottable_Subps_Env := Uninitialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Dottable_Subps_Env := Empty_Env;
      end Initialize_Fields_For_Base_Type_Decl;
   function Base_Type_Decl_F_Name
     (Node : Bare_Base_Type_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Type_Decl_F_Name;
   end;
--# property-start BaseTypeDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5183'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Defining_Names
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_567 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_567 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_567;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3494 '<ArrayLiteral at nodes.lkt:5183:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5183'
--# expr-start 3493 '<FieldAccess for name at nodes.lkt:5183:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Type_Decl_F_Name, Info => Ent.Info);
--# expr-done 3493
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 3494
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_567;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_567;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Defining_Names;
--# end
--# property-start BaseTypeDecl.base_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5190'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Discrete_Base_Subtype_Decl;
Match_1 : Internal_Entity_Base_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Expr : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Subtype_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Base_Subtype,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3497 '<Match at nodes.lkt:5190:48>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5190'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind db Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind st Match_1
--# expr-start 3496 '<FieldAccess for base_subtype at nodes.lkt:5192:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5192'
--# expr-start 3495 '<FieldAccess for get_type at nodes.lkt:5192:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5192'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3495
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3496
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := Ent; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3497
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Base_Subtype;
--# end
--# property-start BaseTypeDecl.anonymous_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5197'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Anonymous_Type_Access_Def;
Cast_Result : Bare_Type_Def;
New_Node_1 : Bare_Synth_Anonymous_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Anonymous_Access_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3502 '<as_entity at nodes.lkt:5197:57>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5197'
--# expr-start 3501 '<Cast to BaseTypeDecl at nodes.lkt:5197:57>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5197'
--# expr-start 3500 '<New SynthAnonymousTypeDecl at nodes.lkt:5197:57>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5197'
--# expr-start 3498 '<FieldAccess for name at nodes.lkt:5198:14>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5198'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 3498
--# expr-start 3499 '<New AnonymousTypeAccessDef at nodes.lkt:5198:72>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5198'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Self = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "AnonymousTypeAccessDef.type_decl cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Anonymous_Type_Access_Def);
Initialize
  (Self => New_Node,
   Kind => Ada_Anonymous_Type_Access_Def,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Anonymous_Type_Access_Def
     (Self => New_Node,
      Access_Def_F_Has_Not_Null => No_Bare_Ada_Node, Anonymous_Type_Access_Def_F_Type_Decl => Self);
--# expr-done 3499
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Cast_Result = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "TypeDecl.type_def cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synth_Anonymous_Type_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synth_Anonymous_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Synth_Anonymous_Type_Decl
     (Self => New_Node_1,
      Base_Type_Decl_F_Name => Fld, Type_Decl_F_Discriminants => No_Bare_Ada_Node, Type_Decl_F_Type_Def => Cast_Result);
--# expr-done 3500
      Cast_Result_1 := New_Node_1;
--# expr-done 3501
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3502
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Anonymous_Access_Type;
--# end
--# property-start BaseTypeDecl.anonymous_access_type_or_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5203'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_109 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_109 := Ent;
if Var_Expr_109 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3503 '<FieldAccess for anonymous_access_type at nodes.lkt:5204:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5204'
      if Var_Expr_109.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr_109.Node, E_Info => Var_Expr_109.Info);
--# end
--# expr-done 3503
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Anonymous_Access_Type_Or_Null;
--# end
--# property-start BaseTypeDecl.attributes_repo 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5207'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Attributes_Repo
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Type_Attributes_Repository
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Type_Attributes_Repository;
      New_Node : Bare_Type_Attributes_Repository;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Base_Type_Decl_Lf_State_Attributes_Repo is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Base_Type_Decl_Lf_Stg_Attributes_Repo;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Base_Type_Decl_Lf_State_Attributes_Repo,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3504 '<New TypeAttributesRepository at nodes.lkt:5208:9>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5208'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Type_Attributes_Repository);
Initialize
  (Self => New_Node,
   Kind => Ada_Type_Attributes_Repository,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
      New_Node.Type_Attributes_Repository_Base_Type :=
         Self;
--# expr-done 3504
         Property_Result := New_Node;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               Self.Bare_Base_Type_Decl_Lf_State_Attributes_Repo :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Base_Type_Decl_Lf_State_Attributes_Repo := Initialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Attributes_Repo := Property_Result;
   return Property_Result;
end Base_Type_Decl_F_Attributes_Repo;
--# end
--# property-start BaseTypeDecl.synthesize_attribute_subprogram 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5212'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Synthesize_Attribute_Subprogram
  (Node : Bare_Base_Type_Decl
      ; Attr_Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Subp_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind attr_name Attr_Name
   Property_Result : Internal_Entity_Basic_Subp_Decl;
      Local_Repo : Bare_Type_Attributes_Repository;
Local_Subp : Bare_Basic_Subp_Decl;
Fld : Bare_Type_Attributes_Repository;
Fld_1 : Bare_Basic_Subp_Decl;
Fld_2 : Bare_Basic_Subp_Decl;
Fld_3 : Bare_Basic_Subp_Decl;
Fld_4 : Bare_Basic_Subp_Decl;
Fld_5 : Bare_Basic_Subp_Decl;
Fld_6 : Bare_Basic_Subp_Decl;
Fld_7 : Bare_Basic_Subp_Decl;
Fld_8 : Bare_Basic_Subp_Decl;
Fld_9 : Bare_Basic_Subp_Decl;
Fld_10 : Bare_Basic_Subp_Decl;
Fld_11 : Bare_Basic_Subp_Decl;
Fld_12 : Bare_Basic_Subp_Decl;
Fld_13 : Bare_Basic_Subp_Decl;
Fld_14 : Bare_Basic_Subp_Decl;
Fld_15 : Bare_Basic_Subp_Decl;
Fld_16 : Bare_Basic_Subp_Decl;
Fld_17 : Bare_Basic_Subp_Decl;
Fld_18 : Bare_Basic_Subp_Decl;
Fld_19 : Bare_Basic_Subp_Decl;
Fld_20 : Bare_Basic_Subp_Decl;
Fld_21 : Bare_Basic_Subp_Decl;
Fld_22 : Bare_Basic_Subp_Decl;
Fld_23 : Bare_Basic_Subp_Decl;
Fld_24 : Bare_Basic_Subp_Decl;
Fld_25 : Bare_Basic_Subp_Decl;
Fld_26 : Bare_Basic_Subp_Decl;
Fld_27 : Bare_Basic_Subp_Decl;
Fld_28 : Bare_Basic_Subp_Decl;
Fld_29 : Bare_Basic_Subp_Decl;
Fld_30 : Bare_Basic_Subp_Decl;
Fld_31 : Bare_Basic_Subp_Decl;
Fld_32 : Bare_Basic_Subp_Decl;
Fld_33 : Bare_Basic_Subp_Decl;
Fld_34 : Bare_Basic_Subp_Decl;
Fld_35 : Bare_Basic_Subp_Decl;
Fld_36 : Bare_Basic_Subp_Decl;
Fld_37 : Bare_Basic_Subp_Decl;
Fld_38 : Bare_Basic_Subp_Decl;
Fld_39 : Bare_Basic_Subp_Decl;
Fld_40 : Bare_Basic_Subp_Decl;
Fld_41 : Bare_Basic_Subp_Decl;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Bare_Basic_Subp_Decl;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_1 : Bare_Basic_Subp_Decl;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_2 : Bare_Basic_Subp_Decl;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_3 : Bare_Basic_Subp_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_4 : Bare_Basic_Subp_Decl;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Bare_Basic_Subp_Decl;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Bare_Basic_Subp_Decl;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Bare_Basic_Subp_Decl;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_8 : Bare_Basic_Subp_Decl;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_9 : Bare_Basic_Subp_Decl;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_10 : Bare_Basic_Subp_Decl;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_11 : Bare_Basic_Subp_Decl;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_12 : Bare_Basic_Subp_Decl;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_13 : Bare_Basic_Subp_Decl;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_14 : Bare_Basic_Subp_Decl;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_15 : Bare_Basic_Subp_Decl;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_16 : Bare_Basic_Subp_Decl;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_17 : Bare_Basic_Subp_Decl;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_18 : Bare_Basic_Subp_Decl;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_19 : Bare_Basic_Subp_Decl;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_20 : Bare_Basic_Subp_Decl;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_21 : Bare_Basic_Subp_Decl;
Sym_22 : Symbol_Type;
Is_Equal_22 : Boolean;
If_Result_22 : Bare_Basic_Subp_Decl;
Sym_23 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_23 : Bare_Basic_Subp_Decl;
Sym_24 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_24 : Bare_Basic_Subp_Decl;
Sym_25 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_25 : Bare_Basic_Subp_Decl;
Sym_26 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_26 : Bare_Basic_Subp_Decl;
Sym_27 : Symbol_Type;
Is_Equal_27 : Boolean;
If_Result_27 : Bare_Basic_Subp_Decl;
Sym_28 : Symbol_Type;
Is_Equal_28 : Boolean;
If_Result_28 : Bare_Basic_Subp_Decl;
Sym_29 : Symbol_Type;
Is_Equal_29 : Boolean;
If_Result_29 : Bare_Basic_Subp_Decl;
Sym_30 : Symbol_Type;
Is_Equal_30 : Boolean;
If_Result_30 : Bare_Basic_Subp_Decl;
Sym_31 : Symbol_Type;
Is_Equal_31 : Boolean;
If_Result_31 : Bare_Basic_Subp_Decl;
Sym_32 : Symbol_Type;
Is_Equal_32 : Boolean;
If_Result_32 : Bare_Basic_Subp_Decl;
Sym_33 : Symbol_Type;
Is_Equal_33 : Boolean;
If_Result_33 : Bare_Basic_Subp_Decl;
Sym_34 : Symbol_Type;
Is_Equal_34 : Boolean;
If_Result_34 : Bare_Basic_Subp_Decl;
Sym_35 : Symbol_Type;
Is_Equal_35 : Boolean;
If_Result_35 : Bare_Basic_Subp_Decl;
Sym_36 : Symbol_Type;
Is_Equal_36 : Boolean;
If_Result_36 : Bare_Basic_Subp_Decl;
Sym_37 : Symbol_Type;
Is_Equal_37 : Boolean;
If_Result_37 : Bare_Basic_Subp_Decl;
Sym_38 : Symbol_Type;
Is_Equal_38 : Boolean;
If_Result_38 : Bare_Basic_Subp_Decl;
Sym_39 : Symbol_Type;
Is_Equal_39 : Boolean;
If_Result_39 : Bare_Basic_Subp_Decl;
Sym_40 : Symbol_Type;
Is_Equal_40 : Boolean;
If_Result_40 : Bare_Basic_Subp_Decl;
Fld_42 : Internal_Entity_Info;
Fld_43 : Env_Rebindings;
Fld_44 : Internal_Entity_Info;
Fld_45 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Basic_Subp_Decl;
Let_Result_1 : Internal_Entity_Basic_Subp_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3677 '<Let at nodes.lkt:5213:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5213'
--# scope-start
--# expr-start 3505 '<FieldAccess for attributes_repo at nodes.lkt:5213:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5213'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.attributes_repo
Fld := Libadalang.Implementation.Base_Type_Decl_F_Attributes_Repo (Node => Self);
--# end
--# expr-done 3505
Local_Repo := Fld; 
--# bind repo Local_Repo
--# expr-start 3676 '<Let at nodes.lkt:5214:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# scope-start
--# expr-start 3669 '<If at nodes.lkt:5214:20>' If_Result_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3507 '<Eq at nodes.lkt:5214:23>' Is_Equal_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3506 '<Symbol succ at nodes.lkt:5214:36>' Sym_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ_413); 
--# expr-done 3506
Is_Equal_40 := Attr_Name = Sym_40; 
--# expr-done 3507
if Is_Equal_40 then
   --# expr-start 3508 '<FieldAccess for succ at nodes.lkt:5214:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.succ
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Succ (Node => Local_Repo);
--# end
--# expr-done 3508
   If_Result_40 := Fld_1;
else
   --# expr-start 3668 '<If at nodes.lkt:5214:20>' If_Result_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3510 '<Eq at nodes.lkt:5215:14>' Is_Equal_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5215'
--# expr-start 3509 '<Symbol pred at nodes.lkt:5215:27>' Sym_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5215'
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred_386); 
--# expr-done 3509
Is_Equal_39 := Attr_Name = Sym_39; 
--# expr-done 3510
if Is_Equal_39 then
   --# expr-start 3511 '<FieldAccess for pred at nodes.lkt:5215:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5215'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.pred
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Pred (Node => Local_Repo);
--# end
--# expr-done 3511
   If_Result_39 := Fld_2;
else
   --# expr-start 3667 '<If at nodes.lkt:5214:20>' If_Result_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3513 '<Eq at nodes.lkt:5216:14>' Is_Equal_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5216'
--# expr-start 3512 '<Symbol min at nodes.lkt:5216:27>' Sym_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5216'
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min_371); 
--# expr-done 3512
Is_Equal_38 := Attr_Name = Sym_38; 
--# expr-done 3513
if Is_Equal_38 then
   --# expr-start 3514 '<FieldAccess for min at nodes.lkt:5216:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5216'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.min
Fld_3 := Libadalang.Implementation.Type_Attributes_Repository_F_Min (Node => Local_Repo);
--# end
--# expr-done 3514
   If_Result_38 := Fld_3;
else
   --# expr-start 3666 '<If at nodes.lkt:5214:20>' If_Result_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3516 '<Eq at nodes.lkt:5217:14>' Is_Equal_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5217'
--# expr-start 3515 '<Symbol max at nodes.lkt:5217:27>' Sym_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5217'
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_370); 
--# expr-done 3515
Is_Equal_37 := Attr_Name = Sym_37; 
--# expr-done 3516
if Is_Equal_37 then
   --# expr-start 3517 '<FieldAccess for max at nodes.lkt:5217:39>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5217'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.max
Fld_4 := Libadalang.Implementation.Type_Attributes_Repository_F_Max (Node => Local_Repo);
--# end
--# expr-done 3517
   If_Result_37 := Fld_4;
else
   --# expr-start 3665 '<If at nodes.lkt:5214:20>' If_Result_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3519 '<Eq at nodes.lkt:5218:14>' Is_Equal_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5218'
--# expr-start 3518 '<Symbol round at nodes.lkt:5218:27>' Sym_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5218'
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Round_405); 
--# expr-done 3518
Is_Equal_36 := Attr_Name = Sym_36; 
--# expr-done 3519
if Is_Equal_36 then
   --# expr-start 3520 '<FieldAccess for round at nodes.lkt:5218:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5218'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.round
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Round (Node => Local_Repo);
--# end
--# expr-done 3520
   If_Result_36 := Fld_5;
else
   --# expr-start 3664 '<If at nodes.lkt:5214:20>' If_Result_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3522 '<Eq at nodes.lkt:5219:14>' Is_Equal_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5219'
--# expr-start 3521 '<Symbol rounding at nodes.lkt:5219:27>' Sym_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5219'
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding_406); 
--# expr-done 3521
Is_Equal_35 := Attr_Name = Sym_35; 
--# expr-done 3522
if Is_Equal_35 then
   --# expr-start 3523 '<FieldAccess for rounding at nodes.lkt:5219:44>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5219'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.rounding
Fld_6 := Libadalang.Implementation.Type_Attributes_Repository_F_Rounding (Node => Local_Repo);
--# end
--# expr-done 3523
   If_Result_35 := Fld_6;
else
   --# expr-start 3663 '<If at nodes.lkt:5214:20>' If_Result_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3525 '<Eq at nodes.lkt:5220:14>' Is_Equal_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5220'
--# expr-start 3524 '<Symbol unbiased_rounding at nodes.lkt:5220:27>' Sym_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5220'
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding_422); 
--# expr-done 3524
Is_Equal_34 := Attr_Name = Sym_34; 
--# expr-done 3525
if Is_Equal_34 then
   --# expr-start 3526 '<FieldAccess for unbiased_rounding at nodes.lkt:5220:53>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5220'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.unbiased_rounding
Fld_7 := Libadalang.Implementation.Type_Attributes_Repository_F_Unbiased_Rounding (Node => Local_Repo);
--# end
--# expr-done 3526
   If_Result_34 := Fld_7;
else
   --# expr-start 3662 '<If at nodes.lkt:5214:20>' If_Result_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3528 '<Eq at nodes.lkt:5221:14>' Is_Equal_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5221'
--# expr-start 3527 '<Symbol ceiling at nodes.lkt:5221:27>' Sym_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5221'
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling_332); 
--# expr-done 3527
Is_Equal_33 := Attr_Name = Sym_33; 
--# expr-done 3528
if Is_Equal_33 then
   --# expr-start 3529 '<FieldAccess for ceiling at nodes.lkt:5221:43>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5221'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.ceiling
Fld_8 := Libadalang.Implementation.Type_Attributes_Repository_F_Ceiling (Node => Local_Repo);
--# end
--# expr-done 3529
   If_Result_33 := Fld_8;
else
   --# expr-start 3661 '<If at nodes.lkt:5214:20>' If_Result_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3531 '<Eq at nodes.lkt:5222:14>' Is_Equal_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5222'
--# expr-start 3530 '<Symbol floor at nodes.lkt:5222:27>' Sym_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5222'
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor_351); 
--# expr-done 3530
Is_Equal_32 := Attr_Name = Sym_32; 
--# expr-done 3531
if Is_Equal_32 then
   --# expr-start 3532 '<FieldAccess for floor at nodes.lkt:5222:41>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5222'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.floor
Fld_9 := Libadalang.Implementation.Type_Attributes_Repository_F_Floor (Node => Local_Repo);
--# end
--# expr-done 3532
   If_Result_32 := Fld_9;
else
   --# expr-start 3660 '<If at nodes.lkt:5214:20>' If_Result_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3534 '<Eq at nodes.lkt:5223:14>' Is_Equal_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5223'
--# expr-start 3533 '<Symbol truncation at nodes.lkt:5223:27>' Sym_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5223'
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation_420); 
--# expr-done 3533
Is_Equal_31 := Attr_Name = Sym_31; 
--# expr-done 3534
if Is_Equal_31 then
   --# expr-start 3535 '<FieldAccess for truncation at nodes.lkt:5223:46>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5223'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.truncation
Fld_10 := Libadalang.Implementation.Type_Attributes_Repository_F_Truncation (Node => Local_Repo);
--# end
--# expr-done 3535
   If_Result_31 := Fld_10;
else
   --# expr-start 3659 '<If at nodes.lkt:5214:20>' If_Result_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3537 '<Eq at nodes.lkt:5224:14>' Is_Equal_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5224'
--# expr-start 3536 '<Symbol machine at nodes.lkt:5224:27>' Sym_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5224'
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_368); 
--# expr-done 3536
Is_Equal_30 := Attr_Name = Sym_30; 
--# expr-done 3537
if Is_Equal_30 then
   --# expr-start 3538 '<FieldAccess for machine at nodes.lkt:5224:43>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5224'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.machine
Fld_11 := Libadalang.Implementation.Type_Attributes_Repository_F_Machine (Node => Local_Repo);
--# end
--# expr-done 3538
   If_Result_30 := Fld_11;
else
   --# expr-start 3658 '<If at nodes.lkt:5214:20>' If_Result_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3540 '<Eq at nodes.lkt:5225:14>' Is_Equal_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5225'
--# expr-start 3539 '<Symbol machine_rounding at nodes.lkt:5225:27>' Sym_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5225'
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding_369); 
--# expr-done 3539
Is_Equal_29 := Attr_Name = Sym_29; 
--# expr-done 3540
if Is_Equal_29 then
   --# expr-start 3541 '<FieldAccess for machine_rounding at nodes.lkt:5225:52>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5225'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.machine_rounding
Fld_12 := Libadalang.Implementation.Type_Attributes_Repository_F_Machine_Rounding (Node => Local_Repo);
--# end
--# expr-done 3541
   If_Result_29 := Fld_12;
else
   --# expr-start 3657 '<If at nodes.lkt:5214:20>' If_Result_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3543 '<Eq at nodes.lkt:5226:14>' Is_Equal_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5226'
--# expr-start 3542 '<Symbol fraction at nodes.lkt:5226:27>' Sym_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5226'
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction_353); 
--# expr-done 3542
Is_Equal_28 := Attr_Name = Sym_28; 
--# expr-done 3543
if Is_Equal_28 then
   --# expr-start 3544 '<FieldAccess for fraction at nodes.lkt:5226:44>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5226'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.fraction
Fld_13 := Libadalang.Implementation.Type_Attributes_Repository_F_Fraction (Node => Local_Repo);
--# end
--# expr-done 3544
   If_Result_28 := Fld_13;
else
   --# expr-start 3656 '<If at nodes.lkt:5214:20>' If_Result_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3546 '<Eq at nodes.lkt:5227:14>' Is_Equal_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5227'
--# expr-start 3545 '<Symbol exponent at nodes.lkt:5227:27>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5227'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent_349); 
--# expr-done 3545
Is_Equal_27 := Attr_Name = Sym_27; 
--# expr-done 3546
if Is_Equal_27 then
   --# expr-start 3547 '<FieldAccess for exponent at nodes.lkt:5227:44>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5227'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.exponent
Fld_14 := Libadalang.Implementation.Type_Attributes_Repository_F_Exponent (Node => Local_Repo);
--# end
--# expr-done 3547
   If_Result_27 := Fld_14;
else
   --# expr-start 3655 '<If at nodes.lkt:5214:20>' If_Result_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3549 '<Eq at nodes.lkt:5228:14>' Is_Equal_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5228'
--# expr-start 3548 '<Symbol copy_sign at nodes.lkt:5228:27>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5228'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign_335); 
--# expr-done 3548
Is_Equal_26 := Attr_Name = Sym_26; 
--# expr-done 3549
if Is_Equal_26 then
   --# expr-start 3550 '<FieldAccess for copy_sign at nodes.lkt:5228:45>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5228'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.copy_sign
Fld_15 := Libadalang.Implementation.Type_Attributes_Repository_F_Copy_Sign (Node => Local_Repo);
--# end
--# expr-done 3550
   If_Result_26 := Fld_15;
else
   --# expr-start 3654 '<If at nodes.lkt:5214:20>' If_Result_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3552 '<Eq at nodes.lkt:5229:14>' Is_Equal_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5229'
--# expr-start 3551 '<Symbol remainder at nodes.lkt:5229:27>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5229'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder_396); 
--# expr-done 3551
Is_Equal_25 := Attr_Name = Sym_25; 
--# expr-done 3552
if Is_Equal_25 then
   --# expr-start 3553 '<FieldAccess for remainder at nodes.lkt:5229:45>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5229'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.remainder
Fld_16 := Libadalang.Implementation.Type_Attributes_Repository_F_Remainder (Node => Local_Repo);
--# end
--# expr-done 3553
   If_Result_25 := Fld_16;
else
   --# expr-start 3653 '<If at nodes.lkt:5214:20>' If_Result_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3555 '<Eq at nodes.lkt:5230:14>' Is_Equal_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5230'
--# expr-start 3554 '<Symbol adjacent at nodes.lkt:5230:27>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5230'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent_321); 
--# expr-done 3554
Is_Equal_24 := Attr_Name = Sym_24; 
--# expr-done 3555
if Is_Equal_24 then
   --# expr-start 3556 '<FieldAccess for adjacent at nodes.lkt:5230:44>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5230'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.adjacent
Fld_17 := Libadalang.Implementation.Type_Attributes_Repository_F_Adjacent (Node => Local_Repo);
--# end
--# expr-done 3556
   If_Result_24 := Fld_17;
else
   --# expr-start 3652 '<If at nodes.lkt:5214:20>' If_Result_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3558 '<Eq at nodes.lkt:5231:14>' Is_Equal_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5231'
--# expr-start 3557 '<Symbol scaling at nodes.lkt:5231:27>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5231'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling_407); 
--# expr-done 3557
Is_Equal_23 := Attr_Name = Sym_23; 
--# expr-done 3558
if Is_Equal_23 then
   --# expr-start 3559 '<FieldAccess for scaling at nodes.lkt:5231:43>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5231'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.scaling
Fld_18 := Libadalang.Implementation.Type_Attributes_Repository_F_Scaling (Node => Local_Repo);
--# end
--# expr-done 3559
   If_Result_23 := Fld_18;
else
   --# expr-start 3651 '<If at nodes.lkt:5214:20>' If_Result_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3561 '<Eq at nodes.lkt:5232:14>' Is_Equal_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5232'
--# expr-start 3560 '<Symbol compose at nodes.lkt:5232:27>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5232'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose_333); 
--# expr-done 3560
Is_Equal_22 := Attr_Name = Sym_22; 
--# expr-done 3561
if Is_Equal_22 then
   --# expr-start 3562 '<FieldAccess for compose at nodes.lkt:5232:43>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5232'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.compose
Fld_19 := Libadalang.Implementation.Type_Attributes_Repository_F_Compose (Node => Local_Repo);
--# end
--# expr-done 3562
   If_Result_22 := Fld_19;
else
   --# expr-start 3650 '<If at nodes.lkt:5214:20>' If_Result_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3564 '<Eq at nodes.lkt:5233:14>' Is_Equal_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5233'
--# expr-start 3563 '<Symbol leading_part at nodes.lkt:5233:27>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5233'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part_364); 
--# expr-done 3563
Is_Equal_21 := Attr_Name = Sym_21; 
--# expr-done 3564
if Is_Equal_21 then
   --# expr-start 3565 '<FieldAccess for leading_part at nodes.lkt:5233:48>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5233'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.leading_part
Fld_20 := Libadalang.Implementation.Type_Attributes_Repository_F_Leading_Part (Node => Local_Repo);
--# end
--# expr-done 3565
   If_Result_21 := Fld_20;
else
   --# expr-start 3649 '<If at nodes.lkt:5214:20>' If_Result_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3567 '<Eq at nodes.lkt:5234:14>' Is_Equal_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5234'
--# expr-start 3566 '<Symbol mod at nodes.lkt:5234:27>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5234'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_372); 
--# expr-done 3566
Is_Equal_20 := Attr_Name = Sym_20; 
--# expr-done 3567
if Is_Equal_20 then
   --# expr-start 3568 '<FieldAccess for mod at nodes.lkt:5234:39>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5234'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.mod
Fld_21 := Libadalang.Implementation.Type_Attributes_Repository_F_Mod (Node => Local_Repo);
--# end
--# expr-done 3568
   If_Result_20 := Fld_21;
else
   --# expr-start 3648 '<If at nodes.lkt:5214:20>' If_Result_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3570 '<Eq at nodes.lkt:5235:14>' Is_Equal_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5235'
--# expr-start 3569 '<Symbol image at nodes.lkt:5235:27>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5235'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image_358); 
--# expr-done 3569
Is_Equal_19 := Attr_Name = Sym_19; 
--# expr-done 3570
if Is_Equal_19 then
   --# expr-start 3571 '<FieldAccess for image at nodes.lkt:5235:41>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5235'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.image
Fld_22 := Libadalang.Implementation.Type_Attributes_Repository_F_Image (Node => Local_Repo);
--# end
--# expr-done 3571
   If_Result_19 := Fld_22;
else
   --# expr-start 3647 '<If at nodes.lkt:5214:20>' If_Result_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3573 '<Eq at nodes.lkt:5236:14>' Is_Equal_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5236'
--# expr-start 3572 '<Symbol wide_image at nodes.lkt:5236:27>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5236'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image_429); 
--# expr-done 3572
Is_Equal_18 := Attr_Name = Sym_18; 
--# expr-done 3573
if Is_Equal_18 then
   --# expr-start 3574 '<FieldAccess for wide_image at nodes.lkt:5236:46>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5236'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_image
Fld_23 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Image (Node => Local_Repo);
--# end
--# expr-done 3574
   If_Result_18 := Fld_23;
else
   --# expr-start 3646 '<If at nodes.lkt:5214:20>' If_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3576 '<Eq at nodes.lkt:5237:14>' Is_Equal_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5237'
--# expr-start 3575 '<Symbol wide_wide_image at nodes.lkt:5237:27>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5237'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image_432); 
--# expr-done 3575
Is_Equal_17 := Attr_Name = Sym_17; 
--# expr-done 3576
if Is_Equal_17 then
   --# expr-start 3577 '<FieldAccess for wide_wide_image at nodes.lkt:5237:51>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5237'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_wide_image
Fld_24 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Wide_Image (Node => Local_Repo);
--# end
--# expr-done 3577
   If_Result_17 := Fld_24;
else
   --# expr-start 3645 '<If at nodes.lkt:5214:20>' If_Result_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3579 '<Eq at nodes.lkt:5238:14>' Is_Equal_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5238'
--# expr-start 3578 '<Symbol put_image at nodes.lkt:5238:27>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5238'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_390); 
--# expr-done 3578
Is_Equal_16 := Attr_Name = Sym_16; 
--# expr-done 3579
if Is_Equal_16 then
   --# expr-start 3580 '<FieldAccess for put_image at nodes.lkt:5238:45>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5238'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.put_image
Fld_25 := Libadalang.Implementation.Type_Attributes_Repository_F_Put_Image (Node => Local_Repo);
--# end
--# expr-done 3580
   If_Result_16 := Fld_25;
else
   --# expr-start 3644 '<If at nodes.lkt:5214:20>' If_Result_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3582 '<Eq at nodes.lkt:5239:14>' Is_Equal_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5239'
--# expr-start 3581 '<Symbol value at nodes.lkt:5239:27>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5239'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value_426); 
--# expr-done 3581
Is_Equal_15 := Attr_Name = Sym_15; 
--# expr-done 3582
if Is_Equal_15 then
   --# expr-start 3583 '<FieldAccess for value at nodes.lkt:5239:41>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5239'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.value
Fld_26 := Libadalang.Implementation.Type_Attributes_Repository_F_Value (Node => Local_Repo);
--# end
--# expr-done 3583
   If_Result_15 := Fld_26;
else
   --# expr-start 3643 '<If at nodes.lkt:5214:20>' If_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3585 '<Eq at nodes.lkt:5240:14>' Is_Equal_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5240'
--# expr-start 3584 '<Symbol wide_value at nodes.lkt:5240:27>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5240'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value_431); 
--# expr-done 3584
Is_Equal_14 := Attr_Name = Sym_14; 
--# expr-done 3585
if Is_Equal_14 then
   --# expr-start 3586 '<FieldAccess for wide_value at nodes.lkt:5240:46>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5240'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_value
Fld_27 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Value (Node => Local_Repo);
--# end
--# expr-done 3586
   If_Result_14 := Fld_27;
else
   --# expr-start 3642 '<If at nodes.lkt:5214:20>' If_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3588 '<Eq at nodes.lkt:5241:14>' Is_Equal_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5241'
--# expr-start 3587 '<Symbol wide_wide_value at nodes.lkt:5241:27>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5241'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value_434); 
--# expr-done 3587
Is_Equal_13 := Attr_Name = Sym_13; 
--# expr-done 3588
if Is_Equal_13 then
   --# expr-start 3589 '<FieldAccess for wide_wide_value at nodes.lkt:5241:51>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5241'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.wide_wide_value
Fld_28 := Libadalang.Implementation.Type_Attributes_Repository_F_Wide_Wide_Value (Node => Local_Repo);
--# end
--# expr-done 3589
   If_Result_13 := Fld_28;
else
   --# expr-start 3641 '<If at nodes.lkt:5214:20>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3591 '<Eq at nodes.lkt:5242:14>' Is_Equal_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5242'
--# expr-start 3590 '<Symbol fixed_value at nodes.lkt:5242:27>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5242'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value_350); 
--# expr-done 3590
Is_Equal_12 := Attr_Name = Sym_12; 
--# expr-done 3591
if Is_Equal_12 then
   --# expr-start 3592 '<FieldAccess for fixed_value at nodes.lkt:5242:47>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5242'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.fixed_value
Fld_29 := Libadalang.Implementation.Type_Attributes_Repository_F_Fixed_Value (Node => Local_Repo);
--# end
--# expr-done 3592
   If_Result_12 := Fld_29;
else
   --# expr-start 3640 '<If at nodes.lkt:5214:20>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3594 '<Eq at nodes.lkt:5243:14>' Is_Equal_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5243'
--# expr-start 3593 '<Symbol integer_value at nodes.lkt:5243:27>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5243'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value_361); 
--# expr-done 3593
Is_Equal_11 := Attr_Name = Sym_11; 
--# expr-done 3594
if Is_Equal_11 then
   --# expr-start 3595 '<FieldAccess for integer_value at nodes.lkt:5243:49>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5243'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.integer_value
Fld_30 := Libadalang.Implementation.Type_Attributes_Repository_F_Integer_Value (Node => Local_Repo);
--# end
--# expr-done 3595
   If_Result_11 := Fld_30;
else
   --# expr-start 3639 '<If at nodes.lkt:5214:20>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3597 '<Eq at nodes.lkt:5244:14>' Is_Equal_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5244'
--# expr-start 3596 '<Symbol pos at nodes.lkt:5244:27>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5244'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos_384); 
--# expr-done 3596
Is_Equal_10 := Attr_Name = Sym_10; 
--# expr-done 3597
if Is_Equal_10 then
   --# expr-start 3598 '<FieldAccess for pos at nodes.lkt:5244:39>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5244'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.pos
Fld_31 := Libadalang.Implementation.Type_Attributes_Repository_F_Pos (Node => Local_Repo);
--# end
--# expr-done 3598
   If_Result_10 := Fld_31;
else
   --# expr-start 3638 '<If at nodes.lkt:5214:20>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3600 '<Eq at nodes.lkt:5245:14>' Is_Equal_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5245'
--# expr-start 3599 '<Symbol val at nodes.lkt:5245:27>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5245'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val_425); 
--# expr-done 3599
Is_Equal_9 := Attr_Name = Sym_9; 
--# expr-done 3600
if Is_Equal_9 then
   --# expr-start 3601 '<FieldAccess for val_attr at nodes.lkt:5245:39>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5245'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.val_attr
Fld_32 := Libadalang.Implementation.Type_Attributes_Repository_F_Val_Attr (Node => Local_Repo);
--# end
--# expr-done 3601
   If_Result_9 := Fld_32;
else
   --# expr-start 3637 '<If at nodes.lkt:5214:20>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3603 '<Eq at nodes.lkt:5246:14>' Is_Equal_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5246'
--# expr-start 3602 '<Symbol enum_rep at nodes.lkt:5246:27>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5246'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep_345); 
--# expr-done 3602
Is_Equal_8 := Attr_Name = Sym_8; 
--# expr-done 3603
if Is_Equal_8 then
   --# expr-start 3604 '<FieldAccess for enum_rep at nodes.lkt:5246:44>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5246'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.enum_rep
Fld_33 := Libadalang.Implementation.Type_Attributes_Repository_F_Enum_Rep (Node => Local_Repo);
--# end
--# expr-done 3604
   If_Result_8 := Fld_33;
else
   --# expr-start 3636 '<If at nodes.lkt:5214:20>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3606 '<Eq at nodes.lkt:5247:14>' Is_Equal_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5247'
--# expr-start 3605 '<Symbol enum_val at nodes.lkt:5247:27>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5247'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val_346); 
--# expr-done 3605
Is_Equal_7 := Attr_Name = Sym_7; 
--# expr-done 3606
if Is_Equal_7 then
   --# expr-start 3607 '<FieldAccess for enum_val at nodes.lkt:5247:44>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5247'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.enum_val
Fld_34 := Libadalang.Implementation.Type_Attributes_Repository_F_Enum_Val (Node => Local_Repo);
--# end
--# expr-done 3607
   If_Result_7 := Fld_34;
else
   --# expr-start 3635 '<If at nodes.lkt:5214:20>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3609 '<Eq at nodes.lkt:5248:14>' Is_Equal_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5248'
--# expr-start 3608 '<Symbol read at nodes.lkt:5248:27>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5248'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read_393); 
--# expr-done 3608
Is_Equal_6 := Attr_Name = Sym_6; 
--# expr-done 3609
if Is_Equal_6 then
   --# expr-start 3610 '<FieldAccess for read at nodes.lkt:5248:40>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5248'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.read
Fld_35 := Libadalang.Implementation.Type_Attributes_Repository_F_Read (Node => Local_Repo);
--# end
--# expr-done 3610
   If_Result_6 := Fld_35;
else
   --# expr-start 3634 '<If at nodes.lkt:5214:20>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3612 '<Eq at nodes.lkt:5249:14>' Is_Equal_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5249'
--# expr-start 3611 '<Symbol write at nodes.lkt:5249:27>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5249'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write_436); 
--# expr-done 3611
Is_Equal_5 := Attr_Name = Sym_5; 
--# expr-done 3612
if Is_Equal_5 then
   --# expr-start 3613 '<FieldAccess for write at nodes.lkt:5249:41>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5249'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.write
Fld_36 := Libadalang.Implementation.Type_Attributes_Repository_F_Write (Node => Local_Repo);
--# end
--# expr-done 3613
   If_Result_5 := Fld_36;
else
   --# expr-start 3633 '<If at nodes.lkt:5214:20>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3615 '<Eq at nodes.lkt:5250:14>' Is_Equal_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5250'
--# expr-start 3614 '<Symbol input at nodes.lkt:5250:27>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5250'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input_360); 
--# expr-done 3614
Is_Equal_4 := Attr_Name = Sym_4; 
--# expr-done 3615
if Is_Equal_4 then
   --# expr-start 3616 '<FieldAccess for input at nodes.lkt:5250:41>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5250'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.input
Fld_37 := Libadalang.Implementation.Type_Attributes_Repository_F_Input (Node => Local_Repo);
--# end
--# expr-done 3616
   If_Result_4 := Fld_37;
else
   --# expr-start 3632 '<If at nodes.lkt:5214:20>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3618 '<Eq at nodes.lkt:5251:14>' Is_Equal_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5251'
--# expr-start 3617 '<Symbol output at nodes.lkt:5251:27>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5251'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output_381); 
--# expr-done 3617
Is_Equal_3 := Attr_Name = Sym_3; 
--# expr-done 3618
if Is_Equal_3 then
   --# expr-start 3619 '<FieldAccess for output at nodes.lkt:5251:42>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5251'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.output
Fld_38 := Libadalang.Implementation.Type_Attributes_Repository_F_Output (Node => Local_Repo);
--# end
--# expr-done 3619
   If_Result_3 := Fld_38;
else
   --# expr-start 3631 '<If at nodes.lkt:5214:20>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3621 '<Eq at nodes.lkt:5252:14>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5252'
--# expr-start 3620 '<Symbol asm_input at nodes.lkt:5252:27>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5252'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input_326); 
--# expr-done 3620
Is_Equal_2 := Attr_Name = Sym_2; 
--# expr-done 3621
if Is_Equal_2 then
   --# expr-start 3622 '<FieldAccess for asm_input at nodes.lkt:5252:45>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5252'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.asm_input
Fld_39 := Libadalang.Implementation.Type_Attributes_Repository_F_Asm_Input (Node => Local_Repo);
--# end
--# expr-done 3622
   If_Result_2 := Fld_39;
else
   --# expr-start 3630 '<If at nodes.lkt:5214:20>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3624 '<Eq at nodes.lkt:5253:14>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5253'
--# expr-start 3623 '<Symbol asm_output at nodes.lkt:5253:27>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5253'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output_327); 
--# expr-done 3623
Is_Equal_1 := Attr_Name = Sym_1; 
--# expr-done 3624
if Is_Equal_1 then
   --# expr-start 3625 '<FieldAccess for asm_output at nodes.lkt:5253:46>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5253'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.asm_output
Fld_40 := Libadalang.Implementation.Type_Attributes_Repository_F_Asm_Output (Node => Local_Repo);
--# end
--# expr-done 3625
   If_Result_1 := Fld_40;
else
   --# expr-start 3629 '<If at nodes.lkt:5214:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5214'
--# expr-start 3627 '<Eq at nodes.lkt:5254:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5254'
--# expr-start 3626 '<Symbol model at nodes.lkt:5254:27>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5254'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_373); 
--# expr-done 3626
Is_Equal := Attr_Name = Sym; 
--# expr-done 3627
if Is_Equal then
   --# expr-start 3628 '<FieldAccess for model at nodes.lkt:5254:41>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5254'
      if Local_Repo = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.model
Fld_41 := Libadalang.Implementation.Type_Attributes_Repository_F_Model (Node => Local_Repo);
--# end
--# expr-done 3628
   If_Result := Fld_41;
else
   If_Result := No_Bare_Ada_Node;
end if;
--# expr-done 3629
   If_Result_1 := If_Result;
end if;
--# expr-done 3630
   If_Result_2 := If_Result_1;
end if;
--# expr-done 3631
   If_Result_3 := If_Result_2;
end if;
--# expr-done 3632
   If_Result_4 := If_Result_3;
end if;
--# expr-done 3633
   If_Result_5 := If_Result_4;
end if;
--# expr-done 3634
   If_Result_6 := If_Result_5;
end if;
--# expr-done 3635
   If_Result_7 := If_Result_6;
end if;
--# expr-done 3636
   If_Result_8 := If_Result_7;
end if;
--# expr-done 3637
   If_Result_9 := If_Result_8;
end if;
--# expr-done 3638
   If_Result_10 := If_Result_9;
end if;
--# expr-done 3639
   If_Result_11 := If_Result_10;
end if;
--# expr-done 3640
   If_Result_12 := If_Result_11;
end if;
--# expr-done 3641
   If_Result_13 := If_Result_12;
end if;
--# expr-done 3642
   If_Result_14 := If_Result_13;
end if;
--# expr-done 3643
   If_Result_15 := If_Result_14;
end if;
--# expr-done 3644
   If_Result_16 := If_Result_15;
end if;
--# expr-done 3645
   If_Result_17 := If_Result_16;
end if;
--# expr-done 3646
   If_Result_18 := If_Result_17;
end if;
--# expr-done 3647
   If_Result_19 := If_Result_18;
end if;
--# expr-done 3648
   If_Result_20 := If_Result_19;
end if;
--# expr-done 3649
   If_Result_21 := If_Result_20;
end if;
--# expr-done 3650
   If_Result_22 := If_Result_21;
end if;
--# expr-done 3651
   If_Result_23 := If_Result_22;
end if;
--# expr-done 3652
   If_Result_24 := If_Result_23;
end if;
--# expr-done 3653
   If_Result_25 := If_Result_24;
end if;
--# expr-done 3654
   If_Result_26 := If_Result_25;
end if;
--# expr-done 3655
   If_Result_27 := If_Result_26;
end if;
--# expr-done 3656
   If_Result_28 := If_Result_27;
end if;
--# expr-done 3657
   If_Result_29 := If_Result_28;
end if;
--# expr-done 3658
   If_Result_30 := If_Result_29;
end if;
--# expr-done 3659
   If_Result_31 := If_Result_30;
end if;
--# expr-done 3660
   If_Result_32 := If_Result_31;
end if;
--# expr-done 3661
   If_Result_33 := If_Result_32;
end if;
--# expr-done 3662
   If_Result_34 := If_Result_33;
end if;
--# expr-done 3663
   If_Result_35 := If_Result_34;
end if;
--# expr-done 3664
   If_Result_36 := If_Result_35;
end if;
--# expr-done 3665
   If_Result_37 := If_Result_36;
end if;
--# expr-done 3666
   If_Result_38 := If_Result_37;
end if;
--# expr-done 3667
   If_Result_39 := If_Result_38;
end if;
--# expr-done 3668
   If_Result_40 := If_Result_39;
end if;
--# expr-done 3669
Local_Subp := If_Result_40; 
--# bind subp Local_Subp
--# expr-start 3675 '<New BasicSubpDecl.entity at nodes.lkt:5257:9>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5257'
--# expr-start 3674 '<New EntityInfo at nodes.lkt:5258:29>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5258'
--# expr-start 3671 '<FieldAccess for from_rebound at nodes.lkt:5259:82>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5259'
--# expr-start 3670 '<FieldAccess for info at nodes.lkt:5259:82>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5259'
Fld_44 := Ent.Info;
--# expr-done 3670
Fld_45 := Fld_44.From_Rebound;
--# expr-done 3671
--# expr-start 3673 '<FieldAccess for rebindings at nodes.lkt:5259:47>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5259'
--# expr-start 3672 '<FieldAccess for info at nodes.lkt:5259:47>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5259'
Fld_42 := Ent.Info;
--# expr-done 3672
Fld_43 := Fld_42.Rebindings;
--# expr-done 3673
New_Struct := (From_Rebound => Fld_45, Md => No_Metadata, Rebindings => Fld_43); 
--# expr-done 3674
New_Struct_1 := (Info => New_Struct, Node => Local_Subp); 
--# expr-done 3675
Let_Result := New_Struct_1; 
--# end
--# expr-done 3676
Let_Result_1 := Let_Result; 
--# end
--# expr-done 3677
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Synthesize_Attribute_Subprogram;
--# end
--# property-start BaseTypeDecl.attribute_subprogram 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5267'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Attribute_Subprogram
  (Node : Bare_Base_Type_Decl
      ; Attr_Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind attr_name Attr_Name
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Var_Expr_110 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Attribute_Def_Clause;
Fld_1 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_4 : Internal_Aspect;
Fld_5 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3697 '<If at nodes.lkt:5268:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
--# expr-start 3678 '<Symbol read at nodes.lkt:5268:25>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read_393); 
--# expr-done 3678
Is_Equal_4 := Attr_Name = Sym_2; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 3679 '<Symbol write at nodes.lkt:5268:35>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write_436); 
--# expr-done 3679
Is_Equal_3 := Attr_Name = Sym_3; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 3680 '<Symbol input at nodes.lkt:5268:46>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input_360); 
--# expr-done 3680
Is_Equal_2 := Attr_Name = Sym_4; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 3681 '<Symbol output at nodes.lkt:5268:57>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output_381); 
--# expr-done 3681
Is_Equal_1 := Attr_Name = Sym_5; 
   If_Result_1 := Is_Equal_1;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
--# expr-start 3682 '<FieldAccess for get_representation_clause at nodes.lkt:5268:72>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_representation_clause
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Ent.Node, Name => Attr_Name, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 3682
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind x Var_Expr_1
--# expr-start 3685 '<FieldAccess for referenced_decl at nodes.lkt:5269:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5269'
--# expr-start 3684 '<Cast to Name at nodes.lkt:5269:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5269'
--# expr-start 3683 '<FieldAccess for expr at nodes.lkt:5269:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5269'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr_1.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_1.Info);
--# expr-done 3683
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 3684
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_2 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result.Node, Imprecise_Fallback => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 3685
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 3686 '<FieldAccess for synthesize_attribute_subprogram at nodes.lkt:5269:68>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5269'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.synthesize_attribute_subprogram
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Attr_Name => Attr_Name, E_Info => Ent.Info);
--# end
--# expr-done 3686
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   Result_Var := Cast_Result_1;
end if;
   If_Result_4 := Result_Var;
else
   --# expr-start 3696 '<If at nodes.lkt:5268:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5268'
--# expr-start 3688 '<Eq at nodes.lkt:5271:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
--# expr-start 3687 '<Symbol put_image at nodes.lkt:5271:27>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_390); 
--# expr-done 3687
Is_Equal := Attr_Name = Sym_1; 
--# expr-done 3688
if Is_Equal then
--# expr-start 3692 '<Cast to Name at nodes.lkt:5271:45>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
Var_Expr_110 := Ent;
if Var_Expr_110 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3691 '<FieldAccess for value at nodes.lkt:5271:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
--# expr-start 3690 '<FieldAccess for get_aspect at nodes.lkt:5271:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
      if Var_Expr_110.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3689 '<Symbol put_image at nodes.lkt:5271:62>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5271'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image_390); 
--# expr-done 3689
--# property-call-start BasicDecl.get_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_110.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Var_Expr_110.Info);
--# end
--# expr-done 3690
Fld_5 := Fld_4.Value;
--# expr-done 3691
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 3692
Var_Expr_0 := Cast_Result_2;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 3693 '<FieldAccess for referenced_decl at nodes.lkt:5272:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5272'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Var_Expr_0.Node, Imprecise_Fallback => False, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3693
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
--# expr-start 3694 '<FieldAccess for synthesize_attribute_subprogram at nodes.lkt:5272:53>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5272'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.synthesize_attribute_subprogram
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Attr_Name => Attr_Name, E_Info => Ent.Info);
--# end
--# expr-done 3694
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   Result_Var_2 := Cast_Result_3;
end if;
   If_Result := Result_Var_2;
else
--# expr-start 3695 '<FieldAccess for synthesize_attribute_subprogram at nodes.lkt:5274:14>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5274'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.synthesize_attribute_subprogram
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Synthesize_Attribute_Subprogram (Node => Ent.Node, Attr_Name => Attr_Name, E_Info => Ent.Info);
--# end
--# expr-done 3695
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   If_Result := Cast_Result_4;
end if;
--# expr-done 3696
   If_Result_4 := If_Result;
end if;
--# expr-done 3697
         Property_Result := If_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Attribute_Subprogram;
--# end
--# property-start BaseTypeDecl.private_completion 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5279'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Private_Completion
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_1 : Bare_Base_Type_Decl;
Var_Expr_2 : Bare_Ada_Node;
Var_Expr_111 : Bare_Public_Part;
Var_Expr_112 : Bare_Base_Package_Decl;
Var_Expr_113 : Bare_Private_Part;
Fld : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Fld_3 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_4 : Symbol_Type;
Fld_5 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Private_Completion,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 3701 '<Cast to BasePackageDecl at nodes.lkt:5280:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
--# expr-start 3699 '<Cast to PublicPart at nodes.lkt:5280:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
--# expr-start 3698 '<FieldAccess for declarative_scope at nodes.lkt:5280:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3698
   if Fld = null
      or else Fld.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3699
Var_Expr_111 := Cast_Result;
if Var_Expr_111 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3700 '<parent at nodes.lkt:5280:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
      if Var_Expr_111 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_111.Parent; 
--# expr-done 3700
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Result_Var;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3701
Var_Expr_112 := Cast_Result_1;
if Var_Expr_112 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3702 '<FieldAccess for private_part at nodes.lkt:5280:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
      if Var_Expr_112 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_112.Base_Package_Decl_F_Private_Part;
--# expr-done 3702
Var_Expr_113 := Fld_2;
if Var_Expr_113 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3708 '<Find at nodes.lkt:5280:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
   --# expr-start 3703 '<FieldAccess for decls at nodes.lkt:5280:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5280'
      if Var_Expr_113 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_113.Declarative_Part_F_Decls;
--# expr-done 3703
      Find_Result := No_Bare_Ada_Node;
      if Fld_3 = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_3;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind d Item_0
--# expr-start 3704 '<Cast to BaseTypeDecl at nodes.lkt:5281:20>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5281'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 3704
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 3707 '<Eq at nodes.lkt:5282:25>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5282'
--# expr-start 3705 '<FieldAccess for name_symbol at nodes.lkt:5282:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5282'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_1);
--# end
--# expr-done 3705
--# expr-start 3706 '<FieldAccess for name_symbol at nodes.lkt:5282:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5282'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3706
Is_Equal := Fld_4 = Fld_5; 
--# expr-done 3707
Scope_Result_1 := Is_Equal;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
      if Result_Var_1 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var_1;
         end loop;
      end;
      end if;
--# expr-done 3708
Var_Expr_2 := Find_Result;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind t Var_Expr_2
--# expr-start 3713 '<New BaseTypeDecl.entity at nodes.lkt:5285:20>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5285'
--# expr-start 3711 '<New EntityInfo at nodes.lkt:5286:47>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5286'
--# expr-start 3710 '<FieldAccess for rebindings at nodes.lkt:5291:32>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5291'
--# expr-start 3709 '<FieldAccess for info at nodes.lkt:5291:32>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5291'
Fld_6 := Ent.Info;
--# expr-done 3709
Fld_7 := Fld_6.Rebindings;
--# expr-done 3710
New_Struct := (From_Rebound => False, Md => No_Metadata, Rebindings => Fld_7); 
--# expr-done 3711
--# expr-start 3712 '<Cast to BaseTypeDecl at nodes.lkt:5286:22>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5286'
Cast_Expr_1 := Var_Expr_2; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Cast_Expr_1;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 3712
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_3); 
--# expr-done 3713
Scope_Result_2 := New_Struct_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Private_Completion;
--# end
--# property-start BaseTypeDecl.is_array_or_rec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5298'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Or_Rec
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Is_A : Boolean;
Sym : Symbol_Type;
Fld_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3715 '<Not at nodes.lkt:5298:35>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5298'
--# expr-start 3714 '<IsNull at nodes.lkt:5298:39>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5298'
Is_Null := Self = null; 
--# expr-done 3714
Not_Val := not (Is_Null); 
--# expr-done 3715
if Not_Val then
--# expr-start 3716 '<FieldAccess for is_array at nodes.lkt:5299:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5299'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3716
if Fld then
   If_Result := True;
else
   --# expr-start 3717 '<FieldAccess for is_record_type at nodes.lkt:5299:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5299'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3717
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 3718 '<IsA BracketAggregate at nodes.lkt:5301:13>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5301'
Is_A := Origin /= null 
and then Origin.Kind in Ada_Bracket_Aggregate_Range; 
--# expr-done 3718
if Is_A then
   --# expr-start 3720 '<FieldAccess for has_aspect at nodes.lkt:5301:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5301'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3719 '<Symbol Aggregate at nodes.lkt:5301:60>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5301'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 3719
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 3720
   If_Result_1 := Fld_2;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Or_Rec;
--# end
--# property-start BaseTypeDecl.is_inherited_primitive 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5308'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Inherited_Primitive
  (Node : Bare_Base_Type_Decl
      ; P : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind p P
   Property_Result : Boolean;
      Fld : Bare_Base_Type_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 3726 '<Not at nodes.lkt:5309:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
--# expr-start 3725 '<Eq at nodes.lkt:5309:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
--# expr-start 3721 '<FieldAccess for node at nodes.lkt:5309:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
Fld := Ent.Node;
--# expr-done 3721
      Cast_Result := Fld;
--# expr-start 3724 '<FieldAccess for primitive at nodes.lkt:5309:22>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
--# expr-start 3723 '<FieldAccess for md at nodes.lkt:5309:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
--# expr-start 3722 '<FieldAccess for info at nodes.lkt:5309:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5309'
Fld_1 := P.Info;
--# expr-done 3722
Fld_2 := Fld_1.Md;
--# expr-done 3723
Fld_3 := Fld_2.Primitive;
--# expr-done 3724
Is_Equal := Cast_Result = Fld_3; 
--# expr-done 3725
Not_Val := not (Is_Equal); 
--# expr-done 3726
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Inherited_Primitive;
--# end
--# property-start BaseTypeDecl.get_record_representation_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Record_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Record_Rep_Clause
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Record_Rep_Clause;
      Var_Expr_1 : Internal_Entity_Record_Rep_Clause;
Var_Expr_114 : Bare_Declarative_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Record_Rep_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3735 '<Cast to RecordRepClause at nodes.lkt:5315:71>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
--# expr-start 3727 '<FieldAccess for declarative_scope at nodes.lkt:5315:71>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3727
Var_Expr_114 := Fld;
if Var_Expr_114 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3734 '<Find at nodes.lkt:5315:71>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
   --# expr-start 3729 '<as_entity at nodes.lkt:5315:71>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
--# expr-start 3728 '<FieldAccess for decls at nodes.lkt:5315:71>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5315'
      if Var_Expr_114 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_114.Declarative_Part_F_Decls;
--# expr-done 3728
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3729
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind d Item_0
--# expr-start 3730 '<Cast to RecordRepClause at nodes.lkt:5316:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5316'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 3730
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Record_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 3733 '<Eq at nodes.lkt:5316:48>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5316'
--# expr-start 3732 '<FieldAccess for referenced_decl at nodes.lkt:5316:48>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5316'
--# expr-start 3731 '<FieldAccess for name at nodes.lkt:5316:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5316'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Record_Rep_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 3731
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 3732
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 3733
Scope_Result := Is_Equiv;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
      end if;
--# expr-done 3734
Scope_Result_1 := Find_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Record_Rep_Clause
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_3 := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 3735
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Record_Representation_Clause;
--# end
--# property-start BaseTypeDecl.get_enum_representation_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Enum_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Rep_Clause
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Enum_Rep_Clause;
      Var_Expr_1 : Internal_Entity_Enum_Rep_Clause;
Var_Expr_115 : Bare_Declarative_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Rep_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3744 '<Cast to EnumRepClause at nodes.lkt:5323:67>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
--# expr-start 3736 '<FieldAccess for declarative_scope at nodes.lkt:5323:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3736
Var_Expr_115 := Fld;
if Var_Expr_115 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3743 '<Find at nodes.lkt:5323:67>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
   --# expr-start 3738 '<as_entity at nodes.lkt:5323:67>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
--# expr-start 3737 '<FieldAccess for decls at nodes.lkt:5323:67>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5323'
      if Var_Expr_115 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_115.Declarative_Part_F_Decls;
--# expr-done 3737
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3738
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind d Item_0
--# expr-start 3739 '<Cast to EnumRepClause at nodes.lkt:5324:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5324'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 3739
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Enum_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 3742 '<Eq at nodes.lkt:5325:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5325'
--# expr-start 3741 '<FieldAccess for referenced_decl at nodes.lkt:5325:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5325'
--# expr-start 3740 '<FieldAccess for type_name at nodes.lkt:5325:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5325'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Enum_Rep_Clause_F_Type_Name, Info => Var_Expr_1.Info);
--# expr-done 3740
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 3741
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 3742
Scope_Result := Is_Equiv;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
      end if;
--# expr-done 3743
Scope_Result_1 := Find_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_3 := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 3744
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Enum_Representation_Clause;
--# end
--# property-start BaseTypeDecl.direct_primitive_subps 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5333'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Direct_Primitive_Subps
  (Node : Bare_Base_Type_Decl
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_579 with Inline_Always;
            procedure Finalizer_Scope_2445 with Inline_Always;
            procedure Finalizer_Scope_2446 with Inline_Always;
            procedure Finalizer_Scope_2447 with Inline_Always;
            procedure Finalizer_Scope_2449 with Inline_Always;
            procedure Finalizer_Scope_2451 with Inline_Always;
            procedure Finalizer_Scope_2452 with Inline_Always;
            procedure Finalizer_Scope_2455 with Inline_Always;
            procedure Finalizer_Scope_2459 with Inline_Always;
            procedure Finalizer_Scope_2462 with Inline_Always;
            procedure Finalizer_Scope_2463 with Inline_Always;
      Local_Scope : Bare_Declarative_Part;
Local_Is_Derived_Tagged : Boolean;
Local_Decl_Parts : Internal_Entity_Array_Access;
Var_Expr_0 : Internal_Entity;
Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity;
Var_Expr_1 : Bare_Declarative_Part;
Local_Enum_Lits : Internal_Inner_Env_Assoc_Array_Access;
Var_Expr_2 : Bare_Enum_Type_Def;
Local_Prim_Subps : Internal_Inner_Env_Assoc_Array_Access;
Var_Expr_5 : Internal_Entity_Base_Subp_Spec;
Local_Bd : Internal_Entity_Basic_Decl;
Local_Predefined_Ops : Internal_Inner_Env_Assoc_Array_Access;
Var_Expr_116 : Internal_Entity_Type_Decl;
Var_Expr_117 : Bare_Declarative_Part;
Var_Expr_118 : Internal_Entity_Private_Part;
Var_Expr_119 : Bare_Type_Decl;
Var_Expr_120 : Internal_Entity_Basic_Decl;
Var_Expr_121 : Bare_Type_Decl;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_2 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_3 : Internal_Entity_Public_Part;
Fld_4 : Internal_Entity_Ada_Node_List;
Fld_5 : Bare_Ada_Node_List;
Fld_6 : Internal_Entity_Info;
Item_19 : Internal_Entity;
Bare_Item_19 : Bare_Ada_Node;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Ada_Node_List;
Fld_9 : Bare_Ada_Node_List;
Fld_10 : Internal_Entity_Info;
Item_20 : Internal_Entity;
Bare_Item_20 : Bare_Ada_Node;
As_Entity_3 : Internal_Entity;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result_1 : Internal_Entity_Array_Access;
Result_Var_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Fld_11 : Bare_Ada_Node;
Fld_12 : Bare_Ada_Node;
Cast_Result_1 : Bare_Declarative_Part;
As_Entity_4 : Internal_Entity_Declarative_Part;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity_Declarative_Part;
Fld_13 : Internal_Entity_Ada_Node_List;
Fld_14 : Bare_Ada_Node_List;
Fld_15 : Internal_Entity_Info;
Item_21 : Internal_Entity;
Bare_Item_21 : Bare_Ada_Node;
As_Entity_5 : Internal_Entity;
Is_Null_5 : Boolean;
If_Result_5 : Internal_Entity;
Map_Result_2 : Internal_Entity_Array_Access;
Scope_Result_2 : Internal_Entity_Array_Access;
Result_Var_2 : Internal_Entity_Array_Access;
If_Result_6 : Internal_Entity_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Cast_Result_3 : Internal_Entity;
Let_Result_1 : Internal_Entity_Array_Access;
Scope_Result_4 : Internal_Entity_Array_Access;
Match_Result : Internal_Entity_Array_Access;
Scope_Result_5 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
Scope_Result_6 : Internal_Entity_Array_Access;
Result_Var_4 : Internal_Entity_Array_Access;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result_4 : Bare_Type_Decl;
Fld_16 : Bare_Type_Def;
Scope_Result_7 : Bare_Type_Def;
Result_Var_5 : Bare_Type_Def;
Cast_Result_5 : Bare_Enum_Type_Def;
Fld_17 : Bare_Enum_Literal_Decl_List;
Item_3 : Bare_Enum_Literal_Decl;
Fld_18 : Bare_Defining_Name;
Fld_19 : Symbol_Type;
Cast_Expr_1 : Bare_Base_Type_Decl;
Cast_Result_6 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Expr_2 : Bare_Enum_Literal_Decl;
Cast_Result_7 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Map_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Scope_Result_8 : Internal_Inner_Env_Assoc_Array_Access;
Result_Var_6 : Internal_Inner_Env_Assoc_Array_Access;
Cast_Expr_3 : Internal_Entity;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Fld_20 : Internal_Entity_Base_Subp_Spec;
Fld_21 : Internal_Entity_Base_Type_Decl_Array_Access;
As_Entity_6 : Internal_Entity_Base_Type_Decl;
Is_Null_6 : Boolean;
If_Result_7 : Internal_Entity_Base_Type_Decl;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity;
Cast_Result_10 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_22 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity_7 : Internal_Entity_Base_Type_Decl;
Is_Null_7 : Boolean;
If_Result_8 : Internal_Entity_Base_Type_Decl;
Fld_23 : Lexical_Env := Empty_Env;
Fld_24 : Internal_Entity_Defining_Name;
Fld_25 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr_5 : Internal_Entity;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Fld_26 : Internal_Entity_Base_Subp_Spec;
Fld_27 : Boolean;
Quantifier_Result_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_28 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_7 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_8 : Boolean;
Map_Result_4 : Internal_Entity_Array_Access;
Cast_Expr_6 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Fld_29 : Internal_Entity_Defining_Name;
Fld_30 : Symbol_Type;
Fld_31 : Bare_Basic_Decl;
Cast_Expr_7 : Bare_Base_Type_Decl;
Cast_Result_13 : Bare_Ada_Node;
New_Struct_2 : Internal_Metadata;
Cast_Result_14 : Bare_Ada_Node;
New_Struct_3 : Internal_Inner_Env_Assoc;
Array_Lit : Internal_Inner_Env_Assoc_Array_Access;
Sym : Symbol_Type;
Fld_32 : Bare_Basic_Decl;
Cast_Expr_8 : Bare_Base_Type_Decl;
Cast_Result_15 : Bare_Ada_Node;
New_Struct_4 : Internal_Metadata;
Cast_Result_16 : Bare_Ada_Node;
New_Struct_5 : Internal_Inner_Env_Assoc;
Array_Lit_1 : Internal_Inner_Env_Assoc_Array_Access;
Fld_33 : Internal_Entity_Defining_Name;
Sym_1 : Symbol_Type;
Fld_34 : Boolean;
If_Result_11 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
Map_Result_5 : Internal_Inner_Env_Assoc_Array_Access;
Cast_Expr_9 : Bare_Base_Type_Decl;
Cast_Result_17 : Bare_Type_Decl;
As_Entity_8 : Internal_Entity_Type_Decl;
Is_Null_8 : Boolean;
If_Result_12 : Internal_Entity_Type_Decl;
Fld_35 : Internal_Env_Assoc_Array_Access;
Fld_36 : Symbol_Type;
Fld_37 : Bare_Ada_Node;
Cast_Expr_10 : Bare_Base_Type_Decl;
Cast_Result_18 : Bare_Ada_Node;
New_Struct_6 : Internal_Metadata;
New_Struct_7 : Internal_Inner_Env_Assoc;
Map_Result_6 : Internal_Inner_Env_Assoc_Array_Access;
Scope_Result_11 : Internal_Inner_Env_Assoc_Array_Access;
Result_Var_9 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_4 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_5 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_6 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_7 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_8 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_579 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Local_Decl_Parts);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Local_Enum_Lits);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Local_Prim_Subps);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_9);
                     Dec_Ref (Local_Predefined_Ops);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_579;
            procedure Finalizer_Scope_2445 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_2445;
            procedure Finalizer_Scope_2446 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2446;
            procedure Finalizer_Scope_2447 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2447;
            procedure Finalizer_Scope_2449 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2449;
            procedure Finalizer_Scope_2451 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_6);
            end Finalizer_Scope_2451;
            procedure Finalizer_Scope_2452 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2452;
            procedure Finalizer_Scope_2455 is
            begin
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_2455;
            procedure Finalizer_Scope_2459 is
            begin
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2459;
            procedure Finalizer_Scope_2462 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_2462;
            procedure Finalizer_Scope_2463 is
            begin
                     Dec_Ref (Fld_35);
                     Dec_Ref (Map_Result_6);
            end Finalizer_Scope_2463;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitive_Subps is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitive_Subps;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitive_Subps,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3827 '<Let at nodes.lkt:5334:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5334'
--# scope-start
--# expr-start 3745 '<FieldAccess for declarative_scope at nodes.lkt:5334:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5334'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 3745
Local_Scope := Fld; 
--# bind scope Local_Scope
--# expr-start 3826 '<Let at nodes.lkt:5335:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5335'
--# scope-start
--# expr-start 3747 '<Cast to TypeDecl at nodes.lkt:5335:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5335'
--# expr-start 3746 '<as_bare_entity at nodes.lkt:5335:33>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5335'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3746
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3747
Var_Expr_116 := Cast_Result;
if Var_Expr_116 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 3748 '<FieldAccess for is_derived_tagged_type at nodes.lkt:5335:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5335'
      if Var_Expr_116.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.is_derived_tagged_type
Fld_1 := Libadalang.Implementation.Type_Decl_P_Is_Derived_Tagged_Type (Node => Var_Expr_116.Node, E_Info => Var_Expr_116.Info);
--# end
--# expr-done 3748
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Local_Is_Derived_Tagged := Result_Var; 
--# bind is_derived_tagged Local_Is_Derived_Tagged
--# expr-start 3825 '<Let at nodes.lkt:5336:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5336'
--# scope-start
Var_Expr_117 := Local_Scope;
if Var_Expr_117 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3750 '<as_bare_entity at nodes.lkt:5336:26>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5336'
--# expr-start 3749 '<parent at nodes.lkt:5336:26>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5336'
      if Var_Expr_117 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_117.Parent; 
--# expr-done 3749
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3750
Var_Expr_0 := If_Result_1;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 3764 '<Match at nodes.lkt:5337:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5337'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind pkg_decl Match_0
--# expr-start 3753 '<TakeWhile at ???>' Map_Result None
   --# expr-start 3752 '<FieldAccess for decls at nodes.lkt:5341:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5341'
--# expr-start 3751 '<FieldAccess for public_part at nodes.lkt:5341:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5341'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Public_Part (Node => Match_0.Node.Base_Package_Decl_F_Public_Part, Info => Match_0.Info);
--# expr-done 3751
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Ada_Node_List (Node => Fld_3.Node.Declarative_Part_F_Decls, Info => Fld_3.Info);
--# expr-done 3752
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_5;
      begin
         for Untyped_Item_19 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_19 := Untyped_Item_19; 
Is_Null_2 := Bare_Item_19 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity_2 := (Info => Fld_6, Node => Bare_Item_19); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_19 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_19;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3753
--# expr-start 3754 '<FieldAccess for private_part at nodes.lkt:5341:57>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5341'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 3754
Var_Expr_118 := Fld_7;
if Var_Expr_118 /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 3756 '<TakeWhile at ???>' Map_Result_1 None
   --# expr-start 3755 '<FieldAccess for decls at nodes.lkt:5341:57>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5341'
      if Var_Expr_118.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_118.Node.Declarative_Part_F_Decls, Info => Var_Expr_118.Info);
--# expr-done 3755
Fld_9 := Fld_8.Node;
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_9 = null then
      Map_Result_1 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_9;
      begin
         for Untyped_Item_20 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_20 := Untyped_Item_20; 
Is_Null_3 := Bare_Item_20 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity;
else
Fld_10 := Fld_8.Info;
As_Entity_3 := (Info => Fld_10, Node => Bare_Item_20); 
   If_Result_3 := As_Entity_3;
end if;
                  Item_20 := If_Result_3; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_20;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3756
Scope_Result_1 := Map_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2449;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result := Concat (Map_Result, Result_Var_1); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2447;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Synthetic_Formal_Param_Decl | Ada_Discrete_Base_Subtype_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
--# expr-start 3763 '<If at nodes.lkt:5346:17>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5346'
if Local_Is_Derived_Tagged then
--# expr-start 3759 '<Cast to DeclarativePart at nodes.lkt:5346:43>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5346'
--# expr-start 3758 '<parent at nodes.lkt:5346:43>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5346'
--# expr-start 3757 '<parent at nodes.lkt:5346:43>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5346'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Parent; 
--# expr-done 3757
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.Parent; 
--# expr-done 3758
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result_1 := Fld_12;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3759
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dp Var_Expr_1
--# expr-start 3762 '<TakeWhile at ???>' Map_Result_2 None
   --# expr-start 3761 '<FieldAccess for decls at nodes.lkt:5347:29>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5347'
--# expr-start 3760 '<as_bare_entity at nodes.lkt:5347:29>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5347'
Is_Null_4 := Var_Expr_1 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity_Declarative_Part;
else
As_Entity_4 := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result_4 := As_Entity_4;
end if;
--# expr-done 3760
      if If_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Ada_Node_List (Node => If_Result_4.Node.Declarative_Part_F_Decls, Info => If_Result_4.Info);
--# expr-done 3761
Fld_14 := Fld_13.Node;
      declare
         Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_14 = null then
      Map_Result_2 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_14;
      begin
         for Untyped_Item_21 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_21 := Untyped_Item_21; 
Is_Null_5 := Bare_Item_21 = null; 
if Is_Null_5 then
   If_Result_5 := No_Entity;
else
Fld_15 := Fld_13.Info;
As_Entity_5 := (Info => Fld_15, Node => Bare_Item_21); 
   If_Result_5 := As_Entity_5;
end if;
                  Item_21 := If_Result_5; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_21;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 3762
Scope_Result_2 := Map_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2452;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_6 := Result_Var_2;
else
   If_Result_6 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 3763
Let_Result_1 := If_Result_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_4 := Let_Result_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2451;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 3764
Scope_Result_5 := Match_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2446;
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Scope_Result_6 := Result_Var_3;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2445;
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Local_Decl_Parts := Result_Var_4; Inc_Ref (Local_Decl_Parts);
--# bind decl_parts Local_Decl_Parts
--# expr-start 3824 '<Let at nodes.lkt:5351:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5351'
--# scope-start
--# expr-start 3767 '<Cast to EnumTypeDef at nodes.lkt:5351:25>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5351'
--# expr-start 3765 '<Cast to TypeDecl at nodes.lkt:5351:25>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5351'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Type_Decl
   then
      Cast_Result_4 := Cast_Expr;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 3765
Var_Expr_119 := Cast_Result_4;
if Var_Expr_119 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3766 '<FieldAccess for type_def at nodes.lkt:5351:25>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5351'
      if Var_Expr_119 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Var_Expr_119.Type_Decl_F_Type_Def;
--# expr-done 3766
Scope_Result_7 := Fld_16;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := No_Bare_Ada_Node;
end if;
   if Result_Var_5 = null
      or else Result_Var_5.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_5 := Result_Var_5;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 3767
Var_Expr_2 := Cast_Result_5;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind etf Var_Expr_2
--# expr-start 3773 '<Map at nodes.lkt:5352:22>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5352'
   --# expr-start 3768 '<FieldAccess for enum_literals at nodes.lkt:5352:22>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5352'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Var_Expr_2.Enum_Type_Def_F_Enum_Literals;
--# expr-done 3768
      declare
         Map_Result_3_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      if Fld_17 = null then
      Map_Result_3 := Create_Internal_Inner_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_17;
      begin
         for Untyped_Item_3 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_3 := Untyped_Item_3; 
   --# scope-start
               --# bind lit Item_3
      --# expr-start 3772 '<New InnerEnvAssoc at nodes.lkt:5353:26>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5353'
--# expr-start 3770 '<FieldAccess for name_symbol at nodes.lkt:5354:25>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5354'
--# expr-start 3769 '<FieldAccess for name at nodes.lkt:5354:25>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5354'
      if Item_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Item_3.Enum_Literal_Decl_F_Name;
--# expr-done 3769
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18);
--# end
--# expr-done 3770
--# expr-start 3771 '<New Metadata at nodes.lkt:5354:69>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5354'
Cast_Expr_1 := Self; 
      Cast_Result_6 := Cast_Expr_1;
New_Struct := (Dottable_Subp => False, Primitive => Cast_Result_6, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3771
Cast_Expr_2 := Item_3; 
      Cast_Result_7 := Cast_Expr_2;
New_Struct_1 := (Key => Fld_19, Metadata => New_Struct, Rebindings => null, Value => Cast_Result_7); 
--# expr-done 3772
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_3 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 3773
Scope_Result_8 := Map_Result_3;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2455;
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Local_Enum_Lits := Result_Var_6; Inc_Ref (Local_Enum_Lits);
--# bind enum_lits Local_Enum_Lits
--# expr-start 3823 '<Let at nodes.lkt:5358:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5358'
--# scope-start
--# expr-start 3813 '<Mapcat at nodes.lkt:5358:26>' Map_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5358'
   --# expr-start 3795 '<Map at nodes.lkt:5358:26>' Map_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5358'
      declare
         Map_Result_4_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Decl_Parts;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_4
--# expr-start 3774 '<Cast to BasicDecl at nodes.lkt:5359:23>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5359'
Cast_Expr_3 := Item_4; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_8 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3774
Var_Expr_120 := Cast_Result_8;
if Var_Expr_120 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3775 '<FieldAccess for subp_spec_or_null at nodes.lkt:5359:23>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5359'
      if Var_Expr_120.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_20 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_120.Node, E_Info => Var_Expr_120.Info);
--# end
--# expr-done 3775
Var_Expr_5 := Fld_20;
if Var_Expr_5 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_5
--# expr-start 3779 '<Contains at nodes.lkt:5360:27>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5360'
   --# expr-start 3776 '<FieldAccess for candidate_primitive_subp_types at nodes.lkt:5360:27>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5360'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.candidate_primitive_subp_types
Fld_21 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Primitive_Subp_Types (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 3776
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_21;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_10
      --# expr-start 3778 '<Eq at ???>' Is_Equiv None
Cast_Expr_4 := Item_10; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
--# expr-start 3777 '<as_bare_entity at nodes.lkt:5360:74>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5360'
Is_Null_6 := Self = null; 
if Is_Null_6 then
   If_Result_7 := No_Entity_Base_Type_Decl;
else
As_Entity_6 := (Info => No_Entity_Info, Node => Self); 
   If_Result_7 := As_Entity_6;
end if;
--# expr-done 3777
      Cast_Result_10 := Create_Internal_Entity
        (Node => If_Result_7.Node,
         Info => If_Result_7.Info);
Is_Equiv := Equivalent (Cast_Result_9, Cast_Result_10); 
--# expr-done 3778
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3779
if Quantifier_Result then
   --# expr-start 3794 '<If at nodes.lkt:5367:21>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5367'
--# expr-start 3782 '<Not at nodes.lkt:5367:24>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5367'
--# expr-start 3781 '<IsA BasePackageDecl at nodes.lkt:5367:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5367'
--# expr-start 3780 '<parent at nodes.lkt:5367:28>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5367'
      if Local_Scope = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Local_Scope.Parent; 
--# expr-done 3780
Is_A := Fld_28 /= null 
and then Fld_28.Kind in Ada_Base_Package_Decl; 
--# expr-done 3781
Not_Val := not (Is_A); 
--# expr-done 3782
if Not_Val then
   --# expr-start 3793 '<bind at nodes.lkt:5368:25>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5368'
--# expr-start 3783 '<FieldAccess for origin_node at nodes.lkt:5368:39>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5368'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_22 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Var_Expr_5.Node);
--# end
--# expr-done 3783
Bound_Origin := Fld_22; 
--# expr-start 3792 '<AnyQuantifier at nodes.lkt:5370:25>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
   --# expr-start 3788 '<EnvGet at nodes.lkt:5370:25>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
--# expr-start 3785 '<FieldAccess for parent_primitives_env at nodes.lkt:5370:25>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
--# expr-start 3784 '<as_bare_entity at nodes.lkt:5370:25>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
Is_Null_7 := Self = null; 
if Is_Null_7 then
   If_Result_8 := No_Entity_Base_Type_Decl;
else
As_Entity_7 := (Info => No_Entity_Info, Node => Self); 
   If_Result_8 := As_Entity_7;
end if;
--# expr-done 3784
      if If_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.parent_primitives_env'
Fld_23 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Parent_Primitives_Env (Node => If_Result_8.Node, E_Info => If_Result_8.Info);
--# end
--# expr-done 3785
--# expr-start 3787 '<FieldAccess for name_symbol at nodes.lkt:5370:73>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
--# expr-start 3786 '<FieldAccess for name at nodes.lkt:5370:73>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5370'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_24 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 3786
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 3787
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_23, Key => Thin (Fld_25), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3788
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind x Item_6
      --# expr-start 3791 '<FieldAccess for match_signature at nodes.lkt:5371:36>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5371'
--# expr-start 3790 '<FieldAccess for subp_spec_or_null at nodes.lkt:5371:36>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5371'
--# expr-start 3789 '<Cast to BasicDecl at nodes.lkt:5371:36>' Cast_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5371'
Cast_Expr_5 := Item_6; 
   if Cast_Expr_5.Node = null
      or else Cast_Expr_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
   else
         Cast_Result_11 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3789
      if Cast_Result_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_11.Node, E_Info => Cast_Result_11.Info);
--# end
--# expr-done 3790
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_27 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_26.Node, Other => Var_Expr_5, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => Fld_26.Info);
--# end
--# expr-done 3791
      Quantifier_Result_1 := Fld_27;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 3792
Dyn_Var_Bind_Result := Quantifier_Result_1; 
--# expr-done 3793
   If_Result_9 := Dyn_Var_Bind_Result;
else
   If_Result_9 := True;
end if;
--# expr-done 3794
   If_Result_10 := If_Result_9;
else
   If_Result_10 := False;
end if;
Scope_Result_9 := If_Result_10;
   --# end
      Finalizer_Scope_2459;
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := False;
end if;
Scope_Result_10 := Result_Var_7;
   --# end
   Result_Var_8 := Scope_Result_10;
else
   Result_Var_8 := False;
end if;
         if Result_Var_8 then
         declare
            Item_To_Append : constant Internal_Entity := Item_4;
         begin
            Internal_Entity_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_4 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_4_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 3795
      declare
         Map_Result_5_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_4;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_7
      --# expr-start 3812 '<Let at nodes.lkt:5380:17>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5380'
--# scope-start
--# expr-start 3796 '<Cast to BasicDecl at nodes.lkt:5380:26>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5380'
Cast_Expr_6 := Item_7; 
   if Cast_Expr_6.Node = null
      or else Cast_Expr_6.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   else
         Cast_Result_12 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3796
Local_Bd := Cast_Result_12; 
--# bind bd Local_Bd
--# expr-start 3802 '<ArrayLiteral at nodes.lkt:5382:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5382'
--# expr-start 3801 '<New InnerEnvAssoc at nodes.lkt:5382:18>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5382'
--# expr-start 3798 '<FieldAccess for name_symbol at nodes.lkt:5383:25>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5383'
--# expr-start 3797 '<FieldAccess for defining_name at nodes.lkt:5383:25>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5383'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_29 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 3797
      if Fld_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_30 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_29.Node);
--# end
--# expr-done 3798
--# expr-start 3799 '<New Metadata at nodes.lkt:5383:83>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5383'
Cast_Expr_7 := Self; 
      Cast_Result_13 := Cast_Expr_7;
New_Struct_2 := (Dottable_Subp => False, Primitive => Cast_Result_13, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3799
--# expr-start 3800 '<FieldAccess for node at nodes.lkt:5383:65>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5383'
Fld_31 := Local_Bd.Node;
--# expr-done 3800
      Cast_Result_14 := Fld_31;
New_Struct_3 := (Key => Fld_30, Metadata => New_Struct_2, Rebindings => null, Value => Cast_Result_14); 
--# expr-done 3801
Array_Lit := Create_Internal_Inner_Env_Assoc_Array (Internal_Internal_Inner_Env_Assoc_Array'(1 => New_Struct_3)); 
--# expr-done 3802
--# expr-start 3811 '<If at nodes.lkt:5385:21>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
--# expr-start 3805 '<FieldAccess for name_is at nodes.lkt:5385:24>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
--# expr-start 3803 '<FieldAccess for defining_name at nodes.lkt:5385:24>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_33 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 3803
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3804 '<Symbol "=" at nodes.lkt:5385:51>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 3804
--# property-call-start Name.name_is
Fld_34 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_33.Node, Sym => Sym_1);
--# end
--# expr-done 3805
if Fld_34 then
   --# expr-start 3810 '<ArrayLiteral at nodes.lkt:5385:66>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
--# expr-start 3809 '<New InnerEnvAssoc at nodes.lkt:5385:67>' New_Struct_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5385'
--# expr-start 3806 '<Symbol "/=" at nodes.lkt:5386:29>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5386'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 3806
--# expr-start 3807 '<New Metadata at nodes.lkt:5386:64>' New_Struct_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5386'
Cast_Expr_8 := Self; 
      Cast_Result_15 := Cast_Expr_8;
New_Struct_4 := (Dottable_Subp => False, Primitive => Cast_Result_15, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3807
--# expr-start 3808 '<FieldAccess for node at nodes.lkt:5386:46>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5386'
Fld_32 := Local_Bd.Node;
--# expr-done 3808
      Cast_Result_16 := Fld_32;
New_Struct_5 := (Key => Sym, Metadata => New_Struct_4, Rebindings => null, Value => Cast_Result_16); 
--# expr-done 3809
Array_Lit_1 := Create_Internal_Inner_Env_Assoc_Array (Internal_Internal_Inner_Env_Assoc_Array'(1 => New_Struct_5)); 
--# expr-done 3810
   If_Result_11 := Array_Lit_1;
else
   If_Result_11 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 3811
Concat_Result_1 := Concat (Array_Lit, If_Result_11); 
Let_Result_2 := Concat_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 3812
         for Item_To_Append of
               Let_Result_2.Items
         loop
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2462;
         end loop;
      end;
         Map_Result_5 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_5_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 3813
Local_Prim_Subps := Map_Result_5; Inc_Ref (Local_Prim_Subps);
--# bind prim_subps Local_Prim_Subps
--# expr-start 3822 '<Let at nodes.lkt:5392:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5392'
--# scope-start
--# expr-start 3814 '<Cast to TypeDecl at nodes.lkt:5392:30>' Cast_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5392'
Cast_Expr_9 := Self; 
   if Cast_Expr_9 = null
      or else Cast_Expr_9.Kind in Ada_Type_Decl
   then
      Cast_Result_17 := Cast_Expr_9;
   else
         Cast_Result_17 := No_Bare_Ada_Node;
   end if;
--# expr-done 3814
Var_Expr_121 := Cast_Result_17;
if Var_Expr_121 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3821 '<Map at nodes.lkt:5392:30>' Map_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5392'
   --# expr-start 3816 '<FieldAccess for predefined_operators at nodes.lkt:5392:30>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5392'
--# expr-start 3815 '<as_bare_entity at nodes.lkt:5392:30>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5392'
Is_Null_8 := Var_Expr_121 = null; 
if Is_Null_8 then
   If_Result_12 := No_Entity_Type_Decl;
else
As_Entity_8 := (Info => No_Entity_Info, Node => Var_Expr_121); 
   If_Result_12 := As_Entity_8;
end if;
--# expr-done 3815
      if If_Result_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.predefined_operators
Fld_35 := Libadalang.Implementation.Type_Decl_P_Predefined_Operators (Node => If_Result_12.Node, E_Info => If_Result_12.Info);
--# end
--# expr-done 3816
      declare
         Map_Result_6_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Env_Assoc_Array_Access := Fld_35;
      begin
         for Item_8 of
               Collection.Items
         loop
   --# scope-start
               --# bind assoc Item_8
      --# expr-start 3820 '<New InnerEnvAssoc at nodes.lkt:5393:24>' New_Struct_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5393'
--# expr-start 3817 '<FieldAccess for key at nodes.lkt:5394:21>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5394'
Fld_36 := Item_8.Key;
--# expr-done 3817
--# expr-start 3818 '<New Metadata at nodes.lkt:5394:60>' New_Struct_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5394'
Cast_Expr_10 := Self; 
      Cast_Result_18 := Cast_Expr_10;
New_Struct_6 := (Dottable_Subp => False, Primitive => Cast_Result_18, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3818
--# expr-start 3819 '<FieldAccess for value at nodes.lkt:5394:38>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5394'
Fld_37 := Item_8.Value;
--# expr-done 3819
New_Struct_7 := (Key => Fld_36, Metadata => New_Struct_6, Rebindings => null, Value => Fld_37); 
--# expr-done 3820
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_7;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_6_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_6 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_6_Vec)));
         for I in Map_Result_6.Items'Range loop
            Map_Result_6.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_6_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_6.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_6_Vec);
      end;
--# expr-done 3821
Scope_Result_11 := Map_Result_6;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_2463;
   Result_Var_9 := Scope_Result_11;
else
   Result_Var_9 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var_9);
Local_Predefined_Ops := Result_Var_9; Inc_Ref (Local_Predefined_Ops);
--# bind predefined_ops Local_Predefined_Ops
Concat_Result_2 := Concat (Local_Enum_Lits, Local_Prim_Subps); 
Concat_Result_3 := Concat (Concat_Result_2, Local_Predefined_Ops); 
Let_Result_3 := Concat_Result_3; Inc_Ref (Let_Result_3);
--# end
--# expr-done 3822
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 3823
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 3824
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 3825
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 3826
Let_Result_8 := Let_Result_7; Inc_Ref (Let_Result_8);
--# end
--# expr-done 3827
         Property_Result := Let_Result_8;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_579;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_579;
                     Finalizer_Scope_2445;
                     Finalizer_Scope_2446;
                     Finalizer_Scope_2447;
                     Finalizer_Scope_2449;
                     Finalizer_Scope_2451;
                     Finalizer_Scope_2452;
                     Finalizer_Scope_2455;
                     Finalizer_Scope_2459;
                     Finalizer_Scope_2462;
                     Finalizer_Scope_2463;
               Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitive_Subps :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitive_Subps := Initialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitive_Subps := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Base_Type_Decl_F_Direct_Primitive_Subps;
--# end
--# property-start BaseTypeDecl.own_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5403'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Own_Primitives_Env
  (Node : Bare_Base_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_580 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Rebound_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_580 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Rebound_Env);
            end Finalizer_Scope_580;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3829 '<rebind_env at nodes.lkt:5404:9>' Rebound_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5404'
--# expr-start 3828 '<FieldAccess for direct_primitives_env at nodes.lkt:5404:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5404'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.direct_primitives_env
Fld := Libadalang.Implementation.Base_Type_Decl_F_Direct_Primitives_Env (Node => Ent.Node);
--# end
--# expr-done 3828
Rebound_Env := Rebind_Env (Fld, With_Rebindings); 
--# expr-done 3829
         Property_Result := Rebound_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_580;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_580;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Own_Primitives_Env;
--# end
--# property-start BaseTypeDecl.own_primitives_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5409'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Own_Primitives_Envs
  (Node : Bare_Base_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_581 with Inline_Always;
            procedure Finalizer_Scope_2465 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Scope_Result : Lexical_Env_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_581 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_581;
            procedure Finalizer_Scope_2465 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2465;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3831 '<Cast to TypeDecl at nodes.lkt:5411:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5411'
--# expr-start 3830 '<FieldAccess for previous_part at nodes.lkt:5411:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5411'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 3830
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3831
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 3834 '<ArrayLiteral at nodes.lkt:5412:21>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5412'
--# expr-start 3832 '<FieldAccess for own_primitives_env at nodes.lkt:5412:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5412'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.own_primitives_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3832
--# expr-start 3833 '<FieldAccess for own_primitives_env at nodes.lkt:5412:64>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5412'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.own_primitives_env
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Own_Primitives_Env (Node => Var_Expr_0.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 3833
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_2)); 
--# expr-done 3834
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2465;
   Result_Var := Scope_Result;
else
   --# expr-start 3836 '<ArrayLiteral at nodes.lkt:5412:117>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5412'
--# expr-start 3835 '<FieldAccess for own_primitives_env at nodes.lkt:5412:118>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5412'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.own_primitives_env
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3835
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3)); 
--# expr-done 3836
   Result_Var := Array_Lit_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_581;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_581;
                     Finalizer_Scope_2465;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Own_Primitives_Envs;
--# end
--# property-start BaseTypeDecl.primitives_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5420'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Primitives_Envs
  (Node : Bare_Base_Type_Decl
      ; With_Rebindings : Env_Rebindings
      ; Stop_At : Internal_Entity_Base_Type_Decl_Array_Access
      ; Include_Self : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   --# bind stop_at Stop_At
   --# bind include_self Include_Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_582 with Inline_Always;
            procedure Finalizer_Scope_2466 with Inline_Always;
            procedure Finalizer_Scope_2471 with Inline_Always;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Base_Subtype_Decl;
Match_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Bare_Base_Subtype_Decl;
Fld_2 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Decl;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Task_Type_Decl;
Cast_Expr_2 : Internal_Entity_Task_Type_Decl;
Cast_Result_5 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity_Base_Subtype_Decl;
Cast_Result_7 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_9 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env_Array_Access;
Fld_5 : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Lexical_Env_Array_Access;
Scope_Result_4 : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Map_Result : Lexical_Env_Array_Access;
Fld_6 : Lexical_Env_Array_Access;
If_Result_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result_1 : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_582 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_582;
            procedure Finalizer_Scope_2466 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2466;
            procedure Finalizer_Scope_2471 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2471;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3852 '<bind at nodes.lkt:5424:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5424'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 3849 '<Mapcat at nodes.lkt:5426:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5426'
   --# expr-start 3837 '<FieldAccess for base_types at nodes.lkt:5426:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5426'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3837
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
--# expr-start 3843 '<Match at nodes.lkt:5427:24>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5427'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind td Match_0
Cast_Expr_1 := Match_0; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result := Cast_Result_3; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind ttd Match_1
Cast_Expr_2 := Match_1; 
      Cast_Result_5 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Let_Result_1 := Cast_Result_5; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind std Match_2
--# expr-start 3842 '<bind at nodes.lkt:5431:25>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5431'
--# expr-start 3839 '<FieldAccess for origin_node at nodes.lkt:5431:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5431'
--# expr-start 3838 '<FieldAccess for node at nodes.lkt:5431:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5431'
Fld_1 := Match_2.Node;
--# expr-done 3838
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_1);
--# end
--# expr-done 3839
Bound_Origin_1 := Fld_2; 
--# expr-start 3841 '<Cast to TypeDecl at nodes.lkt:5433:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5433'
--# expr-start 3840 '<FieldAccess for get_type at nodes.lkt:5433:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5433'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_2.Node, Origin => Bound_Origin_1, E_Info => Match_2.Info);
--# end
--# expr-done 3840
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 3841
Dyn_Var_Bind_Result := Cast_Result_1; 
--# expr-done 3842
      Cast_Result_7 := Create_Internal_Entity_Base_Type_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
Let_Result_2 := Cast_Result_7; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Classwide_Type_Decl .. Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_8; 
--# bind _ Match_3
Cast_Expr_3 := No_Entity_Type_Decl; 
      Cast_Result_9 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Let_Result_3 := Cast_Result_9; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 3843
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_1
--# expr-start 3848 '<If at nodes.lkt:5437:29>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5437'
--# expr-start 3845 '<Contains at nodes.lkt:5437:32>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5437'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Stop_At;
      begin
         for Item_11 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_11
      --# expr-start 3844 '<Eq at ???>' Is_Equiv None
Cast_Expr_4 := Item_11; 
      Cast_Result_10 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Cast_Expr_5 := Var_Expr_1; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Is_Equiv := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 3844
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 3845
if Quantifier_Result then
   If_Result := No_Lexical_Env_Array_Type;
else
   --# expr-start 3846 '<FieldAccess for own_primitives_envs at nodes.lkt:5437:87>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5437'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.own_primitives_envs
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Own_Primitives_Envs (Node => Var_Expr_1.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3846
--# expr-start 3847 '<FieldAccess for primitives_envs at nodes.lkt:5437:129>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5437'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.primitives_envs
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Primitives_Envs (Node => Var_Expr_1.Node, With_Rebindings => With_Rebindings, Stop_At => Stop_At, Include_Self => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3847
Concat_Result := Concat (Fld_4, Fld_5); 
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 3848
Scope_Result_4 := If_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2471;
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2466;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3849
--# expr-start 3851 '<If at nodes.lkt:5440:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5440'
if Include_Self then
   --# expr-start 3850 '<FieldAccess for own_primitives_envs at nodes.lkt:5440:38>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5440'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.own_primitives_envs
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Own_Primitives_Envs (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3850
   If_Result_1 := Fld_6;
else
   If_Result_1 := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3851
Concat_Result_1 := Concat (Map_Result, If_Result_1); 
Dyn_Var_Bind_Result_1 := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 3852
         Property_Result := Dyn_Var_Bind_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_582;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_582;
                     Finalizer_Scope_2466;
                     Finalizer_Scope_2471;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Primitives_Envs;
--# end
--# property-start BaseTypeDecl.compute_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5447'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Compute_Primitives_Env
  (Node : Bare_Base_Type_Decl
      ; Include_Self : Boolean
         := True
      ; Stop_At : Internal_Entity_Base_Type_Decl_Array_Access
         := No_Internal_Entity_Base_Type_Decl_Array_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_self Include_Self
   --# bind stop_at Stop_At
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_583 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env_Array_Access;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_583 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_583;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Compute_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Self);
               Mmz_K.Items (3) := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                                      As_Internal_Entity_Base_Type_Decl_Array_Access => Stop_At);
                  Inc_Ref (Mmz_K.Items (3).As_Internal_Entity_Base_Type_Decl_Array_Access);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 3857 '<env_group at nodes.lkt:5447:147>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5447'
--# expr-start 3855 '<FieldAccess for primitives_envs at nodes.lkt:5447:147>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5447'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3854 '<FieldAccess for rebindings at nodes.lkt:5448:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5448'
--# expr-start 3853 '<FieldAccess for info at nodes.lkt:5448:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5448'
Fld := Ent.Info;
--# expr-done 3853
Fld_1 := Fld.Rebindings;
--# expr-done 3854
--# property-call-start BaseTypeDecl.primitives_envs
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Primitives_Envs (Node => Ent.Node, With_Rebindings => Fld_1, Stop_At => Stop_At, Include_Self => Include_Self, E_Info => Ent.Info);
--# end
--# expr-done 3855
--# expr-start 3856 '<New Metadata at nodes.lkt:5450:17>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5450'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => Cast_Result); 
--# expr-done 3856
Group_Env := Group (Fld_2, New_Struct); 
--# expr-done 3857
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_583;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_583;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Compute_Primitives_Env;
--# end
--# property-start BaseTypeDecl.direct_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5456'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Direct_Primitives_Env
  (Node : Bare_Base_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_584 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_584 is
            begin
                     Dec_Ref (Dyn_Env);
            end Finalizer_Scope_584;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitives_Env is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitives_Env;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitives_Env,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3858 '<DynamicLexicalEnv at nodes.lkt:5457:9>' Dyn_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5457'
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Base_Type_Decl_F_Direct_Primitive_Subps'Access, null, False, Self.Unit.Context.Symbols); 
--# expr-done 3858
         Property_Result := Dyn_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_584;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_584;
               Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitives_Env :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Base_Type_Decl_Lf_State_Direct_Primitives_Env := Initialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitives_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Base_Type_Decl_F_Direct_Primitives_Env;
--# end
--# property-start BaseTypeDecl.get_primitives 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5465'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Primitives
  (Node : Bare_Base_Type_Decl
      ; Only_Inherited : Boolean
         := False
      ; Include_Predefined_Operators : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind only_inherited Only_Inherited
   --# bind include_predefined_operators Include_Predefined_Operators
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_585 with Inline_Always;
            procedure Finalizer_Scope_2474 with Inline_Always;
      Local_Prim_Env : Lexical_Env := Empty_Env;
Local_All_Prim_Names : Symbol_Type_Array_Access;
Local_All_Prims : Internal_Entity_Basic_Decl_Array_Access;
Local_Bds : Internal_Entity_Basic_Decl_Array_Access;
Local_A_Spec : Internal_Entity_Base_Subp_Spec;
Local_A_Prim : Internal_Entity_Base_Type_Decl;
Local_B_Prim : Bare_Base_Type_Decl;
Local_B_Prim_Ent : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Unique_Array : Symbol_Type_Array_Access;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Index_5 : Integer;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Index_7 : Integer;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Bare_Ada_Node;
Cast_Result_3 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
Fld_11 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Fld_13 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_3 : Boolean;
As_Entity_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Comp_Result : Boolean;
Fld_14 : Bare_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_5 : Boolean;
Comp_Result_1 : Boolean;
Fld_16 : Bare_Base_Type_Decl;
Fld_17 : Boolean;
If_Result_6 : Boolean;
Fld_18 : Bare_Base_Type_Decl;
Fld_19 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
If_Result_8 : Boolean;
Let_Result_1 : Boolean;
Quantifier_Result : Boolean;
Not_Val_2 : Boolean;
Let_Result_2 : Boolean;
Let_Result_3 : Boolean;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_5 : Internal_Entity_Basic_Decl_Array_Access;
Map_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_6 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_7 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_585 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Prim_Env);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Local_All_Prim_Names);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_585;
            procedure Finalizer_Scope_2474 is
            begin
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_All_Prims);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Local_Bds);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_2474;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3910 '<Let at nodes.lkt:5469:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5469'
--# scope-start
--# expr-start 3861 '<If at nodes.lkt:5469:24>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5469'
if Only_Inherited then
   --# expr-start 3859 '<FieldAccess for parent_primitives_env at nodes.lkt:5470:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5470'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.parent_primitives_env'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Parent_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3859
   If_Result := Fld;
else
   --# expr-start 3860 '<FieldAccess for primitives_env at nodes.lkt:5472:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5472'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3860
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 3861
Local_Prim_Env := If_Result; Inc_Ref (Local_Prim_Env);
--# bind prim_env Local_Prim_Env
--# expr-start 3909 '<Let at nodes.lkt:5475:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5475'
--# scope-start
--# expr-start 3867 '<unique at nodes.lkt:5475:30>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5475'
--# expr-start 3866 '<Map at nodes.lkt:5475:30>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5475'
   --# expr-start 3862 '<EnvGet at nodes.lkt:5475:30>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5475'
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Local_Prim_Env, Key => Thin (No_Symbol), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3862
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 3865 '<FieldAccess for name_symbol at nodes.lkt:5476:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5476'
--# expr-start 3864 '<FieldAccess for defining_name at nodes.lkt:5476:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5476'
--# expr-start 3863 '<Cast to BasicDecl at nodes.lkt:5476:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5476'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3863
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 3864
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 3865
         declare
            Item_To_Append : constant Symbol_Type := Fld_3;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3866
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 3867
Local_All_Prim_Names := Unique_Array; Inc_Ref (Local_All_Prim_Names);
--# bind all_prim_names Local_All_Prim_Names
--# expr-start 3908 '<Mapcat at nodes.lkt:5481:9>' Map_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5481'
      declare
         Map_Result_4_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Symbol_Type_Array_Access := Local_All_Prim_Names;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind name Item_1
      --# expr-start 3907 '<Let at nodes.lkt:5482:13>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5482'
--# scope-start
--# expr-start 3870 '<Map at nodes.lkt:5482:29>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5482'
   --# expr-start 3868 '<EnvGet at nodes.lkt:5482:29>' Env_Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5482'
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Local_Prim_Env, Key => Thin (Item_1), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3868
      declare
         Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_2
      --# expr-start 3869 '<Cast to BasicDecl at nodes.lkt:5482:59>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5482'
Cast_Expr_1 := Item_2; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3869
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3870
Local_All_Prims := Map_Result_1; Inc_Ref (Local_All_Prims);
--# bind all_prims Local_All_Prims
--# expr-start 3906 '<Let at nodes.lkt:5483:13>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5483'
--# scope-start
--# expr-start 3874 '<If at nodes.lkt:5483:23>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5483'
if Include_Predefined_Operators then
   If_Result_1 := Local_All_Prims;
else
   --# expr-start 3873 '<Map at nodes.lkt:5486:17>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5486'
      declare
         Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_All_Prims;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
         --# expr-start 3872 '<Not at nodes.lkt:5486:41>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5486'
--# expr-start 3871 '<IsA SyntheticSubpDecl at nodes.lkt:5486:45>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5486'
Is_A := Item_3.Node /= null 
and then Item_3.Node.Kind in Ada_Synthetic_Subp_Decl_Range; 
--# expr-done 3871
Not_Val := not (Is_A); 
--# expr-done 3872
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 3873
   If_Result_1 := Map_Result_2;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3874
Local_Bds := If_Result_1; Inc_Ref (Local_Bds);
--# bind bds Local_Bds
--# expr-start 3905 '<Map at nodes.lkt:5488:13>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5488'
      declare
         Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
         Index_5 := 0;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Bds;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_4
               --# bind i Index_5
         --# expr-start 3904 '<Let at nodes.lkt:5489:17>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5489'
--# scope-start
--# expr-start 3875 '<FieldAccess for subp_spec_or_null at nodes.lkt:5489:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5489'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_4.Node, E_Info => Item_4.Info);
--# end
--# expr-done 3875
Local_A_Spec := Fld_4; 
--# bind a_spec Local_A_Spec
--# expr-start 3903 '<Let at nodes.lkt:5490:17>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5490'
--# scope-start
--# expr-start 3880 '<Cast to BaseTypeDecl at nodes.lkt:5491:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5491'
--# expr-start 3879 '<as_bare_entity at nodes.lkt:5491:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5491'
--# expr-start 3878 '<FieldAccess for primitive at nodes.lkt:5491:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5491'
--# expr-start 3877 '<FieldAccess for md at nodes.lkt:5491:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5491'
--# expr-start 3876 '<FieldAccess for info at nodes.lkt:5491:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5491'
Fld_5 := Item_4.Info;
--# expr-done 3876
Fld_6 := Fld_5.Md;
--# expr-done 3877
Fld_7 := Fld_6.Primitive;
--# expr-done 3878
Is_Null := Fld_7 = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_7); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 3879
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3880
Local_A_Prim := Cast_Result_2; 
--# bind a_prim Local_A_Prim
--# expr-start 3902 '<Not at nodes.lkt:5496:17>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5496'
--# expr-start 3901 '<AnyQuantifier at nodes.lkt:5496:21>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5496'
      Quantifier_Result := False;
         Index_7 := 0;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Bds;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind b Item_6
               --# bind j Index_7
      --# expr-start 3900 '<Let at nodes.lkt:5497:21>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5497'
--# scope-start
--# expr-start 3884 '<Cast to BaseTypeDecl at nodes.lkt:5497:34>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5497'
--# expr-start 3883 '<FieldAccess for primitive at nodes.lkt:5497:34>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5497'
--# expr-start 3882 '<FieldAccess for md at nodes.lkt:5497:34>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5497'
--# expr-start 3881 '<FieldAccess for info at nodes.lkt:5497:34>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5497'
Fld_8 := Item_6.Info;
--# expr-done 3881
Fld_9 := Fld_8.Md;
--# expr-done 3882
Fld_10 := Fld_9.Primitive;
--# expr-done 3883
   if Fld_10 = null
      or else Fld_10.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Fld_10;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 3884
Local_B_Prim := Cast_Result_3; 
--# bind b_prim Local_B_Prim
--# expr-start 3886 '<Not at nodes.lkt:5499:22>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5499'
--# expr-start 3885 '<Eq at nodes.lkt:5499:22>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5499'
Is_Equal := Index_5 = Index_7; 
--# expr-done 3885
Not_Val_1 := not (Is_Equal); 
--# expr-done 3886
if Not_Val_1 then
   --# expr-start 3890 '<bind at nodes.lkt:5501:25>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5501'
--# expr-start 3887 '<FieldAccess for origin_node at nodes.lkt:5501:39>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5501'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_11 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Item_6.Node);
--# end
--# expr-done 3887
Bound_Origin := Fld_11; 
--# expr-start 3889 '<FieldAccess for match_signature at nodes.lkt:5503:25>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5503'
      if Local_A_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3888 '<FieldAccess for subp_spec_or_null at nodes.lkt:5504:29>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5504'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_6.Node, E_Info => Item_6.Info);
--# end
--# expr-done 3888
--# property-call-start BaseSubpSpec.match_signature
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Local_A_Spec.Node, Other => Fld_12, Match_Name => False, Use_Entity_Info => True, Origin => Bound_Origin, E_Info => Local_A_Spec.Info);
--# end
--# expr-done 3889
Dyn_Var_Bind_Result := Fld_13; 
--# expr-done 3890
   If_Result_3 := Dyn_Var_Bind_Result;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 3899 '<Let at nodes.lkt:5509:25>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5509'
--# scope-start
--# expr-start 3891 '<as_bare_entity at nodes.lkt:5509:42>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5509'
Is_Null_1 := Local_B_Prim = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Type_Decl;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Local_B_Prim); 
   If_Result_4 := As_Entity_1;
end if;
--# expr-done 3891
Local_B_Prim_Ent := If_Result_4; 
--# bind b_prim_ent Local_B_Prim_Ent
--# expr-start 3898 '<If at nodes.lkt:5513:25>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5513'
--# expr-start 3893 '<FieldAccess for has_base_type at nodes.lkt:5513:28>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5513'
      if Local_A_Prim.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3892 '<FieldAccess for node at nodes.lkt:5513:49>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5513'
Fld_18 := Local_B_Prim_Ent.Node;
--# expr-done 3892
--# property-call-start BaseTypeDecl.has_base_type
Fld_19 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => Local_A_Prim.Node, Target => Fld_18, E_Info => Local_A_Prim.Info);
--# end
--# expr-done 3893
if Fld_19 then
Comp_Result := Index_5 < Index_7; 
if Comp_Result then
   --# expr-start 3895 '<FieldAccess for has_base_type at nodes.lkt:5523:39>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5523'
      if Local_B_Prim_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3894 '<FieldAccess for node at nodes.lkt:5523:64>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5523'
Fld_14 := Local_A_Prim.Node;
--# expr-done 3894
--# property-call-start BaseTypeDecl.has_base_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => Local_B_Prim_Ent.Node, Target => Fld_14, E_Info => Local_B_Prim_Ent.Info);
--# end
--# expr-done 3895
   If_Result_5 := Fld_15;
else
   If_Result_5 := False;
end if;
   If_Result_7 := If_Result_5;
else
Comp_Result_1 := Index_5 > Index_7; 
if Comp_Result_1 then
   If_Result_6 := True;
else
   --# expr-start 3897 '<FieldAccess for has_base_type at nodes.lkt:5536:38>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5536'
      if Local_B_Prim_Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3896 '<FieldAccess for node at nodes.lkt:5536:63>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5536'
Fld_16 := Local_A_Prim.Node;
--# expr-done 3896
--# property-call-start BaseTypeDecl.has_base_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type (Node => Local_B_Prim_Ent.Node, Target => Fld_16, E_Info => Local_B_Prim_Ent.Info);
--# end
--# expr-done 3897
   If_Result_6 := Fld_17;
end if;
   If_Result_7 := If_Result_6;
end if;
--# expr-done 3898
Let_Result := If_Result_7; 
--# end
--# expr-done 3899
   If_Result_8 := Let_Result;
else
   If_Result_8 := False;
end if;
Let_Result_1 := If_Result_8; 
--# end
--# expr-done 3900
      Quantifier_Result := Let_Result_1;
   --# end
               exit when Quantifier_Result;
               Index_7 := Index_7 + 1;
         end loop;
      end;
--# expr-done 3901
Not_Val_2 := not (Quantifier_Result); 
--# expr-done 3902
Let_Result_2 := Not_Val_2; 
--# end
--# expr-done 3903
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 3904
         if Let_Result_3 then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Item_4;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
               Index_5 := Index_5 + 1;
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 3905
Let_Result_4 := Map_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 3906
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 3907
         for Item_To_Append of
               Let_Result_5.Items
         loop
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2474;
         end loop;
      end;
         Map_Result_4 := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_4_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 3908
Let_Result_6 := Map_Result_4; Inc_Ref (Let_Result_6);
--# end
--# expr-done 3909
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 3910
         Property_Result := Let_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_585;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_585;
                     Finalizer_Scope_2474;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Primitives;
--# end
--# property-start BaseTypeDecl.is_array_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5547'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3911 '<FieldAccess for is_array at nodes.lkt:5547:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5547'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3911
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Type;
--# end
--# property-start BaseTypeDecl.find_derived_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5552'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_587 with Inline_Always;
            procedure Finalizer_Scope_2479 with Inline_Always;
            procedure Finalizer_Scope_2481 with Inline_Always;
            procedure Finalizer_Scope_2482 with Inline_Always;
            procedure Finalizer_Scope_2483 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Array_Access;
Var_Expr_3 : Internal_Entity_Type_Decl;
Var_Expr_4 : Boolean;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Map_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Boolean;
Array_Lit : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_587 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_587;
            procedure Finalizer_Scope_2479 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2479;
            procedure Finalizer_Scope_2481 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2481;
            procedure Finalizer_Scope_2482 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2482;
            procedure Finalizer_Scope_2483 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2483;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3912 '<children at nodes.lkt:5554:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5554'
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 3912
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 3919 '<Mapcat at nodes.lkt:5555:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5555'
   --# expr-start 3917 '<Map at nodes.lkt:5555:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5555'
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
         --# expr-start 3916 '<Not at nodes.lkt:5556:24>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5556'
--# expr-start 3913 '<IsNull at nodes.lkt:5556:29>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5556'
Is_Null := Item_1.Node = null; 
--# expr-done 3913
if Is_Null then
   If_Result := True;
else
   --# expr-start 3915 '<Eq at nodes.lkt:5556:42>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5556'
--# expr-start 3914 '<FieldAccess for node at nodes.lkt:5556:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5556'
Fld_1 := Item_1.Node;
--# expr-done 3914
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 3915
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 3916
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 3917
      declare
         Map_Result_1_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 3918 '<FieldAccess for find_derived_types at nodes.lkt:5557:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5557'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => Item_2, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3918
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2481;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 3919
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2479;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 3920 '<Cast to TypeDecl at nodes.lkt:5558:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5558'
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 3920
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind type_decl Var_Expr_3
--# expr-start 3921 '<FieldAccess for is_derived_type at nodes.lkt:5559:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5559'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr_3.Node, Other_Type => Ent, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 3921
Var_Expr_4 := Fld_3;
if Var_Expr_4 /= False then
   --# scope-start
   --# bind _ Var_Expr_4
--# expr-start 3922 '<ArrayLiteral at nodes.lkt:5559:70>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5559'
Array_Lit := Create_Internal_Entity_Type_Decl_Array (Internal_Internal_Entity_Type_Decl_Array'(1 => Var_Expr_3)); 
--# expr-done 3922
Scope_Result_1 := Array_Lit;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2483;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2482;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_587;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_587;
                     Finalizer_Scope_2479;
                     Finalizer_Scope_2481;
                     Finalizer_Scope_2482;
                     Finalizer_Scope_2483;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_Derived_Types;
--# end
--# property-start BaseTypeDecl.synthetic_object_decl_env_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5568'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Synthetic_Object_Decl_Env_Assoc
  (Node : Bare_Base_Type_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_588 with Inline_Always;
      Var_Expr : Bare_Type_Expr;
Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Bare_Type_Expr;
Scope_Result : Bare_Type_Expr;
New_Node : Bare_Synthetic_Type_Expr;
Cast_Result : Bare_Type_Expr;
Result_Var : Bare_Type_Expr;
New_Node_1 : Bare_Synthetic_Object_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_588 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_588;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Base_Type_Decl_Lf_State_Synthetic_Object_Decl_Env_Assoc is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Base_Type_Decl_Lf_Stg_Synthetic_Object_Decl_Env_Assoc;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Base_Type_Decl_Lf_State_Synthetic_Object_Decl_Env_Assoc,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3931 '<New EnvAssoc at nodes.lkt:5568:49>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5568'
--# expr-start 3923 '<New DesignatedEnv at nodes.lkt:5581:21>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5581'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 3923
--# expr-start 3924 '<FieldAccess for name_symbol at nodes.lkt:5569:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5569'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3924
--# expr-start 3930 '<New SyntheticObjectDecl at nodes.lkt:5575:15>' New_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5575'
--# expr-start 3925 '<FieldAccess for name at nodes.lkt:5576:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5576'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Base_Type_Decl_F_Name;
--# expr-done 3925
--# expr-start 3928 '<FieldAccess for node at nodes.lkt:5580:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5580'
--# expr-start 3927 '<FieldAccess for type_expression at nodes.lkt:5580:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5580'
--# expr-start 3926 '<as_bare_entity at nodes.lkt:5580:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5580'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3926
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3927
Fld_3 := Fld_2.Node;
--# expr-done 3928
Var_Expr := Fld_3;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 3929 '<New SyntheticTypeExpr at nodes.lkt:5580:70>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5580'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Self = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "SyntheticTypeExpr.target_type cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Type_Expr
     (Self => New_Node,
      Synthetic_Type_Expr_F_Target_Type => Self);
--# expr-done 3929
      Cast_Result := New_Node;
   Result_Var := Cast_Result;
end if;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node_1 := new Root_Node_Record
  (Ada_Synthetic_Object_Decl);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Synthetic_Object_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
      New_Node_1.Synthetic_Object_Decl_Name :=
         Fld_1;
      New_Node_1.Synthetic_Object_Decl_Type_Expr :=
         Result_Var;
--# expr-done 3930
      Cast_Result_1 := New_Node_1;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result_1); 
--# expr-done 3931
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_588;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_588;
               Self.Bare_Base_Type_Decl_Lf_State_Synthetic_Object_Decl_Env_Assoc :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Base_Type_Decl_Lf_State_Synthetic_Object_Decl_Env_Assoc := Initialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Synthetic_Object_Decl_Env_Assoc := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Base_Type_Decl_F_Synthetic_Object_Decl_Env_Assoc;
--# end
--# property-start BaseTypeDecl.is_scalar_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5589'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Scalar_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 3932 '<FieldAccess for is_int_type at nodes.lkt:5592:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5592'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3932
if Fld then
   If_Result := True;
else
   --# expr-start 3933 '<FieldAccess for is_real_type at nodes.lkt:5592:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5592'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3933
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3934 '<FieldAccess for is_float_type at nodes.lkt:5593:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5593'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3934
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 3935 '<FieldAccess for is_fixed_point at nodes.lkt:5594:14>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5594'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3935
   If_Result_2 := Fld_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 3936 '<FieldAccess for is_enum_type at nodes.lkt:5595:10>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5595'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3936
   If_Result_3 := Fld_4;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Scalar_Type;
--# end
--# property-start BaseTypeDecl.is_implicit_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5599'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Implicit_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Access_Def;
Fld : Internal_Entity_Access_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3937 '<FieldAccess for access_def at nodes.lkt:5600:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3937
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Access_Def then
   --# scope-start
   --# bind d Var_Expr_0
--# expr-start 3939 '<Not at nodes.lkt:5600:37>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
--# expr-start 3938 '<IsA AccessToSubpDef at nodes.lkt:5600:41>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 3938
Not_Val := not (Is_A); 
--# expr-done 3939
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
   --# expr-start 3942 '<Not at nodes.lkt:5600:66>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
--# expr-start 3941 '<IsNull at nodes.lkt:5600:70>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
--# expr-start 3940 '<FieldAccess for get_imp_deref at nodes.lkt:5600:70>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3940
Is_Null := Fld_1.Node = null; 
--# expr-done 3941
Not_Val_1 := not (Is_Null); 
--# expr-done 3942
   If_Result := Not_Val_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Implicit_Deref;
--# end
--# property-start BaseTypeDecl.specific_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5611'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Specific_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Classwide_Type_Decl;
Match_1 : Internal_Entity_Base_Subtype_Decl;
Local_Bt : Internal_Entity_Base_Type_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Classwide_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Subtype_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 3950 '<Match at nodes.lkt:5611:49>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5611'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind cw Match_0
--# expr-start 3944 '<FieldAccess for specific_type at nodes.lkt:5614:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5614'
--# expr-start 3943 '<FieldAccess for type_decl at nodes.lkt:5614:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5614'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 3943
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3944
Let_Result_1 := Fld_1; 
--# end
Scope_Result := Let_Result_1;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind bt Match_1
--# expr-start 3949 '<Let at nodes.lkt:5616:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5616'
--# scope-start
--# expr-start 3945 '<FieldAccess for base_subtype at nodes.lkt:5616:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5616'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Match_1.Node, Origin => No_Bare_Ada_Node, E_Info => Match_1.Info);
--# end
--# expr-done 3945
Local_Bt := Fld_2; 
--# bind bt Local_Bt
--# expr-start 3948 '<If at nodes.lkt:5620:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5620'
--# expr-start 3946 '<Eq at nodes.lkt:5620:16>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5620'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Local_Bt; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3946
if Is_Equiv then
   If_Result := Ent;
else
   --# expr-start 3947 '<FieldAccess for specific_type at nodes.lkt:5620:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5620'
      if Local_Bt.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Local_Bt.Node, E_Info => Local_Bt.Info);
--# end
--# expr-done 3947
   If_Result := Fld_3;
end if;
--# expr-done 3948
Let_Result := If_Result; 
--# end
--# expr-done 3949
Let_Result_2 := Let_Result; 
--# end
Scope_Result_1 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_3 := Ent; 
--# end
Scope_Result_2 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3950
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Specific_Type;
--# end
--# property-start BaseTypeDecl.derefed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5628'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Derefed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3955 '<If at nodes.lkt:5629:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5629'
--# expr-start 3951 '<IsNull at nodes.lkt:5629:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5629'
Is_Null := Ent.Node = null; 
--# expr-done 3951
if Is_Null then
   If_Result := True;
else
   --# expr-start 3953 '<IsNull at nodes.lkt:5629:28>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5629'
--# expr-start 3952 '<FieldAccess for get_imp_deref at nodes.lkt:5629:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5629'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3952
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 3953
   If_Result := Is_Null_1;
end if;
if If_Result then
   If_Result_1 := Ent;
else
   --# expr-start 3954 '<FieldAccess for accessed_type at nodes.lkt:5629:72>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5629'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3954
   If_Result_1 := Fld;
end if;
--# expr-done 3955
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Derefed_Type;
--# end
--# property-start BaseTypeDecl.derefed_base_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5635'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Derefed_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3963 '<If at nodes.lkt:5635:56>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5635'
--# expr-start 3956 '<IsNull at nodes.lkt:5635:59>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5635'
Is_Null_1 := Ent.Node = null; 
--# expr-done 3956
if Is_Null_1 then
   If_Result_1 := Ent;
else
   --# expr-start 3962 '<FieldAccess for base_subtype at nodes.lkt:5635:87>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5635'
--# expr-start 3961 '<If at nodes.lkt:5636:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5636'
--# expr-start 3959 '<Not at nodes.lkt:5636:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5636'
--# expr-start 3958 '<IsNull at nodes.lkt:5636:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5636'
--# expr-start 3957 '<FieldAccess for get_imp_deref at nodes.lkt:5636:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3957
Is_Null := Fld_1.Node = null; 
--# expr-done 3958
Not_Val := not (Is_Null); 
--# expr-done 3959
if Not_Val then
   --# expr-start 3960 '<FieldAccess for accessed_type at nodes.lkt:5636:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3960
   If_Result := Fld;
else
   If_Result := Ent;
end if;
--# expr-done 3961
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 3962
   If_Result_1 := Fld_2;
end if;
--# expr-done 3963
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Derefed_Base_Subtype;
--# end
--# property-start BaseTypeDecl.array_concat_result_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5643'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Concat_Result_Type
  (Node : Bare_Base_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Is_Null_1 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3976 '<If at nodes.lkt:5644:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5644'
--# expr-start 3964 '<IsNull at nodes.lkt:5644:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5644'
Is_Null := Ent.Node = null; 
--# expr-done 3964
if Is_Null then
   If_Result_4 := True;
else
   --# expr-start 3965 '<IsNull at nodes.lkt:5644:28>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5644'
Is_Null_1 := Other.Node = null; 
--# expr-done 3965
   If_Result_4 := Is_Null_1;
end if;
if If_Result_4 then
   If_Result_5 := No_Entity_Base_Type_Decl;
else
   --# expr-start 3975 '<If at nodes.lkt:5644:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5644'
--# expr-start 3966 '<FieldAccess for matching_formal_type at nodes.lkt:5645:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5645'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Other.Node, Formal_Type => Ent, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3966
if Fld_5 then
   If_Result_2 := True;
else
   --# expr-start 3968 '<FieldAccess for matching_formal_type at nodes.lkt:5645:50>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5645'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3967 '<FieldAccess for comp_type at nodes.lkt:5645:77>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5645'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3967
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Other.Node, Formal_Type => Fld_6, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3968
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   --# expr-start 3969 '<FieldAccess for derefed_base_subtype at nodes.lkt:5645:100>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5645'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3969
   If_Result_3 := Fld;
else
   --# expr-start 3974 '<If at nodes.lkt:5644:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5644'
--# expr-start 3970 '<FieldAccess for matching_formal_type at nodes.lkt:5646:14>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5646'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Ent.Node, Formal_Type => Other, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3970
if Fld_2 then
   If_Result := True;
else
   --# expr-start 3972 '<FieldAccess for matching_formal_type at nodes.lkt:5646:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5646'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3971 '<FieldAccess for comp_type at nodes.lkt:5646:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5646'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3971
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Ent.Node, Formal_Type => Fld_3, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3972
   If_Result := Fld_4;
end if;
if If_Result then
   --# expr-start 3973 '<FieldAccess for derefed_base_subtype at nodes.lkt:5646:100>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5646'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3973
   If_Result_1 := Fld_1;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 3974
   If_Result_3 := If_Result_1;
end if;
--# expr-done 3975
   If_Result_5 := If_Result_3;
end if;
--# expr-done 3976
         Property_Result := If_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Concat_Result_Type;
--# end
--# property-start BaseTypeDecl.expected_array_concat_operand_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5656'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expected_Array_Concat_Operand_Type
  (Node : Bare_Base_Type_Decl
      ; Operand_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind operand_type Operand_Type
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Is_Null_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3984 '<If at nodes.lkt:5657:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5657'
--# expr-start 3977 '<IsNull at nodes.lkt:5657:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5657'
Is_Null := Ent.Node = null; 
--# expr-done 3977
if Is_Null then
   If_Result_2 := True;
else
   --# expr-start 3978 '<IsNull at nodes.lkt:5657:28>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5657'
Is_Null_1 := Operand_Type.Node = null; 
--# expr-done 3978
   If_Result_2 := Is_Null_1;
end if;
if If_Result_2 then
   If_Result_3 := No_Entity_Base_Type_Decl;
else
   --# expr-start 3983 '<If at nodes.lkt:5657:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5657'
--# expr-start 3979 '<FieldAccess for matching_formal_type at nodes.lkt:5658:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5658'
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Operand_Type.Node, Formal_Type => Ent, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3979
if Fld_1 then
   If_Result := True;
else
   --# expr-start 3981 '<FieldAccess for matching_formal_type at nodes.lkt:5658:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5658'
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 3980 '<FieldAccess for comp_type at nodes.lkt:5658:91>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5658'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3980
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Operand_Type.Node, Formal_Type => Fld_2, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3981
   If_Result := Fld_3;
end if;
if If_Result then
   --# expr-start 3982 '<FieldAccess for derefed_base_subtype at nodes.lkt:5658:114>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5658'
      if Operand_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Operand_Type.Node, Origin => Origin, E_Info => Operand_Type.Info);
--# end
--# expr-done 3982
   If_Result_1 := Fld;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 3983
   If_Result_3 := If_Result_1;
end if;
--# expr-done 3984
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Expected_Array_Concat_Operand_Type;
--# end
--# property-start BaseTypeDecl.is_non_null_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5662'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Non_Null_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3986 '<Not at nodes.lkt:5663:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5663'
--# expr-start 3985 '<IsNull at nodes.lkt:5663:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5663'
Is_Null := Self = null; 
--# expr-done 3985
Not_Val := not (Is_Null); 
--# expr-done 3986
if Not_Val then
   --# expr-start 3987 '<FieldAccess for is_char_type at nodes.lkt:5663:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5663'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3987
   If_Result := Fld;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Non_Null_Char_Type;
--# end
--# property-start BaseTypeDecl.scalar_base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5665'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 3989 '<as_entity at nodes.lkt:5666:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5666'
--# expr-start 3988 '<FieldAccess for scalar_base_subtype_node at nodes.lkt:5666:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5666'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 3988
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 3989
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Type;
--# end
--# property-start BaseTypeDecl.is_universal_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5673'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3991 '<Not at nodes.lkt:5673:37>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5673'
--# expr-start 3990 '<IsNull at nodes.lkt:5673:41>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5673'
Is_Null := Ent.Node = null; 
--# expr-done 3990
Not_Val := not (Is_Null); 
--# expr-done 3991
if Not_Val then
--# expr-start 3993 '<Eq at nodes.lkt:5674:9>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 3992 '<FieldAccess for universal_int_type at nodes.lkt:5674:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 3992
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 3993
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 3995 '<Eq at nodes.lkt:5674:46>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 3994 '<FieldAccess for universal_fixed_type at nodes.lkt:5674:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 3994
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 3995
   If_Result := Is_Equiv_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 3997 '<Eq at nodes.lkt:5674:85>' Is_Equiv_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
Cast_Expr_2 := Ent; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 3996 '<FieldAccess for universal_real_type at nodes.lkt:5674:93>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5674'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 3996
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 3997
   If_Result_1 := Is_Equiv_2;
end if;
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_not_universal_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5682'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 3999 '<Not at nodes.lkt:5683:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5683'
--# expr-start 3998 '<IsNull at nodes.lkt:5683:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5683'
Is_Null := Ent.Node = null; 
--# expr-done 3998
Not_Val := not (Is_Null); 
--# expr-done 3999
if Not_Val then
   --# expr-start 4001 '<Not at nodes.lkt:5683:30>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5683'
--# expr-start 4000 '<FieldAccess for is_universal_type at nodes.lkt:5683:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5683'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4000
Not_Val_1 := not (Fld); 
--# expr-done 4001
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Not_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_access_type_predicate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5689'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type_Predicate
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_122 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_122 := Ent;
if Var_Expr_122 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4002 '<FieldAccess for is_access_type at nodes.lkt:5689:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5689'
      if Var_Expr_122.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr_122.Node, Origin => Origin, E_Info => Var_Expr_122.Info);
--# end
--# expr-done 4002
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_Type_Predicate;
--# end
--# property-start BaseTypeDecl.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5692'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Ndims
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := 0;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Ndims;
--# end
--# property-start BaseTypeDecl.static_predicate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5697'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Static_Predicate
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4005 '<FieldAccess for value at nodes.lkt:5698:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
--# expr-start 4004 '<FieldAccess for get_aspect at nodes.lkt:5698:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4003 '<Symbol Static_Predicate at nodes.lkt:5698:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Static_Predicate); 
--# expr-done 4003
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4004
Fld_1 := Fld.Value;
--# expr-done 4005
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4008 '<FieldAccess for value at nodes.lkt:5698:56>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
--# expr-start 4007 '<FieldAccess for get_aspect at nodes.lkt:5698:56>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4006 '<Symbol Predicate at nodes.lkt:5698:72>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5698'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 4006
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4007
Fld_3 := Fld_2.Value;
--# expr-done 4008
   Result_Var := Fld_3;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Static_Predicate;
--# end
--# property-start BaseTypeDecl.satisfies_type_predicates 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5705'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Satisfies_Type_Predicates
  (Node : Bare_Base_Type_Decl
      ; Value : Big_Integer_Type
      ; Imprecise_Fallback : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind value Value
   --# bind imprecise_fallback Imprecise_Fallback
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_603 with Inline_Always;
            procedure Finalizer_Scope_2491 with Inline_Always;
      Local_True_Val : Big_Integer_Type;
Local_Satisfies_Own_Predicate : Boolean;
Var_Expr_0 : Internal_Entity_Expr;
Local_From_Type : Internal_Entity_Base_Type_Decl;
Local_Base_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_123 : Internal_Entity_Base_Subtype_Decl;
Big_Int : Big_Integer_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Substitution := No_Substitution;
Array_Lit : Internal_Substitution_Array_Access;
Fld_1 : Big_Integer_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Subtype_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Fld_5 : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
Let_Result_2 : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_603 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Local_True_Val);
            end Finalizer_Scope_603;
            procedure Finalizer_Scope_2491 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2491;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4029 '<Let at nodes.lkt:5706:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5706'
--# scope-start
--# expr-start 4009 '<BigInteger 1 at nodes.lkt:5706:24>' Big_Int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5706'
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 4009
Local_True_Val := Big_Int; Inc_Ref (Local_True_Val);
--# bind true_val Local_True_Val
--# expr-start 4028 '<Let at nodes.lkt:5707:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5707'
--# scope-start
--# expr-start 4010 '<FieldAccess for static_predicate at nodes.lkt:5707:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5707'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.static_predicate
Fld := Libadalang.Implementation.Base_Type_Decl_P_Static_Predicate (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4010
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind pred Var_Expr_0
--# expr-start 4014 '<Eq at nodes.lkt:5708:23>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5708'
--# expr-start 4013 '<FieldAccess for eval_as_int_in_env at nodes.lkt:5708:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5708'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4012 '<ArrayLiteral at nodes.lkt:5709:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5709'
--# expr-start 4011 '<New Substitution at nodes.lkt:5709:18>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5709'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Inc_Ref (Value);
New_Struct := (From_Decl => Cast_Result, To_Value => Value, Value_Type => Ent); 
--# expr-done 4011
Array_Lit := Create_Internal_Substitution_Array (Internal_Internal_Substitution_Array'(1 => New_Struct)); 
--# expr-done 4012
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Var_Expr_0.Node, Env => Array_Lit, E_Info => Var_Expr_0.Info);
--# expr-done 4013
Is_Equal := Equivalent (Local_True_Val, Fld_1); 
--# expr-done 4014
Scope_Result := Is_Equal;
   --# end
      Finalizer_Scope_2491;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Local_Satisfies_Own_Predicate := Result_Var; 
--# bind satisfies_own_predicate Local_Satisfies_Own_Predicate
--# expr-start 4027 '<Let at nodes.lkt:5714:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5714'
--# scope-start
--# expr-start 4015 '<Cast to BaseSubtypeDecl at nodes.lkt:5714:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5714'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 4015
Var_Expr_123 := Cast_Result_1;
if Var_Expr_123 /= No_Entity_Base_Subtype_Decl then
   --# scope-start
--# expr-start 4016 '<FieldAccess for get_type at nodes.lkt:5714:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5714'
      if Var_Expr_123.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Var_Expr_123.Node, Origin => Origin, E_Info => Var_Expr_123.Info);
--# end
--# expr-done 4016
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Local_From_Type := Result_Var_1; 
--# bind from_type Local_From_Type
--# expr-start 4026 '<Let at nodes.lkt:5715:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5715'
--# scope-start
--# expr-start 4017 '<FieldAccess for base_type at nodes.lkt:5715:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5715'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4017
Local_Base_Type := Fld_3; 
--# bind base_type Local_Base_Type
if Local_Satisfies_Own_Predicate then
   --# expr-start 4025 '<If at nodes.lkt:5718:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5718'
--# expr-start 4019 '<Not at nodes.lkt:5718:16>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5718'
--# expr-start 4018 '<IsNull at nodes.lkt:5718:20>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5718'
Is_Null_1 := Local_From_Type.Node = null; 
--# expr-done 4018
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4019
if Not_Val_1 then
   --# expr-start 4020 '<FieldAccess for satisfies_type_predicates at nodes.lkt:5718:43>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5718'
      if Local_From_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Local_From_Type.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Local_From_Type.Info);
--# end
--# expr-done 4020
   If_Result_1 := Fld_4;
else
   --# expr-start 4024 '<If at nodes.lkt:5718:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5718'
--# expr-start 4022 '<Not at nodes.lkt:5719:18>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5719'
--# expr-start 4021 '<IsNull at nodes.lkt:5719:22>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5719'
Is_Null := Local_Base_Type.Node = null; 
--# expr-done 4021
Not_Val := not (Is_Null); 
--# expr-done 4022
if Not_Val then
   --# expr-start 4023 '<FieldAccess for satisfies_type_predicates at nodes.lkt:5719:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5719'
      if Local_Base_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Local_Base_Type.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Local_Base_Type.Info);
--# end
--# expr-done 4023
   If_Result := Fld_5;
else
   If_Result := True;
end if;
--# expr-done 4024
   If_Result_1 := If_Result;
end if;
--# expr-done 4025
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 4026
Let_Result_1 := Let_Result; 
--# end
--# expr-done 4027
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 4028
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 4029
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_603;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_603;
                     Finalizer_Scope_2491;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Satisfies_Type_Predicates;
--# end
--# property-start BaseTypeDecl.is_iterator_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5726'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterator_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_604 with Inline_Always;
      Local_Iifcs : Bare_Basic_Decl;
Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Classwide_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Sym_2 : Symbol_Type;
Fld_6 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_604 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_604;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Is_Iterator_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4044 '<Let at nodes.lkt:5727:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5727'
--# scope-start
--# expr-start 4033 '<FieldAccess for get_unit_root_decl at nodes.lkt:5727:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5727'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4032 '<ArrayLiteral at nodes.lkt:5728:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5728'
--# expr-start 4030 '<Symbol Ada at nodes.lkt:5728:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5728'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 4030
--# expr-start 4031 '<Symbol Iterator_Interfaces at nodes.lkt:5728:22>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5728'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Interfaces); 
--# expr-done 4031
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 4032
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 4033
Local_Iifcs := Fld; 
--# bind iifcs Local_Iifcs
--# expr-start 4043 '<Let at nodes.lkt:5730:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5730'
--# scope-start
--# expr-start 4034 '<Cast to ClasswideTypeDecl at nodes.lkt:5730:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5730'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 4034
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind cw Var_Expr_0
--# expr-start 4035 '<FieldAccess for type_decl at nodes.lkt:5730:57>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5730'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_1 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4035
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Local_Typ := Result_Var; 
--# bind typ Local_Typ
--# expr-start 4039 '<Eq at nodes.lkt:5732:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
--# expr-start 4038 '<FieldAccess for node at nodes.lkt:5732:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
--# expr-start 4037 '<FieldAccess for semantic_parent at nodes.lkt:5732:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
--# expr-start 4036 '<FieldAccess for semantic_parent at nodes.lkt:5732:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Local_Typ.Node, E_Info => Local_Typ.Info);
--# end
--# expr-done 4036
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4037
Fld_4 := Fld_3.Node;
--# expr-done 4038
Cast_Expr_1 := Local_Iifcs; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_4 = Cast_Result_1; 
--# expr-done 4039
if Is_Equal then
   If_Result := True;
else
   --# expr-start 4042 '<FieldAccess for has_aspect at nodes.lkt:5732:66>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
--# expr-start 4040 '<FieldAccess for canonical_part at nodes.lkt:5732:66>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4040
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4041 '<Symbol Iterable at nodes.lkt:5732:99>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5732'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 4041
--# property-call-start BasicDecl.has_aspect
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Fld_5.Node, Name => Sym_2, Imprecise_Fallback => False, E_Info => Fld_5.Info);
--# end
--# expr-done 4042
   If_Result := Fld_6;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 4043
Let_Result_1 := Let_Result; 
--# end
--# expr-done 4044
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_604;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_604;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Is_Iterator_Type;
--# end
--# property-start BaseTypeDecl.cursor_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5738'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Cursor_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_605 with Inline_Always;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_605 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_605;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4048 '<Cast to BaseTypeDecl at nodes.lkt:5741:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5741'
--# expr-start 4047 '<EnvGet at nodes.lkt:5741:9>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5741'
--# expr-start 4045 '<FieldAccess for children_env at nodes.lkt:5741:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5741'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4045
--# expr-start 4046 '<Symbol Cursor at nodes.lkt:5741:39>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5741'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Cursor); 
--# expr-done 4046
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 4047
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4048
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4049 '<FieldAccess for iterable_cursor_type at nodes.lkt:5743:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5743'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_cursor_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Cursor_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4049
   Result_Var := Fld_1;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_605;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_605;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Cursor_Type;
--# end
--# property-start BaseTypeDecl.is_not_root_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5745'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Root_Int_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4051 '<Not at nodes.lkt:5746:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5746'
--# expr-start 4050 '<IsNull at nodes.lkt:5746:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5746'
Is_Null := Self = null; 
--# expr-done 4050
Not_Val := not (Is_Null); 
--# expr-done 4051
if Not_Val then
   --# expr-start 4054 '<Not at nodes.lkt:5746:30>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5746'
--# expr-start 4053 '<Eq at nodes.lkt:5746:30>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5746'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 4052 '<FieldAccess for root_int_type at nodes.lkt:5746:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 4052
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 4053
Not_Val_1 := not (Is_Equiv); 
--# expr-done 4054
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Not_Root_Int_Type;
--# end
--# property-start BaseTypeDecl.allows_universal_int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5752'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_Universal_Int
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Aspect;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4055 '<FieldAccess for is_int_type at nodes.lkt:5752:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5752'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4055
if Fld then
   If_Result := True;
else
--# expr-start 4056 '<FieldAccess for base_subtype at nodes.lkt:5752:62>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5752'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4056
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 4060 '<Not at nodes.lkt:5753:17>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5753'
--# expr-start 4059 '<IsNull at nodes.lkt:5755:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5755'
--# expr-start 4058 '<FieldAccess for get_aspect at nodes.lkt:5755:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5755'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4057 '<Symbol Integer_Literal at nodes.lkt:5755:23>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5755'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Literal); 
--# expr-done 4057
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_0.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4058
Is_Equal := Fld_2 = No_Aspect; 
--# expr-done 4059
Not_Val := not (Is_Equal); 
--# expr-done 4060
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_Universal_Int;
--# end
--# property-start BaseTypeDecl.allows_universal_real 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5762'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_Universal_Real
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Aspect;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4061 '<FieldAccess for is_real_type at nodes.lkt:5762:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5762'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4061
if Fld then
   If_Result := True;
else
--# expr-start 4062 '<FieldAccess for base_subtype at nodes.lkt:5762:64>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5762'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4062
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 4066 '<Not at nodes.lkt:5763:17>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5763'
--# expr-start 4065 '<IsNull at nodes.lkt:5765:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5765'
--# expr-start 4064 '<FieldAccess for get_aspect at nodes.lkt:5765:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5765'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4063 '<Symbol Real_Literal at nodes.lkt:5765:23>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5765'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Real_Literal); 
--# expr-done 4063
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_0.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4064
Is_Equal := Fld_2 = No_Aspect; 
--# expr-done 4065
Not_Val := not (Is_Equal); 
--# expr-done 4066
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_Universal_Real;
--# end
--# property-start BaseTypeDecl.is_not_any_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5776'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Any_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4068 '<Not at nodes.lkt:5776:35>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5776'
--# expr-start 4067 '<IsNull at nodes.lkt:5776:39>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5776'
Is_Null := Self = null; 
--# expr-done 4067
Not_Val := not (Is_Null); 
--# expr-done 4068
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Not_Any_Type;
--# end
--# property-start BaseTypeDecl.allows_string_literal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5787'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Allows_String_Literal
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Aspect;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4070 '<Not at nodes.lkt:5787:41>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5787'
--# expr-start 4069 '<IsNull at nodes.lkt:5787:45>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5787'
Is_Null := Self = null; 
--# expr-done 4069
Not_Val := not (Is_Null); 
--# expr-done 4070
if Not_Val then
--# expr-start 4071 '<FieldAccess for is_str_type at nodes.lkt:5788:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5788'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_str_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Str_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4071
if Fld then
   If_Result := True;
else
--# expr-start 4072 '<FieldAccess for base_subtype at nodes.lkt:5788:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5788'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4072
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 4076 '<Not at nodes.lkt:5789:21>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5789'
--# expr-start 4075 '<IsNull at nodes.lkt:5791:13>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5791'
--# expr-start 4074 '<FieldAccess for get_aspect at nodes.lkt:5791:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5791'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4073 '<Symbol String_Literal at nodes.lkt:5791:27>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5791'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String_Literal); 
--# expr-done 4073
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_0.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4074
Is_Equal := Fld_2 = No_Aspect; 
--# expr-done 4075
Not_Val_1 := not (Is_Equal); 
--# expr-done 4076
Scope_Result := Not_Val_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Allows_String_Literal;
--# end
--# property-start BaseTypeDecl.is_str_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5797'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Str_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_124 : Internal_Entity_Base_Type_Decl;
Fld : Integer;
Is_Equal : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4078 '<Eq at nodes.lkt:5798:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5798'
--# expr-start 4077 '<FieldAccess for array_ndims at nodes.lkt:5798:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4077
Is_Equal := Fld = 1; 
--# expr-done 4078
if Is_Equal then
--# expr-start 4079 '<FieldAccess for comp_type at nodes.lkt:5798:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4079
Var_Expr_124 := Fld_1;
if Var_Expr_124 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4080 '<FieldAccess for is_char_type at nodes.lkt:5798:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5798'
      if Var_Expr_124.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Var_Expr_124.Node, Origin => Origin, E_Info => Var_Expr_124.Info);
--# end
--# expr-done 4080
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Str_Type;
--# end
--# property-start BaseTypeDecl.accessed_type_no_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5803'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type_No_Call
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Access_Def;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4085 '<If at nodes.lkt:5804:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5804'
--# expr-start 4081 '<IsNull at nodes.lkt:5804:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5804'
Is_Null := Ent.Node = null; 
--# expr-done 4081
if Is_Null then
   If_Result := True;
else
   --# expr-start 4083 '<IsA AccessToSubpDef at nodes.lkt:5804:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5804'
--# expr-start 4082 '<FieldAccess for access_def at nodes.lkt:5804:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5804'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4082
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 4083
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_1 := No_Entity_Base_Type_Decl;
else
   --# expr-start 4084 '<FieldAccess for accessed_type at nodes.lkt:5804:102>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5804'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4084
   If_Result_1 := Fld;
end if;
--# expr-done 4085
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Accessed_Type_No_Call;
--# end
--# property-start BaseTypeDecl.final_accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5815'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Final_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; First_Call : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind first_call First_Call
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4086 '<FieldAccess for accessed_type at nodes.lkt:5815:78>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5815'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4086
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_0
--# expr-start 4087 '<FieldAccess for final_accessed_type at nodes.lkt:5816:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5816'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Var_Expr_0.Node, First_Call => False, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4087
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4088 '<If at nodes.lkt:5816:60>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5816'
if First_Call then
   If_Result := No_Entity_Base_Type_Decl;
else
   If_Result := Ent;
end if;
--# expr-done 4088
   Result_Var := If_Result;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Final_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_access_to 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5820'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_To
  (Node : Bare_Base_Type_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_125 : Internal_Entity_Base_Type_Decl;
Var_Expr_126 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_125 := Ent;
if Var_Expr_125 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4089 '<FieldAccess for accessed_type at nodes.lkt:5821:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5821'
      if Var_Expr_125.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_125.Node, Origin => Origin, E_Info => Var_Expr_125.Info);
--# end
--# expr-done 4089
Var_Expr_126 := Fld;
if Var_Expr_126 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4090 '<FieldAccess for matching_formal_type at nodes.lkt:5821:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5821'
      if Var_Expr_126.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Var_Expr_126.Node, Formal_Type => Typ, Origin => Origin, E_Info => Var_Expr_126.Info);
--# end
--# expr-done 4090
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_To;
--# end
--# property-start BaseTypeDecl.is_subp_access_of 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5826'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Subp_Access_Of
  (Node : Bare_Base_Type_Decl
      ; Other : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_127 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4092 '<Cast to AccessToSubpDef at nodes.lkt:5826:61>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5826'
Var_Expr_127 := Ent;
if Var_Expr_127 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4091 '<FieldAccess for access_def at nodes.lkt:5826:61>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5826'
      if Var_Expr_127.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_127.Node, Origin => Origin, E_Info => Var_Expr_127.Info);
--# end
--# expr-done 4091
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 4092
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_0
--# expr-start 4093 '<FieldAccess for subp_spec_or_null at nodes.lkt:5827:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5827'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 4093
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind se Var_Expr_1
--# expr-start 4095 '<FieldAccess for match_signature at nodes.lkt:5828:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5828'
--# expr-start 4094 '<FieldAccess for subp_spec at nodes.lkt:5828:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5828'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_0.Info);
--# expr-done 4094
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Var_Expr_1, Match_Name => False, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4095
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         Property_Result := Result_Var_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Subp_Access_Of;
--# end
--# property-start BaseTypeDecl.is_generic_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5833'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Generic_Formal
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_128 : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4097 '<IsA GenericFormalTypeDecl at nodes.lkt:5834:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5834'
--# expr-start 4096 '<parent at nodes.lkt:5834:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5834'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4096
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal_Type_Decl_Range; 
--# expr-done 4097
if Is_A then
   If_Result := True;
else
--# expr-start 4099 '<Cast to BaseTypeDecl at nodes.lkt:5834:49>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5834'
--# expr-start 4098 '<parent at nodes.lkt:5834:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5834'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 4098
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4099
Var_Expr_128 := Cast_Result;
if Var_Expr_128 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4100 '<FieldAccess for is_generic_formal at nodes.lkt:5834:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5834'
      if Var_Expr_128 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Var_Expr_128);
--# end
--# expr-done 4100
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Generic_Formal;
--# end
--# property-start BaseTypeDecl.is_tagged_type_with_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5840'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4106 '<FieldAccess for is_tagged_type at nodes.lkt:5840:45>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5840'
--# expr-start 4105 '<If at nodes.lkt:5841:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5841'
--# expr-start 4103 '<Not at nodes.lkt:5841:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5841'
--# expr-start 4102 '<IsNull at nodes.lkt:5841:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5841'
--# expr-start 4101 '<FieldAccess for get_imp_deref at nodes.lkt:5841:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4101
Is_Null := Fld_1.Node = null; 
--# expr-done 4102
Not_Val := not (Is_Null); 
--# expr-done 4103
if Not_Val then
   --# expr-start 4104 '<FieldAccess for accessed_type at nodes.lkt:5841:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4104
   If_Result := Fld;
else
   If_Result := Ent;
end if;
--# expr-done 4105
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4106
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Tagged_Type_With_Deref;
--# end
--# property-start BaseTypeDecl.find_all_derived_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5848'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_All_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_618 with Inline_Always;
            procedure Finalizer_Scope_2506 with Inline_Always;
            procedure Finalizer_Scope_2507 with Inline_Always;
      Var_Expr_1 : Bare_Ada_Node;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Unit_Array_Access;
Fld_2 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_618 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_618;
            procedure Finalizer_Scope_2506 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2506;
            procedure Finalizer_Scope_2507 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2507;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4113 '<bind at nodes.lkt:5849:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5849'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 4112 '<Mapcat at nodes.lkt:5851:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5851'
   --# expr-start 4108 '<FieldAccess for filter_is_imported_by at nodes.lkt:5851:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5851'
--# expr-start 4107 '<FieldAccess for canonical_type at nodes.lkt:5851:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5851'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 4107
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Fld.Node, Units => Units, Transitive => True);
--# expr-done 4108
      declare
         Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_0
--# expr-start 4109 '<FieldAccess for root at nodes.lkt:5852:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5852'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Item_0.Ast_Root;
--# expr-done 4109
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr_1
--# expr-start 4111 '<FieldAccess for find_derived_types at nodes.lkt:5853:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5853'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4110 '<as_bare_entity at nodes.lkt:5853:48>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5853'
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 4110
--# property-call-start BaseTypeDecl.find_derived_types
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => If_Result, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4111
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2507;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2506;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4112
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4113
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_618;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_618;
                     Finalizer_Scope_2506;
                     Finalizer_Scope_2507;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_All_Derived_Types;
--# end
--# property-start BaseTypeDecl.array_def_with_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5862'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_3 : Boolean;
If_Result : Internal_Entity_Array_Type_Def;
Fld_4 : Boolean;
If_Result_1 : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4120 '<If at nodes.lkt:5862:56>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5862'
--# expr-start 4114 '<FieldAccess for is_array at nodes.lkt:5862:59>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5862'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4114
if Fld_4 then
   --# expr-start 4115 '<FieldAccess for array_def at nodes.lkt:5862:80>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5862'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4115
   If_Result_1 := Fld;
else
   --# expr-start 4119 '<If at nodes.lkt:5862:56>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5862'
--# expr-start 4116 '<FieldAccess for is_implicit_deref at nodes.lkt:5863:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5863'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4116
if Fld_3 then
--# expr-start 4117 '<FieldAccess for accessed_type at nodes.lkt:5863:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5863'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4117
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 4118 '<FieldAccess for array_def at nodes.lkt:5863:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5863'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4118
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
   If_Result := Result_Var;
else
   If_Result := No_Entity_Array_Type_Def;
end if;
--# expr-done 4119
   If_Result_1 := If_Result;
end if;
--# expr-done 4120
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5867'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4122 '<Not at nodes.lkt:5868:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5868'
--# expr-start 4121 '<IsNull at nodes.lkt:5868:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5868'
Is_Null := Self = null; 
--# expr-done 4121
Not_Val := not (Is_Null); 
--# expr-done 4122
if Not_Val then
   --# expr-start 4125 '<Not at nodes.lkt:5868:30>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5868'
--# expr-start 4124 '<IsNull at nodes.lkt:5868:34>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5868'
--# expr-start 4123 '<FieldAccess for array_def_with_deref at nodes.lkt:5868:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5868'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4123
Is_Null_1 := Fld.Node = null; 
--# expr-done 4124
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4125
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref_or_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5871'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4126 '<IsNull at nodes.lkt:5872:9>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5872'
Is_Null := Self = null; 
--# expr-done 4126
if Is_Null then
   If_Result := True;
else
   --# expr-start 4129 '<Not at nodes.lkt:5872:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5872'
--# expr-start 4128 '<IsNull at nodes.lkt:5872:29>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5872'
--# expr-start 4127 '<FieldAccess for array_def_with_deref at nodes.lkt:5872:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5872'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4127
Is_Null_1 := Fld.Node = null; 
--# expr-done 4128
Not_Val := not (Is_Null_1); 
--# expr-done 4129
   If_Result := Not_Val;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null;
--# end
--# property-start BaseTypeDecl.comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5882'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Is_Subscript : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind is_subscript Is_Subscript
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Local_Ad : Internal_Entity_Array_Type_Def;
Var_Expr_1 : Internal_Entity_Array_Type_Def;
Var_Expr_2 : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Base_Type_Access_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Access_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
Var_Expr_0 := Ent;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 4139 '<Let at nodes.lkt:5884:13>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5884'
--# scope-start
--# expr-start 4132 '<If at nodes.lkt:5884:22>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5884'
if Is_Subscript then
   --# expr-start 4130 '<FieldAccess for array_def_with_deref at nodes.lkt:5884:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5884'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4130
   If_Result := Fld;
else
   --# expr-start 4131 '<FieldAccess for array_def at nodes.lkt:5884:76>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5884'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4131
   If_Result := Fld_1;
end if;
--# expr-done 4132
Local_Ad := If_Result; 
--# bind ad Local_Ad
Var_Expr_1 := Local_Ad;
if Var_Expr_1 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr_1
--# expr-start 4133 '<FieldAccess for comp_type at nodes.lkt:5886:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5886'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4133
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 4134 '<FieldAccess for access_def at nodes.lkt:5886:47>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5886'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4134
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Access_Def then
   --# scope-start
   --# bind v1 Var_Expr_2
--# expr-start 4138 '<Match at nodes.lkt:5887:25>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5887'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_2; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind asd Match_0
--# expr-start 4136 '<FieldAccess for return_type at nodes.lkt:5888:50>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5888'
--# expr-start 4135 '<FieldAccess for subp_spec at nodes.lkt:5888:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5888'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 4135
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4136
Let_Result := Fld_5; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tad Match_1
--# expr-start 4137 '<FieldAccess for accessed_type at nodes.lkt:5889:52>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5889'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4137
Let_Result_1 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4138
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Let_Result_2 := Result_Var_2; 
--# end
--# expr-done 4139
Scope_Result_5 := Let_Result_2;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Comp_Type;
--# end
--# property-start BaseTypeDecl.index_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5902'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Index_Type
  (Node : Bare_Base_Type_Decl
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Array_Type_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 4140 '<FieldAccess for array_def_with_deref at nodes.lkt:5903:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5903'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4140
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr_0
--# expr-start 4141 '<FieldAccess for index_type at nodes.lkt:5903:48>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5903'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.index_type
Fld_1 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Var_Expr_0.Node, Dim => Dim, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4141
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Index_Type;
--# end
--# property-start BaseTypeDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5906'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expr_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Expr_Type;
--# end
--# property-start BaseTypeDecl.is_derived_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5914'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Derived_Type
  (Node : Bare_Base_Type_Decl
      ; Other_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other_type Other_Type
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_625 with Inline_Always;
      Local_Entity_Can : Internal_Entity_Base_Type_Decl;
Local_Other_Can : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Boolean;
Quantifier_Result : Boolean;
If_Result_2 : Boolean;
Fld_6 : Boolean;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Classwide_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_10 : Boolean;
Quantifier_Result_1 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_625 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_625;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4159 '<Let at nodes.lkt:5915:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5915'
--# scope-start
--# expr-start 4142 '<FieldAccess for canonical_type at nodes.lkt:5915:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5915'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4142
Local_Entity_Can := Fld; 
--# bind entity_can Local_Entity_Can
--# expr-start 4158 '<Let at nodes.lkt:5916:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5916'
--# scope-start
--# expr-start 4143 '<FieldAccess for canonical_type at nodes.lkt:5916:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5916'
      if Other_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Other_Type.Node, Origin => Origin, E_Info => Other_Type.Info);
--# end
--# expr-done 4143
Local_Other_Can := Fld_1; 
--# bind other_can Local_Other_Can
--# expr-start 4144 '<Eq at nodes.lkt:5920:13>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5920'
Cast_Expr := Local_Entity_Can; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Local_Other_Can; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 4144
if Is_Equiv then
   If_Result_1 := True;
else
--# expr-start 4145 '<FieldAccess for is_classwide at nodes.lkt:5929:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5929'
      if Local_Other_Can.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Local_Other_Can.Node, E_Info => Local_Other_Can.Info);
--# end
--# expr-done 4145
if Fld_2 then
   --# expr-start 4147 '<Eq at nodes.lkt:5929:42>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5929'
--# expr-start 4146 '<FieldAccess for classwide_type at nodes.lkt:5929:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5929'
      if Local_Entity_Can.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Local_Entity_Can.Node, E_Info => Local_Entity_Can.Info);
--# end
--# expr-done 4146
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr_2 := Local_Other_Can; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 4147
   If_Result := Is_Equiv_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4150 '<AnyQuantifier at nodes.lkt:5932:13>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5932'
   --# expr-start 4148 '<FieldAccess for base_types at nodes.lkt:5932:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5932'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4148
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_0
      --# expr-start 4149 '<FieldAccess for is_derived_type at nodes.lkt:5932:43>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5932'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_0.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Item_0.Info);
--# end
--# expr-done 4149
      Quantifier_Result := Fld_5;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 4150
   If_Result_2 := Quantifier_Result;
end if;
if If_Result_2 then
   If_Result_5 := True;
else
--# expr-start 4151 '<FieldAccess for is_classwide at nodes.lkt:5938:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
      if Local_Entity_Can.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Local_Entity_Can.Node, E_Info => Local_Entity_Can.Info);
--# end
--# expr-done 4151
if Fld_6 then
   --# expr-start 4152 '<FieldAccess for is_classwide at nodes.lkt:5938:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
      if Local_Other_Can.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Local_Other_Can.Node, E_Info => Local_Other_Can.Info);
--# end
--# expr-done 4152
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 4157 '<AnyQuantifier at nodes.lkt:5938:72>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
   --# expr-start 4155 '<FieldAccess for base_types at nodes.lkt:5938:72>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
--# expr-start 4154 '<FieldAccess for type_decl at nodes.lkt:5938:72>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
--# expr-start 4153 '<Cast to ClasswideTypeDecl at nodes.lkt:5938:72>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
Cast_Expr_3 := Local_Entity_Can; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 4153
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_8 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 4154
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4155
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_9;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_1
      --# expr-start 4156 '<FieldAccess for is_derived_type at nodes.lkt:5938:142>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5938'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_1.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Item_1.Info);
--# end
--# expr-done 4156
      Quantifier_Result_1 := Fld_10;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 4157
   If_Result_4 := Quantifier_Result_1;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 4158
Let_Result_1 := Let_Result; 
--# end
--# expr-done 4159
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_625;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_625;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Derived_Type;
--# end
--# property-start BaseTypeDecl.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5954'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Limited_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Incomplete_Type_Decl;
Match_3 : Internal_Entity_Classwide_Type_Decl;
Match_4 : Internal_Entity_Protected_Type_Decl;
Match_5 : Internal_Entity_Task_Type_Decl;
Match_6 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_3 : Internal_Entity_Classwide_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Protected_Type_Decl;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Task_Type_Decl;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Match_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4168 '<Match at nodes.lkt:5958:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5958'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind td Match_0
--# expr-start 4161 '<FieldAccess for is_limited_type at nodes.lkt:5959:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5959'
--# expr-start 4160 '<FieldAccess for type_def at nodes.lkt:5959:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5959'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 4160
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_limited_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Limited_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4161
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind sb Match_1
--# expr-start 4163 '<FieldAccess for is_limited_type at nodes.lkt:5960:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5960'
--# expr-start 4162 '<FieldAccess for get_type at nodes.lkt:5960:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5960'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => No_Bare_Ada_Node, E_Info => Match_1.Info);
--# end
--# expr-done 4162
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4163
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind it Match_2
--# expr-start 4165 '<FieldAccess for is_limited_type at nodes.lkt:5961:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5961'
--# expr-start 4164 '<FieldAccess for full_view at nodes.lkt:5961:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5961'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4164
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 4165
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind cw Match_3
--# expr-start 4167 '<FieldAccess for is_limited_type at nodes.lkt:5962:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5962'
--# expr-start 4166 '<FieldAccess for type_decl at nodes.lkt:5962:43>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5962'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_6 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 4166
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 4167
Let_Result_3 := Fld_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := True; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := True; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_6; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 4168
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Limited_Type;
--# end
--# property-start BaseTypeDecl.iterable_comp_type_or_null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5969'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Comp_Type_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4174 '<If at nodes.lkt:5970:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5970'
--# expr-start 4169 '<IsNull at nodes.lkt:5970:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5970'
Is_Null := Self = null; 
--# expr-done 4169
if Is_Null then
   If_Result_1 := No_Entity_Base_Type_Decl;
else
   --# expr-start 4173 '<FieldAccess for iterable_comp_type at nodes.lkt:5970:62>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5970'
--# expr-start 4172 '<If at nodes.lkt:5971:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5971'
--# expr-start 4170 '<FieldAccess for is_implicit_deref at nodes.lkt:5971:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5971'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4170
if Fld_1 then
   --# expr-start 4171 '<FieldAccess for accessed_type at nodes.lkt:5971:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5971'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4171
   If_Result := Fld;
else
   If_Result := Ent;
end if;
--# expr-done 4172
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4173
   If_Result_1 := Fld_2;
end if;
--# expr-done 4174
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Iterable_Comp_Type_Or_Null;
--# end
--# property-start BaseTypeDecl.matching_prefix_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5978'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Prefix_Type
  (Node : Bare_Base_Type_Decl
      ; Container_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind container_type Container_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Cont_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_129 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4184 '<Let at nodes.lkt:5979:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5979'
--# scope-start
Local_Cont_Type := Container_Type; 
--# bind cont_type Local_Cont_Type
--# expr-start 4176 '<Not at nodes.lkt:5982:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5982'
--# expr-start 4175 '<IsNull at nodes.lkt:5982:17>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5982'
Is_Null := Self = null; 
--# expr-done 4175
Not_Val := not (Is_Null); 
--# expr-done 4176
if Not_Val then
   --# expr-start 4178 '<Not at nodes.lkt:5982:34>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5982'
--# expr-start 4177 '<IsNull at nodes.lkt:5982:38>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5982'
Is_Null_1 := Container_Type.Node = null; 
--# expr-done 4177
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4178
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 4179 '<FieldAccess for matching_formal_prim_type at nodes.lkt:5986:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5986'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Local_Cont_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4179
if Fld then
   If_Result_1 := True;
else
--# expr-start 4180 '<FieldAccess for final_accessed_type at nodes.lkt:5989:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5989'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4180
Var_Expr_129 := Fld_1;
if Var_Expr_129 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4181 '<FieldAccess for matching_formal_prim_type at nodes.lkt:5989:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5989'
      if Var_Expr_129.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Var_Expr_129.Node, Formal_Type => Local_Cont_Type, Origin => Origin, E_Info => Var_Expr_129.Info);
--# end
--# expr-done 4181
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 4182 '<FieldAccess for accessed_type at nodes.lkt:5993:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5993'
      if Local_Cont_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Local_Cont_Type.Node, Origin => Origin, E_Info => Local_Cont_Type.Info);
--# end
--# expr-done 4182
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_0
--# expr-start 4183 '<FieldAccess for matching_formal_prim_type at nodes.lkt:5994:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:5994'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Var_Expr_0, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4183
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_2 := Result_Var_1;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 4184
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Prefix_Type;
--# end
--# property-start BaseTypeDecl.matching_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6002'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Actual_Type : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Anonymous_Type_Decl;
Match_1_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Anonymous_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Match_Result_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4189 '<Let at nodes.lkt:6003:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6003'
--# scope-start
Local_Actual_Type := Ent; 
--# bind actual_type Local_Actual_Type
--# expr-start 4188 '<Match at nodes.lkt:6005:9>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6005'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Expected_Type; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_2; 
--# bind atd Match_0
--# expr-start 4185 '<FieldAccess for access_def_matches at nodes.lkt:6006:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6006'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0.Node, Other => Local_Actual_Type, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4185
Let_Result_2 := Fld; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
--# expr-start 4187 '<Match at nodes.lkt:6007:23>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6007'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Actual_Type; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind atd2 Match_0_1
--# expr-start 4186 '<FieldAccess for access_def_matches at nodes.lkt:6008:57>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6008'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld_1 := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0_1.Node, Other => Expected_Type, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 4186
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4187
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
end case;
--# expr-done 4188
Let_Result_4 := Match_Result_1; 
--# end
--# expr-done 4189
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Access_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_prim_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6016'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Prim_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4191 '<Not at nodes.lkt:6017:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6017'
--# expr-start 4190 '<IsNull at nodes.lkt:6017:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6017'
Is_Null := Formal_Type.Node = null; 
--# expr-done 4190
Not_Val := not (Is_Null); 
--# expr-done 4191
if Not_Val then
   --# expr-start 4193 '<Not at nodes.lkt:6017:37>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6017'
--# expr-start 4192 '<IsNull at nodes.lkt:6017:41>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6017'
Is_Null_1 := Self = null; 
--# expr-done 4192
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4193
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4194 '<FieldAccess for matching_formal_type_impl at nodes.lkt:6017:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6017'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Accept_Derived => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4194
   If_Result_1 := Fld;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Prim_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6021'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4196 '<Not at nodes.lkt:6022:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6022'
--# expr-start 4195 '<IsNull at nodes.lkt:6022:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6022'
Is_Null := Formal_Type.Node = null; 
--# expr-done 4195
Not_Val := not (Is_Null); 
--# expr-done 4196
if Not_Val then
   --# expr-start 4198 '<Not at nodes.lkt:6022:37>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6022'
--# expr-start 4197 '<IsNull at nodes.lkt:6022:41>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6022'
Is_Null_1 := Self = null; 
--# expr-done 4197
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4198
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4199 '<FieldAccess for matching_formal_type_impl at nodes.lkt:6022:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6022'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4199
   If_Result_1 := Fld;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Type;
--# end
--# property-start BaseTypeDecl.matching_membership_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6025'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Membership_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4201 '<Not at nodes.lkt:6026:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6026'
--# expr-start 4200 '<IsNull at nodes.lkt:6026:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6026'
Is_Null := Formal_Type.Node = null; 
--# expr-done 4200
Not_Val := not (Is_Null); 
--# expr-done 4201
if Not_Val then
   --# expr-start 4203 '<Not at nodes.lkt:6026:37>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6026'
--# expr-start 4202 '<IsNull at nodes.lkt:6026:41>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6026'
Is_Null_1 := Self = null; 
--# expr-done 4202
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4203
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4204 '<FieldAccess for matching_formal_type_impl at nodes.lkt:6026:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6026'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Accept_Root_Types => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4204
   If_Result_1 := Fld;
else
   If_Result_1 := False;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Membership_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6029'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Accept_Derived : Boolean
         := False
      ; Accept_Root_Types : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind accept_derived Accept_Derived
   --# bind accept_root_types Accept_Root_Types
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Actual_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result_4 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_5 : Boolean;
Fld_11 : Boolean;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_10 : Boolean;
Fld_16 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Boolean;
Fld_19 : Boolean;
If_Result_13 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4227 '<Let at nodes.lkt:6030:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6030'
--# scope-start
Local_Actual_Type := Ent; 
--# bind actual_type Local_Actual_Type
--# expr-start 4205 '<FieldAccess for is_classwide at nodes.lkt:6034:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6034'
      if Formal_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Formal_Type.Node, E_Info => Formal_Type.Info);
--# end
--# expr-done 4205
if Fld then
   If_Result := True;
else
   If_Result := Accept_Derived;
end if;
if If_Result then
   --# expr-start 4207 '<FieldAccess for is_derived_type at nodes.lkt:6035:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6035'
--# expr-start 4206 '<FieldAccess for specific_type at nodes.lkt:6035:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6035'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Local_Actual_Type.Node, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4206
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld_1.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4207
   If_Result_1 := Fld_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 4208 '<FieldAccess for is_classwide at nodes.lkt:6037:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6037'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Local_Actual_Type.Node, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4208
if Fld_3 then
   --# expr-start 4210 '<FieldAccess for matching_type at nodes.lkt:6037:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6037'
--# expr-start 4209 '<FieldAccess for specific_type at nodes.lkt:6037:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6037'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Local_Actual_Type.Node, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4209
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_4.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4210
   If_Result_2 := Fld_5;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_10 := True;
else
--# expr-start 4211 '<FieldAccess for accessed_type at nodes.lkt:6040:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6040'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4211
Var_Expr_0 := Fld_6;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind actual_accessed_type Var_Expr_0
--# expr-start 4212 '<FieldAccess for accessed_type at nodes.lkt:6041:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6041'
      if Formal_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Formal_Type.Node, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 4212
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind formal_accessed_type Var_Expr_1
--# expr-start 4213 '<FieldAccess for is_classwide at nodes.lkt:6044:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6044'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4213
if Fld_8 then
   If_Result_4 := True;
else
   If_Result_4 := Accept_Derived;
end if;
if If_Result_4 then
   --# expr-start 4215 '<FieldAccess for is_derived_type at nodes.lkt:6045:31>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6045'
--# expr-start 4214 '<FieldAccess for specific_type at nodes.lkt:6045:31>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6045'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4214
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld_9.Node, Other_Type => Var_Expr_1, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 4215
   If_Result_5 := Fld_10;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 4216 '<FieldAccess for is_classwide at nodes.lkt:6047:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6047'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4216
if Fld_11 then
   --# expr-start 4218 '<FieldAccess for matching_type at nodes.lkt:6047:65>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6047'
--# expr-start 4217 '<FieldAccess for specific_type at nodes.lkt:6047:65>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6047'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4217
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_12.Node, Expected_Type => Var_Expr_1, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 4218
   If_Result_6 := Fld_13;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
if If_Result_7 then
   If_Result_9 := True;
else
if Accept_Root_Types then
   --# expr-start 4220 '<FieldAccess for is_derived_type at nodes.lkt:6052:47>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6052'
--# expr-start 4219 '<FieldAccess for specific_type at nodes.lkt:6052:47>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6052'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4219
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld_14.Node, Other_Type => Var_Expr_0, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 4220
   If_Result_8 := Fld_15;
else
   If_Result_8 := False;
end if;
   If_Result_9 := If_Result_8;
end if;
Scope_Result := If_Result_9;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_10 := Result_Var_1;
end if;
if If_Result_10 then
   If_Result_12 := True;
else
--# expr-start 4223 '<Not at nodes.lkt:6057:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6057'
--# expr-start 4222 '<IsNull at nodes.lkt:6057:17>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6057'
--# expr-start 4221 '<FieldAccess for get_imp_deref at nodes.lkt:6057:17>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6057'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Local_Actual_Type.Node, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4221
Is_Null := Fld_16.Node = null; 
--# expr-done 4222
Not_Val := not (Is_Null); 
--# expr-done 4223
if Not_Val then
   --# expr-start 4225 '<FieldAccess for matching_formal_type at nodes.lkt:6057:57>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6057'
--# expr-start 4224 '<FieldAccess for accessed_type at nodes.lkt:6057:57>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6057'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4224
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_17.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 4225
   If_Result_11 := Fld_18;
else
   If_Result_11 := False;
end if;
   If_Result_12 := If_Result_11;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 4226 '<FieldAccess for matching_type at nodes.lkt:6058:14>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6058'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_19 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Local_Actual_Type.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4226
   If_Result_13 := Fld_19;
end if;
Let_Result := If_Result_13; 
--# end
--# expr-done 4227
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Formal_Type_Impl;
--# end
--# property-start BaseTypeDecl.matching_assign_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6063'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Assign_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Actual_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_130 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
Fld_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_5 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4238 '<Let at nodes.lkt:6064:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6064'
--# scope-start
Local_Actual_Type := Ent; 
--# bind actual_type Local_Actual_Type
--# expr-start 4229 '<Not at nodes.lkt:6067:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6067'
--# expr-start 4228 '<IsNull at nodes.lkt:6067:17>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6067'
Is_Null := Self = null; 
--# expr-done 4228
Not_Val := not (Is_Null); 
--# expr-done 4229
if Not_Val then
   --# expr-start 4231 '<Not at nodes.lkt:6067:34>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6067'
--# expr-start 4230 '<IsNull at nodes.lkt:6067:38>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6067'
Is_Null_1 := Expected_Type.Node = null; 
--# expr-done 4230
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4231
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 4232 '<FieldAccess for matching_type at nodes.lkt:6069:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6069'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Ent.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4232
if Fld then
   If_Result_3 := True;
else
--# expr-start 4233 '<FieldAccess for is_classwide at nodes.lkt:6071:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6071'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 4233
if Fld_1 then
   If_Result_1 := True;
else
--# expr-start 4234 '<FieldAccess for accessed_type at nodes.lkt:6071:53>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6071'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4234
Var_Expr_130 := Fld_2;
if Var_Expr_130 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4235 '<FieldAccess for is_classwide at nodes.lkt:6071:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6071'
      if Var_Expr_130.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_130.Node, E_Info => Var_Expr_130.Info);
--# end
--# expr-done 4235
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
end if;
if If_Result_1 then
   --# expr-start 4236 '<FieldAccess for matching_formal_prim_type at nodes.lkt:6072:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6072'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Local_Actual_Type.Node, Formal_Type => Expected_Type, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4236
   If_Result_2 := Fld_4;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 4237 '<FieldAccess for matching_access_type at nodes.lkt:6073:18>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6073'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Ent.Node, Expected_Type => Expected_Type, For_Assignment => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4237
   If_Result_4 := Fld_5;
end if;
   If_Result_5 := If_Result_4;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 4238
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Assign_Type;
--# end
--# property-start BaseTypeDecl.matching_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6081'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Actual_Type : Internal_Entity_Base_Type_Decl;
Local_Uit : Internal_Entity_Base_Type_Decl;
Local_Urt : Internal_Entity_Base_Type_Decl;
Local_Uft : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_4 : Boolean;
If_Result_4 : Boolean;
Cast_Expr_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Cast_Expr_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_5 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Let_Result_1 : Boolean;
If_Result_7 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Expr_8 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Cast_Expr_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity;
Is_Equiv_4 : Boolean;
Fld_7 : Boolean;
If_Result_8 : Boolean;
Cast_Expr_10 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Cast_Expr_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Is_Equiv_5 : Boolean;
Fld_8 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Let_Result_2 : Boolean;
If_Result_11 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Base_Type_Decl;
Cast_Result_12 : Internal_Entity;
Cast_Result_13 : Internal_Entity;
Is_Equiv_6 : Boolean;
If_Result_12 : Boolean;
Fld_11 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
Not_Val_2 : Boolean;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result_13 : Boolean;
If_Result_14 : Boolean;
Fld_14 : Internal_Entity_Expr;
Is_Null_3 : Boolean;
Not_Val_3 : Boolean;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
If_Result_15 : Boolean;
If_Result_16 : Boolean;
Fld_17 : Boolean;
If_Result_17 : Boolean;
If_Result_18 : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4275 '<Let at nodes.lkt:6082:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6082'
--# scope-start
Local_Actual_Type := Ent; 
--# bind actual_type Local_Actual_Type
--# expr-start 4240 '<Not at nodes.lkt:6084:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6084'
--# expr-start 4239 '<IsNull at nodes.lkt:6084:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6084'
Is_Null := Expected_Type.Node = null; 
--# expr-done 4239
Not_Val := not (Is_Null); 
--# expr-done 4240
if Not_Val then
   --# expr-start 4242 '<Not at nodes.lkt:6084:39>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6084'
--# expr-start 4241 '<IsNull at nodes.lkt:6084:43>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6084'
Is_Null_1 := Local_Actual_Type.Node = null; 
--# expr-done 4241
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4242
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 4248 '<Let at nodes.lkt:6086:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6086'
--# scope-start
--# expr-start 4243 '<FieldAccess for universal_int_type at nodes.lkt:6086:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6086'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 4243
Local_Uit := Fld; 
--# bind uit Local_Uit
--# expr-start 4244 '<Eq at nodes.lkt:6089:21>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6089'
Cast_Expr := Local_Actual_Type; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Local_Uit; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 4244
if Is_Equiv then
   --# expr-start 4245 '<FieldAccess for allows_universal_int at nodes.lkt:6089:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6089'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.allows_universal_int
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Allows_Universal_Int (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4245
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 4246 '<Eq at nodes.lkt:6091:21>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6091'
Cast_Expr_2 := Expected_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Cast_Expr_3 := Local_Uit; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 4246
if Is_Equiv_1 then
   --# expr-start 4247 '<FieldAccess for is_int_type at nodes.lkt:6091:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6091'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4247
   If_Result_2 := Fld_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 4248
if Let_Result then
   If_Result_7 := True;
else
   --# expr-start 4254 '<Let at nodes.lkt:6094:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6094'
--# scope-start
--# expr-start 4249 '<FieldAccess for universal_real_type at nodes.lkt:6094:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 4249
Local_Urt := Fld_3; 
--# bind urt Local_Urt
--# expr-start 4250 '<Eq at nodes.lkt:6097:21>' Is_Equiv_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6097'
Cast_Expr_4 := Local_Actual_Type; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Cast_Expr_5 := Local_Urt; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 4250
if Is_Equiv_2 then
   --# expr-start 4251 '<FieldAccess for allows_universal_real at nodes.lkt:6097:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6097'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.allows_universal_real
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Allows_Universal_Real (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4251
   If_Result_4 := Fld_4;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
--# expr-start 4252 '<Eq at nodes.lkt:6099:21>' Is_Equiv_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6099'
Cast_Expr_6 := Expected_Type; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
Cast_Expr_7 := Local_Urt; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
Is_Equiv_3 := Equivalent (Cast_Result_6, Cast_Result_7); 
--# expr-done 4252
if Is_Equiv_3 then
   --# expr-start 4253 '<FieldAccess for is_real_type at nodes.lkt:6099:46>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6099'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4253
   If_Result_5 := Fld_5;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
Let_Result_1 := If_Result_6; 
--# end
--# expr-done 4254
   If_Result_7 := Let_Result_1;
end if;
if If_Result_7 then
   If_Result_11 := True;
else
   --# expr-start 4260 '<Let at nodes.lkt:6102:17>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6102'
--# scope-start
--# expr-start 4255 '<FieldAccess for universal_fixed_type at nodes.lkt:6102:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6102'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 4255
Local_Uft := Fld_6; 
--# bind uft Local_Uft
--# expr-start 4256 '<Eq at nodes.lkt:6105:21>' Is_Equiv_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6105'
Cast_Expr_8 := Expected_Type; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_8.Node,
         Info => Cast_Expr_8.Info);
Cast_Expr_9 := Local_Uft; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_9.Node,
         Info => Cast_Expr_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 4256
if Is_Equiv_4 then
   --# expr-start 4257 '<FieldAccess for is_fixed_point at nodes.lkt:6105:46>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6105'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4257
   If_Result_8 := Fld_7;
else
   If_Result_8 := False;
end if;
if If_Result_8 then
   If_Result_10 := True;
else
--# expr-start 4258 '<Eq at nodes.lkt:6107:21>' Is_Equiv_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6107'
Cast_Expr_10 := Local_Actual_Type; 
      Cast_Result_10 := Create_Internal_Entity
        (Node => Cast_Expr_10.Node,
         Info => Cast_Expr_10.Info);
Cast_Expr_11 := Local_Uft; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_11.Node,
         Info => Cast_Expr_11.Info);
Is_Equiv_5 := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 4258
if Is_Equiv_5 then
   --# expr-start 4259 '<FieldAccess for is_fixed_point at nodes.lkt:6107:44>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6107'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4259
   If_Result_9 := Fld_8;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
Let_Result_2 := If_Result_10; 
--# end
--# expr-done 4260
   If_Result_11 := Let_Result_2;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 4263 '<Eq at nodes.lkt:6109:18>' Is_Equiv_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6109'
--# expr-start 4261 '<FieldAccess for canonical_type at nodes.lkt:6109:18>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6109'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4261
      Cast_Result_12 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
--# expr-start 4262 '<FieldAccess for canonical_type at nodes.lkt:6109:50>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6109'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4262
      Cast_Result_13 := Create_Internal_Entity
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
Is_Equiv_6 := Equivalent (Cast_Result_12, Cast_Result_13); 
--# expr-done 4263
   If_Result_12 := Is_Equiv_6;
end if;
if If_Result_12 then
   If_Result_14 := True;
else
--# expr-start 4266 '<Not at nodes.lkt:6110:17>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6110'
--# expr-start 4265 '<IsNull at nodes.lkt:6110:21>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6110'
--# expr-start 4264 '<FieldAccess for get_imp_deref at nodes.lkt:6110:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6110'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Local_Actual_Type.Node, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4264
Is_Null_2 := Fld_11.Node = null; 
--# expr-done 4265
Not_Val_2 := not (Is_Null_2); 
--# expr-done 4266
if Not_Val_2 then
   --# expr-start 4268 '<FieldAccess for matching_type at nodes.lkt:6110:61>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6110'
--# expr-start 4267 '<FieldAccess for accessed_type at nodes.lkt:6110:61>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6110'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Local_Actual_Type.Node, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4267
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_12.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 4268
   If_Result_13 := Fld_13;
else
   If_Result_13 := False;
end if;
   If_Result_14 := If_Result_13;
end if;
if If_Result_14 then
   If_Result_16 := True;
else
--# expr-start 4271 '<Not at nodes.lkt:6112:17>' Not_Val_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6112'
--# expr-start 4270 '<IsNull at nodes.lkt:6112:21>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6112'
--# expr-start 4269 '<FieldAccess for get_imp_deref at nodes.lkt:6112:21>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6112'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_14 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 4269
Is_Null_3 := Fld_14.Node = null; 
--# expr-done 4270
Not_Val_3 := not (Is_Null_3); 
--# expr-done 4271
if Not_Val_3 then
   --# expr-start 4273 '<FieldAccess for matching_type at nodes.lkt:6112:63>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6112'
--# expr-start 4272 '<FieldAccess for accessed_type at nodes.lkt:6112:63>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6112'
      if Expected_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 4272
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_15.Node, Expected_Type => Local_Actual_Type, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 4273
   If_Result_15 := Fld_16;
else
   If_Result_15 := False;
end if;
   If_Result_16 := If_Result_15;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 4274 '<FieldAccess for matching_access_type at nodes.lkt:6113:18>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6113'
      if Local_Actual_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Local_Actual_Type.Node, Expected_Type => Expected_Type, For_Assignment => False, Origin => Origin, E_Info => Local_Actual_Type.Info);
--# end
--# expr-done 4274
   If_Result_17 := Fld_17;
end if;
   If_Result_18 := If_Result_17;
else
   If_Result_18 := False;
end if;
Let_Result_3 := If_Result_18; 
--# end
--# expr-done 4275
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Type;
--# end
--# property-start BaseTypeDecl.matching_allocator_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6118'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Allocator_Type
  (Node : Bare_Base_Type_Decl
      ; Allocated_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind allocated_type Allocated_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4276 '<FieldAccess for is_access_type at nodes.lkt:6119:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6119'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4276
if Fld then
   --# expr-start 4278 '<FieldAccess for matching_formal_type at nodes.lkt:6119:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6119'
      if Allocated_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4277 '<FieldAccess for accessed_type at nodes.lkt:6119:71>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6119'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4277
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Allocated_Type.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Allocated_Type.Info);
--# end
--# expr-done 4278
   If_Result := Fld_2;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Matching_Allocator_Type;
--# end
--# property-start BaseTypeDecl.derives_from_std_bool_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6125'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Derives_From_Std_Bool_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4280 '<Not at nodes.lkt:6126:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6126'
--# expr-start 4279 '<IsNull at nodes.lkt:6126:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6126'
Is_Null := Self = null; 
--# expr-done 4279
Not_Val := not (Is_Null); 
--# expr-done 4280
if Not_Val then
   --# expr-start 4283 '<FieldAccess for is_derived_type at nodes.lkt:6126:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6126'
--# expr-start 4281 '<FieldAccess for derefed_base_subtype at nodes.lkt:6126:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6126'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.derefed_base_subtype
Fld := Libadalang.Implementation.Base_Type_Decl_P_Derefed_Base_Subtype (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4281
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4282 '<FieldAccess for bool_type at nodes.lkt:6126:74>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6126'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 4282
--# property-call-start BaseTypeDecl.is_derived_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Fld.Node, Other_Type => Fld_1, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4283
   If_Result := Fld_2;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Derives_From_Std_Bool_Type;
--# end
--# property-start BaseTypeDecl.classwide_type_node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6129'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Classwide_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Classwide_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Classwide_Type_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Classwide_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4285 '<New ClasswideTypeDecl at nodes.lkt:6130:9>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6130'
--# expr-start 4284 '<FieldAccess for name at nodes.lkt:6130:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6130'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 4284
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Classwide_Type_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Classwide_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Classwide_Type_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 4285
         Property_Result := New_Node;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Bare_Classwide_Type_Decl,
                        As_Bare_Classwide_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Classwide_Type_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype_node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6135'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Discrete_Base_Subtype_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Discrete_Base_Subtype_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Scalar_Base_Subtype_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Discrete_Base_Subtype_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4287 '<New DiscreteBaseSubtypeDecl at nodes.lkt:6136:9>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6136'
--# expr-start 4286 '<FieldAccess for name at nodes.lkt:6136:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6136'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 4286
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Discrete_Base_Subtype_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Discrete_Base_Subtype_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Discrete_Base_Subtype_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 4287
         Property_Result := New_Node;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Bare_Discrete_Base_Subtype_Decl,
                        As_Bare_Discrete_Base_Subtype_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Subtype_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6140'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4289 '<as_entity at nodes.lkt:6141:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6141'
--# expr-start 4288 '<FieldAccess for scalar_base_subtype_node at nodes.lkt:6141:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6141'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 4288
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 4289
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Scalar_Base_Subtype;
--# end
--# property-start BaseTypeDecl.previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6146'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2535 with Inline_Always;
            procedure Finalizer_Scope_2536 with Inline_Always;
      Var_Expr_0 : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_131 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Bare_Defining_Name;
Fld_4 : Internal_Entity;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Symbol_Type;
Cast_Expr_1 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_7 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Find_Result : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Is_A_1 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2535 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2535;
            procedure Finalizer_Scope_2536 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2536;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Go_To_Incomplete);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4308 '<If at nodes.lkt:6146:78>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6146'
--# expr-start 4290 '<FieldAccess for is_generic_formal at nodes.lkt:6146:81>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6146'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Self);
--# end
--# expr-done 4290
if Fld_9 then
   If_Result_4 := No_Entity_Base_Type_Decl;
else
   --# expr-start 4307 '<If at nodes.lkt:6146:78>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6146'
--# expr-start 4291 '<IsA ClasswideTypeDecl at nodes.lkt:6150:10>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6150'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Classwide_Type_Decl_Range; 
--# expr-done 4291
if Is_A_1 then
--# expr-start 4294 '<FieldAccess for previous_part at nodes.lkt:6150:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6150'
--# expr-start 4293 '<FieldAccess for type_decl at nodes.lkt:6150:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6150'
--# expr-start 4292 '<Cast to ClasswideTypeDecl at nodes.lkt:6150:41>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6150'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 4292
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 4293
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => Go_To_Incomplete, E_Info => Fld.Info);
--# end
--# expr-done 4294
Var_Expr_3 := Fld_1;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_3
--# expr-start 4295 '<FieldAccess for classwide_type at nodes.lkt:6150:123>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6150'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 4295
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_3 := Cast_Result_4;
else
--# expr-start 4296 '<FieldAccess for name at nodes.lkt:6153:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6153'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Base_Type_Decl_F_Name;
--# expr-done 4296
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind type_name Var_Expr_0
--# expr-start 4306 '<Cast to BaseTypeDecl at nodes.lkt:6154:24>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6154'
--# expr-start 4300 '<EnvGet at nodes.lkt:6154:24>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6154'
--# expr-start 4298 '<FieldAccess for immediate_declarative_region at nodes.lkt:6154:24>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6154'
--# expr-start 4297 '<FieldAccess for semantic_parent at nodes.lkt:6154:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6154'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_4 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4297
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 4298
--# expr-start 4299 '<FieldAccess for name_symbol at nodes.lkt:6155:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6155'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_0);
--# end
--# expr-done 4299
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_5, Key => Thin (Fld_6), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True), From => Cast_Result_1)); 
--# expr-done 4300
Var_Expr_1 := Env_Get_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 4305 '<Find at nodes.lkt:6157:21>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6157'
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind pp Item_2
--# expr-start 4301 '<FieldAccess for is_in_private_part at nodes.lkt:6159:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6159'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4301
if Fld_7 then
--# expr-start 4302 '<Cast to BaseTypeDecl at nodes.lkt:6159:51>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6159'
Cast_Expr_2 := Item_2; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4302
Var_Expr_131 := Cast_Result_2;
if Var_Expr_131 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4303 '<FieldAccess for is_private at nodes.lkt:6159:51>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6159'
      if Var_Expr_131.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Var_Expr_131.Node, E_Info => Var_Expr_131.Info);
--# end
--# expr-done 4303
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
if Go_To_Incomplete then
   --# expr-start 4304 '<IsA IncompleteTypeDecl at nodes.lkt:6161:42>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6161'
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Incomplete_Type_Decl_Range; 
--# expr-done 4304
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
      if If_Result_2 then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when If_Result_2;
         end loop;
      end;
--# expr-done 4305
Scope_Result_2 := Find_Result;
   --# end
      Finalizer_Scope_2536;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
   if Result_Var_2.Node = null
      or else Result_Var_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4306
Scope_Result_3 := Cast_Result_3;
   --# end
      Finalizer_Scope_2535;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
   If_Result_3 := Result_Var_3;
end if;
--# expr-done 4307
   If_Result_4 := If_Result_3;
end if;
--# expr-done 4308
         Property_Result := If_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2535;
                     Finalizer_Scope_2536;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.next_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6175'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Var_Expr_0 : Bare_Declarative_Part;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Package_Body;
Match_1 : Internal_Entity_Classwide_Type_Decl;
Local_Td : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Declarative_Part;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_5 : Internal_Entity_Package_Body;
Fld_6 : Internal_Entity_Declarative_Part;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Classwide_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result_4 : Internal_Entity_Classwide_Type_Decl;
Result_Var_4 : Internal_Entity_Classwide_Type_Decl;
Let_Result : Internal_Entity_Classwide_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Fld_13 : Boolean;
Not_Val : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_5 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Next_Part,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4331 '<Match at nodes.lkt:6175:45>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6175'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind itd Match_0
--# expr-start 4309 '<FieldAccess for declarative_scope at nodes.lkt:6176:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6176'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Match_0.Node);
--# end
--# expr-done 4309
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind s Var_Expr_0
--# expr-start 4311 '<FieldAccess for find_next_part_in at nodes.lkt:6179:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6179'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4310 '<as_entity at nodes.lkt:6179:42>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6179'
Is_Null := Var_Expr_0 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_0); 
   If_Result := As_Entity;
end if;
--# expr-done 4310
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_1 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => If_Result, E_Info => Match_0.Info);
--# end
--# expr-done 4311
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_1;
else
--# expr-start 4312 '<FieldAccess for is_in_private_part at nodes.lkt:6180:15>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6180'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4312
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 4317 '<FieldAccess for body_part at nodes.lkt:6181:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6181'
--# expr-start 4316 '<as_entity at nodes.lkt:6181:20>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6181'
--# expr-start 4315 '<Cast to BasePackageDecl at nodes.lkt:6181:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6181'
--# expr-start 4314 '<parent at nodes.lkt:6181:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6181'
--# expr-start 4313 '<FieldAccess for declarative_scope at nodes.lkt:6181:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6181'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_3 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 4313
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 4314
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_4;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4315
Is_Null_1 := Cast_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4316
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_5 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 4317
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Package_Body then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 4319 '<FieldAccess for find_next_part_in at nodes.lkt:6185:24>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6185'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4318 '<FieldAccess for decls at nodes.lkt:6185:46>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6185'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_2.Node.Package_Body_F_Decls, Info => Var_Expr_2.Info);
--# expr-done 4318
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_7 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => Fld_6, E_Info => Match_0.Info);
--# end
--# expr-done 4319
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result_1 := Result_Var_3; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind cwt Match_1
--# expr-start 4325 '<Let at nodes.lkt:6189:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6189'
--# scope-start
--# expr-start 4320 '<FieldAccess for type_decl at nodes.lkt:6189:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6189'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld_8 := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4320
Local_Td := Fld_8; 
--# bind td Local_Td
--# expr-start 4321 '<FieldAccess for next_part at nodes.lkt:6191:13>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6191'
      if Local_Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Local_Td.Node, E_Info => Local_Td.Info);
--# end
--# expr-done 4321
Var_Expr_3 := Fld_9;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind np Var_Expr_3
--# expr-start 4324 '<If at nodes.lkt:6194:25>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6194'
--# expr-start 4322 '<Eq at nodes.lkt:6194:28>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6194'
Cast_Expr := Local_Td; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr_3; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4322
if Is_Equiv then
   If_Result_2 := Match_1;
else
   --# expr-start 4323 '<FieldAccess for classwide_type at nodes.lkt:6194:51>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6194'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 4323
   If_Result_2 := Fld_10;
end if;
--# expr-done 4324
Scope_Result_4 := If_Result_2;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Classwide_Type_Decl;
end if;
Let_Result := Result_Var_4; 
--# end
--# expr-done 4325
      Cast_Result_5 := Create_Internal_Entity_Base_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Protected_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
--# expr-start 4330 '<If at nodes.lkt:6197:19>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6197'
--# expr-start 4326 '<FieldAccess for is_private at nodes.lkt:6197:22>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6197'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4326
if Fld_12 then
   --# expr-start 4328 '<Not at nodes.lkt:6197:44>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6197'
--# expr-start 4327 '<FieldAccess for is_generic_formal at nodes.lkt:6197:48>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6197'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Ent.Node);
--# end
--# expr-done 4327
Not_Val := not (Fld_13); 
--# expr-done 4328
   If_Result_3 := Not_Val;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 4329 '<FieldAccess for private_completion at nodes.lkt:6197:78>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6197'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4329
   If_Result_4 := Fld_11;
else
   If_Result_4 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 4330
Let_Result_3 := If_Result_4; 
--# end
Scope_Result_7 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 4331
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Next_Part;
--# end
--# property-start BaseTypeDecl.full_view 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6202'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Full_View
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 4332 '<FieldAccess for next_part at nodes.lkt:6203:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6203'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4332
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind np Var_Expr_0
--# expr-start 4333 '<FieldAccess for full_view at nodes.lkt:6203:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6203'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4333
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Full_View;
--# end
--# property-start BaseTypeDecl.is_definite_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6211'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Definite_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Type_Decl;
Match_0_1 : Internal_Entity_Derived_Type_Def;
Match_1_1 : Internal_Entity_Array_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Match_2_1 : Internal_Entity_Subtype_Decl;
Match_3 : Internal_Entity_Classwide_Type_Decl;
Match_4 : Internal_Entity_Task_Type_Decl;
Match_5 : Internal_Entity_Protected_Type_Decl;
Match_6 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Discriminant_Part;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Constraint;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Array_Indices;
Is_A : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Fld_7 : Internal_Entity_Subtype_Indication;
Fld_8 : Internal_Entity_Constraint;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Internal_Entity_Discriminant_Part;
Is_Null_3 : Boolean;
Fld_12 : Internal_Entity_Discriminant_Part;
Is_Null_4 : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Subtype_Decl;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Classwide_Type_Decl;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_7 : Internal_Entity_Task_Type_Decl;
Let_Result_7 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_8 : Internal_Entity_Protected_Type_Decl;
Let_Result_8 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_9 : Internal_Entity_Base_Type_Decl;
Let_Result_9 : Boolean;
Scope_Result_9 : Boolean;
Match_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4356 '<Match at nodes.lkt:6211:39>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6211'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Ent; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind _ Match_0
Let_Result_3 := False; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind td Match_1
--# expr-start 4335 '<IsNull at nodes.lkt:6213:30>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6213'
--# expr-start 4334 '<FieldAccess for discriminants at nodes.lkt:6213:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6213'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_1.Node.Type_Decl_F_Discriminants, Info => Match_1.Info);
--# expr-done 4334
Is_Null := Fld.Node = null; 
--# expr-done 4335
if Is_Null then
   --# expr-start 4345 '<Match at nodes.lkt:6213:59>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6213'
--# expr-start 4336 '<FieldAccess for type_def at nodes.lkt:6213:65>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6213'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_1.Node.Type_Decl_F_Type_Def, Info => Match_1.Info);
--# expr-done 4336
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind dtd Match_0_1
--# expr-start 4340 '<Not at nodes.lkt:6214:41>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
--# expr-start 4339 '<IsNull at nodes.lkt:6214:45>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
--# expr-start 4338 '<FieldAccess for constraint at nodes.lkt:6214:45>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
--# expr-start 4337 '<FieldAccess for subtype_indication at nodes.lkt:6214:45>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Match_0_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Match_0_1.Info);
--# expr-done 4337
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Constraint (Node => Fld_2.Node.Subtype_Indication_F_Constraint, Info => Fld_2.Info);
--# expr-done 4338
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 4339
Not_Val := not (Is_Null_1); 
--# expr-done 4340
if Not_Val then
   If_Result := True;
else
   --# expr-start 4342 '<FieldAccess for is_definite_subtype at nodes.lkt:6214:90>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
--# expr-start 4341 '<FieldAccess for base_type at nodes.lkt:6214:90>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6214'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0_1.Node, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 4341
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4342
   If_Result := Fld_5;
end if;
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind atd Match_1_1
--# expr-start 4344 '<IsA ConstrainedArrayIndices at nodes.lkt:6215:39>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6215'
--# expr-start 4343 '<FieldAccess for indices at nodes.lkt:6215:39>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6215'
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Array_Indices (Node => Match_1_1.Node.Array_Type_Def_F_Indices, Info => Match_1_1.Info);
--# expr-done 4343
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Constrained_Array_Indices_Range; 
--# expr-done 4344
Let_Result_1 := Is_A; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4345
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2_1 := Cast_Result_5; 
--# bind st Match_2_1
--# expr-start 4349 '<Not at nodes.lkt:6218:33>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
--# expr-start 4348 '<IsNull at nodes.lkt:6218:37>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
--# expr-start 4347 '<FieldAccess for constraint at nodes.lkt:6218:37>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
--# expr-start 4346 '<FieldAccess for subtype at nodes.lkt:6218:37>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
      if Match_2_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Subtype_Indication (Node => Match_2_1.Node.Subtype_Decl_F_Subtype, Info => Match_2_1.Info);
--# expr-done 4346
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Constraint (Node => Fld_7.Node.Subtype_Indication_F_Constraint, Info => Fld_7.Info);
--# expr-done 4347
Is_Null_2 := Fld_8.Node = null; 
--# expr-done 4348
Not_Val_1 := not (Is_Null_2); 
--# expr-done 4349
if Not_Val_1 then
   If_Result_2 := True;
else
   --# expr-start 4351 '<FieldAccess for is_definite_subtype at nodes.lkt:6218:70>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
--# expr-start 4350 '<FieldAccess for get_type at nodes.lkt:6218:70>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6218'
      if Match_2_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_2_1.Node, Origin => Origin, E_Info => Match_2_1.Info);
--# end
--# expr-done 4350
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 4351
   If_Result_2 := Fld_10;
end if;
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_7; 
--# bind ttd Match_4
--# expr-start 4353 '<IsNull at nodes.lkt:6220:35>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6220'
--# expr-start 4352 '<FieldAccess for discriminants at nodes.lkt:6220:35>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6220'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Discriminant_Part (Node => Match_4.Node.Task_Type_Decl_F_Discriminants, Info => Match_4.Info);
--# expr-done 4352
Is_Null_3 := Fld_11.Node = null; 
--# expr-done 4353
Let_Result_7 := Is_Null_3; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_5 := Cast_Result_8; 
--# bind ptd Match_5
--# expr-start 4355 '<IsNull at nodes.lkt:6221:40>' Is_Null_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6221'
--# expr-start 4354 '<FieldAccess for discriminants at nodes.lkt:6221:40>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6221'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Discriminant_Part (Node => Match_5.Node.Protected_Type_Decl_F_Discriminants, Info => Match_5.Info);
--# expr-done 4354
Is_Null_4 := Fld_12.Node = null; 
--# expr-done 4355
Let_Result_8 := Is_Null_4; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_9 := True; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_9; 
end case;
--# expr-done 4356
         Property_Result := Match_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Definite_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discriminants_list' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discriminants_List
  (Node : Bare_Base_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.discriminants_list
                  Property_Result := Base_Subtype_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.discriminants_list
                  Property_Result := Classwide_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.discriminants_list
                  Property_Result := Incomplete_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.discriminants_list
                  Property_Result := Protected_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.discriminants_list
                  Property_Result := Task_Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.discriminants_list
                  Property_Result := Type_Decl_P_Discriminants_List
                    (Self
                        , Stop_Recurse_At
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Discriminants_List;
--# end
--# property-start BaseTypeDecl.corresponding_char_literal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6249'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Corresponding_Char_Literal
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Root : Internal_Entity_Base_Type_Decl;
Local_Enum_Type : Internal_Entity_Enum_Type_Def;
Local_Sym : Symbol_Type;
Var_Expr_0 : Bare_Char_Literal;
Local_Char_Lit : Internal_Entity_Enum_Literal_Decl;
Var_Expr_132 : Internal_Entity_Type_Decl;
Var_Expr_133 : Internal_Entity_Enum_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Scope_Result : Internal_Entity_Type_Def;
Result_Var : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Enum_Type_Def;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Char_Literal;
Sym : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Result_Var_1 : Symbol_Type;
Fld_2 : Internal_Entity_Enum_Literal_Decl_List;
Fld_3 : Bare_Enum_Literal_Decl_List;
Fld_4 : Internal_Entity_Info;
Item_1 : Internal_Entity_Enum_Literal_Decl;
Bare_Item_1 : Bare_Enum_Literal_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Boolean;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
Find_Result : Internal_Entity_Enum_Literal_Decl;
Scope_Result_2 : Internal_Entity_Enum_Literal_Decl;
Result_Var_2 : Internal_Entity_Enum_Literal_Decl;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Type_Decl;
New_Node : Bare_Synthetic_Char_Enum_Lit;
As_Entity_1 : Internal_Entity_Synthetic_Char_Enum_Lit;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Synthetic_Char_Enum_Lit;
Cast_Result_4 : Internal_Entity_Enum_Literal_Decl;
Is_Null_2 : Boolean;
Fld_8 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Internal_Entity_Enum_Literal_Decl;
Let_Result_1 : Internal_Entity_Enum_Literal_Decl;
Let_Result_2 : Internal_Entity_Enum_Literal_Decl;
Let_Result_3 : Internal_Entity_Enum_Literal_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Corresponding_Char_Literal,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 4377 '<Let at nodes.lkt:6250:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6250'
--# scope-start
--# expr-start 4357 '<FieldAccess for root_type at nodes.lkt:6250:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6250'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4357
Local_Root := Fld; 
--# bind root Local_Root
--# expr-start 4376 '<Let at nodes.lkt:6251:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6251'
--# scope-start
--# expr-start 4360 '<Cast to EnumTypeDef at nodes.lkt:6251:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6251'
--# expr-start 4358 '<Cast to TypeDecl at nodes.lkt:6251:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6251'
Cast_Expr := Local_Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 4358
Var_Expr_132 := Cast_Result;
if Var_Expr_132 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 4359 '<FieldAccess for type_def at nodes.lkt:6251:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6251'
      if Var_Expr_132.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Var_Expr_132.Node.Type_Decl_F_Type_Def, Info => Var_Expr_132.Info);
--# expr-done 4359
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Type_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Type_Def;
   end if;
--# expr-done 4360
Local_Enum_Type := Cast_Result_1; 
--# bind enum_type Local_Enum_Type
--# expr-start 4375 '<Let at nodes.lkt:6252:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6252'
--# scope-start
--# expr-start 4361 '<Cast to CharLiteral at nodes.lkt:6252:19>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6252'
Cast_Expr_1 := Origin; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Char_Literal_Range
   then
      Cast_Result_2 := Cast_Expr_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 4361
Var_Expr_0 := Cast_Result_2;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind l Var_Expr_0
--# expr-start 4362 '<GetSymbol at nodes.lkt:6252:52>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6252'
Sym := Get_Symbol (Var_Expr_0); 
--# expr-done 4362
Scope_Result_1 := Sym;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Symbol;
end if;
Local_Sym := Result_Var_1; 
--# bind sym Local_Sym
--# expr-start 4374 '<Let at nodes.lkt:6253:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6253'
--# scope-start
Var_Expr_133 := Local_Enum_Type;
if Var_Expr_133 /= No_Entity_Enum_Type_Def then
   --# scope-start
--# expr-start 4366 '<Find at nodes.lkt:6253:24>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6253'
   --# expr-start 4363 '<FieldAccess for enum_literals at nodes.lkt:6253:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6253'
      if Var_Expr_133.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Enum_Literal_Decl_List (Node => Var_Expr_133.Node.Enum_Type_Def_F_Enum_Literals, Info => Var_Expr_133.Info);
--# expr-done 4363
Fld_3 := Fld_2.Node;
      Find_Result := No_Entity_Enum_Literal_Decl;
      if Fld_3 = null then
      Find_Result := No_Entity_Enum_Literal_Decl;
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_3;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind lit_decl Item_1
      --# expr-start 4365 '<FieldAccess for name_is at nodes.lkt:6253:68>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6253'
--# expr-start 4364 '<FieldAccess for name at nodes.lkt:6253:68>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6253'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Defining_Name (Node => Item_1.Node.Enum_Literal_Decl_F_Name, Info => Item_1.Info);
--# expr-done 4364
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Local_Sym);
--# end
--# expr-done 4365
      if Fld_6 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_6;
         end loop;
      end;
      end if;
--# expr-done 4366
Scope_Result_2 := Find_Result;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Enum_Literal_Decl;
end if;
Local_Char_Lit := Result_Var_2; 
--# bind char_lit Local_Char_Lit
--# expr-start 4373 '<If at nodes.lkt:6258:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6258'
--# expr-start 4367 '<IsNull at nodes.lkt:6258:12>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6258'
Is_Null_2 := Local_Char_Lit.Node = null; 
--# expr-done 4367
if Is_Null_2 then
   --# expr-start 4368 '<FieldAccess for is_std_char_type at nodes.lkt:6258:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6258'
      if Local_Enum_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EnumTypeDef.is_std_char_type
Fld_8 := Libadalang.Implementation.Enum_Type_Def_P_Is_Std_Char_Type (Node => Local_Enum_Type.Node);
--# end
--# expr-done 4368
   If_Result_2 := Fld_8;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
--# expr-start 4372 '<as_entity at nodes.lkt:6258:67>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6258'
--# expr-start 4371 '<New SyntheticCharEnumLit at nodes.lkt:6258:67>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6258'
--# expr-start 4370 '<Cast to TypeDecl at nodes.lkt:6259:70>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6259'
--# expr-start 4369 '<parent at nodes.lkt:6259:70>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6259'
      if Local_Enum_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Local_Enum_Type.Node, E_Info => Local_Enum_Type.Info);
--# expr-done 4369
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_3 := No_Entity_Type_Decl;
   end if;
--# expr-done 4370
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Synthetic_Char_Enum_Lit);
Initialize
  (Self => New_Node,
   Kind => Ada_Synthetic_Char_Enum_Lit,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Synthetic_Char_Enum_Lit
     (Self => New_Node,
      Enum_Literal_Decl_F_Name => No_Bare_Ada_Node);
      New_Node.Synthetic_Char_Enum_Lit_Char_Symbol :=
         Local_Sym;
      New_Node.Synthetic_Char_Enum_Lit_Enum_Type_Decl :=
         Cast_Result_3;
--# expr-done 4371
Is_Null_1 := New_Node = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Synthetic_Char_Enum_Lit;
else
As_Entity_1 := (Info => E_Info, Node => New_Node); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4372
      Cast_Result_4 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   If_Result_3 := Cast_Result_4;
else
   If_Result_3 := Local_Char_Lit;
end if;
--# expr-done 4373
Let_Result := If_Result_3; 
--# end
--# expr-done 4374
Let_Result_1 := Let_Result; 
--# end
--# expr-done 4375
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 4376
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 4377
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Let_Result_3.Node,
         Info => Let_Result_3.Info);
         Property_Result := Cast_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Corresponding_Char_Literal;
--# end
--# property-start BaseTypeDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6273'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Single_Task_Type_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Protected_Type_Decl;
Match_3 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Protected_Type_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4384 '<Match at nodes.lkt:6273:51>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6273'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind sttd Match_0
--# expr-start 4379 '<FieldAccess for next_part_for_decl at nodes.lkt:6276:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6276'
--# expr-start 4378 '<FieldAccess for parent_basic_decl at nodes.lkt:6276:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6276'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 4378
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4379
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 4380 '<Super at nodes.lkt:6277:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6277'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4380
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
--# expr-start 4381 '<Super at nodes.lkt:6278:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6278'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.next_part_for_decl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4381
Let_Result_2 := Fld_3; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
--# expr-start 4383 '<Cast to BasicDecl at nodes.lkt:6279:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6279'
--# expr-start 4382 '<FieldAccess for next_part at nodes.lkt:6279:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6279'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4382
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-done 4383
Let_Result_3 := Cast_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4384
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseTypeDecl.shapes 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6335'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Shapes
  (Node : Bare_Base_Type_Decl
      ; Include_Discriminants : Boolean
         := True
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_discriminants Include_Discriminants
   --# bind origin Origin
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_648 with Inline_Always;
            procedure Finalizer_Scope_2568 with Inline_Always;
            procedure Finalizer_Scope_2569 with Inline_Always;
            procedure Finalizer_Scope_2570 with Inline_Always;
            procedure Finalizer_Scope_2571 with Inline_Always;
      Local_Rdef : Internal_Entity_Base_Record_Def;
Local_Comps : Internal_Entity_Component_List;
Local_Parent_Type : Internal_Entity_Base_Type_Decl;
Local_Parent_Record : Internal_Entity_Base_Record_Def;
Local_Own_Shapes : Internal_Shape_Array_Access;
Local_All_Shapes : Internal_Shape_Array_Access;
Var_Expr_0 : Internal_Entity_Base_Record_Def;
Local_Discrs : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_134 : Internal_Entity_Derived_Type_Def;
Var_Expr_135 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Record_Def;
Fld_1 : Internal_Entity_Component_List;
Fld_2 : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Result_Var_1 : Internal_Entity_Base_Record_Def;
Fld_5 : Internal_Shape_Array_Access;
Fld_6 : Internal_Shape_Array_Access;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Discriminant_Values_Array_Access;
Fld_10 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct : Internal_Shape := No_Shape;
Map_Result : Internal_Shape_Array_Access;
Map_Result_1 : Internal_Shape_Array_Access;
Scope_Result_2 : Internal_Shape_Array_Access;
Result_Var_2 : Internal_Shape_Array_Access;
Fld_11 : Internal_Entity_Type_Decl;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_2 : Internal_Shape_Array_Access;
If_Result : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
Let_Result_1 : Internal_Shape_Array_Access;
Let_Result_2 : Internal_Shape_Array_Access;
Let_Result_3 : Internal_Shape_Array_Access;
Let_Result_4 : Internal_Shape_Array_Access;
Let_Result_5 : Internal_Shape_Array_Access;
Let_Result_6 : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_648 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Local_Own_Shapes);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Local_All_Shapes);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Local_Discrs);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_648;
            procedure Finalizer_Scope_2568 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2568;
            procedure Finalizer_Scope_2569 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2569;
            procedure Finalizer_Scope_2570 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2570;
            procedure Finalizer_Scope_2571 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2571;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 4413 '<Let at nodes.lkt:6336:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6336'
--# scope-start
--# expr-start 4385 '<FieldAccess for record_def at nodes.lkt:6336:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6336'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4385
Local_Rdef := Fld; 
--# bind rdef Local_Rdef
--# expr-start 4412 '<Let at nodes.lkt:6337:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6337'
--# scope-start
--# expr-start 4386 '<FieldAccess for components at nodes.lkt:6337:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6337'
      if Local_Rdef.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Component_List (Node => Local_Rdef.Node.Base_Record_Def_F_Components, Info => Local_Rdef.Info);
--# expr-done 4386
Local_Comps := Fld_1; 
--# bind comps Local_Comps
--# expr-start 4411 '<Let at nodes.lkt:6338:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6338'
--# scope-start
--# expr-start 4388 '<Cast to DerivedTypeDef at nodes.lkt:6338:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6338'
--# expr-start 4387 '<FieldAccess for type_def at nodes.lkt:6338:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6338'
      if Local_Comps.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_def
Fld_2 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Local_Comps.Node, E_Info => Local_Comps.Info);
--# end
--# expr-done 4387
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 4388
Var_Expr_134 := Cast_Result;
if Var_Expr_134 /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 4389 '<FieldAccess for base_type at nodes.lkt:6338:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6338'
      if Var_Expr_134.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr_134.Node, Origin => Origin, E_Info => Var_Expr_134.Info);
--# end
--# expr-done 4389
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Local_Parent_Type := Result_Var; 
--# bind parent_type Local_Parent_Type
--# expr-start 4410 '<Let at nodes.lkt:6339:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6339'
--# scope-start
Var_Expr_135 := Local_Parent_Type;
if Var_Expr_135 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4390 '<FieldAccess for record_def at nodes.lkt:6339:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6339'
      if Var_Expr_135.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr_135.Node, Origin => Origin, E_Info => Var_Expr_135.Info);
--# end
--# expr-done 4390
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Record_Def;
end if;
Local_Parent_Record := Result_Var_1; 
--# bind parent_record Local_Parent_Record
--# expr-start 4409 '<Let at nodes.lkt:6340:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6340'
--# scope-start
--# expr-start 4391 '<FieldAccess for shapes at nodes.lkt:6340:26>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6340'
      if Local_Comps.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.shapes
Fld_5 := Libadalang.Implementation.Component_List_P_Shapes (Node => Local_Comps.Node, E_Info => Local_Comps.Info);
--# end
--# expr-done 4391
Local_Own_Shapes := Fld_5; Inc_Ref (Local_Own_Shapes);
--# bind own_shapes Local_Own_Shapes
--# expr-start 4408 '<Let at nodes.lkt:6343:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6343'
--# scope-start
Var_Expr_0 := Local_Parent_Record;
if Var_Expr_0 /= No_Entity_Base_Record_Def then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 4399 '<Mapcat at nodes.lkt:6344:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6344'
   --# expr-start 4392 '<FieldAccess for shapes at nodes.lkt:6344:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6344'
      if Local_Parent_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.shapes
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Shapes (Node => Local_Parent_Type.Node, Include_Discriminants => False, Origin => Origin, E_Info => Local_Parent_Type.Info);
--# end
--# expr-done 4392
      declare
         Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_6;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind parent_shape Item_1
      --# expr-start 4398 '<Map at nodes.lkt:6345:35>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6345'
      declare
         Map_Result_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Local_Own_Shapes;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind own_shape Item_2
      --# expr-start 4397 '<New Shape at nodes.lkt:6346:36>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6346'
--# expr-start 4393 '<FieldAccess for components at nodes.lkt:6347:36>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6347'
Fld_7 := Item_1.Components;
Inc_Ref (Fld_7);
--# expr-done 4393
--# expr-start 4394 '<FieldAccess for components at nodes.lkt:6347:62>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6347'
Fld_8 := Item_2.Components;
Inc_Ref (Fld_8);
--# expr-done 4394
Concat_Result := Concat (Fld_7, Fld_8); 
--# expr-start 4395 '<FieldAccess for discriminants_values at nodes.lkt:6347:105>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6347'
Fld_9 := Item_1.Discriminants_Values;
Inc_Ref (Fld_9);
--# expr-done 4395
--# expr-start 4396 '<FieldAccess for discriminants_values at nodes.lkt:6347:141>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6347'
Fld_10 := Item_2.Discriminants_Values;
Inc_Ref (Fld_10);
--# expr-done 4396
Concat_Result_1 := Concat (Fld_9, Fld_10); 
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 4397
         declare
            Item_To_Append : constant Internal_Shape := New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2570;
         end loop;
      end;
         Map_Result := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4398
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2569;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4399
Scope_Result_2 := Map_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2568;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Local_Own_Shapes;
end if;
      Inc_Ref (Result_Var_2);
Local_All_Shapes := Result_Var_2; Inc_Ref (Local_All_Shapes);
--# bind all_shapes Local_All_Shapes
--# expr-start 4407 '<Let at nodes.lkt:6352:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6352'
--# scope-start
--# expr-start 4401 '<FieldAccess for discriminants_list at nodes.lkt:6352:22>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6352'
--# expr-start 4400 '<FieldAccess for type_decl at nodes.lkt:6352:22>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6352'
      if Local_Comps.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_11 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Local_Comps.Node, E_Info => Local_Comps.Info);
--# end
--# expr-done 4400
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_11.Node, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 4401
Local_Discrs := Fld_12; Inc_Ref (Local_Discrs);
--# bind discrs Local_Discrs
--# expr-start 4406 '<If at nodes.lkt:6354:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6354'
if Include_Discriminants then
   --# expr-start 4405 '<Map at nodes.lkt:6354:39>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6354'
      declare
         Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Shape_Array_Access := Local_All_Shapes;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_3
      --# expr-start 4404 '<New Shape at nodes.lkt:6355:20>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6355'
--# expr-start 4402 '<FieldAccess for components at nodes.lkt:6356:37>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6356'
Fld_13 := Item_3.Components;
Inc_Ref (Fld_13);
--# expr-done 4402
Concat_Result_2 := Concat (Local_Discrs, Fld_13); 
--# expr-start 4403 '<FieldAccess for discriminants_values at nodes.lkt:6356:72>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6356'
Fld_14 := Item_3.Discriminants_Values;
Inc_Ref (Fld_14);
--# expr-done 4403
Inc_Ref (Concat_Result_2);
Inc_Ref (Fld_14);
New_Struct_1 := (Components => Concat_Result_2, Discriminants_Values => Fld_14); 
--# expr-done 4404
         declare
            Item_To_Append : constant Internal_Shape := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2571;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Shape_Array
           (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 4405
   If_Result := Map_Result_2;
else
   If_Result := Local_All_Shapes;
end if;
      Inc_Ref (If_Result);
--# expr-done 4406
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4407
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4408
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 4409
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4410
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 4411
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 4412
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 4413
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_648;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_648;
                     Finalizer_Scope_2568;
                     Finalizer_Scope_2569;
                     Finalizer_Scope_2570;
                     Finalizer_Scope_2571;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Shapes;
--# end
--# property-start BaseTypeDecl.dottable_subps_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6364'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Dottable_Subps_Env
  (Node : Bare_Base_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_649 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_649 is
            begin
                     Dec_Ref (Dyn_Env);
            end Finalizer_Scope_649;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Base_Type_Decl_Lf_State_Dottable_Subps_Env is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Base_Type_Decl_Lf_Stg_Dottable_Subps_Env;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Base_Type_Decl_Lf_State_Dottable_Subps_Env,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4414 '<DynamicLexicalEnv at nodes.lkt:6365:9>' Dyn_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6365'
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Base_Type_Decl_P_Dottable_Subps'Access, null, False, Self.Unit.Context.Symbols); 
--# expr-done 4414
         Property_Result := Dyn_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_649;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_649;
               Self.Bare_Base_Type_Decl_Lf_State_Dottable_Subps_Env :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Base_Type_Decl_Lf_State_Dottable_Subps_Env := Initialized;
      Self.Bare_Base_Type_Decl_Lf_Stg_Dottable_Subps_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Base_Type_Decl_F_Dottable_Subps_Env;
--# end
--# property-start BaseTypeDecl.dottable_subps 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6373'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Dottable_Subps
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_650 with Inline_Always;
      Local_Scope : Bare_Declarative_Part;
Local_Pkg : Internal_Entity_Base_Package_Decl;
Var_Expr_0 : Bare_Package_Body;
Var_Expr_136 : Bare_Declarative_Part;
Var_Expr_137 : Bare_Declarative_Part;
Var_Expr_138 : Internal_Entity_Package_Body;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Package_Body;
Bound_Imprecise_Fallback : Boolean;
As_Entity : Internal_Entity_Package_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Package_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_3 : Bare_Ada_Node;
Scope_Result_2 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_2 : Internal_Entity_Base_Package_Decl;
Fld_4 : Internal_Entity_Public_Part;
Cast_Result_3 : Internal_Entity_Declarative_Part;
Fld_5 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Declarative_Part;
Fld_6 : Internal_Entity_Package_Body;
Fld_7 : Internal_Entity_Declarative_Part;
Scope_Result_3 : Internal_Entity_Declarative_Part;
Result_Var_3 : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_8 : Internal_Inner_Env_Assoc_Array_Access;
As_Entity_2 : Internal_Entity_Declarative_Part;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Declarative_Part;
Array_Lit_1 : Internal_Entity_Declarative_Part_Array_Access;
Fld_9 : Internal_Inner_Env_Assoc_Array_Access;
Is_Null_3 : Boolean;
Not_Val : Boolean;
If_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_650 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_650;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Dottable_Subps,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4440 '<Let at nodes.lkt:6374:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6374'
--# scope-start
--# expr-start 4415 '<FieldAccess for declarative_scope at nodes.lkt:6374:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6374'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 4415
Local_Scope := Fld; 
--# bind scope Local_Scope
--# expr-start 4439 '<Let at nodes.lkt:6375:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6375'
--# scope-start
--# expr-start 4417 '<Cast to PackageBody at nodes.lkt:6375:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6375'
Var_Expr_136 := Local_Scope;
if Var_Expr_136 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4416 '<parent at nodes.lkt:6375:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6375'
      if Var_Expr_136 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_136.Parent; 
--# expr-done 4416
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Result_Var;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4417
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind body Var_Expr_0
--# expr-start 4421 '<Cast to BasePackageDecl at nodes.lkt:6376:23>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6376'
--# expr-start 4420 '<bind at nodes.lkt:6377:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6377'
Bound_Imprecise_Fallback := False; 
--# expr-start 4419 '<FieldAccess for previous_part at nodes.lkt:6379:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6379'
--# expr-start 4418 '<as_entity at nodes.lkt:6379:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6379'
Is_Null := Var_Expr_0 = null; 
if Is_Null then
   If_Result := No_Entity_Package_Body;
else
As_Entity := (Info => E_Info, Node => Var_Expr_0); 
   If_Result := As_Entity;
end if;
--# expr-done 4418
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Previous_Part (Node => If_Result.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 4419
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 4420
   if Dyn_Var_Bind_Result.Node = null
      or else Dyn_Var_Bind_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Package_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
   else
         Cast_Result_1 := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 4421
Scope_Result_1 := Cast_Result_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
   --# expr-start 4424 '<as_entity at nodes.lkt:6380:48>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6380'
--# expr-start 4423 '<Cast to BasePackageDecl at nodes.lkt:6380:48>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6380'
Var_Expr_137 := Local_Scope;
if Var_Expr_137 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4422 '<parent at nodes.lkt:6380:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6380'
      if Var_Expr_137 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_137.Parent; 
--# expr-done 4422
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
   if Result_Var_1 = null
      or else Result_Var_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_2 := Result_Var_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 4423
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 4424
   Result_Var_2 := If_Result_1;
end if;
Local_Pkg := Result_Var_2; 
--# bind pkg Local_Pkg
--# expr-start 4438 '<If at nodes.lkt:6392:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6392'
--# expr-start 4426 '<Not at nodes.lkt:6392:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6392'
--# expr-start 4425 '<IsNull at nodes.lkt:6392:16>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6392'
Is_Null_3 := Local_Pkg.Node = null; 
--# expr-done 4425
Not_Val := not (Is_Null_3); 
--# expr-done 4426
if Not_Val then
   --# expr-start 4434 '<FieldAccess for dottable_subps_in_declaratives_parts at nodes.lkt:6392:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6392'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4433 '<ArrayLiteral at nodes.lkt:6393:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
--# expr-start 4428 '<Cast to DeclarativePart at nodes.lkt:6393:14>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
--# expr-start 4427 '<FieldAccess for public_part at nodes.lkt:6393:14>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
      if Local_Pkg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Public_Part (Node => Local_Pkg.Node.Base_Package_Decl_F_Public_Part, Info => Local_Pkg.Info);
--# expr-done 4427
      Cast_Result_3 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-done 4428
--# expr-start 4430 '<Cast to DeclarativePart at nodes.lkt:6393:51>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
--# expr-start 4429 '<FieldAccess for private_part at nodes.lkt:6393:51>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
      if Local_Pkg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Local_Pkg.Node.Base_Package_Decl_F_Private_Part, Info => Local_Pkg.Info);
--# expr-done 4429
      Cast_Result_4 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# expr-done 4430
--# expr-start 4431 '<FieldAccess for body_part at nodes.lkt:6393:89>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
      if Local_Pkg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_6 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Local_Pkg.Node, E_Info => Local_Pkg.Info);
--# end
--# expr-done 4431
Var_Expr_138 := Fld_6;
if Var_Expr_138 /= No_Entity_Package_Body then
   --# scope-start
--# expr-start 4432 '<FieldAccess for decls at nodes.lkt:6393:89>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6393'
      if Var_Expr_138.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_138.Node.Package_Body_F_Decls, Info => Var_Expr_138.Info);
--# expr-done 4432
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Declarative_Part;
end if;
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result_3, 2 => Cast_Result_4, 3 => Result_Var_3)); 
--# expr-done 4433
--# property-call-start BaseTypeDecl.dottable_subps_in_declaratives_parts
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Dottable_Subps_In_Declaratives_Parts (Node => Ent.Node, Parts => Array_Lit, E_Info => Ent.Info);
--# end
--# expr-done 4434
   If_Result_3 := Fld_8;
else
   --# expr-start 4437 '<FieldAccess for dottable_subps_in_declaratives_parts at nodes.lkt:6397:14>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6397'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4436 '<ArrayLiteral at nodes.lkt:6397:56>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6397'
--# expr-start 4435 '<as_entity at nodes.lkt:6397:57>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6397'
Is_Null_2 := Local_Scope = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Declarative_Part;
else
As_Entity_2 := (Info => E_Info, Node => Local_Scope); 
   If_Result_2 := As_Entity_2;
end if;
--# expr-done 4435
Array_Lit_1 := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => If_Result_2)); 
--# expr-done 4436
--# property-call-start BaseTypeDecl.dottable_subps_in_declaratives_parts
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Dottable_Subps_In_Declaratives_Parts (Node => Ent.Node, Parts => Array_Lit_1, E_Info => Ent.Info);
--# end
--# expr-done 4437
   If_Result_3 := Fld_9;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 4438
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 4439
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4440
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_650;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_650;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                        As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Dottable_Subps;
--# end
--# property-start BaseTypeDecl.dottable_subps_in_declaratives_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6403'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Dottable_Subps_In_Declaratives_Parts
  (Node : Bare_Base_Type_Decl
      ; Parts : Internal_Entity_Declarative_Part_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind parts Parts
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_651 with Inline_Always;
            procedure Finalizer_Scope_2576 with Inline_Always;
            procedure Finalizer_Scope_2577 with Inline_Always;
      Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_139 : Internal_Entity_Declarative_Part;
Var_Expr_140 : Internal_Entity_Basic_Decl;
Var_Expr_141 : Internal_Entity_Base_Subp_Spec;
Var_Expr_142 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_22 : Internal_Entity;
Bare_Item_22 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Map_Result_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Bare_Basic_Decl;
New_Struct : Internal_Metadata;
Cast_Result_1 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Let_Result : Internal_Inner_Env_Assoc;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_651 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_651;
            procedure Finalizer_Scope_2576 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2576;
            procedure Finalizer_Scope_2577 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2577;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4456 '<Map at nodes.lkt:6404:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6404'
   --# expr-start 4443 '<Mapcat at nodes.lkt:6404:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6404'
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Declarative_Part_Array_Access := Parts;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_0
Var_Expr_139 := Item_0;
if Var_Expr_139 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 4442 '<TakeWhile at ???>' Map_Result None
   --# expr-start 4441 '<FieldAccess for decls at nodes.lkt:6404:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6404'
      if Var_Expr_139.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_139.Node.Declarative_Part_F_Decls, Info => Var_Expr_139.Info);
--# expr-done 4441
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_22 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_22 := Untyped_Item_22; 
Is_Null := Bare_Item_22 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_22); 
   If_Result := As_Entity;
end if;
                  Item_22 := If_Result; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity := Item_22;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4442
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2577;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2576;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 4443
      declare
         Map_Result_2_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_1
         --# expr-start 4448 '<Eq at nodes.lkt:6411:26>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6411'
--# expr-start 4444 '<Cast to BasicDecl at nodes.lkt:6411:26>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6411'
Cast_Expr_1 := Item_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4444
Var_Expr_140 := Cast_Result_2;
if Var_Expr_140 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4445 '<FieldAccess for subp_spec_or_null at nodes.lkt:6411:26>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6411'
      if Var_Expr_140.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_140.Node, E_Info => Var_Expr_140.Info);
--# end
--# expr-done 4445
Var_Expr_141 := Fld_6;
if Var_Expr_141 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 4446 '<FieldAccess for dottable_subp_of at nodes.lkt:6411:26>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6411'
      if Var_Expr_141.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_7 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Var_Expr_141.Node, E_Info => Var_Expr_141.Info);
--# end
--# expr-done 4446
Var_Expr_142 := Fld_7;
if Var_Expr_142 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4447 '<FieldAccess for base_subtype at nodes.lkt:6411:26>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6411'
      if Var_Expr_142.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Var_Expr_142.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_142.Info);
--# end
--# expr-done 4447
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_3 := Create_Internal_Entity
        (Node => Result_Var_3.Node,
         Info => Result_Var_3.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 4448
         if Is_Equiv then
      --# expr-start 4455 '<Let at nodes.lkt:6406:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6406'
--# scope-start
--# expr-start 4449 '<Cast to BasicDecl at nodes.lkt:6406:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6406'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4449
Local_Bd := Cast_Result; 
--# bind bd Local_Bd
--# expr-start 4454 '<New InnerEnvAssoc at nodes.lkt:6408:17>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6408'
--# expr-start 4451 '<FieldAccess for name_symbol at nodes.lkt:6409:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6409'
--# expr-start 4450 '<FieldAccess for defining_name at nodes.lkt:6409:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6409'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 4450
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_3.Node);
--# end
--# expr-done 4451
--# expr-start 4452 '<New Metadata at nodes.lkt:6409:83>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6409'
New_Struct := (Dottable_Subp => True, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4452
--# expr-start 4453 '<FieldAccess for node at nodes.lkt:6409:65>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6409'
Fld_5 := Local_Bd.Node;
--# expr-done 4453
      Cast_Result_1 := Fld_5;
New_Struct_1 := (Key => Fld_4, Metadata => New_Struct, Rebindings => null, Value => Cast_Result_1); 
--# expr-done 4454
Let_Result := New_Struct_1; 
--# end
--# expr-done 4455
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := Let_Result;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 4456
         Property_Result := Map_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_651;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_651;
                     Finalizer_Scope_2576;
                     Finalizer_Scope_2577;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Dottable_Subps_In_Declaratives_Parts;
--# end
--# property-start BaseTypeDecl.find_base_type_rebindings_among 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6418'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Base_Type_Rebindings_Among
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
      ; Base_Types : Internal_Entity_Base_Type_Decl_Array_Access
      ; Index : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Env_Rebindings
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind base_types Base_Types
   --# bind index Index
   --# bind origin Origin
   Property_Result : Env_Rebindings;
      Var_Expr : Env_Rebindings;
Get_Result : Internal_Entity_Base_Type_Decl;
Fld : Env_Rebindings;
Scope_Result : Env_Rebindings;
Arith_Result : Integer;
Fld_1 : Env_Rebindings;
Result_Var : Env_Rebindings;
Len : Integer;
Comp_Result : Boolean;
If_Result : Env_Rebindings;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4462 '<If at nodes.lkt:6419:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
--# expr-start 4457 '<length at nodes.lkt:6419:21>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
Len := Length (Base_Types); 
--# expr-done 4457
Comp_Result := Index >= Len; 
if Comp_Result then
   If_Result := null;
else
--# expr-start 4459 '<FieldAccess for find_base_type_rebindings at nodes.lkt:6419:71>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
--# expr-start 4458 '<at at nodes.lkt:6419:71>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
Get_Result := Get (Self, Base_Types, Index, True); 
--# expr-done 4458
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Get_Result.Node, Target => Target, Origin => Origin, E_Info => Get_Result.Info);
--# end
--# expr-done 4459
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4461 '<FieldAccess for find_base_type_rebindings_among at nodes.lkt:6419:128>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4460 '<Arithmetic + at nodes.lkt:6419:185>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6419'
Arith_Result := (Index + 1); 
--# expr-done 4460
--# property-call-start BaseTypeDecl.find_base_type_rebindings_among
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings_Among (Node => Ent.Node, Target => Target, Base_Types => Base_Types, Index => Arith_Result, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4461
   Result_Var := Fld_1;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 4462
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Find_Base_Type_Rebindings_Among;
--# end
--# property-start BaseTypeDecl.find_base_type_rebindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6437'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Base_Type_Rebindings
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Env_Rebindings
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   --# bind origin Origin
   Property_Result : Env_Rebindings;
            procedure Finalizer_Scope_653 with Inline_Always;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Env_Rebindings;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Env_Rebindings;
            procedure Finalizer_Scope_653 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_653;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Find_Base_Type_Rebindings,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Target);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Env_Rebindings;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4468 '<If at nodes.lkt:6438:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6438'
--# expr-start 4463 '<Eq at nodes.lkt:6438:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6438'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Target; 
--# expr-done 4463
if Is_Equal then
   --# expr-start 4465 '<FieldAccess for rebindings at nodes.lkt:6438:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6438'
--# expr-start 4464 '<FieldAccess for info at nodes.lkt:6438:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6438'
Fld := Ent.Info;
--# expr-done 4464
Fld_1 := Fld.Rebindings;
--# expr-done 4465
   If_Result := Fld_1;
else
   --# expr-start 4467 '<FieldAccess for find_base_type_rebindings_among at nodes.lkt:6438:58>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6438'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4466 '<FieldAccess for base_types at nodes.lkt:6439:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6439'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4466
--# property-call-start BaseTypeDecl.find_base_type_rebindings_among
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings_Among (Node => Ent.Node, Target => Target, Base_Types => Fld_2, Index => 0, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4467
   If_Result := Fld_3;
end if;
--# expr-done 4468
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_653;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_653;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Env_Rebindings,
                        As_Env_Rebindings => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Type_Decl_P_Find_Base_Type_Rebindings;
--# end
--# property-start BaseTypeDecl.has_base_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6444'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Base_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   Property_Result : Boolean;
            procedure Finalizer_Scope_654 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Boolean;
Quantifier_Result : Boolean;
If_Result : Boolean;
            procedure Finalizer_Scope_654 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_654;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4471 '<Eq at nodes.lkt:6445:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
--# expr-start 4470 '<FieldAccess for node at nodes.lkt:6445:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
--# expr-start 4469 '<FieldAccess for canonical_type at nodes.lkt:6445:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 4469
Fld_1 := Fld.Node;
--# expr-done 4470
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Target; 
--# expr-done 4471
if Is_Equal then
   If_Result := True;
else
   --# expr-start 4474 '<AnyQuantifier at nodes.lkt:6445:49>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
   --# expr-start 4472 '<FieldAccess for base_types at nodes.lkt:6445:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 4472
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_0
      --# expr-start 4473 '<FieldAccess for has_base_type_impl at nodes.lkt:6445:79>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6445'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.has_base_type_impl
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type_Impl (Node => Item_0.Node, Target => Target, E_Info => Item_0.Info);
--# end
--# expr-done 4473
      Quantifier_Result := Fld_3;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 4474
   If_Result := Quantifier_Result;
end if;
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_654;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_654;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Base_Type_Impl;
--# end
--# property-start BaseTypeDecl.has_base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6453'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind target Target
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4479 '<FieldAccess for has_base_type_impl at nodes.lkt:6453:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6453'
--# expr-start 4475 '<FieldAccess for full_view at nodes.lkt:6453:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6453'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4475
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4478 '<FieldAccess for node at nodes.lkt:6454:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6454'
--# expr-start 4477 '<FieldAccess for canonical_type at nodes.lkt:6454:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6454'
--# expr-start 4476 '<as_bare_entity at nodes.lkt:6454:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6454'
Is_Null := Target = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Target); 
   If_Result := As_Entity;
end if;
--# expr-done 4476
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => If_Result.Node, Origin => No_Bare_Ada_Node, E_Info => If_Result.Info);
--# end
--# expr-done 4477
Fld_2 := Fld_1.Node;
--# expr-done 4478
--# property-call-start BaseTypeDecl.has_base_type_impl
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Has_Base_Type_Impl (Node => Fld.Node, Target => Fld_2, E_Info => Fld.Info);
--# end
--# expr-done 4479
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Base_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.parent_primitives_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.parent_primitives_env
                  Property_Result := Base_Type_Decl_P_Parent_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.parent_primitives_env
                  Property_Result := Type_Decl_P_Parent_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start '[dispatcher]BaseTypeDecl.primitives_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Protected_Type_Decl =>
                  --# property-call-start BaseTypeDecl.primitives_env
                  Property_Result := Base_Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.primitives_env
                  Property_Result := Base_Subtype_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.primitives_env
                  Property_Result := Task_Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.primitives_env
                  Property_Result := Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_record_type
                  Property_Result := Base_Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_record_type
                  Property_Result := Base_Subtype_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_task_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Protected_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_task_type
                  Property_Result := Base_Type_Decl_P_Is_Task_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_task_type
                  Property_Result := Base_Subtype_Decl_P_Is_Task_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_task_type
                  Property_Result := Classwide_Type_Decl_P_Is_Task_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.is_task_type
                  Property_Result := Task_Type_Decl_P_Is_Task_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_task_type
                  Property_Result := Type_Decl_P_Is_Task_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_real_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_real_type
                  Property_Result := Base_Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_real_type
                  Property_Result := Base_Subtype_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_real_type
                  Property_Result := Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_float_type
                  Property_Result := Base_Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_float_type
                  Property_Result := Base_Subtype_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_float_type
                  Property_Result := Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_fixed_point
                  Property_Result := Base_Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_fixed_point
                  Property_Result := Base_Subtype_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_fixed_point
                  Property_Result := Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_enum_type
                  Property_Result := Base_Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_enum_type
                  Property_Result := Base_Subtype_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_enum_type
                  Property_Result := Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_classwide' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_classwide
                  Property_Result := Base_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_classwide
                  Property_Result := Base_Subtype_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_classwide
                  Property_Result := Classwide_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_access_type
                  Property_Result := Base_Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_access_type
                  Property_Result := Base_Subtype_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_access_type
                  Property_Result := Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.has_ud_indexing' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.has_ud_indexing
                  Property_Result := Base_Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.has_ud_indexing
                  Property_Result := Base_Subtype_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.has_ud_indexing
                  Property_Result := Classwide_Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.has_ud_indexing
                  Property_Result := Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start '[dispatcher]BaseTypeDecl.constant_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.constant_indexing_fns
                  Property_Result := Base_Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.constant_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.constant_indexing_fns
                  Property_Result := Classwide_Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.constant_indexing_fns
                  Property_Result := Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.variable_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.variable_indexing_fns
                  Property_Result := Base_Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.variable_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.variable_indexing_fns
                  Property_Result := Classwide_Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.variable_indexing_fns
                  Property_Result := Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.get_imp_deref' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.get_imp_deref
                  Property_Result := Base_Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.get_imp_deref
                  Property_Result := Subtype_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.get_imp_deref
                  Property_Result := Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.access_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.access_def
                  Property_Result := Base_Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.access_def
                  Property_Result := Base_Subtype_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.access_def
                  Property_Result := Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Access_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_char_type
                  Property_Result := Base_Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_char_type
                  Property_Result := Base_Subtype_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_char_type
                  Property_Result := Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.classwide_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.classwide_type
                  Property_Result := Base_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.classwide_type
                  Property_Result := Classwide_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.discrete_range
                  Property_Result := Base_Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.discrete_range
                  Property_Result := Subtype_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.discrete_range
                  Property_Result := Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_discrete_type
                  Property_Result := Base_Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_discrete_type
                  Property_Result := Base_Subtype_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_discrete_type
                  Property_Result := Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_int_type
                  Property_Result := Base_Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_int_type
                  Property_Result := Base_Subtype_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_int_type
                  Property_Result := Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.accessed_type
                  Property_Result := Base_Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.accessed_type
                  Property_Result := Base_Subtype_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.accessed_type
                  Property_Result := Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl | Ada_Protected_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_tagged_type
                  Property_Result := Base_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_tagged_type
                  Property_Result := Base_Subtype_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_tagged_type
                  Property_Result := Classwide_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Formal_Type_Decl =>
                  --# property-call-start IncompleteFormalTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Formal_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTaggedTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_tagged_type
                  Property_Result := Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_type
                  Property_Result := Base_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_type
                  Property_Result := Base_Subtype_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_type
                  Property_Result := Classwide_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.base_type
                  Property_Result := Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_types' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_types
                  Property_Result := Base_Type_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_types
                  Property_Result := Base_Subtype_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_interfaces
                  Property_Result := Base_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_interfaces
                  Property_Result := Base_Subtype_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_interfaces
                  Property_Result := Classwide_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.base_interfaces
                  Property_Result := Protected_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.base_interfaces
                  Property_Result := Task_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.base_interfaces
                  Property_Result := Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start '[dispatcher]BaseTypeDecl.record_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.record_def
                  Property_Result := Base_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.record_def
                  Property_Result := Base_Subtype_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.record_def
                  Property_Result := Classwide_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.record_def
                  Property_Result := Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Record_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.array_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_def
                  Property_Result := Base_Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_def
                  Property_Result := Base_Subtype_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.array_def
                  Property_Result := Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Array_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_iterable_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_iterable_type
                  Property_Result := Base_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_iterable_type
                  Property_Result := Base_Subtype_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_iterable_type
                  Property_Result := Classwide_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_iterable_type
                  Property_Result := Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_interface_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_interface_type
                  Property_Result := Base_Type_Decl_P_Is_Interface_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_interface_type
                  Property_Result := Classwide_Type_Decl_P_Is_Interface_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.iterable_comp_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.iterable_comp_type
                  Property_Result := Base_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.iterable_comp_type
                  Property_Result := Base_Subtype_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.iterable_comp_type
                  Property_Result := Classwide_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.iterable_comp_type
                  Property_Result := Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.iterable_cursor_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Iterable_Cursor_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.iterable_cursor_type
                  Property_Result := Base_Type_Decl_P_Iterable_Cursor_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.iterable_cursor_type
                  Property_Result := Base_Subtype_Decl_P_Iterable_Cursor_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.iterable_cursor_type
                  Property_Result := Type_Decl_P_Iterable_Cursor_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Iterable_Cursor_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.canonical_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start BaseTypeDecl.canonical_type
                  Property_Result := Base_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.canonical_type
                  Property_Result := Base_Subtype_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.canonical_type
                  Property_Result := Classwide_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_private' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_private
                  Property_Result := Base_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_private
                  Property_Result := Base_Subtype_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_private
                  Property_Result := Classwide_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.is_private
                  Property_Result := Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Is_Private;
--# end
--# property-start '[dispatcher]BaseTypeDecl.root_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.root_type
                  Property_Result := Base_Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.root_type
                  Property_Result := Base_Subtype_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
                  --# property-call-start TypeDecl.root_type
                  Property_Result := Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Type_Decl_P_Root_Type;
--# end
--# property-start BaseTypeDecl._env_mappings_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6624'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_16
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1680 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1680 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1680;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4482 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4480 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4480
--# expr-start 4481 '<FieldAccess for name_symbol at nodes.lkt:6624:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6624'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4481
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4482
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1680;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1680;
            raise;
      end;
   return Property_Result;
end Env_Mappings_16;
--# end
--# property-start BaseTypeDecl.parent_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6457'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_656 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_656 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_656;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4483 '<FieldAccess for empty_env at nodes.lkt:6457:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6457'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 4483
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_656;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_656;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start BaseTypeDecl.primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6459'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start BaseTypeDecl.is_record_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6467'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4486 '<Not at nodes.lkt:6467:34>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6467'
--# expr-start 4485 '<IsNull at nodes.lkt:6467:38>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6467'
--# expr-start 4484 '<FieldAccess for record_def at nodes.lkt:6467:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6467'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4484
Is_Null := Fld.Node = null; 
--# expr-done 4485
Not_Val := not (Is_Null); 
--# expr-done 4486
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6470'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start BaseTypeDecl.is_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6475'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start BaseTypeDecl.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6480'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start BaseTypeDecl.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6485'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseTypeDecl.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6490'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseTypeDecl.is_classwide 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6492'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start BaseTypeDecl.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6497'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.has_ud_indexing 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6500'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseTypeDecl.constant_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6504'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.variable_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6509'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.get_imp_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6513'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Expr;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start BaseTypeDecl.access_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6516'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Access_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Access_Def;
--# end
--# property-start BaseTypeDecl.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6521'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start BaseTypeDecl.classwide_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6526'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
      Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Bare_Classwide_Type_Decl;
As_Entity : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Boolean;
If_Result_1 : Internal_Entity_Classwide_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4491 '<bind at nodes.lkt:6527:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6527'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 4490 '<If at nodes.lkt:6529:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6529'
--# expr-start 4487 '<FieldAccess for is_tagged_type at nodes.lkt:6529:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6529'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 4487
if Fld_1 then
   --# expr-start 4489 '<as_entity at nodes.lkt:6529:39>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6529'
--# expr-start 4488 '<FieldAccess for classwide_type_node at nodes.lkt:6529:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6529'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.classwide_type_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Classwide_Type_Node (Node => Self);
--# end
--# expr-done 4488
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Classwide_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 4489
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity_Classwide_Type_Decl;
end if;
--# expr-done 4490
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 4491
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6534'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Discrete_Range;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.is_discrete_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6539'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4492 '<FieldAccess for is_int_type at nodes.lkt:6540:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6540'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4492
if Fld then
   If_Result := True;
else
   --# expr-start 4493 '<FieldAccess for is_enum_type at nodes.lkt:6540:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6540'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4493
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 4494 '<FieldAccess for is_char_type at nodes.lkt:6541:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6541'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4494
   If_Result_1 := Fld_2;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseTypeDecl.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6546'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6552'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6557'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseTypeDecl.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6562'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Type;
--# end
--# property-start BaseTypeDecl.base_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6567'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_679 with Inline_Always;
            procedure Finalizer_Scope_2585 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Array_Lit : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_679 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_679;
            procedure Finalizer_Scope_2585 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2585;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4495 '<FieldAccess for base_type at nodes.lkt:6568:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6568'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4495
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 4496 '<ArrayLiteral at nodes.lkt:6568:37>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6568'
Array_Lit := Create_Internal_Entity_Base_Type_Decl_Array (Internal_Internal_Entity_Base_Type_Decl_Array'(1 => Var_Expr_0)); 
--# expr-done 4496
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2585;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 4497 '<FieldAccess for base_interfaces at nodes.lkt:6568:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6568'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4497
Concat_Result := Concat (Result_Var, Fld_1); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_679;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_679;
                     Finalizer_Scope_2585;
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Types;
--# end
--# property-start BaseTypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6571'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.record_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6575'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Record_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Record_Def;
--# end
--# property-start BaseTypeDecl.array_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6578'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Array_Type_Def;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.is_iterable_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6582'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.is_interface_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6587'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Is_A : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4498 '<FieldAccess for full_view at nodes.lkt:6587:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6587'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4498
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 4503 '<Match at nodes.lkt:6588:17>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6588'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind td Match_0
--# expr-start 4500 '<IsA InterfaceTypeDef at nodes.lkt:6589:34>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6589'
--# expr-start 4499 '<FieldAccess for type_def at nodes.lkt:6589:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6589'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 4499
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Interface_Type_Def_Range; 
--# expr-done 4500
Let_Result := Is_A; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind sb Match_1
--# expr-start 4502 '<FieldAccess for is_interface_type at nodes.lkt:6590:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6590'
--# expr-start 4501 '<FieldAccess for get_type at nodes.lkt:6590:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6590'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4501
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_interface_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Interface_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4502
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4503
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start BaseTypeDecl.iterable_comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6596'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.iterable_cursor_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6600'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Cursor_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Iterable_Cursor_Type;
--# end
--# property-start BaseTypeDecl.canonical_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6607'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4506 '<bind at nodes.lkt:6608:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6608'
Bound_Imprecise_Fallback := False; 
--# expr-start 4505 '<Cast to BaseTypeDecl at nodes.lkt:6610:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6610'
--# expr-start 4504 '<FieldAccess for canonical_part at nodes.lkt:6610:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6610'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4504
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4505
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 4506
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.is_private 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6615'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Is_Private;
--# end
--# property-start BaseTypeDecl.root_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6621'
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl  := Bare_Base_Type_Decl (Node);
     Ent : Internal_Entity_Base_Type_Decl :=
       Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Type_Decl_P_Root_Type;
--# end
function Base_Type_Decl_F_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Base_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Base_Type_Decl_F_Direct_Primitive_Subps (E.Node);
   begin
         return Result;
   end;
end;
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Base_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Base_Type_Decl_P_Dottable_Subps (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Base_Subtype_Decl
        (Self : Bare_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Base_Subtype_Decl;
--# property-start BaseSubtypeDecl.from_type_bound 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6631'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_From_Type_Bound
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4508 '<bind at nodes.lkt:6637:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6637'
Bound_Origin := No_Bare_Ada_Node; 
--# expr-start 4507 '<FieldAccess for get_type at nodes.lkt:6639:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6639'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 4507
Dyn_Var_Bind_Result := Fld; 
--# expr-done 4508
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_From_Type_Bound;
--# end
--# property-start '[dispatcher]BaseSubtypeDecl.get_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Base_Subtype_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.get_type
                  Property_Result := Discrete_Base_Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.get_type
                  Property_Result := Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Base_Subtype_Decl_P_Get_Type;
--# end
--# property-start BaseSubtypeDecl.primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6648'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Primitives_Env
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_692 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_692 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_692;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4510 '<FieldAccess for primitives_env at nodes.lkt:6649:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6649'
--# expr-start 4509 '<FieldAccess for from_type_bound at nodes.lkt:6649:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6649'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4509
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4510
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_692;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_692;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Primitives_Env;
--# end
--# property-start BaseSubtypeDecl.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6652'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Ndims
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4512 '<FieldAccess for array_ndims at nodes.lkt:6652:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6652'
--# expr-start 4511 '<FieldAccess for get_type at nodes.lkt:6652:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6652'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4511
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4512
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Array_Ndims;
--# end
--# property-start BaseSubtypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6655'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Defining_Env
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_694 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_694 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_694;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4514 '<FieldAccess for defining_env at nodes.lkt:6655:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6655'
--# expr-start 4513 '<FieldAccess for get_type at nodes.lkt:6655:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6655'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4513
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 4514
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_694;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_694;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Defining_Env;
--# end
--# property-start BaseSubtypeDecl.canonical_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6658'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Canonical_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4516 '<FieldAccess for canonical_type at nodes.lkt:6659:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6659'
--# expr-start 4515 '<FieldAccess for get_type at nodes.lkt:6659:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6659'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4515
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4516
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Canonical_Type;
--# end
--# property-start BaseSubtypeDecl.record_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6662'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Record_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4518 '<FieldAccess for record_def at nodes.lkt:6662:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6662'
--# expr-start 4517 '<FieldAccess for get_type at nodes.lkt:6662:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6662'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4517
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4518
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Record_Def;
--# end
--# property-start BaseSubtypeDecl.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6665'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Accessed_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4520 '<FieldAccess for accessed_type at nodes.lkt:6666:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6666'
--# expr-start 4519 '<FieldAccess for get_type at nodes.lkt:6666:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6666'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4519
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4520
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Accessed_Type;
--# end
--# property-start BaseSubtypeDecl.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6668'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Task_Type
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4522 '<FieldAccess for is_task_type at nodes.lkt:6668:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6668'
--# expr-start 4521 '<FieldAccess for get_type at nodes.lkt:6668:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6668'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 4521
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_task_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4522
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Task_Type;
--# end
--# property-start BaseSubtypeDecl.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6671'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Int_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4524 '<FieldAccess for is_int_type at nodes.lkt:6671:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6671'
--# expr-start 4523 '<FieldAccess for get_type at nodes.lkt:6671:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6671'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4523
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4524
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Int_Type;
--# end
--# property-start BaseSubtypeDecl.is_discrete_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6674'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4526 '<FieldAccess for is_discrete_type at nodes.lkt:6674:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6674'
--# expr-start 4525 '<FieldAccess for get_type at nodes.lkt:6674:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6674'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4525
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4526
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseSubtypeDecl.is_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6677'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Real_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4528 '<FieldAccess for is_real_type at nodes.lkt:6677:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6677'
--# expr-start 4527 '<FieldAccess for get_type at nodes.lkt:6677:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6677'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4527
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4528
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Real_Type;
--# end
--# property-start BaseSubtypeDecl.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6680'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Float_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4530 '<FieldAccess for is_float_type at nodes.lkt:6680:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6680'
--# expr-start 4529 '<FieldAccess for get_type at nodes.lkt:6680:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6680'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4529
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4530
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Float_Type;
--# end
--# property-start BaseSubtypeDecl.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6683'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4532 '<FieldAccess for is_fixed_point at nodes.lkt:6683:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6683'
--# expr-start 4531 '<FieldAccess for get_type at nodes.lkt:6683:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6683'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4531
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4532
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseSubtypeDecl.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6686'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4534 '<FieldAccess for is_enum_type at nodes.lkt:6686:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6686'
--# expr-start 4533 '<FieldAccess for get_type at nodes.lkt:6686:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6686'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4533
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4534
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseSubtypeDecl.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6689'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Access_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4536 '<FieldAccess for is_access_type at nodes.lkt:6689:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6689'
--# expr-start 4535 '<FieldAccess for get_type at nodes.lkt:6689:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6689'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4535
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4536
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Access_Type;
--# end
--# property-start BaseSubtypeDecl.access_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6692'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Access_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4538 '<FieldAccess for access_def at nodes.lkt:6692:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6692'
--# expr-start 4537 '<FieldAccess for get_type at nodes.lkt:6692:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6692'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4537
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4538
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Access_Def;
--# end
--# property-start BaseSubtypeDecl.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6695'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Char_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4540 '<FieldAccess for is_char_type at nodes.lkt:6695:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6695'
--# expr-start 4539 '<FieldAccess for get_type at nodes.lkt:6695:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6695'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4539
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4540
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Char_Type;
--# end
--# property-start BaseSubtypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6698'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4542 '<FieldAccess for is_tagged_type at nodes.lkt:6698:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6698'
--# expr-start 4541 '<FieldAccess for get_type at nodes.lkt:6698:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6698'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4541
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4542
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseSubtypeDecl.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6701'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4544 '<FieldAccess for base_type at nodes.lkt:6701:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6701'
--# expr-start 4543 '<FieldAccess for get_type at nodes.lkt:6701:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6701'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4543
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4544
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Type;
--# end
--# property-start BaseSubtypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6704'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Interfaces
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_710 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_710 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_710;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4546 '<FieldAccess for base_interfaces at nodes.lkt:6705:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6705'
--# expr-start 4545 '<FieldAccess for get_type at nodes.lkt:6705:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6705'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4545
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4546
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_710;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_710;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Interfaces;
--# end
--# property-start BaseSubtypeDecl.base_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6708'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Types
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_711 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_711 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_711;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4548 '<FieldAccess for base_types at nodes.lkt:6709:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6709'
--# expr-start 4547 '<FieldAccess for get_type at nodes.lkt:6709:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6709'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4547
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4548
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_711;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_711;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Base_Types;
--# end
--# property-start BaseSubtypeDecl.array_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6712'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4550 '<FieldAccess for array_def at nodes.lkt:6712:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6712'
--# expr-start 4549 '<FieldAccess for get_type at nodes.lkt:6712:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6712'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4549
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4550
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Array_Def;
--# end
--# property-start BaseSubtypeDecl.is_classwide 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6714'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Classwide
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4552 '<FieldAccess for is_classwide at nodes.lkt:6714:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6714'
--# expr-start 4551 '<FieldAccess for from_type_bound at nodes.lkt:6714:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6714'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4551
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4552
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Classwide;
--# end
--# property-start BaseSubtypeDecl.is_iterable_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6717'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4554 '<FieldAccess for is_iterable_type at nodes.lkt:6717:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6717'
--# expr-start 4553 '<FieldAccess for get_type at nodes.lkt:6717:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6717'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4553
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4554
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseSubtypeDecl.iterable_comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6720'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4556 '<FieldAccess for iterable_comp_type at nodes.lkt:6721:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6721'
--# expr-start 4555 '<FieldAccess for get_type at nodes.lkt:6721:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6721'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4555
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4556
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseSubtypeDecl.iterable_cursor_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6724'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Iterable_Cursor_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4558 '<FieldAccess for iterable_cursor_type at nodes.lkt:6725:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6725'
--# expr-start 4557 '<FieldAccess for get_type at nodes.lkt:6725:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6725'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4557
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_cursor_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Cursor_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4558
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Iterable_Cursor_Type;
--# end
--# property-start BaseSubtypeDecl.is_record_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6728'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Record_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4560 '<FieldAccess for is_record_type at nodes.lkt:6728:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6728'
--# expr-start 4559 '<FieldAccess for get_type at nodes.lkt:6728:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6728'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4559
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4560
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Record_Type;
--# end
--# property-start BaseSubtypeDecl.is_private 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6730'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Private
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4562 '<FieldAccess for is_private at nodes.lkt:6730:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6730'
--# expr-start 4561 '<FieldAccess for from_type_bound at nodes.lkt:6730:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6730'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4561
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4562
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Is_Private;
--# end
--# property-start BaseSubtypeDecl.root_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6733'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Root_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4564 '<FieldAccess for root_type at nodes.lkt:6733:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6733'
--# expr-start 4563 '<FieldAccess for get_type at nodes.lkt:6733:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6733'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4563
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4564
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Root_Type;
--# end
--# property-start BaseSubtypeDecl.has_ud_indexing 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6735'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4566 '<FieldAccess for has_ud_indexing at nodes.lkt:6735:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6735'
--# expr-start 4565 '<FieldAccess for from_type_bound at nodes.lkt:6735:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6735'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4565
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4566
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseSubtypeDecl.constant_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6737'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_721 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_721 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_721;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4568 '<FieldAccess for constant_indexing_fns at nodes.lkt:6738:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6738'
--# expr-start 4567 '<FieldAccess for from_type_bound at nodes.lkt:6738:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6738'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4567
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4568
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_721;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_721;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseSubtypeDecl.variable_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6740'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_722 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_722 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_722;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4570 '<FieldAccess for variable_indexing_fns at nodes.lkt:6741:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6741'
--# expr-start 4569 '<FieldAccess for from_type_bound at nodes.lkt:6741:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6741'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4569
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4570
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_722;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_722;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseSubtypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6744'
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Discriminants_List
  (Node : Bare_Base_Subtype_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl  := Bare_Base_Subtype_Decl (Node);
     Ent : Internal_Entity_Base_Subtype_Decl :=
       Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_723 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_723 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_723;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4572 '<FieldAccess for discriminants_list at nodes.lkt:6745:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6745'
--# expr-start 4571 '<FieldAccess for get_type at nodes.lkt:6745:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6745'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4571
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4572
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_723;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_723;
            raise;
      end;
   return Property_Result;
end Base_Subtype_Decl_P_Discriminants_List;
--# end
   --
   --  Primitives for Bare_Discrete_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Discrete_Base_Subtype_Decl
        (Self : Bare_Discrete_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Discrete_Base_Subtype_Decl;
--# property-start DiscreteBaseSubtypeDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6756'
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discrete_Base_Subtype_Decl  := Bare_Discrete_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discrete_Base_Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start DiscreteBaseSubtypeDecl.get_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6759'
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Discrete_Base_Subtype_Decl  := Bare_Discrete_Base_Subtype_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4575 '<as_entity at nodes.lkt:6760:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6760'
--# expr-start 4574 '<Cast to BaseTypeDecl at nodes.lkt:6760:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6760'
--# expr-start 4573 '<parent at nodes.lkt:6760:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6760'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 4573
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4574
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 4575
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Discrete_Base_Subtype_Decl_P_Get_Type;
--# end
   --
   --  Primitives for Bare_Subtype_Decl
   --
      procedure Subtype_Decl_Pre_Env_Actions
        (Self            : Bare_Subtype_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_17 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:6789:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_18 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_19 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:6801:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Subtype_Decl
        (Self : Bare_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Subtype_Decl_F_Subtype : Bare_Subtype_Indication
         ; Subtype_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Subtype_Decl_F_Subtype := Subtype_Decl_F_Subtype;
            Self.Subtype_Decl_F_Aspects := Subtype_Decl_F_Aspects;
      end Initialize_Fields_For_Subtype_Decl;
   function Subtype_Decl_F_Subtype
     (Node : Bare_Subtype_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.Subtype_Decl_F_Subtype;
   end;
--# property-start SubtypeDecl.get_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6769'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Get_Type
  (Node : Bare_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4579 '<Match at nodes.lkt:6769:44>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6769'
--# expr-start 4577 '<FieldAccess for designated_type at nodes.lkt:6769:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6769'
--# expr-start 4576 '<FieldAccess for subtype at nodes.lkt:6769:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6769'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 4576
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4577
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 4578 '<FieldAccess for get_type at nodes.lkt:6770:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6770'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4578
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4579
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Get_Type;
--# end
--# property-start SubtypeDecl.get_imp_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6775'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Get_Imp_Deref
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Expr;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Subtype_Decl_Subtype_Decl_P_Get_Imp_Deref,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Subtype_Decl,
                                As_Bare_Subtype_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Expr;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4581 '<FieldAccess for get_imp_deref at nodes.lkt:6776:8>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6776'
--# expr-start 4580 '<FieldAccess for get_type at nodes.lkt:6776:8>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6776'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 4580
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4581
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Expr,
                        As_Internal_Entity_Expr => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Subtype_Decl_P_Get_Imp_Deref;
--# end
--# property-start SubtypeDecl.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6778'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Discrete_Range
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4583 '<FieldAccess for discrete_range at nodes.lkt:6778:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6778'
--# expr-start 4582 '<FieldAccess for subtype at nodes.lkt:6778:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 4582
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4583
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Discrete_Range;
--# end
--# property-start SubtypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6781'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Equation
  (Node : Bare_Subtype_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_729 with Inline_Always;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_729 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_729;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4585 '<FieldAccess for sub_equation at nodes.lkt:6781:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6781'
--# expr-start 4584 '<FieldAccess for subtype at nodes.lkt:6781:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6781'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 4584
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 4585
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_729;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_729;
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Xref_Equation;
--# end
--# property-start SubtypeDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6784'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4587 '<FieldAccess for is_static_subtype at nodes.lkt:6784:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6784'
--# expr-start 4586 '<FieldAccess for subtype at nodes.lkt:6784:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6784'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 4586
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4587
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start SubtypeDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6786'
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Entry_Point
  (Node : Bare_Subtype_Decl
  )
   return Boolean
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Decl_P_Xref_Entry_Point;
--# end
--# property-start SubtypeDecl._env_mappings_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6789'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_17
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1681 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Subtype_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1681 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1681;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4590 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4588 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4588
--# expr-start 4589 '<FieldAccess for name_symbol at nodes.lkt:6789:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6789'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4589
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4590
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1681;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1681;
            raise;
      end;
   return Property_Result;
end Env_Mappings_17;
--# end
--# property-start SubtypeDecl._env_trans_parent_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6800'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_18
  (Node : Bare_Subtype_Decl
  )
   return Boolean
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_18;
--# end
--# property-start SubtypeDecl._env_mappings_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6801'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_19
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subtype_Decl  := Bare_Subtype_Decl (Node);
     Ent : Internal_Entity_Subtype_Decl :=
       Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1683 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1683 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1683;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4591 '<FieldAccess for synthetic_object_decl_env_assoc at nodes.lkt:6801:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6801'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.synthetic_object_decl_env_assoc
Fld := Libadalang.Implementation.Base_Type_Decl_F_Synthetic_Object_Decl_Env_Assoc (Node => Ent.Node);
--# end
--# expr-done 4591
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1683;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1683;
            raise;
      end;
   return Property_Result;
end Env_Mappings_19;
--# end
   --
   --  Primitives for Bare_Classwide_Type_Decl
   --
      procedure Initialize_Fields_For_Classwide_Type_Decl
        (Self : Bare_Classwide_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Classwide_Type_Decl;
--# property-start ClasswideTypeDecl.type_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6811'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Type_Decl
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4593 '<Cast to BaseTypeDecl at nodes.lkt:6811:45>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6811'
--# expr-start 4592 '<parent at nodes.lkt:6811:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6811'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4592
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4593
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Type_Decl;
--# end
--# property-start ClasswideTypeDecl.is_classwide 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6813'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Classwide
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Classwide;
--# end
--# property-start ClasswideTypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6816'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start ClasswideTypeDecl.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6819'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Base_Type;
--# end
--# property-start ClasswideTypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6822'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Interfaces
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ClasswideTypeDecl.record_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6826'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Record_Def
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4595 '<FieldAccess for record_def at nodes.lkt:6826:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6826'
--# expr-start 4594 '<FieldAccess for type_decl at nodes.lkt:6826:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6826'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4594
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4595
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Record_Def;
--# end
--# property-start ClasswideTypeDecl.classwide_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6828'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Classwide_Type
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Ent;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Classwide_Type;
--# end
--# property-start ClasswideTypeDecl.is_iterable_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6831'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4597 '<FieldAccess for is_iterable_type at nodes.lkt:6831:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6831'
--# expr-start 4596 '<FieldAccess for type_decl at nodes.lkt:6831:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6831'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4596
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4597
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start ClasswideTypeDecl.has_ud_indexing 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6833'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4599 '<FieldAccess for has_ud_indexing at nodes.lkt:6833:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6833'
--# expr-start 4598 '<FieldAccess for type_decl at nodes.lkt:6833:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6833'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4598
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4599
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start ClasswideTypeDecl.constant_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6835'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_741 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_741 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_741;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4601 '<FieldAccess for constant_indexing_fns at nodes.lkt:6836:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6836'
--# expr-start 4600 '<FieldAccess for type_decl at nodes.lkt:6836:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6836'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4600
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4601
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_741;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_741;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start ClasswideTypeDecl.variable_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6838'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_742 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_742 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_742;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4603 '<FieldAccess for variable_indexing_fns at nodes.lkt:6839:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6839'
--# expr-start 4602 '<FieldAccess for type_decl at nodes.lkt:6839:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6839'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4602
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4603
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_742;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_742;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start ClasswideTypeDecl.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6841'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Task_Type
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4605 '<FieldAccess for is_task_type at nodes.lkt:6841:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6841'
--# expr-start 4604 '<FieldAccess for type_decl at nodes.lkt:6841:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4604
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_task_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4605
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Task_Type;
--# end
--# property-start ClasswideTypeDecl.iterable_comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6844'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4607 '<FieldAccess for iterable_comp_type at nodes.lkt:6845:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6845'
--# expr-start 4606 '<FieldAccess for type_decl at nodes.lkt:6845:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6845'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4606
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4607
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start ClasswideTypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6848'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Defining_Env
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_745 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_745 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_745;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4609 '<FieldAccess for defining_env at nodes.lkt:6848:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6848'
--# expr-start 4608 '<FieldAccess for type_decl at nodes.lkt:6848:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6848'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4608
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 4609
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_745;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_745;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Defining_Env;
--# end
--# property-start ClasswideTypeDecl.is_private 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6850'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Private
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4611 '<FieldAccess for is_private at nodes.lkt:6850:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6850'
--# expr-start 4610 '<FieldAccess for type_decl at nodes.lkt:6850:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6850'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4610
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4611
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Private;
--# end
--# property-start ClasswideTypeDecl.is_in_private_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6852'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_In_Private_Part
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4613 '<FieldAccess for is_in_private_part at nodes.lkt:6852:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6852'
--# expr-start 4612 '<FieldAccess for type_decl at nodes.lkt:6852:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6852'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4612
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4613
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_In_Private_Part;
--# end
--# property-start ClasswideTypeDecl.is_in_public_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6854'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_In_Public_Part
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4615 '<FieldAccess for is_in_public_part at nodes.lkt:6854:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6854'
--# expr-start 4614 '<FieldAccess for type_decl at nodes.lkt:6854:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6854'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4614
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_public_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Public_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4615
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_In_Public_Part;
--# end
--# property-start ClasswideTypeDecl.get_aspect_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6856'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Get_Aspect_Spec
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Spec
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Aspect_Spec;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Aspect_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4617 '<FieldAccess for get_aspect_spec at nodes.lkt:6857:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6857'
--# expr-start 4616 '<FieldAccess for type_decl at nodes.lkt:6857:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6857'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4616
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Spec (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4617
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Get_Aspect_Spec;
--# end
--# property-start ClasswideTypeDecl.is_interface_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6860'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4619 '<FieldAccess for is_interface_type at nodes.lkt:6860:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6860'
--# expr-start 4618 '<FieldAccess for type_decl at nodes.lkt:6860:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6860'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4618
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_interface_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Interface_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4619
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start ClasswideTypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6863'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Discriminants_List
  (Node : Bare_Classwide_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_751 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_751 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_751;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4621 '<FieldAccess for discriminants_list at nodes.lkt:6864:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6864'
--# expr-start 4620 '<FieldAccess for type_decl at nodes.lkt:6864:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6864'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4620
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4621
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_751;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_751;
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Discriminants_List;
--# end
--# property-start ClasswideTypeDecl.canonical_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6867'
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Canonical_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl  := Bare_Classwide_Type_Decl (Node);
     Ent : Internal_Entity_Classwide_Type_Decl :=
       Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Classwide_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4623 '<FieldAccess for canonical_type at nodes.lkt:6868:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6868'
--# expr-start 4622 '<FieldAccess for type_decl at nodes.lkt:6868:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6868'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ClasswideTypeDecl.type_decl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4622
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4623
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 4624 '<FieldAccess for classwide_type at nodes.lkt:6872:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6872'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4624
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
   Result_Var := Var_Expr_0;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classwide_Type_Decl_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Incomplete_Type_Decl
   --
      procedure Incomplete_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Incomplete_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_20 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:6903:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_21 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Incomplete_Type_Decl
        (Self : Bare_Incomplete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Incomplete_Type_Decl_F_Discriminants := Incomplete_Type_Decl_F_Discriminants;
      end Initialize_Fields_For_Incomplete_Type_Decl;
   function Incomplete_Type_Decl_F_Discriminants
     (Node : Bare_Incomplete_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Incomplete_Type_Decl_F_Discriminants;
   end;
--# property-start IncompleteTypeDecl.find_next_part_in 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6888'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Find_Next_Part_In
  (Node : Bare_Incomplete_Type_Decl
      ; Decl_Part : Internal_Entity_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind decl_part Decl_Part
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_753 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_753 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_753;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4632 '<Cast to BaseTypeDecl at nodes.lkt:6888:87>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6888'
--# expr-start 4631 '<Find at nodes.lkt:6888:87>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6888'
   --# expr-start 4627 '<EnvGet at nodes.lkt:6888:87>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6888'
--# expr-start 4625 '<FieldAccess for children_env at nodes.lkt:6888:87>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6888'
      if Decl_Part.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Decl_Part.Node, E_Info => Decl_Part.Info);
--# expr-done 4625
--# expr-start 4626 '<FieldAccess for name_symbol at nodes.lkt:6889:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6889'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4626
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Thin (Fld_1), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 4627
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
--# expr-start 4628 '<Cast to BaseTypeDecl at nodes.lkt:6891:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6891'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4628
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind btd Var_Expr_1
--# expr-start 4630 '<Not at nodes.lkt:6891:47>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6891'
--# expr-start 4629 '<Eq at nodes.lkt:6891:47>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6891'
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4629
Not_Val := not (Is_Equiv); 
--# expr-done 4630
Scope_Result := Not_Val;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 4631
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4632
         Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_753;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_753;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Find_Next_Part_In;
--# end
--# property-start IncompleteTypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6895'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Defining_Env
  (Node : Bare_Incomplete_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_754 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_754 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_754;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4636 '<env_group at nodes.lkt:6896:9>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6896'
--# expr-start 4635 '<ArrayLiteral at nodes.lkt:6896:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6896'
--# expr-start 4633 '<FieldAccess for children_env at nodes.lkt:6896:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6896'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 4633
--# expr-start 4634 '<FieldAccess for dottable_subps_env at nodes.lkt:6896:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6896'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Self);
--# end
--# expr-done 4634
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1)); 
--# expr-done 4635
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4636
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_754;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_754;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Defining_Env;
--# end
--# property-start IncompleteTypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6899'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Discriminants_List
  (Node : Bare_Incomplete_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_755 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_755 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_755;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4638 '<FieldAccess for abstract_formal_params at nodes.lkt:6900:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6900'
--# expr-start 4637 '<FieldAccess for discriminants at nodes.lkt:6900:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6900'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Incomplete_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 4637
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4638
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_755;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_755;
            raise;
      end;
   return Property_Result;
end Incomplete_Type_Decl_P_Discriminants_List;
--# end
--# property-start IncompleteTypeDecl._env_mappings_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6903'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_20
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Type_Decl :=
       Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1684 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Incomplete_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1684 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1684;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4641 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4639 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4639
--# expr-start 4640 '<FieldAccess for name_symbol at nodes.lkt:6903:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6903'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4640
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4641
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1684;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1684;
            raise;
      end;
   return Property_Result;
end Env_Mappings_20;
--# end
--# property-start IncompleteTypeDecl._env_trans_parent_21 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_21
  (Node : Bare_Incomplete_Type_Decl
  )
   return Boolean
is
   Self : Bare_Incomplete_Type_Decl  := Bare_Incomplete_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_21;
--# end
   --
   --  Primitives for Bare_Incomplete_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Formal_Type_Decl
        (Self : Bare_Incomplete_Formal_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Formal_Type_Decl_F_Is_Tagged : Bare_Tagged_Node
         ; Incomplete_Formal_Type_Decl_F_Default_Type : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Formal_Type_Decl_F_Is_Tagged := Incomplete_Formal_Type_Decl_F_Is_Tagged;
            Self.Incomplete_Formal_Type_Decl_F_Default_Type := Incomplete_Formal_Type_Decl_F_Default_Type;
      end Initialize_Fields_For_Incomplete_Formal_Type_Decl;
   function Incomplete_Formal_Type_Decl_F_Is_Tagged
     (Node : Bare_Incomplete_Formal_Type_Decl) return Bare_Tagged_Node
   is
   begin
         return Node.Incomplete_Formal_Type_Decl_F_Is_Tagged;
   end;
   function Incomplete_Formal_Type_Decl_F_Default_Type
     (Node : Bare_Incomplete_Formal_Type_Decl) return Bare_Name
   is
   begin
         return Node.Incomplete_Formal_Type_Decl_F_Default_Type;
   end;
--# property-start IncompleteFormalTypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6914'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Formal_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Formal_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Formal_Type_Decl  := Bare_Incomplete_Formal_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4644 '<Not at nodes.lkt:6914:34>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6914'
--# expr-start 4643 '<IsNull at nodes.lkt:6914:38>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6914'
--# expr-start 4642 '<FieldAccess for is_tagged at nodes.lkt:6914:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6914'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Incomplete_Formal_Type_Decl_F_Is_Tagged;
--# expr-done 4642
Is_Null := Fld = null; 
--# expr-done 4643
Not_Val := not (Is_Null); 
--# expr-done 4644
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Formal_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start IncompleteFormalTypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6917'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Formal_Type_Decl_P_Xref_Equation
  (Node : Bare_Incomplete_Formal_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Incomplete_Formal_Type_Decl  := Bare_Incomplete_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Incomplete_Formal_Type_Decl :=
       Internal_Entity_Incomplete_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_757 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_757 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_757;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4646 '<FieldAccess for sub_equation at nodes.lkt:6917:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6917'
--# expr-start 4645 '<FieldAccess for default_type at nodes.lkt:6917:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6917'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Incomplete_Formal_Type_Decl_F_Default_Type, Info => Ent.Info);
--# expr-done 4645
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 4646
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_757;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_757;
            raise;
      end;
   return Property_Result;
end Incomplete_Formal_Type_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Incomplete_Tagged_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Tagged_Type_Decl
        (Self : Bare_Incomplete_Tagged_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Tagged_Type_Decl_F_Has_Abstract : Bare_Abstract_Node
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Tagged_Type_Decl_F_Has_Abstract := Incomplete_Tagged_Type_Decl_F_Has_Abstract;
      end Initialize_Fields_For_Incomplete_Tagged_Type_Decl;
   function Incomplete_Tagged_Type_Decl_F_Has_Abstract
     (Node : Bare_Incomplete_Tagged_Type_Decl) return Bare_Abstract_Node
   is
   begin
         return Node.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
   end;
--# property-start IncompleteTaggedTypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6925'
pragma Warnings (Off, "is not referenced");
 function Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Tagged_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Tagged_Type_Decl  := Bare_Incomplete_Tagged_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type;
--# end
   --
   --  Primitives for Bare_Protected_Type_Decl
   --
      procedure Protected_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Protected_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_22 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:6969:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_23 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_24 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Type_Decl
        (Self : Bare_Protected_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Protected_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Protected_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Protected_Type_Decl_F_Interfaces : Bare_Parent_List
         ; Protected_Type_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Protected_Type_Decl_F_Discriminants := Protected_Type_Decl_F_Discriminants;
            Self.Protected_Type_Decl_F_Aspects := Protected_Type_Decl_F_Aspects;
            Self.Protected_Type_Decl_F_Interfaces := Protected_Type_Decl_F_Interfaces;
            Self.Protected_Type_Decl_F_Definition := Protected_Type_Decl_F_Definition;
      end Initialize_Fields_For_Protected_Type_Decl;
   function Protected_Type_Decl_F_Discriminants
     (Node : Bare_Protected_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Protected_Type_Decl_F_Discriminants;
   end;
   function Protected_Type_Decl_F_Interfaces
     (Node : Bare_Protected_Type_Decl) return Bare_Parent_List
   is
   begin
         return Node.Protected_Type_Decl_F_Interfaces;
   end;
   function Protected_Type_Decl_F_Definition
     (Node : Bare_Protected_Type_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Protected_Type_Decl_F_Definition;
   end;
--# property-start ProtectedTypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6936'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Discriminants_List
  (Node : Bare_Protected_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_759 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_759 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_759;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4648 '<FieldAccess for abstract_formal_params at nodes.lkt:6937:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6937'
--# expr-start 4647 '<FieldAccess for discriminants at nodes.lkt:6937:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6937'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Protected_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 4647
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4648
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_759;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_759;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Discriminants_List;
--# end
--# property-start ProtectedTypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6940'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Defining_Env
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_760 with Inline_Always;
            procedure Finalizer_Scope_2596 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Private_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Private_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_760 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_760;
            procedure Finalizer_Scope_2596 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2596;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4650 '<FieldAccess for private_part at nodes.lkt:6940:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6940'
--# expr-start 4649 '<FieldAccess for definition at nodes.lkt:6940:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6940'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Protected_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 4649
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Fld.Node.Protected_Def_F_Private_Part, Info => Fld.Info);
--# expr-done 4650
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Private_Part then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 4654 '<env_group at nodes.lkt:6945:9>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6945'
--# expr-start 4653 '<ArrayLiteral at nodes.lkt:6945:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6945'
--# expr-start 4651 '<FieldAccess for children_env at nodes.lkt:6945:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6945'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4651
--# expr-start 4652 '<FieldAccess for children_env at nodes.lkt:6945:31>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6945'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 4652
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 4653
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4654
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2596;
   Result_Var := Scope_Result;
else
   --# expr-start 4655 '<FieldAccess for children_env at nodes.lkt:6946:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6946'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4655
   Result_Var := Fld_4;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_760;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_760;
                     Finalizer_Scope_2596;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Defining_Env;
--# end
--# property-start ProtectedTypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6950'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Base_Interfaces
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_761 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_761 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_761;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4658 '<Map at nodes.lkt:6951:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6951'
   --# expr-start 4656 '<FieldAccess for interfaces at nodes.lkt:6951:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6951'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Protected_Type_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 4656
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 4657 '<FieldAccess for name_designated_type at nodes.lkt:6951:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6951'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 4657
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4658
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_761;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_761;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ProtectedTypeDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6953'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start ProtectedTypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6956'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Equation
  (Node : Bare_Protected_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_763 with Inline_Always;
            procedure Finalizer_Scope_2598 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_763 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_763;
            procedure Finalizer_Scope_2598 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2598;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4662 '<LogicAll at nodes.lkt:6957:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6957'
--# expr-start 4661 '<Map at nodes.lkt:6957:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6957'
   --# expr-start 4659 '<FieldAccess for interfaces at nodes.lkt:6957:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6957'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Protected_Type_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 4659
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind ifc Item_0
      --# expr-start 4660 '<FieldAccess for xref_equation at nodes.lkt:6957:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6957'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 4660
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2598;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4661
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:6957:9") else null)); 
--# expr-done 4662
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_763;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_763;
                     Finalizer_Scope_2598;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Xref_Equation;
--# end
--# property-start ProtectedTypeDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6959'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Declarative_Parts
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_764 with Inline_Always;
            procedure Finalizer_Scope_2599 with Inline_Always;
      Local_Pdef : Internal_Entity_Protected_Def;
Var_Expr_0 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Array_Lit_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_764 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_764;
            procedure Finalizer_Scope_2599 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2599;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4670 '<Let at nodes.lkt:6960:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6960'
--# scope-start
--# expr-start 4663 '<FieldAccess for definition at nodes.lkt:6960:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6960'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Protected_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 4663
Local_Pdef := Fld; 
--# bind pdef Local_Pdef
--# expr-start 4666 '<ArrayLiteral at nodes.lkt:6962:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
--# expr-start 4665 '<Cast to DeclarativePart at nodes.lkt:6962:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
--# expr-start 4664 '<FieldAccess for public_part at nodes.lkt:6962:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
      if Local_Pdef.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Local_Pdef.Node.Protected_Def_F_Public_Part, Info => Local_Pdef.Info);
--# expr-done 4664
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 4665
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result)); 
--# expr-done 4666
--# expr-start 4668 '<Cast to DeclarativePart at nodes.lkt:6962:50>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
--# expr-start 4667 '<FieldAccess for private_part at nodes.lkt:6962:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
      if Local_Pdef.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Local_Pdef.Node.Protected_Def_F_Private_Part, Info => Local_Pdef.Info);
--# expr-done 4667
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 4668
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 4669 '<ArrayLiteral at nodes.lkt:6962:99>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6962'
Array_Lit_1 := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Var_Expr_0)); 
--# expr-done 4669
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2599;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Array_Lit, Result_Var); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4670
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_764;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_764;
                     Finalizer_Scope_2599;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Declarative_Parts;
--# end
--# property-start ProtectedTypeDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6965'
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Env_Names
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_765 with Inline_Always;
            procedure Finalizer_Scope_2600 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_765 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_765;
            procedure Finalizer_Scope_2600 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2600;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4671 '<FieldAccess for top_level_env_name at nodes.lkt:6966:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6966'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4671
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 4673 '<ArrayLiteral at nodes.lkt:6966:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6966'
--# expr-start 4672 '<to_symbol at nodes.lkt:6966:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6966'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 4672
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 4673
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2600;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_765;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_765;
                     Finalizer_Scope_2600;
            raise;
      end;
   return Property_Result;
end Protected_Type_Decl_P_Env_Names;
--# end
--# property-start ProtectedTypeDecl._env_mappings_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6969'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_22
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
     Ent : Internal_Entity_Protected_Type_Decl :=
       Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1686 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Protected_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1686 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1686;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4676 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4674 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4674
--# expr-start 4675 '<FieldAccess for name_symbol at nodes.lkt:6969:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6969'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4675
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4676
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1686;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1686;
            raise;
      end;
   return Property_Result;
end Env_Mappings_22;
--# end
--# property-start ProtectedTypeDecl._env_trans_parent_23 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_23
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_23;
--# end
--# property-start ProtectedTypeDecl._env_names_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6970'
pragma Warnings (Off, "is not referenced");
 function Env_Names_24
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl  := Bare_Protected_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1688 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1688 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1688;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4677 '<FieldAccess for env_names at nodes.lkt:6970:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6970'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ProtectedTypeDecl.env_names
Fld := Libadalang.Implementation.Protected_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4677
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1688;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1688;
            raise;
      end;
   return Property_Result;
end Env_Names_24;
--# end
   --
   --  Primitives for Bare_Task_Type_Decl
   --
      procedure Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_25 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7012:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_26 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_27 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Type_Decl
        (Self : Bare_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Task_Type_Decl_F_Discriminants := Task_Type_Decl_F_Discriminants;
            Self.Task_Type_Decl_F_Aspects := Task_Type_Decl_F_Aspects;
            Self.Task_Type_Decl_F_Definition := Task_Type_Decl_F_Definition;
      end Initialize_Fields_For_Task_Type_Decl;
   function Task_Type_Decl_F_Discriminants
     (Node : Bare_Task_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Task_Type_Decl_F_Discriminants;
   end;
   function Task_Type_Decl_F_Definition
     (Node : Bare_Task_Type_Decl) return Bare_Task_Def
   is
   begin
         return Node.Task_Type_Decl_F_Definition;
   end;
--# property-start TaskTypeDecl.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6980'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Is_Task_Type
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Is_Task_Type;
--# end
--# property-start TaskTypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6983'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Base_Interfaces
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_767 with Inline_Always;
            procedure Finalizer_Scope_2601 with Inline_Always;
      Var_Expr_143 : Internal_Entity_Task_Def;
Fld : Internal_Entity_Task_Def;
Fld_1 : Internal_Entity_Parent_List;
Fld_2 : Bare_Parent_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_767 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_767;
            procedure Finalizer_Scope_2601 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2601;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4678 '<FieldAccess for definition at nodes.lkt:6984:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6984'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 4678
Var_Expr_143 := Fld;
if Var_Expr_143 /= No_Entity_Task_Def then
   --# scope-start
--# expr-start 4681 '<Map at nodes.lkt:6984:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6984'
   --# expr-start 4679 '<FieldAccess for interfaces at nodes.lkt:6984:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6984'
      if Var_Expr_143.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Parent_List (Node => Var_Expr_143.Node.Task_Def_F_Interfaces, Info => Var_Expr_143.Info);
--# expr-done 4679
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 4680 '<FieldAccess for name_designated_type at nodes.lkt:6984:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6984'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 4680
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4681
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2601;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_767;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_767;
                     Finalizer_Scope_2601;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Base_Interfaces;
--# end
--# property-start TaskTypeDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6986'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Env_Names
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_768 with Inline_Always;
            procedure Finalizer_Scope_2603 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_768 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_768;
            procedure Finalizer_Scope_2603 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2603;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4682 '<FieldAccess for top_level_env_name at nodes.lkt:6987:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6987'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4682
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 4684 '<ArrayLiteral at nodes.lkt:6987:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6987'
--# expr-start 4683 '<to_symbol at nodes.lkt:6987:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6987'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 4683
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 4684
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2603;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_768;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_768;
                     Finalizer_Scope_2603;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Env_Names;
--# end
--# property-start TaskTypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6990'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Defining_Env
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_769 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_769 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_769;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4685 '<FieldAccess for children_env at nodes.lkt:6990:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6990'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4685
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_769;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_769;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Defining_Env;
--# end
--# property-start TaskTypeDecl.primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6993'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Primitives_Env
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_770 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_770 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_770;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Task_Type_Decl_Task_Type_Decl_P_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Task_Type_Decl,
                                As_Bare_Task_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4686 '<FieldAccess for compute_primitives_env at nodes.lkt:6994:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6994'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Base_Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => True, E_Info => Ent.Info);
--# end
--# expr-done 4686
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_770;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_770;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Task_Type_Decl_P_Primitives_Env;
--# end
--# property-start TaskTypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6997'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Discriminants_List
  (Node : Bare_Task_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_771 with Inline_Always;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_771 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_771;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4688 '<FieldAccess for abstract_formal_params at nodes.lkt:6998:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6998'
--# expr-start 4687 '<FieldAccess for discriminants at nodes.lkt:6998:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:6998'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Task_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 4687
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4688
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_771;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_771;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Discriminants_List;
--# end
--# property-start TaskTypeDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7000'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TaskTypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7003'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Equation
  (Node : Bare_Task_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_773 with Inline_Always;
            procedure Finalizer_Scope_2604 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Task_Def;
Fld : Internal_Entity_Task_Def;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_773 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_773;
            procedure Finalizer_Scope_2604 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2604;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4689 '<FieldAccess for definition at nodes.lkt:7003:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7003'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 4689
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Task_Def then
   --# scope-start
   --# bind d Var_Expr_0
--# expr-start 4690 '<FieldAccess for xref_equation at nodes.lkt:7004:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7004'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4690
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2604;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7004:47") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_773;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_773;
                     Finalizer_Scope_2604;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Xref_Equation;
--# end
--# property-start TaskTypeDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7007'
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Declarative_Parts
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_774 with Inline_Always;
            procedure Finalizer_Scope_2605 with Inline_Always;
            procedure Finalizer_Scope_2606 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Task_Def;
Var_Expr_1 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Task_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Array_Lit_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result_1 : Internal_Entity_Declarative_Part_Array_Access;
Result_Var_1 : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_774 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_774;
            procedure Finalizer_Scope_2605 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2605;
            procedure Finalizer_Scope_2606 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2606;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4691 '<FieldAccess for definition at nodes.lkt:7007:63>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7007'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 4691
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Task_Def then
   --# scope-start
   --# bind tdef Var_Expr_0
--# expr-start 4694 '<ArrayLiteral at nodes.lkt:7008:19>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
--# expr-start 4693 '<Cast to DeclarativePart at nodes.lkt:7008:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
--# expr-start 4692 '<FieldAccess for public_part at nodes.lkt:7008:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Var_Expr_0.Node.Task_Def_F_Public_Part, Info => Var_Expr_0.Info);
--# expr-done 4692
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 4693
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result)); 
--# expr-done 4694
--# expr-start 4696 '<Cast to DeclarativePart at nodes.lkt:7008:60>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
--# expr-start 4695 '<FieldAccess for private_part at nodes.lkt:7008:60>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Var_Expr_0.Node.Task_Def_F_Private_Part, Info => Var_Expr_0.Info);
--# expr-done 4695
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 4696
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 4697 '<ArrayLiteral at nodes.lkt:7008:109>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7008'
Array_Lit_1 := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Var_Expr_1)); 
--# expr-done 4697
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2606;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Array_Lit, Result_Var); 
Scope_Result_1 := Concat_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2605;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_774;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_774;
                     Finalizer_Scope_2605;
                     Finalizer_Scope_2606;
            raise;
      end;
   return Property_Result;
end Task_Type_Decl_P_Declarative_Parts;
--# end
--# property-start TaskTypeDecl._env_mappings_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7012'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_25
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
     Ent : Internal_Entity_Task_Type_Decl :=
       Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1689 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Task_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1689 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1689;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4700 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4698 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4698
--# expr-start 4699 '<FieldAccess for name_symbol at nodes.lkt:7012:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7012'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4699
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4700
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1689;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1689;
            raise;
      end;
   return Property_Result;
end Env_Mappings_25;
--# end
--# property-start TaskTypeDecl._env_trans_parent_26 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_26
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_26;
--# end
--# property-start TaskTypeDecl._env_names_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7013'
pragma Warnings (Off, "is not referenced");
 function Env_Names_27
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl  := Bare_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1691 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1691 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1691;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4701 '<FieldAccess for env_names at nodes.lkt:7013:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7013'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskTypeDecl.env_names
Fld := Libadalang.Implementation.Task_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4701
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1691;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1691;
            raise;
      end;
   return Property_Result;
end Env_Names_27;
--# end
   --
   --  Primitives for Bare_Single_Task_Type_Decl
   --
      procedure Single_Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_28 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Type_Decl
        (Self : Bare_Single_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Task_Type_Decl
              (Self, Base_Type_Decl_F_Name, Task_Type_Decl_F_Discriminants, Task_Type_Decl_F_Aspects, Task_Type_Decl_F_Definition);
      end Initialize_Fields_For_Single_Task_Type_Decl;
--# property-start SingleTaskTypeDecl._env_trans_parent_28 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_28
  (Node : Bare_Single_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Type_Decl  := Bare_Single_Task_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_28;
--# end
   --
   --  Primitives for Bare_Type_Decl
   --
      procedure Type_Decl_Pre_Env_Actions
        (Self            : Bare_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_29 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7354:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_30 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7355:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_31 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Type_Decl_Post_Env_Actions
        (Self : Bare_Type_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_32 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7366:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Libadalang.Implementation.Ref_Cond_35 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_33 (Node => 
Self);
            Env : Lexical_Env :=
              Libadalang.Implementation.Env_Dest_34 (Node => 
Self);
         begin
               if Is_Foreign_Strict (Env, Self) then
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "unsound foreign environment in RefEnvs ("
                     & "nodes.lkt:7372:9)");
               end if;
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Type_Decl_P_Refined_Parent_Primitives_Env'Access,
               Transitive,
               (Inherited_Primitives => True, others => False),
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Type_Decl
        (Self : Bare_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Type_Decl_F_Discriminants := Type_Decl_F_Discriminants;
            Self.Type_Decl_F_Type_Def := Type_Decl_F_Type_Def;
      end Initialize_Fields_For_Type_Decl;
   function Type_Decl_F_Discriminants
     (Node : Bare_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Type_Decl_F_Discriminants;
   end;
   function Type_Decl_F_Type_Def
     (Node : Bare_Type_Decl) return Bare_Type_Def
   is
   begin
         return Node.Type_Decl_F_Type_Def;
   end;
--# property-start TypeDecl.is_iterable_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7036'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Sym : Symbol_Type;
Fld_1 : Internal_Aspect;
Fld_2 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Sym_1 : Symbol_Type;
Fld_3 : Internal_Aspect;
Fld_4 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4702 '<FieldAccess for is_array at nodes.lkt:7037:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4702
if Fld then
   If_Result := True;
else
   --# expr-start 4707 '<Not at nodes.lkt:7037:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
--# expr-start 4706 '<IsNull at nodes.lkt:7037:32>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
--# expr-start 4705 '<FieldAccess for value at nodes.lkt:7037:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
--# expr-start 4704 '<FieldAccess for get_aspect at nodes.lkt:7037:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4703 '<Symbol Iterator_Element at nodes.lkt:7037:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7037'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 4703
--# property-call-start BasicDecl.get_aspect
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4704
Fld_2 := Fld_1.Value;
--# expr-done 4705
Is_Null := Fld_2.Node = null; 
--# expr-done 4706
Not_Val := not (Is_Null); 
--# expr-done 4707
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 4712 '<Not at nodes.lkt:7041:13>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7041'
--# expr-start 4711 '<IsNull at nodes.lkt:7041:17>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7041'
--# expr-start 4710 '<FieldAccess for value at nodes.lkt:7041:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7041'
--# expr-start 4709 '<FieldAccess for get_aspect at nodes.lkt:7041:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7041'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4708 '<Symbol Iterable at nodes.lkt:7041:33>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7041'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 4708
--# property-call-start BasicDecl.get_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4709
Fld_4 := Fld_3.Value;
--# expr-done 4710
Is_Null_1 := Fld_4.Node = null; 
--# expr-done 4711
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4712
   If_Result_1 := Not_Val_1;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start TypeDecl.iterable_comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7045'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_776 with Inline_Always;
            procedure Finalizer_Scope_2607 with Inline_Always;
      Local_Ie : Internal_Entity_Expr;
Local_It : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Bound_Imprecise_Fallback : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Single_Actual_Array_Access;
Fld_9 : Bare_Base_Id;
Sym_2 : Symbol_Type;
Fld_10 : Boolean;
Find_Result : Internal_Single_Actual;
Fld_11 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_13 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_776 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_776;
            procedure Finalizer_Scope_2607 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_2607;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4746 '<Let at nodes.lkt:7046:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7046'
--# scope-start
--# expr-start 4715 '<FieldAccess for value at nodes.lkt:7046:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7046'
--# expr-start 4714 '<FieldAccess for get_aspect at nodes.lkt:7046:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7046'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4713 '<Symbol Iterator_Element at nodes.lkt:7046:34>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7046'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 4713
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4714
Fld_1 := Fld.Value;
--# expr-done 4715
Local_Ie := Fld_1; 
--# bind ie Local_Ie
--# expr-start 4745 '<Let at nodes.lkt:7047:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7047'
--# scope-start
--# expr-start 4718 '<FieldAccess for value at nodes.lkt:7047:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7047'
--# expr-start 4717 '<FieldAccess for get_aspect at nodes.lkt:7047:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7047'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4716 '<Symbol Iterable at nodes.lkt:7047:34>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7047'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 4716
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4717
Fld_3 := Fld_2.Value;
--# expr-done 4718
Local_It := Fld_3; 
--# bind it Local_It
--# expr-start 4744 '<bind at nodes.lkt:7050:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7050'
Bound_Imprecise_Fallback := False; 
--# expr-start 4743 '<If at nodes.lkt:7052:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7052'
--# expr-start 4719 '<FieldAccess for is_array at nodes.lkt:7052:16>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7052'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4719
if Fld_15 then
   --# expr-start 4720 '<FieldAccess for comp_type at nodes.lkt:7052:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7052'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4720
   If_Result_2 := Fld_4;
else
   --# expr-start 4742 '<If at nodes.lkt:7052:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7052'
--# expr-start 4722 '<Not at nodes.lkt:7053:18>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7053'
--# expr-start 4721 '<IsNull at nodes.lkt:7053:22>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7053'
Is_Null_1 := Local_Ie.Node = null; 
--# expr-done 4721
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4722
if Not_Val_1 then
--# expr-start 4723 '<Cast to Name at nodes.lkt:7053:38>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7053'
Cast_Expr := Local_Ie; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 4723
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 4726 '<bind at nodes.lkt:7055:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7055'
--# expr-start 4724 '<FieldAccess for node_env at nodes.lkt:7055:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7055'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 4724
Bound_Env := Fld_5; Inc_Ref (Bound_Env);
--# expr-start 4725 '<FieldAccess for designated_type_impl at nodes.lkt:7057:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7057'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Var_Expr_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4725
Dyn_Var_Bind_Result := Fld_6; 
--# expr-done 4726
Scope_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_2607;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var;
else
   --# expr-start 4741 '<If at nodes.lkt:7052:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7052'
--# expr-start 4728 '<Not at nodes.lkt:7060:18>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4727 '<IsNull at nodes.lkt:7060:22>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
Is_Null := Local_It.Node = null; 
--# expr-done 4727
Not_Val := not (Is_Null); 
--# expr-done 4728
if Not_Val then
   --# expr-start 4740 '<FieldAccess for expr_type at nodes.lkt:7060:38>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4739 '<FieldAccess for referenced_decl at nodes.lkt:7060:38>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4738 '<Cast to Name at nodes.lkt:7060:38>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4737 '<FieldAccess for expr at nodes.lkt:7060:38>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4736 '<FieldAccess for assoc at nodes.lkt:7060:38>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4735 '<Find at nodes.lkt:7060:38>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
   --# expr-start 4731 '<FieldAccess for unpacked_params at nodes.lkt:7060:38>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4730 '<FieldAccess for assocs at nodes.lkt:7060:38>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4729 '<Cast to Aggregate at nodes.lkt:7060:38>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
Cast_Expr_1 := Local_It; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 4729
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 4730
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_8 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 4731
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_8;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_0
      --# expr-start 4734 '<FieldAccess for name_is at nodes.lkt:7060:93>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
--# expr-start 4732 '<FieldAccess for name at nodes.lkt:7060:93>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
Fld_9 := Item_0.Name;
--# expr-done 4732
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4733 '<Symbol Element at nodes.lkt:7060:109>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7060'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Element); 
--# expr-done 4733
--# property-call-start Name.name_is
Fld_10 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_9, Sym => Sym_2);
--# end
--# expr-done 4734
      if Fld_10 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld_10;
         end loop;
      end;
--# expr-done 4735
Fld_11 := Find_Result.Assoc;
--# expr-done 4736
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 4737
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4738
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_13 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4739
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_13.Node, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 4740
   If_Result := Fld_14;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
--# expr-done 4741
   If_Result_1 := If_Result;
end if;
--# expr-done 4742
   If_Result_2 := If_Result_1;
end if;
--# expr-done 4743
Dyn_Var_Bind_Result_1 := If_Result_2; 
--# expr-done 4744
Let_Result := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 4745
Let_Result_1 := Let_Result; 
--# end
--# expr-done 4746
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_776;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_776;
                     Finalizer_Scope_2607;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start TypeDecl.iterable_cursor_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7068'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Cursor_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2609 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Single_Actual_Array_Access;
Fld_4 : Bare_Base_Id;
Sym_1 : Symbol_Type;
Fld_5 : Boolean;
Find_Result : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2609 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2609;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4749 '<FieldAccess for value at nodes.lkt:7069:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7069'
--# expr-start 4748 '<FieldAccess for get_aspect at nodes.lkt:7069:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7069'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4747 '<Symbol Iterable at nodes.lkt:7069:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7069'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 4747
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4748
Fld_1 := Fld.Value;
--# expr-done 4749
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind it Var_Expr_0
--# expr-start 4761 '<FieldAccess for expr_type at nodes.lkt:7070:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4760 '<FieldAccess for referenced_decl at nodes.lkt:7070:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4759 '<Cast to Name at nodes.lkt:7070:21>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4758 '<FieldAccess for expr at nodes.lkt:7070:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4757 '<FieldAccess for assoc at nodes.lkt:7070:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4756 '<Find at nodes.lkt:7070:21>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
   --# expr-start 4752 '<FieldAccess for unpacked_params at nodes.lkt:7070:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4751 '<FieldAccess for assocs at nodes.lkt:7070:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4750 '<Cast to Aggregate at nodes.lkt:7070:21>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 4750
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 4751
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.unpacked_params
Fld_3 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4752
      Find_Result := No_Single_Actual;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_3;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_1
      --# expr-start 4755 '<FieldAccess for name_is at nodes.lkt:7070:76>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
--# expr-start 4753 '<FieldAccess for name at nodes.lkt:7070:76>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
Fld_4 := Item_1.Name;
--# expr-done 4753
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4754 '<Symbol First at nodes.lkt:7070:92>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7070'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 4754
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_4, Sym => Sym_1);
--# end
--# expr-done 4755
      if Fld_5 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_5;
         end loop;
      end;
--# expr-done 4756
Fld_6 := Find_Result.Assoc;
--# expr-done 4757
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 4758
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4759
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_1.Node, Imprecise_Fallback => False, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4760
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4761
Scope_Result := Fld_9;
   --# end
      Finalizer_Scope_2609;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2609;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Iterable_Cursor_Type;
--# end
--# property-start TypeDecl.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7073'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discrete_Range
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4763 '<FieldAccess for discrete_range at nodes.lkt:7073:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7073'
--# expr-start 4762 '<FieldAccess for type_def at nodes.lkt:7073:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7073'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4762
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.discrete_range'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4763
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Discrete_Range;
--# end
--# property-start TypeDecl.discriminants_list 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7076'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discriminants_List
  (Node : Bare_Type_Decl
      ; Stop_Recurse_At : Internal_Entity_Base_Type_Decl
         := No_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_recurse_at Stop_Recurse_At
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_779 with Inline_Always;
            procedure Finalizer_Scope_2611 with Inline_Always;
      Local_Base_Type : Internal_Entity_Base_Type_Decl;
Local_Self_Discs : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_0 : Internal_Entity_Discriminant_Part;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Discriminant_Part;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Boolean;
If_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_779 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Self_Discs);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_779;
            procedure Finalizer_Scope_2611 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2611;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4783 '<Let at nodes.lkt:7078:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7078'
--# scope-start
--# expr-start 4764 '<FieldAccess for base_type at nodes.lkt:7078:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7078'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4764
Local_Base_Type := Fld; 
--# bind base_type Local_Base_Type
--# expr-start 4782 '<Let at nodes.lkt:7079:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7079'
--# scope-start
--# expr-start 4765 '<FieldAccess for discriminants at nodes.lkt:7079:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7079'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 4765
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Discriminant_Part then
   --# scope-start
   --# bind d Var_Expr_0
--# expr-start 4766 '<FieldAccess for abstract_formal_params at nodes.lkt:7079:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7079'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4766
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2611;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Self_Discs := Result_Var; Inc_Ref (Local_Self_Discs);
--# bind self_discs Local_Self_Discs
--# expr-start 4781 '<If at nodes.lkt:7081:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
--# expr-start 4767 '<FieldAccess for is_access_type at nodes.lkt:7081:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4767
if Fld_8 then
   --# expr-start 4769 '<FieldAccess for discriminants_list at nodes.lkt:7081:39>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
--# expr-start 4768 '<FieldAccess for accessed_type at nodes.lkt:7081:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4768
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_3.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 4769
   If_Result_4 := Fld_4;
else
   --# expr-start 4780 '<If at nodes.lkt:7081:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
--# expr-start 4770 '<length at nodes.lkt:7082:14>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7082'
Len := Length (Local_Self_Discs); 
--# expr-done 4770
Comp_Result := Len > 0; 
if Comp_Result then
   If_Result_3 := Local_Self_Discs;
else
   --# expr-start 4779 '<If at nodes.lkt:7081:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
--# expr-start 4772 '<Not at nodes.lkt:7083:14>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7083'
--# expr-start 4771 '<IsNull at nodes.lkt:7083:18>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7083'
Is_Null_1 := Local_Base_Type.Node = null; 
--# expr-done 4771
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4772
if Not_Val_1 then
   --# expr-start 4773 '<FieldAccess for matching_type at nodes.lkt:7083:40>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7083'
      if Local_Base_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Local_Base_Type.Node, Expected_Type => Stop_Recurse_At, Origin => Origin, E_Info => Local_Base_Type.Info);
--# end
--# expr-done 4773
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := Local_Self_Discs;
else
   --# expr-start 4778 '<If at nodes.lkt:7081:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7081'
--# expr-start 4775 '<Not at nodes.lkt:7084:14>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7084'
--# expr-start 4774 '<IsNull at nodes.lkt:7084:18>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7084'
Is_Null := Local_Base_Type.Node = null; 
--# expr-done 4774
Not_Val := not (Is_Null); 
--# expr-done 4775
if Not_Val then
   --# expr-start 4777 '<FieldAccess for discriminants_list at nodes.lkt:7084:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7084'
--# expr-start 4776 '<FieldAccess for base_type at nodes.lkt:7084:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4776
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_5.Node, Stop_Recurse_At => Stop_Recurse_At, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4777
   If_Result := Fld_6;
else
   If_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 4778
   If_Result_2 := If_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4779
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 4780
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 4781
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4782
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4783
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_779;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_779;
                     Finalizer_Scope_2611;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Discriminants_List;
--# end
--# property-start TypeDecl.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7089'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Ndims
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4785 '<FieldAccess for array_ndims at nodes.lkt:7089:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7089'
--# expr-start 4784 '<FieldAccess for type_def at nodes.lkt:7089:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7089'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4784
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4785
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Array_Ndims;
--# end
--# property-start TypeDecl.is_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7092'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Real_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4787 '<FieldAccess for is_real_type at nodes.lkt:7092:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7092'
--# expr-start 4786 '<FieldAccess for type_def at nodes.lkt:7092:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7092'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4786
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.is_real_type
Fld_1 := Libadalang.Implementation.Type_Def_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4787
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Real_Type;
--# end
--# property-start TypeDecl.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7095'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Float_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4789 '<FieldAccess for is_float_type at nodes.lkt:7095:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7095'
--# expr-start 4788 '<FieldAccess for type_def at nodes.lkt:7095:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7095'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4788
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4789
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Float_Type;
--# end
--# property-start TypeDecl.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7098'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4791 '<FieldAccess for is_fixed_point at nodes.lkt:7098:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7098'
--# expr-start 4790 '<FieldAccess for type_def at nodes.lkt:7098:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7098'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4790
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4791
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start TypeDecl.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7101'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Int_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4793 '<FieldAccess for is_int_type at nodes.lkt:7101:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7101'
--# expr-start 4792 '<FieldAccess for type_def at nodes.lkt:7101:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7101'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4792
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4793
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Int_Type;
--# end
--# property-start TypeDecl.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7104'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Access_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4796 '<FieldAccess for is_access_type at nodes.lkt:7104:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7104'
--# expr-start 4795 '<FieldAccess for type_def at nodes.lkt:7104:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7104'
--# expr-start 4794 '<as_bare_entity at nodes.lkt:7104:34>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7104'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4794
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 4795
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Access_Type (Node => Fld.Node, Origin => Origin);
--# end
--# expr-done 4796
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Access_Type;
--# end
--# property-start TypeDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7107'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Static_Decl
  (Node : Bare_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4799 '<FieldAccess for is_static at nodes.lkt:7107:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7107'
--# expr-start 4798 '<FieldAccess for type_def at nodes.lkt:7107:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7107'
--# expr-start 4797 '<as_bare_entity at nodes.lkt:7107:34>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7107'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4797
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 4798
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4799
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Static_Decl;
--# end
--# property-start TypeDecl.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7110'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Accessed_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_787 with Inline_Always;
      Local_Imp_Deref : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Bound_Include_Ud_Indexing : Boolean;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Lexical_Env := Empty_Env;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Dyn_Var_Bind_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_787 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_787;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4814 '<Let at nodes.lkt:7111:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7111'
--# scope-start
--# expr-start 4800 '<FieldAccess for get_imp_deref at nodes.lkt:7111:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7111'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4800
Local_Imp_Deref := Fld; 
--# bind imp_deref Local_Imp_Deref
--# expr-start 4813 '<If at nodes.lkt:7113:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7113'
--# expr-start 4801 '<IsNull at nodes.lkt:7113:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7113'
Is_Null := Local_Imp_Deref.Node = null; 
--# expr-done 4801
if Is_Null then
   --# expr-start 4803 '<FieldAccess for accessed_type at nodes.lkt:7113:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7113'
--# expr-start 4802 '<FieldAccess for type_def at nodes.lkt:7113:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7113'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4802
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4803
   If_Result := Fld_2;
else
   --# expr-start 4812 '<FieldAccess for accessed_type at nodes.lkt:7116:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7116'
--# expr-start 4811 '<FieldAccess for formal_type at nodes.lkt:7116:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7116'
--# expr-start 4810 '<Cast to BaseFormalParamDecl at nodes.lkt:7116:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7116'
--# expr-start 4809 '<bind at nodes.lkt:7117:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7117'
Bound_Include_Ud_Indexing := False; 
--# expr-start 4808 '<EnvGet at nodes.lkt:7119:17>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7119'
--# expr-start 4805 '<FieldAccess for defining_env at nodes.lkt:7119:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7119'
--# expr-start 4804 '<FieldAccess for type_def at nodes.lkt:7119:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7119'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4804
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.defining_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 4805
--# expr-start 4807 '<FieldAccess for name_symbol at nodes.lkt:7120:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7120'
--# expr-start 4806 '<Cast to Name at nodes.lkt:7120:21>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7120'
Cast_Expr := Local_Imp_Deref; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 4806
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result.Node);
--# end
--# expr-done 4807
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Thin (Fld_5), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4808
Dyn_Var_Bind_Result := Env_Get_Result; 
--# expr-done 4809
   if Dyn_Var_Bind_Result.Node = null
      or else Dyn_Var_Bind_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4810
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Cast_Result_1.Node, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4811
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 4812
   If_Result := Fld_7;
end if;
--# expr-done 4813
Let_Result := If_Result; 
--# end
--# expr-done 4814
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_787;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_787;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Accessed_Type;
--# end
--# property-start TypeDecl.access_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7132'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Access_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Match_0 : Internal_Entity_Access_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Let_Result : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Access_Def;
Scope_Result_1 : Internal_Entity_Access_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Access_Def;
Scope_Result_2 : Internal_Entity_Access_Def;
Match_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4818 '<Match at nodes.lkt:7132:43>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7132'
--# expr-start 4815 '<FieldAccess for type_def at nodes.lkt:7132:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7132'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4815
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ad Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 4817 '<FieldAccess for access_def at nodes.lkt:7134:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7134'
--# expr-start 4816 '<FieldAccess for base_type at nodes.lkt:7134:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7134'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4816
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4817
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Access_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4818
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Access_Def;
--# end
--# property-start TypeDecl.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7139'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4820 '<FieldAccess for is_tagged_type at nodes.lkt:7139:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7139'
--# expr-start 4819 '<FieldAccess for type_def at nodes.lkt:7139:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7139'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4819
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4820
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start TypeDecl.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7141'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Task_Type
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4822 '<FieldAccess for is_task_type at nodes.lkt:7141:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7141'
--# expr-start 4821 '<FieldAccess for type_def at nodes.lkt:7141:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7141'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4821
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_task_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Task_Type (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 4822
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Task_Type;
--# end
--# property-start TypeDecl.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7144'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4824 '<FieldAccess for base_type at nodes.lkt:7144:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7144'
--# expr-start 4823 '<FieldAccess for type_def at nodes.lkt:7144:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7144'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4823
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4824
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Base_Type;
--# end
--# property-start TypeDecl.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7147'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Interfaces
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_792 with Inline_Always;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_792 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_792;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4826 '<FieldAccess for base_interfaces at nodes.lkt:7148:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7148'
--# expr-start 4825 '<FieldAccess for type_def at nodes.lkt:7148:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7148'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4825
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4826
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_792;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_792;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Base_Interfaces;
--# end
--# property-start TypeDecl.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7151'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Char_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4828 '<FieldAccess for is_char_type at nodes.lkt:7151:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7151'
--# expr-start 4827 '<FieldAccess for type_def at nodes.lkt:7151:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4827
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4828
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Char_Type;
--# end
--# property-start TypeDecl.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7154'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Enum_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4830 '<FieldAccess for is_enum_type at nodes.lkt:7154:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7154'
--# expr-start 4829 '<FieldAccess for type_def at nodes.lkt:7154:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7154'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4829
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4830
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Enum_Type;
--# end
--# property-start TypeDecl.is_private 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7156'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Private
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_0 : Bare_Derived_Type_Def;
Fld : Bare_Type_Def;
Is_A : Boolean;
Fld_1 : Bare_Type_Def;
Cast_Result : Bare_Derived_Type_Def;
Fld_2 : Bare_With_Private;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4832 '<IsA PrivateTypeDef at nodes.lkt:7157:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
--# expr-start 4831 '<FieldAccess for type_def at nodes.lkt:7157:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4831
Is_A := Fld /= null 
and then Fld.Kind in Ada_Private_Type_Def_Range; 
--# expr-done 4832
if Is_A then
   If_Result := True;
else
--# expr-start 4834 '<Cast to DerivedTypeDef at nodes.lkt:7157:44>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
--# expr-start 4833 '<FieldAccess for type_def at nodes.lkt:7157:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Type_Decl_F_Type_Def;
--# expr-done 4833
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4834
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dtd Var_Expr_0
--# expr-start 4836 '<FieldAccess for as_bool at nodes.lkt:7157:89>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
--# expr-start 4835 '<FieldAccess for has_with_private at nodes.lkt:7157:89>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7157'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_0.Derived_Type_Def_F_Has_With_Private;
--# expr-done 4835
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 4836
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Private;
--# end
--# property-start TypeDecl.is_derived_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7159'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Derived_Tagged_Type
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Type_Def;
Is_A : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 4838 '<FieldAccess for is_tagged_type at nodes.lkt:7160:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7160'
--# expr-start 4837 '<FieldAccess for type_def at nodes.lkt:7160:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7160'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4837
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 4838
if Fld_1 then
   --# expr-start 4840 '<IsA DerivedTypeDef, InterfaceTypeDef at nodes.lkt:7160:44>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7160'
--# expr-start 4839 '<FieldAccess for type_def at nodes.lkt:7160:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7160'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4839
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4840
   If_Result := Is_A;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Derived_Tagged_Type;
--# end
--# property-start TypeDecl.array_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7163'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Array_Type_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Array_Type_Def;
Scope_Result_1 : Internal_Entity_Array_Type_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Array_Type_Def;
Scope_Result_2 : Internal_Entity_Array_Type_Def;
Match_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4844 '<Match at nodes.lkt:7163:45>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7163'
--# expr-start 4841 '<FieldAccess for type_def at nodes.lkt:7163:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7163'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4841
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 4843 '<FieldAccess for array_def at nodes.lkt:7165:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7165'
--# expr-start 4842 '<FieldAccess for base_type at nodes.lkt:7165:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7165'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4842
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4843
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Array_Type_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4844
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Array_Def;
--# end
--# property-start TypeDecl.root_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7170'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Root_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Def;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4848 '<Match at nodes.lkt:7170:45>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7170'
--# expr-start 4845 '<FieldAccess for type_def at nodes.lkt:7170:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7170'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4845
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 4847 '<FieldAccess for root_type at nodes.lkt:7171:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7171'
--# expr-start 4846 '<FieldAccess for base_type at nodes.lkt:7171:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7171'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4846
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4847
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4848
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Root_Type;
--# end
--# property-start TypeDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7176'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Defining_Env
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_799 with Inline_Always;
            procedure Finalizer_Scope_2621 with Inline_Always;
      Local_Imp_Deref : Internal_Entity_Expr;
Local_Self_Env : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Bound_Include_Ud_Indexing : Boolean;
Fld_5 : Internal_Entity_Basic_Decl_Array_Access;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Array_Lit_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_8 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_799 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Local_Self_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_799;
            procedure Finalizer_Scope_2621 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2621;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4869 '<Let at nodes.lkt:7177:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7177'
--# scope-start
--# expr-start 4849 '<FieldAccess for get_imp_deref at nodes.lkt:7177:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7177'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4849
Local_Imp_Deref := Fld; 
--# bind imp_deref Local_Imp_Deref
--# expr-start 4868 '<Let at nodes.lkt:7180:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7180'
--# scope-start
--# expr-start 4851 '<FieldAccess for defining_env at nodes.lkt:7180:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7180'
--# expr-start 4850 '<FieldAccess for type_def at nodes.lkt:7180:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7180'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4850
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.defining_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_1.Info);
--# end
--# expr-done 4851
Local_Self_Env := Fld_2; Inc_Ref (Local_Self_Env);
--# bind self_env Local_Self_Env
--# expr-start 4867 '<If at nodes.lkt:7182:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
--# expr-start 4853 '<Not at nodes.lkt:7182:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
--# expr-start 4852 '<IsNull at nodes.lkt:7182:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
Is_Null := Local_Imp_Deref.Node = null; 
--# expr-done 4852
Not_Val := not (Is_Null); 
--# expr-done 4853
if Not_Val then
   --# expr-start 4857 '<env_group at nodes.lkt:7182:39>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
--# expr-start 4856 '<ArrayLiteral at nodes.lkt:7182:39>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
--# expr-start 4855 '<FieldAccess for defining_env at nodes.lkt:7182:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
--# expr-start 4854 '<FieldAccess for accessed_type at nodes.lkt:7182:50>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4854
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_3.Info);
--# end
--# expr-done 4855
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Local_Self_Env, 2 => Fld_4)); 
--# expr-done 4856
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4857
   If_Result_2 := Group_Env;
else
   --# expr-start 4866 '<If at nodes.lkt:7182:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7182'
if Include_Ud_Indexing then
   --# expr-start 4858 '<FieldAccess for has_ud_indexing at nodes.lkt:7183:38>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4858
   If_Result := Fld_8;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4865 '<bind at nodes.lkt:7184:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7184'
Bound_Include_Ud_Indexing := False; 
--# expr-start 4864 '<env_group at nodes.lkt:7186:13>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7186'
--# expr-start 4862 '<Map at nodes.lkt:7187:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7187'
   --# expr-start 4859 '<FieldAccess for constant_indexing_fns at nodes.lkt:7188:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7188'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4859
--# expr-start 4860 '<FieldAccess for variable_indexing_fns at nodes.lkt:7188:52>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7188'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4860
Concat_Result := Concat (Fld_5, Fld_6); 
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_0
      --# expr-start 4861 '<FieldAccess for defining_env at nodes.lkt:7189:31>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7189'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_0.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Item_0.Info);
--# end
--# expr-done 4861
         declare
            Item_To_Append : constant Lexical_Env := Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2621;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4862
--# expr-start 4863 '<ArrayLiteral at nodes.lkt:7189:52>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7189'
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Local_Self_Env)); 
--# expr-done 4863
Concat_Result_1 := Concat (Map_Result, Array_Lit_1); 
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 4864
Dyn_Var_Bind_Result := Group_Env_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4865
   If_Result_1 := Dyn_Var_Bind_Result;
else
   If_Result_1 := Local_Self_Env;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4866
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4867
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4868
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4869
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_799;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_799;
                     Finalizer_Scope_2621;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Defining_Env;
--# end
--# property-start TypeDecl.primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7196'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_800 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_800 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_800;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4870 '<FieldAccess for compute_primitives_env at nodes.lkt:7197:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7197'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Base_Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => True, E_Info => Ent.Info);
--# end
--# expr-done 4870
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_800;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_800;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Type_Decl_P_Primitives_Env;
--# end
--# property-start TypeDecl.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7206'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Predefined_Operators
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_801 with Inline_Always;
      Fld : Bare_Type_Def;
Cast_Result : Bare_Real_Type_Def;
Fld_1 : Internal_Env_Assoc_Array_Access;
Fld_2 : Bare_Type_Def;
Fld_3 : Internal_Env_Assoc_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_5 : Internal_Entity;
Is_Equiv_1 : Boolean;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_6 : Internal_Entity;
Is_Equiv_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_801 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_801;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 4881 '<If at nodes.lkt:7207:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7207'
Cast_Expr_2 := Ent; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 4871 '<FieldAccess for universal_int_type at nodes.lkt:7207:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7207'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 4871
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv_2 := Equivalent (Cast_Result_6, Cast_Result_3); 
if Is_Equiv_2 then
   If_Result_1 := True;
else
Cast_Expr_1 := Ent; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 4872 '<FieldAccess for universal_real_type at nodes.lkt:7207:48>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7207'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 4872
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Is_Equiv_1 := Equivalent (Cast_Result_5, Cast_Result_4); 
   If_Result_1 := Is_Equiv_1;
end if;
if If_Result_1 then
   If_Result_2 := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 4880 '<If at nodes.lkt:7207:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7207'
--# expr-start 4874 '<Eq at nodes.lkt:7208:14>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7208'
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 4873 '<FieldAccess for universal_fixed_type at nodes.lkt:7208:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7208'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 4873
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4874
if Is_Equiv then
   --# expr-start 4877 '<FieldAccess for universal_fixed_predefined_operators at nodes.lkt:7208:55>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7208'
--# expr-start 4876 '<Cast to RealTypeDef at nodes.lkt:7208:55>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7208'
--# expr-start 4875 '<FieldAccess for type_def at nodes.lkt:7208:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7208'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4875
   if Fld = null
      or else Fld.Kind in Ada_Real_Type_Def
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4876
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start RealTypeDef.universal_fixed_predefined_operators
Fld_1 := Libadalang.Implementation.Real_Type_Def_P_Universal_Fixed_Predefined_Operators (Node => Cast_Result);
--# end
--# expr-done 4877
   If_Result := Fld_1;
else
   --# expr-start 4879 '<FieldAccess for predefined_operators at nodes.lkt:7209:14>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7209'
--# expr-start 4878 '<FieldAccess for type_def at nodes.lkt:7209:14>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7209'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Decl_F_Type_Def;
--# expr-done 4878
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.predefined_operators'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Predefined_Operators (Node => Fld_2);
--# end
--# expr-done 4879
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 4880
   If_Result_2 := If_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4881
         Property_Result := If_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_801;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_801;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Type_Decl_P_Predefined_Operators;
--# end
--# property-start TypeDecl.record_def 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7212'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Record_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Match_0 : Internal_Entity_Record_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr_144 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Record_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Record_Type_Def;
Let_Result : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Base_Record_Def;
Scope_Result_2 : Internal_Entity_Base_Record_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Base_Record_Def;
Scope_Result_3 : Internal_Entity_Base_Record_Def;
Match_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4889 '<Match at nodes.lkt:7212:47>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7212'
--# expr-start 4882 '<FieldAccess for type_def at nodes.lkt:7212:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7212'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4882
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Record_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Record_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind r Match_0
--# expr-start 4883 '<FieldAccess for record_def at nodes.lkt:7213:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7213'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Base_Record_Def (Node => Match_0.Node.Record_Type_Def_F_Record_Def, Info => Match_0.Info);
--# expr-done 4883
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind d Match_1
--# expr-start 4888 '<If at nodes.lkt:7217:35>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7217'
--# expr-start 4884 '<FieldAccess for is_tagged_type at nodes.lkt:7217:38>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7217'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4884
if Fld_5 then
   --# expr-start 4885 '<FieldAccess for record_extension at nodes.lkt:7217:65>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7217'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Match_1.Node.Derived_Type_Def_F_Record_Extension, Info => Match_1.Info);
--# expr-done 4885
   If_Result := Fld_2;
else
--# expr-start 4886 '<FieldAccess for base_type at nodes.lkt:7217:89>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7217'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4886
Var_Expr_144 := Fld_3;
if Var_Expr_144 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4887 '<FieldAccess for record_def at nodes.lkt:7217:89>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7217'
      if Var_Expr_144.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr_144.Node, Origin => Origin, E_Info => Var_Expr_144.Info);
--# end
--# expr-done 4887
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 4888
Let_Result_1 := If_Result; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Record_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4889
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Record_Def;
--# end
--# property-start TypeDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7221'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7224'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Equation
  (Node : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_804 with Inline_Always;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_804 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_804;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4891 '<FieldAccess for sub_equation at nodes.lkt:7225:5>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7225'
--# expr-start 4890 '<FieldAccess for type_def at nodes.lkt:7225:5>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7225'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4890
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 4891
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_804;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_804;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Xref_Equation;
--# end
--# property-start TypeDecl.is_discrete_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7228'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4893 '<FieldAccess for is_discrete_type at nodes.lkt:7228:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7228'
--# expr-start 4892 '<FieldAccess for type_def at nodes.lkt:7228:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7228'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4892
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4893
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start TypeDecl.parent_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7230'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_806 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Type_Def;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_806 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_806;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4898 '<If at nodes.lkt:7231:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7231'
--# expr-start 4895 '<IsA DerivedTypeDef, InterfaceTypeDef at nodes.lkt:7231:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7231'
--# expr-start 4894 '<FieldAccess for type_def at nodes.lkt:7231:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7231'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Decl_F_Type_Def;
--# expr-done 4894
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4895
if Is_A then
   --# expr-start 4896 '<FieldAccess for compute_primitives_env at nodes.lkt:7231:68>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7231'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Base_Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, E_Info => Ent.Info);
--# end
--# expr-done 4896
   If_Result := Fld;
else
   --# expr-start 4897 '<FieldAccess for empty_env at nodes.lkt:7231:121>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7231'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 4897
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 4898
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_806;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_806;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.refined_parent_primitives_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7251'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Refined_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_807 with Inline_Always;
            procedure Finalizer_Scope_2626 with Inline_Always;
      Var_Expr_145 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_807 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_807;
            procedure Finalizer_Scope_2626 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2626;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4901 '<FieldAccess for compute_primitives_env at nodes.lkt:7251:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7251'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4899 '<FieldAccess for previous_part at nodes.lkt:7252:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7252'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4899
Var_Expr_145 := Fld;
if Var_Expr_145 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4900 '<FieldAccess for base_types at nodes.lkt:7252:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7252'
      if Var_Expr_145.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Var_Expr_145.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_145.Info);
--# end
--# expr-done 4900
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2626;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start BaseTypeDecl.compute_primitives_env
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, Stop_At => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 4901
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_807;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_807;
                     Finalizer_Scope_2626;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Refined_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.get_imp_deref 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7255'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Bare_Discriminant_Part;
Is_Null : Boolean;
Fld_3 : Bare_Type_Def;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4910 '<If at nodes.lkt:7263:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
--# expr-start 4903 '<IsNull at nodes.lkt:7263:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
--# expr-start 4902 '<FieldAccess for discriminants at nodes.lkt:7263:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Decl_F_Discriminants;
--# expr-done 4902
Is_Null := Fld_2 = null; 
--# expr-done 4903
if Is_Null then
   --# expr-start 4906 '<Not at nodes.lkt:7263:43>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
--# expr-start 4905 '<IsA DerivedTypeDef at nodes.lkt:7263:47>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
--# expr-start 4904 '<FieldAccess for type_def at nodes.lkt:7263:47>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Decl_F_Type_Def;
--# expr-done 4904
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Derived_Type_Def_Range; 
--# expr-done 4905
Not_Val := not (Is_A); 
--# expr-done 4906
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := No_Entity_Expr;
else
   --# expr-start 4909 '<FieldAccess for value at nodes.lkt:7263:108>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
--# expr-start 4908 '<FieldAccess for get_aspect at nodes.lkt:7263:108>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4907 '<Symbol Implicit_Dereference at nodes.lkt:7263:124>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7263'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Implicit_Dereference); 
--# expr-done 4907
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4908
Fld_1 := Fld.Value;
--# expr-done 4909
   If_Result_1 := Fld_1;
end if;
--# expr-done 4910
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start TypeDecl.has_ud_indexing 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7265'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4915 '<Not at nodes.lkt:7266:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4914 '<IsNull at nodes.lkt:7266:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4913 '<FieldAccess for value at nodes.lkt:7266:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4912 '<FieldAccess for get_aspect at nodes.lkt:7266:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4911 '<Symbol Constant_Indexing at nodes.lkt:7266:29>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 4911
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4912
Fld_1 := Fld.Value;
--# expr-done 4913
Is_Null := Fld_1.Node = null; 
--# expr-done 4914
Not_Val := not (Is_Null); 
--# expr-done 4915
if Not_Val then
   If_Result := True;
else
   --# expr-start 4920 '<Not at nodes.lkt:7266:74>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4919 '<IsNull at nodes.lkt:7266:78>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4918 '<FieldAccess for value at nodes.lkt:7266:78>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
--# expr-start 4917 '<FieldAccess for get_aspect at nodes.lkt:7266:78>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4916 '<Symbol Variable_Indexing at nodes.lkt:7266:94>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7266'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 4916
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4917
Fld_3 := Fld_2.Value;
--# expr-done 4918
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 4919
Not_Val_1 := not (Is_Null_1); 
--# expr-done 4920
   If_Result := Not_Val_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start TypeDecl.indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7273'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Indexing_Fns
  (Node : Bare_Type_Decl
      ; Name : Internal_Entity_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_810 with Inline_Always;
            procedure Finalizer_Scope_2627 with Inline_Always;
            procedure Finalizer_Scope_2629 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Var_Expr_146 : Internal_Entity_Defining_Name;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_810 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
            end Finalizer_Scope_810;
            procedure Finalizer_Scope_2627 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_2627;
            procedure Finalizer_Scope_2629 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2629;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4937 '<unique at nodes.lkt:7273:70>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7273'
--# expr-start 4936 '<Map at nodes.lkt:7273:70>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7273'
Var_Expr_0 := Name;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 4925 '<bind at nodes.lkt:7275:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7275'
--# expr-start 4921 '<FieldAccess for node_env at nodes.lkt:7275:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7275'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4921
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4924 '<bind at nodes.lkt:7276:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7276'
--# expr-start 4922 '<FieldAccess for origin_node at nodes.lkt:7276:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7276'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Ent.Node);
--# end
--# expr-done 4922
Bound_Origin := Fld_1; 
--# expr-start 4923 '<FieldAccess for all_env_els_impl at nodes.lkt:7279:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7279'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Var_Expr_0.Node, Seq => False, Env => Bound_Env, Origin => Bound_Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 4923
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4924
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 4925
Scope_Result := Dyn_Var_Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2627;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
--# expr-start 4927 '<FieldAccess for subp_spec_or_null at nodes.lkt:7282:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7282'
--# expr-start 4926 '<Cast to BasicDecl at nodes.lkt:7282:40>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7282'
Cast_Expr_1 := Item_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4926
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4927
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 4934 '<bind at nodes.lkt:7284:17>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7284'
--# expr-start 4928 '<FieldAccess for node at nodes.lkt:7284:31>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7284'
Fld_4 := Item_1.Node;
--# expr-done 4928
Bound_Origin_1 := Fld_4; 
--# expr-start 4930 '<at at nodes.lkt:7291:17>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7291'
--# expr-start 4929 '<FieldAccess for unpacked_formal_params at nodes.lkt:7291:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7291'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4929
Get_Result := Get (Self, Fld_5, 0, True); 
--# expr-done 4930
Var_Expr_146 := Get_Result;
if Var_Expr_146 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 4933 '<FieldAccess for matching_formal_type at nodes.lkt:7291:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7291'
--# expr-start 4932 '<FieldAccess for formal_type at nodes.lkt:7291:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7291'
--# expr-start 4931 '<FieldAccess for formal_decl at nodes.lkt:7291:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7291'
      if Var_Expr_146.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Var_Expr_146.Node, E_Info => Var_Expr_146.Info);
--# end
--# expr-done 4931
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_6.Node, Origin => Bound_Origin_1, E_Info => Fld_6.Info);
--# end
--# expr-done 4932
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_7.Node, Formal_Type => Cast_Result_2, Origin => Bound_Origin_1, E_Info => Fld_7.Info);
--# end
--# expr-done 4933
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Dyn_Var_Bind_Result_2 := Result_Var_1; 
--# expr-done 4934
Scope_Result_2 := Dyn_Var_Bind_Result_2;
   --# end
      Finalizer_Scope_2629;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
      --# expr-start 4935 '<Cast to BasicDecl at nodes.lkt:7282:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7282'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4935
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4936
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 4937
         Property_Result := Unique_Array;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_810;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_810;
                     Finalizer_Scope_2627;
                     Finalizer_Scope_2629;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Indexing_Fns;
--# end
--# property-start TypeDecl.all_indexing_fns_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7298'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_All_Indexing_Fns_Impl
  (Node : Bare_Type_Decl
      ; Name : Internal_Entity_Name
      ; Root_Type : Internal_Entity_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind root_type Root_Type
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_811 with Inline_Always;
            procedure Finalizer_Scope_2631 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_811 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_811;
            procedure Finalizer_Scope_2631 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2631;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4938 '<FieldAccess for indexing_fns at nodes.lkt:7298:108>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7298'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.indexing_fns
Fld := Libadalang.Implementation.Type_Decl_P_Indexing_Fns (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 4938
--# expr-start 4939 '<FieldAccess for base_type at nodes.lkt:7298:134>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7298'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 4939
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 4943 '<If at nodes.lkt:7301:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7301'
--# expr-start 4940 '<Eq at nodes.lkt:7301:12>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7301'
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Root_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4940
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 4942 '<FieldAccess for all_indexing_fns_impl at nodes.lkt:7301:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7301'
--# expr-start 4941 '<Cast to TypeDecl at nodes.lkt:7301:71>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7301'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4941
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.all_indexing_fns_impl
Fld_2 := Libadalang.Implementation.Type_Decl_P_All_Indexing_Fns_Impl (Node => Cast_Result.Node, Name => Name, Root_Type => Root_Type, E_Info => Cast_Result.Info);
--# end
--# expr-done 4942
   If_Result := Fld_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 4943
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2631;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Fld, Result_Var); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_811;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_811;
                     Finalizer_Scope_2631;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_All_Indexing_Fns_Impl;
--# end
--# property-start TypeDecl.all_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7306'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_All_Indexing_Fns
  (Node : Bare_Type_Decl
      ; Sym : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_812 with Inline_Always;
            procedure Finalizer_Scope_2632 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Local_Fn_Name : Internal_Entity_Name;
Local_Root_Type : Internal_Entity_Type_Decl;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_4 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_812 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_812;
            procedure Finalizer_Scope_2632 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_2632;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 4945 '<FieldAccess for value at nodes.lkt:7306:67>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7306'
--# expr-start 4944 '<FieldAccess for get_aspect at nodes.lkt:7306:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7306'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Previous_Parts_Only => True, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 4944
Fld_1 := Fld.Value;
--# expr-done 4945
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind value Var_Expr_0
--# expr-start 4952 '<Let at nodes.lkt:7311:13>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7311'
--# scope-start
--# expr-start 4946 '<Cast to Name at nodes.lkt:7311:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7311'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4946
Local_Fn_Name := Cast_Result; 
--# bind fn_name Local_Fn_Name
--# expr-start 4951 '<Let at nodes.lkt:7314:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7314'
--# scope-start
--# expr-start 4949 '<Cast to TypeDecl at nodes.lkt:7314:29>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7314'
--# expr-start 4948 '<parent at nodes.lkt:7314:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7314'
--# expr-start 4947 '<parent at nodes.lkt:7314:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7314'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 4947
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4948
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 4949
Local_Root_Type := Cast_Result_1; 
--# bind root_type Local_Root_Type
--# expr-start 4950 '<FieldAccess for all_indexing_fns_impl at nodes.lkt:7316:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7316'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.all_indexing_fns_impl
Fld_4 := Libadalang.Implementation.Type_Decl_P_All_Indexing_Fns_Impl (Node => Ent.Node, Name => Local_Fn_Name, Root_Type => Local_Root_Type, E_Info => Ent.Info);
--# end
--# expr-done 4950
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4951
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4952
Scope_Result := Let_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2632;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_812;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_812;
                     Finalizer_Scope_2632;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_All_Indexing_Fns;
--# end
--# property-start TypeDecl.constant_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7320'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_813 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_813 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_813;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4954 '<FieldAccess for all_indexing_fns at nodes.lkt:7321:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7321'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4953 '<Symbol Constant_Indexing at nodes.lkt:7321:31>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7321'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 4953
--# property-call-start TypeDecl.all_indexing_fns
Fld := Libadalang.Implementation.Type_Decl_P_All_Indexing_Fns (Node => Ent.Node, Sym => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4954
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_813;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_813;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start TypeDecl.variable_indexing_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7323'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_814 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_814 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_814;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4956 '<FieldAccess for all_indexing_fns at nodes.lkt:7324:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7324'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 4955 '<Symbol Variable_Indexing at nodes.lkt:7324:31>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7324'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 4955
--# property-call-start TypeDecl.all_indexing_fns
Fld := Libadalang.Implementation.Type_Decl_P_All_Indexing_Fns (Node => Ent.Node, Sym => Sym, E_Info => Ent.Info);
--# end
--# expr-done 4956
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_814;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_814;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start TypeDecl.user_defined_literal_fns 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7328'
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_User_Defined_Literal_Fns
  (Node : Bare_Type_Decl
      ; Aspect : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind aspect Aspect
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_815 with Inline_Always;
            procedure Finalizer_Scope_2634 with Inline_Always;
      Local_Expected_Spec : Internal_User_Defined_Function_Subp_Spec := No_User_Defined_Function_Subp_Spec;
Local_Expected_Specs : Internal_User_Defined_Function_Subp_Spec_Array_Access;
Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Array_Lit : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Struct : Internal_User_Defined_Function_Subp_Spec := No_User_Defined_Function_Subp_Spec;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Array_Lit_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
New_Struct_1 : Internal_User_Defined_Function_Subp_Spec := No_User_Defined_Function_Subp_Spec;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_1 : Internal_User_Defined_Function_Subp_Spec := No_User_Defined_Function_Subp_Spec;
Array_Lit_2 : Internal_User_Defined_Function_Subp_Spec_Array_Access;
Array_Lit_3 : Internal_User_Defined_Function_Subp_Spec_Array_Access;
Concat_Result : Internal_User_Defined_Function_Subp_Spec_Array_Access;
Fld_4 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Array_Access;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Cast_Expr_4 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Boolean;
Quantifier_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_815 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (New_Struct);
                     Dec_Ref (Local_Expected_Spec);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Array_Lit_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Local_Expected_Specs);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_815;
            procedure Finalizer_Scope_2634 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2634;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4983 '<Let at nodes.lkt:7332:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7332'
--# scope-start
--# expr-start 4963 '<New UserDefinedFunctionSubpSpec at nodes.lkt:7332:29>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7332'
--# expr-start 4962 '<ArrayLiteral at nodes.lkt:7333:31>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
--# expr-start 4961 '<If at nodes.lkt:7333:32>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
--# expr-start 4958 '<Eq at nodes.lkt:7333:35>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
--# expr-start 4957 '<Symbol String_Literal at nodes.lkt:7333:45>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String_Literal); 
--# expr-done 4957
Is_Equal := Aspect = Sym; 
--# expr-done 4958
if Is_Equal then
   --# expr-start 4959 '<FieldAccess for std_wide_wide_string_type at nodes.lkt:7333:68>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_wide_string_type
Fld := Libadalang.Implementation.Ada_Node_P_Std_Wide_Wide_String_Type (Node => Self);
--# end
--# expr-done 4959
   If_Result := Fld;
else
   --# expr-start 4960 '<FieldAccess for std_string_type at nodes.lkt:7333:106>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7333'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 4960
   If_Result := Fld_1;
end if;
--# expr-done 4961
Array_Lit := Create_Internal_Entity_Base_Type_Decl_Array (Internal_Internal_Entity_Base_Type_Decl_Array'(1 => If_Result)); 
--# expr-done 4962
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Inc_Ref (Array_Lit);
New_Struct := (Subp_Params_Types => Array_Lit, Subp_Return_Type => Cast_Result); 
--# expr-done 4963
Local_Expected_Spec := New_Struct; Inc_Ref (Local_Expected_Spec);
--# bind expected_spec Local_Expected_Spec
--# expr-start 4982 '<Let at nodes.lkt:7338:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
--# scope-start
--# expr-start 4971 '<ArrayLiteral at nodes.lkt:7338:30>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
--# expr-start 4970 '<If at nodes.lkt:7338:31>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
--# expr-start 4965 '<Eq at nodes.lkt:7338:34>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
--# expr-start 4964 '<Symbol Real_Literal at nodes.lkt:7338:44>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Real_Literal); 
--# expr-done 4964
Is_Equal_1 := Aspect = Sym_1; 
--# expr-done 4965
if Is_Equal_1 then
   --# expr-start 4969 '<New UserDefinedFunctionSubpSpec at nodes.lkt:7338:65>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7338'
--# expr-start 4968 '<ArrayLiteral at nodes.lkt:7339:31>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7339'
--# expr-start 4966 '<FieldAccess for std_string_type at nodes.lkt:7339:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7339'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 4966
--# expr-start 4967 '<FieldAccess for std_string_type at nodes.lkt:7339:56>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7339'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 4967
Array_Lit_1 := Create_Internal_Entity_Base_Type_Decl_Array (Internal_Internal_Entity_Base_Type_Decl_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 4968
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Inc_Ref (Array_Lit_1);
New_Struct_1 := (Subp_Params_Types => Array_Lit_1, Subp_Return_Type => Cast_Result_1); 
--# expr-done 4969
   If_Result_1 := New_Struct_1;
else
   If_Result_1 := No_User_Defined_Function_Subp_Spec;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4970
Array_Lit_2 := Create_Internal_User_Defined_Function_Subp_Spec_Array (Internal_Internal_User_Defined_Function_Subp_Spec_Array'(1 => If_Result_1)); 
--# expr-done 4971
--# expr-start 4972 '<ArrayLiteral at nodes.lkt:7340:53>' Array_Lit_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7340'
Array_Lit_3 := Create_Internal_User_Defined_Function_Subp_Spec_Array (Internal_Internal_User_Defined_Function_Subp_Spec_Array'(1 => Local_Expected_Spec)); 
--# expr-done 4972
Concat_Result := Concat (Array_Lit_2, Array_Lit_3); 
Local_Expected_Specs := Concat_Result; Inc_Ref (Local_Expected_Specs);
--# bind expected_specs Local_Expected_Specs
--# expr-start 4973 '<FieldAccess for get_aspect_spec_expr at nodes.lkt:7342:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7342'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Aspect, E_Info => Ent.Info);
--# end
--# expr-done 4973
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind a Var_Expr_0
--# expr-start 4981 '<Map at nodes.lkt:7343:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7343'
   --# expr-start 4975 '<FieldAccess for all_env_elements_internal at nodes.lkt:7343:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7343'
--# expr-start 4974 '<Cast to Name at nodes.lkt:7343:20>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7343'
Cast_Expr_2 := Var_Expr_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4974
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_5 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result_2.Node, Seq => False, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4975
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_5;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
--# expr-start 4977 '<FieldAccess for subp_spec_or_null at nodes.lkt:7344:48>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7344'
--# expr-start 4976 '<Cast to BasicDecl at nodes.lkt:7344:48>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7344'
Cast_Expr_4 := Item_1; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 4976
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 4977
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 4979 '<AnyQuantifier at nodes.lkt:7345:29>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7345'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_User_Defined_Function_Subp_Spec_Array_Access := Local_Expected_Specs;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind es Item_3
      --# expr-start 4978 '<FieldAccess for match_expected_user_defined_function at nodes.lkt:7346:33>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7346'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_expected_user_defined_function
Fld_7 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Expected_User_Defined_Function (Node => Var_Expr_2.Node, Fn => Item_3, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4978
      Quantifier_Result := Fld_7;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 4979
Scope_Result := Quantifier_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         if Result_Var then
      --# expr-start 4980 '<Cast to BasicDecl at nodes.lkt:7344:24>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7344'
Cast_Expr_3 := Item_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4980
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 4981
Scope_Result_1 := Map_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2634;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4982
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 4983
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_815;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_815;
                     Finalizer_Scope_2634;
            raise;
      end;
   return Property_Result;
end Type_Decl_P_User_Defined_Literal_Fns;
--# end
--# property-start TypeDecl._env_mappings_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7354'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_29
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1693 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1693 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1693;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4986 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 4984 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 4984
--# expr-start 4985 '<FieldAccess for name_symbol at nodes.lkt:7354:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7354'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4985
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 4986
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1693;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1693;
            raise;
      end;
   return Property_Result;
end Env_Mappings_29;
--# end
--# property-start TypeDecl._env_mappings_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7355'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_30
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1694 with Inline_Always;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1694 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1694;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4988 '<FieldAccess for predefined_operators at nodes.lkt:7355:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7355'
--# expr-start 4987 '<as_entity at nodes.lkt:7355:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7355'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4987
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.predefined_operators
Fld := Libadalang.Implementation.Type_Decl_P_Predefined_Operators (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4988
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1694;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1694;
            raise;
      end;
   return Property_Result;
end Env_Mappings_30;
--# end
--# property-start TypeDecl._env_trans_parent_31 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_31
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_31;
--# end
--# property-start TypeDecl._env_mappings_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7366'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_32
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
     Ent : Internal_Entity_Type_Decl :=
       Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1696 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1696 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1696;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4989 '<FieldAccess for synthetic_object_decl_env_assoc at nodes.lkt:7366:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7366'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.synthetic_object_decl_env_assoc
Fld := Libadalang.Implementation.Base_Type_Decl_F_Synthetic_Object_Decl_Env_Assoc (Node => Ent.Node);
--# end
--# expr-done 4989
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1696;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1696;
            raise;
      end;
   return Property_Result;
end Env_Mappings_32;
--# end
--# property-start TypeDecl._ref_env_nodes_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_33
  (Node : Bare_Type_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1697 with Inline_Always;
      Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1697 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1697;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4991 '<ArrayLiteral at nodes.lkt:7373:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
--# expr-start 4990 '<Cast to AdaNode at nodes.lkt:7373:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 4990
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 4991
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1697;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1697;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_33;
--# end
--# property-start TypeDecl._env_dest_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
pragma Warnings (Off, "is not referenced");
 function Env_Dest_34
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1698 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1698 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1698;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 4992 '<FieldAccess for node_env at nodes.lkt:7373:99>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 4992
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1698;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1698;
            raise;
      end;
   return Property_Result;
end Env_Dest_34;
--# end
--# property-start TypeDecl._ref_cond_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_35
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl  := Bare_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Type_Def;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 4994 '<IsA DerivedTypeDef, InterfaceTypeDef at nodes.lkt:7373:121>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
--# expr-start 4993 '<FieldAccess for type_def at nodes.lkt:7373:121>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7373'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 4993
Is_A := Fld /= null 
and then Fld.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 4994
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_35;
--# end
function Type_Decl_P_Refined_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Type_Decl
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Type_Decl_P_Refined_Parent_Primitives_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Type_Decl
        (Self : Bare_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
      end Initialize_Fields_For_Anonymous_Type_Decl;
--# property-start AnonymousTypeDecl.access_def_matches 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7391'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Access_Def_Matches
  (Node : Bare_Anonymous_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl  := Bare_Anonymous_Type_Decl (Node);
     Ent : Internal_Entity_Anonymous_Type_Decl :=
       Internal_Entity_Anonymous_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind other Other
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Self_Subp_Access : Internal_Entity_Access_To_Subp_Def;
Local_Other_Subp_Access : Internal_Entity_Access_To_Subp_Def;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Local_Exp : Internal_Entity_Base_Type_Decl;
Local_Act : Internal_Entity_Base_Type_Decl;
Var_Expr_147 : Internal_Entity_Access_Def;
Fld : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Access_To_Subp_Def;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Type_Def;
Cast_Result_3 : Internal_Entity_Access_Def;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Fld_11 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Is_Null : Boolean;
Is_Null_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Let_Result_2 : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5022 '<Let at nodes.lkt:7392:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7392'
--# scope-start
--# expr-start 4996 '<Cast to AccessToSubpDef at nodes.lkt:7392:32>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7392'
--# expr-start 4995 '<FieldAccess for type_def at nodes.lkt:7392:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7392'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 4995
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 4996
Local_Self_Subp_Access := Cast_Result; 
--# bind self_subp_access Local_Self_Subp_Access
--# expr-start 5021 '<Let at nodes.lkt:7393:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7393'
--# scope-start
--# expr-start 4998 '<Cast to AccessToSubpDef at nodes.lkt:7393:33>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7393'
--# expr-start 4997 '<FieldAccess for access_def at nodes.lkt:7393:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7393'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 4997
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 4998
Local_Other_Subp_Access := Cast_Result_1; 
--# bind other_subp_access Local_Other_Subp_Access
--# expr-start 5020 '<If at nodes.lkt:7397:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
--# expr-start 5000 '<Not at nodes.lkt:7397:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
--# expr-start 4999 '<IsNull at nodes.lkt:7397:16>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
Is_Null_2 := Local_Self_Subp_Access.Node = null; 
--# expr-done 4999
Not_Val := not (Is_Null_2); 
--# expr-done 5000
if Not_Val then
   --# expr-start 5002 '<Not at nodes.lkt:7397:45>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
--# expr-start 5001 '<IsNull at nodes.lkt:7397:49>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
Is_Null_3 := Local_Other_Subp_Access.Node = null; 
--# expr-done 5001
Not_Val_1 := not (Is_Null_3); 
--# expr-done 5002
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 5005 '<FieldAccess for match_signature at nodes.lkt:7397:80>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
--# expr-start 5003 '<FieldAccess for subp_spec at nodes.lkt:7397:80>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
      if Local_Other_Subp_Access.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Local_Other_Subp_Access.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Local_Other_Subp_Access.Info);
--# expr-done 5003
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5004 '<FieldAccess for subp_spec at nodes.lkt:7397:124>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
      if Local_Self_Subp_Access.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Local_Self_Subp_Access.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Local_Self_Subp_Access.Info);
--# expr-done 5004
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Cast_Result_2, Match_Name => False, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5005
   If_Result_4 := Fld_4;
else
   --# expr-start 5019 '<If at nodes.lkt:7397:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7397'
--# expr-start 5006 '<IsNull at nodes.lkt:7398:14>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7398'
Is_Null := Local_Self_Subp_Access.Node = null; 
--# expr-done 5006
if Is_Null then
   --# expr-start 5007 '<IsNull at nodes.lkt:7398:43>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7398'
Is_Null_1 := Local_Other_Subp_Access.Node = null; 
--# expr-done 5007
   If_Result_1 := Is_Null_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
--# expr-start 5009 '<Cast to AccessDef at nodes.lkt:7398:74>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7398'
--# expr-start 5008 '<FieldAccess for type_def at nodes.lkt:7398:74>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7398'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 5008
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Access_Def
   then
      Cast_Result_3 := Create_Internal_Entity_Access_Def
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_3 := No_Entity_Access_Def;
   end if;
--# expr-done 5009
Var_Expr_147 := Cast_Result_3;
if Var_Expr_147 /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 5010 '<FieldAccess for accessed_type at nodes.lkt:7398:74>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7398'
      if Var_Expr_147.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Var_Expr_147.Node, Origin => Origin, E_Info => Var_Expr_147.Info);
--# end
--# expr-done 5010
Var_Expr_0 := Fld_6;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ast Var_Expr_0
--# expr-start 5011 '<FieldAccess for accessed_type at nodes.lkt:7399:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7399'
      if Other.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 5011
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind oat Var_Expr_1
--# expr-start 5018 '<Let at nodes.lkt:7404:21>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7404'
--# scope-start
--# expr-start 5012 '<FieldAccess for specific_type at nodes.lkt:7404:31>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7404'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 5012
Local_Exp := Fld_8; 
--# bind exp Local_Exp
--# expr-start 5017 '<Let at nodes.lkt:7405:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7405'
--# scope-start
--# expr-start 5013 '<FieldAccess for specific_type at nodes.lkt:7405:31>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7405'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.specific_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Specific_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5013
Local_Act := Fld_9; 
--# bind act Local_Act
--# expr-start 5016 '<If at nodes.lkt:7407:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7407'
if For_Assignment then
   --# expr-start 5014 '<FieldAccess for matching_assign_type at nodes.lkt:7411:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7411'
      if Local_Act.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Local_Act.Node, Expected_Type => Var_Expr_0, Origin => Origin, E_Info => Local_Act.Info);
--# end
--# expr-done 5014
   If_Result := Fld_10;
else
   --# expr-start 5015 '<FieldAccess for matching_type at nodes.lkt:7412:28>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7412'
      if Local_Act.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Local_Act.Node, Expected_Type => Local_Exp, Origin => Origin, E_Info => Local_Act.Info);
--# end
--# expr-done 5015
   If_Result := Fld_11;
end if;
--# expr-done 5016
Let_Result := If_Result; 
--# end
--# expr-done 5017
Let_Result_1 := Let_Result; 
--# end
--# expr-done 5018
Scope_Result := Let_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
--# expr-done 5019
   If_Result_4 := If_Result_2;
end if;
--# expr-done 5020
Let_Result_2 := If_Result_4; 
--# end
--# expr-done 5021
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 5022
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Decl_P_Access_Def_Matches;
--# end
--# property-start AnonymousTypeDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7419'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Anonymous_Type_Decl
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl  := Bare_Anonymous_Type_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synth_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Synth_Anonymous_Type_Decl
        (Self : Bare_Synth_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
        ) is
      begin
            Initialize_Fields_For_Anonymous_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
      end Initialize_Fields_For_Synth_Anonymous_Type_Decl;
   --
   --  Primitives for Bare_Concrete_Type_Decl
   --
      procedure Initialize_Fields_For_Concrete_Type_Decl
        (Self : Bare_Concrete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Concrete_Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
            Self.Concrete_Type_Decl_F_Aspects := Concrete_Type_Decl_F_Aspects;
      end Initialize_Fields_For_Concrete_Type_Decl;
   --
   --  Primitives for Bare_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Formal_Type_Decl
        (Self : Bare_Formal_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Formal_Type_Decl_F_Default_Type : Bare_Name
         ; Formal_Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def);
            Self.Formal_Type_Decl_F_Default_Type := Formal_Type_Decl_F_Default_Type;
            Self.Formal_Type_Decl_F_Aspects := Formal_Type_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Type_Decl;
   function Formal_Type_Decl_F_Default_Type
     (Node : Bare_Formal_Type_Decl) return Bare_Name
   is
   begin
         return Node.Formal_Type_Decl_F_Default_Type;
   end;
--# property-start FormalTypeDecl.corresponding_actual_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7445'
pragma Warnings (Off, "is not referenced");
 function Formal_Type_Decl_P_Corresponding_Actual_Impl
  (Node : Bare_Formal_Type_Decl
      ; Rb : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Formal_Type_Decl  := Bare_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Formal_Type_Decl :=
       Internal_Entity_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rb Rb
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_818 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Env_Rebindings;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Bare_Ada_Node;
Fld_7 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Formal_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Is_Equal_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_818 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_818;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5036 '<If at nodes.lkt:7445:78>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7445'
--# expr-start 5023 '<IsNull at nodes.lkt:7445:81>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7445'
Is_Equal_1 := Rb = null; 
--# expr-done 5023
if Is_Equal_1 then
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result_1;
else
   --# expr-start 5035 '<If at nodes.lkt:7445:78>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7445'
--# expr-start 5027 '<Eq at nodes.lkt:7446:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
--# expr-start 5024 '<FieldAccess for old_env at nodes.lkt:7446:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Rb.Old_Env;
Inc_Ref (Fld_5);
--# expr-done 5024
--# expr-start 5026 '<FieldAccess for node_env at nodes.lkt:7446:24>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
--# expr-start 5025 '<parent at nodes.lkt:7446:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Parent; 
--# expr-done 5025
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Node_Env (Node => Fld_6);
--# expr-done 5026
Is_Equal := Equivalent (Fld_5, Fld_7); 
--# expr-done 5027
if Is_Equal then
   --# expr-start 5032 '<Cast to BaseTypeDecl at nodes.lkt:7446:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
--# expr-start 5031 '<EnvGet at nodes.lkt:7446:52>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
--# expr-start 5028 '<FieldAccess for new_env at nodes.lkt:7446:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7446'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Rb.New_Env;
Inc_Ref (Fld);
--# expr-done 5028
--# expr-start 5030 '<FieldAccess for name_symbol at nodes.lkt:7447:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7447'
--# expr-start 5029 '<FieldAccess for defining_name at nodes.lkt:7447:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7447'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5029
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 5030
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Thin (Fld_2), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 5031
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5032
   If_Result := Cast_Result;
else
   --# expr-start 5034 '<FieldAccess for corresponding_actual_impl at nodes.lkt:7449:10>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5033 '<FieldAccess for get_parent at nodes.lkt:7449:41>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7449'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Rb.Parent;
--# expr-done 5033
--# property-call-start FormalTypeDecl.corresponding_actual_impl
Fld_4 := Libadalang.Implementation.Formal_Type_Decl_P_Corresponding_Actual_Impl (Node => Ent.Node, Rb => Fld_3, E_Info => Ent.Info);
--# end
--# expr-done 5034
   If_Result := Fld_4;
end if;
--# expr-done 5035
   If_Result_1 := If_Result;
end if;
--# expr-done 5036
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_818;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_818;
            raise;
      end;
   return Property_Result;
end Formal_Type_Decl_P_Corresponding_Actual_Impl;
--# end
--# property-start FormalTypeDecl.get_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7453'
pragma Warnings (Off, "is not referenced");
 function Formal_Type_Decl_P_Get_Actual
  (Node : Bare_Formal_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Formal_Type_Decl  := Bare_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Formal_Type_Decl :=
       Internal_Entity_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5039 '<FieldAccess for corresponding_actual_impl at nodes.lkt:7454:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7454'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5038 '<FieldAccess for rebindings at nodes.lkt:7454:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7454'
--# expr-start 5037 '<FieldAccess for info at nodes.lkt:7454:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7454'
Fld := Ent.Info;
--# expr-done 5037
Fld_1 := Fld.Rebindings;
--# expr-done 5038
--# property-call-start FormalTypeDecl.corresponding_actual_impl
Fld_2 := Libadalang.Implementation.Formal_Type_Decl_P_Corresponding_Actual_Impl (Node => Ent.Node, Rb => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 5039
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Formal_Type_Decl_P_Get_Actual;
--# end
--# property-start FormalTypeDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7457'
pragma Warnings (Off, "is not referenced");
 function Formal_Type_Decl_P_Xref_Equation
  (Node : Bare_Formal_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Type_Decl  := Bare_Formal_Type_Decl (Node);
     Ent : Internal_Entity_Formal_Type_Decl :=
       Internal_Entity_Formal_Type_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_820 with Inline_Always;
            procedure Finalizer_Scope_2641 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_820 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_820;
            procedure Finalizer_Scope_2641 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2641;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5043 '<And at nodes.lkt:7457:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7457'
--# expr-start 5040 '<Super at nodes.lkt:7457:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7457'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDecl.xref_equation
Fld := Libadalang.Implementation.Type_Decl_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 5040
--# expr-start 5041 '<FieldAccess for default_type at nodes.lkt:7457:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7457'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Formal_Type_Decl_F_Default_Type, Info => Ent.Info);
--# expr-done 5041
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind dt Var_Expr_0
--# expr-start 5042 '<FieldAccess for sub_equation at nodes.lkt:7458:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7458'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 5042
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2641;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7458:48") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7457:37") else null)); 
--# expr-done 5043
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_820;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_820;
                     Finalizer_Scope_2641;
            raise;
      end;
   return Property_Result;
end Formal_Type_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Basic_Subp_Decl
   --
      procedure Basic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Basic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_36 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_37 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:7545:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_38 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7551:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_39 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_40 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_41 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_43 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_42 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_45 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_44 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Basic_Subp_Decl_Post_Env_Actions
        (Self : Bare_Basic_Subp_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_46 (Node => 
Self);
      end;
      end;
--# property-start BasicSubpDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7465'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Names
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_821 with Inline_Always;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_821 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_821;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5046 '<ArrayLiteral at nodes.lkt:7466:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7466'
--# expr-start 5045 '<FieldAccess for name at nodes.lkt:7466:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7466'
--# expr-start 5044 '<FieldAccess for subp_decl_spec at nodes.lkt:7466:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7466'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5044
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5045
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_1)); 
--# expr-done 5046
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_821;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_821;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Defining_Names;
--# end
--# property-start BasicSubpDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7469'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Env
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_822 with Inline_Always;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_822 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_822;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5048 '<FieldAccess for defining_env at nodes.lkt:7470:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7470'
--# expr-start 5047 '<FieldAccess for subp_decl_spec at nodes.lkt:7470:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7470'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5047
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5048
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_822;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_822;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Defining_Env;
--# end
--# property-start BasicSubpDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7474'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Type_Expression
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5050 '<FieldAccess for returns at nodes.lkt:7474:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7474'
--# expr-start 5049 '<FieldAccess for subp_decl_spec at nodes.lkt:7474:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7474'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5049
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5050
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Type_Expression;
--# end
--# property-start BasicSubpDecl.get_body_in_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7477'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Get_Body_In_Env
  (Node : Bare_Basic_Subp_Decl
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_824 with Inline_Always;
      Local_Elements : Internal_Entity_Array_Access;
Local_Self_Spec : Internal_Entity_Base_Subp_Spec;
Local_Precise : Internal_Entity_Body_Node;
Local_Result : Internal_Entity_Body_Node;
Var_Expr_148 : Internal_Entity_Body_Node;
Var_Expr_149 : Internal_Entity_Body_Node;
Fld : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Bare_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld_5 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Body_Node;
Dyn_Var_Bind_Result : Internal_Entity_Body_Node;
Is_A : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Body_Node;
Fld_7 : Bare_Body_Node;
As_Entity_2 : Internal_Entity_Body_Node;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Body_Node;
Fld_8 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
Let_Result_1 : Internal_Entity_Body_Node;
Let_Result_2 : Internal_Entity_Body_Node;
Let_Result_3 : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_824 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Local_Elements);
            end Finalizer_Scope_824;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5077 '<Let at nodes.lkt:7478:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7478'
--# scope-start
--# expr-start 5052 '<EnvGet at nodes.lkt:7478:24>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7478'
--# expr-start 5051 '<FieldAccess for name_symbol at nodes.lkt:7479:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7479'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5051
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Thin (Fld), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5052
Local_Elements := Env_Get_Result; Inc_Ref (Local_Elements);
--# bind elements Local_Elements
--# expr-start 5076 '<Let at nodes.lkt:7481:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7481'
--# scope-start
--# expr-start 5055 '<as_bare_entity at nodes.lkt:7481:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7481'
--# expr-start 5054 '<FieldAccess for node at nodes.lkt:7481:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7481'
--# expr-start 5053 '<FieldAccess for subp_decl_spec at nodes.lkt:7481:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7481'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5053
Fld_2 := Fld_1.Node;
--# expr-done 5054
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 5055
Local_Self_Spec := If_Result; 
--# bind self_spec Local_Self_Spec
--# expr-start 5075 '<Let at nodes.lkt:7482:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7482'
--# scope-start
--# expr-start 5064 '<bind at nodes.lkt:7483:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7483'
--# expr-start 5056 '<FieldAccess for origin_node at nodes.lkt:7483:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7483'
      if Local_Self_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Local_Self_Spec.Node);
--# end
--# expr-done 5056
Bound_Origin := Fld_3; 
--# expr-start 5063 '<Cast to Body at nodes.lkt:7485:13>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7485'
--# expr-start 5062 '<Find at nodes.lkt:7485:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7485'
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Elements;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_0
--# expr-start 5059 '<Cast to Body at nodes.lkt:7488:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7488'
--# expr-start 5058 '<as_bare_entity at nodes.lkt:7488:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7488'
--# expr-start 5057 '<FieldAccess for node at nodes.lkt:7488:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7488'
Fld_4 := Item_0.Node;
--# expr-done 5057
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 5058
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 5059
Var_Expr_148 := Cast_Result;
if Var_Expr_148 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 5061 '<FieldAccess for match_other at nodes.lkt:7488:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7488'
--# expr-start 5060 '<FieldAccess for formal_param_holder_or_null at nodes.lkt:7488:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7488'
      if Var_Expr_148.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr_148.Node, E_Info => Var_Expr_148.Info);
--# end
--# expr-done 5060
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Local_Self_Spec; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start '[dispatcher]BaseFormalParamHolder.match_other'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Match_Other (Node => Fld_5.Node, Other => Cast_Result_1, Match_Names => True, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 5061
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 5062
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 5063
Dyn_Var_Bind_Result := Cast_Result_2; 
--# expr-done 5064
Local_Precise := Dyn_Var_Bind_Result; 
--# bind precise Local_Precise
--# expr-start 5074 '<Let at nodes.lkt:7491:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
--# scope-start
--# expr-start 5069 '<If at nodes.lkt:7491:22>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
--# expr-start 5065 '<IsNull at nodes.lkt:7491:25>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
Is_Null_2 := Local_Precise.Node = null; 
--# expr-done 5065
if Is_Null_2 then
   If_Result_2 := Imprecise_Fallback;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 5068 '<Cast to Body at nodes.lkt:7491:69>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
--# expr-start 5067 '<Find at nodes.lkt:7491:69>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Elements;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
      --# expr-start 5066 '<IsA Body at nodes.lkt:7491:90>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7491'
Is_A := Item_1.Node /= null 
and then Item_1.Node.Kind in Ada_Body_Node; 
--# expr-done 5066
      if Is_A then
         Find_Result_1 := Item_1; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 5067
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 5068
   If_Result_3 := Cast_Result_3;
else
   If_Result_3 := Local_Precise;
end if;
--# expr-done 5069
Local_Result := If_Result_3; 
--# bind result Local_Result
--# expr-start 5073 '<Cast to Body at nodes.lkt:7494:9>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7494'
--# expr-start 5071 '<as_entity at nodes.lkt:7494:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7494'
--# expr-start 5070 '<FieldAccess for node at nodes.lkt:7494:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7494'
Fld_7 := Local_Result.Node;
--# expr-done 5070
Is_Null_3 := Fld_7 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Body_Node;
else
As_Entity_2 := (Info => E_Info, Node => Fld_7); 
   If_Result_4 := As_Entity_2;
end if;
--# expr-done 5071
Var_Expr_149 := If_Result_4;
if Var_Expr_149 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 5072 '<FieldAccess for without_md at nodes.lkt:7494:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7494'
      if Var_Expr_149.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.without_md
Fld_8 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr_149.Node, E_Info => Var_Expr_149.Info);
--# end
--# expr-done 5072
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_4 := Create_Internal_Entity_Body_Node
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_4 := No_Entity_Body_Node;
   end if;
--# expr-done 5073
Let_Result := Cast_Result_4; 
--# end
--# expr-done 5074
Let_Result_1 := Let_Result; 
--# end
--# expr-done 5075
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 5076
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 5077
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_824;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_824;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Get_Body_In_Env;
--# end
--# property-start BasicSubpDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7498'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_825 with Inline_Always;
            procedure Finalizer_Scope_2647 with Inline_Always;
            procedure Finalizer_Scope_2649 with Inline_Always;
      Local_Decl_Scope : Bare_Declarative_Part;
Local_Parent_Decl : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Declarative_Part;
Local_Default_Next_Part : Internal_Entity_Basic_Decl;
Local_Other_Part : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Body_Node;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_A : Boolean;
If_Result_1 : Internal_Entity;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Body_Node;
Scope_Result_1 : Internal_Entity_Body_Node;
Result_Var_1 : Internal_Entity_Body_Node;
Scope_Result_2 : Internal_Entity_Body_Node;
Fld_7 : Internal_Entity_Body_Node;
Fld_8 : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Body_Node;
Scope_Result_3 : Internal_Entity_Body_Node;
Result_Var_2 : Internal_Entity_Body_Node;
Result_Var_3 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Body_Node;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Body_Node;
Is_A_1 : Boolean;
If_Result_3 : Internal_Entity_Body_Node;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
Not_Val : Boolean;
If_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_825 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_825;
            procedure Finalizer_Scope_2647 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2647;
            procedure Finalizer_Scope_2649 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2649;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5104 '<Let at nodes.lkt:7499:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7499'
--# scope-start
--# expr-start 5078 '<FieldAccess for declarative_scope at nodes.lkt:7499:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7499'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 5078
Local_Decl_Scope := Fld; 
--# bind decl_scope Local_Decl_Scope
--# expr-start 5103 '<Let at nodes.lkt:7500:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7500'
--# scope-start
--# expr-start 5079 '<as_entity at nodes.lkt:7500:27>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7500'
Is_Null := Local_Decl_Scope = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Local_Decl_Scope); 
   If_Result := As_Entity;
end if;
--# expr-done 5079
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind ds Var_Expr_0
--# expr-start 5081 '<Cast to BasicDecl at nodes.lkt:7501:21>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7501'
--# expr-start 5080 '<FieldAccess for semantic_parent at nodes.lkt:7501:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7501'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 5080
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5081
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Local_Parent_Decl := Result_Var; 
--# bind parent_decl Local_Parent_Decl
--# expr-start 5102 '<Let at nodes.lkt:7503:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7503'
--# scope-start
--# expr-start 5082 '<Super at nodes.lkt:7503:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7503'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5082
Local_Default_Next_Part := Fld_2; 
--# bind default_next_part Local_Default_Next_Part
--# expr-start 5101 '<If at nodes.lkt:7507:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7507'
--# expr-start 5084 '<Not at nodes.lkt:7507:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7507'
--# expr-start 5083 '<IsNull at nodes.lkt:7507:16>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7507'
Is_Null_2 := Local_Default_Next_Part.Node = null; 
--# expr-done 5083
Not_Val := not (Is_Null_2); 
--# expr-done 5084
if Not_Val then
   If_Result_4 := Local_Default_Next_Part;
else
--# expr-start 5100 '<If at nodes.lkt:7507:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7507'
--# expr-start 5085 '<IsA PrivatePart, PublicPart at nodes.lkt:7508:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7508'
Is_A_1 := Local_Decl_Scope /= null 
and then Local_Decl_Scope.Kind in Ada_Private_Part_Range | Ada_Public_Part_Range; 
--# expr-done 5085
if Is_A_1 then
   --# expr-start 5095 '<Let at nodes.lkt:7509:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7509'
--# scope-start
--# expr-start 5089 '<If at nodes.lkt:7509:30>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7509'
--# expr-start 5086 '<IsA PrivatePart at nodes.lkt:7509:33>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7509'
Is_A := Local_Decl_Scope /= null 
and then Local_Decl_Scope.Kind in Ada_Private_Part_Range; 
--# expr-done 5086
if Is_A then
--# expr-start 5087 '<FieldAccess for next_part_for_decl at nodes.lkt:7509:64>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7509'
      if Local_Parent_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Local_Parent_Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Parent_Decl.Info);
--# end
--# expr-done 5087
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   If_Result_1 := Cast_Result_1;
else
--# expr-start 5088 '<FieldAccess for decl_private_part at nodes.lkt:7509:102>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7509'
      if Local_Parent_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.decl_private_part
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Decl_Private_Part (Node => Local_Parent_Decl.Node, E_Info => Local_Parent_Decl.Info);
--# end
--# expr-done 5088
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   If_Result_1 := Cast_Result_2;
end if;
--# expr-done 5089
Local_Other_Part := If_Result_1; 
--# bind other_part Local_Other_Part
Var_Expr_1 := Local_Other_Part;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind op Var_Expr_1
--# expr-start 5091 '<FieldAccess for get_body_in_env at nodes.lkt:7513:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7513'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5090 '<FieldAccess for children_env at nodes.lkt:7513:46>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7513'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 5090
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_6 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_5, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5091
Scope_Result_1 := Fld_6;
   --# end
      Finalizer_Scope_2647;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Body_Node;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_2 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_2;
else
--# expr-start 5092 '<FieldAccess for body_part_for_decl at nodes.lkt:7514:19>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7514'
      if Local_Parent_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Local_Parent_Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Parent_Decl.Info);
--# end
--# expr-done 5092
Var_Expr_2 := Fld_7;
if Var_Expr_2 /= No_Entity_Body_Node then
   --# scope-start
   --# bind np Var_Expr_2
--# expr-start 5094 '<FieldAccess for get_body_in_env at nodes.lkt:7516:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7516'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5093 '<FieldAccess for children_env at nodes.lkt:7516:46>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7516'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 5093
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_9 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_8, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5094
Scope_Result_3 := Fld_9;
   --# end
      Finalizer_Scope_2649;
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Body_Node;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result := Result_Var_3; 
--# end
--# expr-done 5095
   If_Result_3 := Let_Result;
else
   --# expr-start 5099 '<If at nodes.lkt:7507:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7507'
--# expr-start 5096 '<IsNull at nodes.lkt:7520:14>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7520'
Is_Null_1 := Local_Decl_Scope = null; 
--# expr-done 5096
if Is_Null_1 then
   If_Result_2 := No_Entity_Body_Node;
else
   --# expr-start 5098 '<FieldAccess for get_body_in_env at nodes.lkt:7523:14>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7523'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5097 '<FieldAccess for children_env at nodes.lkt:7523:35>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7523'
      if Local_Decl_Scope = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Local_Decl_Scope);
--# expr-done 5097
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_11 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5098
   If_Result_2 := Fld_11;
end if;
--# expr-done 5099
   If_Result_3 := If_Result_2;
end if;
--# expr-done 5100
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_3.Node,
         Info => If_Result_3.Info);
   If_Result_4 := Cast_Result_3;
end if;
--# expr-done 5101
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 5102
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 5103
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 5104
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_825;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_825;
                     Finalizer_Scope_2647;
                     Finalizer_Scope_2649;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicSubpDecl.constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7527'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Subp_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_826 with Inline_Always;
      Fld : Logic_Equation;
            procedure Finalizer_Scope_826 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_826;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5105 '<FieldAccess for subp_constrain_prefix at nodes.lkt:7528:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7528'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_constrain_prefix
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Constrain_Prefix (Node => Ent.Node, Prefix => Prefix, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5105
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_826;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_826;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicSubpDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7531'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Expr_Type
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_150 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5106 '<FieldAccess for subp_spec_or_null at nodes.lkt:7532:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7532'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5106
Var_Expr_150 := Fld;
if Var_Expr_150 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 5107 '<FieldAccess for return_type at nodes.lkt:7532:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7532'
      if Var_Expr_150.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_150.Node, Origin => Origin, E_Info => Var_Expr_150.Info);
--# end
--# expr-done 5107
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Expr_Type;
--# end
--# property-start BasicSubpDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7534'
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Env_Names
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_828 with Inline_Always;
            procedure Finalizer_Scope_2651 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_828 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_828;
            procedure Finalizer_Scope_2651 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2651;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5108 '<FieldAccess for top_level_env_name at nodes.lkt:7535:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7535'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5108
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 5110 '<ArrayLiteral at nodes.lkt:7535:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7535'
--# expr-start 5109 '<to_symbol at nodes.lkt:7535:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7535'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 5109
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 5110
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2651;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_828;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_828;
                     Finalizer_Scope_2651;
            raise;
      end;
   return Property_Result;
end Basic_Subp_Decl_P_Env_Names;
--# end
--# property-start '[dispatcher]BasicSubpDecl.subp_decl_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Basic_Subp_Decl (Self.Kind) is
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
                  --# property-call-start ClassicSubpDecl.subp_decl_spec
                  Property_Result := Classic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.subp_decl_spec
                  Property_Result := Entry_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start EnumLiteralDecl.subp_decl_spec
                  Property_Result := Enum_Literal_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Internal =>
                  --# property-call-start GenericSubpInternal.subp_decl_spec
                  Property_Result := Generic_Subp_Internal_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Subp_Decl =>
                  --# property-call-start SyntheticSubpDecl.subp_decl_spec
                  Property_Result := Synthetic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start BasicSubpDecl._env_do_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7544'
pragma Warnings (Off, "is not referenced");
 function Env_Do_36
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5111 '<FieldAccess for env_hook at nodes.lkt:7544:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7544'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5111
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_36;
--# end
--# property-start BasicSubpDecl._env_init_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7549'
pragma Warnings (Off, "is not referenced");
 function Env_Init_37
  (Node : Bare_Basic_Subp_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1701 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1701 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1701;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5112 '<FieldAccess for child_decl_initial_env at nodes.lkt:7549:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7549'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 5112
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1701;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1701;
            raise;
      end;
   return Property_Result;
end Env_Init_37;
--# end
--# property-start BasicSubpDecl._env_mappings_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7551'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_38
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
     Ent : Internal_Entity_Basic_Subp_Decl :=
       Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1702 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1702 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1702;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5113 '<FieldAccess for child_decl_env_assocs at nodes.lkt:7551:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7551'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5113
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1702;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1702;
            raise;
      end;
   return Property_Result;
end Env_Mappings_38;
--# end
--# property-start BasicSubpDecl._env_trans_parent_39 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_39
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_39;
--# end
--# property-start BasicSubpDecl._env_names_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7552'
pragma Warnings (Off, "is not referenced");
 function Env_Names_40
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1704 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1704 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1704;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5114 '<FieldAccess for env_names at nodes.lkt:7552:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7552'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 5114
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1704;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1704;
            raise;
      end;
   return Property_Result;
end Env_Names_40;
--# end
--# property-start BasicSubpDecl._env_do_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7553'
pragma Warnings (Off, "is not referenced");
 function Env_Do_41
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1705 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1705 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1705;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5115 '<FieldAccess for populate_dependent_units at nodes.lkt:7553:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7553'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5115
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1705;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1705;
            raise;
      end;
   return Property_Result;
end Env_Do_41;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7555'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_42
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1706 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1706 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1706;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5116 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:7555:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5116
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1706;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1706;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_42;
--# end
--# property-start BasicSubpDecl._ref_cond_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7555'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_43
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5118 '<IsA LibraryItem, Subunit at nodes.lkt:7555:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7555'
--# expr-start 5117 '<parent at nodes.lkt:7555:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5117
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5118
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_43;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7558'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_44
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1708 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1708 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1708;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5119 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:7558:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7558'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5119
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1708;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1708;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_44;
--# end
--# property-start BasicSubpDecl._ref_cond_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7558'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_45
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5121 '<IsA LibraryItem, Subunit at nodes.lkt:7558:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7558'
--# expr-start 5120 '<parent at nodes.lkt:7558:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7558'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5120
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5121
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_45;
--# end
--# property-start BasicSubpDecl._env_do_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7561'
pragma Warnings (Off, "is not referenced");
 function Env_Do_46
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl  := Bare_Basic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5122 '<FieldAccess for populate_body_unit at nodes.lkt:7561:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7561'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_body_unit
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Body_Unit (Node => Self);
--# end
--# expr-done 5122
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_46;
--# end
   --
   --  Primitives for Bare_Classic_Subp_Decl
   --
      procedure Initialize_Fields_For_Classic_Subp_Decl
        (Self : Bare_Classic_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Classic_Subp_Decl_F_Overriding := Classic_Subp_Decl_F_Overriding;
            Self.Classic_Subp_Decl_F_Subp_Spec := Classic_Subp_Decl_F_Subp_Spec;
      end Initialize_Fields_For_Classic_Subp_Decl;
   function Classic_Subp_Decl_F_Overriding
     (Node : Bare_Classic_Subp_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Classic_Subp_Decl_F_Overriding;
   end;
   function Classic_Subp_Decl_F_Subp_Spec
     (Node : Bare_Classic_Subp_Decl) return Bare_Subp_Spec
   is
   begin
         return Node.Classic_Subp_Decl_F_Subp_Spec;
   end;
--# property-start ClassicSubpDecl.subp_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7573'
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Classic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Classic_Subp_Decl  := Bare_Classic_Subp_Decl (Node);
     Ent : Internal_Entity_Classic_Subp_Decl :=
       Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5123 '<FieldAccess for subp_spec at nodes.lkt:7573:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7573'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5123
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start ClassicSubpDecl.body_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7578'
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Body_Part
  (Node : Bare_Classic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Classic_Subp_Decl  := Bare_Classic_Subp_Decl (Node);
     Ent : Internal_Entity_Classic_Subp_Decl :=
       Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5125 '<Cast to BaseSubpBody at nodes.lkt:7579:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7579'
--# expr-start 5124 '<FieldAccess for body_part_for_decl at nodes.lkt:7579:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7579'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5124
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5125
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Classic_Subp_Decl_P_Body_Part;
--# end
   --
   --  Primitives for Bare_Abstract_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Subp_Decl
        (Self : Bare_Abstract_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Abstract_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Abstract_Subp_Decl_F_Aspects := Abstract_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_Subp_Decl;
   --
   --  Primitives for Bare_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Formal_Subp_Decl
        (Self : Bare_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Formal_Subp_Decl_F_Default_Expr := Formal_Subp_Decl_F_Default_Expr;
            Self.Formal_Subp_Decl_F_Aspects := Formal_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Subp_Decl;
   function Formal_Subp_Decl_F_Default_Expr
     (Node : Bare_Formal_Subp_Decl) return Bare_Expr
   is
   begin
         return Node.Formal_Subp_Decl_F_Default_Expr;
   end;
--# property-start FormalSubpDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7594'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Defining_Names
  (Node : Bare_Formal_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_832 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_832 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_832;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5128 '<ArrayLiteral at nodes.lkt:7595:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7595'
--# expr-start 5127 '<FieldAccess for name at nodes.lkt:7595:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7595'
--# expr-start 5126 '<FieldAccess for subp_spec at nodes.lkt:7595:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7595'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5126
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5127
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_1)); 
--# expr-done 5128
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_832;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_832;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Defining_Names;
--# end
--# property-start FormalSubpDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7597'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Xref_Entry_Point
  (Node : Bare_Formal_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Xref_Entry_Point;
--# end
--# property-start FormalSubpDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7600'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Xref_Equation
  (Node : Bare_Formal_Subp_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_834 with Inline_Always;
            procedure Finalizer_Scope_2652 with Inline_Always;
            procedure Finalizer_Scope_2653 with Inline_Always;
            procedure Finalizer_Scope_2654 with Inline_Always;
            procedure Finalizer_Scope_2655 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Match_0 : Internal_Entity_Null_Literal;
Match_1 : Internal_Entity_Box_Expr;
Match_2 : Internal_Entity_Name;
Match_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
True_Rel : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Null_Literal;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Box_Expr;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Scope_Result_4 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_834 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_834;
            procedure Finalizer_Scope_2652 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2652;
            procedure Finalizer_Scope_2653 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2653;
            procedure Finalizer_Scope_2654 is
            begin
                     Dec_Ref (True_Rel_1);
            end Finalizer_Scope_2654;
            procedure Finalizer_Scope_2655 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2655;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5129 '<FieldAccess for default_expr at nodes.lkt:7600:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Formal_Subp_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 5129
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 5138 '<Match at nodes.lkt:7601:16>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7601'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Null_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Null_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7602:36") else null)); 
Let_Result := True_Rel; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2653;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Box_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Box_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7603:32") else null)); 
Let_Result_1 := True_Rel_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2654;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_String_Literal | Ada_Int_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind n Match_2
--# expr-start 5137 '<And at nodes.lkt:7604:29>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7604'
--# expr-start 5133 '<If at nodes.lkt:7605:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7605'
--# expr-start 5130 '<IsA AttributeRef at nodes.lkt:7605:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7605'
Is_A := Match_2.Node /= null 
and then Match_2.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 5130
if Is_A then
   --# expr-start 5131 '<FieldAccess for sub_equation at nodes.lkt:7605:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7605'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 5131
   If_Result := Fld_1;
else
   --# expr-start 5132 '<FieldAccess for xref_no_overloading at nodes.lkt:7605:65>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7605'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_2.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 5132
   If_Result := Fld_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 5133
--# expr-start 5136 '<Predicate on BasicDecl.subp_decl_match_signature at nodes.lkt:7606:19>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7606'
--# expr-start 5134 '<FieldAccess for ref_var at nodes.lkt:7606:67>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7606'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 5134
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 5135 '<Cast to BasicDecl at nodes.lkt:7606:80>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7606'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 5135
Pred := Solver.Create_Predicate (Fld_3, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Origin)); 
--# expr-done 5136
And_Pred := Create_And (If_Result, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7604:29") else null)); 
--# expr-done 5137
Let_Result_2 := And_Pred; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2655;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl_Expr .. Ada_Relation_Op | Ada_Case_Expr_Alternative .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5138
Scope_Result_4 := Match_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2652;
   Result_Var := Scope_Result_4;
else
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:7608:24") else null)); 
   Result_Var := True_Rel_2;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_834;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_834;
                     Finalizer_Scope_2652;
                     Finalizer_Scope_2653;
                     Finalizer_Scope_2654;
                     Finalizer_Scope_2655;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Xref_Equation;
--# end
--# property-start FormalSubpDecl.resolve_actual_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7619'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Resolve_Actual_Name
  (Node : Bare_Formal_Subp_Decl
      ; Name : Internal_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_835 with Inline_Always;
      Var_Expr_151 : Internal_Entity_Basic_Decl;
Fld : Bare_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Cast_Expr_3 : Internal_Entity_Formal_Subp_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Is_A_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_835 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_835;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Formal_Subp_Decl_Formal_Subp_Decl_P_Resolve_Actual_Name,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Formal_Subp_Decl,
                                As_Bare_Formal_Subp_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Internal_Entity_Expr,
                                      As_Internal_Entity_Expr => Name);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5154 '<bind at nodes.lkt:7620:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7620'
--# expr-start 5139 '<FieldAccess for node at nodes.lkt:7620:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7620'
Fld := Name.Node;
--# expr-done 5139
      Cast_Result := Fld;
Bound_Origin := Cast_Result; 
--# expr-start 5153 '<If at nodes.lkt:7622:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7622'
--# expr-start 5140 '<IsA AttributeRef at nodes.lkt:7622:12>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7622'
Is_A_2 := Name.Node /= null 
and then Name.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 5140
if Is_A_2 then
   --# expr-start 5142 '<FieldAccess for attribute_subprogram at nodes.lkt:7623:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7623'
--# expr-start 5141 '<Cast to AttributeRef at nodes.lkt:7623:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7623'
Cast_Expr := Name; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Attribute_Ref;
   end if;
--# expr-done 5141
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_1 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5142
   If_Result_2 := Fld_1;
else
   --# expr-start 5152 '<If at nodes.lkt:7622:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7622'
--# expr-start 5143 '<IsA BoxExpr at nodes.lkt:7624:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7624'
Is_A_1 := Name.Node /= null 
and then Name.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 5143
if Is_A_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
   --# expr-start 5151 '<If at nodes.lkt:7622:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7622'
--# expr-start 5144 '<IsA Name at nodes.lkt:7628:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7628'
Is_A := Name.Node /= null 
and then Name.Node.Kind in Ada_Name; 
--# expr-done 5144
if Is_A then
   --# expr-start 5150 '<Cast to BasicDecl at nodes.lkt:7629:13>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7629'
--# expr-start 5149 '<Find at nodes.lkt:7629:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7629'
   --# expr-start 5146 '<FieldAccess for all_env_elements at nodes.lkt:7629:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7629'
--# expr-start 5145 '<Cast to Name at nodes.lkt:7629:13>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7629'
Cast_Expr_1 := Name; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 5145
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 5146
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_0
--# expr-start 5147 '<Cast to BasicDecl at nodes.lkt:7630:25>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7630'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5147
Var_Expr_151 := Cast_Result_3;
if Var_Expr_151 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5148 '<FieldAccess for subp_decl_match_signature at nodes.lkt:7630:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7630'
      if Var_Expr_151.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BasicDecl.subp_decl_match_signature
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Decl_Match_Signature (Node => Var_Expr_151.Node, Other => Cast_Result_4, Origin => Bound_Origin, E_Info => Var_Expr_151.Info);
--# end
--# expr-done 5148
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 5149
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5150
   If_Result := Cast_Result_5;
else
   If_Result := raise Property_Error with "invalid actual for subprogram formal";
end if;
--# expr-done 5151
   If_Result_1 := If_Result;
end if;
--# expr-done 5152
   If_Result_2 := If_Result_1;
end if;
--# expr-done 5153
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 5154
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_835;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_835;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Formal_Subp_Decl_P_Resolve_Actual_Name;
--# end
--# property-start FormalSubpDecl.corresponding_actual_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7641'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Corresponding_Actual_Impl
  (Node : Bare_Formal_Subp_Decl
      ; Rb : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind rb Rb
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_836 with Inline_Always;
      Var_Expr_0 : Bare_Generic_Instantiation;
Var_Expr_1 : Internal_Entity_Expr;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
Fld_1 : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Generic_Instantiation;
Scope_Result : Internal_Entity_Generic_Instantiation;
Result_Var : Internal_Entity_Generic_Instantiation;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Bare_Defining_Name;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Fld_6 : Bare_Basic_Decl;
Fld_7 : Internal_Entity_Info;
Fld_8 : Internal_Metadata;
Fld_9 : Internal_Entity_Info;
Fld_10 : Env_Rebindings;
New_Struct_2 : Internal_Entity_Info;
New_Struct_3 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
Fld_12 : Env_Rebindings;
Fld_13 : Internal_Entity_Basic_Decl;
Fld_14 : Lexical_Env := Empty_Env;
Fld_15 : Bare_Ada_Node;
Fld_16 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity_Formal_Subp_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Is_Equal_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_836 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_836;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5181 '<If at nodes.lkt:7642:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7642'
--# expr-start 5155 '<IsNull at nodes.lkt:7642:12>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7642'
Is_Equal_1 := Rb = null; 
--# expr-done 5155
if Is_Equal_1 then
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   If_Result_1 := Cast_Result_2;
else
   --# expr-start 5180 '<If at nodes.lkt:7642:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7642'
--# expr-start 5159 '<Eq at nodes.lkt:7644:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7644'
--# expr-start 5156 '<FieldAccess for old_env at nodes.lkt:7644:14>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7644'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Rb.Old_Env;
Inc_Ref (Fld_14);
--# expr-done 5156
--# expr-start 5158 '<FieldAccess for node_env at nodes.lkt:7644:28>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7644'
--# expr-start 5157 '<parent at nodes.lkt:7644:28>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7644'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Self.Parent; 
--# expr-done 5157
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Node_Env (Node => Fld_15);
--# expr-done 5158
Is_Equal := Equivalent (Fld_14, Fld_16); 
--# expr-done 5159
if Is_Equal then
--# expr-start 5168 '<FieldAccess for actual_for_formal at nodes.lkt:7645:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7645'
--# expr-start 5162 '<Cast to GenericInstantiation at nodes.lkt:7645:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7645'
--# expr-start 5161 '<env_node at nodes.lkt:7645:13>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7645'
--# expr-start 5160 '<FieldAccess for new_env at nodes.lkt:7645:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7645'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Rb.New_Env;
Inc_Ref (Fld);
--# expr-done 5160
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 5161
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5162
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind inst Var_Expr_0
--# expr-start 5165 '<New GenericInstantiation.entity at nodes.lkt:7646:27>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7646'
--# expr-start 5164 '<New EntityInfo at nodes.lkt:7647:37>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7647'
--# expr-start 5163 '<FieldAccess for get_parent at nodes.lkt:7649:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7649'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Rb.Parent;
--# expr-done 5163
New_Struct := (From_Rebound => False, Md => No_Metadata, Rebindings => Fld_1); 
--# expr-done 5164
New_Struct_1 := (Info => New_Struct, Node => Var_Expr_0); 
--# expr-done 5165
Scope_Result := New_Struct_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Instantiation;
end if;
      if Result_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5167 '<FieldAccess for node at nodes.lkt:7653:33>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7653'
--# expr-start 5166 '<FieldAccess for defining_name at nodes.lkt:7653:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7653'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5166
Fld_3 := Fld_2.Node;
--# expr-done 5167
--# property-call-start GenericInstantiation.actual_for_formal
Fld_4 := Libadalang.Implementation.Generic_Instantiation_P_Actual_For_Formal (Node => Result_Var.Node, Formal_Name => Fld_3, E_Info => Result_Var.Info);
--# end
--# expr-done 5168
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 5169 '<FieldAccess for resolve_actual_name at nodes.lkt:7654:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7654'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start FormalSubpDecl.resolve_actual_name
Fld_5 := Libadalang.Implementation.Formal_Subp_Decl_P_Resolve_Actual_Name (Node => Ent.Node, Name => Var_Expr_1, E_Info => Ent.Info);
--# end
--# expr-done 5169
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Var_Expr_2 := Result_Var_1;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind actual Var_Expr_2
--# expr-start 5177 '<FieldAccess for corresponding_actual at nodes.lkt:7656:29>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7656'
--# expr-start 5176 '<New BasicDecl.entity at nodes.lkt:7656:29>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7656'
--# expr-start 5174 '<New EntityInfo at nodes.lkt:7658:26>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7658'
--# expr-start 5171 '<FieldAccess for md at nodes.lkt:7659:28>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7659'
--# expr-start 5170 '<FieldAccess for info at nodes.lkt:7659:28>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7659'
Fld_7 := Var_Expr_2.Info;
--# expr-done 5170
Fld_8 := Fld_7.Md;
--# expr-done 5171
--# expr-start 5173 '<FieldAccess for rebindings at nodes.lkt:7660:36>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7660'
--# expr-start 5172 '<FieldAccess for info at nodes.lkt:7660:36>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7660'
Fld_9 := Var_Expr_2.Info;
--# expr-done 5172
Fld_10 := Fld_9.Rebindings;
--# expr-done 5173
New_Struct_2 := (From_Rebound => True, Md => Fld_8, Rebindings => Fld_10); 
--# expr-done 5174
--# expr-start 5175 '<FieldAccess for node at nodes.lkt:7657:26>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7657'
Fld_6 := Var_Expr_2.Node;
--# expr-done 5175
New_Struct_3 := (Info => New_Struct_2, Node => Fld_6); 
--# expr-done 5176
      if New_Struct_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.corresponding_actual'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Corresponding_Actual (Node => New_Struct_3.Node, E_Info => New_Struct_3.Info);
--# end
--# expr-done 5177
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var_2;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_3 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_3;
else
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   Result_Var_3 := Cast_Result_1;
end if;
   If_Result := Result_Var_3;
else
   --# expr-start 5179 '<FieldAccess for corresponding_actual_impl at nodes.lkt:7668:13>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7668'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5178 '<FieldAccess for get_parent at nodes.lkt:7668:44>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7668'
      if Rb = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Rb.Parent;
--# expr-done 5178
--# property-call-start FormalSubpDecl.corresponding_actual_impl
Fld_13 := Libadalang.Implementation.Formal_Subp_Decl_P_Corresponding_Actual_Impl (Node => Ent.Node, Rb => Fld_12, E_Info => Ent.Info);
--# end
--# expr-done 5179
   If_Result := Fld_13;
end if;
--# expr-done 5180
   If_Result_1 := If_Result;
end if;
--# expr-done 5181
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_836;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_836;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Corresponding_Actual_Impl;
--# end
--# property-start FormalSubpDecl.corresponding_actual 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7672'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Corresponding_Actual
  (Node : Bare_Formal_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5184 '<FieldAccess for corresponding_actual_impl at nodes.lkt:7673:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7673'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5183 '<FieldAccess for rebindings at nodes.lkt:7673:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7673'
--# expr-start 5182 '<FieldAccess for info at nodes.lkt:7673:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7673'
Fld := Ent.Info;
--# expr-done 5182
Fld_1 := Fld.Rebindings;
--# expr-done 5183
--# property-call-start FormalSubpDecl.corresponding_actual_impl
Fld_2 := Libadalang.Implementation.Formal_Subp_Decl_P_Corresponding_Actual_Impl (Node => Ent.Node, Rb => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 5184
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Corresponding_Actual;
--# end
--# property-start FormalSubpDecl.designated_subprogram_from 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7680'
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Designated_Subprogram_From
  (Node : Bare_Formal_Subp_Decl
      ; Inst : Internal_Entity_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl  := Bare_Formal_Subp_Decl (Node);
     Ent : Internal_Entity_Formal_Subp_Decl :=
       Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind inst Inst
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_838 with Inline_Always;
      Local_Subps : Internal_Entity_Array_Access;
Local_Found : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_152 : Internal_Entity_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Symbol_Type;
Fld_3 : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Internal_Entity_Subp_Spec;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_838 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Local_Subps);
            end Finalizer_Scope_838;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5200 '<Let at nodes.lkt:7681:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7681'
--# scope-start
--# expr-start 5189 '<FieldAccess for env_get at nodes.lkt:7681:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7681'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5185 '<FieldAccess for node_env at nodes.lkt:7682:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7682'
      if Inst.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Inst.Node, E_Info => Inst.Info);
--# expr-done 5185
--# expr-start 5187 '<FieldAccess for name_symbol at nodes.lkt:7682:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7682'
--# expr-start 5186 '<FieldAccess for defining_name at nodes.lkt:7682:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7682'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5186
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 5187
--# expr-start 5188 '<FieldAccess for node at nodes.lkt:7682:87>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7682'
Fld_3 := Inst.Node;
--# expr-done 5188
      Cast_Result := Fld_3;
--# property-call-start AdaNode.env_get
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld, Symbol => Fld_2, From_Node => Cast_Result);
--# end
--# expr-done 5189
Local_Subps := Fld_4; Inc_Ref (Local_Subps);
--# bind subps Local_Subps
--# expr-start 5199 '<Let at nodes.lkt:7685:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7685'
--# scope-start
--# expr-start 5197 '<bind at nodes.lkt:7686:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7686'
--# expr-start 5190 '<FieldAccess for origin_node at nodes.lkt:7686:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7686'
      if Inst.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Inst.Node);
--# end
--# expr-done 5190
Bound_Origin := Fld_5; 
--# expr-start 5196 '<Cast to BasicDecl at nodes.lkt:7688:13>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7688'
--# expr-start 5195 '<Find at nodes.lkt:7688:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7688'
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Subps;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_0
--# expr-start 5192 '<FieldAccess for subp_spec_or_null at nodes.lkt:7689:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7689'
--# expr-start 5191 '<Cast to BasicDecl at nodes.lkt:7689:27>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7689'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5191
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5192
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 5194 '<FieldAccess for match_signature at nodes.lkt:7690:31>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7690'
--# expr-start 5193 '<FieldAccess for subp_spec at nodes.lkt:7690:31>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7690'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5193
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_7.Node, Other => Var_Expr_1, Match_Name => False, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 5194
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 5195
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5196
Dyn_Var_Bind_Result := Cast_Result_2; 
--# expr-done 5197
Local_Found := Dyn_Var_Bind_Result; 
--# bind found Local_Found
Var_Expr_152 := Local_Found;
if Var_Expr_152 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5198 '<FieldAccess for wrap_public_reference at nodes.lkt:7700:9>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7700'
      if Var_Expr_152.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_152.Node, E_Info => Var_Expr_152.Info);
--# end
--# expr-done 5198
Scope_Result_1 := Fld_9;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 5199
Let_Result_1 := Let_Result; 
--# end
--# expr-done 5200
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_838;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_838;
            raise;
      end;
   return Property_Result;
end Formal_Subp_Decl_P_Designated_Subprogram_From;
--# end
   --
   --  Primitives for Bare_Abstract_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Formal_Subp_Decl
        (Self : Bare_Abstract_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Abstract_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Concrete_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Concrete_Formal_Subp_Decl
        (Self : Bare_Concrete_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Concrete_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Decl
   --
      procedure Initialize_Fields_For_Subp_Decl
        (Self : Bare_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Subp_Decl_F_Aspects := Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Decl;
--# property-start SubpDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7716'
pragma Warnings (Off, "is not referenced");
 function Subp_Decl_P_Is_Constant_Object
  (Node : Bare_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subp_Decl  := Bare_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Decl_P_Is_Constant_Object;
--# end
   --
   --  Primitives for Bare_Entry_Decl
   --
      procedure Entry_Decl_Pre_Env_Actions
        (Self            : Bare_Entry_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_47 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7775:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_48 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Decl
        (Self : Bare_Entry_Decl
         ; Entry_Decl_F_Overriding : Bare_Overriding_Node
         ; Entry_Decl_F_Spec : Bare_Entry_Spec
         ; Entry_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Decl_F_Overriding := Entry_Decl_F_Overriding;
            Self.Entry_Decl_F_Spec := Entry_Decl_F_Spec;
            Self.Entry_Decl_F_Aspects := Entry_Decl_F_Aspects;
      end Initialize_Fields_For_Entry_Decl;
   function Entry_Decl_F_Overriding
     (Node : Bare_Entry_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Entry_Decl_F_Overriding;
   end;
   function Entry_Decl_F_Spec
     (Node : Bare_Entry_Decl) return Bare_Entry_Spec
   is
   begin
         return Node.Entry_Decl_F_Spec;
   end;
--# property-start EntryDecl.subp_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7725'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Subp_Decl_Spec
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Entry_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5201 '<FieldAccess for spec at nodes.lkt:7725:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7725'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 5201
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EntryDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7727'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Defining_Names
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_841 with Inline_Always;
      Fld : Internal_Entity_Entry_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_841 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_841;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5204 '<ArrayLiteral at nodes.lkt:7727:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7727'
--# expr-start 5203 '<FieldAccess for name at nodes.lkt:7727:58>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7727'
--# expr-start 5202 '<FieldAccess for spec at nodes.lkt:7727:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7727'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 5202
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5203
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_1)); 
--# expr-done 5204
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_841;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_841;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Defining_Names;
--# end
--# property-start EntryDecl.body_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7732'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Body_Part
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Fld : Internal_Entity_Body_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5205 '<FieldAccess for body_part_for_decl at nodes.lkt:7732:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7732'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5205
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Body_Part;
--# end
--# property-start EntryDecl.has_family 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7735'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Has_Family
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Entry_Spec;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5209 '<Not at nodes.lkt:7735:30>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7735'
--# expr-start 5208 '<IsNull at nodes.lkt:7735:34>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7735'
--# expr-start 5207 '<FieldAccess for family_type at nodes.lkt:7735:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7735'
--# expr-start 5206 '<FieldAccess for spec at nodes.lkt:7735:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7735'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Entry_Decl_F_Spec;
--# expr-done 5206
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Entry_Spec_F_Family_Type;
--# expr-done 5207
Is_Null := Fld_1 = null; 
--# expr-done 5208
Not_Val := not (Is_Null); 
--# expr-done 5209
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Has_Family;
--# end
--# property-start EntryDecl.family_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7741'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Family_Type
  (Node : Bare_Entry_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_153 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Entry_Spec;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5212 '<Cast to TypeExpr at nodes.lkt:7742:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7742'
--# expr-start 5211 '<FieldAccess for family_type at nodes.lkt:7742:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7742'
--# expr-start 5210 '<FieldAccess for spec at nodes.lkt:7742:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7742'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 5210
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Fld.Node.Entry_Spec_F_Family_Type, Info => Fld.Info);
--# expr-done 5211
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Expr
   then
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Expr;
   end if;
--# expr-done 5212
Var_Expr_153 := Cast_Result;
if Var_Expr_153 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 5213 '<FieldAccess for designated_type at nodes.lkt:7742:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7742'
      if Var_Expr_153.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_153.Node, Origin => Origin, E_Info => Var_Expr_153.Info);
--# end
--# expr-done 5213
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Family_Type;
--# end
--# property-start EntryDecl.accept_stmts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7746'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Accept_Stmts
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Accept_Stmt_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_845 with Inline_Always;
      Match_0 : Internal_Entity_Single_Task_Type_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Single_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Expr : Internal_Entity_Task_Type_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Body_Node;
Cast_Result_4 : Internal_Entity;
Fld_3 : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_845 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_845;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5218 '<FieldAccess for find_accept_stmts at nodes.lkt:7746:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7746'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5217 '<FieldAccess for body_part_for_decl at nodes.lkt:7748:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7748'
--# expr-start 5216 '<Match at nodes.lkt:7748:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7748'
--# expr-start 5214 '<FieldAccess for parent_basic_decl at nodes.lkt:7748:15>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5214
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Single_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 5215 '<FieldAccess for parent_basic_decl at nodes.lkt:7749:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7749'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 5215
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tt Match_1
Cast_Expr := Match_1; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Precondition_Failure with "Can only be called on EntryDecl in Tasks"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5216
      if Match_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Match_Result.Node, Imprecise_Fallback => False, E_Info => Match_Result.Info);
--# end
--# expr-done 5217
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# property-call-start EntryDecl.find_accept_stmts
Fld_3 := Libadalang.Implementation.Entry_Decl_P_Find_Accept_Stmts (Node => Ent.Node, Root => Cast_Result_4, E_Info => Ent.Info);
--# end
--# expr-done 5218
         Property_Result := Fld_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_845;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_845;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Accept_Stmts;
--# end
--# property-start EntryDecl.find_accept_stmts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7757'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Find_Accept_Stmts
  (Node : Bare_Entry_Decl
      ; Root : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Accept_Stmt_Array_Access
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   Property_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_846 with Inline_Always;
            procedure Finalizer_Scope_2670 with Inline_Always;
            procedure Finalizer_Scope_2672 with Inline_Always;
            procedure Finalizer_Scope_2673 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Array_Access;
Var_Expr_3 : Internal_Entity_Accept_Stmt;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Accept_Stmt_Array_Access;
Map_Result_1 : Internal_Entity_Accept_Stmt_Array_Access;
Scope_Result : Internal_Entity_Accept_Stmt_Array_Access;
Result_Var : Internal_Entity_Accept_Stmt_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt;
Array_Lit : Internal_Entity_Accept_Stmt_Array_Access;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Accept_Stmt_Array_Access;
Scope_Result_1 : Internal_Entity_Accept_Stmt_Array_Access;
Result_Var_1 : Internal_Entity_Accept_Stmt_Array_Access;
Concat_Result : Internal_Entity_Accept_Stmt_Array_Access;
            procedure Finalizer_Scope_846 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_846;
            procedure Finalizer_Scope_2670 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2670;
            procedure Finalizer_Scope_2672 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2672;
            procedure Finalizer_Scope_2673 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2673;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5219 '<children at nodes.lkt:7757:79>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7757'
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 5219
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind child Var_Expr_0
--# expr-start 5224 '<Mapcat at nodes.lkt:7758:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7758'
   --# expr-start 5222 '<Map at nodes.lkt:7758:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7758'
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
         --# expr-start 5221 '<Not at nodes.lkt:7758:40>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7758'
--# expr-start 5220 '<IsNull at nodes.lkt:7758:44>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7758'
Is_Null := Item_1.Node = null; 
--# expr-done 5220
Not_Val := not (Is_Null); 
--# expr-done 5221
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5222
      declare
         Map_Result_1_Vec : Internal_Entity_Accept_Stmt_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 5223 '<FieldAccess for find_accept_stmts at nodes.lkt:7758:69>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7758'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.find_accept_stmts
Fld_1 := Libadalang.Implementation.Entry_Decl_P_Find_Accept_Stmts (Node => Ent.Node, Root => Item_2, E_Info => Ent.Info);
--# end
--# expr-done 5223
         for Item_To_Append of
               Fld_1.Items
         loop
            Internal_Entity_Accept_Stmt_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2672;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Accept_Stmt_Array
           (Items_Count => Natural (Internal_Entity_Accept_Stmt_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Accept_Stmt_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Accept_Stmt_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Accept_Stmt_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 5224
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2670;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 5225 '<Cast to AcceptStmt at nodes.lkt:7759:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7759'
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Accept_Stmt;
   end if;
--# expr-done 5225
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Entity_Accept_Stmt then
   --# scope-start
   --# bind stmt Var_Expr_3
--# expr-start 5229 '<If at nodes.lkt:7760:19>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7760'
--# expr-start 5227 '<Eq at nodes.lkt:7760:22>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7760'
--# expr-start 5226 '<FieldAccess for corresponding_entry at nodes.lkt:7760:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7760'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.corresponding_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Corresponding_Entry (Node => Var_Expr_3.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 5226
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 5227
if Is_Equiv then
   --# expr-start 5228 '<ArrayLiteral at nodes.lkt:7760:62>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7760'
Array_Lit := Create_Internal_Entity_Accept_Stmt_Array (Internal_Internal_Entity_Accept_Stmt_Array'(1 => Var_Expr_3)); 
--# expr-done 5228
   If_Result := Array_Lit;
else
   If_Result := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 5229
Scope_Result_1 := If_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2673;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Accept_Stmt_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result := Concat (Result_Var, Result_Var_1); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_846;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_846;
                     Finalizer_Scope_2670;
                     Finalizer_Scope_2672;
                     Finalizer_Scope_2673;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Find_Accept_Stmts;
--# end
--# property-start EntryDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7767'
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Next_Part_For_Decl
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_847 with Inline_Always;
      Var_Expr_154 : Internal_Entity_Accept_Stmt;
Fld : Internal_Entity_Accept_Stmt_Array_Access;
Get_Result : Internal_Entity_Accept_Stmt;
Fld_1 : Internal_Entity_Accept_Stmt_Body;
Scope_Result : Internal_Entity_Accept_Stmt_Body;
Result_Var : Internal_Entity_Accept_Stmt_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_847 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_847;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5236 '<If at nodes.lkt:7769:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
--# expr-start 5231 '<IsA SingleTaskTypeDecl, TaskTypeDecl at nodes.lkt:7769:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
--# expr-start 5230 '<FieldAccess for parent_basic_decl at nodes.lkt:7769:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5230
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Single_Task_Type_Decl_Range | Ada_Task_Type_Decl_Range; 
--# expr-done 5231
if Is_A then
--# expr-start 5233 '<at at nodes.lkt:7769:79>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
--# expr-start 5232 '<FieldAccess for accept_stmts at nodes.lkt:7769:79>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.accept_stmts
Fld := Libadalang.Implementation.Entry_Decl_P_Accept_Stmts (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5232
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 5233
Var_Expr_154 := Get_Result;
if Var_Expr_154 /= No_Entity_Accept_Stmt then
   --# scope-start
--# expr-start 5234 '<FieldAccess for body_decl at nodes.lkt:7769:79>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7769'
      if Var_Expr_154.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Accept_Stmt_Body (Node => Var_Expr_154.Node.Accept_Stmt_F_Body_Decl, Info => Var_Expr_154.Info);
--# expr-done 5234
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Accept_Stmt_Body;
end if;
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result := Cast_Result;
else
   --# expr-start 5235 '<Super at nodes.lkt:7771:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7771'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Subp_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5235
   If_Result := Fld_2;
end if;
--# expr-done 5236
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_847;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_847;
            raise;
      end;
   return Property_Result;
end Entry_Decl_P_Next_Part_For_Decl;
--# end
--# property-start EntryDecl._env_mappings_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7775'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_47
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
     Ent : Internal_Entity_Entry_Decl :=
       Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1711 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1711 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1711;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5239 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 5237 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5237
--# expr-start 5238 '<FieldAccess for name_symbol at nodes.lkt:7775:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7775'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5238
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5239
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1711;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1711;
            raise;
      end;
   return Property_Result;
end Env_Mappings_47;
--# end
--# property-start EntryDecl._env_trans_parent_48 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_48
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl  := Bare_Entry_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_48;
--# end
   --
   --  Primitives for Bare_Enum_Literal_Decl
   --
      procedure Enum_Literal_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Literal_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_49 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:7832:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_50 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Enum_Literal_Decl
        (Self : Bare_Enum_Literal_Decl
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Enum_Literal_Decl_F_Name := Enum_Literal_Decl_F_Name;
      end Initialize_Fields_For_Enum_Literal_Decl;
   function Enum_Literal_Decl_F_Name
     (Node : Bare_Enum_Literal_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Enum_Literal_Decl_F_Name;
   end;
--# property-start EnumLiteralDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7786'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Static_Decl
  (Node : Bare_Enum_Literal_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Is_Static_Decl;
--# end
--# property-start EnumLiteralDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7788'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Constant_Object
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Is_Constant_Object;
--# end
--# property-start EnumLiteralDecl.enum_rep 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7795'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Rep
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_850 with Inline_Always;
      Local_Enum_Type : Internal_Entity_Type_Decl;
Local_Rep_Clause : Internal_Entity_Enum_Rep_Clause;
Local_Pos : Integer;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Enum_Rep_Clause;
Fld_2 : Integer;
Big_Int : Big_Integer_Type;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Big_Integer_Type;
Is_Null_1 : Boolean;
If_Result_1 : Big_Integer_Type;
Let_Result : Big_Integer_Type;
Let_Result_1 : Big_Integer_Type;
Let_Result_2 : Big_Integer_Type;
            procedure Finalizer_Scope_850 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_850;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5252 '<Let at nodes.lkt:7796:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7796'
--# scope-start
--# expr-start 5240 '<FieldAccess for enum_type at nodes.lkt:7796:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7796'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5240
Local_Enum_Type := Fld; 
--# bind enum_type Local_Enum_Type
--# expr-start 5251 '<Let at nodes.lkt:7797:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7797'
--# scope-start
--# expr-start 5241 '<FieldAccess for get_enum_representation_clause at nodes.lkt:7797:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7797'
      if Local_Enum_Type.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.get_enum_representation_clause
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Get_Enum_Representation_Clause (Node => Local_Enum_Type.Node, Imprecise_Fallback => False, E_Info => Local_Enum_Type.Info);
--# end
--# expr-done 5241
Local_Rep_Clause := Fld_1; 
--# bind rep_clause Local_Rep_Clause
--# expr-start 5250 '<Let at nodes.lkt:7800:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7800'
--# scope-start
--# expr-start 5242 '<FieldAccess for child_index at nodes.lkt:7800:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7800'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 5242
Local_Pos := Fld_2; 
--# bind pos Local_Pos
--# expr-start 5249 '<If at nodes.lkt:7802:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7802'
--# expr-start 5243 '<IsNull at nodes.lkt:7802:12>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7802'
Is_Null_1 := Local_Rep_Clause.Node = null; 
--# expr-done 5243
if Is_Null_1 then
   --# expr-start 5244 '<BigInteger <Ref for <Var pos at nodes.lkt:7800:9>> at nodes.lkt:7805:13>' Big_Int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7805'
Big_Int := Create_Big_Integer (Local_Pos); 
--# expr-done 5244
   If_Result_1 := Big_Int;
else
   --# expr-start 5248 '<FieldAccess for eval_as_int at nodes.lkt:7811:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7811'
--# expr-start 5247 '<FieldAccess for expr at nodes.lkt:7811:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7811'
--# expr-start 5246 '<FieldAccess for assocs at nodes.lkt:7811:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7811'
--# expr-start 5245 '<FieldAccess for aggregate at nodes.lkt:7811:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7811'
      if Local_Rep_Clause.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Aggregate (Node => Local_Rep_Clause.Node.Enum_Rep_Clause_F_Aggregate, Info => Local_Rep_Clause.Info);
--# expr-done 5245
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Fld_3.Node.Base_Aggregate_F_Assocs, Info => Fld_3.Info);
--# expr-done 5246
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, Local_Pos, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5247
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_8 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 5248
   If_Result_1 := Fld_8;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5249
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5250
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 5251
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 5252
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_850;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_850;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Rep;
--# end
--# property-start EnumLiteralDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7815'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Defining_Names
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_851 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_851 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_851;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5254 '<ArrayLiteral at nodes.lkt:7815:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7815'
--# expr-start 5253 '<FieldAccess for name at nodes.lkt:7815:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7815'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 5253
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5254
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_851;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_851;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Defining_Names;
--# end
--# property-start EnumLiteralDecl.synth_type_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7818'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Synth_Type_Expr
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Lit_Synth_Type_Expr
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      New_Node : Bare_Enum_Lit_Synth_Type_Expr;
As_Entity : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Synth_Type_Expr,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Enum_Lit_Synth_Type_Expr;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5256 '<as_entity at nodes.lkt:7819:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7819'
--# expr-start 5255 '<New EnumLitSynthTypeExpr at nodes.lkt:7819:9>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7819'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Lit_Synth_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Lit_Synth_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 5255
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Lit_Synth_Type_Expr;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 5256
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Enum_Lit_Synth_Type_Expr,
                        As_Internal_Entity_Enum_Lit_Synth_Type_Expr => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Enum_Literal_Decl_P_Synth_Type_Expr;
--# end
--# property-start EnumLiteralDecl.subp_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7822'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Subp_Decl_Spec
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
      New_Node : Bare_Enum_Subp_Spec;
As_Entity : Internal_Entity_Enum_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Subp_Decl_Spec,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Subp_Spec;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 5258 '<as_entity at nodes.lkt:7822:50>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7822'
--# expr-start 5257 '<New EnumSubpSpec at nodes.lkt:7822:50>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7822'
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Subp_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Subp_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 5257
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Subp_Spec;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 5258
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => If_Result.Node,
         Info => If_Result.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Subp_Spec,
                        As_Internal_Entity_Base_Subp_Spec => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Enum_Literal_Decl_P_Subp_Decl_Spec;
--# end
--# property-start '[dispatcher]EnumLiteralDecl.enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Enum_Literal_Decl_Range (Self.Kind) is
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.enum_type
                  Property_Result := Enum_Literal_Decl_P_Enum_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.enum_type
                  Property_Result := Synthetic_Char_Enum_Lit_P_Enum_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start '[dispatcher]EnumLiteralDecl.enum_decl_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Enum_Literal_Decl_Range (Self.Kind) is
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.enum_decl_name
                  Property_Result := Enum_Literal_Decl_P_Enum_Decl_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Char_Enum_Lit =>
                  --# property-call-start SyntheticCharEnumLit.enum_decl_name
                  Property_Result := Synthetic_Char_Enum_Lit_P_Enum_Decl_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Enum_Literal_Decl_P_Enum_Decl_Name;
--# end
--# property-start EnumLiteralDecl._env_mappings_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7832'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_49
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1713 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Enum_Literal_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1713 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1713;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5263 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 5261 '<New DesignatedEnv at nodes.lkt:7833:48>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7833'
--# expr-start 5260 '<FieldAccess for node_env at nodes.lkt:7834:86>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7834'
--# expr-start 5259 '<FieldAccess for enum_type at nodes.lkt:7834:86>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7834'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5259
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 5260
Inc_Ref (Fld_2);
New_Struct := (Direct_Env => Fld_2, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 5261
--# expr-start 5262 '<FieldAccess for name_symbol at nodes.lkt:7833:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7833'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5262
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5263
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1713;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1713;
            raise;
      end;
   return Property_Result;
end Env_Mappings_49;
--# end
--# property-start EnumLiteralDecl._env_trans_parent_50 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_50
  (Node : Bare_Enum_Literal_Decl
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_50;
--# end
--# property-start EnumLiteralDecl.enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7826'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_854 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_854 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_854;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5268 '<Cast to TypeDecl at nodes.lkt:7827:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7827'
--# expr-start 5267 '<as_entity at nodes.lkt:7827:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7827'
--# expr-start 5266 '<Find at nodes.lkt:7827:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7827'
   --# expr-start 5264 '<FieldAccess for parents at nodes.lkt:7827:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7827'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 5264
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 5265 '<IsA TypeDecl at nodes.lkt:7827:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7827'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Type_Decl; 
--# expr-done 5265
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 5266
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 5267
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 5268
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_854;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_854;
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start EnumLiteralDecl.enum_decl_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7829'
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Decl_Name
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Enum_Literal_Decl  := Bare_Enum_Literal_Decl (Node);
     Ent : Internal_Entity_Enum_Literal_Decl :=
       Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5269 '<FieldAccess for name at nodes.lkt:7829:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7829'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 5269
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Literal_Decl_P_Enum_Decl_Name;
--# end
   --
   --  Primitives for Bare_Synthetic_Char_Enum_Lit
   --
      procedure Initialize_Fields_For_Synthetic_Char_Enum_Lit
        (Self : Bare_Synthetic_Char_Enum_Lit
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Enum_Literal_Decl
              (Self, Enum_Literal_Decl_F_Name);
      Self.Synthetic_Char_Enum_Lit_Char_Symbol := No_Symbol;
      Self.Synthetic_Char_Enum_Lit_Enum_Type_Decl := No_Entity_Type_Decl;
      end Initialize_Fields_For_Synthetic_Char_Enum_Lit;
--# property-start SyntheticCharEnumLit.enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7850'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Enum_Type
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5270 '<FieldAccess for enum_type_decl at nodes.lkt:7850:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7850'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Ent.Node.Synthetic_Char_Enum_Lit_Enum_Type_Decl;
--# expr-done 5270
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Enum_Type;
--# end
--# property-start SyntheticCharEnumLit.expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7854'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Expr
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_857 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_857 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_857;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5272 '<at at nodes.lkt:7854:40>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7854'
--# expr-start 5271 '<FieldAccess for defining_names at nodes.lkt:7854:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7854'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5271
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 5272
         Property_Result := Get_Result;
   --# end
      Finalizer_Scope_857;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_857;
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Expr;
--# end
--# property-start SyntheticCharEnumLit.enum_decl_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7856'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Enum_Decl_Name
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
     Ent : Internal_Entity_Synthetic_Char_Enum_Lit :=
       Internal_Entity_Synthetic_Char_Enum_Lit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5273 '<FieldAccess for expr at nodes.lkt:7856:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7856'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SyntheticCharEnumLit.expr
Fld := Libadalang.Implementation.Synthetic_Char_Enum_Lit_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5273
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Enum_Decl_Name;
--# end
--# property-start SyntheticCharEnumLit.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7858'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Char_Enum_Lit_P_Defining_Names
  (Node : Bare_Synthetic_Char_Enum_Lit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Synthetic_Char_Enum_Lit  := Bare_Synthetic_Char_Enum_Lit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_859 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_859 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_859;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5277 '<ArrayLiteral at nodes.lkt:7859:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7859'
--# expr-start 5276 '<as_entity at nodes.lkt:7859:10>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7859'
--# expr-start 5275 '<FieldAccess for synthesize_defining_name at nodes.lkt:7859:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7859'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5274 '<FieldAccess for char_symbol at nodes.lkt:7859:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7859'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Char_Enum_Lit_Char_Symbol;
--# expr-done 5274
--# property-call-start AdaNode.synthesize_defining_name
Fld_1 := Libadalang.Implementation.Ada_Node_P_Synthesize_Defining_Name (Node => Self, Sym => Fld);
--# end
--# expr-done 5275
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5276
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 5277
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_859;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_859;
            raise;
      end;
   return Property_Result;
end Synthetic_Char_Enum_Lit_P_Defining_Names;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Internal
   --
      procedure Generic_Subp_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_51 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_52 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Internal
        (Self : Bare_Generic_Subp_Internal
         ; Generic_Subp_Internal_F_Subp_Spec : Bare_Subp_Spec
         ; Generic_Subp_Internal_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Internal_F_Subp_Spec := Generic_Subp_Internal_F_Subp_Spec;
            Self.Generic_Subp_Internal_F_Aspects := Generic_Subp_Internal_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Internal;
   function Generic_Subp_Internal_F_Subp_Spec
     (Node : Bare_Generic_Subp_Internal) return Bare_Subp_Spec
   is
   begin
         return Node.Generic_Subp_Internal_F_Subp_Spec;
   end;
--# property-start GenericSubpInternal.subp_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7867'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Internal_P_Subp_Decl_Spec
  (Node : Bare_Generic_Subp_Internal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
     Ent : Internal_Entity_Generic_Subp_Internal :=
       Internal_Entity_Generic_Subp_Internal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5278 '<FieldAccess for subp_spec at nodes.lkt:7867:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7867'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5278
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Internal_P_Subp_Decl_Spec;
--# end
--# property-start GenericSubpInternal._env_trans_parent_51 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_51
  (Node : Bare_Generic_Subp_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_51;
--# end
--# property-start GenericSubpInternal._env_names_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7870'
pragma Warnings (Off, "is not referenced");
 function Env_Names_52
  (Node : Bare_Generic_Subp_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Subp_Internal  := Bare_Generic_Subp_Internal (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1716 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1716 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1716;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5279 '<FieldAccess for env_names at nodes.lkt:7870:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7870'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 5279
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1716;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1716;
            raise;
      end;
   return Property_Result;
end Env_Names_52;
--# end
   --
   --  Primitives for Bare_Synthetic_Subp_Decl
   --
      procedure Initialize_Fields_For_Synthetic_Subp_Decl
        (Self : Bare_Synthetic_Subp_Decl
         ; Synthetic_Subp_Decl_F_Spec : Bare_Base_Subp_Spec
        ) is
      begin
            Self.Synthetic_Subp_Decl_F_Spec := Synthetic_Subp_Decl_F_Spec;
      end Initialize_Fields_For_Synthetic_Subp_Decl;
   function Synthetic_Subp_Decl_F_Spec
     (Node : Bare_Synthetic_Subp_Decl) return Bare_Base_Subp_Spec
   is
   begin
         return Node.Synthetic_Subp_Decl_F_Spec;
   end;
--# property-start SyntheticSubpDecl.subp_decl_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7883'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Synthetic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
     Ent : Internal_Entity_Synthetic_Subp_Decl :=
       Internal_Entity_Synthetic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5280 '<FieldAccess for spec at nodes.lkt:7883:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7883'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Subp_Spec (Node => Ent.Node.Synthetic_Subp_Decl_F_Spec, Info => Ent.Info);
--# expr-done 5280
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start SyntheticSubpDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7886'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Synthetic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Basic_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start SyntheticSubpDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7889'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Subp_Decl_P_Is_Static_Decl
  (Node : Bare_Synthetic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Synthetic_Subp_Decl  := Bare_Synthetic_Subp_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Subp_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Body_Node
   --
--# property-start Body.env_hook_body 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7897'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Env_Hook_Body
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_864 with Inline_Always;
      Ignored : Internal_Unit;
Var_Expr_0 : Bare_Subp_Body;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Fld_2 : Internal_Unit;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Subp_Body;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
            procedure Finalizer_Scope_864 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_864;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5291 '<If at nodes.lkt:7900:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7900'
--# expr-start 5281 '<IsA PackageBody, SubpBody at nodes.lkt:7900:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7900'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Package_Body_Range | Ada_Subp_Body_Range; 
--# expr-done 5281
if Is_A_1 then
   --# expr-start 5290 '<Let at nodes.lkt:7901:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7901'
--# scope-start
--# expr-start 5287 '<FieldAccess for get_unit at nodes.lkt:7901:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7901'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5284 '<FieldAccess for as_symbol_array at nodes.lkt:7902:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7902'
--# expr-start 5283 '<FieldAccess for defining_name at nodes.lkt:7902:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7902'
--# expr-start 5282 '<as_bare_entity at nodes.lkt:7902:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7902'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5282
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5283
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 5284
--# expr-start 5286 '<Not at nodes.lkt:7902:149>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7902'
--# expr-start 5285 '<IsA SubpBody at nodes.lkt:7902:153>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7902'
Is_A := Self /= null 
and then Self.Kind in Ada_Subp_Body_Range; 
--# expr-done 5285
Not_Val := not (Is_A); 
--# expr-done 5286
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification, Load_If_Needed => True, Not_Found_Is_Error => Not_Val);
--# expr-done 5287
Ignored := Fld_2; 
--# bind _ Ignored
--# expr-start 5288 '<Cast to SubpBody at nodes.lkt:7907:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7907'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5288
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp_body Var_Expr_0
--# expr-start 5289 '<FieldAccess for env_hook_basic_decl at nodes.lkt:7908:32>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7908'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Var_Expr_0);
--# end
--# expr-done 5289
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5290
   If_Result_1 := Let_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 5291
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_864;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_864;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Env_Hook_Body;
--# end
--# property-start Body.subunit_decl_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7912'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Decl_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_865 with Inline_Always;
            procedure Finalizer_Scope_2677 with Inline_Always;
            procedure Finalizer_Scope_2678 with Inline_Always;
            procedure Finalizer_Scope_2680 with Inline_Always;
            procedure Finalizer_Scope_2681 with Inline_Always;
      Match_0 : Internal_Entity_Generic_Package_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_0 : Lexical_Env := Empty_Env;
Var_Expr_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Generic_Package_Internal;
Fld_5 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_865 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_865;
            procedure Finalizer_Scope_2677 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2677;
            procedure Finalizer_Scope_2678 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2678;
            procedure Finalizer_Scope_2680 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2680;
            procedure Finalizer_Scope_2681 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_2681;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5310 '<bind at nodes.lkt:7913:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7913'
--# expr-start 5292 '<FieldAccess for default_initial_env at nodes.lkt:7913:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7913'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5292
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5303 '<Match at nodes.lkt:7915:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7915'
--# expr-start 5296 '<at at nodes.lkt:7915:15>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7915'
--# expr-start 5295 '<EnvGet at nodes.lkt:7915:15>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7915'
--# expr-start 5293 '<FieldAccess for body_scope at nodes.lkt:7915:15>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7915'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5293
--# expr-start 5294 '<FieldAccess for name_symbol at nodes.lkt:7916:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7916'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5294
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Thin (Fld_2), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5295
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 5296
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gpd Match_0
--# expr-start 5301 '<env_group at nodes.lkt:7922:13>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7922'
--# expr-start 5300 '<ArrayLiteral at nodes.lkt:7922:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7922'
--# expr-start 5297 '<FieldAccess for children_env at nodes.lkt:7922:14>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7922'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 5297
--# expr-start 5299 '<FieldAccess for children_env at nodes.lkt:7922:34>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7922'
--# expr-start 5298 '<FieldAccess for package_decl at nodes.lkt:7922:34>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7922'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_0.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_0.Info);
--# expr-done 5298
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 5299
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_5)); 
--# expr-done 5300
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5301
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2677;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pd Match_1
--# expr-start 5302 '<FieldAccess for children_env at nodes.lkt:7923:35>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7923'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 5302
Let_Result_1 := Fld_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2678;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "PropertyError exception"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 5303
Var_Expr_0 := Match_Result;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_Env then
   --# scope-start
   --# bind public_part Var_Expr_0
--# expr-start 5306 '<at at nodes.lkt:7926:30>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7926'
--# expr-start 5305 '<EnvGet at nodes.lkt:7926:30>' Env_Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7926'
--# expr-start 5304 '<Symbol __privatepart at nodes.lkt:7927:17>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7927'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 5304
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Var_Expr_0, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5305
Get_Result_1 := Get (Self, Env_Get_Result_1, 0, True); 
--# expr-done 5306
Var_Expr_1 := Get_Result_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 5309 '<env_group at nodes.lkt:7930:25>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7930'
--# expr-start 5308 '<ArrayLiteral at nodes.lkt:7930:25>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7930'
--# expr-start 5307 '<FieldAccess for children_env at nodes.lkt:7930:26>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7930'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 5307
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_7, 2 => Var_Expr_0)); 
--# expr-done 5308
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 5309
Scope_Result_3 := Group_Env_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2681;
   Result_Var := Scope_Result_3;
else
   Result_Var := Var_Expr_0;
end if;
      Inc_Ref (Result_Var);
Scope_Result_4 := Result_Var;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2680;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Dyn_Var_Bind_Result := Result_Var_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5310
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_865;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_865;
                     Finalizer_Scope_2677;
                     Finalizer_Scope_2678;
                     Finalizer_Scope_2680;
                     Finalizer_Scope_2681;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subunit_Decl_Env;
--# end
--# property-start Body.in_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7937'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_In_Scope
  (Node : Bare_Body_Node
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_866 with Inline_Always;
            procedure Finalizer_Scope_2683 with Inline_Always;
      Var_Expr_1 : Bare_Subunit;
Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Unit;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_2 : Bare_Compilation_Unit;
Fld_3 : Bare_Ada_Node;
Cast_Result_1 : Bare_Subunit;
Fld_4 : Internal_Entity_Basic_Decl_Array_Access;
Is_A : Boolean;
Fld_5 : Internal_Unit;
Fld_6 : Internal_Unit;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Quantifier_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
            procedure Finalizer_Scope_866 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_866;
            procedure Finalizer_Scope_2683 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2683;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5312 '<Not at nodes.lkt:7937:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7937'
--# expr-start 5311 '<IsNull at nodes.lkt:7937:32>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7937'
Is_Null := Origin = null; 
--# expr-done 5311
Not_Val := not (Is_Null); 
--# expr-done 5312
if Not_Val then
--# expr-start 5315 '<Eq at nodes.lkt:7940:13>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
--# expr-start 5313 '<FieldAccess for unit at nodes.lkt:7940:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Unit (Node => Origin);
--# expr-done 5313
--# expr-start 5314 '<FieldAccess for unit at nodes.lkt:7940:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5314
Is_Equal := Fld = Fld_1; 
--# expr-done 5315
if Is_Equal then
   --# expr-start 5320 '<Not at nodes.lkt:7940:46>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
--# expr-start 5319 '<IsNull at nodes.lkt:7940:50>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
--# expr-start 5318 '<Find at nodes.lkt:7940:50>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
   --# expr-start 5316 '<FieldAccess for parents at nodes.lkt:7940:50>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Origin, True); 
--# expr-done 5316
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 5317 '<Eq at nodes.lkt:7940:79>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7940'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Cast_Result = Item_0; 
--# expr-done 5317
      if Is_Equal_1 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_Equal_1;
         end loop;
      end;
--# expr-done 5318
Is_Null_1 := Find_Result = null; 
--# expr-done 5319
Not_Val_1 := not (Is_Null_1); 
--# expr-done 5320
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 5323 '<Cast to Subunit at nodes.lkt:7943:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7943'
--# expr-start 5322 '<FieldAccess for body at nodes.lkt:7943:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7943'
--# expr-start 5321 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:7943:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7943'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Origin);
--# end
--# expr-done 5321
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Fld_2.Compilation_Unit_F_Body;
--# expr-done 5322
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Subunit_Range
   then
      Cast_Result_1 := Fld_3;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5323
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind su Var_Expr_1
--# expr-start 5329 '<AnyQuantifier at nodes.lkt:7944:25>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7944'
   --# expr-start 5324 '<FieldAccess for bodies_root at nodes.lkt:7944:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7944'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.bodies_root
Fld_4 := Libadalang.Implementation.Subunit_P_Bodies_Root (Node => Var_Expr_1);
--# end
--# expr-done 5324
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_4;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind br Item_2
--# expr-start 5325 '<IsA BaseSubpBody at nodes.lkt:7945:29>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7945'
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 5325
if Is_A then
   --# expr-start 5328 '<Eq at nodes.lkt:7945:52>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7945'
--# expr-start 5326 '<FieldAccess for unit at nodes.lkt:7945:52>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7945'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Unit (Node => Item_2.Node);
--# expr-done 5326
--# expr-start 5327 '<FieldAccess for unit at nodes.lkt:7945:65>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7945'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5327
Is_Equal_2 := Fld_5 = Fld_6; 
--# expr-done 5328
   If_Result_1 := Is_Equal_2;
else
   If_Result_1 := False;
end if;
      Quantifier_Result := If_Result_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 5329
Scope_Result := Quantifier_Result;
   --# end
      Finalizer_Scope_2683;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_2 := Result_Var;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
         Property_Result := If_Result_3;
   --# end
      Finalizer_Scope_866;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_866;
                     Finalizer_Scope_2683;
            raise;
      end;
   return Property_Result;
end Body_Node_P_In_Scope;
--# end
--# property-start Body.body_decl_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7952'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Decl_Scope
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_867 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_867 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_867;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5332 '<bind at nodes.lkt:7953:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7953'
--# expr-start 5330 '<FieldAccess for default_initial_env at nodes.lkt:7953:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7953'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5330
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5331 '<FieldAccess for body_scope at nodes.lkt:7955:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7955'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5331
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5332
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_867;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_867;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Decl_Scope;
--# end
--# property-start Body.body_initial_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7961'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Initial_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_868 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_1 : String_Type;
Str_1 : String_Type;
Concat_Result_1 : String_Type;
Sym_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Fld_3 : Boolean;
If_Result_1 : Symbol_Type;
            procedure Finalizer_Scope_868 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_868;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5342 '<If at nodes.lkt:7961:43>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7961'
--# expr-start 5333 '<FieldAccess for is_library_item at nodes.lkt:7961:46>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7961'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5333
if Fld_3 then
   --# expr-start 5336 '<to_symbol at nodes.lkt:7961:74>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7961'
--# expr-start 5334 '<FieldAccess for top_level_env_name at nodes.lkt:7962:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7962'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5334
--# expr-start 5335 '<String at nodes.lkt:7962:37>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7962'
Str := Create_String (".__privatepart"); 
--# expr-done 5335
Concat_Result := Concat_String (Fld, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 5336
   If_Result_1 := Sym;
else
   --# expr-start 5341 '<If at nodes.lkt:7961:43>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7961'
--# expr-start 5337 '<FieldAccess for is_subunit at nodes.lkt:7964:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7964'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5337
if Fld_2 then
   --# expr-start 5340 '<to_symbol at nodes.lkt:7964:33>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7964'
--# expr-start 5338 '<FieldAccess for top_level_env_name at nodes.lkt:7964:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7964'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5338
--# expr-start 5339 '<String at nodes.lkt:7964:62>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7964'
Str_1 := Create_String ("__stub"); 
--# expr-done 5339
Concat_Result_1 := Concat_String (Fld_1, Str_1); 
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Concat_Result_1); 
--# expr-done 5340
   If_Result := Sym_1;
else
   If_Result := No_Symbol;
end if;
--# expr-done 5341
   If_Result_1 := If_Result;
end if;
--# expr-done 5342
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_868;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_868;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Initial_Env_Name;
--# end
--# property-start Body.body_initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7970'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Initial_Env
  (Node : Bare_Body_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_869 with Inline_Always;
            procedure Finalizer_Scope_2685 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_869 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_869;
            procedure Finalizer_Scope_2685 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2685;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5343 '<FieldAccess for body_initial_env_name at nodes.lkt:7970:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7970'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env_name
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 5343
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind non_null_name Var_Expr_0
--# expr-start 5344 '<New DesignatedEnv at nodes.lkt:7971:28>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7971'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 5344
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2685;
   Result_Var := Scope_Result;
else
   --# expr-start 5345 '<New DesignatedEnv at nodes.lkt:7973:24>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7973'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5345
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_869;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_869;
                     Finalizer_Scope_2685;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Initial_Env;
--# end
--# property-start Body.subp_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7986'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subp_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_870 with Inline_Always;
            procedure Finalizer_Scope_2686 with Inline_Always;
      Local_Parent : Internal_Entity;
Local_Elements : Internal_Entity_Array_Access;
Var_Expr_0 : Internal_Entity_Compilation_Unit;
Local_Already_Precise : Boolean;
Local_Precise : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Generic_Subp_Decl;
Match_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Basic_Subp_Decl;
Match_3 : Internal_Entity_Subp_Body_Stub;
Match_4 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Fld_3 : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Internal_Entity_Compilation_Unit;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Array_Lit : Internal_Entity_Array_Access;
Fld_6 : Bare_Basic_Decl;
Is_A : Boolean;
If_Result_2 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Array_Lit_1 : Internal_Entity_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
If_Result_3 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
If_Result_4 : Internal_Entity_Array_Access;
Is_Null_1 : Boolean;
If_Result_5 : Internal_Entity_Array_Access;
Is_A_3 : Boolean;
Get_Result : Internal_Entity;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_9 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result_6 : Boolean;
Fld_10 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Subp_Spec;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Fld_13 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_14 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity_Subp_Spec;
Fld_16 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Subp_Spec;
Cast_Result_6 : Internal_Entity_Base_Subp_Spec;
Fld_17 : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_7 : Internal_Entity_Generic_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_8 : Internal_Entity_Formal_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Subp_Decl;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_10 : Internal_Entity_Subp_Body_Stub;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_11 : Internal_Entity;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
If_Result_7 : Boolean;
Find_Result : Internal_Entity;
If_Result_8 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
Not_Val_2 : Boolean;
Fld_18 : Bare_Ada_Node;
Cast_Expr_2 : Bare_Body_Node;
Cast_Result_13 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_3 : Boolean;
If_Result_9 : Boolean;
Is_A_4 : Boolean;
If_Result_10 : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result_14 : Internal_Entity_Basic_Decl;
Is_Null_4 : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_7 : Internal_Entity_Basic_Decl;
Let_Result_8 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_870 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Local_Elements);
            end Finalizer_Scope_870;
            procedure Finalizer_Scope_2686 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2686;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5406 '<Let at nodes.lkt:7987:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
--# scope-start
--# expr-start 5350 '<If at nodes.lkt:7987:22>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
--# expr-start 5346 '<FieldAccess for is_library_item at nodes.lkt:7987:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Ent.Node);
--# end
--# expr-done 5346
if Fld_3 then
   --# expr-start 5348 '<parent at nodes.lkt:7987:53>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
--# expr-start 5347 '<parent at nodes.lkt:7987:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5347
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 5348
   If_Result := Fld_1;
else
   --# expr-start 5349 '<FieldAccess for semantic_parent at nodes.lkt:7987:77>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5349
   If_Result := Fld_2;
end if;
--# expr-done 5350
Local_Parent := If_Result; 
--# bind parent Local_Parent
--# expr-start 5405 '<Let at nodes.lkt:7988:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7988'
--# scope-start
--# expr-start 5369 '<If at nodes.lkt:7988:24>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7988'
--# expr-start 5351 '<IsNull at nodes.lkt:7988:27>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7988'
Is_Null_1 := Local_Parent.Node = null; 
--# expr-done 5351
if Is_Null_1 then
   If_Result_5 := No_Internal_Entity_Array_Type;
else
   --# expr-start 5368 '<If at nodes.lkt:7988:24>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7988'
--# expr-start 5352 '<IsA CompilationUnit at nodes.lkt:7991:14>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7991'
Is_A_2 := Local_Parent.Node /= null 
and then Local_Parent.Node.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 5352
if Is_A_2 then
--# expr-start 5354 '<FieldAccess for other_part at nodes.lkt:7991:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7991'
--# expr-start 5353 '<Cast to CompilationUnit at nodes.lkt:7991:45>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7991'
Cast_Expr := Local_Parent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 5353
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Cast_Result.Node);
--# end
--# expr-done 5354
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Compilation_Unit then
   --# scope-start
   --# bind cu Var_Expr_0
--# expr-start 5361 '<If at nodes.lkt:7994:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
--# expr-start 5356 '<IsA BasicSubpDecl, GenericSubpDecl at nodes.lkt:7994:24>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
--# expr-start 5355 '<FieldAccess for decl at nodes.lkt:7994:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_6 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_0.Node);
--# end
--# expr-done 5355
Is_A := Fld_6 /= null 
and then Fld_6.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Subp_Decl_Range; 
--# expr-done 5356
if Is_A then
   --# expr-start 5360 '<ArrayLiteral at nodes.lkt:7994:74>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
--# expr-start 5359 '<as_entity at nodes.lkt:7994:75>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
--# expr-start 5358 '<Cast to AdaNode at nodes.lkt:7994:75>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
--# expr-start 5357 '<FieldAccess for decl at nodes.lkt:7994:75>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7994'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_0.Node);
--# end
--# expr-done 5357
      Cast_Result_1 := Fld_5;
--# expr-done 5358
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 5359
Array_Lit := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => If_Result_1)); 
--# expr-done 5360
   If_Result_2 := Array_Lit;
else
   If_Result_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5361
Scope_Result := If_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2686;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
   If_Result_4 := Result_Var;
else
   --# expr-start 5367 '<If at nodes.lkt:7988:24>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7988'
--# expr-start 5362 '<IsA BodyStub at nodes.lkt:7998:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7998'
Is_A_1 := Local_Parent.Node /= null 
and then Local_Parent.Node.Kind in Ada_Body_Stub; 
--# expr-done 5362
if Is_A_1 then
   --# expr-start 5363 '<ArrayLiteral at nodes.lkt:7998:38>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:7998'
Array_Lit_1 := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => Local_Parent)); 
--# expr-done 5363
   If_Result_3 := Array_Lit_1;
else
   --# expr-start 5366 '<EnvGet at nodes.lkt:8000:14>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8000'
--# expr-start 5364 '<FieldAccess for immediate_declarative_region at nodes.lkt:8000:14>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8000'
      if Local_Parent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_7 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Local_Parent.Node, E_Info => Local_Parent.Info);
--# end
--# expr-done 5364
--# expr-start 5365 '<FieldAccess for name_symbol at nodes.lkt:8001:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8001'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5365
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_7, Key => Thin (Fld_8), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 5366
   If_Result_3 := Env_Get_Result;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 5367
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 5368
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 5369
Local_Elements := If_Result_5; Inc_Ref (Local_Elements);
--# bind elements Local_Elements
--# expr-start 5404 '<Let at nodes.lkt:8006:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8006'
--# scope-start
--# expr-start 5370 '<IsA CompilationUnit, BodyStub at nodes.lkt:8006:31>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8006'
Is_A_3 := Local_Parent.Node /= null 
and then Local_Parent.Node.Kind in Ada_Compilation_Unit_Range | Ada_Body_Stub; 
--# expr-done 5370
Local_Already_Precise := Is_A_3; 
--# bind already_precise Local_Already_Precise
--# expr-start 5403 '<Let at nodes.lkt:8007:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8007'
--# scope-start
--# expr-start 5392 '<Cast to BasicDecl at nodes.lkt:8007:23>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8007'
--# expr-start 5391 '<If at nodes.lkt:8008:13>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8008'
if Local_Already_Precise then
   --# expr-start 5371 '<at at nodes.lkt:8008:37>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8008'
Get_Result := Get (Self, Local_Elements, 0, True); 
--# expr-done 5371
   If_Result_8 := Get_Result;
else
   --# expr-start 5390 '<Find at nodes.lkt:8008:55>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8008'
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Elements;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_1
--# expr-start 5373 '<Not at nodes.lkt:8009:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
--# expr-start 5372 '<IsNull at nodes.lkt:8009:29>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
Is_Null_2 := Item_1.Node = null; 
--# expr-done 5372
Not_Val := not (Is_Null_2); 
--# expr-done 5373
if Not_Val then
   --# expr-start 5376 '<Not at nodes.lkt:8009:44>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
--# expr-start 5375 '<Eq at nodes.lkt:8009:44>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
--# expr-start 5374 '<FieldAccess for node at nodes.lkt:8009:44>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
Fld_9 := Item_1.Node;
--# expr-done 5374
Cast_Expr_1 := Self; 
      Cast_Result_2 := Cast_Expr_1;
Is_Equal := Fld_9 = Cast_Result_2; 
--# expr-done 5375
Not_Val_1 := not (Is_Equal); 
--# expr-done 5376
   If_Result_6 := Not_Val_1;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 5389 '<Match at nodes.lkt:8009:64>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8009'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_7; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_8; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Synthetic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_9; 
--# bind subp_decl Match_2
--# expr-start 5382 '<bind at nodes.lkt:8018:25>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8018'
--# expr-start 5377 '<FieldAccess for origin_node at nodes.lkt:8018:39>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8018'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_10 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5377
Bound_Origin := Fld_10; 
--# expr-start 5381 '<FieldAccess for match_signature at nodes.lkt:8020:25>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8020'
--# expr-start 5378 '<FieldAccess for subp_decl_spec at nodes.lkt:8020:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8020'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 5378
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5380 '<Cast to SubpSpec at nodes.lkt:8021:29>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8021'
--# expr-start 5379 '<FieldAccess for subp_spec_or_null at nodes.lkt:8021:29>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8021'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5379
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_3 := No_Entity_Subp_Spec;
   end if;
--# expr-done 5380
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_11.Node, Other => Cast_Result_4, Match_Name => True, Use_Entity_Info => False, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 5381
Dyn_Var_Bind_Result := Fld_13; 
--# expr-done 5382
Let_Result_2 := Dyn_Var_Bind_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_10; 
--# bind subp_stub Match_3
--# expr-start 5388 '<bind at nodes.lkt:8027:25>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8027'
--# expr-start 5383 '<FieldAccess for origin_node at nodes.lkt:8027:39>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8027'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_14 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5383
Bound_Origin_1 := Fld_14; 
--# expr-start 5387 '<FieldAccess for match_signature at nodes.lkt:8029:25>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8029'
--# expr-start 5384 '<FieldAccess for subp_spec at nodes.lkt:8029:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8029'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Subp_Spec (Node => Match_3.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_3.Info);
--# expr-done 5384
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5386 '<Cast to SubpSpec at nodes.lkt:8030:29>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8030'
--# expr-start 5385 '<FieldAccess for subp_spec_or_null at nodes.lkt:8030:29>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8030'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5385
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_5 := No_Entity_Subp_Spec;
   end if;
--# expr-done 5386
      Cast_Result_6 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_5.Node,
         Info => Cast_Result_5.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_17 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_15.Node, Other => Cast_Result_6, Match_Name => True, Use_Entity_Info => False, Origin => Bound_Origin_1, E_Info => Fld_15.Info);
--# end
--# expr-done 5387
Dyn_Var_Bind_Result_1 := Fld_17; 
--# expr-done 5388
Let_Result_3 := Dyn_Var_Bind_Result_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Formal_Type_Decl | Ada_Accept_Stmt_Body .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_11; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 5389
   If_Result_7 := Match_Result;
else
   If_Result_7 := False;
end if;
      if If_Result_7 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when If_Result_7;
         end loop;
      end;
--# expr-done 5390
   If_Result_8 := Find_Result;
end if;
--# expr-done 5391
   if If_Result_8.Node = null
      or else If_Result_8.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_8.Node,
         Info => If_Result_8.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5392
Local_Precise := Cast_Result_12; 
--# bind precise Local_Precise
--# expr-start 5402 '<If at nodes.lkt:8040:9>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8040'
--# expr-start 5393 '<IsNull at nodes.lkt:8040:12>' Is_Null_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8040'
Is_Null_4 := Local_Precise.Node = null; 
--# expr-done 5393
if Is_Null_4 then
   If_Result_11 := Imprecise_Fallback;
else
   If_Result_11 := False;
end if;
if If_Result_11 then
   --# expr-start 5401 '<Cast to BasicDecl at nodes.lkt:8040:56>' Cast_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8040'
--# expr-start 5400 '<Find at nodes.lkt:8040:56>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8040'
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Elements;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_2
--# expr-start 5395 '<Not at nodes.lkt:8041:21>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
--# expr-start 5394 '<IsNull at nodes.lkt:8041:25>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
Is_Null_3 := Item_2.Node = null; 
--# expr-done 5394
Not_Val_2 := not (Is_Null_3); 
--# expr-done 5395
if Not_Val_2 then
   --# expr-start 5398 '<Not at nodes.lkt:8041:40>' Not_Val_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
--# expr-start 5397 '<Eq at nodes.lkt:8041:40>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
--# expr-start 5396 '<FieldAccess for node at nodes.lkt:8041:40>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
Fld_18 := Item_2.Node;
--# expr-done 5396
Cast_Expr_2 := Self; 
      Cast_Result_13 := Cast_Expr_2;
Is_Equal_1 := Fld_18 = Cast_Result_13; 
--# expr-done 5397
Not_Val_3 := not (Is_Equal_1); 
--# expr-done 5398
   If_Result_9 := Not_Val_3;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
   --# expr-start 5399 '<IsA BasicSubpDecl, SubpBodyStub at nodes.lkt:8041:60>' Is_A_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8041'
Is_A_4 := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Stub_Range; 
--# expr-done 5399
   If_Result_10 := Is_A_4;
else
   If_Result_10 := False;
end if;
      if If_Result_10 then
         Find_Result_1 := Item_2; 
      end if;
   --# end
               exit when If_Result_10;
         end loop;
      end;
--# expr-done 5400
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_14 := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5401
   If_Result_12 := Cast_Result_14;
else
   If_Result_12 := Local_Precise;
end if;
--# expr-done 5402
Let_Result_5 := If_Result_12; 
--# end
--# expr-done 5403
Let_Result_6 := Let_Result_5; 
--# end
--# expr-done 5404
Let_Result_7 := Let_Result_6; 
--# end
--# expr-done 5405
Let_Result_8 := Let_Result_7; 
--# end
--# expr-done 5406
         Property_Result := Let_Result_8;
   --# end
      Finalizer_Scope_870;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_870;
                     Finalizer_Scope_2686;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subp_Previous_Part;
--# end
--# property-start Body.entry_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8047'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Entry_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_871 with Inline_Always;
      Local_Spec : Internal_Entity_Entry_Completion_Formal_Params;
Match_0 : Internal_Entity_Entry_Decl;
Match_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Fld : Internal_Entity_Entry_Completion_Formal_Params;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Entry_Spec;
Cast_Expr_2 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Entry_Decl;
Let_Result_2 : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_871 is
            begin
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_871;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5423 '<Let at nodes.lkt:8048:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8048'
--# scope-start
--# expr-start 5408 '<FieldAccess for params at nodes.lkt:8048:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8048'
--# expr-start 5407 '<Cast to EntryBody at nodes.lkt:8048:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8048'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5407
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Cast_Result.Node.Entry_Body_F_Params, Info => Cast_Result.Info);
--# expr-done 5408
Local_Spec := Fld; 
--# bind spec Local_Spec
--# expr-start 5422 '<Cast to EntryDecl at nodes.lkt:8050:9>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8050'
--# expr-start 5421 '<Find at nodes.lkt:8050:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8050'
   --# expr-start 5410 '<EnvGet at nodes.lkt:8050:9>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8050'
--# expr-start 5409 '<FieldAccess for name_symbol at nodes.lkt:8051:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8051'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5409
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Thin (Fld_1), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5410
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_0
--# expr-start 5412 '<Not at nodes.lkt:8053:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
--# expr-start 5411 '<IsNull at nodes.lkt:8053:25>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
Is_Null := Item_0.Node = null; 
--# expr-done 5411
Not_Val := not (Is_Null); 
--# expr-done 5412
if Not_Val then
   --# expr-start 5415 '<Not at nodes.lkt:8053:40>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
--# expr-start 5414 '<Eq at nodes.lkt:8053:40>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
--# expr-start 5413 '<FieldAccess for node at nodes.lkt:8053:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
Fld_2 := Item_0.Node;
--# expr-done 5413
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_2 = Cast_Result_1; 
--# expr-done 5414
Not_Val_1 := not (Is_Equal); 
--# expr-done 5415
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5420 '<Match at nodes.lkt:8053:60>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8053'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Entry_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind entry_decl Match_0
--# expr-start 5419 '<bind at nodes.lkt:8055:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8055'
--# expr-start 5416 '<FieldAccess for origin_node at nodes.lkt:8055:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8055'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5416
Bound_Origin := Fld_3; 
--# expr-start 5418 '<FieldAccess for match_formal_params at nodes.lkt:8057:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8057'
--# expr-start 5417 '<FieldAccess for spec at nodes.lkt:8057:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8057'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Entry_Spec (Node => Match_0.Node.Entry_Decl_F_Spec, Info => Match_0.Info);
--# expr-done 5417
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := Local_Spec; 
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_4.Node, Other => Cast_Result_2, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5418
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 5419
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5420
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
      if If_Result_1 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when If_Result_1;
         end loop;
      end;
--# expr-done 5421
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5422
Let_Result_2 := Cast_Result_5; 
--# end
--# expr-done 5423
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_871;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_871;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Entry_Previous_Part;
--# end
--# property-start Body.package_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8069'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Package_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_872 with Inline_Always;
            procedure Finalizer_Scope_2697 with Inline_Always;
            procedure Finalizer_Scope_2698 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Compilation_Unit;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Package_Decl;
Match_1 : Internal_Entity_Generic_Package_Decl;
Match_2 : Internal_Entity;
Fld : Bare_Compilation_Unit;
Fld_1 : Internal_Entity_Compilation_Unit;
Fld_2 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Array_Lit : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Scope_Result_1 : Internal_Entity_Array_Access;
Result_Var_1 : Internal_Entity_Array_Access;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Fld_6 : Boolean;
If_Result_1 : Internal_Entity_Array_Access;
Fld_7 : Internal_Entity_Generic_Package_Internal;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Decl;
Cast_Expr : Internal_Entity_Package_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Package_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Find_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_872 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_872;
            procedure Finalizer_Scope_2697 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2697;
            procedure Finalizer_Scope_2698 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2698;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5441 '<Find at nodes.lkt:8069:54>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8069'
   --# expr-start 5438 '<Map at nodes.lkt:8069:54>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8069'
   --# expr-start 5435 '<If at nodes.lkt:8070:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8070'
--# expr-start 5424 '<FieldAccess for is_library_item at nodes.lkt:8070:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5424
if Fld_6 then
--# expr-start 5426 '<FieldAccess for other_part at nodes.lkt:8070:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8070'
--# expr-start 5425 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:8070:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 5425
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Fld);
--# end
--# expr-done 5426
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Compilation_Unit then
   --# scope-start
   --# bind part Var_Expr_0
--# expr-start 5429 '<Cast to AdaNode at nodes.lkt:8071:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8071'
--# expr-start 5428 '<as_entity at nodes.lkt:8071:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8071'
--# expr-start 5427 '<FieldAccess for decl at nodes.lkt:8071:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8071'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_0.Node);
--# end
--# expr-done 5427
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 5428
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
--# expr-done 5429
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 5430 '<ArrayLiteral at nodes.lkt:8071:68>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8071'
Array_Lit := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => Var_Expr_1)); 
--# expr-done 5430
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2698;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2697;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
else
   --# expr-start 5434 '<bind at nodes.lkt:8073:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8073'
--# expr-start 5431 '<FieldAccess for origin_node at nodes.lkt:8073:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8073'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5431
Bound_Origin := Fld_3; 
--# expr-start 5433 '<FieldAccess for all_env_els_impl at nodes.lkt:8075:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8075'
--# expr-start 5432 '<FieldAccess for defining_name at nodes.lkt:8075:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8075'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5432
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_4.Node, Categories => (Inherited_Primitives => False, Nocat => True), Env => Env, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5433
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5434
   If_Result_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5435
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := If_Result_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_2
      --# expr-start 5437 '<Match at nodes.lkt:8080:16>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8080'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_2; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind pkg_decl Match_0
Cast_Expr := Match_0; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind gen_pkg_decl Match_1
--# expr-start 5436 '<FieldAccess for package_decl at nodes.lkt:8082:54>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8082'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_1.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_1.Info);
--# expr-done 5436
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 5437
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Match_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5438
      Find_Result := No_Entity_Basic_Decl;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_3
      --# expr-start 5440 '<Not at nodes.lkt:8085:19>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8085'
--# expr-start 5439 '<IsNull at nodes.lkt:8085:23>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8085'
Is_Null_1 := Item_3.Node = null; 
--# expr-done 5439
Not_Val := not (Is_Null_1); 
--# expr-done 5440
      if Not_Val then
         Find_Result := Item_3; 
      end if;
   --# end
               exit when Not_Val;
         end loop;
      end;
--# expr-done 5441
         Property_Result := Find_Result;
   --# end
      Finalizer_Scope_872;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_872;
                     Finalizer_Scope_2697;
                     Finalizer_Scope_2698;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Package_Previous_Part;
--# end
--# property-start Body.task_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8089'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Task_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_873 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_873 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_873;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5447 '<Cast to BasicDecl at nodes.lkt:8089:51>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8089'
--# expr-start 5446 '<at at nodes.lkt:8089:51>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8089'
--# expr-start 5445 '<bind at nodes.lkt:8090:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8090'
--# expr-start 5442 '<FieldAccess for origin_node at nodes.lkt:8090:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8090'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5442
Bound_Origin := Fld; 
--# expr-start 5444 '<FieldAccess for all_env_els_impl at nodes.lkt:8092:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8092'
--# expr-start 5443 '<FieldAccess for defining_name at nodes.lkt:8092:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8092'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5443
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5444
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5445
Get_Result := Get (Self, Dyn_Var_Bind_Result, 0, True); 
--# expr-done 5446
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5447
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_873;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_873;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Task_Previous_Part;
--# end
--# property-start Body.protected_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8097'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Protected_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_874 with Inline_Always;
      Var_Expr_0 : Internal_Entity;
Match_0 : Internal_Entity_Protected_Type_Decl;
Match_1 : Internal_Entity_Single_Protected_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Protected_Type_Decl;
Cast_Expr : Internal_Entity_Protected_Type_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Single_Protected_Decl;
Cast_Expr_1 : Internal_Entity_Single_Protected_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_874 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_874;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5450 '<at at nodes.lkt:8097:56>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8097'
--# expr-start 5449 '<FieldAccess for env_elements at nodes.lkt:8097:56>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8097'
--# expr-start 5448 '<FieldAccess for defining_name at nodes.lkt:8097:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8097'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5448
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 5449
Get_Result := Get (Self, Fld_1, 0, True); 
--# expr-done 5450
Var_Expr_0 := Get_Result;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5451 '<Match at nodes.lkt:8098:17>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8098'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind prot_type Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind prot_decl Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5451
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_874;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_874;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Protected_Previous_Part;
--# end
--# property-start Body.unbound_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8108'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Unbound_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Pp : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Base_Subp_Body;
Match_1 : Internal_Entity_Subp_Body_Stub;
Match_2 : Internal_Entity_Entry_Body;
Match_3 : Internal_Entity_Package_Body;
Match_4 : Internal_Entity_Package_Body_Stub;
Match_5 : Internal_Entity_Protected_Body;
Match_6 : Internal_Entity_Protected_Body_Stub;
Match_7 : Internal_Entity_Task_Body;
Match_8 : Internal_Entity_Task_Body_Stub;
Match_9 : Internal_Entity_Accept_Stmt_Body;
Local_Pp_Next_Part : Internal_Entity_Basic_Decl;
Var_Expr_155 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Entry_Decl;
Match_Prefix : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Body_Stub;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Package_Body;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Package_Body_Stub;
Let_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Protected_Body;
Let_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_5 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Protected_Body_Stub;
Let_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_6 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Task_Body;
Let_Result_7 : Internal_Entity_Basic_Decl;
Scope_Result_7 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Task_Body_Stub;
Let_Result_8 : Internal_Entity_Basic_Decl;
Scope_Result_8 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Accept_Stmt_Body;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Let_Result_9 : Internal_Entity_Basic_Decl;
Scope_Result_9 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_10 : Internal_Entity_Basic_Decl;
Scope_Result_10 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result_10 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_11 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5469 '<Let at nodes.lkt:8109:9>' Let_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8109'
--# scope-start
--# expr-start 5462 '<Match at nodes.lkt:8109:18>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8109'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Body_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 5452 '<FieldAccess for subp_previous_part at nodes.lkt:8110:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8110'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5452
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 5453 '<FieldAccess for subp_previous_part at nodes.lkt:8111:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8111'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5453
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 5454 '<FieldAccess for entry_previous_part at nodes.lkt:8112:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8112'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.entry_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Entry_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5454
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_3; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
--# expr-start 5455 '<FieldAccess for package_previous_part at nodes.lkt:8113:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8113'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.package_previous_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5455
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Package_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind _ Match_4
--# expr-start 5456 '<FieldAccess for package_previous_part at nodes.lkt:8114:40>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8114'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.package_previous_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5456
Let_Result_4 := Fld_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Protected_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
--# expr-start 5457 '<FieldAccess for protected_previous_part at nodes.lkt:8115:38>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8115'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.protected_previous_part
Fld_5 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5457
Let_Result_5 := Fld_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Protected_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Protected_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_7; 
--# bind _ Match_6
--# expr-start 5458 '<FieldAccess for protected_previous_part at nodes.lkt:8116:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8116'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.protected_previous_part
Fld_6 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5458
Let_Result_6 := Fld_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_8; 
--# bind _ Match_7
--# expr-start 5459 '<FieldAccess for task_previous_part at nodes.lkt:8117:33>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8117'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.task_previous_part
Fld_7 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5459
Let_Result_7 := Fld_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Task_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_9; 
--# bind _ Match_8
--# expr-start 5460 '<FieldAccess for task_previous_part at nodes.lkt:8118:37>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8118'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.task_previous_part
Fld_8 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 5460
Let_Result_8 := Fld_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Accept_Stmt_Body =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Accept_Stmt_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_10; 
--# bind a Match_9
--# expr-start 5461 '<FieldAccess for accept_stmt_previous_part at nodes.lkt:8119:39>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8119'
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmtBody.accept_stmt_previous_part
Fld_9 := Libadalang.Implementation.Accept_Stmt_Body_P_Accept_Stmt_Previous_Part (Node => Match_9.Node, E_Info => Match_9.Info);
--# end
--# expr-done 5461
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Let_Result_9 := Cast_Result_11; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
end case;
--# expr-done 5462
Local_Pp := Match_Result; 
--# bind pp Local_Pp
--# expr-start 5468 '<If at nodes.lkt:8125:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8125'
--# expr-start 5463 '<IsA BodyStub at nodes.lkt:8125:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8125'
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Body_Stub; 
--# expr-done 5463
if Is_A_1 then
   If_Result_1 := Local_Pp;
else
   --# expr-start 5467 '<Let at nodes.lkt:8126:13>' Let_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8126'
--# scope-start
Var_Expr_155 := Local_Pp;
if Var_Expr_155 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5464 '<FieldAccess for next_part_for_decl at nodes.lkt:8126:32>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8126'
      if Var_Expr_155.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr_155.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_155.Info);
--# end
--# expr-done 5464
Scope_Result_10 := Fld_10;
   --# end
   Result_Var := Scope_Result_10;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Local_Pp_Next_Part := Result_Var; 
--# bind pp_next_part Local_Pp_Next_Part
--# expr-start 5466 '<If at nodes.lkt:8128:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8128'
--# expr-start 5465 '<IsA BodyStub at nodes.lkt:8128:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8128'
Is_A := Local_Pp_Next_Part.Node /= null 
and then Local_Pp_Next_Part.Node.Kind in Ada_Body_Stub; 
--# expr-done 5465
if Is_A then
   If_Result := Local_Pp_Next_Part;
else
   If_Result := Local_Pp;
end if;
--# expr-done 5466
Let_Result_10 := If_Result; 
--# end
--# expr-done 5467
   If_Result_1 := Let_Result_10;
end if;
--# expr-done 5468
Let_Result_11 := If_Result_1; 
--# end
--# expr-done 5469
         Property_Result := Let_Result_11;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Unbound_Previous_Part;
--# end
--# property-start Body.previous_part_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8139'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Internal
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_876 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_876 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_876;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5475 '<bind at nodes.lkt:8146:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8146'
--# expr-start 5470 '<FieldAccess for node_env at nodes.lkt:8146:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8146'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 5470
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5474 '<as_entity at nodes.lkt:8148:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8148'
--# expr-start 5473 '<FieldAccess for node at nodes.lkt:8148:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8148'
--# expr-start 5472 '<FieldAccess for unbound_previous_part at nodes.lkt:8148:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8148'
--# expr-start 5471 '<as_bare_entity at nodes.lkt:8148:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8148'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5471
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.unbound_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Unbound_Previous_Part (Node => If_Result.Node, Env => Bound_Env, Imprecise_Fallback => Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 5472
Fld_2 := Fld_1.Node;
--# expr-done 5473
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 5474
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 5475
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_876;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_876;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Internal;
--# end
--# property-start Body.previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8156'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Body_Node_Body_Node_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Body_Node,
                                As_Bare_Body_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5476 '<FieldAccess for previous_part_internal at nodes.lkt:8156:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8156'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5476
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Body_Node_P_Previous_Part;
--# end
--# property-start Body.decl_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8161'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Decl_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 5477 '<FieldAccess for previous_part_internal at nodes.lkt:8161:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8161'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5477
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind prev_part Var_Expr_0
--# expr-start 5479 '<Match at nodes.lkt:8162:24>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8162'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_0; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 5478 '<FieldAccess for previous_part_internal at nodes.lkt:8165:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8165'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 5478
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5479
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Decl_Part;
--# end
--# property-start Body.safe_generic_decl_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8172'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Safe_Generic_Decl_Part
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_879 with Inline_Always;
            procedure Finalizer_Scope_2723 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_156 : Internal_Entity_Array_Access;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Scope_Result : Internal_Entity_Generic_Package_Decl;
Result_Var : Internal_Entity_Generic_Package_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_2 : Internal_Entity_Generic_Decl;
Is_A_1 : Boolean;
If_Result : Internal_Entity_Generic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Decl;
Is_A_2 : Boolean;
If_Result_1 : Internal_Entity_Generic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_879 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_879;
            procedure Finalizer_Scope_2723 is
            begin
                     Dec_Ref (Var_Expr_156);
            end Finalizer_Scope_2723;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5494 '<bind at nodes.lkt:8173:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8173'
Bound_Imprecise_Fallback := False; 
--# expr-start 5493 '<If at nodes.lkt:8175:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
--# expr-start 5480 '<IsA PackageBody at nodes.lkt:8175:12>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
Is_A_2 := Self /= null 
and then Self.Kind in Ada_Package_Body_Range; 
--# expr-done 5480
if Is_A_2 then
--# expr-start 5481 '<FieldAccess for decl_part at nodes.lkt:8175:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5481
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 5483 '<Cast to GenericPackageDecl at nodes.lkt:8175:64>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
--# expr-start 5482 '<parent at nodes.lkt:8175:64>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 5482
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 5483
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Package_Decl;
end if;
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_3;
else
   --# expr-start 5492 '<If at nodes.lkt:8175:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8175'
--# expr-start 5484 '<IsA BaseSubpBody, SubpBodyStub at nodes.lkt:8176:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8176'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range; 
--# expr-done 5484
if Is_A_1 then
--# expr-start 5491 '<Cast to GenericSubpDecl at nodes.lkt:8182:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8182'
--# expr-start 5488 '<EnvGet at nodes.lkt:8182:13>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8182'
--# expr-start 5486 '<env_parent at nodes.lkt:8182:13>' Env_Parent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8182'
--# expr-start 5485 '<FieldAccess for children_env at nodes.lkt:8182:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5485
Env_Parent := AST_Envs.Parent (Fld_2); 
--# expr-done 5486
--# expr-start 5487 '<FieldAccess for name_symbol at nodes.lkt:8183:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5487
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env_Parent, Key => Thin (Fld_3), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5488
Var_Expr_156 := Env_Get_Result;
      Inc_Ref (Var_Expr_156);
if Var_Expr_156 /= No_Internal_Entity_Array_Type then
   --# scope-start
--# expr-start 5490 '<Find at nodes.lkt:8182:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8182'
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_156;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_0
      --# expr-start 5489 '<IsA GenericSubpDecl at nodes.lkt:8184:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8184'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Generic_Subp_Decl_Range; 
--# expr-done 5489
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 5490
Scope_Result_1 := Find_Result;
   --# end
      Finalizer_Scope_2723;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 5491
      Cast_Result_2 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
   If_Result := Cast_Result_2;
else
   If_Result := No_Entity_Generic_Decl;
end if;
--# expr-done 5492
   If_Result_1 := If_Result;
end if;
--# expr-done 5493
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 5494
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_879;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_879;
                     Finalizer_Scope_2723;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Safe_Generic_Decl_Part;
--# end
--# property-start Body.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8192'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Next_Part_For_Decl
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Basic_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Next_Part_For_Decl;
--# end
--# property-start Body.is_subunit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8194'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Is_Subunit
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5496 '<IsA Subunit at nodes.lkt:8194:30>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8194'
--# expr-start 5495 '<parent at nodes.lkt:8194:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8194'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5495
Is_A := Fld /= null 
and then Fld.Kind in Ada_Subunit_Range; 
--# expr-done 5496
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Is_Subunit;
--# end
--# property-start Body.subunit_root 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8198'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Root
  (Node : Bare_Body_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Bare_Subunit;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 5498 '<Cast to Subunit at nodes.lkt:8199:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8199'
--# expr-start 5497 '<parent at nodes.lkt:8199:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8199'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5497
   if Fld = null
      or else Fld.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5498
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind su Var_Expr_0
--# expr-start 5499 '<FieldAccess for body_root at nodes.lkt:8199:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8199'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Var_Expr_0);
--# end
--# expr-done 5499
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Body_Node_P_Subunit_Root;
--# end
--# property-start Body.body_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8207'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Scope
  (Node : Bare_Body_Node
      ; Follow_Private : Boolean
      ; Force_Decl : Boolean
         := False
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_private Follow_Private
   --# bind force_decl Force_Decl
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_883 with Inline_Always;
            procedure Finalizer_Scope_2726 with Inline_Always;
            procedure Finalizer_Scope_2727 with Inline_Always;
      Local_Scope : Lexical_Env := Empty_Env;
Local_Dns : Lexical_Env := Empty_Env;
Local_Public_Scope : Lexical_Env := Empty_Env;
Var_Expr_0 : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Bare_Ada_Node;
Fld_9 : Lexical_Env := Empty_Env;
Fld_10 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Fld_11 : Boolean;
Fld_12 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Lexical_Env := Empty_Env;
Fld_13 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_6 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Package_Decl;
Fld_14 : Bare_Generic_Package_Internal;
Fld_15 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_16 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Env_Node_2 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_883 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Local_Dns);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Local_Scope);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Public_Scope);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_883;
            procedure Finalizer_Scope_2726 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2726;
            procedure Finalizer_Scope_2727 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_2727;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5536 '<Let at nodes.lkt:8208:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8208'
--# scope-start
--# expr-start 5523 '<If at nodes.lkt:8212:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5502 '<Not at nodes.lkt:8212:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5501 '<IsNull at nodes.lkt:8212:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5500 '<FieldAccess for subunit_root at nodes.lkt:8212:16>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subunit_root
Fld_13 := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 5500
Is_Null := Fld_13.Node = null; 
--# expr-done 5501
Not_Val := not (Is_Null); 
--# expr-done 5502
if Not_Val then
   --# expr-start 5504 '<FieldAccess for children_env at nodes.lkt:8212:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5503 '<FieldAccess for subunit_root at nodes.lkt:8212:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subunit_root
Fld := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 5503
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 5504
   If_Result_6 := Fld_1;
else
   --# expr-start 5522 '<If at nodes.lkt:8212:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5505 '<FieldAccess for is_subprogram at nodes.lkt:8216:14>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8216'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 5505
if Fld_11 then
   --# expr-start 5506 '<FieldAccess for is_compilation_unit_root at nodes.lkt:8216:39>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8216'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5506
   If_Result_4 := Fld_12;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 5515 '<Let at nodes.lkt:8217:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8217'
--# scope-start
--# expr-start 5508 '<FieldAccess for scope at nodes.lkt:8217:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8217'
--# expr-start 5507 '<FieldAccess for defining_name at nodes.lkt:8217:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8217'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5507
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_2.Node, Env => Env);
--# end
--# expr-done 5508
Local_Dns := Fld_3; Inc_Ref (Local_Dns);
--# bind dns Local_Dns
--# expr-start 5514 '<If at nodes.lkt:8221:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
--# expr-start 5509 '<IsNull at nodes.lkt:8221:16>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
Is_Equal := Equivalent (Local_Dns, Empty_Env); 
--# expr-done 5509
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5511 '<Eq at nodes.lkt:8221:31>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
--# expr-start 5510 '<env_node at nodes.lkt:8221:31>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
Env_Node := AST_Envs.Env_Node (Local_Dns); 
--# expr-done 5510
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 5511
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 5513 '<FieldAccess for parent_scope at nodes.lkt:8221:57>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
--# expr-start 5512 '<FieldAccess for defining_name at nodes.lkt:8221:57>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8221'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5512
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_4.Node, Env => Env);
--# end
--# expr-done 5513
   If_Result_1 := Fld_5;
else
   If_Result_1 := Local_Dns;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5514
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5515
   If_Result_5 := Let_Result;
else
   --# expr-start 5521 '<If at nodes.lkt:8212:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8212'
--# expr-start 5516 '<FieldAccess for is_compilation_unit_root at nodes.lkt:8225:14>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8225'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5516
if Fld_10 then
   If_Result_2 := True;
else
   If_Result_2 := Force_Decl;
end if;
if If_Result_2 then
   --# expr-start 5518 '<FieldAccess for scope at nodes.lkt:8225:65>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8225'
--# expr-start 5517 '<FieldAccess for defining_name at nodes.lkt:8225:65>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8225'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5517
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_6.Node, Env => Env);
--# end
--# expr-done 5518
   If_Result_3 := Fld_7;
else
   --# expr-start 5520 '<FieldAccess for children_env at nodes.lkt:8228:14>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8228'
--# expr-start 5519 '<parent at nodes.lkt:8228:14>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8228'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Parent; 
--# expr-done 5519
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Children_Env (Node => Fld_8);
--# expr-done 5520
   If_Result_3 := Fld_9;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 5521
   If_Result_5 := If_Result_3;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 5522
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 5523
Local_Scope := If_Result_6; Inc_Ref (Local_Scope);
--# bind scope Local_Scope
--# expr-start 5535 '<Let at nodes.lkt:8231:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8231'
--# scope-start
--# expr-start 5525 '<Cast to GenericPackageDecl at nodes.lkt:8231:28>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8231'
--# expr-start 5524 '<env_node at nodes.lkt:8231:28>' Env_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8231'
Env_Node_1 := AST_Envs.Env_Node (Local_Scope); 
--# expr-done 5524
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_1 := Env_Node_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5525
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr_0
--# expr-start 5527 '<FieldAccess for children_env at nodes.lkt:8232:31>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8232'
--# expr-start 5526 '<FieldAccess for package_decl at nodes.lkt:8232:31>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8232'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Var_Expr_0.Generic_Package_Decl_F_Package_Decl;
--# expr-done 5526
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Fld_14);
--# expr-done 5527
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2726;
   Result_Var := Scope_Result;
else
   Result_Var := Local_Scope;
end if;
      Inc_Ref (Result_Var);
Local_Public_Scope := Result_Var; Inc_Ref (Local_Public_Scope);
--# bind public_scope Local_Public_Scope
--# expr-start 5534 '<If at nodes.lkt:8237:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8237'
if Follow_Private then
--# expr-start 5528 '<env_node at nodes.lkt:8237:31>' Env_Node_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8237'
Env_Node_2 := AST_Envs.Env_Node (Local_Public_Scope); 
--# expr-done 5528
Var_Expr_1 := Env_Node_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 5529 '<IsA BasePackageDecl, SingleProtectedDecl, ProtectedTypeDecl at nodes.lkt:8238:21>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8238'
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Package_Decl | Ada_Single_Protected_Decl_Range | Ada_Protected_Type_Decl_Range; 
--# expr-done 5529
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_7 := Result_Var_2;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 5532 '<at at nodes.lkt:8239:16>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8239'
--# expr-start 5531 '<EnvGet at nodes.lkt:8239:16>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8239'
--# expr-start 5530 '<Symbol __privatepart at nodes.lkt:8240:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8240'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 5530
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Local_Public_Scope, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 5531
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 5532
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 5533 '<FieldAccess for children_env at nodes.lkt:8242:21>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8242'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 5533
Scope_Result_1 := Fld_16;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2727;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Local_Public_Scope;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_8 := Result_Var_1;
else
   If_Result_8 := Local_Public_Scope;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 5534
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 5535
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 5536
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_883;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_883;
                     Finalizer_Scope_2726;
                     Finalizer_Scope_2727;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Body_Scope;
--# end
--# property-start Body.gnatprove_annotations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8248'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Gnatprove_Annotations
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_884 with Inline_Always;
            procedure Finalizer_Scope_2730 with Inline_Always;
            procedure Finalizer_Scope_2734 with Inline_Always;
            procedure Finalizer_Scope_2735 with Inline_Always;
            procedure Finalizer_Scope_2737 with Inline_Always;
            procedure Finalizer_Scope_2738 with Inline_Always;
            procedure Finalizer_Scope_2745 with Inline_Always;
            procedure Finalizer_Scope_2747 with Inline_Always;
      Local_Subp_Decl_Part : Internal_Entity_Basic_Decl;
Local_Aspects : Internal_Entity_Expr_Array_Access;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Local_Pragma_Scope : Internal_Entity_Basic_Decl;
Local_Scope_Decls : Bare_Pragma_Node_Array_Access;
Local_Pragmas : Internal_Entity_Expr_Array_Access;
Local_Enclosing_Subp_Annotations : Internal_Entity_Expr_Array_Access;
Var_Expr_157 : Internal_Entity_Aspect_Spec;
Var_Expr_158 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Pragma_Node_Array_Access;
Var_Expr_159 : Internal_Entity_Basic_Decl;
Var_Expr_160 : Bare_Declarative_Part;
Var_Expr_161 : Internal_Entity_Basic_Decl;
Var_Expr_162 : Bare_Pragma_Node;
Var_Expr_163 : Bare_Base_Assoc;
Var_Expr_164 : Internal_Entity_Name;
Var_Expr_165 : Bare_Base_Assoc;
Var_Expr_166 : Internal_Entity_Name;
Var_Expr_167 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Aspect_Spec;
Scope_Result : Internal_Entity_Aspect_Spec;
Fld_2 : Internal_Entity_Aspect_Spec;
Result_Var : Internal_Entity_Aspect_Spec;
Fld_3 : Internal_Entity_Aspect_Assoc_List;
Fld_4 : Bare_Aspect_Assoc_List;
Fld_5 : Internal_Entity_Info;
Item_1 : Internal_Entity_Aspect_Assoc;
Bare_Item_1 : Bare_Aspect_Assoc;
Fld_6 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Fld_9 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
As_Entity_1 : Internal_Entity_Aspect_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Aspect_Assoc;
Fld_11 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_2 : Boolean;
Fld_14 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_15 : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Basic_Assoc;
Fld_18 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_19 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_4 : Boolean;
Map_Result : Internal_Entity_Expr_Array_Access;
Scope_Result_2 : Internal_Entity_Expr_Array_Access;
Result_Var_2 : Internal_Entity_Expr_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result_3 : Internal_Entity_Expr_Function;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
Fld_20 : Bare_Declarative_Part;
Fld_21 : Bare_Ada_Node_List;
Item_2 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_5 : Bare_Pragma_Node;
Is_A_1 : Boolean;
Map_Result_1 : Bare_Pragma_Node_Array_Access;
Scope_Result_4 : Bare_Pragma_Node_Array_Access;
Fld_22 : Internal_Entity_Pragma_Node_List;
Fld_23 : Bare_Pragma_Node_List;
Fld_24 : Internal_Entity_Info;
Item_3 : Internal_Entity_Pragma_Node;
Bare_Item_3 : Bare_Pragma_Node;
Fld_25 : Bare_Pragma_Node;
As_Entity_3 : Internal_Entity_Pragma_Node;
Is_Null_3 : Boolean;
If_Result_5 : Internal_Entity_Pragma_Node;
Map_Result_2 : Bare_Pragma_Node_Array_Access;
Scope_Result_5 : Bare_Pragma_Node_Array_Access;
Result_Var_4 : Bare_Pragma_Node_Array_Access;
Result_Var_5 : Bare_Pragma_Node_Array_Access;
Scope_Result_6 : Bare_Pragma_Node_Array_Access;
Result_Var_6 : Bare_Pragma_Node_Array_Access;
Scope_Result_7 : Bare_Pragma_Node_Array_Access;
Result_Var_7 : Bare_Pragma_Node_Array_Access;
Fld_26 : Bare_Base_Assoc_List;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_6 : Internal_Entity_Base_Assoc;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Bare_Basic_Decl;
Node_Comp : Boolean;
Fld_29 : Bare_Identifier;
Sym_2 : Symbol_Type;
Fld_30 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
If_Result_7 : Boolean;
Fld_31 : Bare_Base_Assoc_List;
Get_Result_3 : Bare_Base_Assoc;
As_Entity_5 : Internal_Entity_Base_Assoc;
Is_Null_5 : Boolean;
If_Result_8 : Internal_Entity_Base_Assoc;
Fld_32 : Internal_Entity_Expr;
Scope_Result_9 : Internal_Entity_Expr;
Result_Var_9 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Sym_3 : Symbol_Type;
Fld_33 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_10 : Boolean;
If_Result_9 : Boolean;
Fld_34 : Bare_Base_Assoc_List;
Get_Result_4 : Bare_Base_Assoc;
As_Entity_6 : Internal_Entity_Base_Assoc;
Is_Null_6 : Boolean;
If_Result_10 : Internal_Entity_Base_Assoc;
Fld_35 : Internal_Entity_Expr;
Scope_Result_11 : Internal_Entity_Expr;
Result_Var_11 : Internal_Entity_Expr;
Cast_Result_7 : Internal_Entity_Name;
Fld_36 : Internal_Entity_Defining_Name_Array_Access;
Get_Result_5 : Internal_Entity_Defining_Name;
Fld_37 : Symbol_Type;
Fld_38 : Boolean;
Scope_Result_12 : Boolean;
Result_Var_12 : Boolean;
If_Result_11 : Boolean;
Map_Result_3 : Internal_Entity_Expr_Array_Access;
Fld_39 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_8 : Internal_Entity_Base_Subp_Body;
Fld_40 : Internal_Entity_Expr_Array_Access;
Scope_Result_13 : Internal_Entity_Expr_Array_Access;
Result_Var_13 : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
Let_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result_2 : Internal_Entity_Expr_Array_Access;
Let_Result_3 : Internal_Entity_Expr_Array_Access;
Let_Result_4 : Internal_Entity_Expr_Array_Access;
Let_Result_5 : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_884 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Local_Aspects);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (Local_Scope_Decls);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Local_Pragmas);
                     Dec_Ref (Fld_39);
                     Dec_Ref (Scope_Result_13);
                     Dec_Ref (Result_Var_13);
                     Dec_Ref (Local_Enclosing_Subp_Annotations);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_884;
            procedure Finalizer_Scope_2730 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2730;
            procedure Finalizer_Scope_2734 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
            end Finalizer_Scope_2734;
            procedure Finalizer_Scope_2735 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Result_Var_5);
            end Finalizer_Scope_2735;
            procedure Finalizer_Scope_2737 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2737;
            procedure Finalizer_Scope_2738 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2738;
            procedure Finalizer_Scope_2745 is
            begin
                     Dec_Ref (Fld_36);
            end Finalizer_Scope_2745;
            procedure Finalizer_Scope_2747 is
            begin
                     Dec_Ref (Fld_40);
            end Finalizer_Scope_2747;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Body_Node_Body_Node_P_Gnatprove_Annotations,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Body_Node,
                                As_Bare_Body_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Expr_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 5603 '<Let at nodes.lkt:8249:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8249'
--# scope-start
--# expr-start 5537 '<FieldAccess for decl_part at nodes.lkt:8249:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8249'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 5537
Local_Subp_Decl_Part := Fld; 
--# bind subp_decl_part Local_Subp_Decl_Part
--# expr-start 5602 '<Let at nodes.lkt:8252:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8252'
--# scope-start
Var_Expr_0 := Local_Subp_Decl_Part;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind part Var_Expr_0
--# expr-start 5538 '<FieldAccess for aspects at nodes.lkt:8253:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8253'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Var_Expr_0.Node), Info => Var_Expr_0.Info);
--# expr-done 5538
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 5539 '<FieldAccess for aspects at nodes.lkt:8253:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8253'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 5539
   Result_Var := Fld_2;
end if;
Var_Expr_157 := Result_Var;
if Var_Expr_157 /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 5557 '<Map at nodes.lkt:8252:23>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8252'
   --# expr-start 5540 '<FieldAccess for aspect_assocs at nodes.lkt:8252:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8252'
      if Var_Expr_157.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr_157.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr_157.Info);
--# expr-done 5540
Fld_4 := Fld_3.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_4;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Aspect_Assoc;
else
Fld_5 := Fld_3.Info;
As_Entity_1 := (Info => Fld_5, Node => Bare_Item_1); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_1 := If_Result_1; 
   --# scope-start
               --# bind a Item_1
--# expr-start 5543 '<FieldAccess for name_is at nodes.lkt:8256:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8256'
--# expr-start 5541 '<FieldAccess for id at nodes.lkt:8256:17>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8256'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Item_1.Node.Aspect_Assoc_F_Id, Info => Item_1.Info);
--# expr-done 5541
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5542 '<Symbol Annotate at nodes.lkt:8256:30>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8256'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 5542
--# property-call-start Name.name_is
Fld_12 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_11.Node, Sym => Sym);
--# end
--# expr-done 5543
if Fld_12 then
   --# expr-start 5545 '<IsA Aggregate at nodes.lkt:8256:47>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8256'
--# expr-start 5544 '<FieldAccess for expr at nodes.lkt:8256:47>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8256'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Item_1.Node.Aspect_Assoc_F_Expr, Info => Item_1.Info);
--# expr-done 5544
Is_A := Fld_13.Node /= null 
and then Fld_13.Node.Kind in Ada_Aggregate_Range; 
--# expr-done 5545
   If_Result_2 := Is_A;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
--# expr-start 5550 '<Cast to Name at nodes.lkt:8257:19>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
--# expr-start 5549 '<FieldAccess for expr at nodes.lkt:8257:19>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
--# expr-start 5548 '<FieldAccess for assocs at nodes.lkt:8257:19>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
--# expr-start 5547 '<Cast to Aggregate at nodes.lkt:8257:19>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
--# expr-start 5546 '<FieldAccess for expr at nodes.lkt:8257:19>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Item_1.Node.Aspect_Assoc_F_Expr, Info => Item_1.Info);
--# expr-done 5546
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 5547
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 5548
Fld_16 := Fld_15.Node;
Get_Result_1 := Get (Self, Fld_16, 0, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Basic_Assoc;
else
Fld_17 := Fld_15.Info;
As_Entity_2 := (Info => Fld_17, Node => Get_Result_1); 
   If_Result_3 := As_Entity_2;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 5549
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 5550
Var_Expr_158 := Cast_Result_2;
if Var_Expr_158 /= No_Entity_Name then
   --# scope-start
--# expr-start 5552 '<FieldAccess for name_is at nodes.lkt:8257:19>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
      if Var_Expr_158.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5551 '<Symbol GNATprove at nodes.lkt:8257:76>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8257'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnatprove); 
--# expr-done 5551
--# property-call-start Name.name_is
Fld_19 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_158.Node, Sym => Sym_1);
--# end
--# expr-done 5552
Scope_Result_1 := Fld_19;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_4 := Result_Var_1;
else
   If_Result_4 := False;
end if;
         if If_Result_4 then
      --# expr-start 5556 '<FieldAccess for expr at nodes.lkt:8255:20>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8255'
--# expr-start 5555 '<FieldAccess for assocs at nodes.lkt:8255:20>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8255'
--# expr-start 5554 '<Cast to Aggregate at nodes.lkt:8255:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8255'
--# expr-start 5553 '<FieldAccess for expr at nodes.lkt:8255:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8255'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Item_1.Node.Aspect_Assoc_F_Expr, Info => Item_1.Info);
--# expr-done 5553
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 5554
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 5555
Fld_8 := Fld_7.Node;
Get_Result := Get (Self, Fld_8, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5556
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_10;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5557
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2730;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Local_Aspects := Result_Var_2; Inc_Ref (Local_Aspects);
--# bind aspects Local_Aspects
--# expr-start 5601 '<Let at nodes.lkt:8261:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8261'
--# scope-start
Var_Expr := Local_Subp_Decl_Part;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_3 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_3;
else
--# expr-start 5558 '<Cast to ExprFunction at nodes.lkt:8261:47>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8261'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr_Function_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Expr_Function
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Expr_Function;
   end if;
--# expr-done 5558
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
   Result_Var_3 := Cast_Result_4;
end if;
Local_Pragma_Scope := Result_Var_3; 
--# bind pragma_scope Local_Pragma_Scope
--# expr-start 5600 '<Let at nodes.lkt:8265:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8265'
--# scope-start
Var_Expr_159 := Local_Pragma_Scope;
if Var_Expr_159 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5559 '<FieldAccess for declarative_scope at nodes.lkt:8266:13>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
      if Var_Expr_159.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_20 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Var_Expr_159.Node);
--# end
--# expr-done 5559
Var_Expr_160 := Fld_20;
if Var_Expr_160 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5563 '<Map at nodes.lkt:8266:13>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
   --# expr-start 5560 '<FieldAccess for decls at nodes.lkt:8266:13>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
      if Var_Expr_160 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Var_Expr_160.Declarative_Part_F_Decls;
--# expr-done 5560
      declare
         Map_Result_1_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Fld_21 = null then
      Map_Result_1 := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_21;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_2 := Untyped_Item_2; 
   --# scope-start
               --# bind p Item_2
         --# expr-start 5561 '<IsA Pragma at nodes.lkt:8266:92>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
Is_A_1 := Item_2 /= null 
and then Item_2.Kind in Ada_Pragma_Node_Range; 
--# expr-done 5561
         if Is_A_1 then
      --# expr-start 5562 '<Cast to Pragma at nodes.lkt:8266:71>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
Cast_Expr_1 := Item_2; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_5 := Cast_Expr_1;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 5562
         declare
            Item_To_Append : constant Bare_Pragma_Node := Cast_Result_5;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_1_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 5563
Var_Expr_1 := Map_Result_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Bare_Pragma_Node_Array_Type then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_4 := Var_Expr_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2737;
   Result_Var_5 := Scope_Result_4;
else
Var_Expr_161 := Local_Pragma_Scope;
if Var_Expr_161 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5566 '<Map at nodes.lkt:8266:109>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
   --# expr-start 5564 '<FieldAccess for library_item_pragmas at nodes.lkt:8266:109>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
      if Var_Expr_161.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Var_Expr_161.Node, E_Info => Var_Expr_161.Info);
--# end
--# expr-done 5564
Fld_23 := Fld_22.Node;
      declare
         Map_Result_2_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Fld_23 = null then
      Map_Result_2 := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_23;
      begin
         for Untyped_Item_3 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_3 := Untyped_Item_3; 
Is_Null_3 := Bare_Item_3 = null; 
if Is_Null_3 then
   If_Result_5 := No_Entity_Pragma_Node;
else
Fld_24 := Fld_22.Info;
As_Entity_3 := (Info => Fld_24, Node => Bare_Item_3); 
   If_Result_5 := As_Entity_3;
end if;
                  Item_3 := If_Result_5; 
   --# scope-start
               --# bind p Item_3
      --# expr-start 5565 '<FieldAccess for node at nodes.lkt:8266:157>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8266'
Fld_25 := Item_3.Node;
--# expr-done 5565
         declare
            Item_To_Append : constant Bare_Pragma_Node := Fld_25;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_2_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 5566
Scope_Result_5 := Map_Result_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2738;
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
   Result_Var_5 := Result_Var_4;
end if;
      Inc_Ref (Result_Var_5);
Scope_Result_6 := Result_Var_5;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2735;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Scope_Result_7 := Result_Var_6;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2734;
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_7);
Local_Scope_Decls := Result_Var_7; Inc_Ref (Local_Scope_Decls);
--# bind scope_decls Local_Scope_Decls
--# expr-start 5599 '<Let at nodes.lkt:8269:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8269'
--# scope-start
--# expr-start 5592 '<Map at nodes.lkt:8269:23>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8269'
      declare
         Map_Result_3_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Local_Scope_Decls;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_4
--# expr-start 5568 '<OrderingTest '"'"'gt'"'"' at nodes.lkt:8272:21>' Node_Comp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8272'
--# expr-start 5567 '<FieldAccess for node at nodes.lkt:8272:25>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8272'
Fld_28 := Local_Pragma_Scope.Node;
--# expr-done 5567
Node_Comp := Compare (Self, Item_4, Fld_28, Greater_Than); 
--# expr-done 5568
if Node_Comp then
Var_Expr_162 := Item_4;
if Var_Expr_162 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5571 '<FieldAccess for name_is at nodes.lkt:8272:47>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8272'
--# expr-start 5569 '<FieldAccess for id at nodes.lkt:8272:47>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8272'
      if Var_Expr_162 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Var_Expr_162.Pragma_Node_F_Id;
--# expr-done 5569
      if Fld_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5570 '<Symbol Annotate at nodes.lkt:8272:61>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8272'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 5570
--# property-call-start Name.name_is
Fld_30 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_29, Sym => Sym_2);
--# end
--# expr-done 5571
Scope_Result_8 := Fld_30;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
   If_Result_7 := Result_Var_8;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 5576 '<Cast to Name at nodes.lkt:8273:23>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
--# expr-start 5573 '<at at nodes.lkt:8273:23>' Get_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
--# expr-start 5572 '<FieldAccess for args at nodes.lkt:8273:23>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
      if Item_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Item_4.Pragma_Node_F_Args;
--# expr-done 5572
Get_Result_3 := Get (Self, Fld_31, 0, True); 
--# expr-done 5573
Var_Expr_163 := Get_Result_3;
if Var_Expr_163 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5575 '<FieldAccess for assoc_expr at nodes.lkt:8273:23>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
--# expr-start 5574 '<as_entity at nodes.lkt:8273:23>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
Is_Null_5 := Var_Expr_163 = null; 
if Is_Null_5 then
   If_Result_8 := No_Entity_Base_Assoc;
else
As_Entity_5 := (Info => E_Info, Node => Var_Expr_163); 
   If_Result_8 := As_Entity_5;
end if;
--# expr-done 5574
      if If_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_32 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_8.Node, E_Info => If_Result_8.Info);
--# end
--# expr-done 5575
Scope_Result_9 := Fld_32;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Expr;
end if;
   if Result_Var_9.Node = null
      or else Result_Var_9.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Result_Var_9.Node,
         Info => Result_Var_9.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 5576
Var_Expr_164 := Cast_Result_6;
if Var_Expr_164 /= No_Entity_Name then
   --# scope-start
--# expr-start 5578 '<FieldAccess for name_is at nodes.lkt:8273:23>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
      if Var_Expr_164.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5577 '<Symbol GNATprove at nodes.lkt:8273:76>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8273'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnatprove); 
--# expr-done 5577
--# property-call-start Name.name_is
Fld_33 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_164.Node, Sym => Sym_3);
--# end
--# expr-done 5578
Scope_Result_10 := Fld_33;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := False;
end if;
   If_Result_9 := Result_Var_10;
else
   If_Result_9 := False;
end if;
if If_Result_9 then
--# expr-start 5583 '<Cast to Name at nodes.lkt:8274:19>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
--# expr-start 5580 '<at at nodes.lkt:8274:19>' Get_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
--# expr-start 5579 '<FieldAccess for args at nodes.lkt:8274:19>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
      if Item_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Item_4.Pragma_Node_F_Args;
--# expr-done 5579
Get_Result_4 := Get (Self, Fld_34, 2, True); 
--# expr-done 5580
Var_Expr_165 := Get_Result_4;
if Var_Expr_165 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5582 '<FieldAccess for assoc_expr at nodes.lkt:8274:19>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
--# expr-start 5581 '<as_entity at nodes.lkt:8274:19>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
Is_Null_6 := Var_Expr_165 = null; 
if Is_Null_6 then
   If_Result_10 := No_Entity_Base_Assoc;
else
As_Entity_6 := (Info => E_Info, Node => Var_Expr_165); 
   If_Result_10 := As_Entity_6;
end if;
--# expr-done 5581
      if If_Result_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_10.Node, E_Info => If_Result_10.Info);
--# end
--# expr-done 5582
Scope_Result_11 := Fld_35;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := No_Entity_Expr;
end if;
   if Result_Var_11.Node = null
      or else Result_Var_11.Node.Kind in Ada_Name
   then
      Cast_Result_7 := Create_Internal_Entity_Name
        (Node => Result_Var_11.Node,
         Info => Result_Var_11.Info);
   else
         Cast_Result_7 := No_Entity_Name;
   end if;
--# expr-done 5583
Var_Expr_166 := Cast_Result_7;
if Var_Expr_166 /= No_Entity_Name then
   --# scope-start
--# expr-start 5587 '<FieldAccess for name_is at nodes.lkt:8274:19>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
      if Var_Expr_166.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5586 '<FieldAccess for name_symbol at nodes.lkt:8274:72>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
--# expr-start 5585 '<at at nodes.lkt:8274:72>' Get_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
--# expr-start 5584 '<FieldAccess for defining_names at nodes.lkt:8274:72>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8274'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5584
Get_Result_5 := Get (Self, Fld_36, 0, True); 
--# expr-done 5585
      if Get_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Get_Result_5.Node);
--# end
--# expr-done 5586
--# property-call-start Name.name_is
Fld_38 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_166.Node, Sym => Fld_37);
--# end
--# expr-done 5587
Scope_Result_12 := Fld_38;
   --# end
      Finalizer_Scope_2745;
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := False;
end if;
   If_Result_11 := Result_Var_12;
else
   If_Result_11 := False;
end if;
         if If_Result_11 then
      --# expr-start 5591 '<FieldAccess for assoc_expr at nodes.lkt:8270:20>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8270'
--# expr-start 5590 '<as_entity at nodes.lkt:8270:20>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8270'
--# expr-start 5589 '<at at nodes.lkt:8270:20>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8270'
--# expr-start 5588 '<FieldAccess for args at nodes.lkt:8270:20>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8270'
      if Item_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Item_4.Pragma_Node_F_Args;
--# expr-done 5588
Get_Result_2 := Get (Self, Fld_26, 1, True); 
--# expr-done 5589
Is_Null_4 := Get_Result_2 = null; 
if Is_Null_4 then
   If_Result_6 := No_Entity_Base_Assoc;
else
As_Entity_4 := (Info => E_Info, Node => Get_Result_2); 
   If_Result_6 := As_Entity_4;
end if;
--# expr-done 5590
      if If_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_27 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_6.Node, E_Info => If_Result_6.Info);
--# end
--# expr-done 5591
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_27;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 5592
Local_Pragmas := Map_Result_3; Inc_Ref (Local_Pragmas);
--# bind pragmas Local_Pragmas
--# expr-start 5598 '<Let at nodes.lkt:8277:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
--# scope-start
--# expr-start 5596 '<Cast to BaseSubpBody at nodes.lkt:8277:42>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
--# expr-start 5595 '<Find at nodes.lkt:8277:42>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
   --# expr-start 5593 '<FieldAccess for parents at nodes.lkt:8277:42>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => False, E_Info => Ent.Info);
--# expr-done 5593
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_39;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_5
      --# expr-start 5594 '<IsA BaseSubpBody at nodes.lkt:8277:84>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
Is_A_2 := Item_5.Node /= null 
and then Item_5.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 5594
      if Is_A_2 then
         Find_Result := Item_5; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 5595
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_8 := Create_Internal_Entity_Base_Subp_Body
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_8 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 5596
Var_Expr_167 := Cast_Result_8;
if Var_Expr_167 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 5597 '<FieldAccess for gnatprove_annotations at nodes.lkt:8277:42>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8277'
      if Var_Expr_167.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.gnatprove_annotations
Fld_40 := Libadalang.Implementation.Body_Node_P_Gnatprove_Annotations (Node => Var_Expr_167.Node, E_Info => Var_Expr_167.Info);
--# end
--# expr-done 5597
Scope_Result_13 := Fld_40;
   Inc_Ref (Scope_Result_13);
   --# end
      Finalizer_Scope_2747;
   Result_Var_13 := Scope_Result_13;
else
   Result_Var_13 := No_Internal_Entity_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_13);
Local_Enclosing_Subp_Annotations := Result_Var_13; Inc_Ref (Local_Enclosing_Subp_Annotations);
--# bind enclosing_subp_annotations Local_Enclosing_Subp_Annotations
Concat_Result := Concat (Local_Aspects, Local_Pragmas); 
Concat_Result_1 := Concat (Concat_Result, Local_Enclosing_Subp_Annotations); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5598
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 5599
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 5600
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 5601
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5602
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 5603
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_884;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_884;
                     Finalizer_Scope_2730;
                     Finalizer_Scope_2734;
                     Finalizer_Scope_2735;
                     Finalizer_Scope_2737;
                     Finalizer_Scope_2738;
                     Finalizer_Scope_2745;
                     Finalizer_Scope_2747;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Expr_Array_Access,
                        As_Internal_Entity_Expr_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Body_Node_P_Gnatprove_Annotations;
--# end
--# property-start '[dispatcher]Body.previous_part_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Body_Node (Self.Kind) is
               when Ada_Accept_Stmt_Body | Ada_Package_Body_Stub .. Ada_Task_Body =>
                  --# property-call-start Body.previous_part_env_name
                  Property_Result := Body_Node_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.previous_part_env_name
                  Property_Result := Base_Subp_Body_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Body_Node_P_Previous_Part_Env_Name;
--# end
--# property-start '[dispatcher]Body.previous_part_link_env_assoc' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Body_Node (Self.Kind) is
               when Ada_Accept_Stmt_Body .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Task_Body =>
                  --# property-call-start Body.previous_part_link_env_assoc
                  Property_Result := Body_Node_P_Previous_Part_Link_Env_Assoc
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.previous_part_link_env_assoc
                  Property_Result := Body_Stub_P_Previous_Part_Link_Env_Assoc
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc;
--# end
--# property-start Body.previous_part_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8286'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_885 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_1 : String_Type;
Sym_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
            procedure Finalizer_Scope_885 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_885;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5610 '<If at nodes.lkt:8287:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
--# expr-start 5604 '<FieldAccess for is_subunit at nodes.lkt:8287:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5604
if Fld_2 then
   --# expr-start 5607 '<to_symbol at nodes.lkt:8287:35>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
--# expr-start 5605 '<FieldAccess for top_level_env_name at nodes.lkt:8287:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5605
--# expr-start 5606 '<String at nodes.lkt:8287:64>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
Str := Create_String ("__stub"); 
--# expr-done 5606
Concat_Result := Concat_String (Fld, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 5607
   If_Result := Sym;
else
   --# expr-start 5609 '<to_symbol at nodes.lkt:8287:89>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
--# expr-start 5608 '<FieldAccess for top_level_env_name at nodes.lkt:8287:89>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8287'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5608
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld_1); 
--# expr-done 5609
   If_Result := Sym_1;
end if;
--# expr-done 5610
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_885;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_885;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Env_Name;
--# end
--# property-start Body.previous_part_link_env_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8296'
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Node  := Bare_Body_Node (Node);
     Ent : Internal_Entity_Body_Node :=
       Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_886 with Inline_Always;
            procedure Finalizer_Scope_2748 with Inline_Always;
            procedure Finalizer_Scope_2749 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Var_Expr_1 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Scope_Result_1 : Internal_Designated_Env := No_Designated_Env;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Result_Var_1 : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
New_Struct_3 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_886 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (New_Struct_3);
            end Finalizer_Scope_886;
            procedure Finalizer_Scope_2748 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2748;
            procedure Finalizer_Scope_2749 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2749;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5619 '<New EnvAssoc at nodes.lkt:8296:52>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8296'
--# expr-start 5611 '<FieldAccess for previous_part_env_name at nodes.lkt:8297:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 5611
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 5612 '<New DesignatedEnv at nodes.lkt:8298:23>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8298'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 5612
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2748;
   Result_Var_1 := Scope_Result;
else
--# expr-start 5615 '<bind at nodes.lkt:8301:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8301'
--# expr-start 5613 '<FieldAccess for default_initial_env at nodes.lkt:8301:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8301'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5613
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 5614 '<FieldAccess for body_scope at nodes.lkt:8303:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8303'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5614
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5615
Var_Expr_1 := Dyn_Var_Bind_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
   --# bind non_null_env Var_Expr_1
--# expr-start 5616 '<New DesignatedEnv at nodes.lkt:8305:35>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8305'
Inc_Ref (Var_Expr_1);
New_Struct_1 := (Direct_Env => Var_Expr_1, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 5616
Scope_Result_1 := New_Struct_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2749;
   Result_Var := Scope_Result_1;
else
   --# expr-start 5617 '<New DesignatedEnv at nodes.lkt:8307:32>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8307'
Inc_Ref (Empty_Env);
New_Struct_2 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5617
   Result_Var := New_Struct_2;
end if;
      Inc_Ref (Result_Var);
   Result_Var_1 := Result_Var;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 5618 '<Symbol __nextpart at nodes.lkt:8297:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8297'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 5618
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Result_Var_1);
New_Struct_3 := (Dest_Env => Result_Var_1, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5619
         Property_Result := New_Struct_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_886;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_886;
                     Finalizer_Scope_2748;
                     Finalizer_Scope_2749;
            raise;
      end;
   return Property_Result;
end Body_Node_P_Previous_Part_Link_Env_Assoc;
--# end
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Body_Node
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Body_Node_P_Subunit_Decl_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Body_Node
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Body_Node_P_Body_Decl_Scope (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Accept_Stmt_Body
   --
      procedure Initialize_Fields_For_Accept_Stmt_Body
        (Self : Bare_Accept_Stmt_Body
         ; Accept_Stmt_Body_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Accept_Stmt_Body_F_Name := Accept_Stmt_Body_F_Name;
      end Initialize_Fields_For_Accept_Stmt_Body;
   function Accept_Stmt_Body_F_Name
     (Node : Bare_Accept_Stmt_Body) return Bare_Defining_Name
   is
   begin
         return Node.Accept_Stmt_Body_F_Name;
   end;
--# property-start AcceptStmtBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8321'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Defining_Names
  (Node : Bare_Accept_Stmt_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
     Ent : Internal_Entity_Accept_Stmt_Body :=
       Internal_Entity_Accept_Stmt_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_887 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_887 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_887;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5621 '<ArrayLiteral at nodes.lkt:8321:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8321'
--# expr-start 5620 '<FieldAccess for name at nodes.lkt:8321:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8321'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Accept_Stmt_Body_F_Name, Info => Ent.Info);
--# expr-done 5620
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5621
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_887;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_887;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Defining_Names;
--# end
--# property-start AcceptStmtBody.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8324'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Defining_Env
  (Node : Bare_Accept_Stmt_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_888 with Inline_Always;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Accept_Stmt;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_888 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_888;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5624 '<FieldAccess for children_env at nodes.lkt:8325:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8325'
--# expr-start 5623 '<Cast to AcceptStmt at nodes.lkt:8325:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8325'
--# expr-start 5622 '<parent at nodes.lkt:8325:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8325'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5622
   if Fld = null
      or else Fld.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5623
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Cast_Result);
--# expr-done 5624
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_888;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_888;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Defining_Env;
--# end
--# property-start AcceptStmtBody.accept_stmt_previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8327'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_Body_P_Accept_Stmt_Previous_Part
  (Node : Bare_Accept_Stmt_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt_Body  := Bare_Accept_Stmt_Body (Node);
     Ent : Internal_Entity_Accept_Stmt_Body :=
       Internal_Entity_Accept_Stmt_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Entry_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt;
Fld_1 : Internal_Entity_Entry_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5627 '<FieldAccess for corresponding_entry at nodes.lkt:8328:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8328'
--# expr-start 5626 '<Cast to AcceptStmt at nodes.lkt:8328:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8328'
--# expr-start 5625 '<parent at nodes.lkt:8328:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8328'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5625
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5626
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.corresponding_entry
Fld_1 := Libadalang.Implementation.Accept_Stmt_P_Corresponding_Entry (Node => Cast_Result.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5627
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Accept_Stmt_Body_P_Accept_Stmt_Previous_Part;
--# end
   --
   --  Primitives for Bare_Base_Subp_Body
   --
      procedure Base_Subp_Body_Pre_Env_Actions
        (Self            : Bare_Base_Subp_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_53 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_54 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:8389:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_55 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8398:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_56 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_57 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8410:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_58 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_60 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_59 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_62 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_61 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_64 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_63 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_66 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_65 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_68 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_67 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Base_Subp_Body
        (Self : Bare_Base_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Base_Subp_Body_F_Overriding := Base_Subp_Body_F_Overriding;
            Self.Base_Subp_Body_F_Subp_Spec := Base_Subp_Body_F_Subp_Spec;
      end Initialize_Fields_For_Base_Subp_Body;
   function Base_Subp_Body_F_Overriding
     (Node : Bare_Base_Subp_Body) return Bare_Overriding_Node
   is
   begin
         return Node.Base_Subp_Body_F_Overriding;
   end;
   function Base_Subp_Body_F_Subp_Spec
     (Node : Bare_Base_Subp_Body) return Bare_Subp_Spec
   is
   begin
         return Node.Base_Subp_Body_F_Subp_Spec;
   end;
--# property-start BaseSubpBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8337'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Names
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_890 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_890 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_890;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5630 '<ArrayLiteral at nodes.lkt:8338:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8338'
--# expr-start 5629 '<FieldAccess for name at nodes.lkt:8338:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8338'
--# expr-start 5628 '<FieldAccess for subp_spec at nodes.lkt:8338:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8338'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5628
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5629
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_1)); 
--# expr-done 5630
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_890;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_890;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Defining_Names;
--# end
--# property-start BaseSubpBody.constrain_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8341'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Constrain_Prefix
  (Node : Bare_Base_Subp_Body
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_891 with Inline_Always;
      Fld : Logic_Equation;
            procedure Finalizer_Scope_891 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_891;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5631 '<FieldAccess for subp_constrain_prefix at nodes.lkt:8342:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8342'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_constrain_prefix
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Constrain_Prefix (Node => Ent.Node, Prefix => Prefix, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5631
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_891;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_891;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Constrain_Prefix;
--# end
--# property-start BaseSubpBody.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8345'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Env
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_892 with Inline_Always;
            procedure Finalizer_Scope_2751 with Inline_Always;
            procedure Finalizer_Scope_2752 with Inline_Always;
            procedure Finalizer_Scope_2753 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_168 : Internal_Entity_Base_Subp_Spec;
Var_Expr_169 : Internal_Entity_Generic_Subp_Decl;
Var_Expr_170 : Internal_Entity_Generic_Formal_Part;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Subp_Decl;
Fld_10 : Internal_Entity_Generic_Formal_Part;
Fld_11 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Subp_Spec;
Fld_13 : Lexical_Env := Empty_Env;
Fld_14 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_892 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_892;
            procedure Finalizer_Scope_2751 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2751;
            procedure Finalizer_Scope_2752 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2752;
            procedure Finalizer_Scope_2753 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2753;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5653 '<If at nodes.lkt:8346:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5632 '<FieldAccess for in_scope at nodes.lkt:8346:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.in_scope
Fld_14 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 5632
if Fld_14 then
   --# expr-start 5650 '<env_group at nodes.lkt:8346:33>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5649 '<ArrayLiteral at nodes.lkt:8346:33>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5644 '<If at nodes.lkt:8346:34>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5633 '<FieldAccess for subp_spec_or_null at nodes.lkt:8346:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5633
Var_Expr_168 := Fld_4;
if Var_Expr_168 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 5637 '<FieldAccess for paramless at nodes.lkt:8346:37>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Var_Expr_168.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5636 '<FieldAccess for dottable_subp at nodes.lkt:8346:73>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5635 '<FieldAccess for md at nodes.lkt:8346:73>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5634 '<FieldAccess for info at nodes.lkt:8346:73>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
Fld_5 := Ent.Info;
--# expr-done 5634
Fld_6 := Fld_5.Md;
--# expr-done 5635
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 5636
--# property-call-start BaseFormalParamHolder.paramless
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_168.Node, Dottable_Subp => Fld_7, Can_Be => True);
--# end
--# expr-done 5637
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 5642 '<env_group at nodes.lkt:8346:119>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5641 '<ArrayLiteral at nodes.lkt:8346:119>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5638 '<FieldAccess for children_env at nodes.lkt:8346:120>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5638
--# expr-start 5640 '<FieldAccess for defining_env at nodes.lkt:8346:141>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
--# expr-start 5639 '<FieldAccess for subp_spec at nodes.lkt:8346:141>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5639
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5640
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_2)); 
--# expr-done 5641
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5642
   If_Result := Group_Env;
else
   --# expr-start 5643 '<FieldAccess for children_env at nodes.lkt:8346:189>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5643
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 5644
--# expr-start 5645 '<FieldAccess for decl_part at nodes.lkt:8346:210>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8346'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_9 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 5645
Var_Expr_0 := Fld_9;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_0
--# expr-start 5646 '<Cast to GenericSubpDecl at nodes.lkt:8347:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8347'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 5646
Var_Expr_169 := Cast_Result;
if Var_Expr_169 /= No_Entity_Generic_Subp_Decl then
   --# scope-start
--# expr-start 5647 '<FieldAccess for formal_part at nodes.lkt:8347:23>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8347'
      if Var_Expr_169.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_169.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_169.Info);
--# expr-done 5647
Var_Expr_170 := Fld_10;
if Var_Expr_170 /= No_Entity_Generic_Formal_Part then
   --# scope-start
--# expr-start 5648 '<FieldAccess for children_env at nodes.lkt:8347:23>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8347'
      if Var_Expr_170.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_170.Node, E_Info => Var_Expr_170.Info);
--# expr-done 5648
Scope_Result_1 := Fld_11;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2753;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2752;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_3 := Result_Var_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2751;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result, 2 => Result_Var_3)); 
--# expr-done 5649
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 5650
   If_Result_1 := Group_Env_1;
else
   --# expr-start 5652 '<FieldAccess for defining_env at nodes.lkt:8367:13>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8367'
--# expr-start 5651 '<FieldAccess for subp_spec at nodes.lkt:8367:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8367'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5651
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_13 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_12.Node, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 5652
   If_Result_1 := Fld_13;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5653
         Property_Result := If_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_892;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_892;
                     Finalizer_Scope_2751;
                     Finalizer_Scope_2752;
                     Finalizer_Scope_2753;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Defining_Env;
--# end
--# property-start BaseSubpBody.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8370'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Type_Expression
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5655 '<FieldAccess for returns at nodes.lkt:8370:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8370'
--# expr-start 5654 '<FieldAccess for subp_spec at nodes.lkt:8370:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8370'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5654
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5655
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Type_Expression;
--# end
--# property-start BaseSubpBody.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8373'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Expr_Type
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_171 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5656 '<FieldAccess for subp_spec_or_null at nodes.lkt:8374:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8374'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5656
Var_Expr_171 := Fld;
if Var_Expr_171 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 5657 '<FieldAccess for return_type at nodes.lkt:8374:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8374'
      if Var_Expr_171.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_171.Node, Origin => Origin, E_Info => Var_Expr_171.Info);
--# end
--# expr-done 5657
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Expr_Type;
--# end
--# property-start BaseSubpBody.initial_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8376'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Initial_Env_Name
  (Node : Bare_Base_Subp_Body
      ; Follow_Private : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   --# bind follow_private Follow_Private
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5661 '<If at nodes.lkt:8377:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8377'
--# expr-start 5658 '<FieldAccess for is_library_item at nodes.lkt:8377:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8377'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5658
if Fld_2 then
   --# expr-start 5659 '<FieldAccess for child_decl_initial_env_name at nodes.lkt:8377:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8377'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => Follow_Private);
--# end
--# expr-done 5659
   If_Result := Fld;
else
   --# expr-start 5660 '<FieldAccess for body_initial_env_name at nodes.lkt:8377:94>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8377'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env_name
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 5660
   If_Result := Fld_1;
end if;
--# expr-done 5661
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Initial_Env_Name;
--# end
--# property-start BaseSubpBody.previous_part_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8382'
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Previous_Part_Env_Name
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_896 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Fld_1 : String_Type;
Sym_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Fld_3 : Boolean;
If_Result_1 : Symbol_Type;
            procedure Finalizer_Scope_896 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_896;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5670 '<If at nodes.lkt:8383:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
--# expr-start 5662 '<FieldAccess for is_subunit at nodes.lkt:8383:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_3 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5662
if Fld_3 then
   --# expr-start 5665 '<to_symbol at nodes.lkt:8383:35>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
--# expr-start 5663 '<FieldAccess for top_level_env_name at nodes.lkt:8383:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5663
--# expr-start 5664 '<String at nodes.lkt:8383:64>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
Str := Create_String ("__stub"); 
--# expr-done 5664
Concat_Result := Concat_String (Fld, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 5665
   If_Result_1 := Sym;
else
   --# expr-start 5669 '<If at nodes.lkt:8383:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8383'
--# expr-start 5666 '<FieldAccess for is_library_item at nodes.lkt:8384:14>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8384'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5666
if Fld_2 then
   --# expr-start 5668 '<to_symbol at nodes.lkt:8384:42>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8384'
--# expr-start 5667 '<FieldAccess for top_level_env_name at nodes.lkt:8384:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8384'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5667
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld_1); 
--# expr-done 5668
   If_Result := Sym_1;
else
   If_Result := No_Symbol;
end if;
--# expr-done 5669
   If_Result_1 := If_Result;
end if;
--# expr-done 5670
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_896;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_896;
            raise;
      end;
   return Property_Result;
end Base_Subp_Body_P_Previous_Part_Env_Name;
--# end
--# property-start BaseSubpBody._env_do_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8388'
pragma Warnings (Off, "is not referenced");
 function Env_Do_53
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5671 '<FieldAccess for env_hook at nodes.lkt:8388:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8388'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5671
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_53;
--# end
--# property-start BaseSubpBody._env_init_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8390'
pragma Warnings (Off, "is not referenced");
 function Env_Init_54
  (Node : Bare_Base_Subp_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1718 with Inline_Always;
            procedure Finalizer_Scope_2755 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1718 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1718;
            procedure Finalizer_Scope_2755 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2755;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5672 '<FieldAccess for initial_env_name at nodes.lkt:8390:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8390'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => True);
--# end
--# expr-done 5672
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind non_null_name Var_Expr_0
--# expr-start 5673 '<New DesignatedEnv at nodes.lkt:8391:36>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8391'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 5673
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2755;
   Result_Var := Scope_Result;
else
   --# expr-start 5674 '<New DesignatedEnv at nodes.lkt:8393:32>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8393'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5674
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1718;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1718;
                     Finalizer_Scope_2755;
            raise;
      end;
   return Property_Result;
end Env_Init_54;
--# end
--# property-start BaseSubpBody._env_mappings_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8399'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_55
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1719 with Inline_Always;
            procedure Finalizer_Scope_2756 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1719 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1719;
            procedure Finalizer_Scope_2756 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2756;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5678 '<FieldAccess for basic_decl_env_assocs at nodes.lkt:8399:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8399'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5675 '<FieldAccess for initial_env_name at nodes.lkt:8400:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8400'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => False);
--# end
--# expr-done 5675
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind non_null_name Var_Expr_0
--# expr-start 5676 '<New DesignatedEnv at nodes.lkt:8401:40>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8401'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 5676
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2756;
   Result_Var := Scope_Result;
else
   --# expr-start 5677 '<New DesignatedEnv at nodes.lkt:8403:36>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8403'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5677
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
--# property-call-start BasicDecl.basic_decl_env_assocs
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Env_Assocs (Node => Ent.Node, Dest_Env => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 5678
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1719;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1719;
                     Finalizer_Scope_2756;
            raise;
      end;
   return Property_Result;
end Env_Mappings_55;
--# end
--# property-start BaseSubpBody._env_trans_parent_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8409'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_56
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_56;
--# end
--# property-start BaseSubpBody._env_mappings_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8410'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_57
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
     Ent : Internal_Entity_Base_Subp_Body :=
       Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1721 with Inline_Always;
            procedure Finalizer_Scope_2757 with Inline_Always;
            procedure Finalizer_Scope_2758 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Var_Expr_1 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Scope_Result_1 : Internal_Designated_Env := No_Designated_Env;
New_Struct_3 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
Dyn_Var_Bind_Result : Internal_Designated_Env := No_Designated_Env;
Fld_3 : Boolean;
If_Result : Internal_Designated_Env := No_Designated_Env;
Result_Var_1 : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct_4 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1721 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (New_Struct_4);
            end Finalizer_Scope_1721;
            procedure Finalizer_Scope_2757 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2757;
            procedure Finalizer_Scope_2758 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (New_Struct_2);
            end Finalizer_Scope_2758;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5690 '<New EnvAssoc at ???>' New_Struct_4 None
--# expr-start 5679 '<FieldAccess for previous_part_env_name at nodes.lkt:8411:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8411'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 5679
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 5680 '<New DesignatedEnv at nodes.lkt:8412:27>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8412'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 5680
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2757;
   Result_Var_1 := Scope_Result;
else
   --# expr-start 5688 '<If at nodes.lkt:8419:29>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8419'
--# expr-start 5681 '<FieldAccess for has_top_level_env_name at nodes.lkt:8419:32>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8419'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5681
if Fld_3 then
   --# expr-start 5682 '<New DesignatedEnv at nodes.lkt:8419:67>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8419'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => None); 
--# expr-done 5682
   If_Result := New_Struct_1;
else
   --# expr-start 5687 '<bind at nodes.lkt:8422:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8422'
--# expr-start 5683 '<FieldAccess for default_initial_env at nodes.lkt:8422:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8422'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5683
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 5684 '<FieldAccess for body_scope at nodes.lkt:8424:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8424'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_2 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5684
Var_Expr_1 := Fld_2;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
   --# bind non_null_env Var_Expr_1
--# expr-start 5685 '<New DesignatedEnv at nodes.lkt:8425:43>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8425'
Inc_Ref (Var_Expr_1);
New_Struct_2 := (Direct_Env => Var_Expr_1, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 5685
Scope_Result_1 := New_Struct_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2758;
   Result_Var := Scope_Result_1;
else
   --# expr-start 5686 '<New DesignatedEnv at nodes.lkt:8428:37>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8428'
Inc_Ref (Empty_Env);
New_Struct_3 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5686
   Result_Var := New_Struct_3;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5687
   If_Result := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 5688
   Result_Var_1 := If_Result;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 5689 '<Symbol __nextpart at nodes.lkt:8411:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8411'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 5689
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Result_Var_1);
New_Struct_4 := (Dest_Env => Result_Var_1, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5690
         Property_Result := New_Struct_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1721;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1721;
                     Finalizer_Scope_2757;
                     Finalizer_Scope_2758;
            raise;
      end;
   return Property_Result;
end Env_Mappings_57;
--# end
--# property-start BaseSubpBody._env_do_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8435'
pragma Warnings (Off, "is not referenced");
 function Env_Do_58
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1722 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1722 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1722;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5691 '<FieldAccess for populate_dependent_units at nodes.lkt:8435:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8435'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5691
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1722;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1722;
            raise;
      end;
   return Property_Result;
end Env_Do_58;
--# end
--# property-start BaseSubpBody._ref_env_nodes_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8437'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_59
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1723 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1723 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1723;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5692 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:8437:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8437'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5692
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1723;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1723;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_59;
--# end
--# property-start BaseSubpBody._ref_cond_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8437'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_60
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5694 '<IsA LibraryItem, Subunit at nodes.lkt:8437:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8437'
--# expr-start 5693 '<parent at nodes.lkt:8437:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8437'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5693
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5694
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_60;
--# end
--# property-start BaseSubpBody._ref_env_nodes_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8440'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_61
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1725 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1725 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1725;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5695 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:8440:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8440'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5695
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1725;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1725;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_61;
--# end
--# property-start BaseSubpBody._ref_cond_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8440'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_62
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5697 '<IsA LibraryItem, Subunit at nodes.lkt:8440:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8440'
--# expr-start 5696 '<parent at nodes.lkt:8440:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8440'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5696
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5697
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_62;
--# end
--# property-start BaseSubpBody._ref_env_nodes_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_63
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1727 with Inline_Always;
            procedure Finalizer_Scope_2759 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1727 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1727;
            procedure Finalizer_Scope_2759 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2759;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5698 '<Cast to AdaNode at nodes.lkt:8446:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5698
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5699 '<ArrayLiteral at nodes.lkt:8446:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5699
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2759;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1727;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1727;
                     Finalizer_Scope_2759;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_63;
--# end
--# property-start BaseSubpBody._ref_cond_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_64
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5701 '<IsA LibraryItem at nodes.lkt:8446:95>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
--# expr-start 5700 '<parent at nodes.lkt:8446:95>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8446'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5700
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 5701
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_64;
--# end
--# property-start BaseSubpBody._ref_env_nodes_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8449'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_65
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1729 with Inline_Always;
            procedure Finalizer_Scope_2760 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1729 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1729;
            procedure Finalizer_Scope_2760 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2760;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5702 '<Cast to AdaNode at nodes.lkt:8449:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8449'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5702
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_0
--# expr-start 5703 '<ArrayLiteral at nodes.lkt:8449:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8449'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5703
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2760;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1729;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1729;
                     Finalizer_Scope_2760;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_65;
--# end
--# property-start BaseSubpBody._ref_cond_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8449'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_66
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5704 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8449:106>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8449'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5704
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_66;
--# end
--# property-start BaseSubpBody._ref_env_nodes_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8454'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_67
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1731 with Inline_Always;
            procedure Finalizer_Scope_2761 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1731 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1731;
            procedure Finalizer_Scope_2761 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2761;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5705 '<Cast to AdaNode at nodes.lkt:8454:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8454'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5705
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v3 Var_Expr_0
--# expr-start 5706 '<ArrayLiteral at nodes.lkt:8454:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8454'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5706
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2761;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1731;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1731;
                     Finalizer_Scope_2761;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_67;
--# end
--# property-start BaseSubpBody._ref_cond_68 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8454'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_68
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body  := Bare_Base_Subp_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5707 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8454:97>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8454'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5707
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_68;
--# end
   --
   --  Primitives for Bare_Expr_Function
   --
      procedure Initialize_Fields_For_Expr_Function
        (Self : Bare_Expr_Function
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Expr_Function_F_Expr : Bare_Expr
         ; Expr_Function_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Expr_Function_F_Expr := Expr_Function_F_Expr;
            Self.Expr_Function_F_Aspects := Expr_Function_F_Aspects;
      end Initialize_Fields_For_Expr_Function;
   function Expr_Function_F_Expr
     (Node : Bare_Expr_Function) return Bare_Expr
   is
   begin
         return Node.Expr_Function_F_Expr;
   end;
--# property-start ExprFunction.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8465'
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Equation
  (Node : Bare_Expr_Function
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr_Function  := Bare_Expr_Function (Node);
     Ent : Internal_Entity_Expr_Function :=
       Internal_Entity_Expr_Function'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_897 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_897 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_897;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5718 '<And at nodes.lkt:8465:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8465'
--# expr-start 5715 '<And at nodes.lkt:8466:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
--# expr-start 5712 '<Bind at nodes.lkt:8466:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
--# expr-start 5709 '<FieldAccess for expected_type_var at nodes.lkt:8466:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
--# expr-start 5708 '<FieldAccess for expr at nodes.lkt:8466:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 5708
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 5709
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5711 '<FieldAccess for return_type at nodes.lkt:8466:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
--# expr-start 5710 '<FieldAccess for subp_spec at nodes.lkt:8466:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5710
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5711
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8466:9") else null)); 
--# expr-done 5712
--# expr-start 5714 '<FieldAccess for sub_equation at nodes.lkt:8466:78>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
--# expr-start 5713 '<FieldAccess for expr at nodes.lkt:8466:78>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8466'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 5713
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 5714
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8466:9") else null)); 
--# expr-done 5715
--# expr-start 5717 '<FieldAccess for matches_expected_assign_type at nodes.lkt:8467:11>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8467'
--# expr-start 5716 '<FieldAccess for expr at nodes.lkt:8467:11>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8467'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Expr_Function_F_Expr;
--# expr-done 5716
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_6, Origin => Origin);
--# end
--# expr-done 5717
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8465:37") else null)); 
--# expr-done 5718
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_897;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_897;
            raise;
      end;
   return Property_Result;
end Expr_Function_P_Xref_Equation;
--# end
--# property-start ExprFunction.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8469'
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Entry_Point
  (Node : Bare_Expr_Function
  )
   return Boolean
is
   Self : Bare_Expr_Function  := Bare_Expr_Function (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_Function_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Null_Subp_Decl
   --
      procedure Initialize_Fields_For_Null_Subp_Decl
        (Self : Bare_Null_Subp_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Null_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Null_Subp_Decl_F_Aspects := Null_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Null_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Body
   --
      procedure Initialize_Fields_For_Subp_Body
        (Self : Bare_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_F_Aspects : Bare_Aspect_Spec
         ; Subp_Body_F_Decls : Bare_Declarative_Part
         ; Subp_Body_F_Stmts : Bare_Handled_Stmts
         ; Subp_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Body_F_Aspects := Subp_Body_F_Aspects;
            Self.Subp_Body_F_Decls := Subp_Body_F_Decls;
            Self.Subp_Body_F_Stmts := Subp_Body_F_Stmts;
            Self.Subp_Body_F_End_Name := Subp_Body_F_End_Name;
      end Initialize_Fields_For_Subp_Body;
   function Subp_Body_F_Decls
     (Node : Bare_Subp_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Subp_Body_F_Decls;
   end;
   function Subp_Body_F_Stmts
     (Node : Bare_Subp_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Subp_Body_F_Stmts;
   end;
   function Subp_Body_F_End_Name
     (Node : Bare_Subp_Body) return Bare_End_Name
   is
   begin
         return Node.Subp_Body_F_End_Name;
   end;
--# property-start SubpBody.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8484'
pragma Warnings (Off, "is not referenced");
 function Subp_Body_P_Declarative_Parts
  (Node : Bare_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Subp_Body  := Bare_Subp_Body (Node);
     Ent : Internal_Entity_Subp_Body :=
       Internal_Entity_Subp_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_899 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_899 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_899;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5720 '<ArrayLiteral at nodes.lkt:8484:63>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8484'
--# expr-start 5719 '<FieldAccess for decls at nodes.lkt:8484:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8484'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Subp_Body_F_Decls, Info => Ent.Info);
--# expr-done 5719
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Fld)); 
--# expr-done 5720
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_899;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_899;
            raise;
      end;
   return Property_Result;
end Subp_Body_P_Declarative_Parts;
--# end
   --
   --  Primitives for Bare_Subp_Renaming_Decl
   --
      procedure Initialize_Fields_For_Subp_Renaming_Decl
        (Self : Bare_Subp_Renaming_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Renaming_Decl_F_Renames := Subp_Renaming_Decl_F_Renames;
            Self.Subp_Renaming_Decl_F_Aspects := Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Renaming_Decl;
   function Subp_Renaming_Decl_F_Renames
     (Node : Bare_Subp_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Subp_Renaming_Decl_F_Renames;
   end;
--# property-start SubpRenamingDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8492'
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Subp_Renaming_Decl  := Bare_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start SubpRenamingDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8495'
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Subp_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subp_Renaming_Decl  := Bare_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Subp_Renaming_Decl :=
       Internal_Entity_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_901 with Inline_Always;
            procedure Finalizer_Scope_2762 with Inline_Always;
            procedure Finalizer_Scope_2763 with Inline_Always;
            procedure Finalizer_Scope_2764 with Inline_Always;
      Local_Renamed_Object : Internal_Entity_Name;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Explicit_Deref;
Match_2 : Internal_Entity_Name;
Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Equation;
Fld_8 : Bare_Name;
Fld_9 : Logic_Equation;
Fld_10 : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity_Dotted_Name;
Fld_12 : Internal_Entity_Name;
Fld_13 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Subp_Renaming_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
Fld_14 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Subp_Renaming_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Pred_1 : Logic_Equation;
Fld_15 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Subp_Renaming_Decl;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Pred_2 : Logic_Equation;
Is_A : Boolean;
If_Result : Logic_Equation;
Is_A_1 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_A_2 : Boolean;
If_Result_2 : Logic_Equation;
Is_A_3 : Boolean;
If_Result_3 : Logic_Equation;
True_Rel : Logic_Equation;
False_Rel : Logic_Equation;
Fld_16 : Boolean;
If_Result_4 : Logic_Equation;
Or_Pred : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_901 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Pred_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (True_Rel);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_901;
            procedure Finalizer_Scope_2762 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2762;
            procedure Finalizer_Scope_2763 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2763;
            procedure Finalizer_Scope_2764 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2764;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5759 '<Let at nodes.lkt:8496:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8496'
--# scope-start
--# expr-start 5722 '<FieldAccess for renamed_object at nodes.lkt:8496:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8496'
--# expr-start 5721 '<FieldAccess for renames at nodes.lkt:8496:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8496'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5721
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 5722
Local_Renamed_Object := Fld_1; 
--# bind renamed_object Local_Renamed_Object
--# expr-start 5758 '<Or at nodes.lkt:8498:9>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8498'
--# expr-start 5755 '<If at nodes.lkt:8499:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8499'
--# expr-start 5723 '<IsA CharLiteral at nodes.lkt:8499:16>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8499'
Is_A_3 := Local_Renamed_Object.Node /= null 
and then Local_Renamed_Object.Node.Kind in Ada_Char_Literal_Range; 
--# expr-done 5723
if Is_A_3 then
   --# expr-start 5729 '<And at nodes.lkt:8502:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
--# expr-start 5727 '<Bind at nodes.lkt:8502:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
--# expr-start 5724 '<FieldAccess for expected_type_var at nodes.lkt:8502:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Local_Renamed_Object.Node);
--# expr-done 5724
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 5726 '<FieldAccess for return_type at nodes.lkt:8502:57>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
--# expr-start 5725 '<FieldAccess for subp_spec at nodes.lkt:8502:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5725
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 5726
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8502:17") else null)); 
--# expr-done 5727
--# expr-start 5728 '<FieldAccess for sub_equation at nodes.lkt:8502:91>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8502'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Renamed_Object.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Renamed_Object.Info);
--# end
--# expr-done 5728
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8502:17") else null)); 
--# expr-done 5729
   If_Result_3 := And_Pred;
else
   --# expr-start 5754 '<If at nodes.lkt:8499:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8499'
--# expr-start 5730 '<IsA AttributeRef at nodes.lkt:8504:18>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8504'
Is_A_2 := Local_Renamed_Object.Node /= null 
and then Local_Renamed_Object.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 5730
if Is_A_2 then
   --# expr-start 5731 '<FieldAccess for sub_equation at nodes.lkt:8507:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8507'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Renamed_Object.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Renamed_Object.Info);
--# end
--# expr-done 5731
   If_Result_2 := Fld_6;
else
   --# expr-start 5753 '<And at nodes.lkt:8509:18>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8509'
--# expr-start 5736 '<Match at nodes.lkt:8509:18>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8509'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Renamed_Object; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind ce Match_0
--# expr-start 5732 '<FieldAccess for sub_equation at nodes.lkt:8513:38>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8513'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 5732
Let_Result := Fld_7; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2762;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Explicit_Deref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind ed Match_1
--# expr-start 5734 '<FieldAccess for general_xref_equation at nodes.lkt:8514:43>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8514'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5733 '<FieldAccess for node at nodes.lkt:8514:68>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8514'
Fld_8 := Local_Renamed_Object.Node;
--# expr-done 5733
--# property-call-start ExplicitDeref.general_xref_equation
Fld_9 := Libadalang.Implementation.Explicit_Deref_P_General_Xref_Equation (Node => Match_1.Node, Root => Fld_8, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 5734
Let_Result_1 := Fld_9; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2763;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind o Match_2
--# expr-start 5735 '<FieldAccess for xref_no_overloading at nodes.lkt:8517:27>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8517'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_2.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 5735
Let_Result_2 := Fld_10; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2764;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 5736
--# expr-start 5752 '<If at nodes.lkt:8519:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8519'
--# expr-start 5737 '<IsA DottedName at nodes.lkt:8519:20>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8519'
Is_A_1 := Local_Renamed_Object.Node /= null 
and then Local_Renamed_Object.Node.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5737
if Is_A_1 then
   --# expr-start 5743 '<Predicate on BasicDecl.subp_renaming_decl_match_signature at nodes.lkt:8520:21>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8520'
--# expr-start 5738 '<FieldAccess for ref_var at nodes.lkt:8522:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8522'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Renamed_Object.Node);
--# end
--# expr-done 5738
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 5741 '<FieldAccess for ref_var at nodes.lkt:8523:25>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8523'
--# expr-start 5740 '<FieldAccess for prefix at nodes.lkt:8523:25>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8523'
--# expr-start 5739 '<Cast to DottedName at nodes.lkt:8523:25>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8523'
Cast_Expr := Local_Renamed_Object; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Dotted_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_4 := No_Entity_Dotted_Name;
   end if;
--# expr-done 5739
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Cast_Result_4.Node.Dotted_Name_F_Prefix, Info => Cast_Result_4.Info);
--# expr-done 5740
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12.Node);
--# end
--# expr-done 5741
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 5742 '<Cast to BasicDecl at nodes.lkt:8524:25>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8524'
Cast_Expr_1 := Ent; 
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-done 5742
Pred := Solver.Create_N_Predicate ((Fld_11, Fld_13), Create_Basic_Decl_P_Subp_Renaming_Decl_Match_Signature_0_Predicate (Cast_Result_5, Origin)); 
--# expr-done 5743
   If_Result_1 := Pred;
else
   --# expr-start 5751 '<If at nodes.lkt:8519:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8519'
--# expr-start 5744 '<IsA ExplicitDeref at nodes.lkt:8526:22>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8526'
Is_A := Local_Renamed_Object.Node /= null 
and then Local_Renamed_Object.Node.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 5744
if Is_A then
   --# expr-start 5747 '<Predicate on BasicDecl.access_to_subp_decl_match_signature at nodes.lkt:8527:21>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8527'
--# expr-start 5745 '<FieldAccess for ref_var at nodes.lkt:8529:25>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8529'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Renamed_Object.Node);
--# end
--# expr-done 5745
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 5746 '<Cast to BasicDecl at nodes.lkt:8530:25>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8530'
Cast_Expr_2 := Ent; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-done 5746
Pred_1 := Solver.Create_Predicate (Fld_14, Create_Basic_Decl_P_Access_To_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_6, Origin)); 
--# expr-done 5747
   If_Result := Pred_1;
else
   --# expr-start 5750 '<Predicate on BasicDecl.subp_decl_match_signature at nodes.lkt:8533:21>' Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8533'
--# expr-start 5748 '<FieldAccess for ref_var at nodes.lkt:8535:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8535'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Renamed_Object.Node);
--# end
--# expr-done 5748
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 5749 '<Cast to BasicDecl at nodes.lkt:8536:25>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8536'
Cast_Expr_3 := Ent; 
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-done 5749
Pred_2 := Solver.Create_Predicate (Fld_15, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_7, Origin)); 
--# expr-done 5750
   If_Result := Pred_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 5751
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5752
And_Pred_1 := Create_And (Match_Result, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8509:18") else null)); 
--# expr-done 5753
   If_Result_2 := And_Pred_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 5754
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 5755
--# expr-start 5757 '<If at nodes.lkt:8542:13>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8542'
--# expr-start 5756 '<FieldAccess for is_operator_name at nodes.lkt:8542:16>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8542'
      if Local_Renamed_Object.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_operator_name
Fld_16 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Local_Renamed_Object.Node, E_Info => Local_Renamed_Object.Info);
--# end
--# expr-done 5756
if Fld_16 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8542:55") else null)); 
   If_Result_4 := True_Rel;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8542:66") else null)); 
   If_Result_4 := False_Rel;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 5757
Or_Pred := Create_Or (If_Result_3, If_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8498:9") else null)); 
--# expr-done 5758
Let_Result_3 := Or_Pred; Inc_Ref (Let_Result_3);
--# end
--# expr-done 5759
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_901;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_901;
                     Finalizer_Scope_2762;
                     Finalizer_Scope_2763;
                     Finalizer_Scope_2764;
            raise;
      end;
   return Property_Result;
end Subp_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Body_Stub
   --
--# property-start BodyStub.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8552'
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Next_Part_For_Decl
  (Node : Bare_Body_Stub
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
     Ent : Internal_Entity_Body_Stub :=
       Internal_Entity_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_902 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_902 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_902;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5762 '<as_entity at nodes.lkt:8552:51>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8552'
--# expr-start 5761 '<FieldAccess for get_unit_root_decl at nodes.lkt:8552:51>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8552'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5760 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:8553:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8553'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.syntactic_fully_qualified_name
Fld := Libadalang.Implementation.Body_Stub_P_Syntactic_Fully_Qualified_Name (Node => Ent.Node);
--# end
--# expr-done 5760
--# property-call-start AdaNode.get_unit_root_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld, Kind => Unit_Body, Process_Parents => False);
--# end
--# expr-done 5761
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 5762
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_902;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_902;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Next_Part_For_Decl;
--# end
--# property-start BodyStub.syntactic_fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8563'
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_903 with Inline_Always;
      Local_Rel_Name : Symbol_Type_Array_Access;
Local_Top_Body : Bare_Body_Node;
Match_0 : Bare_Subp_Body;
Match_1 : Bare_Protected_Body;
Match_2 : Bare_Package_Body;
Match_3 : Bare_Task_Body;
Match_4 : Bare_Ada_Node;
Local_Cu : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
Cast_Expr : Bare_Subp_Body;
Cast_Result_1 : Bare_Body_Node;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Protected_Body;
Cast_Expr_1 : Bare_Protected_Body;
Cast_Result_3 : Bare_Body_Node;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Package_Body;
Cast_Expr_2 : Bare_Package_Body;
Cast_Result_5 : Bare_Body_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Cast_Result_6 : Bare_Task_Body;
Cast_Expr_3 : Bare_Task_Body;
Cast_Result_7 : Bare_Body_Node;
Let_Result_3 : Bare_Body_Node;
Scope_Result_3 : Bare_Body_Node;
Cast_Result_8 : Bare_Ada_Node;
Let_Result_4 : Bare_Body_Node;
Scope_Result_4 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Ada_Node;
Fld_6 : Bare_Ada_Node;
Cast_Result_9 : Bare_Compilation_Unit;
Fld_7 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Bare_Compilation_Unit;
Fld_8 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Let_Result_5 : Symbol_Type_Array_Access;
Let_Result_6 : Symbol_Type_Array_Access;
Let_Result_7 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_903 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Rel_Name);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_903;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 5779 '<Let at nodes.lkt:8565:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8565'
--# scope-start
--# expr-start 5765 '<FieldAccess for as_symbol_array at nodes.lkt:8565:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8565'
--# expr-start 5764 '<FieldAccess for defining_name at nodes.lkt:8565:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8565'
--# expr-start 5763 '<as_bare_entity at nodes.lkt:8565:24>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8565'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5763
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5764
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 5765
Local_Rel_Name := Fld_1; Inc_Ref (Local_Rel_Name);
--# bind rel_name Local_Rel_Name
--# expr-start 5778 '<Let at nodes.lkt:8575:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8575'
--# scope-start
--# expr-start 5769 '<Match at nodes.lkt:8575:24>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8575'
--# expr-start 5768 '<parent at nodes.lkt:8575:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8575'
--# expr-start 5767 '<parent at nodes.lkt:8575:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8575'
--# expr-start 5766 '<parent at nodes.lkt:8575:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8575'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Parent; 
--# expr-done 5766
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Fld_2.Parent; 
--# expr-done 5767
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 5768
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_4; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind b Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Cast_Expr;
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind b Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Cast_Expr_1;
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind b Match_2
Cast_Expr_2 := Match_2; 
      Cast_Result_5 := Cast_Expr_2;
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_3 := Cast_Result_6; 
--# bind b Match_3
Cast_Expr_3 := Match_3; 
      Cast_Result_7 := Cast_Expr_3;
Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := raise Property_Error with "invalid body stub"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 5769
Local_Top_Body := Match_Result; 
--# bind top_body Local_Top_Body
--# expr-start 5777 '<Let at nodes.lkt:8583:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8583'
--# scope-start
--# expr-start 5775 '<If at nodes.lkt:8583:18>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8583'
--# expr-start 5771 '<IsA LibraryItem, Subunit at nodes.lkt:8583:21>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8583'
--# expr-start 5770 '<parent at nodes.lkt:8583:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8583'
      if Local_Top_Body = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Local_Top_Body.Parent; 
--# expr-done 5770
Is_A := Fld_7 /= null 
and then Fld_7.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5771
if Is_A then
   --# expr-start 5774 '<Cast to CompilationUnit at nodes.lkt:8584:13>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8584'
--# expr-start 5773 '<parent at nodes.lkt:8584:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8584'
--# expr-start 5772 '<parent at nodes.lkt:8584:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8584'
      if Local_Top_Body = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Local_Top_Body.Parent; 
--# expr-done 5772
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Parent; 
--# expr-done 5773
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_9 := Fld_6;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 5774
   If_Result_1 := Cast_Result_9;
else
   If_Result_1 := raise Property_Error with "invalid body stub";
end if;
--# expr-done 5775
Local_Cu := If_Result_1; 
--# bind cu Local_Cu
--# expr-start 5776 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:8588:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8588'
      if Local_Cu = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_8 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Local_Cu);
--# end
--# expr-done 5776
Concat_Result := Concat (Fld_8, Local_Rel_Name); 
Let_Result_5 := Concat_Result; Inc_Ref (Let_Result_5);
--# end
--# expr-done 5777
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 5778
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 5779
         Property_Result := Let_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_903;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_903;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start BodyStub.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8593'
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Env_Names
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_904 with Inline_Always;
      Fld : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_904 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_904;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5783 '<ArrayLiteral at nodes.lkt:8594:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8594'
--# expr-start 5782 '<to_symbol at nodes.lkt:8594:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8594'
--# expr-start 5780 '<FieldAccess for top_level_env_name at nodes.lkt:8594:11>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8594'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5780
--# expr-start 5781 '<String at nodes.lkt:8594:39>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8594'
Str := Create_String ("__stub"); 
--# expr-done 5781
Concat_Result := Concat_String (Fld, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 5782
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 5783
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_904;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_904;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Env_Names;
--# end
--# property-start BodyStub.previous_part_link_env_assoc 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8596'
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Previous_Part_Link_Env_Assoc
  (Node : Bare_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Body_Stub  := Bare_Body_Stub (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_905 with Inline_Always;
      Sym : Symbol_Type;
Fld : String_Type;
Sym_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_905 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_905;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5788 '<New EnvAssoc at nodes.lkt:8596:52>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8596'
--# expr-start 5786 '<New DesignatedEnv at nodes.lkt:8597:49>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8597'
--# expr-start 5785 '<to_symbol at nodes.lkt:8598:56>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8598'
--# expr-start 5784 '<FieldAccess for top_level_env_name at nodes.lkt:8598:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8598'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 5784
Sym_1 := String_To_Symbol (Self, Self.Unit.Context, Fld); 
--# expr-done 5785
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym_1, Kind => Named_Env); 
--# expr-done 5786
--# expr-start 5787 '<Symbol __nextpart at nodes.lkt:8597:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8597'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 5787
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5788
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_905;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_905;
            raise;
      end;
   return Property_Result;
end Body_Stub_P_Previous_Part_Link_Env_Assoc;
--# end
   --
   --  Primitives for Bare_Package_Body_Stub
   --
      procedure Package_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Package_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_69 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8611:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_70 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_71 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Package_Body_Stub
        (Self : Bare_Package_Body_Stub
         ; Package_Body_Stub_F_Name : Bare_Defining_Name
         ; Package_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Body_Stub_F_Name := Package_Body_Stub_F_Name;
            Self.Package_Body_Stub_F_Aspects := Package_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Package_Body_Stub;
   function Package_Body_Stub_F_Name
     (Node : Bare_Package_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_Stub_F_Name;
   end;
--# property-start PackageBodyStub.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8608'
pragma Warnings (Off, "is not referenced");
 function Package_Body_Stub_P_Defining_Names
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
     Ent : Internal_Entity_Package_Body_Stub :=
       Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_906 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_906 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_906;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5790 '<ArrayLiteral at nodes.lkt:8608:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8608'
--# expr-start 5789 '<FieldAccess for name at nodes.lkt:8608:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8608'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5789
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5790
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_906;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_906;
            raise;
      end;
   return Property_Result;
end Package_Body_Stub_P_Defining_Names;
--# end
--# property-start PackageBodyStub._env_mappings_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8611'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_69
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
     Ent : Internal_Entity_Package_Body_Stub :=
       Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1733 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1733 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1733;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5791 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8611:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8611'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5791
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1733;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1733;
            raise;
      end;
   return Property_Result;
end Env_Mappings_69;
--# end
--# property-start PackageBodyStub._env_trans_parent_70 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_70
  (Node : Bare_Package_Body_Stub
  )
   return Boolean
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_70;
--# end
--# property-start PackageBodyStub._env_names_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8612'
pragma Warnings (Off, "is not referenced");
 function Env_Names_71
  (Node : Bare_Package_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Body_Stub  := Bare_Package_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1735 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1735 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1735;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5792 '<FieldAccess for env_names at nodes.lkt:8612:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8612'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5792
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1735;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1735;
            raise;
      end;
   return Property_Result;
end Env_Names_71;
--# end
   --
   --  Primitives for Bare_Protected_Body_Stub
   --
      procedure Protected_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Protected_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_72 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8624:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_73 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_74 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Body_Stub
        (Self : Bare_Protected_Body_Stub
         ; Protected_Body_Stub_F_Name : Bare_Defining_Name
         ; Protected_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Protected_Body_Stub_F_Name := Protected_Body_Stub_F_Name;
            Self.Protected_Body_Stub_F_Aspects := Protected_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Protected_Body_Stub;
   function Protected_Body_Stub_F_Name
     (Node : Bare_Protected_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_Stub_F_Name;
   end;
--# property-start ProtectedBodyStub.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8621'
pragma Warnings (Off, "is not referenced");
 function Protected_Body_Stub_P_Defining_Names
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
     Ent : Internal_Entity_Protected_Body_Stub :=
       Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_907 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_907 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_907;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5794 '<ArrayLiteral at nodes.lkt:8621:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8621'
--# expr-start 5793 '<FieldAccess for name at nodes.lkt:8621:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8621'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5793
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5794
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_907;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_907;
            raise;
      end;
   return Property_Result;
end Protected_Body_Stub_P_Defining_Names;
--# end
--# property-start ProtectedBodyStub._env_mappings_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8624'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_72
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
     Ent : Internal_Entity_Protected_Body_Stub :=
       Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1736 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1736 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1736;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5795 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8624:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8624'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5795
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1736;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1736;
            raise;
      end;
   return Property_Result;
end Env_Mappings_72;
--# end
--# property-start ProtectedBodyStub._env_trans_parent_73 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_73
  (Node : Bare_Protected_Body_Stub
  )
   return Boolean
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_73;
--# end
--# property-start ProtectedBodyStub._env_names_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8625'
pragma Warnings (Off, "is not referenced");
 function Env_Names_74
  (Node : Bare_Protected_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Body_Stub  := Bare_Protected_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1738 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1738 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1738;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5796 '<FieldAccess for env_names at nodes.lkt:8625:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8625'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5796
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1738;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1738;
            raise;
      end;
   return Property_Result;
end Env_Names_74;
--# end
   --
   --  Primitives for Bare_Subp_Body_Stub
   --
      procedure Subp_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Subp_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_75 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8643:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_76 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_77 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      if Libadalang.Implementation.Ref_Cond_79 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_78 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_81 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_80 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Subp_Body_Stub
        (Self : Bare_Subp_Body_Stub
         ; Subp_Body_Stub_F_Overriding : Bare_Overriding_Node
         ; Subp_Body_Stub_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Subp_Body_Stub_F_Overriding := Subp_Body_Stub_F_Overriding;
            Self.Subp_Body_Stub_F_Subp_Spec := Subp_Body_Stub_F_Subp_Spec;
            Self.Subp_Body_Stub_F_Aspects := Subp_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Subp_Body_Stub;
   function Subp_Body_Stub_F_Overriding
     (Node : Bare_Subp_Body_Stub) return Bare_Overriding_Node
   is
   begin
         return Node.Subp_Body_Stub_F_Overriding;
   end;
   function Subp_Body_Stub_F_Subp_Spec
     (Node : Bare_Subp_Body_Stub) return Bare_Subp_Spec
   is
   begin
         return Node.Subp_Body_Stub_F_Subp_Spec;
   end;
--# property-start SubpBodyStub.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8635'
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Defining_Names
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
     Ent : Internal_Entity_Subp_Body_Stub :=
       Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_908 with Inline_Always;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_908 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_908;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5799 '<ArrayLiteral at nodes.lkt:8636:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8636'
--# expr-start 5798 '<FieldAccess for name at nodes.lkt:8636:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8636'
--# expr-start 5797 '<FieldAccess for subp_spec at nodes.lkt:8636:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5797
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5798
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_1)); 
--# expr-done 5799
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_908;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_908;
            raise;
      end;
   return Property_Result;
end Subp_Body_Stub_P_Defining_Names;
--# end
--# property-start SubpBodyStub.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8640'
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Type_Expression
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
     Ent : Internal_Entity_Subp_Body_Stub :=
       Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5801 '<FieldAccess for returns at nodes.lkt:8640:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8640'
--# expr-start 5800 '<FieldAccess for subp_spec at nodes.lkt:8640:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8640'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 5800
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5801
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subp_Body_Stub_P_Type_Expression;
--# end
--# property-start SubpBodyStub._env_mappings_75 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8643'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_75
  (Node : Bare_Subp_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1739 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1739 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1739;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5804 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 5802 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5802
--# expr-start 5803 '<FieldAccess for name_symbol at nodes.lkt:8643:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8643'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5803
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5804
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1739;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1739;
            raise;
      end;
   return Property_Result;
end Env_Mappings_75;
--# end
--# property-start SubpBodyStub._env_trans_parent_76 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_76
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_76;
--# end
--# property-start SubpBodyStub._env_names_77 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8644'
pragma Warnings (Off, "is not referenced");
 function Env_Names_77
  (Node : Bare_Subp_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1741 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1741 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1741;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5805 '<FieldAccess for env_names at nodes.lkt:8644:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8644'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5805
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1741;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1741;
            raise;
      end;
   return Property_Result;
end Env_Names_77;
--# end
--# property-start SubpBodyStub._ref_env_nodes_78 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8646'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_78
  (Node : Bare_Subp_Body_Stub
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1742 with Inline_Always;
            procedure Finalizer_Scope_2770 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1742 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1742;
            procedure Finalizer_Scope_2770 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2770;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5806 '<Cast to AdaNode at nodes.lkt:8646:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8646'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5806
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5807 '<ArrayLiteral at nodes.lkt:8646:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8646'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5807
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2770;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1742;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1742;
                     Finalizer_Scope_2770;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_78;
--# end
--# property-start SubpBodyStub._ref_cond_79 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8646'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_79
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5808 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8646:106>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8646'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5808
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_79;
--# end
--# property-start SubpBodyStub._ref_env_nodes_80 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8651'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_80
  (Node : Bare_Subp_Body_Stub
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1744 with Inline_Always;
            procedure Finalizer_Scope_2771 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1744 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1744;
            procedure Finalizer_Scope_2771 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2771;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5809 '<Cast to AdaNode at nodes.lkt:8651:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8651'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5809
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_0
--# expr-start 5810 '<ArrayLiteral at nodes.lkt:8651:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8651'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5810
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2771;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1744;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1744;
                     Finalizer_Scope_2771;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_80;
--# end
--# property-start SubpBodyStub._ref_cond_81 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8651'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_81
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub  := Bare_Subp_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5811 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8651:97>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8651'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5811
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_81;
--# end
   --
   --  Primitives for Bare_Task_Body_Stub
   --
      procedure Task_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Task_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_82 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8664:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_83 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_84 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Body_Stub
        (Self : Bare_Task_Body_Stub
         ; Task_Body_Stub_F_Name : Bare_Defining_Name
         ; Task_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Task_Body_Stub_F_Name := Task_Body_Stub_F_Name;
            Self.Task_Body_Stub_F_Aspects := Task_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Task_Body_Stub;
   function Task_Body_Stub_F_Name
     (Node : Bare_Task_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_Stub_F_Name;
   end;
--# property-start TaskBodyStub.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8661'
pragma Warnings (Off, "is not referenced");
 function Task_Body_Stub_P_Defining_Names
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
     Ent : Internal_Entity_Task_Body_Stub :=
       Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_910 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_910 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_910;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5813 '<ArrayLiteral at nodes.lkt:8661:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8661'
--# expr-start 5812 '<FieldAccess for name at nodes.lkt:8661:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8661'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 5812
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5813
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_910;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_910;
            raise;
      end;
   return Property_Result;
end Task_Body_Stub_P_Defining_Names;
--# end
--# property-start TaskBodyStub._env_mappings_82 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8664'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_82
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
     Ent : Internal_Entity_Task_Body_Stub :=
       Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1746 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1746 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1746;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5814 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8664:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8664'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5814
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1746;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1746;
            raise;
      end;
   return Property_Result;
end Env_Mappings_82;
--# end
--# property-start TaskBodyStub._env_trans_parent_83 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_83
  (Node : Bare_Task_Body_Stub
  )
   return Boolean
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_83;
--# end
--# property-start TaskBodyStub._env_names_84 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8665'
pragma Warnings (Off, "is not referenced");
 function Env_Names_84
  (Node : Bare_Task_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Body_Stub  := Bare_Task_Body_Stub (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1748 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1748 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1748;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5815 '<FieldAccess for env_names at nodes.lkt:8665:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8665'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 5815
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1748;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1748;
            raise;
      end;
   return Property_Result;
end Env_Names_84;
--# end
   --
   --  Primitives for Bare_Entry_Body
   --
      procedure Entry_Body_Pre_Env_Actions
        (Self            : Bare_Entry_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_85 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_86 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:8692:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_87 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8702:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_88 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Body
        (Self : Bare_Entry_Body
         ; Entry_Body_F_Entry_Name : Bare_Defining_Name
         ; Entry_Body_F_Index_Spec : Bare_Entry_Index_Spec
         ; Entry_Body_F_Params : Bare_Entry_Completion_Formal_Params
         ; Entry_Body_F_Aspects : Bare_Aspect_Spec
         ; Entry_Body_F_Barrier : Bare_Expr
         ; Entry_Body_F_Decls : Bare_Declarative_Part
         ; Entry_Body_F_Stmts : Bare_Handled_Stmts
         ; Entry_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Entry_Body_F_Entry_Name := Entry_Body_F_Entry_Name;
            Self.Entry_Body_F_Index_Spec := Entry_Body_F_Index_Spec;
            Self.Entry_Body_F_Params := Entry_Body_F_Params;
            Self.Entry_Body_F_Aspects := Entry_Body_F_Aspects;
            Self.Entry_Body_F_Barrier := Entry_Body_F_Barrier;
            Self.Entry_Body_F_Decls := Entry_Body_F_Decls;
            Self.Entry_Body_F_Stmts := Entry_Body_F_Stmts;
            Self.Entry_Body_F_End_Name := Entry_Body_F_End_Name;
      end Initialize_Fields_For_Entry_Body;
   function Entry_Body_F_Entry_Name
     (Node : Bare_Entry_Body) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Body_F_Entry_Name;
   end;
   function Entry_Body_F_Index_Spec
     (Node : Bare_Entry_Body) return Bare_Entry_Index_Spec
   is
   begin
         return Node.Entry_Body_F_Index_Spec;
   end;
   function Entry_Body_F_Params
     (Node : Bare_Entry_Body) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Entry_Body_F_Params;
   end;
   function Entry_Body_F_Barrier
     (Node : Bare_Entry_Body) return Bare_Expr
   is
   begin
         return Node.Entry_Body_F_Barrier;
   end;
   function Entry_Body_F_Decls
     (Node : Bare_Entry_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Entry_Body_F_Decls;
   end;
   function Entry_Body_F_Stmts
     (Node : Bare_Entry_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Entry_Body_F_Stmts;
   end;
   function Entry_Body_F_End_Name
     (Node : Bare_Entry_Body) return Bare_End_Name
   is
   begin
         return Node.Entry_Body_F_End_Name;
   end;
--# property-start EntryBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8680'
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Defining_Names
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_911 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_911 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_911;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5817 '<ArrayLiteral at nodes.lkt:8680:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8680'
--# expr-start 5816 '<FieldAccess for entry_name at nodes.lkt:8680:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8680'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Body_F_Entry_Name, Info => Ent.Info);
--# expr-done 5816
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5817
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_911;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_911;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Defining_Names;
--# end
--# property-start EntryBody.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8682'
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Declarative_Parts
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_912 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_912 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_912;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5819 '<ArrayLiteral at nodes.lkt:8682:63>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8682'
--# expr-start 5818 '<FieldAccess for decls at nodes.lkt:8682:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8682'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Entry_Body_F_Decls, Info => Ent.Info);
--# expr-done 5818
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Fld)); 
--# expr-done 5819
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_912;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_912;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Declarative_Parts;
--# end
--# property-start EntryBody.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8684'
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Entry_Point
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Xref_Entry_Point;
--# end
--# property-start EntryBody.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8687'
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Equation
  (Node : Bare_Entry_Body
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_914 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_914 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_914;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5829 '<And at nodes.lkt:8688:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5826 '<And at nodes.lkt:8688:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5823 '<Bind at nodes.lkt:8688:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5821 '<FieldAccess for expected_type_var at nodes.lkt:8688:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5820 '<FieldAccess for barrier at nodes.lkt:8688:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Entry_Body_F_Barrier;
--# expr-done 5820
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 5821
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 5822 '<FieldAccess for bool_type at nodes.lkt:8688:47>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 5822
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8688:9") else null)); 
--# expr-done 5823
--# expr-start 5825 '<FieldAccess for sub_equation at nodes.lkt:8688:69>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5824 '<FieldAccess for barrier at nodes.lkt:8688:69>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 5824
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 5825
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8688:9") else null)); 
--# expr-done 5826
--# expr-start 5828 '<FieldAccess for matches_expected_formal_type at nodes.lkt:8688:101>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
--# expr-start 5827 '<FieldAccess for barrier at nodes.lkt:8688:101>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8688'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 5827
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 5828
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8688:9") else null)); 
--# expr-done 5829
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_914;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_914;
            raise;
      end;
   return Property_Result;
end Entry_Body_P_Xref_Equation;
--# end
--# property-start EntryBody._env_do_85 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8691'
pragma Warnings (Off, "is not referenced");
 function Env_Do_85
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5830 '<FieldAccess for env_hook at nodes.lkt:8691:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8691'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5830
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_85;
--# end
--# property-start EntryBody._env_init_86 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8693'
pragma Warnings (Off, "is not referenced");
 function Env_Init_86
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Designated_Env
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1750 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
New_Struct : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1750 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1750;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5834 '<New DesignatedEnv at nodes.lkt:8693:13>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8693'
--# expr-start 5833 '<bind at nodes.lkt:8695:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8695'
--# expr-start 5831 '<FieldAccess for default_initial_env at nodes.lkt:8695:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8695'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 5831
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5832 '<FieldAccess for body_scope at nodes.lkt:8697:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8697'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 5832
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5833
Inc_Ref (Dyn_Var_Bind_Result);
New_Struct := (Direct_Env => Dyn_Var_Bind_Result, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 5834
         Property_Result := New_Struct;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1750;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1750;
            raise;
      end;
   return Property_Result;
end Env_Init_86;
--# end
--# property-start EntryBody._env_mappings_87 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8703'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_87
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
     Ent : Internal_Entity_Entry_Body :=
       Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1751 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Body;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1751 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1751;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5838 '<ArrayLiteral at nodes.lkt:8703:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8703'
--# expr-start 5837 '<New EnvAssoc at nodes.lkt:8703:14>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8703'
--# expr-start 5835 '<New DesignatedEnv at nodes.lkt:8704:62>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8704'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5835
--# expr-start 5836 '<FieldAccess for name_symbol at nodes.lkt:8704:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8704'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5836
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5837
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => New_Struct_1)); 
--# expr-done 5838
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1751;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1751;
            raise;
      end;
   return Property_Result;
end Env_Mappings_87;
--# end
--# property-start EntryBody._env_trans_parent_88 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_88
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body  := Bare_Entry_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_88;
--# end
   --
   --  Primitives for Bare_Package_Body
   --
      procedure Package_Body_Pre_Env_Actions
        (Self            : Bare_Package_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_89 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_90 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:8741:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_91 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8742:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_92 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_93 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_95 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_94 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_97 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_96 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_99 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_98 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_101 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_100 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_103 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_102 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_105 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_104 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_107 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_106 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Package_Body_P_Package_Decl_Use_Clauses_Envs'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Body
        (Self : Bare_Package_Body
         ; Package_Body_F_Package_Name : Bare_Defining_Name
         ; Package_Body_F_Aspects : Bare_Aspect_Spec
         ; Package_Body_F_Decls : Bare_Declarative_Part
         ; Package_Body_F_Stmts : Bare_Handled_Stmts
         ; Package_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Package_Body_F_Package_Name := Package_Body_F_Package_Name;
            Self.Package_Body_F_Aspects := Package_Body_F_Aspects;
            Self.Package_Body_F_Decls := Package_Body_F_Decls;
            Self.Package_Body_F_Stmts := Package_Body_F_Stmts;
            Self.Package_Body_F_End_Name := Package_Body_F_End_Name;
      end Initialize_Fields_For_Package_Body;
   function Package_Body_F_Package_Name
     (Node : Bare_Package_Body) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_F_Package_Name;
   end;
   function Package_Body_F_Decls
     (Node : Bare_Package_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Package_Body_F_Decls;
   end;
   function Package_Body_F_Stmts
     (Node : Bare_Package_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Package_Body_F_Stmts;
   end;
   function Package_Body_F_End_Name
     (Node : Bare_Package_Body) return Bare_End_Name
   is
   begin
         return Node.Package_Body_F_End_Name;
   end;
--# property-start PackageBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8721'
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Names
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_915 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_915 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_915;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5840 '<ArrayLiteral at nodes.lkt:8721:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8721'
--# expr-start 5839 '<FieldAccess for package_name at nodes.lkt:8721:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8721'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_F_Package_Name, Info => Ent.Info);
--# expr-done 5839
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5840
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_915;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_915;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Defining_Names;
--# end
--# property-start PackageBody.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8724'
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Env
  (Node : Bare_Package_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_916 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_916 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_916;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5841 '<FieldAccess for children_env at nodes.lkt:8724:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8724'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5841
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_916;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_916;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Defining_Env;
--# end
--# property-start PackageBody.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8726'
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Declarative_Parts
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_917 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_917 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_917;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5843 '<ArrayLiteral at nodes.lkt:8726:63>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8726'
--# expr-start 5842 '<FieldAccess for decls at nodes.lkt:8726:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8726'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Package_Body_F_Decls, Info => Ent.Info);
--# expr-done 5842
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Fld)); 
--# expr-done 5843
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_917;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_917;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Declarative_Parts;
--# end
--# property-start PackageBody.package_decl_use_clauses_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8730'
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Package_Decl_Use_Clauses_Envs
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_918 with Inline_Always;
            procedure Finalizer_Scope_2772 with Inline_Always;
            procedure Finalizer_Scope_2773 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Package_Decl;
Var_Expr_172 : Internal_Entity_Private_Part;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity_Public_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Private_Part;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_918 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_918;
            procedure Finalizer_Scope_2772 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2772;
            procedure Finalizer_Scope_2773 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2773;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5846 '<bind at nodes.lkt:8731:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8731'
Bound_Imprecise_Fallback := False; 
--# expr-start 5845 '<Cast to BasePackageDecl at nodes.lkt:8733:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8733'
--# expr-start 5844 '<FieldAccess for decl_part at nodes.lkt:8733:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8733'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5844
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5845
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 5846
Var_Expr_0 := Dyn_Var_Bind_Result;
if Var_Expr_0 /= No_Entity_Base_Package_Decl then
   --# scope-start
   --# bind pd Var_Expr_0
--# expr-start 5852 '<env_group at nodes.lkt:8735:17>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
--# expr-start 5851 '<ArrayLiteral at nodes.lkt:8735:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
--# expr-start 5848 '<FieldAccess for use_clauses_envs at nodes.lkt:8735:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
--# expr-start 5847 '<FieldAccess for public_part at nodes.lkt:8735:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Var_Expr_0.Node.Base_Package_Decl_F_Public_Part, Info => Var_Expr_0.Info);
--# expr-done 5847
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 5848
--# expr-start 5849 '<FieldAccess for private_part at nodes.lkt:8735:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Private_Part (Node => Var_Expr_0.Node.Base_Package_Decl_F_Private_Part, Info => Var_Expr_0.Info);
--# expr-done 5849
Var_Expr_172 := Fld_3;
if Var_Expr_172 /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 5850 '<FieldAccess for use_clauses_envs at nodes.lkt:8735:53>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8735'
      if Var_Expr_172.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_4 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Var_Expr_172.Node, E_Info => Var_Expr_172.Info);
--# end
--# expr-done 5850
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2773;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Result_Var)); 
--# expr-done 5851
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 5852
Scope_Result_1 := Group_Env;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2772;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_918;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_918;
                     Finalizer_Scope_2772;
                     Finalizer_Scope_2773;
            raise;
      end;
   return Property_Result;
end Package_Body_P_Package_Decl_Use_Clauses_Envs;
--# end
--# property-start PackageBody._env_do_89 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8739'
pragma Warnings (Off, "is not referenced");
 function Env_Do_89
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5853 '<FieldAccess for env_hook at nodes.lkt:8739:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8739'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5853
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_89;
--# end
--# property-start PackageBody._env_init_90 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8741'
pragma Warnings (Off, "is not referenced");
 function Env_Init_90
  (Node : Bare_Package_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1754 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1754 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1754;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5854 '<FieldAccess for body_initial_env at nodes.lkt:8741:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8741'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5854
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1754;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1754;
            raise;
      end;
   return Property_Result;
end Env_Init_90;
--# end
--# property-start PackageBody._env_mappings_91 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8742'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_91
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
     Ent : Internal_Entity_Package_Body :=
       Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1755 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1755 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1755;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5855 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8742:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8742'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5855
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1755;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1755;
            raise;
      end;
   return Property_Result;
end Env_Mappings_91;
--# end
--# property-start PackageBody._env_trans_parent_92 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8745'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_92
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5856 '<FieldAccess for is_library_item at nodes.lkt:8745:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8745'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 5856
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_92;
--# end
--# property-start PackageBody._env_do_93 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8746'
pragma Warnings (Off, "is not referenced");
 function Env_Do_93
  (Node : Bare_Package_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1757 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1757 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1757;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5857 '<FieldAccess for populate_dependent_units at nodes.lkt:8746:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5857
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1757;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1757;
            raise;
      end;
   return Property_Result;
end Env_Do_93;
--# end
--# property-start PackageBody._ref_env_nodes_94 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8748'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_94
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1758 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1758 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1758;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5858 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:8748:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5858
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1758;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1758;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_94;
--# end
--# property-start PackageBody._ref_cond_95 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8748'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_95
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5860 '<IsA LibraryItem, Subunit at nodes.lkt:8748:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8748'
--# expr-start 5859 '<parent at nodes.lkt:8748:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5859
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5860
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_95;
--# end
--# property-start PackageBody._ref_env_nodes_96 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8751'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_96
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1760 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1760 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1760;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5861 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:8751:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8751'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5861
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1760;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1760;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_96;
--# end
--# property-start PackageBody._ref_cond_97 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8751'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_97
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5863 '<IsA LibraryItem, Subunit at nodes.lkt:8751:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8751'
--# expr-start 5862 '<parent at nodes.lkt:8751:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8751'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5862
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5863
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_97;
--# end
--# property-start PackageBody._ref_env_nodes_98 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8754'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_98
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1762 with Inline_Always;
            procedure Finalizer_Scope_2774 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1762 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1762;
            procedure Finalizer_Scope_2774 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2774;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5864 '<Cast to AdaNode at nodes.lkt:8754:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8754'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5864
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5865 '<ArrayLiteral at nodes.lkt:8754:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8754'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5865
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2774;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1762;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1762;
                     Finalizer_Scope_2774;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_98;
--# end
--# property-start PackageBody._ref_cond_99 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8754'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_99
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5866 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8754:106>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8754'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5866
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_99;
--# end
--# property-start PackageBody._ref_env_nodes_100 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8759'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_100
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1764 with Inline_Always;
            procedure Finalizer_Scope_2775 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1764 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1764;
            procedure Finalizer_Scope_2775 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2775;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5867 '<Cast to AdaNode at nodes.lkt:8759:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8759'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5867
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_0
--# expr-start 5868 '<ArrayLiteral at nodes.lkt:8759:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8759'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5868
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2775;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1764;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1764;
                     Finalizer_Scope_2775;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_100;
--# end
--# property-start PackageBody._ref_cond_101 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8759'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_101
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5869 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8759:97>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8759'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5869
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_101;
--# end
--# property-start PackageBody._ref_env_nodes_102 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8769'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_102
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1766 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1766 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1766;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5871 '<ArrayLiteral at nodes.lkt:8769:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8769'
--# expr-start 5870 '<Cast to AdaNode at nodes.lkt:8769:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8769'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5870
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5871
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1766;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1766;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_102;
--# end
--# property-start PackageBody._ref_cond_103 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8769'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_103
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5872 '<FieldAccess for is_subunit at nodes.lkt:8769:78>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8769'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5872
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_103;
--# end
--# property-start PackageBody._ref_env_nodes_104 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_104
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1768 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1768 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1768;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5874 '<ArrayLiteral at nodes.lkt:8781:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
--# expr-start 5873 '<Cast to AdaNode at nodes.lkt:8781:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5873
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5874
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1768;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1768;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_104;
--# end
--# property-start PackageBody._ref_cond_105 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_105
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5876 '<Not at nodes.lkt:8781:77>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
--# expr-start 5875 '<FieldAccess for is_compilation_unit_root at nodes.lkt:8781:81>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8781'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5875
Not_Val := not (Fld); 
--# expr-done 5876
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_105;
--# end
--# property-start PackageBody._ref_env_nodes_106 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_106
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1770 with Inline_Always;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1770 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1770;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5878 '<ArrayLiteral at nodes.lkt:8786:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
--# expr-start 5877 '<Cast to AdaNode at nodes.lkt:8786:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5877
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5878
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1770;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1770;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_106;
--# end
--# property-start PackageBody._ref_cond_107 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_107
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body  := Bare_Package_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5880 '<Not at nodes.lkt:8786:81>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
--# expr-start 5879 '<FieldAccess for is_compilation_unit_root at nodes.lkt:8786:85>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 5879
Not_Val := not (Fld); 
--# expr-done 5880
if Not_Val then
   If_Result := True;
else
   --# expr-start 5881 '<FieldAccess for is_subunit at nodes.lkt:8786:120>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8786'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld_1 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5881
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_107;
--# end
function Package_Body_P_Package_Decl_Use_Clauses_Envs
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Package_Body_Range
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Package_Body_P_Package_Decl_Use_Clauses_Envs (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Protected_Body
   --
      procedure Protected_Body_Pre_Env_Actions
        (Self            : Bare_Protected_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_108 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_109 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:8810:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_110 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8811:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_111 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_112 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_114 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_113 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_116 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_115 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_118 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_117 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_120 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_119 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_121 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_123 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_122 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Protected_Body
        (Self : Bare_Protected_Body
         ; Protected_Body_F_Name : Bare_Defining_Name
         ; Protected_Body_F_Aspects : Bare_Aspect_Spec
         ; Protected_Body_F_Decls : Bare_Declarative_Part
         ; Protected_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Body_F_Name := Protected_Body_F_Name;
            Self.Protected_Body_F_Aspects := Protected_Body_F_Aspects;
            Self.Protected_Body_F_Decls := Protected_Body_F_Decls;
            Self.Protected_Body_F_End_Name := Protected_Body_F_End_Name;
      end Initialize_Fields_For_Protected_Body;
   function Protected_Body_F_Name
     (Node : Bare_Protected_Body) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_F_Name;
   end;
   function Protected_Body_F_Decls
     (Node : Bare_Protected_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Protected_Body_F_Decls;
   end;
   function Protected_Body_F_End_Name
     (Node : Bare_Protected_Body) return Bare_End_Name
   is
   begin
         return Node.Protected_Body_F_End_Name;
   end;
--# property-start ProtectedBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8798'
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Defining_Names
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_919 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_919 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_919;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5883 '<ArrayLiteral at nodes.lkt:8798:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8798'
--# expr-start 5882 '<FieldAccess for name at nodes.lkt:8798:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_F_Name, Info => Ent.Info);
--# expr-done 5882
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5883
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_919;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_919;
            raise;
      end;
   return Property_Result;
end Protected_Body_P_Defining_Names;
--# end
--# property-start ProtectedBody.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8800'
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Declarative_Parts
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_920 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_920 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_920;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5885 '<ArrayLiteral at nodes.lkt:8800:63>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8800'
--# expr-start 5884 '<FieldAccess for decls at nodes.lkt:8800:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8800'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Protected_Body_F_Decls, Info => Ent.Info);
--# expr-done 5884
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Fld)); 
--# expr-done 5885
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_920;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_920;
            raise;
      end;
   return Property_Result;
end Protected_Body_P_Declarative_Parts;
--# end
--# property-start ProtectedBody.protected_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8802'
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Protected_Type
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Protected_Type_Decl
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Protected_Type_Decl;
      Match_0 : Internal_Entity_Protected_Type_Decl;
Match_1 : Internal_Entity_Single_Protected_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Protected_Type_Decl;
Let_Result : Internal_Entity_Protected_Type_Decl;
Scope_Result : Internal_Entity_Protected_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Protected_Decl;
Let_Result_1 : Internal_Entity_Protected_Type_Decl;
Scope_Result_1 : Internal_Entity_Protected_Type_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Protected_Type_Decl;
Scope_Result_2 : Internal_Entity_Protected_Type_Decl;
Match_Result : Internal_Entity_Protected_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5887 '<Match at nodes.lkt:8802:55>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8802'
--# expr-start 5886 '<FieldAccess for decl_part at nodes.lkt:8802:61>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8802'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 5886
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := No_Entity_Protected_Type_Decl; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5887
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Body_P_Protected_Type;
--# end
--# property-start ProtectedBody._env_do_108 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8809'
pragma Warnings (Off, "is not referenced");
 function Env_Do_108
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5888 '<FieldAccess for env_hook at nodes.lkt:8809:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8809'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5888
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_108;
--# end
--# property-start ProtectedBody._env_init_109 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8810'
pragma Warnings (Off, "is not referenced");
 function Env_Init_109
  (Node : Bare_Protected_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1773 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1773 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1773;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5889 '<FieldAccess for body_initial_env at nodes.lkt:8810:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8810'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5889
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1773;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1773;
            raise;
      end;
   return Property_Result;
end Env_Init_109;
--# end
--# property-start ProtectedBody._env_mappings_110 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8811'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_110
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
     Ent : Internal_Entity_Protected_Body :=
       Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1774 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1774 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1774;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5890 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8811:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8811'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5890
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1774;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1774;
            raise;
      end;
   return Property_Result;
end Env_Mappings_110;
--# end
--# property-start ProtectedBody._env_trans_parent_111 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_111
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_111;
--# end
--# property-start ProtectedBody._env_do_112 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8813'
pragma Warnings (Off, "is not referenced");
 function Env_Do_112
  (Node : Bare_Protected_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1776 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1776 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1776;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5891 '<FieldAccess for populate_dependent_units at nodes.lkt:8813:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8813'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5891
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1776;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1776;
            raise;
      end;
   return Property_Result;
end Env_Do_112;
--# end
--# property-start ProtectedBody._ref_env_nodes_113 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8815'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_113
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1777 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1777 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1777;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5892 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:8815:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8815'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5892
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1777;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1777;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_113;
--# end
--# property-start ProtectedBody._ref_cond_114 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8815'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_114
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5894 '<IsA LibraryItem, Subunit at nodes.lkt:8815:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8815'
--# expr-start 5893 '<parent at nodes.lkt:8815:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8815'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5893
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5894
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_114;
--# end
--# property-start ProtectedBody._ref_env_nodes_115 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8818'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_115
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1779 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1779 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1779;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5895 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:8818:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8818'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5895
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1779;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1779;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_115;
--# end
--# property-start ProtectedBody._ref_cond_116 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8818'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_116
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5897 '<IsA LibraryItem, Subunit at nodes.lkt:8818:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8818'
--# expr-start 5896 '<parent at nodes.lkt:8818:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8818'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5896
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5897
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_116;
--# end
--# property-start ProtectedBody._ref_env_nodes_117 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8821'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_117
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1781 with Inline_Always;
            procedure Finalizer_Scope_2779 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1781 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1781;
            procedure Finalizer_Scope_2779 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2779;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5898 '<Cast to AdaNode at nodes.lkt:8821:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8821'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5898
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5899 '<ArrayLiteral at nodes.lkt:8821:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8821'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5899
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2779;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1781;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1781;
                     Finalizer_Scope_2779;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_117;
--# end
--# property-start ProtectedBody._ref_cond_118 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8821'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_118
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5900 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8821:106>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8821'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5900
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_118;
--# end
--# property-start ProtectedBody._ref_env_nodes_119 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8826'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_119
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1783 with Inline_Always;
            procedure Finalizer_Scope_2780 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1783 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1783;
            procedure Finalizer_Scope_2780 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2780;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5901 '<Cast to AdaNode at nodes.lkt:8826:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8826'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5901
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_0
--# expr-start 5902 '<ArrayLiteral at nodes.lkt:8826:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8826'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5902
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2780;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1783;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1783;
                     Finalizer_Scope_2780;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_119;
--# end
--# property-start ProtectedBody._ref_cond_120 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8826'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_120
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5903 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8826:97>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8826'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5903
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_120;
--# end
--# property-start ProtectedBody._ref_env_nodes_121 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8829'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_121
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1785 with Inline_Always;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1785 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1785;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5905 '<ArrayLiteral at nodes.lkt:8829:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8829'
--# expr-start 5904 '<Cast to AdaNode at nodes.lkt:8829:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8829'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5904
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5905
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1785;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1785;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_121;
--# end
--# property-start ProtectedBody._ref_env_nodes_122 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8833'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_122
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1786 with Inline_Always;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1786 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1786;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5907 '<ArrayLiteral at nodes.lkt:8833:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8833'
--# expr-start 5906 '<Cast to AdaNode at nodes.lkt:8833:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8833'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5906
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5907
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1786;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1786;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_122;
--# end
--# property-start ProtectedBody._ref_cond_123 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8833'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_123
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body  := Bare_Protected_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5908 '<FieldAccess for is_subunit at nodes.lkt:8833:78>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8833'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5908
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_123;
--# end
   --
   --  Primitives for Bare_Task_Body
   --
      procedure Task_Body_Pre_Env_Actions
        (Self            : Bare_Task_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_124 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_125 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:8871:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_126 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8872:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_127 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_128 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_130 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_129 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_132 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_131 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_134 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_133 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_136 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_135 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_137 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Task_Body_P_Task_Type_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_139 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_138 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Task_Body
        (Self : Bare_Task_Body
         ; Task_Body_F_Name : Bare_Defining_Name
         ; Task_Body_F_Aspects : Bare_Aspect_Spec
         ; Task_Body_F_Decls : Bare_Declarative_Part
         ; Task_Body_F_Stmts : Bare_Handled_Stmts
         ; Task_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Body_F_Name := Task_Body_F_Name;
            Self.Task_Body_F_Aspects := Task_Body_F_Aspects;
            Self.Task_Body_F_Decls := Task_Body_F_Decls;
            Self.Task_Body_F_Stmts := Task_Body_F_Stmts;
            Self.Task_Body_F_End_Name := Task_Body_F_End_Name;
      end Initialize_Fields_For_Task_Body;
   function Task_Body_F_Name
     (Node : Bare_Task_Body) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_F_Name;
   end;
   function Task_Body_F_Decls
     (Node : Bare_Task_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Task_Body_F_Decls;
   end;
   function Task_Body_F_Stmts
     (Node : Bare_Task_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Task_Body_F_Stmts;
   end;
   function Task_Body_F_End_Name
     (Node : Bare_Task_Body) return Bare_End_Name
   is
   begin
         return Node.Task_Body_F_End_Name;
   end;
--# property-start TaskBody.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8846'
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Names
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_922 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_922 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_922;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5910 '<ArrayLiteral at nodes.lkt:8846:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8846'
--# expr-start 5909 '<FieldAccess for name at nodes.lkt:8846:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8846'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_F_Name, Info => Ent.Info);
--# expr-done 5909
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5910
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_922;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_922;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Defining_Names;
--# end
--# property-start TaskBody.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8849'
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Env
  (Node : Bare_Task_Body
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_923 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_923 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_923;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5911 '<FieldAccess for children_env at nodes.lkt:8849:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8849'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5911
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_923;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_923;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Defining_Env;
--# end
--# property-start TaskBody.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8851'
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Declarative_Parts
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_924 with Inline_Always;
      Fld : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_924 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_924;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5913 '<ArrayLiteral at nodes.lkt:8851:63>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8851'
--# expr-start 5912 '<FieldAccess for decls at nodes.lkt:8851:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8851'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Task_Body_F_Decls, Info => Ent.Info);
--# expr-done 5912
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Fld)); 
--# expr-done 5913
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_924;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_924;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Declarative_Parts;
--# end
--# property-start TaskBody.task_type_decl_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8853'
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type_Decl_Scope
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_925 with Inline_Always;
      Local_Pp : Internal_Entity_Private_Part;
Var_Expr_173 : Internal_Entity_Task_Type_Decl;
Var_Expr_174 : Internal_Entity_Task_Def;
Fld : Internal_Entity_Task_Type_Decl;
Fld_1 : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Scope_Result : Internal_Entity_Private_Part;
Result_Var : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Result_Var_1 : Internal_Entity_Private_Part;
Fld_3 : Internal_Entity_Task_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_925 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_925;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5922 '<Let at nodes.lkt:8854:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8854'
--# scope-start
--# expr-start 5914 '<FieldAccess for task_type at nodes.lkt:8854:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8854'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5914
Var_Expr_173 := Fld;
if Var_Expr_173 /= No_Entity_Task_Type_Decl then
   --# scope-start
--# expr-start 5915 '<FieldAccess for definition at nodes.lkt:8854:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8854'
      if Var_Expr_173.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Var_Expr_173.Node.Task_Type_Decl_F_Definition, Info => Var_Expr_173.Info);
--# expr-done 5915
Var_Expr_174 := Fld_1;
if Var_Expr_174 /= No_Entity_Task_Def then
   --# scope-start
--# expr-start 5916 '<FieldAccess for private_part at nodes.lkt:8854:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8854'
      if Var_Expr_174.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Var_Expr_174.Node.Task_Def_F_Private_Part, Info => Var_Expr_174.Info);
--# expr-done 5916
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Private_Part;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Private_Part;
end if;
Local_Pp := Result_Var_1; 
--# bind pp Local_Pp
--# expr-start 5921 '<If at nodes.lkt:8856:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8856'
--# expr-start 5917 '<IsNull at nodes.lkt:8856:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8856'
Is_Null := Local_Pp.Node = null; 
--# expr-done 5917
if Is_Null then
   --# expr-start 5919 '<FieldAccess for children_env at nodes.lkt:8856:28>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8856'
--# expr-start 5918 '<FieldAccess for task_type at nodes.lkt:8856:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8856'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld_3 := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5918
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 5919
   If_Result := Fld_4;
else
   --# expr-start 5920 '<FieldAccess for children_env at nodes.lkt:8856:65>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8856'
      if Local_Pp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Local_Pp.Node, E_Info => Local_Pp.Info);
--# expr-done 5920
   If_Result := Fld_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 5921
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5922
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_925;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_925;
            raise;
      end;
   return Property_Result;
end Task_Body_P_Task_Type_Decl_Scope;
--# end
--# property-start TaskBody.task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8859'
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Task_Type_Decl
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Task_Type_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Single_Task_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Task_Type_Decl;
Scope_Result : Internal_Entity_Task_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Task_Type_Decl;
Scope_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Task_Type_Decl;
Scope_Result_2 : Internal_Entity_Task_Type_Decl;
Match_Result : Internal_Entity_Task_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Task_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5926 '<bind at nodes.lkt:8860:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8860'
Bound_Imprecise_Fallback := False; 
--# expr-start 5925 '<Match at nodes.lkt:8862:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8862'
--# expr-start 5923 '<FieldAccess for decl_part at nodes.lkt:8862:15>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8862'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5923
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 5924 '<FieldAccess for task_type at nodes.lkt:8864:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8864'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_1.Node.Single_Task_Decl_F_Task_Type, Info => Match_1.Info);
--# expr-done 5924
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Protected_Type_Decl | Ada_Anonymous_Type_Decl .. Ada_Single_Protected_Decl | Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5925
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 5926
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Task_Body_P_Task_Type;
--# end
--# property-start TaskBody._env_do_124 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8870'
pragma Warnings (Off, "is not referenced");
 function Env_Do_124
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5927 '<FieldAccess for env_hook at nodes.lkt:8870:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8870'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 5927
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_124;
--# end
--# property-start TaskBody._env_init_125 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8871'
pragma Warnings (Off, "is not referenced");
 function Env_Init_125
  (Node : Bare_Task_Body
  )
   return Internal_Designated_Env
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1789 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1789 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1789;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5928 '<FieldAccess for body_initial_env at nodes.lkt:8871:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8871'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.body_initial_env
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env (Node => Self);
--# end
--# expr-done 5928
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1789;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1789;
            raise;
      end;
   return Property_Result;
end Env_Init_125;
--# end
--# property-start TaskBody._env_mappings_126 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8872'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_126
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
     Ent : Internal_Entity_Task_Body :=
       Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1790 with Inline_Always;
      Fld : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1790 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1790;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5929 '<FieldAccess for previous_part_link_env_assoc at nodes.lkt:8872:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8872'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Body.previous_part_link_env_assoc'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Link_Env_Assoc (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5929
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1790;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1790;
            raise;
      end;
   return Property_Result;
end Env_Mappings_126;
--# end
--# property-start TaskBody._env_trans_parent_127 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_127
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_127;
--# end
--# property-start TaskBody._env_do_128 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8874'
pragma Warnings (Off, "is not referenced");
 function Env_Do_128
  (Node : Bare_Task_Body
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1792 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1792 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1792;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5930 '<FieldAccess for populate_dependent_units at nodes.lkt:8874:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8874'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 5930
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1792;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1792;
            raise;
      end;
   return Property_Result;
end Env_Do_128;
--# end
--# property-start TaskBody._ref_env_nodes_129 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8876'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_129
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1793 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1793 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1793;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5931 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:8876:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8876'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 5931
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1793;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1793;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_129;
--# end
--# property-start TaskBody._ref_cond_130 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8876'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_130
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5933 '<IsA LibraryItem, Subunit at nodes.lkt:8876:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8876'
--# expr-start 5932 '<parent at nodes.lkt:8876:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8876'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5932
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5933
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_130;
--# end
--# property-start TaskBody._ref_env_nodes_131 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8879'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_131
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1795 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1795 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1795;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5934 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:8879:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8879'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 5934
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1795;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1795;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_131;
--# end
--# property-start TaskBody._ref_cond_132 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8879'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_132
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5936 '<IsA LibraryItem, Subunit at nodes.lkt:8879:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8879'
--# expr-start 5935 '<parent at nodes.lkt:8879:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8879'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 5935
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 5936
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_132;
--# end
--# property-start TaskBody._ref_env_nodes_133 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8882'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_133
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1797 with Inline_Always;
            procedure Finalizer_Scope_2786 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1797 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1797;
            procedure Finalizer_Scope_2786 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2786;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5937 '<Cast to AdaNode at nodes.lkt:8882:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8882'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5937
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 5938 '<ArrayLiteral at nodes.lkt:8882:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8882'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5938
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2786;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1797;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1797;
                     Finalizer_Scope_2786;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_133;
--# end
--# property-start TaskBody._ref_cond_134 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8882'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_134
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5939 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8882:106>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8882'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5939
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_134;
--# end
--# property-start TaskBody._ref_env_nodes_135 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8887'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_135
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1799 with Inline_Always;
            procedure Finalizer_Scope_2787 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1799 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1799;
            procedure Finalizer_Scope_2787 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2787;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5940 '<Cast to AdaNode at nodes.lkt:8887:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8887'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5940
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v2 Var_Expr_0
--# expr-start 5941 '<ArrayLiteral at nodes.lkt:8887:41>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8887'
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Var_Expr_0)); 
--# expr-done 5941
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2787;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1799;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1799;
                     Finalizer_Scope_2787;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_135;
--# end
--# property-start TaskBody._ref_cond_136 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8887'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_136
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5942 '<FieldAccess for should_ref_generic_formals at nodes.lkt:8887:97>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8887'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 5942
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_136;
--# end
--# property-start TaskBody._ref_env_nodes_137 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8890'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_137
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1801 with Inline_Always;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1801 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1801;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5944 '<ArrayLiteral at nodes.lkt:8890:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8890'
--# expr-start 5943 '<Cast to AdaNode at nodes.lkt:8890:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8890'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5943
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5944
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1801;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1801;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_137;
--# end
--# property-start TaskBody._ref_env_nodes_138 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8894'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_138
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1802 with Inline_Always;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1802 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1802;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5946 '<ArrayLiteral at nodes.lkt:8894:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8894'
--# expr-start 5945 '<Cast to AdaNode at nodes.lkt:8894:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8894'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 5945
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 5946
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1802;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1802;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_138;
--# end
--# property-start TaskBody._ref_cond_139 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8894'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_139
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body  := Bare_Task_Body (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5947 '<FieldAccess for is_subunit at nodes.lkt:8894:78>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8894'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 5947
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_139;
--# end
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Task_Body_Range
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Task_Body_P_Task_Type_Decl_Scope (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Entry_Index_Spec
   --
      procedure Entry_Index_Spec_Pre_Env_Actions
        (Self            : Bare_Entry_Index_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_140 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8922:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Index_Spec
        (Self : Bare_Entry_Index_Spec
         ; Entry_Index_Spec_F_Id : Bare_Defining_Name
         ; Entry_Index_Spec_F_Subtype : Bare_Ada_Node
         ; Entry_Index_Spec_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Index_Spec_F_Id := Entry_Index_Spec_F_Id;
            Self.Entry_Index_Spec_F_Subtype := Entry_Index_Spec_F_Subtype;
            Self.Entry_Index_Spec_F_Aspects := Entry_Index_Spec_F_Aspects;
      end Initialize_Fields_For_Entry_Index_Spec;
   function Entry_Index_Spec_F_Id
     (Node : Bare_Entry_Index_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Index_Spec_F_Id;
   end;
   function Entry_Index_Spec_F_Subtype
     (Node : Bare_Entry_Index_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Index_Spec_F_Subtype;
   end;
--# property-start EntryIndexSpec.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8905'
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Names
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_927 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_927 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_927;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5949 '<ArrayLiteral at nodes.lkt:8905:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8905'
--# expr-start 5948 '<FieldAccess for id at nodes.lkt:8905:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8905'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Index_Spec_F_Id, Info => Ent.Info);
--# expr-done 5948
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5949
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_927;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_927;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Defining_Names;
--# end
--# property-start EntryIndexSpec.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8908'
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Env
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_928 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_928 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_928;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5951 '<FieldAccess for defining_env at nodes.lkt:8908:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8908'
--# expr-start 5950 '<FieldAccess for expr_type at nodes.lkt:8908:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8908'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5950
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5951
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_928;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_928;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Defining_Env;
--# end
--# property-start EntryIndexSpec.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8911'
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Expr_Type
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5956 '<Match at nodes.lkt:8911:45>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8911'
--# expr-start 5952 '<FieldAccess for subtype at nodes.lkt:8911:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8911'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 5952
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind subt Match_0
--# expr-start 5953 '<FieldAccess for designated_type at nodes.lkt:8912:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8912'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5953
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 5955 '<FieldAccess for expression_type at nodes.lkt:8913:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8913'
--# expr-start 5954 '<Cast to Expr at nodes.lkt:8913:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8913'
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 5954
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5955
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 5956
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Expr_Type;
--# end
--# property-start EntryIndexSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8917'
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Equation
  (Node : Bare_Entry_Index_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_930 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_930 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_930;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5958 '<FieldAccess for sub_equation at nodes.lkt:8917:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8917'
--# expr-start 5957 '<FieldAccess for subtype at nodes.lkt:8917:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8917'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 5957
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 5958
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_930;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_930;
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Xref_Equation;
--# end
--# property-start EntryIndexSpec.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8919'
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Entry_Point
  (Node : Bare_Entry_Index_Spec
  )
   return Boolean
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Entry_Index_Spec_P_Xref_Entry_Point;
--# end
--# property-start EntryIndexSpec._env_mappings_140 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8922'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_140
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Index_Spec  := Bare_Entry_Index_Spec (Node);
     Ent : Internal_Entity_Entry_Index_Spec :=
       Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1804 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Entry_Index_Spec;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1804 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1804;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5961 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 5959 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5959
--# expr-start 5960 '<FieldAccess for name_symbol at nodes.lkt:8922:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8922'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5960
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5961
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1804;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1804;
            raise;
      end;
   return Property_Result;
end Env_Mappings_140;
--# end
   --
   --  Primitives for Bare_Error_Decl
   --
--# property-start ErrorDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8930'
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Defining_Names
  (Node : Bare_Error_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Error_Decl  := Bare_Error_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Error_Decl_P_Defining_Names;
--# end
--# property-start ErrorDecl.child_decl_initial_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8934'
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Error_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Error_Decl  := Bare_Error_Decl (Node);
      --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Symbol;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Error_Decl_P_Child_Decl_Initial_Env_Name;
--# end
   --
   --  Primitives for Bare_Exception_Decl
   --
      procedure Exception_Decl_Pre_Env_Actions
        (Self            : Bare_Exception_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_141 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8960:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Decl
        (Self : Bare_Exception_Decl
         ; Exception_Decl_F_Ids : Bare_Defining_Name_List
         ; Exception_Decl_F_Renames : Bare_Renaming_Clause
         ; Exception_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Exception_Decl_F_Ids := Exception_Decl_F_Ids;
            Self.Exception_Decl_F_Renames := Exception_Decl_F_Renames;
            Self.Exception_Decl_F_Aspects := Exception_Decl_F_Aspects;
      end Initialize_Fields_For_Exception_Decl;
   function Exception_Decl_F_Ids
     (Node : Bare_Exception_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Exception_Decl_F_Ids;
   end;
   function Exception_Decl_F_Renames
     (Node : Bare_Exception_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Exception_Decl_F_Renames;
   end;
--# property-start ExceptionDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8945'
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Defining_Names
  (Node : Bare_Exception_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
     Ent : Internal_Entity_Exception_Decl :=
       Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_934 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Defining_Name;
Bare_Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_934 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_934;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5963 '<Map at nodes.lkt:8946:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8946'
   --# expr-start 5962 '<FieldAccess for ids at nodes.lkt:8946:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8946'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Exception_Decl_F_Ids, Info => Ent.Info);
--# expr-done 5962
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind id Item_0
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_0;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5963
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_934;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_934;
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Defining_Names;
--# end
--# property-start ExceptionDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8950'
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Next_Part_For_Decl
  (Node : Bare_Exception_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Basic_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ExceptionDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8952'
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Entry_Point
  (Node : Bare_Exception_Decl
  )
   return Boolean
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Xref_Entry_Point;
--# end
--# property-start ExceptionDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8955'
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Equation
  (Node : Bare_Exception_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
     Ent : Internal_Entity_Exception_Decl :=
       Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_937 with Inline_Always;
            procedure Finalizer_Scope_2791 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_937 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_937;
            procedure Finalizer_Scope_2791 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2791;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 5964 '<FieldAccess for renames at nodes.lkt:8955:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8955'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Exception_Decl_F_Renames, Info => Ent.Info);
--# expr-done 5964
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 5966 '<FieldAccess for xref_no_overloading at nodes.lkt:8956:16>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8956'
--# expr-start 5965 '<FieldAccess for renamed_object at nodes.lkt:8956:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8956'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_0.Info);
--# expr-done 5965
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 5966
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2791;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8956:68") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_937;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_937;
                     Finalizer_Scope_2791;
            raise;
      end;
   return Property_Result;
end Exception_Decl_P_Xref_Equation;
--# end
--# property-start ExceptionDecl._env_mappings_141 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8960'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_141
  (Node : Bare_Exception_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Decl  := Bare_Exception_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1805 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Exception_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1805 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1805;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5968 '<FieldAccess for env_mappings at nodes.lkt:8960:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5967 '<FieldAccess for ids at nodes.lkt:8960:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Exception_Decl_F_Ids;
--# expr-done 5967
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 5968
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1805;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1805;
            raise;
      end;
   return Property_Result;
end Env_Mappings_141;
--# end
   --
   --  Primitives for Bare_Exception_Handler
   --
      procedure Exception_Handler_Pre_Env_Actions
        (Self            : Bare_Exception_Handler;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_142 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_143 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:8991:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Handler
        (Self : Bare_Exception_Handler
         ; Exception_Handler_F_Exception_Name : Bare_Defining_Name
         ; Exception_Handler_F_Handled_Exceptions : Bare_Alternatives_List
         ; Exception_Handler_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Exception_Handler_F_Exception_Name := Exception_Handler_F_Exception_Name;
            Self.Exception_Handler_F_Handled_Exceptions := Exception_Handler_F_Handled_Exceptions;
            Self.Exception_Handler_F_Stmts := Exception_Handler_F_Stmts;
      end Initialize_Fields_For_Exception_Handler;
   function Exception_Handler_F_Exception_Name
     (Node : Bare_Exception_Handler) return Bare_Defining_Name
   is
   begin
         return Node.Exception_Handler_F_Exception_Name;
   end;
   function Exception_Handler_F_Handled_Exceptions
     (Node : Bare_Exception_Handler) return Bare_Alternatives_List
   is
   begin
         return Node.Exception_Handler_F_Handled_Exceptions;
   end;
   function Exception_Handler_F_Stmts
     (Node : Bare_Exception_Handler) return Bare_Stmt_List
   is
   begin
         return Node.Exception_Handler_F_Stmts;
   end;
--# property-start ExceptionHandler.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8971'
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Defining_Names
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_938 with Inline_Always;
            procedure Finalizer_Scope_2792 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_938 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_938;
            procedure Finalizer_Scope_2792 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2792;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5969 '<FieldAccess for exception_name at nodes.lkt:8972:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8972'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 5969
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 5970 '<ArrayLiteral at nodes.lkt:8972:39>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8972'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Var_Expr_0)); 
--# expr-done 5970
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2792;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_938;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_938;
                     Finalizer_Scope_2792;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Defining_Names;
--# end
--# property-start ExceptionHandler.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8975'
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Expr_Type
  (Node : Bare_Exception_Handler
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_939 with Inline_Always;
            procedure Finalizer_Scope_2793 with Inline_Always;
      Var_Expr_175 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_939 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_939;
            procedure Finalizer_Scope_2793 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2793;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5978 '<Cast to BaseTypeDecl at nodes.lkt:8975:45>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8975'
--# expr-start 5974 '<FieldAccess for get_unit_root_decl at nodes.lkt:8975:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8975'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 5973 '<ArrayLiteral at nodes.lkt:8976:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8976'
--# expr-start 5971 '<Symbol Ada at nodes.lkt:8976:10>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8976'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 5971
--# expr-start 5972 '<Symbol Exceptions at nodes.lkt:8976:18>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8976'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exceptions); 
--# expr-done 5972
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 5973
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 5974
Var_Expr_175 := Fld;
if Var_Expr_175 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5977 '<EnvGet at nodes.lkt:8975:45>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8975'
--# expr-start 5975 '<FieldAccess for children_env at nodes.lkt:8975:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8975'
      if Var_Expr_175 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_175);
--# expr-done 5975
--# expr-start 5976 '<Symbol Exception_Occurrence at nodes.lkt:8978:9>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8978'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exception_Occurrence); 
--# expr-done 5976
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 5977
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_2793;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5978
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_939;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_939;
                     Finalizer_Scope_2793;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Expr_Type;
--# end
--# property-start ExceptionHandler.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8982'
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Equation
  (Node : Bare_Exception_Handler
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_940 with Inline_Always;
            procedure Finalizer_Scope_2794 with Inline_Always;
      Fld : Bare_Alternatives_List;
Item_0 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_940 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_940;
            procedure Finalizer_Scope_2794 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2794;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5983 '<LogicAll at nodes.lkt:8983:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8983'
--# expr-start 5982 '<Map at nodes.lkt:8983:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8983'
   --# expr-start 5979 '<FieldAccess for handled_exceptions at nodes.lkt:8983:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8983'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Exception_Handler_F_Handled_Exceptions;
--# expr-done 5979
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind he Item_0
      --# expr-start 5981 '<FieldAccess for sub_equation at nodes.lkt:8983:51>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8983'
--# expr-start 5980 '<as_entity at nodes.lkt:8983:51>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8983'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 5980
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result.Info);
--# end
--# expr-done 5981
         declare
            Item_To_Append : constant Logic_Equation := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2794;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5982
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:8983:9") else null)); 
--# expr-done 5983
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_940;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_940;
                     Finalizer_Scope_2794;
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Xref_Equation;
--# end
--# property-start ExceptionHandler.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8985'
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Entry_Point
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Xref_Entry_Point;
--# end
--# property-start ExceptionHandler.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8987'
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Is_Constant_Object
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Exception_Handler_P_Is_Constant_Object;
--# end
--# property-start ExceptionHandler._env_trans_parent_142 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_142
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_142;
--# end
--# property-start ExceptionHandler._env_mappings_143 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8992'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_143
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Handler  := Bare_Exception_Handler (Node);
     Ent : Internal_Entity_Exception_Handler :=
       Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1807 with Inline_Always;
            procedure Finalizer_Scope_2795 with Inline_Always;
            procedure Finalizer_Scope_2796 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Exception_Handler;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1807 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1807;
            procedure Finalizer_Scope_2795 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2795;
            procedure Finalizer_Scope_2796 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_2796;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 5984 '<FieldAccess for exception_name at nodes.lkt:8992:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8992'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 5984
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 5989 '<Map at nodes.lkt:8993:24>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8993'
   --# expr-start 5985 '<ArrayLiteral at nodes.lkt:8993:24>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8993'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Var_Expr_0)); 
--# expr-done 5985
      declare
         Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Array_Lit;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
      --# expr-start 5988 '<New EnvAssoc at nodes.lkt:8994:28>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8994'
--# expr-start 5986 '<New DesignatedEnv at nodes.lkt:8995:67>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8995'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 5986
--# expr-start 5987 '<FieldAccess for name_symbol at nodes.lkt:8995:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:8995'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_1.Node);
--# end
--# expr-done 5987
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 5988
         declare
            Item_To_Append : constant Internal_Env_Assoc := New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2796;
         end loop;
      end;
         Map_Result := Create_Internal_Env_Assoc_Array
           (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 5989
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2795;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1807;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1807;
                     Finalizer_Scope_2795;
                     Finalizer_Scope_2796;
            raise;
      end;
   return Property_Result;
end Env_Mappings_143;
--# end
   --
   --  Primitives for Bare_For_Loop_Var_Decl
   --
      procedure For_Loop_Var_Decl_Pre_Env_Actions
        (Self            : Bare_For_Loop_Var_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_144 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9054:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Var_Decl
        (Self : Bare_For_Loop_Var_Decl
         ; For_Loop_Var_Decl_F_Id : Bare_Defining_Name
         ; For_Loop_Var_Decl_F_Id_Type : Bare_Type_Expr
        ) is
      begin
            Self.For_Loop_Var_Decl_F_Id := For_Loop_Var_Decl_F_Id;
            Self.For_Loop_Var_Decl_F_Id_Type := For_Loop_Var_Decl_F_Id_Type;
      end Initialize_Fields_For_For_Loop_Var_Decl;
   function For_Loop_Var_Decl_F_Id
     (Node : Bare_For_Loop_Var_Decl) return Bare_Defining_Name
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id;
   end;
   function For_Loop_Var_Decl_F_Id_Type
     (Node : Bare_For_Loop_Var_Decl) return Bare_Type_Expr
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id_Type;
   end;
--# property-start ForLoopVarDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9012'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Names
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_943 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_943 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_943;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 5991 '<ArrayLiteral at nodes.lkt:9012:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9012'
--# expr-start 5990 '<FieldAccess for id at nodes.lkt:9012:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9012'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 5990
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 5991
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_943;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_943;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Defining_Names;
--# end
--# property-start ForLoopVarDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9015'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Env
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_944 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_944 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_944;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 5993 '<FieldAccess for defining_env at nodes.lkt:9015:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9015'
--# expr-start 5992 '<FieldAccess for expr_type at nodes.lkt:9015:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9015'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5992
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 5993
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_944;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_944;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Defining_Env;
--# end
--# property-start ForLoopVarDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9020'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Expr_Type
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Ada_Node;
Cast_Result_1 : Bare_For_Loop_Spec;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Type_Expr;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Bare_Type_Expr;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Var_Decl_For_Loop_Var_Decl_P_Expr_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Var_Decl,
                                As_Bare_For_Loop_Var_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6007 '<If at nodes.lkt:9020:45>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9020'
--# expr-start 5995 '<IsNull at nodes.lkt:9020:48>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9020'
--# expr-start 5994 '<FieldAccess for id_type at nodes.lkt:9020:48>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 5994
Is_Null := Fld_8 = null; 
--# expr-done 5995
if Is_Null then
   --# expr-start 6004 '<If at nodes.lkt:9030:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 5998 '<FieldAccess for is_iterated_assoc_spec at nodes.lkt:9030:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 5997 '<Cast to ForLoopSpec at nodes.lkt:9030:12>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 5996 '<parent at nodes.lkt:9030:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 5996
   if Fld_4 = null
      or else Fld_4.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result_1 := Fld_4;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5997
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.is_iterated_assoc_spec
Fld_5 := Libadalang.Implementation.For_Loop_Spec_P_Is_Iterated_Assoc_Spec (Node => Cast_Result_1);
--# end
--# expr-done 5998
if Fld_5 then
   --# expr-start 6001 '<Cast to BaseTypeDecl at nodes.lkt:9030:70>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 6000 '<FieldAccess for type_val at nodes.lkt:9030:70>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 5999 '<FieldAccess for id at nodes.lkt:9030:70>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.For_Loop_Var_Decl_F_Id;
--# expr-done 5999
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_1 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld);
--# end
--# expr-done 6000
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6001
   If_Result := Cast_Result;
else
   --# expr-start 6003 '<FieldAccess for expression_type at nodes.lkt:9030:112>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
--# expr-start 6002 '<FieldAccess for id at nodes.lkt:9030:112>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9030'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 6002
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6003
   If_Result := Fld_3;
end if;
--# expr-done 6004
   If_Result_1 := If_Result;
else
   --# expr-start 6006 '<FieldAccess for designated_type at nodes.lkt:9033:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9033'
--# expr-start 6005 '<FieldAccess for id_type at nodes.lkt:9033:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9033'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 6005
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6006
   If_Result_1 := Fld_7;
end if;
--# expr-done 6007
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end For_Loop_Var_Decl_P_Expr_Type;
--# end
--# property-start ForLoopVarDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9036'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Is_Constant_Object
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Local_Loop_Spec : Internal_Entity_For_Loop_Spec;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_For_Loop_Spec;
Fld_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Name;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Iter_Type;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6016 '<Let at nodes.lkt:9038:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9038'
--# scope-start
--# expr-start 6009 '<Cast to ForLoopSpec at nodes.lkt:9038:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9038'
--# expr-start 6008 '<parent at nodes.lkt:9038:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9038'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6008
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_For_Loop_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_For_Loop_Spec;
   end if;
--# expr-done 6009
Local_Loop_Spec := Cast_Result; 
--# bind loop_spec Local_Loop_Spec
--# expr-start 6015 '<If at nodes.lkt:9042:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
--# expr-start 6011 '<IsA IterType.Of at nodes.lkt:9042:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
--# expr-start 6010 '<FieldAccess for loop_type at nodes.lkt:9042:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
      if Local_Loop_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Iter_Type (Node => Local_Loop_Spec.Node.For_Loop_Spec_F_Loop_Type, Info => Local_Loop_Spec.Info);
--# expr-done 6010
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Iter_Type_Of_Range; 
--# expr-done 6011
if Is_A then
   --# expr-start 6014 '<FieldAccess for is_constant at nodes.lkt:9042:52>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
--# expr-start 6013 '<Cast to Name at nodes.lkt:9042:52>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
--# expr-start 6012 '<FieldAccess for iter_expr at nodes.lkt:9042:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9042'
      if Local_Loop_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Local_Loop_Spec.Node.For_Loop_Spec_F_Iter_Expr, Info => Local_Loop_Spec.Info);
--# expr-done 6012
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6013
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6014
   If_Result := Fld_2;
else
   If_Result := True;
end if;
--# expr-done 6015
Let_Result := If_Result; 
--# end
--# expr-done 6016
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Is_Constant_Object;
--# end
--# property-start ForLoopVarDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9049'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Xref_Equation
  (Node : Bare_For_Loop_Var_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
     Ent : Internal_Entity_For_Loop_Var_Decl :=
       Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_947 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_947 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_947;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6024 '<And at nodes.lkt:9049:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9049'
--# expr-start 6018 '<FieldAccess for sub_equation at nodes.lkt:9049:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9049'
--# expr-start 6017 '<FieldAccess for id at nodes.lkt:9049:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9049'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 6017
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 6018
--# expr-start 6023 '<If at nodes.lkt:9050:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9050'
--# expr-start 6020 '<IsNull at nodes.lkt:9050:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9050'
--# expr-start 6019 '<FieldAccess for id_type at nodes.lkt:9050:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9050'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 6019
Is_Null := Fld_4.Node = null; 
--# expr-done 6020
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9050:38") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 6022 '<FieldAccess for sub_equation at nodes.lkt:9050:49>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9050'
--# expr-start 6021 '<FieldAccess for id_type at nodes.lkt:9050:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9050'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 6021
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 6022
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 6023
And_Pred := Create_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9049:37") else null)); 
--# expr-done 6024
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_947;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_947;
            raise;
      end;
   return Property_Result;
end For_Loop_Var_Decl_P_Xref_Equation;
--# end
--# property-start ForLoopVarDecl._env_mappings_144 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9054'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_144
  (Node : Bare_For_Loop_Var_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_For_Loop_Var_Decl  := Bare_For_Loop_Var_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1808 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_For_Loop_Var_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1808 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1808;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6027 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6025 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6025
--# expr-start 6026 '<FieldAccess for name_symbol at nodes.lkt:9054:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9054'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 6026
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6027
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1808;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1808;
            raise;
      end;
   return Property_Result;
end Env_Mappings_144;
--# end
   --
   --  Primitives for Bare_Generic_Decl
   --
      procedure Initialize_Fields_For_Generic_Decl
        (Self : Bare_Generic_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
        ) is
      begin
            Self.Generic_Decl_F_Formal_Part := Generic_Decl_F_Formal_Part;
      end Initialize_Fields_For_Generic_Decl;
   function Generic_Decl_F_Formal_Part
     (Node : Bare_Generic_Decl) return Bare_Generic_Formal_Part
   is
   begin
         return Node.Generic_Decl_F_Formal_Part;
   end;
--# property-start '[dispatcher]GenericDecl.decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Decl_P_Decl
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Decl (Self.Kind) is
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.decl
                  Property_Result := Generic_Package_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.decl
                  Property_Result := Generic_Subp_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Decl_P_Decl;
--# end
--# property-start GenericDecl.get_aspect_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9067'
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Aspect_Spec
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Spec
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Aspect_Spec;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Aspect_Spec;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6029 '<FieldAccess for get_aspect_spec at nodes.lkt:9070:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9070'
--# expr-start 6028 '<FieldAccess for decl at nodes.lkt:9070:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9070'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6028
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Spec (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6029
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Decl_P_Get_Aspect_Spec;
--# end
--# property-start GenericDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9073'
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl  := Bare_Generic_Decl (Node);
     Ent : Internal_Entity_Generic_Decl :=
       Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6031 '<FieldAccess for next_part_for_decl at nodes.lkt:9074:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9074'
--# expr-start 6030 '<FieldAccess for decl at nodes.lkt:9074:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9074'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6030
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6031
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Decl_P_Next_Part_For_Decl;
--# end
   --
   --  Primitives for Bare_Generic_Package_Decl
   --
      procedure Generic_Package_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_145 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_146 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9100:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_147 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9106:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_148 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_149 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_151 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_150 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_153 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_152 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Decl
        (Self : Bare_Generic_Package_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Package_Decl_F_Package_Decl : Bare_Generic_Package_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Package_Decl_F_Package_Decl := Generic_Package_Decl_F_Package_Decl;
      end Initialize_Fields_For_Generic_Package_Decl;
   function Generic_Package_Decl_F_Package_Decl
     (Node : Bare_Generic_Package_Decl) return Bare_Generic_Package_Internal
   is
   begin
         return Node.Generic_Package_Decl_F_Package_Decl;
   end;
--# property-start GenericPackageDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9083'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_951 with Inline_Always;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_951 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_951;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6033 '<FieldAccess for defining_env at nodes.lkt:9083:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9083'
--# expr-start 6032 '<FieldAccess for package_decl at nodes.lkt:9083:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9083'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 6032
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6033
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_951;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_951;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9085'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_952 with Inline_Always;
      Fld : Bare_Generic_Package_Internal;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_952 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_952;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6037 '<ArrayLiteral at nodes.lkt:9086:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9086'
--# expr-start 6036 '<as_entity at nodes.lkt:9086:10>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9086'
--# expr-start 6035 '<FieldAccess for package_name at nodes.lkt:9086:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9086'
--# expr-start 6034 '<FieldAccess for package_decl at nodes.lkt:9086:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9086'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Generic_Package_Decl_F_Package_Decl;
--# expr-done 6034
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Base_Package_Decl_F_Package_Name;
--# expr-done 6035
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 6036
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 6037
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_952;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_952;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9088'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Declarative_Parts
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_953 with Inline_Always;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_953 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_953;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6039 '<FieldAccess for declarative_parts at nodes.lkt:9089:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9089'
--# expr-start 6038 '<FieldAccess for package_decl at nodes.lkt:9089:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9089'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 6038
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_parts'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Parts (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6039
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_953;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_953;
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Declarative_Parts;
--# end
--# property-start GenericPackageDecl.body_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9094'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Body_Part
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Package_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6041 '<FieldAccess for body_part at nodes.lkt:9094:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9094'
--# expr-start 6040 '<FieldAccess for package_decl at nodes.lkt:9094:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9094'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 6040
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_1 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6041
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Body_Part;
--# end
--# property-start GenericPackageDecl.decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9096'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Decl
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Package_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 6042 '<FieldAccess for package_decl at nodes.lkt:9096:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9096'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 6042
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Decl_P_Decl;
--# end
--# property-start GenericPackageDecl._env_do_145 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9099'
pragma Warnings (Off, "is not referenced");
 function Env_Do_145
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6043 '<FieldAccess for env_hook at nodes.lkt:9099:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9099'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6043
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_145;
--# end
--# property-start GenericPackageDecl._env_init_146 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9104'
pragma Warnings (Off, "is not referenced");
 function Env_Init_146
  (Node : Bare_Generic_Package_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1810 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1810 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1810;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6044 '<FieldAccess for child_decl_initial_env at nodes.lkt:9104:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9104'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 6044
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1810;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1810;
            raise;
      end;
   return Property_Result;
end Env_Init_146;
--# end
--# property-start GenericPackageDecl._env_mappings_147 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9106'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_147
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Decl :=
       Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1811 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1811 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1811;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6045 '<FieldAccess for child_decl_env_assocs at nodes.lkt:9106:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9106'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6045
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1811;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1811;
            raise;
      end;
   return Property_Result;
end Env_Mappings_147;
--# end
--# property-start GenericPackageDecl._env_trans_parent_148 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_148
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_148;
--# end
--# property-start GenericPackageDecl._env_do_149 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9108'
pragma Warnings (Off, "is not referenced");
 function Env_Do_149
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1813 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1813 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1813;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6046 '<FieldAccess for populate_dependent_units at nodes.lkt:9108:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9108'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6046
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1813;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1813;
            raise;
      end;
   return Property_Result;
end Env_Do_149;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_150 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9110'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_150
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1814 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1814 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1814;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6047 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9110:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9110'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6047
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1814;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1814;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_150;
--# end
--# property-start GenericPackageDecl._ref_cond_151 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9110'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_151
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6049 '<IsA LibraryItem, Subunit at nodes.lkt:9110:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9110'
--# expr-start 6048 '<parent at nodes.lkt:9110:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9110'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6048
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6049
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_151;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_152 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9113'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_152
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1816 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1816 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1816;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6050 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9113:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9113'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6050
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1816;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1816;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_152;
--# end
--# property-start GenericPackageDecl._ref_cond_153 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9113'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_153
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl  := Bare_Generic_Package_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6052 '<IsA LibraryItem, Subunit at nodes.lkt:9113:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9113'
--# expr-start 6051 '<parent at nodes.lkt:9113:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9113'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6051
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6052
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_153;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Decl
   --
      procedure Generic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_154 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_155 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9144:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_156 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9150:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_157 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_158 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_160 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_159 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_162 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_161 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Generic_Subp_Decl_Post_Env_Actions
        (Self : Bare_Generic_Subp_Decl; State : in out PLE_Node_State) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_163 (Node => 
Self);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Decl
        (Self : Bare_Generic_Subp_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Subp_Decl_F_Subp_Decl : Bare_Generic_Subp_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Subp_Decl_F_Subp_Decl := Generic_Subp_Decl_F_Subp_Decl;
      end Initialize_Fields_For_Generic_Subp_Decl;
   function Generic_Subp_Decl_F_Subp_Decl
     (Node : Bare_Generic_Subp_Decl) return Bare_Generic_Subp_Internal
   is
   begin
         return Node.Generic_Subp_Decl_F_Subp_Decl;
   end;
--# property-start GenericSubpDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9123'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_956 with Inline_Always;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_956 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_956;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6056 '<ArrayLiteral at nodes.lkt:9124:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9124'
--# expr-start 6055 '<FieldAccess for name at nodes.lkt:9124:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9124'
--# expr-start 6054 '<FieldAccess for subp_spec at nodes.lkt:9124:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9124'
--# expr-start 6053 '<FieldAccess for subp_decl at nodes.lkt:9124:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9124'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 6053
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Fld.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld.Info);
--# expr-done 6054
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6055
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_2)); 
--# expr-done 6056
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_956;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_956;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpDecl.body_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9129'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Body_Part
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6058 '<Cast to BaseSubpBody at nodes.lkt:9130:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9130'
--# expr-start 6057 '<FieldAccess for body_part_for_decl at nodes.lkt:9130:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9130'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6057
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 6058
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Body_Part;
--# end
--# property-start GenericSubpDecl.decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9132'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Decl
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 6059 '<FieldAccess for subp_decl at nodes.lkt:9132:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9132'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 6059
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Decl;
--# end
--# property-start GenericSubpDecl.is_imported 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9135'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Is_Imported
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6061 '<FieldAccess for is_imported at nodes.lkt:9135:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9135'
--# expr-start 6060 '<FieldAccess for subp_decl at nodes.lkt:9135:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9135'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 6060
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_imported'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Imported (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6061
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Is_Imported;
--# end
--# property-start GenericSubpDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9138'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6063 '<FieldAccess for next_part_for_decl at nodes.lkt:9139:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9139'
--# expr-start 6062 '<FieldAccess for subp_decl at nodes.lkt:9139:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9139'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 6062
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 6063
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start GenericSubpDecl._env_do_154 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9143'
pragma Warnings (Off, "is not referenced");
 function Env_Do_154
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6064 '<FieldAccess for env_hook at nodes.lkt:9143:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9143'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6064
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_154;
--# end
--# property-start GenericSubpDecl._env_init_155 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9148'
pragma Warnings (Off, "is not referenced");
 function Env_Init_155
  (Node : Bare_Generic_Subp_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1819 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1819 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1819;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6065 '<FieldAccess for child_decl_initial_env at nodes.lkt:9148:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9148'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self, Private_Part => True);
--# end
--# expr-done 6065
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1819;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1819;
            raise;
      end;
   return Property_Result;
end Env_Init_155;
--# end
--# property-start GenericSubpDecl._env_mappings_156 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9150'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_156
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Decl :=
       Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1820 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1820 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1820;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6066 '<FieldAccess for child_decl_env_assocs at nodes.lkt:9150:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9150'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_env_assocs
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Env_Assocs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6066
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1820;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1820;
            raise;
      end;
   return Property_Result;
end Env_Mappings_156;
--# end
--# property-start GenericSubpDecl._env_trans_parent_157 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_157
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_157;
--# end
--# property-start GenericSubpDecl._env_do_158 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9152'
pragma Warnings (Off, "is not referenced");
 function Env_Do_158
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1822 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1822 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1822;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6067 '<FieldAccess for populate_dependent_units at nodes.lkt:9152:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9152'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6067
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1822;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1822;
            raise;
      end;
   return Property_Result;
end Env_Do_158;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_159 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9154'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_159
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1823 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1823 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1823;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6068 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9154:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9154'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6068
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1823;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1823;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_159;
--# end
--# property-start GenericSubpDecl._ref_cond_160 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9154'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_160
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6070 '<IsA LibraryItem, Subunit at nodes.lkt:9154:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9154'
--# expr-start 6069 '<parent at nodes.lkt:9154:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9154'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6069
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6070
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_160;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_161 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9157'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_161
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1825 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1825 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1825;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6071 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9157:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9157'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6071
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1825;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1825;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_161;
--# end
--# property-start GenericSubpDecl._ref_cond_162 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9157'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_162
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6073 '<IsA LibraryItem, Subunit at nodes.lkt:9157:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9157'
--# expr-start 6072 '<parent at nodes.lkt:9157:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9157'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6072
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6073
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_162;
--# end
--# property-start GenericSubpDecl._env_do_163 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9160'
pragma Warnings (Off, "is not referenced");
 function Env_Do_163
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl  := Bare_Generic_Subp_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6074 '<FieldAccess for populate_body_unit at nodes.lkt:9160:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9160'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_body_unit
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Body_Unit (Node => Self);
--# end
--# expr-done 6074
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_163;
--# end
   --
   --  Primitives for Bare_Generic_Instantiation
   --
      procedure Generic_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_164 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_165 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9394:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_166 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9403:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_167 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_168 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_169 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_171 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_170 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_173 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_172 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_175 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_174 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Generic_Instantiation_P_Parent_Instantiation_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Instantiation
        (Self : Bare_Generic_Instantiation
        ) is
      begin
      Self.Bare_Generic_Instantiation_Lf_State_Instantiation_Env := Uninitialized;
      Self.Bare_Generic_Instantiation_Lf_Stg_Instantiation_Env := Empty_Env;
      Self.Bare_Generic_Instantiation_Lf_State_Actual_Expr_Decls := Uninitialized;
      Self.Bare_Generic_Instantiation_Lf_Stg_Actual_Expr_Decls := No_Bare_Anonymous_Expr_Decl_Array_Type;
      end Initialize_Fields_For_Generic_Instantiation;
--# property-start GenericInstantiation.instantiation_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9168'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Instantiation_Env
  (Node : Bare_Generic_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_961 with Inline_Always;
      Dyn_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_961 is
            begin
                     Dec_Ref (Dyn_Env);
            end Finalizer_Scope_961;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Generic_Instantiation_Lf_State_Instantiation_Env is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Generic_Instantiation_Lf_Stg_Instantiation_Env;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Generic_Instantiation_Lf_State_Instantiation_Env,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6075 '<DynamicLexicalEnv at nodes.lkt:9169:9>' Dyn_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9169'
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Generic_Instantiation_P_Instantiation_Bindings'Access, Ada_Node_P_Resolve_Generic_Actual'Access, False, Self.Unit.Context.Symbols); 
--# expr-done 6075
         Property_Result := Dyn_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_961;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_961;
               Self.Bare_Generic_Instantiation_Lf_State_Instantiation_Env :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Generic_Instantiation_Lf_State_Instantiation_Env := Initialized;
      Self.Bare_Generic_Instantiation_Lf_Stg_Instantiation_Env := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Generic_Instantiation_F_Instantiation_Env;
--# end
--# property-start GenericInstantiation.instantiation_bindings 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9171'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiation_Bindings
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_962 with Inline_Always;
      Ignored : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_962 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_962;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6078 '<Let at nodes.lkt:9176:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9176'
--# scope-start
--# expr-start 6076 '<FieldAccess for nonbound_generic_decl_from_self at nodes.lkt:9176:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9176'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_self
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self (Node => Self);
--# end
--# expr-done 6076
Ignored := Fld; 
--# bind _ Ignored
--# expr-start 6077 '<FieldAccess for instantiation_bindings_internal at nodes.lkt:9178:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9178'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_bindings_internal
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Instantiation_Bindings_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6077
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6078
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_962;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_962;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Instantiation_Bindings;
--# end
--# property-start GenericInstantiation.instantiation_bindings_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9182'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiation_Bindings_Internal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_963 with Inline_Always;
            procedure Finalizer_Scope_2797 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Generic_Formal_Part;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Param_Match_Array_Access;
Index_1 : Integer;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Symbol_Type;
Fld_7 : Bare_Anonymous_Expr_Decl_Array_Access;
Get_Result : Bare_Anonymous_Expr_Decl;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Bare_Expr;
Cast_Result_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Ada_Node;
Fld_12 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Is_A : Boolean;
If_Result : Bare_Ada_Node;
Cast_Expr : Bare_Expr;
Cast_Result_3 : Bare_Ada_Node;
Fld_14 : Internal_Single_Actual;
Fld_15 : Internal_Entity_Basic_Assoc;
Fld_16 : Internal_Entity_Expr;
Is_A_1 : Boolean;
Fld_17 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Base_Formal_Param_Decl;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
New_Struct_1 : Internal_Inner_Env_Assoc;
Map_Result : Internal_Inner_Env_Assoc_Array_Access;
Fld_19 : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Fld_20 : Boolean;
If_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_963 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_963;
            procedure Finalizer_Scope_2797 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2797;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Instantiation_Generic_Instantiation_P_Instantiation_Bindings_Internal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Instantiation,
                                As_Bare_Generic_Instantiation => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6113 '<If at nodes.lkt:9183:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
--# expr-start 6079 '<FieldAccess for is_any_formal at nodes.lkt:9183:12>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_20 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6079
if Fld_20 then
   If_Result_3 := No_Internal_Inner_Env_Assoc_Array_Type;
else
   --# expr-start 6112 '<Map at nodes.lkt:9183:70>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
   --# expr-start 6108 '<Map at nodes.lkt:9183:70>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
   --# expr-start 6084 '<FieldAccess for match_param_list at nodes.lkt:9183:70>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
--# expr-start 6082 '<FieldAccess for formal_part at nodes.lkt:9183:70>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
--# expr-start 6081 '<Cast to GenericDecl at nodes.lkt:9183:70>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
--# expr-start 6080 '<FieldAccess for nonbound_generic_decl_from_self at nodes.lkt:9183:70>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_self
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self (Node => Self);
--# end
--# expr-done 6080
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6081
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Generic_Formal_Part (Node => Cast_Result.Node.Generic_Decl_F_Formal_Part, Info => Cast_Result.Info);
--# expr-done 6082
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6083 '<FieldAccess for generic_inst_params at nodes.lkt:9183:155>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6083
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_1.Node, Params => Fld_2, Is_Dottable_Subp => False, E_Info => Fld_1.Info);
--# end
--# expr-done 6084
      declare
         Map_Result_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
         Index_1 := 0;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
               --# bind i Index_1
      --# expr-start 6107 '<New InnerEnvAssoc at nodes.lkt:9184:24>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9184'
--# expr-start 6087 '<FieldAccess for name_symbol at nodes.lkt:9185:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9185'
--# expr-start 6086 '<FieldAccess for name at nodes.lkt:9185:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9185'
--# expr-start 6085 '<FieldAccess for formal at nodes.lkt:9185:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9185'
Fld_4 := Item_0.Formal;
--# expr-done 6085
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Fld_4.Node.Defining_Name_F_Name, Info => Fld_4.Info);
--# expr-done 6086
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_5.Node);
--# end
--# expr-done 6087
--# expr-start 6088 '<New Metadata at nodes.lkt:9194:60>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9194'
New_Struct := (Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 6088
--# expr-start 6106 '<If at nodes.lkt:9192:23>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6092 '<IsA BoxExpr at nodes.lkt:9192:26>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6091 '<FieldAccess for expr at nodes.lkt:9192:26>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6090 '<FieldAccess for assoc at nodes.lkt:9192:26>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6089 '<FieldAccess for actual at nodes.lkt:9192:26>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
Fld_14 := Item_0.Actual;
--# expr-done 6089
Fld_15 := Fld_14.Assoc;
--# expr-done 6090
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# end
--# expr-done 6091
Is_A_1 := Fld_16.Node /= null 
and then Fld_16.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 6092
if Is_A_1 then
   If_Result_1 := True;
else
   --# expr-start 6095 '<IsA GenericFormalSubpDecl at nodes.lkt:9192:63>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6094 '<FieldAccess for formal_decl at nodes.lkt:9192:63>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6093 '<FieldAccess for formal at nodes.lkt:9192:63>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
Fld_17 := Item_0.Formal;
--# expr-done 6093
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_18 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_17.Node, E_Info => Fld_17.Info);
--# end
--# expr-done 6094
Is_A_2 := Fld_18.Node /= null 
and then Fld_18.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range; 
--# expr-done 6095
   If_Result_1 := Is_A_2;
end if;
if If_Result_1 then
Cast_Expr := No_Bare_Ada_Node; 
      Cast_Result_3 := Cast_Expr;
   If_Result_2 := Cast_Result_3;
else
   --# expr-start 6105 '<If at nodes.lkt:9192:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9192'
--# expr-start 6098 '<IsA GenericFormalObjDecl at nodes.lkt:9193:22>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9193'
--# expr-start 6097 '<FieldAccess for formal_decl at nodes.lkt:9193:22>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9193'
--# expr-start 6096 '<FieldAccess for formal at nodes.lkt:9193:22>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9193'
Fld_12 := Item_0.Formal;
--# expr-done 6096
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_13 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# end
--# expr-done 6097
Is_A := Fld_13.Node /= null 
and then Fld_13.Node.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 6098
if Is_A then
--# expr-start 6100 '<at at nodes.lkt:9193:75>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9193'
--# expr-start 6099 '<FieldAccess for actual_expr_decls at nodes.lkt:9193:75>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9193'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.actual_expr_decls
Fld_7 := Libadalang.Implementation.Generic_Instantiation_F_Actual_Expr_Decls (Node => Ent.Node);
--# end
--# expr-done 6099
Get_Result := Get (Self, Fld_7, Index_1, True); 
--# expr-done 6100
      Cast_Result_1 := Get_Result;
   If_Result := Cast_Result_1;
else
--# expr-start 6104 '<FieldAccess for node at nodes.lkt:9194:22>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9194'
--# expr-start 6103 '<FieldAccess for expr at nodes.lkt:9194:22>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9194'
--# expr-start 6102 '<FieldAccess for assoc at nodes.lkt:9194:22>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9194'
--# expr-start 6101 '<FieldAccess for actual at nodes.lkt:9194:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9194'
Fld_8 := Item_0.Actual;
--# expr-done 6101
Fld_9 := Fld_8.Assoc;
--# expr-done 6102
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 6103
Fld_11 := Fld_10.Node;
--# expr-done 6104
      Cast_Result_2 := Fld_11;
   If_Result := Cast_Result_2;
end if;
--# expr-done 6105
   If_Result_2 := If_Result;
end if;
--# expr-done 6106
New_Struct_1 := (Key => Fld_6, Metadata => New_Struct, Rebindings => null, Value => If_Result_2); 
--# expr-done 6107
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2797;
               Index_1 := Index_1 + 1;
         end loop;
      end;
         Map_Result := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6108
      declare
         Map_Result_1_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Inner_Env_Assoc_Array_Access := Map_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind env_assoc Item_2
         --# expr-start 6111 '<Not at nodes.lkt:9197:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9197'
--# expr-start 6110 '<IsNull at nodes.lkt:9197:32>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9197'
--# expr-start 6109 '<FieldAccess for value at nodes.lkt:9197:32>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9197'
Fld_19 := Item_2.Value;
--# expr-done 6109
Is_Null := Fld_19 = null; 
--# expr-done 6110
Not_Val := not (Is_Null); 
--# expr-done 6111
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc := Item_2;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Inner_Env_Assoc_Array
           (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6112
   If_Result_3 := Map_Result_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 6113
         Property_Result := If_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_963;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_963;
                     Finalizer_Scope_2797;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                        As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Generic_Instantiation_P_Instantiation_Bindings_Internal;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_entity_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_entity_name
                  Property_Result := Generic_Package_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_entity_name
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_inst_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_inst_params
                  Property_Result := Generic_Package_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_inst_params
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericInstantiation.is_any_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9209'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Is_Any_Formal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 6114 '<FieldAccess for generic_inst_params at nodes.lkt:9210:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9210'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6114
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 6116 '<IsA BoxExpr at nodes.lkt:9210:51>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9210'
--# expr-start 6115 '<FieldAccess for expr at nodes.lkt:9210:51>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9210'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 6115
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 6116
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Is_Any_Formal;
--# end
--# property-start GenericInstantiation.nonbound_generic_decl_from_entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9217'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Nonbound_Generic_Decl_From_Entity
  (Node : Bare_Generic_Instantiation
      ; Follow_Renaming : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind follow_renaming Follow_Renaming
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_967 with Inline_Always;
      Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Body_Node;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Boolean;
Find_Result : Internal_Entity;
Bound_Imprecise_Fallback : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Generic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity_Generic_Renaming_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
If_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_967 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_967;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6120 '<Find at nodes.lkt:9218:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9218'
   --# expr-start 6118 '<FieldAccess for all_env_elements_internal at nodes.lkt:9218:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9218'
--# expr-start 6117 '<FieldAccess for generic_entity_name at nodes.lkt:9218:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9218'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6117
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True), E_Info => Fld.Info);
--# end
--# expr-done 6118
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
      --# expr-start 6119 '<FieldAccess for has_visibility at nodes.lkt:9220:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9220'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_2 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 6119
      if Fld_2 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
--# expr-done 6120
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 6125 '<Match at nodes.lkt:9221:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9221'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind b Match_0
--# expr-start 6122 '<bind at nodes.lkt:9223:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9223'
Bound_Imprecise_Fallback := False; 
--# expr-start 6121 '<FieldAccess for decl_part at nodes.lkt:9225:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9225'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 6121
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 6122
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind rd Match_1
--# expr-start 6124 '<If at nodes.lkt:9227:49>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9227'
if Follow_Renaming then
--# expr-start 6123 '<FieldAccess for resolve at nodes.lkt:9227:73>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9227'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_4 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6123
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   If_Result := Cast_Result_1;
else
Cast_Expr_1 := Match_1; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   If_Result := Cast_Result_2;
end if;
--# expr-done 6124
Let_Result_1 := If_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind d Match_2
Let_Result_2 := Match_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Basic_Decl; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 6125
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_967;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_967;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Nonbound_Generic_Decl_From_Entity;
--# end
--# property-start GenericInstantiation.nonbound_generic_decl_from_self 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9239'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self
  (Node : Bare_Generic_Instantiation
      ; Follow_Renaming : Boolean
         := True
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   --# bind follow_renaming Follow_Renaming
   Property_Result : Internal_Entity_Basic_Decl;
      As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6127 '<FieldAccess for nonbound_generic_decl_from_entity at nodes.lkt:9240:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9240'
--# expr-start 6126 '<as_bare_entity at nodes.lkt:9240:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9240'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 6126
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_entity
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Entity (Node => If_Result.Node, Follow_Renaming => Follow_Renaming, E_Info => If_Result.Info);
--# end
--# expr-done 6127
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self;
--# end
--# property-start '[dispatcher]GenericInstantiation.designated_generic_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Generic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.designated_generic_decl
                  Property_Result := Generic_Package_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.designated_generic_decl
                  Property_Result := Generic_Subp_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericInstantiation.designated_bare_generic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9251'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Designated_Bare_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Decl : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Internal_Entity_Info;
Fld_5 : Env_Rebindings;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Info;
Fld_8 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6140 '<Let at nodes.lkt:9252:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9252'
--# scope-start
--# expr-start 6128 '<FieldAccess for designated_generic_decl at nodes.lkt:9252:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9252'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6128
Local_Decl := Fld; 
--# bind decl Local_Decl
--# expr-start 6139 '<Cast to BasicDecl at nodes.lkt:9254:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9254'
--# expr-start 6138 '<New AdaNode.entity at nodes.lkt:9254:9>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9254'
--# expr-start 6136 '<New EntityInfo at nodes.lkt:9255:34>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9255'
--# expr-start 6130 '<FieldAccess for from_rebound at nodes.lkt:9256:91>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
--# expr-start 6129 '<FieldAccess for info at nodes.lkt:9256:91>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
Fld_7 := Local_Decl.Info;
--# expr-done 6129
Fld_8 := Fld_7.From_Rebound;
--# expr-done 6130
--# expr-start 6132 '<FieldAccess for md at nodes.lkt:9256:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
--# expr-start 6131 '<FieldAccess for info at nodes.lkt:9256:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
Fld_2 := Local_Decl.Info;
--# expr-done 6131
Fld_3 := Fld_2.Md;
--# expr-done 6132
--# expr-start 6135 '<FieldAccess for get_parent at nodes.lkt:9256:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
--# expr-start 6134 '<FieldAccess for rebindings at nodes.lkt:9256:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
--# expr-start 6133 '<FieldAccess for info at nodes.lkt:9256:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9256'
Fld_4 := Local_Decl.Info;
--# expr-done 6133
Fld_5 := Fld_4.Rebindings;
--# expr-done 6134
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Parent;
--# expr-done 6135
New_Struct := (From_Rebound => Fld_8, Md => Fld_3, Rebindings => Fld_6); 
--# expr-done 6136
--# expr-start 6137 '<FieldAccess for node at nodes.lkt:9255:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9255'
Fld_1 := Local_Decl.Node;
--# expr-done 6137
      Cast_Result := Fld_1;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result); 
--# expr-done 6138
   if New_Struct_1.Node = null
      or else New_Struct_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => New_Struct_1.Node,
         Info => New_Struct_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 6139
Let_Result := Cast_Result_1; 
--# end
--# expr-done 6140
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Designated_Bare_Generic_Decl;
--# end
--# property-start GenericInstantiation.actual_expr_decls 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9262'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Actual_Expr_Decls
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Anonymous_Expr_Decl_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_971 with Inline_Always;
      Match_0 : Bare_Generic_Subp_Instantiation;
Match_1 : Bare_Generic_Package_Instantiation;
Fld : Bare_Assoc_List;
Fld_1 : Bare_Assoc_List;
Match_Prefix : Bare_Generic_Instantiation;
Cast_Result : Bare_Generic_Subp_Instantiation;
Let_Result : Bare_Assoc_List;
Scope_Result : Bare_Assoc_List;
Cast_Result_1 : Bare_Generic_Package_Instantiation;
Let_Result_1 : Bare_Assoc_List;
Scope_Result_1 : Bare_Assoc_List;
Match_Result : Bare_Assoc_List;
Item_0 : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Bare_Expr;
New_Node : Bare_Anonymous_Expr_Decl;
Map_Result : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_971 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_971;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Generic_Instantiation_Lf_State_Actual_Expr_Decls is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Generic_Instantiation_Lf_Stg_Actual_Expr_Decls;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Generic_Instantiation_Lf_State_Actual_Expr_Decls,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6148 '<Map at nodes.lkt:9262:51>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9262'
   --# expr-start 6143 '<Match at nodes.lkt:9262:51>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9262'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Generic_Instantiation (Match_Prefix.Kind) is
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind subp Match_0
--# expr-start 6141 '<FieldAccess for params at nodes.lkt:9263:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9263'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Generic_Subp_Instantiation_F_Params;
--# expr-done 6141
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind pkg Match_1
--# expr-start 6142 '<FieldAccess for params at nodes.lkt:9264:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9264'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Generic_Package_Instantiation_F_Params;
--# expr-done 6142
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 6143
      declare
         Map_Result_Vec : Bare_Anonymous_Expr_Decl_Vectors.Vector;
      begin
      if Match_Result = null then
      Map_Result := Create_Bare_Anonymous_Expr_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Match_Result;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind assoc Item_0
      --# expr-start 6147 '<New AnonymousExprDecl at nodes.lkt:9266:20>' New_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9266'
--# expr-start 6146 '<FieldAccess for node at nodes.lkt:9266:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9266'
--# expr-start 6145 '<FieldAccess for expr at nodes.lkt:9266:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9266'
--# expr-start 6144 '<as_bare_entity at nodes.lkt:9266:43>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9266'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
As_Entity := (Info => No_Entity_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 6144
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6145
Fld_3 := Fld_2.Node;
--# expr-done 6146
if Is_Foreign_Strict (Self.Self_Env, Self) then
   Raise_Property_Exception
     (Self,
      Property_Error'Identity,
      "synthetic nodes cannot have foreign lexical envs");
end if;
      if Fld_3 = null then
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            "AnonymousExprDecl.expr cannot be null in synthetic nodes; add a"
            & " nullable annotation to this field to allow it");
      end if;
New_Node := new Root_Node_Record
  (Ada_Anonymous_Expr_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Anonymous_Expr_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Anonymous_Expr_Decl
     (Self => New_Node,
      Anonymous_Expr_Decl_F_Expr => Fld_3);
--# expr-done 6147
         declare
            Item_To_Append : constant Bare_Anonymous_Expr_Decl := New_Node;
         begin
            Bare_Anonymous_Expr_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Anonymous_Expr_Decl_Array
           (Items_Count => Natural (Bare_Anonymous_Expr_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Anonymous_Expr_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Anonymous_Expr_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Anonymous_Expr_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6148
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_971;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_971;
               Self.Bare_Generic_Instantiation_Lf_State_Actual_Expr_Decls :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Generic_Instantiation_Lf_State_Actual_Expr_Decls := Initialized;
      Self.Bare_Generic_Instantiation_Lf_Stg_Actual_Expr_Decls := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Generic_Instantiation_F_Actual_Expr_Decls;
--# end
--# property-start GenericInstantiation.parent_instantiation_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9295'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Parent_Instantiation_Env
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_972 with Inline_Always;
            procedure Finalizer_Scope_2809 with Inline_Always;
            procedure Finalizer_Scope_2810 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_176 : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_972 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_972;
            procedure Finalizer_Scope_2809 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2809;
            procedure Finalizer_Scope_2810 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_2810;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6151 '<Cast to DottedName at nodes.lkt:9296:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9296'
--# expr-start 6150 '<FieldAccess for name at nodes.lkt:9296:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9296'
--# expr-start 6149 '<FieldAccess for defining_name at nodes.lkt:9296:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9296'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6149
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 6150
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 6151
Var_Expr_176 := Cast_Result;
if Var_Expr_176 /= No_Entity_Dotted_Name then
   --# scope-start
--# expr-start 6152 '<FieldAccess for prefix at nodes.lkt:9296:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9296'
      if Var_Expr_176.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_176.Node.Dotted_Name_F_Prefix, Info => Var_Expr_176.Info);
--# expr-done 6152
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind parent_name Var_Expr_0
--# expr-start 6156 '<bind at nodes.lkt:9298:17>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9298'
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Bound_Origin := Cast_Result_1; 
--# expr-start 6155 '<bind at nodes.lkt:9299:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9299'
--# expr-start 6153 '<FieldAccess for children_env at nodes.lkt:9299:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9299'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6153
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 6154 '<FieldAccess for designated_env_no_overloading at nodes.lkt:9301:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9301'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Var_Expr_0.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => False, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 6154
Dyn_Var_Bind_Result := Fld_4; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6155
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 6156
Scope_Result := Dyn_Var_Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2810;
   Result_Var := Scope_Result;
else
   --# expr-start 6157 '<FieldAccess for empty_env at nodes.lkt:9302:28>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9302'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 6157
   Result_Var := Fld_5;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2809;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_972;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_972;
                     Finalizer_Scope_2809;
                     Finalizer_Scope_2810;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Parent_Instantiation_Env;
--# end
--# property-start GenericInstantiation.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9305'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Entry_Point
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Xref_Entry_Point;
--# end
--# property-start GenericInstantiation.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9308'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Equation
  (Node : Bare_Generic_Instantiation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_974 with Inline_Always;
            procedure Finalizer_Scope_2811 with Inline_Always;
            procedure Finalizer_Scope_2812 with Inline_Always;
            procedure Finalizer_Scope_2813 with Inline_Always;
            procedure Finalizer_Scope_2815 with Inline_Always;
            procedure Finalizer_Scope_2816 with Inline_Always;
            procedure Finalizer_Scope_2817 with Inline_Always;
            procedure Finalizer_Scope_2818 with Inline_Always;
            procedure Finalizer_Scope_2819 with Inline_Always;
            procedure Finalizer_Scope_2820 with Inline_Always;
            procedure Finalizer_Scope_2821 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Name;
Local_Actual_Name : Internal_Entity_Name;
Match_0_1 : Internal_Entity_Base_Type_Decl;
Match_1_1 : Internal_Entity_Generic_Package_Instantiation;
Match_2 : Internal_Entity_Formal_Subp_Decl;
Match_3 : Internal_Entity_Object_Decl;
Match_4 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Bare_Base_Id;
Var_Expr_177 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Match_Result : Logic_Equation;
And_Pred : Logic_Equation;
True_Rel_1 : Logic_Equation;
Fld_6 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Internal_Param_Match_Array_Access;
Fld_10 : Internal_Single_Actual;
Fld_11 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_13 : Internal_Single_Actual;
Fld_14 : Internal_Entity_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_4 : Internal_Entity_Generic_Formal;
Fld_18 : Internal_Entity_Basic_Decl;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
Fld_22 : Logic_Equation;
Fld_23 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel_2 : Logic_Equation;
Or_Pred : Logic_Equation;
Is_A_1 : Boolean;
If_Result : Logic_Equation;
Fld_24 : Internal_Single_Actual;
Fld_25 : Internal_Entity_Basic_Assoc;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_29 : Internal_Single_Actual;
Fld_30 : Internal_Entity_Basic_Assoc;
Fld_31 : Internal_Entity_Expr;
Fld_32 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_33 : Internal_Single_Actual;
Fld_34 : Internal_Entity_Basic_Assoc;
Fld_35 : Internal_Entity_Expr;
Fld_36 : Logic_Equation;
And_Pred_3 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Match_Prefix_1 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_8 : Internal_Entity_Generic_Package_Instantiation;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_9 : Internal_Entity_Formal_Subp_Decl;
Let_Result_4 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Object_Decl;
Let_Result_5 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Let_Result_6 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Fld_37 : Internal_Single_Actual;
Fld_38 : Bare_Base_Id;
Fld_39 : Logic_Var;
Fld_40 : Internal_Entity_Defining_Name;
Fld_41 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Fld_42 : Internal_Entity_Basic_Decl;
Cast_Result_12 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
True_Rel_4 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result_7 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Scope_Result_8 : Logic_Equation_Array_Access;
Result_Var_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_43 : Boolean;
If_Result_2 : Logic_Equation;
And_Pred_5 : Logic_Equation;
            procedure Finalizer_Scope_974 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_974;
            procedure Finalizer_Scope_2811 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2811;
            procedure Finalizer_Scope_2812 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2812;
            procedure Finalizer_Scope_2813 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2813;
            procedure Finalizer_Scope_2815 is
            begin
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (True_Rel_4);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_2815;
            procedure Finalizer_Scope_2816 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_2816;
            procedure Finalizer_Scope_2817 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2817;
            procedure Finalizer_Scope_2818 is
            begin
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_2818;
            procedure Finalizer_Scope_2819 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2819;
            procedure Finalizer_Scope_2820 is
            begin
                     Dec_Ref (True_Rel_3);
            end Finalizer_Scope_2820;
            procedure Finalizer_Scope_2821 is
            begin
                     Dec_Ref (Bind_Result_2);
            end Finalizer_Scope_2821;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6227 '<And at nodes.lkt:9308:37>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9308'
--# expr-start 6166 '<And at nodes.lkt:9309:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
--# expr-start 6161 '<Bind at nodes.lkt:9309:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
--# expr-start 6159 '<FieldAccess for ref_var at nodes.lkt:9309:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
--# expr-start 6158 '<FieldAccess for generic_entity_name at nodes.lkt:9309:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6158
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld.Node);
--# end
--# expr-done 6159
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 6160 '<FieldAccess for nonbound_generic_decl_from_self at nodes.lkt:9309:51>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_self
Fld_2 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self (Node => Ent.Node, Follow_Renaming => False);
--# end
--# expr-done 6160
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9309:9") else null)); 
--# expr-done 6161
--# expr-start 6165 '<Match at nodes.lkt:9309:100>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
--# expr-start 6162 '<FieldAccess for generic_entity_name at nodes.lkt:9309:106>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9309'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6162
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 6164 '<FieldAccess for xref_no_overloading at nodes.lkt:9310:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9310'
--# expr-start 6163 '<FieldAccess for prefix at nodes.lkt:9310:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9310'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 6163
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 6164
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2811;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9311:23") else null)); 
Let_Result_1 := True_Rel; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2812;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 6165
And_Pred := Create_And (Bind_Result, Match_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9309:9") else null)); 
--# expr-done 6166
--# expr-start 6226 '<If at nodes.lkt:9314:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
--# expr-start 6167 '<FieldAccess for is_any_formal at nodes.lkt:9314:12>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_43 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6167
if Fld_43 then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9314:38") else null)); 
   If_Result_2 := True_Rel_1;
else
   --# expr-start 6225 '<LogicAll at nodes.lkt:9314:49>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
--# expr-start 6168 '<FieldAccess for designated_generic_decl at nodes.lkt:9314:49>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6168
Var_Expr_177 := Fld_6;
if Var_Expr_177 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 6224 '<Map at nodes.lkt:9314:49>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
   --# expr-start 6177 '<Map at nodes.lkt:9314:49>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
   --# expr-start 6171 '<FieldAccess for match_param_list at nodes.lkt:9314:49>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
--# expr-start 6169 '<FieldAccess for formal_part at nodes.lkt:9314:49>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
      if Var_Expr_177.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_177.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_177.Info);
--# expr-done 6169
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6170 '<FieldAccess for generic_inst_params at nodes.lkt:9314:110>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6170
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_7.Node, Params => Fld_8, Is_Dottable_Subp => False, E_Info => Fld_7.Info);
--# end
--# expr-done 6171
      declare
         Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_9;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
         --# expr-start 6176 '<Not at nodes.lkt:9315:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9315'
--# expr-start 6175 '<IsA BoxExpr at nodes.lkt:9315:25>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9315'
--# expr-start 6174 '<FieldAccess for expr at nodes.lkt:9315:25>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9315'
--# expr-start 6173 '<FieldAccess for assoc at nodes.lkt:9315:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9315'
--# expr-start 6172 '<FieldAccess for actual at nodes.lkt:9315:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9315'
Fld_10 := Item_0.Actual;
--# expr-done 6172
Fld_11 := Fld_10.Assoc;
--# expr-done 6173
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 6174
Is_A := Fld_12.Node /= null 
and then Fld_12.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 6175
Not_Val := not (Is_A); 
--# expr-done 6176
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Param_Match := Item_0;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Match_Array
           (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6177
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_1
      --# expr-start 6223 '<Let at nodes.lkt:9318:17>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9318'
--# scope-start
--# expr-start 6181 '<Cast to Name at nodes.lkt:9318:35>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9318'
--# expr-start 6180 '<FieldAccess for expr at nodes.lkt:9318:35>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9318'
--# expr-start 6179 '<FieldAccess for assoc at nodes.lkt:9318:35>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9318'
--# expr-start 6178 '<FieldAccess for actual at nodes.lkt:9318:35>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9318'
Fld_13 := Item_1.Actual;
--# expr-done 6178
Fld_14 := Fld_13.Assoc;
--# expr-done 6179
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 6180
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 6181
Local_Actual_Name := Cast_Result_3; 
--# bind actual_name Local_Actual_Name
--# expr-start 6222 '<And at nodes.lkt:9320:17>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
--# expr-start 6213 '<Match at nodes.lkt:9320:17>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
--# expr-start 6185 '<FieldAccess for decl at nodes.lkt:9320:23>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
--# expr-start 6184 '<Cast to GenericFormal at nodes.lkt:9320:23>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
--# expr-start 6183 '<FieldAccess for formal_decl at nodes.lkt:9320:23>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
--# expr-start 6182 '<FieldAccess for formal at nodes.lkt:9320:23>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9320'
Fld_16 := Item_1.Formal;
--# expr-done 6182
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 6183
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Generic_Formal
   then
      Cast_Result_4 := Create_Internal_Entity_Generic_Formal
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_4 := No_Entity_Generic_Formal;
   end if;
--# expr-done 6184
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_4.Node.Generic_Formal_F_Decl, Info => Cast_Result_4.Info);
--# expr-done 6185
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld_18; 
case Ada_Basic_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0_1 := Cast_Result_7; 
--# bind _ Match_0_1
--# expr-start 6186 '<FieldAccess for xref_type_equation at nodes.lkt:9321:45>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9321'
      if Local_Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_19 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Local_Actual_Name.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Actual_Name.Info);
--# end
--# expr-done 6186
Let_Result_2 := Fld_19; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2816;
            Match_Result_1 := Scope_Result_2; Inc_Ref (Match_Result_1);
         when Ada_Generic_Package_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1_1 := Cast_Result_8; 
--# bind _ Match_1_1
--# expr-start 6187 '<FieldAccess for xref_no_overloading at nodes.lkt:9322:60>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9322'
      if Local_Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_20 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Local_Actual_Name.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Actual_Name.Info);
--# end
--# expr-done 6187
Let_Result_3 := Fld_20; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2817;
            Match_Result_1 := Scope_Result_3; Inc_Ref (Match_Result_1);
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_9; 
--# bind subp_decl Match_2
--# expr-start 6196 '<If at nodes.lkt:9323:63>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9323'
--# expr-start 6188 '<IsA AttributeRef at nodes.lkt:9323:66>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9323'
Is_A_1 := Local_Actual_Name.Node /= null 
and then Local_Actual_Name.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 6188
if Is_A_1 then
   --# expr-start 6189 '<FieldAccess for sub_equation at nodes.lkt:9323:99>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9323'
      if Local_Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Actual_Name.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Actual_Name.Info);
--# end
--# expr-done 6189
   If_Result := Fld_21;
else
   --# expr-start 6195 '<Or at nodes.lkt:9323:131>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9323'
--# expr-start 6194 '<And at nodes.lkt:9324:25>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9324'
--# expr-start 6190 '<FieldAccess for xref_no_overloading at nodes.lkt:9324:25>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9324'
      if Local_Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_22 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Local_Actual_Name.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Actual_Name.Info);
--# end
--# expr-done 6190
--# expr-start 6193 '<Predicate on BasicDecl.subp_decl_match_signature at nodes.lkt:9324:75>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9324'
--# expr-start 6191 '<FieldAccess for ref_var at nodes.lkt:9324:123>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9324'
      if Local_Actual_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Actual_Name.Node);
--# end
--# expr-done 6191
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 6192 '<Cast to BasicDecl at nodes.lkt:9324:146>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9324'
Cast_Expr := Match_2; 
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 6192
Pred := Solver.Create_Predicate (Fld_23, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_5, Origin)); 
--# expr-done 6193
And_Pred_1 := Create_And (Fld_22, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9324:25") else null)); 
--# expr-done 6194
True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9325:26") else null)); 
Or_Pred := Create_Or (And_Pred_1, True_Rel_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9323:131") else null)); 
--# expr-done 6195
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 6196
Let_Result_4 := If_Result; Inc_Ref (Let_Result_4);
--# end
Scope_Result_4 := Let_Result_4;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2818;
            Match_Result_1 := Scope_Result_4; Inc_Ref (Match_Result_1);
         when Ada_Object_Decl .. Ada_No_Type_Object_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_10; 
--# bind obj_decl Match_3
--# expr-start 6212 '<And at nodes.lkt:9326:50>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9326'
--# expr-start 6207 '<And at nodes.lkt:9327:25>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6202 '<Bind at nodes.lkt:9327:25>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6200 '<FieldAccess for expected_type_var at nodes.lkt:9327:29>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6199 '<FieldAccess for expr at nodes.lkt:9327:29>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6198 '<FieldAccess for assoc at nodes.lkt:9327:29>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6197 '<FieldAccess for actual at nodes.lkt:9327:29>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
Fld_24 := Item_1.Actual;
--# expr-done 6197
Fld_25 := Fld_24.Assoc;
--# expr-done 6198
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_26 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_25.Node, E_Info => Fld_25.Info);
--# end
--# expr-done 6199
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_26.Node);
--# expr-done 6200
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 6201 '<FieldAccess for expr_type at nodes.lkt:9327:73>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_3.Node, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 6201
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_27, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9327:25") else null)); 
--# expr-done 6202
--# expr-start 6206 '<FieldAccess for sub_equation at nodes.lkt:9327:99>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6205 '<FieldAccess for expr at nodes.lkt:9327:99>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6204 '<FieldAccess for assoc at nodes.lkt:9327:99>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
--# expr-start 6203 '<FieldAccess for actual at nodes.lkt:9327:99>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9327'
Fld_29 := Item_1.Actual;
--# expr-done 6203
Fld_30 := Fld_29.Assoc;
--# expr-done 6204
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_31 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_30.Node, E_Info => Fld_30.Info);
--# end
--# expr-done 6205
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_32 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_31.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_31.Info);
--# end
--# expr-done 6206
And_Pred_2 := Create_And (Bind_Result_1, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9327:25") else null)); 
--# expr-done 6207
--# expr-start 6211 '<FieldAccess for matches_expected_assign_type at nodes.lkt:9328:27>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9328'
--# expr-start 6210 '<FieldAccess for expr at nodes.lkt:9328:27>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9328'
--# expr-start 6209 '<FieldAccess for assoc at nodes.lkt:9328:27>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9328'
--# expr-start 6208 '<FieldAccess for actual at nodes.lkt:9328:27>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9328'
Fld_33 := Item_1.Actual;
--# expr-done 6208
Fld_34 := Fld_33.Assoc;
--# expr-done 6209
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_34.Node, E_Info => Fld_34.Info);
--# end
--# expr-done 6210
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_36 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_35.Node, Origin => Origin);
--# end
--# expr-done 6211
And_Pred_3 := Create_And (And_Pred_2, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9326:50") else null)); 
--# expr-done 6212
Let_Result_5 := And_Pred_3; Inc_Ref (Let_Result_5);
--# end
Scope_Result_5 := Let_Result_5;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2819;
            Match_Result_1 := Scope_Result_5; Inc_Ref (Match_Result_1);
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Decl | Ada_Generic_Subp_Instantiation .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_11; 
--# bind _ Match_4
True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9329:31") else null)); 
Let_Result_6 := True_Rel_3; Inc_Ref (Let_Result_6);
--# end
Scope_Result_6 := Let_Result_6;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2820;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
end case;
--# expr-done 6213
--# expr-start 6215 '<FieldAccess for name at nodes.lkt:9330:23>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9330'
--# expr-start 6214 '<FieldAccess for actual at nodes.lkt:9330:23>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9330'
Fld_37 := Item_1.Actual;
--# expr-done 6214
Fld_38 := Fld_37.Name;
--# expr-done 6215
Var_Expr_2 := Fld_38;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 6221 '<Bind at nodes.lkt:9331:28>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
--# expr-start 6216 '<FieldAccess for ref_var at nodes.lkt:9331:32>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_39 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_2);
--# end
--# expr-done 6216
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 6220 '<FieldAccess for basic_decl at nodes.lkt:9331:45>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
--# expr-start 6219 '<as_bare_entity at nodes.lkt:9331:45>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
--# expr-start 6218 '<FieldAccess for node at nodes.lkt:9331:45>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
--# expr-start 6217 '<FieldAccess for formal at nodes.lkt:9331:45>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9331'
Fld_40 := Item_1.Formal;
--# expr-done 6217
Fld_41 := Fld_40.Node;
--# expr-done 6218
Is_Null := Fld_41 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Defining_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_41); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6219
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_42 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 6220
      Cast_Result_12 := Create_Internal_Entity
        (Node => Fld_42.Node,
         Info => Fld_42.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_39, Cast_Result_12, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9331:28") else null)); 
--# expr-done 6221
Scope_Result_7 := Bind_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2821;
   Result_Var := Scope_Result_7;
else
   True_Rel_4 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9331:102") else null)); 
   Result_Var := True_Rel_4;
end if;
      Inc_Ref (Result_Var);
And_Pred_4 := Create_And (Match_Result_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9320:17") else null)); 
--# expr-done 6222
Let_Result_7 := And_Pred_4; Inc_Ref (Let_Result_7);
--# end
--# expr-done 6223
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_7;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2815;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6224
Scope_Result_8 := Map_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2813;
   Result_Var_1 := Scope_Result_8;
else
   Result_Var_1 := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Result_Var_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9314:49") else null)); 
--# expr-done 6225
   If_Result_2 := Logic_Boolean_Op;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6226
And_Pred_5 := Create_And (And_Pred, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9308:37") else null)); 
--# expr-done 6227
         Property_Result := And_Pred_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_974;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_974;
                     Finalizer_Scope_2811;
                     Finalizer_Scope_2812;
                     Finalizer_Scope_2813;
                     Finalizer_Scope_2815;
                     Finalizer_Scope_2816;
                     Finalizer_Scope_2817;
                     Finalizer_Scope_2818;
                     Finalizer_Scope_2819;
                     Finalizer_Scope_2820;
                     Finalizer_Scope_2821;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Xref_Equation;
--# end
--# property-start GenericInstantiation.initial_env_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9337'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Initial_Env_Name
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6230 '<If at nodes.lkt:9338:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9338'
--# expr-start 6228 '<FieldAccess for is_library_item at nodes.lkt:9338:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9338'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 6228
if Fld_1 then
   --# expr-start 6229 '<FieldAccess for child_decl_initial_env_name at nodes.lkt:9338:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9338'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 6229
   If_Result := Fld;
else
   If_Result := No_Symbol;
end if;
--# expr-done 6230
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Initial_Env_Name;
--# end
--# property-start GenericInstantiation.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9340'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Env_Names
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_976 with Inline_Always;
            procedure Finalizer_Scope_2822 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_976 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_976;
            procedure Finalizer_Scope_2822 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2822;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6231 '<FieldAccess for top_level_env_name at nodes.lkt:9341:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9341'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 6231
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 6233 '<ArrayLiteral at nodes.lkt:9341:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9341'
--# expr-start 6232 '<to_symbol at nodes.lkt:9341:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9341'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 6232
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 6233
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2822;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_976;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_976;
                     Finalizer_Scope_2822;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Env_Names;
--# end
--# property-start GenericInstantiation.inst_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9346'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_977 with Inline_Always;
            procedure Finalizer_Scope_2823 with Inline_Always;
            procedure Finalizer_Scope_2824 with Inline_Always;
            procedure Finalizer_Scope_2825 with Inline_Always;
            procedure Finalizer_Scope_2826 with Inline_Always;
            procedure Finalizer_Scope_2827 with Inline_Always;
            procedure Finalizer_Scope_2829 with Inline_Always;
      Local_Ap : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Generic_Formal_Package;
Match_1 : Internal_Entity_Generic_Formal_Type_Decl;
Match_2 : Internal_Entity_Generic_Formal_Obj_Decl;
Match_3 : Internal_Entity_Generic_Formal_Subp_Decl;
Local_Subp : Internal_Entity_Formal_Subp_Decl;
Match_4 : Internal_Entity_Pragma_Node;
Match_5 : Internal_Entity_Use_Clause;
Match_6 : Internal_Entity;
Var_Expr_178 : Internal_Entity_Generic_Decl;
Var_Expr_179 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Ada_Node_List;
Fld_4 : Bare_Ada_Node_List;
Fld_5 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Package_Instantiation;
Fld_7 : Internal_Entity_Defining_Name;
New_Struct : Internal_Param_Actual;
Array_Lit : Internal_Param_Actual_Array_Access;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Defining_Name;
New_Struct_1 : Internal_Param_Actual;
Array_Lit_1 : Internal_Param_Actual_Array_Access;
Fld_10 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Object_Decl;
Fld_11 : Internal_Entity_Defining_Name_List;
Fld_12 : Bare_Defining_Name_List;
Fld_13 : Internal_Entity_Info;
Item_1 : Internal_Entity_Defining_Name;
Bare_Item_1 : Bare_Defining_Name;
Fld_14 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Object_Decl;
Fld_15 : Internal_Entity_Expr;
New_Struct_2 : Internal_Param_Actual;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Param_Actual_Array_Access;
Fld_16 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Formal_Subp_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Basic_Decl;
Fld_19 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_20 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Expr;
Fld_21 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_1 : Internal_Entity_Expr;
New_Struct_3 : Internal_Param_Actual;
Let_Result : Internal_Param_Actual;
Array_Lit_2 : Internal_Param_Actual_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_6 : Internal_Entity_Generic_Formal_Package;
Let_Result_1 : Internal_Param_Actual_Array_Access;
Scope_Result_1 : Internal_Param_Actual_Array_Access;
Cast_Result_7 : Internal_Entity_Generic_Formal_Type_Decl;
Let_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_2 : Internal_Param_Actual_Array_Access;
Cast_Result_8 : Internal_Entity_Generic_Formal_Obj_Decl;
Let_Result_3 : Internal_Param_Actual_Array_Access;
Scope_Result_3 : Internal_Param_Actual_Array_Access;
Cast_Result_9 : Internal_Entity_Generic_Formal_Subp_Decl;
Let_Result_4 : Internal_Param_Actual_Array_Access;
Scope_Result_4 : Internal_Param_Actual_Array_Access;
Cast_Result_10 : Internal_Entity_Pragma_Node;
Let_Result_5 : Internal_Param_Actual_Array_Access;
Scope_Result_5 : Internal_Param_Actual_Array_Access;
Cast_Result_11 : Internal_Entity_Use_Clause;
Let_Result_6 : Internal_Param_Actual_Array_Access;
Scope_Result_6 : Internal_Param_Actual_Array_Access;
Cast_Result_12 : Internal_Entity;
Let_Result_7 : Internal_Param_Actual_Array_Access;
Scope_Result_7 : Internal_Param_Actual_Array_Access;
Match_Result : Internal_Param_Actual_Array_Access;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Index_3 : Integer;
Fld_22 : Internal_Entity_Defining_Name;
Fld_23 : Internal_Entity_Defining_Name;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Internal_Entity_Expr;
New_Struct_4 : Internal_Param_Actual;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_8 : Internal_Param_Actual_Array_Access;
Result_Var_1 : Internal_Param_Actual_Array_Access;
Let_Result_8 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_977 is
            begin
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_977;
            procedure Finalizer_Scope_2823 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2823;
            procedure Finalizer_Scope_2824 is
            begin
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2824;
            procedure Finalizer_Scope_2825 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2825;
            procedure Finalizer_Scope_2826 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2826;
            procedure Finalizer_Scope_2827 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2827;
            procedure Finalizer_Scope_2829 is
            begin
                     Dec_Ref (Array_Lit_2);
            end Finalizer_Scope_2829;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6276 '<Let at nodes.lkt:9347:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9347'
--# scope-start
--# expr-start 6234 '<FieldAccess for generic_inst_params at nodes.lkt:9347:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9347'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6234
Local_Ap := Fld; 
--# bind ap Local_Ap
--# expr-start 6235 '<FieldAccess for designated_generic_decl at nodes.lkt:9349:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9349'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6235
Var_Expr_178 := Fld_1;
if Var_Expr_178 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 6275 '<Map at nodes.lkt:9349:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9349'
   --# expr-start 6269 '<Mapcat at nodes.lkt:9349:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9349'
   --# expr-start 6237 '<FieldAccess for decls at nodes.lkt:9349:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9349'
--# expr-start 6236 '<FieldAccess for formal_part at nodes.lkt:9349:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9349'
      if Var_Expr_178.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_178.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_178.Info);
--# expr-done 6236
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Ada_Node_List (Node => Fld_2.Node.Generic_Formal_Part_F_Decls, Info => Fld_2.Info);
--# expr-done 6237
Fld_4 := Fld_3.Node;
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result_1 := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_4;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
Fld_5 := Fld_3.Info;
As_Entity_1 := (Info => Fld_5, Node => Bare_Item_0); 
   If_Result_2 := As_Entity_1;
end if;
                  Item_0 := If_Result_2; 
   --# scope-start
               --# bind d Item_0
      --# expr-start 6268 '<Match at nodes.lkt:9351:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9351'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Formal_Package =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Generic_Formal_Package
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_6; 
--# bind t Match_0
--# expr-start 6242 '<ArrayLiteral at nodes.lkt:9352:49>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9352'
--# expr-start 6241 '<New ParamActual at nodes.lkt:9352:50>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9352'
--# expr-start 6240 '<FieldAccess for name at nodes.lkt:9353:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9353'
--# expr-start 6239 '<Cast to GenericPackageInstantiation at nodes.lkt:9353:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9353'
--# expr-start 6238 '<FieldAccess for decl at nodes.lkt:9353:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9353'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Generic_Formal_F_Decl, Info => Match_0.Info);
--# expr-done 6238
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 6239
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Defining_Name (Node => Cast_Result.Node.Generic_Package_Instantiation_F_Name, Info => Cast_Result.Info);
--# expr-done 6240
New_Struct := (Actual => No_Entity_Expr, Param => Fld_7); 
--# expr-done 6241
Array_Lit := Create_Internal_Param_Actual_Array (Internal_Internal_Param_Actual_Array'(1 => New_Struct)); 
--# expr-done 6242
Let_Result_1 := Array_Lit; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2825;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_7; 
--# bind t Match_1
--# expr-start 6247 '<ArrayLiteral at nodes.lkt:9355:50>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9355'
--# expr-start 6246 '<New ParamActual at nodes.lkt:9355:51>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9355'
--# expr-start 6245 '<FieldAccess for name at nodes.lkt:9356:27>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9356'
--# expr-start 6244 '<Cast to BaseTypeDecl at nodes.lkt:9356:27>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9356'
--# expr-start 6243 '<FieldAccess for decl at nodes.lkt:9356:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9356'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Generic_Formal_F_Decl, Info => Match_1.Info);
--# expr-done 6243
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6244
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Defining_Name (Node => Cast_Result_1.Node.Base_Type_Decl_F_Name, Info => Cast_Result_1.Info);
--# expr-done 6245
New_Struct_1 := (Actual => No_Entity_Expr, Param => Fld_9); 
--# expr-done 6246
Array_Lit_1 := Create_Internal_Param_Actual_Array (Internal_Internal_Param_Actual_Array'(1 => New_Struct_1)); 
--# expr-done 6247
Let_Result_2 := Array_Lit_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2826;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Obj_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_8; 
--# bind o Match_2
--# expr-start 6255 '<Map at nodes.lkt:9358:49>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9358'
   --# expr-start 6250 '<FieldAccess for ids at nodes.lkt:9358:49>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9358'
--# expr-start 6249 '<Cast to ObjectDecl at nodes.lkt:9358:49>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9358'
--# expr-start 6248 '<FieldAccess for decl at nodes.lkt:9358:49>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9358'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Basic_Decl (Node => Match_2.Node.Generic_Formal_F_Decl, Info => Match_2.Info);
--# expr-done 6248
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Object_Decl
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_2 := No_Entity_Object_Decl;
   end if;
--# expr-done 6249
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Defining_Name_List (Node => Cast_Result_2.Node.Object_Decl_F_Ids, Info => Cast_Result_2.Info);
--# expr-done 6250
Fld_12 := Fld_11.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_12 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_12;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_13 := Fld_11.Info;
As_Entity := (Info => Fld_13, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind i Item_1
      --# expr-start 6254 '<New ParamActual at nodes.lkt:9359:28>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9359'
--# expr-start 6253 '<FieldAccess for default_expr at nodes.lkt:9360:41>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9360'
--# expr-start 6252 '<Cast to ObjectDecl at nodes.lkt:9360:41>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9360'
--# expr-start 6251 '<FieldAccess for decl at nodes.lkt:9360:41>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9360'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Basic_Decl (Node => Match_2.Node.Generic_Formal_F_Decl, Info => Match_2.Info);
--# expr-done 6251
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Object_Decl
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_3 := No_Entity_Object_Decl;
   end if;
--# expr-done 6252
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Cast_Result_3.Node.Object_Decl_F_Default_Expr, Info => Cast_Result_3.Info);
--# expr-done 6253
New_Struct_2 := (Actual => Fld_15, Param => Item_1); 
--# expr-done 6254
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_2;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6255
Let_Result_3 := Map_Result; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2827;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Generic_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_9; 
--# bind sp Match_3
--# expr-start 6267 '<ArrayLiteral at nodes.lkt:9363:51>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9363'
--# expr-start 6266 '<Let at nodes.lkt:9364:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9364'
--# scope-start
--# expr-start 6257 '<Cast to FormalSubpDecl at nodes.lkt:9364:32>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9364'
--# expr-start 6256 '<FieldAccess for decl at nodes.lkt:9364:32>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9364'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Basic_Decl (Node => Match_3.Node.Generic_Formal_F_Decl, Info => Match_3.Info);
--# expr-done 6256
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_4 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 6257
Local_Subp := Cast_Result_4; 
--# bind subp Local_Subp
--# expr-start 6265 '<New ParamActual at nodes.lkt:9366:21>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9366'
--# expr-start 6263 '<If at nodes.lkt:9367:58>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
--# expr-start 6259 '<IsA BoxExpr at nodes.lkt:9367:61>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
--# expr-start 6258 '<FieldAccess for default_expr at nodes.lkt:9367:61>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Local_Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Local_Subp.Info);
--# expr-done 6258
Is_A := Fld_21.Node /= null 
and then Fld_21.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 6259
if Is_A then
--# expr-start 6260 '<FieldAccess for designated_subprogram_from at nodes.lkt:9367:95>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start FormalSubpDecl.designated_subprogram_from
Fld_18 := Libadalang.Implementation.Formal_Subp_Decl_P_Designated_Subprogram_From (Node => Local_Subp.Node, Inst => Ent, E_Info => Local_Subp.Info);
--# end
--# expr-done 6260
Var_Expr_179 := Fld_18;
if Var_Expr_179 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6261 '<FieldAccess for defining_name at nodes.lkt:9367:95>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
      if Var_Expr_179.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_19 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_179.Node, E_Info => Var_Expr_179.Info);
--# end
--# expr-done 6261
Scope_Result := Fld_19;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
      Cast_Result_5 := Create_Internal_Entity_Expr
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_5;
else
   --# expr-start 6262 '<FieldAccess for default_expr at nodes.lkt:9367:160>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
      if Local_Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Local_Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Local_Subp.Info);
--# expr-done 6262
   If_Result_1 := Fld_20;
end if;
--# expr-done 6263
--# expr-start 6264 '<FieldAccess for defining_name at nodes.lkt:9367:31>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9367'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 6264
New_Struct_3 := (Actual => If_Result_1, Param => Fld_17); 
--# expr-done 6265
Let_Result := New_Struct_3; 
--# end
--# expr-done 6266
Array_Lit_2 := Create_Internal_Param_Actual_Array (Internal_Internal_Param_Actual_Array'(1 => Let_Result)); 
--# expr-done 6267
Let_Result_4 := Array_Lit_2; Inc_Ref (Let_Result_4);
--# end
Scope_Result_4 := Let_Result_4;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2829;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Pragma_Node =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Pragma_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_10; 
--# bind _ Match_4
Let_Result_5 := No_Internal_Param_Actual_Array_Type; Inc_Ref (Let_Result_5);
--# end
Scope_Result_5 := Let_Result_5;
   Inc_Ref (Scope_Result_5);
   --# end
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Use_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_11; 
--# bind _ Match_5
Let_Result_6 := No_Internal_Param_Actual_Array_Type; Inc_Ref (Let_Result_6);
--# end
Scope_Result_6 := Let_Result_6;
   Inc_Ref (Scope_Result_6);
   --# end
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Discriminant_Spec | Ada_Param_Spec .. Ada_Pp_Then_Kw | Ada_Private_Absent .. Ada_Until_Present | Ada_Value_Sequence .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_12; 
--# bind _ Match_6
Let_Result_7 := raise Property_Error with "unexpected declaration in generic formal part"; Inc_Ref (Let_Result_7);
--# end
Scope_Result_7 := Let_Result_7;
   Inc_Ref (Scope_Result_7);
   --# end
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
end case;
--# expr-done 6268
         for Item_To_Append of
               Match_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2824;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6269
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_3 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Map_Result_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_2
               --# bind i Index_3
      --# expr-start 6274 '<New ParamActual at nodes.lkt:9375:24>' New_Struct_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9375'
--# expr-start 6272 '<FieldAccess for actual_for_param_at at nodes.lkt:9378:24>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9378'
      if Local_Ap.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6270 '<FieldAccess for param at nodes.lkt:9378:47>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9378'
Fld_23 := Item_2.Param;
--# expr-done 6270
--# expr-start 6271 '<FieldAccess for actual at nodes.lkt:9378:60>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9378'
Fld_24 := Item_2.Actual;
--# expr-done 6271
--# property-call-start AssocList.actual_for_param_at
Fld_25 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Local_Ap.Node, Param => Fld_23, Pos => Index_3, Default_Expr => Fld_24, E_Info => Local_Ap.Info);
--# end
--# expr-done 6272
--# expr-start 6273 '<FieldAccess for param at nodes.lkt:9376:23>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9376'
Fld_22 := Item_2.Param;
--# expr-done 6273
New_Struct_4 := (Actual => Fld_25, Param => Fld_22); 
--# expr-done 6274
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_4;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_3 := Index_3 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 6275
Scope_Result_8 := Map_Result_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2823;
   Result_Var_1 := Scope_Result_8;
else
   Result_Var_1 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_8 := Result_Var_1; Inc_Ref (Let_Result_8);
--# end
--# expr-done 6276
         Property_Result := Let_Result_8;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_977;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_977;
                     Finalizer_Scope_2823;
                     Finalizer_Scope_2824;
                     Finalizer_Scope_2825;
                     Finalizer_Scope_2826;
                     Finalizer_Scope_2827;
                     Finalizer_Scope_2829;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Inst_Params;
--# end
--# property-start GenericInstantiation.actual_for_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9386'
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Actual_For_Formal
  (Node : Bare_Generic_Instantiation
      ; Formal_Name : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind formal_name Formal_Name
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_2835 with Inline_Always;
      Local_Gen_Decl : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Param_Match;
Var_Expr_180 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Generic_Formal_Part;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Bare_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
Is_Equal : Boolean;
Find_Result : Internal_Param_Match;
Fld_6 : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_2835 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2835;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6289 '<Let at nodes.lkt:9387:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9387'
--# scope-start
--# expr-start 6278 '<Cast to GenericDecl at nodes.lkt:9387:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9387'
--# expr-start 6277 '<FieldAccess for nonbound_generic_decl_from_self at nodes.lkt:9387:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9387'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_self
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Self (Node => Self);
--# end
--# expr-done 6277
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Generic_Decl;
   end if;
--# expr-done 6278
Local_Gen_Decl := Cast_Result; 
--# bind gen_decl Local_Gen_Decl
Var_Expr_180 := Local_Gen_Decl;
if Var_Expr_180 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 6285 '<Find at nodes.lkt:9389:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
   --# expr-start 6281 '<FieldAccess for match_param_list at nodes.lkt:9389:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
--# expr-start 6279 '<FieldAccess for formal_part at nodes.lkt:9389:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
      if Var_Expr_180.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_180.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_180.Info);
--# expr-done 6279
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6280 '<FieldAccess for generic_inst_params at nodes.lkt:9389:48>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6280
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_1.Node, Params => Fld_2, Is_Dottable_Subp => False, E_Info => Fld_1.Info);
--# end
--# expr-done 6281
      Find_Result := No_Param_Match;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
      --# expr-start 6284 '<Eq at nodes.lkt:9389:96>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
--# expr-start 6283 '<FieldAccess for node at nodes.lkt:9389:96>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
--# expr-start 6282 '<FieldAccess for formal at nodes.lkt:9389:96>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
Fld_4 := Item_0.Formal;
--# expr-done 6282
Fld_5 := Fld_4.Node;
--# expr-done 6283
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Formal_Name; 
--# expr-done 6284
      if Is_Equal then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_Equal;
         end loop;
      end;
--# expr-done 6285
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Param_Match then
   --# scope-start
   --# bind pm Var_Expr_1
--# expr-start 6288 '<FieldAccess for expr at nodes.lkt:9389:138>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
--# expr-start 6287 '<FieldAccess for assoc at nodes.lkt:9389:138>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
--# expr-start 6286 '<FieldAccess for actual at nodes.lkt:9389:138>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9389'
Fld_6 := Var_Expr_1.Actual;
--# expr-done 6286
Fld_7 := Fld_6.Assoc;
--# expr-done 6287
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6288
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_2835;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 6289
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2835;
            raise;
      end;
   return Property_Result;
end Generic_Instantiation_P_Actual_For_Formal;
--# end
--# property-start GenericInstantiation._env_do_164 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9393'
pragma Warnings (Off, "is not referenced");
 function Env_Do_164
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6290 '<FieldAccess for env_hook at nodes.lkt:9393:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9393'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6290
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_164;
--# end
--# property-start GenericInstantiation._env_init_165 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9395'
pragma Warnings (Off, "is not referenced");
 function Env_Init_165
  (Node : Bare_Generic_Instantiation
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1829 with Inline_Always;
            procedure Finalizer_Scope_2838 with Inline_Always;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1829 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1829;
            procedure Finalizer_Scope_2838 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2838;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6291 '<FieldAccess for initial_env_name at nodes.lkt:9395:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9395'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.initial_env_name
Fld := Libadalang.Implementation.Generic_Instantiation_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 6291
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind non_null_name Var_Expr_0
--# expr-start 6292 '<New DesignatedEnv at nodes.lkt:9396:36>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9396'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Var_Expr_0, Kind => Named_Env); 
--# expr-done 6292
Scope_Result := New_Struct;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2838;
   Result_Var := Scope_Result;
else
   --# expr-start 6293 '<New DesignatedEnv at nodes.lkt:9398:32>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9398'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6293
   Result_Var := New_Struct_1;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1829;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1829;
                     Finalizer_Scope_2838;
            raise;
      end;
   return Property_Result;
end Env_Init_165;
--# end
--# property-start GenericInstantiation._env_mappings_166 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9403'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_166
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
     Ent : Internal_Entity_Generic_Instantiation :=
       Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1830 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1830 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1830;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6296 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6294 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6294
--# expr-start 6295 '<FieldAccess for name_symbol at nodes.lkt:9403:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9403'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 6295
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6296
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1830;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1830;
            raise;
      end;
   return Property_Result;
end Env_Mappings_166;
--# end
--# property-start GenericInstantiation._env_trans_parent_167 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_167
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_167;
--# end
--# property-start GenericInstantiation._env_names_168 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9404'
pragma Warnings (Off, "is not referenced");
 function Env_Names_168
  (Node : Bare_Generic_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1832 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1832 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1832;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6297 '<FieldAccess for env_names at nodes.lkt:9404:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9404'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.env_names
Fld := Libadalang.Implementation.Generic_Instantiation_P_Env_Names (Node => Self);
--# end
--# expr-done 6297
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1832;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1832;
            raise;
      end;
   return Property_Result;
end Env_Names_168;
--# end
--# property-start GenericInstantiation._env_do_169 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9405'
pragma Warnings (Off, "is not referenced");
 function Env_Do_169
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1833 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1833 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1833;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6298 '<FieldAccess for populate_dependent_units at nodes.lkt:9405:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9405'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6298
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1833;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1833;
            raise;
      end;
   return Property_Result;
end Env_Do_169;
--# end
--# property-start GenericInstantiation._ref_env_nodes_170 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9407'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_170
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1834 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1834 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1834;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6299 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9407:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9407'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6299
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1834;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1834;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_170;
--# end
--# property-start GenericInstantiation._ref_cond_171 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9407'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_171
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6301 '<IsA LibraryItem, Subunit at nodes.lkt:9407:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9407'
--# expr-start 6300 '<parent at nodes.lkt:9407:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9407'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6300
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6301
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_171;
--# end
--# property-start GenericInstantiation._ref_env_nodes_172 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9410'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_172
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1836 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1836 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1836;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6302 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9410:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9410'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6302
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1836;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1836;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_172;
--# end
--# property-start GenericInstantiation._ref_cond_173 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9410'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_173
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6304 '<IsA LibraryItem, Subunit at nodes.lkt:9410:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9410'
--# expr-start 6303 '<parent at nodes.lkt:9410:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9410'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6303
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6304
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_173;
--# end
--# property-start GenericInstantiation._ref_env_nodes_174 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_174
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1838 with Inline_Always;
      Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Array_Lit : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1838 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1838;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6306 '<ArrayLiteral at nodes.lkt:9413:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
--# expr-start 6305 '<Cast to AdaNode at nodes.lkt:9413:14>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 6305
Array_Lit := Create_Bare_Ada_Node_Array (Internal_Bare_Ada_Node_Array'(1 => Cast_Result)); 
--# expr-done 6306
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1838;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1838;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_174;
--# end
--# property-start GenericInstantiation._ref_cond_175 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_175
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation  := Bare_Generic_Instantiation (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6308 '<IsA LibraryItem, Subunit at nodes.lkt:9413:85>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
--# expr-start 6307 '<parent at nodes.lkt:9413:85>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9413'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6307
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6308
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_175;
--# end
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Generic_Instantiation
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Internal_Inner_Env_Assoc_Array_Access :=
         Generic_Instantiation_P_Instantiation_Bindings (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Generic_Instantiation_P_Parent_Instantiation_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Generic_Instantiation
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Generic_Instantiation_P_Parent_Instantiation_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Generic_Package_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Package_Instantiation
        (Self : Bare_Generic_Package_Instantiation
         ; Generic_Package_Instantiation_F_Name : Bare_Defining_Name
         ; Generic_Package_Instantiation_F_Generic_Pkg_Name : Bare_Name
         ; Generic_Package_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Package_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Package_Instantiation_F_Name := Generic_Package_Instantiation_F_Name;
            Self.Generic_Package_Instantiation_F_Generic_Pkg_Name := Generic_Package_Instantiation_F_Generic_Pkg_Name;
            Self.Generic_Package_Instantiation_F_Params := Generic_Package_Instantiation_F_Params;
            Self.Generic_Package_Instantiation_F_Aspects := Generic_Package_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Instantiation;
   function Generic_Package_Instantiation_F_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Name;
   end;
   function Generic_Package_Instantiation_F_Generic_Pkg_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Generic_Pkg_Name;
   end;
   function Generic_Package_Instantiation_F_Params
     (Node : Bare_Generic_Package_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Package_Instantiation_F_Params;
   end;
--# property-start GenericPackageInstantiation.generic_entity_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9425'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6309 '<FieldAccess for generic_pkg_name at nodes.lkt:9425:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9425'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Generic_Pkg_Name, Info => Ent.Info);
--# expr-done 6309
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericPackageInstantiation.generic_inst_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9427'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6310 '<FieldAccess for params at nodes.lkt:9427:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9427'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Package_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 6310
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericPackageInstantiation.designated_package 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9429'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Package
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Package_Decl
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_2839 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Basic_Decl;
Cast_Result : Bare_Generic_Package_Decl;
Fld_2 : Bare_Generic_Package_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Info;
Fld_8 : Env_Rebindings;
Fld_9 : Env_Rebindings;
Fld_10 : Bare_Basic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Fld_13 : Env_Rebindings;
Fld_14 : Internal_Entity_Info;
Fld_15 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Base_Package_Decl;
New_Struct_1 : Internal_Entity_Base_Package_Decl;
Scope_Result : Internal_Entity_Base_Package_Decl;
Result_Var : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_2839 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2839;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6311 '<FieldAccess for nonbound_generic_decl_from_entity at nodes.lkt:9429:57>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9429'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_entity
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Entity (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6311
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 6329 '<New BasePackageDecl.entity at nodes.lkt:9430:16>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9430'
--# expr-start 6325 '<New EntityInfo at nodes.lkt:9431:67>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9431'
--# expr-start 6313 '<FieldAccess for from_rebound at nodes.lkt:9443:33>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9443'
--# expr-start 6312 '<FieldAccess for info at nodes.lkt:9443:33>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9443'
Fld_14 := Var_Expr_0.Info;
--# expr-done 6312
Fld_15 := Fld_14.From_Rebound;
--# expr-done 6313
--# expr-start 6315 '<FieldAccess for md at nodes.lkt:9432:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9432'
--# expr-start 6314 '<FieldAccess for info at nodes.lkt:9432:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9432'
Fld_3 := Var_Expr_0.Info;
--# expr-done 6314
Fld_4 := Fld_3.Md;
--# expr-done 6315
--# expr-start 6324 '<FieldAccess for add_rebinding at nodes.lkt:9432:42>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9432'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6320 '<FieldAccess for insert_rebindings at nodes.lkt:9436:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9436'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6317 '<FieldAccess for rebindings at nodes.lkt:9436:44>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9436'
--# expr-start 6316 '<FieldAccess for info at nodes.lkt:9436:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9436'
Fld_5 := Var_Expr_0.Info;
--# expr-done 6316
Fld_6 := Fld_5.Rebindings;
--# expr-done 6317
--# expr-start 6319 '<FieldAccess for rebindings at nodes.lkt:9436:63>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9436'
--# expr-start 6318 '<FieldAccess for info at nodes.lkt:9436:63>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9436'
Fld_7 := Ent.Info;
--# expr-done 6318
Fld_8 := Fld_7.Rebindings;
--# expr-done 6319
--# property-call-start AdaNode.insert_rebindings
Fld_9 := Libadalang.Implementation.Ada_Node_P_Insert_Rebindings (Node => Self, Base => Fld_6, To_Insert => Fld_8);
--# end
--# expr-done 6320
--# expr-start 6322 '<FieldAccess for children_env at nodes.lkt:9442:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9442'
--# expr-start 6321 '<FieldAccess for node at nodes.lkt:9442:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9442'
Fld_10 := Var_Expr_0.Node;
--# expr-done 6321
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 6322
--# expr-start 6323 '<FieldAccess for instantiation_env at nodes.lkt:9442:44>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9442'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 6323
--# property-call-start AdaNode.add_rebinding
Fld_13 := Libadalang.Implementation.Ada_Node_P_Add_Rebinding (Node => Self, Base => Fld_9, Old_Env => Fld_11, New_Env => Fld_12);
--# end
--# expr-done 6324
New_Struct := (From_Rebound => Fld_15, Md => Fld_4, Rebindings => Fld_13); 
--# expr-done 6325
--# expr-start 6328 '<FieldAccess for package_decl at nodes.lkt:9431:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9431'
--# expr-start 6327 '<Cast to GenericPackageDecl at nodes.lkt:9431:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9431'
--# expr-start 6326 '<FieldAccess for node at nodes.lkt:9431:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9431'
Fld_1 := Var_Expr_0.Node;
--# expr-done 6326
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6327
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Generic_Package_Decl_F_Package_Decl;
--# expr-done 6328
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 6329
Scope_Result := New_Struct_1;
   --# end
      Finalizer_Scope_2839;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Package_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2839;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Designated_Package;
--# end
--# property-start GenericPackageInstantiation.designated_generic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9448'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
      Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6332 '<Cast to GenericDecl at nodes.lkt:9449:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9449'
--# expr-start 6331 '<parent at nodes.lkt:9449:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9449'
--# expr-start 6330 '<FieldAccess for designated_package at nodes.lkt:9449:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6330
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6331
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6332
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericPackageInstantiation.defining_env_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9458'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env_Impl
  (Node : Bare_Generic_Package_Instantiation
      ; Inst_From_Formal : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind inst_from_formal Inst_From_Formal
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_983 with Inline_Always;
      Local_Dp : Internal_Entity_Base_Package_Decl;
Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_983 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_983;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6347 '<Let at nodes.lkt:9459:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9459'
--# scope-start
--# expr-start 6333 '<FieldAccess for designated_package at nodes.lkt:9459:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9459'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6333
Local_Dp := Fld; 
--# bind dp Local_Dp
--# expr-start 6346 '<If at nodes.lkt:9480:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9480'
--# expr-start 6334 '<IsNull at nodes.lkt:9480:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9480'
Is_Null := Local_Dp.Node = null; 
--# expr-done 6334
if Is_Null then
   If_Result_2 := Empty_Env;
else
   --# expr-start 6345 '<env_group at nodes.lkt:9482:14>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9482'
--# expr-start 6344 '<ArrayLiteral at nodes.lkt:9482:14>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9482'
--# expr-start 6342 '<If at nodes.lkt:9483:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9483'
--# expr-start 6335 '<FieldAccess for is_formal at nodes.lkt:9483:16>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9483'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_formal
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Self);
--# end
--# expr-done 6335
if Fld_5 then
   If_Result := True;
else
   If_Result := Inst_From_Formal;
end if;
if If_Result then
   --# expr-start 6340 '<env_group at nodes.lkt:9484:17>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9484'
--# expr-start 6339 '<ArrayLiteral at nodes.lkt:9484:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9484'
--# expr-start 6336 '<FieldAccess for children_env at nodes.lkt:9484:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9484'
      if Local_Dp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Local_Dp.Node, E_Info => Local_Dp.Info);
--# expr-done 6336
--# expr-start 6338 '<FieldAccess for children_env at nodes.lkt:9484:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9484'
--# expr-start 6337 '<parent at nodes.lkt:9484:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9484'
      if Local_Dp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Local_Dp.Node, E_Info => Local_Dp.Info);
--# expr-done 6337
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 6338
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_3)); 
--# expr-done 6339
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 6340
   If_Result_1 := Group_Env;
else
   --# expr-start 6341 '<FieldAccess for children_env at nodes.lkt:9486:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9486'
      if Local_Dp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Local_Dp.Node, E_Info => Local_Dp.Info);
--# expr-done 6341
   If_Result_1 := Fld_4;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6342
--# expr-start 6343 '<FieldAccess for children_env at nodes.lkt:9497:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9497'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6343
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result_1, 2 => Fld_6)); 
--# expr-done 6344
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 6345
   If_Result_2 := Group_Env_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6346
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6347
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_983;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_983;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Env_Impl;
--# end
--# property-start GenericPackageInstantiation.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9502'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Package_Instantiation
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_984 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_984 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_984;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6348 '<FieldAccess for defining_env_impl at nodes.lkt:9502:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9502'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6348
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_984;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_984;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Env;
--# end
--# property-start GenericPackageInstantiation.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9504'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Instantiation  := Bare_Generic_Package_Instantiation (Node);
     Ent : Internal_Entity_Generic_Package_Instantiation :=
       Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_985 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_985 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_985;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6350 '<ArrayLiteral at nodes.lkt:9504:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9504'
--# expr-start 6349 '<FieldAccess for name at nodes.lkt:9504:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9504'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 6349
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6350
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_985;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_985;
            raise;
      end;
   return Property_Result;
end Generic_Package_Instantiation_P_Defining_Names;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Subp_Instantiation
        (Self : Bare_Generic_Subp_Instantiation
         ; Generic_Subp_Instantiation_F_Overriding : Bare_Overriding_Node
         ; Generic_Subp_Instantiation_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Instantiation_F_Subp_Name : Bare_Defining_Name
         ; Generic_Subp_Instantiation_F_Generic_Subp_Name : Bare_Name
         ; Generic_Subp_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Subp_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Subp_Instantiation_F_Overriding := Generic_Subp_Instantiation_F_Overriding;
            Self.Generic_Subp_Instantiation_F_Kind := Generic_Subp_Instantiation_F_Kind;
            Self.Generic_Subp_Instantiation_F_Subp_Name := Generic_Subp_Instantiation_F_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Generic_Subp_Name := Generic_Subp_Instantiation_F_Generic_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Params := Generic_Subp_Instantiation_F_Params;
            Self.Generic_Subp_Instantiation_F_Aspects := Generic_Subp_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Instantiation;
   function Generic_Subp_Instantiation_F_Overriding
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Overriding_Node
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Overriding;
   end;
   function Generic_Subp_Instantiation_F_Kind
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Kind;
   end;
   function Generic_Subp_Instantiation_F_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Generic_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Generic_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Params
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Params;
   end;
--# property-start GenericSubpInstantiation.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9516'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_986 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_986 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_986;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6352 '<ArrayLiteral at nodes.lkt:9516:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9516'
--# expr-start 6351 '<FieldAccess for subp_name at nodes.lkt:9516:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9516'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Subp_Name, Info => Ent.Info);
--# expr-done 6351
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6352
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_986;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_986;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Defining_Names;
--# end
--# property-start GenericSubpInstantiation.generic_entity_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9518'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6353 '<FieldAccess for generic_subp_name at nodes.lkt:9518:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9518'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Generic_Subp_Name, Info => Ent.Info);
--# expr-done 6353
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericSubpInstantiation.generic_inst_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9520'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6354 '<FieldAccess for params at nodes.lkt:9520:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9520'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Subp_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 6354
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericSubpInstantiation.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9523'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Expr_Type
  (Node : Bare_Generic_Subp_Instantiation
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_181 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6355 '<FieldAccess for subp_spec_or_null at nodes.lkt:9524:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9524'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6355
Var_Expr_181 := Fld;
if Var_Expr_181 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 6356 '<FieldAccess for return_type at nodes.lkt:9524:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9524'
      if Var_Expr_181.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_181.Node, Origin => Origin, E_Info => Var_Expr_181.Info);
--# end
--# expr-done 6356
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Expr_Type;
--# end
--# property-start GenericSubpInstantiation.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9527'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Subp_Instantiation
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_990 with Inline_Always;
            procedure Finalizer_Scope_2841 with Inline_Always;
      Var_Expr_182 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_990 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_990;
            procedure Finalizer_Scope_2841 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2841;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6357 '<FieldAccess for subp_spec_or_null at nodes.lkt:9528:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9528'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6357
Var_Expr_182 := Fld;
if Var_Expr_182 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 6358 '<FieldAccess for defining_env at nodes.lkt:9528:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9528'
      if Var_Expr_182.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Var_Expr_182.Node, Origin => Origin, E_Info => Var_Expr_182.Info);
--# end
--# expr-done 6358
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2841;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_990;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_990;
                     Finalizer_Scope_2841;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Defining_Env;
--# end
--# property-start GenericSubpInstantiation.designated_subp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9532'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Subp
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Subp_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Subp_Decl;
            procedure Finalizer_Scope_2842 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Basic_Decl;
Cast_Result : Bare_Generic_Subp_Decl;
Fld_2 : Bare_Generic_Subp_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Info;
Fld_8 : Env_Rebindings;
Fld_9 : Env_Rebindings;
Fld_10 : Bare_Basic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Fld_13 : Env_Rebindings;
Fld_14 : Internal_Entity_Info;
Fld_15 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Basic_Subp_Decl;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Scope_Result : Internal_Entity_Basic_Subp_Decl;
Result_Var : Internal_Entity_Basic_Subp_Decl;
            procedure Finalizer_Scope_2842 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2842;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 6359 '<FieldAccess for nonbound_generic_decl_from_entity at nodes.lkt:9532:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9532'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl_from_entity
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl_From_Entity (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6359
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 6377 '<New BasicSubpDecl.entity at nodes.lkt:9533:16>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9533'
--# expr-start 6373 '<New EntityInfo at nodes.lkt:9534:61>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9534'
--# expr-start 6361 '<FieldAccess for from_rebound at nodes.lkt:9537:33>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9537'
--# expr-start 6360 '<FieldAccess for info at nodes.lkt:9537:33>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9537'
Fld_14 := Var_Expr_0.Info;
--# expr-done 6360
Fld_15 := Fld_14.From_Rebound;
--# expr-done 6361
--# expr-start 6363 '<FieldAccess for md at nodes.lkt:9535:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9535'
--# expr-start 6362 '<FieldAccess for info at nodes.lkt:9535:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9535'
Fld_3 := Ent.Info;
--# expr-done 6362
Fld_4 := Fld_3.Md;
--# expr-done 6363
--# expr-start 6372 '<FieldAccess for add_rebinding at nodes.lkt:9535:45>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9535'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6368 '<FieldAccess for insert_rebindings at nodes.lkt:9536:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6365 '<FieldAccess for rebindings at nodes.lkt:9536:44>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
--# expr-start 6364 '<FieldAccess for info at nodes.lkt:9536:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
Fld_5 := Var_Expr_0.Info;
--# expr-done 6364
Fld_6 := Fld_5.Rebindings;
--# expr-done 6365
--# expr-start 6367 '<FieldAccess for rebindings at nodes.lkt:9536:63>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
--# expr-start 6366 '<FieldAccess for info at nodes.lkt:9536:63>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
Fld_7 := Ent.Info;
--# expr-done 6366
Fld_8 := Fld_7.Rebindings;
--# expr-done 6367
--# property-call-start AdaNode.insert_rebindings
Fld_9 := Libadalang.Implementation.Ada_Node_P_Insert_Rebindings (Node => Self, Base => Fld_6, To_Insert => Fld_8);
--# end
--# expr-done 6368
--# expr-start 6370 '<FieldAccess for children_env at nodes.lkt:9536:86>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
--# expr-start 6369 '<FieldAccess for node at nodes.lkt:9536:86>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
Fld_10 := Var_Expr_0.Node;
--# expr-done 6369
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 6370
--# expr-start 6371 '<FieldAccess for instantiation_env at nodes.lkt:9536:109>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9536'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 6371
--# property-call-start AdaNode.add_rebinding
Fld_13 := Libadalang.Implementation.Ada_Node_P_Add_Rebinding (Node => Self, Base => Fld_9, Old_Env => Fld_11, New_Env => Fld_12);
--# end
--# expr-done 6372
New_Struct := (From_Rebound => Fld_15, Md => Fld_4, Rebindings => Fld_13); 
--# expr-done 6373
--# expr-start 6376 '<FieldAccess for subp_decl at nodes.lkt:9534:18>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9534'
--# expr-start 6375 '<Cast to GenericSubpDecl at nodes.lkt:9534:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9534'
--# expr-start 6374 '<FieldAccess for node at nodes.lkt:9534:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9534'
Fld_1 := Var_Expr_0.Node;
--# expr-done 6374
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6375
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Generic_Subp_Decl_F_Subp_Decl;
--# expr-done 6376
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 6377
Scope_Result := New_Struct_1;
   --# end
      Finalizer_Scope_2842;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Subp_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2842;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Designated_Subp;
--# end
--# property-start GenericSubpInstantiation.designated_generic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9542'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Subp_Instantiation  := Bare_Generic_Subp_Instantiation (Node);
     Ent : Internal_Entity_Generic_Subp_Instantiation :=
       Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
      Fld : Internal_Entity_Basic_Subp_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6380 '<Cast to GenericDecl at nodes.lkt:9543:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9543'
--# expr-start 6379 '<parent at nodes.lkt:9543:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9543'
--# expr-start 6378 '<FieldAccess for designated_subp at nodes.lkt:9543:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9543'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6378
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6379
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6380
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Instantiation_P_Designated_Generic_Decl;
--# end
   --
   --  Primitives for Bare_Generic_Renaming_Decl
   --
--# property-start '[dispatcher]GenericRenamingDecl.renaming_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Generic_Renaming_Decl (Self.Kind) is
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.renaming_name
                  Property_Result := Generic_Package_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.renaming_name
                  Property_Result := Generic_Subp_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Generic_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericRenamingDecl.resolve 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9553'
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Resolve
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Renaming_Decl :=
       Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_994 with Inline_Always;
      Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Generic_Decl;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_2 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_1 : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result_2 : Internal_Entity_Generic_Decl;
Match_Result : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_994 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_994;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6386 '<at at nodes.lkt:9557:9>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9557'
--# expr-start 6385 '<Map at nodes.lkt:9557:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9557'
   --# expr-start 6382 '<FieldAccess for all_env_elements_internal at nodes.lkt:9557:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9557'
--# expr-start 6381 '<FieldAccess for renaming_name at nodes.lkt:9557:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9557'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6381
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True), E_Info => Fld.Info);
--# end
--# expr-done 6382
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
         --# expr-start 6384 '<Not at nodes.lkt:9559:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9559'
--# expr-start 6383 '<IsA Body at nodes.lkt:9559:29>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9559'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Body_Node; 
--# expr-done 6383
Not_Val := not (Is_A); 
--# expr-done 6384
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6385
Get_Result := Get (Self, Map_Result, 0, True); 
--# expr-done 6386
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 6388 '<Match at nodes.lkt:9560:21>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9560'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind gd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind grd Match_1
--# expr-start 6387 '<FieldAccess for resolve at nodes.lkt:9562:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9562'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_2 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6387
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Generic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6388
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_994;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_994;
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Resolve;
--# end
--# property-start GenericRenamingDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9567'
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Generic_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start GenericRenamingDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9570'
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Generic_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Renaming_Decl  := Bare_Generic_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Renaming_Decl :=
       Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_996 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_996 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_996;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6390 '<FieldAccess for xref_no_overloading at nodes.lkt:9570:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9570'
--# expr-start 6389 '<FieldAccess for renaming_name at nodes.lkt:9570:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9570'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6389
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 6390
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_996;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_996;
            raise;
      end;
   return Property_Result;
end Generic_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Package_Renaming_Decl
   --
      procedure Generic_Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_176 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_177 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9588:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_178 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9589:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_179 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_180 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_182 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_181 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_184 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_183 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Renaming_Decl
        (Self : Bare_Generic_Package_Renaming_Decl
         ; Generic_Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Package_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Generic_Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Package_Renaming_Decl_F_Name := Generic_Package_Renaming_Decl_F_Name;
            Self.Generic_Package_Renaming_Decl_F_Renames := Generic_Package_Renaming_Decl_F_Renames;
            Self.Generic_Package_Renaming_Decl_F_Aspects := Generic_Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Renaming_Decl;
   function Generic_Package_Renaming_Decl_F_Name
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Name;
   end;
   function Generic_Package_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Renames;
   end;
--# property-start GenericPackageRenamingDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9579'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_997 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_997 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_997;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6392 '<ArrayLiteral at nodes.lkt:9579:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9579'
--# expr-start 6391 '<FieldAccess for name at nodes.lkt:9579:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9579'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 6391
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6392
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_997;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_997;
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageRenamingDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9582'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_998 with Inline_Always;
      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_998 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_998;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6394 '<FieldAccess for defining_env at nodes.lkt:9582:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9582'
--# expr-start 6393 '<FieldAccess for resolve at nodes.lkt:9582:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9582'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6393
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6394
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_998;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_998;
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageRenamingDecl.renaming_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9584'
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6396 '<FieldAccess for renamed_object at nodes.lkt:9584:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9584'
--# expr-start 6395 '<FieldAccess for renames at nodes.lkt:9584:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9584'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 6395
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 6396
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Package_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericPackageRenamingDecl._env_do_176 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9587'
pragma Warnings (Off, "is not referenced");
 function Env_Do_176
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6397 '<FieldAccess for env_hook at nodes.lkt:9587:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9587'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6397
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_176;
--# end
--# property-start GenericPackageRenamingDecl._env_init_177 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9588'
pragma Warnings (Off, "is not referenced");
 function Env_Init_177
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1841 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1841 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1841;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6398 '<FieldAccess for child_decl_initial_env at nodes.lkt:9588:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9588'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 6398
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1841;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1841;
            raise;
      end;
   return Property_Result;
end Env_Init_177;
--# end
--# property-start GenericPackageRenamingDecl._env_mappings_178 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9589'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_178
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
       Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1842 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1842 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1842;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6401 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6399 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6399
--# expr-start 6400 '<FieldAccess for name_symbol at nodes.lkt:9589:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9589'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 6400
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6401
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1842;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1842;
            raise;
      end;
   return Property_Result;
end Env_Mappings_178;
--# end
--# property-start GenericPackageRenamingDecl._env_trans_parent_179 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_179
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_179;
--# end
--# property-start GenericPackageRenamingDecl._env_do_180 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9591'
pragma Warnings (Off, "is not referenced");
 function Env_Do_180
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1844 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1844 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1844;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6402 '<FieldAccess for populate_dependent_units at nodes.lkt:9591:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9591'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6402
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1844;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1844;
            raise;
      end;
   return Property_Result;
end Env_Do_180;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_181 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9593'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_181
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1845 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1845 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1845;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6403 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9593:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9593'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6403
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1845;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1845;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_181;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_182 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9593'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_182
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6405 '<IsA LibraryItem, Subunit at nodes.lkt:9593:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9593'
--# expr-start 6404 '<parent at nodes.lkt:9593:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9593'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6404
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6405
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_182;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_183 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9596'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_183
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1847 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1847 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1847;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6406 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9596:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9596'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6406
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1847;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1847;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_183;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_184 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9596'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_184
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl  := Bare_Generic_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6408 '<IsA LibraryItem, Subunit at nodes.lkt:9596:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9596'
--# expr-start 6407 '<parent at nodes.lkt:9596:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9596'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6407
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6408
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_184;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Renaming_Decl
   --
      procedure Generic_Subp_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_185 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_186 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9614:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_187 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9615:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_188 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_189 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_191 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_190 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_193 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_192 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Renaming_Decl
        (Self : Bare_Generic_Subp_Renaming_Decl
         ; Generic_Subp_Renaming_Decl_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Subp_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Generic_Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Renaming_Decl_F_Kind := Generic_Subp_Renaming_Decl_F_Kind;
            Self.Generic_Subp_Renaming_Decl_F_Name := Generic_Subp_Renaming_Decl_F_Name;
            Self.Generic_Subp_Renaming_Decl_F_Renames := Generic_Subp_Renaming_Decl_F_Renames;
            Self.Generic_Subp_Renaming_Decl_F_Aspects := Generic_Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Renaming_Decl;
   function Generic_Subp_Renaming_Decl_F_Kind
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Kind;
   end;
   function Generic_Subp_Renaming_Decl_F_Name
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Name;
   end;
   function Generic_Subp_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Renames;
   end;
--# property-start GenericSubpRenamingDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9608'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1000 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1000 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1000;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6410 '<ArrayLiteral at nodes.lkt:9608:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9608'
--# expr-start 6409 '<FieldAccess for name at nodes.lkt:9608:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9608'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 6409
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6410
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1000;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1000;
            raise;
      end;
   return Property_Result;
end Generic_Subp_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpRenamingDecl.renaming_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9610'
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6412 '<FieldAccess for renamed_object at nodes.lkt:9610:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9610'
--# expr-start 6411 '<FieldAccess for renames at nodes.lkt:9610:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9610'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 6411
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 6412
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Generic_Subp_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericSubpRenamingDecl._env_do_185 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9613'
pragma Warnings (Off, "is not referenced");
 function Env_Do_185
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6413 '<FieldAccess for env_hook at nodes.lkt:9613:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9613'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6413
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_185;
--# end
--# property-start GenericSubpRenamingDecl._env_init_186 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9614'
pragma Warnings (Off, "is not referenced");
 function Env_Init_186
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1850 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1850 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1850;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6414 '<FieldAccess for child_decl_initial_env at nodes.lkt:9614:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9614'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 6414
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1850;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1850;
            raise;
      end;
   return Property_Result;
end Env_Init_186;
--# end
--# property-start GenericSubpRenamingDecl._env_mappings_187 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9615'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_187
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
     Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
       Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1851 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Generic_Subp_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1851 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1851;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6417 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6415 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6415
--# expr-start 6416 '<FieldAccess for name_symbol at nodes.lkt:9615:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9615'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 6416
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6417
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1851;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1851;
            raise;
      end;
   return Property_Result;
end Env_Mappings_187;
--# end
--# property-start GenericSubpRenamingDecl._env_trans_parent_188 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_188
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_188;
--# end
--# property-start GenericSubpRenamingDecl._env_do_189 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9617'
pragma Warnings (Off, "is not referenced");
 function Env_Do_189
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1853 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1853 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1853;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6418 '<FieldAccess for populate_dependent_units at nodes.lkt:9617:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9617'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6418
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1853;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1853;
            raise;
      end;
   return Property_Result;
end Env_Do_189;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_190 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9619'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_190
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1854 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1854 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1854;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6419 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9619:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9619'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6419
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1854;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1854;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_190;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_191 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9619'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_191
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6421 '<IsA LibraryItem, Subunit at nodes.lkt:9619:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9619'
--# expr-start 6420 '<parent at nodes.lkt:9619:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9619'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6420
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6421
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_191;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_192 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9622'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_192
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1856 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1856 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1856;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6422 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9622:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9622'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6422
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1856;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1856;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_192;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_193 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9622'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_193
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl  := Bare_Generic_Subp_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6424 '<IsA LibraryItem, Subunit at nodes.lkt:9622:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9622'
--# expr-start 6423 '<parent at nodes.lkt:9622:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9622'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6423
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6424
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_193;
--# end
   --
   --  Primitives for Bare_Label_Decl
   --
      procedure Label_Decl_Pre_Env_Actions
        (Self            : Bare_Label_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_194 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9635:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Label_Decl
        (Self : Bare_Label_Decl
         ; Label_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Label_Decl_F_Name := Label_Decl_F_Name;
      end Initialize_Fields_For_Label_Decl;
   function Label_Decl_F_Name
     (Node : Bare_Label_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Label_Decl_F_Name;
   end;
--# property-start LabelDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9632'
pragma Warnings (Off, "is not referenced");
 function Label_Decl_P_Defining_Names
  (Node : Bare_Label_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Label_Decl  := Bare_Label_Decl (Node);
     Ent : Internal_Entity_Label_Decl :=
       Internal_Entity_Label_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1002 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1002 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1002;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6426 '<ArrayLiteral at nodes.lkt:9632:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9632'
--# expr-start 6425 '<FieldAccess for name at nodes.lkt:9632:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9632'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Label_Decl_F_Name, Info => Ent.Info);
--# expr-done 6425
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6426
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1002;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1002;
            raise;
      end;
   return Property_Result;
end Label_Decl_P_Defining_Names;
--# end
--# property-start LabelDecl._env_mappings_194 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9635'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_194
  (Node : Bare_Label_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Label_Decl  := Bare_Label_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1858 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Label_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1858 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1858;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6429 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6427 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6427
--# expr-start 6428 '<FieldAccess for name_symbol at nodes.lkt:9635:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9635'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 6428
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6429
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1858;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1858;
            raise;
      end;
   return Property_Result;
end Env_Mappings_194;
--# end
   --
   --  Primitives for Bare_Named_Stmt_Decl
   --
      procedure Initialize_Fields_For_Named_Stmt_Decl
        (Self : Bare_Named_Stmt_Decl
         ; Named_Stmt_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Named_Stmt_Decl_F_Name := Named_Stmt_Decl_F_Name;
      end Initialize_Fields_For_Named_Stmt_Decl;
   function Named_Stmt_Decl_F_Name
     (Node : Bare_Named_Stmt_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Named_Stmt_Decl_F_Name;
   end;
--# property-start NamedStmtDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9644'
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Names
  (Node : Bare_Named_Stmt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Named_Stmt_Decl  := Bare_Named_Stmt_Decl (Node);
     Ent : Internal_Entity_Named_Stmt_Decl :=
       Internal_Entity_Named_Stmt_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1003 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1003 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1003;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6431 '<ArrayLiteral at nodes.lkt:9644:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9644'
--# expr-start 6430 '<FieldAccess for name at nodes.lkt:9644:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9644'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Named_Stmt_Decl_F_Name, Info => Ent.Info);
--# expr-done 6430
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6431
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1003;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1003;
            raise;
      end;
   return Property_Result;
end Named_Stmt_Decl_P_Defining_Names;
--# end
--# property-start NamedStmtDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9647'
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Env
  (Node : Bare_Named_Stmt_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Named_Stmt_Decl  := Bare_Named_Stmt_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1004 with Inline_Always;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld_1 : Bare_Composite_Stmt;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1004 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1004;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6435 '<FieldAccess for children_env at nodes.lkt:9648:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9648'
--# expr-start 6434 '<FieldAccess for stmt at nodes.lkt:9648:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9648'
--# expr-start 6433 '<Cast to NamedStmt at nodes.lkt:9648:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9648'
--# expr-start 6432 '<parent at nodes.lkt:9648:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9648'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6432
   if Fld = null
      or else Fld.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6433
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Cast_Result.Named_Stmt_F_Stmt;
--# expr-done 6434
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 6435
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1004;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1004;
            raise;
      end;
   return Property_Result;
end Named_Stmt_Decl_P_Defining_Env;
--# end
   --
   --  Primitives for Bare_Number_Decl
   --
      procedure Number_Decl_Pre_Env_Actions
        (Self            : Bare_Number_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_195 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9680:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Number_Decl
        (Self : Bare_Number_Decl
         ; Number_Decl_F_Ids : Bare_Defining_Name_List
         ; Number_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Number_Decl_F_Ids := Number_Decl_F_Ids;
            Self.Number_Decl_F_Expr := Number_Decl_F_Expr;
      end Initialize_Fields_For_Number_Decl;
   function Number_Decl_F_Ids
     (Node : Bare_Number_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Number_Decl_F_Ids;
   end;
   function Number_Decl_F_Expr
     (Node : Bare_Number_Decl) return Bare_Expr
   is
   begin
         return Node.Number_Decl_F_Expr;
   end;
--# property-start NumberDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9657'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Defining_Names
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1005 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Defining_Name;
Bare_Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1005 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1005;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6437 '<Map at nodes.lkt:9658:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9658'
   --# expr-start 6436 '<FieldAccess for ids at nodes.lkt:9658:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9658'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Number_Decl_F_Ids, Info => Ent.Info);
--# expr-done 6436
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind id Item_0
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_0;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6437
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1005;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1005;
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Defining_Names;
--# end
--# property-start NumberDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9662'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Expr_Type
  (Node : Bare_Number_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6443 '<If at nodes.lkt:9663:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
--# expr-start 6440 '<FieldAccess for is_int_type at nodes.lkt:9663:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
--# expr-start 6439 '<FieldAccess for expression_type at nodes.lkt:9663:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
--# expr-start 6438 '<FieldAccess for expr at nodes.lkt:9663:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 6438
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6439
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6440
if Fld_4 then
   --# expr-start 6441 '<FieldAccess for universal_int_type at nodes.lkt:9663:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 6441
   If_Result := Fld;
else
   --# expr-start 6442 '<FieldAccess for universal_real_type at nodes.lkt:9663:90>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9663'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 6442
   If_Result := Fld_1;
end if;
--# expr-done 6443
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Expr_Type;
--# end
--# property-start NumberDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9665'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Entry_Point
  (Node : Bare_Number_Decl
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Xref_Entry_Point;
--# end
--# property-start NumberDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9668'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Static_Decl
  (Node : Bare_Number_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Is_Static_Decl;
--# end
--# property-start NumberDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9671'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Equation
  (Node : Bare_Number_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
     Ent : Internal_Entity_Number_Decl :=
       Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1009 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1009 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1009;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6458 '<And at nodes.lkt:9671:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9671'
--# expr-start 6455 '<And at nodes.lkt:9672:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9672'
--# expr-start 6452 '<Or at nodes.lkt:9673:13>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
--# expr-start 6447 '<Bind at nodes.lkt:9673:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
--# expr-start 6445 '<FieldAccess for expected_type_var at nodes.lkt:9673:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
--# expr-start 6444 '<FieldAccess for expr at nodes.lkt:9673:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Number_Decl_F_Expr;
--# expr-done 6444
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 6445
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 6446 '<FieldAccess for universal_int_type at nodes.lkt:9673:48>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 6446
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9673:13") else null)); 
--# expr-done 6447
--# expr-start 6451 '<Bind at nodes.lkt:9673:78>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
--# expr-start 6449 '<FieldAccess for expected_type_var at nodes.lkt:9673:82>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
--# expr-start 6448 '<FieldAccess for expr at nodes.lkt:9673:82>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Number_Decl_F_Expr;
--# expr-done 6448
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 6449
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 6450 '<FieldAccess for universal_real_type at nodes.lkt:9673:113>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9673'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 6450
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9673:78") else null)); 
--# expr-done 6451
Or_Pred := Create_Or (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9673:13") else null)); 
--# expr-done 6452
--# expr-start 6454 '<FieldAccess for sub_equation at nodes.lkt:9674:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9674'
--# expr-start 6453 '<FieldAccess for expr at nodes.lkt:9674:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9674'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 6453
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 6454
And_Pred := Create_And (Or_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9672:9") else null)); 
--# expr-done 6455
--# expr-start 6457 '<FieldAccess for matches_expected_type at nodes.lkt:9675:11>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9675'
--# expr-start 6456 '<FieldAccess for expr at nodes.lkt:9675:11>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9675'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 6456
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 6457
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9671:37") else null)); 
--# expr-done 6458
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1009;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1009;
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Xref_Equation;
--# end
--# property-start NumberDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9677'
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Constant_Object
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Number_Decl_P_Is_Constant_Object;
--# end
--# property-start NumberDecl._env_mappings_195 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9680'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_195
  (Node : Bare_Number_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Number_Decl  := Bare_Number_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1859 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Number_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1859 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1859;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6460 '<FieldAccess for env_mappings at nodes.lkt:9680:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9680'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6459 '<FieldAccess for ids at nodes.lkt:9680:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9680'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Number_Decl_F_Ids;
--# expr-done 6459
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 6460
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1859;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1859;
            raise;
      end;
   return Property_Result;
end Env_Mappings_195;
--# end
   --
   --  Primitives for Bare_Object_Decl
   --
      procedure Object_Decl_Pre_Env_Actions
        (Self            : Bare_Object_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_196 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9775:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Object_Decl
        (Self : Bare_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Object_Decl_F_Ids := Object_Decl_F_Ids;
            Self.Object_Decl_F_Has_Aliased := Object_Decl_F_Has_Aliased;
            Self.Object_Decl_F_Has_Constant := Object_Decl_F_Has_Constant;
            Self.Object_Decl_F_Mode := Object_Decl_F_Mode;
            Self.Object_Decl_F_Type_Expr := Object_Decl_F_Type_Expr;
            Self.Object_Decl_F_Default_Expr := Object_Decl_F_Default_Expr;
            Self.Object_Decl_F_Renaming_Clause := Object_Decl_F_Renaming_Clause;
            Self.Object_Decl_F_Aspects := Object_Decl_F_Aspects;
      end Initialize_Fields_For_Object_Decl;
   function Object_Decl_F_Ids
     (Node : Bare_Object_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Object_Decl_F_Ids;
   end;
   function Object_Decl_F_Has_Aliased
     (Node : Bare_Object_Decl) return Bare_Aliased_Node
   is
   begin
         return Node.Object_Decl_F_Has_Aliased;
   end;
   function Object_Decl_F_Has_Constant
     (Node : Bare_Object_Decl) return Bare_Constant_Node
   is
   begin
         return Node.Object_Decl_F_Has_Constant;
   end;
   function Object_Decl_F_Mode
     (Node : Bare_Object_Decl) return Bare_Mode
   is
   begin
         return Node.Object_Decl_F_Mode;
   end;
   function Object_Decl_F_Type_Expr
     (Node : Bare_Object_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Object_Decl_F_Type_Expr;
   end;
   function Object_Decl_F_Default_Expr
     (Node : Bare_Object_Decl) return Bare_Expr
   is
   begin
         return Node.Object_Decl_F_Default_Expr;
   end;
   function Object_Decl_F_Renaming_Clause
     (Node : Bare_Object_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Object_Decl_F_Renaming_Clause;
   end;
--# property-start ObjectDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9697'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Names
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1011 with Inline_Always;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Defining_Name;
Bare_Item_0 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1011 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1011;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6462 '<Map at nodes.lkt:9698:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9698'
   --# expr-start 6461 '<FieldAccess for ids at nodes.lkt:9698:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9698'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Object_Decl_F_Ids, Info => Ent.Info);
--# expr-done 6461
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind id Item_0
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_0;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6462
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1011;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1011;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Defining_Names;
--# end
--# property-start ObjectDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9701'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Env
  (Node : Bare_Object_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1012 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1012 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1012;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6464 '<FieldAccess for defining_env at nodes.lkt:9701:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9701'
--# expr-start 6463 '<FieldAccess for type_expr at nodes.lkt:9701:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9701'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 6463
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6464
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1012;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1012;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Defining_Env;
--# end
--# property-start ObjectDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9703'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Type_Expression
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6465 '<FieldAccess for type_expr at nodes.lkt:9703:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9703'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 6465
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Type_Expression;
--# end
--# property-start ObjectDecl.is_constant_object 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9705'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Constant_Object
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Constant_Node;
Var_Expr_1 : Internal_Entity_Mode;
Var_Expr_183 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Constant_Node;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_2 : Internal_Entity_Mode;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
Fld_7 : Internal_Entity_Type_Expr;
Fld_8 : Internal_Entity_Base_Type_Decl;
Is_A_2 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6466 '<FieldAccess for has_constant at nodes.lkt:9705:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9705'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constant_Node (Node => Ent.Node.Object_Decl_F_Has_Constant, Info => Ent.Info);
--# expr-done 6466
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Constant_Node then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 6467 '<FieldAccess for as_bool at nodes.lkt:9705:66>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9705'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Var_Expr_0.Node);
--# end
--# expr-done 6467
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := True;
else
--# expr-start 6468 '<FieldAccess for mode at nodes.lkt:9707:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9707'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Mode (Node => Ent.Node.Object_Decl_F_Mode, Info => Ent.Info);
--# expr-done 6468
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Mode then
   --# scope-start
   --# bind m Var_Expr_1
--# expr-start 6470 '<IsA GenericFormalObjDecl at nodes.lkt:9708:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9708'
--# expr-start 6469 '<parent at nodes.lkt:9708:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9708'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 6469
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 6470
if Is_A then
   --# expr-start 6471 '<IsA Mode.In, Mode.Default at nodes.lkt:9708:60>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9708'
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Mode_In_Range | Ada_Mode_Default_Range; 
--# expr-done 6471
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 6472 '<FieldAccess for renaming_clause at nodes.lkt:9712:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9712'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 6472
Var_Expr_183 := Fld_4;
if Var_Expr_183 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 6474 '<FieldAccess for is_constant at nodes.lkt:9712:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9712'
--# expr-start 6473 '<FieldAccess for renamed_object at nodes.lkt:9712:9>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9712'
      if Var_Expr_183.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_183.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_183.Info);
--# expr-done 6473
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 6474
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 6477 '<IsA ProtectedTypeDecl at nodes.lkt:9715:9>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9715'
--# expr-start 6476 '<FieldAccess for designated_type_decl at nodes.lkt:9715:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9715'
--# expr-start 6475 '<FieldAccess for type_expr at nodes.lkt:9715:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9715'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 6475
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_8 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6476
Is_A_2 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Protected_Type_Decl_Range; 
--# expr-done 6477
   If_Result_3 := Is_A_2;
end if;
         Property_Result := If_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Is_Constant_Object;
--# end
--# property-start ObjectDecl.is_static_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9719'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Static_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_184 : Internal_Entity_Renaming_Clause;
Fld : Bare_Constant_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6479 '<FieldAccess for as_bool at nodes.lkt:9720:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9720'
--# expr-start 6478 '<FieldAccess for has_constant at nodes.lkt:9720:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9720'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Object_Decl_F_Has_Constant;
--# expr-done 6478
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 6479
if Fld_1 then
--# expr-start 6480 '<FieldAccess for default_expr at nodes.lkt:9720:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9720'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 6480
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_0
--# expr-start 6481 '<FieldAccess for is_static_expr at nodes.lkt:9720:72>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9720'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_3 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 6481
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 6482 '<FieldAccess for renaming_clause at nodes.lkt:9721:10>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9721'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 6482
Var_Expr_184 := Fld_4;
if Var_Expr_184 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 6484 '<FieldAccess for is_static_expr at nodes.lkt:9721:10>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9721'
--# expr-start 6483 '<FieldAccess for renamed_object at nodes.lkt:9721:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9721'
      if Var_Expr_184.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_184.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_184.Info);
--# expr-done 6483
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_6 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 6484
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Is_Static_Decl;
--# end
--# property-start ObjectDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9724'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Equation
  (Node : Bare_Object_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1016 with Inline_Always;
            procedure Finalizer_Scope_2855 with Inline_Always;
            procedure Finalizer_Scope_2856 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_7 : Internal_Entity_Renaming_Clause;
Fld_8 : Internal_Entity_Name;
Fld_9 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Name;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1016 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1016;
            procedure Finalizer_Scope_2855 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2855;
            procedure Finalizer_Scope_2856 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2856;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6507 '<Let at nodes.lkt:9725:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9725'
--# scope-start
--# expr-start 6485 '<FieldAccess for expr_type at nodes.lkt:9725:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9725'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6485
Local_Typ := Fld; 
--# bind typ Local_Typ
--# expr-start 6506 '<And at nodes.lkt:9727:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9727'
--# expr-start 6495 '<And at nodes.lkt:9728:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9728'
--# expr-start 6487 '<FieldAccess for sub_equation at nodes.lkt:9728:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9728'
--# expr-start 6486 '<FieldAccess for type_expr at nodes.lkt:9728:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9728'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 6486
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 6487
--# expr-start 6488 '<FieldAccess for default_expr at nodes.lkt:9728:47>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9728'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 6488
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr_0
--# expr-start 6494 '<And at nodes.lkt:9729:25>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9729'
--# expr-start 6492 '<And at nodes.lkt:9730:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9730'
--# expr-start 6490 '<Bind at nodes.lkt:9730:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9730'
--# expr-start 6489 '<FieldAccess for expected_type_var at nodes.lkt:9730:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9730'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 6489
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9730:21") else null)); 
--# expr-done 6490
--# expr-start 6491 '<FieldAccess for sub_equation at nodes.lkt:9730:58>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9730'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 6491
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9730:21") else null)); 
--# expr-done 6492
--# expr-start 6493 '<FieldAccess for matches_expected_assign_type at nodes.lkt:9731:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9731'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 6493
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9729:25") else null)); 
--# expr-done 6494
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2855;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9731:70") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9728:13") else null)); 
--# expr-done 6495
--# expr-start 6496 '<FieldAccess for renaming_clause at nodes.lkt:9733:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9733'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 6496
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind rc Var_Expr_1
--# expr-start 6505 '<And at nodes.lkt:9734:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9734'
--# expr-start 6502 '<And at nodes.lkt:9735:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
--# expr-start 6499 '<Bind at nodes.lkt:9735:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
--# expr-start 6498 '<FieldAccess for expected_type_var at nodes.lkt:9735:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
--# expr-start 6497 '<FieldAccess for renamed_object at nodes.lkt:9735:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 6497
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8.Node);
--# expr-done 6498
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Cast_Expr_1 := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_9, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9735:17") else null)); 
--# expr-done 6499
--# expr-start 6501 '<FieldAccess for sub_equation at nodes.lkt:9735:69>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
--# expr-start 6500 '<FieldAccess for renamed_object at nodes.lkt:9735:69>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9735'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 6500
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 6501
And_Pred_3 := Create_And (Bind_Result_1, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9735:17") else null)); 
--# expr-done 6502
--# expr-start 6504 '<FieldAccess for matches_expected_assign_type at nodes.lkt:9736:19>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9736'
--# expr-start 6503 '<FieldAccess for renamed_object at nodes.lkt:9736:19>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9736'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 6503
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 6504
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9734:21") else null)); 
--# expr-done 6505
Scope_Result_1 := And_Pred_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2856;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9736:81") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_5 := Create_And (And_Pred_2, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9727:9") else null)); 
--# expr-done 6506
Let_Result := And_Pred_5; Inc_Ref (Let_Result);
--# end
--# expr-done 6507
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1016;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1016;
                     Finalizer_Scope_2855;
                     Finalizer_Scope_2856;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Xref_Equation;
--# end
--# property-start ObjectDecl.next_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9741'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Next_Part_For_Name
  (Node : Bare_Object_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_2857 with Inline_Always;
      Var_Expr_185 : Internal_Entity_Private_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Package_Decl;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Fld_5 : Bare_Constant_Node;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_2857 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2857;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6519 '<If at nodes.lkt:9742:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6508 '<FieldAccess for is_in_public_part at nodes.lkt:9742:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_public_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6508
if Fld_4 then
   --# expr-start 6510 '<FieldAccess for as_bool at nodes.lkt:9742:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6509 '<FieldAccess for has_constant at nodes.lkt:9742:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Object_Decl_F_Has_Constant;
--# expr-done 6509
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_6 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_5);
--# end
--# expr-done 6510
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6518 '<Cast to BasicDecl at nodes.lkt:9742:74>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6515 '<FieldAccess for private_part at nodes.lkt:9742:74>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6514 '<Cast to BasePackageDecl at nodes.lkt:9742:74>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6513 '<as_entity at nodes.lkt:9742:74>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6512 '<parent at nodes.lkt:9742:74>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6511 '<FieldAccess for declarative_scope at nodes.lkt:9742:74>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 6511
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 6512
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 6513
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 6514
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Cast_Result.Node.Base_Package_Decl_F_Private_Part, Info => Cast_Result.Info);
--# expr-done 6515
Var_Expr_185 := Fld_2;
if Var_Expr_185 /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 6517 '<EnvGet at nodes.lkt:9742:74>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
--# expr-start 6516 '<FieldAccess for children_env at nodes.lkt:9742:74>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9742'
      if Var_Expr_185.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr_185.Node, E_Info => Var_Expr_185.Info);
--# expr-done 6516
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 6517
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_2857;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 6518
   If_Result_2 := Cast_Result_1;
else
   If_Result_2 := No_Entity_Basic_Decl;
end if;
--# expr-done 6519
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2857;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Next_Part_For_Name;
--# end
--# property-start ObjectDecl.previous_part_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9747'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Previous_Part_For_Name
  (Node : Bare_Object_Decl
      ; Sym : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sym Sym
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1018 with Inline_Always;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Package_Decl;
Fld_2 : Internal_Entity_Public_Part;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Fld_5 : Bare_Constant_Node;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1018 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1018;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6531 '<If at nodes.lkt:9748:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6520 '<FieldAccess for is_in_private_part at nodes.lkt:9748:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6520
if Fld_4 then
   --# expr-start 6522 '<FieldAccess for as_bool at nodes.lkt:9748:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6521 '<FieldAccess for has_constant at nodes.lkt:9748:42>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Object_Decl_F_Has_Constant;
--# expr-done 6521
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_6 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_5);
--# end
--# expr-done 6522
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6530 '<Cast to BasicDecl at nodes.lkt:9748:75>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6529 '<EnvGet at nodes.lkt:9748:75>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6528 '<FieldAccess for children_env at nodes.lkt:9748:75>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6527 '<FieldAccess for public_part at nodes.lkt:9748:75>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6526 '<Cast to BasePackageDecl at nodes.lkt:9748:75>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6525 '<as_entity at nodes.lkt:9748:75>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6524 '<parent at nodes.lkt:9748:75>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
--# expr-start 6523 '<FieldAccess for declarative_scope at nodes.lkt:9748:75>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 6523
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 6524
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 6525
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 6526
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Public_Part (Node => Cast_Result.Node.Base_Package_Decl_F_Public_Part, Info => Cast_Result.Info);
--# expr-done 6527
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 6528
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 6529
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 6530
   If_Result_2 := Cast_Result_1;
else
   If_Result_2 := No_Entity_Basic_Decl;
end if;
--# expr-done 6531
         Property_Result := If_Result_2;
   --# end
      Finalizer_Scope_1018;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1018;
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Previous_Part_For_Name;
--# end
--# property-start ObjectDecl.private_part_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9755'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Private_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6534 '<FieldAccess for next_part_for_name at nodes.lkt:9755:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9755'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6533 '<FieldAccess for name_symbol at nodes.lkt:9756:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9756'
--# expr-start 6532 '<FieldAccess for defining_name_or_raise at nodes.lkt:9756:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9756'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6532
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 6533
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Ent.Node, Sym => Fld_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 6534
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Private_Part_Decl;
--# end
--# property-start ObjectDecl.public_part_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9762'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Public_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6537 '<FieldAccess for previous_part_for_name at nodes.lkt:9762:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9762'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6536 '<FieldAccess for name_symbol at nodes.lkt:9763:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9763'
--# expr-start 6535 '<FieldAccess for defining_name_or_raise at nodes.lkt:9763:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9763'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6535
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 6536
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Ent.Node, Sym => Fld_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 6537
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Public_Part_Decl;
--# end
--# property-start ObjectDecl.next_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9767'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Next_Part_For_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6538 '<FieldAccess for private_part_decl at nodes.lkt:9767:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9767'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ObjectDecl.private_part_decl
Fld := Libadalang.Implementation.Object_Decl_P_Private_Part_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6538
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ObjectDecl.previous_part_for_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9770'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
     Ent : Internal_Entity_Object_Decl :=
       Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6539 '<FieldAccess for public_part_decl at nodes.lkt:9770:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9770'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ObjectDecl.public_part_decl
Fld := Libadalang.Implementation.Object_Decl_P_Public_Part_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6539
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start ObjectDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9772'
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Entry_Point
  (Node : Bare_Object_Decl
  )
   return Boolean
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Object_Decl_P_Xref_Entry_Point;
--# end
--# property-start ObjectDecl._env_mappings_196 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9775'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_196
  (Node : Bare_Object_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Object_Decl  := Bare_Object_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1860 with Inline_Always;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Object_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1860 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1860;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6541 '<FieldAccess for env_mappings at nodes.lkt:9775:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9775'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6540 '<FieldAccess for ids at nodes.lkt:9775:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9775'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Object_Decl_F_Ids;
--# expr-done 6540
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 6541
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1860;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1860;
            raise;
      end;
   return Property_Result;
end Env_Mappings_196;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt_Object_Decl
   --
      procedure Initialize_Fields_For_Extended_Return_Stmt_Object_Decl
        (Self : Bare_Extended_Return_Stmt_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_Extended_Return_Stmt_Object_Decl;
   --
   --  Primitives for Bare_No_Type_Object_Renaming_Decl
   --
      procedure Initialize_Fields_For_No_Type_Object_Renaming_Decl
        (Self : Bare_No_Type_Object_Renaming_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_No_Type_Object_Renaming_Decl;
--# property-start NoTypeObjectRenamingDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9790'
pragma Warnings (Off, "is not referenced");
 function No_Type_Object_Renaming_Decl_P_Expr_Type
  (Node : Bare_No_Type_Object_Renaming_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_No_Type_Object_Renaming_Decl  := Bare_No_Type_Object_Renaming_Decl (Node);
     Ent : Internal_Entity_No_Type_Object_Renaming_Decl :=
       Internal_Entity_No_Type_Object_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6544 '<FieldAccess for expression_type at nodes.lkt:9791:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9791'
--# expr-start 6543 '<FieldAccess for renamed_object at nodes.lkt:9791:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9791'
--# expr-start 6542 '<FieldAccess for renaming_clause at nodes.lkt:9791:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9791'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 6542
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 6543
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6544
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end No_Type_Object_Renaming_Decl_P_Expr_Type;
--# end
--# property-start NoTypeObjectRenamingDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9794'
pragma Warnings (Off, "is not referenced");
 function No_Type_Object_Renaming_Decl_P_Defining_Env
  (Node : Bare_No_Type_Object_Renaming_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_No_Type_Object_Renaming_Decl  := Bare_No_Type_Object_Renaming_Decl (Node);
     Ent : Internal_Entity_No_Type_Object_Renaming_Decl :=
       Internal_Entity_No_Type_Object_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1025 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1025 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1025;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6546 '<FieldAccess for defining_env at nodes.lkt:9794:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9794'
--# expr-start 6545 '<FieldAccess for expr_type at nodes.lkt:9794:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9794'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6545
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6546
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1025;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1025;
            raise;
      end;
   return Property_Result;
end No_Type_Object_Renaming_Decl_P_Defining_Env;
--# end
--# property-start NoTypeObjectRenamingDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9797'
pragma Warnings (Off, "is not referenced");
 function No_Type_Object_Renaming_Decl_P_Xref_Equation
  (Node : Bare_No_Type_Object_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_No_Type_Object_Renaming_Decl  := Bare_No_Type_Object_Renaming_Decl (Node);
     Ent : Internal_Entity_No_Type_Object_Renaming_Decl :=
       Internal_Entity_No_Type_Object_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1026 with Inline_Always;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
            procedure Finalizer_Scope_1026 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1026;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6549 '<FieldAccess for sub_equation at nodes.lkt:9798:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9798'
--# expr-start 6548 '<FieldAccess for renamed_object at nodes.lkt:9798:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9798'
--# expr-start 6547 '<FieldAccess for renaming_clause at nodes.lkt:9798:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 6547
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 6548
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 6549
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1026;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1026;
            raise;
      end;
   return Property_Result;
end No_Type_Object_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Package_Renaming_Decl
   --
      procedure Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_197 (Node => 
Self);
      end;
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_198 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:9858:9");
      end;
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_199 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9859:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_200 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      declare
         Dummy : Bare_Compilation_Unit_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_201 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_203 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_202 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_205 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_204 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Renaming_Decl
        (Self : Bare_Package_Renaming_Decl
         ; Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Package_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Renaming_Decl_F_Name := Package_Renaming_Decl_F_Name;
            Self.Package_Renaming_Decl_F_Renames := Package_Renaming_Decl_F_Renames;
            Self.Package_Renaming_Decl_F_Aspects := Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Package_Renaming_Decl;
   function Package_Renaming_Decl_F_Name
     (Node : Bare_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Package_Renaming_Decl_F_Name;
   end;
   function Package_Renaming_Decl_F_Renames
     (Node : Bare_Package_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Package_Renaming_Decl_F_Renames;
   end;
--# property-start PackageRenamingDecl.renamed_package 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9809'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1027 with Inline_Always;
      Local_Node_Env : Lexical_Env := Empty_Env;
Ignored : Internal_Entity_Array_Access;
Fld : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Renaming_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1027 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Node_Env);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Ignored);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1027;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6560 '<Let at nodes.lkt:9819:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9819'
--# scope-start
--# expr-start 6550 '<FieldAccess for node_env at nodes.lkt:9819:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9819'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6550
Local_Node_Env := Fld; Inc_Ref (Local_Node_Env);
--# bind node_env Local_Node_Env
--# expr-start 6559 '<Let at nodes.lkt:9820:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9820'
--# scope-start
--# expr-start 6552 '<EnvGet at nodes.lkt:9820:17>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9820'
--# expr-start 6551 '<Symbol __dummy at nodes.lkt:9821:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9821'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dummy); 
--# expr-done 6551
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Local_Node_Env, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 6552
Ignored := Env_Get_Result; Inc_Ref (Ignored);
--# bind _ Ignored
--# expr-start 6558 '<bind at nodes.lkt:9827:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9827'
Bound_Env := Local_Node_Env; Inc_Ref (Bound_Env);
--# expr-start 6557 '<Cast to BasicDecl at nodes.lkt:9829:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9829'
--# expr-start 6556 '<at at nodes.lkt:9829:13>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9829'
--# expr-start 6555 '<FieldAccess for env_elements at nodes.lkt:9829:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9829'
--# expr-start 6554 '<FieldAccess for renamed_object at nodes.lkt:9829:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9829'
--# expr-start 6553 '<FieldAccess for renames at nodes.lkt:9829:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9829'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 6553
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Fld_1.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_1.Info);
--# expr-done 6554
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_2.Node, Env => Bound_Env, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 6555
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 6556
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6557
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 6558
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6559
Let_Result_1 := Let_Result; 
--# end
--# expr-done 6560
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_1027;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1027;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.final_renamed_package 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9836'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Final_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Local_Pkg : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6564 '<Let at nodes.lkt:9837:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9837'
--# scope-start
--# expr-start 6561 '<FieldAccess for renamed_package at nodes.lkt:9837:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9837'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6561
Local_Pkg := Fld; 
--# bind pkg Local_Pkg
--# expr-start 6562 '<Cast to PackageRenamingDecl at nodes.lkt:9839:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9839'
Cast_Expr := Local_Pkg; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 6562
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 6563 '<FieldAccess for final_renamed_package at nodes.lkt:9840:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9840'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld_1 := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 6563
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Local_Pkg;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 6564
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Final_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9844'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1029 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1029 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1029;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6566 '<ArrayLiteral at nodes.lkt:9844:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9844'
--# expr-start 6565 '<FieldAccess for name at nodes.lkt:9844:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9844'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 6565
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6566
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1029;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1029;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start PackageRenamingDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9847'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1030 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1030 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1030;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6568 '<FieldAccess for defining_env at nodes.lkt:9848:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9848'
--# expr-start 6567 '<FieldAccess for renamed_package at nodes.lkt:9848:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9848'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6567
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6568
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1030;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1030;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start PackageRenamingDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9850'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start PackageRenamingDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9853'
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Package_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1032 with Inline_Always;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
            procedure Finalizer_Scope_1032 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1032;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6571 '<FieldAccess for xref_no_overloading at nodes.lkt:9854:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9854'
--# expr-start 6570 '<FieldAccess for renamed_object at nodes.lkt:9854:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9854'
--# expr-start 6569 '<FieldAccess for renames at nodes.lkt:9854:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9854'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 6569
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 6570
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 6571
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1032;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1032;
            raise;
      end;
   return Property_Result;
end Package_Renaming_Decl_P_Xref_Equation;
--# end
--# property-start PackageRenamingDecl._env_do_197 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9857'
pragma Warnings (Off, "is not referenced");
 function Env_Do_197
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6572 '<FieldAccess for env_hook at nodes.lkt:9857:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9857'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 6572
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Do_197;
--# end
--# property-start PackageRenamingDecl._env_init_198 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9858'
pragma Warnings (Off, "is not referenced");
 function Env_Init_198
  (Node : Bare_Package_Renaming_Decl
  )
   return Internal_Designated_Env
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1862 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1862 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1862;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6573 '<FieldAccess for child_decl_initial_env at nodes.lkt:9858:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9858'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.child_decl_initial_env
Fld := Libadalang.Implementation.Basic_Decl_P_Child_Decl_Initial_Env (Node => Self);
--# end
--# expr-done 6573
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1862;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1862;
            raise;
      end;
   return Property_Result;
end Env_Init_198;
--# end
--# property-start PackageRenamingDecl._env_mappings_199 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9859'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_199
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
     Ent : Internal_Entity_Package_Renaming_Decl :=
       Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1863 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1863 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1863;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6576 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6574 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6574
--# expr-start 6575 '<FieldAccess for name_symbol at nodes.lkt:9859:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9859'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 6575
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6576
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1863;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1863;
            raise;
      end;
   return Property_Result;
end Env_Mappings_199;
--# end
--# property-start PackageRenamingDecl._env_trans_parent_200 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_200
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_200;
--# end
--# property-start PackageRenamingDecl._env_do_201 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9861'
pragma Warnings (Off, "is not referenced");
 function Env_Do_201
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Compilation_Unit_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1865 with Inline_Always;
      Fld : Bare_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1865 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1865;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6577 '<FieldAccess for populate_dependent_units at nodes.lkt:9861:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9861'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 6577
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1865;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1865;
            raise;
      end;
   return Property_Result;
end Env_Do_201;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_202 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9863'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_202
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1866 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1866 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1866;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6578 '<FieldAccess for top_level_use_package_clauses at nodes.lkt:9863:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9863'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 6578
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1866;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1866;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_202;
--# end
--# property-start PackageRenamingDecl._ref_cond_203 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9863'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_203
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6580 '<IsA LibraryItem, Subunit at nodes.lkt:9863:94>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9863'
--# expr-start 6579 '<parent at nodes.lkt:9863:94>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9863'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6579
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6580
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_203;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_204 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9866'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_204
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1868 with Inline_Always;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1868 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1868;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6581 '<FieldAccess for top_level_use_type_clauses at nodes.lkt:9866:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9866'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 6581
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1868;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1868;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_204;
--# end
--# property-start PackageRenamingDecl._ref_cond_205 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9866'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_205
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl  := Bare_Package_Renaming_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6583 '<IsA LibraryItem, Subunit at nodes.lkt:9866:84>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9866'
--# expr-start 6582 '<parent at nodes.lkt:9866:84>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9866'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 6582
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 6583
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_205;
--# end
   --
   --  Primitives for Bare_Single_Protected_Decl
   --
      procedure Single_Protected_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Protected_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_206 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9905:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_207 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_208 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Protected_Decl
        (Self : Bare_Single_Protected_Decl
         ; Single_Protected_Decl_F_Name : Bare_Defining_Name
         ; Single_Protected_Decl_F_Aspects : Bare_Aspect_Spec
         ; Single_Protected_Decl_F_Interfaces : Bare_Parent_List
         ; Single_Protected_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Self.Single_Protected_Decl_F_Name := Single_Protected_Decl_F_Name;
            Self.Single_Protected_Decl_F_Aspects := Single_Protected_Decl_F_Aspects;
            Self.Single_Protected_Decl_F_Interfaces := Single_Protected_Decl_F_Interfaces;
            Self.Single_Protected_Decl_F_Definition := Single_Protected_Decl_F_Definition;
      end Initialize_Fields_For_Single_Protected_Decl;
   function Single_Protected_Decl_F_Name
     (Node : Bare_Single_Protected_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Single_Protected_Decl_F_Name;
   end;
   function Single_Protected_Decl_F_Interfaces
     (Node : Bare_Single_Protected_Decl) return Bare_Parent_List
   is
   begin
         return Node.Single_Protected_Decl_F_Interfaces;
   end;
   function Single_Protected_Decl_F_Definition
     (Node : Bare_Single_Protected_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Single_Protected_Decl_F_Definition;
   end;
--# property-start SingleProtectedDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9878'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Names
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1033 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1033 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1033;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6585 '<ArrayLiteral at nodes.lkt:9878:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9878'
--# expr-start 6584 '<FieldAccess for name at nodes.lkt:9878:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9878'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Single_Protected_Decl_F_Name, Info => Ent.Info);
--# expr-done 6584
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld)); 
--# expr-done 6585
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1033;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1033;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Defining_Names;
--# end
--# property-start SingleProtectedDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9881'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Env
  (Node : Bare_Single_Protected_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1034 with Inline_Always;
            procedure Finalizer_Scope_2859 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Private_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Private_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1034 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1034;
            procedure Finalizer_Scope_2859 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2859;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6587 '<FieldAccess for private_part at nodes.lkt:9881:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9881'
--# expr-start 6586 '<FieldAccess for definition at nodes.lkt:9881:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9881'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Single_Protected_Decl_F_Definition, Info => Ent.Info);
--# expr-done 6586
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Private_Part (Node => Fld.Node.Protected_Def_F_Private_Part, Info => Fld.Info);
--# expr-done 6587
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Private_Part then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 6591 '<env_group at nodes.lkt:9886:9>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9886'
--# expr-start 6590 '<ArrayLiteral at nodes.lkt:9886:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9886'
--# expr-start 6588 '<FieldAccess for children_env at nodes.lkt:9886:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9886'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6588
--# expr-start 6589 '<FieldAccess for children_env at nodes.lkt:9886:31>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9886'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 6589
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 6590
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 6591
Scope_Result := Group_Env;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2859;
   Result_Var := Scope_Result;
else
   --# expr-start 6592 '<FieldAccess for children_env at nodes.lkt:9886:75>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9886'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6592
   Result_Var := Fld_4;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1034;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1034;
                     Finalizer_Scope_2859;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Defining_Env;
--# end
--# property-start SingleProtectedDecl.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9889'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Entry_Point
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Xref_Entry_Point;
--# end
--# property-start SingleProtectedDecl.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9892'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Equation
  (Node : Bare_Single_Protected_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1036 with Inline_Always;
            procedure Finalizer_Scope_2860 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1036 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1036;
            procedure Finalizer_Scope_2860 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2860;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6596 '<LogicAll at nodes.lkt:9893:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9893'
--# expr-start 6595 '<Map at nodes.lkt:9893:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9893'
   --# expr-start 6593 '<FieldAccess for interfaces at nodes.lkt:9893:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9893'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Single_Protected_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 6593
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind ifc Item_0
      --# expr-start 6594 '<FieldAccess for xref_equation at nodes.lkt:9893:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9893'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 6594
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2860;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6595
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9893:9") else null)); 
--# expr-done 6596
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1036;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1036;
                     Finalizer_Scope_2860;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Xref_Equation;
--# end
--# property-start SingleProtectedDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9895'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Env_Names
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1037 with Inline_Always;
            procedure Finalizer_Scope_2861 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1037 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1037;
            procedure Finalizer_Scope_2861 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2861;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6597 '<FieldAccess for top_level_env_name at nodes.lkt:9896:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9896'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 6597
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 6599 '<ArrayLiteral at nodes.lkt:9896:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9896'
--# expr-start 6598 '<to_symbol at nodes.lkt:9896:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9896'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 6598
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 6599
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2861;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1037;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1037;
                     Finalizer_Scope_2861;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Env_Names;
--# end
--# property-start SingleProtectedDecl.declarative_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9898'
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Declarative_Parts
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1038 with Inline_Always;
            procedure Finalizer_Scope_2862 with Inline_Always;
      Local_Pdef : Internal_Entity_Protected_Def;
Var_Expr_0 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Protected_Def;
Fld_1 : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_2 : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Declarative_Part;
Array_Lit_1 : Internal_Entity_Declarative_Part_Array_Access;
Scope_Result : Internal_Entity_Declarative_Part_Array_Access;
Result_Var : Internal_Entity_Declarative_Part_Array_Access;
Concat_Result : Internal_Entity_Declarative_Part_Array_Access;
Let_Result : Internal_Entity_Declarative_Part_Array_Access;
            procedure Finalizer_Scope_1038 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1038;
            procedure Finalizer_Scope_2862 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2862;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6607 '<Let at nodes.lkt:9899:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9899'
--# scope-start
--# expr-start 6600 '<FieldAccess for definition at nodes.lkt:9899:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9899'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Protected_Def (Node => Ent.Node.Single_Protected_Decl_F_Definition, Info => Ent.Info);
--# expr-done 6600
Local_Pdef := Fld; 
--# bind pdef Local_Pdef
--# expr-start 6603 '<ArrayLiteral at nodes.lkt:9901:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
--# expr-start 6602 '<Cast to DeclarativePart at nodes.lkt:9901:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
--# expr-start 6601 '<FieldAccess for public_part at nodes.lkt:9901:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
      if Local_Pdef.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Local_Pdef.Node.Protected_Def_F_Public_Part, Info => Local_Pdef.Info);
--# expr-done 6601
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 6602
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result)); 
--# expr-done 6603
--# expr-start 6605 '<Cast to DeclarativePart at nodes.lkt:9901:50>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
--# expr-start 6604 '<FieldAccess for private_part at nodes.lkt:9901:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
      if Local_Pdef.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Local_Pdef.Node.Protected_Def_F_Private_Part, Info => Local_Pdef.Info);
--# expr-done 6604
      Cast_Result_1 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 6605
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind v1 Var_Expr_0
--# expr-start 6606 '<ArrayLiteral at nodes.lkt:9901:99>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9901'
Array_Lit_1 := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Var_Expr_0)); 
--# expr-done 6606
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2862;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Declarative_Part_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Array_Lit, Result_Var); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6607
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1038;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1038;
                     Finalizer_Scope_2862;
            raise;
      end;
   return Property_Result;
end Single_Protected_Decl_P_Declarative_Parts;
--# end
--# property-start SingleProtectedDecl._env_mappings_206 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9905'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_206
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
     Ent : Internal_Entity_Single_Protected_Decl :=
       Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1870 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Single_Protected_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1870 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1870;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6610 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6608 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6608
--# expr-start 6609 '<FieldAccess for name_symbol at nodes.lkt:9905:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9905'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 6609
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6610
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1870;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1870;
            raise;
      end;
   return Property_Result;
end Env_Mappings_206;
--# end
--# property-start SingleProtectedDecl._env_trans_parent_207 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_207
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_207;
--# end
--# property-start SingleProtectedDecl._env_names_208 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9906'
pragma Warnings (Off, "is not referenced");
 function Env_Names_208
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl  := Bare_Single_Protected_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1872 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1872 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1872;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6611 '<FieldAccess for env_names at nodes.lkt:9906:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9906'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleProtectedDecl.env_names
Fld := Libadalang.Implementation.Single_Protected_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 6611
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1872;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1872;
            raise;
      end;
   return Property_Result;
end Env_Names_208;
--# end
   --
   --  Primitives for Bare_Single_Task_Decl
   --
      procedure Single_Task_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_209 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:9928:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_210 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_211 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Decl
        (Self : Bare_Single_Task_Decl
         ; Single_Task_Decl_F_Task_Type : Bare_Single_Task_Type_Decl
        ) is
      begin
            Self.Single_Task_Decl_F_Task_Type := Single_Task_Decl_F_Task_Type;
      end Initialize_Fields_For_Single_Task_Decl;
   function Single_Task_Decl_F_Task_Type
     (Node : Bare_Single_Task_Decl) return Bare_Single_Task_Type_Decl
   is
   begin
         return Node.Single_Task_Decl_F_Task_Type;
   end;
--# property-start SingleTaskDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9915'
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Names
  (Node : Bare_Single_Task_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1039 with Inline_Always;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1039 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1039;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6613 '<FieldAccess for defining_names at nodes.lkt:9916:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9916'
--# expr-start 6612 '<FieldAccess for task_type at nodes.lkt:9916:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9916'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 6612
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6613
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1039;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1039;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Defining_Names;
--# end
--# property-start SingleTaskDecl.expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9919'
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Expr_Type
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 6614 '<FieldAccess for task_type at nodes.lkt:9919:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9919'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 6614
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Expr_Type;
--# end
--# property-start SingleTaskDecl.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9922'
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Env
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
         := False
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
     Ent : Internal_Entity_Single_Task_Decl :=
       Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1041 with Inline_Always;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1041 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1041;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6616 '<FieldAccess for defining_env at nodes.lkt:9922:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9922'
--# expr-start 6615 '<FieldAccess for task_type at nodes.lkt:9922:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9922'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 6615
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 6616
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1041;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1041;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Defining_Env;
--# end
--# property-start SingleTaskDecl.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9924'
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Env_Names
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1042 with Inline_Always;
            procedure Finalizer_Scope_2863 with Inline_Always;
      Var_Expr_0 : String_Type;
Fld : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1042 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1042;
            procedure Finalizer_Scope_2863 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2863;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6617 '<FieldAccess for top_level_env_name at nodes.lkt:9925:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9925'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 6617
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= Empty_String then
   --# scope-start
   --# bind fqn Var_Expr_0
--# expr-start 6619 '<ArrayLiteral at nodes.lkt:9925:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9925'
--# expr-start 6618 '<to_symbol at nodes.lkt:9925:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9925'
Sym := String_To_Symbol (Self, Self.Unit.Context, Var_Expr_0); 
--# expr-done 6618
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 6619
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2863;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1042;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1042;
                     Finalizer_Scope_2863;
            raise;
      end;
   return Property_Result;
end Single_Task_Decl_P_Env_Names;
--# end
--# property-start SingleTaskDecl._env_mappings_209 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9928'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_209
  (Node : Bare_Single_Task_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1873 with Inline_Always;
      Fld : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Single_Task_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1873 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1873;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6622 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 6620 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 6620
--# expr-start 6621 '<FieldAccess for name_symbol at nodes.lkt:9928:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9928'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 6621
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 6622
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1873;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1873;
            raise;
      end;
   return Property_Result;
end Env_Mappings_209;
--# end
--# property-start SingleTaskDecl._env_trans_parent_210 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_210
  (Node : Bare_Single_Task_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_210;
--# end
--# property-start SingleTaskDecl._env_names_211 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9929'
pragma Warnings (Off, "is not referenced");
 function Env_Names_211
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl  := Bare_Single_Task_Decl (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1875 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1875 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1875;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6623 '<FieldAccess for env_names at nodes.lkt:9929:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9929'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTaskDecl.env_names
Fld := Libadalang.Implementation.Single_Task_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 6623
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1875;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1875;
            raise;
      end;
   return Property_Result;
end Env_Names_211;
--# end
   --
   --  Primitives for Bare_Synthetic_Object_Decl
   --
      procedure Initialize_Fields_For_Synthetic_Object_Decl
        (Self : Bare_Synthetic_Object_Decl
        ) is
      begin
      Self.Synthetic_Object_Decl_Name := No_Bare_Ada_Node;
      Self.Synthetic_Object_Decl_Type_Expr := No_Bare_Ada_Node;
      end Initialize_Fields_For_Synthetic_Object_Decl;
--# property-start SyntheticObjectDecl.type_expression 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9952'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Object_Decl_P_Type_Expression
  (Node : Bare_Synthetic_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Synthetic_Object_Decl  := Bare_Synthetic_Object_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6625 '<as_entity at nodes.lkt:9952:47>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9952'
--# expr-start 6624 '<FieldAccess for type_expr at nodes.lkt:9952:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9952'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Object_Decl_Type_Expr;
--# expr-done 6624
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 6625
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Object_Decl_P_Type_Expression;
--# end
--# property-start SyntheticObjectDecl.defining_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9954'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Object_Decl_P_Defining_Names
  (Node : Bare_Synthetic_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Synthetic_Object_Decl  := Bare_Synthetic_Object_Decl (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1044 with Inline_Always;
      Fld : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1044 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1044;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6628 '<ArrayLiteral at nodes.lkt:9954:57>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9954'
--# expr-start 6627 '<as_entity at nodes.lkt:9954:58>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9954'
--# expr-start 6626 '<FieldAccess for name at nodes.lkt:9954:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9954'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Object_Decl_Name;
--# expr-done 6626
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 6627
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => If_Result)); 
--# expr-done 6628
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1044;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1044;
            raise;
      end;
   return Property_Result;
end Synthetic_Object_Decl_P_Defining_Names;
--# end
--# property-start SyntheticObjectDecl.is_referred_by 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9971'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Object_Decl_P_Is_Referred_By
  (Node : Bare_Synthetic_Object_Decl
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Synthetic_Object_Decl  := Bare_Synthetic_Object_Decl (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_1045 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Is_Null : Boolean;
Fld : Bare_Type_Expr;
Cast_Result : Bare_Synthetic_Type_Expr;
Fld_1 : Bare_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Boolean;
If_Result : Boolean;
            procedure Finalizer_Scope_1045 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1045;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 6632 '<IsNull at nodes.lkt:9972:9>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9972'
--# expr-start 6631 '<Find at nodes.lkt:9972:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9972'
   --# expr-start 6629 '<FieldAccess for parents at nodes.lkt:9972:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9972'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Origin, True); 
--# expr-done 6629
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 6630 '<IsA AccessDef at nodes.lkt:9972:38>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9972'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Access_Def; 
--# expr-done 6630
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 6631
Is_Null := Find_Result = null; 
--# expr-done 6632
if Is_Null then
   --# expr-start 6638 '<FieldAccess for is_children_env at nodes.lkt:9972:66>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9972'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6636 '<FieldAccess for children_env at nodes.lkt:9973:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9973'
--# expr-start 6635 '<FieldAccess for target_type at nodes.lkt:9973:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9973'
--# expr-start 6634 '<Cast to SyntheticTypeExpr at nodes.lkt:9973:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9973'
--# expr-start 6633 '<FieldAccess for type_expr at nodes.lkt:9973:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9973'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Object_Decl_Type_Expr;
--# expr-done 6633
   if Fld = null
      or else Fld.Kind in Ada_Synthetic_Type_Expr_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6634
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Cast_Result.Synthetic_Type_Expr_F_Target_Type;
--# expr-done 6635
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 6636
--# expr-start 6637 '<FieldAccess for children_env at nodes.lkt:9973:78>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9973'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Origin);
--# expr-done 6637
--# property-call-start AdaNode.is_children_env
Fld_4 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Fld_2, Current_Env => Fld_3);
--# end
--# expr-done 6638
   If_Result := Fld_4;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1045;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1045;
            raise;
      end;
   return Property_Result;
end Synthetic_Object_Decl_P_Is_Referred_By;
--# end
   --
   --  Primitives for Bare_Case_Stmt_Alternative
   --
      procedure Initialize_Fields_For_Case_Stmt_Alternative
        (Self : Bare_Case_Stmt_Alternative
         ; Case_Stmt_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Stmt_Alternative_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Case_Stmt_Alternative_F_Choices := Case_Stmt_Alternative_F_Choices;
            Self.Case_Stmt_Alternative_F_Stmts := Case_Stmt_Alternative_F_Stmts;
      end Initialize_Fields_For_Case_Stmt_Alternative;
   function Case_Stmt_Alternative_F_Choices
     (Node : Bare_Case_Stmt_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Choices;
   end;
   function Case_Stmt_Alternative_F_Stmts
     (Node : Bare_Case_Stmt_Alternative) return Bare_Stmt_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Stmts;
   end;
--# property-start CaseStmtAlternative.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9982'
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Entry_Point
  (Node : Bare_Case_Stmt_Alternative
  )
   return Boolean
is
   Self : Bare_Case_Stmt_Alternative  := Bare_Case_Stmt_Alternative (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Case_Stmt_Alternative_P_Xref_Entry_Point;
--# end
--# property-start CaseStmtAlternative.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9985'
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Equation
  (Node : Bare_Case_Stmt_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt_Alternative  := Bare_Case_Stmt_Alternative (Node);
     Ent : Internal_Entity_Case_Stmt_Alternative :=
       Internal_Entity_Case_Stmt_Alternative'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1047 with Inline_Always;
            procedure Finalizer_Scope_2865 with Inline_Always;
            procedure Finalizer_Scope_2866 with Inline_Always;
            procedure Finalizer_Scope_2867 with Inline_Always;
            procedure Finalizer_Scope_2868 with Inline_Always;
      Local_Case_Stmt : Internal_Entity_Case_Stmt;
Local_Selected_Type : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Alternatives_List;
Fld_5 : Bare_Alternatives_List;
Fld_6 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_A : Boolean;
Cast_Expr_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_11 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_12 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_5 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_6 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_7 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result_4 : Logic_Equation;
Let_Result_5 : Logic_Equation;
            procedure Finalizer_Scope_1047 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_1047;
            procedure Finalizer_Scope_2865 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2865;
            procedure Finalizer_Scope_2866 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2866;
            procedure Finalizer_Scope_2867 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2867;
            procedure Finalizer_Scope_2868 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_2868;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6664 '<Let at nodes.lkt:9986:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9986'
--# scope-start
--# expr-start 6641 '<Cast to CaseStmt at nodes.lkt:9986:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9986'
--# expr-start 6640 '<parent at nodes.lkt:9986:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9986'
--# expr-start 6639 '<parent at nodes.lkt:9986:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9986'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6639
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6640
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6641
Local_Case_Stmt := Cast_Result; 
--# bind case_stmt Local_Case_Stmt
--# expr-start 6663 '<Let at nodes.lkt:9988:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9988'
--# scope-start
--# expr-start 6643 '<FieldAccess for expression_type at nodes.lkt:9988:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9988'
--# expr-start 6642 '<FieldAccess for expr at nodes.lkt:9988:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9988'
      if Local_Case_Stmt.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Local_Case_Stmt.Node.Case_Stmt_F_Expr, Info => Local_Case_Stmt.Info);
--# expr-done 6642
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6643
Local_Selected_Type := Fld_3; 
--# bind selected_type Local_Selected_Type
--# expr-start 6662 '<LogicAll at nodes.lkt:9990:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9990'
--# expr-start 6661 '<Map at nodes.lkt:9990:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9990'
   --# expr-start 6644 '<FieldAccess for choices at nodes.lkt:9990:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9990'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Case_Stmt_Alternative_F_Choices, Info => Ent.Info);
--# expr-done 6644
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_0); 
   If_Result_2 := As_Entity;
end if;
                  Item_0 := If_Result_2; 
   --# scope-start
               --# bind c Item_0
      --# expr-start 6660 '<Match at nodes.lkt:9991:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9991'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind e Match_0
--# expr-start 6658 '<If at nodes.lkt:9993:33>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6645 '<IsA Name at nodes.lkt:9993:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
Is_A := Match_0.Node /= null 
and then Match_0.Node.Kind in Ada_Name; 
--# expr-done 6645
if Is_A then
   --# expr-start 6649 '<Not at nodes.lkt:9993:50>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6648 '<IsNull at nodes.lkt:9993:54>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6647 '<FieldAccess for name_designated_type at nodes.lkt:9993:54>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6646 '<Cast to Name at nodes.lkt:9993:54>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
Cast_Expr_2 := Match_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 6646
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_11 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 6647
Is_Null := Fld_11.Node = null; 
--# expr-done 6648
Not_Val := not (Is_Null); 
--# expr-done 6649
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 6651 '<FieldAccess for xref_type_equation at nodes.lkt:9993:101>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6650 '<Cast to Name at nodes.lkt:9993:101>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 6650
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_7 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6651
   If_Result_1 := Fld_7;
else
   --# expr-start 6657 '<And at nodes.lkt:9993:138>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9993'
--# expr-start 6655 '<And at nodes.lkt:9994:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9994'
--# expr-start 6653 '<Bind at nodes.lkt:9994:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9994'
--# expr-start 6652 '<FieldAccess for expected_type_var at nodes.lkt:9994:25>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9994'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 6652
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr_1 := Local_Selected_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9994:21") else null)); 
--# expr-done 6653
--# expr-start 6654 '<FieldAccess for sub_equation at nodes.lkt:9994:67>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9994'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 6654
And_Pred := Create_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9994:21") else null)); 
--# expr-done 6655
--# expr-start 6656 '<FieldAccess for matches_expected_type at nodes.lkt:9995:23>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9995'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 6656
And_Pred_1 := Create_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9993:138") else null)); 
--# expr-done 6657
   If_Result_1 := And_Pred_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6658
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2866;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind t Match_1
--# expr-start 6659 '<FieldAccess for xref_equation at nodes.lkt:9998:46>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:9998'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_12 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 6659
Let_Result_1 := Fld_12; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2867;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9999:45") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2868;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_7; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 6660
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2865;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6661
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:9990:9") else null)); 
--# expr-done 6662
Let_Result_4 := Logic_Boolean_Op; Inc_Ref (Let_Result_4);
--# end
--# expr-done 6663
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 6664
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1047;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1047;
                     Finalizer_Scope_2865;
                     Finalizer_Scope_2866;
                     Finalizer_Scope_2867;
                     Finalizer_Scope_2868;
            raise;
      end;
   return Property_Result;
end Case_Stmt_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Compilation_Unit
   --
      procedure Initialize_Fields_For_Compilation_Unit
        (Self : Bare_Compilation_Unit
         ; Compilation_Unit_F_Prelude : Bare_Ada_Node_List
         ; Compilation_Unit_F_Body : Bare_Ada_Node
         ; Compilation_Unit_F_Pragmas : Bare_Pragma_Node_List
        ) is
      begin
            Self.Compilation_Unit_F_Prelude := Compilation_Unit_F_Prelude;
            Self.Compilation_Unit_F_Body := Compilation_Unit_F_Body;
            Self.Compilation_Unit_F_Pragmas := Compilation_Unit_F_Pragmas;
      Self.Compilation_Unit_No_Env := Empty_Env;
      end Initialize_Fields_For_Compilation_Unit;
   function Compilation_Unit_F_Prelude
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Prelude;
   end;
   function Compilation_Unit_F_Body
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node
   is
   begin
         return Node.Compilation_Unit_F_Body;
   end;
   function Compilation_Unit_F_Pragmas
     (Node : Bare_Compilation_Unit) return Bare_Pragma_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Pragmas;
   end;
--# property-start CompilationUnit.syntactic_fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10025'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1049 with Inline_Always;
            procedure Finalizer_Scope_2870 with Inline_Always;
            procedure Finalizer_Scope_2871 with Inline_Always;
            procedure Finalizer_Scope_2872 with Inline_Always;
      Match_0 : Internal_Entity_Library_Item;
Match_1 : Internal_Entity_Subunit;
Match_2 : Internal_Entity;
Var_Expr_186 : Internal_Entity_Defining_Name;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Symbol_Type_Array_Access;
Fld_6 : Internal_Entity_Body_Node;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Library_Item;
Let_Result : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Subunit;
Let_Result_1 : Symbol_Type_Array_Access;
Scope_Result_2 : Symbol_Type_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Symbol_Type_Array_Access;
Scope_Result_3 : Symbol_Type_Array_Access;
Match_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1049 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1049;
            procedure Finalizer_Scope_2870 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2870;
            procedure Finalizer_Scope_2871 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2871;
            procedure Finalizer_Scope_2872 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2872;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6675 '<Match at nodes.lkt:10025:59>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10025'
--# expr-start 6666 '<FieldAccess for body at nodes.lkt:10025:65>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10025'
--# expr-start 6665 '<as_bare_entity at nodes.lkt:10025:65>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10025'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 6665
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => If_Result.Node.Compilation_Unit_F_Body, Info => If_Result.Info);
--# expr-done 6666
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 6668 '<FieldAccess for defining_name at nodes.lkt:10026:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10026'
--# expr-start 6667 '<FieldAccess for item at nodes.lkt:10026:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10026'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Library_Item_F_Item, Info => Match_0.Info);
--# expr-done 6667
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6668
Var_Expr_186 := Fld_2;
if Var_Expr_186 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 6669 '<FieldAccess for as_symbol_array at nodes.lkt:10026:33>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10026'
      if Var_Expr_186.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Var_Expr_186.Node);
--# end
--# expr-done 6669
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2871;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2870;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 6671 '<FieldAccess for as_symbol_array at nodes.lkt:10027:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10027'
--# expr-start 6670 '<FieldAccess for name at nodes.lkt:10027:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10027'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Subunit_F_Name, Info => Match_1.Info);
--# expr-done 6670
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_4.Node);
--# end
--# expr-done 6671
--# expr-start 6674 '<FieldAccess for as_symbol_array at nodes.lkt:10027:57>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10027'
--# expr-start 6673 '<FieldAccess for defining_name at nodes.lkt:10027:57>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10027'
--# expr-start 6672 '<FieldAccess for body at nodes.lkt:10027:57>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10027'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Body_Node (Node => Match_1.Node.Subunit_F_Body, Info => Match_1.Info);
--# expr-done 6672
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 6673
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_7.Node);
--# end
--# expr-done 6674
Concat_Result := Concat (Fld_5, Fld_8); 
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2872;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 6675
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1049;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1049;
                     Finalizer_Scope_2870;
                     Finalizer_Scope_2871;
                     Finalizer_Scope_2872;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start CompilationUnit.unit_kind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10033'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Kind
  (Node : Bare_Compilation_Unit
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Match_0 : Bare_Library_Item;
Match_0_1 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Match_1_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Analysis_Unit_Kind;
Scope_Result : Analysis_Unit_Kind;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Analysis_Unit_Kind;
Scope_Result_1 : Analysis_Unit_Kind;
Match_Result : Analysis_Unit_Kind;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Library_Item;
Let_Result_2 : Analysis_Unit_Kind;
Scope_Result_2 : Analysis_Unit_Kind;
Cast_Result_3 : Bare_Subunit;
Let_Result_3 : Analysis_Unit_Kind;
Scope_Result_3 : Analysis_Unit_Kind;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Analysis_Unit_Kind;
Scope_Result_4 : Analysis_Unit_Kind;
Match_Result_1 : Analysis_Unit_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6679 '<Match at nodes.lkt:10033:41>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10033'
--# expr-start 6676 '<FieldAccess for body at nodes.lkt:10033:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 6676
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix_1;
Match_0 := Cast_Result_2; 
--# bind li Match_0
--# expr-start 6678 '<Match at nodes.lkt:10034:33>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10034'
--# expr-start 6677 '<FieldAccess for item at nodes.lkt:10034:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10034'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 6677
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Accept_Stmt_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind _ Match_0_1
Let_Result := Unit_Body; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Subp_Decl | Ada_Entry_Index_Spec .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := Unit_Specification; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 6678
Let_Result_2 := Match_Result; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_1_1 := Cast_Result_3; 
--# bind _ Match_1_1
Let_Result_3 := Unit_Body; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_4 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
end case;
--# expr-done 6679
         Property_Result := Match_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Unit_Kind;
--# end
--# property-start CompilationUnit.referenced_units_in 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10044'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Referenced_Units_In
  (Node : Bare_Compilation_Unit
      ; Name : Internal_Entity_Name
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1051 with Inline_Always;
            procedure Finalizer_Scope_2880 with Inline_Always;
            procedure Finalizer_Scope_2881 with Inline_Always;
      Local_Self_Refd : Bare_Compilation_Unit;
Var_Expr_0 : Internal_Entity_Dotted_Name;
Var_Expr_1 : Internal_Entity_Compilation_Unit;
Var_Expr_187 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Array_Lit : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1051 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1051;
            procedure Finalizer_Scope_2880 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2880;
            procedure Finalizer_Scope_2881 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2881;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6687 '<Let at nodes.lkt:10045:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10045'
--# scope-start
--# expr-start 6680 '<FieldAccess for referenced_decl at nodes.lkt:10045:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10045'
      if Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Name.Node, Imprecise_Fallback => False, E_Info => Name.Info);
--# end
--# expr-done 6680
Var_Expr_187 := Fld;
if Var_Expr_187 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 6681 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:10045:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10045'
      if Var_Expr_187.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr_187.Node);
--# end
--# expr-done 6681
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Local_Self_Refd := Result_Var; 
--# bind self_refd Local_Self_Refd
--# expr-start 6682 '<Cast to DottedName at nodes.lkt:10047:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10047'
Cast_Expr := Name; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 6682
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 6684 '<FieldAccess for referenced_units_in at nodes.lkt:10048:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10048'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6683 '<FieldAccess for prefix at nodes.lkt:10048:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10048'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 6683
--# property-call-start CompilationUnit.referenced_units_in
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Referenced_Units_In (Node => Self, Name => Fld_2);
--# end
--# expr-done 6684
Scope_Result_1 := Fld_3;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2880;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 6685 '<as_bare_entity at nodes.lkt:10049:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10049'
Is_Null := Local_Self_Refd = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Local_Self_Refd); 
   If_Result := As_Entity;
end if;
--# expr-done 6685
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Compilation_Unit then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 6686 '<ArrayLiteral at nodes.lkt:10049:49>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10049'
Array_Lit := Create_Internal_Entity_Compilation_Unit_Array (Internal_Internal_Entity_Compilation_Unit_Array'(1 => Var_Expr_1)); 
--# expr-done 6686
Scope_Result_2 := Array_Lit;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2881;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var_1, Result_Var_2); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6687
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1051;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1051;
                     Finalizer_Scope_2880;
                     Finalizer_Scope_2881;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Referenced_Units_In;
--# end
--# property-start CompilationUnit.withed_units 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10059'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Withed_Units
  (Node : Bare_Compilation_Unit
      ; Include_Privates : Boolean
         := True
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1052 with Inline_Always;
            procedure Finalizer_Scope_2882 with Inline_Always;
      Fld : Bare_Name_Array_Access;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1052 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
            end Finalizer_Scope_1052;
            procedure Finalizer_Scope_2882 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2882;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Withed_Units,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Privates);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6692 '<unique at nodes.lkt:10060:9>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10060'
--# expr-start 6691 '<Mapcat at nodes.lkt:10060:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10060'
   --# expr-start 6688 '<FieldAccess for top_level_with_package_clauses at nodes.lkt:10060:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10060'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self, Include_Privates => Include_Privates);
--# end
--# expr-done 6688
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 6690 '<FieldAccess for referenced_units_in at nodes.lkt:10064:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10064'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6689 '<as_bare_entity at nodes.lkt:10064:45>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10064'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 6689
--# property-call-start CompilationUnit.referenced_units_in
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Referenced_Units_In (Node => Self, Name => If_Result);
--# end
--# expr-done 6690
         for Item_To_Append of
               Fld_1.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2882;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6691
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 6692
         Property_Result := Unique_Array;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1052;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1052;
                     Finalizer_Scope_2882;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Withed_Units;
--# end
--# property-start CompilationUnit.imported_units 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10073'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Imported_Units
  (Node : Bare_Compilation_Unit
      ; Include_Privates : Boolean
         := True
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_privates Include_Privates
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1053 with Inline_Always;
            procedure Finalizer_Scope_2883 with Inline_Always;
            procedure Finalizer_Scope_2884 with Inline_Always;
            procedure Finalizer_Scope_2885 with Inline_Always;
            procedure Finalizer_Scope_2886 with Inline_Always;
            procedure Finalizer_Scope_2887 with Inline_Always;
            procedure Finalizer_Scope_2888 with Inline_Always;
      Var_Expr_0 : Bare_Base_Subp_Body;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Bare_Ada_Node;
Var_Expr : Internal_Entity_Compilation_Unit_Array_Access;
Var_Expr_188 : Bare_Basic_Decl;
Var_Expr_189 : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Basic_Decl;
Cast_Result : Bare_Base_Subp_Body;
As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Compilation_Unit;
As_Entity_1 : Internal_Entity_Compilation_Unit;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Compilation_Unit;
Array_Lit : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_6 : Bare_Compilation_Unit;
As_Entity_2 : Internal_Entity_Compilation_Unit;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Compilation_Unit;
Array_Lit_1 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_3 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_4 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_3 : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_5 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_4 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_5 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1053 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1053;
            procedure Finalizer_Scope_2883 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2883;
            procedure Finalizer_Scope_2884 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2884;
            procedure Finalizer_Scope_2885 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2885;
            procedure Finalizer_Scope_2886 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_2886;
            procedure Finalizer_Scope_2887 is
            begin
                     Dec_Ref (Var_Expr_189);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_2887;
            procedure Finalizer_Scope_2888 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2888;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Privates);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6693 '<FieldAccess for withed_units at nodes.lkt:10074:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10074'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.withed_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Withed_Units (Node => Self, Include_Privates => Include_Privates);
--# end
--# expr-done 6693
--# expr-start 6695 '<Cast to BaseSubpBody at nodes.lkt:10079:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10079'
--# expr-start 6694 '<FieldAccess for decl at nodes.lkt:10079:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10079'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 6694
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6695
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp Var_Expr_0
--# expr-start 6697 '<FieldAccess for subp_previous_part at nodes.lkt:10083:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10083'
--# expr-start 6696 '<as_bare_entity at nodes.lkt:10083:27>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10083'
Is_Null := Var_Expr_0 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_0); 
   If_Result := As_Entity;
end if;
--# expr-done 6696
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.subp_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => If_Result.Node, Imprecise_Fallback => False, E_Info => If_Result.Info);
--# end
--# expr-done 6697
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 6700 '<ArrayLiteral at nodes.lkt:10084:29>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10084'
--# expr-start 6699 '<as_bare_entity at nodes.lkt:10084:30>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10084'
--# expr-start 6698 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:10084:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10084'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr_1.Node);
--# end
--# expr-done 6698
Is_Null_1 := Fld_3 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Compilation_Unit;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 6699
Array_Lit := Create_Internal_Entity_Compilation_Unit_Array (Internal_Internal_Entity_Compilation_Unit_Array'(1 => If_Result_1)); 
--# expr-done 6700
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2884;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2883;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Var_Expr := Result_Var_1;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Compilation_Unit_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_2 := Var_Expr;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2885;
   Result_Var_5 := Scope_Result_2;
else
--# expr-start 6701 '<FieldAccess for decl at nodes.lkt:10086:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10086'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 6701
Var_Expr_188 := Fld_4;
if Var_Expr_188 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6702 '<FieldAccess for node_env at nodes.lkt:10086:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10086'
      if Var_Expr_188 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Var_Expr_188);
--# expr-done 6702
Var_Expr_189 := Fld_5;
      Inc_Ref (Var_Expr_189);
if Var_Expr_189 /= Empty_Env then
   --# scope-start
--# expr-start 6703 '<env_node at nodes.lkt:10086:19>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10086'
Env_Node := AST_Envs.Env_Node (Var_Expr_189); 
--# expr-done 6703
Var_Expr_2 := Env_Node;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 6706 '<ArrayLiteral at nodes.lkt:10087:24>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10087'
--# expr-start 6705 '<as_bare_entity at nodes.lkt:10087:25>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10087'
--# expr-start 6704 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:10087:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10087'
      if Var_Expr_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_6 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr_2);
--# end
--# expr-done 6704
Is_Null_2 := Fld_6 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Compilation_Unit;
else
As_Entity_2 := (Info => No_Entity_Info, Node => Fld_6); 
   If_Result_2 := As_Entity_2;
end if;
--# expr-done 6705
Array_Lit_1 := Create_Internal_Entity_Compilation_Unit_Array (Internal_Internal_Entity_Compilation_Unit_Array'(1 => If_Result_2)); 
--# expr-done 6706
Scope_Result_3 := Array_Lit_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2888;
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_4 := Result_Var_2;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2887;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Scope_Result_5 := Result_Var_3;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2886;
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
   Result_Var_5 := Result_Var_4;
end if;
      Inc_Ref (Result_Var_5);
Concat_Result := Concat (Fld, Result_Var_5); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1053;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1053;
                     Finalizer_Scope_2883;
                     Finalizer_Scope_2884;
                     Finalizer_Scope_2885;
                     Finalizer_Scope_2886;
                     Finalizer_Scope_2887;
                     Finalizer_Scope_2888;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Imported_Units;
--# end
--# property-start CompilationUnit.privately_imported_units 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10099'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Privately_Imported_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Unit_Array_Access;
            procedure Finalizer_Scope_1054 with Inline_Always;
      Local_All_Imports : Internal_Entity_Compilation_Unit_Array_Access;
Local_Public_Imports : Internal_Entity_Compilation_Unit_Array_Access;
Var_Expr_190 : Bare_With_Clause;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_2 : Internal_Unit;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Unit_Array_Access;
Let_Result : Internal_Unit_Array_Access;
Let_Result_1 : Internal_Unit_Array_Access;
Fld_3 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr_2 : Bare_Ada_Node;
Cast_Result_2 : Bare_With_Clause;
Fld_4 : Bare_Private_Node;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result_1 : Boolean;
If_Result : Internal_Unit_Array_Access;
            procedure Finalizer_Scope_1054 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_All_Imports);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Public_Imports);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1054;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Privately_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6721 '<If at nodes.lkt:10102:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10102'
--# expr-start 6711 '<AnyQuantifier at nodes.lkt:10102:12>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10102'
   --# expr-start 6707 '<FieldAccess for prelude at nodes.lkt:10102:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10102'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Compilation_Unit_F_Prelude;
--# expr-done 6707
      Quantifier_Result_1 := False;
      if Fld_3 = null then
      null;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_3;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind clause Item_0
--# expr-start 6708 '<Cast to WithClause at nodes.lkt:10103:25>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10103'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_With_Clause_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 6708
Var_Expr_190 := Cast_Result_2;
if Var_Expr_190 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6710 '<FieldAccess for as_bool at nodes.lkt:10103:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10103'
--# expr-start 6709 '<FieldAccess for has_private at nodes.lkt:10103:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10103'
      if Var_Expr_190 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr_190.With_Clause_F_Has_Private;
--# expr-done 6709
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_5 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_4);
--# end
--# expr-done 6710
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result_1 := Result_Var;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 6711
if Quantifier_Result_1 then
   --# expr-start 6720 '<Let at nodes.lkt:10105:13>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10105'
--# scope-start
--# expr-start 6712 '<FieldAccess for imported_units at nodes.lkt:10105:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10105'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Self, Include_Privates => True);
--# end
--# expr-done 6712
Local_All_Imports := Fld; Inc_Ref (Local_All_Imports);
--# bind all_imports Local_All_Imports
--# expr-start 6719 '<Let at nodes.lkt:10106:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10106'
--# scope-start
--# expr-start 6713 '<FieldAccess for imported_units at nodes.lkt:10106:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10106'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Self, Include_Privates => False);
--# end
--# expr-done 6713
Local_Public_Imports := Fld_1; Inc_Ref (Local_Public_Imports);
--# bind public_imports Local_Public_Imports
--# expr-start 6718 '<Map at nodes.lkt:10108:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10108'
      declare
         Map_Result_Vec : Internal_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Local_All_Imports;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind cu Item_1
         --# expr-start 6716 '<Not at nodes.lkt:10109:44>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10109'
--# expr-start 6715 '<Contains at nodes.lkt:10109:48>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10109'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Local_Public_Imports;
      begin
         for Item_12 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_12
      --# expr-start 6714 '<Eq at ???>' Is_Equiv None
Cast_Expr := Item_12; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 6714
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6715
Not_Val := not (Quantifier_Result); 
--# expr-done 6716
         if Not_Val then
      --# expr-start 6717 '<FieldAccess for unit at nodes.lkt:10109:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10109'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Item_1.Node);
--# expr-done 6717
         declare
            Item_To_Append : constant Internal_Unit := Fld_2;
         begin
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Unit_Array
           (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6718
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6719
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6720
   If_Result := Let_Result_1;
else
   If_Result := No_Internal_Unit_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 6721
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1054;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1054;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Unit_Array_Access,
                        As_Internal_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Privately_Imported_Units;
--# end
--# property-start CompilationUnit.unit_dependencies_helper 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10118'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies_Helper
  (Node : Bare_Compilation_Unit
      ; Visited : Internal_Entity_Compilation_Unit_Array_Access
      ; To_Visit : Internal_Entity_Compilation_Unit_Array_Access
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind visited Visited
   --# bind to_visit To_Visit
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1055 with Inline_Always;
            procedure Finalizer_Scope_2893 with Inline_Always;
      Local_Now_Visited : Internal_Entity_Compilation_Unit_Array_Access;
Local_New_Imports : Internal_Entity_Compilation_Unit_Array_Access;
Local_To_Visit_Next : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1055 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Local_Now_Visited);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Local_New_Imports);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_To_Visit_Next);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1055;
            procedure Finalizer_Scope_2893 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2893;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6734 '<Let at nodes.lkt:10119:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10119'
--# scope-start
Concat_Result := Concat (Visited, To_Visit); 
Local_Now_Visited := Concat_Result; Inc_Ref (Local_Now_Visited);
--# bind now_visited Local_Now_Visited
--# expr-start 6733 '<Let at nodes.lkt:10120:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10120'
--# scope-start
--# expr-start 6724 '<unique at nodes.lkt:10120:27>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10120'
--# expr-start 6723 '<Mapcat at nodes.lkt:10120:27>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10120'
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := To_Visit;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_0
      --# expr-start 6722 '<FieldAccess for imported_units at nodes.lkt:10120:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10120'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Item_0.Node);
--# end
--# expr-done 6722
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2893;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6723
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 6724
Local_New_Imports := Unique_Array; Inc_Ref (Local_New_Imports);
--# bind new_imports Local_New_Imports
--# expr-start 6732 '<Let at nodes.lkt:10121:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10121'
--# scope-start
--# expr-start 6728 '<Map at nodes.lkt:10121:29>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10121'
      declare
         Map_Result_1_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Local_New_Imports;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_1
         --# expr-start 6727 '<Not at nodes.lkt:10121:55>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10121'
--# expr-start 6726 '<Contains at nodes.lkt:10121:59>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10121'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Local_Now_Visited;
      begin
         for Item_13 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_13
      --# expr-start 6725 '<Eq at ???>' Is_Equiv None
Cast_Expr := Item_13; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 6725
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6726
Not_Val := not (Quantifier_Result); 
--# expr-done 6727
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit := Item_1;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6728
Local_To_Visit_Next := Map_Result_1; Inc_Ref (Local_To_Visit_Next);
--# bind to_visit_next Local_To_Visit_Next
--# expr-start 6731 '<If at nodes.lkt:10123:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10123'
--# expr-start 6729 '<length at nodes.lkt:10123:12>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10123'
Len := Length (Local_To_Visit_Next); 
--# expr-done 6729
Comp_Result := Len > 0; 
if Comp_Result then
   --# expr-start 6730 '<FieldAccess for unit_dependencies_helper at nodes.lkt:10123:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10123'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => Local_Now_Visited, To_Visit => Local_To_Visit_Next);
--# end
--# expr-done 6730
   If_Result := Fld_1;
else
   If_Result := Local_Now_Visited;
end if;
      Inc_Ref (If_Result);
--# expr-done 6731
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6732
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6733
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6734
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1055;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1055;
                     Finalizer_Scope_2893;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Unit_Dependencies_Helper;
--# end
--# property-start CompilationUnit.unit_dependencies 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10132'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies
  (Node : Bare_Compilation_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1056 with Inline_Always;
      Array_Lit : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Cast_Result : Bare_Compilation_Unit;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1056 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1056;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Unit_Dependencies,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6741 '<Map at nodes.lkt:10132:63>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10132'
   --# expr-start 6737 '<unique at nodes.lkt:10132:63>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10132'
--# expr-start 6736 '<FieldAccess for unit_dependencies_helper at nodes.lkt:10132:63>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10132'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6735 '<ArrayLiteral at nodes.lkt:10133:47>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10133'
Array_Lit := Create_Internal_Entity_Compilation_Unit_Array (Internal_Internal_Entity_Compilation_Unit_Array'(1 => Ent)); 
--# expr-done 6735
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => No_Internal_Entity_Compilation_Unit_Array_Type, To_Visit => Array_Lit);
--# end
--# expr-done 6736
Unique_Array := Make_Unique (Fld); 
--# expr-done 6737
      declare
         Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Unique_Array;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_0
         --# expr-start 6740 '<Not at nodes.lkt:10136:16>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10136'
--# expr-start 6739 '<Eq at nodes.lkt:10136:16>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10136'
--# expr-start 6738 '<FieldAccess for node at nodes.lkt:10136:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10136'
Fld_1 := Item_0.Node;
--# expr-done 6738
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Self; 
--# expr-done 6739
Not_Val := not (Is_Equal); 
--# expr-done 6740
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit := Item_0;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Compilation_Unit_Array
           (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6741
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1056;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1056;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                        As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Unit_Dependencies;
--# end
--# property-start CompilationUnit.decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10141'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Decl
  (Node : Bare_Compilation_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Decl;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Cast_Result_1 : Bare_Subunit;
Cast_Result_2 : Bare_Basic_Decl;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6745 '<Match at nodes.lkt:10141:29>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10141'
--# expr-start 6742 '<FieldAccess for body at nodes.lkt:10141:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10141'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 6742
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 6743 '<FieldAccess for item at nodes.lkt:10142:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10142'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 6743
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 6744 '<FieldAccess for body at nodes.lkt:10143:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10143'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_1.Subunit_F_Body;
--# expr-done 6744
      Cast_Result_2 := Fld_2;
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6745
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Decl;
--# end
--# property-start CompilationUnit.is_preelaborable_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10153'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable_Impl
  (Node : Bare_Compilation_Unit
      ; From_Body : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_body From_Body
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subunit;
Match_1 : Internal_Entity_Library_Item;
Match_0_1 : Internal_Entity_Subp_Decl;
Match_1_1 : Internal_Entity_Generic_Subp_Decl;
Match_2 : Internal_Entity_Subp_Body;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_3 : Internal_Entity_Package_Decl;
Match_4 : Internal_Entity_Generic_Package_Decl;
Match_5 : Internal_Entity_Package_Body;
Match_6 : Internal_Entity_Basic_Decl;
Match_2_1 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity;
Fld_12 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_13 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_14 : Boolean;
Fld_15 : Internal_Entity_Generic_Package_Internal;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Internal_Unit;
Fld_19 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Compilation_Unit;
Fld_20 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity_Generic_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Body;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_6 : Internal_Entity_Package_Decl;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_7 : Internal_Entity_Generic_Package_Decl;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_8 : Internal_Entity_Package_Body;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Subunit;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_11 : Internal_Entity_Library_Item;
Let_Result_8 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_12 : Internal_Entity;
Let_Result_9 : Boolean;
Scope_Result_10 : Boolean;
Match_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6772 '<Match at nodes.lkt:10153:56>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10153'
--# expr-start 6746 '<FieldAccess for body at nodes.lkt:10153:62>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10153'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Compilation_Unit_F_Body, Info => Ent.Info);
--# expr-done 6746
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Node.Kind) is
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_10; 
--# bind su Match_0
--# expr-start 6751 '<FieldAccess for is_preelaborable_impl at nodes.lkt:10156:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10156'
--# expr-start 6750 '<Cast to CompilationUnit at nodes.lkt:10156:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10156'
--# expr-start 6749 '<parent at nodes.lkt:10156:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10156'
--# expr-start 6748 '<parent at nodes.lkt:10156:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10156'
--# expr-start 6747 '<FieldAccess for body_root at nodes.lkt:10156:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10156'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Match_0.Node);
--# end
--# expr-done 6747
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 6748
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 6749
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6750
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 6751
Let_Result_7 := Fld_4; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_11; 
--# bind li Match_1
--# expr-start 6771 '<Match at nodes.lkt:10157:33>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10157'
--# expr-start 6752 '<FieldAccess for item at nodes.lkt:10157:39>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10157'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Library_Item_F_Item, Info => Match_1.Info);
--# expr-done 6752
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_5; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_3; 
--# bind subp_decl Match_0_1
--# expr-start 6753 '<FieldAccess for does_aspects_make_preelaborable at nodes.lkt:10158:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10158'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_0_1.Node, From_Body => From_Body, E_Info => Match_0_1.Info);
--# end
--# expr-done 6753
Let_Result := Fld_6; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_4; 
--# bind gen_subp_decl Match_1_1
--# expr-start 6755 '<FieldAccess for does_aspects_make_preelaborable at nodes.lkt:10159:52>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10159'
--# expr-start 6754 '<FieldAccess for decl at nodes.lkt:10159:52>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10159'
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Match_1_1.Node, E_Info => Match_1_1.Info);
--# end
--# expr-done 6754
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_7.Node, From_Body => From_Body, E_Info => Fld_7.Info);
--# end
--# expr-done 6755
Let_Result_1 := Fld_8; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind subp_body Match_2
--# expr-start 6756 '<FieldAccess for does_aspects_make_preelaborable at nodes.lkt:10163:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10163'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_2.Node, From_Body => False, E_Info => Match_2.Info);
--# end
--# expr-done 6756
if Fld_9 then
   If_Result := True;
else
--# expr-start 6757 '<FieldAccess for decl_part at nodes.lkt:10167:17>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10167'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2.Info);
--# end
--# expr-done 6757
Var_Expr_0 := Fld_10;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr_0
--# expr-start 6761 '<FieldAccess for is_preelaborable_impl at nodes.lkt:10168:29>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10168'
--# expr-start 6760 '<Cast to CompilationUnit at nodes.lkt:10168:29>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10168'
--# expr-start 6759 '<parent at nodes.lkt:10168:29>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10168'
--# expr-start 6758 '<parent at nodes.lkt:10168:29>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10168'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 6758
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# expr-done 6759
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6760
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_13 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_1.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 6761
Scope_Result := Fld_13;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result_2 := If_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind pkg_decl Match_3
--# expr-start 6762 '<FieldAccess for does_aspects_make_preelaborable at nodes.lkt:10171:43>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10171'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_3.Node, From_Body => From_Body, E_Info => Match_3.Info);
--# end
--# expr-done 6762
Let_Result_3 := Fld_14; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind gen_pkg_decl Match_4
--# expr-start 6764 '<FieldAccess for does_aspects_make_preelaborable at nodes.lkt:10172:54>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10172'
--# expr-start 6763 '<FieldAccess for package_decl at nodes.lkt:10172:54>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10172'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_4.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_4.Info);
--# expr-done 6763
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_15.Node, From_Body => From_Body, E_Info => Fld_15.Info);
--# end
--# expr-done 6764
Let_Result_4 := Fld_16; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind pkg_body Match_5
--# expr-start 6770 '<FieldAccess for is_preelaborable_impl at nodes.lkt:10176:17>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
--# expr-start 6769 '<Cast to CompilationUnit at nodes.lkt:10176:17>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
--# expr-start 6768 '<as_bare_entity at nodes.lkt:10176:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
--# expr-start 6767 '<FieldAccess for root at nodes.lkt:10176:17>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
--# expr-start 6766 '<FieldAccess for unit at nodes.lkt:10176:17>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
--# expr-start 6765 '<FieldAccess for decl_part at nodes.lkt:10176:17>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10176'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_17 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 6765
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Unit (Node => Fld_17.Node);
--# expr-done 6766
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Fld_18.Ast_Root;
--# expr-done 6767
Is_Null := Fld_19 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_19); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6768
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Compilation_Unit
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6769
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_20 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_2.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 6770
Let_Result_5 := Fld_20; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 6771
Let_Result_8 := Match_Result; 
--# end
Scope_Result_9 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2_1 := Cast_Result_12; 
--# bind _ Match_2_1
Let_Result_9 := False; 
--# end
Scope_Result_10 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_10; 
end case;
--# expr-done 6772
         Property_Result := Match_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Preelaborable_Impl;
--# end
--# property-start CompilationUnit.is_preelaborable 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10185'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable
  (Node : Bare_Compilation_Unit
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
     Ent : Internal_Entity_Compilation_Unit :=
       Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6773 '<FieldAccess for is_preelaborable_impl at nodes.lkt:10185:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10185'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Ent.Node, From_Body => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 6773
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Preelaborable;
--# end
--# property-start CompilationUnit.self_restrictions 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10190'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Self_Restrictions
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1060 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Pragma_Node;
Var_Expr_191 : Internal_Entity_Base_Assoc;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Base_Assoc_List;
Fld_4 : Bare_Base_Assoc_List;
Fld_5 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_7 : Symbol_Type;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Pragma_Node;
Fld_8 : Internal_Entity_Identifier;
Fld_9 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1060 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1060;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6786 '<Map at nodes.lkt:10190:46>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10190'
   --# expr-start 6775 '<FieldAccess for prelude at nodes.lkt:10190:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10190'
--# expr-start 6774 '<as_bare_entity at nodes.lkt:10190:46>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10190'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 6774
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => If_Result.Node.Compilation_Unit_F_Prelude, Info => If_Result.Info);
--# expr-done 6775
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Symbol_Type_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_2 := Bare_Item_0 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity_2 := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_0 := If_Result_2; 
   --# scope-start
               --# bind n Item_0
--# expr-start 6776 '<Cast to Pragma at nodes.lkt:10191:85>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Pragma_Node;
   end if;
--# expr-done 6776
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 6780 '<Eq at nodes.lkt:10192:20>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10192'
--# expr-start 6778 '<FieldAccess for name_symbol at nodes.lkt:10192:20>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10192'
--# expr-start 6777 '<FieldAccess for id at nodes.lkt:10192:20>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10192'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Identifier (Node => Var_Expr_1.Node.Pragma_Node_F_Id, Info => Var_Expr_1.Info);
--# expr-done 6777
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_8.Node);
--# end
--# expr-done 6778
--# expr-start 6779 '<Symbol Restrictions at nodes.lkt:10192:42>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10192'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Restrictions); 
--# expr-done 6779
Is_Equal := Fld_9 = Sym; 
--# expr-done 6780
Scope_Result_1 := Is_Equal;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
         if Result_Var_1 then
      --# expr-start 6785 '<FieldAccess for name_symbol at nodes.lkt:10191:16>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
--# expr-start 6784 '<Cast to BaseId at nodes.lkt:10191:16>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
--# expr-start 6782 '<FieldAccess for args at nodes.lkt:10191:16>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
--# expr-start 6781 '<Cast to Pragma at nodes.lkt:10191:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 6781
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Assoc_List (Node => Cast_Result.Node.Pragma_Node_F_Args, Info => Cast_Result.Info);
--# expr-done 6782
Fld_4 := Fld_3.Node;
Get_Result := Get (Self, Fld_4, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Assoc;
else
Fld_5 := Fld_3.Info;
As_Entity_1 := (Info => Fld_5, Node => Get_Result); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_191 := If_Result_1;
if Var_Expr_191 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 6783 '<FieldAccess for assoc_expr at nodes.lkt:10191:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10191'
      if Var_Expr_191.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_191.Node, E_Info => Var_Expr_191.Info);
--# end
--# expr-done 6783
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 6784
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_1.Node);
--# end
--# expr-done 6785
         declare
            Item_To_Append : constant Symbol_Type := Fld_7;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6786
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1060;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1060;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Self_Restrictions;
--# end
--# property-start CompilationUnit.other_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10199'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Other_Part
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1061 with Inline_Always;
      Local_Other_Kind : Analysis_Unit_Kind;
Var_Expr_192 : Bare_Compilation_Unit;
Fld : Analysis_Unit_Kind;
Is_Equal : Boolean;
If_Result : Analysis_Unit_Kind;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Compilation_Unit;
Scope_Result : Internal_Entity_Compilation_Unit;
Result_Var : Internal_Entity_Compilation_Unit;
Let_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1061 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1061;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6793 '<Let at nodes.lkt:10200:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10200'
--# scope-start
--# expr-start 6789 '<If at nodes.lkt:10200:26>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10200'
--# expr-start 6788 '<Eq at nodes.lkt:10200:29>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10200'
--# expr-start 6787 '<FieldAccess for unit_kind at nodes.lkt:10200:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10200'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.unit_kind
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Kind (Node => Self);
--# end
--# expr-done 6787
Is_Equal := Fld = Unit_Specification; 
--# expr-done 6788
if Is_Equal then
   If_Result := Unit_Body;
else
   If_Result := Unit_Specification;
end if;
--# expr-done 6789
Local_Other_Kind := If_Result; 
--# bind other_kind Local_Other_Kind
--# expr-start 6791 '<FieldAccess for designated_compilation_unit at nodes.lkt:10202:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10202'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6790 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:10203:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10203'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 6790
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Local_Other_Kind, Not_Found_Is_Error => False);
--# end
--# expr-done 6791
Var_Expr_192 := Fld_2;
if Var_Expr_192 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6792 '<as_bare_entity at nodes.lkt:10202:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10202'
Is_Null := Var_Expr_192 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_192); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6792
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Compilation_Unit;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 6793
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1061;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1061;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Other_Part;
--# end
--# property-start CompilationUnit.has_restriction 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10214'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Has_Restriction
  (Node : Bare_Compilation_Unit
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_2915 with Inline_Always;
            procedure Finalizer_Scope_2917 with Inline_Always;
      Match_0 : Bare_Library_Item;
Local_Spec : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Var_Expr_193 : Internal_Entity_Compilation_Unit;
Var_Expr_194 : Internal_Entity_Compilation_Unit;
Fld : Bare_Ada_Node;
Fld_1 : Symbol_Type_Array_Access;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
Fld_2 : Internal_Entity_Compilation_Unit;
Fld_3 : Symbol_Type_Array_Access;
Is_Equal_1 : Boolean;
Quantifier_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Node;
Fld_5 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_6 : Internal_Entity_Defining_Name;
Sym : Symbol_Type;
Fld_7 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
If_Result_4 : Boolean;
Fld_8 : Internal_Entity_Compilation_Unit;
Fld_9 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_2 : Bare_Subunit;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_5 : Boolean;
            procedure Finalizer_Scope_2915 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2915;
            procedure Finalizer_Scope_2917 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2917;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6819 '<If at nodes.lkt:10215:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10215'
--# expr-start 6795 '<Eq at nodes.lkt:10215:12>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10215'
--# expr-start 6794 '<Symbol No_Elaboration_Code at nodes.lkt:10215:20>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10215'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_No_Elaboration_Code); 
--# expr-done 6794
Is_Equal_2 := Name = Sym_1; 
--# expr-done 6795
if Is_Equal_2 then
   --# expr-start 6818 '<Match at nodes.lkt:10215:48>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10215'
--# expr-start 6796 '<FieldAccess for body at nodes.lkt:10215:54>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10215'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 6796
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_0 := Cast_Result_1; 
--# bind li Match_0
--# expr-start 6799 '<Contains at nodes.lkt:10218:37>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10218'
   --# expr-start 6797 '<FieldAccess for self_restrictions at nodes.lkt:10218:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10218'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.self_restrictions
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Self_Restrictions (Node => Self);
--# end
--# expr-done 6797
      Quantifier_Result := False;
      declare
         Collection : constant Symbol_Type_Array_Access := Fld_1;
      begin
         for Item_14 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_14
      --# expr-start 6798 '<Eq at ???>' Is_Equal None
Is_Equal := Item_14 = Name; 
--# expr-done 6798
      Quantifier_Result := Is_Equal;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6799
if Quantifier_Result then
   If_Result := True;
else
--# expr-start 6800 '<FieldAccess for other_part at nodes.lkt:10219:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10219'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Self);
--# end
--# expr-done 6800
Var_Expr_193 := Fld_2;
if Var_Expr_193 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6803 '<Contains at nodes.lkt:10219:20>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10219'
   --# expr-start 6801 '<FieldAccess for self_restrictions at nodes.lkt:10219:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10219'
      if Var_Expr_193.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.self_restrictions
Fld_3 := Libadalang.Implementation.Compilation_Unit_P_Self_Restrictions (Node => Var_Expr_193.Node);
--# end
--# expr-done 6801
      Quantifier_Result_1 := False;
      declare
         Collection : constant Symbol_Type_Array_Access := Fld_3;
      begin
         for Item_15 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_15
      --# expr-start 6802 '<Eq at ???>' Is_Equal_1 None
Is_Equal_1 := Item_15 = Name; 
--# expr-done 6802
      Quantifier_Result_1 := Is_Equal_1;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 6803
Scope_Result := Quantifier_Result_1;
   --# end
      Finalizer_Scope_2917;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_4 := True;
else
   --# expr-start 6815 '<Let at nodes.lkt:10224:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10224'
--# scope-start
--# expr-start 6805 '<as_bare_entity at nodes.lkt:10224:32>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10224'
--# expr-start 6804 '<FieldAccess for item at nodes.lkt:10224:32>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10224'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_0.Library_Item_F_Item;
--# expr-done 6804
Is_Null := Fld_4 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6805
Var_Expr_0 := If_Result_1;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind i Var_Expr_0
--# expr-start 6809 '<If at nodes.lkt:10225:32>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10225'
--# expr-start 6806 '<IsA Body at nodes.lkt:10225:35>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10225'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Body_Node; 
--# expr-done 6806
if Is_A then
   --# expr-start 6808 '<FieldAccess for decl_part at nodes.lkt:10226:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10226'
--# expr-start 6807 '<Cast to Body at nodes.lkt:10226:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10226'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 6807
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_5 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result.Node, Imprecise_Fallback => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 6808
   If_Result_2 := Fld_5;
else
   If_Result_2 := Var_Expr_0;
end if;
--# expr-done 6809
Scope_Result_1 := If_Result_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Local_Spec := Result_Var_1; 
--# bind spec Local_Spec
--# expr-start 6811 '<Not at nodes.lkt:10231:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10231'
--# expr-start 6810 '<IsNull at nodes.lkt:10231:25>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10231'
Is_Null_1 := Local_Spec.Node = null; 
--# expr-done 6810
Not_Val := not (Is_Null_1); 
--# expr-done 6811
if Not_Val then
   --# expr-start 6814 '<FieldAccess for has_aspect at nodes.lkt:10231:42>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10231'
--# expr-start 6812 '<FieldAccess for defining_name at nodes.lkt:10231:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10231'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Local_Spec.Node, E_Info => Local_Spec.Info);
--# end
--# expr-done 6812
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6813 '<Symbol No_Elaboration_Code_All at nodes.lkt:10232:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10232'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_No_Elaboration_Code_All); 
--# expr-done 6813
--# property-call-start DefiningName.has_aspect
Fld_7 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Fld_6.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Fld_6.Info);
--# end
--# expr-done 6814
   If_Result_3 := Fld_7;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 6815
   If_Result_4 := Let_Result;
end if;
Let_Result_1 := If_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
      Finalizer_Scope_2915;
            Match_Result := Scope_Result_3; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind su Match_1
--# expr-start 6816 '<FieldAccess for root_unit at nodes.lkt:10237:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10237'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld_8 := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Match_1);
--# end
--# expr-done 6816
Var_Expr_194 := Fld_8;
if Var_Expr_194 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6817 '<FieldAccess for has_restriction at nodes.lkt:10237:33>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10237'
      if Var_Expr_194.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.has_restriction
Fld_9 := Libadalang.Implementation.Compilation_Unit_P_Has_Restriction (Node => Var_Expr_194.Node, Name => Name);
--# end
--# expr-done 6817
Scope_Result_2 := Fld_9;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Let_Result_2 := Result_Var_2; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_3 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 6818
   If_Result_5 := Match_Result;
else
   If_Result_5 := raise Property_Error with "Unsupported restriction";
end if;
--# expr-done 6819
         Property_Result := If_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2915;
                     Finalizer_Scope_2917;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Has_Restriction;
--# end
--# property-start CompilationUnit.is_text_io_child 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10246'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Text_Io_Child
  (Node : Bare_Compilation_Unit
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1063 with Inline_Always;
      Local_Name_Parts : Symbol_Type_Array_Access;
Fld : Symbol_Type_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Get_Result_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_2 : Boolean;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1063 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Name_Parts);
            end Finalizer_Scope_1063;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6830 '<Let at nodes.lkt:10247:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10247'
--# scope-start
--# expr-start 6820 '<FieldAccess for syntactic_fully_qualified_name at nodes.lkt:10247:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10247'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 6820
Local_Name_Parts := Fld; Inc_Ref (Local_Name_Parts);
--# bind name_parts Local_Name_Parts
--# expr-start 6822 '<Eq at nodes.lkt:10249:9>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
--# expr-start 6821 '<length at nodes.lkt:10249:9>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Len := Length (Local_Name_Parts); 
--# expr-done 6821
Is_Equal := Len = 3; 
--# expr-done 6822
if Is_Equal then
   --# expr-start 6825 '<Eq at nodes.lkt:10249:38>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
--# expr-start 6823 '<at at nodes.lkt:10249:38>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Get_Result := Get (Self, Local_Name_Parts, 0, True); 
--# expr-done 6823
--# expr-start 6824 '<Symbol ada at nodes.lkt:10249:56>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada_320); 
--# expr-done 6824
Is_Equal_1 := Get_Result = Sym; 
--# expr-done 6825
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 6826 '<at at nodes.lkt:10249:67>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Get_Result_1 := Get (Self, Local_Name_Parts, 1, True); 
--# expr-done 6826
--# expr-start 6827 '<Symbol text_io at nodes.lkt:10249:85>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Text_Io); 
--# expr-done 6827
Is_Equal_4 := Get_Result_1 = Sym_1; 
if Is_Equal_4 then
   If_Result_2 := True;
else
--# expr-start 6828 '<Symbol wide_text_io at nodes.lkt:10249:98>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Text_Io); 
--# expr-done 6828
Is_Equal_3 := Get_Result_1 = Sym_2; 
if Is_Equal_3 then
   If_Result_1 := True;
else
--# expr-start 6829 '<Symbol wide_wide_text_io at nodes.lkt:10249:116>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10249'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Text_Io); 
--# expr-done 6829
Is_Equal_2 := Get_Result_1 = Sym_3; 
   If_Result_1 := Is_Equal_2;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 6830
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1063;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1063;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Is_Text_Io_Child;
--# end
--# property-start CompilationUnit.local_config_pragmas 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10259'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Local_Config_Pragmas
  (Node : Bare_Compilation_Unit
  )
   return Bare_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1065 with Inline_Always;
      Fld : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Pragma_Node;
Is_A : Boolean;
Map_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1065 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1065;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 6834 '<Map at nodes.lkt:10260:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10260'
   --# expr-start 6831 '<FieldAccess for prelude at nodes.lkt:10260:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10260'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Compilation_Unit_F_Prelude;
--# expr-done 6831
      declare
         Map_Result_Vec : Bare_Pragma_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind n Item_0
         --# expr-start 6832 '<IsA Pragma at nodes.lkt:10260:60>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10260'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Pragma_Node_Range; 
--# expr-done 6832
         if Is_A then
      --# expr-start 6833 '<Cast to Pragma at nodes.lkt:10260:39>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10260'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6833
         declare
            Item_To_Append : constant Bare_Pragma_Node := Cast_Result;
         begin
            Bare_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Pragma_Node_Array
           (Items_Count => Natural (Bare_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6834
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1065;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1065;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Local_Config_Pragmas;
--# end
--# property-start CompilationUnit.sources_config_pragmas 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10264'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Sources_Config_Pragmas
  (Node : Bare_Compilation_Unit
      ; Include_Other_Part : Boolean
         := True
  )
   return Bare_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind include_other_part Include_Other_Part
   Property_Result : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1066 with Inline_Always;
            procedure Finalizer_Scope_2924 with Inline_Always;
            procedure Finalizer_Scope_2925 with Inline_Always;
      Local_Current_Unit : Bare_Pragma_Node_Array_Access;
Local_Related_Units : Bare_Pragma_Node_Array_Access;
Var_Expr_195 : Internal_Entity_Compilation_Unit;
Var_Expr_196 : Internal_Entity_Compilation_Unit;
Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit;
Fld_2 : Bare_Pragma_Node_Array_Access;
Scope_Result : Bare_Pragma_Node_Array_Access;
Result_Var : Bare_Pragma_Node_Array_Access;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_4 : Internal_Entity_Compilation_Unit;
Fld_5 : Bare_Pragma_Node_Array_Access;
Scope_Result_1 : Bare_Pragma_Node_Array_Access;
Result_Var_1 : Bare_Pragma_Node_Array_Access;
Fld_6 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Bare_Pragma_Node_Array_Access;
Fld_7 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
Let_Result : Bare_Pragma_Node_Array_Access;
Let_Result_1 : Bare_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1066 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Current_Unit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Local_Related_Units);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1066;
            procedure Finalizer_Scope_2924 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2924;
            procedure Finalizer_Scope_2925 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2925;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6849 '<Let at nodes.lkt:10266:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10266'
--# scope-start
--# expr-start 6835 '<FieldAccess for local_config_pragmas at nodes.lkt:10266:28>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10266'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.local_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_Local_Config_Pragmas (Node => Self);
--# end
--# expr-done 6835
Local_Current_Unit := Fld; Inc_Ref (Local_Current_Unit);
--# bind current_unit Local_Current_Unit
--# expr-start 6848 '<Let at nodes.lkt:10268:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10268'
--# scope-start
--# expr-start 6847 '<If at nodes.lkt:10270:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
if Include_Other_Part then
   --# expr-start 6837 '<IsA LibraryItem at nodes.lkt:10270:35>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
--# expr-start 6836 '<FieldAccess for body at nodes.lkt:10270:35>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Compilation_Unit_F_Body;
--# expr-done 6836
Is_A_1 := Fld_7 /= null 
and then Fld_7.Kind in Ada_Library_Item_Range; 
--# expr-done 6837
   If_Result_1 := Is_A_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
--# expr-start 6838 '<FieldAccess for other_part at nodes.lkt:10270:65>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.other_part
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Other_Part (Node => Self);
--# end
--# expr-done 6838
Var_Expr_195 := Fld_1;
if Var_Expr_195 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6839 '<FieldAccess for local_config_pragmas at nodes.lkt:10270:65>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
      if Var_Expr_195.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.local_config_pragmas
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Local_Config_Pragmas (Node => Var_Expr_195.Node);
--# end
--# expr-done 6839
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2924;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
   If_Result_2 := Result_Var;
else
   --# expr-start 6846 '<If at nodes.lkt:10270:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10270'
--# expr-start 6841 '<IsA Subunit at nodes.lkt:10273:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
--# expr-start 6840 '<FieldAccess for body at nodes.lkt:10273:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Compilation_Unit_F_Body;
--# expr-done 6840
Is_A := Fld_6 /= null 
and then Fld_6.Kind in Ada_Subunit_Range; 
--# expr-done 6841
if Is_A then
--# expr-start 6844 '<FieldAccess for root_unit at nodes.lkt:10273:40>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
--# expr-start 6843 '<Cast to Subunit at nodes.lkt:10273:40>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
--# expr-start 6842 '<FieldAccess for body at nodes.lkt:10273:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Compilation_Unit_F_Body;
--# expr-done 6842
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6843
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld_4 := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Cast_Result);
--# end
--# expr-done 6844
Var_Expr_196 := Fld_4;
if Var_Expr_196 /= No_Entity_Compilation_Unit then
   --# scope-start
--# expr-start 6845 '<FieldAccess for sources_config_pragmas at nodes.lkt:10273:40>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10273'
      if Var_Expr_196.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld_5 := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Var_Expr_196.Node, Include_Other_Part => Include_Other_Part);
--# end
--# expr-done 6845
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2925;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result := Result_Var_1;
else
   If_Result := No_Bare_Pragma_Node_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 6846
   If_Result_2 := If_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6847
Local_Related_Units := If_Result_2; Inc_Ref (Local_Related_Units);
--# bind related_units Local_Related_Units
Concat_Result := Concat (Local_Current_Unit, Local_Related_Units); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6848
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6849
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1066;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1066;
                     Finalizer_Scope_2924;
                     Finalizer_Scope_2925;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Sources_Config_Pragmas;
--# end
--# property-start CompilationUnit.all_config_pragmas 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10282'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_All_Config_Pragmas
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1067 with Inline_Always;
      Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1067 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1067;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6853 '<Map at nodes.lkt:10282:55>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10282'
   --# expr-start 6850 '<FieldAccess for sources_config_pragmas at nodes.lkt:10283:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10283'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Self);
--# end
--# expr-done 6850
--# expr-start 6851 '<FieldAccess for external_config_pragmas at nodes.lkt:10283:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10283'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Compilation_Unit_P_External_Config_Pragmas (Node => Self);
--# expr-done 6851
Concat_Result := Concat (Fld, Fld_1); 
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Concat_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 6852 '<as_bare_entity at nodes.lkt:10284:18>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10284'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 6852
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := If_Result;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6853
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1067;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1067;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_All_Config_Pragmas;
--# end
--# property-start CompilationUnit.spark_config_pragma 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10288'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Spark_Config_Pragma
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1068 with Inline_Always;
      Neg : Integer;
Fld : Bare_Pragma_Node_Array_Access;
Fld_1 : Bare_Pragma_Node_Array_Access;
Concat_Result : Bare_Pragma_Node_Array_Access;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Get_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1068 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1068;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6861 '<at at nodes.lkt:10288:49>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10288'
--# expr-start 6860 '<Map at nodes.lkt:10288:49>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10288'
   --# expr-start 6854 '<FieldAccess for external_config_pragmas at nodes.lkt:10289:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10289'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_External_Config_Pragmas (Node => Self);
--# expr-done 6854
--# expr-start 6855 '<FieldAccess for sources_config_pragmas at nodes.lkt:10289:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10289'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.sources_config_pragmas
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Sources_Config_Pragmas (Node => Self, Include_Other_Part => False);
--# end
--# expr-done 6855
Concat_Result := Concat (Fld, Fld_1); 
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Pragma_Node_Array_Access := Concat_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
         --# expr-start 6858 '<FieldAccess for name_is at nodes.lkt:10291:41>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10291'
--# expr-start 6856 '<FieldAccess for id at nodes.lkt:10291:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10291'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Item_0.Pragma_Node_F_Id;
--# expr-done 6856
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6857 '<Symbol SPARK_Mode at nodes.lkt:10291:54>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10291'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 6857
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_2, Sym => Sym);
--# end
--# expr-done 6858
         if Fld_3 then
      --# expr-start 6859 '<as_bare_entity at nodes.lkt:10291:16>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10291'
Is_Null := Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Item_0); 
   If_Result := As_Entity;
end if;
--# expr-done 6859
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := If_Result;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6860
Neg := -1; 
Get_Result := Get (Self, Map_Result, Neg, True); 
--# expr-done 6861
         Property_Result := Get_Result;
   --# end
      Finalizer_Scope_1068;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1068;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Spark_Config_Pragma;
--# end
--# property-start CompilationUnit.config_pragmas 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10297'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Config_Pragmas
  (Node : Bare_Compilation_Unit
      ; Name : Symbol_Type
  )
   return Internal_Entity_Pragma_Node_Array_Access
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1069 with Inline_Always;
      Fld : Internal_Entity_Pragma_Node_Array_Access;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
            procedure Finalizer_Scope_1069 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1069;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6866 '<Map at nodes.lkt:10298:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10298'
   --# expr-start 6862 '<FieldAccess for all_config_pragmas at nodes.lkt:10298:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10298'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.all_config_pragmas
Fld := Libadalang.Implementation.Compilation_Unit_P_All_Config_Pragmas (Node => Self);
--# end
--# expr-done 6862
      declare
         Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Pragma_Node_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
         --# expr-start 6865 '<Eq at nodes.lkt:10298:49>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10298'
--# expr-start 6864 '<FieldAccess for name_symbol at nodes.lkt:10298:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10298'
--# expr-start 6863 '<FieldAccess for id at nodes.lkt:10298:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10298'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Item_0.Node.Pragma_Node_F_Id, Info => Item_0.Info);
--# expr-done 6863
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 6864
Is_Equal := Fld_2 = Name; 
--# expr-done 6865
         if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node := Item_0;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Pragma_Node_Array
           (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6866
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1069;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1069;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Config_Pragmas;
--# end
--# property-start CompilationUnit.stub_for 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10342'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Stub_For
  (Node : Bare_Compilation_Unit
      ; Su : Bare_Subunit
  )
   return Bare_Body_Stub
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind su Su
   Property_Result : Bare_Body_Stub;
      Fld : Bare_Body_Stub;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Stub_For,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Subunit,
                                      As_Bare_Subunit => Su);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Bare_Body_Stub;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 6867 '<FieldAccess for stub_for_impl at nodes.lkt:10342:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10342'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Stub_For_Impl (Node => Self, Su => Su);
--# expr-done 6867
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Bare_Body_Stub,
                        As_Bare_Body_Stub => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Compilation_Unit_P_Stub_For;
--# end
--# property-start CompilationUnit.has_private_view 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10350'
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Has_Private_View
  (Node : Bare_Compilation_Unit
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Compilation_Unit  := Bare_Compilation_Unit (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_1072 with Inline_Always;
      Local_Decl : Bare_Basic_Decl;
Var_Expr_197 : Bare_Library_Item;
Fld : Bare_Basic_Decl;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Fld_2 : Bare_Private_Node;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_4 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_5 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1072 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1072;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6878 '<Let at nodes.lkt:10351:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10351'
--# scope-start
--# expr-start 6868 '<FieldAccess for decl at nodes.lkt:10351:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10351'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 6868
Local_Decl := Fld; 
--# bind decl Local_Decl
--# expr-start 6870 '<Cast to LibraryItem at nodes.lkt:10355:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10355'
--# expr-start 6869 '<FieldAccess for body at nodes.lkt:10355:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10355'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Compilation_Unit_F_Body;
--# expr-done 6869
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Library_Item_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 6870
Var_Expr_197 := Cast_Result;
if Var_Expr_197 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6872 '<FieldAccess for as_bool at nodes.lkt:10355:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10355'
--# expr-start 6871 '<FieldAccess for has_private at nodes.lkt:10355:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10355'
      if Var_Expr_197 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_197.Library_Item_F_Has_Private;
--# expr-done 6871
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Private.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_Private_Node_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 6872
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
   --# expr-start 6874 '<Not at nodes.lkt:10360:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10360'
--# expr-start 6873 '<IsA BasePackageDecl, GenericPackageDecl at nodes.lkt:10360:17>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10360'
Is_A := Local_Decl /= null 
and then Local_Decl.Kind in Ada_Base_Package_Decl | Ada_Generic_Package_Decl_Range; 
--# expr-done 6873
Not_Val := not (Is_A); 
--# expr-done 6874
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6877 '<FieldAccess for has_private_part_parent at nodes.lkt:10364:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10364'
      if Origin = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6876 '<env_node at nodes.lkt:10364:44>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10364'
--# expr-start 6875 '<FieldAccess for children_env at nodes.lkt:10364:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10364'
      if Local_Decl = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Local_Decl);
--# expr-done 6875
Env_Node := AST_Envs.Env_Node (Fld_4); 
--# expr-done 6876
--# property-call-start AdaNode.has_private_part_parent
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_Private_Part_Parent (Node => Origin, Barrier => Env_Node);
--# end
--# expr-done 6877
   If_Result_1 := Fld_5;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 6878
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1072;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1072;
            raise;
      end;
   return Property_Result;
end Compilation_Unit_P_Has_Private_View;
--# end
   --
   --  Primitives for Bare_Component_Clause
   --
      procedure Initialize_Fields_For_Component_Clause
        (Self : Bare_Component_Clause
         ; Component_Clause_F_Id : Bare_Identifier
         ; Component_Clause_F_Position : Bare_Expr
         ; Component_Clause_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Component_Clause_F_Id := Component_Clause_F_Id;
            Self.Component_Clause_F_Position := Component_Clause_F_Position;
            Self.Component_Clause_F_Range := Component_Clause_F_Range;
      end Initialize_Fields_For_Component_Clause;
   function Component_Clause_F_Id
     (Node : Bare_Component_Clause) return Bare_Identifier
   is
   begin
         return Node.Component_Clause_F_Id;
   end;
   function Component_Clause_F_Position
     (Node : Bare_Component_Clause) return Bare_Expr
   is
   begin
         return Node.Component_Clause_F_Position;
   end;
   function Component_Clause_F_Range
     (Node : Bare_Component_Clause) return Bare_Range_Spec
   is
   begin
         return Node.Component_Clause_F_Range;
   end;
--# property-start ComponentClause.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10375'
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Entry_Point
  (Node : Bare_Component_Clause
  )
   return Boolean
is
   Self : Bare_Component_Clause  := Bare_Component_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Component_Clause_P_Xref_Entry_Point;
--# end
--# property-start ComponentClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10378'
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Equation
  (Node : Bare_Component_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Clause  := Bare_Component_Clause (Node);
     Ent : Internal_Entity_Component_Clause :=
       Internal_Entity_Component_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1074 with Inline_Always;
      Local_Rep_Clause : Internal_Entity_Record_Rep_Clause;
Local_Record_Env : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Range_Spec;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1074 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Local_Record_Env);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1074;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6894 '<Let at nodes.lkt:10381:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10381'
--# scope-start
--# expr-start 6881 '<Cast to RecordRepClause at nodes.lkt:10381:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10381'
--# expr-start 6880 '<parent at nodes.lkt:10381:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10381'
--# expr-start 6879 '<parent at nodes.lkt:10381:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10381'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 6879
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 6880
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6881
Local_Rep_Clause := Cast_Result; 
--# bind rep_clause Local_Rep_Clause
--# expr-start 6893 '<Let at nodes.lkt:10384:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10384'
--# scope-start
--# expr-start 6883 '<FieldAccess for designated_env_no_overloading at nodes.lkt:10384:26>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10384'
--# expr-start 6882 '<FieldAccess for name at nodes.lkt:10384:26>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10384'
      if Local_Rep_Clause.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Local_Rep_Clause.Node.Record_Rep_Clause_F_Name, Info => Local_Rep_Clause.Info);
--# expr-done 6882
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 6883
Local_Record_Env := Fld_3; Inc_Ref (Local_Record_Env);
--# bind record_env Local_Record_Env
--# expr-start 6892 '<And at nodes.lkt:10386:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10386'
--# expr-start 6889 '<And at nodes.lkt:10386:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10386'
--# expr-start 6886 '<bind at nodes.lkt:10389:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10389'
Bound_Env := Local_Record_Env; Inc_Ref (Bound_Env);
--# expr-start 6885 '<FieldAccess for xref_equation at nodes.lkt:10391:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10391'
--# expr-start 6884 '<FieldAccess for id at nodes.lkt:10391:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10391'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Component_Clause_F_Id, Info => Ent.Info);
--# expr-done 6884
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 6885
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6886
--# expr-start 6888 '<FieldAccess for sub_equation at nodes.lkt:10393:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10393'
--# expr-start 6887 '<FieldAccess for position at nodes.lkt:10393:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10393'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Clause_F_Position, Info => Ent.Info);
--# expr-done 6887
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 6888
And_Pred := Create_And (Dyn_Var_Bind_Result, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10386:9") else null)); 
--# expr-done 6889
--# expr-start 6891 '<FieldAccess for sub_equation at nodes.lkt:10393:48>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10393'
--# expr-start 6890 '<FieldAccess for range at nodes.lkt:10393:48>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10393'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Component_Clause_F_Range, Info => Ent.Info);
--# expr-done 6890
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_8.Info);
--# end
--# expr-done 6891
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10386:9") else null)); 
--# expr-done 6892
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6893
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6894
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1074;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1074;
            raise;
      end;
   return Property_Result;
end Component_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Component_Def
   --
      procedure Initialize_Fields_For_Component_Def
        (Self : Bare_Component_Def
         ; Component_Def_F_Has_Aliased : Bare_Aliased_Node
         ; Component_Def_F_Has_Constant : Bare_Constant_Node
         ; Component_Def_F_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Component_Def_F_Has_Aliased := Component_Def_F_Has_Aliased;
            Self.Component_Def_F_Has_Constant := Component_Def_F_Has_Constant;
            Self.Component_Def_F_Type_Expr := Component_Def_F_Type_Expr;
      end Initialize_Fields_For_Component_Def;
   function Component_Def_F_Has_Aliased
     (Node : Bare_Component_Def) return Bare_Aliased_Node
   is
   begin
         return Node.Component_Def_F_Has_Aliased;
   end;
   function Component_Def_F_Has_Constant
     (Node : Bare_Component_Def) return Bare_Constant_Node
   is
   begin
         return Node.Component_Def_F_Has_Constant;
   end;
   function Component_Def_F_Type_Expr
     (Node : Bare_Component_Def) return Bare_Type_Expr
   is
   begin
         return Node.Component_Def_F_Type_Expr;
   end;
--# property-start ComponentDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10404'
pragma Warnings (Off, "is not referenced");
 function Component_Def_P_Xref_Equation
  (Node : Bare_Component_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Def  := Bare_Component_Def (Node);
     Ent : Internal_Entity_Component_Def :=
       Internal_Entity_Component_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1075 with Inline_Always;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1075 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1075;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6896 '<FieldAccess for sub_equation at nodes.lkt:10404:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10404'
--# expr-start 6895 '<FieldAccess for type_expr at nodes.lkt:10404:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10404'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Component_Def_F_Type_Expr, Info => Ent.Info);
--# expr-done 6895
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 6896
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1075;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1075;
            raise;
      end;
   return Property_Result;
end Component_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Constant_Node
   --
--# property-start '[dispatcher]Constant.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Constant_Node_P_As_Bool
  (Node : Bare_Constant_Node
  )
   return Boolean
is
   Self : Bare_Constant_Node  := Bare_Constant_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Constant_Node (Self.Kind) is
               when Ada_Constant_Absent =>
                  --# property-call-start Constant.Absent.as_bool
                  Property_Result := Constant_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Constant_Present =>
                  --# property-call-start Constant.Present.as_bool
                  Property_Result := Constant_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Constant_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Absent
   --
--# property-start Constant.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10409'
pragma Warnings (Off, "is not referenced");
 function Constant_Absent_P_As_Bool
  (Node : Bare_Constant_Absent
  )
   return Boolean
is
   Self : Bare_Constant_Absent  := Bare_Constant_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constant_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Present
   --
--# property-start Constant.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10409'
pragma Warnings (Off, "is not referenced");
 function Constant_Present_P_As_Bool
  (Node : Bare_Constant_Present
  )
   return Boolean
is
   Self : Bare_Constant_Present  := Bare_Constant_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constant_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constraint
   --
--# property-start Constraint.subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10415'
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Subtype
  (Node : Bare_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constraint  := Bare_Constraint (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Subtype_Indication;
As_Entity : Internal_Entity_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6902 '<bind at nodes.lkt:10416:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10416'
--# expr-start 6897 '<FieldAccess for origin_node at nodes.lkt:10416:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6897
Bound_Origin := Fld; 
--# expr-start 6901 '<FieldAccess for designated_type at nodes.lkt:10418:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10418'
--# expr-start 6900 '<as_entity at nodes.lkt:10418:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10418'
--# expr-start 6899 '<Cast to SubtypeIndication at nodes.lkt:10418:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10418'
--# expr-start 6898 '<parent at nodes.lkt:10418:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10418'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 6898
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Fld_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 6899
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subtype_Indication;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 6900
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => If_Result.Node, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 6901
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 6902
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constraint_P_Subtype;
--# end
--# property-start Constraint.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10422'
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Is_Static
  (Node : Bare_Constraint
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constraint  := Bare_Constraint (Node);
     Ent : Internal_Entity_Constraint :=
       Internal_Entity_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Range_Constraint;
Match_1 : Internal_Entity_Composite_Constraint;
Match_0_1 : Internal_Entity_Subtype_Indication;
Match_1_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Match_2_1 : Internal_Entity_Digits_Constraint;
Var_Expr_2 : Internal_Entity_Range_Spec;
Match_3 : Internal_Entity_Delta_Constraint;
Var_Expr_3 : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Bare_Assoc_List;
Fld_5 : Internal_Entity_Info;
Item_1 : Internal_Entity_Basic_Assoc;
Bare_Item_1 : Bare_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Composite_Constraint_Assoc;
Fld_6 : Internal_Entity;
Fld_7 : Boolean;
Fld_8 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Quantifier_Result : Boolean;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Fld_11 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Boolean;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Quantifier_Result_1 : Boolean;
Fld_14 : Boolean;
If_Result_2 : Boolean;
Fld_15 : Internal_Entity_Range_Spec;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
Fld_18 : Internal_Entity_Range_Spec;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_1 : Boolean;
Match_Prefix_1 : Internal_Entity_Constraint;
Cast_Result_4 : Internal_Entity_Range_Constraint;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_5 : Internal_Entity_Composite_Constraint;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_6 : Internal_Entity_Digits_Constraint;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_7 : Internal_Entity_Delta_Constraint;
Let_Result_6 : Boolean;
Scope_Result_8 : Boolean;
Match_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6925 '<Match at nodes.lkt:10422:29>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10422'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Ent; 
case Ada_Constraint (Match_Prefix_1.Node.Kind) is
         when Ada_Range_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Range_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_4; 
--# bind rc Match_0
--# expr-start 6905 '<FieldAccess for is_static_expr at nodes.lkt:10423:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10423'
--# expr-start 6904 '<FieldAccess for range at nodes.lkt:10423:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10423'
--# expr-start 6903 '<FieldAccess for range at nodes.lkt:10423:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10423'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Match_0.Node.Range_Constraint_F_Range, Info => Match_0.Info);
--# expr-done 6903
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 6904
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 6905
Let_Result_3 := Fld_2; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Composite_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Composite_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_5; 
--# bind cc Match_1
--# expr-start 6918 '<If at nodes.lkt:10424:41>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10424'
--# expr-start 6906 '<FieldAccess for is_index_constraint at nodes.lkt:10424:44>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10424'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld_14 := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 6906
if Fld_14 then
   --# expr-start 6913 '<AllQuantifier at nodes.lkt:10424:74>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10424'
   --# expr-start 6907 '<FieldAccess for constraints at nodes.lkt:10424:74>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10424'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Match_1.Node.Composite_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 6907
Fld_4 := Fld_3.Node;
      Quantifier_Result := True;
      if Fld_4 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_4;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind c Item_1
      --# expr-start 6912 '<Match at nodes.lkt:10425:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10425'
--# expr-start 6909 '<FieldAccess for constraint_expr at nodes.lkt:10425:26>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10425'
--# expr-start 6908 '<Cast to CompositeConstraintAssoc at nodes.lkt:10425:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10425'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Composite_Constraint_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint_Assoc;
   end if;
--# expr-done 6908
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity (Node => Cast_Result.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Cast_Result.Info);
--# expr-done 6909
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_6; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_1; 
--# bind st Match_0_1
--# expr-start 6910 '<FieldAccess for is_static_subtype at nodes.lkt:10426:47>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10426'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_7 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0_1.Info);
--# end
--# expr-done 6910
Let_Result := Fld_7; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_2; 
--# bind e Match_1_1
--# expr-start 6911 '<FieldAccess for is_static_expr at nodes.lkt:10427:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10427'
      if Match_1_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_8 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Match_1_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1_1.Info);
--# end
--# expr-done 6911
Let_Result_1 := Fld_8; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 6912
      Quantifier_Result := Match_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 6913
   If_Result_2 := Quantifier_Result;
else
   --# expr-start 6917 '<AllQuantifier at nodes.lkt:10430:16>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10430'
   --# expr-start 6914 '<FieldAccess for constraints at nodes.lkt:10430:16>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10430'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Assoc_List (Node => Match_1.Node.Composite_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 6914
Fld_10 := Fld_9.Node;
      Quantifier_Result_1 := True;
      if Fld_10 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_10;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_0); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind c Item_0
      --# expr-start 6916 '<FieldAccess for is_static_expr at nodes.lkt:10430:42>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10430'
--# expr-start 6915 '<FieldAccess for expr at nodes.lkt:10430:42>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10430'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 6915
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_13 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_12.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_12.Info);
--# end
--# expr-done 6916
      Quantifier_Result_1 := Fld_13;
   --# end
               exit when not Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 6917
   If_Result_2 := Quantifier_Result_1;
end if;
--# expr-done 6918
Let_Result_4 := If_Result_2; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Digits_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Digits_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2_1 := Cast_Result_6; 
--# bind dc Match_2_1
--# expr-start 6919 '<FieldAccess for range at nodes.lkt:10431:38>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10431'
      if Match_2_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Range_Spec (Node => Match_2_1.Node.Digits_Constraint_F_Range, Info => Match_2_1.Info);
--# expr-done 6919
Var_Expr_2 := Fld_15;
if Var_Expr_2 /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr_2
--# expr-start 6921 '<FieldAccess for is_static_expr at nodes.lkt:10432:24>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10432'
--# expr-start 6920 '<FieldAccess for range at nodes.lkt:10432:24>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10432'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Range_Spec_F_Range, Info => Var_Expr_2.Info);
--# expr-done 6920
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_17 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 6921
Scope_Result_3 := Fld_17;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := True;
end if;
Let_Result_5 := Result_Var; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Delta_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Delta_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_7; 
--# bind dc Match_3
--# expr-start 6922 '<FieldAccess for range at nodes.lkt:10434:37>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10434'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Range_Spec (Node => Match_3.Node.Delta_Constraint_F_Range, Info => Match_3.Info);
--# expr-done 6922
Var_Expr_3 := Fld_18;
if Var_Expr_3 /= No_Entity_Range_Spec then
   --# scope-start
   --# bind range Var_Expr_3
--# expr-start 6924 '<FieldAccess for is_static_expr at nodes.lkt:10435:24>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10435'
--# expr-start 6923 '<FieldAccess for range at nodes.lkt:10435:24>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10435'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Var_Expr_3.Node.Range_Spec_F_Range, Info => Var_Expr_3.Info);
--# expr-done 6923
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_20 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_19.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_19.Info);
--# end
--# expr-done 6924
Scope_Result_4 := Fld_20;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := True;
end if;
Let_Result_6 := Result_Var_1; 
--# end
Scope_Result_8 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_8; 
end case;
--# expr-done 6925
         Property_Result := Match_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Constraint_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Composite_Constraint
   --
      procedure Initialize_Fields_For_Composite_Constraint
        (Self : Bare_Composite_Constraint
         ; Composite_Constraint_F_Constraints : Bare_Assoc_List
        ) is
      begin
            Self.Composite_Constraint_F_Constraints := Composite_Constraint_F_Constraints;
      end Initialize_Fields_For_Composite_Constraint;
   function Composite_Constraint_F_Constraints
     (Node : Bare_Composite_Constraint) return Bare_Assoc_List
   is
   begin
         return Node.Composite_Constraint_F_Constraints;
   end;
--# property-start CompositeConstraint.is_index_constraint 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10449'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Is_Index_Constraint
  (Node : Bare_Composite_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 6927 '<FieldAccess for is_array_type at nodes.lkt:10449:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10449'
--# expr-start 6926 '<FieldAccess for subtype at nodes.lkt:10449:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6926
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Type (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 6927
if Fld_1 then
   If_Result_1 := True;
else
--# expr-start 6929 '<FieldAccess for is_access_type at nodes.lkt:10450:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10450'
--# expr-start 6928 '<FieldAccess for subtype at nodes.lkt:10450:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10450'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_2 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6928
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld_2.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6929
if Fld_3 then
   --# expr-start 6932 '<FieldAccess for is_array_type at nodes.lkt:10450:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10450'
--# expr-start 6931 '<FieldAccess for accessed_type at nodes.lkt:10450:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10450'
--# expr-start 6930 '<FieldAccess for subtype at nodes.lkt:10450:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10450'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_4 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6930
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_4.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_4.Info);
--# end
--# expr-done 6931
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Type (Node => Fld_5.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_5.Info);
--# end
--# expr-done 6932
   If_Result := Fld_6;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Is_Index_Constraint;
--# end
--# property-start CompositeConstraint.complete_item_weight 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10454'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Complete_Item_Weight
  (Node : Bare_Composite_Constraint
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
            procedure Finalizer_Scope_1082 with Inline_Always;
      Var_Expr_198 : Internal_Entity_Enum_Literal_Decl;
Fld : Integer;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_4 : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Result_Var : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Integer;
            procedure Finalizer_Scope_1082 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1082;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6941 '<If at nodes.lkt:10457:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10457'
--# expr-start 6939 '<AnyQuantifier at nodes.lkt:10457:12>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10457'
   --# expr-start 6934 '<FieldAccess for discriminants_list at nodes.lkt:10457:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10457'
--# expr-start 6933 '<FieldAccess for subtype at nodes.lkt:10457:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10457'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_1 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6933
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 6934
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind td Item_0
      --# expr-start 6938 '<Eq at nodes.lkt:10458:21>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10458'
--# expr-start 6935 '<FieldAccess for formal_type at nodes.lkt:10458:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10458'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Item_0.Node, Origin => Origin, E_Info => Item_0.Info);
--# end
--# expr-done 6935
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 6936 '<Cast to EnumLiteralDecl at nodes.lkt:10458:41>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10458'
Cast_Expr := Item; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 6936
Var_Expr_198 := Cast_Result;
if Var_Expr_198 /= No_Entity_Enum_Literal_Decl then
   --# scope-start
--# expr-start 6937 '<FieldAccess for enum_type at nodes.lkt:10458:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10458'
      if Var_Expr_198.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Var_Expr_198.Node, E_Info => Var_Expr_198.Info);
--# end
--# expr-done 6937
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Decl;
end if;
      Cast_Result_2 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 6938
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 6939
if Quantifier_Result then
   If_Result := 100;
else
   --# expr-start 6940 '<Super at nodes.lkt:10459:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10459'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.complete_item_weight
Fld := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Item, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6940
   If_Result := Fld;
end if;
--# expr-done 6941
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1082;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1082;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Complete_Item_Weight;
--# end
--# property-start CompositeConstraint.is_discriminant_constraint 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10463'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Is_Discriminant_Constraint
  (Node : Bare_Composite_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 6943 '<Not at nodes.lkt:10463:46>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10463'
--# expr-start 6942 '<FieldAccess for is_index_constraint at nodes.lkt:10463:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10463'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6942
Not_Val := not (Fld); 
--# expr-done 6943
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Is_Discriminant_Constraint;
--# end
--# property-start CompositeConstraint.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10466'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Xref_Equation
  (Node : Bare_Composite_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1084 with Inline_Always;
            procedure Finalizer_Scope_2943 with Inline_Always;
            procedure Finalizer_Scope_2944 with Inline_Always;
            procedure Finalizer_Scope_2945 with Inline_Always;
            procedure Finalizer_Scope_2946 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Bare_Base_Id;
Local_Ex : Internal_Entity;
Var_Expr_4 : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Item_2 : Internal_Entity_Basic_Assoc;
Bare_Item_2 : Bare_Basic_Assoc;
Index_3 : Integer;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Composite_Constraint_Assoc;
Fld_4 : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_8 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Assoc_List;
Fld_11 : Internal_Param_Match_Array_Access;
Fld_12 : Internal_Single_Actual;
Fld_13 : Internal_Entity_Basic_Assoc;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Var;
Fld_16 : Internal_Entity_Defining_Name;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_19 : Internal_Single_Actual;
Fld_20 : Internal_Entity_Basic_Assoc;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_27 : Internal_Single_Actual;
Fld_28 : Bare_Base_Id;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Defining_Name;
Fld_31 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_32 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1084 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1084;
            procedure Finalizer_Scope_2943 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2943;
            procedure Finalizer_Scope_2944 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2944;
            procedure Finalizer_Scope_2945 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2945;
            procedure Finalizer_Scope_2946 is
            begin
                     Dec_Ref (Bind_Result_2);
            end Finalizer_Scope_2946;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 6990 '<Let at nodes.lkt:10467:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10467'
--# scope-start
--# expr-start 6944 '<FieldAccess for subtype at nodes.lkt:10467:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10467'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6944
Local_Typ := Fld; 
--# bind typ Local_Typ
--# expr-start 6989 '<If at nodes.lkt:10469:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10469'
--# expr-start 6945 '<FieldAccess for is_index_constraint at nodes.lkt:10469:12>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10469'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.is_index_constraint
Fld_32 := Libadalang.Implementation.Composite_Constraint_P_Is_Index_Constraint (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6945
if Fld_32 then
   --# expr-start 6958 '<LogicAll at nodes.lkt:10469:44>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10469'
--# expr-start 6957 '<Map at nodes.lkt:10469:44>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10469'
   --# expr-start 6946 '<FieldAccess for constraints at nodes.lkt:10469:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10469'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Composite_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 6946
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
         Index_3 := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null := Bare_Item_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_2); 
   If_Result := As_Entity;
end if;
                  Item_2 := If_Result; 
   --# scope-start
               --# bind c Item_2
               --# bind i Index_3
      --# expr-start 6956 '<Let at nodes.lkt:10471:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10471'
--# scope-start
--# expr-start 6948 '<FieldAccess for constraint_expr at nodes.lkt:10471:26>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10471'
--# expr-start 6947 '<Cast to CompositeConstraintAssoc at nodes.lkt:10471:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10471'
Cast_Expr := Item_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Composite_Constraint_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint_Assoc;
   end if;
--# expr-done 6947
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity (Node => Cast_Result.Node.Composite_Constraint_Assoc_F_Constraint_Expr, Info => Cast_Result.Info);
--# expr-done 6948
Local_Ex := Fld_4; 
--# bind ex Local_Ex
--# expr-start 6955 '<And at nodes.lkt:10491:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10491'
--# expr-start 6949 '<Cast to Expr at nodes.lkt:10491:17>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10491'
Cast_Expr_1 := Local_Ex; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Expr
   then
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Expr;
   end if;
--# expr-done 6949
Var_Expr_4 := Cast_Result_1;
if Var_Expr_4 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_4
--# expr-start 6953 '<Bind at nodes.lkt:10492:28>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10492'
--# expr-start 6950 '<FieldAccess for expected_type_var at nodes.lkt:10492:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10492'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_4.Node);
--# expr-done 6950
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 6952 '<FieldAccess for base_subtype at nodes.lkt:10492:55>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10492'
--# expr-start 6951 '<FieldAccess for index_type at nodes.lkt:10492:55>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10492'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Local_Typ.Node, Dim => Index_3, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 6951
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6952
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10492:28") else null)); 
--# expr-done 6953
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2944;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10492:102") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6954 '<FieldAccess for sub_equation at nodes.lkt:10493:23>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10493'
      if Local_Ex.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Ex.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Ex.Info);
--# end
--# expr-done 6954
And_Pred := Create_And (Result_Var, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10491:17") else null)); 
--# expr-done 6955
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6956
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2943;
               Index_3 := Index_3 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6957
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10469:44") else null)); 
--# expr-done 6958
   If_Result_1 := Logic_Boolean_Op;
else
   --# expr-start 6988 '<LogicAll at nodes.lkt:10497:13>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10497'
--# expr-start 6987 '<Map at nodes.lkt:10497:13>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10497'
   --# expr-start 6961 '<FieldAccess for match_formals at nodes.lkt:10497:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10497'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 6959 '<FieldAccess for discriminants_list at nodes.lkt:10498:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10498'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 6959
--# expr-start 6960 '<FieldAccess for constraints at nodes.lkt:10498:43>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10498'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Composite_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 6960
--# property-call-start AdaNode.match_formals
Fld_11 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_9, Params => Fld_10, Is_Dottable_Subp => False);
--# end
--# expr-done 6961
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_11;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_0
      --# expr-start 6986 '<And at nodes.lkt:10500:25>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10500'
--# expr-start 6979 '<And at nodes.lkt:10501:21>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10501'
--# expr-start 6974 '<And at nodes.lkt:10502:25>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6969 '<Bind at nodes.lkt:10502:25>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6965 '<FieldAccess for expected_type_var at nodes.lkt:10502:29>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6964 '<FieldAccess for expr at nodes.lkt:10502:29>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6963 '<FieldAccess for assoc at nodes.lkt:10502:29>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6962 '<FieldAccess for actual at nodes.lkt:10502:29>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
Fld_12 := Item_0.Actual;
--# expr-done 6962
Fld_13 := Fld_12.Assoc;
--# expr-done 6963
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 6964
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_14.Node);
--# expr-done 6965
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 6968 '<FieldAccess for formal_type at nodes.lkt:10502:73>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6967 '<FieldAccess for formal_decl at nodes.lkt:10502:73>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6966 '<FieldAccess for formal at nodes.lkt:10502:73>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
Fld_16 := Item_0.Formal;
--# expr-done 6966
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_17 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 6967
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_18 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_17.Node, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 6968
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_15, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10502:25") else null)); 
--# expr-done 6969
--# expr-start 6973 '<FieldAccess for sub_equation at nodes.lkt:10502:116>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6972 '<FieldAccess for expr at nodes.lkt:10502:116>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6971 '<FieldAccess for assoc at nodes.lkt:10502:116>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
--# expr-start 6970 '<FieldAccess for actual at nodes.lkt:10502:116>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10502'
Fld_19 := Item_0.Actual;
--# expr-done 6970
Fld_20 := Fld_19.Assoc;
--# expr-done 6971
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 6972
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_21.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_21.Info);
--# end
--# expr-done 6973
And_Pred_1 := Create_And (Bind_Result_1, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10502:25") else null)); 
--# expr-done 6974
--# expr-start 6978 '<FieldAccess for matches_expected_formal_type at nodes.lkt:10503:27>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10503'
--# expr-start 6977 '<FieldAccess for expr at nodes.lkt:10503:27>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10503'
--# expr-start 6976 '<FieldAccess for assoc at nodes.lkt:10503:27>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10503'
--# expr-start 6975 '<FieldAccess for actual at nodes.lkt:10503:27>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10503'
Fld_23 := Item_0.Actual;
--# expr-done 6975
Fld_24 := Fld_23.Assoc;
--# expr-done 6976
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_24.Node, E_Info => Fld_24.Info);
--# end
--# expr-done 6977
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_26 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_25.Node, Origin => Origin);
--# end
--# expr-done 6978
And_Pred_2 := Create_And (And_Pred_1, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10501:21") else null)); 
--# expr-done 6979
--# expr-start 6981 '<FieldAccess for name at nodes.lkt:10504:23>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10504'
--# expr-start 6980 '<FieldAccess for actual at nodes.lkt:10504:23>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10504'
Fld_27 := Item_0.Actual;
--# expr-done 6980
Fld_28 := Fld_27.Name;
--# expr-done 6981
Var_Expr_1 := Fld_28;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 6985 '<Bind at nodes.lkt:10505:31>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10505'
--# expr-start 6982 '<FieldAccess for ref_var at nodes.lkt:10505:35>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10505'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_1);
--# end
--# expr-done 6982
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 6984 '<FieldAccess for formal_decl at nodes.lkt:10505:51>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10505'
--# expr-start 6983 '<FieldAccess for formal at nodes.lkt:10505:51>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10505'
Fld_30 := Item_0.Formal;
--# expr-done 6983
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_31 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_30.Node, E_Info => Fld_30.Info);
--# end
--# expr-done 6984
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_29, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10505:31") else null)); 
--# expr-done 6985
Scope_Result_1 := Bind_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2946;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10505:89") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Create_And (And_Pred_2, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10500:25") else null)); 
--# expr-done 6986
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2945;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6987
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10497:13") else null)); 
--# expr-done 6988
   If_Result_1 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6989
Let_Result_1 := If_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6990
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1084;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1084;
                     Finalizer_Scope_2943;
                     Finalizer_Scope_2944;
                     Finalizer_Scope_2945;
                     Finalizer_Scope_2946;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Xref_Equation;
--# end
--# property-start CompositeConstraint.discriminant_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10514'
pragma Warnings (Off, "is not referenced");
 function Composite_Constraint_P_Discriminant_Params
  (Node : Bare_Composite_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Composite_Constraint  := Bare_Composite_Constraint (Node);
     Ent : Internal_Entity_Composite_Constraint :=
       Internal_Entity_Composite_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1085 with Inline_Always;
            procedure Finalizer_Scope_2947 with Inline_Always;
            procedure Finalizer_Scope_2949 with Inline_Always;
      Local_Discrs : Internal_Param_Actual_Array_Access;
Local_Ds : Internal_Entity_Discriminant_Spec;
Var_Expr_2 : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity_Discriminant_Spec;
Fld_2 : Internal_Entity_Defining_Name_List;
Fld_3 : Bare_Defining_Name_List;
Fld_4 : Internal_Entity_Info;
Item_1 : Internal_Entity_Defining_Name;
Bare_Item_1 : Bare_Defining_Name;
Fld_5 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Param_Actual_Array_Access;
Let_Result : Internal_Param_Actual_Array_Access;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Fld_6 : Internal_Entity_Assoc_List;
Index_4 : Integer;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Internal_Entity_Defining_Name;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result : Internal_Param_Actual_Array_Access;
Result_Var : Internal_Param_Actual_Array_Access;
Let_Result_1 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1085 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Discrs);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1085;
            procedure Finalizer_Scope_2947 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2947;
            procedure Finalizer_Scope_2949 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2949;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7007 '<Let at nodes.lkt:10516:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10516'
--# scope-start
--# expr-start 6999 '<Mapcat at nodes.lkt:10516:22>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10516'
   --# expr-start 6992 '<FieldAccess for discriminants_list at nodes.lkt:10516:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10516'
--# expr-start 6991 '<FieldAccess for subtype at nodes.lkt:10516:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10516'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6991
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 6992
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_0
      --# expr-start 6998 '<Let at nodes.lkt:10518:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10518'
--# scope-start
--# expr-start 6993 '<Cast to DiscriminantSpec at nodes.lkt:10518:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10518'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Discriminant_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 6993
Local_Ds := Cast_Result; 
--# bind ds Local_Ds
--# expr-start 6997 '<Map at nodes.lkt:10520:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10520'
   --# expr-start 6994 '<FieldAccess for ids at nodes.lkt:10520:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10520'
      if Local_Ds.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Defining_Name_List (Node => Local_Ds.Node.Discriminant_Spec_F_Ids, Info => Local_Ds.Info);
--# expr-done 6994
Fld_3 := Fld_2.Node;
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result := Create_Internal_Param_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_3;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind i Item_1
      --# expr-start 6996 '<New ParamActual at nodes.lkt:10521:28>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10521'
--# expr-start 6995 '<FieldAccess for default_expr at nodes.lkt:10521:56>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10521'
      if Local_Ds.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Local_Ds.Node.Discriminant_Spec_F_Default_Expr, Info => Local_Ds.Info);
--# expr-done 6995
New_Struct := (Actual => Fld_5, Param => Item_1); 
--# expr-done 6996
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 6997
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6998
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2947;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 6999
Local_Discrs := Map_Result_1; Inc_Ref (Local_Discrs);
--# bind discrs Local_Discrs
--# expr-start 7000 '<FieldAccess for constraints at nodes.lkt:10527:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10527'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Composite_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 7000
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind c Var_Expr_2
--# expr-start 7006 '<Map at nodes.lkt:10528:20>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10528'
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_4 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Local_Discrs;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_3
               --# bind i Index_4
      --# expr-start 7005 '<New ParamActual at nodes.lkt:10529:28>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10529'
--# expr-start 7003 '<FieldAccess for actual_for_param_at at nodes.lkt:10530:44>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10530'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7001 '<FieldAccess for param at nodes.lkt:10530:66>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10530'
Fld_8 := Item_3.Param;
--# expr-done 7001
--# expr-start 7002 '<FieldAccess for actual at nodes.lkt:10530:79>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10530'
Fld_9 := Item_3.Actual;
--# expr-done 7002
--# property-call-start AssocList.actual_for_param_at
Fld_10 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Var_Expr_2.Node, Param => Fld_8, Pos => Index_4, Default_Expr => Fld_9, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7003
--# expr-start 7004 '<FieldAccess for param at nodes.lkt:10530:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10530'
Fld_7 := Item_3.Param;
--# expr-done 7004
New_Struct_1 := (Actual => Fld_10, Param => Fld_7); 
--# expr-done 7005
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_1;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_4 := Index_4 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 7006
Scope_Result := Map_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2949;
   Result_Var := Scope_Result;
else
   Result_Var := Local_Discrs;
end if;
      Inc_Ref (Result_Var);
Let_Result_1 := Result_Var; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7007
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1085;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1085;
                     Finalizer_Scope_2947;
                     Finalizer_Scope_2949;
            raise;
      end;
   return Property_Result;
end Composite_Constraint_P_Discriminant_Params;
--# end
   --
   --  Primitives for Bare_Delta_Constraint
   --
      procedure Initialize_Fields_For_Delta_Constraint
        (Self : Bare_Delta_Constraint
         ; Delta_Constraint_F_Delta : Bare_Expr
         ; Delta_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Delta_Constraint_F_Delta := Delta_Constraint_F_Delta;
            Self.Delta_Constraint_F_Range := Delta_Constraint_F_Range;
      end Initialize_Fields_For_Delta_Constraint;
   function Delta_Constraint_F_Delta
     (Node : Bare_Delta_Constraint) return Bare_Expr
   is
   begin
         return Node.Delta_Constraint_F_Delta;
   end;
   function Delta_Constraint_F_Range
     (Node : Bare_Delta_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Delta_Constraint_F_Range;
   end;
--# property-start DeltaConstraint.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10544'
pragma Warnings (Off, "is not referenced");
 function Delta_Constraint_P_Xref_Equation
  (Node : Bare_Delta_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delta_Constraint  := Bare_Delta_Constraint (Node);
     Ent : Internal_Entity_Delta_Constraint :=
       Internal_Entity_Delta_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1086 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Logic_Equation;
Fld_9 : Bare_Range_Spec;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1086 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1086;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7022 '<And at nodes.lkt:10544:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10544'
--# expr-start 7016 '<And at nodes.lkt:10544:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10544'
--# expr-start 7013 '<And at nodes.lkt:10544:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10544'
--# expr-start 7010 '<FieldAccess for universal_real_bind at nodes.lkt:10547:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10547'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7009 '<FieldAccess for expected_type_var at nodes.lkt:10547:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10547'
--# expr-start 7008 '<FieldAccess for delta at nodes.lkt:10547:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10547'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Delta, Info => Ent.Info);
--# expr-done 7008
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 7009
--# property-call-start AdaNode.universal_real_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 7010
--# expr-start 7012 '<FieldAccess for sub_equation at nodes.lkt:10548:11>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10548'
--# expr-start 7011 '<FieldAccess for delta at nodes.lkt:10548:11>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10548'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Delta, Info => Ent.Info);
--# expr-done 7011
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 7012
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10544:37") else null)); 
--# expr-done 7013
--# expr-start 7015 '<FieldAccess for matches_expected_type at nodes.lkt:10548:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10548'
--# expr-start 7014 '<FieldAccess for delta at nodes.lkt:10548:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10548'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Delta, Info => Ent.Info);
--# expr-done 7014
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 7015
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10544:37") else null)); 
--# expr-done 7016
--# expr-start 7021 '<If at nodes.lkt:10549:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10549'
--# expr-start 7018 '<IsNull at nodes.lkt:10549:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10549'
--# expr-start 7017 '<FieldAccess for range at nodes.lkt:10549:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10549'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Delta_Constraint_F_Range;
--# expr-done 7017
Is_Null := Fld_9 = null; 
--# expr-done 7018
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10549:36") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 7020 '<FieldAccess for sub_equation at nodes.lkt:10549:47>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10549'
--# expr-start 7019 '<FieldAccess for range at nodes.lkt:10549:47>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10549'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Delta_Constraint_F_Range, Info => Ent.Info);
--# expr-done 7019
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 7020
   If_Result := Fld_8;
end if;
      Inc_Ref (If_Result);
--# expr-done 7021
And_Pred_2 := Create_And (And_Pred_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10544:37") else null)); 
--# expr-done 7022
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1086;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1086;
            raise;
      end;
   return Property_Result;
end Delta_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Digits_Constraint
   --
      procedure Initialize_Fields_For_Digits_Constraint
        (Self : Bare_Digits_Constraint
         ; Digits_Constraint_F_Digits : Bare_Expr
         ; Digits_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Digits_Constraint_F_Digits := Digits_Constraint_F_Digits;
            Self.Digits_Constraint_F_Range := Digits_Constraint_F_Range;
      end Initialize_Fields_For_Digits_Constraint;
   function Digits_Constraint_F_Digits
     (Node : Bare_Digits_Constraint) return Bare_Expr
   is
   begin
         return Node.Digits_Constraint_F_Digits;
   end;
   function Digits_Constraint_F_Range
     (Node : Bare_Digits_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Digits_Constraint_F_Range;
   end;
--# property-start DigitsConstraint.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10560'
pragma Warnings (Off, "is not referenced");
 function Digits_Constraint_P_Xref_Equation
  (Node : Bare_Digits_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Digits_Constraint  := Bare_Digits_Constraint (Node);
     Ent : Internal_Entity_Digits_Constraint :=
       Internal_Entity_Digits_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1087 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Logic_Equation;
Fld_9 : Bare_Range_Spec;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1087 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1087;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7037 '<And at nodes.lkt:10560:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10560'
--# expr-start 7031 '<And at nodes.lkt:10560:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10560'
--# expr-start 7028 '<And at nodes.lkt:10560:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10560'
--# expr-start 7025 '<FieldAccess for universal_int_bind at nodes.lkt:10563:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10563'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7024 '<FieldAccess for expected_type_var at nodes.lkt:10563:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10563'
--# expr-start 7023 '<FieldAccess for digits at nodes.lkt:10563:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10563'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 7023
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 7024
--# property-call-start AdaNode.universal_int_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 7025
--# expr-start 7027 '<FieldAccess for sub_equation at nodes.lkt:10564:11>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10564'
--# expr-start 7026 '<FieldAccess for digits at nodes.lkt:10564:11>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10564'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 7026
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 7027
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10560:37") else null)); 
--# expr-done 7028
--# expr-start 7030 '<FieldAccess for matches_expected_type at nodes.lkt:10564:42>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10564'
--# expr-start 7029 '<FieldAccess for digits at nodes.lkt:10564:42>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10564'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 7029
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 7030
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10560:37") else null)); 
--# expr-done 7031
--# expr-start 7036 '<If at nodes.lkt:10565:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10565'
--# expr-start 7033 '<IsNull at nodes.lkt:10565:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10565'
--# expr-start 7032 '<FieldAccess for range at nodes.lkt:10565:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10565'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Digits_Constraint_F_Range;
--# expr-done 7032
Is_Null := Fld_9 = null; 
--# expr-done 7033
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10565:36") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 7035 '<FieldAccess for sub_equation at nodes.lkt:10565:47>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10565'
--# expr-start 7034 '<FieldAccess for range at nodes.lkt:10565:47>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10565'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Digits_Constraint_F_Range, Info => Ent.Info);
--# expr-done 7034
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 7035
   If_Result := Fld_8;
end if;
      Inc_Ref (If_Result);
--# expr-done 7036
And_Pred_2 := Create_And (And_Pred_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10560:37") else null)); 
--# expr-done 7037
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1087;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1087;
            raise;
      end;
   return Property_Result;
end Digits_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Range_Constraint
   --
      procedure Initialize_Fields_For_Range_Constraint
        (Self : Bare_Range_Constraint
         ; Range_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Range_Constraint_F_Range := Range_Constraint_F_Range;
      end Initialize_Fields_For_Range_Constraint;
   function Range_Constraint_F_Range
     (Node : Bare_Range_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Range_Constraint_F_Range;
   end;
--# property-start RangeConstraint.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10574'
pragma Warnings (Off, "is not referenced");
 function Range_Constraint_P_Xref_Equation
  (Node : Bare_Range_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Constraint  := Bare_Range_Constraint (Node);
     Ent : Internal_Entity_Range_Constraint :=
       Internal_Entity_Range_Constraint'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1088 with Inline_Always;
      Fld : Bare_Range_Spec;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1088 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1088;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7050 '<And at nodes.lkt:10575:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7046 '<And at nodes.lkt:10575:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7043 '<Bind at nodes.lkt:10575:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7040 '<FieldAccess for expected_type_var at nodes.lkt:10575:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7039 '<FieldAccess for range at nodes.lkt:10575:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7038 '<FieldAccess for range at nodes.lkt:10575:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Range_Constraint_F_Range;
--# expr-done 7038
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Range_Spec_F_Range;
--# expr-done 7039
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_1);
--# expr-done 7040
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7042 '<FieldAccess for base_subtype at nodes.lkt:10575:51>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7041 '<FieldAccess for subtype at nodes.lkt:10575:51>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.subtype
Fld_3 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7041
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 7042
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10575:9") else null)); 
--# expr-done 7043
--# expr-start 7045 '<FieldAccess for sub_equation at nodes.lkt:10575:86>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7044 '<FieldAccess for range at nodes.lkt:10575:86>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 7044
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_5.Info);
--# end
--# expr-done 7045
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10575:9") else null)); 
--# expr-done 7046
--# expr-start 7049 '<FieldAccess for matches_expected_type at nodes.lkt:10575:116>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7048 '<FieldAccess for range at nodes.lkt:10575:116>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
--# expr-start 7047 '<FieldAccess for range at nodes.lkt:10575:116>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10575'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 7047
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Fld_7.Node.Range_Spec_F_Range, Info => Fld_7.Info);
--# expr-done 7048
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 7049
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10575:9") else null)); 
--# expr-done 7050
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1088;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1088;
            raise;
      end;
   return Property_Result;
end Range_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Declarative_Part
   --
      procedure Initialize_Fields_For_Declarative_Part
        (Self : Bare_Declarative_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Declarative_Part_F_Decls := Declarative_Part_F_Decls;
      end Initialize_Fields_For_Declarative_Part;
   function Declarative_Part_F_Decls
     (Node : Bare_Declarative_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Declarative_Part_F_Decls;
   end;
--# property-start DeclarativePart.use_clauses_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10585'
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Use_Clauses_Envs
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1089 with Inline_Always;
            procedure Finalizer_Scope_2951 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_2 : Lexical_Env := Empty_Env;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1089 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1089;
            procedure Finalizer_Scope_2951 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2951;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7057 '<env_group at nodes.lkt:10585:42>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10585'
--# expr-start 7056 '<Map at nodes.lkt:10585:42>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10585'
   --# expr-start 7052 '<children at nodes.lkt:10585:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10585'
--# expr-start 7051 '<FieldAccess for decls at nodes.lkt:10585:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10585'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 7051
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 7052
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_0
         --# expr-start 7053 '<IsA UseClause at nodes.lkt:10586:52>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10586'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Use_Clause; 
--# expr-done 7053
         if Is_A then
      --# expr-start 7055 '<FieldAccess for used_envs at nodes.lkt:10586:16>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10586'
--# expr-start 7054 '<Cast to UseClause at nodes.lkt:10586:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10586'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 7054
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.used_envs
Fld_2 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 7055
         declare
            Item_To_Append : constant Lexical_Env := Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_2951;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7056
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 7057
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1089;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1089;
                     Finalizer_Scope_2951;
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Use_Clauses_Envs;
--# end
--# property-start DeclarativePart.spark_mode_pragma 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10591'
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Spark_Mode_Pragma
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1090 with Inline_Always;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1090 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1090;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7066 '<Cast to Pragma at nodes.lkt:10594:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10594'
--# expr-start 7065 '<Find at nodes.lkt:10594:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10594'
   --# expr-start 7060 '<Map at nodes.lkt:10594:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10594'
   --# expr-start 7058 '<FieldAccess for decls at nodes.lkt:10594:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10594'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 7058
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind decl Item_0
         --# expr-start 7059 '<IsA Pragma at nodes.lkt:10594:41>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10594'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Pragma_Node_Range; 
--# expr-done 7059
         Take_While_Ok := Is_A;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7060
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_1
      --# expr-start 7064 '<FieldAccess for name_is at nodes.lkt:10595:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10595'
--# expr-start 7062 '<FieldAccess for id at nodes.lkt:10595:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10595'
--# expr-start 7061 '<Cast to Pragma at nodes.lkt:10595:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10595'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 7061
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Cast_Result.Node.Pragma_Node_F_Id, Info => Cast_Result.Info);
--# expr-done 7062
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7063 '<Symbol SPARK_Mode at nodes.lkt:10595:50>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10595'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 7063
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 7064
      if Fld_4 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 7065
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Pragma_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_1 := No_Entity_Pragma_Node;
   end if;
--# expr-done 7066
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1090;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1090;
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Spark_Mode_Pragma;
--# end
--# property-start DeclarativePart.spark_mode_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10599'
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Spark_Mode_Aspect
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Declarative_Part  := Bare_Declarative_Part (Node);
     Ent : Internal_Entity_Declarative_Part :=
       Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
      Var_Expr_0 : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Aspect;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_3 : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_4 : Internal_Aspect;
Result_Var : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Fld_5 : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 7067 '<FieldAccess for spark_mode_pragma at nodes.lkt:10601:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10601'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.spark_mode_pragma
Fld := Libadalang.Implementation.Declarative_Part_P_Spark_Mode_Pragma (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7067
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind pragma Var_Expr_0
--# expr-start 7068 '<FieldAccess for as_aspect at nodes.lkt:10603:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10603'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.as_aspect
Fld_1 := Libadalang.Implementation.Pragma_Node_P_As_Aspect (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7068
Scope_Result := Fld_1;
   --# end
   Result_Var_2 := Scope_Result;
else
--# expr-start 7070 '<Cast to BasicDecl at nodes.lkt:10606:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10606'
--# expr-start 7069 '<parent at nodes.lkt:10606:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10606'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7069
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7070
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 7072 '<FieldAccess for get_aspect at nodes.lkt:10607:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10607'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7071 '<Symbol SPARK_Mode at nodes.lkt:10607:39>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10607'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 7071
--# property-call-start BasicDecl.get_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Var_Expr_1.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7072
Var_Expr := Fld_3;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var := Scope_Result_1;
else
   --# expr-start 7073 '<Super at nodes.lkt:10610:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10610'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.spark_mode_aspect
Fld_4 := Libadalang.Implementation.Ada_Node_P_Spark_Mode_Aspect (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7073
   Result_Var := Fld_4;
end if;
Scope_Result_2 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   --# expr-start 7074 '<Super at nodes.lkt:10611:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10611'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.spark_mode_aspect
Fld_5 := Libadalang.Implementation.Ada_Node_P_Spark_Mode_Aspect (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7074
   Result_Var_1 := Fld_5;
end if;
   Result_Var_2 := Result_Var_1;
end if;
         Property_Result := Result_Var_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Declarative_Part_P_Spark_Mode_Aspect;
--# end
   --
   --  Primitives for Bare_Private_Part
   --
      procedure Private_Part_Pre_Env_Actions
        (Self            : Bare_Private_Part;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_212 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:10641:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_213 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_214 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Private_Part
        (Self : Bare_Private_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Private_Part;
--# property-start PrivatePart.env_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10621'
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Env_Names
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1092 with Inline_Always;
            procedure Finalizer_Scope_2957 with Inline_Always;
            procedure Finalizer_Scope_2958 with Inline_Always;
      Var_Expr_0 : Bare_Base_Package_Decl;
Var_Expr_1 : String_Type;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Result_Var_1 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1092 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1092;
            procedure Finalizer_Scope_2957 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2957;
            procedure Finalizer_Scope_2958 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2958;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 7076 '<Cast to BasePackageDecl at nodes.lkt:10621:38>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10621'
--# expr-start 7075 '<parent at nodes.lkt:10621:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10621'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7075
   if Fld = null
      or else Fld.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7076
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pkg Var_Expr_0
--# expr-start 7077 '<FieldAccess for top_level_env_name at nodes.lkt:10622:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10622'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Var_Expr_0);
--# end
--# expr-done 7077
Var_Expr_1 := Fld_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_String then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 7080 '<ArrayLiteral at nodes.lkt:10623:23>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10623'
--# expr-start 7079 '<to_symbol at nodes.lkt:10623:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10623'
--# expr-start 7078 '<String at nodes.lkt:10623:32>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10623'
Str := Create_String (".__privatepart"); 
--# expr-done 7078
Concat_Result := Concat_String (Var_Expr_1, Str); 
Sym := String_To_Symbol (Self, Self.Unit.Context, Concat_Result); 
--# expr-done 7079
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 7080
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2958;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2957;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1092;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1092;
                     Finalizer_Scope_2957;
                     Finalizer_Scope_2958;
            raise;
      end;
   return Property_Result;
end Private_Part_P_Env_Names;
--# end
--# property-start PrivatePart.immediate_declarative_region 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10627'
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Immediate_Declarative_Region
  (Node : Bare_Private_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
     Ent : Internal_Entity_Private_Part :=
       Internal_Entity_Private_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1093 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1093 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1093;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7082 '<FieldAccess for immediate_declarative_region at nodes.lkt:10628:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10628'
--# expr-start 7081 '<FieldAccess for semantic_parent at nodes.lkt:10628:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10628'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7081
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.immediate_declarative_region'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Immediate_Declarative_Region (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7082
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1093;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1093;
            raise;
      end;
   return Property_Result;
end Private_Part_P_Immediate_Declarative_Region;
--# end
--# property-start PrivatePart.spark_mode_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10631'
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Spark_Mode_Aspect
  (Node : Bare_Private_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
     Ent : Internal_Entity_Private_Part :=
       Internal_Entity_Private_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
      Var_Expr_0 : Internal_Entity_Pragma_Node;
Var_Expr_199 : Internal_Entity_Package_Decl;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Fld_3 : Internal_Entity_Public_Part;
Fld_4 : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Result_Var : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 7083 '<FieldAccess for spark_mode_pragma at nodes.lkt:10633:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10633'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DeclarativePart.spark_mode_pragma
Fld := Libadalang.Implementation.Declarative_Part_P_Spark_Mode_Pragma (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7083
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind pragma Var_Expr_0
--# expr-start 7084 '<FieldAccess for as_aspect at nodes.lkt:10634:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10634'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.as_aspect
Fld_1 := Libadalang.Implementation.Pragma_Node_P_As_Aspect (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7084
Scope_Result := Fld_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 7086 '<Cast to PackageDecl at nodes.lkt:10637:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10637'
--# expr-start 7085 '<parent at nodes.lkt:10637:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10637'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7085
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Package_Decl;
   end if;
--# expr-done 7086
Var_Expr_199 := Cast_Result;
if Var_Expr_199 /= No_Entity_Package_Decl then
   --# scope-start
--# expr-start 7088 '<FieldAccess for spark_mode_aspect at nodes.lkt:10637:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10637'
--# expr-start 7087 '<FieldAccess for public_part at nodes.lkt:10637:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10637'
      if Var_Expr_199.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Public_Part (Node => Var_Expr_199.Node.Base_Package_Decl_F_Public_Part, Info => Var_Expr_199.Info);
--# expr-done 7087
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 7088
Scope_Result_1 := Fld_4;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Aspect;
end if;
   Result_Var_1 := Result_Var;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Part_P_Spark_Mode_Aspect;
--# end
--# property-start PrivatePart._env_mappings_212 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10641'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_212
  (Node : Bare_Private_Part
  )
   return Internal_Env_Assoc
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1876 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Expr : Bare_Private_Part;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1876 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1876;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7091 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 7089 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 7089
--# expr-start 7090 '<Symbol __privatepart at nodes.lkt:10641:23>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10641'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 7090
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Sym, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 7091
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1876;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1876;
            raise;
      end;
   return Property_Result;
end Env_Mappings_212;
--# end
--# property-start PrivatePart._env_trans_parent_213 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10643'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_213
  (Node : Bare_Private_Part
  )
   return Boolean
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_213;
--# end
--# property-start PrivatePart._env_names_214 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10643'
pragma Warnings (Off, "is not referenced");
 function Env_Names_214
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part  := Bare_Private_Part (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1878 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1878 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1878;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7092 '<FieldAccess for env_names at nodes.lkt:10643:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10643'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PrivatePart.env_names
Fld := Libadalang.Implementation.Private_Part_P_Env_Names (Node => Self);
--# end
--# expr-done 7092
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1878;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1878;
            raise;
      end;
   return Property_Result;
end Env_Names_214;
--# end
   --
   --  Primitives for Bare_Public_Part
   --
      procedure Initialize_Fields_For_Public_Part
        (Self : Bare_Public_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Public_Part;
   --
   --  Primitives for Bare_Elsif_Expr_Part
   --
      procedure Initialize_Fields_For_Elsif_Expr_Part
        (Self : Bare_Elsif_Expr_Part
         ; Elsif_Expr_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Expr_Part_F_Then_Expr : Bare_Expr
        ) is
      begin
            Self.Elsif_Expr_Part_F_Cond_Expr := Elsif_Expr_Part_F_Cond_Expr;
            Self.Elsif_Expr_Part_F_Then_Expr := Elsif_Expr_Part_F_Then_Expr;
      end Initialize_Fields_For_Elsif_Expr_Part;
   function Elsif_Expr_Part_F_Cond_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Cond_Expr;
   end;
   function Elsif_Expr_Part_F_Then_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Then_Expr;
   end;
   --
   --  Primitives for Bare_Elsif_Stmt_Part
   --
      procedure Initialize_Fields_For_Elsif_Stmt_Part
        (Self : Bare_Elsif_Stmt_Part
         ; Elsif_Stmt_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Stmt_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Elsif_Stmt_Part_F_Cond_Expr := Elsif_Stmt_Part_F_Cond_Expr;
            Self.Elsif_Stmt_Part_F_Stmts := Elsif_Stmt_Part_F_Stmts;
      end Initialize_Fields_For_Elsif_Stmt_Part;
   function Elsif_Stmt_Part_F_Cond_Expr
     (Node : Bare_Elsif_Stmt_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Stmt_Part_F_Cond_Expr;
   end;
   function Elsif_Stmt_Part_F_Stmts
     (Node : Bare_Elsif_Stmt_Part) return Bare_Stmt_List
   is
   begin
         return Node.Elsif_Stmt_Part_F_Stmts;
   end;
--# property-start ElsifStmtPart.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10663'
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Entry_Point
  (Node : Bare_Elsif_Stmt_Part
  )
   return Boolean
is
   Self : Bare_Elsif_Stmt_Part  := Bare_Elsif_Stmt_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Elsif_Stmt_Part_P_Xref_Entry_Point;
--# end
--# property-start ElsifStmtPart.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10666'
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Equation
  (Node : Bare_Elsif_Stmt_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Elsif_Stmt_Part  := Bare_Elsif_Stmt_Part (Node);
     Ent : Internal_Entity_Elsif_Stmt_Part :=
       Internal_Entity_Elsif_Stmt_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1096 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1096 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1096;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7097 '<And at nodes.lkt:10667:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10667'
--# expr-start 7094 '<FieldAccess for sub_equation at nodes.lkt:10667:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10667'
--# expr-start 7093 '<FieldAccess for cond_expr at nodes.lkt:10667:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10667'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Elsif_Stmt_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 7093
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 7094
--# expr-start 7096 '<FieldAccess for expect_bool_derived_type at nodes.lkt:10667:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10667'
--# expr-start 7095 '<FieldAccess for cond_expr at nodes.lkt:10667:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10667'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Elsif_Stmt_Part_F_Cond_Expr;
--# expr-done 7095
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_3 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_2, Origin => Origin);
--# end
--# expr-done 7096
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10667:9") else null)); 
--# expr-done 7097
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1096;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1096;
            raise;
      end;
   return Property_Result;
end Elsif_Stmt_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Expr
   --
      procedure Initialize_Fields_For_Expr
        (Self : Bare_Expr
        ) is
      begin
      Self.Expr_Logic_Vars := System.Null_Address;
      end Initialize_Fields_For_Expr;
--# property-start Expr.type_val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10682'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Type_Val
  (Node : Bare_Expr
  )
   return Internal_Entity
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity;
      Fld : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7099 '<get_value at nodes.lkt:10682:39>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10682'
--# expr-start 7098 '<FieldAccess for type_var at nodes.lkt:10682:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10682'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7098
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Fld); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Fld); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 7099
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Type_Val;
--# end
--# property-start Expr.expression_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10687'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7103 '<Cast to BaseTypeDecl at nodes.lkt:10688:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10688'
--# expr-start 7102 '<FieldAccess for value at nodes.lkt:10688:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10688'
--# expr-start 7101 '<FieldAccess for logic_val at nodes.lkt:10688:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10688'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 7100 '<FieldAccess for type_var at nodes.lkt:10688:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10688'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7100
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 7101
Fld_2 := Fld_1.Value;
--# expr-done 7102
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7103
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Expression_Type;
--# end
--# property-start Expr.expected_expression_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10693'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expected_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7107 '<Cast to BaseTypeDecl at nodes.lkt:10694:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10694'
--# expr-start 7106 '<FieldAccess for value at nodes.lkt:10694:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10694'
--# expr-start 7105 '<FieldAccess for logic_val at nodes.lkt:10694:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10694'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 7104 '<FieldAccess for expected_type_var at nodes.lkt:10694:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10694'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7104
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 7105
Fld_2 := Fld_1.Value;
--# expr-done 7106
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 7107
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Expected_Expression_Type;
--# end
--# property-start Expr.matches_expected_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10697'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_168 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
            procedure Finalizer_Scope_168 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_168;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7110 '<Predicate on BaseTypeDecl.matching_type at nodes.lkt:10698:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10698'
--# expr-start 7108 '<FieldAccess for type_var at nodes.lkt:10698:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10698'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7108
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7109 '<FieldAccess for expected_type_var at nodes.lkt:10698:86>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10698'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7109
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Type_0_Predicate (Origin, Self)); 
--# expr-done 7110
         Property_Result := Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_168;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_168;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Type;
--# end
--# property-start Expr.matches_expected_assign_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10701'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Assign_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_169 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
            procedure Finalizer_Scope_169 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_169;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7113 '<Predicate on BaseTypeDecl.matching_assign_type at nodes.lkt:10702:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10702'
--# expr-start 7111 '<FieldAccess for type_var at nodes.lkt:10702:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10702'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7111
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7112 '<FieldAccess for expected_type_var at nodes.lkt:10702:93>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10702'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7112
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Assign_Type_0_Predicate (Origin, Self)); 
--# expr-done 7113
         Property_Result := Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_169;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_169;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Assign_Type;
--# end
--# property-start Expr.matches_expected_formal_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10705'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Formal_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_170 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
            procedure Finalizer_Scope_170 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_170;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7116 '<Predicate on BaseTypeDecl.matching_formal_type at nodes.lkt:10706:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10706'
--# expr-start 7114 '<FieldAccess for type_var at nodes.lkt:10706:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10706'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7114
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7115 '<FieldAccess for expected_type_var at nodes.lkt:10706:93>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10706'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7115
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Formal_Type_0_Predicate (Origin, Self)); 
--# expr-done 7116
         Property_Result := Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_170;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_170;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Formal_Type;
--# end
--# property-start Expr.matches_expected_membership_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10709'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Membership_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_171 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
            procedure Finalizer_Scope_171 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_171;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7119 '<Predicate on BaseTypeDecl.matching_membership_type at nodes.lkt:10710:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10710'
--# expr-start 7117 '<FieldAccess for type_var at nodes.lkt:10710:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10710'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7117
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7118 '<FieldAccess for expected_type_var at nodes.lkt:10710:76>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10710'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7118
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Membership_Type_0_Predicate (Origin)); 
--# expr-done 7119
         Property_Result := Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_171;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_171;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Membership_Type;
--# end
--# property-start Expr.matches_expected_prefix_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10713'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matches_Expected_Prefix_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_172 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
            procedure Finalizer_Scope_172 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_172;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7122 '<Predicate on BaseTypeDecl.matching_prefix_type at nodes.lkt:10714:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10714'
--# expr-start 7120 '<FieldAccess for type_var at nodes.lkt:10714:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10714'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7120
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7121 '<FieldAccess for expected_type_var at nodes.lkt:10714:72>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10714'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7121
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_N_Predicate ((Fld, Fld_1), Create_Base_Type_Decl_P_Matching_Prefix_Type_0_Predicate (Origin)); 
--# expr-done 7122
         Property_Result := Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_172;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_172;
            raise;
      end;
   return Property_Result;
end Expr_P_Matches_Expected_Prefix_Type;
--# end
--# property-start Expr.expect_bool_derived_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10720'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expect_Bool_Derived_Type
  (Node : Bare_Expr
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_173 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_173 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_173;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7132 '<And at nodes.lkt:10720:48>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10720'
--# expr-start 7129 '<Or at nodes.lkt:10721:9>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
--# expr-start 7125 '<Bind at nodes.lkt:10721:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
--# expr-start 7123 '<FieldAccess for expected_type_var at nodes.lkt:10721:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7123
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7124 '<FieldAccess for bool_type at nodes.lkt:10721:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7124
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10721:9") else null)); 
--# expr-done 7125
--# expr-start 7128 '<Bind at nodes.lkt:10721:60>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
--# expr-start 7126 '<FieldAccess for type_var at nodes.lkt:10721:90>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7126
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7127 '<FieldAccess for expected_type_var at nodes.lkt:10721:64>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10721'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7127
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result_1 := Solver.Create_Propagate (Fld_3, Fld_2, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10721:60") else null)); 
--# expr-done 7128
Or_Pred := Create_Or (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10721:9") else null)); 
--# expr-done 7129
--# expr-start 7131 '<Predicate on BaseTypeDecl.derives_from_std_bool_type at nodes.lkt:10722:11>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10722'
--# expr-start 7130 '<FieldAccess for type_var at nodes.lkt:10722:63>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10722'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7130
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Derives_From_Std_Bool_Type_0_Predicate (Origin, Self)); 
--# expr-done 7131
And_Pred := Create_And (Or_Pred, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10720:48") else null)); 
--# expr-done 7132
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_173;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_173;
            raise;
      end;
   return Property_Result;
end Expr_P_Expect_Bool_Derived_Type;
--# end
--# property-start Expr.annotate_argument_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10734'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Annotate_Argument_Equation
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_174 with Inline_Always;
            procedure Finalizer_Scope_2961 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Array_Lit : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_4 : Logic_Var;
Domain_Equation : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Or_Pred : Logic_Equation;
            procedure Finalizer_Scope_174 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_174;
            procedure Finalizer_Scope_2961 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2961;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7143 '<Or at nodes.lkt:10734:50>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10734'
--# expr-start 7140 '<And at nodes.lkt:10735:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
--# expr-start 7133 '<FieldAccess for sub_equation at nodes.lkt:10735:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 7133
--# expr-start 7139 '<domain at nodes.lkt:10735:33>' Domain_Equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
--# expr-start 7137 '<ArrayLiteral at nodes.lkt:10735:67>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
--# expr-start 7134 '<FieldAccess for std_string_type at nodes.lkt:10735:96>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 7134
--# expr-start 7135 '<FieldAccess for std_wide_string_type at nodes.lkt:10735:120>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_string_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Wide_String_Type (Node => Self);
--# end
--# expr-done 7135
--# expr-start 7136 '<FieldAccess for std_wide_wide_string_type at nodes.lkt:10735:149>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_wide_string_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Wide_String_Type (Node => Self);
--# end
--# expr-done 7136
Array_Lit := Create_Internal_Entity_Base_Type_Decl_Array (Internal_Internal_Entity_Base_Type_Decl_Array'(1 => No_Entity_Base_Type_Decl, 2 => Fld_1, 3 => Fld_2, 4 => Fld_3)); 
--# expr-done 7137
--# expr-start 7138 '<FieldAccess for expected_type_var at nodes.lkt:10735:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10735'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7138
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
declare
   Dom : Internal_Entity_Base_Type_Decl_Array_Access := Array_Lit;
   A   : Entity_Vars.Value_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity_Base_Type_Decl := Get (Self, Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Solver.Create_Domain
     (Fld_4, A, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10735:33") else null));
end;
--# expr-done 7139
And_Pred := Create_And (Fld, Domain_Equation, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10735:9") else null)); 
--# expr-done 7140
--# expr-start 7141 '<Cast to Name at nodes.lkt:10736:10>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10736'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 7141
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_0
--# expr-start 7142 '<FieldAccess for xref_no_overloading at nodes.lkt:10737:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10737'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7142
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2961;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10737:59") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
Or_Pred := Create_Or (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:10734:50") else null)); 
--# expr-done 7143
         Property_Result := Or_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_174;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_174;
                     Finalizer_Scope_2961;
            raise;
      end;
   return Property_Result;
end Expr_P_Annotate_Argument_Equation;
--# end
--# property-start Expr.is_dynamically_tagged 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10744'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dynamically_Tagged
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2966 with Inline_Always;
      Match_0 : Internal_Entity_Qual_Expr;
Match_1 : Internal_Entity_Name;
Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Match_2 : Internal_Entity_Cond_Expr;
Match_3 : Internal_Entity_Decl_Expr;
Match_4 : Internal_Entity_Paren_Expr;
Match_5 : Internal_Entity_Expr;
Var_Expr_200 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
Fld_8 : Boolean;
Fld_9 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_10 : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_2 : Boolean;
Fld_12 : Internal_Entity_Expr_Array_Access;
Fld_13 : Boolean;
Quantifier_Result : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Internal_Entity_Cond_Expr;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Internal_Entity_Decl_Expr;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_5 : Internal_Entity_Paren_Expr;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_6 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
If_Result_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
            procedure Finalizer_Scope_2966 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2966;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7165 '<bind at nodes.lkt:10746:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10746'
--# expr-start 7144 '<FieldAccess for origin_node at nodes.lkt:10746:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7144
Bound_Origin := Fld; 
--# expr-start 7146 '<FieldAccess for is_classwide at nodes.lkt:10748:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
--# expr-start 7145 '<FieldAccess for expression_type at nodes.lkt:10748:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7145
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 7146
if Fld_2 then
   If_Result := True;
else
--# expr-start 7148 '<FieldAccess for accessed_type at nodes.lkt:10748:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
--# expr-start 7147 '<FieldAccess for expression_type at nodes.lkt:10748:50>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7147
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_3.Node, Origin => Bound_Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 7148
Var_Expr_200 := Fld_4;
if Var_Expr_200 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7149 '<FieldAccess for is_classwide at nodes.lkt:10748:50>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
      if Var_Expr_200.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_200.Node, E_Info => Var_Expr_200.Info);
--# end
--# expr-done 7149
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_3 := True;
else
   --# expr-start 7164 '<Match at nodes.lkt:10748:108>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10748'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind qual_expr Match_0
--# expr-start 7151 '<FieldAccess for is_dynamically_tagged at nodes.lkt:10749:41>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10749'
--# expr-start 7150 '<FieldAccess for suffix at nodes.lkt:10749:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10749'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_0.Node.Qual_Expr_F_Suffix, Info => Match_0.Info);
--# expr-done 7150
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_7 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 7151
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind n Match_1
--# expr-start 7152 '<FieldAccess for is_direct_call at nodes.lkt:10754:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10754'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_8 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7152
if Fld_8 then
--# expr-start 7154 '<Cast to BaseSubpSpec at nodes.lkt:10754:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10754'
--# expr-start 7153 '<FieldAccess for called_subp_spec at nodes.lkt:10754:52>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10754'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.called_subp_spec
Fld_9 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7153
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 7154
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 7155 '<FieldAccess for has_controlling_result at nodes.lkt:10755:27>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10755'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.has_controlling_result
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Has_Controlling_Result (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7155
if Fld_10 then
   --# expr-start 7156 '<FieldAccess for has_dynamic_controlling_operand at nodes.lkt:10755:61>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10755'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_controlling_operand
Fld_11 := Libadalang.Implementation.Expr_P_Has_Dynamic_Controlling_Operand (Node => Match_1.Node, Spec => Var_Expr_0, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 7156
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_1 := If_Result_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_2 := Result_Var_1;
else
   If_Result_2 := False;
end if;
Let_Result_1 := If_Result_2; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind cond_expr Match_2
--# expr-start 7159 '<AllQuantifier at nodes.lkt:10757:41>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10757'
   --# expr-start 7157 '<FieldAccess for dependent_exprs at nodes.lkt:10757:41>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10757'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_12 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 7157
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_12;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
      --# expr-start 7158 '<FieldAccess for is_dynamically_tagged at nodes.lkt:10757:80>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10757'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_13 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Item_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_1.Info);
--# end
--# expr-done 7158
      Quantifier_Result := Fld_13;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 7159
Let_Result_2 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
      Finalizer_Scope_2966;
            Match_Result := Scope_Result_4; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind decl_expr Match_3
--# expr-start 7161 '<FieldAccess for is_dynamically_tagged at nodes.lkt:10758:41>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10758'
--# expr-start 7160 '<FieldAccess for expr at nodes.lkt:10758:41>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10758'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Match_3.Node.Decl_Expr_F_Expr, Info => Match_3.Info);
--# expr-done 7160
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_15 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_14.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 7161
Let_Result_3 := Fld_15; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind paren_expr Match_4
--# expr-start 7163 '<FieldAccess for is_dynamically_tagged at nodes.lkt:10759:43>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10759'
--# expr-start 7162 '<FieldAccess for expr at nodes.lkt:10759:43>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10759'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Match_4.Node.Paren_Expr_F_Expr, Info => Match_4.Info);
--# expr-done 7162
      if Fld_16.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_17 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 7163
Let_Result_4 := Fld_17; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Concat_Operand | Ada_Contract_Cases | Ada_Format_String_Literal .. Ada_Membership_Expr | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 7164
   If_Result_3 := Match_Result;
end if;
Dyn_Var_Bind_Result := If_Result_3; 
--# expr-done 7165
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2966;
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dynamically_Tagged;
--# end
--# property-start Expr.has_dynamic_controlling_operand 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10767'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Dynamic_Controlling_Operand
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_176 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Expected_Type_For_Expr_Array_Access;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
            procedure Finalizer_Scope_176 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_176;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7173 '<AnyQuantifier at nodes.lkt:10774:9>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10774'
   --# expr-start 7166 '<FieldAccess for potential_actuals_for_dispatch at nodes.lkt:10774:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10774'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.potential_actuals_for_dispatch'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Potential_Actuals_For_Dispatch (Node => Ent.Node, Spec => Spec, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7166
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Expected_Type_For_Expr_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_0
--# expr-start 7168 '<FieldAccess for get_candidate_type_for_primitive at nodes.lkt:10775:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10775'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7167 '<FieldAccess for expected_type at nodes.lkt:10775:58>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10775'
Fld_1 := Item_0.Expected_Type;
--# expr-done 7167
--# property-call-start BaseSubpSpec.get_candidate_type_for_primitive
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Candidate_Type_For_Primitive (Node => Spec.Node, Type_Expr => Fld_1, E_Info => Spec.Info);
--# end
--# expr-done 7168
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
--# expr-start 7170 '<FieldAccess for is_tagged_type at nodes.lkt:10781:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10781'
--# expr-start 7169 '<FieldAccess for full_view at nodes.lkt:10781:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10781'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7169
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_3.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 7170
if Fld_4 then
   --# expr-start 7172 '<FieldAccess for is_dynamically_tagged at nodes.lkt:10782:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10782'
--# expr-start 7171 '<FieldAccess for expr at nodes.lkt:10782:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10782'
Fld_5 := Item_0.Expr;
--# expr-done 7171
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_6 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 7172
   If_Result := Fld_6;
else
   If_Result := False;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      Quantifier_Result := Result_Var;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 7173
         Property_Result := Quantifier_Result;
   --# end
      Finalizer_Scope_176;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_176;
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Dynamic_Controlling_Operand;
--# end
--# property-start Expr.parent_candidate_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10789'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Parent_Candidate_Dispatching_Call
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_2977 with Inline_Always;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Un_Op;
Match_3 : Internal_Entity_Bin_Op;
Match_4 : Internal_Entity_Cond_Expr;
Match_5 : Internal_Entity_Paren_Expr;
Match_6 : Internal_Entity_Qual_Expr;
Match_7 : Internal_Entity_Decl_Expr;
Match_8 : Internal_Entity_Param_Assoc;
Match_9 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Name;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Dotted_Name;
Cast_Result_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Base_Id;
Cast_Expr_3 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
Cast_Expr_4 : Internal_Entity_Un_Op;
Cast_Result_6 : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Op;
Cast_Expr_5 : Internal_Entity_Expr;
Cast_Result_7 : Internal_Entity;
Cast_Result_8 : Internal_Entity;
Is_Equiv_2 : Boolean;
If_Result_2 : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Expr;
Cast_Expr_6 : Internal_Entity_Bin_Op;
Cast_Result_9 : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Op;
Cast_Expr_7 : Internal_Entity_Expr;
Cast_Result_10 : Internal_Entity;
Cast_Result_11 : Internal_Entity;
Is_Equiv_3 : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Internal_Entity_Expr_Array_Access;
Cast_Expr_8 : Internal_Entity_Expr;
Cast_Result_12 : Internal_Entity;
Cast_Expr_9 : Internal_Entity_Expr;
Cast_Result_13 : Internal_Entity;
Is_Equiv_4 : Boolean;
Quantifier_Result : Boolean;
If_Result_4 : Internal_Entity_Expr;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Internal_Entity;
Fld_15 : Internal_Entity;
Cast_Result_14 : Internal_Entity_Call_Expr;
Match_Prefix : Internal_Entity;
Cast_Result_15 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Cast_Result_16 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Cast_Result_17 : Internal_Entity_Un_Op;
Let_Result_2 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Cast_Result_18 : Internal_Entity_Bin_Op;
Let_Result_3 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Cast_Result_19 : Internal_Entity_Cond_Expr;
Let_Result_4 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Cast_Result_20 : Internal_Entity_Paren_Expr;
Let_Result_5 : Internal_Entity_Expr;
Scope_Result_5 : Internal_Entity_Expr;
Cast_Result_21 : Internal_Entity_Qual_Expr;
Let_Result_6 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Cast_Result_22 : Internal_Entity_Decl_Expr;
Let_Result_7 : Internal_Entity_Expr;
Scope_Result_7 : Internal_Entity_Expr;
Cast_Result_23 : Internal_Entity_Param_Assoc;
Cast_Result_24 : Internal_Entity_Expr;
Let_Result_8 : Internal_Entity_Expr;
Scope_Result_8 : Internal_Entity_Expr;
Cast_Result_25 : Internal_Entity;
Let_Result_9 : Internal_Entity_Expr;
Scope_Result_9 : Internal_Entity_Expr;
Match_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_2977 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2977;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7202 '<Match at nodes.lkt:10789:61>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10789'
--# expr-start 7174 '<parent at nodes.lkt:10789:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10789'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7174
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_15; 
--# bind ce Match_0
--# expr-start 7178 '<If at nodes.lkt:10790:30>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10790'
--# expr-start 7176 '<Eq at nodes.lkt:10790:33>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10790'
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 7175 '<FieldAccess for name at nodes.lkt:10790:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10790'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 7175
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 7176
if Is_Equiv then
   --# expr-start 7177 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10790:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10790'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_1 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 7177
   If_Result := Fld_1;
else
Cast_Expr := Match_0; 
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result := Cast_Result;
end if;
--# expr-done 7178
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_16; 
--# bind dn Match_1
--# expr-start 7182 '<If at nodes.lkt:10791:32>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10791'
--# expr-start 7180 '<Eq at nodes.lkt:10791:35>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10791'
Cast_Expr_3 := Ent; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 7179 '<FieldAccess for suffix at nodes.lkt:10791:43>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10791'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_1.Node.Dotted_Name_F_Suffix, Info => Match_1.Info);
--# expr-done 7179
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 7180
if Is_Equiv_1 then
   --# expr-start 7181 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10791:58>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10791'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_3 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7181
   If_Result_1 := Fld_3;
else
Cast_Expr_2 := Match_1; 
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   If_Result_1 := Cast_Result_3;
end if;
--# expr-done 7182
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_17; 
--# bind uo Match_2
--# expr-start 7186 '<If at nodes.lkt:10792:26>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10792'
--# expr-start 7184 '<Eq at nodes.lkt:10792:29>' Is_Equiv_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10792'
Cast_Expr_5 := Ent; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# expr-start 7183 '<FieldAccess for op at nodes.lkt:10792:37>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10792'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Op (Node => Match_2.Node.Un_Op_F_Op, Info => Match_2.Info);
--# expr-done 7183
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Is_Equiv_2 := Equivalent (Cast_Result_7, Cast_Result_8); 
--# expr-done 7184
if Is_Equiv_2 then
   --# expr-start 7185 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10792:48>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10792'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_5 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 7185
   If_Result_2 := Fld_5;
else
Cast_Expr_4 := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   If_Result_2 := Cast_Result_6;
end if;
--# expr-done 7186
Let_Result_2 := If_Result_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_18; 
--# bind bo Match_3
--# expr-start 7190 '<If at nodes.lkt:10793:27>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10793'
--# expr-start 7188 '<Eq at nodes.lkt:10793:30>' Is_Equiv_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10793'
Cast_Expr_7 := Ent; 
      Cast_Result_10 := Create_Internal_Entity
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
--# expr-start 7187 '<FieldAccess for op at nodes.lkt:10793:38>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10793'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Op (Node => Match_3.Node.Bin_Op_F_Op, Info => Match_3.Info);
--# expr-done 7187
      Cast_Result_11 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Is_Equiv_3 := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 7188
if Is_Equiv_3 then
   --# expr-start 7189 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10793:49>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10793'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_7 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 7189
   If_Result_3 := Fld_7;
else
Cast_Expr_6 := Match_3; 
      Cast_Result_9 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   If_Result_3 := Cast_Result_9;
end if;
--# expr-done 7190
Let_Result_3 := If_Result_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_19; 
--# bind ce Match_4
--# expr-start 7195 '<If at nodes.lkt:10794:30>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10794'
--# expr-start 7193 '<Contains at nodes.lkt:10794:33>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10794'
   --# expr-start 7191 '<FieldAccess for dependent_exprs at nodes.lkt:10794:33>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10794'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_10 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 7191
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_10;
      begin
         for Item_16 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_16
      --# expr-start 7192 '<Eq at ???>' Is_Equiv_4 None
Cast_Expr_8 := Item_16; 
      Cast_Result_12 := Create_Internal_Entity
        (Node => Cast_Expr_8.Node,
         Info => Cast_Expr_8.Info);
Cast_Expr_9 := Ent; 
      Cast_Result_13 := Create_Internal_Entity
        (Node => Cast_Expr_9.Node,
         Info => Cast_Expr_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_12, Cast_Result_13); 
--# expr-done 7192
      Quantifier_Result := Is_Equiv_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 7193
if Quantifier_Result then
   --# expr-start 7194 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10794:74>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10794'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_9 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 7194
   If_Result_4 := Fld_9;
else
   If_Result_4 := No_Entity_Expr;
end if;
--# expr-done 7195
Let_Result_4 := If_Result_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
      Finalizer_Scope_2977;
            Match_Result := Scope_Result_4; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_20 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_20; 
--# bind pe Match_5
--# expr-start 7196 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10795:31>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10795'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_11 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_5.Node, E_Info => Match_5.Info);
--# end
--# expr-done 7196
Let_Result_5 := Fld_11; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_21 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_21; 
--# bind qe Match_6
--# expr-start 7197 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10796:30>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10796'
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_12 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_6.Node, E_Info => Match_6.Info);
--# end
--# expr-done 7197
Let_Result_6 := Fld_12; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_22 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_22; 
--# bind de Match_7
--# expr-start 7198 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10797:30>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10797'
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_13 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Match_7.Node, E_Info => Match_7.Info);
--# end
--# expr-done 7198
Let_Result_7 := Fld_13; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_23 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_23; 
--# bind pa Match_8
--# expr-start 7201 '<Cast to CallExpr at nodes.lkt:10798:32>' Cast_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10798'
--# expr-start 7200 '<parent at nodes.lkt:10798:32>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10798'
--# expr-start 7199 '<parent at nodes.lkt:10798:32>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10798'
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Parent (Node => Match_8.Node, E_Info => Match_8.Info);
--# expr-done 7199
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 7200
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_14 := Create_Internal_Entity_Call_Expr
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_14 := No_Entity_Call_Expr;
   end if;
--# expr-done 7201
      Cast_Result_24 := Create_Internal_Entity_Expr
        (Node => Cast_Result_14.Node,
         Info => Cast_Result_14.Info);
Let_Result_8 := Cast_Result_24; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Abort_Absent .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Concat_Operand | Ada_Contract_Cases | Ada_Format_String_Literal .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr .. Ada_Raise_Expr | Ada_For_Loop_Iter_Filter .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_25 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_25; 
--# bind _ Match_9
Let_Result_9 := No_Entity_Expr; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
end case;
--# expr-done 7202
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_2977;
            raise;
      end;
   return Property_Result;
end Expr_P_Parent_Candidate_Dispatching_Call;
--# end
--# property-start Expr.has_dynamic_context 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10807'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Dynamic_Context
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Assign_Stmt;
Var_Expr_201 : Internal_Entity_Expr;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assign_Stmt;
Fld_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
Result_Var_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 7204 '<Cast to AssignStmt at nodes.lkt:10807:39>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10807'
--# expr-start 7203 '<parent at nodes.lkt:10807:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10807'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7203
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assign_Stmt;
   end if;
--# expr-done 7204
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Assign_Stmt then
   --# scope-start
   --# bind a Var_Expr_0
--# expr-start 7206 '<Eq at nodes.lkt:10812:13>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10812'
--# expr-start 7205 '<FieldAccess for expr at nodes.lkt:10812:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10812'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr_0.Node.Assign_Stmt_F_Expr, Info => Var_Expr_0.Info);
--# expr-done 7205
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 7206
if Is_Equiv then
   --# expr-start 7208 '<FieldAccess for is_classwide at nodes.lkt:10813:15>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10813'
--# expr-start 7207 '<FieldAccess for expected_expression_type at nodes.lkt:10813:15>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10813'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expected_expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expected_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7207
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 7208
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 7209 '<FieldAccess for parent_candidate_dispatching_call at nodes.lkt:10818:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10818'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.parent_candidate_dispatching_call
Fld_4 := Libadalang.Implementation.Expr_P_Parent_Candidate_Dispatching_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7209
Var_Expr_201 := Fld_4;
if Var_Expr_201 /= No_Entity_Expr then
   --# scope-start
--# expr-start 7210 '<FieldAccess for is_dispatching_call at nodes.lkt:10818:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10818'
      if Var_Expr_201.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_5 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Var_Expr_201.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_201.Info);
--# end
--# expr-done 7210
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   Result_Var_1 := Result_Var;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Dynamic_Context;
--# end
--# property-start Expr.is_dispatching_call_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10825'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call_Impl
  (Node : Bare_Expr
      ; Decl : Internal_Entity_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind decl Decl
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Local_Spec : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Is_A : Boolean;
Fld_2 : Boolean;
If_Result : Boolean;
Fld_3 : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7217 '<Let at nodes.lkt:10826:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10826'
--# scope-start
--# expr-start 7212 '<FieldAccess for subp_spec_or_null at nodes.lkt:10826:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10826'
--# expr-start 7211 '<FieldAccess for canonical_part at nodes.lkt:10826:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10826'
      if Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Decl.Info);
--# end
--# expr-done 7211
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld.Node, Follow_Generic => True, E_Info => Fld.Info);
--# end
--# expr-done 7212
Local_Spec := Fld_1; 
--# bind spec Local_Spec
--# expr-start 7213 '<IsA AbstractFormalSubpDecl at nodes.lkt:10831:13>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10831'
Is_A := Decl.Node /= null 
and then Decl.Node.Kind in Ada_Abstract_Formal_Subp_Decl_Range; 
--# expr-done 7213
if Is_A then
   If_Result := True;
else
   --# expr-start 7214 '<FieldAccess for has_dynamic_controlling_operand at nodes.lkt:10835:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10835'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_controlling_operand
Fld_2 := Libadalang.Implementation.Expr_P_Has_Dynamic_Controlling_Operand (Node => Ent.Node, Spec => Local_Spec, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7214
   If_Result := Fld_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 7215 '<FieldAccess for has_controlling_result at nodes.lkt:10841:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10841'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.has_controlling_result
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Has_Controlling_Result (Node => Local_Spec.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Spec.Info);
--# end
--# expr-done 7215
if Fld_3 then
   --# expr-start 7216 '<FieldAccess for has_dynamic_context at nodes.lkt:10841:47>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.has_dynamic_context
Fld_4 := Libadalang.Implementation.Expr_P_Has_Dynamic_Context (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7216
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 7217
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dispatching_Call_Impl;
--# end
--# property-start Expr.is_static_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10849'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Static_Expr
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Num_Literal;
Match_1 : Internal_Entity_String_Literal;
Match_2 : Internal_Entity_Attribute_Ref;
Var_Expr_0 : Internal_Entity_Assoc_List;
Match_3 : Internal_Entity_Call_Expr;
Match_4 : Internal_Entity_Qual_Expr;
Match_5 : Internal_Entity_Name;
Match_6 : Internal_Entity_Membership_Expr;
Match_7 : Internal_Entity_Bin_Op;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Match_8 : Internal_Entity_Concat_Operand;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Match_9 : Internal_Entity_Concat_Op;
Match_10 : Internal_Entity_Un_Op;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Match_11 : Internal_Entity_If_Expr;
Match_12 : Internal_Entity_Paren_Expr;
Match_13 : Internal_Entity_Expr;
Var_Expr_202 : Internal_Entity_Base_Type_Decl;
Var_Expr_203 : Internal_Entity_Base_Type_Decl;
Var_Expr_204 : Internal_Entity_Base_Type_Decl;
Var_Expr_205 : Internal_Entity_Basic_Decl;
Var_Expr_206 : Internal_Entity_Base_Type_Decl;
Var_Expr_207 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Not_Val : Boolean;
Fld_7 : Internal_Entity_Identifier;
Fld_8 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Fld_14 : Internal_Entity_Name;
Fld_15 : Internal_Entity_Basic_Decl;
Fld_16 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_17 : Internal_Entity_Identifier;
Fld_18 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_6 : Boolean;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
Is_Equal_8 : Boolean;
If_Result_8 : Boolean;
Is_Equal_9 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Boolean;
Fld_19 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_20 : Bare_Assoc_List;
Fld_21 : Internal_Entity_Info;
Item_1 : Internal_Entity_Basic_Assoc;
Bare_Item_1 : Bare_Basic_Assoc;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Boolean;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_13 : Internal_Entity_Basic_Assoc;
Quantifier_Result : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_14 : Boolean;
Fld_24 : Internal_Entity_Name;
Fld_25 : Boolean;
Fld_26 : Internal_Entity_Assoc_List;
Fld_27 : Bare_Assoc_List;
Fld_28 : Internal_Entity_Info;
Item_2 : Internal_Entity_Basic_Assoc;
Bare_Item_2 : Bare_Basic_Assoc;
Fld_29 : Internal_Entity_Expr;
Fld_30 : Boolean;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_15 : Internal_Entity_Basic_Assoc;
Quantifier_Result_1 : Boolean;
If_Result_16 : Boolean;
Fld_31 : Internal_Entity_Name;
Fld_32 : Internal_Entity_Base_Type_Decl;
Fld_33 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Boolean;
If_Result_17 : Boolean;
Fld_36 : Internal_Entity_Basic_Decl;
Fld_37 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
Fld_38 : Internal_Entity_Expr;
Fld_39 : Boolean;
Fld_40 : Internal_Entity_Expr_Alternatives_List;
Fld_41 : Bare_Expr_Alternatives_List;
Fld_42 : Internal_Entity_Info;
Item_3 : Internal_Entity_Expr;
Bare_Item_3 : Bare_Expr;
Fld_43 : Boolean;
As_Entity_2 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
If_Result_18 : Internal_Entity_Expr;
Quantifier_Result_2 : Boolean;
If_Result_19 : Boolean;
Fld_44 : Internal_Entity_Expr;
Fld_45 : Boolean;
Fld_46 : Internal_Entity_Expr;
Fld_47 : Boolean;
If_Result_20 : Boolean;
Fld_48 : Internal_Entity_Op;
Fld_49 : Internal_Entity_Basic_Decl;
Fld_50 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
If_Result_21 : Boolean;
Fld_51 : Internal_Entity_Expr;
Fld_52 : Boolean;
Fld_53 : Internal_Entity_Op_Concat;
Fld_54 : Internal_Entity_Basic_Decl;
Fld_55 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
If_Result_22 : Boolean;
Fld_56 : Internal_Entity_Expr;
Fld_57 : Boolean;
Fld_58 : Internal_Entity_Concat_Operand_List;
Fld_59 : Bare_Concat_Operand_List;
Fld_60 : Internal_Entity_Info;
Item_6 : Internal_Entity_Concat_Operand;
Bare_Item_6 : Bare_Concat_Operand;
Fld_61 : Boolean;
As_Entity_3 : Internal_Entity_Concat_Operand;
Is_Null_3 : Boolean;
If_Result_23 : Internal_Entity_Concat_Operand;
Quantifier_Result_3 : Boolean;
If_Result_24 : Boolean;
Fld_62 : Internal_Entity_Expr;
Fld_63 : Boolean;
Fld_64 : Internal_Entity_Op;
Fld_65 : Internal_Entity_Basic_Decl;
Fld_66 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_9 : Boolean;
If_Result_25 : Boolean;
Fld_67 : Internal_Entity_Expr;
Fld_68 : Boolean;
Fld_69 : Internal_Entity_Expr;
Fld_70 : Boolean;
If_Result_26 : Boolean;
Fld_71 : Internal_Entity_Elsif_Expr_Part_List;
Fld_72 : Bare_Elsif_Expr_Part_List;
Fld_73 : Internal_Entity_Info;
Item_8 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_8 : Bare_Elsif_Expr_Part;
Fld_74 : Internal_Entity_Expr;
Fld_75 : Boolean;
Fld_76 : Internal_Entity_Expr;
Fld_77 : Boolean;
If_Result_27 : Boolean;
As_Entity_4 : Internal_Entity_Elsif_Expr_Part;
Is_Null_4 : Boolean;
If_Result_28 : Internal_Entity_Elsif_Expr_Part;
Quantifier_Result_4 : Boolean;
If_Result_29 : Boolean;
Fld_78 : Internal_Entity_Expr;
Fld_79 : Boolean;
If_Result_30 : Boolean;
Fld_80 : Internal_Entity_Expr;
Fld_81 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Num_Literal;
Let_Result : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_1 : Internal_Entity_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_2 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Boolean;
Scope_Result_12 : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_3 : Boolean;
Scope_Result_13 : Boolean;
Cast_Result_4 : Internal_Entity_Qual_Expr;
Let_Result_4 : Boolean;
Scope_Result_14 : Boolean;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_5 : Boolean;
Scope_Result_15 : Boolean;
Cast_Result_6 : Internal_Entity_Membership_Expr;
Let_Result_6 : Boolean;
Scope_Result_16 : Boolean;
Cast_Result_7 : Internal_Entity_Bin_Op;
Let_Result_7 : Boolean;
Scope_Result_17 : Boolean;
Cast_Result_8 : Internal_Entity_Concat_Operand;
Let_Result_8 : Boolean;
Scope_Result_18 : Boolean;
Cast_Result_9 : Internal_Entity_Concat_Op;
Let_Result_9 : Boolean;
Scope_Result_19 : Boolean;
Cast_Result_10 : Internal_Entity_Un_Op;
Let_Result_10 : Boolean;
Scope_Result_20 : Boolean;
Cast_Result_11 : Internal_Entity_If_Expr;
Let_Result_11 : Boolean;
Scope_Result_21 : Boolean;
Cast_Result_12 : Internal_Entity_Paren_Expr;
Let_Result_12 : Boolean;
Scope_Result_22 : Boolean;
Cast_Result_13 : Internal_Entity_Expr;
Let_Result_13 : Boolean;
Scope_Result_23 : Boolean;
Match_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7308 '<bind at nodes.lkt:10850:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10850'
--# expr-start 7218 '<FieldAccess for origin_node at nodes.lkt:10850:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10850'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7218
Bound_Origin := Fld; 
--# expr-start 7307 '<Match at nodes.lkt:10852:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10852'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Int_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Num_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_10 := Let_Result;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_String_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_11 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind ar Match_2
--# expr-start 7220 '<FieldAccess for is_static_expr at nodes.lkt:10856:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10856'
--# expr-start 7219 '<FieldAccess for prefix at nodes.lkt:10856:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10856'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7219
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 7220
if Fld_2 then
--# expr-start 7225 '<Not at nodes.lkt:10858:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
--# expr-start 7222 '<FieldAccess for name_designated_type at nodes.lkt:10858:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
--# expr-start 7221 '<FieldAccess for prefix at nodes.lkt:10858:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7221
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 7222
Var_Expr_202 := Fld_4;
if Var_Expr_202 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7223 '<FieldAccess for root_type at nodes.lkt:10858:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
      if Var_Expr_202.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Var_Expr_202.Node, Origin => Bound_Origin, E_Info => Var_Expr_202.Info);
--# end
--# expr-done 7223
Var_Expr_203 := Fld_5;
if Var_Expr_203 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7224 '<FieldAccess for is_formal at nodes.lkt:10858:29>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
      if Var_Expr_203.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_formal
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Var_Expr_203.Node);
--# end
--# expr-done 7224
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Not_Val := not (Result_Var_1); 
--# expr-done 7225
if Not_Val then
   --# expr-start 7229 '<Eq at nodes.lkt:10858:92>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
--# expr-start 7227 '<FieldAccess for name_symbol at nodes.lkt:10858:92>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
--# expr-start 7226 '<FieldAccess for attribute at nodes.lkt:10858:92>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 7226
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_7.Node);
--# end
--# expr-done 7227
--# expr-start 7228 '<Symbol Base at nodes.lkt:10858:122>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10858'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 7228
Is_Equal := Fld_8 = Sym; 
--# expr-done 7229
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_6 := True;
else
--# expr-start 7231 '<FieldAccess for name_designated_type at nodes.lkt:10860:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
--# expr-start 7230 '<FieldAccess for prefix at nodes.lkt:10860:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7230
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_10 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 7231
Var_Expr_204 := Fld_10;
if Var_Expr_204 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7232 '<FieldAccess for is_static_decl at nodes.lkt:10860:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
      if Var_Expr_204.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_204.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_204.Info);
--# end
--# expr-done 7232
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 7234 '<FieldAccess for name_symbol at nodes.lkt:10860:80>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
--# expr-start 7233 '<FieldAccess for attribute at nodes.lkt:10860:80>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 7233
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 7234
--# expr-start 7235 '<Symbol First at nodes.lkt:10860:110>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 7235
Is_Equal_5 := Fld_13 = Sym_1; 
if Is_Equal_5 then
   If_Result_4 := True;
else
--# expr-start 7236 '<Symbol Last at nodes.lkt:10860:121>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 7236
Is_Equal_4 := Fld_13 = Sym_2; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 7237 '<Symbol Range at nodes.lkt:10860:131>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 7237
Is_Equal_3 := Fld_13 = Sym_3; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 7238 '<Symbol Val at nodes.lkt:10860:142>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 7238
Is_Equal_2 := Fld_13 = Sym_4; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 7239 '<Symbol Pos at nodes.lkt:10860:151>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10860'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos); 
--# expr-done 7239
Is_Equal_1 := Fld_13 = Sym_5; 
   If_Result_1 := Is_Equal_1;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
if If_Result_6 then
   If_Result_11 := True;
else
--# expr-start 7241 '<FieldAccess for referenced_decl at nodes.lkt:10862:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
--# expr-start 7240 '<FieldAccess for prefix at nodes.lkt:10862:25>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 7240
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_15 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_14.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 7241
Var_Expr_205 := Fld_15;
if Var_Expr_205 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7242 '<FieldAccess for is_array at nodes.lkt:10862:25>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
      if Var_Expr_205.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_array
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Var_Expr_205.Node, Origin => Bound_Origin, E_Info => Var_Expr_205.Info);
--# end
--# expr-done 7242
Scope_Result_3 := Fld_16;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 7244 '<FieldAccess for name_symbol at nodes.lkt:10862:69>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
--# expr-start 7243 '<FieldAccess for attribute at nodes.lkt:10862:69>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 7243
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_17.Node);
--# end
--# expr-done 7244
--# expr-start 7245 '<Symbol First at nodes.lkt:10862:99>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 7245
Is_Equal_9 := Fld_18 = Sym_6; 
if Is_Equal_9 then
   If_Result_9 := True;
else
--# expr-start 7246 '<Symbol Last at nodes.lkt:10862:110>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 7246
Is_Equal_8 := Fld_18 = Sym_7; 
if Is_Equal_8 then
   If_Result_8 := True;
else
--# expr-start 7247 '<Symbol Length at nodes.lkt:10862:120>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 7247
Is_Equal_7 := Fld_18 = Sym_8; 
if Is_Equal_7 then
   If_Result_7 := True;
else
--# expr-start 7248 '<Symbol Range at nodes.lkt:10862:132>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10862'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 7248
Is_Equal_6 := Fld_18 = Sym_9; 
   If_Result_7 := Is_Equal_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
else
   If_Result_10 := False;
end if;
   If_Result_11 := If_Result_10;
end if;
   If_Result_12 := If_Result_11;
else
   If_Result_12 := False;
end if;
if If_Result_12 then
--# expr-start 7249 '<FieldAccess for args at nodes.lkt:10865:19>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10865'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Assoc_List (Node => Match_2.Node.Attribute_Ref_F_Args, Info => Match_2.Info);
--# expr-done 7249
Var_Expr_0 := Fld_19;
if Var_Expr_0 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind args Var_Expr_0
--# expr-start 7252 '<AllQuantifier at nodes.lkt:10866:27>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10866'
Saved := Var_Expr_0; 
Fld_20 := Saved.Node;
      Quantifier_Result := True;
      if Fld_20 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_20;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result_13 := No_Entity_Basic_Assoc;
else
Fld_21 := Saved.Info;
As_Entity := (Info => Fld_21, Node => Bare_Item_1); 
   If_Result_13 := As_Entity;
end if;
                  Item_1 := If_Result_13; 
   --# scope-start
               --# bind arg Item_1
      --# expr-start 7251 '<FieldAccess for is_static_expr at nodes.lkt:10866:45>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10866'
--# expr-start 7250 '<FieldAccess for expr at nodes.lkt:10866:45>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10866'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 7250
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_23 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_22.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_22.Info);
--# end
--# expr-done 7251
      Quantifier_Result := Fld_23;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 7252
Scope_Result_4 := Quantifier_Result;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := True;
end if;
   If_Result_14 := Result_Var_4;
else
   If_Result_14 := False;
end if;
Let_Result_2 := If_Result_14; 
--# end
Scope_Result_12 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ce Match_3
--# expr-start 7254 '<FieldAccess for is_static_expr at nodes.lkt:10871:34>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
--# expr-start 7253 '<FieldAccess for name at nodes.lkt:10871:34>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Match_3.Node.Call_Expr_F_Name, Info => Match_3.Info);
--# expr-done 7253
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_25 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_24.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_24.Info);
--# end
--# expr-done 7254
if Fld_25 then
   --# expr-start 7258 '<AllQuantifier at nodes.lkt:10871:63>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
   --# expr-start 7255 '<FieldAccess for params at nodes.lkt:10871:63>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_26 := Libadalang.Implementation.Call_Expr_P_Params (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 7255
Fld_27 := Fld_26.Node;
      Quantifier_Result_1 := True;
      if Fld_27 = null then
      null;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_27;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null_1 := Bare_Item_2 = null; 
if Is_Null_1 then
   If_Result_15 := No_Entity_Basic_Assoc;
else
Fld_28 := Fld_26.Info;
As_Entity_1 := (Info => Fld_28, Node => Bare_Item_2); 
   If_Result_15 := As_Entity_1;
end if;
                  Item_2 := If_Result_15; 
   --# scope-start
               --# bind pa Item_2
      --# expr-start 7257 '<FieldAccess for is_static_expr at nodes.lkt:10871:87>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
--# expr-start 7256 '<FieldAccess for expr at nodes.lkt:10871:87>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10871'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_2.Node, E_Info => Item_2.Info);
--# end
--# expr-done 7256
      if Fld_29.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_30 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_29.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_29.Info);
--# end
--# expr-done 7257
      Quantifier_Result_1 := Fld_30;
   --# end
               exit when not Quantifier_Result_1;
         end loop;
      end;
      end if;
--# expr-done 7258
   If_Result_16 := Quantifier_Result_1;
else
   If_Result_16 := False;
end if;
Let_Result_3 := If_Result_16; 
--# end
Scope_Result_13 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind qe Match_4
--# expr-start 7260 '<FieldAccess for name_designated_type at nodes.lkt:10872:34>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10872'
--# expr-start 7259 '<FieldAccess for prefix at nodes.lkt:10872:34>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10872'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Name (Node => Match_4.Node.Qual_Expr_F_Prefix, Info => Match_4.Info);
--# expr-done 7259
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_32 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_31.Node, E_Info => Fld_31.Info);
--# end
--# expr-done 7260
Var_Expr_206 := Fld_32;
if Var_Expr_206 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 7261 '<FieldAccess for is_static_decl at nodes.lkt:10872:34>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10872'
      if Var_Expr_206.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_33 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_206.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_206.Info);
--# end
--# expr-done 7261
Scope_Result_5 := Fld_33;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 7263 '<FieldAccess for is_static_expr at nodes.lkt:10872:89>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10872'
--# expr-start 7262 '<FieldAccess for suffix at nodes.lkt:10872:89>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10872'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Match_4.Node.Qual_Expr_F_Suffix, Info => Match_4.Info);
--# expr-done 7262
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_35 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_34.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_34.Info);
--# end
--# expr-done 7263
   If_Result_17 := Fld_35;
else
   If_Result_17 := False;
end if;
Let_Result_4 := If_Result_17; 
--# end
Scope_Result_14 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Array_Subcomponent_Choice_Name | Ada_Defining_Name .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Op_Xor | Ada_Null_Literal | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind n Match_5
--# expr-start 7264 '<FieldAccess for referenced_decl at nodes.lkt:10873:29>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10873'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_36 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 7264
Var_Expr_207 := Fld_36;
if Var_Expr_207 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7265 '<FieldAccess for is_static_decl at nodes.lkt:10873:29>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10873'
      if Var_Expr_207.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_37 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_207.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_207.Info);
--# end
--# expr-done 7265
Scope_Result_6 := Fld_37;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := False;
end if;
Let_Result_5 := Result_Var_6; 
--# end
Scope_Result_15 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Membership_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Membership_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_6; 
--# bind me Match_6
--# expr-start 7267 '<FieldAccess for is_static_expr at nodes.lkt:10874:40>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10874'
--# expr-start 7266 '<FieldAccess for expr at nodes.lkt:10874:40>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10874'
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Create_Internal_Entity_Expr (Node => Match_6.Node.Membership_Expr_F_Expr, Info => Match_6.Info);
--# expr-done 7266
      if Fld_38.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_39 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_38.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_38.Info);
--# end
--# expr-done 7267
if Fld_39 then
   --# expr-start 7270 '<AllQuantifier at nodes.lkt:10874:69>' Quantifier_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10874'
   --# expr-start 7268 '<FieldAccess for membership_exprs at nodes.lkt:10874:69>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10874'
      if Match_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Create_Internal_Entity_Expr_Alternatives_List (Node => Match_6.Node.Membership_Expr_F_Membership_Exprs, Info => Match_6.Info);
--# expr-done 7268
Fld_41 := Fld_40.Node;
      Quantifier_Result_2 := True;
      if Fld_41 = null then
      null;
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_41;
      begin
         for Untyped_Item_3 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_3 := Untyped_Item_3; 
Is_Null_2 := Bare_Item_3 = null; 
if Is_Null_2 then
   If_Result_18 := No_Entity_Expr;
else
Fld_42 := Fld_40.Info;
As_Entity_2 := (Info => Fld_42, Node => Bare_Item_3); 
   If_Result_18 := As_Entity_2;
end if;
                  Item_3 := If_Result_18; 
   --# scope-start
               --# bind e Item_3
      --# expr-start 7269 '<FieldAccess for is_static_expr at nodes.lkt:10874:100>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10874'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_43 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Item_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_3.Info);
--# end
--# expr-done 7269
      Quantifier_Result_2 := Fld_43;
   --# end
               exit when not Quantifier_Result_2;
         end loop;
      end;
      end if;
--# expr-done 7270
   If_Result_19 := Quantifier_Result_2;
else
   If_Result_19 := False;
end if;
Let_Result_6 := If_Result_19; 
--# end
Scope_Result_16 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_7; 
--# bind bo Match_7
--# expr-start 7272 '<FieldAccess for is_static_expr at nodes.lkt:10876:17>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10876'
--# expr-start 7271 '<FieldAccess for left at nodes.lkt:10876:17>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10876'
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Left, Info => Match_7.Info);
--# expr-done 7271
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_45 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_44.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_44.Info);
--# end
--# expr-done 7272
if Fld_45 then
   --# expr-start 7274 '<FieldAccess for is_static_expr at nodes.lkt:10876:46>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10876'
--# expr-start 7273 '<FieldAccess for right at nodes.lkt:10876:46>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10876'
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Right, Info => Match_7.Info);
--# expr-done 7273
      if Fld_46.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_47 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_46.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_46.Info);
--# end
--# expr-done 7274
   If_Result_20 := Fld_47;
else
   If_Result_20 := False;
end if;
if If_Result_20 then
--# expr-start 7276 '<FieldAccess for referenced_decl at nodes.lkt:10877:19>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10877'
--# expr-start 7275 '<FieldAccess for op at nodes.lkt:10877:19>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10877'
      if Match_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Op (Node => Match_7.Node.Bin_Op_F_Op, Info => Match_7.Info);
--# expr-done 7275
      if Fld_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_49 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_48.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_48.Info);
--# end
--# expr-done 7276
Var_Expr_4 := Fld_49;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_4
--# expr-start 7277 '<FieldAccess for is_static_decl at nodes.lkt:10878:27>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10878'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_50 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7277
Scope_Result_7 := Fld_50;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := True;
end if;
   If_Result_21 := Result_Var_7;
else
   If_Result_21 := False;
end if;
Let_Result_7 := If_Result_21; 
--# end
Scope_Result_17 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_Concat_Operand =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Concat_Operand
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_8; 
--# bind co Match_8
--# expr-start 7279 '<FieldAccess for is_static_expr at nodes.lkt:10880:39>' Fld_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10880'
--# expr-start 7278 '<FieldAccess for operand at nodes.lkt:10880:39>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10880'
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_51 := Create_Internal_Entity_Expr (Node => Match_8.Node.Concat_Operand_F_Operand, Info => Match_8.Info);
--# expr-done 7278
      if Fld_51.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_52 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_51.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_51.Info);
--# end
--# expr-done 7279
if Fld_52 then
--# expr-start 7281 '<FieldAccess for referenced_decl at nodes.lkt:10880:71>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10880'
--# expr-start 7280 '<FieldAccess for operator at nodes.lkt:10880:71>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10880'
      if Match_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Create_Internal_Entity_Op_Concat (Node => Match_8.Node.Concat_Operand_F_Operator, Info => Match_8.Info);
--# expr-done 7280
      if Fld_53.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_54 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_53.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_53.Info);
--# end
--# expr-done 7281
Var_Expr_5 := Fld_54;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_5
--# expr-start 7282 '<FieldAccess for is_static_decl at nodes.lkt:10881:27>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10881'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_55 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 7282
Scope_Result_8 := Fld_55;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := True;
end if;
   If_Result_22 := Result_Var_8;
else
   If_Result_22 := False;
end if;
Let_Result_8 := If_Result_22; 
--# end
Scope_Result_18 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Concat_Op =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Concat_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_9; 
--# bind co Match_9
--# expr-start 7284 '<FieldAccess for is_static_expr at nodes.lkt:10883:34>' Fld_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10883'
--# expr-start 7283 '<FieldAccess for first_operand at nodes.lkt:10883:34>' Fld_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10883'
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Create_Internal_Entity_Expr (Node => Match_9.Node.Concat_Op_F_First_Operand, Info => Match_9.Info);
--# expr-done 7283
      if Fld_56.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_57 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_56.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_56.Info);
--# end
--# expr-done 7284
if Fld_57 then
   --# expr-start 7287 '<AllQuantifier at nodes.lkt:10883:72>' Quantifier_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10883'
   --# expr-start 7285 '<FieldAccess for other_operands at nodes.lkt:10883:72>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10883'
      if Match_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Create_Internal_Entity_Concat_Operand_List (Node => Match_9.Node.Concat_Op_F_Other_Operands, Info => Match_9.Info);
--# expr-done 7285
Fld_59 := Fld_58.Node;
      Quantifier_Result_3 := True;
      if Fld_59 = null then
      null;
      else
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_59;
      begin
         for Untyped_Item_6 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_6 := Untyped_Item_6; 
Is_Null_3 := Bare_Item_6 = null; 
if Is_Null_3 then
   If_Result_23 := No_Entity_Concat_Operand;
else
Fld_60 := Fld_58.Info;
As_Entity_3 := (Info => Fld_60, Node => Bare_Item_6); 
   If_Result_23 := As_Entity_3;
end if;
                  Item_6 := If_Result_23; 
   --# scope-start
               --# bind o Item_6
      --# expr-start 7286 '<FieldAccess for is_static_expr at nodes.lkt:10883:101>' Fld_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10883'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_61 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Item_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_6.Info);
--# end
--# expr-done 7286
      Quantifier_Result_3 := Fld_61;
   --# end
               exit when not Quantifier_Result_3;
         end loop;
      end;
      end if;
--# expr-done 7287
   If_Result_24 := Quantifier_Result_3;
else
   If_Result_24 := False;
end if;
Let_Result_9 := If_Result_24; 
--# end
Scope_Result_19 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_19; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_10 := Cast_Result_10; 
--# bind uo Match_10
--# expr-start 7289 '<FieldAccess for is_static_expr at nodes.lkt:10884:30>' Fld_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10884'
--# expr-start 7288 '<FieldAccess for expr at nodes.lkt:10884:30>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10884'
      if Match_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_62 := Create_Internal_Entity_Expr (Node => Match_10.Node.Un_Op_F_Expr, Info => Match_10.Info);
--# expr-done 7288
      if Fld_62.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_63 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_62.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_62.Info);
--# end
--# expr-done 7289
if Fld_63 then
--# expr-start 7291 '<FieldAccess for referenced_decl at nodes.lkt:10884:59>' Fld_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10884'
--# expr-start 7290 '<FieldAccess for op at nodes.lkt:10884:59>' Fld_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10884'
      if Match_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_64 := Create_Internal_Entity_Op (Node => Match_10.Node.Un_Op_F_Op, Info => Match_10.Info);
--# expr-done 7290
      if Fld_64.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_65 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_64.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_64.Info);
--# end
--# expr-done 7291
Var_Expr_7 := Fld_65;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_7
--# expr-start 7292 '<FieldAccess for is_static_decl at nodes.lkt:10885:27>' Fld_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10885'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_66 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 7292
Scope_Result_9 := Fld_66;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := True;
end if;
   If_Result_25 := Result_Var_9;
else
   If_Result_25 := False;
end if;
Let_Result_10 := If_Result_25; 
--# end
Scope_Result_20 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_20; 
         when Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_If_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_11 := Cast_Result_11; 
--# bind i Match_11
--# expr-start 7294 '<FieldAccess for is_static_expr at nodes.lkt:10889:21>' Fld_68 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10889'
--# expr-start 7293 '<FieldAccess for cond_expr at nodes.lkt:10889:21>' Fld_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10889'
      if Match_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_67 := Create_Internal_Entity_Expr (Node => Match_11.Node.If_Expr_F_Cond_Expr, Info => Match_11.Info);
--# expr-done 7293
      if Fld_67.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_68 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_67.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_67.Info);
--# end
--# expr-done 7294
if Fld_68 then
   --# expr-start 7296 '<FieldAccess for is_static_expr at nodes.lkt:10889:54>' Fld_70 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10889'
--# expr-start 7295 '<FieldAccess for then_expr at nodes.lkt:10889:54>' Fld_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10889'
      if Match_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_69 := Create_Internal_Entity_Expr (Node => Match_11.Node.If_Expr_F_Then_Expr, Info => Match_11.Info);
--# expr-done 7295
      if Fld_69.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_70 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_69.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_69.Info);
--# end
--# expr-done 7296
   If_Result_26 := Fld_70;
else
   If_Result_26 := False;
end if;
if If_Result_26 then
   --# expr-start 7302 '<AllQuantifier at nodes.lkt:10890:23>' Quantifier_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10890'
   --# expr-start 7297 '<FieldAccess for alternatives at nodes.lkt:10890:23>' Fld_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10890'
      if Match_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_71 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Match_11.Node.If_Expr_F_Alternatives, Info => Match_11.Info);
--# expr-done 7297
Fld_72 := Fld_71.Node;
      Quantifier_Result_4 := True;
      if Fld_72 = null then
      null;
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_72;
      begin
         for Untyped_Item_8 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_8 := Untyped_Item_8; 
Is_Null_4 := Bare_Item_8 = null; 
if Is_Null_4 then
   If_Result_28 := No_Entity_Elsif_Expr_Part;
else
Fld_73 := Fld_71.Info;
As_Entity_4 := (Info => Fld_73, Node => Bare_Item_8); 
   If_Result_28 := As_Entity_4;
end if;
                  Item_8 := If_Result_28; 
   --# scope-start
               --# bind a Item_8
--# expr-start 7299 '<FieldAccess for is_static_expr at nodes.lkt:10891:28>' Fld_75 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10891'
--# expr-start 7298 '<FieldAccess for cond_expr at nodes.lkt:10891:28>' Fld_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10891'
      if Item_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_74 := Create_Internal_Entity_Expr (Node => Item_8.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_8.Info);
--# expr-done 7298
      if Fld_74.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_75 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_74.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_74.Info);
--# end
--# expr-done 7299
if Fld_75 then
   --# expr-start 7301 '<FieldAccess for is_static_expr at nodes.lkt:10891:61>' Fld_77 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10891'
--# expr-start 7300 '<FieldAccess for then_expr at nodes.lkt:10891:61>' Fld_76 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10891'
      if Item_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_76 := Create_Internal_Entity_Expr (Node => Item_8.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_8.Info);
--# expr-done 7300
      if Fld_76.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_77 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_76.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_76.Info);
--# end
--# expr-done 7301
   If_Result_27 := Fld_77;
else
   If_Result_27 := False;
end if;
      Quantifier_Result_4 := If_Result_27;
   --# end
               exit when not Quantifier_Result_4;
         end loop;
      end;
      end if;
--# expr-done 7302
   If_Result_29 := Quantifier_Result_4;
else
   If_Result_29 := False;
end if;
if If_Result_29 then
   --# expr-start 7304 '<FieldAccess for is_static_expr at nodes.lkt:10893:19>' Fld_79 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10893'
--# expr-start 7303 '<FieldAccess for else_expr at nodes.lkt:10893:19>' Fld_78 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10893'
      if Match_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_78 := Create_Internal_Entity_Expr (Node => Match_11.Node.If_Expr_F_Else_Expr, Info => Match_11.Info);
--# expr-done 7303
      if Fld_78.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_79 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_78.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_78.Info);
--# end
--# expr-done 7304
   If_Result_30 := Fld_79;
else
   If_Result_30 := False;
end if;
Let_Result_11 := If_Result_30; 
--# end
Scope_Result_21 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_21; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_12 := Cast_Result_12; 
--# bind pe Match_12
--# expr-start 7306 '<FieldAccess for is_static_expr at nodes.lkt:10894:35>' Fld_81 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10894'
--# expr-start 7305 '<FieldAccess for expr at nodes.lkt:10894:35>' Fld_80 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10894'
      if Match_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_80 := Create_Internal_Entity_Expr (Node => Match_12.Node.Paren_Expr_F_Expr, Info => Match_12.Info);
--# expr-done 7305
      if Fld_80.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_81 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_80.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_80.Info);
--# end
--# expr-done 7306
Let_Result_12 := Fld_81; 
--# end
Scope_Result_22 := Let_Result_12;
   --# end
            Match_Result := Scope_Result_22; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Case_Expr_Alternative | Ada_Case_Expr | Ada_Contract_Cases .. Ada_Format_String_Literal | Ada_Quantified_Expr .. Ada_Raise_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_13 := Cast_Result_13; 
--# bind _ Match_13
Let_Result_13 := False; 
--# end
Scope_Result_23 := Let_Result_13;
   --# end
            Match_Result := Scope_Result_23; 
end case;
--# expr-done 7307
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 7308
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Static_Expr;
--# end
--# property-start Expr.eval_as_int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10913'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_Int
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_181 with Inline_Always;
      Fld : Big_Integer_Type;
            procedure Finalizer_Scope_181 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_181;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7309 '<FieldAccess for eval_as_int_in_env at nodes.lkt:10914:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10914'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 7309
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_181;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_181;
            raise;
      end;
   return Property_Result;
end Expr_P_Eval_As_Int;
--# end
--# property-start Expr.eval_as_string 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10948'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_String
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_183 with Inline_Always;
      Fld : String_Type;
            procedure Finalizer_Scope_183 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_183;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7310 '<FieldAccess for eval_as_string_in_env at nodes.lkt:10949:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10949'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_String_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 7310
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_183;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_183;
            raise;
      end;
   return Property_Result;
end Expr_P_Eval_As_String;
--# end
--# property-start Expr.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10970'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Discrete_Range
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Match_0 : Internal_Entity_Attribute_Ref;
Match_1 : Internal_Entity_Name;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Match_2 : Internal_Entity_Bin_Op;
Match_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Discrete_Range;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Attribute_Ref;
Let_Result : Internal_Discrete_Range;
Scope_Result_1 : Internal_Discrete_Range;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_1 : Internal_Discrete_Range;
Scope_Result_2 : Internal_Discrete_Range;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result_2 : Internal_Discrete_Range;
Scope_Result_3 : Internal_Discrete_Range;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_3 : Internal_Discrete_Range;
Scope_Result_4 : Internal_Discrete_Range;
Match_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7318 '<Match at nodes.lkt:10970:43>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10970'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 7312 '<FieldAccess for discrete_range at nodes.lkt:10972:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10972'
--# expr-start 7311 '<FieldAccess for prefix at nodes.lkt:10972:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10972'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Attribute_Ref_F_Prefix, Info => Match_0.Info);
--# expr-done 7311
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_1 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7312
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 7313 '<FieldAccess for name_designated_type at nodes.lkt:10973:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10973'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7313
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr_0
--# expr-start 7314 '<FieldAccess for discrete_range at nodes.lkt:10973:61>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10973'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7314
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Discrete_Range;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind bo Match_2
--# expr-start 7317 '<New DiscreteRange at nodes.lkt:10974:27>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10974'
--# expr-start 7315 '<FieldAccess for right at nodes.lkt:10974:71>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10974'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Right, Info => Match_2.Info);
--# expr-done 7315
--# expr-start 7316 '<FieldAccess for left at nodes.lkt:10974:51>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10974'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Left, Info => Match_2.Info);
--# expr-done 7316
New_Struct := (High_Bound => Fld_5, Low_Bound => Fld_4); 
--# expr-done 7317
Let_Result_2 := New_Struct; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Discrete_Range; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 7318
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Discrete_Range;
--# end
--# property-start Expr.env_elements 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10979'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Env_Elements
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind no_visibility No_Visibility
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_186 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_186 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_186;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7321 '<Map at nodes.lkt:10980:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10980'
   --# expr-start 7319 '<FieldAccess for env_elements_impl at nodes.lkt:10980:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10980'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7319
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
         --# expr-start 7320 '<FieldAccess for has_visibility at nodes.lkt:10980:48>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10980'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 7320
         if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7321
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_186;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_186;
            raise;
      end;
   return Property_Result;
end Expr_P_Env_Elements;
--# end
--# property-start Expr.matching_nodes 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10985'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matching_Nodes
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_187 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_187 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_187;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7324 '<bind at nodes.lkt:10986:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10986'
--# expr-start 7322 '<FieldAccess for node_env at nodes.lkt:10986:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10986'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 7322
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 7323 '<FieldAccess for env_elements at nodes.lkt:10988:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:10988'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Bound_Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 7323
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7324
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_187;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_187;
            raise;
      end;
   return Property_Result;
end Expr_P_Matching_Nodes;
--# end
--# property-start '[dispatcher]Expr.has_context_free_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Has_Context_Free_Type
  (Node : Bare_Expr
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr | Ada_Box_Expr .. Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Format_String_Literal .. Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Identifier .. Ada_Op_Xor | Ada_Int_Literal .. Ada_Update_Attribute_Ref | Ada_Quantified_Expr =>
                  --# property-call-start Expr.has_context_free_type
                  Property_Result := Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.has_context_free_type
                  Property_Result := Allocator_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.has_context_free_type
                  Property_Result := Base_Aggregate_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.has_context_free_type
                  Property_Result := Bin_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.has_context_free_type
                  Property_Result := Relation_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Concat_Op =>
                  --# property-call-start ConcatOp.has_context_free_type
                  Property_Result := Concat_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Concat_Operand =>
                  --# property-call-start ConcatOperand.has_context_free_type
                  Property_Result := Concat_Operand_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.has_context_free_type
                  Property_Result := Case_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.has_context_free_type
                  Property_Result := If_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.has_context_free_type
                  Property_Result := Decl_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.has_context_free_type
                  Property_Result := Attribute_Ref_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.has_context_free_type
                  Property_Result := Dotted_Name_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.has_context_free_type
                  Property_Result := Char_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.has_context_free_type
                  Property_Result := String_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.has_context_free_type
                  Property_Result := Null_Literal_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.has_context_free_type
                  Property_Result := Paren_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.has_context_free_type
                  Property_Result := Raise_Expr_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.has_context_free_type
                  Property_Result := Un_Op_P_Has_Context_Free_Type
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Has_Context_Free_Type;
--# end
--# property-start '[dispatcher]Expr.potential_actuals_for_dispatch' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.potential_actuals_for_dispatch
                  Property_Result := Expr_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.potential_actuals_for_dispatch
                  Property_Result := Bin_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.potential_actuals_for_dispatch
                  Property_Result := Name_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.potential_actuals_for_dispatch
                  Property_Result := Un_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start '[dispatcher]Expr.is_dispatching_call' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.is_dispatching_call
                  Property_Result := Expr_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.is_dispatching_call
                  Property_Result := Bin_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Identifier | Ada_String_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.is_dispatching_call
                  Property_Result := Name_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.is_dispatching_call
                  Property_Result := Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.is_dispatching_call
                  Property_Result := Un_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Is_Dispatching_Call;
--# end
--# property-start '[dispatcher]Expr.first_corresponding_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
                  --# property-call-start Expr.first_corresponding_decl
                  Property_Result := Expr_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.first_corresponding_decl
                  Property_Result := Name_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_First_Corresponding_Decl;
--# end
--# property-start '[dispatcher]Expr.designated_env_no_overloading' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env_no_overloading
                  Property_Result := Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env_no_overloading
                  Property_Result := Call_Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env_no_overloading
                  Property_Result := Dotted_Name_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env_no_overloading
                  Property_Result := Base_Id_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start '[dispatcher]Expr.designated_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Synthetic_Identifier | Ada_Update_Attribute_Ref .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env
                  Property_Result := Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name =>
                  --# property-call-start ArraySubcomponentChoiceName.designated_env
                  Property_Result := Array_Subcomponent_Choice_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_env
                  Property_Result := Attribute_Ref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env
                  Property_Result := Call_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env
                  Property_Result := Dotted_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.designated_env
                  Property_Result := Explicit_Deref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.designated_env
                  Property_Result := Qual_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env
                  Property_Result := Base_Id_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.designated_env
                  Property_Result := Target_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , No_Visibility
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Designated_Env;
--# end
--# property-start '[dispatcher]Expr.env_elements_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Attribute_Ref | Ada_Discrete_Subtype_Name | Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.env_elements_impl
                  Property_Result := Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.env_elements_impl
                  Property_Result := Call_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.env_elements_impl
                  Property_Result := Defining_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.env_elements_impl
                  Property_Result := Dotted_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.env_elements_impl
                  Property_Result := End_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.env_elements_impl
                  Property_Result := Explicit_Deref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.env_elements_impl
                  Property_Result := Qual_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Op_Xor =>
                  --# property-call-start BaseId.env_elements_impl
                  Property_Result := Base_Id_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.env_elements_impl
                  Property_Result := String_Literal_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Expr_P_Env_Elements_Impl;
--# end
--# property-start Expr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11000'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Has_Context_Free_Type
  (Node : Bare_Expr
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Has_Context_Free_Type;
--# end
--# property-start Expr.potential_actuals_for_dispatch 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11011'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Expr.potential_actuals_for_dispatch not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Expr.is_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11031'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Precondition_Failure with "Invalid node type: expected Name, UnOp or BinOp";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Is_Dispatching_Call;
--# end
--# property-start Expr.first_corresponding_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11037'
pragma Warnings (Off, "is not referenced");
 function Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Basic_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.designated_env_no_overloading 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11045'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
     Ent : Internal_Entity_Expr :=
       Internal_Entity_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_192 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_192 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_192;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7325 '<FieldAccess for designated_env at nodes.lkt:11046:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11046'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Ent.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Ent.Info);
--# end
--# expr-done 7325
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_192;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_192;
            raise;
      end;
   return Property_Result;
end Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start Expr.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11055'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Expr.designated_env not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Designated_Env;
--# end
--# property-start Expr.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11060'
pragma Warnings (Off, "is not referenced");
 function Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr  := Bare_Expr (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Expr.env_elements_impl not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Expr_P_Env_Elements_Impl;
--# end
   --
   --  Primitives for Bare_Abstract_State_Decl_Expr
   --
      procedure Initialize_Fields_For_Abstract_State_Decl_Expr
        (Self : Bare_Abstract_State_Decl_Expr
         ; Abstract_State_Decl_Expr_F_State_Decl : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Abstract_State_Decl_Expr_F_State_Decl := Abstract_State_Decl_Expr_F_State_Decl;
      end Initialize_Fields_For_Abstract_State_Decl_Expr;
   function Abstract_State_Decl_Expr_F_State_Decl
     (Node : Bare_Abstract_State_Decl_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Abstract_State_Decl_Expr_F_State_Decl;
   end;
--# property-start AbstractStateDeclExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11070'
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_Expr_P_Xref_Equation
  (Node : Bare_Abstract_State_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abstract_State_Decl_Expr  := Bare_Abstract_State_Decl_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1097 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1097 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1097;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11070:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1097;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1097;
            raise;
      end;
   return Property_Result;
end Abstract_State_Decl_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Allocator
   --
      procedure Initialize_Fields_For_Allocator
        (Self : Bare_Allocator
         ; Allocator_F_Subpool : Bare_Name
         ; Allocator_F_Type_Or_Expr : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Allocator_F_Subpool := Allocator_F_Subpool;
            Self.Allocator_F_Type_Or_Expr := Allocator_F_Type_Or_Expr;
      end Initialize_Fields_For_Allocator;
   function Allocator_F_Subpool
     (Node : Bare_Allocator) return Bare_Name
   is
   begin
         return Node.Allocator_F_Subpool;
   end;
   function Allocator_F_Type_Or_Expr
     (Node : Bare_Allocator) return Bare_Ada_Node
   is
   begin
         return Node.Allocator_F_Type_Or_Expr;
   end;
--# property-start Allocator.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11078'
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Has_Context_Free_Type
  (Node : Bare_Allocator
  )
   return Boolean
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Allocator_P_Has_Context_Free_Type;
--# end
--# property-start Allocator.get_allocated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11082'
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Get_Allocated_Type
  (Node : Bare_Allocator
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
     Ent : Internal_Entity_Allocator :=
       Internal_Entity_Allocator'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Qual_Expr;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7332 '<bind at nodes.lkt:11083:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11083'
--# expr-start 7326 '<FieldAccess for origin_node at nodes.lkt:11083:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11083'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7326
Bound_Origin := Fld; 
--# expr-start 7331 '<Match at nodes.lkt:11085:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11085'
--# expr-start 7327 '<FieldAccess for type_or_expr at nodes.lkt:11085:15>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11085'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 7327
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 7328 '<FieldAccess for designated_type at nodes.lkt:11086:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11086'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 7328
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind q Match_1
--# expr-start 7330 '<FieldAccess for name_designated_type at nodes.lkt:11087:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11087'
--# expr-start 7329 '<FieldAccess for prefix at nodes.lkt:11087:41>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11087'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_1.Node.Qual_Expr_F_Prefix, Info => Match_1.Info);
--# expr-done 7329
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 7330
Let_Result_1 := Fld_4; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Explicit_Deref | Ada_Reduce_Attribute_Ref .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7331
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 7332
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Allocator_P_Get_Allocated_Type;
--# end
--# property-start Allocator.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11093'
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Xref_Equation
  (Node : Bare_Allocator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Allocator  := Bare_Allocator (Node);
     Ent : Internal_Entity_Allocator :=
       Internal_Entity_Allocator'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1100 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1100 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1100;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7342 '<And at nodes.lkt:11093:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11093'
--# expr-start 7338 '<And at nodes.lkt:11094:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
--# expr-start 7334 '<FieldAccess for sub_equation at nodes.lkt:11094:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
--# expr-start 7333 '<FieldAccess for type_or_expr at nodes.lkt:11094:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 7333
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 7334
--# expr-start 7337 '<Bind at nodes.lkt:11094:46>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
--# expr-start 7335 '<FieldAccess for expected_type_var at nodes.lkt:11094:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7335
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7336 '<FieldAccess for type_var at nodes.lkt:11094:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7336
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11094:46") else null)); 
--# expr-done 7337
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11094:9") else null)); 
--# expr-done 7338
--# expr-start 7341 '<Predicate on BaseTypeDecl.matching_allocator_type at nodes.lkt:11095:11>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11095'
--# expr-start 7339 '<FieldAccess for type_var at nodes.lkt:11095:60>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11095'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7339
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 7340 '<FieldAccess for get_allocated_type at nodes.lkt:11095:77>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11095'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Allocator.get_allocated_type
Fld_5 := Libadalang.Implementation.Allocator_P_Get_Allocated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7340
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (Fld_5, Origin)); 
--# expr-done 7341
And_Pred_1 := Create_And (And_Pred, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11093:37") else null)); 
--# expr-done 7342
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1100;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1100;
            raise;
      end;
   return Property_Result;
end Allocator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Base_Aggregate
   --
      procedure Initialize_Fields_For_Base_Aggregate
        (Self : Bare_Base_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Base_Aggregate_F_Ancestor_Expr := Base_Aggregate_F_Ancestor_Expr;
            Self.Base_Aggregate_F_Assocs := Base_Aggregate_F_Assocs;
      end Initialize_Fields_For_Base_Aggregate;
   function Base_Aggregate_F_Ancestor_Expr
     (Node : Bare_Base_Aggregate) return Bare_Expr
   is
   begin
         return Node.Base_Aggregate_F_Ancestor_Expr;
   end;
   function Base_Aggregate_F_Assocs
     (Node : Bare_Base_Aggregate) return Bare_Assoc_List
   is
   begin
         return Node.Base_Aggregate_F_Assocs;
   end;
--# property-start BaseAggregate.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11104'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Has_Context_Free_Type
  (Node : Bare_Base_Aggregate
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Has_Context_Free_Type;
--# end
--# property-start BaseAggregate.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11107'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Xref_Equation
  (Node : Bare_Base_Aggregate
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1102 with Inline_Always;
            procedure Finalizer_Scope_3024 with Inline_Always;
            procedure Finalizer_Scope_3025 with Inline_Always;
      Local_Type_Constraint : Logic_Equation;
Var_Expr_0 : Internal_Entity_Expr;
True_Rel : Logic_Equation;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Sym : Symbol_Type;
Fld_4 : Boolean;
Sym_1 : Symbol_Type;
Fld_5 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_6 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Fld_8 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Fld_9 : Boolean;
If_Result_4 : Boolean;
Sym_6 : Symbol_Type;
Fld_10 : Boolean;
If_Result_5 : Boolean;
Sym_7 : Symbol_Type;
Fld_11 : Boolean;
If_Result_6 : Boolean;
Fld_12 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Var;
Fld_18 : Bare_Expr;
Pred_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_A : Boolean;
If_Result_9 : Logic_Equation;
Is_A_1 : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_19 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_20 : Internal_Entity_Assoc_List;
Fld_21 : Bare_Assoc_List;
Fld_22 : Internal_Entity_Info;
Item_1 : Internal_Entity_Basic_Assoc;
Bare_Item_1 : Bare_Basic_Assoc;
Fld_23 : Logic_Equation;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_12 : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1102 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Local_Type_Constraint);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1102;
            procedure Finalizer_Scope_3024 is
            begin
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_11);
            end Finalizer_Scope_3024;
            procedure Finalizer_Scope_3025 is
            begin
                     Dec_Ref (Fld_23);
            end Finalizer_Scope_3025;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7392 '<Let at nodes.lkt:11113:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11113'
--# scope-start
--# expr-start 7368 '<If at nodes.lkt:11113:31>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11113'
--# expr-start 7344 '<FieldAccess for in_aspect at nodes.lkt:11117:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11117'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7343 '<Symbol Global at nodes.lkt:11117:28>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11117'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Global); 
--# expr-done 7343
--# property-call-start AdaNode.in_aspect
Fld_4 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 7344
if Fld_4 then
   If_Result := True;
else
   --# expr-start 7346 '<FieldAccess for in_aspect at nodes.lkt:11118:16>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11118'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7345 '<Symbol Refined_Global at nodes.lkt:11118:31>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11118'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Global); 
--# expr-done 7345
--# property-call-start AdaNode.in_aspect
Fld_5 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 7346
   If_Result := Fld_5;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7348 '<FieldAccess for in_aspect at nodes.lkt:11119:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11119'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7347 '<Symbol Depends at nodes.lkt:11119:31>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11119'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 7347
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_2);
--# end
--# expr-done 7348
   If_Result_1 := Fld_6;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7350 '<FieldAccess for in_aspect at nodes.lkt:11120:16>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11120'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7349 '<Symbol Refined_Depends at nodes.lkt:11120:31>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11120'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 7349
--# property-call-start AdaNode.in_aspect
Fld_7 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_3);
--# end
--# expr-done 7350
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 7352 '<FieldAccess for in_aspect at nodes.lkt:11121:16>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11121'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7351 '<Symbol Test_Case at nodes.lkt:11121:31>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11121'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 7351
--# property-call-start AdaNode.in_aspect
Fld_8 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_4);
--# end
--# expr-done 7352
   If_Result_3 := Fld_8;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 7354 '<FieldAccess for in_aspect at nodes.lkt:11122:16>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11122'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7353 '<Symbol Refined_State at nodes.lkt:11122:31>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11122'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_State); 
--# expr-done 7353
--# property-call-start AdaNode.in_aspect
Fld_9 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_5);
--# end
--# expr-done 7354
   If_Result_4 := Fld_9;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 7356 '<FieldAccess for in_aspect at nodes.lkt:11123:16>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11123'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7355 '<Symbol Aggregate at nodes.lkt:11123:31>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11123'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aggregate); 
--# expr-done 7355
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_6);
--# end
--# expr-done 7356
   If_Result_5 := Fld_10;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7358 '<FieldAccess for in_aspect at nodes.lkt:11124:16>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11124'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7357 '<Symbol Subprogram_Variant at nodes.lkt:11124:31>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11124'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Subprogram_Variant); 
--# expr-done 7357
--# property-call-start AdaNode.in_aspect
Fld_11 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_7);
--# end
--# expr-done 7358
   If_Result_6 := Fld_11;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7359 '<FieldAccess for is_contract_cases_base_aggregate at nodes.lkt:11129:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11129'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.is_contract_cases_base_aggregate
Fld_12 := Libadalang.Implementation.Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate (Node => Self);
--# end
--# expr-done 7359
   If_Result_7 := Fld_12;
end if;
if If_Result_7 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11130:16") else null)); 
   If_Result_8 := True_Rel;
else
   --# expr-start 7367 '<bind at nodes.lkt:11131:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11131'
--# expr-start 7360 '<FieldAccess for origin_node at nodes.lkt:11131:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11131'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7360
Bound_Origin := Fld; 
--# expr-start 7366 '<And at nodes.lkt:11133:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
--# expr-start 7362 '<Predicate on BaseTypeDecl.is_array_or_rec at nodes.lkt:11133:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
--# expr-start 7361 '<FieldAccess for expected_type_var at nodes.lkt:11133:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7361
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (Bound_Origin)); 
--# expr-done 7362
--# expr-start 7365 '<Bind at nodes.lkt:11133:84>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
--# expr-start 7363 '<FieldAccess for expected_type_var at nodes.lkt:11133:88>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7363
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7364 '<FieldAccess for type_var at nodes.lkt:11133:114>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7364
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11133:84") else null)); 
--# expr-done 7365
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11133:13") else null)); 
--# expr-done 7366
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7367
   If_Result_8 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 7368
Local_Type_Constraint := If_Result_8; Inc_Ref (Local_Type_Constraint);
--# bind type_constraint Local_Type_Constraint
--# expr-start 7391 '<And at nodes.lkt:11136:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11136'
--# expr-start 7386 '<And at nodes.lkt:11137:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11137'
--# expr-start 7369 '<FieldAccess for ancestor_expr at nodes.lkt:11137:33>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11137'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 7369
Var_Expr_0 := Fld_13;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr_0
--# expr-start 7385 '<If at nodes.lkt:11142:17>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7370 '<IsA Name at nodes.lkt:11142:20>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
Is_A_1 := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Name; 
--# expr-done 7370
if Is_A_1 then
   --# expr-start 7374 '<Not at nodes.lkt:11142:35>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7373 '<IsNull at nodes.lkt:11142:39>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7372 '<FieldAccess for name_designated_type at nodes.lkt:11142:39>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7371 '<Cast to Name at nodes.lkt:11142:39>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
Cast_Expr_1 := Var_Expr_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 7371
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_19 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7372
Is_Null := Fld_19.Node = null; 
--# expr-done 7373
Not_Val := not (Is_Null); 
--# expr-done 7374
   If_Result_10 := Not_Val;
else
   If_Result_10 := False;
end if;
if If_Result_10 then
   --# expr-start 7376 '<FieldAccess for xref_type_equation at nodes.lkt:11142:87>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7375 '<Cast to Name at nodes.lkt:11142:87>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 7375
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_14 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result.Info);
--# end
--# expr-done 7376
   If_Result_11 := Fld_14;
else
   --# expr-start 7384 '<If at nodes.lkt:11142:17>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11142'
--# expr-start 7377 '<IsA DeltaAggregate at nodes.lkt:11143:22>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11143'
Is_A := Self /= null 
and then Self.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 7377
if Is_A then
   --# expr-start 7378 '<FieldAccess for sub_equation at nodes.lkt:11143:50>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11143'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7378
   If_Result_9 := Fld_15;
else
   --# expr-start 7383 '<And at nodes.lkt:11149:22>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11149'
--# expr-start 7379 '<FieldAccess for sub_equation at nodes.lkt:11149:22>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11149'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7379
--# expr-start 7382 '<Predicate on BaseTypeDecl.is_tagged_type_with_deref at nodes.lkt:11149:44>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11149'
--# expr-start 7380 '<FieldAccess for type_var at nodes.lkt:11149:95>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11149'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 7380
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
--# expr-start 7381 '<FieldAccess for node at nodes.lkt:11149:125>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11149'
Fld_18 := Var_Expr_0.Node;
--# expr-done 7381
Pred_1 := Solver.Create_Predicate (Fld_17, Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate (Origin, Fld_18)); 
--# expr-done 7382
And_Pred_1 := Create_And (Fld_16, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11149:22") else null)); 
--# expr-done 7383
   If_Result_9 := And_Pred_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 7384
   If_Result_11 := If_Result_9;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 7385
Scope_Result := If_Result_11;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3024;
   Result_Var := Scope_Result;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11149:147") else null)); 
   Result_Var := True_Rel_1;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (Local_Type_Constraint, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11137:13") else null)); 
--# expr-done 7386
--# expr-start 7390 '<LogicAll at nodes.lkt:11151:15>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11151'
--# expr-start 7389 '<Map at nodes.lkt:11151:15>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11151'
   --# expr-start 7387 '<FieldAccess for assocs at nodes.lkt:11151:15>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7387
Fld_21 := Fld_20.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_21 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_21;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_12 := No_Entity_Basic_Assoc;
else
Fld_22 := Fld_20.Info;
As_Entity := (Info => Fld_22, Node => Bare_Item_1); 
   If_Result_12 := As_Entity;
end if;
                  Item_1 := If_Result_12; 
   --# scope-start
               --# bind assoc Item_1
      --# expr-start 7388 '<FieldAccess for sub_equation at nodes.lkt:11151:48>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11151'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_23 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_1.Info);
--# end
--# expr-done 7388
         declare
            Item_To_Append : constant Logic_Equation := Fld_23;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3025;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7389
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11151:15") else null)); 
--# expr-done 7390
And_Pred_3 := Create_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11136:9") else null)); 
--# expr-done 7391
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7392
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1102;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1102;
                     Finalizer_Scope_3024;
                     Finalizer_Scope_3025;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Xref_Equation;
--# end
--# property-start BaseAggregate.is_contract_cases_base_aggregate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11156'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate
  (Node : Bare_Base_Aggregate
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_0 : Bare_Aspect_Assoc;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld_1 : Bare_Name;
Sym : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 7394 '<Cast to AspectAssoc at nodes.lkt:11156:52>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11156'
--# expr-start 7393 '<parent at nodes.lkt:11156:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11156'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 7393
   if Fld = null
      or else Fld.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7394
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind aspect Var_Expr_0
--# expr-start 7397 '<FieldAccess for name_is at nodes.lkt:11157:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11157'
--# expr-start 7395 '<FieldAccess for id at nodes.lkt:11157:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11157'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_0.Aspect_Assoc_F_Id;
--# expr-done 7395
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7396 '<Symbol Contract_Cases at nodes.lkt:11157:39>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11157'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 7396
--# property-call-start Name.name_is
Fld_2 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 7397
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Is_Contract_Cases_Base_Aggregate;
--# end
--# property-start BaseAggregate.multidim_root_aggregate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11165'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Multidim_Root_Aggregate
  (Node : Bare_Base_Aggregate
      ; R : Integer
         := 0
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Multidim_Aggregate_Info
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind r R
   --# bind origin Origin
   Property_Result : Internal_Multidim_Aggregate_Info;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_208 : Internal_Entity_Aggregate;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Struct : Internal_Multidim_Aggregate_Info;
Fld_1 : Integer;
Comp_Result : Boolean;
If_Result : Internal_Multidim_Aggregate_Info;
Scope_Result : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Aggregate;
Arith_Result : Integer;
Fld_5 : Internal_Multidim_Aggregate_Info;
Scope_Result_1 : Internal_Multidim_Aggregate_Info;
Result_Var : Internal_Multidim_Aggregate_Info;
Result_Var_1 : Internal_Multidim_Aggregate_Info;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Multidim_Root_Aggregate,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Integer,
                                      As_Integer => R);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Multidim_Aggregate_Info;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 7399 '<Cast to BaseTypeDecl at nodes.lkt:11168:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11168'
--# expr-start 7398 '<FieldAccess for type_val at nodes.lkt:11168:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11168'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Ent.Node);
--# end
--# expr-done 7398
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7399
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind tv Var_Expr_0
--# expr-start 7402 '<If at nodes.lkt:11172:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11172'
--# expr-start 7400 '<FieldAccess for array_ndims at nodes.lkt:11172:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11172'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7400
Comp_Result := Fld_1 > 1; 
if Comp_Result then
   --# expr-start 7401 '<New MultidimAggregateInfo at nodes.lkt:11172:42>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11172'
New_Struct := (Agg => Ent, Rank => R, Typ => Var_Expr_0); 
--# expr-done 7401
   If_Result := New_Struct;
else
   If_Result := No_Multidim_Aggregate_Info;
end if;
--# expr-done 7402
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 7406 '<Cast to Aggregate at nodes.lkt:11179:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
--# expr-start 7405 '<parent at nodes.lkt:11179:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
--# expr-start 7404 '<parent at nodes.lkt:11179:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
--# expr-start 7403 '<parent at nodes.lkt:11179:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7403
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 7404
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 7405
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 7406
Var_Expr_208 := Cast_Result_1;
if Var_Expr_208 /= No_Entity_Aggregate then
   --# scope-start
--# expr-start 7408 '<FieldAccess for multidim_root_aggregate at nodes.lkt:11179:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
      if Var_Expr_208.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7407 '<Arithmetic + at nodes.lkt:11179:90>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11179'
Arith_Result := (R + 1); 
--# expr-done 7407
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_5 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Var_Expr_208.Node, R => Arith_Result, Origin => Origin, E_Info => Var_Expr_208.Info);
--# end
--# expr-done 7408
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Multidim_Aggregate_Info;
end if;
   Result_Var_1 := Result_Var;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Multidim_Aggregate_Info,
                        As_Internal_Multidim_Aggregate_Info => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Multidim_Root_Aggregate;
--# end
--# property-start BaseAggregate.all_discriminants 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11198'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1105 with Inline_Always;
      Local_Td : Internal_Entity_Base_Type_Decl;
Local_Stop_Recurse_At : Internal_Entity_Base_Type_Decl;
Local_Record_Decl : Internal_Entity_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Internal_Entity_Type_Decl;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1105 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1105;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7418 '<Let at nodes.lkt:11199:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11199'
--# scope-start
--# expr-start 7410 '<Cast to BaseTypeDecl at nodes.lkt:11199:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11199'
--# expr-start 7409 '<FieldAccess for type_val at nodes.lkt:11199:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11199'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 7409
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7410
Local_Td := Cast_Result; 
--# bind td Local_Td
--# expr-start 7417 '<Let at nodes.lkt:11200:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11200'
--# scope-start
--# expr-start 7411 '<FieldAccess for ancestor_expr_type at nodes.lkt:11200:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11200'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Ent.Node, Resolve_Type => False, E_Info => Ent.Info);
--# end
--# expr-done 7411
Local_Stop_Recurse_At := Fld_1; 
--# bind stop_recurse_at Local_Stop_Recurse_At
--# expr-start 7416 '<Let at nodes.lkt:11201:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11201'
--# scope-start
--# expr-start 7414 '<FieldAccess for type_decl at nodes.lkt:11201:27>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11201'
--# expr-start 7413 '<FieldAccess for comps at nodes.lkt:11201:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11201'
--# expr-start 7412 '<FieldAccess for record_def at nodes.lkt:11201:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11201'
      if Local_Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Local_Td.Node, Origin => Origin, E_Info => Local_Td.Info);
--# end
--# expr-done 7412
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 7413
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.type_decl
Fld_4 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 7414
Local_Record_Decl := Fld_4; 
--# bind record_decl Local_Record_Decl
--# expr-start 7415 '<FieldAccess for discriminants_list at nodes.lkt:11203:9>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11203'
      if Local_Record_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Local_Record_Decl.Node, Stop_Recurse_At => Local_Stop_Recurse_At, Origin => Origin, E_Info => Local_Record_Decl.Info);
--# end
--# expr-done 7415
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
--# expr-done 7416
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7417
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 7418
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1105;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1105;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                        As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_All_Discriminants;
--# end
--# property-start BaseAggregate.all_components 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11215'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1106 with Inline_Always;
      Local_Td : Internal_Entity_Base_Type_Decl;
Local_Stop_Recurse_At : Internal_Entity_Base_Type_Decl;
Local_Comp_List : Internal_Entity_Component_List;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Assoc_List;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Is_A : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1106 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1106;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7431 '<Let at nodes.lkt:11216:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11216'
--# scope-start
--# expr-start 7420 '<Cast to BaseTypeDecl at nodes.lkt:11216:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11216'
--# expr-start 7419 '<FieldAccess for type_val at nodes.lkt:11216:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11216'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 7419
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7420
Local_Td := Cast_Result; 
--# bind td Local_Td
--# expr-start 7430 '<Let at nodes.lkt:11217:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11217'
--# scope-start
--# expr-start 7421 '<FieldAccess for ancestor_expr_type at nodes.lkt:11217:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11217'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.ancestor_expr_type
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Ancestor_Expr_Type (Node => Ent.Node, Resolve_Type => False, E_Info => Ent.Info);
--# end
--# expr-done 7421
Local_Stop_Recurse_At := Fld_1; 
--# bind stop_recurse_at Local_Stop_Recurse_At
--# expr-start 7429 '<Let at nodes.lkt:11218:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11218'
--# scope-start
--# expr-start 7423 '<FieldAccess for comps at nodes.lkt:11218:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11218'
--# expr-start 7422 '<FieldAccess for record_def at nodes.lkt:11218:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11218'
      if Local_Td.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Local_Td.Node, Origin => Origin, E_Info => Local_Td.Info);
--# end
--# expr-done 7422
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 7423
Local_Comp_List := Fld_3; 
--# bind comp_list Local_Comp_List
--# expr-start 7428 '<If at nodes.lkt:11220:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11220'
--# expr-start 7424 '<IsA DeltaAggregate at nodes.lkt:11220:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11220'
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 7424
if Is_A then
   --# expr-start 7425 '<FieldAccess for abstract_formal_params_for_delta_assocs at nodes.lkt:11223:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11223'
      if Local_Comp_List.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_delta_assocs
Fld_4 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Delta_Assocs (Node => Local_Comp_List.Node, E_Info => Local_Comp_List.Info);
--# end
--# expr-done 7425
   If_Result := Fld_4;
else
   --# expr-start 7427 '<FieldAccess for abstract_formal_params_for_assocs at nodes.lkt:11224:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11224'
      if Local_Comp_List.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7426 '<FieldAccess for assocs at nodes.lkt:11224:60>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11224'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7426
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_6 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Local_Comp_List.Node, Assocs => Fld_5, Stop_Recurse_At => Local_Stop_Recurse_At, Env => Env, Origin => Origin, E_Info => Local_Comp_List.Info);
--# end
--# expr-done 7427
   If_Result := Fld_6;
end if;
      Inc_Ref (If_Result);
--# expr-done 7428
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7429
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7430
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 7431
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1106;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1106;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                        As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_All_Components;
--# end
--# property-start BaseAggregate.matched_discriminants 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11235'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1107 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1107 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1107;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7434 '<FieldAccess for match_formals at nodes.lkt:11235:54>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11235'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7432 '<FieldAccess for all_discriminants at nodes.lkt:11236:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11236'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7432
--# expr-start 7433 '<FieldAccess for assocs at nodes.lkt:11236:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11236'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7433
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 7434
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1107;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1107;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                        As_Internal_Param_Match_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Matched_Discriminants;
--# end
--# property-start BaseAggregate.matched_components 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11247'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1108 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_1108 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1108;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 7437 '<FieldAccess for match_formals at nodes.lkt:11247:51>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11247'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7435 '<FieldAccess for all_components at nodes.lkt:11248:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11248'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_components
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7435
--# expr-start 7436 '<FieldAccess for assocs at nodes.lkt:11248:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11248'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7436
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 7437
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1108;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1108;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                        As_Internal_Param_Match_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Aggregate_P_Matched_Components;
--# end
--# property-start BaseAggregate.first_unmatched_formal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11258'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_First_Unmatched_Formal
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1109 with Inline_Always;
            procedure Finalizer_Scope_3029 with Inline_Always;
            procedure Finalizer_Scope_3031 with Inline_Always;
      Local_Unmatched_Discr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Fld_6 : Internal_Param_Match_Array_Access;
Fld_7 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val_1 : Boolean;
Find_Result_1 : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val_2 : Boolean;
If_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1109 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1109;
            procedure Finalizer_Scope_3029 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3029;
            procedure Finalizer_Scope_3031 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3031;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7457 '<Let at nodes.lkt:11260:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11260'
--# scope-start
--# expr-start 7445 '<Find at nodes.lkt:11260:31>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11260'
   --# expr-start 7439 '<FieldAccess for unpack_formals at nodes.lkt:11260:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11260'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7438 '<FieldAccess for all_discriminants at nodes.lkt:11260:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11260'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7438
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 7439
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_0
      --# expr-start 7444 '<Not at nodes.lkt:11261:20>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11261'
--# expr-start 7443 '<AnyQuantifier at nodes.lkt:11261:24>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11261'
   --# expr-start 7440 '<FieldAccess for matched_discriminants at nodes.lkt:11261:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11261'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_2 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7440
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_2;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_1
      --# expr-start 7442 '<Eq at nodes.lkt:11261:64>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11261'
--# expr-start 7441 '<FieldAccess for formal at nodes.lkt:11261:64>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11261'
Fld_3 := Item_1.Formal;
--# expr-done 7441
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Item_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 7442
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 7443
Not_Val := not (Quantifier_Result); 
--# expr-done 7444
      if Not_Val then
         Find_Result := Item_0; 
      end if;
   --# end
      Finalizer_Scope_3029;
               exit when Not_Val;
         end loop;
      end;
--# expr-done 7445
Local_Unmatched_Discr := Find_Result; 
--# bind unmatched_discr Local_Unmatched_Discr
--# expr-start 7456 '<If at nodes.lkt:11264:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11264'
--# expr-start 7447 '<Not at nodes.lkt:11264:12>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11264'
--# expr-start 7446 '<IsNull at nodes.lkt:11264:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11264'
Is_Null := Local_Unmatched_Discr.Node = null; 
--# expr-done 7446
Not_Val_2 := not (Is_Null); 
--# expr-done 7447
if Not_Val_2 then
   If_Result := Local_Unmatched_Discr;
else
   --# expr-start 7455 '<Find at nodes.lkt:11271:13>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11271'
   --# expr-start 7449 '<FieldAccess for unpack_formals at nodes.lkt:11271:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11271'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7448 '<FieldAccess for all_components at nodes.lkt:11271:33>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11271'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.all_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7448
--# property-call-start AdaNode.unpack_formals
Fld_5 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld_4);
--# end
--# expr-done 7449
      Find_Result_1 := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_5;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_2
      --# expr-start 7454 '<Not at nodes.lkt:11272:24>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11272'
--# expr-start 7453 '<AnyQuantifier at nodes.lkt:11272:28>' Quantifier_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11272'
   --# expr-start 7450 '<FieldAccess for matched_components at nodes.lkt:11272:28>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11272'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_6 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7450
      Quantifier_Result_1 := False;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_6;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_3
      --# expr-start 7452 '<Eq at nodes.lkt:11272:65>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11272'
--# expr-start 7451 '<FieldAccess for formal at nodes.lkt:11272:65>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11272'
Fld_7 := Item_3.Formal;
--# expr-done 7451
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Cast_Expr_1 := Item_2; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 7452
      Quantifier_Result_1 := Is_Equiv_1;
   --# end
               exit when Quantifier_Result_1;
         end loop;
      end;
--# expr-done 7453
Not_Val_1 := not (Quantifier_Result_1); 
--# expr-done 7454
      if Not_Val_1 then
         Find_Result_1 := Item_2; 
      end if;
   --# end
      Finalizer_Scope_3031;
               exit when Not_Val_1;
         end loop;
      end;
--# expr-done 7455
   If_Result := Find_Result_1;
end if;
--# expr-done 7456
Let_Result := If_Result; 
--# end
--# expr-done 7457
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1109;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1109;
                     Finalizer_Scope_3029;
                     Finalizer_Scope_3031;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_First_Unmatched_Formal;
--# end
--# property-start BaseAggregate.aggregate_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11280'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Aggregate_Params
  (Node : Bare_Base_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1110 with Inline_Always;
      Fld : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1110 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1110;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7459 '<FieldAccess for zip_with_params at nodes.lkt:11280:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11280'
--# expr-start 7458 '<FieldAccess for assocs at nodes.lkt:11280:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11280'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7458
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Fld.Node, Imprecise_Fallback => False, E_Info => Fld.Info);
--# end
--# expr-done 7459
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1110;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1110;
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Aggregate_Params;
--# end
--# property-start BaseAggregate.is_subaggregate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11285'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Is_Subaggregate
  (Node : Bare_Base_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Ignored : Boolean;
Fld : Boolean;
Cast_Expr : Bare_Base_Aggregate;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Integer;
Comp_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 7464 '<Let at nodes.lkt:11288:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11288'
--# scope-start
--# expr-start 7460 '<FieldAccess for resolve_names_from_closest_entry_point at nodes.lkt:11288:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11288'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7460
Ignored := Fld; 
--# bind _ Ignored
--# expr-start 7463 '<bind at nodes.lkt:11291:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11291'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 7462 '<FieldAccess for rank at nodes.lkt:11293:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11293'
--# expr-start 7461 '<FieldAccess for multidim_root_aggregate at nodes.lkt:11293:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11293'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 7461
Fld_2 := Fld_1.Rank;
--# expr-done 7462
Comp_Result := Fld_2 > 0; 
Dyn_Var_Bind_Result := Comp_Result; 
--# expr-done 7463
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 7464
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Is_Subaggregate;
--# end
--# property-start BaseAggregate.ancestor_expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11304'
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Ancestor_Expr_Type
  (Node : Bare_Base_Aggregate
      ; Resolve_Type : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Aggregate  := Bare_Base_Aggregate (Node);
     Ent : Internal_Entity_Base_Aggregate :=
       Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind resolve_type Resolve_Type
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 7465 '<FieldAccess for ancestor_expr at nodes.lkt:11304:79>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11304'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 7465
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr_0
--# expr-start 7466 '<Cast to Name at nodes.lkt:11305:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11305'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 7466
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_1
--# expr-start 7467 '<FieldAccess for name_designated_type at nodes.lkt:11305:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11305'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7467
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7471 '<If at nodes.lkt:11306:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11306'
if Resolve_Type then
   --# expr-start 7468 '<FieldAccess for expression_type at nodes.lkt:11306:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11306'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 7468
   If_Result := Fld_2;
else
   --# expr-start 7470 '<Cast to BaseTypeDecl at nodes.lkt:11306:60>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11306'
--# expr-start 7469 '<FieldAccess for type_val at nodes.lkt:11306:60>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11306'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Var_Expr_0.Node);
--# end
--# expr-done 7469
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7470
   If_Result := Cast_Result_1;
end if;
--# expr-done 7471
   Result_Var_1 := If_Result;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Aggregate_P_Ancestor_Expr_Type;
--# end
   --
   --  Primitives for Bare_Aggregate
   --
      procedure Initialize_Fields_For_Aggregate
        (Self : Bare_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Aggregate;
--# property-start Aggregate.add_named_param_at 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11315'
pragma Warnings (Off, "is not referenced");
 function Aggregate_P_Add_Named_Param_At
  (Node : Bare_Aggregate
      ; Index : Integer
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Aggregate  := Bare_Aggregate (Node);
     Ent : Internal_Entity_Aggregate :=
       Internal_Entity_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index Index
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1113 with Inline_Always;
            procedure Finalizer_Scope_3036 with Inline_Always;
            procedure Finalizer_Scope_3037 with Inline_Always;
      Local_Add_Named : Internal_Entity_Basic_Assoc;
Local_Add_Named_Params : Internal_Entity_Param_Spec_Array_Access;
Var_Expr_1 : Internal_Entity_Basic_Assoc;
Var_Expr_2 : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Aggregate_Assoc;
Fld_3 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_1 : Bare_Name;
Sym : Symbol_Type;
Fld_4 : Boolean;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Find_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Fld_8 : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
Get_Result_1 : Internal_Entity_Param_Spec;
Fld_9 : Internal_Entity_Type_Expr;
Fld_10 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1113 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Add_Named_Params);
            end Finalizer_Scope_1113;
            procedure Finalizer_Scope_3036 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3036;
            procedure Finalizer_Scope_3037 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3037;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7489 '<Let at nodes.lkt:11322:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11322'
--# scope-start
--# expr-start 7479 '<Find at nodes.lkt:11322:25>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11322'
   --# expr-start 7472 '<FieldAccess for assocs at nodes.lkt:11322:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11322'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7472
Fld_1 := Fld.Node;
      Find_Result := No_Entity_Basic_Assoc;
      if Fld_1 = null then
      Find_Result := No_Entity_Basic_Assoc;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind assoc Item_0
      --# expr-start 7478 '<FieldAccess for name_is at nodes.lkt:11323:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
--# expr-start 7476 '<Cast to Name at nodes.lkt:11323:24>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
--# expr-start 7475 '<at at nodes.lkt:11323:24>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
--# expr-start 7474 '<FieldAccess for names at nodes.lkt:11323:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
--# expr-start 7473 '<Cast to AggregateAssoc at nodes.lkt:11323:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Aggregate_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Aggregate_Assoc;
   end if;
--# expr-done 7473
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Cast_Result.Node);
--# end
--# expr-done 7474
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 7475
   if Get_Result = null
      or else Get_Result.Kind in Ada_Name
   then
      Cast_Result_1 := Get_Result;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 7476
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7477 '<Symbol Add_Named at nodes.lkt:11323:78>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11323'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Add_Named); 
--# expr-done 7477
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result_1, Sym => Sym);
--# end
--# expr-done 7478
      if Fld_4 then
         Find_Result := Item_0; 
      end if;
   --# end
      Finalizer_Scope_3036;
               exit when Fld_4;
         end loop;
      end;
      end if;
--# expr-done 7479
Local_Add_Named := Find_Result; 
--# bind add_named Local_Add_Named
--# expr-start 7488 '<Let at nodes.lkt:11325:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11325'
--# scope-start
Var_Expr_1 := Local_Add_Named;
if Var_Expr_1 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind an Var_Expr_1
--# expr-start 7484 '<FieldAccess for params at nodes.lkt:11326:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11326'
--# expr-start 7483 '<FieldAccess for subp_spec_or_null at nodes.lkt:11326:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11326'
--# expr-start 7482 '<FieldAccess for referenced_decl at nodes.lkt:11326:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11326'
--# expr-start 7481 '<Cast to Name at nodes.lkt:11326:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11326'
--# expr-start 7480 '<FieldAccess for expr at nodes.lkt:11326:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11326'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7480
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 7481
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => False, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 7482
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 7483
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 7484
Scope_Result := Fld_8;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3037;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Add_Named_Params := Result_Var; Inc_Ref (Local_Add_Named_Params);
--# bind add_named_params Local_Add_Named_Params
Var_Expr_2 := Local_Add_Named;
if Var_Expr_2 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 7487 '<FieldAccess for designated_type_decl at nodes.lkt:11330:20>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11330'
--# expr-start 7486 '<FieldAccess for type_expression at nodes.lkt:11330:20>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11330'
--# expr-start 7485 '<at at nodes.lkt:11330:20>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11330'
Get_Result_1 := Get (Self, Local_Add_Named_Params, Index, True); 
--# expr-done 7485
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7486
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_10 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 7487
Scope_Result_1 := Fld_10;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 7488
Let_Result_1 := Let_Result; 
--# end
--# expr-done 7489
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_1113;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1113;
                     Finalizer_Scope_3036;
                     Finalizer_Scope_3037;
            raise;
      end;
   return Property_Result;
end Aggregate_P_Add_Named_Param_At;
--# end
--# property-start Aggregate.element_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11336'
pragma Warnings (Off, "is not referenced");
 function Aggregate_P_Element_Type
  (Node : Bare_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Aggregate  := Bare_Aggregate (Node);
     Ent : Internal_Entity_Aggregate :=
       Internal_Entity_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3039 with Inline_Always;
            procedure Finalizer_Scope_3040 with Inline_Always;
      Local_Add_Unnamed : Internal_Entity_Basic_Assoc;
Local_Unnamed_Element_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Basic_Assoc;
Local_Named_Element_Type : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity_Aggregate_Assoc;
Fld_3 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_1 : Bare_Name;
Sym : Symbol_Type;
Fld_4 : Boolean;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Find_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Fld_8 : Internal_Entity_Param_Spec_Array_Access;
Get_Result_1 : Internal_Entity_Param_Spec;
Fld_9 : Internal_Entity_Type_Expr;
Fld_10 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3039 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3039;
            procedure Finalizer_Scope_3040 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3040;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7511 '<Let at nodes.lkt:11342:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11342'
--# scope-start
--# expr-start 7497 '<Find at nodes.lkt:11342:27>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11342'
   --# expr-start 7490 '<FieldAccess for assocs at nodes.lkt:11342:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11342'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 7490
Fld_1 := Fld.Node;
      Find_Result := No_Entity_Basic_Assoc;
      if Fld_1 = null then
      Find_Result := No_Entity_Basic_Assoc;
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind assoc Item_0
      --# expr-start 7496 '<FieldAccess for name_is at nodes.lkt:11343:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
--# expr-start 7494 '<Cast to Name at nodes.lkt:11343:24>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
--# expr-start 7493 '<at at nodes.lkt:11343:24>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
--# expr-start 7492 '<FieldAccess for names at nodes.lkt:11343:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
--# expr-start 7491 '<Cast to AggregateAssoc at nodes.lkt:11343:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Aggregate_Assoc_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Aggregate_Assoc;
   end if;
--# expr-done 7491
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Cast_Result.Node);
--# end
--# expr-done 7492
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 7493
   if Get_Result = null
      or else Get_Result.Kind in Ada_Name
   then
      Cast_Result_1 := Get_Result;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 7494
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7495 '<Symbol Add_Unnamed at nodes.lkt:11343:78>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11343'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Add_Unnamed); 
--# expr-done 7495
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Cast_Result_1, Sym => Sym);
--# end
--# expr-done 7496
      if Fld_4 then
         Find_Result := Item_0; 
      end if;
   --# end
      Finalizer_Scope_3039;
               exit when Fld_4;
         end loop;
      end;
      end if;
--# expr-done 7497
Local_Add_Unnamed := Find_Result; 
--# bind add_unnamed Local_Add_Unnamed
--# expr-start 7510 '<Let at nodes.lkt:11345:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11345'
--# scope-start
Var_Expr_1 := Local_Add_Unnamed;
if Var_Expr_1 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind au Var_Expr_1
--# expr-start 7505 '<FieldAccess for designated_type_decl at nodes.lkt:11346:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7504 '<FieldAccess for type_expression at nodes.lkt:11346:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7503 '<at at nodes.lkt:11346:21>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7502 '<FieldAccess for params at nodes.lkt:11346:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7501 '<FieldAccess for subp_spec_or_null at nodes.lkt:11346:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7500 '<FieldAccess for referenced_decl at nodes.lkt:11346:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7499 '<Cast to Name at nodes.lkt:11346:21>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
--# expr-start 7498 '<FieldAccess for expr at nodes.lkt:11346:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11346'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7498
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 7499
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => False, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 7500
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 7501
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 7502
Get_Result_1 := Get (Self, Fld_8, 1, True); 
--# expr-done 7503
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7504
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_10 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 7505
Scope_Result := Fld_10;
   --# end
      Finalizer_Scope_3040;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Local_Unnamed_Element_Type := Result_Var; 
--# bind unnamed_element_type Local_Unnamed_Element_Type
--# expr-start 7509 '<Let at nodes.lkt:11349:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11349'
--# scope-start
--# expr-start 7506 '<FieldAccess for add_named_param_at at nodes.lkt:11349:34>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11349'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.add_named_param_at
Fld_11 := Libadalang.Implementation.Aggregate_P_Add_Named_Param_At (Node => Ent.Node, Index => 2, E_Info => Ent.Info);
--# end
--# expr-done 7506
Local_Named_Element_Type := Fld_11; 
--# bind named_element_type Local_Named_Element_Type
--# expr-start 7508 '<If at nodes.lkt:11351:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11351'
--# expr-start 7507 '<IsNull at nodes.lkt:11351:12>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11351'
Is_Null_1 := Local_Add_Unnamed.Node = null; 
--# expr-done 7507
if Is_Null_1 then
   If_Result_1 := Local_Named_Element_Type;
else
   If_Result_1 := Local_Unnamed_Element_Type;
end if;
--# expr-done 7508
Let_Result := If_Result_1; 
--# end
--# expr-done 7509
Let_Result_1 := Let_Result; 
--# end
--# expr-done 7510
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 7511
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3039;
                     Finalizer_Scope_3040;
            raise;
      end;
   return Property_Result;
end Aggregate_P_Element_Type;
--# end
--# property-start Aggregate.key_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11356'
pragma Warnings (Off, "is not referenced");
 function Aggregate_P_Key_Type
  (Node : Bare_Aggregate
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Aggregate  := Bare_Aggregate (Node);
     Ent : Internal_Entity_Aggregate :=
       Internal_Entity_Aggregate'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Local_Key_Type : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7514 '<Let at nodes.lkt:11358:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11358'
--# scope-start
--# expr-start 7512 '<FieldAccess for add_named_param_at at nodes.lkt:11358:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11358'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Aggregate.add_named_param_at
Fld := Libadalang.Implementation.Aggregate_P_Add_Named_Param_At (Node => Ent.Node, Index => 1, E_Info => Ent.Info);
--# end
--# expr-done 7512
Local_Key_Type := Fld; 
--# bind key_type Local_Key_Type
Var_Expr := Local_Key_Type;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 7513 '<FieldAccess for universal_int_type at nodes.lkt:11363:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11363'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Ent.Node);
--# end
--# expr-done 7513
   Result_Var := Fld_1;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 7514
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Aggregate_P_Key_Type;
--# end
   --
   --  Primitives for Bare_Bracket_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Aggregate
        (Self : Bare_Bracket_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Aggregate;
   --
   --  Primitives for Bare_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Delta_Aggregate
        (Self : Bare_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Delta_Aggregate;
   --
   --  Primitives for Bare_Bracket_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Delta_Aggregate
        (Self : Bare_Bracket_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Delta_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Delta_Aggregate;
   --
   --  Primitives for Bare_Null_Record_Aggregate
   --
      procedure Initialize_Fields_For_Null_Record_Aggregate
        (Self : Bare_Null_Record_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Null_Record_Aggregate;
   --
   --  Primitives for Bare_Bin_Op
   --
      procedure Initialize_Fields_For_Bin_Op
        (Self : Bare_Bin_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Bin_Op_F_Left := Bin_Op_F_Left;
            Self.Bin_Op_F_Op := Bin_Op_F_Op;
            Self.Bin_Op_F_Right := Bin_Op_F_Right;
      end Initialize_Fields_For_Bin_Op;
   function Bin_Op_F_Left
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Left;
   end;
   function Bin_Op_F_Op
     (Node : Bare_Bin_Op) return Bare_Op
   is
   begin
         return Node.Bin_Op_F_Op;
   end;
   function Bin_Op_F_Right
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Right;
   end;
--# property-start BinOp.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11392'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Has_Context_Free_Type
  (Node : Bare_Bin_Op
  )
   return Boolean
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Op;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7516 '<IsA Op.AndThen, Op.OrElse at nodes.lkt:11392:41>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11392'
--# expr-start 7515 '<FieldAccess for op at nodes.lkt:11392:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11392'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Op;
--# expr-done 7515
Is_A := Fld /= null 
and then Fld.Kind in Ada_Op_And_Then_Range | Ada_Op_Or_Else_Range; 
--# expr-done 7516
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Has_Context_Free_Type;
--# end
--# property-start BinOp.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11395'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Xref_Equation
  (Node : Bare_Bin_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1117 with Inline_Always;
      Fld : Internal_Entity_Op;
Cast_Result : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_9 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_10 : Bare_Op;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_11 : Bare_Op;
Is_A_1 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1117 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1117;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7538 '<bind at nodes.lkt:11396:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11396'
--# expr-start 7518 '<New LogicContext at nodes.lkt:11396:30>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11396'
--# expr-start 7517 '<FieldAccess for op at nodes.lkt:11397:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11397'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 7517
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
New_Struct := (Decl_Node => No_Entity, Ref_Node => Cast_Result); 
--# expr-done 7518
Bound_Logic_Context := New_Struct; 
--# expr-start 7537 '<And at nodes.lkt:11400:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11400'
--# expr-start 7523 '<And at nodes.lkt:11401:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11401'
--# expr-start 7520 '<FieldAccess for sub_equation at nodes.lkt:11401:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11401'
--# expr-start 7519 '<FieldAccess for left at nodes.lkt:11401:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 7519
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 7520
--# expr-start 7522 '<FieldAccess for sub_equation at nodes.lkt:11401:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11401'
--# expr-start 7521 '<FieldAccess for right at nodes.lkt:11401:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 7521
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 7522
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11401:13") else null)); 
--# expr-done 7523
--# expr-start 7536 '<If at nodes.lkt:11403:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11403'
--# expr-start 7525 '<IsA Op.DoubleDot at nodes.lkt:11403:16>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11403'
--# expr-start 7524 '<FieldAccess for op at nodes.lkt:11403:16>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11403'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Op;
--# expr-done 7524
Is_A_1 := Fld_11 /= null 
and then Fld_11.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 7525
if Is_A_1 then
   --# expr-start 7526 '<FieldAccess for double_dot_equation at nodes.lkt:11403:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11403'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.double_dot_equation
Fld_5 := Libadalang.Implementation.Bin_Op_P_Double_Dot_Equation (Node => Ent.Node, Origin => Origin, Env => Env, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7526
   If_Result_1 := Fld_5;
else
   --# expr-start 7535 '<If at nodes.lkt:11403:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11403'
--# expr-start 7528 '<IsA Op.AndThen, Op.OrElse at nodes.lkt:11404:18>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11404'
--# expr-start 7527 '<FieldAccess for op at nodes.lkt:11404:18>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11404'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Bin_Op_F_Op;
--# expr-done 7527
Is_A := Fld_10 /= null 
and then Fld_10.Kind in Ada_Op_And_Then_Range | Ada_Op_Or_Else_Range; 
--# expr-done 7528
if Is_A then
   --# expr-start 7529 '<FieldAccess for test_eq at nodes.lkt:11404:57>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11404'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.test_eq
Fld_6 := Libadalang.Implementation.Bin_Op_P_Test_Eq (Node => Self, Origin => Origin);
--# end
--# expr-done 7529
   If_Result := Fld_6;
else
   --# expr-start 7534 '<Or at nodes.lkt:11405:18>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11405'
--# expr-start 7532 '<Or at nodes.lkt:11406:17>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11406'
--# expr-start 7530 '<FieldAccess for overload_equation at nodes.lkt:11406:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.overload_equation
Fld_7 := Libadalang.Implementation.Bin_Op_P_Overload_Equation (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7530
--# expr-start 7531 '<FieldAccess for no_overload_equation at nodes.lkt:11406:45>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BinOp.no_overload_equation'
Fld_8 := Libadalang.Implementation.Dispatcher_Bin_Op_P_No_Overload_Equation (Node => Ent.Node, Origin => Origin, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7531
Or_Pred := Create_Or (Fld_7, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11406:17") else null)); 
--# expr-done 7532
--# expr-start 7533 '<FieldAccess for universal_fixed_predefined_operators_equation at nodes.lkt:11407:18>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11407'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.universal_fixed_predefined_operators_equation
Fld_9 := Libadalang.Implementation.Bin_Op_P_Universal_Fixed_Predefined_Operators_Equation (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 7533
Or_Pred_1 := Create_Or (Or_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11405:18") else null)); 
--# expr-done 7534
   If_Result := Or_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 7535
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7536
And_Pred_1 := Create_And (And_Pred, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11400:9") else null)); 
--# expr-done 7537
Dyn_Var_Bind_Result := And_Pred_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7538
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1117;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1117;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Xref_Equation;
--# end
--# property-start BinOp.double_dot_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11412'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Double_Dot_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1118 with Inline_Always;
      True_Rel : Logic_Equation;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Bare_Expr;
Fld_14 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_15 : Bare_Expr;
Fld_16 : Logic_Equation;
And_Pred_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_17 : Logic_Var;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Equation;
And_Pred_6 : Logic_Equation;
            procedure Finalizer_Scope_1118 is
            begin
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_1118;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7573 '<And at nodes.lkt:11412:43>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11412'
--# expr-start 7569 '<And at nodes.lkt:11413:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11413'
--# expr-start 7546 '<Or at nodes.lkt:11414:13>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
--# expr-start 7542 '<Or at nodes.lkt:11414:13>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11414:13") else null)); 
--# expr-start 7541 '<Bind at nodes.lkt:11414:22>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
--# expr-start 7539 '<FieldAccess for expected_type_var at nodes.lkt:11414:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7539
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7540 '<FieldAccess for root_int_type at nodes.lkt:11414:52>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 7540
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11414:22") else null)); 
--# expr-done 7541
Or_Pred := Create_Or (True_Rel, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11414:13") else null)); 
--# expr-done 7542
--# expr-start 7545 '<Bind at nodes.lkt:11414:77>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
--# expr-start 7543 '<FieldAccess for expected_type_var at nodes.lkt:11414:81>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7543
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7544 '<FieldAccess for int_type at nodes.lkt:11414:107>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11414'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 7544
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_2, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11414:77") else null)); 
--# expr-done 7545
Or_Pred_1 := Create_Or (Or_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11414:13") else null)); 
--# expr-done 7546
--# expr-start 7568 '<And at nodes.lkt:11416:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7565 '<And at nodes.lkt:11416:13>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7562 '<And at nodes.lkt:11416:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7557 '<And at nodes.lkt:11416:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7552 '<And at nodes.lkt:11416:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7548 '<Predicate on AdaNode.is_not_null at nodes.lkt:11416:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7547 '<FieldAccess for expected_type_var at nodes.lkt:11416:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7547
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7548
--# expr-start 7551 '<Bind at nodes.lkt:11416:75>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7549 '<FieldAccess for expected_type_var at nodes.lkt:11416:79>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7549
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7550 '<FieldAccess for type_var at nodes.lkt:11416:105>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7550
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result_2 := Solver.Create_Unify (Fld_5, Fld_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:75") else null)); 
--# expr-done 7551
And_Pred := Create_And (Pred, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:13") else null)); 
--# expr-done 7552
--# expr-start 7556 '<Bind at nodes.lkt:11416:126>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7553 '<FieldAccess for type_var at nodes.lkt:11416:130>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7553
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7555 '<FieldAccess for expected_type_var at nodes.lkt:11416:147>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7554 '<FieldAccess for left at nodes.lkt:11416:147>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Left;
--# expr-done 7554
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8);
--# expr-done 7555
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Bind_Result_3 := Solver.Create_Unify (Fld_7, Fld_9, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:126") else null)); 
--# expr-done 7556
And_Pred_1 := Create_And (And_Pred, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:13") else null)); 
--# expr-done 7557
--# expr-start 7561 '<Bind at nodes.lkt:11416:182>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7558 '<FieldAccess for type_var at nodes.lkt:11416:186>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7558
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 7560 '<FieldAccess for expected_type_var at nodes.lkt:11416:203>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7559 '<FieldAccess for right at nodes.lkt:11416:203>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Right;
--# expr-done 7559
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_11);
--# expr-done 7560
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_4 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:182") else null)); 
--# expr-done 7561
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:13") else null)); 
--# expr-done 7562
--# expr-start 7564 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11416:239>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7563 '<FieldAccess for left at nodes.lkt:11416:239>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Bin_Op_F_Left;
--# expr-done 7563
      if Fld_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_13, Origin => Origin);
--# end
--# expr-done 7564
And_Pred_3 := Create_And (And_Pred_2, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:13") else null)); 
--# expr-done 7565
--# expr-start 7567 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11416:284>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
--# expr-start 7566 '<FieldAccess for right at nodes.lkt:11416:284>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11416'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Self.Bin_Op_F_Right;
--# expr-done 7566
      if Fld_15 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_16 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_15, Origin => Origin);
--# end
--# expr-done 7567
And_Pred_4 := Create_And (And_Pred_3, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11416:13") else null)); 
--# expr-done 7568
And_Pred_5 := Create_And (Or_Pred_1, And_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11413:9") else null)); 
--# expr-done 7569
--# expr-start 7572 '<FieldAccess for use_expected_type_or at nodes.lkt:11418:11>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11418'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7571 '<FieldAccess for infer_from_either_operands at nodes.lkt:11426:9>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11426'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7570 '<FieldAccess for type_var at nodes.lkt:11426:41>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11426'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7570
--# property-call-start BinOp.infer_from_either_operands
Fld_18 := Libadalang.Implementation.Bin_Op_P_Infer_From_Either_Operands (Node => Ent.Node, Dest_Var => Fld_17, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7571
--# property-call-start BinOp.use_expected_type_or
Fld_19 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_18);
--# end
--# expr-done 7572
And_Pred_6 := Create_And (And_Pred_5, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11412:43") else null)); 
--# expr-done 7573
         Property_Result := And_Pred_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1118;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1118;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Double_Dot_Equation;
--# end
--# property-start BinOp.test_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11430'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Test_Eq
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1119 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1119 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1119;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7593 '<And at nodes.lkt:11431:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7588 '<And at nodes.lkt:11431:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7585 '<And at nodes.lkt:11431:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7582 '<And at nodes.lkt:11431:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7577 '<Bind at nodes.lkt:11431:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7575 '<FieldAccess for expected_type_var at nodes.lkt:11431:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7574 '<FieldAccess for left at nodes.lkt:11431:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 7574
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 7575
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 7576 '<FieldAccess for bool_type at nodes.lkt:11431:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7576
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:9") else null)); 
--# expr-done 7577
--# expr-start 7581 '<Bind at nodes.lkt:11431:66>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7579 '<FieldAccess for expected_type_var at nodes.lkt:11431:70>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7578 '<FieldAccess for right at nodes.lkt:11431:70>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Bin_Op_F_Right;
--# expr-done 7578
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 7579
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 7580 '<FieldAccess for bool_type at nodes.lkt:11431:102>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7580
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:66") else null)); 
--# expr-done 7581
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:9") else null)); 
--# expr-done 7582
--# expr-start 7584 '<FieldAccess for expect_bool_derived_type at nodes.lkt:11431:124>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7583 '<FieldAccess for left at nodes.lkt:11431:124>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Left;
--# expr-done 7583
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_7 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_6, Origin => Origin);
--# end
--# expr-done 7584
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:9") else null)); 
--# expr-done 7585
--# expr-start 7587 '<FieldAccess for expect_bool_derived_type at nodes.lkt:11431:165>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7586 '<FieldAccess for right at nodes.lkt:11431:165>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Right;
--# expr-done 7586
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_9 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_8, Origin => Origin);
--# end
--# expr-done 7587
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:9") else null)); 
--# expr-done 7588
--# expr-start 7592 '<Bind at nodes.lkt:11431:207>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7589 '<FieldAccess for type_var at nodes.lkt:11431:211>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7589
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 7591 '<FieldAccess for type_var at nodes.lkt:11431:228>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
--# expr-start 7590 '<FieldAccess for left at nodes.lkt:11431:228>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11431'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Left;
--# expr-done 7590
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 7591
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_2 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:207") else null)); 
--# expr-done 7592
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11431:9") else null)); 
--# expr-done 7593
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1119;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1119;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Test_Eq;
--# end
--# property-start BinOp.arguments_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11434'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Arguments_Eq
  (Node : Bare_Bin_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind origin Origin
   --# bind env Env
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1120 with Inline_Always;
      Local_Ps : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1120 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Ps);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1120;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7611 '<Let at nodes.lkt:11435:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11435'
--# scope-start
--# expr-start 7594 '<FieldAccess for unpacked_formal_params at nodes.lkt:11435:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11435'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7594
Local_Ps := Fld; Inc_Ref (Local_Ps);
--# bind ps Local_Ps
--# expr-start 7610 '<If at nodes.lkt:11437:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11437'
--# expr-start 7596 '<Eq at nodes.lkt:11437:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11437'
--# expr-start 7595 '<length at nodes.lkt:11437:12>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11437'
Len := Length (Local_Ps); 
--# expr-done 7595
Is_Equal := Len = 2; 
--# expr-done 7596
if Is_Equal then
   --# expr-start 7609 '<And at nodes.lkt:11440:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11440'
--# expr-start 7605 '<And at nodes.lkt:11441:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11441'
--# expr-start 7600 '<FieldAccess for call_argument_equation at nodes.lkt:11441:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11441'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7598 '<FieldAccess for formal_decl at nodes.lkt:11441:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11441'
--# expr-start 7597 '<at at nodes.lkt:11441:45>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11441'
Get_Result := Get (Self, Local_Ps, 0, True); 
--# expr-done 7597
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7598
--# expr-start 7599 '<FieldAccess for left at nodes.lkt:11441:67>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11441'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 7599
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_1, Arg => Fld_2, Origin => Origin, Logic_Context => Logic_Context, E_Info => Spec.Info);
--# end
--# expr-done 7600
--# expr-start 7604 '<FieldAccess for call_argument_equation at nodes.lkt:11444:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11444'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7602 '<FieldAccess for formal_decl at nodes.lkt:11444:49>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11444'
--# expr-start 7601 '<at at nodes.lkt:11444:49>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11444'
Get_Result_1 := Get (Self, Local_Ps, 1, True); 
--# expr-done 7601
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7602
--# expr-start 7603 '<FieldAccess for right at nodes.lkt:11444:71>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11444'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 7603
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_4, Arg => Fld_5, Origin => Origin, Logic_Context => Logic_Context, E_Info => Spec.Info);
--# end
--# expr-done 7604
And_Pred := Create_And (Fld_3, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11441:17") else null)); 
--# expr-done 7605
--# expr-start 7608 '<Bind at nodes.lkt:11448:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11448'
--# expr-start 7606 '<FieldAccess for type_var at nodes.lkt:11448:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11448'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7606
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7607 '<FieldAccess for return_type at nodes.lkt:11448:38>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11448'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Spec.Node, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 7607
      Cast_Result := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Assign (Fld_7, Cast_Result, Solver_Ifc.No_Converter, Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11448:17") else null)); 
--# expr-done 7608
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11440:13") else null)); 
--# expr-done 7609
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11450:16") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 7610
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7611
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1120;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1120;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Arguments_Eq;
--# end
--# property-start BinOp.entity_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11454'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Entity_Eq
  (Node : Bare_Bin_Op
      ; Subp : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind origin Origin
   --# bind env Env
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1121 with Inline_Always;
      Local_Spec : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Op;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Op;
Fld_5 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1121 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1121;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7622 '<Let at nodes.lkt:11455:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11455'
--# scope-start
--# expr-start 7612 '<FieldAccess for subp_spec_or_null at nodes.lkt:11455:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11455'
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 7612
Local_Spec := Fld; 
--# bind spec Local_Spec
--# expr-start 7621 '<And at nodes.lkt:11457:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11457'
--# expr-start 7617 '<And at nodes.lkt:11457:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11457'
--# expr-start 7613 '<FieldAccess for arguments_eq at nodes.lkt:11457:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11457'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.arguments_eq
Fld_1 := Libadalang.Implementation.Bin_Op_P_Arguments_Eq (Node => Ent.Node, Spec => Local_Spec, Origin => Origin, Env => Env, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7613
--# expr-start 7616 '<Bind at nodes.lkt:11459:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11459'
--# expr-start 7615 '<FieldAccess for ref_var at nodes.lkt:11459:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11459'
--# expr-start 7614 '<FieldAccess for op at nodes.lkt:11459:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11459'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Op;
--# expr-done 7614
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 7615
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Cast_Expr := Subp; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11459:13") else null)); 
--# expr-done 7616
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11457:9") else null)); 
--# expr-done 7617
--# expr-start 7620 '<Bind at nodes.lkt:11460:15>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11460'
--# expr-start 7619 '<FieldAccess for subp_spec_var at nodes.lkt:11460:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11460'
--# expr-start 7618 '<FieldAccess for op at nodes.lkt:11460:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11460'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 7618
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_4);
--# end
--# expr-done 7619
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr_1 := Local_Spec; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11460:15") else null)); 
--# expr-done 7620
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11457:9") else null)); 
--# expr-done 7621
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7622
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1121;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1121;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Entity_Eq;
--# end
--# property-start BinOp.overload_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11464'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1122 with Inline_Always;
            procedure Finalizer_Scope_3042 with Inline_Always;
      Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_3 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1122 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1122;
            procedure Finalizer_Scope_3042 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3042;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7630 '<LogicAny at nodes.lkt:11464:41>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11464'
--# expr-start 7629 '<Map at nodes.lkt:11464:41>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11464'
   --# expr-start 7624 '<FieldAccess for subprograms at nodes.lkt:11464:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11464'
--# expr-start 7623 '<FieldAccess for op at nodes.lkt:11464:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11464'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 7623
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7624
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_0
      --# expr-start 7628 '<bind at nodes.lkt:11466:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11466'
--# expr-start 7626 '<New LogicContext at nodes.lkt:11466:34>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11466'
Cast_Expr := Item_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 7625 '<FieldAccess for op at nodes.lkt:11466:56>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11466'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 7625
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
New_Struct := (Decl_Node => Cast_Result_1, Ref_Node => Cast_Result); 
--# expr-done 7626
Bound_Logic_Context := New_Struct; 
--# expr-start 7627 '<FieldAccess for entity_eq at nodes.lkt:11468:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11468'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.entity_eq
Fld_3 := Libadalang.Implementation.Bin_Op_P_Entity_Eq (Node => Ent.Node, Subp => Item_0, Origin => Origin, Env => Env, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7627
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7628
         declare
            Item_To_Append : constant Logic_Equation := Dyn_Var_Bind_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3042;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7629
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11464:41") else null)); 
--# expr-done 7630
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1122;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1122;
                     Finalizer_Scope_3042;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Overload_Equation;
--# end
--# property-start BinOp.universal_fixed_predefined_operators_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11475'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Universal_Fixed_Predefined_Operators_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1123 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Bare_Expr;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_12 : Bare_Op;
Is_A : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1123 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1123;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7652 '<If at nodes.lkt:11475:69>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11475'
--# expr-start 7632 '<IsA Op.Mult, Op.Div at nodes.lkt:11475:72>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11475'
--# expr-start 7631 '<FieldAccess for op at nodes.lkt:11475:72>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11475'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Bin_Op_F_Op;
--# expr-done 7631
Is_A := Fld_12 /= null 
and then Fld_12.Kind in Ada_Op_Mult_Range | Ada_Op_Div_Range; 
--# expr-done 7632
if Is_A then
   --# expr-start 7651 '<And at nodes.lkt:11475:105>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11475'
--# expr-start 7648 '<And at nodes.lkt:11476:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11476'
--# expr-start 7645 '<And at nodes.lkt:11477:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11477'
--# expr-start 7640 '<And at nodes.lkt:11478:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
--# expr-start 7635 '<Bind at nodes.lkt:11478:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
--# expr-start 7633 '<FieldAccess for type_var at nodes.lkt:11478:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7633
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7634 '<FieldAccess for universal_fixed_type at nodes.lkt:11478:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 7634
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11478:17") else null)); 
--# expr-done 7635
--# expr-start 7639 '<Bind at nodes.lkt:11478:71>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
--# expr-start 7637 '<FieldAccess for expected_type_var at nodes.lkt:11478:75>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
--# expr-start 7636 '<FieldAccess for left at nodes.lkt:11478:75>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Left;
--# expr-done 7636
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 7637
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7638 '<FieldAccess for universal_fixed_type at nodes.lkt:11478:106>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 7638
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_3, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11478:71") else null)); 
--# expr-done 7639
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11478:17") else null)); 
--# expr-done 7640
--# expr-start 7644 '<Bind at nodes.lkt:11479:19>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11479'
--# expr-start 7642 '<FieldAccess for expected_type_var at nodes.lkt:11479:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11479'
--# expr-start 7641 '<FieldAccess for right at nodes.lkt:11479:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11479'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Bin_Op_F_Right;
--# expr-done 7641
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5);
--# expr-done 7642
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 7643 '<FieldAccess for universal_fixed_type at nodes.lkt:11479:55>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11479'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 7643
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_6, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11479:19") else null)); 
--# expr-done 7644
And_Pred_1 := Create_And (And_Pred, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11477:13") else null)); 
--# expr-done 7645
--# expr-start 7647 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11480:15>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11480'
--# expr-start 7646 '<FieldAccess for left at nodes.lkt:11480:15>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11480'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Bin_Op_F_Left;
--# expr-done 7646
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_8, Origin => Origin);
--# end
--# expr-done 7647
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11476:9") else null)); 
--# expr-done 7648
--# expr-start 7650 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11481:11>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11481'
--# expr-start 7649 '<FieldAccess for right at nodes.lkt:11481:11>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11481'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Bin_Op_F_Right;
--# expr-done 7649
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_11 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_10, Origin => Origin);
--# end
--# expr-done 7650
And_Pred_3 := Create_And (And_Pred_2, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11475:105") else null)); 
--# expr-done 7651
   If_Result := And_Pred_3;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11481:58") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 7652
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1123;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1123;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Universal_Fixed_Predefined_Operators_Equation;
--# end
--# property-start BinOp.use_expected_type_or 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11487'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Use_Expected_Type_Or
  (Node : Bare_Bin_Op
      ; Eq : Logic_Equation
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind eq Eq
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1124 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Or_Pred : Logic_Equation;
            procedure Finalizer_Scope_1124 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1124;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7655 '<Or at nodes.lkt:11488:9>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11488'
--# expr-start 7654 '<Predicate on AdaNode.is_not_null at nodes.lkt:11488:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11488'
--# expr-start 7653 '<FieldAccess for expected_type_var at nodes.lkt:11488:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11488'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7653
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7654
Or_Pred := Create_Or (Pred, Eq, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11488:9") else null)); 
--# expr-done 7655
         Property_Result := Or_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1124;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1124;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Use_Expected_Type_Or;
--# end
--# property-start BinOp.infer_from_left_operand 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11495'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Infer_From_Left_Operand
  (Node : Bare_Bin_Op
      ; Dest_Var : Logic_Var
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind dest_var Dest_Var
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1125 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1125 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1125;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7662 '<And at nodes.lkt:11496:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
--# expr-start 7658 '<Predicate on BaseTypeDecl.is_not_universal_type at nodes.lkt:11496:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
--# expr-start 7657 '<FieldAccess for type_var at nodes.lkt:11496:56>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
--# expr-start 7656 '<FieldAccess for left at nodes.lkt:11496:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 7656
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 7657
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 7658
--# expr-start 7661 '<Bind at nodes.lkt:11496:82>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
--# expr-start 7660 '<FieldAccess for type_var at nodes.lkt:11496:96>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
--# expr-start 7659 '<FieldAccess for left at nodes.lkt:11496:96>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11496'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Left;
--# expr-done 7659
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7660
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Dest_Var.Value := No_Entity;
Entity_Vars.Reset (Dest_Var);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Propagate (Fld_3, Dest_Var, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11496:82") else null)); 
--# expr-done 7661
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11496:9") else null)); 
--# expr-done 7662
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1125;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1125;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Infer_From_Left_Operand;
--# end
--# property-start BinOp.infer_from_right_operand 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11503'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Infer_From_Right_Operand
  (Node : Bare_Bin_Op
      ; Dest_Var : Logic_Var
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
      --# bind self Self
   --# bind dest_var Dest_Var
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1126 with Inline_Always;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1126 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1126;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7669 '<And at nodes.lkt:11504:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
--# expr-start 7665 '<Predicate on BaseTypeDecl.is_not_universal_type at nodes.lkt:11504:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
--# expr-start 7664 '<FieldAccess for type_var at nodes.lkt:11504:56>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
--# expr-start 7663 '<FieldAccess for right at nodes.lkt:11504:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Bin_Op_F_Right;
--# expr-done 7663
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 7664
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 7665
--# expr-start 7668 '<Bind at nodes.lkt:11504:83>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
--# expr-start 7667 '<FieldAccess for type_var at nodes.lkt:11504:97>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
--# expr-start 7666 '<FieldAccess for right at nodes.lkt:11504:97>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11504'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Right;
--# expr-done 7666
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7667
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Dest_Var.Value := No_Entity;
Entity_Vars.Reset (Dest_Var);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Propagate (Fld_3, Dest_Var, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11504:83") else null)); 
--# expr-done 7668
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11504:9") else null)); 
--# expr-done 7669
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1126;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1126;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Infer_From_Right_Operand;
--# end
--# property-start BinOp.infer_from_either_operands 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11511'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Infer_From_Either_Operands
  (Node : Bare_Bin_Op
      ; Dest_Var : Logic_Var
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dest_var Dest_Var
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1127 with Inline_Always;
      Local_Infer_Left : Logic_Equation;
Local_Infer_Right : Logic_Equation;
Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1127 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Infer_Left);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Infer_Right);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1127;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7674 '<Let at nodes.lkt:11512:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11512'
--# scope-start
--# expr-start 7670 '<FieldAccess for infer_from_left_operand at nodes.lkt:11512:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11512'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.infer_from_left_operand
Fld := Libadalang.Implementation.Bin_Op_P_Infer_From_Left_Operand (Node => Ent.Node, Dest_Var => Dest_Var, Origin => Origin, Logic_Context => Logic_Context);
--# end
--# expr-done 7670
Local_Infer_Left := Fld; Inc_Ref (Local_Infer_Left);
--# bind infer_left Local_Infer_Left
--# expr-start 7673 '<Let at nodes.lkt:11513:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11513'
--# scope-start
--# expr-start 7671 '<FieldAccess for infer_from_right_operand at nodes.lkt:11513:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11513'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BinOp.infer_from_right_operand
Fld_1 := Libadalang.Implementation.Bin_Op_P_Infer_From_Right_Operand (Node => Ent.Node, Dest_Var => Dest_Var, Origin => Origin, Logic_Context => Logic_Context);
--# end
--# expr-done 7671
Local_Infer_Right := Fld_1; Inc_Ref (Local_Infer_Right);
--# bind infer_right Local_Infer_Right
--# expr-start 7672 '<Or at nodes.lkt:11515:9>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11515'
Or_Pred := Create_Or (Local_Infer_Left, Local_Infer_Right, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11515:9") else null)); 
--# expr-done 7672
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7673
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7674
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1127;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1127;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Infer_From_Either_Operands;
--# end
--# property-start BinOp.potential_actuals_for_dispatch 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11519'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Bin_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1128 with Inline_Always;
      Local_Params : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Array_Lit : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1128 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Params);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1128;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7687 '<Let at nodes.lkt:11520:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11520'
--# scope-start
--# expr-start 7675 '<FieldAccess for unpacked_formal_params at nodes.lkt:11520:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11520'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7675
Local_Params := Fld; Inc_Ref (Local_Params);
--# bind params Local_Params
--# expr-start 7686 '<ArrayLiteral at nodes.lkt:11522:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11522'
--# expr-start 7680 '<New ExpectedTypeForExpr at nodes.lkt:11522:10>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11522'
--# expr-start 7678 '<FieldAccess for type_expression at nodes.lkt:11523:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11523'
--# expr-start 7677 '<FieldAccess for formal_decl at nodes.lkt:11523:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11523'
--# expr-start 7676 '<at at nodes.lkt:11523:27>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11523'
Get_Result := Get (Self, Local_Params, 0, True); 
--# expr-done 7676
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7677
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 7678
--# expr-start 7679 '<FieldAccess for left at nodes.lkt:11523:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11523'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 7679
New_Struct := (Expected_Type => Fld_2, Expr => Fld_3); 
--# expr-done 7680
--# expr-start 7685 '<New ExpectedTypeForExpr at nodes.lkt:11524:12>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11524'
--# expr-start 7683 '<FieldAccess for type_expression at nodes.lkt:11525:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11525'
--# expr-start 7682 '<FieldAccess for formal_decl at nodes.lkt:11525:27>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11525'
--# expr-start 7681 '<at at nodes.lkt:11525:27>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11525'
Get_Result_1 := Get (Self, Local_Params, 1, True); 
--# expr-done 7681
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7682
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 7683
--# expr-start 7684 '<FieldAccess for right at nodes.lkt:11525:76>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11525'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 7684
New_Struct_1 := (Expected_Type => Fld_5, Expr => Fld_6); 
--# expr-done 7685
Array_Lit := Create_Internal_Expected_Type_For_Expr_Array (Internal_Internal_Expected_Type_For_Expr_Array'(1 => New_Struct, 2 => New_Struct_1)); 
--# expr-done 7686
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 7687
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1128;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1128;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start BinOp.is_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11530'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Is_Dispatching_Call
  (Node : Bare_Bin_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Op;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7689 '<FieldAccess for is_dispatching_call at nodes.lkt:11530:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11530'
--# expr-start 7688 '<FieldAccess for op at nodes.lkt:11530:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11530'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 7688
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7689
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Bin_Op_P_Is_Dispatching_Call;
--# end
--# property-start '[dispatcher]BinOp.no_overload_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Bin_Op_Range (Self.Kind) is
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.no_overload_equation
                  Property_Result := Bin_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                        , Logic_Context
                        , E_Info
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.no_overload_equation
                  Property_Result := Relation_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                        , Logic_Context
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Bin_Op_P_No_Overload_Equation;
--# end
--# property-start BinOp.no_overload_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11535'
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op  := Bare_Bin_Op (Node);
     Ent : Internal_Entity_Bin_Op :=
       Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1130 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Bare_Expr;
Fld_10 : Logic_Var;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_12 : Bare_Expr;
Fld_13 : Logic_Var;
Fld_14 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_18 : Bare_Expr;
Fld_19 : Logic_Var;
Fld_20 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
Fld_21 : Bare_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_24 : Logic_Var;
Fld_25 : Logic_Equation;
Fld_26 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_27 : Bare_Expr;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_30 : Bare_Expr;
Fld_31 : Logic_Var;
Fld_32 : Logic_Var;
Bind_Result_7 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Equation;
Fld_35 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_36 : Logic_Var;
Fld_37 : Bare_Expr;
Fld_38 : Logic_Var;
Bind_Result_8 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Bare_Expr;
Fld_41 : Logic_Var;
Bind_Result_9 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_42 : Logic_Var;
Fld_43 : Logic_Equation;
Fld_44 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_45 : Bare_Op;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_46 : Bare_Op;
Is_A_1 : Boolean;
If_Result_1 : Logic_Equation;
Fld_47 : Logic_Var;
Pred : Logic_Equation;
Fld_48 : Logic_Var;
Fld_49 : Logic_Var;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result_10 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_50 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_11 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_51 : Bare_Expr;
Fld_52 : Logic_Equation;
Fld_53 : Bare_Expr;
Fld_54 : Logic_Equation;
And_Pred_12 : Logic_Equation;
And_Pred_13 : Logic_Equation;
            procedure Finalizer_Scope_1130 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_34);
                     Dec_Ref (Fld_35);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Bind_Result_8);
                     Dec_Ref (Bind_Result_9);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Fld_43);
                     Dec_Ref (Fld_44);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result_10);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Bind_Result_11);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Fld_52);
                     Dec_Ref (Fld_54);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (And_Pred_13);
            end Finalizer_Scope_1130;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7778 '<And at nodes.lkt:11535:44>' And_Pred_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11535'
--# expr-start 7772 '<And at nodes.lkt:11536:9>' And_Pred_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11536'
--# expr-start 7762 '<If at nodes.lkt:11538:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
--# expr-start 7691 '<IsA Op.Mult, Op.Div at nodes.lkt:11538:16>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
--# expr-start 7690 '<FieldAccess for op at nodes.lkt:11538:16>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Self.Bin_Op_F_Op;
--# expr-done 7690
Is_A_1 := Fld_46 /= null 
and then Fld_46.Kind in Ada_Op_Mult_Range | Ada_Op_Div_Range; 
--# expr-done 7691
if Is_A_1 then
   --# expr-start 7732 '<Or at nodes.lkt:11538:49>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
--# expr-start 7718 '<Or at nodes.lkt:11538:49>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
--# expr-start 7704 '<And at nodes.lkt:11540:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11540'
--# expr-start 7700 '<And at nodes.lkt:11541:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
--# expr-start 7695 '<Bind at nodes.lkt:11541:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
--# expr-start 7692 '<FieldAccess for type_var at nodes.lkt:11541:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7692
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7694 '<FieldAccess for expected_type_var at nodes.lkt:11541:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
--# expr-start 7693 '<FieldAccess for left at nodes.lkt:11541:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Bin_Op_F_Left;
--# expr-done 7693
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_1);
--# expr-done 7694
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11541:21") else null)); 
--# expr-done 7695
--# expr-start 7699 '<Bind at nodes.lkt:11541:77>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
--# expr-start 7696 '<FieldAccess for type_var at nodes.lkt:11541:81>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7696
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7698 '<FieldAccess for expected_type_var at nodes.lkt:11541:98>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
--# expr-start 7697 '<FieldAccess for right at nodes.lkt:11541:98>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11541'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Right;
--# expr-done 7697
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 7698
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_1 := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11541:77") else null)); 
--# expr-done 7699
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11541:21") else null)); 
--# expr-done 7700
--# expr-start 7703 '<FieldAccess for use_expected_type_or at nodes.lkt:11542:23>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11542'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7702 '<FieldAccess for infer_from_either_operands at nodes.lkt:11543:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11543'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7701 '<FieldAccess for type_var at nodes.lkt:11543:53>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11543'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7701
--# property-call-start BinOp.infer_from_either_operands
Fld_7 := Libadalang.Implementation.Bin_Op_P_Infer_From_Either_Operands (Node => Ent.Node, Dest_Var => Fld_6, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7702
--# property-call-start BinOp.use_expected_type_or
Fld_8 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_7);
--# end
--# expr-done 7703
And_Pred_1 := Create_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11540:17") else null)); 
--# expr-done 7704
--# expr-start 7717 '<And at nodes.lkt:11547:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11547'
--# expr-start 7713 '<And at nodes.lkt:11548:21>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
--# expr-start 7708 '<Bind at nodes.lkt:11548:21>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
--# expr-start 7706 '<FieldAccess for expected_type_var at nodes.lkt:11548:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
--# expr-start 7705 '<FieldAccess for left at nodes.lkt:11548:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Bin_Op_F_Left;
--# expr-done 7705
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9);
--# expr-done 7706
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 7707 '<FieldAccess for int_type at nodes.lkt:11548:56>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_11 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 7707
      Cast_Result := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_10, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11548:21") else null)); 
--# expr-done 7708
--# expr-start 7712 '<Bind at nodes.lkt:11548:77>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
--# expr-start 7710 '<FieldAccess for expected_type_var at nodes.lkt:11548:81>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
--# expr-start 7709 '<FieldAccess for right at nodes.lkt:11548:81>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Bin_Op_F_Right;
--# expr-done 7709
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_12);
--# expr-done 7710
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 7711 '<FieldAccess for type_var at nodes.lkt:11548:113>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11548'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7711
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
Bind_Result_3 := Solver.Create_Unify (Fld_13, Fld_14, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11548:77") else null)); 
--# expr-done 7712
And_Pred_2 := Create_And (Bind_Result_2, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11548:21") else null)); 
--# expr-done 7713
--# expr-start 7716 '<FieldAccess for use_expected_type_or at nodes.lkt:11549:23>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11549'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7715 '<FieldAccess for infer_from_right_operand at nodes.lkt:11550:21>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11550'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7714 '<FieldAccess for type_var at nodes.lkt:11550:51>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11550'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7714
--# property-call-start BinOp.infer_from_right_operand
Fld_16 := Libadalang.Implementation.Bin_Op_P_Infer_From_Right_Operand (Node => Ent.Node, Dest_Var => Fld_15, Origin => Origin, Logic_Context => Logic_Context);
--# end
--# expr-done 7715
--# property-call-start BinOp.use_expected_type_or
Fld_17 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_16);
--# end
--# expr-done 7716
And_Pred_3 := Create_And (And_Pred_2, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11547:17") else null)); 
--# expr-done 7717
Or_Pred := Create_Or (And_Pred_1, And_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11538:49") else null)); 
--# expr-done 7718
--# expr-start 7731 '<And at nodes.lkt:11554:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11554'
--# expr-start 7727 '<And at nodes.lkt:11555:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
--# expr-start 7722 '<Bind at nodes.lkt:11555:21>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
--# expr-start 7720 '<FieldAccess for expected_type_var at nodes.lkt:11555:25>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
--# expr-start 7719 '<FieldAccess for right at nodes.lkt:11555:25>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Self.Bin_Op_F_Right;
--# expr-done 7719
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_18);
--# expr-done 7720
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 7721 '<FieldAccess for int_type at nodes.lkt:11555:57>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_20 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 7721
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_20.Node,
         Info => Fld_20.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_19, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11555:21") else null)); 
--# expr-done 7722
--# expr-start 7726 '<Bind at nodes.lkt:11555:78>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
--# expr-start 7724 '<FieldAccess for expected_type_var at nodes.lkt:11555:82>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
--# expr-start 7723 '<FieldAccess for left at nodes.lkt:11555:82>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Self.Bin_Op_F_Left;
--# expr-done 7723
      if Fld_21 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_21);
--# expr-done 7724
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 7725 '<FieldAccess for type_var at nodes.lkt:11555:113>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7725
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
Bind_Result_5 := Solver.Create_Unify (Fld_22, Fld_23, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11555:78") else null)); 
--# expr-done 7726
And_Pred_4 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11555:21") else null)); 
--# expr-done 7727
--# expr-start 7730 '<FieldAccess for use_expected_type_or at nodes.lkt:11556:23>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11556'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7729 '<FieldAccess for infer_from_left_operand at nodes.lkt:11557:21>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11557'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7728 '<FieldAccess for type_var at nodes.lkt:11557:50>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11557'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7728
--# property-call-start BinOp.infer_from_left_operand
Fld_25 := Libadalang.Implementation.Bin_Op_P_Infer_From_Left_Operand (Node => Ent.Node, Dest_Var => Fld_24, Origin => Origin, Logic_Context => Logic_Context);
--# end
--# expr-done 7729
--# property-call-start BinOp.use_expected_type_or
Fld_26 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_25);
--# end
--# expr-done 7730
And_Pred_5 := Create_And (And_Pred_4, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11554:17") else null)); 
--# expr-done 7731
Or_Pred_1 := Create_Or (Or_Pred, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11538:49") else null)); 
--# expr-done 7732
   If_Result_1 := Or_Pred_1;
else
   --# expr-start 7761 '<If at nodes.lkt:11538:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11538'
--# expr-start 7734 '<IsA Op.Pow at nodes.lkt:11562:18>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11562'
--# expr-start 7733 '<FieldAccess for op at nodes.lkt:11562:18>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11562'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_45 := Self.Bin_Op_F_Op;
--# expr-done 7733
Is_A := Fld_45 /= null 
and then Fld_45.Kind in Ada_Op_Pow_Range; 
--# expr-done 7734
if Is_A then
   --# expr-start 7747 '<And at nodes.lkt:11562:41>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11562'
--# expr-start 7743 '<And at nodes.lkt:11563:17>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
--# expr-start 7738 '<Bind at nodes.lkt:11563:17>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
--# expr-start 7736 '<FieldAccess for expected_type_var at nodes.lkt:11563:21>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
--# expr-start 7735 '<FieldAccess for right at nodes.lkt:11563:21>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Self.Bin_Op_F_Right;
--# expr-done 7735
      if Fld_27 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_27);
--# expr-done 7736
Fld_28.Value := No_Entity;
Entity_Vars.Reset (Fld_28);
--# expr-start 7737 '<FieldAccess for int_type at nodes.lkt:11563:53>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_29 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 7737
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_28, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11563:17") else null)); 
--# expr-done 7738
--# expr-start 7742 '<Bind at nodes.lkt:11563:74>' Bind_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
--# expr-start 7740 '<FieldAccess for expected_type_var at nodes.lkt:11563:78>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
--# expr-start 7739 '<FieldAccess for left at nodes.lkt:11563:78>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Self.Bin_Op_F_Left;
--# expr-done 7739
      if Fld_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_30);
--# expr-done 7740
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 7741 '<FieldAccess for type_var at nodes.lkt:11563:109>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7741
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Bind_Result_7 := Solver.Create_Unify (Fld_31, Fld_32, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11563:74") else null)); 
--# expr-done 7742
And_Pred_6 := Create_And (Bind_Result_6, Bind_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11563:17") else null)); 
--# expr-done 7743
--# expr-start 7746 '<FieldAccess for use_expected_type_or at nodes.lkt:11564:19>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11564'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7745 '<FieldAccess for infer_from_left_operand at nodes.lkt:11565:17>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11565'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7744 '<FieldAccess for type_var at nodes.lkt:11565:46>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11565'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7744
--# property-call-start BinOp.infer_from_left_operand
Fld_34 := Libadalang.Implementation.Bin_Op_P_Infer_From_Left_Operand (Node => Ent.Node, Dest_Var => Fld_33, Origin => Origin, Logic_Context => Logic_Context);
--# end
--# expr-done 7745
--# property-call-start BinOp.use_expected_type_or
Fld_35 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_34);
--# end
--# expr-done 7746
And_Pred_7 := Create_And (And_Pred_6, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11562:41") else null)); 
--# expr-done 7747
   If_Result := And_Pred_7;
else
   --# expr-start 7760 '<And at nodes.lkt:11569:18>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11569'
--# expr-start 7756 '<And at nodes.lkt:11570:17>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
--# expr-start 7751 '<Bind at nodes.lkt:11570:17>' Bind_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
--# expr-start 7748 '<FieldAccess for type_var at nodes.lkt:11570:21>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7748
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
--# expr-start 7750 '<FieldAccess for expected_type_var at nodes.lkt:11570:38>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
--# expr-start 7749 '<FieldAccess for left at nodes.lkt:11570:38>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Self.Bin_Op_F_Left;
--# expr-done 7749
      if Fld_37 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_37);
--# expr-done 7750
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
Bind_Result_8 := Solver.Create_Unify (Fld_36, Fld_38, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11570:17") else null)); 
--# expr-done 7751
--# expr-start 7755 '<Bind at nodes.lkt:11570:73>' Bind_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
--# expr-start 7752 '<FieldAccess for type_var at nodes.lkt:11570:77>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7752
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 7754 '<FieldAccess for expected_type_var at nodes.lkt:11570:94>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
--# expr-start 7753 '<FieldAccess for right at nodes.lkt:11570:94>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11570'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Self.Bin_Op_F_Right;
--# expr-done 7753
      if Fld_40 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_40);
--# expr-done 7754
Fld_41.Value := No_Entity;
Entity_Vars.Reset (Fld_41);
Bind_Result_9 := Solver.Create_Unify (Fld_39, Fld_41, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11570:73") else null)); 
--# expr-done 7755
And_Pred_8 := Create_And (Bind_Result_8, Bind_Result_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11570:17") else null)); 
--# expr-done 7756
--# expr-start 7759 '<FieldAccess for use_expected_type_or at nodes.lkt:11571:19>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11571'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7758 '<FieldAccess for infer_from_either_operands at nodes.lkt:11572:17>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11572'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7757 '<FieldAccess for type_var at nodes.lkt:11572:49>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11572'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7757
--# property-call-start BinOp.infer_from_either_operands
Fld_43 := Libadalang.Implementation.Bin_Op_P_Infer_From_Either_Operands (Node => Ent.Node, Dest_Var => Fld_42, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7758
--# property-call-start BinOp.use_expected_type_or
Fld_44 := Libadalang.Implementation.Bin_Op_P_Use_Expected_Type_Or (Node => Self, Eq => Fld_43);
--# end
--# expr-done 7759
And_Pred_9 := Create_And (And_Pred_8, Fld_44, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11569:18") else null)); 
--# expr-done 7760
   If_Result := And_Pred_9;
end if;
      Inc_Ref (If_Result);
--# expr-done 7761
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7762
--# expr-start 7771 '<Or at nodes.lkt:11575:13>' Or_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11575'
--# expr-start 7768 '<And at nodes.lkt:11584:17>' And_Pred_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
--# expr-start 7764 '<Predicate on AdaNode.is_not_null at nodes.lkt:11584:17>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
--# expr-start 7763 '<FieldAccess for expected_type_var at nodes.lkt:11584:49>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_47 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7763
Fld_47.Value := No_Entity;
Entity_Vars.Reset (Fld_47);
Pred := Solver.Create_Predicate (Fld_47, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7764
--# expr-start 7767 '<Bind at nodes.lkt:11584:79>' Bind_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
--# expr-start 7765 '<FieldAccess for expected_type_var at nodes.lkt:11584:100>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_49 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7765
Fld_49.Value := No_Entity;
Entity_Vars.Reset (Fld_49);
--# expr-start 7766 '<FieldAccess for type_var at nodes.lkt:11584:83>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11584'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7766
Fld_48.Value := No_Entity;
Entity_Vars.Reset (Fld_48);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result_10 := Solver.Create_Propagate (Fld_49, Fld_48, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11584:79") else null)); 
--# expr-done 7767
And_Pred_10 := Create_And (Pred, Bind_Result_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11584:17") else null)); 
--# expr-done 7768
--# expr-start 7770 '<Bind at nodes.lkt:11590:17>' Bind_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11590'
--# expr-start 7769 '<FieldAccess for expected_type_var at nodes.lkt:11590:21>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11590'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7769
Fld_50.Value := No_Entity;
Entity_Vars.Reset (Fld_50);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_11 := Solver.Create_Assign (Fld_50, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11590:17") else null)); 
--# expr-done 7770
Or_Pred_2 := Create_Or (And_Pred_10, Bind_Result_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11575:13") else null)); 
--# expr-done 7771
And_Pred_11 := Create_And (If_Result_1, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11536:9") else null)); 
--# expr-done 7772
--# expr-start 7777 '<And at nodes.lkt:11594:9>' And_Pred_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11594'
--# expr-start 7774 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11594:9>' Fld_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11594'
--# expr-start 7773 '<FieldAccess for left at nodes.lkt:11594:9>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11594'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_51 := Self.Bin_Op_F_Left;
--# expr-done 7773
      if Fld_51 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_52 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_51, Origin => Origin);
--# end
--# expr-done 7774
--# expr-start 7776 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11594:54>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11594'
--# expr-start 7775 '<FieldAccess for right at nodes.lkt:11594:54>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11594'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Self.Bin_Op_F_Right;
--# expr-done 7775
      if Fld_53 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_54 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_53, Origin => Origin);
--# end
--# expr-done 7776
And_Pred_12 := Create_And (Fld_52, Fld_54, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11594:9") else null)); 
--# expr-done 7777
And_Pred_13 := Create_And (And_Pred_11, And_Pred_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11535:44") else null)); 
--# expr-done 7778
         Property_Result := And_Pred_13;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1130;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1130;
            raise;
      end;
   return Property_Result;
end Bin_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Relation_Op
   --
      procedure Initialize_Fields_For_Relation_Op
        (Self : Bare_Relation_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Bin_Op
              (Self, Bin_Op_F_Left, Bin_Op_F_Op, Bin_Op_F_Right);
      end Initialize_Fields_For_Relation_Op;
--# property-start RelationOp.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11601'
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_Has_Context_Free_Type
  (Node : Bare_Relation_Op
  )
   return Boolean
is
   Self : Bare_Relation_Op  := Bare_Relation_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Relation_Op_P_Has_Context_Free_Type;
--# end
--# property-start RelationOp.no_overload_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11604'
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_No_Overload_Equation
  (Node : Bare_Relation_Op
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Relation_Op  := Bare_Relation_Op (Node);
     Ent : Internal_Entity_Relation_Op :=
       Internal_Entity_Relation_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1132 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Bare_Expr;
Fld_10 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1132 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1132;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7797 '<And at nodes.lkt:11604:44>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11604'
--# expr-start 7794 '<And at nodes.lkt:11605:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11605'
--# expr-start 7791 '<And at nodes.lkt:11606:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11606'
--# expr-start 7787 '<And at nodes.lkt:11607:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
--# expr-start 7781 '<Bind at nodes.lkt:11607:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
--# expr-start 7779 '<FieldAccess for type_var at nodes.lkt:11607:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7779
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7780 '<FieldAccess for bool_type at nodes.lkt:11607:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 7780
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11607:17") else null)); 
--# expr-done 7781
--# expr-start 7786 '<Bind at nodes.lkt:11607:60>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
--# expr-start 7783 '<FieldAccess for expected_type_var at nodes.lkt:11607:64>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
--# expr-start 7782 '<FieldAccess for left at nodes.lkt:11607:64>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Bin_Op_F_Left;
--# expr-done 7782
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 7783
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7785 '<FieldAccess for expected_type_var at nodes.lkt:11607:95>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
--# expr-start 7784 '<FieldAccess for right at nodes.lkt:11607:95>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11607'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Bin_Op_F_Right;
--# expr-done 7784
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 7785
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_1 := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11607:60") else null)); 
--# expr-done 7786
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11607:17") else null)); 
--# expr-done 7787
--# expr-start 7790 '<FieldAccess for infer_from_either_operands at nodes.lkt:11608:19>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11608'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7789 '<FieldAccess for expected_type_var at nodes.lkt:11608:51>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11608'
--# expr-start 7788 '<FieldAccess for left at nodes.lkt:11608:51>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11608'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Bin_Op_F_Left;
--# expr-done 7788
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_6);
--# expr-done 7789
--# property-call-start BinOp.infer_from_either_operands
Fld_8 := Libadalang.Implementation.Bin_Op_P_Infer_From_Either_Operands (Node => Ent.Node, Dest_Var => Fld_7, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 7790
And_Pred_1 := Create_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11606:13") else null)); 
--# expr-done 7791
--# expr-start 7793 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11609:15>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11609'
--# expr-start 7792 '<FieldAccess for left at nodes.lkt:11609:15>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11609'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Bin_Op_F_Left;
--# expr-done 7792
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_9, Origin => Origin);
--# end
--# expr-done 7793
And_Pred_2 := Create_And (And_Pred_1, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11605:9") else null)); 
--# expr-done 7794
--# expr-start 7796 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11610:11>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11610'
--# expr-start 7795 '<FieldAccess for right at nodes.lkt:11610:11>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11610'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Bin_Op_F_Right;
--# expr-done 7795
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_12 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_11, Origin => Origin);
--# end
--# expr-done 7796
And_Pred_3 := Create_And (And_Pred_2, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11604:44") else null)); 
--# expr-done 7797
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1132;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1132;
            raise;
      end;
   return Property_Result;
end Relation_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Box_Expr
   --
      procedure Initialize_Fields_For_Box_Expr
        (Self : Bare_Box_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Box_Expr;
--# property-start BoxExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11619'
pragma Warnings (Off, "is not referenced");
 function Box_Expr_P_Xref_Equation
  (Node : Bare_Box_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Box_Expr  := Bare_Box_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1133 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Var;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1133 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1133;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7800 '<Bind at nodes.lkt:11620:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11620'
--# expr-start 7798 '<FieldAccess for type_var at nodes.lkt:11620:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11620'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7798
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 7799 '<FieldAccess for expected_type_var at nodes.lkt:11620:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11620'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7799
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Bind_Result := Solver.Create_Unify (Fld, Fld_1, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11620:9") else null)); 
--# expr-done 7800
         Property_Result := Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1133;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1133;
            raise;
      end;
   return Property_Result;
end Box_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Case_Expr_Alternative
   --
      procedure Initialize_Fields_For_Case_Expr_Alternative
        (Self : Bare_Case_Expr_Alternative
         ; Case_Expr_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Expr_Alternative_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Case_Expr_Alternative_F_Choices := Case_Expr_Alternative_F_Choices;
            Self.Case_Expr_Alternative_F_Expr := Case_Expr_Alternative_F_Expr;
      end Initialize_Fields_For_Case_Expr_Alternative;
   function Case_Expr_Alternative_F_Choices
     (Node : Bare_Case_Expr_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Expr_Alternative_F_Choices;
   end;
   function Case_Expr_Alternative_F_Expr
     (Node : Bare_Case_Expr_Alternative) return Bare_Expr
   is
   begin
         return Node.Case_Expr_Alternative_F_Expr;
   end;
   --
   --  Primitives for Bare_Concat_Op
   --
      procedure Initialize_Fields_For_Concat_Op
        (Self : Bare_Concat_Op
         ; Concat_Op_F_First_Operand : Bare_Expr
         ; Concat_Op_F_Other_Operands : Bare_Concat_Operand_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Concat_Op_F_First_Operand := Concat_Op_F_First_Operand;
            Self.Concat_Op_F_Other_Operands := Concat_Op_F_Other_Operands;
      end Initialize_Fields_For_Concat_Op;
   function Concat_Op_F_First_Operand
     (Node : Bare_Concat_Op) return Bare_Expr
   is
   begin
         return Node.Concat_Op_F_First_Operand;
   end;
   function Concat_Op_F_Other_Operands
     (Node : Bare_Concat_Op) return Bare_Concat_Operand_List
   is
   begin
         return Node.Concat_Op_F_Other_Operands;
   end;
--# property-start ConcatOp.operands 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11663'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Operands
  (Node : Bare_Concat_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1134 with Inline_Always;
      Fld : Internal_Entity_Expr;
Array_Lit : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Concat_Operand_List;
Fld_2 : Bare_Concat_Operand_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Concat_Operand;
Bare_Item_0 : Bare_Concat_Operand;
Fld_4 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Concat_Operand;
Is_Null : Boolean;
If_Result : Internal_Entity_Concat_Operand;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1134 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1134;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 7802 '<ArrayLiteral at nodes.lkt:11664:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11664'
--# expr-start 7801 '<FieldAccess for first_operand at nodes.lkt:11664:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11664'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7801
Array_Lit := Create_Internal_Entity_Expr_Array (Internal_Internal_Entity_Expr_Array'(1 => Fld)); 
--# expr-done 7802
--# expr-start 7805 '<Map at nodes.lkt:11664:32>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11664'
   --# expr-start 7803 '<FieldAccess for other_operands at nodes.lkt:11664:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11664'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7803
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Concat_Operand;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind oo Item_0
      --# expr-start 7804 '<FieldAccess for operand at nodes.lkt:11664:64>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11664'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_0.Node.Concat_Operand_F_Operand, Info => Item_0.Info);
--# expr-done 7804
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7805
Concat_Result := Concat (Array_Lit, Map_Result); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1134;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1134;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Operands;
--# end
--# property-start ConcatOp.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11666'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Has_Context_Free_Type
  (Node : Bare_Concat_Op
  )
   return Boolean
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Has_Context_Free_Type;
--# end
--# property-start ConcatOp.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11669'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Xref_Equation
  (Node : Bare_Concat_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1136 with Inline_Always;
            procedure Finalizer_Scope_3044 with Inline_Always;
            procedure Finalizer_Scope_3045 with Inline_Always;
            procedure Finalizer_Scope_3046 with Inline_Always;
      Local_Operand_Count : Integer;
Local_Last_Operand : Bare_Concat_Operand;
Local_Concat_Subprograms : Internal_Entity_Basic_Decl_Array_Access;
Local_Pos : Integer;
Local_Left : Internal_Entity_Expr;
Local_Concat_Operand : Internal_Entity_Concat_Operand;
Local_Right : Internal_Entity_Expr;
Local_Spec : Internal_Entity_Base_Subp_Spec;
Fld : Bare_Concat_Operand_List;
Len : Integer;
Arith_Result : Integer;
Fld_1 : Bare_Concat_Operand_List;
Get_Result : Bare_Concat_Operand;
Fld_2 : Internal_Entity_Concat_Operand_List;
Fld_3 : Bare_Concat_Operand_List;
Fld_4 : Internal_Entity_Info;
Get_Result_1 : Bare_Concat_Operand;
As_Entity : Internal_Entity_Concat_Operand;
Is_Null : Boolean;
If_Result : Internal_Entity_Concat_Operand;
Fld_5 : Internal_Entity_Op_Concat;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
Fld_9 : Internal_Entity_Concat_Operand_List;
Fld_10 : Bare_Concat_Operand_List;
Fld_11 : Internal_Entity_Info;
Item_0 : Internal_Entity_Concat_Operand;
Bare_Item_0 : Bare_Concat_Operand;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
As_Entity_1 : Internal_Entity_Concat_Operand;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Concat_Operand;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Fld_14 : Internal_Entity_Concat_Operand_List;
Fld_15 : Bare_Concat_Operand_List;
Fld_16 : Internal_Entity_Info;
Item_1 : Internal_Entity_Concat_Operand;
Bare_Item_1 : Bare_Concat_Operand;
Index_2 : Integer;
Arith_Result_1 : Integer;
Arith_Result_2 : Integer;
Arith_Result_3 : Integer;
Fld_17 : Internal_Entity_Concat_Operand_List;
Fld_18 : Bare_Concat_Operand_List;
Fld_19 : Internal_Entity_Info;
Get_Result_2 : Bare_Concat_Operand;
As_Entity_2 : Internal_Entity_Concat_Operand;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Concat_Operand;
Fld_20 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Concat_Operand;
Cast_Result : Internal_Entity_Expr;
Comp_Result : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_21 : Internal_Entity_Concat_Operand_List;
Fld_22 : Bare_Concat_Operand_List;
Fld_23 : Internal_Entity_Info;
Get_Result_3 : Bare_Concat_Operand;
As_Entity_3 : Internal_Entity_Concat_Operand;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Concat_Operand;
Fld_24 : Internal_Entity_Concat_Operand_List;
Fld_25 : Bare_Concat_Operand_List;
Fld_26 : Internal_Entity_Info;
Get_Result_4 : Bare_Concat_Operand;
As_Entity_4 : Internal_Entity_Concat_Operand;
Is_Null_4 : Boolean;
If_Result_5 : Internal_Entity_Concat_Operand;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Equation;
Fld_29 : Internal_Entity_Base_Subp_Spec;
Fld_30 : Internal_Entity_Op_Concat;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_31 : Logic_Equation;
Fld_32 : Internal_Entity_Op_Concat;
Fld_33 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_34 : Internal_Entity_Op_Concat;
Fld_35 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_4 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
As_Entity_5 : Internal_Entity_Concat_Operand;
Is_Null_5 : Boolean;
If_Result_6 : Internal_Entity_Concat_Operand;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_36 : Logic_Var;
Fld_37 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_38 : Logic_Var;
Fld_39 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Let_Result_6 : Logic_Equation;
Let_Result_7 : Logic_Equation;
            procedure Finalizer_Scope_1136 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Local_Concat_Subprograms);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_1136;
            procedure Finalizer_Scope_3044 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_3044;
            procedure Finalizer_Scope_3045 is
            begin
                     Dec_Ref (Fld_28);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_3045;
            procedure Finalizer_Scope_3046 is
            begin
                     Dec_Ref (Fld_31);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3046;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7867 '<Let at nodes.lkt:11670:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11670'
--# scope-start
--# expr-start 7807 '<length at nodes.lkt:11670:29>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11670'
--# expr-start 7806 '<FieldAccess for other_operands at nodes.lkt:11670:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11670'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Concat_Op_F_Other_Operands;
--# expr-done 7806
Len := Length (Fld); 
--# expr-done 7807
Local_Operand_Count := Len; 
--# bind operand_count Local_Operand_Count
--# expr-start 7866 '<Let at nodes.lkt:11671:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11671'
--# scope-start
--# expr-start 7810 '<at at nodes.lkt:11671:28>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11671'
--# expr-start 7808 '<FieldAccess for other_operands at nodes.lkt:11671:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11671'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Concat_Op_F_Other_Operands;
--# expr-done 7808
--# expr-start 7809 '<Arithmetic - at nodes.lkt:11671:49>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11671'
Arith_Result := (Local_Operand_Count - 1); 
--# expr-done 7809
Get_Result := Get (Self, Fld_1, Arith_Result, True); 
--# expr-done 7810
Local_Last_Operand := Get_Result; 
--# bind last_operand Local_Last_Operand
--# expr-start 7865 '<Let at nodes.lkt:11672:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11672'
--# scope-start
--# expr-start 7813 '<FieldAccess for subprograms at nodes.lkt:11672:34>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11672'
--# expr-start 7812 '<FieldAccess for operator at nodes.lkt:11672:34>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11672'
--# expr-start 7811 '<FieldAccess for other_operands at nodes.lkt:11672:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11672'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7811
Fld_3 := Fld_2.Node;
Get_Result_1 := Get (Self, Fld_3, 0, True); 
Is_Null := Get_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Concat_Operand;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result_1); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Op_Concat (Node => If_Result.Node.Concat_Operand_F_Operator, Info => If_Result.Info);
--# expr-done 7812
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_6 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 7813
Local_Concat_Subprograms := Fld_6; Inc_Ref (Local_Concat_Subprograms);
--# bind concat_subprograms Local_Concat_Subprograms
--# expr-start 7864 '<And at nodes.lkt:11675:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11675'
--# expr-start 7860 '<And at nodes.lkt:11676:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11676'
--# expr-start 7856 '<And at nodes.lkt:11677:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11677'
--# expr-start 7821 '<And at nodes.lkt:11678:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
--# expr-start 7815 '<FieldAccess for sub_equation at nodes.lkt:11678:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
--# expr-start 7814 '<FieldAccess for first_operand at nodes.lkt:11678:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7814
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 7815
--# expr-start 7820 '<LogicAll at nodes.lkt:11678:59>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
--# expr-start 7819 '<Map at nodes.lkt:11678:59>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
   --# expr-start 7816 '<FieldAccess for other_operands at nodes.lkt:11678:59>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11678'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7816
Fld_10 := Fld_9.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_10 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_10;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Concat_Operand;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Bare_Item_0); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind concat_operand Item_0
      --# expr-start 7818 '<FieldAccess for sub_equation at nodes.lkt:11679:45>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11679'
--# expr-start 7817 '<FieldAccess for operand at nodes.lkt:11679:45>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11679'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Item_0.Node.Concat_Operand_F_Operand, Info => Item_0.Info);
--# expr-done 7817
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_12.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_12.Info);
--# end
--# expr-done 7818
         declare
            Item_To_Append : constant Logic_Equation := Fld_13;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3044;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 7819
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11678:59") else null)); 
--# expr-done 7820
And_Pred := Create_And (Fld_8, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11678:21") else null)); 
--# expr-done 7821
--# expr-start 7855 '<LogicAll at nodes.lkt:11689:21>' Logic_Boolean_Op_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11689'
--# expr-start 7854 '<Map at nodes.lkt:11689:21>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11689'
   --# expr-start 7822 '<FieldAccess for other_operands at nodes.lkt:11689:21>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11689'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7822
Fld_15 := Fld_14.Node;
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_15 = null then
      Map_Result_2 := Create_Logic_Equation_Array (0);
      else
         Index_2 := 0;
      declare
         Collection : constant Bare_Concat_Operand_List := Fld_15;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_5 := Bare_Item_1 = null; 
if Is_Null_5 then
   If_Result_6 := No_Entity_Concat_Operand;
else
Fld_16 := Fld_14.Info;
As_Entity_5 := (Info => Fld_16, Node => Bare_Item_1); 
   If_Result_6 := As_Entity_5;
end if;
                  Item_1 := If_Result_6; 
   --# scope-start
               --# bind _ Item_1
               --# bind index Index_2
      --# expr-start 7853 '<Let at nodes.lkt:11691:29>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11691'
--# scope-start
--# expr-start 7824 '<Arithmetic - at nodes.lkt:11691:39>' Arith_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11691'
--# expr-start 7823 '<Arithmetic - at nodes.lkt:11691:39>' Arith_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11691'
Arith_Result_1 := (Local_Operand_Count - Index_2); 
--# expr-done 7823
Arith_Result_2 := (Arith_Result_1 - 1); 
--# expr-done 7824
Local_Pos := Arith_Result_2; 
--# bind pos Local_Pos
--# expr-start 7852 '<Let at nodes.lkt:11694:33>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11694'
--# scope-start
--# expr-start 7828 '<If at nodes.lkt:11694:44>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11694'
Comp_Result := Local_Pos > 0; 
if Comp_Result then
--# expr-start 7825 '<FieldAccess for other_operands at nodes.lkt:11694:60>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11694'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7825
Fld_18 := Fld_17.Node;
--# expr-start 7826 '<Arithmetic - at nodes.lkt:11694:81>' Arith_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11694'
Arith_Result_3 := (Local_Pos - 1); 
--# expr-done 7826
Get_Result_2 := Get (Self, Fld_18, Arith_Result_3, True); 
Is_Null_2 := Get_Result_2 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Concat_Operand;
else
Fld_19 := Fld_17.Info;
As_Entity_2 := (Info => Fld_19, Node => Get_Result_2); 
   If_Result_2 := As_Entity_2;
end if;
Cast_Expr := If_Result_2; 
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_3 := Cast_Result;
else
   --# expr-start 7827 '<FieldAccess for first_operand at nodes.lkt:11694:95>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11694'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Ent.Node.Concat_Op_F_First_Operand, Info => Ent.Info);
--# expr-done 7827
   If_Result_3 := Fld_20;
end if;
--# expr-done 7828
Local_Left := If_Result_3; 
--# bind left Local_Left
--# expr-start 7851 '<Let at nodes.lkt:11695:33>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11695'
--# scope-start
--# expr-start 7829 '<FieldAccess for other_operands at nodes.lkt:11695:54>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11695'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7829
Fld_22 := Fld_21.Node;
Get_Result_3 := Get (Self, Fld_22, Local_Pos, True); 
Is_Null_3 := Get_Result_3 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Concat_Operand;
else
Fld_23 := Fld_21.Info;
As_Entity_3 := (Info => Fld_23, Node => Get_Result_3); 
   If_Result_4 := As_Entity_3;
end if;
Local_Concat_Operand := If_Result_4; 
--# bind concat_operand Local_Concat_Operand
--# expr-start 7850 '<Let at nodes.lkt:11696:33>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11696'
--# scope-start
--# expr-start 7831 '<FieldAccess for operand at nodes.lkt:11696:45>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11696'
--# expr-start 7830 '<FieldAccess for other_operands at nodes.lkt:11696:45>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11696'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Concat_Operand_List (Node => Ent.Node.Concat_Op_F_Other_Operands, Info => Ent.Info);
--# expr-done 7830
Fld_25 := Fld_24.Node;
Get_Result_4 := Get (Self, Fld_25, Local_Pos, True); 
Is_Null_4 := Get_Result_4 = null; 
if Is_Null_4 then
   If_Result_5 := No_Entity_Concat_Operand;
else
Fld_26 := Fld_24.Info;
As_Entity_4 := (Info => Fld_26, Node => Get_Result_4); 
   If_Result_5 := As_Entity_4;
end if;
      if If_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => If_Result_5.Node.Concat_Operand_F_Operand, Info => If_Result_5.Info);
--# expr-done 7831
Local_Right := Fld_27; 
--# bind right Local_Right
--# expr-start 7849 '<Or at nodes.lkt:11698:33>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11698'
--# expr-start 7832 '<FieldAccess for operator_no_subprogram_equation at nodes.lkt:11703:37>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11703'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.operator_no_subprogram_equation
Fld_28 := Libadalang.Implementation.Concat_Op_P_Operator_No_Subprogram_Equation (Node => Ent.Node, Left => Local_Left, Op => Local_Concat_Operand, Right => Local_Right, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7832
--# expr-start 7848 '<LogicAny at nodes.lkt:11709:37>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11709'
--# expr-start 7847 '<Map at nodes.lkt:11709:37>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11709'
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Concat_Subprograms;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_3
      --# expr-start 7846 '<Let at nodes.lkt:11711:45>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11711'
--# scope-start
--# expr-start 7833 '<FieldAccess for subp_spec_or_null at nodes.lkt:11711:56>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11711'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_29 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_3.Node, E_Info => Item_3.Info);
--# end
--# expr-done 7833
Local_Spec := Fld_29; 
--# bind spec Local_Spec
--# expr-start 7845 '<bind at nodes.lkt:11714:49>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11714'
--# expr-start 7835 '<New LogicContext at nodes.lkt:11714:70>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11714'
Cast_Expr_1 := Item_3; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 7834 '<FieldAccess for operator at nodes.lkt:11715:62>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11715'
      if Local_Concat_Operand.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Op_Concat (Node => Local_Concat_Operand.Node.Concat_Operand_F_Operator, Info => Local_Concat_Operand.Info);
--# expr-done 7834
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
New_Struct := (Decl_Node => Cast_Result_2, Ref_Node => Cast_Result_1); 
--# expr-done 7835
Bound_Logic_Context := New_Struct; 
--# expr-start 7844 '<And at nodes.lkt:11718:49>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7840 '<And at nodes.lkt:11718:49>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7836 '<FieldAccess for arguments_eq at nodes.lkt:11718:49>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.arguments_eq
Fld_31 := Libadalang.Implementation.Concat_Op_P_Arguments_Eq (Node => Ent.Node, Spec => Local_Spec, Left => Local_Left, Op => Local_Concat_Operand, Right => Local_Right, Origin => Origin, Env => Env, Logic_Context => Bound_Logic_Context);
--# end
--# expr-done 7836
--# expr-start 7839 '<Bind at nodes.lkt:11718:106>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7838 '<FieldAccess for ref_var at nodes.lkt:11718:110>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7837 '<FieldAccess for operator at nodes.lkt:11718:110>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
      if Local_Concat_Operand.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Op_Concat (Node => Local_Concat_Operand.Node.Concat_Operand_F_Operator, Info => Local_Concat_Operand.Info);
--# expr-done 7837
      if Fld_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_32.Node);
--# end
--# expr-done 7838
Fld_33.Value := No_Entity;
Entity_Vars.Reset (Fld_33);
Cast_Expr_2 := Item_3; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result := Solver.Create_Assign (Fld_33, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11718:106") else null)); 
--# expr-done 7839
And_Pred_1 := Create_And (Fld_31, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11718:49") else null)); 
--# expr-done 7840
--# expr-start 7843 '<Bind at nodes.lkt:11718:155>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7842 '<FieldAccess for subp_spec_var at nodes.lkt:11718:159>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
--# expr-start 7841 '<FieldAccess for operator at nodes.lkt:11718:159>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11718'
      if Local_Concat_Operand.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Op_Concat (Node => Local_Concat_Operand.Node.Concat_Operand_F_Operator, Info => Local_Concat_Operand.Info);
--# expr-done 7841
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_34.Node);
--# end
--# expr-done 7842
Fld_35.Value := No_Entity;
Entity_Vars.Reset (Fld_35);
Cast_Expr_3 := Local_Spec; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_35, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11718:155") else null)); 
--# expr-done 7843
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11718:49") else null)); 
--# expr-done 7844
Dyn_Var_Bind_Result := And_Pred_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7845
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7846
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3046;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 7847
Logic_Boolean_Op_1 := Solver.Create_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11709:37") else null)); 
--# expr-done 7848
Or_Pred := Create_Or (Fld_28, Logic_Boolean_Op_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11698:33") else null)); 
--# expr-done 7849
Let_Result_1 := Or_Pred; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7850
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 7851
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 7852
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7853
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3045;
               Index_2 := Index_2 + 1;
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 7854
Logic_Boolean_Op_2 := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11689:21") else null)); 
--# expr-done 7855
And_Pred_3 := Create_And (And_Pred, Logic_Boolean_Op_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11677:17") else null)); 
--# expr-done 7856
--# expr-start 7859 '<Bind at nodes.lkt:11729:17>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11729'
--# expr-start 7857 '<FieldAccess for type_var at nodes.lkt:11729:21>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11729'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7857
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
--# expr-start 7858 '<FieldAccess for type_var at nodes.lkt:11729:38>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11729'
      if Local_Last_Operand = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Local_Last_Operand);
--# expr-done 7858
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
Bind_Result_2 := Solver.Create_Unify (Fld_36, Fld_37, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11729:17") else null)); 
--# expr-done 7859
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11676:13") else null)); 
--# expr-done 7860
--# expr-start 7863 '<Bind at nodes.lkt:11731:15>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11731'
--# expr-start 7861 '<FieldAccess for expected_type_var at nodes.lkt:11731:19>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11731'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 7861
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
--# expr-start 7862 '<FieldAccess for expected_type_var at nodes.lkt:11731:45>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11731'
      if Local_Last_Operand = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Local_Last_Operand);
--# expr-done 7862
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
Bind_Result_3 := Solver.Create_Unify (Fld_38, Fld_39, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11731:15") else null)); 
--# expr-done 7863
And_Pred_5 := Create_And (And_Pred_4, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11675:9") else null)); 
--# expr-done 7864
Let_Result_5 := And_Pred_5; Inc_Ref (Let_Result_5);
--# end
--# expr-done 7865
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 7866
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 7867
         Property_Result := Let_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1136;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1136;
                     Finalizer_Scope_3044;
                     Finalizer_Scope_3045;
                     Finalizer_Scope_3046;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Xref_Equation;
--# end
--# property-start ConcatOp.arguments_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11735'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Arguments_Eq
  (Node : Bare_Concat_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Logic_Context : Internal_Logic_Context
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   --# bind env Env
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1137 with Inline_Always;
      Local_Ps : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Logic_Equation;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1137 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Ps);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1137;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7883 '<Let at nodes.lkt:11736:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11736'
--# scope-start
--# expr-start 7868 '<FieldAccess for unpacked_formal_params at nodes.lkt:11736:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11736'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7868
Local_Ps := Fld; Inc_Ref (Local_Ps);
--# bind ps Local_Ps
--# expr-start 7882 '<If at nodes.lkt:11738:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11738'
--# expr-start 7870 '<Eq at nodes.lkt:11738:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11738'
--# expr-start 7869 '<length at nodes.lkt:11738:12>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11738'
Len := Length (Local_Ps); 
--# expr-done 7869
Is_Equal := Len = 2; 
--# expr-done 7870
if Is_Equal then
   --# expr-start 7881 '<And at nodes.lkt:11740:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11740'
--# expr-start 7877 '<And at nodes.lkt:11741:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11741'
--# expr-start 7873 '<FieldAccess for call_argument_equation at nodes.lkt:11741:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11741'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7872 '<FieldAccess for formal_decl at nodes.lkt:11741:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11741'
--# expr-start 7871 '<at at nodes.lkt:11741:45>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11741'
Get_Result := Get (Self, Local_Ps, 0, True); 
--# expr-done 7871
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7872
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_1, Arg => Left, Origin => Origin, Logic_Context => Logic_Context, E_Info => Spec.Info);
--# end
--# expr-done 7873
--# expr-start 7876 '<FieldAccess for call_argument_equation at nodes.lkt:11743:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11743'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7875 '<FieldAccess for formal_decl at nodes.lkt:11743:49>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11743'
--# expr-start 7874 '<at at nodes.lkt:11743:49>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11743'
Get_Result_1 := Get (Self, Local_Ps, 1, True); 
--# expr-done 7874
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 7875
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Spec.Node, Param => Fld_3, Arg => Right, Origin => Origin, Logic_Context => Logic_Context, E_Info => Spec.Info);
--# end
--# expr-done 7876
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11741:17") else null)); 
--# expr-done 7877
--# expr-start 7880 '<Bind at nodes.lkt:11747:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11747'
--# expr-start 7878 '<FieldAccess for type_var at nodes.lkt:11747:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11747'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7878
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7879 '<FieldAccess for return_type at nodes.lkt:11747:36>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11747'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Spec.Node, Origin => Origin, E_Info => Spec.Info);
--# end
--# expr-done 7879
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11747:17") else null)); 
--# expr-done 7880
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11740:13") else null)); 
--# expr-done 7881
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11749:16") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 7882
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7883
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1137;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1137;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Arguments_Eq;
--# end
--# property-start ConcatOp.operator_no_subprogram_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11755'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Operator_No_Subprogram_Equation
  (Node : Bare_Concat_Op
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
     Ent : Internal_Entity_Concat_Op :=
       Internal_Entity_Concat_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1138 with Inline_Always;
      Fld : Internal_Entity_Op_Concat;
Cast_Result : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Logic_Var;
Pred_2 : Logic_Equation;
Fld_4 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_5 : Logic_Var;
Node_For_Entity : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result_1 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Var;
Logic_Ctx_1 : Internal_Logic_Context_Access;
Bind_Result_4 : Logic_Equation;
Fld_17 : Logic_Var;
Fld_18 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_19 : Logic_Var;
Fld_20 : Logic_Var;
Bind_Result_6 : Logic_Equation;
Fld_21 : Logic_Var;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1138 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1138;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 7932 '<bind at nodes.lkt:11756:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11756'
--# expr-start 7885 '<New LogicContext at nodes.lkt:11756:30>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11756'
--# expr-start 7884 '<FieldAccess for operator at nodes.lkt:11757:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11757'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op_Concat (Node => Op.Node.Concat_Operand_F_Operator, Info => Op.Info);
--# expr-done 7884
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
New_Struct := (Decl_Node => No_Entity, Ref_Node => Cast_Result); 
--# expr-done 7885
Bound_Logic_Context := New_Struct; 
--# expr-start 7931 '<And at nodes.lkt:11760:9>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11760'
--# expr-start 7890 '<And at nodes.lkt:11761:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11761'
--# expr-start 7887 '<Predicate on BaseTypeDecl.is_array_def_with_deref_or_null at nodes.lkt:11761:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11761'
--# expr-start 7886 '<FieldAccess for expected_type_var at nodes.lkt:11761:70>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11761'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7886
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_Or_Null_0_Predicate (Origin)); 
--# expr-done 7887
--# expr-start 7889 '<Predicate on BaseTypeDecl.is_array_def_with_deref at nodes.lkt:11761:98>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11761'
--# expr-start 7888 '<FieldAccess for type_var at nodes.lkt:11761:147>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11761'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7888
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Pred_1 := Solver.Create_Predicate (Fld_2, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin)); 
--# expr-done 7889
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11761:13") else null)); 
--# expr-done 7890
--# expr-start 7930 '<Or at nodes.lkt:11763:13>' Or_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11763'
--# expr-start 7894 '<And at nodes.lkt:11766:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11766'
--# expr-start 7892 '<Predicate on AdaNode.is_not_null at nodes.lkt:11766:17>' Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11766'
--# expr-start 7891 '<FieldAccess for expected_type_var at nodes.lkt:11766:49>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11766'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7891
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred_2 := Solver.Create_Predicate (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 7892
--# expr-start 7893 '<FieldAccess for array_concat_expected_type_equation at nodes.lkt:11766:77>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11766'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ConcatOp.array_concat_expected_type_equation
Fld_4 := Libadalang.Implementation.Concat_Op_P_Array_Concat_Expected_Type_Equation (Node => Ent.Node, Left => Left, Op => Op, Right => Right, Origin => Origin, Logic_Context => Bound_Logic_Context);
--# end
--# expr-done 7893
And_Pred_1 := Create_And (Pred_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11766:17") else null)); 
--# expr-done 7894
--# expr-start 7929 '<And at nodes.lkt:11775:17>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11775'
--# expr-start 7897 '<Bind at nodes.lkt:11775:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11775'
--# expr-start 7895 '<FieldAccess for expected_type_var at nodes.lkt:11775:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11775'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7895
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7896 '<No BaseTypeDecl at nodes.lkt:11775:45>' Node_For_Entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11775'
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 7896
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11775:17") else null)); 
--# expr-done 7897
--# expr-start 7928 '<Or at nodes.lkt:11776:21>' Or_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11776'
--# expr-start 7912 '<And at nodes.lkt:11778:25>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11778'
--# expr-start 7904 '<And at nodes.lkt:11779:29>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
--# expr-start 7900 '<Bind at nodes.lkt:11779:29>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
--# expr-start 7898 '<FieldAccess for type_var at nodes.lkt:11779:48>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7898
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 7899 '<FieldAccess for type_var at nodes.lkt:11779:33>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7899
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Logic_Ctx := Allocate_Logic_Context (Bound_Logic_Context); 
Bind_Result_1 := Solver.Create_Propagate (Fld_7, Fld_6, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11779:29") else null)); 
--# expr-done 7900
--# expr-start 7903 '<Bind at nodes.lkt:11779:139>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
--# expr-start 7901 '<FieldAccess for type_var at nodes.lkt:11779:143>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7901
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 7902 '<FieldAccess for expected_type_var at nodes.lkt:11779:158>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11779'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7902
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Bind_Result_2 := Solver.Create_Unify (Fld_8, Fld_9, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11779:139") else null)); 
--# expr-done 7903
And_Pred_2 := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11779:29") else null)); 
--# expr-done 7904
--# expr-start 7911 '<Or at nodes.lkt:11781:29>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
--# expr-start 7907 '<Bind at nodes.lkt:11781:29>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
--# expr-start 7905 '<FieldAccess for type_var at nodes.lkt:11781:33>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7905
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 7906 '<FieldAccess for expected_type_var at nodes.lkt:11781:48>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7906
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result_3 := Solver.Create_Unify (Fld_10, Fld_11, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11781:29") else null)); 
--# expr-done 7907
--# expr-start 7910 '<FieldAccess for comp_bind at nodes.lkt:11781:78>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7908 '<FieldAccess for type_var at nodes.lkt:11781:93>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7908
--# expr-start 7909 '<FieldAccess for expected_type_var at nodes.lkt:11781:108>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11781'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7909
--# property-call-start AdaNode.comp_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_12, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 7910
Or_Pred := Create_Or (Bind_Result_3, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11781:29") else null)); 
--# expr-done 7911
And_Pred_3 := Create_And (And_Pred_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11778:25") else null)); 
--# expr-done 7912
--# expr-start 7927 '<And at nodes.lkt:11785:25>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11785'
--# expr-start 7919 '<And at nodes.lkt:11786:29>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
--# expr-start 7915 '<Bind at nodes.lkt:11786:29>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
--# expr-start 7913 '<FieldAccess for type_var at nodes.lkt:11786:48>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7913
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
--# expr-start 7914 '<FieldAccess for type_var at nodes.lkt:11786:33>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7914
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
Logic_Ctx_1 := Allocate_Logic_Context (Bound_Logic_Context); 
Bind_Result_4 := Solver.Create_Propagate (Fld_16, Fld_15, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_1, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11786:29") else null)); 
--# expr-done 7915
--# expr-start 7918 '<Bind at nodes.lkt:11786:140>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
--# expr-start 7916 '<FieldAccess for type_var at nodes.lkt:11786:144>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7916
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
--# expr-start 7917 '<FieldAccess for expected_type_var at nodes.lkt:11786:159>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11786'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7917
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
Bind_Result_5 := Solver.Create_Unify (Fld_17, Fld_18, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11786:140") else null)); 
--# expr-done 7918
And_Pred_4 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11786:29") else null)); 
--# expr-done 7919
--# expr-start 7926 '<Or at nodes.lkt:11788:29>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
--# expr-start 7922 '<Bind at nodes.lkt:11788:29>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
--# expr-start 7920 '<FieldAccess for type_var at nodes.lkt:11788:33>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7920
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 7921 '<FieldAccess for expected_type_var at nodes.lkt:11788:48>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7921
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Bind_Result_6 := Solver.Create_Unify (Fld_19, Fld_20, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11788:29") else null)); 
--# expr-done 7922
--# expr-start 7925 '<FieldAccess for comp_bind at nodes.lkt:11788:77>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7923 '<FieldAccess for type_var at nodes.lkt:11788:92>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7923
--# expr-start 7924 '<FieldAccess for expected_type_var at nodes.lkt:11788:107>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11788'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7924
--# property-call-start AdaNode.comp_bind
Fld_23 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_21, Right => Fld_22, Origin => Origin);
--# end
--# expr-done 7925
Or_Pred_1 := Create_Or (Bind_Result_6, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11788:29") else null)); 
--# expr-done 7926
And_Pred_5 := Create_And (And_Pred_4, Or_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11785:25") else null)); 
--# expr-done 7927
Or_Pred_2 := Create_Or (And_Pred_3, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11776:21") else null)); 
--# expr-done 7928
And_Pred_6 := Create_And (Bind_Result, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11775:17") else null)); 
--# expr-done 7929
Or_Pred_3 := Create_Or (And_Pred_1, And_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11763:13") else null)); 
--# expr-done 7930
And_Pred_7 := Create_And (And_Pred, Or_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11760:9") else null)); 
--# expr-done 7931
Dyn_Var_Bind_Result := And_Pred_7; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7932
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1138;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1138;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Operator_No_Subprogram_Equation;
--# end
--# property-start ConcatOp.array_concat_expected_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11801'
pragma Warnings (Off, "is not referenced");
 function Concat_Op_P_Array_Concat_Expected_Type_Equation
  (Node : Bare_Concat_Op
      ; Left : Internal_Entity_Expr
      ; Op : Internal_Entity_Concat_Operand
      ; Right : Internal_Entity_Expr
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
  )
   return Logic_Equation
is
   Self : Bare_Concat_Op  := Bare_Concat_Op (Node);
      --# bind self Self
   --# bind left Left
   --# bind op Op
   --# bind right Right
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1139 with Inline_Always;
      Local_Left_Ctx_Free : Boolean;
Local_Right_Ctx_Free : Boolean;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Logic_Ctx_1 : Internal_Logic_Context_Access;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Fld_10 : Logic_Var;
Logic_Ctx_2 : Internal_Logic_Context_Access;
Bind_Result_2 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Logic_Ctx_3 : Internal_Logic_Context_Access;
Bind_Result_3 : Logic_Equation;
Fld_13 : Logic_Var;
Fld_14 : Logic_Var;
Fld_15 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Logic_Var;
Fld_18 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_19 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_20 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_21 : Logic_Var;
Fld_22 : Logic_Var;
Logic_Ctx_4 : Internal_Logic_Context_Access;
Bind_Result_4 : Logic_Equation;
Fld_23 : Logic_Var;
Fld_24 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Logic_Var;
Logic_Ctx_5 : Internal_Logic_Context_Access;
Bind_Result_6 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Logic_Var;
Fld_29 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_30 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Logic_Var;
Bind_Result_7 : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Var;
Fld_35 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_36 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_37 : Logic_Var;
Fld_38 : Logic_Var;
Logic_Ctx_6 : Internal_Logic_Context_Access;
Bind_Result_8 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Logic_Var;
Bind_Result_9 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_41 : Logic_Var;
Fld_42 : Logic_Var;
Logic_Ctx_7 : Internal_Logic_Context_Access;
Bind_Result_10 : Logic_Equation;
Fld_43 : Logic_Var;
Fld_44 : Logic_Var;
Fld_45 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Fld_46 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
Fld_47 : Logic_Var;
Fld_48 : Logic_Var;
Bind_Result_11 : Logic_Equation;
Fld_49 : Logic_Var;
Fld_50 : Logic_Var;
Fld_51 : Logic_Equation;
Or_Pred_4 : Logic_Equation;
And_Pred_14 : Logic_Equation;
Fld_52 : Logic_Equation;
And_Pred_15 : Logic_Equation;
Fld_53 : Logic_Var;
Fld_54 : Logic_Var;
Logic_Ctx_8 : Internal_Logic_Context_Access;
Bind_Result_12 : Logic_Equation;
Fld_55 : Logic_Var;
Fld_56 : Logic_Var;
Bind_Result_13 : Logic_Equation;
Fld_57 : Logic_Var;
Fld_58 : Logic_Var;
Fld_59 : Logic_Equation;
Or_Pred_5 : Logic_Equation;
And_Pred_16 : Logic_Equation;
Fld_60 : Logic_Var;
Fld_61 : Logic_Var;
Bind_Result_14 : Logic_Equation;
Fld_62 : Logic_Var;
Fld_63 : Logic_Var;
Fld_64 : Logic_Equation;
Or_Pred_6 : Logic_Equation;
And_Pred_17 : Logic_Equation;
Fld_65 : Logic_Equation;
And_Pred_18 : Logic_Equation;
Fld_66 : Logic_Equation;
And_Pred_19 : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1139 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Or_Pred_2);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Bind_Result_8);
                     Dec_Ref (Bind_Result_9);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Bind_Result_10);
                     Dec_Ref (Fld_45);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Fld_46);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (Bind_Result_11);
                     Dec_Ref (Fld_51);
                     Dec_Ref (Or_Pred_4);
                     Dec_Ref (And_Pred_14);
                     Dec_Ref (Fld_52);
                     Dec_Ref (And_Pred_15);
                     Dec_Ref (Bind_Result_12);
                     Dec_Ref (Bind_Result_13);
                     Dec_Ref (Fld_59);
                     Dec_Ref (Or_Pred_5);
                     Dec_Ref (And_Pred_16);
                     Dec_Ref (Bind_Result_14);
                     Dec_Ref (Fld_64);
                     Dec_Ref (Or_Pred_6);
                     Dec_Ref (And_Pred_17);
                     Dec_Ref (Fld_65);
                     Dec_Ref (And_Pred_18);
                     Dec_Ref (Fld_66);
                     Dec_Ref (And_Pred_19);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1139;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8046 '<Let at nodes.lkt:11802:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11802'
--# scope-start
--# expr-start 7933 '<FieldAccess for has_context_free_type at nodes.lkt:11802:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11802'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Left.Node);
--# end
--# expr-done 7933
Local_Left_Ctx_Free := Fld; 
--# bind left_ctx_free Local_Left_Ctx_Free
--# expr-start 8045 '<Let at nodes.lkt:11803:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11803'
--# scope-start
--# expr-start 7934 '<FieldAccess for has_context_free_type at nodes.lkt:11803:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11803'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Right.Node);
--# end
--# expr-done 7934
Local_Right_Ctx_Free := Fld_1; 
--# bind right_ctx_free Local_Right_Ctx_Free
--# expr-start 8044 '<If at nodes.lkt:11808:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11808'
if Local_Left_Ctx_Free then
   If_Result_2 := Local_Right_Ctx_Free;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 7964 '<Or at nodes.lkt:11808:50>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11808'
--# expr-start 7948 '<And at nodes.lkt:11815:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11815'
--# expr-start 7943 '<And at nodes.lkt:11816:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
--# expr-start 7938 '<NPropagate at nodes.lkt:11816:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
--# expr-start 7935 '<FieldAccess for type_var at nodes.lkt:11816:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7935
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 7936 '<FieldAccess for type_var at nodes.lkt:11816:107>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7936
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 7937 '<FieldAccess for type_var at nodes.lkt:11816:124>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7937
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_N_Propagate (Fld_2, Logic_Functor_867'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_3, 2 => Fld_4), Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11816:17") else null)); 
--# expr-done 7938
--# expr-start 7942 '<NPropagate at nodes.lkt:11816:146>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
--# expr-start 7939 '<FieldAccess for expected_type_var at nodes.lkt:11816:157>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7939
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 7940 '<FieldAccess for type_var at nodes.lkt:11816:257>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7940
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 7941 '<FieldAccess for type_var at nodes.lkt:11816:272>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11816'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7941
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Logic_Ctx_1 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_1 := Solver.Create_N_Propagate (Fld_5, Logic_Functor_868'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_6, 2 => Fld_7), Logic_Ctx_1, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11816:146") else null)); 
--# expr-done 7942
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11816:17") else null)); 
--# expr-done 7943
--# expr-start 7947 '<NPropagate at nodes.lkt:11817:19>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11817'
--# expr-start 7944 '<FieldAccess for expected_type_var at nodes.lkt:11817:30>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11817'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7944
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 7945 '<FieldAccess for type_var at nodes.lkt:11817:131>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11817'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7945
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 7946 '<FieldAccess for type_var at nodes.lkt:11817:146>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11817'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7946
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Logic_Ctx_2 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_2 := Solver.Create_N_Propagate (Fld_8, Logic_Functor_868'(N => 2, Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), (1 => Fld_9, 2 => Fld_10), Logic_Ctx_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11817:19") else null)); 
--# expr-done 7947
And_Pred_1 := Create_And (And_Pred, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11815:13") else null)); 
--# expr-done 7948
--# expr-start 7963 '<And at nodes.lkt:11823:13>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11823'
--# expr-start 7961 '<And at nodes.lkt:11824:17>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11824'
--# expr-start 7959 '<And at nodes.lkt:11825:21>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11825'
--# expr-start 7955 '<And at nodes.lkt:11826:25>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
--# expr-start 7951 '<Bind at nodes.lkt:11826:25>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
--# expr-start 7949 '<FieldAccess for expected_type_var at nodes.lkt:11826:44>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7949
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 7950 '<FieldAccess for type_var at nodes.lkt:11826:29>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7950
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Logic_Ctx_3 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_3 := Solver.Create_Propagate (Fld_12, Fld_11, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11826:25") else null)); 
--# expr-done 7951
--# expr-start 7954 '<FieldAccess for comp_bind at nodes.lkt:11826:142>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7952 '<FieldAccess for type_var at nodes.lkt:11826:157>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7952
--# expr-start 7953 '<FieldAccess for expected_type_var at nodes.lkt:11826:172>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11826'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7953
--# property-call-start AdaNode.comp_bind
Fld_15 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_13, Right => Fld_14, Origin => Origin);
--# end
--# expr-done 7954
And_Pred_2 := Create_And (Bind_Result_3, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11826:25") else null)); 
--# expr-done 7955
--# expr-start 7958 '<FieldAccess for comp_bind at nodes.lkt:11827:27>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11827'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7956 '<FieldAccess for type_var at nodes.lkt:11827:42>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11827'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7956
--# expr-start 7957 '<FieldAccess for expected_type_var at nodes.lkt:11827:57>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11827'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7957
--# property-call-start AdaNode.comp_bind
Fld_18 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_16, Right => Fld_17, Origin => Origin);
--# end
--# expr-done 7958
And_Pred_3 := Create_And (And_Pred_2, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11825:21") else null)); 
--# expr-done 7959
--# expr-start 7960 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11828:23>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11828'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_19 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7960
And_Pred_4 := Create_And (And_Pred_3, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11824:17") else null)); 
--# expr-done 7961
--# expr-start 7962 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11829:19>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11829'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_20 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7962
And_Pred_5 := Create_And (And_Pred_4, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11823:13") else null)); 
--# expr-done 7963
Or_Pred := Create_Or (And_Pred_1, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11808:50") else null)); 
--# expr-done 7964
   If_Result_3 := Or_Pred;
else
   --# expr-start 8043 '<If at nodes.lkt:11808:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11808'
if Local_Left_Ctx_Free then
   --# expr-start 7991 '<And at nodes.lkt:11831:33>' And_Pred_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11831'
--# expr-start 7989 '<And at nodes.lkt:11832:13>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11832'
--# expr-start 7981 '<Or at nodes.lkt:11833:17>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11833'
--# expr-start 7971 '<And at nodes.lkt:11837:21>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
--# expr-start 7967 '<Bind at nodes.lkt:11837:21>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
--# expr-start 7965 '<FieldAccess for type_var at nodes.lkt:11837:40>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Left.Node);
--# expr-done 7965
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 7966 '<FieldAccess for type_var at nodes.lkt:11837:25>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7966
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Logic_Ctx_4 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_4 := Solver.Create_Propagate (Fld_22, Fld_21, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11837:21") else null)); 
--# expr-done 7967
--# expr-start 7970 '<Bind at nodes.lkt:11837:131>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
--# expr-start 7968 '<FieldAccess for type_var at nodes.lkt:11837:135>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7968
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 7969 '<FieldAccess for expected_type_var at nodes.lkt:11837:150>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11837'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7969
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
Bind_Result_5 := Solver.Create_Unify (Fld_23, Fld_24, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11837:131") else null)); 
--# expr-done 7970
And_Pred_6 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11837:21") else null)); 
--# expr-done 7971
--# expr-start 7980 '<And at nodes.lkt:11843:21>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11843'
--# expr-start 7978 '<And at nodes.lkt:11844:25>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
--# expr-start 7974 '<Bind at nodes.lkt:11844:25>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
--# expr-start 7972 '<FieldAccess for expected_type_var at nodes.lkt:11844:44>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7972
Fld_26.Value := No_Entity;
Entity_Vars.Reset (Fld_26);
--# expr-start 7973 '<FieldAccess for type_var at nodes.lkt:11844:29>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7973
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
Logic_Ctx_5 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_6 := Solver.Create_Propagate (Fld_26, Fld_25, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11844:25") else null)); 
--# expr-done 7974
--# expr-start 7977 '<FieldAccess for comp_bind at nodes.lkt:11844:142>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7975 '<FieldAccess for type_var at nodes.lkt:11844:157>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7975
--# expr-start 7976 '<FieldAccess for expected_type_var at nodes.lkt:11844:172>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11844'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 7976
--# property-call-start AdaNode.comp_bind
Fld_29 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_27, Right => Fld_28, Origin => Origin);
--# end
--# expr-done 7977
And_Pred_7 := Create_And (Bind_Result_6, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11844:25") else null)); 
--# expr-done 7978
--# expr-start 7979 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11845:27>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11845'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_30 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 7979
And_Pred_8 := Create_And (And_Pred_7, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11843:21") else null)); 
--# expr-done 7980
Or_Pred_1 := Create_Or (And_Pred_6, And_Pred_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11833:17") else null)); 
--# expr-done 7981
--# expr-start 7988 '<Or at nodes.lkt:11850:17>' Or_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
--# expr-start 7984 '<Bind at nodes.lkt:11850:17>' Bind_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
--# expr-start 7982 '<FieldAccess for type_var at nodes.lkt:11850:21>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7982
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
--# expr-start 7983 '<FieldAccess for expected_type_var at nodes.lkt:11850:36>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7983
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Bind_Result_7 := Solver.Create_Unify (Fld_31, Fld_32, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11850:17") else null)); 
--# expr-done 7984
--# expr-start 7987 '<FieldAccess for comp_bind at nodes.lkt:11850:66>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 7985 '<FieldAccess for type_var at nodes.lkt:11850:81>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7985
--# expr-start 7986 '<FieldAccess for expected_type_var at nodes.lkt:11850:96>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11850'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7986
--# property-call-start AdaNode.comp_bind
Fld_35 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_33, Right => Fld_34, Origin => Origin);
--# end
--# expr-done 7987
Or_Pred_2 := Create_Or (Bind_Result_7, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11850:17") else null)); 
--# expr-done 7988
And_Pred_9 := Create_And (Or_Pred_1, Or_Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11832:13") else null)); 
--# expr-done 7989
--# expr-start 7990 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11852:15>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11852'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_36 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 7990
And_Pred_10 := Create_And (And_Pred_9, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11831:33") else null)); 
--# expr-done 7991
   If_Result_1 := And_Pred_10;
else
   --# expr-start 8042 '<If at nodes.lkt:11808:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11808'
if Local_Right_Ctx_Free then
   --# expr-start 8018 '<And at nodes.lkt:11853:34>' And_Pred_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11853'
--# expr-start 8016 '<And at nodes.lkt:11854:13>' And_Pred_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11854'
--# expr-start 8008 '<Or at nodes.lkt:11855:17>' Or_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11855'
--# expr-start 7998 '<And at nodes.lkt:11859:21>' And_Pred_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
--# expr-start 7994 '<Bind at nodes.lkt:11859:21>' Bind_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
--# expr-start 7992 '<FieldAccess for type_var at nodes.lkt:11859:40>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Right.Node);
--# expr-done 7992
Fld_38.Value := No_Entity;
Entity_Vars.Reset (Fld_38);
--# expr-start 7993 '<FieldAccess for type_var at nodes.lkt:11859:25>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7993
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
Logic_Ctx_6 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_8 := Solver.Create_Propagate (Fld_38, Fld_37, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11859:21") else null)); 
--# expr-done 7994
--# expr-start 7997 '<Bind at nodes.lkt:11859:132>' Bind_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
--# expr-start 7995 '<FieldAccess for type_var at nodes.lkt:11859:136>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 7995
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 7996 '<FieldAccess for expected_type_var at nodes.lkt:11859:151>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11859'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 7996
Fld_40.Value := No_Entity;
Entity_Vars.Reset (Fld_40);
Bind_Result_9 := Solver.Create_Unify (Fld_39, Fld_40, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11859:132") else null)); 
--# expr-done 7997
And_Pred_11 := Create_And (Bind_Result_8, Bind_Result_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11859:21") else null)); 
--# expr-done 7998
--# expr-start 8007 '<And at nodes.lkt:11865:21>' And_Pred_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11865'
--# expr-start 8005 '<And at nodes.lkt:11866:25>' And_Pred_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
--# expr-start 8001 '<Bind at nodes.lkt:11866:25>' Bind_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
--# expr-start 7999 '<FieldAccess for expected_type_var at nodes.lkt:11866:44>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 7999
Fld_42.Value := No_Entity;
Entity_Vars.Reset (Fld_42);
--# expr-start 8000 '<FieldAccess for type_var at nodes.lkt:11866:29>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8000
Fld_41.Value := No_Entity;
Entity_Vars.Reset (Fld_41);
Logic_Ctx_7 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_10 := Solver.Create_Propagate (Fld_42, Fld_41, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11866:25") else null)); 
--# expr-done 8001
--# expr-start 8004 '<FieldAccess for comp_bind at nodes.lkt:11866:142>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8002 '<FieldAccess for type_var at nodes.lkt:11866:157>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8002
--# expr-start 8003 '<FieldAccess for expected_type_var at nodes.lkt:11866:172>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11866'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 8003
--# property-call-start AdaNode.comp_bind
Fld_45 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_43, Right => Fld_44, Origin => Origin);
--# end
--# expr-done 8004
And_Pred_12 := Create_And (Bind_Result_10, Fld_45, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11866:25") else null)); 
--# expr-done 8005
--# expr-start 8006 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11867:27>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11867'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_46 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 8006
And_Pred_13 := Create_And (And_Pred_12, Fld_46, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11865:21") else null)); 
--# expr-done 8007
Or_Pred_3 := Create_Or (And_Pred_11, And_Pred_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11855:17") else null)); 
--# expr-done 8008
--# expr-start 8015 '<Or at nodes.lkt:11872:17>' Or_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
--# expr-start 8011 '<Bind at nodes.lkt:11872:17>' Bind_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
--# expr-start 8009 '<FieldAccess for type_var at nodes.lkt:11872:21>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_47 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8009
Fld_47.Value := No_Entity;
Entity_Vars.Reset (Fld_47);
--# expr-start 8010 '<FieldAccess for expected_type_var at nodes.lkt:11872:36>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 8010
Fld_48.Value := No_Entity;
Entity_Vars.Reset (Fld_48);
Bind_Result_11 := Solver.Create_Unify (Fld_47, Fld_48, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11872:17") else null)); 
--# expr-done 8011
--# expr-start 8014 '<FieldAccess for comp_bind at nodes.lkt:11872:65>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8012 '<FieldAccess for type_var at nodes.lkt:11872:80>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_49 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8012
--# expr-start 8013 '<FieldAccess for expected_type_var at nodes.lkt:11872:95>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11872'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 8013
--# property-call-start AdaNode.comp_bind
Fld_51 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_49, Right => Fld_50, Origin => Origin);
--# end
--# expr-done 8014
Or_Pred_4 := Create_Or (Bind_Result_11, Fld_51, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11872:17") else null)); 
--# expr-done 8015
And_Pred_14 := Create_And (Or_Pred_3, Or_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11854:13") else null)); 
--# expr-done 8016
--# expr-start 8017 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11874:15>' Fld_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11874'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_52 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 8017
And_Pred_15 := Create_And (And_Pred_14, Fld_52, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11853:34") else null)); 
--# expr-done 8018
   If_Result := And_Pred_15;
else
   --# expr-start 8041 '<And at nodes.lkt:11878:14>' And_Pred_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11878'
--# expr-start 8039 '<And at nodes.lkt:11879:13>' And_Pred_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11879'
--# expr-start 8037 '<And at nodes.lkt:11880:17>' And_Pred_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11880'
--# expr-start 8029 '<And at nodes.lkt:11881:21>' And_Pred_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11881'
--# expr-start 8021 '<Bind at nodes.lkt:11881:21>' Bind_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11881'
--# expr-start 8019 '<FieldAccess for expected_type_var at nodes.lkt:11881:40>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11881'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Op.Node);
--# expr-done 8019
Fld_54.Value := No_Entity;
Entity_Vars.Reset (Fld_54);
--# expr-start 8020 '<FieldAccess for type_var at nodes.lkt:11881:25>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11881'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8020
Fld_53.Value := No_Entity;
Entity_Vars.Reset (Fld_53);
Logic_Ctx_8 := Allocate_Logic_Context (Logic_Context); 
Bind_Result_12 := Solver.Create_Propagate (Fld_54, Fld_53, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Logic_Ctx_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11881:21") else null)); 
--# expr-done 8021
--# expr-start 8028 '<Or at nodes.lkt:11882:25>' Or_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
--# expr-start 8024 '<Bind at nodes.lkt:11882:25>' Bind_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
--# expr-start 8022 '<FieldAccess for type_var at nodes.lkt:11882:29>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8022
Fld_55.Value := No_Entity;
Entity_Vars.Reset (Fld_55);
--# expr-start 8023 '<FieldAccess for expected_type_var at nodes.lkt:11882:44>' Fld_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_56 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 8023
Fld_56.Value := No_Entity;
Entity_Vars.Reset (Fld_56);
Bind_Result_13 := Solver.Create_Unify (Fld_55, Fld_56, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11882:25") else null)); 
--# expr-done 8024
--# expr-start 8027 '<FieldAccess for comp_bind at nodes.lkt:11882:73>' Fld_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8025 '<FieldAccess for type_var at nodes.lkt:11882:88>' Fld_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_57 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8025
--# expr-start 8026 '<FieldAccess for expected_type_var at nodes.lkt:11882:103>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11882'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_58 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Left.Node);
--# expr-done 8026
--# property-call-start AdaNode.comp_bind
Fld_59 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_57, Right => Fld_58, Origin => Origin);
--# end
--# expr-done 8027
Or_Pred_5 := Create_Or (Bind_Result_13, Fld_59, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11882:25") else null)); 
--# expr-done 8028
And_Pred_16 := Create_And (Bind_Result_12, Or_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11881:21") else null)); 
--# expr-done 8029
--# expr-start 8036 '<Or at nodes.lkt:11885:21>' Or_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
--# expr-start 8032 '<Bind at nodes.lkt:11885:21>' Bind_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
--# expr-start 8030 '<FieldAccess for type_var at nodes.lkt:11885:25>' Fld_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_60 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8030
Fld_60.Value := No_Entity;
Entity_Vars.Reset (Fld_60);
--# expr-start 8031 '<FieldAccess for expected_type_var at nodes.lkt:11885:40>' Fld_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_61 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 8031
Fld_61.Value := No_Entity;
Entity_Vars.Reset (Fld_61);
Bind_Result_14 := Solver.Create_Unify (Fld_60, Fld_61, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11885:21") else null)); 
--# expr-done 8032
--# expr-start 8035 '<FieldAccess for comp_bind at nodes.lkt:11885:70>' Fld_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8033 '<FieldAccess for type_var at nodes.lkt:11885:85>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
      if Op.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_62 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Op.Node);
--# expr-done 8033
--# expr-start 8034 '<FieldAccess for expected_type_var at nodes.lkt:11885:100>' Fld_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11885'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_63 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Right.Node);
--# expr-done 8034
--# property-call-start AdaNode.comp_bind
Fld_64 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_62, Right => Fld_63, Origin => Origin);
--# end
--# expr-done 8035
Or_Pred_6 := Create_Or (Bind_Result_14, Fld_64, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11885:21") else null)); 
--# expr-done 8036
And_Pred_17 := Create_And (And_Pred_16, Or_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11880:17") else null)); 
--# expr-done 8037
--# expr-start 8038 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11887:19>' Fld_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11887'
      if Left.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_65 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Left.Node, Origin => Origin);
--# end
--# expr-done 8038
And_Pred_18 := Create_And (And_Pred_17, Fld_65, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11879:13") else null)); 
--# expr-done 8039
--# expr-start 8040 '<FieldAccess for matches_expected_formal_type at nodes.lkt:11888:15>' Fld_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11888'
      if Right.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_66 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Right.Node, Origin => Origin);
--# end
--# expr-done 8040
And_Pred_19 := Create_And (And_Pred_18, Fld_66, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11878:14") else null)); 
--# expr-done 8041
   If_Result := And_Pred_19;
end if;
      Inc_Ref (If_Result);
--# expr-done 8042
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8043
   If_Result_3 := If_Result_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 8044
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 8045
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 8046
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1139;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1139;
            raise;
      end;
   return Property_Result;
end Concat_Op_P_Array_Concat_Expected_Type_Equation;
--# end
   --
   --  Primitives for Bare_Concat_Operand
   --
      procedure Initialize_Fields_For_Concat_Operand
        (Self : Bare_Concat_Operand
         ; Concat_Operand_F_Operator : Bare_Op_Concat
         ; Concat_Operand_F_Operand : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Concat_Operand_F_Operator := Concat_Operand_F_Operator;
            Self.Concat_Operand_F_Operand := Concat_Operand_F_Operand;
      end Initialize_Fields_For_Concat_Operand;
   function Concat_Operand_F_Operator
     (Node : Bare_Concat_Operand) return Bare_Op_Concat
   is
   begin
         return Node.Concat_Operand_F_Operator;
   end;
   function Concat_Operand_F_Operand
     (Node : Bare_Concat_Operand) return Bare_Expr
   is
   begin
         return Node.Concat_Operand_F_Operand;
   end;
--# property-start ConcatOperand.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11900'
pragma Warnings (Off, "is not referenced");
 function Concat_Operand_P_Has_Context_Free_Type
  (Node : Bare_Concat_Operand
  )
   return Boolean
is
   Self : Bare_Concat_Operand  := Bare_Concat_Operand (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Concat_Operand_P_Has_Context_Free_Type;
--# end
   --
   --  Primitives for Bare_Cond_Expr
   --
      procedure Initialize_Fields_For_Cond_Expr
        (Self : Bare_Cond_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Cond_Expr;
--# property-start '[dispatcher]CondExpr.dependent_exprs' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Cond_Expr_P_Dependent_Exprs
  (Node : Bare_Cond_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Cond_Expr  := Bare_Cond_Expr (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Expr_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Cond_Expr (Self.Kind) is
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.dependent_exprs
                  Property_Result := Case_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.dependent_exprs
                  Property_Result := If_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Cond_Expr_P_Dependent_Exprs;
--# end
   --
   --  Primitives for Bare_Case_Expr
   --
      procedure Initialize_Fields_For_Case_Expr
        (Self : Bare_Case_Expr
         ; Case_Expr_F_Expr : Bare_Expr
         ; Case_Expr_F_Cases : Bare_Case_Expr_Alternative_List
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.Case_Expr_F_Expr := Case_Expr_F_Expr;
            Self.Case_Expr_F_Cases := Case_Expr_F_Cases;
      end Initialize_Fields_For_Case_Expr;
   function Case_Expr_F_Expr
     (Node : Bare_Case_Expr) return Bare_Expr
   is
   begin
         return Node.Case_Expr_F_Expr;
   end;
   function Case_Expr_F_Cases
     (Node : Bare_Case_Expr) return Bare_Case_Expr_Alternative_List
   is
   begin
         return Node.Case_Expr_F_Cases;
   end;
--# property-start CaseExpr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11917'
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Has_Context_Free_Type
  (Node : Bare_Case_Expr
  )
   return Boolean
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Has_Context_Free_Type;
--# end
--# property-start CaseExpr.dependent_exprs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11919'
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Dependent_Exprs
  (Node : Bare_Case_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
     Ent : Internal_Entity_Case_Expr :=
       Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1143 with Inline_Always;
      Fld : Internal_Entity_Case_Expr_Alternative_List;
Fld_1 : Bare_Case_Expr_Alternative_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_0 : Bare_Case_Expr_Alternative;
Fld_3 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Map_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1143 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1143;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8049 '<Map at nodes.lkt:11919:50>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11919'
   --# expr-start 8047 '<FieldAccess for cases at nodes.lkt:11919:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11919'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 8047
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind c Item_0
      --# expr-start 8048 '<FieldAccess for expr at nodes.lkt:11919:72>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11919'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8048
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_3;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8049
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1143;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1143;
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Dependent_Exprs;
--# end
--# property-start CaseExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11922'
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Xref_Equation
  (Node : Bare_Case_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Expr  := Bare_Case_Expr (Node);
     Ent : Internal_Entity_Case_Expr :=
       Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1144 with Inline_Always;
            procedure Finalizer_Scope_3048 with Inline_Always;
            procedure Finalizer_Scope_3049 with Inline_Always;
            procedure Finalizer_Scope_3050 with Inline_Always;
            procedure Finalizer_Scope_3051 with Inline_Always;
            procedure Finalizer_Scope_3052 with Inline_Always;
      Ignored : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Expr;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Pred : Logic_Equation;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Case_Expr_Alternative_List;
Fld_5 : Bare_Case_Expr_Alternative_List;
Fld_6 : Internal_Entity_Info;
Item_0 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_0 : Bare_Case_Expr_Alternative;
Fld_7 : Internal_Entity_Alternatives_List;
Fld_8 : Bare_Alternatives_List;
Fld_9 : Internal_Entity_Info;
Item_1 : Internal_Entity;
Bare_Item_1 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_10 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Expr;
Fld_13 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_15 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_A : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_16 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_17 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Logic_Var;
Fld_32 : Internal_Entity_Expr;
Fld_33 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Var;
Fld_36 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_7 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Or_Pred : Logic_Equation;
True_Rel_2 : Logic_Equation;
Fld_37 : Internal_Entity_Expr;
Fld_38 : Boolean;
If_Result_3 : Logic_Equation;
And_Pred_8 : Logic_Equation;
As_Entity_1 : Internal_Entity_Case_Expr_Alternative;
Is_Null_2 : Boolean;
If_Result_4 : Internal_Entity_Case_Expr_Alternative;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1144 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1144;
            procedure Finalizer_Scope_3048 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_25);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_3048;
            procedure Finalizer_Scope_3049 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3049;
            procedure Finalizer_Scope_3050 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_3050;
            procedure Finalizer_Scope_3051 is
            begin
                     Dec_Ref (Fld_17);
            end Finalizer_Scope_3051;
            procedure Finalizer_Scope_3052 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_3052;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8114 '<Let at nodes.lkt:11925:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11925'
--# scope-start
--# expr-start 8054 '<FieldAccess for resolve_names_internal_with_eq at nodes.lkt:11925:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11925'
--# expr-start 8050 '<FieldAccess for expr at nodes.lkt:11925:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11925'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Expr_F_Expr, Info => Ent.Info);
--# expr-done 8050
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8053 '<Predicate on BaseTypeDecl.is_discrete_type at nodes.lkt:11926:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11926'
--# expr-start 8052 '<FieldAccess for type_var at nodes.lkt:11926:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11926'
--# expr-start 8051 '<FieldAccess for expr at nodes.lkt:11926:55>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11926'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Case_Expr_F_Expr;
--# expr-done 8051
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 8052
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Pred := Solver.Create_Predicate (Fld_2, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 8053
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8054
Ignored := Fld_3; 
--# bind _ Ignored
--# expr-start 8113 '<LogicAll at nodes.lkt:11929:9>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11929'
--# expr-start 8112 '<Map at nodes.lkt:11929:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11929'
   --# expr-start 8055 '<FieldAccess for cases at nodes.lkt:11929:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11929'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 8055
Fld_5 := Fld_4.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_5;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_2 := Bare_Item_0 = null; 
if Is_Null_2 then
   If_Result_4 := No_Entity_Case_Expr_Alternative;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_0); 
   If_Result_4 := As_Entity_1;
end if;
                  Item_0 := If_Result_4; 
   --# scope-start
               --# bind alt Item_0
      --# expr-start 8111 '<And at nodes.lkt:11930:22>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11930'
--# expr-start 8092 '<And at nodes.lkt:11931:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11931'
--# expr-start 8089 '<And at nodes.lkt:11932:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11932'
--# expr-start 8086 '<And at nodes.lkt:11933:25>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11933'
--# expr-start 8081 '<And at nodes.lkt:11934:29>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11934'
--# expr-start 8076 '<LogicAll at nodes.lkt:11934:29>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11934'
--# expr-start 8075 '<Map at nodes.lkt:11934:29>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11934'
   --# expr-start 8056 '<FieldAccess for choices at nodes.lkt:11934:29>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11934'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Alternatives_List (Node => Item_0.Node.Case_Expr_Alternative_F_Choices, Info => Item_0.Info);
--# expr-done 8056
Fld_8 := Fld_7.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_8 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_8;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Bare_Item_1); 
   If_Result_2 := As_Entity;
end if;
                  Item_1 := If_Result_2; 
   --# scope-start
               --# bind c Item_1
      --# expr-start 8074 '<Match at nodes.lkt:11935:40>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11935'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 8072 '<If at nodes.lkt:11937:53>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8057 '<IsA Name at nodes.lkt:11937:56>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
Is_A := Match_0.Node /= null 
and then Match_0.Node.Kind in Ada_Name; 
--# expr-done 8057
if Is_A then
   --# expr-start 8061 '<Not at nodes.lkt:11937:70>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8060 '<IsNull at nodes.lkt:11937:74>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8059 '<FieldAccess for name_designated_type at nodes.lkt:11937:74>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8058 '<Cast to Name at nodes.lkt:11937:74>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
Cast_Expr_1 := Match_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 8058
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_16 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8059
Is_Null := Fld_16.Node = null; 
--# expr-done 8060
Not_Val := not (Is_Null); 
--# expr-done 8061
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 8063 '<FieldAccess for xref_type_equation at nodes.lkt:11937:121>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8062 '<Cast to Name at nodes.lkt:11937:121>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 8062
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_10 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result.Info);
--# end
--# expr-done 8063
   If_Result_1 := Fld_10;
else
   --# expr-start 8071 '<And at nodes.lkt:11937:158>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11937'
--# expr-start 8069 '<And at nodes.lkt:11938:41>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
--# expr-start 8067 '<Bind at nodes.lkt:11938:41>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
--# expr-start 8064 '<FieldAccess for expected_type_var at nodes.lkt:11938:45>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 8064
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 8066 '<FieldAccess for type_val at nodes.lkt:11938:68>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
--# expr-start 8065 '<FieldAccess for expr at nodes.lkt:11938:68>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Case_Expr_F_Expr;
--# expr-done 8065
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_13 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_12);
--# end
--# expr-done 8066
Bind_Result := Solver.Create_Assign (Fld_11, Fld_13, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11938:41") else null)); 
--# expr-done 8067
--# expr-start 8068 '<FieldAccess for sub_equation at nodes.lkt:11938:94>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11938'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 8068
And_Pred := Create_And (Bind_Result, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11938:41") else null)); 
--# expr-done 8069
--# expr-start 8070 '<FieldAccess for matches_expected_type at nodes.lkt:11939:43>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11939'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_15 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 8070
And_Pred_1 := Create_And (And_Pred, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11937:158") else null)); 
--# expr-done 8071
   If_Result_1 := And_Pred_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8072
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3050;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 8073 '<FieldAccess for xref_equation at nodes.lkt:11942:66>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11942'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_17 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 8073
Let_Result_1 := Fld_17; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3051;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11943:65") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3052;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8074
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3049;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8075
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11934:29") else null)); 
--# expr-done 8076
--# expr-start 8080 '<Bind at nodes.lkt:11948:33>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11948'
--# expr-start 8077 '<FieldAccess for type_var at nodes.lkt:11948:37>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11948'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8077
Fld_18.Value := No_Entity;
Entity_Vars.Reset (Fld_18);
--# expr-start 8079 '<FieldAccess for expected_type_var at nodes.lkt:11948:54>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11948'
--# expr-start 8078 '<FieldAccess for expr at nodes.lkt:11948:54>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11948'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8078
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_19.Node);
--# expr-done 8079
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Bind_Result_1 := Solver.Create_Unify (Fld_18, Fld_20, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11948:33") else null)); 
--# expr-done 8080
And_Pred_2 := Create_And (Logic_Boolean_Op, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11934:29") else null)); 
--# expr-done 8081
--# expr-start 8085 '<Bind at nodes.lkt:11950:31>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11950'
--# expr-start 8082 '<FieldAccess for expected_type_var at nodes.lkt:11950:65>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11950'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8082
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 8084 '<FieldAccess for expected_type_var at nodes.lkt:11950:35>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11950'
--# expr-start 8083 '<FieldAccess for expr at nodes.lkt:11950:35>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11950'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8083
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_21.Node);
--# expr-done 8084
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
Bind_Result_2 := Solver.Create_Propagate (Fld_23, Fld_22, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11950:31") else null)); 
--# expr-done 8085
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11933:25") else null)); 
--# expr-done 8086
--# expr-start 8088 '<FieldAccess for sub_equation at nodes.lkt:11951:27>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11951'
--# expr-start 8087 '<FieldAccess for expr at nodes.lkt:11951:27>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11951'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8087
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_25 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_24.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_24.Info);
--# end
--# expr-done 8088
And_Pred_4 := Create_And (And_Pred_3, Fld_25, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11932:21") else null)); 
--# expr-done 8089
--# expr-start 8091 '<FieldAccess for matches_expected_type at nodes.lkt:11952:23>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11952'
--# expr-start 8090 '<FieldAccess for expr at nodes.lkt:11952:23>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11952'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8090
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_27 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_26.Node, Origin => Origin);
--# end
--# expr-done 8091
And_Pred_5 := Create_And (And_Pred_4, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11931:17") else null)); 
--# expr-done 8092
--# expr-start 8110 '<If at nodes.lkt:11954:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11954'
--# expr-start 8094 '<FieldAccess for has_context_free_type at nodes.lkt:11954:20>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11954'
--# expr-start 8093 '<FieldAccess for expr at nodes.lkt:11954:20>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11954'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8093
      if Fld_37.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_38 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld_37.Node);
--# end
--# expr-done 8094
if Fld_38 then
   --# expr-start 8109 '<Or at nodes.lkt:11954:58>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11954'
--# expr-start 8108 '<And at nodes.lkt:11955:21>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11955'
--# expr-start 8103 '<And at nodes.lkt:11956:25>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8097 '<Predicate on BaseTypeDecl.is_not_universal_type at nodes.lkt:11956:25>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8096 '<FieldAccess for type_var at nodes.lkt:11956:72>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8095 '<FieldAccess for expr at nodes.lkt:11956:72>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8095
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_28.Node);
--# expr-done 8096
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
Pred_1 := Solver.Create_Predicate (Fld_29, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 8097
--# expr-start 8102 '<Bind at nodes.lkt:11956:97>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8099 '<FieldAccess for type_var at nodes.lkt:11956:131>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8098 '<FieldAccess for expr at nodes.lkt:11956:131>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8098
      if Fld_32.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_32.Node);
--# expr-done 8099
Fld_33.Value := No_Entity;
Entity_Vars.Reset (Fld_33);
--# expr-start 8101 '<FieldAccess for expected_type_var at nodes.lkt:11956:101>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
--# expr-start 8100 '<FieldAccess for expr at nodes.lkt:11956:101>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11956'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8100
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_30.Node);
--# expr-done 8101
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
Bind_Result_3 := Solver.Create_Propagate (Fld_33, Fld_31, Logic_Functor_841'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11956:97") else null)); 
--# expr-done 8102
And_Pred_6 := Create_And (Pred_1, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11956:25") else null)); 
--# expr-done 8103
--# expr-start 8107 '<Bind at nodes.lkt:11957:27>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11957'
--# expr-start 8105 '<FieldAccess for expected_type_var at nodes.lkt:11957:31>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11957'
--# expr-start 8104 '<FieldAccess for expr at nodes.lkt:11957:31>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11957'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Item_0.Node.Case_Expr_Alternative_F_Expr, Info => Item_0.Info);
--# expr-done 8104
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_34.Node);
--# expr-done 8105
Fld_35.Value := No_Entity;
Entity_Vars.Reset (Fld_35);
--# expr-start 8106 '<FieldAccess for expected_type_var at nodes.lkt:11957:61>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11957'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8106
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
Bind_Result_4 := Solver.Create_Unify (Fld_35, Fld_36, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11957:27") else null)); 
--# expr-done 8107
And_Pred_7 := Create_And (And_Pred_6, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11955:21") else null)); 
--# expr-done 8108
True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11958:22") else null)); 
Or_Pred := Create_Or (And_Pred_7, True_Rel_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11954:58") else null)); 
--# expr-done 8109
   If_Result_3 := Or_Pred;
else
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11958:33") else null)); 
   If_Result_3 := True_Rel_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 8110
And_Pred_8 := Create_And (And_Pred_5, If_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11930:22") else null)); 
--# expr-done 8111
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_8;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3048;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 8112
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11929:9") else null)); 
--# expr-done 8113
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8114
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1144;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1144;
                     Finalizer_Scope_3048;
                     Finalizer_Scope_3049;
                     Finalizer_Scope_3050;
                     Finalizer_Scope_3051;
                     Finalizer_Scope_3052;
            raise;
      end;
   return Property_Result;
end Case_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_If_Expr
   --
      procedure Initialize_Fields_For_If_Expr
        (Self : Bare_If_Expr
         ; If_Expr_F_Cond_Expr : Bare_Expr
         ; If_Expr_F_Then_Expr : Bare_Expr
         ; If_Expr_F_Alternatives : Bare_Elsif_Expr_Part_List
         ; If_Expr_F_Else_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.If_Expr_F_Cond_Expr := If_Expr_F_Cond_Expr;
            Self.If_Expr_F_Then_Expr := If_Expr_F_Then_Expr;
            Self.If_Expr_F_Alternatives := If_Expr_F_Alternatives;
            Self.If_Expr_F_Else_Expr := If_Expr_F_Else_Expr;
      end Initialize_Fields_For_If_Expr;
   function If_Expr_F_Cond_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Cond_Expr;
   end;
   function If_Expr_F_Then_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Then_Expr;
   end;
   function If_Expr_F_Alternatives
     (Node : Bare_If_Expr) return Bare_Elsif_Expr_Part_List
   is
   begin
         return Node.If_Expr_F_Alternatives;
   end;
   function If_Expr_F_Else_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Else_Expr;
   end;
--# property-start IfExpr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11971'
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Has_Context_Free_Type
  (Node : Bare_If_Expr
  )
   return Boolean
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end If_Expr_P_Has_Context_Free_Type;
--# end
--# property-start IfExpr.dependent_exprs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11973'
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Dependent_Exprs
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1146 with Inline_Always;
            procedure Finalizer_Scope_3055 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Array_Lit : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Elsif_Expr_Part_List;
Fld_2 : Bare_Elsif_Expr_Part_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_0 : Bare_Elsif_Expr_Part;
Fld_4 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Array_Lit_1 : Internal_Entity_Expr_Array_Access;
Scope_Result : Internal_Entity_Expr_Array_Access;
Result_Var : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1146 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1146;
            procedure Finalizer_Scope_3055 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_3055;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8116 '<ArrayLiteral at nodes.lkt:11974:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
--# expr-start 8115 '<FieldAccess for then_expr at nodes.lkt:11974:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 8115
Array_Lit := Create_Internal_Entity_Expr_Array (Internal_Internal_Entity_Expr_Array'(1 => Fld)); 
--# expr-done 8116
--# expr-start 8119 '<Map at nodes.lkt:11974:28>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
   --# expr-start 8117 '<FieldAccess for alternatives at nodes.lkt:11974:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 8117
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind a Item_0
      --# expr-start 8118 '<FieldAccess for then_expr at nodes.lkt:11974:57>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_0.Info);
--# expr-done 8118
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8119
Concat_Result := Concat (Array_Lit, Map_Result); 
--# expr-start 8120 '<FieldAccess for else_expr at nodes.lkt:11974:72>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 8120
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 8121 '<ArrayLiteral at nodes.lkt:11974:98>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11974'
Array_Lit_1 := Create_Internal_Entity_Expr_Array (Internal_Internal_Entity_Expr_Array'(1 => Var_Expr_1)); 
--# expr-done 8121
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3055;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result_1 := Concat (Concat_Result, Result_Var); 
         Property_Result := Concat_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1146;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1146;
                     Finalizer_Scope_3055;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Dependent_Exprs;
--# end
--# property-start IfExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11977'
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Xref_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1147 with Inline_Always;
            procedure Finalizer_Scope_3056 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_6 : Internal_Entity_Elsif_Expr_Part_List;
Fld_7 : Bare_Elsif_Expr_Part_List;
Fld_8 : Internal_Entity_Info;
Item_0 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_0 : Bare_Elsif_Expr_Part;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_3 : Logic_Equation;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Fld_19 : Bare_Expr;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_20 : Logic_Var;
Pred : Logic_Equation;
Fld_21 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_22 : Logic_Var;
Node_For_Entity : Bare_Base_Type_Decl;
As_Entity_1 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_8 : Logic_Equation;
            procedure Finalizer_Scope_1147 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_18);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_1147;
            procedure Finalizer_Scope_3056 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3056;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8161 '<And at nodes.lkt:11977:37>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11977'
--# expr-start 8150 '<And at nodes.lkt:11979:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11979'
--# expr-start 8141 '<And at nodes.lkt:11980:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11980'
--# expr-start 8129 '<And at nodes.lkt:11981:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11981'
--# expr-start 8126 '<And at nodes.lkt:11982:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11982'
--# expr-start 8123 '<FieldAccess for sub_equation at nodes.lkt:11982:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11982'
--# expr-start 8122 '<FieldAccess for cond_expr at nodes.lkt:11982:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11982'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 8122
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 8123
--# expr-start 8125 '<FieldAccess for expect_bool_derived_type at nodes.lkt:11982:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11982'
--# expr-start 8124 '<FieldAccess for cond_expr at nodes.lkt:11982:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11982'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 8124
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_3 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_2.Node, Origin => Origin);
--# end
--# expr-done 8125
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11982:21") else null)); 
--# expr-done 8126
--# expr-start 8128 '<FieldAccess for sub_equation at nodes.lkt:11985:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11985'
--# expr-start 8127 '<FieldAccess for then_expr at nodes.lkt:11985:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11985'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 8127
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 8128
And_Pred_1 := Create_And (And_Pred, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11981:17") else null)); 
--# expr-done 8129
--# expr-start 8140 '<LogicAll at nodes.lkt:11987:19>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11987'
--# expr-start 8139 '<Map at nodes.lkt:11987:19>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11987'
   --# expr-start 8130 '<FieldAccess for alternatives at nodes.lkt:11987:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 8130
Fld_7 := Fld_6.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_7;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind elsif Item_0
      --# expr-start 8138 '<And at nodes.lkt:11988:28>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11988'
--# expr-start 8135 '<And at nodes.lkt:11990:21>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11990'
--# expr-start 8132 '<FieldAccess for sub_equation at nodes.lkt:11990:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11990'
--# expr-start 8131 '<FieldAccess for cond_expr at nodes.lkt:11990:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11990'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_0.Info);
--# expr-done 8131
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_9.Info);
--# end
--# expr-done 8132
--# expr-start 8134 '<FieldAccess for expect_bool_derived_type at nodes.lkt:11990:56>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11990'
--# expr-start 8133 '<FieldAccess for cond_expr at nodes.lkt:11990:56>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11990'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_0.Info);
--# expr-done 8133
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_12 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_11.Node, Origin => Origin);
--# end
--# expr-done 8134
And_Pred_2 := Create_And (Fld_10, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11990:21") else null)); 
--# expr-done 8135
--# expr-start 8137 '<FieldAccess for sub_equation at nodes.lkt:11991:23>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11991'
--# expr-start 8136 '<FieldAccess for then_expr at nodes.lkt:11991:23>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11991'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_0.Info);
--# expr-done 8136
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_13.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_13.Info);
--# end
--# expr-done 8137
And_Pred_3 := Create_And (And_Pred_2, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11988:28") else null)); 
--# expr-done 8138
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3056;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8139
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11987:19") else null)); 
--# expr-done 8140
And_Pred_4 := Create_And (And_Pred_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11980:13") else null)); 
--# expr-done 8141
--# expr-start 8149 '<If at nodes.lkt:11994:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11994'
--# expr-start 8144 '<Not at nodes.lkt:11994:16>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11994'
--# expr-start 8143 '<IsNull at nodes.lkt:11994:20>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11994'
--# expr-start 8142 '<FieldAccess for else_expr at nodes.lkt:11994:20>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11994'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Self.If_Expr_F_Else_Expr;
--# expr-done 8142
Is_Null_1 := Fld_19 = null; 
--# expr-done 8143
Not_Val := not (Is_Null_1); 
--# expr-done 8144
if Not_Val then
   --# expr-start 8146 '<FieldAccess for sub_equation at nodes.lkt:11996:17>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11996'
--# expr-start 8145 '<FieldAccess for else_expr at nodes.lkt:11996:17>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11996'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 8145
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_15.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_15.Info);
--# end
--# expr-done 8146
   If_Result_1 := Fld_16;
else
   --# expr-start 8148 '<FieldAccess for expect_bool_derived_type at nodes.lkt:11999:17>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11999'
--# expr-start 8147 '<FieldAccess for then_expr at nodes.lkt:11999:17>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:11999'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 8147
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_18 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_17.Node, Origin => Origin);
--# end
--# expr-done 8148
   If_Result_1 := Fld_18;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8149
And_Pred_5 := Create_And (And_Pred_4, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11979:9") else null)); 
--# expr-done 8150
--# expr-start 8160 '<Or at nodes.lkt:12003:9>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12003'
--# expr-start 8154 '<And at nodes.lkt:12004:13>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12004'
--# expr-start 8152 '<Predicate on AdaNode.is_not_null at nodes.lkt:12004:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12004'
--# expr-start 8151 '<FieldAccess for expected_type_var at nodes.lkt:12004:45>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12004'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8151
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Pred := Solver.Create_Predicate (Fld_20, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 8152
--# expr-start 8153 '<FieldAccess for expected_type_equation at nodes.lkt:12004:75>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12004'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IfExpr.expected_type_equation
Fld_21 := Libadalang.Implementation.If_Expr_P_Expected_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 8153
And_Pred_6 := Create_And (Pred, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12004:13") else null)); 
--# expr-done 8154
--# expr-start 8159 '<And at nodes.lkt:12006:13>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12006'
--# expr-start 8157 '<Bind at nodes.lkt:12006:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12006'
--# expr-start 8155 '<FieldAccess for expected_type_var at nodes.lkt:12006:17>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12006'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8155
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
--# expr-start 8156 '<No BaseTypeDecl at nodes.lkt:12006:43>' Node_For_Entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12006'
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 8156
Is_Null_2 := Node_For_Entity = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Type_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Node_For_Entity); 
   If_Result_2 := As_Entity_1;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
Bind_Result := Solver.Create_Assign (Fld_22, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12006:13") else null)); 
--# expr-done 8157
--# expr-start 8158 '<FieldAccess for no_expected_type_equation at nodes.lkt:12006:67>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12006'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IfExpr.no_expected_type_equation
Fld_23 := Libadalang.Implementation.If_Expr_P_No_Expected_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 8158
And_Pred_7 := Create_And (Bind_Result, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12006:13") else null)); 
--# expr-done 8159
Or_Pred := Create_Or (And_Pred_6, And_Pred_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12003:9") else null)); 
--# expr-done 8160
And_Pred_8 := Create_And (And_Pred_5, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:11977:37") else null)); 
--# expr-done 8161
         Property_Result := And_Pred_8;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1147;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1147;
                     Finalizer_Scope_3056;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Xref_Equation;
--# end
--# property-start IfExpr.expected_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12014'
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Expected_Type_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1148 with Inline_Always;
            procedure Finalizer_Scope_3057 with Inline_Always;
            procedure Finalizer_Scope_3059 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Bare_Expr;
Fld_10 : Logic_Var;
Fld_11 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Equation;
And_Pred_3 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_16 : Bare_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_17 : Internal_Entity_Elsif_Expr_Part_List;
Fld_18 : Bare_Elsif_Expr_Part_List;
Fld_19 : Internal_Entity_Info;
Item_0 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_0 : Bare_Elsif_Expr_Part;
Fld_20 : Logic_Var;
Fld_21 : Internal_Entity_Expr;
Fld_22 : Logic_Var;
Bind_Result_4 : Logic_Equation;
Fld_23 : Internal_Entity_Expr;
Fld_24 : Logic_Var;
Fld_25 : Logic_Var;
Bind_Result_5 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_26 : Internal_Entity_Expr;
Fld_27 : Logic_Equation;
And_Pred_6 : Logic_Equation;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_28 : Internal_Entity_Expr_Array_Access;
Fld_29 : Boolean;
Map_Result_1 : Internal_Entity_Expr_Array_Access;
Fld_30 : Logic_Var;
Pred : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Logic_Var;
Bind_Result_6 : Logic_Equation;
And_Pred_8 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
And_Pred_9 : Logic_Equation;
            procedure Finalizer_Scope_1148 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_1148;
            procedure Finalizer_Scope_3057 is
            begin
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_3057;
            procedure Finalizer_Scope_3059 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3059;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8219 '<And at nodes.lkt:12014:46>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12014'
--# expr-start 8206 '<And at nodes.lkt:12015:9>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12015'
--# expr-start 8190 '<And at nodes.lkt:12016:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12016'
--# expr-start 8165 '<Bind at nodes.lkt:12016:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12016'
--# expr-start 8162 '<FieldAccess for type_var at nodes.lkt:12016:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12016'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8162
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 8164 '<FieldAccess for expected_type_var at nodes.lkt:12016:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12016'
--# expr-start 8163 '<FieldAccess for then_expr at nodes.lkt:12016:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12016'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.If_Expr_F_Then_Expr;
--# expr-done 8163
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_1);
--# expr-done 8164
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12016:13") else null)); 
--# expr-done 8165
--# expr-start 8189 '<If at nodes.lkt:12017:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12017'
--# expr-start 8168 '<Not at nodes.lkt:12017:20>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12017'
--# expr-start 8167 '<IsNull at nodes.lkt:12017:24>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12017'
--# expr-start 8166 '<FieldAccess for else_expr at nodes.lkt:12017:24>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12017'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.If_Expr_F_Else_Expr;
--# expr-done 8166
Is_Null := Fld_16 = null; 
--# expr-done 8167
Not_Val := not (Is_Null); 
--# expr-done 8168
if Not_Val then
   --# expr-start 8188 '<And at nodes.lkt:12017:52>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12017'
--# expr-start 8185 '<And at nodes.lkt:12018:21>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12018'
--# expr-start 8182 '<And at nodes.lkt:12019:25>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12019'
--# expr-start 8177 '<And at nodes.lkt:12020:29>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
--# expr-start 8172 '<Bind at nodes.lkt:12020:29>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
--# expr-start 8169 '<FieldAccess for type_var at nodes.lkt:12020:33>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8169
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 8171 '<FieldAccess for expected_type_var at nodes.lkt:12020:50>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
--# expr-start 8170 '<FieldAccess for else_expr at nodes.lkt:12020:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.If_Expr_F_Else_Expr;
--# expr-done 8170
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 8171
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_1 := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12020:29") else null)); 
--# expr-done 8172
--# expr-start 8176 '<Bind at nodes.lkt:12020:90>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
--# expr-start 8173 '<FieldAccess for expected_type_var at nodes.lkt:12020:130>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8173
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 8175 '<FieldAccess for expected_type_var at nodes.lkt:12020:94>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
--# expr-start 8174 '<FieldAccess for else_expr at nodes.lkt:12020:94>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.If_Expr_F_Else_Expr;
--# expr-done 8174
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_6);
--# expr-done 8175
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_2 := Solver.Create_Propagate (Fld_8, Fld_7, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12020:90") else null)); 
--# expr-done 8176
And_Pred := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12020:29") else null)); 
--# expr-done 8177
--# expr-start 8181 '<Bind at nodes.lkt:12021:31>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12021'
--# expr-start 8178 '<FieldAccess for expected_type_var at nodes.lkt:12021:71>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12021'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8178
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 8180 '<FieldAccess for expected_type_var at nodes.lkt:12021:35>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12021'
--# expr-start 8179 '<FieldAccess for then_expr at nodes.lkt:12021:35>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12021'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.If_Expr_F_Then_Expr;
--# expr-done 8179
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9);
--# expr-done 8180
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Bind_Result_3 := Solver.Create_Propagate (Fld_11, Fld_10, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12021:31") else null)); 
--# expr-done 8181
And_Pred_1 := Create_And (And_Pred, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12019:25") else null)); 
--# expr-done 8182
--# expr-start 8184 '<FieldAccess for matches_expected_formal_type at nodes.lkt:12022:27>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12022'
--# expr-start 8183 '<FieldAccess for else_expr at nodes.lkt:12022:27>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12022'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 8183
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 8184
And_Pred_2 := Create_And (And_Pred_1, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12018:21") else null)); 
--# expr-done 8185
--# expr-start 8187 '<FieldAccess for matches_expected_formal_type at nodes.lkt:12023:23>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12023'
--# expr-start 8186 '<FieldAccess for then_expr at nodes.lkt:12023:23>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12023'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 8186
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_15 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_14.Node, Origin => Origin);
--# end
--# expr-done 8187
And_Pred_3 := Create_And (And_Pred_2, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12017:52") else null)); 
--# expr-done 8188
   If_Result := And_Pred_3;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12023:74") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 8189
And_Pred_4 := Create_And (Bind_Result, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12016:13") else null)); 
--# expr-done 8190
--# expr-start 8205 '<LogicAll at nodes.lkt:12025:15>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12025'
--# expr-start 8204 '<Map at nodes.lkt:12025:15>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12025'
   --# expr-start 8191 '<FieldAccess for alternatives at nodes.lkt:12025:15>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12025'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 8191
Fld_18 := Fld_17.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_18 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_18;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Elsif_Expr_Part;
else
Fld_19 := Fld_17.Info;
As_Entity := (Info => Fld_19, Node => Bare_Item_0); 
   If_Result_1 := As_Entity;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind elsif Item_0
      --# expr-start 8203 '<And at nodes.lkt:12026:24>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12026'
--# expr-start 8200 '<And at nodes.lkt:12027:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
--# expr-start 8195 '<Bind at nodes.lkt:12027:17>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
--# expr-start 8192 '<FieldAccess for type_var at nodes.lkt:12027:21>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8192
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
--# expr-start 8194 '<FieldAccess for expected_type_var at nodes.lkt:12027:38>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
--# expr-start 8193 '<FieldAccess for then_expr at nodes.lkt:12027:38>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_0.Info);
--# expr-done 8193
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_21.Node);
--# expr-done 8194
Fld_22.Value := No_Entity;
Entity_Vars.Reset (Fld_22);
Bind_Result_4 := Solver.Create_Unify (Fld_20, Fld_22, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12027:17") else null)); 
--# expr-done 8195
--# expr-start 8199 '<Bind at nodes.lkt:12027:79>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
--# expr-start 8196 '<FieldAccess for expected_type_var at nodes.lkt:12027:120>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8196
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 8198 '<FieldAccess for expected_type_var at nodes.lkt:12027:83>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
--# expr-start 8197 '<FieldAccess for then_expr at nodes.lkt:12027:83>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12027'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_0.Info);
--# expr-done 8197
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_23.Node);
--# expr-done 8198
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
Bind_Result_5 := Solver.Create_Propagate (Fld_25, Fld_24, Logic_Functor_866'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12027:79") else null)); 
--# expr-done 8199
And_Pred_5 := Create_And (Bind_Result_4, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12027:17") else null)); 
--# expr-done 8200
--# expr-start 8202 '<FieldAccess for matches_expected_formal_type at nodes.lkt:12028:19>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12028'
--# expr-start 8201 '<FieldAccess for then_expr at nodes.lkt:12028:19>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12028'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Expr (Node => Item_0.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_0.Info);
--# expr-done 8201
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_27 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_26.Node, Origin => Origin);
--# end
--# expr-done 8202
And_Pred_6 := Create_And (And_Pred_5, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12026:24") else null)); 
--# expr-done 8203
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3057;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8204
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12025:15") else null)); 
--# expr-done 8205
And_Pred_7 := Create_And (And_Pred_4, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12015:9") else null)); 
--# expr-done 8206
--# expr-start 8218 '<LogicAll at nodes.lkt:12030:11>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12030'
--# expr-start 8217 '<Map at nodes.lkt:12030:11>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12030'
   --# expr-start 8209 '<Map at nodes.lkt:12030:11>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12030'
   --# expr-start 8207 '<FieldAccess for dependent_exprs at nodes.lkt:12030:11>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12030'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_28 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8207
      declare
         Map_Result_1_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_28;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
         --# expr-start 8208 '<FieldAccess for has_context_free_type at nodes.lkt:12030:48>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12030'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_29 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Item_1.Node);
--# end
--# expr-done 8208
         if Fld_29 then
         declare
            Item_To_Append : constant Internal_Entity_Expr := Item_1;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 8209
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Map_Result_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_2
      --# expr-start 8216 '<Or at nodes.lkt:12031:16>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12031'
--# expr-start 8215 '<And at nodes.lkt:12032:13>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
--# expr-start 8211 '<Predicate on BaseTypeDecl.is_not_universal_type at nodes.lkt:12032:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
--# expr-start 8210 '<FieldAccess for type_var at nodes.lkt:12032:60>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_2.Node);
--# expr-done 8210
Fld_30.Value := No_Entity;
Entity_Vars.Reset (Fld_30);
Pred := Solver.Create_Predicate (Fld_30, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate); 
--# expr-done 8211
--# expr-start 8214 '<Bind at nodes.lkt:12032:78>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
--# expr-start 8212 '<FieldAccess for type_var at nodes.lkt:12032:105>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_2.Node);
--# expr-done 8212
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
--# expr-start 8213 '<FieldAccess for expected_type_var at nodes.lkt:12032:82>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12032'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Item_2.Node);
--# expr-done 8213
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
Bind_Result_6 := Solver.Create_Propagate (Fld_32, Fld_31, Logic_Functor_841'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12032:78") else null)); 
--# expr-done 8214
And_Pred_8 := Create_And (Pred, Bind_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12032:13") else null)); 
--# expr-done 8215
True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12033:14") else null)); 
Or_Pred := Create_Or (And_Pred_8, True_Rel_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12031:16") else null)); 
--# expr-done 8216
         declare
            Item_To_Append : constant Logic_Equation := Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3059;
         end loop;
      end;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 8217
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12030:11") else null)); 
--# expr-done 8218
And_Pred_9 := Create_And (And_Pred_7, Logic_Boolean_Op_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12014:46") else null)); 
--# expr-done 8219
         Property_Result := And_Pred_9;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1148;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1148;
                     Finalizer_Scope_3057;
                     Finalizer_Scope_3059;
            raise;
      end;
   return Property_Result;
end If_Expr_P_Expected_Type_Equation;
--# end
--# property-start IfExpr.no_expected_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12041'
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_No_Expected_Type_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr  := Bare_If_Expr (Node);
     Ent : Internal_Entity_If_Expr :=
       Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1149 with Inline_Always;
            procedure Finalizer_Scope_3061 with Inline_Always;
      Fld : Internal_Entity_Expr_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Expr_Array_Access;
Fld_2 : Logic_Var;
Pred : Logic_Equation;
Fld_3 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1149 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1149;
            procedure Finalizer_Scope_3061 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3061;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8233 '<LogicAll at nodes.lkt:12042:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12042'
--# expr-start 8232 '<Map at nodes.lkt:12042:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12042'
   --# expr-start 8222 '<Map at nodes.lkt:12042:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12042'
   --# expr-start 8220 '<FieldAccess for dependent_exprs at nodes.lkt:12042:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12042'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8220
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
         --# expr-start 8221 '<FieldAccess for has_context_free_type at nodes.lkt:12042:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12042'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Item_0.Node);
--# end
--# expr-done 8221
         if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity_Expr := Item_0;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8222
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
      --# expr-start 8231 '<Or at nodes.lkt:12043:20>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12043'
--# expr-start 8224 '<Predicate on BaseTypeDecl.is_universal_type at nodes.lkt:12043:20>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12043'
--# expr-start 8223 '<FieldAccess for type_var at nodes.lkt:12043:63>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12043'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_1.Node);
--# expr-done 8223
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Pred := Solver.Create_Predicate (Fld_2, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate); 
--# expr-done 8224
--# expr-start 8230 '<And at nodes.lkt:12044:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
--# expr-start 8226 '<Predicate on AdaNode.is_not_null at nodes.lkt:12044:17>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
--# expr-start 8225 '<FieldAccess for type_var at nodes.lkt:12044:49>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_1.Node);
--# expr-done 8225
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred_1 := Solver.Create_Predicate (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 8226
--# expr-start 8229 '<Bind at nodes.lkt:12044:67>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
--# expr-start 8227 '<FieldAccess for type_var at nodes.lkt:12044:88>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_1.Node);
--# expr-done 8227
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 8228 '<FieldAccess for type_var at nodes.lkt:12044:71>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12044'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8228
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Propagate (Fld_5, Fld_4, Logic_Functor_841'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12044:67") else null)); 
--# expr-done 8229
And_Pred := Create_And (Pred_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12044:17") else null)); 
--# expr-done 8230
Or_Pred := Create_Or (Pred, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12043:20") else null)); 
--# expr-done 8231
         declare
            Item_To_Append : constant Logic_Equation := Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3061;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 8232
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12042:9") else null)); 
--# expr-done 8233
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1149;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1149;
                     Finalizer_Scope_3061;
            raise;
      end;
   return Property_Result;
end If_Expr_P_No_Expected_Type_Equation;
--# end
   --
   --  Primitives for Bare_Contract_Cases
   --
      procedure Initialize_Fields_For_Contract_Cases
        (Self : Bare_Contract_Cases
         ; Contract_Cases_F_Contract_Cases : Bare_Contract_Case_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Contract_Cases_F_Contract_Cases := Contract_Cases_F_Contract_Cases;
      end Initialize_Fields_For_Contract_Cases;
   function Contract_Cases_F_Contract_Cases
     (Node : Bare_Contract_Cases) return Bare_Contract_Case_Assoc_List
   is
   begin
         return Node.Contract_Cases_F_Contract_Cases;
   end;
   --
   --  Primitives for Bare_Decl_Expr
   --
      procedure Decl_Expr_Pre_Env_Actions
        (Self            : Bare_Decl_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_215 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Decl_Expr
        (Self : Bare_Decl_Expr
         ; Decl_Expr_F_Decls : Bare_Ada_Node_List
         ; Decl_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Decl_Expr_F_Decls := Decl_Expr_F_Decls;
            Self.Decl_Expr_F_Expr := Decl_Expr_F_Expr;
      end Initialize_Fields_For_Decl_Expr;
   function Decl_Expr_F_Decls
     (Node : Bare_Decl_Expr) return Bare_Ada_Node_List
   is
   begin
         return Node.Decl_Expr_F_Decls;
   end;
   function Decl_Expr_F_Expr
     (Node : Bare_Decl_Expr) return Bare_Expr
   is
   begin
         return Node.Decl_Expr_F_Expr;
   end;
--# property-start DeclExpr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12063'
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Has_Context_Free_Type
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8235 '<FieldAccess for has_context_free_type at nodes.lkt:12063:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12063'
--# expr-start 8234 '<FieldAccess for expr at nodes.lkt:12063:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12063'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Decl_Expr_F_Expr;
--# expr-done 8234
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld);
--# end
--# expr-done 8235
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Decl_Expr_P_Has_Context_Free_Type;
--# end
--# property-start DeclExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12066'
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Xref_Equation
  (Node : Bare_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
     Ent : Internal_Entity_Decl_Expr :=
       Internal_Entity_Decl_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1151 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1151 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1151;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8249 '<And at nodes.lkt:12066:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12066'
--# expr-start 8244 '<And at nodes.lkt:12066:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12066'
--# expr-start 8239 '<bind at nodes.lkt:12067:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12067'
--# expr-start 8236 '<FieldAccess for children_env at nodes.lkt:12067:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12067'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8236
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8238 '<FieldAccess for sub_equation at nodes.lkt:12069:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12069'
--# expr-start 8237 '<FieldAccess for expr at nodes.lkt:12069:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12069'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Decl_Expr_F_Expr, Info => Ent.Info);
--# expr-done 8237
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 8238
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8239
--# expr-start 8243 '<Bind at nodes.lkt:12070:11>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
--# expr-start 8241 '<FieldAccess for expected_type_var at nodes.lkt:12070:15>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
--# expr-start 8240 '<FieldAccess for expr at nodes.lkt:12070:15>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Decl_Expr_F_Expr;
--# expr-done 8240
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 8241
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 8242 '<FieldAccess for expected_type_var at nodes.lkt:12070:46>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8242
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_4, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12070:11") else null)); 
--# expr-done 8243
And_Pred := Create_And (Dyn_Var_Bind_Result, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12066:37") else null)); 
--# expr-done 8244
--# expr-start 8248 '<Bind at nodes.lkt:12070:76>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
--# expr-start 8246 '<FieldAccess for type_var at nodes.lkt:12070:80>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
--# expr-start 8245 '<FieldAccess for expr at nodes.lkt:12070:80>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Decl_Expr_F_Expr;
--# expr-done 8245
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 8246
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 8247 '<FieldAccess for type_var at nodes.lkt:12070:102>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12070'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8247
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Unify (Fld_7, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12070:76") else null)); 
--# expr-done 8248
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12066:37") else null)); 
--# expr-done 8249
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1151;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1151;
            raise;
      end;
   return Property_Result;
end Decl_Expr_P_Xref_Equation;
--# end
--# property-start DeclExpr._env_trans_parent_215 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_215
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr  := Bare_Decl_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_215;
--# end
   --
   --  Primitives for Bare_Format_String_Literal
   --
      procedure Initialize_Fields_For_Format_String_Literal
        (Self : Bare_Format_String_Literal
         ; Format_String_Literal_F_Opening_Chunk : Bare_Format_String_Tok_Start
         ; Format_String_Literal_F_Mid_Exprs : Bare_Format_String_Chunk_List
         ; Format_String_Literal_F_Trailing_Expr : Bare_Format_String_Chunk
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Format_String_Literal_F_Opening_Chunk := Format_String_Literal_F_Opening_Chunk;
            Self.Format_String_Literal_F_Mid_Exprs := Format_String_Literal_F_Mid_Exprs;
            Self.Format_String_Literal_F_Trailing_Expr := Format_String_Literal_F_Trailing_Expr;
      end Initialize_Fields_For_Format_String_Literal;
   function Format_String_Literal_F_Opening_Chunk
     (Node : Bare_Format_String_Literal) return Bare_Format_String_Tok_Start
   is
   begin
         return Node.Format_String_Literal_F_Opening_Chunk;
   end;
   function Format_String_Literal_F_Mid_Exprs
     (Node : Bare_Format_String_Literal) return Bare_Format_String_Chunk_List
   is
   begin
         return Node.Format_String_Literal_F_Mid_Exprs;
   end;
   function Format_String_Literal_F_Trailing_Expr
     (Node : Bare_Format_String_Literal) return Bare_Format_String_Chunk
   is
   begin
         return Node.Format_String_Literal_F_Trailing_Expr;
   end;
--# property-start FormatStringLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12089'
pragma Warnings (Off, "is not referenced");
 function Format_String_Literal_P_Xref_Equation
  (Node : Bare_Format_String_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Format_String_Literal  := Bare_Format_String_Literal (Node);
     Ent : Internal_Entity_Format_String_Literal :=
       Internal_Entity_Format_String_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1152 with Inline_Always;
            procedure Finalizer_Scope_3062 with Inline_Always;
            procedure Finalizer_Scope_3063 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Format_String_Chunk;
Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Internal_Entity_Format_String_Chunk;
Fld_4 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_5 : Internal_Entity_Format_String_Chunk_List;
Fld_6 : Bare_Format_String_Chunk_List;
Fld_7 : Internal_Entity_Info;
Item_1 : Internal_Entity_Format_String_Chunk;
Bare_Item_1 : Bare_Format_String_Chunk;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
As_Entity : Internal_Entity_Format_String_Chunk;
Is_Null : Boolean;
If_Result : Internal_Entity_Format_String_Chunk;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1152 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1152;
            procedure Finalizer_Scope_3062 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3062;
            procedure Finalizer_Scope_3063 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_3063;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8264 '<And at nodes.lkt:12089:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12089'
--# expr-start 8258 '<And at nodes.lkt:12090:10>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12090'
--# expr-start 8255 '<And at nodes.lkt:12090:10>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12090'
--# expr-start 8251 '<Predicate on BaseTypeDecl.allows_string_literal at nodes.lkt:12090:10>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12090'
--# expr-start 8250 '<FieldAccess for expected_type_var at nodes.lkt:12092:14>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12092'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8250
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Allows_String_Literal_0_Predicate (Origin, Self)); 
--# expr-done 8251
--# expr-start 8254 '<Bind at nodes.lkt:12094:16>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12094'
--# expr-start 8252 '<FieldAccess for expected_type_var at nodes.lkt:12094:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 8252
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 8253 '<FieldAccess for type_var at nodes.lkt:12094:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8253
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12094:16") else null)); 
--# expr-done 8254
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12090:10") else null)); 
--# expr-done 8255
--# expr-start 8256 '<FieldAccess for trailing_expr at nodes.lkt:12095:14>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12095'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Format_String_Chunk (Node => Ent.Node.Format_String_Literal_F_Trailing_Expr, Info => Ent.Info);
--# expr-done 8256
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Format_String_Chunk then
   --# scope-start
   --# bind te Var_Expr_0
--# expr-start 8257 '<FieldAccess for sub_equation at nodes.lkt:12096:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12096'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8257
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3062;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12097:25") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12090:10") else null)); 
--# expr-done 8258
--# expr-start 8263 '<LogicAll at nodes.lkt:12099:11>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12099'
--# expr-start 8262 '<Map at nodes.lkt:12099:11>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12099'
   --# expr-start 8259 '<FieldAccess for mid_exprs at nodes.lkt:12099:11>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12099'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Format_String_Chunk_List (Node => Ent.Node.Format_String_Literal_F_Mid_Exprs, Info => Ent.Info);
--# expr-done 8259
Fld_6 := Fld_5.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_6 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Format_String_Chunk_List := Fld_6;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Format_String_Chunk;
else
Fld_7 := Fld_5.Info;
As_Entity := (Info => Fld_7, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind m Item_1
      --# expr-start 8261 '<FieldAccess for sub_equation at nodes.lkt:12099:43>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12099'
--# expr-start 8260 '<FieldAccess for expr at nodes.lkt:12099:43>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12099'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Item_1.Node.Format_String_Chunk_F_Expr, Info => Item_1.Info);
--# expr-done 8260
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_8.Info);
--# end
--# expr-done 8261
         declare
            Item_To_Append : constant Logic_Equation := Fld_9;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3063;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8262
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12099:11") else null)); 
--# expr-done 8263
And_Pred_2 := Create_And (And_Pred_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12089:37") else null)); 
--# expr-done 8264
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1152;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1152;
                     Finalizer_Scope_3062;
                     Finalizer_Scope_3063;
            raise;
      end;
   return Property_Result;
end Format_String_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Membership_Expr
   --
      procedure Initialize_Fields_For_Membership_Expr
        (Self : Bare_Membership_Expr
         ; Membership_Expr_F_Expr : Bare_Expr
         ; Membership_Expr_F_Op : Bare_Op
         ; Membership_Expr_F_Membership_Exprs : Bare_Expr_Alternatives_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Membership_Expr_F_Expr := Membership_Expr_F_Expr;
            Self.Membership_Expr_F_Op := Membership_Expr_F_Op;
            Self.Membership_Expr_F_Membership_Exprs := Membership_Expr_F_Membership_Exprs;
      end Initialize_Fields_For_Membership_Expr;
   function Membership_Expr_F_Expr
     (Node : Bare_Membership_Expr) return Bare_Expr
   is
   begin
         return Node.Membership_Expr_F_Expr;
   end;
   function Membership_Expr_F_Op
     (Node : Bare_Membership_Expr) return Bare_Op
   is
   begin
         return Node.Membership_Expr_F_Op;
   end;
   function Membership_Expr_F_Membership_Exprs
     (Node : Bare_Membership_Expr) return Bare_Expr_Alternatives_List
   is
   begin
         return Node.Membership_Expr_F_Membership_Exprs;
   end;
--# property-start MembershipExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12112'
pragma Warnings (Off, "is not referenced");
 function Membership_Expr_P_Xref_Equation
  (Node : Bare_Membership_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Membership_Expr  := Bare_Membership_Expr (Node);
     Ent : Internal_Entity_Membership_Expr :=
       Internal_Entity_Membership_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1153 with Inline_Always;
            procedure Finalizer_Scope_3064 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_209 : Internal_Entity_Name;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr_Alternatives_List;
Fld_5 : Bare_Expr_Alternatives_List;
Fld_6 : Internal_Entity_Info;
Item_0 : Internal_Entity_Expr;
Bare_Item_0 : Bare_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Bare_Expr;
Fld_10 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Bare_Expr;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_16 : Bare_Expr;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Boolean;
If_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_19 : Logic_Var;
Fld_20 : Bare_Expr;
Fld_21 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_22 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
As_Entity : Internal_Entity_Expr;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Expr;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_6 : Logic_Equation;
            procedure Finalizer_Scope_1153 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_1153;
            procedure Finalizer_Scope_3064 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3064;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8307 '<And at nodes.lkt:12112:37>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12112'
--# expr-start 8270 '<And at nodes.lkt:12113:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
--# expr-start 8267 '<Bind at nodes.lkt:12113:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
--# expr-start 8265 '<FieldAccess for type_var at nodes.lkt:12113:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8265
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 8266 '<FieldAccess for bool_type at nodes.lkt:12113:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 8266
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12113:9") else null)); 
--# expr-done 8267
--# expr-start 8269 '<FieldAccess for sub_equation at nodes.lkt:12113:52>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
--# expr-start 8268 '<FieldAccess for expr at nodes.lkt:12113:52>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12113'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 8268
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 8269
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12113:9") else null)); 
--# expr-done 8270
--# expr-start 8306 '<LogicAll at nodes.lkt:12114:11>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12114'
--# expr-start 8305 '<Map at nodes.lkt:12114:11>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12114'
   --# expr-start 8271 '<FieldAccess for membership_exprs at nodes.lkt:12114:11>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12114'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr_Alternatives_List (Node => Ent.Node.Membership_Expr_F_Membership_Exprs, Info => Ent.Info);
--# expr-done 8271
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Expr;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_0); 
   If_Result_2 := As_Entity;
end if;
                  Item_0 := If_Result_2; 
   --# scope-start
               --# bind m Item_0
      --# expr-start 8304 '<Let at nodes.lkt:12116:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12116'
--# scope-start
--# expr-start 8272 '<Cast to Name at nodes.lkt:12116:23>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12116'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 8272
Var_Expr_209 := Cast_Result_1;
if Var_Expr_209 /= No_Entity_Name then
   --# scope-start
--# expr-start 8273 '<FieldAccess for name_designated_type at nodes.lkt:12116:23>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12116'
      if Var_Expr_209.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_7 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_209.Node, E_Info => Var_Expr_209.Info);
--# end
--# expr-done 8273
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Local_Typ := Result_Var; 
--# bind typ Local_Typ
--# expr-start 8303 '<If at nodes.lkt:12118:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12118'
--# expr-start 8275 '<Not at nodes.lkt:12118:16>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12118'
--# expr-start 8274 '<IsNull at nodes.lkt:12118:20>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12118'
Is_Null := Local_Typ.Node = null; 
--# expr-done 8274
Not_Val := not (Is_Null); 
--# expr-done 8275
if Not_Val then
   --# expr-start 8294 '<And at nodes.lkt:12120:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12120'
--# expr-start 8277 '<FieldAccess for xref_type_equation at nodes.lkt:12120:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12120'
--# expr-start 8276 '<Cast to Name at nodes.lkt:12120:17>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12120'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 8276
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_8 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 8277
--# expr-start 8293 '<If at nodes.lkt:12129:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8278 '<FieldAccess for is_tagged_type at nodes.lkt:12129:24>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 8278
if Fld_18 then
   --# expr-start 8285 '<And at nodes.lkt:12129:50>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8281 '<Bind at nodes.lkt:12129:50>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8280 '<FieldAccess for expected_type_var at nodes.lkt:12129:54>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8279 '<FieldAccess for expr at nodes.lkt:12129:54>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Membership_Expr_F_Expr;
--# expr-done 8279
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9);
--# expr-done 8280
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr_2 := No_Entity_Base_Type_Decl; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_10, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12129:50") else null)); 
--# expr-done 8281
--# expr-start 8284 '<Predicate on BaseTypeDecl.is_tagged_type_with_deref at nodes.lkt:12129:117>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8283 '<FieldAccess for type_var at nodes.lkt:12129:168>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
--# expr-start 8282 '<FieldAccess for expr at nodes.lkt:12129:168>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12129'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Membership_Expr_F_Expr;
--# expr-done 8282
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 8283
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Pred := Solver.Create_Predicate (Fld_12, Create_Base_Type_Decl_P_Is_Tagged_Type_With_Deref_0_Predicate (Origin, Self)); 
--# expr-done 8284
And_Pred_1 := Create_And (Bind_Result_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12129:50") else null)); 
--# expr-done 8285
   If_Result := And_Pred_1;
else
   --# expr-start 8292 '<And at nodes.lkt:12133:25>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
--# expr-start 8289 '<Bind at nodes.lkt:12133:25>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
--# expr-start 8287 '<FieldAccess for expected_type_var at nodes.lkt:12133:29>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
--# expr-start 8286 '<FieldAccess for expr at nodes.lkt:12133:29>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Membership_Expr_F_Expr;
--# expr-done 8286
      if Fld_13 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_13);
--# expr-done 8287
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 8288 '<FieldAccess for base_subtype at nodes.lkt:12133:60>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 8288
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_14, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12133:25") else null)); 
--# expr-done 8289
--# expr-start 8291 '<FieldAccess for matches_expected_membership_type at nodes.lkt:12133:84>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
--# expr-start 8290 '<FieldAccess for expr at nodes.lkt:12133:84>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12133'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Membership_Expr_F_Expr;
--# expr-done 8290
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_membership_type
Fld_17 := Libadalang.Implementation.Expr_P_Matches_Expected_Membership_Type (Node => Fld_16, Origin => Origin);
--# end
--# expr-done 8291
And_Pred_2 := Create_And (Bind_Result_2, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12133:25") else null)); 
--# expr-done 8292
   If_Result := And_Pred_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 8293
And_Pred_3 := Create_And (Fld_8, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12120:17") else null)); 
--# expr-done 8294
   If_Result_1 := And_Pred_3;
else
   --# expr-start 8302 '<And at nodes.lkt:12138:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
--# expr-start 8300 '<And at nodes.lkt:12138:17>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
--# expr-start 8298 '<Bind at nodes.lkt:12138:17>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
--# expr-start 8295 '<FieldAccess for expected_type_var at nodes.lkt:12138:21>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Item_0.Node);
--# expr-done 8295
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 8297 '<FieldAccess for type_var at nodes.lkt:12138:44>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
--# expr-start 8296 '<FieldAccess for expr at nodes.lkt:12138:44>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Self.Membership_Expr_F_Expr;
--# expr-done 8296
      if Fld_20 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20);
--# expr-done 8297
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Bind_Result_3 := Solver.Create_Unify (Fld_19, Fld_21, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12138:17") else null)); 
--# expr-done 8298
--# expr-start 8299 '<FieldAccess for sub_equation at nodes.lkt:12138:70>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 8299
And_Pred_4 := Create_And (Bind_Result_3, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12138:17") else null)); 
--# expr-done 8300
--# expr-start 8301 '<FieldAccess for matches_expected_type at nodes.lkt:12138:91>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12138'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Item_0.Node, Origin => Origin);
--# end
--# expr-done 8301
And_Pred_5 := Create_And (And_Pred_4, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12138:17") else null)); 
--# expr-done 8302
   If_Result_1 := And_Pred_5;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8303
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8304
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3064;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8305
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12114:11") else null)); 
--# expr-done 8306
And_Pred_6 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12112:37") else null)); 
--# expr-done 8307
         Property_Result := And_Pred_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1153;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1153;
                     Finalizer_Scope_3064;
            raise;
      end;
   return Property_Result;
end Membership_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Name
   --
      procedure Initialize_Fields_For_Name
        (Self : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Name;
--# property-start Name.enclosing_defining_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12150'
pragma Warnings (Off, "is not referenced");
 function Name_P_Enclosing_Defining_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_195 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_195 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_195;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8311 '<Cast to DefiningName at nodes.lkt:12151:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12151'
--# expr-start 8310 '<Find at nodes.lkt:12151:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12151'
   --# expr-start 8308 '<FieldAccess for parents at nodes.lkt:12151:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 8308
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 8309 '<IsA DefiningName at nodes.lkt:12151:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12151'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Defining_Name_Range; 
--# expr-done 8309
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 8310
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Defining_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Defining_Name
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result := No_Entity_Defining_Name;
   end if;
--# expr-done 8311
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_195;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_195;
            raise;
      end;
   return Property_Result;
end Name_P_Enclosing_Defining_Name;
--# end
--# property-start Name.is_defining 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12155'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Defining
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_210 : Bare_Dotted_Name;
Is_A : Boolean;
Fld : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Is_A_2 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_4 : Bare_Base_Id;
Scope_Result : Bare_Base_Id;
Result_Var : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 8312 '<IsA DefiningName at nodes.lkt:12157:9>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12157'
Is_A := Self /= null 
and then Self.Kind in Ada_Defining_Name_Range; 
--# expr-done 8312
if Is_A then
   If_Result := True;
else
   --# expr-start 8314 '<IsA DefiningName at nodes.lkt:12161:9>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12161'
--# expr-start 8313 '<parent at nodes.lkt:12161:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12161'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 8313
Is_A_1 := Fld /= null 
and then Fld.Kind in Ada_Defining_Name_Range; 
--# expr-done 8314
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 8317 '<IsA DefiningName at nodes.lkt:12165:9>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
--# expr-start 8316 '<parent at nodes.lkt:12165:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
--# expr-start 8315 '<parent at nodes.lkt:12165:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 8315
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 8316
Is_A_2 := Fld_2 /= null 
and then Fld_2.Kind in Ada_Defining_Name_Range; 
--# expr-done 8317
if Is_A_2 then
   --# expr-start 8321 '<Eq at nodes.lkt:12165:48>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
--# expr-start 8319 '<Cast to DottedName at nodes.lkt:12165:48>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
--# expr-start 8318 '<parent at nodes.lkt:12165:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 8318
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8319
Var_Expr_210 := Cast_Result;
if Var_Expr_210 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8320 '<FieldAccess for suffix at nodes.lkt:12165:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12165'
      if Var_Expr_210 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Var_Expr_210.Dotted_Name_F_Suffix;
--# expr-done 8320
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_1 := Result_Var;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 8321
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Defining;
--# end
--# property-start Name.name_is 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12170'
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Is
  (Node : Bare_Name
      ; Sym : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind sym Sym
   Property_Result : Boolean;
      Var_Expr_0 : Symbol_Type;
Fld : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 8322 '<FieldAccess for name_symbol at nodes.lkt:12170:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12170'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 8322
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Symbol then
   --# scope-start
   --# bind ns Var_Expr_0
--# expr-start 8323 '<Eq at nodes.lkt:12170:68>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12170'
Is_Equal := Var_Expr_0 = Sym; 
--# expr-done 8323
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Is;
--# end
--# property-start Name.is_direct_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12175'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Direct_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8324 '<FieldAccess for is_call at nodes.lkt:12176:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12176'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8324
if Fld then
   --# expr-start 8328 '<Not at nodes.lkt:12176:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12176'
--# expr-start 8327 '<IsA AccessToSubpDef at nodes.lkt:12176:32>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12176'
--# expr-start 8326 '<parent at nodes.lkt:12176:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12176'
--# expr-start 8325 '<FieldAccess for called_subp_spec at nodes.lkt:12176:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12176'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8325
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 8326
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 8327
Not_Val := not (Is_A); 
--# expr-done 8328
   If_Result := Not_Val;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Direct_Call;
--# end
--# property-start Name.is_access_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12181'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Access_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8329 '<FieldAccess for is_call at nodes.lkt:12182:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8329
if Fld then
   --# expr-start 8332 '<IsA AccessToSubpDef at nodes.lkt:12182:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12182'
--# expr-start 8331 '<parent at nodes.lkt:12182:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12182'
--# expr-start 8330 '<FieldAccess for called_subp_spec at nodes.lkt:12182:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8330
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 8331
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 8332
   If_Result := Is_A;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Access_Call;
--# end
--# property-start Name.is_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12186'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8334 '<Not at nodes.lkt:12187:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12187'
--# expr-start 8333 '<FieldAccess for is_defining at nodes.lkt:12187:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12187'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 8333
Not_Val := not (Fld); 
--# expr-done 8334
if Not_Val then
   --# expr-start 8337 '<Not at nodes.lkt:12187:36>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12187'
--# expr-start 8336 '<IsNull at nodes.lkt:12187:40>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12187'
--# expr-start 8335 '<FieldAccess for called_formal_subp_spec at nodes.lkt:12187:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12187'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_formal_subp_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Called_Formal_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8335
Is_Null := Fld_1.Node = null; 
--# expr-done 8336
Not_Val_1 := not (Is_Null); 
--# expr-done 8337
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Call;
--# end
--# property-start Name.is_dot_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12192'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dot_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8339 '<Not at nodes.lkt:12193:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
--# expr-start 8338 '<FieldAccess for is_defining at nodes.lkt:12193:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 8338
Not_Val := not (Fld); 
--# expr-done 8339
if Not_Val then
   --# expr-start 8343 '<FieldAccess for dottable_subp at nodes.lkt:12193:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
--# expr-start 8342 '<FieldAccess for md at nodes.lkt:12193:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
--# expr-start 8341 '<FieldAccess for info at nodes.lkt:12193:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
--# expr-start 8340 '<FieldAccess for referenced_decl at nodes.lkt:12193:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12193'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8340
Fld_2 := Fld_1.Info;
--# expr-done 8341
Fld_3 := Fld_2.Md;
--# expr-done 8342
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 8343
   If_Result := Fld_4;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Dot_Call;
--# end
--# property-start Name.failsafe_referenced_def_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12199'
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Def_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Def
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_3070 with Inline_Always;
      Local_Ref_Decl : Internal_Refd_Decl;
Local_Def_Name : Internal_Entity_Defining_Name;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Symbol_Type;
Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Fld_6 : Ref_Result_Kind;
New_Struct : Internal_Refd_Def;
Let_Result : Internal_Refd_Def;
Let_Result_1 : Internal_Refd_Def;
            procedure Finalizer_Scope_3070 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3070;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8354 '<Let at nodes.lkt:12200:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12200'
--# scope-start
--# expr-start 8344 '<FieldAccess for failsafe_referenced_decl at nodes.lkt:12200:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12200'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_decl
Fld := Libadalang.Implementation.Name_P_Failsafe_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8344
Local_Ref_Decl := Fld; 
--# bind ref_decl Local_Ref_Decl
--# expr-start 8353 '<Let at nodes.lkt:12201:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12201'
--# scope-start
--# expr-start 8345 '<FieldAccess for decl at nodes.lkt:12201:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12201'
Fld_1 := Local_Ref_Decl.Decl;
--# expr-done 8345
Var_Expr_0 := Fld_1;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr_0
--# expr-start 8346 '<FieldAccess for name_symbol at nodes.lkt:12202:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12202'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 8346
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Symbol then
   --# scope-start
   --# bind rel_name Var_Expr_1
--# expr-start 8349 '<Find at nodes.lkt:12203:31>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12203'
   --# expr-start 8347 '<FieldAccess for defining_names at nodes.lkt:12203:31>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12203'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8347
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_3;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_2
      --# expr-start 8348 '<FieldAccess for name_is at nodes.lkt:12203:70>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12203'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_2.Node, Sym => Var_Expr_1);
--# end
--# expr-done 8348
      if Fld_4 then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 8349
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_3070;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 8350 '<FieldAccess for defining_name at nodes.lkt:12204:19>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12204'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8350
   Result_Var_1 := Fld_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Local_Def_Name := Result_Var_2; 
--# bind def_name Local_Def_Name
--# expr-start 8352 '<New RefdDef at nodes.lkt:12207:9>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12207'
--# expr-start 8351 '<FieldAccess for kind at nodes.lkt:12207:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12207'
Fld_6 := Local_Ref_Decl.Kind;
--# expr-done 8351
New_Struct := (Def_Name => Local_Def_Name, Kind => Fld_6); 
--# expr-done 8352
Let_Result := New_Struct; 
--# end
--# expr-done 8353
Let_Result_1 := Let_Result; 
--# end
--# expr-done 8354
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3070;
            raise;
      end;
   return Property_Result;
end Name_P_Failsafe_Referenced_Def_Name;
--# end
--# property-start Name.referenced_defining_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12214'
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Defining_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_3074 with Inline_Always;
      Local_Ref_Decl : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Symbol_Type;
Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_3074 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3074;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8361 '<Let at nodes.lkt:12215:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12215'
--# scope-start
--# expr-start 8355 '<FieldAccess for referenced_decl at nodes.lkt:12215:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12215'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8355
Local_Ref_Decl := Fld; 
--# bind ref_decl Local_Ref_Decl
Var_Expr_0 := Local_Ref_Decl;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr_0
--# expr-start 8356 '<FieldAccess for name_symbol at nodes.lkt:12218:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12218'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 8356
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Symbol then
   --# scope-start
   --# bind rel_name Var_Expr_1
--# expr-start 8359 '<Find at nodes.lkt:12219:31>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12219'
   --# expr-start 8357 '<FieldAccess for defining_names at nodes.lkt:12219:31>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12219'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8357
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_2
      --# expr-start 8358 '<FieldAccess for name_is at nodes.lkt:12219:70>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12219'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_2.Node, Sym => Var_Expr_1);
--# end
--# expr-done 8358
      if Fld_3 then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 8359
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_3074;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 8360 '<FieldAccess for defining_name at nodes.lkt:12220:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12220'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8360
   Result_Var_1 := Fld_4;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 8361
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3074;
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Defining_Name;
--# end
--# property-start Name.gnat_xref_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12227'
pragma Warnings (Off, "is not referenced");
 function Name_P_Gnat_Xref_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Local_Dn : Internal_Entity_Defining_Name;
Var_Expr_0 : Boolean;
Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_211 : Internal_Entity_Basic_Decl;
Var_Expr_212 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_3 : Internal_Entity_Defining_Name;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Defining_Name;
Scope_Result_3 : Internal_Entity_Defining_Name;
Result_Var_3 : Internal_Entity_Defining_Name;
Fld_8 : Internal_Entity_Defining_Name;
Is_A : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result : Internal_Entity_Defining_Name;
Is_A_1 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
If_Result_2 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Cast_Expr_3 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result_3 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
If_Result_4 : Internal_Entity_Defining_Name;
Is_A_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
If_Result_5 : Internal_Entity_Defining_Name;
Is_A_5 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_9 : Boolean;
If_Result_6 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
Let_Result_1 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8389 '<Let at nodes.lkt:12228:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12228'
--# scope-start
--# expr-start 8362 '<FieldAccess for is_defining at nodes.lkt:12228:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12228'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 8362
Var_Expr_0 := Fld;
if Var_Expr_0 /= False then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 8363 '<FieldAccess for enclosing_defining_name at nodes.lkt:12228:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12228'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_1 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8363
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Local_Dn := Result_Var; 
--# bind dn Local_Dn
--# expr-start 8388 '<Let at nodes.lkt:12229:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12229'
--# scope-start
Var_Expr_1 := Local_Dn;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 8364 '<FieldAccess for basic_decl at nodes.lkt:12229:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12229'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8364
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Local_Bd := Result_Var_1; 
--# bind bd Local_Bd
--# expr-start 8387 '<If at nodes.lkt:12231:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_7 := Local_Bd;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_7
--# expr-start 8365 '<IsA ParamSpec at nodes.lkt:12231:26>' Is_A_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Is_A_5 := Var_Expr_7.Node /= null 
and then Var_Expr_7.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 8365
Scope_Result_9 := Is_A_5;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := False;
end if;
if Result_Var_9 then
   --# expr-start 8367 '<FieldAccess for decl_param at nodes.lkt:12231:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
--# expr-start 8366 '<Cast to ParamSpec at nodes.lkt:12231:48>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Cast_Expr := Local_Bd; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 8366
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result.Node, Param => Local_Dn, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 8367
   If_Result_6 := Fld_3;
else
   --# expr-start 8386 '<If at nodes.lkt:12231:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_6 := Local_Bd;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_6
--# expr-start 8368 '<IsA Body at nodes.lkt:12232:28>' Is_A_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12232'
Is_A_4 := Var_Expr_6.Node /= null 
and then Var_Expr_6.Node.Kind in Ada_Body_Node; 
--# expr-done 8368
Scope_Result_8 := Is_A_4;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
if Result_Var_8 then
--# expr-start 8370 '<FieldAccess for decl_part at nodes.lkt:12232:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12232'
--# expr-start 8369 '<Cast to Body at nodes.lkt:12232:45>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12232'
Cast_Expr_1 := Local_Bd; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 8369
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8370
Var_Expr_211 := Fld_4;
if Var_Expr_211 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 8371 '<FieldAccess for defining_name at nodes.lkt:12232:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12232'
      if Var_Expr_211.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_211.Node, E_Info => Var_Expr_211.Info);
--# end
--# expr-done 8371
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
   If_Result_5 := Result_Var_2;
else
   --# expr-start 8385 '<If at nodes.lkt:12231:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_5 := Local_Bd;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 8372 '<IsA BaseTypeDecl at nodes.lkt:12234:21>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12234'
Is_A_3 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 8372
if Is_A_3 then
   --# expr-start 8374 '<FieldAccess for is_in_private_part at nodes.lkt:12234:44>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12234'
--# expr-start 8373 '<Cast to BaseTypeDecl at nodes.lkt:12234:44>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12234'
Cast_Expr_3 := Var_Expr_5; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8373
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 8374
   If_Result_3 := Fld_9;
else
   If_Result_3 := False;
end if;
Scope_Result_7 := If_Result_3;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
--# expr-start 8376 '<FieldAccess for previous_part at nodes.lkt:12235:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12235'
--# expr-start 8375 '<Cast to BaseTypeDecl at nodes.lkt:12235:16>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12235'
Cast_Expr_2 := Local_Bd; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 8375
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Cast_Result_2.Node, Go_To_Incomplete => True, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 8376
Var_Expr_212 := Fld_6;
if Var_Expr_212 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8377 '<FieldAccess for defining_name at nodes.lkt:12235:16>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12235'
      if Var_Expr_212.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_212.Node, E_Info => Var_Expr_212.Info);
--# end
--# expr-done 8377
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Defining_Name;
end if;
   If_Result_4 := Result_Var_3;
else
   --# expr-start 8384 '<If at nodes.lkt:12231:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_4 := Local_Bd;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_4
--# expr-start 8378 '<IsA ObjectDecl at nodes.lkt:12236:28>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12236'
Is_A_2 := Var_Expr_4.Node /= null 
and then Var_Expr_4.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 8378
Scope_Result_6 := Is_A_2;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := False;
end if;
if Result_Var_6 then
   If_Result_2 := No_Entity_Defining_Name;
else
   --# expr-start 8383 '<If at nodes.lkt:12231:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_3 := Local_Bd;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 8379 '<IsA NumberDecl at nodes.lkt:12241:28>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12241'
Is_A_1 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Number_Decl_Range; 
--# expr-done 8379
Scope_Result_5 := Is_A_1;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   If_Result_1 := No_Entity_Defining_Name;
else
   --# expr-start 8382 '<If at nodes.lkt:12231:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12231'
Var_Expr_2 := Local_Bd;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 8380 '<IsA BasicDecl at nodes.lkt:12245:28>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12245'
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Basic_Decl; 
--# expr-done 8380
Scope_Result_4 := Is_A;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   If_Result := Local_Dn;
else
   --# expr-start 8381 '<FieldAccess for referenced_defining_name at nodes.lkt:12246:14>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12246'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8381
   If_Result := Fld_8;
end if;
--# expr-done 8382
   If_Result_1 := If_Result;
end if;
--# expr-done 8383
   If_Result_2 := If_Result_1;
end if;
--# expr-done 8384
   If_Result_4 := If_Result_2;
end if;
--# expr-done 8385
   If_Result_5 := If_Result_4;
end if;
--# expr-done 8386
   If_Result_6 := If_Result_5;
end if;
--# expr-done 8387
Let_Result := If_Result_6; 
--# end
--# expr-done 8388
Let_Result_1 := Let_Result; 
--# end
--# expr-done 8389
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Gnat_Xref_Decl;
--# end
--# property-start Name.all_env_elements_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12251'
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements_Internal
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_205 with Inline_Always;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_205 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_205;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8394 '<bind at nodes.lkt:12252:9>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12252'
--# expr-start 8390 '<FieldAccess for origin_node at nodes.lkt:12252:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8390
Bound_Origin := Fld; 
--# expr-start 8393 '<bind at nodes.lkt:12253:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12253'
--# expr-start 8391 '<FieldAccess for node_env at nodes.lkt:12253:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12253'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8391
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 8392 '<FieldAccess for all_env_els_impl at nodes.lkt:12255:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12255'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8392
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8393
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 8394
         Property_Result := Dyn_Var_Bind_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_205;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_205;
            raise;
      end;
   return Property_Result;
end Name_P_All_Env_Elements_Internal;
--# end
--# property-start Name.all_env_elements 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12262'
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_206 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_206 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_206;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8395 '<FieldAccess for all_env_elements_internal at nodes.lkt:12263:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12263'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, E_Info => Ent.Info);
--# end
--# expr-done 8395
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_206;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_206;
            raise;
      end;
   return Property_Result;
end Name_P_All_Env_Elements;
--# end
--# property-start Name.first_corresponding_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12265'
pragma Warnings (Off, "is not referenced");
 function Name_P_First_Corresponding_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_207 with Inline_Always;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_207 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_207;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8405 '<If at nodes.lkt:12266:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8397 '<IsA DottedName at nodes.lkt:12266:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8396 '<parent at nodes.lkt:12266:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 8396
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8397
if Is_A then
   --# expr-start 8398 '<FieldAccess for is_suffix at nodes.lkt:12266:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 8398
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 8401 '<FieldAccess for first_corresponding_decl at nodes.lkt:12266:64>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8400 '<Cast to DottedName at nodes.lkt:12266:64>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8399 '<parent at nodes.lkt:12266:64>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8399
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 8400
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8401
   If_Result_1 := Fld_1;
else
   --# expr-start 8404 '<Cast to BasicDecl at nodes.lkt:12266:123>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8403 '<at at nodes.lkt:12266:123>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
--# expr-start 8402 '<FieldAccess for all_env_elements_internal at nodes.lkt:12266:123>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12266'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8402
Get_Result := Get (Self, Fld_2, 0, True); 
--# expr-done 8403
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 8404
   If_Result_1 := Cast_Result_1;
end if;
--# expr-done 8405
         Property_Result := If_Result_1;
   --# end
      Finalizer_Scope_207;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_207;
            raise;
      end;
   return Property_Result;
end Name_P_First_Corresponding_Decl;
--# end
--# property-start Name.all_args_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12275'
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Args_Xref_Equation
  (Node : Bare_Name
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_208 with Inline_Always;
            procedure Finalizer_Scope_3087 with Inline_Always;
            procedure Finalizer_Scope_3088 with Inline_Always;
            procedure Finalizer_Scope_3089 with Inline_Always;
            procedure Finalizer_Scope_3090 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Call_Expr;
Var_Expr_2 : Internal_Entity_Name;
Var_Expr_213 : Internal_Entity_Assoc_List;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Item_1 : Internal_Entity_Basic_Assoc;
Bare_Item_1 : Bare_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Scope_Result : Logic_Equation_Array_Access;
Result_Var : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_2 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_208 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_208;
            procedure Finalizer_Scope_3087 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_3087;
            procedure Finalizer_Scope_3088 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3088;
            procedure Finalizer_Scope_3089 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3089;
            procedure Finalizer_Scope_3090 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3090;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8414 '<And at nodes.lkt:12275:56>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12275'
--# expr-start 8406 '<Cast to CallExpr at nodes.lkt:12275:56>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12275'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 8406
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_0
--# expr-start 8411 '<LogicAll at nodes.lkt:12276:17>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12276'
--# expr-start 8407 '<FieldAccess for params at nodes.lkt:12276:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12276'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8407
Var_Expr_213 := Fld;
if Var_Expr_213 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 8410 '<Map at nodes.lkt:12276:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12276'
Saved := Var_Expr_213; 
Fld_1 := Saved.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_1;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Saved.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind pa Item_1
      --# expr-start 8409 '<FieldAccess for sub_equation at nodes.lkt:12276:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12276'
--# expr-start 8408 '<FieldAccess for expr at nodes.lkt:12276:48>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12276'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 8408
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 8409
         declare
            Item_To_Append : constant Logic_Equation := Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3089;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8410
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3088;
   Result_Var := Scope_Result;
else
   Result_Var := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var);
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Result_Var.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12276:17") else null)); 
--# expr-done 8411
Scope_Result_1 := Logic_Boolean_Op;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3087;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12276:87") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 8412 '<FieldAccess for parent_name at nodes.lkt:12277:11>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12277'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_5 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 8412
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_2
--# expr-start 8413 '<FieldAccess for all_args_xref_equation at nodes.lkt:12278:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12278'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_6 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Var_Expr_2.Node, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 8413
Scope_Result_2 := Fld_6;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3090;
   Result_Var_2 := Scope_Result_2;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12278:66") else null)); 
   Result_Var_2 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_2);
And_Pred := Create_And (Result_Var_1, Result_Var_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12275:56") else null)); 
--# expr-done 8414
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_208;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_208;
                     Finalizer_Scope_3087;
                     Finalizer_Scope_3088;
                     Finalizer_Scope_3089;
                     Finalizer_Scope_3090;
            raise;
      end;
   return Property_Result;
end Name_P_All_Args_Xref_Equation;
--# end
--# property-start Name.undefined_reference_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12283'
pragma Warnings (Off, "is not referenced");
 function Name_P_Undefined_Reference_Equation
  (Node : Bare_Name
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_209 with Inline_Always;
      Fld : Logic_Var;
Bind_Result : Logic_Equation;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_209 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_209;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8419 '<And at nodes.lkt:12284:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12284'
--# expr-start 8416 '<Bind at nodes.lkt:12284:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12284'
--# expr-start 8415 '<FieldAccess for ref_var at nodes.lkt:12284:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12284'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8415
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Bind_Result := Solver.Create_Assign (Fld, No_Entity, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12284:9") else null)); 
--# expr-done 8416
--# expr-start 8418 '<Predicate on AdaNode.missing_entity_error at nodes.lkt:12284:56>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12284'
--# expr-start 8417 '<FieldAccess for ref_var at nodes.lkt:12284:97>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12284'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8417
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Pred := Solver.Create_Predicate (Fld_1, Create_Ada_Node_P_Missing_Entity_Error_0_Predicate (Self)); 
--# expr-done 8418
And_Pred := Create_And (Bind_Result, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12284:9") else null)); 
--# expr-done 8419
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_209;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_209;
            raise;
      end;
   return Property_Result;
end Name_P_Undefined_Reference_Equation;
--# end
--# property-start Name.bottom_up_name_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12287'
pragma Warnings (Off, "is not referenced");
 function Name_P_Bottom_Up_Name_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_210 with Inline_Always;
            procedure Finalizer_Scope_3091 with Inline_Always;
            procedure Finalizer_Scope_3092 with Inline_Always;
            procedure Finalizer_Scope_3093 with Inline_Always;
            procedure Finalizer_Scope_3094 with Inline_Always;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Explicit_Deref;
Match_2 : Internal_Entity_Qual_Expr;
Match_3 : Internal_Entity_Name;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Equation;
False_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Qual_Expr;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
            procedure Finalizer_Scope_210 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_210;
            procedure Finalizer_Scope_3091 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3091;
            procedure Finalizer_Scope_3092 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3092;
            procedure Finalizer_Scope_3093 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3093;
            procedure Finalizer_Scope_3094 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_3094;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8425 '<Match at nodes.lkt:12287:47>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12287'
--# expr-start 8421 '<as_entity at nodes.lkt:12287:53>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12287'
--# expr-start 8420 '<FieldAccess for innermost_name at nodes.lkt:12287:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12287'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.innermost_name
Fld := Libadalang.Implementation.Name_P_Innermost_Name (Node => Self);
--# end
--# expr-done 8420
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 8421
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := If_Result; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 8422 '<FieldAccess for general_xref_equation at nodes.lkt:12288:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12288'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.general_xref_equation
Fld_1 := Libadalang.Implementation.Call_Expr_P_General_Xref_Equation (Node => Match_0.Node, Root => Self, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 8422
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3091;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Explicit_Deref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 8423 '<FieldAccess for general_xref_equation at nodes.lkt:12289:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12289'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ExplicitDeref.general_xref_equation
Fld_2 := Libadalang.Implementation.Explicit_Deref_P_General_Xref_Equation (Node => Match_1.Node, Root => Self, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 8423
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3092;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind qe Match_2
--# expr-start 8424 '<FieldAccess for general_xref_equation at nodes.lkt:12290:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12290'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start QualExpr.general_xref_equation
Fld_3 := Libadalang.Implementation.Qual_Expr_P_General_Xref_Equation (Node => Match_2.Node, Root => Self, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 8424
Let_Result_2 := Fld_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3093;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Reduce_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12291:19") else null)); 
Let_Result_3 := False_Rel; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3094;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8425
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_210;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_210;
                     Finalizer_Scope_3091;
                     Finalizer_Scope_3092;
                     Finalizer_Scope_3093;
                     Finalizer_Scope_3094;
            raise;
      end;
   return Property_Result;
end Name_P_Bottom_Up_Name_Equation;
--# end
--# property-start Name.innermost_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12304'
pragma Warnings (Off, "is not referenced");
 function Name_P_Innermost_Name
  (Node : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Bare_Name;
      Local_Name : Bare_Name;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Name;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Call_Expr;
Let_Result : Bare_Name;
Scope_Result : Bare_Name;
Cast_Result_1 : Bare_Explicit_Deref;
Let_Result_1 : Bare_Name;
Scope_Result_1 : Bare_Name;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Bare_Name;
Scope_Result_2 : Bare_Name;
Match_Result : Bare_Name;
Fld_2 : Bare_Name;
Is_A : Boolean;
If_Result : Bare_Name;
Is_A_1 : Boolean;
If_Result_1 : Bare_Name;
Let_Result_3 : Bare_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8434 '<Let at nodes.lkt:12305:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12305'
--# scope-start
--# expr-start 8428 '<Match at nodes.lkt:12305:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12305'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 8426 '<FieldAccess for name at nodes.lkt:12306:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12306'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Call_Expr_F_Name;
--# expr-done 8426
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 8427 '<FieldAccess for prefix at nodes.lkt:12307:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12307'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Match_1.Explicit_Deref_F_Prefix;
--# expr-done 8427
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 8428
Local_Name := Match_Result; 
--# bind name Local_Name
--# expr-start 8433 '<If at nodes.lkt:12311:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12311'
--# expr-start 8429 '<IsA CallExpr, ExplicitDeref at nodes.lkt:12311:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12311'
Is_A_1 := Local_Name /= null 
and then Local_Name.Kind in Ada_Call_Expr_Range | Ada_Explicit_Deref_Range; 
--# expr-done 8429
if Is_A_1 then
   --# expr-start 8430 '<FieldAccess for innermost_name at nodes.lkt:12311:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12311'
      if Local_Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.innermost_name
Fld_2 := Libadalang.Implementation.Name_P_Innermost_Name (Node => Local_Name);
--# end
--# expr-done 8430
   If_Result_1 := Fld_2;
else
   --# expr-start 8432 '<If at nodes.lkt:12311:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12311'
--# expr-start 8431 '<IsA QualExpr at nodes.lkt:12312:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12312'
Is_A := Local_Name /= null 
and then Local_Name.Kind in Ada_Qual_Expr_Range; 
--# expr-done 8431
if Is_A then
   If_Result := Local_Name;
else
   If_Result := Self;
end if;
--# expr-done 8432
   If_Result_1 := If_Result;
end if;
--# expr-done 8433
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 8434
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Innermost_Name;
--# end
--# property-start Name.leftmost_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12335'
pragma Warnings (Off, "is not referenced");
 function Name_P_Leftmost_Name
  (Node : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Bare_Name;
      Match_0 : Bare_Call_Expr;
Match_1 : Bare_Array_Subcomponent_Choice_Name;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Attribute_Ref;
Match_4 : Bare_Name;
Var_Expr_214 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Name;
Fld_2 : Bare_Name;
Fld_3 : Bare_Name;
Scope_Result : Bare_Name;
Result_Var : Bare_Name;
Fld_4 : Bare_Name;
Fld_5 : Bare_Name;
Fld_6 : Bare_Name;
Fld_7 : Bare_Name;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Call_Expr;
Let_Result : Bare_Name;
Scope_Result_1 : Bare_Name;
Cast_Result_1 : Bare_Array_Subcomponent_Choice_Name;
Let_Result_1 : Bare_Name;
Scope_Result_2 : Bare_Name;
Cast_Result_2 : Bare_Dotted_Name;
Let_Result_2 : Bare_Name;
Scope_Result_3 : Bare_Name;
Cast_Result_3 : Bare_Attribute_Ref;
Let_Result_3 : Bare_Name;
Scope_Result_4 : Bare_Name;
Cast_Result_4 : Bare_Name;
Let_Result_4 : Bare_Name;
Scope_Result_5 : Bare_Name;
Match_Result : Bare_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8443 '<Match at nodes.lkt:12335:33>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12335'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 8436 '<FieldAccess for leftmost_name at nodes.lkt:12336:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12336'
--# expr-start 8435 '<FieldAccess for name at nodes.lkt:12336:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12336'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Call_Expr_F_Name;
--# expr-done 8435
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_1 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Fld);
--# end
--# expr-done 8436
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Subcomponent_Choice_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind ascn Match_1
--# expr-start 8437 '<FieldAccess for name at nodes.lkt:12338:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12338'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_1.Array_Subcomponent_Choice_Name_F_Name;
--# expr-done 8437
Var_Expr_214 := Fld_2;
if Var_Expr_214 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 8438 '<FieldAccess for leftmost_name at nodes.lkt:12338:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12338'
      if Var_Expr_214 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_3 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Var_Expr_214);
--# end
--# expr-done 8438
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind dn Match_2
--# expr-start 8440 '<FieldAccess for leftmost_name at nodes.lkt:12339:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12339'
--# expr-start 8439 '<FieldAccess for prefix at nodes.lkt:12339:32>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12339'
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 8439
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_5 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Fld_4);
--# end
--# expr-done 8440
Let_Result_2 := Fld_5; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind ar Match_3
--# expr-start 8442 '<FieldAccess for leftmost_name at nodes.lkt:12340:34>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12340'
--# expr-start 8441 '<FieldAccess for prefix at nodes.lkt:12340:34>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12340'
      if Match_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Match_3.Attribute_Ref_F_Prefix;
--# expr-done 8441
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.leftmost_name
Fld_7 := Libadalang.Implementation.Name_P_Leftmost_Name (Node => Fld_6);
--# end
--# expr-done 8442
Let_Result_3 := Fld_7; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := Self; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 8443
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Leftmost_Name;
--# end
--# property-start Name.parent_name_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12346'
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name_Equation
  (Node : Bare_Name
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_213 with Inline_Always;
            procedure Finalizer_Scope_3110 with Inline_Always;
            procedure Finalizer_Scope_3111 with Inline_Always;
            procedure Finalizer_Scope_3113 with Inline_Always;
            procedure Finalizer_Scope_3114 with Inline_Always;
      Local_As_Subp_Access : Internal_Entity_Access_To_Subp_Def;
Local_Is_Paramless : Boolean;
Local_Can_Be_Paramless : Boolean;
Local_Comp_Type : Internal_Entity_Base_Type_Decl;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Var_Expr_0 : Internal_Entity_Name;
Var_Expr_215 : Internal_Entity_Base_Type_Decl;
Var_Expr_216 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_217 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_218 : Internal_Entity_Base_Type_Decl;
Var_Expr_219 : Internal_Entity_Base_Type_Decl;
Var_Expr_220 : Internal_Entity_Base_Type_Decl;
Var_Expr_221 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_222 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Not_Val : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
False_Rel : Logic_Equation;
As_Entity : Internal_Entity_Call_Expr;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Call_Expr;
Fld_8 : Logic_Equation;
As_Entity_1 : Internal_Entity_Explicit_Deref;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Explicit_Deref;
Fld_9 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Internal_Entity_Subp_Spec;
Scope_Result_6 : Internal_Entity_Subp_Spec;
Result_Var_6 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Bare_Ada_Node;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Match_Prefix : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_3 : Bare_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_4 : Bare_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_14 : Internal_Entity_Name;
Fld_15 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_18 : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
Cast_Expr : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_21 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_7 : Boolean;
If_Result_6 : Logic_Equation;
Is_A_1 : Boolean;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Logic_Equation;
Scope_Result_11 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null_4 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Let_Result_6 : Logic_Equation;
            procedure Finalizer_Scope_213 is
            begin
                     Dec_Ref (False_Rel);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_213;
            procedure Finalizer_Scope_3110 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3110;
            procedure Finalizer_Scope_3111 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3111;
            procedure Finalizer_Scope_3113 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_3113;
            procedure Finalizer_Scope_3114 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_18);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_8);
            end Finalizer_Scope_3114;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8491 '<Let at nodes.lkt:12347:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12347'
--# scope-start
--# expr-start 8445 '<Cast to AccessToSubpDef at nodes.lkt:12347:30>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12347'
Var_Expr_215 := Typ;
if Var_Expr_215 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8444 '<FieldAccess for access_def at nodes.lkt:12347:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12347'
      if Var_Expr_215.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_215.Node, Origin => Origin, E_Info => Var_Expr_215.Info);
--# end
--# expr-done 8444
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 8445
Local_As_Subp_Access := Cast_Result; 
--# bind as_subp_access Local_As_Subp_Access
--# expr-start 8490 '<Let at nodes.lkt:12348:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12348'
--# scope-start
Var_Expr_216 := Local_As_Subp_Access;
if Var_Expr_216 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 8447 '<FieldAccess for paramless at nodes.lkt:12348:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12348'
--# expr-start 8446 '<FieldAccess for subp_spec at nodes.lkt:12348:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12348'
      if Var_Expr_216.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_216.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_216.Info);
--# expr-done 8446
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_1.Node, Dottable_Subp => False, Can_Be => False);
--# end
--# expr-done 8447
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Local_Is_Paramless := Result_Var_1; 
--# bind is_paramless Local_Is_Paramless
--# expr-start 8489 '<Let at nodes.lkt:12349:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12349'
--# scope-start
Var_Expr_217 := Local_As_Subp_Access;
if Var_Expr_217 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 8449 '<FieldAccess for paramless at nodes.lkt:12349:32>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12349'
--# expr-start 8448 '<FieldAccess for subp_spec at nodes.lkt:12349:32>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12349'
      if Var_Expr_217.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_217.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_217.Info);
--# expr-done 8448
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_3.Node, Dottable_Subp => False, Can_Be => True);
--# end
--# expr-done 8449
Scope_Result_2 := Fld_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Local_Can_Be_Paramless := Result_Var_2; 
--# bind can_be_paramless Local_Can_Be_Paramless
--# expr-start 8488 '<Let at nodes.lkt:12350:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12350'
--# scope-start
--# expr-start 8455 '<If at nodes.lkt:12353:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
Var_Expr_218 := Typ;
if Var_Expr_218 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8450 '<FieldAccess for is_iterable_type at nodes.lkt:12353:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
      if Var_Expr_218.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_218.Node, Origin => Origin, E_Info => Var_Expr_218.Info);
--# end
--# expr-done 8450
Scope_Result_5 := Fld_7;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
Var_Expr_219 := Typ;
if Var_Expr_219 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8451 '<FieldAccess for iterable_comp_type at nodes.lkt:12353:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
      if Var_Expr_219.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_219.Node, Origin => Origin, E_Info => Var_Expr_219.Info);
--# end
--# expr-done 8451
Scope_Result_3 := Fld_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
   If_Result := Result_Var_3;
else
Var_Expr_220 := Typ;
if Var_Expr_220 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8454 '<FieldAccess for comp_type at nodes.lkt:12353:72>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
      if Var_Expr_220.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8453 '<Not at nodes.lkt:12353:100>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
--# expr-start 8452 '<IsA ExplicitDeref at nodes.lkt:12353:104>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12353'
Is_A := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 8452
Not_Val := not (Is_A); 
--# expr-done 8453
--# property-call-start BaseTypeDecl.comp_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_220.Node, Is_Subscript => Not_Val, Origin => Origin, E_Info => Var_Expr_220.Info);
--# end
--# expr-done 8454
Scope_Result_4 := Fld_6;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
   If_Result := Result_Var_4;
end if;
--# expr-done 8455
Local_Comp_Type := If_Result; 
--# bind comp_type Local_Comp_Type
--# expr-start 8487 '<If at nodes.lkt:12355:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12355'
--# expr-start 8456 '<IsNull at nodes.lkt:12355:12>' Is_Null_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12355'
Is_Null_4 := Typ.Node = null; 
--# expr-done 8456
if Is_Null_4 then
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12355:29") else null)); 
   If_Result_9 := False_Rel;
else
   --# expr-start 8486 '<And at nodes.lkt:12355:41>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12355'
--# expr-start 8468 '<Match at nodes.lkt:12355:41>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12355'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind ce Match_0
--# expr-start 8458 '<FieldAccess for subscriptable_type_equation at nodes.lkt:12356:34>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12356'
--# expr-start 8457 '<as_entity at nodes.lkt:12356:34>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12356'
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Call_Expr;
else
As_Entity := (Info => E_Info, Node => Match_0); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 8457
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.subscriptable_type_equation
Fld_8 := Libadalang.Implementation.Call_Expr_P_Subscriptable_Type_Equation (Node => If_Result_1.Node, Typ => Typ, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result_1.Info);
--# end
--# expr-done 8458
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
Scope_Result_7 := Let_Result;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3110;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind ed Match_1
--# expr-start 8464 '<And at nodes.lkt:12357:39>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12357'
--# expr-start 8460 '<FieldAccess for eq_for_type at nodes.lkt:12357:39>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12357'
--# expr-start 8459 '<as_entity at nodes.lkt:12357:39>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12357'
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Explicit_Deref;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 8459
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ExplicitDeref.eq_for_type
Fld_9 := Libadalang.Implementation.Explicit_Deref_P_Eq_For_Type (Node => If_Result_2.Node, Typ => Typ, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 8460
--# expr-start 8463 '<Bind at nodes.lkt:12361:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12361'
--# expr-start 8461 '<FieldAccess for subp_spec_var at nodes.lkt:12361:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12361'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 8461
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Var_Expr_221 := Local_As_Subp_Access;
if Var_Expr_221 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 8462 '<FieldAccess for subp_spec at nodes.lkt:12361:43>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12361'
      if Var_Expr_221.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_221.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_221.Info);
--# expr-done 8462
Scope_Result_6 := Fld_11;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Subp_Spec;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
Bind_Result := Solver.Create_Assign (Fld_10, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12361:17") else null)); 
--# expr-done 8463
And_Pred := Create_And (Fld_9, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12357:39") else null)); 
--# expr-done 8464
Let_Result_1 := And_Pred; Inc_Ref (Let_Result_1);
--# end
Scope_Result_8 := Let_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3111;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 8467 '<Bind at nodes.lkt:12363:23>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12363'
--# expr-start 8465 '<FieldAccess for type_var at nodes.lkt:12363:27>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12363'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 8465
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 8466 '<FieldAccess for node at nodes.lkt:12363:44>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12363'
Fld_13 := No_Entity.Node;
--# expr-done 8466
Is_Null_2 := Fld_13 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Fld_13); 
   If_Result_3 := As_Entity_2;
end if;
Bind_Result_1 := Solver.Create_Assign (Fld_12, If_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12363:23") else null)); 
--# expr-done 8467
Let_Result_2 := Bind_Result_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_9 := Let_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_3113;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 8468
--# expr-start 8469 '<FieldAccess for parent_name at nodes.lkt:12364:15>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12364'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_14 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 8469
Var_Expr_0 := Fld_14;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_0
--# expr-start 8485 '<If at nodes.lkt:12365:21>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12365'
--# expr-start 8470 '<IsA ExplicitDeref at nodes.lkt:12365:24>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12365'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 8470
if Is_A_1 then
   --# expr-start 8472 '<Not at nodes.lkt:12365:50>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12365'
--# expr-start 8471 '<IsNull at nodes.lkt:12365:54>' Is_Null_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12365'
Is_Null_3 := Local_As_Subp_Access.Node = null; 
--# expr-done 8471
Not_Val_1 := not (Is_Null_3); 
--# expr-done 8472
   If_Result_7 := Not_Val_1;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
   --# expr-start 8479 '<If at nodes.lkt:12372:17>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12372'
if Local_Is_Paramless then
   --# expr-start 8473 '<FieldAccess for parent_name_equation at nodes.lkt:12372:38>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12372'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Local_Comp_Type, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8473
   If_Result_5 := Fld_15;
else
   --# expr-start 8478 '<If at nodes.lkt:12372:17>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12372'
if Local_Can_Be_Paramless then
   --# expr-start 8476 '<Or at nodes.lkt:12377:44>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12377'
--# expr-start 8474 '<FieldAccess for parent_name_equation at nodes.lkt:12377:44>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12377'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Local_Comp_Type, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8474
--# expr-start 8475 '<FieldAccess for parent_name_equation at nodes.lkt:12377:88>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12377'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_17 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8475
Or_Pred := Create_Or (Fld_16, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12377:44") else null)); 
--# expr-done 8476
   If_Result_4 := Or_Pred;
else
   --# expr-start 8477 '<FieldAccess for parent_name_equation at nodes.lkt:12381:22>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12381'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8477
   If_Result_4 := Fld_18;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 8478
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 8479
   If_Result_8 := If_Result_5;
else
   --# expr-start 8484 '<If at nodes.lkt:12365:21>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12365'
--# expr-start 8480 '<Cast to CallExpr at nodes.lkt:12384:18>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12384'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 8480
Var_Expr_222 := Cast_Result_5;
if Var_Expr_222 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8481 '<FieldAccess for check_array_slice at nodes.lkt:12384:18>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12384'
      if Var_Expr_222.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_21 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Var_Expr_222.Node, Typ => Typ, Origin => Origin, E_Info => Var_Expr_222.Info);
--# end
--# expr-done 8481
Scope_Result_10 := Fld_21;
   --# end
   Result_Var_7 := Scope_Result_10;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   --# expr-start 8482 '<FieldAccess for parent_name_equation at nodes.lkt:12384:65>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12384'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_19 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8482
   If_Result_6 := Fld_19;
else
   --# expr-start 8483 '<FieldAccess for parent_name_equation at nodes.lkt:12387:18>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12387'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_20 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Local_Comp_Type, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8483
   If_Result_6 := Fld_20;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 8484
   If_Result_8 := If_Result_6;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 8485
Scope_Result_11 := If_Result_8;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_3114;
   Result_Var_8 := Scope_Result_11;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12387:72") else null)); 
   Result_Var_8 := True_Rel;
end if;
      Inc_Ref (Result_Var_8);
And_Pred_1 := Create_And (Match_Result, Result_Var_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12355:41") else null)); 
--# expr-done 8486
   If_Result_9 := And_Pred_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 8487
Let_Result_3 := If_Result_9; Inc_Ref (Let_Result_3);
--# end
--# expr-done 8488
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8489
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 8490
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 8491
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_213;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_213;
                     Finalizer_Scope_3110;
                     Finalizer_Scope_3111;
                     Finalizer_Scope_3113;
                     Finalizer_Scope_3114;
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Name_Equation;
--# end
--# property-start Name.subtype_indication_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12392'
pragma Warnings (Off, "is not referenced");
 function Name_P_Subtype_Indication_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_214 with Inline_Always;
      Fld : Logic_Equation;
            procedure Finalizer_Scope_214 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_214;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8492 '<FieldAccess for xref_type_equation at nodes.lkt:12392:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12392'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 8492
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_214;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_214;
            raise;
      end;
   return Property_Result;
end Name_P_Subtype_Indication_Equation;
--# end
--# property-start Name.can_designate_primitive 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12400'
pragma Warnings (Off, "is not referenced");
 function Name_P_Can_Designate_Primitive
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Name;
Match_1 : Bare_Renaming_Clause;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Renaming_Clause;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8498 '<Match at nodes.lkt:12400:43>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12400'
--# expr-start 8493 '<parent at nodes.lkt:12400:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12400'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 8493
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind n Match_0
--# expr-start 8494 '<FieldAccess for can_designate_primitive at nodes.lkt:12401:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12401'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.can_designate_primitive
Fld_1 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Match_0);
--# end
--# expr-done 8494
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Renaming_Clause .. Ada_Synthetic_Renaming_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind r Match_1
--# expr-start 8497 '<Not at nodes.lkt:12402:35>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12402'
--# expr-start 8496 '<IsA PackageRenamingDecl at nodes.lkt:12402:39>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12402'
--# expr-start 8495 '<parent at nodes.lkt:12402:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12402'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_1.Parent; 
--# expr-done 8495
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Package_Renaming_Decl_Range; 
--# expr-done 8496
Not_Val := not (Is_A); 
--# expr-done 8497
Let_Result_1 := Not_Val; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Range_Spec | Ada_Reverse_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 8498
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Can_Designate_Primitive;
--# end
--# property-start Name.parent_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12407'
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name
  (Node : Bare_Name
      ; Stop_At : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind stop_at Stop_At
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Is_Null : Boolean;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8503 '<If at nodes.lkt:12408:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12408'
--# expr-start 8499 '<IsNull at nodes.lkt:12408:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12408'
Is_Null := Stop_At = null; 
--# expr-done 8499
if Is_Null then
   If_Result := True;
else
   --# expr-start 8500 '<Eq at nodes.lkt:12408:31>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12408'
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Stop_At; 
--# expr-done 8500
   If_Result := Is_Equal;
end if;
if If_Result then
   If_Result_1 := No_Entity_Name;
else
   --# expr-start 8502 '<Cast to Name at nodes.lkt:12408:76>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12408'
--# expr-start 8501 '<parent at nodes.lkt:12408:76>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12408'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8501
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 8502
   If_Result_1 := Cast_Result;
end if;
--# expr-done 8503
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Name;
--# end
--# property-start Name.parent_callexpr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12424'
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Callexpr
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Call_Expr
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_217 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Id;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_217 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_217;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8516 '<Cast to CallExpr at nodes.lkt:12424:47>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12424'
--# expr-start 8515 '<Find at nodes.lkt:12424:47>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12424'
   --# expr-start 8513 '<Map at nodes.lkt:12424:47>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12424'
   --# expr-start 8504 '<FieldAccess for parents at nodes.lkt:12424:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12424'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 8504
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
--# expr-start 8505 '<IsA CallExpr at nodes.lkt:12425:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12425'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 8505
if Is_A then
   If_Result_1 := True;
else
--# expr-start 8506 '<IsA DottedName, BaseId at nodes.lkt:12426:13>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12426'
Is_A_1 := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Dotted_Name_Range | Ada_Base_Id; 
--# expr-done 8506
if Is_A_1 then
   --# expr-start 8512 '<Match at nodes.lkt:12426:42>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12426'
--# expr-start 8507 '<parent at nodes.lkt:12426:48>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12426'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Item_0.Node, E_Info => Item_0.Info);
--# expr-done 8507
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind pfx Match_0
--# expr-start 8509 '<Eq at nodes.lkt:12427:41>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12427'
--# expr-start 8508 '<FieldAccess for suffix at nodes.lkt:12427:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12427'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 8508
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result, Item_0); 
--# expr-done 8509
Let_Result := Is_Equiv; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ce Match_1
--# expr-start 8511 '<Eq at nodes.lkt:12428:38>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12428'
--# expr-start 8510 '<FieldAccess for name at nodes.lkt:12428:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12428'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 8510
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_1, Item_0); 
--# expr-done 8511
Let_Result_1 := Is_Equiv_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 8512
   If_Result := Match_Result;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Take_While_Ok := If_Result_1;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8513
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 8514 '<IsA CallExpr at nodes.lkt:12432:19>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12432'
Is_A_2 := Item_1.Node /= null 
and then Item_1.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 8514
      if Is_A_2 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 8515
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 8516
         Property_Result := Cast_Result_5;
   --# end
      Finalizer_Scope_217;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_217;
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Callexpr;
--# end
--# property-start Name.is_range_attribute 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12435'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Range_Attribute
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Var_Expr_0 : Bare_Attribute_Ref;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Attribute_Ref;
As_Entity : Internal_Entity_Attribute_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 8517 '<Cast to AttributeRef at nodes.lkt:12435:38>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12435'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8517
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind attr_ref Var_Expr_0
--# expr-start 8521 '<FieldAccess for name_is at nodes.lkt:12436:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12436'
--# expr-start 8519 '<FieldAccess for attribute at nodes.lkt:12436:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12436'
--# expr-start 8518 '<as_bare_entity at nodes.lkt:12436:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12436'
Is_Null := Var_Expr_0 = null; 
if Is_Null then
   If_Result := No_Entity_Attribute_Ref;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_0); 
   If_Result := As_Entity;
end if;
--# expr-done 8518
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => If_Result.Node.Attribute_Ref_F_Attribute, Info => If_Result.Info);
--# expr-done 8519
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8520 '<Symbol Range at nodes.lkt:12436:65>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12436'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 8520
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 8521
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Range_Attribute;
--# end
--# property-start Name.called_subp_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12444'
pragma Warnings (Off, "is not referenced");
 function Name_P_Called_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Var_Expr_223 : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Entity_Base_Formal_Param_Holder;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Result_Var : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8522 '<FieldAccess for called_formal_subp_spec at nodes.lkt:12445:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12445'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_formal_subp_spec'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Called_Formal_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8522
Var_Expr_223 := Fld;
if Var_Expr_223 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 8523 '<FieldAccess for corresponding_actual at nodes.lkt:12445:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12445'
      if Var_Expr_223.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.corresponding_actual'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Corresponding_Actual (Node => Var_Expr_223.Node, E_Info => Var_Expr_223.Info);
--# end
--# expr-done 8523
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Formal_Param_Holder;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Called_Subp_Spec;
--# end
--# property-start Name.referenced_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12453'
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8525 '<FieldAccess for decl at nodes.lkt:12454:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12454'
--# expr-start 8524 '<FieldAccess for referenced_decl_internal at nodes.lkt:12454:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12454'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8524
Fld_1 := Fld.Decl;
--# expr-done 8525
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Decl;
--# end
--# property-start Name.failsafe_referenced_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12460'
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Fld : Internal_Refd_Decl;
New_Struct : Internal_Refd_Decl;
Try_Result : Internal_Refd_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8528 '<Try at nodes.lkt:12461:9>' Try_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12461'
begin
   --# expr-start 8526 '<FieldAccess for referenced_decl_internal at nodes.lkt:12462:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12462'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8526
   Try_Result := Fld;
exception
   when Property_Error =>
      --# expr-start 8527 '<New RefdDecl at nodes.lkt:12463:15>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12463'
New_Struct := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 8527
      Try_Result := New_Struct;
end;
--# expr-done 8528
         Property_Result := Try_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Failsafe_Referenced_Decl;
--# end
--# property-start Name.get_referenced_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12487'
pragma Warnings (Off, "is not referenced");
 function Name_P_Get_Referenced_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Logic_Val_Result;
            procedure Finalizer_Scope_3126 with Inline_Always;
      Local_Val_Result : Internal_Logic_Val_Result;
Local_Entity : Internal_Entity_Basic_Decl;
Local_Generic_Context_Node : Bare_Ada_Node;
Local_Entity_With_Generic_Context : Internal_Entity_Basic_Decl;
Var_Expr_0 : Boolean;
Var_Expr_224 : Env_Rebindings;
Var_Expr_225 : Internal_Entity_Body_Node;
Var_Expr_226 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_9 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Body_Node;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Result_Var_2 : Bare_Basic_Decl;
Cast_Result_4 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Fld_12 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Scope_Result_3 : Internal_Logic_Val_Result;
Result_Var_3 : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
Let_Result_1 : Internal_Logic_Val_Result;
Let_Result_2 : Internal_Logic_Val_Result;
Let_Result_3 : Internal_Logic_Val_Result;
            procedure Finalizer_Scope_3126 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3126;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8554 '<Let at nodes.lkt:12488:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12488'
--# scope-start
--# expr-start 8530 '<FieldAccess for logic_val at nodes.lkt:12488:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12488'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 8529 '<FieldAccess for ref_var at nodes.lkt:12488:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12488'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 8529
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 8530
Local_Val_Result := Fld_1; 
--# bind val_result Local_Val_Result
--# expr-start 8553 '<Let at nodes.lkt:12489:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12489'
--# scope-start
--# expr-start 8532 '<Cast to BasicDecl at nodes.lkt:12489:22>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12489'
--# expr-start 8531 '<FieldAccess for value at nodes.lkt:12489:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12489'
Fld_2 := Local_Val_Result.Value;
--# expr-done 8531
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8532
Local_Entity := Cast_Result_1; 
--# bind entity Local_Entity
--# expr-start 8552 '<Let at nodes.lkt:12490:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12490'
--# scope-start
--# expr-start 8534 '<FieldAccess for rebindings at nodes.lkt:12490:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12490'
--# expr-start 8533 '<FieldAccess for info at nodes.lkt:12490:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12490'
Fld_3 := Ent.Info;
--# expr-done 8533
Fld_4 := Fld_3.Rebindings;
--# expr-done 8534
Var_Expr_224 := Fld_4;
if Var_Expr_224 /= null then
   --# scope-start
--# expr-start 8536 '<env_node at nodes.lkt:12490:36>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12490'
--# expr-start 8535 '<FieldAccess for old_env at nodes.lkt:12490:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12490'
      if Var_Expr_224 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Var_Expr_224.Old_Env;
Inc_Ref (Fld_5);
--# expr-done 8535
Env_Node := AST_Envs.Env_Node (Fld_5); 
--# expr-done 8536
Scope_Result := Env_Node;
   --# end
      Finalizer_Scope_3126;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Local_Generic_Context_Node := Result_Var; 
--# bind generic_context_node Local_Generic_Context_Node
--# expr-start 8551 '<Let at nodes.lkt:12493:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12493'
--# scope-start
--# expr-start 8548 '<If at nodes.lkt:12493:43>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12493'
--# expr-start 8538 '<Not at nodes.lkt:12493:46>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12493'
--# expr-start 8537 '<IsNull at nodes.lkt:12493:50>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12493'
Is_Null := Local_Generic_Context_Node = null; 
--# expr-done 8537
Not_Val := not (Is_Null); 
--# expr-done 8538
if Not_Val then
--# expr-start 8540 '<Eq at nodes.lkt:12497:17>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12497'
--# expr-start 8539 '<FieldAccess for node at nodes.lkt:12497:41>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12497'
Fld_9 := Local_Entity.Node;
--# expr-done 8539
      Cast_Result_2 := Fld_9;
Is_Equal := Local_Generic_Context_Node = Cast_Result_2; 
--# expr-done 8540
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8544 '<Eq at nodes.lkt:12501:17>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12501'
--# expr-start 8541 '<Cast to Body at nodes.lkt:12501:41>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12501'
Cast_Expr_1 := Local_Entity; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 8541
Var_Expr_225 := Cast_Result_3;
if Var_Expr_225 /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 8542 '<FieldAccess for decl_part at nodes.lkt:12501:41>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12501'
      if Var_Expr_225.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_225.Node, Imprecise_Fallback => False, E_Info => Var_Expr_225.Info);
--# end
--# expr-done 8542
Var_Expr_226 := Fld_10;
if Var_Expr_226 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 8543 '<FieldAccess for node at nodes.lkt:12501:41>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12501'
Fld_11 := Var_Expr_226.Node;
--# expr-done 8543
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
      Cast_Result_4 := Result_Var_2;
Is_Equal_1 := Local_Generic_Context_Node = Cast_Result_4; 
--# expr-done 8544
   If_Result := Is_Equal_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 8547 '<FieldAccess for unshed_rebindings_helper at nodes.lkt:12505:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12505'
      if Local_Entity.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8546 '<FieldAccess for rebindings at nodes.lkt:12505:45>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12505'
--# expr-start 8545 '<FieldAccess for info at nodes.lkt:12505:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12505'
Fld_6 := Ent.Info;
--# expr-done 8545
Fld_7 := Fld_6.Rebindings;
--# expr-done 8546
--# property-call-start BasicDecl.unshed_rebindings_helper
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings_Helper (Node => Local_Entity.Node, Rebindings => Fld_7, E_Info => Local_Entity.Info);
--# end
--# expr-done 8547
   If_Result_2 := Fld_8;
else
   If_Result_2 := Local_Entity;
end if;
--# expr-done 8548
Local_Entity_With_Generic_Context := If_Result_2; 
--# bind entity_with_generic_context Local_Entity_With_Generic_Context
--# expr-start 8549 '<FieldAccess for success at nodes.lkt:12508:9>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12508'
Fld_12 := Local_Val_Result.Success;
--# expr-done 8549
Var_Expr_0 := Fld_12;
if Var_Expr_0 /= False then
   --# scope-start
   --# bind s Var_Expr_0
--# expr-start 8550 '<New LogicValResult at nodes.lkt:12509:20>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12509'
Cast_Expr_2 := Local_Entity_With_Generic_Context; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
New_Struct := (Success => Var_Expr_0, Value => Cast_Result_5); 
--# expr-done 8550
Scope_Result_3 := New_Struct;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Logic_Val_Result;
end if;
Let_Result := Result_Var_3; 
--# end
--# expr-done 8551
Let_Result_1 := Let_Result; 
--# end
--# expr-done 8552
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 8553
Let_Result_3 := Let_Result_2; 
--# end
--# expr-done 8554
         Property_Result := Let_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3126;
            raise;
      end;
   return Property_Result;
end Name_P_Get_Referenced_Decl;
--# end
--# property-start Name.referenced_decl_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12518'
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl_Internal
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Local_V : Internal_Logic_Val_Result;
Local_V_1 : Internal_Logic_Val_Result;
Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Refd_Decl;
Var_Expr_2 : Internal_Refd_Decl;
Var_Expr_227 : Internal_Entity_Name;
Var_Expr_228 : Internal_Entity_Basic_Decl;
Fld : Internal_Logic_Val_Result;
Try_Result : Internal_Logic_Val_Result;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Refd_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
New_Struct_1 : Internal_Refd_Decl;
Scope_Result : Internal_Refd_Decl;
New_Struct_2 : Internal_Refd_Decl;
Result_Var : Internal_Refd_Decl;
Scope_Result_1 : Internal_Refd_Decl;
Result_Var_1 : Internal_Refd_Decl;
Fld_3 : Boolean;
If_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
Fld_4 : Internal_Logic_Val_Result;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
New_Struct_3 : Internal_Refd_Decl;
New_Struct_4 : Internal_Refd_Decl;
Fld_6 : Boolean;
If_Result_1 : Internal_Refd_Decl;
Let_Result_1 : Internal_Refd_Decl;
If_Result_2 : Internal_Refd_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_9 : Ref_Result_Kind;
New_Struct_5 : Internal_Refd_Decl;
Fld_10 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Fld_11 : Ref_Result_Kind;
Is_Equal : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Refd_Decl;
Scope_Result_3 : Internal_Refd_Decl;
Result_Var_3 : Internal_Refd_Decl;
Scope_Result_4 : Internal_Refd_Decl;
Result_Var_4 : Internal_Refd_Decl;
Fld_12 : Boolean;
If_Result_5 : Internal_Refd_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8585 '<If at nodes.lkt:12521:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12521'
--# expr-start 8555 '<FieldAccess for is_defining at nodes.lkt:12521:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12521'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_12 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 8555
if Fld_12 then
   If_Result_5 := No_Refd_Decl;
else
--# expr-start 8575 '<If at nodes.lkt:12522:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12522'
if Imprecise_Fallback then
   --# expr-start 8566 '<Let at nodes.lkt:12524:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12524'
--# scope-start
--# expr-start 8557 '<Try at nodes.lkt:12524:25>' Try_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12524'
begin
   --# expr-start 8556 '<FieldAccess for get_referenced_decl at nodes.lkt:12525:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12525'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.get_referenced_decl
Fld := Libadalang.Implementation.Name_P_Get_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8556
   Try_Result := Fld;
exception
   when Property_Error =>
      Try_Result := No_Logic_Val_Result;
end;
--# expr-done 8557
Local_V_1 := Try_Result; 
--# bind v Local_V_1
--# expr-start 8565 '<If at nodes.lkt:12527:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12527'
--# expr-start 8558 '<FieldAccess for success at nodes.lkt:12527:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12527'
Fld_3 := Local_V_1.Success;
--# expr-done 8558
if Fld_3 then
   --# expr-start 8561 '<New RefdDecl at nodes.lkt:12530:21>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12530'
--# expr-start 8560 '<Cast to BasicDecl at nodes.lkt:12531:30>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12531'
--# expr-start 8559 '<FieldAccess for value at nodes.lkt:12531:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12531'
Fld_1 := Local_V_1.Value;
--# expr-done 8559
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 8560
New_Struct := (Decl => Cast_Result, Kind => Precise); 
--# expr-done 8561
   If_Result := New_Struct;
else
Var_Expr_227 := Ent;
if Var_Expr_227 /= No_Entity_Name then
   --# scope-start
--# expr-start 8562 '<FieldAccess for first_corresponding_decl at nodes.lkt:12536:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12536'
      if Var_Expr_227.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Var_Expr_227.Node, E_Info => Var_Expr_227.Info);
--# end
--# expr-done 8562
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind fcd Var_Expr_0
--# expr-start 8563 '<New RefdDecl at nodes.lkt:12537:34>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12537'
New_Struct_1 := (Decl => Var_Expr_0, Kind => Imprecise); 
--# expr-done 8563
Scope_Result := New_Struct_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8564 '<New RefdDecl at nodes.lkt:12537:96>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12537'
New_Struct_2 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 8564
   Result_Var := New_Struct_2;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Refd_Decl;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 8565
Let_Result := If_Result; 
--# end
--# expr-done 8566
   If_Result_2 := Let_Result;
else
   --# expr-start 8574 '<Let at nodes.lkt:12542:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12542'
--# scope-start
--# expr-start 8567 '<FieldAccess for get_referenced_decl at nodes.lkt:12542:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12542'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.get_referenced_decl
Fld_4 := Libadalang.Implementation.Name_P_Get_Referenced_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8567
Local_V := Fld_4; 
--# bind v Local_V
--# expr-start 8573 '<If at nodes.lkt:12544:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12544'
--# expr-start 8568 '<FieldAccess for success at nodes.lkt:12544:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12544'
Fld_6 := Local_V.Success;
--# expr-done 8568
if Fld_6 then
   --# expr-start 8571 '<New RefdDecl at nodes.lkt:12544:35>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12544'
--# expr-start 8570 '<Cast to BasicDecl at nodes.lkt:12545:26>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12545'
--# expr-start 8569 '<FieldAccess for value at nodes.lkt:12545:26>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12545'
Fld_5 := Local_V.Value;
--# expr-done 8569
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8570
New_Struct_3 := (Decl => Cast_Result_1, Kind => Precise); 
--# expr-done 8571
   If_Result_1 := New_Struct_3;
else
   --# expr-start 8572 '<New RefdDecl at nodes.lkt:12546:24>' New_Struct_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12546'
New_Struct_4 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 8572
   If_Result_1 := New_Struct_4;
end if;
--# expr-done 8573
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 8574
   If_Result_2 := Let_Result_1;
end if;
--# expr-done 8575
Var_Expr_1 := If_Result_2;
if Var_Expr_1 /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr_1
--# expr-start 8579 '<New RefdDecl at nodes.lkt:12551:22>' New_Struct_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12551'
--# expr-start 8576 '<FieldAccess for decl at nodes.lkt:12552:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12552'
Fld_7 := Var_Expr_1.Decl;
--# expr-done 8576
Var_Expr_228 := Fld_7;
if Var_Expr_228 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 8577 '<FieldAccess for wrap_public_reference at nodes.lkt:12552:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12552'
      if Var_Expr_228.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_228.Node, E_Info => Var_Expr_228.Info);
--# end
--# expr-done 8577
Scope_Result_2 := Fld_8;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
--# expr-start 8578 '<FieldAccess for kind at nodes.lkt:12552:62>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12552'
Fld_9 := Var_Expr_1.Kind;
--# expr-done 8578
New_Struct_5 := (Decl => Result_Var_2, Kind => Fld_9); 
--# expr-done 8579
Var_Expr_2 := New_Struct_5;
if Var_Expr_2 /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr_2
--# expr-start 8584 '<If at nodes.lkt:12556:26>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12556'
--# expr-start 8581 '<IsNull at nodes.lkt:12556:29>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12556'
--# expr-start 8580 '<FieldAccess for decl at nodes.lkt:12556:29>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12556'
Fld_10 := Var_Expr_2.Decl;
--# expr-done 8580
Is_Null := Fld_10.Node = null; 
--# expr-done 8581
if Is_Null then
   --# expr-start 8583 '<Eq at nodes.lkt:12556:50>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12556'
--# expr-start 8582 '<FieldAccess for kind at nodes.lkt:12556:50>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12556'
Fld_11 := Var_Expr_2.Kind;
--# expr-done 8582
Is_Equal := Fld_11 = Precise; 
--# expr-done 8583
   If_Result_3 := Is_Equal;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := No_Refd_Decl;
else
   If_Result_4 := Var_Expr_2;
end if;
--# expr-done 8584
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Refd_Decl;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Refd_Decl;
end if;
   If_Result_5 := Result_Var_4;
end if;
--# expr-done 8585
         Property_Result := If_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Decl_Internal;
--# end
--# property-start Name.name_designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12564'
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_224 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_224 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_224;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8590 '<bind at nodes.lkt:12565:9>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12565'
--# expr-start 8586 '<FieldAccess for node_env at nodes.lkt:12565:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12565'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8586
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8589 '<bind at nodes.lkt:12566:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12566'
--# expr-start 8587 '<FieldAccess for origin_node at nodes.lkt:12566:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12566'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8587
Bound_Origin := Fld_1; 
--# expr-start 8588 '<FieldAccess for designated_type_impl at nodes.lkt:12568:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12568'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8588
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8589
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 8590
         Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_224;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_224;
            raise;
      end;
   return Property_Result;
end Name_P_Name_Designated_Type;
--# end
--# property-start Name.is_static_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12574'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Subtype
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8592 '<FieldAccess for is_static_decl at nodes.lkt:12575:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12575'
--# expr-start 8591 '<FieldAccess for name_designated_type at nodes.lkt:12575:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12575'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8591
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8592
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Static_Subtype;
--# end
--# property-start Name.name_designated_type_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12581'
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_226 with Inline_Always;
            procedure Finalizer_Scope_3135 with Inline_Always;
      Var_Expr_229 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_226 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_226;
            procedure Finalizer_Scope_3135 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3135;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 8593 '<FieldAccess for name_designated_type at nodes.lkt:12582:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12582'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8593
Var_Expr_229 := Fld;
if Var_Expr_229 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8594 '<FieldAccess for primitives_env at nodes.lkt:12582:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12582'
      if Var_Expr_229.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr_229.Node, E_Info => Var_Expr_229.Info);
--# end
--# expr-done 8594
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3135;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_226;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_226;
                     Finalizer_Scope_3135;
            raise;
      end;
   return Property_Result;
end Name_P_Name_Designated_Type_Env;
--# end
--# property-start Name.referenced_unit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12586'
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Unit
  (Node : Bare_Name
      ; Kind : Analysis_Unit_Kind
      ; Not_Found_Is_Error : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind kind Kind
   --# bind not_found_is_error Not_Found_Is_Error
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_227 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Compilation_Unit;
            procedure Finalizer_Scope_227 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_227;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8596 '<FieldAccess for designated_compilation_unit at nodes.lkt:12586:101>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12586'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8595 '<FieldAccess for as_symbol_array at nodes.lkt:12587:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12587'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Self);
--# end
--# expr-done 8595
--# property-call-start AdaNode.designated_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld, Kind => Kind, Load_If_Needed => True, Not_Found_Is_Error => Not_Found_Is_Error);
--# end
--# expr-done 8596
         Property_Result := Fld_1;
   --# end
      Finalizer_Scope_227;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_227;
            raise;
      end;
   return Property_Result;
end Name_P_Referenced_Unit;
--# end
--# property-start Name.matches 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12595'
pragma Warnings (Off, "is not referenced");
 function Name_P_Matches
  (Node : Bare_Name
      ; N : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Is_Equal : Boolean;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Defining_Name;
Fld_2 : Bare_Name;
Fld_3 : Boolean;
Cast_Expr_1 : Bare_Name;
Cast_Result_1 : Bare_Defining_Name;
Fld_4 : Bare_Name;
Fld_5 : Boolean;
Cast_Expr_2 : Bare_Name;
Cast_Result_2 : Bare_Dotted_Name;
Fld_6 : Bare_Name;
Cast_Expr_3 : Bare_Name;
Cast_Result_3 : Bare_Dotted_Name;
Fld_7 : Bare_Name;
Fld_8 : Boolean;
Cast_Expr_4 : Bare_Name;
Cast_Result_4 : Bare_Dotted_Name;
Fld_9 : Bare_Base_Id;
Cast_Expr_5 : Bare_Name;
Cast_Result_5 : Bare_Dotted_Name;
Fld_10 : Bare_Base_Id;
Cast_Result_6 : Bare_Name;
Fld_11 : Boolean;
If_Result : Boolean;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Is_A_2 : Boolean;
If_Result_3 : Boolean;
Is_A_3 : Boolean;
If_Result_4 : Boolean;
Is_A_4 : Boolean;
Is_A_5 : Boolean;
If_Result_5 : Boolean;
Is_A_6 : Boolean;
Is_A_7 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8627 '<If at nodes.lkt:12595:34>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12595'
--# expr-start 8597 '<IsA Identifier, SyntheticIdentifier at nodes.lkt:12596:9>' Is_A_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12596'
Is_A_4 := Self /= null 
and then Self.Kind in Ada_Identifier_Range | Ada_Synthetic_Identifier_Range; 
--# expr-done 8597
if Is_A_4 then
   --# expr-start 8598 '<IsA Identifier, SyntheticIdentifier at nodes.lkt:12596:54>' Is_A_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12596'
Is_A_5 := N /= null 
and then N.Kind in Ada_Identifier_Range | Ada_Synthetic_Identifier_Range; 
--# expr-done 8598
   If_Result_5 := Is_A_5;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 8599 '<IsA StringLiteral at nodes.lkt:12598:9>' Is_A_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12598'
Is_A_6 := Self /= null 
and then Self.Kind in Ada_String_Literal_Range; 
--# expr-done 8599
if Is_A_6 then
   --# expr-start 8600 '<IsA StringLiteral at nodes.lkt:12598:35>' Is_A_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12598'
Is_A_7 := N /= null 
and then N.Kind in Ada_String_Literal_Range; 
--# expr-done 8600
   If_Result_6 := Is_A_7;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
if If_Result_7 then
   --# expr-start 8603 '<Eq at nodes.lkt:12599:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12599'
--# expr-start 8601 '<FieldAccess for name_symbol at nodes.lkt:12599:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12599'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 8601
--# expr-start 8602 '<FieldAccess for name_symbol at nodes.lkt:12599:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12599'
      if N = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => N);
--# end
--# expr-done 8602
Is_Equal := Fld = Fld_1; 
--# expr-done 8603
   If_Result_8 := Is_Equal;
else
   --# expr-start 8626 '<If at nodes.lkt:12595:34>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12595'
--# expr-start 8604 '<IsA DefiningName at nodes.lkt:12600:10>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12600'
Is_A_3 := Self /= null 
and then Self.Kind in Ada_Defining_Name_Range; 
--# expr-done 8604
if Is_A_3 then
   --# expr-start 8607 '<FieldAccess for matches at nodes.lkt:12600:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12600'
--# expr-start 8606 '<FieldAccess for name at nodes.lkt:12600:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12600'
--# expr-start 8605 '<Cast to DefiningName at nodes.lkt:12600:36>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12600'
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Defining_Name_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8605
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Defining_Name_F_Name;
--# expr-done 8606
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.matches
Fld_3 := Libadalang.Implementation.Name_P_Matches (Node => Fld_2, N => N);
--# end
--# expr-done 8607
   If_Result_4 := Fld_3;
else
   --# expr-start 8625 '<If at nodes.lkt:12595:34>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12595'
--# expr-start 8608 '<IsA DefiningName at nodes.lkt:12601:10>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12601'
Is_A_2 := N /= null 
and then N.Kind in Ada_Defining_Name_Range; 
--# expr-done 8608
if Is_A_2 then
   --# expr-start 8611 '<FieldAccess for matches at nodes.lkt:12601:33>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12601'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8610 '<FieldAccess for name at nodes.lkt:12601:46>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12601'
--# expr-start 8609 '<Cast to DefiningName at nodes.lkt:12601:46>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12601'
Cast_Expr_1 := N; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Defining_Name_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 8609
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Cast_Result_1.Defining_Name_F_Name;
--# expr-done 8610
--# property-call-start Name.matches
Fld_5 := Libadalang.Implementation.Name_P_Matches (Node => Self, N => Fld_4);
--# end
--# expr-done 8611
   If_Result_3 := Fld_5;
else
   --# expr-start 8624 '<If at nodes.lkt:12595:34>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12595'
--# expr-start 8612 '<IsA DottedName at nodes.lkt:12602:10>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Is_A := Self /= null 
and then Self.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8612
if Is_A then
   --# expr-start 8613 '<IsA DottedName at nodes.lkt:12602:33>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Is_A_1 := N /= null 
and then N.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8613
   If_Result_1 := Is_A_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
--# expr-start 8618 '<FieldAccess for matches at nodes.lkt:12602:54>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8615 '<FieldAccess for prefix at nodes.lkt:12602:54>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8614 '<Cast to DottedName at nodes.lkt:12602:54>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Cast_Expr_2 := Self; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 8614
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Cast_Result_2.Dotted_Name_F_Prefix;
--# expr-done 8615
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8617 '<FieldAccess for prefix at nodes.lkt:12602:89>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8616 '<Cast to DottedName at nodes.lkt:12602:89>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Cast_Expr_3 := N; 
   if Cast_Expr_3 = null
      or else Cast_Expr_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_3 := Cast_Expr_3;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 8616
      if Cast_Result_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Cast_Result_3.Dotted_Name_F_Prefix;
--# expr-done 8617
--# property-call-start Name.matches
Fld_8 := Libadalang.Implementation.Name_P_Matches (Node => Fld_6, N => Fld_7);
--# end
--# expr-done 8618
if Fld_8 then
   --# expr-start 8623 '<FieldAccess for matches at nodes.lkt:12602:118>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8620 '<FieldAccess for suffix at nodes.lkt:12602:118>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8619 '<Cast to DottedName at nodes.lkt:12602:118>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Cast_Expr_4 := Self; 
   if Cast_Expr_4 = null
      or else Cast_Expr_4.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_4 := Cast_Expr_4;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 8619
      if Cast_Result_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Cast_Result_4.Dotted_Name_F_Suffix;
--# expr-done 8620
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8622 '<FieldAccess for suffix at nodes.lkt:12602:153>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
--# expr-start 8621 '<Cast to DottedName at nodes.lkt:12602:153>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12602'
Cast_Expr_5 := N; 
   if Cast_Expr_5 = null
      or else Cast_Expr_5.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_5 := Cast_Expr_5;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 8621
      if Cast_Result_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Cast_Result_5.Dotted_Name_F_Suffix;
--# expr-done 8622
      Cast_Result_6 := Fld_10;
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => Fld_9, N => Cast_Result_6);
--# end
--# expr-done 8623
   If_Result := Fld_11;
else
   If_Result := False;
end if;
   If_Result_2 := If_Result;
else
   If_Result_2 := False;
end if;
--# expr-done 8624
   If_Result_3 := If_Result_2;
end if;
--# expr-done 8625
   If_Result_4 := If_Result_3;
end if;
--# expr-done 8626
   If_Result_8 := If_Result_4;
end if;
--# expr-done 8627
         Property_Result := If_Result_8;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Matches;
--# end
--# property-start Name.name_matches 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12610'
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Matches
  (Node : Bare_Name
      ; N : Internal_Entity_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Fld : Bare_Name;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8629 '<FieldAccess for matches at nodes.lkt:12610:47>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12610'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8628 '<FieldAccess for node at nodes.lkt:12610:60>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12610'
Fld := N.Node;
--# expr-done 8628
--# property-call-start Name.matches
Fld_1 := Libadalang.Implementation.Name_P_Matches (Node => Self, N => Fld);
--# end
--# expr-done 8629
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Matches;
--# end
--# property-start Name.use_package_name_designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12618'
pragma Warnings (Off, "is not referenced");
 function Name_P_Use_Package_Name_Designated_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_230 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Use_Package_Clause;
Fld_2 : Integer;
Fld_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_230 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_230;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8634 '<FieldAccess for designated_env at nodes.lkt:12619:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12619'
--# expr-start 8632 '<Cast to UsePackageClause at nodes.lkt:12619:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12619'
--# expr-start 8631 '<parent at nodes.lkt:12619:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12619'
--# expr-start 8630 '<parent at nodes.lkt:12619:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12619'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8630
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 8631
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Use_Package_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8632
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8633 '<FieldAccess for child_index at nodes.lkt:12619:65>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12619'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 8633
--# property-call-start UsePackageClause.designated_env
Fld_3 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Env (Node => Cast_Result.Node, Index => Fld_2, E_Info => Cast_Result.Info);
--# end
--# expr-done 8634
         Property_Result := Fld_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_230;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_230;
            raise;
      end;
   return Property_Result;
end Name_P_Use_Package_Name_Designated_Env;
--# end
--# property-start Name.xref_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12625'
pragma Warnings (Off, "is not referenced");
 function Name_P_Xref_Type_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_231 with Inline_Always;
            procedure Finalizer_Scope_3136 with Inline_Always;
            procedure Finalizer_Scope_3137 with Inline_Always;
            procedure Finalizer_Scope_3138 with Inline_Always;
            procedure Finalizer_Scope_3139 with Inline_Always;
            procedure Finalizer_Scope_3140 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Base_Id;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_8 : Logic_Equation;
Result_Var : Logic_Equation;
Fld_9 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Internal_Entity_Name;
Fld_13 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Name;
Fld_16 : Logic_Var;
Bind_Result_2 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_17 : Internal_Entity_Identifier;
Fld_18 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Fld_19 : Internal_Entity_Identifier;
Fld_20 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Base_Id;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Match_Result : Logic_Equation;
            procedure Finalizer_Scope_231 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_231;
            procedure Finalizer_Scope_3136 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3136;
            procedure Finalizer_Scope_3137 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3137;
            procedure Finalizer_Scope_3138 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_3138;
            procedure Finalizer_Scope_3139 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3139;
            procedure Finalizer_Scope_3140 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_3140;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8668 '<Match at nodes.lkt:12625:42>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12625'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 8642 '<And at nodes.lkt:12626:32>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12626'
--# expr-start 8636 '<FieldAccess for xref_no_overloading at nodes.lkt:12626:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12626'
--# expr-start 8635 '<FieldAccess for prefix at nodes.lkt:12626:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12626'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8635
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => True, All_Els => False, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 8636
--# expr-start 8641 '<bind at nodes.lkt:12627:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12627'
--# expr-start 8638 '<FieldAccess for designated_env_no_overloading at nodes.lkt:12627:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12627'
--# expr-start 8637 '<FieldAccess for prefix at nodes.lkt:12627:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12627'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8637
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 8638
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 8640 '<FieldAccess for xref_type_equation at nodes.lkt:12629:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12629'
--# expr-start 8639 '<FieldAccess for suffix at nodes.lkt:12629:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12629'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 8639
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_5 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 8640
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8641
And_Pred := Create_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12626:32") else null)); 
--# expr-done 8642
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3136;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind i Match_1
--# expr-start 8643 '<FieldAccess for designated_type_impl at nodes.lkt:12631:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12631'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 8643
Var_Expr_0 := Fld_6;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind type Var_Expr_0
--# expr-start 8645 '<Bind at nodes.lkt:12632:23>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12632'
--# expr-start 8644 '<FieldAccess for ref_var at nodes.lkt:12632:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12632'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 8644
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Cast_Expr := Var_Expr_0; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_7, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12632:23") else null)); 
--# expr-done 8645
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3138;
   Result_Var := Scope_Result;
else
   --# expr-start 8646 '<FieldAccess for undefined_reference_equation at nodes.lkt:12633:25>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12633'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.undefined_reference_equation
Fld_8 := Libadalang.Implementation.Name_P_Undefined_Reference_Equation (Node => Ent.Node);
--# end
--# expr-done 8646
   Result_Var := Fld_8;
end if;
      Inc_Ref (Result_Var);
Let_Result_1 := Result_Var; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3137;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind ar Match_2
--# expr-start 8667 '<And at nodes.lkt:12635:34>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12635'
--# expr-start 8648 '<FieldAccess for xref_type_equation at nodes.lkt:12635:34>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12635'
--# expr-start 8647 '<FieldAccess for prefix at nodes.lkt:12635:34>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12635'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 8647
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_10 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_9.Info);
--# end
--# expr-done 8648
--# expr-start 8666 '<If at nodes.lkt:12636:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8652 '<Eq at nodes.lkt:12636:16>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8650 '<FieldAccess for sym at nodes.lkt:12636:16>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8649 '<FieldAccess for attribute at nodes.lkt:12636:16>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 8649
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_20 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_19.Node);
--# end
--# expr-done 8650
--# expr-start 8651 '<Symbol Class at nodes.lkt:12636:38>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 8651
Is_Equal_1 := Fld_20 = Sym_1; 
--# expr-done 8652
if Is_Equal_1 then
   --# expr-start 8656 '<Bind at nodes.lkt:12636:52>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8654 '<FieldAccess for ref_var at nodes.lkt:12636:70>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8653 '<FieldAccess for prefix at nodes.lkt:12636:70>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 8653
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12.Node);
--# end
--# expr-done 8654
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 8655 '<FieldAccess for ref_var at nodes.lkt:12636:56>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 8655
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result_1 := Solver.Create_Propagate (Fld_13, Fld_11, Logic_Functor_945'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12636:52") else null)); 
--# expr-done 8656
   If_Result_1 := Bind_Result_1;
else
   --# expr-start 8665 '<If at nodes.lkt:12636:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12636'
--# expr-start 8660 '<Eq at nodes.lkt:12637:18>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
--# expr-start 8658 '<FieldAccess for sym at nodes.lkt:12637:18>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
--# expr-start 8657 '<FieldAccess for attribute at nodes.lkt:12637:18>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 8657
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_18 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_17.Node);
--# end
--# expr-done 8658
--# expr-start 8659 '<Symbol Base at nodes.lkt:12637:40>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 8659
Is_Equal := Fld_18 = Sym; 
--# expr-done 8660
if Is_Equal then
   --# expr-start 8664 '<Bind at nodes.lkt:12637:53>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
--# expr-start 8662 '<FieldAccess for ref_var at nodes.lkt:12637:71>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
--# expr-start 8661 '<FieldAccess for prefix at nodes.lkt:12637:71>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 8661
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_15.Node);
--# end
--# expr-done 8662
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
--# expr-start 8663 '<FieldAccess for ref_var at nodes.lkt:12637:57>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12637'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 8663
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
Bind_Result_2 := Solver.Create_Propagate (Fld_16, Fld_14, Logic_Functor_870'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12637:53") else null)); 
--# expr-done 8664
   If_Result := Bind_Result_2;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12638:18") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 8665
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8666
And_Pred_1 := Create_And (Fld_10, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12635:34") else null)); 
--# expr-done 8667
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3139;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12640:19") else null)); 
Let_Result_3 := False_Rel; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3140;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 8668
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_231;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_231;
                     Finalizer_Scope_3136;
                     Finalizer_Scope_3137;
                     Finalizer_Scope_3138;
                     Finalizer_Scope_3139;
                     Finalizer_Scope_3140;
            raise;
      end;
   return Property_Result;
end Name_P_Xref_Type_Equation;
--# end
--# property-start Name.xref_no_overloading 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12649'
pragma Warnings (Off, "is not referenced");
 function Name_P_Xref_No_Overloading
  (Node : Bare_Name
      ; Sequential : Boolean
         := True
      ; All_Els : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind sequential Sequential
   --# bind all_els All_Els
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_232 with Inline_Always;
            procedure Finalizer_Scope_3141 with Inline_Always;
            procedure Finalizer_Scope_3142 with Inline_Always;
            procedure Finalizer_Scope_3144 with Inline_Always;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Base_Id;
Match_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
If_Result : Lookup_Kind;
Fld_8 : Bare_Name;
If_Result_1 : Bare_Name;
Cast_Result : Bare_Ada_Node;
Fld_9 : Internal_Entity_Array_Access;
Fld_10 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_11 : Logic_Var;
Domain_Equation : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Boolean;
If_Result_2 : Lookup_Kind;
Fld_14 : Bare_Name;
If_Result_3 : Bare_Name;
Cast_Result_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
False_Rel : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Base_Id;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Match_Result : Logic_Equation;
            procedure Finalizer_Scope_232 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_232;
            procedure Finalizer_Scope_3141 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3141;
            procedure Finalizer_Scope_3142 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_3142;
            procedure Finalizer_Scope_3144 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_3144;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8696 '<Match at nodes.lkt:12649:89>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12649'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind dn Match_0
--# expr-start 8676 '<And at nodes.lkt:12650:32>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12650'
--# expr-start 8670 '<FieldAccess for xref_no_overloading at nodes.lkt:12650:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12650'
--# expr-start 8669 '<FieldAccess for prefix at nodes.lkt:12650:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12650'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8669
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 8670
--# expr-start 8675 '<bind at nodes.lkt:12651:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12651'
--# expr-start 8672 '<FieldAccess for designated_env at nodes.lkt:12651:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12651'
--# expr-start 8671 '<FieldAccess for prefix at nodes.lkt:12651:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12651'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8671
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 8672
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 8674 '<FieldAccess for xref_no_overloading at nodes.lkt:12653:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12653'
--# expr-start 8673 '<FieldAccess for suffix at nodes.lkt:12653:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12653'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 8673
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Sequential => Sequential, All_Els => All_Els, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 8674
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8675
And_Pred := Create_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12650:32") else null)); 
--# expr-done 8676
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3141;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind i Match_1
--# expr-start 8695 '<If at nodes.lkt:12655:27>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
if All_Els then
   --# expr-start 8677 '<FieldAccess for is_suffix at nodes.lkt:12655:42>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_16 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 8677
   If_Result_4 := Fld_16;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 8687 '<domain at nodes.lkt:12655:64>' Domain_Equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
--# expr-start 8685 '<Map at nodes.lkt:12655:85>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
   --# expr-start 8683 '<FieldAccess for env_get at nodes.lkt:12655:85>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8678 '<FieldAccess for name_symbol at nodes.lkt:12656:18>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12656'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 8678
--# expr-start 8680 '<If at nodes.lkt:12656:98>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12656'
--# expr-start 8679 '<FieldAccess for is_prefix at nodes.lkt:12656:101>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12656'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_7 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 8679
if Fld_7 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 8680
--# expr-start 8682 '<If at nodes.lkt:12656:45>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12656'
if Sequential then
   --# expr-start 8681 '<FieldAccess for node at nodes.lkt:12656:64>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12656'
Fld_8 := Ent.Node;
--# expr-done 8681
   If_Result_1 := Fld_8;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 8682
      Cast_Result := If_Result_1;
--# property-call-start AdaNode.env_get
Fld_9 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_6, Lookup => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 8683
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_9;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
         --# expr-start 8684 '<FieldAccess for has_visibility at nodes.lkt:12657:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12657'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_10 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 8684
         if Fld_10 then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8685
--# expr-start 8686 '<FieldAccess for ref_var at nodes.lkt:12655:72>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12655'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 8686
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
declare
   Dom : Internal_Entity_Array_Access := Map_Result;
   A   : Entity_Vars.Value_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Self, Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Solver.Create_Domain
     (Fld_11, A, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12655:64") else null));
end;
--# expr-done 8687
   If_Result_5 := Domain_Equation;
else
   --# expr-start 8694 '<Bind at nodes.lkt:12657:55>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12657'
--# expr-start 8688 '<FieldAccess for ref_var at nodes.lkt:12657:59>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12657'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 8688
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
--# expr-start 8693 '<FieldAccess for env_get_first_visible at nodes.lkt:12657:72>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12657'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8690 '<If at nodes.lkt:12658:86>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12658'
--# expr-start 8689 '<FieldAccess for is_prefix at nodes.lkt:12658:89>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12658'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_13 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 8689
if Fld_13 then
   If_Result_2 := Recursive;
else
   If_Result_2 := Flat;
end if;
--# expr-done 8690
--# expr-start 8692 '<If at nodes.lkt:12658:28>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12658'
if Sequential then
   --# expr-start 8691 '<FieldAccess for node at nodes.lkt:12658:47>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12658'
Fld_14 := Ent.Node;
--# expr-done 8691
   If_Result_3 := Fld_14;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 8692
      Cast_Result_1 := If_Result_3;
--# property-call-start SingleTokNode.env_get_first_visible
Fld_15 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Match_1.Node, Lex_Env => Env, Lookup_Type => If_Result_2, From_Node => Cast_Result_1, No_Visibility => False);
--# end
--# expr-done 8693
Bind_Result := Solver.Create_Assign (Fld_12, Fld_15, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12657:55") else null)); 
--# expr-done 8694
   If_Result_5 := Bind_Result;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 8695
Let_Result_1 := If_Result_5; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3142;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:12660:19") else null)); 
Let_Result_2 := False_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3144;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 8696
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_232;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_232;
                     Finalizer_Scope_3141;
                     Finalizer_Scope_3142;
                     Finalizer_Scope_3144;
            raise;
      end;
   return Property_Result;
end Name_P_Xref_No_Overloading;
--# end
--# property-start Name.is_prefix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12667'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Prefix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Prefix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 8698 '<IsA DottedName at nodes.lkt:12669:13>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
--# expr-start 8697 '<parent at nodes.lkt:12669:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 8697
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8698
if Is_A then
   --# expr-start 8702 '<Eq at nodes.lkt:12669:43>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
--# expr-start 8701 '<FieldAccess for prefix at nodes.lkt:12669:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
--# expr-start 8700 '<Cast to DottedName at nodes.lkt:12669:43>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
--# expr-start 8699 '<parent at nodes.lkt:12669:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12669'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 8699
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8700
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Prefix;
--# expr-done 8701
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 8702
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 8705 '<FieldAccess for is_prefix at nodes.lkt:12670:15>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12670'
--# expr-start 8704 '<Cast to Name at nodes.lkt:12670:15>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12670'
--# expr-start 8703 '<parent at nodes.lkt:12670:15>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12670'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 8703
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 8704
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_4 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Cast_Result_2);
--# end
--# expr-done 8705
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8708 '<Not at nodes.lkt:12671:10>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12671'
--# expr-start 8707 '<IsA DottedName at nodes.lkt:12671:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12671'
--# expr-start 8706 '<parent at nodes.lkt:12671:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12671'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 8706
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8707
Not_Val := not (Is_A_1); 
--# expr-done 8708
   If_Result_2 := Not_Val;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Name_P_Is_Prefix;
--# end
--# property-start Name.is_suffix 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12675'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Suffix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Suffix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 8710 '<IsA DottedName at nodes.lkt:12677:13>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
--# expr-start 8709 '<parent at nodes.lkt:12677:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 8709
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8710
if Is_A then
   --# expr-start 8714 '<Eq at nodes.lkt:12677:43>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
--# expr-start 8713 '<FieldAccess for suffix at nodes.lkt:12677:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
--# expr-start 8712 '<Cast to DottedName at nodes.lkt:12677:43>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
--# expr-start 8711 '<parent at nodes.lkt:12677:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12677'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Parent; 
--# expr-done 8711
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8712
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Suffix;
--# expr-done 8713
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 8714
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 8717 '<FieldAccess for is_suffix at nodes.lkt:12678:15>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12678'
--# expr-start 8716 '<Cast to Name at nodes.lkt:12678:15>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12678'
--# expr-start 8715 '<parent at nodes.lkt:12678:15>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12678'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Parent; 
--# expr-done 8715
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 8716
      if Cast_Result_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Cast_Result_2);
--# end
--# expr-done 8717
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8720 '<Not at nodes.lkt:12679:10>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12679'
--# expr-start 8719 '<IsA DottedName at nodes.lkt:12679:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12679'
--# expr-start 8718 '<parent at nodes.lkt:12679:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12679'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 8718
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8719
Not_Val := not (Is_A_1); 
--# expr-done 8720
   If_Result_2 := Not_Val;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Name_P_Is_Suffix;
--# end
--# property-start Name.is_operator_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12683'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Operator_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8721 '<FieldAccess for name_symbol at nodes.lkt:12684:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 8721
--# expr-start 8722 '<Symbol "=" at nodes.lkt:12684:31>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 8722
Is_Equal_20 := Fld = Sym; 
if Is_Equal_20 then
   If_Result_19 := True;
else
--# expr-start 8723 '<Symbol "=" at nodes.lkt:12684:42>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 8723
Is_Equal_19 := Fld = Sym_1; 
if Is_Equal_19 then
   If_Result_18 := True;
else
--# expr-start 8724 '<Symbol "/=" at nodes.lkt:12684:53>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 8724
Is_Equal_18 := Fld = Sym_2; 
if Is_Equal_18 then
   If_Result_17 := True;
else
--# expr-start 8725 '<Symbol "<" at nodes.lkt:12684:65>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 8725
Is_Equal_17 := Fld = Sym_3; 
if Is_Equal_17 then
   If_Result_16 := True;
else
--# expr-start 8726 '<Symbol "<=" at nodes.lkt:12684:76>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 8726
Is_Equal_16 := Fld = Sym_4; 
if Is_Equal_16 then
   If_Result_15 := True;
else
--# expr-start 8727 '<Symbol ">" at nodes.lkt:12684:88>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 8727
Is_Equal_15 := Fld = Sym_5; 
if Is_Equal_15 then
   If_Result_14 := True;
else
--# expr-start 8728 '<Symbol ">=" at nodes.lkt:12684:99>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 8728
Is_Equal_14 := Fld = Sym_6; 
if Is_Equal_14 then
   If_Result_13 := True;
else
--# expr-start 8729 '<Symbol "and" at nodes.lkt:12684:111>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 8729
Is_Equal_13 := Fld = Sym_7; 
if Is_Equal_13 then
   If_Result_12 := True;
else
--# expr-start 8730 '<Symbol "or" at nodes.lkt:12684:124>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 8730
Is_Equal_12 := Fld = Sym_8; 
if Is_Equal_12 then
   If_Result_11 := True;
else
--# expr-start 8731 '<Symbol "xor" at nodes.lkt:12684:136>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 8731
Is_Equal_11 := Fld = Sym_9; 
if Is_Equal_11 then
   If_Result_10 := True;
else
--# expr-start 8732 '<Symbol "abs" at nodes.lkt:12684:149>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 8732
Is_Equal_10 := Fld = Sym_10; 
if Is_Equal_10 then
   If_Result_9 := True;
else
--# expr-start 8733 '<Symbol "*" at nodes.lkt:12684:162>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 8733
Is_Equal_9 := Fld = Sym_11; 
if Is_Equal_9 then
   If_Result_8 := True;
else
--# expr-start 8734 '<Symbol "/" at nodes.lkt:12684:173>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 8734
Is_Equal_8 := Fld = Sym_12; 
if Is_Equal_8 then
   If_Result_7 := True;
else
--# expr-start 8735 '<Symbol "mod" at nodes.lkt:12684:184>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 8735
Is_Equal_7 := Fld = Sym_13; 
if Is_Equal_7 then
   If_Result_6 := True;
else
--# expr-start 8736 '<Symbol "rem" at nodes.lkt:12684:197>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 8736
Is_Equal_6 := Fld = Sym_14; 
if Is_Equal_6 then
   If_Result_5 := True;
else
--# expr-start 8737 '<Symbol "+" at nodes.lkt:12684:210>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 8737
Is_Equal_5 := Fld = Sym_15; 
if Is_Equal_5 then
   If_Result_4 := True;
else
--# expr-start 8738 '<Symbol "-" at nodes.lkt:12684:221>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 8738
Is_Equal_4 := Fld = Sym_16; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 8739 '<Symbol "&""+" at nodes.lkt:12684:232>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_1); 
--# expr-done 8739
Is_Equal_3 := Fld = Sym_17; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 8740 '<Symbol "-" at nodes.lkt:12684:248>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 8740
Is_Equal_2 := Fld = Sym_18; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 8741 '<Symbol "not" at nodes.lkt:12684:259>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 8741
Is_Equal_1 := Fld = Sym_19; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 8742 '<Symbol "abs" at nodes.lkt:12684:272>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12684'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 8742
Is_Equal := Fld = Sym_20; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
end if;
   If_Result_12 := If_Result_11;
end if;
   If_Result_13 := If_Result_12;
end if;
   If_Result_14 := If_Result_13;
end if;
   If_Result_15 := If_Result_14;
end if;
   If_Result_16 := If_Result_15;
end if;
   If_Result_17 := If_Result_16;
end if;
   If_Result_18 := If_Result_17;
end if;
   If_Result_19 := If_Result_18;
end if;
         Property_Result := If_Result_19;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Operator_Name;
--# end
--# property-start Name.is_write_reference 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12702'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Write_Reference
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_3148 with Inline_Always;
            procedure Finalizer_Scope_3151 with Inline_Always;
      Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity_Dotted_Name;
Var_Expr_0 : Internal_Entity_Base_Formal_Param_Holder;
Match_3 : Internal_Entity_Param_Assoc;
Match_4 : Internal_Entity_Attribute_Ref;
Match_5 : Internal_Entity_Alternatives_List;
Match_6 : Internal_Entity;
Var_Expr_230 : Internal_Entity_Param_Spec;
Var_Expr_231 : Internal_Entity_Mode;
Var_Expr_232 : Internal_Entity_Call_Expr;
Var_Expr_233 : Internal_Entity_Param_Spec;
Var_Expr_234 : Internal_Entity_Mode;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_6 : Internal_Entity_Base_Id;
Fld_7 : Internal_Entity_Base_Formal_Param_Holder;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_6 : Internal_Entity_Param_Spec;
Fld_12 : Internal_Entity_Mode;
Fld_13 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_14 : Internal_Entity;
Fld_15 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Defining_Name_Array_Access;
Fld_18 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Param_Spec;
Fld_19 : Internal_Entity_Mode;
Fld_20 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Quantifier_Result : Boolean;
Fld_21 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_22 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_23 : Internal_Entity;
Fld_24 : Internal_Entity;
Cast_Result_9 : Internal_Entity_Call_Expr;
Fld_25 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
If_Result_4 : Boolean;
Fld_26 : Internal_Entity_Name;
Cast_Result_10 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Name;
Cast_Result_11 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_27 : Boolean;
If_Result_5 : Boolean;
Fld_28 : Internal_Entity;
Is_A : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_12 : Internal_Entity_Assign_Stmt;
Let_Result : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_13 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_14 : Internal_Entity_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_15 : Internal_Entity_Param_Assoc;
Let_Result_3 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_16 : Internal_Entity_Attribute_Ref;
Let_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_17 : Internal_Entity_Alternatives_List;
Let_Result_5 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_18 : Internal_Entity;
Let_Result_6 : Boolean;
Scope_Result_12 : Boolean;
Match_Result : Boolean;
            procedure Finalizer_Scope_3148 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_3148;
            procedure Finalizer_Scope_3151 is
            begin
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_3151;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8786 '<Match at nodes.lkt:12702:38>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12702'
--# expr-start 8743 '<parent at nodes.lkt:12702:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12702'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8743
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_12; 
--# bind a Match_0
--# expr-start 8745 '<Eq at nodes.lkt:12705:31>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12705'
--# expr-start 8744 '<FieldAccess for dest at nodes.lkt:12705:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12705'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 8744
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 8745
Let_Result := Is_Equiv; 
--# end
Scope_Result_6 := Let_Result;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_13; 
--# bind c Match_1
--# expr-start 8747 '<Eq at nodes.lkt:12709:29>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12709'
--# expr-start 8746 '<FieldAccess for name at nodes.lkt:12709:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12709'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 8746
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 8747
if Is_Equiv_1 then
   --# expr-start 8748 '<FieldAccess for is_write_reference at nodes.lkt:12711:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12711'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_3 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 8748
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Let_Result_1 := If_Result; 
--# end
Scope_Result_7 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_14; 
--# bind d Match_2
--# expr-start 8749 '<FieldAccess for is_write_reference at nodes.lkt:12721:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12721'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_4 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2.Info);
--# end
--# expr-done 8749
if Fld_4 then
   If_Result_3 := True;
else
--# expr-start 8751 '<Eq at nodes.lkt:12724:13>' Is_Equiv_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12724'
--# expr-start 8750 '<FieldAccess for prefix at nodes.lkt:12724:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12724'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Match_2.Node.Dotted_Name_F_Prefix, Info => Match_2.Info);
--# expr-done 8750
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 8751
if Is_Equiv_2 then
--# expr-start 8753 '<FieldAccess for called_subp_spec at nodes.lkt:12724:34>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12724'
--# expr-start 8752 '<FieldAccess for suffix at nodes.lkt:12724:34>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12724'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Base_Id (Node => Match_2.Node.Dotted_Name_F_Suffix, Info => Match_2.Info);
--# expr-done 8752
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.called_subp_spec
Fld_7 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 8753
Var_Expr_0 := Fld_7;
if Var_Expr_0 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 8756 '<FieldAccess for dottable_subp at nodes.lkt:12725:27>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
--# expr-start 8755 '<FieldAccess for md at nodes.lkt:12725:27>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
--# expr-start 8754 '<FieldAccess for info at nodes.lkt:12725:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
Fld_8 := Var_Expr_0.Info;
--# expr-done 8754
Fld_9 := Fld_8.Md;
--# expr-done 8755
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 8756
if Fld_10 then
--# expr-start 8759 '<Cast to ParamSpec at nodes.lkt:12725:58>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
--# expr-start 8758 '<at at nodes.lkt:12725:58>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
--# expr-start 8757 '<FieldAccess for abstract_formal_params at nodes.lkt:12725:58>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8757
Get_Result := Get (Self, Fld_11, 0, True); 
--# expr-done 8758
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Param_Spec
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_6 := No_Entity_Param_Spec;
   end if;
--# expr-done 8759
Var_Expr_230 := Cast_Result_6;
if Var_Expr_230 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 8760 '<FieldAccess for mode at nodes.lkt:12725:58>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
      if Var_Expr_230.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Mode (Node => Var_Expr_230.Node.Param_Spec_F_Mode, Info => Var_Expr_230.Info);
--# expr-done 8760
Var_Expr_231 := Fld_12;
if Var_Expr_231 /= No_Entity_Mode then
   --# scope-start
--# expr-start 8761 '<FieldAccess for is_writable at nodes.lkt:12725:58>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12725'
      if Var_Expr_231.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Mode.is_writable
Fld_13 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_231.Node);
--# end
--# expr-done 8761
Scope_Result := Fld_13;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_3148;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result_2 := If_Result_3; 
--# end
Scope_Result_8 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_15; 
--# bind p Match_3
--# expr-start 8780 '<If at nodes.lkt:12730:31>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12730'
--# expr-start 8769 '<bind at nodes.lkt:12731:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12731'
--# expr-start 8762 '<FieldAccess for node_env at nodes.lkt:12731:24>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12731'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 8762
Bound_Env := Fld_21; Inc_Ref (Bound_Env);
--# expr-start 8768 '<bind at nodes.lkt:12732:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12732'
--# expr-start 8763 '<FieldAccess for origin_node at nodes.lkt:12732:27>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12732'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_22 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8763
Bound_Origin := Fld_22; 
--# expr-start 8766 '<Cast to CallExpr at nodes.lkt:12734:13>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12734'
--# expr-start 8765 '<parent at nodes.lkt:12734:13>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12734'
--# expr-start 8764 '<parent at nodes.lkt:12734:13>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12734'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Parent (Node => Match_3.Node, E_Info => Match_3.Info);
--# expr-done 8764
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Parent (Node => Fld_23.Node, E_Info => Fld_23.Info);
--# expr-done 8765
   if Fld_24.Node = null
      or else Fld_24.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Call_Expr
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
   else
         Cast_Result_9 := No_Entity_Call_Expr;
   end if;
--# expr-done 8766
Var_Expr_232 := Cast_Result_9;
if Var_Expr_232 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8767 '<FieldAccess for is_type_conversion at nodes.lkt:12734:13>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12734'
      if Var_Expr_232.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_25 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Var_Expr_232.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Var_Expr_232.Info);
--# end
--# expr-done 8767
Scope_Result_5 := Fld_25;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_5; 
--# expr-done 8768
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 8769
if Dyn_Var_Bind_Result_1 then
   --# expr-start 8773 '<FieldAccess for is_write_reference at nodes.lkt:12735:16>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
--# expr-start 8772 '<Cast to CallExpr at nodes.lkt:12735:16>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
--# expr-start 8771 '<parent at nodes.lkt:12735:16>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
--# expr-start 8770 '<parent at nodes.lkt:12735:16>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Parent (Node => Match_3.Node, E_Info => Match_3.Info);
--# expr-done 8770
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 8771
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 8772
      if Cast_Result_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_write_reference
Fld_16 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Cast_Result_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 8773
   If_Result_4 := Fld_16;
else
   --# expr-start 8779 '<AnyQuantifier at nodes.lkt:12735:71>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
   --# expr-start 8774 '<FieldAccess for get_params at nodes.lkt:12735:71>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12735'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicAssoc.get_params
Fld_17 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Match_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_3.Info);
--# end
--# expr-done 8774
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_17;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_1
--# expr-start 8776 '<Cast to ParamSpec at nodes.lkt:12736:20>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12736'
--# expr-start 8775 '<FieldAccess for basic_decl at nodes.lkt:12736:20>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12736'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_18 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 8775
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Param_Spec
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_8 := No_Entity_Param_Spec;
   end if;
--# expr-done 8776
Var_Expr_233 := Cast_Result_8;
if Var_Expr_233 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 8777 '<FieldAccess for mode at nodes.lkt:12736:20>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12736'
      if Var_Expr_233.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Mode (Node => Var_Expr_233.Node.Param_Spec_F_Mode, Info => Var_Expr_233.Info);
--# expr-done 8777
Var_Expr_234 := Fld_19;
if Var_Expr_234 /= No_Entity_Mode then
   --# scope-start
--# expr-start 8778 '<FieldAccess for is_writable at nodes.lkt:12736:20>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12736'
      if Var_Expr_234.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Mode.is_writable
Fld_20 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_234.Node);
--# end
--# expr-done 8778
Scope_Result_3 := Fld_20;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
      Quantifier_Result := Result_Var_4;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 8779
   If_Result_4 := Quantifier_Result;
end if;
--# expr-done 8780
Let_Result_3 := If_Result_4; 
--# end
Scope_Result_9 := Let_Result_3;
   --# end
      Finalizer_Scope_3151;
            Match_Result := Scope_Result_9; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_16; 
--# bind a Match_4
--# expr-start 8782 '<Eq at nodes.lkt:12740:33>' Is_Equiv_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12740'
--# expr-start 8781 '<FieldAccess for prefix at nodes.lkt:12740:33>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12740'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Name (Node => Match_4.Node.Attribute_Ref_F_Prefix, Info => Match_4.Info);
--# expr-done 8781
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
Cast_Expr_3 := Ent; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_3 := Equivalent (Cast_Result_10, Cast_Result_11); 
--# expr-done 8782
if Is_Equiv_3 then
   --# expr-start 8783 '<FieldAccess for is_access_attr at nodes.lkt:12740:54>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12740'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_27 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Match_4.Node);
--# end
--# expr-done 8783
   If_Result_5 := Fld_27;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_10 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Alternatives_List =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Create_Internal_Entity_Alternatives_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_17; 
--# bind l Match_5
--# expr-start 8785 '<IsA AggregateAssoc at nodes.lkt:12741:37>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12741'
--# expr-start 8784 '<parent at nodes.lkt:12741:37>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12741'
      if Match_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Parent (Node => Match_5.Node, E_Info => Match_5.Info);
--# expr-done 8784
Is_A := Fld_28.Node /= null 
and then Fld_28.Node.Kind in Ada_Aggregate_Assoc_Range; 
--# expr-done 8785
Let_Result_5 := Is_A; 
--# end
Scope_Result_11 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Abort_Absent .. Ada_Abstract_State_Decl_List | Ada_Constraint_List .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Array_Subcomponent_Choice_Name | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_18; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_12 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_12; 
end case;
--# expr-done 8786
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3148;
                     Finalizer_Scope_3151;
            raise;
      end;
   return Property_Result;
end Name_P_Is_Write_Reference;
--# end
--# property-start Name.potential_actuals_for_dispatch 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12749'
pragma Warnings (Off, "is not referenced");
 function Name_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Name
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_237 with Inline_Always;
            procedure Finalizer_Scope_3159 with Inline_Always;
            procedure Finalizer_Scope_3164 with Inline_Always;
            procedure Finalizer_Scope_3165 with Inline_Always;
            procedure Finalizer_Scope_3166 with Inline_Always;
      Local_Prefix : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr_0 : Boolean;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Identifier;
Match_3 : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Local_Args : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr_235 : Internal_Entity_Call_Expr;
Var_Expr_236 : Internal_Entity_Assoc_List;
Fld : Boolean;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Dotted_Name;
Scope_Result : Internal_Entity_Dotted_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Dotted_Name;
Scope_Result_1 : Internal_Entity_Dotted_Name;
Cast_Result_4 : Internal_Entity_Identifier;
Let_Result_2 : Internal_Entity_Dotted_Name;
Scope_Result_2 : Internal_Entity_Dotted_Name;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Internal_Entity_Dotted_Name;
Scope_Result_3 : Internal_Entity_Dotted_Name;
Match_Result : Internal_Entity_Dotted_Name;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Name;
Cast_Result_6 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Array_Lit : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_5 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Cast_Expr : Internal_Entity_Name;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Param_Actual_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Type_Expr;
Fld_12 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Map_Result : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_6 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_2 : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_7 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_3 : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result_5 : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_237 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Prefix);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Local_Args);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_237;
            procedure Finalizer_Scope_3159 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3159;
            procedure Finalizer_Scope_3164 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3164;
            procedure Finalizer_Scope_3165 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_3165;
            procedure Finalizer_Scope_3166 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3166;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8810 '<Let at nodes.lkt:12753:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12753'
--# scope-start
--# expr-start 8787 '<FieldAccess for is_dot_call at nodes.lkt:12753:22>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12753'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8787
Var_Expr_0 := Fld;
if Var_Expr_0 /= False then
   --# scope-start
   --# bind _ Var_Expr_0
--# expr-start 8792 '<Match at nodes.lkt:12754:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12754'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind c Match_0
--# expr-start 8789 '<Cast to DottedName at nodes.lkt:12755:37>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12755'
--# expr-start 8788 '<FieldAccess for name at nodes.lkt:12755:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12755'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 8788
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8789
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind d Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Identifier
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind i Match_2
--# expr-start 8791 '<Cast to DottedName at nodes.lkt:12757:39>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12757'
--# expr-start 8790 '<parent at nodes.lkt:12757:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12757'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Match_2.Node, E_Info => Match_2.Info);
--# expr-done 8790
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8791
Let_Result_2 := Cast_Result_1; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Dotted_Name; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 8792
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 8799 '<ArrayLiteral at nodes.lkt:12760:24>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12760'
--# expr-start 8798 '<New ExpectedTypeForExpr at nodes.lkt:12760:25>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12760'
--# expr-start 8796 '<FieldAccess for type_expression at nodes.lkt:12761:35>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12761'
--# expr-start 8795 '<FieldAccess for formal_decl at nodes.lkt:12761:35>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12761'
--# expr-start 8794 '<at at nodes.lkt:12761:35>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12761'
--# expr-start 8793 '<FieldAccess for unpacked_formal_params at nodes.lkt:12761:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12761'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 8793
Get_Result := Get (Self, Fld_3, 0, True); 
--# expr-done 8794
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 8795
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 8796
--# expr-start 8797 '<FieldAccess for prefix at nodes.lkt:12761:107>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12761'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 8797
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
New_Struct := (Expected_Type => Fld_5, Expr => Cast_Result_6); 
--# expr-done 8798
Array_Lit := Create_Internal_Expected_Type_For_Expr_Array (Internal_Internal_Expected_Type_For_Expr_Array'(1 => New_Struct)); 
--# expr-done 8799
Scope_Result_4 := Array_Lit;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3164;
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_5 := Result_Var;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3159;
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Local_Prefix := Result_Var_1; Inc_Ref (Local_Prefix);
--# bind prefix Local_Prefix
--# expr-start 8809 '<Let at nodes.lkt:12767:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12767'
--# scope-start
--# expr-start 8800 '<Cast to CallExpr at nodes.lkt:12767:20>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12767'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 8800
Var_Expr_235 := Cast_Result_7;
if Var_Expr_235 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8801 '<FieldAccess for params at nodes.lkt:12767:20>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12767'
      if Var_Expr_235.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_235.Node, E_Info => Var_Expr_235.Info);
--# end
--# expr-done 8801
Var_Expr_236 := Fld_7;
if Var_Expr_236 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 8808 '<Map at nodes.lkt:12767:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12767'
   --# expr-start 8802 '<FieldAccess for zip_with_params at nodes.lkt:12767:20>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12767'
      if Var_Expr_236.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AssocList.zip_with_params
Fld_8 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Var_Expr_236.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_236.Info);
--# end
--# expr-done 8802
      declare
         Map_Result_Vec : Internal_Expected_Type_For_Expr_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Fld_8;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_2
      --# expr-start 8807 '<New ExpectedTypeForExpr at nodes.lkt:12768:21>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12768'
--# expr-start 8805 '<FieldAccess for type_expression at nodes.lkt:12769:31>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12769'
--# expr-start 8804 '<FieldAccess for basic_decl at nodes.lkt:12769:31>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12769'
--# expr-start 8803 '<FieldAccess for param at nodes.lkt:12769:31>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12769'
Fld_9 := Item_2.Param;
--# expr-done 8803
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_10 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 8804
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 8805
--# expr-start 8806 '<FieldAccess for actual at nodes.lkt:12769:77>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12769'
Fld_12 := Item_2.Actual;
--# expr-done 8806
New_Struct_1 := (Expected_Type => Fld_11, Expr => Fld_12); 
--# expr-done 8807
         declare
            Item_To_Append : constant Internal_Expected_Type_For_Expr := New_Struct_1;
         begin
            Internal_Expected_Type_For_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Expected_Type_For_Expr_Array
           (Items_Count => Natural (Internal_Expected_Type_For_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Expected_Type_For_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Expected_Type_For_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Expected_Type_For_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8808
Scope_Result_6 := Map_Result;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3166;
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_7 := Result_Var_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3165;
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Local_Args := Result_Var_3; Inc_Ref (Local_Args);
--# bind args Local_Args
Concat_Result := Concat (Local_Prefix, Local_Args); 
Let_Result_4 := Concat_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8809
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 8810
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_237;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_237;
                     Finalizer_Scope_3159;
                     Finalizer_Scope_3164;
                     Finalizer_Scope_3165;
                     Finalizer_Scope_3166;
            raise;
      end;
   return Property_Result;
end Name_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Name.is_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12777'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dispatching_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Name;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 8811 '<FieldAccess for is_access_call at nodes.lkt:12777:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12777'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_access_call
Fld := Libadalang.Implementation.Name_P_Is_Access_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8811
if Fld then
   If_Result_1 := True;
else
--# expr-start 8812 '<FieldAccess for is_direct_call at nodes.lkt:12778:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8812
if Fld_1 then
   --# expr-start 8816 '<FieldAccess for is_dispatching_call_impl at nodes.lkt:12778:35>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12778'
--# expr-start 8814 '<Cast to Name at nodes.lkt:12778:36>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12778'
--# expr-start 8813 '<FieldAccess for parent_callexpr at nodes.lkt:12778:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_callexpr
Fld_2 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8813
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 8814
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
      if Result_Var.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8815 '<FieldAccess for referenced_decl at nodes.lkt:12778:103>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8815
--# property-call-start Expr.is_dispatching_call_impl
Fld_4 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Result_Var.Node, Decl => Fld_3, Imprecise_Fallback => Imprecise_Fallback, E_Info => Result_Var.Info);
--# end
--# expr-done 8816
   If_Result := Fld_4;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Dispatching_Call;
--# end
--# property-start Name.is_static_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12789'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 8817 '<FieldAccess for is_call at nodes.lkt:12790:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12790'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8817
if Fld then
   --# expr-start 8819 '<Not at nodes.lkt:12790:28>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12790'
--# expr-start 8818 '<FieldAccess for is_dispatching_call at nodes.lkt:12790:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12790'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 8818
Not_Val := not (Fld_1); 
--# expr-done 8819
   If_Result := Not_Val;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Static_Call;
--# end
--# property-start Name.as_single_tok_node_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12796'
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Single_Tok_Node_Array
  (Node : Bare_Name
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_240 with Inline_Always;
            procedure Finalizer_Scope_3169 with Inline_Always;
            procedure Finalizer_Scope_3170 with Inline_Always;
            procedure Finalizer_Scope_3171 with Inline_Always;
      Match_0 : Bare_Defining_Name;
Match_1 : Bare_Single_Tok_Node;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Array_Lit : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Bare_Name;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Bare_Base_Id;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Defining_Name;
Let_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Cast_Result_1 : Bare_Single_Tok_Node;
Let_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_2 : Bare_Dotted_Name;
Let_Result_2 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_3 : Bare_Name;
Let_Result_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Match_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_240 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_240;
            procedure Finalizer_Scope_3169 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3169;
            procedure Finalizer_Scope_3170 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3170;
            procedure Finalizer_Scope_3171 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_3171;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8827 '<Match at nodes.lkt:12796:60>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12796'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind dname Match_0
--# expr-start 8821 '<FieldAccess for as_single_tok_node_array at nodes.lkt:12797:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12797'
--# expr-start 8820 '<FieldAccess for name at nodes.lkt:12797:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12797'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Match_0.Defining_Name_F_Name;
--# expr-done 8820
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld);
--# end
--# expr-done 8821
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3169;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind tok Match_1
--# expr-start 8822 '<ArrayLiteral at nodes.lkt:12798:36>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12798'
Array_Lit := Create_Bare_Single_Tok_Node_Array (Internal_Bare_Single_Tok_Node_Array'(1 => Match_1)); 
--# expr-done 8822
Let_Result_1 := Array_Lit; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3170;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind dot Match_2
--# expr-start 8824 '<FieldAccess for as_single_tok_node_array at nodes.lkt:12799:33>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12799'
--# expr-start 8823 '<FieldAccess for prefix at nodes.lkt:12799:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12799'
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 8823
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_3 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_2);
--# end
--# expr-done 8824
--# expr-start 8826 '<FieldAccess for as_single_tok_node_array at nodes.lkt:12799:73>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12799'
--# expr-start 8825 '<FieldAccess for suffix at nodes.lkt:12799:73>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12799'
      if Match_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 8825
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_4);
--# end
--# expr-done 8826
Concat_Result := Concat (Fld_3, Fld_5); 
Let_Result_2 := Concat_Result; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3171;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Precondition_Failure with "PropertyError exception"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8827
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_240;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_240;
                     Finalizer_Scope_3169;
                     Finalizer_Scope_3170;
                     Finalizer_Scope_3171;
            raise;
      end;
   return Property_Result;
end Name_P_As_Single_Tok_Node_Array;
--# end
--# property-start Name.call_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12806'
pragma Warnings (Off, "is not referenced");
 function Name_P_Call_Params
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_241 with Inline_Always;
            procedure Finalizer_Scope_3179 with Inline_Always;
            procedure Finalizer_Scope_3180 with Inline_Always;
            procedure Finalizer_Scope_3181 with Inline_Always;
            procedure Finalizer_Scope_3184 with Inline_Always;
      Local_Formal_Spec : Internal_Entity_Base_Formal_Param_Holder;
Local_Is_Call : Boolean;
Local_Is_Prefix_Call : Boolean;
Local_Offset : Integer;
Local_Call_Prefix : Internal_Entity_Expr;
Var_Expr_0 : Internal_Entity_Call_Expr;
Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Local_Aparams : Internal_Entity_Assoc_List;
Local_Fparams : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Local_Actual_Formals : Internal_Entity_Defining_Name_Array_Access;
Local_Dparams : Internal_Param_Actual_Array_Access;
Var_Expr_4 : Internal_Entity_Assoc_List;
Var_Expr : Internal_Entity_Name;
Var_Expr_237 : Internal_Entity_Call_Expr;
Var_Expr_238 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_239 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_240 : Internal_Entity_Param_Spec;
Fld : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Boolean;
Is_A : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Integer;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_2 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Scope_Result_1 : Internal_Entity_Name;
Result_Var_1 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Attribute_Ref;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Expr;
Scope_Result_3 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_2 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Match_Result : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_6 : Internal_Entity_Call_Expr;
Fld_5 : Internal_Entity_Assoc_List;
Scope_Result_5 : Internal_Entity_Assoc_List;
Result_Var_2 : Internal_Entity_Assoc_List;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Base_Formal_Param_Holder;
Fld_8 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_7 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_4 : Internal_Entity_Defining_Name_Array_Access;
Index_2 : Integer;
Fld_9 : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_10 : Internal_Entity_Expr;
Scope_Result_8 : Internal_Entity_Expr;
Result_Var_5 : Internal_Entity_Expr;
Is_Equal : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result : Internal_Param_Actual_Array_Access;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Index_6 : Integer;
Get_Result : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Defining_Name;
Arith_Result : Integer;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Internal_Entity_Expr;
Is_Equal_1 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_9 : Internal_Param_Actual_Array_Access;
Result_Var_6 : Internal_Param_Actual_Array_Access;
Let_Result_3 : Internal_Param_Actual_Array_Access;
If_Result_7 : Internal_Param_Actual_Array_Access;
Let_Result_4 : Internal_Param_Actual_Array_Access;
Let_Result_5 : Internal_Param_Actual_Array_Access;
Let_Result_6 : Internal_Param_Actual_Array_Access;
Let_Result_7 : Internal_Param_Actual_Array_Access;
Let_Result_8 : Internal_Param_Actual_Array_Access;
Let_Result_9 : Internal_Param_Actual_Array_Access;
Let_Result_10 : Internal_Param_Actual_Array_Access;
Let_Result_11 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_241 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Local_Fparams);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Local_Actual_Formals);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Dparams);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Let_Result_8);
                     Dec_Ref (Let_Result_9);
                     Dec_Ref (Let_Result_10);
                     Dec_Ref (Let_Result_11);
            end Finalizer_Scope_241;
            procedure Finalizer_Scope_3179 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3179;
            procedure Finalizer_Scope_3180 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3180;
            procedure Finalizer_Scope_3181 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3181;
            procedure Finalizer_Scope_3184 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_3184;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 8872 '<Let at nodes.lkt:12807:9>' Let_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12807'
--# scope-start
--# expr-start 8828 '<FieldAccess for called_formal_subp_spec at nodes.lkt:12807:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12807'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.called_formal_subp_spec'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Called_Formal_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8828
Local_Formal_Spec := Fld; 
--# bind formal_spec Local_Formal_Spec
--# expr-start 8871 '<Let at nodes.lkt:12808:9>' Let_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12808'
--# scope-start
--# expr-start 8830 '<Not at nodes.lkt:12808:23>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12808'
--# expr-start 8829 '<IsNull at nodes.lkt:12808:27>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12808'
Is_Null := Local_Formal_Spec.Node = null; 
--# expr-done 8829
Not_Val := not (Is_Null); 
--# expr-done 8830
Local_Is_Call := Not_Val; 
--# bind is_call Local_Is_Call
--# expr-start 8870 '<Let at nodes.lkt:12809:9>' Let_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12809'
--# scope-start
--# expr-start 8833 '<If at nodes.lkt:12809:30>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12809'
if Local_Is_Call then
--# expr-start 8831 '<FieldAccess for is_dot_call at nodes.lkt:12809:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12809'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 8831
if Fld_1 then
   If_Result := True;
else
   --# expr-start 8832 '<IsA AttributeRef at nodes.lkt:12809:68>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12809'
Is_A := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 8832
   If_Result := Is_A;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 8833
Local_Is_Prefix_Call := If_Result_1; 
--# bind is_prefix_call Local_Is_Prefix_Call
--# expr-start 8869 '<Let at nodes.lkt:12810:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12810'
--# scope-start
--# expr-start 8834 '<If at nodes.lkt:12810:22>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12810'
if Local_Is_Prefix_Call then
   If_Result_2 := 1;
else
   If_Result_2 := 0;
end if;
--# expr-done 8834
Local_Offset := If_Result_2; 
--# bind offset Local_Offset
--# expr-start 8868 '<Let at nodes.lkt:12811:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12811'
--# scope-start
--# expr-start 8839 '<Match at nodes.lkt:12811:27>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12811'
--# expr-start 8835 '<Cast to CallExpr at nodes.lkt:12811:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12811'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 8835
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_0
--# expr-start 8836 '<FieldAccess for name at nodes.lkt:12811:62>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12811'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Call_Expr_F_Name, Info => Var_Expr_0.Info);
--# expr-done 8836
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Ent;
end if;
      if Result_Var_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Result_Var_1; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 8837 '<FieldAccess for prefix at nodes.lkt:12812:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12812'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 8837
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ar Match_1
--# expr-start 8838 '<FieldAccess for prefix at nodes.lkt:12813:38>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12813'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 8838
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Let_Result_1 := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Expr; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 8839
Local_Call_Prefix := Match_Result; 
--# bind call_prefix Local_Call_Prefix
--# expr-start 8867 '<Let at nodes.lkt:12817:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12817'
--# scope-start
--# expr-start 8840 '<Cast to CallExpr at nodes.lkt:12817:23>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12817'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_6 := No_Entity_Call_Expr;
   end if;
--# expr-done 8840
Var_Expr_237 := Cast_Result_6;
if Var_Expr_237 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8841 '<FieldAccess for params at nodes.lkt:12817:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12817'
      if Var_Expr_237.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_5 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_237.Node, E_Info => Var_Expr_237.Info);
--# end
--# expr-done 8841
Scope_Result_5 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Entity_Assoc_List;
end if;
Local_Aparams := Result_Var_2; 
--# bind aparams Local_Aparams
--# expr-start 8866 '<Let at nodes.lkt:12819:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12819'
--# scope-start
Var_Expr_238 := Local_Formal_Spec;
if Var_Expr_238 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 8842 '<FieldAccess for abstract_formal_params at nodes.lkt:12819:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12819'
      if Var_Expr_238.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_238.Node, E_Info => Var_Expr_238.Info);
--# end
--# expr-done 8842
Scope_Result_6 := Fld_6;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3179;
   Result_Var_3 := Scope_Result_6;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Local_Fparams := Result_Var_3; Inc_Ref (Local_Fparams);
--# bind fparams Local_Fparams
--# expr-start 8865 '<Let at nodes.lkt:12823:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12823'
--# scope-start
Var_Expr_239 := Local_Formal_Spec;
if Var_Expr_239 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 8844 '<FieldAccess for unpacked_formal_params at nodes.lkt:12823:30>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12823'
--# expr-start 8843 '<FieldAccess for corresponding_actual at nodes.lkt:12823:30>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12823'
      if Var_Expr_239.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.corresponding_actual'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Corresponding_Actual (Node => Var_Expr_239.Node, E_Info => Var_Expr_239.Info);
--# end
--# expr-done 8843
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 8844
Scope_Result_7 := Fld_8;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3180;
   Result_Var_4 := Scope_Result_7;
else
   Result_Var_4 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Local_Actual_Formals := Result_Var_4; Inc_Ref (Local_Actual_Formals);
--# bind actual_formals Local_Actual_Formals
--# expr-start 8864 '<If at nodes.lkt:12825:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12825'
if Local_Is_Call then
   --# expr-start 8863 '<Let at nodes.lkt:12828:13>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12828'
--# scope-start
--# expr-start 8852 '<Mapcat at nodes.lkt:12828:27>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12828'
      declare
         Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Local_Fparams;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
               --# bind i Index_2
      --# expr-start 8851 '<Map at nodes.lkt:12829:27>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12829'
   --# expr-start 8845 '<FieldAccess for defining_names at nodes.lkt:12829:27>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12829'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 8845
      declare
         Map_Result_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_9;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_3
      --# expr-start 8850 '<New ParamActual at nodes.lkt:12830:28>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12830'
--# expr-start 8849 '<If at nodes.lkt:12834:32>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12834'
if Local_Is_Prefix_Call then
   --# expr-start 8846 '<Eq at nodes.lkt:12834:54>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12834'
Is_Equal := Index_2 = 0; 
--# expr-done 8846
   If_Result_3 := Is_Equal;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := Local_Call_Prefix;
else
--# expr-start 8847 '<Cast to ParamSpec at nodes.lkt:12834:83>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12834'
Cast_Expr_2 := Item_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 8847
Var_Expr_240 := Cast_Result_7;
if Var_Expr_240 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 8848 '<FieldAccess for default_expr at nodes.lkt:12834:83>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12834'
      if Var_Expr_240.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Var_Expr_240.Node.Param_Spec_F_Default_Expr, Info => Var_Expr_240.Info);
--# expr-done 8848
Scope_Result_8 := Fld_10;
   --# end
   Result_Var_5 := Scope_Result_8;
else
   Result_Var_5 := No_Entity_Expr;
end if;
   If_Result_4 := Result_Var_5;
end if;
--# expr-done 8849
New_Struct := (Actual => If_Result_4, Param => Item_3); 
--# expr-done 8850
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8851
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3181;
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 8852
Local_Dparams := Map_Result_1; Inc_Ref (Local_Dparams);
--# bind dparams Local_Dparams
Var_Expr_4 := Local_Aparams;
if Var_Expr_4 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ap Var_Expr_4
--# expr-start 8862 '<Map at nodes.lkt:12842:25>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12842'
      declare
         Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
      begin
         Index_6 := 0;
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Local_Dparams;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_5
               --# bind i Index_6
      --# expr-start 8861 '<New ParamActual at nodes.lkt:12843:32>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12843'
--# expr-start 8859 '<If at nodes.lkt:12849:32>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
if Local_Is_Prefix_Call then
   --# expr-start 8853 '<Eq at nodes.lkt:12849:54>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
Is_Equal_1 := Index_6 = 0; 
--# expr-done 8853
   If_Result_5 := Is_Equal_1;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 8854 '<FieldAccess for actual at nodes.lkt:12849:66>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
Fld_11 := Item_5.Actual;
--# expr-done 8854
   If_Result_6 := Fld_11;
else
   --# expr-start 8858 '<FieldAccess for actual_for_param_at at nodes.lkt:12849:81>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8855 '<FieldAccess for param at nodes.lkt:12849:104>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
Fld_12 := Item_5.Param;
--# expr-done 8855
--# expr-start 8856 '<Arithmetic - at nodes.lkt:12849:114>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
Arith_Result := (Index_6 - Local_Offset); 
--# expr-done 8856
--# expr-start 8857 '<FieldAccess for actual at nodes.lkt:12849:126>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12849'
Fld_13 := Item_5.Actual;
--# expr-done 8857
--# property-call-start AssocList.actual_for_param_at
Fld_14 := Libadalang.Implementation.Assoc_List_P_Actual_For_Param_At (Node => Var_Expr_4.Node, Param => Fld_12, Pos => Arith_Result, Default_Expr => Fld_13, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 8858
   If_Result_6 := Fld_14;
end if;
--# expr-done 8859
--# expr-start 8860 '<at at nodes.lkt:12844:31>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12844'
Get_Result := Get (Self, Local_Actual_Formals, Index_6, True); 
--# expr-done 8860
New_Struct_1 := (Actual => If_Result_6, Param => Get_Result); 
--# expr-done 8861
         declare
            Item_To_Append : constant Internal_Param_Actual := New_Struct_1;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
               Index_6 := Index_6 + 1;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Param_Actual_Array
           (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 8862
Scope_Result_9 := Map_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_3184;
   Result_Var_6 := Scope_Result_9;
else
   Result_Var_6 := Local_Dparams;
end if;
      Inc_Ref (Result_Var_6);
Let_Result_3 := Result_Var_6; Inc_Ref (Let_Result_3);
--# end
--# expr-done 8863
   If_Result_7 := Let_Result_3;
else
   If_Result_7 := raise Precondition_Failure with "this name doesn't reference a call expression";
end if;
      Inc_Ref (If_Result_7);
--# expr-done 8864
Let_Result_4 := If_Result_7; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8865
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 8866
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 8867
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 8868
Let_Result_8 := Let_Result_7; Inc_Ref (Let_Result_8);
--# end
--# expr-done 8869
Let_Result_9 := Let_Result_8; Inc_Ref (Let_Result_9);
--# end
--# expr-done 8870
Let_Result_10 := Let_Result_9; Inc_Ref (Let_Result_10);
--# end
--# expr-done 8871
Let_Result_11 := Let_Result_10; Inc_Ref (Let_Result_11);
--# end
--# expr-done 8872
         Property_Result := Let_Result_11;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_241;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_241;
                     Finalizer_Scope_3179;
                     Finalizer_Scope_3180;
                     Finalizer_Scope_3181;
                     Finalizer_Scope_3184;
            raise;
      end;
   return Property_Result;
end Name_P_Call_Params;
--# end
--# property-start '[dispatcher]Name.parent_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.parent_scope
                  Property_Result := Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.parent_scope
                  Property_Result := Defining_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.parent_scope
                  Property_Result := Dotted_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.parent_scope
                  Property_Result := End_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.parent_scope
                  Property_Result := Base_Id_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Parent_Scope;
--# end
--# property-start '[dispatcher]Name.all_env_els_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.all_env_els_impl
                  Property_Result := Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.all_env_els_impl
                  Property_Result := Defining_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.all_env_els_impl
                  Property_Result := Dotted_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.all_env_els_impl
                  Property_Result := Base_Id_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_All_Env_Els_Impl;
--# end
--# property-start '[dispatcher]Name.scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.scope
                  Property_Result := Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.scope
                  Property_Result := Defining_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.scope
                  Property_Result := Dotted_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.scope
                  Property_Result := End_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.scope
                  Property_Result := Base_Id_P_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Scope;
--# end
--# property-start '[dispatcher]Name.ref_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name | Ada_Synthetic_Identifier =>
                  --# property-call-start Name.ref_var
                  Property_Result := Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name =>
                  --# property-call-start ArraySubcomponentChoiceName.ref_var
                  Property_Result := Array_Subcomponent_Choice_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.ref_var
                  Property_Result := Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.ref_var
                  Property_Result := Call_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.ref_var
                  Property_Result := Defining_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.ref_var
                  Property_Result := Dotted_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.ref_var
                  Property_Result := End_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.ref_var
                  Property_Result := Explicit_Deref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.ref_var
                  Property_Result := Qual_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Reduce_Attribute_Ref =>
                  --# property-call-start ReduceAttributeRef.ref_var
                  Property_Result := Reduce_Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.ref_var
                  Property_Result := Single_Tok_Node_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.ref_var
                  Property_Result := Target_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Update_Attribute_Ref =>
                  --# property-call-start UpdateAttributeRef.ref_var
                  Property_Result := Update_Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Ref_Var;
--# end
--# property-start '[dispatcher]Name.subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.subp_spec_var
                  Property_Result := Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.subp_spec_var
                  Property_Result := Call_Expr_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.subp_spec_var
                  Property_Result := Dotted_Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.subp_spec_var
                  Property_Result := Explicit_Deref_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.subp_spec_var
                  Property_Result := Single_Tok_Node_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.defines_subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.defines_subp_spec_var
                  Property_Result := Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.defines_subp_spec_var
                  Property_Result := Call_Expr_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.defines_subp_spec_var
                  Property_Result := Dotted_Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.defines_subp_spec_var
                  Property_Result := Explicit_Deref_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.defines_subp_spec_var
                  Property_Result := Single_Tok_Node_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.called_formal_subp_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Called_Formal_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.called_formal_subp_spec
                  Property_Result := Name_P_Called_Formal_Subp_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.called_formal_subp_spec
                  Property_Result := Attribute_Ref_P_Called_Formal_Subp_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Called_Formal_Subp_Spec;
--# end
--# property-start '[dispatcher]Name.designated_type_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Reduce_Attribute_Ref | Ada_Null_Literal .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.designated_type_impl
                  Property_Result := Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name =>
                  --# property-call-start ArraySubcomponentChoiceName.designated_type_impl
                  Property_Result := Array_Subcomponent_Choice_Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_type_impl
                  Property_Result := Attribute_Ref_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_type_impl
                  Property_Result := Call_Expr_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_type_impl
                  Property_Result := Dotted_Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_type_impl
                  Property_Result := Base_Id_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Designated_Type_Impl;
--# end
--# property-start '[dispatcher]Name.relative_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name | Ada_Discrete_Subtype_Name | Ada_Reduce_Attribute_Ref | Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.relative_name
                  Property_Result := Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.relative_name
                  Property_Result := Attribute_Ref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.relative_name
                  Property_Result := Call_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name .. Ada_Synthetic_Defining_Name =>
                  --# property-call-start DefiningName.relative_name
                  Property_Result := Defining_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.relative_name
                  Property_Result := Dotted_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.relative_name
                  Property_Result := End_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.relative_name
                  Property_Result := Explicit_Deref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.relative_name
                  Property_Result := Qual_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.relative_name
                  Property_Result := Single_Tok_Node_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Identifier =>
                  --# property-call-start SyntheticIdentifier.relative_name
                  Property_Result := Synthetic_Identifier_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.relative_name
                  Property_Result := Target_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Relative_Name;
--# end
--# property-start '[dispatcher]Name.name_symbol' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Defining_Name | Ada_Discrete_Subtype_Name .. Ada_Reduce_Attribute_Ref | Ada_Target_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.name_symbol
                  Property_Result := Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Array_Subcomponent_Choice_Name =>
                  --# property-call-start ArraySubcomponentChoiceName.name_symbol
                  Property_Result := Array_Subcomponent_Choice_Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Defining_Name =>
                  --# property-call-start SyntheticDefiningName.name_symbol
                  Property_Result := Synthetic_Defining_Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Identifier | Ada_String_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.name_symbol
                  Property_Result := Single_Tok_Node_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.name_symbol
                  Property_Result := Op_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Identifier =>
                  --# property-call-start SyntheticIdentifier.name_symbol
                  Property_Result := Synthetic_Identifier_P_Name_Symbol
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Name_Symbol;
--# end
--# property-start '[dispatcher]Name.as_symbol_array' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Defining_Name | Ada_Discrete_Subtype_Name .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.as_symbol_array
                  Property_Result := Name_P_As_Symbol_Array
                    (Self
                    );
                  --# end
               when Ada_Synthetic_Defining_Name =>
                  --# property-call-start SyntheticDefiningName.as_symbol_array
                  Property_Result := Synthetic_Defining_Name_P_As_Symbol_Array
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_As_Symbol_Array;
--# end
--# property-start '[dispatcher]Name.canonical_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Canonical_Text
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name .. Ada_Reduce_Attribute_Ref | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.canonical_text
                  Property_Result := Name_P_Canonical_Text
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.canonical_text
                  Property_Result := Single_Tok_Node_P_Canonical_Text
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Canonical_Text;
--# end
--# property-start '[dispatcher]Name.is_constant' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Is_Constant
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Array_Subcomponent_Choice_Name | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Reduce_Attribute_Ref .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Null_Literal | Ada_Synthetic_Identifier .. Ada_Update_Attribute_Ref =>
                  --# property-call-start Name.is_constant
                  Property_Result := Name_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.is_constant
                  Property_Result := Attribute_Ref_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.is_constant
                  Property_Result := Call_Expr_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.is_constant
                  Property_Result := Dotted_Name_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.is_constant
                  Property_Result := Explicit_Deref_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.is_constant
                  Property_Result := Qual_Expr_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start Identifier.is_constant
                  Property_Result := Identifier_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal .. Ada_Real_Literal =>
                  --# property-call-start NumLiteral.is_constant
                  Property_Result := Num_Literal_P_Is_Constant
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Name_P_Is_Constant;
--# end
--# property-start Name.parent_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12859'
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.parent_scope not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Parent_Scope;
--# end
--# property-start Name.all_env_els_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12862'
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.all_env_els_impl not implemented";
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_All_Env_Els_Impl;
--# end
--# property-start Name.scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12869'
pragma Warnings (Off, "is not referenced");
 function Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Scope;
--# end
--# property-start Name.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12877'
pragma Warnings (Off, "is not referenced");
 function Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.ref_var not implemented";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Ref_Var;
--# end
--# property-start Name.subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12881'
pragma Warnings (Off, "is not referenced");
 function Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.subp_spec_var not implemented";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Subp_Spec_Var;
--# end
--# property-start Name.defines_subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12883'
pragma Warnings (Off, "is not referenced");
 function Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.called_formal_subp_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12896'
pragma Warnings (Off, "is not referenced");
 function Name_P_Called_Formal_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name  := Bare_Name (Node);
     Ent : Internal_Entity_Name :=
       Internal_Entity_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_3 : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8878 '<If at nodes.lkt:12897:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12897'
--# expr-start 8873 '<FieldAccess for defines_subp_spec_var at nodes.lkt:12897:12>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12897'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.defines_subp_spec_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Defines_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 8873
if Fld_3 then
   --# expr-start 8877 '<Cast to BaseFormalParamHolder at nodes.lkt:12897:46>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12897'
--# expr-start 8876 '<FieldAccess for value at nodes.lkt:12897:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12897'
--# expr-start 8875 '<FieldAccess for logic_val at nodes.lkt:12897:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12897'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 8874 '<FieldAccess for subp_spec_var at nodes.lkt:12898:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12898'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 8874
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 8875
Fld_2 := Fld_1.Value;
--# expr-done 8876
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Formal_Param_Holder
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Formal_Param_Holder;
   end if;
--# expr-done 8877
   If_Result := Cast_Result_1;
else
   If_Result := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 8878
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Called_Formal_Subp_Spec;
--# end
--# property-start Name.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12906'
pragma Warnings (Off, "is not referenced");
 function Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Designated_Type_Impl;
--# end
--# property-start Name.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12912'
pragma Warnings (Off, "is not referenced");
 function Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.relative_name not implemented";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Relative_Name;
--# end
--# property-start Name.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12914'
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8881 '<FieldAccess for name_symbol at nodes.lkt:12915:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12915'
--# expr-start 8880 '<FieldAccess for relative_name at nodes.lkt:12915:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12915'
--# expr-start 8879 '<as_bare_entity at nodes.lkt:12915:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12915'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 8879
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 8880
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8881
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Name_Symbol;
--# end
--# property-start Name.as_symbol_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12925'
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_252 with Inline_Always;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_252 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_252;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8884 '<Map at nodes.lkt:12926:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12926'
   --# expr-start 8882 '<FieldAccess for as_single_tok_node_array at nodes.lkt:12926:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12926'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Self);
--# end
--# expr-done 8882
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 8883 '<GetSymbol at nodes.lkt:12926:52>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12926'
Sym := Get_Symbol (Item_0); 
--# expr-done 8883
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8884
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_252;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_252;
            raise;
      end;
   return Property_Result;
end Name_P_As_Symbol_Array;
--# end
--# property-start Name.canonical_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12933'
pragma Warnings (Off, "is not referenced");
 function Name_P_Canonical_Text
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_253 with Inline_Always;
      Fld : Symbol_Type_Array_Access;
Str : String_Type;
Fld_1 : String_Type;
Sym : Symbol_Type;
            procedure Finalizer_Scope_253 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Str);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_253;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8888 '<to_symbol at nodes.lkt:12934:9>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12934'
--# expr-start 8887 '<FieldAccess for sym_join at nodes.lkt:12934:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12934'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8885 '<FieldAccess for as_symbol_array at nodes.lkt:12934:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12934'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Self);
--# end
--# expr-done 8885
--# expr-start 8886 '<String at nodes.lkt:12934:47>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12934'
Str := Create_String ("."); 
--# expr-done 8886
--# property-call-start AdaNode.sym_join
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Self, Syms => Fld, Sep => Str);
--# end
--# expr-done 8887
Sym := String_To_Symbol (Self, Self.Unit.Context, Fld_1); 
--# expr-done 8888
         Property_Result := Sym;
   --# end
      Finalizer_Scope_253;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_253;
            raise;
      end;
   return Property_Result;
end Name_P_Canonical_Text;
--# end
--# property-start Name.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12938'
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Constant
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name  := Bare_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := raise Property_Error with "Property Name.is_constant not implemented";
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Name_P_Is_Constant;
--# end
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Name
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Name_P_Name_Designated_Type_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Children_Env (E.Node));
begin
      if E.Node /= null
         and then E.Node.Kind not in Ada_Name
      then
         Raise_Property_Exception
           (E.Node,
            Property_Error'Identity,
            "mismatching node type");
      end if;
   declare
      Result : constant Lexical_Env :=
         Name_P_Use_Package_Name_Designated_Env (E.Node, E_Info);
   begin
         return Result;
   end;
end;
   --
   --  Primitives for Bare_Array_Subcomponent_Choice_Name
   --
      procedure Initialize_Fields_For_Array_Subcomponent_Choice_Name
        (Self : Bare_Array_Subcomponent_Choice_Name
         ; Array_Subcomponent_Choice_Name_F_Name : Bare_Name
         ; Array_Subcomponent_Choice_Name_F_Suffix : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Array_Subcomponent_Choice_Name_F_Name := Array_Subcomponent_Choice_Name_F_Name;
            Self.Array_Subcomponent_Choice_Name_F_Suffix := Array_Subcomponent_Choice_Name_F_Suffix;
      end Initialize_Fields_For_Array_Subcomponent_Choice_Name;
   function Array_Subcomponent_Choice_Name_F_Name
     (Node : Bare_Array_Subcomponent_Choice_Name) return Bare_Name
   is
   begin
         return Node.Array_Subcomponent_Choice_Name_F_Name;
   end;
   function Array_Subcomponent_Choice_Name_F_Suffix
     (Node : Bare_Array_Subcomponent_Choice_Name) return Bare_Ada_Node
   is
   begin
         return Node.Array_Subcomponent_Choice_Name_F_Suffix;
   end;
--# property-start ArraySubcomponentChoiceName.suffix_exprs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14653'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Suffix_Exprs
  (Node : Bare_Array_Subcomponent_Choice_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
     Ent : Internal_Entity_Array_Subcomponent_Choice_Name :=
       Internal_Entity_Array_Subcomponent_Choice_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1276 with Inline_Always;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Cast_Expr : Internal_Entity_Basic_Assoc;
Cast_Result_1 : Internal_Entity_Param_Assoc;
Fld_3 : Internal_Entity_Expr;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Map_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_1276 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1276;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8893 '<Map at nodes.lkt:14654:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14654'
   --# expr-start 8890 '<Cast to AssocList at nodes.lkt:14654:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14654'
--# expr-start 8889 '<FieldAccess for suffix at nodes.lkt:14654:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14654'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Suffix, Info => Ent.Info);
--# expr-done 8889
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8890
Fld_1 := Cast_Result.Node;
      declare
         Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Cast_Result.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind a Item_0
      --# expr-start 8892 '<FieldAccess for expr at nodes.lkt:14655:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14655'
--# expr-start 8891 '<Cast to ParamAssoc at nodes.lkt:14655:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14655'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Param_Assoc_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Param_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 8891
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8892
         declare
            Item_To_Append : constant Internal_Entity_Expr := Fld_3;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Expr_Array
           (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8893
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1276;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1276;
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Suffix_Exprs;
--# end
--# property-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14660'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type
  (Node : Bare_Array_Subcomponent_Choice_Name
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1277 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Delta_Aggregate;
Fld : Bare_Expr;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1277 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1277;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Array_Subcomponent_Choice_Name_Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Array_Subcomponent_Choice_Name,
                                As_Bare_Array_Subcomponent_Choice_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 8900 '<FieldAccess for expression_type at nodes.lkt:14661:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
--# expr-start 8899 '<as_bare_entity at nodes.lkt:14661:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
--# expr-start 8898 '<FieldAccess for ancestor_expr at nodes.lkt:14661:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
--# expr-start 8897 '<Cast to DeltaAggregate at nodes.lkt:14661:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
--# expr-start 8896 '<Find at nodes.lkt:14661:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
   --# expr-start 8894 '<FieldAccess for parents at nodes.lkt:14661:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14661'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, False); 
--# expr-done 8894
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 8895 '<IsA DeltaAggregate at nodes.lkt:14662:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14662'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 8895
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 8896
   if Find_Result = null
      or else Find_Result.Kind in Ada_Delta_Aggregate_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8897
      if Cast_Result = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Cast_Result.Base_Aggregate_F_Ancestor_Expr;
--# expr-done 8898
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Expr;
else
As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 8899
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 8900
         Property_Result := Fld_1;
   --# end
      Finalizer_Scope_1277;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1277;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                        As_Internal_Entity_Base_Type_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type;
--# end
--# property-start ArraySubcomponentChoiceName.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14665'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Ref_Var
  (Node : Bare_Array_Subcomponent_Choice_Name
  )
   return Logic_Var
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      As_Entity : Internal_Entity_Array_Subcomponent_Choice_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Array_Subcomponent_Choice_Name;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8904 '<FieldAccess for ref_var at nodes.lkt:14666:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14666'
--# expr-start 8903 '<FieldAccess for name at nodes.lkt:14666:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14666'
--# expr-start 8902 '<FieldAccess for delta_aggregate_ancestor_expr_type at nodes.lkt:14666:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14666'
--# expr-start 8901 '<as_bare_entity at nodes.lkt:14666:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14666'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Array_Subcomponent_Choice_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 8901
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type
Fld := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type (Node => If_Result.Node);
--# end
--# expr-done 8902
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Base_Type_Decl_F_Name, Info => Fld.Info);
--# expr-done 8903
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_1.Node);
--# end
--# expr-done 8904
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Ref_Var;
--# end
--# property-start ArraySubcomponentChoiceName.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14668'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Name_Symbol
  (Node : Bare_Array_Subcomponent_Choice_Name
  )
   return Symbol_Type
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Var_Expr_0 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 8905 '<FieldAccess for name at nodes.lkt:14669:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14669'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Array_Subcomponent_Choice_Name_F_Name;
--# expr-done 8905
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 8906 '<FieldAccess for name_symbol at nodes.lkt:14670:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14670'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr_0);
--# end
--# expr-done 8906
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Name_Symbol;
--# end
--# property-start ArraySubcomponentChoiceName.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14673'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Xref_Equation
  (Node : Bare_Array_Subcomponent_Choice_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
     Ent : Internal_Entity_Array_Subcomponent_Choice_Name :=
       Internal_Entity_Array_Subcomponent_Choice_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1280 with Inline_Always;
            procedure Finalizer_Scope_3190 with Inline_Always;
            procedure Finalizer_Scope_3191 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_3 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_2 : Internal_Entity_Expr_Array_Access;
Index_2 : Integer;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Name;
Fld_10 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Internal_Entity_Name;
Fld_13 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1280 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1280;
            procedure Finalizer_Scope_3190 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3190;
            procedure Finalizer_Scope_3191 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3191;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8935 '<And at nodes.lkt:14675:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14675'
--# expr-start 8920 '<And at nodes.lkt:14675:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14675'
--# expr-start 8907 '<FieldAccess for name at nodes.lkt:14675:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14675'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8907
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 8908 '<FieldAccess for sub_equation at nodes.lkt:14676:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14676'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8908
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3190;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14676:50") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8919 '<LogicAll at nodes.lkt:14678:13>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14678'
--# expr-start 8918 '<Map at nodes.lkt:14678:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14678'
   --# expr-start 8909 '<FieldAccess for suffix_exprs at nodes.lkt:14678:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14678'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.suffix_exprs
Fld_2 := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Suffix_Exprs (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8909
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant Internal_Entity_Expr_Array_Access := Fld_2;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_1
               --# bind i Index_2
      --# expr-start 8917 '<And at nodes.lkt:14680:23>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14680'
--# expr-start 8915 '<And at nodes.lkt:14680:23>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14680'
--# expr-start 8910 '<FieldAccess for sub_equation at nodes.lkt:14680:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14680'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_1.Info);
--# end
--# expr-done 8910
--# expr-start 8914 '<Bind at nodes.lkt:14681:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14681'
--# expr-start 8911 '<FieldAccess for expected_type_var at nodes.lkt:14681:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14681'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Item_1.Node);
--# expr-done 8911
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 8913 '<FieldAccess for index_type at nodes.lkt:14682:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14682'
--# expr-start 8912 '<FieldAccess for delta_aggregate_ancestor_expr_type at nodes.lkt:14682:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14682'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type
Fld_5 := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type (Node => Self);
--# end
--# expr-done 8912
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Fld_5.Node, Dim => Index_2, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8913
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14681:17") else null)); 
--# expr-done 8914
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14680:23") else null)); 
--# expr-done 8915
--# expr-start 8916 '<FieldAccess for matches_expected_type at nodes.lkt:14683:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14683'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Item_1.Node, Origin => Origin);
--# end
--# expr-done 8916
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14680:23") else null)); 
--# expr-done 8917
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3191;
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 8918
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14678:13") else null)); 
--# expr-done 8919
And_Pred_2 := Create_And (Result_Var, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14675:9") else null)); 
--# expr-done 8920
--# expr-start 8934 '<If at nodes.lkt:14688:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14688'
--# expr-start 8922 '<IsA DottedName at nodes.lkt:14688:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14688'
--# expr-start 8921 '<FieldAccess for name at nodes.lkt:14688:16>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14688'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8921
Is_A := Fld_17.Node /= null 
and then Fld_17.Node.Kind in Ada_Dotted_Name_Range; 
--# expr-done 8922
if Is_A then
   --# expr-start 8926 '<Bind at nodes.lkt:14689:18>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14689'
--# expr-start 8924 '<FieldAccess for type_var at nodes.lkt:14689:39>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14689'
--# expr-start 8923 '<FieldAccess for name at nodes.lkt:14689:39>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14689'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8923
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_9.Node);
--# expr-done 8924
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 8925 '<FieldAccess for type_var at nodes.lkt:14689:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14689'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 8925
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result_1 := Solver.Create_Propagate (Fld_10, Fld_8, Logic_Functor_895'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14689:18") else null)); 
--# expr-done 8926
   If_Result := Bind_Result_1;
else
   --# expr-start 8933 '<Bind at nodes.lkt:14691:18>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14691'
--# expr-start 8927 '<FieldAccess for type_var at nodes.lkt:14691:22>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14691'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 8927
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 8928 '<FieldAccess for name at nodes.lkt:14692:22>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14692'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8928
Var_Expr_3 := Fld_12;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 8930 '<FieldAccess for comp_type at nodes.lkt:14693:33>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14693'
--# expr-start 8929 '<FieldAccess for name_designated_type at nodes.lkt:14693:33>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14693'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_13 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 8929
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Fld_13.Node, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 8930
Scope_Result_1 := Fld_14;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 8932 '<FieldAccess for comp_type at nodes.lkt:14695:26>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14695'
--# expr-start 8931 '<FieldAccess for delta_aggregate_ancestor_expr_type at nodes.lkt:14695:26>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14695'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type
Fld_15 := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type (Node => Self);
--# end
--# expr-done 8931
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Fld_15.Node, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 8932
   Result_Var_1 := Fld_16;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_11, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14691:18") else null)); 
--# expr-done 8933
   If_Result := Bind_Result_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 8934
And_Pred_3 := Create_And (And_Pred_2, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14675:9") else null)); 
--# expr-done 8935
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1280;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1280;
                     Finalizer_Scope_3190;
                     Finalizer_Scope_3191;
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Xref_Equation;
--# end
--# property-start ArraySubcomponentChoiceName.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14700'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Designated_Type_Impl
  (Node : Bare_Array_Subcomponent_Choice_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
     Ent : Internal_Entity_Array_Subcomponent_Choice_Name :=
       Internal_Entity_Array_Subcomponent_Choice_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_0 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 8936 '<FieldAccess for name at nodes.lkt:14701:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14701'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8936
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 8937 '<FieldAccess for designated_type_impl at nodes.lkt:14702:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14702'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8937
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8939 '<FieldAccess for comp_type at nodes.lkt:14703:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14703'
--# expr-start 8938 '<FieldAccess for delta_aggregate_ancestor_expr_type at nodes.lkt:14703:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14703'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type
Fld_2 := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type (Node => Self);
--# end
--# expr-done 8938
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8939
   Result_Var := Fld_3;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Designated_Type_Impl;
--# end
--# property-start ArraySubcomponentChoiceName.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14706'
pragma Warnings (Off, "is not referenced");
 function Array_Subcomponent_Choice_Name_P_Designated_Env
  (Node : Bare_Array_Subcomponent_Choice_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Array_Subcomponent_Choice_Name  := Bare_Array_Subcomponent_Choice_Name (Node);
     Ent : Internal_Entity_Array_Subcomponent_Choice_Name :=
       Internal_Entity_Array_Subcomponent_Choice_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1282 with Inline_Always;
            procedure Finalizer_Scope_3194 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Bound_Include_Ud_Indexing : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1282 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1282;
            procedure Finalizer_Scope_3194 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3194;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 8940 '<FieldAccess for name at nodes.lkt:14707:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14707'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Array_Subcomponent_Choice_Name_F_Name, Info => Ent.Info);
--# expr-done 8940
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_0
--# expr-start 8942 '<FieldAccess for defining_env at nodes.lkt:14708:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14708'
--# expr-start 8941 '<FieldAccess for name_designated_type at nodes.lkt:14708:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14708'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 8941
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_1.Info);
--# end
--# expr-done 8942
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3194;
   Result_Var := Scope_Result;
else
   --# expr-start 8945 '<bind at nodes.lkt:14710:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14710'
Bound_Include_Ud_Indexing := False; 
--# expr-start 8944 '<FieldAccess for defining_env at nodes.lkt:14712:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14712'
--# expr-start 8943 '<FieldAccess for delta_aggregate_ancestor_expr_type at nodes.lkt:14712:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14712'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArraySubcomponentChoiceName.delta_aggregate_ancestor_expr_type
Fld_3 := Libadalang.Implementation.Array_Subcomponent_Choice_Name_P_Delta_Aggregate_Ancestor_Expr_Type (Node => Self);
--# end
--# expr-done 8943
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 8944
Dyn_Var_Bind_Result := Fld_4; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8945
   Result_Var := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1282;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1282;
                     Finalizer_Scope_3194;
            raise;
      end;
   return Property_Result;
end Array_Subcomponent_Choice_Name_P_Designated_Env;
--# end
   --
   --  Primitives for Bare_Attribute_Ref
   --
      procedure Initialize_Fields_For_Attribute_Ref
        (Self : Bare_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Attribute_Ref_F_Prefix := Attribute_Ref_F_Prefix;
            Self.Attribute_Ref_F_Attribute := Attribute_Ref_F_Attribute;
            Self.Attribute_Ref_F_Args := Attribute_Ref_F_Args;
      Self.Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Attribute_Ref;
   function Attribute_Ref_F_Prefix
     (Node : Bare_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Attribute_Ref_F_Prefix;
   end;
   function Attribute_Ref_F_Attribute
     (Node : Bare_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Attribute_Ref_F_Attribute;
   end;
   function Attribute_Ref_F_Args
     (Node : Bare_Attribute_Ref) return Bare_Assoc_List
   is
   begin
         return Node.Attribute_Ref_F_Args;
   end;
--# property-start AttributeRef.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12948'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Ref_Var
  (Node : Bare_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8946 '<FieldAccess for r_ref_var at nodes.lkt:12948:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12948'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 8946
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Ref_Var;
--# end
--# property-start AttributeRef.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12950'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Relative_Name
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8948 '<FieldAccess for relative_name at nodes.lkt:12950:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12950'
--# expr-start 8947 '<FieldAccess for prefix at nodes.lkt:12950:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12950'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8947
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8948
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Relative_Name;
--# end
--# property-start AttributeRef.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12952'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Constant
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Is_Constant;
--# end
--# property-start AttributeRef.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12955'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Type_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_241 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Identifier;
Fld_7 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_8 : Bare_Identifier;
Fld_9 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8964 '<If at nodes.lkt:12956:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
--# expr-start 8952 '<Eq at nodes.lkt:12956:12>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
--# expr-start 8950 '<FieldAccess for sym at nodes.lkt:12956:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
--# expr-start 8949 '<FieldAccess for attribute at nodes.lkt:12956:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Attribute_Ref_F_Attribute;
--# expr-done 8949
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_9 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_8);
--# end
--# expr-done 8950
--# expr-start 8951 '<Symbol Class at nodes.lkt:12956:36>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 8951
Is_Equal_1 := Fld_9 = Sym_1; 
--# expr-done 8952
if Is_Equal_1 then
--# expr-start 8954 '<FieldAccess for designated_type_impl at nodes.lkt:12956:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
--# expr-start 8953 '<FieldAccess for prefix at nodes.lkt:12956:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8953
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8954
Var_Expr_241 := Fld_1;
if Var_Expr_241 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8955 '<FieldAccess for classwide_type at nodes.lkt:12956:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
      if Var_Expr_241.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr_241.Node, E_Info => Var_Expr_241.Info);
--# end
--# expr-done 8955
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_1;
else
   --# expr-start 8963 '<If at nodes.lkt:12956:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12956'
--# expr-start 8959 '<Eq at nodes.lkt:12957:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
--# expr-start 8957 '<FieldAccess for sym at nodes.lkt:12957:14>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
--# expr-start 8956 '<FieldAccess for attribute at nodes.lkt:12957:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Attribute_Ref_F_Attribute;
--# expr-done 8956
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_7 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_6);
--# end
--# expr-done 8957
--# expr-start 8958 '<Symbol Base at nodes.lkt:12957:38>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 8958
Is_Equal := Fld_7 = Sym; 
--# expr-done 8959
if Is_Equal then
--# expr-start 8962 '<FieldAccess for scalar_base_subtype at nodes.lkt:12957:51>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
--# expr-start 8961 '<FieldAccess for designated_type_impl at nodes.lkt:12957:51>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
--# expr-start 8960 '<FieldAccess for prefix at nodes.lkt:12957:51>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12957'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8960
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 8961
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 8962
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   If_Result := Cast_Result;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
--# expr-done 8963
   If_Result_1 := If_Result;
end if;
--# expr-done 8964
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Designated_Type_Impl;
--# end
--# property-start AttributeRef.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12960'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Has_Context_Free_Type
  (Node : Bare_Attribute_Ref
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8966 '<Not at nodes.lkt:12960:41>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12960'
--# expr-start 8965 '<FieldAccess for is_access_attr at nodes.lkt:12960:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Self);
--# end
--# expr-done 8965
Not_Val := not (Fld); 
--# expr-done 8966
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Has_Context_Free_Type;
--# end
--# property-start AttributeRef.is_access_attr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12962'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Access_Attr
  (Node : Bare_Attribute_Ref
  )
   return Boolean
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 8968 '<FieldAccess for name_symbol at nodes.lkt:12963:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12963'
--# expr-start 8967 '<FieldAccess for attribute at nodes.lkt:12963:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12963'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 8967
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 8968
--# expr-start 8969 '<Symbol Access at nodes.lkt:12963:41>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12963'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access); 
--# expr-done 8969
Is_Equal_2 := Fld_1 = Sym; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 8970 '<Symbol Unchecked_Access at nodes.lkt:12963:53>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12963'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Access); 
--# expr-done 8970
Is_Equal_1 := Fld_1 = Sym_1; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 8971 '<Symbol Unrestricted_Access at nodes.lkt:12963:75>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12963'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unrestricted_Access); 
--# expr-done 8971
Is_Equal := Fld_1 = Sym_2; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Is_Access_Attr;
--# end
--# property-start AttributeRef.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12966'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1160 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_6 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_7 : Boolean;
Fld_8 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_9 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1160 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_1160;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 8990 '<If at nodes.lkt:12967:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
--# expr-start 8972 '<FieldAccess for is_access_attr at nodes.lkt:12967:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_7 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Ent.Node);
--# end
--# expr-done 8972
if Fld_7 then
   If_Result_2 := True;
else
   --# expr-start 8975 '<FieldAccess for name_is at nodes.lkt:12967:37>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
--# expr-start 8973 '<FieldAccess for attribute at nodes.lkt:12967:37>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8973
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8974 '<Symbol Old at nodes.lkt:12967:60>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Old); 
--# expr-done 8974
--# property-call-start Name.name_is
Fld_9 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_8.Node, Sym => Sym_1);
--# end
--# expr-done 8975
   If_Result_2 := Fld_9;
end if;
if If_Result_2 then
   --# expr-start 8977 '<FieldAccess for designated_env at nodes.lkt:12968:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12968'
--# expr-start 8976 '<FieldAccess for prefix at nodes.lkt:12968:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12968'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8976
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 8977
   If_Result_3 := Fld_1;
else
   --# expr-start 8989 '<If at nodes.lkt:12967:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12967'
--# expr-start 8980 '<FieldAccess for name_is at nodes.lkt:12969:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12969'
--# expr-start 8978 '<FieldAccess for attribute at nodes.lkt:12969:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12969'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8978
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8979 '<Symbol Result at nodes.lkt:12969:37>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12969'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Result); 
--# expr-done 8979
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Sym);
--# end
--# expr-done 8980
if Fld_6 then
   --# expr-start 8988 '<FieldAccess for defining_env at nodes.lkt:12970:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
--# expr-start 8987 '<FieldAccess for return_type at nodes.lkt:12970:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
--# expr-start 8986 '<FieldAccess for subp_spec_or_null at nodes.lkt:12970:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
--# expr-start 8985 '<Cast to BasicDecl at nodes.lkt:12970:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
--# expr-start 8984 '<as_entity at nodes.lkt:12970:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
--# expr-start 8983 '<Find at nodes.lkt:12970:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
   --# expr-start 8981 '<FieldAccess for parents at nodes.lkt:12970:13>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12970'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 8981
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 8982 '<IsA BasicSubpDecl, BaseSubpBody at nodes.lkt:12971:24>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12971'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 8982
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 8983
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 8984
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 8985
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8986
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8987
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_3.Info);
--# end
--# expr-done 8988
   If_Result_1 := Fld_4;
else
   If_Result_1 := Empty_Env;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8989
   If_Result_3 := If_Result_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 8990
         Property_Result := If_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1160;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1160;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Designated_Env;
--# end
--# property-start AttributeRef.attribute_subprogram 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12978'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Attribute_Subprogram
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_242 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Fld_4 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 8992 '<FieldAccess for name_designated_type at nodes.lkt:12979:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12979'
--# expr-start 8991 '<FieldAccess for prefix at nodes.lkt:12979:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12979'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 8991
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8992
Var_Expr_242 := Fld_1;
if Var_Expr_242 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8995 '<FieldAccess for attribute_subprogram at nodes.lkt:12979:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12979'
      if Var_Expr_242.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 8994 '<FieldAccess for name_symbol at nodes.lkt:12979:66>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12979'
--# expr-start 8993 '<FieldAccess for attribute at nodes.lkt:12979:66>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12979'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8993
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 8994
--# property-call-start BaseTypeDecl.attribute_subprogram
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Attribute_Subprogram (Node => Var_Expr_242.Node, Attr_Name => Fld_3, E_Info => Var_Expr_242.Info);
--# end
--# expr-done 8995
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Attribute_Subprogram;
--# end
--# property-start AttributeRef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12982'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1162 with Inline_Always;
            procedure Finalizer_Scope_3198 with Inline_Always;
            procedure Finalizer_Scope_3199 with Inline_Always;
      Local_Rel_Name : Symbol_Type;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Name;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_8 : Logic_Equation;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_12 : Logic_Equation;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Equation;
Fld_22 : Logic_Equation;
Fld_23 : Internal_Entity_Name;
Fld_24 : Logic_Equation;
Fld_25 : Internal_Entity_Name;
Fld_26 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_31 : Internal_Entity_Name;
Fld_32 : Logic_Equation;
Fld_33 : Logic_Var;
Fld_34 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_35 : Internal_Entity_Name;
Fld_36 : Logic_Equation;
Fld_37 : Logic_Var;
Fld_38 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Fld_41 : Logic_Equation;
Fld_42 : Logic_Equation;
Fld_43 : Logic_Equation;
Fld_44 : Internal_Entity_Name;
Fld_45 : Logic_Equation;
Fld_46 : Logic_Var;
Fld_47 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_48 : Internal_Entity_Name;
Fld_49 : Logic_Equation;
Fld_50 : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_9 : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_10 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_51 : Logic_Var;
Fld_52 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_53 : Internal_Entity_Name;
Fld_54 : Logic_Equation;
Fld_55 : Logic_Var;
Fld_56 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_57 : Logic_Equation;
Fld_58 : Logic_Equation;
Fld_59 : Logic_Equation;
Fld_60 : Logic_Var;
Sym : Symbol_Type;
Fld_61 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Fld_62 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_12 : Internal_Entity;
Bind_Result_7 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_63 : Logic_Equation;
Fld_64 : Logic_Equation;
Fld_65 : Logic_Equation;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_1 : Logic_Equation;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_2 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_4 : Logic_Equation;
Sym_6 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Logic_Equation;
Sym_7 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Logic_Equation;
Sym_8 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Logic_Equation;
Sym_9 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_8 : Logic_Equation;
Sym_10 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_9 : Logic_Equation;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_10 : Boolean;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Logic_Equation;
Sym_13 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_12 : Logic_Equation;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_13 : Boolean;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
If_Result_15 : Logic_Equation;
Sym_17 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_16 : Logic_Equation;
Sym_18 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_17 : Logic_Equation;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_18 : Boolean;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
If_Result_19 : Logic_Equation;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Sym_50 : Symbol_Type;
Is_Equal_20 : Boolean;
Is_Equal_21 : Boolean;
If_Result_20 : Boolean;
Is_Equal_22 : Boolean;
If_Result_21 : Boolean;
Is_Equal_23 : Boolean;
If_Result_22 : Boolean;
Is_Equal_24 : Boolean;
If_Result_23 : Boolean;
Is_Equal_25 : Boolean;
If_Result_24 : Boolean;
Is_Equal_26 : Boolean;
If_Result_25 : Boolean;
Is_Equal_27 : Boolean;
If_Result_26 : Boolean;
Is_Equal_28 : Boolean;
If_Result_27 : Boolean;
Is_Equal_29 : Boolean;
If_Result_28 : Boolean;
Is_Equal_30 : Boolean;
If_Result_29 : Boolean;
Is_Equal_31 : Boolean;
If_Result_30 : Boolean;
Is_Equal_32 : Boolean;
If_Result_31 : Boolean;
Is_Equal_33 : Boolean;
If_Result_32 : Boolean;
Is_Equal_34 : Boolean;
If_Result_33 : Boolean;
Is_Equal_35 : Boolean;
If_Result_34 : Boolean;
Is_Equal_36 : Boolean;
If_Result_35 : Boolean;
Is_Equal_37 : Boolean;
If_Result_36 : Boolean;
Is_Equal_38 : Boolean;
If_Result_37 : Boolean;
Is_Equal_39 : Boolean;
If_Result_38 : Boolean;
Is_Equal_40 : Boolean;
If_Result_39 : Boolean;
Is_Equal_41 : Boolean;
If_Result_40 : Boolean;
Is_Equal_42 : Boolean;
If_Result_41 : Boolean;
Is_Equal_43 : Boolean;
If_Result_42 : Boolean;
Is_Equal_44 : Boolean;
If_Result_43 : Boolean;
Is_Equal_45 : Boolean;
If_Result_44 : Boolean;
Is_Equal_46 : Boolean;
If_Result_45 : Boolean;
Is_Equal_47 : Boolean;
If_Result_46 : Boolean;
Is_Equal_48 : Boolean;
If_Result_47 : Boolean;
Is_Equal_49 : Boolean;
If_Result_48 : Boolean;
If_Result_49 : Logic_Equation;
Sym_51 : Symbol_Type;
Is_Equal_50 : Boolean;
If_Result_50 : Logic_Equation;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Sym_68 : Symbol_Type;
Is_Equal_51 : Boolean;
Is_Equal_52 : Boolean;
If_Result_51 : Boolean;
Is_Equal_53 : Boolean;
If_Result_52 : Boolean;
Is_Equal_54 : Boolean;
If_Result_53 : Boolean;
Is_Equal_55 : Boolean;
If_Result_54 : Boolean;
Is_Equal_56 : Boolean;
If_Result_55 : Boolean;
Is_Equal_57 : Boolean;
If_Result_56 : Boolean;
Is_Equal_58 : Boolean;
If_Result_57 : Boolean;
Is_Equal_59 : Boolean;
If_Result_58 : Boolean;
Is_Equal_60 : Boolean;
If_Result_59 : Boolean;
Is_Equal_61 : Boolean;
If_Result_60 : Boolean;
Is_Equal_62 : Boolean;
If_Result_61 : Boolean;
Is_Equal_63 : Boolean;
If_Result_62 : Boolean;
Is_Equal_64 : Boolean;
If_Result_63 : Boolean;
Is_Equal_65 : Boolean;
If_Result_64 : Boolean;
Is_Equal_66 : Boolean;
If_Result_65 : Boolean;
Is_Equal_67 : Boolean;
If_Result_66 : Boolean;
If_Result_67 : Logic_Equation;
Sym_69 : Symbol_Type;
Sym_70 : Symbol_Type;
Is_Equal_68 : Boolean;
Is_Equal_69 : Boolean;
If_Result_68 : Boolean;
If_Result_69 : Logic_Equation;
Sym_71 : Symbol_Type;
Sym_72 : Symbol_Type;
Is_Equal_70 : Boolean;
Is_Equal_71 : Boolean;
If_Result_70 : Boolean;
If_Result_71 : Logic_Equation;
Sym_73 : Symbol_Type;
Is_Equal_72 : Boolean;
If_Result_72 : Logic_Equation;
Sym_74 : Symbol_Type;
Is_Equal_73 : Boolean;
If_Result_73 : Logic_Equation;
Sym_75 : Symbol_Type;
Is_Equal_74 : Boolean;
If_Result_74 : Logic_Equation;
Sym_76 : Symbol_Type;
Sym_77 : Symbol_Type;
Sym_78 : Symbol_Type;
Sym_79 : Symbol_Type;
Sym_80 : Symbol_Type;
Sym_81 : Symbol_Type;
Sym_82 : Symbol_Type;
Sym_83 : Symbol_Type;
Sym_84 : Symbol_Type;
Sym_85 : Symbol_Type;
Is_Equal_75 : Boolean;
Is_Equal_76 : Boolean;
If_Result_75 : Boolean;
Is_Equal_77 : Boolean;
If_Result_76 : Boolean;
Is_Equal_78 : Boolean;
If_Result_77 : Boolean;
Is_Equal_79 : Boolean;
If_Result_78 : Boolean;
Is_Equal_80 : Boolean;
If_Result_79 : Boolean;
Is_Equal_81 : Boolean;
If_Result_80 : Boolean;
Is_Equal_82 : Boolean;
If_Result_81 : Boolean;
Is_Equal_83 : Boolean;
If_Result_82 : Boolean;
Is_Equal_84 : Boolean;
If_Result_83 : Boolean;
If_Result_84 : Logic_Equation;
Sym_86 : Symbol_Type;
Sym_87 : Symbol_Type;
Is_Equal_85 : Boolean;
Is_Equal_86 : Boolean;
If_Result_85 : Boolean;
If_Result_86 : Logic_Equation;
Sym_88 : Symbol_Type;
Is_Equal_87 : Boolean;
If_Result_87 : Logic_Equation;
Sym_89 : Symbol_Type;
Sym_90 : Symbol_Type;
Sym_91 : Symbol_Type;
Is_Equal_88 : Boolean;
Is_Equal_89 : Boolean;
If_Result_88 : Boolean;
Is_Equal_90 : Boolean;
If_Result_89 : Boolean;
If_Result_90 : Logic_Equation;
Sym_92 : Symbol_Type;
Is_Equal_91 : Boolean;
If_Result_91 : Logic_Equation;
Sym_93 : Symbol_Type;
Is_Equal_92 : Boolean;
If_Result_92 : Logic_Equation;
Sym_94 : Symbol_Type;
Is_Equal_93 : Boolean;
If_Result_93 : Logic_Equation;
Sym_95 : Symbol_Type;
Is_Equal_94 : Boolean;
If_Result_94 : Logic_Equation;
Sym_96 : Symbol_Type;
Sym_97 : Symbol_Type;
Sym_98 : Symbol_Type;
Is_Equal_95 : Boolean;
Is_Equal_96 : Boolean;
If_Result_95 : Boolean;
Is_Equal_97 : Boolean;
If_Result_96 : Boolean;
If_Result_97 : Logic_Equation;
Sym_99 : Symbol_Type;
Sym_100 : Symbol_Type;
Sym_101 : Symbol_Type;
Sym_102 : Symbol_Type;
Sym_103 : Symbol_Type;
Sym_104 : Symbol_Type;
Sym_105 : Symbol_Type;
Is_Equal_98 : Boolean;
Is_Equal_99 : Boolean;
If_Result_98 : Boolean;
Is_Equal_100 : Boolean;
If_Result_99 : Boolean;
Is_Equal_101 : Boolean;
If_Result_100 : Boolean;
Is_Equal_102 : Boolean;
If_Result_101 : Boolean;
Is_Equal_103 : Boolean;
If_Result_102 : Boolean;
Is_Equal_104 : Boolean;
If_Result_103 : Boolean;
If_Result_104 : Logic_Equation;
Sym_106 : Symbol_Type;
Sym_107 : Symbol_Type;
Is_Equal_105 : Boolean;
Is_Equal_106 : Boolean;
If_Result_105 : Boolean;
If_Result_106 : Logic_Equation;
Sym_108 : Symbol_Type;
Sym_109 : Symbol_Type;
Sym_110 : Symbol_Type;
Sym_111 : Symbol_Type;
Sym_112 : Symbol_Type;
Sym_113 : Symbol_Type;
Sym_114 : Symbol_Type;
Sym_115 : Symbol_Type;
Sym_116 : Symbol_Type;
Sym_117 : Symbol_Type;
Sym_118 : Symbol_Type;
Sym_119 : Symbol_Type;
Sym_120 : Symbol_Type;
Sym_121 : Symbol_Type;
Sym_122 : Symbol_Type;
Sym_123 : Symbol_Type;
Sym_124 : Symbol_Type;
Sym_125 : Symbol_Type;
Sym_126 : Symbol_Type;
Sym_127 : Symbol_Type;
Sym_128 : Symbol_Type;
Sym_129 : Symbol_Type;
Sym_130 : Symbol_Type;
Sym_131 : Symbol_Type;
Sym_132 : Symbol_Type;
Sym_133 : Symbol_Type;
Sym_134 : Symbol_Type;
Sym_135 : Symbol_Type;
Sym_136 : Symbol_Type;
Sym_137 : Symbol_Type;
Sym_138 : Symbol_Type;
Sym_139 : Symbol_Type;
Sym_140 : Symbol_Type;
Sym_141 : Symbol_Type;
Sym_142 : Symbol_Type;
Sym_143 : Symbol_Type;
Sym_144 : Symbol_Type;
Is_Equal_107 : Boolean;
Is_Equal_108 : Boolean;
If_Result_107 : Boolean;
Is_Equal_109 : Boolean;
If_Result_108 : Boolean;
Is_Equal_110 : Boolean;
If_Result_109 : Boolean;
Is_Equal_111 : Boolean;
If_Result_110 : Boolean;
Is_Equal_112 : Boolean;
If_Result_111 : Boolean;
Is_Equal_113 : Boolean;
If_Result_112 : Boolean;
Is_Equal_114 : Boolean;
If_Result_113 : Boolean;
Is_Equal_115 : Boolean;
If_Result_114 : Boolean;
Is_Equal_116 : Boolean;
If_Result_115 : Boolean;
Is_Equal_117 : Boolean;
If_Result_116 : Boolean;
Is_Equal_118 : Boolean;
If_Result_117 : Boolean;
Is_Equal_119 : Boolean;
If_Result_118 : Boolean;
Is_Equal_120 : Boolean;
If_Result_119 : Boolean;
Is_Equal_121 : Boolean;
If_Result_120 : Boolean;
Is_Equal_122 : Boolean;
If_Result_121 : Boolean;
Is_Equal_123 : Boolean;
If_Result_122 : Boolean;
Is_Equal_124 : Boolean;
If_Result_123 : Boolean;
Is_Equal_125 : Boolean;
If_Result_124 : Boolean;
Is_Equal_126 : Boolean;
If_Result_125 : Boolean;
Is_Equal_127 : Boolean;
If_Result_126 : Boolean;
Is_Equal_128 : Boolean;
If_Result_127 : Boolean;
Is_Equal_129 : Boolean;
If_Result_128 : Boolean;
Is_Equal_130 : Boolean;
If_Result_129 : Boolean;
Is_Equal_131 : Boolean;
If_Result_130 : Boolean;
Is_Equal_132 : Boolean;
If_Result_131 : Boolean;
Is_Equal_133 : Boolean;
If_Result_132 : Boolean;
Is_Equal_134 : Boolean;
If_Result_133 : Boolean;
Is_Equal_135 : Boolean;
If_Result_134 : Boolean;
Is_Equal_136 : Boolean;
If_Result_135 : Boolean;
Is_Equal_137 : Boolean;
If_Result_136 : Boolean;
Is_Equal_138 : Boolean;
If_Result_137 : Boolean;
Is_Equal_139 : Boolean;
If_Result_138 : Boolean;
Is_Equal_140 : Boolean;
If_Result_139 : Boolean;
Is_Equal_141 : Boolean;
If_Result_140 : Boolean;
Is_Equal_142 : Boolean;
If_Result_141 : Boolean;
Is_Equal_143 : Boolean;
If_Result_142 : Boolean;
If_Result_143 : Logic_Equation;
Sym_145 : Symbol_Type;
Sym_146 : Symbol_Type;
Sym_147 : Symbol_Type;
Sym_148 : Symbol_Type;
Is_Equal_144 : Boolean;
Is_Equal_145 : Boolean;
If_Result_144 : Boolean;
Is_Equal_146 : Boolean;
If_Result_145 : Boolean;
Is_Equal_147 : Boolean;
If_Result_146 : Boolean;
If_Result_147 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_1162 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_34);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_36);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Fld_42);
                     Dec_Ref (Fld_43);
                     Dec_Ref (Fld_45);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Fld_52);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_54);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_57);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Fld_59);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_63);
                     Dec_Ref (Fld_64);
                     Dec_Ref (Fld_65);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (If_Result_12);
                     Dec_Ref (If_Result_15);
                     Dec_Ref (If_Result_16);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (If_Result_19);
                     Dec_Ref (If_Result_49);
                     Dec_Ref (If_Result_50);
                     Dec_Ref (If_Result_67);
                     Dec_Ref (If_Result_69);
                     Dec_Ref (If_Result_71);
                     Dec_Ref (If_Result_72);
                     Dec_Ref (If_Result_73);
                     Dec_Ref (If_Result_74);
                     Dec_Ref (If_Result_84);
                     Dec_Ref (If_Result_86);
                     Dec_Ref (If_Result_87);
                     Dec_Ref (If_Result_90);
                     Dec_Ref (If_Result_91);
                     Dec_Ref (If_Result_92);
                     Dec_Ref (If_Result_93);
                     Dec_Ref (If_Result_94);
                     Dec_Ref (If_Result_97);
                     Dec_Ref (If_Result_104);
                     Dec_Ref (If_Result_106);
                     Dec_Ref (If_Result_143);
                     Dec_Ref (If_Result_147);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1162;
            procedure Finalizer_Scope_3198 is
            begin
                     Dec_Ref (Fld_49);
            end Finalizer_Scope_3198;
            procedure Finalizer_Scope_3199 is
            begin
                     Dec_Ref (Fld_50);
            end Finalizer_Scope_3199;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9283 '<Let at nodes.lkt:12983:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12983'
--# scope-start
--# expr-start 8997 '<FieldAccess for name_symbol at nodes.lkt:12983:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12983'
--# expr-start 8996 '<FieldAccess for attribute at nodes.lkt:12983:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12983'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 8996
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8997
Local_Rel_Name := Fld_1; 
--# bind rel_name Local_Rel_Name
--# expr-start 9282 '<If at nodes.lkt:12986:9>' If_Result_147 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 8998 '<Symbol First at nodes.lkt:12986:24>' Sym_145 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
Sym_145 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 8998
Is_Equal_147 := Local_Rel_Name = Sym_145; 
if Is_Equal_147 then
   If_Result_146 := True;
else
--# expr-start 8999 '<Symbol Last at nodes.lkt:12986:35>' Sym_146 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
Sym_146 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 8999
Is_Equal_146 := Local_Rel_Name = Sym_146; 
if Is_Equal_146 then
   If_Result_145 := True;
else
--# expr-start 9000 '<Symbol Range at nodes.lkt:12986:45>' Sym_147 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
Sym_147 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 9000
Is_Equal_145 := Local_Rel_Name = Sym_147; 
if Is_Equal_145 then
   If_Result_144 := True;
else
--# expr-start 9001 '<Symbol Length at nodes.lkt:12986:56>' Sym_148 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
Sym_148 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 9001
Is_Equal_144 := Local_Rel_Name = Sym_148; 
   If_Result_144 := Is_Equal_144;
end if;
   If_Result_145 := If_Result_144;
end if;
   If_Result_146 := If_Result_145;
end if;
if If_Result_146 then
   --# expr-start 9002 '<FieldAccess for array_attr_equation at nodes.lkt:12986:71>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.array_attr_equation
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Array_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9002
   If_Result_147 := Fld_2;
else
   --# expr-start 9281 '<If at nodes.lkt:12986:9>' If_Result_143 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9003 '<Symbol Succ at nodes.lkt:12988:26>' Sym_108 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_108 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ); 
--# expr-done 9003
Is_Equal_143 := Local_Rel_Name = Sym_108; 
if Is_Equal_143 then
   If_Result_142 := True;
else
--# expr-start 9004 '<Symbol Pred at nodes.lkt:12988:36>' Sym_109 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_109 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred); 
--# expr-done 9004
Is_Equal_142 := Local_Rel_Name = Sym_109; 
if Is_Equal_142 then
   If_Result_141 := True;
else
--# expr-start 9005 '<Symbol Min at nodes.lkt:12988:46>' Sym_110 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_110 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min); 
--# expr-done 9005
Is_Equal_141 := Local_Rel_Name = Sym_110; 
if Is_Equal_141 then
   If_Result_140 := True;
else
--# expr-start 9006 '<Symbol Max at nodes.lkt:12988:55>' Sym_111 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_111 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max); 
--# expr-done 9006
Is_Equal_140 := Local_Rel_Name = Sym_111; 
if Is_Equal_140 then
   If_Result_139 := True;
else
--# expr-start 9007 '<Symbol Ceiling at nodes.lkt:12988:64>' Sym_112 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_112 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling); 
--# expr-done 9007
Is_Equal_139 := Local_Rel_Name = Sym_112; 
if Is_Equal_139 then
   If_Result_138 := True;
else
--# expr-start 9008 '<Symbol Floor at nodes.lkt:12988:77>' Sym_113 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_113 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor); 
--# expr-done 9008
Is_Equal_138 := Local_Rel_Name = Sym_113; 
if Is_Equal_138 then
   If_Result_137 := True;
else
--# expr-start 9009 '<Symbol Rounding at nodes.lkt:12988:88>' Sym_114 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_114 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding); 
--# expr-done 9009
Is_Equal_137 := Local_Rel_Name = Sym_114; 
if Is_Equal_137 then
   If_Result_136 := True;
else
--# expr-start 9010 '<Symbol Unbiased_Rounding at nodes.lkt:12988:102>' Sym_115 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_115 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding); 
--# expr-done 9010
Is_Equal_136 := Local_Rel_Name = Sym_115; 
if Is_Equal_136 then
   If_Result_135 := True;
else
--# expr-start 9011 '<Symbol Leading_Part at nodes.lkt:12988:125>' Sym_116 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_116 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part); 
--# expr-done 9011
Is_Equal_135 := Local_Rel_Name = Sym_116; 
if Is_Equal_135 then
   If_Result_134 := True;
else
--# expr-start 9012 '<Symbol Truncation at nodes.lkt:12988:143>' Sym_117 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_117 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation); 
--# expr-done 9012
Is_Equal_134 := Local_Rel_Name = Sym_117; 
if Is_Equal_134 then
   If_Result_133 := True;
else
--# expr-start 9013 '<Symbol Exponent at nodes.lkt:12988:159>' Sym_118 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_118 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent); 
--# expr-done 9013
Is_Equal_133 := Local_Rel_Name = Sym_118; 
if Is_Equal_133 then
   If_Result_132 := True;
else
--# expr-start 9014 '<Symbol Fraction at nodes.lkt:12988:173>' Sym_119 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_119 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction); 
--# expr-done 9014
Is_Equal_132 := Local_Rel_Name = Sym_119; 
if Is_Equal_132 then
   If_Result_131 := True;
else
--# expr-start 9015 '<Symbol Copy_Sign at nodes.lkt:12988:187>' Sym_120 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_120 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign); 
--# expr-done 9015
Is_Equal_131 := Local_Rel_Name = Sym_120; 
if Is_Equal_131 then
   If_Result_130 := True;
else
--# expr-start 9016 '<Symbol Remainder at nodes.lkt:12988:202>' Sym_121 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_121 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder); 
--# expr-done 9016
Is_Equal_130 := Local_Rel_Name = Sym_121; 
if Is_Equal_130 then
   If_Result_129 := True;
else
--# expr-start 9017 '<Symbol Adjacent at nodes.lkt:12988:217>' Sym_122 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_122 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent); 
--# expr-done 9017
Is_Equal_129 := Local_Rel_Name = Sym_122; 
if Is_Equal_129 then
   If_Result_128 := True;
else
--# expr-start 9018 '<Symbol Machine at nodes.lkt:12988:231>' Sym_123 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_123 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine); 
--# expr-done 9018
Is_Equal_128 := Local_Rel_Name = Sym_123; 
if Is_Equal_128 then
   If_Result_127 := True;
else
--# expr-start 9019 '<Symbol Machine_Rounding at nodes.lkt:12988:244>' Sym_124 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_124 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding); 
--# expr-done 9019
Is_Equal_127 := Local_Rel_Name = Sym_124; 
if Is_Equal_127 then
   If_Result_126 := True;
else
--# expr-start 9020 '<Symbol Scaling at nodes.lkt:12988:266>' Sym_125 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_125 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling); 
--# expr-done 9020
Is_Equal_126 := Local_Rel_Name = Sym_125; 
if Is_Equal_126 then
   If_Result_125 := True;
else
--# expr-start 9021 '<Symbol Compose at nodes.lkt:12988:279>' Sym_126 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_126 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose); 
--# expr-done 9021
Is_Equal_125 := Local_Rel_Name = Sym_126; 
if Is_Equal_125 then
   If_Result_124 := True;
else
--# expr-start 9022 '<Symbol Mod at nodes.lkt:12988:292>' Sym_127 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_127 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_170); 
--# expr-done 9022
Is_Equal_124 := Local_Rel_Name = Sym_127; 
if Is_Equal_124 then
   If_Result_123 := True;
else
--# expr-start 9023 '<Symbol Value at nodes.lkt:12988:301>' Sym_128 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_128 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 9023
Is_Equal_123 := Local_Rel_Name = Sym_128; 
if Is_Equal_123 then
   If_Result_122 := True;
else
--# expr-start 9024 '<Symbol Wide_Value at nodes.lkt:12988:312>' Sym_129 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_129 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value); 
--# expr-done 9024
Is_Equal_122 := Local_Rel_Name = Sym_129; 
if Is_Equal_122 then
   If_Result_121 := True;
else
--# expr-start 9025 '<Symbol Wide_Wide_Value at nodes.lkt:12988:328>' Sym_130 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_130 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value); 
--# expr-done 9025
Is_Equal_121 := Local_Rel_Name = Sym_130; 
if Is_Equal_121 then
   If_Result_120 := True;
else
--# expr-start 9026 '<Symbol Fixed_Value at nodes.lkt:12988:349>' Sym_131 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_131 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value); 
--# expr-done 9026
Is_Equal_120 := Local_Rel_Name = Sym_131; 
if Is_Equal_120 then
   If_Result_119 := True;
else
--# expr-start 9027 '<Symbol Integer_Value at nodes.lkt:12988:366>' Sym_132 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_132 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value); 
--# expr-done 9027
Is_Equal_119 := Local_Rel_Name = Sym_132; 
if Is_Equal_119 then
   If_Result_118 := True;
else
--# expr-start 9028 '<Symbol Pos at nodes.lkt:12988:385>' Sym_133 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_133 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos); 
--# expr-done 9028
Is_Equal_118 := Local_Rel_Name = Sym_133; 
if Is_Equal_118 then
   If_Result_117 := True;
else
--# expr-start 9029 '<Symbol Val at nodes.lkt:12988:394>' Sym_134 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_134 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 9029
Is_Equal_117 := Local_Rel_Name = Sym_134; 
if Is_Equal_117 then
   If_Result_116 := True;
else
--# expr-start 9030 '<Symbol Enum_Val at nodes.lkt:12988:403>' Sym_135 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_135 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val); 
--# expr-done 9030
Is_Equal_116 := Local_Rel_Name = Sym_135; 
if Is_Equal_116 then
   If_Result_115 := True;
else
--# expr-start 9031 '<Symbol Write at nodes.lkt:12988:417>' Sym_136 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_136 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 9031
Is_Equal_115 := Local_Rel_Name = Sym_136; 
if Is_Equal_115 then
   If_Result_114 := True;
else
--# expr-start 9032 '<Symbol Read at nodes.lkt:12988:428>' Sym_137 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_137 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 9032
Is_Equal_114 := Local_Rel_Name = Sym_137; 
if Is_Equal_114 then
   If_Result_113 := True;
else
--# expr-start 9033 '<Symbol Output at nodes.lkt:12988:438>' Sym_138 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_138 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 9033
Is_Equal_113 := Local_Rel_Name = Sym_138; 
if Is_Equal_113 then
   If_Result_112 := True;
else
--# expr-start 9034 '<Symbol Input at nodes.lkt:12988:450>' Sym_139 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_139 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 9034
Is_Equal_112 := Local_Rel_Name = Sym_139; 
if Is_Equal_112 then
   If_Result_111 := True;
else
--# expr-start 9035 '<Symbol Put_Image at nodes.lkt:12988:461>' Sym_140 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_140 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 9035
Is_Equal_111 := Local_Rel_Name = Sym_140; 
if Is_Equal_111 then
   If_Result_110 := True;
else
--# expr-start 9036 '<Symbol Asm_Input at nodes.lkt:12988:476>' Sym_141 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_141 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input); 
--# expr-done 9036
Is_Equal_110 := Local_Rel_Name = Sym_141; 
if Is_Equal_110 then
   If_Result_109 := True;
else
--# expr-start 9037 '<Symbol Asm_Output at nodes.lkt:12988:491>' Sym_142 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_142 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output); 
--# expr-done 9037
Is_Equal_109 := Local_Rel_Name = Sym_142; 
if Is_Equal_109 then
   If_Result_108 := True;
else
--# expr-start 9038 '<Symbol Model at nodes.lkt:12988:507>' Sym_143 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_143 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model); 
--# expr-done 9038
Is_Equal_108 := Local_Rel_Name = Sym_143; 
if Is_Equal_108 then
   If_Result_107 := True;
else
--# expr-start 9039 '<Symbol Round at nodes.lkt:12988:518>' Sym_144 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
Sym_144 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Round); 
--# expr-done 9039
Is_Equal_107 := Local_Rel_Name = Sym_144; 
   If_Result_107 := Is_Equal_107;
end if;
   If_Result_108 := If_Result_107;
end if;
   If_Result_109 := If_Result_108;
end if;
   If_Result_110 := If_Result_109;
end if;
   If_Result_111 := If_Result_110;
end if;
   If_Result_112 := If_Result_111;
end if;
   If_Result_113 := If_Result_112;
end if;
   If_Result_114 := If_Result_113;
end if;
   If_Result_115 := If_Result_114;
end if;
   If_Result_116 := If_Result_115;
end if;
   If_Result_117 := If_Result_116;
end if;
   If_Result_118 := If_Result_117;
end if;
   If_Result_119 := If_Result_118;
end if;
   If_Result_120 := If_Result_119;
end if;
   If_Result_121 := If_Result_120;
end if;
   If_Result_122 := If_Result_121;
end if;
   If_Result_123 := If_Result_122;
end if;
   If_Result_124 := If_Result_123;
end if;
   If_Result_125 := If_Result_124;
end if;
   If_Result_126 := If_Result_125;
end if;
   If_Result_127 := If_Result_126;
end if;
   If_Result_128 := If_Result_127;
end if;
   If_Result_129 := If_Result_128;
end if;
   If_Result_130 := If_Result_129;
end if;
   If_Result_131 := If_Result_130;
end if;
   If_Result_132 := If_Result_131;
end if;
   If_Result_133 := If_Result_132;
end if;
   If_Result_134 := If_Result_133;
end if;
   If_Result_135 := If_Result_134;
end if;
   If_Result_136 := If_Result_135;
end if;
   If_Result_137 := If_Result_136;
end if;
   If_Result_138 := If_Result_137;
end if;
   If_Result_139 := If_Result_138;
end if;
   If_Result_140 := If_Result_139;
end if;
   If_Result_141 := If_Result_140;
end if;
   If_Result_142 := If_Result_141;
end if;
if If_Result_142 then
   --# expr-start 9040 '<FieldAccess for attribute_subprogram_equation at nodes.lkt:12988:532>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12988'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram_equation
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9040
   If_Result_143 := Fld_3;
else
   --# expr-start 9280 '<If at nodes.lkt:12986:9>' If_Result_106 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9041 '<Symbol Size at nodes.lkt:12989:26>' Sym_106 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12989'
Sym_106 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Size); 
--# expr-done 9041
Is_Equal_106 := Local_Rel_Name = Sym_106; 
if Is_Equal_106 then
   If_Result_105 := True;
else
--# expr-start 9042 '<Symbol VADS_Size at nodes.lkt:12989:36>' Sym_107 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12989'
Sym_107 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Vads_Size); 
--# expr-done 9042
Is_Equal_105 := Local_Rel_Name = Sym_107; 
   If_Result_105 := Is_Equal_105;
end if;
if If_Result_105 then
   --# expr-start 9043 '<FieldAccess for size_equation at nodes.lkt:12989:54>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12989'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.size_equation
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Size_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9043
   If_Result_106 := Fld_4;
else
   --# expr-start 9279 '<If at nodes.lkt:12986:9>' If_Result_104 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9044 '<Symbol Max_Size_In_Storage_Elements at nodes.lkt:12990:26>' Sym_99 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12990'
Sym_99 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_Size_In_Storage_Elements); 
--# expr-done 9044
Is_Equal_104 := Local_Rel_Name = Sym_99; 
if Is_Equal_104 then
   If_Result_103 := True;
else
--# expr-start 9045 '<Symbol Max_Alignment_For_Allocation at nodes.lkt:12990:60>' Sym_100 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12990'
Sym_100 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_Alignment_For_Allocation); 
--# expr-done 9045
Is_Equal_103 := Local_Rel_Name = Sym_100; 
if Is_Equal_103 then
   If_Result_102 := True;
else
--# expr-start 9046 '<Symbol Aft at nodes.lkt:12991:15>' Sym_101 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12991'
Sym_101 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aft); 
--# expr-done 9046
Is_Equal_102 := Local_Rel_Name = Sym_101; 
if Is_Equal_102 then
   If_Result_101 := True;
else
--# expr-start 9047 '<Symbol Object_Size at nodes.lkt:12991:24>' Sym_102 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12991'
Sym_102 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Object_Size); 
--# expr-done 9047
Is_Equal_101 := Local_Rel_Name = Sym_102; 
if Is_Equal_101 then
   If_Result_100 := True;
else
--# expr-start 9048 '<Symbol Value_Size at nodes.lkt:12991:41>' Sym_103 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12991'
Sym_103 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value_Size); 
--# expr-done 9048
Is_Equal_100 := Local_Rel_Name = Sym_103; 
if Is_Equal_100 then
   If_Result_99 := True;
else
--# expr-start 9049 '<Symbol Storage_Size at nodes.lkt:12991:57>' Sym_104 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12991'
Sym_104 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Size); 
--# expr-done 9049
Is_Equal_99 := Local_Rel_Name = Sym_104; 
if Is_Equal_99 then
   If_Result_98 := True;
else
--# expr-start 9050 '<Symbol Scale at nodes.lkt:12992:15>' Sym_105 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12992'
Sym_105 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scale); 
--# expr-done 9050
Is_Equal_98 := Local_Rel_Name = Sym_105; 
   If_Result_98 := Is_Equal_98;
end if;
   If_Result_99 := If_Result_98;
end if;
   If_Result_100 := If_Result_99;
end if;
   If_Result_101 := If_Result_100;
end if;
   If_Result_102 := If_Result_101;
end if;
   If_Result_103 := If_Result_102;
end if;
if If_Result_103 then
   --# expr-start 9051 '<FieldAccess for subtype_attr_equation at nodes.lkt:12993:18>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12993'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.subtype_attr_equation
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Subtype_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9051
   If_Result_104 := Fld_5;
else
   --# expr-start 9278 '<If at nodes.lkt:12986:9>' If_Result_97 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9052 '<Symbol Access at nodes.lkt:12994:26>' Sym_96 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12994'
Sym_96 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Access); 
--# expr-done 9052
Is_Equal_97 := Local_Rel_Name = Sym_96; 
if Is_Equal_97 then
   If_Result_96 := True;
else
--# expr-start 9053 '<Symbol Unchecked_Access at nodes.lkt:12994:38>' Sym_97 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12994'
Sym_97 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Access); 
--# expr-done 9053
Is_Equal_96 := Local_Rel_Name = Sym_97; 
if Is_Equal_96 then
   If_Result_95 := True;
else
--# expr-start 9054 '<Symbol Unrestricted_Access at nodes.lkt:12994:60>' Sym_98 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12994'
Sym_98 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unrestricted_Access); 
--# expr-done 9054
Is_Equal_95 := Local_Rel_Name = Sym_98; 
   If_Result_95 := Is_Equal_95;
end if;
   If_Result_96 := If_Result_95;
end if;
if If_Result_96 then
   --# expr-start 9055 '<FieldAccess for access_equation at nodes.lkt:12994:88>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12994'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.access_equation
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Access_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9055
   If_Result_97 := Fld_6;
else
   --# expr-start 9277 '<If at nodes.lkt:12986:9>' If_Result_94 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9057 '<Eq at nodes.lkt:12995:14>' Is_Equal_94 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12995'
--# expr-start 9056 '<Symbol Image at nodes.lkt:12995:26>' Sym_95 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12995'
Sym_95 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 9056
Is_Equal_94 := Local_Rel_Name = Sym_95; 
--# expr-done 9057
if Is_Equal_94 then
   --# expr-start 9059 '<FieldAccess for image_equation at nodes.lkt:12995:40>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12995'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9058 '<FieldAccess for std_string_type at nodes.lkt:12995:60>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12995'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 9058
      Cast_Result := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
--# property-call-start AttributeRef.image_equation
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Cast_Result, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9059
   If_Result_94 := Fld_8;
else
   --# expr-start 9276 '<If at nodes.lkt:12986:9>' If_Result_93 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9061 '<Eq at nodes.lkt:12996:14>' Is_Equal_93 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12996'
--# expr-start 9060 '<Symbol Wide_Image at nodes.lkt:12996:26>' Sym_94 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12996'
Sym_94 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 9060
Is_Equal_93 := Local_Rel_Name = Sym_94; 
--# expr-done 9061
if Is_Equal_93 then
   --# expr-start 9063 '<FieldAccess for image_equation at nodes.lkt:12996:45>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12996'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9062 '<FieldAccess for std_wide_string_type at nodes.lkt:12996:65>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12996'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_string_type
Fld_9 := Libadalang.Implementation.Ada_Node_P_Std_Wide_String_Type (Node => Self);
--# end
--# expr-done 9062
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
--# property-call-start AttributeRef.image_equation
Fld_10 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Cast_Result_1, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9063
   If_Result_93 := Fld_10;
else
   --# expr-start 9275 '<If at nodes.lkt:12986:9>' If_Result_92 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9065 '<Eq at nodes.lkt:12997:14>' Is_Equal_92 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12997'
--# expr-start 9064 '<Symbol Wide_Wide_Image at nodes.lkt:12997:26>' Sym_93 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12997'
Sym_93 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 9064
Is_Equal_92 := Local_Rel_Name = Sym_93; 
--# expr-done 9065
if Is_Equal_92 then
   --# expr-start 9067 '<FieldAccess for image_equation at nodes.lkt:12997:50>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12997'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9066 '<FieldAccess for std_wide_wide_string_type at nodes.lkt:12997:70>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12997'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_wide_string_type
Fld_11 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Wide_String_Type (Node => Self);
--# end
--# expr-done 9066
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
--# property-call-start AttributeRef.image_equation
Fld_12 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Cast_Result_2, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9067
   If_Result_92 := Fld_12;
else
   --# expr-start 9274 '<If at nodes.lkt:12986:9>' If_Result_91 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9069 '<Eq at nodes.lkt:12998:14>' Is_Equal_91 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12998'
--# expr-start 9068 '<Symbol Enum_Rep at nodes.lkt:12998:26>' Sym_92 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12998'
Sym_92 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep); 
--# expr-done 9068
Is_Equal_91 := Local_Rel_Name = Sym_92; 
--# expr-done 9069
if Is_Equal_91 then
   --# expr-start 9070 '<FieldAccess for enum_rep_equation at nodes.lkt:12998:43>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12998'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.enum_rep_equation
Fld_13 := Libadalang.Implementation.Attribute_Ref_P_Enum_Rep_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9070
   If_Result_91 := Fld_13;
else
   --# expr-start 9273 '<If at nodes.lkt:12986:9>' If_Result_90 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9071 '<Symbol Invalid_Value at nodes.lkt:12999:26>' Sym_89 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12999'
Sym_89 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Invalid_Value); 
--# expr-done 9071
Is_Equal_90 := Local_Rel_Name = Sym_89; 
if Is_Equal_90 then
   If_Result_89 := True;
else
--# expr-start 9072 '<Symbol First_Valid at nodes.lkt:12999:45>' Sym_90 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12999'
Sym_90 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First_Valid); 
--# expr-done 9072
Is_Equal_89 := Local_Rel_Name = Sym_90; 
if Is_Equal_89 then
   If_Result_88 := True;
else
--# expr-start 9073 '<Symbol Last_Valid at nodes.lkt:12999:62>' Sym_91 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12999'
Sym_91 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last_Valid); 
--# expr-done 9073
Is_Equal_88 := Local_Rel_Name = Sym_91; 
   If_Result_88 := Is_Equal_88;
end if;
   If_Result_89 := If_Result_88;
end if;
if If_Result_89 then
   --# expr-start 9074 '<FieldAccess for self_type_equation at nodes.lkt:12999:81>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12999'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.self_type_equation
Fld_14 := Libadalang.Implementation.Attribute_Ref_P_Self_Type_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9074
   If_Result_90 := Fld_14;
else
   --# expr-start 9272 '<If at nodes.lkt:12986:9>' If_Result_87 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9076 '<Eq at nodes.lkt:13000:14>' Is_Equal_87 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13000'
--# expr-start 9075 '<Symbol Identity at nodes.lkt:13000:26>' Sym_88 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13000'
Sym_88 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Identity); 
--# expr-done 9075
Is_Equal_87 := Local_Rel_Name = Sym_88; 
--# expr-done 9076
if Is_Equal_87 then
   --# expr-start 9077 '<FieldAccess for identity_equation at nodes.lkt:13000:43>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13000'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.identity_equation
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Identity_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9077
   If_Result_87 := Fld_15;
else
   --# expr-start 9271 '<If at nodes.lkt:12986:9>' If_Result_86 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9078 '<Symbol Address at nodes.lkt:13001:26>' Sym_86 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13001'
Sym_86 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 9078
Is_Equal_86 := Local_Rel_Name = Sym_86; 
if Is_Equal_86 then
   If_Result_85 := True;
else
--# expr-start 9079 '<Symbol Code_Address at nodes.lkt:13001:39>' Sym_87 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13001'
Sym_87 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Code_Address); 
--# expr-done 9079
Is_Equal_85 := Local_Rel_Name = Sym_87; 
   If_Result_85 := Is_Equal_85;
end if;
if If_Result_85 then
   --# expr-start 9080 '<FieldAccess for address_equation at nodes.lkt:13002:13>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.address_equation
Fld_16 := Libadalang.Implementation.Attribute_Ref_P_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9080
   If_Result_86 := Fld_16;
else
   --# expr-start 9270 '<If at nodes.lkt:12986:9>' If_Result_84 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9081 '<Symbol Small at nodes.lkt:13003:26>' Sym_76 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_76 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small); 
--# expr-done 9081
Is_Equal_84 := Local_Rel_Name = Sym_76; 
if Is_Equal_84 then
   If_Result_83 := True;
else
--# expr-start 9082 '<Symbol Model_Small at nodes.lkt:13003:37>' Sym_77 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_77 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Small); 
--# expr-done 9082
Is_Equal_83 := Local_Rel_Name = Sym_77; 
if Is_Equal_83 then
   If_Result_82 := True;
else
--# expr-start 9083 '<Symbol Safe_Small at nodes.lkt:13003:54>' Sym_78 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_78 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Small); 
--# expr-done 9083
Is_Equal_82 := Local_Rel_Name = Sym_78; 
if Is_Equal_82 then
   If_Result_81 := True;
else
--# expr-start 9084 '<Symbol Epsilon at nodes.lkt:13003:70>' Sym_79 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_79 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Epsilon); 
--# expr-done 9084
Is_Equal_81 := Local_Rel_Name = Sym_79; 
if Is_Equal_81 then
   If_Result_80 := True;
else
--# expr-start 9085 '<Symbol Model_Epsilon at nodes.lkt:13003:83>' Sym_80 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_80 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Epsilon); 
--# expr-done 9085
Is_Equal_80 := Local_Rel_Name = Sym_80; 
if Is_Equal_80 then
   If_Result_79 := True;
else
--# expr-start 9086 '<Symbol Large at nodes.lkt:13003:102>' Sym_81 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_81 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Large); 
--# expr-done 9086
Is_Equal_79 := Local_Rel_Name = Sym_81; 
if Is_Equal_79 then
   If_Result_78 := True;
else
--# expr-start 9087 '<Symbol Safe_Large at nodes.lkt:13003:113>' Sym_82 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_82 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Large); 
--# expr-done 9087
Is_Equal_78 := Local_Rel_Name = Sym_82; 
if Is_Equal_78 then
   If_Result_77 := True;
else
--# expr-start 9088 '<Symbol Delta at nodes.lkt:13003:129>' Sym_83 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_83 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Delta); 
--# expr-done 9088
Is_Equal_77 := Local_Rel_Name = Sym_83; 
if Is_Equal_77 then
   If_Result_76 := True;
else
--# expr-start 9089 '<Symbol Safe_First at nodes.lkt:13003:140>' Sym_84 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_84 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_First); 
--# expr-done 9089
Is_Equal_76 := Local_Rel_Name = Sym_84; 
if Is_Equal_76 then
   If_Result_75 := True;
else
--# expr-start 9090 '<Symbol Safe_Last at nodes.lkt:13003:156>' Sym_85 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
Sym_85 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Safe_Last); 
--# expr-done 9090
Is_Equal_75 := Local_Rel_Name = Sym_85; 
   If_Result_75 := Is_Equal_75;
end if;
   If_Result_76 := If_Result_75;
end if;
   If_Result_77 := If_Result_76;
end if;
   If_Result_78 := If_Result_77;
end if;
   If_Result_79 := If_Result_78;
end if;
   If_Result_80 := If_Result_79;
end if;
   If_Result_81 := If_Result_80;
end if;
   If_Result_82 := If_Result_81;
end if;
   If_Result_83 := If_Result_82;
end if;
if If_Result_83 then
   --# expr-start 9091 '<FieldAccess for universal_real_equation at nodes.lkt:13003:174>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13003'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.universal_real_equation
Fld_17 := Libadalang.Implementation.Attribute_Ref_P_Universal_Real_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9091
   If_Result_84 := Fld_17;
else
   --# expr-start 9269 '<If at nodes.lkt:12986:9>' If_Result_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9093 '<Eq at nodes.lkt:13004:14>' Is_Equal_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13004'
--# expr-start 9092 '<Symbol Img at nodes.lkt:13004:26>' Sym_75 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13004'
Sym_75 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Img); 
--# expr-done 9092
Is_Equal_74 := Local_Rel_Name = Sym_75; 
--# expr-done 9093
if Is_Equal_74 then
   --# expr-start 9095 '<FieldAccess for img_equation at nodes.lkt:13004:38>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13004'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9094 '<FieldAccess for std_string_type at nodes.lkt:13004:56>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13004'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_18 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 9094
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
--# property-call-start AttributeRef.img_equation
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Img_Equation (Node => Ent.Node, Str_Type => Cast_Result_3, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9095
   If_Result_74 := Fld_19;
else
   --# expr-start 9268 '<If at nodes.lkt:12986:9>' If_Result_73 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9097 '<Eq at nodes.lkt:13005:14>' Is_Equal_73 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13005'
--# expr-start 9096 '<Symbol Tag at nodes.lkt:13005:26>' Sym_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13005'
Sym_74 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tag); 
--# expr-done 9096
Is_Equal_73 := Local_Rel_Name = Sym_74; 
--# expr-done 9097
if Is_Equal_73 then
   --# expr-start 9098 '<FieldAccess for tag_attr_equation at nodes.lkt:13005:38>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13005'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.tag_attr_equation
Fld_20 := Libadalang.Implementation.Attribute_Ref_P_Tag_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9098
   If_Result_73 := Fld_20;
else
   --# expr-start 9267 '<If at nodes.lkt:12986:9>' If_Result_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9100 '<Eq at nodes.lkt:13006:14>' Is_Equal_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13006'
--# expr-start 9099 '<Symbol Result at nodes.lkt:13006:26>' Sym_73 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13006'
Sym_73 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Result); 
--# expr-done 9099
Is_Equal_72 := Local_Rel_Name = Sym_73; 
--# expr-done 9100
if Is_Equal_72 then
   --# expr-start 9101 '<FieldAccess for result_attr_equation at nodes.lkt:13006:41>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13006'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.result_attr_equation
Fld_21 := Libadalang.Implementation.Attribute_Ref_P_Result_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9101
   If_Result_72 := Fld_21;
else
   --# expr-start 9266 '<If at nodes.lkt:12986:9>' If_Result_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9102 '<Symbol Old at nodes.lkt:13007:26>' Sym_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13007'
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Old); 
--# expr-done 9102
Is_Equal_71 := Local_Rel_Name = Sym_71; 
if Is_Equal_71 then
   If_Result_70 := True;
else
--# expr-start 9103 '<Symbol Loop_Entry at nodes.lkt:13007:35>' Sym_72 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13007'
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Entry); 
--# expr-done 9103
Is_Equal_70 := Local_Rel_Name = Sym_72; 
   If_Result_70 := Is_Equal_70;
end if;
if If_Result_70 then
   --# expr-start 9104 '<FieldAccess for bind_to_prefix_eq at nodes.lkt:13007:54>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13007'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.bind_to_prefix_eq
Fld_22 := Libadalang.Implementation.Attribute_Ref_P_Bind_To_Prefix_Eq (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9104
   If_Result_71 := Fld_22;
else
   --# expr-start 9265 '<If at nodes.lkt:12986:9>' If_Result_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9105 '<Symbol Class at nodes.lkt:13008:26>' Sym_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13008'
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Class); 
--# expr-done 9105
Is_Equal_69 := Local_Rel_Name = Sym_69; 
if Is_Equal_69 then
   If_Result_68 := True;
else
--# expr-start 9106 '<Symbol Base at nodes.lkt:13008:37>' Sym_70 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13008'
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Base); 
--# expr-done 9106
Is_Equal_68 := Local_Rel_Name = Sym_70; 
   If_Result_68 := Is_Equal_68;
end if;
if If_Result_68 then
   --# expr-start 9108 '<FieldAccess for sub_equation at nodes.lkt:13008:50>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13008'
--# expr-start 9107 '<FieldAccess for prefix at nodes.lkt:13008:50>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13008'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9107
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_24 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_23.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_23.Info);
--# end
--# expr-done 9108
   If_Result_69 := Fld_24;
else
   --# expr-start 9264 '<If at nodes.lkt:12986:9>' If_Result_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9109 '<Symbol Valid at nodes.lkt:13009:26>' Sym_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13009'
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Valid); 
--# expr-done 9109
Is_Equal_67 := Local_Rel_Name = Sym_52; 
if Is_Equal_67 then
   If_Result_66 := True;
else
--# expr-start 9110 '<Symbol Machine_Overflows at nodes.lkt:13009:37>' Sym_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13009'
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Overflows); 
--# expr-done 9110
Is_Equal_66 := Local_Rel_Name = Sym_53; 
if Is_Equal_66 then
   If_Result_65 := True;
else
--# expr-start 9111 '<Symbol Machine_Rounds at nodes.lkt:13009:60>' Sym_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13009'
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounds); 
--# expr-done 9111
Is_Equal_65 := Local_Rel_Name = Sym_54; 
if Is_Equal_65 then
   If_Result_64 := True;
else
--# expr-start 9112 '<Symbol Has_Access_Values at nodes.lkt:13010:15>' Sym_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13010'
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Access_Values); 
--# expr-done 9112
Is_Equal_64 := Local_Rel_Name = Sym_55; 
if Is_Equal_64 then
   If_Result_63 := True;
else
--# expr-start 9113 '<Symbol Has_Discriminants at nodes.lkt:13010:38>' Sym_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13010'
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Discriminants); 
--# expr-done 9113
Is_Equal_63 := Local_Rel_Name = Sym_56; 
if Is_Equal_63 then
   If_Result_62 := True;
else
--# expr-start 9114 '<Symbol Has_Tagged_Values at nodes.lkt:13010:61>' Sym_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13010'
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Tagged_Values); 
--# expr-done 9114
Is_Equal_62 := Local_Rel_Name = Sym_57; 
if Is_Equal_62 then
   If_Result_61 := True;
else
--# expr-start 9115 '<Symbol Definite at nodes.lkt:13011:15>' Sym_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13011'
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Definite); 
--# expr-done 9115
Is_Equal_61 := Local_Rel_Name = Sym_58; 
if Is_Equal_61 then
   If_Result_60 := True;
else
--# expr-start 9116 '<Symbol Constrained at nodes.lkt:13011:29>' Sym_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13011'
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constrained); 
--# expr-done 9116
Is_Equal_60 := Local_Rel_Name = Sym_59; 
if Is_Equal_60 then
   If_Result_59 := True;
else
--# expr-start 9117 '<Symbol Initialized at nodes.lkt:13011:46>' Sym_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13011'
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initialized); 
--# expr-done 9117
Is_Equal_59 := Local_Rel_Name = Sym_60; 
if Is_Equal_59 then
   If_Result_58 := True;
else
--# expr-start 9118 '<Symbol Valid_Scalars at nodes.lkt:13011:63>' Sym_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13011'
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Valid_Scalars); 
--# expr-done 9118
Is_Equal_58 := Local_Rel_Name = Sym_61; 
if Is_Equal_58 then
   If_Result_57 := True;
else
--# expr-start 9119 '<Symbol Unconstrained_Array at nodes.lkt:13012:15>' Sym_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13012'
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unconstrained_Array); 
--# expr-done 9119
Is_Equal_57 := Local_Rel_Name = Sym_62; 
if Is_Equal_57 then
   If_Result_56 := True;
else
--# expr-start 9120 '<Symbol Library_Level at nodes.lkt:13012:40>' Sym_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13012'
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Library_Level); 
--# expr-done 9120
Is_Equal_56 := Local_Rel_Name = Sym_63; 
if Is_Equal_56 then
   If_Result_55 := True;
else
--# expr-start 9121 '<Symbol Preelaborable_Initialization at nodes.lkt:13013:15>' Sym_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13013'
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborable_Initialization); 
--# expr-done 9121
Is_Equal_55 := Local_Rel_Name = Sym_64; 
if Is_Equal_55 then
   If_Result_54 := True;
else
--# expr-start 9122 '<Symbol Denorm at nodes.lkt:13013:49>' Sym_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13013'
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Denorm); 
--# expr-done 9122
Is_Equal_54 := Local_Rel_Name = Sym_65; 
if Is_Equal_54 then
   If_Result_53 := True;
else
--# expr-start 9123 '<Symbol Signed_Zeros at nodes.lkt:13013:61>' Sym_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13013'
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Signed_Zeros); 
--# expr-done 9123
Is_Equal_53 := Local_Rel_Name = Sym_66; 
if Is_Equal_53 then
   If_Result_52 := True;
else
--# expr-start 9124 '<Symbol Fast_Math at nodes.lkt:13014:15>' Sym_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13014'
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fast_Math); 
--# expr-done 9124
Is_Equal_52 := Local_Rel_Name = Sym_67; 
if Is_Equal_52 then
   If_Result_51 := True;
else
--# expr-start 9125 '<Symbol Passed_By_Reference at nodes.lkt:13014:30>' Sym_68 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13014'
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Passed_By_Reference); 
--# expr-done 9125
Is_Equal_51 := Local_Rel_Name = Sym_68; 
   If_Result_51 := Is_Equal_51;
end if;
   If_Result_52 := If_Result_51;
end if;
   If_Result_53 := If_Result_52;
end if;
   If_Result_54 := If_Result_53;
end if;
   If_Result_55 := If_Result_54;
end if;
   If_Result_56 := If_Result_55;
end if;
   If_Result_57 := If_Result_56;
end if;
   If_Result_58 := If_Result_57;
end if;
   If_Result_59 := If_Result_58;
end if;
   If_Result_60 := If_Result_59;
end if;
   If_Result_61 := If_Result_60;
end if;
   If_Result_62 := If_Result_61;
end if;
   If_Result_63 := If_Result_62;
end if;
   If_Result_64 := If_Result_63;
end if;
   If_Result_65 := If_Result_64;
end if;
   If_Result_66 := If_Result_65;
end if;
if If_Result_66 then
   --# expr-start 9131 '<And at nodes.lkt:13015:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
--# expr-start 9127 '<FieldAccess for sub_equation at nodes.lkt:13015:13>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
--# expr-start 9126 '<FieldAccess for prefix at nodes.lkt:13015:13>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9126
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_25.Info);
--# end
--# expr-done 9127
--# expr-start 9130 '<Bind at nodes.lkt:13015:44>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
--# expr-start 9128 '<FieldAccess for type_var at nodes.lkt:13015:48>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9128
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 9129 '<FieldAccess for bool_type at nodes.lkt:13015:65>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13015'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_28 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 9129
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result := Solver.Create_Assign (Fld_27, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13015:44") else null)); 
--# expr-done 9130
And_Pred := Create_And (Fld_26, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13015:13") else null)); 
--# expr-done 9131
   If_Result_67 := And_Pred;
else
   --# expr-start 9263 '<If at nodes.lkt:12986:9>' If_Result_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9133 '<Eq at nodes.lkt:13020:14>' Is_Equal_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13020'
--# expr-start 9132 '<Symbol Enabled at nodes.lkt:13020:26>' Sym_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13020'
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enabled); 
--# expr-done 9132
Is_Equal_50 := Local_Rel_Name = Sym_51; 
--# expr-done 9133
if Is_Equal_50 then
   --# expr-start 9136 '<Bind at nodes.lkt:13020:42>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13020'
--# expr-start 9134 '<FieldAccess for type_var at nodes.lkt:13020:46>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9134
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 9135 '<FieldAccess for bool_type at nodes.lkt:13020:63>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_30 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 9135
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_29, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13020:42") else null)); 
--# expr-done 9136
   If_Result_50 := Bind_Result_1;
else
   --# expr-start 9262 '<If at nodes.lkt:12986:9>' If_Result_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9137 '<Symbol Width at nodes.lkt:13021:26>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Width); 
--# expr-done 9137
Is_Equal_49 := Local_Rel_Name = Sym_21; 
if Is_Equal_49 then
   If_Result_48 := True;
else
--# expr-start 9138 '<Symbol Component_Size at nodes.lkt:13021:37>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Component_Size); 
--# expr-done 9138
Is_Equal_48 := Local_Rel_Name = Sym_22; 
if Is_Equal_48 then
   If_Result_47 := True;
else
--# expr-start 9139 '<Symbol Position at nodes.lkt:13021:57>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Position); 
--# expr-done 9139
Is_Equal_47 := Local_Rel_Name = Sym_23; 
if Is_Equal_47 then
   If_Result_46 := True;
else
--# expr-start 9140 '<Symbol Mantissa at nodes.lkt:13021:71>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mantissa); 
--# expr-done 9140
Is_Equal_46 := Local_Rel_Name = Sym_24; 
if Is_Equal_46 then
   If_Result_45 := True;
else
--# expr-start 9141 '<Symbol Model_Mantissa at nodes.lkt:13021:85>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Mantissa); 
--# expr-done 9141
Is_Equal_45 := Local_Rel_Name = Sym_25; 
if Is_Equal_45 then
   If_Result_44 := True;
else
--# expr-start 9142 '<Symbol Machine_Mantissa at nodes.lkt:13021:105>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Mantissa); 
--# expr-done 9142
Is_Equal_44 := Local_Rel_Name = Sym_26; 
if Is_Equal_44 then
   If_Result_43 := True;
else
--# expr-start 9143 '<Symbol Fore at nodes.lkt:13021:127>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fore); 
--# expr-done 9143
Is_Equal_43 := Local_Rel_Name = Sym_27; 
if Is_Equal_43 then
   If_Result_42 := True;
else
--# expr-start 9144 '<Symbol Aft at nodes.lkt:13021:137>' Sym_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aft); 
--# expr-done 9144
Is_Equal_42 := Local_Rel_Name = Sym_28; 
if Is_Equal_42 then
   If_Result_41 := True;
else
--# expr-start 9145 '<Symbol Digits at nodes.lkt:13021:146>' Sym_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Digits); 
--# expr-done 9145
Is_Equal_41 := Local_Rel_Name = Sym_29; 
if Is_Equal_41 then
   If_Result_40 := True;
else
--# expr-start 9146 '<Symbol Modulus at nodes.lkt:13021:158>' Sym_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Modulus); 
--# expr-done 9146
Is_Equal_40 := Local_Rel_Name = Sym_30; 
if Is_Equal_40 then
   If_Result_39 := True;
else
--# expr-start 9147 '<Symbol Word_Size at nodes.lkt:13021:171>' Sym_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Word_Size); 
--# expr-done 9147
Is_Equal_39 := Local_Rel_Name = Sym_31; 
if Is_Equal_39 then
   If_Result_38 := True;
else
--# expr-start 9148 '<Symbol Wchar_T_Size at nodes.lkt:13021:186>' Sym_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wchar_T_Size); 
--# expr-done 9148
Is_Equal_38 := Local_Rel_Name = Sym_32; 
if Is_Equal_38 then
   If_Result_37 := True;
else
--# expr-start 9149 '<Symbol Max_Integer_Size at nodes.lkt:13021:204>' Sym_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max_Integer_Size); 
--# expr-done 9149
Is_Equal_37 := Local_Rel_Name = Sym_33; 
if Is_Equal_37 then
   If_Result_36 := True;
else
--# expr-start 9150 '<Symbol Address_Size at nodes.lkt:13021:226>' Sym_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address_Size); 
--# expr-done 9150
Is_Equal_36 := Local_Rel_Name = Sym_34; 
if Is_Equal_36 then
   If_Result_35 := True;
else
--# expr-start 9151 '<Symbol Maximum_Alignment at nodes.lkt:13021:244>' Sym_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Maximum_Alignment); 
--# expr-done 9151
Is_Equal_35 := Local_Rel_Name = Sym_35; 
if Is_Equal_35 then
   If_Result_34 := True;
else
--# expr-start 9152 '<Symbol System_Allocator_Alignment at nodes.lkt:13021:267>' Sym_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System_Allocator_Alignment); 
--# expr-done 9152
Is_Equal_34 := Local_Rel_Name = Sym_36; 
if Is_Equal_34 then
   If_Result_33 := True;
else
--# expr-start 9153 '<Symbol Finalization_Size at nodes.lkt:13021:299>' Sym_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Finalization_Size); 
--# expr-done 9153
Is_Equal_33 := Local_Rel_Name = Sym_37; 
if Is_Equal_33 then
   If_Result_32 := True;
else
--# expr-start 9154 '<Symbol Descriptor_Size at nodes.lkt:13021:322>' Sym_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Descriptor_Size); 
--# expr-done 9154
Is_Equal_32 := Local_Rel_Name = Sym_38; 
if Is_Equal_32 then
   If_Result_31 := True;
else
--# expr-start 9155 '<Symbol Alignment at nodes.lkt:13021:343>' Sym_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Alignment); 
--# expr-done 9155
Is_Equal_31 := Local_Rel_Name = Sym_39; 
if Is_Equal_31 then
   If_Result_30 := True;
else
--# expr-start 9156 '<Symbol First_Bit at nodes.lkt:13021:358>' Sym_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First_Bit); 
--# expr-done 9156
Is_Equal_30 := Local_Rel_Name = Sym_40; 
if Is_Equal_30 then
   If_Result_29 := True;
else
--# expr-start 9157 '<Symbol Last_Bit at nodes.lkt:13021:373>' Sym_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last_Bit); 
--# expr-done 9157
Is_Equal_29 := Local_Rel_Name = Sym_41; 
if Is_Equal_29 then
   If_Result_28 := True;
else
--# expr-start 9158 '<Symbol Default_Bit_Order at nodes.lkt:13021:387>' Sym_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Bit_Order); 
--# expr-done 9158
Is_Equal_28 := Local_Rel_Name = Sym_42; 
if Is_Equal_28 then
   If_Result_27 := True;
else
--# expr-start 9159 '<Symbol Range_Length at nodes.lkt:13021:410>' Sym_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range_Length); 
--# expr-done 9159
Is_Equal_27 := Local_Rel_Name = Sym_43; 
if Is_Equal_27 then
   If_Result_26 := True;
else
--# expr-start 9160 '<Symbol Storage_Unit at nodes.lkt:13021:428>' Sym_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Unit); 
--# expr-done 9160
Is_Equal_26 := Local_Rel_Name = Sym_44; 
if Is_Equal_26 then
   If_Result_25 := True;
else
--# expr-start 9161 '<Symbol Stream_Size at nodes.lkt:13021:446>' Sym_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Stream_Size); 
--# expr-done 9161
Is_Equal_25 := Local_Rel_Name = Sym_45; 
if Is_Equal_25 then
   If_Result_24 := True;
else
--# expr-start 9162 '<Symbol Small_Numerator at nodes.lkt:13021:463>' Sym_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small_Numerator); 
--# expr-done 9162
Is_Equal_24 := Local_Rel_Name = Sym_46; 
if Is_Equal_24 then
   If_Result_23 := True;
else
--# expr-start 9163 '<Symbol Small_Denominator at nodes.lkt:13021:484>' Sym_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Small_Denominator); 
--# expr-done 9163
Is_Equal_23 := Local_Rel_Name = Sym_47; 
if Is_Equal_23 then
   If_Result_22 := True;
else
--# expr-start 9164 '<Symbol Machine_Emin at nodes.lkt:13021:507>' Sym_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Emin); 
--# expr-done 9164
Is_Equal_22 := Local_Rel_Name = Sym_48; 
if Is_Equal_22 then
   If_Result_21 := True;
else
--# expr-start 9165 '<Symbol Machine_Emax at nodes.lkt:13021:525>' Sym_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Emax); 
--# expr-done 9165
Is_Equal_21 := Local_Rel_Name = Sym_49; 
if Is_Equal_21 then
   If_Result_20 := True;
else
--# expr-start 9166 '<Symbol Model_Emin at nodes.lkt:13021:543>' Sym_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model_Emin); 
--# expr-done 9166
Is_Equal_20 := Local_Rel_Name = Sym_50; 
   If_Result_20 := Is_Equal_20;
end if;
   If_Result_21 := If_Result_20;
end if;
   If_Result_22 := If_Result_21;
end if;
   If_Result_23 := If_Result_22;
end if;
   If_Result_24 := If_Result_23;
end if;
   If_Result_25 := If_Result_24;
end if;
   If_Result_26 := If_Result_25;
end if;
   If_Result_27 := If_Result_26;
end if;
   If_Result_28 := If_Result_27;
end if;
   If_Result_29 := If_Result_28;
end if;
   If_Result_30 := If_Result_29;
end if;
   If_Result_31 := If_Result_30;
end if;
   If_Result_32 := If_Result_31;
end if;
   If_Result_33 := If_Result_32;
end if;
   If_Result_34 := If_Result_33;
end if;
   If_Result_35 := If_Result_34;
end if;
   If_Result_36 := If_Result_35;
end if;
   If_Result_37 := If_Result_36;
end if;
   If_Result_38 := If_Result_37;
end if;
   If_Result_39 := If_Result_38;
end if;
   If_Result_40 := If_Result_39;
end if;
   If_Result_41 := If_Result_40;
end if;
   If_Result_42 := If_Result_41;
end if;
   If_Result_43 := If_Result_42;
end if;
   If_Result_44 := If_Result_43;
end if;
   If_Result_45 := If_Result_44;
end if;
   If_Result_46 := If_Result_45;
end if;
   If_Result_47 := If_Result_46;
end if;
   If_Result_48 := If_Result_47;
end if;
if If_Result_48 then
   --# expr-start 9171 '<And at nodes.lkt:13021:562>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
--# expr-start 9168 '<FieldAccess for sub_equation at nodes.lkt:13021:562>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
--# expr-start 9167 '<FieldAccess for prefix at nodes.lkt:13021:562>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9167
      if Fld_31.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_32 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_31.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_31.Info);
--# end
--# expr-done 9168
--# expr-start 9170 '<FieldAccess for universal_int_bind at nodes.lkt:13021:593>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9169 '<FieldAccess for type_var at nodes.lkt:13021:617>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13021'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9169
--# property-call-start AdaNode.universal_int_bind
Fld_34 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_33, Origin => Origin);
--# end
--# expr-done 9170
And_Pred_1 := Create_And (Fld_32, Fld_34, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13021:562") else null)); 
--# expr-done 9171
   If_Result_49 := And_Pred_1;
else
   --# expr-start 9261 '<If at nodes.lkt:12986:9>' If_Result_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9172 '<Symbol External_Tag at nodes.lkt:13022:26>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_External_Tag); 
--# expr-done 9172
Is_Equal_19 := Local_Rel_Name = Sym_19; 
if Is_Equal_19 then
   If_Result_18 := True;
else
--# expr-start 9173 '<Symbol Type_Key at nodes.lkt:13022:44>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Key); 
--# expr-done 9173
Is_Equal_18 := Local_Rel_Name = Sym_20; 
   If_Result_18 := Is_Equal_18;
end if;
if If_Result_18 then
   --# expr-start 9179 '<And at nodes.lkt:13022:61>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
--# expr-start 9175 '<FieldAccess for sub_equation at nodes.lkt:13022:61>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
--# expr-start 9174 '<FieldAccess for prefix at nodes.lkt:13022:61>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9174
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_35.Info);
--# end
--# expr-done 9175
--# expr-start 9178 '<Bind at nodes.lkt:13022:92>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
--# expr-start 9176 '<FieldAccess for type_var at nodes.lkt:13022:96>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9176
Fld_37.Value := No_Entity;
Entity_Vars.Reset (Fld_37);
--# expr-start 9177 '<FieldAccess for std_string_type at nodes.lkt:13022:113>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13022'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_38 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 9177
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_37, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13022:92") else null)); 
--# expr-done 9178
And_Pred_2 := Create_And (Fld_36, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13022:61") else null)); 
--# expr-done 9179
   If_Result_19 := And_Pred_2;
else
   --# expr-start 9260 '<If at nodes.lkt:12986:9>' If_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9181 '<Eq at nodes.lkt:13023:14>' Is_Equal_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13023'
--# expr-start 9180 '<Symbol Target_Name at nodes.lkt:13023:26>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13023'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Target_Name); 
--# expr-done 9180
Is_Equal_17 := Local_Rel_Name = Sym_18; 
--# expr-done 9181
if Is_Equal_17 then
   --# expr-start 9184 '<Bind at nodes.lkt:13023:46>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13023'
--# expr-start 9182 '<FieldAccess for type_var at nodes.lkt:13023:50>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13023'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9182
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
--# expr-start 9183 '<FieldAccess for std_string_type at nodes.lkt:13023:67>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13023'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_40 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 9183
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_40.Node,
         Info => Fld_40.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_39, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13023:46") else null)); 
--# expr-done 9184
   If_Result_17 := Bind_Result_3;
else
   --# expr-start 9259 '<If at nodes.lkt:12986:9>' If_Result_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9186 '<Eq at nodes.lkt:13024:14>' Is_Equal_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13024'
--# expr-start 9185 '<Symbol Storage_Pool at nodes.lkt:13024:26>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13024'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Pool); 
--# expr-done 9185
Is_Equal_16 := Local_Rel_Name = Sym_17; 
--# expr-done 9186
if Is_Equal_16 then
   --# expr-start 9187 '<FieldAccess for storage_pool_equation at nodes.lkt:13024:47>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13024'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.storage_pool_equation
Fld_41 := Libadalang.Implementation.Attribute_Ref_P_Storage_Pool_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9187
   If_Result_16 := Fld_41;
else
   --# expr-start 9258 '<If at nodes.lkt:12986:9>' If_Result_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9188 '<Symbol Bit_Order at nodes.lkt:13025:26>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13025'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bit_Order); 
--# expr-done 9188
Is_Equal_15 := Local_Rel_Name = Sym_14; 
if Is_Equal_15 then
   If_Result_14 := True;
else
--# expr-start 9189 '<Symbol Scalar_Storage_Order at nodes.lkt:13025:41>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13025'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scalar_Storage_Order); 
--# expr-done 9189
Is_Equal_14 := Local_Rel_Name = Sym_15; 
if Is_Equal_14 then
   If_Result_13 := True;
else
--# expr-start 9190 '<Symbol Default_Scalar_Storage_Order at nodes.lkt:13025:67>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13025'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Default_Scalar_Storage_Order); 
--# expr-done 9190
Is_Equal_13 := Local_Rel_Name = Sym_16; 
   If_Result_13 := Is_Equal_13;
end if;
   If_Result_14 := If_Result_13;
end if;
if If_Result_14 then
   --# expr-start 9191 '<FieldAccess for order_equation at nodes.lkt:13025:104>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13025'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.order_equation
Fld_42 := Libadalang.Implementation.Attribute_Ref_P_Order_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9191
   If_Result_15 := Fld_42;
else
   --# expr-start 9257 '<If at nodes.lkt:12986:9>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9193 '<Eq at nodes.lkt:13026:14>' Is_Equal_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13026'
--# expr-start 9192 '<Symbol Type_Class at nodes.lkt:13026:26>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13026'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Class); 
--# expr-done 9192
Is_Equal_12 := Local_Rel_Name = Sym_13; 
--# expr-done 9193
if Is_Equal_12 then
   --# expr-start 9194 '<FieldAccess for type_class_equation at nodes.lkt:13026:45>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13026'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.type_class_equation
Fld_43 := Libadalang.Implementation.Attribute_Ref_P_Type_Class_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9194
   If_Result_12 := Fld_43;
else
   --# expr-start 9256 '<If at nodes.lkt:12986:9>' If_Result_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9195 '<Symbol Callable at nodes.lkt:13028:26>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Callable); 
--# expr-done 9195
Is_Equal_11 := Local_Rel_Name = Sym_11; 
if Is_Equal_11 then
   If_Result_10 := True;
else
--# expr-start 9196 '<Symbol Terminated at nodes.lkt:13028:40>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Terminated); 
--# expr-done 9196
Is_Equal_10 := Local_Rel_Name = Sym_12; 
   If_Result_10 := Is_Equal_10;
end if;
if If_Result_10 then
   --# expr-start 9202 '<And at nodes.lkt:13028:59>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
--# expr-start 9198 '<FieldAccess for sub_equation at nodes.lkt:13028:59>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
--# expr-start 9197 '<FieldAccess for prefix at nodes.lkt:13028:59>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_44 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9197
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_45 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_44.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_44.Info);
--# end
--# expr-done 9198
--# expr-start 9201 '<Bind at nodes.lkt:13028:90>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
--# expr-start 9199 '<FieldAccess for type_var at nodes.lkt:13028:94>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_46 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9199
Fld_46.Value := No_Entity;
Entity_Vars.Reset (Fld_46);
--# expr-start 9200 '<FieldAccess for bool_type at nodes.lkt:13028:111>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13028'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_47 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 9200
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_47.Node,
         Info => Fld_47.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_46, Cast_Result_8, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13028:90") else null)); 
--# expr-done 9201
And_Pred_3 := Create_And (Fld_45, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13028:59") else null)); 
--# expr-done 9202
   If_Result_11 := And_Pred_3;
else
   --# expr-start 9255 '<If at nodes.lkt:12986:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9204 '<Eq at nodes.lkt:13030:14>' Is_Equal_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13030'
--# expr-start 9203 '<Symbol Count at nodes.lkt:13030:26>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13030'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Count); 
--# expr-done 9203
Is_Equal_9 := Local_Rel_Name = Sym_10; 
--# expr-done 9204
if Is_Equal_9 then
   --# expr-start 9211 '<And at nodes.lkt:13030:40>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13030'
--# expr-start 9208 '<Match at nodes.lkt:13030:40>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13030'
--# expr-start 9205 '<FieldAccess for prefix at nodes.lkt:13030:46>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13030'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_48 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9205
      if Fld_48.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_48; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_9; 
--# bind ce Match_0
--# expr-start 9206 '<FieldAccess for sub_equation at nodes.lkt:13033:34>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13033'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_49 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 9206
Let_Result := Fld_49; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3198;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Attribute_Ref | Ada_Defining_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_10; 
--# bind o Match_1
--# expr-start 9207 '<FieldAccess for xref_no_overloading at nodes.lkt:13036:23>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13036'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_50 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 9207
Let_Result_1 := Fld_50; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3199;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 9208
--# expr-start 9210 '<FieldAccess for universal_int_bind at nodes.lkt:13037:15>' Fld_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13037'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9209 '<FieldAccess for type_var at nodes.lkt:13037:39>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13037'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_51 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9209
--# property-call-start AdaNode.universal_int_bind
Fld_52 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_51, Origin => Origin);
--# end
--# expr-done 9210
And_Pred_4 := Create_And (Match_Result, Fld_52, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13030:40") else null)); 
--# expr-done 9211
   If_Result_9 := And_Pred_4;
else
   --# expr-start 9254 '<If at nodes.lkt:12986:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9213 '<Eq at nodes.lkt:13038:14>' Is_Equal_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
--# expr-start 9212 '<Symbol Caller at nodes.lkt:13038:26>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Caller); 
--# expr-done 9212
Is_Equal_8 := Local_Rel_Name = Sym_9; 
--# expr-done 9213
if Is_Equal_8 then
   --# expr-start 9219 '<And at nodes.lkt:13038:41>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
--# expr-start 9215 '<FieldAccess for xref_no_overloading at nodes.lkt:13038:41>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
--# expr-start 9214 '<FieldAccess for prefix at nodes.lkt:13038:41>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_53 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9214
      if Fld_53.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_54 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_53.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_53.Info);
--# end
--# expr-done 9215
--# expr-start 9218 '<Bind at nodes.lkt:13038:79>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
--# expr-start 9216 '<FieldAccess for type_var at nodes.lkt:13038:83>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9216
Fld_55.Value := No_Entity;
Entity_Vars.Reset (Fld_55);
--# expr-start 9217 '<FieldAccess for task_id_type at nodes.lkt:13038:100>' Fld_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13038'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.task_id_type
Fld_56 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 9217
      Cast_Result_11 := Create_Internal_Entity
        (Node => Fld_56.Node,
         Info => Fld_56.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_55, Cast_Result_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13038:79") else null)); 
--# expr-done 9218
And_Pred_5 := Create_And (Fld_54, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13038:41") else null)); 
--# expr-done 9219
   If_Result_8 := And_Pred_5;
else
   --# expr-start 9253 '<If at nodes.lkt:12986:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9221 '<Eq at nodes.lkt:13039:14>' Is_Equal_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13039'
--# expr-start 9220 '<Symbol Machine_Radix at nodes.lkt:13039:26>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13039'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Radix); 
--# expr-done 9220
Is_Equal_7 := Local_Rel_Name = Sym_8; 
--# expr-done 9221
if Is_Equal_7 then
   --# expr-start 9222 '<FieldAccess for universal_int_equation at nodes.lkt:13039:48>' Fld_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13039'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.universal_int_equation
Fld_57 := Libadalang.Implementation.Attribute_Ref_P_Universal_Int_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9222
   If_Result_7 := Fld_57;
else
   --# expr-start 9252 '<If at nodes.lkt:12986:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9224 '<Eq at nodes.lkt:13040:14>' Is_Equal_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13040'
--# expr-start 9223 '<Symbol To_Address at nodes.lkt:13040:26>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13040'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_To_Address); 
--# expr-done 9223
Is_Equal_6 := Local_Rel_Name = Sym_7; 
--# expr-done 9224
if Is_Equal_6 then
   --# expr-start 9225 '<FieldAccess for to_address_equation at nodes.lkt:13040:45>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13040'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.to_address_equation
Fld_58 := Libadalang.Implementation.Attribute_Ref_P_To_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9225
   If_Result_6 := Fld_58;
else
   --# expr-start 9251 '<If at nodes.lkt:12986:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9227 '<Eq at nodes.lkt:13041:14>' Is_Equal_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13041'
--# expr-start 9226 '<Symbol Index at nodes.lkt:13041:26>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13041'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Index); 
--# expr-done 9226
Is_Equal_5 := Local_Rel_Name = Sym_6; 
--# expr-done 9227
if Is_Equal_5 then
   --# expr-start 9228 '<FieldAccess for index_equation at nodes.lkt:13041:40>' Fld_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13041'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.index_equation
Fld_59 := Libadalang.Implementation.Attribute_Ref_P_Index_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9228
   If_Result_5 := Fld_59;
else
   --# expr-start 9250 '<If at nodes.lkt:12986:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9230 '<Eq at nodes.lkt:13042:14>' Is_Equal_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
--# expr-start 9229 '<Symbol Abort_Signal at nodes.lkt:13042:26>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort_Signal); 
--# expr-done 9229
Is_Equal_4 := Local_Rel_Name = Sym_5; 
--# expr-done 9230
if Is_Equal_4 then
   --# expr-start 9237 '<And at nodes.lkt:13042:47>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
--# expr-start 9234 '<Bind at nodes.lkt:13042:47>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
--# expr-start 9231 '<FieldAccess for ref_var at nodes.lkt:13042:51>' Fld_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_60 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9231
Fld_60.Value := No_Entity;
Entity_Vars.Reset (Fld_60);
--# expr-start 9233 '<FieldAccess for std_entity at nodes.lkt:13042:67>' Fld_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9232 '<Symbol abort_signal_ at nodes.lkt:13042:83>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abort_Signal_315); 
--# expr-done 9232
--# property-call-start AdaNode.std_entity
Fld_61 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 9233
Bind_Result_6 := Solver.Create_Assign (Fld_60, Fld_61, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13042:47") else null)); 
--# expr-done 9234
--# expr-start 9236 '<Bind at nodes.lkt:13042:106>' Bind_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
--# expr-start 9235 '<FieldAccess for type_var at nodes.lkt:13042:110>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13042'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_62 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9235
Fld_62.Value := No_Entity;
Entity_Vars.Reset (Fld_62);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result_12 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_7 := Solver.Create_Assign (Fld_62, Cast_Result_12, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13042:106") else null)); 
--# expr-done 9236
And_Pred_6 := Create_And (Bind_Result_6, Bind_Result_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13042:47") else null)); 
--# expr-done 9237
   If_Result_4 := And_Pred_6;
else
   --# expr-start 9249 '<If at nodes.lkt:12986:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9238 '<Symbol Has_Same_Storage at nodes.lkt:13043:26>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13043'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 9238
Is_Equal_3 := Local_Rel_Name = Sym_3; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 9239 '<Symbol Overlaps_Storage at nodes.lkt:13043:48>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13043'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 9239
Is_Equal_2 := Local_Rel_Name = Sym_4; 
   If_Result_2 := Is_Equal_2;
end if;
if If_Result_2 then
   --# expr-start 9240 '<FieldAccess for storage_equation at nodes.lkt:13043:73>' Fld_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13043'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.storage_equation
Fld_63 := Libadalang.Implementation.Attribute_Ref_P_Storage_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9240
   If_Result_3 := Fld_63;
else
   --# expr-start 9248 '<If at nodes.lkt:12986:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9242 '<Eq at nodes.lkt:13044:14>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13044'
--# expr-start 9241 '<Symbol Deref at nodes.lkt:13044:26>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13044'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Deref); 
--# expr-done 9241
Is_Equal_1 := Local_Rel_Name = Sym_2; 
--# expr-done 9242
if Is_Equal_1 then
   --# expr-start 9243 '<FieldAccess for deref_equation at nodes.lkt:13044:40>' Fld_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13044'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.deref_equation
Fld_64 := Libadalang.Implementation.Attribute_Ref_P_Deref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9243
   If_Result_1 := Fld_64;
else
   --# expr-start 9247 '<If at nodes.lkt:12986:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:12986'
--# expr-start 9245 '<Eq at nodes.lkt:13045:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13045'
--# expr-start 9244 '<Symbol Mechanism_Code at nodes.lkt:13045:26>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13045'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mechanism_Code); 
--# expr-done 9244
Is_Equal := Local_Rel_Name = Sym_1; 
--# expr-done 9245
if Is_Equal then
   --# expr-start 9246 '<FieldAccess for mechanism_code_equation at nodes.lkt:13045:49>' Fld_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13045'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.mechanism_code_equation
Fld_65 := Libadalang.Implementation.Attribute_Ref_P_Mechanism_Code_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9246
   If_Result := Fld_65;
else
   If_Result := raise Property_Error with "Unhandled attribute";
end if;
      Inc_Ref (If_Result);
--# expr-done 9247
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 9248
   If_Result_3 := If_Result_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9249
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 9250
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 9251
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 9252
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 9253
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 9254
   If_Result_9 := If_Result_8;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 9255
   If_Result_11 := If_Result_9;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 9256
   If_Result_12 := If_Result_11;
end if;
      Inc_Ref (If_Result_12);
--# expr-done 9257
   If_Result_15 := If_Result_12;
end if;
      Inc_Ref (If_Result_15);
--# expr-done 9258
   If_Result_16 := If_Result_15;
end if;
      Inc_Ref (If_Result_16);
--# expr-done 9259
   If_Result_17 := If_Result_16;
end if;
      Inc_Ref (If_Result_17);
--# expr-done 9260
   If_Result_19 := If_Result_17;
end if;
      Inc_Ref (If_Result_19);
--# expr-done 9261
   If_Result_49 := If_Result_19;
end if;
      Inc_Ref (If_Result_49);
--# expr-done 9262
   If_Result_50 := If_Result_49;
end if;
      Inc_Ref (If_Result_50);
--# expr-done 9263
   If_Result_67 := If_Result_50;
end if;
      Inc_Ref (If_Result_67);
--# expr-done 9264
   If_Result_69 := If_Result_67;
end if;
      Inc_Ref (If_Result_69);
--# expr-done 9265
   If_Result_71 := If_Result_69;
end if;
      Inc_Ref (If_Result_71);
--# expr-done 9266
   If_Result_72 := If_Result_71;
end if;
      Inc_Ref (If_Result_72);
--# expr-done 9267
   If_Result_73 := If_Result_72;
end if;
      Inc_Ref (If_Result_73);
--# expr-done 9268
   If_Result_74 := If_Result_73;
end if;
      Inc_Ref (If_Result_74);
--# expr-done 9269
   If_Result_84 := If_Result_74;
end if;
      Inc_Ref (If_Result_84);
--# expr-done 9270
   If_Result_86 := If_Result_84;
end if;
      Inc_Ref (If_Result_86);
--# expr-done 9271
   If_Result_87 := If_Result_86;
end if;
      Inc_Ref (If_Result_87);
--# expr-done 9272
   If_Result_90 := If_Result_87;
end if;
      Inc_Ref (If_Result_90);
--# expr-done 9273
   If_Result_91 := If_Result_90;
end if;
      Inc_Ref (If_Result_91);
--# expr-done 9274
   If_Result_92 := If_Result_91;
end if;
      Inc_Ref (If_Result_92);
--# expr-done 9275
   If_Result_93 := If_Result_92;
end if;
      Inc_Ref (If_Result_93);
--# expr-done 9276
   If_Result_94 := If_Result_93;
end if;
      Inc_Ref (If_Result_94);
--# expr-done 9277
   If_Result_97 := If_Result_94;
end if;
      Inc_Ref (If_Result_97);
--# expr-done 9278
   If_Result_104 := If_Result_97;
end if;
      Inc_Ref (If_Result_104);
--# expr-done 9279
   If_Result_106 := If_Result_104;
end if;
      Inc_Ref (If_Result_106);
--# expr-done 9280
   If_Result_143 := If_Result_106;
end if;
      Inc_Ref (If_Result_143);
--# expr-done 9281
   If_Result_147 := If_Result_143;
end if;
      Inc_Ref (If_Result_147);
--# expr-done 9282
Let_Result_2 := If_Result_147; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9283
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1162;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1162;
                     Finalizer_Scope_3198;
                     Finalizer_Scope_3199;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Xref_Equation;
--# end
--# property-start AttributeRef.attribute_subprogram_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13051'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Attribute_Subprogram_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1163 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1163 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1163;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9289 '<And at nodes.lkt:13052:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
--# expr-start 9285 '<FieldAccess for xref_type_equation at nodes.lkt:13052:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
--# expr-start 9284 '<FieldAccess for prefix at nodes.lkt:13052:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9284
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_1 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9285
--# expr-start 9288 '<Bind at nodes.lkt:13052:46>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
--# expr-start 9286 '<FieldAccess for ref_var at nodes.lkt:13052:50>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9286
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 9287 '<FieldAccess for attribute_subprogram at nodes.lkt:13052:66>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13052'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9287
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13052:46") else null)); 
--# expr-done 9288
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13052:9") else null)); 
--# expr-done 9289
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1163;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1163;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Attribute_Subprogram_Equation;
--# end
--# property-start AttributeRef.type_class_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13057'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Type_Class_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1164 with Inline_Always;
            procedure Finalizer_Scope_3200 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_243 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1164 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1164;
            procedure Finalizer_Scope_3200 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3200;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9303 '<Let at nodes.lkt:13058:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13058'
--# scope-start
--# expr-start 9297 '<Cast to BaseTypeDecl at nodes.lkt:13058:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13058'
--# expr-start 9293 '<FieldAccess for get_unit_root_decl at nodes.lkt:13058:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13058'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9292 '<ArrayLiteral at nodes.lkt:13059:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13059'
--# expr-start 9290 '<Symbol System at nodes.lkt:13059:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13059'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 9290
--# expr-start 9291 '<Symbol Aux_DEC at nodes.lkt:13059:25>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13059'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Aux_Dec); 
--# expr-done 9291
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 9292
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 9293
Var_Expr_243 := Fld;
if Var_Expr_243 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9296 '<EnvGet at nodes.lkt:13058:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13058'
--# expr-start 9294 '<FieldAccess for children_env at nodes.lkt:13058:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13058'
      if Var_Expr_243 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_243);
--# expr-done 9294
--# expr-start 9295 '<Symbol Type_Class at nodes.lkt:13060:37>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13060'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Type_Class); 
--# expr-done 9295
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9296
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_3200;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9297
Local_Typ := Cast_Result; 
--# bind typ Local_Typ
--# expr-start 9302 '<And at nodes.lkt:13062:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13062'
--# expr-start 9299 '<FieldAccess for xref_equation at nodes.lkt:13062:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13062'
--# expr-start 9298 '<FieldAccess for prefix at nodes.lkt:13062:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13062'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9298
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9299
--# expr-start 9301 '<Bind at nodes.lkt:13062:41>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13062'
--# expr-start 9300 '<FieldAccess for type_var at nodes.lkt:13062:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13062'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9300
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13062:41") else null)); 
--# expr-done 9301
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13062:9") else null)); 
--# expr-done 9302
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9303
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1164;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1164;
                     Finalizer_Scope_3200;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Type_Class_Equation;
--# end
--# property-start AttributeRef.storage_pool_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13067'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Pool_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1165 with Inline_Always;
            procedure Finalizer_Scope_3201 with Inline_Always;
      Local_Typ : Internal_Entity_Classwide_Type_Decl;
Var_Expr_244 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1165 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1165;
            procedure Finalizer_Scope_3201 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3201;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9318 '<Let at nodes.lkt:13068:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
--# scope-start
--# expr-start 9312 '<FieldAccess for classwide_type at nodes.lkt:13068:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
--# expr-start 9311 '<Cast to BaseTypeDecl at nodes.lkt:13068:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
--# expr-start 9307 '<FieldAccess for get_unit_root_decl at nodes.lkt:13068:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9306 '<ArrayLiteral at nodes.lkt:13069:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13069'
--# expr-start 9304 '<Symbol System at nodes.lkt:13069:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13069'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 9304
--# expr-start 9305 '<Symbol Storage_Pools at nodes.lkt:13069:25>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13069'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Pools); 
--# expr-done 9305
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 9306
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 9307
Var_Expr_244 := Fld;
if Var_Expr_244 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9310 '<EnvGet at nodes.lkt:13068:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
--# expr-start 9308 '<FieldAccess for children_env at nodes.lkt:13068:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13068'
      if Var_Expr_244 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_244);
--# expr-done 9308
--# expr-start 9309 '<Symbol Root_Storage_Pool at nodes.lkt:13071:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13071'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Root_Storage_Pool); 
--# expr-done 9309
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9310
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_3201;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9311
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 9312
Local_Typ := Fld_2; 
--# bind typ Local_Typ
--# expr-start 9317 '<And at nodes.lkt:13074:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13074'
--# expr-start 9314 '<FieldAccess for xref_equation at nodes.lkt:13074:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13074'
--# expr-start 9313 '<FieldAccess for prefix at nodes.lkt:13074:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13074'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9313
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 9314
--# expr-start 9316 '<Bind at nodes.lkt:13074:41>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13074'
--# expr-start 9315 '<FieldAccess for type_var at nodes.lkt:13074:45>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13074'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9315
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13074:41") else null)); 
--# expr-done 9316
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13074:9") else null)); 
--# expr-done 9317
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9318
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1165;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1165;
                     Finalizer_Scope_3201;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Storage_Pool_Equation;
--# end
--# property-start AttributeRef.order_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13079'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Order_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1166 with Inline_Always;
            procedure Finalizer_Scope_3202 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_245 : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1166 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1166;
            procedure Finalizer_Scope_3202 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3202;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9331 '<Let at nodes.lkt:13080:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13080'
--# scope-start
--# expr-start 9325 '<Cast to BaseTypeDecl at nodes.lkt:13080:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13080'
--# expr-start 9321 '<FieldAccess for get_unit_root_decl at nodes.lkt:13080:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13080'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9320 '<ArrayLiteral at nodes.lkt:13081:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13081'
--# expr-start 9319 '<Symbol System at nodes.lkt:13081:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13081'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 9319
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 9320
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 9321
Var_Expr_245 := Fld;
if Var_Expr_245 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9324 '<EnvGet at nodes.lkt:13080:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13080'
--# expr-start 9322 '<FieldAccess for children_env at nodes.lkt:13080:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13080'
      if Var_Expr_245 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_245);
--# expr-done 9322
--# expr-start 9323 '<Symbol Bit_Order at nodes.lkt:13082:37>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13082'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Bit_Order); 
--# expr-done 9323
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_1), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9324
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_3202;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9325
Local_Typ := Cast_Result; 
--# bind typ Local_Typ
--# expr-start 9330 '<And at nodes.lkt:13084:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13084'
--# expr-start 9327 '<FieldAccess for xref_equation at nodes.lkt:13084:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13084'
--# expr-start 9326 '<FieldAccess for prefix at nodes.lkt:13084:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9326
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9327
--# expr-start 9329 '<Bind at nodes.lkt:13084:41>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13084'
--# expr-start 9328 '<FieldAccess for type_var at nodes.lkt:13084:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13084'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9328
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13084:41") else null)); 
--# expr-done 9329
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13084:9") else null)); 
--# expr-done 9330
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9331
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1166;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1166;
                     Finalizer_Scope_3202;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Order_Equation;
--# end
--# property-start AttributeRef.bind_to_prefix_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13088'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Bind_To_Prefix_Eq
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1167 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1167 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1167;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9343 '<And at nodes.lkt:13089:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9338 '<And at nodes.lkt:13089:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9335 '<Bind at nodes.lkt:13089:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9333 '<FieldAccess for expected_type_var at nodes.lkt:13089:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9332 '<FieldAccess for prefix at nodes.lkt:13089:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 9332
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 9333
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 9334 '<FieldAccess for expected_type_var at nodes.lkt:13089:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9334
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13089:9") else null)); 
--# expr-done 9335
--# expr-start 9337 '<FieldAccess for sub_equation at nodes.lkt:13089:76>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9336 '<FieldAccess for prefix at nodes.lkt:13089:76>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9336
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 9337
And_Pred := Create_And (Bind_Result, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13089:9") else null)); 
--# expr-done 9338
--# expr-start 9342 '<Bind at nodes.lkt:13089:107>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9339 '<FieldAccess for type_var at nodes.lkt:13089:111>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9339
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 9341 '<FieldAccess for type_var at nodes.lkt:13089:128>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
--# expr-start 9340 '<FieldAccess for prefix at nodes.lkt:13089:128>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9340
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 9341
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_Unify (Fld_5, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13089:107") else null)); 
--# expr-done 9342
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13089:9") else null)); 
--# expr-done 9343
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1167;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1167;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Bind_To_Prefix_Eq;
--# end
--# property-start AttributeRef.result_attr_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13092'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Result_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1168 with Inline_Always;
      Local_Containing_Decl : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Concrete_Type_Decl;
Local_Returns : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Concrete_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Env_Node : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Concrete_Type_Decl;
Fld : Bare_Type_Def;
Is_A_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Find_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Type_Def;
Cast_Result_2 : Internal_Entity_Access_To_Subp_Def;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Concrete_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Match_Result : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_1168 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1168;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9366 '<Let at nodes.lkt:13097:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
--# scope-start
--# expr-start 9352 '<Cast to BasicDecl at nodes.lkt:13097:31>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
--# expr-start 9351 '<as_entity at nodes.lkt:13097:31>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
--# expr-start 9350 '<Find at nodes.lkt:13097:31>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
   --# expr-start 9345 '<FieldAccess for parents at nodes.lkt:13097:31>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
--# expr-start 9344 '<env_node at nodes.lkt:13097:31>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13097'
Env_Node := AST_Envs.Env_Node (Env); 
--# expr-done 9344
      if Env_Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Env_Node, True); 
--# expr-done 9345
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
--# expr-start 9346 '<IsA BasicSubpDecl, BaseSubpBody at nodes.lkt:13098:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13098'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 9346
if Is_A then
   If_Result := True;
else
--# expr-start 9347 '<Cast to ConcreteTypeDecl at nodes.lkt:13098:57>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13098'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Concrete_Type_Decl_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 9347
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 9349 '<IsA AccessToSubpDef at nodes.lkt:13098:91>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13098'
--# expr-start 9348 '<FieldAccess for type_def at nodes.lkt:13098:91>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13098'
      if Var_Expr_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Var_Expr_1.Type_Decl_F_Type_Def;
--# expr-done 9348
Is_A_1 := Fld /= null 
and then Fld.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 9349
Scope_Result := Is_A_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
      if If_Result then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when If_Result;
         end loop;
      end;
--# expr-done 9350
Is_Null := Find_Result = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Find_Result); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 9351
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 9352
Local_Containing_Decl := Cast_Result_1; 
--# bind containing_decl Local_Containing_Decl
--# expr-start 9365 '<Let at nodes.lkt:13100:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13100'
--# scope-start
--# expr-start 9357 '<Match at nodes.lkt:13100:23>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13100'
      if Local_Containing_Decl.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Containing_Decl; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Concrete_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Concrete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind sd Match_0
--# expr-start 9355 '<FieldAccess for subp_spec at nodes.lkt:13101:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13101'
--# expr-start 9354 '<Cast to AccessToSubpDef at nodes.lkt:13101:42>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13101'
--# expr-start 9353 '<FieldAccess for type_def at nodes.lkt:13101:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13101'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 9353
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_2 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 9354
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Cast_Result_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Cast_Result_2.Info);
--# expr-done 9355
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result := Cast_Result_4; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Synth_Anonymous_Type_Decl | Ada_Formal_Type_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind bd Match_1
--# expr-start 9356 '<FieldAccess for subp_spec_or_null at nodes.lkt:13102:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13102'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 9356
Let_Result_1 := Fld_3; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 9357
Var_Expr_2 := Match_Result;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 9358 '<FieldAccess for return_type at nodes.lkt:13103:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13103'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 9358
Scope_Result_3 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Local_Returns := Result_Var_1; 
--# bind returns Local_Returns
--# expr-start 9364 '<And at nodes.lkt:13105:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
--# expr-start 9360 '<Bind at nodes.lkt:13105:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
--# expr-start 9359 '<FieldAccess for type_var at nodes.lkt:13105:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9359
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr_1 := Local_Returns; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13105:9") else null)); 
--# expr-done 9360
--# expr-start 9363 '<Bind at nodes.lkt:13105:43>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
--# expr-start 9362 '<FieldAccess for ref_var at nodes.lkt:13105:47>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
--# expr-start 9361 '<FieldAccess for prefix at nodes.lkt:13105:47>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13105'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9361
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6.Node);
--# end
--# expr-done 9362
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Cast_Expr_2 := Local_Containing_Decl; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_7, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13105:43") else null)); 
--# expr-done 9363
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13105:9") else null)); 
--# expr-done 9364
Let_Result_2 := And_Pred; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9365
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 9366
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1168;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1168;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Result_Attr_Equation;
--# end
--# property-start AttributeRef.tag_attr_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13109'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Tag_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1169 with Inline_Always;
            procedure Finalizer_Scope_3208 with Inline_Always;
      Local_Tag_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_246 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1169 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1169;
            procedure Finalizer_Scope_3208 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3208;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9380 '<Let at nodes.lkt:13110:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13110'
--# scope-start
--# expr-start 9374 '<Cast to BaseTypeDecl at nodes.lkt:13110:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13110'
--# expr-start 9370 '<FieldAccess for get_unit_root_decl at nodes.lkt:13110:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13110'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9369 '<ArrayLiteral at nodes.lkt:13111:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13111'
--# expr-start 9367 '<Symbol Ada at nodes.lkt:13111:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13111'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ada); 
--# expr-done 9367
--# expr-start 9368 '<Symbol Tags at nodes.lkt:13111:22>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13111'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tags); 
--# expr-done 9368
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 9369
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 9370
Var_Expr_246 := Fld;
if Var_Expr_246 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9373 '<EnvGet at nodes.lkt:13110:24>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13110'
--# expr-start 9371 '<FieldAccess for children_env at nodes.lkt:13110:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13110'
      if Var_Expr_246 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_246);
--# expr-done 9371
--# expr-start 9372 '<Symbol Tag at nodes.lkt:13112:37>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13112'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Tag); 
--# expr-done 9372
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9373
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_3208;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9374
Local_Tag_Type := Cast_Result; 
--# bind tag_type Local_Tag_Type
--# expr-start 9379 '<And at nodes.lkt:13114:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13114'
--# expr-start 9376 '<FieldAccess for xref_equation at nodes.lkt:13116:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13116'
--# expr-start 9375 '<FieldAccess for prefix at nodes.lkt:13116:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13116'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9375
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9376
--# expr-start 9378 '<Bind at nodes.lkt:13119:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13119'
--# expr-start 9377 '<FieldAccess for type_var at nodes.lkt:13119:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13119'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9377
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Tag_Type; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13119:13") else null)); 
--# expr-done 9378
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13114:9") else null)); 
--# expr-done 9379
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9380
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1169;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1169;
                     Finalizer_Scope_3208;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Tag_Attr_Equation;
--# end
--# property-start AttributeRef.address_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13124'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1170 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1170 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1170;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9393 '<And at nodes.lkt:13128:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13128'
--# expr-start 9389 '<Or at nodes.lkt:13129:13>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13129'
--# expr-start 9386 '<And at nodes.lkt:13130:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
--# expr-start 9382 '<FieldAccess for xref_no_overloading at nodes.lkt:13130:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
--# expr-start 9381 '<FieldAccess for prefix at nodes.lkt:13130:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9381
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9382
--# expr-start 9385 '<Predicate on BasicDecl.is_subprogram at nodes.lkt:13130:55>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
--# expr-start 9384 '<FieldAccess for ref_var at nodes.lkt:13130:91>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
--# expr-start 9383 '<FieldAccess for prefix at nodes.lkt:13130:91>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13130'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9383
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 9384
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred := Solver.Create_Predicate (Fld_3, Create_Basic_Decl_P_Is_Subprogram_0_Predicate); 
--# expr-done 9385
And_Pred := Create_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13130:17") else null)); 
--# expr-done 9386
--# expr-start 9388 '<FieldAccess for sub_equation at nodes.lkt:13131:18>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13131'
--# expr-start 9387 '<FieldAccess for prefix at nodes.lkt:13131:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13131'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9387
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 9388
Or_Pred := Create_Or (And_Pred, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13129:13") else null)); 
--# expr-done 9389
--# expr-start 9392 '<Bind at nodes.lkt:13132:15>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13132'
--# expr-start 9390 '<FieldAccess for type_var at nodes.lkt:13132:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13132'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9390
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 9391 '<FieldAccess for system_address_type at nodes.lkt:13132:36>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13132'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.system_address_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_System_Address_Type (Node => Ent.Node);
--# end
--# expr-done 9391
      Cast_Result := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13132:15") else null)); 
--# expr-done 9392
And_Pred_1 := Create_And (Or_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13128:9") else null)); 
--# expr-done 9393
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1170;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1170;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Address_Equation;
--# end
--# property-start AttributeRef.identity_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13135'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Identity_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1171 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1171 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1171;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9400 '<And at nodes.lkt:13138:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
--# expr-start 9395 '<FieldAccess for sub_equation at nodes.lkt:13138:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
--# expr-start 9394 '<FieldAccess for prefix at nodes.lkt:13138:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9394
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9395
--# expr-start 9399 '<Bind at nodes.lkt:13138:40>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
--# expr-start 9397 '<FieldAccess for ref_var at nodes.lkt:13138:61>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
--# expr-start 9396 '<FieldAccess for prefix at nodes.lkt:13138:61>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9396
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 9397
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 9398 '<FieldAccess for type_var at nodes.lkt:13138:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13138'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9398
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Propagate (Fld_4, Fld_2, Logic_Functor_681'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13138:40") else null)); 
--# expr-done 9399
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13138:9") else null)); 
--# expr-done 9400
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1171;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1171;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Identity_Equation;
--# end
--# property-start AttributeRef.universal_real_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13141'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Real_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1172 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1172 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1172;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9405 '<And at nodes.lkt:13142:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13142'
--# expr-start 9402 '<FieldAccess for universal_real_bind at nodes.lkt:13142:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13142'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9401 '<FieldAccess for type_var at nodes.lkt:13142:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13142'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9401
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 9402
--# expr-start 9404 '<FieldAccess for sub_equation at nodes.lkt:13142:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13142'
--# expr-start 9403 '<FieldAccess for prefix at nodes.lkt:13142:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13142'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9403
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9404
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13142:9") else null)); 
--# expr-done 9405
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1172;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1172;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Universal_Real_Equation;
--# end
--# property-start AttributeRef.universal_int_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13145'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Int_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1173 with Inline_Always;
            procedure Finalizer_Scope_3209 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
Item_0 : Internal_Entity_Basic_Assoc;
Bare_Item_0 : Bare_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1173 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1173;
            procedure Finalizer_Scope_3209 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3209;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9426 '<Let at nodes.lkt:13146:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13146'
--# scope-start
--# expr-start 9407 '<FieldAccess for name_designated_type at nodes.lkt:13146:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13146'
--# expr-start 9406 '<FieldAccess for prefix at nodes.lkt:13146:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13146'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9406
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9407
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9425 '<And at nodes.lkt:13148:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13148'
--# expr-start 9412 '<And at nodes.lkt:13149:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13149'
--# expr-start 9409 '<FieldAccess for sub_equation at nodes.lkt:13149:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13149'
--# expr-start 9408 '<FieldAccess for prefix at nodes.lkt:13149:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13149'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9408
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9409
--# expr-start 9411 '<FieldAccess for universal_int_bind at nodes.lkt:13149:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13149'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9410 '<FieldAccess for type_var at nodes.lkt:13149:68>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13149'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9410
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 9411
And_Pred := Create_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13149:13") else null)); 
--# expr-done 9412
--# expr-start 9424 '<LogicAll at nodes.lkt:13150:15>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13150'
--# expr-start 9423 '<Map at nodes.lkt:13150:15>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13150'
   --# expr-start 9413 '<FieldAccess for args at nodes.lkt:13150:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13150'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 9413
Fld_7 := Fld_6.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_7 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind arg Item_0
      --# expr-start 9422 '<And at nodes.lkt:13151:22>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13151'
--# expr-start 9419 '<And at nodes.lkt:13152:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
--# expr-start 9416 '<Bind at nodes.lkt:13152:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
--# expr-start 9415 '<FieldAccess for expected_type_var at nodes.lkt:13152:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
--# expr-start 9414 '<FieldAccess for expr at nodes.lkt:13152:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 9414
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9.Node);
--# expr-done 9415
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_10, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13152:17") else null)); 
--# expr-done 9416
--# expr-start 9418 '<FieldAccess for sub_equation at nodes.lkt:13152:62>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
--# expr-start 9417 '<FieldAccess for expr at nodes.lkt:13152:62>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13152'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 9417
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_11.Info);
--# end
--# expr-done 9418
And_Pred_1 := Create_And (Bind_Result, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13152:17") else null)); 
--# expr-done 9419
--# expr-start 9421 '<FieldAccess for matches_expected_type at nodes.lkt:13153:19>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13153'
--# expr-start 9420 '<FieldAccess for expr at nodes.lkt:13153:19>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13153'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 9420
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 9421
And_Pred_2 := Create_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13151:22") else null)); 
--# expr-done 9422
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3209;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9423
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13150:15") else null)); 
--# expr-done 9424
And_Pred_3 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13148:9") else null)); 
--# expr-done 9425
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 9426
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1173;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1173;
                     Finalizer_Scope_3209;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Universal_Int_Equation;
--# end
--# property-start AttributeRef.image_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13158'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Image_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1174 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1174 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1174;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9442 '<Let at nodes.lkt:13159:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13159'
--# scope-start
--# expr-start 9428 '<FieldAccess for name_designated_type at nodes.lkt:13159:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13159'
--# expr-start 9427 '<FieldAccess for prefix at nodes.lkt:13159:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13159'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9427
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9428
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9441 '<If at nodes.lkt:13161:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13161'
--# expr-start 9429 '<IsNull at nodes.lkt:13161:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13161'
Is_Null := Local_Typ.Node = null; 
--# expr-done 9429
if Is_Null then
   --# expr-start 9434 '<And at nodes.lkt:13163:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13163'
--# expr-start 9431 '<FieldAccess for sub_equation at nodes.lkt:13163:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13163'
--# expr-start 9430 '<FieldAccess for prefix at nodes.lkt:13163:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13163'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9430
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9431
--# expr-start 9433 '<Bind at nodes.lkt:13163:44>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13163'
--# expr-start 9432 '<FieldAccess for type_var at nodes.lkt:13163:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13163'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9432
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Assign (Fld_4, Str_Type, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13163:44") else null)); 
--# expr-done 9433
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13163:13") else null)); 
--# expr-done 9434
   If_Result := And_Pred;
else
   --# expr-start 9440 '<And at nodes.lkt:13164:16>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
--# expr-start 9436 '<FieldAccess for xref_type_equation at nodes.lkt:13164:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
--# expr-start 9435 '<FieldAccess for prefix at nodes.lkt:13164:16>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9435
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_6 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_5.Info);
--# end
--# expr-done 9436
--# expr-start 9439 '<Bind at nodes.lkt:13164:53>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
--# expr-start 9437 '<FieldAccess for ref_var at nodes.lkt:13164:57>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9437
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 9438 '<FieldAccess for attribute_subprogram at nodes.lkt:13164:73>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13164'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9438
      Cast_Result := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_7, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13164:53") else null)); 
--# expr-done 9439
And_Pred_1 := Create_And (Fld_6, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13164:16") else null)); 
--# expr-done 9440
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 9441
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9442
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1174;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1174;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Image_Equation;
--# end
--# property-start AttributeRef.img_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13168'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Img_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1175 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1175 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1175;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9447 '<And at nodes.lkt:13168:61>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13168'
--# expr-start 9444 '<FieldAccess for xref_equation at nodes.lkt:13170:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13170'
--# expr-start 9443 '<FieldAccess for prefix at nodes.lkt:13170:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13170'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9443
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9444
--# expr-start 9446 '<Bind at nodes.lkt:13173:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13173'
--# expr-start 9445 '<FieldAccess for type_var at nodes.lkt:13173:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13173'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9445
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Assign (Fld_2, Str_Type, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13173:9") else null)); 
--# expr-done 9446
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13168:61") else null)); 
--# expr-done 9447
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1175;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1175;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Img_Equation;
--# end
--# property-start AttributeRef.enum_rep_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13177'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Enum_Rep_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1176 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1176 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1176;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9464 '<Let at nodes.lkt:13178:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13178'
--# scope-start
--# expr-start 9449 '<FieldAccess for name_designated_type at nodes.lkt:13178:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13178'
--# expr-start 9448 '<FieldAccess for prefix at nodes.lkt:13178:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13178'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9448
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9449
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9463 '<If at nodes.lkt:13180:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13180'
--# expr-start 9450 '<IsNull at nodes.lkt:13180:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13180'
Is_Null := Local_Typ.Node = null; 
--# expr-done 9450
if Is_Null then
   --# expr-start 9456 '<And at nodes.lkt:13182:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
--# expr-start 9452 '<FieldAccess for sub_equation at nodes.lkt:13182:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
--# expr-start 9451 '<FieldAccess for prefix at nodes.lkt:13182:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9451
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9452
--# expr-start 9455 '<Bind at nodes.lkt:13182:44>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
--# expr-start 9453 '<FieldAccess for type_var at nodes.lkt:13182:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9453
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 9454 '<FieldAccess for universal_int_type at nodes.lkt:13182:65>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13182'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 9454
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13182:44") else null)); 
--# expr-done 9455
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13182:13") else null)); 
--# expr-done 9456
   If_Result := And_Pred;
else
   --# expr-start 9462 '<And at nodes.lkt:13183:16>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
--# expr-start 9458 '<FieldAccess for xref_type_equation at nodes.lkt:13183:16>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
--# expr-start 9457 '<FieldAccess for prefix at nodes.lkt:13183:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9457
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_7 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 9458
--# expr-start 9461 '<Bind at nodes.lkt:13183:53>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
--# expr-start 9459 '<FieldAccess for ref_var at nodes.lkt:13183:57>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9459
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 9460 '<FieldAccess for attribute_subprogram at nodes.lkt:13183:73>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13183'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AttributeRef.attribute_subprogram
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Attribute_Subprogram (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9460
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13183:53") else null)); 
--# expr-done 9461
And_Pred_1 := Create_And (Fld_7, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13183:16") else null)); 
--# expr-done 9462
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 9463
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9464
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1176;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1176;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Enum_Rep_Equation;
--# end
--# property-start AttributeRef.self_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13189'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Self_Type_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1177 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1177 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1177;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9472 '<Let at nodes.lkt:13190:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13190'
--# scope-start
--# expr-start 9466 '<FieldAccess for name_designated_type at nodes.lkt:13190:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13190'
--# expr-start 9465 '<FieldAccess for prefix at nodes.lkt:13190:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13190'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9465
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9466
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9471 '<And at nodes.lkt:13192:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13192'
--# expr-start 9468 '<FieldAccess for xref_type_equation at nodes.lkt:13192:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13192'
--# expr-start 9467 '<FieldAccess for prefix at nodes.lkt:13192:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13192'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9467
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_3 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9468
--# expr-start 9470 '<Bind at nodes.lkt:13192:46>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13192'
--# expr-start 9469 '<FieldAccess for type_var at nodes.lkt:13192:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13192'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9469
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13192:46") else null)); 
--# expr-done 9470
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13192:9") else null)); 
--# expr-done 9471
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9472
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1177;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1177;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Self_Type_Equation;
--# end
--# property-start AttributeRef.access_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13196'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Access_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1178 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Fld_13 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_14 : Bare_Name;
Fld_15 : Logic_Var;
Fld_16 : Bare_Name;
Fld_17 : Logic_Var;
Bind_Result_3 : Logic_Equation;
Fld_18 : Bare_Name;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_20 : Internal_Entity_Name;
Fld_21 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Bare_Name;
Fld_23 : Logic_Var;
Fld_24 : Logic_Var;
Bind_Result_5 : Logic_Equation;
Fld_25 : Internal_Entity_Name;
Fld_26 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Bare_Name;
Fld_29 : Logic_Var;
Bind_Result_6 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_30 : Logic_Var;
Fld_31 : Bare_Name;
Fld_32 : Logic_Var;
Bind_Result_7 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_33 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_34 : Bare_Ada_Node;
Is_A_1 : Boolean;
Fld_35 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1178 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (Fld_26);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Or_Pred_2);
            end Finalizer_Scope_1178;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9532 '<Or at nodes.lkt:13196:39>' Or_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13196'
--# expr-start 9483 '<And at nodes.lkt:13198:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13198'
--# expr-start 9479 '<And at nodes.lkt:13198:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13198'
--# expr-start 9474 '<FieldAccess for xref_no_overloading at nodes.lkt:13198:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13198'
--# expr-start 9473 '<FieldAccess for prefix at nodes.lkt:13198:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13198'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9473
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, All_Els => True, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9474
--# expr-start 9478 '<Predicate on BaseTypeDecl.is_subp_access_of at nodes.lkt:13199:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13199'
--# expr-start 9475 '<FieldAccess for type_var at nodes.lkt:13201:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13201'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9475
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 9477 '<FieldAccess for ref_var at nodes.lkt:13201:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13201'
--# expr-start 9476 '<FieldAccess for prefix at nodes.lkt:13201:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13201'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9476
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 9477
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_N_Predicate ((Fld_2, Fld_4), Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (Origin)); 
--# expr-done 9478
And_Pred := Create_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13198:9") else null)); 
--# expr-done 9479
--# expr-start 9482 '<Bind at nodes.lkt:13203:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13203'
--# expr-start 9480 '<FieldAccess for type_var at nodes.lkt:13203:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13203'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9480
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 9481 '<FieldAccess for expected_type_var at nodes.lkt:13203:34>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13203'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9481
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Bind_Result := Solver.Create_Unify (Fld_5, Fld_6, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13203:13") else null)); 
--# expr-done 9482
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13198:9") else null)); 
--# expr-done 9483
--# expr-start 9531 '<And at nodes.lkt:13206:9>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13206'
--# expr-start 9485 '<FieldAccess for sub_equation at nodes.lkt:13206:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13206'
--# expr-start 9484 '<FieldAccess for prefix at nodes.lkt:13206:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13206'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9484
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 9485
--# expr-start 9530 '<Or at nodes.lkt:13215:13>' Or_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
--# expr-start 9506 '<And at nodes.lkt:13215:13>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
--# expr-start 9493 '<And at nodes.lkt:13215:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
--# expr-start 9488 '<Bind at nodes.lkt:13215:13>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
--# expr-start 9486 '<FieldAccess for expected_type_var at nodes.lkt:13215:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9486
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 9487 '<FieldAccess for type_var at nodes.lkt:13215:43>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13215'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9487
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Bind_Result_1 := Solver.Create_Unify (Fld_9, Fld_10, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13215:13") else null)); 
--# expr-done 9488
--# expr-start 9492 '<Bind at nodes.lkt:13216:17>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13216'
--# expr-start 9489 '<FieldAccess for expected_type_var at nodes.lkt:13216:54>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13216'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9489
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 9491 '<FieldAccess for expected_type_var at nodes.lkt:13216:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13216'
--# expr-start 9490 '<FieldAccess for prefix at nodes.lkt:13216:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13216'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9490
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_11);
--# expr-done 9491
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_2 := Solver.Create_Propagate (Fld_13, Fld_12, Logic_Functor_885'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13216:17") else null)); 
--# expr-done 9492
And_Pred_2 := Create_And (Bind_Result_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13215:13") else null)); 
--# expr-done 9493
--# expr-start 9505 '<Or at nodes.lkt:13223:17>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
--# expr-start 9502 '<And at nodes.lkt:13223:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
--# expr-start 9498 '<Bind at nodes.lkt:13223:17>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
--# expr-start 9495 '<FieldAccess for expected_type_var at nodes.lkt:13223:21>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
--# expr-start 9494 '<FieldAccess for prefix at nodes.lkt:13223:21>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9494
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_14);
--# expr-done 9495
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
--# expr-start 9497 '<FieldAccess for type_var at nodes.lkt:13223:54>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
--# expr-start 9496 '<FieldAccess for prefix at nodes.lkt:13223:54>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13223'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9496
      if Fld_16 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_16);
--# expr-done 9497
Fld_17.Value := No_Entity;
Entity_Vars.Reset (Fld_17);
Bind_Result_3 := Solver.Create_Unify (Fld_15, Fld_17, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13223:17") else null)); 
--# expr-done 9498
--# expr-start 9501 '<Bind at nodes.lkt:13224:21>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13224'
--# expr-start 9500 '<FieldAccess for type_var at nodes.lkt:13224:25>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13224'
--# expr-start 9499 '<FieldAccess for prefix at nodes.lkt:13224:25>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13224'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9499
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 9500
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_19, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13224:21") else null)); 
--# expr-done 9501
And_Pred_3 := Create_And (Bind_Result_3, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13223:17") else null)); 
--# expr-done 9502
--# expr-start 9504 '<FieldAccess for matches_expected_formal_type at nodes.lkt:13228:20>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13228'
--# expr-start 9503 '<FieldAccess for prefix at nodes.lkt:13228:20>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13228'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9503
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_21 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_20.Node, Origin => Origin);
--# end
--# expr-done 9504
Or_Pred := Create_Or (And_Pred_3, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13223:17") else null)); 
--# expr-done 9505
And_Pred_4 := Create_And (And_Pred_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13215:13") else null)); 
--# expr-done 9506
--# expr-start 9529 '<If at nodes.lkt:13236:13>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13236'
--# expr-start 9508 '<IsA DottedName at nodes.lkt:13236:16>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13236'
--# expr-start 9507 '<parent at nodes.lkt:13236:16>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13236'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Self.Parent; 
--# expr-done 9507
Is_A_1 := Fld_34 /= null 
and then Fld_34.Kind in Ada_Dotted_Name_Range; 
--# expr-done 9508
if Is_A_1 then
   --# expr-start 9509 '<FieldAccess for is_prefix at nodes.lkt:13236:46>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13236'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_35 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 9509
   If_Result_1 := Fld_35;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 9521 '<And at nodes.lkt:13237:17>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
--# expr-start 9516 '<And at nodes.lkt:13237:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
--# expr-start 9513 '<Bind at nodes.lkt:13237:17>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
--# expr-start 9511 '<FieldAccess for expected_type_var at nodes.lkt:13237:21>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
--# expr-start 9510 '<FieldAccess for prefix at nodes.lkt:13237:21>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9510
      if Fld_22 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_22);
--# expr-done 9511
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
--# expr-start 9512 '<FieldAccess for expected_type_var at nodes.lkt:13237:54>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13237'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 9512
Fld_24.Value := No_Entity;
Entity_Vars.Reset (Fld_24);
Bind_Result_5 := Solver.Create_Unify (Fld_23, Fld_24, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13237:17") else null)); 
--# expr-done 9513
--# expr-start 9515 '<FieldAccess for matches_expected_prefix_type at nodes.lkt:13238:21>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13238'
--# expr-start 9514 '<FieldAccess for prefix at nodes.lkt:13238:21>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13238'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9514
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_prefix_type
Fld_26 := Libadalang.Implementation.Expr_P_Matches_Expected_Prefix_Type (Node => Fld_25.Node, Origin => Origin);
--# end
--# expr-done 9515
And_Pred_5 := Create_And (Bind_Result_5, Fld_26, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13237:17") else null)); 
--# expr-done 9516
--# expr-start 9520 '<Bind at nodes.lkt:13239:21>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13239'
--# expr-start 9518 '<FieldAccess for type_var at nodes.lkt:13239:42>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13239'
--# expr-start 9517 '<FieldAccess for prefix at nodes.lkt:13239:42>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13239'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9517
      if Fld_28 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_28);
--# expr-done 9518
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 9519 '<FieldAccess for type_var at nodes.lkt:13239:25>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13239'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9519
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
Bind_Result_6 := Solver.Create_Propagate (Fld_29, Fld_27, Logic_Functor_843'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13239:21") else null)); 
--# expr-done 9520
And_Pred_6 := Create_And (And_Pred_5, Bind_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13237:17") else null)); 
--# expr-done 9521
   If_Result_2 := And_Pred_6;
else
   --# expr-start 9528 '<If at nodes.lkt:13236:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13236'
--# expr-start 9523 '<IsA ExplicitDeref at nodes.lkt:13241:18>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13241'
--# expr-start 9522 '<parent at nodes.lkt:13241:18>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13241'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Self.Parent; 
--# expr-done 9522
Is_A := Fld_33 /= null 
and then Fld_33.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 9523
if Is_A then
   --# expr-start 9527 '<Bind at nodes.lkt:13242:17>' Bind_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13242'
--# expr-start 9525 '<FieldAccess for type_var at nodes.lkt:13242:38>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13242'
--# expr-start 9524 '<FieldAccess for prefix at nodes.lkt:13242:38>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13242'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9524
      if Fld_31 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_31);
--# expr-done 9525
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
--# expr-start 9526 '<FieldAccess for type_var at nodes.lkt:13242:21>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13242'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9526
Fld_30.Value := No_Entity;
Entity_Vars.Reset (Fld_30);
Bind_Result_7 := Solver.Create_Propagate (Fld_32, Fld_30, Logic_Functor_843'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13242:17") else null)); 
--# expr-done 9527
   If_Result := Bind_Result_7;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13244:18") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 9528
   If_Result_2 := If_Result;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9529
Or_Pred_1 := Create_Or (And_Pred_4, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13215:13") else null)); 
--# expr-done 9530
And_Pred_7 := Create_And (Fld_8, Or_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13206:9") else null)); 
--# expr-done 9531
Or_Pred_2 := Create_Or (And_Pred_1, And_Pred_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13196:39") else null)); 
--# expr-done 9532
         Property_Result := Or_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1178;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1178;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Access_Equation;
--# end
--# property-start AttributeRef.size_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13249'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Size_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1179 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1179 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1179;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9549 '<Let at nodes.lkt:13250:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13250'
--# scope-start
--# expr-start 9534 '<FieldAccess for name_designated_type at nodes.lkt:13250:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13250'
--# expr-start 9533 '<FieldAccess for prefix at nodes.lkt:13250:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13250'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9533
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9534
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9548 '<If at nodes.lkt:13252:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9536 '<Not at nodes.lkt:13252:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9535 '<IsNull at nodes.lkt:13252:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
Is_Null := Local_Typ.Node = null; 
--# expr-done 9535
Not_Val := not (Is_Null); 
--# expr-done 9536
if Not_Val then
   --# expr-start 9542 '<And at nodes.lkt:13252:33>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9539 '<Bind at nodes.lkt:13252:33>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9538 '<FieldAccess for ref_var at nodes.lkt:13252:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9537 '<FieldAccess for prefix at nodes.lkt:13252:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9537
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 9538
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Cast_Expr := Local_Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13252:33") else null)); 
--# expr-done 9539
--# expr-start 9541 '<FieldAccess for universal_int_bind at nodes.lkt:13252:69>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9540 '<FieldAccess for type_var at nodes.lkt:13252:93>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9540
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 9541
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13252:33") else null)); 
--# expr-done 9542
   If_Result := And_Pred;
else
   --# expr-start 9547 '<And at nodes.lkt:13252:115>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9544 '<FieldAccess for sub_equation at nodes.lkt:13252:115>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
--# expr-start 9543 '<FieldAccess for prefix at nodes.lkt:13252:115>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9543
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 9544
--# expr-start 9546 '<FieldAccess for universal_int_bind at nodes.lkt:13252:146>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9545 '<FieldAccess for type_var at nodes.lkt:13252:170>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13252'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9545
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 9546
And_Pred_1 := Create_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13252:115") else null)); 
--# expr-done 9547
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 9548
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 9549
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1179;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1179;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Size_Equation;
--# end
--# property-start AttributeRef.array_attr_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13256'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Array_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1180 with Inline_Always;
            procedure Finalizer_Scope_3211 with Inline_Always;
      Local_Is_Length : Boolean;
Local_Typ : Internal_Entity_Base_Type_Decl;
Local_Dim : Integer;
Var_Expr_0 : Internal_Entity_Basic_Assoc;
Var_Expr_1 : Internal_Entity_Expr;
Ignored : Boolean;
Local_Res : Boolean;
Local_Pfx_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Boolean;
Fld_9 : Big_Integer_Type;
Small_Int : Integer;
Let_Result : Integer;
Scope_Result : Integer;
Result_Var : Integer;
Scope_Result_1 : Integer;
Result_Var_1 : Integer;
Arith_Result : Integer;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_17 : Boolean;
Fld_18 : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Logic_Equation;
Fld_19 : Boolean;
If_Result_4 : Logic_Equation;
Fld_20 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Internal_Entity_Name;
Fld_22 : Internal_Entity_Name;
Fld_23 : Logic_Var;
Pred : Logic_Equation;
Fld_24 : Boolean;
Fld_25 : Internal_Entity_Name;
Fld_26 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_27 : Logic_Var;
Fld_28 : Logic_Equation;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
If_Result_7 : Logic_Equation;
False_Rel_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
Let_Result_5 : Logic_Equation;
            procedure Finalizer_Scope_1180 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_1180;
            procedure Finalizer_Scope_3211 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_3211;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9602 '<Let at nodes.lkt:13257:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13257'
--# scope-start
--# expr-start 9552 '<FieldAccess for name_is at nodes.lkt:13257:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13257'
--# expr-start 9550 '<FieldAccess for attribute at nodes.lkt:13257:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13257'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 9550
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9551 '<Symbol Length at nodes.lkt:13257:48>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13257'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 9551
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 9552
Local_Is_Length := Fld_1; 
--# bind is_length Local_Is_Length
--# expr-start 9601 '<Let at nodes.lkt:13258:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13258'
--# scope-start
--# expr-start 9554 '<FieldAccess for name_designated_type at nodes.lkt:13258:19>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13258'
--# expr-start 9553 '<FieldAccess for prefix at nodes.lkt:13258:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13258'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9553
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 9554
Local_Typ := Fld_3; 
--# bind typ Local_Typ
--# expr-start 9600 '<Let at nodes.lkt:13261:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13261'
--# scope-start
--# expr-start 9561 '<Arithmetic - at nodes.lkt:13261:19>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13261'
--# expr-start 9555 '<FieldAccess for args at nodes.lkt:13261:19>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13261'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 9555
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind arg Var_Expr_0
--# expr-start 9556 '<FieldAccess for expr at nodes.lkt:13262:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13262'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9556
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_1
--# expr-start 9560 '<Let at nodes.lkt:13264:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13264'
--# scope-start
--# expr-start 9557 '<FieldAccess for resolve_names_internal at nodes.lkt:13264:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13264'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_8 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Var_Expr_1.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9557
Ignored := Fld_8; 
--# bind _ Ignored
--# expr-start 9559 '<as_int at nodes.lkt:13266:21>' Small_Int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13266'
--# expr-start 9558 '<FieldAccess for eval_as_int at nodes.lkt:13266:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13266'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_9 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9558
Small_Int := To_Integer (Self, Fld_9); 
--# expr-done 9559
Let_Result := Small_Int; 
--# end
--# expr-done 9560
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_3211;
   Result_Var := Scope_Result;
else
   Result_Var := 1;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := 1;
end if;
Arith_Result := (Result_Var_1 - 1); 
--# expr-done 9561
Local_Dim := Arith_Result; 
--# bind dim Local_Dim
--# expr-start 9599 '<If at nodes.lkt:13271:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13271'
--# expr-start 9563 '<Not at nodes.lkt:13271:12>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13271'
--# expr-start 9562 '<IsNull at nodes.lkt:13271:16>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13271'
Is_Null_1 := Local_Typ.Node = null; 
--# expr-done 9562
Not_Val_1 := not (Is_Null_1); 
--# expr-done 9563
if Not_Val_1 then
   --# expr-start 9581 '<And at nodes.lkt:13273:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13273'
--# expr-start 9565 '<FieldAccess for xref_type_equation at nodes.lkt:13273:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13273'
--# expr-start 9564 '<FieldAccess for prefix at nodes.lkt:13273:13>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13273'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9564
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_11 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 9565
--# expr-start 9580 '<If at nodes.lkt:13274:17>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
--# expr-start 9566 '<FieldAccess for is_array_def_with_deref at nodes.lkt:13274:20>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_def_with_deref
Fld_20 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Def_With_Deref (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 9566
if Fld_20 then
   If_Result_5 := Local_Is_Length;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 9568 '<FieldAccess for universal_int_bind at nodes.lkt:13274:69>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9567 '<FieldAccess for type_var at nodes.lkt:13274:93>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9567
--# property-call-start AdaNode.universal_int_bind
Fld_13 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_12, Origin => Origin);
--# end
--# expr-done 9568
   If_Result_6 := Fld_13;
else
   --# expr-start 9579 '<If at nodes.lkt:13274:17>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
--# expr-start 9569 '<FieldAccess for is_array_def_with_deref at nodes.lkt:13276:22>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13276'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_array_def_with_deref
Fld_19 := Libadalang.Implementation.Base_Type_Decl_P_Is_Array_Def_With_Deref (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 9569
if Fld_19 then
   --# expr-start 9572 '<Bind at nodes.lkt:13276:57>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13276'
--# expr-start 9570 '<FieldAccess for type_var at nodes.lkt:13276:61>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13276'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9570
Fld_14.Value := No_Entity;
Entity_Vars.Reset (Fld_14);
--# expr-start 9571 '<FieldAccess for index_type at nodes.lkt:13276:78>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13276'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Local_Typ.Node, Dim => Local_Dim, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 9571
      Cast_Result := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result := Solver.Create_Assign (Fld_14, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13276:57") else null)); 
--# expr-done 9572
   If_Result_4 := Bind_Result;
else
   --# expr-start 9578 '<If at nodes.lkt:13274:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13274'
--# expr-start 9573 '<FieldAccess for is_discrete_type at nodes.lkt:13278:22>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13278'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 9573
if Fld_17 then
   If_Result_2 := True;
else
--# expr-start 9574 '<FieldAccess for is_real_type at nodes.lkt:13278:49>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13278'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Local_Typ.Node, Origin => Origin, E_Info => Local_Typ.Info);
--# end
--# expr-done 9574
if Fld_18 then
   --# expr-start 9575 '<Not at nodes.lkt:13278:72>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13278'
Not_Val := not (Local_Is_Length); 
--# expr-done 9575
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
   --# expr-start 9577 '<Bind at nodes.lkt:13278:92>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13278'
--# expr-start 9576 '<FieldAccess for type_var at nodes.lkt:13278:96>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13278'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9576
Fld_16.Value := No_Entity;
Entity_Vars.Reset (Fld_16);
Cast_Expr := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_16, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13278:92") else null)); 
--# expr-done 9577
   If_Result_3 := Bind_Result_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13279:22") else null)); 
   If_Result_3 := False_Rel;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9578
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 9579
   If_Result_6 := If_Result_4;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 9580
And_Pred := Create_And (Fld_11, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13273:13") else null)); 
--# expr-done 9581
   If_Result_9 := And_Pred;
else
   --# expr-start 9598 '<Let at nodes.lkt:13284:13>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13284'
--# scope-start
--# expr-start 9586 '<FieldAccess for resolve_names_internal_with_eq at nodes.lkt:13284:23>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13284'
--# expr-start 9582 '<FieldAccess for prefix at nodes.lkt:13284:23>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13284'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9582
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9585 '<Predicate on BaseTypeDecl.is_array_def_with_deref at nodes.lkt:13285:17>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13285'
--# expr-start 9584 '<FieldAccess for type_var at nodes.lkt:13285:66>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13285'
--# expr-start 9583 '<FieldAccess for prefix at nodes.lkt:13285:66>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13285'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9583
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_22.Node);
--# expr-done 9584
Fld_23.Value := No_Entity;
Entity_Vars.Reset (Fld_23);
Pred := Solver.Create_Predicate (Fld_23, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin)); 
--# expr-done 9585
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_24 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_21.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld_21.Info);
--# end
--# expr-done 9586
Local_Res := Fld_24; 
--# bind res Local_Res
--# expr-start 9597 '<Let at nodes.lkt:13287:13>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13287'
--# scope-start
--# expr-start 9589 '<Cast to BaseTypeDecl at nodes.lkt:13287:27>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13287'
--# expr-start 9588 '<FieldAccess for type_val at nodes.lkt:13287:27>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13287'
--# expr-start 9587 '<FieldAccess for prefix at nodes.lkt:13287:27>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13287'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9587
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_26 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_25.Node);
--# end
--# expr-done 9588
   if Fld_26.Node = null
      or else Fld_26.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9589
Local_Pfx_Typ := Cast_Result_2; 
--# bind pfx_typ Local_Pfx_Typ
--# expr-start 9596 '<If at nodes.lkt:13289:13>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
if Local_Res then
   --# expr-start 9595 '<If at nodes.lkt:13289:25>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
if Local_Is_Length then
   --# expr-start 9591 '<FieldAccess for universal_int_bind at nodes.lkt:13289:43>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9590 '<FieldAccess for type_var at nodes.lkt:13289:67>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9590
--# property-call-start AdaNode.universal_int_bind
Fld_28 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_27, Origin => Origin);
--# end
--# expr-done 9591
   If_Result_7 := Fld_28;
else
   --# expr-start 9594 '<Bind at nodes.lkt:13289:89>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
--# expr-start 9592 '<FieldAccess for type_var at nodes.lkt:13289:93>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9592
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 9593 '<FieldAccess for index_type at nodes.lkt:13289:110>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13289'
      if Local_Pfx_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_30 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Local_Pfx_Typ.Node, Dim => Local_Dim, Origin => Origin, E_Info => Local_Pfx_Typ.Info);
--# end
--# expr-done 9593
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_29, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13289:89") else null)); 
--# expr-done 9594
   If_Result_7 := Bind_Result_2;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 9595
   If_Result_8 := If_Result_7;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13289:140") else null)); 
   If_Result_8 := False_Rel_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 9596
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 9597
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9598
   If_Result_9 := Let_Result_2;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 9599
Let_Result_3 := If_Result_9; Inc_Ref (Let_Result_3);
--# end
--# expr-done 9600
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 9601
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 9602
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1180;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1180;
                     Finalizer_Scope_3211;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Array_Attr_Equation;
--# end
--# property-start AttributeRef.subtype_attr_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13296'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Subtype_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1181 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1181 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1181;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9610 '<And at nodes.lkt:13297:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
--# expr-start 9607 '<Bind at nodes.lkt:13297:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
--# expr-start 9604 '<FieldAccess for ref_var at nodes.lkt:13297:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
--# expr-start 9603 '<FieldAccess for prefix at nodes.lkt:13297:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 9603
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9604
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 9606 '<FieldAccess for name_designated_type at nodes.lkt:13297:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
--# expr-start 9605 '<FieldAccess for prefix at nodes.lkt:13297:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9605
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 9606
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13297:9") else null)); 
--# expr-done 9607
--# expr-start 9609 '<FieldAccess for universal_int_bind at nodes.lkt:13297:76>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9608 '<FieldAccess for type_var at nodes.lkt:13297:100>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9608
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 9609
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13297:9") else null)); 
--# expr-done 9610
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1181;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1181;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Subtype_Attr_Equation;
--# end
--# property-start AttributeRef.to_address_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13301'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_To_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1182 with Inline_Always;
            procedure Finalizer_Scope_3212 with Inline_Always;
      Local_To_Address_Subp : Internal_Entity_Basic_Subp_Decl;
Var_Expr_247 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Subp_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Subp_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1182 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1182;
            procedure Finalizer_Scope_3212 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3212;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9624 '<Let at nodes.lkt:13304:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13304'
--# scope-start
--# expr-start 9618 '<Cast to BasicSubpDecl at nodes.lkt:13304:31>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13304'
--# expr-start 9614 '<FieldAccess for get_unit_root_decl at nodes.lkt:13304:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13304'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9613 '<ArrayLiteral at nodes.lkt:13305:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13305'
--# expr-start 9611 '<Symbol System at nodes.lkt:13305:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13305'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 9611
--# expr-start 9612 '<Symbol Storage_Elements at nodes.lkt:13305:25>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13305'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Storage_Elements); 
--# expr-done 9612
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 9613
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 9614
Var_Expr_247 := Fld;
if Var_Expr_247 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9617 '<EnvGet at nodes.lkt:13304:31>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13304'
--# expr-start 9615 '<FieldAccess for children_env at nodes.lkt:13304:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13304'
      if Var_Expr_247 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_247);
--# expr-done 9615
--# expr-start 9616 '<Symbol To_Address at nodes.lkt:13306:37>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13306'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_To_Address); 
--# expr-done 9616
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9617
Scope_Result := Env_Get_Result;
   --# end
      Finalizer_Scope_3212;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 9618
Local_To_Address_Subp := Cast_Result; 
--# bind to_address_subp Local_To_Address_Subp
--# expr-start 9623 '<And at nodes.lkt:13308:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13308'
--# expr-start 9620 '<FieldAccess for sub_equation at nodes.lkt:13308:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13308'
--# expr-start 9619 '<FieldAccess for prefix at nodes.lkt:13308:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13308'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9619
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 9620
--# expr-start 9622 '<Bind at nodes.lkt:13308:40>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13308'
--# expr-start 9621 '<FieldAccess for ref_var at nodes.lkt:13308:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13308'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 9621
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr := Local_To_Address_Subp; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13308:40") else null)); 
--# expr-done 9622
And_Pred := Create_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13308:9") else null)); 
--# expr-done 9623
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9624
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1182;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1182;
                     Finalizer_Scope_3212;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_To_Address_Equation;
--# end
--# property-start AttributeRef.index_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13313'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Index_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1183 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1183 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1183;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9635 '<Let at nodes.lkt:13314:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
--# scope-start
--# expr-start 9629 '<FieldAccess for family_type at nodes.lkt:13314:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
--# expr-start 9628 '<Cast to EntryDecl at nodes.lkt:13314:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
--# expr-start 9627 '<EnvGet at nodes.lkt:13314:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
--# expr-start 9626 '<FieldAccess for name_symbol at nodes.lkt:13314:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
--# expr-start 9625 '<FieldAccess for prefix at nodes.lkt:13314:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9625
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 9626
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Thin (Fld_1), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 9627
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9628
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.family_type
Fld_2 := Libadalang.Implementation.Entry_Decl_P_Family_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 9629
Local_Typ := Fld_2; 
--# bind typ Local_Typ
--# expr-start 9634 '<And at nodes.lkt:13316:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13316'
--# expr-start 9631 '<FieldAccess for sub_equation at nodes.lkt:13316:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13316'
--# expr-start 9630 '<FieldAccess for prefix at nodes.lkt:13316:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13316'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9630
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 9631
--# expr-start 9633 '<Bind at nodes.lkt:13316:40>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13316'
--# expr-start 9632 '<FieldAccess for type_var at nodes.lkt:13316:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13316'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9632
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Local_Typ; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13316:40") else null)); 
--# expr-done 9633
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13316:9") else null)); 
--# expr-done 9634
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 9635
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1183;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1183;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Index_Equation;
--# end
--# property-start AttributeRef.deref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13321'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Deref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1184 with Inline_Always;
            procedure Finalizer_Scope_3213 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Assoc_List;
Fld_6 : Bare_Assoc_List;
Fld_7 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1184 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1184;
            procedure Finalizer_Scope_3213 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3213;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9654 '<And at nodes.lkt:13322:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13322'
--# expr-start 9642 '<And at nodes.lkt:13322:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13322'
--# expr-start 9637 '<FieldAccess for xref_type_equation at nodes.lkt:13322:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13322'
--# expr-start 9636 '<FieldAccess for prefix at nodes.lkt:13322:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13322'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9636
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_1 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9637
--# expr-start 9641 '<Bind at nodes.lkt:13323:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13323'
--# expr-start 9638 '<FieldAccess for type_var at nodes.lkt:13323:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13323'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9638
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 9640 '<FieldAccess for ref_var at nodes.lkt:13323:34>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13323'
--# expr-start 9639 '<FieldAccess for prefix at nodes.lkt:13323:34>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13323'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 9639
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 9640
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13323:13") else null)); 
--# expr-done 9641
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13322:9") else null)); 
--# expr-done 9642
--# expr-start 9643 '<FieldAccess for args at nodes.lkt:13324:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13324'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 9643
Fld_6 := Fld_5.Node;
Get_Result := Get (Self, Fld_6, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_7 := Fld_5.Info;
As_Entity := (Info => Fld_7, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind arg Var_Expr_0
--# expr-start 9653 '<And at nodes.lkt:13325:22>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13325'
--# expr-start 9650 '<And at nodes.lkt:13325:22>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13325'
--# expr-start 9645 '<FieldAccess for sub_equation at nodes.lkt:13325:22>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13325'
--# expr-start 9644 '<FieldAccess for expr at nodes.lkt:13325:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13325'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9644
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_8.Info);
--# end
--# expr-done 9645
--# expr-start 9649 '<Bind at nodes.lkt:13326:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13326'
--# expr-start 9647 '<FieldAccess for expected_type_var at nodes.lkt:13326:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13326'
--# expr-start 9646 '<FieldAccess for expr at nodes.lkt:13326:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13326'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9646
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_10.Node);
--# expr-done 9647
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
--# expr-start 9648 '<FieldAccess for system_address_type at nodes.lkt:13326:53>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13326'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.system_address_type
Fld_12 := Libadalang.Implementation.Ada_Node_P_System_Address_Type (Node => Self);
--# end
--# expr-done 9648
      Cast_Result := Create_Internal_Entity
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_11, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13326:17") else null)); 
--# expr-done 9649
And_Pred_1 := Create_And (Fld_9, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13325:22") else null)); 
--# expr-done 9650
--# expr-start 9652 '<FieldAccess for matches_expected_type at nodes.lkt:13327:17>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13327'
--# expr-start 9651 '<FieldAccess for expr at nodes.lkt:13327:17>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13327'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9651
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 9652
And_Pred_2 := Create_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13325:22") else null)); 
--# expr-done 9653
Scope_Result := And_Pred_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3213;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13328:25") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_3 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13322:9") else null)); 
--# expr-done 9654
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1184;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1184;
                     Finalizer_Scope_3213;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Deref_Equation;
--# end
--# property-start AttributeRef.mechanism_code_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13333'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Mechanism_Code_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1185 with Inline_Always;
            procedure Finalizer_Scope_3214 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Equation;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Var;
Fld_11 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1185 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1185;
            procedure Finalizer_Scope_3214 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3214;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9670 '<And at nodes.lkt:13334:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13334'
--# expr-start 9659 '<And at nodes.lkt:13334:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13334'
--# expr-start 9656 '<FieldAccess for xref_no_overloading at nodes.lkt:13334:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13334'
--# expr-start 9655 '<FieldAccess for prefix at nodes.lkt:13334:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13334'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9655
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9656
--# expr-start 9658 '<FieldAccess for universal_int_bind at nodes.lkt:13335:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13335'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9657 '<FieldAccess for type_var at nodes.lkt:13335:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13335'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9657
--# property-call-start AdaNode.universal_int_bind
Fld_3 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_2, Origin => Origin);
--# end
--# expr-done 9658
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13334:9") else null)); 
--# expr-done 9659
--# expr-start 9660 '<FieldAccess for args at nodes.lkt:13336:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13336'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 9660
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind arg Var_Expr_0
--# expr-start 9669 '<And at nodes.lkt:13337:22>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13337'
--# expr-start 9666 '<And at nodes.lkt:13337:22>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13337'
--# expr-start 9662 '<FieldAccess for sub_equation at nodes.lkt:13337:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13337'
--# expr-start 9661 '<FieldAccess for expr at nodes.lkt:13337:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13337'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9661
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 9662
--# expr-start 9665 '<FieldAccess for universal_int_bind at nodes.lkt:13338:17>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13338'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9664 '<FieldAccess for expected_type_var at nodes.lkt:13338:41>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13338'
--# expr-start 9663 '<FieldAccess for expr at nodes.lkt:13338:41>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13338'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9663
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_9.Node);
--# expr-done 9664
--# property-call-start AdaNode.universal_int_bind
Fld_11 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_10, Origin => Origin);
--# end
--# expr-done 9665
And_Pred_1 := Create_And (Fld_8, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13337:22") else null)); 
--# expr-done 9666
--# expr-start 9668 '<FieldAccess for matches_expected_type at nodes.lkt:13339:17>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13339'
--# expr-start 9667 '<FieldAccess for expr at nodes.lkt:13339:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13339'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9667
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 9668
And_Pred_2 := Create_And (And_Pred_1, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13337:22") else null)); 
--# expr-done 9669
Scope_Result := And_Pred_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3214;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13340:25") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_3 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13334:9") else null)); 
--# expr-done 9670
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1185;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1185;
                     Finalizer_Scope_3214;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Mechanism_Code_Equation;
--# end
--# property-start AttributeRef.storage_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13346'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1186 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1186 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1186;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9679 '<And at nodes.lkt:13346:40>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13346'
--# expr-start 9676 '<And at nodes.lkt:13348:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13348'
--# expr-start 9672 '<FieldAccess for sub_equation at nodes.lkt:13348:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13348'
--# expr-start 9671 '<FieldAccess for prefix at nodes.lkt:13348:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13348'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9671
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9672
--# expr-start 9675 '<Bind at nodes.lkt:13350:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13350'
--# expr-start 9673 '<FieldAccess for type_var at nodes.lkt:13350:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13350'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9673
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 9674 '<FieldAccess for bool_type at nodes.lkt:13350:34>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13350'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 9674
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13350:13") else null)); 
--# expr-done 9675
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13348:9") else null)); 
--# expr-done 9676
--# expr-start 9678 '<FieldAccess for sub_equation at nodes.lkt:13354:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13354'
--# expr-start 9677 '<FieldAccess for args at nodes.lkt:13354:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13354'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 9677
Fld_5 := Fld_4.Node;
Get_Result := Get (Self, Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => If_Result.Info);
--# end
--# expr-done 9678
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13346:40") else null)); 
--# expr-done 9679
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1186;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1186;
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Storage_Equation;
--# end
--# property-start AttributeRef.called_formal_subp_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13357'
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Called_Formal_Subp_Spec
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Attribute_Ref  := Bare_Attribute_Ref (Node);
     Ent : Internal_Entity_Attribute_Ref :=
       Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Local_Rel_Name : Symbol_Type;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9690 '<Let at nodes.lkt:13358:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13358'
--# scope-start
--# expr-start 9681 '<FieldAccess for name_symbol at nodes.lkt:13358:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13358'
--# expr-start 9680 '<FieldAccess for attribute at nodes.lkt:13358:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13358'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 9680
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 9681
Local_Rel_Name := Fld_1; 
--# bind rel_name Local_Rel_Name
--# expr-start 9689 '<If at nodes.lkt:13360:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
--# expr-start 9682 '<Symbol Image at nodes.lkt:13360:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 9682
Is_Equal_2 := Local_Rel_Name = Sym; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 9683 '<Symbol Wide_Image at nodes.lkt:13360:35>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 9683
Is_Equal_1 := Local_Rel_Name = Sym_1; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 9684 '<Symbol Wide_Wide_Image at nodes.lkt:13360:51>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 9684
Is_Equal := Local_Rel_Name = Sym_2; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
--# expr-start 9686 '<FieldAccess for expression_type at nodes.lkt:13360:75>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
--# expr-start 9685 '<FieldAccess for prefix at nodes.lkt:13360:75>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13360'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 9685
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 9686
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_0
--# expr-start 9688 '<FieldAccess for subp_spec_or_null at nodes.lkt:13361:22>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13361'
--# expr-start 9687 '<FieldAccess for attribute_subprogram at nodes.lkt:13361:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13361'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.attribute_subprogram
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Attribute_Subprogram (Node => Var_Expr_0.Node, Attr_Name => Local_Rel_Name, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9687
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 9688
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_2 := Cast_Result;
else
   If_Result_2 := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 9689
Let_Result := If_Result_2; 
--# end
--# expr-done 9690
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Attribute_Ref_P_Called_Formal_Subp_Spec;
--# end
   --
   --  Primitives for Bare_Call_Expr
   --
      procedure Initialize_Fields_For_Call_Expr
        (Self : Bare_Call_Expr
         ; Call_Expr_F_Name : Bare_Name
         ; Call_Expr_F_Suffix : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Call_Expr_F_Name := Call_Expr_F_Name;
            Self.Call_Expr_F_Suffix := Call_Expr_F_Suffix;
      Self.Call_Expr_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Call_Expr;
   function Call_Expr_F_Name
     (Node : Bare_Call_Expr) return Bare_Name
   is
   begin
         return Node.Call_Expr_F_Name;
   end;
   function Call_Expr_F_Suffix
     (Node : Bare_Call_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Call_Expr_F_Suffix;
   end;
--# property-start CallExpr.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13377'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Ref_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9692 '<FieldAccess for ref_var at nodes.lkt:13377:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13377'
--# expr-start 9691 '<FieldAccess for name at nodes.lkt:13377:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13377'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 9691
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 9692
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Ref_Var;
--# end
--# property-start CallExpr.subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13379'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9693 '<FieldAccess for r_called_spec at nodes.lkt:13379:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13379'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Call_Expr_R_Called_Spec'Unrestricted_Access;
--# expr-done 9693
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subp_Spec_Var;
--# end
--# property-start CallExpr.defines_subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13381'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Defines_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Defines_Subp_Spec_Var;
--# end
--# property-start CallExpr.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13383'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Relative_Name
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9695 '<FieldAccess for relative_name at nodes.lkt:13383:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13383'
--# expr-start 9694 '<FieldAccess for name at nodes.lkt:13383:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13383'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9694
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9695
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Relative_Name;
--# end
--# property-start CallExpr.kind 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Kind
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Call_Expr_Kind
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Call_Expr_Kind;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Call_Expr_Kind;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Call_Expr_Kind;
Fld_6 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_3 : Call_Expr_Kind;
Fld_10 : Boolean;
If_Result_4 : Call_Expr_Kind;
Fld_11 : Boolean;
If_Result_5 : Call_Expr_Kind;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9718 '<If at nodes.lkt:13388:32>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
--# expr-start 9696 '<FieldAccess for is_call at nodes.lkt:13388:35>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_call
Fld_11 := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9696
if Fld_11 then
   If_Result_5 := Call;
else
   --# expr-start 9717 '<If at nodes.lkt:13388:32>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
--# expr-start 9697 '<FieldAccess for is_array_slice at nodes.lkt:13389:10>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13389'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_array_slice
Fld_10 := Libadalang.Implementation.Call_Expr_P_Is_Array_Slice (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9697
if Fld_10 then
   If_Result_4 := Array_Slice;
else
   --# expr-start 9716 '<If at nodes.lkt:13388:32>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
--# expr-start 9704 '<bind at nodes.lkt:13391:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13391'
--# expr-start 9698 '<FieldAccess for origin_node at nodes.lkt:13391:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13391'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9698
Bound_Origin := Fld_6; 
--# expr-start 9703 '<Not at nodes.lkt:13393:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13393'
--# expr-start 9702 '<IsNull at nodes.lkt:13393:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13393'
--# expr-start 9700 '<FieldAccess for expression_type at nodes.lkt:13393:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13393'
--# expr-start 9699 '<FieldAccess for name at nodes.lkt:13393:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13393'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9699
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_8 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 9700
Var_Expr_0 := Fld_8;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_0
--# expr-start 9701 '<FieldAccess for array_def_with_deref at nodes.lkt:13393:53>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13393'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr_0.Node, Origin => Bound_Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9701
Scope_Result := Fld_9;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Is_Null := Result_Var.Node = null; 
--# expr-done 9702
Not_Val := not (Is_Null); 
--# expr-done 9703
Dyn_Var_Bind_Result := Not_Val; 
--# expr-done 9704
if Dyn_Var_Bind_Result then
   If_Result_3 := Array_Index;
else
   --# expr-start 9715 '<If at nodes.lkt:13388:32>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
--# expr-start 9707 '<Eq at nodes.lkt:13397:10>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
--# expr-start 9706 '<length at nodes.lkt:13397:10>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
--# expr-start 9705 '<FieldAccess for params at nodes.lkt:13397:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9705
Fld_3 := Fld_2.Node; 
Len := Length (Fld_3); 
--# expr-done 9706
Is_Equal := Len = 1; 
--# expr-done 9707
if Is_Equal then
   --# expr-start 9710 '<IsA BaseTypeDecl at nodes.lkt:13397:42>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
--# expr-start 9709 '<FieldAccess for referenced_decl at nodes.lkt:13397:42>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
--# expr-start 9708 '<FieldAccess for name at nodes.lkt:13397:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13397'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9708
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => False, E_Info => Fld_4.Info);
--# end
--# expr-done 9709
Is_A_1 := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 9710
   If_Result_1 := Is_A_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := Type_Conversion;
else
   --# expr-start 9714 '<If at nodes.lkt:13388:32>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13388'
--# expr-start 9713 '<IsA EntryDecl at nodes.lkt:13401:10>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13401'
--# expr-start 9712 '<FieldAccess for referenced_decl at nodes.lkt:13401:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13401'
--# expr-start 9711 '<FieldAccess for name at nodes.lkt:13401:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9711
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld.Node, Imprecise_Fallback => False, E_Info => Fld.Info);
--# end
--# expr-done 9712
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Entry_Decl_Range; 
--# expr-done 9713
if Is_A then
   If_Result := Family_Index;
else
   If_Result := raise Property_Error with "undetermined CallExpr kind";
end if;
--# expr-done 9714
   If_Result_2 := If_Result;
end if;
--# expr-done 9715
   If_Result_3 := If_Result_2;
end if;
--# expr-done 9716
   If_Result_4 := If_Result_3;
end if;
--# expr-done 9717
   If_Result_5 := If_Result_4;
end if;
--# expr-done 9718
         Property_Result := If_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Kind;
--# end
--# property-start CallExpr.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13405'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Constant
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Name;
Match_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Call_Expr_Kind;
Is_Equal : Boolean;
If_Result_1 : Boolean;
Fld_8 : Call_Expr_Kind;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9730 '<If at nodes.lkt:13406:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
--# expr-start 9720 '<Eq at nodes.lkt:13406:12>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
--# expr-start 9719 '<FieldAccess for kind at nodes.lkt:13406:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.kind
Fld_8 := Libadalang.Implementation.Call_Expr_P_Kind (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9719
Is_Equal_1 := Fld_8 = Type_Conversion; 
--# expr-done 9720
if Is_Equal_1 then
   --# expr-start 9724 '<Match at nodes.lkt:13406:61>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
--# expr-start 9722 '<FieldAccess for expr at nodes.lkt:13406:67>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
--# expr-start 9721 '<FieldAccess for params at nodes.lkt:13406:67>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9721
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9722
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind n Match_0
--# expr-start 9723 '<FieldAccess for is_constant at nodes.lkt:13409:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13409'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 9723
Let_Result := Fld_4; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 9724
   If_Result_2 := Match_Result;
else
   --# expr-start 9729 '<If at nodes.lkt:13406:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13406'
--# expr-start 9726 '<Eq at nodes.lkt:13413:14>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13413'
--# expr-start 9725 '<FieldAccess for kind at nodes.lkt:13413:14>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13413'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.kind
Fld_7 := Libadalang.Implementation.Call_Expr_P_Kind (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9725
Is_Equal := Fld_7 = Call; 
--# expr-done 9726
if Is_Equal then
   If_Result_1 := True;
else
   --# expr-start 9728 '<FieldAccess for is_constant_object at nodes.lkt:13416:14>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13416'
--# expr-start 9727 '<FieldAccess for referenced_decl at nodes.lkt:13416:14>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13416'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 9727
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 9728
   If_Result_1 := Fld_6;
end if;
--# expr-done 9729
   If_Result_2 := If_Result_1;
end if;
--# expr-done 9730
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Constant;
--# end
--# property-start CallExpr.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13419'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1194 with Inline_Always;
            procedure Finalizer_Scope_3219 with Inline_Always;
            procedure Finalizer_Scope_3220 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Basic_Decl;
Match_1 : Internal_Entity;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Bound_Include_Ud_Indexing : Boolean;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1194 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1194;
            procedure Finalizer_Scope_3219 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3219;
            procedure Finalizer_Scope_3220 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3220;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9743 '<Let at nodes.lkt:13420:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13420'
--# scope-start
--# expr-start 9732 '<FieldAccess for name_designated_type at nodes.lkt:13420:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13420'
--# expr-start 9731 '<FieldAccess for name at nodes.lkt:13420:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13420'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9731
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 9732
Local_Typ := Fld_1; 
--# bind typ Local_Typ
--# expr-start 9742 '<If at nodes.lkt:13422:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13422'
--# expr-start 9734 '<Not at nodes.lkt:13422:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13422'
--# expr-start 9733 '<IsNull at nodes.lkt:13422:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13422'
Is_Null := Local_Typ.Node = null; 
--# expr-done 9733
Not_Val := not (Is_Null); 
--# expr-done 9734
if Not_Val then
   --# expr-start 9735 '<FieldAccess for defining_env at nodes.lkt:13422:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13422'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Local_Typ.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Local_Typ.Info);
--# end
--# expr-done 9735
   If_Result := Fld_2;
else
   --# expr-start 9741 '<bind at nodes.lkt:13427:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13427'
Bound_Include_Ud_Indexing := True; 
--# expr-start 9740 '<env_group at nodes.lkt:13429:17>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13429'
--# expr-start 9739 '<Map at nodes.lkt:13429:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13429'
   --# expr-start 9736 '<FieldAccess for env_elements at nodes.lkt:13429:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13429'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => No_Visibility, E_Info => Ent.Info);
--# end
--# expr-done 9736
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
      --# expr-start 9738 '<Match at nodes.lkt:13430:28>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13430'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 9737 '<FieldAccess for defining_env at nodes.lkt:13431:55>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13431'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Match_0.Node, Origin => Origin, Include_Ud_Indexing => Bound_Include_Ud_Indexing, Dottable_Type => No_Bare_Ada_Node, E_Info => Match_0.Info);
--# end
--# expr-done 9737
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3220;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := Empty_Env; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 9738
         declare
            Item_To_Append : constant Lexical_Env := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3219;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9739
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 9740
Dyn_Var_Bind_Result := Group_Env; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9741
   If_Result := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 9742
Let_Result_2 := If_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 9743
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1194;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1194;
                     Finalizer_Scope_3219;
                     Finalizer_Scope_3220;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Env;
--# end
--# property-start CallExpr.designated_env_no_overloading 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13444'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Empty_Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start CallExpr.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13448'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Env_Elements_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1196 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1196 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1196;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9745 '<FieldAccess for env_elements_impl at nodes.lkt:13449:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13449'
--# expr-start 9744 '<FieldAccess for name at nodes.lkt:13449:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13449'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9744
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 9745
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1196;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1196;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Env_Elements_Impl;
--# end
--# property-start CallExpr.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13454'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Type_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3222 with Inline_Always;
            procedure Finalizer_Scope_3223 with Inline_Always;
      Local_Prefix_Tpe : Internal_Entity_Base_Type_Decl;
Local_Matches_Formals : Boolean;
Var_Expr_0 : Internal_Entity_Assoc_List;
Var_Expr_248 : Internal_Entity_Base_Type_Decl;
Var_Expr_249 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3222 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3222;
            procedure Finalizer_Scope_3223 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3223;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9764 '<Let at nodes.lkt:13456:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13456'
--# scope-start
--# expr-start 9747 '<FieldAccess for designated_type_impl at nodes.lkt:13456:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13456'
--# expr-start 9746 '<FieldAccess for name at nodes.lkt:13456:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13456'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9746
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 9747
Local_Prefix_Tpe := Fld_1; 
--# bind prefix_tpe Local_Prefix_Tpe
--# expr-start 9763 '<Let at nodes.lkt:13460:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13460'
--# scope-start
--# expr-start 9748 '<FieldAccess for params at nodes.lkt:13460:31>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13460'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9748
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ps Var_Expr_0
--# expr-start 9761 '<AllQuantifier at nodes.lkt:13461:21>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13461'
   --# expr-start 9750 '<FieldAccess for match_formals at nodes.lkt:13461:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13461'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Var_Expr_248 := Local_Prefix_Tpe;
if Var_Expr_248 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 9749 '<FieldAccess for discriminants_list at nodes.lkt:13462:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13462'
      if Var_Expr_248.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_248.Node, Origin => Origin, E_Info => Var_Expr_248.Info);
--# end
--# expr-done 9749
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3223;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start AdaNode.match_formals
Fld_4 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Result_Var, Params => Var_Expr_0, Is_Dottable_Subp => False);
--# end
--# expr-done 9750
      Quantifier_Result := True;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_1
--# expr-start 9751 '<FieldAccess for has_matched at nodes.lkt:13464:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13464'
Fld_5 := Item_1.Has_Matched;
--# expr-done 9751
if Fld_5 then
   --# expr-start 9760 '<FieldAccess for matching_type at nodes.lkt:13464:44>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13464'
--# expr-start 9754 '<FieldAccess for formal_type at nodes.lkt:13464:44>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13464'
--# expr-start 9753 '<FieldAccess for formal_decl at nodes.lkt:13464:44>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13464'
--# expr-start 9752 '<FieldAccess for formal at nodes.lkt:13464:44>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13464'
Fld_6 := Item_1.Formal;
--# expr-done 9752
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 9753
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 9754
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9758 '<Cast to Name at nodes.lkt:13465:21>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13465'
--# expr-start 9757 '<FieldAccess for expr at nodes.lkt:13465:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13465'
--# expr-start 9756 '<FieldAccess for assoc at nodes.lkt:13465:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13465'
--# expr-start 9755 '<FieldAccess for actual at nodes.lkt:13465:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13465'
Fld_9 := Item_1.Actual;
--# expr-done 9755
Fld_10 := Fld_9.Assoc;
--# expr-done 9756
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 9757
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 9758
Var_Expr_249 := Cast_Result;
if Var_Expr_249 /= No_Entity_Name then
   --# scope-start
--# expr-start 9759 '<FieldAccess for name_designated_type at nodes.lkt:13465:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13465'
      if Var_Expr_249.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_12 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_249.Node, E_Info => Var_Expr_249.Info);
--# end
--# expr-done 9759
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_8.Node, Expected_Type => Result_Var_1, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 9760
   If_Result := Fld_13;
else
   If_Result := False;
end if;
      Quantifier_Result := If_Result;
   --# end
               exit when not Quantifier_Result;
         end loop;
      end;
--# expr-done 9761
Scope_Result_2 := Quantifier_Result;
   --# end
      Finalizer_Scope_3222;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Local_Matches_Formals := Result_Var_2; 
--# bind matches_formals Local_Matches_Formals
--# expr-start 9762 '<If at nodes.lkt:13472:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13472'
if Local_Matches_Formals then
   If_Result_1 := Local_Prefix_Tpe;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 9762
Let_Result := If_Result_1; 
--# end
--# expr-done 9763
Let_Result_1 := Let_Result; 
--# end
--# expr-done 9764
         Property_Result := Let_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3222;
                     Finalizer_Scope_3223;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Designated_Type_Impl;
--# end
--# property-start CallExpr.params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13475'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Params
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 9766 '<Cast to AssocList at nodes.lkt:13475:39>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13475'
--# expr-start 9765 '<FieldAccess for suffix at nodes.lkt:13475:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13475'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9765
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 9766
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Params;
--# end
--# property-start CallExpr.check_array_slice 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13480'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Array_Slice
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Atd : Internal_Entity_Array_Type_Def;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Name;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9782 '<Let at nodes.lkt:13481:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13481'
--# scope-start
Var_Expr_0 := Typ;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 9767 '<FieldAccess for array_def_with_deref at nodes.lkt:13481:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13481'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9767
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Local_Atd := Result_Var; 
--# bind atd Local_Atd
--# expr-start 9769 '<Not at nodes.lkt:13483:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13483'
--# expr-start 9768 '<IsNull at nodes.lkt:13483:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13483'
Is_Null := Local_Atd.Node = null; 
--# expr-done 9768
Not_Val := not (Is_Null); 
--# expr-done 9769
if Not_Val then
--# expr-start 9770 '<FieldAccess for suffix at nodes.lkt:13483:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13483'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9770
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind sfx Var_Expr_1
--# expr-start 9771 '<IsA BinOp at nodes.lkt:13486:17>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13486'
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Bin_Op_Range; 
--# expr-done 9771
if Is_A then
   If_Result := True;
else
   --# expr-start 9772 '<IsA AttributeRef at nodes.lkt:13489:17>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13489'
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 9772
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 9773 '<IsA SubtypeIndication at nodes.lkt:13492:17>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13492'
Is_A_2 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Subtype_Indication_Range; 
--# expr-done 9773
   If_Result_1 := Is_A_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 9774 '<Cast to AssocList at nodes.lkt:13495:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13495'
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 9774
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr_2
--# expr-start 9776 '<Eq at nodes.lkt:13496:29>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13496'
--# expr-start 9775 '<length at nodes.lkt:13496:29>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13496'
Fld_2 := Var_Expr_2.Node; 
Len := Length (Fld_2); 
--# expr-done 9775
Is_Equal := Len = 1; 
--# expr-done 9776
if Is_Equal then
--# expr-start 9778 '<Cast to Name at nodes.lkt:13496:50>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13496'
--# expr-start 9777 '<FieldAccess for expr at nodes.lkt:13496:50>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13496'
Saved := Var_Expr_2; 
Fld_3 := Saved.Node;
Get_Result := Get (Self, Fld_3, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result_2 := As_Entity;
end if;
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 9777
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 9778
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 9781 '<Not at nodes.lkt:13497:32>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13497'
--# expr-start 9780 '<IsNull at nodes.lkt:13497:36>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13497'
--# expr-start 9779 '<FieldAccess for name_designated_type at nodes.lkt:13497:36>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13497'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 9779
Is_Null_2 := Fld_6.Node = null; 
--# expr-done 9780
Not_Val_1 := not (Is_Null_2); 
--# expr-done 9781
Scope_Result_1 := Not_Val_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
else
   If_Result_3 := False;
end if;
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
end if;
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_5 := Result_Var_3;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 9782
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Check_Array_Slice;
--# end
--# property-start CallExpr.is_array_slice 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13507'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Array_Slice
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 9787 '<bind at nodes.lkt:13508:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13508'
--# expr-start 9783 '<FieldAccess for origin_node at nodes.lkt:13508:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13508'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 9783
Bound_Origin := Fld; 
--# expr-start 9786 '<FieldAccess for check_array_slice at nodes.lkt:13510:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13510'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9785 '<FieldAccess for expression_type at nodes.lkt:13510:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13510'
--# expr-start 9784 '<FieldAccess for name at nodes.lkt:13510:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13510'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9784
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 9785
--# property-call-start CallExpr.check_array_slice
Fld_3 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Fld_2, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 9786
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 9787
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Array_Slice;
--# end
--# property-start CallExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13514'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1201 with Inline_Always;
      Fld : Logic_Equation;
            procedure Finalizer_Scope_1201 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1201;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9788 '<FieldAccess for bottom_up_name_equation at nodes.lkt:13514:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13514'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9788
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1201;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1201;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Xref_Equation;
--# end
--# property-start CallExpr.type_conv_self_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13520'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Type_Conv_Self_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1202 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1202 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1202;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9795 '<And at nodes.lkt:13521:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
--# expr-start 9790 '<FieldAccess for subtype_indication_equation at nodes.lkt:13521:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
--# expr-start 9789 '<FieldAccess for name at nodes.lkt:13521:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9789
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 9790
--# expr-start 9794 '<Bind at nodes.lkt:13521:53>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
--# expr-start 9791 '<FieldAccess for type_var at nodes.lkt:13521:57>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9791
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 9793 '<FieldAccess for ref_var at nodes.lkt:13521:74>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
--# expr-start 9792 '<FieldAccess for name at nodes.lkt:13521:74>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13521'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Call_Expr_F_Name;
--# expr-done 9792
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 9793
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13521:53") else null)); 
--# expr-done 9794
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13521:9") else null)); 
--# expr-done 9795
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1202;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1202;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Type_Conv_Self_Xref_Equation;
--# end
--# property-start CallExpr.entry_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13526'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entry_Equation
  (Node : Bare_Call_Expr
      ; E : Internal_Entity_Entry_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind e E
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1203 with Inline_Always;
            procedure Finalizer_Scope_3230 with Inline_Always;
            procedure Finalizer_Scope_3231 with Inline_Always;
            procedure Finalizer_Scope_3232 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Call_Expr;
Fld_12 : Internal_Entity_Assoc_List;
Fld_13 : Bare_Assoc_List;
Fld_14 : Internal_Entity_Info;
Item_2 : Internal_Entity_Basic_Assoc;
Bare_Item_2 : Bare_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_17 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Cast_Expr_2 : Internal_Entity_Entry_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_18 : Logic_Equation;
Fld_19 : Boolean;
If_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_1203 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_18);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_1203;
            procedure Finalizer_Scope_3230 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3230;
            procedure Finalizer_Scope_3231 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3231;
            procedure Finalizer_Scope_3232 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_3232;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9817 '<If at nodes.lkt:13526:70>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13526'
--# expr-start 9796 '<FieldAccess for has_family at nodes.lkt:13526:73>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13526'
      if E.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.has_family
Fld_19 := Libadalang.Implementation.Entry_Decl_P_Has_Family (Node => E.Node);
--# end
--# expr-done 9796
if Fld_19 then
   --# expr-start 9815 '<And at nodes.lkt:13528:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13528'
--# expr-start 9797 '<FieldAccess for family_type at nodes.lkt:13528:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13528'
      if E.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.family_type
Fld := Libadalang.Implementation.Entry_Decl_P_Family_Type (Node => E.Node, Origin => Origin, E_Info => E.Info);
--# end
--# expr-done 9797
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ft Var_Expr_0
--# expr-start 9805 '<And at nodes.lkt:13529:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9801 '<Bind at nodes.lkt:13529:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9800 '<FieldAccess for expected_type_var at nodes.lkt:13529:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9799 '<FieldAccess for expr at nodes.lkt:13529:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9798 '<FieldAccess for params at nodes.lkt:13529:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_1 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9798
Fld_2 := Fld_1.Node;
Get_Result := Get (Self, Fld_2, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9799
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4.Node);
--# expr-done 9800
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Var_Expr_0; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13529:21") else null)); 
--# expr-done 9801
--# expr-start 9804 '<FieldAccess for matches_expected_type at nodes.lkt:13529:79>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9803 '<FieldAccess for expr at nodes.lkt:13529:79>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
--# expr-start 9802 '<FieldAccess for params at nodes.lkt:13529:79>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13529'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_6 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9802
Fld_7 := Fld_6.Node;
Get_Result_1 := Get (Self, Fld_7, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 9803
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_10 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_9.Node, Origin => Origin);
--# end
--# expr-done 9804
And_Pred := Create_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13529:21") else null)); 
--# expr-done 9805
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3230;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13529:141") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 9807 '<Cast to CallExpr at nodes.lkt:13533:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13533'
--# expr-start 9806 '<FieldAccess for parent_name at nodes.lkt:13533:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13533'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_11 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9806
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Call_Expr
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result_1 := No_Entity_Call_Expr;
   end if;
--# expr-done 9807
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind c Var_Expr_1
--# expr-start 9814 '<And at nodes.lkt:13534:24>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
--# expr-start 9812 '<LogicAll at nodes.lkt:13534:24>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
--# expr-start 9811 '<Map at nodes.lkt:13534:24>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
   --# expr-start 9808 '<FieldAccess for params at nodes.lkt:13534:24>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_12 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9808
Fld_13 := Fld_12.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_13 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_13;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null_2 := Bare_Item_2 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_14 := Fld_12.Info;
As_Entity_2 := (Info => Fld_14, Node => Bare_Item_2); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_2 := If_Result_2; 
   --# scope-start
               --# bind pa Item_2
      --# expr-start 9810 '<FieldAccess for sub_equation at nodes.lkt:13534:53>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
--# expr-start 9809 '<FieldAccess for expr at nodes.lkt:13534:53>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_2.Node, E_Info => Item_2.Info);
--# end
--# expr-done 9809
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_15.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_15.Info);
--# end
--# expr-done 9810
         declare
            Item_To_Append : constant Logic_Equation := Fld_16;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3232;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9811
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13534:24") else null)); 
--# expr-done 9812
--# expr-start 9813 '<FieldAccess for entity_equation at nodes.lkt:13534:83>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13534'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_1 := E; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start CallExpr.entity_equation
Fld_17 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Var_Expr_1.Node, S => Cast_Result_2, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, Logic_Context => Logic_Context, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9813
And_Pred_1 := Create_And (Logic_Boolean_Op, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13534:24") else null)); 
--# expr-done 9814
Scope_Result_1 := And_Pred_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3231;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13534:123") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_2 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13528:9") else null)); 
--# expr-done 9815
   If_Result_3 := And_Pred_2;
else
   --# expr-start 9816 '<FieldAccess for entity_equation at nodes.lkt:13542:9>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13542'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_2 := E; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start CallExpr.entity_equation
Fld_18 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_3, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 9816
   If_Result_3 := Fld_18;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 9817
         Property_Result := If_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1203;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1203;
                     Finalizer_Scope_3230;
                     Finalizer_Scope_3231;
                     Finalizer_Scope_3232;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Entry_Equation;
--# end
--# property-start CallExpr.entity_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13546'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entity_Equation
  (Node : Bare_Call_Expr
      ; S : Internal_Entity_Basic_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind s S
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1204 with Inline_Always;
            procedure Finalizer_Scope_3233 with Inline_Always;
            procedure Finalizer_Scope_3234 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Logic_Equation;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_12 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_13 : Internal_Entity_Info;
Fld_14 : Internal_Metadata;
Fld_15 : Boolean;
Fld_16 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Fld_19 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_20 : Internal_Entity_Assoc_List;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Fld_21 : Boolean;
If_Result_1 : Logic_Equation;
Fld_22 : Boolean;
If_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_1204 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_1204;
            procedure Finalizer_Scope_3233 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_3233;
            procedure Finalizer_Scope_3234 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_3234;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9848 '<If at nodes.lkt:13550:5>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
--# expr-start 9818 '<FieldAccess for is_paramless at nodes.lkt:13550:8>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_paramless
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Is_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9818
if Fld_22 then
   --# expr-start 9820 '<FieldAccess for parent_name_equation at nodes.lkt:13550:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9819 '<FieldAccess for expr_type at nodes.lkt:13550:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9819
--# property-call-start Name.parent_name_equation
Fld_1 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9820
   If_Result_2 := Fld_1;
else
   --# expr-start 9847 '<If at nodes.lkt:13550:5>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
--# expr-start 9821 '<FieldAccess for can_be_paramless at nodes.lkt:13553:10>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13553'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_21 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9821
if Fld_21 then
   --# expr-start 9833 '<Or at nodes.lkt:13553:36>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13553'
--# expr-start 9823 '<FieldAccess for parent_name_equation at nodes.lkt:13553:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13553'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9822 '<FieldAccess for expr_type at nodes.lkt:13553:62>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13553'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9822
--# property-call-start Name.parent_name_equation
Fld_3 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_2, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9823
--# expr-start 9832 '<And at nodes.lkt:13554:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13554'
--# expr-start 9828 '<FieldAccess for subprogram_equation at nodes.lkt:13554:9>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13554'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9824 '<FieldAccess for subp_spec_or_null at nodes.lkt:13555:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13555'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9824
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-start 9827 '<FieldAccess for dottable_subp at nodes.lkt:13555:36>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13555'
--# expr-start 9826 '<FieldAccess for md at nodes.lkt:13555:36>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13555'
--# expr-start 9825 '<FieldAccess for info at nodes.lkt:13555:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13555'
Fld_5 := S.Info;
--# expr-done 9825
Fld_6 := Fld_5.Md;
--# expr-done 9826
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 9827
--# property-call-start CallExpr.subprogram_equation
Fld_8 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result, Dottable_Subp => Fld_7, Env => Env, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 9828
--# expr-start 9829 '<FieldAccess for parent_name at nodes.lkt:13556:15>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13556'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_9 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9829
Var_Expr_1 := Fld_9;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_1
--# expr-start 9831 '<FieldAccess for parent_name_equation at nodes.lkt:13557:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13557'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9830 '<FieldAccess for expr_type at nodes.lkt:13557:45>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13557'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9830
--# property-call-start Name.parent_name_equation
Fld_11 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_1.Node, Typ => Fld_10, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9831
Scope_Result := Fld_11;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3233;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13557:79") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_8, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13554:9") else null)); 
--# expr-done 9832
Or_Pred := Create_Or (Fld_3, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13553:36") else null)); 
--# expr-done 9833
   If_Result_1 := Or_Pred;
else
   --# expr-start 9846 '<If at nodes.lkt:13550:5>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13550'
--# expr-start 9836 '<Not at nodes.lkt:13560:10>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13560'
--# expr-start 9835 '<IsNull at nodes.lkt:13560:14>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13560'
--# expr-start 9834 '<FieldAccess for params at nodes.lkt:13560:14>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13560'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_20 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9834
Is_Null := Fld_20.Node = null; 
--# expr-done 9835
Not_Val := not (Is_Null); 
--# expr-done 9836
if Not_Val then
   --# expr-start 9845 '<And at nodes.lkt:13560:41>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13560'
--# expr-start 9841 '<FieldAccess for subprogram_equation at nodes.lkt:13560:41>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13560'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9837 '<FieldAccess for subp_spec_or_null at nodes.lkt:13561:9>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13561'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 9837
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
--# expr-start 9840 '<FieldAccess for dottable_subp at nodes.lkt:13561:32>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13561'
--# expr-start 9839 '<FieldAccess for md at nodes.lkt:13561:32>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13561'
--# expr-start 9838 '<FieldAccess for info at nodes.lkt:13561:32>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13561'
Fld_13 := S.Info;
--# expr-done 9838
Fld_14 := Fld_13.Md;
--# expr-done 9839
Fld_15 := Fld_14.Dottable_Subp;
--# expr-done 9840
--# property-call-start CallExpr.subprogram_equation
Fld_16 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => Fld_15, Env => Env, Origin => Origin, Logic_Context => Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 9841
--# expr-start 9842 '<FieldAccess for parent_name at nodes.lkt:13562:11>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13562'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_17 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9842
Var_Expr_0 := Fld_17;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_0
--# expr-start 9844 '<FieldAccess for parent_name_equation at nodes.lkt:13563:17>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13563'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9843 '<FieldAccess for expr_type at nodes.lkt:13563:41>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13563'
      if S.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 9843
--# property-call-start Name.parent_name_equation
Fld_19 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Fld_18, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9844
Scope_Result_1 := Fld_19;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3234;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13563:75") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Create_And (Fld_16, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13560:41") else null)); 
--# expr-done 9845
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13565:10") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 9846
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 9847
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9848
         Property_Result := If_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1204;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1204;
                     Finalizer_Scope_3233;
                     Finalizer_Scope_3234;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Entity_Equation;
--# end
--# property-start CallExpr.is_type_conversion 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13569'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Type_Conversion
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Is_A : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 9851 '<Not at nodes.lkt:13569:38>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13569'
--# expr-start 9850 '<IsA QualExpr at nodes.lkt:13569:42>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13569'
--# expr-start 9849 '<FieldAccess for name at nodes.lkt:13569:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13569'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9849
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Qual_Expr_Range; 
--# expr-done 9850
Not_Val := not (Is_A); 
--# expr-done 9851
if Not_Val then
   --# expr-start 9855 '<Not at nodes.lkt:13572:9>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13572'
--# expr-start 9854 '<IsNull at nodes.lkt:13572:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13572'
--# expr-start 9853 '<FieldAccess for designated_type_impl at nodes.lkt:13572:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13572'
--# expr-start 9852 '<FieldAccess for name at nodes.lkt:13572:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13572'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9852
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 9853
Is_Null := Fld_2.Node = null; 
--# expr-done 9854
Not_Val_1 := not (Is_Null); 
--# expr-done 9855
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Is_Type_Conversion;
--# end
--# property-start CallExpr.xref_stop_resolution 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13576'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Stop_Resolution
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 9856 '<Super at nodes.lkt:13577:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13577'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.xref_stop_resolution
Fld := Libadalang.Implementation.Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9856
if Fld then
   If_Result := True;
else
   --# expr-start 9857 '<FieldAccess for is_type_conversion at nodes.lkt:13577:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13577'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_1 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9857
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Xref_Stop_Resolution;
--# end
--# property-start CallExpr.stop_resolution_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13580'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Stop_Resolution_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1207 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1207 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1207;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9861 '<If at nodes.lkt:13581:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13581'
--# expr-start 9858 '<FieldAccess for is_type_conversion at nodes.lkt:13581:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13581'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_2 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9858
if Fld_2 then
   --# expr-start 9859 '<FieldAccess for type_conv_self_xref_equation at nodes.lkt:13581:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13581'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.type_conv_self_xref_equation
Fld := Libadalang.Implementation.Call_Expr_P_Type_Conv_Self_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9859
   If_Result := Fld;
else
   --# expr-start 9860 '<Super at nodes.lkt:13581:84>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13581'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.stop_resolution_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9860
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 9861
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1207;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1207;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Stop_Resolution_Equation;
--# end
--# property-start CallExpr.general_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13586'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_General_Xref_Equation
  (Node : Bare_Call_Expr
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1208 with Inline_Always;
            procedure Finalizer_Scope_3235 with Inline_Always;
            procedure Finalizer_Scope_3236 with Inline_Always;
            procedure Finalizer_Scope_3237 with Inline_Always;
            procedure Finalizer_Scope_3239 with Inline_Always;
            procedure Finalizer_Scope_3240 with Inline_Always;
      Local_Subps : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Entry_Decl;
Var_Expr_2 : Boolean;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Internal_Entity_Name;
Var_Expr_250 : Bare_Aggregate_Assoc;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Name;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Fld_12 : Boolean;
Fld_13 : Logic_Equation;
Fld_14 : Internal_Entity_Name;
Fld_15 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_16 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Name;
Fld_19 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_20 : Internal_Entity_Name;
Fld_21 : Logic_Var;
Is_Logic_Var_Defined_1 : Boolean;
Eq_Solution_1 : Internal_Entity;
If_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_22 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_23 : Lexical_Env := Empty_Env;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_4 : Bare_Aggregate_Assoc;
As_Entity_1 : Internal_Entity_Aggregate_Assoc;
Is_Null_1 : Boolean;
If_Result_3 : Internal_Entity_Aggregate_Assoc;
Fld_24 : Internal_Entity_Base_Aggregate;
Scope_Result_3 : Internal_Entity_Base_Aggregate;
Result_Var_3 : Internal_Entity_Base_Aggregate;
Is_A : Boolean;
If_Result_4 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_25 : Logic_Equation;
Dyn_Var_Bind_Result_1 : Logic_Equation;
Fld_26 : Internal_Entity_Array_Access;
True_Rel_1 : Logic_Equation;
Fld_27 : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity;
New_Struct_1 : Internal_Logic_Context;
Bound_Logic_Context_1 : Internal_Logic_Context;
Fld_28 : Bare_Name;
Fld_29 : Logic_Var;
Cast_Expr_2 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Fld_30 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Cast_Expr_3 : Internal_Entity;
Cast_Result_8 : Internal_Entity_Entry_Decl;
Fld_31 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Expr_4 : Internal_Entity;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Fld_32 : Logic_Equation;
Result_Var_4 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Dyn_Var_Bind_Result_2 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Is_Equal : Boolean;
If_Result_5 : Logic_Equation;
Let_Result : Logic_Equation;
Fld_33 : Internal_Entity_Name;
Fld_34 : Logic_Equation;
And_Pred_5 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_35 : Internal_Entity_Name;
Is_A_1 : Boolean;
If_Result_6 : Logic_Equation;
Fld_36 : Boolean;
If_Result_7 : Logic_Equation;
            procedure Finalizer_Scope_1208 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_23);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Local_Subps);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_34);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
            end Finalizer_Scope_1208;
            procedure Finalizer_Scope_3235 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_3235;
            procedure Finalizer_Scope_3236 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3236;
            procedure Finalizer_Scope_3237 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_3237;
            procedure Finalizer_Scope_3239 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_3239;
            procedure Finalizer_Scope_3240 is
            begin
                     Dec_Ref (Fld_31);
            end Finalizer_Scope_3240;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 9929 '<If at nodes.lkt:13586:55>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13586'
--# expr-start 9862 '<FieldAccess for is_type_conversion at nodes.lkt:13586:58>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13586'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.is_type_conversion
Fld_36 := Libadalang.Implementation.Call_Expr_P_Is_Type_Conversion (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 9862
if Fld_36 then
   --# expr-start 9875 '<And at nodes.lkt:13588:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13588'
--# expr-start 9870 '<And at nodes.lkt:13589:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13589'
--# expr-start 9868 '<And at nodes.lkt:13590:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
--# expr-start 9863 '<FieldAccess for type_conv_self_xref_equation at nodes.lkt:13590:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.type_conv_self_xref_equation
Fld := Libadalang.Implementation.Call_Expr_P_Type_Conv_Self_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9863
--# expr-start 9867 '<Bind at nodes.lkt:13590:57>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
--# expr-start 9866 '<FieldAccess for expected_type_var at nodes.lkt:13590:61>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
--# expr-start 9865 '<FieldAccess for expr at nodes.lkt:13590:61>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
--# expr-start 9864 '<FieldAccess for params at nodes.lkt:13590:61>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13590'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_1 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9864
Fld_2 := Fld_1.Node;
Get_Result := Get (Self, Fld_2, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 9865
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4.Node);
--# expr-done 9866
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := No_Entity_Base_Type_Decl; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13590:57") else null)); 
--# expr-done 9867
And_Pred := Create_And (Fld, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13590:17") else null)); 
--# expr-done 9868
--# expr-start 9869 '<FieldAccess for all_args_xref_equation at nodes.lkt:13591:19>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13591'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_6 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9869
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13589:13") else null)); 
--# expr-done 9870
--# expr-start 9871 '<FieldAccess for parent_name at nodes.lkt:13592:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13592'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 9871
Var_Expr_4 := Fld_7;
if Var_Expr_4 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_4
--# expr-start 9874 '<FieldAccess for parent_name_equation at nodes.lkt:13593:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13593'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9873 '<FieldAccess for name_designated_type at nodes.lkt:13593:45>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13593'
--# expr-start 9872 '<FieldAccess for name at nodes.lkt:13593:45>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13593'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9872
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_9 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 9873
--# property-call-start Name.parent_name_equation
Fld_10 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_4.Node, Typ => Fld_9, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 9874
Scope_Result := Fld_10;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3235;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13593:98") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13588:9") else null)); 
--# expr-done 9875
   If_Result_7 := And_Pred_2;
else
   --# expr-start 9928 '<If at nodes.lkt:13586:55>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13586'
--# expr-start 9877 '<IsA AttributeRef at nodes.lkt:13599:10>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13599'
--# expr-start 9876 '<FieldAccess for name at nodes.lkt:13599:10>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13599'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9876
Is_A_1 := Fld_35.Node /= null 
and then Fld_35.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 9877
if Is_A_1 then
--# expr-start 9879 '<FieldAccess for resolve_names_internal at nodes.lkt:13599:41>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13599'
--# expr-start 9878 '<FieldAccess for name at nodes.lkt:13599:41>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13599'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9878
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_12 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld_11.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 9879
Var_Expr_2 := Fld_12;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 9896 '<And at nodes.lkt:13600:16>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13600'
--# expr-start 9880 '<FieldAccess for all_args_xref_equation at nodes.lkt:13600:16>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_13 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9880
--# expr-start 9883 '<Cast to BaseTypeDecl at nodes.lkt:13600:54>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13600'
--# expr-start 9882 '<FieldAccess for type_val at nodes.lkt:13600:54>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13600'
--# expr-start 9881 '<FieldAccess for name at nodes.lkt:13600:54>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13600'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9881
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_15 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_14.Node);
--# end
--# expr-done 9882
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 9883
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_3
--# expr-start 9884 '<FieldAccess for parent_name_equation at nodes.lkt:13601:22>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13601'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Var_Expr_3, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9884
Scope_Result_1 := Fld_16;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3237;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 9895 '<bind at nodes.lkt:13602:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13602'
--# expr-start 9889 '<New LogicContext at nodes.lkt:13602:38>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13602'
--# expr-start 9887 '<get_value at nodes.lkt:13603:51>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13603'
--# expr-start 9886 '<FieldAccess for ref_var at nodes.lkt:13603:51>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13603'
--# expr-start 9885 '<FieldAccess for name at nodes.lkt:13603:51>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13603'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9885
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_18.Node);
--# end
--# expr-done 9886
Is_Logic_Var_Defined := Entity_Vars.Is_Defined (Fld_19); 
if Is_Logic_Var_Defined then
Eq_Solution := Entity_Vars.Get_Value (Fld_19); 
   If_Result_1 := Eq_Solution;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 9887
--# expr-start 9888 '<FieldAccess for name at nodes.lkt:13603:30>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13603'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9888
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
New_Struct := (Decl_Node => If_Result_1, Ref_Node => Cast_Result_2); 
--# expr-done 9889
Bound_Logic_Context := New_Struct; 
--# expr-start 9894 '<FieldAccess for entity_equation at nodes.lkt:13609:17>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13609'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9893 '<Cast to BasicDecl at nodes.lkt:13610:21>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13610'
--# expr-start 9892 '<get_value at nodes.lkt:13610:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13610'
--# expr-start 9891 '<FieldAccess for ref_var at nodes.lkt:13610:21>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13610'
--# expr-start 9890 '<FieldAccess for name at nodes.lkt:13610:21>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13610'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9890
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_20.Node);
--# end
--# expr-done 9891
Is_Logic_Var_Defined_1 := Entity_Vars.Is_Defined (Fld_21); 
if Is_Logic_Var_Defined_1 then
Eq_Solution_1 := Entity_Vars.Get_Value (Fld_21); 
   If_Result_2 := Eq_Solution_1;
else
   If_Result_2 := No_Entity;
end if;
--# expr-done 9892
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 9893
--# property-call-start CallExpr.entity_equation
Fld_22 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_3, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 9894
Dyn_Var_Bind_Result := Fld_22; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9895
   Result_Var_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Create_And (Fld_13, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13600:16") else null)); 
--# expr-done 9896
Scope_Result_2 := And_Pred_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3236;
   Result_Var_2 := Scope_Result_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13613:24") else null)); 
   Result_Var_2 := False_Rel;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_6 := Result_Var_2;
else
   --# expr-start 9927 '<And at nodes.lkt:13615:10>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13615'
--# expr-start 9904 '<bind at nodes.lkt:13619:9>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
--# expr-start 9902 '<If at nodes.lkt:13619:20>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
--# expr-start 9900 '<IsA DeltaAggregate at nodes.lkt:13619:23>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
--# expr-start 9897 '<Cast to AggregateAssoc at nodes.lkt:13619:23>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
Cast_Expr_1 := Entry_Point; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Aggregate_Assoc_Range
   then
      Cast_Result_4 := Cast_Expr_1;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 9897
Var_Expr_250 := Cast_Result_4;
if Var_Expr_250 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 9899 '<FieldAccess for base_aggregate at nodes.lkt:13619:23>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
--# expr-start 9898 '<as_entity at nodes.lkt:13619:23>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13619'
Is_Null_1 := Var_Expr_250 = null; 
if Is_Null_1 then
   If_Result_3 := No_Entity_Aggregate_Assoc;
else
As_Entity_1 := (Info => E_Info, Node => Var_Expr_250); 
   If_Result_3 := As_Entity_1;
end if;
--# expr-done 9898
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld_24 := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 9899
Scope_Result_3 := Fld_24;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Aggregate;
end if;
Is_A := Result_Var_3.Node /= null 
and then Result_Var_3.Node.Kind in Ada_Delta_Aggregate_Range; 
--# expr-done 9900
if Is_A then
   --# expr-start 9901 '<FieldAccess for children_env at nodes.lkt:13622:13>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13622'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 9901
   If_Result_4 := Fld_23;
else
   If_Result_4 := Env;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 9902
Bound_Env := If_Result_4; Inc_Ref (Bound_Env);
--# expr-start 9903 '<FieldAccess for all_args_xref_equation at nodes.lkt:13625:9>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13625'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_25 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 9903
Dyn_Var_Bind_Result_1 := Fld_25; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 9904
--# expr-start 9926 '<And at nodes.lkt:13629:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13629'
--# expr-start 9923 '<Let at nodes.lkt:13630:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13630'
--# scope-start
--# expr-start 9905 '<FieldAccess for env_elements at nodes.lkt:13630:25>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13630'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_26 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 9905
Local_Subps := Fld_26; Inc_Ref (Local_Subps);
--# bind subps Local_Subps
--# expr-start 9922 '<If at nodes.lkt:13632:13>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13632'
--# expr-start 9906 '<IsNull at nodes.lkt:13632:16>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13632'
Is_Equal := Equivalent (Local_Subps, No_Internal_Entity_Array_Type); 
--# expr-done 9906
if Is_Equal then
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13632:35") else null)); 
   If_Result_5 := True_Rel_1;
else
   --# expr-start 9921 '<LogicAny at nodes.lkt:13632:46>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13632'
--# expr-start 9920 '<Map at nodes.lkt:13632:46>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13632'
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Subps;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_0
      --# expr-start 9919 '<bind at nodes.lkt:13634:21>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13634'
--# expr-start 9908 '<New LogicContext at nodes.lkt:13634:42>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13634'
--# expr-start 9907 '<FieldAccess for name at nodes.lkt:13634:64>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13634'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9907
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_27.Node,
         Info => Fld_27.Info);
New_Struct_1 := (Decl_Node => Item_0, Ref_Node => Cast_Result_5); 
--# expr-done 9908
Bound_Logic_Context_1 := New_Struct_1; 
--# expr-start 9918 '<And at nodes.lkt:13636:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
--# expr-start 9913 '<Bind at nodes.lkt:13636:21>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
--# expr-start 9910 '<FieldAccess for ref_var at nodes.lkt:13636:25>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
--# expr-start 9909 '<FieldAccess for name at nodes.lkt:13636:25>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Self.Call_Expr_F_Name;
--# expr-done 9909
      if Fld_28 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_28);
--# end
--# expr-done 9910
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 9912 '<FieldAccess for corresponding_actual at nodes.lkt:13636:46>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
--# expr-start 9911 '<Cast to BasicDecl at nodes.lkt:13636:46>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_6 := No_Entity_Basic_Decl;
   end if;
--# expr-done 9911
      if Cast_Result_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.corresponding_actual'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Corresponding_Actual (Node => Cast_Result_6.Node, E_Info => Cast_Result_6.Info);
--# end
--# expr-done 9912
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_29, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13636:21") else null)); 
--# expr-done 9913
--# expr-start 9914 '<Cast to EntryDecl at nodes.lkt:13636:90>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13636'
Cast_Expr_3 := Item_0; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_8 := No_Entity_Entry_Decl;
   end if;
--# expr-done 9914
Var_Expr_1 := Cast_Result_8;
if Var_Expr_1 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 9915 '<FieldAccess for entry_equation at nodes.lkt:13637:32>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13637'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.entry_equation
Fld_31 := Libadalang.Implementation.Call_Expr_P_Entry_Equation (Node => Ent.Node, E => Var_Expr_1, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, Logic_Context => Bound_Logic_Context_1, E_Info => Ent.Info);
--# end
--# expr-done 9915
Scope_Result_4 := Fld_31;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3240;
   Result_Var_4 := Scope_Result_4;
else
   --# expr-start 9917 '<FieldAccess for entity_equation at nodes.lkt:13637:74>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13637'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9916 '<Cast to BasicDecl at nodes.lkt:13637:95>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13637'
Cast_Expr_4 := Item_0; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_9 := No_Entity_Basic_Decl;
   end if;
--# expr-done 9916
--# property-call-start CallExpr.entity_equation
Fld_32 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => Cast_Result_9, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, Logic_Context => Bound_Logic_Context_1, E_Info => Ent.Info);
--# end
--# expr-done 9917
   Result_Var_4 := Fld_32;
end if;
      Inc_Ref (Result_Var_4);
And_Pred_4 := Create_And (Bind_Result_1, Result_Var_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13636:21") else null)); 
--# expr-done 9918
Dyn_Var_Bind_Result_2 := And_Pred_4; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 9919
         declare
            Item_To_Append : constant Logic_Equation := Dyn_Var_Bind_Result_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3239;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9920
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13632:46") else null)); 
--# expr-done 9921
   If_Result_5 := Logic_Boolean_Op;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 9922
Let_Result := If_Result_5; Inc_Ref (Let_Result);
--# end
--# expr-done 9923
--# expr-start 9925 '<FieldAccess for sub_equation at nodes.lkt:13641:15>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13641'
--# expr-start 9924 '<FieldAccess for name at nodes.lkt:13641:15>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13641'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9924
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_33.Info);
--# end
--# expr-done 9925
And_Pred_5 := Create_And (Let_Result, Fld_34, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13629:9") else null)); 
--# expr-done 9926
And_Pred_6 := Create_And (Dyn_Var_Bind_Result_1, And_Pred_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13615:10") else null)); 
--# expr-done 9927
   If_Result_6 := And_Pred_6;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 9928
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 9929
         Property_Result := If_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1208;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1208;
                     Finalizer_Scope_3235;
                     Finalizer_Scope_3236;
                     Finalizer_Scope_3237;
                     Finalizer_Scope_3239;
                     Finalizer_Scope_3240;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_General_Xref_Equation;
--# end
--# property-start CallExpr.subscriptable_type_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13649'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subscriptable_Type_Equation
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1209 with Inline_Always;
            procedure Finalizer_Scope_3243 with Inline_Always;
            procedure Finalizer_Scope_3244 with Inline_Always;
            procedure Finalizer_Scope_3245 with Inline_Always;
            procedure Finalizer_Scope_3246 with Inline_Always;
            procedure Finalizer_Scope_3247 with Inline_Always;
            procedure Finalizer_Scope_3248 with Inline_Always;
            procedure Finalizer_Scope_3249 with Inline_Always;
            procedure Finalizer_Scope_3250 with Inline_Always;
            procedure Finalizer_Scope_3251 with Inline_Always;
            procedure Finalizer_Scope_3252 with Inline_Always;
            procedure Finalizer_Scope_3253 with Inline_Always;
            procedure Finalizer_Scope_3254 with Inline_Always;
      Local_Atd : Internal_Entity_Array_Type_Def;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Local_Real_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Local_Formals : Internal_Entity_Defining_Name_Array_Access;
Local_Ret_Type : Internal_Entity_Base_Type_Decl;
Local_Params : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Assoc_List;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Name;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Subtype_Indication;
Match_4 : Internal_Entity;
Var_Expr_9 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_251 : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_5 : Internal_Entity_Subp_Spec;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result_2 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Bare_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
False_Rel_1 : Logic_Equation;
Fld_12 : Logic_Equation;
Fld_13 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_14 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
If_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
False_Rel_2 : Logic_Equation;
Result_Var_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
False_Rel_3 : Logic_Equation;
Result_Var_4 : Logic_Equation;
Fld_15 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Item_7 : Internal_Entity_Basic_Assoc;
Bare_Item_7 : Bare_Basic_Assoc;
Index_8 : Integer;
Fld_18 : Internal_Entity_Array_Indices;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Scope_Result_5 : Logic_Equation_Array_Access;
Result_Var_5 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_21 : Logic_Var;
Fld_22 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_23 : Internal_Entity_Array_Indices;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Equation;
Fld_26 : Internal_Entity_Array_Indices;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_29 : Logic_Var;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_32 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_33 : Internal_Entity_Expr;
Fld_34 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_35 : Internal_Entity_Expr;
Fld_36 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_37 : Logic_Equation;
Fld_38 : Internal_Entity_Array_Indices;
Cast_Expr_3 : Internal_Entity_Attribute_Ref;
Cast_Result_8 : Internal_Entity_Expr;
Fld_39 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_40 : Logic_Var;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_41 : Logic_Equation;
Fld_42 : Logic_Var;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
And_Pred_9 : Logic_Equation;
False_Rel_4 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_11 : Internal_Entity_Assoc_List;
Let_Result : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_12 : Internal_Entity_Bin_Op;
Let_Result_1 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_13 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_14 : Internal_Entity_Subtype_Indication;
Let_Result_3 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Cast_Result_15 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_10 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_43 : Internal_Entity_Basic_Decl_Array_Access;
Fld_44 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_45 : Internal_Entity_Base_Subp_Spec;
Fld_46 : Internal_Entity_Defining_Name_Array_Access;
Fld_47 : Internal_Entity_Base_Subp_Spec;
Fld_48 : Internal_Entity_Base_Type_Decl;
Fld_49 : Internal_Entity_Assoc_List;
Fld_50 : Logic_Var;
Cast_Expr_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_16 : Internal_Entity;
Bind_Result_6 : Logic_Equation;
Saved_1 : Internal_Entity_Assoc_List;
Fld_51 : Bare_Assoc_List;
Fld_52 : Internal_Entity_Info;
Item_3 : Internal_Entity_Basic_Assoc;
Bare_Item_3 : Bare_Basic_Assoc;
Index_4 : Integer;
Fld_53 : Internal_Entity_Expr;
Fld_54 : Logic_Var;
Arith_Result : Integer;
Get_Result_1 : Internal_Entity_Defining_Name;
Fld_55 : Internal_Entity_Base_Formal_Param_Decl;
Fld_56 : Internal_Entity_Base_Type_Decl;
Cast_Result_17 : Internal_Entity;
Bind_Result_7 : Logic_Equation;
Fld_57 : Internal_Entity_Expr;
Fld_58 : Logic_Equation;
And_Pred_10 : Logic_Equation;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Basic_Assoc;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
And_Pred_11 : Logic_Equation;
False_Rel_5 : Logic_Equation;
Len : Integer;
Fld_59 : Bare_Assoc_List;
Len_1 : Integer;
Arith_Result_1 : Integer;
Is_Equal : Boolean;
If_Result_5 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Let_Result_6 : Logic_Equation;
Let_Result_7 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
False_Rel_6 : Logic_Equation;
Is_Null_4 : Boolean;
Not_Val : Boolean;
Fld_60 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Logic_Equation;
Is_Null_5 : Boolean;
Not_Val_1 : Boolean;
Fld_61 : Internal_Entity_Array_Indices;
Is_Null_6 : Boolean;
Not_Val_2 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Logic_Equation;
Fld_62 : Internal_Entity_Access_Def;
Is_A : Boolean;
If_Result_10 : Logic_Equation;
Let_Result_8 : Logic_Equation;
Let_Result_9 : Logic_Equation;
            procedure Finalizer_Scope_1209 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Fld_43);
                     Dec_Ref (Fld_44);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (False_Rel_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (Let_Result_8);
                     Dec_Ref (Let_Result_9);
            end Finalizer_Scope_1209;
            procedure Finalizer_Scope_3243 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3243;
            procedure Finalizer_Scope_3244 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (False_Rel_3);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3244;
            procedure Finalizer_Scope_3245 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (False_Rel_2);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_3245;
            procedure Finalizer_Scope_3246 is
            begin
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3246;
            procedure Finalizer_Scope_3247 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3247;
            procedure Finalizer_Scope_3248 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_3248;
            procedure Finalizer_Scope_3249 is
            begin
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_28);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_34);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_3249;
            procedure Finalizer_Scope_3250 is
            begin
                     Dec_Ref (Fld_37);
                     Dec_Ref (Fld_39);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_3250;
            procedure Finalizer_Scope_3251 is
            begin
                     Dec_Ref (Fld_41);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_3251;
            procedure Finalizer_Scope_3252 is
            begin
                     Dec_Ref (False_Rel_4);
            end Finalizer_Scope_3252;
            procedure Finalizer_Scope_3253 is
            begin
                     Dec_Ref (Fld_46);
                     Dec_Ref (Local_Formals);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (False_Rel_5);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_3253;
            procedure Finalizer_Scope_3254 is
            begin
                     Dec_Ref (Bind_Result_7);
                     Dec_Ref (Fld_58);
                     Dec_Ref (And_Pred_10);
            end Finalizer_Scope_3254;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10042 '<Let at nodes.lkt:13650:9>' Let_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13650'
--# scope-start
Var_Expr_0 := Typ;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 9930 '<FieldAccess for array_def_with_deref at nodes.lkt:13650:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13650'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 9930
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Local_Atd := Result_Var; 
--# bind atd Local_Atd
--# expr-start 10041 '<Let at nodes.lkt:13651:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13651'
--# scope-start
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 9933 '<If at nodes.lkt:13652:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13652'
--# expr-start 9931 '<FieldAccess for is_implicit_deref at nodes.lkt:13652:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13652'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9931
if Fld_2 then
   --# expr-start 9932 '<FieldAccess for accessed_type at nodes.lkt:13652:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13652'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 9932
   If_Result := Fld_1;
else
   If_Result := Var_Expr_1;
end if;
--# expr-done 9933
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Local_Real_Typ := Result_Var_1; 
--# bind real_typ Local_Real_Typ
--# expr-start 10040 '<If at nodes.lkt:13656:9>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
--# expr-start 9935 '<IsA AccessToSubpDef at nodes.lkt:13656:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
--# expr-start 9934 '<FieldAccess for access_def at nodes.lkt:13656:12>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_62 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9934
Is_A := Fld_62.Node /= null 
and then Fld_62.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 9935
if Is_A then
--# expr-start 9937 '<Cast to AccessToSubpDef at nodes.lkt:13656:53>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
--# expr-start 9936 '<FieldAccess for access_def at nodes.lkt:13656:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 9936
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 9937
Var_Expr_9 := Cast_Result;
if Var_Expr_9 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind asd Var_Expr_9
--# expr-start 9942 '<bind at nodes.lkt:13658:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13658'
--# expr-start 9939 '<New LogicContext at nodes.lkt:13658:38>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13658'
Cast_Expr := Typ; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 9938 '<FieldAccess for name at nodes.lkt:13658:60>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13658'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 9938
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
New_Struct := (Decl_Node => Cast_Result_2, Ref_Node => Cast_Result_1); 
--# expr-done 9939
Bound_Logic_Context := New_Struct; 
--# expr-start 9941 '<FieldAccess for subprogram_equation at nodes.lkt:13660:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13660'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9940 '<FieldAccess for subp_spec at nodes.lkt:13660:42>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13660'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_9.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_9.Info);
--# expr-done 9940
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# property-call-start CallExpr.subprogram_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_3, Dottable_Subp => False, Env => Env, Origin => Origin, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 9941
Dyn_Var_Bind_Result := Fld_6; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 9942
Scope_Result_2 := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3243;
   Result_Var_2 := Scope_Result_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13661:28") else null)); 
   Result_Var_2 := False_Rel;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_10 := Result_Var_2;
else
   --# expr-start 10039 '<If at nodes.lkt:13656:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
--# expr-start 9944 '<Not at nodes.lkt:13663:14>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
--# expr-start 9943 '<IsNull at nodes.lkt:13663:18>' Is_Null_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
Is_Null_5 := Local_Atd.Node = null; 
--# expr-done 9943
Not_Val_1 := not (Is_Null_5); 
--# expr-done 9944
if Not_Val_1 then
   --# expr-start 9947 '<Not at nodes.lkt:13663:34>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
--# expr-start 9946 '<IsNull at nodes.lkt:13663:38>' Is_Null_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
--# expr-start 9945 '<FieldAccess for indices at nodes.lkt:13663:38>' Fld_61 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_61 := Create_Internal_Entity_Array_Indices (Node => Local_Atd.Node.Array_Type_Def_F_Indices, Info => Local_Atd.Info);
--# expr-done 9945
Is_Null_6 := Fld_61.Node = null; 
--# expr-done 9946
Not_Val_2 := not (Is_Null_6); 
--# expr-done 9947
   If_Result_8 := Not_Val_2;
else
   If_Result_8 := False;
end if;
if If_Result_8 then
   --# expr-start 10002 '<Match at nodes.lkt:13663:63>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
--# expr-start 9948 '<FieldAccess for suffix at nodes.lkt:13663:69>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13663'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 9948
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_7; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Assoc_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_11; 
--# bind _ Match_0
--# expr-start 9969 '<Or at nodes.lkt:13664:34>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13664'
--# expr-start 9949 '<FieldAccess for params at nodes.lkt:13666:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13666'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_8 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9949
Fld_9 := Fld_8.Node;
Get_Result := Get (Self, Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_1 := As_Entity;
end if;
Var_Expr_5 := If_Result_1;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind param Var_Expr_5
--# expr-start 9951 '<Cast to Name at nodes.lkt:13667:32>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13667'
--# expr-start 9950 '<FieldAccess for expr at nodes.lkt:13667:32>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13667'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 9950
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 9951
Var_Expr_6 := Cast_Result_4;
if Var_Expr_6 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_6
--# expr-start 9958 '<If at nodes.lkt:13668:35>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
--# expr-start 9953 '<IsNull at nodes.lkt:13668:38>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
--# expr-start 9952 '<FieldAccess for name_designated_type at nodes.lkt:13668:38>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_14 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 9952
Is_Null_1 := Fld_14.Node = null; 
--# expr-done 9953
if Is_Null_1 then
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13668:79") else null)); 
   If_Result_2 := False_Rel_1;
else
   --# expr-start 9957 '<And at nodes.lkt:13668:91>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
--# expr-start 9954 '<FieldAccess for xref_type_equation at nodes.lkt:13668:91>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
      if Var_Expr_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_12 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Var_Expr_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 9954
--# expr-start 9956 '<Bind at nodes.lkt:13668:121>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
--# expr-start 9955 '<FieldAccess for type_var at nodes.lkt:13668:125>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13668'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9955
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Cast_Expr_1 := Local_Real_Typ; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Solver.Create_Assign (Fld_13, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13668:121") else null)); 
--# expr-done 9956
And_Pred := Create_And (Fld_12, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13668:91") else null)); 
--# expr-done 9957
   If_Result_2 := And_Pred;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 9958
Scope_Result_3 := If_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3246;
   Result_Var_3 := Scope_Result_3;
else
   False_Rel_2 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13668:165") else null)); 
   Result_Var_3 := False_Rel_2;
end if;
      Inc_Ref (Result_Var_3);
Scope_Result_4 := Result_Var_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3245;
   Result_Var_4 := Scope_Result_4;
else
   False_Rel_3 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13669:36") else null)); 
   Result_Var_4 := False_Rel_3;
end if;
      Inc_Ref (Result_Var_4);
--# expr-start 9968 '<And at nodes.lkt:13673:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13673'
--# expr-start 9964 '<LogicAll at nodes.lkt:13673:17>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13673'
--# expr-start 9959 '<FieldAccess for params at nodes.lkt:13673:17>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13673'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_15 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 9959
Var_Expr_251 := Fld_15;
if Var_Expr_251 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 9963 '<Map at nodes.lkt:13673:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13673'
Saved := Var_Expr_251; 
Fld_16 := Saved.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_16 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
         Index_8 := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_16;
      begin
         for Untyped_Item_7 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_7 := Untyped_Item_7; 
Is_Null_2 := Bare_Item_7 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Basic_Assoc;
else
Fld_17 := Saved.Info;
As_Entity_1 := (Info => Fld_17, Node => Bare_Item_7); 
   If_Result_3 := As_Entity_1;
end if;
                  Item_7 := If_Result_3; 
   --# scope-start
               --# bind pa Item_7
               --# bind i Index_8
      --# expr-start 9962 '<FieldAccess for constrain_index_expr at nodes.lkt:13674:32>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13674'
--# expr-start 9960 '<FieldAccess for indices at nodes.lkt:13674:32>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13674'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Array_Indices (Node => Local_Atd.Node.Array_Type_Def_F_Indices, Info => Local_Atd.Info);
--# expr-done 9960
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9961 '<FieldAccess for expr at nodes.lkt:13674:65>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13674'
      if Item_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_7.Node, E_Info => Item_7.Info);
--# end
--# expr-done 9961
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_18.Node, Index_Expr => Fld_19, Dim => Index_8, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 9962
         declare
            Item_To_Append : constant Logic_Equation := Fld_20;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3248;
               Index_8 := Index_8 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 9963
Scope_Result_5 := Map_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3247;
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Result_Var_5.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13673:17") else null)); 
--# expr-done 9964
--# expr-start 9967 '<Bind at nodes.lkt:13675:23>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13675'
--# expr-start 9965 '<FieldAccess for type_var at nodes.lkt:13675:27>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13675'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9965
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
--# expr-start 9966 '<FieldAccess for comp_type at nodes.lkt:13675:44>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13675'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_22 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Local_Atd.Node, Origin => Origin, E_Info => Local_Atd.Info);
--# end
--# expr-done 9966
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_22.Node,
         Info => Fld_22.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_21, Cast_Result_6, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13675:23") else null)); 
--# expr-done 9967
And_Pred_1 := Create_And (Logic_Boolean_Op, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13673:17") else null)); 
--# expr-done 9968
Or_Pred := Create_Or (Result_Var_4, And_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13664:34") else null)); 
--# expr-done 9969
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_6 := Let_Result;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3244;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_12; 
--# bind bo Match_1
--# expr-start 9990 '<And at nodes.lkt:13679:31>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13679'
--# expr-start 9987 '<And at nodes.lkt:13680:17>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13680'
--# expr-start 9984 '<And at nodes.lkt:13681:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13681'
--# expr-start 9981 '<And at nodes.lkt:13682:25>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13682'
--# expr-start 9976 '<And at nodes.lkt:13683:29>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
--# expr-start 9972 '<FieldAccess for constrain_index_expr at nodes.lkt:13683:29>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
--# expr-start 9970 '<FieldAccess for indices at nodes.lkt:13683:29>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Array_Indices (Node => Local_Atd.Node.Array_Type_Def_F_Indices, Info => Local_Atd.Info);
--# expr-done 9970
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9971 '<FieldAccess for left at nodes.lkt:13683:62>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 9971
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_23.Node, Index_Expr => Fld_24, Dim => 0, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 9972
--# expr-start 9975 '<FieldAccess for constrain_index_expr at nodes.lkt:13683:78>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
--# expr-start 9973 '<FieldAccess for indices at nodes.lkt:13683:78>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Array_Indices (Node => Local_Atd.Node.Array_Type_Def_F_Indices, Info => Local_Atd.Info);
--# expr-done 9973
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 9974 '<FieldAccess for right at nodes.lkt:13683:111>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13683'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9974
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_28 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_26.Node, Index_Expr => Fld_27, Dim => 0, Origin => Origin, E_Info => Fld_26.Info);
--# end
--# expr-done 9975
And_Pred_2 := Create_And (Fld_25, Fld_28, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13683:29") else null)); 
--# expr-done 9976
--# expr-start 9980 '<Bind at nodes.lkt:13684:31>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13684'
--# expr-start 9977 '<FieldAccess for expected_type_var at nodes.lkt:13684:35>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13684'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_1.Node);
--# expr-done 9977
Fld_29.Value := No_Entity;
Entity_Vars.Reset (Fld_29);
--# expr-start 9979 '<FieldAccess for expected_type_var at nodes.lkt:13684:59>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13684'
--# expr-start 9978 '<FieldAccess for right at nodes.lkt:13684:59>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13684'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9978
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_30.Node);
--# expr-done 9979
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
Bind_Result_2 := Solver.Create_Unify (Fld_29, Fld_31, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13684:31") else null)); 
--# expr-done 9980
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13682:25") else null)); 
--# expr-done 9981
--# expr-start 9983 '<Bind at nodes.lkt:13685:27>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13685'
--# expr-start 9982 '<FieldAccess for type_var at nodes.lkt:13685:31>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13685'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9982
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
Cast_Expr_2 := Local_Real_Typ; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_32, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13685:27") else null)); 
--# expr-done 9983
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13681:21") else null)); 
--# expr-done 9984
--# expr-start 9986 '<FieldAccess for sub_equation at nodes.lkt:13686:23>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13686'
--# expr-start 9985 '<FieldAccess for left at nodes.lkt:13686:23>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13686'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 9985
      if Fld_33.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_33.Info);
--# end
--# expr-done 9986
And_Pred_5 := Create_And (And_Pred_4, Fld_34, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13680:17") else null)); 
--# expr-done 9987
--# expr-start 9989 '<FieldAccess for sub_equation at nodes.lkt:13687:19>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13687'
--# expr-start 9988 '<FieldAccess for right at nodes.lkt:13687:19>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13687'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 9988
      if Fld_35.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_35.Info);
--# end
--# expr-done 9989
And_Pred_6 := Create_And (And_Pred_5, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13679:31") else null)); 
--# expr-done 9990
Let_Result_1 := And_Pred_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_7 := Let_Result_1;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3249;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_13; 
--# bind ar Match_2
--# expr-start 9997 '<And at nodes.lkt:13690:38>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13690'
--# expr-start 9994 '<And at nodes.lkt:13691:17>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13691'
--# expr-start 9991 '<FieldAccess for sub_equation at nodes.lkt:13691:17>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13691'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_37 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 9991
--# expr-start 9993 '<FieldAccess for constrain_index_expr at nodes.lkt:13691:39>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13691'
--# expr-start 9992 '<FieldAccess for indices at nodes.lkt:13691:39>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13691'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_38 := Create_Internal_Entity_Array_Indices (Node => Local_Atd.Node.Array_Type_Def_F_Indices, Info => Local_Atd.Info);
--# expr-done 9992
      if Fld_38.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_3 := Match_2; 
      Cast_Result_8 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_39 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_38.Node, Index_Expr => Cast_Result_8, Dim => 0, Origin => Origin, E_Info => Fld_38.Info);
--# end
--# expr-done 9993
And_Pred_7 := Create_And (Fld_37, Fld_39, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13691:17") else null)); 
--# expr-done 9994
--# expr-start 9996 '<Bind at nodes.lkt:13692:19>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13692'
--# expr-start 9995 '<FieldAccess for type_var at nodes.lkt:13692:23>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13692'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9995
Fld_40.Value := No_Entity;
Entity_Vars.Reset (Fld_40);
Cast_Expr_4 := Local_Real_Typ; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_40, Cast_Result_9, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13692:19") else null)); 
--# expr-done 9996
And_Pred_8 := Create_And (And_Pred_7, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13690:38") else null)); 
--# expr-done 9997
Let_Result_2 := And_Pred_8; Inc_Ref (Let_Result_2);
--# end
Scope_Result_8 := Let_Result_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3250;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_14; 
--# bind st Match_3
--# expr-start 10001 '<And at nodes.lkt:13695:43>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13695'
--# expr-start 9998 '<FieldAccess for sub_equation at nodes.lkt:13695:43>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13695'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_41 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_3.Info);
--# end
--# expr-done 9998
--# expr-start 10000 '<Bind at nodes.lkt:13695:65>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13695'
--# expr-start 9999 '<FieldAccess for type_var at nodes.lkt:13695:69>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13695'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 9999
Fld_42.Value := No_Entity;
Entity_Vars.Reset (Fld_42);
Cast_Expr_5 := Local_Real_Typ; 
      Cast_Result_10 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_42, Cast_Result_10, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13695:65") else null)); 
--# expr-done 10000
And_Pred_9 := Create_And (Fld_41, Bind_Result_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13695:43") else null)); 
--# expr-done 10001
Let_Result_3 := And_Pred_9; Inc_Ref (Let_Result_3);
--# end
Scope_Result_9 := Let_Result_3;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_3251;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Case_Expr_Alternative_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_15; 
--# bind _ Match_4
False_Rel_4 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13696:23") else null)); 
Let_Result_4 := False_Rel_4; Inc_Ref (Let_Result_4);
--# end
Scope_Result_10 := Let_Result_4;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_3252;
            Match_Result := Scope_Result_10; Inc_Ref (Match_Result);
end case;
--# expr-done 10002
   If_Result_9 := Match_Result;
else
   --# expr-start 10038 '<If at nodes.lkt:13656:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13656'
--# expr-start 10004 '<Not at nodes.lkt:13699:14>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13699'
--# expr-start 10003 '<IsNull at nodes.lkt:13699:18>' Is_Null_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13699'
Is_Null_4 := Typ.Node = null; 
--# expr-done 10003
Not_Val := not (Is_Null_4); 
--# expr-done 10004
if Not_Val then
   --# expr-start 10005 '<FieldAccess for has_ud_indexing at nodes.lkt:13699:34>' Fld_60 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13699'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_60 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 10005
   If_Result_6 := Fld_60;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 10037 '<LogicAny at nodes.lkt:13699:61>' Logic_Boolean_Op_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13699'
--# expr-start 10036 '<Map at nodes.lkt:13699:61>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13699'
   --# expr-start 10006 '<FieldAccess for constant_indexing_fns at nodes.lkt:13700:13>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13700'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_43 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 10006
--# expr-start 10007 '<FieldAccess for variable_indexing_fns at nodes.lkt:13700:43>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13700'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_44 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 10007
Concat_Result := Concat (Fld_43, Fld_44); 
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_2
      --# expr-start 10035 '<Let at nodes.lkt:13703:17>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13703'
--# scope-start
--# expr-start 10009 '<FieldAccess for unpacked_formal_params at nodes.lkt:13703:31>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13703'
--# expr-start 10008 '<FieldAccess for subp_spec_or_null at nodes.lkt:13703:31>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13703'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_45 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_2.Node, E_Info => Item_2.Info);
--# end
--# expr-done 10008
      if Fld_45.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_46 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_45.Node, E_Info => Fld_45.Info);
--# end
--# expr-done 10009
Local_Formals := Fld_46; Inc_Ref (Local_Formals);
--# bind formals Local_Formals
--# expr-start 10034 '<Let at nodes.lkt:13704:17>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13704'
--# scope-start
--# expr-start 10011 '<FieldAccess for return_type at nodes.lkt:13704:32>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13704'
--# expr-start 10010 '<FieldAccess for subp_spec_or_null at nodes.lkt:13704:32>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13704'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_47 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_2.Node, E_Info => Item_2.Info);
--# end
--# expr-done 10010
      if Fld_47.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_48 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_47.Node, Origin => Origin, E_Info => Fld_47.Info);
--# end
--# expr-done 10011
Local_Ret_Type := Fld_48; 
--# bind ret_type Local_Ret_Type
--# expr-start 10033 '<Let at nodes.lkt:13705:17>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13705'
--# scope-start
--# expr-start 10012 '<FieldAccess for params at nodes.lkt:13705:30>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13705'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_49 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10012
Local_Params := Fld_49; 
--# bind params Local_Params
--# expr-start 10032 '<If at nodes.lkt:13709:17>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10016 '<Eq at nodes.lkt:13709:20>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10013 '<length at nodes.lkt:13709:20>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
Len := Length (Local_Formals); 
--# expr-done 10013
--# expr-start 10015 '<Arithmetic + at nodes.lkt:13709:40>' Arith_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10014 '<length at nodes.lkt:13709:40>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
Fld_59 := Local_Params.Node; 
Len_1 := Length (Fld_59); 
--# expr-done 10014
Arith_Result_1 := (Len_1 + 1); 
--# expr-done 10015
Is_Equal := Len = Arith_Result_1; 
--# expr-done 10016
if Is_Equal then
   --# expr-start 10031 '<And at nodes.lkt:13709:65>' And_Pred_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10018 '<Bind at nodes.lkt:13709:65>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10017 '<FieldAccess for type_var at nodes.lkt:13709:69>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10017
Fld_50.Value := No_Entity;
Entity_Vars.Reset (Fld_50);
Cast_Expr_6 := Local_Ret_Type; 
      Cast_Result_16 := Create_Internal_Entity
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
Bind_Result_6 := Solver.Create_Assign (Fld_50, Cast_Result_16, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13709:65") else null)); 
--# expr-done 10018
--# expr-start 10030 '<LogicAll at nodes.lkt:13709:100>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
--# expr-start 10029 '<Map at nodes.lkt:13709:100>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13709'
Saved_1 := Local_Params; 
Fld_51 := Saved_1.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_51 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
         Index_4 := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_51;
      begin
         for Untyped_Item_3 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_3 := Untyped_Item_3; 
Is_Null_3 := Bare_Item_3 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Basic_Assoc;
else
Fld_52 := Saved_1.Info;
As_Entity_2 := (Info => Fld_52, Node => Bare_Item_3); 
   If_Result_4 := As_Entity_2;
end if;
                  Item_3 := If_Result_4; 
   --# scope-start
               --# bind param Item_3
               --# bind i Index_4
      --# expr-start 10028 '<And at nodes.lkt:13710:35>' And_Pred_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10025 '<Bind at nodes.lkt:13710:35>' Bind_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10020 '<FieldAccess for expected_type_var at nodes.lkt:13710:39>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10019 '<FieldAccess for expr at nodes.lkt:13710:39>' Fld_53 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_53 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_3.Node, E_Info => Item_3.Info);
--# end
--# expr-done 10019
      if Fld_53.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_53.Node);
--# expr-done 10020
Fld_54.Value := No_Entity;
Entity_Vars.Reset (Fld_54);
--# expr-start 10024 '<FieldAccess for formal_type at nodes.lkt:13710:73>' Fld_56 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10023 '<FieldAccess for formal_decl at nodes.lkt:13710:73>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10022 '<at at nodes.lkt:13710:73>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10021 '<Arithmetic + at nodes.lkt:13710:82>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
Arith_Result := (Index_4 + 1); 
--# expr-done 10021
Get_Result_1 := Get (Self, Local_Formals, Arith_Result, True); 
--# expr-done 10022
      if Get_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_55 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# end
--# expr-done 10023
      if Fld_55.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_56 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_55.Node, Origin => Origin, E_Info => Fld_55.Info);
--# end
--# expr-done 10024
      Cast_Result_17 := Create_Internal_Entity
        (Node => Fld_56.Node,
         Info => Fld_56.Info);
Bind_Result_7 := Solver.Create_Assign (Fld_54, Cast_Result_17, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13710:35") else null)); 
--# expr-done 10025
--# expr-start 10027 '<FieldAccess for matches_expected_type at nodes.lkt:13710:122>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
--# expr-start 10026 '<FieldAccess for expr at nodes.lkt:13710:122>' Fld_57 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13710'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_57 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_3.Node, E_Info => Item_3.Info);
--# end
--# expr-done 10026
      if Fld_57.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_58 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_57.Node, Origin => Origin);
--# end
--# expr-done 10027
And_Pred_10 := Create_And (Bind_Result_7, Fld_58, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13710:35") else null)); 
--# expr-done 10028
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3254;
               Index_4 := Index_4 + 1;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10029
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13709:100") else null)); 
--# expr-done 10030
And_Pred_11 := Create_And (Bind_Result_6, Logic_Boolean_Op_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13709:65") else null)); 
--# expr-done 10031
   If_Result_5 := And_Pred_11;
else
   False_Rel_5 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13711:24") else null)); 
   If_Result_5 := False_Rel_5;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 10032
Let_Result_5 := If_Result_5; Inc_Ref (Let_Result_5);
--# end
--# expr-done 10033
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 10034
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 10035
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_7;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3253;
         end loop;
      end;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10036
Logic_Boolean_Op_2 := Solver.Create_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13699:61") else null)); 
--# expr-done 10037
   If_Result_7 := Logic_Boolean_Op_2;
else
   False_Rel_6 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13714:14") else null)); 
   If_Result_7 := False_Rel_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 10038
   If_Result_9 := If_Result_7;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 10039
   If_Result_10 := If_Result_9;
end if;
      Inc_Ref (If_Result_10);
--# expr-done 10040
Let_Result_8 := If_Result_10; Inc_Ref (Let_Result_8);
--# end
--# expr-done 10041
Let_Result_9 := Let_Result_8; Inc_Ref (Let_Result_9);
--# end
--# expr-done 10042
         Property_Result := Let_Result_9;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1209;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1209;
                     Finalizer_Scope_3243;
                     Finalizer_Scope_3244;
                     Finalizer_Scope_3245;
                     Finalizer_Scope_3246;
                     Finalizer_Scope_3247;
                     Finalizer_Scope_3248;
                     Finalizer_Scope_3249;
                     Finalizer_Scope_3250;
                     Finalizer_Scope_3251;
                     Finalizer_Scope_3252;
                     Finalizer_Scope_3253;
                     Finalizer_Scope_3254;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subscriptable_Type_Equation;
--# end
--# property-start CallExpr.subprogram_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13718'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subprogram_Equation
  (Node : Bare_Call_Expr
      ; Subp_Spec : Internal_Entity_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp_spec Subp_Spec
   --# bind dottable_subp Dottable_Subp
   --# bind env Env
   --# bind origin Origin
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1210 with Inline_Always;
            procedure Finalizer_Scope_3255 with Inline_Always;
            procedure Finalizer_Scope_3257 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Formal_Param_Holder;
Local_N : Internal_Entity_Base_Formal_Param_Decl;
Var_Expr_252 : Internal_Entity_Base_Subp_Spec;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
Fld_2 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Fld_7 : Internal_Single_Actual;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_11 : Internal_Single_Actual;
Fld_12 : Bare_Base_Id;
Fld_13 : Logic_Var;
Fld_14 : Internal_Entity_Defining_Name;
Fld_15 : Internal_Entity_Defining_Name;
Fld_16 : Internal_Entity_Base_Formal_Param_Decl;
Fld_17 : Bare_Base_Formal_Param_Decl;
Cast_Result_3 : Bare_Ada_Node;
Fld_18 : Internal_Entity_Info;
Fld_19 : Env_Rebindings;
Fld_20 : Internal_Entity_Info;
Fld_21 : Boolean;
Fld_22 : Internal_Entity;
Let_Result : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Bare_Base_Id;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_25 : Boolean;
If_Result_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
            procedure Finalizer_Scope_1210 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1210;
            procedure Finalizer_Scope_3255 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3255;
            procedure Finalizer_Scope_3257 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_3257;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_0 := Subp_Spec;
if Var_Expr_0 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind subp_spec Var_Expr_0
--# expr-start 10081 '<And at nodes.lkt:13719:24>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13719'
--# expr-start 10049 '<And at nodes.lkt:13722:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13722'
--# expr-start 10046 '<Bind at nodes.lkt:13722:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13722'
--# expr-start 10043 '<FieldAccess for type_var at nodes.lkt:13722:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13722'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10043
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 10044 '<Cast to BaseSubpSpec at nodes.lkt:13722:34>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13722'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 10044
Var_Expr_252 := Cast_Result;
if Var_Expr_252 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 10045 '<FieldAccess for return_type at nodes.lkt:13722:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13722'
      if Var_Expr_252.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_252.Node, Origin => Origin, E_Info => Var_Expr_252.Info);
--# end
--# expr-done 10045
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Assign (Fld, Cast_Result_1, Solver_Ifc.No_Converter, Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13722:13") else null)); 
--# expr-done 10046
--# expr-start 10048 '<Bind at nodes.lkt:13725:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13725'
--# expr-start 10047 '<FieldAccess for subp_spec_var at nodes.lkt:13725:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13725'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 10047
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Cast_Expr_1 := Var_Expr_0; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_2, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13725:17") else null)); 
--# expr-done 10048
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13722:13") else null)); 
--# expr-done 10049
--# expr-start 10080 '<LogicAll at nodes.lkt:13730:13>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13730'
--# expr-start 10079 '<Map at nodes.lkt:13730:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13730'
   --# expr-start 10051 '<FieldAccess for match_param_list at nodes.lkt:13730:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13730'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10050 '<FieldAccess for params at nodes.lkt:13730:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13730'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_3 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10050
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Var_Expr_0.Node, Params => Fld_3, Is_Dottable_Subp => Dottable_Subp, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10051
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_1
      --# expr-start 10078 '<If at nodes.lkt:13731:25>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13731'
--# expr-start 10052 '<FieldAccess for has_matched at nodes.lkt:13731:28>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13731'
Fld_25 := Item_1.Has_Matched;
--# expr-done 10052
if Fld_25 then
   --# expr-start 10077 '<And at nodes.lkt:13731:48>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13731'
--# expr-start 10058 '<FieldAccess for call_argument_equation at nodes.lkt:13731:48>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13731'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10054 '<FieldAccess for formal_decl at nodes.lkt:13732:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13732'
--# expr-start 10053 '<FieldAccess for formal at nodes.lkt:13732:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13732'
Fld_5 := Item_1.Formal;
--# expr-done 10053
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 10054
--# expr-start 10057 '<FieldAccess for expr at nodes.lkt:13732:46>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13732'
--# expr-start 10056 '<FieldAccess for assoc at nodes.lkt:13732:46>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13732'
--# expr-start 10055 '<FieldAccess for actual at nodes.lkt:13732:46>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13732'
Fld_7 := Item_1.Actual;
--# expr-done 10055
Fld_8 := Fld_7.Assoc;
--# expr-done 10056
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 10057
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Var_Expr_0.Node, Param => Fld_6, Arg => Fld_9, Origin => Origin, Logic_Context => Logic_Context, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10058
--# expr-start 10076 '<If at nodes.lkt:13736:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10061 '<IsNull at nodes.lkt:13736:24>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10060 '<FieldAccess for name at nodes.lkt:13736:24>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10059 '<FieldAccess for actual at nodes.lkt:13736:24>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
Fld_23 := Item_1.Actual;
--# expr-done 10059
Fld_24 := Fld_23.Name;
--# expr-done 10060
Is_Null := Fld_24 = null; 
--# expr-done 10061
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13736:52") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 10075 '<Bind at nodes.lkt:13736:63>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10064 '<FieldAccess for ref_var at nodes.lkt:13736:67>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10063 '<FieldAccess for name at nodes.lkt:13736:67>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
--# expr-start 10062 '<FieldAccess for actual at nodes.lkt:13736:67>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13736'
Fld_11 := Item_1.Actual;
--# expr-done 10062
Fld_12 := Fld_11.Name;
--# expr-done 10063
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 10064
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 10074 '<Let at nodes.lkt:13737:25>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13737'
--# scope-start
--# expr-start 10067 '<FieldAccess for formal_decl at nodes.lkt:13737:33>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13737'
--# expr-start 10066 '<FieldAccess for corresponding_actual_param at nodes.lkt:13737:33>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13737'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10065 '<FieldAccess for formal at nodes.lkt:13737:70>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13737'
Fld_14 := Item_1.Formal;
--# expr-done 10065
--# property-call-start BaseFormalParamHolder.corresponding_actual_param
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Corresponding_Actual_Param (Node => Var_Expr_0.Node, Name => Fld_14, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10066
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_16 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# end
--# expr-done 10067
Local_N := Fld_16; 
--# bind n Local_N
--# expr-start 10073 '<FieldAccess for entity_no_md at nodes.lkt:13739:25>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13739'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10068 '<FieldAccess for node at nodes.lkt:13740:29>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13740'
Fld_17 := Local_N.Node;
--# expr-done 10068
      Cast_Result_3 := Fld_17;
--# expr-start 10070 '<FieldAccess for rebindings at nodes.lkt:13740:37>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13740'
--# expr-start 10069 '<FieldAccess for info at nodes.lkt:13740:37>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13740'
Fld_18 := Local_N.Info;
--# expr-done 10069
Fld_19 := Fld_18.Rebindings;
--# expr-done 10070
--# expr-start 10072 '<FieldAccess for from_rebound at nodes.lkt:13740:56>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13740'
--# expr-start 10071 '<FieldAccess for info at nodes.lkt:13740:56>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13740'
Fld_20 := Local_N.Info;
--# expr-done 10071
Fld_21 := Fld_20.From_Rebound;
--# expr-done 10072
--# property-call-start AdaNode.entity_no_md
Fld_22 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Cast_Result_3, Rebindings => Fld_19, From_Rebound => Fld_21);
--# end
--# expr-done 10073
Let_Result := Fld_22; 
--# end
--# expr-done 10074
Bind_Result_2 := Solver.Create_Assign (Fld_13, Let_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13736:63") else null)); 
--# expr-done 10075
   If_Result := Bind_Result_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 10076
And_Pred_1 := Create_And (Fld_10, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13731:48") else null)); 
--# expr-done 10077
   If_Result_1 := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13743:24") else null)); 
   If_Result_1 := False_Rel;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10078
         declare
            Item_To_Append : constant Logic_Equation := If_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3257;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10079
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13730:13") else null)); 
--# expr-done 10080
And_Pred_2 := Create_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13719:24") else null)); 
--# expr-done 10081
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3255;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:13745:24") else null)); 
   Result_Var_1 := False_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1210;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1210;
                     Finalizer_Scope_3255;
                     Finalizer_Scope_3257;
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Subprogram_Equation;
--# end
--# property-start CallExpr.check_for_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13753'
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_For_Type
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr  := Bare_Call_Expr (Node);
     Ent : Internal_Entity_Call_Expr :=
       Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Atd : Internal_Entity_Array_Type_Def;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Match_0 : Bare_Assoc_List;
Match_1 : Bare_Bin_Op;
Match_2 : Bare_Subtype_Indication;
Match_3 : Bare_Attribute_Ref;
Match_4 : Bare_Ada_Node;
Var_Expr_3 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_4 : Bare_Assoc_List;
Var_Expr_5 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Integer;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Integer;
Is_Equal_1 : Boolean;
Fld_5 : Integer;
Is_Equal_2 : Boolean;
Fld_6 : Integer;
Is_Equal_3 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Assoc_List;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subtype_Indication;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Attribute_Ref;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
Scope_Result_6 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Access_Def;
Cast_Result_5 : Internal_Entity_Access_To_Subp_Def;
Fld_8 : Internal_Entity_Subp_Spec;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Fld_11 : Boolean;
Fld_12 : Bare_Ada_Node;
Cast_Result_6 : Bare_Assoc_List;
Len_1 : Integer;
Comp_Result : Boolean;
Scope_Result_8 : Boolean;
Result_Var_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_13 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_17 : Boolean;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_4 : Boolean;
If_Result_5 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10114 '<Let at nodes.lkt:13757:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13757'
--# scope-start
Var_Expr_0 := Typ;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 10082 '<FieldAccess for array_def_with_deref at nodes.lkt:13757:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13757'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10082
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Local_Atd := Result_Var; 
--# bind atd Local_Atd
--# expr-start 10113 '<bind at nodes.lkt:13760:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13760'
--# expr-start 10083 '<FieldAccess for origin_node at nodes.lkt:13760:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13760'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10083
Bound_Origin := Fld_1; 
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
Var_Expr_2 := Local_Atd;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 10094 '<Match at nodes.lkt:13767:36>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13767'
--# expr-start 10084 '<FieldAccess for suffix at nodes.lkt:13767:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13767'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Call_Expr_F_Suffix;
--# expr-done 10084
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind al Match_0
--# expr-start 10087 '<Eq at nodes.lkt:13769:55>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13769'
--# expr-start 10085 '<FieldAccess for array_ndims at nodes.lkt:13769:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13769'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Local_Atd.Node, Origin => Bound_Origin, E_Info => Local_Atd.Info);
--# end
--# expr-done 10085
--# expr-start 10086 '<length at nodes.lkt:13769:76>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13769'
Len := Length (Match_0); 
--# expr-done 10086
Is_Equal := Fld_3 = Len; 
--# expr-done 10087
Let_Result := Is_Equal; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 10089 '<Eq at nodes.lkt:13772:50>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13772'
--# expr-start 10088 '<FieldAccess for array_ndims at nodes.lkt:13772:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13772'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Local_Atd.Node, Origin => Bound_Origin, E_Info => Local_Atd.Info);
--# end
--# expr-done 10088
Is_Equal_1 := Fld_4 = 1; 
--# expr-done 10089
Let_Result_1 := Is_Equal_1; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 10091 '<Eq at nodes.lkt:13773:62>' Is_Equal_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13773'
--# expr-start 10090 '<FieldAccess for array_ndims at nodes.lkt:13773:62>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13773'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Local_Atd.Node, Origin => Bound_Origin, E_Info => Local_Atd.Info);
--# end
--# expr-done 10090
Is_Equal_2 := Fld_5 = 1; 
--# expr-done 10091
Let_Result_2 := Is_Equal_2; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 10093 '<Eq at nodes.lkt:13774:57>' Is_Equal_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13774'
--# expr-start 10092 '<FieldAccess for array_ndims at nodes.lkt:13774:57>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13774'
      if Local_Atd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Local_Atd.Node, Origin => Bound_Origin, E_Info => Local_Atd.Info);
--# end
--# expr-done 10092
Is_Equal_3 := Fld_6 = 1; 
--# expr-done 10093
Let_Result_3 := Is_Equal_3; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Case_Expr_Alternative_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 10094
Scope_Result_6 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_6;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result := True;
else
--# expr-start 10096 '<Cast to AccessToSubpDef at nodes.lkt:13780:25>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13780'
--# expr-start 10095 '<FieldAccess for access_def at nodes.lkt:13780:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13780'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10095
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_5 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 10096
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_3
--# expr-start 10099 '<FieldAccess for is_matching_param_list at nodes.lkt:13781:37>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13781'
--# expr-start 10097 '<FieldAccess for subp_spec at nodes.lkt:13781:37>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13781'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_3.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_3.Info);
--# expr-done 10097
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10098 '<FieldAccess for params at nodes.lkt:13781:73>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13781'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_9 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10098
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Fld_8.Node, Params => Fld_9, Is_Dottable_Subp => False, Env => Env);
--# end
--# expr-done 10099
Scope_Result_7 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := False;
end if;
   If_Result := Result_Var_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 10100 '<FieldAccess for has_ud_indexing at nodes.lkt:13785:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13785'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10100
if Fld_11 then
--# expr-start 10102 '<Cast to AssocList at nodes.lkt:13785:51>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13785'
--# expr-start 10101 '<FieldAccess for suffix at nodes.lkt:13785:51>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13785'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Call_Expr_F_Suffix;
--# expr-done 10101
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_6 := Fld_12;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 10102
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind al Var_Expr_4
--# expr-start 10103 '<length at nodes.lkt:13785:88>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13785'
Len_1 := Length (Var_Expr_4); 
--# expr-done 10103
Comp_Result := Len_1 >= 1; 
Scope_Result_8 := Comp_Result;
   --# end
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := False;
end if;
   If_Result_1 := Result_Var_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
--# expr-start 10105 '<Cast to CallExpr at nodes.lkt:13790:21>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13790'
--# expr-start 10104 '<parent at nodes.lkt:13790:21>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13790'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10104
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 10105
Var_Expr_5 := Cast_Result_7;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_5
--# expr-start 10112 '<FieldAccess for check_for_type at nodes.lkt:13797:25>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13797'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10111 '<If at nodes.lkt:13798:29>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13798'
--# expr-start 10106 '<FieldAccess for check_array_slice at nodes.lkt:13798:32>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_17 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Var_Expr_1, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 10106
if Fld_17 then
   If_Result_4 := Var_Expr_1;
else
   --# expr-start 10110 '<If at nodes.lkt:13802:33>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13802'
--# expr-start 10107 '<FieldAccess for is_iterable_type at nodes.lkt:13802:36>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13802'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10107
if Fld_16 then
   --# expr-start 10108 '<FieldAccess for iterable_comp_type at nodes.lkt:13803:37>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13803'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10108
   If_Result_3 := Fld_14;
else
   --# expr-start 10109 '<FieldAccess for comp_type at nodes.lkt:13805:37>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13805'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_1.Node, Is_Subscript => True, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10109
   If_Result_3 := Fld_15;
end if;
--# expr-done 10110
   If_Result_4 := If_Result_3;
end if;
--# expr-done 10111
--# property-call-start CallExpr.check_for_type
Fld_18 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_5.Node, Typ => If_Result_4, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 10112
Scope_Result_9 := Fld_18;
   --# end
   Result_Var_4 := Scope_Result_9;
else
   Result_Var_4 := True;
end if;
   If_Result_5 := Result_Var_4;
else
   If_Result_5 := False;
end if;
Scope_Result_10 := If_Result_5;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_5; 
--# expr-done 10113
Let_Result_5 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 10114
         Property_Result := Let_Result_5;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Call_Expr_P_Check_For_Type;
--# end
   --
   --  Primitives for Bare_Defining_Name
   --
      procedure Initialize_Fields_For_Defining_Name
        (Self : Bare_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Defining_Name_F_Name := Defining_Name_F_Name;
      end Initialize_Fields_For_Defining_Name;
   function Defining_Name_F_Name
     (Node : Bare_Defining_Name) return Bare_Name
   is
   begin
         return Node.Defining_Name_F_Name;
   end;
--# property-start DefiningName.parent_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13827'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Parent_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1212 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1212 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1212;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10116 '<FieldAccess for parent_scope at nodes.lkt:13827:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13827'
--# expr-start 10115 '<FieldAccess for name at nodes.lkt:13827:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13827'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 10115
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 10116
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1212;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1212;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Parent_Scope;
--# end
--# property-start DefiningName.scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13830'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1213 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1213 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1213;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10118 '<FieldAccess for scope at nodes.lkt:13830:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13830'
--# expr-start 10117 '<FieldAccess for name at nodes.lkt:13830:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13830'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 10117
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 10118
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1213;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1213;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Scope;
--# end
--# property-start DefiningName.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13832'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Relative_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10120 '<FieldAccess for relative_name at nodes.lkt:13832:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13832'
--# expr-start 10119 '<FieldAccess for name at nodes.lkt:13832:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13832'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 10119
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10120
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Relative_Name;
--# end
--# property-start DefiningName.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13834'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Ref_Var
  (Node : Bare_Defining_Name
  )
   return Logic_Var
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10122 '<FieldAccess for ref_var at nodes.lkt:13834:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13834'
--# expr-start 10121 '<FieldAccess for name at nodes.lkt:13834:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13834'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 10121
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10122
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Ref_Var;
--# end
--# property-start DefiningName.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13837'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Env_Elements_Impl
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1216 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1216 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1216;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10124 '<FieldAccess for env_elements_impl at nodes.lkt:13838:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13838'
--# expr-start 10123 '<FieldAccess for name at nodes.lkt:13838:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13838'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 10123
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 10124
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1216;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1216;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Env_Elements_Impl;
--# end
--# property-start DefiningName.formal_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13842'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Formal_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl;
            procedure Finalizer_Scope_1217 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
            procedure Finalizer_Scope_1217 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1217;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10128 '<Cast to BaseFormalParamDecl at nodes.lkt:13843:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13843'
--# expr-start 10127 '<Find at nodes.lkt:13843:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13843'
   --# expr-start 10125 '<FieldAccess for parents at nodes.lkt:13843:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13843'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 10125
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 10126 '<IsA BaseFormalParamDecl at nodes.lkt:13843:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13843'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 10126
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10127
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10128
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_1217;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1217;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Formal_Decl;
--# end
--# property-start DefiningName.fully_qualified_name_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13847'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name_Impl
  (Node : Bare_Defining_Name
      ; Include_Profile : Boolean
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   --# bind suffix Suffix
   Property_Result : String_Type_Array_Access;
            procedure Finalizer_Scope_1218 with Inline_Always;
            procedure Finalizer_Scope_3270 with Inline_Always;
            procedure Finalizer_Scope_3271 with Inline_Always;
            procedure Finalizer_Scope_3272 with Inline_Always;
            procedure Finalizer_Scope_3273 with Inline_Always;
            procedure Finalizer_Scope_3274 with Inline_Always;
            procedure Finalizer_Scope_3275 with Inline_Always;
            procedure Finalizer_Scope_3276 with Inline_Always;
            procedure Finalizer_Scope_3277 with Inline_Always;
            procedure Finalizer_Scope_3278 with Inline_Always;
      Local_Def_Name_Array : String_Type_Array_Access;
Match_0 : Bare_Synthetic_Defining_Name;
Match_1 : Bare_Defining_Name;
Local_Bd : Internal_Entity_Basic_Decl;
Local_Self_Name : String_Type_Array_Access;
Local_Parent_Decl : Internal_Entity_Basic_Decl;
Local_Is_Instantiated : Boolean;
Local_Fqn : String_Type_Array_Access;
Var_Expr_3 : String_Type_Array_Access;
Var_Expr_4 : Internal_Entity_Subunit;
Var_Expr_253 : Internal_Entity_Basic_Decl;
Var_Expr : String_Type_Array_Access;
Fld : Symbol_Type;
Fld_1 : String_Type;
Array_Lit : String_Type_Array_Access;
Fld_2 : Bare_Single_Tok_Node_Array_Access;
Fld_3 : String_Type;
Map_Result : String_Type_Array_Access;
Match_Prefix : Bare_Defining_Name;
Cast_Result : Bare_Synthetic_Defining_Name;
Let_Result : String_Type_Array_Access;
Scope_Result : String_Type_Array_Access;
Cast_Result_1 : Bare_Defining_Name;
Let_Result_1 : String_Type_Array_Access;
Scope_Result_1 : String_Type_Array_Access;
Match_Result : String_Type_Array_Access;
Fld_4 : Internal_Entity_Basic_Decl;
Index_2 : Integer;
Fld_5 : String_Type;
Str : String_Type;
If_Result : String_Type;
Concat_Result : String_Type;
Str_1 : String_Type;
Len : Integer;
Arith_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : String_Type;
Concat_Result_1 : String_Type;
Map_Result_1 : String_Type_Array_Access;
Fld_6 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result_2 : Boolean;
Fld_7 : String_Type_Array_Access;
Concat_Result_2 : String_Type_Array_Access;
If_Result_3 : String_Type_Array_Access;
Scope_Result_2 : String_Type_Array_Access;
Result_Var : String_Type_Array_Access;
Scope_Result_3 : String_Type_Array_Access;
Result_Var_1 : String_Type_Array_Access;
Not_Val : Boolean;
Fld_8 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : String_Type_Array_Access;
Fld_9 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Subunit;
Fld_10 : Internal_Entity_Name;
Fld_11 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_4 : Bare_Single_Tok_Node_Array_Access;
Result_Var_2 : Bare_Single_Tok_Node_Array_Access;
Fld_12 : String_Type;
Map_Result_2 : String_Type_Array_Access;
Concat_Result_3 : String_Type_Array_Access;
Scope_Result_5 : String_Type_Array_Access;
Result_Var_3 : String_Type_Array_Access;
Let_Result_2 : String_Type_Array_Access;
Let_Result_3 : String_Type_Array_Access;
Let_Result_4 : String_Type_Array_Access;
Let_Result_5 : String_Type_Array_Access;
Let_Result_6 : String_Type_Array_Access;
Let_Result_7 : String_Type_Array_Access;
            procedure Finalizer_Scope_1218 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Local_Def_Name_Array);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Self_Name);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Local_Fqn);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Concat_Result_3);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Let_Result_7);
            end Finalizer_Scope_1218;
            procedure Finalizer_Scope_3270 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3270;
            procedure Finalizer_Scope_3271 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3271;
            procedure Finalizer_Scope_3272 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3272;
            procedure Finalizer_Scope_3273 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Str);
                     Dec_Ref (If_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Str_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_3273;
            procedure Finalizer_Scope_3274 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3274;
            procedure Finalizer_Scope_3275 is
            begin
                     Dec_Ref (Var_Expr_3);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_3275;
            procedure Finalizer_Scope_3276 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_3276;
            procedure Finalizer_Scope_3277 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_3277;
            procedure Finalizer_Scope_3278 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_3278;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10165 '<Let at nodes.lkt:13848:9>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13848'
--# scope-start
--# expr-start 10135 '<Match at nodes.lkt:13848:30>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13848'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Defining_Name_Range (Match_Prefix.Kind) is
         when Ada_Synthetic_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind scel Match_0
--# expr-start 10131 '<ArrayLiteral at nodes.lkt:13849:49>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13849'
--# expr-start 10130 '<FieldAccess for image at nodes.lkt:13849:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13849'
--# expr-start 10129 '<FieldAccess for name_symbol at nodes.lkt:13849:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13849'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_0);
--# end
--# expr-done 10129
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 10130
Array_Lit := Create_String_Type_Array (Internal_String_Type_Array'(1 => Fld_1)); 
--# expr-done 10131
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3270;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 10134 '<Map at nodes.lkt:13850:23>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13850'
   --# expr-start 10132 '<FieldAccess for as_single_tok_node_array at nodes.lkt:13850:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13850'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_2 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Match_1);
--# end
--# expr-done 10132
      declare
         Map_Result_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 10133 '<FieldAccess for text at nodes.lkt:13850:63>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13850'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Text (Node => Item_0);
--# expr-done 10133
         declare
            Item_To_Append : constant String_Type := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3272;
         end loop;
      end;
         Map_Result := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := String_Type_Vectors.Get
              (Map_Result_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10134
Let_Result_1 := Map_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3271;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 10135
Local_Def_Name_Array := Match_Result; Inc_Ref (Local_Def_Name_Array);
--# bind def_name_array Local_Def_Name_Array
--# expr-start 10164 '<Let at nodes.lkt:13852:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13852'
--# scope-start
--# expr-start 10136 '<FieldAccess for basic_decl at nodes.lkt:13852:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13852'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_4 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10136
Local_Bd := Fld_4; 
--# bind bd Local_Bd
--# expr-start 10163 '<Let at nodes.lkt:13853:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13853'
--# scope-start
--# expr-start 10145 '<Map at nodes.lkt:13853:25>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13853'
      declare
         Map_Result_1_Vec : String_Type_Vectors.Vector;
      begin
         Index_2 := 0;
      declare
         Collection : constant String_Type_Array_Access := Local_Def_Name_Array;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_1
               --# bind i Index_2
--# expr-start 10139 '<If at nodes.lkt:13855:16>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13855'
if Include_Profile then
   --# expr-start 10137 '<FieldAccess for custom_id_text at nodes.lkt:13855:40>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13855'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10137
   If_Result := Fld_5;
else
   --# expr-start 10138 '<String at nodes.lkt:13855:65>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13855'
Str := Create_String (""); 
--# expr-done 10138
   If_Result := Str;
end if;
      Inc_Ref (If_Result);
--# expr-done 10139
Concat_Result := Concat_String (Item_1, If_Result); 
--# expr-start 10144 '<If at nodes.lkt:13856:16>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13856'
--# expr-start 10142 '<Eq at nodes.lkt:13856:19>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13856'
--# expr-start 10141 '<Arithmetic - at nodes.lkt:13856:24>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13856'
--# expr-start 10140 '<length at nodes.lkt:13856:24>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13856'
Len := Length (Local_Def_Name_Array); 
--# expr-done 10140
Arith_Result := (Len - 1); 
--# expr-done 10141
Is_Equal := Index_2 = Arith_Result; 
--# expr-done 10142
if Is_Equal then
   If_Result_1 := Suffix;
else
   --# expr-start 10143 '<String at nodes.lkt:13856:69>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13856'
Str_1 := Create_String (""); 
--# expr-done 10143
   If_Result_1 := Str_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10144
Concat_Result_1 := Concat_String (Concat_Result, If_Result_1); 
         declare
            Item_To_Append : constant String_Type := Concat_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3273;
               Index_2 := Index_2 + 1;
         end loop;
      end;
         Map_Result_1 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := String_Type_Vectors.Get
              (Map_Result_1_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10145
Local_Self_Name := Map_Result_1; Inc_Ref (Local_Self_Name);
--# bind self_name Local_Self_Name
--# expr-start 10162 '<Let at nodes.lkt:13858:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13858'
--# scope-start
--# expr-start 10146 '<FieldAccess for parent_basic_decl at nodes.lkt:13858:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13858'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_6 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10146
Local_Parent_Decl := Fld_6; 
--# bind parent_decl Local_Parent_Decl
--# expr-start 10161 '<Let at nodes.lkt:13859:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13859'
--# scope-start
--# expr-start 10147 '<IsA GenericDecl, Body at nodes.lkt:13859:32>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13859'
Is_A := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Generic_Decl | Ada_Body_Node; 
--# expr-done 10147
if Is_A then
   --# expr-start 10148 '<IsA GenericInstantiation at nodes.lkt:13860:17>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13860'
Is_A_1 := Local_Parent_Decl.Node /= null 
and then Local_Parent_Decl.Node.Kind in Ada_Generic_Instantiation; 
--# expr-done 10148
   If_Result_2 := Is_A_1;
else
   If_Result_2 := False;
end if;
Local_Is_Instantiated := If_Result_2; 
--# bind is_instantiated Local_Is_Instantiated
--# expr-start 10160 '<Let at nodes.lkt:13861:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13861'
--# scope-start
--# expr-start 10153 '<If at nodes.lkt:13861:19>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13861'
--# expr-start 10149 '<Not at nodes.lkt:13861:22>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13861'
Not_Val := not (Local_Is_Instantiated); 
--# expr-done 10149
if Not_Val then
   --# expr-start 10150 '<FieldAccess for is_compilation_unit_root at nodes.lkt:13861:46>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13861'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Local_Bd.Node);
--# end
--# expr-done 10150
   If_Result_4 := Fld_8;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   If_Result_5 := Local_Self_Name;
else
Var_Expr_253 := Local_Parent_Decl;
if Var_Expr_253 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10151 '<FieldAccess for fully_qualified_name_string_array at nodes.lkt:13864:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13864'
      if Var_Expr_253.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.fully_qualified_name_string_array
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_String_Array (Node => Var_Expr_253.Node, Include_Profile => Include_Profile, E_Info => Var_Expr_253.Info);
--# end
--# expr-done 10151
Var_Expr_3 := Fld_7;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_String_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr_3
--# expr-start 10152 '<If at nodes.lkt:13870:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13870'
if Local_Is_Instantiated then
   If_Result_3 := Var_Expr_3;
else
Concat_Result_2 := Concat (Var_Expr_3, Local_Self_Name); 
   If_Result_3 := Concat_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 10152
Scope_Result_2 := If_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3275;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_String_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_3 := Result_Var;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3274;
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := No_String_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_5 := Result_Var_1;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 10153
Local_Fqn := If_Result_5; Inc_Ref (Local_Fqn);
--# bind fqn Local_Fqn
--# expr-start 10159 '<Map at nodes.lkt:13873:9>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13873'
--# expr-start 10155 '<Cast to Subunit at nodes.lkt:13873:9>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13873'
--# expr-start 10154 '<parent at nodes.lkt:13873:9>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13873'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# expr-done 10154
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Subunit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Subunit
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_2 := No_Entity_Subunit;
   end if;
--# expr-done 10155
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Subunit then
   --# scope-start
   --# bind su Var_Expr_4
--# expr-start 10157 '<FieldAccess for as_single_tok_node_array at nodes.lkt:13874:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13874'
--# expr-start 10156 '<FieldAccess for name at nodes.lkt:13874:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13874'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_4.Node.Subunit_F_Name, Info => Var_Expr_4.Info);
--# expr-done 10156
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_11 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_10.Node);
--# end
--# expr-done 10157
Scope_Result_4 := Fld_11;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3276;
   Result_Var_2 := Scope_Result_4;
else
   Result_Var_2 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
      declare
         Map_Result_2_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Result_Var_2;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_5
      --# expr-start 10158 '<FieldAccess for text at nodes.lkt:13875:22>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13875'
      if Item_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Text (Node => Item_5);
--# expr-done 10158
         declare
            Item_To_Append : constant String_Type := Fld_12;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3277;
         end loop;
      end;
         Map_Result_2 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := String_Type_Vectors.Get
              (Map_Result_2_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10159
Concat_Result_3 := Concat (Map_Result_2, Local_Fqn); 
Var_Expr := Concat_Result_3;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_String_Type_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_5 := Var_Expr;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3278;
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := Local_Fqn;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10160
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10161
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 10162
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 10163
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 10164
Let_Result_7 := Let_Result_6; Inc_Ref (Let_Result_7);
--# end
--# expr-done 10165
         Property_Result := Let_Result_7;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1218;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1218;
                     Finalizer_Scope_3270;
                     Finalizer_Scope_3271;
                     Finalizer_Scope_3272;
                     Finalizer_Scope_3273;
                     Finalizer_Scope_3274;
                     Finalizer_Scope_3275;
                     Finalizer_Scope_3276;
                     Finalizer_Scope_3277;
                     Finalizer_Scope_3278;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name_Impl;
--# end
--# property-start DefiningName.canonical_fully_qualified_name_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Fully_Qualified_Name_Impl
  (Node : Bare_Defining_Name
      ; Include_Profile : Boolean
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind include_profile Include_Profile
   --# bind suffix Suffix
   Property_Result : String_Type;
            procedure Finalizer_Scope_1219 with Inline_Always;
            procedure Finalizer_Scope_3280 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Fld_1 : String_Type;
Map_Result_1 : String_Type_Array_Access;
Join_Result : String_Type;
            procedure Finalizer_Scope_1219 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_1219;
            procedure Finalizer_Scope_3280 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3280;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10172 '<Join at nodes.lkt:13879:94>' Join_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
--# expr-start 10166 '<String at nodes.lkt:13879:94>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
Str := Create_String ("."); 
--# expr-done 10166
--# expr-start 10171 '<Map at nodes.lkt:13879:103>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
   --# expr-start 10169 '<Map at nodes.lkt:13879:103>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
   --# expr-start 10167 '<FieldAccess for fully_qualified_name_impl at nodes.lkt:13879:103>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13879'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => Include_Profile, Suffix => Suffix, E_Info => Ent.Info);
--# end
--# expr-done 10167
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 10168 '<to_symbol at nodes.lkt:13883:17>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13883'
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_0); 
--# expr-done 10168
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10169
      declare
         Map_Result_1_Vec : String_Type_Vectors.Vector;
      begin
      declare
         Collection : constant Symbol_Type_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_1
      --# expr-start 10170 '<FieldAccess for image at nodes.lkt:13883:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13883'
Fld_1 := Libadalang.Implementation.Image (Self => Item_1);
--# expr-done 10170
         declare
            Item_To_Append : constant String_Type := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            String_Type_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3280;
         end loop;
      end;
         Map_Result_1 := Create_String_Type_Array
           (Items_Count => Natural (String_Type_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := String_Type_Vectors.Get
              (Map_Result_1_Vec,
               I + String_Type_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         String_Type_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10171
Join_Result := Join_Strings (Str, Map_Result_1); 
--# expr-done 10172
         Property_Result := Join_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1219;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1219;
                     Finalizer_Scope_3280;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Fully_Qualified_Name_Impl;
--# end
--# property-start DefiningName.unique_identifying_name_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13886'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Unique_Identifying_Name_Impl
  (Node : Bare_Defining_Name
      ; Suffix : String_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind suffix Suffix
   Property_Result : String_Type;
            procedure Finalizer_Scope_1220 with Inline_Always;
            procedure Finalizer_Scope_3281 with Inline_Always;
            procedure Finalizer_Scope_3282 with Inline_Always;
      Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : String_Type;
Fld_2 : String_Type;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : String_Type;
Scope_Result : String_Type;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : String_Type;
Scope_Result_1 : String_Type;
Match_Result : String_Type;
            procedure Finalizer_Scope_1220 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1220;
            procedure Finalizer_Scope_3281 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3281;
            procedure Finalizer_Scope_3282 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3282;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10176 '<Match at nodes.lkt:13886:64>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13886'
--# expr-start 10173 '<FieldAccess for basic_decl at nodes.lkt:13886:70>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13886'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10173
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
--# expr-start 10174 '<FieldAccess for custom_id_text at nodes.lkt:13887:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13887'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 10174
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3281;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Concrete_Type_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 10175 '<FieldAccess for canonical_fully_qualified_name_impl at nodes.lkt:13888:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13888'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => True, Suffix => Suffix, E_Info => Ent.Info);
--# end
--# expr-done 10175
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3282;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 10176
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1220;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1220;
                     Finalizer_Scope_3281;
                     Finalizer_Scope_3282;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Unique_Identifying_Name_Impl;
--# end
--# property-start DefiningName.canonical_fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13894'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1221 with Inline_Always;
      Str : String_Type;
Fld : String_Type;
            procedure Finalizer_Scope_1221 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
            end Finalizer_Scope_1221;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10178 '<FieldAccess for canonical_fully_qualified_name_impl at nodes.lkt:13895:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13895'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10177 '<String at nodes.lkt:13895:80>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13895'
Str := Create_String (""); 
--# expr-done 10177
--# property-call-start DefiningName.canonical_fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 10178
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1221;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1221;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start DefiningName.unique_identifying_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13906'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Unique_Identifying_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1222 with Inline_Always;
      Str : String_Type;
Fld : String_Type;
            procedure Finalizer_Scope_1222 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
            end Finalizer_Scope_1222;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10180 '<FieldAccess for unique_identifying_name_impl at nodes.lkt:13907:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13907'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10179 '<String at nodes.lkt:13907:50>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13907'
Str := Create_String (""); 
--# expr-done 10179
--# property-call-start DefiningName.unique_identifying_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Unique_Identifying_Name_Impl (Node => Ent.Node, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 10180
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1222;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1222;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Unique_Identifying_Name;
--# end
--# property-start DefiningName.fully_qualified_name_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13912'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name_Array
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1223 with Inline_Always;
      Str : String_Type;
Fld : String_Type_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1223 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1223;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10184 '<Map at nodes.lkt:13913:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13913'
   --# expr-start 10182 '<FieldAccess for fully_qualified_name_impl at nodes.lkt:13913:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13913'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10181 '<String at nodes.lkt:13913:70>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13913'
Str := Create_String (""); 
--# expr-done 10181
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str, E_Info => Ent.Info);
--# end
--# expr-done 10182
      declare
         Map_Result_Vec : Symbol_Type_Vectors.Vector;
      begin
      declare
         Collection : constant String_Type_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 10183 '<to_symbol at nodes.lkt:13913:85>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13913'
Sym := String_To_Symbol (Self, Self.Unit.Context, Item_0); 
--# expr-done 10183
         declare
            Item_To_Append : constant Symbol_Type := Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Symbol_Type_Array
           (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Symbol_Type_Vectors.Get
              (Map_Result_Vec,
               I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Symbol_Type_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10184
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1223;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1223;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name_Array;
--# end
--# property-start DefiningName.fully_qualified_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13917'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Fully_Qualified_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1224 with Inline_Always;
      Str : String_Type;
Str_1 : String_Type;
Fld : String_Type_Array_Access;
Join_Result : String_Type;
            procedure Finalizer_Scope_1224 is
            begin
                     Dec_Ref (Str);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld);
                     Dec_Ref (Join_Result);
            end Finalizer_Scope_1224;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10188 '<Join at nodes.lkt:13918:9>' Join_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13918'
--# expr-start 10185 '<String at nodes.lkt:13918:9>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13918'
Str := Create_String ("."); 
--# expr-done 10185
--# expr-start 10187 '<FieldAccess for fully_qualified_name_impl at nodes.lkt:13918:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13918'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10186 '<String at nodes.lkt:13918:79>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13918'
Str_1 := Create_String (""); 
--# expr-done 10186
--# property-call-start DefiningName.fully_qualified_name_impl
Fld := Libadalang.Implementation.Defining_Name_P_Fully_Qualified_Name_Impl (Node => Ent.Node, Include_Profile => False, Suffix => Str_1, E_Info => Ent.Info);
--# end
--# expr-done 10187
Join_Result := Join_Strings (Str, Fld); 
--# expr-done 10188
         Property_Result := Join_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1224;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1224;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Fully_Qualified_Name;
--# end
--# property-start DefiningName.all_env_els_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13921'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Env_Els_Impl
  (Node : Bare_Defining_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1225 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1225 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1225;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10190 '<FieldAccess for all_env_els_impl at nodes.lkt:13922:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13922'
--# expr-start 10189 '<FieldAccess for name at nodes.lkt:13922:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13922'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 10189
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 10190
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1225;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1225;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Env_Els_Impl;
--# end
--# property-start DefiningName.basic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13926'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 10191 '<FieldAccess for basic_decl_internal at nodes.lkt:13926:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13926'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_internal
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10191
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 10195 '<If at nodes.lkt:13927:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13927'
--# expr-start 10192 '<IsA GenericPackageInternal, GenericSubpInternal, SingleTaskTypeDecl at nodes.lkt:13927:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13927'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range | Ada_Single_Task_Type_Decl_Range; 
--# expr-done 10192
if Is_A then
   --# expr-start 10194 '<Cast to BasicDecl at nodes.lkt:13927:97>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13927'
--# expr-start 10193 '<parent at nodes.lkt:13927:97>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13927'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 10193
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10194
   If_Result := Cast_Result;
else
   If_Result := Var_Expr_0;
end if;
--# expr-done 10195
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Basic_Decl;
--# end
--# property-start DefiningName.basic_decl_internal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13932'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl_Internal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1227 with Inline_Always;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1227 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1227;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Basic_Decl_Internal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10199 '<Cast to BasicDecl at nodes.lkt:13933:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13933'
--# expr-start 10198 '<Find at nodes.lkt:13933:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13933'
   --# expr-start 10196 '<FieldAccess for parents at nodes.lkt:13933:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13933'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 10196
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 10197 '<IsA BasicDecl at nodes.lkt:13933:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13933'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Basic_Decl; 
--# expr-done 10197
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10198
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10199
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_1227;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1227;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Basic_Decl_Internal;
--# end
--# property-start DefiningName.find_refs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13939'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1228 with Inline_Always;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1228 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1228;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10201 '<FieldAccess for find_refs_impl at nodes.lkt:13940:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13940'
--# expr-start 10200 '<FieldAccess for canonical_part at nodes.lkt:13940:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13940'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10200
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_refs_impl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Fld.Node, Root => Root, Skip_Name => Self, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10201
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1228;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1228;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Refs;
--# end
--# property-start DefiningName.find_refs_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13947'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs_Impl
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Skip_Name : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind skip_name Skip_Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1229 with Inline_Always;
            procedure Finalizer_Scope_3286 with Inline_Always;
            procedure Finalizer_Scope_3288 with Inline_Always;
            procedure Finalizer_Scope_3289 with Inline_Always;
            procedure Finalizer_Scope_3290 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Array_Access;
Var_Expr_3 : Internal_Entity_Base_Id;
Var_Expr_4 : Ref_Result_Kind;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Scope_Result : Internal_Ref_Result_Array_Access;
Result_Var : Internal_Ref_Result_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_3 : Ref_Result_Kind;
New_Struct : Internal_Ref_Result;
Array_Lit : Internal_Ref_Result_Array_Access;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Scope_Result_2 : Internal_Ref_Result_Array_Access;
Result_Var_2 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1229 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1229;
            procedure Finalizer_Scope_3286 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_3286;
            procedure Finalizer_Scope_3288 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3288;
            procedure Finalizer_Scope_3289 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3289;
            procedure Finalizer_Scope_3290 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3290;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10202 '<children at nodes.lkt:13949:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13949'
      if Root.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 10202
Var_Expr_0 := Fld;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 10209 '<Mapcat at nodes.lkt:13950:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13950'
   --# expr-start 10207 '<Map at nodes.lkt:13950:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13950'
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
         --# expr-start 10206 '<Not at nodes.lkt:13951:24>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13951'
--# expr-start 10203 '<IsNull at nodes.lkt:13951:29>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13951'
Is_Null := Item_1.Node = null; 
--# expr-done 10203
if Is_Null then
   If_Result := True;
else
   --# expr-start 10205 '<Eq at nodes.lkt:13951:42>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13951'
--# expr-start 10204 '<FieldAccess for node at nodes.lkt:13951:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13951'
Fld_1 := Item_1.Node;
--# expr-done 10204
Cast_Expr := Skip_Name; 
      Cast_Result := Cast_Expr;
Is_Equal := Fld_1 = Cast_Result; 
--# expr-done 10205
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 10206
         if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity := Item_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10207
      declare
         Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 10208 '<FieldAccess for find_refs_impl at nodes.lkt:13952:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13952'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_refs_impl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Ent.Node, Root => Item_2, Skip_Name => Skip_Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10208
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3288;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10209
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3286;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 10210 '<Cast to BaseId at nodes.lkt:13953:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13953'
Cast_Expr_1 := Root; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 10210
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Base_Id then
   --# scope-start
   --# bind id Var_Expr_3
--# expr-start 10211 '<FieldAccess for is_referenced_by at nodes.lkt:13954:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13954'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_referenced_by
Fld_3 := Libadalang.Implementation.Defining_Name_P_Is_Referenced_By (Node => Ent.Node, Id => Var_Expr_3, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10211
Var_Expr_4 := Fld_3;
if Var_Expr_4 /= No_Ref then
   --# scope-start
   --# bind ref_kind Var_Expr_4
--# expr-start 10214 '<If at nodes.lkt:13955:31>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13955'
Is_Equal_2 := Var_Expr_4 = Precise; 
if Is_Equal_2 then
   If_Result_1 := True;
else
Is_Equal_1 := Var_Expr_4 = Imprecise; 
   If_Result_1 := Is_Equal_1;
end if;
if If_Result_1 then
   --# expr-start 10213 '<ArrayLiteral at nodes.lkt:13955:99>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13955'
--# expr-start 10212 '<New RefResult at nodes.lkt:13955:100>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13955'
New_Struct := (Kind => Var_Expr_4, Ref => Var_Expr_3); 
--# expr-done 10212
Array_Lit := Create_Internal_Ref_Result_Array (Internal_Internal_Ref_Result_Array'(1 => New_Struct)); 
--# expr-done 10213
   If_Result_2 := Array_Lit;
else
   If_Result_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10214
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3290;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3289;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1229;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1229;
                     Finalizer_Scope_3286;
                     Finalizer_Scope_3288;
                     Finalizer_Scope_3289;
                     Finalizer_Scope_3290;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Refs_Impl;
--# end
--# property-start DefiningName.is_derivable_equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13964'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Derivable_Equal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Subp_Spec;
Var_Expr_254 : Internal_Entity_Type_Expr;
Sym : Symbol_Type;
Fld : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Derivable_Equal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 10216 '<FieldAccess for name_is at nodes.lkt:13965:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13965'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10215 '<Symbol "=" at nodes.lkt:13965:22>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13965'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 10215
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Sym);
--# end
--# expr-done 10216
if Fld then
--# expr-start 10218 '<FieldAccess for subp_spec_or_null at nodes.lkt:13965:36>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13965'
--# expr-start 10217 '<FieldAccess for basic_decl at nodes.lkt:13965:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13965'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10217
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 10218
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr_0
--# expr-start 10222 '<Eq at nodes.lkt:13966:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13966'
--# expr-start 10219 '<FieldAccess for returns at nodes.lkt:13966:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13966'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10219
Var_Expr_254 := Fld_3;
if Var_Expr_254 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 10220 '<FieldAccess for designated_type_decl at nodes.lkt:13966:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13966'
      if Var_Expr_254.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_254.Node, E_Info => Var_Expr_254.Info);
--# end
--# expr-done 10220
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# expr-start 10221 '<FieldAccess for bool_type at nodes.lkt:13966:59>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13966'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Ent.Node);
--# end
--# expr-done 10221
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 10222
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
else
   If_Result := False;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Derivable_Equal;
--# end
--# property-start DefiningName.is_potential_reference 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13972'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Potential_Reference
  (Node : Bare_Defining_Name
      ; Symbol : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind symbol Symbol
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Potential_Reference,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Symbol);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 10223 '<FieldAccess for name_is at nodes.lkt:13972:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13972'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Symbol);
--# end
--# expr-done 10223
if Fld then
   If_Result_1 := True;
else
--# expr-start 10224 '<FieldAccess for is_derivable_equal at nodes.lkt:13973:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13973'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_derivable_equal
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Derivable_Equal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10224
if Fld_1 then
   --# expr-start 10226 '<Eq at nodes.lkt:13973:39>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13973'
--# expr-start 10225 '<Symbol "/=" at nodes.lkt:13973:49>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13973'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 10225
Is_Equal := Symbol = Sym; 
--# expr-done 10226
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Potential_Reference;
--# end
--# property-start DefiningName.is_referenced_by 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13986'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Referenced_By
  (Node : Bare_Defining_Name
      ; Id : Internal_Entity_Base_Id
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Ref_Result_Kind
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind id Id
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Ref_Result_Kind;
            procedure Finalizer_Scope_3293 with Inline_Always;
            procedure Finalizer_Scope_3296 with Inline_Always;
      Var_Expr_0 : Internal_Refd_Def;
Local_Canon : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_255 : Internal_Entity_Defining_Name;
Var_Expr_256 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
New_Struct : Internal_Refd_Def;
Fld_1 : Internal_Refd_Def;
Fld_2 : Boolean;
If_Result : Internal_Refd_Def;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Bare_Defining_Name;
Scope_Result : Bare_Defining_Name;
Result_Var : Bare_Defining_Name;
Scope_Result_1 : Bare_Defining_Name;
Result_Var_1 : Bare_Defining_Name;
Fld_6 : Ref_Result_Kind;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Defining_Name;
Is_Equal : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Ref_Result_Kind;
Let_Result : Ref_Result_Kind;
Scope_Result_3 : Ref_Result_Kind;
Result_Var_3 : Ref_Result_Kind;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_4 : Ref_Result_Kind;
            procedure Finalizer_Scope_3293 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3293;
            procedure Finalizer_Scope_3296 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_3296;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10248 '<If at nodes.lkt:13987:9>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13987'
--# expr-start 10228 '<FieldAccess for is_potential_reference at nodes.lkt:13987:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13987'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10227 '<FieldAccess for name_symbol at nodes.lkt:13987:40>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13987'
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Id.Node);
--# end
--# expr-done 10227
--# property-call-start DefiningName.is_potential_reference
Fld_13 := Libadalang.Implementation.Defining_Name_P_Is_Potential_Reference (Node => Ent.Node, Symbol => Fld_12, E_Info => Ent.Info);
--# end
--# expr-done 10228
if Fld_13 then
--# expr-start 10233 '<If at nodes.lkt:13988:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13988'
--# expr-start 10229 '<FieldAccess for is_defining at nodes.lkt:13988:16>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13988'
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_2 := Libadalang.Implementation.Name_P_Is_Defining (Node => Id.Node);
--# end
--# expr-done 10229
if Fld_2 then
   --# expr-start 10231 '<New RefdDef at nodes.lkt:13988:38>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13988'
--# expr-start 10230 '<FieldAccess for enclosing_defining_name at nodes.lkt:13989:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13989'
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Id.Node, E_Info => Id.Info);
--# end
--# expr-done 10230
New_Struct := (Def_Name => Fld, Kind => Precise); 
--# expr-done 10231
   If_Result := New_Struct;
else
   --# expr-start 10232 '<FieldAccess for failsafe_referenced_def_name at nodes.lkt:13990:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13990'
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_1 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 10232
   If_Result := Fld_1;
end if;
--# expr-done 10233
Var_Expr_0 := If_Result;
if Var_Expr_0 /= No_Refd_Def then
   --# scope-start
   --# bind def_res Var_Expr_0
--# expr-start 10247 '<Let at nodes.lkt:13993:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13993'
--# scope-start
--# expr-start 10234 '<FieldAccess for def_name at nodes.lkt:13993:29>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13993'
Fld_3 := Var_Expr_0.Def_Name;
--# expr-done 10234
Var_Expr_255 := Fld_3;
if Var_Expr_255 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 10235 '<FieldAccess for canonical_part at nodes.lkt:13993:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13993'
      if Var_Expr_255.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld_4 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Var_Expr_255.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_255.Info);
--# end
--# expr-done 10235
Var_Expr_256 := Fld_4;
if Var_Expr_256 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 10236 '<FieldAccess for node at nodes.lkt:13993:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13993'
Fld_5 := Var_Expr_256.Node;
--# expr-done 10236
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Local_Canon := Result_Var_1; 
--# bind canon Local_Canon
--# expr-start 10246 '<If at nodes.lkt:13995:17>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13995'
--# expr-start 10237 '<Eq at nodes.lkt:13997:21>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:13997'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Local_Canon; 
--# expr-done 10237
if Is_Equal then
   If_Result_2 := True;
else
--# expr-start 10239 '<FieldAccess for base_subp_declarations at nodes.lkt:14002:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14002'
--# expr-start 10238 '<FieldAccess for basic_decl at nodes.lkt:14002:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14002'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10238
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 10239
Var_Expr_1 := Fld_8;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_1
--# expr-start 10243 '<AnyQuantifier at nodes.lkt:14003:36>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14003'
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Var_Expr_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_2
      --# expr-start 10242 '<Eq at nodes.lkt:14003:53>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14003'
--# expr-start 10241 '<FieldAccess for node at nodes.lkt:14003:53>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14003'
--# expr-start 10240 '<FieldAccess for defining_name at nodes.lkt:14003:53>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14003'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_2.Node, E_Info => Item_2.Info);
--# end
--# expr-done 10240
Fld_10 := Fld_9.Node;
--# expr-done 10241
      Cast_Result_1 := Fld_10;
Is_Equal_1 := Cast_Result_1 = Local_Canon; 
--# expr-done 10242
      Quantifier_Result := Is_Equal_1;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 10243
if Quantifier_Result then
   --# expr-start 10244 '<FieldAccess for is_dispatching_call at nodes.lkt:14003:90>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14003'
      if Id.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_11 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 10244
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_3296;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   --# expr-start 10245 '<FieldAccess for kind at nodes.lkt:14005:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14005'
Fld_6 := Var_Expr_0.Kind;
--# expr-done 10245
   If_Result_3 := Fld_6;
else
   If_Result_3 := No_Ref;
end if;
--# expr-done 10246
Let_Result := If_Result_3; 
--# end
--# expr-done 10247
Scope_Result_3 := Let_Result;
   --# end
      Finalizer_Scope_3293;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Ref;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := No_Ref;
end if;
--# expr-done 10248
         Property_Result := If_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3293;
                     Finalizer_Scope_3296;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Referenced_By;
--# end
--# property-start DefiningName.find_all_references 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14017'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_References
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1233 with Inline_Always;
            procedure Finalizer_Scope_3298 with Inline_Always;
            procedure Finalizer_Scope_3299 with Inline_Always;
            procedure Finalizer_Scope_3300 with Inline_Always;
            procedure Finalizer_Scope_3301 with Inline_Always;
            procedure Finalizer_Scope_3302 with Inline_Always;
            procedure Finalizer_Scope_3305 with Inline_Always;
            procedure Finalizer_Scope_3307 with Inline_Always;
      Local_Dn : Internal_Entity_Defining_Name;
Local_Bases : Internal_Entity_Basic_Decl_Array_Access;
Local_All_Units : Internal_Unit_Array_Access;
Local_Refs : Internal_Ref_Result_Array_Access;
Var_Expr_2 : Bare_Ada_Node;
Var_Expr_5 : Internal_Entity_Renaming_Clause;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_3 : Internal_Entity_Basic_Decl;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Unit_Array_Access;
Map_Result : Internal_Unit_Array_Access;
Unique_Array : Internal_Unit_Array_Access;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Fld_7 : Internal_Entity_Base_Id;
Fld_8 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Bare_Defining_Name;
Fld_13 : Bare_Defining_Name;
Cast_Result_2 : Bare_Defining_Name;
Is_Equal : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result_2 : Internal_Ref_Result_Array_Access;
Fld_14 : Internal_Entity_Base_Id;
Fld_15 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Find_Result_1 : Internal_Entity;
Fld_16 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Ref_Result_Array_Access;
Scope_Result_3 : Internal_Ref_Result_Array_Access;
Result_Var_3 : Internal_Ref_Result_Array_Access;
Map_Result_3 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
If_Result_1 : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
Let_Result_1 : Internal_Ref_Result_Array_Access;
Let_Result_2 : Internal_Ref_Result_Array_Access;
Let_Result_3 : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1233 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Local_Bases);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Local_All_Units);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Refs);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1233;
            procedure Finalizer_Scope_3298 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_3298;
            procedure Finalizer_Scope_3299 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3299;
            procedure Finalizer_Scope_3300 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3300;
            procedure Finalizer_Scope_3301 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3301;
            procedure Finalizer_Scope_3302 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3302;
            procedure Finalizer_Scope_3305 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_3305;
            procedure Finalizer_Scope_3307 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_3307;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10287 '<Let at nodes.lkt:14018:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14018'
--# scope-start
--# expr-start 10249 '<FieldAccess for canonical_part at nodes.lkt:14018:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14018'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10249
Local_Dn := Fld; 
--# bind dn Local_Dn
--# expr-start 10286 '<Let at nodes.lkt:14027:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14027'
--# scope-start
--# expr-start 10254 '<bind at nodes.lkt:14028:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14028'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 10251 '<FieldAccess for root_subp_declarations at nodes.lkt:14030:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14030'
--# expr-start 10250 '<FieldAccess for basic_decl at nodes.lkt:14030:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14030'
      if Local_Dn.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Local_Dn.Node, E_Info => Local_Dn.Info);
--# end
--# expr-done 10250
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.root_subp_declarations
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations (Node => Fld_1.Node, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 10251
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3298;
   Result_Var := Scope_Result;
else
   --# expr-start 10253 '<ArrayLiteral at nodes.lkt:14030:58>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14030'
--# expr-start 10252 '<FieldAccess for basic_decl at nodes.lkt:14030:59>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14030'
      if Local_Dn.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Local_Dn.Node, E_Info => Local_Dn.Info);
--# end
--# expr-done 10252
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Fld_3)); 
--# expr-done 10253
   Result_Var := Array_Lit;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10254
Local_Bases := Dyn_Var_Bind_Result; Inc_Ref (Local_Bases);
--# bind bases Local_Bases
--# expr-start 10285 '<Let at nodes.lkt:14032:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14032'
--# scope-start
--# expr-start 10257 '<unique at nodes.lkt:14032:25>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14032'
--# expr-start 10256 '<Mapcat at nodes.lkt:14032:25>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14032'
      declare
         Map_Result_Vec : Internal_Unit_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Local_Bases;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind base Item_0
      --# expr-start 10255 '<FieldAccess for filter_is_imported_by at nodes.lkt:14033:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14033'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Item_0.Node, Units => Units, Transitive => True);
--# expr-done 10255
         for Item_To_Append of
               Fld_4.Items
         loop
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3299;
         end loop;
      end;
         Map_Result := Create_Internal_Unit_Array
           (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Unit_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Unit_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10256
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 10257
Local_All_Units := Unique_Array; Inc_Ref (Local_All_Units);
--# bind all_units Local_All_Units
--# expr-start 10284 '<Let at nodes.lkt:14035:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14035'
--# scope-start
--# expr-start 10261 '<Mapcat at nodes.lkt:14035:20>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14035'
      declare
         Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Unit_Array_Access := Local_All_Units;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_1
--# expr-start 10258 '<FieldAccess for root at nodes.lkt:14036:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14036'
      if Item_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Item_1.Ast_Root;
--# expr-done 10258
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr_2
--# expr-start 10260 '<FieldAccess for find_refs_impl at nodes.lkt:14037:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14037'
      if Local_Dn.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10259 '<as_bare_entity at nodes.lkt:14037:42>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14037'
Is_Null := Var_Expr_2 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_2); 
   If_Result := As_Entity;
end if;
--# expr-done 10259
--# property-call-start DefiningName.find_refs_impl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Find_Refs_Impl (Node => Local_Dn.Node, Root => If_Result, Skip_Name => Self, Imprecise_Fallback => Imprecise_Fallback, E_Info => Local_Dn.Info);
--# end
--# expr-done 10260
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3301;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         for Item_To_Append of
               Result_Var_1.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3300;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10261
Local_Refs := Map_Result_1; Inc_Ref (Local_Refs);
--# bind refs Local_Refs
--# expr-start 10283 '<If at nodes.lkt:14041:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14041'
if Follow_Renamings then
--# expr-start 10282 '<Mapcat at nodes.lkt:14041:41>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14041'
   --# expr-start 10273 '<Map at nodes.lkt:14041:41>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14041'
      declare
         Map_Result_2_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Local_Refs;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_3
--# expr-start 10266 '<Cast to RenamingClause at nodes.lkt:14042:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14042'
--# expr-start 10265 '<Find at nodes.lkt:14042:20>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14042'
   --# expr-start 10263 '<FieldAccess for parents at nodes.lkt:14042:20>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14042'
--# expr-start 10262 '<FieldAccess for ref at nodes.lkt:14042:20>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14042'
Fld_7 := Item_3.Ref;
--# expr-done 10262
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parents (Node => Fld_7.Node, With_Self => True, E_Info => Fld_7.Info);
--# expr-done 10263
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_8;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_4
      --# expr-start 10264 '<IsA RenamingClause at nodes.lkt:14042:48>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14042'
Is_A := Item_4.Node /= null 
and then Item_4.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 10264
      if Is_A then
         Find_Result := Item_4; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10265
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Renaming_Clause_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Renaming_Clause
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_1 := No_Entity_Renaming_Clause;
   end if;
--# expr-done 10266
Var_Expr_5 := Cast_Result_1;
if Var_Expr_5 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind r Var_Expr_5
--# expr-start 10272 '<Eq at nodes.lkt:14043:24>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
--# expr-start 10270 '<FieldAccess for node at nodes.lkt:14043:24>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
--# expr-start 10269 '<FieldAccess for canonical_part at nodes.lkt:14043:24>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
--# expr-start 10268 '<FieldAccess for referenced_defining_name at nodes.lkt:14043:24>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
--# expr-start 10267 '<FieldAccess for renamed_object at nodes.lkt:14043:24>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Var_Expr_5.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_5.Info);
--# expr-done 10267
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_9.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_9.Info);
--# end
--# expr-done 10268
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.canonical_part
Fld_11 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Fld_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_10.Info);
--# end
--# expr-done 10269
Fld_12 := Fld_11.Node;
--# expr-done 10270
      Cast_Result_2 := Fld_12;
--# expr-start 10271 '<FieldAccess for node at nodes.lkt:14043:93>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14043'
Fld_13 := Local_Dn.Node;
--# expr-done 10271
Is_Equal := Cast_Result_2 = Fld_13; 
--# expr-done 10272
Scope_Result_2 := Is_Equal;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
         if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result := Item_3;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_3302;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10273
      declare
         Map_Result_3_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Map_Result_2;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_6
--# expr-start 10279 '<Cast to BasicDecl at nodes.lkt:14046:20>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
--# expr-start 10278 '<parent at nodes.lkt:14046:20>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
--# expr-start 10277 '<Find at nodes.lkt:14046:20>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
   --# expr-start 10275 '<FieldAccess for parents at nodes.lkt:14046:20>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
--# expr-start 10274 '<FieldAccess for ref at nodes.lkt:14046:20>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
Fld_14 := Item_6.Ref;
--# expr-done 10274
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Parents (Node => Fld_14.Node, With_Self => True, E_Info => Fld_14.Info);
--# expr-done 10275
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_15;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_7
      --# expr-start 10276 '<IsA RenamingClause at nodes.lkt:14046:48>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14046'
Is_A_1 := Item_7.Node /= null 
and then Item_7.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 10276
      if Is_A_1 then
         Find_Result_1 := Item_7; 
      end if;
   --# end
               exit when Is_A_1;
         end loop;
      end;
--# expr-done 10277
      if Find_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Parent (Node => Find_Result_1.Node, E_Info => Find_Result_1.Info);
--# expr-done 10278
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10279
Var_Expr_8 := Cast_Result_3;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_8
--# expr-start 10281 '<FieldAccess for find_all_references at nodes.lkt:14056:17>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14056'
--# expr-start 10280 '<FieldAccess for defining_name at nodes.lkt:14056:17>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14056'
      if Var_Expr_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_8.Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 10280
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_all_references
Fld_18 := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Fld_17.Node, Units => Units, Follow_Renamings => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_17.Info);
--# end
--# expr-done 10281
Scope_Result_3 := Fld_18;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3307;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
         for Item_To_Append of
               Result_Var_3.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3305;
         end loop;
      end;
         Map_Result_3 := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 10282
Concat_Result := Concat (Local_Refs, Map_Result_3); 
   If_Result_1 := Concat_Result;
else
   If_Result_1 := Local_Refs;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10283
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10284
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10285
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10286
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10287
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1233;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1233;
                     Finalizer_Scope_3298;
                     Finalizer_Scope_3299;
                     Finalizer_Scope_3300;
                     Finalizer_Scope_3301;
                     Finalizer_Scope_3302;
                     Finalizer_Scope_3305;
                     Finalizer_Scope_3307;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_All_References;
--# end
--# property-start DefiningName.find_matching_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14063'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Matching_Name
  (Node : Bare_Defining_Name
      ; Bd : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind bd Bd
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_3308 with Inline_Always;
      Var_Expr_257 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type;
Fld_3 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_3308 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_3308;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
Var_Expr_257 := Bd;
if Var_Expr_257 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10292 '<Find at nodes.lkt:14063:75>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14063'
   --# expr-start 10288 '<FieldAccess for defining_names at nodes.lkt:14063:75>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14063'
      if Var_Expr_257.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr_257.Node, E_Info => Var_Expr_257.Info);
--# end
--# expr-done 10288
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind di Item_0
      --# expr-start 10291 '<FieldAccess for name_is at nodes.lkt:14064:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14064'
--# expr-start 10289 '<FieldAccess for name at nodes.lkt:14064:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14064'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 10289
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10290 '<FieldAccess for name_symbol at nodes.lkt:14064:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14064'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_0.Node);
--# end
--# expr-done 10290
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Fld_2);
--# end
--# expr-done 10291
      if Fld_3 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 10292
Scope_Result := Find_Result;
   --# end
      Finalizer_Scope_3308;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3308;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Matching_Name;
--# end
--# property-start DefiningName.find_all_calls 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14077'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_Calls
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1235 with Inline_Always;
      Fld : Internal_Ref_Result_Array_Access;
Fld_1 : Internal_Entity_Base_Id;
Fld_2 : Boolean;
Map_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_1235 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1235;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10296 '<Map at nodes.lkt:14078:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14078'
   --# expr-start 10293 '<FieldAccess for find_all_references at nodes.lkt:14078:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14078'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.find_all_references
Fld := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Ent.Node, Units => Units, Follow_Renamings => Follow_Renamings, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10293
      declare
         Map_Result_Vec : Internal_Ref_Result_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_0
         --# expr-start 10295 '<FieldAccess for is_direct_call at nodes.lkt:14078:73>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14078'
--# expr-start 10294 '<FieldAccess for ref at nodes.lkt:14078:73>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14078'
Fld_1 := Item_0.Ref;
--# expr-done 10294
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_direct_call
Fld_2 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 10295
         if Fld_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result := Item_0;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Ref_Result_Array
           (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Ref_Result_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Ref_Result_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10296
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1235;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1235;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_All_Calls;
--# end
--# property-start DefiningName.next_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14083'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Next_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10300 '<FieldAccess for find_matching_name at nodes.lkt:14083:45>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14083'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10299 '<FieldAccess for next_part_for_name at nodes.lkt:14084:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14084'
--# expr-start 10297 '<FieldAccess for basic_decl at nodes.lkt:14084:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10297
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10298 '<FieldAccess for name_symbol at nodes.lkt:14084:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 10298
--# property-call-start '[dispatcher]BasicDecl.next_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10299
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 10300
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Next_Part;
--# end
--# property-start DefiningName.previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14090'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Previous_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10304 '<FieldAccess for find_matching_name at nodes.lkt:14090:49>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14090'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10303 '<FieldAccess for previous_part_for_name at nodes.lkt:14091:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14091'
--# expr-start 10301 '<FieldAccess for basic_decl at nodes.lkt:14091:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14091'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10301
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10302 '<FieldAccess for name_symbol at nodes.lkt:14091:50>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14091'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 10302
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10303
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 10304
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Previous_Part;
--# end
--# property-start DefiningName.canonical_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14097'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10308 '<FieldAccess for find_matching_name at nodes.lkt:14097:50>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14097'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10307 '<FieldAccess for canonical_part_for_name at nodes.lkt:14098:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14098'
--# expr-start 10305 '<FieldAccess for basic_decl at nodes.lkt:14098:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14098'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10305
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10306 '<FieldAccess for name_symbol at nodes.lkt:14098:51>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14098'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 10306
--# property-call-start BasicDecl.canonical_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10307
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 10308
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Canonical_Part;
--# end
--# property-start DefiningName.most_visible_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14106'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Most_Visible_Part
  (Node : Bare_Defining_Name
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10312 '<FieldAccess for find_matching_name at nodes.lkt:14106:53>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14106'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10311 '<FieldAccess for most_visible_part_for_name at nodes.lkt:14107:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14107'
--# expr-start 10309 '<FieldAccess for basic_decl at nodes.lkt:14107:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14107'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10309
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10310 '<FieldAccess for name_symbol at nodes.lkt:14107:54>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14107'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 10310
--# property-call-start BasicDecl.most_visible_part_for_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part_For_Name (Node => Fld.Node, Sym => Fld_1, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 10311
--# property-call-start DefiningName.find_matching_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_2, E_Info => Ent.Info);
--# end
--# expr-done 10312
         Property_Result := Fld_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Most_Visible_Part;
--# end
--# property-start DefiningName.all_previous_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14113'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Previous_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1240 with Inline_Always;
            procedure Finalizer_Scope_3311 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1240 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1240;
            procedure Finalizer_Scope_3311 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_3311;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10313 '<FieldAccess for previous_part at nodes.lkt:14113:61>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14113'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.previous_part
Fld := Libadalang.Implementation.Defining_Name_P_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10313
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind pp Var_Expr_0
--# expr-start 10317 '<If at nodes.lkt:14114:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14114'
--# expr-start 10314 '<Eq at nodes.lkt:14114:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14114'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 10314
if Is_Equiv then
   If_Result := No_Internal_Entity_Defining_Name_Array_Type;
else
   --# expr-start 10315 '<FieldAccess for all_previous_parts at nodes.lkt:14114:75>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14114'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_previous_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Previous_Parts (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10315
--# expr-start 10316 '<ArrayLiteral at nodes.lkt:14114:101>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14114'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Var_Expr_0)); 
--# expr-done 10316
Concat_Result := Concat (Fld_1, Array_Lit); 
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 10317
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3311;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1240;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1240;
                     Finalizer_Scope_3311;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Previous_Parts;
--# end
--# property-start DefiningName.all_next_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14120'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Next_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1241 with Inline_Always;
            procedure Finalizer_Scope_3312 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr : Internal_Entity_Defining_Name;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1241 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1241;
            procedure Finalizer_Scope_3312 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_3312;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10318 '<FieldAccess for next_part at nodes.lkt:14120:57>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14120'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.next_part
Fld := Libadalang.Implementation.Defining_Name_P_Next_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10318
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind np Var_Expr_0
--# expr-start 10322 '<If at nodes.lkt:14121:17>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14121'
--# expr-start 10319 '<Eq at nodes.lkt:14121:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14121'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 10319
if Is_Equiv then
   If_Result := No_Internal_Entity_Defining_Name_Array_Type;
else
   --# expr-start 10320 '<ArrayLiteral at nodes.lkt:14121:75>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14121'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Var_Expr_0)); 
--# expr-done 10320
--# expr-start 10321 '<FieldAccess for all_next_parts at nodes.lkt:14121:82>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14121'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_next_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Next_Parts (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10321
Concat_Result := Concat (Array_Lit, Fld_1); 
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 10322
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3312;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1241;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1241;
                     Finalizer_Scope_3312;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Next_Parts;
--# end
--# property-start DefiningName.all_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14128'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Parts
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1242 with Inline_Always;
      Local_Prevs : Internal_Entity_Defining_Name_Array_Access;
Local_Nexts : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Concat_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1242 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Prevs);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Nexts);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1242;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10327 '<Let at nodes.lkt:14129:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14129'
--# scope-start
--# expr-start 10323 '<FieldAccess for all_previous_parts at nodes.lkt:14129:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14129'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_previous_parts
Fld := Libadalang.Implementation.Defining_Name_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10323
Local_Prevs := Fld; Inc_Ref (Local_Prevs);
--# bind prevs Local_Prevs
--# expr-start 10326 '<Let at nodes.lkt:14130:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14130'
--# scope-start
--# expr-start 10324 '<FieldAccess for all_next_parts at nodes.lkt:14130:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14130'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_next_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Next_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10324
Local_Nexts := Fld_1; Inc_Ref (Local_Nexts);
--# bind nexts Local_Nexts
--# expr-start 10325 '<ArrayLiteral at nodes.lkt:14132:17>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14132'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Ent)); 
--# expr-done 10325
Concat_Result := Concat (Local_Prevs, Array_Lit); 
Concat_Result_1 := Concat (Concat_Result, Local_Nexts); 
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10326
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10327
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1242;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1242;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_All_Parts;
--# end
--# property-start DefiningName.get_aspect_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14138'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Aspect_Impl
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Inherited : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind inherited Inherited
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr_0 : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Aspect_Assoc;
Var_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Var_Expr_3 : Internal_Entity_At_Clause;
Var_Expr : Internal_Aspect;
Var_Expr_4 : Internal_Aspect;
Var_Expr_5 : Internal_Aspect;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Result_Var : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Aspect_Assoc;
Fld_4 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Aspect_Assoc;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Scope_Result_3 : Internal_Aspect;
Fld_5 : Internal_Entity_Attribute_Def_Clause;
Fld_6 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Cast_Result_1 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
Scope_Result_4 : Internal_Aspect;
Result_Var_3 : Internal_Aspect;
Result_Var_4 : Internal_Aspect;
Scope_Result_5 : Internal_Aspect;
Fld_7 : Internal_Entity_At_Clause;
Fld_8 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_At_Clause;
Cast_Result_2 : Internal_Entity;
New_Struct_2 : Internal_Aspect;
Scope_Result_6 : Internal_Aspect;
Result_Var_5 : Internal_Aspect;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Aspect;
Result_Var_6 : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10328 '<FieldAccess for get_pragma at nodes.lkt:14140:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14140'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_pragma
Fld := Libadalang.Implementation.Defining_Name_P_Get_Pragma (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 10328
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 10329 '<FieldAccess for as_aspect at nodes.lkt:14140:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14140'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.as_aspect
Fld_1 := Libadalang.Implementation.Pragma_Node_P_As_Aspect (Node => Var_Expr_0.Node, Inherited => Inherited, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10329
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Aspect;
end if;
Var_Expr_5 := Result_Var;
if Var_Expr_5 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_1 := Var_Expr_5;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 10331 '<FieldAccess for get_aspect_assoc at nodes.lkt:14140:73>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14140'
--# expr-start 10330 '<FieldAccess for basic_decl_internal at nodes.lkt:14140:73>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14140'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_internal
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10330
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_aspect_assoc
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Assoc (Node => Fld_2.Node, Name => Name, E_Info => Fld_2.Info);
--# end
--# expr-done 10331
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_1
--# expr-start 10333 '<New Aspect at nodes.lkt:14141:25>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14141'
Cast_Expr := Var_Expr_1; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 10332 '<FieldAccess for expr at nodes.lkt:14142:49>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14142'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Var_Expr_1.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_1.Info);
--# expr-done 10332
New_Struct := (Exists => True, Inherited => Inherited, Node => Cast_Result, Value => Fld_4); 
--# expr-done 10333
Scope_Result_2 := New_Struct;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Aspect;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Var_Expr_4 := Result_Var_2;
if Var_Expr_4 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_3 := Var_Expr_4;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 10334 '<FieldAccess for get_representation_clause at nodes.lkt:14145:15>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14145'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_representation_clause
Fld_5 := Libadalang.Implementation.Defining_Name_P_Get_Representation_Clause (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10334
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind rc Var_Expr_2
--# expr-start 10336 '<New Aspect at nodes.lkt:14146:21>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14146'
Cast_Expr_1 := Var_Expr_2; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 10335 '<FieldAccess for expr at nodes.lkt:14147:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14147'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 10335
New_Struct_1 := (Exists => True, Inherited => Inherited, Node => Cast_Result_1, Value => Fld_6); 
--# expr-done 10336
Scope_Result_4 := New_Struct_1;
   --# end
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Aspect;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr := Result_Var_4;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_5 := Var_Expr;
   --# end
   Result_Var_6 := Scope_Result_5;
else
   --# expr-start 10342 '<If at nodes.lkt:14151:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14151'
--# expr-start 10338 '<Eq at nodes.lkt:14151:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14151'
--# expr-start 10337 '<Symbol Address at nodes.lkt:14151:20>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14151'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Address); 
--# expr-done 10337
Is_Equal := Name = Sym; 
--# expr-done 10338
if Is_Equal then
--# expr-start 10339 '<FieldAccess for get_at_clause at nodes.lkt:14151:36>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_at_clause
Fld_7 := Libadalang.Implementation.Defining_Name_P_Get_At_Clause (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10339
Var_Expr_3 := Fld_7;
if Var_Expr_3 /= No_Entity_At_Clause then
   --# scope-start
   --# bind atc Var_Expr_3
--# expr-start 10341 '<New Aspect at nodes.lkt:14152:22>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14152'
Cast_Expr_2 := Var_Expr_3; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 10340 '<FieldAccess for expr at nodes.lkt:14153:46>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14153'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr_3.Node.At_Clause_F_Expr, Info => Var_Expr_3.Info);
--# expr-done 10340
New_Struct_2 := (Exists => True, Inherited => Inherited, Node => Cast_Result_2, Value => Fld_8); 
--# expr-done 10341
Scope_Result_6 := New_Struct_2;
   --# end
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Aspect;
end if;
   If_Result := Result_Var_5;
else
   If_Result := No_Aspect;
end if;
--# expr-done 10342
   Result_Var_6 := If_Result;
end if;
         Property_Result := Result_Var_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Aspect_Impl;
--# end
--# property-start DefiningName.get_aspect_on_parts 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14168'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Aspect_On_Parts
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Inherited : Boolean
      ; Previous_Parts_Only : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind inherited Inherited
   --# bind previous_parts_only Previous_Parts_Only
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
            procedure Finalizer_Scope_1244 with Inline_Always;
      Local_Parts_To_Check : Internal_Entity_Defining_Name_Array_Access;
Local_Self_Aspects : Internal_Aspect_Array_Access;
Var_Expr_2 : Internal_Entity_Aspect_Assoc;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Aspect;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Array_Lit_1 : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Concat_Result : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
If_Result : Internal_Entity_Defining_Name_Array_Access;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Aspect_Assoc;
Fld_4 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Aspect_Assoc;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
Scope_Result : Internal_Aspect;
Result_Var : Internal_Aspect;
Map_Result : Internal_Aspect_Array_Access;
Fld_5 : Internal_Aspect;
Map_Result_1 : Internal_Aspect_Array_Access;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_2 : Boolean;
Is_Equal_3 : Boolean;
If_Result_3 : Boolean;
Is_Equal_4 : Boolean;
If_Result_4 : Boolean;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Is_Equal_6 : Boolean;
If_Result_6 : Boolean;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
Is_Equal_8 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Internal_Aspect_Array_Access;
Fld_6 : Boolean;
Find_Result : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_7 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Subtype_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
If_Result_10 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Aspect;
Is_Null : Boolean;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Internal_Aspect;
Let_Result : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Let_Result_1 : Internal_Aspect;
Let_Result_2 : Internal_Aspect;
            procedure Finalizer_Scope_1244 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Local_Parts_To_Check);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Local_Self_Aspects);
            end Finalizer_Scope_1244;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10383 '<Let at nodes.lkt:14169:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14169'
--# scope-start
--# expr-start 10350 '<If at nodes.lkt:14173:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14173'
--# expr-start 10344 '<Eq at nodes.lkt:14173:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14173'
--# expr-start 10343 '<Symbol SPARK_Mode at nodes.lkt:14173:20>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14173'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 10343
Is_Equal := Name = Sym; 
--# expr-done 10344
if Is_Equal then
   --# expr-start 10345 '<ArrayLiteral at nodes.lkt:14173:39>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14173'
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Ent)); 
--# expr-done 10345
   If_Result_1 := Array_Lit;
else
   --# expr-start 10349 '<If at nodes.lkt:14173:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14173'
if Previous_Parts_Only then
   --# expr-start 10346 '<ArrayLiteral at nodes.lkt:14174:39>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14174'
Array_Lit_1 := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Ent)); 
--# expr-done 10346
--# expr-start 10347 '<FieldAccess for all_previous_parts at nodes.lkt:14174:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14174'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_previous_parts
Fld := Libadalang.Implementation.Defining_Name_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10347
Concat_Result := Concat (Array_Lit_1, Fld); 
   If_Result := Concat_Result;
else
   --# expr-start 10348 '<FieldAccess for all_parts at nodes.lkt:14175:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14175'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_parts
Fld_1 := Libadalang.Implementation.Defining_Name_P_All_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10348
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 10349
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10350
Local_Parts_To_Check := If_Result_1; Inc_Ref (Local_Parts_To_Check);
--# bind parts_to_check Local_Parts_To_Check
--# expr-start 10382 '<Let at nodes.lkt:14176:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14176'
--# scope-start
--# expr-start 10366 '<If at nodes.lkt:14180:9>' If_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
--# expr-start 10351 '<Symbol Implicit_Dereference at nodes.lkt:14180:20>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Implicit_Dereference); 
--# expr-done 10351
Is_Equal_8 := Name = Sym_1; 
if Is_Equal_8 then
   If_Result_8 := True;
else
--# expr-start 10352 '<Symbol Constant_Indexing at nodes.lkt:14180:46>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Constant_Indexing); 
--# expr-done 10352
Is_Equal_7 := Name = Sym_2; 
if Is_Equal_7 then
   If_Result_7 := True;
else
--# expr-start 10353 '<Symbol Variable_Indexing at nodes.lkt:14180:69>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Variable_Indexing); 
--# expr-done 10353
Is_Equal_6 := Name = Sym_3; 
if Is_Equal_6 then
   If_Result_6 := True;
else
--# expr-start 10354 '<Symbol Iterable at nodes.lkt:14180:92>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterable); 
--# expr-done 10354
Is_Equal_5 := Name = Sym_4; 
if Is_Equal_5 then
   If_Result_5 := True;
else
--# expr-start 10355 '<Symbol Iterator_Element at nodes.lkt:14180:106>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Iterator_Element); 
--# expr-done 10355
Is_Equal_4 := Name = Sym_5; 
if Is_Equal_4 then
   If_Result_4 := True;
else
--# expr-start 10356 '<Symbol Integer_Literal at nodes.lkt:14180:128>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Literal); 
--# expr-done 10356
Is_Equal_3 := Name = Sym_6; 
if Is_Equal_3 then
   If_Result_3 := True;
else
--# expr-start 10357 '<Symbol Real_Literal at nodes.lkt:14180:149>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Real_Literal); 
--# expr-done 10357
Is_Equal_2 := Name = Sym_7; 
if Is_Equal_2 then
   If_Result_2 := True;
else
--# expr-start 10358 '<Symbol String_Literal at nodes.lkt:14180:167>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String_Literal); 
--# expr-done 10358
Is_Equal_1 := Name = Sym_8; 
   If_Result_2 := Is_Equal_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
if If_Result_8 then
   --# expr-start 10363 '<Map at nodes.lkt:14180:190>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14180'
      declare
         Map_Result_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Parts_To_Check;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
--# expr-start 10360 '<FieldAccess for get_aspect_assoc at nodes.lkt:14181:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14181'
--# expr-start 10359 '<FieldAccess for basic_decl_internal at nodes.lkt:14181:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14181'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_internal
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl_Internal (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 10359
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.get_aspect_assoc
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Assoc (Node => Fld_2.Node, Name => Name, E_Info => Fld_2.Info);
--# end
--# expr-done 10360
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_2
--# expr-start 10362 '<New Aspect at nodes.lkt:14182:25>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14182'
Cast_Expr := Var_Expr_2; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 10361 '<FieldAccess for expr at nodes.lkt:14183:49>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14183'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 10361
New_Struct := (Exists => True, Inherited => Inherited, Node => Cast_Result, Value => Fld_4); 
--# expr-done 10362
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Aspect;
end if;
         declare
            Item_To_Append : constant Internal_Aspect := Result_Var;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10363
   If_Result_9 := Map_Result;
else
   --# expr-start 10365 '<Map at nodes.lkt:14187:14>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14187'
      declare
         Map_Result_1_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Local_Parts_To_Check;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 10364 '<FieldAccess for get_aspect_impl at nodes.lkt:14188:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14188'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect_impl
Fld_5 := Libadalang.Implementation.Defining_Name_P_Get_Aspect_Impl (Node => Item_0.Node, Name => Name, Inherited => Inherited, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_0.Info);
--# end
--# expr-done 10364
         declare
            Item_To_Append : constant Internal_Aspect := Fld_5;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10365
   If_Result_9 := Map_Result_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 10366
Local_Self_Aspects := If_Result_9; Inc_Ref (Local_Self_Aspects);
--# bind self_aspects Local_Self_Aspects
--# expr-start 10368 '<Find at nodes.lkt:14191:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14191'
      Find_Result := No_Aspect;
      declare
         Collection : constant Internal_Aspect_Array_Access := Local_Self_Aspects;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_3
      --# expr-start 10367 '<FieldAccess for exists at nodes.lkt:14191:34>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14191'
Fld_6 := Item_3.Exists;
--# expr-done 10367
      if Fld_6 then
         Find_Result := Item_3; 
      end if;
   --# end
               exit when Fld_6;
         end loop;
      end;
--# expr-done 10368
Var_Expr := Find_Result;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 10370 '<Cast to BaseTypeDecl at nodes.lkt:14191:48>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14191'
--# expr-start 10369 '<FieldAccess for basic_decl at nodes.lkt:14191:48>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14191'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10369
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10370
Var_Expr_4 := Cast_Result_1;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bd Var_Expr_4
--# expr-start 10381 '<Let at nodes.lkt:14195:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
--# scope-start
--# expr-start 10375 '<If at nodes.lkt:14195:27>' If_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
--# expr-start 10371 '<IsA BaseSubtypeDecl at nodes.lkt:14195:30>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
Is_A := Var_Expr_4.Node /= null 
and then Var_Expr_4.Node.Kind in Ada_Base_Subtype_Decl; 
--# expr-done 10371
if Is_A then
   --# expr-start 10373 '<FieldAccess for get_type at nodes.lkt:14195:57>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
--# expr-start 10372 '<Cast to BaseSubtypeDecl at nodes.lkt:14195:57>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
Cast_Expr_1 := Var_Expr_4; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 10372
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Cast_Result_2.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 10373
   If_Result_10 := Fld_8;
else
   --# expr-start 10374 '<FieldAccess for base_type at nodes.lkt:14195:96>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14195'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Var_Expr_4.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 10374
   If_Result_10 := Fld_9;
end if;
--# expr-done 10375
Local_Typ := If_Result_10; 
--# bind typ Local_Typ
--# expr-start 10380 '<If at nodes.lkt:14197:17>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14197'
--# expr-start 10376 '<IsNull at nodes.lkt:14197:20>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14197'
Is_Null := Local_Typ.Node = null; 
--# expr-done 10376
if Is_Null then
   If_Result_11 := True;
else
   --# expr-start 10377 '<Eq at nodes.lkt:14197:35>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14197'
Cast_Expr_2 := Local_Typ; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Cast_Expr_3 := Var_Expr_4; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 10377
   If_Result_11 := Is_Equiv;
end if;
if If_Result_11 then
   If_Result_12 := No_Aspect;
else
   --# expr-start 10379 '<FieldAccess for get_aspect_on_parts at nodes.lkt:14197:68>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14197'
--# expr-start 10378 '<FieldAccess for name at nodes.lkt:14197:68>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14197'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Defining_Name (Node => Local_Typ.Node.Base_Type_Decl_F_Name, Info => Local_Typ.Info);
--# expr-done 10378
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect_on_parts
Fld_11 := Libadalang.Implementation.Defining_Name_P_Get_Aspect_On_Parts (Node => Fld_10.Node, Name => Name, Inherited => True, Previous_Parts_Only => Previous_Parts_Only, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_10.Info);
--# end
--# expr-done 10379
   If_Result_12 := Fld_11;
end if;
--# expr-done 10380
Let_Result := If_Result_12; 
--# end
--# expr-done 10381
Scope_Result_2 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Aspect;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Let_Result_1 := Result_Var_2; 
--# end
--# expr-done 10382
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 10383
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_1244;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1244;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Aspect_On_Parts;
--# end
--# property-start DefiningName.get_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14225'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Aspect
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Previous_Parts_Only : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind previous_parts_only Previous_Parts_Only
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Fld : Internal_Aspect;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Get_Aspect,
            Items    => new Mmz_Key_Array (1 ..  5))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Name);
               Mmz_K.Items (3) := (Kind => Mmz_Boolean,
                                      As_Boolean => Previous_Parts_Only);
               Mmz_K.Items (4) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (5) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Aspect;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10384 '<FieldAccess for get_aspect_on_parts at nodes.lkt:14226:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14226'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect_on_parts
Fld := Libadalang.Implementation.Defining_Name_P_Get_Aspect_On_Parts (Node => Ent.Node, Name => Name, Inherited => False, Previous_Parts_Only => Previous_Parts_Only, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10384
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Aspect,
                        As_Internal_Aspect => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Get_Aspect;
--# end
--# property-start DefiningName.has_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14240'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Has_Aspect
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Previous_Parts_Only : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind previous_parts_only Previous_Parts_Only
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_3326 with Inline_Always;
      Local_A : Internal_Aspect;
Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Aspect;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_5 : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_3326 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Big_Int);
            end Finalizer_Scope_3326;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10397 '<Let at nodes.lkt:14241:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14241'
--# scope-start
--# expr-start 10385 '<FieldAccess for get_aspect at nodes.lkt:14241:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14241'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_aspect
Fld := Libadalang.Implementation.Defining_Name_P_Get_Aspect (Node => Ent.Node, Name => Name, Previous_Parts_Only => Previous_Parts_Only, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10385
Local_A := Fld; 
--# bind a Local_A
--# expr-start 10386 '<FieldAccess for exists at nodes.lkt:14243:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14243'
Fld_1 := Local_A.Exists;
--# expr-done 10386
if Fld_1 then
   --# expr-start 10396 '<If at nodes.lkt:14244:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14244'
--# expr-start 10387 '<FieldAccess for is_contract_aspect at nodes.lkt:14244:16>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14244'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_contract_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Name);
--# end
--# expr-done 10387
if Fld_6 then
   If_Result_1 := True;
else
--# expr-start 10388 '<FieldAccess for value at nodes.lkt:14248:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14248'
Fld_2 := Local_A.Value;
--# expr-done 10388
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind value Var_Expr_0
--# expr-start 10392 '<Not at nodes.lkt:14253:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14253'
--# expr-start 10391 '<Eq at nodes.lkt:14253:21>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14253'
--# expr-start 10389 '<FieldAccess for expression_type at nodes.lkt:14253:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14253'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10389
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 10390 '<FieldAccess for bool_type at nodes.lkt:14253:48>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14253'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 10390
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 10391
Not_Val := not (Is_Equiv); 
--# expr-done 10392
if Not_Val then
   If_Result := True;
else
   --# expr-start 10395 '<Eq at nodes.lkt:14254:22>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14254'
--# expr-start 10393 '<FieldAccess for eval_as_int at nodes.lkt:14254:22>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14254'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_5 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10393
--# expr-start 10394 '<BigInteger 1 at nodes.lkt:14254:45>' Big_Int 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14254'
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 10394
Is_Equal := Equivalent (Fld_5, Big_Int); 
--# expr-done 10395
   If_Result := Is_Equal;
end if;
Scope_Result := If_Result;
   --# end
      Finalizer_Scope_3326;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result_1 := Result_Var;
end if;
--# expr-done 10396
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 10397
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3326;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Has_Aspect;
--# end
--# property-start DefiningName.is_valid_pragma_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14262'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Valid_Pragma_For_Name
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Decl : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind decl Decl
   Property_Result : Boolean;
            procedure Finalizer_Scope_3327 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Pragma_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Pragma_Node;
Node_Comp : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
            procedure Finalizer_Scope_3327 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_3327;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10398 '<Cast to Pragma at nodes.lkt:14262:79>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14262'
Cast_Expr := Decl; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 10398
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_0
--# expr-start 10400 '<FieldAccess for name_is at nodes.lkt:14265:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14265'
--# expr-start 10399 '<FieldAccess for id at nodes.lkt:14265:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14265'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Var_Expr_0.Node.Pragma_Node_F_Id, Info => Var_Expr_0.Info);
--# expr-done 10399
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Name);
--# end
--# expr-done 10400
if Fld_1 then
   --# expr-start 10405 '<Not at nodes.lkt:14268:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14268'
--# expr-start 10404 '<IsNull at nodes.lkt:14268:17>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14268'
--# expr-start 10403 '<Find at nodes.lkt:14268:17>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14268'
   --# expr-start 10401 '<FieldAccess for associated_entities at nodes.lkt:14268:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14268'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities
Fld_2 := Libadalang.Implementation.Pragma_Node_P_Associated_Entities (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10401
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_1
      --# expr-start 10402 '<Eq at nodes.lkt:14268:53>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14268'
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Item_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 10402
      if Is_Equiv then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Is_Equiv;
         end loop;
      end;
--# expr-done 10403
Is_Null := Find_Result.Node = null; 
--# expr-done 10404
Not_Val := not (Is_Null); 
--# expr-done 10405
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 10407 '<OrderingTest '"'"'lt'"'"' at nodes.lkt:14271:13>' Node_Comp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14271'
--# expr-start 10406 '<FieldAccess for node at nodes.lkt:14271:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14271'
Fld_3 := Var_Expr_0.Node;
--# expr-done 10406
Node_Comp := Compare (Self, Self, Fld_3, Less_Than); 
--# expr-done 10407
   If_Result_1 := Node_Comp;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_3327;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3327;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Valid_Pragma_For_Name;
--# end
--# property-start DefiningName.find_valid_pragma_for_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14280'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Valid_Pragma_For_Name
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Regions : Internal_Entity_Ada_Node_List_Array_Access
      ; Region_Index : Integer
         := 0
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind regions Regions
   --# bind region_index Region_Index
   Property_Result : Internal_Entity_Pragma_Node;
      Var_Expr_0 : Internal_Entity_Ada_Node_List;
Var_Expr : Internal_Entity_Pragma_Node;
Get_Result : Internal_Entity_Ada_Node_List;
Saved : Internal_Entity_Ada_Node_List;
Fld : Bare_Ada_Node_List;
Fld_1 : Internal_Entity_Info;
Item_1 : Internal_Entity;
Bare_Item_1 : Bare_Ada_Node;
Fld_2 : Boolean;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Scope_Result : Internal_Entity_Pragma_Node;
Arith_Result : Integer;
Fld_3 : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Scope_Result_1 : Internal_Entity_Pragma_Node;
Result_Var_1 : Internal_Entity_Pragma_Node;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10408 '<at at nodes.lkt:14280:133>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14280'
Get_Result := Get (Self, Regions, Region_Index, True); 
--# expr-done 10408
Var_Expr_0 := Get_Result;
if Var_Expr_0 /= No_Entity_Ada_Node_List then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 10411 '<Cast to Pragma at nodes.lkt:14281:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14281'
--# expr-start 10410 '<Find at nodes.lkt:14281:16>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14281'
Saved := Var_Expr_0; 
Fld := Saved.Node;
      Find_Result := No_Entity;
      if Fld = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_1 := Saved.Info;
As_Entity := (Info => Fld_1, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind d Item_1
      --# expr-start 10409 '<FieldAccess for is_valid_pragma_for_name at nodes.lkt:14282:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14282'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_valid_pragma_for_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_1, E_Info => Ent.Info);
--# end
--# expr-done 10409
      if Fld_2 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_2;
         end loop;
      end;
      end if;
--# expr-done 10410
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 10411
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 10413 '<FieldAccess for find_valid_pragma_for_name at nodes.lkt:14283:26>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14283'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10412 '<Arithmetic + at nodes.lkt:14283:73>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14283'
Arith_Result := (Region_Index + 1); 
--# expr-done 10412
--# property-call-start DefiningName.find_valid_pragma_for_name
Fld_3 := Libadalang.Implementation.Defining_Name_P_Find_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Regions => Regions, Region_Index => Arith_Result, E_Info => Ent.Info);
--# end
--# expr-done 10413
   Result_Var := Fld_3;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Pragma_Node;
end if;
         Property_Result := Result_Var_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Find_Valid_Pragma_For_Name;
--# end
--# property-start DefiningName.get_pragma 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14292'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Pragma
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1249 with Inline_Always;
      Local_Bd : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Enum_Literal_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Pragma_Node_List;
Var_Expr : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Pragma_Node_List;
Saved : Internal_Entity_Pragma_Node_List;
Fld_5 : Bare_Pragma_Node_List;
Fld_6 : Internal_Entity_Info;
Item_1 : Internal_Entity_Pragma_Node;
Bare_Item_1 : Bare_Pragma_Node;
Fld_7 : Internal_Entity_Identifier;
Fld_8 : Boolean;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Find_Result : Internal_Entity_Pragma_Node;
Scope_Result_2 : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Scope_Result_3 : Internal_Entity_Pragma_Node;
Fld_9 : Internal_Entity_Ada_Node_List_Array_Access;
Fld_10 : Internal_Entity_Pragma_Node;
Result_Var_1 : Internal_Entity_Pragma_Node;
Let_Result_2 : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_1249 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_1249;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10426 '<Let at nodes.lkt:14293:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14293'
--# scope-start
--# expr-start 10419 '<Match at nodes.lkt:14293:18>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14293'
--# expr-start 10414 '<FieldAccess for basic_decl at nodes.lkt:14293:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14293'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10414
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind eld Match_0
--# expr-start 10418 '<Cast to BasicDecl at nodes.lkt:14296:42>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14296'
--# expr-start 10417 '<parent at nodes.lkt:14296:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14296'
--# expr-start 10416 '<parent at nodes.lkt:14296:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14296'
--# expr-start 10415 '<parent at nodes.lkt:14296:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14296'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 10415
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 10416
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 10417
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10418
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind o Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 10419
Local_Bd := Match_Result; 
--# bind bd Local_Bd
--# expr-start 10420 '<FieldAccess for library_item_pragmas at nodes.lkt:14301:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14301'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10420
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Pragma_Node_List then
   --# scope-start
   --# bind plist Var_Expr_0
--# expr-start 10423 '<Find at nodes.lkt:14302:24>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14302'
Saved := Var_Expr_0; 
Fld_5 := Saved.Node;
      Find_Result := No_Entity_Pragma_Node;
      if Fld_5 = null then
      Find_Result := No_Entity_Pragma_Node;
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_5;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
Fld_6 := Saved.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind p Item_1
      --# expr-start 10422 '<FieldAccess for name_is at nodes.lkt:14304:24>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14304'
--# expr-start 10421 '<FieldAccess for id at nodes.lkt:14304:24>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14304'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Identifier (Node => Item_1.Node.Pragma_Node_F_Id, Info => Item_1.Info);
--# expr-done 10421
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_8 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_7.Node, Sym => Name);
--# end
--# expr-done 10422
      if Fld_8 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_8;
         end loop;
      end;
      end if;
--# expr-done 10423
Scope_Result_2 := Find_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_3 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   --# expr-start 10425 '<FieldAccess for find_valid_pragma_for_name at nodes.lkt:14307:9>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14307'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10424 '<FieldAccess for pragma_regions at nodes.lkt:14307:47>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14307'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.pragma_regions
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Pragma_Regions (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10424
--# property-call-start DefiningName.find_valid_pragma_for_name
Fld_10 := Libadalang.Implementation.Defining_Name_P_Find_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Regions => Fld_9, E_Info => Ent.Info);
--# end
--# expr-done 10425
   Result_Var_1 := Fld_10;
end if;
Let_Result_2 := Result_Var_1; 
--# end
--# expr-done 10426
         Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_1249;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1249;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Pragma;
--# end
--# property-start DefiningName.get_representation_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Representation_Clause
  (Node : Bare_Defining_Name
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
      Var_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Local_Attr : Internal_Entity_Attribute_Ref;
Var_Expr_258 : Bare_Declarative_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Attribute_Def_Clause;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Attribute_Def_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10440 '<Cast to AttributeDefClause at nodes.lkt:14314:79>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
--# expr-start 10427 '<FieldAccess for declarative_scope at nodes.lkt:14314:79>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 10427
Var_Expr_258 := Fld;
if Var_Expr_258 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 10439 '<Find at nodes.lkt:14314:79>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
   --# expr-start 10429 '<as_entity at nodes.lkt:14314:79>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
--# expr-start 10428 '<FieldAccess for decls at nodes.lkt:14314:79>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14314'
      if Var_Expr_258 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_258.Declarative_Part_F_Decls;
--# expr-done 10428
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 10429
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result_2 := As_Entity_1;
end if;
                  Item_0 := If_Result_2; 
   --# scope-start
               --# bind d Item_0
--# expr-start 10430 '<Cast to AttributeDefClause at nodes.lkt:14315:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14315'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 10430
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 10438 '<Let at nodes.lkt:14317:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14317'
--# scope-start
--# expr-start 10432 '<Cast to AttributeRef at nodes.lkt:14317:28>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14317'
--# expr-start 10431 '<FieldAccess for attribute_expr at nodes.lkt:14317:28>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14317'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Var_Expr_1.Info);
--# expr-done 10431
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10432
Local_Attr := Cast_Result_1; 
--# bind attr Local_Attr
--# expr-start 10434 '<FieldAccess for name_is at nodes.lkt:14319:17>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14319'
--# expr-start 10433 '<FieldAccess for attribute at nodes.lkt:14319:17>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14319'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Local_Attr.Node.Attribute_Ref_F_Attribute, Info => Local_Attr.Info);
--# expr-done 10433
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Name);
--# end
--# expr-done 10434
if Fld_6 then
   --# expr-start 10437 '<Eq at nodes.lkt:14319:50>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14319'
--# expr-start 10436 '<FieldAccess for referenced_defining_name at nodes.lkt:14319:50>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14319'
--# expr-start 10435 '<FieldAccess for prefix at nodes.lkt:14319:50>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14319'
      if Local_Attr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Local_Attr.Node.Attribute_Ref_F_Prefix, Info => Local_Attr.Info);
--# expr-done 10435
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 10436
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 10437
   If_Result_1 := Is_Equiv;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 10438
Scope_Result := Let_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
      end if;
--# expr-done 10439
Scope_Result_1 := Find_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_4 := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 10440
         Property_Result := Cast_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Representation_Clause;
--# end
--# property-start DefiningName.get_at_clause 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_At_Clause
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_At_Clause
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_At_Clause;
      Var_Expr_1 : Internal_Entity_At_Clause;
Var_Expr_259 : Bare_Declarative_Part;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_At_Clause;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Internal_Entity_Defining_Name;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Defining_Name;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Find_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_At_Clause;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10449 '<Cast to AtClause at nodes.lkt:14327:45>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
--# expr-start 10441 '<FieldAccess for declarative_scope at nodes.lkt:14327:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 10441
Var_Expr_259 := Fld;
if Var_Expr_259 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 10448 '<Find at nodes.lkt:14327:45>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
   --# expr-start 10443 '<as_entity at nodes.lkt:14327:45>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
--# expr-start 10442 '<FieldAccess for decls at nodes.lkt:14327:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14327'
      if Var_Expr_259 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Var_Expr_259.Declarative_Part_F_Decls;
--# expr-done 10442
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 10443
Fld_2 := If_Result.Node;
      Find_Result := No_Entity;
      if Fld_2 = null then
      Find_Result := No_Entity;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_1 := Bare_Item_0 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind d Item_0
--# expr-start 10444 '<Cast to AtClause at nodes.lkt:14328:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14328'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_At_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_At_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_At_Clause;
   end if;
--# expr-done 10444
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_At_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 10447 '<Eq at nodes.lkt:14329:20>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14329'
--# expr-start 10446 '<FieldAccess for referenced_defining_name at nodes.lkt:14329:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14329'
--# expr-start 10445 '<FieldAccess for name at nodes.lkt:14329:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14329'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Var_Expr_1.Node.At_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 10445
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 10446
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 10447
Scope_Result := Is_Equiv;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
      end if;
--# expr-done 10448
Scope_Result_1 := Find_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_At_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_At_Clause
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_3 := No_Entity_At_Clause;
   end if;
--# expr-done 10449
         Property_Result := Cast_Result_3;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_At_Clause;
--# end
--# property-start DefiningName.get_annotations_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14336'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Annotations_Impl
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_1252 with Inline_Always;
            procedure Finalizer_Scope_3345 with Inline_Always;
            procedure Finalizer_Scope_3347 with Inline_Always;
      Local_Bd : Internal_Entity_Basic_Decl;
Local_Pragmas : Internal_Aspect_Array_Access;
Match_0 : Internal_Entity_Enum_Literal_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Local_Aspects : Internal_Aspect_Array_Access;
Var_Expr_260 : Internal_Entity_Aspect_Spec;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Ada_Node_List_Array_Access;
Saved : Internal_Entity_Ada_Node_List;
Fld_5 : Bare_Ada_Node_List;
Fld_6 : Internal_Entity_Info;
Item_1 : Internal_Entity;
Bare_Item_1 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result_3 : Internal_Entity_Pragma_Node;
Fld_7 : Internal_Entity_Base_Assoc_List;
Fld_8 : Bare_Base_Assoc_List;
Fld_9 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Fld_10 : Internal_Entity_Expr;
New_Struct : Internal_Aspect;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Sym : Symbol_Type;
Fld_11 : Boolean;
Map_Result : Internal_Aspect_Array_Access;
Map_Result_1 : Internal_Aspect_Array_Access;
Fld_12 : Internal_Entity_Aspect_Spec;
Fld_13 : Internal_Entity_Aspect_Assoc_List;
Fld_14 : Bare_Aspect_Assoc_List;
Fld_15 : Internal_Entity_Info;
Item_2 : Internal_Entity_Aspect_Assoc;
Bare_Item_2 : Bare_Aspect_Assoc;
Fld_16 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Base_Aggregate;
Fld_17 : Internal_Entity_Assoc_List;
Fld_18 : Bare_Assoc_List;
Fld_19 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_20 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Aspect_Assoc;
Cast_Result_5 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
As_Entity_3 : Internal_Entity_Aspect_Assoc;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Aspect_Assoc;
Fld_21 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_22 : Boolean;
Map_Result_2 : Internal_Aspect_Array_Access;
Scope_Result_2 : Internal_Aspect_Array_Access;
Result_Var : Internal_Aspect_Array_Access;
Concat_Result : Internal_Aspect_Array_Access;
Let_Result_2 : Internal_Aspect_Array_Access;
Let_Result_3 : Internal_Aspect_Array_Access;
Let_Result_4 : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_1252 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Local_Pragmas);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Aspects);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1252;
            procedure Finalizer_Scope_3345 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3345;
            procedure Finalizer_Scope_3347 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_3347;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10478 '<Let at nodes.lkt:14337:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14337'
--# scope-start
--# expr-start 10450 '<FieldAccess for basic_decl_internal at nodes.lkt:14337:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14337'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl_internal
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10450
Local_Bd := Fld; 
--# bind bd Local_Bd
--# expr-start 10477 '<Let at nodes.lkt:14339:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14339'
--# scope-start
--# expr-start 10464 '<Mapcat at nodes.lkt:14339:23>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14339'
   --# expr-start 10456 '<FieldAccess for pragma_regions at nodes.lkt:14339:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14339'
--# expr-start 10455 '<Match at nodes.lkt:14339:23>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14339'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Bd; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Enum_Literal_Decl .. Ada_Synthetic_Char_Enum_Lit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind eld Match_0
--# expr-start 10454 '<Cast to BasicDecl at nodes.lkt:14342:42>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14342'
--# expr-start 10453 '<parent at nodes.lkt:14342:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14342'
--# expr-start 10452 '<parent at nodes.lkt:14342:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14342'
--# expr-start 10451 '<parent at nodes.lkt:14342:42>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14342'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 10451
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 10452
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 10453
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10454
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind o Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 10455
      if Match_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.pragma_regions
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Pragma_Regions (Node => Match_Result.Node, E_Info => Match_Result.Info);
--# end
--# expr-done 10456
      declare
         Map_Result_1_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Ada_Node_List_Array_Access := Fld_4;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_0
      --# expr-start 10463 '<Map at nodes.lkt:14345:20>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14345'
Saved := Item_0; 
Fld_5 := Saved.Node;
      declare
         Map_Result_Vec : Internal_Aspect_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Internal_Aspect_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_5;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_6 := Saved.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_1); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_1 := If_Result_1; 
   --# scope-start
               --# bind d Item_1
         --# expr-start 10458 '<FieldAccess for is_valid_pragma_for_name at nodes.lkt:14348:27>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14348'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10457 '<Symbol Annotate at nodes.lkt:14348:57>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14348'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 10457
--# property-call-start DefiningName.is_valid_pragma_for_name
Fld_11 := Libadalang.Implementation.Defining_Name_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Sym, Decl => Item_1, E_Info => Ent.Info);
--# end
--# expr-done 10458
         if Fld_11 then
      --# expr-start 10462 '<New Aspect at nodes.lkt:14346:24>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14346'
--# expr-start 10461 '<FieldAccess for assoc_expr at nodes.lkt:14347:48>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14347'
--# expr-start 10460 '<FieldAccess for args at nodes.lkt:14347:48>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14347'
--# expr-start 10459 '<Cast to Pragma at nodes.lkt:14347:48>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14347'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Pragma_Node;
   end if;
--# expr-done 10459
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Base_Assoc_List (Node => Cast_Result_3.Node.Pragma_Node_F_Args, Info => Cast_Result_3.Info);
--# expr-done 10460
Fld_8 := Fld_7.Node;
Get_Result := Get (Self, Fld_8, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10461
New_Struct := (Exists => True, Inherited => False, Node => Item_1, Value => Fld_10); 
--# expr-done 10462
         declare
            Item_To_Append : constant Internal_Aspect := New_Struct;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10463
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Aspect_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3345;
         end loop;
      end;
         Map_Result_1 := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10464
Local_Pragmas := Map_Result_1; Inc_Ref (Local_Pragmas);
--# bind pragmas Local_Pragmas
--# expr-start 10476 '<Let at nodes.lkt:14352:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14352'
--# scope-start
--# expr-start 10465 '<FieldAccess for get_aspect_spec at nodes.lkt:14352:23>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14352'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_spec'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Spec (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10465
Var_Expr_260 := Fld_12;
if Var_Expr_260 /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 10475 '<Map at nodes.lkt:14352:23>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14352'
   --# expr-start 10466 '<FieldAccess for aspect_assocs at nodes.lkt:14352:23>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14352'
      if Var_Expr_260.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr_260.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr_260.Info);
--# expr-done 10466
Fld_14 := Fld_13.Node;
      declare
         Map_Result_2_Vec : Internal_Aspect_Vectors.Vector;
      begin
      if Fld_14 = null then
      Map_Result_2 := Create_Internal_Aspect_Array (0);
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_14;
      begin
         for Untyped_Item_2 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_2 := Untyped_Item_2; 
Is_Null_3 := Bare_Item_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Aspect_Assoc;
else
Fld_15 := Fld_13.Info;
As_Entity_3 := (Info => Fld_15, Node => Bare_Item_2); 
   If_Result_3 := As_Entity_3;
end if;
                  Item_2 := If_Result_3; 
   --# scope-start
               --# bind asp Item_2
         --# expr-start 10469 '<FieldAccess for name_is at nodes.lkt:14355:25>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14355'
--# expr-start 10467 '<FieldAccess for id at nodes.lkt:14355:25>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14355'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Name (Node => Item_2.Node.Aspect_Assoc_F_Id, Info => Item_2.Info);
--# expr-done 10467
      if Fld_21.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10468 '<Symbol Annotate at nodes.lkt:14355:40>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14355'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 10468
--# property-call-start Name.name_is
Fld_22 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_21.Node, Sym => Sym_1);
--# end
--# expr-done 10469
         if Fld_22 then
      --# expr-start 10474 '<New Aspect at nodes.lkt:14353:22>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14353'
Cast_Expr_1 := Item_2; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 10473 '<FieldAccess for expr at nodes.lkt:14354:46>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14354'
--# expr-start 10472 '<FieldAccess for assocs at nodes.lkt:14354:46>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14354'
--# expr-start 10471 '<Cast to BaseAggregate at nodes.lkt:14354:46>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14354'
--# expr-start 10470 '<FieldAccess for expr at nodes.lkt:14354:46>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14354'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Item_2.Node.Aspect_Assoc_F_Expr, Info => Item_2.Info);
--# expr-done 10470
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result_4 := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_4 := No_Entity_Base_Aggregate;
   end if;
--# expr-done 10471
      if Cast_Result_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_4.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_4.Info);
--# expr-done 10472
Fld_18 := Fld_17.Node;
Get_Result_1 := Get (Self, Fld_18, 0, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_19 := Fld_17.Info;
As_Entity_2 := (Info => Fld_19, Node => Get_Result_1); 
   If_Result_2 := As_Entity_2;
end if;
      if If_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 10473
New_Struct_1 := (Exists => True, Inherited => False, Node => Cast_Result_5, Value => Fld_20); 
--# expr-done 10474
         declare
            Item_To_Append : constant Internal_Aspect := New_Struct_1;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10475
Scope_Result_2 := Map_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3347;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Internal_Aspect_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Aspects := Result_Var; Inc_Ref (Local_Aspects);
--# bind aspects Local_Aspects
Concat_Result := Concat (Local_Pragmas, Local_Aspects); 
Let_Result_2 := Concat_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10476
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10477
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 10478
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1252;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1252;
                     Finalizer_Scope_3345;
                     Finalizer_Scope_3347;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Annotations_Impl;
--# end
--# property-start DefiningName.get_annotations 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14374'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Get_Annotations
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect_Array_Access
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_1253 with Inline_Always;
            procedure Finalizer_Scope_3349 with Inline_Always;
            procedure Finalizer_Scope_3350 with Inline_Always;
      Local_Self_Annotations : Internal_Aspect_Array_Access;
Local_Inherited_Annotations : Internal_Aspect_Array_Access;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Local_Typ : Internal_Entity_Base_Type_Decl;
Local_Config_Annotations : Internal_Aspect_Array_Access;
Fld : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Aspect_Array_Access;
Map_Result : Internal_Aspect_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Subtype_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Internal_Aspect_Array_Access;
Fld_7 : Boolean;
Fld_8 : Internal_Entity;
Fld_9 : Internal_Entity_Expr;
New_Struct : Internal_Aspect;
Map_Result_1 : Internal_Aspect_Array_Access;
Is_Null : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Aspect_Array_Access;
Let_Result : Internal_Aspect_Array_Access;
Scope_Result : Internal_Aspect_Array_Access;
Result_Var : Internal_Aspect_Array_Access;
Fld_10 : Bare_Compilation_Unit;
Sym : Symbol_Type;
Fld_11 : Internal_Entity_Pragma_Node_Array_Access;
Fld_12 : Internal_Entity_Base_Assoc_List;
Fld_13 : Bare_Base_Assoc_List;
Fld_14 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_15 : Internal_Entity_Expr;
Cast_Expr_3 : Internal_Entity_Pragma_Node;
Cast_Result_4 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
Map_Result_2 : Internal_Aspect_Array_Access;
Concat_Result : Internal_Aspect_Array_Access;
Concat_Result_1 : Internal_Aspect_Array_Access;
Unique_Array : Internal_Aspect_Array_Access;
Let_Result_1 : Internal_Aspect_Array_Access;
Let_Result_2 : Internal_Aspect_Array_Access;
Let_Result_3 : Internal_Aspect_Array_Access;
            procedure Finalizer_Scope_1253 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Self_Annotations);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Inherited_Annotations);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Local_Config_Annotations);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1253;
            procedure Finalizer_Scope_3349 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3349;
            procedure Finalizer_Scope_3350 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3350;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 10510 '<Let at nodes.lkt:14375:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14375'
--# scope-start
--# expr-start 10481 '<Mapcat at nodes.lkt:14375:32>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14375'
   --# expr-start 10479 '<FieldAccess for all_parts at nodes.lkt:14375:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14375'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.all_parts
Fld := Libadalang.Implementation.Defining_Name_P_All_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 10479
      declare
         Map_Result_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 10480 '<FieldAccess for get_annotations_impl at nodes.lkt:14375:63>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14375'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_annotations_impl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Get_Annotations_Impl (Node => Item_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_0.Info);
--# end
--# expr-done 10480
         for Item_To_Append of
               Fld_1.Items
         loop
            Internal_Aspect_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3349;
         end loop;
      end;
         Map_Result := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10481
Local_Self_Annotations := Map_Result; Inc_Ref (Local_Self_Annotations);
--# bind self_annotations Local_Self_Annotations
--# expr-start 10509 '<Let at nodes.lkt:14376:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14376'
--# scope-start
--# expr-start 10483 '<Cast to BaseTypeDecl at nodes.lkt:14376:37>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14376'
--# expr-start 10482 '<FieldAccess for basic_decl at nodes.lkt:14376:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14376'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10482
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10483
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 10499 '<Let at nodes.lkt:14378:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
--# scope-start
--# expr-start 10488 '<If at nodes.lkt:14378:27>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
--# expr-start 10484 '<IsA BaseSubtypeDecl at nodes.lkt:14378:30>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Base_Subtype_Decl; 
--# expr-done 10484
if Is_A then
   --# expr-start 10486 '<FieldAccess for get_type at nodes.lkt:14378:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
--# expr-start 10485 '<Cast to BaseSubtypeDecl at nodes.lkt:14378:57>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 10485
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Cast_Result_1.Node, Origin => No_Bare_Ada_Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10486
   If_Result := Fld_3;
else
   --# expr-start 10487 '<FieldAccess for base_type at nodes.lkt:14378:96>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14378'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Var_Expr_1.Node, Origin => No_Bare_Ada_Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10487
   If_Result := Fld_4;
end if;
--# expr-done 10488
Local_Typ := If_Result; 
--# bind typ Local_Typ
--# expr-start 10498 '<If at nodes.lkt:14380:17>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
--# expr-start 10489 '<IsNull at nodes.lkt:14380:20>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
Is_Null := Local_Typ.Node = null; 
--# expr-done 10489
if Is_Null then
   If_Result_1 := True;
else
   --# expr-start 10490 '<Eq at nodes.lkt:14380:35>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
Cast_Expr_1 := Local_Typ; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 10490
   If_Result_1 := Is_Equiv;
end if;
if If_Result_1 then
   If_Result_2 := No_Internal_Aspect_Array_Type;
else
   --# expr-start 10497 '<Map at nodes.lkt:14380:75>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
   --# expr-start 10492 '<FieldAccess for get_annotations at nodes.lkt:14380:75>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
--# expr-start 10491 '<FieldAccess for name at nodes.lkt:14380:75>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14380'
      if Local_Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Defining_Name (Node => Local_Typ.Node.Base_Type_Decl_F_Name, Info => Local_Typ.Info);
--# expr-done 10491
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.get_annotations
Fld_6 := Libadalang.Implementation.Defining_Name_P_Get_Annotations (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 10492
      declare
         Map_Result_1_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Aspect_Array_Access := Fld_6;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_2
      --# expr-start 10496 '<New Aspect at nodes.lkt:14381:28>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14381'
--# expr-start 10493 '<FieldAccess for exists at nodes.lkt:14382:32>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14382'
Fld_7 := Item_2.Exists;
--# expr-done 10493
--# expr-start 10494 '<FieldAccess for node at nodes.lkt:14382:47>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14382'
Fld_8 := Item_2.Node;
--# expr-done 10494
--# expr-start 10495 '<FieldAccess for value at nodes.lkt:14382:61>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14382'
Fld_9 := Item_2.Value;
--# expr-done 10495
New_Struct := (Exists => Fld_7, Inherited => True, Node => Fld_8, Value => Fld_9); 
--# expr-done 10496
         declare
            Item_To_Append : constant Internal_Aspect := New_Struct;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10497
   If_Result_2 := Map_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10498
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 10499
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3350;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Aspect_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Inherited_Annotations := Result_Var; Inc_Ref (Local_Inherited_Annotations);
--# bind inherited_annotations Local_Inherited_Annotations
--# expr-start 10508 '<Let at nodes.lkt:14387:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14387'
--# scope-start
--# expr-start 10506 '<Map at nodes.lkt:14387:34>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14387'
   --# expr-start 10502 '<FieldAccess for config_pragmas at nodes.lkt:14387:34>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14387'
--# expr-start 10500 '<FieldAccess for enclosing_compilation_unit at nodes.lkt:14387:34>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14387'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_10 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Ent.Node);
--# end
--# expr-done 10500
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10501 '<Symbol Annotate at nodes.lkt:14387:83>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14387'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 10501
--# property-call-start CompilationUnit.config_pragmas
Fld_11 := Libadalang.Implementation.Compilation_Unit_P_Config_Pragmas (Node => Fld_10, Name => Sym);
--# end
--# expr-done 10502
      declare
         Map_Result_2_Vec : Internal_Aspect_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Pragma_Node_Array_Access := Fld_11;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
      --# expr-start 10505 '<New Aspect at nodes.lkt:14388:20>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14388'
Cast_Expr_3 := Item_3; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 10504 '<FieldAccess for assoc_expr at nodes.lkt:14389:44>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14389'
--# expr-start 10503 '<FieldAccess for args at nodes.lkt:14389:44>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14389'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Base_Assoc_List (Node => Item_3.Node.Pragma_Node_F_Args, Info => Item_3.Info);
--# expr-done 10503
Fld_13 := Fld_12.Node;
Get_Result := Get (Self, Fld_13, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_14 := Fld_12.Info;
As_Entity := (Info => Fld_14, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 10504
New_Struct_1 := (Exists => True, Inherited => False, Node => Cast_Result_4, Value => Fld_15); 
--# expr-done 10505
         declare
            Item_To_Append : constant Internal_Aspect := New_Struct_1;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Aspect_Array
           (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Aspect_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Aspect_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 10506
Local_Config_Annotations := Map_Result_2; Inc_Ref (Local_Config_Annotations);
--# bind config_annotations Local_Config_Annotations
--# expr-start 10507 '<unique at nodes.lkt:14398:9>' Unique_Array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14398'
Concat_Result := Concat (Local_Self_Annotations, Local_Inherited_Annotations); 
Concat_Result_1 := Concat (Concat_Result, Local_Config_Annotations); 
Unique_Array := Make_Unique (Concat_Result_1); 
--# expr-done 10507
Let_Result_1 := Unique_Array; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10508
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10509
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10510
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1253;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1253;
                     Finalizer_Scope_3349;
                     Finalizer_Scope_3350;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Get_Annotations;
--# end
--# property-start DefiningName.is_imported 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14406'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Imported
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 10512 '<FieldAccess for has_aspect at nodes.lkt:14407:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14407'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10511 '<Symbol Import at nodes.lkt:14407:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14407'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 10511
--# property-call-start DefiningName.has_aspect
Fld := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 10512
if Fld then
   If_Result := True;
else
   --# expr-start 10514 '<FieldAccess for has_aspect at nodes.lkt:14407:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14407'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10513 '<Symbol Interface at nodes.lkt:14407:55>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14407'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 10513
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 10514
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Is_Imported;
--# end
--# property-start DefiningName.is_ghost_code 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14413'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Ghost_Code
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Local_Bd : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Generic_Instantiation;
Match_0 : Internal_Entity_Subp_Renaming_Decl;
Match_1 : Internal_Entity_Package_Renaming_Decl;
Match_2 : Internal_Entity_Generic_Renaming_Decl;
Match_3 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Name;
Var_Expr_261 : Internal_Entity_Basic_Decl;
Var_Expr_262 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Instantiation;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Renaming_Clause;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Name;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Renaming_Decl;
Let_Result : Internal_Entity_Name;
Scope_Result_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Package_Renaming_Decl;
Let_Result_1 : Internal_Entity_Name;
Scope_Result_3 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_2 : Internal_Entity_Name;
Scope_Result_4 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Name;
Scope_Result_5 : Internal_Entity_Name;
Match_Result : Internal_Entity_Name;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_2 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_3 : Boolean;
If_Result_2 : Boolean;
Let_Result_4 : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Ghost_Code,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Boolean;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10531 '<Let at nodes.lkt:14414:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14414'
--# scope-start
--# expr-start 10515 '<FieldAccess for basic_decl at nodes.lkt:14414:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14414'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10515
Local_Bd := Fld; 
--# bind bd Local_Bd
--# expr-start 10517 '<FieldAccess for has_aspect at nodes.lkt:14416:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14416'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10516 '<Symbol Ghost at nodes.lkt:14416:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14416'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ghost); 
--# expr-done 10516
--# property-call-start DefiningName.has_aspect
Fld_1 := Libadalang.Implementation.Defining_Name_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 10517
if Fld_1 then
   If_Result := True;
else
--# expr-start 10518 '<FieldAccess for parent_basic_decl at nodes.lkt:14416:38>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14416'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Local_Bd.Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10518
Var_Expr_261 := Fld_2;
if Var_Expr_261 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10519 '<FieldAccess for is_ghost_code at nodes.lkt:14416:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14416'
      if Var_Expr_261.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Var_Expr_261.Node, E_Info => Var_Expr_261.Info);
--# end
--# expr-done 10519
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 10520 '<Cast to GenericInstantiation at nodes.lkt:14418:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14418'
Cast_Expr := Local_Bd; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Create_Internal_Entity_Generic_Instantiation
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Instantiation;
   end if;
--# expr-done 10520
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind gi Var_Expr_0
--# expr-start 10522 '<FieldAccess for is_ghost_code at nodes.lkt:14419:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14419'
--# expr-start 10521 '<FieldAccess for designated_bare_generic_decl at nodes.lkt:14419:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14419'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericInstantiation.designated_bare_generic_decl
Fld_4 := Libadalang.Implementation.Generic_Instantiation_P_Designated_Bare_Generic_Decl (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10521
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 10522
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 10528 '<Match at nodes.lkt:14423:13>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14423'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Bd; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind sr Match_0
--# expr-start 10524 '<FieldAccess for renamed_object at nodes.lkt:14424:46>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14424'
--# expr-start 10523 '<FieldAccess for renames at nodes.lkt:14424:46>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14424'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Renaming_Clause (Node => Match_0.Node.Subp_Renaming_Decl_F_Renames, Info => Match_0.Info);
--# expr-done 10523
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Fld_6.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_6.Info);
--# expr-done 10524
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind pr Match_1
--# expr-start 10526 '<FieldAccess for renamed_object at nodes.lkt:14425:49>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14425'
--# expr-start 10525 '<FieldAccess for renames at nodes.lkt:14425:49>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14425'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Renaming_Clause (Node => Match_1.Node.Package_Renaming_Decl_F_Renames, Info => Match_1.Info);
--# expr-done 10525
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Fld_8.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_8.Info);
--# expr-done 10526
Let_Result_1 := Fld_9; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind gr Match_2
--# expr-start 10527 '<FieldAccess for renaming_name at nodes.lkt:14426:49>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14426'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld_10 := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 10527
Let_Result_2 := Fld_10; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_No_Type_Object_Renaming_Decl | Ada_Single_Protected_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Name; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 10528
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind c Var_Expr_1
--# expr-start 10529 '<FieldAccess for referenced_defining_name at nodes.lkt:14429:24>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14429'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_defining_name
Fld_11 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Var_Expr_1.Node, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10529
Var_Expr_262 := Fld_11;
if Var_Expr_262 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 10530 '<FieldAccess for is_ghost_code at nodes.lkt:14429:24>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14429'
      if Var_Expr_262.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_12 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr_262.Node, E_Info => Var_Expr_262.Info);
--# end
--# expr-done 10530
Scope_Result_6 := Fld_12;
   --# end
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := False;
end if;
Scope_Result_7 := Result_Var_2;
   --# end
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := False;
end if;
   If_Result_2 := Result_Var_3;
end if;
Let_Result_4 := If_Result_2; 
--# end
--# expr-done 10531
         Property_Result := Let_Result_4;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Boolean,
                        As_Boolean => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end Defining_Name_P_Is_Ghost_Code;
--# end
--# property-start DefiningName.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14435'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Equation
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
     Ent : Internal_Entity_Defining_Name :=
       Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1256 with Inline_Always;
            procedure Finalizer_Scope_3361 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1256 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1256;
            procedure Finalizer_Scope_3361 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3361;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10533 '<Cast to DottedName at nodes.lkt:14439:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14439'
--# expr-start 10532 '<FieldAccess for name at nodes.lkt:14439:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14439'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 10532
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 10533
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 10537 '<bind at nodes.lkt:14444:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14444'
--# expr-start 10534 '<FieldAccess for std_env at nodes.lkt:14444:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14444'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Ent.Node);
--# end
--# expr-done 10534
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 10536 '<FieldAccess for xref_equation at nodes.lkt:14446:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14446'
--# expr-start 10535 '<FieldAccess for prefix at nodes.lkt:14446:17>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14446'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 10535
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 10536
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10537
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3361;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14447:28") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1256;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1256;
                     Finalizer_Scope_3361;
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Xref_Equation;
--# end
--# property-start DefiningName.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14452'
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Entry_Point
  (Node : Bare_Defining_Name
  )
   return Boolean
is
   Self : Bare_Defining_Name  := Bare_Defining_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Name;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10539 '<IsA DottedName at nodes.lkt:14452:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14452'
--# expr-start 10538 '<FieldAccess for name at nodes.lkt:14452:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14452'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 10538
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 10539
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Defining_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synthetic_Defining_Name
   --
      procedure Initialize_Fields_For_Synthetic_Defining_Name
        (Self : Bare_Synthetic_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Defining_Name
              (Self, Defining_Name_F_Name);
      end Initialize_Fields_For_Synthetic_Defining_Name;
--# property-start SyntheticDefiningName.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14460'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Defining_Name_P_Name_Symbol
  (Node : Bare_Synthetic_Defining_Name
  )
   return Symbol_Type
is
   Self : Bare_Synthetic_Defining_Name  := Bare_Synthetic_Defining_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Bare_Name;
Fld_1 : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10541 '<FieldAccess for name_symbol at nodes.lkt:14460:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14460'
--# expr-start 10540 '<FieldAccess for name at nodes.lkt:14460:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14460'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 10540
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 10541
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Defining_Name_P_Name_Symbol;
--# end
--# property-start SyntheticDefiningName.as_symbol_array 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14462'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Defining_Name_P_As_Symbol_Array
  (Node : Bare_Synthetic_Defining_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Synthetic_Defining_Name  := Bare_Synthetic_Defining_Name (Node);
      --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1259 with Inline_Always;
      Fld : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1259 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1259;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10543 '<ArrayLiteral at nodes.lkt:14462:44>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14462'
--# expr-start 10542 '<FieldAccess for name_symbol at nodes.lkt:14462:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14462'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 10542
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Fld)); 
--# expr-done 10543
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1259;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1259;
            raise;
      end;
   return Property_Result;
end Synthetic_Defining_Name_P_As_Symbol_Array;
--# end
   --
   --  Primitives for Bare_Discrete_Subtype_Name
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Name
        (Self : Bare_Discrete_Subtype_Name
         ; Discrete_Subtype_Name_F_Subtype : Bare_Discrete_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Discrete_Subtype_Name_F_Subtype := Discrete_Subtype_Name_F_Subtype;
      end Initialize_Fields_For_Discrete_Subtype_Name;
   function Discrete_Subtype_Name_F_Subtype
     (Node : Bare_Discrete_Subtype_Name) return Bare_Discrete_Subtype_Indication
   is
   begin
         return Node.Discrete_Subtype_Name_F_Subtype;
   end;
   --
   --  Primitives for Bare_Dotted_Name
   --
      procedure Initialize_Fields_For_Dotted_Name
        (Self : Bare_Dotted_Name
         ; Dotted_Name_F_Prefix : Bare_Name
         ; Dotted_Name_F_Suffix : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Dotted_Name_F_Prefix := Dotted_Name_F_Prefix;
            Self.Dotted_Name_F_Suffix := Dotted_Name_F_Suffix;
      end Initialize_Fields_For_Dotted_Name;
   function Dotted_Name_F_Prefix
     (Node : Bare_Dotted_Name) return Bare_Name
   is
   begin
         return Node.Dotted_Name_F_Prefix;
   end;
   function Dotted_Name_F_Suffix
     (Node : Bare_Dotted_Name) return Bare_Base_Id
   is
   begin
         return Node.Dotted_Name_F_Suffix;
   end;
--# property-start DottedName.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14475'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Ref_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10545 '<FieldAccess for ref_var at nodes.lkt:14475:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14475'
--# expr-start 10544 '<FieldAccess for suffix at nodes.lkt:14475:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14475'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 10544
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10545
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Ref_Var;
--# end
--# property-start DottedName.subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14477'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10547 '<FieldAccess for subp_spec_var at nodes.lkt:14477:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14477'
--# expr-start 10546 '<FieldAccess for suffix at nodes.lkt:14477:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14477'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 10546
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld);
--# end
--# expr-done 10547
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Subp_Spec_Var;
--# end
--# property-start DottedName.defines_subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14479'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start DottedName.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14481'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Has_Context_Free_Type
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Base_Id;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10550 '<Not at nodes.lkt:14481:41>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14481'
--# expr-start 10549 '<IsA CharLiteral at nodes.lkt:14481:45>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14481'
--# expr-start 10548 '<FieldAccess for suffix at nodes.lkt:14481:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14481'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 10548
Is_A := Fld /= null 
and then Fld.Kind in Ada_Char_Literal_Range; 
--# expr-done 10549
Not_Val := not (Is_A); 
--# expr-done 10550
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Has_Context_Free_Type;
--# end
--# property-start DottedName.complete_items 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14484'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete_Items
  (Node : Bare_Dotted_Name
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1264 with Inline_Always;
            procedure Finalizer_Scope_3362 with Inline_Always;
      Local_Complete_Env : Lexical_Env := Empty_Env;
Local_Visible_Env : Lexical_Env := Empty_Env;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Bound_No_Visibility : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Bound_No_Visibility_1 : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Info;
Fld_8 : Internal_Metadata;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Info;
Fld_11 : Internal_Metadata;
Fld_12 : Boolean;
Fld_13 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_14 : Symbol_Type;
Fld_15 : Internal_Entity_Array_Access;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_16 : Integer;
New_Struct : Internal_Completion_Item;
Is_A : Boolean;
Not_Val : Boolean;
Fld_17 : Analysis_Unit_Kind;
Is_Equal : Boolean;
Fld_18 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Map_Result : Internal_Completion_Item_Array_Access;
Let_Result : Internal_Completion_Item_Array_Access;
Let_Result_1 : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Completion_Item_Array_Access;
Dyn_Var_Bind_Result_3 : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1264 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Local_Complete_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Local_Visible_Env);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
                     Dec_Ref (Dyn_Var_Bind_Result_3);
            end Finalizer_Scope_1264;
            procedure Finalizer_Scope_3362 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_3362;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10585 '<bind at nodes.lkt:14485:9>' Dyn_Var_Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14485'
--# expr-start 10551 '<FieldAccess for origin_node at nodes.lkt:14485:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14485'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10551
Bound_Origin := Fld; 
--# expr-start 10584 '<bind at nodes.lkt:14486:9>' Dyn_Var_Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14486'
--# expr-start 10552 '<FieldAccess for node_env at nodes.lkt:14486:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14486'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 10552
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 10583 '<Let at nodes.lkt:14487:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14487'
--# scope-start
--# expr-start 10555 '<bind at nodes.lkt:14488:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14488'
Bound_No_Visibility := True; 
--# expr-start 10554 '<FieldAccess for designated_env at nodes.lkt:14489:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14489'
--# expr-start 10553 '<FieldAccess for prefix at nodes.lkt:14489:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14489'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10553
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => Bound_No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 10554
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10555
Local_Complete_Env := Dyn_Var_Bind_Result; Inc_Ref (Local_Complete_Env);
--# bind complete_env Local_Complete_Env
--# expr-start 10582 '<Let at nodes.lkt:14491:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14491'
--# scope-start
--# expr-start 10558 '<bind at nodes.lkt:14492:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14492'
Bound_No_Visibility_1 := False; 
--# expr-start 10557 '<FieldAccess for designated_env at nodes.lkt:14493:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14493'
--# expr-start 10556 '<FieldAccess for prefix at nodes.lkt:14493:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14493'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10556
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_4.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => Bound_No_Visibility_1, E_Info => Fld_4.Info);
--# end
--# expr-done 10557
Dyn_Var_Bind_Result_1 := Fld_5; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 10558
Local_Visible_Env := Dyn_Var_Bind_Result_1; Inc_Ref (Local_Visible_Env);
--# bind visible_env Local_Visible_Env
--# expr-start 10581 '<Map at nodes.lkt:14499:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14499'
   --# expr-start 10559 '<FieldAccess for env_get_public at nodes.lkt:14499:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14499'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.env_get_public
Fld_6 := Libadalang.Implementation.Ada_Node_P_Env_Get_Public (Node => Self, Env => Local_Complete_Env, Symbol => No_Symbol, Lookup => Flat);
--# end
--# expr-done 10559
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
--# expr-start 10561 '<Not at nodes.lkt:14526:17>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14526'
--# expr-start 10560 '<IsA PrivatePart at nodes.lkt:14526:21>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14526'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Private_Part_Range; 
--# expr-done 10560
Not_Val := not (Is_A); 
--# expr-done 10561
if Not_Val then
--# expr-start 10563 '<Eq at nodes.lkt:14528:17>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14528'
--# expr-start 10562 '<FieldAccess for owning_unit_kind at nodes.lkt:14528:17>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14528'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.owning_unit_kind
Fld_17 := Libadalang.Implementation.Ada_Node_P_Owning_Unit_Kind (Node => Item_0.Node);
--# end
--# expr-done 10562
Is_Equal := Fld_17 = Unit_Specification; 
--# expr-done 10563
if Is_Equal then
   If_Result_2 := True;
else
   --# expr-start 10564 '<FieldAccess for has_visibility at nodes.lkt:14529:20>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14529'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_18 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 10564
   If_Result_2 := Fld_18;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
         if If_Result_3 then
      --# expr-start 10580 '<New CompletionItem at nodes.lkt:14502:20>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14502'
--# expr-start 10565 '<Cast to BasicDecl at nodes.lkt:14503:22>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14503'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10565
--# expr-start 10568 '<FieldAccess for dottable_subp at nodes.lkt:14504:29>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14504'
--# expr-start 10567 '<FieldAccess for md at nodes.lkt:14504:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14504'
--# expr-start 10566 '<FieldAccess for info at nodes.lkt:14504:29>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14504'
Fld_7 := Item_0.Info;
--# expr-done 10566
Fld_8 := Fld_7.Md;
--# expr-done 10567
Fld_9 := Fld_8.Dottable_Subp;
--# expr-done 10568
--# expr-start 10571 '<FieldAccess for dottable_subp at nodes.lkt:14507:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14507'
--# expr-start 10570 '<FieldAccess for md at nodes.lkt:14507:21>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14507'
--# expr-start 10569 '<FieldAccess for info at nodes.lkt:14507:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14507'
Fld_10 := Item_0.Info;
--# expr-done 10569
Fld_11 := Fld_10.Md;
--# expr-done 10570
Fld_12 := Fld_11.Dottable_Subp;
--# expr-done 10571
if Fld_12 then
   If_Result_1 := True;
else
--# expr-start 10572 '<FieldAccess for has_visibility at nodes.lkt:14510:21>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14510'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_13 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 10572
if Fld_13 then
   --# expr-start 10577 '<Contains at nodes.lkt:14511:25>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14511'
   --# expr-start 10575 '<FieldAccess for env_get_public at nodes.lkt:14511:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14511'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10574 '<FieldAccess for name_symbol at nodes.lkt:14513:25>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14513'
--# expr-start 10573 '<Cast to BasicDecl at nodes.lkt:14513:25>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14513'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10573
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Cast_Result_1.Node);
--# end
--# expr-done 10574
--# property-call-start AdaNode.env_get_public
Fld_15 := Libadalang.Implementation.Ada_Node_P_Env_Get_Public (Node => Self, Env => Local_Visible_Env, Symbol => Fld_14, Lookup => Flat);
--# end
--# expr-done 10575
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_15;
      begin
         for Item_17 of
               Collection.Items
         loop
   --# scope-start
               --# bind item Item_17
      --# expr-start 10576 '<Eq at ???>' Is_Equiv None
Is_Equiv := Equivalent (Item_17, Item_0); 
--# expr-done 10576
      Quantifier_Result := Is_Equiv;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 10577
   If_Result := Quantifier_Result;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-start 10579 '<FieldAccess for complete_item_weight at nodes.lkt:14517:24>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14517'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10578 '<Cast to BasicDecl at nodes.lkt:14517:50>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14517'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10578
--# property-call-start '[dispatcher]AdaNode.complete_item_weight'
Fld_16 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Cast_Result_2, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 10579
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_9, Is_Visible => If_Result_1, Weight => Fld_16); 
--# expr-done 10580
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
      Finalizer_Scope_3362;
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10581
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10582
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10583
Dyn_Var_Bind_Result_2 := Let_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 10584
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Dyn_Var_Bind_Result_3);
--# expr-done 10585
         Property_Result := Dyn_Var_Bind_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1264;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1264;
                     Finalizer_Scope_3362;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Complete_Items;
--# end
--# property-start DottedName.complete_item_weight 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14535'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete_Item_Weight
  (Node : Bare_Dotted_Name
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Integer;
Is_A : Boolean;
If_Result : Integer;
Is_A_1 : Boolean;
If_Result_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10590 '<If at nodes.lkt:14537:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14537'
--# expr-start 10586 '<IsA ComponentDecl, DiscriminantSpec at nodes.lkt:14537:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14537'
Is_A_1 := Item.Node /= null 
and then Item.Node.Kind in Ada_Component_Decl_Range | Ada_Discriminant_Spec_Range; 
--# expr-done 10586
if Is_A_1 then
   If_Result_1 := 100;
else
   --# expr-start 10589 '<If at nodes.lkt:14537:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14537'
--# expr-start 10587 '<IsA BasicSubpDecl, BaseSubpBody at nodes.lkt:14539:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14539'
Is_A := Item.Node /= null 
and then Item.Node.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 10587
if Is_A then
   If_Result := 75;
else
   --# expr-start 10588 '<Super at nodes.lkt:14541:14>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14541'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.complete_item_weight
Fld := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight (Node => Ent.Node, Item => Item, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10588
   If_Result := Fld;
end if;
--# expr-done 10589
   If_Result_1 := If_Result;
end if;
--# expr-done 10590
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Complete_Item_Weight;
--# end
--# property-start DottedName.designated_env_no_overloading 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14544'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env_No_Overloading
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1266 with Inline_Always;
      Local_Pfx_Env : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1266 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Pfx_Env);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1266;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10596 '<Let at nodes.lkt:14545:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14545'
--# scope-start
--# expr-start 10592 '<FieldAccess for designated_env_no_overloading at nodes.lkt:14545:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14545'
--# expr-start 10591 '<FieldAccess for prefix at nodes.lkt:14545:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14545'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10591
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 10592
Local_Pfx_Env := Fld_1; Inc_Ref (Local_Pfx_Env);
--# bind pfx_env Local_Pfx_Env
--# expr-start 10595 '<bind at nodes.lkt:14548:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14548'
Bound_Env := Local_Pfx_Env; Inc_Ref (Bound_Env);
--# expr-start 10594 '<FieldAccess for designated_env_no_overloading at nodes.lkt:14550:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14550'
--# expr-start 10593 '<FieldAccess for suffix at nodes.lkt:14550:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14550'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10593
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 10594
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10595
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10596
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1266;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1266;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Env_No_Overloading;
--# end
--# property-start DottedName.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14555'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1267 with Inline_Always;
      Local_Pfx_Env : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1267 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Pfx_Env);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1267;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10602 '<Let at nodes.lkt:14556:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14556'
--# scope-start
--# expr-start 10598 '<FieldAccess for designated_env at nodes.lkt:14556:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14556'
--# expr-start 10597 '<FieldAccess for prefix at nodes.lkt:14556:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14556'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10597
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 10598
Local_Pfx_Env := Fld_1; Inc_Ref (Local_Pfx_Env);
--# bind pfx_env Local_Pfx_Env
--# expr-start 10601 '<bind at nodes.lkt:14559:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14559'
Bound_Env := Local_Pfx_Env; Inc_Ref (Bound_Env);
--# expr-start 10600 '<FieldAccess for designated_env at nodes.lkt:14561:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14561'
--# expr-start 10599 '<FieldAccess for suffix at nodes.lkt:14561:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14561'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10599
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld_2.Info);
--# end
--# expr-done 10600
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10601
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10602
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1267;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1267;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Env;
--# end
--# property-start DottedName.all_env_els_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14566'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_All_Env_Els_Impl
  (Node : Bare_Dotted_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1268 with Inline_Always;
      Local_Pfx_Env : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1268 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Pfx_Env);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1268;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10608 '<Let at nodes.lkt:14567:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14567'
--# scope-start
--# expr-start 10604 '<FieldAccess for designated_env at nodes.lkt:14567:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14567'
--# expr-start 10603 '<FieldAccess for prefix at nodes.lkt:14567:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14567'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10603
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 10604
Local_Pfx_Env := Fld_1; Inc_Ref (Local_Pfx_Env);
--# bind pfx_env Local_Pfx_Env
--# expr-start 10607 '<bind at nodes.lkt:14570:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14570'
Bound_Env := Local_Pfx_Env; Inc_Ref (Bound_Env);
--# expr-start 10606 '<FieldAccess for all_env_els_impl at nodes.lkt:14572:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14572'
--# expr-start 10605 '<FieldAccess for suffix at nodes.lkt:14572:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14572'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10605
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_2.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 10606
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10607
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10608
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1268;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1268;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_All_Env_Els_Impl;
--# end
--# property-start DottedName.scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14577'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1269 with Inline_Always;
            procedure Finalizer_Scope_3364 with Inline_Always;
      Var_Expr_0 : Bare_Base_Id;
Fld : Bare_Base_Id;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1269 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1269;
            procedure Finalizer_Scope_3364 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3364;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10609 '<FieldAccess for suffix at nodes.lkt:14577:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14577'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 10609
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sfx Var_Expr_0
--# expr-start 10612 '<bind at nodes.lkt:14579:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14579'
--# expr-start 10610 '<FieldAccess for parent_scope at nodes.lkt:14579:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14579'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Self, Env => Env);
--# end
--# expr-done 10610
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 10611 '<FieldAccess for scope at nodes.lkt:14581:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14581'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Var_Expr_0, Env => Bound_Env);
--# end
--# expr-done 10611
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10612
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3364;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1269;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1269;
                     Finalizer_Scope_3364;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Scope;
--# end
--# property-start DottedName.parent_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14586'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Parent_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1270 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1270 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1270;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10614 '<FieldAccess for scope at nodes.lkt:14586:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14586'
--# expr-start 10613 '<FieldAccess for prefix at nodes.lkt:14586:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14586'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Dotted_Name_F_Prefix;
--# expr-done 10613
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 10614
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1270;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1270;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Parent_Scope;
--# end
--# property-start DottedName.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14588'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Relative_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10616 '<FieldAccess for relative_name at nodes.lkt:14588:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14588'
--# expr-start 10615 '<FieldAccess for suffix at nodes.lkt:14588:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14588'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10615
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10616
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Relative_Name;
--# end
--# property-start DottedName.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14591'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Env_Elements_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1272 with Inline_Always;
      Local_Pfx_Env : Lexical_Env := Empty_Env;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1272 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Local_Pfx_Env);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1272;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10624 '<Let at nodes.lkt:14592:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14592'
--# scope-start
--# expr-start 10620 '<bind at nodes.lkt:14593:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14593'
--# expr-start 10617 '<FieldAccess for origin_node at nodes.lkt:14593:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14593'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10617
Bound_Origin := Fld; 
--# expr-start 10619 '<FieldAccess for designated_env at nodes.lkt:14595:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14595'
--# expr-start 10618 '<FieldAccess for prefix at nodes.lkt:14595:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14595'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10618
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, No_Visibility => False, E_Info => Fld_1.Info);
--# end
--# expr-done 10619
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10620
Local_Pfx_Env := Dyn_Var_Bind_Result; Inc_Ref (Local_Pfx_Env);
--# bind pfx_env Local_Pfx_Env
--# expr-start 10623 '<bind at nodes.lkt:14599:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14599'
Bound_Env := Local_Pfx_Env; Inc_Ref (Bound_Env);
--# expr-start 10622 '<FieldAccess for env_elements_impl at nodes.lkt:14601:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14601'
--# expr-start 10621 '<FieldAccess for suffix at nodes.lkt:14601:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14601'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10621
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_3.Node, Env => Bound_Env, E_Info => Fld_3.Info);
--# end
--# expr-done 10622
Dyn_Var_Bind_Result_1 := Fld_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 10623
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 10624
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1272;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1272;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Env_Elements_Impl;
--# end
--# property-start DottedName.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14606'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Type_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1273 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1273 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1273;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10629 '<bind at nodes.lkt:14607:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14607'
--# expr-start 10626 '<FieldAccess for designated_env_no_overloading at nodes.lkt:14607:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14607'
--# expr-start 10625 '<FieldAccess for prefix at nodes.lkt:14607:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14607'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10625
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld.Info);
--# end
--# expr-done 10626
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 10628 '<FieldAccess for designated_type_impl at nodes.lkt:14609:9>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14609'
--# expr-start 10627 '<FieldAccess for suffix at nodes.lkt:14609:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14609'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10627
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 10628
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 10629
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1273;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1273;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Designated_Type_Impl;
--# end
--# property-start DottedName.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14613'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Xref_Equation
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1274 with Inline_Always;
            procedure Finalizer_Scope_3365 with Inline_Always;
            procedure Finalizer_Scope_3366 with Inline_Always;
      Local_Base : Logic_Equation;
Var_Expr_0 : Internal_Entity_Array_Access;
Local_Actual : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Base_Id;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Bare_Base_Id;
Fld_11 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_12 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Basic_Decl;
Fld_14 : Bare_Base_Id;
Fld_15 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_16 : Bare_Name;
Cast_Result_2 : Bare_Expr;
Fld_17 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_18 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_19 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1274 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Local_Base);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1274;
            procedure Finalizer_Scope_3365 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_3365;
            procedure Finalizer_Scope_3366 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3366;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10666 '<Let at nodes.lkt:14614:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14614'
--# scope-start
--# expr-start 10637 '<And at nodes.lkt:14614:20>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14614'
--# expr-start 10631 '<FieldAccess for sub_equation at nodes.lkt:14614:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14614'
--# expr-start 10630 '<FieldAccess for prefix at nodes.lkt:14614:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14614'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10630
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 10631
--# expr-start 10636 '<bind at nodes.lkt:14615:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14615'
--# expr-start 10633 '<FieldAccess for designated_env at nodes.lkt:14615:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14615'
--# expr-start 10632 '<FieldAccess for prefix at nodes.lkt:14615:24>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14615'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10632
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_2.Info);
--# end
--# expr-done 10633
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 10635 '<FieldAccess for sub_equation at nodes.lkt:14617:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14617'
--# expr-start 10634 '<FieldAccess for suffix at nodes.lkt:14617:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14617'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10634
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_4.Info);
--# end
--# expr-done 10635
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10636
And_Pred := Create_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14614:20") else null)); 
--# expr-done 10637
Local_Base := And_Pred; Inc_Ref (Local_Base);
--# bind base Local_Base
--# expr-start 10665 '<If at nodes.lkt:14620:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14620'
--# expr-start 10640 '<Not at nodes.lkt:14620:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14620'
--# expr-start 10639 '<IsNull at nodes.lkt:14620:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14620'
--# expr-start 10638 '<FieldAccess for designated_type_impl at nodes.lkt:14620:16>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14620'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10638
Is_Null := Fld_19.Node = null; 
--# expr-done 10639
Not_Val := not (Is_Null); 
--# expr-done 10640
if Not_Val then
   If_Result := Local_Base;
else
   --# expr-start 10664 '<And at nodes.lkt:14620:67>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14620'
--# expr-start 10650 '<And at nodes.lkt:14621:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14621'
--# expr-start 10645 '<And at nodes.lkt:14622:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14622'
--# expr-start 10644 '<Bind at nodes.lkt:14622:26>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14622'
--# expr-start 10641 '<FieldAccess for expected_type_var at nodes.lkt:14622:30>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14622'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 10641
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 10643 '<FieldAccess for expected_type_var at nodes.lkt:14622:56>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14622'
--# expr-start 10642 '<FieldAccess for suffix at nodes.lkt:14622:56>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14622'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Dotted_Name_F_Suffix;
--# expr-done 10642
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7);
--# expr-done 10643
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Bind_Result := Solver.Create_Unify (Fld_6, Fld_8, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14622:26") else null)); 
--# expr-done 10644
And_Pred_1 := Create_And (Local_Base, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14622:17") else null)); 
--# expr-done 10645
--# expr-start 10649 '<Bind at nodes.lkt:14623:19>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14623'
--# expr-start 10646 '<FieldAccess for type_var at nodes.lkt:14623:23>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14623'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10646
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 10648 '<FieldAccess for type_var at nodes.lkt:14623:40>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14623'
--# expr-start 10647 '<FieldAccess for suffix at nodes.lkt:14623:40>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14623'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Dotted_Name_F_Suffix;
--# expr-done 10647
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 10648
Fld_11.Value := No_Entity;
Entity_Vars.Reset (Fld_11);
Bind_Result_1 := Solver.Create_Unify (Fld_9, Fld_11, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14623:19") else null)); 
--# expr-done 10649
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14621:13") else null)); 
--# expr-done 10650
--# expr-start 10651 '<FieldAccess for env_elements at nodes.lkt:14624:15>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14624'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_12 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 10651
Var_Expr_0 := Fld_12;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind env_els Var_Expr_0
--# expr-start 10662 '<LogicAny at nodes.lkt:14625:26>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14625'
--# expr-start 10661 '<Map at nodes.lkt:14625:26>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14625'
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
      --# expr-start 10660 '<Let at nodes.lkt:14626:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14626'
--# scope-start
--# expr-start 10653 '<FieldAccess for corresponding_actual at nodes.lkt:14626:30>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14626'
--# expr-start 10652 '<Cast to BasicDecl at nodes.lkt:14626:30>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14626'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10652
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.corresponding_actual'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Corresponding_Actual (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 10653
Local_Actual := Fld_13; 
--# bind actual Local_Actual
--# expr-start 10659 '<And at nodes.lkt:14627:17>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14627'
--# expr-start 10656 '<Bind at nodes.lkt:14627:17>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14627'
--# expr-start 10655 '<FieldAccess for ref_var at nodes.lkt:14627:21>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14627'
--# expr-start 10654 '<FieldAccess for suffix at nodes.lkt:14627:21>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14627'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Dotted_Name_F_Suffix;
--# expr-done 10654
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_14);
--# end
--# expr-done 10655
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
Cast_Expr_1 := Local_Actual; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_15, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14627:17") else null)); 
--# expr-done 10656
--# expr-start 10658 '<FieldAccess for constrain_prefix at nodes.lkt:14628:21>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14628'
      if Local_Actual.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10657 '<FieldAccess for prefix at nodes.lkt:14628:45>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14628'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Dotted_Name_F_Prefix;
--# expr-done 10657
      Cast_Result_2 := Fld_16;
--# property-call-start '[dispatcher]BasicDecl.constrain_prefix'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Constrain_Prefix (Node => Local_Actual.Node, Prefix => Cast_Result_2, Origin => Origin, E_Info => Local_Actual.Info);
--# end
--# expr-done 10658
And_Pred_3 := Create_And (Bind_Result_2, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14627:17") else null)); 
--# expr-done 10659
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 10660
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3366;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10661
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14625:26") else null)); 
--# expr-done 10662
Scope_Result := Logic_Boolean_Op;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3365;
   Result_Var := Scope_Result;
else
   --# expr-start 10663 '<FieldAccess for undefined_reference_equation at nodes.lkt:14629:29>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14629'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.undefined_reference_equation
Fld_18 := Libadalang.Implementation.Name_P_Undefined_Reference_Equation (Node => Ent.Node);
--# end
--# expr-done 10663
   Result_Var := Fld_18;
end if;
      Inc_Ref (Result_Var);
And_Pred_4 := Create_And (And_Pred_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14620:67") else null)); 
--# expr-done 10664
   If_Result := And_Pred_4;
end if;
      Inc_Ref (If_Result);
--# expr-done 10665
Let_Result_1 := If_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10666
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1274;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1274;
                     Finalizer_Scope_3365;
                     Finalizer_Scope_3366;
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Xref_Equation;
--# end
--# property-start DottedName.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14633'
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Is_Constant
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Dotted_Name  := Bare_Dotted_Name (Node);
     Ent : Internal_Entity_Dotted_Name :=
       Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Name;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10668 '<FieldAccess for is_constant at nodes.lkt:14636:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14636'
--# expr-start 10667 '<FieldAccess for prefix at nodes.lkt:14636:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 10667
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10668
if Fld_1 then
   If_Result := True;
else
   --# expr-start 10670 '<FieldAccess for is_constant at nodes.lkt:14636:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14636'
--# expr-start 10669 '<FieldAccess for suffix at nodes.lkt:14636:38>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14636'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 10669
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.is_constant'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Is_Constant (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 10670
   If_Result := Fld_3;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Dotted_Name_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_End_Name
   --
      procedure Initialize_Fields_For_End_Name
        (Self : Bare_End_Name
         ; End_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.End_Name_F_Name := End_Name_F_Name;
      end Initialize_Fields_For_End_Name;
   function End_Name_F_Name
     (Node : Bare_End_Name) return Bare_Name
   is
   begin
         return Node.End_Name_F_Name;
   end;
--# property-start EndName.parent_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14723'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Parent_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1283 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1283 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1283;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10672 '<FieldAccess for parent_scope at nodes.lkt:14723:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14723'
--# expr-start 10671 '<FieldAccess for name at nodes.lkt:14723:38>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14723'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 10671
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 10672
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1283;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1283;
            raise;
      end;
   return Property_Result;
end End_Name_P_Parent_Scope;
--# end
--# property-start EndName.scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14726'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1284 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1284 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1284;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10674 '<FieldAccess for scope at nodes.lkt:14726:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14726'
--# expr-start 10673 '<FieldAccess for name at nodes.lkt:14726:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14726'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 10673
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 10674
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1284;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1284;
            raise;
      end;
   return Property_Result;
end End_Name_P_Scope;
--# end
--# property-start EndName.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14728'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Relative_Name
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10676 '<FieldAccess for relative_name at nodes.lkt:14728:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14728'
--# expr-start 10675 '<FieldAccess for name at nodes.lkt:14728:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14728'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 10675
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10676
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Relative_Name;
--# end
--# property-start EndName.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14730'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Ref_Var
  (Node : Bare_End_Name
  )
   return Logic_Var
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10678 '<FieldAccess for ref_var at nodes.lkt:14730:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14730'
--# expr-start 10677 '<FieldAccess for name at nodes.lkt:14730:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14730'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 10677
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10678
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Ref_Var;
--# end
--# property-start EndName.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14733'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Env_Elements_Impl
  (Node : Bare_End_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1287 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1287 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1287;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10680 '<FieldAccess for env_elements_impl at nodes.lkt:14734:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14734'
--# expr-start 10679 '<FieldAccess for name at nodes.lkt:14734:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14734'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 10679
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 10680
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1287;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1287;
            raise;
      end;
   return Property_Result;
end End_Name_P_Env_Elements_Impl;
--# end
--# property-start EndName.basic_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14739'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Basic_Decl
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1288 with Inline_Always;
      Var_Expr_1 : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Named_Stmt;
Fld : Bare_Named_Stmt_Decl;
As_Entity_1 : Internal_Entity_Named_Stmt_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Named_Stmt_Decl;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1288 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1288;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_End_Name_End_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_End_Name,
                                As_Bare_End_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
--# expr-start 10683 '<Find at nodes.lkt:14740:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14740'
   --# expr-start 10681 '<FieldAccess for parents at nodes.lkt:14740:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14740'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 10681
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 10682 '<IsA BasicDecl, NamedStmt at nodes.lkt:14740:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14740'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Basic_Decl | Ada_Named_Stmt_Range; 
--# expr-done 10682
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10683
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 10690 '<If at nodes.lkt:14741:20>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
--# expr-start 10684 '<IsA BasicDecl at nodes.lkt:14741:23>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
Is_A_1 := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Basic_Decl; 
--# expr-done 10684
if Is_A_1 then
   --# expr-start 10686 '<as_entity at nodes.lkt:14741:43>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
--# expr-start 10685 '<Cast to BasicDecl at nodes.lkt:14741:43>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
Cast_Expr := Var_Expr_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 10685
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 10686
   If_Result_2 := If_Result;
else
--# expr-start 10689 '<as_entity at nodes.lkt:14741:74>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
--# expr-start 10688 '<FieldAccess for decl at nodes.lkt:14741:74>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
--# expr-start 10687 '<Cast to NamedStmt at nodes.lkt:14741:74>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14741'
Cast_Expr_1 := Var_Expr_1; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 10687
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Cast_Result_1.Named_Stmt_F_Decl;
--# expr-done 10688
Is_Null_1 := Fld = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Named_Stmt_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Fld); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 10689
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   If_Result_2 := Cast_Result_2;
end if;
--# expr-done 10690
Scope_Result := If_Result_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      Finalizer_Scope_1288;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1288;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                        As_Internal_Entity_Basic_Decl => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
         end if;
      end if;
   return Property_Result;
end End_Name_P_Basic_Decl;
--# end
--# property-start EndName.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14745'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Equation
  (Node : Bare_End_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
     Ent : Internal_Entity_End_Name :=
       Internal_Entity_End_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1289 with Inline_Always;
            procedure Finalizer_Scope_3369 with Inline_Always;
            procedure Finalizer_Scope_3370 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Accept_Stmt_With_Stmts;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Result_Var : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Fld_6 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1289 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1289;
            procedure Finalizer_Scope_3369 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_3369;
            procedure Finalizer_Scope_3370 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3370;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10705 '<And at nodes.lkt:14745:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14745'
--# expr-start 10692 '<Cast to AcceptStmtWithStmts at nodes.lkt:14745:37>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14745'
--# expr-start 10691 '<parent at nodes.lkt:14745:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14745'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10691
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range
   then
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Accept_Stmt_With_Stmts;
   end if;
--# expr-done 10692
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Accept_Stmt_With_Stmts then
   --# scope-start
   --# bind stmt Var_Expr_0
--# expr-start 10695 '<Bind at nodes.lkt:14746:19>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
--# expr-start 10693 '<FieldAccess for ref_var at nodes.lkt:14746:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10693
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 10694 '<FieldAccess for designated_entry at nodes.lkt:14746:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Var_Expr_0.Node, Origin => Origin, Env => Env, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10694
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14746:19") else null)); 
--# expr-done 10695
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3369;
   Result_Var := Scope_Result;
else
   --# expr-start 10698 '<Bind at nodes.lkt:14746:77>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
--# expr-start 10696 '<FieldAccess for ref_var at nodes.lkt:14746:81>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 10696
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 10697 '<FieldAccess for basic_decl at nodes.lkt:14746:97>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14746'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EndName.basic_decl
Fld_4 := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 10697
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_3, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14746:77") else null)); 
--# expr-done 10698
   Result_Var := Bind_Result_1;
end if;
      Inc_Ref (Result_Var);
--# expr-start 10700 '<Cast to DottedName at nodes.lkt:14747:11>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14747'
--# expr-start 10699 '<FieldAccess for name at nodes.lkt:14747:11>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14747'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 10699
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_3 := No_Entity_Dotted_Name;
   end if;
--# expr-done 10700
Var_Expr_1 := Cast_Result_3;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 10704 '<bind at nodes.lkt:14752:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14752'
--# expr-start 10701 '<FieldAccess for std_env at nodes.lkt:14752:24>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14752'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_env
Fld_6 := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Ent.Node);
--# end
--# expr-done 10701
Bound_Env := Fld_6; Inc_Ref (Bound_Env);
--# expr-start 10703 '<FieldAccess for xref_no_overloading at nodes.lkt:14754:13>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14754'
--# expr-start 10702 '<FieldAccess for prefix at nodes.lkt:14754:13>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14754'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 10702
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_7.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 10703
Dyn_Var_Bind_Result := Fld_8; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10704
Scope_Result_1 := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3370;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14755:24") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
And_Pred := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14745:37") else null)); 
--# expr-done 10705
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1289;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1289;
                     Finalizer_Scope_3369;
                     Finalizer_Scope_3370;
            raise;
      end;
   return Property_Result;
end End_Name_P_Xref_Equation;
--# end
--# property-start EndName.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14758'
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Entry_Point
  (Node : Bare_End_Name
  )
   return Boolean
is
   Self : Bare_End_Name  := Bare_End_Name (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end End_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Explicit_Deref
   --
      procedure Initialize_Fields_For_Explicit_Deref
        (Self : Bare_Explicit_Deref
         ; Explicit_Deref_F_Prefix : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Explicit_Deref_F_Prefix := Explicit_Deref_F_Prefix;
      Self.Explicit_Deref_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Explicit_Deref;
   function Explicit_Deref_F_Prefix
     (Node : Bare_Explicit_Deref) return Bare_Name
   is
   begin
         return Node.Explicit_Deref_F_Prefix;
   end;
--# property-start ExplicitDeref.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14766'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Ref_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10707 '<FieldAccess for ref_var at nodes.lkt:14766:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14766'
--# expr-start 10706 '<FieldAccess for prefix at nodes.lkt:14766:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14766'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 10706
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10707
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Ref_Var;
--# end
--# property-start ExplicitDeref.subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14768'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10708 '<FieldAccess for r_called_spec at nodes.lkt:14768:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14768'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_R_Called_Spec'Unrestricted_Access;
--# expr-done 10708
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.defines_subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14770'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Defines_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Boolean
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Defines_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14772'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Relative_Name
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10710 '<FieldAccess for relative_name at nodes.lkt:14772:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14772'
--# expr-start 10709 '<FieldAccess for prefix at nodes.lkt:14772:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14772'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10709
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10710
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Relative_Name;
--# end
--# property-start ExplicitDeref.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14775'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Designated_Env
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1295 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1295 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1295;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10712 '<FieldAccess for designated_env at nodes.lkt:14778:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14778'
--# expr-start 10711 '<FieldAccess for prefix at nodes.lkt:14778:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10711
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => Fld.Info);
--# end
--# expr-done 10712
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1295;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1295;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Designated_Env;
--# end
--# property-start ExplicitDeref.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14781'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Env_Elements_Impl
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1296 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1296 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1296;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10714 '<FieldAccess for env_elements_impl at nodes.lkt:14782:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14782'
--# expr-start 10713 '<FieldAccess for prefix at nodes.lkt:14782:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14782'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10713
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 10714
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1296;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1296;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Env_Elements_Impl;
--# end
--# property-start ExplicitDeref.eq_for_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14785'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Eq_For_Type
  (Node : Bare_Explicit_Deref
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1297 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
False_Rel : Logic_Equation;
Fld_6 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1297 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1297;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10726 '<If at nodes.lkt:14785:60>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14785'
--# expr-start 10715 '<FieldAccess for is_access_type at nodes.lkt:14785:63>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14785'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 10715
if Fld_6 then
   --# expr-start 10725 '<And at nodes.lkt:14785:89>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14785'
--# expr-start 10721 '<And at nodes.lkt:14786:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
--# expr-start 10718 '<Bind at nodes.lkt:14786:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
--# expr-start 10717 '<FieldAccess for expected_type_var at nodes.lkt:14786:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
--# expr-start 10716 '<FieldAccess for prefix at nodes.lkt:14786:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 10716
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld);
--# expr-done 10717
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
Cast_Expr := Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14786:9") else null)); 
--# expr-done 10718
--# expr-start 10720 '<FieldAccess for matches_expected_type at nodes.lkt:14786:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
--# expr-start 10719 '<FieldAccess for prefix at nodes.lkt:14786:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14786'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10719
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_3 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_2.Node, Origin => Origin);
--# end
--# expr-done 10720
And_Pred := Create_And (Bind_Result, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14786:9") else null)); 
--# expr-done 10721
--# expr-start 10724 '<Bind at nodes.lkt:14787:11>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14787'
--# expr-start 10722 '<FieldAccess for type_var at nodes.lkt:14787:15>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14787'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10722
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 10723 '<FieldAccess for accessed_type at nodes.lkt:14787:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14787'
      if Typ.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 10723
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14787:11") else null)); 
--# expr-done 10724
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14785:89") else null)); 
--# expr-done 10725
   If_Result := And_Pred_1;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14787:58") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 10726
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1297;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1297;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Eq_For_Type;
--# end
--# property-start ExplicitDeref.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14790'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1298 with Inline_Always;
      Fld : Logic_Equation;
            procedure Finalizer_Scope_1298 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1298;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10727 '<FieldAccess for bottom_up_name_equation at nodes.lkt:14790:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14790'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10727
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1298;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1298;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Xref_Equation;
--# end
--# property-start ExplicitDeref.general_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14793'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_General_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1299 with Inline_Always;
            procedure Finalizer_Scope_3371 with Inline_Always;
            procedure Finalizer_Scope_3372 with Inline_Always;
            procedure Finalizer_Scope_3373 with Inline_Always;
      Local_Typ : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_263 : Internal_Entity_Base_Type_Decl;
Fld : Logic_Equation;
Fld_1 : Internal_Entity_Name;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Bare_Name;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
Fld_12 : Internal_Entity_Array_Access;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Type_Decl;
Fld_14 : Bare_Name;
Fld_15 : Logic_Var;
Bind_Result_2 : Logic_Equation;
Fld_16 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel_2 : Logic_Equation;
Fld_17 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_18 : Internal_Entity_Name;
Is_A : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_1299 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_1299;
            procedure Finalizer_Scope_3371 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3371;
            procedure Finalizer_Scope_3372 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3372;
            procedure Finalizer_Scope_3373 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3373;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10762 '<And at nodes.lkt:14797:9>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14797'
--# expr-start 10728 '<FieldAccess for all_args_xref_equation at nodes.lkt:14797:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14797'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10728
--# expr-start 10761 '<If at nodes.lkt:14798:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14798'
--# expr-start 10730 '<IsA AttributeRef at nodes.lkt:14798:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14798'
--# expr-start 10729 '<FieldAccess for prefix at nodes.lkt:14798:16>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14798'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10729
Is_A := Fld_18.Node /= null 
and then Fld_18.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 10730
if Is_A then
--# expr-start 10732 '<FieldAccess for resolve_names_internal at nodes.lkt:14799:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14799'
--# expr-start 10731 '<FieldAccess for prefix at nodes.lkt:14799:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14799'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10731
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld_1.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 10732
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 10735 '<Cast to BaseTypeDecl at nodes.lkt:14800:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14800'
--# expr-start 10734 '<FieldAccess for type_val at nodes.lkt:14800:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14800'
--# expr-start 10733 '<FieldAccess for prefix at nodes.lkt:14800:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14800'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10733
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_4 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_3.Node);
--# end
--# expr-done 10734
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10735
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_2
--# expr-start 10744 '<And at nodes.lkt:14802:25>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14802'
--# expr-start 10742 '<And at nodes.lkt:14802:25>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14802'
--# expr-start 10738 '<Bind at nodes.lkt:14802:25>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14802'
--# expr-start 10737 '<FieldAccess for expected_type_var at nodes.lkt:14802:29>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14802'
--# expr-start 10736 '<FieldAccess for prefix at nodes.lkt:14802:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14802'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Explicit_Deref_F_Prefix;
--# expr-done 10736
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_5);
--# expr-done 10737
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Cast_Expr := Var_Expr_2; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_6, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14802:25") else null)); 
--# expr-done 10738
--# expr-start 10741 '<Bind at nodes.lkt:14803:29>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14803'
--# expr-start 10740 '<FieldAccess for type_var at nodes.lkt:14803:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14803'
--# expr-start 10739 '<FieldAccess for prefix at nodes.lkt:14803:33>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14803'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Explicit_Deref_F_Prefix;
--# expr-done 10739
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 10740
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr_1 := Var_Expr_2; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14803:29") else null)); 
--# expr-done 10741
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14802:25") else null)); 
--# expr-done 10742
--# expr-start 10743 '<FieldAccess for parent_name_equation at nodes.lkt:14804:29>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14804'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_9 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Var_Expr_2, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10743
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14802:25") else null)); 
--# expr-done 10744
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3372;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14806:33") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3371;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14808:29") else null)); 
   Result_Var_1 := False_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
else
   --# expr-start 10760 '<And at nodes.lkt:14811:13>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14811'
--# expr-start 10746 '<FieldAccess for sub_equation at nodes.lkt:14811:13>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14811'
--# expr-start 10745 '<FieldAccess for prefix at nodes.lkt:14811:13>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14811'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10745
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 10746
--# expr-start 10759 '<LogicAny at nodes.lkt:14812:17>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14812'
--# expr-start 10758 '<Map at nodes.lkt:14812:17>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14812'
   --# expr-start 10747 '<FieldAccess for env_elements at nodes.lkt:14812:17>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14812'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_12 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 10747
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_12;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_0
      --# expr-start 10757 '<Let at nodes.lkt:14814:21>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14814'
--# scope-start
--# expr-start 10749 '<FieldAccess for expr_type at nodes.lkt:14814:31>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14814'
--# expr-start 10748 '<Cast to BasicDecl at nodes.lkt:14814:31>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14814'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10748
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result_3.Node, Origin => Origin, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 10749
Local_Typ := Fld_13; 
--# bind typ Local_Typ
--# expr-start 10756 '<If at nodes.lkt:14815:21>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14815'
Var_Expr_263 := Local_Typ;
if Var_Expr_263 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 10750 '<FieldAccess for is_access_type at nodes.lkt:14815:24>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14815'
      if Var_Expr_263.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr_263.Node, Origin => Origin, E_Info => Var_Expr_263.Info);
--# end
--# expr-done 10750
Scope_Result_2 := Fld_17;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 10755 '<And at nodes.lkt:14816:25>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14816'
--# expr-start 10753 '<Bind at nodes.lkt:14816:25>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14816'
--# expr-start 10752 '<FieldAccess for ref_var at nodes.lkt:14816:29>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14816'
--# expr-start 10751 '<FieldAccess for prefix at nodes.lkt:14816:29>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14816'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Self.Explicit_Deref_F_Prefix;
--# expr-done 10751
      if Fld_14 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_14);
--# end
--# expr-done 10752
Fld_15.Value := No_Entity;
Entity_Vars.Reset (Fld_15);
Bind_Result_2 := Solver.Create_Assign (Fld_15, Item_0, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14816:25") else null)); 
--# expr-done 10753
--# expr-start 10754 '<FieldAccess for parent_name_equation at nodes.lkt:14817:29>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14817'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Local_Typ, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10754
And_Pred_2 := Create_And (Bind_Result_2, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14816:25") else null)); 
--# expr-done 10755
   If_Result := And_Pred_2;
else
   False_Rel_2 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14819:25") else null)); 
   If_Result := False_Rel_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 10756
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 10757
         declare
            Item_To_Append : constant Logic_Equation := Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3373;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10758
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14812:17") else null)); 
--# expr-done 10759
And_Pred_3 := Create_And (Fld_11, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14811:13") else null)); 
--# expr-done 10760
   If_Result_1 := And_Pred_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10761
And_Pred_4 := Create_And (Fld, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14797:9") else null)); 
--# expr-done 10762
         Property_Result := And_Pred_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1299;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1299;
                     Finalizer_Scope_3371;
                     Finalizer_Scope_3372;
                     Finalizer_Scope_3373;
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_General_Xref_Equation;
--# end
--# property-start ExplicitDeref.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14824'
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Is_Constant
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Explicit_Deref  := Bare_Explicit_Deref (Node);
     Ent : Internal_Entity_Explicit_Deref :=
       Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_264 : Internal_Entity_Base_Type_Decl;
Var_Expr_265 : Internal_Entity_Type_Access_Def;
Cast_Expr : Bare_Explicit_Deref;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Type_Access_Def;
Fld_3 : Internal_Entity_Constant_Node;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10769 '<bind at nodes.lkt:14828:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14828'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 10766 '<Cast to TypeAccessDef at nodes.lkt:14830:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
--# expr-start 10764 '<FieldAccess for expression_type at nodes.lkt:14830:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
--# expr-start 10763 '<FieldAccess for prefix at nodes.lkt:14830:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 10763
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10764
Var_Expr_264 := Fld_1;
if Var_Expr_264 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 10765 '<FieldAccess for access_def at nodes.lkt:14830:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
      if Var_Expr_264.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_264.Node, Origin => Bound_Origin, E_Info => Var_Expr_264.Info);
--# end
--# expr-done 10765
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Type_Access_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Access_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Type_Access_Def;
   end if;
--# expr-done 10766
Var_Expr_265 := Cast_Result_1;
if Var_Expr_265 /= No_Entity_Type_Access_Def then
   --# scope-start
--# expr-start 10768 '<FieldAccess for as_bool at nodes.lkt:14830:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
--# expr-start 10767 '<FieldAccess for has_constant at nodes.lkt:14830:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14830'
      if Var_Expr_265.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Constant_Node (Node => Var_Expr_265.Node.Type_Access_Def_F_Has_Constant, Info => Var_Expr_265.Info);
--# expr-done 10767
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_4 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_3.Node);
--# end
--# expr-done 10768
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 10769
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Explicit_Deref_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Qual_Expr
   --
      procedure Initialize_Fields_For_Qual_Expr
        (Self : Bare_Qual_Expr
         ; Qual_Expr_F_Prefix : Bare_Name
         ; Qual_Expr_F_Suffix : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Qual_Expr_F_Prefix := Qual_Expr_F_Prefix;
            Self.Qual_Expr_F_Suffix := Qual_Expr_F_Suffix;
      end Initialize_Fields_For_Qual_Expr;
   function Qual_Expr_F_Prefix
     (Node : Bare_Qual_Expr) return Bare_Name
   is
   begin
         return Node.Qual_Expr_F_Prefix;
   end;
   function Qual_Expr_F_Suffix
     (Node : Bare_Qual_Expr) return Bare_Expr
   is
   begin
         return Node.Qual_Expr_F_Suffix;
   end;
--# property-start QualExpr.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14839'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Ref_Var
  (Node : Bare_Qual_Expr
  )
   return Logic_Var
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10771 '<FieldAccess for ref_var at nodes.lkt:14839:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14839'
--# expr-start 10770 '<FieldAccess for prefix at nodes.lkt:14839:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14839'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Qual_Expr_F_Prefix;
--# expr-done 10770
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 10771
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Ref_Var;
--# end
--# property-start QualExpr.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14841'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Relative_Name
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10773 '<FieldAccess for relative_name at nodes.lkt:14841:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14841'
--# expr-start 10772 '<FieldAccess for prefix at nodes.lkt:14841:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14841'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10772
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10773
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Relative_Name;
--# end
--# property-start QualExpr.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14843'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Is_Constant
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Is_Constant;
--# end
--# property-start QualExpr.general_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14846'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_General_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1304 with Inline_Always;
            procedure Finalizer_Scope_3377 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1304 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1304;
            procedure Finalizer_Scope_3377 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3377;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10781 '<And at nodes.lkt:14847:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14847'
--# expr-start 10776 '<And at nodes.lkt:14847:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14847'
--# expr-start 10774 '<FieldAccess for xref_equation at nodes.lkt:14847:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14847'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10774
--# expr-start 10775 '<FieldAccess for all_args_xref_equation at nodes.lkt:14847:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14847'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_args_xref_equation
Fld_1 := Libadalang.Implementation.Name_P_All_Args_Xref_Equation (Node => Ent.Node, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10775
And_Pred := Create_And (Fld, Fld_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14847:9") else null)); 
--# expr-done 10776
--# expr-start 10777 '<FieldAccess for parent_name at nodes.lkt:14847:72>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14847'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_name
Fld_2 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root, E_Info => Ent.Info);
--# end
--# expr-done 10777
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_0
--# expr-start 10780 '<FieldAccess for parent_name_equation at nodes.lkt:14848:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14848'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10779 '<FieldAccess for designated_type_impl at nodes.lkt:14848:45>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14848'
--# expr-start 10778 '<FieldAccess for prefix at nodes.lkt:14848:45>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14848'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10778
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 10779
--# property-call-start Name.parent_name_equation
Fld_5 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_0.Node, Typ => Fld_4, Root => Root, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10780
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3377;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14848:100") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14847:9") else null)); 
--# expr-done 10781
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1304;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1304;
                     Finalizer_Scope_3377;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_General_Xref_Equation;
--# end
--# property-start QualExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14852'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1305 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Fld_13 : Internal_Entity;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
            procedure Finalizer_Scope_1305 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_1305;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10803 '<And at nodes.lkt:14852:37>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14852'
--# expr-start 10795 '<And at nodes.lkt:14853:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14853'
--# expr-start 10792 '<And at nodes.lkt:14854:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14854'
--# expr-start 10786 '<And at nodes.lkt:14855:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14855'
--# expr-start 10783 '<FieldAccess for xref_type_equation at nodes.lkt:14855:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14855'
--# expr-start 10782 '<FieldAccess for prefix at nodes.lkt:14855:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14855'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10782
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_1 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 10783
--# expr-start 10785 '<FieldAccess for sub_equation at nodes.lkt:14855:54>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14855'
--# expr-start 10784 '<FieldAccess for suffix at nodes.lkt:14855:54>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14855'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 10784
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 10785
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14855:17") else null)); 
--# expr-done 10786
--# expr-start 10791 '<Bind at nodes.lkt:14856:19>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14856'
--# expr-start 10788 '<FieldAccess for expected_type_var at nodes.lkt:14856:23>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14856'
--# expr-start 10787 '<FieldAccess for suffix at nodes.lkt:14856:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14856'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Qual_Expr_F_Suffix;
--# expr-done 10787
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 10788
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 10790 '<FieldAccess for ref_var at nodes.lkt:14856:56>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14856'
--# expr-start 10789 '<FieldAccess for prefix at nodes.lkt:14856:56>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14856'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Qual_Expr_F_Prefix;
--# expr-done 10789
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 10790
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result := Solver.Create_Unify (Fld_5, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14856:19") else null)); 
--# expr-done 10791
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14854:13") else null)); 
--# expr-done 10792
--# expr-start 10794 '<FieldAccess for matches_expected_type at nodes.lkt:14857:15>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14857'
--# expr-start 10793 '<FieldAccess for suffix at nodes.lkt:14857:15>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14857'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 10793
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 10794
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14853:9") else null)); 
--# expr-done 10795
--# expr-start 10802 '<If at nodes.lkt:14866:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
--# expr-start 10797 '<IsA CallStmt at nodes.lkt:14866:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
--# expr-start 10796 '<parent at nodes.lkt:14866:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 10796
Is_A := Fld_13.Node /= null 
and then Fld_13.Node.Kind in Ada_Call_Stmt_Range; 
--# expr-done 10797
if Is_A then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14866:41") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 10801 '<Bind at nodes.lkt:14866:52>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
--# expr-start 10798 '<FieldAccess for type_var at nodes.lkt:14866:56>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10798
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 10800 '<FieldAccess for ref_var at nodes.lkt:14866:73>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
--# expr-start 10799 '<FieldAccess for prefix at nodes.lkt:14866:73>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14866'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Qual_Expr_F_Prefix;
--# expr-done 10799
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_11);
--# end
--# expr-done 10800
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_1 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14866:52") else null)); 
--# expr-done 10801
   If_Result := Bind_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 10802
And_Pred_3 := Create_And (And_Pred_2, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14852:37") else null)); 
--# expr-done 10803
         Property_Result := And_Pred_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1305;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1305;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Xref_Equation;
--# end
--# property-start QualExpr.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14870'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Env
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1306 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1306 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1306;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10806 '<FieldAccess for defining_env at nodes.lkt:14871:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14871'
--# expr-start 10805 '<FieldAccess for name_designated_type at nodes.lkt:14871:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14871'
--# expr-start 10804 '<FieldAccess for prefix at nodes.lkt:14871:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14871'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10804
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 10805
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Fld_1.Info);
--# end
--# expr-done 10806
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1306;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1306;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Designated_Env;
--# end
--# property-start QualExpr.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14874'
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Env_Elements_Impl
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Qual_Expr  := Bare_Qual_Expr (Node);
     Ent : Internal_Entity_Qual_Expr :=
       Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1307 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1307 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1307;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10808 '<FieldAccess for env_elements_impl at nodes.lkt:14875:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14875'
--# expr-start 10807 '<FieldAccess for prefix at nodes.lkt:14875:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14875'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 10807
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 10808
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1307;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1307;
            raise;
      end;
   return Property_Result;
end Qual_Expr_P_Env_Elements_Impl;
--# end
   --
   --  Primitives for Bare_Reduce_Attribute_Ref
   --
      procedure Initialize_Fields_For_Reduce_Attribute_Ref
        (Self : Bare_Reduce_Attribute_Ref
         ; Reduce_Attribute_Ref_F_Prefix : Bare_Ada_Node
         ; Reduce_Attribute_Ref_F_Attribute : Bare_Identifier
         ; Reduce_Attribute_Ref_F_Args : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Reduce_Attribute_Ref_F_Prefix := Reduce_Attribute_Ref_F_Prefix;
            Self.Reduce_Attribute_Ref_F_Attribute := Reduce_Attribute_Ref_F_Attribute;
            Self.Reduce_Attribute_Ref_F_Args := Reduce_Attribute_Ref_F_Args;
      Self.Reduce_Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Reduce_Attribute_Ref;
   function Reduce_Attribute_Ref_F_Prefix
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Ada_Node
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Prefix;
   end;
   function Reduce_Attribute_Ref_F_Attribute
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Attribute;
   end;
   function Reduce_Attribute_Ref_F_Args
     (Node : Bare_Reduce_Attribute_Ref) return Bare_Assoc_List
   is
   begin
         return Node.Reduce_Attribute_Ref_F_Args;
   end;
--# property-start ReduceAttributeRef.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14885'
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Ref_Var
  (Node : Bare_Reduce_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10809 '<FieldAccess for r_ref_var at nodes.lkt:14885:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14885'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Reduce_Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 10809
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Ref_Var;
--# end
--# property-start ReduceAttributeRef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14894'
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Xref_Equation
  (Node : Bare_Reduce_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
     Ent : Internal_Entity_Reduce_Attribute_Ref :=
       Internal_Entity_Reduce_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1309 with Inline_Always;
            procedure Finalizer_Scope_3378 with Inline_Always;
            procedure Finalizer_Scope_3379 with Inline_Always;
            procedure Finalizer_Scope_3380 with Inline_Always;
      Local_Reducer : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Boolean;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Base_Id;
Fld_4 : Symbol_Type;
Fld_5 : Bare_Ada_Node;
Fld_6 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_7 : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_8 : Logic_Equation;
Scope_Result : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Scope_Result_1 : Logic_Equation;
False_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1309 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1309;
            procedure Finalizer_Scope_3378 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (False_Rel_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3378;
            procedure Finalizer_Scope_3379 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3379;
            procedure Finalizer_Scope_3380 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_3380;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10822 '<Let at nodes.lkt:14895:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14895'
--# scope-start
--# expr-start 10811 '<FieldAccess for expr at nodes.lkt:14895:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14895'
--# expr-start 10810 '<FieldAccess for args at nodes.lkt:14895:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14895'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10810
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10811
Local_Reducer := Fld_3; 
--# bind reducer Local_Reducer
--# expr-start 10821 '<LogicAny at nodes.lkt:14897:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14897'
--# expr-start 10820 '<Map at nodes.lkt:14897:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14897'
   --# expr-start 10815 '<FieldAccess for env_get at nodes.lkt:14897:9>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14897'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10813 '<FieldAccess for sym at nodes.lkt:14898:29>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14898'
--# expr-start 10812 '<Cast to BaseId at nodes.lkt:14898:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14898'
Cast_Expr := Local_Reducer; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Id
   then
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Id;
   end if;
--# expr-done 10812
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Cast_Result.Node);
--# end
--# expr-done 10813
--# expr-start 10814 '<FieldAccess for origin_node at nodes.lkt:14898:65>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14898'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10814
--# property-call-start AdaNode.env_get
Fld_6 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_4, From_Node => Fld_5);
--# end
--# expr-done 10815
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_0
--# expr-start 10816 '<Cast to BasicDecl at nodes.lkt:14900:23>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14900'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10816
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 10817 '<FieldAccess for is_valid_reducer_candidate at nodes.lkt:14901:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14901'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_valid_reducer_candidate
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Reducer_Candidate (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10817
Var_Expr_2 := Fld_7;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 10819 '<FieldAccess for xref_equation_for_reducer_candidate at nodes.lkt:14902:28>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14902'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10818 '<Cast to BasicDecl at nodes.lkt:14902:69>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14902'
Cast_Expr_2 := Item_0; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10818
--# property-call-start ReduceAttributeRef.xref_equation_for_reducer_candidate
Fld_8 := Libadalang.Implementation.Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate (Node => Ent.Node, Subp => Cast_Result_2, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 10819
Scope_Result := Fld_8;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3380;
   Result_Var := Scope_Result;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14902:102") else null)); 
   Result_Var := False_Rel;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3379;
   Result_Var_1 := Scope_Result_1;
else
   False_Rel_1 := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14903:32") else null)); 
   Result_Var_1 := False_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3378;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10820
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14897:9") else null)); 
--# expr-done 10821
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 10822
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1309;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1309;
                     Finalizer_Scope_3378;
                     Finalizer_Scope_3379;
                     Finalizer_Scope_3380;
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Xref_Equation;
--# end
--# property-start ReduceAttributeRef.xref_equation_for_reducer_candidate 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14910'
pragma Warnings (Off, "is not referenced");
 function Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate
  (Node : Bare_Reduce_Attribute_Ref
      ; Subp : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Reduce_Attribute_Ref  := Bare_Reduce_Attribute_Ref (Node);
     Ent : Internal_Entity_Reduce_Attribute_Ref :=
       Internal_Entity_Reduce_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1310 with Inline_Always;
            procedure Finalizer_Scope_3381 with Inline_Always;
      Local_Reducer : Internal_Entity_Base_Id;
Local_Initial_Value_Expression : Internal_Entity_Expr;
Local_Param_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Local_Accum_Type : Internal_Entity_Base_Type_Decl;
Local_Value_Type : Internal_Entity_Base_Type_Decl;
Var_Expr_0 : Internal_Entity_Value_Sequence;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Get_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Value_Sequence;
Fld_11 : Internal_Entity_Iterated_Assoc;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_14 : Internal_Entity_Iterated_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_17 : Internal_Entity;
Fld_18 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_20 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_22 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1310 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Local_Param_Types);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1310;
            procedure Finalizer_Scope_3381 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3381;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10862 '<Let at nodes.lkt:14911:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14911'
--# scope-start
--# expr-start 10825 '<Cast to BaseId at nodes.lkt:14911:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14911'
--# expr-start 10824 '<FieldAccess for expr at nodes.lkt:14911:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14911'
--# expr-start 10823 '<FieldAccess for args at nodes.lkt:14911:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14911'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10823
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 10824
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Id
   then
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Id;
   end if;
--# expr-done 10825
Local_Reducer := Cast_Result; 
--# bind reducer Local_Reducer
--# expr-start 10861 '<Let at nodes.lkt:14912:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14912'
--# scope-start
--# expr-start 10827 '<FieldAccess for expr at nodes.lkt:14912:40>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14912'
--# expr-start 10826 '<FieldAccess for args at nodes.lkt:14912:40>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14912'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Reduce_Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 10826
Fld_5 := Fld_4.Node;
Get_Result_1 := Get (Self, Fld_5, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 10827
Local_Initial_Value_Expression := Fld_7; 
--# bind initial_value_expression Local_Initial_Value_Expression
--# expr-start 10860 '<Let at nodes.lkt:14916:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14916'
--# scope-start
--# expr-start 10829 '<FieldAccess for param_types at nodes.lkt:14916:27>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14916'
--# expr-start 10828 '<FieldAccess for subp_spec_or_null at nodes.lkt:14916:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14916'
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 10828
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 10829
Local_Param_Types := Fld_9; Inc_Ref (Local_Param_Types);
--# bind param_types Local_Param_Types
--# expr-start 10859 '<Let at nodes.lkt:14917:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14917'
--# scope-start
--# expr-start 10830 '<at at nodes.lkt:14917:26>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14917'
Get_Result_2 := Get (Self, Local_Param_Types, 0, True); 
--# expr-done 10830
Local_Accum_Type := Get_Result_2; 
--# bind accum_type Local_Accum_Type
--# expr-start 10858 '<Let at nodes.lkt:14918:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14918'
--# scope-start
--# expr-start 10831 '<at at nodes.lkt:14918:26>' Get_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14918'
Get_Result_3 := Get (Self, Local_Param_Types, 1, True); 
--# expr-done 10831
Local_Value_Type := Get_Result_3; 
--# bind value_type Local_Value_Type
--# expr-start 10857 '<And at nodes.lkt:14920:9>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10855 '<And at nodes.lkt:14920:9>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10853 '<And at nodes.lkt:14920:9>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10850 '<And at nodes.lkt:14920:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10847 '<And at nodes.lkt:14920:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10844 '<And at nodes.lkt:14920:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10833 '<Cast to ValueSequence at nodes.lkt:14920:9>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
--# expr-start 10832 '<FieldAccess for prefix at nodes.lkt:14920:9>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14920'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity (Node => Ent.Node.Reduce_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10832
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Value_Sequence_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Value_Sequence
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_1 := No_Entity_Value_Sequence;
   end if;
--# expr-done 10833
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Value_Sequence then
   --# scope-start
   --# bind vs Var_Expr_0
--# expr-start 10841 '<And at nodes.lkt:14921:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10837 '<Bind at nodes.lkt:14921:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10836 '<FieldAccess for expected_type_var at nodes.lkt:14921:25>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10835 '<FieldAccess for expr at nodes.lkt:14921:25>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10834 '<FieldAccess for iter_assoc at nodes.lkt:14921:25>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Iterated_Assoc (Node => Var_Expr_0.Node.Value_Sequence_F_Iter_Assoc, Info => Var_Expr_0.Info);
--# expr-done 10834
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 10835
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_12.Node);
--# expr-done 10836
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
Cast_Expr := Local_Value_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_13, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14921:21") else null)); 
--# expr-done 10837
--# expr-start 10840 '<FieldAccess for matches_expected_type at nodes.lkt:14921:83>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10839 '<FieldAccess for expr at nodes.lkt:14921:83>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
--# expr-start 10838 '<FieldAccess for iter_assoc at nodes.lkt:14921:83>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14921'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Iterated_Assoc (Node => Var_Expr_0.Node.Value_Sequence_F_Iter_Assoc, Info => Var_Expr_0.Info);
--# expr-done 10838
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 10839
      if Fld_15.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_16 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_15.Node, Origin => Origin);
--# end
--# expr-done 10840
And_Pred := Create_And (Bind_Result, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14921:21") else null)); 
--# expr-done 10841
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3381;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14921:141") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 10843 '<FieldAccess for sub_equation at nodes.lkt:14922:15>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
--# expr-start 10842 '<FieldAccess for prefix at nodes.lkt:14922:15>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Create_Internal_Entity (Node => Ent.Node.Reduce_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 10842
      if Fld_17.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_17.Info);
--# end
--# expr-done 10843
And_Pred_1 := Create_And (Result_Var, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10844
--# expr-start 10846 '<Bind at nodes.lkt:14922:46>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
--# expr-start 10845 '<FieldAccess for type_var at nodes.lkt:14922:50>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 10845
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
Cast_Expr_1 := Local_Accum_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_19, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14922:46") else null)); 
--# expr-done 10846
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10847
--# expr-start 10849 '<Bind at nodes.lkt:14922:83>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
--# expr-start 10848 '<FieldAccess for ref_var at nodes.lkt:14922:87>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Local_Reducer.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Reducer.Node);
--# end
--# expr-done 10848
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Cast_Expr_2 := Subp; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_20, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14922:83") else null)); 
--# expr-done 10849
And_Pred_3 := Create_And (And_Pred_2, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10850
--# expr-start 10852 '<Bind at nodes.lkt:14922:116>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
--# expr-start 10851 '<FieldAccess for expected_type_var at nodes.lkt:14922:120>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Local_Initial_Value_Expression.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Local_Initial_Value_Expression.Node);
--# expr-done 10851
Fld_21.Value := No_Entity;
Entity_Vars.Reset (Fld_21);
Cast_Expr_3 := Local_Accum_Type; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_21, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14922:116") else null)); 
--# expr-done 10852
And_Pred_4 := Create_And (And_Pred_3, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10853
--# expr-start 10854 '<FieldAccess for sub_equation at nodes.lkt:14922:182>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Local_Initial_Value_Expression.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_22 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Initial_Value_Expression.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Initial_Value_Expression.Info);
--# end
--# expr-done 10854
And_Pred_5 := Create_And (And_Pred_4, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10855
--# expr-start 10856 '<FieldAccess for matches_expected_formal_type at nodes.lkt:14922:226>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14922'
      if Local_Initial_Value_Expression.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_23 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Local_Initial_Value_Expression.Node, Origin => Origin);
--# end
--# expr-done 10856
And_Pred_6 := Create_And (And_Pred_5, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:14920:9") else null)); 
--# expr-done 10857
Let_Result := And_Pred_6; Inc_Ref (Let_Result);
--# end
--# expr-done 10858
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10859
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10860
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10861
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 10862
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1310;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1310;
                     Finalizer_Scope_3381;
            raise;
      end;
   return Property_Result;
end Reduce_Attribute_Ref_P_Xref_Equation_For_Reducer_Candidate;
--# end
   --
   --  Primitives for Bare_Single_Tok_Node
   --
      procedure Initialize_Fields_For_Single_Tok_Node
        (Self : Bare_Single_Tok_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      end Initialize_Fields_For_Single_Tok_Node;
--# property-start SingleTokNode.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14929'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Relative_Name
  (Node : Bare_Single_Tok_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
     Ent : Internal_Entity_Single_Tok_Node :=
       Internal_Entity_Single_Tok_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Cast_Expr : Internal_Entity_Single_Tok_Node;
Cast_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Relative_Name;
--# end
--# property-start SingleTokNode.defines_subp_spec_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14937'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Defines_Subp_Spec_Var
  (Node : Bare_Single_Tok_Node
  )
   return Boolean
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Defines_Subp_Spec_Var;
--# end
--# property-start SingleTokNode.sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14942'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Sym
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10863 '<GetSymbol at nodes.lkt:14942:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14942'
Sym := Get_Symbol (Self); 
--# expr-done 10863
         Property_Result := Sym;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Sym;
--# end
--# property-start SingleTokNode.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14944'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Name_Symbol
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10864 '<GetSymbol at nodes.lkt:14944:33>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14944'
Sym := Get_Symbol (Self); 
--# expr-done 10864
         Property_Result := Sym;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Name_Symbol;
--# end
--# property-start SingleTokNode.canonical_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14946'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Canonical_Text
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 10865 '<FieldAccess for sym at nodes.lkt:14946:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14946'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Self);
--# end
--# expr-done 10865
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Canonical_Text;
--# end
--# property-start SingleTokNode.env_get_first_visible 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14953'
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Env_Get_First_Visible
  (Node : Bare_Single_Tok_Node
      ; Lex_Env : Lexical_Env
      ; Lookup_Type : Lookup_Kind
      ; From_Node : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
  )
   return Internal_Entity
is
   Self : Bare_Single_Tok_Node  := Bare_Single_Tok_Node (Node);
      --# bind self Self
   --# bind lex_env Lex_Env
   --# bind lookup_type Lookup_Type
   --# bind from_node From_Node
   --# bind no_visibility No_Visibility
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_262 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
If_Result : Boolean;
Find_Result : Internal_Entity;
            procedure Finalizer_Scope_262 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_262;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10869 '<Find at nodes.lkt:14953:116>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14953'
   --# expr-start 10867 '<FieldAccess for env_get at nodes.lkt:14953:116>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14953'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10866 '<GetSymbol at nodes.lkt:14954:18>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14954'
Sym := Get_Symbol (Self); 
--# expr-done 10866
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Lex_Env, Symbol => Sym, Lookup => Lookup_Type, From_Node => From_Node, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 10867
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_0
if No_Visibility then
   If_Result := True;
else
   --# expr-start 10868 '<FieldAccess for has_visibility at nodes.lkt:14960:14>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 10868
   If_Result := Fld_1;
end if;
      if If_Result then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when If_Result;
         end loop;
      end;
--# expr-done 10869
         Property_Result := Find_Result;
   --# end
      Finalizer_Scope_262;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_262;
            raise;
      end;
   return Property_Result;
end Single_Tok_Node_P_Env_Get_First_Visible;
--# end
   --
   --  Primitives for Bare_Base_Id
   --
      procedure Initialize_Fields_For_Base_Id
        (Self : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Base_Id;
--# property-start BaseId.scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14970'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_263 with Inline_Always;
            procedure Finalizer_Scope_3383 with Inline_Always;
      Local_Elt : Internal_Entity;
Local_Ret : Lexical_Env := Empty_Env;
Var_Expr_0 : Bare_Generic_Package_Decl;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Env_Get_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Fld_3 : Bare_Generic_Package_Internal;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_263 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Local_Ret);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_263;
            procedure Finalizer_Scope_3383 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3383;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Scope,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Lexical_Env;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 10885 '<Let at nodes.lkt:14971:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14971'
--# scope-start
--# expr-start 10873 '<EnvGet at nodes.lkt:14971:19>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14971'
--# expr-start 10870 '<GetSymbol at nodes.lkt:14972:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14972'
Sym := Get_Symbol (Self); 
--# expr-done 10870
--# expr-start 10872 '<If at nodes.lkt:14973:20>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14973'
--# expr-start 10871 '<FieldAccess for is_prefix at nodes.lkt:14973:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14973'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10871
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10872
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (If_Result), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 10873
Local_Elt := Env_Get_Result; 
--# bind elt Local_Elt
--# expr-start 10884 '<Let at nodes.lkt:14976:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
--# scope-start
--# expr-start 10879 '<If at nodes.lkt:14976:19>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
--# expr-start 10875 '<Not at nodes.lkt:14976:22>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
--# expr-start 10874 '<IsNull at nodes.lkt:14976:26>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
Is_Null := Local_Elt.Node = null; 
--# expr-done 10874
Not_Val := not (Is_Null); 
--# expr-done 10875
if Not_Val then
   --# expr-start 10877 '<IsA BasicDecl at nodes.lkt:14976:42>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
--# expr-start 10876 '<FieldAccess for node at nodes.lkt:14976:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
Fld_2 := Local_Elt.Node;
--# expr-done 10876
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Basic_Decl; 
--# expr-done 10877
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 10878 '<FieldAccess for children_env at nodes.lkt:14976:69>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14976'
      if Local_Elt.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Local_Elt.Node, E_Info => Local_Elt.Info);
--# expr-done 10878
   If_Result_2 := Fld_1;
else
   If_Result_2 := Empty_Env;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10879
Local_Ret := If_Result_2; Inc_Ref (Local_Ret);
--# bind ret Local_Ret
--# expr-start 10881 '<Cast to GenericPackageDecl at nodes.lkt:14980:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14980'
--# expr-start 10880 '<env_node at nodes.lkt:14980:9>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14980'
Env_Node := AST_Envs.Env_Node (Local_Ret); 
--# expr-done 10880
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 10881
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr_0
--# expr-start 10883 '<FieldAccess for children_env at nodes.lkt:14981:31>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14981'
--# expr-start 10882 '<FieldAccess for package_decl at nodes.lkt:14981:31>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14981'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_0.Generic_Package_Decl_F_Package_Decl;
--# expr-done 10882
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 10883
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3383;
   Result_Var := Scope_Result;
else
   Result_Var := Local_Ret;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 10884
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10885
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_263;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_263;
                     Finalizer_Scope_3383;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Lexical_Env,
                        As_Lexical_Env => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Id_P_Scope;
--# end
--# property-start BaseId.designated_env_no_overloading 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14986'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env_No_Overloading
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_264 with Inline_Always;
            procedure Finalizer_Scope_3384 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Var_Expr_266 : Internal_Entity_Basic_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_6 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Is_A : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_264 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_264;
            procedure Finalizer_Scope_3384 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_3384;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10890 '<Cast to BasicDecl at nodes.lkt:14986:55>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14986'
--# expr-start 10889 '<FieldAccess for env_get_first_visible at nodes.lkt:14986:55>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14986'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10887 '<If at nodes.lkt:14987:26>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14987'
--# expr-start 10886 '<FieldAccess for is_prefix at nodes.lkt:14987:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14987'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10886
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10887
--# expr-start 10888 '<FieldAccess for origin_node at nodes.lkt:14987:104>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14987'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10888
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1, No_Visibility => No_Visibility);
--# end
--# expr-done 10889
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10890
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 10899 '<If at nodes.lkt:14996:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14996'
--# expr-start 10891 '<IsA ObjectDecl at nodes.lkt:14996:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14996'
Is_A := Var_Expr_0.Node /= null 
and then Var_Expr_0.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 10891
if Is_A then
   --# expr-start 10892 '<FieldAccess for empty_env at nodes.lkt:14996:34>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14996'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 10892
   If_Result_2 := Fld_3;
else
   --# expr-start 10898 '<If at nodes.lkt:14996:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14996'
Var_Expr_266 := Var_Expr_0;
if Var_Expr_266 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10893 '<FieldAccess for is_package at nodes.lkt:14997:14>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14997'
      if Var_Expr_266.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_package
Fld_7 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr_266.Node);
--# end
--# expr-done 10893
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 10894 '<FieldAccess for pkg_env at nodes.lkt:14997:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14997'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.pkg_env
Fld_4 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Var_Expr_0, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10894
   If_Result_1 := Fld_4;
else
   --# expr-start 10897 '<bind at nodes.lkt:14999:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14999'
--# expr-start 10895 '<FieldAccess for origin_node at nodes.lkt:14999:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:14999'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10895
Bound_Origin := Fld_5; 
--# expr-start 10896 '<FieldAccess for defining_env at nodes.lkt:15001:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15001'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_0.Node, Origin => Bound_Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10896
Dyn_Var_Bind_Result := Fld_6; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 10897
   If_Result_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 10898
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 10899
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3384;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
         Property_Result := Result_Var_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_264;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_264;
                     Finalizer_Scope_3384;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Env_No_Overloading;
--# end
--# property-start BaseId.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15008'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_265 with Inline_Always;
            procedure Finalizer_Scope_3387 with Inline_Always;
            procedure Finalizer_Scope_3389 with Inline_Always;
      Local_Bd : Bare_Ada_Node;
Local_Env_El : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_267 : Internal_Entity_Basic_Decl;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Map_Result_1 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_9 : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_265 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_265;
            procedure Finalizer_Scope_3387 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3387;
            procedure Finalizer_Scope_3389 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_3389;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10925 '<Let at nodes.lkt:15009:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15009'
--# scope-start
--# expr-start 10902 '<Find at nodes.lkt:15009:18>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15009'
   --# expr-start 10900 '<FieldAccess for parents at nodes.lkt:15009:18>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15009'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 10900
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 10901 '<IsA GenericPackageInstantiation at nodes.lkt:15009:45>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15009'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10901
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 10902
Local_Bd := Find_Result; 
--# bind bd Local_Bd
--# expr-start 10924 '<Let at nodes.lkt:15010:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15010'
--# scope-start
--# expr-start 10907 '<Cast to BasicDecl at nodes.lkt:15010:22>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15010'
--# expr-start 10906 '<FieldAccess for env_get_first_visible at nodes.lkt:15010:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15010'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10904 '<If at nodes.lkt:15011:30>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15011'
--# expr-start 10903 '<FieldAccess for is_prefix at nodes.lkt:15011:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15011'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10903
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 10904
--# expr-start 10905 '<FieldAccess for origin_node at nodes.lkt:15011:108>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15011'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10905
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1, No_Visibility => No_Visibility);
--# end
--# expr-done 10906
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 10907
Local_Env_El := Cast_Result; 
--# bind env_el Local_Env_El
--# expr-start 10923 '<If at nodes.lkt:15015:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
Var_Expr_267 := Local_Env_El;
if Var_Expr_267 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 10908 '<FieldAccess for is_package at nodes.lkt:15015:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
      if Var_Expr_267.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.is_package
Fld_8 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr_267.Node);
--# end
--# expr-done 10908
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 10911 '<Not at nodes.lkt:15015:37>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
--# expr-start 10910 '<Eq at nodes.lkt:15015:37>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
--# expr-start 10909 '<FieldAccess for node at nodes.lkt:15015:37>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
Fld_9 := Local_Env_El.Node;
--# expr-done 10909
      Cast_Result_3 := Fld_9;
Is_Equal_1 := Cast_Result_3 = Local_Bd; 
--# expr-done 10910
Not_Val_1 := not (Is_Equal_1); 
--# expr-done 10911
   If_Result_2 := Not_Val_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 10912 '<FieldAccess for pkg_env at nodes.lkt:15015:60>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Local_Env_El, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 10912
   If_Result_3 := Fld_3;
else
   --# expr-start 10922 '<env_group at nodes.lkt:15015:86>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
--# expr-start 10921 '<Map at nodes.lkt:15015:86>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
--# expr-start 10913 '<FieldAccess for env_elements_impl at nodes.lkt:15015:86>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15015'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_4 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 10913
Var_Expr_1 := Fld_4;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind all_env_els Var_Expr_1
--# expr-start 10918 '<Map at nodes.lkt:15016:30>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15016'
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_2
--# expr-start 10916 '<Not at nodes.lkt:15019:21>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15019'
--# expr-start 10915 '<Eq at nodes.lkt:15019:21>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15019'
--# expr-start 10914 '<FieldAccess for node at nodes.lkt:15019:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15019'
Fld_5 := Item_2.Node;
--# expr-done 10914
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Local_Bd; 
--# expr-done 10915
Not_Val := not (Is_Equal); 
--# expr-done 10916
if Not_Val then
   --# expr-start 10917 '<FieldAccess for has_visibility at nodes.lkt:15020:23>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15020'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_2);
--# end
--# expr-done 10917
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
         if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_2;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 10918
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3387;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_1_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_3
      --# expr-start 10920 '<FieldAccess for defining_env at nodes.lkt:15022:22>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15022'
--# expr-start 10919 '<Cast to BasicDecl at nodes.lkt:15022:22>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15022'
Cast_Expr := Item_3; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 10919
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Cast_Result_2.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 10920
         declare
            Item_To_Append : constant Lexical_Env := Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3389;
         end loop;
      end;
         Map_Result_1 := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_1_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 10921
Group_Env := Group (Map_Result_1, No_Metadata); 
--# expr-done 10922
   If_Result_3 := Group_Env;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 10923
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 10924
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10925
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_265;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_265;
                     Finalizer_Scope_3387;
                     Finalizer_Scope_3389;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Env;
--# end
--# property-start BaseId.pkg_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15034'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Pkg_Env
  (Node : Bare_Base_Id
      ; From_Pkg : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind from_pkg From_Pkg
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_266 with Inline_Always;
            procedure Finalizer_Scope_3392 with Inline_Always;
            procedure Finalizer_Scope_3393 with Inline_Always;
            procedure Finalizer_Scope_3394 with Inline_Always;
            procedure Finalizer_Scope_3395 with Inline_Always;
            procedure Finalizer_Scope_3396 with Inline_Always;
            procedure Finalizer_Scope_3398 with Inline_Always;
      Local_Pkg : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Package_Renaming_Decl;
Local_Bd : Internal_Entity_Basic_Decl;
Local_Is_Inst_From_Formal : Boolean;
Local_Env : Lexical_Env := Empty_Env;
Local_Private_Part_Env : Lexical_Env := Empty_Env;
Var_Expr_1 : Internal_Entity;
Local_Package_Body_Env : Lexical_Env := Empty_Env;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Local_Formals_Env : Lexical_Env := Empty_Env;
Var_Expr_4 : Internal_Entity_Generic_Package_Decl;
Var_Expr_268 : Bare_Ada_Node;
Var_Expr : Bare_Ada_Node;
Var_Expr_269 : Bare_Ada_Node;
Var_Expr_5 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Info;
Fld_7 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_8 : Lexical_Env := Empty_Env;
Fld_9 : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_11 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Fld_12 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Fld_13 : Lexical_Env := Empty_Env;
Is_A_2 : Boolean;
If_Result_4 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Fld_14 : Internal_Entity_Generic_Formal_Part;
Fld_15 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_4 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Array_Lit_2 : Lexical_Env_Array_Access;
Group_Env_2 : Lexical_Env := Empty_Env;
Is_A_3 : Boolean;
If_Result_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_16 : Lexical_Env := Empty_Env;
Scope_Result_5 : Lexical_Env := Empty_Env;
Result_Var_5 : Lexical_Env := Empty_Env;
Scope_Result_6 : Bare_Ada_Node;
Cast_Expr_3 : Bare_Base_Id;
Cast_Result_3 : Bare_Ada_Node;
Result_Var_6 : Bare_Ada_Node;
Fld_17 : Lexical_Env := Empty_Env;
Fld_18 : Boolean;
If_Result_6 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Not_Val : Boolean;
Env_Node_1 : Bare_Ada_Node;
Fld_19 : Lexical_Env := Empty_Env;
Scope_Result_7 : Lexical_Env := Empty_Env;
Result_Var_7 : Lexical_Env := Empty_Env;
Scope_Result_8 : Bare_Ada_Node;
Cast_Expr_4 : Bare_Base_Id;
Cast_Result_4 : Bare_Ada_Node;
Result_Var_8 : Bare_Ada_Node;
Fld_20 : Lexical_Env := Empty_Env;
Fld_21 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
Let_Result_4 : Lexical_Env := Empty_Env;
Let_Result_5 : Lexical_Env := Empty_Env;
Let_Result_6 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_266 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Local_Env);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_Private_Part_Env);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Local_Package_Body_Env);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Local_Formals_Env);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Group_Env_2);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Fld_17);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (Fld_20);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_266;
            procedure Finalizer_Scope_3392 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_3392;
            procedure Finalizer_Scope_3393 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Env_Get_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_4);
            end Finalizer_Scope_3393;
            procedure Finalizer_Scope_3394 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_3394;
            procedure Finalizer_Scope_3395 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_3395;
            procedure Finalizer_Scope_3396 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_3396;
            procedure Finalizer_Scope_3398 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_3398;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 10985 '<Let at nodes.lkt:15038:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15038'
--# scope-start
--# expr-start 10926 '<Cast to PackageRenamingDecl at nodes.lkt:15038:19>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15038'
Cast_Expr := From_Pkg; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 10926
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 10927 '<FieldAccess for final_renamed_package at nodes.lkt:15039:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15039'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 10927
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := From_Pkg;
end if;
Local_Pkg := Result_Var; 
--# bind pkg Local_Pkg
--# expr-start 10984 '<Let at nodes.lkt:15047:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15047'
--# scope-start
--# expr-start 10930 '<FieldAccess for unshed_rebindings at nodes.lkt:15047:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15047'
      if Local_Pkg.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10929 '<FieldAccess for rebindings at nodes.lkt:15047:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15047'
--# expr-start 10928 '<FieldAccess for info at nodes.lkt:15047:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15047'
Fld_1 := Ent.Info;
--# expr-done 10928
Fld_2 := Fld_1.Rebindings;
--# expr-done 10929
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Local_Pkg.Node, Rebindings => Fld_2, E_Info => Local_Pkg.Info);
--# end
--# expr-done 10930
Local_Bd := Fld_3; 
--# bind bd Local_Bd
--# expr-start 10983 '<Let at nodes.lkt:15054:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15054'
--# scope-start
--# expr-start 10931 '<IsA GenericPackageInstantiation at nodes.lkt:15054:35>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15054'
Is_A := Local_Pkg.Node /= null 
and then Local_Pkg.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10931
if Is_A then
--# expr-start 10933 '<FieldAccess for from_rebound at nodes.lkt:15055:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15055'
--# expr-start 10932 '<FieldAccess for info at nodes.lkt:15055:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15055'
Fld_4 := From_Pkg.Info;
--# expr-done 10932
Fld_5 := Fld_4.From_Rebound;
--# expr-done 10933
if Fld_5 then
   If_Result := True;
else
   --# expr-start 10935 '<FieldAccess for from_rebound at nodes.lkt:15055:43>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15055'
--# expr-start 10934 '<FieldAccess for info at nodes.lkt:15055:43>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15055'
Fld_6 := Local_Pkg.Info;
--# expr-done 10934
Fld_7 := Fld_6.From_Rebound;
--# expr-done 10935
   If_Result := Fld_7;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Local_Is_Inst_From_Formal := If_Result_1; 
--# bind is_inst_from_formal Local_Is_Inst_From_Formal
--# expr-start 10982 '<Let at nodes.lkt:15057:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
--# scope-start
--# expr-start 10940 '<If at nodes.lkt:15057:19>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
--# expr-start 10936 '<IsA GenericPackageInstantiation at nodes.lkt:15057:22>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
Is_A_1 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 10936
if Is_A_1 then
   If_Result_2 := Local_Is_Inst_From_Formal;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 10938 '<FieldAccess for defining_env_impl at nodes.lkt:15057:85>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
--# expr-start 10937 '<Cast to GenericPackageInstantiation at nodes.lkt:15057:85>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
Cast_Expr_1 := Local_Bd; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 10937
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld_8 := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Cast_Result_1.Node, Inst_From_Formal => True, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 10938
   If_Result_3 := Fld_8;
else
   --# expr-start 10939 '<FieldAccess for defining_env at nodes.lkt:15057:149>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15057'
      if Local_Bd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Local_Bd.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => No_Bare_Ada_Node, E_Info => Local_Bd.Info);
--# end
--# expr-done 10939
   If_Result_3 := Fld_9;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 10940
Local_Env := If_Result_3; Inc_Ref (Local_Env);
--# bind env Local_Env
--# expr-start 10981 '<Let at nodes.lkt:15060:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15060'
--# scope-start
--# expr-start 10943 '<at at nodes.lkt:15060:32>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15060'
--# expr-start 10942 '<EnvGet at nodes.lkt:15060:32>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15060'
--# expr-start 10941 '<Symbol __privatepart at nodes.lkt:15061:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15061'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Privatepart); 
--# expr-done 10941
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Local_Env, Key => Thin (Sym), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10942
Get_Result := Get (Self, Env_Get_Result, 0, True); 
--# expr-done 10943
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 10944 '<FieldAccess for children_env at nodes.lkt:15063:21>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15063'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 10944
Scope_Result_1 := Fld_10;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3392;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Local_Env;
end if;
      Inc_Ref (Result_Var_1);
Local_Private_Part_Env := Result_Var_1; Inc_Ref (Local_Private_Part_Env);
--# bind private_part_env Local_Private_Part_Env
--# expr-start 10980 '<Let at nodes.lkt:15065:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15065'
--# scope-start
--# expr-start 10947 '<at at nodes.lkt:15065:32>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15065'
--# expr-start 10946 '<EnvGet at nodes.lkt:15065:32>' Env_Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15065'
--# expr-start 10945 '<Symbol __nextpart at nodes.lkt:15066:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15066'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 10945
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Local_Private_Part_Env, Key => Thin (Sym_1), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10946
Get_Result_1 := Get (Self, Env_Get_Result_1, 0, True); 
--# expr-done 10947
Var_Expr_2 := Get_Result_1;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_2
--# expr-start 10955 '<If at nodes.lkt:15071:13>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15071'
--# expr-start 10948 '<IsA PackageBodyStub at nodes.lkt:15071:16>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15071'
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Package_Body_Stub_Range; 
--# expr-done 10948
if Is_A_2 then
--# expr-start 10952 '<at at nodes.lkt:15071:43>' Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15071'
--# expr-start 10951 '<EnvGet at nodes.lkt:15071:43>' Env_Get_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15071'
--# expr-start 10949 '<FieldAccess for children_env at nodes.lkt:15071:43>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15071'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 10949
--# expr-start 10950 '<Symbol __nextpart at nodes.lkt:15072:17>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15072'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Nextpart); 
--# expr-done 10950
Env_Get_Result_2 := Construct_Entity_Array (AST_Envs.Get (Self => Fld_11, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 10951
Get_Result_2 := Get (Self, Env_Get_Result_2, 0, True); 
--# expr-done 10952
Var_Expr_3 := Get_Result_2;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_3
--# expr-start 10953 '<FieldAccess for children_env at nodes.lkt:15073:30>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15073'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 10953
Scope_Result_2 := Fld_12;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3394;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_4 := Result_Var_2;
else
   --# expr-start 10954 '<FieldAccess for children_env at nodes.lkt:15073:54>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15073'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 10954
   If_Result_4 := Fld_13;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 10955
Scope_Result_3 := If_Result_4;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3393;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Local_Package_Body_Env := Result_Var_3; Inc_Ref (Local_Package_Body_Env);
--# bind package_body_env Local_Package_Body_Env
--# expr-start 10979 '<Let at nodes.lkt:15075:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15075'
--# scope-start
--# expr-start 10956 '<Cast to GenericPackageDecl at nodes.lkt:15075:27>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15075'
Cast_Expr_2 := Local_Bd; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 10956
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Generic_Package_Decl then
   --# scope-start
   --# bind pkg_g Var_Expr_4
--# expr-start 10958 '<FieldAccess for children_env at nodes.lkt:15076:24>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15076'
--# expr-start 10957 '<FieldAccess for formal_part at nodes.lkt:15076:24>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15076'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_4.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_4.Info);
--# expr-done 10957
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 10958
Scope_Result_4 := Fld_15;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3395;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := Empty_Env;
end if;
      Inc_Ref (Result_Var_4);
Local_Formals_Env := Result_Var_4; Inc_Ref (Local_Formals_Env);
--# bind formals_env Local_Formals_Env
--# expr-start 10978 '<If at nodes.lkt:15081:9>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15081'
--# expr-start 10960 '<Not at nodes.lkt:15081:12>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15081'
--# expr-start 10959 '<Eq at nodes.lkt:15081:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15081'
Is_Equal := Equivalent (Local_Package_Body_Env, Empty_Env); 
--# expr-done 10959
Not_Val := not (Is_Equal); 
--# expr-done 10960
if Not_Val then
   --# expr-start 10964 '<FieldAccess for is_children_env at nodes.lkt:15088:13>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15088'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10961 '<env_node at nodes.lkt:15089:17>' Env_Node_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15089'
Env_Node_1 := AST_Envs.Env_Node (Local_Package_Body_Env); 
--# expr-done 10961
Var_Expr_268 := Env_Node_1;
if Var_Expr_268 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 10962 '<FieldAccess for children_env at nodes.lkt:15089:17>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15089'
      if Var_Expr_268 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Children_Env (Node => Var_Expr_268);
--# expr-done 10962
Scope_Result_7 := Fld_19;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3398;
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := Empty_Env;
end if;
      Inc_Ref (Result_Var_7);
--# expr-start 10963 '<FieldAccess for node_env at nodes.lkt:15089:60>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15089'
Var_Expr := Origin;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_8 := Var_Expr;
   --# end
   Result_Var_8 := Scope_Result_8;
else
Cast_Expr_4 := Self; 
      Cast_Result_4 := Cast_Expr_4;
   Result_Var_8 := Cast_Result_4;
end if;
      if Result_Var_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Node_Env (Node => Result_Var_8);
--# expr-done 10963
--# property-call-start AdaNode.is_children_env
Fld_21 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Result_Var_7, Current_Env => Fld_20);
--# end
--# expr-done 10964
   If_Result_7 := Fld_21;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
   --# expr-start 10966 '<env_group at nodes.lkt:15091:16>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15091'
--# expr-start 10965 '<ArrayLiteral at nodes.lkt:15091:16>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15091'
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Local_Package_Body_Env, 2 => Local_Private_Part_Env, 3 => Local_Env, 4 => Local_Formals_Env)); 
--# expr-done 10965
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 10966
   If_Result_8 := Group_Env;
else
   --# expr-start 10977 '<If at nodes.lkt:15081:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15081'
--# expr-start 10970 '<FieldAccess for is_children_env at nodes.lkt:15096:14>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15096'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10967 '<env_node at nodes.lkt:15097:13>' Env_Node 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15097'
Env_Node := AST_Envs.Env_Node (Local_Private_Part_Env); 
--# expr-done 10967
Var_Expr_269 := Env_Node;
if Var_Expr_269 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 10968 '<FieldAccess for children_env at nodes.lkt:15097:13>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15097'
      if Var_Expr_269 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Libadalang.Implementation.Children_Env (Node => Var_Expr_269);
--# expr-done 10968
Scope_Result_5 := Fld_16;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3396;
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := Empty_Env;
end if;
      Inc_Ref (Result_Var_5);
--# expr-start 10969 '<FieldAccess for node_env at nodes.lkt:15097:56>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15097'
Var_Expr_5 := Origin;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_6 := Var_Expr_5;
   --# end
   Result_Var_6 := Scope_Result_6;
else
Cast_Expr_3 := Self; 
      Cast_Result_3 := Cast_Expr_3;
   Result_Var_6 := Cast_Result_3;
end if;
      if Result_Var_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Node_Env (Node => Result_Var_6);
--# expr-done 10969
--# property-call-start AdaNode.is_children_env
Fld_18 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Result_Var_5, Current_Env => Fld_17);
--# end
--# expr-done 10970
if Fld_18 then
   --# expr-start 10972 '<env_group at nodes.lkt:15098:16>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15098'
--# expr-start 10971 '<ArrayLiteral at nodes.lkt:15098:16>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15098'
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Local_Private_Part_Env, 2 => Local_Env, 3 => Local_Formals_Env)); 
--# expr-done 10971
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 10972
   If_Result_6 := Group_Env_1;
else
   --# expr-start 10976 '<If at nodes.lkt:15081:9>' If_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15081'
--# expr-start 10973 '<IsA GenericPackageDecl at nodes.lkt:15103:14>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15103'
Is_A_3 := Local_Bd.Node /= null 
and then Local_Bd.Node.Kind in Ada_Generic_Package_Decl_Range; 
--# expr-done 10973
if Is_A_3 then
   --# expr-start 10975 '<env_group at nodes.lkt:15103:44>' Group_Env_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15103'
--# expr-start 10974 '<ArrayLiteral at nodes.lkt:15103:44>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15103'
Array_Lit_2 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Local_Env, 2 => Local_Formals_Env)); 
--# expr-done 10974
Group_Env_2 := Group (Array_Lit_2, No_Metadata); 
--# expr-done 10975
   If_Result_5 := Group_Env_2;
else
   If_Result_5 := Local_Env;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 10976
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 10977
   If_Result_8 := If_Result_6;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 10978
Let_Result := If_Result_8; Inc_Ref (Let_Result);
--# end
--# expr-done 10979
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 10980
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 10981
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 10982
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 10983
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 10984
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 10985
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_266;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_266;
                     Finalizer_Scope_3392;
                     Finalizer_Scope_3393;
                     Finalizer_Scope_3394;
                     Finalizer_Scope_3395;
                     Finalizer_Scope_3396;
                     Finalizer_Scope_3398;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Pkg_Env;
--# end
--# property-start BaseId.parent_scope 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15111'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Parent_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := Env;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Id_P_Parent_Scope;
--# end
--# property-start BaseId.designated_type_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15114'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3410 with Inline_Always;
      Var_Expr_0 : Internal_Entity;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity;
Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Task_Body;
Match_2 : Internal_Entity_Protected_Body;
Match_3 : Internal_Entity_Task_Body_Stub;
Match_4 : Internal_Entity_Protected_Body_Stub;
Match_5 : Internal_Entity;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Internal_Entity_Generic_Formal_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Symbol_Type;
Fld_5 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_6 : Symbol_Type;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Info;
Fld_10 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Result_Var : Internal_Entity;
Fld_11 : Internal_Entity_Task_Type_Decl;
Fld_12 : Internal_Entity_Protected_Type_Decl;
Fld_13 : Internal_Entity_Body_Node;
Cast_Result_2 : Internal_Entity_Task_Body;
Fld_14 : Internal_Entity_Task_Type_Decl;
Fld_15 : Internal_Entity_Body_Node;
Cast_Result_3 : Internal_Entity_Protected_Body;
Fld_16 : Internal_Entity_Protected_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Task_Body;
Cast_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity_Protected_Body;
Cast_Result_8 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity_Task_Body_Stub;
Cast_Result_10 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity_Protected_Body_Stub;
Cast_Result_12 : Internal_Entity_Base_Type_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_13 : Internal_Entity;
Let_Result_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_7 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_8 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity;
Cast_Result_14 : Internal_Entity_Generic_Formal_Type_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_9 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_19 : Internal_Entity_Generic_Instantiation_Array_Access;
Fld_20 : Internal_Entity_Generic_Decl;
Fld_21 : Bare_Generic_Decl;
Fld_22 : Internal_Entity_Basic_Decl;
Fld_23 : Bare_Basic_Decl;
Cast_Result_15 : Bare_Basic_Decl;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_10 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Scope_Result_11 : Internal_Entity_Base_Type_Decl;
Result_Var_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_12 : Internal_Entity_Base_Type_Decl;
Result_Var_6 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_3410 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_3410;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 10989 '<FieldAccess for env_get_first_visible at nodes.lkt:15115:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10987 '<If at nodes.lkt:15117:25>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15117'
--# expr-start 10986 '<FieldAccess for is_prefix at nodes.lkt:15117:28>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15117'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 10986
if Fld then
   If_Result := Recursive;
else
   If_Result := Minimal;
end if;
--# expr-done 10987
--# expr-start 10988 '<FieldAccess for origin_node at nodes.lkt:15116:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15116'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10988
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1, No_Visibility => False);
--# end
--# expr-done 10989
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr_0
--# expr-start 10990 '<Cast to BaseTypeDecl at nodes.lkt:15121:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15121'
Cast_Expr := Var_Expr_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 10990
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 11002 '<If at nodes.lkt:15122:24>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15122'
--# expr-start 10991 '<IsNull at nodes.lkt:15122:27>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15122'
Is_Null := Origin = null; 
--# expr-done 10991
if Is_Null then
   --# expr-start 10995 '<bind at nodes.lkt:15126:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15126'
--# expr-start 10992 '<FieldAccess for origin_node at nodes.lkt:15126:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15126'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 10992
Bound_Origin := Fld_3; 
--# expr-start 10994 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:15128:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15128'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10993 '<FieldAccess for name_symbol at nodes.lkt:15129:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15129'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_1.Node);
--# end
--# expr-done 10993
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Var_Expr_1.Node, Sym => Fld_4, Seq => False, Origin => Bound_Origin, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10994
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 10995
   If_Result_2 := Dyn_Var_Bind_Result;
else
   --# expr-start 11001 '<If at nodes.lkt:15132:22>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15132'
--# expr-start 10997 '<FieldAccess for from_rebound at nodes.lkt:15132:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15132'
--# expr-start 10996 '<FieldAccess for info at nodes.lkt:15132:25>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15132'
Fld_9 := Ent.Info;
--# expr-done 10996
Fld_10 := Fld_9.From_Rebound;
--# expr-done 10997
if Fld_10 then
   --# expr-start 10999 '<FieldAccess for most_visible_forward_part_for_name at nodes.lkt:15137:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15137'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 10998 '<FieldAccess for name_symbol at nodes.lkt:15138:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15138'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_1.Node);
--# end
--# expr-done 10998
--# property-call-start BasicDecl.most_visible_forward_part_for_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Forward_Part_For_Name (Node => Var_Expr_1.Node, Sym => Fld_6, Seq => False, Origin => Origin, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 10999
   If_Result_1 := Fld_7;
else
   --# expr-start 11000 '<FieldAccess for most_visible_part at nodes.lkt:15146:21>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15146'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.most_visible_part
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Most_Visible_Part (Node => Var_Expr_1.Node, Origin => Origin, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11000
   If_Result_1 := Fld_8;
end if;
--# expr-done 11001
   If_Result_2 := If_Result_1;
end if;
--# expr-done 11002
Scope_Result := If_Result_2;
   --# end
      Cast_Result_1 := Create_Internal_Entity
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result_1;
else
   Result_Var := Var_Expr_0;
end if;
Var_Expr_2 := Result_Var;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind v1 Var_Expr_2
--# expr-start 11011 '<Match at nodes.lkt:15149:25>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15149'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Var_Expr_2; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind tb Match_1
--# expr-start 11003 '<FieldAccess for task_type at nodes.lkt:15151:42>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15151'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld_11 := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 11003
      Cast_Result_6 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
Let_Result_1 := Cast_Result_6; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Protected_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind pb Match_2
--# expr-start 11004 '<FieldAccess for protected_type at nodes.lkt:15152:47>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15152'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ProtectedBody.protected_type
Fld_12 := Libadalang.Implementation.Protected_Body_P_Protected_Type (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 11004
      Cast_Result_8 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
Let_Result_2 := Cast_Result_8; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Task_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_9; 
--# bind tbs Match_3
--# expr-start 11007 '<FieldAccess for task_type at nodes.lkt:15154:25>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15154'
--# expr-start 11006 '<Cast to TaskBody at nodes.lkt:15154:25>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15154'
--# expr-start 11005 '<FieldAccess for body_part_for_decl at nodes.lkt:15154:25>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15154'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Match_3.Node, Imprecise_Fallback => False, E_Info => Match_3.Info);
--# end
--# expr-done 11005
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Task_Body_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Task_Body
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_2 := No_Entity_Task_Body;
   end if;
--# expr-done 11006
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TaskBody.task_type
Fld_14 := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 11007
      Cast_Result_10 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
Let_Result_3 := Cast_Result_10; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Protected_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_11; 
--# bind pbs Match_4
--# expr-start 11010 '<FieldAccess for protected_type at nodes.lkt:15156:25>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15156'
--# expr-start 11009 '<Cast to ProtectedBody at nodes.lkt:15156:25>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15156'
--# expr-start 11008 '<FieldAccess for body_part_for_decl at nodes.lkt:15156:25>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15156'
      if Match_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Match_4.Node, Imprecise_Fallback => False, E_Info => Match_4.Info);
--# end
--# expr-done 11008
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Protected_Body_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Protected_Body
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_3 := No_Entity_Protected_Body;
   end if;
--# expr-done 11009
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ProtectedBody.protected_type
Fld_16 := Libadalang.Implementation.Protected_Body_P_Protected_Type (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 11010
      Cast_Result_12 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
Let_Result_4 := Cast_Result_12; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abort_Absent .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Package_Body_Stub | Ada_Subp_Body_Stub | Ada_Entry_Body .. Ada_Package_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_13; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 11011
Scope_Result_7 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_7;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_8 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_8;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Var_Expr_3 := Result_Var_2;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind type Var_Expr_3
--# expr-start 11013 '<Cast to GenericFormalTypeDecl at nodes.lkt:15170:13>' Cast_Result_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15170'
--# expr-start 11012 '<parent at nodes.lkt:15170:13>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15170'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Libadalang.Implementation.Parent (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 11012
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Generic_Formal_Type_Decl_Range
   then
      Cast_Result_14 := Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Cast_Result_14 := No_Entity_Generic_Formal_Type_Decl;
   end if;
--# expr-done 11013
Var_Expr_4 := Cast_Result_14;
if Var_Expr_4 /= No_Entity_Generic_Formal_Type_Decl then
   --# scope-start
   --# bind formal_decl Var_Expr_4
--# expr-start 11022 '<If at nodes.lkt:15172:21>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15172'
--# expr-start 11020 '<AnyQuantifier at nodes.lkt:15172:24>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15172'
   --# expr-start 11014 '<FieldAccess for generic_instantiations at nodes.lkt:15172:24>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15172'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld_19 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 11014
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Generic_Instantiation_Array_Access := Fld_19;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind inst Item_5
      --# expr-start 11019 '<Eq at nodes.lkt:15173:35>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15173'
--# expr-start 11016 '<FieldAccess for node at nodes.lkt:15173:35>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15173'
--# expr-start 11015 '<FieldAccess for designated_generic_decl at nodes.lkt:15173:35>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15173'
      if Item_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_20 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Item_5.Node, E_Info => Item_5.Info);
--# end
--# expr-done 11015
Fld_21 := Fld_20.Node;
--# expr-done 11016
      Cast_Result_15 := Fld_21;
--# expr-start 11018 '<FieldAccess for node at nodes.lkt:15174:25>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15174'
--# expr-start 11017 '<FieldAccess for parent_decl at nodes.lkt:15174:25>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15174'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld_22 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 11017
Fld_23 := Fld_22.Node;
--# expr-done 11018
Is_Equal := Cast_Result_15 = Fld_23; 
--# expr-done 11019
      Quantifier_Result := Is_Equal;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 11020
if Quantifier_Result then
--# expr-start 11021 '<FieldAccess for default_type at nodes.lkt:15176:26>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15176'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start GenericFormalTypeDecl.default_type
Fld_18 := Libadalang.Implementation.Generic_Formal_Type_Decl_P_Default_Type (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 11021
Var_Expr_5 := Fld_18;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_9 := Var_Expr_5;
   --# end
   Result_Var_3 := Scope_Result_9;
else
   Result_Var_3 := Var_Expr_3;
end if;
   If_Result_3 := Result_Var_3;
else
   If_Result_3 := Var_Expr_3;
end if;
--# expr-done 11022
Scope_Result_10 := If_Result_3;
   --# end
      Finalizer_Scope_3410;
   Result_Var_4 := Scope_Result_10;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_11 := Var_Expr;
   --# end
   Result_Var_5 := Scope_Result_11;
else
   Result_Var_5 := Var_Expr_3;
end if;
Scope_Result_12 := Result_Var_5;
   --# end
   Result_Var_6 := Scope_Result_12;
else
   Result_Var_6 := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3410;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Designated_Type_Impl;
--# end
--# property-start BaseId.all_env_els_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15182'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_All_Env_Els_Impl
  (Node : Bare_Base_Id
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_269 with Inline_Always;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Bare_Ada_Node;
If_Result_2 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_269 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_269;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11030 '<FieldAccess for env_get at nodes.lkt:15182:156>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15182'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11023 '<FieldAccess for name_symbol at nodes.lkt:15183:14>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 11023
--# expr-start 11025 '<If at nodes.lkt:15183:41>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
--# expr-start 11024 '<FieldAccess for is_prefix at nodes.lkt:15183:44>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld_1 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 11024
if Fld_1 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 11025
--# expr-start 11029 '<If at nodes.lkt:15183:119>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
if Seq then
   --# expr-start 11028 '<If at nodes.lkt:15183:131>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
--# expr-start 11027 '<Not at nodes.lkt:15183:134>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
--# expr-start 11026 '<IsNull at nodes.lkt:15183:138>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15183'
Is_Null := Seq_From = null; 
--# expr-done 11026
Not_Val := not (Is_Null); 
--# expr-done 11027
if Not_Val then
   If_Result_1 := Seq_From;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 11028
   If_Result_2 := If_Result_1;
else
   If_Result_2 := No_Bare_Ada_Node;
end if;
--# expr-done 11029
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld, Lookup => If_Result, From_Node => If_Result_2, Categories => Categories);
--# end
--# expr-done 11030
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_269;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_269;
            raise;
      end;
   return Property_Result;
end Base_Id_P_All_Env_Els_Impl;
--# end
--# property-start BaseId.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15188'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_270 with Inline_Always;
            procedure Finalizer_Scope_3415 with Inline_Always;
            procedure Finalizer_Scope_3421 with Inline_Always;
            procedure Finalizer_Scope_3422 with Inline_Always;
      Local_Items : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Anonymous_Expr_Decl;
Var_Expr_3 : Internal_Entity;
Local_Pc : Internal_Entity_Call_Expr;
Local_Is_Prefix : Boolean;
Var_Expr_4 : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Subp_Spec;
Match_2 : Internal_Entity;
Var_Expr_10 : Boolean;
Var_Expr_11 : Internal_Entity;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Boolean;
If_Result_1 : Ref_Categories;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Anonymous_Expr_Decl;
Fld_4 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr_1 : Internal_Entity_Base_Id;
Cast_Result_1 : Internal_Entity;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_8 : Internal_Entity_Call_Expr;
Fld_9 : Boolean;
Not_Val : Boolean;
Fld_10 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_11 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Fld_12 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Defining_Name;
Fld_14 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_15 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Find_Result_1 : Internal_Entity;
Array_Lit : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
If_Result_2 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Fld_16 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Assoc_List;
Fld_17 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_18 : Internal_Entity_Base_Subp_Spec;
Fld_19 : Boolean;
Scope_Result_4 : Boolean;
Fld_20 : Internal_Entity_Base_Type_Decl;
Fld_21 : Boolean;
Result_Var_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_5 : Internal_Entity_Base_Type_Decl;
Let_Result : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_1 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_7 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Scope_Result_8 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Not_Val_1 : Boolean;
Map_Result_3 : Internal_Entity_Array_Access;
Result_Var_5 : Internal_Entity_Array_Access;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result_3 : Internal_Entity_Array_Access;
Let_Result_4 : Internal_Entity_Array_Access;
Let_Result_5 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_270 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Items);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_270;
            procedure Finalizer_Scope_3415 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3415;
            procedure Finalizer_Scope_3421 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3421;
            procedure Finalizer_Scope_3422 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_3422;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Env_Elements_Impl,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Entity_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 11081 '<Let at nodes.lkt:15189:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15189'
--# scope-start
--# expr-start 11045 '<Map at nodes.lkt:15189:21>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15189'
   --# expr-start 11037 '<FieldAccess for env_get at nodes.lkt:15189:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15189'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11031 '<GetSymbol at nodes.lkt:15190:18>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15190'
Sym := Get_Symbol (Self); 
--# expr-done 11031
--# expr-start 11033 '<If at nodes.lkt:15190:38>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15190'
--# expr-start 11032 '<FieldAccess for is_prefix at nodes.lkt:15190:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15190'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 11032
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 11033
--# expr-start 11034 '<FieldAccess for origin_node at nodes.lkt:15191:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15191'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 11034
--# expr-start 11036 '<If at nodes.lkt:15191:54>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15191'
--# expr-start 11035 '<FieldAccess for can_designate_primitive at nodes.lkt:15191:57>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15191'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.can_designate_primitive
Fld_2 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Self);
--# end
--# expr-done 11035
if Fld_2 then
   If_Result_1 := (Inherited_Primitives => True, Nocat => True);
else
   If_Result_1 := (Inherited_Primitives => False, Nocat => True);
end if;
--# expr-done 11036
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Sym, Lookup => If_Result, From_Node => Fld_1, Categories => If_Result_1);
--# end
--# expr-done 11037
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
--# expr-start 11038 '<Cast to AnonymousExprDecl at nodes.lkt:15193:20>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15193'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Expr_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Anonymous_Expr_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Anonymous_Expr_Decl;
   end if;
--# expr-done 11038
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Anonymous_Expr_Decl then
   --# scope-start
   --# bind aed Var_Expr_1
--# expr-start 11041 '<Find at nodes.lkt:15194:26>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15194'
   --# expr-start 11039 '<FieldAccess for parents at nodes.lkt:15194:26>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15194'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 11039
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_4;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_2
      --# expr-start 11040 '<IsA GenericFormal at nodes.lkt:15194:53>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15194'
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Generic_Formal; 
--# expr-done 11040
      if Is_A then
         Find_Result := Item_2; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11041
Var_Expr_3 := Find_Result;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind _ Var_Expr_3
--# expr-start 11044 '<FieldAccess for is_directly_reachable at nodes.lkt:15197:28>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15197'
--# expr-start 11043 '<FieldAccess for formal_decl at nodes.lkt:15197:28>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15197'
--# expr-start 11042 '<FieldAccess for get_formal at nodes.lkt:15197:28>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15197'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AnonymousExprDecl.get_formal
Fld_5 := Libadalang.Implementation.Anonymous_Expr_Decl_P_Get_Formal (Node => Var_Expr_1.Node, Imprecise_Fallback => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11042
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_6 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 11043
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start BasicDecl.is_directly_reachable
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Is_Directly_Reachable (Node => Fld_6.Node, Origin => Cast_Result_1);
--# end
--# expr-done 11044
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_3415;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := True;
end if;
         if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11045
Local_Items := Map_Result; Inc_Ref (Local_Items);
--# bind items Local_Items
--# expr-start 11080 '<Let at nodes.lkt:15203:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15203'
--# scope-start
--# expr-start 11046 '<FieldAccess for parent_callexpr at nodes.lkt:15203:18>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15203'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.parent_callexpr
Fld_8 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11046
Local_Pc := Fld_8; 
--# bind pc Local_Pc
--# expr-start 11079 '<Let at nodes.lkt:15204:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15204'
--# scope-start
--# expr-start 11048 '<Not at nodes.lkt:15204:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15204'
--# expr-start 11047 '<FieldAccess for is_suffix at nodes.lkt:15204:29>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15204'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld_9 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 11047
Not_Val := not (Fld_9); 
--# expr-done 11048
Local_Is_Prefix := Not_Val; 
--# bind is_prefix Local_Is_Prefix
--# expr-start 11078 '<bind at nodes.lkt:15207:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15207'
--# expr-start 11049 '<FieldAccess for origin_node at nodes.lkt:15207:27>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15207'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_10 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 11049
Bound_Origin := Fld_10; 
--# expr-start 11077 '<If at nodes.lkt:15209:13>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15209'
--# expr-start 11050 '<IsNull at nodes.lkt:15209:16>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15209'
Is_Null := Local_Pc.Node = null; 
--# expr-done 11050
if Is_Null then
   --# expr-start 11053 '<Map at nodes.lkt:15214:17>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15214'
      declare
         Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Items;
      begin
         for Item_8 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_8
         --# expr-start 11052 '<FieldAccess for can_be_paramless at nodes.lkt:15215:28>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15215'
--# expr-start 11051 '<Cast to BasicDecl at nodes.lkt:15215:28>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15215'
Cast_Expr_2 := Item_8; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11051
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 11052
         if Fld_11 then
         declare
            Item_To_Append : constant Internal_Entity := Item_8;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11053
--# expr-start 11063 '<If at nodes.lkt:15222:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15222'
if Local_Is_Prefix then
--# expr-start 11061 '<Find at nodes.lkt:15222:39>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15222'
   --# expr-start 11054 '<FieldAccess for semantic_parents at nodes.lkt:15222:39>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15222'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parents
Fld_12 := Libadalang.Implementation.Ada_Node_P_Semantic_Parents (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11054
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_12;
      begin
         for Item_9 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_9
--# expr-start 11055 '<IsA TaskBody, BaseSubpBody at nodes.lkt:15223:33>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15223'
Is_A_1 := Item_9.Node /= null 
and then Item_9.Node.Kind in Ada_Task_Body_Range | Ada_Base_Subp_Body; 
--# expr-done 11055
Var_Expr_10 := Is_A_1;
if Var_Expr_10 /= False then
   --# scope-start
   --# bind _ Var_Expr_10
--# expr-start 11060 '<FieldAccess for name_is at nodes.lkt:15224:36>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15224'
--# expr-start 11058 '<FieldAccess for name at nodes.lkt:15224:36>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15224'
--# expr-start 11057 '<FieldAccess for defining_name at nodes.lkt:15224:36>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15224'
--# expr-start 11056 '<Cast to BasicDecl at nodes.lkt:15224:36>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15224'
Cast_Expr_3 := Item_9; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 11056
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 11057
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Name (Node => Fld_13.Node.Defining_Name_F_Name, Info => Fld_13.Info);
--# expr-done 11058
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11059 '<GetSymbol at nodes.lkt:15224:81>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15224'
Sym_1 := Get_Symbol (Self); 
--# expr-done 11059
--# property-call-start Name.name_is
Fld_15 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_14.Node, Sym => Sym_1);
--# end
--# expr-done 11060
Scope_Result_2 := Fld_15;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
      if Result_Var_2 then
         Find_Result_1 := Item_9; 
      end if;
   --# end
               exit when Result_Var_2;
         end loop;
      end;
--# expr-done 11061
Var_Expr_11 := Find_Result_1;
if Var_Expr_11 /= No_Entity then
   --# scope-start
   --# bind b Var_Expr_11
--# expr-start 11062 '<ArrayLiteral at nodes.lkt:15227:32>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15227'
Array_Lit := Create_Internal_Entity_Array (Internal_Internal_Entity_Array'(1 => Var_Expr_11)); 
--# expr-done 11062
Scope_Result_3 := Array_Lit;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3421;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
   If_Result_2 := Result_Var_3;
else
   If_Result_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 11063
Concat_Result := Concat (Map_Result_1, If_Result_2); 
   If_Result_3 := Concat_Result;
else
--# expr-start 11065 '<Cast to AssocList at nodes.lkt:15237:18>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15237'
--# expr-start 11064 '<FieldAccess for suffix at nodes.lkt:15237:18>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15237'
      if Local_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity (Node => Local_Pc.Node.Call_Expr_F_Suffix, Info => Local_Pc.Info);
--# expr-done 11064
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Assoc_List
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_4 := No_Entity_Assoc_List;
   end if;
--# expr-done 11065
Var_Expr_4 := Cast_Result_4;
if Var_Expr_4 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind params Var_Expr_4
--# expr-start 11073 '<Map at nodes.lkt:15238:29>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15238'
      declare
         Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Items;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_5
         --# expr-start 11072 '<Match at nodes.lkt:15239:28>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15239'
      if Item_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_5; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_5; 
--# bind _ Match_0
--# expr-start 11067 '<Eq at nodes.lkt:15241:49>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15241'
--# expr-start 11066 '<length at nodes.lkt:15241:49>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15241'
Fld_17 := Var_Expr_4.Node; 
Len := Length (Fld_17); 
--# expr-done 11066
Is_Equal := Len = 1; 
--# expr-done 11067
Let_Result := Is_Equal; 
--# end
Scope_Result_5 := Let_Result;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_6; 
--# bind b Match_1
--# expr-start 11068 '<FieldAccess for subp_spec_or_null at nodes.lkt:15242:46>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15242'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 11068
Var_Expr_6 := Fld_18;
if Var_Expr_6 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_6
--# expr-start 11069 '<FieldAccess for call_matches_spec at nodes.lkt:15243:39>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15243'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.call_matches_spec
Fld_19 := Libadalang.Implementation.Base_Id_P_Call_Matches_Spec (Node => Ent.Node, Spec => Var_Expr_6, Pc => Local_Pc, Params => Var_Expr_4, B => Match_1, Env => Env, Origin => Bound_Origin);
--# end
--# expr-done 11069
Scope_Result_4 := Fld_19;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   --# expr-start 11071 '<FieldAccess for check_for_type at nodes.lkt:15247:41>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15247'
      if Local_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11070 '<FieldAccess for expr_type at nodes.lkt:15247:59>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15247'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_20 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 11070
--# property-call-start CallExpr.check_for_type
Fld_21 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Local_Pc.Node, Typ => Fld_20, Env => Env, Origin => Bound_Origin, E_Info => Local_Pc.Info);
--# end
--# expr-done 11071
   Result_Var_4 := Fld_21;
end if;
Let_Result_1 := Result_Var_4; 
--# end
Scope_Result_6 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_7 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 11072
         if Match_Result then
         declare
            Item_To_Append : constant Internal_Entity := Item_5;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 11073
Scope_Result_8 := Map_Result_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3422;
   Result_Var_5 := Scope_Result_8;
else
   --# expr-start 11076 '<Map at nodes.lkt:15255:29>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15255'
      declare
         Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Items;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_7
         --# expr-start 11075 '<Not at nodes.lkt:15255:49>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15255'
--# expr-start 11074 '<IsA BaseTypeDecl at nodes.lkt:15255:53>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15255'
Is_A_2 := Item_7.Node /= null 
and then Item_7.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 11074
Not_Val_1 := not (Is_A_2); 
--# expr-done 11075
         if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity := Item_7;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_3 := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 11076
   Result_Var_5 := Map_Result_3;
end if;
      Inc_Ref (Result_Var_5);
   If_Result_3 := Result_Var_5;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 11077
Dyn_Var_Bind_Result := If_Result_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11078
Let_Result_3 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 11079
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 11080
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 11081
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_270;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_270;
                     Finalizer_Scope_3415;
                     Finalizer_Scope_3421;
                     Finalizer_Scope_3422;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Entity_Array_Access,
                        As_Internal_Entity_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Base_Id_P_Env_Elements_Impl;
--# end
--# property-start BaseId.call_matches_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15264'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Call_Matches_Spec
  (Node : Bare_Base_Id
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Pc : Internal_Entity_Call_Expr
      ; Params : Internal_Entity_Assoc_List
      ; B : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
      --# bind self Self
   --# bind spec Spec
   --# bind pc Pc
   --# bind params Params
   --# bind b B
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Local_Family_Type : Internal_Entity;
Local_Real_Pc : Internal_Entity_Call_Expr;
Local_Real_Params : Internal_Entity_Assoc_List;
Var_Expr_0 : Internal_Entity_Call_Expr;
Var_Expr_1 : Internal_Entity_Call_Expr;
Var_Expr_270 : Internal_Entity_Entry_Spec;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Entry_Spec;
Fld : Internal_Entity;
Scope_Result : Internal_Entity;
Result_Var : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Call_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Call_Expr;
Fld_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Call_Expr;
Fld_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Assoc_List;
Scope_Result_1 : Internal_Entity_Assoc_List;
Result_Var_1 : Internal_Entity_Assoc_List;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Assoc_List;
Is_Null_2 : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Internal_Metadata;
Fld_6 : Boolean;
Fld_7 : Boolean;
Fld_8 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Call_Expr;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Fld_13 : Internal_Entity_Info;
Fld_14 : Internal_Metadata;
Fld_15 : Boolean;
Fld_16 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
Let_Result_1 : Boolean;
Let_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11111 '<Let at nodes.lkt:15265:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15265'
--# scope-start
--# expr-start 11082 '<Cast to EntrySpec at nodes.lkt:15265:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15265'
Cast_Expr := Spec; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Entry_Spec;
   end if;
--# expr-done 11082
Var_Expr_270 := Cast_Result;
if Var_Expr_270 /= No_Entity_Entry_Spec then
   --# scope-start
--# expr-start 11083 '<FieldAccess for family_type at nodes.lkt:15265:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15265'
      if Var_Expr_270.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity (Node => Var_Expr_270.Node.Entry_Spec_F_Family_Type, Info => Var_Expr_270.Info);
--# expr-done 11083
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity;
end if;
Local_Family_Type := Result_Var; 
--# bind family_type Local_Family_Type
--# expr-start 11110 '<Let at nodes.lkt:15270:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15270'
--# scope-start
--# expr-start 11087 '<If at nodes.lkt:15270:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15270'
--# expr-start 11084 '<IsNull at nodes.lkt:15270:26>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15270'
Is_Null := Local_Family_Type.Node = null; 
--# expr-done 11084
if Is_Null then
   If_Result := Pc;
else
   --# expr-start 11086 '<Cast to CallExpr at nodes.lkt:15270:59>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15270'
--# expr-start 11085 '<parent at nodes.lkt:15270:59>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15270'
      if Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Pc.Node, E_Info => Pc.Info);
--# expr-done 11085
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Call_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Call_Expr;
   end if;
--# expr-done 11086
   If_Result := Cast_Result_1;
end if;
--# expr-done 11087
Local_Real_Pc := If_Result; 
--# bind real_pc Local_Real_Pc
--# expr-start 11109 '<Let at nodes.lkt:15271:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
--# scope-start
--# expr-start 11093 '<If at nodes.lkt:15271:27>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
--# expr-start 11088 '<IsNull at nodes.lkt:15271:30>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
Is_Null_1 := Local_Family_Type.Node = null; 
--# expr-done 11088
if Is_Null_1 then
   If_Result_1 := Params;
else
--# expr-start 11090 '<Cast to CallExpr at nodes.lkt:15271:67>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
--# expr-start 11089 '<parent at nodes.lkt:15271:67>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
      if Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Pc.Node, E_Info => Pc.Info);
--# expr-done 11089
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Call_Expr
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_2 := No_Entity_Call_Expr;
   end if;
--# expr-done 11090
Var_Expr_0 := Cast_Result_2;
if Var_Expr_0 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_0
--# expr-start 11092 '<Cast to AssocList at nodes.lkt:15271:101>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
--# expr-start 11091 '<FieldAccess for suffix at nodes.lkt:15271:101>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15271'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity (Node => Var_Expr_0.Node.Call_Expr_F_Suffix, Info => Var_Expr_0.Info);
--# expr-done 11091
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Assoc_List
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11092
Scope_Result_1 := Cast_Result_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Assoc_List;
end if;
   If_Result_1 := Result_Var_1;
end if;
--# expr-done 11093
Local_Real_Params := If_Result_1; 
--# bind real_params Local_Real_Params
--# expr-start 11094 '<IsNull at nodes.lkt:15277:13>' Is_Null_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15277'
Is_Null_2 := Local_Real_Pc.Node = null; 
--# expr-done 11094
if Is_Null_2 then
   If_Result_3 := True;
else
--# expr-start 11098 '<FieldAccess for is_matching_param_list at nodes.lkt:15279:17>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11097 '<FieldAccess for dottable_subp at nodes.lkt:15279:58>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
--# expr-start 11096 '<FieldAccess for md at nodes.lkt:15279:58>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
--# expr-start 11095 '<FieldAccess for info at nodes.lkt:15279:58>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
Fld_4 := B.Info;
--# expr-done 11095
Fld_5 := Fld_4.Md;
--# expr-done 11096
Fld_6 := Fld_5.Dottable_Subp;
--# expr-done 11097
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Spec.Node, Params => Local_Real_Params, Is_Dottable_Subp => Fld_6, Env => Env);
--# end
--# expr-done 11098
if Fld_7 then
--# expr-start 11100 '<Cast to CallExpr at nodes.lkt:15279:87>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
--# expr-start 11099 '<parent at nodes.lkt:15279:87>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15279'
      if Local_Real_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Local_Real_Pc.Node, E_Info => Local_Real_Pc.Info);
--# expr-done 11099
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Call_Expr
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_4 := No_Entity_Call_Expr;
   end if;
--# expr-done 11100
Var_Expr_1 := Cast_Result_4;
if Var_Expr_1 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_1
--# expr-start 11102 '<FieldAccess for check_for_type at nodes.lkt:15280:29>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15280'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11101 '<FieldAccess for expr_type at nodes.lkt:15280:47>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15280'
      if B.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => B.Node, Origin => Origin, E_Info => B.Info);
--# end
--# expr-done 11101
--# property-call-start CallExpr.check_for_type
Fld_10 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_1.Node, Typ => Fld_9, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11102
Scope_Result_2 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_5 := True;
else
--# expr-start 11104 '<FieldAccess for check_for_type at nodes.lkt:15286:13>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
      if Local_Real_Pc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11103 '<FieldAccess for expr_type at nodes.lkt:15286:36>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
      if B.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => B.Node, Origin => Origin, E_Info => B.Info);
--# end
--# expr-done 11103
--# property-call-start CallExpr.check_for_type
Fld_12 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Local_Real_Pc.Node, Typ => Fld_11, Env => Env, Origin => Origin, E_Info => Local_Real_Pc.Info);
--# end
--# expr-done 11104
if Fld_12 then
   --# expr-start 11108 '<FieldAccess for paramless at nodes.lkt:15286:55>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11107 '<FieldAccess for dottable_subp at nodes.lkt:15286:70>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
--# expr-start 11106 '<FieldAccess for md at nodes.lkt:15286:70>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
--# expr-start 11105 '<FieldAccess for info at nodes.lkt:15286:70>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15286'
Fld_13 := B.Info;
--# expr-done 11105
Fld_14 := Fld_13.Md;
--# expr-done 11106
Fld_15 := Fld_14.Dottable_Subp;
--# expr-done 11107
--# property-call-start BaseFormalParamHolder.paramless
Fld_16 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Spec.Node, Dottable_Subp => Fld_15);
--# end
--# expr-done 11108
   If_Result_4 := Fld_16;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 11109
Let_Result_1 := Let_Result; 
--# end
--# expr-done 11110
Let_Result_2 := Let_Result_1; 
--# end
--# expr-done 11111
         Property_Result := Let_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Base_Id_P_Call_Matches_Spec;
--# end
--# property-start BaseId.denotes_the_property_function 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15290'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Denotes_The_Property_Function
  (Node : Bare_Base_Id
      ; Subp_Spec : Internal_Entity_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp_spec Subp_Spec
   Property_Result : Boolean;
            procedure Finalizer_Scope_272 with Inline_Always;
            procedure Finalizer_Scope_3434 with Inline_Always;
      Local_Primitive_Types : Internal_Entity_Base_Type_Decl_Array_Access;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subp_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Param_Spec_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Find_Result : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Find_Result_1 : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Equal_1 : Boolean;
Not_Val_2 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
Let_Result_3 : Boolean;
            procedure Finalizer_Scope_272 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Local_Primitive_Types);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_272;
            procedure Finalizer_Scope_3434 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3434;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11131 '<Let at nodes.lkt:15295:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15295'
--# scope-start
--# expr-start 11112 '<FieldAccess for primitive_subp_types at nodes.lkt:15295:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15295'
      if Subp_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_Types (Node => Subp_Spec.Node, Imprecise_Fallback => False, E_Info => Subp_Spec.Info);
--# end
--# expr-done 11112
Local_Primitive_Types := Fld; Inc_Ref (Local_Primitive_Types);
--# bind primitive_types Local_Primitive_Types
--# expr-start 11115 '<Eq at nodes.lkt:15301:13>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15301'
--# expr-start 11114 '<length at nodes.lkt:15301:13>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15301'
--# expr-start 11113 '<FieldAccess for params at nodes.lkt:15301:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15301'
      if Subp_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Subp_Spec.Node, E_Info => Subp_Spec.Info);
--# end
--# expr-done 11113
Len := Length (Fld_1); 
--# expr-done 11114
Is_Equal := Len = 1; 
--# expr-done 11115
if Is_Equal then
   --# expr-start 11130 '<Match at nodes.lkt:15305:13>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15305'
--# expr-start 11116 '<FieldAccess for parent_basic_decl at nodes.lkt:15305:19>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15305'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11116
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_4; 
--# bind td Match_0
--# expr-start 11120 '<Not at nodes.lkt:15308:38>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15308'
--# expr-start 11119 '<IsNull at nodes.lkt:15308:42>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15308'
--# expr-start 11118 '<Find at nodes.lkt:15308:42>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15308'
      Find_Result := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Local_Primitive_Types;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_0
      --# expr-start 11117 '<Eq at nodes.lkt:15308:70>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15308'
Cast_Expr := Item_0; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Match_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 11117
      if Is_Equiv then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_Equiv;
         end loop;
      end;
--# expr-done 11118
Is_Null := Find_Result.Node = null; 
--# expr-done 11119
Not_Val := not (Is_Null); 
--# expr-done 11120
Let_Result := Not_Val; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind sd Match_1
--# expr-start 11129 '<Not at nodes.lkt:15311:38>' Not_Val_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15311'
--# expr-start 11128 '<IsNull at nodes.lkt:15311:42>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15311'
--# expr-start 11127 '<Map at nodes.lkt:15311:42>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15311'
   --# expr-start 11122 '<FieldAccess for primitive_subp_types at nodes.lkt:15311:42>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15311'
--# expr-start 11121 '<FieldAccess for subp_spec at nodes.lkt:15311:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15311'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 11121
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.primitive_subp_types
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Subp_Types (Node => Fld_3.Node, Imprecise_Fallback => False, E_Info => Fld_3.Info);
--# end
--# expr-done 11122
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_4;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind t1 Item_1
         --# expr-start 11126 '<Not at nodes.lkt:15312:29>' Not_Val_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15312'
--# expr-start 11125 '<IsNull at nodes.lkt:15312:33>' Is_Null_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15312'
--# expr-start 11124 '<Find at nodes.lkt:15312:33>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15312'
      Find_Result_1 := No_Entity_Base_Type_Decl;
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Local_Primitive_Types;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind t2 Item_2
      --# expr-start 11123 '<Eq at nodes.lkt:15312:62>' Is_Equiv_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15312'
Cast_Expr_2 := Item_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Cast_Expr_3 := Item_2; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 11123
      if Is_Equiv_1 then
         Find_Result_1 := Item_2; 
      end if;
   --# end
               exit when Is_Equiv_1;
         end loop;
      end;
--# expr-done 11124
Is_Null_1 := Find_Result_1.Node = null; 
--# expr-done 11125
Not_Val_1 := not (Is_Null_1); 
--# expr-done 11126
         if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Item_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11127
Is_Equal_1 := Equivalent (Map_Result, No_Internal_Entity_Base_Type_Decl_Array_Type); 
--# expr-done 11128
Not_Val_2 := not (Is_Equal_1); 
--# expr-done 11129
Let_Result_1 := Not_Val_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
      Finalizer_Scope_3434;
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Synthetic_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_6; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 11130
   If_Result := Match_Result;
else
   If_Result := False;
end if;
Let_Result_3 := If_Result; 
--# end
--# expr-done 11131
         Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_272;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_272;
                     Finalizer_Scope_3434;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Denotes_The_Property_Function;
--# end
--# property-start BaseId.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15320'
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id  := Bare_Base_Id (Node);
     Ent : Internal_Entity_Base_Id :=
       Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_273 with Inline_Always;
            procedure Finalizer_Scope_3438 with Inline_Always;
            procedure Finalizer_Scope_3439 with Inline_Always;
      Local_Is_Prefix : Boolean;
Var_Expr_0 : Internal_Entity_Array_Access;
Var_Expr_271 : Internal_Entity_Base_Subp_Body;
Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result_2 : Logic_Equation;
Fld_7 : Logic_Var;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Cast_Result_4 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr_3 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Base_Subp_Body;
Fld_9 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_10 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_273 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_273;
            procedure Finalizer_Scope_3438 is
            begin
                     Dec_Ref (Var_Expr_0);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_3438;
            procedure Finalizer_Scope_3439 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3439;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11156 '<Let at nodes.lkt:15321:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15321'
--# scope-start
--# expr-start 11133 '<Not at nodes.lkt:15321:25>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15321'
--# expr-start 11132 '<FieldAccess for is_suffix at nodes.lkt:15321:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15321'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_suffix
Fld := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 11132
Not_Val := not (Fld); 
--# expr-done 11133
Local_Is_Prefix := Not_Val; 
--# bind is_prefix Local_Is_Prefix
--# expr-start 11134 '<FieldAccess for env_elements at nodes.lkt:15323:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15323'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, No_Visibility => False, E_Info => Ent.Info);
--# end
--# expr-done 11134
Var_Expr_0 := Fld_1;
      Inc_Ref (Var_Expr_0);
if Var_Expr_0 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind env_els Var_Expr_0
--# expr-start 11154 '<LogicAny at nodes.lkt:15324:26>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15324'
--# expr-start 11153 '<Map at nodes.lkt:15324:26>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15324'
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_0;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_1
      --# expr-start 11152 '<And at nodes.lkt:15325:24>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15325'
--# expr-start 11138 '<Bind at nodes.lkt:15325:24>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15325'
--# expr-start 11135 '<FieldAccess for ref_var at nodes.lkt:15325:28>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15325'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 11135
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 11137 '<FieldAccess for corresponding_actual at nodes.lkt:15325:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15325'
--# expr-start 11136 '<Cast to BasicDecl at nodes.lkt:15325:44>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15325'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 11136
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.corresponding_actual'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Corresponding_Actual (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 11137
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15325:24") else null)); 
--# expr-done 11138
--# expr-start 11151 '<If at nodes.lkt:15328:21>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15328'
if Local_Is_Prefix then
--# expr-start 11139 '<Cast to BaseSubpBody at nodes.lkt:15328:38>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15328'
Cast_Expr_3 := Item_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_5 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_5 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 11139
Var_Expr_271 := Cast_Result_5;
if Var_Expr_271 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 11140 '<FieldAccess for in_scope at nodes.lkt:15328:38>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15328'
      if Var_Expr_271.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Body.in_scope
Fld_9 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Var_Expr_271.Node, Origin => Origin);
--# end
--# expr-done 11140
Scope_Result := Fld_9;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 11142 '<Bind at nodes.lkt:15328:74>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15328'
--# expr-start 11141 '<FieldAccess for type_var at nodes.lkt:15328:78>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15328'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11141
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Cast_Expr_1 := No_Entity_Base_Type_Decl; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_4, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15328:74") else null)); 
--# expr-done 11142
   If_Result_1 := Bind_Result_1;
else
   --# expr-start 11150 '<And at nodes.lkt:15334:25>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
--# expr-start 11145 '<Bind at nodes.lkt:15334:25>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
--# expr-start 11143 '<FieldAccess for ref_var at nodes.lkt:15334:46>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 11143
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 11144 '<FieldAccess for type_var at nodes.lkt:15334:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11144
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result_2 := Solver.Create_Propagate (Fld_6, Fld_5, Logic_Functor_720'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15334:25") else null)); 
--# expr-done 11145
--# expr-start 11149 '<Bind at nodes.lkt:15334:97>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
--# expr-start 11146 '<FieldAccess for subp_spec_var at nodes.lkt:15334:101>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 11146
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
--# expr-start 11148 '<FieldAccess for subp_spec_or_null at nodes.lkt:15334:123>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
--# expr-start 11147 '<Cast to BasicDecl at nodes.lkt:15334:123>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15334'
Cast_Expr_2 := Item_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 11147
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 11148
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_7, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15334:97") else null)); 
--# expr-done 11149
And_Pred := Create_And (Bind_Result_2, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15334:25") else null)); 
--# expr-done 11150
   If_Result_1 := And_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 11151
And_Pred_1 := Create_And (Bind_Result, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15325:24") else null)); 
--# expr-done 11152
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3439;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11153
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15324:26") else null)); 
--# expr-done 11154
Scope_Result_1 := Logic_Boolean_Op;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3438;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 11155 '<FieldAccess for undefined_reference_equation at nodes.lkt:15337:28>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15337'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.undefined_reference_equation
Fld_10 := Libadalang.Implementation.Name_P_Undefined_Reference_Equation (Node => Ent.Node);
--# end
--# expr-done 11155
   Result_Var_1 := Fld_10;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11156
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_273;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_273;
                     Finalizer_Scope_3438;
                     Finalizer_Scope_3439;
            raise;
      end;
   return Property_Result;
end Base_Id_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Char_Literal
   --
      procedure Initialize_Fields_For_Char_Literal
        (Self : Bare_Char_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Char_Literal;
--# property-start CharLiteral.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15350'
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Has_Context_Free_Type
  (Node : Bare_Char_Literal
  )
   return Boolean
is
   Self : Bare_Char_Literal  := Bare_Char_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Char_Literal_P_Has_Context_Free_Type;
--# end
--# property-start CharLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15353'
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Xref_Equation
  (Node : Bare_Char_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Char_Literal  := Bare_Char_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1313 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Cast_Expr : Bare_Char_Literal;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1313 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1313;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11167 '<And at nodes.lkt:15354:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
--# expr-start 11162 '<And at nodes.lkt:15354:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
--# expr-start 11158 '<Predicate on BaseTypeDecl.is_non_null_char_type at nodes.lkt:15354:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
--# expr-start 11157 '<FieldAccess for expected_type_var at nodes.lkt:15354:56>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11157
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Is_Non_Null_Char_Type_0_Predicate (Origin)); 
--# expr-done 11158
--# expr-start 11161 '<Bind at nodes.lkt:15354:86>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
--# expr-start 11159 '<FieldAccess for expected_type_var at nodes.lkt:15354:90>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11159
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11160 '<FieldAccess for type_var at nodes.lkt:15354:116>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15354'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11160
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15354:86") else null)); 
--# expr-done 11161
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15354:9") else null)); 
--# expr-done 11162
--# expr-start 11166 '<bind at nodes.lkt:15363:17>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15363'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 11165 '<Bind at nodes.lkt:15365:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15365'
--# expr-start 11163 '<FieldAccess for type_var at nodes.lkt:15365:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15365'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11163
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 11164 '<FieldAccess for ref_var at nodes.lkt:15365:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15365'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 11164
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result_1 := Solver.Create_Propagate (Fld_4, Fld_3, Logic_Functor_919'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Bound_Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15365:17") else null)); 
--# expr-done 11165
Dyn_Var_Bind_Result := Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11166
And_Pred_1 := Create_And (And_Pred, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15354:9") else null)); 
--# expr-done 11167
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1313;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1313;
            raise;
      end;
   return Property_Result;
end Char_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Identifier
   --
      procedure Initialize_Fields_For_Identifier
        (Self : Bare_Identifier
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Identifier;
--# property-start Identifier.is_attr_with_args 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15387'
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Attr_With_Args
  (Node : Bare_Identifier
  )
   return Boolean
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
      --# bind self Self
   Property_Result : Boolean;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11168 '<GetSymbol at nodes.lkt:15388:9>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15388'
Sym := Get_Symbol (Self); 
--# expr-done 11168
--# expr-start 11169 '<Symbol First at nodes.lkt:15388:24>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15388'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_First); 
--# expr-done 11169
Is_Equal_7 := Sym = Sym_1; 
if Is_Equal_7 then
   If_Result_6 := True;
else
--# expr-start 11170 '<Symbol Last at nodes.lkt:15388:35>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15388'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Last); 
--# expr-done 11170
Is_Equal_6 := Sym = Sym_2; 
if Is_Equal_6 then
   If_Result_5 := True;
else
--# expr-start 11171 '<Symbol Range at nodes.lkt:15388:45>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15388'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Range); 
--# expr-done 11171
Is_Equal_5 := Sym = Sym_3; 
if Is_Equal_5 then
   If_Result_4 := True;
else
--# expr-start 11172 '<Symbol Length at nodes.lkt:15388:56>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15388'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Length); 
--# expr-done 11172
Is_Equal_4 := Sym = Sym_4; 
if Is_Equal_4 then
   If_Result_3 := True;
else
--# expr-start 11173 '<Symbol Has_Same_Storage at nodes.lkt:15389:11>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15389'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Has_Same_Storage); 
--# expr-done 11173
Is_Equal_3 := Sym = Sym_5; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 11174 '<Symbol Overlaps_Storage at nodes.lkt:15389:33>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15389'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Overlaps_Storage); 
--# expr-done 11174
Is_Equal_2 := Sym = Sym_6; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 11175 '<Symbol Deref at nodes.lkt:15389:55>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15389'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Deref); 
--# expr-done 11175
Is_Equal_1 := Sym = Sym_7; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 11176 '<Symbol Mechanism_Code at nodes.lkt:15390:11>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15390'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mechanism_Code); 
--# expr-done 11176
Is_Equal := Sym = Sym_8; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
         Property_Result := If_Result_6;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Identifier_P_Is_Attr_With_Args;
--# end
--# property-start Identifier.complete_items 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15393'
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Complete_Items
  (Node : Bare_Identifier
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
     Ent : Internal_Entity_Identifier :=
       Internal_Entity_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_275 with Inline_Always;
      Fld : Internal_Entity;
Fld_1 : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_275 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_275;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11178 '<FieldAccess for complete_items at nodes.lkt:15393:51>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15393'
--# expr-start 11177 '<parent at nodes.lkt:15393:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15393'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11177
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.complete_items'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Complete_Items (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11178
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_275;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_275;
            raise;
      end;
   return Property_Result;
end Identifier_P_Complete_Items;
--# end
--# property-start Identifier.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15395'
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Constant
  (Node : Bare_Identifier
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Identifier  := Bare_Identifier (Node);
     Ent : Internal_Entity_Identifier :=
       Internal_Entity_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_276 with Inline_Always;
            procedure Finalizer_Scope_3443 with Inline_Always;
      Local_Rd : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Boolean;
Fld_4 : Boolean;
Is_A : Boolean;
If_Result : Boolean;
Fld_5 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_7 : Internal_Entity_Subp_Spec;
Fld_8 : Internal_Entity_Subp_Kind;
Is_A_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Find_Result_1 : Internal_Entity;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_276 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_276;
            procedure Finalizer_Scope_3443 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_3443;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11200 '<Let at nodes.lkt:15396:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15396'
--# scope-start
--# expr-start 11179 '<FieldAccess for referenced_decl at nodes.lkt:15396:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15396'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => False, E_Info => Ent.Info);
--# end
--# expr-done 11179
Local_Rd := Fld; 
--# bind rd Local_Rd
--# expr-start 11187 '<If at nodes.lkt:15401:13>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
--# expr-start 11180 '<FieldAccess for is_defining at nodes.lkt:15401:16>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.is_defining
Fld_5 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 11180
if Fld_5 then
   --# expr-start 11183 '<FieldAccess for is_constant_object at nodes.lkt:15401:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
--# expr-start 11182 '<FieldAccess for basic_decl at nodes.lkt:15401:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
--# expr-start 11181 '<FieldAccess for enclosing_defining_name at nodes.lkt:15401:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_1 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11181
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 11182
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 11183
   If_Result_1 := Fld_3;
else
   --# expr-start 11186 '<If at nodes.lkt:15401:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15401'
--# expr-start 11184 '<IsA ObjectDecl, ComponentDecl, EnumLiteralDecl, ParamSpec, NumberDecl at nodes.lkt:15405:18>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15405'
Is_A := Local_Rd.Node /= null 
and then Local_Rd.Node.Kind in Ada_Object_Decl_Range | Ada_Component_Decl_Range | Ada_Enum_Literal_Decl_Range | Ada_Param_Spec_Range | Ada_Number_Decl_Range; 
--# expr-done 11184
if Is_A then
   --# expr-start 11185 '<FieldAccess for is_constant_object at nodes.lkt:15405:99>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15405'
      if Local_Rd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_constant_object'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Constant_Object (Node => Local_Rd.Node, E_Info => Local_Rd.Info);
--# end
--# expr-done 11185
   If_Result := Fld_4;
else
   If_Result := False;
end if;
--# expr-done 11186
   If_Result_1 := If_Result;
end if;
--# expr-done 11187
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 11193 '<Find at nodes.lkt:15410:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15410'
   --# expr-start 11188 '<FieldAccess for parents at nodes.lkt:15410:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15410'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 11188
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
--# expr-start 11189 '<Cast to BaseSubpBody at nodes.lkt:15411:24>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15411'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 11189
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
   --# bind v1 Var_Expr_1
--# expr-start 11192 '<IsA SubpKind.Function at nodes.lkt:15412:29>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15412'
--# expr-start 11191 '<FieldAccess for subp_kind at nodes.lkt:15412:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15412'
--# expr-start 11190 '<FieldAccess for subp_spec at nodes.lkt:15412:29>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15412'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 11190
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Subp_Kind (Node => Fld_7.Node.Subp_Spec_F_Subp_Kind, Info => Fld_7.Info);
--# expr-done 11191
Is_A_1 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Subp_Kind_Function_Range; 
--# expr-done 11192
Scope_Result := Is_A_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 11193
Var_Expr_2 := Find_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 11194 '<FieldAccess for is_in_private_part at nodes.lkt:15420:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15420'
      if Local_Rd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Local_Rd.Node, E_Info => Local_Rd.Info);
--# end
--# expr-done 11194
if Fld_9 then
   --# expr-start 11199 '<Not at nodes.lkt:15421:23>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15421'
--# expr-start 11198 '<IsNull at nodes.lkt:15421:27>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15421'
--# expr-start 11197 '<Find at nodes.lkt:15421:27>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15421'
   --# expr-start 11195 '<FieldAccess for parents at nodes.lkt:15421:27>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15421'
      if Local_Rd.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Parents (Node => Local_Rd.Node, With_Self => True, E_Info => Local_Rd.Info);
--# expr-done 11195
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_10;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_3
      --# expr-start 11196 '<IsA ProtectedTypeDecl at nodes.lkt:15421:52>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15421'
Is_A_2 := Item_3.Node /= null 
and then Item_3.Node.Kind in Ada_Protected_Type_Decl_Range; 
--# expr-done 11196
      if Is_A_2 then
         Find_Result_1 := Item_3; 
      end if;
   --# end
               exit when Is_A_2;
         end loop;
      end;
--# expr-done 11197
Is_Null := Find_Result_1.Node = null; 
--# expr-done 11198
Not_Val := not (Is_Null); 
--# expr-done 11199
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
Scope_Result_1 := If_Result_2;
   --# end
      Finalizer_Scope_3443;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 11200
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_276;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_276;
                     Finalizer_Scope_3443;
            raise;
      end;
   return Property_Result;
end Identifier_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Op
   --
      procedure Initialize_Fields_For_Op
        (Self : Bare_Op
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Op;
--# property-start Op.subprogram_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15451'
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprogram_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Match_0 : Bare_Op_And;
Match_1 : Bare_Op_Or;
Match_2 : Bare_Op_Xor;
Match_3 : Bare_Op_Abs;
Match_4 : Bare_Op_Not;
Match_5 : Bare_Op_Pow;
Match_6 : Bare_Op_Mult;
Match_7 : Bare_Op_Div;
Match_8 : Bare_Op_Mod;
Match_9 : Bare_Op_Rem;
Match_10 : Bare_Op_Plus;
Match_11 : Bare_Op_Minus;
Match_12 : Bare_Op_Concat;
Match_13 : Bare_Op_Eq;
Match_14 : Bare_Op_Neq;
Match_15 : Bare_Op_Lt;
Match_16 : Bare_Op_Lte;
Match_17 : Bare_Op_Gt;
Match_18 : Bare_Op_Gte;
Match_19 : Bare_Op;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_And;
Let_Result : Symbol_Type;
Scope_Result : Symbol_Type;
Cast_Result_1 : Bare_Op_Or;
Let_Result_1 : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Cast_Result_2 : Bare_Op_Xor;
Let_Result_2 : Symbol_Type;
Scope_Result_2 : Symbol_Type;
Cast_Result_3 : Bare_Op_Abs;
Let_Result_3 : Symbol_Type;
Scope_Result_3 : Symbol_Type;
Cast_Result_4 : Bare_Op_Not;
Let_Result_4 : Symbol_Type;
Scope_Result_4 : Symbol_Type;
Cast_Result_5 : Bare_Op_Pow;
Let_Result_5 : Symbol_Type;
Scope_Result_5 : Symbol_Type;
Cast_Result_6 : Bare_Op_Mult;
Let_Result_6 : Symbol_Type;
Scope_Result_6 : Symbol_Type;
Cast_Result_7 : Bare_Op_Div;
Let_Result_7 : Symbol_Type;
Scope_Result_7 : Symbol_Type;
Cast_Result_8 : Bare_Op_Mod;
Let_Result_8 : Symbol_Type;
Scope_Result_8 : Symbol_Type;
Cast_Result_9 : Bare_Op_Rem;
Let_Result_9 : Symbol_Type;
Scope_Result_9 : Symbol_Type;
Cast_Result_10 : Bare_Op_Plus;
Let_Result_10 : Symbol_Type;
Scope_Result_10 : Symbol_Type;
Cast_Result_11 : Bare_Op_Minus;
Let_Result_11 : Symbol_Type;
Scope_Result_11 : Symbol_Type;
Cast_Result_12 : Bare_Op_Concat;
Let_Result_12 : Symbol_Type;
Scope_Result_12 : Symbol_Type;
Cast_Result_13 : Bare_Op_Eq;
Let_Result_13 : Symbol_Type;
Scope_Result_13 : Symbol_Type;
Cast_Result_14 : Bare_Op_Neq;
Let_Result_14 : Symbol_Type;
Scope_Result_14 : Symbol_Type;
Cast_Result_15 : Bare_Op_Lt;
Let_Result_15 : Symbol_Type;
Scope_Result_15 : Symbol_Type;
Cast_Result_16 : Bare_Op_Lte;
Let_Result_16 : Symbol_Type;
Scope_Result_16 : Symbol_Type;
Cast_Result_17 : Bare_Op_Gt;
Let_Result_17 : Symbol_Type;
Scope_Result_17 : Symbol_Type;
Cast_Result_18 : Bare_Op_Gte;
Let_Result_18 : Symbol_Type;
Scope_Result_18 : Symbol_Type;
Cast_Result_19 : Bare_Op;
Let_Result_19 : Symbol_Type;
Scope_Result_19 : Symbol_Type;
Match_Result : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11221 '<Match at nodes.lkt:15451:39>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15451'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Self; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_And =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 11201 '<Symbol "and" at nodes.lkt:15452:27>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15452'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 11201
Let_Result := Sym; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Op_Or =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 11202 '<Symbol "or" at nodes.lkt:15453:26>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15453'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 11202
Let_Result_1 := Sym_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 11203 '<Symbol "xor" at nodes.lkt:15454:27>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15454'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 11203
Let_Result_2 := Sym_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Op_Abs =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 11204 '<Symbol "abs" at nodes.lkt:15455:27>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15455'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 11204
Let_Result_3 := Sym_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Op_Not =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
--# expr-start 11205 '<Symbol "not" at nodes.lkt:15456:27>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15456'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 11205
Let_Result_4 := Sym_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_5 := Cast_Result_5; 
--# bind _ Match_5
--# expr-start 11206 '<Symbol "**" at nodes.lkt:15457:27>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15457'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 11206
Let_Result_5 := Sym_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Op_Mult =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_6 := Cast_Result_6; 
--# bind _ Match_6
--# expr-start 11207 '<Symbol "*" at nodes.lkt:15458:28>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15458'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 11207
Let_Result_6 := Sym_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Op_Div =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_7 := Cast_Result_7; 
--# bind _ Match_7
--# expr-start 11208 '<Symbol "/" at nodes.lkt:15459:27>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15459'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 11208
Let_Result_7 := Sym_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Op_Mod =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_8 := Cast_Result_8; 
--# bind _ Match_8
--# expr-start 11209 '<Symbol "mod" at nodes.lkt:15460:27>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15460'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 11209
Let_Result_8 := Sym_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Op_Rem =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix;
Match_9 := Cast_Result_9; 
--# bind _ Match_9
--# expr-start 11210 '<Symbol "rem" at nodes.lkt:15461:27>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15461'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 11210
Let_Result_9 := Sym_9; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Op_Plus =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix;
Match_10 := Cast_Result_10; 
--# bind _ Match_10
--# expr-start 11211 '<Symbol "+" at nodes.lkt:15462:28>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15462'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 11211
Let_Result_10 := Sym_10; 
--# end
Scope_Result_10 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Op_Minus =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Match_Prefix;
Match_11 := Cast_Result_11; 
--# bind _ Match_11
--# expr-start 11212 '<Symbol "-" at nodes.lkt:15463:29>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15463'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 11212
Let_Result_11 := Sym_11; 
--# end
Scope_Result_11 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Match_Prefix;
Match_12 := Cast_Result_12; 
--# bind _ Match_12
--# expr-start 11213 '<Symbol "&" at nodes.lkt:15464:30>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15464'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 11213
Let_Result_12 := Sym_12; 
--# end
Scope_Result_12 := Let_Result_12;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Op_Eq =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Match_Prefix;
Match_13 := Cast_Result_13; 
--# bind _ Match_13
--# expr-start 11214 '<Symbol "=" at nodes.lkt:15465:26>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15465'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 11214
Let_Result_13 := Sym_13; 
--# end
Scope_Result_13 := Let_Result_13;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Op_Neq =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Match_Prefix;
Match_14 := Cast_Result_14; 
--# bind _ Match_14
--# expr-start 11215 '<Symbol "/=" at nodes.lkt:15466:27>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15466'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 11215
Let_Result_14 := Sym_14; 
--# end
Scope_Result_14 := Let_Result_14;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Op_Lt =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Match_Prefix;
Match_15 := Cast_Result_15; 
--# bind _ Match_15
--# expr-start 11216 '<Symbol "<" at nodes.lkt:15467:26>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15467'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 11216
Let_Result_15 := Sym_15; 
--# end
Scope_Result_15 := Let_Result_15;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Op_Lte =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Match_Prefix;
Match_16 := Cast_Result_16; 
--# bind _ Match_16
--# expr-start 11217 '<Symbol "<=" at nodes.lkt:15468:27>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15468'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 11217
Let_Result_16 := Sym_16; 
--# end
Scope_Result_16 := Let_Result_16;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Op_Gt =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Match_Prefix;
Match_17 := Cast_Result_17; 
--# bind _ Match_17
--# expr-start 11218 '<Symbol ">" at nodes.lkt:15469:26>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15469'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 11218
Let_Result_17 := Sym_17; 
--# end
Scope_Result_17 := Let_Result_17;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_Op_Gte =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Match_Prefix;
Match_18 := Cast_Result_18; 
--# bind _ Match_18
--# expr-start 11219 '<Symbol ">=" at nodes.lkt:15470:27>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15470'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 11219
Let_Result_18 := Sym_18; 
--# end
Scope_Result_18 := Let_Result_18;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Op_And_Then | Ada_Op_Double_Dot | Ada_Op_In | Ada_Op_Not_In | Ada_Op_Or_Else =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Match_Prefix;
Match_19 := Cast_Result_19; 
--# bind _ Match_19
--# expr-start 11220 '<Symbol <<>> at nodes.lkt:15471:19>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15471'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_20); 
--# expr-done 11220
Let_Result_19 := Sym_19; 
--# end
Scope_Result_19 := Let_Result_19;
   --# end
            Match_Result := Scope_Result_19; 
end case;
--# expr-done 11221
         Property_Result := Match_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Op_P_Subprogram_Symbol;
--# end
--# property-start Op.subprograms_for_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15478'
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms_For_Symbol
  (Node : Bare_Op
      ; Sym : Symbol_Type
      ; From_Node : Internal_Entity
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   --# bind sym Sym
   --# bind from_node From_Node
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1315 with Inline_Always;
      Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1315 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1315;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11222 '<FieldAccess for root_type_ops at nodes.lkt:15478:101>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_type_ops
Fld := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops (Node => Self, Sym => Sym);
--# end
--# expr-done 11222
--# expr-start 11231 '<Map at nodes.lkt:15478:127>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15478'
   --# expr-start 11225 '<FieldAccess for env_get at nodes.lkt:15478:127>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15478'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11223 '<FieldAccess for node_env at nodes.lkt:15479:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15479'
      if From_Node.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 11223
--# expr-start 11224 '<FieldAccess for node at nodes.lkt:15479:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15479'
Fld_2 := From_Node.Node;
--# expr-done 11224
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_1, Symbol => Sym, From_Node => Fld_2);
--# end
--# expr-done 11225
      declare
         Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
--# expr-start 11227 '<FieldAccess for is_subprogram at nodes.lkt:15481:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15481'
--# expr-start 11226 '<Cast to BasicDecl at nodes.lkt:15481:41>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15481'
Cast_Expr_1 := Item_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11226
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 11227
if Fld_4 then
   --# expr-start 11229 '<Not at nodes.lkt:15492:13>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15492'
--# expr-start 11228 '<IsA SyntheticSubpDecl at nodes.lkt:15492:17>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15492'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Synthetic_Subp_Decl_Range; 
--# expr-done 11228
Not_Val := not (Is_A); 
--# expr-done 11229
   If_Result := Not_Val;
else
   If_Result := False;
end if;
         if If_Result then
      --# expr-start 11230 '<Cast to BasicDecl at nodes.lkt:15481:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15481'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11230
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl := Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Basic_Decl_Array
           (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11231
Concat_Result := Concat (Fld, Map_Result); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1315;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1315;
            raise;
      end;
   return Property_Result;
end Op_P_Subprograms_For_Symbol;
--# end
--# property-start Op.subprograms 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15498'
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms
  (Node : Bare_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op  := Bare_Op (Node);
     Ent : Internal_Entity_Op :=
       Internal_Entity_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1316 with Inline_Always;
      Fld : Symbol_Type;
Cast_Expr : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1316 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1316;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11233 '<FieldAccess for subprograms_for_symbol at nodes.lkt:15499:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15499'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11232 '<FieldAccess for subprogram_symbol at nodes.lkt:15499:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15499'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 11232
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_1 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Self, Sym => Fld, From_Node => Cast_Result);
--# end
--# expr-done 11233
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1316;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1316;
            raise;
      end;
   return Property_Result;
end Op_P_Subprograms;
--# end
--# property-start Op.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15501'
pragma Warnings (Off, "is not referenced");
 function Op_P_Name_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11234 '<FieldAccess for subprogram_symbol at nodes.lkt:15501:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15501'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 11234
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Op_P_Name_Symbol;
--# end
--# property-start Op.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15504'
pragma Warnings (Off, "is not referenced");
 function Op_P_Xref_Equation
  (Node : Bare_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Op  := Bare_Op (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1318 with Inline_Always;
      False_Rel : Logic_Equation;
            procedure Finalizer_Scope_1318 is
            begin
                     Dec_Ref (False_Rel);
            end Finalizer_Scope_1318;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15507:9") else null)); 
         Property_Result := False_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1318;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1318;
            raise;
      end;
   return Property_Result;
end Op_P_Xref_Equation;
--# end
--# property-start Op.is_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15510'
pragma Warnings (Off, "is not referenced");
 function Op_P_Is_Dispatching_Call
  (Node : Bare_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Op  := Bare_Op (Node);
     Ent : Internal_Entity_Op :=
       Internal_Entity_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_3466 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Concat_Op;
Match_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Concat_Op;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var : Boolean;
            procedure Finalizer_Scope_3466 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3466;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11235 '<FieldAccess for referenced_decl at nodes.lkt:15510:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15510'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 11235
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr_0
--# expr-start 11241 '<Match at nodes.lkt:15513:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15513'
--# expr-start 11239 '<Cast to Expr at nodes.lkt:15513:15>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15513'
--# expr-start 11238 '<Find at nodes.lkt:15513:15>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15513'
   --# expr-start 11236 '<FieldAccess for parents at nodes.lkt:15513:15>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15513'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 11236
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
      --# expr-start 11237 '<IsA BinOp, ConcatOp, UnOp at nodes.lkt:15513:42>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15513'
Is_A := Item_1.Node /= null 
and then Item_1.Node.Kind in Ada_Bin_Op_Range | Ada_Concat_Op_Range | Ada_Un_Op_Range; 
--# expr-done 11237
      if Is_A then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11238
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11239
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Cast_Result; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Concat_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Concat_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
Let_Result := False; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Case_Expr_Alternative | Ada_Concat_Operand .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 11240 '<FieldAccess for is_dispatching_call_impl at nodes.lkt:15517:23>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15517'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_dispatching_call_impl
Fld_2 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Match_1.Node, Decl => Var_Expr_0, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 11240
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 11241
Scope_Result_2 := Match_Result;
   --# end
      Finalizer_Scope_3466;
   Result_Var := Scope_Result_2;
else
   Result_Var := False;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_3466;
            raise;
      end;
   return Property_Result;
end Op_P_Is_Dispatching_Call;
--# end
   --
   --  Primitives for Bare_Op_Abs
   --
      procedure Initialize_Fields_For_Op_Abs
        (Self : Bare_Op_Abs
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Abs;
   --
   --  Primitives for Bare_Op_And
   --
      procedure Initialize_Fields_For_Op_And
        (Self : Bare_Op_And
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And;
   --
   --  Primitives for Bare_Op_And_Then
   --
      procedure Initialize_Fields_For_Op_And_Then
        (Self : Bare_Op_And_Then
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And_Then;
   --
   --  Primitives for Bare_Op_Concat
   --
      procedure Initialize_Fields_For_Op_Concat
        (Self : Bare_Op_Concat
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Concat;
   --
   --  Primitives for Bare_Op_Div
   --
      procedure Initialize_Fields_For_Op_Div
        (Self : Bare_Op_Div
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Div;
   --
   --  Primitives for Bare_Op_Double_Dot
   --
      procedure Initialize_Fields_For_Op_Double_Dot
        (Self : Bare_Op_Double_Dot
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Double_Dot;
   --
   --  Primitives for Bare_Op_Eq
   --
      procedure Initialize_Fields_For_Op_Eq
        (Self : Bare_Op_Eq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Eq;
   --
   --  Primitives for Bare_Op_Gt
   --
      procedure Initialize_Fields_For_Op_Gt
        (Self : Bare_Op_Gt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gt;
   --
   --  Primitives for Bare_Op_Gte
   --
      procedure Initialize_Fields_For_Op_Gte
        (Self : Bare_Op_Gte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gte;
   --
   --  Primitives for Bare_Op_In
   --
      procedure Initialize_Fields_For_Op_In
        (Self : Bare_Op_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_In;
   --
   --  Primitives for Bare_Op_Lt
   --
      procedure Initialize_Fields_For_Op_Lt
        (Self : Bare_Op_Lt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lt;
   --
   --  Primitives for Bare_Op_Lte
   --
      procedure Initialize_Fields_For_Op_Lte
        (Self : Bare_Op_Lte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lte;
   --
   --  Primitives for Bare_Op_Minus
   --
      procedure Initialize_Fields_For_Op_Minus
        (Self : Bare_Op_Minus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Minus;
   --
   --  Primitives for Bare_Op_Mod
   --
      procedure Initialize_Fields_For_Op_Mod
        (Self : Bare_Op_Mod
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mod;
   --
   --  Primitives for Bare_Op_Mult
   --
      procedure Initialize_Fields_For_Op_Mult
        (Self : Bare_Op_Mult
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mult;
   --
   --  Primitives for Bare_Op_Neq
   --
      procedure Initialize_Fields_For_Op_Neq
        (Self : Bare_Op_Neq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Neq;
   --
   --  Primitives for Bare_Op_Not
   --
      procedure Initialize_Fields_For_Op_Not
        (Self : Bare_Op_Not
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not;
   --
   --  Primitives for Bare_Op_Not_In
   --
      procedure Initialize_Fields_For_Op_Not_In
        (Self : Bare_Op_Not_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not_In;
   --
   --  Primitives for Bare_Op_Or
   --
      procedure Initialize_Fields_For_Op_Or
        (Self : Bare_Op_Or
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or;
   --
   --  Primitives for Bare_Op_Or_Else
   --
      procedure Initialize_Fields_For_Op_Or_Else
        (Self : Bare_Op_Or_Else
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or_Else;
   --
   --  Primitives for Bare_Op_Plus
   --
      procedure Initialize_Fields_For_Op_Plus
        (Self : Bare_Op_Plus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Plus;
   --
   --  Primitives for Bare_Op_Pow
   --
      procedure Initialize_Fields_For_Op_Pow
        (Self : Bare_Op_Pow
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Pow;
   --
   --  Primitives for Bare_Op_Rem
   --
      procedure Initialize_Fields_For_Op_Rem
        (Self : Bare_Op_Rem
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Rem;
   --
   --  Primitives for Bare_Op_Xor
   --
      procedure Initialize_Fields_For_Op_Xor
        (Self : Bare_Op_Xor
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Xor;
   --
   --  Primitives for Bare_String_Literal
   --
      procedure Initialize_Fields_For_String_Literal
        (Self : Bare_String_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_String_Literal;
--# property-start StringLiteral.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15530'
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Has_Context_Free_Type
  (Node : Bare_String_Literal
  )
   return Boolean
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end String_Literal_P_Has_Context_Free_Type;
--# end
--# property-start StringLiteral.env_elements_impl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15536'
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Env_Elements_Impl
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
     Ent : Internal_Entity_String_Literal :=
       Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1322 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1322 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1322;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11245 '<Map at nodes.lkt:15537:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15537'
   --# expr-start 11243 '<FieldAccess for root_type_ops at nodes.lkt:15537:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15537'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11242 '<GetSymbol at nodes.lkt:15537:28>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15537'
Sym := Get_Symbol (Self); 
--# expr-done 11242
--# property-call-start AdaNode.root_type_ops
Fld := Libadalang.Implementation.Ada_Node_P_Root_Type_Ops (Node => Self, Sym => Sym);
--# end
--# expr-done 11243
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_0
      --# expr-start 11244 '<Cast to AdaNode at nodes.lkt:15537:53>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15537'
Cast_Expr := Item_0; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 11244
         declare
            Item_To_Append : constant Internal_Entity := Cast_Result;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11245
--# expr-start 11246 '<Super at nodes.lkt:15537:71>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15537'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.env_elements_impl
Fld_1 := Libadalang.Implementation.Base_Id_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 11246
Concat_Result := Concat (Map_Result, Fld_1); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1322;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1322;
            raise;
      end;
   return Property_Result;
end String_Literal_P_Env_Elements_Impl;
--# end
--# property-start StringLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15540'
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Xref_Equation
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_String_Literal  := Bare_String_Literal (Node);
     Ent : Internal_Entity_String_Literal :=
       Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1323 with Inline_Always;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_5 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_1323 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1323;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11259 '<If at nodes.lkt:15543:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15543'
--# expr-start 11248 '<IsA Name at nodes.lkt:15543:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15543'
--# expr-start 11247 '<parent at nodes.lkt:15543:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15543'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 11247
Is_A := Fld_5 /= null 
and then Fld_5.Kind in Ada_Name; 
--# expr-done 11248
if Is_A then
   --# expr-start 11249 '<Super at nodes.lkt:15543:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15543'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseId.xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Ent.Info);
--# end
--# expr-done 11249
   If_Result := Fld;
else
   --# expr-start 11258 '<And at nodes.lkt:15543:55>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15543'
--# expr-start 11255 '<And at nodes.lkt:15544:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
--# expr-start 11252 '<Bind at nodes.lkt:15544:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
--# expr-start 11250 '<FieldAccess for expected_type_var at nodes.lkt:15544:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11250
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11251 '<FieldAccess for type_var at nodes.lkt:15544:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11251
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15544:13") else null)); 
--# expr-done 11252
--# expr-start 11254 '<Predicate on BaseTypeDecl.is_not_any_type at nodes.lkt:15544:64>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
--# expr-start 11253 '<FieldAccess for expected_type_var at nodes.lkt:15544:105>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15544'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11253
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred := Solver.Create_Predicate (Fld_3, Create_Base_Type_Decl_P_Is_Not_Any_Type_0_Predicate (Self)); 
--# expr-done 11254
And_Pred := Create_And (Bind_Result, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15544:13") else null)); 
--# expr-done 11255
--# expr-start 11257 '<Predicate on BaseTypeDecl.allows_string_literal at nodes.lkt:15545:15>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15545'
--# expr-start 11256 '<FieldAccess for expected_type_var at nodes.lkt:15545:62>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15545'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11256
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred_1 := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Allows_String_Literal_0_Predicate (Origin, Self)); 
--# expr-done 11257
And_Pred_1 := Create_And (And_Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15543:55") else null)); 
--# expr-done 11258
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 11259
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1323;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1323;
            raise;
      end;
   return Property_Result;
end String_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Literal
   --
      procedure Initialize_Fields_For_Null_Literal
        (Self : Bare_Null_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Null_Literal;
--# property-start NullLiteral.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15551'
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Has_Context_Free_Type
  (Node : Bare_Null_Literal
  )
   return Boolean
is
   Self : Bare_Null_Literal  := Bare_Null_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Null_Literal_P_Has_Context_Free_Type;
--# end
--# property-start NullLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15554'
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Xref_Equation
  (Node : Bare_Null_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Literal  := Bare_Null_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1325 with Inline_Always;
      Fld : Logic_Var;
Pred : Logic_Equation;
Fld_1 : Logic_Var;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1325 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1325;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11265 '<And at nodes.lkt:15555:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
--# expr-start 11261 '<Predicate on BaseTypeDecl.is_access_type_predicate at nodes.lkt:15555:9>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
--# expr-start 11260 '<FieldAccess for expected_type_var at nodes.lkt:15555:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11260
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
Pred := Solver.Create_Predicate (Fld, Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (Origin)); 
--# expr-done 11261
--# expr-start 11264 '<Bind at nodes.lkt:15555:89>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
--# expr-start 11262 '<FieldAccess for expected_type_var at nodes.lkt:15555:93>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11262
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 11263 '<FieldAccess for type_var at nodes.lkt:15555:119>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15555'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11263
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
Bind_Result := Solver.Create_Unify (Fld_1, Fld_2, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15555:89") else null)); 
--# expr-done 11264
And_Pred := Create_And (Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15555:9") else null)); 
--# expr-done 11265
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1325;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1325;
            raise;
      end;
   return Property_Result;
end Null_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Num_Literal
   --
      procedure Initialize_Fields_For_Num_Literal
        (Self : Bare_Num_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Num_Literal;
--# property-start NumLiteral.is_constant 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15562'
pragma Warnings (Off, "is not referenced");
 function Num_Literal_P_Is_Constant
  (Node : Bare_Num_Literal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Num_Literal  := Bare_Num_Literal (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Num_Literal_P_Is_Constant;
--# end
   --
   --  Primitives for Bare_Int_Literal
   --
      procedure Initialize_Fields_For_Int_Literal
        (Self : Bare_Int_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Int_Literal;
--# property-start IntLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15569'
pragma Warnings (Off, "is not referenced");
 function Int_Literal_P_Xref_Equation
  (Node : Bare_Int_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Int_Literal  := Bare_Int_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1327 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1327 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1327;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11267 '<FieldAccess for universal_int_bind at nodes.lkt:15569:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15569'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11266 '<FieldAccess for type_var at nodes.lkt:15569:61>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15569'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11266
--# property-call-start AdaNode.universal_int_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 11267
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1327;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1327;
            raise;
      end;
   return Property_Result;
end Int_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Literal
   --
      procedure Initialize_Fields_For_Real_Literal
        (Self : Bare_Real_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Real_Literal;
--# property-start RealLiteral.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15581'
pragma Warnings (Off, "is not referenced");
 function Real_Literal_P_Xref_Equation
  (Node : Bare_Real_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Literal  := Bare_Real_Literal (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1329 with Inline_Always;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1329 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1329;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11269 '<FieldAccess for universal_real_bind at nodes.lkt:15581:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15581'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11268 '<FieldAccess for type_var at nodes.lkt:15581:62>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15581'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11268
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 11269
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1329;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1329;
            raise;
      end;
   return Property_Result;
end Real_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synthetic_Identifier
   --
      procedure Initialize_Fields_For_Synthetic_Identifier
        (Self : Bare_Synthetic_Identifier
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Synthetic_Identifier_Sym := No_Symbol;
      end Initialize_Fields_For_Synthetic_Identifier;
--# property-start SyntheticIdentifier.name_symbol 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15589'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Identifier_P_Name_Symbol
  (Node : Bare_Synthetic_Identifier
  )
   return Symbol_Type
is
   Self : Bare_Synthetic_Identifier  := Bare_Synthetic_Identifier (Node);
      --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11270 '<FieldAccess for sym at nodes.lkt:15589:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15589'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Synthetic_Identifier_Sym;
--# expr-done 11270
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Identifier_P_Name_Symbol;
--# end
--# property-start SyntheticIdentifier.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15591'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Identifier_P_Relative_Name
  (Node : Bare_Synthetic_Identifier
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Synthetic_Identifier  := Bare_Synthetic_Identifier (Node);
     Ent : Internal_Entity_Synthetic_Identifier :=
       Internal_Entity_Synthetic_Identifier'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Cast_Expr : Internal_Entity_Synthetic_Identifier;
Cast_Result : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synthetic_Identifier_P_Relative_Name;
--# end
   --
   --  Primitives for Bare_Target_Name
   --
      procedure Initialize_Fields_For_Target_Name
        (Self : Bare_Target_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Target_Name_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Target_Name;
--# property-start TargetName.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15598'
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Ref_Var
  (Node : Bare_Target_Name
  )
   return Logic_Var
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11271 '<FieldAccess for r_ref_var at nodes.lkt:15598:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15598'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Target_Name_R_Ref_Var'Unrestricted_Access;
--# expr-done 11271
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Target_Name_P_Ref_Var;
--# end
--# property-start TargetName.assign_statement 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15600'
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Assign_Statement
  (Node : Bare_Target_Name
  )
   return Bare_Assign_Stmt
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_1333 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_1333 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1333;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11275 '<Cast to AssignStmt at nodes.lkt:15601:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15601'
--# expr-start 11274 '<Find at nodes.lkt:15601:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15601'
   --# expr-start 11272 '<FieldAccess for parents at nodes.lkt:15601:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15601'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 11272
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 11273 '<IsA AssignStmt at nodes.lkt:15601:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15601'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Assign_Stmt_Range; 
--# expr-done 11273
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11274
   if Find_Result = null
      or else Find_Result.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Find_Result;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11275
         Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_1333;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1333;
            raise;
      end;
   return Property_Result;
end Target_Name_P_Assign_Statement;
--# end
--# property-start TargetName.relative_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15603'
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Relative_Name
  (Node : Bare_Target_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Name;
      Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11279 '<FieldAccess for relative_name at nodes.lkt:15604:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15604'
--# expr-start 11278 '<as_entity at nodes.lkt:15604:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15604'
--# expr-start 11277 '<FieldAccess for dest at nodes.lkt:15604:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15604'
--# expr-start 11276 '<FieldAccess for assign_statement at nodes.lkt:15604:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15604'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 11276
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 11277
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 11278
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11279
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Target_Name_P_Relative_Name;
--# end
--# property-start TargetName.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15607'
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Designated_Env
  (Node : Bare_Target_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; No_Visibility : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind no_visibility No_Visibility
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1335 with Inline_Always;
      Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1335 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1335;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11283 '<FieldAccess for designated_env at nodes.lkt:15608:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15608'
--# expr-start 11282 '<as_entity at nodes.lkt:15608:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15608'
--# expr-start 11281 '<FieldAccess for dest at nodes.lkt:15608:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15608'
--# expr-start 11280 '<FieldAccess for assign_statement at nodes.lkt:15608:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15608'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 11280
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 11281
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 11282
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result.Node, Env => Env, Origin => Origin, No_Visibility => No_Visibility, E_Info => If_Result.Info);
--# end
--# expr-done 11283
         Property_Result := Fld_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1335;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1335;
            raise;
      end;
   return Property_Result;
end Target_Name_P_Designated_Env;
--# end
--# property-start TargetName.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15611'
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Xref_Equation
  (Node : Bare_Target_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Target_Name  := Bare_Target_Name (Node);
     Ent : Internal_Entity_Target_Name :=
       Internal_Entity_Target_Name'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1336 with Inline_Always;
      Local_Dest : Bare_Name;
Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
Fld_2 : Logic_Var;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr_1 : Bare_Name;
Cast_Result_1 : Bare_Ada_Node;
Fld_6 : Logic_Var;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1336 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1336;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11293 '<Let at nodes.lkt:15616:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15616'
--# scope-start
--# expr-start 11285 '<FieldAccess for dest at nodes.lkt:15616:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15616'
--# expr-start 11284 '<FieldAccess for assign_statement at nodes.lkt:15616:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15616'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Ent.Node);
--# end
--# expr-done 11284
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 11285
Local_Dest := Fld_1; 
--# bind dest Local_Dest
--# expr-start 11292 '<And at nodes.lkt:15618:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
--# expr-start 11288 '<FieldAccess for bind_to_non_local at nodes.lkt:15618:9>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11286 '<FieldAccess for type_var at nodes.lkt:15618:32>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11286
Cast_Expr := Local_Dest; 
      Cast_Result := Cast_Expr;
--# expr-start 11287 '<FieldAccess for type_var at nodes.lkt:15618:55>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Local_Dest = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Local_Dest);
--# expr-done 11287
--# property-call-start AdaNode.bind_to_non_local
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bind_To_Non_Local (Node => Self, Dest => Fld_2, Outer_Node => Cast_Result, Node_Var => Fld_3, Entry_Point => Entry_Point);
--# end
--# expr-done 11288
--# expr-start 11291 '<FieldAccess for bind_to_non_local at nodes.lkt:15618:76>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11289 '<FieldAccess for ref_var at nodes.lkt:15618:99>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 11289
Cast_Expr_1 := Local_Dest; 
      Cast_Result_1 := Cast_Expr_1;
--# expr-start 11290 '<FieldAccess for ref_var at nodes.lkt:15618:121>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15618'
      if Local_Dest = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Local_Dest);
--# end
--# expr-done 11290
--# property-call-start AdaNode.bind_to_non_local
Fld_7 := Libadalang.Implementation.Ada_Node_P_Bind_To_Non_Local (Node => Self, Dest => Fld_5, Outer_Node => Cast_Result_1, Node_Var => Fld_6, Entry_Point => Entry_Point);
--# end
--# expr-done 11291
And_Pred := Create_And (Fld_4, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15618:9") else null)); 
--# expr-done 11292
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11293
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1336;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1336;
            raise;
      end;
   return Property_Result;
end Target_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Update_Attribute_Ref
   --
      procedure Initialize_Fields_For_Update_Attribute_Ref
        (Self : Bare_Update_Attribute_Ref
         ; Update_Attribute_Ref_F_Prefix : Bare_Name
         ; Update_Attribute_Ref_F_Attribute : Bare_Identifier
         ; Update_Attribute_Ref_F_Values : Bare_Base_Aggregate
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Update_Attribute_Ref_F_Prefix := Update_Attribute_Ref_F_Prefix;
            Self.Update_Attribute_Ref_F_Attribute := Update_Attribute_Ref_F_Attribute;
            Self.Update_Attribute_Ref_F_Values := Update_Attribute_Ref_F_Values;
      Self.Update_Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Update_Attribute_Ref;
   function Update_Attribute_Ref_F_Prefix
     (Node : Bare_Update_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Update_Attribute_Ref_F_Prefix;
   end;
   function Update_Attribute_Ref_F_Attribute
     (Node : Bare_Update_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Update_Attribute_Ref_F_Attribute;
   end;
   function Update_Attribute_Ref_F_Values
     (Node : Bare_Update_Attribute_Ref) return Bare_Base_Aggregate
   is
   begin
         return Node.Update_Attribute_Ref_F_Values;
   end;
--# property-start UpdateAttributeRef.ref_var 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15630'
pragma Warnings (Off, "is not referenced");
 function Update_Attribute_Ref_P_Ref_Var
  (Node : Bare_Update_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Update_Attribute_Ref  := Bare_Update_Attribute_Ref (Node);
      --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11294 '<FieldAccess for r_ref_var at nodes.lkt:15630:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15630'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Update_Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 11294
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Update_Attribute_Ref_P_Ref_Var;
--# end
--# property-start UpdateAttributeRef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15633'
pragma Warnings (Off, "is not referenced");
 function Update_Attribute_Ref_P_Xref_Equation
  (Node : Bare_Update_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Update_Attribute_Ref  := Bare_Update_Attribute_Ref (Node);
     Ent : Internal_Entity_Update_Attribute_Ref :=
       Internal_Entity_Update_Attribute_Ref'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1338 with Inline_Always;
      Ignored : Boolean;
Local_Prefix_Type : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Aggregate;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1338 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1338;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11308 '<Let at nodes.lkt:15638:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15638'
--# scope-start
--# expr-start 11296 '<FieldAccess for resolve_names_internal at nodes.lkt:15638:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15638'
--# expr-start 11295 '<FieldAccess for prefix at nodes.lkt:15638:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15638'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Update_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 11295
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 11296
Ignored := Fld_1; 
--# bind _ Ignored
--# expr-start 11307 '<Let at nodes.lkt:15639:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15639'
--# scope-start
--# expr-start 11299 '<Cast to BaseTypeDecl at nodes.lkt:15639:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15639'
--# expr-start 11298 '<FieldAccess for type_val at nodes.lkt:15639:27>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15639'
--# expr-start 11297 '<FieldAccess for prefix at nodes.lkt:15639:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15639'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Update_Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 11297
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_2.Node);
--# end
--# expr-done 11298
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 11299
Local_Prefix_Type := Cast_Result; 
--# bind prefix_type Local_Prefix_Type
--# expr-start 11306 '<And at nodes.lkt:15641:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
--# expr-start 11303 '<Bind at nodes.lkt:15641:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
--# expr-start 11302 '<FieldAccess for type_var at nodes.lkt:15641:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
--# expr-start 11301 '<Cast to Aggregate at nodes.lkt:15641:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
--# expr-start 11300 '<FieldAccess for values at nodes.lkt:15641:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Aggregate (Node => Ent.Node.Update_Attribute_Ref_F_Values, Info => Ent.Info);
--# expr-done 11300
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11301
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result_1.Node);
--# expr-done 11302
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Cast_Expr := Local_Prefix_Type; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15641:9") else null)); 
--# expr-done 11303
--# expr-start 11305 '<Bind at nodes.lkt:15641:69>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
--# expr-start 11304 '<FieldAccess for type_var at nodes.lkt:15641:73>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15641'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11304
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
Cast_Expr_1 := Local_Prefix_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_6, Cast_Result_3, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15641:69") else null)); 
--# expr-done 11305
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15641:9") else null)); 
--# expr-done 11306
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11307
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11308
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1338;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1338;
            raise;
      end;
   return Property_Result;
end Update_Attribute_Ref_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Paren_Expr
   --
      procedure Initialize_Fields_For_Paren_Expr
        (Self : Bare_Paren_Expr
         ; Paren_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Paren_Expr_F_Expr := Paren_Expr_F_Expr;
      end Initialize_Fields_For_Paren_Expr;
   function Paren_Expr_F_Expr
     (Node : Bare_Paren_Expr) return Bare_Expr
   is
   begin
         return Node.Paren_Expr_F_Expr;
   end;
--# property-start ParenExpr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15649'
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Has_Context_Free_Type
  (Node : Bare_Paren_Expr
  )
   return Boolean
is
   Self : Bare_Paren_Expr  := Bare_Paren_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11310 '<FieldAccess for has_context_free_type at nodes.lkt:15649:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15649'
--# expr-start 11309 '<FieldAccess for expr at nodes.lkt:15649:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15649'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Paren_Expr_F_Expr;
--# expr-done 11309
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.has_context_free_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Has_Context_Free_Type (Node => Fld);
--# end
--# expr-done 11310
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Paren_Expr_P_Has_Context_Free_Type;
--# end
--# property-start ParenExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15652'
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Xref_Equation
  (Node : Bare_Paren_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Paren_Expr  := Bare_Paren_Expr (Node);
     Ent : Internal_Entity_Paren_Expr :=
       Internal_Entity_Paren_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1340 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Fld_7 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1340 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1340;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11322 '<And at nodes.lkt:15652:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15652'
--# expr-start 11317 '<And at nodes.lkt:15653:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
--# expr-start 11312 '<FieldAccess for sub_equation at nodes.lkt:15653:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
--# expr-start 11311 '<FieldAccess for expr at nodes.lkt:15653:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Paren_Expr_F_Expr, Info => Ent.Info);
--# expr-done 11311
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11312
--# expr-start 11316 '<Bind at nodes.lkt:15653:38>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
--# expr-start 11314 '<FieldAccess for expected_type_var at nodes.lkt:15653:42>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
--# expr-start 11313 '<FieldAccess for expr at nodes.lkt:15653:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Paren_Expr_F_Expr;
--# expr-done 11313
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11314
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11315 '<FieldAccess for expected_type_var at nodes.lkt:15653:73>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15653'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11315
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Bind_Result := Solver.Create_Unify (Fld_3, Fld_4, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15653:38") else null)); 
--# expr-done 11316
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15653:9") else null)); 
--# expr-done 11317
--# expr-start 11321 '<Bind at nodes.lkt:15654:11>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15654'
--# expr-start 11319 '<FieldAccess for type_var at nodes.lkt:15654:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15654'
--# expr-start 11318 '<FieldAccess for expr at nodes.lkt:15654:15>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15654'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Paren_Expr_F_Expr;
--# expr-done 11318
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_5);
--# expr-done 11319
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 11320 '<FieldAccess for type_var at nodes.lkt:15654:37>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15654'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11320
Fld_7.Value := No_Entity;
Entity_Vars.Reset (Fld_7);
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_7, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15654:11") else null)); 
--# expr-done 11321
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15652:37") else null)); 
--# expr-done 11322
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1340;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1340;
            raise;
      end;
   return Property_Result;
end Paren_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Quantified_Expr
   --
      procedure Quantified_Expr_Pre_Env_Actions
        (Self            : Bare_Quantified_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_216 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Quantified_Expr
        (Self : Bare_Quantified_Expr
         ; Quantified_Expr_F_Quantifier : Bare_Quantifier
         ; Quantified_Expr_F_Loop_Spec : Bare_For_Loop_Spec
         ; Quantified_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Quantified_Expr_F_Quantifier := Quantified_Expr_F_Quantifier;
            Self.Quantified_Expr_F_Loop_Spec := Quantified_Expr_F_Loop_Spec;
            Self.Quantified_Expr_F_Expr := Quantified_Expr_F_Expr;
      end Initialize_Fields_For_Quantified_Expr;
   function Quantified_Expr_F_Quantifier
     (Node : Bare_Quantified_Expr) return Bare_Quantifier
   is
   begin
         return Node.Quantified_Expr_F_Quantifier;
   end;
   function Quantified_Expr_F_Loop_Spec
     (Node : Bare_Quantified_Expr) return Bare_For_Loop_Spec
   is
   begin
         return Node.Quantified_Expr_F_Loop_Spec;
   end;
   function Quantified_Expr_F_Expr
     (Node : Bare_Quantified_Expr) return Bare_Expr
   is
   begin
         return Node.Quantified_Expr_F_Expr;
   end;
--# property-start QuantifiedExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15664'
pragma Warnings (Off, "is not referenced");
 function Quantified_Expr_P_Xref_Equation
  (Node : Bare_Quantified_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Quantified_Expr  := Bare_Quantified_Expr (Node);
     Ent : Internal_Entity_Quantified_Expr :=
       Internal_Entity_Quantified_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1341 with Inline_Always;
      Local_Spec_Success : Boolean;
Fld : Internal_Entity_For_Loop_Spec;
Fld_1 : Boolean;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1341 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1341;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11343 '<Let at nodes.lkt:15667:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15667'
--# scope-start
--# expr-start 11324 '<FieldAccess for resolve_names at nodes.lkt:15667:28>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15667'
--# expr-start 11323 '<FieldAccess for loop_spec at nodes.lkt:15667:28>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15667'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Quantified_Expr_F_Loop_Spec, Info => Ent.Info);
--# expr-done 11323
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 11324
Local_Spec_Success := Fld_1; 
--# bind spec_success Local_Spec_Success
--# expr-start 11342 '<If at nodes.lkt:15669:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15669'
if Local_Spec_Success then
   --# expr-start 11341 '<And at nodes.lkt:15669:30>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15669'
--# expr-start 11336 '<And at nodes.lkt:15670:13>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15670'
--# expr-start 11333 '<And at nodes.lkt:15671:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15671'
--# expr-start 11328 '<Bind at nodes.lkt:15671:17>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15671'
--# expr-start 11326 '<FieldAccess for expected_type_var at nodes.lkt:15671:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15671'
--# expr-start 11325 '<FieldAccess for expr at nodes.lkt:15671:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15671'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Quantified_Expr_F_Expr;
--# expr-done 11325
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11326
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11327 '<FieldAccess for bool_type at nodes.lkt:15671:52>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15671'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11327
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15671:17") else null)); 
--# expr-done 11328
--# expr-start 11332 '<bind at nodes.lkt:15672:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15672'
--# expr-start 11329 '<FieldAccess for children_env at nodes.lkt:15672:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15672'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11329
Bound_Env := Fld_5; Inc_Ref (Bound_Env);
--# expr-start 11331 '<FieldAccess for sub_equation at nodes.lkt:15674:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15674'
--# expr-start 11330 '<FieldAccess for expr at nodes.lkt:15674:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15674'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 11330
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 11331
Dyn_Var_Bind_Result := Fld_7; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11332
And_Pred := Create_And (Bind_Result, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15671:17") else null)); 
--# expr-done 11333
--# expr-start 11335 '<FieldAccess for matches_expected_formal_type at nodes.lkt:15676:19>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15676'
--# expr-start 11334 '<FieldAccess for expr at nodes.lkt:15676:19>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15676'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 11334
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 11335
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15670:13") else null)); 
--# expr-done 11336
--# expr-start 11340 '<Bind at nodes.lkt:15677:15>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15677'
--# expr-start 11337 '<FieldAccess for type_var at nodes.lkt:15677:19>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15677'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11337
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 11339 '<FieldAccess for type_var at nodes.lkt:15677:36>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15677'
--# expr-start 11338 '<FieldAccess for expr at nodes.lkt:15677:36>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15677'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Quantified_Expr_F_Expr;
--# expr-done 11338
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 11339
Fld_12.Value := No_Entity;
Entity_Vars.Reset (Fld_12);
Bind_Result_1 := Solver.Create_Unify (Fld_10, Fld_12, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15677:15") else null)); 
--# expr-done 11340
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15669:30") else null)); 
--# expr-done 11341
   If_Result := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15677:63") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 11342
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 11343
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1341;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1341;
            raise;
      end;
   return Property_Result;
end Quantified_Expr_P_Xref_Equation;
--# end
--# property-start QuantifiedExpr._env_trans_parent_216 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_216
  (Node : Bare_Quantified_Expr
  )
   return Boolean
is
   Self : Bare_Quantified_Expr  := Bare_Quantified_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_216;
--# end
   --
   --  Primitives for Bare_Raise_Expr
   --
      procedure Initialize_Fields_For_Raise_Expr
        (Self : Bare_Raise_Expr
         ; Raise_Expr_F_Exception_Name : Bare_Name
         ; Raise_Expr_F_Error_Message : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Raise_Expr_F_Exception_Name := Raise_Expr_F_Exception_Name;
            Self.Raise_Expr_F_Error_Message := Raise_Expr_F_Error_Message;
      end Initialize_Fields_For_Raise_Expr;
   function Raise_Expr_F_Exception_Name
     (Node : Bare_Raise_Expr) return Bare_Name
   is
   begin
         return Node.Raise_Expr_F_Exception_Name;
   end;
   function Raise_Expr_F_Error_Message
     (Node : Bare_Raise_Expr) return Bare_Expr
   is
   begin
         return Node.Raise_Expr_F_Error_Message;
   end;
--# property-start RaiseExpr.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15690'
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Has_Context_Free_Type
  (Node : Bare_Raise_Expr
  )
   return Boolean
is
   Self : Bare_Raise_Expr  := Bare_Raise_Expr (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Raise_Expr_P_Has_Context_Free_Type;
--# end
--# property-start RaiseExpr.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15693'
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Xref_Equation
  (Node : Bare_Raise_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Expr  := Bare_Raise_Expr (Node);
     Ent : Internal_Entity_Raise_Expr :=
       Internal_Entity_Raise_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1343 with Inline_Always;
            procedure Finalizer_Scope_3472 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1343 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1343;
            procedure Finalizer_Scope_3472 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3472;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11356 '<And at nodes.lkt:15694:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
--# expr-start 11349 '<And at nodes.lkt:15694:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
--# expr-start 11345 '<FieldAccess for sub_equation at nodes.lkt:15694:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
--# expr-start 11344 '<FieldAccess for exception_name at nodes.lkt:15694:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Expr_F_Exception_Name, Info => Ent.Info);
--# expr-done 11344
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11345
--# expr-start 11348 '<Bind at nodes.lkt:15694:48>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
--# expr-start 11346 '<FieldAccess for expected_type_var at nodes.lkt:15694:52>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11346
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 11347 '<FieldAccess for type_var at nodes.lkt:15694:78>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11347
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Unify (Fld_2, Fld_3, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15694:48") else null)); 
--# expr-done 11348
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15694:9") else null)); 
--# expr-done 11349
--# expr-start 11350 '<FieldAccess for error_message at nodes.lkt:15694:99>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15694'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Expr_F_Error_Message, Info => Ent.Info);
--# expr-done 11350
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr_0
--# expr-start 11355 '<And at nodes.lkt:15695:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15695'
--# expr-start 11353 '<Bind at nodes.lkt:15698:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15698'
--# expr-start 11351 '<FieldAccess for expected_type_var at nodes.lkt:15698:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15698'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 11351
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 11352 '<FieldAccess for std_string_type at nodes.lkt:15698:45>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15698'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 11352
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15698:17") else null)); 
--# expr-done 11353
--# expr-start 11354 '<FieldAccess for sub_equation at nodes.lkt:15699:19>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15699'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11354
And_Pred_1 := Create_And (Bind_Result_1, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15695:21") else null)); 
--# expr-done 11355
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3472;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15699:50") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_2 := Create_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15694:9") else null)); 
--# expr-done 11356
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1343;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1343;
                     Finalizer_Scope_3472;
            raise;
      end;
   return Property_Result;
end Raise_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Un_Op
   --
      procedure Initialize_Fields_For_Un_Op
        (Self : Bare_Un_Op
         ; Un_Op_F_Op : Bare_Op
         ; Un_Op_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Un_Op_F_Op := Un_Op_F_Op;
            Self.Un_Op_F_Expr := Un_Op_F_Expr;
      end Initialize_Fields_For_Un_Op;
   function Un_Op_F_Op
     (Node : Bare_Un_Op) return Bare_Op
   is
   begin
         return Node.Un_Op_F_Op;
   end;
   function Un_Op_F_Expr
     (Node : Bare_Un_Op) return Bare_Expr
   is
   begin
         return Node.Un_Op_F_Expr;
   end;
--# property-start UnOp.has_context_free_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15712'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Has_Context_Free_Type
  (Node : Bare_Un_Op
  )
   return Boolean
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Un_Op_P_Has_Context_Free_Type;
--# end
--# property-start UnOp.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15715'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Xref_Equation
  (Node : Bare_Un_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1345 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Expr;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Op;
Cast_Result : Internal_Entity;
New_Struct : Internal_Logic_Context;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Sym : Symbol_Type;
Fld_10 : Boolean;
Sym_1 : Symbol_Type;
Fld_11 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1345 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1345;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11377 '<And at nodes.lkt:15715:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15715'
--# expr-start 11358 '<FieldAccess for sub_equation at nodes.lkt:15715:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15715'
--# expr-start 11357 '<FieldAccess for expr at nodes.lkt:15715:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15715'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 11357
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11358
--# expr-start 11376 '<If at nodes.lkt:15716:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
--# expr-start 11360 '<FieldAccess for in_aspect at nodes.lkt:15716:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11359 '<Symbol Depends at nodes.lkt:15716:27>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Depends); 
--# expr-done 11359
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 11360
if Fld_10 then
   If_Result := True;
else
   --# expr-start 11362 '<FieldAccess for in_aspect at nodes.lkt:15716:42>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11361 '<Symbol Refined_Depends at nodes.lkt:15716:57>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Refined_Depends); 
--# expr-done 11361
--# property-call-start AdaNode.in_aspect
Fld_11 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 11362
   If_Result := Fld_11;
end if;
if If_Result then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15716:82") else null)); 
   If_Result_1 := True_Rel;
else
   --# expr-start 11375 '<Or at nodes.lkt:15716:93>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
--# expr-start 11363 '<FieldAccess for overload_equation at nodes.lkt:15716:93>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15716'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnOp.overload_equation
Fld_2 := Libadalang.Implementation.Un_Op_P_Overload_Equation (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 11363
--# expr-start 11374 '<And at nodes.lkt:15717:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11367 '<Bind at nodes.lkt:15717:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11364 '<FieldAccess for expected_type_var at nodes.lkt:15717:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Self);
--# expr-done 11364
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11366 '<FieldAccess for expected_type_var at nodes.lkt:15717:43>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11365 '<FieldAccess for expr at nodes.lkt:15717:43>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Un_Op_F_Expr;
--# expr-done 11365
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_4);
--# expr-done 11366
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Bind_Result := Solver.Create_Unify (Fld_3, Fld_5, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15717:13") else null)); 
--# expr-done 11367
--# expr-start 11373 '<Bind at nodes.lkt:15717:78>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11368 '<FieldAccess for type_var at nodes.lkt:15717:82>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11368
Fld_6.Value := No_Entity;
Entity_Vars.Reset (Fld_6);
--# expr-start 11370 '<FieldAccess for type_var at nodes.lkt:15717:99>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11369 '<FieldAccess for expr at nodes.lkt:15717:99>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Un_Op_F_Expr;
--# expr-done 11369
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 11370
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 11372 '<New LogicContext at nodes.lkt:15717:131>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15717'
--# expr-start 11371 '<FieldAccess for op at nodes.lkt:15718:26>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15718'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 11371
      Cast_Result := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
New_Struct := (Decl_Node => No_Entity, Ref_Node => Cast_Result); 
--# expr-done 11372
Logic_Ctx := Allocate_Logic_Context (New_Struct); 
Bind_Result_1 := Solver.Create_Unify (Fld_6, Fld_8, Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15717:78") else null)); 
--# expr-done 11373
And_Pred := Create_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15717:13") else null)); 
--# expr-done 11374
Or_Pred := Create_Or (Fld_2, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15716:93") else null)); 
--# expr-done 11375
   If_Result_1 := Or_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 11376
And_Pred_1 := Create_And (Fld_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15715:37") else null)); 
--# expr-done 11377
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1345;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1345;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Xref_Equation;
--# end
--# property-start UnOp.overload_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15724'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Overload_Equation
  (Node : Bare_Un_Op
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1346 with Inline_Always;
            procedure Finalizer_Scope_3473 with Inline_Always;
      Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Context;
Bound_Logic_Context : Internal_Logic_Context;
Fld_3 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1346 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1346;
            procedure Finalizer_Scope_3473 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3473;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11385 '<LogicAny at nodes.lkt:15724:41>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15724'
--# expr-start 11384 '<Map at nodes.lkt:15724:41>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15724'
   --# expr-start 11379 '<FieldAccess for subprograms at nodes.lkt:15724:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15724'
--# expr-start 11378 '<FieldAccess for op at nodes.lkt:15724:41>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15724'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 11378
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 11379
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_0
      --# expr-start 11383 '<bind at nodes.lkt:15726:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15726'
--# expr-start 11381 '<New LogicContext at nodes.lkt:15726:34>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15726'
Cast_Expr := Item_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 11380 '<FieldAccess for op at nodes.lkt:15726:56>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15726'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 11380
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
New_Struct := (Decl_Node => Cast_Result_1, Ref_Node => Cast_Result); 
--# expr-done 11381
Bound_Logic_Context := New_Struct; 
--# expr-start 11382 '<FieldAccess for entity_eq at nodes.lkt:15728:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15728'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UnOp.entity_eq
Fld_3 := Libadalang.Implementation.Un_Op_P_Entity_Eq (Node => Ent.Node, Subp => Item_0, Origin => Origin, Env => Env, Logic_Context => Bound_Logic_Context, E_Info => Ent.Info);
--# end
--# expr-done 11382
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 11383
         declare
            Item_To_Append : constant Logic_Equation := Dyn_Var_Bind_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3473;
         end loop;
      end;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11384
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15724:41") else null)); 
--# expr-done 11385
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1346;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1346;
                     Finalizer_Scope_3473;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Overload_Equation;
--# end
--# property-start UnOp.entity_eq 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15733'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Entity_Eq
  (Node : Bare_Un_Op
      ; Subp : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Logic_Context : Internal_Logic_Context
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind subp Subp
   --# bind origin Origin
   --# bind env Env
   --# bind logic_context Logic_Context
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1347 with Inline_Always;
      Local_Spec : Internal_Entity_Base_Subp_Spec;
Local_Ps : Internal_Entity_Defining_Name_Array_Access;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Logic_Ctx : Internal_Logic_Context_Access;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Op;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_9 : Bare_Op;
Fld_10 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
And_Pred_2 : Logic_Equation;
False_Rel : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1347 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Ps);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (False_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1347;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11408 '<Let at nodes.lkt:15734:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15734'
--# scope-start
--# expr-start 11386 '<FieldAccess for subp_spec_or_null at nodes.lkt:15734:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15734'
      if Subp.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Subp.Node, E_Info => Subp.Info);
--# end
--# expr-done 11386
Local_Spec := Fld; 
--# bind spec Local_Spec
--# expr-start 11407 '<Let at nodes.lkt:15735:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15735'
--# scope-start
--# expr-start 11387 '<FieldAccess for unpacked_formal_params at nodes.lkt:15735:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15735'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Local_Spec.Node, E_Info => Local_Spec.Info);
--# end
--# expr-done 11387
Local_Ps := Fld_1; Inc_Ref (Local_Ps);
--# bind ps Local_Ps
--# expr-start 11406 '<If at nodes.lkt:15737:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15737'
--# expr-start 11389 '<Eq at nodes.lkt:15737:12>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15737'
--# expr-start 11388 '<length at nodes.lkt:15737:12>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15737'
Len := Length (Local_Ps); 
--# expr-done 11388
Is_Equal := Len = 1; 
--# expr-done 11389
if Is_Equal then
   --# expr-start 11405 '<And at nodes.lkt:15740:13>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15740'
--# expr-start 11401 '<And at nodes.lkt:15741:17>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15741'
--# expr-start 11397 '<And at nodes.lkt:15742:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15742'
--# expr-start 11393 '<FieldAccess for call_argument_equation at nodes.lkt:15742:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15742'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11391 '<FieldAccess for formal_decl at nodes.lkt:15742:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15742'
--# expr-start 11390 '<at at nodes.lkt:15742:49>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15742'
Get_Result := Get (Self, Local_Ps, 0, True); 
--# expr-done 11390
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.formal_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Formal_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 11391
--# expr-start 11392 '<FieldAccess for expr at nodes.lkt:15742:71>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15742'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 11392
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Local_Spec.Node, Param => Fld_2, Arg => Fld_3, Origin => Origin, Logic_Context => Logic_Context, E_Info => Local_Spec.Info);
--# end
--# expr-done 11393
--# expr-start 11396 '<Bind at nodes.lkt:15744:25>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15744'
--# expr-start 11394 '<FieldAccess for type_var at nodes.lkt:15744:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15744'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 11394
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 11395 '<FieldAccess for return_type at nodes.lkt:15744:46>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15744'
      if Local_Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_6 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Local_Spec.Node, Origin => Origin, E_Info => Local_Spec.Info);
--# end
--# expr-done 11395
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Logic_Ctx := Allocate_Logic_Context (Logic_Context); 
Bind_Result := Solver.Create_Assign (Fld_5, Cast_Result, Solver_Ifc.No_Converter, Logic_Ctx, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15744:25") else null)); 
--# expr-done 11396
And_Pred := Create_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15742:21") else null)); 
--# expr-done 11397
--# expr-start 11400 '<Bind at nodes.lkt:15748:21>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15748'
--# expr-start 11399 '<FieldAccess for ref_var at nodes.lkt:15748:25>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15748'
--# expr-start 11398 '<FieldAccess for op at nodes.lkt:15748:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Un_Op_F_Op;
--# expr-done 11398
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 11399
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
Cast_Expr := Subp; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_8, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15748:21") else null)); 
--# expr-done 11400
And_Pred_1 := Create_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15741:17") else null)); 
--# expr-done 11401
--# expr-start 11404 '<Bind at nodes.lkt:15750:19>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15750'
--# expr-start 11403 '<FieldAccess for subp_spec_var at nodes.lkt:15750:23>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15750'
--# expr-start 11402 '<FieldAccess for op at nodes.lkt:15750:23>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15750'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Un_Op_F_Op;
--# expr-done 11402
      if Fld_9 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld_9);
--# end
--# expr-done 11403
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
Cast_Expr_1 := Local_Spec; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_10, Cast_Result_2, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15750:19") else null)); 
--# expr-done 11404
And_Pred_2 := Create_And (And_Pred_1, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15740:13") else null)); 
--# expr-done 11405
   If_Result := And_Pred_2;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15751:16") else null)); 
   If_Result := False_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 11406
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 11407
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11408
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1347;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1347;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Entity_Eq;
--# end
--# property-start UnOp.potential_actuals_for_dispatch 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15755'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Un_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1348 with Inline_Always;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Array_Lit : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_1348 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1348;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11414 '<ArrayLiteral at nodes.lkt:15755:98>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15755'
--# expr-start 11413 '<New ExpectedTypeForExpr at nodes.lkt:15755:99>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15755'
--# expr-start 11411 '<FieldAccess for type_expression at nodes.lkt:15756:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15756'
--# expr-start 11410 '<at at nodes.lkt:15756:23>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15756'
--# expr-start 11409 '<FieldAccess for abstract_formal_params at nodes.lkt:15756:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15756'
      if Spec.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 11409
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 11410
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 11411
--# expr-start 11412 '<FieldAccess for expr at nodes.lkt:15756:81>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15756'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 11412
New_Struct := (Expected_Type => Fld_1, Expr => Fld_2); 
--# expr-done 11413
Array_Lit := Create_Internal_Expected_Type_For_Expr_Array (Internal_Internal_Expected_Type_For_Expr_Array'(1 => New_Struct)); 
--# expr-done 11414
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1348;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1348;
            raise;
      end;
   return Property_Result;
end Un_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start UnOp.is_dispatching_call 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15760'
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Is_Dispatching_Call
  (Node : Bare_Un_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Un_Op  := Bare_Un_Op (Node);
     Ent : Internal_Entity_Un_Op :=
       Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Op;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11416 '<FieldAccess for is_dispatching_call at nodes.lkt:15760:39>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15760'
--# expr-start 11415 '<FieldAccess for op at nodes.lkt:15760:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15760'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 11415
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 11416
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Un_Op_P_Is_Dispatching_Call;
--# end
   --
   --  Primitives for Bare_For_Loop_Iter_Filter
   --
      procedure Initialize_Fields_For_For_Loop_Iter_Filter
        (Self : Bare_For_Loop_Iter_Filter
         ; For_Loop_Iter_Filter_F_Expr : Bare_Expr
        ) is
      begin
            Self.For_Loop_Iter_Filter_F_Expr := For_Loop_Iter_Filter_F_Expr;
      end Initialize_Fields_For_For_Loop_Iter_Filter;
   function For_Loop_Iter_Filter_F_Expr
     (Node : Bare_For_Loop_Iter_Filter) return Bare_Expr
   is
   begin
         return Node.For_Loop_Iter_Filter_F_Expr;
   end;
--# property-start ForLoopIterFilter.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15772'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Iter_Filter_P_Xref_Equation
  (Node : Bare_For_Loop_Iter_Filter
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Iter_Filter  := Bare_For_Loop_Iter_Filter (Node);
     Ent : Internal_Entity_For_Loop_Iter_Filter :=
       Internal_Entity_For_Loop_Iter_Filter'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1350 with Inline_Always;
      Ignored : Boolean;
Var_Expr_0 : Internal_Entity_For_Loop_Spec;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_For_Loop_Spec;
Fld_1 : Boolean;
Fld_2 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1350 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1350;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11432 '<Let at nodes.lkt:15773:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15773'
--# scope-start
--# expr-start 11418 '<Cast to ForLoopSpec at nodes.lkt:15773:17>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15773'
--# expr-start 11417 '<parent at nodes.lkt:15773:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15773'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11417
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_For_Loop_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_For_Loop_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_For_Loop_Spec;
   end if;
--# expr-done 11418
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_For_Loop_Spec then
   --# scope-start
   --# bind spec Var_Expr_0
--# expr-start 11421 '<If at nodes.lkt:15774:23>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15774'
--# expr-start 11419 '<FieldAccess for is_iterated_assoc_spec at nodes.lkt:15774:26>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15774'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.is_iterated_assoc_spec
Fld_2 := Libadalang.Implementation.For_Loop_Spec_P_Is_Iterated_Assoc_Spec (Node => Var_Expr_0.Node);
--# end
--# expr-done 11419
if Fld_2 then
   --# expr-start 11420 '<FieldAccess for resolve_names_from_closest_entry_point at nodes.lkt:15774:61>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15774'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11420
   If_Result := Fld_1;
else
   If_Result := True;
end if;
--# expr-done 11421
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Ignored := Result_Var; 
--# bind _ Ignored
--# expr-start 11431 '<And at nodes.lkt:15777:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11428 '<And at nodes.lkt:15777:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11425 '<Bind at nodes.lkt:15777:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11423 '<FieldAccess for expected_type_var at nodes.lkt:15777:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11422 '<FieldAccess for expr at nodes.lkt:15777:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.For_Loop_Iter_Filter_F_Expr;
--# expr-done 11422
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_3);
--# expr-done 11423
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
--# expr-start 11424 '<FieldAccess for bool_type at nodes.lkt:15777:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11424
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Solver.Create_Assign (Fld_4, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15777:9") else null)); 
--# expr-done 11425
--# expr-start 11427 '<FieldAccess for sub_equation at nodes.lkt:15777:66>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11426 '<FieldAccess for expr at nodes.lkt:15777:66>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.For_Loop_Iter_Filter_F_Expr, Info => Ent.Info);
--# expr-done 11426
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 11427
And_Pred := Create_And (Bind_Result, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15777:9") else null)); 
--# expr-done 11428
--# expr-start 11430 '<FieldAccess for matches_expected_formal_type at nodes.lkt:15777:95>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
--# expr-start 11429 '<FieldAccess for expr at nodes.lkt:15777:95>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15777'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.For_Loop_Iter_Filter_F_Expr, Info => Ent.Info);
--# expr-done 11429
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_8.Node, Origin => Origin);
--# end
--# expr-done 11430
And_Pred_1 := Create_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15777:9") else null)); 
--# expr-done 11431
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 11432
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1350;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1350;
            raise;
      end;
   return Property_Result;
end For_Loop_Iter_Filter_P_Xref_Equation;
--# end
--# property-start ForLoopIterFilter.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15780'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Iter_Filter_P_Xref_Entry_Point
  (Node : Bare_For_Loop_Iter_Filter
  )
   return Boolean
is
   Self : Bare_For_Loop_Iter_Filter  := Bare_For_Loop_Iter_Filter (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Iter_Filter_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Format_String_Chunk
   --
      procedure Initialize_Fields_For_Format_String_Chunk
        (Self : Bare_Format_String_Chunk
         ; Format_String_Chunk_F_Expr : Bare_Expr
         ; Format_String_Chunk_F_String_Tok : Bare_Format_String_Tok_Node
        ) is
      begin
            Self.Format_String_Chunk_F_Expr := Format_String_Chunk_F_Expr;
            Self.Format_String_Chunk_F_String_Tok := Format_String_Chunk_F_String_Tok;
      end Initialize_Fields_For_Format_String_Chunk;
   function Format_String_Chunk_F_Expr
     (Node : Bare_Format_String_Chunk) return Bare_Expr
   is
   begin
         return Node.Format_String_Chunk_F_Expr;
   end;
   function Format_String_Chunk_F_String_Tok
     (Node : Bare_Format_String_Chunk) return Bare_Format_String_Tok_Node
   is
   begin
         return Node.Format_String_Chunk_F_String_Tok;
   end;
--# property-start FormatStringChunk.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15789'
pragma Warnings (Off, "is not referenced");
 function Format_String_Chunk_P_Xref_Equation
  (Node : Bare_Format_String_Chunk
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Format_String_Chunk  := Bare_Format_String_Chunk (Node);
     Ent : Internal_Entity_Format_String_Chunk :=
       Internal_Entity_Format_String_Chunk'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1352 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Node_For_Entity : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1352 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1352;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11439 '<And at nodes.lkt:15789:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15789'
--# expr-start 11434 '<FieldAccess for sub_equation at nodes.lkt:15789:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15789'
--# expr-start 11433 '<FieldAccess for expr at nodes.lkt:15789:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15789'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Format_String_Chunk_F_Expr, Info => Ent.Info);
--# expr-done 11433
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11434
--# expr-start 11438 '<Bind at nodes.lkt:15792:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15792'
--# expr-start 11436 '<FieldAccess for expected_type_var at nodes.lkt:15792:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15792'
--# expr-start 11435 '<FieldAccess for expr at nodes.lkt:15792:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15792'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Format_String_Chunk_F_Expr;
--# expr-done 11435
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11436
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 11437 '<No AdaNode at nodes.lkt:15792:44>' Node_For_Entity 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15792'
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 11437
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
Bind_Result := Solver.Create_Assign (Fld_3, If_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15792:9") else null)); 
--# expr-done 11438
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15789:37") else null)); 
--# expr-done 11439
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1352;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1352;
            raise;
      end;
   return Property_Result;
end Format_String_Chunk_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Format_String_Tok_Node
   --
   --
   --  Primitives for Bare_Format_String_Tok_End
   --
   --
   --  Primitives for Bare_Format_String_Tok_Mid
   --
   --
   --  Primitives for Bare_Format_String_Tok_Start
   --
   --
   --  Primitives for Bare_Format_String_Tok_String
   --
   --
   --  Primitives for Bare_Handled_Stmts
   --
      procedure Initialize_Fields_For_Handled_Stmts
        (Self : Bare_Handled_Stmts
         ; Handled_Stmts_F_Stmts : Bare_Stmt_List
         ; Handled_Stmts_F_Exceptions : Bare_Ada_Node_List
        ) is
      begin
            Self.Handled_Stmts_F_Stmts := Handled_Stmts_F_Stmts;
            Self.Handled_Stmts_F_Exceptions := Handled_Stmts_F_Exceptions;
      end Initialize_Fields_For_Handled_Stmts;
   function Handled_Stmts_F_Stmts
     (Node : Bare_Handled_Stmts) return Bare_Stmt_List
   is
   begin
         return Node.Handled_Stmts_F_Stmts;
   end;
   function Handled_Stmts_F_Exceptions
     (Node : Bare_Handled_Stmts) return Bare_Ada_Node_List
   is
   begin
         return Node.Handled_Stmts_F_Exceptions;
   end;
--# property-start HandledStmts.spark_mode_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15827'
pragma Warnings (Off, "is not referenced");
 function Handled_Stmts_P_Spark_Mode_Aspect
  (Node : Bare_Handled_Stmts
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Handled_Stmts  := Bare_Handled_Stmts (Node);
     Ent : Internal_Entity_Handled_Stmts :=
       Internal_Entity_Handled_Stmts'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Aspect;
            procedure Finalizer_Scope_1353 with Inline_Always;
      Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity_Package_Body;
Fld : Internal_Entity_Stmt_List;
Fld_1 : Bare_Stmt_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_4 : Boolean;
Find_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Pragma_Node;
Fld_5 : Internal_Aspect;
Scope_Result : Internal_Aspect;
Fld_6 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Package_Body;
Fld_7 : Internal_Entity_Declarative_Part;
Fld_8 : Internal_Aspect;
Scope_Result_1 : Internal_Aspect;
Fld_9 : Internal_Entity;
Fld_10 : Internal_Aspect;
Result_Var : Internal_Aspect;
Result_Var_1 : Internal_Aspect;
            procedure Finalizer_Scope_1353 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1353;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11447 '<Find at nodes.lkt:15828:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15828'
   --# expr-start 11442 '<Map at nodes.lkt:15828:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15828'
   --# expr-start 11440 '<FieldAccess for stmts at nodes.lkt:15828:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15828'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Stmt_List (Node => Ent.Node.Handled_Stmts_F_Stmts, Info => Ent.Info);
--# expr-done 11440
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
            Take_While_Ok : Boolean := False;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Stmt_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind stmt Item_0
         --# expr-start 11441 '<IsA Pragma at nodes.lkt:15828:41>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15828'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Pragma_Node_Range; 
--# expr-done 11441
         Take_While_Ok := Is_A;
         if Take_While_Ok then
         declare
            Item_To_Append : constant Internal_Entity := Item_0;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
               exit when not Take_While_Ok;
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11442
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind stmt Item_1
      --# expr-start 11446 '<FieldAccess for name_is at nodes.lkt:15829:23>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15829'
--# expr-start 11444 '<FieldAccess for id at nodes.lkt:15829:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15829'
--# expr-start 11443 '<Cast to Pragma at nodes.lkt:15829:23>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15829'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 11443
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Cast_Result.Node.Pragma_Node_F_Id, Info => Cast_Result.Info);
--# expr-done 11444
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11445 '<Symbol SPARK_Mode at nodes.lkt:15829:50>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15829'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 11445
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 11446
      if Fld_4 then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Fld_4;
         end loop;
      end;
--# expr-done 11447
Var_Expr_2 := Find_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind spark_mode Var_Expr_2
--# expr-start 11449 '<FieldAccess for as_aspect at nodes.lkt:15831:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15831'
--# expr-start 11448 '<Cast to Pragma at nodes.lkt:15831:29>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15831'
Cast_Expr_1 := Var_Expr_2; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Pragma_Node;
   end if;
--# expr-done 11448
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.as_aspect
Fld_5 := Libadalang.Implementation.Pragma_Node_P_As_Aspect (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 11449
Scope_Result := Fld_5;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 11451 '<Cast to PackageBody at nodes.lkt:15833:25>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15833'
--# expr-start 11450 '<parent at nodes.lkt:15833:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15833'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11450
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Package_Body
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_2 := No_Entity_Package_Body;
   end if;
--# expr-done 11451
Var_Expr_3 := Cast_Result_2;
if Var_Expr_3 /= No_Entity_Package_Body then
   --# scope-start
   --# bind body Var_Expr_3
--# expr-start 11453 '<FieldAccess for spark_mode_aspect at nodes.lkt:15834:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15834'
--# expr-start 11452 '<FieldAccess for decls at nodes.lkt:15834:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15834'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_3.Node.Package_Body_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 11452
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 11453
Scope_Result_1 := Fld_8;
   --# end
   Result_Var := Scope_Result_1;
else
   --# expr-start 11455 '<FieldAccess for spark_mode_aspect at nodes.lkt:15835:29>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15835'
--# expr-start 11454 '<FieldAccess for semantic_parent at nodes.lkt:15835:29>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15835'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_9 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11454
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.spark_mode_aspect'
Fld_10 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Spark_Mode_Aspect (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 11455
   Result_Var := Fld_10;
end if;
   Result_Var_1 := Result_Var;
end if;
         Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_1353;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1353;
            raise;
      end;
   return Property_Result;
end Handled_Stmts_P_Spark_Mode_Aspect;
--# end
   --
   --  Primitives for Bare_Interface_Kind
   --
   --
   --  Primitives for Bare_Interface_Kind_Limited
   --
   --
   --  Primitives for Bare_Interface_Kind_Protected
   --
   --
   --  Primitives for Bare_Interface_Kind_Synchronized
   --
   --
   --  Primitives for Bare_Interface_Kind_Task
   --
   --
   --  Primitives for Bare_Iter_Type
   --
   --
   --  Primitives for Bare_Iter_Type_In
   --
   --
   --  Primitives for Bare_Iter_Type_Of
   --
   --
   --  Primitives for Bare_Library_Item
   --
      procedure Initialize_Fields_For_Library_Item
        (Self : Bare_Library_Item
         ; Library_Item_F_Has_Private : Bare_Private_Node
         ; Library_Item_F_Item : Bare_Basic_Decl
        ) is
      begin
            Self.Library_Item_F_Has_Private := Library_Item_F_Has_Private;
            Self.Library_Item_F_Item := Library_Item_F_Item;
      end Initialize_Fields_For_Library_Item;
   function Library_Item_F_Has_Private
     (Node : Bare_Library_Item) return Bare_Private_Node
   is
   begin
         return Node.Library_Item_F_Has_Private;
   end;
   function Library_Item_F_Item
     (Node : Bare_Library_Item) return Bare_Basic_Decl
   is
   begin
         return Node.Library_Item_F_Item;
   end;
   --
   --  Primitives for Bare_Limited_Node
   --
--# property-start '[dispatcher]Limited.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Limited_Node_P_As_Bool
  (Node : Bare_Limited_Node
  )
   return Boolean
is
   Self : Bare_Limited_Node  := Bare_Limited_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Limited_Node (Self.Kind) is
               when Ada_Limited_Absent =>
                  --# property-call-start Limited.Absent.as_bool
                  Property_Result := Limited_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Limited_Present =>
                  --# property-call-start Limited.Present.as_bool
                  Property_Result := Limited_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Limited_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Absent
   --
--# property-start Limited.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15858'
pragma Warnings (Off, "is not referenced");
 function Limited_Absent_P_As_Bool
  (Node : Bare_Limited_Absent
  )
   return Boolean
is
   Self : Bare_Limited_Absent  := Bare_Limited_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Limited_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Present
   --
--# property-start Limited.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15858'
pragma Warnings (Off, "is not referenced");
 function Limited_Present_P_As_Bool
  (Node : Bare_Limited_Present
  )
   return Boolean
is
   Self : Bare_Limited_Present  := Bare_Limited_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Limited_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Loop_Spec
   --
   --
   --  Primitives for Bare_For_Loop_Spec
   --
      procedure Initialize_Fields_For_For_Loop_Spec
        (Self : Bare_For_Loop_Spec
         ; For_Loop_Spec_F_Var_Decl : Bare_For_Loop_Var_Decl
         ; For_Loop_Spec_F_Loop_Type : Bare_Iter_Type
         ; For_Loop_Spec_F_Has_Reverse : Bare_Reverse_Node
         ; For_Loop_Spec_F_Iter_Expr : Bare_Ada_Node
         ; For_Loop_Spec_F_Iter_Filter : Bare_For_Loop_Iter_Filter
        ) is
      begin
            Self.For_Loop_Spec_F_Var_Decl := For_Loop_Spec_F_Var_Decl;
            Self.For_Loop_Spec_F_Loop_Type := For_Loop_Spec_F_Loop_Type;
            Self.For_Loop_Spec_F_Has_Reverse := For_Loop_Spec_F_Has_Reverse;
            Self.For_Loop_Spec_F_Iter_Expr := For_Loop_Spec_F_Iter_Expr;
            Self.For_Loop_Spec_F_Iter_Filter := For_Loop_Spec_F_Iter_Filter;
      end Initialize_Fields_For_For_Loop_Spec;
   function For_Loop_Spec_F_Var_Decl
     (Node : Bare_For_Loop_Spec) return Bare_For_Loop_Var_Decl
   is
   begin
         return Node.For_Loop_Spec_F_Var_Decl;
   end;
   function For_Loop_Spec_F_Loop_Type
     (Node : Bare_For_Loop_Spec) return Bare_Iter_Type
   is
   begin
         return Node.For_Loop_Spec_F_Loop_Type;
   end;
   function For_Loop_Spec_F_Has_Reverse
     (Node : Bare_For_Loop_Spec) return Bare_Reverse_Node
   is
   begin
         return Node.For_Loop_Spec_F_Has_Reverse;
   end;
   function For_Loop_Spec_F_Iter_Expr
     (Node : Bare_For_Loop_Spec) return Bare_Ada_Node
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Expr;
   end;
   function For_Loop_Spec_F_Iter_Filter
     (Node : Bare_For_Loop_Spec) return Bare_For_Loop_Iter_Filter
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Filter;
   end;
--# property-start ForLoopSpec.is_iterated_assoc_spec 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15876'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Is_Iterated_Assoc_Spec
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11457 '<IsA IteratedAssoc at nodes.lkt:15876:42>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15876'
--# expr-start 11456 '<parent at nodes.lkt:15876:42>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15876'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11456
Is_A := Fld /= null 
and then Fld.Kind in Ada_Iterated_Assoc_Range; 
--# expr-done 11457
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Is_Iterated_Assoc_Spec;
--# end
--# property-start ForLoopSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15879'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
     Ent : Internal_Entity_For_Loop_Spec :=
       Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1358 with Inline_Always;
            procedure Finalizer_Scope_3479 with Inline_Always;
            procedure Finalizer_Scope_3480 with Inline_Always;
            procedure Finalizer_Scope_3481 with Inline_Always;
            procedure Finalizer_Scope_3482 with Inline_Always;
            procedure Finalizer_Scope_3483 with Inline_Always;
            procedure Finalizer_Scope_3484 with Inline_Always;
            procedure Finalizer_Scope_3485 with Inline_Always;
            procedure Finalizer_Scope_3486 with Inline_Always;
            procedure Finalizer_Scope_3487 with Inline_Always;
            procedure Finalizer_Scope_3488 with Inline_Always;
      Match_0 : Bare_Iter_Type_In;
Match_0_1 : Internal_Entity_Bin_Op;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Match_4 : Internal_Entity;
Match_1_1 : Bare_Iter_Type_Of;
Var_Expr_1 : Internal_Entity_Expr;
Var_Expr_2 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_For_Loop_Var_Decl;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Iter_Type;
Fld_3 : Internal_Entity;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_For_Loop_Var_Decl;
Fld_7 : Bare_Defining_Name;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_10 : Logic_Equation;
Fld_11 : Bare_For_Loop_Var_Decl;
Fld_12 : Bare_Defining_Name;
Fld_13 : Logic_Var;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Equation;
Fld_17 : Bare_For_Loop_Var_Decl;
Fld_18 : Bare_Defining_Name;
Fld_19 : Logic_Var;
Fld_20 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Logic_Equation;
Fld_22 : Internal_Entity_Base_Type_Decl;
Fld_23 : Bare_For_Loop_Var_Decl;
Fld_24 : Bare_Defining_Name;
Fld_25 : Logic_Var;
Fld_26 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_27 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_28 : Logic_Var;
Pred_2 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_29 : Bare_For_Loop_Var_Decl;
Fld_30 : Bare_Defining_Name;
Fld_31 : Logic_Var;
Fld_32 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_6 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_6 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_33 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Expr;
Fld_34 : Internal_Entity_For_Loop_Var_Decl;
Fld_35 : Internal_Entity_Type_Expr;
Fld_36 : Logic_Var;
Fld_37 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_38 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_39 : Logic_Var;
Pred_3 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_40 : Bare_For_Loop_Var_Decl;
Fld_41 : Bare_Defining_Name;
Fld_42 : Logic_Var;
Fld_43 : Logic_Var;
Bind_Result_6 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_2 : Logic_Equation;
Match_Prefix_1 : Bare_Iter_Type;
Cast_Result_9 : Bare_Iter_Type_In;
Let_Result_5 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_10 : Bare_Iter_Type_Of;
Let_Result_6 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Match_Result_1 : Logic_Equation;
And_Pred_10 : Logic_Equation;
            procedure Finalizer_Scope_1358 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (And_Pred_10);
            end Finalizer_Scope_1358;
            procedure Finalizer_Scope_3479 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3479;
            procedure Finalizer_Scope_3480 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3480;
            procedure Finalizer_Scope_3481 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3481;
            procedure Finalizer_Scope_3482 is
            begin
                     Dec_Ref (Fld_16);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3482;
            procedure Finalizer_Scope_3483 is
            begin
                     Dec_Ref (Fld_21);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Pred_2);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_6);
            end Finalizer_Scope_3483;
            procedure Finalizer_Scope_3484 is
            begin
                     Dec_Ref (Bind_Result_3);
            end Finalizer_Scope_3484;
            procedure Finalizer_Scope_3485 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_3485;
            procedure Finalizer_Scope_3486 is
            begin
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_3486;
            procedure Finalizer_Scope_3487 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Pred_3);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Bind_Result_6);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_3487;
            procedure Finalizer_Scope_3488 is
            begin
                     Dec_Ref (Bind_Result_5);
            end Finalizer_Scope_3488;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11526 '<And at nodes.lkt:15880:9>' And_Pred_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15880'
--# expr-start 11459 '<FieldAccess for sub_equation at nodes.lkt:15880:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15880'
--# expr-start 11458 '<FieldAccess for var_decl at nodes.lkt:15880:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15880'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 11458
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11459
--# expr-start 11525 '<Match at nodes.lkt:15880:42>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15880'
--# expr-start 11460 '<FieldAccess for loop_type at nodes.lkt:15880:48>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15880'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.For_Loop_Spec_F_Loop_Type;
--# expr-done 11460
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Fld_2; 
case Ada_Iter_Type (Match_Prefix_1.Kind) is
         when Ada_Iter_Type_In =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix_1;
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 11506 '<Match at nodes.lkt:15884:13>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15884'
--# expr-start 11461 '<FieldAccess for iter_expr at nodes.lkt:15884:19>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15884'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 11461
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_3; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_2; 
--# bind binop Match_0_1
--# expr-start 11471 '<And at nodes.lkt:15886:38>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15886'
--# expr-start 11465 '<And at nodes.lkt:15887:21>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15887'
--# expr-start 11462 '<FieldAccess for sub_equation at nodes.lkt:15887:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15887'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0_1.Info);
--# end
--# expr-done 11462
--# expr-start 11464 '<Predicate on BaseTypeDecl.is_not_root_int_type at nodes.lkt:15890:25>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15890'
--# expr-start 11463 '<FieldAccess for type_var at nodes.lkt:15890:71>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15890'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 11463
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Not_Root_Int_Type_0_Predicate); 
--# expr-done 11464
And_Pred := Create_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15887:21") else null)); 
--# expr-done 11465
--# expr-start 11470 '<Bind at nodes.lkt:15892:23>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15892'
--# expr-start 11468 '<FieldAccess for type_var at nodes.lkt:15892:27>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15892'
--# expr-start 11467 '<FieldAccess for id at nodes.lkt:15892:27>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15892'
--# expr-start 11466 '<FieldAccess for var_decl at nodes.lkt:15892:27>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15892'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11466
      if Fld_6 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Fld_6.For_Loop_Var_Decl_F_Id;
--# expr-done 11467
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 11468
Fld_8.Value := No_Entity;
Entity_Vars.Reset (Fld_8);
--# expr-start 11469 '<FieldAccess for type_var at nodes.lkt:15892:56>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15892'
      if Match_0_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 11469
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
Bind_Result := Solver.Create_Unify (Fld_8, Fld_9, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15892:23") else null)); 
--# expr-done 11470
And_Pred_1 := Create_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15886:38") else null)); 
--# expr-done 11471
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3480;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 11479 '<And at nodes.lkt:15896:46>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
--# expr-start 11472 '<FieldAccess for sub_equation at nodes.lkt:15896:46>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 11472
--# expr-start 11478 '<Bind at nodes.lkt:15896:67>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
--# expr-start 11475 '<FieldAccess for type_var at nodes.lkt:15896:71>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
--# expr-start 11474 '<FieldAccess for id at nodes.lkt:15896:71>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
--# expr-start 11473 '<FieldAccess for var_decl at nodes.lkt:15896:71>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11473
      if Fld_11 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Fld_11.For_Loop_Var_Decl_F_Id;
--# expr-done 11474
      if Fld_12 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_12);
--# expr-done 11475
Fld_13.Value := No_Entity;
Entity_Vars.Reset (Fld_13);
--# expr-start 11477 '<FieldAccess for canonical_type at nodes.lkt:15896:100>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
--# expr-start 11476 '<FieldAccess for designated_type at nodes.lkt:15896:100>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15896'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 11476
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 11477
      Cast_Result := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_13, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15896:67") else null)); 
--# expr-done 11478
And_Pred_2 := Create_And (Fld_10, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15896:46") else null)); 
--# expr-done 11479
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3481;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind r Match_2
--# expr-start 11486 '<And at nodes.lkt:15897:41>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
--# expr-start 11480 '<FieldAccess for sub_equation at nodes.lkt:15897:41>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_16 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_2.Info);
--# end
--# expr-done 11480
--# expr-start 11485 '<Bind at nodes.lkt:15897:62>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
--# expr-start 11483 '<FieldAccess for type_var at nodes.lkt:15897:66>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
--# expr-start 11482 '<FieldAccess for id at nodes.lkt:15897:66>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
--# expr-start 11481 '<FieldAccess for var_decl at nodes.lkt:15897:66>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_17 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11481
      if Fld_17 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Fld_17.For_Loop_Var_Decl_F_Id;
--# expr-done 11482
      if Fld_18 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 11483
Fld_19.Value := No_Entity;
Entity_Vars.Reset (Fld_19);
--# expr-start 11484 '<FieldAccess for type_var at nodes.lkt:15897:95>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15897'
      if Match_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_2.Node);
--# expr-done 11484
Fld_20.Value := No_Entity;
Entity_Vars.Reset (Fld_20);
Bind_Result_2 := Solver.Create_Unify (Fld_19, Fld_20, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15897:62") else null)); 
--# expr-done 11485
And_Pred_3 := Create_And (Fld_16, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15897:41") else null)); 
--# expr-done 11486
Let_Result_2 := And_Pred_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3482;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name | Ada_Call_Expr .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind t Match_3
--# expr-start 11505 '<And at nodes.lkt:15903:33>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15903'
--# expr-start 11487 '<FieldAccess for sub_equation at nodes.lkt:15903:33>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15903'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_3.Info);
--# end
--# expr-done 11487
--# expr-start 11488 '<FieldAccess for name_designated_type at nodes.lkt:15903:54>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15903'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_22 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 11488
Var_Expr_0 := Fld_22;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_0
--# expr-start 11493 '<Bind at nodes.lkt:15904:30>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
--# expr-start 11491 '<FieldAccess for type_var at nodes.lkt:15904:34>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
--# expr-start 11490 '<FieldAccess for id at nodes.lkt:15904:34>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
--# expr-start 11489 '<FieldAccess for var_decl at nodes.lkt:15904:34>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11489
      if Fld_23 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Fld_23.For_Loop_Var_Decl_F_Id;
--# expr-done 11490
      if Fld_24 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24);
--# expr-done 11491
Fld_25.Value := No_Entity;
Entity_Vars.Reset (Fld_25);
--# expr-start 11492 '<FieldAccess for canonical_type at nodes.lkt:15904:63>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_26 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11492
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_25, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15904:30") else null)); 
--# expr-done 11493
Scope_Result := Bind_Result_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3484;
   Result_Var := Scope_Result;
else
   --# expr-start 11504 '<And at nodes.lkt:15904:98>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
--# expr-start 11498 '<And at nodes.lkt:15904:98>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15904'
--# expr-start 11495 '<Predicate on AdaNode.is_not_null at nodes.lkt:15907:25>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15907'
--# expr-start 11494 '<FieldAccess for type_var at nodes.lkt:15907:57>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15907'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_3.Node);
--# expr-done 11494
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
Pred_1 := Solver.Create_Predicate (Fld_27, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 11495
--# expr-start 11497 '<Predicate on BaseTypeDecl.is_iterator_type at nodes.lkt:15908:27>' Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
--# expr-start 11496 '<FieldAccess for type_var at nodes.lkt:15908:69>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_3.Node);
--# expr-done 11496
Fld_28.Value := No_Entity;
Entity_Vars.Reset (Fld_28);
Pred_2 := Solver.Create_Predicate (Fld_28, Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (Origin)); 
--# expr-done 11497
And_Pred_4 := Create_And (Pred_1, Pred_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15904:98") else null)); 
--# expr-done 11498
--# expr-start 11503 '<Bind at nodes.lkt:15908:87>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
--# expr-start 11499 '<FieldAccess for type_var at nodes.lkt:15908:120>' Fld_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
      if Match_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_32 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_3.Node);
--# expr-done 11499
Fld_32.Value := No_Entity;
Entity_Vars.Reset (Fld_32);
--# expr-start 11502 '<FieldAccess for type_var at nodes.lkt:15908:91>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
--# expr-start 11501 '<FieldAccess for id at nodes.lkt:15908:91>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
--# expr-start 11500 '<FieldAccess for var_decl at nodes.lkt:15908:91>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15908'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11500
      if Fld_29 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Fld_29.For_Loop_Var_Decl_F_Id;
--# expr-done 11501
      if Fld_30 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_30);
--# expr-done 11502
Fld_31.Value := No_Entity;
Entity_Vars.Reset (Fld_31);
Bind_Result_4 := Solver.Create_Propagate (Fld_32, Fld_31, Logic_Functor_878'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15908:87") else null)); 
--# expr-done 11503
And_Pred_5 := Create_And (And_Pred_4, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15904:98") else null)); 
--# expr-done 11504
   Result_Var := And_Pred_5;
end if;
      Inc_Ref (Result_Var);
And_Pred_6 := Create_And (Fld_21, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15903:33") else null)); 
--# expr-done 11505
Let_Result_3 := And_Pred_6; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_3483;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_6; 
--# bind _ Match_4
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15910:27") else null)); 
Let_Result_4 := True_Rel; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3485;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 11506
Let_Result_5 := Match_Result; Inc_Ref (Let_Result_5);
--# end
Scope_Result_8 := Let_Result_5;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_3479;
            Match_Result_1 := Scope_Result_8; Inc_Ref (Match_Result_1);
         when Ada_Iter_Type_Of =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix_1;
Match_1_1 := Cast_Result_10; 
--# bind _ Match_1_1
--# expr-start 11508 '<Cast to Expr at nodes.lkt:15915:36>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15915'
--# expr-start 11507 '<FieldAccess for iter_expr at nodes.lkt:15915:36>' Fld_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15915'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_33 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 11507
   if Fld_33.Node = null
      or else Fld_33.Node.Kind in Ada_Expr
   then
      Cast_Result_7 := Create_Internal_Entity_Expr
        (Node => Fld_33.Node,
         Info => Fld_33.Info);
   else
         Cast_Result_7 := No_Entity_Expr;
   end if;
--# expr-done 11508
Var_Expr_1 := Cast_Result_7;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind iter_expr Var_Expr_1
--# expr-start 11524 '<And at nodes.lkt:15916:32>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15916'
--# expr-start 11518 '<And at nodes.lkt:15917:21>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15917'
--# expr-start 11515 '<And at nodes.lkt:15918:25>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15918'
--# expr-start 11510 '<FieldAccess for id_type at nodes.lkt:15918:25>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15918'
--# expr-start 11509 '<FieldAccess for var_decl at nodes.lkt:15918:25>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15918'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_34 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 11509
      if Fld_34.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Create_Internal_Entity_Type_Expr (Node => Fld_34.Node.For_Loop_Var_Decl_F_Id_Type, Info => Fld_34.Info);
--# expr-done 11510
Var_Expr_2 := Fld_35;
if Var_Expr_2 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind typ Var_Expr_2
--# expr-start 11513 '<Bind at nodes.lkt:15919:38>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15919'
--# expr-start 11511 '<FieldAccess for expected_type_var at nodes.lkt:15919:42>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15919'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 11511
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
--# expr-start 11512 '<FieldAccess for designated_type at nodes.lkt:15919:73>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15919'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_37 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 11512
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_37.Node,
         Info => Fld_37.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_36, Cast_Result_8, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15919:38") else null)); 
--# expr-done 11513
Scope_Result_6 := Bind_Result_5;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3488;
   Result_Var_1 := Scope_Result_6;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15919:109") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
--# expr-start 11514 '<FieldAccess for sub_equation at nodes.lkt:15920:31>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15920'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_38 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 11514
And_Pred_7 := Create_And (Result_Var_1, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15918:25") else null)); 
--# expr-done 11515
--# expr-start 11517 '<Predicate on AdaNode.is_not_null at nodes.lkt:15921:27>' Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15921'
--# expr-start 11516 '<FieldAccess for type_var at nodes.lkt:15921:59>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15921'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 11516
Fld_39.Value := No_Entity;
Entity_Vars.Reset (Fld_39);
Pred_3 := Solver.Create_Predicate (Fld_39, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 11517
And_Pred_8 := Create_And (And_Pred_7, Pred_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15917:21") else null)); 
--# expr-done 11518
--# expr-start 11523 '<Bind at nodes.lkt:15922:23>' Bind_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15922'
--# expr-start 11519 '<FieldAccess for type_var at nodes.lkt:15922:56>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15922'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 11519
Fld_43.Value := No_Entity;
Entity_Vars.Reset (Fld_43);
--# expr-start 11522 '<FieldAccess for type_var at nodes.lkt:15922:27>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15922'
--# expr-start 11521 '<FieldAccess for id at nodes.lkt:15922:27>' Fld_41 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15922'
--# expr-start 11520 '<FieldAccess for var_decl at nodes.lkt:15922:27>' Fld_40 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15922'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_40 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 11520
      if Fld_40 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_41 := Fld_40.For_Loop_Var_Decl_F_Id;
--# expr-done 11521
      if Fld_41 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_41);
--# expr-done 11522
Fld_42.Value := No_Entity;
Entity_Vars.Reset (Fld_42);
Bind_Result_6 := Solver.Create_Propagate (Fld_43, Fld_42, Logic_Functor_900'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15922:23") else null)); 
--# expr-done 11523
And_Pred_9 := Create_And (And_Pred_8, Bind_Result_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15916:32") else null)); 
--# expr-done 11524
Scope_Result_7 := And_Pred_9;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3487;
   Result_Var_2 := Scope_Result_7;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15922:142") else null)); 
   Result_Var_2 := False_Rel;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_6 := Result_Var_2; Inc_Ref (Let_Result_6);
--# end
Scope_Result_9 := Let_Result_6;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_3486;
            Match_Result_1 := Scope_Result_9; Inc_Ref (Match_Result_1);
end case;
--# expr-done 11525
And_Pred_10 := Create_And (Fld_1, Match_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15880:9") else null)); 
--# expr-done 11526
         Property_Result := And_Pred_10;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1358;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1358;
                     Finalizer_Scope_3479;
                     Finalizer_Scope_3480;
                     Finalizer_Scope_3481;
                     Finalizer_Scope_3482;
                     Finalizer_Scope_3483;
                     Finalizer_Scope_3484;
                     Finalizer_Scope_3485;
                     Finalizer_Scope_3486;
                     Finalizer_Scope_3487;
                     Finalizer_Scope_3488;
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Xref_Equation;
--# end
--# property-start ForLoopSpec.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15929'
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Entry_Point
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec  := Bare_For_Loop_Spec (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11528 '<Not at nodes.lkt:15929:36>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15929'
--# expr-start 11527 '<FieldAccess for is_iterated_assoc_spec at nodes.lkt:15929:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15929'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ForLoopSpec.is_iterated_assoc_spec
Fld := Libadalang.Implementation.For_Loop_Spec_P_Is_Iterated_Assoc_Spec (Node => Self);
--# end
--# expr-done 11527
Not_Val := not (Fld); 
--# expr-done 11528
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end For_Loop_Spec_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_While_Loop_Spec
   --
      procedure Initialize_Fields_For_While_Loop_Spec
        (Self : Bare_While_Loop_Spec
         ; While_Loop_Spec_F_Expr : Bare_Expr
        ) is
      begin
            Self.While_Loop_Spec_F_Expr := While_Loop_Spec_F_Expr;
      end Initialize_Fields_For_While_Loop_Spec;
   function While_Loop_Spec_F_Expr
     (Node : Bare_While_Loop_Spec) return Bare_Expr
   is
   begin
         return Node.While_Loop_Spec_F_Expr;
   end;
--# property-start WhileLoopSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15937'
pragma Warnings (Off, "is not referenced");
 function While_Loop_Spec_P_Xref_Equation
  (Node : Bare_While_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_While_Loop_Spec  := Bare_While_Loop_Spec (Node);
     Ent : Internal_Entity_While_Loop_Spec :=
       Internal_Entity_While_Loop_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1360 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1360 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1360;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11533 '<And at nodes.lkt:15938:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15938'
--# expr-start 11530 '<FieldAccess for sub_equation at nodes.lkt:15938:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15938'
--# expr-start 11529 '<FieldAccess for expr at nodes.lkt:15938:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15938'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 11529
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11530
--# expr-start 11532 '<FieldAccess for expect_bool_derived_type at nodes.lkt:15938:38>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15938'
--# expr-start 11531 '<FieldAccess for expr at nodes.lkt:15938:38>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15938'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 11531
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_3 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_2.Node, Origin => Origin);
--# end
--# expr-done 11532
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15938:9") else null)); 
--# expr-done 11533
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1360;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1360;
            raise;
      end;
   return Property_Result;
end While_Loop_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mode
   --
--# property-start Mode.is_writable 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15947'
pragma Warnings (Off, "is not referenced");
 function Mode_P_Is_Writable
  (Node : Bare_Mode
  )
   return Boolean
is
   Self : Bare_Mode  := Bare_Mode (Node);
      --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11534 '<IsA Mode.Out, Mode.InOut at nodes.lkt:15947:31>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15947'
Is_A := Self /= null 
and then Self.Kind in Ada_Mode_Out_Range | Ada_Mode_In_Out_Range; 
--# expr-done 11534
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mode_P_Is_Writable;
--# end
   --
   --  Primitives for Bare_Mode_Default
   --
   --
   --  Primitives for Bare_Mode_In
   --
   --
   --  Primitives for Bare_Mode_In_Out
   --
   --
   --  Primitives for Bare_Mode_Out
   --
   --
   --  Primitives for Bare_Multi_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Multi_Abstract_State_Decl
        (Self : Bare_Multi_Abstract_State_Decl
         ; Multi_Abstract_State_Decl_F_Decls : Bare_Abstract_State_Decl_List
        ) is
      begin
            Self.Multi_Abstract_State_Decl_F_Decls := Multi_Abstract_State_Decl_F_Decls;
      end Initialize_Fields_For_Multi_Abstract_State_Decl;
   function Multi_Abstract_State_Decl_F_Decls
     (Node : Bare_Multi_Abstract_State_Decl) return Bare_Abstract_State_Decl_List
   is
   begin
         return Node.Multi_Abstract_State_Decl_F_Decls;
   end;
   --
   --  Primitives for Bare_Not_Null
   --
--# property-start '[dispatcher]NotNull.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Not_Null_P_As_Bool
  (Node : Bare_Not_Null
  )
   return Boolean
is
   Self : Bare_Not_Null  := Bare_Not_Null (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Not_Null (Self.Kind) is
               when Ada_Not_Null_Absent =>
                  --# property-call-start NotNull.Absent.as_bool
                  Property_Result := Not_Null_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Not_Null_Present =>
                  --# property-call-start NotNull.Present.as_bool
                  Property_Result := Not_Null_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Not_Null_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Absent
   --
--# property-start NotNull.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15959'
pragma Warnings (Off, "is not referenced");
 function Not_Null_Absent_P_As_Bool
  (Node : Bare_Not_Null_Absent
  )
   return Boolean
is
   Self : Bare_Not_Null_Absent  := Bare_Not_Null_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Not_Null_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Present
   --
--# property-start NotNull.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15959'
pragma Warnings (Off, "is not referenced");
 function Not_Null_Present_P_As_Bool
  (Node : Bare_Not_Null_Present
  )
   return Boolean
is
   Self : Bare_Not_Null_Present  := Bare_Not_Null_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Not_Null_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Null_Component_Decl
   --
   --
   --  Primitives for Bare_Others_Designator
   --
--# property-start OthersDesignator.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:15969'
pragma Warnings (Off, "is not referenced");
 function Others_Designator_P_Xref_Equation
  (Node : Bare_Others_Designator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Others_Designator  := Bare_Others_Designator (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1365 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1365 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1365;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:15969:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1365;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1365;
            raise;
      end;
   return Property_Result;
end Others_Designator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Overriding_Node
   --
   --
   --  Primitives for Bare_Overriding_Not_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Unspecified
   --
   --
   --  Primitives for Bare_Params
   --
      procedure Initialize_Fields_For_Params
        (Self : Bare_Params
         ; Params_F_Params : Bare_Param_Spec_List
        ) is
      begin
            Self.Params_F_Params := Params_F_Params;
      end Initialize_Fields_For_Params;
   function Params_F_Params
     (Node : Bare_Params) return Bare_Param_Spec_List
   is
   begin
         return Node.Params_F_Params;
   end;
   --
   --  Primitives for Bare_Paren_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Paren_Abstract_State_Decl
        (Self : Bare_Paren_Abstract_State_Decl
         ; Paren_Abstract_State_Decl_F_Decl : Bare_Ada_Node
        ) is
      begin
            Self.Paren_Abstract_State_Decl_F_Decl := Paren_Abstract_State_Decl_F_Decl;
      end Initialize_Fields_For_Paren_Abstract_State_Decl;
   function Paren_Abstract_State_Decl_F_Decl
     (Node : Bare_Paren_Abstract_State_Decl) return Bare_Ada_Node
   is
   begin
         return Node.Paren_Abstract_State_Decl_F_Decl;
   end;
   --
   --  Primitives for Bare_Pp_Directive
   --
   --
   --  Primitives for Bare_Pp_Else_Directive
   --
   --
   --  Primitives for Bare_Pp_Elsif_Directive
   --
      procedure Initialize_Fields_For_Pp_Elsif_Directive
        (Self : Bare_Pp_Elsif_Directive
         ; Pp_Elsif_Directive_F_Expr : Bare_Expr
         ; Pp_Elsif_Directive_F_Then_Kw : Bare_Pp_Then_Kw
        ) is
      begin
            Self.Pp_Elsif_Directive_F_Expr := Pp_Elsif_Directive_F_Expr;
            Self.Pp_Elsif_Directive_F_Then_Kw := Pp_Elsif_Directive_F_Then_Kw;
      end Initialize_Fields_For_Pp_Elsif_Directive;
   function Pp_Elsif_Directive_F_Expr
     (Node : Bare_Pp_Elsif_Directive) return Bare_Expr
   is
   begin
         return Node.Pp_Elsif_Directive_F_Expr;
   end;
   function Pp_Elsif_Directive_F_Then_Kw
     (Node : Bare_Pp_Elsif_Directive) return Bare_Pp_Then_Kw
   is
   begin
         return Node.Pp_Elsif_Directive_F_Then_Kw;
   end;
   --
   --  Primitives for Bare_Pp_End_If_Directive
   --
   --
   --  Primitives for Bare_Pp_If_Directive
   --
      procedure Initialize_Fields_For_Pp_If_Directive
        (Self : Bare_Pp_If_Directive
         ; Pp_If_Directive_F_Expr : Bare_Expr
         ; Pp_If_Directive_F_Then_Kw : Bare_Pp_Then_Kw
        ) is
      begin
            Self.Pp_If_Directive_F_Expr := Pp_If_Directive_F_Expr;
            Self.Pp_If_Directive_F_Then_Kw := Pp_If_Directive_F_Then_Kw;
      end Initialize_Fields_For_Pp_If_Directive;
   function Pp_If_Directive_F_Expr
     (Node : Bare_Pp_If_Directive) return Bare_Expr
   is
   begin
         return Node.Pp_If_Directive_F_Expr;
   end;
   function Pp_If_Directive_F_Then_Kw
     (Node : Bare_Pp_If_Directive) return Bare_Pp_Then_Kw
   is
   begin
         return Node.Pp_If_Directive_F_Then_Kw;
   end;
   --
   --  Primitives for Bare_Pp_Then_Kw
   --
   --
   --  Primitives for Bare_Pragma_Node
   --
      procedure Pragma_Pre_Env_Actions
        (Self            : Bare_Pragma_Node;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_217 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:16275:9");
      end;
      end;
      procedure Initialize_Fields_For_Pragma_Node
        (Self : Bare_Pragma_Node
         ; Pragma_Node_F_Id : Bare_Identifier
         ; Pragma_Node_F_Args : Bare_Base_Assoc_List
        ) is
      begin
            Self.Pragma_Node_F_Id := Pragma_Node_F_Id;
            Self.Pragma_Node_F_Args := Pragma_Node_F_Args;
      end Initialize_Fields_For_Pragma_Node;
   function Pragma_Node_F_Id
     (Node : Bare_Pragma_Node) return Bare_Identifier
   is
   begin
         return Node.Pragma_Node_F_Id;
   end;
   function Pragma_Node_F_Args
     (Node : Bare_Pragma_Node) return Bare_Base_Assoc_List
   is
   begin
         return Node.Pragma_Node_F_Args;
   end;
--# property-start Pragma.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16032'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Entry_Point
  (Node : Bare_Pragma_Node
  )
   return Boolean
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Xref_Entry_Point;
--# end
--# property-start Pragma.is_ghost_code 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16036'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Is_Ghost_Code
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11536 '<FieldAccess for sym at nodes.lkt:16039:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
--# expr-start 11535 '<FieldAccess for id at nodes.lkt:16039:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11535
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld.Node);
--# end
--# expr-done 11536
--# expr-start 11537 '<Symbol Assert at nodes.lkt:16039:26>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert); 
--# expr-done 11537
Is_Equal_3 := Fld_1 = Sym; 
if Is_Equal_3 then
   If_Result_2 := True;
else
--# expr-start 11538 '<Symbol Assert_And_Cut at nodes.lkt:16039:38>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert_And_Cut); 
--# expr-done 11538
Is_Equal_2 := Fld_1 = Sym_1; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 11539 '<Symbol Assume at nodes.lkt:16039:58>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assume); 
--# expr-done 11539
Is_Equal_1 := Fld_1 = Sym_2; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 11540 '<Symbol Loop_Invariant at nodes.lkt:16039:70>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16039'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Invariant); 
--# expr-done 11540
Is_Equal := Fld_1 = Sym_3; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Is_Ghost_Code;
--# end
--# property-start Pragma.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16042'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Equation
  (Node : Bare_Pragma_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1368 with Inline_Always;
            procedure Finalizer_Scope_3489 with Inline_Always;
            procedure Finalizer_Scope_3490 with Inline_Always;
            procedure Finalizer_Scope_3491 with Inline_Always;
            procedure Finalizer_Scope_3492 with Inline_Always;
            procedure Finalizer_Scope_3493 with Inline_Always;
            procedure Finalizer_Scope_3494 with Inline_Always;
            procedure Finalizer_Scope_3495 with Inline_Always;
            procedure Finalizer_Scope_3499 with Inline_Always;
            procedure Finalizer_Scope_3500 with Inline_Always;
            procedure Finalizer_Scope_3502 with Inline_Always;
            procedure Finalizer_Scope_3505 with Inline_Always;
      Local_Expr : Internal_Entity_Expr;
Local_Proc : Internal_Entity_Expr;
Var_Expr_5 : Internal_Entity_Pragma_Argument_Assoc;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_9 : Internal_Entity_Expr;
Local_Expr_1 : Internal_Entity_Expr;
Var_Expr_12 : Internal_Entity_Base_Assoc;
Var_Expr_13 : Internal_Entity_Expr;
Var_Expr_272 : Internal_Entity_Base_Assoc;
Var_Expr_273 : Internal_Entity_Pragma_Argument_Assoc;
Var_Expr_274 : Internal_Entity_Name;
Fld : Internal_Entity_Base_Assoc_List;
Fld_1 : Bare_Base_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Fld_6 : Internal_Entity_Base_Assoc_List;
Fld_7 : Bare_Base_Assoc_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Var;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_14 : Internal_Entity_Base_Assoc_List;
Fld_15 : Bare_Base_Assoc_List;
Fld_16 : Internal_Entity_Info;
Item_11 : Internal_Entity_Base_Assoc;
Bare_Item_11 : Bare_Base_Assoc;
Fld_17 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_19 : Internal_Entity_Name_Array_Access;
Fld_20 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_21 : Internal_Entity_Base_Assoc_List;
Fld_22 : Bare_Base_Assoc_List;
Fld_23 : Internal_Entity_Info;
Item_8 : Internal_Entity_Base_Assoc;
Bare_Item_8 : Bare_Base_Assoc;
Fld_24 : Internal_Entity_Expr;
True_Rel_2 : Logic_Equation;
Cast_Expr : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Identifier;
Fld_25 : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Identifier;
Fld_26 : Symbol_Type;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_3 : Boolean;
Is_Equal_2 : Boolean;
If_Result_4 : Boolean;
Is_Equal_3 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Logic_Equation;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_29 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_30 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Is_A : Boolean;
If_Result_7 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
True_Rel_3 : Logic_Equation;
Result_Var_2 : Logic_Equation;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_8 : Internal_Entity_Base_Assoc;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
True_Rel_4 : Logic_Equation;
Fld_31 : Internal_Entity_Base_Assoc_List;
Fld_32 : Bare_Base_Assoc_List;
Fld_33 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_9 : Internal_Entity_Base_Assoc;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Var;
Fld_36 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Fld_37 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_38 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
False_Rel : Logic_Equation;
Result_Var_3 : Logic_Equation;
Fld_39 : Internal_Entity_Base_Assoc_List;
Fld_40 : Bare_Base_Assoc_List;
Fld_41 : Internal_Entity_Info;
Item_4 : Internal_Entity_Base_Assoc;
Bare_Item_4 : Bare_Base_Assoc;
As_Entity_5 : Internal_Entity_Base_Assoc;
Is_Null_5 : Boolean;
If_Result_10 : Internal_Entity_Base_Assoc;
Cast_Expr_2 : Internal_Entity_Base_Assoc;
Cast_Result_6 : Internal_Entity_Pragma_Argument_Assoc;
Fld_42 : Internal_Entity_Name;
Fld_43 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
Is_Equal_5 : Boolean;
If_Result_11 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Map_Result_3 : Internal_Entity_Base_Assoc_Array_Access;
Fld_44 : Internal_Entity_Expr;
Fld_45 : Logic_Var;
Fld_46 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_3 : Logic_Equation;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_49 : Internal_Entity_Expr;
Fld_50 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Map_Result_4 : Logic_Equation_Array_Access;
Logic_Boolean_Op_3 : Logic_Equation;
Fld_51 : Internal_Entity_Base_Assoc_List;
Fld_52 : Bare_Base_Assoc_List;
Fld_53 : Internal_Entity_Info;
Get_Result_3 : Bare_Base_Assoc;
As_Entity_6 : Internal_Entity_Base_Assoc;
Is_Null_6 : Boolean;
If_Result_12 : Internal_Entity_Base_Assoc;
Fld_54 : Internal_Entity_Expr;
Cast_Result_8 : Internal_Entity_Base_Aggregate;
Fld_55 : Internal_Entity_Assoc_List;
Fld_56 : Bare_Assoc_List;
Fld_57 : Internal_Entity_Info;
Item_3 : Internal_Entity_Basic_Assoc;
Bare_Item_3 : Bare_Basic_Assoc;
Cast_Expr_3 : Internal_Entity_Basic_Assoc;
Cast_Result_9 : Internal_Entity_Aggregate_Assoc;
Fld_58 : Logic_Equation;
As_Entity_7 : Internal_Entity_Basic_Assoc;
Is_Null_7 : Boolean;
If_Result_13 : Internal_Entity_Basic_Assoc;
Map_Result_5 : Logic_Equation_Array_Access;
Logic_Boolean_Op_4 : Logic_Equation;
Fld_59 : Internal_Entity_Base_Assoc_List;
Fld_60 : Bare_Base_Assoc_List;
Fld_61 : Internal_Entity_Info;
Get_Result_4 : Bare_Base_Assoc;
As_Entity_8 : Internal_Entity_Base_Assoc;
Is_Null_8 : Boolean;
If_Result_14 : Internal_Entity_Base_Assoc;
Fld_62 : Internal_Entity_Expr;
Fld_63 : Logic_Var;
Fld_64 : Internal_Entity_Base_Type_Decl;
Cast_Result_10 : Internal_Entity;
Bind_Result_4 : Logic_Equation;
Fld_65 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_66 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Let_Result_1 : Logic_Equation;
True_Rel_5 : Logic_Equation;
Fld_67 : Internal_Entity_Base_Assoc_List;
Fld_68 : Bare_Base_Assoc_List;
Len : Integer;
Is_Equal_6 : Boolean;
If_Result_15 : Logic_Equation;
Fld_69 : Internal_Entity_Base_Assoc_List;
Fld_70 : Bare_Base_Assoc_List;
Len_1 : Integer;
Arith_Result : Integer;
Fld_71 : Internal_Entity_Base_Assoc_List;
Fld_72 : Bare_Base_Assoc_List;
Fld_73 : Internal_Entity_Info;
Get_Result_5 : Bare_Base_Assoc;
As_Entity_9 : Internal_Entity_Base_Assoc;
Is_Null_9 : Boolean;
If_Result_16 : Internal_Entity_Base_Assoc;
Fld_74 : Internal_Entity_Expr;
Scope_Result_5 : Internal_Entity_Expr;
Result_Var_5 : Internal_Entity_Expr;
Fld_75 : Logic_Var;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Bind_Result_5 : Logic_Equation;
Fld_76 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Let_Result_2 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Fld_77 : Internal_Entity_Base_Assoc_List;
Fld_78 : Bare_Base_Assoc_List;
Fld_79 : Internal_Entity_Info;
Item_1 : Internal_Entity_Base_Assoc;
Bare_Item_1 : Bare_Base_Assoc;
Index_2 : Integer;
True_Rel_6 : Logic_Equation;
Fld_80 : Internal_Entity_Expr;
Cast_Result_12 : Internal_Entity_Name;
Fld_81 : Logic_Equation;
Fld_82 : Internal_Entity_Expr;
Fld_83 : Logic_Equation;
Cast_Expr_5 : Internal_Entity_Base_Assoc;
Cast_Result_13 : Internal_Entity_Pragma_Argument_Assoc;
Fld_84 : Internal_Entity_Name;
Sym_6 : Symbol_Type;
Fld_85 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
If_Result_17 : Logic_Equation;
Comp_Result : Boolean;
If_Result_18 : Logic_Equation;
As_Entity_10 : Internal_Entity_Base_Assoc;
Is_Null_10 : Boolean;
If_Result_19 : Internal_Entity_Base_Assoc;
Map_Result_6 : Logic_Equation_Array_Access;
Logic_Boolean_Op_5 : Logic_Equation;
Fld_86 : Internal_Entity_Base_Assoc_List;
Fld_87 : Bare_Base_Assoc_List;
Fld_88 : Internal_Entity_Info;
Item_0 : Internal_Entity_Base_Assoc;
Bare_Item_0 : Bare_Base_Assoc;
Fld_89 : Internal_Entity_Expr;
Fld_90 : Logic_Equation;
True_Rel_7 : Logic_Equation;
Or_Pred : Logic_Equation;
As_Entity_11 : Internal_Entity_Base_Assoc;
Is_Null_11 : Boolean;
If_Result_20 : Internal_Entity_Base_Assoc;
Map_Result_7 : Logic_Equation_Array_Access;
Logic_Boolean_Op_6 : Logic_Equation;
Fld_91 : Internal_Entity_Identifier;
Sym_7 : Symbol_Type;
Fld_92 : Boolean;
If_Result_21 : Logic_Equation;
Fld_93 : Internal_Entity_Identifier;
Sym_8 : Symbol_Type;
Fld_94 : Boolean;
If_Result_22 : Logic_Equation;
Fld_95 : Internal_Entity_Identifier;
Sym_9 : Symbol_Type;
Fld_96 : Boolean;
If_Result_23 : Logic_Equation;
Fld_97 : Internal_Entity_Identifier;
Sym_10 : Symbol_Type;
Fld_98 : Boolean;
If_Result_24 : Logic_Equation;
Fld_99 : Internal_Entity_Identifier;
Fld_100 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_7 : Boolean;
Is_Equal_8 : Boolean;
If_Result_25 : Boolean;
Is_Equal_9 : Boolean;
If_Result_26 : Boolean;
Is_Equal_10 : Boolean;
If_Result_27 : Boolean;
Is_Equal_11 : Boolean;
If_Result_28 : Boolean;
Is_Equal_12 : Boolean;
If_Result_29 : Boolean;
Is_Equal_13 : Boolean;
If_Result_30 : Boolean;
Is_Equal_14 : Boolean;
If_Result_31 : Boolean;
Is_Equal_15 : Boolean;
If_Result_32 : Boolean;
If_Result_33 : Logic_Equation;
Fld_101 : Internal_Entity_Identifier;
Fld_102 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Is_Equal_16 : Boolean;
Is_Equal_17 : Boolean;
If_Result_34 : Boolean;
Is_Equal_18 : Boolean;
If_Result_35 : Boolean;
If_Result_36 : Logic_Equation;
Fld_103 : Internal_Entity_Identifier;
Sym_23 : Symbol_Type;
Fld_104 : Boolean;
If_Result_37 : Logic_Equation;
Fld_105 : Internal_Entity_Identifier;
Fld_106 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Is_Equal_19 : Boolean;
Is_Equal_20 : Boolean;
If_Result_38 : Boolean;
Is_Equal_21 : Boolean;
If_Result_39 : Boolean;
Is_Equal_22 : Boolean;
If_Result_40 : Boolean;
Is_Equal_23 : Boolean;
If_Result_41 : Boolean;
Is_Equal_24 : Boolean;
If_Result_42 : Boolean;
Is_Equal_25 : Boolean;
If_Result_43 : Boolean;
Is_Equal_26 : Boolean;
If_Result_44 : Boolean;
Is_Equal_27 : Boolean;
If_Result_45 : Boolean;
Is_Equal_28 : Boolean;
If_Result_46 : Boolean;
If_Result_47 : Logic_Equation;
Fld_107 : Internal_Entity_Identifier;
Sym_34 : Symbol_Type;
Fld_108 : Boolean;
If_Result_48 : Logic_Equation;
Fld_109 : Internal_Entity_Identifier;
Sym_35 : Symbol_Type;
Fld_110 : Boolean;
Fld_111 : Internal_Entity_Identifier;
Sym_36 : Symbol_Type;
Fld_112 : Boolean;
If_Result_49 : Boolean;
Fld_113 : Internal_Entity_Identifier;
Sym_37 : Symbol_Type;
Fld_114 : Boolean;
If_Result_50 : Boolean;
Fld_115 : Internal_Entity_Identifier;
Sym_38 : Symbol_Type;
Fld_116 : Boolean;
If_Result_51 : Boolean;
If_Result_52 : Logic_Equation;
            procedure Finalizer_Scope_1368 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (True_Rel_4);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (False_Rel);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Logic_Boolean_Op_3);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Logic_Boolean_Op_4);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (Fld_65);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Fld_66);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (True_Rel_5);
                     Dec_Ref (If_Result_15);
                     Dec_Ref (Bind_Result_5);
                     Dec_Ref (Fld_76);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Map_Result_6);
                     Dec_Ref (Logic_Boolean_Op_5);
                     Dec_Ref (Map_Result_7);
                     Dec_Ref (Logic_Boolean_Op_6);
                     Dec_Ref (If_Result_21);
                     Dec_Ref (If_Result_22);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_33);
                     Dec_Ref (If_Result_36);
                     Dec_Ref (If_Result_37);
                     Dec_Ref (If_Result_47);
                     Dec_Ref (If_Result_48);
                     Dec_Ref (If_Result_52);
            end Finalizer_Scope_1368;
            procedure Finalizer_Scope_3489 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_3489;
            procedure Finalizer_Scope_3490 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3490;
            procedure Finalizer_Scope_3491 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_3491;
            procedure Finalizer_Scope_3492 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_3492;
            procedure Finalizer_Scope_3493 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (True_Rel_3);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_3493;
            procedure Finalizer_Scope_3494 is
            begin
                     Dec_Ref (True_Rel_2);
                     Dec_Ref (Fld_25);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (If_Result_7);
            end Finalizer_Scope_3494;
            procedure Finalizer_Scope_3495 is
            begin
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_7);
            end Finalizer_Scope_3495;
            procedure Finalizer_Scope_3499 is
            begin
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (Fld_48);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Fld_50);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_3499;
            procedure Finalizer_Scope_3500 is
            begin
                     Dec_Ref (Fld_58);
            end Finalizer_Scope_3500;
            procedure Finalizer_Scope_3502 is
            begin
                     Dec_Ref (True_Rel_6);
                     Dec_Ref (Fld_81);
                     Dec_Ref (Fld_83);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (If_Result_18);
            end Finalizer_Scope_3502;
            procedure Finalizer_Scope_3505 is
            begin
                     Dec_Ref (Fld_90);
                     Dec_Ref (True_Rel_7);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_3505;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11737 '<If at nodes.lkt:16043:9>' If_Result_52 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11543 '<FieldAccess for name_is at nodes.lkt:16043:12>' Fld_110 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11541 '<FieldAccess for id at nodes.lkt:16043:12>' Fld_109 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_109 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11541
      if Fld_109.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11542 '<Symbol Assert at nodes.lkt:16043:28>' Sym_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Assert); 
--# expr-done 11542
--# property-call-start Name.name_is
Fld_110 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_109.Node, Sym => Sym_35);
--# end
--# expr-done 11543
if Fld_110 then
   If_Result_49 := True;
else
   --# expr-start 11546 '<FieldAccess for name_is at nodes.lkt:16043:42>' Fld_112 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11544 '<FieldAccess for id at nodes.lkt:16043:42>' Fld_111 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_111 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11544
      if Fld_111.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11545 '<Symbol Loop_Invariant at nodes.lkt:16043:58>' Sym_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Loop_Invariant); 
--# expr-done 11545
--# property-call-start Name.name_is
Fld_112 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_111.Node, Sym => Sym_36);
--# end
--# expr-done 11546
   If_Result_49 := Fld_112;
end if;
if If_Result_49 then
   If_Result_50 := True;
else
   --# expr-start 11549 '<FieldAccess for name_is at nodes.lkt:16043:80>' Fld_114 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11547 '<FieldAccess for id at nodes.lkt:16043:80>' Fld_113 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_113 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11547
      if Fld_113.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11548 '<Symbol Compile_Time_Warning at nodes.lkt:16043:96>' Sym_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compile_Time_Warning); 
--# expr-done 11548
--# property-call-start Name.name_is
Fld_114 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_113.Node, Sym => Sym_37);
--# end
--# expr-done 11549
   If_Result_50 := Fld_114;
end if;
if If_Result_50 then
   If_Result_51 := True;
else
   --# expr-start 11552 '<FieldAccess for name_is at nodes.lkt:16043:124>' Fld_116 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11550 '<FieldAccess for id at nodes.lkt:16043:124>' Fld_115 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_115 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11550
      if Fld_115.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11551 '<Symbol Compile_Time_Error at nodes.lkt:16043:140>' Sym_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compile_Time_Error); 
--# expr-done 11551
--# property-call-start Name.name_is
Fld_116 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_115.Node, Sym => Sym_38);
--# end
--# expr-done 11552
   If_Result_51 := Fld_116;
end if;
if If_Result_51 then
   --# expr-start 11568 '<And at nodes.lkt:16043:168>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11558 '<Let at nodes.lkt:16044:13>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16044'
--# scope-start
--# expr-start 11554 '<FieldAccess for assoc_expr at nodes.lkt:16044:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16044'
--# expr-start 11553 '<FieldAccess for args at nodes.lkt:16044:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16044'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11553
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11554
Local_Expr_1 := Fld_3; 
--# bind expr Local_Expr_1
--# expr-start 11557 '<And at nodes.lkt:16046:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16046'
--# expr-start 11555 '<FieldAccess for sub_equation at nodes.lkt:16046:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16046'
      if Local_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Expr_1.Info);
--# end
--# expr-done 11555
--# expr-start 11556 '<FieldAccess for expect_bool_derived_type at nodes.lkt:16046:37>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16046'
      if Local_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_5 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Local_Expr_1.Node, Origin => Origin);
--# end
--# expr-done 11556
And_Pred := Create_And (Fld_4, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16046:13") else null)); 
--# expr-done 11557
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 11558
--# expr-start 11559 '<FieldAccess for args at nodes.lkt:16047:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16047'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11559
Fld_7 := Fld_6.Node;
Get_Result_1 := Get (Self, Fld_7, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_12 := If_Result_1;
if Var_Expr_12 /= No_Entity_Base_Assoc then
   --# scope-start
   --# bind arg Var_Expr_12
--# expr-start 11560 '<FieldAccess for assoc_expr at nodes.lkt:16048:22>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16048'
      if Var_Expr_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_12.Node, E_Info => Var_Expr_12.Info);
--# end
--# expr-done 11560
Var_Expr_13 := Fld_9;
if Var_Expr_13 /= No_Entity_Expr then
   --# scope-start
   --# bind msg Var_Expr_13
--# expr-start 11567 '<And at nodes.lkt:16049:26>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16049'
--# expr-start 11565 '<And at nodes.lkt:16050:21>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16050'
--# expr-start 11563 '<Bind at nodes.lkt:16050:21>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16050'
--# expr-start 11561 '<FieldAccess for expected_type_var at nodes.lkt:16050:25>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16050'
      if Var_Expr_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_13.Node);
--# expr-done 11561
Fld_10.Value := No_Entity;
Entity_Vars.Reset (Fld_10);
--# expr-start 11562 '<FieldAccess for std_string_type at nodes.lkt:16050:50>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16050'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_11 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 11562
      Cast_Result := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
Bind_Result := Solver.Create_Assign (Fld_10, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16050:21") else null)); 
--# expr-done 11563
--# expr-start 11564 '<FieldAccess for sub_equation at nodes.lkt:16050:78>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16050'
      if Var_Expr_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_13.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_13.Info);
--# end
--# expr-done 11564
And_Pred_1 := Create_And (Bind_Result, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16050:21") else null)); 
--# expr-done 11565
--# expr-start 11566 '<FieldAccess for matches_expected_type at nodes.lkt:16051:23>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16051'
      if Var_Expr_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_13.Node, Origin => Origin);
--# end
--# expr-done 11566
And_Pred_2 := Create_And (And_Pred_1, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16049:26") else null)); 
--# expr-done 11567
Scope_Result := And_Pred_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3490;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16051:64") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3489;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16052:28") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Create_And (Let_Result, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16043:168") else null)); 
--# expr-done 11568
   If_Result_52 := And_Pred_3;
else
   --# expr-start 11736 '<If at nodes.lkt:16043:9>' If_Result_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11571 '<FieldAccess for name_is at nodes.lkt:16054:14>' Fld_108 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
--# expr-start 11569 '<FieldAccess for id at nodes.lkt:16054:14>' Fld_107 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_107 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11569
      if Fld_107.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11570 '<Symbol Unreferenced at nodes.lkt:16054:30>' Sym_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unreferenced); 
--# expr-done 11570
--# property-call-start Name.name_is
Fld_108 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_107.Node, Sym => Sym_34);
--# end
--# expr-done 11571
if Fld_108 then
   --# expr-start 11577 '<LogicAll at nodes.lkt:16054:52>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
--# expr-start 11576 '<Map at nodes.lkt:16054:52>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
   --# expr-start 11572 '<FieldAccess for args at nodes.lkt:16054:52>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16054'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11572
Fld_15 := Fld_14.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_15 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_15;
      begin
         for Untyped_Item_11 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_11 := Untyped_Item_11; 
Is_Null_2 := Bare_Item_11 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_2 := (Info => Fld_16, Node => Bare_Item_11); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_11 := If_Result_2; 
   --# scope-start
               --# bind assoc Item_11
      --# expr-start 11575 '<FieldAccess for xref_no_overloading at nodes.lkt:16055:24>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16055'
--# expr-start 11574 '<Cast to Name at nodes.lkt:16055:24>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16055'
--# expr-start 11573 '<FieldAccess for assoc_expr at nodes.lkt:16055:24>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16055'
      if Item_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_11.Node, E_Info => Item_11.Info);
--# end
--# expr-done 11573
   if Fld_17.Node = null
      or else Fld_17.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11574
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 11575
         declare
            Item_To_Append : constant Logic_Equation := Fld_18;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3491;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11576
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16054:52") else null)); 
--# expr-done 11577
   If_Result_48 := Logic_Boolean_Op;
else
   --# expr-start 11735 '<If at nodes.lkt:16043:9>' If_Result_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11579 '<FieldAccess for name_symbol at nodes.lkt:16057:14>' Fld_106 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
--# expr-start 11578 '<FieldAccess for id at nodes.lkt:16057:14>' Fld_105 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_105 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11578
      if Fld_105.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_106 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_105.Node);
--# end
--# expr-done 11579
--# expr-start 11580 '<Symbol Import at nodes.lkt:16057:39>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 11580
Is_Equal_28 := Fld_106 = Sym_24; 
if Is_Equal_28 then
   If_Result_46 := True;
else
--# expr-start 11581 '<Symbol Export at nodes.lkt:16057:51>' Sym_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 11581
Is_Equal_27 := Fld_106 = Sym_25; 
if Is_Equal_27 then
   If_Result_45 := True;
else
--# expr-start 11582 '<Symbol Interface at nodes.lkt:16057:63>' Sym_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 11582
Is_Equal_26 := Fld_106 = Sym_26; 
if Is_Equal_26 then
   If_Result_44 := True;
else
--# expr-start 11583 '<Symbol Convention at nodes.lkt:16057:78>' Sym_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 11583
Is_Equal_25 := Fld_106 = Sym_27; 
if Is_Equal_25 then
   If_Result_43 := True;
else
--# expr-start 11584 '<Symbol Pack at nodes.lkt:16057:94>' Sym_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pack); 
--# expr-done 11584
Is_Equal_24 := Fld_106 = Sym_28; 
if Is_Equal_24 then
   If_Result_42 := True;
else
--# expr-start 11585 '<Symbol Pure at nodes.lkt:16057:104>' Sym_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 11585
Is_Equal_23 := Fld_106 = Sym_29; 
if Is_Equal_23 then
   If_Result_41 := True;
else
--# expr-start 11586 '<Symbol Preelaborate at nodes.lkt:16057:114>' Sym_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 11586
Is_Equal_22 := Fld_106 = Sym_30; 
if Is_Equal_22 then
   If_Result_40 := True;
else
--# expr-start 11587 '<Symbol Elaborate_Body at nodes.lkt:16057:132>' Sym_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elaborate_Body); 
--# expr-done 11587
Is_Equal_21 := Fld_106 = Sym_31; 
if Is_Equal_21 then
   If_Result_39 := True;
else
--# expr-start 11588 '<Symbol Inline at nodes.lkt:16057:152>' Sym_32 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Inline); 
--# expr-done 11588
Is_Equal_20 := Fld_106 = Sym_32; 
if Is_Equal_20 then
   If_Result_38 := True;
else
--# expr-start 11589 '<Symbol Volatile at nodes.lkt:16057:164>' Sym_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile); 
--# expr-done 11589
Is_Equal_19 := Fld_106 = Sym_33; 
   If_Result_38 := Is_Equal_19;
end if;
   If_Result_39 := If_Result_38;
end if;
   If_Result_40 := If_Result_39;
end if;
   If_Result_41 := If_Result_40;
end if;
   If_Result_42 := If_Result_41;
end if;
   If_Result_43 := If_Result_42;
end if;
   If_Result_44 := If_Result_43;
end if;
   If_Result_45 := If_Result_44;
end if;
   If_Result_46 := If_Result_45;
end if;
if If_Result_46 then
   --# expr-start 11593 '<LogicAll at nodes.lkt:16057:181>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
--# expr-start 11592 '<Map at nodes.lkt:16057:181>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
   --# expr-start 11590 '<FieldAccess for associated_entity_names at nodes.lkt:16057:181>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld_19 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11590
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Name_Array_Access := Fld_19;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_10
      --# expr-start 11591 '<FieldAccess for xref_no_overloading at nodes.lkt:16057:229>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16057'
      if Item_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_20 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_10.Info);
--# end
--# expr-done 11591
         declare
            Item_To_Append : constant Logic_Equation := Fld_20;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3492;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11592
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16057:181") else null)); 
--# expr-done 11593
   If_Result_47 := Logic_Boolean_Op_1;
else
   --# expr-start 11734 '<If at nodes.lkt:16043:9>' If_Result_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11596 '<FieldAccess for name_is at nodes.lkt:16058:14>' Fld_104 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
--# expr-start 11594 '<FieldAccess for id at nodes.lkt:16058:14>' Fld_103 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_103 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11594
      if Fld_103.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11595 '<Symbol Warnings at nodes.lkt:16058:30>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Warnings); 
--# expr-done 11595
--# property-call-start Name.name_is
Fld_104 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_103.Node, Sym => Sym_23);
--# end
--# expr-done 11596
if Fld_104 then
   --# expr-start 11618 '<LogicAll at nodes.lkt:16058:48>' Logic_Boolean_Op_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
--# expr-start 11617 '<Map at nodes.lkt:16058:48>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
   --# expr-start 11597 '<FieldAccess for args at nodes.lkt:16058:48>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16058'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11597
Fld_22 := Fld_21.Node;
      declare
         Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_22 = null then
      Map_Result_2 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_22;
      begin
         for Untyped_Item_8 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_8 := Untyped_Item_8; 
Is_Null_3 := Bare_Item_8 = null; 
if Is_Null_3 then
   If_Result_8 := No_Entity_Base_Assoc;
else
Fld_23 := Fld_21.Info;
As_Entity_3 := (Info => Fld_23, Node => Bare_Item_8); 
   If_Result_8 := As_Entity_3;
end if;
                  Item_8 := If_Result_8; 
   --# scope-start
               --# bind arg Item_8
--# expr-start 11598 '<FieldAccess for assoc_expr at nodes.lkt:16059:22>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16059'
      if Item_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_8.Node, E_Info => Item_8.Info);
--# end
--# expr-done 11598
Var_Expr_9 := Fld_24;
if Var_Expr_9 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_9
--# expr-start 11616 '<If at nodes.lkt:16060:17>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16060'
--# expr-start 11599 '<IsA Identifier at nodes.lkt:16060:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16060'
Is_A := Var_Expr_9.Node /= null 
and then Var_Expr_9.Node.Kind in Ada_Identifier_Range; 
--# expr-done 11599
if Is_A then
   --# expr-start 11608 '<If at nodes.lkt:16061:21>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16061'
--# expr-start 11601 '<FieldAccess for name_symbol at nodes.lkt:16061:24>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16061'
--# expr-start 11600 '<Cast to Identifier at nodes.lkt:16061:24>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16061'
Cast_Expr_1 := Var_Expr_9; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Identifier
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Identifier;
   end if;
--# expr-done 11600
      if Cast_Result_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result_3.Node);
--# end
--# expr-done 11601
--# expr-start 11602 '<Symbol On at nodes.lkt:16062:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16062'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_On); 
--# expr-done 11602
Is_Equal_3 := Fld_26 = Sym; 
if Is_Equal_3 then
   If_Result_5 := True;
else
--# expr-start 11603 '<Symbol Off at nodes.lkt:16062:33>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16062'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Off); 
--# expr-done 11603
Is_Equal_2 := Fld_26 = Sym_1; 
if Is_Equal_2 then
   If_Result_4 := True;
else
--# expr-start 11604 '<Symbol GNAT at nodes.lkt:16062:42>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16062'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnat); 
--# expr-done 11604
Is_Equal_1 := Fld_26 = Sym_2; 
if Is_Equal_1 then
   If_Result_3 := True;
else
--# expr-start 11605 '<Symbol GNATprove at nodes.lkt:16062:52>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16062'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Gnatprove); 
--# expr-done 11605
Is_Equal := Fld_26 = Sym_3; 
   If_Result_3 := Is_Equal;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   True_Rel_2 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16063:26") else null)); 
   If_Result_6 := True_Rel_2;
else
   --# expr-start 11607 '<FieldAccess for xref_no_overloading at nodes.lkt:16064:26>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16064'
--# expr-start 11606 '<Cast to Identifier at nodes.lkt:16064:26>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16064'
Cast_Expr := Var_Expr_9; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Identifier_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Identifier
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_2 := No_Entity_Identifier;
   end if;
--# expr-done 11606
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_25 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 11607
   If_Result_6 := Fld_25;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 11608
   If_Result_7 := If_Result_6;
else
   --# expr-start 11615 '<And at nodes.lkt:16066:21>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16066'
--# expr-start 11613 '<And at nodes.lkt:16066:21>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16066'
--# expr-start 11611 '<Bind at nodes.lkt:16066:21>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16066'
--# expr-start 11609 '<FieldAccess for expected_type_var at nodes.lkt:16066:25>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16066'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_9.Node);
--# expr-done 11609
Fld_27.Value := No_Entity;
Entity_Vars.Reset (Fld_27);
--# expr-start 11610 '<FieldAccess for std_string_type at nodes.lkt:16066:51>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16066'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_28 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 11610
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_27, Cast_Result_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16066:21") else null)); 
--# expr-done 11611
--# expr-start 11612 '<FieldAccess for sub_equation at nodes.lkt:16067:25>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16067'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_9.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 11612
And_Pred_4 := Create_And (Bind_Result_1, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16066:21") else null)); 
--# expr-done 11613
--# expr-start 11614 '<FieldAccess for matches_expected_type at nodes.lkt:16068:25>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16068'
      if Var_Expr_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_30 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_9.Node, Origin => Origin);
--# end
--# expr-done 11614
And_Pred_5 := Create_And (And_Pred_4, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16066:21") else null)); 
--# expr-done 11615
   If_Result_7 := And_Pred_5;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 11616
Scope_Result_2 := If_Result_7;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3494;
   Result_Var_2 := Scope_Result_2;
else
   True_Rel_3 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16070:29") else null)); 
   Result_Var_2 := True_Rel_3;
end if;
      Inc_Ref (Result_Var_2);
         declare
            Item_To_Append : constant Logic_Equation := Result_Var_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3493;
         end loop;
      end;
      end if;
         Map_Result_2 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_2_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 11617
Logic_Boolean_Op_2 := Solver.Create_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16058:48") else null)); 
--# expr-done 11618
   If_Result_37 := Logic_Boolean_Op_2;
else
   --# expr-start 11733 '<If at nodes.lkt:16043:9>' If_Result_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11620 '<FieldAccess for name_symbol at nodes.lkt:16078:14>' Fld_102 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16078'
--# expr-start 11619 '<FieldAccess for id at nodes.lkt:16078:14>' Fld_101 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16078'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_101 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11619
      if Fld_101.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_102 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_101.Node);
--# end
--# expr-done 11620
--# expr-start 11621 '<Symbol Style_Checks at nodes.lkt:16078:39>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16078'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Style_Checks); 
--# expr-done 11621
Is_Equal_18 := Fld_102 = Sym_20; 
if Is_Equal_18 then
   If_Result_35 := True;
else
--# expr-start 11622 '<Symbol Import_Function at nodes.lkt:16078:57>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16078'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import_Function); 
--# expr-done 11622
Is_Equal_17 := Fld_102 = Sym_21; 
if Is_Equal_17 then
   If_Result_34 := True;
else
--# expr-start 11623 '<Symbol Import_Procedure at nodes.lkt:16078:78>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16078'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import_Procedure); 
--# expr-done 11623
Is_Equal_16 := Fld_102 = Sym_22; 
   If_Result_34 := Is_Equal_16;
end if;
   If_Result_35 := If_Result_34;
end if;
if If_Result_35 then
   True_Rel_4 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16078:103") else null)); 
   If_Result_36 := True_Rel_4;
else
   --# expr-start 11732 '<If at nodes.lkt:16043:9>' If_Result_33 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11625 '<FieldAccess for name_symbol at nodes.lkt:16079:14>' Fld_100 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
--# expr-start 11624 '<FieldAccess for id at nodes.lkt:16079:14>' Fld_99 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_99 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11624
      if Fld_99.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_100 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_99.Node);
--# end
--# expr-done 11625
--# expr-start 11626 '<Symbol Pre at nodes.lkt:16079:39>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 11626
Is_Equal_15 := Fld_100 = Sym_11; 
if Is_Equal_15 then
   If_Result_32 := True;
else
--# expr-start 11627 '<Symbol Post at nodes.lkt:16079:48>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 11627
Is_Equal_14 := Fld_100 = Sym_12; 
if Is_Equal_14 then
   If_Result_31 := True;
else
--# expr-start 11628 '<Symbol Pre'"'"'Class at nodes.lkt:16079:58>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 11628
Is_Equal_13 := Fld_100 = Sym_13; 
if Is_Equal_13 then
   If_Result_30 := True;
else
--# expr-start 11629 '<Symbol Post'"'"'Class at nodes.lkt:16079:73>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 11629
Is_Equal_12 := Fld_100 = Sym_14; 
if Is_Equal_12 then
   If_Result_29 := True;
else
--# expr-start 11630 '<Symbol Precondition at nodes.lkt:16079:89>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 11630
Is_Equal_11 := Fld_100 = Sym_15; 
if Is_Equal_11 then
   If_Result_28 := True;
else
--# expr-start 11631 '<Symbol Postcondition at nodes.lkt:16079:107>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 11631
Is_Equal_10 := Fld_100 = Sym_16; 
if Is_Equal_10 then
   If_Result_27 := True;
else
--# expr-start 11632 '<Symbol Precondition'"'"'Class at nodes.lkt:16079:126>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 11632
Is_Equal_9 := Fld_100 = Sym_17; 
if Is_Equal_9 then
   If_Result_26 := True;
else
--# expr-start 11633 '<Symbol Postcondition'"'"'Class at nodes.lkt:16079:150>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 11633
Is_Equal_8 := Fld_100 = Sym_18; 
if Is_Equal_8 then
   If_Result_25 := True;
else
--# expr-start 11634 '<Symbol Initial_Condition at nodes.lkt:16079:175>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Initial_Condition); 
--# expr-done 11634
Is_Equal_7 := Fld_100 = Sym_19; 
   If_Result_25 := Is_Equal_7;
end if;
   If_Result_26 := If_Result_25;
end if;
   If_Result_27 := If_Result_26;
end if;
   If_Result_28 := If_Result_27;
end if;
   If_Result_29 := If_Result_28;
end if;
   If_Result_30 := If_Result_29;
end if;
   If_Result_31 := If_Result_30;
end if;
   If_Result_32 := If_Result_31;
end if;
if If_Result_32 then
--# expr-start 11636 '<FieldAccess for assoc_expr at nodes.lkt:16079:201>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
--# expr-start 11635 '<FieldAccess for args at nodes.lkt:16079:201>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16079'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_31 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11635
Fld_32 := Fld_31.Node;
Get_Result_2 := Get (Self, Fld_32, 0, True); 
Is_Null_4 := Get_Result_2 = null; 
if Is_Null_4 then
   If_Result_9 := No_Entity_Base_Assoc;
else
Fld_33 := Fld_31.Info;
As_Entity_4 := (Info => Fld_33, Node => Get_Result_2); 
   If_Result_9 := As_Entity_4;
end if;
      if If_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_34 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_9.Node, E_Info => If_Result_9.Info);
--# end
--# expr-done 11636
Var_Expr_7 := Fld_34;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_7
--# expr-start 11643 '<And at nodes.lkt:16080:23>' And_Pred_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16080'
--# expr-start 11641 '<And at nodes.lkt:16081:17>' And_Pred_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16081'
--# expr-start 11639 '<Bind at nodes.lkt:16081:17>' Bind_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16081'
--# expr-start 11637 '<FieldAccess for expected_type_var at nodes.lkt:16081:21>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16081'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_7.Node);
--# expr-done 11637
Fld_35.Value := No_Entity;
Entity_Vars.Reset (Fld_35);
--# expr-start 11638 '<FieldAccess for bool_type at nodes.lkt:16081:47>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16081'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_36 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11638
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_36.Node,
         Info => Fld_36.Info);
Bind_Result_2 := Solver.Create_Assign (Fld_35, Cast_Result_5, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16081:17") else null)); 
--# expr-done 11639
--# expr-start 11640 '<FieldAccess for sub_equation at nodes.lkt:16081:69>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16081'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_37 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 11640
And_Pred_6 := Create_And (Bind_Result_2, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16081:17") else null)); 
--# expr-done 11641
--# expr-start 11642 '<FieldAccess for matches_expected_formal_type at nodes.lkt:16082:19>' Fld_38 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16082'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_38 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Var_Expr_7.Node, Origin => Origin);
--# end
--# expr-done 11642
And_Pred_7 := Create_And (And_Pred_6, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16080:23") else null)); 
--# expr-done 11643
Scope_Result_3 := And_Pred_7;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3495;
   Result_Var_3 := Scope_Result_3;
else
   False_Rel := Solver.Create_False ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16082:68") else null)); 
   Result_Var_3 := False_Rel;
end if;
      Inc_Ref (Result_Var_3);
   If_Result_33 := Result_Var_3;
else
   --# expr-start 11731 '<If at nodes.lkt:16043:9>' If_Result_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11646 '<FieldAccess for name_is at nodes.lkt:16084:14>' Fld_98 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
--# expr-start 11644 '<FieldAccess for id at nodes.lkt:16084:14>' Fld_97 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_97 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11644
      if Fld_97.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11645 '<Symbol Test_Case at nodes.lkt:16084:30>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 11645
--# property-call-start Name.name_is
Fld_98 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_97.Node, Sym => Sym_10);
--# end
--# expr-done 11646
if Fld_98 then
   --# expr-start 11665 '<LogicAll at nodes.lkt:16084:49>' Logic_Boolean_Op_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
--# expr-start 11664 '<Map at nodes.lkt:16084:49>' Map_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
   --# expr-start 11653 '<Map at nodes.lkt:16084:49>' Map_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
   --# expr-start 11647 '<FieldAccess for args at nodes.lkt:16084:49>' Fld_39 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16084'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_39 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11647
Fld_40 := Fld_39.Node;
      declare
         Map_Result_3_Vec : Internal_Entity_Base_Assoc_Vectors.Vector;
      begin
      if Fld_40 = null then
      Map_Result_3 := Create_Internal_Entity_Base_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_40;
      begin
         for Untyped_Item_4 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_4 := Untyped_Item_4; 
Is_Null_5 := Bare_Item_4 = null; 
if Is_Null_5 then
   If_Result_10 := No_Entity_Base_Assoc;
else
Fld_41 := Fld_39.Info;
As_Entity_5 := (Info => Fld_41, Node => Bare_Item_4); 
   If_Result_10 := As_Entity_5;
end if;
                  Item_4 := If_Result_10; 
   --# scope-start
               --# bind arg Item_4
--# expr-start 11648 '<Cast to PragmaArgumentAssoc at nodes.lkt:16085:22>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16085'
Cast_Expr_2 := Item_4; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Pragma_Argument_Assoc_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Pragma_Argument_Assoc
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_6 := No_Entity_Pragma_Argument_Assoc;
   end if;
--# expr-done 11648
Var_Expr_5 := Cast_Result_6;
if Var_Expr_5 /= No_Entity_Pragma_Argument_Assoc then
   --# scope-start
   --# bind parg Var_Expr_5
--# expr-start 11650 '<FieldAccess for name_symbol at nodes.lkt:16086:27>' Fld_43 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16086'
--# expr-start 11649 '<FieldAccess for name at nodes.lkt:16086:27>' Fld_42 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16086'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_42 := Create_Internal_Entity_Name (Node => Var_Expr_5.Node.Pragma_Argument_Assoc_F_Name, Info => Var_Expr_5.Info);
--# expr-done 11649
      if Fld_42.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_43 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_42.Node);
--# end
--# expr-done 11650
--# expr-start 11651 '<Symbol Requires at nodes.lkt:16086:54>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16086'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Requires); 
--# expr-done 11651
Is_Equal_5 := Fld_43 = Sym_4; 
if Is_Equal_5 then
   If_Result_11 := True;
else
--# expr-start 11652 '<Symbol Ensures at nodes.lkt:16086:68>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16086'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ensures); 
--# expr-done 11652
Is_Equal_4 := Fld_43 = Sym_5; 
   If_Result_11 := Is_Equal_4;
end if;
Scope_Result_4 := If_Result_11;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
         if Result_Var_4 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Assoc := Item_4;
         begin
            Internal_Entity_Base_Assoc_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result_3 := Create_Internal_Entity_Base_Assoc_Array
           (Items_Count => Natural (Internal_Entity_Base_Assoc_Vectors.Length (Map_Result_3_Vec)));
         for I in Map_Result_3.Items'Range loop
            Map_Result_3.Items (I) := Internal_Entity_Base_Assoc_Vectors.Get
              (Map_Result_3_Vec,
               I + Internal_Entity_Base_Assoc_Vectors.Index_Type'First - Map_Result_3.Items'First);
         end loop;
         Internal_Entity_Base_Assoc_Vectors.Destroy (Map_Result_3_Vec);
      end;
--# expr-done 11653
      declare
         Map_Result_4_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Assoc_Array_Access := Map_Result_3;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind arg Item_6
      --# expr-start 11663 '<And at nodes.lkt:16089:22>' And_Pred_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16089'
--# expr-start 11660 '<And at nodes.lkt:16090:17>' And_Pred_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
--# expr-start 11657 '<Bind at nodes.lkt:16090:17>' Bind_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
--# expr-start 11655 '<FieldAccess for expected_type_var at nodes.lkt:16090:21>' Fld_45 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
--# expr-start 11654 '<FieldAccess for assoc_expr at nodes.lkt:16090:21>' Fld_44 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_44 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_6.Node, E_Info => Item_6.Info);
--# end
--# expr-done 11654
      if Fld_44.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_45 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_44.Node);
--# expr-done 11655
Fld_45.Value := No_Entity;
Entity_Vars.Reset (Fld_45);
--# expr-start 11656 '<FieldAccess for bool_type at nodes.lkt:16090:59>' Fld_46 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_46 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11656
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_46.Node,
         Info => Fld_46.Info);
Bind_Result_3 := Solver.Create_Assign (Fld_45, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16090:17") else null)); 
--# expr-done 11657
--# expr-start 11659 '<FieldAccess for sub_equation at nodes.lkt:16090:81>' Fld_48 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
--# expr-start 11658 '<FieldAccess for assoc_expr at nodes.lkt:16090:81>' Fld_47 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16090'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_47 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_6.Node, E_Info => Item_6.Info);
--# end
--# expr-done 11658
      if Fld_47.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_48 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_47.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_47.Info);
--# end
--# expr-done 11659
And_Pred_8 := Create_And (Bind_Result_3, Fld_48, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16090:17") else null)); 
--# expr-done 11660
--# expr-start 11662 '<FieldAccess for matches_expected_formal_type at nodes.lkt:16091:19>' Fld_50 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16091'
--# expr-start 11661 '<FieldAccess for assoc_expr at nodes.lkt:16091:19>' Fld_49 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16091'
      if Item_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_49 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_6.Node, E_Info => Item_6.Info);
--# end
--# expr-done 11661
      if Fld_49.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_50 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Fld_49.Node, Origin => Origin);
--# end
--# expr-done 11662
And_Pred_9 := Create_And (And_Pred_8, Fld_50, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16089:22") else null)); 
--# expr-done 11663
         declare
            Item_To_Append : constant Logic_Equation := And_Pred_9;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3499;
         end loop;
      end;
         Map_Result_4 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_4_Vec)));
         for I in Map_Result_4.Items'Range loop
            Map_Result_4.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_4_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_4.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_4_Vec);
      end;
--# expr-done 11664
Logic_Boolean_Op_3 := Solver.Create_All (Relation_Array (Map_Result_4.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16084:49") else null)); 
--# expr-done 11665
   If_Result_24 := Logic_Boolean_Op_3;
else
   --# expr-start 11730 '<If at nodes.lkt:16043:9>' If_Result_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11668 '<FieldAccess for name_is at nodes.lkt:16093:14>' Fld_96 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
--# expr-start 11666 '<FieldAccess for id at nodes.lkt:16093:14>' Fld_95 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_95 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11666
      if Fld_95.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11667 '<Symbol Contract_Cases at nodes.lkt:16093:30>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 11667
--# property-call-start Name.name_is
Fld_96 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_95.Node, Sym => Sym_9);
--# end
--# expr-done 11668
if Fld_96 then
   --# expr-start 11676 '<LogicAll at nodes.lkt:16093:54>' Logic_Boolean_Op_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
--# expr-start 11675 '<Map at nodes.lkt:16093:54>' Map_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
   --# expr-start 11672 '<FieldAccess for assocs at nodes.lkt:16093:54>' Fld_55 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
--# expr-start 11671 '<Cast to BaseAggregate at nodes.lkt:16093:54>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
--# expr-start 11670 '<FieldAccess for assoc_expr at nodes.lkt:16093:54>' Fld_54 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
--# expr-start 11669 '<FieldAccess for args at nodes.lkt:16093:54>' Fld_51 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16093'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_51 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11669
Fld_52 := Fld_51.Node;
Get_Result_3 := Get (Self, Fld_52, 0, True); 
Is_Null_6 := Get_Result_3 = null; 
if Is_Null_6 then
   If_Result_12 := No_Entity_Base_Assoc;
else
Fld_53 := Fld_51.Info;
As_Entity_6 := (Info => Fld_53, Node => Get_Result_3); 
   If_Result_12 := As_Entity_6;
end if;
      if If_Result_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_54 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_12.Node, E_Info => If_Result_12.Info);
--# end
--# expr-done 11670
   if Fld_54.Node = null
      or else Fld_54.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result_8 := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_54.Node,
         Info => Fld_54.Info);
   else
         Cast_Result_8 := No_Entity_Base_Aggregate;
   end if;
--# expr-done 11671
      if Cast_Result_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_55 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_8.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_8.Info);
--# expr-done 11672
Fld_56 := Fld_55.Node;
      declare
         Map_Result_5_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_56 = null then
      Map_Result_5 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_56;
      begin
         for Untyped_Item_3 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_3 := Untyped_Item_3; 
Is_Null_7 := Bare_Item_3 = null; 
if Is_Null_7 then
   If_Result_13 := No_Entity_Basic_Assoc;
else
Fld_57 := Fld_55.Info;
As_Entity_7 := (Info => Fld_57, Node => Bare_Item_3); 
   If_Result_13 := As_Entity_7;
end if;
                  Item_3 := If_Result_13; 
   --# scope-start
               --# bind assoc Item_3
      --# expr-start 11674 '<FieldAccess for contract_cases_assoc_equation at nodes.lkt:16094:24>' Fld_58 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16094'
--# expr-start 11673 '<Cast to AggregateAssoc at nodes.lkt:16094:24>' Cast_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16094'
Cast_Expr_3 := Item_3; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Aggregate_Assoc_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Aggregate_Assoc
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_9 := No_Entity_Aggregate_Assoc;
   end if;
--# expr-done 11673
      if Cast_Result_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AggregateAssoc.contract_cases_assoc_equation
Fld_58 := Libadalang.Implementation.Aggregate_Assoc_P_Contract_Cases_Assoc_Equation (Node => Cast_Result_9.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_9.Info);
--# end
--# expr-done 11674
         declare
            Item_To_Append : constant Logic_Equation := Fld_58;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3500;
         end loop;
      end;
      end if;
         Map_Result_5 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_5_Vec)));
         for I in Map_Result_5.Items'Range loop
            Map_Result_5.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_5_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_5.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_5_Vec);
      end;
--# expr-done 11675
Logic_Boolean_Op_4 := Solver.Create_All (Relation_Array (Map_Result_5.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16093:54") else null)); 
--# expr-done 11676
   If_Result_23 := Logic_Boolean_Op_4;
else
   --# expr-start 11729 '<If at nodes.lkt:16043:9>' If_Result_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11679 '<FieldAccess for name_is at nodes.lkt:16096:14>' Fld_94 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16096'
--# expr-start 11677 '<FieldAccess for id at nodes.lkt:16096:14>' Fld_93 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16096'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_93 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11677
      if Fld_93.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11678 '<Symbol Debug at nodes.lkt:16096:30>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16096'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Debug); 
--# expr-done 11678
--# property-call-start Name.name_is
Fld_94 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_93.Node, Sym => Sym_8);
--# end
--# expr-done 11679
if Fld_94 then
   --# expr-start 11704 '<And at nodes.lkt:16096:45>' And_Pred_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16096'
--# expr-start 11693 '<If at nodes.lkt:16099:13>' If_Result_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16099'
--# expr-start 11682 '<Eq at nodes.lkt:16099:16>' Is_Equal_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16099'
--# expr-start 11681 '<length at nodes.lkt:16099:16>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16099'
--# expr-start 11680 '<FieldAccess for args at nodes.lkt:16099:16>' Fld_67 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16099'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_67 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11680
Fld_68 := Fld_67.Node; 
Len := Length (Fld_68); 
--# expr-done 11681
Is_Equal_6 := Len = 2; 
--# expr-done 11682
if Is_Equal_6 then
   --# expr-start 11692 '<Let at nodes.lkt:16100:17>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16100'
--# scope-start
--# expr-start 11684 '<FieldAccess for assoc_expr at nodes.lkt:16100:28>' Fld_62 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16100'
--# expr-start 11683 '<FieldAccess for args at nodes.lkt:16100:28>' Fld_59 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16100'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_59 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11683
Fld_60 := Fld_59.Node;
Get_Result_4 := Get (Self, Fld_60, 0, True); 
Is_Null_8 := Get_Result_4 = null; 
if Is_Null_8 then
   If_Result_14 := No_Entity_Base_Assoc;
else
Fld_61 := Fld_59.Info;
As_Entity_8 := (Info => Fld_61, Node => Get_Result_4); 
   If_Result_14 := As_Entity_8;
end if;
      if If_Result_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_62 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_14.Node, E_Info => If_Result_14.Info);
--# end
--# expr-done 11684
Local_Expr := Fld_62; 
--# bind expr Local_Expr
--# expr-start 11691 '<And at nodes.lkt:16102:17>' And_Pred_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16102'
--# expr-start 11689 '<And at nodes.lkt:16103:21>' And_Pred_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16103'
--# expr-start 11687 '<Bind at nodes.lkt:16103:21>' Bind_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16103'
--# expr-start 11685 '<FieldAccess for expected_type_var at nodes.lkt:16103:25>' Fld_63 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16103'
      if Local_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_63 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Local_Expr.Node);
--# expr-done 11685
Fld_63.Value := No_Entity;
Entity_Vars.Reset (Fld_63);
--# expr-start 11686 '<FieldAccess for bool_type at nodes.lkt:16103:51>' Fld_64 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16103'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_64 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 11686
      Cast_Result_10 := Create_Internal_Entity
        (Node => Fld_64.Node,
         Info => Fld_64.Info);
Bind_Result_4 := Solver.Create_Assign (Fld_63, Cast_Result_10, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16103:21") else null)); 
--# expr-done 11687
--# expr-start 11688 '<FieldAccess for sub_equation at nodes.lkt:16103:73>' Fld_65 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16103'
      if Local_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_65 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Expr.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Expr.Info);
--# end
--# expr-done 11688
And_Pred_10 := Create_And (Bind_Result_4, Fld_65, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16103:21") else null)); 
--# expr-done 11689
--# expr-start 11690 '<FieldAccess for matches_expected_formal_type at nodes.lkt:16104:23>' Fld_66 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16104'
      if Local_Expr.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_formal_type
Fld_66 := Libadalang.Implementation.Expr_P_Matches_Expected_Formal_Type (Node => Local_Expr.Node, Origin => Origin);
--# end
--# expr-done 11690
And_Pred_11 := Create_And (And_Pred_10, Fld_66, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16102:17") else null)); 
--# expr-done 11691
Let_Result_1 := And_Pred_11; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11692
   If_Result_15 := Let_Result_1;
else
   True_Rel_5 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16105:20") else null)); 
   If_Result_15 := True_Rel_5;
end if;
      Inc_Ref (If_Result_15);
--# expr-done 11693
--# expr-start 11703 '<Let at nodes.lkt:16107:13>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
--# scope-start
--# expr-start 11694 '<FieldAccess for args at nodes.lkt:16107:24>' Fld_71 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_71 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11694
Fld_72 := Fld_71.Node;
--# expr-start 11697 '<Arithmetic - at nodes.lkt:16107:35>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
--# expr-start 11696 '<length at nodes.lkt:16107:35>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
--# expr-start 11695 '<FieldAccess for args at nodes.lkt:16107:35>' Fld_69 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_69 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11695
Fld_70 := Fld_69.Node; 
Len_1 := Length (Fld_70); 
--# expr-done 11696
Arith_Result := (Len_1 - 1); 
--# expr-done 11697
Get_Result_5 := Get (Self, Fld_72, Arith_Result, True); 
Is_Null_9 := Get_Result_5 = null; 
if Is_Null_9 then
   If_Result_16 := No_Entity_Base_Assoc;
else
Fld_73 := Fld_71.Info;
As_Entity_9 := (Info => Fld_73, Node => Get_Result_5); 
   If_Result_16 := As_Entity_9;
end if;
Var_Expr_272 := If_Result_16;
if Var_Expr_272 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11698 '<FieldAccess for assoc_expr at nodes.lkt:16107:24>' Fld_74 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16107'
      if Var_Expr_272.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_74 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_272.Node, E_Info => Var_Expr_272.Info);
--# end
--# expr-done 11698
Scope_Result_5 := Fld_74;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Expr;
end if;
Local_Proc := Result_Var_5; 
--# bind proc Local_Proc
--# expr-start 11702 '<And at nodes.lkt:16109:13>' And_Pred_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16109'
--# expr-start 11700 '<Bind at nodes.lkt:16109:13>' Bind_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16109'
--# expr-start 11699 '<FieldAccess for type_var at nodes.lkt:16109:17>' Fld_75 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16109'
      if Local_Proc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_75 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Local_Proc.Node);
--# expr-done 11699
Fld_75.Value := No_Entity;
Entity_Vars.Reset (Fld_75);
Cast_Expr_4 := No_Entity_Base_Type_Decl; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Bind_Result_5 := Solver.Create_Assign (Fld_75, Cast_Result_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16109:13") else null)); 
--# expr-done 11700
--# expr-start 11701 '<FieldAccess for sub_equation at nodes.lkt:16109:66>' Fld_76 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16109'
      if Local_Proc.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_76 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Local_Proc.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Local_Proc.Info);
--# end
--# expr-done 11701
And_Pred_12 := Create_And (Bind_Result_5, Fld_76, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16109:13") else null)); 
--# expr-done 11702
Let_Result_2 := And_Pred_12; Inc_Ref (Let_Result_2);
--# end
--# expr-done 11703
And_Pred_13 := Create_And (If_Result_15, Let_Result_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16096:45") else null)); 
--# expr-done 11704
   If_Result_22 := And_Pred_13;
else
   --# expr-start 11728 '<If at nodes.lkt:16043:9>' If_Result_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16043'
--# expr-start 11707 '<FieldAccess for name_is at nodes.lkt:16111:14>' Fld_92 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16111'
--# expr-start 11705 '<FieldAccess for id at nodes.lkt:16111:14>' Fld_91 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16111'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_91 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11705
      if Fld_91.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11706 '<Symbol Annotate at nodes.lkt:16111:30>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16111'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 11706
--# property-call-start Name.name_is
Fld_92 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_91.Node, Sym => Sym_7);
--# end
--# expr-done 11707
if Fld_92 then
   --# expr-start 11721 '<LogicAll at nodes.lkt:16117:13>' Logic_Boolean_Op_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16117'
--# expr-start 11720 '<Map at nodes.lkt:16117:13>' Map_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16117'
   --# expr-start 11708 '<FieldAccess for args at nodes.lkt:16117:13>' Fld_77 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16117'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_77 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11708
Fld_78 := Fld_77.Node;
      declare
         Map_Result_6_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_78 = null then
      Map_Result_6 := Create_Logic_Equation_Array (0);
      else
         Index_2 := 0;
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_78;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_10 := Bare_Item_1 = null; 
if Is_Null_10 then
   If_Result_19 := No_Entity_Base_Assoc;
else
Fld_79 := Fld_77.Info;
As_Entity_10 := (Info => Fld_79, Node => Bare_Item_1); 
   If_Result_19 := As_Entity_10;
end if;
                  Item_1 := If_Result_19; 
   --# scope-start
               --# bind arg Item_1
               --# bind i Index_2
      --# expr-start 11719 '<If at nodes.lkt:16118:29>' If_Result_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16118'
Comp_Result := Index_2 < 2; 
if Comp_Result then
   True_Rel_6 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16118:43") else null)); 
   If_Result_18 := True_Rel_6;
else
   --# expr-start 11718 '<If at nodes.lkt:16118:29>' If_Result_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16118'
--# expr-start 11709 '<Cast to PragmaArgumentAssoc at nodes.lkt:16119:22>' Cast_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
Cast_Expr_5 := Item_1; 
   if Cast_Expr_5.Node = null
      or else Cast_Expr_5.Node.Kind in Ada_Pragma_Argument_Assoc_Range
   then
      Cast_Result_13 := Create_Internal_Entity_Pragma_Argument_Assoc
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
   else
         Cast_Result_13 := No_Entity_Pragma_Argument_Assoc;
   end if;
--# expr-done 11709
Var_Expr_273 := Cast_Result_13;
if Var_Expr_273 /= No_Entity_Pragma_Argument_Assoc then
   --# scope-start
--# expr-start 11710 '<FieldAccess for name at nodes.lkt:16119:22>' Fld_84 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
      if Var_Expr_273.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_84 := Create_Internal_Entity_Name (Node => Var_Expr_273.Node.Pragma_Argument_Assoc_F_Name, Info => Var_Expr_273.Info);
--# expr-done 11710
Var_Expr_274 := Fld_84;
if Var_Expr_274 /= No_Entity_Name then
   --# scope-start
--# expr-start 11712 '<FieldAccess for name_is at nodes.lkt:16119:22>' Fld_85 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
      if Var_Expr_274.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11711 '<Symbol Entity at nodes.lkt:16119:65>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Entity); 
--# expr-done 11711
--# property-call-start Name.name_is
Fld_85 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_274.Node, Sym => Sym_6);
--# end
--# expr-done 11712
Scope_Result_6 := Fld_85;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := False;
end if;
Scope_Result_7 := Result_Var_6;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   --# expr-start 11715 '<FieldAccess for xref_no_overloading at nodes.lkt:16119:81>' Fld_81 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
--# expr-start 11714 '<Cast to Name at nodes.lkt:16119:81>' Cast_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
--# expr-start 11713 '<FieldAccess for assoc_expr at nodes.lkt:16119:81>' Fld_80 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16119'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_80 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 11713
   if Fld_80.Node = null
      or else Fld_80.Node.Kind in Ada_Name
   then
      Cast_Result_12 := Create_Internal_Entity_Name
        (Node => Fld_80.Node,
         Info => Fld_80.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11714
      if Cast_Result_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_81 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_12.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result_12.Info);
--# end
--# expr-done 11715
   If_Result_17 := Fld_81;
else
   --# expr-start 11717 '<FieldAccess for annotate_argument_equation at nodes.lkt:16120:22>' Fld_83 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16120'
--# expr-start 11716 '<FieldAccess for assoc_expr at nodes.lkt:16120:22>' Fld_82 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16120'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_82 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 11716
      if Fld_82.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.annotate_argument_equation
Fld_83 := Libadalang.Implementation.Expr_P_Annotate_Argument_Equation (Node => Fld_82.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_82.Info);
--# end
--# expr-done 11717
   If_Result_17 := Fld_83;
end if;
      Inc_Ref (If_Result_17);
--# expr-done 11718
   If_Result_18 := If_Result_17;
end if;
      Inc_Ref (If_Result_18);
--# expr-done 11719
         declare
            Item_To_Append : constant Logic_Equation := If_Result_18;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_6_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3502;
               Index_2 := Index_2 + 1;
         end loop;
      end;
      end if;
         Map_Result_6 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_6_Vec)));
         for I in Map_Result_6.Items'Range loop
            Map_Result_6.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_6_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_6.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_6_Vec);
      end;
--# expr-done 11720
Logic_Boolean_Op_5 := Solver.Create_All (Relation_Array (Map_Result_6.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16117:13") else null)); 
--# expr-done 11721
   If_Result_21 := Logic_Boolean_Op_5;
else
   --# expr-start 11727 '<LogicAll at nodes.lkt:16123:14>' Logic_Boolean_Op_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16123'
--# expr-start 11726 '<Map at nodes.lkt:16123:14>' Map_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16123'
   --# expr-start 11722 '<FieldAccess for args at nodes.lkt:16123:14>' Fld_86 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16123'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_86 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11722
Fld_87 := Fld_86.Node;
      declare
         Map_Result_7_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_87 = null then
      Map_Result_7 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_87;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_11 := Bare_Item_0 = null; 
if Is_Null_11 then
   If_Result_20 := No_Entity_Base_Assoc;
else
Fld_88 := Fld_86.Info;
As_Entity_11 := (Info => Fld_88, Node => Bare_Item_0); 
   If_Result_20 := As_Entity_11;
end if;
                  Item_0 := If_Result_20; 
   --# scope-start
               --# bind a Item_0
      --# expr-start 11725 '<Or at nodes.lkt:16124:20>' Or_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16124'
--# expr-start 11724 '<FieldAccess for sub_equation at nodes.lkt:16128:17>' Fld_90 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16128'
--# expr-start 11723 '<FieldAccess for assoc_expr at nodes.lkt:16128:17>' Fld_89 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16128'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_89 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 11723
      if Fld_89.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_90 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_89.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_89.Info);
--# end
--# expr-done 11724
True_Rel_7 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16129:18") else null)); 
Or_Pred := Create_Or (Fld_90, True_Rel_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16124:20") else null)); 
--# expr-done 11725
         declare
            Item_To_Append : constant Logic_Equation := Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_7_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3505;
         end loop;
      end;
      end if;
         Map_Result_7 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_7_Vec)));
         for I in Map_Result_7.Items'Range loop
            Map_Result_7.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_7_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_7.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_7_Vec);
      end;
--# expr-done 11726
Logic_Boolean_Op_6 := Solver.Create_All (Relation_Array (Map_Result_7.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16123:14") else null)); 
--# expr-done 11727
   If_Result_21 := Logic_Boolean_Op_6;
end if;
      Inc_Ref (If_Result_21);
--# expr-done 11728
   If_Result_22 := If_Result_21;
end if;
      Inc_Ref (If_Result_22);
--# expr-done 11729
   If_Result_23 := If_Result_22;
end if;
      Inc_Ref (If_Result_23);
--# expr-done 11730
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
--# expr-done 11731
   If_Result_33 := If_Result_24;
end if;
      Inc_Ref (If_Result_33);
--# expr-done 11732
   If_Result_36 := If_Result_33;
end if;
      Inc_Ref (If_Result_36);
--# expr-done 11733
   If_Result_37 := If_Result_36;
end if;
      Inc_Ref (If_Result_37);
--# expr-done 11734
   If_Result_47 := If_Result_37;
end if;
      Inc_Ref (If_Result_47);
--# expr-done 11735
   If_Result_48 := If_Result_47;
end if;
      Inc_Ref (If_Result_48);
--# expr-done 11736
   If_Result_52 := If_Result_48;
end if;
      Inc_Ref (If_Result_52);
--# expr-done 11737
         Property_Result := If_Result_52;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1368;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1368;
                     Finalizer_Scope_3489;
                     Finalizer_Scope_3490;
                     Finalizer_Scope_3491;
                     Finalizer_Scope_3492;
                     Finalizer_Scope_3493;
                     Finalizer_Scope_3494;
                     Finalizer_Scope_3495;
                     Finalizer_Scope_3499;
                     Finalizer_Scope_3500;
                     Finalizer_Scope_3502;
                     Finalizer_Scope_3505;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Xref_Equation;
--# end
--# property-start Pragma.value_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16140'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Value_Expr
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Var_Expr_275 : Internal_Entity_Base_Assoc;
Var_Expr_276 : Internal_Entity_Base_Assoc;
Fld : Internal_Entity_Base_Assoc_List;
Fld_1 : Bare_Base_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Base_Assoc_List;
Fld_5 : Bare_Base_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Assoc;
Fld_7 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Fld_12 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_13 : Boolean;
If_Result_3 : Internal_Entity_Expr;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_4 : Boolean;
Is_Equal_2 : Boolean;
If_Result_5 : Boolean;
Is_Equal_3 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Internal_Entity_Expr;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11754 '<If at nodes.lkt:16141:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
--# expr-start 11739 '<FieldAccess for name_symbol at nodes.lkt:16141:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
--# expr-start 11738 '<FieldAccess for id at nodes.lkt:16141:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11738
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 11739
--# expr-start 11740 '<Symbol Import at nodes.lkt:16141:37>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 11740
Is_Equal_3 := Fld_15 = Sym_1; 
if Is_Equal_3 then
   If_Result_6 := True;
else
--# expr-start 11741 '<Symbol Export at nodes.lkt:16141:49>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 11741
Is_Equal_2 := Fld_15 = Sym_2; 
if Is_Equal_2 then
   If_Result_5 := True;
else
--# expr-start 11742 '<Symbol Interface at nodes.lkt:16141:61>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 11742
Is_Equal_1 := Fld_15 = Sym_3; 
if Is_Equal_1 then
   If_Result_4 := True;
else
--# expr-start 11743 '<Symbol Convention at nodes.lkt:16141:76>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 11743
Is_Equal := Fld_15 = Sym_4; 
   If_Result_4 := Is_Equal;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
if If_Result_6 then
   --# expr-start 11745 '<FieldAccess for assoc_expr at nodes.lkt:16141:95>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
--# expr-start 11744 '<FieldAccess for args at nodes.lkt:16141:95>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11744
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11745
   If_Result_7 := Fld_3;
else
   --# expr-start 11753 '<If at nodes.lkt:16141:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16141'
--# expr-start 11748 '<FieldAccess for name_is at nodes.lkt:16145:14>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16145'
--# expr-start 11746 '<FieldAccess for id at nodes.lkt:16145:14>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16145'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11746
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11747 '<Symbol SPARK_Mode at nodes.lkt:16145:30>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16145'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Spark_Mode); 
--# expr-done 11747
--# property-call-start Name.name_is
Fld_13 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_12.Node, Sym => Sym);
--# end
--# expr-done 11748
if Fld_13 then
--# expr-start 11749 '<FieldAccess for args at nodes.lkt:16145:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16145'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11749
Fld_5 := Fld_4.Node;
Get_Result_1 := Get (Self, Fld_5, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_275 := If_Result_1;
if Var_Expr_275 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11750 '<FieldAccess for assoc_expr at nodes.lkt:16145:50>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16145'
      if Var_Expr_275.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_275.Node, E_Info => Var_Expr_275.Info);
--# end
--# expr-done 11750
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   If_Result_3 := Result_Var;
else
--# expr-start 11751 '<FieldAccess for args at nodes.lkt:16146:14>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16146'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11751
Fld_9 := Fld_8.Node;
Get_Result_2 := Get (Self, Fld_9, 1, True); 
Is_Null_2 := Get_Result_2 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity_2 := (Info => Fld_10, Node => Get_Result_2); 
   If_Result_2 := As_Entity_2;
end if;
Var_Expr_276 := If_Result_2;
if Var_Expr_276 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11752 '<FieldAccess for assoc_expr at nodes.lkt:16146:14>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16146'
      if Var_Expr_276.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_276.Node, E_Info => Var_Expr_276.Info);
--# end
--# expr-done 11752
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
   If_Result_3 := Result_Var_1;
end if;
--# expr-done 11753
   If_Result_7 := If_Result_3;
end if;
--# expr-done 11754
         Property_Result := If_Result_7;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Value_Expr;
--# end
--# property-start Pragma.associated_entity_names 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16148'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entity_Names
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name_Array_Access;
            procedure Finalizer_Scope_1370 with Inline_Always;
            procedure Finalizer_Scope_3510 with Inline_Always;
            procedure Finalizer_Scope_3512 with Inline_Always;
            procedure Finalizer_Scope_3516 with Inline_Always;
            procedure Finalizer_Scope_3517 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Base_Assoc;
Var_Expr_2 : Internal_Entity_Name;
Var_Expr_3 : Internal_Entity_Name;
Var_Expr_4 : Internal_Entity_Name;
Var_Expr_277 : Internal_Entity_Base_Assoc;
Var_Expr_278 : Internal_Entity_Base_Assoc;
Var_Expr_279 : Internal_Entity_Pragma_Argument_Assoc;
Var_Expr_280 : Internal_Entity_Name;
Fld : Internal_Entity_Base_Assoc_List;
Fld_1 : Bare_Base_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Fld_3 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Array_Lit : Internal_Entity_Name_Array_Access;
Fld_4 : Internal_Entity_Base_Assoc_List;
Fld_5 : Bare_Base_Assoc_List;
Fld_6 : Internal_Entity_Info;
Item_5 : Internal_Entity_Base_Assoc;
Bare_Item_5 : Bare_Base_Assoc;
Fld_7 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Assoc;
Map_Result : Internal_Entity_Name_Array_Access;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Array_Lit_1 : Internal_Entity_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Name_Array_Access;
Result_Var_1 : Internal_Entity_Name_Array_Access;
Fld_12 : Internal_Entity_Base_Assoc_List;
Fld_13 : Bare_Base_Assoc_List;
Fld_14 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_15 : Internal_Entity_Expr;
Scope_Result_2 : Internal_Entity_Expr;
Result_Var_2 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Array_Lit_2 : Internal_Entity_Name_Array_Access;
Is_A : Boolean;
Not_Val : Boolean;
If_Result_4 : Internal_Entity_Name_Array_Access;
Scope_Result_3 : Internal_Entity_Name_Array_Access;
Result_Var_3 : Internal_Entity_Name_Array_Access;
Fld_16 : Internal_Entity_Base_Assoc_List;
Fld_17 : Bare_Base_Assoc_List;
Fld_18 : Internal_Entity_Info;
Item_0 : Internal_Entity_Base_Assoc;
Bare_Item_0 : Bare_Base_Assoc;
Cast_Expr : Internal_Entity_Base_Assoc;
Cast_Result_4 : Internal_Entity_Pragma_Argument_Assoc;
Fld_19 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_20 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_5 : Internal_Entity_Base_Assoc;
Find_Result : Internal_Entity_Base_Assoc;
Cast_Expr_1 : Internal_Entity_Base_Assoc;
Cast_Result_5 : Internal_Entity_Pragma_Argument_Assoc;
Fld_21 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Array_Lit_3 : Internal_Entity_Name_Array_Access;
Scope_Result_6 : Internal_Entity_Name_Array_Access;
Result_Var_6 : Internal_Entity_Name_Array_Access;
Scope_Result_7 : Internal_Entity_Name_Array_Access;
Result_Var_7 : Internal_Entity_Name_Array_Access;
Fld_22 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_23 : Boolean;
If_Result_6 : Internal_Entity_Name_Array_Access;
Fld_24 : Internal_Entity_Identifier;
Sym_2 : Symbol_Type;
Fld_25 : Boolean;
If_Result_7 : Internal_Entity_Name_Array_Access;
Fld_26 : Internal_Entity_Identifier;
Fld_27 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_8 : Boolean;
Is_Equal_2 : Boolean;
If_Result_9 : Boolean;
Is_Equal_3 : Boolean;
If_Result_10 : Boolean;
Is_Equal_4 : Boolean;
If_Result_11 : Boolean;
Is_Equal_5 : Boolean;
If_Result_12 : Boolean;
Is_Equal_6 : Boolean;
If_Result_13 : Boolean;
Is_Equal_7 : Boolean;
If_Result_14 : Boolean;
Is_Equal_8 : Boolean;
If_Result_15 : Boolean;
Is_Equal_9 : Boolean;
If_Result_16 : Boolean;
Is_Equal_10 : Boolean;
If_Result_17 : Boolean;
Is_Equal_11 : Boolean;
If_Result_18 : Boolean;
Is_Equal_12 : Boolean;
If_Result_19 : Boolean;
Is_Equal_13 : Boolean;
If_Result_20 : Boolean;
Is_Equal_14 : Boolean;
If_Result_21 : Boolean;
Is_Equal_15 : Boolean;
If_Result_22 : Boolean;
Is_Equal_16 : Boolean;
If_Result_23 : Boolean;
If_Result_24 : Internal_Entity_Name_Array_Access;
Fld_28 : Internal_Entity_Identifier;
Sym_20 : Symbol_Type;
Fld_29 : Boolean;
If_Result_25 : Internal_Entity_Name_Array_Access;
Fld_30 : Internal_Entity_Identifier;
Fld_31 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Is_Equal_17 : Boolean;
Is_Equal_18 : Boolean;
If_Result_26 : Boolean;
Is_Equal_19 : Boolean;
If_Result_27 : Boolean;
Is_Equal_20 : Boolean;
If_Result_28 : Boolean;
If_Result_29 : Internal_Entity_Name_Array_Access;
            procedure Finalizer_Scope_1370 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (If_Result_29);
            end Finalizer_Scope_1370;
            procedure Finalizer_Scope_3510 is
            begin
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_3510;
            procedure Finalizer_Scope_3512 is
            begin
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (If_Result_4);
            end Finalizer_Scope_3512;
            procedure Finalizer_Scope_3516 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
            end Finalizer_Scope_3516;
            procedure Finalizer_Scope_3517 is
            begin
                     Dec_Ref (Array_Lit_3);
            end Finalizer_Scope_3517;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11822 '<If at nodes.lkt:16149:9>' If_Result_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11756 '<FieldAccess for name_symbol at nodes.lkt:16149:12>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11755 '<FieldAccess for id at nodes.lkt:16149:12>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_30 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11755
      if Fld_30.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 11756
--# expr-start 11757 '<Symbol Import at nodes.lkt:16149:37>' Sym_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Import); 
--# expr-done 11757
Is_Equal_20 := Fld_31 = Sym_21; 
if Is_Equal_20 then
   If_Result_28 := True;
else
--# expr-start 11758 '<Symbol Export at nodes.lkt:16149:49>' Sym_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Export); 
--# expr-done 11758
Is_Equal_19 := Fld_31 = Sym_22; 
if Is_Equal_19 then
   If_Result_27 := True;
else
--# expr-start 11759 '<Symbol Interface at nodes.lkt:16149:61>' Sym_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interface); 
--# expr-done 11759
Is_Equal_18 := Fld_31 = Sym_23; 
if Is_Equal_18 then
   If_Result_26 := True;
else
--# expr-start 11760 '<Symbol Convention at nodes.lkt:16149:76>' Sym_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Convention); 
--# expr-done 11760
Is_Equal_17 := Fld_31 = Sym_24; 
   If_Result_26 := Is_Equal_17;
end if;
   If_Result_27 := If_Result_26;
end if;
   If_Result_28 := If_Result_27;
end if;
if If_Result_28 then
   --# expr-start 11764 '<ArrayLiteral at nodes.lkt:16149:95>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11763 '<Cast to Name at nodes.lkt:16149:96>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11762 '<FieldAccess for assoc_expr at nodes.lkt:16149:96>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11761 '<FieldAccess for args at nodes.lkt:16149:96>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11761
Fld_1 := Fld.Node;
Get_Result := Get (Self, Fld_1, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11762
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11763
Array_Lit := Create_Internal_Entity_Name_Array (Internal_Internal_Entity_Name_Array'(1 => Cast_Result)); 
--# expr-done 11764
   If_Result_29 := Array_Lit;
else
   --# expr-start 11821 '<If at nodes.lkt:16149:9>' If_Result_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11767 '<FieldAccess for name_is at nodes.lkt:16150:14>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
--# expr-start 11765 '<FieldAccess for id at nodes.lkt:16150:14>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11765
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11766 '<Symbol Inline at nodes.lkt:16150:30>' Sym_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Inline); 
--# expr-done 11766
--# property-call-start Name.name_is
Fld_29 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_28.Node, Sym => Sym_20);
--# end
--# expr-done 11767
if Fld_29 then
   --# expr-start 11771 '<Map at nodes.lkt:16150:46>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
   --# expr-start 11768 '<FieldAccess for args at nodes.lkt:16150:46>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11768
Fld_5 := Fld_4.Node;
      declare
         Map_Result_Vec : Internal_Entity_Name_Vectors.Vector;
      begin
      if Fld_5 = null then
      Map_Result := Create_Internal_Entity_Name_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_5;
      begin
         for Untyped_Item_5 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_5 := Untyped_Item_5; 
Is_Null_1 := Bare_Item_5 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Assoc;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_5); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_5 := If_Result_1; 
   --# scope-start
               --# bind a Item_5
      --# expr-start 11770 '<Cast to Name at nodes.lkt:16150:67>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
--# expr-start 11769 '<FieldAccess for assoc_expr at nodes.lkt:16150:67>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16150'
      if Item_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_5.Node, E_Info => Item_5.Info);
--# end
--# expr-done 11769
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 11770
         declare
            Item_To_Append : constant Internal_Entity_Name := Cast_Result_1;
         begin
            Internal_Entity_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Name_Array
           (Items_Count => Natural (Internal_Entity_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11771
   If_Result_25 := Map_Result;
else
   --# expr-start 11820 '<If at nodes.lkt:16149:9>' If_Result_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11773 '<FieldAccess for name_symbol at nodes.lkt:16151:14>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
--# expr-start 11772 '<FieldAccess for id at nodes.lkt:16151:14>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_26 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11772
      if Fld_26.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_26.Node);
--# end
--# expr-done 11773
--# expr-start 11774 '<Symbol Pack at nodes.lkt:16151:39>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pack); 
--# expr-done 11774
Is_Equal_16 := Fld_27 = Sym_3; 
if Is_Equal_16 then
   If_Result_23 := True;
else
--# expr-start 11775 '<Symbol Pure at nodes.lkt:16151:49>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pure); 
--# expr-done 11775
Is_Equal_15 := Fld_27 = Sym_4; 
if Is_Equal_15 then
   If_Result_22 := True;
else
--# expr-start 11776 '<Symbol Preelaborate at nodes.lkt:16151:59>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Preelaborate); 
--# expr-done 11776
Is_Equal_14 := Fld_27 = Sym_5; 
if Is_Equal_14 then
   If_Result_21 := True;
else
--# expr-start 11777 '<Symbol Elaborate_Body at nodes.lkt:16151:77>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Elaborate_Body); 
--# expr-done 11777
Is_Equal_13 := Fld_27 = Sym_6; 
if Is_Equal_13 then
   If_Result_20 := True;
else
--# expr-start 11778 '<Symbol Volatile at nodes.lkt:16151:97>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile); 
--# expr-done 11778
Is_Equal_12 := Fld_27 = Sym_7; 
if Is_Equal_12 then
   If_Result_19 := True;
else
--# expr-start 11779 '<Symbol Volatile_Components at nodes.lkt:16151:111>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Volatile_Components); 
--# expr-done 11779
Is_Equal_11 := Fld_27 = Sym_8; 
if Is_Equal_11 then
   If_Result_18 := True;
else
--# expr-start 11780 '<Symbol Unchecked_Union at nodes.lkt:16151:136>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unchecked_Union); 
--# expr-done 11780
Is_Equal_10 := Fld_27 = Sym_9; 
if Is_Equal_10 then
   If_Result_17 := True;
else
--# expr-start 11781 '<Symbol Atomic at nodes.lkt:16151:157>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Atomic); 
--# expr-done 11781
Is_Equal_9 := Fld_27 = Sym_10; 
if Is_Equal_9 then
   If_Result_16 := True;
else
--# expr-start 11782 '<Symbol Atomic_Components at nodes.lkt:16151:169>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Atomic_Components); 
--# expr-done 11782
Is_Equal_8 := Fld_27 = Sym_11; 
if Is_Equal_8 then
   If_Result_15 := True;
else
--# expr-start 11783 '<Symbol No_Return at nodes.lkt:16151:192>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_No_Return); 
--# expr-done 11783
Is_Equal_7 := Fld_27 = Sym_12; 
if Is_Equal_7 then
   If_Result_14 := True;
else
--# expr-start 11784 '<Symbol Discard_Names at nodes.lkt:16151:207>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Discard_Names); 
--# expr-done 11784
Is_Equal_6 := Fld_27 = Sym_13; 
if Is_Equal_6 then
   If_Result_13 := True;
else
--# expr-start 11785 '<Symbol Independent at nodes.lkt:16151:226>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Independent); 
--# expr-done 11785
Is_Equal_5 := Fld_27 = Sym_14; 
if Is_Equal_5 then
   If_Result_12 := True;
else
--# expr-start 11786 '<Symbol Independent_Components at nodes.lkt:16151:243>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Independent_Components); 
--# expr-done 11786
Is_Equal_4 := Fld_27 = Sym_15; 
if Is_Equal_4 then
   If_Result_11 := True;
else
--# expr-start 11787 '<Symbol Asynchronous at nodes.lkt:16151:271>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asynchronous); 
--# expr-done 11787
Is_Equal_3 := Fld_27 = Sym_16; 
if Is_Equal_3 then
   If_Result_10 := True;
else
--# expr-start 11788 '<Symbol Interrupt_Handler at nodes.lkt:16151:289>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Interrupt_Handler); 
--# expr-done 11788
Is_Equal_2 := Fld_27 = Sym_17; 
if Is_Equal_2 then
   If_Result_9 := True;
else
--# expr-start 11789 '<Symbol Attach_Handler at nodes.lkt:16151:312>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Attach_Handler); 
--# expr-done 11789
Is_Equal_1 := Fld_27 = Sym_18; 
if Is_Equal_1 then
   If_Result_8 := True;
else
--# expr-start 11790 '<Symbol Predicate at nodes.lkt:16151:332>' Sym_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 11790
Is_Equal := Fld_27 = Sym_19; 
   If_Result_8 := Is_Equal;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
end if;
   If_Result_12 := If_Result_11;
end if;
   If_Result_13 := If_Result_12;
end if;
   If_Result_14 := If_Result_13;
end if;
   If_Result_15 := If_Result_14;
end if;
   If_Result_16 := If_Result_15;
end if;
   If_Result_17 := If_Result_16;
end if;
   If_Result_18 := If_Result_17;
end if;
   If_Result_19 := If_Result_18;
end if;
   If_Result_20 := If_Result_19;
end if;
   If_Result_21 := If_Result_20;
end if;
   If_Result_22 := If_Result_21;
end if;
   If_Result_23 := If_Result_22;
end if;
if If_Result_23 then
--# expr-start 11793 '<Cast to Name at nodes.lkt:16151:350>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
--# expr-start 11791 '<FieldAccess for args at nodes.lkt:16151:350>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11791
Fld_9 := Fld_8.Node;
Get_Result_1 := Get (Self, Fld_9, 0, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity_2 := (Info => Fld_10, Node => Get_Result_1); 
   If_Result_2 := As_Entity_2;
end if;
Var_Expr_277 := If_Result_2;
if Var_Expr_277 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11792 '<FieldAccess for assoc_expr at nodes.lkt:16151:350>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
      if Var_Expr_277.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_277.Node, E_Info => Var_Expr_277.Info);
--# end
--# expr-done 11792
Scope_Result := Fld_11;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 11793
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Name then
   --# scope-start
   --# bind v1 Var_Expr_4
--# expr-start 11794 '<ArrayLiteral at nodes.lkt:16151:398>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16151'
Array_Lit_1 := Create_Internal_Entity_Name_Array (Internal_Internal_Entity_Name_Array'(1 => Var_Expr_4)); 
--# expr-done 11794
Scope_Result_1 := Array_Lit_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3510;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_24 := Result_Var_1;
else
   --# expr-start 11819 '<If at nodes.lkt:16149:9>' If_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11797 '<FieldAccess for name_is at nodes.lkt:16152:14>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
--# expr-start 11795 '<FieldAccess for id at nodes.lkt:16152:14>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11795
      if Fld_24.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11796 '<Symbol Obsolescent at nodes.lkt:16152:30>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Obsolescent); 
--# expr-done 11796
--# property-call-start Name.name_is
Fld_25 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_24.Node, Sym => Sym_2);
--# end
--# expr-done 11797
if Fld_25 then
--# expr-start 11800 '<Cast to Name at nodes.lkt:16152:51>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
--# expr-start 11798 '<FieldAccess for args at nodes.lkt:16152:51>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11798
Fld_13 := Fld_12.Node;
Get_Result_2 := Get (Self, Fld_13, 0, True); 
Is_Null_3 := Get_Result_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_14 := Fld_12.Info;
As_Entity_3 := (Info => Fld_14, Node => Get_Result_2); 
   If_Result_3 := As_Entity_3;
end if;
Var_Expr_278 := If_Result_3;
if Var_Expr_278 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 11799 '<FieldAccess for assoc_expr at nodes.lkt:16152:51>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16152'
      if Var_Expr_278.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_278.Node, E_Info => Var_Expr_278.Info);
--# end
--# expr-done 11799
Scope_Result_2 := Fld_15;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Expr;
end if;
   if Result_Var_2.Node = null
      or else Result_Var_2.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
   else
         Cast_Result_3 := No_Entity_Name;
   end if;
--# expr-done 11800
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_3
--# expr-start 11804 '<If at nodes.lkt:16156:23>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16156'
--# expr-start 11802 '<Not at nodes.lkt:16156:26>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16156'
--# expr-start 11801 '<IsA StringLiteral at nodes.lkt:16156:30>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16156'
Is_A := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_String_Literal_Range; 
--# expr-done 11801
Not_Val := not (Is_A); 
--# expr-done 11802
if Not_Val then
   --# expr-start 11803 '<ArrayLiteral at nodes.lkt:16156:57>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16156'
Array_Lit_2 := Create_Internal_Entity_Name_Array (Internal_Internal_Entity_Name_Array'(1 => Var_Expr_3)); 
--# expr-done 11803
   If_Result_4 := Array_Lit_2;
else
   If_Result_4 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 11804
Scope_Result_3 := If_Result_4;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3512;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
   If_Result_7 := Result_Var_3;
else
   --# expr-start 11818 '<If at nodes.lkt:16149:9>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16149'
--# expr-start 11807 '<FieldAccess for name_is at nodes.lkt:16158:14>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16158'
--# expr-start 11805 '<FieldAccess for id at nodes.lkt:16158:14>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16158'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11805
      if Fld_22.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11806 '<Symbol Annotate at nodes.lkt:16158:30>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16158'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Annotate); 
--# expr-done 11806
--# property-call-start Name.name_is
Fld_23 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_22.Node, Sym => Sym_1);
--# end
--# expr-done 11807
if Fld_23 then
--# expr-start 11813 '<Find at nodes.lkt:16161:13>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16161'
   --# expr-start 11808 '<FieldAccess for args at nodes.lkt:16161:13>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16161'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 11808
Fld_17 := Fld_16.Node;
      Find_Result := No_Entity_Base_Assoc;
      if Fld_17 = null then
      Find_Result := No_Entity_Base_Assoc;
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_17;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_4 := Bare_Item_0 = null; 
if Is_Null_4 then
   If_Result_5 := No_Entity_Base_Assoc;
else
Fld_18 := Fld_16.Info;
As_Entity_4 := (Info => Fld_18, Node => Bare_Item_0); 
   If_Result_5 := As_Entity_4;
end if;
                  Item_0 := If_Result_5; 
   --# scope-start
               --# bind a Item_0
--# expr-start 11809 '<Cast to PragmaArgumentAssoc at nodes.lkt:16162:24>' Cast_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16162'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Argument_Assoc_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Pragma_Argument_Assoc
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_4 := No_Entity_Pragma_Argument_Assoc;
   end if;
--# expr-done 11809
Var_Expr_279 := Cast_Result_4;
if Var_Expr_279 /= No_Entity_Pragma_Argument_Assoc then
   --# scope-start
--# expr-start 11810 '<FieldAccess for name at nodes.lkt:16162:24>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16162'
      if Var_Expr_279.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Var_Expr_279.Node.Pragma_Argument_Assoc_F_Name, Info => Var_Expr_279.Info);
--# expr-done 11810
Var_Expr_280 := Fld_19;
if Var_Expr_280 /= No_Entity_Name then
   --# scope-start
--# expr-start 11812 '<FieldAccess for name_is at nodes.lkt:16162:24>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16162'
      if Var_Expr_280.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11811 '<Symbol Entity at nodes.lkt:16162:65>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16162'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Entity); 
--# expr-done 11811
--# property-call-start Name.name_is
Fld_20 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_280.Node, Sym => Sym);
--# end
--# expr-done 11812
Scope_Result_4 := Fld_20;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
Scope_Result_5 := Result_Var_4;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
      if Result_Var_5 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var_5;
         end loop;
      end;
      end if;
--# expr-done 11813
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Entity_Base_Assoc then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 11816 '<Cast to Name at nodes.lkt:16164:24>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16164'
--# expr-start 11815 '<FieldAccess for expr at nodes.lkt:16164:24>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16164'
--# expr-start 11814 '<Cast to PragmaArgumentAssoc at nodes.lkt:16164:24>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16164'
Cast_Expr_1 := Var_Expr_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Pragma_Argument_Assoc_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Pragma_Argument_Assoc
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_5 := No_Entity_Pragma_Argument_Assoc;
   end if;
--# expr-done 11814
      if Cast_Result_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_21 := Create_Internal_Entity_Expr (Node => Cast_Result_5.Node.Pragma_Argument_Assoc_F_Expr, Info => Cast_Result_5.Info);
--# expr-done 11815
   if Fld_21.Node = null
      or else Fld_21.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 11816
Var_Expr_2 := Cast_Result_6;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind v2 Var_Expr_2
--# expr-start 11817 '<ArrayLiteral at nodes.lkt:16164:75>' Array_Lit_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16164'
Array_Lit_3 := Create_Internal_Entity_Name_Array (Internal_Internal_Entity_Name_Array'(1 => Var_Expr_2)); 
--# expr-done 11817
Scope_Result_6 := Array_Lit_3;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3517;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Scope_Result_7 := Result_Var_6;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3516;
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (Result_Var_7);
   If_Result_6 := Result_Var_7;
else
   If_Result_6 := No_Internal_Entity_Name_Array_Type;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 11818
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 11819
   If_Result_24 := If_Result_7;
end if;
      Inc_Ref (If_Result_24);
--# expr-done 11820
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
--# expr-done 11821
   If_Result_29 := If_Result_25;
end if;
      Inc_Ref (If_Result_29);
--# expr-done 11822
         Property_Result := If_Result_29;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1370;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1370;
                     Finalizer_Scope_3510;
                     Finalizer_Scope_3512;
                     Finalizer_Scope_3516;
                     Finalizer_Scope_3517;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entity_Names;
--# end
--# property-start Pragma.associated_entities_helper 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16169'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entities_Helper
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1371 with Inline_Always;
            procedure Finalizer_Scope_3518 with Inline_Always;
            procedure Finalizer_Scope_3519 with Inline_Always;
            procedure Finalizer_Scope_3520 with Inline_Always;
      Var_Expr_1 : Internal_Entity;
Fld : Internal_Entity_Name_Array_Access;
Fld_1 : Internal_Entity;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name_Array_Access;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
Find_Result : Internal_Entity_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_8 : Internal_Unit;
Fld_9 : Internal_Unit;
Is_Equal : Boolean;
Fld_10 : Bare_Defining_Name;
Node_Comp : Boolean;
If_Result : Boolean;
Map_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Map_Result_2 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1371 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1371;
            procedure Finalizer_Scope_3518 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_3518;
            procedure Finalizer_Scope_3519 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3519;
            procedure Finalizer_Scope_3520 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_3520;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11841 '<Mapcat at nodes.lkt:16169:69>' Map_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16169'
   --# expr-start 11823 '<FieldAccess for associated_entity_names at nodes.lkt:16169:69>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16169'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11823
      declare
         Map_Result_2_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Name_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind name Item_0
      --# expr-start 11840 '<Map at nodes.lkt:16170:19>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16170'
--# expr-start 11824 '<FieldAccess for semantic_parent at nodes.lkt:16170:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16170'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11824
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind parent Var_Expr_1
--# expr-start 11834 '<Map at nodes.lkt:16171:25>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16171'
   --# expr-start 11827 '<EnvGet at nodes.lkt:16171:25>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16171'
--# expr-start 11825 '<FieldAccess for children_env at nodes.lkt:16171:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16171'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 11825
--# expr-start 11826 '<FieldAccess for name_symbol at nodes.lkt:16172:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16172'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_0.Node);
--# end
--# expr-done 11826
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_2, Key => Thin (Fld_3), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 11827
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_2
      --# expr-start 11833 '<Find at nodes.lkt:16174:27>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16174'
   --# expr-start 11830 '<FieldAccess for defining_names at nodes.lkt:16174:27>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16174'
--# expr-start 11829 '<FieldAccess for wrap_public_reference at nodes.lkt:16174:27>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16174'
--# expr-start 11828 '<Cast to BasicDecl at nodes.lkt:16174:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16174'
Cast_Expr := Item_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 11828
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 11829
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 11830
      Find_Result := No_Entity_Defining_Name;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_5;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_3
      --# expr-start 11832 '<FieldAccess for name_is at nodes.lkt:16181:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16181'
      if Item_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11831 '<FieldAccess for name_symbol at nodes.lkt:16181:32>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16181'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_0.Node);
--# end
--# expr-done 11831
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_3.Node, Sym => Fld_6);
--# end
--# expr-done 11832
      if Fld_7 then
         Find_Result := Item_3; 
      end if;
   --# end
               exit when Fld_7;
         end loop;
      end;
--# expr-done 11833
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Find_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3520;
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11834
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3519;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
      declare
         Map_Result_1_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_4
--# expr-start 11837 '<Eq at nodes.lkt:16187:17>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16187'
--# expr-start 11835 '<FieldAccess for unit at nodes.lkt:16187:17>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16187'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Unit (Node => Item_4.Node);
--# expr-done 11835
--# expr-start 11836 '<FieldAccess for unit at nodes.lkt:16187:31>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16187'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 11836
Is_Equal := Fld_8 = Fld_9; 
--# expr-done 11837
if Is_Equal then
   --# expr-start 11839 '<OrderingTest '"'"'lt'"'"' at nodes.lkt:16188:19>' Node_Comp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16188'
--# expr-start 11838 '<FieldAccess for node at nodes.lkt:16188:19>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16188'
Fld_10 := Item_4.Node;
--# expr-done 11838
Node_Comp := Compare (Self, Fld_10, Self, Less_Than); 
--# expr-done 11839
   If_Result := Node_Comp;
else
   If_Result := False;
end if;
         if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_4;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11840
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_3518;
         end loop;
      end;
         Map_Result_2 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_2_Vec)));
         for I in Map_Result_2.Items'Range loop
            Map_Result_2.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_2_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_2.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_2_Vec);
      end;
--# expr-done 11841
         Property_Result := Map_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1371;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1371;
                     Finalizer_Scope_3518;
                     Finalizer_Scope_3519;
                     Finalizer_Scope_3520;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entities_Helper;
--# end
--# property-start Pragma.associated_entities 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16195'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entities
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1372 with Inline_Always;
            procedure Finalizer_Scope_3523 with Inline_Always;
            procedure Finalizer_Scope_3525 with Inline_Always;
            procedure Finalizer_Scope_3526 with Inline_Always;
            procedure Finalizer_Scope_3527 with Inline_Always;
            procedure Finalizer_Scope_3528 with Inline_Always;
            procedure Finalizer_Scope_3530 with Inline_Always;
            procedure Finalizer_Scope_3531 with Inline_Always;
            procedure Finalizer_Scope_3532 with Inline_Always;
      Local_Top_Level_Decl : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_0 : Bare_Compilation_Unit;
Local_Enclosing_Program_Unit : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Name_Array_Access;
Local_P : Internal_Entity_Defining_Name_Array_Access;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Bare_Ada_Node_List;
Var_Expr_6 : Bare_Ada_Node_Array_Access;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_1 : Internal_Entity_Defining_Name_Array_Access;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Defining_Name;
Array_Lit : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Name_Array_Access;
Fld_6 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_1 : Internal_Entity_Defining_Name_Array_Access;
Fld_7 : Internal_Entity_Defining_Name;
Array_Lit_1 : Internal_Entity_Defining_Name_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Internal_Entity_Name;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_2 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_2 : Internal_Entity_Defining_Name_Array_Access;
Is_Equal_1 : Boolean;
Not_Val : Boolean;
If_Result_4 : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
Scope_Result_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_9 : Bare_Ada_Node;
Cast_Result_5 : Bare_Ada_Node_List;
Item_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Node_Comp : Boolean;
If_Result_5 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Len_1 : Integer;
Arith_Result : Integer;
Get_Result_1 : Bare_Ada_Node;
Scope_Result_4 : Bare_Ada_Node;
Result_Var_3 : Bare_Ada_Node;
Cast_Result_6 : Bare_Basic_Decl;
As_Entity_2 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
If_Result_6 : Internal_Entity_Basic_Decl;
Array_Lit_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit_3 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_6 : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit_4 : Internal_Entity_Basic_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_14 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_15 : Internal_Entity_Defining_Name;
Map_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Result_Var_7 : Internal_Entity_Defining_Name_Array_Access;
Let_Result_1 : Internal_Entity_Defining_Name_Array_Access;
Let_Result_2 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1372 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Local_Top_Level_Decl);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Array_Lit_3);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Array_Lit_4);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1372;
            procedure Finalizer_Scope_3523 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3523;
            procedure Finalizer_Scope_3525 is
            begin
                     Dec_Ref (Var_Expr_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Local_P);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_3525;
            procedure Finalizer_Scope_3526 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_3526;
            procedure Finalizer_Scope_3527 is
            begin
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_3527;
            procedure Finalizer_Scope_3528 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_4);
            end Finalizer_Scope_3528;
            procedure Finalizer_Scope_3530 is
            begin
                     Dec_Ref (Var_Expr_6);
            end Finalizer_Scope_3530;
            procedure Finalizer_Scope_3531 is
            begin
                     Dec_Ref (Array_Lit_2);
            end Finalizer_Scope_3531;
            procedure Finalizer_Scope_3532 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_3532;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11893 '<Let at nodes.lkt:16196:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16196'
--# scope-start
--# expr-start 11844 '<Cast to CompilationUnit at nodes.lkt:16196:30>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16196'
--# expr-start 11843 '<parent at nodes.lkt:16196:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16196'
--# expr-start 11842 '<parent at nodes.lkt:16196:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16196'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11842
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 11843
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 11844
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind cu Var_Expr_0
--# expr-start 11850 '<ArrayLiteral at nodes.lkt:16197:21>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
--# expr-start 11849 '<FieldAccess for defining_name at nodes.lkt:16197:22>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
--# expr-start 11848 '<as_entity at nodes.lkt:16197:22>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
--# expr-start 11847 '<FieldAccess for item at nodes.lkt:16197:22>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
--# expr-start 11846 '<Cast to LibraryItem at nodes.lkt:16197:22>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
--# expr-start 11845 '<FieldAccess for body at nodes.lkt:16197:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16197'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Var_Expr_0.Compilation_Unit_F_Body;
--# expr-done 11845
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 11846
      if Cast_Result_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 11847
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 11848
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11849
Array_Lit := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_4)); 
--# expr-done 11850
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3523;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
Local_Top_Level_Decl := Result_Var; Inc_Ref (Local_Top_Level_Decl);
--# bind top_level_decl Local_Top_Level_Decl
--# expr-start 11892 '<Let at nodes.lkt:16199:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
--# scope-start
--# expr-start 11855 '<as_entity at nodes.lkt:16199:38>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
--# expr-start 11854 '<Cast to BasicDecl at nodes.lkt:16199:38>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
--# expr-start 11853 '<Find at nodes.lkt:16199:38>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
   --# expr-start 11851 '<FieldAccess for parents at nodes.lkt:16199:38>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, False); 
--# expr-done 11851
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
      --# expr-start 11852 '<IsA BasicDecl at nodes.lkt:16199:80>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16199'
Is_A := Item_1 /= null 
and then Item_1.Kind in Ada_Basic_Decl; 
--# expr-done 11852
      if Is_A then
         Find_Result := Item_1; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 11853
   if Find_Result = null
      or else Find_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Find_Result;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 11854
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 11855
Local_Enclosing_Program_Unit := If_Result_1; 
--# bind enclosing_program_unit Local_Enclosing_Program_Unit
--# expr-start 11856 '<FieldAccess for associated_entity_names at nodes.lkt:16211:9>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16211'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entity_names
Fld_5 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11856
Var_Expr_2 := Fld_5;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Name_Array_Type then
   --# scope-start
   --# bind names Var_Expr_2
--# expr-start 11869 '<Let at nodes.lkt:16213:17>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16213'
--# scope-start
--# expr-start 11857 '<FieldAccess for associated_entities_helper at nodes.lkt:16213:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16213'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities_helper
Fld_6 := Libadalang.Implementation.Pragma_Node_P_Associated_Entities_Helper (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11857
Var_Expr_1 := Fld_6;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Defining_Name_Array_Type then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3526;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Local_Top_Level_Decl;
end if;
      Inc_Ref (Result_Var_1);
Local_P := Result_Var_1; Inc_Ref (Local_P);
--# bind p Local_P
--# expr-start 11868 '<If at nodes.lkt:16215:17>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16215'
--# expr-start 11859 '<Not at nodes.lkt:16215:20>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16215'
--# expr-start 11858 '<Eq at nodes.lkt:16215:20>' Is_Equal_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16215'
Is_Equal_1 := Equivalent (Local_P, No_Internal_Entity_Defining_Name_Array_Type); 
--# expr-done 11858
Not_Val := not (Is_Equal_1); 
--# expr-done 11859
if Not_Val then
   If_Result_4 := Local_P;
else
Var_Expr_3 := Local_Enclosing_Program_Unit;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind epu Var_Expr_3
--# expr-start 11867 '<If at nodes.lkt:16216:30>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
--# expr-start 11861 '<Eq at nodes.lkt:16216:33>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
--# expr-start 11860 '<length at nodes.lkt:16216:33>' Len 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
Len := Length (Var_Expr_2); 
--# expr-done 11860
Is_Equal := Len = 1; 
--# expr-done 11861
if Is_Equal then
   --# expr-start 11864 '<Eq at nodes.lkt:16216:57>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
--# expr-start 11863 '<FieldAccess for referenced_decl at nodes.lkt:16216:57>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
--# expr-start 11862 '<at at nodes.lkt:16216:57>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
Get_Result := Get (Self, Var_Expr_2, 0, True); 
--# expr-done 11862
      if Get_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Get_Result.Node, Imprecise_Fallback => False, E_Info => Get_Result.Info);
--# end
--# expr-done 11863
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr := Var_Expr_3; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 11864
   If_Result_2 := Is_Equiv;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 11866 '<ArrayLiteral at nodes.lkt:16216:97>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
--# expr-start 11865 '<FieldAccess for defining_name at nodes.lkt:16216:98>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16216'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 11865
Array_Lit_1 := Create_Internal_Entity_Defining_Name_Array (Internal_Internal_Entity_Defining_Name_Array'(1 => Fld_7)); 
--# expr-done 11866
   If_Result_3 := Array_Lit_1;
else
   If_Result_3 := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 11867
Scope_Result_2 := If_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3527;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Local_Top_Level_Decl;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_4 := Result_Var_2;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 11868
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 11869
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_3525;
   Result_Var_7 := Scope_Result_3;
else
   --# expr-start 11891 '<Map at nodes.lkt:16218:28>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16218'
   --# expr-start 11889 '<If at nodes.lkt:16220:17>' If_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16220'
--# expr-start 11872 '<FieldAccess for is_contract_aspect at nodes.lkt:16222:21>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16222'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11871 '<FieldAccess for name_symbol at nodes.lkt:16222:45>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16222'
--# expr-start 11870 '<FieldAccess for id at nodes.lkt:16222:45>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16222'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11870
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_10.Node);
--# end
--# expr-done 11871
--# property-call-start AdaNode.is_contract_aspect
Fld_12 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Fld_11);
--# end
--# expr-done 11872
if Fld_12 then
   If_Result_7 := True;
else
   --# expr-start 11875 '<FieldAccess for name_is at nodes.lkt:16225:21>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16225'
--# expr-start 11873 '<FieldAccess for id at nodes.lkt:16225:21>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16225'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 11873
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11874 '<Symbol Obsolescent at nodes.lkt:16225:37>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16225'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Obsolescent); 
--# expr-done 11874
--# property-call-start Name.name_is
Fld_14 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_13.Node, Sym => Sym);
--# end
--# expr-done 11875
   If_Result_7 := Fld_14;
end if;
if If_Result_7 then
--# expr-start 11877 '<Cast to AdaNode.list at nodes.lkt:16235:21>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16235'
--# expr-start 11876 '<parent at nodes.lkt:16235:21>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16235'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Parent; 
--# expr-done 11876
   if Fld_9 = null
      or else Fld_9.Kind in Ada_Ada_Node_List_Range
   then
      Cast_Result_5 := Fld_9;
   else
         Cast_Result_5 := No_Bare_Ada_Node;
   end if;
--# expr-done 11877
Var_Expr_4 := Cast_Result_5;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind decls Var_Expr_4
--# expr-start 11885 '<as_entity at nodes.lkt:16236:36>' If_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16236'
--# expr-start 11884 '<Cast to BasicDecl at nodes.lkt:16236:36>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16236'
--# expr-start 11880 '<Map at nodes.lkt:16236:36>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16236'
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Var_Expr_4 = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Var_Expr_4;
      begin
         for Untyped_Item_5 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_5 := Untyped_Item_5; 
   --# scope-start
               --# bind decl Item_5
--# expr-start 11878 '<IsA BasicDecl at nodes.lkt:16237:39>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16237'
Is_A_1 := Item_5 /= null 
and then Item_5.Kind in Ada_Basic_Decl; 
--# expr-done 11878
if Is_A_1 then
   --# expr-start 11879 '<OrderingTest '"'"'gt'"'"' at nodes.lkt:16237:61>' Node_Comp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16237'
Node_Comp := Compare (Self, Self, Item_5, Greater_Than); 
--# expr-done 11879
   If_Result_5 := Node_Comp;
else
   If_Result_5 := False;
end if;
         if If_Result_5 then
         declare
            Item_To_Append : constant Bare_Ada_Node := Item_5;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 11880
Var_Expr_6 := Map_Result;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Bare_Ada_Node_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_6
--# expr-start 11883 '<at at nodes.lkt:16238:41>' Get_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16238'
--# expr-start 11882 '<Arithmetic - at nodes.lkt:16238:48>' Arith_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16238'
--# expr-start 11881 '<length at nodes.lkt:16238:48>' Len_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16238'
Len_1 := Length (Var_Expr_6); 
--# expr-done 11881
Arith_Result := (Len_1 - 1); 
--# expr-done 11882
Get_Result_1 := Get (Self, Var_Expr_6, Arith_Result, True); 
--# expr-done 11883
Scope_Result_4 := Get_Result_1;
   --# end
      Finalizer_Scope_3530;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Bare_Ada_Node;
end if;
   if Result_Var_3 = null
      or else Result_Var_3.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Result_Var_3;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 11884
Is_Null_2 := Cast_Result_6 = null; 
if Is_Null_2 then
   If_Result_6 := No_Entity_Basic_Decl;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_6); 
   If_Result_6 := As_Entity_2;
end if;
--# expr-done 11885
Var_Expr_7 := If_Result_6;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind v1 Var_Expr_7
--# expr-start 11886 '<ArrayLiteral at nodes.lkt:16238:104>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16238'
Array_Lit_2 := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Var_Expr_7)); 
--# expr-done 11886
Scope_Result_5 := Array_Lit_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_3531;
   Result_Var_4 := Scope_Result_5;
else
   Result_Var_4 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Scope_Result_6 := Result_Var_4;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3528;
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Var_Expr := Result_Var_5;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_7 := Var_Expr;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3532;
   Result_Var_6 := Scope_Result_7;
else
   --# expr-start 11887 '<ArrayLiteral at nodes.lkt:16239:27>' Array_Lit_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16239'
Array_Lit_3 := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Local_Enclosing_Program_Unit)); 
--# expr-done 11887
   Result_Var_6 := Array_Lit_3;
end if;
      Inc_Ref (Result_Var_6);
   If_Result_8 := Result_Var_6;
else
   --# expr-start 11888 '<ArrayLiteral at nodes.lkt:16243:21>' Array_Lit_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16243'
Array_Lit_4 := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Local_Enclosing_Program_Unit)); 
--# expr-done 11888
   If_Result_8 := Array_Lit_4;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 11889
      declare
         Map_Result_1_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := If_Result_8;
      begin
         for Item_8 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_8
      --# expr-start 11890 '<FieldAccess for defining_name at nodes.lkt:16245:27>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16245'
      if Item_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_8.Node, E_Info => Item_8.Info);
--# end
--# expr-done 11890
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Fld_15;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result_1 := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_1_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 11891
   Result_Var_7 := Map_Result_1;
end if;
      Inc_Ref (Result_Var_7);
Let_Result_1 := Result_Var_7; Inc_Ref (Let_Result_1);
--# end
--# expr-done 11892
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 11893
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1372;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1372;
                     Finalizer_Scope_3523;
                     Finalizer_Scope_3525;
                     Finalizer_Scope_3526;
                     Finalizer_Scope_3527;
                     Finalizer_Scope_3528;
                     Finalizer_Scope_3530;
                     Finalizer_Scope_3531;
                     Finalizer_Scope_3532;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Associated_Entities;
--# end
--# property-start Pragma.initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16254'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Initial_Env
  (Node : Bare_Pragma_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1373 with Inline_Always;
            procedure Finalizer_Scope_3534 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Defining_Name;
Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Scope_Result : Internal_Designated_Env := No_Designated_Env;
New_Struct_2 : Internal_Designated_Env := No_Designated_Env;
Result_Var : Internal_Designated_Env := No_Designated_Env;
New_Struct_3 : Internal_Designated_Env := No_Designated_Env;
As_Entity_1 : Internal_Entity_Pragma_Node;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Pragma_Node;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Is_Equal_2 : Boolean;
If_Result_3 : Boolean;
Is_Equal_3 : Boolean;
If_Result_4 : Boolean;
Is_Equal_4 : Boolean;
If_Result_5 : Boolean;
Is_Equal_5 : Boolean;
If_Result_6 : Boolean;
Is_Equal_6 : Boolean;
If_Result_7 : Boolean;
Is_Equal_7 : Boolean;
If_Result_8 : Boolean;
Is_Equal_8 : Boolean;
If_Result_9 : Boolean;
Is_Equal_9 : Boolean;
If_Result_10 : Boolean;
Is_Equal_10 : Boolean;
If_Result_11 : Boolean;
If_Result_12 : Internal_Designated_Env := No_Designated_Env;
Fld_4 : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_13 : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1373 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (New_Struct_3);
                     Dec_Ref (If_Result_12);
                     Dec_Ref (If_Result_13);
            end Finalizer_Scope_1373;
            procedure Finalizer_Scope_3534 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_3534;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11921 '<If at nodes.lkt:16255:9>' If_Result_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
--# expr-start 11896 '<IsA CompilationUnit at nodes.lkt:16255:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
--# expr-start 11895 '<parent at nodes.lkt:16255:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
--# expr-start 11894 '<parent at nodes.lkt:16255:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 11894
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Fld_4.Parent; 
--# expr-done 11895
Is_A := Fld_5 /= null 
and then Fld_5.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 11896
if Is_A then
   --# expr-start 11898 '<New DesignatedEnv at nodes.lkt:16255:55>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
--# expr-start 11897 '<Symbol Standard at nodes.lkt:16256:56>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16256'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 11897
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 11898
   If_Result_13 := New_Struct;
else
   --# expr-start 11920 '<If at nodes.lkt:16255:9>' If_Result_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16255'
--# expr-start 11901 '<FieldAccess for name_symbol at nodes.lkt:16258:14>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
--# expr-start 11900 '<FieldAccess for id at nodes.lkt:16258:14>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
--# expr-start 11899 '<as_bare_entity at nodes.lkt:16258:14>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Is_Null_1 := Self = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Pragma_Node;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Self); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 11899
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => If_Result_1.Node.Pragma_Node_F_Id, Info => If_Result_1.Info);
--# expr-done 11900
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 11901
--# expr-start 11902 '<Symbol Pre at nodes.lkt:16258:54>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre); 
--# expr-done 11902
Is_Equal_10 := Fld_3 = Sym_1; 
if Is_Equal_10 then
   If_Result_11 := True;
else
--# expr-start 11903 '<Symbol Post at nodes.lkt:16258:63>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post); 
--# expr-done 11903
Is_Equal_9 := Fld_3 = Sym_2; 
if Is_Equal_9 then
   If_Result_10 := True;
else
--# expr-start 11904 '<Symbol Pre'"'"'Class at nodes.lkt:16258:73>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pre_Class); 
--# expr-done 11904
Is_Equal_8 := Fld_3 = Sym_3; 
if Is_Equal_8 then
   If_Result_9 := True;
else
--# expr-start 11905 '<Symbol Post'"'"'Class at nodes.lkt:16258:88>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Post_Class); 
--# expr-done 11905
Is_Equal_7 := Fld_3 = Sym_4; 
if Is_Equal_7 then
   If_Result_8 := True;
else
--# expr-start 11906 '<Symbol Precondition at nodes.lkt:16258:104>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition); 
--# expr-done 11906
Is_Equal_6 := Fld_3 = Sym_5; 
if Is_Equal_6 then
   If_Result_7 := True;
else
--# expr-start 11907 '<Symbol Postcondition at nodes.lkt:16258:122>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition); 
--# expr-done 11907
Is_Equal_5 := Fld_3 = Sym_6; 
if Is_Equal_5 then
   If_Result_6 := True;
else
--# expr-start 11908 '<Symbol Precondition'"'"'Class at nodes.lkt:16258:141>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Precondition_Class); 
--# expr-done 11908
Is_Equal_4 := Fld_3 = Sym_7; 
if Is_Equal_4 then
   If_Result_5 := True;
else
--# expr-start 11909 '<Symbol Postcondition'"'"'Class at nodes.lkt:16258:165>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Postcondition_Class); 
--# expr-done 11909
Is_Equal_3 := Fld_3 = Sym_8; 
if Is_Equal_3 then
   If_Result_4 := True;
else
--# expr-start 11910 '<Symbol Test_Case at nodes.lkt:16258:190>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Test_Case); 
--# expr-done 11910
Is_Equal_2 := Fld_3 = Sym_9; 
if Is_Equal_2 then
   If_Result_3 := True;
else
--# expr-start 11911 '<Symbol Contract_Cases at nodes.lkt:16258:205>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Contract_Cases); 
--# expr-done 11911
Is_Equal_1 := Fld_3 = Sym_10; 
if Is_Equal_1 then
   If_Result_2 := True;
else
--# expr-start 11912 '<Symbol Predicate at nodes.lkt:16258:225>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Predicate); 
--# expr-done 11912
Is_Equal := Fld_3 = Sym_11; 
   If_Result_2 := Is_Equal;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
   If_Result_11 := If_Result_10;
end if;
if If_Result_11 then
--# expr-start 11915 '<at at nodes.lkt:16258:243>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
--# expr-start 11914 '<FieldAccess for associated_entities at nodes.lkt:16258:243>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
--# expr-start 11913 '<as_bare_entity at nodes.lkt:16258:243>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16258'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 11913
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.associated_entities
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entities (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 11914
Get_Result := Get (Self, Fld, 0, True); 
--# expr-done 11915
Var_Expr_0 := Get_Result;
if Var_Expr_0 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind ent Var_Expr_0
--# expr-start 11917 '<New DesignatedEnv at nodes.lkt:16259:22>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16259'
--# expr-start 11916 '<FieldAccess for children_env at nodes.lkt:16260:86>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16260'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# expr-done 11916
Inc_Ref (Fld_1);
New_Struct_1 := (Direct_Env => Fld_1, Env_Name => No_Symbol, Kind => Direct_Env); 
--# expr-done 11917
Scope_Result := New_Struct_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3534;
   Result_Var := Scope_Result;
else
   --# expr-start 11918 '<New DesignatedEnv at nodes.lkt:16261:28>' New_Struct_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16261'
Inc_Ref (Empty_Env);
New_Struct_2 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 11918
   Result_Var := New_Struct_2;
end if;
      Inc_Ref (Result_Var);
   If_Result_12 := Result_Var;
else
   --# expr-start 11919 '<New DesignatedEnv at nodes.lkt:16265:14>' New_Struct_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16265'
Inc_Ref (Empty_Env);
New_Struct_3 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 11919
   If_Result_12 := New_Struct_3;
end if;
      Inc_Ref (If_Result_12);
--# expr-done 11920
   If_Result_13 := If_Result_12;
end if;
      Inc_Ref (If_Result_13);
--# expr-done 11921
         Property_Result := If_Result_13;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1373;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1373;
                     Finalizer_Scope_3534;
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_Initial_Env;
--# end
--# property-start Pragma.as_aspect 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16270'
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_As_Aspect
  (Node : Bare_Pragma_Node
      ; Inherited : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
     Ent : Internal_Entity_Pragma_Node :=
       Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind inherited Inherited
   Property_Result : Internal_Aspect;
      Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Pragma_Node;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11923 '<New Aspect at nodes.lkt:16270:54>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16270'
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 11922 '<FieldAccess for value_expr at nodes.lkt:16271:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16271'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.value_expr
Fld := Libadalang.Implementation.Pragma_Node_P_Value_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11922
New_Struct := (Exists => True, Inherited => Inherited, Node => Cast_Result, Value => Fld); 
--# expr-done 11923
         Property_Result := New_Struct;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Pragma_Node_P_As_Aspect;
--# end
--# property-start Pragma._env_init_217 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16275'
pragma Warnings (Off, "is not referenced");
 function Env_Init_217
  (Node : Bare_Pragma_Node
  )
   return Internal_Designated_Env
is
   Self : Bare_Pragma_Node  := Bare_Pragma_Node (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1881 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1881 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1881;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11924 '<FieldAccess for initial_env at nodes.lkt:16275:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16275'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Pragma.initial_env
Fld := Libadalang.Implementation.Pragma_Node_P_Initial_Env (Node => Self);
--# end
--# expr-done 11924
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1881;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1881;
            raise;
      end;
   return Property_Result;
end Env_Init_217;
--# end
   --
   --  Primitives for Bare_Private_Node
   --
--# property-start '[dispatcher]Private.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Private_Node_P_As_Bool
  (Node : Bare_Private_Node
  )
   return Boolean
is
   Self : Bare_Private_Node  := Bare_Private_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Private_Node (Self.Kind) is
               when Ada_Private_Absent =>
                  --# property-call-start Private.Absent.as_bool
                  Property_Result := Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Private_Present =>
                  --# property-call-start Private.Present.as_bool
                  Property_Result := Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Private_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Absent
   --
--# property-start Private.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16281'
pragma Warnings (Off, "is not referenced");
 function Private_Absent_P_As_Bool
  (Node : Bare_Private_Absent
  )
   return Boolean
is
   Self : Bare_Private_Absent  := Bare_Private_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Present
   --
--# property-start Private.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16281'
pragma Warnings (Off, "is not referenced");
 function Private_Present_P_As_Bool
  (Node : Bare_Private_Present
  )
   return Boolean
is
   Self : Bare_Private_Present  := Bare_Private_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Def
   --
      procedure Initialize_Fields_For_Protected_Def
        (Self : Bare_Protected_Def
         ; Protected_Def_F_Public_Part : Bare_Public_Part
         ; Protected_Def_F_Private_Part : Bare_Private_Part
         ; Protected_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Def_F_Public_Part := Protected_Def_F_Public_Part;
            Self.Protected_Def_F_Private_Part := Protected_Def_F_Private_Part;
            Self.Protected_Def_F_End_Name := Protected_Def_F_End_Name;
      end Initialize_Fields_For_Protected_Def;
   function Protected_Def_F_Public_Part
     (Node : Bare_Protected_Def) return Bare_Public_Part
   is
   begin
         return Node.Protected_Def_F_Public_Part;
   end;
   function Protected_Def_F_Private_Part
     (Node : Bare_Protected_Def) return Bare_Private_Part
   is
   begin
         return Node.Protected_Def_F_Private_Part;
   end;
   function Protected_Def_F_End_Name
     (Node : Bare_Protected_Def) return Bare_End_Name
   is
   begin
         return Node.Protected_Def_F_End_Name;
   end;
   --
   --  Primitives for Bare_Protected_Node
   --
--# property-start '[dispatcher]Protected.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Protected_Node_P_As_Bool
  (Node : Bare_Protected_Node
  )
   return Boolean
is
   Self : Bare_Protected_Node  := Bare_Protected_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Protected_Node (Self.Kind) is
               when Ada_Protected_Absent =>
                  --# property-call-start Protected.Absent.as_bool
                  Property_Result := Protected_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Protected_Present =>
                  --# property-call-start Protected.Present.as_bool
                  Property_Result := Protected_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Protected_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Absent
   --
--# property-start Protected.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16293'
pragma Warnings (Off, "is not referenced");
 function Protected_Absent_P_As_Bool
  (Node : Bare_Protected_Absent
  )
   return Boolean
is
   Self : Bare_Protected_Absent  := Bare_Protected_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Present
   --
--# property-start Protected.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16293'
pragma Warnings (Off, "is not referenced");
 function Protected_Present_P_As_Bool
  (Node : Bare_Protected_Present
  )
   return Boolean
is
   Self : Bare_Protected_Present  := Bare_Protected_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Protected_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Quantifier
   --
   --
   --  Primitives for Bare_Quantifier_All
   --
   --
   --  Primitives for Bare_Quantifier_Some
   --
   --
   --  Primitives for Bare_Range_Spec
   --
      procedure Initialize_Fields_For_Range_Spec
        (Self : Bare_Range_Spec
         ; Range_Spec_F_Range : Bare_Expr
        ) is
      begin
            Self.Range_Spec_F_Range := Range_Spec_F_Range;
      end Initialize_Fields_For_Range_Spec;
   function Range_Spec_F_Range
     (Node : Bare_Range_Spec) return Bare_Expr
   is
   begin
         return Node.Range_Spec_F_Range;
   end;
--# property-start RangeSpec.xref_stop_resolution 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16306'
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Stop_Resolution
  (Node : Bare_Range_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Range_Spec  := Bare_Range_Spec (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11926 '<IsA ComponentClause at nodes.lkt:16306:40>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16306'
--# expr-start 11925 '<parent at nodes.lkt:16306:40>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16306'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 11925
Is_A := Fld /= null 
and then Fld.Kind in Ada_Component_Clause_Range; 
--# expr-done 11926
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Range_Spec_P_Xref_Stop_Resolution;
--# end
--# property-start RangeSpec.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16309'
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Equation
  (Node : Bare_Range_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Spec  := Bare_Range_Spec (Node);
     Ent : Internal_Entity_Range_Spec :=
       Internal_Entity_Range_Spec'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1382 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Expr;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_12 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
Fld_13 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1382 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1382;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11947 '<And at nodes.lkt:16309:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16309'
--# expr-start 11928 '<FieldAccess for xref_equation at nodes.lkt:16309:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16309'
--# expr-start 11927 '<FieldAccess for range at nodes.lkt:16309:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16309'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 11927
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 11928
--# expr-start 11946 '<If at nodes.lkt:16312:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11930 '<IsA ComponentClause, SignedIntTypeDef at nodes.lkt:16312:12>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11929 '<parent at nodes.lkt:16312:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Self.Parent; 
--# expr-done 11929
Is_A_1 := Fld_13 /= null 
and then Fld_13.Kind in Ada_Component_Clause_Range | Ada_Signed_Int_Type_Def_Range; 
--# expr-done 11930
if Is_A_1 then
   --# expr-start 11936 '<And at nodes.lkt:16312:67>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11933 '<FieldAccess for universal_int_bind at nodes.lkt:16312:67>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11932 '<FieldAccess for expected_type_var at nodes.lkt:16312:91>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11931 '<FieldAccess for range at nodes.lkt:16312:91>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Range_Spec_F_Range;
--# expr-done 11931
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 11932
--# property-call-start AdaNode.universal_int_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 11933
--# expr-start 11935 '<FieldAccess for matches_expected_type at nodes.lkt:16312:127>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11934 '<FieldAccess for range at nodes.lkt:16312:127>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 11934
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 11935
And_Pred := Create_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16312:67") else null)); 
--# expr-done 11936
   If_Result_1 := And_Pred;
else
   --# expr-start 11945 '<If at nodes.lkt:16312:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16312'
--# expr-start 11938 '<IsA DeltaConstraint, DigitsConstraint, OrdinaryFixedPointDef, DecimalFixedPointDef at nodes.lkt:16315:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
--# expr-start 11937 '<parent at nodes.lkt:16315:14>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Parent; 
--# expr-done 11937
Is_A := Fld_12 /= null 
and then Fld_12.Kind in Ada_Delta_Constraint_Range | Ada_Digits_Constraint_Range | Ada_Ordinary_Fixed_Point_Def_Range | Ada_Decimal_Fixed_Point_Def_Range; 
--# expr-done 11938
if Is_A then
   --# expr-start 11944 '<And at nodes.lkt:16315:116>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
--# expr-start 11941 '<FieldAccess for universal_real_bind at nodes.lkt:16315:116>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11940 '<FieldAccess for expected_type_var at nodes.lkt:16315:141>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
--# expr-start 11939 '<FieldAccess for range at nodes.lkt:16315:141>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Range_Spec_F_Range;
--# expr-done 11939
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7);
--# expr-done 11940
--# property-call-start AdaNode.universal_real_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 11941
--# expr-start 11943 '<FieldAccess for matches_expected_type at nodes.lkt:16315:177>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
--# expr-start 11942 '<FieldAccess for range at nodes.lkt:16315:177>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16315'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 11942
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_11 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_10.Node, Origin => Origin);
--# end
--# expr-done 11943
And_Pred_1 := Create_And (Fld_9, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16315:116") else null)); 
--# expr-done 11944
   If_Result := And_Pred_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16316:14") else null)); 
   If_Result := True_Rel;
end if;
      Inc_Ref (If_Result);
--# expr-done 11945
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 11946
And_Pred_2 := Create_And (Fld_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16309:37") else null)); 
--# expr-done 11947
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1382;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1382;
            raise;
      end;
   return Property_Result;
end Range_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Renaming_Clause
   --
      procedure Initialize_Fields_For_Renaming_Clause
        (Self : Bare_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Self.Renaming_Clause_F_Renamed_Object := Renaming_Clause_F_Renamed_Object;
      end Initialize_Fields_For_Renaming_Clause;
   function Renaming_Clause_F_Renamed_Object
     (Node : Bare_Renaming_Clause) return Bare_Name
   is
   begin
         return Node.Renaming_Clause_F_Renamed_Object;
   end;
   --
   --  Primitives for Bare_Synthetic_Renaming_Clause
   --
      procedure Initialize_Fields_For_Synthetic_Renaming_Clause
        (Self : Bare_Synthetic_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Renaming_Clause
              (Self, Renaming_Clause_F_Renamed_Object);
      end Initialize_Fields_For_Synthetic_Renaming_Clause;
   --
   --  Primitives for Bare_Reverse_Node
   --
--# property-start '[dispatcher]Reverse.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Reverse_Node_P_As_Bool
  (Node : Bare_Reverse_Node
  )
   return Boolean
is
   Self : Bare_Reverse_Node  := Bare_Reverse_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Reverse_Node (Self.Kind) is
               when Ada_Reverse_Absent =>
                  --# property-call-start Reverse.Absent.as_bool
                  Property_Result := Reverse_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Reverse_Present =>
                  --# property-call-start Reverse.Present.as_bool
                  Property_Result := Reverse_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Reverse_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Absent
   --
--# property-start Reverse.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16333'
pragma Warnings (Off, "is not referenced");
 function Reverse_Absent_P_As_Bool
  (Node : Bare_Reverse_Absent
  )
   return Boolean
is
   Self : Bare_Reverse_Absent  := Bare_Reverse_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reverse_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Present
   --
--# property-start Reverse.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16333'
pragma Warnings (Off, "is not referenced");
 function Reverse_Present_P_As_Bool
  (Node : Bare_Reverse_Present
  )
   return Boolean
is
   Self : Bare_Reverse_Present  := Bare_Reverse_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Reverse_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Select_When_Part
   --
      procedure Initialize_Fields_For_Select_When_Part
        (Self : Bare_Select_When_Part
         ; Select_When_Part_F_Cond_Expr : Bare_Expr
         ; Select_When_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_When_Part_F_Cond_Expr := Select_When_Part_F_Cond_Expr;
            Self.Select_When_Part_F_Stmts := Select_When_Part_F_Stmts;
      end Initialize_Fields_For_Select_When_Part;
   function Select_When_Part_F_Cond_Expr
     (Node : Bare_Select_When_Part) return Bare_Expr
   is
   begin
         return Node.Select_When_Part_F_Cond_Expr;
   end;
   function Select_When_Part_F_Stmts
     (Node : Bare_Select_When_Part) return Bare_Stmt_List
   is
   begin
         return Node.Select_When_Part_F_Stmts;
   end;
--# property-start SelectWhenPart.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16342'
pragma Warnings (Off, "is not referenced");
 function Select_When_Part_P_Xref_Equation
  (Node : Bare_Select_When_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_When_Part  := Bare_Select_When_Part (Node);
     Ent : Internal_Entity_Select_When_Part :=
       Internal_Entity_Select_When_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1386 with Inline_Always;
            procedure Finalizer_Scope_3535 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1386 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1386;
            procedure Finalizer_Scope_3535 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3535;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11948 '<FieldAccess for cond_expr at nodes.lkt:16342:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16342'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Select_When_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 11948
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 11951 '<And at nodes.lkt:16343:16>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16343'
--# expr-start 11949 '<FieldAccess for sub_equation at nodes.lkt:16343:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16343'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11949
--# expr-start 11950 '<FieldAccess for expect_bool_derived_type at nodes.lkt:16343:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16343'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_2 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 11950
And_Pred := Create_And (Fld_1, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16343:16") else null)); 
--# expr-done 11951
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3535;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16343:79") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1386;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1386;
                     Finalizer_Scope_3535;
            raise;
      end;
   return Property_Result;
end Select_When_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Stmt
   --
--# property-start Stmt.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16350'
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Xref_Entry_Point
  (Node : Bare_Stmt
  )
   return Boolean
is
   Self : Bare_Stmt  := Bare_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Stmt_P_Xref_Entry_Point;
--# end
--# property-start Stmt.is_ghost_code 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16354'
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Is_Ghost_Code
  (Node : Bare_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Stmt  := Bare_Stmt (Node);
     Ent : Internal_Entity_Stmt :=
       Internal_Entity_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Stmt;
Match_2 : Internal_Entity_Stmt;
Var_Expr_1 : Internal_Refd_Def;
Var_Expr_281 : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Refd_Def;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Refd_Def;
Match_Prefix : Internal_Entity_Stmt;
Cast_Result : Internal_Entity_Assign_Stmt;
Let_Result : Internal_Refd_Def;
Scope_Result_1 : Internal_Refd_Def;
Cast_Result_1 : Internal_Entity_Call_Stmt;
Let_Result_1 : Internal_Refd_Def;
Scope_Result_2 : Internal_Refd_Def;
Cast_Result_2 : Internal_Entity_Stmt;
Let_Result_2 : Internal_Refd_Def;
Scope_Result_3 : Internal_Refd_Def;
Match_Result : Internal_Refd_Def;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_1 : Boolean;
Fld_8 : Ref_Result_Kind;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 11952 '<FieldAccess for parent_basic_decl at nodes.lkt:16357:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16357'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 11952
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_0
--# expr-start 11953 '<FieldAccess for is_ghost_code at nodes.lkt:16357:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16357'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.is_ghost_code
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Is_Ghost_Code (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11953
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result_1 := True;
else
--# expr-start 11958 '<Match at nodes.lkt:16361:9>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16361'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Stmt (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ass Match_0
--# expr-start 11955 '<FieldAccess for failsafe_referenced_def_name at nodes.lkt:16362:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16362'
--# expr-start 11954 '<FieldAccess for dest at nodes.lkt:16362:37>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16362'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 11954
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_3 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Fld_2.Node, Imprecise_Fallback => False, E_Info => Fld_2.Info);
--# end
--# expr-done 11955
Let_Result := Fld_3; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Call_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Call_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind call Match_1
--# expr-start 11957 '<FieldAccess for failsafe_referenced_def_name at nodes.lkt:16363:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16363'
--# expr-start 11956 '<FieldAccess for call at nodes.lkt:16363:36>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16363'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Stmt_F_Call, Info => Match_1.Info);
--# expr-done 11956
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_5 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Fld_4.Node, Imprecise_Fallback => False, E_Info => Fld_4.Info);
--# end
--# expr-done 11957
Let_Result_1 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Accept_Stmt .. Ada_Abort_Stmt | Ada_Delay_Stmt .. Ada_Terminate_Alternative =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Refd_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 11958
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Refd_Def then
   --# scope-start
   --# bind res Var_Expr_1
--# expr-start 11963 '<If at nodes.lkt:16372:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16372'
--# expr-start 11960 '<Eq at nodes.lkt:16372:16>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16372'
--# expr-start 11959 '<FieldAccess for kind at nodes.lkt:16372:16>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16372'
Fld_8 := Var_Expr_1.Kind;
--# expr-done 11959
Is_Equal := Fld_8 = Error; 
--# expr-done 11960
if Is_Equal then
   If_Result := raise Property_Error with "Name resolution error";
else
--# expr-start 11961 '<FieldAccess for def_name at nodes.lkt:16372:109>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16372'
Fld_6 := Var_Expr_1.Def_Name;
--# expr-done 11961
Var_Expr_281 := Fld_6;
if Var_Expr_281 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 11962 '<FieldAccess for is_ghost_code at nodes.lkt:16372:109>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16372'
      if Var_Expr_281.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start DefiningName.is_ghost_code
Fld_7 := Libadalang.Implementation.Defining_Name_P_Is_Ghost_Code (Node => Var_Expr_281.Node, E_Info => Var_Expr_281.Info);
--# end
--# expr-done 11962
Scope_Result_4 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 11963
Scope_Result_5 := If_Result;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := False;
end if;
   If_Result_1 := Result_Var_2;
end if;
         Property_Result := If_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Stmt_P_Is_Ghost_Code;
--# end
   --
   --  Primitives for Bare_Composite_Stmt
   --
   --
   --  Primitives for Bare_Accept_Stmt
   --
      procedure Accept_Stmt_Pre_Env_Actions
        (Self            : Bare_Accept_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_218 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:16416:9");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_219 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Accept_Stmt
        (Self : Bare_Accept_Stmt
         ; Accept_Stmt_F_Body_Decl : Bare_Accept_Stmt_Body
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
        ) is
      begin
            Self.Accept_Stmt_F_Body_Decl := Accept_Stmt_F_Body_Decl;
            Self.Accept_Stmt_F_Entry_Index_Expr := Accept_Stmt_F_Entry_Index_Expr;
            Self.Accept_Stmt_F_Params := Accept_Stmt_F_Params;
      end Initialize_Fields_For_Accept_Stmt;
   function Accept_Stmt_F_Body_Decl
     (Node : Bare_Accept_Stmt) return Bare_Accept_Stmt_Body
   is
   begin
         return Node.Accept_Stmt_F_Body_Decl;
   end;
   function Accept_Stmt_F_Entry_Index_Expr
     (Node : Bare_Accept_Stmt) return Bare_Expr
   is
   begin
         return Node.Accept_Stmt_F_Entry_Index_Expr;
   end;
   function Accept_Stmt_F_Params
     (Node : Bare_Accept_Stmt) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Accept_Stmt_F_Params;
   end;
--# property-start AcceptStmt.designated_entry 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Designated_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1389 with Inline_Always;
      Var_Expr_1 : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Accept_Stmt_Body;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Entry_Spec;
Fld_4 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Find_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1389 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1389;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11972 '<Cast to EntryDecl at nodes.lkt:16389:49>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
--# expr-start 11971 '<Find at nodes.lkt:16389:49>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
   --# expr-start 11966 '<FieldAccess for all_env_els_impl at nodes.lkt:16389:49>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
--# expr-start 11965 '<FieldAccess for name at nodes.lkt:16389:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
--# expr-start 11964 '<FieldAccess for body_decl at nodes.lkt:16389:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16389'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Accept_Stmt_Body (Node => Ent.Node.Accept_Stmt_F_Body_Decl, Info => Ent.Info);
--# expr-done 11964
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Accept_Stmt_Body_F_Name, Info => Fld.Info);
--# expr-done 11965
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 11966
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_0
--# expr-start 11967 '<Cast to EntryDecl at nodes.lkt:16390:16>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16390'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Entry_Decl;
   end if;
--# expr-done 11967
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 11970 '<FieldAccess for match_formal_params at nodes.lkt:16391:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16391'
--# expr-start 11968 '<FieldAccess for spec at nodes.lkt:16391:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16391'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_1.Node.Entry_Decl_F_Spec, Info => Var_Expr_1.Info);
--# expr-done 11968
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 11969 '<FieldAccess for params at nodes.lkt:16391:47>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16391'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Ent.Node.Accept_Stmt_F_Params, Info => Ent.Info);
--# expr-done 11969
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_3.Node, Other => Cast_Result_1, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 11970
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
      if Result_Var then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Result_Var;
         end loop;
      end;
--# expr-done 11971
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 11972
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1389;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1389;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Designated_Entry;
--# end
--# property-start AcceptStmt.corresponding_entry 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16398'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Corresponding_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1390 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Entry_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1390 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1390;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 11975 '<bind at nodes.lkt:16399:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16399'
--# expr-start 11973 '<FieldAccess for node_env at nodes.lkt:16399:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16399'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11973
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 11974 '<FieldAccess for designated_entry at nodes.lkt:16401:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16401'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_1 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 11974
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 11975
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1390;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1390;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Corresponding_Entry;
--# end
--# property-start AcceptStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16405'
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Xref_Equation
  (Node : Bare_Accept_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1391 with Inline_Always;
            procedure Finalizer_Scope_3544 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Bare_Accept_Stmt_Body;
Fld_1 : Bare_Defining_Name;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Entry_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Entry_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_8 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1391 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1391;
            procedure Finalizer_Scope_3544 is
            begin
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3544;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11988 '<And at nodes.lkt:16406:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
--# expr-start 11980 '<Bind at nodes.lkt:16406:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
--# expr-start 11978 '<FieldAccess for ref_var at nodes.lkt:16406:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
--# expr-start 11977 '<FieldAccess for name at nodes.lkt:16406:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
--# expr-start 11976 '<FieldAccess for body_decl at nodes.lkt:16406:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Accept_Stmt_F_Body_Decl;
--# expr-done 11976
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Accept_Stmt_Body_F_Name;
--# expr-done 11977
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_1);
--# end
--# expr-done 11978
Fld_2.Value := No_Entity;
Entity_Vars.Reset (Fld_2);
--# expr-start 11979 '<FieldAccess for designated_entry at nodes.lkt:16406:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_3 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 11979
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Solver.Create_Assign (Fld_2, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16406:9") else null)); 
--# expr-done 11980
--# expr-start 11981 '<FieldAccess for entry_index_expr at nodes.lkt:16406:73>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16406'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Accept_Stmt_F_Entry_Index_Expr, Info => Ent.Info);
--# expr-done 11981
Var_Expr_0 := Fld_4;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr_0
--# expr-start 11987 '<And at nodes.lkt:16407:20>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16407'
--# expr-start 11985 '<Bind at nodes.lkt:16411:17>' Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16411'
--# expr-start 11982 '<FieldAccess for expected_type_var at nodes.lkt:16411:21>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16411'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 11982
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 11984 '<FieldAccess for family_type at nodes.lkt:16411:44>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16411'
--# expr-start 11983 '<FieldAccess for designated_entry at nodes.lkt:16411:44>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16411'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_6 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 11983
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start EntryDecl.family_type
Fld_7 := Libadalang.Implementation.Entry_Decl_P_Family_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 11984
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
Bind_Result_1 := Solver.Create_Assign (Fld_5, Cast_Result_1, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16411:17") else null)); 
--# expr-done 11985
--# expr-start 11986 '<FieldAccess for sub_equation at nodes.lkt:16412:19>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16412'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11986
And_Pred := Create_And (Bind_Result_1, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16407:20") else null)); 
--# expr-done 11987
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3544;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16412:49") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16406:9") else null)); 
--# expr-done 11988
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1391;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1391;
                     Finalizer_Scope_3544;
            raise;
      end;
   return Property_Result;
end Accept_Stmt_P_Xref_Equation;
--# end
--# property-start AcceptStmt._env_mappings_218 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16417'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_218
  (Node : Bare_Accept_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
     Ent : Internal_Entity_Accept_Stmt :=
       Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1882 with Inline_Always;
      Fld : Internal_Entity_Accept_Stmt_Body;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type;
Fld_4 : Bare_Accept_Stmt_Body;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1882 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_1882;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 11996 '<ArrayLiteral at nodes.lkt:16417:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16417'
--# expr-start 11995 '<New EnvAssoc at nodes.lkt:16417:14>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16417'
--# expr-start 11989 '<New DesignatedEnv at nodes.lkt:16418:103>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 11989
--# expr-start 11993 '<FieldAccess for name_symbol at nodes.lkt:16418:21>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
--# expr-start 11992 '<FieldAccess for relative_name at nodes.lkt:16418:21>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
--# expr-start 11991 '<FieldAccess for name at nodes.lkt:16418:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
--# expr-start 11990 '<FieldAccess for body_decl at nodes.lkt:16418:21>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Accept_Stmt_Body (Node => Ent.Node.Accept_Stmt_F_Body_Decl, Info => Ent.Info);
--# expr-done 11990
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Accept_Stmt_Body_F_Name, Info => Fld.Info);
--# expr-done 11991
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 11992
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 11993
--# expr-start 11994 '<FieldAccess for body_decl at nodes.lkt:16418:78>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16418'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Accept_Stmt_F_Body_Decl;
--# expr-done 11994
      Cast_Result := Fld_4;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_3, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 11995
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => New_Struct_1)); 
--# expr-done 11996
         Property_Result := Array_Lit;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1882;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1882;
            raise;
      end;
   return Property_Result;
end Env_Mappings_218;
--# end
--# property-start AcceptStmt._env_trans_parent_219 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_219
  (Node : Bare_Accept_Stmt
  )
   return Boolean
is
   Self : Bare_Accept_Stmt  := Bare_Accept_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_219;
--# end
   --
   --  Primitives for Bare_Accept_Stmt_With_Stmts
   --
      procedure Initialize_Fields_For_Accept_Stmt_With_Stmts
        (Self : Bare_Accept_Stmt_With_Stmts
         ; Accept_Stmt_F_Body_Decl : Bare_Accept_Stmt_Body
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
         ; Accept_Stmt_With_Stmts_F_Stmts : Bare_Handled_Stmts
         ; Accept_Stmt_With_Stmts_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Accept_Stmt
              (Self, Accept_Stmt_F_Body_Decl, Accept_Stmt_F_Entry_Index_Expr, Accept_Stmt_F_Params);
            Self.Accept_Stmt_With_Stmts_F_Stmts := Accept_Stmt_With_Stmts_F_Stmts;
            Self.Accept_Stmt_With_Stmts_F_End_Name := Accept_Stmt_With_Stmts_F_End_Name;
      end Initialize_Fields_For_Accept_Stmt_With_Stmts;
   function Accept_Stmt_With_Stmts_F_Stmts
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_Handled_Stmts
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_Stmts;
   end;
   function Accept_Stmt_With_Stmts_F_End_Name
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_End_Name
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_End_Name;
   end;
   --
   --  Primitives for Bare_Base_Loop_Stmt
   --
      procedure Initialize_Fields_For_Base_Loop_Stmt
        (Self : Bare_Base_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Loop_Stmt_F_Spec := Base_Loop_Stmt_F_Spec;
            Self.Base_Loop_Stmt_F_Stmts := Base_Loop_Stmt_F_Stmts;
            Self.Base_Loop_Stmt_F_End_Name := Base_Loop_Stmt_F_End_Name;
      end Initialize_Fields_For_Base_Loop_Stmt;
   function Base_Loop_Stmt_F_Spec
     (Node : Bare_Base_Loop_Stmt) return Bare_Loop_Spec
   is
   begin
         return Node.Base_Loop_Stmt_F_Spec;
   end;
   function Base_Loop_Stmt_F_Stmts
     (Node : Bare_Base_Loop_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Base_Loop_Stmt_F_Stmts;
   end;
   function Base_Loop_Stmt_F_End_Name
     (Node : Bare_Base_Loop_Stmt) return Bare_End_Name
   is
   begin
         return Node.Base_Loop_Stmt_F_End_Name;
   end;
--# property-start BaseLoopStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16441'
pragma Warnings (Off, "is not referenced");
 function Base_Loop_Stmt_P_Xref_Equation
  (Node : Bare_Base_Loop_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Loop_Stmt  := Bare_Base_Loop_Stmt (Node);
     Ent : Internal_Entity_Base_Loop_Stmt :=
       Internal_Entity_Base_Loop_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1392 with Inline_Always;
            procedure Finalizer_Scope_3545 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Loop_Spec;
Fld : Internal_Entity_Loop_Spec;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1392 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1392;
            procedure Finalizer_Scope_3545 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3545;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 11997 '<FieldAccess for spec at nodes.lkt:16441:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16441'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Loop_Spec (Node => Ent.Node.Base_Loop_Stmt_F_Spec, Info => Ent.Info);
--# expr-done 11997
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Loop_Spec then
   --# scope-start
   --# bind s Var_Expr_0
--# expr-start 11998 '<FieldAccess for xref_equation at nodes.lkt:16442:16>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16442'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 11998
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3545;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16442:47") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1392;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1392;
                     Finalizer_Scope_3545;
            raise;
      end;
   return Property_Result;
end Base_Loop_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_For_Loop_Stmt
   --
      procedure For_Loop_Stmt_Pre_Env_Actions
        (Self            : Bare_For_Loop_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_220 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Stmt
        (Self : Bare_For_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_For_Loop_Stmt;
--# property-start ForLoopStmt._env_trans_parent_220 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_220
  (Node : Bare_For_Loop_Stmt
  )
   return Boolean
is
   Self : Bare_For_Loop_Stmt  := Bare_For_Loop_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_220;
--# end
   --
   --  Primitives for Bare_Loop_Stmt
   --
      procedure Initialize_Fields_For_Loop_Stmt
        (Self : Bare_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_Loop_Stmt;
   --
   --  Primitives for Bare_While_Loop_Stmt
   --
      procedure Initialize_Fields_For_While_Loop_Stmt
        (Self : Bare_While_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_While_Loop_Stmt;
   --
   --  Primitives for Bare_Block_Stmt
   --
      procedure Block_Stmt_Pre_Env_Actions
        (Self            : Bare_Block_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_221 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
--# property-start BlockStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16467'
pragma Warnings (Off, "is not referenced");
 function Block_Stmt_P_Xref_Equation
  (Node : Bare_Block_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Block_Stmt  := Bare_Block_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1393 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1393 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1393;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16467:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1393;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1393;
            raise;
      end;
   return Property_Result;
end Block_Stmt_P_Xref_Equation;
--# end
--# property-start BlockStmt._env_trans_parent_221 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_221
  (Node : Bare_Block_Stmt
  )
   return Boolean
is
   Self : Bare_Block_Stmt  := Bare_Block_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_221;
--# end
   --
   --  Primitives for Bare_Begin_Block
   --
      procedure Initialize_Fields_For_Begin_Block
        (Self : Bare_Begin_Block
         ; Begin_Block_F_Stmts : Bare_Handled_Stmts
         ; Begin_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Begin_Block_F_Stmts := Begin_Block_F_Stmts;
            Self.Begin_Block_F_End_Name := Begin_Block_F_End_Name;
      end Initialize_Fields_For_Begin_Block;
   function Begin_Block_F_Stmts
     (Node : Bare_Begin_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Begin_Block_F_Stmts;
   end;
   function Begin_Block_F_End_Name
     (Node : Bare_Begin_Block) return Bare_End_Name
   is
   begin
         return Node.Begin_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Decl_Block
   --
      procedure Initialize_Fields_For_Decl_Block
        (Self : Bare_Decl_Block
         ; Decl_Block_F_Decls : Bare_Declarative_Part
         ; Decl_Block_F_Stmts : Bare_Handled_Stmts
         ; Decl_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Decl_Block_F_Decls := Decl_Block_F_Decls;
            Self.Decl_Block_F_Stmts := Decl_Block_F_Stmts;
            Self.Decl_Block_F_End_Name := Decl_Block_F_End_Name;
      end Initialize_Fields_For_Decl_Block;
   function Decl_Block_F_Decls
     (Node : Bare_Decl_Block) return Bare_Declarative_Part
   is
   begin
         return Node.Decl_Block_F_Decls;
   end;
   function Decl_Block_F_Stmts
     (Node : Bare_Decl_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Decl_Block_F_Stmts;
   end;
   function Decl_Block_F_End_Name
     (Node : Bare_Decl_Block) return Bare_End_Name
   is
   begin
         return Node.Decl_Block_F_End_Name;
   end;
--# property-start DeclBlock.immediate_declarative_region 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16486'
pragma Warnings (Off, "is not referenced");
 function Decl_Block_P_Immediate_Declarative_Region
  (Node : Bare_Decl_Block
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Decl_Block  := Bare_Decl_Block (Node);
     Ent : Internal_Entity_Decl_Block :=
       Internal_Entity_Decl_Block'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1394 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1394 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1394;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 11999 '<FieldAccess for children_env at nodes.lkt:16487:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16487'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 11999
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1394;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1394;
            raise;
      end;
   return Property_Result;
end Decl_Block_P_Immediate_Declarative_Region;
--# end
   --
   --  Primitives for Bare_Case_Stmt
   --
      procedure Initialize_Fields_For_Case_Stmt
        (Self : Bare_Case_Stmt
         ; Case_Stmt_F_Expr : Bare_Expr
         ; Case_Stmt_F_Pragmas : Bare_Pragma_Node_List
         ; Case_Stmt_F_Alternatives : Bare_Case_Stmt_Alternative_List
        ) is
      begin
            Self.Case_Stmt_F_Expr := Case_Stmt_F_Expr;
            Self.Case_Stmt_F_Pragmas := Case_Stmt_F_Pragmas;
            Self.Case_Stmt_F_Alternatives := Case_Stmt_F_Alternatives;
      end Initialize_Fields_For_Case_Stmt;
   function Case_Stmt_F_Expr
     (Node : Bare_Case_Stmt) return Bare_Expr
   is
   begin
         return Node.Case_Stmt_F_Expr;
   end;
   function Case_Stmt_F_Pragmas
     (Node : Bare_Case_Stmt) return Bare_Pragma_Node_List
   is
   begin
         return Node.Case_Stmt_F_Pragmas;
   end;
   function Case_Stmt_F_Alternatives
     (Node : Bare_Case_Stmt) return Bare_Case_Stmt_Alternative_List
   is
   begin
         return Node.Case_Stmt_F_Alternatives;
   end;
--# property-start CaseStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16497'
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_P_Xref_Equation
  (Node : Bare_Case_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt  := Bare_Case_Stmt (Node);
     Ent : Internal_Entity_Case_Stmt :=
       Internal_Entity_Case_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1395 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1395 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1395;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12009 '<And at nodes.lkt:16497:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16497'
--# expr-start 12001 '<FieldAccess for sub_equation at nodes.lkt:16497:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16497'
--# expr-start 12000 '<FieldAccess for expr at nodes.lkt:16497:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16497'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 12000
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12001
--# expr-start 12008 '<And at nodes.lkt:16498:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16498'
--# expr-start 12004 '<Predicate on AdaNode.is_not_null at nodes.lkt:16502:13>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16502'
--# expr-start 12003 '<FieldAccess for type_var at nodes.lkt:16502:45>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16502'
--# expr-start 12002 '<FieldAccess for expr at nodes.lkt:16502:45>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16502'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Case_Stmt_F_Expr;
--# expr-done 12002
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 12003
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Pred := Solver.Create_Predicate (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate); 
--# expr-done 12004
--# expr-start 12007 '<Predicate on BaseTypeDecl.is_discrete_type at nodes.lkt:16505:13>' Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16505'
--# expr-start 12006 '<FieldAccess for type_var at nodes.lkt:16505:55>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16505'
--# expr-start 12005 '<FieldAccess for expr at nodes.lkt:16505:55>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16505'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Case_Stmt_F_Expr;
--# expr-done 12005
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 12006
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
Pred_1 := Solver.Create_Predicate (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin)); 
--# expr-done 12007
And_Pred := Create_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16498:9") else null)); 
--# expr-done 12008
And_Pred_1 := Create_And (Fld_1, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16497:37") else null)); 
--# expr-done 12009
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1395;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1395;
            raise;
      end;
   return Property_Result;
end Case_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt
   --
      procedure Extended_Return_Stmt_Pre_Env_Actions
        (Self            : Bare_Extended_Return_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_222 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Extended_Return_Stmt
        (Self : Bare_Extended_Return_Stmt
         ; Extended_Return_Stmt_F_Decl : Bare_Extended_Return_Stmt_Object_Decl
         ; Extended_Return_Stmt_F_Stmts : Bare_Handled_Stmts
        ) is
      begin
            Self.Extended_Return_Stmt_F_Decl := Extended_Return_Stmt_F_Decl;
            Self.Extended_Return_Stmt_F_Stmts := Extended_Return_Stmt_F_Stmts;
      end Initialize_Fields_For_Extended_Return_Stmt;
   function Extended_Return_Stmt_F_Decl
     (Node : Bare_Extended_Return_Stmt) return Bare_Extended_Return_Stmt_Object_Decl
   is
   begin
         return Node.Extended_Return_Stmt_F_Decl;
   end;
   function Extended_Return_Stmt_F_Stmts
     (Node : Bare_Extended_Return_Stmt) return Bare_Handled_Stmts
   is
   begin
         return Node.Extended_Return_Stmt_F_Stmts;
   end;
--# property-start ExtendedReturnStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16516'
pragma Warnings (Off, "is not referenced");
 function Extended_Return_Stmt_P_Xref_Equation
  (Node : Bare_Extended_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Extended_Return_Stmt  := Bare_Extended_Return_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1396 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1396 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1396;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16516:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1396;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1396;
            raise;
      end;
   return Property_Result;
end Extended_Return_Stmt_P_Xref_Equation;
--# end
--# property-start ExtendedReturnStmt._env_trans_parent_222 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_222
  (Node : Bare_Extended_Return_Stmt
  )
   return Boolean
is
   Self : Bare_Extended_Return_Stmt  := Bare_Extended_Return_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_222;
--# end
   --
   --  Primitives for Bare_If_Stmt
   --
      procedure Initialize_Fields_For_If_Stmt
        (Self : Bare_If_Stmt
         ; If_Stmt_F_Cond_Expr : Bare_Expr
         ; If_Stmt_F_Then_Stmts : Bare_Stmt_List
         ; If_Stmt_F_Alternatives : Bare_Elsif_Stmt_Part_List
         ; If_Stmt_F_Else_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.If_Stmt_F_Cond_Expr := If_Stmt_F_Cond_Expr;
            Self.If_Stmt_F_Then_Stmts := If_Stmt_F_Then_Stmts;
            Self.If_Stmt_F_Alternatives := If_Stmt_F_Alternatives;
            Self.If_Stmt_F_Else_Stmts := If_Stmt_F_Else_Stmts;
      end Initialize_Fields_For_If_Stmt;
   function If_Stmt_F_Cond_Expr
     (Node : Bare_If_Stmt) return Bare_Expr
   is
   begin
         return Node.If_Stmt_F_Cond_Expr;
   end;
   function If_Stmt_F_Then_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Then_Stmts;
   end;
   function If_Stmt_F_Alternatives
     (Node : Bare_If_Stmt) return Bare_Elsif_Stmt_Part_List
   is
   begin
         return Node.If_Stmt_F_Alternatives;
   end;
   function If_Stmt_F_Else_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Else_Stmts;
   end;
--# property-start IfStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16531'
pragma Warnings (Off, "is not referenced");
 function If_Stmt_P_Xref_Equation
  (Node : Bare_If_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Stmt  := Bare_If_Stmt (Node);
     Ent : Internal_Entity_If_Stmt :=
       Internal_Entity_If_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1397 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1397 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1397;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12014 '<And at nodes.lkt:16532:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16532'
--# expr-start 12011 '<FieldAccess for sub_equation at nodes.lkt:16532:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16532'
--# expr-start 12010 '<FieldAccess for cond_expr at nodes.lkt:16532:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16532'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 12010
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12011
--# expr-start 12013 '<FieldAccess for expect_bool_derived_type at nodes.lkt:16532:43>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16532'
--# expr-start 12012 '<FieldAccess for cond_expr at nodes.lkt:16532:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16532'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.If_Stmt_F_Cond_Expr;
--# expr-done 12012
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_3 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Fld_2, Origin => Origin);
--# end
--# expr-done 12013
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16532:9") else null)); 
--# expr-done 12014
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1397;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1397;
            raise;
      end;
   return Property_Result;
end If_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Named_Stmt
   --
      procedure Named_Stmt_Pre_Env_Actions
        (Self            : Bare_Named_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Resolver : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_223 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Value,
            Mapping.Metadata,
            Resolver,
            Mapping.Dest_Env,
            DSL_Location => "nodes.lkt:16546:9");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_224 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Named_Stmt
        (Self : Bare_Named_Stmt
         ; Named_Stmt_F_Decl : Bare_Named_Stmt_Decl
         ; Named_Stmt_F_Stmt : Bare_Composite_Stmt
        ) is
      begin
            Self.Named_Stmt_F_Decl := Named_Stmt_F_Decl;
            Self.Named_Stmt_F_Stmt := Named_Stmt_F_Stmt;
      end Initialize_Fields_For_Named_Stmt;
   function Named_Stmt_F_Decl
     (Node : Bare_Named_Stmt) return Bare_Named_Stmt_Decl
   is
   begin
         return Node.Named_Stmt_F_Decl;
   end;
   function Named_Stmt_F_Stmt
     (Node : Bare_Named_Stmt) return Bare_Composite_Stmt
   is
   begin
         return Node.Named_Stmt_F_Stmt;
   end;
--# property-start NamedStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16543'
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_P_Xref_Equation
  (Node : Bare_Named_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1398 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1398 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1398;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16543:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1398;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1398;
            raise;
      end;
   return Property_Result;
end Named_Stmt_P_Xref_Equation;
--# end
--# property-start NamedStmt._env_mappings_223 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16546'
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_223
  (Node : Bare_Named_Stmt
  )
   return Internal_Env_Assoc
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1887 with Inline_Always;
      Fld : Bare_Named_Stmt_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Named_Stmt_Decl;
New_Struct : Internal_Designated_Env := No_Designated_Env;
Cast_Result : Bare_Ada_Node;
New_Struct_1 : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1887 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1887;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12019 '<New EnvAssoc at ???>' New_Struct_1 None
--# expr-start 12015 '<New DesignatedEnv at ???>' New_Struct None
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 12015
--# expr-start 12017 '<FieldAccess for name_symbol at nodes.lkt:16546:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16546'
--# expr-start 12016 '<FieldAccess for decl at nodes.lkt:16546:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16546'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Named_Stmt_F_Decl;
--# expr-done 12016
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 12017
--# expr-start 12018 '<FieldAccess for decl at nodes.lkt:16546:48>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16546'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Named_Stmt_F_Decl;
--# expr-done 12018
      Cast_Result := Fld_2;
Inc_Ref (New_Struct);
New_Struct_1 := (Dest_Env => New_Struct, Key => Fld_1, Metadata => No_Metadata, Value => Cast_Result); 
--# expr-done 12019
         Property_Result := New_Struct_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1887;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1887;
            raise;
      end;
   return Property_Result;
end Env_Mappings_223;
--# end
--# property-start NamedStmt._env_trans_parent_224 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_224
  (Node : Bare_Named_Stmt
  )
   return Boolean
is
   Self : Bare_Named_Stmt  := Bare_Named_Stmt (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_224;
--# end
   --
   --  Primitives for Bare_Select_Stmt
   --
      procedure Initialize_Fields_For_Select_Stmt
        (Self : Bare_Select_Stmt
         ; Select_Stmt_F_Guards : Bare_Select_When_Part_List
         ; Select_Stmt_F_Else_Stmts : Bare_Stmt_List
         ; Select_Stmt_F_Abort_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_Stmt_F_Guards := Select_Stmt_F_Guards;
            Self.Select_Stmt_F_Else_Stmts := Select_Stmt_F_Else_Stmts;
            Self.Select_Stmt_F_Abort_Stmts := Select_Stmt_F_Abort_Stmts;
      end Initialize_Fields_For_Select_Stmt;
   function Select_Stmt_F_Guards
     (Node : Bare_Select_Stmt) return Bare_Select_When_Part_List
   is
   begin
         return Node.Select_Stmt_F_Guards;
   end;
   function Select_Stmt_F_Else_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Else_Stmts;
   end;
   function Select_Stmt_F_Abort_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Abort_Stmts;
   end;
--# property-start SelectStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16558'
pragma Warnings (Off, "is not referenced");
 function Select_Stmt_P_Xref_Equation
  (Node : Bare_Select_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_Stmt  := Bare_Select_Stmt (Node);
     Ent : Internal_Entity_Select_Stmt :=
       Internal_Entity_Select_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1399 with Inline_Always;
            procedure Finalizer_Scope_3546 with Inline_Always;
      Fld : Internal_Entity_Select_When_Part_List;
Fld_1 : Bare_Select_When_Part_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Select_When_Part;
Bare_Item_0 : Bare_Select_When_Part;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Select_When_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Select_When_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1399 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1399;
            procedure Finalizer_Scope_3546 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3546;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12023 '<LogicAll at nodes.lkt:16559:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16559'
--# expr-start 12022 '<Map at nodes.lkt:16559:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16559'
   --# expr-start 12020 '<FieldAccess for guards at nodes.lkt:16559:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16559'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Select_When_Part_List (Node => Ent.Node.Select_Stmt_F_Guards, Info => Ent.Info);
--# expr-done 12020
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Select_When_Part_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Select_When_Part;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind wp Item_0
      --# expr-start 12021 '<FieldAccess for sub_equation at nodes.lkt:16559:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16559'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 12021
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3546;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12022
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16559:9") else null)); 
--# expr-done 12023
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1399;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1399;
                     Finalizer_Scope_3546;
            raise;
      end;
   return Property_Result;
end Select_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Error_Stmt
   --
   --
   --  Primitives for Bare_Simple_Stmt
   --
   --
   --  Primitives for Bare_Abort_Stmt
   --
      procedure Initialize_Fields_For_Abort_Stmt
        (Self : Bare_Abort_Stmt
         ; Abort_Stmt_F_Names : Bare_Name_List
        ) is
      begin
            Self.Abort_Stmt_F_Names := Abort_Stmt_F_Names;
      end Initialize_Fields_For_Abort_Stmt;
   function Abort_Stmt_F_Names
     (Node : Bare_Abort_Stmt) return Bare_Name_List
   is
   begin
         return Node.Abort_Stmt_F_Names;
   end;
--# property-start AbortStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16576'
pragma Warnings (Off, "is not referenced");
 function Abort_Stmt_P_Xref_Equation
  (Node : Bare_Abort_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abort_Stmt  := Bare_Abort_Stmt (Node);
     Ent : Internal_Entity_Abort_Stmt :=
       Internal_Entity_Abort_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1400 with Inline_Always;
            procedure Finalizer_Scope_3547 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1400 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1400;
            procedure Finalizer_Scope_3547 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3547;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12030 '<LogicAll at nodes.lkt:16576:37>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16576'
--# expr-start 12029 '<Map at nodes.lkt:16576:37>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16576'
   --# expr-start 12024 '<FieldAccess for names at nodes.lkt:16576:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16576'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Abort_Stmt_F_Names, Info => Ent.Info);
--# expr-done 12024
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind name Item_0
      --# expr-start 12028 '<And at nodes.lkt:16577:19>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16577'
--# expr-start 12025 '<FieldAccess for sub_equation at nodes.lkt:16577:19>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16577'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 12025
--# expr-start 12027 '<Predicate on BaseTypeDecl.is_task_type at nodes.lkt:16577:43>' Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16577'
--# expr-start 12026 '<FieldAccess for type_var at nodes.lkt:16577:81>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16577'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_0.Node);
--# expr-done 12026
Fld_4.Value := No_Entity;
Entity_Vars.Reset (Fld_4);
Pred := Solver.Create_Predicate (Fld_4, Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate); 
--# expr-done 12027
And_Pred := Create_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16577:19") else null)); 
--# expr-done 12028
         declare
            Item_To_Append : constant Logic_Equation := And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3547;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12029
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16576:37") else null)); 
--# expr-done 12030
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1400;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1400;
                     Finalizer_Scope_3547;
            raise;
      end;
   return Property_Result;
end Abort_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Assign_Stmt
   --
      procedure Initialize_Fields_For_Assign_Stmt
        (Self : Bare_Assign_Stmt
         ; Assign_Stmt_F_Dest : Bare_Name
         ; Assign_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Assign_Stmt_F_Dest := Assign_Stmt_F_Dest;
            Self.Assign_Stmt_F_Expr := Assign_Stmt_F_Expr;
      end Initialize_Fields_For_Assign_Stmt;
   function Assign_Stmt_F_Dest
     (Node : Bare_Assign_Stmt) return Bare_Name
   is
   begin
         return Node.Assign_Stmt_F_Dest;
   end;
   function Assign_Stmt_F_Expr
     (Node : Bare_Assign_Stmt) return Bare_Expr
   is
   begin
         return Node.Assign_Stmt_F_Expr;
   end;
--# property-start AssignStmt.complete_item_weight 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16587'
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Complete_Item_Weight
  (Node : Bare_Assign_Stmt
      ; Item : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Assign_Stmt  := Bare_Assign_Stmt (Node);
     Ent : Internal_Entity_Assign_Stmt :=
       Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind item Item
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Name;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12032 '<FieldAccess for complete_item_weight_matching_type at nodes.lkt:16588:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16588'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12031 '<FieldAccess for dest at nodes.lkt:16588:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16588'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 12031
--# property-call-start AdaNode.complete_item_weight_matching_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Complete_Item_Weight_Matching_Type (Node => Self, Item => Item, Name => Fld, Origin => Origin);
--# end
--# expr-done 12032
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Assign_Stmt_P_Complete_Item_Weight;
--# end
--# property-start AssignStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16591'
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Xref_Equation
  (Node : Bare_Assign_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Assign_Stmt  := Bare_Assign_Stmt (Node);
     Ent : Internal_Entity_Assign_Stmt :=
       Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1402 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Bare_Name;
Fld_5 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1402 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1402;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12046 '<And at nodes.lkt:16591:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16591'
--# expr-start 12043 '<And at nodes.lkt:16592:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16592'
--# expr-start 12040 '<And at nodes.lkt:16593:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
--# expr-start 12034 '<FieldAccess for sub_equation at nodes.lkt:16593:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
--# expr-start 12033 '<FieldAccess for dest at nodes.lkt:16593:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 12033
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12034
--# expr-start 12039 '<Bind at nodes.lkt:16593:42>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
--# expr-start 12036 '<FieldAccess for type_var at nodes.lkt:16593:77>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
--# expr-start 12035 '<FieldAccess for dest at nodes.lkt:16593:77>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Assign_Stmt_F_Dest;
--# expr-done 12035
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 12036
Fld_5.Value := No_Entity;
Entity_Vars.Reset (Fld_5);
--# expr-start 12038 '<FieldAccess for expected_type_var at nodes.lkt:16593:46>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
--# expr-start 12037 '<FieldAccess for expr at nodes.lkt:16593:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16593'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Assign_Stmt_F_Expr;
--# expr-done 12037
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 12038
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Propagate (Fld_5, Fld_3, Logic_Functor_865'(Ref_Count => 1, Cache_Set => False, Cache_Key => <>, Cache_Value => <>, Origin => Origin), Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16593:42") else null)); 
--# expr-done 12039
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16593:13") else null)); 
--# expr-done 12040
--# expr-start 12042 '<FieldAccess for sub_equation at nodes.lkt:16594:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16594'
--# expr-start 12041 '<FieldAccess for expr at nodes.lkt:16594:15>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16594'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Assign_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 12041
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 12042
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16592:9") else null)); 
--# expr-done 12043
--# expr-start 12045 '<FieldAccess for matches_expected_assign_type at nodes.lkt:16595:11>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16595'
--# expr-start 12044 '<FieldAccess for expr at nodes.lkt:16595:11>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16595'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Self.Assign_Stmt_F_Expr;
--# expr-done 12044
      if Fld_8 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_9 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Fld_8, Origin => Origin);
--# end
--# expr-done 12045
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16591:37") else null)); 
--# expr-done 12046
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1402;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1402;
            raise;
      end;
   return Property_Result;
end Assign_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Call_Stmt
   --
      procedure Initialize_Fields_For_Call_Stmt
        (Self : Bare_Call_Stmt
         ; Call_Stmt_F_Call : Bare_Name
        ) is
      begin
            Self.Call_Stmt_F_Call := Call_Stmt_F_Call;
      end Initialize_Fields_For_Call_Stmt;
   function Call_Stmt_F_Call
     (Node : Bare_Call_Stmt) return Bare_Name
   is
   begin
         return Node.Call_Stmt_F_Call;
   end;
--# property-start CallStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16603'
pragma Warnings (Off, "is not referenced");
 function Call_Stmt_P_Xref_Equation
  (Node : Bare_Call_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Stmt  := Bare_Call_Stmt (Node);
     Ent : Internal_Entity_Call_Stmt :=
       Internal_Entity_Call_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1403 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1403 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1403;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12052 '<And at nodes.lkt:16603:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16603'
--# expr-start 12048 '<FieldAccess for sub_equation at nodes.lkt:16603:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16603'
--# expr-start 12047 '<FieldAccess for call at nodes.lkt:16603:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16603'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Stmt_F_Call, Info => Ent.Info);
--# expr-done 12047
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12048
--# expr-start 12051 '<Bind at nodes.lkt:16605:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16605'
--# expr-start 12050 '<FieldAccess for type_var at nodes.lkt:16605:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16605'
--# expr-start 12049 '<FieldAccess for call at nodes.lkt:16605:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16605'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Call_Stmt_F_Call;
--# expr-done 12049
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 12050
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
Bind_Result := Solver.Create_Assign (Fld_3, No_Entity, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16605:9") else null)); 
--# expr-done 12051
And_Pred := Create_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16603:37") else null)); 
--# expr-done 12052
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1403;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1403;
            raise;
      end;
   return Property_Result;
end Call_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Delay_Stmt
   --
      procedure Initialize_Fields_For_Delay_Stmt
        (Self : Bare_Delay_Stmt
         ; Delay_Stmt_F_Has_Until : Bare_Until_Node
         ; Delay_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Delay_Stmt_F_Has_Until := Delay_Stmt_F_Has_Until;
            Self.Delay_Stmt_F_Expr := Delay_Stmt_F_Expr;
      end Initialize_Fields_For_Delay_Stmt;
   function Delay_Stmt_F_Has_Until
     (Node : Bare_Delay_Stmt) return Bare_Until_Node
   is
   begin
         return Node.Delay_Stmt_F_Has_Until;
   end;
   function Delay_Stmt_F_Expr
     (Node : Bare_Delay_Stmt) return Bare_Expr
   is
   begin
         return Node.Delay_Stmt_F_Expr;
   end;
--# property-start DelayStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16615'
pragma Warnings (Off, "is not referenced");
 function Delay_Stmt_P_Xref_Equation
  (Node : Bare_Delay_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delay_Stmt  := Bare_Delay_Stmt (Node);
     Ent : Internal_Entity_Delay_Stmt :=
       Internal_Entity_Delay_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1404 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Until_Node;
Fld_8 : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1404 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1404;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12066 '<And at nodes.lkt:16615:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16615'
--# expr-start 12054 '<FieldAccess for sub_equation at nodes.lkt:16615:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16615'
--# expr-start 12053 '<FieldAccess for expr at nodes.lkt:16615:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16615'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 12053
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12054
--# expr-start 12065 '<If at nodes.lkt:16616:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12056 '<FieldAccess for as_bool at nodes.lkt:16616:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12055 '<FieldAccess for has_until at nodes.lkt:16616:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Self.Delay_Stmt_F_Has_Until;
--# expr-done 12055
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Until.as_bool'
Fld_8 := Libadalang.Implementation.Dispatcher_Until_Node_P_As_Bool (Node => Fld_7);
--# end
--# expr-done 12056
if Fld_8 then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16616:42") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 12064 '<And at nodes.lkt:16616:53>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12061 '<Bind at nodes.lkt:16616:53>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12058 '<FieldAccess for expected_type_var at nodes.lkt:16616:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12057 '<FieldAccess for expr at nodes.lkt:16616:57>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Delay_Stmt_F_Expr;
--# expr-done 12057
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_2);
--# expr-done 12058
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 12060 '<FieldAccess for std_entity at nodes.lkt:16616:88>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12059 '<Symbol Duration at nodes.lkt:16616:104>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Duration); 
--# expr-done 12059
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 12060
Bind_Result := Solver.Create_Assign (Fld_3, Fld_4, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16616:53") else null)); 
--# expr-done 12061
--# expr-start 12063 '<FieldAccess for matches_expected_type at nodes.lkt:16616:122>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
--# expr-start 12062 '<FieldAccess for expr at nodes.lkt:16616:122>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16616'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 12062
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 12063
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16616:53") else null)); 
--# expr-done 12064
   If_Result := And_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 12065
And_Pred_1 := Create_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16615:37") else null)); 
--# expr-done 12066
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1404;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1404;
            raise;
      end;
   return Property_Result;
end Delay_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Exit_Stmt
   --
      procedure Initialize_Fields_For_Exit_Stmt
        (Self : Bare_Exit_Stmt
         ; Exit_Stmt_F_Loop_Name : Bare_Name
         ; Exit_Stmt_F_Cond_Expr : Bare_Expr
        ) is
      begin
            Self.Exit_Stmt_F_Loop_Name := Exit_Stmt_F_Loop_Name;
            Self.Exit_Stmt_F_Cond_Expr := Exit_Stmt_F_Cond_Expr;
      end Initialize_Fields_For_Exit_Stmt;
   function Exit_Stmt_F_Loop_Name
     (Node : Bare_Exit_Stmt) return Bare_Name
   is
   begin
         return Node.Exit_Stmt_F_Loop_Name;
   end;
   function Exit_Stmt_F_Cond_Expr
     (Node : Bare_Exit_Stmt) return Bare_Expr
   is
   begin
         return Node.Exit_Stmt_F_Cond_Expr;
   end;
--# property-start ExitStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16626'
pragma Warnings (Off, "is not referenced");
 function Exit_Stmt_P_Xref_Equation
  (Node : Bare_Exit_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exit_Stmt  := Bare_Exit_Stmt (Node);
     Ent : Internal_Entity_Exit_Stmt :=
       Internal_Entity_Exit_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1405 with Inline_Always;
            procedure Finalizer_Scope_3548 with Inline_Always;
            procedure Finalizer_Scope_3549 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1405 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1405;
            procedure Finalizer_Scope_3548 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3548;
            procedure Finalizer_Scope_3549 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3549;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12073 '<And at nodes.lkt:16626:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16626'
--# expr-start 12067 '<FieldAccess for cond_expr at nodes.lkt:16626:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16626'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Exit_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 12067
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind cond Var_Expr_0
--# expr-start 12070 '<And at nodes.lkt:16627:19>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16627'
--# expr-start 12068 '<FieldAccess for sub_equation at nodes.lkt:16627:19>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16627'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12068
--# expr-start 12069 '<FieldAccess for expect_bool_derived_type at nodes.lkt:16627:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16627'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.expect_bool_derived_type
Fld_2 := Libadalang.Implementation.Expr_P_Expect_Bool_Derived_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 12069
And_Pred := Create_And (Fld_1, Fld_2, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16627:19") else null)); 
--# expr-done 12070
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3548;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16627:88") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 12071 '<FieldAccess for loop_name at nodes.lkt:16628:11>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16628'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Exit_Stmt_F_Loop_Name, Info => Ent.Info);
--# expr-done 12071
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind ln Var_Expr_1
--# expr-start 12072 '<FieldAccess for xref_no_overloading at nodes.lkt:16629:17>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16629'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_4 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 12072
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3549;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16629:55") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16626:37") else null)); 
--# expr-done 12073
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1405;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1405;
                     Finalizer_Scope_3548;
                     Finalizer_Scope_3549;
            raise;
      end;
   return Property_Result;
end Exit_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Goto_Stmt
   --
      procedure Initialize_Fields_For_Goto_Stmt
        (Self : Bare_Goto_Stmt
         ; Goto_Stmt_F_Label_Name : Bare_Name
        ) is
      begin
            Self.Goto_Stmt_F_Label_Name := Goto_Stmt_F_Label_Name;
      end Initialize_Fields_For_Goto_Stmt;
   function Goto_Stmt_F_Label_Name
     (Node : Bare_Goto_Stmt) return Bare_Name
   is
   begin
         return Node.Goto_Stmt_F_Label_Name;
   end;
--# property-start GotoStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16638'
pragma Warnings (Off, "is not referenced");
 function Goto_Stmt_P_Xref_Equation
  (Node : Bare_Goto_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Goto_Stmt  := Bare_Goto_Stmt (Node);
     Ent : Internal_Entity_Goto_Stmt :=
       Internal_Entity_Goto_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1406 with Inline_Always;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1406 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1406;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12075 '<FieldAccess for xref_no_overloading at nodes.lkt:16639:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16639'
--# expr-start 12074 '<FieldAccess for label_name at nodes.lkt:16639:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16639'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Goto_Stmt_F_Label_Name, Info => Ent.Info);
--# expr-done 12074
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => False, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12075
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1406;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1406;
            raise;
      end;
   return Property_Result;
end Goto_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Label
   --
      procedure Initialize_Fields_For_Label
        (Self : Bare_Label
         ; Label_F_Decl : Bare_Label_Decl
        ) is
      begin
            Self.Label_F_Decl := Label_F_Decl;
      end Initialize_Fields_For_Label;
   function Label_F_Decl
     (Node : Bare_Label) return Bare_Label_Decl
   is
   begin
         return Node.Label_F_Decl;
   end;
--# property-start Label.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16647'
pragma Warnings (Off, "is not referenced");
 function Label_P_Xref_Equation
  (Node : Bare_Label
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Label  := Bare_Label (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1407 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1407 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1407;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16647:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1407;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1407;
            raise;
      end;
   return Property_Result;
end Label_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Stmt
   --
--# property-start NullStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16653'
pragma Warnings (Off, "is not referenced");
 function Null_Stmt_P_Xref_Equation
  (Node : Bare_Null_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Stmt  := Bare_Null_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1408 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1408 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1408;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16653:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1408;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1408;
            raise;
      end;
   return Property_Result;
end Null_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Stmt
   --
      procedure Initialize_Fields_For_Raise_Stmt
        (Self : Bare_Raise_Stmt
         ; Raise_Stmt_F_Exception_Name : Bare_Name
         ; Raise_Stmt_F_Error_Message : Bare_Expr
        ) is
      begin
            Self.Raise_Stmt_F_Exception_Name := Raise_Stmt_F_Exception_Name;
            Self.Raise_Stmt_F_Error_Message := Raise_Stmt_F_Error_Message;
      end Initialize_Fields_For_Raise_Stmt;
   function Raise_Stmt_F_Exception_Name
     (Node : Bare_Raise_Stmt) return Bare_Name
   is
   begin
         return Node.Raise_Stmt_F_Exception_Name;
   end;
   function Raise_Stmt_F_Error_Message
     (Node : Bare_Raise_Stmt) return Bare_Expr
   is
   begin
         return Node.Raise_Stmt_F_Error_Message;
   end;
--# property-start RaiseStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16662'
pragma Warnings (Off, "is not referenced");
 function Raise_Stmt_P_Xref_Equation
  (Node : Bare_Raise_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Stmt  := Bare_Raise_Stmt (Node);
     Ent : Internal_Entity_Raise_Stmt :=
       Internal_Entity_Raise_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1409 with Inline_Always;
            procedure Finalizer_Scope_3550 with Inline_Always;
            procedure Finalizer_Scope_3551 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
            procedure Finalizer_Scope_1409 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1409;
            procedure Finalizer_Scope_3550 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3550;
            procedure Finalizer_Scope_3551 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3551;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12084 '<And at nodes.lkt:16662:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16662'
--# expr-start 12076 '<FieldAccess for exception_name at nodes.lkt:16662:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16662'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Stmt_F_Exception_Name, Info => Ent.Info);
--# expr-done 12076
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Name then
   --# scope-start
   --# bind en Var_Expr_0
--# expr-start 12077 '<FieldAccess for sub_equation at nodes.lkt:16663:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16663'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12077
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3550;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16663:48") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
--# expr-start 12078 '<FieldAccess for error_message at nodes.lkt:16664:11>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16664'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Stmt_F_Error_Message, Info => Ent.Info);
--# expr-done 12078
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr_1
--# expr-start 12083 '<And at nodes.lkt:16665:17>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16665'
--# expr-start 12081 '<Bind at nodes.lkt:16668:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16668'
--# expr-start 12079 '<FieldAccess for expected_type_var at nodes.lkt:16668:17>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16668'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 12079
Fld_3.Value := No_Entity;
Entity_Vars.Reset (Fld_3);
--# expr-start 12080 '<FieldAccess for std_string_type at nodes.lkt:16668:41>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16668'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_string_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_String_Type (Node => Self);
--# end
--# expr-done 12080
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Bind_Result := Solver.Create_Assign (Fld_3, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16668:13") else null)); 
--# expr-done 12081
--# expr-start 12082 '<FieldAccess for sub_equation at nodes.lkt:16669:15>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16669'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 12082
And_Pred := Create_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16665:17") else null)); 
--# expr-done 12083
Scope_Result_1 := And_Pred;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3551;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16669:46") else null)); 
   Result_Var_1 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Create_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16662:37") else null)); 
--# expr-done 12084
         Property_Result := And_Pred_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1409;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1409;
                     Finalizer_Scope_3550;
                     Finalizer_Scope_3551;
            raise;
      end;
   return Property_Result;
end Raise_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Requeue_Stmt
   --
      procedure Initialize_Fields_For_Requeue_Stmt
        (Self : Bare_Requeue_Stmt
         ; Requeue_Stmt_F_Call_Name : Bare_Name
         ; Requeue_Stmt_F_Has_Abort : Bare_Abort_Node
        ) is
      begin
            Self.Requeue_Stmt_F_Call_Name := Requeue_Stmt_F_Call_Name;
            Self.Requeue_Stmt_F_Has_Abort := Requeue_Stmt_F_Has_Abort;
      end Initialize_Fields_For_Requeue_Stmt;
   function Requeue_Stmt_F_Call_Name
     (Node : Bare_Requeue_Stmt) return Bare_Name
   is
   begin
         return Node.Requeue_Stmt_F_Call_Name;
   end;
   function Requeue_Stmt_F_Has_Abort
     (Node : Bare_Requeue_Stmt) return Bare_Abort_Node
   is
   begin
         return Node.Requeue_Stmt_F_Has_Abort;
   end;
--# property-start RequeueStmt.innermost_entry_or_accept_stmt_params 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16678'
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params
  (Node : Bare_Requeue_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Requeue_Stmt  := Bare_Requeue_Stmt (Node);
     Ent : Internal_Entity_Requeue_Stmt :=
       Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1410 with Inline_Always;
      Match_0 : Internal_Entity_Accept_Stmt_With_Stmts;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Let_Result : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Entry_Body;
Let_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1410 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1410;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12091 '<Cast to BaseFormalParamHolder at nodes.lkt:16678:82>' Cast_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16678'
--# expr-start 12090 '<Match at nodes.lkt:16678:82>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16678'
--# expr-start 12087 '<Find at nodes.lkt:16678:88>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16678'
   --# expr-start 12085 '<FieldAccess for parents at nodes.lkt:16678:88>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16678'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 12085
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 12086 '<IsA AcceptStmtWithStmts, EntryBody at nodes.lkt:16679:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16679'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range | Ada_Entry_Body_Range; 
--# expr-done 12086
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 12087
      if Find_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Find_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_With_Stmts =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 12088 '<FieldAccess for params at nodes.lkt:16681:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16681'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_0.Node.Accept_Stmt_F_Params, Info => Match_0.Info);
--# expr-done 12088
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 12089 '<FieldAccess for params at nodes.lkt:16682:30>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16682'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 12089
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Accept_Stmt | Ada_For_Loop_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Entry_Completion_Formal_Params; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 12090
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Result.Node,
         Info => Match_Result.Info);
--# expr-done 12091
         Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1410;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1410;
            raise;
      end;
   return Property_Result;
end Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params;
--# end
--# property-start RequeueStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16687'
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Xref_Equation
  (Node : Bare_Requeue_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Requeue_Stmt  := Bare_Requeue_Stmt (Node);
     Ent : Internal_Entity_Requeue_Stmt :=
       Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1411 with Inline_Always;
            procedure Finalizer_Scope_3563 with Inline_Always;
            procedure Finalizer_Scope_3564 with Inline_Always;
            procedure Finalizer_Scope_3565 with Inline_Always;
            procedure Finalizer_Scope_3570 with Inline_Always;
            procedure Finalizer_Scope_3571 with Inline_Always;
      Local_Ce : Internal_Entity_Call_Expr;
Local_Name : Internal_Entity_Name;
Var_Expr_0 : Internal_Entity_Call_Expr;
Local_Targets : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Base_Subp_Spec;
Var_Expr_4 : Internal_Entity_Base_Subp_Spec;
Var_Expr_5 : Internal_Entity_Base_Subp_Spec;
Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Name;
Local_Fam_Type : Internal_Entity_Base_Type_Decl;
Local_First_Param : Internal_Entity_Expr;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Var_Expr_282 : Internal_Entity_Name;
Var_Expr_283 : Internal_Entity_Entry_Decl;
Var_Expr_284 : Internal_Entity_Subtype_Indication;
Var_Expr_285 : Internal_Entity_Call_Expr;
Var_Expr_286 : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Integer;
Is_Equal : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Fld_8 : Internal_Entity_Base_Formal_Param_Holder;
Is_A : Boolean;
Not_Val : Boolean;
Fld_9 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Formal_Param_Holder;
Fld_12 : Internal_Entity_Info;
Fld_13 : Internal_Metadata;
Fld_14 : Boolean;
Fld_15 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Sym : Symbol_Type;
Fld_16 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Sym_1 : Symbol_Type;
Fld_17 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_18 : Internal_Entity_Name;
Fld_19 : Logic_Equation;
Fld_20 : Internal_Entity_Name;
Fld_21 : Logic_Equation;
Fld_22 : Internal_Entity_Name;
Is_A_1 : Boolean;
If_Result_4 : Logic_Equation;
Fld_23 : Internal_Entity_Name;
Fld_24 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_25 : Internal_Entity_Base_Id;
Fld_26 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_27 : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Entry_Decl;
Fld_28 : Internal_Entity_Entry_Spec;
Fld_29 : Internal_Entity;
Scope_Result_8 : Internal_Entity;
Result_Var_6 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Subtype_Indication;
Fld_30 : Internal_Entity_Base_Type_Decl;
Scope_Result_9 : Internal_Entity_Base_Type_Decl;
Result_Var_7 : Internal_Entity_Base_Type_Decl;
Fld_31 : Internal_Entity_Assoc_List;
Fld_32 : Bare_Assoc_List;
Fld_33 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_5 : Internal_Entity_Basic_Assoc;
Fld_34 : Internal_Entity_Expr;
Scope_Result_10 : Internal_Entity_Expr;
Result_Var_8 : Internal_Entity_Expr;
Scope_Result_11 : Internal_Entity_Expr;
Result_Var_9 : Internal_Entity_Expr;
Fld_35 : Logic_Equation;
Fld_36 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_37 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result_12 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_10 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_13 : Logic_Equation;
True_Rel_1 : Logic_Equation;
Result_Var_11 : Logic_Equation;
Let_Result_2 : Logic_Equation;
Let_Result_3 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result_4 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Let_Result_6 : Logic_Equation;
            procedure Finalizer_Scope_1411 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Local_Targets);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_1411;
            procedure Finalizer_Scope_3563 is
            begin
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_21);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_26);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_3563;
            procedure Finalizer_Scope_3564 is
            begin
                     Dec_Ref (Fld_27);
            end Finalizer_Scope_3564;
            procedure Finalizer_Scope_3565 is
            begin
                     Dec_Ref (Scope_Result_13);
                     Dec_Ref (True_Rel_1);
                     Dec_Ref (Result_Var_11);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_3565;
            procedure Finalizer_Scope_3570 is
            begin
                     Dec_Ref (Fld_35);
                     Dec_Ref (Scope_Result_12);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_10);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_3570;
            procedure Finalizer_Scope_3571 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3571;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12156 '<Let at nodes.lkt:16688:9>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16688'
--# scope-start
--# expr-start 12093 '<Cast to CallExpr at nodes.lkt:16688:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16688'
--# expr-start 12092 '<FieldAccess for call_name at nodes.lkt:16688:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16688'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 12092
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 12093
Local_Ce := Cast_Result; 
--# bind ce Local_Ce
--# expr-start 12155 '<Let at nodes.lkt:16689:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16689'
--# scope-start
Var_Expr_0 := Local_Ce;
if Var_Expr_0 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_0
--# expr-start 12094 '<FieldAccess for name at nodes.lkt:16690:21>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16690'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Call_Expr_F_Name, Info => Var_Expr_0.Info);
--# expr-done 12094
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12095 '<FieldAccess for call_name at nodes.lkt:16690:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16690'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 12095
   Result_Var := Fld_2;
end if;
Local_Name := Result_Var; 
--# bind name Local_Name
--# expr-start 12154 '<Let at nodes.lkt:16692:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16692'
--# scope-start
--# expr-start 12120 '<Map at nodes.lkt:16692:23>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16692'
   --# expr-start 12096 '<FieldAccess for all_env_elements_internal at nodes.lkt:16692:23>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16692'
      if Local_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_3 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Local_Name.Node, E_Info => Local_Name.Info);
--# end
--# expr-done 12096
      declare
         Map_Result_Vec : Internal_Entity_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_1
--# expr-start 12097 '<Cast to BasicDecl at nodes.lkt:16693:20>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16693'
Cast_Expr := Item_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 12097
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_2
--# expr-start 12098 '<FieldAccess for subp_spec_or_null at nodes.lkt:16698:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16698'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 12098
Var_Expr_3 := Fld_4;
if Var_Expr_3 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_3
--# expr-start 12100 '<Eq at nodes.lkt:16699:33>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16699'
--# expr-start 12099 '<FieldAccess for nb_max_params at nodes.lkt:16699:33>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16699'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Var_Expr_3.Node);
--# end
--# expr-done 12099
Is_Equal := Fld_5 = 0; 
--# expr-done 12100
if Is_Equal then
   --# expr-start 12102 '<IsNull at nodes.lkt:16699:61>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16699'
--# expr-start 12101 '<FieldAccess for returns at nodes.lkt:16699:61>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16699'
      if Var_Expr_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 12101
Is_Null := Fld_6.Node = null; 
--# expr-done 12102
   If_Result := Is_Null;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result_1 := True;
else
--# expr-start 12103 '<FieldAccess for subp_spec_or_null at nodes.lkt:16704:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16704'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 12103
Var_Expr_4 := Fld_7;
if Var_Expr_4 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_4
--# expr-start 12107 '<FieldAccess for match_formal_params at nodes.lkt:16705:33>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16705'
      if Var_Expr_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12104 '<FieldAccess for innermost_entry_or_accept_stmt_params at nodes.lkt:16706:29>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16706'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start RequeueStmt.innermost_entry_or_accept_stmt_params
Fld_8 := Libadalang.Implementation.Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12104
--# expr-start 12106 '<Not at nodes.lkt:16706:87>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16706'
--# expr-start 12105 '<IsA SubpRenamingDecl at nodes.lkt:16706:91>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16706'
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Subp_Renaming_Decl_Range; 
--# expr-done 12105
Not_Val := not (Is_A); 
--# expr-done 12106
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Var_Expr_4.Node, Other => Fld_8, Match_Names => Not_Val, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 12107
Scope_Result_2 := Fld_9;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_1 := Result_Var_2;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
   --# expr-start 12119 '<If at nodes.lkt:16716:21>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
--# expr-start 12110 '<Cast to Name at nodes.lkt:16716:24>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
--# expr-start 12109 '<FieldAccess for get_aspect_spec_expr at nodes.lkt:16716:24>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12108 '<Symbol Synchronization at nodes.lkt:16716:47>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Synchronization); 
--# expr-done 12108
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr_2.Node, Name => Sym, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 12109
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 12110
Var_Expr_282 := Cast_Result_2;
if Var_Expr_282 /= No_Entity_Name then
   --# scope-start
--# expr-start 12112 '<FieldAccess for name_is at nodes.lkt:16716:24>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
      if Var_Expr_282.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12111 '<Symbol By_Entry at nodes.lkt:16716:85>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_By_Entry); 
--# expr-done 12111
--# property-call-start Name.name_is
Fld_17 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_282.Node, Sym => Sym_1);
--# end
--# expr-done 12112
Scope_Result_4 := Fld_17;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
--# expr-start 12113 '<FieldAccess for subp_spec_or_null at nodes.lkt:16716:103>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16716'
      if Var_Expr_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 12113
Var_Expr_5 := Fld_10;
if Var_Expr_5 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_5
--# expr-start 12118 '<FieldAccess for match_formal_params at nodes.lkt:16717:33>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16717'
      if Var_Expr_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12114 '<FieldAccess for innermost_entry_or_accept_stmt_params at nodes.lkt:16718:29>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16718'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start RequeueStmt.innermost_entry_or_accept_stmt_params
Fld_11 := Libadalang.Implementation.Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12114
--# expr-start 12117 '<FieldAccess for dottable_subp at nodes.lkt:16718:113>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16718'
--# expr-start 12116 '<FieldAccess for md at nodes.lkt:16718:113>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16718'
--# expr-start 12115 '<FieldAccess for info at nodes.lkt:16718:113>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16718'
Fld_12 := Var_Expr_2.Info;
--# expr-done 12115
Fld_13 := Fld_12.Md;
--# expr-done 12116
Fld_14 := Fld_13.Dottable_Subp;
--# expr-done 12117
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Var_Expr_5.Node, Other => Fld_11, Match_Names => False, Ignore_First_Param => Fld_14, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 12118
Scope_Result_3 := Fld_15;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_2 := Result_Var_3;
else
   If_Result_2 := False;
end if;
--# expr-done 12119
   If_Result_3 := If_Result_2;
end if;
Scope_Result_5 := If_Result_3;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
         if Result_Var_5 then
         declare
            Item_To_Append : constant Internal_Entity := Item_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Array
           (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12120
Local_Targets := Map_Result; Inc_Ref (Local_Targets);
--# bind targets Local_Targets
--# expr-start 12153 '<And at nodes.lkt:16725:9>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16725'
--# expr-start 12135 '<Match at nodes.lkt:16728:13>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16728'
      if Local_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Local_Name; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind dn Match_0
--# expr-start 12133 '<And at nodes.lkt:16731:40>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16731'
--# expr-start 12127 '<If at nodes.lkt:16732:21>' If_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
--# expr-start 12122 '<IsA CallExpr at nodes.lkt:16732:24>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
--# expr-start 12121 '<FieldAccess for prefix at nodes.lkt:16732:24>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_22 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 12121
Is_A_1 := Fld_22.Node /= null 
and then Fld_22.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 12122
if Is_A_1 then
   --# expr-start 12124 '<FieldAccess for sub_equation at nodes.lkt:16732:51>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
--# expr-start 12123 '<FieldAccess for prefix at nodes.lkt:16732:51>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_18 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 12123
      if Fld_18.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_18.Info);
--# end
--# expr-done 12124
   If_Result_4 := Fld_19;
else
   --# expr-start 12126 '<FieldAccess for xref_no_overloading at nodes.lkt:16732:81>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
--# expr-start 12125 '<FieldAccess for prefix at nodes.lkt:16732:81>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16732'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 12125
      if Fld_20.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_21 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_20.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_20.Info);
--# end
--# expr-done 12126
   If_Result_4 := Fld_21;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 12127
--# expr-start 12132 '<bind at nodes.lkt:16734:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16734'
--# expr-start 12129 '<FieldAccess for designated_env at nodes.lkt:16734:32>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16734'
--# expr-start 12128 '<FieldAccess for prefix at nodes.lkt:16734:32>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16734'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 12128
      if Fld_23.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_24 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_23.Node, Env => Env, Origin => Origin, No_Visibility => False, E_Info => Fld_23.Info);
--# end
--# expr-done 12129
Bound_Env := Fld_24; Inc_Ref (Bound_Env);
--# expr-start 12131 '<FieldAccess for xref_no_overloading at nodes.lkt:16736:21>' Fld_26 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16736'
--# expr-start 12130 '<FieldAccess for suffix at nodes.lkt:16736:21>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16736'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_25 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 12130
      if Fld_25.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_26 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_25.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_25.Info);
--# end
--# expr-done 12131
Dyn_Var_Bind_Result := Fld_26; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12132
And_Pred := Create_And (If_Result_4, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16731:40") else null)); 
--# expr-done 12133
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_6 := Let_Result;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_3563;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Array_Subcomponent_Choice_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind o Match_1
--# expr-start 12134 '<FieldAccess for xref_no_overloading at nodes.lkt:16738:27>' Fld_27 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16738'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_27 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 12134
Let_Result_1 := Fld_27; Inc_Ref (Let_Result_1);
--# end
Scope_Result_7 := Let_Result_1;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_3564;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
end case;
--# expr-done 12135
--# expr-start 12152 '<LogicAny at nodes.lkt:16742:13>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16742'
--# expr-start 12151 '<Map at nodes.lkt:16742:13>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16742'
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Local_Targets;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_6
      --# expr-start 12150 '<Let at nodes.lkt:16747:21>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
--# scope-start
--# expr-start 12139 '<Cast to SubtypeIndication at nodes.lkt:16747:36>' Cast_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
--# expr-start 12136 '<Cast to EntryDecl at nodes.lkt:16747:36>' Cast_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
Cast_Expr_1 := Item_6; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_5 := No_Entity_Entry_Decl;
   end if;
--# expr-done 12136
Var_Expr_283 := Cast_Result_5;
if Var_Expr_283 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 12138 '<FieldAccess for family_type at nodes.lkt:16747:36>' Fld_29 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
--# expr-start 12137 '<FieldAccess for spec at nodes.lkt:16747:36>' Fld_28 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
      if Var_Expr_283.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_28 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_283.Node.Entry_Decl_F_Spec, Info => Var_Expr_283.Info);
--# expr-done 12137
      if Fld_28.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_29 := Create_Internal_Entity (Node => Fld_28.Node.Entry_Spec_F_Family_Type, Info => Fld_28.Info);
--# expr-done 12138
Scope_Result_8 := Fld_29;
   --# end
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Entity;
end if;
   if Result_Var_6.Node = null
      or else Result_Var_6.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Subtype_Indication
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
   else
         Cast_Result_6 := No_Entity_Subtype_Indication;
   end if;
--# expr-done 12139
Var_Expr_284 := Cast_Result_6;
if Var_Expr_284 /= No_Entity_Subtype_Indication then
   --# scope-start
--# expr-start 12140 '<FieldAccess for designated_type at nodes.lkt:16747:36>' Fld_30 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16747'
      if Var_Expr_284.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_30 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_284.Node, Origin => Origin, E_Info => Var_Expr_284.Info);
--# end
--# expr-done 12140
Scope_Result_9 := Fld_30;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := No_Entity_Base_Type_Decl;
end if;
Local_Fam_Type := Result_Var_7; 
--# bind fam_type Local_Fam_Type
--# expr-start 12149 '<Let at nodes.lkt:16748:21>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16748'
--# scope-start
Var_Expr_285 := Local_Ce;
if Var_Expr_285 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 12141 '<FieldAccess for params at nodes.lkt:16748:39>' Fld_31 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16748'
      if Var_Expr_285.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CallExpr.params
Fld_31 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_285.Node, E_Info => Var_Expr_285.Info);
--# end
--# expr-done 12141
Fld_32 := Fld_31.Node;
Get_Result := Get (Self, Fld_32, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_5 := No_Entity_Basic_Assoc;
else
Fld_33 := Fld_31.Info;
As_Entity := (Info => Fld_33, Node => Get_Result); 
   If_Result_5 := As_Entity;
end if;
Var_Expr_286 := If_Result_5;
if Var_Expr_286 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 12142 '<FieldAccess for expr at nodes.lkt:16748:39>' Fld_34 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16748'
      if Var_Expr_286.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_286.Node, E_Info => Var_Expr_286.Info);
--# end
--# expr-done 12142
Scope_Result_10 := Fld_34;
   --# end
   Result_Var_8 := Scope_Result_10;
else
   Result_Var_8 := No_Entity_Expr;
end if;
Scope_Result_11 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_11;
else
   Result_Var_9 := No_Entity_Expr;
end if;
Local_First_Param := Result_Var_9; 
--# bind first_param Local_First_Param
Var_Expr_7 := Local_First_Param;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind p Var_Expr_7
--# expr-start 12148 '<And at nodes.lkt:16751:32>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16751'
--# expr-start 12143 '<FieldAccess for sub_equation at nodes.lkt:16751:32>' Fld_35 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16751'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_35 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 12143
Var_Expr_8 := Local_Fam_Type;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind eft Var_Expr_8
--# expr-start 12147 '<And at nodes.lkt:16752:38>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16752'
--# expr-start 12145 '<Bind at nodes.lkt:16752:38>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16752'
--# expr-start 12144 '<FieldAccess for expected_type_var at nodes.lkt:16752:42>' Fld_36 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16752'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_7.Node);
--# expr-done 12144
Fld_36.Value := No_Entity;
Entity_Vars.Reset (Fld_36);
Cast_Expr_2 := Var_Expr_8; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Bind_Result := Solver.Create_Assign (Fld_36, Cast_Result_7, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16752:38") else null)); 
--# expr-done 12145
--# expr-start 12146 '<FieldAccess for matches_expected_type at nodes.lkt:16752:74>' Fld_37 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16752'
      if Var_Expr_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_37 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Var_Expr_7.Node, Origin => Origin);
--# end
--# expr-done 12146
And_Pred_1 := Create_And (Bind_Result, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16752:38") else null)); 
--# expr-done 12147
Scope_Result_12 := And_Pred_1;
   Inc_Ref (Scope_Result_12);
   --# end
      Finalizer_Scope_3571;
   Result_Var_10 := Scope_Result_12;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16752:113") else null)); 
   Result_Var_10 := True_Rel;
end if;
      Inc_Ref (Result_Var_10);
And_Pred_2 := Create_And (Fld_35, Result_Var_10, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16751:32") else null)); 
--# expr-done 12148
Scope_Result_13 := And_Pred_2;
   Inc_Ref (Scope_Result_13);
   --# end
      Finalizer_Scope_3570;
   Result_Var_11 := Scope_Result_13;
else
   True_Rel_1 := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16753:40") else null)); 
   Result_Var_11 := True_Rel_1;
end if;
      Inc_Ref (Result_Var_11);
Let_Result_2 := Result_Var_11; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12149
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 12150
         declare
            Item_To_Append : constant Logic_Equation := Let_Result_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3565;
         end loop;
      end;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 12151
Logic_Boolean_Op := Solver.Create_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16742:13") else null)); 
--# expr-done 12152
And_Pred_3 := Create_And (Match_Result, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16725:9") else null)); 
--# expr-done 12153
Let_Result_4 := And_Pred_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 12154
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 12155
Let_Result_6 := Let_Result_5; Inc_Ref (Let_Result_6);
--# end
--# expr-done 12156
         Property_Result := Let_Result_6;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1411;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1411;
                     Finalizer_Scope_3563;
                     Finalizer_Scope_3564;
                     Finalizer_Scope_3565;
                     Finalizer_Scope_3570;
                     Finalizer_Scope_3571;
            raise;
      end;
   return Property_Result;
end Requeue_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Return_Stmt
   --
      procedure Initialize_Fields_For_Return_Stmt
        (Self : Bare_Return_Stmt
         ; Return_Stmt_F_Return_Expr : Bare_Expr
        ) is
      begin
            Self.Return_Stmt_F_Return_Expr := Return_Stmt_F_Return_Expr;
      end Initialize_Fields_For_Return_Stmt;
   function Return_Stmt_F_Return_Expr
     (Node : Bare_Return_Stmt) return Bare_Expr
   is
   begin
         return Node.Return_Stmt_F_Return_Expr;
   end;
--# property-start ReturnStmt.subp 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16766'
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Subp
  (Node : Bare_Return_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Subp_Body
is
   Self : Bare_Return_Stmt  := Bare_Return_Stmt (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1412 with Inline_Always;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
As_Entity : Internal_Entity_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1412 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_1412;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12161 '<as_entity at nodes.lkt:16767:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16767'
--# expr-start 12160 '<Cast to SubpBody at nodes.lkt:16767:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16767'
--# expr-start 12159 '<Find at nodes.lkt:16767:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16767'
   --# expr-start 12157 '<FieldAccess for parents at nodes.lkt:16767:9>' Node_Parents 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16767'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Node_Parents := Parents (Self, True); 
--# expr-done 12157
      Find_Result := No_Bare_Ada_Node;
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 12158 '<IsA SubpBody at nodes.lkt:16767:36>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16767'
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Subp_Body_Range; 
--# expr-done 12158
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 12159
   if Find_Result = null
      or else Find_Result.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Find_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12160
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subp_Body;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 12161
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1412;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1412;
            raise;
      end;
   return Property_Result;
end Return_Stmt_P_Subp;
--# end
--# property-start ReturnStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16770'
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Xref_Equation
  (Node : Bare_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Return_Stmt  := Bare_Return_Stmt (Node);
     Ent : Internal_Entity_Return_Stmt :=
       Internal_Entity_Return_Stmt'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1413 with Inline_Always;
            procedure Finalizer_Scope_3573 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Subp_Body;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Internal_Entity_Type_Expr;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_1413 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1413;
            procedure Finalizer_Scope_3573 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_3573;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12162 '<FieldAccess for return_expr at nodes.lkt:16770:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16770'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Return_Stmt_F_Return_Expr, Info => Ent.Info);
--# expr-done 12162
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Expr then
   --# scope-start
   --# bind rexpr Var_Expr_0
--# expr-start 12172 '<And at nodes.lkt:16771:20>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16771'
--# expr-start 12170 '<And at nodes.lkt:16772:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
--# expr-start 12168 '<Bind at nodes.lkt:16772:13>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
--# expr-start 12163 '<FieldAccess for expected_type_var at nodes.lkt:16772:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Var_Expr_0.Node);
--# expr-done 12163
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 12167 '<FieldAccess for designated_type at nodes.lkt:16772:44>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
--# expr-start 12166 '<FieldAccess for returns at nodes.lkt:16772:44>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
--# expr-start 12165 '<FieldAccess for subp_spec at nodes.lkt:16772:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
--# expr-start 12164 '<FieldAccess for subp at nodes.lkt:16772:44>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ReturnStmt.subp
Fld_2 := Libadalang.Implementation.Return_Stmt_P_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12164
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Fld_2.Node.Base_Subp_Body_F_Subp_Spec, Info => Fld_2.Info);
--# expr-done 12165
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 12166
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 12167
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16772:13") else null)); 
--# expr-done 12168
--# expr-start 12169 '<FieldAccess for sub_equation at nodes.lkt:16772:99>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16772'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12169
And_Pred := Create_And (Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16772:13") else null)); 
--# expr-done 12170
--# expr-start 12171 '<FieldAccess for matches_expected_assign_type at nodes.lkt:16773:15>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16773'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_assign_type
Fld_7 := Libadalang.Implementation.Expr_P_Matches_Expected_Assign_Type (Node => Var_Expr_0.Node, Origin => Origin);
--# end
--# expr-done 12171
And_Pred_1 := Create_And (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16771:20") else null)); 
--# expr-done 12172
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3573;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16773:65") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1413;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1413;
                     Finalizer_Scope_3573;
            raise;
      end;
   return Property_Result;
end Return_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Simple_Decl_Stmt
   --
      procedure Initialize_Fields_For_Simple_Decl_Stmt
        (Self : Bare_Simple_Decl_Stmt
         ; Simple_Decl_Stmt_F_Decl : Bare_Object_Decl
        ) is
      begin
            Self.Simple_Decl_Stmt_F_Decl := Simple_Decl_Stmt_F_Decl;
      end Initialize_Fields_For_Simple_Decl_Stmt;
   function Simple_Decl_Stmt_F_Decl
     (Node : Bare_Simple_Decl_Stmt) return Bare_Object_Decl
   is
   begin
         return Node.Simple_Decl_Stmt_F_Decl;
   end;
--# property-start SimpleDeclStmt.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16782'
pragma Warnings (Off, "is not referenced");
 function Simple_Decl_Stmt_P_Xref_Equation
  (Node : Bare_Simple_Decl_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Simple_Decl_Stmt  := Bare_Simple_Decl_Stmt (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1414 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1414 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1414;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16782:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1414;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1414;
            raise;
      end;
   return Property_Result;
end Simple_Decl_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Terminate_Alternative
   --
--# property-start TerminateAlternative.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16788'
pragma Warnings (Off, "is not referenced");
 function Terminate_Alternative_P_Xref_Equation
  (Node : Bare_Terminate_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Terminate_Alternative  := Bare_Terminate_Alternative (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1415 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1415 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1415;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16788:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1415;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1415;
            raise;
      end;
   return Property_Result;
end Terminate_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Subp_Kind
   --
   --
   --  Primitives for Bare_Subp_Kind_Function
   --
   --
   --  Primitives for Bare_Subp_Kind_Procedure
   --
   --
   --  Primitives for Bare_Subunit
   --
      procedure Initialize_Fields_For_Subunit
        (Self : Bare_Subunit
         ; Subunit_F_Name : Bare_Name
         ; Subunit_F_Body : Bare_Body_Node
        ) is
      begin
            Self.Subunit_F_Name := Subunit_F_Name;
            Self.Subunit_F_Body := Subunit_F_Body;
      end Initialize_Fields_For_Subunit;
   function Subunit_F_Name
     (Node : Bare_Subunit) return Bare_Name
   is
   begin
         return Node.Subunit_F_Name;
   end;
   function Subunit_F_Body
     (Node : Bare_Subunit) return Bare_Body_Node
   is
   begin
         return Node.Subunit_F_Body;
   end;
--# property-start Subunit.root_unit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16802'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Root_Unit
  (Node : Bare_Subunit
  )
   return Internal_Entity_Compilation_Unit
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1416 with Inline_Always;
      Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
            procedure Finalizer_Scope_1416 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1416;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12176 '<as_bare_entity at nodes.lkt:16802:48>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16802'
--# expr-start 12175 '<FieldAccess for designated_compilation_unit at nodes.lkt:16802:48>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16802'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12174 '<FieldAccess for as_symbol_array at nodes.lkt:16803:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16803'
--# expr-start 12173 '<FieldAccess for name at nodes.lkt:16803:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16803'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 12173
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 12174
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Unit_Body, Load_If_Needed => True, Not_Found_Is_Error => True);
--# end
--# expr-done 12175
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 12176
         Property_Result := If_Result;
   --# end
      Finalizer_Scope_1416;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1416;
            raise;
      end;
   return Property_Result;
end Subunit_P_Root_Unit;
--# end
--# property-start Subunit.env_hook_subunit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16807'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Env_Hook_Subunit
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Boolean;
      Ignored : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity_Compilation_Unit;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12178 '<Let at nodes.lkt:16810:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16810'
--# scope-start
--# expr-start 12177 '<FieldAccess for root_unit at nodes.lkt:16810:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16810'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 12177
Ignored := Fld; 
--# bind _ Ignored
Let_Result := False; 
--# end
--# expr-done 12178
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Env_Hook_Subunit;
--# end
--# property-start Subunit.body_root 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16817'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Body_Root
  (Node : Bare_Subunit
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 12181 '<as_bare_entity at nodes.lkt:16818:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16818'
--# expr-start 12180 '<FieldAccess for decl at nodes.lkt:16818:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16818'
--# expr-start 12179 '<FieldAccess for root_unit at nodes.lkt:16818:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16818'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 12179
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld.Node);
--# end
--# expr-done 12180
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 12181
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Body_Root;
--# end
--# property-start Subunit.bodies_root 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16830'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Bodies_Root
  (Node : Bare_Subunit
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1419 with Inline_Always;
            procedure Finalizer_Scope_3574 with Inline_Always;
      Local_Br : Internal_Entity_Basic_Decl;
Var_Expr_0 : Internal_Entity_Subunit;
Fld : Internal_Entity_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Subunit;
Array_Lit : Internal_Entity_Basic_Decl_Array_Access;
Fld_3 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Array_Lit_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_1419 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1419;
            procedure Finalizer_Scope_3574 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_3574;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12190 '<Let at nodes.lkt:16831:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16831'
--# scope-start
--# expr-start 12184 '<as_bare_entity at nodes.lkt:16831:18>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16831'
--# expr-start 12183 '<FieldAccess for decl at nodes.lkt:16831:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16831'
--# expr-start 12182 '<FieldAccess for root_unit at nodes.lkt:16831:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16831'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 12182
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld.Node);
--# end
--# expr-done 12183
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 12184
Local_Br := If_Result; 
--# bind br Local_Br
--# expr-start 12186 '<Cast to Subunit at nodes.lkt:16833:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16833'
--# expr-start 12185 '<parent at nodes.lkt:16833:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16833'
      if Local_Br.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Local_Br.Node, E_Info => Local_Br.Info);
--# expr-done 12185
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Subunit_Range
   then
      Cast_Result := Create_Internal_Entity_Subunit
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Subunit;
   end if;
--# expr-done 12186
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Subunit then
   --# scope-start
   --# bind su Var_Expr_0
--# expr-start 12187 '<ArrayLiteral at nodes.lkt:16834:21>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16834'
Array_Lit := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Local_Br)); 
--# expr-done 12187
--# expr-start 12188 '<FieldAccess for bodies_root at nodes.lkt:16834:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16834'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.bodies_root
Fld_3 := Libadalang.Implementation.Subunit_P_Bodies_Root (Node => Var_Expr_0.Node);
--# end
--# expr-done 12188
Concat_Result := Concat (Array_Lit, Fld_3); 
Scope_Result := Concat_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3574;
   Result_Var := Scope_Result;
else
   --# expr-start 12189 '<ArrayLiteral at nodes.lkt:16834:58>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16834'
Array_Lit_1 := Create_Internal_Entity_Basic_Decl_Array (Internal_Internal_Entity_Basic_Decl_Array'(1 => Local_Br)); 
--# expr-done 12189
   Result_Var := Array_Lit_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 12190
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1419;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1419;
                     Finalizer_Scope_3574;
            raise;
      end;
   return Property_Result;
end Subunit_P_Bodies_Root;
--# end
--# property-start Subunit.stub 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16847'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Stub
  (Node : Bare_Subunit
  )
   return Bare_Body_Stub
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Bare_Body_Stub;
      Local_Root : Bare_Compilation_Unit;
Local_Root_Body : Bare_Body_Node;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Local_Decls : Bare_Ada_Node_List;
Match_0_1 : Bare_Subp_Body;
Match_1_1 : Bare_Package_Body;
Match_2_1 : Bare_Body_Node;
Local_Stub_Name : Internal_Entity_Name;
Local_Subunit_Name : Internal_Entity_Name;
Var_Expr_287 : Internal_Entity_Body_Stub;
Var_Expr_288 : Internal_Entity_Name;
Fld : Internal_Entity_Compilation_Unit;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Fld_4 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Subunit;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Declarative_Part;
Fld_6 : Bare_Ada_Node_List;
Fld_7 : Bare_Declarative_Part;
Fld_8 : Bare_Ada_Node_List;
Match_Prefix_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Subp_Body;
Let_Result_3 : Bare_Ada_Node_List;
Scope_Result_3 : Bare_Ada_Node_List;
Cast_Result_5 : Bare_Package_Body;
Let_Result_4 : Bare_Ada_Node_List;
Scope_Result_4 : Bare_Ada_Node_List;
Cast_Result_6 : Bare_Body_Node;
Let_Result_5 : Bare_Ada_Node_List;
Scope_Result_5 : Bare_Ada_Node_List;
Match_Result_1 : Bare_Ada_Node_List;
Item_0 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result_7 : Bare_Body_Stub;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Name;
Scope_Result_6 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_11 : Bare_Body_Node;
As_Entity_1 : Internal_Entity_Body_Node;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Body_Node;
Fld_12 : Internal_Entity_Defining_Name;
Fld_13 : Internal_Entity_Name;
Fld_14 : Symbol_Type;
Fld_15 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_1 : Boolean;
Let_Result_6 : Boolean;
Let_Result_7 : Boolean;
Find_Result : Bare_Ada_Node;
Cast_Result_8 : Bare_Body_Stub;
Let_Result_8 : Bare_Body_Stub;
Let_Result_9 : Bare_Body_Stub;
Let_Result_10 : Bare_Body_Stub;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12219 '<Let at nodes.lkt:16852:9>' Let_Result_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16852'
--# scope-start
--# expr-start 12192 '<FieldAccess for node at nodes.lkt:16852:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16852'
--# expr-start 12191 '<FieldAccess for root_unit at nodes.lkt:16852:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16852'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.root_unit
Fld := Libadalang.Implementation.Subunit_P_Root_Unit (Node => Self);
--# end
--# expr-done 12191
Fld_1 := Fld.Node;
--# expr-done 12192
Local_Root := Fld_1; 
--# bind root Local_Root
--# expr-start 12218 '<Let at nodes.lkt:16853:9>' Let_Result_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16853'
--# scope-start
--# expr-start 12197 '<Match at nodes.lkt:16853:25>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16853'
--# expr-start 12193 '<FieldAccess for body at nodes.lkt:16853:31>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16853'
      if Local_Root = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Local_Root.Compilation_Unit_F_Body;
--# expr-done 12193
      if Fld_2 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_0 := Cast_Result_1; 
--# bind li Match_0
--# expr-start 12195 '<Cast to Body at nodes.lkt:16854:37>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16854'
--# expr-start 12194 '<FieldAccess for item at nodes.lkt:16854:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16854'
      if Match_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Match_0.Library_Item_F_Item;
--# expr-done 12194
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Body_Node
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12195
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind su Match_1
--# expr-start 12196 '<FieldAccess for body at nodes.lkt:16855:33>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16855'
      if Match_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Match_1.Subunit_F_Body;
--# expr-done 12196
Let_Result_1 := Fld_4; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 12197
Local_Root_Body := Match_Result; 
--# bind root_body Local_Root_Body
--# expr-start 12217 '<Let at nodes.lkt:16858:9>' Let_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16858'
--# scope-start
--# expr-start 12202 '<Match at nodes.lkt:16858:21>' Match_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16858'
      if Local_Root_Body = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix_1 := Local_Root_Body; 
case Ada_Body_Node (Match_Prefix_1.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_0_1 := Cast_Result_4; 
--# bind subp Match_0_1
--# expr-start 12199 '<FieldAccess for decls at nodes.lkt:16859:36>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16859'
--# expr-start 12198 '<FieldAccess for decls at nodes.lkt:16859:36>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16859'
      if Match_0_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Match_0_1.Subp_Body_F_Decls;
--# expr-done 12198
      if Fld_5 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Fld_5.Declarative_Part_F_Decls;
--# expr-done 12199
Let_Result_3 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_1_1 := Cast_Result_5; 
--# bind pkg Match_1_1
--# expr-start 12201 '<FieldAccess for decls at nodes.lkt:16860:38>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16860'
--# expr-start 12200 '<FieldAccess for decls at nodes.lkt:16860:38>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16860'
      if Match_1_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Match_1_1.Package_Body_F_Decls;
--# expr-done 12200
      if Fld_7 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Fld_7.Declarative_Part_F_Decls;
--# expr-done 12201
Let_Result_4 := Fld_8; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Accept_Stmt_Body .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix_1;
Match_2_1 := Cast_Result_6; 
--# bind _ Match_2_1
Let_Result_5 := No_Bare_Ada_Node; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 12202
Local_Decls := Match_Result_1; 
--# bind decls Local_Decls
--# expr-start 12216 '<Cast to BodyStub at nodes.lkt:16866:9>' Cast_Result_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16866'
--# expr-start 12215 '<Find at nodes.lkt:16866:9>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16866'
      Find_Result := No_Bare_Ada_Node;
      if Local_Decls = null then
      Find_Result := No_Bare_Ada_Node;
      else
      declare
         Collection : constant Bare_Ada_Node_List := Local_Decls;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind d Item_0
      --# expr-start 12214 '<Let at nodes.lkt:16870:17>' Let_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16870'
--# scope-start
--# expr-start 12204 '<as_bare_entity at nodes.lkt:16870:33>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16870'
--# expr-start 12203 '<Cast to BodyStub at nodes.lkt:16870:33>' Cast_Result_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16870'
Cast_Expr := Item_0; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Body_Stub
   then
      Cast_Result_7 := Cast_Expr;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 12203
Is_Null := Cast_Result_7 = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_7); 
   If_Result := As_Entity;
end if;
--# expr-done 12204
Var_Expr_287 := If_Result;
if Var_Expr_287 /= No_Entity_Body_Stub then
   --# scope-start
--# expr-start 12206 '<FieldAccess for name at nodes.lkt:16870:33>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16870'
--# expr-start 12205 '<FieldAccess for defining_name_or_raise at nodes.lkt:16870:33>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16870'
      if Var_Expr_287.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => Var_Expr_287.Node, E_Info => Var_Expr_287.Info);
--# end
--# expr-done 12205
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Fld_9.Node.Defining_Name_F_Name, Info => Fld_9.Info);
--# expr-done 12206
Scope_Result_6 := Fld_10;
   --# end
   Result_Var := Scope_Result_6;
else
   Result_Var := No_Entity_Name;
end if;
Local_Stub_Name := Result_Var; 
--# bind stub_name Local_Stub_Name
--# expr-start 12213 '<Let at nodes.lkt:16873:17>' Let_Result_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16873'
--# scope-start
--# expr-start 12210 '<FieldAccess for name at nodes.lkt:16873:36>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16873'
--# expr-start 12209 '<FieldAccess for defining_name_or_raise at nodes.lkt:16873:36>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16873'
--# expr-start 12208 '<as_bare_entity at nodes.lkt:16873:36>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16873'
--# expr-start 12207 '<FieldAccess for body at nodes.lkt:16873:36>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16873'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Self.Subunit_F_Body;
--# expr-done 12207
Is_Null_1 := Fld_11 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Body_Node;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_11); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 12208
      if If_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name_or_raise
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name_Or_Raise (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 12209
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Fld_12.Node.Defining_Name_F_Name, Info => Fld_12.Info);
--# expr-done 12210
Local_Subunit_Name := Fld_13; 
--# bind subunit_name Local_Subunit_Name
Var_Expr_288 := Local_Stub_Name;
if Var_Expr_288 /= No_Entity_Name then
   --# scope-start
--# expr-start 12212 '<FieldAccess for name_is at nodes.lkt:16875:17>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16875'
      if Var_Expr_288.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12211 '<FieldAccess for name_symbol at nodes.lkt:16875:36>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16875'
      if Local_Subunit_Name.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Local_Subunit_Name.Node);
--# end
--# expr-done 12211
--# property-call-start Name.name_is
Fld_15 := Libadalang.Implementation.Name_P_Name_Is (Node => Var_Expr_288.Node, Sym => Fld_14);
--# end
--# expr-done 12212
Scope_Result_7 := Fld_15;
   --# end
   Result_Var_1 := Scope_Result_7;
else
   Result_Var_1 := False;
end if;
Let_Result_6 := Result_Var_1; 
--# end
--# expr-done 12213
Let_Result_7 := Let_Result_6; 
--# end
--# expr-done 12214
      if Let_Result_7 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Let_Result_7;
         end loop;
      end;
      end if;
--# expr-done 12215
   if Find_Result = null
      or else Find_Result.Kind in Ada_Body_Stub
   then
      Cast_Result_8 := Find_Result;
   else
         Cast_Result_8 := No_Bare_Ada_Node;
   end if;
--# expr-done 12216
Let_Result_8 := Cast_Result_8; 
--# end
--# expr-done 12217
Let_Result_9 := Let_Result_8; 
--# end
--# expr-done 12218
Let_Result_10 := Let_Result_9; 
--# end
--# expr-done 12219
         Property_Result := Let_Result_10;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Stub;
--# end
--# property-start Subunit.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16880'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Entry_Point
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subunit_P_Xref_Entry_Point;
--# end
--# property-start Subunit.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16883'
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Equation
  (Node : Bare_Subunit
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subunit  := Bare_Subunit (Node);
     Ent : Internal_Entity_Subunit :=
       Internal_Entity_Subunit'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1422 with Inline_Always;
            procedure Finalizer_Scope_3584 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Dotted_Name;
Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Fld_4 : Bare_Body_Node;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1422 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1422;
            procedure Finalizer_Scope_3584 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_3584;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12231 '<And at nodes.lkt:16884:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16884'
--# expr-start 12223 '<Bind at nodes.lkt:16884:9>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16884'
--# expr-start 12221 '<FieldAccess for ref_var at nodes.lkt:16884:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16884'
--# expr-start 12220 '<FieldAccess for name at nodes.lkt:16884:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16884'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 12220
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 12221
Fld_1.Value := No_Entity;
Entity_Vars.Reset (Fld_1);
--# expr-start 12222 '<FieldAccess for body_root at nodes.lkt:16884:34>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16884'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Subunit.body_root
Fld_2 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Ent.Node);
--# end
--# expr-done 12222
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Solver.Create_Assign (Fld_1, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16884:9") else null)); 
--# expr-done 12223
--# expr-start 12225 '<Cast to DottedName at nodes.lkt:16887:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16887'
--# expr-start 12224 '<FieldAccess for name at nodes.lkt:16887:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16887'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Subunit_F_Name, Info => Ent.Info);
--# expr-done 12224
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Dotted_Name;
   end if;
--# expr-done 12225
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 12230 '<bind at nodes.lkt:16889:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16889'
--# expr-start 12227 '<FieldAccess for node_env at nodes.lkt:16889:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16889'
--# expr-start 12226 '<FieldAccess for body at nodes.lkt:16889:32>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16889'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Subunit_F_Body;
--# expr-done 12226
      if Fld_4 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Node_Env (Node => Fld_4);
--# expr-done 12227
Bound_Env := Fld_5; Inc_Ref (Bound_Env);
--# expr-start 12229 '<FieldAccess for xref_no_overloading at nodes.lkt:16891:21>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16891'
--# expr-start 12228 '<FieldAccess for prefix at nodes.lkt:16891:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16891'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Dotted_Name_F_Prefix, Info => Var_Expr_0.Info);
--# expr-done 12228
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_7 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_6.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_6.Info);
--# end
--# expr-done 12229
Dyn_Var_Bind_Result := Fld_7; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12230
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3584;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16892:32") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16884:9") else null)); 
--# expr-done 12231
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1422;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1422;
                     Finalizer_Scope_3584;
            raise;
      end;
   return Property_Result;
end Subunit_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synchronized_Node
   --
--# property-start '[dispatcher]Synchronized.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Synchronized_Node_P_As_Bool
  (Node : Bare_Synchronized_Node
  )
   return Boolean
is
   Self : Bare_Synchronized_Node  := Bare_Synchronized_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Synchronized_Node (Self.Kind) is
               when Ada_Synchronized_Absent =>
                  --# property-call-start Synchronized.Absent.as_bool
                  Property_Result := Synchronized_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Synchronized_Present =>
                  --# property-call-start Synchronized.Present.as_bool
                  Property_Result := Synchronized_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Synchronized_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Absent
   --
--# property-start Synchronized.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16899'
pragma Warnings (Off, "is not referenced");
 function Synchronized_Absent_P_As_Bool
  (Node : Bare_Synchronized_Absent
  )
   return Boolean
is
   Self : Bare_Synchronized_Absent  := Bare_Synchronized_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synchronized_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Present
   --
--# property-start Synchronized.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16899'
pragma Warnings (Off, "is not referenced");
 function Synchronized_Present_P_As_Bool
  (Node : Bare_Synchronized_Present
  )
   return Boolean
is
   Self : Bare_Synchronized_Present  := Bare_Synchronized_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Synchronized_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Node
   --
--# property-start '[dispatcher]Tagged.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Tagged_Node_P_As_Bool
  (Node : Bare_Tagged_Node
  )
   return Boolean
is
   Self : Bare_Tagged_Node  := Bare_Tagged_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Tagged_Node (Self.Kind) is
               when Ada_Tagged_Absent =>
                  --# property-call-start Tagged.Absent.as_bool
                  Property_Result := Tagged_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Tagged_Present =>
                  --# property-call-start Tagged.Present.as_bool
                  Property_Result := Tagged_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Tagged_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Absent
   --
--# property-start Tagged.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16904'
pragma Warnings (Off, "is not referenced");
 function Tagged_Absent_P_As_Bool
  (Node : Bare_Tagged_Absent
  )
   return Boolean
is
   Self : Bare_Tagged_Absent  := Bare_Tagged_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Tagged_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Present
   --
--# property-start Tagged.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16904'
pragma Warnings (Off, "is not referenced");
 function Tagged_Present_P_As_Bool
  (Node : Bare_Tagged_Present
  )
   return Boolean
is
   Self : Bare_Tagged_Present  := Bare_Tagged_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Tagged_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Task_Def
   --
      procedure Initialize_Fields_For_Task_Def
        (Self : Bare_Task_Def
         ; Task_Def_F_Interfaces : Bare_Parent_List
         ; Task_Def_F_Public_Part : Bare_Public_Part
         ; Task_Def_F_Private_Part : Bare_Private_Part
         ; Task_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Def_F_Interfaces := Task_Def_F_Interfaces;
            Self.Task_Def_F_Public_Part := Task_Def_F_Public_Part;
            Self.Task_Def_F_Private_Part := Task_Def_F_Private_Part;
            Self.Task_Def_F_End_Name := Task_Def_F_End_Name;
      end Initialize_Fields_For_Task_Def;
   function Task_Def_F_Interfaces
     (Node : Bare_Task_Def) return Bare_Parent_List
   is
   begin
         return Node.Task_Def_F_Interfaces;
   end;
   function Task_Def_F_Public_Part
     (Node : Bare_Task_Def) return Bare_Public_Part
   is
   begin
         return Node.Task_Def_F_Public_Part;
   end;
   function Task_Def_F_Private_Part
     (Node : Bare_Task_Def) return Bare_Private_Part
   is
   begin
         return Node.Task_Def_F_Private_Part;
   end;
   function Task_Def_F_End_Name
     (Node : Bare_Task_Def) return Bare_End_Name
   is
   begin
         return Node.Task_Def_F_End_Name;
   end;
--# property-start TaskDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16915'
pragma Warnings (Off, "is not referenced");
 function Task_Def_P_Xref_Equation
  (Node : Bare_Task_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Def  := Bare_Task_Def (Node);
     Ent : Internal_Entity_Task_Def :=
       Internal_Entity_Task_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1429 with Inline_Always;
            procedure Finalizer_Scope_3585 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1429 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1429;
            procedure Finalizer_Scope_3585 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3585;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12235 '<LogicAll at nodes.lkt:16916:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16916'
--# expr-start 12234 '<Map at nodes.lkt:16916:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16916'
   --# expr-start 12232 '<FieldAccess for interfaces at nodes.lkt:16916:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16916'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Task_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12232
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind ifc Item_0
      --# expr-start 12233 '<FieldAccess for xref_equation at nodes.lkt:16916:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16916'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 12233
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3585;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12234
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:16916:9") else null)); 
--# expr-done 12235
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1429;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1429;
                     Finalizer_Scope_3585;
            raise;
      end;
   return Property_Result;
end Task_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Type_Attributes_Repository
   --
      procedure Initialize_Fields_For_Type_Attributes_Repository
        (Self : Bare_Type_Attributes_Repository
        ) is
      begin
      Self.Type_Attributes_Repository_Base_Type := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Expr := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Expr := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Type_Expr := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Type_Expr := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Type_Expr := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Type_Expr := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Param := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Param := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Param := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Param := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Param := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Param := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Root_Stream_Param := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Root_Stream_Param := null;
      Self.Bare_Type_Attributes_Repository_Lf_State_Succ := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Succ := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Pred := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Pred := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Min := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Min := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Max := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Max := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Round := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Round := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Rounding := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Rounding := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Unbiased_Rounding := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Unbiased_Rounding := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Ceiling := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Ceiling := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Floor := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Floor := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Truncation := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Truncation := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Machine := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Machine_Rounding := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine_Rounding := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Fraction := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Fraction := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Exponent := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Exponent := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Copy_Sign := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Copy_Sign := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Remainder := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Remainder := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Adjacent := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Adjacent := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Scaling := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Scaling := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Compose := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Compose := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Leading_Part := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Leading_Part := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Mod := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Mod := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Value := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Value := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Value := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Value := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Value := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Value := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Fixed_Value := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Fixed_Value := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Integer_Value := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Integer_Value := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Pos := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Pos := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Val_Attr := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Val_Attr := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Rep := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Rep := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Val := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Val := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Read := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Read := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Write := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Write := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Input := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Input := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Output := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Output := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Image := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Image := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Image := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Image := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Image := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Image := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Put_Image := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Put_Image := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Input := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Input := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Output := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Output := No_Bare_Ada_Node;
      Self.Bare_Type_Attributes_Repository_Lf_State_Model := Uninitialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Model := No_Bare_Ada_Node;
      end Initialize_Fields_For_Type_Attributes_Repository;
--# property-start TypeAttributesRepository.base_type_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16928'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Base_Type_Expr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Type_Expr_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1430 with Inline_Always;
      Fld : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1430 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1430;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Expr is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Expr;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Expr,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
         --# expr-start 12238 '<CreateSynthNodeBuilder at nodes.lkt:16929:9>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16929'
--# expr-start 12237 '<CreateCopyNodeBuilder at nodes.lkt:16930:25>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16930'
--# expr-start 12236 '<FieldAccess for base_type at nodes.lkt:16930:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16930'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12236
Builder := Create_Copy_Node_Builder (Fld); 
--# expr-done 12237
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12238
         Property_Result := Builder_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1430;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1430;
               Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Expr :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Expr := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Expr := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Base_Type_Expr;
--# end
--# property-start TypeAttributesRepository.universal_int_type_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16934'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Int_Type_Expr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Type_Expr_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1431 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1431 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1431;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Type_Expr is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Type_Expr;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Type_Expr,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12242 '<CreateSynthNodeBuilder at nodes.lkt:16935:9>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16935'
--# expr-start 12241 '<CreateCopyNodeBuilder at nodes.lkt:16936:25>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16936'
--# expr-start 12240 '<FieldAccess for node at nodes.lkt:16936:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16936'
--# expr-start 12239 '<FieldAccess for universal_int_type at nodes.lkt:16936:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16936'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 12239
Fld_1 := Fld.Node;
--# expr-done 12240
Builder := Create_Copy_Node_Builder (Fld_1); 
--# expr-done 12241
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12242
         Property_Result := Builder_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1431;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1431;
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Type_Expr :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Type_Expr := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Type_Expr := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Int_Type_Expr;
--# end
--# property-start TypeAttributesRepository.universal_real_type_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16940'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Real_Type_Expr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Type_Expr_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1432 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
            procedure Finalizer_Scope_1432 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1432;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Type_Expr is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Type_Expr;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Type_Expr,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12246 '<CreateSynthNodeBuilder at nodes.lkt:16941:9>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16941'
--# expr-start 12245 '<CreateCopyNodeBuilder at nodes.lkt:16942:25>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16942'
--# expr-start 12244 '<FieldAccess for node at nodes.lkt:16942:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16942'
--# expr-start 12243 '<FieldAccess for universal_real_type at nodes.lkt:16942:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16942'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 12243
Fld_1 := Fld.Node;
--# expr-done 12244
Builder := Create_Copy_Node_Builder (Fld_1); 
--# expr-done 12245
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12246
         Property_Result := Builder_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1432;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1432;
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Type_Expr :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Type_Expr := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Type_Expr := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Real_Type_Expr;
--# end
--# property-start TypeAttributesRepository.base_type_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16946'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Base_Type_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1433 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1433 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
            end Finalizer_Scope_1433;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Param is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Param;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Param,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
         --# expr-start 12249 '<CreateSynthNodeBuilder at nodes.lkt:16947:9>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16947'
--# expr-start 12247 '<Symbol Value at nodes.lkt:16948:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16948'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 12247
--# expr-start 12248 '<FieldAccess for base_type_expr at nodes.lkt:16948:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16948'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12248
Builder := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym, Fld); 
--# expr-done 12249
         Property_Result := Builder;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1433;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1433;
               Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Param :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Base_Type_Param := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Param := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Base_Type_Param;
--# end
--# property-start TypeAttributesRepository.universal_int_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16952'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Int_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1434 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1434 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
            end Finalizer_Scope_1434;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Param is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Param;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Param,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12252 '<CreateSynthNodeBuilder at nodes.lkt:16953:9>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16953'
--# expr-start 12250 '<Symbol Value at nodes.lkt:16954:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16954'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 12250
--# expr-start 12251 '<FieldAccess for universal_int_type_expr at nodes.lkt:16954:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16954'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 12251
Builder := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym, Fld); 
--# expr-done 12252
         Property_Result := Builder;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1434;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1434;
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Param :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Int_Param := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Param := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Int_Param;
--# end
--# property-start TypeAttributesRepository.universal_real_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16958'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Universal_Real_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1435 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1435 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
            end Finalizer_Scope_1435;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Param is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Param;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Param,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12255 '<CreateSynthNodeBuilder at nodes.lkt:16959:9>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16959'
--# expr-start 12253 '<Symbol Value at nodes.lkt:16960:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16960'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 12253
--# expr-start 12254 '<FieldAccess for universal_real_type_expr at nodes.lkt:16960:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_type_expr
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Type_Expr (Node => Self);
--# end
--# expr-done 12254
Builder := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym, Fld); 
--# expr-done 12255
         Property_Result := Builder;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1435;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1435;
               Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Param :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Universal_Real_Param := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Param := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Universal_Real_Param;
--# end
--# property-start TypeAttributesRepository.root_stream_param 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16964'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Root_Stream_Param
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Synthetic_Formal_Param_Decl_Node_Builder
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1436 with Inline_Always;
      Sym : Symbol_Type;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
            procedure Finalizer_Scope_1436 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
            end Finalizer_Scope_1436;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Root_Stream_Param is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Root_Stream_Param;
               Inc_Ref (Property_Result);
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Root_Stream_Param,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12262 '<CreateSynthNodeBuilder at nodes.lkt:16965:9>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16965'
--# expr-start 12256 '<Symbol S at nodes.lkt:16966:24>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16966'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 12256
--# expr-start 12261 '<CreateSynthNodeBuilder at nodes.lkt:16966:41>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16966'
--# expr-start 12260 '<CreateCopyNodeBuilder at nodes.lkt:16967:29>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16967'
--# expr-start 12259 '<FieldAccess for node at nodes.lkt:16967:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16967'
--# expr-start 12258 '<FieldAccess for anonymous_access_type at nodes.lkt:16967:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16967'
--# expr-start 12257 '<FieldAccess for root_stream_type at nodes.lkt:16967:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16967'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_stream_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Stream_Type (Node => Self);
--# end
--# expr-done 12257
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12258
Fld_2 := Fld_1.Node;
--# expr-done 12259
Builder := Create_Copy_Node_Builder (Fld_2); 
--# expr-done 12260
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12261
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym, Builder_1); 
--# expr-done 12262
         Property_Result := Builder_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1436;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1436;
               Self.Bare_Type_Attributes_Repository_Lf_State_Root_Stream_Param :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Root_Stream_Param := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Root_Stream_Param := Property_Result;
         Inc_Ref (Property_Result);
   return Property_Result;
end Type_Attributes_Repository_F_Root_Stream_Param;
--# end
--# property-start TypeAttributesRepository.succ 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16972'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Succ
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1437 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1437 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1437;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Succ is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Succ;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Succ,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12267 '<CreateSynthNodeBuilder at nodes.lkt:16972:27>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16972'
--# expr-start 12266 '<CreateSynthNodeBuilder at nodes.lkt:16973:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16973'
--# expr-start 12263 '<Symbol Succ at nodes.lkt:16974:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16974'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Succ); 
--# expr-done 12263
--# expr-start 12264 '<FieldAccess for base_type_param at nodes.lkt:16974:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16974'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12264
--# expr-start 12265 '<FieldAccess for base_type_expr at nodes.lkt:16974:85>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16974'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12265
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12266
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12267
--# expr-start 12268 '<FieldAccess for base_type at nodes.lkt:16976:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16976'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12268
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1437;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1437;
               Self.Bare_Type_Attributes_Repository_Lf_State_Succ :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Succ := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Succ := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Succ;
--# end
--# property-start TypeAttributesRepository.pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16979'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Pred
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1438 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1438 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1438;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Pred is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Pred;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Pred,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12273 '<CreateSynthNodeBuilder at nodes.lkt:16979:27>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16979'
--# expr-start 12272 '<CreateSynthNodeBuilder at nodes.lkt:16980:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16980'
--# expr-start 12269 '<Symbol Pred at nodes.lkt:16981:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16981'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pred); 
--# expr-done 12269
--# expr-start 12270 '<FieldAccess for base_type_param at nodes.lkt:16981:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16981'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12270
--# expr-start 12271 '<FieldAccess for base_type_expr at nodes.lkt:16981:85>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16981'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12271
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12272
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12273
--# expr-start 12274 '<FieldAccess for base_type at nodes.lkt:16983:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16983'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12274
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1438;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1438;
               Self.Bare_Type_Attributes_Repository_Lf_State_Pred :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Pred := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Pred := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Pred;
--# end
--# property-start TypeAttributesRepository.min 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16986'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Min
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1439 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1439 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1439;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Min is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Min;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Min,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12280 '<CreateSynthNodeBuilder at nodes.lkt:16986:26>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16986'
--# expr-start 12279 '<CreateSynthNodeBuilder at nodes.lkt:16987:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16987'
--# expr-start 12275 '<Symbol Min at nodes.lkt:16988:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16988'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Min); 
--# expr-done 12275
--# expr-start 12276 '<FieldAccess for base_type_param at nodes.lkt:16988:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16988'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12276
--# expr-start 12277 '<FieldAccess for base_type_param at nodes.lkt:16988:78>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16988'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12277
--# expr-start 12278 '<FieldAccess for base_type_expr at nodes.lkt:16988:117>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16988'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12278
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12279
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12280
--# expr-start 12281 '<FieldAccess for base_type at nodes.lkt:16990:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16990'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12281
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1439;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1439;
               Self.Bare_Type_Attributes_Repository_Lf_State_Min :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Min := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Min := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Min;
--# end
--# property-start TypeAttributesRepository.max 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16993'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Max
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1440 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1440 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1440;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Max is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Max;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Max,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12287 '<CreateSynthNodeBuilder at nodes.lkt:16993:26>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16993'
--# expr-start 12286 '<CreateSynthNodeBuilder at nodes.lkt:16994:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16994'
--# expr-start 12282 '<Symbol Max at nodes.lkt:16995:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16995'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Max); 
--# expr-done 12282
--# expr-start 12283 '<FieldAccess for base_type_param at nodes.lkt:16995:44>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16995'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12283
--# expr-start 12284 '<FieldAccess for base_type_param at nodes.lkt:16995:78>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16995'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12284
--# expr-start 12285 '<FieldAccess for base_type_expr at nodes.lkt:16995:117>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16995'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12285
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12286
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12287
--# expr-start 12288 '<FieldAccess for base_type at nodes.lkt:16997:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:16997'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12288
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1440;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1440;
               Self.Bare_Type_Attributes_Repository_Lf_State_Max :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Max := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Max := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Max;
--# end
--# property-start TypeAttributesRepository.round 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17000'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Round
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1441 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1441 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1441;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Round is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Round;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Round,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12293 '<CreateSynthNodeBuilder at nodes.lkt:17001:5>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17001'
--# expr-start 12292 '<CreateSynthNodeBuilder at nodes.lkt:17002:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17002'
--# expr-start 12289 '<Symbol Round at nodes.lkt:17003:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17003'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Round); 
--# expr-done 12289
--# expr-start 12290 '<FieldAccess for universal_real_param at nodes.lkt:17003:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17003'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Param (Node => Self);
--# end
--# expr-done 12290
--# expr-start 12291 '<FieldAccess for base_type_expr at nodes.lkt:17003:91>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17003'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12291
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12292
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12293
--# expr-start 12294 '<FieldAccess for base_type at nodes.lkt:17005:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17005'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12294
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1441;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1441;
               Self.Bare_Type_Attributes_Repository_Lf_State_Round :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Round := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Round := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Round;
--# end
--# property-start TypeAttributesRepository.rounding 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17008'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1442 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1442 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1442;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Rounding is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Rounding;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Rounding,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12299 '<CreateSynthNodeBuilder at nodes.lkt:17008:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17008'
--# expr-start 12298 '<CreateSynthNodeBuilder at nodes.lkt:17009:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17009'
--# expr-start 12295 '<Symbol Rounding at nodes.lkt:17010:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17010'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rounding); 
--# expr-done 12295
--# expr-start 12296 '<FieldAccess for base_type_param at nodes.lkt:17010:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17010'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12296
--# expr-start 12297 '<FieldAccess for base_type_expr at nodes.lkt:17010:89>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17010'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12297
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12298
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12299
--# expr-start 12300 '<FieldAccess for base_type at nodes.lkt:17012:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17012'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12300
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1442;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1442;
               Self.Bare_Type_Attributes_Repository_Lf_State_Rounding :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Rounding := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Rounding;
--# end
--# property-start TypeAttributesRepository.unbiased_rounding 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17015'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Unbiased_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1443 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1443 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1443;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Unbiased_Rounding is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Unbiased_Rounding;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Unbiased_Rounding,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12305 '<CreateSynthNodeBuilder at nodes.lkt:17015:40>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17015'
--# expr-start 12304 '<CreateSynthNodeBuilder at nodes.lkt:17016:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17016'
--# expr-start 12301 '<Symbol Unbiased_Rounding at nodes.lkt:17017:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17017'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Unbiased_Rounding); 
--# expr-done 12301
--# expr-start 12302 '<FieldAccess for base_type_param at nodes.lkt:17017:59>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17017'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12302
--# expr-start 12303 '<FieldAccess for base_type_expr at nodes.lkt:17017:98>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17017'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12303
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12304
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12305
--# expr-start 12306 '<FieldAccess for base_type at nodes.lkt:17019:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17019'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12306
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1443;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1443;
               Self.Bare_Type_Attributes_Repository_Lf_State_Unbiased_Rounding :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Unbiased_Rounding := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Unbiased_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Unbiased_Rounding;
--# end
--# property-start TypeAttributesRepository.ceiling 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17022'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Ceiling
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1444 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1444 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1444;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Ceiling is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Ceiling;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Ceiling,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12311 '<CreateSynthNodeBuilder at nodes.lkt:17022:30>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17022'
--# expr-start 12310 '<CreateSynthNodeBuilder at nodes.lkt:17023:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17023'
--# expr-start 12307 '<Symbol Ceiling at nodes.lkt:17024:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17024'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Ceiling); 
--# expr-done 12307
--# expr-start 12308 '<FieldAccess for base_type_param at nodes.lkt:17024:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17024'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12308
--# expr-start 12309 '<FieldAccess for base_type_expr at nodes.lkt:17024:88>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17024'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12309
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12310
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12311
--# expr-start 12312 '<FieldAccess for base_type at nodes.lkt:17026:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17026'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12312
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1444;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1444;
               Self.Bare_Type_Attributes_Repository_Lf_State_Ceiling :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Ceiling := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Ceiling := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Ceiling;
--# end
--# property-start TypeAttributesRepository.floor 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17029'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Floor
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1445 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1445 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1445;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Floor is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Floor;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Floor,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12317 '<CreateSynthNodeBuilder at nodes.lkt:17029:28>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17029'
--# expr-start 12316 '<CreateSynthNodeBuilder at nodes.lkt:17030:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17030'
--# expr-start 12313 '<Symbol Floor at nodes.lkt:17031:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17031'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Floor); 
--# expr-done 12313
--# expr-start 12314 '<FieldAccess for base_type_param at nodes.lkt:17031:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17031'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12314
--# expr-start 12315 '<FieldAccess for base_type_expr at nodes.lkt:17031:86>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17031'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12315
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12316
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12317
--# expr-start 12318 '<FieldAccess for base_type at nodes.lkt:17033:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12318
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1445;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1445;
               Self.Bare_Type_Attributes_Repository_Lf_State_Floor :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Floor := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Floor := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Floor;
--# end
--# property-start TypeAttributesRepository.truncation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17036'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Truncation
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1446 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1446 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1446;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Truncation is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Truncation;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Truncation,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12323 '<CreateSynthNodeBuilder at nodes.lkt:17036:33>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17036'
--# expr-start 12322 '<CreateSynthNodeBuilder at nodes.lkt:17037:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17037'
--# expr-start 12319 '<Symbol Truncation at nodes.lkt:17038:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17038'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Truncation); 
--# expr-done 12319
--# expr-start 12320 '<FieldAccess for base_type_param at nodes.lkt:17038:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17038'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12320
--# expr-start 12321 '<FieldAccess for base_type_expr at nodes.lkt:17038:91>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17038'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12321
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12322
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12323
--# expr-start 12324 '<FieldAccess for base_type at nodes.lkt:17040:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17040'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12324
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1446;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1446;
               Self.Bare_Type_Attributes_Repository_Lf_State_Truncation :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Truncation := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Truncation := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Truncation;
--# end
--# property-start TypeAttributesRepository.machine 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17043'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Machine
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1447 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1447 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1447;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Machine is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Machine,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12329 '<CreateSynthNodeBuilder at nodes.lkt:17043:30>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17043'
--# expr-start 12328 '<CreateSynthNodeBuilder at nodes.lkt:17044:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17044'
--# expr-start 12325 '<Symbol Machine at nodes.lkt:17045:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17045'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine); 
--# expr-done 12325
--# expr-start 12326 '<FieldAccess for base_type_param at nodes.lkt:17045:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17045'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12326
--# expr-start 12327 '<FieldAccess for base_type_expr at nodes.lkt:17045:88>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17045'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12327
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12328
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12329
--# expr-start 12330 '<FieldAccess for base_type at nodes.lkt:17047:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17047'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12330
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1447;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1447;
               Self.Bare_Type_Attributes_Repository_Lf_State_Machine :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Machine := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Machine;
--# end
--# property-start TypeAttributesRepository.machine_rounding 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17050'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Machine_Rounding
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1448 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1448 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1448;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Machine_Rounding is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine_Rounding;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Machine_Rounding,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12335 '<CreateSynthNodeBuilder at nodes.lkt:17050:39>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17050'
--# expr-start 12334 '<CreateSynthNodeBuilder at nodes.lkt:17051:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17051'
--# expr-start 12331 '<Symbol Machine_Rounding at nodes.lkt:17052:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17052'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Rounding); 
--# expr-done 12331
--# expr-start 12332 '<FieldAccess for base_type_param at nodes.lkt:17052:58>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17052'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12332
--# expr-start 12333 '<FieldAccess for base_type_expr at nodes.lkt:17052:97>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17052'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12333
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12334
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12335
--# expr-start 12336 '<FieldAccess for base_type at nodes.lkt:17054:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17054'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12336
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1448;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1448;
               Self.Bare_Type_Attributes_Repository_Lf_State_Machine_Rounding :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Machine_Rounding := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Machine_Rounding := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Machine_Rounding;
--# end
--# property-start TypeAttributesRepository.fraction 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17057'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Fraction
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1449 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1449 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1449;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Fraction is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Fraction;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Fraction,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12341 '<CreateSynthNodeBuilder at nodes.lkt:17057:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17057'
--# expr-start 12340 '<CreateSynthNodeBuilder at nodes.lkt:17058:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17058'
--# expr-start 12337 '<Symbol Fraction at nodes.lkt:17059:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17059'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fraction); 
--# expr-done 12337
--# expr-start 12338 '<FieldAccess for base_type_param at nodes.lkt:17059:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17059'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12338
--# expr-start 12339 '<FieldAccess for base_type_expr at nodes.lkt:17059:89>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17059'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12339
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12340
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12341
--# expr-start 12342 '<FieldAccess for base_type at nodes.lkt:17061:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17061'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12342
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1449;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1449;
               Self.Bare_Type_Attributes_Repository_Lf_State_Fraction :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Fraction := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Fraction := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Fraction;
--# end
--# property-start TypeAttributesRepository.exponent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17064'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Exponent
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1450 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1450 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1450;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Exponent is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Exponent;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Exponent,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12347 '<CreateSynthNodeBuilder at nodes.lkt:17064:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17064'
--# expr-start 12346 '<CreateSynthNodeBuilder at nodes.lkt:17065:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17065'
--# expr-start 12343 '<Symbol Exponent at nodes.lkt:17066:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17066'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Exponent); 
--# expr-done 12343
--# expr-start 12344 '<FieldAccess for base_type_param at nodes.lkt:17066:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17066'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12344
--# expr-start 12345 '<FieldAccess for universal_int_type_expr at nodes.lkt:17066:89>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17066'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 12345
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12346
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12347
--# expr-start 12348 '<FieldAccess for base_type at nodes.lkt:17068:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17068'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12348
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1450;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1450;
               Self.Bare_Type_Attributes_Repository_Lf_State_Exponent :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Exponent := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Exponent := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Exponent;
--# end
--# property-start TypeAttributesRepository.copy_sign 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17071'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Copy_Sign
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1451 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1451 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1451;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Copy_Sign is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Copy_Sign;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Copy_Sign,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12354 '<CreateSynthNodeBuilder at nodes.lkt:17071:32>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17071'
--# expr-start 12353 '<CreateSynthNodeBuilder at nodes.lkt:17072:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17072'
--# expr-start 12349 '<Symbol Copy_Sign at nodes.lkt:17073:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17073'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Copy_Sign); 
--# expr-done 12349
--# expr-start 12350 '<FieldAccess for base_type_param at nodes.lkt:17073:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17073'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12350
--# expr-start 12351 '<FieldAccess for base_type_param at nodes.lkt:17073:84>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17073'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12351
--# expr-start 12352 '<FieldAccess for base_type_expr at nodes.lkt:17073:123>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17073'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12352
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12353
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12354
--# expr-start 12355 '<FieldAccess for base_type at nodes.lkt:17075:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17075'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12355
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1451;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1451;
               Self.Bare_Type_Attributes_Repository_Lf_State_Copy_Sign :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Copy_Sign := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Copy_Sign := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Copy_Sign;
--# end
--# property-start TypeAttributesRepository.remainder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17078'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Remainder
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1452 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1452 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1452;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Remainder is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Remainder;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Remainder,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12361 '<CreateSynthNodeBuilder at nodes.lkt:17078:32>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17078'
--# expr-start 12360 '<CreateSynthNodeBuilder at nodes.lkt:17079:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17079'
--# expr-start 12356 '<Symbol Remainder at nodes.lkt:17080:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17080'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Remainder); 
--# expr-done 12356
--# expr-start 12357 '<FieldAccess for base_type_param at nodes.lkt:17080:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17080'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12357
--# expr-start 12358 '<FieldAccess for base_type_param at nodes.lkt:17080:84>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17080'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12358
--# expr-start 12359 '<FieldAccess for base_type_expr at nodes.lkt:17080:123>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17080'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12359
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12360
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12361
--# expr-start 12362 '<FieldAccess for base_type at nodes.lkt:17082:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17082'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12362
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1452;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1452;
               Self.Bare_Type_Attributes_Repository_Lf_State_Remainder :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Remainder := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Remainder := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Remainder;
--# end
--# property-start TypeAttributesRepository.adjacent 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17085'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Adjacent
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1453 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1453 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1453;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Adjacent is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Adjacent;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Adjacent,
               "lazy field memoization");
      end case;
      begin
   --# scope-start
--# expr-start 12368 '<CreateSynthNodeBuilder at nodes.lkt:17085:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17085'
--# expr-start 12367 '<CreateSynthNodeBuilder at nodes.lkt:17086:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17086'
--# expr-start 12363 '<Symbol Adjacent at nodes.lkt:17087:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17087'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Adjacent); 
--# expr-done 12363
--# expr-start 12364 '<FieldAccess for base_type_param at nodes.lkt:17087:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17087'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12364
--# expr-start 12365 '<FieldAccess for base_type_param at nodes.lkt:17087:83>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17087'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12365
--# expr-start 12366 '<FieldAccess for base_type_expr at nodes.lkt:17087:122>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17087'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12366
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12367
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12368
--# expr-start 12369 '<FieldAccess for base_type at nodes.lkt:17089:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12369
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1453;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1453;
               Self.Bare_Type_Attributes_Repository_Lf_State_Adjacent :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Adjacent := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Adjacent := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Adjacent;
--# end
--# property-start TypeAttributesRepository.scaling 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17092'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Scaling
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1454 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1454 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1454;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Scaling is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Scaling;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Scaling,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12375 '<CreateSynthNodeBuilder at nodes.lkt:17092:30>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17092'
--# expr-start 12374 '<CreateSynthNodeBuilder at nodes.lkt:17093:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17093'
--# expr-start 12370 '<Symbol Scaling at nodes.lkt:17094:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17094'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Scaling); 
--# expr-done 12370
--# expr-start 12371 '<FieldAccess for base_type_param at nodes.lkt:17094:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12371
--# expr-start 12372 '<FieldAccess for universal_int_param at nodes.lkt:17094:82>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12372
--# expr-start 12373 '<FieldAccess for base_type_expr at nodes.lkt:17094:125>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17094'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12373
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12374
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12375
--# expr-start 12376 '<FieldAccess for base_type at nodes.lkt:17096:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17096'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12376
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1454;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1454;
               Self.Bare_Type_Attributes_Repository_Lf_State_Scaling :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Scaling := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Scaling := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Scaling;
--# end
--# property-start TypeAttributesRepository.compose 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17099'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Compose
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1455 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1455 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1455;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Compose is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Compose;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Compose,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12382 '<CreateSynthNodeBuilder at nodes.lkt:17099:30>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17099'
--# expr-start 12381 '<CreateSynthNodeBuilder at nodes.lkt:17100:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17100'
--# expr-start 12377 '<Symbol Compose at nodes.lkt:17101:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17101'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Compose); 
--# expr-done 12377
--# expr-start 12378 '<FieldAccess for base_type_param at nodes.lkt:17101:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17101'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12378
--# expr-start 12379 '<FieldAccess for universal_int_param at nodes.lkt:17101:82>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17101'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12379
--# expr-start 12380 '<FieldAccess for base_type_expr at nodes.lkt:17101:125>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17101'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12380
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12381
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12382
--# expr-start 12383 '<FieldAccess for base_type at nodes.lkt:17103:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17103'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12383
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1455;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1455;
               Self.Bare_Type_Attributes_Repository_Lf_State_Compose :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Compose := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Compose := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Compose;
--# end
--# property-start TypeAttributesRepository.leading_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17106'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Leading_Part
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1456 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1456 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1456;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Leading_Part is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Leading_Part;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Leading_Part,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12389 '<CreateSynthNodeBuilder at nodes.lkt:17106:35>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17106'
--# expr-start 12388 '<CreateSynthNodeBuilder at nodes.lkt:17107:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17107'
--# expr-start 12384 '<Symbol Leading_Part at nodes.lkt:17108:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17108'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Leading_Part); 
--# expr-done 12384
--# expr-start 12385 '<FieldAccess for base_type_param at nodes.lkt:17108:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17108'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12385
--# expr-start 12386 '<FieldAccess for universal_int_param at nodes.lkt:17108:87>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17108'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12386
--# expr-start 12387 '<FieldAccess for base_type_expr at nodes.lkt:17108:130>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17108'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12387
Builder := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Fld_2); 
--# expr-done 12388
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12389
--# expr-start 12390 '<FieldAccess for base_type at nodes.lkt:17110:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17110'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12390
      Cast_Result := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1456;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1456;
               Self.Bare_Type_Attributes_Repository_Lf_State_Leading_Part :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Leading_Part := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Leading_Part := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Leading_Part;
--# end
--# property-start TypeAttributesRepository.mod 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17113'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Mod
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1457 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1457 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1457;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Mod is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Mod;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Mod,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12395 '<CreateSynthNodeBuilder at nodes.lkt:17113:26>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17113'
--# expr-start 12394 '<CreateSynthNodeBuilder at nodes.lkt:17114:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17114'
--# expr-start 12391 '<Symbol Mod at nodes.lkt:17115:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17115'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod_170); 
--# expr-done 12391
--# expr-start 12392 '<FieldAccess for universal_int_param at nodes.lkt:17115:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12392
--# expr-start 12393 '<FieldAccess for base_type_expr at nodes.lkt:17115:88>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12393
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12394
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12395
--# expr-start 12396 '<FieldAccess for base_type at nodes.lkt:17117:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17117'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12396
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1457;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1457;
               Self.Bare_Type_Attributes_Repository_Lf_State_Mod :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Mod := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Mod := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Mod;
--# end
--# property-start TypeAttributesRepository.value 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17120'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1458 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_3 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_4 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1458 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
            end Finalizer_Scope_1458;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Value is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Value;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Value,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12408 '<CreateSynthNodeBuilder at nodes.lkt:17120:28>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17120'
--# expr-start 12407 '<CreateSynthNodeBuilder at nodes.lkt:17121:14>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17121'
--# expr-start 12397 '<Symbol Value at nodes.lkt:17122:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17122'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Value); 
--# expr-done 12397
--# expr-start 12405 '<CreateSynthNodeBuilder at nodes.lkt:17122:47>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17122'
--# expr-start 12398 '<Symbol Val at nodes.lkt:17123:28>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17123'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 12398
--# expr-start 12404 '<CreateSynthNodeBuilder at nodes.lkt:17123:47>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17123'
--# expr-start 12403 '<CreateCopyNodeBuilder at nodes.lkt:17124:33>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17124'
--# expr-start 12402 '<FieldAccess for node at nodes.lkt:17124:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17124'
--# expr-start 12401 '<Cast to BaseTypeDecl at nodes.lkt:17124:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17124'
--# expr-start 12400 '<FieldAccess for std_entity at nodes.lkt:17124:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17124'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12399 '<Symbol String at nodes.lkt:17124:49>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17124'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 12399
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 12400
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12401
Fld_1 := Cast_Result.Node;
--# expr-done 12402
Builder := Create_Copy_Node_Builder (Fld_1); 
--# expr-done 12403
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12404
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_1, Builder_1); 
--# expr-done 12405
--# expr-start 12406 '<FieldAccess for base_type_expr at nodes.lkt:17126:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17126'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12406
Builder_3 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Builder_2, Fld_2); 
--# expr-done 12407
Builder_4 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_3); 
--# expr-done 12408
--# expr-start 12409 '<FieldAccess for base_type at nodes.lkt:17128:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17128'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12409
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_4).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1458;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1458;
               Self.Bare_Type_Attributes_Repository_Lf_State_Value :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Value := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Value;
--# end
--# property-start TypeAttributesRepository.wide_value 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17131'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1459 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_3 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_4 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1459 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
            end Finalizer_Scope_1459;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Value is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Value;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Value,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12421 '<CreateSynthNodeBuilder at nodes.lkt:17131:33>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17131'
--# expr-start 12420 '<CreateSynthNodeBuilder at nodes.lkt:17132:14>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17132'
--# expr-start 12410 '<Symbol Wide_Value at nodes.lkt:17133:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17133'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Value); 
--# expr-done 12410
--# expr-start 12418 '<CreateSynthNodeBuilder at nodes.lkt:17133:52>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17133'
--# expr-start 12411 '<Symbol Val at nodes.lkt:17134:28>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17134'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 12411
--# expr-start 12417 '<CreateSynthNodeBuilder at nodes.lkt:17134:47>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17134'
--# expr-start 12416 '<CreateCopyNodeBuilder at nodes.lkt:17135:33>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17135'
--# expr-start 12415 '<FieldAccess for node at nodes.lkt:17135:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17135'
--# expr-start 12414 '<Cast to BaseTypeDecl at nodes.lkt:17135:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17135'
--# expr-start 12413 '<FieldAccess for std_entity at nodes.lkt:17135:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17135'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12412 '<Symbol Wide_String at nodes.lkt:17135:49>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17135'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 12412
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 12413
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12414
Fld_1 := Cast_Result.Node;
--# expr-done 12415
Builder := Create_Copy_Node_Builder (Fld_1); 
--# expr-done 12416
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12417
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_1, Builder_1); 
--# expr-done 12418
--# expr-start 12419 '<FieldAccess for base_type_expr at nodes.lkt:17137:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17137'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12419
Builder_3 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Builder_2, Fld_2); 
--# expr-done 12420
Builder_4 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_3); 
--# expr-done 12421
--# expr-start 12422 '<FieldAccess for base_type at nodes.lkt:17139:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17139'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12422
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_4).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1459;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1459;
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Value :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Value := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Value;
--# end
--# property-start TypeAttributesRepository.wide_wide_value 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17142'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Wide_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1460 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_2 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_3 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_4 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1460 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
            end Finalizer_Scope_1460;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Value is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Value;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Value,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12434 '<CreateSynthNodeBuilder at nodes.lkt:17142:38>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17142'
--# expr-start 12433 '<CreateSynthNodeBuilder at nodes.lkt:17143:14>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17143'
--# expr-start 12423 '<Symbol Wide_Wide_Value at nodes.lkt:17144:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17144'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Value); 
--# expr-done 12423
--# expr-start 12431 '<CreateSynthNodeBuilder at nodes.lkt:17144:57>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17144'
--# expr-start 12424 '<Symbol Val at nodes.lkt:17145:28>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17145'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 12424
--# expr-start 12430 '<CreateSynthNodeBuilder at nodes.lkt:17145:47>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17145'
--# expr-start 12429 '<CreateCopyNodeBuilder at nodes.lkt:17146:33>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17146'
--# expr-start 12428 '<FieldAccess for node at nodes.lkt:17146:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17146'
--# expr-start 12427 '<Cast to BaseTypeDecl at nodes.lkt:17146:33>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17146'
--# expr-start 12426 '<FieldAccess for std_entity at nodes.lkt:17146:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17146'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12425 '<Symbol Wide_Wide_String at nodes.lkt:17146:49>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17146'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 12425
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_2);
--# end
--# expr-done 12426
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12427
Fld_1 := Cast_Result.Node;
--# expr-done 12428
Builder := Create_Copy_Node_Builder (Fld_1); 
--# expr-done 12429
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12430
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_1, Builder_1); 
--# expr-done 12431
--# expr-start 12432 '<FieldAccess for base_type_expr at nodes.lkt:17148:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17148'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_2 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12432
Builder_3 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Builder_2, Fld_2); 
--# expr-done 12433
Builder_4 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_3); 
--# expr-done 12434
--# expr-start 12435 '<FieldAccess for base_type at nodes.lkt:17150:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17150'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12435
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_4).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1460;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1460;
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Value :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Value := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Wide_Value;
--# end
--# property-start TypeAttributesRepository.fixed_value 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17153'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Fixed_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1461 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1461 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1461;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Fixed_Value is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Fixed_Value;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Fixed_Value,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12440 '<CreateSynthNodeBuilder at nodes.lkt:17153:34>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17153'
--# expr-start 12439 '<CreateSynthNodeBuilder at nodes.lkt:17154:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17154'
--# expr-start 12436 '<Symbol Fixed_Value at nodes.lkt:17155:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17155'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Fixed_Value); 
--# expr-done 12436
--# expr-start 12437 '<FieldAccess for universal_int_param at nodes.lkt:17155:53>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17155'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12437
--# expr-start 12438 '<FieldAccess for base_type_expr at nodes.lkt:17155:96>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17155'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12438
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12439
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12440
--# expr-start 12441 '<FieldAccess for base_type at nodes.lkt:17157:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17157'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12441
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1461;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1461;
               Self.Bare_Type_Attributes_Repository_Lf_State_Fixed_Value :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Fixed_Value := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Fixed_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Fixed_Value;
--# end
--# property-start TypeAttributesRepository.integer_value 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17160'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Integer_Value
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1462 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1462 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1462;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Integer_Value is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Integer_Value;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Integer_Value,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12446 '<CreateSynthNodeBuilder at nodes.lkt:17160:36>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17160'
--# expr-start 12445 '<CreateSynthNodeBuilder at nodes.lkt:17161:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17161'
--# expr-start 12442 '<Symbol Integer_Value at nodes.lkt:17162:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17162'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Integer_Value); 
--# expr-done 12442
--# expr-start 12443 '<FieldAccess for universal_real_param at nodes.lkt:17162:55>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17162'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Param (Node => Self);
--# end
--# expr-done 12443
--# expr-start 12444 '<FieldAccess for base_type_expr at nodes.lkt:17162:99>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17162'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12444
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12445
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12446
--# expr-start 12447 '<FieldAccess for base_type at nodes.lkt:17164:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17164'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12447
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1462;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1462;
               Self.Bare_Type_Attributes_Repository_Lf_State_Integer_Value :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Integer_Value := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Integer_Value := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Integer_Value;
--# end
--# property-start TypeAttributesRepository.pos 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17167'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Pos
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1463 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1463 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1463;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Pos is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Pos;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Pos,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12452 '<CreateSynthNodeBuilder at nodes.lkt:17167:26>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17167'
--# expr-start 12451 '<CreateSynthNodeBuilder at nodes.lkt:17168:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17168'
--# expr-start 12448 '<Symbol Pos at nodes.lkt:17169:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17169'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Pos); 
--# expr-done 12448
--# expr-start 12449 '<FieldAccess for base_type_param at nodes.lkt:17169:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17169'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12449
--# expr-start 12450 '<FieldAccess for universal_int_type_expr at nodes.lkt:17169:84>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17169'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 12450
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12451
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12452
--# expr-start 12453 '<FieldAccess for base_type at nodes.lkt:17171:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17171'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12453
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1463;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1463;
               Self.Bare_Type_Attributes_Repository_Lf_State_Pos :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Pos := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Pos := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Pos;
--# end
--# property-start TypeAttributesRepository.val_attr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17175'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Val_Attr
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1464 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1464 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1464;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Val_Attr is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Val_Attr;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Val_Attr,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12458 '<CreateSynthNodeBuilder at nodes.lkt:17175:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17175'
--# expr-start 12457 '<CreateSynthNodeBuilder at nodes.lkt:17176:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17176'
--# expr-start 12454 '<Symbol Val at nodes.lkt:17177:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17177'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Val); 
--# expr-done 12454
--# expr-start 12455 '<FieldAccess for universal_int_param at nodes.lkt:17177:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17177'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12455
--# expr-start 12456 '<FieldAccess for base_type_expr at nodes.lkt:17177:88>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17177'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12456
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12457
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12458
--# expr-start 12459 '<FieldAccess for base_type at nodes.lkt:17179:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17179'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12459
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1464;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1464;
               Self.Bare_Type_Attributes_Repository_Lf_State_Val_Attr :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Val_Attr := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Val_Attr := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Val_Attr;
--# end
--# property-start TypeAttributesRepository.enum_rep 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17182'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Enum_Rep
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1465 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1465 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1465;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Rep is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Rep;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Rep,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12464 '<CreateSynthNodeBuilder at nodes.lkt:17182:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17182'
--# expr-start 12463 '<CreateSynthNodeBuilder at nodes.lkt:17183:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17183'
--# expr-start 12460 '<Symbol Enum_Rep at nodes.lkt:17184:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17184'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Rep); 
--# expr-done 12460
--# expr-start 12461 '<FieldAccess for base_type_param at nodes.lkt:17184:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17184'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12461
--# expr-start 12462 '<FieldAccess for universal_int_type_expr at nodes.lkt:17184:89>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17184'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Type_Expr (Node => Self);
--# end
--# expr-done 12462
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12463
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12464
--# expr-start 12465 '<FieldAccess for base_type at nodes.lkt:17186:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17186'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12465
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1465;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1465;
               Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Rep :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Rep := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Rep := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Enum_Rep;
--# end
--# property-start TypeAttributesRepository.enum_val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17189'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Enum_Val
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1466 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1466 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1466;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Val is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Val;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Val,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12470 '<CreateSynthNodeBuilder at nodes.lkt:17189:31>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17189'
--# expr-start 12469 '<CreateSynthNodeBuilder at nodes.lkt:17190:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17190'
--# expr-start 12466 '<Symbol Enum_Val at nodes.lkt:17191:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17191'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Enum_Val); 
--# expr-done 12466
--# expr-start 12467 '<FieldAccess for universal_int_param at nodes.lkt:17191:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17191'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_int_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Int_Param (Node => Self);
--# end
--# expr-done 12467
--# expr-start 12468 '<FieldAccess for base_type_expr at nodes.lkt:17191:93>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17191'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12468
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12469
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12470
--# expr-start 12471 '<FieldAccess for base_type at nodes.lkt:17193:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17193'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12471
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1466;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1466;
               Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Val :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Enum_Val := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Enum_Val := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Enum_Val;
--# end
--# property-start TypeAttributesRepository.read 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17196'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Read
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1467 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder : Bare_Type_Expr_Node_Builder;
Builder_1 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_2 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1467 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
            end Finalizer_Scope_1467;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Read is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Read;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Read,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12477 '<CreateSynthNodeBuilder at nodes.lkt:17196:27>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17196'
--# expr-start 12476 '<CreateSynthNodeBuilder at nodes.lkt:17197:14>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17197'
--# expr-start 12472 '<Symbol Read at nodes.lkt:17198:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17198'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Read); 
--# expr-done 12472
--# expr-start 12473 '<FieldAccess for root_stream_param at nodes.lkt:17198:45>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17198'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 12473
--# expr-start 12474 '<FieldAccess for base_type_param at nodes.lkt:17198:81>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17198'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12474
--# expr-start 12475 '<CreateCopyNodeBuilder at nodes.lkt:17198:120>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17198'
Builder := Create_Copy_Node_Builder (No_Bare_Ada_Node); 
--# expr-done 12475
Builder_1 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Builder); 
--# expr-done 12476
Builder_2 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_1); 
--# expr-done 12477
--# expr-start 12478 '<FieldAccess for base_type at nodes.lkt:17200:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17200'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12478
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_2).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1467;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1467;
               Self.Bare_Type_Attributes_Repository_Lf_State_Read :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Read := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Read := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Read;
--# end
--# property-start TypeAttributesRepository.write 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17203'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Write
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1468 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder : Bare_Type_Expr_Node_Builder;
Builder_1 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_2 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1468 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
            end Finalizer_Scope_1468;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Write is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Write;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Write,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12484 '<CreateSynthNodeBuilder at nodes.lkt:17203:28>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17203'
--# expr-start 12483 '<CreateSynthNodeBuilder at nodes.lkt:17204:14>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17204'
--# expr-start 12479 '<Symbol Write at nodes.lkt:17205:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17205'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Write); 
--# expr-done 12479
--# expr-start 12480 '<FieldAccess for root_stream_param at nodes.lkt:17205:46>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17205'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 12480
--# expr-start 12481 '<FieldAccess for base_type_param at nodes.lkt:17205:82>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17205'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12481
--# expr-start 12482 '<CreateCopyNodeBuilder at nodes.lkt:17205:121>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17205'
Builder := Create_Copy_Node_Builder (No_Bare_Ada_Node); 
--# expr-done 12482
Builder_1 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Builder); 
--# expr-done 12483
Builder_2 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_1); 
--# expr-done 12484
--# expr-start 12485 '<FieldAccess for base_type at nodes.lkt:17207:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17207'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12485
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_2).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1468;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1468;
               Self.Bare_Type_Attributes_Repository_Lf_State_Write :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Write := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Write := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Write;
--# end
--# property-start TypeAttributesRepository.input 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17210'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Input
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1469 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1469 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1469;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Input is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Input;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Input,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12490 '<CreateSynthNodeBuilder at nodes.lkt:17210:28>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17210'
--# expr-start 12489 '<CreateSynthNodeBuilder at nodes.lkt:17211:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17211'
--# expr-start 12486 '<Symbol Input at nodes.lkt:17212:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17212'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Input); 
--# expr-done 12486
--# expr-start 12487 '<FieldAccess for root_stream_param at nodes.lkt:17212:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17212'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 12487
--# expr-start 12488 '<FieldAccess for base_type_expr at nodes.lkt:17212:88>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17212'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12488
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12489
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12490
--# expr-start 12491 '<FieldAccess for base_type at nodes.lkt:17214:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17214'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12491
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1469;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1469;
               Self.Bare_Type_Attributes_Repository_Lf_State_Input :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Input := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Input := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Input;
--# end
--# property-start TypeAttributesRepository.output 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17217'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Output
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1470 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder : Bare_Type_Expr_Node_Builder;
Builder_1 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_2 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1470 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
            end Finalizer_Scope_1470;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Output is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Output;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Output,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12497 '<CreateSynthNodeBuilder at nodes.lkt:17217:29>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17217'
--# expr-start 12496 '<CreateSynthNodeBuilder at nodes.lkt:17218:14>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17218'
--# expr-start 12492 '<Symbol Output at nodes.lkt:17219:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17219'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Output); 
--# expr-done 12492
--# expr-start 12493 '<FieldAccess for root_stream_param at nodes.lkt:17219:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17219'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.root_stream_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Root_Stream_Param (Node => Self);
--# end
--# expr-done 12493
--# expr-start 12494 '<FieldAccess for base_type_param at nodes.lkt:17219:83>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17219'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12494
--# expr-start 12495 '<CreateCopyNodeBuilder at nodes.lkt:17219:122>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17219'
Builder := Create_Copy_Node_Builder (No_Bare_Ada_Node); 
--# expr-done 12495
Builder_1 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Fld, Fld_1, Builder); 
--# expr-done 12496
Builder_2 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_1); 
--# expr-done 12497
--# expr-start 12498 '<FieldAccess for base_type at nodes.lkt:17221:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17221'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12498
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_2).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1470;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1470;
               Self.Bare_Type_Attributes_Repository_Lf_State_Output :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Output := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Output := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Output;
--# end
--# property-start TypeAttributesRepository.image 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17224'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1471 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_3 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1471 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Builder_3);
            end Finalizer_Scope_1471;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Image is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Image;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Image,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12508 '<CreateSynthNodeBuilder at nodes.lkt:17224:28>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17224'
--# expr-start 12507 '<CreateSynthNodeBuilder at nodes.lkt:17225:14>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17225'
--# expr-start 12499 '<Symbol Image at nodes.lkt:17226:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17226'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Image); 
--# expr-done 12499
--# expr-start 12500 '<FieldAccess for base_type_param at nodes.lkt:17226:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17226'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12500
--# expr-start 12506 '<CreateSynthNodeBuilder at nodes.lkt:17226:86>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17226'
--# expr-start 12505 '<CreateCopyNodeBuilder at nodes.lkt:17227:29>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17227'
--# expr-start 12504 '<FieldAccess for node at nodes.lkt:17227:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17227'
--# expr-start 12503 '<Cast to BaseTypeDecl at nodes.lkt:17227:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17227'
--# expr-start 12502 '<FieldAccess for std_entity at nodes.lkt:17227:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17227'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12501 '<Symbol String at nodes.lkt:17227:45>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17227'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 12501
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 12502
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12503
Fld_2 := Cast_Result.Node;
--# expr-done 12504
Builder := Create_Copy_Node_Builder (Fld_2); 
--# expr-done 12505
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12506
Builder_2 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Builder_1); 
--# expr-done 12507
Builder_3 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_2); 
--# expr-done 12508
--# expr-start 12509 '<FieldAccess for base_type at nodes.lkt:17230:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17230'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12509
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_3).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1471;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1471;
               Self.Bare_Type_Attributes_Repository_Lf_State_Image :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Image := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Image;
--# end
--# property-start TypeAttributesRepository.wide_image 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17233'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1472 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_3 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1472 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Builder_3);
            end Finalizer_Scope_1472;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Image is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Image;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Image,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12519 '<CreateSynthNodeBuilder at nodes.lkt:17233:33>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17233'
--# expr-start 12518 '<CreateSynthNodeBuilder at nodes.lkt:17234:14>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17234'
--# expr-start 12510 '<Symbol Wide_Image at nodes.lkt:17235:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17235'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Image); 
--# expr-done 12510
--# expr-start 12511 '<FieldAccess for base_type_param at nodes.lkt:17235:52>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17235'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12511
--# expr-start 12517 '<CreateSynthNodeBuilder at nodes.lkt:17235:91>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17235'
--# expr-start 12516 '<CreateCopyNodeBuilder at nodes.lkt:17236:29>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17236'
--# expr-start 12515 '<FieldAccess for node at nodes.lkt:17236:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17236'
--# expr-start 12514 '<Cast to BaseTypeDecl at nodes.lkt:17236:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17236'
--# expr-start 12513 '<FieldAccess for std_entity at nodes.lkt:17236:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17236'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12512 '<Symbol Wide_String at nodes.lkt:17236:45>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17236'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_String); 
--# expr-done 12512
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 12513
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12514
Fld_2 := Cast_Result.Node;
--# expr-done 12515
Builder := Create_Copy_Node_Builder (Fld_2); 
--# expr-done 12516
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12517
Builder_2 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Builder_1); 
--# expr-done 12518
Builder_3 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_2); 
--# expr-done 12519
--# expr-start 12520 '<FieldAccess for base_type at nodes.lkt:17239:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17239'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12520
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_3).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1472;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1472;
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Image :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Image := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Image;
--# end
--# property-start TypeAttributesRepository.wide_wide_image 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17242'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Wide_Wide_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1473 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_3 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_3 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_2 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1473 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Builder_3);
            end Finalizer_Scope_1473;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Image is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Image;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Image,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12530 '<CreateSynthNodeBuilder at nodes.lkt:17242:38>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17242'
--# expr-start 12529 '<CreateSynthNodeBuilder at nodes.lkt:17243:14>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17243'
--# expr-start 12521 '<Symbol Wide_Wide_Image at nodes.lkt:17244:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17244'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_Image); 
--# expr-done 12521
--# expr-start 12522 '<FieldAccess for base_type_param at nodes.lkt:17244:57>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17244'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12522
--# expr-start 12528 '<CreateSynthNodeBuilder at nodes.lkt:17244:96>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17244'
--# expr-start 12527 '<CreateCopyNodeBuilder at nodes.lkt:17245:29>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17245'
--# expr-start 12526 '<FieldAccess for node at nodes.lkt:17245:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17245'
--# expr-start 12525 '<Cast to BaseTypeDecl at nodes.lkt:17245:29>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17245'
--# expr-start 12524 '<FieldAccess for std_entity at nodes.lkt:17245:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17245'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12523 '<Symbol Wide_Wide_String at nodes.lkt:17245:45>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17245'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Wide_Wide_String); 
--# expr-done 12523
--# property-call-start AdaNode.std_entity
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_1);
--# end
--# expr-done 12524
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12525
Fld_2 := Cast_Result.Node;
--# expr-done 12526
Builder := Create_Copy_Node_Builder (Fld_2); 
--# expr-done 12527
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12528
Builder_2 := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Builder_1); 
--# expr-done 12529
Builder_3 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_2); 
--# expr-done 12530
--# expr-start 12531 '<FieldAccess for base_type at nodes.lkt:17248:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17248'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12531
      Cast_Result_1 := Fld_3;
Cast_Expr := Node_Builder_Type'(Builder_3).all.Build (Parent => Cast_Result_1, Self_Node => Self); 
      Cast_Result_2 := Cast_Expr;
         Property_Result := Cast_Result_2;
   --# end
      Finalizer_Scope_1473;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1473;
               Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Image :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Wide_Wide_Image := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Wide_Wide_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Wide_Wide_Image;
--# end
--# property-start TypeAttributesRepository.put_image 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17251'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Put_Image
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1474 with Inline_Always;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Classwide_Type_Decl;
Fld_2 : Bare_Classwide_Type_Decl;
Builder : Bare_Classwide_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_3 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder_3 : Bare_Type_Expr_Node_Builder;
Builder_4 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_5 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_4 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1474 is
            begin
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
                     Dec_Ref (Builder_5);
            end Finalizer_Scope_1474;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Put_Image is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Put_Image;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Put_Image,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12543 '<CreateSynthNodeBuilder at nodes.lkt:17251:32>' Builder_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17251'
--# expr-start 12542 '<CreateSynthNodeBuilder at nodes.lkt:17252:14>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17252'
--# expr-start 12532 '<Symbol Put_Image at nodes.lkt:17253:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17253'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Put_Image); 
--# expr-done 12532
--# expr-start 12539 '<CreateSynthNodeBuilder at nodes.lkt:17253:50>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17253'
--# expr-start 12533 '<Symbol Buffer at nodes.lkt:17254:28>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17254'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Buffer); 
--# expr-done 12533
--# expr-start 12538 '<CreateSynthNodeBuilder at nodes.lkt:17254:50>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17254'
--# expr-start 12537 '<CreateCopyNodeBuilder at nodes.lkt:17255:33>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17255'
--# expr-start 12536 '<FieldAccess for node at nodes.lkt:17255:33>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17255'
--# expr-start 12535 '<FieldAccess for classwide_type at nodes.lkt:17255:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17255'
--# expr-start 12534 '<FieldAccess for root_buffer_type at nodes.lkt:17255:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17255'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_buffer_type
Fld := Libadalang.Implementation.Ada_Node_P_Root_Buffer_Type (Node => Self);
--# end
--# expr-done 12534
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12535
Fld_2 := Fld_1.Node;
--# expr-done 12536
Builder := Create_Copy_Node_Builder (Fld_2); 
--# expr-done 12537
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12538
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_1, Builder_1); 
--# expr-done 12539
--# expr-start 12540 '<FieldAccess for base_type_param at nodes.lkt:17257:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17257'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_3 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12540
--# expr-start 12541 '<CreateCopyNodeBuilder at nodes.lkt:17257:67>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17257'
Builder_3 := Create_Copy_Node_Builder (No_Bare_Ada_Node); 
--# expr-done 12541
Builder_4 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym, Builder_2, Fld_3, Builder_3); 
--# expr-done 12542
Builder_5 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_4); 
--# expr-done 12543
--# expr-start 12544 '<FieldAccess for base_type at nodes.lkt:17259:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17259'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12544
      Cast_Result := Fld_4;
Cast_Expr := Node_Builder_Type'(Builder_5).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1474;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1474;
               Self.Bare_Type_Attributes_Repository_Lf_State_Put_Image :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Put_Image := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Put_Image := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Put_Image;
--# end
--# property-start TypeAttributesRepository.asm_input 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17262'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Asm_Input
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1475 with Inline_Always;
            procedure Finalizer_Scope_3593 with Inline_Always;
      Local_Input_Type : Bare_Base_Type_Decl;
Var_Expr_289 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Fld_2 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_5 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder_3 : Bare_Base_Type_Decl_Node_Builder;
Builder_4 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_5 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_6 : Bare_Synthetic_Subp_Decl_Node_Builder;
Let_Result : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_6 : Bare_Base_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1475 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
                     Dec_Ref (Builder_5);
                     Dec_Ref (Builder_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1475;
            procedure Finalizer_Scope_3593 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3593;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Input is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Input;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Input,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12568 '<Let at nodes.lkt:17263:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
--# scope-start
--# expr-start 12553 '<Cast to BaseTypeDecl at nodes.lkt:17263:26>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
--# expr-start 12548 '<FieldAccess for get_unit_root_decl at nodes.lkt:17263:26>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12547 '<ArrayLiteral at nodes.lkt:17264:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17264'
--# expr-start 12545 '<Symbol System at nodes.lkt:17264:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17264'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 12545
--# expr-start 12546 '<Symbol Machine_Code at nodes.lkt:17264:25>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17264'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Code); 
--# expr-done 12546
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 12547
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 12548
Var_Expr_289 := Fld;
if Var_Expr_289 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 12552 '<FieldAccess for node at nodes.lkt:17263:26>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
--# expr-start 12551 '<EnvGet at nodes.lkt:17263:26>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
--# expr-start 12549 '<FieldAccess for children_env at nodes.lkt:17263:26>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17263'
      if Var_Expr_289 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_289);
--# expr-done 12549
--# expr-start 12550 '<Symbol Asm_Input_Operand at nodes.lkt:17266:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17266'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input_Operand); 
--# expr-done 12550
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 12551
Fld_2 := Env_Get_Result.Node;
--# expr-done 12552
Scope_Result := Fld_2;
   --# end
      Finalizer_Scope_3593;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Result_Var;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12553
Local_Input_Type := Cast_Result; 
--# bind input_type Local_Input_Type
--# expr-start 12567 '<CreateSynthNodeBuilder at nodes.lkt:17269:9>' Builder_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17269'
--# expr-start 12566 '<CreateSynthNodeBuilder at nodes.lkt:17270:18>' Builder_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17270'
--# expr-start 12554 '<Symbol Asm_Input at nodes.lkt:17271:29>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17271'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Input); 
--# expr-done 12554
--# expr-start 12562 '<CreateSynthNodeBuilder at nodes.lkt:17271:54>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17271'
--# expr-start 12555 '<Symbol S at nodes.lkt:17272:32>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17272'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 12555
--# expr-start 12561 '<CreateSynthNodeBuilder at nodes.lkt:17272:49>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17272'
--# expr-start 12560 '<CreateCopyNodeBuilder at nodes.lkt:17273:37>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17273'
--# expr-start 12559 '<FieldAccess for node at nodes.lkt:17273:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17273'
--# expr-start 12558 '<Cast to BaseTypeDecl at nodes.lkt:17273:37>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17273'
--# expr-start 12557 '<FieldAccess for std_entity at nodes.lkt:17273:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17273'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12556 '<Symbol String at nodes.lkt:17273:53>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17273'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 12556
--# property-call-start AdaNode.std_entity
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 12557
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12558
Fld_4 := Cast_Result_1.Node;
--# expr-done 12559
Builder := Create_Copy_Node_Builder (Fld_4); 
--# expr-done 12560
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12561
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_4, Builder_1); 
--# expr-done 12562
--# expr-start 12563 '<FieldAccess for base_type_param at nodes.lkt:17275:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17275'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12563
--# expr-start 12565 '<CreateSynthNodeBuilder at nodes.lkt:17275:71>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17275'
--# expr-start 12564 '<CreateCopyNodeBuilder at nodes.lkt:17275:109>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17275'
Builder_3 := Create_Copy_Node_Builder (Local_Input_Type); 
--# expr-done 12564
Builder_4 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder_3); 
--# expr-done 12565
Builder_5 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym_3, Builder_2, Fld_5, Builder_4); 
--# expr-done 12566
Builder_6 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_5); 
--# expr-done 12567
Let_Result := Builder_6; Inc_Ref (Let_Result);
--# end
--# expr-done 12568
--# expr-start 12569 '<FieldAccess for base_type at nodes.lkt:17278:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17278'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12569
      Cast_Result_2 := Fld_6;
Cast_Expr := Node_Builder_Type'(Let_Result).all.Build (Parent => Cast_Result_2, Self_Node => Self); 
      Cast_Result_3 := Cast_Expr;
         Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1475;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1475;
                     Finalizer_Scope_3593;
               Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Input :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Input := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Input := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Asm_Input;
--# end
--# property-start TypeAttributesRepository.asm_output 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17281'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Asm_Output
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1476 with Inline_Always;
            procedure Finalizer_Scope_3594 with Inline_Always;
      Local_Output_Type : Bare_Base_Type_Decl;
Var_Expr_290 : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Fld_2 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Builder : Bare_Base_Type_Decl_Node_Builder;
Builder_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_2 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_5 : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Builder_3 : Bare_Base_Type_Decl_Node_Builder;
Builder_4 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder_5 : Bare_Synthetic_Binary_Spec_Node_Builder;
Builder_6 : Bare_Synthetic_Subp_Decl_Node_Builder;
Let_Result : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_6 : Bare_Base_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_3 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1476 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
                     Dec_Ref (Builder_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Builder_3);
                     Dec_Ref (Builder_4);
                     Dec_Ref (Builder_5);
                     Dec_Ref (Builder_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1476;
            procedure Finalizer_Scope_3594 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3594;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Output is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Output;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Output,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12593 '<Let at nodes.lkt:17282:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
--# scope-start
--# expr-start 12578 '<Cast to BaseTypeDecl at nodes.lkt:17282:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
--# expr-start 12573 '<FieldAccess for get_unit_root_decl at nodes.lkt:17282:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12572 '<ArrayLiteral at nodes.lkt:17283:13>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17283'
--# expr-start 12570 '<Symbol System at nodes.lkt:17283:14>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17283'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_System); 
--# expr-done 12570
--# expr-start 12571 '<Symbol Machine_Code at nodes.lkt:17283:25>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17283'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Machine_Code); 
--# expr-done 12571
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 12572
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 12573
Var_Expr_290 := Fld;
if Var_Expr_290 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 12577 '<FieldAccess for node at nodes.lkt:17282:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
--# expr-start 12576 '<EnvGet at nodes.lkt:17282:27>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
--# expr-start 12574 '<FieldAccess for children_env at nodes.lkt:17282:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17282'
      if Var_Expr_290 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr_290);
--# expr-done 12574
--# expr-start 12575 '<Symbol Asm_Output_Operand at nodes.lkt:17285:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17285'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output_Operand); 
--# expr-done 12575
Env_Get_Result := AST_Envs.Get_First (Self => Fld_1, Key => Thin (Sym_2), Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 12576
Fld_2 := Env_Get_Result.Node;
--# expr-done 12577
Scope_Result := Fld_2;
   --# end
      Finalizer_Scope_3594;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Result_Var;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12578
Local_Output_Type := Cast_Result; 
--# bind output_type Local_Output_Type
--# expr-start 12592 '<CreateSynthNodeBuilder at nodes.lkt:17288:9>' Builder_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17288'
--# expr-start 12591 '<CreateSynthNodeBuilder at nodes.lkt:17289:18>' Builder_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17289'
--# expr-start 12579 '<Symbol Asm_Output at nodes.lkt:17290:29>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17290'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Asm_Output); 
--# expr-done 12579
--# expr-start 12587 '<CreateSynthNodeBuilder at nodes.lkt:17290:55>' Builder_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17290'
--# expr-start 12580 '<Symbol S at nodes.lkt:17291:32>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17291'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_S); 
--# expr-done 12580
--# expr-start 12586 '<CreateSynthNodeBuilder at nodes.lkt:17291:49>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17291'
--# expr-start 12585 '<CreateCopyNodeBuilder at nodes.lkt:17292:37>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17292'
--# expr-start 12584 '<FieldAccess for node at nodes.lkt:17292:37>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17292'
--# expr-start 12583 '<Cast to BaseTypeDecl at nodes.lkt:17292:37>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17292'
--# expr-start 12582 '<FieldAccess for std_entity at nodes.lkt:17292:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17292'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12581 '<Symbol String at nodes.lkt:17292:53>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17292'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_String); 
--# expr-done 12581
--# property-call-start AdaNode.std_entity
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 12582
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 12583
Fld_4 := Cast_Result_1.Node;
--# expr-done 12584
Builder := Create_Copy_Node_Builder (Fld_4); 
--# expr-done 12585
Builder_1 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder); 
--# expr-done 12586
Builder_2 := Create_Bare_Synthetic_Formal_Param_Decl_Node_Builder (Sym_4, Builder_1); 
--# expr-done 12587
--# expr-start 12588 '<FieldAccess for base_type_param at nodes.lkt:17294:32>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17294'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_param
Fld_5 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Param (Node => Self);
--# end
--# expr-done 12588
--# expr-start 12590 '<CreateSynthNodeBuilder at nodes.lkt:17294:71>' Builder_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17294'
--# expr-start 12589 '<CreateCopyNodeBuilder at nodes.lkt:17294:109>' Builder_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17294'
Builder_3 := Create_Copy_Node_Builder (Local_Output_Type); 
--# expr-done 12589
Builder_4 := Create_Bare_Synthetic_Type_Expr_Node_Builder (Builder_3); 
--# expr-done 12590
Builder_5 := Create_Bare_Synthetic_Binary_Spec_Node_Builder (Sym_3, Builder_2, Fld_5, Builder_4); 
--# expr-done 12591
Builder_6 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder_5); 
--# expr-done 12592
Let_Result := Builder_6; Inc_Ref (Let_Result);
--# end
--# expr-done 12593
--# expr-start 12594 '<FieldAccess for base_type at nodes.lkt:17297:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17297'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12594
      Cast_Result_2 := Fld_6;
Cast_Expr := Node_Builder_Type'(Let_Result).all.Build (Parent => Cast_Result_2, Self_Node => Self); 
      Cast_Result_3 := Cast_Expr;
         Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1476;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1476;
                     Finalizer_Scope_3594;
               Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Output :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Asm_Output := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Asm_Output := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Asm_Output;
--# end
--# property-start TypeAttributesRepository.model 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17300'
pragma Warnings (Off, "is not referenced");
 function Type_Attributes_Repository_F_Model
  (Node : Bare_Type_Attributes_Repository
  )
   return Bare_Basic_Subp_Decl
is
   Self : Bare_Type_Attributes_Repository  := Bare_Type_Attributes_Repository (Node);
      --# bind self Self
   Property_Result : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1477 with Inline_Always;
      Sym : Symbol_Type;
Fld : Bare_Synthetic_Formal_Param_Decl_Node_Builder;
Fld_1 : Bare_Synthetic_Type_Expr_Node_Builder;
Builder : Bare_Synthetic_Unary_Spec_Node_Builder;
Builder_1 : Bare_Synthetic_Subp_Decl_Node_Builder;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Cast_Expr : Bare_Synthetic_Subp_Decl;
Cast_Result_1 : Bare_Basic_Subp_Decl;
            procedure Finalizer_Scope_1477 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Builder);
                     Dec_Ref (Builder_1);
            end Finalizer_Scope_1477;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      case Self.Bare_Type_Attributes_Repository_Lf_State_Model is
         when Uninitialized =>
            null;
         when Initialized =>
            Property_Result := Self.Bare_Type_Attributes_Repository_Lf_Stg_Model;
            return Property_Result;
         when Error_Initialization_State =>
            Reraise_Initialization_Error
              (Self,
               Self.Bare_Type_Attributes_Repository_Lf_State_Model,
               "lazy field memoization");
      end case;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12599 '<CreateSynthNodeBuilder at nodes.lkt:17300:28>' Builder_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17300'
--# expr-start 12598 '<CreateSynthNodeBuilder at nodes.lkt:17301:14>' Builder 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17301'
--# expr-start 12595 '<Symbol Model at nodes.lkt:17302:25>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17302'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Model); 
--# expr-done 12595
--# expr-start 12596 '<FieldAccess for universal_real_param at nodes.lkt:17302:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17302'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.universal_real_param
Fld := Libadalang.Implementation.Type_Attributes_Repository_F_Universal_Real_Param (Node => Self);
--# end
--# expr-done 12596
--# expr-start 12597 '<FieldAccess for base_type_expr at nodes.lkt:17302:91>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17302'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeAttributesRepository.base_type_expr
Fld_1 := Libadalang.Implementation.Type_Attributes_Repository_F_Base_Type_Expr (Node => Self);
--# end
--# expr-done 12597
Builder := Create_Bare_Synthetic_Unary_Spec_Node_Builder (Sym, Fld, Fld_1); 
--# expr-done 12598
Builder_1 := Create_Bare_Synthetic_Subp_Decl_Node_Builder (Builder); 
--# expr-done 12599
--# expr-start 12600 '<FieldAccess for base_type at nodes.lkt:17304:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17304'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Type_Attributes_Repository_Base_Type;
--# expr-done 12600
      Cast_Result := Fld_2;
Cast_Expr := Node_Builder_Type'(Builder_1).all.Build (Parent => Cast_Result, Self_Node => Self); 
      Cast_Result_1 := Cast_Expr;
         Property_Result := Cast_Result_1;
   --# end
      Finalizer_Scope_1477;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1477;
               Self.Bare_Type_Attributes_Repository_Lf_State_Model :=
                 Initialization_Error (Exc);
            raise;
      end;
      Self.Bare_Type_Attributes_Repository_Lf_State_Model := Initialized;
      Self.Bare_Type_Attributes_Repository_Lf_Stg_Model := Property_Result;
   return Property_Result;
end Type_Attributes_Repository_F_Model;
--# end
   --
   --  Primitives for Bare_Type_Def
   --
--# property-start TypeDef.is_real_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17312'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Real_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12601 '<FieldAccess for is_float_type at nodes.lkt:17312:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17312'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12601
if Fld then
   If_Result := True;
else
   --# expr-start 12602 '<FieldAccess for is_fixed_point at nodes.lkt:17312:56>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17312'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12602
   If_Result := Fld_1;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Real_Type;
--# end
--# property-start TypeDef.predefined_equality_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17314'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Predefined_Equality_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1479 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1479 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1479;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12613 '<Let at nodes.lkt:17315:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17315'
--# scope-start
--# expr-start 12604 '<Cast to TypeDecl at nodes.lkt:17315:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17315'
--# expr-start 12603 '<parent at nodes.lkt:17315:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17315'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12603
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12604
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12612 '<Let at nodes.lkt:17316:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17316'
--# scope-start
--# expr-start 12606 '<FieldAccess for node at nodes.lkt:17316:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17316'
--# expr-start 12605 '<FieldAccess for bool_type at nodes.lkt:17316:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17316'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12605
Fld_2 := Fld_1.Node;
--# expr-done 12606
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12611 '<ArrayLiteral at nodes.lkt:17318:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17318'
--# expr-start 12608 '<FieldAccess for create_binop_assoc at nodes.lkt:17318:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17318'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12607 '<Symbol "=" at nodes.lkt:17319:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17319'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12607
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Local_Bool_Type);
--# end
--# expr-done 12608
--# expr-start 12610 '<FieldAccess for create_binop_assoc at nodes.lkt:17320:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17320'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12609 '<Symbol "/=" at nodes.lkt:17321:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17321'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12609
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Local_Bool_Type);
--# end
--# expr-done 12610
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4)); 
--# expr-done 12611
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12612
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12613
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1479;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1479;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Predefined_Equality_Operators;
--# end
--# property-start TypeDef.base_types 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17327'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Types
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1480 with Inline_Always;
            procedure Finalizer_Scope_3595 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Array_Lit : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1480 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1480;
            procedure Finalizer_Scope_3595 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_3595;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12614 '<FieldAccess for base_type at nodes.lkt:17328:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17328'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12614
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 12615 '<ArrayLiteral at nodes.lkt:17328:37>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17328'
Array_Lit := Create_Internal_Entity_Base_Type_Decl_Array (Internal_Internal_Entity_Base_Type_Decl_Array'(1 => Var_Expr_0)); 
--# expr-done 12615
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3595;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 12616 '<FieldAccess for base_interfaces at nodes.lkt:17328:45>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17328'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12616
Concat_Result := Concat (Result_Var, Fld_1); 
         Property_Result := Concat_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1480;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1480;
                     Finalizer_Scope_3595;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Types;
--# end
--# property-start TypeDef.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17331'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Defining_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1481 with Inline_Always;
            procedure Finalizer_Scope_3597 with Inline_Always;
            procedure Finalizer_Scope_3598 with Inline_Always;
            procedure Finalizer_Scope_3599 with Inline_Always;
      Var_Expr : Bare_Ada_Node;
Var_Expr_291 : Internal_Entity_Base_Type_Decl;
Var_Expr_292 : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Scope_Result : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Bound_Dottable_Type : Bare_Ada_Node;
Fld_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Array_Lit_2 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Cast_Expr : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Array_Lit_3 : Lexical_Env_Array_Access;
Group_Env_2 : Lexical_Env := Empty_Env;
Cast_Expr_1 : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Access_Def;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Fld_14 : Lexical_Env := Empty_Env;
Array_Lit_4 : Lexical_Env_Array_Access;
Group_Env_3 : Lexical_Env := Empty_Env;
Fld_15 : Lexical_Env := Empty_Env;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Is_A_2 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Is_A_3 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1481 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Array_Lit_3);
                     Dec_Ref (Group_Env_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Array_Lit_4);
                     Dec_Ref (Group_Env_3);
                     Dec_Ref (Fld_15);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_1481;
            procedure Finalizer_Scope_3597 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_3597;
            procedure Finalizer_Scope_3598 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_3598;
            procedure Finalizer_Scope_3599 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_3599;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12653 '<If at nodes.lkt:17337:9>' If_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12617 '<IsA RecordTypeDef, PrivateTypeDef at nodes.lkt:17337:12>' Is_A_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
Is_A_3 := Self /= null 
and then Self.Kind in Ada_Record_Type_Def_Range | Ada_Private_Type_Def_Range; 
--# expr-done 12617
if Is_A_3 then
   --# expr-start 12622 '<env_group at nodes.lkt:17337:56>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12621 '<ArrayLiteral at nodes.lkt:17337:56>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12618 '<FieldAccess for children_env at nodes.lkt:17337:57>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12618
--# expr-start 12619 '<FieldAccess for dottable_subps_env at nodes.lkt:17337:78>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_1 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12619
--# expr-start 12620 '<FieldAccess for previous_part_env at nodes.lkt:17337:105>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12620
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1, 3 => Fld_2)); 
--# expr-done 12621
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 12622
   If_Result_3 := Group_Env;
else
   --# expr-start 12652 '<If at nodes.lkt:17337:9>' If_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12623 '<IsA DerivedTypeDef, InterfaceTypeDef at nodes.lkt:17340:14>' Is_A_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17340'
Is_A_2 := Self /= null 
and then Self.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 12623
if Is_A_2 then
   --# expr-start 12634 '<env_group at nodes.lkt:17344:13>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17344'
--# expr-start 12626 '<ArrayLiteral at nodes.lkt:17345:17>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17345'
--# expr-start 12624 '<FieldAccess for children_env at nodes.lkt:17345:18>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17345'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12624
--# expr-start 12625 '<FieldAccess for dottable_subps_env at nodes.lkt:17345:39>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17345'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12625
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_4)); 
--# expr-done 12626
--# expr-start 12631 '<bind at nodes.lkt:17346:21>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17346'
Var_Expr := Dottable_Type;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12627 '<parent at nodes.lkt:17346:60>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17346'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Self.Parent; 
--# expr-done 12627
   Result_Var := Fld_5;
end if;
Bound_Dottable_Type := Result_Var; 
--# expr-start 12630 '<Map at nodes.lkt:17348:21>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17348'
   --# expr-start 12628 '<FieldAccess for base_types at nodes.lkt:17348:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17348'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.base_types
Fld_6 := Libadalang.Implementation.Type_Def_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12628
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_6;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_0
Var_Expr_291 := Item_0;
if Var_Expr_291 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12629 '<FieldAccess for defining_env at nodes.lkt:17348:51>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17348'
      if Var_Expr_291.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_291.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Bound_Dottable_Type, E_Info => Var_Expr_291.Info);
--# end
--# expr-done 12629
Scope_Result_1 := Fld_7;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3598;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Lexical_Env := Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3597;
         end loop;
      end;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12630
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12631
Concat_Result := Concat (Array_Lit_1, Dyn_Var_Bind_Result); 
--# expr-start 12633 '<ArrayLiteral at nodes.lkt:17349:21>' Array_Lit_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17349'
--# expr-start 12632 '<FieldAccess for previous_part_env at nodes.lkt:17349:22>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17349'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_8 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12632
Array_Lit_2 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_8)); 
--# expr-done 12633
Concat_Result_1 := Concat (Concat_Result, Array_Lit_2); 
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 12634
   If_Result_2 := Group_Env_1;
else
   --# expr-start 12651 '<If at nodes.lkt:17337:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12635 '<IsA ArrayTypeDef at nodes.lkt:17354:14>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Array_Type_Def_Range; 
--# expr-done 12635
if Is_A_1 then
   --# expr-start 12641 '<env_group at nodes.lkt:17354:40>' Group_Env_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
--# expr-start 12640 '<ArrayLiteral at nodes.lkt:17354:40>' Array_Lit_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
--# expr-start 12638 '<FieldAccess for defining_env at nodes.lkt:17354:41>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
--# expr-start 12637 '<FieldAccess for comp_type at nodes.lkt:17354:41>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
--# expr-start 12636 '<Cast to ArrayTypeDef at nodes.lkt:17354:41>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Array_Type_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Array_Type_Def;
   end if;
--# expr-done 12636
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_9 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 12637
      if Fld_9.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld_9.Info);
--# end
--# expr-done 12638
--# expr-start 12639 '<FieldAccess for dottable_subps_env at nodes.lkt:17354:91>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17354'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_11 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12639
Array_Lit_3 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_10, 2 => Fld_11)); 
--# expr-done 12640
Group_Env_2 := Group (Array_Lit_3, No_Metadata); 
--# expr-done 12641
   If_Result_1 := Group_Env_2;
else
   --# expr-start 12650 '<If at nodes.lkt:17337:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17337'
--# expr-start 12642 '<IsA AccessDef at nodes.lkt:17355:14>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
Is_A := Self /= null 
and then Self.Kind in Ada_Access_Def; 
--# expr-done 12642
if Is_A then
   --# expr-start 12648 '<env_group at nodes.lkt:17355:37>' Group_Env_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
--# expr-start 12647 '<ArrayLiteral at nodes.lkt:17355:37>' Array_Lit_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
--# expr-start 12644 '<FieldAccess for accessed_type at nodes.lkt:17355:38>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
--# expr-start 12643 '<Cast to AccessDef at nodes.lkt:17355:38>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Access_Def
   then
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Access_Def;
   end if;
--# expr-done 12643
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_12 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Cast_Result_1.Node, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 12644
Var_Expr_292 := Fld_12;
if Var_Expr_292 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12645 '<FieldAccess for defining_env at nodes.lkt:17355:38>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
      if Var_Expr_292.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_292.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Var_Expr_292.Info);
--# end
--# expr-done 12645
Scope_Result_2 := Fld_13;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3599;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
--# expr-start 12646 '<FieldAccess for dottable_subps_env at nodes.lkt:17355:90>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17355'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_14 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12646
Array_Lit_4 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Result_Var_2, 2 => Fld_14)); 
--# expr-done 12647
Group_Env_3 := Group (Array_Lit_4, No_Metadata); 
--# expr-done 12648
   If_Result := Group_Env_3;
else
   --# expr-start 12649 '<FieldAccess for dottable_subps_env at nodes.lkt:17360:14>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17360'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_15 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, Dottable_Type => Dottable_Type, E_Info => Ent.Info);
--# end
--# expr-done 12649
   If_Result := Fld_15;
end if;
      Inc_Ref (If_Result);
--# expr-done 12650
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 12651
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 12652
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 12653
         Property_Result := If_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1481;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1481;
                     Finalizer_Scope_3597;
                     Finalizer_Scope_3598;
                     Finalizer_Scope_3599;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Defining_Env;
--# end
--# property-start TypeDef.containing_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17363'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Containing_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12655 '<Cast to TypeDecl at nodes.lkt:17363:47>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17363'
--# expr-start 12654 '<parent at nodes.lkt:17363:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17363'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12654
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 12655
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Containing_Type;
--# end
--# property-start TypeDef.previous_part 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17365'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12657 '<FieldAccess for previous_part at nodes.lkt:17366:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17366'
--# expr-start 12656 '<FieldAccess for containing_type at nodes.lkt:17366:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17366'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12656
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.previous_part
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => True, E_Info => Fld.Info);
--# end
--# expr-done 12657
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Previous_Part;
--# end
--# property-start TypeDef.previous_part_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17369'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Dottable_Type : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1484 with Inline_Always;
            procedure Finalizer_Scope_3600 with Inline_Always;
      Var_Expr_293 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1484 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1484;
            procedure Finalizer_Scope_3600 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3600;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12658 '<FieldAccess for previous_part at nodes.lkt:17370:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17370'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.previous_part
Fld := Libadalang.Implementation.Type_Def_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12658
Var_Expr_293 := Fld;
if Var_Expr_293 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12659 '<FieldAccess for defining_env at nodes.lkt:17370:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17370'
      if Var_Expr_293.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_293.Node, Origin => Origin, Include_Ud_Indexing => False, Dottable_Type => Dottable_Type, E_Info => Var_Expr_293.Info);
--# end
--# expr-done 12659
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3600;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1484;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1484;
                     Finalizer_Scope_3600;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Previous_Part_Env;
--# end
--# property-start TypeDef.dottable_subps_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17373'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Dottable_Subps_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
      ; Dottable_Type : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1485 with Inline_Always;
      Var_Expr_0 : Bare_Ada_Node;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Info;
Fld_3 : Env_Rebindings;
Rebound_Env : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Fld_4 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Scope_Result : Internal_Metadata;
Result_Var : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1485 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Rebound_Env);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1485;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12668 '<env_group at nodes.lkt:17384:9>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
--# expr-start 12665 '<ArrayLiteral at nodes.lkt:17384:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
--# expr-start 12664 '<rebind_env at nodes.lkt:17384:10>' Rebound_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
--# expr-start 12661 '<FieldAccess for dottable_subps_env at nodes.lkt:17384:10>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
--# expr-start 12660 '<FieldAccess for containing_type at nodes.lkt:17384:10>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 12660
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Fld.Node);
--# end
--# expr-done 12661
--# expr-start 12663 '<FieldAccess for rebindings at nodes.lkt:17384:63>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
--# expr-start 12662 '<FieldAccess for info at nodes.lkt:17384:63>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17384'
Fld_2 := Ent.Info;
--# expr-done 12662
Fld_3 := Fld_2.Rebindings;
--# expr-done 12663
Rebound_Env := Rebind_Env (Fld_1, Fld_3); 
--# expr-done 12664
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Rebound_Env)); 
--# expr-done 12665
Var_Expr_0 := Dottable_Type;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind t Var_Expr_0
--# expr-start 12667 '<New Metadata at nodes.lkt:17386:24>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17386'
--# expr-start 12666 '<parent at nodes.lkt:17387:31>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17387'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Parent; 
--# expr-done 12666
New_Struct := (Dottable_Subp => False, Primitive => Fld_4, Primitive_Real_Type => Var_Expr_0); 
--# expr-done 12667
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Metadata;
end if;
Group_Env := Group (Array_Lit, Result_Var); 
--# expr-done 12668
         Property_Result := Group_Env;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1485;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1485;
            raise;
      end;
   return Property_Result;
end Type_Def_P_Dottable_Subps_Env;
--# end
--# property-start '[dispatcher]TypeDef.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.discrete_range
                  Property_Result := Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.discrete_range
                  Property_Result := Derived_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.discrete_range
                  Property_Result := Mod_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.discrete_range
                  Property_Result := Signed_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Discrete_Range;
--# end
--# property-start '[dispatcher]TypeDef.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.array_ndims
                  Property_Result := Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.array_ndims
                  Property_Result := Array_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.array_ndims
                  Property_Result := Derived_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Array_Ndims;
--# end
--# property-start '[dispatcher]TypeDef.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Decimal_Fixed_Point_Def | Ada_Ordinary_Fixed_Point_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_float_type
                  Property_Result := Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_float_type
                  Property_Result := Derived_Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.is_float_type
                  Property_Result := Floating_Point_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Private_Type_Def | Ada_Floating_Point_Def | Ada_Record_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_fixed_point
                  Property_Result := Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_fixed_point
                  Property_Result := Derived_Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.is_fixed_point
                  Property_Result := Decimal_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.is_fixed_point
                  Property_Result := Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]TypeDef.predefined_operators' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Predefined_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Derived_Type_Def | Ada_Interface_Type_Def =>
                  --# property-call-start TypeDef.predefined_operators
                  Property_Result := Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.predefined_operators
                  Property_Result := Access_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.predefined_operators
                  Property_Result := Array_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.predefined_operators
                  Property_Result := Enum_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.predefined_operators
                  Property_Result := Formal_Discrete_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.predefined_operators
                  Property_Result := Mod_Int_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.predefined_operators
                  Property_Result := Private_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start RealTypeDef.predefined_operators
                  Property_Result := Real_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.predefined_operators
                  Property_Result := Decimal_Fixed_Point_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.predefined_operators
                  Property_Result := Ordinary_Fixed_Point_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.predefined_operators
                  Property_Result := Record_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.predefined_operators
                  Property_Result := Signed_Int_Type_Def_P_Predefined_Operators
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Predefined_Operators;
--# end
--# property-start '[dispatcher]TypeDef.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Enum_Type_Def | Ada_Interface_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_discrete_type
                  Property_Result := Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.is_discrete_type
                  Property_Result := Formal_Discrete_Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_int_type
                  Property_Result := Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_int_type
                  Property_Result := Derived_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_int_type
                  Property_Result := Mod_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_int_type
                  Property_Result := Signed_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Int_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_access_type
                  Property_Result := Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.is_access_type
                  Property_Result := Access_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_access_type
                  Property_Result := Derived_Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Access_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_char_type
                  Property_Result := Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_char_type
                  Property_Result := Derived_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_char_type
                  Property_Result := Enum_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_enum_type
                  Property_Result := Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_enum_type
                  Property_Result := Derived_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_enum_type
                  Property_Result := Enum_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]TypeDef.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.accessed_type
                  Property_Result := Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.accessed_type
                  Property_Result := Access_To_Subp_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Access_Def =>
                  --# property-call-start AnonymousTypeAccessDef.accessed_type
                  Property_Result := Anonymous_Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.accessed_type
                  Property_Result := Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.accessed_type
                  Property_Result := Derived_Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Accessed_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def | Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_tagged_type
                  Property_Result := Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_tagged_type
                  Property_Result := Derived_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_tagged_type
                  Property_Result := Interface_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.is_tagged_type
                  Property_Result := Private_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_tagged_type
                  Property_Result := Record_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_task_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Task_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_task_type
                  Property_Result := Type_Def_P_Is_Task_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_task_type
                  Property_Result := Derived_Type_Def_P_Is_Task_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_task_type
                  Property_Result := Interface_Type_Def_P_Is_Task_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Task_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_limited_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Limited_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def | Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_limited_type
                  Property_Result := Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.is_limited_type
                  Property_Result := Array_Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_limited_type
                  Property_Result := Derived_Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_limited_type
                  Property_Result := Interface_Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.is_limited_type
                  Property_Result := Private_Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_limited_type
                  Property_Result := Record_Type_Def_P_Is_Limited_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Limited_Type;
--# end
--# property-start '[dispatcher]TypeDef.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_type
                  Property_Result := Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_type
                  Property_Result := Derived_Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Base_Type;
--# end
--# property-start '[dispatcher]TypeDef.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_interfaces
                  Property_Result := Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_interfaces
                  Property_Result := Derived_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.base_interfaces
                  Property_Result := Interface_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Base_Interfaces;
--# end
--# property-start '[dispatcher]TypeDef.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Formal_Discrete_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def | Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_static
                  Property_Result := Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.is_static
                  Property_Result := Array_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_static
                  Property_Result := Derived_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_static
                  Property_Result := Enum_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_static
                  Property_Result := Mod_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.is_static
                  Property_Result := Real_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_static
                  Property_Result := Signed_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Def_P_Is_Static;
--# end
--# property-start TypeDef.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17393'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Discrete_Range;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Discrete_Range;
--# end
--# property-start TypeDef.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17398'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := 0;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17402'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Float_Type;
--# end
--# property-start TypeDef.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17406'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Fixed_Point;
--# end
--# property-start TypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17413'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Predefined_Operators
  (Node : Bare_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Env_Assoc_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Predefined_Operators;
--# end
--# property-start TypeDef.is_discrete_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17416'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
     Ent : Internal_Entity_Type_Def :=
       Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Result_Var : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12669 '<FieldAccess for base_type at nodes.lkt:17416:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17416'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12669
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 12670 '<FieldAccess for is_discrete_type at nodes.lkt:17417:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17417'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12670
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 12671 '<FieldAccess for is_int_type at nodes.lkt:17417:52>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17417'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12671
if Fld_2 then
   If_Result := True;
else
   --# expr-start 12672 '<FieldAccess for is_enum_type at nodes.lkt:17417:74>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17417'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12672
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 12673 '<FieldAccess for is_char_type at nodes.lkt:17417:97>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17417'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12673
   If_Result_1 := Fld_4;
end if;
   Result_Var := If_Result_1;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Discrete_Type;
--# end
--# property-start TypeDef.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17422'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Int_Type;
--# end
--# property-start TypeDef.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17426'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Access_Type;
--# end
--# property-start TypeDef.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17429'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Char_Type;
--# end
--# property-start TypeDef.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17432'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Enum_Type;
--# end
--# property-start TypeDef.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17435'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Accessed_Type;
--# end
--# property-start TypeDef.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17439'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Tagged_Type;
--# end
--# property-start TypeDef.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17442'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Task_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Task_Type;
--# end
--# property-start TypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17444'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Limited_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Limited_Type;
--# end
--# property-start TypeDef.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17448'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Entity_Base_Type_Decl;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17451'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
            Inc_Ref (Property_Result);
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17455'
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def  := Bare_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Access_Def
   --
      procedure Initialize_Fields_For_Access_Def
        (Self : Bare_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Self.Access_Def_F_Has_Not_Null := Access_Def_F_Has_Not_Null;
      end Initialize_Fields_For_Access_Def;
   function Access_Def_F_Has_Not_Null
     (Node : Bare_Access_Def) return Bare_Not_Null
   is
   begin
         return Node.Access_Def_F_Has_Not_Null;
   end;
--# property-start AccessDef.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17464'
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Is_Access_Type
  (Node : Bare_Access_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Access_Def  := Bare_Access_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Access_Def_P_Is_Access_Type;
--# end
--# property-start AccessDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17467'
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Predefined_Operators
  (Node : Bare_Access_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Access_Def  := Bare_Access_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1504 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1504 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1504;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Access_Def_Access_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Access_Def,
                                As_Bare_Access_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12674 '<FieldAccess for predefined_equality_operators at nodes.lkt:17468:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17468'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 12674
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1504;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1504;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Access_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Access_To_Subp_Def
   --
      procedure Access_To_Subp_Def_Pre_Env_Actions
        (Self            : Bare_Access_To_Subp_Def;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_225 (Node => 
Self);
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Names);
      end;
      end;
      procedure Initialize_Fields_For_Access_To_Subp_Def
        (Self : Bare_Access_To_Subp_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Access_To_Subp_Def_F_Has_Protected : Bare_Protected_Node
         ; Access_To_Subp_Def_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Access_To_Subp_Def_F_Has_Protected := Access_To_Subp_Def_F_Has_Protected;
            Self.Access_To_Subp_Def_F_Subp_Spec := Access_To_Subp_Def_F_Subp_Spec;
      end Initialize_Fields_For_Access_To_Subp_Def;
   function Access_To_Subp_Def_F_Has_Protected
     (Node : Bare_Access_To_Subp_Def) return Bare_Protected_Node
   is
   begin
         return Node.Access_To_Subp_Def_F_Has_Protected;
   end;
   function Access_To_Subp_Def_F_Subp_Spec
     (Node : Bare_Access_To_Subp_Def) return Bare_Subp_Spec
   is
   begin
         return Node.Access_To_Subp_Def_F_Subp_Spec;
   end;
--# property-start AccessToSubpDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17477'
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Xref_Equation
  (Node : Bare_Access_To_Subp_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1505 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1505 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1505;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17477:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1505;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1505;
            raise;
      end;
   return Property_Result;
end Access_To_Subp_Def_P_Xref_Equation;
--# end
--# property-start AccessToSubpDef.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17480'
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Accessed_Type
  (Node : Bare_Access_To_Subp_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
     Ent : Internal_Entity_Access_To_Subp_Def :=
       Internal_Entity_Access_To_Subp_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12676 '<FieldAccess for return_type at nodes.lkt:17480:49>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17480'
--# expr-start 12675 '<FieldAccess for subp_spec at nodes.lkt:17480:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17480'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Ent.Info);
--# expr-done 12675
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12676
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Access_To_Subp_Def_P_Accessed_Type;
--# end
--# property-start AccessToSubpDef._env_trans_parent_225 '<builtin>:0'
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_225
  (Node : Bare_Access_To_Subp_Def
  )
   return Boolean
is
   Self : Bare_Access_To_Subp_Def  := Bare_Access_To_Subp_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Env_Trans_Parent_225;
--# end
   --
   --  Primitives for Bare_Base_Type_Access_Def
   --
      procedure Initialize_Fields_For_Base_Type_Access_Def
        (Self : Bare_Base_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
      end Initialize_Fields_For_Base_Type_Access_Def;
   --
   --  Primitives for Bare_Anonymous_Type_Access_Def
   --
      procedure Initialize_Fields_For_Anonymous_Type_Access_Def
        (Self : Bare_Anonymous_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Anonymous_Type_Access_Def_F_Type_Decl : Bare_Base_Type_Decl
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Anonymous_Type_Access_Def_F_Type_Decl := Anonymous_Type_Access_Def_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type_Access_Def;
   function Anonymous_Type_Access_Def_F_Type_Decl
     (Node : Bare_Anonymous_Type_Access_Def) return Bare_Base_Type_Decl
   is
   begin
         return Node.Anonymous_Type_Access_Def_F_Type_Decl;
   end;
--# property-start AnonymousTypeAccessDef.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17501'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Access_Def_P_Accessed_Type
  (Node : Bare_Anonymous_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type_Access_Def  := Bare_Anonymous_Type_Access_Def (Node);
     Ent : Internal_Entity_Anonymous_Type_Access_Def :=
       Internal_Entity_Anonymous_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12677 '<FieldAccess for type_decl at nodes.lkt:17501:49>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17501'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Type_Decl (Node => Ent.Node.Anonymous_Type_Access_Def_F_Type_Decl, Info => Ent.Info);
--# expr-done 12677
         Property_Result := Fld;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_Access_Def_P_Accessed_Type;
--# end
   --
   --  Primitives for Bare_Type_Access_Def
   --
      procedure Initialize_Fields_For_Type_Access_Def
        (Self : Bare_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Type_Access_Def_F_Has_All : Bare_All_Node
         ; Type_Access_Def_F_Has_Constant : Bare_Constant_Node
         ; Type_Access_Def_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Type_Access_Def_F_Has_All := Type_Access_Def_F_Has_All;
            Self.Type_Access_Def_F_Has_Constant := Type_Access_Def_F_Has_Constant;
            Self.Type_Access_Def_F_Subtype_Indication := Type_Access_Def_F_Subtype_Indication;
      end Initialize_Fields_For_Type_Access_Def;
   function Type_Access_Def_F_Has_All
     (Node : Bare_Type_Access_Def) return Bare_All_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_All;
   end;
   function Type_Access_Def_F_Has_Constant
     (Node : Bare_Type_Access_Def) return Bare_Constant_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_Constant;
   end;
   function Type_Access_Def_F_Subtype_Indication
     (Node : Bare_Type_Access_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Type_Access_Def_F_Subtype_Indication;
   end;
--# property-start TypeAccessDef.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17511'
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Accessed_Type
  (Node : Bare_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Access_Def  := Bare_Type_Access_Def (Node);
     Ent : Internal_Entity_Type_Access_Def :=
       Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12679 '<FieldAccess for designated_type at nodes.lkt:17512:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17512'
--# expr-start 12678 '<FieldAccess for subtype_indication at nodes.lkt:17512:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17512'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12678
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12679
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Access_Def_P_Accessed_Type;
--# end
--# property-start TypeAccessDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17515'
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Xref_Equation
  (Node : Bare_Type_Access_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Access_Def  := Bare_Type_Access_Def (Node);
     Ent : Internal_Entity_Type_Access_Def :=
       Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1509 with Inline_Always;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1509 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1509;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12681 '<FieldAccess for xref_equation at nodes.lkt:17515:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17515'
--# expr-start 12680 '<FieldAccess for subtype_indication at nodes.lkt:17515:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17515'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12680
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12681
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1509;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1509;
            raise;
      end;
   return Property_Result;
end Type_Access_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Array_Type_Def
   --
      procedure Initialize_Fields_For_Array_Type_Def
        (Self : Bare_Array_Type_Def
         ; Array_Type_Def_F_Indices : Bare_Array_Indices
         ; Array_Type_Def_F_Component_Type : Bare_Component_Def
        ) is
      begin
            Self.Array_Type_Def_F_Indices := Array_Type_Def_F_Indices;
            Self.Array_Type_Def_F_Component_Type := Array_Type_Def_F_Component_Type;
      end Initialize_Fields_For_Array_Type_Def;
   function Array_Type_Def_F_Indices
     (Node : Bare_Array_Type_Def) return Bare_Array_Indices
   is
   begin
         return Node.Array_Type_Def_F_Indices;
   end;
   function Array_Type_Def_F_Component_Type
     (Node : Bare_Array_Type_Def) return Bare_Component_Def
   is
   begin
         return Node.Array_Type_Def_F_Component_Type;
   end;
--# property-start ArrayTypeDef.comp_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17525'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Comp_Type
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12684 '<FieldAccess for designated_type at nodes.lkt:17526:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17526'
--# expr-start 12683 '<FieldAccess for type_expr at nodes.lkt:17526:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17526'
--# expr-start 12682 '<FieldAccess for component_type at nodes.lkt:17526:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17526'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 12682
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 12683
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 12684
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Comp_Type;
--# end
--# property-start ArrayTypeDef.index_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17529'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Index_Type
  (Node : Bare_Array_Type_Def
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12686 '<FieldAccess for index_type at nodes.lkt:17530:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17530'
--# expr-start 12685 '<FieldAccess for indices at nodes.lkt:17530:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17530'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12685
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Fld.Node, Dim => Dim, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12686
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Index_Type;
--# end
--# property-start ArrayTypeDef.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17533'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Array_Ndims
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Bare_Array_Indices;
Fld_1 : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12688 '<FieldAccess for ndims at nodes.lkt:17533:30>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17533'
--# expr-start 12687 '<FieldAccess for indices at nodes.lkt:17533:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17533'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Array_Type_Def_F_Indices;
--# expr-done 12687
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Ndims (Node => Fld);
--# end
--# expr-done 12688
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Array_Ndims;
--# end
--# property-start ArrayTypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17535'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Is_Limited_Type
  (Node : Bare_Array_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12691 '<bind at nodes.lkt:17536:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17536'
Bound_Origin := No_Bare_Ada_Node; 
--# expr-start 12690 '<FieldAccess for is_limited_type at nodes.lkt:17538:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17538'
--# expr-start 12689 '<FieldAccess for comp_type at nodes.lkt:17538:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17538'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12689
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12690
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 12691
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Is_Limited_Type;
--# end
--# property-start ArrayTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17542'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Equation
  (Node : Bare_Array_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1514 with Inline_Always;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Component_Def;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1514 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1514;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12696 '<And at nodes.lkt:17543:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17543'
--# expr-start 12693 '<FieldAccess for sub_equation at nodes.lkt:17543:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17543'
--# expr-start 12692 '<FieldAccess for indices at nodes.lkt:17543:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17543'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12692
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12693
--# expr-start 12695 '<FieldAccess for sub_equation at nodes.lkt:17543:41>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17543'
--# expr-start 12694 '<FieldAccess for component_type at nodes.lkt:17543:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17543'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 12694
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_2.Info);
--# end
--# expr-done 12695
And_Pred := Create_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17543:9") else null)); 
--# expr-done 12696
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1514;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1514;
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Xref_Equation;
--# end
--# property-start ArrayTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17546'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Predefined_Operators
  (Node : Bare_Array_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1515 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Comp_Type_Expr : Bare_Type_Expr;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Bare_Component_Def;
Fld_4 : Bare_Type_Expr;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1515 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1515;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Array_Type_Def_Array_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Array_Type_Def,
                                As_Bare_Array_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12734 '<Let at nodes.lkt:17547:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17547'
--# scope-start
--# expr-start 12698 '<Cast to TypeDecl at nodes.lkt:17547:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17547'
--# expr-start 12697 '<parent at nodes.lkt:17547:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17547'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12697
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12698
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12733 '<Let at nodes.lkt:17548:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17548'
--# scope-start
--# expr-start 12700 '<FieldAccess for node at nodes.lkt:17548:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17548'
--# expr-start 12699 '<FieldAccess for bool_type at nodes.lkt:17548:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17548'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12699
Fld_2 := Fld_1.Node;
--# expr-done 12700
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12732 '<Let at nodes.lkt:17549:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17549'
--# scope-start
--# expr-start 12702 '<FieldAccess for type_expr at nodes.lkt:17549:30>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17549'
--# expr-start 12701 '<FieldAccess for component_type at nodes.lkt:17549:30>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17549'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Self.Array_Type_Def_F_Component_Type;
--# expr-done 12701
      if Fld_3 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Fld_3.Component_Def_F_Type_Expr;
--# expr-done 12702
Local_Comp_Type_Expr := Fld_4; 
--# bind comp_type_expr Local_Comp_Type_Expr
--# expr-start 12731 '<ArrayLiteral at nodes.lkt:17559:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17559'
--# expr-start 12704 '<FieldAccess for create_binop_assoc at nodes.lkt:17559:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17559'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12703 '<Symbol "<" at nodes.lkt:17560:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17560'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12703
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Local_Bool_Type);
--# end
--# expr-done 12704
--# expr-start 12706 '<FieldAccess for create_binop_assoc at nodes.lkt:17561:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17561'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12705 '<Symbol "<=" at nodes.lkt:17562:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17562'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12705
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Local_Bool_Type);
--# end
--# expr-done 12706
--# expr-start 12708 '<FieldAccess for create_binop_assoc at nodes.lkt:17563:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17563'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12707 '<Symbol "=" at nodes.lkt:17564:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17564'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12707
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Local_Bool_Type);
--# end
--# expr-done 12708
--# expr-start 12710 '<FieldAccess for create_binop_assoc at nodes.lkt:17565:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17565'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12709 '<Symbol "/=" at nodes.lkt:17566:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17566'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12709
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Local_Bool_Type);
--# end
--# expr-done 12710
--# expr-start 12712 '<FieldAccess for create_binop_assoc at nodes.lkt:17567:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17567'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12711 '<Symbol ">" at nodes.lkt:17568:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17568'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12711
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Local_Bool_Type);
--# end
--# expr-done 12712
--# expr-start 12714 '<FieldAccess for create_binop_assoc at nodes.lkt:17569:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17569'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12713 '<Symbol ">=" at nodes.lkt:17570:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17570'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12713
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Local_Bool_Type);
--# end
--# expr-done 12714
--# expr-start 12716 '<FieldAccess for create_binop_assoc at nodes.lkt:17571:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17571'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12715 '<Symbol "and" at nodes.lkt:17572:13>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17572'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12715
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12716
--# expr-start 12718 '<FieldAccess for create_binop_assoc at nodes.lkt:17573:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17573'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12717 '<Symbol "or" at nodes.lkt:17574:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17574'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12717
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12718
--# expr-start 12720 '<FieldAccess for create_binop_assoc at nodes.lkt:17575:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17575'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12719 '<Symbol "xor" at nodes.lkt:17576:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17576'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 12719
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12720
--# expr-start 12722 '<FieldAccess for create_unop_assoc at nodes.lkt:17577:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17577'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12721 '<Symbol "not" at nodes.lkt:17577:35>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17577'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12721
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
--# property-call-start AdaNode.create_unop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_9, Rhs => Cast_Result_22, Ret => Cast_Result_23);
--# end
--# expr-done 12722
--# expr-start 12724 '<FieldAccess for create_binop_assoc_l_r_expr at nodes.lkt:17578:9>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17578'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12723 '<Symbol "&" at nodes.lkt:17579:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17579'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12723
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc_l_r_expr
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_L_R_Expr (Node => Self, Op => Sym_10, Lhs => Local_Comp_Type_Expr, Rhs => Local_Comp_Type_Expr, Ret => Cast_Result_24);
--# end
--# expr-done 12724
--# expr-start 12726 '<FieldAccess for create_binop_assoc_l_expr at nodes.lkt:17580:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17580'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12725 '<Symbol "&" at nodes.lkt:17581:13>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17581'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12725
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc_l_expr
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_L_Expr (Node => Self, Op => Sym_11, Lhs => Local_Comp_Type_Expr, Rhs => Cast_Result_25, Ret => Cast_Result_26);
--# end
--# expr-done 12726
--# expr-start 12728 '<FieldAccess for create_binop_assoc_r_expr at nodes.lkt:17582:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17582'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12727 '<Symbol "&" at nodes.lkt:17583:13>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17583'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12727
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc_r_expr
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc_R_Expr (Node => Self, Op => Sym_12, Lhs => Cast_Result_27, Rhs => Local_Comp_Type_Expr, Ret => Cast_Result_28);
--# end
--# expr-done 12728
--# expr-start 12730 '<FieldAccess for create_binop_assoc at nodes.lkt:17584:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17584'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12729 '<Symbol "&" at nodes.lkt:17585:13>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17585'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym); 
--# expr-done 12729
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Cast_Result_31);
--# end
--# expr-done 12730
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18)); 
--# expr-done 12731
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12732
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12733
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12734
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1515;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1515;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Array_Type_Def_P_Predefined_Operators;
--# end
--# property-start ArrayTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17590'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Is_Static
  (Node : Bare_Array_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
     Ent : Internal_Entity_Array_Type_Def :=
       Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12736 '<FieldAccess for is_static at nodes.lkt:17590:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17590'
--# expr-start 12735 '<FieldAccess for indices at nodes.lkt:17590:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17590'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 12735
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]ArrayIndices.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12736
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Is_Static;
--# end
--# property-start ArrayTypeDef.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17592'
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Entry_Point
  (Node : Bare_Array_Type_Def
  )
   return Boolean
is
   Self : Bare_Array_Type_Def  := Bare_Array_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Array_Type_Def_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Derived_Type_Def
   --
      procedure Initialize_Fields_For_Derived_Type_Def
        (Self : Bare_Derived_Type_Def
         ; Derived_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Derived_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Derived_Type_Def_F_Has_Synchronized : Bare_Synchronized_Node
         ; Derived_Type_Def_F_Subtype_Indication : Bare_Subtype_Indication
         ; Derived_Type_Def_F_Interfaces : Bare_Parent_List
         ; Derived_Type_Def_F_Record_Extension : Bare_Base_Record_Def
         ; Derived_Type_Def_F_Has_With_Private : Bare_With_Private
        ) is
      begin
            Self.Derived_Type_Def_F_Has_Abstract := Derived_Type_Def_F_Has_Abstract;
            Self.Derived_Type_Def_F_Has_Limited := Derived_Type_Def_F_Has_Limited;
            Self.Derived_Type_Def_F_Has_Synchronized := Derived_Type_Def_F_Has_Synchronized;
            Self.Derived_Type_Def_F_Subtype_Indication := Derived_Type_Def_F_Subtype_Indication;
            Self.Derived_Type_Def_F_Interfaces := Derived_Type_Def_F_Interfaces;
            Self.Derived_Type_Def_F_Record_Extension := Derived_Type_Def_F_Record_Extension;
            Self.Derived_Type_Def_F_Has_With_Private := Derived_Type_Def_F_Has_With_Private;
      end Initialize_Fields_For_Derived_Type_Def;
   function Derived_Type_Def_F_Has_Abstract
     (Node : Bare_Derived_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Abstract;
   end;
   function Derived_Type_Def_F_Has_Limited
     (Node : Bare_Derived_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Limited;
   end;
   function Derived_Type_Def_F_Has_Synchronized
     (Node : Bare_Derived_Type_Def) return Bare_Synchronized_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Synchronized;
   end;
   function Derived_Type_Def_F_Subtype_Indication
     (Node : Bare_Derived_Type_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Derived_Type_Def_F_Subtype_Indication;
   end;
   function Derived_Type_Def_F_Interfaces
     (Node : Bare_Derived_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Derived_Type_Def_F_Interfaces;
   end;
   function Derived_Type_Def_F_Record_Extension
     (Node : Bare_Derived_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Derived_Type_Def_F_Record_Extension;
   end;
   function Derived_Type_Def_F_Has_With_Private
     (Node : Bare_Derived_Type_Def) return Bare_With_Private
   is
   begin
         return Node.Derived_Type_Def_F_Has_With_Private;
   end;
--# property-start DerivedTypeDef.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17606'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Array_Ndims
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
Scope_Result : Integer;
Fld_2 : Integer;
Result_Var : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12737 '<FieldAccess for base_type at nodes.lkt:17606:30>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17606'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12737
Var_Expr_0 := Fld;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 12738 '<FieldAccess for array_ndims at nodes.lkt:17607:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17607'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Var_Expr_0.Node, Origin => Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12738
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 12739 '<Super at nodes.lkt:17607:47>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17607'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.array_ndims
Fld_2 := Libadalang.Implementation.Type_Def_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12739
   Result_Var := Fld_2;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Array_Ndims;
--# end
--# property-start DerivedTypeDef.base_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17611'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12741 '<FieldAccess for designated_type at nodes.lkt:17612:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17612'
--# expr-start 12740 '<FieldAccess for subtype_indication at nodes.lkt:17612:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17612'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12740
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12741
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Base_Type;
--# end
--# property-start DerivedTypeDef.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17614'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Interfaces
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1520 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1520 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1520;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12744 '<Map at nodes.lkt:17615:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17615'
   --# expr-start 12742 '<FieldAccess for interfaces at nodes.lkt:17615:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17615'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12742
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 12743 '<FieldAccess for name_designated_type at nodes.lkt:17615:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17615'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 12743
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12744
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1520;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1520;
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Base_Interfaces;
--# end
--# property-start DerivedTypeDef.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17618'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Task_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12746 '<FieldAccess for is_task_type at nodes.lkt:17618:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17618'
--# expr-start 12745 '<FieldAccess for base_type at nodes.lkt:17618:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17618'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12745
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_task_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12746
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Task_Type;
--# end
--# property-start DerivedTypeDef.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17621'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Int_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12748 '<FieldAccess for is_int_type at nodes.lkt:17621:31>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17621'
--# expr-start 12747 '<FieldAccess for base_type at nodes.lkt:17621:31>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17621'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12747
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12748
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Int_Type;
--# end
--# property-start DerivedTypeDef.is_access_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17624'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Access_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Derived_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Derived_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12751 '<FieldAccess for is_access_type at nodes.lkt:17625:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17625'
--# expr-start 12750 '<FieldAccess for base_type at nodes.lkt:17625:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17625'
--# expr-start 12749 '<as_bare_entity at nodes.lkt:17625:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17625'
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Derived_Type_Def;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 12749
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 12750
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12751
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Access_Type;
--# end
--# property-start DerivedTypeDef.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17628'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Char_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12753 '<FieldAccess for is_char_type at nodes.lkt:17628:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17628'
--# expr-start 12752 '<FieldAccess for base_type at nodes.lkt:17628:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17628'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12752
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12753
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Char_Type;
--# end
--# property-start DerivedTypeDef.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17631'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Float_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12755 '<FieldAccess for is_float_type at nodes.lkt:17631:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17631'
--# expr-start 12754 '<FieldAccess for base_type at nodes.lkt:17631:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17631'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12754
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12755
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Float_Type;
--# end
--# property-start DerivedTypeDef.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17634'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12757 '<FieldAccess for is_fixed_point at nodes.lkt:17634:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17634'
--# expr-start 12756 '<FieldAccess for base_type at nodes.lkt:17634:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17634'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12756
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12757
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start DerivedTypeDef.accessed_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17637'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Accessed_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_294 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12758 '<FieldAccess for base_type at nodes.lkt:17638:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17638'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12758
Var_Expr_294 := Fld;
if Var_Expr_294 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 12759 '<FieldAccess for accessed_type at nodes.lkt:17638:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17638'
      if Var_Expr_294.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_294.Node, Origin => Origin, E_Info => Var_Expr_294.Info);
--# end
--# expr-done 12759
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Accessed_Type;
--# end
--# property-start DerivedTypeDef.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17641'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_With_Private;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 12762 '<Not at nodes.lkt:17642:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17642'
--# expr-start 12761 '<IsNull at nodes.lkt:17642:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17642'
--# expr-start 12760 '<FieldAccess for record_extension at nodes.lkt:17642:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17642'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Derived_Type_Def_F_Record_Extension, Info => Ent.Info);
--# expr-done 12760
Is_Null := Fld.Node = null; 
--# expr-done 12761
Not_Val := not (Is_Null); 
--# expr-done 12762
if Not_Val then
   If_Result := True;
else
   --# expr-start 12764 '<FieldAccess for as_bool at nodes.lkt:17642:46>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17642'
--# expr-start 12763 '<FieldAccess for has_with_private at nodes.lkt:17642:46>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17642'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_With_Private (Node => Ent.Node.Derived_Type_Def_F_Has_With_Private, Info => Ent.Info);
--# expr-done 12763
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_1.Node);
--# end
--# expr-done 12764
   If_Result := Fld_2;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start DerivedTypeDef.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17645'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Enum_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12766 '<FieldAccess for is_enum_type at nodes.lkt:17645:32>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17645'
--# expr-start 12765 '<FieldAccess for base_type at nodes.lkt:17645:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17645'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 12765
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 12766
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Enum_Type;
--# end
--# property-start DerivedTypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17647'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Limited_Type
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Base_Type_Decl;
Var_Expr_295 : Internal_Entity_Base_Record_Def;
Var_Expr_296 : Internal_Entity_Component_List;
Fld : Bare_Limited_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Bound_Origin : Bare_Ada_Node;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
Not_Val : Boolean;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
If_Result_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 12768 '<FieldAccess for as_bool at nodes.lkt:17648:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17648'
--# expr-start 12767 '<FieldAccess for has_limited at nodes.lkt:17648:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17648'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Derived_Type_Def_F_Has_Limited;
--# expr-done 12767
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 12768
if Fld_1 then
   If_Result := True;
else
--# expr-start 12769 '<FieldAccess for record_extension at nodes.lkt:17648:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17648'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Derived_Type_Def_F_Record_Extension, Info => Ent.Info);
--# expr-done 12769
Var_Expr_295 := Fld_2;
if Var_Expr_295 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 12770 '<FieldAccess for comps at nodes.lkt:17648:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17648'
      if Var_Expr_295.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Var_Expr_295.Node, E_Info => Var_Expr_295.Info);
--# end
--# expr-done 12770
Var_Expr_296 := Fld_3;
if Var_Expr_296 /= No_Entity_Component_List then
   --# scope-start
--# expr-start 12771 '<FieldAccess for has_limited_component at nodes.lkt:17648:39>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17648'
      if Var_Expr_296.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.has_limited_component
Fld_4 := Libadalang.Implementation.Component_List_P_Has_Limited_Component (Node => Var_Expr_296.Node, E_Info => Var_Expr_296.Info);
--# end
--# expr-done 12771
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
if If_Result then
   If_Result_2 := True;
else
   --# expr-start 12776 '<bind at nodes.lkt:17650:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17650'
Bound_Origin := No_Bare_Ada_Node; 
--# expr-start 12772 '<FieldAccess for base_type at nodes.lkt:17652:9>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17652'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 12772
Var_Expr_0 := Fld_5;
if Var_Expr_0 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_0
--# expr-start 12774 '<Not at nodes.lkt:17657:17>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17657'
--# expr-start 12773 '<FieldAccess for is_interface_type at nodes.lkt:17657:21>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17657'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_interface_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Interface_Type (Node => Var_Expr_0.Node, Origin => Bound_Origin, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12773
Not_Val := not (Fld_6); 
--# expr-done 12774
if Not_Val then
   --# expr-start 12775 '<FieldAccess for is_limited_type at nodes.lkt:17658:19>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17658'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_limited_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Limited_Type (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 12775
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_2; 
--# expr-done 12776
   If_Result_2 := Dyn_Var_Bind_Result;
end if;
         Property_Result := If_Result_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Limited_Type;
--# end
--# property-start DerivedTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17663'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Static
  (Node : Bare_Derived_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12778 '<FieldAccess for is_static_subtype at nodes.lkt:17663:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17663'
--# expr-start 12777 '<FieldAccess for subtype_indication at nodes.lkt:17663:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17663'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12777
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12778
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Is_Static;
--# end
--# property-start DerivedTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17666'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Xref_Equation
  (Node : Bare_Derived_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1532 with Inline_Always;
            procedure Finalizer_Scope_3609 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Parent_List;
Fld_4 : Bare_Parent_List;
Fld_5 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_6 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1532 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1532;
            procedure Finalizer_Scope_3609 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_3609;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12787 '<bind at nodes.lkt:17670:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17670'
--# expr-start 12779 '<FieldAccess for children_env at nodes.lkt:17670:24>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17670'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 12779
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 12786 '<And at nodes.lkt:17672:13>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
--# expr-start 12781 '<FieldAccess for xref_equation at nodes.lkt:17672:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
--# expr-start 12780 '<FieldAccess for subtype_indication at nodes.lkt:17672:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12780
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_1.Info);
--# end
--# expr-done 12781
--# expr-start 12785 '<LogicAll at nodes.lkt:17672:57>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
--# expr-start 12784 '<Map at nodes.lkt:17672:57>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
   --# expr-start 12782 '<FieldAccess for interfaces at nodes.lkt:17672:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12782
Fld_4 := Fld_3.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_4 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_4;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind ifc Item_0
      --# expr-start 12783 '<FieldAccess for xref_equation at nodes.lkt:17672:92>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17672'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_0.Node, Env => Bound_Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 12783
         declare
            Item_To_Append : constant Logic_Equation := Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3609;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12784
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17672:57") else null)); 
--# expr-done 12785
And_Pred := Create_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17672:13") else null)); 
--# expr-done 12786
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 12787
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1532;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1532;
                     Finalizer_Scope_3609;
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Xref_Equation;
--# end
--# property-start DerivedTypeDef.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17675'
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Discrete_Range
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Derived_Type_Def  := Bare_Derived_Type_Def (Node);
     Ent : Internal_Entity_Derived_Type_Def :=
       Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12789 '<FieldAccess for discrete_range at nodes.lkt:17676:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17676'
--# expr-start 12788 '<FieldAccess for subtype_indication at nodes.lkt:17676:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17676'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 12788
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 12789
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Derived_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Enum_Type_Def
   --
      procedure Initialize_Fields_For_Enum_Type_Def
        (Self : Bare_Enum_Type_Def
         ; Enum_Type_Def_F_Enum_Literals : Bare_Enum_Literal_Decl_List
        ) is
      begin
            Self.Enum_Type_Def_F_Enum_Literals := Enum_Type_Def_F_Enum_Literals;
      end Initialize_Fields_For_Enum_Type_Def;
   function Enum_Type_Def_F_Enum_Literals
     (Node : Bare_Enum_Type_Def) return Bare_Enum_Literal_Decl_List
   is
   begin
         return Node.Enum_Type_Def_F_Enum_Literals;
   end;
--# property-start EnumTypeDef.is_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17684'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Char_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Enum_Literal_Decl_List;
Item_0 : Bare_Enum_Literal_Decl;
Fld_1 : Bare_Defining_Name;
Fld_2 : Bare_Name;
Is_A : Boolean;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12794 '<AnyQuantifier at nodes.lkt:17685:9>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17685'
   --# expr-start 12790 '<FieldAccess for enum_literals at nodes.lkt:17685:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17685'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Enum_Type_Def_F_Enum_Literals;
--# expr-done 12790
      Quantifier_Result := False;
      if Fld = null then
      null;
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind lit Item_0
      --# expr-start 12793 '<IsA CharLiteral at nodes.lkt:17685:41>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17685'
--# expr-start 12792 '<FieldAccess for name at nodes.lkt:17685:41>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17685'
--# expr-start 12791 '<FieldAccess for name at nodes.lkt:17685:41>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17685'
      if Item_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Item_0.Enum_Literal_Decl_F_Name;
--# expr-done 12791
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Fld_1.Defining_Name_F_Name;
--# expr-done 12792
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Char_Literal_Range; 
--# expr-done 12793
      Quantifier_Result := Is_A;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 12794
         Property_Result := Quantifier_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Char_Type;
--# end
--# property-start EnumTypeDef.is_enum_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17688'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Enum_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Enum_Type;
--# end
--# property-start EnumTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17691'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Xref_Equation
  (Node : Bare_Enum_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1536 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1536 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1536;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17691:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1536;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1536;
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Xref_Equation;
--# end
--# property-start EnumTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17694'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Static
  (Node : Bare_Enum_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Static;
--# end
--# property-start EnumTypeDef.is_std_char_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17696'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Std_Char_Type
  (Node : Bare_Enum_Type_Def
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
      Local_Self_Type : Bare_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Cast_Result_1 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Base_Type_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal : Boolean;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12803 '<Let at nodes.lkt:17697:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17697'
--# scope-start
--# expr-start 12796 '<Cast to TypeDecl at nodes.lkt:17697:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17697'
--# expr-start 12795 '<parent at nodes.lkt:17697:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17697'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12795
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12796
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12798 '<FieldAccess for node at nodes.lkt:17699:22>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
--# expr-start 12797 '<FieldAccess for std_char_type at nodes.lkt:17699:22>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_char_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Std_Char_Type (Node => Self);
--# end
--# expr-done 12797
Fld_2 := Fld_1.Node;
--# expr-done 12798
      Cast_Result_1 := Fld_2;
Is_Equal_2 := Local_Self_Type = Cast_Result_1; 
if Is_Equal_2 then
   If_Result_1 := True;
else
--# expr-start 12800 '<FieldAccess for node at nodes.lkt:17699:50>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
--# expr-start 12799 '<FieldAccess for std_wide_char_type at nodes.lkt:17699:50>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_char_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Char_Type (Node => Self);
--# end
--# expr-done 12799
Fld_4 := Fld_3.Node;
--# expr-done 12800
      Cast_Result_2 := Fld_4;
Is_Equal_1 := Local_Self_Type = Cast_Result_2; 
if Is_Equal_1 then
   If_Result := True;
else
--# expr-start 12802 '<FieldAccess for node at nodes.lkt:17699:83>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
--# expr-start 12801 '<FieldAccess for std_wide_wide_char_type at nodes.lkt:17699:83>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17699'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.std_wide_wide_char_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Std_Wide_Wide_Char_Type (Node => Self);
--# end
--# expr-done 12801
Fld_6 := Fld_5.Node;
--# expr-done 12802
      Cast_Result_3 := Fld_6;
Is_Equal := Local_Self_Type = Cast_Result_3; 
   If_Result := Is_Equal;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 12803
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Type_Def_P_Is_Std_Char_Type;
--# end
--# property-start EnumTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17703'
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Predefined_Operators
  (Node : Bare_Enum_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Enum_Type_Def  := Bare_Enum_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1539 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Defaults : Internal_Env_Assoc_Array_Access;
Local_Specials : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Sym_6 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit_1 : Internal_Env_Assoc_Array_Access;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
Let_Result_3 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1539 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Local_Defaults);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Specials);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1539;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Type_Def_Enum_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Type_Def,
                                As_Bare_Enum_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12835 '<Let at nodes.lkt:17704:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17704'
--# scope-start
--# expr-start 12805 '<Cast to TypeDecl at nodes.lkt:17704:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17704'
--# expr-start 12804 '<parent at nodes.lkt:17704:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17704'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12804
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12805
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12834 '<Let at nodes.lkt:17705:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17705'
--# scope-start
--# expr-start 12807 '<FieldAccess for node at nodes.lkt:17705:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17705'
--# expr-start 12806 '<FieldAccess for bool_type at nodes.lkt:17705:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17705'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12806
Fld_2 := Fld_1.Node;
--# expr-done 12807
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12833 '<Let at nodes.lkt:17706:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17706'
--# scope-start
--# expr-start 12820 '<ArrayLiteral at nodes.lkt:17706:24>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17706'
--# expr-start 12809 '<FieldAccess for create_binop_assoc at nodes.lkt:17706:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17706'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12808 '<Symbol "<" at nodes.lkt:17707:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17707'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12808
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Local_Bool_Type);
--# end
--# expr-done 12809
--# expr-start 12811 '<FieldAccess for create_binop_assoc at nodes.lkt:17708:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17708'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12810 '<Symbol "<=" at nodes.lkt:17709:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17709'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12810
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Local_Bool_Type);
--# end
--# expr-done 12811
--# expr-start 12813 '<FieldAccess for create_binop_assoc at nodes.lkt:17710:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17710'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12812 '<Symbol "=" at nodes.lkt:17711:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17711'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12812
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Local_Bool_Type);
--# end
--# expr-done 12813
--# expr-start 12815 '<FieldAccess for create_binop_assoc at nodes.lkt:17712:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17712'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12814 '<Symbol "/=" at nodes.lkt:17713:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17713'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12814
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Local_Bool_Type);
--# end
--# expr-done 12815
--# expr-start 12817 '<FieldAccess for create_binop_assoc at nodes.lkt:17714:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17714'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12816 '<Symbol ">" at nodes.lkt:17715:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17715'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12816
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Local_Bool_Type);
--# end
--# expr-done 12817
--# expr-start 12819 '<FieldAccess for create_binop_assoc at nodes.lkt:17716:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17716'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12818 '<Symbol ">=" at nodes.lkt:17717:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17717'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12818
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Local_Bool_Type);
--# end
--# expr-done 12819
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4, 3 => Fld_5, 4 => Fld_6, 5 => Fld_7, 6 => Fld_8)); 
--# expr-done 12820
Local_Defaults := Array_Lit; Inc_Ref (Local_Defaults);
--# bind defaults Local_Defaults
--# expr-start 12832 '<Let at nodes.lkt:17720:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17720'
--# scope-start
--# expr-start 12831 '<If at nodes.lkt:17720:24>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17720'
--# expr-start 12821 '<Eq at nodes.lkt:17720:27>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17720'
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
Is_Equal := Cast_Result_24 = Local_Bool_Type; 
--# expr-done 12821
if Is_Equal then
   --# expr-start 12830 '<ArrayLiteral at nodes.lkt:17720:55>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17720'
--# expr-start 12823 '<FieldAccess for create_binop_assoc at nodes.lkt:17720:56>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17720'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12822 '<Symbol "and" at nodes.lkt:17721:13>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17721'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12822
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12823
--# expr-start 12825 '<FieldAccess for create_binop_assoc at nodes.lkt:17722:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17722'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12824 '<Symbol "or" at nodes.lkt:17723:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17723'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12824
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12825
--# expr-start 12827 '<FieldAccess for create_binop_assoc at nodes.lkt:17724:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17724'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12826 '<Symbol "xor" at nodes.lkt:17725:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17725'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Xor); 
--# expr-done 12826
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12827
--# expr-start 12829 '<FieldAccess for create_unop_assoc at nodes.lkt:17726:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17726'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12828 '<Symbol "not" at nodes.lkt:17726:35>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17726'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12828
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
--# property-call-start AdaNode.create_unop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_9, Rhs => Cast_Result_22, Ret => Cast_Result_23);
--# end
--# expr-done 12829
Array_Lit_1 := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_9, 2 => Fld_10, 3 => Fld_11, 4 => Fld_12)); 
--# expr-done 12830
   If_Result := Array_Lit_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 12831
Local_Specials := If_Result; Inc_Ref (Local_Specials);
--# bind specials Local_Specials
Concat_Result := Concat (Local_Defaults, Local_Specials); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12832
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12833
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12834
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 12835
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1539;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1539;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Enum_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Formal_Discrete_Type_Def
   --
--# property-start FormalDiscreteTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17736'
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Xref_Equation
  (Node : Bare_Formal_Discrete_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1540 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1540 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1540;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17736:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1540;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1540;
            raise;
      end;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Xref_Equation;
--# end
--# property-start FormalDiscreteTypeDef.is_discrete_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17739'
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Formal_Discrete_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start FormalDiscreteTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17742'
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Predefined_Operators
  (Node : Bare_Formal_Discrete_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Formal_Discrete_Type_Def  := Bare_Formal_Discrete_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1542 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Fld_4 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1542 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1542;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Formal_Discrete_Type_Def_Formal_Discrete_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Formal_Discrete_Type_Def,
                                As_Bare_Formal_Discrete_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12854 '<Let at nodes.lkt:17743:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17743'
--# scope-start
--# expr-start 12837 '<Cast to TypeDecl at nodes.lkt:17743:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17743'
--# expr-start 12836 '<parent at nodes.lkt:17743:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17743'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12836
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12837
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12853 '<Let at nodes.lkt:17744:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17744'
--# scope-start
--# expr-start 12839 '<FieldAccess for node at nodes.lkt:17744:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17744'
--# expr-start 12838 '<FieldAccess for bool_type at nodes.lkt:17744:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17744'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12838
Fld_2 := Fld_1.Node;
--# expr-done 12839
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12852 '<ArrayLiteral at nodes.lkt:17746:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17746'
--# expr-start 12841 '<FieldAccess for create_binop_assoc at nodes.lkt:17746:10>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17746'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12840 '<Symbol "<" at nodes.lkt:17747:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17747'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12840
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Local_Bool_Type);
--# end
--# expr-done 12841
--# expr-start 12843 '<FieldAccess for create_binop_assoc at nodes.lkt:17748:12>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17748'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12842 '<Symbol "<=" at nodes.lkt:17749:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17749'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12842
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
--# property-call-start AdaNode.create_binop_assoc
Fld_4 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_3, Rhs => Cast_Result_4, Ret => Local_Bool_Type);
--# end
--# expr-done 12843
--# expr-start 12845 '<FieldAccess for create_binop_assoc at nodes.lkt:17750:12>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17750'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12844 '<Symbol "=" at nodes.lkt:17751:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17751'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12844
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_5, Rhs => Cast_Result_6, Ret => Local_Bool_Type);
--# end
--# expr-done 12845
--# expr-start 12847 '<FieldAccess for create_binop_assoc at nodes.lkt:17752:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17752'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12846 '<Symbol "/=" at nodes.lkt:17753:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17753'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12846
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Local_Bool_Type);
--# end
--# expr-done 12847
--# expr-start 12849 '<FieldAccess for create_binop_assoc at nodes.lkt:17754:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17754'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12848 '<Symbol ">" at nodes.lkt:17755:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17755'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12848
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_9, Rhs => Cast_Result_10, Ret => Local_Bool_Type);
--# end
--# expr-done 12849
--# expr-start 12851 '<FieldAccess for create_binop_assoc at nodes.lkt:17756:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17756'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12850 '<Symbol ">=" at nodes.lkt:17757:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17757'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12850
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_11, Rhs => Cast_Result_12, Ret => Local_Bool_Type);
--# end
--# expr-done 12851
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_3, 2 => Fld_4, 3 => Fld_5, 4 => Fld_6, 5 => Fld_7, 6 => Fld_8)); 
--# expr-done 12852
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12853
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12854
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1542;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1542;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Formal_Discrete_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Interface_Type_Def
   --
      procedure Initialize_Fields_For_Interface_Type_Def
        (Self : Bare_Interface_Type_Def
         ; Interface_Type_Def_F_Interface_Kind : Bare_Interface_Kind
         ; Interface_Type_Def_F_Interfaces : Bare_Parent_List
        ) is
      begin
            Self.Interface_Type_Def_F_Interface_Kind := Interface_Type_Def_F_Interface_Kind;
            Self.Interface_Type_Def_F_Interfaces := Interface_Type_Def_F_Interfaces;
      end Initialize_Fields_For_Interface_Type_Def;
   function Interface_Type_Def_F_Interface_Kind
     (Node : Bare_Interface_Type_Def) return Bare_Interface_Kind
   is
   begin
         return Node.Interface_Type_Def_F_Interface_Kind;
   end;
   function Interface_Type_Def_F_Interfaces
     (Node : Bare_Interface_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Interface_Type_Def_F_Interfaces;
   end;
--# property-start InterfaceTypeDef.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17768'
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Interface_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start InterfaceTypeDef.is_task_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17771'
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Task_Type
  (Node : Bare_Interface_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
     Ent : Internal_Entity_Interface_Type_Def :=
       Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Interface_Kind;
Is_A : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12856 '<IsA InterfaceKind.Task at nodes.lkt:17771:32>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17771'
--# expr-start 12855 '<FieldAccess for interface_kind at nodes.lkt:17771:32>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17771'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Interface_Kind (Node => Ent.Node.Interface_Type_Def_F_Interface_Kind, Info => Ent.Info);
--# expr-done 12855
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Interface_Kind_Task_Range; 
--# expr-done 12856
         Property_Result := Is_A;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Is_Task_Type;
--# end
--# property-start InterfaceTypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17775'
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Limited_Type
  (Node : Bare_Interface_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Interface_Kind;
Is_Null : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12859 '<Not at nodes.lkt:17775:35>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17775'
--# expr-start 12858 '<IsNull at nodes.lkt:17775:39>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17775'
--# expr-start 12857 '<FieldAccess for interface_kind at nodes.lkt:17775:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17775'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Interface_Type_Def_F_Interface_Kind;
--# expr-done 12857
Is_Null := Fld = null; 
--# expr-done 12858
Not_Val := not (Is_Null); 
--# expr-done 12859
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Is_Limited_Type;
--# end
--# property-start InterfaceTypeDef.base_interfaces 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17777'
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Base_Interfaces
  (Node : Bare_Interface_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
     Ent : Internal_Entity_Interface_Type_Def :=
       Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1546 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_1546 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1546;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12862 '<Map at nodes.lkt:17778:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17778'
   --# expr-start 12860 '<FieldAccess for interfaces at nodes.lkt:17778:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17778'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12860
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind i Item_0
      --# expr-start 12861 '<FieldAccess for name_designated_type at nodes.lkt:17778:36>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17778'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 12861
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl := Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
           (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12862
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1546;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1546;
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Base_Interfaces;
--# end
--# property-start InterfaceTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17781'
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Xref_Equation
  (Node : Bare_Interface_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Interface_Type_Def  := Bare_Interface_Type_Def (Node);
     Ent : Internal_Entity_Interface_Type_Def :=
       Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1547 with Inline_Always;
            procedure Finalizer_Scope_3612 with Inline_Always;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1547 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1547;
            procedure Finalizer_Scope_3612 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3612;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12866 '<LogicAll at nodes.lkt:17782:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17782'
--# expr-start 12865 '<Map at nodes.lkt:17782:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17782'
   --# expr-start 12863 '<FieldAccess for interfaces at nodes.lkt:17782:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17782'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 12863
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind ifc Item_0
      --# expr-start 12864 '<FieldAccess for xref_equation at nodes.lkt:17782:44>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17782'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 12864
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3612;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 12865
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17782:9") else null)); 
--# expr-done 12866
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1547;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1547;
                     Finalizer_Scope_3612;
            raise;
      end;
   return Property_Result;
end Interface_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mod_Int_Type_Def
   --
      procedure Initialize_Fields_For_Mod_Int_Type_Def
        (Self : Bare_Mod_Int_Type_Def
         ; Mod_Int_Type_Def_F_Expr : Bare_Expr
        ) is
      begin
            Self.Mod_Int_Type_Def_F_Expr := Mod_Int_Type_Def_F_Expr;
      end Initialize_Fields_For_Mod_Int_Type_Def;
   function Mod_Int_Type_Def_F_Expr
     (Node : Bare_Mod_Int_Type_Def) return Bare_Expr
   is
   begin
         return Node.Mod_Int_Type_Def_F_Expr;
   end;
--# property-start ModIntTypeDef.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17790'
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Mod_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start ModIntTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17793'
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Mod_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1549 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1549 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1549;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12868 '<FieldAccess for sub_equation at nodes.lkt:17793:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17793'
--# expr-start 12867 '<FieldAccess for expr at nodes.lkt:17793:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17793'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12867
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 12868
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1549;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1549;
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start ModIntTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17796'
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Static
  (Node : Bare_Mod_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 12870 '<FieldAccess for is_static_expr at nodes.lkt:17796:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17796'
--# expr-start 12869 '<FieldAccess for expr at nodes.lkt:17796:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17796'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12869
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 12870
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Is_Static;
--# end
--# property-start ModIntTypeDef.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17798'
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Mod_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
     Ent : Internal_Entity_Mod_Int_Type_Def :=
       Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12872 '<New DiscreteRange at nodes.lkt:17798:43>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17798'
--# expr-start 12871 '<FieldAccess for expr at nodes.lkt:17799:50>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17799'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 12871
New_Struct := (High_Bound => Fld, Low_Bound => No_Entity_Expr); 
--# expr-done 12872
         Property_Result := New_Struct;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Mod_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start ModIntTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17803'
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Predefined_Operators
  (Node : Bare_Mod_Int_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Mod_Int_Type_Def  := Bare_Mod_Int_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1552 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Int_Type : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_38 : Bare_Type_Decl;
Cast_Result_39 : Bare_Base_Type_Decl;
Cast_Expr_39 : Bare_Type_Decl;
Cast_Result_40 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_40 : Bare_Type_Decl;
Cast_Result_41 : Bare_Base_Type_Decl;
Cast_Expr_41 : Bare_Type_Decl;
Cast_Result_42 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Sym_17 : Symbol_Type;
Cast_Expr_42 : Bare_Type_Decl;
Cast_Result_43 : Bare_Base_Type_Decl;
Cast_Expr_43 : Bare_Type_Decl;
Cast_Result_44 : Bare_Base_Type_Decl;
Fld_22 : Internal_Env_Assoc := No_Env_Assoc;
Sym_18 : Symbol_Type;
Cast_Expr_44 : Bare_Type_Decl;
Cast_Result_45 : Bare_Base_Type_Decl;
Cast_Expr_45 : Bare_Type_Decl;
Cast_Result_46 : Bare_Base_Type_Decl;
Fld_23 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1552 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1552;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Mod_Int_Type_Def_Mod_Int_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Mod_Int_Type_Def,
                                As_Bare_Mod_Int_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12920 '<Let at nodes.lkt:17804:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17804'
--# scope-start
--# expr-start 12874 '<Cast to TypeDecl at nodes.lkt:17804:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17804'
--# expr-start 12873 '<parent at nodes.lkt:17804:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17804'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12873
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12874
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12919 '<Let at nodes.lkt:17805:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17805'
--# scope-start
--# expr-start 12876 '<FieldAccess for node at nodes.lkt:17805:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17805'
--# expr-start 12875 '<FieldAccess for bool_type at nodes.lkt:17805:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17805'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12875
Fld_2 := Fld_1.Node;
--# expr-done 12876
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12918 '<Let at nodes.lkt:17806:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17806'
--# scope-start
--# expr-start 12878 '<FieldAccess for node at nodes.lkt:17806:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17806'
--# expr-start 12877 '<FieldAccess for int_type at nodes.lkt:17806:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17806'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12877
Fld_4 := Fld_3.Node;
--# expr-done 12878
Local_Int_Type := Fld_4; 
--# bind int_type Local_Int_Type
--# expr-start 12917 '<ArrayLiteral at nodes.lkt:17808:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17808'
--# expr-start 12880 '<FieldAccess for create_binop_assoc at nodes.lkt:17808:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17808'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12879 '<Symbol "+" at nodes.lkt:17809:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17809'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12879
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12880
--# expr-start 12882 '<FieldAccess for create_binop_assoc at nodes.lkt:17810:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17810'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12881 '<Symbol "-" at nodes.lkt:17811:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17811'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12881
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12882
--# expr-start 12884 '<FieldAccess for create_binop_assoc at nodes.lkt:17812:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17812'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12883 '<Symbol "*" at nodes.lkt:17813:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17813'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12883
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12884
--# expr-start 12886 '<FieldAccess for create_binop_assoc at nodes.lkt:17814:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17814'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12885 '<Symbol "/" at nodes.lkt:17815:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17815'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12885
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12886
--# expr-start 12888 '<FieldAccess for create_binop_assoc at nodes.lkt:17816:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17816'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12887 '<Symbol "mod" at nodes.lkt:17817:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17817'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 12887
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 12888
--# expr-start 12890 '<FieldAccess for create_binop_assoc at nodes.lkt:17818:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17818'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12889 '<Symbol "rem" at nodes.lkt:17819:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17819'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 12889
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 12890
--# expr-start 12892 '<FieldAccess for create_binop_assoc at nodes.lkt:17820:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17820'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12891 '<Symbol "and" at nodes.lkt:17821:13>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17821'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_And); 
--# expr-done 12891
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Cast_Result_21);
--# end
--# expr-done 12892
--# expr-start 12894 '<FieldAccess for create_binop_assoc at nodes.lkt:17822:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17822'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12893 '<Symbol "or" at nodes.lkt:17823:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17823'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Or); 
--# expr-done 12893
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_22, Rhs => Cast_Result_23, Ret => Cast_Result_24);
--# end
--# expr-done 12894
--# expr-start 12896 '<FieldAccess for create_binop_assoc at nodes.lkt:17824:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17824'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12895 '<Symbol "**" at nodes.lkt:17825:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17825'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12895
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_25, Rhs => Local_Int_Type, Ret => Cast_Result_26);
--# end
--# expr-done 12896
--# expr-start 12898 '<FieldAccess for create_binop_assoc at nodes.lkt:17826:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17826'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12897 '<Symbol "<" at nodes.lkt:17827:13>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17827'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12897
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Local_Bool_Type);
--# end
--# expr-done 12898
--# expr-start 12900 '<FieldAccess for create_binop_assoc at nodes.lkt:17828:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17828'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12899 '<Symbol "<=" at nodes.lkt:17829:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17829'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12899
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Local_Bool_Type);
--# end
--# expr-done 12900
--# expr-start 12902 '<FieldAccess for create_binop_assoc at nodes.lkt:17830:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17830'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12901 '<Symbol "=" at nodes.lkt:17831:13>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17831'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12901
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Local_Self_Type; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Local_Bool_Type);
--# end
--# expr-done 12902
--# expr-start 12904 '<FieldAccess for create_binop_assoc at nodes.lkt:17832:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17832'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12903 '<Symbol "/=" at nodes.lkt:17833:13>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17833'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12903
Cast_Expr_32 := Local_Self_Type; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Local_Self_Type; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_33, Rhs => Cast_Result_34, Ret => Local_Bool_Type);
--# end
--# expr-done 12904
--# expr-start 12906 '<FieldAccess for create_binop_assoc at nodes.lkt:17834:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17834'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12905 '<Symbol ">" at nodes.lkt:17835:13>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17835'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12905
Cast_Expr_34 := Local_Self_Type; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Local_Self_Type; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_35, Rhs => Cast_Result_36, Ret => Local_Bool_Type);
--# end
--# expr-done 12906
--# expr-start 12908 '<FieldAccess for create_binop_assoc at nodes.lkt:17836:12>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17836'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12907 '<Symbol ">=" at nodes.lkt:17837:13>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17837'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12907
Cast_Expr_36 := Local_Self_Type; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Local_Self_Type; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_binop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_14, Lhs => Cast_Result_37, Rhs => Cast_Result_38, Ret => Local_Bool_Type);
--# end
--# expr-done 12908
--# expr-start 12910 '<FieldAccess for create_unop_assoc at nodes.lkt:17838:12>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12909 '<Symbol "+" at nodes.lkt:17838:35>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12909
Cast_Expr_38 := Local_Self_Type; 
      Cast_Result_39 := Cast_Expr_38;
Cast_Expr_39 := Local_Self_Type; 
      Cast_Result_40 := Cast_Expr_39;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_39, Ret => Cast_Result_40);
--# end
--# expr-done 12910
--# expr-start 12912 '<FieldAccess for create_unop_assoc at nodes.lkt:17838:68>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12911 '<Symbol "-" at nodes.lkt:17838:91>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12911
Cast_Expr_40 := Local_Self_Type; 
      Cast_Result_41 := Cast_Expr_40;
Cast_Expr_41 := Local_Self_Type; 
      Cast_Result_42 := Cast_Expr_41;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_41, Ret => Cast_Result_42);
--# end
--# expr-done 12912
--# expr-start 12914 '<FieldAccess for create_unop_assoc at nodes.lkt:17838:124>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12913 '<Symbol "abs" at nodes.lkt:17838:147>' Sym_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12913
Cast_Expr_42 := Local_Self_Type; 
      Cast_Result_43 := Cast_Expr_42;
Cast_Expr_43 := Local_Self_Type; 
      Cast_Result_44 := Cast_Expr_43;
--# property-call-start AdaNode.create_unop_assoc
Fld_22 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_17, Rhs => Cast_Result_43, Ret => Cast_Result_44);
--# end
--# expr-done 12914
--# expr-start 12916 '<FieldAccess for create_unop_assoc at nodes.lkt:17838:182>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12915 '<Symbol "not" at nodes.lkt:17838:205>' Sym_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17838'
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Not); 
--# expr-done 12915
Cast_Expr_44 := Local_Self_Type; 
      Cast_Result_45 := Cast_Expr_44;
Cast_Expr_45 := Local_Self_Type; 
      Cast_Result_46 := Cast_Expr_45;
--# property-call-start AdaNode.create_unop_assoc
Fld_23 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_18, Rhs => Cast_Result_45, Ret => Cast_Result_46);
--# end
--# expr-done 12916
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21, 18 => Fld_22, 19 => Fld_23)); 
--# expr-done 12917
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12918
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12919
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12920
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1552;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1552;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Mod_Int_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Private_Type_Def
   --
      procedure Initialize_Fields_For_Private_Type_Def
        (Self : Bare_Private_Type_Def
         ; Private_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Private_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Private_Type_Def_F_Has_Limited : Bare_Limited_Node
        ) is
      begin
            Self.Private_Type_Def_F_Has_Abstract := Private_Type_Def_F_Has_Abstract;
            Self.Private_Type_Def_F_Has_Tagged := Private_Type_Def_F_Has_Tagged;
            Self.Private_Type_Def_F_Has_Limited := Private_Type_Def_F_Has_Limited;
      end Initialize_Fields_For_Private_Type_Def;
   function Private_Type_Def_F_Has_Abstract
     (Node : Bare_Private_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Abstract;
   end;
   function Private_Type_Def_F_Has_Tagged
     (Node : Bare_Private_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Tagged;
   end;
   function Private_Type_Def_F_Has_Limited
     (Node : Bare_Private_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Limited;
   end;
--# property-start PrivateTypeDef.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17853'
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Private_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12922 '<FieldAccess for as_bool at nodes.lkt:17853:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17853'
--# expr-start 12921 '<FieldAccess for has_tagged at nodes.lkt:17853:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17853'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Private_Type_Def_F_Has_Tagged;
--# expr-done 12921
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 12922
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start PrivateTypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17855'
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Is_Limited_Type
  (Node : Bare_Private_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Limited_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 12924 '<FieldAccess for as_bool at nodes.lkt:17855:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17855'
--# expr-start 12923 '<FieldAccess for has_limited at nodes.lkt:17855:35>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17855'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Private_Type_Def_F_Has_Limited;
--# expr-done 12923
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 12924
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Private_Type_Def_P_Is_Limited_Type;
--# end
--# property-start PrivateTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17858'
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Xref_Equation
  (Node : Bare_Private_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1555 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1555 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1555;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17858:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1555;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1555;
            raise;
      end;
   return Property_Result;
end Private_Type_Def_P_Xref_Equation;
--# end
--# property-start PrivateTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17861'
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Predefined_Operators
  (Node : Bare_Private_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Private_Type_Def  := Bare_Private_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1556 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
Fld_1 : Bare_Limited_Node;
Fld_2 : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1556 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1556;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Private_Type_Def_Private_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Private_Type_Def,
                                As_Bare_Private_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12928 '<If at nodes.lkt:17862:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17862'
--# expr-start 12926 '<FieldAccess for as_bool at nodes.lkt:17862:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17862'
--# expr-start 12925 '<FieldAccess for has_limited at nodes.lkt:17862:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17862'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Private_Type_Def_F_Has_Limited;
--# expr-done 12925
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld_1);
--# end
--# expr-done 12926
if Fld_2 then
   If_Result := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 12927 '<FieldAccess for predefined_equality_operators at nodes.lkt:17862:71>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17862'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 12927
   If_Result := Fld;
end if;
      Inc_Ref (If_Result);
--# expr-done 12928
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1556;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1556;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Private_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Real_Type_Def
   --
--# property-start RealTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17869'
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Is_Static
  (Node : Bare_Real_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Real_Type_Def  := Bare_Real_Type_Def (Node);
      --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Real_Type_Def_P_Is_Static;
--# end
--# property-start RealTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17872'
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Predefined_Operators
  (Node : Bare_Real_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Real_Type_Def  := Bare_Real_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1558 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Int_Type : Bare_Base_Type_Decl;
Local_Root_Int_Type : Bare_Base_Type_Decl;
Local_Defaults : Internal_Env_Assoc_Array_Access;
Local_Specials : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_22 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_23 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit_1 : Internal_Env_Assoc_Array_Access;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Bare_Base_Type_Decl;
Cast_Expr_38 : Bare_Type_Decl;
Cast_Result_39 : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
Concat_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
Let_Result_3 : Internal_Env_Assoc_Array_Access;
Let_Result_4 : Internal_Env_Assoc_Array_Access;
Let_Result_5 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1558 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Local_Defaults);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Local_Specials);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_1558;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Real_Type_Def_Real_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Real_Type_Def,
                                As_Bare_Real_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12982 '<Let at nodes.lkt:17873:9>' Let_Result_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17873'
--# scope-start
--# expr-start 12930 '<Cast to TypeDecl at nodes.lkt:17873:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17873'
--# expr-start 12929 '<parent at nodes.lkt:17873:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17873'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 12929
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 12930
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 12981 '<Let at nodes.lkt:17874:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17874'
--# scope-start
--# expr-start 12932 '<FieldAccess for node at nodes.lkt:17874:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17874'
--# expr-start 12931 '<FieldAccess for bool_type at nodes.lkt:17874:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17874'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 12931
Fld_2 := Fld_1.Node;
--# expr-done 12932
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 12980 '<Let at nodes.lkt:17875:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17875'
--# scope-start
--# expr-start 12934 '<FieldAccess for node at nodes.lkt:17875:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17875'
--# expr-start 12933 '<FieldAccess for int_type at nodes.lkt:17875:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17875'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 12933
Fld_4 := Fld_3.Node;
--# expr-done 12934
Local_Int_Type := Fld_4; 
--# bind int_type Local_Int_Type
--# expr-start 12979 '<Let at nodes.lkt:17876:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17876'
--# scope-start
--# expr-start 12936 '<FieldAccess for node at nodes.lkt:17876:29>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17876'
--# expr-start 12935 '<FieldAccess for root_int_type at nodes.lkt:17876:29>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17876'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Root_Int_Type (Node => Self);
--# end
--# expr-done 12935
Fld_6 := Fld_5.Node;
--# expr-done 12936
Local_Root_Int_Type := Fld_6; 
--# bind root_int_type Local_Root_Int_Type
--# expr-start 12978 '<Let at nodes.lkt:17877:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17877'
--# scope-start
--# expr-start 12965 '<ArrayLiteral at nodes.lkt:17877:24>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17877'
--# expr-start 12938 '<FieldAccess for create_binop_assoc at nodes.lkt:17877:25>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17877'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12937 '<Symbol "+" at nodes.lkt:17878:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17878'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12937
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 12938
--# expr-start 12940 '<FieldAccess for create_binop_assoc at nodes.lkt:17879:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17879'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12939 '<Symbol "-" at nodes.lkt:17880:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17880'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12939
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 12940
--# expr-start 12942 '<FieldAccess for create_binop_assoc at nodes.lkt:17881:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17881'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12941 '<Symbol "*" at nodes.lkt:17882:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17882'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12941
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 12942
--# expr-start 12944 '<FieldAccess for create_binop_assoc at nodes.lkt:17883:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17883'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12943 '<Symbol "/" at nodes.lkt:17884:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17884'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12943
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 12944
--# expr-start 12946 '<FieldAccess for create_binop_assoc at nodes.lkt:17885:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17885'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12945 '<Symbol "**" at nodes.lkt:17886:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17886'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 12945
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Local_Int_Type, Ret => Cast_Result_14);
--# end
--# expr-done 12946
--# expr-start 12948 '<FieldAccess for create_binop_assoc at nodes.lkt:17887:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17887'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12947 '<Symbol "<" at nodes.lkt:17888:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17888'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 12947
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Cast_Result_16, Ret => Local_Bool_Type);
--# end
--# expr-done 12948
--# expr-start 12950 '<FieldAccess for create_binop_assoc at nodes.lkt:17889:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17889'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12949 '<Symbol "<=" at nodes.lkt:17890:13>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17890'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 12949
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Cast_Result_18, Ret => Local_Bool_Type);
--# end
--# expr-done 12950
--# expr-start 12952 '<FieldAccess for create_binop_assoc at nodes.lkt:17891:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17891'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12951 '<Symbol "=" at nodes.lkt:17892:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17892'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 12951
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Cast_Result_20, Ret => Local_Bool_Type);
--# end
--# expr-done 12952
--# expr-start 12954 '<FieldAccess for create_binop_assoc at nodes.lkt:17893:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17893'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12953 '<Symbol "/=" at nodes.lkt:17894:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17894'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 12953
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Local_Bool_Type);
--# end
--# expr-done 12954
--# expr-start 12956 '<FieldAccess for create_binop_assoc at nodes.lkt:17895:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17895'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12955 '<Symbol ">" at nodes.lkt:17896:13>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17896'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 12955
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Local_Bool_Type);
--# end
--# expr-done 12956
--# expr-start 12958 '<FieldAccess for create_binop_assoc at nodes.lkt:17897:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17897'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12957 '<Symbol ">=" at nodes.lkt:17898:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17898'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 12957
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Local_Bool_Type);
--# end
--# expr-done 12958
--# expr-start 12960 '<FieldAccess for create_unop_assoc at nodes.lkt:17899:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12959 '<Symbol "+" at nodes.lkt:17899:35>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 12959
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_unop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_11, Rhs => Cast_Result_27, Ret => Cast_Result_28);
--# end
--# expr-done 12960
--# expr-start 12962 '<FieldAccess for create_unop_assoc at nodes.lkt:17899:68>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12961 '<Symbol "-" at nodes.lkt:17899:91>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 12961
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_12, Rhs => Cast_Result_29, Ret => Cast_Result_30);
--# end
--# expr-done 12962
--# expr-start 12964 '<FieldAccess for create_unop_assoc at nodes.lkt:17899:124>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12963 '<Symbol "abs" at nodes.lkt:17899:147>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17899'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 12963
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Local_Self_Type; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_13, Rhs => Cast_Result_31, Ret => Cast_Result_32);
--# end
--# expr-done 12964
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_7, 2 => Fld_8, 3 => Fld_9, 4 => Fld_10, 5 => Fld_11, 6 => Fld_12, 7 => Fld_13, 8 => Fld_14, 9 => Fld_15, 10 => Fld_16, 11 => Fld_17, 12 => Fld_18, 13 => Fld_19, 14 => Fld_20)); 
--# expr-done 12965
Local_Defaults := Array_Lit; Inc_Ref (Local_Defaults);
--# bind defaults Local_Defaults
--# expr-start 12977 '<Let at nodes.lkt:17901:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
--# scope-start
--# expr-start 12976 '<If at nodes.lkt:17901:24>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
--# expr-start 12968 '<Eq at nodes.lkt:17901:27>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
Cast_Expr_38 := Local_Self_Type; 
      Cast_Result_39 := Cast_Expr_38;
--# expr-start 12967 '<FieldAccess for node at nodes.lkt:17901:40>' Fld_25 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
--# expr-start 12966 '<FieldAccess for root_real_type at nodes.lkt:17901:40>' Fld_24 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.root_real_type
Fld_24 := Libadalang.Implementation.Ada_Node_P_Root_Real_Type (Node => Self);
--# end
--# expr-done 12966
Fld_25 := Fld_24.Node;
--# expr-done 12967
Is_Equal := Cast_Result_39 = Fld_25; 
--# expr-done 12968
if Is_Equal then
   --# expr-start 12975 '<ArrayLiteral at nodes.lkt:17901:72>' Array_Lit_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
--# expr-start 12970 '<FieldAccess for create_binop_assoc at nodes.lkt:17901:73>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17901'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12969 '<Symbol "*" at nodes.lkt:17902:13>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17902'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12969
Cast_Expr_32 := Local_Self_Type; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Local_Self_Type; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_binop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_14, Lhs => Local_Root_Int_Type, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 12970
--# expr-start 12972 '<FieldAccess for create_binop_assoc at nodes.lkt:17903:12>' Fld_22 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17903'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12971 '<Symbol "*" at nodes.lkt:17904:13>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17904'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12971
Cast_Expr_34 := Local_Self_Type; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Local_Self_Type; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_binop_assoc
Fld_22 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_15, Lhs => Cast_Result_35, Rhs => Local_Root_Int_Type, Ret => Cast_Result_36);
--# end
--# expr-done 12972
--# expr-start 12974 '<FieldAccess for create_binop_assoc at nodes.lkt:17905:12>' Fld_23 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17905'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12973 '<Symbol "/" at nodes.lkt:17906:13>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17906'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12973
Cast_Expr_36 := Local_Self_Type; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Local_Self_Type; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_binop_assoc
Fld_23 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_16, Lhs => Cast_Result_37, Rhs => Local_Root_Int_Type, Ret => Cast_Result_38);
--# end
--# expr-done 12974
Array_Lit_1 := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_21, 2 => Fld_22, 3 => Fld_23)); 
--# expr-done 12975
   If_Result := Array_Lit_1;
else
   If_Result := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 12976
Local_Specials := If_Result; Inc_Ref (Local_Specials);
--# bind specials Local_Specials
Concat_Result := Concat (Local_Defaults, Local_Specials); 
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 12977
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 12978
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 12979
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 12980
Let_Result_4 := Let_Result_3; Inc_Ref (Let_Result_4);
--# end
--# expr-done 12981
Let_Result_5 := Let_Result_4; Inc_Ref (Let_Result_5);
--# end
--# expr-done 12982
         Property_Result := Let_Result_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1558;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1558;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Real_Type_Def_P_Predefined_Operators;
--# end
--# property-start RealTypeDef.universal_fixed_predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17915'
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Universal_Fixed_Predefined_Operators
  (Node : Bare_Real_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Real_Type_Def  := Bare_Real_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1559 with Inline_Always;
      Local_Uf : Bare_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Fld_2 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Fld_3 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1559 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1559;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Real_Type_Def_Real_Type_Def_P_Universal_Fixed_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Real_Type_Def,
                                As_Bare_Real_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 12990 '<Let at nodes.lkt:17916:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17916'
--# scope-start
--# expr-start 12984 '<FieldAccess for node at nodes.lkt:17916:18>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17916'
--# expr-start 12983 '<FieldAccess for universal_fixed_type at nodes.lkt:17916:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17916'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.universal_fixed_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Fixed_Type (Node => Self);
--# end
--# expr-done 12983
Fld_1 := Fld.Node;
--# expr-done 12984
Local_Uf := Fld_1; 
--# bind uf Local_Uf
--# expr-start 12989 '<ArrayLiteral at nodes.lkt:17918:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17918'
--# expr-start 12986 '<FieldAccess for create_binop_assoc at nodes.lkt:17918:10>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17918'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12985 '<Symbol "*" at nodes.lkt:17918:34>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17918'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 12985
--# property-call-start AdaNode.create_binop_assoc
Fld_2 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Local_Uf, Rhs => Local_Uf, Ret => Local_Uf);
--# end
--# expr-done 12986
--# expr-start 12988 '<FieldAccess for create_binop_assoc at nodes.lkt:17918:57>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17918'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12987 '<Symbol "/" at nodes.lkt:17918:81>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17918'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 12987
--# property-call-start AdaNode.create_binop_assoc
Fld_3 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Local_Uf, Rhs => Local_Uf, Ret => Local_Uf);
--# end
--# expr-done 12988
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_2, 2 => Fld_3)); 
--# expr-done 12989
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 12990
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1559;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1559;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Real_Type_Def_P_Universal_Fixed_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Decimal_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Decimal_Fixed_Point_Def
        (Self : Bare_Decimal_Fixed_Point_Def
         ; Decimal_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Digits : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Decimal_Fixed_Point_Def_F_Delta := Decimal_Fixed_Point_Def_F_Delta;
            Self.Decimal_Fixed_Point_Def_F_Digits := Decimal_Fixed_Point_Def_F_Digits;
            Self.Decimal_Fixed_Point_Def_F_Range := Decimal_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Decimal_Fixed_Point_Def;
   function Decimal_Fixed_Point_Def_F_Delta
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Delta;
   end;
   function Decimal_Fixed_Point_Def_F_Digits
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Digits;
   end;
   function Decimal_Fixed_Point_Def_F_Range
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Range;
   end;
--# property-start DecimalFixedPointDef.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17929'
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
--# property-start DecimalFixedPointDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Xref_Equation
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
     Ent : Internal_Entity_Decimal_Fixed_Point_Def :=
       Internal_Entity_Decimal_Fixed_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1561 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred_4 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_14 : Internal_Entity_Range_Spec;
Fld_15 : Logic_Equation;
Fld_16 : Bare_Range_Spec;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_5 : Logic_Equation;
            procedure Finalizer_Scope_1561 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_15);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_1561;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13015 '<And at nodes.lkt:17933:37>' And_Pred_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 13009 '<And at nodes.lkt:17933:37>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 13006 '<And at nodes.lkt:17933:37>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 13003 '<And at nodes.lkt:17933:37>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 12999 '<And at nodes.lkt:17933:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 12996 '<And at nodes.lkt:17933:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17933'
--# expr-start 12993 '<FieldAccess for universal_real_bind at nodes.lkt:17936:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17936'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 12992 '<FieldAccess for expected_type_var at nodes.lkt:17936:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17936'
--# expr-start 12991 '<FieldAccess for delta at nodes.lkt:17936:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17936'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12991
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 12992
--# property-call-start AdaNode.universal_real_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 12993
--# expr-start 12995 '<FieldAccess for sub_equation at nodes.lkt:17937:11>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17937'
--# expr-start 12994 '<FieldAccess for delta at nodes.lkt:17937:11>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17937'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12994
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 12995
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 12996
--# expr-start 12998 '<FieldAccess for matches_expected_type at nodes.lkt:17937:41>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17937'
--# expr-start 12997 '<FieldAccess for delta at nodes.lkt:17937:41>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17937'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 12997
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 12998
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 12999
--# expr-start 13002 '<FieldAccess for universal_int_bind at nodes.lkt:17939:9>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17939'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13001 '<FieldAccess for expected_type_var at nodes.lkt:17939:33>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17939'
--# expr-start 13000 '<FieldAccess for digits at nodes.lkt:17939:33>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17939'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 13000
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_7.Node);
--# expr-done 13001
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 13002
And_Pred_2 := Create_And (And_Pred_1, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 13003
--# expr-start 13005 '<FieldAccess for sub_equation at nodes.lkt:17940:11>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17940'
--# expr-start 13004 '<FieldAccess for digits at nodes.lkt:17940:11>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17940'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 13004
      if Fld_10.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_10.Info);
--# end
--# expr-done 13005
And_Pred_3 := Create_And (And_Pred_2, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 13006
--# expr-start 13008 '<FieldAccess for matches_expected_type at nodes.lkt:17940:42>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17940'
--# expr-start 13007 '<FieldAccess for digits at nodes.lkt:17940:42>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17940'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Digits, Info => Ent.Info);
--# expr-done 13007
      if Fld_12.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_12.Node, Origin => Origin);
--# end
--# expr-done 13008
And_Pred_4 := Create_And (And_Pred_3, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 13009
--# expr-start 13014 '<If at nodes.lkt:17941:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17941'
--# expr-start 13011 '<IsNull at nodes.lkt:17941:12>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17941'
--# expr-start 13010 '<FieldAccess for range at nodes.lkt:17941:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17941'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_16 := Self.Decimal_Fixed_Point_Def_F_Range;
--# expr-done 13010
Is_Null := Fld_16 = null; 
--# expr-done 13011
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17941:36") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 13013 '<FieldAccess for sub_equation at nodes.lkt:17941:47>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17941'
--# expr-start 13012 '<FieldAccess for range at nodes.lkt:17941:47>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17941'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Decimal_Fixed_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 13012
      if Fld_14.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_14.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_14.Info);
--# end
--# expr-done 13013
   If_Result := Fld_15;
end if;
      Inc_Ref (If_Result);
--# expr-done 13014
And_Pred_5 := Create_And (And_Pred_4, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17933:37") else null)); 
--# expr-done 13015
         Property_Result := And_Pred_5;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1561;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1561;
            raise;
      end;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Xref_Equation;
--# end
--# property-start DecimalFixedPointDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17945'
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Predefined_Operators
  (Node : Bare_Decimal_Fixed_Point_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Decimal_Fixed_Point_Def  := Bare_Decimal_Fixed_Point_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1562 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Int_Type : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1562 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1562;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Decimal_Fixed_Point_Def_Decimal_Fixed_Point_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Decimal_Fixed_Point_Def,
                                As_Bare_Decimal_Fixed_Point_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 13059 '<Let at nodes.lkt:17946:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17946'
--# scope-start
--# expr-start 13017 '<Cast to TypeDecl at nodes.lkt:17946:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17946'
--# expr-start 13016 '<parent at nodes.lkt:17946:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17946'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13016
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 13017
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 13058 '<Let at nodes.lkt:17947:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17947'
--# scope-start
--# expr-start 13019 '<FieldAccess for node at nodes.lkt:17947:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17947'
--# expr-start 13018 '<FieldAccess for bool_type at nodes.lkt:17947:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17947'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 13018
Fld_2 := Fld_1.Node;
--# expr-done 13019
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 13057 '<Let at nodes.lkt:17948:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17948'
--# scope-start
--# expr-start 13021 '<FieldAccess for node at nodes.lkt:17948:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17948'
--# expr-start 13020 '<FieldAccess for int_type at nodes.lkt:17948:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17948'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 13020
Fld_4 := Fld_3.Node;
--# expr-done 13021
Local_Int_Type := Fld_4; 
--# bind int_type Local_Int_Type
--# expr-start 13056 '<ArrayLiteral at nodes.lkt:17950:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17950'
--# expr-start 13023 '<FieldAccess for create_binop_assoc at nodes.lkt:17950:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17950'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13022 '<Symbol "+" at nodes.lkt:17951:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17951'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13022
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 13023
--# expr-start 13025 '<FieldAccess for create_binop_assoc at nodes.lkt:17952:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17952'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13024 '<Symbol "-" at nodes.lkt:17953:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17953'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13024
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 13025
--# expr-start 13027 '<FieldAccess for create_binop_assoc at nodes.lkt:17954:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17954'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13026 '<Symbol "*" at nodes.lkt:17955:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17955'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13026
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 13027
--# expr-start 13029 '<FieldAccess for create_binop_assoc at nodes.lkt:17956:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17956'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13028 '<Symbol "/" at nodes.lkt:17957:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17957'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 13028
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 13029
--# expr-start 13031 '<FieldAccess for create_binop_assoc at nodes.lkt:17958:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13030 '<Symbol "*" at nodes.lkt:17958:36>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13030
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Local_Int_Type, Rhs => Cast_Result_13, Ret => Cast_Result_14);
--# end
--# expr-done 13031
--# expr-start 13033 '<FieldAccess for create_binop_assoc at nodes.lkt:17958:79>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13032 '<Symbol "*" at nodes.lkt:17958:103>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13032
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Local_Int_Type, Ret => Cast_Result_16);
--# end
--# expr-done 13033
--# expr-start 13035 '<FieldAccess for create_binop_assoc at nodes.lkt:17958:146>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13034 '<Symbol "/" at nodes.lkt:17958:170>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 13034
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Local_Int_Type, Ret => Cast_Result_18);
--# end
--# expr-done 13035
--# expr-start 13037 '<FieldAccess for create_binop_assoc at nodes.lkt:17958:213>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17958'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13036 '<Symbol "**" at nodes.lkt:17959:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17959'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 13036
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Local_Int_Type, Ret => Cast_Result_20);
--# end
--# expr-done 13037
--# expr-start 13039 '<FieldAccess for create_binop_assoc at nodes.lkt:17960:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17960'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13038 '<Symbol "<" at nodes.lkt:17961:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17961'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 13038
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Local_Bool_Type);
--# end
--# expr-done 13039
--# expr-start 13041 '<FieldAccess for create_binop_assoc at nodes.lkt:17962:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17962'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13040 '<Symbol "<=" at nodes.lkt:17963:13>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17963'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 13040
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Local_Bool_Type);
--# end
--# expr-done 13041
--# expr-start 13043 '<FieldAccess for create_binop_assoc at nodes.lkt:17964:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17964'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13042 '<Symbol "=" at nodes.lkt:17965:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17965'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 13042
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Local_Bool_Type);
--# end
--# expr-done 13043
--# expr-start 13045 '<FieldAccess for create_binop_assoc at nodes.lkt:17966:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17966'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13044 '<Symbol "/=" at nodes.lkt:17967:13>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17967'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 13044
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Local_Bool_Type);
--# end
--# expr-done 13045
--# expr-start 13047 '<FieldAccess for create_binop_assoc at nodes.lkt:17968:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17968'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13046 '<Symbol ">" at nodes.lkt:17969:13>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17969'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 13046
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Local_Bool_Type);
--# end
--# expr-done 13047
--# expr-start 13049 '<FieldAccess for create_binop_assoc at nodes.lkt:17970:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17970'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13048 '<Symbol ">=" at nodes.lkt:17971:13>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17971'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 13048
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Local_Self_Type; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Local_Bool_Type);
--# end
--# expr-done 13049
--# expr-start 13051 '<FieldAccess for create_unop_assoc at nodes.lkt:17972:12>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13050 '<Symbol "+" at nodes.lkt:17972:35>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13050
Cast_Expr_32 := Local_Self_Type; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Local_Self_Type; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 13051
--# expr-start 13053 '<FieldAccess for create_unop_assoc at nodes.lkt:17972:68>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13052 '<Symbol "-" at nodes.lkt:17972:91>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13052
Cast_Expr_34 := Local_Self_Type; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Local_Self_Type; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 13053
--# expr-start 13055 '<FieldAccess for create_unop_assoc at nodes.lkt:17972:124>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13054 '<Symbol "abs" at nodes.lkt:17972:147>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17972'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 13054
Cast_Expr_36 := Local_Self_Type; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Local_Self_Type; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 13055
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21)); 
--# expr-done 13056
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 13057
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 13058
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 13059
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1562;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1562;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Decimal_Fixed_Point_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Floating_Point_Def
   --
      procedure Initialize_Fields_For_Floating_Point_Def
        (Self : Bare_Floating_Point_Def
         ; Floating_Point_Def_F_Num_Digits : Bare_Expr
         ; Floating_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Floating_Point_Def_F_Num_Digits := Floating_Point_Def_F_Num_Digits;
            Self.Floating_Point_Def_F_Range := Floating_Point_Def_F_Range;
      end Initialize_Fields_For_Floating_Point_Def;
   function Floating_Point_Def_F_Num_Digits
     (Node : Bare_Floating_Point_Def) return Bare_Expr
   is
   begin
         return Node.Floating_Point_Def_F_Num_Digits;
   end;
   function Floating_Point_Def_F_Range
     (Node : Bare_Floating_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Floating_Point_Def_F_Range;
   end;
--# property-start FloatingPointDef.is_float_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17982'
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Is_Float_Type
  (Node : Bare_Floating_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Floating_Point_Def  := Bare_Floating_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Floating_Point_Def_P_Is_Float_Type;
--# end
--# property-start FloatingPointDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17986'
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Xref_Equation
  (Node : Bare_Floating_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Floating_Point_Def  := Bare_Floating_Point_Def (Node);
     Ent : Internal_Entity_Floating_Point_Def :=
       Internal_Entity_Floating_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1564 with Inline_Always;
            procedure Finalizer_Scope_3613 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Range_Spec;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_13 : Internal_Entity_Expr;
Fld_14 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_1564 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_1564;
            procedure Finalizer_Scope_3613 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_3613;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13079 '<And at nodes.lkt:17986:37>' And_Pred_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17986'
--# expr-start 13068 '<And at nodes.lkt:17986:37>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17986'
--# expr-start 13065 '<And at nodes.lkt:17986:37>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17986'
--# expr-start 13062 '<FieldAccess for universal_int_bind at nodes.lkt:17989:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17989'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13061 '<FieldAccess for expected_type_var at nodes.lkt:17989:33>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17989'
--# expr-start 13060 '<FieldAccess for num_digits at nodes.lkt:17989:33>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17989'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 13060
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 13061
--# property-call-start AdaNode.universal_int_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 13062
--# expr-start 13064 '<FieldAccess for sub_equation at nodes.lkt:17990:11>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17990'
--# expr-start 13063 '<FieldAccess for num_digits at nodes.lkt:17990:11>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17990'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 13063
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 13064
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17986:37") else null)); 
--# expr-done 13065
--# expr-start 13067 '<FieldAccess for matches_expected_type at nodes.lkt:17990:46>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17990'
--# expr-start 13066 '<FieldAccess for num_digits at nodes.lkt:17990:46>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17990'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Floating_Point_Def_F_Num_Digits, Info => Ent.Info);
--# expr-done 13066
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 13067
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17986:37") else null)); 
--# expr-done 13068
--# expr-start 13069 '<FieldAccess for range at nodes.lkt:17993:9>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17993'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Floating_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 13069
Var_Expr_0 := Fld_7;
if Var_Expr_0 /= No_Entity_Range_Spec then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 13078 '<And at nodes.lkt:17994:20>' And_Pred_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17994'
--# expr-start 13075 '<And at nodes.lkt:17995:17>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
--# expr-start 13072 '<FieldAccess for universal_real_bind at nodes.lkt:17995:17>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13071 '<FieldAccess for expected_type_var at nodes.lkt:17995:42>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
--# expr-start 13070 '<FieldAccess for range at nodes.lkt:17995:42>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr_0.Node.Range_Spec_F_Range, Info => Var_Expr_0.Info);
--# expr-done 13070
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld_8.Node);
--# expr-done 13071
--# property-call-start AdaNode.universal_real_bind
Fld_10 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_9, Origin => Origin);
--# end
--# expr-done 13072
--# expr-start 13074 '<FieldAccess for sub_equation at nodes.lkt:17995:75>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
--# expr-start 13073 '<FieldAccess for range at nodes.lkt:17995:75>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17995'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Var_Expr_0.Node.Range_Spec_F_Range, Info => Var_Expr_0.Info);
--# expr-done 13073
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_11.Info);
--# end
--# expr-done 13074
And_Pred_2 := Create_And (Fld_10, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17995:17") else null)); 
--# expr-done 13075
--# expr-start 13077 '<FieldAccess for matches_expected_type at nodes.lkt:17996:19>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17996'
--# expr-start 13076 '<FieldAccess for range at nodes.lkt:17996:19>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:17996'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_13 := Create_Internal_Entity_Expr (Node => Var_Expr_0.Node.Range_Spec_F_Range, Info => Var_Expr_0.Info);
--# expr-done 13076
      if Fld_13.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_14 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_13.Node, Origin => Origin);
--# end
--# expr-done 13077
And_Pred_3 := Create_And (And_Pred_2, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17994:20") else null)); 
--# expr-done 13078
Scope_Result := And_Pred_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3613;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17996:64") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred_4 := Create_And (And_Pred_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:17986:37") else null)); 
--# expr-done 13079
         Property_Result := And_Pred_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1564;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1564;
                     Finalizer_Scope_3613;
            raise;
      end;
   return Property_Result;
end Floating_Point_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Ordinary_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Ordinary_Fixed_Point_Def
        (Self : Bare_Ordinary_Fixed_Point_Def
         ; Ordinary_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Ordinary_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Ordinary_Fixed_Point_Def_F_Delta := Ordinary_Fixed_Point_Def_F_Delta;
            Self.Ordinary_Fixed_Point_Def_F_Range := Ordinary_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Ordinary_Fixed_Point_Def;
   function Ordinary_Fixed_Point_Def_F_Delta
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Delta;
   end;
   function Ordinary_Fixed_Point_Def_F_Range
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Range;
   end;
--# property-start OrdinaryFixedPointDef.is_fixed_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18007'
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
--# property-start OrdinaryFixedPointDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18011'
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Xref_Equation
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
     Ent : Internal_Entity_Ordinary_Fixed_Point_Def :=
       Internal_Entity_Ordinary_Fixed_Point_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1566 with Inline_Always;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
True_Rel : Logic_Equation;
Fld_7 : Internal_Entity_Range_Spec;
Fld_8 : Logic_Equation;
Fld_9 : Bare_Range_Spec;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_1566 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_1566;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13094 '<And at nodes.lkt:18014:9>' And_Pred_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
--# expr-start 13088 '<And at nodes.lkt:18014:9>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
--# expr-start 13085 '<And at nodes.lkt:18014:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
--# expr-start 13082 '<FieldAccess for universal_real_bind at nodes.lkt:18014:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13081 '<FieldAccess for expected_type_var at nodes.lkt:18014:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
--# expr-start 13080 '<FieldAccess for delta at nodes.lkt:18014:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18014'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 13080
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Fld.Node);
--# expr-done 13081
--# property-call-start AdaNode.universal_real_bind
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Ent.Node, Type_Var => Fld_1, Origin => Origin);
--# end
--# expr-done 13082
--# expr-start 13084 '<FieldAccess for sub_equation at nodes.lkt:18015:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18015'
--# expr-start 13083 '<FieldAccess for delta at nodes.lkt:18015:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18015'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 13083
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_3.Info);
--# end
--# expr-done 13084
And_Pred := Create_And (Fld_2, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18014:9") else null)); 
--# expr-done 13085
--# expr-start 13087 '<FieldAccess for matches_expected_type at nodes.lkt:18016:13>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18016'
--# expr-start 13086 '<FieldAccess for delta at nodes.lkt:18016:13>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18016'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Delta, Info => Ent.Info);
--# expr-done 13086
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_6 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Fld_5.Node, Origin => Origin);
--# end
--# expr-done 13087
And_Pred_1 := Create_And (And_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18014:9") else null)); 
--# expr-done 13088
--# expr-start 13093 '<If at nodes.lkt:18017:14>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18017'
--# expr-start 13090 '<IsNull at nodes.lkt:18017:17>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18017'
--# expr-start 13089 '<FieldAccess for range at nodes.lkt:18017:17>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18017'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Self.Ordinary_Fixed_Point_Def_F_Range;
--# expr-done 13089
Is_Null := Fld_9 = null; 
--# expr-done 13090
if Is_Null then
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18017:41") else null)); 
   If_Result := True_Rel;
else
   --# expr-start 13092 '<FieldAccess for sub_equation at nodes.lkt:18017:52>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18017'
--# expr-start 13091 '<FieldAccess for range at nodes.lkt:18017:52>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18017'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Ordinary_Fixed_Point_Def_F_Range, Info => Ent.Info);
--# expr-done 13091
      if Fld_7.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_8 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_7.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld_7.Info);
--# end
--# expr-done 13092
   If_Result := Fld_8;
end if;
      Inc_Ref (If_Result);
--# expr-done 13093
And_Pred_2 := Create_And (And_Pred_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18014:9") else null)); 
--# expr-done 13094
         Property_Result := And_Pred_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1566;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1566;
            raise;
      end;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Xref_Equation;
--# end
--# property-start OrdinaryFixedPointDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18020'
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Predefined_Operators
  (Node : Bare_Ordinary_Fixed_Point_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ordinary_Fixed_Point_Def  := Bare_Ordinary_Fixed_Point_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1567 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Int_Type : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Sym_16 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_21 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1567 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1567;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ordinary_Fixed_Point_Def_Ordinary_Fixed_Point_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ordinary_Fixed_Point_Def,
                                As_Bare_Ordinary_Fixed_Point_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 13138 '<Let at nodes.lkt:18021:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18021'
--# scope-start
--# expr-start 13096 '<Cast to TypeDecl at nodes.lkt:18021:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18021'
--# expr-start 13095 '<parent at nodes.lkt:18021:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18021'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13095
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 13096
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 13137 '<Let at nodes.lkt:18022:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18022'
--# scope-start
--# expr-start 13098 '<FieldAccess for node at nodes.lkt:18022:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18022'
--# expr-start 13097 '<FieldAccess for bool_type at nodes.lkt:18022:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18022'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 13097
Fld_2 := Fld_1.Node;
--# expr-done 13098
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 13136 '<Let at nodes.lkt:18023:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18023'
--# scope-start
--# expr-start 13100 '<FieldAccess for node at nodes.lkt:18023:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18023'
--# expr-start 13099 '<FieldAccess for int_type at nodes.lkt:18023:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18023'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 13099
Fld_4 := Fld_3.Node;
--# expr-done 13100
Local_Int_Type := Fld_4; 
--# bind int_type Local_Int_Type
--# expr-start 13135 '<ArrayLiteral at nodes.lkt:18025:9>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18025'
--# expr-start 13102 '<FieldAccess for create_binop_assoc at nodes.lkt:18025:10>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18025'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13101 '<Symbol "+" at nodes.lkt:18026:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18026'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13101
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 13102
--# expr-start 13104 '<FieldAccess for create_binop_assoc at nodes.lkt:18027:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18027'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13103 '<Symbol "-" at nodes.lkt:18028:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18028'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13103
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 13104
--# expr-start 13106 '<FieldAccess for create_binop_assoc at nodes.lkt:18029:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18029'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13105 '<Symbol "*" at nodes.lkt:18030:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18030'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13105
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 13106
--# expr-start 13108 '<FieldAccess for create_binop_assoc at nodes.lkt:18031:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18031'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13107 '<Symbol "/" at nodes.lkt:18032:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18032'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 13107
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 13108
--# expr-start 13110 '<FieldAccess for create_binop_assoc at nodes.lkt:18033:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13109 '<Symbol "*" at nodes.lkt:18033:36>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13109
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Local_Int_Type, Rhs => Cast_Result_13, Ret => Cast_Result_14);
--# end
--# expr-done 13110
--# expr-start 13112 '<FieldAccess for create_binop_assoc at nodes.lkt:18033:79>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13111 '<Symbol "*" at nodes.lkt:18033:103>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13111
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_15, Rhs => Local_Int_Type, Ret => Cast_Result_16);
--# end
--# expr-done 13112
--# expr-start 13114 '<FieldAccess for create_binop_assoc at nodes.lkt:18033:146>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13113 '<Symbol "/" at nodes.lkt:18033:170>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 13113
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_17, Rhs => Local_Int_Type, Ret => Cast_Result_18);
--# end
--# expr-done 13114
--# expr-start 13116 '<FieldAccess for create_binop_assoc at nodes.lkt:18033:213>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18033'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13115 '<Symbol "**" at nodes.lkt:18034:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18034'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 13115
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_19, Rhs => Local_Int_Type, Ret => Cast_Result_20);
--# end
--# expr-done 13116
--# expr-start 13118 '<FieldAccess for create_binop_assoc at nodes.lkt:18035:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18035'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13117 '<Symbol "<" at nodes.lkt:18036:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18036'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 13117
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Local_Bool_Type);
--# end
--# expr-done 13118
--# expr-start 13120 '<FieldAccess for create_binop_assoc at nodes.lkt:18037:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18037'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13119 '<Symbol "<=" at nodes.lkt:18038:13>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18038'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 13119
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Local_Bool_Type);
--# end
--# expr-done 13120
--# expr-start 13122 '<FieldAccess for create_binop_assoc at nodes.lkt:18039:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18039'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13121 '<Symbol "=" at nodes.lkt:18040:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18040'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 13121
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Local_Bool_Type);
--# end
--# expr-done 13122
--# expr-start 13124 '<FieldAccess for create_binop_assoc at nodes.lkt:18041:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18041'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13123 '<Symbol "/=" at nodes.lkt:18042:13>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18042'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 13123
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Local_Bool_Type);
--# end
--# expr-done 13124
--# expr-start 13126 '<FieldAccess for create_binop_assoc at nodes.lkt:18043:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18043'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13125 '<Symbol ">" at nodes.lkt:18044:13>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18044'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 13125
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Local_Bool_Type);
--# end
--# expr-done 13126
--# expr-start 13128 '<FieldAccess for create_binop_assoc at nodes.lkt:18045:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18045'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13127 '<Symbol ">=" at nodes.lkt:18046:13>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18046'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 13127
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Local_Self_Type; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_13, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Local_Bool_Type);
--# end
--# expr-done 13128
--# expr-start 13130 '<FieldAccess for create_unop_assoc at nodes.lkt:18047:12>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13129 '<Symbol "+" at nodes.lkt:18047:35>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13129
Cast_Expr_32 := Local_Self_Type; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Local_Self_Type; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 13130
--# expr-start 13132 '<FieldAccess for create_unop_assoc at nodes.lkt:18047:68>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13131 '<Symbol "-" at nodes.lkt:18047:91>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13131
Cast_Expr_34 := Local_Self_Type; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Local_Self_Type; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 13132
--# expr-start 13134 '<FieldAccess for create_unop_assoc at nodes.lkt:18047:124>' Fld_21 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13133 '<Symbol "abs" at nodes.lkt:18047:147>' Sym_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18047'
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 13133
Cast_Expr_36 := Local_Self_Type; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Local_Self_Type; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_21 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_16, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 13134
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20, 17 => Fld_21)); 
--# expr-done 13135
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 13136
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 13137
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 13138
         Property_Result := Let_Result_2;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1567;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1567;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Ordinary_Fixed_Point_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Record_Type_Def
   --
      procedure Initialize_Fields_For_Record_Type_Def
        (Self : Bare_Record_Type_Def
         ; Record_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Record_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Record_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Record_Type_Def_F_Record_Def : Bare_Base_Record_Def
        ) is
      begin
            Self.Record_Type_Def_F_Has_Abstract := Record_Type_Def_F_Has_Abstract;
            Self.Record_Type_Def_F_Has_Tagged := Record_Type_Def_F_Has_Tagged;
            Self.Record_Type_Def_F_Has_Limited := Record_Type_Def_F_Has_Limited;
            Self.Record_Type_Def_F_Record_Def := Record_Type_Def_F_Record_Def;
      end Initialize_Fields_For_Record_Type_Def;
   function Record_Type_Def_F_Has_Abstract
     (Node : Bare_Record_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Abstract;
   end;
   function Record_Type_Def_F_Has_Tagged
     (Node : Bare_Record_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Tagged;
   end;
   function Record_Type_Def_F_Has_Limited
     (Node : Bare_Record_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Limited;
   end;
   function Record_Type_Def_F_Record_Def
     (Node : Bare_Record_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Record_Type_Def_F_Record_Def;
   end;
--# property-start RecordTypeDef.is_tagged_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18059'
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13140 '<FieldAccess for as_bool at nodes.lkt:18059:34>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18059'
--# expr-start 13139 '<FieldAccess for has_tagged at nodes.lkt:18059:34>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18059'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Record_Type_Def_F_Has_Tagged;
--# expr-done 13139
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 13140
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Record_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start RecordTypeDef.is_limited_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18061'
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Limited_Type
  (Node : Bare_Record_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
     Ent : Internal_Entity_Record_Type_Def :=
       Internal_Entity_Record_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Bare_Limited_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Fld_4 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 13142 '<FieldAccess for as_bool at nodes.lkt:18062:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18062'
--# expr-start 13141 '<FieldAccess for has_limited at nodes.lkt:18062:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18062'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Record_Type_Def_F_Has_Limited;
--# expr-done 13141
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 13142
if Fld_1 then
   If_Result := True;
else
   --# expr-start 13145 '<FieldAccess for has_limited_component at nodes.lkt:18062:39>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18062'
--# expr-start 13144 '<FieldAccess for comps at nodes.lkt:18062:39>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18062'
--# expr-start 13143 '<FieldAccess for record_def at nodes.lkt:18062:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18062'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Record_Type_Def_F_Record_Def, Info => Ent.Info);
--# expr-done 13143
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 13144
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.has_limited_component
Fld_4 := Libadalang.Implementation.Component_List_P_Has_Limited_Component (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 13145
   If_Result := Fld_4;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Record_Type_Def_P_Is_Limited_Type;
--# end
--# property-start RecordTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18065'
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Xref_Equation
  (Node : Bare_Record_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1570 with Inline_Always;
      True_Rel : Logic_Equation;
            procedure Finalizer_Scope_1570 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_1570;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18065:37") else null)); 
         Property_Result := True_Rel;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1570;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1570;
            raise;
      end;
   return Property_Result;
end Record_Type_Def_P_Xref_Equation;
--# end
--# property-start RecordTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18068'
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Predefined_Operators
  (Node : Bare_Record_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Record_Type_Def  := Bare_Record_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1571 with Inline_Always;
      Fld : Internal_Env_Assoc_Array_Access;
Fld_1 : Bare_Limited_Node;
Fld_2 : Boolean;
If_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1571 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1571;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Record_Type_Def_Record_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Record_Type_Def,
                                As_Bare_Record_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 13149 '<If at nodes.lkt:18069:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18069'
--# expr-start 13147 '<FieldAccess for as_bool at nodes.lkt:18069:12>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18069'
--# expr-start 13146 '<FieldAccess for has_limited at nodes.lkt:18069:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18069'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Record_Type_Def_F_Has_Limited;
--# expr-done 13146
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Limited.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_Limited_Node_P_As_Bool (Node => Fld_1);
--# end
--# expr-done 13147
if Fld_2 then
   If_Result := No_Internal_Env_Assoc_Array_Type;
else
   --# expr-start 13148 '<FieldAccess for predefined_equality_operators at nodes.lkt:18069:71>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18069'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeDef.predefined_equality_operators
Fld := Libadalang.Implementation.Type_Def_P_Predefined_Equality_Operators (Node => Self);
--# end
--# expr-done 13148
   If_Result := Fld;
end if;
      Inc_Ref (If_Result);
--# expr-done 13149
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1571;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1571;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Record_Type_Def_P_Predefined_Operators;
--# end
   --
   --  Primitives for Bare_Signed_Int_Type_Def
   --
      procedure Initialize_Fields_For_Signed_Int_Type_Def
        (Self : Bare_Signed_Int_Type_Def
         ; Signed_Int_Type_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Signed_Int_Type_Def_F_Range := Signed_Int_Type_Def_F_Range;
      end Initialize_Fields_For_Signed_Int_Type_Def;
   function Signed_Int_Type_Def_F_Range
     (Node : Bare_Signed_Int_Type_Def) return Bare_Range_Spec
   is
   begin
         return Node.Signed_Int_Type_Def_F_Range;
   end;
--# property-start SignedIntTypeDef.is_int_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18077'
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Signed_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start SignedIntTypeDef.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18080'
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Signed_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1573 with Inline_Always;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1573 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1573;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13151 '<FieldAccess for xref_equation at nodes.lkt:18080:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18080'
--# expr-start 13150 '<FieldAccess for range at nodes.lkt:18080:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18080'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 13150
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 13151
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1573;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1573;
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start SignedIntTypeDef.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18082'
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Signed_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13154 '<FieldAccess for discrete_range at nodes.lkt:18082:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18082'
--# expr-start 13153 '<FieldAccess for range at nodes.lkt:18082:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18082'
--# expr-start 13152 '<FieldAccess for range at nodes.lkt:18082:43>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18082'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 13152
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 13153
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_2 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 13154
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start SignedIntTypeDef.predefined_operators 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18085'
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Predefined_Operators
  (Node : Bare_Signed_Int_Type_Def
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
      --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1575 with Inline_Always;
      Local_Self_Type : Bare_Type_Decl;
Local_Bool_Type : Bare_Base_Type_Decl;
Local_Int_Type : Bare_Base_Type_Decl;
Local_Defaults : Internal_Env_Assoc_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Base_Type_Decl;
Sym : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result_1 : Bare_Base_Type_Decl;
Cast_Expr_1 : Bare_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_3 : Bare_Base_Type_Decl;
Fld_5 : Internal_Env_Assoc := No_Env_Assoc;
Sym_1 : Symbol_Type;
Cast_Expr_3 : Bare_Type_Decl;
Cast_Result_4 : Bare_Base_Type_Decl;
Cast_Expr_4 : Bare_Type_Decl;
Cast_Result_5 : Bare_Base_Type_Decl;
Cast_Expr_5 : Bare_Type_Decl;
Cast_Result_6 : Bare_Base_Type_Decl;
Fld_6 : Internal_Env_Assoc := No_Env_Assoc;
Sym_2 : Symbol_Type;
Cast_Expr_6 : Bare_Type_Decl;
Cast_Result_7 : Bare_Base_Type_Decl;
Cast_Expr_7 : Bare_Type_Decl;
Cast_Result_8 : Bare_Base_Type_Decl;
Cast_Expr_8 : Bare_Type_Decl;
Cast_Result_9 : Bare_Base_Type_Decl;
Fld_7 : Internal_Env_Assoc := No_Env_Assoc;
Sym_3 : Symbol_Type;
Cast_Expr_9 : Bare_Type_Decl;
Cast_Result_10 : Bare_Base_Type_Decl;
Cast_Expr_10 : Bare_Type_Decl;
Cast_Result_11 : Bare_Base_Type_Decl;
Cast_Expr_11 : Bare_Type_Decl;
Cast_Result_12 : Bare_Base_Type_Decl;
Fld_8 : Internal_Env_Assoc := No_Env_Assoc;
Sym_4 : Symbol_Type;
Cast_Expr_12 : Bare_Type_Decl;
Cast_Result_13 : Bare_Base_Type_Decl;
Cast_Expr_13 : Bare_Type_Decl;
Cast_Result_14 : Bare_Base_Type_Decl;
Cast_Expr_14 : Bare_Type_Decl;
Cast_Result_15 : Bare_Base_Type_Decl;
Fld_9 : Internal_Env_Assoc := No_Env_Assoc;
Sym_5 : Symbol_Type;
Cast_Expr_15 : Bare_Type_Decl;
Cast_Result_16 : Bare_Base_Type_Decl;
Cast_Expr_16 : Bare_Type_Decl;
Cast_Result_17 : Bare_Base_Type_Decl;
Cast_Expr_17 : Bare_Type_Decl;
Cast_Result_18 : Bare_Base_Type_Decl;
Fld_10 : Internal_Env_Assoc := No_Env_Assoc;
Sym_6 : Symbol_Type;
Cast_Expr_18 : Bare_Type_Decl;
Cast_Result_19 : Bare_Base_Type_Decl;
Cast_Expr_19 : Bare_Type_Decl;
Cast_Result_20 : Bare_Base_Type_Decl;
Fld_11 : Internal_Env_Assoc := No_Env_Assoc;
Sym_7 : Symbol_Type;
Cast_Expr_20 : Bare_Type_Decl;
Cast_Result_21 : Bare_Base_Type_Decl;
Cast_Expr_21 : Bare_Type_Decl;
Cast_Result_22 : Bare_Base_Type_Decl;
Fld_12 : Internal_Env_Assoc := No_Env_Assoc;
Sym_8 : Symbol_Type;
Cast_Expr_22 : Bare_Type_Decl;
Cast_Result_23 : Bare_Base_Type_Decl;
Cast_Expr_23 : Bare_Type_Decl;
Cast_Result_24 : Bare_Base_Type_Decl;
Fld_13 : Internal_Env_Assoc := No_Env_Assoc;
Sym_9 : Symbol_Type;
Cast_Expr_24 : Bare_Type_Decl;
Cast_Result_25 : Bare_Base_Type_Decl;
Cast_Expr_25 : Bare_Type_Decl;
Cast_Result_26 : Bare_Base_Type_Decl;
Fld_14 : Internal_Env_Assoc := No_Env_Assoc;
Sym_10 : Symbol_Type;
Cast_Expr_26 : Bare_Type_Decl;
Cast_Result_27 : Bare_Base_Type_Decl;
Cast_Expr_27 : Bare_Type_Decl;
Cast_Result_28 : Bare_Base_Type_Decl;
Fld_15 : Internal_Env_Assoc := No_Env_Assoc;
Sym_11 : Symbol_Type;
Cast_Expr_28 : Bare_Type_Decl;
Cast_Result_29 : Bare_Base_Type_Decl;
Cast_Expr_29 : Bare_Type_Decl;
Cast_Result_30 : Bare_Base_Type_Decl;
Fld_16 : Internal_Env_Assoc := No_Env_Assoc;
Sym_12 : Symbol_Type;
Cast_Expr_30 : Bare_Type_Decl;
Cast_Result_31 : Bare_Base_Type_Decl;
Cast_Expr_31 : Bare_Type_Decl;
Cast_Result_32 : Bare_Base_Type_Decl;
Fld_17 : Internal_Env_Assoc := No_Env_Assoc;
Sym_13 : Symbol_Type;
Cast_Expr_32 : Bare_Type_Decl;
Cast_Result_33 : Bare_Base_Type_Decl;
Cast_Expr_33 : Bare_Type_Decl;
Cast_Result_34 : Bare_Base_Type_Decl;
Fld_18 : Internal_Env_Assoc := No_Env_Assoc;
Sym_14 : Symbol_Type;
Cast_Expr_34 : Bare_Type_Decl;
Cast_Result_35 : Bare_Base_Type_Decl;
Cast_Expr_35 : Bare_Type_Decl;
Cast_Result_36 : Bare_Base_Type_Decl;
Fld_19 : Internal_Env_Assoc := No_Env_Assoc;
Sym_15 : Symbol_Type;
Cast_Expr_36 : Bare_Type_Decl;
Cast_Result_37 : Bare_Base_Type_Decl;
Cast_Expr_37 : Bare_Type_Decl;
Cast_Result_38 : Bare_Base_Type_Decl;
Fld_20 : Internal_Env_Assoc := No_Env_Assoc;
Array_Lit : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Env_Assoc_Array_Access;
Let_Result_2 : Internal_Env_Assoc_Array_Access;
Let_Result_3 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1575 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Local_Defaults);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1575;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Signed_Int_Type_Def_Signed_Int_Type_Def_P_Predefined_Operators,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Signed_Int_Type_Def,
                                As_Bare_Signed_Int_Type_Def => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            if Find_Memoized_Value
              (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
            then
               --# memoization-lookup
               if Mmz_Val.Kind = Mmz_Evaluating then
                  --# memoization-return
                  Raise_Property_Exception
                    (Self,
                     Property_Error'Identity,
                     "Infinite recursion detected");
               elsif Mmz_Val.Kind = Mmz_Error then
                  --# memoization-return
                  Reraise_Memoized_Error (Mmz_Val);
               else
                  Property_Result := Mmz_Val.As_Internal_Env_Assoc_Array_Access;
                     Inc_Ref (Property_Result);
                  --# memoization-return
                  return Property_Result;
               end if;
               --# end
            end if;
         end if;
      end if;
      begin
   --# scope-start
         --# expr-start 13197 '<Let at nodes.lkt:18086:9>' Let_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18086'
--# scope-start
--# expr-start 13156 '<Cast to TypeDecl at nodes.lkt:18086:25>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18086'
--# expr-start 13155 '<parent at nodes.lkt:18086:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18086'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13155
   if Fld = null
      or else Fld.Kind in Ada_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 13156
Local_Self_Type := Cast_Result; 
--# bind self_type Local_Self_Type
--# expr-start 13196 '<Let at nodes.lkt:18087:9>' Let_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18087'
--# scope-start
--# expr-start 13158 '<FieldAccess for node at nodes.lkt:18087:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18087'
--# expr-start 13157 '<FieldAccess for bool_type at nodes.lkt:18087:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18087'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.bool_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 13157
Fld_2 := Fld_1.Node;
--# expr-done 13158
Local_Bool_Type := Fld_2; 
--# bind bool_type Local_Bool_Type
--# expr-start 13195 '<Let at nodes.lkt:18088:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18088'
--# scope-start
--# expr-start 13160 '<FieldAccess for node at nodes.lkt:18088:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18088'
--# expr-start 13159 '<FieldAccess for int_type at nodes.lkt:18088:24>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18088'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.int_type
Fld_3 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 13159
Fld_4 := Fld_3.Node;
--# expr-done 13160
Local_Int_Type := Fld_4; 
--# bind int_type Local_Int_Type
--# expr-start 13194 '<Let at nodes.lkt:18089:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18089'
--# scope-start
--# expr-start 13193 '<ArrayLiteral at nodes.lkt:18089:24>' Array_Lit 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18089'
--# expr-start 13162 '<FieldAccess for create_binop_assoc at nodes.lkt:18089:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18089'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13161 '<Symbol "+" at nodes.lkt:18090:13>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18090'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13161
Cast_Expr := Local_Self_Type; 
      Cast_Result_1 := Cast_Expr;
Cast_Expr_1 := Local_Self_Type; 
      Cast_Result_2 := Cast_Expr_1;
Cast_Expr_2 := Local_Self_Type; 
      Cast_Result_3 := Cast_Expr_2;
--# property-call-start AdaNode.create_binop_assoc
Fld_5 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym, Lhs => Cast_Result_1, Rhs => Cast_Result_2, Ret => Cast_Result_3);
--# end
--# expr-done 13162
--# expr-start 13164 '<FieldAccess for create_binop_assoc at nodes.lkt:18091:12>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18091'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13163 '<Symbol "-" at nodes.lkt:18092:13>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18092'
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13163
Cast_Expr_3 := Local_Self_Type; 
      Cast_Result_4 := Cast_Expr_3;
Cast_Expr_4 := Local_Self_Type; 
      Cast_Result_5 := Cast_Expr_4;
Cast_Expr_5 := Local_Self_Type; 
      Cast_Result_6 := Cast_Expr_5;
--# property-call-start AdaNode.create_binop_assoc
Fld_6 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_1, Lhs => Cast_Result_4, Rhs => Cast_Result_5, Ret => Cast_Result_6);
--# end
--# expr-done 13164
--# expr-start 13166 '<FieldAccess for create_binop_assoc at nodes.lkt:18093:12>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18093'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13165 '<Symbol "*" at nodes.lkt:18094:13>' Sym_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18094'
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_2); 
--# expr-done 13165
Cast_Expr_6 := Local_Self_Type; 
      Cast_Result_7 := Cast_Expr_6;
Cast_Expr_7 := Local_Self_Type; 
      Cast_Result_8 := Cast_Expr_7;
Cast_Expr_8 := Local_Self_Type; 
      Cast_Result_9 := Cast_Expr_8;
--# property-call-start AdaNode.create_binop_assoc
Fld_7 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_2, Lhs => Cast_Result_7, Rhs => Cast_Result_8, Ret => Cast_Result_9);
--# end
--# expr-done 13166
--# expr-start 13168 '<FieldAccess for create_binop_assoc at nodes.lkt:18095:12>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18095'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13167 '<Symbol "/" at nodes.lkt:18096:13>' Sym_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18096'
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_6); 
--# expr-done 13167
Cast_Expr_9 := Local_Self_Type; 
      Cast_Result_10 := Cast_Expr_9;
Cast_Expr_10 := Local_Self_Type; 
      Cast_Result_11 := Cast_Expr_10;
Cast_Expr_11 := Local_Self_Type; 
      Cast_Result_12 := Cast_Expr_11;
--# property-call-start AdaNode.create_binop_assoc
Fld_8 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_3, Lhs => Cast_Result_10, Rhs => Cast_Result_11, Ret => Cast_Result_12);
--# end
--# expr-done 13168
--# expr-start 13170 '<FieldAccess for create_binop_assoc at nodes.lkt:18097:12>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18097'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13169 '<Symbol "mod" at nodes.lkt:18098:13>' Sym_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18098'
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Mod); 
--# expr-done 13169
Cast_Expr_12 := Local_Self_Type; 
      Cast_Result_13 := Cast_Expr_12;
Cast_Expr_13 := Local_Self_Type; 
      Cast_Result_14 := Cast_Expr_13;
Cast_Expr_14 := Local_Self_Type; 
      Cast_Result_15 := Cast_Expr_14;
--# property-call-start AdaNode.create_binop_assoc
Fld_9 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_4, Lhs => Cast_Result_13, Rhs => Cast_Result_14, Ret => Cast_Result_15);
--# end
--# expr-done 13170
--# expr-start 13172 '<FieldAccess for create_binop_assoc at nodes.lkt:18099:12>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18099'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13171 '<Symbol "rem" at nodes.lkt:18100:13>' Sym_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18100'
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Rem); 
--# expr-done 13171
Cast_Expr_15 := Local_Self_Type; 
      Cast_Result_16 := Cast_Expr_15;
Cast_Expr_16 := Local_Self_Type; 
      Cast_Result_17 := Cast_Expr_16;
Cast_Expr_17 := Local_Self_Type; 
      Cast_Result_18 := Cast_Expr_17;
--# property-call-start AdaNode.create_binop_assoc
Fld_10 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_5, Lhs => Cast_Result_16, Rhs => Cast_Result_17, Ret => Cast_Result_18);
--# end
--# expr-done 13172
--# expr-start 13174 '<FieldAccess for create_binop_assoc at nodes.lkt:18101:12>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18101'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13173 '<Symbol "**" at nodes.lkt:18102:13>' Sym_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18102'
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_3); 
--# expr-done 13173
Cast_Expr_18 := Local_Self_Type; 
      Cast_Result_19 := Cast_Expr_18;
Cast_Expr_19 := Local_Self_Type; 
      Cast_Result_20 := Cast_Expr_19;
--# property-call-start AdaNode.create_binop_assoc
Fld_11 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_6, Lhs => Cast_Result_19, Rhs => Local_Int_Type, Ret => Cast_Result_20);
--# end
--# expr-done 13174
--# expr-start 13176 '<FieldAccess for create_binop_assoc at nodes.lkt:18103:12>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18103'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13175 '<Symbol "<" at nodes.lkt:18104:13>' Sym_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18104'
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_8); 
--# expr-done 13175
Cast_Expr_20 := Local_Self_Type; 
      Cast_Result_21 := Cast_Expr_20;
Cast_Expr_21 := Local_Self_Type; 
      Cast_Result_22 := Cast_Expr_21;
--# property-call-start AdaNode.create_binop_assoc
Fld_12 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_7, Lhs => Cast_Result_21, Rhs => Cast_Result_22, Ret => Local_Bool_Type);
--# end
--# expr-done 13176
--# expr-start 13178 '<FieldAccess for create_binop_assoc at nodes.lkt:18105:12>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18105'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13177 '<Symbol "<=" at nodes.lkt:18106:13>' Sym_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18106'
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_9); 
--# expr-done 13177
Cast_Expr_22 := Local_Self_Type; 
      Cast_Result_23 := Cast_Expr_22;
Cast_Expr_23 := Local_Self_Type; 
      Cast_Result_24 := Cast_Expr_23;
--# property-call-start AdaNode.create_binop_assoc
Fld_13 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_8, Lhs => Cast_Result_23, Rhs => Cast_Result_24, Ret => Local_Bool_Type);
--# end
--# expr-done 13178
--# expr-start 13180 '<FieldAccess for create_binop_assoc at nodes.lkt:18107:12>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18107'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13179 '<Symbol "=" at nodes.lkt:18108:13>' Sym_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18108'
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_10); 
--# expr-done 13179
Cast_Expr_24 := Local_Self_Type; 
      Cast_Result_25 := Cast_Expr_24;
Cast_Expr_25 := Local_Self_Type; 
      Cast_Result_26 := Cast_Expr_25;
--# property-call-start AdaNode.create_binop_assoc
Fld_14 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_9, Lhs => Cast_Result_25, Rhs => Cast_Result_26, Ret => Local_Bool_Type);
--# end
--# expr-done 13180
--# expr-start 13182 '<FieldAccess for create_binop_assoc at nodes.lkt:18109:12>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18109'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13181 '<Symbol "/=" at nodes.lkt:18110:13>' Sym_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18110'
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_7); 
--# expr-done 13181
Cast_Expr_26 := Local_Self_Type; 
      Cast_Result_27 := Cast_Expr_26;
Cast_Expr_27 := Local_Self_Type; 
      Cast_Result_28 := Cast_Expr_27;
--# property-call-start AdaNode.create_binop_assoc
Fld_15 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_10, Lhs => Cast_Result_27, Rhs => Cast_Result_28, Ret => Local_Bool_Type);
--# end
--# expr-done 13182
--# expr-start 13184 '<FieldAccess for create_binop_assoc at nodes.lkt:18111:12>' Fld_16 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18111'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13183 '<Symbol ">" at nodes.lkt:18112:13>' Sym_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18112'
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_11); 
--# expr-done 13183
Cast_Expr_28 := Local_Self_Type; 
      Cast_Result_29 := Cast_Expr_28;
Cast_Expr_29 := Local_Self_Type; 
      Cast_Result_30 := Cast_Expr_29;
--# property-call-start AdaNode.create_binop_assoc
Fld_16 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_11, Lhs => Cast_Result_29, Rhs => Cast_Result_30, Ret => Local_Bool_Type);
--# end
--# expr-done 13184
--# expr-start 13186 '<FieldAccess for create_binop_assoc at nodes.lkt:18113:12>' Fld_17 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18113'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13185 '<Symbol ">=" at nodes.lkt:18114:13>' Sym_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18114'
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_12); 
--# expr-done 13185
Cast_Expr_30 := Local_Self_Type; 
      Cast_Result_31 := Cast_Expr_30;
Cast_Expr_31 := Local_Self_Type; 
      Cast_Result_32 := Cast_Expr_31;
--# property-call-start AdaNode.create_binop_assoc
Fld_17 := Libadalang.Implementation.Ada_Node_P_Create_Binop_Assoc (Node => Self, Op => Sym_12, Lhs => Cast_Result_31, Rhs => Cast_Result_32, Ret => Local_Bool_Type);
--# end
--# expr-done 13186
--# expr-start 13188 '<FieldAccess for create_unop_assoc at nodes.lkt:18115:12>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13187 '<Symbol "+" at nodes.lkt:18115:35>' Sym_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_4); 
--# expr-done 13187
Cast_Expr_32 := Local_Self_Type; 
      Cast_Result_33 := Cast_Expr_32;
Cast_Expr_33 := Local_Self_Type; 
      Cast_Result_34 := Cast_Expr_33;
--# property-call-start AdaNode.create_unop_assoc
Fld_18 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_13, Rhs => Cast_Result_33, Ret => Cast_Result_34);
--# end
--# expr-done 13188
--# expr-start 13190 '<FieldAccess for create_unop_assoc at nodes.lkt:18115:68>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13189 '<Symbol "-" at nodes.lkt:18115:91>' Sym_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_5); 
--# expr-done 13189
Cast_Expr_34 := Local_Self_Type; 
      Cast_Result_35 := Cast_Expr_34;
Cast_Expr_35 := Local_Self_Type; 
      Cast_Result_36 := Cast_Expr_35;
--# property-call-start AdaNode.create_unop_assoc
Fld_19 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_14, Rhs => Cast_Result_35, Ret => Cast_Result_36);
--# end
--# expr-done 13190
--# expr-start 13192 '<FieldAccess for create_unop_assoc at nodes.lkt:18115:124>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13191 '<Symbol "abs" at nodes.lkt:18115:147>' Sym_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18115'
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Abs); 
--# expr-done 13191
Cast_Expr_36 := Local_Self_Type; 
      Cast_Result_37 := Cast_Expr_36;
Cast_Expr_37 := Local_Self_Type; 
      Cast_Result_38 := Cast_Expr_37;
--# property-call-start AdaNode.create_unop_assoc
Fld_20 := Libadalang.Implementation.Ada_Node_P_Create_Unop_Assoc (Node => Self, Op => Sym_15, Rhs => Cast_Result_37, Ret => Cast_Result_38);
--# end
--# expr-done 13192
Array_Lit := Create_Internal_Env_Assoc_Array (Internal_Internal_Env_Assoc_Array'(1 => Fld_5, 2 => Fld_6, 3 => Fld_7, 4 => Fld_8, 5 => Fld_9, 6 => Fld_10, 7 => Fld_11, 8 => Fld_12, 9 => Fld_13, 10 => Fld_14, 11 => Fld_15, 12 => Fld_16, 13 => Fld_17, 14 => Fld_18, 15 => Fld_19, 16 => Fld_20)); 
--# expr-done 13193
Local_Defaults := Array_Lit; Inc_Ref (Local_Defaults);
--# bind defaults Local_Defaults
Let_Result := Local_Defaults; Inc_Ref (Let_Result);
--# end
--# expr-done 13194
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 13195
Let_Result_2 := Let_Result_1; Inc_Ref (Let_Result_2);
--# end
--# expr-done 13196
Let_Result_3 := Let_Result_2; Inc_Ref (Let_Result_3);
--# end
--# expr-done 13197
         Property_Result := Let_Result_3;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1575;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1575;
               if Self /= null then
                     if not Self.Unit.Context.In_Populate_Lexical_Env then
                        Add_Memoized_Error
                          (Self.Unit, Mmz_Handle, Exc, Mmz_Stored);
                     end if;
               end if;
            raise;
      end;
      if Self /= null then
         if not Self.Unit.Context.In_Populate_Lexical_Env then
            Mmz_Val := (Kind => Mmz_Internal_Env_Assoc_Array_Access,
                        As_Internal_Env_Assoc_Array_Access => Property_Result);
            Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
               if Mmz_Stored then
                  Inc_Ref (Property_Result);
               end if;
         end if;
      end if;
   return Property_Result;
end Signed_Int_Type_Def_P_Predefined_Operators;
--# end
--# property-start SignedIntTypeDef.is_static 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18121'
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Static
  (Node : Bare_Signed_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def  := Bare_Signed_Int_Type_Def (Node);
     Ent : Internal_Entity_Signed_Int_Type_Def :=
       Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13200 '<FieldAccess for is_static_expr at nodes.lkt:18121:29>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18121'
--# expr-start 13199 '<FieldAccess for range at nodes.lkt:18121:29>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18121'
--# expr-start 13198 '<FieldAccess for range at nodes.lkt:18121:29>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18121'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 13198
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 13199
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 13200
         Property_Result := Fld_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Signed_Int_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Type_Expr
   --
--# property-start TypeExpr.array_ndims 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18133'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Array_Ndims
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Integer;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Integer;
Dyn_Var_Bind_Result : Integer;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13204 '<bind at nodes.lkt:18134:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18134'
--# expr-start 13201 '<FieldAccess for origin_node at nodes.lkt:18134:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18134'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 13201
Bound_Origin := Fld; 
--# expr-start 13203 '<FieldAccess for array_ndims at nodes.lkt:18136:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18136'
--# expr-start 13202 '<FieldAccess for designated_type at nodes.lkt:18136:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18136'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13202
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 13203
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 13204
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Array_Ndims;
--# end
--# property-start TypeExpr.type_name 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18141'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Type_Name
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr_0 : Internal_Entity_Subtype_Indication;
Cast_Expr : Internal_Entity_Type_Expr;
Cast_Result : Internal_Entity_Subtype_Indication;
Fld : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 13205 '<Cast to SubtypeIndication at nodes.lkt:18142:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18142'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Subtype_Indication;
   end if;
--# expr-done 13205
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Subtype_Indication then
   --# scope-start
   --# bind sti Var_Expr_0
--# expr-start 13206 '<FieldAccess for name at nodes.lkt:18142:48>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18142'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Var_Expr_0.Node.Subtype_Indication_F_Name, Info => Var_Expr_0.Info);
--# expr-done 13206
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Type_Name;
--# end
--# property-start TypeExpr.defining_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18145'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Defining_Env
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
      ; Include_Ud_Indexing : Boolean
      ; Dottable_Type : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   --# bind include_ud_indexing Include_Ud_Indexing
   --# bind dottable_type Dottable_Type
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1579 with Inline_Always;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1579 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1579;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13208 '<FieldAccess for defining_env at nodes.lkt:18146:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18146'
--# expr-start 13207 '<FieldAccess for designated_type at nodes.lkt:18146:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18146'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 13207
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, Include_Ud_Indexing => Include_Ud_Indexing, Dottable_Type => Dottable_Type, E_Info => Fld.Info);
--# end
--# expr-done 13208
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1579;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1579;
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Defining_Env;
--# end
--# property-start '[dispatcher]TypeExpr.designated_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Expr_P_Designated_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
      --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Expr (Self.Kind) is
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.designated_type
                  Property_Result := Anonymous_Type_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.designated_type
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.designated_type
                  Property_Result := Subtype_Indication_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Synthetic_Type_Expr =>
                  --# property-call-start SyntheticTypeExpr.designated_type
                  Property_Result := Synthetic_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Expr_P_Designated_Type;
--# end
--# property-start TypeExpr.designated_type_decl 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18155'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 13211 '<bind at nodes.lkt:18156:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18156'
--# expr-start 13209 '<FieldAccess for origin_node at nodes.lkt:18156:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18156'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 13209
Bound_Origin := Fld; 
--# expr-start 13210 '<FieldAccess for designated_type at nodes.lkt:18158:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18158'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13210
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 13211
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Designated_Type_Decl;
--# end
--# property-start TypeExpr.designated_type_decl_from 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18164'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl_From
  (Node : Bare_Type_Expr
      ; Origin_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin_node Origin_Node
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 13215 '<bind at nodes.lkt:18165:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18165'
--# expr-start 13213 '<FieldAccess for origin_node at nodes.lkt:18165:23>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18165'
--# expr-start 13212 '<FieldAccess for node at nodes.lkt:18165:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18165'
Fld := Origin_Node.Node;
--# expr-done 13212
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld);
--# end
--# expr-done 13213
Bound_Origin := Fld_1; 
--# expr-start 13214 '<FieldAccess for designated_type at nodes.lkt:18167:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18167'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13214
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 13215
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Designated_Type_Decl_From;
--# end
--# property-start TypeExpr.element_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18173'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Element_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Local_D : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Fld_2 : Internal_Entity_Type_Def;
Is_A_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13223 '<Let at nodes.lkt:18174:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18174'
--# scope-start
--# expr-start 13216 '<FieldAccess for designated_type at nodes.lkt:18174:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18174'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 13216
Local_D := Fld; 
--# bind d Local_D
--# expr-start 13222 '<If at nodes.lkt:18176:9>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
--# expr-start 13217 '<IsA AnonymousTypeDecl at nodes.lkt:18176:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
Is_A := Local_D.Node /= null 
and then Local_D.Node.Kind in Ada_Anonymous_Type_Decl_Range; 
--# expr-done 13217
if Is_A then
   --# expr-start 13220 '<IsA AccessDef at nodes.lkt:18176:39>' Is_A_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
--# expr-start 13219 '<FieldAccess for type_def at nodes.lkt:18176:39>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
--# expr-start 13218 '<Cast to AnonymousTypeDecl at nodes.lkt:18176:39>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
Cast_Expr := Local_D; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 13218
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Type_Def (Node => Cast_Result.Node.Type_Decl_F_Type_Def, Info => Cast_Result.Info);
--# expr-done 13219
Is_A_1 := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_Def; 
--# expr-done 13220
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 13221 '<FieldAccess for accessed_type at nodes.lkt:18176:90>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18176'
      if Local_D.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Local_D.Node, Origin => Origin, E_Info => Local_D.Info);
--# end
--# expr-done 13221
   If_Result_1 := Fld_1;
else
   If_Result_1 := Local_D;
end if;
--# expr-done 13222
Let_Result := If_Result_1; 
--# end
--# expr-done 13223
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Element_Type;
--# end
--# property-start TypeExpr.canonical_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18181'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Canonical_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr_297 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 13224 '<FieldAccess for designated_type at nodes.lkt:18182:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18182'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 13224
Var_Expr_297 := Fld;
if Var_Expr_297 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 13225 '<FieldAccess for canonical_type at nodes.lkt:18182:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18182'
      if Var_Expr_297.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_297.Node, Origin => Origin, E_Info => Var_Expr_297.Info);
--# end
--# expr-done 13225
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
         Property_Result := Result_Var;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Canonical_Type;
--# end
--# property-start TypeExpr.subtype_constraint 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18188'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Subtype_Constraint
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Constraint
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Constraint;
      Var_Expr_0 : Internal_Entity_Subtype_Indication;
Var_Expr_1 : Internal_Entity_Subtype_Decl;
Var_Expr : Internal_Entity_Constraint;
Cast_Expr : Internal_Entity_Type_Expr;
Cast_Result : Internal_Entity_Subtype_Indication;
Fld : Internal_Entity_Constraint;
Scope_Result : Internal_Entity_Constraint;
Result_Var : Internal_Entity_Constraint;
Scope_Result_1 : Internal_Entity_Constraint;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Constraint;
Scope_Result_2 : Internal_Entity_Constraint;
Result_Var_1 : Internal_Entity_Constraint;
Result_Var_2 : Internal_Entity_Constraint;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 13226 '<Cast to SubtypeIndication at nodes.lkt:18189:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Subtype_Indication;
   end if;
--# expr-done 13226
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Subtype_Indication then
   --# scope-start
   --# bind si Var_Expr_0
--# expr-start 13227 '<FieldAccess for constraint at nodes.lkt:18189:47>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Var_Expr_0.Node.Subtype_Indication_F_Constraint, Info => Var_Expr_0.Info);
--# expr-done 13227
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Constraint;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 13229 '<Cast to SubtypeDecl at nodes.lkt:18189:66>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
--# expr-start 13228 '<FieldAccess for designated_type at nodes.lkt:18189:66>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 13228
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Subtype_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Subtype_Decl;
   end if;
--# expr-done 13229
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Subtype_Decl then
   --# scope-start
   --# bind st Var_Expr_1
--# expr-start 13231 '<FieldAccess for subtype_constraint at nodes.lkt:18189:116>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
--# expr-start 13230 '<FieldAccess for subtype at nodes.lkt:18189:116>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18189'
      if Var_Expr_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Var_Expr_1.Node.Subtype_Decl_F_Subtype, Info => Var_Expr_1.Info);
--# expr-done 13230
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.subtype_constraint
Fld_3 := Libadalang.Implementation.Type_Expr_P_Subtype_Constraint (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 13231
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Entity_Constraint;
end if;
   Result_Var_2 := Result_Var_1;
end if;
         Property_Result := Result_Var_2;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Subtype_Constraint;
--# end
--# property-start TypeExpr.discriminant_constraints 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18195'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Discriminant_Constraints
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1586 with Inline_Always;
            procedure Finalizer_Scope_3619 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Composite_Constraint;
Fld : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Composite_Constraint;
Fld_1 : Internal_Param_Actual_Array_Access;
Scope_Result : Internal_Param_Actual_Array_Access;
Result_Var : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_1586 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1586;
            procedure Finalizer_Scope_3619 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3619;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
--# expr-start 13233 '<Cast to CompositeConstraint at nodes.lkt:18196:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18196'
--# expr-start 13232 '<FieldAccess for subtype_constraint at nodes.lkt:18196:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18196'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.subtype_constraint
Fld := Libadalang.Implementation.Type_Expr_P_Subtype_Constraint (Node => Ent.Node, Origin => No_Bare_Ada_Node, E_Info => Ent.Info);
--# end
--# expr-done 13232
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Composite_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Composite_Constraint
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Composite_Constraint;
   end if;
--# expr-done 13233
Var_Expr_0 := Cast_Result;
if Var_Expr_0 /= No_Entity_Composite_Constraint then
   --# scope-start
   --# bind cc Var_Expr_0
--# expr-start 13234 '<FieldAccess for discriminant_params at nodes.lkt:18196:70>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18196'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompositeConstraint.discriminant_params
Fld_1 := Libadalang.Implementation.Composite_Constraint_P_Discriminant_Params (Node => Var_Expr_0.Node, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 13234
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3619;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var);
         Property_Result := Result_Var;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1586;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1586;
                     Finalizer_Scope_3619;
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Discriminant_Constraints;
--# end
--# property-start '[dispatcher]TypeExpr.is_definite_subtype' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Expr_P_Is_Definite_Subtype
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Type_Expr (Self.Kind) is
               when Ada_Anonymous_Type .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr =>
                  --# property-call-start TypeExpr.is_definite_subtype
                  Property_Result := Type_Expr_P_Is_Definite_Subtype
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.is_definite_subtype
                  Property_Result := Subtype_Indication_P_Is_Definite_Subtype
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Type_Expr_P_Is_Definite_Subtype;
--# end
--# property-start TypeExpr.custom_id_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18203'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Custom_Id_Text
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1588 with Inline_Always;
      Cast_Expr : Bare_Type_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : String_Type;
Dyn_Var_Bind_Result : String_Type;
            procedure Finalizer_Scope_1588 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1588;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13237 '<bind at nodes.lkt:18204:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18204'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 13236 '<FieldAccess for canonical_fully_qualified_name at nodes.lkt:18205:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18205'
--# expr-start 13235 '<FieldAccess for designated_type at nodes.lkt:18205:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18205'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13235
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 13236
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 13237
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1588;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1588;
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Custom_Id_Text;
--# end
--# property-start TypeExpr.is_definite_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18200'
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Is_Definite_Subtype
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Expr  := Bare_Type_Expr (Node);
     Ent : Internal_Entity_Type_Expr :=
       Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13239 '<FieldAccess for is_definite_subtype at nodes.lkt:18201:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18201'
--# expr-start 13238 '<FieldAccess for designated_type_decl at nodes.lkt:18201:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18201'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 13238
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld.Node, Origin => No_Bare_Ada_Node, E_Info => Fld.Info);
--# end
--# expr-done 13239
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Type_Expr_P_Is_Definite_Subtype;
--# end
   --
   --  Primitives for Bare_Anonymous_Type
   --
      procedure Initialize_Fields_For_Anonymous_Type
        (Self : Bare_Anonymous_Type
         ; Anonymous_Type_F_Type_Decl : Bare_Anonymous_Type_Decl
        ) is
      begin
            Self.Anonymous_Type_F_Type_Decl := Anonymous_Type_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type;
   function Anonymous_Type_F_Type_Decl
     (Node : Bare_Anonymous_Type) return Bare_Anonymous_Type_Decl
   is
   begin
         return Node.Anonymous_Type_F_Type_Decl;
   end;
--# property-start AnonymousType.designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18214'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Designated_Type
  (Node : Bare_Anonymous_Type
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 13240 '<FieldAccess for type_decl at nodes.lkt:18214:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18214'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 13240
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
         Property_Result := Cast_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Designated_Type;
--# end
--# property-start AnonymousType.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18217'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Xref_Equation
  (Node : Bare_Anonymous_Type
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1590 with Inline_Always;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1590 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1590;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13242 '<FieldAccess for sub_equation at nodes.lkt:18217:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18217'
--# expr-start 13241 '<FieldAccess for type_decl at nodes.lkt:18217:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18217'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 13241
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 13242
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1590;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1590;
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Xref_Equation;
--# end
--# property-start AnonymousType.custom_id_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18224'
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Custom_Id_Text
  (Node : Bare_Anonymous_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Anonymous_Type  := Bare_Anonymous_Type (Node);
     Ent : Internal_Entity_Anonymous_Type :=
       Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1591 with Inline_Always;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : String_Type;
            procedure Finalizer_Scope_1591 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1591;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13244 '<FieldAccess for text at nodes.lkt:18224:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18224'
--# expr-start 13243 '<FieldAccess for type_decl at nodes.lkt:18224:36>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18224'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 13243
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Fld.Node);
--# expr-done 13244
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1591;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1591;
            raise;
      end;
   return Property_Result;
end Anonymous_Type_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Enum_Lit_Synth_Type_Expr
   --
--# property-start EnumLitSynthTypeExpr.designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18231'
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Designated_Type
  (Node : Bare_Enum_Lit_Synth_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Enum_Lit_Synth_Type_Expr  := Bare_Enum_Lit_Synth_Type_Expr (Node);
     Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
       Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
--# expr-start 13247 '<FieldAccess for enum_type at nodes.lkt:18232:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18232'
--# expr-start 13246 '<Cast to EnumLiteralDecl at nodes.lkt:18232:9>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18232'
--# expr-start 13245 '<parent at nodes.lkt:18232:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18232'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13245
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 13246
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]EnumLiteralDecl.enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Enum_Literal_Decl_P_Enum_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 13247
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
         Property_Result := Cast_Result_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Enum_Lit_Synth_Type_Expr_P_Designated_Type;
--# end
--# property-start EnumLitSynthTypeExpr.custom_id_text 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18234'
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
  (Node : Bare_Enum_Lit_Synth_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return String_Type
is
   Self : Bare_Enum_Lit_Synth_Type_Expr  := Bare_Enum_Lit_Synth_Type_Expr (Node);
     Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
       Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : String_Type;
            procedure Finalizer_Scope_1593 with Inline_Always;
      Cast_Expr : Bare_Enum_Lit_Synth_Type_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : String_Type;
Str : String_Type;
Concat_Result : String_Type;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type_Array_Access;
Str_1 : String_Type;
Fld_5 : String_Type;
Concat_Result_1 : String_Type;
Dyn_Var_Bind_Result : String_Type;
            procedure Finalizer_Scope_1593 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Str);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Str_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1593;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13257 '<bind at nodes.lkt:18235:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18235'
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 13249 '<FieldAccess for canonical_fully_qualified_name at nodes.lkt:18237:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18237'
--# expr-start 13248 '<FieldAccess for designated_type at nodes.lkt:18237:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18237'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13248
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 13249
--# expr-start 13250 '<String at nodes.lkt:18237:67>' Str 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18237'
Str := Create_String ("."); 
--# expr-done 13250
Concat_Result := Concat_String (Fld_1, Str); 
--# expr-start 13256 '<FieldAccess for sym_join at nodes.lkt:18239:9>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18239'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13254 '<FieldAccess for as_symbol_array at nodes.lkt:18240:13>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18240'
--# expr-start 13253 '<FieldAccess for defining_name at nodes.lkt:18240:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18240'
--# expr-start 13252 '<Cast to EnumLiteralDecl at nodes.lkt:18240:13>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18240'
--# expr-start 13251 '<parent at nodes.lkt:18240:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18240'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13251
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 13252
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 13253
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.as_symbol_array'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_As_Symbol_Array (Node => Fld_3.Node);
--# end
--# expr-done 13254
--# expr-start 13255 '<String at nodes.lkt:18240:80>' Str_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18240'
Str_1 := Create_String (""); 
--# expr-done 13255
--# property-call-start AdaNode.sym_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Ent.Node, Syms => Fld_4, Sep => Str_1);
--# end
--# expr-done 13256
Concat_Result_1 := Concat_String (Concat_Result, Fld_5); 
Dyn_Var_Bind_Result := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 13257
         Property_Result := Dyn_Var_Bind_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1593;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1593;
            raise;
      end;
   return Property_Result;
end Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Subtype_Indication
   --
      procedure Initialize_Fields_For_Subtype_Indication
        (Self : Bare_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Self.Subtype_Indication_F_Has_Not_Null := Subtype_Indication_F_Has_Not_Null;
            Self.Subtype_Indication_F_Name := Subtype_Indication_F_Name;
            Self.Subtype_Indication_F_Constraint := Subtype_Indication_F_Constraint;
      end Initialize_Fields_For_Subtype_Indication;
   function Subtype_Indication_F_Has_Not_Null
     (Node : Bare_Subtype_Indication) return Bare_Not_Null
   is
   begin
         return Node.Subtype_Indication_F_Has_Not_Null;
   end;
   function Subtype_Indication_F_Name
     (Node : Bare_Subtype_Indication) return Bare_Name
   is
   begin
         return Node.Subtype_Indication_F_Name;
   end;
   function Subtype_Indication_F_Constraint
     (Node : Bare_Subtype_Indication) return Bare_Constraint
   is
   begin
         return Node.Subtype_Indication_F_Constraint;
   end;
--# property-start SubtypeIndication.designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18256'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Designated_Type
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1594 with Inline_Always;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1594 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1594;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13261 '<bind at nodes.lkt:18257:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18257'
--# expr-start 13258 '<FieldAccess for node_env at nodes.lkt:18257:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18257'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13258
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 13260 '<FieldAccess for designated_type_impl at nodes.lkt:18259:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18259'
--# expr-start 13259 '<FieldAccess for name at nodes.lkt:18259:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18259'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 13259
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 13260
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 13261
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_1594;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1594;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Designated_Type;
--# end
--# property-start SubtypeIndication.is_definite_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18262'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Is_Definite_Subtype
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Constraint;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
--# expr-start 13264 '<Not at nodes.lkt:18263:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18263'
--# expr-start 13263 '<IsNull at nodes.lkt:18263:13>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18263'
--# expr-start 13262 '<FieldAccess for constraint at nodes.lkt:18263:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18263'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 13262
Is_Null := Fld.Node = null; 
--# expr-done 13263
Not_Val := not (Is_Null); 
--# expr-done 13264
if Not_Val then
   If_Result := True;
else
   --# expr-start 13266 '<FieldAccess for is_definite_subtype at nodes.lkt:18263:40>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18263'
--# expr-start 13265 '<FieldAccess for designated_type_decl at nodes.lkt:18263:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18263'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_1 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 13265
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_1.Node, Origin => No_Bare_Ada_Node, E_Info => Fld_1.Info);
--# end
--# expr-done 13266
   If_Result := Fld_2;
end if;
         Property_Result := If_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Is_Definite_Subtype;
--# end
--# property-start SubtypeIndication.complete_items 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18270'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Complete_Items
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Array_Access
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1596 with Inline_Always;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Package_Decl;
Match_2 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity;
Is_A : Boolean;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Boolean;
If_Result_1 : Integer;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Integer;
Scope_Result : Integer;
Cast_Result_3 : Internal_Entity_Package_Decl;
Let_Result_1 : Integer;
Scope_Result_1 : Integer;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Integer;
Scope_Result_2 : Integer;
Match_Result : Integer;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
            procedure Finalizer_Scope_1596 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1596;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13282 '<Map at nodes.lkt:18270:51>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18270'
   --# expr-start 13268 '<EnvGet at nodes.lkt:18270:51>' Env_Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18270'
--# expr-start 13267 '<FieldAccess for children_env at nodes.lkt:18270:51>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18270'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13267
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => Thin (No_Symbol), Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 13268
      declare
         Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_0
      --# expr-start 13281 '<New CompletionItem at nodes.lkt:18271:16>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18271'
--# expr-start 13269 '<Cast to BasicDecl at nodes.lkt:18272:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18272'
Cast_Expr := Item_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 13269
--# expr-start 13272 '<FieldAccess for dottable_subp at nodes.lkt:18273:25>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18273'
--# expr-start 13271 '<FieldAccess for md at nodes.lkt:18273:25>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18273'
--# expr-start 13270 '<FieldAccess for info at nodes.lkt:18273:25>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18273'
Fld_1 := Item_0.Info;
--# expr-done 13270
Fld_2 := Fld_1.Md;
--# expr-done 13271
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 13272
--# expr-start 13273 '<FieldAccess for has_visibility at nodes.lkt:18274:24>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18274'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.has_visibility
Fld_4 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_0);
--# end
--# expr-done 13273
--# expr-start 13280 '<Match at nodes.lkt:18275:20>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18275'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_0; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Formal_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind btd Match_0
--# expr-start 13279 '<If at nodes.lkt:18282:17>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
--# expr-start 13275 '<IsA DerivedTypeDef at nodes.lkt:18282:20>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
--# expr-start 13274 '<parent at nodes.lkt:18282:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13274
Is_A := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Derived_Type_Def_Range; 
--# expr-done 13275
if Is_A then
   --# expr-start 13278 '<Eq at nodes.lkt:18282:54>' Is_Equiv 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
--# expr-start 13277 '<parent at nodes.lkt:18282:54>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
--# expr-start 13276 '<parent at nodes.lkt:18282:54>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18282'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13276
      if Fld_6.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 13277
Cast_Expr_1 := Match_0; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Fld_7, Cast_Result_1); 
--# expr-done 13278
   If_Result := Is_Equiv;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := 0;
else
   If_Result_1 := 100;
end if;
--# expr-done 13279
Let_Result := If_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
Let_Result_1 := 50; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Abstract_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := 0; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 13280
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_4, Weight => Match_Result); 
--# expr-done 13281
         declare
            Item_To_Append : constant Internal_Completion_Item := New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Completion_Item_Array
           (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13282
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1596;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1596;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Complete_Items;
--# end
--# property-start SubtypeIndication.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18290'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Xref_Equation
  (Node : Bare_Subtype_Indication
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1597 with Inline_Always;
            procedure Finalizer_Scope_3624 with Inline_Always;
      Var_Expr_0 : Internal_Entity_Constraint;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Constraint;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1597 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1597;
            procedure Finalizer_Scope_3624 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3624;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13287 '<And at nodes.lkt:18293:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18293'
--# expr-start 13284 '<FieldAccess for subtype_indication_equation at nodes.lkt:18293:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18293'
--# expr-start 13283 '<FieldAccess for name at nodes.lkt:18293:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18293'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 13283
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 13284
--# expr-start 13285 '<FieldAccess for constraint at nodes.lkt:18293:53>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18293'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 13285
Var_Expr_0 := Fld_2;
if Var_Expr_0 /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 13286 '<FieldAccess for sub_equation at nodes.lkt:18294:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18294'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 13286
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3624;
   Result_Var := Scope_Result;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18294:50") else null)); 
   Result_Var := True_Rel;
end if;
      Inc_Ref (Result_Var);
And_Pred := Create_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18293:9") else null)); 
--# expr-done 13287
         Property_Result := And_Pred;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1597;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1597;
                     Finalizer_Scope_3624;
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Xref_Equation;
--# end
--# property-start SubtypeIndication.discrete_range 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18297'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Discrete_Range
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Local_Rc : Internal_Entity_Range_Constraint;
Var_Expr_0 : Internal_Entity_Range_Constraint;
Fld : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Range_Constraint;
Fld_1 : Internal_Entity_Range_Spec;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13295 '<Let at nodes.lkt:18298:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18298'
--# scope-start
--# expr-start 13289 '<Cast to RangeConstraint at nodes.lkt:18298:18>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18298'
--# expr-start 13288 '<FieldAccess for constraint at nodes.lkt:18298:18>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18298'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 13288
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Range_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Range_Constraint
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 13289
Local_Rc := Cast_Result; 
--# bind rc Local_Rc
Var_Expr_0 := Local_Rc;
if Var_Expr_0 /= No_Entity_Range_Constraint then
   --# scope-start
   --# bind r Var_Expr_0
--# expr-start 13292 '<FieldAccess for discrete_range at nodes.lkt:18301:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18301'
--# expr-start 13291 '<FieldAccess for range at nodes.lkt:18301:20>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18301'
--# expr-start 13290 '<FieldAccess for range at nodes.lkt:18301:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18301'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Range_Spec (Node => Var_Expr_0.Node.Range_Constraint_F_Range, Info => Var_Expr_0.Info);
--# expr-done 13290
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Fld_1.Node.Range_Spec_F_Range, Info => Fld_1.Info);
--# expr-done 13291
      if Fld_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.discrete_range
Fld_3 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 13292
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 13294 '<FieldAccess for discrete_range at nodes.lkt:18304:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18304'
--# expr-start 13293 '<FieldAccess for designated_type_decl at nodes.lkt:18304:25>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18304'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 13293
      if Fld_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 13294
   Result_Var := Fld_5;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 13295
         Property_Result := Let_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Discrete_Range;
--# end
--# property-start SubtypeIndication.is_static_subtype 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18312'
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Is_Static_Subtype
  (Node : Bare_Subtype_Indication
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Indication  := Bare_Subtype_Indication (Node);
     Ent : Internal_Entity_Subtype_Indication :=
       Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr_0 : Internal_Entity_Constraint;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Sym : Symbol_Type;
Fld_2 : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Constraint;
Fld_4 : Boolean;
Scope_Result : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env_For_Unit (Self);
      end if;
      begin
   --# scope-start
         --# expr-start 13306 '<bind at nodes.lkt:18313:9>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18313'
--# expr-start 13296 '<FieldAccess for origin_node at nodes.lkt:18313:23>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18313'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 13296
Bound_Origin := Fld; 
--# expr-start 13301 '<Not at nodes.lkt:18315:9>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18315'
--# expr-start 13300 '<FieldAccess for has_aspect at nodes.lkt:18315:13>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18315'
--# expr-start 13298 '<Cast to BasicDecl at nodes.lkt:18315:13>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18315'
--# expr-start 13297 '<FieldAccess for semantic_parent at nodes.lkt:18315:13>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18315'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 13297
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 13298
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13299 '<Symbol Dynamic_Predicate at nodes.lkt:18315:61>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18315'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Dynamic_Predicate); 
--# expr-done 13299
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Cast_Result.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 13300
Not_Val := not (Fld_2); 
--# expr-done 13301
if Not_Val then
--# expr-start 13302 '<FieldAccess for constraint at nodes.lkt:18316:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18316'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 13302
Var_Expr_0 := Fld_3;
if Var_Expr_0 /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr_0
--# expr-start 13303 '<FieldAccess for is_static at nodes.lkt:18317:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18317'
      if Var_Expr_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Constraint.is_static
Fld_4 := Libadalang.Implementation.Constraint_P_Is_Static (Node => Var_Expr_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_0.Info);
--# end
--# expr-done 13303
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 13305 '<FieldAccess for is_static_decl at nodes.lkt:18318:25>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18318'
--# expr-start 13304 '<FieldAccess for designated_type at nodes.lkt:18318:25>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18318'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 13304
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 13305
   Result_Var := Fld_6;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
Dyn_Var_Bind_Result := If_Result; 
--# expr-done 13306
         Property_Result := Dyn_Var_Bind_Result;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Subtype_Indication_P_Is_Static_Subtype;
--# end
   --
   --  Primitives for Bare_Constrained_Subtype_Indication
   --
      procedure Initialize_Fields_For_Constrained_Subtype_Indication
        (Self : Bare_Constrained_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Constrained_Subtype_Indication;
   --
   --  Primitives for Bare_Discrete_Subtype_Indication
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Indication
        (Self : Bare_Discrete_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Discrete_Subtype_Indication;
   --
   --  Primitives for Bare_Synthetic_Type_Expr
   --
      procedure Initialize_Fields_For_Synthetic_Type_Expr
        (Self : Bare_Synthetic_Type_Expr
         ; Synthetic_Type_Expr_F_Target_Type : Bare_Base_Type_Decl
        ) is
      begin
            Self.Synthetic_Type_Expr_F_Target_Type := Synthetic_Type_Expr_F_Target_Type;
      end Initialize_Fields_For_Synthetic_Type_Expr;
   function Synthetic_Type_Expr_F_Target_Type
     (Node : Bare_Synthetic_Type_Expr) return Bare_Base_Type_Decl
   is
   begin
         return Node.Synthetic_Type_Expr_F_Target_Type;
   end;
--# property-start SyntheticTypeExpr.designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18338'
pragma Warnings (Off, "is not referenced");
 function Synthetic_Type_Expr_P_Designated_Type
  (Node : Bare_Synthetic_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Synthetic_Type_Expr  := Bare_Synthetic_Type_Expr (Node);
     Ent : Internal_Entity_Synthetic_Type_Expr :=
       Internal_Entity_Synthetic_Type_Expr'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1600 with Inline_Always;
      Local_Relevant_Rebindings : Env_Rebindings;
Fld : Internal_Entity_Info;
Fld_1 : Bare_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Shed_Entity_Info : Internal_Entity_Info;
Fld_3 : Env_Rebindings;
Fld_4 : Bare_Base_Type_Decl;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1600 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1600;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13317 '<Let at nodes.lkt:18347:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
--# scope-start
--# expr-start 13311 '<FieldAccess for rebindings at nodes.lkt:18347:35>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
--# expr-start 13310 '<shed_rebindings at nodes.lkt:18347:35>' Shed_Entity_Info 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
--# expr-start 13307 '<FieldAccess for info at nodes.lkt:18347:83>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
Fld := Ent.Info;
--# expr-done 13307
--# expr-start 13309 '<FieldAccess for children_env at nodes.lkt:18347:35>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
--# expr-start 13308 '<FieldAccess for target_type at nodes.lkt:18347:35>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18347'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Self.Synthetic_Type_Expr_F_Target_Type;
--# expr-done 13308
      if Fld_1 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 13309
Shed_Entity_Info := AST_Envs.Shed_Rebindings (Fld, Fld_2); 
--# expr-done 13310
Fld_3 := Shed_Entity_Info.Rebindings;
--# expr-done 13311
Local_Relevant_Rebindings := Fld_3; 
--# bind relevant_rebindings Local_Relevant_Rebindings
--# expr-start 13316 '<New BaseTypeDecl.entity at nodes.lkt:18350:9>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18350'
--# expr-start 13314 '<New EntityInfo at nodes.lkt:18351:41>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18351'
--# expr-start 13313 '<FieldAccess for from_rebound at nodes.lkt:18352:81>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18352'
--# expr-start 13312 '<FieldAccess for info at nodes.lkt:18352:81>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18352'
Fld_5 := Ent.Info;
--# expr-done 13312
Fld_6 := Fld_5.From_Rebound;
--# expr-done 13313
New_Struct := (From_Rebound => Fld_6, Md => No_Metadata, Rebindings => Local_Relevant_Rebindings); 
--# expr-done 13314
--# expr-start 13315 '<FieldAccess for target_type at nodes.lkt:18351:18>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18351'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Self.Synthetic_Type_Expr_F_Target_Type;
--# expr-done 13315
New_Struct_1 := (Info => New_Struct, Node => Fld_4); 
--# expr-done 13316
Let_Result := New_Struct_1; 
--# end
--# expr-done 13317
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1600;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1600;
            raise;
      end;
   return Property_Result;
end Synthetic_Type_Expr_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Index
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Index
        (Self : Bare_Unconstrained_Array_Index
         ; Unconstrained_Array_Index_F_Subtype_Name : Bare_Name
         ; Unconstrained_Array_Index_F_Lower_Bound : Bare_Expr
        ) is
      begin
            Self.Unconstrained_Array_Index_F_Subtype_Name := Unconstrained_Array_Index_F_Subtype_Name;
            Self.Unconstrained_Array_Index_F_Lower_Bound := Unconstrained_Array_Index_F_Lower_Bound;
      end Initialize_Fields_For_Unconstrained_Array_Index;
   function Unconstrained_Array_Index_F_Subtype_Name
     (Node : Bare_Unconstrained_Array_Index) return Bare_Name
   is
   begin
         return Node.Unconstrained_Array_Index_F_Subtype_Name;
   end;
   function Unconstrained_Array_Index_F_Lower_Bound
     (Node : Bare_Unconstrained_Array_Index) return Bare_Expr
   is
   begin
         return Node.Unconstrained_Array_Index_F_Lower_Bound;
   end;
--# property-start UnconstrainedArrayIndex.designated_type 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18364'
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Index_P_Designated_Type
  (Node : Bare_Unconstrained_Array_Index
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Index  := Bare_Unconstrained_Array_Index (Node);
     Ent : Internal_Entity_Unconstrained_Array_Index :=
       Internal_Entity_Unconstrained_Array_Index'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13319 '<FieldAccess for name_designated_type at nodes.lkt:18365:9>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18365'
--# expr-start 13318 '<FieldAccess for subtype_name at nodes.lkt:18365:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18365'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Unconstrained_Array_Index_F_Subtype_Name, Info => Ent.Info);
--# expr-done 13318
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 13319
         Property_Result := Fld_1;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Unconstrained_Array_Index_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Until_Node
   --
--# property-start '[dispatcher]Until.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Until_Node_P_As_Bool
  (Node : Bare_Until_Node
  )
   return Boolean
is
   Self : Bare_Until_Node  := Bare_Until_Node (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_Until_Node (Self.Kind) is
               when Ada_Until_Absent =>
                  --# property-call-start Until.Absent.as_bool
                  Property_Result := Until_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Until_Present =>
                  --# property-call-start Until.Present.as_bool
                  Property_Result := Until_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_Until_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Absent
   --
--# property-start Until.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18370'
pragma Warnings (Off, "is not referenced");
 function Until_Absent_P_As_Bool
  (Node : Bare_Until_Absent
  )
   return Boolean
is
   Self : Bare_Until_Absent  := Bare_Until_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Until_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Present
   --
--# property-start Until.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18370'
pragma Warnings (Off, "is not referenced");
 function Until_Present_P_As_Bool
  (Node : Bare_Until_Present
  )
   return Boolean
is
   Self : Bare_Until_Present  := Bare_Until_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Until_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Use_Clause
   --
--# property-start UseClause.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18376'
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Xref_Entry_Point
  (Node : Bare_Use_Clause
  )
   return Boolean
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Xref_Entry_Point;
--# end
--# property-start UseClause.used_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18380'
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Used_Envs
  (Node : Bare_Use_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
     Ent : Internal_Entity_Use_Clause :=
       Internal_Entity_Use_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1606 with Inline_Always;
            procedure Finalizer_Scope_3627 with Inline_Always;
            procedure Finalizer_Scope_3628 with Inline_Always;
            procedure Finalizer_Scope_3629 with Inline_Always;
      Match_0 : Internal_Entity_Use_Package_Clause;
Match_1 : Internal_Entity_Use_Type_Clause;
Fld : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name_List;
Fld_2 : Bare_Name_List;
Fld_3 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_4 : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Use_Clause;
Cast_Result : Internal_Entity_Use_Package_Clause;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Use_Type_Clause;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1606 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1606;
            procedure Finalizer_Scope_3627 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_3627;
            procedure Finalizer_Scope_3628 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_3628;
            procedure Finalizer_Scope_3629 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3629;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13326 '<Match at nodes.lkt:18380:35>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18380'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Ent; 
case Ada_Use_Clause (Match_Prefix.Node.Kind) is
         when Ada_Use_Package_Clause =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Use_Package_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind upc Match_0
--# expr-start 13321 '<env_group at nodes.lkt:18381:39>' Group_Env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18381'
--# expr-start 13320 '<FieldAccess for designated_envs at nodes.lkt:18381:39>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18381'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 13320
Group_Env := Group (Fld, No_Metadata); 
--# expr-done 13321
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3627;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Use_Type_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind utc Match_1
--# expr-start 13325 '<env_group at nodes.lkt:18382:36>' Group_Env_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18382'
--# expr-start 13324 '<Map at nodes.lkt:18382:36>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18382'
   --# expr-start 13322 '<FieldAccess for types at nodes.lkt:18382:36>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18382'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name_List (Node => Match_1.Node.Use_Type_Clause_F_Types, Info => Match_1.Info);
--# expr-done 13322
Fld_2 := Fld_1.Node;
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld_2 = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_2;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind n Item_0
      --# expr-start 13323 '<FieldAccess for name_designated_type_env at nodes.lkt:18382:57>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18382'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type_env
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type_Env (Node => Item_0.Node, E_Info => Item_0.Info);
--# end
--# expr-done 13323
         declare
            Item_To_Append : constant Lexical_Env := Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3629;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13324
Group_Env_1 := Group (Map_Result, No_Metadata); 
--# expr-done 13325
Let_Result_1 := Group_Env_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3628;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 13326
         Property_Result := Match_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1606;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1606;
                     Finalizer_Scope_3627;
                     Finalizer_Scope_3628;
                     Finalizer_Scope_3629;
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Used_Envs;
--# end
--# property-start UseClause.initial_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18387'
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Initial_Env
  (Node : Bare_Use_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Clause  := Bare_Use_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1607 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
New_Struct_1 : Internal_Designated_Env := No_Designated_Env;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1607 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (New_Struct_1);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1607;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13333 '<If at nodes.lkt:18388:9>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18388'
--# expr-start 13329 '<IsA CompilationUnit at nodes.lkt:18388:12>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18388'
--# expr-start 13328 '<parent at nodes.lkt:18388:12>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18388'
--# expr-start 13327 '<parent at nodes.lkt:18388:12>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18388'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13327
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 13328
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 13329
if Is_A then
   --# expr-start 13331 '<New DesignatedEnv at nodes.lkt:18388:55>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18388'
--# expr-start 13330 '<Symbol Standard at nodes.lkt:18389:56>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18389'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 13330
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 13331
   If_Result := New_Struct;
else
   --# expr-start 13332 '<New DesignatedEnv at nodes.lkt:18390:16>' New_Struct_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18390'
Inc_Ref (Empty_Env);
New_Struct_1 := (Direct_Env => Empty_Env, Env_Name => No_Symbol, Kind => Current_Env); 
--# expr-done 13332
   If_Result := New_Struct_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 13333
         Property_Result := If_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1607;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1607;
            raise;
      end;
   return Property_Result;
end Use_Clause_P_Initial_Env;
--# end
   --
   --  Primitives for Bare_Use_Package_Clause
   --
      procedure Use_Package_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Package_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_226 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:18424:9");
      end;
      end;
      procedure Use_Package_Clause_Post_Env_Actions
        (Self : Bare_Use_Package_Clause; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_228 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_227 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Package_Clause
        (Self : Bare_Use_Package_Clause
         ; Use_Package_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.Use_Package_Clause_F_Packages := Use_Package_Clause_F_Packages;
      end Initialize_Fields_For_Use_Package_Clause;
   function Use_Package_Clause_F_Packages
     (Node : Bare_Use_Package_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Package_Clause_F_Packages;
   end;
--# property-start UsePackageClause.designated_env 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18401'
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Env
  (Node : Bare_Use_Package_Clause
      ; Index : Integer
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind index Index
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1608 with Inline_Always;
      Local_Pkg : Bare_Name;
Fld : Bare_Name_List;
Get_Result : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1608 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1608;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13342 '<Let at nodes.lkt:18402:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18402'
--# scope-start
--# expr-start 13335 '<at at nodes.lkt:18402:19>' Get_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18402'
--# expr-start 13334 '<FieldAccess for packages at nodes.lkt:18402:19>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18402'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 13334
Get_Result := Get (Self, Fld, Index, True); 
--# expr-done 13335
Local_Pkg := Get_Result; 
--# bind pkg Local_Pkg
--# expr-start 13341 '<bind at nodes.lkt:18405:13>' Dyn_Var_Bind_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18405'
--# expr-start 13336 '<FieldAccess for node_env at nodes.lkt:18405:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18405'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13336
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 13340 '<bind at nodes.lkt:18406:13>' Dyn_Var_Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18406'
--# expr-start 13337 '<FieldAccess for origin_node at nodes.lkt:18406:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18406'
      if Local_Pkg = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Local_Pkg);
--# end
--# expr-done 13337
Bound_Origin := Fld_2; 
--# expr-start 13339 '<FieldAccess for designated_env at nodes.lkt:18408:13>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18408'
--# expr-start 13338 '<as_bare_entity at nodes.lkt:18408:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18408'
Is_Null := Local_Pkg = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Local_Pkg); 
   If_Result := As_Entity;
end if;
--# expr-done 13338
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result.Node, Env => Bound_Env, Origin => Bound_Origin, No_Visibility => False, E_Info => If_Result.Info);
--# end
--# expr-done 13339
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 13340
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 13341
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 13342
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1608;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1608;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Designated_Env;
--# end
--# property-start UsePackageClause.designated_envs 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18416'
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Envs
  (Node : Bare_Use_Package_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_1609 with Inline_Always;
            procedure Finalizer_Scope_3630 with Inline_Always;
      Fld : Bare_Name_List;
Item_0 : Bare_Name;
Index_1 : Integer;
Fld_1 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_1609 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1609;
            procedure Finalizer_Scope_3630 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3630;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13345 '<Map at nodes.lkt:18417:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18417'
   --# expr-start 13343 '<FieldAccess for packages at nodes.lkt:18417:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18417'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 13343
      declare
         Map_Result_Vec : Lexical_Env_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Lexical_Env_Array (0);
      else
         Index_1 := 0;
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind _ Item_0
               --# bind i Index_1
      --# expr-start 13344 '<FieldAccess for designated_env at nodes.lkt:18417:38>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18417'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UsePackageClause.designated_env
Fld_1 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Env (Node => Ent.Node, Index => Index_1, E_Info => Ent.Info);
--# end
--# expr-done 13344
         declare
            Item_To_Append : constant Lexical_Env := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3630;
               Index_1 := Index_1 + 1;
         end loop;
      end;
      end if;
         Map_Result := Create_Lexical_Env_Array
           (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Lexical_Env_Vectors.Get
              (Map_Result_Vec,
               I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13345
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1609;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1609;
                     Finalizer_Scope_3630;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Designated_Envs;
--# end
--# property-start UsePackageClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18420'
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Xref_Equation
  (Node : Bare_Use_Package_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
     Ent : Internal_Entity_Use_Package_Clause :=
       Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1610 with Inline_Always;
            procedure Finalizer_Scope_3631 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1610 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1610;
            procedure Finalizer_Scope_3631 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3631;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13349 '<LogicAll at nodes.lkt:18421:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18421'
--# expr-start 13348 '<Map at nodes.lkt:18421:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18421'
   --# expr-start 13346 '<FieldAccess for packages at nodes.lkt:18421:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18421'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Package_Clause_F_Packages, Info => Ent.Info);
--# expr-done 13346
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind p Item_0
      --# expr-start 13347 '<FieldAccess for xref_no_overloading at nodes.lkt:18421:40>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18421'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 13347
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3631;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13348
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18421:9") else null)); 
--# expr-done 13349
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1610;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1610;
                     Finalizer_Scope_3631;
            raise;
      end;
   return Property_Result;
end Use_Package_Clause_P_Xref_Equation;
--# end
--# property-start UsePackageClause._env_init_226 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18424'
pragma Warnings (Off, "is not referenced");
 function Env_Init_226
  (Node : Bare_Use_Package_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1890 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1890 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1890;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13350 '<FieldAccess for initial_env at nodes.lkt:18424:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18424'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.initial_env
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env (Node => Self);
--# end
--# expr-done 13350
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1890;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1890;
            raise;
      end;
   return Property_Result;
end Env_Init_226;
--# end
--# property-start UsePackageClause._ref_env_nodes_227 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_227
  (Node : Bare_Use_Package_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1891 with Inline_Always;
      Fld : Bare_Name_List;
Item_0 : Bare_Name;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1891 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1891;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13353 '<Map at nodes.lkt:18429:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
   --# expr-start 13351 '<FieldAccess for packages at nodes.lkt:18429:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 13351
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind n Item_0
      --# expr-start 13352 '<Cast to AdaNode at nodes.lkt:18429:38>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
Cast_Expr := Item_0; 
      Cast_Result := Cast_Expr;
--# expr-done 13352
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13353
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1891;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1891;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_227;
--# end
--# property-start UsePackageClause._ref_cond_228 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_228
  (Node : Bare_Use_Package_Clause
  )
   return Boolean
is
   Self : Bare_Use_Package_Clause  := Bare_Use_Package_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13357 '<Not at nodes.lkt:18429:97>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
--# expr-start 13356 '<IsA CompilationUnit at nodes.lkt:18429:101>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
--# expr-start 13355 '<parent at nodes.lkt:18429:101>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
--# expr-start 13354 '<parent at nodes.lkt:18429:101>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18429'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13354
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 13355
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 13356
Not_Val := not (Is_A); 
--# expr-done 13357
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_228;
--# end
   --
   --  Primitives for Bare_Use_Type_Clause
   --
      procedure Use_Type_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Type_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_229 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:18444:9");
      end;
      end;
      procedure Use_Type_Clause_Post_Env_Actions
        (Self : Bare_Use_Type_Clause; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_231 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_230 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Type_Clause
        (Self : Bare_Use_Type_Clause
         ; Use_Type_Clause_F_Has_All : Bare_All_Node
         ; Use_Type_Clause_F_Types : Bare_Name_List
        ) is
      begin
            Self.Use_Type_Clause_F_Has_All := Use_Type_Clause_F_Has_All;
            Self.Use_Type_Clause_F_Types := Use_Type_Clause_F_Types;
      end Initialize_Fields_For_Use_Type_Clause;
   function Use_Type_Clause_F_Has_All
     (Node : Bare_Use_Type_Clause) return Bare_All_Node
   is
   begin
         return Node.Use_Type_Clause_F_Has_All;
   end;
   function Use_Type_Clause_F_Types
     (Node : Bare_Use_Type_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Type_Clause_F_Types;
   end;
--# property-start UseTypeClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18440'
pragma Warnings (Off, "is not referenced");
 function Use_Type_Clause_P_Xref_Equation
  (Node : Bare_Use_Type_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
     Ent : Internal_Entity_Use_Type_Clause :=
       Internal_Entity_Use_Type_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1611 with Inline_Always;
            procedure Finalizer_Scope_3633 with Inline_Always;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
Item_0 : Internal_Entity_Name;
Bare_Item_0 : Bare_Name;
Fld_3 : Logic_Equation;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1611 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1611;
            procedure Finalizer_Scope_3633 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_3633;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13361 '<LogicAll at nodes.lkt:18441:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18441'
--# expr-start 13360 '<Map at nodes.lkt:18441:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18441'
   --# expr-start 13358 '<FieldAccess for types at nodes.lkt:18441:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18441'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Type_Clause_F_Types, Info => Ent.Info);
--# expr-done 13358
Fld_1 := Fld.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_1 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_0); 
   If_Result := As_Entity;
end if;
                  Item_0 := If_Result; 
   --# scope-start
               --# bind p Item_0
      --# expr-start 13359 '<FieldAccess for xref_type_equation at nodes.lkt:18441:37>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18441'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_3 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Item_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Item_0.Info);
--# end
--# expr-done 13359
         declare
            Item_To_Append : constant Logic_Equation := Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3633;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13360
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18441:9") else null)); 
--# expr-done 13361
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1611;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1611;
                     Finalizer_Scope_3633;
            raise;
      end;
   return Property_Result;
end Use_Type_Clause_P_Xref_Equation;
--# end
--# property-start UseTypeClause._env_init_229 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18444'
pragma Warnings (Off, "is not referenced");
 function Env_Init_229
  (Node : Bare_Use_Type_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1893 with Inline_Always;
      Fld : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1893 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1893;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13362 '<FieldAccess for initial_env at nodes.lkt:18444:25>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18444'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start UseClause.initial_env
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env (Node => Self);
--# end
--# expr-done 13362
         Property_Result := Fld;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1893;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1893;
            raise;
      end;
   return Property_Result;
end Env_Init_229;
--# end
--# property-start UseTypeClause._ref_env_nodes_230 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_230
  (Node : Bare_Use_Type_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1894 with Inline_Always;
      Fld : Bare_Name_List;
Item_0 : Bare_Name;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1894 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1894;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13365 '<Map at nodes.lkt:18449:13>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
   --# expr-start 13363 '<FieldAccess for types at nodes.lkt:18449:13>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Use_Type_Clause_F_Types;
--# expr-done 13363
      declare
         Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind n Item_0
      --# expr-start 13364 '<Cast to AdaNode at nodes.lkt:18449:35>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
Cast_Expr := Item_0; 
      Cast_Result := Cast_Expr;
--# expr-done 13364
         declare
            Item_To_Append : constant Bare_Ada_Node := Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      end if;
         Map_Result := Create_Bare_Ada_Node_Array
           (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
              (Map_Result_Vec,
               I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13365
         Property_Result := Map_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1894;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1894;
            raise;
      end;
   return Property_Result;
end Ref_Env_Nodes_230;
--# end
--# property-start UseTypeClause._ref_cond_231 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_231
  (Node : Bare_Use_Type_Clause
  )
   return Boolean
is
   Self : Bare_Use_Type_Clause  := Bare_Use_Type_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13369 '<Not at nodes.lkt:18449:87>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
--# expr-start 13368 '<IsA CompilationUnit at nodes.lkt:18449:91>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
--# expr-start 13367 '<parent at nodes.lkt:18449:91>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
--# expr-start 13366 '<parent at nodes.lkt:18449:91>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18449'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.Parent; 
--# expr-done 13366
      if Fld = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 13367
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 13368
Not_Val := not (Is_A); 
--# expr-done 13369
         Property_Result := Not_Val;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Ref_Cond_231;
--# end
   --
   --  Primitives for Bare_Value_Sequence
   --
      procedure Initialize_Fields_For_Value_Sequence
        (Self : Bare_Value_Sequence
         ; Value_Sequence_F_Iter_Assoc : Bare_Iterated_Assoc
        ) is
      begin
            Self.Value_Sequence_F_Iter_Assoc := Value_Sequence_F_Iter_Assoc;
      end Initialize_Fields_For_Value_Sequence;
   function Value_Sequence_F_Iter_Assoc
     (Node : Bare_Value_Sequence) return Bare_Iterated_Assoc
   is
   begin
         return Node.Value_Sequence_F_Iter_Assoc;
   end;
--# property-start ValueSequence.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18463'
pragma Warnings (Off, "is not referenced");
 function Value_Sequence_P_Xref_Equation
  (Node : Bare_Value_Sequence
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Value_Sequence  := Bare_Value_Sequence (Node);
     Ent : Internal_Entity_Value_Sequence :=
       Internal_Entity_Value_Sequence'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1612 with Inline_Always;
      Fld : Internal_Entity_Iterated_Assoc;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1612 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1612;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13371 '<FieldAccess for xref_equation_for_reduce at nodes.lkt:18463:37>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18463'
--# expr-start 13370 '<FieldAccess for iter_assoc at nodes.lkt:18463:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18463'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Iterated_Assoc (Node => Ent.Node.Value_Sequence_F_Iter_Assoc, Info => Ent.Info);
--# expr-done 13370
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start IteratedAssoc.xref_equation_for_reduce
Fld_1 := Libadalang.Implementation.Iterated_Assoc_P_Xref_Equation_For_Reduce (Node => Fld.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Fld.Info);
--# end
--# expr-done 13371
         Property_Result := Fld_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1612;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1612;
            raise;
      end;
   return Property_Result;
end Value_Sequence_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Variant
   --
      procedure Initialize_Fields_For_Variant
        (Self : Bare_Variant
         ; Variant_F_Choices : Bare_Alternatives_List
         ; Variant_F_Components : Bare_Component_List
        ) is
      begin
            Self.Variant_F_Choices := Variant_F_Choices;
            Self.Variant_F_Components := Variant_F_Components;
      end Initialize_Fields_For_Variant;
   function Variant_F_Choices
     (Node : Bare_Variant) return Bare_Alternatives_List
   is
   begin
         return Node.Variant_F_Choices;
   end;
   function Variant_F_Components
     (Node : Bare_Variant) return Bare_Component_List
   is
   begin
         return Node.Variant_F_Components;
   end;
--# property-start Variant.default_discriminant_expr 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18475'
pragma Warnings (Off, "is not referenced");
 function Variant_P_Default_Discriminant_Expr
  (Node : Bare_Variant
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Variant  := Bare_Variant (Node);
     Ent : Internal_Entity_Variant :=
       Internal_Entity_Variant'(Node => Self, Info => E_Info);
      --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
            procedure Finalizer_Scope_1613 with Inline_Always;
            procedure Finalizer_Scope_3636 with Inline_Always;
      Local_Discr_Specs : Internal_Entity_Discriminant_Spec_List;
Local_Discr_Symbol : Symbol_Type;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result : Internal_Entity;
Cast_Result : Internal_Entity_Concrete_Type_Decl;
Fld_1 : Internal_Entity_Discriminant_Part;
Cast_Result_1 : Internal_Entity_Known_Discriminant_Part;
Fld_2 : Internal_Entity_Discriminant_Spec_List;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Variant_Part;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Bare_Identifier;
Sym : Symbol_Type;
Saved : Internal_Entity_Discriminant_Spec_List;
Fld_7 : Bare_Discriminant_Spec_List;
Fld_8 : Internal_Entity_Info;
Item_1 : Internal_Entity_Discriminant_Spec;
Bare_Item_1 : Bare_Discriminant_Spec;
Fld_9 : Internal_Entity_Defining_Name_Array_Access;
Fld_10 : Boolean;
Quantifier_Result : Boolean;
As_Entity : Internal_Entity_Discriminant_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Discriminant_Spec;
Find_Result_1 : Internal_Entity_Discriminant_Spec;
Fld_11 : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Expr;
            procedure Finalizer_Scope_1613 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1613;
            procedure Finalizer_Scope_3636 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_3636;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13390 '<Let at nodes.lkt:18478:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
--# scope-start
--# expr-start 13378 '<FieldAccess for discr_specs at nodes.lkt:18478:27>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
--# expr-start 13377 '<Cast to KnownDiscriminantPart at nodes.lkt:18478:27>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
--# expr-start 13376 '<FieldAccess for discriminants at nodes.lkt:18478:27>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
--# expr-start 13375 '<Cast to ConcreteTypeDecl at nodes.lkt:18478:27>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
--# expr-start 13374 '<Find at nodes.lkt:18478:27>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
   --# expr-start 13372 '<FieldAccess for parents at nodes.lkt:18478:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => False, E_Info => Ent.Info);
--# expr-done 13372
      Find_Result := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
      --# expr-start 13373 '<IsA ConcreteTypeDecl at nodes.lkt:18478:69>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18478'
Is_A := Item_0.Node /= null 
and then Item_0.Node.Kind in Ada_Concrete_Type_Decl_Range; 
--# expr-done 13373
      if Is_A then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 13374
   if Find_Result.Node = null
      or else Find_Result.Node.Kind in Ada_Concrete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Concrete_Type_Decl
        (Node => Find_Result.Node,
         Info => Find_Result.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 13375
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Discriminant_Part (Node => Cast_Result.Node.Type_Decl_F_Discriminants, Info => Cast_Result.Info);
--# expr-done 13376
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Known_Discriminant_Part_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Known_Discriminant_Part
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 13377
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Discriminant_Spec_List (Node => Cast_Result_1.Node.Known_Discriminant_Part_F_Discr_Specs, Info => Cast_Result_1.Info);
--# expr-done 13378
Local_Discr_Specs := Fld_2; 
--# bind discr_specs Local_Discr_Specs
--# expr-start 13389 '<Let at nodes.lkt:18479:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
--# scope-start
--# expr-start 13383 '<GetSymbol at nodes.lkt:18479:28>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
--# expr-start 13382 '<FieldAccess for discr_name at nodes.lkt:18479:28>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
--# expr-start 13381 '<Cast to VariantPart at nodes.lkt:18479:28>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
--# expr-start 13380 '<parent at nodes.lkt:18479:28>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
--# expr-start 13379 '<parent at nodes.lkt:18479:28>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18479'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 13379
      if Fld_3.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 13380
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Variant_Part_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Variant_Part
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Raise_Property_Exception
           (Self, Property_Error'Identity, "invalid object cast");
   end if;
--# expr-done 13381
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Cast_Result_2.Node.Variant_Part_F_Discr_Name, Info => Cast_Result_2.Info);
--# expr-done 13382
Fld_6 := Fld_5.Node; 
Sym := Get_Symbol (Fld_6); 
--# expr-done 13383
Local_Discr_Symbol := Sym; 
--# bind discr_symbol Local_Discr_Symbol
--# expr-start 13388 '<FieldAccess for default_expr at nodes.lkt:18483:9>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18483'
--# expr-start 13387 '<Find at nodes.lkt:18483:9>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18483'
Saved := Local_Discr_Specs; 
Fld_7 := Saved.Node;
      Find_Result_1 := No_Entity_Discriminant_Spec;
      if Fld_7 = null then
      Find_Result_1 := No_Entity_Discriminant_Spec;
      else
      declare
         Collection : constant Bare_Discriminant_Spec_List := Fld_7;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null := Bare_Item_1 = null; 
if Is_Null then
   If_Result := No_Entity_Discriminant_Spec;
else
Fld_8 := Saved.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_1); 
   If_Result := As_Entity;
end if;
                  Item_1 := If_Result; 
   --# scope-start
               --# bind d Item_1
      --# expr-start 13386 '<AnyQuantifier at nodes.lkt:18484:20>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18484'
   --# expr-start 13384 '<FieldAccess for defining_names at nodes.lkt:18484:20>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18484'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_1.Node, E_Info => Item_1.Info);
--# end
--# expr-done 13384
      Quantifier_Result := False;
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_9;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 13385 '<FieldAccess for name_is at nodes.lkt:18484:50>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18484'
      if Item_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_is
Fld_10 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_2.Node, Sym => Local_Discr_Symbol);
--# end
--# expr-done 13385
      Quantifier_Result := Fld_10;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
--# expr-done 13386
      if Quantifier_Result then
         Find_Result_1 := Item_1; 
      end if;
   --# end
      Finalizer_Scope_3636;
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 13387
      if Find_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Expr (Node => Find_Result_1.Node.Discriminant_Spec_F_Default_Expr, Info => Find_Result_1.Info);
--# expr-done 13388
Let_Result := Fld_11; 
--# end
--# expr-done 13389
Let_Result_1 := Let_Result; 
--# end
--# expr-done 13390
         Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_1613;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1613;
                     Finalizer_Scope_3636;
            raise;
      end;
   return Property_Result;
end Variant_P_Default_Discriminant_Expr;
--# end
--# property-start Variant.matches 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18489'
pragma Warnings (Off, "is not referenced");
 function Variant_P_Matches
  (Node : Bare_Variant
      ; Expr : Internal_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Variant  := Bare_Variant (Node);
     Ent : Internal_Entity_Variant :=
       Internal_Entity_Variant'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind expr Expr
   Property_Result : Boolean;
            procedure Finalizer_Scope_1614 with Inline_Always;
      Local_Expr_Val : Big_Integer_Type;
Fld : Internal_Entity_Expr;
Is_A : Boolean;
If_Result : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Fld_2 : Internal_Entity_Alternatives_List;
Fld_3 : Bare_Alternatives_List;
Fld_4 : Internal_Entity_Info;
Item_0 : Internal_Entity;
Bare_Item_0 : Bare_Ada_Node;
Fld_5 : Boolean;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1614 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Local_Expr_Val);
            end Finalizer_Scope_1614;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13398 '<Let at nodes.lkt:18492:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18492'
--# scope-start
--# expr-start 13394 '<FieldAccess for eval_as_int at nodes.lkt:18492:24>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18492'
--# expr-start 13393 '<If at nodes.lkt:18495:13>' If_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18495'
--# expr-start 13391 '<IsA BoxExpr at nodes.lkt:18495:16>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18495'
Is_A := Expr.Node /= null 
and then Expr.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 13391
if Is_A then
   --# expr-start 13392 '<FieldAccess for default_discriminant_expr at nodes.lkt:18495:37>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18495'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Variant.default_discriminant_expr
Fld := Libadalang.Implementation.Variant_P_Default_Discriminant_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 13392
   If_Result := Fld;
else
   If_Result := Expr;
end if;
--# expr-done 13393
      if If_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 13394
Local_Expr_Val := Fld_1; Inc_Ref (Local_Expr_Val);
--# bind expr_val Local_Expr_Val
--# expr-start 13397 '<AnyQuantifier at nodes.lkt:18498:9>' Quantifier_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18498'
   --# expr-start 13395 '<FieldAccess for choices at nodes.lkt:18498:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18498'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Variant_F_Choices, Info => Ent.Info);
--# expr-done 13395
Fld_3 := Fld_2.Node;
      Quantifier_Result := False;
      if Fld_3 = null then
      null;
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_3;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null := Bare_Item_0 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_0); 
   If_Result_1 := As_Entity;
end if;
                  Item_0 := If_Result_1; 
   --# scope-start
               --# bind c Item_0
      --# expr-start 13396 '<FieldAccess for choice_match at nodes.lkt:18498:33>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18498'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.choice_match
Fld_5 := Libadalang.Implementation.Ada_Node_P_Choice_Match (Node => Item_0.Node, Value => Local_Expr_Val, E_Info => Item_0.Info);
--# end
--# expr-done 13396
      Quantifier_Result := Fld_5;
   --# end
               exit when Quantifier_Result;
         end loop;
      end;
      end if;
--# expr-done 13397
Let_Result := Quantifier_Result; 
--# end
--# expr-done 13398
         Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1614;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1614;
            raise;
      end;
   return Property_Result;
end Variant_P_Matches;
--# end
   --
   --  Primitives for Bare_Variant_Part
   --
      procedure Initialize_Fields_For_Variant_Part
        (Self : Bare_Variant_Part
         ; Variant_Part_F_Discr_Name : Bare_Identifier
         ; Variant_Part_F_Variant : Bare_Variant_List
        ) is
      begin
            Self.Variant_Part_F_Discr_Name := Variant_Part_F_Discr_Name;
            Self.Variant_Part_F_Variant := Variant_Part_F_Variant;
      end Initialize_Fields_For_Variant_Part;
   function Variant_Part_F_Discr_Name
     (Node : Bare_Variant_Part) return Bare_Identifier
   is
   begin
         return Node.Variant_Part_F_Discr_Name;
   end;
   function Variant_Part_F_Variant
     (Node : Bare_Variant_Part) return Bare_Variant_List
   is
   begin
         return Node.Variant_Part_F_Variant;
   end;
--# property-start VariantPart.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18510'
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Entry_Point
  (Node : Bare_Variant_Part
  )
   return Boolean
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Xref_Entry_Point;
--# end
--# property-start VariantPart.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18513'
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Equation
  (Node : Bare_Variant_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
     Ent : Internal_Entity_Variant_Part :=
       Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1616 with Inline_Always;
            procedure Finalizer_Scope_3639 with Inline_Always;
            procedure Finalizer_Scope_3640 with Inline_Always;
            procedure Finalizer_Scope_3641 with Inline_Always;
            procedure Finalizer_Scope_3642 with Inline_Always;
            procedure Finalizer_Scope_3643 with Inline_Always;
      Ignored : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Variant_List;
Fld_3 : Bare_Variant_List;
Fld_4 : Internal_Entity_Info;
Item_0 : Internal_Entity_Variant;
Bare_Item_0 : Bare_Variant;
Fld_5 : Internal_Entity_Alternatives_List;
Fld_6 : Bare_Alternatives_List;
Fld_7 : Internal_Entity_Info;
Item_1 : Internal_Entity;
Bare_Item_1 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Bare_Identifier;
Fld_11 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_A : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_15 : Logic_Equation;
True_Rel : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
As_Entity : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_2 : Boolean;
If_Result_3 : Internal_Entity_Variant;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1616 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1616;
            procedure Finalizer_Scope_3639 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_3639;
            procedure Finalizer_Scope_3640 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_3640;
            procedure Finalizer_Scope_3641 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_3641;
            procedure Finalizer_Scope_3642 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_3642;
            procedure Finalizer_Scope_3643 is
            begin
                     Dec_Ref (True_Rel);
            end Finalizer_Scope_3643;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13425 '<Let at nodes.lkt:18514:9>' Let_Result_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18514'
--# scope-start
--# expr-start 13400 '<FieldAccess for resolve_names_internal at nodes.lkt:18514:17>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18514'
--# expr-start 13399 '<FieldAccess for discr_name at nodes.lkt:18514:17>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18514'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Variant_Part_F_Discr_Name, Info => Ent.Info);
--# expr-done 13399
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Generate_Diagnostics => False, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 13400
Ignored := Fld_1; 
--# bind _ Ignored
--# expr-start 13424 '<LogicAll at nodes.lkt:18516:9>' Logic_Boolean_Op_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18516'
--# expr-start 13423 '<Map at nodes.lkt:18516:9>' Map_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18516'
   --# expr-start 13401 '<FieldAccess for variant at nodes.lkt:18516:9>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18516'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 13401
Fld_3 := Fld_2.Node;
      declare
         Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_3 = null then
      Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_3;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_0 := Untyped_Item_0; 
Is_Null_2 := Bare_Item_0 = null; 
if Is_Null_2 then
   If_Result_3 := No_Entity_Variant;
else
Fld_4 := Fld_2.Info;
As_Entity_1 := (Info => Fld_4, Node => Bare_Item_0); 
   If_Result_3 := As_Entity_1;
end if;
                  Item_0 := If_Result_3; 
   --# scope-start
               --# bind var Item_0
      --# expr-start 13422 '<LogicAll at nodes.lkt:18517:22>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18517'
--# expr-start 13421 '<Map at nodes.lkt:18517:22>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18517'
   --# expr-start 13402 '<FieldAccess for choices at nodes.lkt:18517:22>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18517'
      if Item_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_5 := Create_Internal_Entity_Alternatives_List (Node => Item_0.Node.Variant_F_Choices, Info => Item_0.Info);
--# expr-done 13402
Fld_6 := Fld_5.Node;
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld_6 = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_6;
      begin
         for Untyped_Item_1 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_1 := Untyped_Item_1; 
Is_Null_1 := Bare_Item_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
Fld_7 := Fld_5.Info;
As_Entity := (Info => Fld_7, Node => Bare_Item_1); 
   If_Result_2 := As_Entity;
end if;
                  Item_1 := If_Result_2; 
   --# scope-start
               --# bind c Item_1
      --# expr-start 13420 '<Match at nodes.lkt:18518:24>' Match_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18518'
      if Item_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Match_Prefix := Item_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 13418 '<If at nodes.lkt:18520:37>' If_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13403 '<IsA Name at nodes.lkt:18520:40>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
Is_A := Match_0.Node /= null 
and then Match_0.Node.Kind in Ada_Name; 
--# expr-done 13403
if Is_A then
   --# expr-start 13407 '<Not at nodes.lkt:18520:54>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13406 '<IsNull at nodes.lkt:18520:58>' Is_Null 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13405 '<FieldAccess for name_designated_type at nodes.lkt:18520:58>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13404 '<Cast to Name at nodes.lkt:18520:58>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
Cast_Expr_1 := Match_0; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 13404
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.name_designated_type
Fld_14 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 13405
Is_Null := Fld_14.Node = null; 
--# expr-done 13406
Not_Val := not (Is_Null); 
--# expr-done 13407
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 13409 '<FieldAccess for xref_type_equation at nodes.lkt:18520:105>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13408 '<Cast to Name at nodes.lkt:18520:105>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
Cast_Expr := Match_0; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 13408
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Name.xref_type_equation
Fld_8 := Libadalang.Implementation.Name_P_Xref_Type_Equation (Node => Cast_Result.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Cast_Result.Info);
--# end
--# expr-done 13409
   If_Result_1 := Fld_8;
else
   --# expr-start 13417 '<And at nodes.lkt:18520:142>' And_Pred_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13415 '<And at nodes.lkt:18520:142>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13413 '<Bind at nodes.lkt:18520:142>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13410 '<FieldAccess for expected_type_var at nodes.lkt:18520:146>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Expected_Type_Var (Node => Match_0.Node);
--# expr-done 13410
Fld_9.Value := No_Entity;
Entity_Vars.Reset (Fld_9);
--# expr-start 13412 '<FieldAccess for type_val at nodes.lkt:18520:169>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
--# expr-start 13411 '<FieldAccess for discr_name at nodes.lkt:18520:169>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_10 := Self.Variant_Part_F_Discr_Name;
--# expr-done 13411
      if Fld_10 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.type_val
Fld_11 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_10);
--# end
--# expr-done 13412
Bind_Result := Solver.Create_Assign (Fld_9, Fld_11, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18520:142") else null)); 
--# expr-done 13413
--# expr-start 13414 '<FieldAccess for sub_equation at nodes.lkt:18520:201>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_0.Info);
--# end
--# expr-done 13414
And_Pred := Create_And (Bind_Result, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18520:142") else null)); 
--# expr-done 13415
--# expr-start 13416 '<FieldAccess for matches_expected_type at nodes.lkt:18520:222>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18520'
      if Match_0.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Expr.matches_expected_type
Fld_13 := Libadalang.Implementation.Expr_P_Matches_Expected_Type (Node => Match_0.Node, Origin => Origin);
--# end
--# expr-done 13416
And_Pred_1 := Create_And (And_Pred, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18520:142") else null)); 
--# expr-done 13417
   If_Result_1 := And_Pred_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 13418
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_3641;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 13419 '<FieldAccess for xref_equation at nodes.lkt:18523:50>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18523'
      if Match_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_15 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, Entry_Point => Entry_Point, E_Info => Match_1.Info);
--# end
--# expr-done 13419
Let_Result_1 := Fld_15; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3642;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18524:49") else null)); 
Let_Result_2 := True_Rel; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_3643;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_For_Loop_Iter_Filter .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Synthetic_Type_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 13420
         declare
            Item_To_Append : constant Logic_Equation := Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3640;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13421
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18517:22") else null)); 
--# expr-done 13422
         declare
            Item_To_Append : constant Logic_Equation := Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3639;
         end loop;
      end;
      end if;
         Map_Result_1 := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
         for I in Map_Result_1.Items'Range loop
            Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_1_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end;
--# expr-done 13423
Logic_Boolean_Op_1 := Solver.Create_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18516:9") else null)); 
--# expr-done 13424
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 13425
         Property_Result := Let_Result_4;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1616;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1616;
                     Finalizer_Scope_3639;
                     Finalizer_Scope_3640;
                     Finalizer_Scope_3641;
                     Finalizer_Scope_3642;
                     Finalizer_Scope_3643;
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Xref_Equation;
--# end
--# property-start VariantPart.get_components 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18533'
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Get_Components
  (Node : Bare_Variant_Part
      ; Discriminants : Internal_Param_Match_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Variant_Part  := Bare_Variant_Part (Node);
     Ent : Internal_Entity_Variant_Part :=
       Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind discriminants Discriminants
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1617 with Inline_Always;
            procedure Finalizer_Scope_3648 with Inline_Always;
      Local_Discr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Param_Match;
Local_Variant : Internal_Entity_Variant;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Find_Result : Internal_Param_Match;
Fld_4 : Internal_Single_Actual;
Fld_5 : Internal_Entity_Basic_Assoc;
Fld_6 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Fld_7 : Internal_Entity_Array_Access;
Is_A : Boolean;
Find_Result_1 : Internal_Entity;
Cast_Result : Internal_Entity_Concrete_Type_Decl;
Fld_8 : Internal_Entity_Discriminant_Part;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity_Discriminant_Spec;
Fld_10 : Internal_Entity_Defining_Name_Array_Access;
Fld_11 : Internal_Entity_Name;
Fld_12 : Bare_Identifier;
Sym_1 : Symbol_Type;
Fld_13 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
Find_Result_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_2 : Internal_Entity_Discriminant_Spec;
Fld_14 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_15 : Internal_Entity_Variant_List;
Fld_16 : Bare_Variant_List;
Fld_17 : Internal_Entity_Info;
Item_5 : Internal_Entity_Variant;
Bare_Item_5 : Bare_Variant;
Fld_18 : Boolean;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Find_Result_3 : Internal_Entity_Variant;
Fld_19 : Internal_Entity_Component_List;
Fld_20 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1617 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1617;
            procedure Finalizer_Scope_3648 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_3648;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13459 '<Let at nodes.lkt:18535:9>' Let_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18535'
--# scope-start
--# expr-start 13431 '<Find at nodes.lkt:18535:21>' Find_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18535'
      Find_Result := No_Param_Match;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Discriminants;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_0
      --# expr-start 13430 '<FieldAccess for name_is at nodes.lkt:18536:20>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18536'
--# expr-start 13427 '<FieldAccess for name at nodes.lkt:18536:20>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18536'
--# expr-start 13426 '<FieldAccess for formal at nodes.lkt:18536:20>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18536'
Fld := Item_0.Formal;
--# expr-done 13426
      if Fld.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 13427
      if Fld_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13429 '<GetSymbol at nodes.lkt:18536:42>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18536'
--# expr-start 13428 '<FieldAccess for discr_name at nodes.lkt:18536:42>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18536'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_2 := Self.Variant_Part_F_Discr_Name;
--# expr-done 13428
Sym := Get_Symbol (Fld_2); 
--# expr-done 13429
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Sym);
--# end
--# expr-done 13430
      if Fld_3 then
         Find_Result := Item_0; 
      end if;
   --# end
               exit when Fld_3;
         end loop;
      end;
--# expr-done 13431
Var_Expr_1 := Find_Result;
if Var_Expr_1 /= No_Param_Match then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 13434 '<FieldAccess for expr at nodes.lkt:18538:20>' Fld_6 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13433 '<FieldAccess for assoc at nodes.lkt:18538:20>' Fld_5 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13432 '<FieldAccess for actual at nodes.lkt:18538:20>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
Fld_4 := Var_Expr_1.Actual;
--# expr-done 13432
Fld_5 := Fld_4.Assoc;
--# expr-done 13433
      if Fld_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 13434
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 13452 '<FieldAccess for default_expr at nodes.lkt:18538:55>' Fld_14 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13451 '<Cast to DiscriminantSpec at nodes.lkt:18538:55>' Cast_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13450 '<Find at nodes.lkt:18538:55>' Find_Result_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
   --# expr-start 13440 '<FieldAccess for abstract_formal_params at nodes.lkt:18538:55>' Fld_9 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13439 '<FieldAccess for discriminants at nodes.lkt:18538:55>' Fld_8 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13438 '<Cast to ConcreteTypeDecl at nodes.lkt:18538:55>' Cast_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
--# expr-start 13437 '<Find at nodes.lkt:18538:55>' Find_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
   --# expr-start 13435 '<FieldAccess for parents at nodes.lkt:18538:55>' Fld_7 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_7 := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 13435
      Find_Result_1 := No_Entity;
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_7;
      begin
         for Item_2 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_2
      --# expr-start 13436 '<IsA ConcreteTypeDecl at nodes.lkt:18538:82>' Is_A 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18538'
Is_A := Item_2.Node /= null 
and then Item_2.Node.Kind in Ada_Concrete_Type_Decl_Range; 
--# expr-done 13436
      if Is_A then
         Find_Result_1 := Item_2; 
      end if;
   --# end
               exit when Is_A;
         end loop;
      end;
--# expr-done 13437
   if Find_Result_1.Node = null
      or else Find_Result_1.Node.Kind in Ada_Concrete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Concrete_Type_Decl
        (Node => Find_Result_1.Node,
         Info => Find_Result_1.Info);
   else
         Cast_Result := No_Entity_Concrete_Type_Decl;
   end if;
--# expr-done 13438
      if Cast_Result.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_8 := Create_Internal_Entity_Discriminant_Part (Node => Cast_Result.Node.Type_Decl_F_Discriminants, Info => Cast_Result.Info);
--# expr-done 13439
      if Fld_8.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 13440
      Find_Result_2 := No_Entity_Base_Formal_Param_Decl;
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Fld_9;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_3
      --# expr-start 13449 '<Not at nodes.lkt:18539:24>' Not_Val 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18539'
--# expr-start 13448 '<IsNull at nodes.lkt:18539:28>' Is_Equal 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18539'
--# expr-start 13447 '<Map at nodes.lkt:18539:28>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18539'
   --# expr-start 13442 '<FieldAccess for defining_names at nodes.lkt:18539:28>' Fld_10 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18539'
--# expr-start 13441 '<Cast to DiscriminantSpec at nodes.lkt:18539:28>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18539'
Cast_Expr := Item_3; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Discriminant_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 13441
      if Cast_Result_1.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 13442
      declare
         Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
      begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_10;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_4
         --# expr-start 13446 '<FieldAccess for name_is at nodes.lkt:18544:28>' Fld_13 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18544'
--# expr-start 13443 '<FieldAccess for name at nodes.lkt:18544:28>' Fld_11 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18544'
      if Item_4.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Item_4.Node.Defining_Name_F_Name, Info => Item_4.Info);
--# expr-done 13443
      if Fld_11.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13445 '<GetSymbol at nodes.lkt:18544:43>' Sym_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18544'
--# expr-start 13444 '<FieldAccess for discr_name at nodes.lkt:18544:43>' Fld_12 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18544'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_12 := Self.Variant_Part_F_Discr_Name;
--# expr-done 13444
Sym_1 := Get_Symbol (Fld_12); 
--# expr-done 13445
--# property-call-start Name.name_is
Fld_13 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_11.Node, Sym => Sym_1);
--# end
--# expr-done 13446
         if Fld_13 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name := Item_4;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
         end if;
   --# end
         end loop;
      end;
         Map_Result := Create_Internal_Entity_Defining_Name_Array
           (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
              (Map_Result_Vec,
               I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13447
Is_Equal := Equivalent (Map_Result, No_Internal_Entity_Defining_Name_Array_Type); 
--# expr-done 13448
Not_Val := not (Is_Equal); 
--# expr-done 13449
      if Not_Val then
         Find_Result_2 := Item_3; 
      end if;
   --# end
      Finalizer_Scope_3648;
               exit when Not_Val;
         end loop;
      end;
--# expr-done 13450
   if Find_Result_2.Node = null
      or else Find_Result_2.Node.Kind in Ada_Discriminant_Spec_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Discriminant_Spec
        (Node => Find_Result_2.Node,
         Info => Find_Result_2.Info);
   else
         Cast_Result_2 := No_Entity_Discriminant_Spec;
   end if;
--# expr-done 13451
      if Cast_Result_2.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Cast_Result_2.Node.Discriminant_Spec_F_Default_Expr, Info => Cast_Result_2.Info);
--# expr-done 13452
   Result_Var := Fld_14;
end if;
Local_Discr := Result_Var; 
--# bind discr Local_Discr
--# expr-start 13458 '<Let at nodes.lkt:18550:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18550'
--# scope-start
--# expr-start 13455 '<Find at nodes.lkt:18550:23>' Find_Result_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18550'
   --# expr-start 13453 '<FieldAccess for variant at nodes.lkt:18550:23>' Fld_15 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18550'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_15 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 13453
Fld_16 := Fld_15.Node;
      Find_Result_3 := No_Entity_Variant;
      if Fld_16 = null then
      Find_Result_3 := No_Entity_Variant;
      else
      declare
         Collection : constant Bare_Variant_List := Fld_16;
      begin
         for Untyped_Item_5 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_5 := Untyped_Item_5; 
Is_Null := Bare_Item_5 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_17 := Fld_15.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_5); 
   If_Result := As_Entity;
end if;
                  Item_5 := If_Result; 
   --# scope-start
               --# bind v Item_5
      --# expr-start 13454 '<FieldAccess for matches at nodes.lkt:18550:48>' Fld_18 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18550'
      if Item_5.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start Variant.matches
Fld_18 := Libadalang.Implementation.Variant_P_Matches (Node => Item_5.Node, Expr => Local_Discr, E_Info => Item_5.Info);
--# end
--# expr-done 13454
      if Fld_18 then
         Find_Result_3 := Item_5; 
      end if;
   --# end
               exit when Fld_18;
         end loop;
      end;
      end if;
--# expr-done 13455
Local_Variant := Find_Result_3; 
--# bind variant Local_Variant
--# expr-start 13457 '<FieldAccess for abstract_formal_params_impl at nodes.lkt:18555:9>' Fld_20 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18555'
--# expr-start 13456 '<FieldAccess for components at nodes.lkt:18555:9>' Fld_19 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18555'
      if Local_Variant.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_19 := Create_Internal_Entity_Component_List (Node => Local_Variant.Node.Variant_F_Components, Info => Local_Variant.Info);
--# expr-done 13456
      if Fld_19.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_20 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_19.Node, Discriminants => Discriminants, Include_Discriminants => False, Recurse => False, E_Info => Fld_19.Info);
--# end
--# expr-done 13457
Let_Result := Fld_20; Inc_Ref (Let_Result);
--# end
--# expr-done 13458
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 13459
         Property_Result := Let_Result_1;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1617;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1617;
                     Finalizer_Scope_3648;
            raise;
      end;
   return Property_Result;
end Variant_Part_P_Get_Components;
--# end
   --
   --  Primitives for Bare_With_Clause
   --
      procedure With_Clause_Pre_Env_Actions
        (Self            : Bare_With_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Env : constant Internal_Designated_Env :=
           Libadalang.Implementation.Env_Init_232 (Node => 
Self);
      begin
         Set_Initial_Env
           (Self,
            State,
            Env,
            DSL_Location => "nodes.lkt:18583:9");
      end;
      end;
      procedure Initialize_Fields_For_With_Clause
        (Self : Bare_With_Clause
         ; With_Clause_F_Has_Limited : Bare_Limited_Node
         ; With_Clause_F_Has_Private : Bare_Private_Node
         ; With_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.With_Clause_F_Has_Limited := With_Clause_F_Has_Limited;
            Self.With_Clause_F_Has_Private := With_Clause_F_Has_Private;
            Self.With_Clause_F_Packages := With_Clause_F_Packages;
      end Initialize_Fields_For_With_Clause;
   function With_Clause_F_Has_Limited
     (Node : Bare_With_Clause) return Bare_Limited_Node
   is
   begin
         return Node.With_Clause_F_Has_Limited;
   end;
   function With_Clause_F_Has_Private
     (Node : Bare_With_Clause) return Bare_Private_Node
   is
   begin
         return Node.With_Clause_F_Has_Private;
   end;
   function With_Clause_F_Packages
     (Node : Bare_With_Clause) return Bare_Name_List
   is
   begin
         return Node.With_Clause_F_Packages;
   end;
--# property-start WithClause.child_unit_xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18568'
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Child_Unit_Xref_Equation
  (Node : Bare_With_Clause
      ; Name : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
     Ent : Internal_Entity_With_Clause :=
       Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind name Name
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1618 with Inline_Always;
            procedure Finalizer_Scope_3652 with Inline_Always;
      Local_Self_Eq : Logic_Equation;
Var_Expr_0 : Bare_Dotted_Name;
Var_Expr_298 : Bare_Compilation_Unit;
Fld : Logic_Var;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Dotted_Name;
Fld_3 : Bare_Name;
Fld_4 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
True_Rel : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1618 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Local_Self_Eq);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (True_Rel);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1618;
            procedure Finalizer_Scope_3652 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_3652;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13468 '<Let at nodes.lkt:18569:9>' Let_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18569'
--# scope-start
--# expr-start 13463 '<Bind at nodes.lkt:18569:23>' Bind_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18569'
--# expr-start 13460 '<FieldAccess for ref_var at nodes.lkt:18569:27>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18569'
      if Name = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Name);
--# end
--# expr-done 13460
Fld.Value := No_Entity;
Entity_Vars.Reset (Fld);
--# expr-start 13461 '<FieldAccess for withed_unit_helper at nodes.lkt:18569:43>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18569'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start AdaNode.withed_unit_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Withed_Unit_Helper (Node => Self, Unit_Name => Name);
--# end
--# expr-done 13461
Var_Expr_298 := Fld_1;
if Var_Expr_298 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 13462 '<FieldAccess for decl at nodes.lkt:18569:43>' Fld_2 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18569'
      if Var_Expr_298 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start CompilationUnit.decl
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr_298);
--# end
--# expr-done 13462
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => If_Result.Node,
         Info => If_Result.Info);
Bind_Result := Solver.Create_Assign (Fld, Cast_Result, Solver_Ifc.No_Converter, Debug_String => (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18569:23") else null)); 
--# expr-done 13463
Local_Self_Eq := Bind_Result; Inc_Ref (Local_Self_Eq);
--# bind self_eq Local_Self_Eq
--# expr-start 13467 '<And at nodes.lkt:18571:9>' And_Pred 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18571'
--# expr-start 13464 '<Cast to DottedName at nodes.lkt:18571:21>' Cast_Result_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18571'
Cast_Expr := Name; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 13464
Var_Expr_0 := Cast_Result_1;
if Var_Expr_0 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dn Var_Expr_0
--# expr-start 13466 '<FieldAccess for child_unit_xref_equation at nodes.lkt:18572:21>' Fld_4 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18572'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# expr-start 13465 '<FieldAccess for prefix at nodes.lkt:18572:51>' Fld_3 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18572'
      if Var_Expr_0 = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld_3 := Var_Expr_0.Dotted_Name_F_Prefix;
--# expr-done 13465
--# property-call-start WithClause.child_unit_xref_equation
Fld_4 := Libadalang.Implementation.With_Clause_P_Child_Unit_Xref_Equation (Node => Ent.Node, Name => Fld_3, E_Info => Ent.Info);
--# end
--# expr-done 13466
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_3652;
   Result_Var_1 := Scope_Result_1;
else
   True_Rel := Solver.Create_True ((if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18572:75") else null)); 
   Result_Var_1 := True_Rel;
end if;
      Inc_Ref (Result_Var_1);
And_Pred := Create_And (Local_Self_Eq, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18571:9") else null)); 
--# expr-done 13467
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 13468
         Property_Result := Let_Result;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1618;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1618;
                     Finalizer_Scope_3652;
            raise;
      end;
   return Property_Result;
end With_Clause_P_Child_Unit_Xref_Equation;
--# end
--# property-start WithClause.xref_entry_point 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18576'
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Entry_Point
  (Node : Bare_With_Clause
  )
   return Boolean
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Clause_P_Xref_Entry_Point;
--# end
--# property-start WithClause.xref_equation 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18579'
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Equation
  (Node : Bare_With_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
      ; Entry_Point : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
     Ent : Internal_Entity_With_Clause :=
       Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
      --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   --# bind entry_point Entry_Point
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1620 with Inline_Always;
            procedure Finalizer_Scope_3653 with Inline_Always;
      Fld : Bare_Name_List;
Item_0 : Bare_Name;
Fld_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1620 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1620;
            procedure Finalizer_Scope_3653 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_3653;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      begin
   --# scope-start
         --# expr-start 13472 '<LogicAll at nodes.lkt:18580:9>' Logic_Boolean_Op 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18580'
--# expr-start 13471 '<Map at nodes.lkt:18580:9>' Map_Result 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18580'
   --# expr-start 13469 '<FieldAccess for packages at nodes.lkt:18580:9>' Fld 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18580'
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
Fld := Self.With_Clause_F_Packages;
--# expr-done 13469
      declare
         Map_Result_Vec : Logic_Equation_Vectors.Vector;
      begin
      if Fld = null then
      Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_0 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_0 := Untyped_Item_0; 
   --# scope-start
               --# bind p Item_0
      --# expr-start 13470 '<FieldAccess for child_unit_xref_equation at nodes.lkt:18580:40>' Fld_1 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18580'
      if Ent.Node = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dereferencing a null access");
      end if;
--# property-call-start WithClause.child_unit_xref_equation
Fld_1 := Libadalang.Implementation.With_Clause_P_Child_Unit_Xref_Equation (Node => Ent.Node, Name => Item_0, E_Info => Ent.Info);
--# end
--# expr-done 13470
         declare
            Item_To_Append : constant Logic_Equation := Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_3653;
         end loop;
      end;
      end if;
         Map_Result := Create_Logic_Equation_Array
           (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
         for I in Map_Result.Items'Range loop
            Map_Result.Items (I) := Logic_Equation_Vectors.Get
              (Map_Result_Vec,
               I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
         end loop;
         Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end;
--# expr-done 13471
Logic_Boolean_Op := Solver.Create_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then New_Unit_String (Node.Unit, "nodes.lkt:18580:9") else null)); 
--# expr-done 13472
         Property_Result := Logic_Boolean_Op;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1620;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1620;
                     Finalizer_Scope_3653;
            raise;
      end;
   return Property_Result;
end With_Clause_P_Xref_Equation;
--# end
--# property-start WithClause._env_init_232 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18584'
pragma Warnings (Off, "is not referenced");
 function Env_Init_232
  (Node : Bare_With_Clause
  )
   return Internal_Designated_Env
is
   Self : Bare_With_Clause  := Bare_With_Clause (Node);
      --# bind self Self
   Property_Result : Internal_Designated_Env;
            procedure Finalizer_Scope_1896 with Inline_Always;
      Sym : Symbol_Type;
New_Struct : Internal_Designated_Env := No_Designated_Env;
            procedure Finalizer_Scope_1896 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1896;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         --# expr-start 13474 '<New DesignatedEnv at nodes.lkt:18584:13>' New_Struct 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18584'
--# expr-start 13473 '<Symbol Standard at nodes.lkt:18585:60>' Sym 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18585'
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Sym_Standard); 
--# expr-done 13473
Inc_Ref (Empty_Env);
New_Struct := (Direct_Env => Empty_Env, Env_Name => Sym, Kind => Named_Env); 
--# expr-done 13474
         Property_Result := New_Struct;
            Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1896;
      exception
         when Exc : Precondition_Failure | Property_Error =>
                     Finalizer_Scope_1896;
            raise;
      end;
   return Property_Result;
end Env_Init_232;
--# end
   --
   --  Primitives for Bare_With_Private
   --
--# property-start '[dispatcher]WithPrivate.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_With_Private_P_As_Bool
  (Node : Bare_With_Private
  )
   return Boolean
is
   Self : Bare_With_Private  := Bare_With_Private (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      if Self = null then
         Raise_Property_Exception
           (Self, Property_Error'Identity, "dispatching on null node");
      end if;
      case Ada_With_Private (Self.Kind) is
               when Ada_With_Private_Absent =>
                  --# property-call-start WithPrivate.Absent.as_bool
                  Property_Result := With_Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_With_Private_Present =>
                  --# property-call-start WithPrivate.Present.as_bool
                  Property_Result := With_Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   return Property_Result;
end Dispatcher_With_Private_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Absent
   --
--# property-start WithPrivate.Absent.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18593'
pragma Warnings (Off, "is not referenced");
 function With_Private_Absent_P_As_Bool
  (Node : Bare_With_Private_Absent
  )
   return Boolean
is
   Self : Bare_With_Private_Absent  := Bare_With_Private_Absent (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := False;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Present
   --
--# property-start WithPrivate.Present.as_bool 'C:\M\B\src\build-UCRT64\ada\nodes.lkt:18593'
pragma Warnings (Off, "is not referenced");
 function With_Private_Present_P_As_Bool
  (Node : Bare_With_Private_Present
  )
   return Boolean
is
   Self : Bare_With_Private_Present  := Bare_With_Private_Present (Node);
      --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   pragma Assert (Self = Node);
      begin
   --# scope-start
         Property_Result := True;
   --# end
      exception
         when Exc : Precondition_Failure | Property_Error =>
            raise;
      end;
   return Property_Result;
end With_Private_Present_P_As_Bool;
--# end
   ----------------------------
   -- Destroy_Synthetic_Node --
   ----------------------------
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node) is
      procedure Free is new Ada.Unchecked_Deallocation
        (Root_Node_Record, Bare_Ada_Node);
   begin
      --  Don't call Node.Destroy, as Node's children may be gone already: they
      --  have their own destructor and there is no specified order for the
      --  call of these destructors.
      Free_User_Fields (Node);
      Free (Node);
   end Destroy_Synthetic_Node;
   -----------
   -- Image --
   -----------
   function Image (Value : Boolean) return String
   is (if Value then "True" else "False");
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image
        (Node       : Bare_Ada_Node;
         Decoration : Boolean := True) return String is
      begin
         if Node = null then
            return "None";
         else
            declare
               Result : constant String :=
                 (Kind_Name (Node) & " "
                  & Basename (Node.Unit) & ":"
                  & Image (Sloc_Range (Node)));
            begin
               return (if Decoration then "<" & Result & ">" else Result);
            end;
         end if;
      end Trace_Image;
   Kind_Names : array (Ada_Node_Kind_Type) of Unbounded_String :=
     (Ada_Abort_Absent => To_Unbounded_String ("AbortAbsent"), 
Ada_Abort_Present => To_Unbounded_String ("AbortPresent"), 
Ada_Abstract_Absent => To_Unbounded_String ("AbstractAbsent"), 
Ada_Abstract_Present => To_Unbounded_String ("AbstractPresent"), 
Ada_Ada_Node_List => To_Unbounded_String ("AdaNodeList"), 
Ada_Abstract_State_Decl_List => To_Unbounded_String ("AbstractStateDeclList"), 
Ada_Alternatives_List => To_Unbounded_String ("AlternativesList"), 
Ada_Constraint_List => To_Unbounded_String ("ConstraintList"), 
Ada_Decl_List => To_Unbounded_String ("DeclList"), 
Ada_Stmt_List => To_Unbounded_String ("StmtList"), 
Ada_Aspect_Assoc_List => To_Unbounded_String ("AspectAssocList"), 
Ada_Base_Assoc_List => To_Unbounded_String ("BaseAssocList"), 
Ada_Assoc_List => To_Unbounded_String ("AssocList"), 
Ada_Case_Expr_Alternative_List => To_Unbounded_String ("CaseExprAlternativeList"), 
Ada_Case_Stmt_Alternative_List => To_Unbounded_String ("CaseStmtAlternativeList"), 
Ada_Compilation_Unit_List => To_Unbounded_String ("CompilationUnitList"), 
Ada_Concat_Operand_List => To_Unbounded_String ("ConcatOperandList"), 
Ada_Contract_Case_Assoc_List => To_Unbounded_String ("ContractCaseAssocList"), 
Ada_Defining_Name_List => To_Unbounded_String ("DefiningNameList"), 
Ada_Discriminant_Spec_List => To_Unbounded_String ("DiscriminantSpecList"), 
Ada_Elsif_Expr_Part_List => To_Unbounded_String ("ElsifExprPartList"), 
Ada_Elsif_Stmt_Part_List => To_Unbounded_String ("ElsifStmtPartList"), 
Ada_Enum_Literal_Decl_List => To_Unbounded_String ("EnumLiteralDeclList"), 
Ada_Expr_Alternatives_List => To_Unbounded_String ("ExprAlternativesList"), 
Ada_Format_String_Chunk_List => To_Unbounded_String ("FormatStringChunkList"), 
Ada_Discriminant_Choice_List => To_Unbounded_String ("DiscriminantChoiceList"), 
Ada_Name_List => To_Unbounded_String ("NameList"), 
Ada_Parent_List => To_Unbounded_String ("ParentList"), 
Ada_Param_Spec_List => To_Unbounded_String ("ParamSpecList"), 
Ada_Pragma_Node_List => To_Unbounded_String ("PragmaNodeList"), 
Ada_Select_When_Part_List => To_Unbounded_String ("SelectWhenPartList"), 
Ada_Unconstrained_Array_Index_List => To_Unbounded_String ("UnconstrainedArrayIndexList"), 
Ada_Variant_List => To_Unbounded_String ("VariantList"), 
Ada_Aliased_Absent => To_Unbounded_String ("AliasedAbsent"), 
Ada_Aliased_Present => To_Unbounded_String ("AliasedPresent"), 
Ada_All_Absent => To_Unbounded_String ("AllAbsent"), 
Ada_All_Present => To_Unbounded_String ("AllPresent"), 
Ada_Constrained_Array_Indices => To_Unbounded_String ("ConstrainedArrayIndices"), 
Ada_Unconstrained_Array_Indices => To_Unbounded_String ("UnconstrainedArrayIndices"), 
Ada_Aspect_Assoc => To_Unbounded_String ("AspectAssoc"), 
Ada_At_Clause => To_Unbounded_String ("AtClause"), 
Ada_Attribute_Def_Clause => To_Unbounded_String ("AttributeDefClause"), 
Ada_Enum_Rep_Clause => To_Unbounded_String ("EnumRepClause"), 
Ada_Record_Rep_Clause => To_Unbounded_String ("RecordRepClause"), 
Ada_Aspect_Spec => To_Unbounded_String ("AspectSpec"), 
Ada_Contract_Case_Assoc => To_Unbounded_String ("ContractCaseAssoc"), 
Ada_Pragma_Argument_Assoc => To_Unbounded_String ("PragmaArgumentAssoc"), 
Ada_Entry_Spec => To_Unbounded_String ("EntrySpec"), 
Ada_Enum_Subp_Spec => To_Unbounded_String ("EnumSubpSpec"), 
Ada_Subp_Spec => To_Unbounded_String ("SubpSpec"), 
Ada_Synthetic_Binary_Spec => To_Unbounded_String ("SyntheticBinarySpec"), 
Ada_Synthetic_Unary_Spec => To_Unbounded_String ("SyntheticUnarySpec"), 
Ada_Component_List => To_Unbounded_String ("ComponentList"), 
Ada_Known_Discriminant_Part => To_Unbounded_String ("KnownDiscriminantPart"), 
Ada_Unknown_Discriminant_Part => To_Unbounded_String ("UnknownDiscriminantPart"), 
Ada_Entry_Completion_Formal_Params => To_Unbounded_String ("EntryCompletionFormalParams"), 
Ada_Generic_Formal_Part => To_Unbounded_String ("GenericFormalPart"), 
Ada_Null_Record_Def => To_Unbounded_String ("NullRecordDef"), 
Ada_Record_Def => To_Unbounded_String ("RecordDef"), 
Ada_Aggregate_Assoc => To_Unbounded_String ("AggregateAssoc"), 
Ada_Multi_Dim_Array_Assoc => To_Unbounded_String ("MultiDimArrayAssoc"), 
Ada_Composite_Constraint_Assoc => To_Unbounded_String ("CompositeConstraintAssoc"), 
Ada_Iterated_Assoc => To_Unbounded_String ("IteratedAssoc"), 
Ada_Param_Assoc => To_Unbounded_String ("ParamAssoc"), 
Ada_Abstract_State_Decl => To_Unbounded_String ("AbstractStateDecl"), 
Ada_Anonymous_Expr_Decl => To_Unbounded_String ("AnonymousExprDecl"), 
Ada_Component_Decl => To_Unbounded_String ("ComponentDecl"), 
Ada_Discriminant_Spec => To_Unbounded_String ("DiscriminantSpec"), 
Ada_Generic_Formal_Obj_Decl => To_Unbounded_String ("GenericFormalObjDecl"), 
Ada_Generic_Formal_Package => To_Unbounded_String ("GenericFormalPackage"), 
Ada_Generic_Formal_Subp_Decl => To_Unbounded_String ("GenericFormalSubpDecl"), 
Ada_Generic_Formal_Type_Decl => To_Unbounded_String ("GenericFormalTypeDecl"), 
Ada_Param_Spec => To_Unbounded_String ("ParamSpec"), 
Ada_Synthetic_Formal_Param_Decl => To_Unbounded_String ("SyntheticFormalParamDecl"), 
Ada_Generic_Package_Internal => To_Unbounded_String ("GenericPackageInternal"), 
Ada_Package_Decl => To_Unbounded_String ("PackageDecl"), 
Ada_Discrete_Base_Subtype_Decl => To_Unbounded_String ("DiscreteBaseSubtypeDecl"), 
Ada_Subtype_Decl => To_Unbounded_String ("SubtypeDecl"), 
Ada_Classwide_Type_Decl => To_Unbounded_String ("ClasswideTypeDecl"), 
Ada_Incomplete_Type_Decl => To_Unbounded_String ("IncompleteTypeDecl"), 
Ada_Incomplete_Formal_Type_Decl => To_Unbounded_String ("IncompleteFormalTypeDecl"), 
Ada_Incomplete_Tagged_Type_Decl => To_Unbounded_String ("IncompleteTaggedTypeDecl"), 
Ada_Protected_Type_Decl => To_Unbounded_String ("ProtectedTypeDecl"), 
Ada_Task_Type_Decl => To_Unbounded_String ("TaskTypeDecl"), 
Ada_Single_Task_Type_Decl => To_Unbounded_String ("SingleTaskTypeDecl"), 
Ada_Anonymous_Type_Decl => To_Unbounded_String ("AnonymousTypeDecl"), 
Ada_Synth_Anonymous_Type_Decl => To_Unbounded_String ("SynthAnonymousTypeDecl"), 
Ada_Concrete_Type_Decl => To_Unbounded_String ("ConcreteTypeDecl"), 
Ada_Formal_Type_Decl => To_Unbounded_String ("FormalTypeDecl"), 
Ada_Abstract_Subp_Decl => To_Unbounded_String ("AbstractSubpDecl"), 
Ada_Abstract_Formal_Subp_Decl => To_Unbounded_String ("AbstractFormalSubpDecl"), 
Ada_Concrete_Formal_Subp_Decl => To_Unbounded_String ("ConcreteFormalSubpDecl"), 
Ada_Subp_Decl => To_Unbounded_String ("SubpDecl"), 
Ada_Entry_Decl => To_Unbounded_String ("EntryDecl"), 
Ada_Enum_Literal_Decl => To_Unbounded_String ("EnumLiteralDecl"), 
Ada_Synthetic_Char_Enum_Lit => To_Unbounded_String ("SyntheticCharEnumLit"), 
Ada_Generic_Subp_Internal => To_Unbounded_String ("GenericSubpInternal"), 
Ada_Synthetic_Subp_Decl => To_Unbounded_String ("SyntheticSubpDecl"), 
Ada_Accept_Stmt_Body => To_Unbounded_String ("AcceptStmtBody"), 
Ada_Expr_Function => To_Unbounded_String ("ExprFunction"), 
Ada_Null_Subp_Decl => To_Unbounded_String ("NullSubpDecl"), 
Ada_Subp_Body => To_Unbounded_String ("SubpBody"), 
Ada_Subp_Renaming_Decl => To_Unbounded_String ("SubpRenamingDecl"), 
Ada_Package_Body_Stub => To_Unbounded_String ("PackageBodyStub"), 
Ada_Protected_Body_Stub => To_Unbounded_String ("ProtectedBodyStub"), 
Ada_Subp_Body_Stub => To_Unbounded_String ("SubpBodyStub"), 
Ada_Task_Body_Stub => To_Unbounded_String ("TaskBodyStub"), 
Ada_Entry_Body => To_Unbounded_String ("EntryBody"), 
Ada_Package_Body => To_Unbounded_String ("PackageBody"), 
Ada_Protected_Body => To_Unbounded_String ("ProtectedBody"), 
Ada_Task_Body => To_Unbounded_String ("TaskBody"), 
Ada_Entry_Index_Spec => To_Unbounded_String ("EntryIndexSpec"), 
Ada_Error_Decl => To_Unbounded_String ("ErrorDecl"), 
Ada_Exception_Decl => To_Unbounded_String ("ExceptionDecl"), 
Ada_Exception_Handler => To_Unbounded_String ("ExceptionHandler"), 
Ada_For_Loop_Var_Decl => To_Unbounded_String ("ForLoopVarDecl"), 
Ada_Generic_Package_Decl => To_Unbounded_String ("GenericPackageDecl"), 
Ada_Generic_Subp_Decl => To_Unbounded_String ("GenericSubpDecl"), 
Ada_Generic_Package_Instantiation => To_Unbounded_String ("GenericPackageInstantiation"), 
Ada_Generic_Subp_Instantiation => To_Unbounded_String ("GenericSubpInstantiation"), 
Ada_Generic_Package_Renaming_Decl => To_Unbounded_String ("GenericPackageRenamingDecl"), 
Ada_Generic_Subp_Renaming_Decl => To_Unbounded_String ("GenericSubpRenamingDecl"), 
Ada_Label_Decl => To_Unbounded_String ("LabelDecl"), 
Ada_Named_Stmt_Decl => To_Unbounded_String ("NamedStmtDecl"), 
Ada_Number_Decl => To_Unbounded_String ("NumberDecl"), 
Ada_Object_Decl => To_Unbounded_String ("ObjectDecl"), 
Ada_Extended_Return_Stmt_Object_Decl => To_Unbounded_String ("ExtendedReturnStmtObjectDecl"), 
Ada_No_Type_Object_Renaming_Decl => To_Unbounded_String ("NoTypeObjectRenamingDecl"), 
Ada_Package_Renaming_Decl => To_Unbounded_String ("PackageRenamingDecl"), 
Ada_Single_Protected_Decl => To_Unbounded_String ("SingleProtectedDecl"), 
Ada_Single_Task_Decl => To_Unbounded_String ("SingleTaskDecl"), 
Ada_Synthetic_Object_Decl => To_Unbounded_String ("SyntheticObjectDecl"), 
Ada_Case_Stmt_Alternative => To_Unbounded_String ("CaseStmtAlternative"), 
Ada_Compilation_Unit => To_Unbounded_String ("CompilationUnit"), 
Ada_Component_Clause => To_Unbounded_String ("ComponentClause"), 
Ada_Component_Def => To_Unbounded_String ("ComponentDef"), 
Ada_Constant_Absent => To_Unbounded_String ("ConstantAbsent"), 
Ada_Constant_Present => To_Unbounded_String ("ConstantPresent"), 
Ada_Composite_Constraint => To_Unbounded_String ("CompositeConstraint"), 
Ada_Delta_Constraint => To_Unbounded_String ("DeltaConstraint"), 
Ada_Digits_Constraint => To_Unbounded_String ("DigitsConstraint"), 
Ada_Range_Constraint => To_Unbounded_String ("RangeConstraint"), 
Ada_Declarative_Part => To_Unbounded_String ("DeclarativePart"), 
Ada_Private_Part => To_Unbounded_String ("PrivatePart"), 
Ada_Public_Part => To_Unbounded_String ("PublicPart"), 
Ada_Elsif_Expr_Part => To_Unbounded_String ("ElsifExprPart"), 
Ada_Elsif_Stmt_Part => To_Unbounded_String ("ElsifStmtPart"), 
Ada_Abstract_State_Decl_Expr => To_Unbounded_String ("AbstractStateDeclExpr"), 
Ada_Allocator => To_Unbounded_String ("Allocator"), 
Ada_Aggregate => To_Unbounded_String ("Aggregate"), 
Ada_Bracket_Aggregate => To_Unbounded_String ("BracketAggregate"), 
Ada_Delta_Aggregate => To_Unbounded_String ("DeltaAggregate"), 
Ada_Bracket_Delta_Aggregate => To_Unbounded_String ("BracketDeltaAggregate"), 
Ada_Null_Record_Aggregate => To_Unbounded_String ("NullRecordAggregate"), 
Ada_Bin_Op => To_Unbounded_String ("BinOp"), 
Ada_Relation_Op => To_Unbounded_String ("RelationOp"), 
Ada_Box_Expr => To_Unbounded_String ("BoxExpr"), 
Ada_Case_Expr_Alternative => To_Unbounded_String ("CaseExprAlternative"), 
Ada_Concat_Op => To_Unbounded_String ("ConcatOp"), 
Ada_Concat_Operand => To_Unbounded_String ("ConcatOperand"), 
Ada_Case_Expr => To_Unbounded_String ("CaseExpr"), 
Ada_If_Expr => To_Unbounded_String ("IfExpr"), 
Ada_Contract_Cases => To_Unbounded_String ("ContractCases"), 
Ada_Decl_Expr => To_Unbounded_String ("DeclExpr"), 
Ada_Format_String_Literal => To_Unbounded_String ("FormatStringLiteral"), 
Ada_Membership_Expr => To_Unbounded_String ("MembershipExpr"), 
Ada_Array_Subcomponent_Choice_Name => To_Unbounded_String ("ArraySubcomponentChoiceName"), 
Ada_Attribute_Ref => To_Unbounded_String ("AttributeRef"), 
Ada_Call_Expr => To_Unbounded_String ("CallExpr"), 
Ada_Defining_Name => To_Unbounded_String ("DefiningName"), 
Ada_Synthetic_Defining_Name => To_Unbounded_String ("SyntheticDefiningName"), 
Ada_Discrete_Subtype_Name => To_Unbounded_String ("DiscreteSubtypeName"), 
Ada_Dotted_Name => To_Unbounded_String ("DottedName"), 
Ada_End_Name => To_Unbounded_String ("EndName"), 
Ada_Explicit_Deref => To_Unbounded_String ("ExplicitDeref"), 
Ada_Qual_Expr => To_Unbounded_String ("QualExpr"), 
Ada_Reduce_Attribute_Ref => To_Unbounded_String ("ReduceAttributeRef"), 
Ada_Char_Literal => To_Unbounded_String ("Chr"), 
Ada_Identifier => To_Unbounded_String ("Id"), 
Ada_Op_Abs => To_Unbounded_String ("OpAbs"), 
Ada_Op_And => To_Unbounded_String ("OpAnd"), 
Ada_Op_And_Then => To_Unbounded_String ("OpAndThen"), 
Ada_Op_Concat => To_Unbounded_String ("OpConcat"), 
Ada_Op_Div => To_Unbounded_String ("OpDiv"), 
Ada_Op_Double_Dot => To_Unbounded_String ("OpDoubleDot"), 
Ada_Op_Eq => To_Unbounded_String ("OpEq"), 
Ada_Op_Gt => To_Unbounded_String ("OpGt"), 
Ada_Op_Gte => To_Unbounded_String ("OpGte"), 
Ada_Op_In => To_Unbounded_String ("OpIn"), 
Ada_Op_Lt => To_Unbounded_String ("OpLt"), 
Ada_Op_Lte => To_Unbounded_String ("OpLte"), 
Ada_Op_Minus => To_Unbounded_String ("OpMinus"), 
Ada_Op_Mod => To_Unbounded_String ("OpMod"), 
Ada_Op_Mult => To_Unbounded_String ("OpMult"), 
Ada_Op_Neq => To_Unbounded_String ("OpNeq"), 
Ada_Op_Not => To_Unbounded_String ("OpNot"), 
Ada_Op_Not_In => To_Unbounded_String ("OpNotIn"), 
Ada_Op_Or => To_Unbounded_String ("OpOr"), 
Ada_Op_Or_Else => To_Unbounded_String ("OpOrElse"), 
Ada_Op_Plus => To_Unbounded_String ("OpPlus"), 
Ada_Op_Pow => To_Unbounded_String ("OpPow"), 
Ada_Op_Rem => To_Unbounded_String ("OpRem"), 
Ada_Op_Xor => To_Unbounded_String ("OpXor"), 
Ada_String_Literal => To_Unbounded_String ("Str"), 
Ada_Null_Literal => To_Unbounded_String ("Null"), 
Ada_Int_Literal => To_Unbounded_String ("Int"), 
Ada_Real_Literal => To_Unbounded_String ("Real"), 
Ada_Synthetic_Identifier => To_Unbounded_String ("SyntheticIdentifier"), 
Ada_Target_Name => To_Unbounded_String ("TargetName"), 
Ada_Update_Attribute_Ref => To_Unbounded_String ("UpdateAttributeRef"), 
Ada_Paren_Expr => To_Unbounded_String ("ParenExpr"), 
Ada_Quantified_Expr => To_Unbounded_String ("QuantifiedExpr"), 
Ada_Raise_Expr => To_Unbounded_String ("RaiseExpr"), 
Ada_Un_Op => To_Unbounded_String ("UnOp"), 
Ada_For_Loop_Iter_Filter => To_Unbounded_String ("ForLoopIterFilter"), 
Ada_Format_String_Chunk => To_Unbounded_String ("FormatStringChunk"), 
Ada_Format_String_Tok_End => To_Unbounded_String ("FormatStringTokEnd"), 
Ada_Format_String_Tok_Mid => To_Unbounded_String ("FormatStringTokMid"), 
Ada_Format_String_Tok_Start => To_Unbounded_String ("FormatStringTokStart"), 
Ada_Format_String_Tok_String => To_Unbounded_String ("FormatStringTokString"), 
Ada_Handled_Stmts => To_Unbounded_String ("HandledStmts"), 
Ada_Interface_Kind_Limited => To_Unbounded_String ("InterfaceKindLimited"), 
Ada_Interface_Kind_Protected => To_Unbounded_String ("InterfaceKindProtected"), 
Ada_Interface_Kind_Synchronized => To_Unbounded_String ("InterfaceKindSynchronized"), 
Ada_Interface_Kind_Task => To_Unbounded_String ("InterfaceKindTask"), 
Ada_Iter_Type_In => To_Unbounded_String ("IterTypeIn"), 
Ada_Iter_Type_Of => To_Unbounded_String ("IterTypeOf"), 
Ada_Library_Item => To_Unbounded_String ("LibraryItem"), 
Ada_Limited_Absent => To_Unbounded_String ("LimitedAbsent"), 
Ada_Limited_Present => To_Unbounded_String ("LimitedPresent"), 
Ada_For_Loop_Spec => To_Unbounded_String ("ForLoopSpec"), 
Ada_While_Loop_Spec => To_Unbounded_String ("WhileLoopSpec"), 
Ada_Mode_Default => To_Unbounded_String ("ModeDefault"), 
Ada_Mode_In => To_Unbounded_String ("ModeIn"), 
Ada_Mode_In_Out => To_Unbounded_String ("ModeInOut"), 
Ada_Mode_Out => To_Unbounded_String ("ModeOut"), 
Ada_Multi_Abstract_State_Decl => To_Unbounded_String ("MultiAbstractStateDecl"), 
Ada_Not_Null_Absent => To_Unbounded_String ("NotNullAbsent"), 
Ada_Not_Null_Present => To_Unbounded_String ("NotNullPresent"), 
Ada_Null_Component_Decl => To_Unbounded_String ("NullComponentDecl"), 
Ada_Others_Designator => To_Unbounded_String ("OthersDesignator"), 
Ada_Overriding_Not_Overriding => To_Unbounded_String ("OverridingNotOverriding"), 
Ada_Overriding_Overriding => To_Unbounded_String ("OverridingOverriding"), 
Ada_Overriding_Unspecified => To_Unbounded_String ("OverridingUnspecified"), 
Ada_Params => To_Unbounded_String ("Params"), 
Ada_Paren_Abstract_State_Decl => To_Unbounded_String ("ParenAbstractStateDecl"), 
Ada_Pp_Else_Directive => To_Unbounded_String ("PpElseDirective"), 
Ada_Pp_Elsif_Directive => To_Unbounded_String ("PpElsifDirective"), 
Ada_Pp_End_If_Directive => To_Unbounded_String ("PpEndIfDirective"), 
Ada_Pp_If_Directive => To_Unbounded_String ("PpIfDirective"), 
Ada_Pp_Then_Kw => To_Unbounded_String ("PpThenKw"), 
Ada_Pragma_Node => To_Unbounded_String ("PragmaNode"), 
Ada_Private_Absent => To_Unbounded_String ("PrivateAbsent"), 
Ada_Private_Present => To_Unbounded_String ("PrivatePresent"), 
Ada_Protected_Def => To_Unbounded_String ("ProtectedDef"), 
Ada_Protected_Absent => To_Unbounded_String ("ProtectedAbsent"), 
Ada_Protected_Present => To_Unbounded_String ("ProtectedPresent"), 
Ada_Quantifier_All => To_Unbounded_String ("QuantifierAll"), 
Ada_Quantifier_Some => To_Unbounded_String ("QuantifierSome"), 
Ada_Range_Spec => To_Unbounded_String ("RangeSpec"), 
Ada_Renaming_Clause => To_Unbounded_String ("RenamingClause"), 
Ada_Synthetic_Renaming_Clause => To_Unbounded_String ("SyntheticRenamingClause"), 
Ada_Reverse_Absent => To_Unbounded_String ("ReverseAbsent"), 
Ada_Reverse_Present => To_Unbounded_String ("ReversePresent"), 
Ada_Select_When_Part => To_Unbounded_String ("SelectWhenPart"), 
Ada_Accept_Stmt => To_Unbounded_String ("AcceptStmt"), 
Ada_Accept_Stmt_With_Stmts => To_Unbounded_String ("AcceptStmtWithStmts"), 
Ada_For_Loop_Stmt => To_Unbounded_String ("ForLoopStmt"), 
Ada_Loop_Stmt => To_Unbounded_String ("LoopStmt"), 
Ada_While_Loop_Stmt => To_Unbounded_String ("WhileLoopStmt"), 
Ada_Begin_Block => To_Unbounded_String ("BeginBlock"), 
Ada_Decl_Block => To_Unbounded_String ("DeclBlock"), 
Ada_Case_Stmt => To_Unbounded_String ("CaseStmt"), 
Ada_Extended_Return_Stmt => To_Unbounded_String ("ExtendedReturnStmt"), 
Ada_If_Stmt => To_Unbounded_String ("IfStmt"), 
Ada_Named_Stmt => To_Unbounded_String ("NamedStmt"), 
Ada_Select_Stmt => To_Unbounded_String ("SelectStmt"), 
Ada_Error_Stmt => To_Unbounded_String ("ErrorStmt"), 
Ada_Abort_Stmt => To_Unbounded_String ("AbortStmt"), 
Ada_Assign_Stmt => To_Unbounded_String ("AssignStmt"), 
Ada_Call_Stmt => To_Unbounded_String ("CallStmt"), 
Ada_Delay_Stmt => To_Unbounded_String ("DelayStmt"), 
Ada_Exit_Stmt => To_Unbounded_String ("ExitStmt"), 
Ada_Goto_Stmt => To_Unbounded_String ("GotoStmt"), 
Ada_Label => To_Unbounded_String ("Label"), 
Ada_Null_Stmt => To_Unbounded_String ("NullStmt"), 
Ada_Raise_Stmt => To_Unbounded_String ("RaiseStmt"), 
Ada_Requeue_Stmt => To_Unbounded_String ("RequeueStmt"), 
Ada_Return_Stmt => To_Unbounded_String ("ReturnStmt"), 
Ada_Simple_Decl_Stmt => To_Unbounded_String ("SimpleDeclStmt"), 
Ada_Terminate_Alternative => To_Unbounded_String ("TerminateAlternative"), 
Ada_Subp_Kind_Function => To_Unbounded_String ("SubpKindFunction"), 
Ada_Subp_Kind_Procedure => To_Unbounded_String ("SubpKindProcedure"), 
Ada_Subunit => To_Unbounded_String ("Subunit"), 
Ada_Synchronized_Absent => To_Unbounded_String ("SynchronizedAbsent"), 
Ada_Synchronized_Present => To_Unbounded_String ("SynchronizedPresent"), 
Ada_Tagged_Absent => To_Unbounded_String ("TaggedAbsent"), 
Ada_Tagged_Present => To_Unbounded_String ("TaggedPresent"), 
Ada_Task_Def => To_Unbounded_String ("TaskDef"), 
Ada_Type_Attributes_Repository => To_Unbounded_String ("TypeAttributesRepository"), 
Ada_Access_To_Subp_Def => To_Unbounded_String ("AccessToSubpDef"), 
Ada_Anonymous_Type_Access_Def => To_Unbounded_String ("AnonymousTypeAccessDef"), 
Ada_Type_Access_Def => To_Unbounded_String ("TypeAccessDef"), 
Ada_Array_Type_Def => To_Unbounded_String ("ArrayTypeDef"), 
Ada_Derived_Type_Def => To_Unbounded_String ("DerivedTypeDef"), 
Ada_Enum_Type_Def => To_Unbounded_String ("EnumTypeDef"), 
Ada_Formal_Discrete_Type_Def => To_Unbounded_String ("FormalDiscreteTypeDef"), 
Ada_Interface_Type_Def => To_Unbounded_String ("InterfaceTypeDef"), 
Ada_Mod_Int_Type_Def => To_Unbounded_String ("ModIntTypeDef"), 
Ada_Private_Type_Def => To_Unbounded_String ("PrivateTypeDef"), 
Ada_Decimal_Fixed_Point_Def => To_Unbounded_String ("DecimalFixedPointDef"), 
Ada_Floating_Point_Def => To_Unbounded_String ("FloatingPointDef"), 
Ada_Ordinary_Fixed_Point_Def => To_Unbounded_String ("OrdinaryFixedPointDef"), 
Ada_Record_Type_Def => To_Unbounded_String ("RecordTypeDef"), 
Ada_Signed_Int_Type_Def => To_Unbounded_String ("SignedIntTypeDef"), 
Ada_Anonymous_Type => To_Unbounded_String ("AnonymousType"), 
Ada_Enum_Lit_Synth_Type_Expr => To_Unbounded_String ("EnumLitSynthTypeExpr"), 
Ada_Subtype_Indication => To_Unbounded_String ("SubtypeIndication"), 
Ada_Constrained_Subtype_Indication => To_Unbounded_String ("ConstrainedSubtypeIndication"), 
Ada_Discrete_Subtype_Indication => To_Unbounded_String ("DiscreteSubtypeIndication"), 
Ada_Synthetic_Type_Expr => To_Unbounded_String ("SyntheticTypeExpr"), 
Ada_Unconstrained_Array_Index => To_Unbounded_String ("UnconstrainedArrayIndex"), 
Ada_Until_Absent => To_Unbounded_String ("UntilAbsent"), 
Ada_Until_Present => To_Unbounded_String ("UntilPresent"), 
Ada_Use_Package_Clause => To_Unbounded_String ("UsePackageClause"), 
Ada_Use_Type_Clause => To_Unbounded_String ("UseTypeClause"), 
Ada_Value_Sequence => To_Unbounded_String ("ValueSequence"), 
Ada_Variant => To_Unbounded_String ("Variant"), 
Ada_Variant_Part => To_Unbounded_String ("VariantPart"), 
Ada_With_Clause => To_Unbounded_String ("WithClause"), 
Ada_With_Private_Absent => To_Unbounded_String ("WithPrivateAbsent"), 
Ada_With_Private_Present => To_Unbounded_String ("WithPrivatePresent"));
   ---------------
   -- Kind_Name --
   ---------------
   function Kind_Name (Node : Bare_Ada_Node) return String is
   begin
      return To_String (Kind_Names (Node.Kind));
   end Kind_Name;
   --------------------
   -- Children_Count --
   --------------------
   function Children_Count (Node : Bare_Ada_Node) return Natural is
      C : Integer := Kind_To_Node_Children_Count (Node.Kind);
   begin
      if C = -1 then
         return Node.Count;
      else
         return C;
      end if;
   end Children_Count;
   ----------------------
   -- Free_User_Fields --
   ----------------------
   procedure Free_User_Fields (Node : Bare_Ada_Node) is
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record);
      --  Reset the LV logic variable, clearing the value it stores
      ---------------------
      -- Reset_Logic_Var --
      ---------------------
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record) is
      begin
         LV.Value := No_Entity;
         Entity_Vars.Reset (LV'Unrestricted_Access);
         Entity_Vars.Destroy (LV);
      end Reset_Logic_Var;
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      case Ada_Ada_Node (K) is
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
Dec_Ref (N_Bare_Base_Type_Decl.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitive_Subps);
Dec_Ref (N_Bare_Base_Type_Decl.Bare_Base_Type_Decl_Lf_Stg_Direct_Primitives_Env);
Dec_Ref (N_Bare_Base_Type_Decl.Bare_Base_Type_Decl_Lf_Stg_Synthetic_Object_Decl_Env_Assoc);
Dec_Ref (N_Bare_Base_Type_Decl.Bare_Base_Type_Decl_Lf_Stg_Dottable_Subps_Env);
end;
when Ada_Generic_Instantiation =>
declare
N_Bare_Generic_Instantiation : constant Bare_Generic_Instantiation := Node;
begin
Dec_Ref (N_Bare_Generic_Instantiation.Bare_Generic_Instantiation_Lf_Stg_Instantiation_Env);
Dec_Ref (N_Bare_Generic_Instantiation.Bare_Generic_Instantiation_Lf_Stg_Actual_Expr_Decls);
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
Dec_Ref (N_Bare_Compilation_Unit.Compilation_Unit_No_Env);
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var);
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Reset_Logic_Var (N_Bare_Call_Expr.Call_Expr_R_Called_Spec);
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Reset_Logic_Var (N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec);
end;
when Ada_Reduce_Attribute_Ref_Range =>
declare
N_Bare_Reduce_Attribute_Ref : constant Bare_Reduce_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Reduce_Attribute_Ref.Reduce_Attribute_Ref_R_Ref_Var);
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Reset_Logic_Var (N_Bare_Target_Name.Target_Name_R_Ref_Var);
end;
when Ada_Update_Attribute_Ref_Range =>
declare
N_Bare_Update_Attribute_Ref : constant Bare_Update_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Update_Attribute_Ref.Update_Attribute_Ref_R_Ref_Var);
end;
when Ada_Type_Attributes_Repository_Range =>
declare
N_Bare_Type_Attributes_Repository : constant Bare_Type_Attributes_Repository := Node;
begin
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Expr);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Type_Expr);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Type_Expr);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Base_Type_Param);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Int_Param);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Universal_Real_Param);
Dec_Ref (N_Bare_Type_Attributes_Repository.Bare_Type_Attributes_Repository_Lf_Stg_Root_Stream_Param);
end;
when others => null;
end case;
   end Free_User_Fields;
   ----------------
   -- Token_Data --
   ----------------
   function Token_Data (Unit : Internal_Unit) return Token_Data_Handler_Access
   is (Unit.TDH'Access);
   -------------------
   -- Lookup_Symbol --
   -------------------
   function Lookup_Symbol
     (Context : Internal_Context; Symbol : Text_Type) return Symbol_Type
   is
      Canon_Symbol : constant Symbolization_Result :=
            Libadalang.Sources.Canonicalize (Symbol)
      ;
   begin
      if Canon_Symbol.Success then
         return To_Symbol
           (Context.Symbols, Find (Context.Symbols, Canon_Symbol.Symbol));
      else
         raise Invalid_Symbol_Error with Image (Canon_Symbol.Error_Message);
      end if;
   end Lookup_Symbol;
   -------------------------
   -- Create_Special_Unit --
   -------------------------
   function Create_Special_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      Unit : Internal_Unit := new Analysis_Unit_Type'
        (Context                      => Context,
         Is_Internal                  => False,
         Ast_Root                     => null,
         Filename                     => Normalized_Filename,
         Charset                      => To_Unbounded_String (Charset),
         TDH                          => <>,
         Diagnostics                  => <>,
         Rule                         => Rule,
         Ast_Mem_Pool                 => No_Pool,
         Destroyables                 => Destroyable_Vectors.Empty_Vector,
         Referenced_Units             => <>,
         Exiled_Entries               => Exiled_Entry_Vectors.Empty_Vector,
         Foreign_Nodes                =>
            Foreign_Node_Entry_Vectors.Empty_Vector,
         Exiled_Entries_In_NED        =>
            Exiled_Entry_In_NED_Vectors.Empty_Vector,
         Exiled_Envs                  => Exiled_Env_Vectors.Empty_Vector,
         Named_Envs                   => Named_Env_Vectors.Empty_Vector,
         Nodes_With_Foreign_Env       => <>,
         Rebindings                   => Env_Rebindings_Vectors.Empty_Vector,
         Cache_Version                => <>,
         Unit_Version                 => <>,
         Memoization_Map            => <>,
         others => <>
      );
   begin
      Initialize
        (Unit.TDH, Context.Symbols, Unit.all'Address, Context.Tab_Stop);
      return Unit;
   end Create_Special_Unit;
   --------------------
   -- Templates_Unit --
   --------------------
   function Templates_Unit (Context : Internal_Context) return Internal_Unit is
   begin
      if Context.Templates_Unit = No_Analysis_Unit then
         Context.Templates_Unit := Create_Special_Unit
           (Context             => Context,
            Normalized_Filename => No_File,
            Charset             => Default_Charset,
            Rule                => Compilation_Rule);
      end if;
      return Context.Templates_Unit;
   end Templates_Unit;
   --------------
   -- Set_Rule --
   --------------
   procedure Set_Rule (Unit : Internal_Unit; Rule : Grammar_Rule) is
   begin
      Unit.Rule := Rule;
   end Set_Rule;
   ------------------------------
   -- Normalized_Unit_Filename --
   ------------------------------
   function Normalized_Unit_Filename
     (Context : Internal_Context; Filename : String) return Virtual_File
   is
      use Virtual_File_Maps;
      Key : constant Unbounded_String := To_Unbounded_String (Filename);
      Cur : Cursor := Context.Filenames.Find (Key);
   begin
      if Cur = No_Element then
         declare
            F : constant Virtual_File := Create
              (Create_From_Base (+Filename).Full_Name,
               Normalize => True);
         begin
            Context.Filenames.Insert (Key, F);
            return F;
         end;
      else
         return Element (Cur);
      end if;
   end Normalized_Unit_Filename;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure)
   is
   begin
      Destroyable_Vectors.Append (Unit.Destroyables, (Object, Destroy));
   end Register_Destroyable_Helper;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Node : Bare_Ada_Node)
   is
      procedure Helper is new Register_Destroyable_Gen
        (Root_Node_Record,
         Bare_Ada_Node,
         Destroy_Synthetic_Node);
   begin
      Helper (Unit, Node);
   end Register_Destroyable;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Env : AST_Envs.Lexical_Env_Access)
   is
      procedure Helper is new Register_Destroyable_Gen
        (AST_Envs.Lexical_Env_Record, AST_Envs.Lexical_Env_Access, Destroy);
   begin
      Helper (Unit, Env);
   end Register_Destroyable;
   -----------------------
   -- Invalidate_Caches --
   -----------------------
   procedure Invalidate_Caches
     (Context : Internal_Context; Invalidate_Envs : Boolean) is
   begin
      --  Increase Context's version number. If we are about to overflow, reset
      --  all version numbers from analysis units.
      if Context.Cache_Version = Version_Number'Last then
         Context.Cache_Version := 1;
         for Unit of Context.Units loop
            Unit.Cache_Version := 0;
         end loop;
      else
         Context.Cache_Version := Context.Cache_Version + 1;
      end if;
      if Invalidate_Envs then
         Context.Reparse_Cache_Version := Context.Cache_Version;
      end if;
   end Invalidate_Caches;
   ------------------
   --  Reset_Envs  --
   ------------------
   procedure Reset_Envs (Unit : Internal_Unit) is
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node);
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node);
      --------------------------
      -- Deactivate_Refd_Envs --
      --------------------------
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Deactivate_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Deactivate_Refd_Envs (Child (Node, I));
         end loop;
      end Deactivate_Refd_Envs;
      -------------------------
      -- Recompute_Refd_Envs --
      -------------------------
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Recompute_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Recompute_Refd_Envs (Child (Node, I));
         end loop;
      end Recompute_Refd_Envs;
   begin
      --  First pass will deactivate every referenced envs that Unit possesses
      Deactivate_Refd_Envs (Unit.Ast_Root);
      --  Second pass will recompute the env they are pointing to
      Recompute_Refd_Envs (Unit.Ast_Root);
   end Reset_Envs;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Reparsed : in out Reparsed_Unit) is
   begin
      Free (Reparsed.TDH);
      Reparsed.Diagnostics := Diagnostics_Vectors.Empty_Vector;
      Free (Reparsed.Ast_Mem_Pool);
      Reparsed.Ast_Root := null;
   end Destroy;
   --------------
   -- Basename --
   --------------
   function Basename (Filename : String) return String is
   begin
      return +Create (+Filename).Base_Name;
   end Basename;
   --------------
   -- Basename --
   --------------
   function Basename (Unit : Internal_Unit) return String is
   begin
      return +Unit.Filename.Base_Name;
   end Basename;
   ------------------
   -- Reset_Caches --
   ------------------
   procedure Reset_Caches (Unit : Internal_Unit) is
      Cache_Version : constant Version_Number := Unit.Cache_Version;
   begin
      if Cache_Version < Unit.Context.Reparse_Cache_Version then
         Unit.Cache_Version := Unit.Context.Reparse_Cache_Version;
         Reset_Envs (Unit);
      end if;
      if Cache_Version < Unit.Context.Cache_Version then
         Unit.Cache_Version := Unit.Context.Cache_Version;
            Destroy (Unit.Memoization_Map);
      end if;
   end Reset_Caches;
   --------------------
   -- Reference_Unit --
   --------------------
   procedure Reference_Unit (From, Referenced : Internal_Unit) is
      Dummy : Boolean;
   begin
      Dummy := Analysis_Unit_Sets.Add (From.Referenced_Units, Referenced);
   end Reference_Unit;
   ------------------------
   -- Is_Referenced_From --
   ------------------------
   function Is_Referenced_From
     (Self, Unit : Internal_Unit) return Boolean is
   begin
      if Unit = null or else Self = null then
         return False;
      elsif Unit = Self then
         return True;
      else
         return Analysis_Unit_Sets.Has (Unit.Referenced_Units, Self);
      end if;
   end Is_Referenced_From;
   ----------------
   -- Do_Parsing --
   ----------------
   procedure Do_Parsing
     (Unit   : Internal_Unit;
      Input  : Internal_Lexer_Input;
      Result : out Reparsed_Unit)
   is
      Context  : constant Internal_Context := Unit.Context;
      Unit_TDH : constant Token_Data_Handler_Access := Token_Data (Unit);
      Saved_TDH : Token_Data_Handler;
      --  Holder to save tokens data in Unit.
      --
      --  By design, parsing is required to bind the nodes it creates to an
      --  analysis unit. However, this procedure is supposed to preserve the
      --  Unit itself and return its parsing result in Result.
      --
      --  In order to implement this, we first move "old" token data in this
      --  variable, then we do parsing. Only then, we can move "new" token data
      --  from the unit to Result, and restore the "old" token data to Unit.
      --  This last step is what Rotate_TDH (see below) is above.
      procedure Rotate_TDH;
      --  Move token data from Unit to Result and restore data in Saved_TDH to
      --  Unit.
      ----------------
      -- Rotate_TDH --
      ----------------
      procedure Rotate_TDH is
      begin
         Move (Result.TDH, Unit_TDH.all);
         Move (Unit_TDH.all, Saved_TDH);
      end Rotate_TDH;
   begin
      GNATCOLL.Traces.Trace (Main_Trace, "Parsing unit " & Basename (Unit));
      Result.Ast_Root := null;
      Move (Saved_TDH, Unit_TDH.all);
      Initialize (Unit_TDH.all,
                  Saved_TDH.Symbols,
                  Unit.all'Address,
                  Unit.Context.Tab_Stop);
      --  This is where lexing occurs, so this is where we get most "setup"
      --  issues: missing input file, bad charset, etc. If we have such an
      --  error, catch it, turn it into diagnostics and abort parsing.
      --
      --  As it is quite common, first check if the file is readable: if not,
      --  don't bother opening it and directly emit a diagnostic. This avoid
      --  pointless exceptions which harm debugging. Note that this
      --  optimization is valid only when there is no file reader, which can
      --  work even when there is no real source file.
      if Context.File_Reader = null
         and then Input.Kind = File
         and then (Input.Filename.Is_Directory
                   or else (not Input.Filename.Is_Readable))
      then
         declare
            Name : constant String := Basename (Unit);
         begin
            GNATCOLL.Traces.Trace
              (Main_Trace, "WARNING: File is not readable: " & Name);
            Append
              (Result.Diagnostics,
               No_Source_Location_Range,
               "Cannot read " & To_Text (Name));
            Rotate_TDH;
            return;
         end;
      end if;
      --  Initialize the parser, which fetches the source buffer and extract
      --  all tokens.
      Init_Parser
        (Input, Context.With_Trivia, Unit, Unit_TDH, Unit.Context.Parser);
      --  If we could run the lexer, run the parser and get the root node
      if Unit_TDH.Source_Buffer /= null then
         Result.Ast_Mem_Pool := Create;
         Unit.Context.Parser.Mem_Pool := Result.Ast_Mem_Pool;
         Result.Ast_Root := Bare_Ada_Node
           (Parse (Unit.Context.Parser, Rule => Unit.Rule));
      end if;
      --  Forward token data and diagnostics to the returned unit
      Rotate_TDH;
      Result.Diagnostics.Append_Vector (Unit.Context.Parser.Diagnostics);
   end Do_Parsing;
   --------------------------
   -- Update_After_Reparse --
   --------------------------
   procedure Update_After_Reparse
     (Unit : Internal_Unit; Reparsed : in out Reparsed_Unit) is
   begin
      --  Remove the `symbol -> AST node` associations for Unit's nodes in
      --  foreign lexical environments.
      Remove_Exiled_Entries (Unit);
      --  Remove the named envs that Unit created
      declare
         Named_Envs_Needing_Update : NED_Maps.Map;
      begin
         Remove_Named_Envs (Unit, Named_Envs_Needing_Update);
         Update_Named_Envs (Unit.Context, Named_Envs_Needing_Update);
      end;
      --  Explicitly clear the env caches of this unit while it is still fully
      --  alive to make sure that ``Lexical_Env_Cache_Updated`` accesses valid
      --  data. Otherwise the env caches end up being cleared during the call
      --  to ``Destroy_Unit_Destroyables`` where the unit is already partially
      --  destroyed.
      Reset_Envs_Caches (Unit);
      --  At this point, envs and nodes that don't belong to this unit no
      --  longer reference this unit's envs and nodes. It is thus now safe to
      --  deallocate this unit's obsolete data.
      --  Replace Unit's diagnostics by Reparsed's
      Unit.Diagnostics := Reparsed.Diagnostics;
      Reparsed.Diagnostics.Clear;
      --  As (re-)loading a unit can change how any AST node property in the
      --  whole analysis context behaves, we have to invalidate caches. This
      --  is likely overkill, but kill all caches here as it's easy to do.
      --
      --  As an optimization, invalidate referenced envs cache only if this is
      --  not the first time we parse Unit.
      Invalidate_Caches
        (Unit.Context, Invalidate_Envs => Unit.Ast_Root /= null);
      --  Likewise for token data
      Free (Unit.TDH);
      Move (Unit.TDH, Reparsed.TDH);
      --  Reparsing will invalidate all lexical environments related to this
      --  unit, so destroy all related rebindings as well. This browses AST
      --  nodes, so we have to do this before destroying the old AST nodes
      --  pool.
      Destroy_Rebindings (Unit.Rebindings'Access);
      --  Clear the set of units referenced from that one, as it may no longer
      --  hold in the reparsed unit.
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);
      --  Destroy the old AST node and replace it by the new one
      if Unit.Ast_Root /= null then
         Destroy (Unit.Ast_Root);
      end if;
      Unit.Ast_Root := Reparsed.Ast_Root;
      --  Likewise for memory pools
      Free (Unit.Ast_Mem_Pool);
      Unit.Ast_Mem_Pool := Reparsed.Ast_Mem_Pool;
      Reparsed.Ast_Mem_Pool := No_Pool;
      --  Increment unit version number to invalidate caches and stale node
      --  reference. Also propagate it to the TDH.
      Unit.Unit_Version := Unit.Unit_Version + 1;
      Unit.TDH.Version := Unit.Unit_Version;
      --  Compute the PLE_Roots_Starting_Token table
      Unit.PLE_Roots_Starting_Token.Clear;
         if Unit.Ast_Root /= null
            and then Unit.Ast_Root.Kind
                     = Ada_Compilation_Unit_List
         then
            for I in 1 .. Children_Count (Unit.Ast_Root) loop
               Unit.PLE_Roots_Starting_Token.Append
                 (Child (Unit.Ast_Root, I).Token_Start_Index);
            end loop;
         end if;
      --  Update all the lexical envs entries affected by the reparse
      declare
         Unit_Name     : constant String := +Unit.Filename.Base_Name;
         Context       : constant Internal_Context := Unit.Context;
         Foreign_Nodes : Bare_Ada_Node_Vectors.Vector :=
           Bare_Ada_Node_Vectors.Empty_Vector;
         Saved_In_Populate_Lexical_Env : constant Boolean :=
           Context.In_Populate_Lexical_Env;
         Saved_Env_Populated_Roots     : constant Boolean_Vectors.Vector :=
           Unit.Env_Populated_Roots;
      begin
         Context.In_Populate_Lexical_Env := True;
         if Main_Trace.Active then
            Main_Trace.Trace
              ("Updating lexical envs for " & Unit_Name & " after reparse");
            Main_Trace.Increase_Indent;
         end if;
         --  Collect all nodes that are foreign in this Unit's lexical envs.
         --  Exclude them from the corresponding lists of exiled entries.
         Extract_Foreign_Nodes (Unit, Foreign_Nodes);
         --  Temporarily reset Env_Populated_Roots so that Populate_Lexical_Env
         --  accepts to do its work on reparsed trees.
         Unit.Env_Populated_Roots := Boolean_Vectors.Empty_Vector;
         --  Now that Unit has been reparsed, we can destroy all its
         --  destroyables, which refer to the old tree (i.e. dangling
         --  pointers).
         Destroy_Unit_Destroyables (Unit);
         for FN of Foreign_Nodes loop
            declare
               Node_Image : constant String := Image (Short_Text_Image (FN));
               Unit_Name  : constant String := +FN.Unit.Filename.Base_Name;
            begin
               GNATCOLL.Traces.Trace
                 (Main_Trace, "Rerooting: " & Node_Image
                              & " (from " & Unit_Name & ")");
            end;
            Reroot_Foreign_Node (FN);
         end loop;
         Foreign_Nodes.Destroy;
         --  Re-populate all PLE roots that were requested so far for this
         --  unit. In the case where the unit has no PLE root, run PLE on the
         --  whole unit iff it was requested on at least one PLE root.
         declare
            function At_Least_One_Root_Populated return Boolean
            is (for some B of Saved_Env_Populated_Roots => B);
         begin
               if not Unit.PLE_Roots_Starting_Token.Is_Empty then
                  for I in 1 .. Children_Count (Unit.Ast_Root) loop
                     if Saved_Env_Populated_Roots.Last_Index >= I
                        and then Saved_Env_Populated_Roots.Get (I)
                     then
                        Populate_Lexical_Env (Unit, I);
                     end if;
                  end loop;
               elsif At_Least_One_Root_Populated then
                  Populate_Lexical_Env (Unit, 1);
               end if;
         end;
         --  Restore the unit's original Env_Populated_Roots flags
         Unit.Env_Populated_Roots.Destroy;
         Unit.Env_Populated_Roots := Saved_Env_Populated_Roots;
         Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
         if Main_Trace.Is_Active then
            Main_Trace.Decrease_Indent;
         end if;
      end;
   end Update_After_Reparse;
   -------------------------------
   -- Destroy_Unit_Destroyables --
   -------------------------------
   procedure Destroy_Unit_Destroyables (Unit : Internal_Unit) is
   begin
      for D of Unit.Destroyables loop
         D.Destroy (D.Object);
      end loop;
      Destroyable_Vectors.Clear (Unit.Destroyables);
   end Destroy_Unit_Destroyables;
   ---------------------------
   -- Remove_Exiled_Entries --
   ---------------------------
   procedure Remove_Exiled_Entries (Unit : Internal_Unit) is
   begin
      for EE of Unit.Exiled_Entries loop
         AST_Envs.Remove (EE.Env, Thin (EE.Key), EE.Node);
         --  Also strip foreign nodes information from "outer" units so that it
         --  does not contain stale information (i.e. dangling pointers to
         --  nodes that belong to the units in the queue).
         if EE.Env.Owner /= No_Generic_Unit then
            declare
               Foreign_Nodes : Foreign_Node_Entry_Vectors.Vector renames
                  Convert_Unit (EE.Env.Owner).Foreign_Nodes;
               Current       : Positive := Foreign_Nodes.First_Index;
            begin
               while Current <= Foreign_Nodes.Last_Index loop
                  if Foreign_Nodes.Get (Current).Node = EE.Node then
                     Foreign_Nodes.Pop (Current);
                  else
                     Current := Current + 1;
                  end if;
               end loop;
            end;
         end if;
      end loop;
      Unit.Exiled_Entries.Clear;
   end Remove_Exiled_Entries;
   -----------------------
   -- Remove_Named_Envs --
   -----------------------
   procedure Remove_Named_Envs
     (Unit                      : Internal_Unit;
      Named_Envs_Needing_Update : in out NED_Maps.Map) is
   begin
      --  Remove nodes in this unit from the Named_Env_Descriptor.Foreign_Nodes
      --  components in which they are registered and from the foreign
      --  environments themselves.
      for EE of Unit.Exiled_Entries_In_NED loop
         Remove (EE.Named_Env.Foreign_Nodes, EE.Key, EE.Node);
         Remove (EE.Named_Env.Env_With_Precedence, Thin (EE.Key), EE.Node);
      end loop;
      Unit.Exiled_Entries_In_NED.Clear;
      --  Remove nodes in this unit from the
      --  Named_Env_Descriptor.Nodes_With_Foreign_Env components in which they
      --  are registered.
      for Cur in Unit.Nodes_With_Foreign_Env.Iterate loop
         declare
            use Node_To_Named_Env_Maps;
            Node : constant Bare_Ada_Node := Key (Cur);
            NE   : constant Named_Env_Descriptor_Access := Element (Cur);
         begin
            NE.Nodes_With_Foreign_Env.Delete (Node);
         end;
      end loop;
      Unit.Nodes_With_Foreign_Env.Clear;
      --  Remove ends in this unit from the Named_Env_Descriptor.Foreign_Envs
      --  components in which they are registered.
      for EE of Unit.Exiled_Envs loop
         EE.Named_Env.Foreign_Envs.Delete (Env_Node (EE.Env));
      end loop;
      Unit.Exiled_Envs.Clear;
      --  Remove named envs that this unit created
      for NE of Unit.Named_Envs loop
         declare
            NED_Access : constant Named_Env_Descriptor_Access :=
               Unit.Context.Named_Envs.Element (NE.Name);
            NED        : Named_Env_Descriptor renames NED_Access.all;
         begin
            NED.Envs.Delete (Env_Node (NE.Env));
            --  If this named environment had precedence, we must schedule an
            --  update for this name environment entry.
            if NE.Env = NED.Env_With_Precedence then
               Named_Envs_Needing_Update.Include (NE.Name, NED_Access);
               NED.Env_With_Precedence := Empty_Env;
            end if;
         end;
      end loop;
      Unit.Named_Envs.Clear;
   end Remove_Named_Envs;
   ---------------------------
   -- Extract_Foreign_Nodes --
   ---------------------------
   procedure Extract_Foreign_Nodes
     (Unit          : Internal_Unit;
      Foreign_Nodes : in out Bare_Ada_Node_Vectors.Vector) is
   begin
      --  Go through all foreign nodes registered in Unit's lexical
      --  environments.
      for FN of Unit.Foreign_Nodes loop
         --  Collect them
         Foreign_Nodes.Append (FN.Node);
         --  For each foreign node, remove the corresponding exiled entry in
         --  that foreign unit (each foreign node in unit A has a corresponding
         --  exiled entry in unit B).
         declare
            Exiled_Entries : Exiled_Entry_Vectors.Vector renames
               FN.Unit.Exiled_Entries;
            Current        : Positive := Exiled_Entries.First_Index;
         begin
            while Current <= Exiled_Entries.Last_Index loop
               if Exiled_Entries.Get (Current).Node = FN.Node then
                  Exiled_Entries.Pop (Current);
               else
                  Current := Current + 1;
               end if;
            end loop;
         end;
      end loop;
      Unit.Foreign_Nodes.Clear;
   end Extract_Foreign_Nodes;
   --------------------------
   -- Reroot_Foreign_Nodes --
   --------------------------
   procedure Reroot_Foreign_Node (Node : Bare_Ada_Node) is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  First, filter the exiled entries in foreign units so that they don't
      --  contain references to this unit's lexical environments.  We need to
      --  do that before running the partial Populate_Lexical_Env pass so that
      --  we don't remove exiled entries that this pass will produce.
      declare
         Exiled_Entries : Exiled_Entry_Vectors.Vector renames
            Unit.Exiled_Entries;
         Current        : Positive := Exiled_Entries.First_Index;
      begin
         while Current <= Exiled_Entries.Last_Index loop
            if Exiled_Entries.Get (Current).Node = Node then
               Exiled_Entries.Pop (Current);
            else
               Current := Current + 1;
            end if;
         end loop;
      end;
      --  Re-do a partial Populate_Lexical_Env pass for each foreign node that
      --  this unit contains so that they are relocated in our new lexical
      --  environments.
      declare
         Unit_State : aliased PLE_Unit_State :=
           (Named_Envs_Needing_Update => <>);
         State      : PLE_Node_State :=
           (Unit_State  => Unit_State'Unchecked_Access,
            Current_Env => Node.Self_Env,
            Current_NED => null);
      begin
         Pre_Env_Actions (Node, State, Add_To_Env_Only => True);
         Post_Env_Actions (Node, State);
      end;
   end Reroot_Foreign_Node;
   ----------
   -- Text --
   ----------
   function Text (Node : Bare_Ada_Node) return String_Type is
   begin
      return Create_String (Text (Node));
   end Text;
   ------------------------
   -- Destroy_Rebindings --
   ------------------------
   procedure Destroy_Rebindings
     (Rebindings : access Env_Rebindings_Vectors.Vector)
   is
      procedure Recurse (R : in out Env_Rebindings);
      --  Destroy R's children and then destroy R. It is up to the caller to
      --  remove R from its parent's Children vector.
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector);
      --  Remove R from Rebindings
      -------------
      -- Recurse --
      -------------
      procedure Recurse (R : in out Env_Rebindings) is
      begin
         for C of R.Children loop
            declare
               C_Var : Env_Rebindings := C;
            begin
               Recurse (C_Var);
            end;
         end loop;
         R.Children.Destroy;
         Unregister (R, Convert_Unit (R.Old_Env.Owner).Rebindings);
         Unregister (R, Convert_Unit (R.New_Env.Owner).Rebindings);
         Release_Rebinding (R);
      end Recurse;
      ----------------
      -- Unregister --
      ----------------
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector) is
      begin
         for I in 1 .. Rebindings.Length loop
            if Rebindings.Get (I) = R then
               Rebindings.Pop (I);
               return;
            end if;
         end loop;
         --  We are always supposed to find R in Rebindings, so this should be
         --  unreachable.
         raise Program_Error;
      end Unregister;
   begin
      while Rebindings.Length > 0 loop
         declare
            R : Env_Rebindings := Rebindings.Get (1);
         begin
            --  Here, we basically undo what has been done in AST_Envs.Append
            --  If this rebinding has no parent, then during its creation we
            --  registered it in its Old_Env. Otherwise, it is registered
            --  in its Parent's Children list.
            if R.Parent = null then
               Unwrap (R.Old_Env).Rebindings_Pool.Delete (R.New_Env);
            else
               Unregister (R, R.Parent.Children);
            end if;
            --  In all cases it's registered in Old_Env's and New_Env's units
            Recurse (R);
         end;
      end loop;
   end Destroy_Rebindings;
   --------------------------
   -- Get_Rewriting_Handle --
   --------------------------
   function Get_Rewriting_Handle
     (Context : Internal_Context) return Rewriting_Handle_Pointer is
   begin
      return Context.Rewriting_Handle;
   end Get_Rewriting_Handle;
   --------------------------
   -- Set_Rewriting_Handle --
   --------------------------
   procedure Set_Rewriting_Handle
     (Context : Internal_Context; Handle : Rewriting_Handle_Pointer) is
   begin
      Context.Rewriting_Handle := Handle;
   end Set_Rewriting_Handle;
   -----------------------
   -- Create_Safety_Net --
   -----------------------
   function Create_Safety_Net
     (Context : Internal_Context) return Iterator_Safety_Net
   is
   begin
      return (Context         => Context,
              Context_Serial  => Context.Serial_Number,
              Context_Version => Context.Cache_Version);
   end Create_Safety_Net;
   ----------------------
   -- Check_Safety_Net --
   ----------------------
   procedure Check_Safety_Net (Self : Iterator_Safety_Net) is
   begin
      if Self.Context = null then
         return;
      end if;
      --  Check that the context is still the same (not released nor reused)
      if Self.Context.Serial_Number /= Self.Context_Serial
         or else Self.Context.Cache_Version /= Self.Context_Version
      then
         raise Stale_Reference_Error;
      end if;
   end Check_Safety_Net;
   ----------------------
   -- String_To_Symbol --
   ----------------------
   function String_To_Symbol
     (Self    : Bare_Ada_Node;
      Context : Internal_Context;
      S       : String_Type) return Symbol_Type is
   begin
      return (if S.Length > 0
              then Lookup_Symbol (Context, S.Content)
              else No_Symbol);
   exception
      when Exc : Invalid_Symbol_Error =>
         Raise_Property_Exception
           (Self,
            Property_Error'Identity,
            Ada.Exceptions.Exception_Message (Exc));
   end String_To_Symbol;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Self : String_Type) is
   begin
      if Self.Ref_Count >= 0 then
         Self.Ref_Count := Self.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Self : in out String_Type) is
   begin
      if Self = null or else Self.Ref_Count < 0 then
         return;
      end if;
      if Self.Ref_Count = 1 then
         Free (Self);
      else
         Self.Ref_Count := Self.Ref_Count - 1;
         Self := null;
      end if;
   end Dec_Ref;
   -------------------
   -- Create_String --
   -------------------
   function Create_String (Content : Text_Type) return String_Type is
   begin
      return Result : constant String_Type := new String_Record'
        (Length    => Content'Length,
         Ref_Count => 1,
         Content   => Content);
   end Create_String;
   -------------------
   -- Create_String --
   -------------------
   function Create_String (Content : Unbounded_Text_Type) return String_Type is
      S : Big_Wide_Wide_String_Access;
      L : Natural;
   begin
      Get_Wide_Wide_String (Content, S, L);
      return Create_String (S.all (1 .. L));
   end Create_String;
   -------------------
   -- Concat_String --
   -------------------
   function Concat_String (Left, Right : String_Type) return String_Type is
   begin
      return Result : constant String_Type :=
        new String_Record (Length => Left.Length + Right.Length)
      do
         Result.Ref_Count := 1;
         Result.Content (1 .. Left.Length) := Left.Content;
         Result.Content (Left.Length + 1 .. Result.Length) := Right.Content;
      end return;
   end Concat_String;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : String_Type) return Boolean is
   begin
      return Left.Content = Right.Content;
   end Equivalent;
begin
   No_Big_Integer.Value.Set (0);
end Libadalang.Implementation;